'use strict';

var communicationCommon = require('@azure/communication-common');
var reselect = require('reselect');
var communicationCalling = require('@azure/communication-calling');
var memoizeOne = require('memoize-one');
var logger = require('@azure/logger');
var events = require('events');
var immer = require('immer');
var React = require('react');
var react = require('@fluentui/react');
var reactComponents = require('@fluentui/react-components');
var reactIcons = require('@fluentui/react-icons');
var uuid = require('uuid');
var reactChat = require('@fluentui-contrib/react-chat');
var react$1 = require('@griffel/react');
var parse = require('html-react-parser');
var Linkify = require('react-linkify');
var DOMPurify = require('dompurify');
var reactFileTypeIcons = require('@fluentui/react-file-type-icons');
var reactHooks = require('@fluentui/react-hooks');
var reactUseDraggableScroll = require('react-use-draggable-scroll');
var copy = require('copy-to-clipboard');
var communicationChat = require('@azure/communication-chat');
var nanoid = require('nanoid');
var communicationCallingEffects = require('@azure/communication-calling-effects');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var reselect__namespace = /*#__PURE__*/_interopNamespaceDefault(reselect);
var React__namespace = /*#__PURE__*/_interopNamespaceDefault(React);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const argsCmp = (args1, args2, objCmp) => {
    return args1.length === args2.length && args1.every((arg1, index) => objCmp(args2[index], arg1));
};
/**
 * The function memoize a series of function calls in a single pass,
 * it memoizes all the args and return in a single run of the callback function, and read it in the next round of execution
 * note: this is a memory opimized function which will only memoize one round of bulk calls
 * @param  fnToMemoize - the function needs to be bulk memorized and a key key paramter needs to be provided as cache id
 * @param  shouldCacheUpdate - the validate function for comparing 2 argument, return true when 2 args are equal
 * @returns callback function includes a series calls of memoizedFn, and each call will get cache result if args are the same(according to shouldCacheUpdate fn)
 * @example
 * ```ts
 * const items = [{id:1, value:3}];
 * const heavyFn = (_key, value) => { // key is not used in the function, but it is a cache id
 *   // assume this is a heavy caculation
 *   return value+1;
 * }
 *
 * const memoizeHeavyFnAll = memoizeFnAll(heavyFn);
 * const generateValueArray = (memoizedHeavyFn) => (
 *   items.map(item => {
 *     memoizedHeavyFn(item.id, item.value);
 *   })
 * );
 *
 * const result = memoizeHeavyFnAll(generateValueArray); // Cache: {}, nextCache: {1: 4 *new}, heavyFn call times: 1
 *
 * // Argument changed
 * items[0].value = 2
 * const result0 = memoizeHeavyFnAll(generateValueArray); // Cache: {1: 4}, nextCache: {1: 3 *new}, heavyFn call times: 1
 *
 * // Cache added
 * items.push({id:3, value:4});
 * const result1 = memoizeHeavyFnAll(generateValueArray); // Cache: {1: 3 *hit}, nextCache: {1: 3, 3: 5 *new}, heavyFn call times: 1
 *
 * // Cache removed
 * delete items[0];
 * const result2 = memoizeHeavyFnAll(generateValueArray); // Cache: {1: 3, 3: 5 *hit}, nextCache: {3: 5}, heavyFn call times: 0
 * ```
 *
 * @public
 */
const memoizeFnAll = (fnToMemoize, shouldCacheUpdate = Object.is) => {
    let cache = new Map();
    let nextCache = new Map();
    return (callback) => {
        const memoizedFn = (key, ...args) => {
            const value = cache.get(key);
            if (value) {
                const [preArgs, ret] = value;
                if (argsCmp(preArgs, args, shouldCacheUpdate)) {
                    nextCache.set(key, [args, ret]);
                    return ret;
                }
            }
            const ret = fnToMemoize(key, ...args);
            nextCache.set(key, [args, ret]);
            return ret;
        };
        const retValue = callback(memoizedFn);
        cache = nextCache;
        nextCache = new Map();
        return retValue;
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A string representation of a {@link @azure/communication-common#CommunicationIdentifier}.
 *
 * This string representation of CommunicationIdentifier is guaranteed to be stable for
 * a unique Communication user. Thus,
 * - it can be used to persist a user's identity in external databases.
 * - it can be used as keys into a Map to store data for the user.
 *
 * @public
 */
const toFlatCommunicationIdentifier = (identifier) => {
    return communicationCommon.getIdentifierRawId(identifier);
};
/**
 * Reverse operation of {@link toFlatCommunicationIdentifier}.
 *
 * @public
 */
const fromFlatCommunicationIdentifier = (id) => {
    // if the id passed is a phone number we need to build the rawId to pass in
    const rawId = id.indexOf('+') === 0 ? '4:' + id : id;
    return communicationCommon.createIdentifierFromRawId(rawId);
};
/**
 * Returns a CommunicationIdentifier.
 * @internal
 */
const _toCommunicationIdentifier = (id) => {
    if (typeof id === 'string') {
        return fromFlatCommunicationIdentifier(id);
    }
    return id;
};
/**
 * Check if an object is identifier.
 *
 * @internal
 */
const _isValidIdentifier = (identifier) => {
    return communicationCommon.isCommunicationUserIdentifier(identifier) || communicationCommon.isPhoneNumberIdentifier(identifier) || communicationCommon.isMicrosoftTeamsUserIdentifier(identifier) || communicationCommon.isUnknownIdentifier(identifier);
};
/**
 * Check if given identifier is a Microsoft Teams user.
 *
 * @internal
 * @param rawId - The rawId of the identifier.
 * @returns True if the identifier is a Microsoft Teams user. False otherwise.
 */
const _isIdentityMicrosoftTeamsUser = (rawId) => {
    if (!rawId) {
        return false;
    }
    const identifier = _toCommunicationIdentifier(rawId);
    return communicationCommon.isMicrosoftTeamsUserIdentifier(identifier);
};

function getDefaultExportFromCjs (x) {
	return x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// GENERATED FILE. DO NOT EDIT MANUALLY.
var telemetryVersion = '1.28.0';


var telemetryVersion$1 = /*@__PURE__*/getDefaultExportFromCjs(telemetryVersion);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
// Removes long suffixes that don't fit the constraints for telemetry application ID.
// e.g., the build suffix is dropped for alpha package versions.
const sanitize = (version) => {
    const alphaIndex = version.search(/alpha/);
    if (alphaIndex >= 0) {
        return version.substring(0, alphaIndex + 5);
    }
    return version;
};
/**
 * Takes a telemetry implementation hint and returns the numerical value.
 *
 * @private
 */
const getTelemetryImplementationHint = (telemetryImplementationHint) => {
    switch (telemetryImplementationHint) {
        case 'Call':
            return 1;
        case 'Chat':
            return 2;
        case 'CallWithChat':
            return 3;
        case 'StatefulComponents':
            return 4;
        default:
            return 0;
    }
};
/**
 * Application ID to be included in telemetry data from the UI library.
 * Template: acXYYY/<version>
 * Where:
 * - X describes a platform, [r: web, i: iOS, a: Android]
 * - YYY describes what's running on this platform (optional, currently unused by this library):
 *    Y[0] is high-level artifact,
 *      [0: undefined, 1: AzureCommunicationLibrary, 2: ACS SampleApp]
 *    Y[1] is specific implementation,
 *      [0: undefined, 1: Call Composite, 2: Chat Composite, 3: CallWithChatComposite, 4: UI Components]
 *    Y[2] is reserved for implementation details,
 *      [0: undefined]
 *
 * @internal
 */
const _getApplicationId = (telemetryImplementationHint) => {
    // We assume AzureCommunicationLibrary, as we don't currently have any public API to inject otherwise.
    // This is consistent with the native iOS and Android implementations of telemetry.
    const highLevelArtifact = 1;
    const specificImplementation = getTelemetryImplementationHint(telemetryImplementationHint);
    const implementationDetails = 0;
    const version = telemetryVersion$1;
    return sanitize(`acr${highLevelArtifact}${specificImplementation}${implementationDetails}/${version}`);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 *
 * Replace the pattern "\{\}" in str with the values passed in as vars
 *
 * @example
 * ```ts
 *   _formatString("hello {name}. '{name}' is a rare name.", {name: "Foo"});
 *   // returns "hello Foo. 'Foo' is a rare name."
 * ```
 * @param str - The string to be formatted
 * @param variables - Variables to use to format the string
 * @returns a formatted string
 */
const _formatString = (str, vars) => {
    if (!str) {
        return '';
    }
    if (!vars) {
        return str;
    }
    // regex to search for the pattern "\{\}"
    const placeholdersRegex = /{(\w+)}/g;
    return str.replace(placeholdersRegex, (_, k) => {
        const replaceValue = vars[k];
        return replaceValue === undefined ? `{${k}}` : replaceValue;
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Wrap JSON.stringify in a try-catch as JSON.stringify throws an exception if it fails.
 *
 * Use this only in areas where the JSON.stringify is non-critical and OK for the JSON.stringify to fail, such as logging.
 *
 * @internal
 */
const _safeJSONStringify = (value, replacer = createSafeReplacer(), space) => {
    try {
        return JSON.stringify(value, replacer, space);
    }
    catch (e) {
        console.error(e);
        return undefined;
    }
};
// Log all visited refs to avoid circular ref
const createSafeReplacer = () => {
    const visited = new Set();
    return function replacer(key, value) {
        if (typeof value !== 'object') {
            return value;
        }
        if (visited.has(value)) {
            return 'Visited-Ref';
        }
        else {
            visited.add(value);
            return value;
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * Converts units of rem to units of pixels
 * @param rem - units of rem
 * @returns units of pixels
 */
const _convertRemToPx = (rem) => {
    return rem * getBrowserFontSizeInPx();
};
/**
 * @internal
 * Converts units of pixels to units of rem
 * @param px - units of px
 * @returns units of rem
 */
const _convertPxToRem = (px) => {
    return px / getBrowserFontSizeInPx();
};
const getBrowserFontSizeInPx = () => {
    let fontSizeInPx = parseFloat(getComputedStyle(document.documentElement).fontSize);
    // If browser font size is not a number, default to 16 px
    if (Number.isNaN(fontSizeInPx)) {
        fontSizeInPx = 16;
    }
    return fontSizeInPx;
};
/**
 * @internal
 * Disable dismiss on resize to work around a couple Fluent UI bugs
 * - The Callout is dismissed whenever *any child of window (inclusive)* is resized. In practice, this
 * happens when we change the VideoGallery layout, or even when the video stream element is internally resized
 * by the headless SDK.
 * - We also want to prevent dismiss when chat pane is scrolling especially a new message is added.
 * A side effect of this workaround is that the context menu stays open when window is resized, and may
 * get detached from original target visually. That bug is preferable to the bug when this value is not set -
 * The Callout (frequently) gets dismissed automatically.
 */
const _preventDismissOnEvent = (ev) => {
    return ev.type === 'resize' || ev.type === 'scroll';
};
/**
 * @internal
 * Helper function to get the keys of an object
 */
function _getKeys(obj) {
    return Object.keys(obj);
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * Converts px value to rem value.
 * For example, an input of `16` will return `1rem`.
 */
const _pxToRem = (px) => `${px / 16}rem`;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * This is a log function to log structural data for easier parse in telemetry
 */
const _logEvent = (logger, event) => {
    logger[event.level](_safeJSONStringify(event));
};

/**
 * @private
 */
const getDeviceManager$1 = (state) => state.deviceManager;
/**
 * @private
 */
const getRole$1 = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.role;
};
/**
 * @private
 */
const isHideAttendeeNamesEnabled = (state, props) => {
    var _a, _b;
    return (_b = (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.hideAttendeeNames) !== null && _b !== void 0 ? _b : false;
};
/**
 * @private
 */
const getCapabilities = (state, props) => { var _a, _b; return (_b = (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.capabilitiesFeature) === null || _b === void 0 ? void 0 : _b.capabilities; };
/**
 * @private
 */
const getCallExists = (state, props) => !!state.calls[props.callId];
/**
 * @private
 */
const getDominantSpeakers = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.dominantSpeakers; };
/**
 * @private
 */
const getRemoteParticipants$1 = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.remoteParticipants;
};
/**
 * @private
 */
const getRemoteParticipantsEnded = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.remoteParticipantsEnded;
};
/**
 * @private
 */
const getLocalParticipantRaisedHand$1 = (state, props) => {
    var _a, _b;
    return (_b = (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.raiseHand) === null || _b === void 0 ? void 0 : _b.localParticipantRaisedHand;
};
/**
 * @private
 */
const getSpotlightCallFeature = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.spotlight;
};
/**
 * @private
 */
const getLocalParticipantReactionState = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.localParticipantReaction;
};
/**
 * @private
 */
const getIsScreenSharingOn = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.isScreenSharingOn; };
/**
 * @private
 */
const getIsMuted = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.isMuted; };
/**
 * @private
 */
const getOptimalVideoCount = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.optimalVideoCount.maxRemoteVideoStreams; };
/**
 * @private
 */
const getLocalVideoStreams$1 = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.localVideoStreams; };
/**
 * @private
 */
const getScreenShareRemoteParticipant = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.screenShareRemoteParticipant; };
/**
 * @private
 */
const getDisplayName$2 = (state) => { var _a; return (_a = state.callAgent) === null || _a === void 0 ? void 0 : _a.displayName; };
/**
 * @private
 */
const getIdentifier = (state) => toFlatCommunicationIdentifier(state.userId);
/**
 * @private
 */
const getLatestErrors$1 = (state) => state.latestErrors;
/**
 * @private
 */
const getLatestNotifications$1 = (state) => state.latestNotifications;
/**
 * @private
 */
const getDiagnostics = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.diagnostics; };
/**
 * @private
 */
const getCallState = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.state; };
/**
 * @private
 */
const getEnvironmentInfo$1 = (state) => {
    return state.environmentInfo;
};
/** @private */
const getParticipantCount = (state, props) => {
    return undefined;
};
/** @private */
const getCaptions = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.captionsFeature.captions;
};
/** @private */
const getCaptionsStatus$1 = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.captionsFeature.isCaptionsFeatureActive;
};
/** @private */
const getStartCaptionsInProgress = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.captionsFeature.startCaptionsInProgress;
};
/** @private */
const getCurrentCaptionLanguage = (state, props) => {
    var _a, _b;
    // we default to 'en' if the currentCaptionLanguage is not set
    if (((_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.captionsFeature.currentCaptionLanguage) === '') {
        return 'en';
    }
    return (_b = state.calls[props.callId]) === null || _b === void 0 ? void 0 : _b.captionsFeature.currentCaptionLanguage;
};
/** @private */
const getCurrentSpokenLanguage = (state, props) => {
    var _a;
    // when currentSpokenLanguage is '', it means the spoken language is not set.
    // In this case we suggest showing the captions modal to set the spoken language when starting captions
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.captionsFeature.currentSpokenLanguage;
};
/** @private */
const getSupportedCaptionLanguages = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.captionsFeature.supportedCaptionLanguages;
};
/** @private */
const getSupportedSpokenLanguages = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.captionsFeature.supportedSpokenLanguages;
};
/** @private */
const getMeetingConferencePhones = (state, props) => {
    var _a, _b;
    return (_b = (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.meetingConference) === null || _b === void 0 ? void 0 : _b.conferencePhones;
};
/**
 * selector for retrieving the incoming calls from state
 * @returns the incoming calls in the call client state
 * @private
 */
const getIncomingCalls = (state) => {
    return Object.values(state.incomingCalls);
};
/**
 * selector for retrieving the incoming calls that have been removed from state
 * @returns the incoming calls that have been removed
 * @private
 */
const getRemovedIncomingCalls = (state) => {
    return Object.values(state.incomingCallsEnded);
};
/** @private */
const getAssignedBreakoutRoom$1 = (state, props) => {
    var _a, _b;
    return (_b = (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.breakoutRooms) === null || _b === void 0 ? void 0 : _b.assignedBreakoutRoom;
};
/** @private */
const getRealTimeTextStatus = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.realTimeTextFeature.isRealTimeTextFeatureActive;
};
/** @private */
const getRealTimeText = (state, props) => {
    var _a;
    return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.realTimeTextFeature.realTimeTexts;
};
/**
 * @private
 */
const getTogetherModeCallFeature = (state, props) => { var _a; return (_a = state.calls[props.callId]) === null || _a === void 0 ? void 0 : _a.togetherMode; };

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$$ = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Check if the call state represents being in the call
 *
 * @internal
 */
const _isInCall = (callStatus) => !!callStatus && !['None', 'Disconnected', 'Connecting', 'Ringing', 'EarlyMedia', 'Disconnecting'].includes(callStatus);
/**
 * Check if the call state represents being in the lobby or waiting to be admitted.
 *
 * @internal
 */
const _isInLobbyOrConnecting = (callStatus) => !!callStatus && ['Connecting', 'Ringing', 'InLobby', 'EarlyMedia'].includes(callStatus);
/**
 * Check if the device manager local video is on when not part of a call
 * i.e. do unparented views exist.
 *
 * @internal
 */
const _isPreviewOn = (deviceManager) => {
    var _a;
    // TODO: we should take in a LocalVideoStream that developer wants to use as their 'Preview' view. We should also
    // handle cases where 'Preview' view is in progress and not necessary completed.
    return ((_a = deviceManager.unparentedViews[0]) === null || _a === void 0 ? void 0 : _a.view) !== undefined;
};
/**
 * Dispose of all preview views
 * We assume all unparented views are local preview views.
 *
 * @private
 */
const disposeAllLocalPreviewViews = (callClient) => __awaiter$$(void 0, void 0, void 0, function* () {
    const unparentedViews = callClient.getState().deviceManager.unparentedViews;
    for (const view of unparentedViews) {
        yield callClient.disposeView(undefined, undefined, view);
    }
});
/**
 * Update the users displayNames based on the type of user they are
 *
 * @internal
 */
const _updateUserDisplayNames = (participants) => {
    if (participants) {
        return memoizedUpdateDisplayName(memoizedFn => {
            return Object.values(participants).map(p => {
                const pid = toFlatCommunicationIdentifier(p.identifier);
                return memoizedFn(pid, p);
            });
        });
    }
    else {
        return [];
    }
};
/**
 * Get the display name of a participant for given userId
 *
 * @internal
 *
 */
const getRemoteParticipantDisplayName = (participantUserId, remoteParticipants) => {
    let displayName;
    if (remoteParticipants) {
        const participant = remoteParticipants[participantUserId];
        if (participant) {
            displayName = participant.displayName;
        }
    }
    return displayName;
};
const memoizedUpdateDisplayName = memoizeFnAll((participantId, participant) => {
    if (communicationCommon.isPhoneNumberIdentifier(participant.identifier)) {
        return Object.assign(Object.assign({}, participant), { displayName: participant.identifier.phoneNumber });
    }
    else {
        return participant;
    }
});
/**
 * @private
 * A type guard to ensure all participants are acceptable type for Teams call
 */
const isTeamsCallParticipants = (participants) => {
    return participants.every(p => !communicationCommon.isCommunicationUserIdentifier(p));
};
/**
 * @private
 * Checks whether the user is a 'Ringing' PSTN user or in a 'Connecting' state.
 */
const _convertParticipantState = (participant) => {
    return communicationCommon.isPhoneNumberIdentifier(participant.identifier) && participant.state === 'Connecting' ? 'Ringing' : participant.state;
};
/**
 * @private
 * Changes the display name of the participant based on the local and remote user's role.
 */
const maskDisplayNameWithRole = (displayName, localUserRole, participantRole, isHideAttendeeNamesEnabled) => {
    let maskedDisplayName = displayName;
    if (isHideAttendeeNamesEnabled && participantRole && participantRole === 'Attendee') {
        if (localUserRole && localUserRole === 'Attendee') {
            maskedDisplayName = '{AttendeeRole}';
        }
        if (localUserRole && (localUserRole === 'Presenter' || localUserRole === 'Co-organizer' || localUserRole === 'Organizer')) {
            maskedDisplayName = `{AttendeeRole}(${displayName})`;
        }
    }
    return maskedDisplayName;
};
/**
 * Helper to create a local video stream from the selected camera.
 * @private
 */
const createLocalVideoStream = (callClient) => __awaiter$$(void 0, void 0, void 0, function* () {
    const camera = yield (callClient === null || callClient === void 0 ? void 0 : callClient.getState().deviceManager.selectedCamera);
    if (camera) {
        return new communicationCalling.LocalVideoStream(camera);
    }
    return undefined;
});
/**
 * Get call state if existing, if not and the call not exists in ended record return undefined, if it never exists, throw an error.
 * @private
 */
const getCallStateIfExist = (state, callId) => {
    if (!state.calls[callId]) {
        // If call has ended, we don't need to throw an error.
        if (state.callsEnded[callId]) {
            return undefined;
        }
        throw new Error(`Call Not Found: ${callId}`);
    }
    return state.calls[callId];
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Selector for {@link MicrophoneButton} component.
 *
 * @public
 */
const microphoneButtonSelector = reselect__namespace.createSelector([getCallExists, getIsMuted, getDeviceManager$1, getCapabilities, getRole$1, getCallState], (callExists, isMuted, deviceManager, capabilities, role, callState) => {
    const permission = deviceManager.deviceAccess ? deviceManager.deviceAccess.audio : true;
    const incapable = (capabilities === null || capabilities === void 0 ? void 0 : capabilities.unmuteMic.isPresent) === false && (capabilities === null || capabilities === void 0 ? void 0 : capabilities.unmuteMic.reason) !== 'NotInitialized' || role === 'Consumer';
    return {
        disabled: !callExists || !permission || incapable || callState === 'LocalHold',
        checked: callExists ? !isMuted : false,
        microphones: deviceManager.microphones,
        speakers: deviceManager.speakers,
        selectedMicrophone: deviceManager.selectedMicrophone,
        selectedSpeaker: deviceManager.selectedSpeaker
    };
});
/**
 * Selector for {@link CameraButton} component.
 *
 * @public
 */
const cameraButtonSelector = reselect__namespace.createSelector([getLocalVideoStreams$1, getDeviceManager$1, getCapabilities, getRole$1, getCallState], (localVideoStreams, deviceManager, capabilities, role, callState) => {
    const previewOn = _isPreviewOn(deviceManager);
    const localVideoFromCall = localVideoStreams === null || localVideoStreams === void 0 ? void 0 : localVideoStreams.find(stream => stream.mediaStreamType === 'Video');
    const permission = deviceManager.deviceAccess ? deviceManager.deviceAccess.video : true;
    const incapable = (capabilities === null || capabilities === void 0 ? void 0 : capabilities.turnVideoOn.isPresent) === false && (capabilities === null || capabilities === void 0 ? void 0 : capabilities.turnVideoOn.reason) !== 'NotInitialized' || role === 'Consumer';
    return {
        disabled: !deviceManager.selectedCamera || !permission || !deviceManager.cameras.length || incapable || callState === 'LocalHold',
        checked: localVideoStreams !== undefined && localVideoStreams.length > 0 ? !!localVideoFromCall : previewOn,
        cameras: deviceManager.cameras,
        selectedCamera: deviceManager.selectedCamera
    };
});
/**
 * Selector for {@link RaiseHandButton} component.
 *
 * @public
 */
const raiseHandButtonSelector = reselect__namespace.createSelector([getLocalParticipantRaisedHand$1, getCallState], (raisedHand, callState) => {
    return {
        checked: raisedHand ? true : false,
        disabled: callState === 'InLobby' ? true : callState === 'Connecting'
    };
});
/**
 * Selector for {@link ReactionButton} component.
 *
 * @public
 */
const reactionButtonSelector = reselect__namespace.createSelector([getLocalParticipantReactionState, getCallState], (reaction, callState) => {
    return {
        checked: reaction ? true : false,
        disabled: callState !== 'Connected'
    };
});
/**
 * Selector for {@link ScreenShareButton} component.
 *
 * @public
 */
const screenShareButtonSelector = reselect__namespace.createSelector([getIsScreenSharingOn, getCallState, getCapabilities, getRole$1], (isScreenSharingOn, callState, capabilities, role) => {
    let disabled = undefined;
    disabled = disabled || (capabilities === null || capabilities === void 0 ? void 0 : capabilities.shareScreen.isPresent) === false && (capabilities === null || capabilities === void 0 ? void 0 : capabilities.shareScreen.reason) !== 'NotInitialized' || role === 'Consumer' || role === 'Attendee';
    disabled = disabled || ['InLobby', 'Connecting', 'LocalHold'].includes(callState !== null && callState !== void 0 ? callState : 'None');
    return {
        checked: isScreenSharingOn,
        disabled
    };
});
/**
 * Selector for {@link DevicesButton} component.
 *
 * @public
 */
const devicesButtonSelector = reselect__namespace.createSelector([getDeviceManager$1], deviceManager => {
    return {
        microphones: removeBlankNameDevices(deviceManager.microphones),
        speakers: removeBlankNameDevices(deviceManager.speakers),
        cameras: removeBlankNameDevices(deviceManager.cameras),
        selectedMicrophone: deviceManager.selectedMicrophone,
        selectedSpeaker: deviceManager.selectedSpeaker,
        selectedCamera: deviceManager.selectedCamera
    };
});
function removeBlankNameDevices(devices) {
    return devices.filter(device => device.name !== '');
}
/**
 * Selector for the {@link HoldButton} component.
 * @public
 */
const holdButtonSelector = reselect__namespace.createSelector([getCallState], callState => {
    return {
        checked: callState === 'LocalHold'
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$_ = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const areStreamsEqual = (prevStream, newStream) => {
    return !!prevStream && !!newStream && prevStream.source.id === newStream.source.id;
};
/**
 * Create the common implementation of {@link CallingHandlers} for all types of Call
 *
 * @private
 */
const createDefaultCommonCallingHandlers = memoizeOne((callClient, deviceManager, call, options) => {
    const onStartLocalVideo = () => __awaiter$_(void 0, void 0, void 0, function* () {
        // Before the call object creates a stream, dispose of any local preview streams.
        // @TODO: is there any way to parent the unparented view to the call object instead
        // of disposing and creating a new stream?
        yield disposeAllLocalPreviewViews(callClient);
        const callId = call === null || call === void 0 ? void 0 : call.id;
        let videoDeviceInfo = callClient.getState().deviceManager.selectedCamera;
        if (!videoDeviceInfo) {
            const cameras = yield (deviceManager === null || deviceManager === void 0 ? void 0 : deviceManager.getCameras());
            videoDeviceInfo = cameras && cameras.length > 0 ? cameras[0] : undefined;
            if (videoDeviceInfo) {
                deviceManager === null || deviceManager === void 0 ? void 0 : deviceManager.selectCamera(videoDeviceInfo);
            }
        }
        if (!callId || !videoDeviceInfo) {
            return;
        }
        const stream = new communicationCalling.LocalVideoStream(videoDeviceInfo);
        if (call && !call.localVideoStreams.find(s => areStreamsEqual(s, stream))) {
            yield call.startVideo(stream);
        }
    });
    const onStopLocalVideo = (stream) => __awaiter$_(void 0, void 0, void 0, function* () {
        const callId = call === null || call === void 0 ? void 0 : call.id;
        if (!callId) {
            return;
        }
        if (call && call.localVideoStreams.find(s => areStreamsEqual(s, stream))) {
            yield call.stopVideo(stream);
        }
    });
    const onToggleCamera = (options) => __awaiter$_(void 0, void 0, void 0, function* () {
        const previewOn = _isPreviewOn(callClient.getState().deviceManager);
        // the disposal of the unparented views is to workaround: https://skype.visualstudio.com/SPOOL/_workitems/edit/3030558.
        // The root cause of the issue is caused by never transitioning the unparented view to the
        // call object when going from configuration page (disconnected call state) to connecting.
        //
        // Currently the only time the local video stream is moved from unparented view to the call
        // object is when we transition from connecting -> call state. If the camera was on,
        // inside the MediaGallery we trigger toggleCamera. This triggers onStartLocalVideo which
        // destroys the unparentedView and creates a new stream in the call - so all looks well.
        //
        // However, if someone turns off their camera during the lobbyOrConnecting screen, the
        // call.localVideoStreams will be empty (as the stream is currently stored in the unparented
        // views and was never transitioned to the call object) and thus we incorrectly try to create
        // a new video stream for the call object, instead of only stopping the unparented view.
        //
        // The correct fix for this is to ensure that callAgent.onStartCall is called with the
        // localvideostream as a videoOption. That will mean call.onLocalVideoStreamsUpdated will
        // be triggered when the call is in connecting state, which we can then transition the
        // local video stream to the stateful call client and get into a clean state.
        if (call && (_isInCall(call.state) || _isInLobbyOrConnecting(call.state))) {
            const stream = call.localVideoStreams.find(stream => stream.mediaStreamType === 'Video');
            const unparentedViews = callClient.getState().deviceManager.unparentedViews;
            if (stream || unparentedViews.length > 0) {
                unparentedViews.forEach(view => view.mediaStreamType === 'Video' && callClient.disposeView(undefined, undefined, view));
                if (stream) {
                    yield onStopLocalVideo(stream);
                }
            }
            else {
                yield onStartLocalVideo();
            }
        }
        else {
            /**
             * This will create a unparented view to be used on the configuration page and the connecting screen
             *
             * If the device that the stream will come from is not on from permissions checks, then it will take time
             * to create the stream since device is off. If we are turn the camera on immedietly on the configuration page we see it is
             * fast but that is because the device is already primed to return a stream.
             *
             * On the connecting page the device has already turned off and the connecting window is so small we do not see the resulting
             * unparented view from the code below.
             */
            const selectedCamera = callClient.getState().deviceManager.selectedCamera;
            if (selectedCamera) {
                if (previewOn) {
                    yield onDisposeLocalStreamView();
                }
                else {
                    yield callClient.createView(undefined, undefined, {
                        source: selectedCamera,
                        mediaStreamType: 'Video'
                    }, options);
                }
            }
        }
    });
    const onSelectMicrophone = (device) => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!deviceManager) {
            return;
        }
        return deviceManager.selectMicrophone(device);
    });
    const onSelectSpeaker = (device) => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!deviceManager) {
            return;
        }
        return deviceManager.selectSpeaker(device);
    });
    const onSelectCamera = (device, options) => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!deviceManager) {
            return;
        }
        if (call && _isInCall(call.state)) {
            deviceManager.selectCamera(device);
            const stream = call.localVideoStreams.find(stream => stream.mediaStreamType === 'Video');
            yield (stream === null || stream === void 0 ? void 0 : stream.switchSource(device));
            yield Promise.all([
                /// TODO: TEMPORARY SOLUTION
                /// The Calling SDK needs to wait until the stream is ready before resolving the switchSource promise.
                /// This is a temporary solution to wait for the stream to be ready before resolving the promise.
                /// This allows the onSelectCamera to be throttled to prevent the streams from getting in to a frozen state
                /// if the user switches cameras too rapidly.
                /// This is to be removed once the Calling SDK has issued a fix.
                stream === null || stream === void 0 ? void 0 : stream.getMediaStream(),
                // An extra wait here is also necessary to prevent the remote stream freezing issue.
                // If this is removed, please test switching cameras rapidly won't cause stream to freeze for remote users.
                // When this mitigation was introduced, the repro interval time that caused the issue was:
                // - iPhone 11, safari, v8.3.1: 750ms
                // - Pixel 6, chrome, Android 15: 400ms
                // - Windows 11, edge: 100ms
                new Promise(resolve => setTimeout(resolve, 1000))
            ]);
        }
        else {
            const previewOn = _isPreviewOn(callClient.getState().deviceManager);
            if (!previewOn) {
                deviceManager.selectCamera(device);
                return;
            }
            yield onDisposeLocalStreamView();
            deviceManager.selectCamera(device);
            yield callClient.createView(undefined, undefined, {
                source: device,
                mediaStreamType: 'Video'
            }, options);
        }
    });
    const onRaiseHand = () => __awaiter$_(void 0, void 0, void 0, function* () { var _a; return yield ((_a = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.RaiseHand)) === null || _a === void 0 ? void 0 : _a.raiseHand()); });
    const onLowerHand = () => __awaiter$_(void 0, void 0, void 0, function* () { var _b; return yield ((_b = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.RaiseHand)) === null || _b === void 0 ? void 0 : _b.lowerHand()); });
    const onToggleRaiseHand = () => __awaiter$_(void 0, void 0, void 0, function* () {
        const raiseHandFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.RaiseHand);
        const localUserId = callClient.getState().userId;
        const isLocalRaisedHand = raiseHandFeature === null || raiseHandFeature === void 0 ? void 0 : raiseHandFeature.getRaisedHands().find(publishedState => toFlatCommunicationIdentifier(publishedState.identifier) === toFlatCommunicationIdentifier(localUserId));
        if (isLocalRaisedHand) {
            yield (raiseHandFeature === null || raiseHandFeature === void 0 ? void 0 : raiseHandFeature.lowerHand());
        }
        else {
            yield (raiseHandFeature === null || raiseHandFeature === void 0 ? void 0 : raiseHandFeature.raiseHand());
        }
    });
    const onReactionClick = (reaction) => __awaiter$_(void 0, void 0, void 0, function* () {
        var _c;
        if (reaction === 'like' || reaction === 'applause' || reaction === 'heart' || reaction === 'laugh' || reaction === 'surprised') {
            yield ((_c = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Reaction)) === null || _c === void 0 ? void 0 : _c.sendReaction({
                reactionType: reaction
            }));
        }
        else {
            console.warn(`Can not recognize ${reaction} as meeting reaction`);
        }
        return;
    });
    const onToggleMicrophone = () => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!call || !(_isInCall(call.state) || _isInLobbyOrConnecting(call.state))) {
            throw new Error(`Please invoke onToggleMicrophone after call is started`);
        }
        return call.isMuted ? yield call.unmute() : yield call.mute();
    });
    const onStartScreenShare = () => __awaiter$_(void 0, void 0, void 0, function* () { return yield (call === null || call === void 0 ? void 0 : call.startScreenSharing()); });
    const onStopScreenShare = () => __awaiter$_(void 0, void 0, void 0, function* () { return yield (call === null || call === void 0 ? void 0 : call.stopScreenSharing()); });
    const onToggleScreenShare = () => __awaiter$_(void 0, void 0, void 0, function* () { return (call === null || call === void 0 ? void 0 : call.isScreenSharingOn) ? yield onStopScreenShare() : yield onStartScreenShare(); });
    const onHangUp = (forEveryone) => __awaiter$_(void 0, void 0, void 0, function* () {
        return yield (call === null || call === void 0 ? void 0 : call.hangUp({
            forEveryone: forEveryone === true ? true : false
        }));
    });
    const onToggleHold = () => __awaiter$_(void 0, void 0, void 0, function* () { return (call === null || call === void 0 ? void 0 : call.state) === 'LocalHold' ? yield (call === null || call === void 0 ? void 0 : call.resume()) : yield (call === null || call === void 0 ? void 0 : call.hold()); });
    const onCreateLocalStreamView = (...args_1) => __awaiter$_(void 0, [...args_1], void 0, function* (options = {
        scalingMode: 'Crop',
        isMirrored: true
    }) {
        if (!call || call.localVideoStreams.length === 0) {
            return;
        }
        const callState = callClient.getState().calls[call.id];
        if (!callState) {
            return;
        }
        const localStream = callState.localVideoStreams.find(item => item.mediaStreamType === 'Video');
        const localScreenSharingStream = callState.localVideoStreams.find(item => item.mediaStreamType === 'ScreenSharing');
        let createViewResult = undefined;
        if (localStream && !localStream.view) {
            createViewResult = yield callClient.createView(call.id, undefined, localStream, options);
        }
        if (localScreenSharingStream && !localScreenSharingStream.view && call.isScreenSharingOn) {
            // Hardcoded `scalingMode` since it is highly unlikely that CONTOSO would ever want to use a different scaling mode for screenshare.
            // Using `Crop` would crop the contents of screenshare and `Stretch` would warp it.
            // `Fit` is the only mode that maintains the integrity of the screen being shared.
            createViewResult = yield callClient.createView(call.id, undefined, localScreenSharingStream, {
                scalingMode: 'Fit'
            });
        }
        return (createViewResult === null || createViewResult === void 0 ? void 0 : createViewResult.view) ? {
            view: createViewResult === null || createViewResult === void 0 ? void 0 : createViewResult.view
        } : undefined;
    });
    const onCreateRemoteStreamView = (userId_1, ...args_2) => __awaiter$_(void 0, [userId_1, ...args_2], void 0, function* (userId, options = {
        scalingMode: 'Crop'
    }) {
        if (!call) {
            return;
        }
        const callState = getCallStateIfExist(callClient.getState(), call.id);
        if (!callState) {
            return;
        }
        const participant = Object.values(callState.remoteParticipants).find(participant => toFlatCommunicationIdentifier(participant.identifier) === userId);
        if (!participant || !participant.videoStreams) {
            return;
        }
        /**
         * There is a bug from the calling sdk where if a user leaves and rejoins immediately
         * it adds 2 more potential streams this remote participant can use. The old 2 streams
         * still show as available and that is how we got a frozen stream in this case. The stopgap
         * until streams accurately reflect their availability is to always prioritize the latest streams of a certain type
         * e.g findLast instead of find
         */
        // Find the first available stream, if there is none, then get the first stream
        const remoteVideoStream = Object.values(participant.videoStreams).findLast(i => i.mediaStreamType === 'Video' && i.isAvailable) || Object.values(participant.videoStreams).findLast(i => i.mediaStreamType === 'Video');
        const screenShareStream = Object.values(participant.videoStreams).findLast(i => i.mediaStreamType === 'ScreenSharing' && i.isAvailable) || Object.values(participant.videoStreams).findLast(i => i.mediaStreamType === 'ScreenSharing');
        let createViewResult = undefined;
        if (remoteVideoStream && remoteVideoStream.isAvailable && !remoteVideoStream.view) {
            createViewResult = yield callClient.createView(call.id, participant.identifier, remoteVideoStream, options);
        }
        if (screenShareStream && screenShareStream.isAvailable && !screenShareStream.view) {
            // Hardcoded `scalingMode` since it is highly unlikely that CONTOSO would ever want to use a different scaling mode for screenshare.
            // Using `Crop` would crop the contents of screenshare and `Stretch` would warp it.
            // `Fit` is the only mode that maintains the integrity of the screen being shared.
            createViewResult = yield callClient.createView(call.id, participant.identifier, screenShareStream, {
                scalingMode: 'Fit'
            });
        }
        return (createViewResult === null || createViewResult === void 0 ? void 0 : createViewResult.view) ? {
            view: createViewResult === null || createViewResult === void 0 ? void 0 : createViewResult.view
        } : undefined;
    });
    const onDisposeRemoteStreamView = (userId) => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!call) {
            return;
        }
        const callState = getCallStateIfExist(callClient.getState(), call.id);
        if (!callState) {
            return;
        }
        const participant = Object.values(callState.remoteParticipants).find(participant => toFlatCommunicationIdentifier(participant.identifier) === userId);
        if (!participant || !participant.videoStreams) {
            return;
        }
        const remoteVideoStream = Object.values(participant.videoStreams).find(i => i.mediaStreamType === 'Video');
        const screenShareStream = Object.values(participant.videoStreams).find(i => i.mediaStreamType === 'ScreenSharing');
        if (remoteVideoStream && remoteVideoStream.view) {
            callClient.disposeView(call.id, participant.identifier, remoteVideoStream);
        }
        if (screenShareStream && screenShareStream.view) {
            callClient.disposeView(call.id, participant.identifier, screenShareStream);
        }
    });
    const onDisposeRemoteVideoStreamView = (userId) => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!call) {
            return;
        }
        const callState = getCallStateIfExist(callClient.getState(), call.id);
        if (!callState) {
            return;
        }
        const participant = Object.values(callState.remoteParticipants).find(participant => toFlatCommunicationIdentifier(participant.identifier) === userId);
        if (!participant || !participant.videoStreams) {
            return;
        }
        const remoteVideoStream = Object.values(participant.videoStreams).filter(i => i.mediaStreamType === 'Video');
        for (const stream of remoteVideoStream) {
            if (stream.view) {
                callClient.disposeView(call.id, participant.identifier, stream);
            }
        }
    });
    const onDisposeRemoteScreenShareStreamView = (userId) => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!call) {
            return;
        }
        const callState = getCallStateIfExist(callClient.getState(), call.id);
        if (!callState) {
            return;
        }
        const participant = Object.values(callState.remoteParticipants).find(participant => toFlatCommunicationIdentifier(participant.identifier) === userId);
        if (!participant || !participant.videoStreams) {
            return;
        }
        const screenShareStreams = Object.values(participant.videoStreams).filter(i => i.mediaStreamType === 'ScreenSharing');
        for (const stream of screenShareStreams) {
            if (stream.view) {
                callClient.disposeView(call.id, participant.identifier, stream);
            }
        }
    });
    const onDisposeLocalScreenShareStreamView = () => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!call) {
            return;
        }
        const callState = getCallStateIfExist(callClient.getState(), call.id);
        if (!callState) {
            return;
        }
        const screenShareStream = callState === null || callState === void 0 ? void 0 : callState.localVideoStreams.find(item => item.mediaStreamType === 'ScreenSharing');
        if (screenShareStream && screenShareStream.view) {
            callClient.disposeView(call.id, undefined, screenShareStream);
        }
    });
    const onDisposeLocalStreamView = () => __awaiter$_(void 0, void 0, void 0, function* () {
        // If the user is currently in a call, dispose of the local stream view attached to that call.
        const callState = call && callClient.getState().calls[call.id];
        const localStream = callState === null || callState === void 0 ? void 0 : callState.localVideoStreams.find(item => item.mediaStreamType === 'Video');
        if (call && callState && localStream) {
            callClient.disposeView(call.id, undefined, localStream);
        }
        // If the user is not in a call we currently assume any unparented view is a LocalPreview and stop all
        // since those are only used for LocalPreview currently.
        // TODO: we need to remember which LocalVideoStream was used for LocalPreview and dispose that one.
        yield disposeAllLocalPreviewViews(callClient);
    });
    const onSendDtmfTone = (dtmfTone) => __awaiter$_(void 0, void 0, void 0, function* () { return yield (call === null || call === void 0 ? void 0 : call.sendDtmf(dtmfTone)); });
    const notImplemented = () => {
        throw new Error('Not implemented, cannot call a method from an abstract object');
    };
    const onRemoveVideoBackgroundEffects = () => __awaiter$_(void 0, void 0, void 0, function* () {
        const stream = (call === null || call === void 0 ? void 0 : call.localVideoStreams.find(stream => stream.mediaStreamType === 'Video')) || (deviceManager === null || deviceManager === void 0 ? void 0 : deviceManager.getUnparentedVideoStreams().find(stream => stream.mediaStreamType === 'Video'));
        if (stream) {
            if (!(options === null || options === void 0 ? void 0 : options.onResolveVideoBackgroundEffectsDependency)) {
                throw new Error(`Video background effects dependency not resolved`);
            }
            else {
                return stream.feature(communicationCalling.Features.VideoEffects).stopEffects();
            }
        }
    });
    const onBlurVideoBackground = (backgroundBlurConfig) => __awaiter$_(void 0, void 0, void 0, function* () {
        var _d;
        const stream = (call === null || call === void 0 ? void 0 : call.localVideoStreams.find(stream => stream.mediaStreamType === 'Video')) || (deviceManager === null || deviceManager === void 0 ? void 0 : deviceManager.getUnparentedVideoStreams().find(stream => stream.mediaStreamType === 'Video'));
        if (stream) {
            if (!(options === null || options === void 0 ? void 0 : options.onResolveVideoBackgroundEffectsDependency)) {
                throw new Error(`Video background effects dependency not resolved`);
            }
            const createEffect = (options === null || options === void 0 ? void 0 : options.onResolveVideoBackgroundEffectsDependency) && ((_d = (yield options.onResolveVideoBackgroundEffectsDependency())) === null || _d === void 0 ? void 0 : _d.createBackgroundBlurEffect);
            return createEffect && stream.feature(communicationCalling.Features.VideoEffects).startEffects(createEffect(backgroundBlurConfig));
        }
    });
    const onReplaceVideoBackground = (backgroundReplacementConfig) => __awaiter$_(void 0, void 0, void 0, function* () {
        var _e;
        const stream = (call === null || call === void 0 ? void 0 : call.localVideoStreams.find(stream => stream.mediaStreamType === 'Video')) || (deviceManager === null || deviceManager === void 0 ? void 0 : deviceManager.getUnparentedVideoStreams().find(stream => stream.mediaStreamType === 'Video'));
        if (stream) {
            if (!(options === null || options === void 0 ? void 0 : options.onResolveVideoBackgroundEffectsDependency)) {
                throw new Error(`Video background effects dependency not resolved`);
            }
            const createEffect = (options === null || options === void 0 ? void 0 : options.onResolveVideoBackgroundEffectsDependency) && ((_e = (yield options.onResolveVideoBackgroundEffectsDependency())) === null || _e === void 0 ? void 0 : _e.createBackgroundReplacementEffect);
            return createEffect && stream.feature(communicationCalling.Features.VideoEffects).startEffects(createEffect(backgroundReplacementConfig));
        }
    });
    const onStartNoiseSuppressionEffect = () => __awaiter$_(void 0, void 0, void 0, function* () {
        var _f;
        const audioEffects = (options === null || options === void 0 ? void 0 : options.onResolveDeepNoiseSuppressionDependency) && ((_f = (yield options.onResolveDeepNoiseSuppressionDependency())) === null || _f === void 0 ? void 0 : _f.deepNoiseSuppressionEffect);
        const stream = call === null || call === void 0 ? void 0 : call.localAudioStreams.find(stream => stream.mediaStreamType === 'Audio');
        if (stream && audioEffects && audioEffects.noiseSuppression) {
            const audioEffectsFeature = stream.feature(communicationCalling.Features.AudioEffects);
            const isNoiseSuppressionSupported = yield audioEffectsFeature.isSupported(audioEffects.noiseSuppression);
            if (isNoiseSuppressionSupported) {
                return yield audioEffectsFeature.startEffects(audioEffects);
            }
            else {
                console.warn('Deep Noise Suppression is not supported on this platform.');
            }
        }
    });
    const onStopNoiseSuppressionEffect = () => __awaiter$_(void 0, void 0, void 0, function* () {
        const stream = call === null || call === void 0 ? void 0 : call.localAudioStreams.find(stream => stream.mediaStreamType === 'Audio');
        if (stream && (options === null || options === void 0 ? void 0 : options.onResolveDeepNoiseSuppressionDependency)) {
            const audioEffects = {
                noiseSuppression: true
            };
            return yield stream.feature(communicationCalling.Features.AudioEffects).stopEffects(audioEffects);
        }
    });
    const onStartCaptions = (options) => __awaiter$_(void 0, void 0, void 0, function* () {
        const captionsFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Captions).captions;
        yield (captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.startCaptions(options));
    });
    const onStopCaptions = () => __awaiter$_(void 0, void 0, void 0, function* () {
        const captionsFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Captions).captions;
        yield (captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.stopCaptions());
    });
    const onSetSpokenLanguage = (language) => __awaiter$_(void 0, void 0, void 0, function* () {
        const captionsFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Captions).captions;
        yield (captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.setSpokenLanguage(language));
    });
    const onSetCaptionLanguage = (language) => __awaiter$_(void 0, void 0, void 0, function* () {
        const captionsFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Captions).captions;
        yield captionsFeature.setCaptionLanguage(language);
    });
    const onSendRealTimeText = (text, isFinalized) => __awaiter$_(void 0, void 0, void 0, function* () {
        const realTimeTextFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.RealTimeText);
        yield (realTimeTextFeature === null || realTimeTextFeature === void 0 ? void 0 : realTimeTextFeature.sendRealTimeText(text, isFinalized));
    });
    const onSubmitSurvey = (survey) => __awaiter$_(void 0, void 0, void 0, function* () { return yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.CallSurvey).submitSurvey(survey)); });
    const onStartSpotlight = (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Spotlight).startSpotlight(participants));
    });
    const onStopSpotlight = (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Spotlight).stopSpotlight(participants));
    });
    const onStopAllSpotlight = () => __awaiter$_(void 0, void 0, void 0, function* () {
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Spotlight).stopAllSpotlight());
    });
    const onMuteParticipant = (userId) => __awaiter$_(void 0, void 0, void 0, function* () {
        if (call === null || call === void 0 ? void 0 : call.remoteParticipants) {
            call === null || call === void 0 ? void 0 : call.remoteParticipants.forEach((participant) => __awaiter$_(void 0, void 0, void 0, function* () {
                // Using toFlatCommunicationIdentifier to convert the CommunicationIdentifier to string
                // as _toCommunicationIdentifier(userId) comparison to participant.identifier did not work for this case
                if (toFlatCommunicationIdentifier(participant.identifier) === userId) {
                    yield participant.mute();
                }
            }));
        }
    });
    const onMuteAllRemoteParticipants = () => __awaiter$_(void 0, void 0, void 0, function* () {
        call === null || call === void 0 ? void 0 : call.muteAllRemoteParticipants();
    });
    const canStartSpotlight = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Capabilities).capabilities.spotlightParticipant.isPresent;
    const canRemoveSpotlight = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Capabilities).capabilities.removeParticipantsSpotlight.isPresent;
    const onStartLocalSpotlight = canStartSpotlight ? () => __awaiter$_(void 0, void 0, void 0, function* () {
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Spotlight).startSpotlight());
    }) : undefined;
    const onStopLocalSpotlight = () => __awaiter$_(void 0, void 0, void 0, function* () {
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Spotlight).stopSpotlight());
    });
    const onStartRemoteSpotlight = canStartSpotlight ? (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Spotlight).startSpotlight(participants));
    }) : undefined;
    const onStopRemoteSpotlight = canRemoveSpotlight ? (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.Spotlight).stopSpotlight(participants));
    }) : undefined;
    const onCreateTogetherModeStreamView = (...args_3) => __awaiter$_(void 0, [...args_3], void 0, function* (options = {
        scalingMode: 'Fit',
        isMirrored: false,
        viewKind: 'main'
    }) {
        if (!call) {
            return;
        }
        const callState = callClient.getState().calls[call.id];
        if (!callState) {
            return;
        }
        const togetherModeStreams = callState.togetherMode.streams;
        const togetherModeCreateViewResult = {};
        const mainVideoStream = togetherModeStreams.mainVideoStream;
        if (mainVideoStream && mainVideoStream.isAvailable && !mainVideoStream.view) {
            const createViewResult = yield callClient.createView(call.id, undefined, mainVideoStream, options);
            // SDK currently only supports 1 Video media stream type
            togetherModeCreateViewResult.mainVideoView = (createViewResult === null || createViewResult === void 0 ? void 0 : createViewResult.view) ? {
                view: createViewResult === null || createViewResult === void 0 ? void 0 : createViewResult.view
            } : undefined;
        }
        return togetherModeCreateViewResult;
    });
    const onDisposeTogetherModeStreamView = () => __awaiter$_(void 0, void 0, void 0, function* () {
        if (!call) {
            return;
        }
        const callState = callClient.getState().calls[call.id];
        if (!callState) {
            throw new Error(`Call Not Found: ${call.id}`);
        }
        const togetherModeStreams = callState.togetherMode.streams;
        if (!togetherModeStreams.mainVideoStream) {
            return;
        }
        if (togetherModeStreams.mainVideoStream.view) {
            callClient.disposeView(call.id, undefined, togetherModeStreams.mainVideoStream);
        }
    });
    const onSetTogetherModeSceneSize = (width, height) => {
        const togetherModeFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.TogetherMode);
        if (togetherModeFeature) {
            togetherModeFeature.sceneSize = {
                width,
                height
            };
        }
    };
    const onForbidAudio = (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).forbidAudio(participants));
    });
    const onPermitAudio = (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).permitAudio(participants));
    });
    const onForbidOthersAudio = () => __awaiter$_(void 0, void 0, void 0, function* () {
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).forbidOthersAudio());
    });
    const onPermitOthersAudio = () => __awaiter$_(void 0, void 0, void 0, function* () {
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).permitOthersAudio());
    });
    const onForbidVideo = (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).forbidVideo(participants));
    });
    const onPermitVideo = (userIds) => __awaiter$_(void 0, void 0, void 0, function* () {
        const participants = userIds === null || userIds === void 0 ? void 0 : userIds.map(userId => _toCommunicationIdentifier(userId));
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).permitVideo(participants));
    });
    const onForbidOthersVideo = () => __awaiter$_(void 0, void 0, void 0, function* () {
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).forbidOthersVideo());
    });
    const onPermitOthersVideo = () => __awaiter$_(void 0, void 0, void 0, function* () {
        yield (call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.MediaAccess).permitOthersVideo());
    });
    return {
        onHangUp,
        onToggleHold,
        onSelectCamera,
        onSelectMicrophone,
        onSelectSpeaker,
        onStartScreenShare,
        onStopScreenShare,
        onToggleCamera,
        onToggleMicrophone,
        onToggleScreenShare,
        onCreateLocalStreamView,
        onCreateRemoteStreamView,
        onStartLocalVideo,
        onDisposeRemoteStreamView,
        onDisposeLocalStreamView,
        onDisposeRemoteScreenShareStreamView,
        onDisposeLocalScreenShareStreamView,
        onDisposeRemoteVideoStreamView,
        onRaiseHand,
        onLowerHand,
        onToggleRaiseHand,
        onReactionClick,
        onAddParticipant: notImplemented,
        onRemoveParticipant: notImplemented,
        onStartCall: notImplemented,
        onSendDtmfTone,
        onRemoveVideoBackgroundEffects,
        onBlurVideoBackground,
        onReplaceVideoBackground,
        onStartNoiseSuppressionEffect,
        onStopNoiseSuppressionEffect,
        onStartCaptions,
        onStopCaptions,
        onSetCaptionLanguage,
        onSetSpokenLanguage,
        onSubmitSurvey,
        onStartSpotlight,
        onStopSpotlight,
        onStopAllSpotlight,
        onStartLocalSpotlight,
        onStopLocalSpotlight,
        onStartRemoteSpotlight,
        onStopRemoteSpotlight,
        onMuteParticipant,
        onMuteAllRemoteParticipants,
        onAcceptCall: notImplemented,
        onRejectCall: notImplemented,
        onCreateTogetherModeStreamView,
        onStartTogetherMode: notImplemented,
        onSetTogetherModeSceneSize,
        onDisposeTogetherModeStreamView,
        onForbidAudio,
        onPermitAudio,
        onForbidOthersAudio,
        onPermitOthersAudio,
        onForbidVideo,
        onPermitVideo,
        onForbidOthersVideo,
        onPermitOthersVideo,
        onSendRealTimeText
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$Z = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Create the default implementation of {@link CallingHandlers} for teams call.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const createDefaultCallingHandlers = memoizeOne((...args) => {
    const [callClient, callAgent, deviceManager, call, options] = args;
    const commonCallingHandlers = createDefaultCommonCallingHandlers(callClient, deviceManager, call, options);
    return Object.assign(Object.assign({}, commonCallingHandlers), { 
        // FIXME: onStartCall API should use string, not the underlying SDK types.
        onStartCall: (participants, options) => {
            return callAgent === null || callAgent === void 0 ? void 0 : callAgent.startCall(participants, options);
        }, onAddParticipant: (userId, options) => __awaiter$Z(void 0, void 0, void 0, function* () {
            const participant = _toCommunicationIdentifier(userId);
            if (communicationCommon.isPhoneNumberIdentifier(participant)) {
                call === null || call === void 0 ? void 0 : call.addParticipant(participant, options);
            }
            else if (communicationCommon.isCommunicationUserIdentifier(participant) || communicationCommon.isMicrosoftTeamsUserIdentifier(participant)) {
                call === null || call === void 0 ? void 0 : call.addParticipant(participant);
            }
        }), onRemoveParticipant: (userId) => __awaiter$Z(void 0, void 0, void 0, function* () {
            const participant = _toCommunicationIdentifier(userId);
            yield (call === null || call === void 0 ? void 0 : call.removeParticipant(participant));
        }), onAcceptCall: (incomingCallId, useVideo) => __awaiter$Z(void 0, void 0, void 0, function* () {
            const localVideoStream = useVideo ? yield createLocalVideoStream(callClient) : undefined;
            const incomingCall = callAgent === null || callAgent === void 0 ? void 0 : callAgent.incomingCalls.find((incomingCall) => incomingCall.id === incomingCallId);
            if (incomingCall) {
                yield incomingCall.accept(localVideoStream ? {
                    videoOptions: {
                        localVideoStreams: [localVideoStream]
                    }
                } : undefined);
            }
        }), onRejectCall: (incomingCallId) => __awaiter$Z(void 0, void 0, void 0, function* () {
            const incomingCall = callAgent === null || callAgent === void 0 ? void 0 : callAgent.incomingCalls.find((incomingCall) => incomingCall.id === incomingCallId);
            if (incomingCall) {
                yield incomingCall.reject();
            }
        }) });
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$Y = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Create the default implementation of {@link TeamsCallingHandlers} for teams call.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const createDefaultTeamsCallingHandlers = memoizeOne((callClient, callAgent, deviceManager, call, options) => {
    return Object.assign(Object.assign({}, createDefaultCommonCallingHandlers(callClient, deviceManager, call, options)), { onStartCall: (participants, options) => {
            if (!isTeamsCallParticipants(participants)) {
                throw new Error('CommunicationIdentifier in Teams call is not supported!');
            }
            if (callAgent) {
                // Remove when teams identity in stable support multiple participants
                return teamsSingleParticipantTrampoline(callAgent, participants, options);
            }
            return undefined;
        }, onRemoveParticipant: (userId) => __awaiter$Y(void 0, void 0, void 0, function* () {
            const participant = _toCommunicationIdentifier(userId);
            if (communicationCommon.isCommunicationUserIdentifier(participant)) {
                throw new Error('CommunicationIdentifier in Teams call is not supported!');
            }
            if (communicationCommon.isMicrosoftTeamsAppIdentifier(participant)) {
                throw new Error('Removing Microsoft Teams app identifier is not supported!');
            }
            yield (call === null || call === void 0 ? void 0 : call.removeParticipant(participant));
        }), onAcceptCall: (incomingCallId, useVideo) => __awaiter$Y(void 0, void 0, void 0, function* () {
            const localVideoStream = useVideo ? yield createLocalVideoStream(callClient) : undefined;
            const incomingCall = callAgent === null || callAgent === void 0 ? void 0 : callAgent.incomingCalls.find((incomingCall) => incomingCall.id === incomingCallId);
            if (incomingCall) {
                yield incomingCall.accept(localVideoStream ? {
                    videoOptions: {
                        localVideoStreams: [localVideoStream]
                    }
                } : undefined);
            }
        }), onRejectCall: (incomingCallId) => __awaiter$Y(void 0, void 0, void 0, function* () {
            const incomingCall = callAgent === null || callAgent === void 0 ? void 0 : callAgent.incomingCalls.find((incomingCall) => incomingCall.id === incomingCallId);
            if (incomingCall) {
                yield incomingCall.reject();
            }
        }), onStartTogetherMode: () => __awaiter$Y(void 0, void 0, void 0, function* () {
            if (!call) {
                return;
            }
            const callState = callClient.getState().calls[call.id];
            if (!callState) {
                return;
            }
            const togetherModeFeature = call === null || call === void 0 ? void 0 : call.feature(communicationCalling.Features.TogetherMode);
            yield (togetherModeFeature === null || togetherModeFeature === void 0 ? void 0 : togetherModeFeature.start());
        }) });
});
const teamsSingleParticipantTrampoline = (callAgent, participants, options) => {
    if (participants.length !== 1) {
        throw new Error('Only one participant is supported in Teams call!');
    }
    else {
        return callAgent.startCall(participants[0], options);
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Select the active errors from the state for the `Notification` component.
 *
 * Invariants:
 *   - `ErrorType` is never repeated in the returned errors.
 *   - Errors are returned in a fixed order by `ErrorType`.
 *
 * @public
 */
const notificationStackSelector = reselect.createSelector([getLatestErrors$1, getLatestNotifications$1, getDiagnostics, getDeviceManager$1, getCallState, getEnvironmentInfo$1, getMeetingConferencePhones, getAssignedBreakoutRoom$1], (latestErrors, latestNotifications, diagnostics, deviceManager, callStatus, environmentInfo, meetingConference, assignedBreakoutRoom) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u;
    // The order in which the errors are returned is significant: The `Notification` shows errors on the UI in that order.
    // There are several options for the ordering:
    //   - Sorted by when the errors happened (latest first / oldest first).
    //   - Stable sort by error type.
    //
    // We chose to stable sort by error type: We intend to show only a small number of errors on the UI and we do not
    // have timestamps for errors.
    const activeErrorMessages = [];
    const isSafari = () => {
        return (environmentInfo === null || environmentInfo === void 0 ? void 0 : environmentInfo.environment.browser) === 'safari';
    };
    const isMacOS = () => {
        return (environmentInfo === null || environmentInfo === void 0 ? void 0 : environmentInfo.environment.platform) === 'mac';
    };
    // Errors reported via diagnostics are more reliable than from API method failures, so process those first.
    let isTeamsMeetingWithPhones = false;
    if (meetingConference && meetingConference.length > 0) {
        isTeamsMeetingWithPhones = true;
    }
    if (((_a = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.network.latest.networkReceiveQuality) === null || _a === void 0 ? void 0 : _a.value) === communicationCalling.DiagnosticQuality.Bad || ((_b = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.network.latest.networkReceiveQuality) === null || _b === void 0 ? void 0 : _b.value) === communicationCalling.DiagnosticQuality.Poor) {
        if (isTeamsMeetingWithPhones) {
            activeErrorMessages.push({
                type: 'teamsMeetingCallNetworkQualityLow'
            });
        }
        else {
            activeErrorMessages.push({
                type: 'callNetworkQualityLow'
            });
        }
    }
    if (((_c = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.noSpeakerDevicesEnumerated) === null || _c === void 0 ? void 0 : _c.value) === true) {
        activeErrorMessages.push({
            type: 'callNoSpeakerFound'
        });
    }
    if (((_d = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.noMicrophoneDevicesEnumerated) === null || _d === void 0 ? void 0 : _d.value) === true) {
        activeErrorMessages.push({
            type: 'callNoMicrophoneFound'
        });
    }
    if (((_e = deviceManager.deviceAccess) === null || _e === void 0 ? void 0 : _e.audio) === false && isSafari()) {
        activeErrorMessages.push({
            type: 'callMicrophoneAccessDeniedSafari'
        });
    }
    if (((_f = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphonePermissionDenied) === null || _f === void 0 ? void 0 : _f.value) === true && isMacOS()) {
        activeErrorMessages.push({
            type: 'callMacOsMicrophoneAccessDenied'
        });
    }
    else if (((_g = deviceManager.deviceAccess) === null || _g === void 0 ? void 0 : _g.audio) === false && !isSafari() || ((_h = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphonePermissionDenied) === null || _h === void 0 ? void 0 : _h.value) === true && !isMacOS()) {
        activeErrorMessages.push({
            type: 'callMicrophoneAccessDenied'
        });
    }
    const microphoneMuteUnexpectedlyDiagnostic = (diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphoneMuteUnexpectedly) || (diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphoneNotFunctioning);
    if (microphoneMuteUnexpectedlyDiagnostic) {
        if (microphoneMuteUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Bad) {
            // Inform the user that microphone stopped working and inform them to start microphone again
            activeErrorMessages.push({
                type: 'callMicrophoneMutedBySystem'
            });
        }
        else if (microphoneMuteUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Good) {
            // Inform the user that microphone recovered
            activeErrorMessages.push({
                type: 'callMicrophoneUnmutedBySystem'
            });
        }
    }
    const cameraStoppedUnexpectedlyDiagnostic = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.cameraStoppedUnexpectedly;
    if (cameraStoppedUnexpectedlyDiagnostic) {
        if (cameraStoppedUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Bad) {
            // Inform the user that camera stopped working and inform them to start video again
            activeErrorMessages.push({
                type: 'callVideoStoppedBySystem'
            });
        }
        else if (cameraStoppedUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Good) {
            // Inform the user that camera recovered
            activeErrorMessages.push({
                type: 'callVideoRecoveredBySystem'
            });
        }
    }
    if (((_j = deviceManager.deviceAccess) === null || _j === void 0 ? void 0 : _j.video) === false && isSafari()) {
        activeErrorMessages.push({
            type: 'callCameraAccessDeniedSafari'
        });
    }
    else if (((_k = deviceManager.deviceAccess) === null || _k === void 0 ? void 0 : _k.video) === false) {
        activeErrorMessages.push({
            type: 'callCameraAccessDenied'
        });
    }
    else {
        if (((_l = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.cameraFreeze) === null || _l === void 0 ? void 0 : _l.value) === true) {
            activeErrorMessages.push({
                type: 'cameraFrozenForRemoteParticipants'
            });
        }
    }
    /**
     * show the Mac specific strings if the platform is detected as mac
     */
    if (((_m = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.cameraPermissionDenied) === null || _m === void 0 ? void 0 : _m.value) === true && isMacOS()) {
        activeErrorMessages.push({
            type: 'callMacOsCameraAccessDenied'
        });
    }
    /**
     * This UFD only works on mac still so we should only see it fire on mac.
     */
    if (((_o = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.screenshareRecordingDisabled) === null || _o === void 0 ? void 0 : _o.value) === true && isMacOS()) {
        activeErrorMessages.push({
            type: 'callMacOsScreenShareAccessDenied'
        });
    }
    else if (((_p = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.screenshareRecordingDisabled) === null || _p === void 0 ? void 0 : _p.value) === true) {
        activeErrorMessages.push({
            type: 'startScreenShareGeneric'
        });
    }
    // Prefer to show errors with privacy implications.
    appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.stopVideo', 'stopVideoGeneric');
    appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.mute', 'muteGeneric');
    appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.stopScreenSharing', 'stopScreenShareGeneric');
    if (((_q = latestErrors['Call.startVideo']) === null || _q === void 0 ? void 0 : _q.message) === 'Call.startVideo: Video operation failure SourceUnavailableError') {
        appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.startVideo', 'callCameraAlreadyInUse');
    }
    else if (((_r = latestErrors['Call.startVideo']) === null || _r === void 0 ? void 0 : _r.message) === 'Call.startVideo: Video operation failure permissionDeniedError') {
        appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.startVideo', 'callCameraAccessDenied');
    }
    else {
        appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.startVideo', 'startVideoGeneric');
    }
    appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.unmute', 'unmuteGeneric');
    if (latestNotifications && !latestNotifications['capabilityUnmuteMicAbsent'] && !latestNotifications['capabilityUnmuteMicPresent']) {
        appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.mutedByOthers', 'mutedByRemoteParticipant');
    }
    appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'VideoEffectsFeature.startEffects', 'unableToStartVideoEffect');
    if (((_s = latestErrors['CallAgent.join']) === null || _s === void 0 ? void 0 : _s.message) === 'CallAgent.join: Invalid meeting link') {
        appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'CallAgent.join', 'failedToJoinCallInvalidMeetingLink');
    }
    else {
        appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'CallAgent.join', 'failedToJoinCallGeneric');
    }
    if ((_t = latestErrors['Call.feature']) === null || _t === void 0 ? void 0 : _t.message.match(/Call\.feature: startSpotlight failed\. \d+ is the max number of participants that can be Spotlighted/g)) {
        appendActiveErrorIfDefined$1(activeErrorMessages, latestErrors, 'Call.feature', 'startSpotlightWhileMaxParticipantsAreSpotlighted');
    }
    //below is for active notifications
    const activeNotifications = [];
    if (((_u = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.speakingWhileMicrophoneIsMuted) === null || _u === void 0 ? void 0 : _u.value) &&
        // only show this notification when the participant is in an active call and not on screens such as Lobby
        callStatus === 'Connected') {
        activeNotifications.push({
            type: 'speakingWhileMuted',
            timestamp: new Date(Date.now()),
            autoDismiss: true,
            ariaLive: 'off' // "You're muted" is too noisy, so we don't want to announce it.
        });
    }
    if (latestNotifications['assignedBreakoutRoomOpened']) {
        activeNotifications.push({
            type: 'assignedBreakoutRoomOpened',
            timestamp: latestNotifications['assignedBreakoutRoomOpened'].timestamp
        });
    }
    if (latestNotifications['assignedBreakoutRoomOpenedPromptJoin'] && assignedBreakoutRoom) {
        activeNotifications.push({
            type: 'assignedBreakoutRoomOpenedPromptJoin',
            timestamp: latestNotifications['assignedBreakoutRoomOpenedPromptJoin'].timestamp,
            onClickPrimaryButton: () => assignedBreakoutRoom.join()
        });
    }
    if (latestNotifications['assignedBreakoutRoomChanged']) {
        activeNotifications.push({
            type: 'assignedBreakoutRoomChanged',
            timestamp: latestNotifications['assignedBreakoutRoomChanged'].timestamp
        });
    }
    if (latestNotifications['assignedBreakoutRoomClosed']) {
        activeNotifications.push({
            type: 'assignedBreakoutRoomClosed',
            timestamp: latestNotifications['assignedBreakoutRoomClosed'].timestamp
        });
    }
    if (latestNotifications['breakoutRoomJoined']) {
        activeNotifications.push({
            type: 'breakoutRoomJoined',
            timestamp: latestNotifications['breakoutRoomJoined'].timestamp
        });
    }
    if (latestNotifications['breakoutRoomClosingSoon']) {
        activeNotifications.push({
            type: 'breakoutRoomClosingSoon',
            timestamp: latestNotifications['breakoutRoomClosingSoon'].timestamp
        });
    }
    if (latestNotifications['capabilityTurnVideoOnPresent']) {
        activeNotifications.push({
            type: 'capabilityTurnVideoOnPresent',
            timestamp: latestNotifications['capabilityTurnVideoOnPresent'].timestamp
        });
    }
    if (latestNotifications['capabilityTurnVideoOnAbsent']) {
        activeNotifications.push({
            type: 'capabilityTurnVideoOnAbsent',
            timestamp: latestNotifications['capabilityTurnVideoOnAbsent'].timestamp
        });
    }
    if (latestNotifications['capabilityUnmuteMicPresent']) {
        activeNotifications.push({
            type: 'capabilityUnmuteMicPresent',
            timestamp: latestNotifications['capabilityUnmuteMicPresent'].timestamp
        });
    }
    if (latestNotifications['capabilityUnmuteMicAbsent']) {
        activeNotifications.push({
            type: 'capabilityUnmuteMicAbsent',
            timestamp: latestNotifications['capabilityUnmuteMicAbsent'].timestamp
        });
    }
    if (latestNotifications['togetherModeStarted']) {
        activeNotifications.push({
            type: 'togetherModeStarted',
            timestamp: latestNotifications['togetherModeStarted'].timestamp,
            autoDismiss: true
        });
    }
    if (latestNotifications['togetherModeEnded']) {
        activeNotifications.push({
            type: 'togetherModeEnded',
            timestamp: latestNotifications['togetherModeEnded'].timestamp,
            autoDismiss: true
        });
    }
    return {
        activeErrorMessages: activeErrorMessages,
        activeNotifications: activeNotifications
    };
});
const appendActiveErrorIfDefined$1 = (activeErrorMessages, latestErrors, target, activeErrorType) => {
    if (latestErrors[target] === undefined) {
        return;
    }
    activeErrorMessages.push({
        type: activeErrorType,
        timestamp: latestErrors[target].timestamp
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Select the active and removed incoming calls from the stateful client for the IncomingCallNotificationStackComponent.
 * @public
 */
const incomingCallStackSelector = reselect.createSelector([getIncomingCalls, getRemovedIncomingCalls, getDeviceManager$1], (incomingCalls, removedIncomingCalls, deviceManager) => {
    // Convert incoming call state to active incoming call
    const componentIncomingCalls = incomingCalls.map((incomingCall) => {
        return Object.assign(Object.assign({}, incomingCall), { callerInfo: {
                displayName: incomingCall.callerInfo.displayName || 'Unknown Caller'
            }, videoAvailable: incomingCall.callerInfo.identifier && communicationCommon.isPhoneNumberIdentifier(incomingCall.callerInfo.identifier) || (deviceManager === null || deviceManager === void 0 ? void 0 : deviceManager.cameras.length) === 0 ? false : true });
    });
    const componentRemovedIncomingCalls = removedIncomingCalls.map((incomingCall) => {
        return Object.assign(Object.assign({}, incomingCall), { callerInfo: {
                displayName: incomingCall.callerInfo.displayName || 'Unknown Caller'
            }, videoAvailable: incomingCall.callerInfo.identifier && communicationCommon.isPhoneNumberIdentifier(incomingCall.callerInfo.identifier) || (deviceManager === null || deviceManager === void 0 ? void 0 : deviceManager.cameras.length) === 0 ? false : true });
    });
    return {
        activeIncomingCalls: componentIncomingCalls,
        removedIncomingCalls: componentRemovedIncomingCalls
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Selector for all remote participants in a call, active and inactive.
 * @public
 */
const allRemoteParticipantsSelector = reselect__namespace.createSelector([getRemoteParticipants$1, getRemoteParticipantsEnded], (remoteParticipants, remoteParticipantsEnded) => {
    // Combine remoteParticipants and remoteParticipantsEnded into a single object
    return Object.assign(Object.assign({}, remoteParticipants), remoteParticipantsEnded);
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Selector for {@link StartCaptionsButton} component.
 *
 * @public
 */
const startCaptionsButtonSelector = reselect__namespace.createSelector([getCaptionsStatus$1, getCurrentCaptionLanguage, getCurrentSpokenLanguage], (isCaptionsFeatureActive, currentCaptionLanguage, currentSpokenLanguage) => {
    return {
        checked: isCaptionsFeatureActive !== null && isCaptionsFeatureActive !== void 0 ? isCaptionsFeatureActive : false,
        currentCaptionLanguage: currentCaptionLanguage !== null && currentCaptionLanguage !== void 0 ? currentCaptionLanguage : '',
        currentSpokenLanguage: currentSpokenLanguage !== null && currentSpokenLanguage !== void 0 ? currentSpokenLanguage : 'en-us'
    };
});
/**
 * Selector for Changing caption language and spoken language
 *
 * @public
 */
const captionSettingsSelector = reselect__namespace.createSelector([getSupportedCaptionLanguages, getCurrentCaptionLanguage, getSupportedSpokenLanguages, getCurrentSpokenLanguage, getCaptionsStatus$1], (supportedCaptionLanguages, currentCaptionLanguage, supportedSpokenLanguages, currentSpokenLanguage, isCaptionsFeatureActive) => {
    return {
        supportedCaptionLanguages: supportedCaptionLanguages !== null && supportedCaptionLanguages !== void 0 ? supportedCaptionLanguages : [],
        currentCaptionLanguage: currentCaptionLanguage !== null && currentCaptionLanguage !== void 0 ? currentCaptionLanguage : 'en',
        supportedSpokenLanguages: supportedSpokenLanguages !== null && supportedSpokenLanguages !== void 0 ? supportedSpokenLanguages : ['en-us'],
        currentSpokenLanguage: currentSpokenLanguage !== null && currentSpokenLanguage !== void 0 ? currentSpokenLanguage : 'en-us',
        isCaptionsFeatureActive: isCaptionsFeatureActive !== null && isCaptionsFeatureActive !== void 0 ? isCaptionsFeatureActive : false
    };
});
/**
 * Selector for {@link CaptionsBanner} component.
 *
 * @public
 */
const captionsBannerSelector = reselect__namespace.createSelector([getCaptions, getRealTimeText, getCaptionsStatus$1, getRealTimeTextStatus, getStartCaptionsInProgress, allRemoteParticipantsSelector, getDisplayName$2, getIdentifier], (captions, realTimeTexts, isCaptionsFeatureActive, isRealTimeTextActive, startCaptionsInProgress, allRemoteParticipants, displayName, identifier) => {
    var _a, _b;
    const captionsInfo = captions === null || captions === void 0 ? void 0 : captions.map((c, index) => {
        const userId = getCaptionsSpeakerIdentifier(c);
        let finalDisplayName;
        if (userId === identifier) {
            finalDisplayName = displayName;
        }
        else {
            finalDisplayName = getRemoteParticipantDisplayName(userId, allRemoteParticipants);
        }
        return {
            id: (finalDisplayName !== null && finalDisplayName !== void 0 ? finalDisplayName : 'Unnamed Participant') + index,
            displayName: finalDisplayName !== null && finalDisplayName !== void 0 ? finalDisplayName : 'Unnamed Participant',
            captionText: c.captionText,
            userId,
            createdTimeStamp: c.timestamp,
            isFinalized: c.resultType === 'Final'
        };
    });
    const completedRealTimeTexts = (_a = realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.completedMessages) === null || _a === void 0 ? void 0 : _a.filter(rtt => rtt.message !== '').map(rtt => {
        const userId = getRealTimeTextSpeakerIdentifier(rtt);
        return {
            id: rtt.sequenceId,
            displayName: getRealTimeTextDisplayName(rtt, identifier, allRemoteParticipants, displayName, userId),
            message: rtt.message,
            userId,
            isTyping: rtt.resultType === 'Partial',
            isMe: rtt.isMe,
            finalizedTimeStamp: rtt.updatedTimestamp
        };
    });
    const inProgressRealTimeTexts = (_b = realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.currentInProgress) === null || _b === void 0 ? void 0 : _b.filter(rtt => rtt.message !== '').map(rtt => {
        const userId = getRealTimeTextSpeakerIdentifier(rtt);
        return {
            id: rtt.sequenceId,
            displayName: getRealTimeTextDisplayName(rtt, identifier, allRemoteParticipants, displayName, userId),
            message: rtt.message,
            userId,
            isTyping: rtt.resultType === 'Partial',
            isMe: rtt.isMe,
            finalizedTimeStamp: rtt.updatedTimestamp
        };
    });
    const myInProgress = (realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.myInProgress) && realTimeTexts.myInProgress.message !== '' ? {
        id: realTimeTexts.myInProgress.sequenceId,
        displayName: displayName,
        message: realTimeTexts.myInProgress.message,
        userId: identifier,
        isTyping: realTimeTexts.myInProgress.resultType === 'Partial',
        isMe: true,
        finalizedTimeStamp: realTimeTexts.myInProgress.updatedTimestamp
    } : undefined;
    // find the last final local real time text caption if myInProgress is not available
    let latestLocalRealTimeText;
    if (!myInProgress) {
        latestLocalRealTimeText = realTimeTexts && realTimeTexts.completedMessages && realTimeTexts.completedMessages.slice().reverse().find(rtt => rtt.isMe);
    }
    return {
        captions: captionsInfo !== null && captionsInfo !== void 0 ? captionsInfo : [],
        realTimeTexts: {
            completedMessages: completedRealTimeTexts,
            currentInProgress: inProgressRealTimeTexts,
            myInProgress: myInProgress
        },
        isCaptionsOn: isCaptionsFeatureActive !== null && isCaptionsFeatureActive !== void 0 ? isCaptionsFeatureActive : false,
        startCaptionsInProgress: startCaptionsInProgress !== null && startCaptionsInProgress !== void 0 ? startCaptionsInProgress : false,
        isRealTimeTextOn: isRealTimeTextActive !== null && isRealTimeTextActive !== void 0 ? isRealTimeTextActive : false,
        latestLocalRealTimeText: (myInProgress !== null && myInProgress !== void 0 ? myInProgress : latestLocalRealTimeText)
    };
});
const getCaptionsSpeakerIdentifier = (captions) => {
    return captions.speaker.identifier ? toFlatCommunicationIdentifier(captions.speaker.identifier) : '';
};
const getRealTimeTextSpeakerIdentifier = (realTimeText) => {
    return realTimeText.sender.identifier ? toFlatCommunicationIdentifier(realTimeText.sender.identifier) : '';
};
const getRealTimeTextDisplayName = (realTimeText, identifier, allRemoteParticipants, displayName, userId) => {
    let finalDisplayName;
    if (userId === identifier) {
        finalDisplayName = displayName;
    }
    else {
        finalDisplayName = getRemoteParticipantDisplayName(userId, allRemoteParticipants);
    }
    return finalDisplayName !== null && finalDisplayName !== void 0 ? finalDisplayName : 'Unnamed Participant';
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$X = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * ProxyDeviceManager proxies DeviceManager and subscribes to all events that affect device manager state. State updates
 * are set on the provided context. Also any queries for state are proxied and stored in state as well. Only one device
 * manager should exist for a given CallClient so if CallClient.getDeviceManager is called multiple times, either a
 * cached ProxyDeviceManager should be returned or the existing ProxyDeviceManager should be destructed via destructor()
 * and a new ProxyDeviceManager created.
 */
class ProxyDeviceManager {
    constructor(deviceManager, context) {
        this.setDeviceManager = () => {
            // isSpeakerSelectionAvailable, selectedMicrophone, and selectedSpeaker are properties on DeviceManager. Since they
            // are not functions we can't proxy them so we'll update whenever we think they may need updating such as at
            // construction time or when certain events happen.
            this._context.setDeviceManagerIsSpeakerSelectionAvailable(this._deviceManager.isSpeakerSelectionAvailable);
            this._context.setDeviceManagerSelectedMicrophone(this._deviceManager.selectedMicrophone);
            this._context.setDeviceManagerSelectedSpeaker(this._deviceManager.selectedSpeaker);
        };
        this.subscribe = () => {
            this._deviceManager.on('videoDevicesUpdated', this.videoDevicesUpdated);
            this._deviceManager.on('audioDevicesUpdated', this.audioDevicesUpdated);
            this._deviceManager.on('selectedMicrophoneChanged', this.selectedMicrophoneChanged);
            this._deviceManager.on('selectedSpeakerChanged', this.selectedSpeakerChanged);
            // Subscribe to browser camera permissions changes
            if (navigator.permissions) {
                try {
                    navigator.permissions.query({
                        name: 'camera'
                    }).then((cameraPermissions) => {
                        cameraPermissions.addEventListener('change', this.permissionsApiStateChangeHandler);
                    });
                }
                catch (e) {
                    console.info('Could not subscribe to Permissions API Camera changed events, API is not supported by browser', e);
                }
                // Subscribe to browser microphone permissions changes
                try {
                    navigator.permissions.query({
                        name: 'microphone'
                    }).then((micPermissions) => {
                        micPermissions.addEventListener('change', this.permissionsApiStateChangeHandler);
                    });
                }
                catch (e) {
                    console.info('Could not subscribe to Permissions API Microphone changed events, API is not supported by browser', e);
                }
            }
        };
        /**
         * This is used to unsubscribe DeclarativeDeviceManager from the DeviceManager events.
         */
        this.unsubscribe = () => {
            this._deviceManager.off('videoDevicesUpdated', this.videoDevicesUpdated);
            this._deviceManager.off('audioDevicesUpdated', this.audioDevicesUpdated);
            this._deviceManager.off('selectedMicrophoneChanged', this.selectedMicrophoneChanged);
            this._deviceManager.off('selectedSpeakerChanged', this.selectedSpeakerChanged);
            if (navigator.permissions) {
                // Unsubscribe from browser camera permissions changes
                try {
                    navigator.permissions.query({
                        name: 'camera'
                    }).then((cameraPermissions) => {
                        cameraPermissions.removeEventListener('change', this.permissionsApiStateChangeHandler);
                    });
                }
                catch (e) {
                    console.info('Could not Unsubscribe to Permissions API Camera changed events, API is not supported by browser', e);
                }
                // Unsubscribe from browser microphone permissions changes
                try {
                    navigator.permissions.query({
                        name: 'microphone'
                    }).then((micPermissions) => {
                        micPermissions.removeEventListener('change', this.permissionsApiStateChangeHandler);
                    });
                }
                catch (e) {
                    console.info('Could not Unsubscribe to Permissions API Camera changed events, API is not supported by browser', e);
                }
            }
        };
        this.permissionsApiStateChangeHandler = () => __awaiter$X(this, void 0, void 0, function* () {
            yield this.updateDevicePermissionState();
        });
        /**
         * Used to set a camera inside the proxy device manager.
         *
         * @param videoDeviceInfo VideoDeviceInfo
         */
        this.selectCamera = (videoDeviceInfo) => {
            this._context.setDeviceManagerSelectedCamera(videoDeviceInfo);
        };
        this.videoDevicesUpdated = () => __awaiter$X(this, void 0, void 0, function* () {
            // Device Manager always has a camera with '' name if there are no real camera devices available.
            // We don't want to show that in the UI.
            const realCameras = (yield this._deviceManager.getCameras()).filter(c => !!c.name);
            this._context.setDeviceManagerCameras(dedupeById(realCameras));
        });
        this.audioDevicesUpdated = () => __awaiter$X(this, void 0, void 0, function* () {
            this._context.setDeviceManagerMicrophones(dedupeById(yield this._deviceManager.getMicrophones()));
            if (this._deviceManager.isSpeakerSelectionAvailable) {
                this._context.setDeviceManagerSpeakers(dedupeById(yield this._deviceManager.getSpeakers()));
            }
        });
        this.selectedMicrophoneChanged = () => {
            this._context.setDeviceManagerSelectedMicrophone(this._deviceManager.selectedMicrophone);
        };
        this.selectedSpeakerChanged = () => {
            this._context.setDeviceManagerSelectedSpeaker(this._deviceManager.selectedSpeaker);
        };
        this.updateDevicePermissionState = (sdkDeviceAccessState) => __awaiter$X(this, void 0, void 0, function* () {
            let hasCameraPermission = !!(sdkDeviceAccessState === null || sdkDeviceAccessState === void 0 ? void 0 : sdkDeviceAccessState.video);
            let hasMicPermission = !!(sdkDeviceAccessState === null || sdkDeviceAccessState === void 0 ? void 0 : sdkDeviceAccessState.audio);
            // Supplement the SDK values with values from the Permissions API to get a better understanding of the device
            // permission state. The SDK only uses the getUserMedia API to determine the device permission state. However,
            // this returns false if the camera is in use by another application. The Permissions API can provide more
            // information about the device permission state, but is not supported yet in Firefox or Android WebView.
            // Note: It also has the limitation where it cannot detect if the device is blocked by the Operating System
            // permissions.
            if (navigator.permissions) {
                try {
                    const [cameraPermissions, micPermissions] = yield Promise.all([navigator.permissions.query({
                            name: 'camera'
                        }), navigator.permissions.query({
                            name: 'microphone'
                        })]);
                    hasCameraPermission = cameraPermissions.state === 'granted';
                    hasMicPermission = micPermissions.state === 'granted';
                }
                catch (e) {
                    console.info('Permissions API is not supported by browser', e);
                }
            }
            this._context.setDeviceManagerDeviceAccess({
                video: hasCameraPermission,
                audio: hasMicPermission
            });
            this.setDeviceManager();
        });
        this._deviceManager = deviceManager;
        this._context = context;
        this.setDeviceManager();
        this.subscribe();
    }
    get(target, prop) {
        switch (prop) {
            case 'getCameras':
                {
                    return this._context.withAsyncErrorTeedToState(() => {
                        return target.getCameras().then((cameras) => {
                            // Device Manager always has a camera with '' name if there are no real camera devices available.
                            // We don't want to show that in the UI.
                            const realCameras = cameras.filter(c => !!c.name);
                            this._context.setDeviceManagerCameras(dedupeById(realCameras));
                            return realCameras;
                        });
                    }, 'DeviceManager.getCameras');
                }
            case 'getMicrophones':
                {
                    return this._context.withAsyncErrorTeedToState(() => {
                        return target.getMicrophones().then((microphones) => {
                            this._context.setDeviceManagerMicrophones(dedupeById(microphones));
                            return microphones;
                        });
                    }, 'DeviceManager.getMicrophones');
                }
            case 'getSpeakers':
                {
                    return this._context.withAsyncErrorTeedToState(() => {
                        return target.getSpeakers().then((speakers) => {
                            this._context.setDeviceManagerSpeakers(dedupeById(speakers));
                            return speakers;
                        });
                    }, 'DeviceManager.getSpeakers');
                }
            case 'selectMicrophone':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => {
                        return target.selectMicrophone(...args).then(() => {
                            this._context.setDeviceManagerSelectedMicrophone(target.selectedMicrophone);
                        });
                    }, 'DeviceManager.selectMicrophone');
                }
            case 'selectSpeaker':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => {
                        return target.selectSpeaker(...args).then(() => {
                            this._context.setDeviceManagerSelectedSpeaker(target.selectedSpeaker);
                        });
                    }, 'DeviceManager.selectSpeaker');
                }
            case 'askDevicePermission':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => {
                        return target.askDevicePermission(...args).then((deviceAccess) => __awaiter$X(this, void 0, void 0, function* () {
                            yield this.updateDevicePermissionState(deviceAccess);
                            return deviceAccess;
                        }));
                    }, 'DeviceManager.askDevicePermission');
                }
            default:
                return Reflect.get(target, prop);
        }
    }
}
// TODO: Remove this when SDK no longer returns duplicate audio and video devices
/** Helper function to dedupe duplicate audio and video devices obtained from SDK */
const dedupeById = (devices) => {
    const ids = new Set();
    const uniqueDevices = [];
    devices.forEach((device) => {
        if (!ids.has(device.id)) {
            uniqueDevices.push(device);
            ids.add(device.id);
        }
    });
    return uniqueDevices;
};
/**
 * Creates a declarative DeviceManager by proxying DeviceManager with ProxyDeviceManager. The declarative DeviceManager
 * will put state updates in the given context.
 *
 * @param deviceManager - DeviceManager from SDK
 * @param context - CallContext from StatefulCallClient
 *
 * @private
 */
const deviceManagerDeclaratify = (deviceManager, context, internalContext) => {
    const proxyDeviceManager = new ProxyDeviceManager(deviceManager, context);
    Object.defineProperty(deviceManager, 'unsubscribe', {
        configurable: false,
        value: () => proxyDeviceManager.unsubscribe()
    });
    Object.defineProperty(deviceManager, 'selectCamera', {
        configurable: false,
        value: (videoDeviceInfo) => proxyDeviceManager.selectCamera(videoDeviceInfo)
    });
    Object.defineProperty(deviceManager, 'getUnparentedVideoStreams', {
        configurable: false,
        value: () => internalContext.getUnparentedRenderInfos()
    });
    return new Proxy(deviceManager, proxyDeviceManager);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 */
const _isACSCall = (call) => {
    return !call.kind || call.kind === 'Call';
};
/**
 * @internal
 */
const _isACSCallAgent = (callAgent) => {
    return !callAgent.kind || callAgent.kind === 'CallAgent';
};
/**
 * @internal
 */
const _isTeamsCall = (call) => {
    return call.kind === 'TeamsCall';
};
/**
 * @internal
 */
const _isTeamsCallAgent = (callAgent) => {
    return callAgent.kind === 'TeamsCallAgent';
};
/**
 * @internal
 */
const _isTeamsIncomingCall = (call) => {
    return call.kind === 'TeamsIncomingCall';
};
/**
 * @internal
 */
const _isTeamsMeeting = (call) => {
    return 'info' in call && !!call.info.threadId;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
function convertSdkLocalStreamToDeclarativeLocalStream(stream) {
    const localVideoStreamEffectsAPI = stream.feature(communicationCalling.Features.VideoEffects);
    return {
        source: stream.source,
        mediaStreamType: stream.mediaStreamType,
        view: undefined,
        videoEffects: convertFromSDKToDeclarativeVideoStreamVideoEffects(localVideoStreamEffectsAPI.activeEffects)
    };
}
/**
 * @private
 */
function convertSdkRemoteStreamToDeclarativeRemoteStream(stream) {
    return {
        id: stream.id,
        mediaStreamType: stream.mediaStreamType,
        isAvailable: stream.isAvailable,
        isReceiving: stream.isReceiving,
        view: undefined,
        streamSize: stream.size
    };
}
/**
 * @private
 */
function convertSdkCallFeatureStreamToDeclarativeCallFeatureStream(stream, featureName) {
    return {
        feature: featureName,
        id: stream.id,
        mediaStreamType: stream.mediaStreamType,
        isAvailable: stream.isAvailable,
        isReceiving: stream.isReceiving,
        view: undefined,
        streamSize: stream.size
    };
}
/**
 * @private
 */
function convertSdkParticipantToDeclarativeParticipant(participant) {
    const declarativeVideoStreams = {};
    for (const videoStream of participant.videoStreams) {
        declarativeVideoStreams[videoStream.id] = convertSdkRemoteStreamToDeclarativeRemoteStream(videoStream);
    }
    return {
        identifier: participant.identifier,
        displayName: participant.displayName,
        state: participant.state,
        callEndReason: participant.callEndReason,
        videoStreams: declarativeVideoStreams,
        isMuted: participant.isMuted,
        isSpeaking: participant.isSpeaking,
        raisedHand: undefined,
        role: participant.role,
        spotlight: undefined,
        mediaAccess: undefined
    };
}
/**
 * @private
 *
 * Note at the time of writing only one LocalVideoStream is supported by the SDK.
 */
function convertSdkCallToDeclarativeCall(call) {
    const declarativeRemoteParticipants = {};
    call.remoteParticipants.forEach((participant) => {
        declarativeRemoteParticipants[toFlatCommunicationIdentifier(participant.identifier)] = convertSdkParticipantToDeclarativeParticipant(participant);
    });
    let hideAttendeeNames = false;
    if (call.feature(communicationCalling.Features.Capabilities).capabilities && call.feature(communicationCalling.Features.Capabilities).capabilities.viewAttendeeNames) {
        const viewAttendeeNames = call.feature(communicationCalling.Features.Capabilities).capabilities.viewAttendeeNames;
        if (!viewAttendeeNames.isPresent && viewAttendeeNames.reason === 'MeetingRestricted') {
            hideAttendeeNames = true;
        }
    }
    let callInfo;
    if ('info' in call) {
        callInfo = call.info;
    }
    return {
        id: call.id,
        kind: _isACSCall(call) ? 'Call' : 'TeamsCall',
        callerInfo: call.callerInfo,
        state: call.state,
        callEndReason: call.callEndReason,
        diagnostics: {
            network: {
                latest: {}
            },
            media: {
                latest: {}
            }
        },
        direction: call.direction,
        isMuted: call.isMuted,
        isScreenSharingOn: call.isScreenSharingOn,
        localVideoStreams: call.localVideoStreams.map(convertSdkLocalStreamToDeclarativeLocalStream),
        remoteParticipants: declarativeRemoteParticipants,
        remoteParticipantsEnded: {},
        recording: {
            isRecordingActive: false
        },
        pptLive: {
            isActive: false
        },
        raiseHand: {
            raisedHands: []
        },
        togetherMode: {
            isActive: false,
            streams: {},
            seatingPositions: {}
        },
        localParticipantReaction: undefined,
        transcription: {
            isTranscriptionActive: false
        },
        screenShareRemoteParticipant: undefined,
        startTime: new Date(),
        endTime: undefined,
        role: call.role,
        captionsFeature: {
            captions: [],
            supportedSpokenLanguages: [],
            supportedCaptionLanguages: [],
            currentCaptionLanguage: '',
            currentSpokenLanguage: '',
            isCaptionsFeatureActive: false,
            startCaptionsInProgress: false,
            captionsKind: _isTeamsCall(call) ? 'TeamsCaptions' : 'Captions'
        },
        realTimeTextFeature: {
            realTimeTexts: {},
            isRealTimeTextFeatureActive: false
        },
        transfer: {
            acceptedTransfers: {}
        },
        optimalVideoCount: {
            maxRemoteVideoStreams: call.feature(communicationCalling.Features.OptimalVideoCount).optimalVideoCount
        },
        hideAttendeeNames,
        info: callInfo,
        meetingConference: {
            conferencePhones: []
        }
    };
}
/**
 * @private
 */
function convertSdkIncomingCallToDeclarativeIncomingCall(call) {
    if (_isTeamsIncomingCall(call)) {
        const newInfo = Object.assign(Object.assign({}, call.info), { kind: call.kind });
        return {
            id: call.id,
            info: newInfo,
            callerInfo: call.callerInfo,
            startTime: new Date(),
            endTime: undefined
        };
    }
    else {
        const newInfo = Object.assign(Object.assign({}, call.info), { kind: call.kind });
        return {
            id: call.id,
            info: newInfo,
            callerInfo: call.callerInfo,
            startTime: new Date(),
            endTime: undefined
        };
    }
}
/**
 * @private
 */
function convertFromSDKToDeclarativeVideoStreamRendererView(view) {
    return {
        scalingMode: view.scalingMode,
        isMirrored: view.isMirrored,
        target: view.target
    };
}
/**
 * @private
 */
function convertFromTeamsSDKToCaptionInfoState(caption) {
    return Object.assign({}, caption);
}
/**
 * @private
 */
function convertFromSDKToCaptionInfoState(caption) {
    return Object.assign({ captionText: caption.spokenText }, caption);
}
/**
 * @private
 */
function convertFromSDKRealTimeTextToRealTimeTextInfoState(realTimeText) {
    return Object.assign({ message: realTimeText.text, isMe: realTimeText.isLocal }, realTimeText);
}
/** @private */
function convertFromSDKToDeclarativeVideoStreamVideoEffects(videoEffects) {
    return {
        activeEffects: videoEffects
    };
}
/**
 * @private
 */
function convertFromSDKToRaisedHandState(raisedHand) {
    return {
        raisedHandOrderPosition: raisedHand.order
    };
}
/** @private */
function convertConferencePhoneInfo(meetingConferencePhoneInfo) {
    if (!meetingConferencePhoneInfo) {
        return [];
    }
    return meetingConferencePhoneInfo.phoneNumbers.flatMap(phoneNumber => {
        var _a, _b, _c, _d;
        const common = {
            conferenceId: meetingConferencePhoneInfo.phoneConferenceId,
            country: phoneNumber.countryName,
            city: phoneNumber.cityName,
            phoneNumber: '',
            isTollFree: false
        };
        const toll = Object.assign({}, common);
        toll.phoneNumber = (_b = (_a = phoneNumber.tollPhoneNumber) === null || _a === void 0 ? void 0 : _a.phoneNumber) !== null && _b !== void 0 ? _b : '';
        toll.isTollFree = false;
        const tollFree = Object.assign({}, common);
        tollFree.phoneNumber = (_d = (_c = phoneNumber.tollFreePhoneNumber) === null || _c === void 0 ? void 0 : _c.phoneNumber) !== null && _d !== void 0 ? _d : '';
        tollFree.isTollFree = true;
        return [toll, tollFree].filter(phoneInfo => phoneInfo.phoneNumber);
    });
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Error thrown from failed stateful API methods.
 *
 * @public
 */
class CallError extends Error {
    /** needs to be a (innerError as CommunicationServicesError) */
    constructor(target, innerError, timestamp) {
        super();
        this.target = target;
        this.innerError = innerError;
        if ('code' in innerError) {
            this.code = innerError.code;
        }
        if ('subCode' in innerError) {
            this.subCode = innerError.subCode;
        }
        // Testing note: It is easier to mock Date::now() than the Date() constructor.
        this.timestamp = timestamp !== null && timestamp !== void 0 ? timestamp : new Date(Date.now());
        this.name = 'CallError';
        this.message = `${this.target}: ${this.innerError.message} code=${this.code} subCode=${this.subCode}`;
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const callingStatefulLogger = logger.createClientLogger('communication-react:calling-stateful');
/**
 * @private
 */
var EventNames;
(function (EventNames) {
    // Info
    EventNames["VIEW_RENDER_SUCCEED"] = "VIEW_RENDER_SUCCEED";
    EventNames["START_DISPOSE_STREAM"] = "START_DISPOSE_STREAM";
    EventNames["START_STREAM_RENDERING"] = "START_STREAM_RENDERING";
    EventNames["DISPOSING_RENDERER"] = "DISPOSING_RENDERER";
    EventNames["CREATING_VIEW"] = "CREATING_VIEW";
    // Warning
    EventNames["CREATE_STREAM_INVALID_PARAMS"] = "CREATE_STREAM_INVALID_PARAMS";
    EventNames["DISPOSE_STREAM_INVALID_PARAMS"] = "DISPOSE_STREAM_INVALID_PARAMS";
    EventNames["STREAM_ALREADY_RENDERED"] = "STREAM_ALREADY_RENDERED";
    EventNames["STREAM_ALREADY_DISPOSED"] = "STREAM_ALREADY_DISPOSED";
    EventNames["STREAM_STOPPING"] = "STREAM_STOPPING";
    EventNames["CREATED_STREAM_STOPPING"] = "CREATED_STREAM_STOPPING";
    EventNames["STREAM_RENDERING"] = "STREAM_RENDERING";
    // Error
    EventNames["STREAM_NOT_FOUND"] = "STREAM_NOT_FOUND";
    EventNames["RENDER_INFO_NOT_FOUND"] = "RENDER_INFO_NOT_FOUND";
    EventNames["DISPOSE_INFO_NOT_FOUND"] = "DISPOSE_INFO_NOT_FOUND";
    EventNames["RENDERER_NOT_FOUND"] = "RENDERER_NOT_FOUND";
    EventNames["CREATE_STREAM_FAIL"] = "CREATE_STREAM_FAIL";
})(EventNames || (EventNames = {}));

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 * Call Id will change during the call for at least 1 time
 * This is to avoid async bug that call id has been changed during an async-await function
 * but the function still uses stale call id to access state
 */
class CallIdHistory {
    constructor() {
        this._callIdHistory = new Map();
    }
    updateCallIdHistory(newCallId, oldCallId) {
        // callId for a call can fluctuate between some set of values.
        // But if a newCallId already exists, and maps to different call, we're in trouble.
        // This can only happen if a callId is reused across two distinct calls.
        const existing = this._callIdHistory.get(newCallId);
        if (existing !== undefined && this.latestCallId(newCallId) !== oldCallId) {
            console.trace(`${newCallId} alredy exists and maps to ${existing}, which is not the same as ${oldCallId}`);
        }
        // The latest callId never maps to another callId.
        this._callIdHistory.delete(newCallId);
        this._callIdHistory.set(oldCallId, newCallId);
    }
    latestCallId(callId) {
        let latest = callId;
        /* eslint no-constant-condition: ["error", { "checkLoops": false }] */
        while (true) {
            const newer = this._callIdHistory.get(latest);
            if (newer === undefined) {
                break;
            }
            latest = newer;
        }
        return latest;
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$W = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
immer.enableMapSet();
// Needed to generate state diff for verbose logging.
immer.enablePatches();
// TODO: How can we make this configurable?
/**
 * @private
 */
const MAX_CALL_HISTORY_LENGTH = 10;
/**
 * @private
 */
const REACTION_ANIMATION_TIME_MS = 4133;
/**
 * @private
 */
let CallContext$2 = class CallContext {
    constructor(userId, maxListeners = 50) {
        this._callIdHistory = new CallIdHistory();
        this._timeOutId = {};
        this._latestCallIdOfNotification = {};
        /**
         * Tees direct errors to state.
         * @remarks
         * This is typically used for errors that are caught and intended to be shown to the user.
         *
         * @param error The raw error to report.
         * @param target The error target to tee error to.
         *
         * @private
         */
        this.teeErrorToState = (error, target) => {
            const callError = toCallError(target, error);
            this.setLatestError(target, callError);
        };
        this._logger = logger.createClientLogger('communication-react:calling-context');
        this._state = {
            calls: {},
            callsEnded: {},
            incomingCalls: {},
            incomingCallsEnded: {},
            deviceManager: {
                isSpeakerSelectionAvailable: false,
                cameras: [],
                microphones: [],
                speakers: [],
                unparentedViews: []
            },
            callAgent: undefined,
            userId: userId,
            environmentInfo: undefined,
            latestErrors: {},
            latestNotifications: {}
        };
        this._emitter = new events.EventEmitter();
        this._emitter.setMaxListeners(maxListeners);
        this._atomicId = 0;
    }
    getState() {
        return this._state;
    }
    modifyState(modifier) {
        const priorState = this._state;
        this._state = immer.produce(this._state, modifier, (patches) => {
            if (logger.getLogLevel() === 'verbose') {
                // Log to `info` because AzureLogger.verbose() doesn't show up in console.
                this._logger.info(`State change: ${_safeJSONStringify(patches)}`);
            }
        });
        if (this._state !== priorState) {
            this._emitter.emit('stateChanged', this._state);
        }
    }
    onStateChange(handler) {
        this._emitter.on('stateChanged', handler);
    }
    offStateChange(handler) {
        this._emitter.off('stateChanged', handler);
    }
    // Disposing of the CallAgentDeclarative will not clear the state. If we create a new CallAgentDeclarative, we should
    // make sure the state is clean because any left over state (if previous CallAgentDeclarative was disposed) may be
    // invalid.
    clearCallRelatedState() {
        this.modifyState((draft) => {
            draft.calls = {};
            draft.incomingCalls = {};
            draft.callsEnded = {};
            draft.incomingCallsEnded = {};
        });
    }
    setCallAgent(callAgent) {
        this.modifyState((draft) => {
            draft.callAgent = callAgent;
        });
    }
    setCall(call) {
        this.modifyState((draft) => {
            const latestCallId = this._callIdHistory.latestCallId(call.id);
            const existingCall = draft.calls[latestCallId];
            if (existingCall) {
                existingCall.callerInfo = call.callerInfo;
                existingCall.state = call.state;
                existingCall.callEndReason = call.callEndReason;
                existingCall.direction = call.direction;
                existingCall.isMuted = call.isMuted;
                existingCall.isScreenSharingOn = call.isScreenSharingOn;
                existingCall.localVideoStreams = call.localVideoStreams;
                existingCall.remoteParticipants = call.remoteParticipants;
                existingCall.transcription.isTranscriptionActive = call.transcription.isTranscriptionActive;
                existingCall.optimalVideoCount.maxRemoteVideoStreams = call.optimalVideoCount.maxRemoteVideoStreams;
                existingCall.recording.isRecordingActive = call.recording.isRecordingActive;
                existingCall.raiseHand.raisedHands = call.raiseHand.raisedHands;
                existingCall.pptLive.isActive = call.pptLive.isActive;
                existingCall.raiseHand.localParticipantRaisedHand = call.raiseHand.localParticipantRaisedHand;
                existingCall.role = call.role;
                // We don't update the startTime and endTime if we are updating an existing active call
                existingCall.captionsFeature.currentSpokenLanguage = call.captionsFeature.currentSpokenLanguage;
                existingCall.captionsFeature.currentCaptionLanguage = call.captionsFeature.currentCaptionLanguage;
                existingCall.info = call.info;
                existingCall.meetingConference = call.meetingConference;
            }
            else {
                draft.calls[latestCallId] = call;
            }
        });
        /**
         * Remove the incoming call that matches the call if there is one
         */
        if (this._state.incomingCalls[call.id]) {
            this.removeIncomingCall(call.id);
        }
    }
    removeCall(callId) {
        this.modifyState((draft) => {
            delete draft.calls[this._callIdHistory.latestCallId(callId)];
        });
    }
    setCallEnded(callId, callEndReason) {
        const latestCallId = this._callIdHistory.latestCallId(callId);
        this.modifyState((draft) => {
            const call = draft.calls[latestCallId];
            if (call) {
                call.endTime = new Date();
                call.callEndReason = callEndReason;
                delete draft.calls[latestCallId];
                // Performance note: This loop should run only once because the number of entries
                // is never allowed to exceed MAX_CALL_HISTORY_LENGTH. A loop is used for correctness.
                while (Object.keys(draft.callsEnded).length >= MAX_CALL_HISTORY_LENGTH) {
                    const oldestCall = findOldestCallEnded(draft.callsEnded);
                    if (oldestCall) {
                        delete draft.callsEnded[oldestCall];
                    }
                }
                draft.callsEnded[latestCallId] = call;
            }
        });
    }
    setCallState(callId, state) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.state = state;
            }
        });
    }
    setCallId(newCallId, oldCallId) {
        this._callIdHistory.updateCallIdHistory(newCallId, oldCallId);
        this.modifyState((draft) => {
            const call = draft.calls[oldCallId];
            if (call) {
                call.id = newCallId;
                delete draft.calls[oldCallId];
                draft.calls[newCallId] = call;
            }
            // Update call ids in latestCallIdsThatPushedNotifications
            Object.keys(this._latestCallIdOfNotification).forEach((key) => {
                if (this._latestCallIdOfNotification[key] === oldCallId) {
                    this._latestCallIdOfNotification[key] = newCallId;
                }
            });
            // Update the open breakout room call id if it matches the old call id
            if (this._openBreakoutRoom === oldCallId) {
                this._openBreakoutRoom = newCallId;
            }
        });
    }
    setEnvironmentInfo(envInfo) {
        this.modifyState((draft) => {
            draft.environmentInfo = envInfo;
        });
    }
    setCallIsScreenSharingOn(callId, isScreenSharingOn) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.isScreenSharingOn = isScreenSharingOn;
            }
        });
    }
    setCallRemoteParticipants(callId, addRemoteParticipant, removeRemoteParticipant) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                removeRemoteParticipant.forEach((id) => {
                    delete call.remoteParticipants[id];
                });
                addRemoteParticipant.forEach((participant) => {
                    call.remoteParticipants[toFlatCommunicationIdentifier(participant.identifier)] = participant;
                });
            }
        });
    }
    setCallRemoteParticipantsEnded(callId, addRemoteParticipant, removeRemoteParticipant) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                removeRemoteParticipant.forEach((id) => {
                    delete call.remoteParticipantsEnded[id];
                });
                addRemoteParticipant.forEach((participant) => {
                    call.remoteParticipantsEnded[toFlatCommunicationIdentifier(participant.identifier)] = participant;
                });
            }
        });
    }
    setCallLocalVideoStream(callId, streamsAdded, streamsRemoved) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                for (const removedStream of streamsRemoved) {
                    const index = call.localVideoStreams.findIndex(i => i.mediaStreamType === removedStream.mediaStreamType);
                    if (index > -1) {
                        call.localVideoStreams.splice(index, 1);
                    }
                }
                for (const addedStream of streamsAdded) {
                    const index = call.localVideoStreams.findIndex(i => i.mediaStreamType === addedStream.mediaStreamType);
                    if (index > -1) {
                        call.localVideoStreams[index] = addedStream;
                    }
                    else {
                        call.localVideoStreams.push(addedStream);
                    }
                }
            }
        });
    }
    setCallLocalVideoStreamVideoEffects(callId, videoEffects) {
        this.modifyState((draft) => {
            var _a;
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const stream = (_a = call.localVideoStreams) === null || _a === void 0 ? void 0 : _a.find(i => i.mediaStreamType === 'Video');
                if (stream) {
                    stream.videoEffects = videoEffects;
                }
            }
        });
    }
    setCallIsMicrophoneMuted(callId, isMicrophoneMuted) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.isMuted = isMicrophoneMuted;
            }
        });
    }
    setRole(callId, role) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.role = role;
            }
        });
    }
    setCallDominantSpeakers(callId, dominantSpeakers) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.dominantSpeakers = dominantSpeakers;
            }
        });
    }
    setCallRecordingActive(callId, isRecordingActive) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.recording.isRecordingActive = isRecordingActive;
            }
        });
    }
    setCallPPTLiveActive(callId, isActive) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.pptLive.isActive = isActive;
            }
        });
    }
    setCallParticipantPPTLive(callId, participantKey, target) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call && participantKey) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    participant.contentSharingStream = target;
                    call.contentSharingRemoteParticipant = participantKey;
                }
            }
        });
    }
    setTogetherModeVideoStreams(callId, addedStreams, removedStreams) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                for (const stream of removedStreams) {
                    if (stream.mediaStreamType === 'Video') {
                        call.togetherMode.streams.mainVideoStream = undefined;
                        call.togetherMode.isActive = false;
                        call.togetherMode.seatingPositions = {};
                    }
                }
                for (const newStream of addedStreams) {
                    // This should only be called by the subscriber and some properties are add by other components so if the
                    // stream already exists, only update the values that subscriber knows about.
                    const mainVideoStream = call.togetherMode.streams.mainVideoStream;
                    if (mainVideoStream && mainVideoStream.id === newStream.id) {
                        mainVideoStream.mediaStreamType = newStream.mediaStreamType;
                        mainVideoStream.isAvailable = newStream.isAvailable;
                        mainVideoStream.isReceiving = newStream.isReceiving;
                    }
                    else {
                        call.togetherMode.streams.mainVideoStream = newStream;
                    }
                    call.togetherMode.isActive = true;
                }
            }
        });
    }
    setTogetherModeVideoStreamIsAvailable(callId, streamId, isAvailable) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const stream = call.togetherMode.streams.mainVideoStream;
                if (stream && (stream === null || stream === void 0 ? void 0 : stream.id) === streamId) {
                    stream.isAvailable = isAvailable;
                }
            }
        });
    }
    setTogetherModeVideoStreamIsReceiving(callId, streamId, isReceiving) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const stream = call.togetherMode.streams.mainVideoStream;
                if (stream && (stream === null || stream === void 0 ? void 0 : stream.id) === streamId) {
                    stream.isReceiving = isReceiving;
                }
            }
        });
    }
    setTogetherModeVideoStreamSize(callId, streamId, size) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const stream = call.togetherMode.streams.mainVideoStream;
                if (stream && (stream === null || stream === void 0 ? void 0 : stream.id) === streamId) {
                    stream.streamSize = size;
                }
            }
        });
    }
    removeTogetherModeVideoStream(callId, removedStream) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                for (const stream of removedStream) {
                    if (stream.mediaStreamType === 'Video') {
                        call.togetherMode.streams.mainVideoStream = undefined;
                        call.togetherMode.isActive = false;
                    }
                }
            }
        });
    }
    setTogetherModeSeatingCoordinates(callId, seatingMap) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const seatingPositions = {};
                for (const [userId, seatingPosition] of seatingMap.entries()) {
                    seatingPositions[userId] = seatingPosition;
                }
                call.togetherMode.seatingPositions = seatingPositions;
            }
        });
    }
    setCallRaisedHands(callId, raisedHands) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.raiseHand.raisedHands = raisedHands.map(raisedHand => {
                    return convertFromSDKToRaisedHandState(raisedHand);
                });
                const raisedHand = raisedHands.find(raisedHand => toFlatCommunicationIdentifier(raisedHand.identifier) === toFlatCommunicationIdentifier(this._state.userId));
                if (raisedHand) {
                    call.raiseHand.localParticipantRaisedHand = convertFromSDKToRaisedHandState(raisedHand);
                }
                else {
                    call.raiseHand.localParticipantRaisedHand = undefined;
                }
            }
        });
    }
    setParticipantIsRaisedHand(callId, participantKey, raisedHand) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    participant.raisedHand = raisedHand ? convertFromSDKToRaisedHandState(raisedHand) : raisedHand;
                }
            }
        });
    }
    setReceivedReactionFromParticipant(callId, participantKey, reactionMessage) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (!call) {
                return;
            }
            clearTimeout(this._timeOutId[participantKey]);
            const participant = call.remoteParticipants[participantKey];
            const newReactionState = reactionMessage ? {
                reactionMessage: reactionMessage,
                receivedOn: new Date()
            } : undefined;
            if (participantKey === toFlatCommunicationIdentifier(this._state.userId)) {
                call.localParticipantReaction = newReactionState;
            }
            else if (!participant) {
                // Warn if we can't find the participant in the call but we are trying to set their reaction state to a new reaction.
                if (reactionMessage !== null) {
                    console.warn(`Participant ${participantKey} not found in call ${callId}. Cannot set reaction state.`);
                }
            }
            else {
                participant.reactionState = newReactionState;
            }
            if (reactionMessage) {
                this._timeOutId[participantKey] = setTimeout(() => {
                    clearParticipantReactionState(this, callId, participantKey);
                }, REACTION_ANIMATION_TIME_MS);
            }
        });
    }
    setCallTranscriptionActive(callId, isTranscriptionActive) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.transcription.isTranscriptionActive = isTranscriptionActive;
            }
        });
    }
    setCapabilities(callId, capabilities, capabilitiesChangeInfo) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.capabilitiesFeature = {
                    capabilities,
                    latestCapabilitiesChangeInfo: capabilitiesChangeInfo
                };
            }
        });
    }
    setHideAttendeeNames(callId, capabilitiesChangeInfo) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (capabilitiesChangeInfo.oldValue.viewAttendeeNames !== capabilitiesChangeInfo.newValue.viewAttendeeNames) {
                const viewAttendeeNames = capabilitiesChangeInfo.newValue.viewAttendeeNames;
                if (call && viewAttendeeNames && !viewAttendeeNames.isPresent && viewAttendeeNames.reason === 'MeetingRestricted') {
                    call.hideAttendeeNames = true;
                }
                else if (call) {
                    call.hideAttendeeNames = false;
                }
            }
        });
    }
    setSpotlight(callId, spotlightedParticipants, maxParticipantsToSpotlight) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.spotlight = Object.assign(Object.assign({}, call.spotlight), { spotlightedParticipants,
                    maxParticipantsToSpotlight });
            }
        });
    }
    setTeamsMeetingConference(callId, teamsMeetingConferenceDetails) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.meetingConference = {
                    conferencePhones: convertConferencePhoneInfo(teamsMeetingConferenceDetails)
                };
            }
        });
    }
    setParticipantSpotlighted(callId, spotlightedParticipant) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[toFlatCommunicationIdentifier(spotlightedParticipant.identifier)];
                if (participant) {
                    participant.spotlight = {
                        spotlightedOrderPosition: spotlightedParticipant.order
                    };
                }
                else if (call.spotlight && toFlatCommunicationIdentifier(draft.userId) === toFlatCommunicationIdentifier(spotlightedParticipant.identifier)) {
                    call.spotlight.localParticipantSpotlight = {
                        spotlightedOrderPosition: spotlightedParticipant.order
                    };
                }
            }
        });
    }
    setParticipantNotSpotlighted(callId, spotlightedParticipant) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[toFlatCommunicationIdentifier(spotlightedParticipant.identifier)];
                if (participant) {
                    participant.spotlight = undefined;
                }
                else if (call.spotlight && toFlatCommunicationIdentifier(draft.userId) === toFlatCommunicationIdentifier(spotlightedParticipant.identifier)) {
                    call.spotlight.localParticipantSpotlight = undefined;
                }
            }
        });
    }
    setAssignedBreakoutRoom(callId, breakoutRoom) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.breakoutRooms = Object.assign(Object.assign({}, call.breakoutRooms), { assignedBreakoutRoom: breakoutRoom });
            }
        });
    }
    setBreakoutRoomSettings(callId, breakoutRoomSettings) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.breakoutRooms = Object.assign(Object.assign({}, call.breakoutRooms), { breakoutRoomSettings: breakoutRoomSettings });
            }
        });
    }
    setBreakoutRoomDisplayName(callId, breakoutRoomDisplayName) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.breakoutRooms = Object.assign(Object.assign({}, call.breakoutRooms), { breakoutRoomDisplayName });
            }
        });
    }
    setOpenBreakoutRoom(callId) {
        this._openBreakoutRoom = callId;
    }
    deleteOpenBreakoutRoom() {
        this._openBreakoutRoom = undefined;
    }
    getOpenBreakoutRoom() {
        return this._openBreakoutRoom;
    }
    setCallScreenShareParticipant(callId, participantKey) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.screenShareRemoteParticipant = participantKey;
            }
        });
    }
    setLocalVideoStreamRendererView(callId, localVideoMediaStreamType, view) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const localVideoStream = call.localVideoStreams.find(localVideoStream => localVideoStream.mediaStreamType === localVideoMediaStreamType);
                if (localVideoStream) {
                    localVideoStream.view = view;
                }
            }
        });
    }
    setTogetherModeVideoStreamRendererView(callId, togetherModeStreamType, view) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                if (togetherModeStreamType === 'Video') {
                    const togetherModeStream = call.togetherMode.streams.mainVideoStream;
                    if (togetherModeStream) {
                        togetherModeStream.view = view;
                    }
                }
            }
        });
    }
    setParticipantState(callId, participantKey, state) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    participant.state = state;
                }
            }
        });
    }
    setParticipantIsMuted(callId, participantKey, muted) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    participant.isMuted = muted;
                }
            }
        });
    }
    setOptimalVideoCount(callId, optimalVideoCount) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.optimalVideoCount.maxRemoteVideoStreams = optimalVideoCount;
            }
        });
    }
    setParticipantRole(callId, participantKey, role) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    participant.role = role;
                }
            }
        });
    }
    setParticipantDisplayName(callId, participantKey, displayName) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    participant.displayName = displayName;
                }
            }
        });
    }
    setParticipantIsSpeaking(callId, participantKey, isSpeaking) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    participant.isSpeaking = isSpeaking;
                }
            }
        });
    }
    setParticipantVideoStream(callId, participantKey, stream) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    // Set is called by subscriber will not modify any rendered stream so if there is existing stream only
                    // modify the values that subscriber has access to.
                    const existingStream = participant.videoStreams[stream.id];
                    if (existingStream) {
                        existingStream.isAvailable = stream.isAvailable;
                        existingStream.isReceiving = stream.isReceiving;
                        existingStream.mediaStreamType = stream.mediaStreamType;
                    }
                    else {
                        participant.videoStreams[stream.id] = stream;
                    }
                }
            }
        });
    }
    setRemoteVideoStreamIsAvailable(callId, participantKey, streamId, isAvailable) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    const stream = participant.videoStreams[streamId];
                    if (stream) {
                        stream.isAvailable = isAvailable;
                    }
                }
            }
        });
    }
    setRemoteVideoStreamIsReceiving(callId, participantKey, streamId, isReceiving) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    const stream = participant.videoStreams[streamId];
                    if (stream) {
                        stream.isReceiving = isReceiving;
                    }
                }
            }
        });
    }
    setRemoteVideoStreamSize(callId, participantKey, streamId, size) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    const stream = participant.videoStreams[streamId];
                    if (stream) {
                        stream.streamSize = size;
                    }
                }
            }
        });
    }
    setRemoteVideoStreams(callId, participantKey, addRemoteVideoStream, removeRemoteVideoStream) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    for (const id of removeRemoteVideoStream) {
                        delete participant.videoStreams[id];
                    }
                    for (const newStream of addRemoteVideoStream) {
                        // This should only be called by the subscriber and some properties are add by other components so if the
                        // stream already exists, only update the values that subscriber knows about.
                        const stream = participant.videoStreams[newStream.id];
                        if (stream) {
                            stream.mediaStreamType = newStream.mediaStreamType;
                            stream.isAvailable = newStream.isAvailable;
                            stream.isReceiving = newStream.isReceiving;
                        }
                        else {
                            participant.videoStreams[newStream.id] = newStream;
                        }
                    }
                }
            }
        });
    }
    setRemoteVideoStreamRendererView(callId, participantKey, streamId, view) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    const stream = participant.videoStreams[streamId];
                    if (stream) {
                        stream.view = view;
                    }
                }
            }
        });
    }
    setRemoteVideoStreamViewScalingMode(callId, participantKey, streamId, scalingMode) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const participant = call.remoteParticipants[participantKey];
                if (participant) {
                    const stream = participant.videoStreams[streamId];
                    if (stream && stream.view) {
                        stream.view.scalingMode = scalingMode;
                    }
                }
            }
        });
    }
    setIncomingCall(call) {
        this.modifyState((draft) => {
            const existingCall = draft.incomingCalls[call.id];
            if (existingCall) {
                existingCall.callerInfo = call.callerInfo;
            }
            else {
                draft.incomingCalls[call.id] = call;
            }
        });
    }
    removeIncomingCall(callId) {
        this.modifyState((draft) => {
            delete draft.incomingCalls[callId];
        });
    }
    setIncomingCallEnded(callId, callEndReason) {
        this.modifyState((draft) => {
            const call = draft.incomingCalls[callId];
            if (call) {
                call.endTime = new Date();
                call.callEndReason = callEndReason;
                delete draft.incomingCalls[callId];
                // Performance note: This loop should run only once because the number of entries
                // is never allowed to exceed MAX_CALL_HISTORY_LENGTH. A loop is used for correctness.
                while (Object.keys(draft.incomingCallsEnded).length >= MAX_CALL_HISTORY_LENGTH) {
                    const oldestCall = findOldestCallEnded(draft.incomingCallsEnded);
                    if (oldestCall) {
                        delete draft.incomingCallsEnded[oldestCall];
                    }
                }
                draft.incomingCallsEnded[callId] = call;
            }
        });
    }
    setDeviceManagerIsSpeakerSelectionAvailable(isSpeakerSelectionAvailable) {
        this.modifyState((draft) => {
            draft.deviceManager.isSpeakerSelectionAvailable = isSpeakerSelectionAvailable;
        });
    }
    setDeviceManagerSelectedMicrophone(selectedMicrophone) {
        this.modifyState((draft) => {
            draft.deviceManager.selectedMicrophone = selectedMicrophone;
        });
    }
    setDeviceManagerSelectedSpeaker(selectedSpeaker) {
        this.modifyState((draft) => {
            draft.deviceManager.selectedSpeaker = selectedSpeaker;
        });
    }
    setDeviceManagerSelectedCamera(selectedCamera) {
        this.modifyState((draft) => {
            draft.deviceManager.selectedCamera = selectedCamera;
        });
    }
    setDeviceManagerCameras(cameras) {
        this.modifyState((draft) => {
            /**
             * SDK initializes cameras with one dummy camera with value { id: 'camera:id', name: '', deviceType: 'USBCamera' } immediately after
             * camera permissions are granted. So selectedCamera will have this value before the actual cameras are obtained. Therefore we should reset
             * selectedCamera to the first camera when there are cameras AND when current selectedCamera does not exist in the new array of cameras *
             */
            if (cameras.length > 0 && !cameras.some(camera => { var _a; return camera.id === ((_a = draft.deviceManager.selectedCamera) === null || _a === void 0 ? void 0 : _a.id); })) {
                draft.deviceManager.selectedCamera = cameras[0];
            }
            draft.deviceManager.cameras = cameras;
        });
    }
    setDeviceManagerMicrophones(microphones) {
        this.modifyState((draft) => {
            draft.deviceManager.microphones = microphones;
        });
    }
    setDeviceManagerSpeakers(speakers) {
        this.modifyState((draft) => {
            draft.deviceManager.speakers = speakers;
        });
    }
    setDeviceManagerDeviceAccess(deviceAccess) {
        this.modifyState((draft) => {
            draft.deviceManager.deviceAccess = deviceAccess;
        });
    }
    setDeviceManagerUnparentedView(localVideoStream, view) {
        this.modifyState((draft) => {
            draft.deviceManager.unparentedViews.push({
                source: localVideoStream.source,
                mediaStreamType: localVideoStream.mediaStreamType,
                view: view
            });
        });
    }
    deleteDeviceManagerUnparentedView(localVideoStream) {
        this.modifyState((draft) => {
            const foundIndex = draft.deviceManager.unparentedViews.findIndex(stream => stream.mediaStreamType === localVideoStream.mediaStreamType);
            if (foundIndex !== -1) {
                draft.deviceManager.unparentedViews.splice(foundIndex, 1);
            }
        });
    }
    setDeviceManagerUnparentedViewVideoEffects(localVideoStream, videoEffects) {
        this.modifyState((draft) => {
            const view = draft.deviceManager.unparentedViews.find(stream => stream.mediaStreamType === localVideoStream.mediaStreamType);
            if (view) {
                view.videoEffects = videoEffects;
            }
        });
    }
    getAndIncrementAtomicId() {
        const id = this._atomicId;
        this._atomicId++;
        return id;
    }
    processNewRealTimeText(realTimeTexts, newRealTimeText) {
        var _a, _b, _c, _d, _e;
        // if the new message is final, push it to completed messages
        if (newRealTimeText.resultType === 'Final') {
            if (!realTimeTexts.completedMessages) {
                realTimeTexts.completedMessages = [];
            }
            (_a = realTimeTexts.completedMessages) === null || _a === void 0 ? void 0 : _a.push(newRealTimeText);
            // clear the corresponding in progress message
            if (newRealTimeText.isMe) {
                realTimeTexts.myInProgress = undefined;
            }
            else {
                realTimeTexts.currentInProgress = (_b = realTimeTexts.currentInProgress) === null || _b === void 0 ? void 0 : _b.filter(message => message.sequenceId !== newRealTimeText.sequenceId);
            }
        }
        else {
            // if receive partial real time text
            // if message is from me, assign it to myInProgress
            if (newRealTimeText.isMe) {
                realTimeTexts.myInProgress = newRealTimeText;
            }
            // if message is from others, assign it to currentInProgress
            else {
                if (!realTimeTexts.currentInProgress) {
                    realTimeTexts.currentInProgress = [];
                    realTimeTexts.currentInProgress.push(newRealTimeText);
                    return;
                }
                // find the index of the existing in progress message
                // replace the existing in progress message with the new one
                const existingIndex = (_c = realTimeTexts.currentInProgress) === null || _c === void 0 ? void 0 : _c.findIndex(message => message.sequenceId === newRealTimeText.sequenceId);
                if (existingIndex === -1) {
                    (_d = realTimeTexts.currentInProgress) === null || _d === void 0 ? void 0 : _d.push(newRealTimeText);
                }
                else {
                    realTimeTexts.currentInProgress[existingIndex] = newRealTimeText;
                }
            }
        }
        // edge case check for in progress messages time out
        if (!realTimeTexts.completedMessages) {
            realTimeTexts.completedMessages = [];
        }
        this.findTimeoutRealTimeText(realTimeTexts.currentInProgress, realTimeTexts.completedMessages);
        this.findTimeoutRealTimeText(realTimeTexts.myInProgress, realTimeTexts.completedMessages);
        // we only want to store up to 50 finalized messages
        if (realTimeTexts.completedMessages && ((_e = realTimeTexts.completedMessages) === null || _e === void 0 ? void 0 : _e.length) > 50) {
            realTimeTexts.completedMessages.shift();
        }
    }
    findTimeoutRealTimeText(inProgressRealTimeTexts, completedRealTimeTexts) {
        // if inProgressRealTimeTexts is an array
        if (inProgressRealTimeTexts && Array.isArray(inProgressRealTimeTexts)) {
            // find the in progress real time text that has not been updated for 5 seconds
            for (let i = inProgressRealTimeTexts.length - 1; i >= 0; i--) {
                const realTimeText = inProgressRealTimeTexts[i];
                if ((realTimeText === null || realTimeText === void 0 ? void 0 : realTimeText.updatedTimestamp) && Date.now() - (realTimeText === null || realTimeText === void 0 ? void 0 : realTimeText.updatedTimestamp.getTime()) > 5000) {
                    // turn the in progress real time text to final
                    realTimeText.resultType = 'Final';
                    // move the in progress real time text to completed
                    completedRealTimeTexts.push(realTimeText);
                    // remove the in progress real time text from in progress
                    if (inProgressRealTimeTexts) {
                        inProgressRealTimeTexts.splice(i, 1);
                    }
                }
            }
        }
        else {
            // if inProgressRealTimeTexts is a single object
            if (inProgressRealTimeTexts && inProgressRealTimeTexts.updatedTimestamp && Date.now() - inProgressRealTimeTexts.updatedTimestamp.getTime() > 5000) {
                // turn the in progress real time text to final
                inProgressRealTimeTexts.resultType = 'Final';
                // move the in progress real time text to completed
                completedRealTimeTexts.push(inProgressRealTimeTexts);
                // remove the in progress real time text from in progress
                inProgressRealTimeTexts = undefined;
            }
        }
    }
    processNewCaption(captions, newCaption) {
        var _a;
        // time stamp when new caption comes in
        newCaption.lastUpdatedTimestamp = new Date();
        // if this is the first caption, push it in
        if (captions.length === 0) {
            captions.push(newCaption);
        }
        // if the last caption is final, then push the new one in
        else if (((_a = captions[captions.length - 1]) === null || _a === void 0 ? void 0 : _a.resultType) === 'Final') {
            captions.push(newCaption);
        }
        // if the last caption is Partial, then check if the speaker is the same as the new caption, if so, update the last caption
        else {
            const lastCaption = captions[captions.length - 1];
            if (lastCaption && lastCaption.speaker.identifier && newCaption.speaker.identifier && toFlatCommunicationIdentifier(lastCaption.speaker.identifier) === toFlatCommunicationIdentifier(newCaption.speaker.identifier)) {
                captions[captions.length - 1] = newCaption;
            }
            // if different speaker, ignore the interjector until the current speaker finishes
            // edge case: if we dont receive the final caption from the current speaker for 5 secs, we turn the current speaker caption to final and push in the new interjector
            else if (lastCaption === null || lastCaption === void 0 ? void 0 : lastCaption.lastUpdatedTimestamp) {
                if (Date.now() - lastCaption.lastUpdatedTimestamp.getTime() > 5000) {
                    lastCaption.resultType = 'Final';
                    captions.push(newCaption);
                }
            }
        }
        // If the array length exceeds 50, remove the oldest caption
        if (captions.length > 50) {
            captions.shift();
        }
    }
    addTeamsCaption(callId, caption) {
        this.modifyState((draft) => {
            var _a;
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                const currentCaptionLanguage = call.captionsFeature.currentCaptionLanguage;
                if (caption.captionLanguage.toUpperCase() === currentCaptionLanguage.toUpperCase() || currentCaptionLanguage === '' || currentCaptionLanguage === undefined) {
                    this.processNewCaption((_a = call === null || call === void 0 ? void 0 : call.captionsFeature.captions) !== null && _a !== void 0 ? _a : [], convertFromTeamsSDKToCaptionInfoState(caption));
                }
            }
        });
    }
    addCaption(callId, caption) {
        this.modifyState((draft) => {
            var _a;
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                this.processNewCaption((_a = call === null || call === void 0 ? void 0 : call.captionsFeature.captions) !== null && _a !== void 0 ? _a : [], convertFromSDKToCaptionInfoState(caption));
            }
        });
    }
    addRealTimeText(callId, realTimeText) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                this.processNewRealTimeText(call.realTimeTextFeature.realTimeTexts, convertFromSDKRealTimeTextToRealTimeTextInfoState(realTimeText));
            }
        });
    }
    setCaptionsKind(callId, kind) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.captionsKind = kind;
            }
        });
    }
    clearCaptions(callId) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.captions = call.captionsFeature.captions.filter(c => 'message' in c);
            }
        });
    }
    setIsCaptionActive(callId, isCaptionsActive) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.isCaptionsFeatureActive = isCaptionsActive;
            }
        });
    }
    setIsRealTimeTextActive(callId, isRealTimeTextActive) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.realTimeTextFeature.isRealTimeTextFeatureActive = isRealTimeTextActive;
            }
        });
    }
    setStartCaptionsInProgress(callId, startCaptionsInProgress) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.startCaptionsInProgress = startCaptionsInProgress;
            }
        });
    }
    setSelectedSpokenLanguage(callId, spokenLanguage) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.currentSpokenLanguage = spokenLanguage;
            }
        });
    }
    setSelectedCaptionLanguage(callId, captionLanguage) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.currentCaptionLanguage = captionLanguage;
            }
        });
    }
    setAvailableCaptionLanguages(callId, captionLanguages) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.supportedCaptionLanguages = captionLanguages;
            }
        });
    }
    setAvailableSpokenLanguages(callId, spokenLanguages) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.captionsFeature.supportedSpokenLanguages = spokenLanguages;
            }
        });
    }
    setAcceptedTransfer(callId, acceptedTransfer) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (call) {
                call.transfer.acceptedTransfers[acceptedTransfer.callId] = acceptedTransfer;
            }
        });
    }
    /**
     * Tees any errors encountered in an async function to the state.
     *
     * @param action Async function to execute.
     * @param target The error target to tee error to.
     * @returns Result of calling `f`. Also re-raises any exceptions thrown from `f`.
     * @throws CallError. Exceptions thrown from `f` are tagged with the failed `target.
     */
    withAsyncErrorTeedToState(action, target) {
        return (...args) => __awaiter$W(this, void 0, void 0, function* () {
            try {
                return yield action(...args);
            }
            catch (error) {
                const callError = toCallError(target, error);
                this.setLatestError(target, callError);
                throw callError;
            }
        });
    }
    /**
     * Tees any errors encountered in an function to the state.
     *
     * @param action Function to execute.
     * @param target The error target to tee error to.
     * @returns Result of calling `f`. Also re-raises any exceptions thrown from `f`.
     * @throws CallError. Exceptions thrown from `f` are tagged with the failed `target.
     */
    withErrorTeedToState(action, target) {
        return (...args) => {
            try {
                callingStatefulLogger.info(`Calling stateful client target function called: ${target}`);
                return action(...args);
            }
            catch (error) {
                const callError = toCallError(target, error);
                this.setLatestError(target, callError);
                throw callError;
            }
        };
    }
    setLatestError(target, error) {
        this.modifyState((draft) => {
            draft.latestErrors[target] = error;
        });
    }
    setLatestNotification(callId, notification) {
        this._latestCallIdOfNotification[notification.target] = callId;
        this.modifyState((draft) => {
            draft.latestNotifications[notification.target] = notification;
        });
    }
    deleteLatestNotification(notificationTarget, callId) {
        const callIdOfNotification = this._latestCallIdOfNotification[notificationTarget];
        // Only delete the notification if the call that pushed the notification is the same as the callId if specified
        if (callId && callIdOfNotification !== callId) {
            return;
        }
        this.modifyState((draft) => {
            delete draft.latestNotifications[notificationTarget];
        });
    }
    setMediaAccesses(callId, mediaAccesses) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (!call) {
                return;
            }
            mediaAccesses.forEach(participantMediaAccess => {
                const participant = call.remoteParticipants[toFlatCommunicationIdentifier(participantMediaAccess.participant)];
                if (participant) {
                    participant.mediaAccess = {
                        isAudioPermitted: participantMediaAccess.isAudioPermitted,
                        isVideoPermitted: participantMediaAccess.isVideoPermitted
                    };
                }
            });
        });
    }
    setMeetingMediaAccess(callId, meetingMediaAccess) {
        this.modifyState((draft) => {
            const call = draft.calls[this._callIdHistory.latestCallId(callId)];
            if (!call) {
                return;
            }
            if (meetingMediaAccess) {
                call.meetingMediaAccess = {
                    isAudioPermitted: meetingMediaAccess.isAudioPermitted,
                    isVideoPermitted: meetingMediaAccess.isVideoPermitted
                };
            }
        });
    }
};
const toCallError = (target, error) => {
    if (error instanceof Error) {
        return new CallError(target, error);
    }
    return new CallError(target, new Error(error));
};
const findOldestCallEnded = (calls) => {
    var _a, _b, _c, _d;
    const callEntries = Object.entries(calls);
    const firstCallEntry = callEntries[0];
    if (!firstCallEntry) {
        return undefined; // no calls exist
    }
    let [oldestCallId, oldestCall] = firstCallEntry;
    if (oldestCall.endTime === undefined) {
        return oldestCallId;
    }
    for (const [callId, call] of callEntries.slice(1)) {
        if (call.endTime === undefined) {
            return callId;
        }
        if (((_b = (_a = call.endTime) === null || _a === void 0 ? void 0 : _a.getTime()) !== null && _b !== void 0 ? _b : 0) < ((_d = (_c = oldestCall.endTime) === null || _c === void 0 ? void 0 : _c.getTime()) !== null && _d !== void 0 ? _d : 0)) {
            [oldestCallId, oldestCall] = [callId, call];
        }
    }
    return oldestCallId;
};
function clearParticipantReactionState(callContext, callId, participantKey) {
    callContext.setReceivedReactionFromParticipant(callId, participantKey, null);
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class TeamsCaptionsSubscriber {
    constructor(callIdRef, context, captions) {
        this.subscribe = () => {
            this._captions.on('CaptionsActiveChanged', this.isCaptionsActiveChanged);
            this._captions.on('CaptionsReceived', this.onCaptionsReceived);
            this._captions.on('CaptionLanguageChanged', this.isCaptionLanguageChanged);
            this._captions.on('SpokenLanguageChanged', this.isSpokenLanguageChanged);
        };
        this.unsubscribe = () => {
            this._captions.off('CaptionsActiveChanged', this.isCaptionsActiveChanged);
            this._captions.off('CaptionsReceived', this.onCaptionsReceived);
            this._captions.off('CaptionLanguageChanged', this.isCaptionLanguageChanged);
            this._captions.off('SpokenLanguageChanged', this.isSpokenLanguageChanged);
        };
        this.onCaptionsReceived = (caption) => {
            this._context.addTeamsCaption(this._callIdRef.callId, caption);
        };
        this.isCaptionsActiveChanged = () => {
            this._context.setIsCaptionActive(this._callIdRef.callId, this._captions.isCaptionsFeatureActive);
        };
        this.isCaptionLanguageChanged = () => {
            this._context.setSelectedCaptionLanguage(this._callIdRef.callId, this._captions.activeCaptionLanguage);
        };
        this.isSpokenLanguageChanged = () => {
            this._context.setSelectedSpokenLanguage(this._callIdRef.callId, this._captions.activeSpokenLanguage);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._captions = captions;
        if (this._captions.isCaptionsFeatureActive) {
            this._context.setIsCaptionActive(this._callIdRef.callId, this._captions.isCaptionsFeatureActive);
        }
        this._context.setAvailableSpokenLanguages(this._callIdRef.callId, this._captions.supportedSpokenLanguages);
        if ('supportedCaptionLanguages' in this._captions) {
            this._context.setAvailableCaptionLanguages(this._callIdRef.callId, this._captions.supportedCaptionLanguages);
        }
        this._context.setSelectedSpokenLanguage(this._callIdRef.callId, this._captions.activeSpokenLanguage);
        this._context.setSelectedCaptionLanguage(this._callIdRef.callId, this._captions.activeCaptionLanguage);
        this.subscribe();
    }
}
/**
 * @private
 */
class CaptionsSubscriber {
    constructor(callIdRef, context, captions) {
        this.subscribe = () => {
            this._captions.on('CaptionsActiveChanged', this.isCaptionsActiveChanged);
            this._captions.on('SpokenLanguageChanged', this.isSpokenLanguageChanged);
            this._captions.on('CaptionsReceived', this.onCaptionsReceived);
        };
        this.unsubscribe = () => {
            this._captions.off('CaptionsActiveChanged', this.isCaptionsActiveChanged);
            this._captions.off('SpokenLanguageChanged', this.isSpokenLanguageChanged);
            this._captions.off('CaptionsReceived', this.onCaptionsReceived);
        };
        this.onCaptionsReceived = (caption) => {
            this._context.addCaption(this._callIdRef.callId, caption);
        };
        this.isCaptionsActiveChanged = () => {
            this._context.setIsCaptionActive(this._callIdRef.callId, this._captions.isCaptionsFeatureActive);
        };
        this.isSpokenLanguageChanged = () => {
            this._context.setSelectedSpokenLanguage(this._callIdRef.callId, this._captions.activeSpokenLanguage);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._captions = captions;
        if (this._captions.isCaptionsFeatureActive) {
            this._context.setIsCaptionActive(this._callIdRef.callId, this._captions.isCaptionsFeatureActive);
        }
        this._context.setAvailableSpokenLanguages(this._callIdRef.callId, this._captions.supportedSpokenLanguages);
        this._context.setSelectedSpokenLanguage(this._callIdRef.callId, this._captions.activeSpokenLanguage);
        this.subscribe();
    }
}
/**
 * @private
 */
class CaptionsFeatureSubscriber {
    constructor(callIdRef, context, captions) {
        this.subscribe = () => {
            this._captions.on('CaptionsKindChanged', this.isCaptionsKindChanged);
        };
        this.unsubscribe = () => {
            var _a, _b;
            this._captions.off('CaptionsKindChanged', this.isCaptionsKindChanged);
            (_a = this._TeamsCaptionsSubscriber) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            (_b = this._CaptionsSubscriber) === null || _b === void 0 ? void 0 : _b.unsubscribe();
        };
        this.isCaptionsKindChanged = () => {
            var _a;
            this._context.setCaptionsKind(this._callIdRef.callId, this._captions.captions.kind);
            // ACS call can turn into teams call but teams call will never turn into ACS call so we only need to handle the case when captions kind is TeamsCaptions
            if (this._captions.captions.kind === 'TeamsCaptions') {
                (_a = this._CaptionsSubscriber) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                this._TeamsCaptionsSubscriber = new TeamsCaptionsSubscriber(this._callIdRef, this._context, this._captions.captions);
            }
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._captions = captions;
        this._context.setCaptionsKind(this._callIdRef.callId, this._captions.captions.kind);
        if (this._captions.captions.kind === 'TeamsCaptions') {
            this._TeamsCaptionsSubscriber = new TeamsCaptionsSubscriber(this._callIdRef, this._context, this._captions.captions);
        }
        else {
            this._CaptionsSubscriber = new CaptionsSubscriber(this._callIdRef, this._context, this._captions.captions);
        }
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Subscribes to a LocalVideoStream's video effects events and updates the call context appropriately.
 * @private
 */
class LocalVideoStreamVideoEffectsSubscriber {
    constructor(args) {
        this.subscribe = () => {
            this._localVideoStreamEffectsAPI.on('effectsStarted', this.effectsStarted);
            this._localVideoStreamEffectsAPI.on('effectsStopped', this.effectsStopped);
            this._localVideoStreamEffectsAPI.on('effectsError', this.effectsError);
        };
        this.unsubscribe = () => {
            this._localVideoStreamEffectsAPI.off('effectsStarted', this.effectsStarted);
            this._localVideoStreamEffectsAPI.off('effectsStopped', this.effectsStopped);
            this._localVideoStreamEffectsAPI.off('effectsError', this.effectsError);
        };
        this.effectsStarted = () => {
            this.updateStatefulVideoEffects();
        };
        this.effectsStopped = () => {
            this.updateStatefulVideoEffects();
        };
        this.effectsError = (error) => {
            // When there is an error the effects have stopped. Ensure state is updated to reflect if effects are active or not.
            this.updateStatefulVideoEffects();
            this._context.teeErrorToState(new Error(error.message), 'VideoEffectsFeature.startEffects');
        };
        this.updateStatefulVideoEffects = () => {
            const statefulVideoEffects = convertFromSDKToDeclarativeVideoStreamVideoEffects(this._localVideoStreamEffectsAPI.activeEffects);
            if (this._parent === 'unparented') {
                this._context.setDeviceManagerUnparentedViewVideoEffects(this._localVideoStream, statefulVideoEffects);
            }
            else {
                this._context.setCallLocalVideoStreamVideoEffects(this._parent.callId, statefulVideoEffects);
            }
        };
        this._parent = args.parent;
        this._context = args.context;
        this._localVideoStream = args.localVideoStream;
        this._localVideoStreamEffectsAPI = args.localVideoStreamEffectsAPI;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class RemoteVideoStreamSubscriber {
    constructor(callIdRef, participantKey, remoteVideoStream, context) {
        this.subscribe = () => {
            this._remoteVideoStream.on('isAvailableChanged', this.isAvailableChanged);
            this._remoteVideoStream.on('isReceivingChanged', this.isReceivingChanged);
            this._remoteVideoStream.on('sizeChanged', this.isSizeChanged);
            this.checkAndUpdateScreenShareState();
        };
        this.unsubscribe = () => {
            this._remoteVideoStream.off('isAvailableChanged', this.isAvailableChanged);
            this._remoteVideoStream.off('isReceivingChanged', this.isReceivingChanged);
            this._remoteVideoStream.off('sizeChanged', this.isSizeChanged);
        };
        this.includesActiveScreenShareStream = (streams) => {
            for (const stream of Object.values(streams)) {
                if (stream.mediaStreamType === 'ScreenSharing' && stream.isAvailable) {
                    return true;
                }
            }
            return false;
        };
        /**
         * Update the state with the active screen share stream. If there is an existing stream will overwrite it if this one
         * is active (newer stream takes priority). If there is an existing stream and this one is set to unavailable, and the
         * existing stream is different participant, then don't set the active screen share stream to undefined, else set it
         * to undefined.
         */
        this.checkAndUpdateScreenShareState = () => {
            var _a, _b, _c;
            if (this._remoteVideoStream.mediaStreamType !== 'ScreenSharing') {
                return;
            }
            if (this._remoteVideoStream.isAvailable) {
                this._context.setCallScreenShareParticipant(this._callIdRef.callId, this._participantKey);
                return;
            }
            const existingScreenShare = (_a = this._context.getState().calls[this._callIdRef.callId]) === null || _a === void 0 ? void 0 : _a.screenShareRemoteParticipant;
            // If somehow we end up with an event where a RemoteParticipant's ScreenShare stream is set to
            // unavailable but there exists already another different participant actively sharing, and they are still
            // sharing then this event shouldn't set the screenShareRemoteParticipant to undefined.
            if (!existingScreenShare || existingScreenShare === this._participantKey) {
                this._context.setCallScreenShareParticipant(this._callIdRef.callId, undefined);
                return;
            }
            const streams = (_c = (_b = this._context.getState().calls[this._callIdRef.callId]) === null || _b === void 0 ? void 0 : _b.remoteParticipants[existingScreenShare]) === null || _c === void 0 ? void 0 : _c.videoStreams;
            if (!streams) {
                this._context.setCallScreenShareParticipant(this._callIdRef.callId, undefined);
                return;
            }
            // If the existing ScreenShare that is not owned by the current RemoteParticipant is still active, don't
            // overwrite it with undefined. So only overwrite if it is not active.
            if (!this.includesActiveScreenShareStream(streams)) {
                this._context.setCallScreenShareParticipant(this._callIdRef.callId, undefined);
                return;
            }
        };
        this.isAvailableChanged = () => {
            this._context.setRemoteVideoStreamIsAvailable(this._callIdRef.callId, this._participantKey, this._remoteVideoStream.id, this._remoteVideoStream.isAvailable);
            this.checkAndUpdateScreenShareState();
        };
        this.isReceivingChanged = () => {
            this._context.setRemoteVideoStreamIsReceiving(this._callIdRef.callId, this._participantKey, this._remoteVideoStream.id, this._remoteVideoStream.isReceiving);
        };
        this.isSizeChanged = () => {
            var _a, _b, _c, _d, _e, _f, _g;
            if (((_a = this._remoteVideoStream) === null || _a === void 0 ? void 0 : _a.size.width) === 0 && ((_b = this._remoteVideoStream) === null || _b === void 0 ? void 0 : _b.size.height) === 0) {
                return;
            }
            const streamSize = (_e = (_d = (_c = this._context.getState().calls[this._callIdRef.callId]) === null || _c === void 0 ? void 0 : _c.remoteParticipants[this._participantKey]) === null || _d === void 0 ? void 0 : _d.videoStreams[this._remoteVideoStream.id]) === null || _e === void 0 ? void 0 : _e.streamSize;
            const existingAspectRatio = streamSize ? streamSize.width / streamSize.height : undefined;
            const newAspectRatio = ((_f = this._remoteVideoStream) === null || _f === void 0 ? void 0 : _f.size.width) / ((_g = this._remoteVideoStream) === null || _g === void 0 ? void 0 : _g.size.height);
            if (!streamSize || existingAspectRatio !== newAspectRatio) {
                this._context.setRemoteVideoStreamSize(this._callIdRef.callId, this._participantKey, this._remoteVideoStream.id, this._remoteVideoStream.size);
            }
        };
        this._callIdRef = callIdRef;
        this._participantKey = participantKey;
        this._remoteVideoStream = remoteVideoStream;
        this._context = context;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * helper function to manage logging for stream disposals
 *
 * @param eventName Name of event that occured when managing streams
 * @param streamLogInfo Data about the stream in the event
 * @returns
 */
function _logDisposeStreamEvent(eventName, streamLogInfo) {
    switch (eventName) {
        case EventNames.DISPOSE_STREAM_INVALID_PARAMS:
            _logEvent(callingStatefulLogger, {
                name: EventNames.DISPOSE_STREAM_INVALID_PARAMS,
                level: 'warning',
                message: 'Dispose View invalid combination of parameters.',
                data: {
                    streamType: streamLogInfo.streamType
                }
            });
            return;
        case EventNames.START_DISPOSE_STREAM:
            _logEvent(callingStatefulLogger, {
                name: EventNames.START_DISPOSE_STREAM,
                level: 'info',
                message: 'Start disposing stream.',
                data: streamLogInfo
            });
            return;
        case EventNames.DISPOSE_INFO_NOT_FOUND:
            _logEvent(callingStatefulLogger, {
                name: EventNames.DISPOSE_INFO_NOT_FOUND,
                level: 'error',
                message: 'Cannot find render info when disposing stream.',
                data: streamLogInfo
            });
            return;
        case EventNames.STREAM_STOPPING:
            _logEvent(callingStatefulLogger, {
                name: EventNames.STREAM_STOPPING,
                level: 'info',
                message: 'Stream is currently marked as stopping, will continue if is local preview',
                data: streamLogInfo
            });
            return;
        case EventNames.DISPOSING_RENDERER:
            _logEvent(callingStatefulLogger, {
                name: EventNames.DISPOSING_RENDERER,
                level: 'info',
                message: 'Disposing view renderer.',
                data: streamLogInfo
            });
            return;
        case EventNames.RENDERER_NOT_FOUND:
            _logEvent(callingStatefulLogger, {
                name: EventNames.RENDERER_NOT_FOUND,
                level: 'error',
                message: 'Cannot find renderer when disposing stream.',
                data: streamLogInfo
            });
            return;
        case EventNames.STREAM_ALREADY_DISPOSED:
            _logEvent(callingStatefulLogger, {
                name: EventNames.STREAM_ALREADY_DISPOSED,
                level: 'info',
                message: 'Stream is already disposed.',
                data: streamLogInfo
            });
            return;
        default:
            return;
    }
}
/**
 * helper function to manage logging for local stream creations
 *
 * @param eventName Name of the event to occured when creating a local stream
 * @param streamLogInfo Data about the stream in the event
 * @param error that is thrown by caller
 * @returns
 */
function _logCreateStreamEvent(eventName, streamLogInfo, error) {
    switch (eventName) {
        case EventNames.CREATE_STREAM_INVALID_PARAMS:
            _logEvent(callingStatefulLogger, {
                name: EventNames.CREATE_STREAM_INVALID_PARAMS,
                level: 'warning',
                message: 'Create View invalid combination of parameters.',
                data: {
                    streamType: streamLogInfo.streamType
                }
            });
            return;
        case EventNames.START_STREAM_RENDERING:
            _logEvent(callingStatefulLogger, {
                name: EventNames.START_STREAM_RENDERING,
                level: 'info',
                message: 'Start creating view for local video.',
                data: streamLogInfo
            });
            return;
        case EventNames.STREAM_NOT_FOUND:
            _logEvent(callingStatefulLogger, {
                name: EventNames.STREAM_NOT_FOUND,
                level: 'error',
                message: 'Stream not found in state.',
                data: {
                    callId: streamLogInfo.callId
                }
            });
            return;
        case EventNames.STREAM_ALREADY_RENDERED:
            _logEvent(callingStatefulLogger, {
                name: EventNames.STREAM_ALREADY_RENDERED,
                level: 'warning',
                message: 'Stream is already rendered.',
                data: streamLogInfo
            });
            return;
        case EventNames.STREAM_RENDERING:
            _logEvent(callingStatefulLogger, {
                name: EventNames.STREAM_RENDERING,
                level: 'warning',
                message: 'Stream is rendering.',
                data: streamLogInfo
            });
            return;
        case EventNames.STREAM_STOPPING:
            _logEvent(callingStatefulLogger, {
                name: EventNames.STREAM_STOPPING,
                level: 'warning',
                message: 'Stream was marked as stopping by dispose view. Resetting state to "Rendering".',
                data: streamLogInfo
            });
            return;
        case EventNames.CREATE_STREAM_FAIL:
            _logEvent(callingStatefulLogger, {
                name: EventNames.CREATE_STREAM_FAIL,
                level: 'error',
                message: 'Failed to create view.',
                data: {
                    error: error,
                    streamType: streamLogInfo.streamType,
                    callId: streamLogInfo.callId
                }
            });
            return;
        case EventNames.RENDER_INFO_NOT_FOUND:
            _logEvent(callingStatefulLogger, {
                name: EventNames.RENDER_INFO_NOT_FOUND,
                level: 'error',
                message: 'Cannot find render info after create the view. ',
                data: streamLogInfo
            });
            return;
        case EventNames.CREATED_STREAM_STOPPING:
            _logEvent(callingStatefulLogger, {
                name: EventNames.CREATED_STREAM_STOPPING,
                level: 'warning',
                message: 'Render info status is stopping, dispose renderer.',
                data: streamLogInfo
            });
            return;
        case EventNames.VIEW_RENDER_SUCCEED:
            _logEvent(callingStatefulLogger, {
                name: EventNames.VIEW_RENDER_SUCCEED,
                level: 'info',
                message: `Successfully render the view.`,
                data: streamLogInfo
            });
            return;
        case EventNames.CREATING_VIEW:
            _logEvent(callingStatefulLogger, {
                name: EventNames.CREATING_VIEW,
                level: 'info',
                message: 'Start creating view for remote video.',
                data: streamLogInfo
            });
            return;
        default:
            return;
    }
}
/**
 * helper function to fire streamUtils logging events
 *
 * @param eventName Name of event from streamUtils
 * @param streamLogInfo informaiton about the event and who called it
 * @param error if any errors present will be added to message in logging
 */
function _logStreamEvent(eventName, streamLogInfo, error) {
    if (streamLogInfo.streamEventType === 'disposeViewLocal' || streamLogInfo.streamEventType === 'disposeViewRemote' || streamLogInfo.streamEventType === 'disposeViewUnparented') {
        _logDisposeStreamEvent(eventName, streamLogInfo);
    }
    else if (streamLogInfo.streamEventType === 'createViewLocal' || streamLogInfo.streamEventType === 'createViewRemote') {
        _logCreateStreamEvent(eventName, streamLogInfo, error);
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$V = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function createViewVideo(context, internalContext, callId, stream, participantId, options) {
    return __awaiter$V(this, void 0, void 0, function* () {
        // we can only have 3 types of createView
        let streamEventType;
        // we will reuse these for local as well but we need to make sure the remote stream is passed in like before.
        if (participantId) {
            streamEventType = 'createViewRemote';
        }
        else if (callId) {
            streamEventType = 'createViewLocal';
        }
        else {
            // TODO update for when unparented view.
            throw new Error('unparented createView not implemented yet here');
        }
        const streamType = stream === null || stream === void 0 ? void 0 : stream.mediaStreamType;
        const localStreamKey = stream.mediaStreamType;
        const remoteStreamId = stream.id;
        // we want to check to see if there is a participantId this will tell us whether its a local stream or a remote one.
        const participantKey = streamEventType === 'createViewRemote' && participantId ? typeof participantId === 'string' ? participantId : toFlatCommunicationIdentifier(participantId) : undefined;
        const streamLogInfo = {
            callId,
            participantKey,
            streamId: remoteStreamId !== null && remoteStreamId !== void 0 ? remoteStreamId : localStreamKey,
            streamType,
            streamEventType
        };
        // make different logging announcement based on whether or not we are starting a local or remote
        _logStreamEvent(EventNames.CREATING_VIEW, streamLogInfo);
        // if we have a participant Id and a stream get the remote info, else get the local render info from state.
        const renderInfo = streamEventType === 'createViewRemote' && participantKey ? internalContext.getRemoteRenderInfoForParticipant(callId, participantKey, remoteStreamId) : internalContext.getLocalRenderInfo(callId, localStreamKey);
        if (!renderInfo) {
            _logStreamEvent(EventNames.STREAM_NOT_FOUND, streamLogInfo);
            return;
        }
        if (renderInfo.status === 'Rendered') {
            _logStreamEvent(EventNames.STREAM_ALREADY_RENDERED, streamLogInfo);
            return;
        }
        if (renderInfo.status === 'Rendering') {
            // Do not log to console here as this is a very common situation due to UI rerenders while
            // the video rendering is in progress.
            _logStreamEvent(EventNames.STREAM_RENDERING, streamLogInfo);
            return;
        }
        // "Stopping" only happens if the stream was in "rendering" but `disposeView` was called.
        // Now that `createView` has been re-called, we can flip the state back to "rendering".
        if (renderInfo.status === 'Stopping') {
            if (streamEventType === 'createViewRemote' && participantKey) {
                _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
                internalContext.setRemoteRenderInfo(callId, participantKey, remoteStreamId, renderInfo.stream, 'Rendering', renderInfo.renderer);
            }
            else if (streamEventType === 'createViewLocal') {
                _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
                internalContext.setLocalRenderInfo(callId, localStreamKey, renderInfo.stream, 'Rendering', renderInfo.renderer);
            }
            return;
        }
        const renderer = new communicationCalling.VideoStreamRenderer(renderInfo.stream);
        if (streamEventType === 'createViewRemote' && participantKey) {
            internalContext.setRemoteRenderInfo(callId, participantKey, remoteStreamId, renderInfo.stream, 'Rendering', undefined);
        }
        else {
            internalContext.setLocalRenderInfo(callId, localStreamKey, renderInfo.stream, 'Rendering', renderer);
        }
        let view;
        try {
            view = yield renderer.createView(options);
        }
        catch (e) {
            if (streamEventType === 'createViewRemote' && participantKey) {
                _logStreamEvent(EventNames.CREATE_STREAM_FAIL, streamLogInfo);
                internalContext.setRemoteRenderInfo(callId, participantKey, remoteStreamId, renderInfo.stream, 'NotRendered', undefined);
            }
            else if (streamEventType === 'createViewLocal') {
                _logStreamEvent(EventNames.CREATE_STREAM_FAIL, streamLogInfo, e);
                internalContext.setLocalRenderInfo(callId, localStreamKey, renderInfo.stream, 'NotRendered', undefined);
            }
            throw e;
        }
        // Since render could take some time, we need to check if the stream is still valid and if we received a signal to
        // stop rendering.
        const refreshedRenderInfo = streamEventType === 'createViewRemote' && participantKey ? internalContext.getRemoteRenderInfoForParticipant(callId, participantKey, remoteStreamId) : internalContext.getLocalRenderInfo(callId, localStreamKey);
        if (!refreshedRenderInfo) {
            // RenderInfo was removed. This should not happen unless stream was removed from the call so dispose the renderer
            // and clean up the state.
            _logStreamEvent(EventNames.RENDER_INFO_NOT_FOUND, streamLogInfo);
            renderer.dispose();
            if (streamEventType === 'createViewRemote' && participantKey) {
                context.setRemoteVideoStreamRendererView(callId, participantKey, remoteStreamId, undefined);
            }
            else {
                context.setLocalVideoStreamRendererView(callId, localStreamKey, undefined);
            }
            return;
        }
        if (refreshedRenderInfo.status === 'Stopping') {
            // Stop render was called on this stream after we had started rendering. We will dispose this view and do not
            // put the view into the state.
            _logStreamEvent(EventNames.CREATED_STREAM_STOPPING, streamLogInfo);
            renderer.dispose();
            if (streamEventType === 'createViewRemote' && participantKey) {
                internalContext.setRemoteRenderInfo(callId, participantKey, remoteStreamId, refreshedRenderInfo.stream, 'NotRendered', undefined);
                context.setRemoteVideoStreamRendererView(callId, participantKey, remoteStreamId, undefined);
            }
            else if (streamEventType === 'createViewLocal') {
                internalContext.setLocalRenderInfo(callId, localStreamKey, refreshedRenderInfo.stream, 'NotRendered', undefined);
                context.setLocalVideoStreamRendererView(callId, localStreamKey, undefined);
            }
            return;
        }
        // Else the stream still exists and status is not telling us to stop rendering. Complete the render process by
        // updating the state.
        if (streamEventType === 'createViewRemote' && participantKey) {
            internalContext.setRemoteRenderInfo(callId, participantKey, remoteStreamId, refreshedRenderInfo.stream, 'Rendered', renderer);
            context.setRemoteVideoStreamRendererView(callId, participantKey, remoteStreamId, convertFromSDKToDeclarativeVideoStreamRendererView(view));
            _logStreamEvent(EventNames.VIEW_RENDER_SUCCEED, streamLogInfo);
        }
        else if (streamEventType === 'createViewLocal') {
            internalContext.setLocalRenderInfo(callId, localStreamKey, refreshedRenderInfo.stream, 'Rendered', renderer);
            context.setLocalVideoStreamRendererView(callId, localStreamKey, convertFromSDKToDeclarativeVideoStreamRendererView(view));
            _logStreamEvent(EventNames.VIEW_RENDER_SUCCEED, streamLogInfo);
        }
        return {
            renderer,
            view
        };
    });
}
function createViewUnparentedVideo(context, internalContext, stream, options) {
    return __awaiter$V(this, void 0, void 0, function* () {
        const renderInfo = internalContext.getUnparentedRenderInfo(stream);
        if (renderInfo && renderInfo.status === 'Rendered') {
            console.warn('Unparented LocalVideoStream is already rendered');
            return;
        }
        if (renderInfo && renderInfo.status === 'Rendering') {
            // Do not log to console here as this is a very common situation due to UI rerenders while
            // the video rendering is in progress.
            return;
        }
        if (renderInfo && renderInfo.status === 'Stopping') {
            console.warn('Unparented LocalVideoStream is in the middle of stopping');
            return;
        }
        const localVideoStream = new communicationCalling.LocalVideoStream(stream.source);
        const renderer = new communicationCalling.VideoStreamRenderer(localVideoStream);
        internalContext.setUnparentedRenderInfo(stream, localVideoStream, 'Rendering', undefined);
        let view;
        try {
            view = yield renderer.createView(options);
        }
        catch (e) {
            // Special case for unparented views. Since they are not tied to anything and created by us based on the calls to
            // this function we'll delete it to clean up the data since keeping it around doesn't help us and if developer wants
            // to create a new view they can check that the view is not rendered and call this function again.
            internalContext.deleteUnparentedRenderInfo(stream);
            throw e;
        }
        // Since render could take some time, we need to check if the stream is still valid and if we received a signal to
        // stop rendering.
        const refreshedRenderInfo = internalContext.getUnparentedRenderInfo(stream);
        if (!refreshedRenderInfo) {
            // Unparented stream's RenderInfo was deleted. Currently this shouldn't happen but if it does we'll just dispose the
            // renderer and clean up state. If developer wanted the stream they could call this function again and that should
            // generate new working state via this function.
            renderer.dispose();
            context.deleteDeviceManagerUnparentedView(stream);
            return;
        }
        if (refreshedRenderInfo.status === 'Stopping') {
            // Stop render was called on this stream after we had started rendering. We will dispose this view and do not
            // put the view into the state. Special case for unparented views, delete them from state when stopped to free up
            // the memory since we were the ones generating this and not tied to any Call state.
            internalContext.deleteUnparentedRenderInfo(stream);
            context.deleteDeviceManagerUnparentedView(stream);
            return;
        }
        // Else the stream still exists and status is not telling us to stop rendering. Complete the render process by
        // updating the state.
        internalContext.setUnparentedRenderInfo(stream, localVideoStream, 'Rendered', renderer);
        internalContext.subscribeToUnparentedViewVideoEffects(localVideoStream, context);
        context.setDeviceManagerUnparentedView(stream, convertFromSDKToDeclarativeVideoStreamRendererView(view));
        return {
            renderer,
            view
        };
    });
}
function disposeViewVideo(context, internalContext, callId, stream, participantId) {
    // we can only have 3 types of createView
    let streamEventType;
    // we will reuse these for local as well but we need to make sure the remote stream is passed in like before.
    if (participantId) {
        streamEventType = 'disposeViewRemote';
    }
    else if (callId) {
        streamEventType = 'disposeViewLocal';
    }
    else {
        // TODO update for when unparented view.
        streamEventType = 'disposeViewUnparented';
    }
    const streamType = stream.mediaStreamType;
    const localStreamKey = stream.mediaStreamType;
    const remoteStreamId = stream.id;
    // we want to check to see if there is a participantId this will tell us whether its a local stream or a remote one.
    const participantKey = streamEventType === 'disposeViewRemote' && participantId ? typeof participantId === 'string' ? participantId : toFlatCommunicationIdentifier(participantId) : undefined;
    const streamLogInfo = {
        callId,
        participantKey,
        streamId: remoteStreamId !== null && remoteStreamId !== void 0 ? remoteStreamId : localStreamKey,
        streamType
    };
    _logStreamEvent(EventNames.START_DISPOSE_STREAM, streamLogInfo);
    if (streamEventType === 'disposeViewRemote' && participantKey) {
        context.setRemoteVideoStreamRendererView(callId, participantKey, remoteStreamId, undefined);
    }
    const renderInfo = streamEventType === 'disposeViewRemote' && participantKey ? internalContext.getRemoteRenderInfoForParticipant(callId, participantKey, remoteStreamId) : internalContext.getLocalRenderInfo(callId, localStreamKey);
    if (!renderInfo) {
        _logStreamEvent(EventNames.DISPOSE_INFO_NOT_FOUND, streamLogInfo);
        return;
    }
    // Nothing to dispose of or clean up -- we can safely exit early here.
    if (renderInfo.status === 'NotRendered') {
        _logStreamEvent(EventNames.STREAM_ALREADY_DISPOSED, streamLogInfo);
        return;
    }
    // Status is already marked as "stopping" so we can exit early here. This is because stopping only occurs
    // when the stream is being created in createView but hasn't been completed being created yet. The createView
    // method will see the "stopping" status and perform the cleanup
    if (renderInfo.status === 'Stopping') {
        _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
        return;
    }
    // If the stream is in the middle of being rendered (i.e. has state "Rendering"), we need the status as
    // "stopping" without performing any cleanup. This will tell the `createView` method that it should stop
    // rendering and clean up the state once the view has finished being created.
    if (renderInfo.status === 'Rendering') {
        _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
        if (streamEventType === 'disposeViewRemote' && participantKey) {
            internalContext.setRemoteRenderInfo(callId, participantKey, remoteStreamId, renderInfo.stream, 'Stopping', undefined);
        }
        else {
            internalContext.setLocalRenderInfo(callId, localStreamKey, renderInfo.stream, 'Stopping', renderInfo.renderer);
        }
        return;
    }
    if (renderInfo.renderer) {
        _logStreamEvent(EventNames.DISPOSING_RENDERER, streamLogInfo);
        renderInfo.renderer.dispose();
        // Else the state must be in the "Rendered" state, so we can dispose the renderer and clean up the state.
        if (streamEventType === 'disposeViewRemote' && participantKey) {
            internalContext.setRemoteRenderInfo(callId, participantKey, remoteStreamId, renderInfo.stream, 'NotRendered', undefined);
        }
        else if (streamEventType === 'disposeViewLocal') {
            internalContext.setLocalRenderInfo(callId, localStreamKey, renderInfo.stream, 'NotRendered', undefined);
            context.setLocalVideoStreamRendererView(callId, localStreamKey, undefined);
        }
    }
    else {
        _logStreamEvent(EventNames.RENDERER_NOT_FOUND, streamLogInfo);
    }
}
function disposeViewUnparentedVideo(context, internalContext, stream) {
    const streamType = stream.mediaStreamType;
    const streamLogInfo = {
        streamType,
        streamEventType: 'disposeViewUnparented'
    };
    _logStreamEvent(EventNames.START_DISPOSE_STREAM, streamLogInfo);
    context.deleteDeviceManagerUnparentedView(stream);
    const renderInfo = internalContext.getUnparentedRenderInfo(stream);
    if (!renderInfo) {
        _logStreamEvent(EventNames.DISPOSE_INFO_NOT_FOUND, streamLogInfo);
        return;
    }
    if (renderInfo.status === 'Rendering') {
        _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
        internalContext.setUnparentedRenderInfo(stream, renderInfo.stream, 'Stopping', undefined);
    }
    else {
        internalContext.deleteUnparentedRenderInfo(stream);
    }
    if (renderInfo.renderer) {
        _logStreamEvent(EventNames.DISPOSING_RENDERER, streamLogInfo);
        renderInfo.renderer.dispose();
    }
    else {
        _logStreamEvent(EventNames.RENDERER_NOT_FOUND, streamLogInfo);
    }
}
/**
 * @private
 */
function createView$1(context, internalContext, callId, participantId, stream, options) {
    const streamType = stream.mediaStreamType;
    if (callId) {
        return createViewVideo(context, internalContext, callId, stream, participantId, options);
    }
    else if (!('id' in stream) && !callId) {
        // Render LocalVideoStream that is not part of a Call
        // Because it is not part of the call we don't tee errors to state naturally (e.g. via a Call Client function such as startVideo).
        // We do not have a startLocalPreviewVideo function, so as a workaround we ensure any errors are propagated here.
        return context.withAsyncErrorTeedToState(() => __awaiter$V(this, void 0, void 0, function* () { return yield createViewUnparentedVideo(context, internalContext, stream, options); }), 'Call.startVideo')();
    }
    else {
        _logStreamEvent(EventNames.CREATE_STREAM_INVALID_PARAMS, {
            streamType
        });
        return Promise.resolve(undefined);
    }
}
/**
 * @private
 */
function disposeView$1(context, internalContext, callId, participantId, stream) {
    const streamType = stream.mediaStreamType;
    if (callId) {
        disposeViewVideo(context, internalContext, callId, stream, participantId);
    }
    else if (!('id' in stream) && !callId) {
        // Stop rendering LocalVideoStream that is not part of a Call
        // Because it is not part of the call we don't tee errors to state naturally (e.g. via a Call Client function such as startVideo).
        // We do not have a stopLocalPreviewVideo function, so as a workaround we ensure any errors are propagated here.
        context.withErrorTeedToState(() => disposeViewUnparentedVideo(context, internalContext, stream), 'Call.stopVideo')();
    }
    else {
        _logStreamEvent(EventNames.DISPOSE_STREAM_INVALID_PARAMS, {
            streamType
        });
        return;
    }
}
/**
 * @private
 * Only stops videos that are tied to a Call.
 */
function disposeAllViewsFromCall(context, internalContext, callId) {
    const remoteStreams = internalContext.getRemoteRenderInfoForCall(callId);
    if (remoteStreams) {
        for (const [participantKey, participantStreams] of remoteStreams.entries()) {
            for (const [_, remoteStreamAndRenderer] of participantStreams.entries()) {
                // We don't want to accept SDK stream as parameter but we also don't cache the declarative stream so we have to
                // convert the SDK stream to declarative stream which is not pretty so this could use some further refactoring.
                disposeView$1(context, internalContext, callId, participantKey, convertSdkRemoteStreamToDeclarativeRemoteStream(remoteStreamAndRenderer.stream));
            }
        }
    }
    const localStreams = internalContext.getLocalRenderInfosForCall(callId);
    if (localStreams) {
        for (const localStreamAndRenderer of localStreams.values()) {
            if (localStreamAndRenderer && localStreamAndRenderer.renderer) {
                // We don't want to accept SDK stream as parameter but we also don't cache the declarative stream so we have to
                // convert the SDK stream to declarative stream which is not pretty so this could use some further refactoring.
                disposeView$1(context, internalContext, callId, undefined, convertSdkLocalStreamToDeclarativeLocalStream(localStreamAndRenderer.stream));
            }
        }
    }
    const callFeatureStreams = internalContext.getCallFeatureRenderInfosForCall(callId);
    if (callFeatureStreams) {
        for (const featureStreams of callFeatureStreams.values()) {
            for (const streamAndRenderer of featureStreams.values()) {
                disposeView$1(context, internalContext, callId, undefined, convertSdkRemoteStreamToDeclarativeRemoteStream(streamAndRenderer.stream));
            }
        }
    }
}
/**
 * @private
 */
function disposeAllViews(context, internalContext) {
    const callIds = internalContext.getCallIds();
    for (const callId of callIds) {
        disposeAllViewsFromCall(context, internalContext, callId);
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Keeps track of the listeners assigned to a particular participant because when we get an event from SDK, it doesn't
 * tell us which participant it is for. If we keep track of this then we know which participant in the state that needs
 * an update and also which property of that participant. Also we can use this when unregistering to a participant.
 */
let ParticipantSubscriber$1 = class ParticipantSubscriber {
    constructor(callIdRef, participant, context, internalContext) {
        this.subscribe = () => {
            this._participant.on('stateChanged', this.stateChanged);
            this._participant.on('isMutedChanged', this.isMutedChanged);
            this._participant.on('displayNameChanged', this.displayNameChanged);
            this._participant.on('isSpeakingChanged', this.isSpeakingChanged);
            this._participant.on('videoStreamsUpdated', this.videoStreamsUpdated);
            this._participant.on('roleChanged', this.roleChanged);
            if (this._participant.videoStreams.length > 0) {
                for (const stream of this._participant.videoStreams) {
                    this._internalContext.setRemoteRenderInfo(this._callIdRef.callId, this._participantKey, stream.id, stream, 'NotRendered', undefined);
                    this.addRemoteVideoStreamSubscriber(stream);
                }
                this._context.setRemoteVideoStreams(this._callIdRef.callId, this._participantKey, this._participant.videoStreams.map(convertSdkRemoteStreamToDeclarativeRemoteStream), []);
            }
        };
        this.unsubscribe = () => {
            this._participant.off('stateChanged', this.stateChanged);
            this._participant.off('isMutedChanged', this.isMutedChanged);
            this._participant.off('displayNameChanged', this.displayNameChanged);
            this._participant.off('isSpeakingChanged', this.isSpeakingChanged);
            this._participant.off('videoStreamsUpdated', this.videoStreamsUpdated);
            this._participant.off('roleChanged', this.roleChanged);
            // If unsubscribing it means the participant left the call. If they have any rendering streams we should stop them
            // as it doesn't make sense to render for an ended participant.
            if (this._participant.videoStreams.length > 0) {
                for (const stream of this._participant.videoStreams) {
                    disposeView$1(this._context, this._internalContext, this._callIdRef.callId, this._participantKey, convertSdkRemoteStreamToDeclarativeRemoteStream(stream));
                    this._internalContext.deleteRemoteRenderInfo(this._callIdRef.callId, this._participantKey, stream.id);
                }
            }
        };
        this.addRemoteVideoStreamSubscriber = (remoteVideoStream) => {
            var _a;
            (_a = this._remoteVideoStreamSubscribers.get(remoteVideoStream.id)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            this._remoteVideoStreamSubscribers.set(remoteVideoStream.id, new RemoteVideoStreamSubscriber(this._callIdRef, this._participantKey, remoteVideoStream, this._context));
        };
        this.stateChanged = () => {
            this._context.setParticipantState(this._callIdRef.callId, this._participantKey, this._participant.state);
        };
        this.isMutedChanged = () => {
            this._context.setParticipantIsMuted(this._callIdRef.callId, this._participantKey, this._participant.isMuted);
        };
        this.roleChanged = () => {
            this._context.setParticipantRole(this._callIdRef.callId, this._participantKey, this._participant.role);
        };
        this.displayNameChanged = () => {
            this._context.setParticipantDisplayName(this._callIdRef.callId, this._participantKey, this._participant.displayName || '');
        };
        this.isSpeakingChanged = () => {
            this._context.setParticipantIsSpeaking(this._callIdRef.callId, this._participantKey, this._participant.isSpeaking);
        };
        this.videoStreamsUpdated = (event) => {
            var _a;
            for (const stream of event.removed) {
                (_a = this._remoteVideoStreamSubscribers.get(stream.id)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                disposeView$1(this._context, this._internalContext, this._callIdRef.callId, this._participantKey, convertSdkRemoteStreamToDeclarativeRemoteStream(stream));
                this._internalContext.deleteRemoteRenderInfo(this._callIdRef.callId, this._participantKey, stream.id);
            }
            for (const stream of event.added) {
                this._internalContext.setRemoteRenderInfo(this._callIdRef.callId, this._participantKey, stream.id, stream, 'NotRendered', undefined);
                this.addRemoteVideoStreamSubscriber(stream);
            }
            this._context.setRemoteVideoStreams(this._callIdRef.callId, this._participantKey, event.added.map(convertSdkRemoteStreamToDeclarativeRemoteStream), event.removed.map((stream) => stream.id));
        };
        this._callIdRef = callIdRef;
        this._participant = participant;
        this._context = context;
        this._internalContext = internalContext;
        this._participantKey = toFlatCommunicationIdentifier(this._participant.identifier);
        this._remoteVideoStreamSubscribers = new Map();
        this.subscribe();
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class RecordingSubscriber {
    constructor(callIdRef, context, recording) {
        this.subscribe = () => {
            this._recording.on('isRecordingActiveChanged', this.isAvailableChanged);
        };
        this.unsubscribe = () => {
            this._recording.off('isRecordingActiveChanged', this.isAvailableChanged);
        };
        this.isAvailableChanged = () => {
            this._context.setCallRecordingActive(this._callIdRef.callId, this._recording.isRecordingActive);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._recording = recording;
        // If recording as already started when we joined the call, make sure it is reflected in state as there may not
        // be an event for it.
        if (this._recording.isRecordingActive) {
            this._context.setCallRecordingActive(this._callIdRef.callId, this._recording.isRecordingActive);
        }
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$U = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
class PPTLiveSubscriber {
    constructor(callIdRef, context, call) {
        this.subscribe = () => {
            this._pptLive.on('isActiveChanged', this.isAvailableChanged);
        };
        this.unsubscribe = () => {
            this._pptLive.off('isActiveChanged', this.isAvailableChanged);
        };
        this.isAvailableChanged = () => {
            this._context.setCallPPTLiveActive(this._callIdRef.callId, this._pptLive.isActive);
            this.checkAndUpdatePPTLiveParticipant();
        };
        this.checkAndUpdatePPTLiveParticipant = () => __awaiter$U(this, void 0, void 0, function* () {
            if (!this._pptLive.activePresenterId) {
                return;
            }
            if (this._pptLive.isActive) {
                // TODO： need to refactor if Web Calling SDK has this logic ready
                if (this._call.isScreenSharingOn) {
                    yield this._call.stopScreenSharing();
                }
                this._context.setCallParticipantPPTLive(this._callIdRef.callId, toFlatCommunicationIdentifier(this._pptLive.activePresenterId), this._pptLive.target);
            }
            else {
                this._context.setCallParticipantPPTLive(this._callIdRef.callId, toFlatCommunicationIdentifier(this._pptLive.activePresenterId), undefined);
            }
        });
        this._callIdRef = callIdRef;
        this._context = context;
        this._pptLive = call.feature(communicationCalling.Features.PPTLive);
        this._call = call;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class TranscriptionSubscriber {
    constructor(callIdRef, context, transcription) {
        this.subscribe = () => {
            this._transcription.on('isTranscriptionActiveChanged', this.isTranscriptionActiveChanged);
        };
        this.unsubscribe = () => {
            this._transcription.off('isTranscriptionActiveChanged', this.isTranscriptionActiveChanged);
        };
        this.isTranscriptionActiveChanged = () => {
            this._context.setCallTranscriptionActive(this._callIdRef.callId, this._transcription.isTranscriptionActive);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._transcription = transcription;
        // If transcription as already started when we joined the call, make sure it is reflected in state as there may not
        // be an event for it.
        if (this._transcription.isTranscriptionActive) {
            this._context.setCallTranscriptionActive(this._callIdRef.callId, this._transcription.isTranscriptionActive);
        }
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class UserFacingDiagnosticsSubscriber {
    constructor(callIdRef, context, diagnostics) {
        this.unsubscribe = () => {
            this._diagnostics.network.off('diagnosticChanged', this.networkDiagnosticsChanged.bind(this));
            this._diagnostics.media.off('diagnosticChanged', this.mediaDiagnosticsChanged.bind(this));
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._diagnostics = diagnostics;
        this.setInitialDiagnostics();
        this.subscribe();
    }
    setInitialDiagnostics() {
        const network = this._diagnostics.network.getLatest();
        const media = this._diagnostics.media.getLatest();
        if (Object.entries(network).length === 0 && Object.entries(media).length === 0) {
            return;
        }
        this._context.modifyState(state => {
            const call = state.calls[this._callIdRef.callId];
            if (call === undefined) {
                return;
            }
            call.diagnostics = {
                network: {
                    latest: network
                },
                media: {
                    latest: media
                }
            };
        });
    }
    subscribe() {
        this._diagnostics.network.on('diagnosticChanged', this.networkDiagnosticsChanged.bind(this));
        this._diagnostics.media.on('diagnosticChanged', this.mediaDiagnosticsChanged.bind(this));
    }
    networkDiagnosticsChanged(args) {
        this._context.modifyState(state => {
            var _a;
            const call = state.calls[this._callIdRef.callId];
            if (call === undefined) {
                return;
            }
            const network = (_a = call.diagnostics) === null || _a === void 0 ? void 0 : _a.network.latest;
            if (network) {
                network[args.diagnostic] = latestFromEvent(args);
            }
        });
    }
    mediaDiagnosticsChanged(args) {
        this._context.modifyState(state => {
            var _a;
            const call = state.calls[this._callIdRef.callId];
            if (call === undefined) {
                return;
            }
            const media = (_a = call.diagnostics) === null || _a === void 0 ? void 0 : _a.media.latest;
            if (media) {
                media[args.diagnostic] = latestFromEvent(args);
            }
        });
    }
}
const latestFromEvent = (args) => ({
    value: args.value,
    valueType: args.valueType
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class RaiseHandSubscriber {
    constructor(callIdRef, context, raiseHand) {
        this.subscribe = () => {
            this._raiseHand.on('raisedHandEvent', this.raisedHand);
            this._raiseHand.on('loweredHandEvent', this.loweredHand);
        };
        this.unsubscribe = () => {
            this._raiseHand.off('raisedHandEvent', this.raisedHand);
            this._raiseHand.off('loweredHandEvent', this.loweredHand);
        };
        this.raisedHand = () => {
            this._context.setCallRaisedHands(this._callIdRef.callId, this._raiseHand.getRaisedHands());
            for (const raisedHand of this._raiseHand.getRaisedHands()) {
                this._context.setParticipantIsRaisedHand(this._callIdRef.callId, toFlatCommunicationIdentifier(raisedHand.identifier), raisedHand);
            }
        };
        this.loweredHand = (event) => {
            this.raisedHand();
            this._context.setParticipantIsRaisedHand(this._callIdRef.callId, toFlatCommunicationIdentifier(event.identifier), undefined);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._raiseHand = raiseHand;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Subscribes to a Optimal Video Count Feature events and updates the call context appropriately.
 * @private
 */
class OptimalVideoCountSubscriber {
    constructor(args) {
        this.subscribe = () => {
            this._localOptimalVideoCountFeature.on('optimalVideoCountChanged', this.optimalVideoCountChanged);
        };
        this.unsubscribe = () => {
            this._localOptimalVideoCountFeature.off('optimalVideoCountChanged', this.optimalVideoCountChanged);
        };
        this.optimalVideoCountChanged = () => {
            this.updateOptimalVideoCountState({
                maxRemoteVideoStreams: this._localOptimalVideoCountFeature.optimalVideoCount
            });
        };
        this.updateOptimalVideoCountState = (newOptimalVideoCountState) => {
            this._context.setOptimalVideoCount(this._callIdRef.callId, newOptimalVideoCountState.maxRemoteVideoStreams);
        };
        this._callIdRef = args.callIdRef;
        this._context = args.context;
        this._localOptimalVideoCountFeature = args.localOptimalVideoCountFeature;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class CapabilitiesSubscriber {
    constructor(callIdRef, context, capabilities) {
        this._capabilitiesInitialized = false;
        this.subscribe = () => {
            this._capabilitiesFeature.on('capabilitiesChanged', this.capabilitiesChanged);
        };
        this.unsubscribe = () => {
            this._capabilitiesFeature.off('capabilitiesChanged', this.capabilitiesChanged);
        };
        this.capabilitiesChanged = (data) => {
            this._context.setCapabilities(this._callIdRef.callId, this._capabilitiesFeature.capabilities, data);
            if (data.oldValue.viewAttendeeNames !== data.newValue.viewAttendeeNames) {
                this._context.setHideAttendeeNames(this._callIdRef.callId, data);
            }
            this.setUnmuteMicAndTurnVideoOnNotification(data);
            this._capabilitiesInitialized = true;
        };
        this.setUnmuteMicAndTurnVideoOnNotification = (data) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            if (((_a = data.oldValue.turnVideoOn) === null || _a === void 0 ? void 0 : _a.isPresent) !== ((_b = data.newValue.turnVideoOn) === null || _b === void 0 ? void 0 : _b.isPresent)) {
                if (((_c = data.oldValue.turnVideoOn) === null || _c === void 0 ? void 0 : _c.isPresent) === false && ((_d = data.newValue.turnVideoOn) === null || _d === void 0 ? void 0 : _d.isPresent) && this._capabilitiesInitialized) {
                    const capabilityTurnVideoOnAbsent = this._context.getState().latestNotifications.capabilityTurnVideoOnAbsent;
                    if (capabilityTurnVideoOnAbsent) {
                        this._context.deleteLatestNotification('capabilityTurnVideoOnAbsent', this._callIdRef.callId);
                    }
                    const capabilityTurnVideoOnPresent = this._context.getState().latestNotifications.capabilityTurnVideoOnPresent;
                    if (!capabilityTurnVideoOnPresent) {
                        this._context.setLatestNotification(this._callIdRef.callId, {
                            target: 'capabilityTurnVideoOnPresent',
                            timestamp: new Date(Date.now())
                        });
                    }
                }
                if (((_e = data.newValue.turnVideoOn) === null || _e === void 0 ? void 0 : _e.isPresent) === false) {
                    const capabilityTurnVideoOnPresent = this._context.getState().latestNotifications.capabilityTurnVideoOnPresent;
                    if (capabilityTurnVideoOnPresent) {
                        this._context.deleteLatestNotification('capabilityTurnVideoOnPresent', this._callIdRef.callId);
                    }
                    const capabilityTurnVideoOnAbsent = this._context.getState().latestNotifications.capabilityTurnVideoOnAbsent;
                    if (!capabilityTurnVideoOnAbsent) {
                        this._context.setLatestNotification(this._callIdRef.callId, {
                            target: 'capabilityTurnVideoOnAbsent',
                            timestamp: new Date(Date.now())
                        });
                    }
                }
            }
            if (((_f = data.oldValue.unmuteMic) === null || _f === void 0 ? void 0 : _f.isPresent) !== ((_g = data.newValue.unmuteMic) === null || _g === void 0 ? void 0 : _g.isPresent)) {
                if (((_h = data.oldValue.unmuteMic) === null || _h === void 0 ? void 0 : _h.isPresent) === false && ((_j = data.newValue.unmuteMic) === null || _j === void 0 ? void 0 : _j.isPresent) && this._capabilitiesInitialized) {
                    const capabilityUnmuteMicAbsent = this._context.getState().latestNotifications.capabilityUnmuteMicAbsent;
                    if (capabilityUnmuteMicAbsent) {
                        this._context.deleteLatestNotification('capabilityUnmuteMicAbsent', this._callIdRef.callId);
                    }
                    const capabilityUnmuteMicPresent = this._context.getState().latestNotifications.capabilityUnmuteMicPresent;
                    if (!capabilityUnmuteMicPresent) {
                        this._context.setLatestNotification(this._callIdRef.callId, {
                            target: 'capabilityUnmuteMicPresent',
                            timestamp: new Date(Date.now())
                        });
                    }
                }
                if (((_k = data.newValue.unmuteMic) === null || _k === void 0 ? void 0 : _k.isPresent) === false) {
                    const capabilityUnmuteMicPresent = this._context.getState().latestNotifications.capabilityUnmuteMicPresent;
                    if (capabilityUnmuteMicPresent) {
                        this._context.deleteLatestNotification('capabilityUnmuteMicPresent', this._callIdRef.callId);
                    }
                    const capabilityUnmuteMicAbsent = this._context.getState().latestNotifications.capabilityUnmuteMicAbsent;
                    if (!capabilityUnmuteMicAbsent) {
                        this._context.setLatestNotification(this._callIdRef.callId, {
                            target: 'capabilityUnmuteMicAbsent',
                            timestamp: new Date(Date.now())
                        });
                    }
                }
            }
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._capabilitiesFeature = capabilities;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class ReactionSubscriber {
    constructor(callIdRef, context, reaction) {
        this.subscribe = () => {
            this._reaction.on('reaction', this.onReactionEvent);
        };
        this.unsubscribe = () => {
            this._reaction.off('reaction', this.onReactionEvent);
        };
        this.onReactionEvent = (event) => {
            this._context.setReceivedReactionFromParticipant(this._callIdRef.callId, event.identifier, event.reactionMessage);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._reaction = reaction;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class SpotlightSubscriber {
    constructor(callIdRef, context, spotlight) {
        this.subscribe = () => {
            this._spotlightFeature.on('spotlightChanged', this.spotlightChanged);
        };
        this.unsubscribe = () => {
            this._spotlightFeature.off('spotlightChanged', this.spotlightChanged);
        };
        this.spotlightChanged = (args) => {
            this._context.setSpotlight(this._callIdRef.callId, this._spotlightFeature.getSpotlightedParticipants(), this._spotlightFeature.maxParticipantsToSpotlight);
            for (const addedParticipant of args.added) {
                this._context.setParticipantSpotlighted(this._callIdRef.callId, addedParticipant);
            }
            for (const removedParticipant of args.removed) {
                this._context.setParticipantNotSpotlighted(this._callIdRef.callId, removedParticipant);
            }
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._spotlightFeature = spotlight;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const MILLSECONDS_BEFORE_END_TIME_TO_SHOW_CLOSING_NOTIFICATION = 30000;
/**
 * @private
 */
class BreakoutRoomsSubscriber {
    constructor(callIdRef, context, breakoutRoomsFeature) {
        this.unsubscribe = () => {
            this._breakoutRoomsFeature.off('breakoutRoomsUpdated', this.onBreakoutRoomsUpdated);
            // Clear breakout room joined notification for this call.
            this._context.deleteLatestNotification('breakoutRoomJoined', this._callIdRef.callId);
            // Clear breakout room closing soon notification for this call.
            this._context.deleteLatestNotification('breakoutRoomClosingSoon', this._callIdRef.callId);
            clearTimeout(this._breakoutRoomClosingSoonTimeoutId);
        };
        this.onBreakoutRoomsUpdated = (eventData) => {
            if (eventData.type === 'assignedBreakoutRooms') {
                this.onAssignedBreakoutRoomUpdated(eventData.data);
            }
            else if (eventData.type === 'join') {
                this.onBreakoutRoomsJoined(eventData.data);
            }
            else if (eventData.type === 'breakoutRoomsSettings' && eventData.data) {
                this.onBreakoutRoomSettingsUpdated(eventData.data);
            }
        };
        this.onAssignedBreakoutRoomUpdated = (breakoutRoom) => {
            var _a, _b, _c, _d;
            const callState = this._context.getState().calls[this._callIdRef.callId];
            const currentAssignedBreakoutRoom = (_a = callState === null || callState === void 0 ? void 0 : callState.breakoutRooms) === null || _a === void 0 ? void 0 : _a.assignedBreakoutRoom;
            // This call won't exist in the calls array in state if this call is a breakout room that was re-assigned.
            // If so, do nothing.
            if (callState === undefined) {
                return;
            }
            if (!breakoutRoom) {
                // This scenario covers the case where the user is unassigned from a breakout room.
                if ((_b = currentAssignedBreakoutRoom === null || currentAssignedBreakoutRoom === void 0 ? void 0 : currentAssignedBreakoutRoom.call) === null || _b === void 0 ? void 0 : _b.id) {
                    this._context.deleteLatestNotification('breakoutRoomJoined', currentAssignedBreakoutRoom.call.id);
                    this._context.deleteLatestNotification('breakoutRoomClosingSoon', currentAssignedBreakoutRoom.call.id);
                    clearTimeout(this._breakoutRoomClosingSoonTimeoutId);
                }
                this._context.setAssignedBreakoutRoom(this._callIdRef.callId, breakoutRoom);
                return;
            }
            // TODO: Fix the condition in this if statement to check for different breakout room ID instead of the display name
            // when Calling SDK fixes the breakout room call id to be correct
            if (breakoutRoom.state === 'open' && ((_c = callState.breakoutRooms) === null || _c === void 0 ? void 0 : _c.breakoutRoomDisplayName) && callState.breakoutRooms.breakoutRoomDisplayName !== breakoutRoom.displayName) {
                this._context.setLatestNotification(this._callIdRef.callId, {
                    target: 'assignedBreakoutRoomChanged',
                    timestamp: new Date(Date.now())
                });
            }
            else if (breakoutRoom.state === 'open' && !((_d = callState === null || callState === void 0 ? void 0 : callState.breakoutRooms) === null || _d === void 0 ? void 0 : _d.breakoutRoomSettings) && this._context.getOpenBreakoutRoom() === undefined) {
                const target = breakoutRoom.autoMoveParticipantToBreakoutRoom === false ? 'assignedBreakoutRoomOpenedPromptJoin' : 'assignedBreakoutRoomOpened';
                this._context.setLatestNotification(this._callIdRef.callId, {
                    target,
                    timestamp: new Date(Date.now())
                });
            }
            else if (breakoutRoom.state === 'closed' && (currentAssignedBreakoutRoom === null || currentAssignedBreakoutRoom === void 0 ? void 0 : currentAssignedBreakoutRoom.state) === 'open') {
                // This scenario covers the case where the breakout room is opened but then closed before the user joins.
                this._context.deleteLatestNotification('assignedBreakoutRoomOpened', this._callIdRef.callId);
                this._context.deleteLatestNotification('assignedBreakoutRoomOpenedPromptJoin', this._callIdRef.callId);
                this._context.deleteLatestNotification('assignedBreakoutRoomChanged', this._callIdRef.callId);
                this._context.deleteLatestNotification('breakoutRoomJoined', this._callIdRef.callId);
            }
            else if (breakoutRoom.state === 'closed') {
                // This scenario covers the case where the breakout room is closed
                this._context.deleteLatestNotification('assignedBreakoutRoomOpened', this._callIdRef.callId);
                this._context.deleteLatestNotification('assignedBreakoutRoomOpenedPromptJoin', this._callIdRef.callId);
                this._context.deleteLatestNotification('assignedBreakoutRoomChanged', this._callIdRef.callId);
                this._context.deleteLatestNotification('breakoutRoomJoined', this._callIdRef.callId);
                this._context.deleteLatestNotification('breakoutRoomClosingSoon', this._callIdRef.callId);
                clearTimeout(this._breakoutRoomClosingSoonTimeoutId);
                const openBreakoutRoomId = this._context.getOpenBreakoutRoom();
                if (openBreakoutRoomId && this._context.getState().calls[openBreakoutRoomId]) {
                    // Show notification that the assigned breakout room was closed if the user is in that breakout room.
                    this._context.setLatestNotification(this._callIdRef.callId, {
                        target: 'assignedBreakoutRoomClosed',
                        timestamp: new Date(Date.now())
                    });
                }
                this._context.deleteOpenBreakoutRoom();
            }
            this._context.setAssignedBreakoutRoom(this._callIdRef.callId, breakoutRoom);
        };
        this.onBreakoutRoomsJoined = (call) => {
            var _a, _b, _c;
            this._context.deleteLatestNotification('assignedBreakoutRoomOpened', this._callIdRef.callId);
            this._context.deleteLatestNotification('assignedBreakoutRoomOpenedPromptJoin', this._callIdRef.callId);
            this._context.deleteLatestNotification('assignedBreakoutRoomChanged', this._callIdRef.callId);
            // Send latest notification for breakoutRoomJoined on behalf of the call that was joined.
            this._context.setLatestNotification(call.id, {
                target: 'breakoutRoomJoined',
                timestamp: new Date(Date.now())
            });
            this._context.setOpenBreakoutRoom(call.id);
            // If assigned breakout room has a display name, set the display name for its call state.
            const assignedBreakoutRoomDisplayName = (_c = (_b = (_a = this._context.getState().calls[this._callIdRef.callId]) === null || _a === void 0 ? void 0 : _a.breakoutRooms) === null || _b === void 0 ? void 0 : _b.assignedBreakoutRoom) === null || _c === void 0 ? void 0 : _c.displayName;
            if (assignedBreakoutRoomDisplayName) {
                this._context.setBreakoutRoomDisplayName(call.id, assignedBreakoutRoomDisplayName);
            }
        };
        this.onBreakoutRoomSettingsUpdated = (breakoutRoomSettings) => {
            // If the roomEndTime is available, set a timeout to show a notification before the room closes.
            if (typeof breakoutRoomSettings.roomEndTime === 'string' && !Number.isNaN(Date.parse(breakoutRoomSettings.roomEndTime))) {
                const now = new Date(Date.now());
                const roomClosingTime = new Date(breakoutRoomSettings.roomEndTime).getTime();
                const timeBeforeClosing = roomClosingTime - now.getTime();
                const timeBeforeSendingClosingSoonNotification = Math.max(timeBeforeClosing - MILLSECONDS_BEFORE_END_TIME_TO_SHOW_CLOSING_NOTIFICATION, 0);
                if (!this._breakoutRoomClosingSoonTimeoutId) {
                    this._breakoutRoomClosingSoonTimeoutId = setTimeout(() => this._context.setLatestNotification(this._callIdRef.callId, {
                        target: 'breakoutRoomClosingSoon',
                        timestamp: now
                    }), timeBeforeSendingClosingSoonNotification);
                }
            }
            this._context.setBreakoutRoomSettings(this._callIdRef.callId, breakoutRoomSettings);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._breakoutRoomsFeature = breakoutRoomsFeature;
        this.subscribe();
    }
    subscribe() {
        this._breakoutRoomsFeature.on('breakoutRoomsUpdated', this.onBreakoutRoomsUpdated);
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class TogetherModeVideoStreamSubscriber {
    constructor(callIdRef, stream, context) {
        this.subscribe = () => {
            this._togetherModeStream.on('isAvailableChanged', this.isAvailableChanged);
            this._togetherModeStream.on('isReceivingChanged', this.isReceivingChanged);
            this._togetherModeStream.on('sizeChanged', this.isSizeChanged);
        };
        this.unsubscribe = () => {
            this._togetherModeStream.off('isAvailableChanged', this.isAvailableChanged);
            this._togetherModeStream.off('isReceivingChanged', this.isReceivingChanged);
            this._togetherModeStream.off('sizeChanged', this.isSizeChanged);
        };
        this.isAvailableChanged = () => {
            this._context.setTogetherModeVideoStreamIsAvailable(this._callIdRef.callId, this._togetherModeStream.id, this._togetherModeStream.isAvailable);
        };
        this.isReceivingChanged = () => {
            this._context.setTogetherModeVideoStreamIsReceiving(this._callIdRef.callId, this._togetherModeStream.id, this._togetherModeStream.isReceiving);
        };
        this.isSizeChanged = () => {
            this._context.setTogetherModeVideoStreamSize(this._callIdRef.callId, this._togetherModeStream.id, this._togetherModeStream.size);
        };
        this._callIdRef = callIdRef;
        this._togetherModeStream = stream;
        this._context = context;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * TogetherModeSubscriber is responsible for subscribing to together mode events and updating the call context accordingly.
 */
class TogetherModeSubscriber {
    constructor(callIdRef, context, internalContext, togetherMode) {
        this._featureName = 'togetherMode';
        this.subscribe = () => {
            this._togetherMode.on('togetherModeStreamsUpdated', this.onTogetherModeStreamUpdated);
            this._togetherMode.on('togetherModeSceneUpdated', this.onSceneUpdated);
            this._togetherMode.on('togetherModeSeatingUpdated', this.onSeatUpdated);
        };
        this.unsubscribe = () => {
            this._togetherMode.off('togetherModeStreamsUpdated', this.onTogetherModeStreamUpdated);
            this._togetherMode.off('togetherModeSceneUpdated', this.onSceneUpdated);
            this._togetherMode.off('togetherModeSeatingUpdated', this.onSeatUpdated);
        };
        this.onSceneUpdated = (args) => {
            this._context.setTogetherModeSeatingCoordinates(this._callIdRef.callId, args);
        };
        this.onSeatUpdated = (args) => {
            this._context.setTogetherModeSeatingCoordinates(this._callIdRef.callId, args);
        };
        this.addRemoteVideoStreamSubscriber = (togetherModeVideoStream) => {
            if (this._togetherModeVideoStreamSubscribers.has(togetherModeVideoStream.id)) {
                return;
            }
            this._togetherModeVideoStreamSubscribers.set(togetherModeVideoStream.id, new TogetherModeVideoStreamSubscriber(this._callIdRef, togetherModeVideoStream, this._context));
        };
        this.updateTogetherModeStreams = (addedStreams, removedStreams) => {
            removedStreams.forEach(stream => {
                var _a;
                (_a = this._togetherModeVideoStreamSubscribers.get(stream.id)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                this._togetherModeVideoStreamSubscribers.delete(stream.id);
                this._internalContext.deleteCallFeatureRenderInfo(this._callIdRef.callId, this._featureName, stream.mediaStreamType);
            });
            addedStreams.filter(stream => stream.isAvailable).forEach(stream => {
                this._internalContext.setCallFeatureRenderInfo(this._callIdRef.callId, this._featureName, stream.mediaStreamType, stream, 'NotRendered', undefined);
                this.addRemoteVideoStreamSubscriber(stream);
            });
            this._context.setTogetherModeVideoStreams(this._callIdRef.callId, addedStreams.filter(stream => stream.isAvailable).map(stream => convertSdkCallFeatureStreamToDeclarativeCallFeatureStream(stream, this._featureName)), removedStreams.map(stream => convertSdkCallFeatureStreamToDeclarativeCallFeatureStream(stream, this._featureName)));
            const notificationTarget = this._togetherMode.togetherModeStream.length ? 'togetherModeStarted' : 'togetherModeEnded';
            this._context.setLatestNotification(this._callIdRef.callId, {
                target: notificationTarget,
                timestamp: new Date()
            });
        };
        this.onTogetherModeStreamUpdated = (args) => {
            this.updateTogetherModeStreams(args.added, args.removed);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._internalContext = internalContext;
        this._togetherMode = togetherMode;
        this._togetherModeVideoStreamSubscribers = new Map();
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class MediaAccessSubscriber {
    constructor(callIdRef, context, mediaAccessCallFeature) {
        this.subscribe = () => {
            this._mediaAccessCallFeature.on('mediaAccessChanged', this.mediaAccessChanged);
            this._mediaAccessCallFeature.on('meetingMediaAccessChanged', this.meetingMediaAccessChanged);
        };
        this.unsubscribe = () => {
            this._mediaAccessCallFeature.off('mediaAccessChanged', this.mediaAccessChanged);
            this._mediaAccessCallFeature.off('meetingMediaAccessChanged', this.meetingMediaAccessChanged);
        };
        this.mediaAccessChanged = (data) => this._context.setMediaAccesses(this._callIdRef.callId, data.mediaAccesses);
        this.meetingMediaAccessChanged = (data) => this._context.setMeetingMediaAccess(this._callIdRef.callId, data.meetingMediaAccess);
        this._callIdRef = callIdRef;
        this._context = context;
        this._mediaAccessCallFeature = mediaAccessCallFeature;
        const mediaAccesses = this._mediaAccessCallFeature.getAllOthersMediaAccess();
        const meetingMediaAccess = this._mediaAccessCallFeature.getMeetingMediaAccess();
        this._context.setMediaAccesses(this._callIdRef.callId, mediaAccesses);
        this._context.setMeetingMediaAccess(this._callIdRef.callId, meetingMediaAccess);
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class RealTimeTextSubscriber {
    constructor(callIdRef, context, realTimeText) {
        this.subscribe = () => {
            this._realTimeTextFeature.on('realTimeTextReceived', this.realTimeTextReceived);
        };
        this.unsubscribe = () => {
            this._realTimeTextFeature.off('realTimeTextReceived', this.realTimeTextReceived);
        };
        this.realTimeTextReceived = (data) => {
            this._context.setIsRealTimeTextActive(this._callIdRef.callId, true);
            this._context.addRealTimeText(this._callIdRef.callId, data);
        };
        this._callIdRef = callIdRef;
        this._context = context;
        this._realTimeTextFeature = realTimeText;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Keeps track of the listeners assigned to a particular call because when we get an event from SDK, it doesn't tell us
 * which call it is for. If we keep track of this then we know which call in the state that needs an update and also
 * which property of that call. Also we can use this when unregistering to a call.
 */
class CallSubscriber {
    constructor(call, context, internalContext) {
        this._safeSubscribeInitCaptionSubscriber = () => {
            this._safeSubscribe(this.initCaptionSubscriber);
        };
        this._safeSubscribeInitTeamsMeetingConference = () => {
            this._safeSubscribe(this.initTeamsMeetingConference);
        };
        this._safeSubscribeInitRealTimeText = () => {
            this._safeSubscribe(this.initRealTimeText);
        };
        this.subscribe = () => {
            this._call.on('stateChanged', this.stateChanged);
            this._call.on('stateChanged', this._safeSubscribeInitCaptionSubscriber);
            this._call.on('stateChanged', this._safeSubscribeInitTeamsMeetingConference);
            this._call.on('idChanged', this.idChanged);
            this._call.on('isScreenSharingOnChanged', this.isScreenSharingOnChanged);
            this._call.on('remoteParticipantsUpdated', this.remoteParticipantsUpdated);
            this._call.on('localVideoStreamsUpdated', this.localVideoStreamsUpdated);
            this._call.on('isMutedChanged', this.isMuteChanged);
            this._call.on('roleChanged', this.callRoleChangedHandler);
            this._call.feature(communicationCalling.Features.DominantSpeakers).on('dominantSpeakersChanged', this.dominantSpeakersChanged);
            this._call.on('mutedByOthers', this.mutedByOthersHandler);
            this._safeSubscribeInitRealTimeText();
            for (const localVideoStream of this._call.localVideoStreams) {
                this._internalContext.setLocalRenderInfo(this._callIdRef.callId, localVideoStream.mediaStreamType, localVideoStream, 'NotRendered', undefined);
            }
            if (this._call.remoteParticipants.length > 0) {
                this._call.remoteParticipants.forEach((participant) => {
                    this.addParticipantListener(participant);
                });
                this._context.setCallRemoteParticipants(this._callIdRef.callId, this._call.remoteParticipants.map(convertSdkParticipantToDeclarativeParticipant), []);
            }
        };
        this.unsubscribe = () => {
            var _a, _b, _c;
            this._call.off('stateChanged', this.stateChanged);
            this._call.off('stateChanged', this._safeSubscribeInitCaptionSubscriber);
            this._call.off('stateChanged', this._safeSubscribeInitTeamsMeetingConference);
            this._call.off('idChanged', this.idChanged);
            this._call.off('isScreenSharingOnChanged', this.isScreenSharingOnChanged);
            this._call.off('remoteParticipantsUpdated', this.remoteParticipantsUpdated);
            this._call.off('localVideoStreamsUpdated', this.localVideoStreamsUpdated);
            this._call.off('isMutedChanged', this.isMuteChanged);
            this._call.off('roleChanged', this.callRoleChangedHandler);
            this._call.off('mutedByOthers', this.mutedByOthersHandler);
            this._participantSubscribers.forEach((participantSubscriber) => {
                participantSubscriber.unsubscribe();
            });
            this._participantSubscribers.clear();
            // If we are unsubscribing that means we no longer want to display any video for this call (callEnded or callAgent
            // disposed) and we should not be updating it any more. So if video is rendering we stop rendering.
            for (const localVideoStream of this._call.localVideoStreams) {
                const mediaStreamType = localVideoStream.mediaStreamType;
                disposeView$1(this._context, this._internalContext, this._callIdRef.callId, undefined, convertSdkLocalStreamToDeclarativeLocalStream(localVideoStream));
                this._internalContext.deleteLocalRenderInfo(this._callIdRef.callId, mediaStreamType);
            }
            this._diagnosticsSubscriber.unsubscribe();
            this._recordingSubscriber.unsubscribe();
            this._transcriptionSubscriber.unsubscribe();
            this._optimalVideoCountSubscriber.unsubscribe();
            this._pptLiveSubscriber.unsubscribe();
            (_a = this._CaptionsFeatureSubscriber) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            if (this._realTimeTextSubscriber) {
                this._realTimeTextSubscriber.unsubscribe();
            }
            (_b = this._raiseHandSubscriber) === null || _b === void 0 ? void 0 : _b.unsubscribe();
            this._capabilitiesSubscriber.unsubscribe();
            (_c = this._reactionSubscriber) === null || _c === void 0 ? void 0 : _c.unsubscribe();
            this._spotlightSubscriber.unsubscribe();
            this._breakoutRoomsSubscriber.unsubscribe();
            this._togetherModeSubscriber.unsubscribe();
            this._mediaAccessSubscriber.unsubscribe();
        };
        this.stateChanged = () => {
            this._context.setCallState(this._callIdRef.callId, this._call.state);
        };
        this.initCaptionSubscriber = () => {
            // subscribe to captions here so that we don't call captions when call is not initialized
            if (this._call.state === 'Connected' && !this._CaptionsFeatureSubscriber) {
                this._CaptionsFeatureSubscriber = new CaptionsFeatureSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.Captions));
                this._call.off('stateChanged', this._safeSubscribeInitCaptionSubscriber);
            }
        };
        this.initTeamsMeetingConference = () => {
            if (this._call.state === 'Connected' && _isTeamsMeeting(this._call)) {
                this._call.feature(communicationCalling.Features.TeamsMeetingAudioConferencing).getTeamsMeetingAudioConferencingDetails().then(teamsMeetingConferenceDetails => {
                    this._context.setTeamsMeetingConference(this._callIdRef.callId, teamsMeetingConferenceDetails);
                });
                this._call.off('stateChanged', this._safeSubscribeInitTeamsMeetingConference);
            }
        };
        this.initRealTimeText = () => {
            if (this._context.getState().userId.kind !== 'microsoftTeamsUser') {
                this._realTimeTextSubscriber = new RealTimeTextSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.RealTimeText));
            }
        };
        this.idChanged = () => {
            this._internalContext.setCallId(this._call.id, this._callIdRef.callId);
            this._context.setCallId(this._call.id, this._callIdRef.callId);
            this._callIdRef.callId = this._call.id;
        };
        this.isScreenSharingOnChanged = () => {
            this._context.setCallIsScreenSharingOn(this._callIdRef.callId, this._call.isScreenSharingOn);
        };
        this.isMuteChanged = () => {
            this._context.setCallIsMicrophoneMuted(this._callIdRef.callId, this._call.isMuted);
        };
        this.callRoleChangedHandler = () => {
            this._context.setRole(this._callIdRef.callId, this._call.role);
        };
        // TODO: Tee to notification state once available
        this.mutedByOthersHandler = () => {
            this._context.teeErrorToState({
                name: 'mutedByOthers',
                message: 'Muted by another participant'
            }, 'Call.mutedByOthers');
        };
        this.remoteParticipantsUpdated = (event) => {
            event.added.forEach((participant) => {
                this.addParticipantListener(participant);
            });
            event.removed.forEach((participant) => {
                this.removeParticipantListener(participant);
            });
            // Remove any added participants from remoteParticipantsEnded if they are there and add any removed participants to
            // remoteParticipantsEnded.
            this._context.setCallRemoteParticipantsEnded(this._callIdRef.callId, event.removed.map(convertSdkParticipantToDeclarativeParticipant), event.added.map((participant) => {
                return toFlatCommunicationIdentifier(participant.identifier);
            }));
            // Add added participants to remoteParticipants and remove removed participants from remoteParticipants.
            this._context.setCallRemoteParticipants(this._callIdRef.callId, event.added.map(convertSdkParticipantToDeclarativeParticipant), event.removed.map((participant) => {
                return toFlatCommunicationIdentifier(participant.identifier);
            }));
        };
        this.localVideoStreamsUpdated = (event) => {
            var _a, _b;
            for (const localVideoStream of event.added) {
                const mediaStreamType = localVideoStream.mediaStreamType;
                // IMPORTANT: The internalContext should be set before context. This is done to ensure that the internal context
                // has the required data when component re-renders due to external state changes.
                this._internalContext.setLocalRenderInfo(this._callIdRef.callId, mediaStreamType, localVideoStream, 'NotRendered', undefined);
                // Subscribe to video effect changes
                (_a = this._localVideoStreamVideoEffectsSubscribers.get(mediaStreamType)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                this._localVideoStreamVideoEffectsSubscribers.set(mediaStreamType, new LocalVideoStreamVideoEffectsSubscriber({
                    parent: this._callIdRef,
                    context: this._context,
                    localVideoStream: localVideoStream,
                    localVideoStreamEffectsAPI: localVideoStream.feature(communicationCalling.Features.VideoEffects)
                }));
            }
            for (const localVideoStream of event.removed) {
                const mediaStreamType = localVideoStream.mediaStreamType;
                (_b = this._localVideoStreamVideoEffectsSubscribers.get(mediaStreamType)) === null || _b === void 0 ? void 0 : _b.unsubscribe();
                disposeView$1(this._context, this._internalContext, this._callIdRef.callId, undefined, convertSdkLocalStreamToDeclarativeLocalStream(localVideoStream));
                this._internalContext.deleteLocalRenderInfo(this._callIdRef.callId, mediaStreamType);
            }
            this._context.setCallLocalVideoStream(this._callIdRef.callId, event.added.map(convertSdkLocalStreamToDeclarativeLocalStream), event.removed.map(convertSdkLocalStreamToDeclarativeLocalStream));
        };
        this.dominantSpeakersChanged = () => {
            const dominantSpeakers = this._call.feature(communicationCalling.Features.DominantSpeakers).dominantSpeakers;
            this._context.setCallDominantSpeakers(this._callIdRef.callId, dominantSpeakers);
        };
        this._call = call;
        this._callIdRef = {
            callId: call.id
        };
        this._context = context;
        this._internalContext = internalContext;
        this._diagnosticsSubscriber = new UserFacingDiagnosticsSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.UserFacingDiagnostics));
        this._participantSubscribers = new Map();
        this._recordingSubscriber = new RecordingSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.Recording));
        this._pptLiveSubscriber = new PPTLiveSubscriber(this._callIdRef, this._context, this._call);
        this._transcriptionSubscriber = new TranscriptionSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.Transcription));
        this._raiseHandSubscriber = new RaiseHandSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.RaiseHand));
        this._reactionSubscriber = new ReactionSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.Reaction));
        this._optimalVideoCountSubscriber = new OptimalVideoCountSubscriber({
            callIdRef: this._callIdRef,
            context: this._context,
            localOptimalVideoCountFeature: this._call.feature(communicationCalling.Features.OptimalVideoCount)
        });
        this._localVideoStreamVideoEffectsSubscribers = new Map();
        this._capabilitiesSubscriber = new CapabilitiesSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.Capabilities));
        this._spotlightSubscriber = new SpotlightSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.Spotlight));
        // Clear assigned breakout room closed notification for this call.
        this._context.deleteLatestNotification('assignedBreakoutRoomClosed');
        this._breakoutRoomsSubscriber = new BreakoutRoomsSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.BreakoutRooms));
        this._togetherModeSubscriber = new TogetherModeSubscriber(this._callIdRef, this._context, this._internalContext, this._call.feature(communicationCalling.Features.TogetherMode));
        this._mediaAccessSubscriber = new MediaAccessSubscriber(this._callIdRef, this._context, this._call.feature(communicationCalling.Features.MediaAccess));
        this.subscribe();
    }
    // This is a helper function to safely call subscriber functions. This is needed in order to prevent events
    // with the same event type from failing to fire due to a subscriber throwing an error upon subscription.
    // Wrap your listeners with this helper function if your listener can fail due to initialization or fail
    // during a function call. This will prevent other events using the same event type from failing to fire.
    _safeSubscribe(subscriber) {
        try {
            subscriber();
        }
        catch (e) {
            this._context.teeErrorToState(e, 'Call.on');
        }
    }
    addParticipantListener(participant) {
        var _a;
        const participantKey = toFlatCommunicationIdentifier(participant.identifier);
        (_a = this._participantSubscribers.get(participantKey)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._participantSubscribers.set(participantKey, new ParticipantSubscriber$1(this._callIdRef, participant, this._context, this._internalContext));
    }
    removeParticipantListener(participant) {
        const participantKey = toFlatCommunicationIdentifier(participant.identifier);
        const participantSubscriber = this._participantSubscribers.get(participantKey);
        if (participantSubscriber) {
            participantSubscriber.unsubscribe();
            this._participantSubscribers.delete(participantKey);
        }
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Keeps track of the listeners assigned to a particular incoming call because when we get an event from SDK, it doesn't
 * tell us which incoming call it is for. If we keep track of this then we know which incoming call in the state that
 * needs an update and also which property of that incoming call. Also we can use this when unregistering to a incoming
 * call.
 */
class IncomingCallSubscriber {
    constructor(incomingCall, 
    // setIncomingCallEnded callback is used so parent can clean up IncomingCallSubscriber.
    setIncomingCallEnded) {
        this.subscribe = () => {
            this._incomingCall.on('callEnded', this.callEnded);
        };
        this.unsubscribe = () => {
            this._incomingCall.off('callEnded', this.callEnded);
        };
        this.callEnded = (event) => {
            this._setIncomingCallEnded(this._incomingCall.id, event.callEndReason);
        };
        this._incomingCall = incomingCall;
        this._setIncomingCallEnded = setIncomingCallEnded;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$T = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
class ProxyIncomingCall {
    constructor(context) {
        this._context = context;
    }
    get(target, prop) {
        switch (prop) {
            case 'accept':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$T(this, void 0, void 0, function* () {
                            return yield target.accept(...args);
                        });
                    }, 'IncomingCall.accept');
                }
            case 'reject':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$T(this, void 0, void 0, function* () {
                            return yield target.reject(...args);
                        });
                    }, 'IncomingCall.reject');
                }
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * Creates a declarative Incoming Call by proxying IncomingCall using ProxyIncomingCall.
 * @param incomingCall - IncomingCall from SDK
 * @returns proxied IncomingCall
 */
const incomingCallDeclaratify = (incomingCall, context) => {
    const proxyIncomingCall = new ProxyIncomingCall(context);
    return new Proxy(incomingCall, proxyIncomingCall);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$S = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
class ProxyTeamsIncomingCall {
    constructor(context) {
        this._context = context;
    }
    get(target, prop) {
        switch (prop) {
            case 'accept':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$S(this, void 0, void 0, function* () {
                            return yield target.accept(...args);
                        });
                    }, 'IncomingCall.accept');
                }
            case 'reject':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$S(this, void 0, void 0, function* () {
                            return yield target.reject(...args);
                        });
                    }, 'IncomingCall.reject');
                }
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * Creates a declarative TeamsIncomingCall by proxying TeamsIncomingCall using ProxyIncomingCall.
 * @param incomingCall - TeamsIncomingCall from SDK
 * @returns proxied TeamsIncomingCall
 */
const teamsIncomingCallDeclaratify = (incomingCall, context) => {
    const proxyIncomingCall = new ProxyTeamsIncomingCall(context);
    return new Proxy(incomingCall, proxyIncomingCall);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$R = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * ProxyCallAgent proxies CallAgent and saves any returned state in the given context. It will subscribe to all state
 * updates in the CallAgent and in the contained Calls and RemoteParticipants. When dispose is called it will
 * unsubscribe from all state updates.
 */
class ProxyCallAgentCommon {
    constructor(context, internalContext) {
        // Unsubscribe is called when CallAgent is disposed. This should mean no more updating of existing call but we don't
        // remove any existing state.
        this.unregisterSubscriber = () => {
            for (const [_, callSubscriber] of this._callSubscribers.entries()) {
                callSubscriber.unsubscribe();
            }
            this._callSubscribers.clear();
            for (const [_, incomingCallSubscriber] of this._incomingCallSubscribers.entries()) {
                incomingCallSubscriber.unsubscribe();
            }
            this._incomingCallSubscribers.clear();
            this._incomingCalls.clear();
            for (const [_, declarativeCall] of this._declarativeCalls.entries()) {
                declarativeCall.unsubscribe();
            }
            this._declarativeCalls.clear();
        };
        this.callsUpdated = (event) => {
            const addedStatefulCall = [];
            for (const call of event.added) {
                const statefulCall = this.addCall(call);
                addedStatefulCall.push(statefulCall);
            }
            const removedStatefulCall = [];
            for (const call of event.removed) {
                disposeAllViewsFromCall(this._context, this._internalContext, call.id);
                const callSubscriber = this._callSubscribers.get(call);
                if (callSubscriber) {
                    callSubscriber.unsubscribe();
                    this._callSubscribers.delete(call);
                }
                this._context.setCallEnded(call.id, call.callEndReason);
                const declarativeCall = this._declarativeCalls.get(call);
                if (declarativeCall) {
                    declarativeCall.unsubscribe();
                    removedStatefulCall.push(declarativeCall);
                    this._declarativeCalls.delete(call);
                }
                else {
                    removedStatefulCall.push(this.callDeclaratify(call, this._context));
                }
            }
            for (const externalCallsUpdatedListener of this._externalCallsUpdatedListeners) {
                externalCallsUpdatedListener({
                    added: addedStatefulCall,
                    removed: removedStatefulCall
                });
            }
        };
        this.setIncomingCallEnded = (incomingCallId, callEndReason) => {
            const incomingCallSubscriber = this._incomingCallSubscribers.get(incomingCallId);
            if (incomingCallSubscriber) {
                incomingCallSubscriber.unsubscribe();
                this._incomingCallSubscribers.delete(incomingCallId);
            }
            this._incomingCalls.delete(incomingCallId);
            this._context.setIncomingCallEnded(incomingCallId, callEndReason);
        };
        this.incomingCall = ({ incomingCall }) => {
            // Make sure to not subscribe to the incoming call if we are already subscribed to it.
            if (!this._incomingCallSubscribers.has(incomingCall.id)) {
                this._incomingCallSubscribers.set(incomingCall.id, new IncomingCallSubscriber(incomingCall, this.setIncomingCallEnded));
            }
            if (_isTeamsIncomingCall(incomingCall)) {
                this._incomingCalls.set(incomingCall.id, teamsIncomingCallDeclaratify(incomingCall, this._context));
            }
            else {
                this._incomingCalls.set(incomingCall.id, incomingCallDeclaratify(incomingCall, this._context));
            }
            this._context.setIncomingCall(convertSdkIncomingCallToDeclarativeIncomingCall(incomingCall));
        };
        this.addCall = (call) => {
            var _a;
            (_a = this._callSubscribers.get(call)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            // For API extentions we need to have the call in the state when we are subscribing as we may want to update the
            // state during the subscription process in the subscriber so we add the call to state before subscribing.
            this._context.setCall(convertSdkCallToDeclarativeCall(call));
            this._callSubscribers.set(call, new CallSubscriber(call, this._context, this._internalContext));
            return this.getOrCreateDeclarativeCall(call);
        };
        this.getOrCreateDeclarativeCall = (call) => {
            const declarativeCall = this._declarativeCalls.get(call);
            if (declarativeCall) {
                return declarativeCall;
            }
            const newDeclarativeCall = this.callDeclaratify(call, this._context);
            this._declarativeCalls.set(call, newDeclarativeCall);
            return newDeclarativeCall;
        };
        this._context = context;
        this._internalContext = internalContext;
        this._callSubscribers = new Map();
        this._incomingCallSubscribers = new Map();
        this._incomingCalls = new Map();
        this._declarativeCalls = new Map();
        this._externalCallsUpdatedListeners = new Set();
    }
    // We can't directly override get function because it is proxied,
    // Add a getCommon function and call it in child class
    getCommon(target, prop) {
        switch (prop) {
            case 'startCall':
                {
                    return this._context.withErrorTeedToState((...args) => {
                        const call = this.startCall(target, args);
                        this.addCall(call);
                        return this.getOrCreateDeclarativeCall(call);
                    }, 'CallAgent.startCall');
                }
            case 'join':
                {
                    return this._context.withErrorTeedToState((...args) => {
                        const call = this.joinCall(target, args);
                        this.addCall(call);
                        return this.getOrCreateDeclarativeCall(call);
                    }, 'CallAgent.join');
                }
            case 'calls':
                {
                    return Array.from(this._declarativeCalls.values());
                }
            case 'on':
                {
                    return (...args) => {
                        // typescript is not smart enough to handle multiple overloads and pull the correct type here so force casting args
                        const event = args[0];
                        const isCallsUpdated = event === 'callsUpdated';
                        if (isCallsUpdated) {
                            const listener = args[1];
                            this._externalCallsUpdatedListeners.add(listener);
                        }
                        else {
                            this.agentSubscribe(target, args);
                        }
                    };
                }
            case 'off':
                {
                    return (...args) => {
                        // typescript is not smart enough to handle multiple overloads and pull the correct type here so force casting args
                        const event = args[0];
                        const isCallsUpdated = event === 'callsUpdated';
                        if (isCallsUpdated) {
                            const listener = args[1];
                            this._externalCallsUpdatedListeners.delete(listener);
                        }
                        else {
                            this.agentUnsubscribe(target, args);
                        }
                    };
                }
            case 'dispose':
                {
                    // Wrapping CallAgent.dispose in a callback type (): Promise<void> to accomodate the change of CallAgent.dispose
                    // in calling beta version 1.8.0-beta.1 from callback type (): Promise<void> to (): void
                    const callAgentDisposeAsyncCallbackWrapper = () => __awaiter$R(this, void 0, void 0, function* () {
                        yield target.dispose();
                        return Promise.resolve();
                    });
                    return () => {
                        return callAgentDisposeAsyncCallbackWrapper().then(() => {
                            this.unsubscribe();
                        });
                    };
                }
            /**
             * This attribute is a special case and doesn't exist on the CallAgent interface.
             * We need this to be able to return a declarative incoming call object using the call agent.
             * In a standard headless SDK usage, the right way to get an incoming call is to use the `incomingCall` event.
             * However, using the declarative layer, the ideal usage would be to:
             * 1. subscribe to the `onStateChange` event
             * 2. Get the incoming call from the new state and it's ID
             * 3. Use `callAgent.incomingCalls` and filter an incoming call ID to get a declarative incoming call object
             */
            case 'incomingCalls':
                {
                    return Array.from(this._incomingCalls.values());
                }
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * @private
 */
const clearCallRelatedState = (context, internalContext) => {
    // Make sure there are no existing call data if creating a new CallAgentDeclarative (if creating a new
    // CallAgentDeclarative after disposing of the hold one will mean context have old call state). TODO: should we stop
    // rendering when the previous callAgent is disposed?
    disposeAllViews(context, internalContext);
    context.clearCallRelatedState();
    internalContext.clearCallRelatedState();
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$Q = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
class ProxyCallCommon {
    constructor(context) {
        this._context = context;
    }
    unsubscribe() {
        /** No subscriptions yet. But there will be one for transfer feature soon. */
    }
    getContext() {
        return this._context;
    }
    get(target, prop) {
        switch (prop) {
            case 'mute':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.mute(...args);
                        });
                    }, 'Call.mute');
                }
            case 'unmute':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.unmute(...args);
                        });
                    }, 'Call.unmute');
                }
            case 'startVideo':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.startVideo(...args);
                        });
                    }, 'Call.startVideo');
                }
            case 'stopVideo':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.stopVideo(...args);
                        });
                    }, 'Call.stopVideo');
                }
            case 'startScreenSharing':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.startScreenSharing(...args);
                        });
                    }, 'Call.startScreenSharing');
                }
            case 'stopScreenSharing':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.stopScreenSharing(...args);
                        });
                    }, 'Call.stopScreenSharing');
                }
            case 'hold':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.hold(...args);
                        });
                    }, 'Call.hold');
                }
            case 'resume':
                {
                    return this._context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$Q(this, void 0, void 0, function* () {
                            return yield target.resume(...args);
                        });
                    }, 'Call.resume');
                }
            case 'feature':
                {
                    // these are mini version of Proxy object - if it grows too big, a real Proxy object should be used.
                    return this._context.withErrorTeedToState((...args) => {
                        if (args[0] === communicationCalling.Features.Captions) {
                            const captionsFeature = target.feature(communicationCalling.Features.Captions);
                            let proxyFeature;
                            if (captionsFeature.captions.kind === 'Captions') {
                                proxyFeature = new ProxyCaptions(this._context, target);
                                return {
                                    captions: new Proxy(captionsFeature.captions, proxyFeature),
                                    on: (...args) => {
                                        const isCaptionsKindChanged = args[0] === 'CaptionsKindChanged';
                                        if (isCaptionsKindChanged) {
                                            const listener = args[1];
                                            const newListener = () => {
                                                listener();
                                            };
                                            return captionsFeature.on('CaptionsKindChanged', newListener);
                                        }
                                    },
                                    off: (...args) => {
                                        const isCaptionsKindChanged = args[0] === 'CaptionsKindChanged';
                                        if (isCaptionsKindChanged) {
                                            return captionsFeature.off('CaptionsKindChanged', args[1]);
                                        }
                                    }
                                };
                            }
                            proxyFeature = new ProxyTeamsCaptions(this._context, target);
                            return {
                                captions: new Proxy(captionsFeature.captions, proxyFeature)
                            };
                        }
                        if (args[0] === communicationCalling.Features.Transfer) {
                            const transferFeature = target.feature(communicationCalling.Features.Transfer);
                            const proxyFeature = new ProxyTransferCallFeature(this._context, target);
                            return new Proxy(transferFeature, proxyFeature);
                        }
                        if (args[0] === communicationCalling.Features.Spotlight) {
                            const spotlightFeature = target.feature(communicationCalling.Features.Spotlight);
                            const proxyFeature = new ProxySpotlightCallFeature(this._context);
                            return new Proxy(spotlightFeature, proxyFeature);
                        }
                        if (args[0] === communicationCalling.Features.RealTimeText && this._context.getState().userId.kind === 'microsoftTeamsUser') {
                            return;
                        }
                        return target.feature(...args);
                    }, 'Call.feature');
                }
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * @private
 */
class ProxyTeamsCaptions {
    constructor(context, call) {
        this._context = context;
        this._call = call;
    }
    get(target, prop) {
        switch (prop) {
            case 'startCaptions':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    var _a, _b;
                    this._context.setStartCaptionsInProgress(this._call.id, true);
                    try {
                        const ret = yield target.startCaptions(...args);
                        this._context.setSelectedSpokenLanguage(this._call.id, (_b = (_a = args[0]) === null || _a === void 0 ? void 0 : _a.spokenLanguage) !== null && _b !== void 0 ? _b : 'en-us');
                        return ret;
                    }
                    catch (e) {
                        this._context.setStartCaptionsInProgress(this._call.id, false);
                        throw e;
                    }
                }), 'Call.feature');
            case 'stopCaptions':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    const ret = yield target.stopCaptions(...args);
                    this._context.setIsCaptionActive(this._call.id, false);
                    this._context.setStartCaptionsInProgress(this._call.id, false);
                    this._context.clearCaptions(this._call.id);
                    return ret;
                }), 'Call.feature');
            case 'setSpokenLanguage':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    const ret = yield target.setSpokenLanguage(...args);
                    this._context.setSelectedSpokenLanguage(this._call.id, args[0]);
                    return ret;
                }), 'Call.feature');
            case 'setCaptionLanguage':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    const ret = yield target.setCaptionLanguage(...args);
                    this._context.setSelectedCaptionLanguage(this._call.id, args[0]);
                    return ret;
                }), 'Call.feature');
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * @private
 */
class ProxyCaptions {
    constructor(context, call) {
        this._context = context;
        this._call = call;
    }
    get(target, prop) {
        switch (prop) {
            case 'startCaptions':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    var _a, _b;
                    this._context.setStartCaptionsInProgress(this._call.id, true);
                    try {
                        const ret = yield target.startCaptions(...args);
                        this._context.setSelectedSpokenLanguage(this._call.id, (_b = (_a = args[0]) === null || _a === void 0 ? void 0 : _a.spokenLanguage) !== null && _b !== void 0 ? _b : 'en-us');
                        return ret;
                    }
                    catch (e) {
                        this._context.setStartCaptionsInProgress(this._call.id, false);
                        throw e;
                    }
                }), 'Call.feature');
            case 'stopCaptions':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    const ret = yield target.stopCaptions(...args);
                    this._context.setIsCaptionActive(this._call.id, false);
                    this._context.setStartCaptionsInProgress(this._call.id, false);
                    this._context.clearCaptions(this._call.id);
                    return ret;
                }), 'Call.feature');
            case 'setSpokenLanguage':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    const ret = yield target.setSpokenLanguage(...args);
                    this._context.setSelectedSpokenLanguage(this._call.id, args[0]);
                    return ret;
                }), 'Call.feature');
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * @private
 */
class ProxySpotlightCallFeature {
    constructor(context) {
        this._context = context;
    }
    get(target, prop) {
        switch (prop) {
            case 'startSpotlight':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    const ret = yield target.startSpotlight(...args);
                    return ret;
                }), 'Call.feature');
            case 'stopSpotlight':
                return this._context.withAsyncErrorTeedToState((...args) => __awaiter$Q(this, void 0, void 0, function* () {
                    const ret = yield target.stopSpotlight(...args);
                    return ret;
                }), 'Call.feature');
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * @private
 */
class ProxyTransferCallFeature {
    constructor(context, call) {
        this._context = context;
        this._call = call;
    }
    get(target, prop) {
        switch (prop) {
            case 'on':
                return (...args) => {
                    const isTransferAccepted = args[0] === 'transferAccepted';
                    if (isTransferAccepted) {
                        const listener = args[1];
                        const newListener = (args) => {
                            this._context.setAcceptedTransfer(this._call.id, {
                                callId: args.targetCall.id,
                                timestamp: new Date()
                            });
                            listener(args);
                        };
                        return target.on('transferAccepted', newListener);
                    }
                };
            default:
                return Reflect.get(target, prop);
        }
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$P = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
class ProxyCall extends ProxyCallCommon {
    get(target, prop) {
        switch (prop) {
            case 'addParticipant':
                {
                    return this.getContext().withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$P(this, void 0, void 0, function* () {
                            return yield target.addParticipant(...args);
                        });
                    }, 'Call.addParticipant');
                }
            default:
                return super.get(target, prop);
        }
    }
}
/**
 * Creates a declarative Call by proxying Call with ProxyCall.
 * This should only be used with CallAgentDeclarative as CallAgentDeclarative will add that
 * call to the context properly (need to have the Call in context to update it - CallAgentDeclarative will add Call to
 * context)
 *
 * @param call - Call from SDK
 * @param context - CallContext from StatefulCallClient
 */
const callDeclaratify = (call, context) => {
    const proxyCall = new ProxyCall(context);
    Object.defineProperty(call, 'unsubscribe', {
        configurable: false,
        value: () => proxyCall.unsubscribe()
    });
    return new Proxy(call, proxyCall);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * ProxyCallAgent proxies CallAgent and saves any returned state in the given context. It will subscribe to all state
 * updates in the CallAgent and in the contained Calls and RemoteParticipants. When dispose is called it will
 * unsubscribe from all state updates.
 */
class ProxyCallAgent extends ProxyCallAgentCommon {
    constructor(callAgent, context, internalContext) {
        super(context, internalContext);
        this.subscribe = () => {
            this._callAgent.on('callsUpdated', this.callsUpdated);
            this._callAgent.on('incomingCall', this.incomingCall);
            // There could be scenario that when ProxyCallAgent is created that the given CallAgent already has Calls. In this
            // case we need to make sure to subscribe to those already existing Calls.
            for (const call of this._callAgent.calls) {
                this.addCall(call);
            }
        };
        this.unsubscribe = () => {
            this._callAgent.off('callsUpdated', this.callsUpdated);
            this._callAgent.off('incomingCall', this.incomingCall);
            this.unregisterSubscriber();
        };
        this._callAgent = callAgent;
        this.subscribe();
    }
    callDeclaratify(call, context) {
        if (_isACSCall(call)) {
            return callDeclaratify(call, context);
        }
        throw new Error('Not reachable code, DeclarativeCallAgent.callDeclaratify must be called with an ACS call.');
    }
    startCall(agent, args) {
        if (_isACSCallAgent(agent)) {
            return agent.startCall(...args);
        }
        throw Error('Unreachable code, DeclarativeCallAgent.startCall must be called with an ACS callAgent.');
    }
    joinCall(agent, args) {
        if (_isACSCallAgent(agent)) {
            return agent.join(...args);
        }
        throw Error('Unreachable code, DeclarativeCallAgent.joinCall must be called with an ACS callAgent.');
    }
    agentSubscribe(agent, args) {
        if (_isACSCallAgent(agent)) {
            return agent.on(...args);
        }
        throw Error('Unreachable code, DeclarativeCallAgent.agentSubscribe must be called with an ACS callAgent.');
    }
    agentUnsubscribe(agent, args) {
        if (_isACSCallAgent(agent)) {
            return agent.off(...args);
        }
        throw Error('Unreachable code, DeclarativeCallAgent.agentUnsubscribe must be called with an ACS callAgent.');
    }
    get(target, prop) {
        return super.getCommon(target, prop);
    }
}
/**
 * Creates a declarative CallAgent by proxying CallAgent with ProxyCallAgent which will track state updates by updating
 * the given context.
 *
 * @param callAgent - CallAgent from SDK
 * @param context - CallContext from StatefulCallClient
 * @param internalContext- InternalCallContext from StatefulCallClient
 */
const callAgentDeclaratify = (callAgent, context, internalContext) => {
    clearCallRelatedState(context, internalContext);
    return new Proxy(callAgent, new ProxyCallAgent(callAgent, context, internalContext));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Contains internal data used between different Declarative components to share data.
 */
class InternalCallContext {
    constructor() {
        // <CallId, <ParticipantKey, <StreamId, RemoteRenderInfo>>
        this._remoteRenderInfos = new Map();
        // <CallId, <MediaStreamType, LocalRenderInfo>>.
        this._localRenderInfos = new Map();
        // <CallId, <featureName, <MediaStreamType, CallFeatureRenderInfo>>>.
        this._callFeatureRenderInfos = new Map();
        // Used for keeping track of rendered LocalVideoStreams that are not part of a Call.
        this._unparentedRenderInfos = new Map();
        this._callIdHistory = new CallIdHistory();
        // Used for keeping track of video effects subscribers that are not part of a Call.
        // The key is the stream ID. We assume each stream ID
        this._unparentedViewVideoEffectsSubscriber = new Map();
    }
    setCallId(newCallId, oldCallId) {
        this._callIdHistory.updateCallIdHistory(newCallId, oldCallId);
        const remoteRenderInfos = this._remoteRenderInfos.get(oldCallId);
        if (remoteRenderInfos) {
            this._remoteRenderInfos.delete(oldCallId);
            this._remoteRenderInfos.set(newCallId, remoteRenderInfos);
        }
        const localRenderInfos = this._localRenderInfos.get(oldCallId);
        if (localRenderInfos) {
            this._localRenderInfos.delete(oldCallId);
            this._localRenderInfos.set(newCallId, localRenderInfos);
        }
        const callFeatureRenderInfos = this._callFeatureRenderInfos.get(oldCallId);
        if (callFeatureRenderInfos) {
            this._callFeatureRenderInfos.delete(oldCallId);
            this._callFeatureRenderInfos.set(newCallId, callFeatureRenderInfos);
        }
    }
    getCallIds() {
        return this._remoteRenderInfos.keys();
    }
    getRemoteRenderInfoForCall(callId) {
        return this._remoteRenderInfos.get(this._callIdHistory.latestCallId(callId));
    }
    getRemoteRenderInfoForParticipant(callId, participantKey, streamId) {
        const callRenderInfos = this._remoteRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!callRenderInfos) {
            return undefined;
        }
        const participantRenderInfos = callRenderInfos.get(participantKey);
        if (!participantRenderInfos) {
            return undefined;
        }
        return participantRenderInfos.get(streamId);
    }
    setRemoteRenderInfo(callId, participantKey, streamId, stream, status, renderer) {
        let callRenderInfos = this._remoteRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!callRenderInfos) {
            callRenderInfos = new Map();
            this._remoteRenderInfos.set(this._callIdHistory.latestCallId(callId), callRenderInfos);
        }
        let participantRenderInfos = callRenderInfos.get(participantKey);
        if (!participantRenderInfos) {
            participantRenderInfos = new Map();
            callRenderInfos.set(participantKey, participantRenderInfos);
        }
        participantRenderInfos.set(streamId, {
            stream,
            status,
            renderer
        });
    }
    deleteRemoteRenderInfo(callId, participantKey, streamId) {
        const callRenderInfos = this._remoteRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!callRenderInfos) {
            return;
        }
        const participantRenderInfos = callRenderInfos.get(participantKey);
        if (!participantRenderInfos) {
            return;
        }
        participantRenderInfos.delete(streamId);
    }
    setLocalRenderInfo(callId, streamKey, stream, status, renderer) {
        let localRenderInfosForCall = this._localRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!localRenderInfosForCall) {
            localRenderInfosForCall = new Map();
            this._localRenderInfos.set(this._callIdHistory.latestCallId(callId), localRenderInfosForCall);
        }
        localRenderInfosForCall.set(streamKey, {
            stream,
            status,
            renderer
        });
    }
    getLocalRenderInfosForCall(callId) {
        return this._localRenderInfos.get(this._callIdHistory.latestCallId(callId));
    }
    getLocalRenderInfo(callId, streamKey) {
        const localRenderInfosForCall = this._localRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!localRenderInfosForCall) {
            return undefined;
        }
        return localRenderInfosForCall.get(streamKey);
    }
    deleteLocalRenderInfo(callId, streamKey) {
        const localRenderInfoForCall = this._localRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!localRenderInfoForCall) {
            return;
        }
        localRenderInfoForCall.delete(streamKey);
    }
    getUnparentedRenderInfo(localVideoStream) {
        return this._unparentedRenderInfos.get(localVideoStream.mediaStreamType);
    }
    getUnparentedRenderInfos() {
        return [...this._unparentedRenderInfos].map(([, renderInfo]) => renderInfo.stream);
    }
    setUnparentedRenderInfo(statefulStream, stream, status, renderer) {
        this._unparentedRenderInfos.set(statefulStream.mediaStreamType, {
            stream,
            status,
            renderer
        });
    }
    deleteUnparentedRenderInfo(localVideoStream) {
        var _a;
        (_a = this._unparentedViewVideoEffectsSubscriber.get(localVideoStream.mediaStreamType)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._unparentedRenderInfos.delete(localVideoStream.mediaStreamType);
    }
    subscribeToUnparentedViewVideoEffects(localVideoStream, callContext) {
        var _a;
        {
            // Ensure we aren't setting multiple subscriptions
            (_a = this._unparentedViewVideoEffectsSubscriber.get(localVideoStream.mediaStreamType)) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            this._unparentedViewVideoEffectsSubscriber.set(localVideoStream.mediaStreamType, new LocalVideoStreamVideoEffectsSubscriber({
                parent: 'unparented',
                context: callContext,
                localVideoStream: localVideoStream,
                localVideoStreamEffectsAPI: localVideoStream.feature(communicationCalling.Features.VideoEffects)
            }));
        }
    }
    // UnparentedRenderInfos are not cleared as they are not part of the Call state.
    clearCallRelatedState() {
        this._remoteRenderInfos.clear();
        this._localRenderInfos.clear();
        this._callFeatureRenderInfos.clear();
    }
    getCallFeatureRenderInfosForCall(callId) {
        return this._callFeatureRenderInfos.get(this._callIdHistory.latestCallId(callId));
    }
    getCallFeatureRenderInfo(callId, featureNameKey, streamKey) {
        var _a, _b;
        const callFeatureRenderInfosForCall = (_b = (_a = this._callFeatureRenderInfos.get(this._callIdHistory.latestCallId(callId))) === null || _a === void 0 ? void 0 : _a.get(featureNameKey)) === null || _b === void 0 ? void 0 : _b.get(streamKey);
        if (!callFeatureRenderInfosForCall) {
            return undefined;
        }
        return callFeatureRenderInfosForCall;
    }
    setCallFeatureRenderInfo(callId, featureNameKey, streamKey, stream, status, renderer) {
        let callRenderInfos = this._callFeatureRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!callRenderInfos) {
            callRenderInfos = new Map();
            // If the callId is not found, create a new map for the callId.
            this._callFeatureRenderInfos.set(this._callIdHistory.latestCallId(callId), callRenderInfos);
        }
        let featureRenderInfos = callRenderInfos.get(featureNameKey);
        if (!featureRenderInfos) {
            featureRenderInfos = new Map();
            callRenderInfos.set(featureNameKey, featureRenderInfos);
        }
        featureRenderInfos.set(streamKey, {
            stream,
            status,
            renderer
        });
    }
    deleteCallFeatureRenderInfo(callId, featureName, streamKey) {
        var _a;
        const callFeatureRenderInfoForCall = this._callFeatureRenderInfos.get(this._callIdHistory.latestCallId(callId));
        if (!callFeatureRenderInfoForCall || !callFeatureRenderInfoForCall.get(featureName)) {
            return;
        }
        (_a = callFeatureRenderInfoForCall.get(featureName)) === null || _a === void 0 ? void 0 : _a.delete(streamKey);
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
class ProxyTeamsCall extends ProxyCallCommon {
    get(target, prop) {
        switch (prop) {
            default:
                return super.get(target, prop);
        }
    }
}
/**
 * Creates a declarative Call by proxying Call with ProxyCall.
 * This should only be used with CallAgentDeclarative as CallAgentDeclarative will add that
 * call to the context properly (need to have the Call in context to update it - CallAgentDeclarative will add Call to
 * context)
 *
 * @param call - TeamsCall from SDK
 * @param context - CallContext from StatefulCallClient
 */
const teamsCallDeclaratify = (call, context) => {
    const proxyCall = new ProxyTeamsCall(context);
    proxyCall.unsubscribe();
    Object.defineProperty(call, 'unsubscribe', {
        configurable: false,
        value: () => proxyCall.unsubscribe()
    });
    return new Proxy(call, proxyCall);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * ProxyTeamsCallAgent proxies TeamsCallAgent and saves any returned state in the given context. It will subscribe to all state
 * updates in the TeamsCallAgent and in the contained TeamsCalls and RemoteParticipants. When dispose is called it will
 * unsubscribe from all state updates.
 */
class ProxyTeamsCallAgent extends ProxyCallAgentCommon {
    constructor(callAgent, context, internalContext) {
        super(context, internalContext);
        this.subscribe = () => {
            this._callAgent.on('callsUpdated', this.callsUpdated);
            this._callAgent.on('incomingCall', this.incomingCall);
            // There could be scenario that when ProxyTeamsCallAgent is created that the given CallAgent already has TeamsCalls. In this
            // case we need to make sure to subscribe to those already existing Calls.
            for (const call of this._callAgent.calls) {
                this.addCall(call);
            }
        };
        this.unsubscribe = () => {
            this._callAgent.off('callsUpdated', this.callsUpdated);
            this._callAgent.off('incomingCall', this.incomingCall);
            this.unregisterSubscriber();
        };
        this._callAgent = callAgent;
        this.subscribe();
    }
    callDeclaratify(call, context) {
        if (_isTeamsCall(call)) {
            return teamsCallDeclaratify(call, context);
        }
        throw new Error('Not reachable code, DeclarativeTeamsCallAgent.callDeclaratify must be called with an TeamsCall.');
    }
    startCall(agent, args) {
        if (_isTeamsCallAgent(agent)) {
            return agent.startCall(...args);
        }
        throw new Error('Not reachable code, DeclarativeTeamsCallAgent.startCall must be called with an TeamsCallAgent.');
    }
    joinCall(agent, args) {
        if (_isTeamsCallAgent(agent)) {
            return agent.join(...args);
        }
        throw new Error('Not reachable code, DeclarativeTeamsCallAgent.joinCall must be called with an TeamsCallAgent.');
    }
    agentSubscribe(agent, args) {
        if (_isTeamsCallAgent(agent)) {
            agent.on(...args);
            return;
        }
        throw new Error('Not reachable code, DeclarativeTeamsCallAgent.agentSubscribe must be called with an TeamsCallAgent.');
    }
    agentUnsubscribe(agent, args) {
        if (_isTeamsCallAgent(agent)) {
            agent.off(...args);
            return;
        }
        throw new Error('Not reachable code, DeclarativeTeamsCallAgent.agentUnsubscribe must be called with an TeamsCallAgent.');
    }
    get(target, prop) {
        return super.getCommon(target, prop);
    }
}
/**
 * Creates a declarative CallAgent by proxying TeamsCallAgent with ProxyTeamsCallAgent which will track state updates by updating
 * the given context.
 *
 * @param callAgent - TeamsCallAgent from SDK
 * @param context - CallContext from StatefulCallClient
 * @param internalContext- InternalCallContext from StatefulCallClient
 */
const teamsCallAgentDeclaratify = (callAgent, context, internalContext) => {
    clearCallRelatedState(context, internalContext);
    return new Proxy(callAgent, new ProxyTeamsCallAgent(callAgent, context, internalContext));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$O = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
class ProxyVideoStreamRendererView {
    constructor(context, callId, participantId, _streamId) {
        this._context = context;
        this._callId = callId;
        this._participantId = participantId;
        this._streamId = _streamId;
    }
    get(target, prop) {
        switch (prop) {
            case 'updateScalingMode':
                {
                    return (...args) => __awaiter$O(this, void 0, void 0, function* () {
                        yield target.updateScalingMode(...args);
                        this._context.setRemoteVideoStreamViewScalingMode(this._callId, this._participantId, this._streamId, args[0]);
                    });
                }
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * Creates a declarative VideoStreamRendererView that is backed by a VideoStreamRendererView from the SDK.
 * Calling methods on this declarative object triggers state updates in the stateful client.
 */
const videoStreamRendererViewDeclaratify = (view, context, callId, participantId, streamId) => {
    const proxyVideoStreamRendererView = new ProxyVideoStreamRendererView(context, callId, participantId, streamId);
    return new Proxy(view, proxyVideoStreamRendererView);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$N = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 *
 */
function createView(context, internalContext, callId, stream, options) {
    const streamType = stream.mediaStreamType;
    if (callId && isCallFeatureStream(stream)) {
        return createCallFeatureViewVideo(context, internalContext, callId, stream, options);
    }
    else {
        _logStreamEvent(EventNames.CREATE_STREAM_INVALID_PARAMS, {
            streamType
        });
        return Promise.resolve(undefined);
    }
}
// This function is used to create a view for a stream that is part of a call feature.
function createCallFeatureViewVideo(context, internalContext, callId, stream, options) {
    return __awaiter$N(this, void 0, void 0, function* () {
        const streamEventType = 'createViewCallFeature';
        const streamType = stream === null || stream === void 0 ? void 0 : stream.mediaStreamType;
        const callFeatureStreamId = stream && stream.id;
        const streamLogInfo = {
            callId,
            undefined,
            streamId: callFeatureStreamId,
            streamType,
            streamEventType
        };
        // make different logging announcement based on whether or not we are starting a local or remote
        _logStreamEvent(EventNames.CREATING_VIEW, streamLogInfo);
        const featureName = getStreamFeatureName(stream);
        // if we have a participant Id and a stream get the remote info, else get the local render info from state.
        const renderInfo = internalContext.getCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType);
        if (!renderInfo) {
            _logStreamEvent(EventNames.STREAM_NOT_FOUND, streamLogInfo);
            return;
        }
        if (renderInfo.status === 'Rendered') {
            _logStreamEvent(EventNames.STREAM_ALREADY_RENDERED, streamLogInfo);
            return;
        }
        if (renderInfo.status === 'Rendering') {
            // Do not log to console here as this is a very common situation due to UI rerenders while
            // the video rendering is in progress.
            _logStreamEvent(EventNames.STREAM_RENDERING, streamLogInfo);
            return;
        }
        // "Stopping" only happens if the stream was in "rendering" but `disposeView` was called.
        // Now that `createView` has been re-called, we can flip the state back to "rendering".
        if (renderInfo.status === 'Stopping') {
            _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
            internalContext.setCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType, renderInfo.stream, 'Rendering', renderInfo.renderer);
            return;
        }
        const renderer = new communicationCalling.VideoStreamRenderer(renderInfo.stream);
        internalContext.setCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType, renderInfo.stream, 'Rendering', undefined);
        let view;
        try {
            view = yield renderer.createView(options);
        }
        catch (e) {
            _logStreamEvent(EventNames.CREATE_STREAM_FAIL, streamLogInfo, e);
            internalContext.setCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType, renderInfo.stream, 'NotRendered', undefined);
            throw e;
        }
        // Since render could take some time, we need to check if the stream is still valid and if we received a signal to
        // stop rendering.
        const refreshedRenderInfo = internalContext.getCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType);
        if (!refreshedRenderInfo) {
            // RenderInfo was removed. This should not happen unless stream was removed from the call so dispose the renderer
            // and clean up the state.
            _logStreamEvent(EventNames.RENDER_INFO_NOT_FOUND, streamLogInfo);
            renderer.dispose();
            setCallFeatureVideoRendererView(callId, featureName, context, stream.mediaStreamType, undefined);
            return;
        }
        if (refreshedRenderInfo.status === 'Stopping') {
            // Stop render was called on this stream after we had started rendering. We will dispose this view and do not
            // put the view into the state.
            _logStreamEvent(EventNames.CREATED_STREAM_STOPPING, streamLogInfo);
            renderer.dispose();
            internalContext.setCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType, refreshedRenderInfo.stream, 'NotRendered', undefined);
            setCallFeatureVideoRendererView(callId, featureName, context, stream.mediaStreamType, undefined);
            return;
        }
        // Else the stream still exists and status is not telling us to stop rendering. Complete the render process by
        // updating the state.
        internalContext.setCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType, refreshedRenderInfo.stream, 'Rendered', renderer);
        setCallFeatureVideoRendererView(callId, featureName, context, stream.mediaStreamType, convertFromSDKToDeclarativeVideoStreamRendererView(view));
        _logStreamEvent(EventNames.VIEW_RENDER_SUCCEED, streamLogInfo);
        return {
            renderer,
            view
        };
    });
}
/**
 * @private
 */
function disposeView(context, internalContext, callId, stream) {
    const streamType = stream.mediaStreamType;
    if (callId && isCallFeatureStream(stream)) {
        return disposeCallFeatureViewVideo(context, internalContext, callId, stream);
    }
    else {
        _logStreamEvent(EventNames.DISPOSE_STREAM_INVALID_PARAMS, {
            streamType
        });
        return;
    }
}
/**
 * @private
 */
function disposeCallFeatureViewVideo(context, internalContext, callId, stream) {
    const streamType = stream.mediaStreamType;
    const callFeatureStreamId = stream && stream.id;
    const streamLogInfo = {
        callId,
        undefined,
        streamId: callFeatureStreamId,
        streamType
    };
    _logStreamEvent(EventNames.START_DISPOSE_STREAM, streamLogInfo);
    const featureName = getStreamFeatureName(stream);
    {
        setCallFeatureVideoRendererView(callId, featureName, context, streamType, undefined);
    }
    const renderInfo = internalContext.getCallFeatureRenderInfo(callId, featureName, stream.mediaStreamType);
    if (!renderInfo) {
        _logStreamEvent(EventNames.DISPOSE_INFO_NOT_FOUND, streamLogInfo);
        return;
    }
    // Nothing to dispose of or clean up -- we can safely exit early here.
    if (renderInfo.status === 'NotRendered') {
        _logStreamEvent(EventNames.STREAM_ALREADY_DISPOSED, streamLogInfo);
        return;
    }
    // Status is already marked as "stopping" so we can exit early here. This is because stopping only occurs
    // when the stream is being created in createView but hasn't been completed being created yet. The createView
    // method will see the "stopping" status and perform the cleanup
    if (renderInfo.status === 'Stopping') {
        _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
        return;
    }
    // If the stream is in the middle of being rendered (i.e. has state "Rendering"), we need the status as
    // "stopping" without performing any cleanup. This will tell the `createView` method that it should stop
    // rendering and clean up the state once the view has finished being created.
    if (renderInfo.status === 'Rendering') {
        _logStreamEvent(EventNames.STREAM_STOPPING, streamLogInfo);
        internalContext.setCallFeatureRenderInfo(callId, featureName, streamType, renderInfo.stream, 'Stopping', renderInfo.renderer);
        return;
    }
    if (renderInfo.renderer) {
        _logStreamEvent(EventNames.DISPOSING_RENDERER, streamLogInfo);
        renderInfo.renderer.dispose();
        // Else the state must be in the "Rendered" state, so we can dispose the renderer and clean up the state.
        internalContext.setCallFeatureRenderInfo(callId, featureName, streamType, renderInfo.stream, 'NotRendered', undefined);
        setCallFeatureVideoRendererView(callId, featureName, context, streamType, undefined);
    }
    else {
        _logStreamEvent(EventNames.RENDERER_NOT_FOUND, streamLogInfo);
    }
}
/**
 * @private
 */
const setCallFeatureVideoRendererView = (callId, featureName, context, streamType, view) => {
    if (featureName === 'togetherMode') {
        context.setTogetherModeVideoStreamRendererView(callId, streamType, view);
    }
};
/**
 * @private
 */
const getStreamFeatureName = (stream) => {
    if (stream.feature) {
        return stream.feature;
    }
    throw new Error('Feature name not found');
};
/**
 * @private
 */
function isCallFeatureStream(stream) {
    return 'feature' in stream || false;
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$M = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * ProxyCallClient proxies CallClient {@link @azure/communication-calling#CallClient} and subscribes to all events that
 * affect state. ProxyCallClient keeps its own copy of the call state and when state is updated, ProxyCallClient emits
 * the event 'stateChanged'.
 */
class ProxyCallClient {
    constructor(context, internalContext) {
        this._context = context;
        this._internalContext = internalContext;
    }
    get(target, prop) {
        switch (prop) {
            case 'createCallAgent':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$M(this, void 0, void 0, function* () {
                        // createCallAgent will throw an exception if the previous callAgent was not disposed. If the previous
                        // callAgent was disposed then it would have unsubscribed to events so we can just create a new declarative
                        // callAgent if the createCallAgent succeeds.
                        const callAgent = yield target.createCallAgent(...args);
                        this._callAgent = callAgentDeclaratify(callAgent, this._context, this._internalContext);
                        this._context.setCallAgent({
                            displayName: this._callAgent.displayName
                        });
                        return this._callAgent;
                    }), 'CallClient.createCallAgent');
                }
            case 'createTeamsCallAgent':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$M(this, void 0, void 0, function* () {
                        // createCallAgent will throw an exception if the previous callAgent was not disposed. If the previous
                        // callAgent was disposed then it would have unsubscribed to events so we can just create a new declarative
                        // callAgent if the createCallAgent succeeds.
                        const callAgent = yield target.createTeamsCallAgent(...args);
                        this._callAgent = teamsCallAgentDeclaratify(callAgent, this._context, this._internalContext);
                        this._context.setCallAgent({
                            displayName: undefined
                        });
                        return this._callAgent;
                    }), 'CallClient.createTeamsCallAgent');
                }
            case 'getDeviceManager':
                {
                    return this._context.withAsyncErrorTeedToState(() => __awaiter$M(this, void 0, void 0, function* () {
                        // As of writing, the SDK always returns the same instance of DeviceManager so we keep a reference of
                        // DeviceManager and if it does not change we return the cached DeclarativeDeviceManager. If it does not we'll
                        // throw an error that indicate we need to fix this issue as our implementation has diverged from the SDK.
                        const deviceManager = yield target.getDeviceManager();
                        if (this._sdkDeviceManager) {
                            if (this._sdkDeviceManager === deviceManager) {
                                return this._deviceManager;
                            }
                            else {
                                throw new Error('Multiple DeviceManager not supported. This means a incompatible version of communication-calling is ' + 'used OR calling declarative was not properly updated to communication-calling version.');
                            }
                        }
                        else {
                            this._sdkDeviceManager = deviceManager;
                        }
                        this._deviceManager = deviceManagerDeclaratify(deviceManager, this._context, this._internalContext);
                        return this._deviceManager;
                    }), 'CallClient.getDeviceManager');
                }
            case 'feature':
                {
                    return this._context.withErrorTeedToState((...args) => {
                        if (args[0] === communicationCalling.Features.DebugInfo) {
                            const feature = target.feature(communicationCalling.Features.DebugInfo);
                            /**
                             * add to this object if we want to proxy anything else off the DebugInfo feature object.
                             */
                            return Object.assign(Object.assign({}, feature), { getEnvironmentInfo: () => __awaiter$M(this, void 0, void 0, function* () {
                                    const environmentInfo = yield feature.getEnvironmentInfo();
                                    this._context.setEnvironmentInfo(environmentInfo);
                                    return environmentInfo;
                                }) });
                        }
                        return Reflect.get(target, prop);
                    }, 'CallClient.feature');
                }
            default:
                return Reflect.get(target, prop);
        }
    }
}
/**
 * Creates a StatefulCallClient {@link StatefulCallClient} by proxying CallClient
 * {@link @azure/communication-calling#CallClient} with ProxyCallClient {@link ProxyCallClient} which then allows access
 * to state in a declarative way.
 *
 * It is important to use the {@link @azure/communication-calling#DeviceManager} and
 * {@link @azure/communication-calling#CallAgent} and {@link @azure/communication-calling#Call} (and etc.) that are
 * obtained from the StatefulCallClient in order for their state changes to be proxied properly.
 *
 * @param args - {@link StatefulCallClientArgs}
 * @param options - {@link StatefulCallClientOptions}
 *
 * @public
 */
const createStatefulCallClient = (args, options) => {
    return _createStatefulCallClientInner(args, options);
};
/**
 * This inner function is used to allow injection of TelemetryImplementationHint without changing the public API.
 *
 * @internal
 */
const _createStatefulCallClientInner = (args, options, telemetryImplementationHint = 'StatefulComponents') => {
    callingStatefulLogger.info(`Creating calling stateful client using library version: ${_getApplicationId(telemetryImplementationHint)}`);
    return createStatefulCallClientWithDeps(new communicationCalling.CallClient(withTelemetryTag(telemetryImplementationHint, options === null || options === void 0 ? void 0 : options.callClientOptions)), new CallContext$2(communicationCommon.getIdentifierKind(args.userId), options === null || options === void 0 ? void 0 : options.maxStateChangeListeners), new InternalCallContext());
};
/**
 * Package-internal version of createStatefulCallClient that allows dependency injection.
 *
 * This function should not be exported from the package.
 */
const createStatefulCallClientWithDeps = (callClient, context, internalContext) => {
    Object.defineProperty(callClient, 'getState', {
        configurable: false,
        value: () => context.getState()
    });
    Object.defineProperty(callClient, 'onStateChange', {
        configurable: false,
        value: (handler) => context.onStateChange(handler)
    });
    Object.defineProperty(callClient, 'offStateChange', {
        configurable: false,
        value: (handler) => context.offStateChange(handler)
    });
    Object.defineProperty(callClient, 'createView', {
        configurable: false,
        value: (callId, participantId, stream, options) => __awaiter$M(void 0, void 0, void 0, function* () {
            if ('feature' in stream) {
                return yield createView(context, internalContext, callId, stream, options);
            }
            const participantIdKind = participantId ? communicationCommon.getIdentifierKind(participantId) : undefined;
            const result = yield createView$1(context, internalContext, callId, participantIdKind, stream, options);
            // We only need to declaratify the VideoStreamRendererView object for remote participants. Because the updateScalingMode only needs to be called on remote participant stream views.
            if ('id' in stream && callId && participantId && result) {
                const participantKey = toFlatCommunicationIdentifier(participantId);
                result.view = videoStreamRendererViewDeclaratify(result.view, context, callId, participantKey, stream.id);
            }
            return result;
        })
    });
    Object.defineProperty(callClient, 'disposeView', {
        configurable: false,
        value: (callId, participantId, stream) => {
            if ('feature' in stream) {
                disposeView(context, internalContext, callId, stream);
            }
            const participantIdKind = participantId ? communicationCommon.getIdentifierKind(participantId) : undefined;
            disposeView$1(context, internalContext, callId, participantIdKind, stream);
        }
    });
    const newStatefulCallClient = new Proxy(callClient, new ProxyCallClient(context, internalContext));
    // Populate initial state
    newStatefulCallClient.feature(communicationCalling.Features.DebugInfo).getEnvironmentInfo();
    return newStatefulCallClient;
};
const withTelemetryTag = (telemetryImplementationHint, options) => {
    var _a, _b;
    const tags = (_b = (_a = options === null || options === void 0 ? void 0 : options.diagnostics) === null || _a === void 0 ? void 0 : _a.tags) !== null && _b !== void 0 ? _b : [];
    tags.push(_getApplicationId(telemetryImplementationHint));
    return Object.assign(Object.assign({}, options), { diagnostics: Object.assign(Object.assign({}, options === null || options === void 0 ? void 0 : options.diagnostics), { tags }) });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const CallAgentContext = React.createContext(undefined);
const CallAgentProviderBase = (props) => {
    const { callAgent } = props;
    const initialState = {
        callAgent
    };
    return React.createElement(CallAgentContext.Provider, { value: initialState }, props.children);
};
/**
 * A {@link React.Context} that stores a {@link @azure/communication-calling#CallAgent}.
 *
 * Calling components from this package must be wrapped with a {@link CallAgentProvider}.
 *
 * @public
 */
const CallAgentProvider = (props) => React.createElement(CallAgentProviderBase, Object.assign({}, props));
/**
 * Hook to obtain {@link @azure/communication-calling#CallAgent} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useCallAgent = () => {
    var _a;
    const callAgent = (_a = React.useContext(CallAgentContext)) === null || _a === void 0 ? void 0 : _a.callAgent;
    if (callAgent && !_isACSCallAgent(callAgent)) {
        throw new Error('TeamsCallAgent object was provided, try useTeamsCall() instead');
    }
    return callAgent;
};
/**
 * Hook to obtain {@link @azure/communication-calling#TeamsCallAgent} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useTeamsCallAgent = () => {
    var _a;
    const callAgent = (_a = React.useContext(CallAgentContext)) === null || _a === void 0 ? void 0 : _a.callAgent;
    if (callAgent && _isACSCallAgent(callAgent)) {
        throw new Error('Regular CallAgent object was provided, try useCall() instead');
    }
    return callAgent;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const CallClientContext = React.createContext(undefined);
/**
 * @private
 */
const CallClientProviderBase = (props) => {
    const { callClient } = props;
    const [deviceManager, setDeviceManager] = React.useState(undefined);
    /**
     * Initialize the DeviceManager inside CallClientState
     */
    React.useEffect(() => {
        callClient.getDeviceManager().then(manager => {
            manager.getCameras();
            manager.getMicrophones();
            manager.getSpeakers();
            setDeviceManager(manager);
        }).catch(error => {
            throw new Error(error);
        });
    }, [callClient]);
    const initialState = {
        callClient,
        deviceManager
    };
    return React.createElement(CallClientContext.Provider, { value: initialState }, props.children);
};
/**
 * A {@link React.Context} that stores a {@link StatefulCallClient}.
 *
 * Calling components from this package must be wrapped with a {@link CallClientProvider}.
 *
 * @public
 */
const CallClientProvider = (props) => React.createElement(CallClientProviderBase, Object.assign({}, props));
/**
 * Hook to obtain {@link StatefulCallClient} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useCallClient = () => {
    const context = React.useContext(CallClientContext);
    if (context === undefined) {
        throw new Error('CallClient Context is undefined');
    }
    return context.callClient;
};
/**
 * Hook to obtain {@link StatefulDeviceManager} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useDeviceManager = () => {
    var _a;
    return (_a = React.useContext(CallClientContext)) === null || _a === void 0 ? void 0 : _a.deviceManager;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const CallContext$1 = React.createContext(undefined);
/**
 * @private
 */
const CallProviderBase = (props) => {
    const { children, call } = props;
    const initialState = {
        call
    };
    return React.createElement(CallContext$1.Provider, { value: initialState }, children);
};
/**
 * A {@link React.Context} that stores a {@link @azure/communication-calling#Call}.
 *
 * Calling components from this package must be wrapped with a {@link CallProvider}.
 *
 * @public
 */
const CallProvider = (props) => React.createElement(CallProviderBase, Object.assign({}, props));
/**
 * Hook to obtain {@link @azure/communication-calling#Call} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * you must have previously used the CallProvider with a Call object to use this hook
 *
 * @public
 */
const useCall = () => {
    var _a;
    const call = (_a = React.useContext(CallContext$1)) === null || _a === void 0 ? void 0 : _a.call;
    if (call && !_isACSCall(call)) {
        throw new Error('Incorrect call type: Must provide a Regular Call object.');
    }
    return call;
};
/**
 * Hook to obtain {@link @azure/communication-calling#TeamsCall} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * you must have previously used the CallProvider with a TeamsCall object to use this hook
 *
 * @public
 */
const useTeamsCall = () => {
    var _a;
    const call = (_a = React.useContext(CallContext$1)) === null || _a === void 0 ? void 0 : _a.call;
    if (call && !_isTeamsCall(call)) {
        throw new Error('Incorrect call type: Must provide a TeamsCall object.');
    }
    return call;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const typingIndicatorContainerStyle = react.mergeStyles({
    minHeight: '2.125rem',
    // flexFlow set to column-reverse to align the text to the bottom of the container
    flexFlow: 'column-reverse'
});
/**
 * @private
 */
const typingIndicatorStringStyle = react.mergeStyles({
    fontWeight: 400,
    width: '100%',
    alignSelf: 'center',
    wordBreak: 'break-word'
});

var participantItem$n={isMeText:"(you)",menuTitle:"More Options",removeButtonLabel:"Remove",sharingIconLabel:"Sharing",mutedIconLabel:"Muted",handRaisedIconLabel:"Hand raised - Position {position}",displayNamePlaceholder:"Unnamed participant",participantStateRinging:"Calling...",participantStateHold:"On hold",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, More Options",attendeeRole:"Attendee",micDisabledIconLabel:"Micophone disabled",cameraDisabledIconLabel:"Camera disabled"};var ParticipantList$o={overflowParticipantCount:"+{overflowCount} more"};var typingIndicator$n={singleUser:"{user} is typing ...",multipleUsers:"{users} are typing ...",multipleUsersAbbreviateOne:"{users} and 1 other are typing ...",multipleUsersAbbreviateMany:"{users} and {numOthers} others are typing ...",delimiter:", "};var sendBox$n={placeholderText:"Enter a message",textTooLong:"Your message length is over the maximum limit.",sendButtonAriaLabel:"Send message",attachmentUploadsPendingError:"File is uploading, please wait.",removeAttachment:"Remove file",uploading:"Uploading",uploadCompleted:"Upload completed",attachmentMoreMenu:"More Options"};var richTextSendBox$n={placeholderText:"Enter a message",textTooLong:"Your message length is over the maximum limit.",sendButtonAriaLabel:"Send message",attachmentUploadsPendingError:"File is uploading, please wait.",imageUploadsPendingError:"Image is uploading, please wait.",removeAttachment:"Remove file",uploading:"Uploading",uploadCompleted:"Upload completed",richTextBoldTooltip:"Bold",richTextItalicTooltip:"Italic",richTextUnderlineTooltip:"Underline",richTextBulletListTooltip:"Bulleted list",richTextNumberListTooltip:"Numbered list",richTextIncreaseIndentTooltip:"Increase indent",richTextDecreaseIndentTooltip:"Decrease indent",richTextInsertTableTooltip:"Insert table",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insert",richTextInsertTableMenuTitle:"Insert {column} x {row} table",richTextInsertRowAboveMenu:"Insert above",richTextInsertRowBelowMenu:"Insert below",richTextInsertColumnLeftMenu:"Insert left",richTextInsertColumnRightMenu:"Insert right",richTextDeleteRowOrColumnMenu:"Delete",richTextDeleteColumnMenu:"Delete column",richTextDeleteRowMenu:"Delete row",richTextDeleteTableMenu:"Delete table",richTextToolbarAriaLabel:"Message Formatting Toolbar",richTextToolbarMoreButtonAriaLabel:"More options",attachmentMoreMenu:"More Options",richTextNewBulletedListItemAnnouncement:"Bullet, new line",richTextNewNumberedListItemAnnouncement:"{0}, new line",richTextBulletedListAppliedAnnouncement:"Bulleted list applied",richTextNumberedListAppliedAnnouncement:"Numbered list applied"};var realTimeTextModal$n={realTimeTextModalAriaLabel:"Turn on RTT Modal",realTimeTextModalTitle:"Turn on RTT?",realTimeTextModalText:"Real-time text (RTT) will be turned on for everyone in the meeting.",realTimeTextConfirmButtonLabel:"Turn on",realTimeTextCancelButtonLabel:"Cancel",realTimeTextCloseModalButtonAriaLabel:"Close RTT Modal"};var captionsBanner$n={captionsBannerSpinnerText:"Starting captions...",realTimeTextInputBoxDefaultText:"Type message in real time",realTimeTextInputErrorMessage:"Message can not exceed 2000 characters",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (real-time text) is enabled for all participants for the entire duration of the meeting.",realTimeTextBannerLinkLabel:"Learn more",captionsOnlyContainerTitle:"Captions",realTimeTextOnlyContainerTitle:"Real-Time Text",captionsAndRealTimeTextContainerTitle:"Captions and Real-Time Text",expandButtonAriaLabel:"Expand",minimizeButtonAriaLabel:"Collapse"};var mentionPopover$n={mentionPopoverHeader:"Suggestions"};var imageOverlay$n={downloadButtonLabel:"Download",dismissButtonAriaLabel:"Close",overlayVisibleMessage:"Image Details Overlay"};var messageStatusIndicator$n={deliveredAriaLabel:"Message sent",deliveredTooltipText:"Sent",seenAriaLabel:"Message seen by others",seenTooltipText:"Seen",readByTooltipText:"Read by {messageThreadReadCount} of {remoteParticipantsCount}",sendingAriaLabel:"Message sending",sendingTooltipText:"Sending",failedToSendAriaLabel:"Message failed to send",failedToSendTooltipText:"Failed to send"};var endCallButton$n={label:"Leave",tooltipContent:"Leave call",leaveOption:"Leave",endCallOption:"End call for everyone"};var cameraButton$n={onLabel:"Camera",offLabel:"Camera",tooltipDisabledContent:"Camera is disabled",tooltipOnContent:"Turn off camera",tooltipOffContent:"Turn on camera",tooltipVideoLoadingContent:"Video is loading",cameraMenuTitle:"Camera",cameraMenuTooltip:"Choose camera",cameraButtonSplitRoleDescription:"Split button",onSplitButtonAriaLabel:"Turn off camera",offSplitButtonAriaLabel:"Turn on camera",cameraActionTurnedOnAnnouncement:"Your camera has been turned on",cameraActionTurnedOffAnnouncement:"Your camera has been turned off",onSplitButtonPrimaryActionCamera:"Turn off camera",offSplitButtonPrimaryActionCamera:"Turn on camera",cameraPrimaryActionSplitButtonTitle:"Use camera",videoEffectsMenuItemTitle:"Effects",cameraButtonAriaDescription:"camera options"};var microphoneButton$n={onLabel:"Mic",offLabel:"Mic",tooltipDisabledContent:"Microphone is disabled",tooltipOnContent:"Mute microphone",tooltipOffContent:"Unmute microphone",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choose microphone",speakerMenuTitle:"Speaker",speakerMenuTooltip:"Choose speaker",microphoneButtonSplitRoleDescription:"Split button",onSplitButtonAriaLabel:"Mute mic",offSplitButtonAriaLabel:"Unmute mic",microphoneActionTurnedOnAnnouncement:"Your microphone has been turned on",microphoneActionTurnedOffAnnouncement:"Your microphone has been turned off",offSplitButtonMicrophonePrimaryAction:"Unmute mic",onSplitButtonMicrophonePrimaryAction:"Mute mic",microphonePrimaryActionSplitButtonTitle:"Use microphone",microphoneAriaDescription:"Audio options",deepNoiseSuppressionTitle:"Noise suppression",deepNoiseSuppressionOnAnnouncement:"Deep noise suppression has been turned on",deepNoiseSuppressionOffAnnouncement:"Deep noise suppression has been turned off"};var devicesButton$n={label:"Devices",tooltipContent:"Manage devices",cameraMenuTitle:"Camera",cameraMenuTooltip:"Choose camera",audioDeviceMenuTitle:"Audio Device",audioDeviceMenuTooltip:"Choose audio device",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choose microphone",speakerMenuTitle:"Speaker",speakerMenuTooltip:"Choose speaker"};var participantsButton$n={label:"People",ariaLabel:"People",tooltipContent:"Show participants",menuHeader:"In this call",participantsListButtonLabel:"{numParticipants} people",muteAllButtonLabel:"Mute all",copyInviteLinkButtonLabel:"Copy invite link",copyInviteLinkActionedAriaLabel:"Invite link copied"};var screenShareButton$n={onLabel:"Stop presenting",offLabel:"Present",tooltipDisabledContent:"Presenting is disabled",tooltipOnContent:"Presenting your screen",tooltipOffContent:"Present your screen"};var raiseHandButton$n={offLabel:"Raise",onLabel:"Lower",tooltipDisabledContent:"Raise Hand action is disabled",tooltipOnContent:"Lower Hand",tooltipOffContent:"Raise Hand"};var reactionButton$n={label:"React",ariaLabel:"React",tooltipDisabledContent:"Reaction action is disabled",tooltipContent:"Send a reaction",likeReactionTooltipContent:"Like",heartReactionTooltipContent:"Love",laughReactionTooltipContent:"Laugh",applauseReactionTooltipContent:"Applause",surprisedReactionTooltipContent:"Surprised"};var realTimeText$n={isTypingText:"RTT typing"};var captionsSettingsModal$n={captionsSettingsModalTitle:"Captions Settings",captionsSettingsSpokenLanguageDropdownLabel:"Spoken language",captionsSettingsCaptionLanguageDropdownLabel:"Captions language",captionsSettingsSpokenLanguageDropdownInfoText:"Language that everyone on this call is speaking.",captionsSettingsCaptionLanguageDropdownInfoText:"Captions will appear in this langugage.",captionsSettingsConfirmButtonLabel:"Confirm",captionsSettingsCancelButtonLabel:"Cancel",captionsSettingsModalAriaLabel:"Captions Setting Modal",captionsSettingsCloseModalButtonAriaLabel:"Close Captions Setting"};var spokenLanguages$n={"ar-ae":"Arabic - U.A.E.","ar-sa":"Arabic - Saudi Arabia","da-dk":"Danish","de-de":"German - Germany","en-au":"English - Australia","en-ca":"English - Canada","en-gb":"English - United Kingdom","en-in":"English - India","en-nz":"English - New Zealand","en-us":"English - United States","es-es":"Spanish - Spain (Modern Sort)","es-mx":"Spanish - Mexico","fi-fi":"Finnish","fr-ca":"French - Canada","fr-fr":"French - France","hi-in":"Hindi","it-it":"Italian - Italy","ja-jp":"Japanese","ko-kr":"Korean","nb-no":"Norwegian (Bokmål)","nl-be":"Dutch - Belgium","nl-nl":"Dutch - Netherlands","pl-pl":"Polish","pt-br":"Portuguese - Brazil","ru-ru":"Russian","sv-se":"Swedish","zh-cn":"Chinese - People's Republic of China","zh-hk":"Chinese - Hong Kong SAR","cs-cz":"Czech","pt-pt":"Portuguese - Portugal","tr-tr":"Turkish","vi-vn":"Vietnamese","th-th":"Thai","he-il":"Hebrew","cy-gb":"Welsh","uk-ua":"Ukrainian","el-gr":"Greek","hu-hu":"Hungarian","ro-ro":"Romanian","sk-sk":"Slovak","zh-tw":"Chinese - Taiwan"};var captionLanguages$n={ar:"Arabic",da:"Danish",de:"German",en:"English",es:"Spanish",fi:"Finnish","fr-ca":"French - Canada",fr:"French - France",hi:"Hindi",it:"Italian",ja:"Japanese",ko:"Korean",nb:"Norwegian (Bokmål)",nl:"Dutch",pl:"Polish",pt:"Portuguese - Brazil",ru:"Russian",sv:"Swedish","zh-Hans":"Chinese (Simplified)","zh-Hant":"Chinese (Traditional)",cs:"Czech","pt-pt":"Portuguese - Portugal",tr:"Turkish",vi:"Vietnamese",th:"Thai",he:"Hebrew",cy:"Welsh",uk:"Ukrainian",el:"Greek",hu:"Hungarian",ro:"Romanian",sk:"Slovak"};var startCaptionsButton$n={onLabel:"Turn on captions",offLabel:"Turn off captions",tooltipOnContent:"Turn off captions",tooltipOffContent:"Turn on captions"};var startRealTimeTextButton$n={label:"Turn on RTT for this call",tooltipOffContent:"Turn on RTT for this call"};var messageThread$n={yesterday:"Yesterday",sunday:"Sunday",monday:"Monday",tuesday:"Tuesday",wednesday:"Wednesday",thursday:"Thursday",friday:"Friday",saturday:"Saturday",participantJoined:"joined the chat.",participantLeft:"left the chat.",editMessage:"Edit",removeMessage:"Delete",resendMessage:"Try sending again",failToSendTag:"Failed to send",editedTag:"Edited",liveAuthorIntro:"Message from {author}",editedMessageLiveAuthorIntro:"Edited message from {author}",editedMessageLocalUserLiveAuthorIntro:"Edited message",messageContentAriaText:"{status} {author} said {message}",messageContentMineAriaText:"{status} You said {message}",editBoxTextLimit:"Your message is over the limit of {limitNumber} characters",editBoxPlaceholderText:"Edit your message",newMessagesIndicator:"New messages",noDisplayNameSub:"No name",editBoxCancelButton:"Cancel",editBoxSubmitButton:"Done",messageReadCount:"Read by {messageReadByCount} of {remoteParticipantsCount}",actionMenuMoreOptions:"More Options",downloadAttachment:"Download file",openAttachment:"Open file",blockedWarningText:"This message was deleted due to organizational policy.",blockedWarningLinkText:"Details",attachmentCardGroupMessage:"The message has {attachmentCount} attachment",messageDeletedAnnouncementAriaLabel:"The message is deleted",richTextBoldTooltip:"Bold",richTextItalicTooltip:"Italic",richTextUnderlineTooltip:"Underline",richTextBulletListTooltip:"Bulleted list",richTextNumberListTooltip:"Numbered list",richTextIncreaseIndentTooltip:"Increase indent",richTextDecreaseIndentTooltip:"Decrease indent",richTextInsertTableTooltip:"Insert table",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insert",richTextInsertTableMenuTitle:"Insert {column} x {row} table",richTextInsertRowAboveMenu:"Insert above",richTextInsertRowBelowMenu:"Insert below",richTextInsertColumnLeftMenu:"Insert left",richTextInsertColumnRightMenu:"Insert right",richTextDeleteRowOrColumnMenu:"Delete",richTextDeleteColumnMenu:"Delete column",richTextDeleteRowMenu:"Delete row",richTextDeleteTableMenu:"Delete table",richTextToolbarMoreButtonAriaLabel:"More options",attachmentUploadsPendingError:"File is uploading, please wait.",imageUploadsPendingError:"Image is uploading, please wait.",richTextNewBulletedListItemAnnouncement:"Bullet, new line",richTextNewNumberedListItemAnnouncement:"{0}, new line",richTextBulletedListAppliedAnnouncement:"Bulleted list applied",richTextNumberedListAppliedAnnouncement:"Numbered list applied"};var errorBar$n={unableToReachChatService:"You are offline",accessDenied:"Unable to access chat services - please check the user credentials provided",userNotInChatThread:"You are no longer in this chat thread",sendMessageNotInChatThread:"Failed to send message because you are no longer in this chat thread",sendMessageGeneric:"Failed to send message",callingNetworkFailure:"Troubling connecting call - you seem to be offline",startVideoGeneric:"Failed to start video",stopVideoGeneric:"Failed to stop video",muteGeneric:"Failed to mute microphone",unmuteGeneric:"Failed to unmute microphone",speakingWhileMuted:"Your microphone is muted",startScreenShareGeneric:"There was an issue starting screen share.",stopScreenShareGeneric:"Failed to stop screen sharing",callNetworkQualityLow:"Network quality is low.",teamsMeetingCallNetworkQualityLow:"Network quality is low.",callNoSpeakerFound:"No speakers or headphones found. Connect an audio device to hear the call.",callNoMicrophoneFound:"No microphones found. Connect an audio input device.",callMicrophoneAccessDenied:"Unable to access microphone. Click the lock in the address bar to grant permission to this webpage.",callMicrophoneAccessDeniedSafari:"Unable to access microphone. Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",callMicrophoneMutedBySystem:"You are muted by your system.",callMicrophoneUnmutedBySystem:"Your microphone recovered and you were unmuted by your system.",callMacOsMicrophoneAccessDenied:"Unable to access microphone. Grant microphone permission in your macOS privacy settings.",callLocalVideoFreeze:"Network bandwidth is poor. Your video may appear paused for others on the call.",callCameraAccessDenied:"Unable to access camera. Click the lock in the address bar to grant permission to this webpage.",callCameraAccessDeniedSafari:"Unable to access camera. Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",callCameraAlreadyInUse:"Unable to access camera. It may already be in use by another application.",callVideoStoppedBySystem:"Your video has been stopped by your system.",callVideoRecoveredBySystem:"Your video has resumed.",callMacOsCameraAccessDenied:"MacOS is blocking access to your camera. Update your privacy settings to allow this browser to access your camera.",callMacOsScreenShareAccessDenied:"MacOS is blocking screen sharing. Update your privacy settings to allow this browser to record your screen.",dismissButtonAriaLabel:"Close",failedToJoinCallGeneric:"Failed to join call.",failedToJoinCallInvalidMeetingLink:"Unable to join Meeting. Invalid Link.",cameraFrozenForRemoteParticipants:"Users in the call are having issues seeing your video. Please check your devices and network.",unableToStartVideoEffect:"Unable to apply video effect.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Cannot start spotlight on participant(s) because the maximum number of participants are already spotlighted.",mutedByRemoteParticipant:"You were muted by another participant."};var notificationStack$n={callingNetworkFailure:{title:"Troubling connecting call",message:"You seem to be offline. Please check your network connection and try again.",dismissButtonAriaLabel:"Close"},startVideoGeneric:{title:"Failed to start video.",dismissButtonAriaLabel:"Close"},stopVideoGeneric:{title:"Failed to stop video.",dismissButtonAriaLabel:"Close"},muteGeneric:{title:"Failed to mute microphone.",dismissButtonAriaLabel:"Close"},unmuteGeneric:{title:"Failed to unmute microphone.",dismissButtonAriaLabel:"Close"},speakingWhileMuted:{title:"You're muted.",dismissButtonAriaLabel:"Close"},startScreenShareGeneric:{title:"There was an issue starting screen share.",dismissButtonAriaLabel:"Close"},stopScreenShareGeneric:{title:"Failed to stop screen sharing.",dismissButtonAriaLabel:"Close"},callNetworkQualityLow:{title:"Network quality is low.",dismissButtonAriaLabel:"Close"},teamsMeetingCallNetworkQualityLow:{title:"Network quality is low.",dismissButtonAriaLabel:"Close",message:"Join this call from your phone for better sound. You can continue viewing the meeting on this device.",primaryButtonLabel:"Join by Phone"},callNoSpeakerFound:{title:"No speakers or headphones found",message:"Connect an audio device to hear the call.",dismissButtonAriaLabel:"Close"},callNoMicrophoneFound:{title:"No microphones found",message:"Connect an audio input device.",dismissButtonAriaLabel:"Close"},callMicrophoneAccessDenied:{title:"Unable to access microphone",message:"Click the lock in the address bar to grant permission to this webpage.",dismissButtonAriaLabel:"Close"},callMicrophoneAccessDeniedSafari:{title:"Unable to access microphone",message:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",dismissButtonAriaLabel:"Close"},callMicrophoneMutedBySystem:{title:"You are muted by your system.",dismissButtonAriaLabel:"Close"},callMicrophoneUnmutedBySystem:{title:"Your microphone recovered and you were unmuted by your system.",dismissButtonAriaLabel:"Close"},callMacOsMicrophoneAccessDenied:{title:"Unable to access microphone",message:"Grant microphone permission in your macOS privacy settings.",dismissButtonAriaLabel:"Close"},callLocalVideoFreeze:{title:"Network bandwidth is poor",message:"Your video may appear paused for others on the call.",dismissButtonAriaLabel:"Close"},callCameraAccessDenied:{title:"Unable to access camera",message:"Click the lock in the address bar to grant permission to this webpage.",dismissButtonAriaLabel:"Close"},callCameraAccessDeniedSafari:{title:"Unable to access camera",message:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",dismissButtonAriaLabel:"Close"},callCameraAlreadyInUse:{title:"Unable to access camera",message:"It may already be in use by another application.",dismissButtonAriaLabel:"Close"},callVideoStoppedBySystem:{title:"Your video has been stopped by your system.",dismissButtonAriaLabel:"Close"},callVideoRecoveredBySystem:{title:"Your video has resumed.",dismissButtonAriaLabel:"Close"},callMacOsCameraAccessDenied:{title:"MacOS is blocking access to your camera",message:"Update your privacy settings to allow this browser to access your camera.",dismissButtonAriaLabel:"Close"},callMacOsScreenShareAccessDenied:{title:"MacOS is blocking screen sharing",message:"Update your privacy settings to allow this browser to record your screen.",dismissButtonAriaLabel:"Close"},failedToJoinCallGeneric:{title:"Failed to join call.",dismissButtonAriaLabel:"Close"},failedToJoinCallInvalidMeetingLink:{title:"Unable to join Meeting. Invalid Link.",dismissButtonAriaLabel:"Close"},cameraFrozenForRemoteParticipants:{title:"Users in the call are having issues seeing your video",message:"Please check your devices and network.",dismissButtonAriaLabel:"Close"},unableToStartVideoEffect:{title:"Unable to apply video effect.",dismissButtonAriaLabel:"Close"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Cannot start spotlight on participant(s)",message:"The maximum number of participants are already spotlighted.",dismissButtonAriaLabel:"Close"},mutedByRemoteParticipant:{title:"You were muted by another participant.",dismissButtonAriaLabel:"Close"},recordingStarted:{title:"Recording has started.",message:"By joining, you are giving consent for this meeting to be transcribed.",dismissButtonAriaLabel:"Close"},transcriptionStarted:{title:"Transcription has started.",message:"By joining, you are giving consent for this meeting to be transcribed.",dismissButtonAriaLabel:"Close"},recordingStopped:{title:"Recording is being saved.",message:"Recording has stopped.",dismissButtonAriaLabel:"Close"},transcriptionStopped:{title:"Transcription is being saved.",message:"Transcription has stopped.",dismissButtonAriaLabel:"Close"},recordingAndTranscriptionStarted:{title:"Recording and transcription have started.",message:"By joining, you are giving consent for this meeting to be transcribed.",dismissButtonAriaLabel:"Close"},recordingAndTranscriptionStopped:{title:"Recording and transcription are being saved.",message:"Recording and transcription have stopped.",dismissButtonAriaLabel:"Close"},recordingStoppedStillTranscribing:{title:"Recording has stopped.",message:"You are now only transcribing this meeting.",dismissButtonAriaLabel:"Close"},transcriptionStoppedStillRecording:{title:"Transcription has stopped.",message:"You are now only recording this meeting.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomOpened:{title:"We'll move you to your assigned room in 5 seconds.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomChanged:{title:"Your breakout room has changed",message:"We'll move you to your assigned room in 5 seconds.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomClosed:{title:"Your breakout room has closed",message:"We'll move you back to your meeting in 5 seconds.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomOpenedPromptJoin:{title:"Join breakout room?",message:"You've been assigned to a breakout room.",dismissButtonAriaLabel:"Close",primaryButtonLabel:"Join room",secondaryButtonLabel:"Later"},breakoutRoomJoined:{title:"You have joined a breakout room.",dismissButtonAriaLabel:"Close"},breakoutRoomClosingSoon:{title:"Room time limit about to expire.",message:"This room will close in 30 seconds",dismissButtonAriaLabel:"Close"},capabilityTurnVideoOnPresent:{title:"Your camera has been enabled",message:"Turn it on if you'd like to share video.",dismissButtonAriaLabel:"Close"},capabilityTurnVideoOnAbsent:{title:"Your camera has been disabled",message:"You can no longer share video.",dismissButtonAriaLabel:"Close"},capabilityUnmuteMicPresent:{title:"Your mic has been enabled",message:"Unmute if you'd like to speak.",dismissButtonAriaLabel:"Close"},capabilityUnmuteMicAbsent:{title:"Your mic has been disabled",message:"You can no longer unmute.",dismissButtonAriaLabel:"Close"},togetherModeStarted:{title:"Togethermode has started",dismissButtonAriaLabel:"Close"},togetherModeEnded:{title:"Togethermode has ended",dismissButtonAriaLabel:"Close"},transcriptionError:{title:"Something went wrong with transcription.",message:"Transcription has stopped. please try again later",dismissButtonAriaLabel:"Close"},transcriptionStartedByYou:{title:"Transcription has started.",message:"Started by you. Let everyone know they are being transcribed.",dismissButtonAriaLabel:"Close"}};var videoGallery$n={screenIsBeingSharedMessage:"You are sharing your screen",screenShareLoadingMessage:"Loading {participant}'s screen",localScreenShareLoadingMessage:"Loading your screen",localVideoLabel:"You",localVideoCameraSwitcherLabel:"Switch camera",localVideoMovementLabel:"Movable Local Video Tile",localVideoMovementAriaLabel:"Movable Local Video Tile",localVideoSelectedDescription:"{cameraName} selected",displayNamePlaceholder:"Unnamed participant",fitRemoteParticipantToFrame:"Fit to frame",fillRemoteParticipantFrame:"Fill frame",pinParticipantForMe:"Pin for me",pinParticipantForMeLimitReached:"Pin (limit reached)",unpinParticipantForMe:"Unpin",pinParticipantMenuItemAriaLabel:"Pin {participantName}",unpinParticipantMenuItemAriaLabel:"Unpin {participantName}",pinnedParticipantAnnouncementAriaLabel:"Pinned {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Unpinned {participantName}",startSpotlightVideoTileMenuLabel:"Spotlight for everyone",addSpotlightVideoTileMenuLabel:"Add spotlight",spotlightLimitReachedMenuTitle:"Spotlight limit reached",stopSpotlightVideoTileMenuLabel:"Stop spotlighting",stopSpotlightOnSelfVideoTileMenuLabel:"Exit spotlight",attendeeRole:"Attendee",muteParticipantMenuItemLabel:"Mute",waitingScreenText:"Waiting for others to join",forbidAudio:"Disable mic",permitAudio:"Allow mic",forbidAudioTileMenuLabel:"Disable mic",permitAudioTileMenuLabel:"Allow mic",forbidVideo:"Disable camera",permitVideo:"Allow camera",forbidVideoTileMenuLabel:"Disable camera",permitVideoTileMenuLabel:"Allow camera"};var dialpad$n={placeholderText:"Enter phone number",deleteButtonAriaLabel:"Delete"};var holdButton$n={onLabel:"Resume",offLabel:"Hold",tooltipOnContent:"Resume call",tooltipOffContent:"Hold call"};var videoTile$n={participantStateRinging:"Calling...",participantReconnecting:"Reconnecting...",participantStateHold:"On hold",moreOptionsButtonAriaLabel:"More Options {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"muted",moreOptionsParticipantMutedStateUnmutedAriaLabel:"unmuted",moreOptionsParticipantHandRaisedAriaLabel:"Hand raised",moreOptionsParticipantIsSpeakingAriaLabel:"Speaking",moreOptionsParticipantMicDisabledAriaLabel:"Microphone disabled",moreOptionsParticipantCameraDisabledAriaLabel:"Camera disabled"};var CameraAndMicrophoneSitePermissionsRequest$n={primaryText:"Allow {appName} to use your camera and microphone",secondaryText:"This is so participants can see and hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera and microphone",ariaLabel:"Allow camera and microphone access"};var CameraSitePermissionsRequest$n={primaryText:"Allow {appName} to use your camera",secondaryText:"This is so participants can see you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera",ariaLabel:"Allow camera access"};var MicrophoneSitePermissionsRequest$n={primaryText:"Allow {appName} to use your microphone",secondaryText:"This is so participants can hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without microphone",ariaLabel:"Allow microphone access"};var CameraAndMicrophoneSitePermissionsCheck$n={primaryText:"Checking for camera and microphone access",secondaryText:"Allow access if prompted. This is so participants can see and hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera and microphone",ariaLabel:"Checking for camera and microphone access. Allow access if prompted."};var CameraSitePermissionsCheck$n={primaryText:"Checking for camera access",secondaryText:"Allow access if prompted. This is so participants can see you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera",ariaLabel:"Checking for camera access. Allow access if prompted."};var MicrophoneSitePermissionsCheck$n={primaryText:"Checking for microphone access",secondaryText:"Allow access if prompted. This is so participants can hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without microphone",ariaLabel:"Checking for microphone access. Allow access if prompted."};var CameraAndMicrophoneSitePermissionsDenied$n={primaryText:"Unable to access camera and microphone",secondaryText:"Click the lock icon in the address bar to grant microphone permissions to this webpage. A page refresh may be required.",primaryButtonText:"Continue without camera and microphone",linkText:"Need help? Get troubleshooting help"};var CameraAndMicrophoneSitePermissionsDeniedSafari$n={primaryText:"Unable to access camera and microphone",secondaryText:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",primaryButtonText:"Continue without camera and microphone",linkText:"Need help? Get troubleshooting help"};var CameraSitePermissionsDenied$n={primaryText:"Unable to access camera",secondaryText:"Click the lock icon in the address bar to grant camera permissions to this webpage. A page refresh may be required.",primaryButtonText:"Continue without camera",linkText:"Need help? Get troubleshooting help"};var MicrophoneSitePermissionsDenied$n={primaryText:"Unable to access microphone",secondaryText:"Click the lock icon in the address bar to grant microphone permissions to this webpage. A page refresh may be required.",primaryButtonText:"Continue without microphone",linkText:"Need help? Get troubleshooting help"};var CameraSitePermissionsDeniedSafari$n={primaryText:"Unable to access camera",secondaryText:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",primaryButtonText:"Continue without camera",linkText:"Need help? Get troubleshooting help"};var MicrophoneSitePermissionsDeniedSafari$n={primaryText:"Unable to access microphone",secondaryText:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",primaryButtonText:"Continue without microphone",linkText:"Need help? Get troubleshooting help"};var UnsupportedBrowser$n={primaryText:"Browser not supported",secondaryText:"Please join this call using a compatible browser.",moreHelpLinkText:"See compatibility requirements"};var UnsupportedBrowserVersion$n={primaryText:"Browser update needed",secondaryText:"To ensure the best call possible, please update your browser and then try joining the call again.",moreHelpLinkText:"See compatibility requirements",continueAnywayButtonText:"Start call without updating"};var UnsupportedOperatingSystem$n={primaryText:"Operating system not supported",secondaryText:"Please join this call using a device with a compatible operating system.",moreHelpLinkText:"See compatibility requirements"};var BrowserPermissionDenied$n={primaryText:"Can't use your camera or microphone",secondaryText:"Your browser might not have access to your camera or microphone. To fix this, open System Preferences.",primaryButtonText:"Try again",linkText:"Need help? Get troubleshooting help"};var BrowserPermissionDeniedIOS$n={primaryText:"Allow microphone access to continue",secondaryText:"So other participants can hear you.",primaryButtonText:"Try again",imageAltText:"Microphone and camera device permission location for iOS",linkText:"Need help? Get troubleshooting help",step1Text:"Go to the Settings app",step2Text:"Scroll down to settings for this browser",step3Text:"Turn on Microphone (Camera optional)",step4Text:"Try joining the call again",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$n={leftNavButtonAriaLabel:"previous page {current} of {total}",rightNavButtonAriaLabel:"next page {current} of {total}"};var meetingConferencePhoneInfo$n={meetingConferencePhoneInfoModalTitle:"Join call from your phone",meetingConferencePhoneInfoModalDialIn:"Dial number",meetingConferencePhoneInfoModalMeetingId:"Enter meeting ID",meetingConferencePhoneInfoModalWait:"Wait to be admitted by organizer",meetingConferencePhoneInfoModalNoPhoneAvailable:"No phone available",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Toll-free)",meetingConferencePhoneInfoModalToll:"(Toll)"};var IncomingCallNotification$o={incomingCallNotificationPlaceholderId:"Unknown User",incomingCallNotificationPlaceholderAlert:"{callerName} is calling",incomingCallNoticicationAcceptWithAudioAriaLabel:"Accept with audio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Accept with video",incomingCallNoticicationRejectAriaLabel:"Reject",incomingCallNotificationAcceptButtonLabel:"Accept",incomingCallNotificationRejectButtonLabel:"Decline",incomingCallNotificationAccceptWithVideoButtonLabel:"Accept with Video",incomingCallNotificationDismissButtonAriaLabel:"Dismiss"};var en_US$1 = {participantItem:participantItem$n,ParticipantList:ParticipantList$o,typingIndicator:typingIndicator$n,sendBox:sendBox$n,richTextSendBox:richTextSendBox$n,realTimeTextModal:realTimeTextModal$n,captionsBanner:captionsBanner$n,mentionPopover:mentionPopover$n,imageOverlay:imageOverlay$n,messageStatusIndicator:messageStatusIndicator$n,endCallButton:endCallButton$n,cameraButton:cameraButton$n,microphoneButton:microphoneButton$n,devicesButton:devicesButton$n,participantsButton:participantsButton$n,screenShareButton:screenShareButton$n,raiseHandButton:raiseHandButton$n,reactionButton:reactionButton$n,realTimeText:realTimeText$n,captionsSettingsModal:captionsSettingsModal$n,spokenLanguages:spokenLanguages$n,captionLanguages:captionLanguages$n,startCaptionsButton:startCaptionsButton$n,startRealTimeTextButton:startRealTimeTextButton$n,messageThread:messageThread$n,errorBar:errorBar$n,notificationStack:notificationStack$n,videoGallery:videoGallery$n,dialpad:dialpad$n,holdButton:holdButton$n,videoTile:videoTile$n,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$n,CameraSitePermissionsRequest:CameraSitePermissionsRequest$n,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$n,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$n,CameraSitePermissionsCheck:CameraSitePermissionsCheck$n,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$n,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$n,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$n,CameraSitePermissionsDenied:CameraSitePermissionsDenied$n,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$n,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$n,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$n,UnsupportedBrowser:UnsupportedBrowser$n,UnsupportedBrowserVersion:UnsupportedBrowserVersion$n,UnsupportedOperatingSystem:UnsupportedOperatingSystem$n,BrowserPermissionDenied:BrowserPermissionDenied$n,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$n,verticalGallery:verticalGallery$n,meetingConferencePhoneInfo:meetingConferencePhoneInfo$n,IncomingCallNotification:IncomingCallNotification$o};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for English (US).
 *
 * @public
 */
const COMPONENT_LOCALE_EN_US = {
    strings: en_US$1
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Context for providing localized strings to components exported from this library.
 *
 * @public
 */
const LocaleContext$1 = React.createContext(COMPONENT_LOCALE_EN_US);
/**
 * Provider to provide localized strings for this library's react components.
 *
 * @remarks Components will be provided localized strings in English (US) by default if this
 * provider is not used.
 *
 * @public
 */
const LocalizationProvider$1 = (props) => {
    const { locale, children } = props;
    return React.createElement(LocaleContext$1.Provider, { value: locale }, children);
};
/** React hook to access locale */
const useLocale$1 = () => React.useContext(LocaleContext$1);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const defaultIdentifiers = {
    sendboxTextField: 'sendbox-textfield',
    participantButtonPeopleMenuItem: 'participant-button-people-menu-item',
    participantItemMenuButton: 'participant-item-menu-button',
    participantList: 'participant-list',
    participantListPeopleButton: 'participant-list-people-button',
    participantListRemoveParticipantButton: 'participant-list-remove-participant-button',
    messageContent: 'message-content',
    messageTimestamp: 'message-timestamp',
    typingIndicator: 'typing-indicator',
    videoGallery: 'video-gallery',
    videoTile: 'video-tile',
    overflowGalleryLeftNavButton: 'overflow-gallery-left-nav-button',
    overflowGalleryRightNavButton: 'overflow-gallery-right-nav-button',
    verticalGalleryVideoTile: 'vertical-gallery-video-tile',
    horizontalGalleryVideoTile: 'horizontal-gallery-video-tile',
    verticalGalleryPageCounter: 'vertical-gallery-page-counter'
};
/**
 * @private
 */
const IdentifierContext = React.createContext(defaultIdentifiers);
/**
 * React Context provider for {@link _Identifiers}.
 *
 * @experimental
 *
 * See documentation for {@link _Identifiers}.
 *
 * @internal
 */
const _IdentifierProvider = (props) => {
    const { identifiers, children } = props;
    return React.createElement(IdentifierContext.Provider, { value: identifiers !== null && identifiers !== void 0 ? identifiers : defaultIdentifiers }, children);
};
/**
 * @private
 */
const useIdentifiers = () => React.useContext(IdentifierContext);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const MAXIMUM_LENGTH_OF_TYPING_USERS = 35;
/**
 * Helper function to create element wrapping all typing users
 * @param typingUsers typing users
 * @param delimiter string to separate typing users
 * @param onRenderUser optional callback to render each typing user
 * @param userDisplayNameStyles optional additional IStyle to apply to each element containing users name
 * @returns element wrapping all typing users
 */
const getUsersElement = (typingUsers, delimiter, onRenderUser, userDisplayNameStyles) => {
    const userElements = [];
    typingUsers.forEach((user, index) => {
        let truncatedDisplayName = user.displayName;
        if (truncatedDisplayName && truncatedDisplayName.length > 50) {
            truncatedDisplayName = truncatedDisplayName.substring(0, 50) + '...';
        }
        userElements.push(onRenderUser ? onRenderUser(user) : React.createElement(react.Text, { className: react.mergeStyles(userDisplayNameStyles), key: `user-${index}` }, truncatedDisplayName));
        userElements.push(React.createElement(react.Text, { key: `comma-${index}` }, `${delimiter}`));
    });
    // pop last comma
    userElements.pop();
    return React.createElement(React.Fragment, null, userElements);
};
/**
 * Helper function to get a string of all typing users
 * @param typingUsers typing users
 * @param delimiter string to separate typing users
 * @returns string of all typing users
 */
const getNamesString = (typingUsers, delimiter) => {
    const userNames = [];
    typingUsers.forEach(user => {
        if (user.displayName) {
            userNames.push(user.displayName);
        }
    });
    return userNames.join(delimiter);
};
/**
 * Helper function to create span elements making up the typing indicator string
 * @param strings TypingIndicatorStrings containing strings to create span elements
 * @param usersElement JSX.Element containing all typing users
 * @param numTypingUsers number of total typing users
 * @param numUserNotMentioned number of typing users abbreviated
 * @returns array of span elements making up the typing indicator string
 */
const getSpanElements = (strings, usersElement, numTypingUsers, numTypingUsersAbbreviated) => {
    let variables = {};
    let typingString = '';
    if (numTypingUsers === 1) {
        typingString = strings.singleUser;
        variables = {
            user: usersElement
        };
    }
    else if (numTypingUsers > 1 && numTypingUsersAbbreviated === 0) {
        typingString = strings.multipleUsers;
        variables = {
            users: usersElement
        };
    }
    else if (numTypingUsers > 1 && numTypingUsersAbbreviated === 1) {
        typingString = strings.multipleUsersAbbreviateOne;
        variables = {
            users: usersElement
        };
    }
    else if (numTypingUsers > 1 && numTypingUsersAbbreviated > 1) {
        typingString = strings.multipleUsersAbbreviateMany;
        variables = {
            users: usersElement,
            numOthers: React.createElement(React.Fragment, null, numTypingUsersAbbreviated)
        };
    }
    return formatInlineElements(typingString, variables);
};
/**
 * Helper function to get the string making up the typing indicator string
 * @param strings TypingIndicatorStrings containing strings to create span elements
 * @param namesString string of all typing users
 * @param numTypingUsers number of total typing users
 * @param numUserNotMentioned number of typing users abbreviated
 * @returns typing indicator string
 */
const getIndicatorString = (strings, namesString, numTypingUsers, numTypingUsersAbbreviated) => {
    if (numTypingUsers === 1) {
        return strings.singleUser.replace('{user}', namesString);
    }
    if (numTypingUsers > 1 && numTypingUsersAbbreviated === 0) {
        return strings.multipleUsers.replace('{users}', namesString);
    }
    if (numTypingUsers > 1 && numTypingUsersAbbreviated === 1) {
        return strings.multipleUsersAbbreviateOne.replace('{users}', namesString);
    }
    if (numTypingUsers > 1 && numTypingUsersAbbreviated > 1) {
        return strings.multipleUsersAbbreviateMany.replace('{users}', namesString).replace('{numOthers}', `${numTypingUsersAbbreviated}`);
    }
    return undefined;
};
const IndicatorComponent = (typingUsers, strings, onRenderUser, styles) => {
    const typingUsersMentioned = [];
    let totalCharacterCount = 0;
    const ids = useIdentifiers();
    for (const typingUser of typingUsers) {
        if (!typingUser.displayName) {
            continue;
        }
        let additionalCharCount = typingUser.displayName.length;
        // The typing users will be separated by the delimiter. We account for that additional length when we generate the final string.
        if (typingUsersMentioned.length > 0) {
            additionalCharCount += strings.delimiter.length;
        }
        if (totalCharacterCount + additionalCharCount <= MAXIMUM_LENGTH_OF_TYPING_USERS || typingUsersMentioned.length === 0) {
            typingUsersMentioned.push(typingUser);
            totalCharacterCount += additionalCharCount;
        }
        else {
            break;
        }
    }
    const usersElement = getUsersElement(typingUsersMentioned, strings.delimiter, onRenderUser, styles === null || styles === void 0 ? void 0 : styles.typingUserDisplayName);
    const numUserNotMentioned = typingUsers.length - typingUsersMentioned.length;
    const spanElements = getSpanElements(strings, usersElement, typingUsers.length, numUserNotMentioned);
    const labelString = getIndicatorString(strings, getNamesString(typingUsersMentioned, strings.delimiter), typingUsers.length, numUserNotMentioned);
    return React.createElement("div", { "data-ui-id": ids.typingIndicator, className: react.mergeStyles(typingIndicatorStringStyle, styles === null || styles === void 0 ? void 0 : styles.typingString), key: "typingStringKey", role: "status", "aria-label": labelString }, spanElements);
};
/**
 * Component to notify local user when one or more participants in the chat thread are typing.
 *
 * @public
 */
const TypingIndicator = (props) => {
    const { typingUsers, onRenderUser, styles } = props;
    const { strings } = useLocale$1();
    const typingUsersToRender = typingUsers.filter(typingUser => typingUser.displayName !== undefined);
    const indicatorComponent = IndicatorComponent(typingUsersToRender, Object.assign(Object.assign({}, strings.typingIndicator), props.strings), onRenderUser, styles);
    return React.createElement(react.Stack, { className: react.mergeStyles(typingIndicatorContainerStyle, styles === null || styles === void 0 ? void 0 : styles.root) }, indicatorComponent);
};
/**
 * Create an array of span elements by replacing the pattern "\{\}" in str with the elements
 * passed in as vars and creating inline elements from the rest
 *
 * @param str - The string to be formatted
 * @param vars - Variables to use to format the string
 * @returns formatted JSX elements
 */
const formatInlineElements = (str, vars) => {
    if (!str) {
        return [];
    }
    if (!vars) {
        return [];
    }
    const elements = [];
    // regex to search for the pattern "{}"
    const placeholdersRegex = /{(\w+)}/g;
    const regex = RegExp(placeholdersRegex);
    let array = regex.exec(str);
    let prev = 0;
    while (array !== null) {
        if (prev !== array.index) {
            elements.push(React.createElement(react.Text, { key: elements.length }, str.substring(prev, array.index)));
        }
        elements.push(React.createElement(react.Text, { key: elements.length }, vars[array[0].substring(1, array[0].length - 1)]));
        prev = regex.lastIndex;
        array = regex.exec(str);
    }
    elements.push(React.createElement(react.Text, { key: elements.length }, str.substring(prev)));
    return elements;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 * @param dismissedErrors
 * @param toDismiss
 * @returns DismissedError[]
 * Always returns a new Array so that the state variable is updated, trigerring a render.
 */
const dismissError = (dismissedErrors, toDismiss) => {
    var _a;
    const now = new Date(Date.now());
    for (const error of dismissedErrors) {
        if (error.type === toDismiss.type) {
            // Bump the timestamp for latest dismissal of this error to now.
            error.dismissedAt = now;
            error.activeSince = toDismiss.timestamp;
            return Array.from(dismissedErrors);
        }
    }
    const toDismissTimestamp = (_a = toDismiss.timestamp) !== null && _a !== void 0 ? _a : now;
    // Record that this error was dismissed for the first time right now.
    return [...dismissedErrors, {
            type: toDismiss.type,
            // the error time could be sometimes later than the button click time, which cause the dismiss not working
            // so we set the dismiss time to the later one
            dismissedAt: now > toDismissTimestamp ? now : toDismissTimestamp,
            activeSince: toDismiss.timestamp
        }];
};
/**
 * @private
 * @param activeErrorMessages
 * @param dismissedErrors
 * @returns DismissedError[]
 *  Returns a new Array if and only if contents change, to avoid re-rendering when nothing was dropped.
 */
const dropDismissalsForInactiveErrors = (activeErrorMessages, dismissedErrors) => {
    const active = new Map();
    for (const message of activeErrorMessages) {
        active.set(message.type, message);
    }
    // For an error such that:
    // * It was previously active, and dismissed.
    // * It did not have a timestamp associated with it.
    // * It is no longer active.
    //
    // We remove it from dismissals. When it becomes active again next time, it will be shown again on the UI.
    const shouldDeleteDismissal = (dismissed) => dismissed.activeSince === undefined && active.get(dismissed.type) === undefined;
    if (dismissedErrors.some(dismissed => shouldDeleteDismissal(dismissed))) {
        return dismissedErrors.filter(dismissed => !shouldDeleteDismissal(dismissed));
    }
    return dismissedErrors;
};
/**
 * @private
 * @param dismissedNotifications
 * @param toDismiss
 * @returns DismissedNotification[]
 * Always returns a new Array so that the state variable is updated, trigerring a render.
 */
const dismissNotification = (dismissedNotifications, toDismiss) => {
    var _a;
    const now = new Date(Date.now());
    for (const notification of dismissedNotifications) {
        if (notification.type === toDismiss.type) {
            // Bump the timestamp for latest dismissal of this error to now.
            notification.dismissedAt = now;
            notification.activeSince = toDismiss.timestamp;
            return Array.from(dismissedNotifications);
        }
    }
    const toDismissTimestamp = (_a = toDismiss.timestamp) !== null && _a !== void 0 ? _a : now;
    // Record that this error was dismissed for the first time right now.
    return [...dismissedNotifications, {
            type: toDismiss.type,
            // the error time could be sometimes later than the button click time, which cause the dismiss not working
            // so we set the dismiss time to the later one
            dismissedAt: now > toDismissTimestamp ? now : toDismissTimestamp,
            activeSince: toDismiss.timestamp
        }];
};
/**
 * @private
 * @param activeNotifications
 * @param dismissedNotifications
 * @returns DismissedError[]
 *  Returns a new Array if and only if contents change, to avoid re-rendering when nothing was dropped.
 */
const dropDismissalsForInactiveNotifications = (activeNotifications, dismissedNotifications) => {
    const active = new Map();
    for (const message of activeNotifications) {
        active.set(message.type, message);
    }
    // For an error such that:
    // * It was previously active, and dismissed.
    // * It did not have a timestamp associated with it.
    // * It is no longer active.
    //
    // We remove it from dismissals. When it becomes active again next time, it will be shown again on the UI.
    const shouldDeleteDismissal = (dismissed) => dismissed.activeSince === undefined && active.get(dismissed.type) === undefined;
    if (dismissedNotifications.some(dismissed => shouldDeleteDismissal(dismissed))) {
        return dismissedNotifications.filter(dismissed => !shouldDeleteDismissal(dismissed));
    }
    return dismissedNotifications;
};
/**
 * @private
 * @param activeErrorMessages
 * @param dismissedErrors
 * @returns ActiveErrorMessage[]
 */
const errorsToShow = (activeErrorMessages, dismissedErrors, mountTimestamp) => {
    const dismissed = new Map();
    for (const error of dismissedErrors) {
        dismissed.set(error.type, error);
    }
    return activeErrorMessages.filter(error => {
        if (mountTimestamp && error.timestamp && mountTimestamp > error.timestamp) {
            // Error has a timestamp and it is older than when the component was mounted.
            return false;
        }
        const dismissal = dismissed.get(error.type);
        if (!dismissal) {
            // This error was never dismissed.
            return true;
        }
        if (!error.timestamp) {
            // No timestamp associated with the error. In this case, the existence of a dismissal is enough to suppress the error.
            return false;
        }
        // Error has an associated timestamp, so compare with last dismissal.
        return error.timestamp > dismissal.dismissedAt;
    });
};
/**
 * @private
 * @param activeNotifications
 * @param dismissedNotifications
 * @returns ActiveNotification[]
 */
const notificationsToShow = (activeNotifications, dismissedNotifications, mountTimestamp) => {
    const dismissed = new Map();
    for (const notification of dismissedNotifications) {
        dismissed.set(notification.type, notification);
    }
    return activeNotifications.filter(notification => {
        if (mountTimestamp && notification.timestamp && mountTimestamp > notification.timestamp) {
            // Notification has a timestamp and it is older than when the component was mounted.
            return false;
        }
        const dismissal = dismissed.get(notification.type);
        if (!dismissal) {
            // This error was never dismissed.
            return true;
        }
        if (!notification.timestamp) {
            // No timestamp associated with the error. In this case, the existence of a dismissal is enough to suppress the error.
            return false;
        }
        // Error has an associated timestamp, so compare with last dismissal.
        return notification.timestamp > dismissal.dismissedAt;
    });
};
/**
 * @private
 * @param errorType
 * @returns MessageBarType
 */
const messageBarType = (errorType) => {
    switch (errorType) {
        case 'callNetworkQualityLow':
        case 'callNoSpeakerFound':
        case 'callNoMicrophoneFound':
        case 'callMicrophoneAccessDenied':
        case 'callMicrophoneAccessDeniedSafari':
        case 'callMicrophoneMutedBySystem':
        case 'callMicrophoneUnmutedBySystem':
        case 'callMacOsMicrophoneAccessDenied':
        case 'callLocalVideoFreeze':
        case 'callCameraAccessDenied':
        case 'callCameraAccessDeniedSafari':
        case 'callCameraAlreadyInUse':
        case 'callVideoStoppedBySystem':
        case 'callVideoRecoveredBySystem':
        case 'callMacOsCameraAccessDenied':
        case 'callMacOsScreenShareAccessDenied':
        case 'startScreenShareGeneric':
        case 'cameraFrozenForRemoteParticipants':
            return react.MessageBarType.warning;
        default:
            return react.MessageBarType.error;
    }
};
/**
 * @private
 * @param errorType
 * @returns IIconProps | undefined
 */
const messageBarIconProps = (errorType) => {
    const iconName = customIconName[errorType];
    return iconName ? {
        iconName
    } : undefined;
};
/**
 * @private
 */
const customIconName = {
    callNetworkQualityLow: 'ErrorBarCallNetworkQualityLow',
    callNoSpeakerFound: 'ErrorBarCallNoSpeakerFound',
    callNoMicrophoneFound: 'ErrorBarCallNoMicrophoneFound',
    callMicrophoneAccessDenied: 'ErrorBarCallMicrophoneAccessDenied',
    callMicrophoneAccessDeniedSafari: 'ErrorBarCallMicrophoneAccessDenied',
    callMicrophoneMutedBySystem: 'ErrorBarCallMicrophoneMutedBySystem',
    callMicrophoneUnmutedBySystem: 'ErrorBarCallMicrophoneUnmutedBySystem',
    callMacOsMicrophoneAccessDenied: 'ErrorBarCallMacOsMicrophoneAccessDenied',
    callLocalVideoFreeze: 'ErrorBarCallLocalVideoFreeze',
    callCameraAccessDenied: 'ErrorBarCallCameraAccessDenied',
    callCameraAccessDeniedSafari: 'ErrorBarCallCameraAccessDenied',
    callCameraAlreadyInUse: 'ErrorBarCallCameraAlreadyInUse',
    callVideoStoppedBySystem: 'ErrorBarCallVideoStoppedBySystem',
    callVideoRecoveredBySystem: 'ErrorBarCallVideoRecoveredBySystem',
    callMacOsCameraAccessDenied: 'ErrorBarCallMacOsCameraAccessDenied'
};
/**
 * @private
 * @param NotificationType
 * @returns IIconProps | undefined
 */
const NotificationIconProps = (notificationType) => {
    const iconName = customNotificationIconName[notificationType];
    return iconName ? {
        iconName
    } : undefined;
};
/**
 * @private
 */
const customNotificationIconName = {
    callNetworkQualityLow: 'ErrorBarCallNetworkQualityLow',
    teamsMeetingCallNetworkQualityLow: 'ErrorBarCallNetworkQualityLow',
    callNoSpeakerFound: 'ErrorBarCallNoSpeakerFound',
    callNoMicrophoneFound: 'ErrorBarCallNoMicrophoneFound',
    callMicrophoneAccessDenied: 'ErrorBarCallMicrophoneAccessDenied',
    callMicrophoneAccessDeniedSafari: 'ErrorBarCallMicrophoneAccessDenied',
    callMicrophoneMutedBySystem: 'ErrorBarCallMicrophoneMutedBySystem',
    callMicrophoneUnmutedBySystem: 'ErrorBarCallMicrophoneUnmutedBySystem',
    callMacOsMicrophoneAccessDenied: 'ErrorBarCallMacOsMicrophoneAccessDenied',
    callLocalVideoFreeze: 'ErrorBarCallLocalVideoFreeze',
    callCameraAccessDenied: 'ErrorBarCallCameraAccessDenied',
    callCameraAccessDeniedSafari: 'ErrorBarCallCameraAccessDenied',
    callCameraAlreadyInUse: 'ErrorBarCallCameraAlreadyInUse',
    callVideoStoppedBySystem: 'ErrorBarCallVideoStoppedBySystem',
    callVideoRecoveredBySystem: 'ErrorBarCallVideoRecoveredBySystem',
    callMacOsCameraAccessDenied: 'ErrorBarCallMacOsCameraAccessDenied',
    mutedByRemoteParticipant: 'ErrorBarMutedByRemoteParticipant',
    speakingWhileMuted: 'ErrorBarCallMicrophoneMutedBySystem',
    recordingStarted: 'NotificationBarRecording',
    transcriptionStarted: 'NotificationBarRecording',
    recordingStopped: 'NotificationBarRecording',
    transcriptionStopped: 'NotificationBarRecording',
    recordingAndTranscriptionStarted: 'NotificationBarRecording',
    recordingAndTranscriptionStopped: 'NotificationBarRecording',
    recordingStoppedStillTranscribing: 'NotificationBarRecording',
    transcriptionStoppedStillRecording: 'NotificationBarRecording',
    assignedBreakoutRoomOpened: 'NotificationBarBreakoutRoomOpened',
    assignedBreakoutRoomOpenedPromptJoin: 'NotificationBarBreakoutRoomPromptJoin',
    assignedBreakoutRoomChanged: 'NotificationBarBreakoutRoomChanged',
    assignedBreakoutRoomClosed: 'NotificationBarBreakoutRoomClosed',
    breakoutRoomJoined: 'NotificationBarBreakoutRoomJoined',
    breakoutRoomClosingSoon: 'NotificationBarBreakoutRoomClosingSoon',
    capabilityTurnVideoOnPresent: 'ControlButtonCameraOff',
    capabilityTurnVideoOnAbsent: 'ControlButtonCameraProhibited',
    capabilityUnmuteMicPresent: 'ControlButtonMicOff',
    capabilityUnmuteMicAbsent: 'ControlButtonMicProhibited',
    togetherModeStarted: 'NotificationBarTogetherModeIcon',
    togetherModeEnded: 'NotificationBarTogetherModeIcon',
    transcriptionError: 'NotificationBarTranscriptionError',
    transcriptionStartedByYou: 'NotificationBarTranscriptionStartedByYou'
};
/**
 * Chunk an array into rows of a given size.
 * @private
 */
function chunk(options, itemsPerRow) {
    const rows = [];
    let currentRow = [];
    for (const option of options) {
        currentRow.push(option);
        if (currentRow.length === itemsPerRow) {
            rows.push(currentRow);
            currentRow = [];
        }
    }
    if (currentRow.length > 0) {
        rows.push(currentRow);
    }
    return rows;
}
/**
 * @private
 */
const defaultSpokenLanguage = 'en-us';
/**
 * @private
 */
const SAFARI_COMPOSITION_KEYCODE = 229;
/**
 * Determine if the press of the enter key is from a composition session or not (Safari only)
 *
 * @private
 */
const isEnterKeyEventFromCompositionSession = (e) => {
    // Uses KeyCode 229 and which code 229 to determine if the press of the enter key is from a composition session or not (the code check is needed for Safari only, for everything else e.isComposing check is enough)
    const isComposing = e.isComposing || e.keyCode === SAFARI_COMPOSITION_KEYCODE || e.which === SAFARI_COMPOSITION_KEYCODE;
    return isComposing && e.key === 'Enter';
};
/**
 * @private
 */
const formatMoreButtonAriaDescription = (displayName, isMuted, isHandRaised, state, isSpeaking, strings, isMicDisabled, isCameraDisabled) => {
    const mutedState = isMuted ? strings === null || strings === void 0 ? void 0 : strings.moreOptionsParticipantMutedStateMutedAriaLabel : strings === null || strings === void 0 ? void 0 : strings.moreOptionsParticipantMutedStateUnmutedAriaLabel;
    const handRaisedState = isHandRaised ? strings === null || strings === void 0 ? void 0 : strings.moreOptionsParticipantHandRaisedAriaLabel : undefined;
    const isSpeakingState = isSpeaking ? strings === null || strings === void 0 ? void 0 : strings.moreOptionsParticipantIsSpeakingAriaLabel : undefined;
    const micDisabledState = isMicDisabled ? strings === null || strings === void 0 ? void 0 : strings.moreOptionsParticipantMicDisabledAriaLabel : undefined;
    const cameraDisabledState = isCameraDisabled ? strings === null || strings === void 0 ? void 0 : strings.moreOptionsParticipantCameraDisabledAriaLabel : undefined;
    const description = (strings === null || strings === void 0 ? void 0 : strings.moreOptionsButtonAriaLabel) ? _formatString(strings === null || strings === void 0 ? void 0 : strings.moreOptionsButtonAriaLabel, {
        displayName: displayName !== null && displayName !== void 0 ? displayName : ' ',
        isMuted: mutedState !== null && mutedState !== void 0 ? mutedState : ' ',
        isHandRaised: handRaisedState !== null && handRaisedState !== void 0 ? handRaisedState : ' ',
        state: state !== null && state !== void 0 ? state : ' ',
        isSpeaking: isSpeakingState !== null && isSpeakingState !== void 0 ? isSpeakingState : ' ',
        micDisabledState: micDisabledState !== null && micDisabledState !== void 0 ? micDisabledState : ' ',
        cameraDisabledState: cameraDisabledState !== null && cameraDisabledState !== void 0 ? cameraDisabledState : ' '
    }) : '';
    return description;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A component to show error messages on the UI.
 * All strings that can be shown are accepted as the {@link ErrorBarProps.strings} so that they can be localized.
 * Active errors are selected by {@link ErrorBarProps.activeErrorMessages}.
 *
 * This component internally tracks dismissed by the user.
 *   * Errors that have an associated timestamp: The error is shown on the UI again if it occurs after being dismissed.
 *   * Errors that do not have a timestamp: The error is dismissed until it disappears from the props.
 *         If the error recurs, it is shown in the UI.
 *
 * Uses {@link @fluentui/react#MessageBar} UI element.
 *
 * @public
 */
const ErrorBar = (props) => {
    var _a;
    const localeStrings = useLocale$1().strings.errorBar;
    const strings = (_a = props.strings) !== null && _a !== void 0 ? _a : localeStrings;
    const trackDismissedErrorsInternally = !props.onDismissError;
    // Timestamp for when this comopnent is first mounted.
    // Never updated through the lifecycle of this component.
    const mountTimestamp = React.useRef(new Date(Date.now()));
    const [dismissedErrors, setDismissedErrors] = React.useState([]);
    // dropDismissalsForInactiveErrors only returns a new object if `dismissedErrors` actually changes.
    // Without this behaviour, this `useEffect` block would cause a render loop.
    React.useEffect(() => {
        trackDismissedErrorsInternally && setDismissedErrors(dropDismissalsForInactiveErrors(props.activeErrorMessages, dismissedErrors));
    }, [props.activeErrorMessages, dismissedErrors, trackDismissedErrorsInternally]);
    const toShow = errorsToShow(props.activeErrorMessages, dismissedErrors, props.ignorePremountErrors ? mountTimestamp.current : undefined);
    return React.createElement(react.Stack, { "data-ui-id": "notifications-stack" }, toShow.map(error => React.createElement(react.MessageBar, Object.assign({}, props, { styles: {
            innerText: {
                alignSelf: 'center'
            },
            icon: {
                height: 0
            },
            content: {
                lineHeight: 'inherit'
            },
            dismissal: {
                height: '2rem',
                paddingBottom: '0.8rem'
            }
        }, key: error.type, messageBarType: messageBarType(error.type), messageBarIconProps: messageBarIconProps(error.type), onDismiss: () => { var _a; return trackDismissedErrorsInternally ? setDismissedErrors(dismissError(dismissedErrors, error)) : (_a = props.onDismissError) === null || _a === void 0 ? void 0 : _a.call(props, error); }, dismissButtonAriaLabel: `${strings[error.type]}, ${strings.dismissButtonAriaLabel}`, dismissIconProps: {
            iconName: 'ErrorBarClear'
        } }), strings[error.type])));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const gridLayoutStyle = react.mergeStyles({
    width: '100%',
    height: '100%',
    display: 'grid',
    gridGap: '0.5rem'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A component to lay out audio / video participants tiles in a call.
 *
 * @public
 */
const GridLayout = (props) => {
    const { children, styles } = props;
    const numberOfChildren = React.Children.count(children);
    const [currentWidth, setCurrentWidth] = React.useState(0);
    const [currentHeight, setCurrentHeight] = React.useState(0);
    const containerRef = React.useRef(null);
    const observer = React.useRef(new ResizeObserver((entries) => {
        if (!entries[0]) {
            return;
        }
        const { width, height } = entries[0].contentRect;
        setCurrentWidth(width);
        setCurrentHeight(height);
    }));
    React.useEffect(() => {
        if (containerRef.current) {
            observer.current.observe(containerRef.current);
        }
        const currentObserver = observer.current;
        return () => currentObserver.disconnect();
    }, [observer, containerRef]);
    const gridProps = React.useMemo(() => {
        return calculateGridProps(numberOfChildren, currentWidth, currentHeight);
    }, [numberOfChildren, currentWidth, currentHeight]);
    const cssGridStyles = React.useMemo(() => createGridStyles(numberOfChildren, gridProps), [numberOfChildren, gridProps]);
    return React.createElement("div", { ref: containerRef, className: react.mergeStyles(gridLayoutStyle, cssGridStyles, styles === null || styles === void 0 ? void 0 : styles.root), "data-ui-id": "grid-layout" }, children);
};
/**
 * The cell aspect ratio we aim for in a grid
 */
const TARGET_CELL_ASPECT_RATIO = 16 / 9;
/**
 * The minimum cell aspect ratio we allow
 */
const MINIMUM_CELL_ASPECT_RATIO_ALLOWED = 8 / 9;
const isCloserThan = (a, b, target) => {
    return Math.abs(target - a) < Math.abs(target - b);
};
/**
 * Get the best GridProps to place a number of items in a grid as evenly as possible given the width and height of the grid
 * @param numberOfItems - number of items to place in grid
 * @param width - width of grid
 * @param height - height of grid
 * @returns GridProps
 */
const calculateGridProps = (numberOfItems, width, height) => {
    if (numberOfItems <= 0) {
        return {
            fillDirection: 'horizontal',
            rows: 0,
            columns: 0
        };
    }
    // If width or height are 0 then we return rows and column evenly
    if (width <= 0 || height <= 0) {
        return {
            fillDirection: 'horizontal',
            rows: Math.ceil(Math.sqrt(numberOfItems)),
            columns: Math.ceil(Math.sqrt(numberOfItems))
        };
    }
    const aspectRatio = width / height;
    // Approximate how many rows to divide the grid to achieve cells close to the TARGET_CELL_ASPECT_RATIO
    let rows = Math.floor(Math.sqrt(TARGET_CELL_ASPECT_RATIO / aspectRatio * numberOfItems)) || 1;
    // Make sure rows do not exceed numberOfItems
    rows = Math.min(rows, numberOfItems);
    // Given the rows, get the minimum columns needed to create enough cells for the number of items
    let columns = Math.ceil(numberOfItems / rows);
    // Default fill direction to horizontal
    let fillDirection = 'horizontal';
    while (rows < numberOfItems) {
        // If cell aspect ratio is less than MINIMUM_CELL_ASPECT_RATIO_ALLOWED then try more rows
        if (rows / columns * aspectRatio < MINIMUM_CELL_ASPECT_RATIO_ALLOWED) {
            rows += 1;
            columns = Math.ceil(numberOfItems / rows);
            continue;
        }
        if (numberOfItems < rows * columns) {
            // We need to check that stretching columns vertically will result in only one less cell in stretched columns.
            // Likewise, we need to check that stretching rows horizonally will result in only one less cell in stretched rows.
            // e.g. For 4 rows, 2 columns, but only 6 items, we cannot stretch vertically because that would result in a
            // column of 2 cells which is less by more than 1 compared to the unstretched column.
            //  _________
            // |____|    |
            // |____|____|
            // |____|    |
            // |____|____|
            const canStretchVertically = numberOfItems >= rows + (columns - 1) * (rows - 1);
            const canStretchHorizontally = numberOfItems >= columns + (rows - 1) * (columns - 1);
            if (!canStretchVertically && !canStretchHorizontally) {
                rows += 1;
                columns = Math.ceil(numberOfItems / rows);
                continue;
            }
            else if (!canStretchVertically) {
                break;
            }
            else if (!canStretchHorizontally) {
                fillDirection = 'vertical';
                break;
            }
            // We need to figure out whether the big cells should stretch horizontally or vertically
            // to fill in the empty spaces
            // e.g. For 2 rows, 3 columns, but only 5 items, we need to choose whether to stretch cells
            //       horizontally            or           vertically
            //  ______________________               _______________________
            // |       |       |      |             |       |       |       |
            // |_______|_______|______|             |_______|_______|       |
            // |           |          |             |       |       |       |
            // |___________|__________|             |_______|_______|_______|
            // Calculate the aspect ratio of big cells stretched horizontally
            const horizontallyStretchedCellRatio = rows / (columns - 1) * aspectRatio;
            // Calculate the aspect ratio of big cells stretched vertically
            const verticallyStretchedCellRatio = (rows - 1) / columns * aspectRatio;
            // We know the horizontally stretched cells aspect ratio is higher than MINIMUM_CELL_ASPECT_RATIO_ALLOWED. If vertically stretched cells
            // is also higher than the MINIMUM_CELL_ASPECT_RATIO_ALLOWED, then choose which aspect ratio is better.
            if (verticallyStretchedCellRatio >= MINIMUM_CELL_ASPECT_RATIO_ALLOWED) {
                // If vertically stetched cell has an aspect ratio closer to TARGET_CELL_ASPECT_RATIO then change the fill direction to vertical
                if (isCloserThan(verticallyStretchedCellRatio, horizontallyStretchedCellRatio, TARGET_CELL_ASPECT_RATIO)) {
                    fillDirection = 'vertical';
                }
            }
        }
        break;
    }
    return {
        fillDirection,
        rows,
        columns
    };
};
/**
 * Creates a styles classname with CSS Grid related styles given GridProps and the number of items to distribute as evenly as possible.
 * @param numberOfItems - number of items to place in grid
 * @param gridProps - GridProps that define the number of rows, number of columns, and the fill direction
 * @returns - classname
 */
const createGridStyles = (numberOfItems, gridProps) => {
    const isHorizontal = gridProps.fillDirection === 'horizontal';
    // Blocks are either rows or columns depending on whether we fill horizontally or vertically. Each block may differ in the number of cells.
    const blocks = isHorizontal ? gridProps.rows : gridProps.columns;
    const smallCellsPerBlock = Math.ceil(numberOfItems / blocks);
    const bigCellsPerBlock = Math.floor(numberOfItems / blocks);
    const numBigCells = (gridProps.rows * gridProps.columns - numberOfItems) * bigCellsPerBlock;
    // Get grid units
    // e.g. If some blocks have 2 big cells while others have 3 small cells, we need to work with 6 units per block
    const units = smallCellsPerBlock * bigCellsPerBlock;
    const gridStyles = isHorizontal ? {
        gridTemplateColumns: `repeat(${units}, minmax(0, 1fr))`,
        gridTemplateRows: `repeat(${blocks}, minmax(0, 1fr))`,
        gridAutoFlow: 'row'
    } : {
        gridTemplateColumns: `repeat(${blocks}, minmax(0, 1fr))`,
        gridTemplateRows: `repeat(${units}, minmax(0, 1fr))`,
        gridAutoFlow: 'column'
    };
    const smallCellStyle = isHorizontal ? {
        '> *': {
            gridColumn: `auto / span ${units / smallCellsPerBlock}`
        }
    } : {
        '> *': {
            gridRow: `auto / span ${units / smallCellsPerBlock}`
        }
    };
    // If there are big cells, we are choosing to place the latest children into the big cells.
    // That is why we use the '> *:nth-last-child(-n + ${numBigCells})' CSS selector below
    const bigCellStyle = numBigCells ? {
        [`> *:nth-last-child(-n + ${numBigCells})`]: isHorizontal ? {
            gridColumn: `auto / span ${units / bigCellsPerBlock}`
        } : {
            gridRow: `auto / span ${units / bigCellsPerBlock}`
        }
    } : {};
    return react.mergeStyles(gridStyles, smallCellStyle, bigCellStyle);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const sendBoxWrapperStyles = react.mergeStyles({
    margin: '0.25rem',
    overflow: 'hidden',
    /**
     * margin-top set for all the child components of sendbox except first
     */
    ':not(:first-child)': {
        marginTop: '0.25rem'
    }
});
/**
 * @private
 */
const sendButtonStyle = react.mergeStyles({
    height: '2.25rem',
    width: '2.25rem'
});
/**
 * @private
 */
const sendIconStyle = (props) => {
    const { theme, isSendBoxButtonDisabled, customSendIconStyle, defaultTextColor = theme.palette.neutralTertiary } = props;
    return react.mergeStyles(editorTextBoxButtonStyle, {
        color: isSendBoxButtonDisabled ? defaultTextColor : theme.palette.themePrimary
    }, customSendIconStyle);
};
/**
 * @private
 */
const editorTextBoxButtonStyle = {
    width: '1.25rem',
    height: '1.25rem',
    margin: 'auto'
};
/**
 * @private
 */
react.mergeStyles({
    margin: '0 0.25rem 0.25rem 0.25rem',
    overflow: 'auto'
});
/**
 * @private
 */
reactComponents.makeStyles({
    clearBackground: {
        backgroundColor: 'transparent'
    }
});
/**
 * @private
 */
react.mergeStyles({
    width: '100%',
    padding: '0.50rem'
});
/**
 * @private
 */
const defaultSendBoxInactiveBorderThicknessREM = 0.0625;
/**
 * @private
 */
const defaultSendBoxActiveBorderThicknessREM = 0.125;
/**
 * @private
 */
const borderAndBoxShadowStyle = (props) => {
    return react.mergeStyles(borderEditBoxStyle(Object.assign(Object.assign({}, props), { defaultBorderColor: props.theme.palette.neutralSecondary })));
};
/**
 * @private
 */
const borderEditBoxStyle = (props) => {
    const { theme, hasErrorMessage, disabled, defaultBorderColor } = props;
    const borderColor = hasErrorMessage ? theme.semanticColors.errorText : defaultBorderColor;
    const borderColorActive = hasErrorMessage ? theme.semanticColors.errorText : theme.palette.themePrimary;
    const borderThickness = disabled ? 0 : defaultSendBoxInactiveBorderThicknessREM;
    const borderActiveThickness = disabled ? 0 : defaultSendBoxActiveBorderThicknessREM;
    return {
        borderRadius: theme.effects.roundedCorner4,
        border: `${borderThickness}rem solid ${borderColor}`,
        // The border thickness of the sendbox wrapper changes on hover, to prevent the border thickness change causing the
        // input box to shift we apply a margin to compensate. This margin is then removed on hover when the border is thicker.
        margin: `${defaultSendBoxActiveBorderThicknessREM - borderThickness}rem`,
        ':hover, :active, :focus, :focus-within': {
            border: `${borderActiveThickness}rem solid ${borderColorActive}`,
            margin: `${defaultSendBoxActiveBorderThicknessREM - borderActiveThickness}rem`
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Preset light theme for components exported from this library.
 *
 * @public
 */
const lightTheme = {
    palette: {
        themePrimary: '#0078d4',
        themeLighterAlt: '#eff6fc',
        themeLighter: '#deecf9',
        themeLight: '#c7e0f4',
        themeTertiary: '#71afe5',
        themeSecondary: '#2b88d8',
        themeDarkAlt: '#106ebe',
        themeDark: '#59b0f7',
        themeDarker: '#004578',
        neutralLighterAlt: '#faf9f8',
        neutralLighter: '#f3f2f1',
        neutralLight: '#edebe9',
        neutralQuaternaryAlt: '#e1dfdd',
        neutralQuaternary: '#d0d0d0',
        neutralTertiaryAlt: '#c8c6c4',
        neutralTertiary: '#a19f9d',
        neutralSecondary: '#605e5c',
        neutralPrimaryAlt: '#3b3a39',
        neutralPrimary: '#323130',
        neutralDark: '#201f1e',
        black: '#000000',
        white: '#ffffff',
        whiteTranslucent40: 'rgba(255, 255, 255, 0.4)'
    },
    callingPalette: {
        callRed: '#a42e43',
        callRedDark: '#8b2c3d',
        callRedDarker: '#772a38',
        iconWhite: '#ffffff',
        raiseHandGold: '#eaa300',
        videoTileLabelBackgroundLight: 'rgba(255,255,255,0.8)'
    },
    semanticColors: {
        errorText: '#a80000'
    }
};
const partialDarkTheme = {
    palette: {
        themePrimary: '#2899f5',
        themeLighterAlt: '#02060a',
        themeLighter: '#061827',
        themeLight: '#0c2e49',
        themeTertiary: '#185b93',
        themeSecondary: '#2286d7',
        themeDarkAlt: '#3ca2f6',
        themeDark: '#59b0f7',
        themeDarker: '#84c5f9',
        neutralLighterAlt: '#302e2d',
        neutralLighter: '#383735',
        neutralLight: '#464443',
        neutralQuaternaryAlt: '#4e4d4b',
        neutralQuaternary: '#4d4b49',
        neutralTertiaryAlt: '#72706e',
        neutralTertiary: '#c8c8c8',
        neutralSecondary: '#d0d0d0',
        neutralPrimaryAlt: '#dadada',
        neutralPrimary: '#ffffff',
        neutralDark: '#f4f4f4',
        black: '#f8f8f8',
        white: '#252423',
        whiteTranslucent40: 'rgba(0, 0, 0, 0.4)'
    },
    semanticColors: {
        errorText: '#f1707b'
    }
};
/**
 * Preset dark theme for components exported from this library.
 *
 * @public
 */
const darkTheme = Object.assign(Object.assign({}, react.createTheme(Object.assign(Object.assign({}, partialDarkTheme), { isInverted: true }))), { callingPalette: {
        callRed: '#c4314b',
        callRedDark: '#a42e43',
        callRedDarker: '#8b2c3d',
        iconWhite: '#ffffff',
        raiseHandGold: '#eaa300',
        videoTileLabelBackgroundLight: 'rgba(37,36,35,0.8)'
    } });

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const wrapper = react.mergeStyles({
    height: '100%',
    width: '100%',
    // Add NorthStar styling used previously in the library
    '*': {
        boxSizing: 'border-box'
    },
    '*:before': {
        boxSizing: 'border-box'
    },
    '*:after': {
        boxSizing: 'border-box'
    },
    /* Adding priority for HTML `hidden` attribute to be applied correctly */
    '[hidden]': {
        display: 'none!important'
    }
});
const defaultTheme = Object.assign({}, react.mergeThemes(react.getTheme(), lightTheme));
/** Theme context for library's react components */
const ThemeContext = React.createContext(defaultTheme);
/**
 * Provider to apply a Fluent theme across this library's react components.
 *
 * @remarks Components in this library are composed primarily from [Fluent UI](https://developer.microsoft.com/fluentui#/controls/web),
 * controls, mixing v8 and v9 controls.
 * This provider handles applying any theme provided to the underlying Fluent UI controls. *
 * @public
 */
const FluentThemeProvider = (props) => {
    const { fluentTheme, rtl, children, rootStyle } = props;
    const fluentV8Theme = React.useMemo(() => {
        const mergedTheme = react.mergeThemes(defaultTheme, fluentTheme);
        return react.mergeThemes(mergedTheme, {
            rtl
        });
    }, [fluentTheme, rtl]);
    return React.createElement(ThemeContext.Provider, { value: fluentV8Theme },
        React.createElement(react.ThemeProvider, { theme: fluentV8Theme, className: wrapper, style: rootStyle }, children));
};
/**
 * React hook to access theme
 *
 * @public
 */
const useTheme = () => React.useContext(ThemeContext);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const WifiWarning16Filled = () => 
// All ErrorBar icons are 16px x 16px (when 1rem = 16 px).
// There is no 16px version of this icon in the fluent icon package, so set height and width to 1rem.
React.createElement(reactIcons.WifiWarning20Filled, { style: {
        height: '1rem',
        width: '1rem'
    } });
const MoreHorizontal18Regular = () => 
// MoreHorizontal icons are 16px x 16px or 20px x 20px so scaling to get desired size
React.createElement("div", { className: react.mergeStyles({
        transform: 'scale(0.9)'
    }) },
    React.createElement(reactIcons.MoreHorizontal20Regular, null));
const GalleryLeftButton = () => {
    const rtl = useTheme().rtl;
    return rtl ? React.createElement(reactIcons.ChevronRight20Regular, null) : React.createElement(reactIcons.ChevronLeft20Regular, null);
};
const GalleryRightButton = () => {
    const rtl = useTheme().rtl;
    return rtl ? React.createElement(reactIcons.ChevronLeft20Regular, null) : React.createElement(reactIcons.ChevronRight20Regular, null);
};
const ControlButtonRaiseHandIcon = () => {
    return React.createElement(reactIcons.HandRight20Regular, null);
};
const ControlButtonLowerHandIcon = () => {
    return React.createElement(reactIcons.HandRightOff20Regular, null);
};
/**
 * The default set of icons that are available to use in the UI components.
 *
 * @remark Icons used only in the composites are available in {@link DEFAULT_COMPOSITE_ICONS}.
 *
 * @public
 */
const DEFAULT_COMPONENT_ICONS = {
    ChatMessageOptions: React.createElement(MoreHorizontal18Regular, null),
    ControlButtonCameraOff: React.createElement(reactIcons.VideoOff20Filled, null),
    ControlButtonCameraOn: React.createElement(reactIcons.Video20Filled, null),
    ControlButtonEndCall: React.createElement(reactIcons.CallEnd20Filled, null),
    ControlButtonMicOff: React.createElement(reactIcons.MicOff20Filled, null),
    ControlButtonMicOn: React.createElement(reactIcons.Mic20Filled, null),
    ControlButtonOptions: React.createElement(reactIcons.Settings20Filled, null),
    ControlButtonParticipants: React.createElement(reactIcons.People20Filled, null),
    ControlButtonParticipantsContextualMenuItem: React.createElement(reactIcons.People20Regular, null),
    ControlButtonScreenShareStart: React.createElement(reactIcons.ShareScreenStart20Filled, null),
    ControlButtonScreenShareStop: React.createElement(reactIcons.ShareScreenStop20Filled, null),
    ControlButtonRaiseHand: React.createElement(ControlButtonRaiseHandIcon, null),
    ControlButtonLowerHand: React.createElement(ControlButtonLowerHandIcon, null),
    ControlButtonMicProhibited: React.createElement(reactIcons.MicProhibited20Filled, null),
    ControlButtonMicProhibitedSmall: React.createElement(reactIcons.MicProhibited16Filled, null),
    ControlButtonCameraProhibited: React.createElement(reactIcons.VideoProhibited20Filled, null),
    ControlButtonCameraProhibitedSmall: React.createElement(reactIcons.VideoProhibited16Filled, null),
    RaiseHandContextualMenuItem: React.createElement(reactIcons.HandRight20Regular, null),
    LowerHandContextualMenuItem: React.createElement(reactIcons.HandRightOff20Regular, null),
    ReactionButtonIcon: React.createElement(reactIcons.Emoji20Regular, null),
    OpenAttachment: React.createElement(reactIcons.Open20Regular, null),
    EditBoxCancel: React.createElement(reactIcons.Dismiss20Regular, null),
    EditBoxSubmit: React.createElement(reactIcons.Checkmark20Regular, null),
    ErrorBarCallCameraAccessDenied: React.createElement(reactIcons.VideoProhibited16Filled, null),
    ErrorBarCallCameraAlreadyInUse: React.createElement(reactIcons.VideoProhibited16Filled, null),
    ErrorBarCallLocalVideoFreeze: React.createElement(WifiWarning16Filled, null),
    ErrorBarCallMacOsCameraAccessDenied: React.createElement(reactIcons.VideoProhibited16Filled, null),
    ErrorBarCallMacOsMicrophoneAccessDenied: React.createElement(reactIcons.MicProhibited16Filled, null),
    ErrorBarCallMicrophoneAccessDenied: React.createElement(reactIcons.MicProhibited16Filled, null),
    ErrorBarCallMicrophoneMutedBySystem: React.createElement(reactIcons.MicOff16Regular, null),
    ErrorBarCallMicrophoneUnmutedBySystem: React.createElement(reactIcons.Mic16Filled, null),
    ErrorBarCallNetworkQualityLow: React.createElement(WifiWarning16Filled, null),
    ErrorBarCallNoMicrophoneFound: React.createElement(reactIcons.MicProhibited16Filled, null),
    ErrorBarCallNoSpeakerFound: React.createElement(reactIcons.SpeakerMute16Filled, null),
    ErrorBarClear: React.createElement(reactIcons.Dismiss16Regular, null),
    ErrorBarCallVideoRecoveredBySystem: React.createElement(reactIcons.Video16Filled, null),
    ErrorBarCallVideoStoppedBySystem: React.createElement(reactIcons.VideoProhibited16Filled, null),
    ErrorBarMutedByRemoteParticipant: React.createElement(reactIcons.MicOff16Regular, null),
    NotificationBarRecording: React.createElement(reactIcons.Record16Regular, null),
    NotificationBarBreakoutRoomOpened: React.createElement(reactIcons.DoorArrowRight16Regular, null),
    NotificationBarBreakoutRoomPromptJoin: React.createElement(reactIcons.DoorArrowRight16Regular, null),
    NotificationBarBreakoutRoomChanged: React.createElement(reactIcons.DoorArrowRight16Regular, null),
    NotificationBarBreakoutRoomJoined: React.createElement(reactIcons.ConferenceRoom16Regular, null),
    NotificationBarBreakoutRoomClosingSoon: React.createElement(reactIcons.DoorArrowLeft16Regular, null),
    NotificationBarBreakoutRoomClosed: React.createElement(reactIcons.DoorArrowLeft16Regular, null),
    NotificationBarTranscriptionError: React.createElement(reactIcons.Warning16Regular, null),
    NotificationBarTranscriptionStartedByYou: React.createElement(reactIcons.SlideTextEdit16Regular, null),
    HorizontalGalleryLeftButton: React.createElement(GalleryLeftButton, null),
    HorizontalGalleryRightButton: React.createElement(GalleryRightButton, null),
    MessageDelivered: React.createElement(reactIcons.CheckmarkCircle16Regular, null),
    MessageEdit: React.createElement(reactIcons.Edit20Regular, null),
    MessageFailed: React.createElement(reactIcons.ErrorCircle16Regular, null),
    MessageRemove: React.createElement(reactIcons.Delete20Regular, null),
    MessageResend: React.createElement(reactIcons.ArrowClockwise16Regular, null),
    MessageSeen: React.createElement(reactIcons.Eye16Regular, null),
    MessageSending: React.createElement(reactIcons.Circle16Regular, null),
    OptionsCamera: React.createElement(reactIcons.Video20Regular, null),
    OptionsMic: React.createElement(reactIcons.Mic20Regular, null),
    OptionsSpeaker: React.createElement(reactIcons.Speaker220Regular, null),
    ParticipantItemMicOff: React.createElement(reactIcons.MicOff20Regular, null),
    ParticipantItemOptions: React.createElement(React.Fragment, null),
    ParticipantItemOptionsHovered: React.createElement(reactIcons.MoreHorizontal20Filled, null),
    ParticipantItemScreenShareStart: React.createElement(reactIcons.ShareScreenStart20Filled, null),
    ParticipantItemSpotlighted: React.createElement(reactIcons.VideoPersonStar20Filled, null),
    HoldCallContextualMenuItem: React.createElement(reactIcons.CallPause20Regular, null),
    HoldCallButton: React.createElement(reactIcons.CallPause20Filled, null),
    ResumeCall: React.createElement(reactIcons.Play20Regular, null),
    SendBoxSend: React.createElement(reactIcons.Send20Regular, null),
    SendBoxSendHovered: React.createElement(reactIcons.Send20Filled, null),
    VideoTileMicOff: React.createElement(reactIcons.MicOff16Filled, null),
    VideoTileCameraOff: React.createElement(reactIcons.VideoProhibited16Filled, null),
    DialpadBackspace: React.createElement(reactIcons.Backspace20Regular, null),
    VideoTilePinned: React.createElement(reactIcons.Pin16Filled, null),
    ParticipantItemPinned: React.createElement(reactIcons.Pin20Filled, null),
    VideoTileMoreOptions: React.createElement(reactIcons.MoreHorizontal20Filled, null),
    VideoTileScaleFit: React.createElement(reactIcons.ScaleFit20Regular, null),
    VideoTileScaleFill: React.createElement(reactIcons.ScaleFill20Regular, null),
    PinParticipant: React.createElement(reactIcons.Pin20Regular, null),
    UnpinParticipant: React.createElement(reactIcons.PinOff20Regular, null),
    SplitButtonPrimaryActionCameraOn: React.createElement(reactIcons.Video20Filled, null),
    SplitButtonPrimaryActionCameraOff: React.createElement(reactIcons.VideoOff20Filled, null),
    SplitButtonPrimaryActionMicUnmuted: React.createElement(reactIcons.Mic20Filled, null),
    SplitButtonPrimaryActionMicMuted: React.createElement(reactIcons.MicOff20Filled, null),
    VerticalGalleryLeftButton: React.createElement(GalleryLeftButton, null),
    VerticalGalleryRightButton: React.createElement(GalleryRightButton, null),
    ControlButtonVideoEffectsOption: React.createElement(reactIcons.VideoBackgroundEffect20Regular, null),
    ConfigurationScreenVideoEffectsButton: React.createElement(reactIcons.VideoBackgroundEffect20Filled, null),
    CaptionsIcon: React.createElement(reactIcons.ClosedCaption20Regular, null),
    CaptionsOffIcon: React.createElement(reactIcons.ClosedCaptionOff20Regular, null),
    CaptionsSettingsIcon: React.createElement(reactIcons.Settings20Regular, null),
    ChangeSpokenLanguageIcon: React.createElement(reactIcons.PersonVoice20Regular, null),
    ChangeCaptionLanguageIcon: React.createElement(reactIcons.Translate20Regular, null),
    ContextMenuCameraIcon: React.createElement(reactIcons.Video20Regular, null),
    ContextMenuMicIcon: React.createElement(reactIcons.Mic20Regular, null),
    ContextMenuSpeakerIcon: React.createElement(reactIcons.Speaker220Regular, null),
    ContextMenuRemoveParticipant: React.createElement(reactIcons.PersonDelete20Regular, null),
    SurveyStarIcon: React.createElement(reactIcons.Star28Regular, null),
    SurveyStarIconFilled: React.createElement(reactIcons.Star28Filled, null),
    StartSpotlightContextualMenuItem: React.createElement(reactIcons.VideoPersonStar20Filled, null),
    StopSpotlightContextualMenuItem: React.createElement(reactIcons.VideoPersonStarOff20Filled, null),
    VideoTileSpotlighted: React.createElement(reactIcons.VideoPersonStar20Filled, { style: {
            height: '16px',
            width: '16px'
        } }),
    ContextualMenuMicMutedIcon: React.createElement(reactIcons.MicOff20Regular, null),
    IncomingCallNotificationRejectIcon: React.createElement(reactIcons.CallEnd20Filled, null),
    IncomingCallNotificationAcceptIcon: React.createElement(reactIcons.Call20Filled, null),
    IncomingCallNotificationAcceptWithVideoIcon: React.createElement(reactIcons.Video20Filled, null),
    NotificationBarTogetherModeIcon: React.createElement(reactIcons.PeopleAudience20Regular, null),
    RealTimeTextIcon: React.createElement(reactIcons.SlideTextCall20Regular, null),
    ExpandIcon: React.createElement(reactIcons.ArrowMaximize20Regular, null),
    MinimizeIcon: React.createElement(reactIcons.ArrowMinimize20Regular, null)
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const inputBoxWrapperStyle = react.mergeStyles({
    padding: '0'
});
/**
 * @private
 */
const inputBoxStyle = react.mergeStyles({
    overflow: 'auto',
    minHeight: '2.25rem',
    // prevents the input text box from being sized to 0px when the CallWithChatComposite chat pane is closed.
    maxHeight: '8.25rem',
    outline: 'red 5px',
    fontWeight: react.FontWeights.regular,
    fontSize: '0.875rem',
    width: '100%',
    height: '2.25rem',
    lineHeight: '1.5rem',
    '::-webkit-input-placeholder': {
        fontSize: '1rem' // Prevents screen zooming on iOS
    },
    '::-moz-placeholder': {
        fontSize: '0.875rem'
    },
    ':-moz-placeholder': {
        fontSize: '0.875rem'
    }
});
/**
 *
 * @private
 */
const textContainerStyle = {
    alignSelf: 'center',
    position: 'relative',
    width: '100%'
};
/**
 * @private
 */
const textFieldStyle = {
    root: {
        width: '100%',
        minHeight: '0',
        fontSize: '8.25rem'
    },
    wrapper: {},
    fieldGroup: {
        outline: 'none',
        border: 'none',
        height: 'auto',
        minHeight: '0',
        /**
         * Removing fieldGroup border to make sure no border is rendered around the text area in sendbox.
         */
        ':after': {
            border: 'none'
        }
    },
    field: {
        borderRadius: '0.25rem'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const InputBoxComponent = (props) => {
    const { styles, id, 'data-ui-id': dataUiId, textValue, onChange, textFieldRef, placeholderText, onKeyDown, onEnterKeyDown, supportNewline, inputClassName, errorMessage, disabled, children } = props;
    const mergedRootStyle = react.mergeStyles(inputBoxWrapperStyle, styles === null || styles === void 0 ? void 0 : styles.root);
    const mergedInputFieldStyle = react.mergeStyles(inputBoxStyle, inputClassName);
    const mergedTextContainerStyle = react.mergeStyles(textContainerStyle, styles === null || styles === void 0 ? void 0 : styles.textFieldContainer);
    const mergedTextFieldStyle = react.concatStyleSets(textFieldStyle, {
        fieldGroup: styles === null || styles === void 0 ? void 0 : styles.textField,
        errorMessage: styles === null || styles === void 0 ? void 0 : styles.systemMessage,
        suffix: {
            backgroundColor: 'transparent',
            padding: '0 0'
        }
    });
    const onTextFieldKeyDown = React.useCallback((ev) => {
        if (isEnterKeyEventFromCompositionSession(ev.nativeEvent)) {
            return;
        }
        if (ev.key === 'Enter' && (ev.shiftKey === false || !supportNewline)) {
            ev.preventDefault();
            onEnterKeyDown && onEnterKeyDown();
        }
        onKeyDown && onKeyDown(ev);
    }, [onEnterKeyDown, onKeyDown, supportNewline]);
    const onRenderChildren = () => {
        return React.createElement(React.Fragment, null, children);
    };
    const renderTextField = () => {
        const textFieldProps = {
            autoFocus: props.autoFocus === 'sendBoxTextField',
            multiline: true,
            autoAdjustHeight: true,
            multiple: false,
            resizable: false,
            componentRef: textFieldRef,
            id,
            inputClassName: mergedInputFieldStyle,
            placeholder: placeholderText,
            autoComplete: 'off',
            styles: mergedTextFieldStyle,
            disabled,
            errorMessage,
            onRenderSuffix: props.children ? onRenderChildren : undefined
        };
        return React.createElement("div", { style: textFieldProps.errorMessage ? {
                padding: '0 0 5px 5px'
            } : undefined },
            React.createElement(react.TextField, Object.assign({}, textFieldProps, { "data-ui-id": dataUiId, value: textValue, onChange: onChange, onKeyDown: onTextFieldKeyDown, onFocus: e => {
                    // Fix for setting the cursor to the correct position when multiline is true
                    // This approach should be reviewed during migration to FluentUI v9
                    e.currentTarget.value = '';
                    e.currentTarget.value = textValue;
                } })));
    };
    return React.createElement(react.Stack, { className: mergedRootStyle },
        React.createElement("div", { className: mergedTextContainerStyle }, renderTextField()));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const isDarkThemed = (theme) => {
    const themeBlackBrightness = getPerceptualBrightnessOfHexColor(theme.palette.black);
    const themeWhiteBrightness = getPerceptualBrightnessOfHexColor(theme.palette.white);
    if (Number.isNaN(themeBlackBrightness) || Number.isNaN(themeWhiteBrightness)) {
        return false;
    }
    return themeBlackBrightness > themeWhiteBrightness;
};
const getPerceptualBrightnessOfHexColor = (hexColor) => {
    // return NaN if hexColor is not a hex code
    if (!/^#[0-9A-Fa-f]{6}$/i.test(hexColor)) {
        return NaN;
    }
    const r = parseInt(hexColor.substring(1, 3), 16);
    const g = parseInt(hexColor.substring(3, 5), 16);
    const b = parseInt(hexColor.substring(5, 7), 16);
    // arithmetic mean μ of the red, green, and blue color coordinates. Source: https://en.wikipedia.org/wiki/Brightness
    return (r + g + b) / 3;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const inputBoxButtonStyle = react.mergeStyles({
    color: 'grey',
    margin: 'auto',
    width: '1.0625rem',
    height: '1.0625rem',
    backgroundColor: 'transparent',
    '&:hover': {
        backgroundColor: 'transparent'
    }
});
/**
 * @private
 */
const inputBoxButtonTooltipStyle = react.mergeStyles({
    // The toolTip host container show be a flex box, so that alignItems: 'center' works for inside buttons
    display: 'flex'
});
/**
 * @private
 */
const iconWrapperStyle$1 = react.mergeStyles({
    pointerEvents: 'none'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const InputBoxButton = (props) => {
    const { onRenderIcon, onClick, ariaLabel, className, id, tooltipContent, 'data-testId': dataTestId, disabled, ariaExpanded } = props;
    const [isHover, setIsHover] = React.useState(false);
    const mergedButtonStyle = react.mergeStyles(inputBoxButtonStyle, className);
    const theme = react.useTheme();
    const calloutStyle = {
        root: {
            padding: 0
        },
        calloutMain: {
            padding: '0.5rem'
        }
    };
    // Place callout with no gap between it and the button.
    const calloutProps = {
        gapSpace: 0,
        styles: calloutStyle,
        backgroundColor: isDarkThemed(theme) ? theme.palette.neutralLighter : ''
    };
    return React.createElement(react.TooltipHost, { hostClassName: inputBoxButtonTooltipStyle, content: tooltipContent, calloutProps: Object.assign({}, calloutProps) },
        React.createElement(react.IconButton, { className: mergedButtonStyle, ariaLabel: ariaLabel, onClick: onClick, id: id, onMouseEnter: () => {
                setIsHover(true);
            }, onMouseLeave: () => {
                setIsHover(false);
            }, 
            // VoiceOver fix: Avoid icon from stealing focus when IconButton is double-tapped to send message by wrapping with Stack with pointerEvents style to none
            onRenderIcon: () => React.createElement(react.Stack, { className: iconWrapperStyle$1 }, onRenderIcon(isHover)), "data-testid": dataTestId, "aria-expanded": ariaExpanded, disabled: disabled, allowDisabledFocus: true }));
};

(window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const MAXIMUM_LENGTH_OF_MESSAGE = 8000;
const EMPTY_MESSAGE_REGEX = /^\s*$/;
/**
 * @private
 */
const isMessageTooLong = (valueLength) => {
    return valueLength > MAXIMUM_LENGTH_OF_MESSAGE;
};
/**
 * @private
 */
const sanitizeText = (message) => {
    if (EMPTY_MESSAGE_REGEX.test(message)) {
        return '';
    }
    else {
        return message;
    }
};
/**
 * Determines whether the send box should be disabled.
 *
 * @param hasContent - Indicates whether the send box has content.
 * @param hasCompletedAttachmentUploads - Indicates whether attachment uploads have completed.
 * @param hasError - Indicates whether there is an error.
 * @param disabled - Indicates whether the send box is disabled.
 * @returns A boolean value indicating whether the send box should be disabled.
 */
const isSendBoxButtonDisabled = ({ hasContent, hasError, disabled }) => {
    return (
    // no content
    !hasContent ||
        //error message exists
        hasError || disabled);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component for typing and sending messages.
 *
 * Supports sending typing notification when user starts entering text.
 * Supports an optional message below the text input field.
 *
 * @public
 */
const SendBox = (props) => {
    const { disabled, systemMessage, supportNewline, onSendMessage, onTyping, onRenderIcon, onRenderSystemMessage, styles, autoFocus } = props;
    const theme = useTheme();
    const localeStrings = useLocale$1().strings.sendBox;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const ids = useIdentifiers();
    const [textValue, setTextValue] = React.useState('');
    const [textValueOverflow, setTextValueOverflow] = React.useState(false);
    const sendTextFieldRef = React.useRef(null);
    const sendMessageOnClick = () => {
        var _a;
        // don't send a message when disabled
        if (disabled || textValueOverflow) {
            return;
        }
        // Don't send message until all attachments have been uploaded successfully
        const message = textValue;
        // we don't want to send empty messages including spaces, newlines, tabs
        // Message can be empty if there is a valid attachment upload
        if (sanitizeText(message).length > 0) {
            onSendMessage && onSendMessage(message);
            setTextValue('');
            (_a = sendTextFieldRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        }
    };
    const setText = (newValue) => {
        if (newValue === undefined) {
            return;
        }
        setTextValueOverflow(isMessageTooLong(newValue.length));
        setTextValue(newValue);
    };
    const textTooLongMessage = textValueOverflow ? strings.textTooLong : undefined;
    const errorMessage = systemMessage !== null && systemMessage !== void 0 ? systemMessage : textTooLongMessage;
    const isSendBoxButtonDisabledValue = React.useMemo(() => {
        return isSendBoxButtonDisabled({
            hasContent: sanitizeText(textValue).length > 0,
            hasError: !!errorMessage,
            disabled: !!disabled
        });
    }, [disabled, errorMessage, textValue]);
    const mergedSendButtonStyle = React.useMemo(() => react.mergeStyles(sendButtonStyle, styles === null || styles === void 0 ? void 0 : styles.sendMessageIconContainer), [styles === null || styles === void 0 ? void 0 : styles.sendMessageIconContainer]);
    const mergedStyles = React.useMemo(() => react.concatStyleSets(styles), [styles]);
    const mergedSendIconStyle = React.useMemo(() => sendIconStyle({
        theme,
        isSendBoxButtonDisabled: isSendBoxButtonDisabledValue,
        customSendIconStyle: styles === null || styles === void 0 ? void 0 : styles.sendMessageIcon
    }), [theme, isSendBoxButtonDisabledValue, styles === null || styles === void 0 ? void 0 : styles.sendMessageIcon]);
    const onRenderSendIcon = React.useCallback((isHover) => onRenderIcon ? onRenderIcon(isHover) : React.createElement(react.Icon, { iconName: isHover && textValue ? 'SendBoxSendHovered' : 'SendBoxSend', className: mergedSendIconStyle }), [mergedSendIconStyle, onRenderIcon, textValue]);
    // Ensure that errors are cleared when there are no attachments in sendBox
    return React.createElement(react.Stack, { className: react.mergeStyles(sendBoxWrapperStyles, {
            overflow: 'visible'
        } // This is needed for the mention popup to be visible
        ) },
        React.createElement(react.Stack, { className: borderAndBoxShadowStyle({
                theme,
                hasErrorMessage: !!errorMessage,
                disabled: !!disabled
            }) },
            React.createElement(InputBoxComponent, { autoFocus: autoFocus, "data-ui-id": ids.sendboxTextField, disabled: disabled, errorMessage: onRenderSystemMessage ? onRenderSystemMessage(errorMessage) : errorMessage, textFieldRef: sendTextFieldRef, id: "sendbox", placeholderText: strings.placeholderText, textValue: textValue, onChange: (_, newValue) => setText(newValue), onKeyDown: ev => {
                    const keyWasSendingMessage = ev.key === 'Enter' && (ev.shiftKey === false || !supportNewline);
                    if (!keyWasSendingMessage) {
                        onTyping === null || onTyping === void 0 ? void 0 : onTyping();
                    }
                }, onEnterKeyDown: () => {
                    sendMessageOnClick();
                }, styles: mergedStyles, supportNewline: supportNewline, maxLength: MAXIMUM_LENGTH_OF_MESSAGE },
                React.createElement(InputBoxButton, { onRenderIcon: onRenderSendIcon, onClick: e => {
                        if (!textValueOverflow) {
                            sendMessageOnClick();
                        }
                        e.stopPropagation();
                    }, id: 'sendIconWrapper', className: mergedSendButtonStyle, ariaLabel: localeStrings.sendButtonAriaLabel, tooltipContent: localeStrings.sendButtonAriaLabel, disabled: isSendBoxButtonDisabledValue }))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const cancelIcon = {
    iconName: 'Cancel'
};
/**
 * @private
 */
const downloadIcon = {
    iconName: 'Download'
};
/**
 * @private
 */
const overlayStyles$1 = (theme) => {
    var _a;
    return {
        root: {
            background: (_a = theme.semanticColors) === null || _a === void 0 ? void 0 : _a.bodyBackground
        }
    };
};
/**
 * @private
 */
const focusTrapZoneStyle = {
    boxShadow: 'none',
    background: 'transparent',
    display: 'flex',
    width: '100%',
    height: '100%',
    maxWidth: '100%',
    maxHeight: '100%'
};
/**
 * @private
 */
const scrollableContentStyle = {
    overflowY: 'hidden',
    display: 'flex',
    maxWidth: '100%',
    maxHeight: '100%',
    flexDirection: 'column',
    flexWrap: 'nowrap'
};
/**
 * @private
 */
const themeProviderRootStyle = {
    background: 'transparent',
    display: 'flex',
    flexDirection: 'column'
};
/**
 * @private
 */
const headerStyle = {
    fontSize: 'inherit',
    margin: '0',
    width: '100%',
    minHeight: '3.5rem',
    flexDirection: 'row',
    justifyContent: 'space-between',
    padding: '0.25rem 0.75rem'
};
/**
 * @private
 */
const titleBarContainerStyle = {
    flexDirection: 'row',
    justifyContent: 'start',
    alignContent: 'center',
    alignItems: 'center'
};
/**
 * @private
 */
const titleStyle$2 = (theme) => {
    var _a;
    return {
        paddingLeft: '0.5rem',
        color: (_a = theme.palette) === null || _a === void 0 ? void 0 : _a.black,
        fontFamily: 'inherit',
        fontSize: '0.875rem',
        fontStyle: 'normal',
        fontWeight: '600',
        lineHeight: '1.25rem'
    };
};
/**
 * @private
 */
const controlBarContainerStyle = {
    flexDirection: 'row',
    justifyContent: 'start',
    flexWrap: 'wrap',
    alignContent: 'center',
    alignItems: 'center'
};
/**
 * @private
 */
const downloadIconStyle = {
    marginRight: '0.5em',
    fontSize: '0.875rem' // 14px
};
/**
 * @private
 */
const bodyContainer = {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    width: '100%',
    height: '100%',
    overflow: 'hidden',
    padding: '1rem 2rem 3rem 2rem',
    '@media (max-width: 25rem) or (max-height: 25rem)': {
        padding: '0rem 1rem 2rem 1rem'
    }
};
/**
 * @private
 */
const normalImageStyle = {
    objectFit: 'contain',
    maxHeight: '100%',
    maxWidth: '100%'
};
/**
 * @private
 */
const brokenImageStyle = (theme) => {
    var _a;
    return {
        color: (_a = theme.palette) === null || _a === void 0 ? void 0 : _a.black
    };
};
/**
 * @private
 */
const closeButtonStyles = (theme) => {
    var _a, _b, _c;
    return {
        color: (_a = theme.palette) === null || _a === void 0 ? void 0 : _a.black,
        ':hover': {
            color: (_b = theme.palette) === null || _b === void 0 ? void 0 : _b.black
        },
        ':active': {
            color: (_c = theme.palette) === null || _c === void 0 ? void 0 : _c.black
        }
    };
};
/**
 * @private
 */
const downloadButtonStyle = {
    margin: '0 0.5rem',
    height: '32px',
    borderWidth: '1px',
    fontSize: '0.875rem',
    // 14px
    fontWeight: 600,
    padding: '0.38rem 0.75rem',
    borderRadius: '4px',
    whiteSpace: 'nowrap',
    '@media (max-width: 25rem)': {
        display: 'none'
    }
};
/**
 * @private
 */
const smallDownloadButtonContainerStyle = (theme) => {
    var _a, _b, _c;
    return {
        marginRight: '0.5rem',
        whiteSpace: 'nowrap',
        color: (_a = theme.palette) === null || _a === void 0 ? void 0 : _a.black,
        ':hover': {
            color: (_b = theme.palette) === null || _b === void 0 ? void 0 : _b.black
        },
        ':active': {
            color: (_c = theme.palette) === null || _c === void 0 ? void 0 : _c.black
        },
        '@media (min-width: 25rem)': {
            display: 'none'
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * Announcer component to maker aria announcements on actions
 */
const Announcer$1 = (props) => {
    const { announcementString, ariaLive } = props;
    return React.createElement(react.Stack, { "aria-label": announcementString, "aria-live": ariaLive, role: "status", "aria-atomic": true, styles: announcerStyles });
};
/**
 * Styles to hide the announcer from view but still existing on the DOM tree it so that narration can happen.
 */
const announcerStyles = {
    root: {
        position: 'absolute',
        width: '1px',
        height: '1px',
        padding: 0,
        margin: '-1px',
        overflow: 'hidden',
        clip: 'rect(0,0,0,0)',
        whiteSpace: 'nowrap',
        border: 0
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component to render a fullscreen modal for a selected image.
 *
 * @public
 */
const ImageOverlay = (props) => {
    const { isOpen, imageSrc, title, titleIcon, altText, onDownloadButtonClicked, onDismiss } = props;
    const localeStrings = useLocale$1().strings.imageOverlay;
    const [isImageLoaded, setIsImageLoaded] = React.useState(true);
    const overlayTheme = React.useMemo(() => {
        return Object.assign(Object.assign({}, darkTheme), { semanticColors: Object.assign(Object.assign({}, darkTheme.semanticColors), { bodyBackground: 'rgba(0, 0, 0, 0.85)' }) });
    }, []);
    const imageStyle = isImageLoaded ? normalImageStyle : brokenImageStyle(overlayTheme);
    const renderHeaderBar = () => {
        return React.createElement(react.Stack, { className: react.mergeStyles(headerStyle), role: "heading", "aria-label": title || 'Image', "aria-level": 2 },
            React.createElement(react.Stack, { className: react.mergeStyles(titleBarContainerStyle) },
                titleIcon,
                React.createElement(react.Stack.Item, { className: react.mergeStyles(titleStyle$2(overlayTheme)) }, title)),
            React.createElement(react.Stack, { className: react.mergeStyles(controlBarContainerStyle) },
                onDownloadButtonClicked && React.createElement(react.DefaultButton, { className: react.mergeStyles(downloadButtonStyle), text: localeStrings.downloadButtonLabel, onClick: () => onDownloadButtonClicked && onDownloadButtonClicked(imageSrc), onRenderIcon: () => React.createElement(react.Icon, { iconName: downloadIcon.iconName, className: react.mergeStyles(downloadIconStyle) }), "aria-live": 'polite', "aria-label": localeStrings.downloadButtonLabel, disabled: imageSrc === '' }),
                onDownloadButtonClicked && React.createElement(react.IconButton, { iconProps: downloadIcon, className: react.mergeStyles(smallDownloadButtonContainerStyle(overlayTheme)), onClick: () => onDownloadButtonClicked && onDownloadButtonClicked(imageSrc), "aria-label": localeStrings.downloadButtonLabel, "aria-live": 'polite', disabled: imageSrc === '' }),
                React.createElement(react.IconButton, { iconProps: cancelIcon, className: react.mergeStyles(closeButtonStyles(overlayTheme)), onClick: onDismiss, "aria-label": localeStrings.dismissButtonAriaLabel, "aria-live": 'polite' })));
    };
    const renderBodyWithLightDismiss = () => {
        return React.createElement(react.Stack, { className: react.mergeStyles(bodyContainer), onClick: () => props.onDismiss() }, imageSrc && React.createElement("img", { src: imageSrc, className: react.mergeStyles(imageStyle), alt: altText || 'image', "aria-label": 'image-overlay-main-image', "data-ui-id": 'image-overlay-main-image', "aria-live": 'polite', onError: () => {
                setIsImageLoaded(false);
            }, onClick: event => event.stopPropagation(), onDoubleClick: event => {
                event.persist();
            } }));
    };
    return React.createElement(react.Modal, { titleAriaId: title, isOpen: isOpen, onDismiss: onDismiss, overlay: {
            styles: Object.assign({}, overlayStyles$1(overlayTheme))
        }, styles: {
            main: focusTrapZoneStyle,
            scrollableContent: scrollableContentStyle
        }, isDarkOverlay: true },
        React.createElement(Announcer$1, { ariaLive: 'polite', announcementString: localeStrings.overlayVisibleMessage }),
        React.createElement(FluentThemeProvider, { fluentTheme: overlayTheme, rootStyle: themeProviderRootStyle },
            renderHeaderBar(),
            renderBodyWithLightDismiss()));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const MESSAGE_STATUS_INDICATOR_SIZE_REM = 1;
/**
 * ErrorIcon seems designed slightly smaller than other icons we try to match the size and then fix positioning here.
 *
 * @private
 */
const MessageStatusIndicatorErrorIconStyle = react.mergeStyles({
    marginRight: '-0.06rem',
    fontSize: '1.06rem'
});
/**
 * @private
 */
const MessageStatusIndicatorIconStyle = react.mergeStyles({
    fontSize: `${MESSAGE_STATUS_INDICATOR_SIZE_REM}rem`,
    width: `${MESSAGE_STATUS_INDICATOR_SIZE_REM}rem`,
    height: `${MESSAGE_STATUS_INDICATOR_SIZE_REM}rem`
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/// Adapted from: https://github.com/AlmeroSteyn/react-aria-live/blob/master/src/modules/AnnouncerContext.js
/** @private */
const AnnouncerContext = React.createContext({
    announceAssertive: logContextWarning,
    announcePolite: logContextWarning
});
function logContextWarning() {
    console.warn('Announcement failed, LiveAnnouncer context is missing');
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/// Adapted from: https://github.com/AlmeroSteyn/react-aria-live/blob/master/src/modules/AnnouncerMessage.js
/** @private */
const AnnouncerMessage = (props) => {
    const { message, ariaLive, clearOnUnmount, announceAssertive, announcePolite } = props;
    const announce = React.useCallback(() => {
        if (ariaLive === 'assertive') {
            announceAssertive(message || '', uuid.v1());
        }
        else if (ariaLive === 'polite') {
            announcePolite(message || '', uuid.v1());
        }
    }, [announceAssertive, announcePolite, ariaLive, message]);
    React.useEffect(() => {
        announce();
        return () => {
            if (clearOnUnmount) {
                announceAssertive('', uuid.v1());
                announcePolite('', uuid.v1());
            }
        };
    }, [message, clearOnUnmount, announce, announceAssertive, announcePolite]);
    return null;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/// Adapted from: https://github.com/AlmeroSteyn/react-aria-live/blob/master/src/modules/LiveMessage.js
/** @private */
const LiveMessage = (props) => React.createElement(AnnouncerContext.Consumer, null, contextProps => React.createElement(AnnouncerMessage, Object.assign({}, contextProps, props)));

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component to display message status icon
 *
 * @internal
 */
const MessageStatusIcon = (props) => {
    const { shouldAnnounce, iconName, iconClassName, ariaLabel } = props;
    return React.createElement(React.Fragment, null,
        ariaLabel && shouldAnnounce && React.createElement(LiveMessage, { message: ariaLabel, ariaLive: "polite" }),
        React.createElement("div", null,
            React.createElement(react.Icon, { role: 'status', "aria-live": shouldAnnounce ? 'polite' : 'off', "data-ui-id": 'chat-composite-message-status-icon', "aria-label": ariaLabel, iconName: iconName, className: iconClassName })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component to display the status of a sent message.
 *
 * Adds an icon and tooltip corresponding to the message status.
 *
 * @internal
 */
const MessageStatusIndicatorInternal = (props) => {
    const { status, styles, remoteParticipantsCount, onToggleToolTip, readCount, shouldAnnounce } = props;
    const localeStrings = useLocale$1().strings.messageStatusIndicator;
    const [isTooltipToggled, setIsTooltipToggled] = React.useState(false);
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const theme = useTheme();
    const calloutStyle = {
        root: {
            padding: 0
        },
        calloutMain: {
            padding: '0.5rem'
        }
    };
    // Place callout with no gap between it and the button.
    const calloutProps = {
        gapSpace: 0,
        styles: calloutStyle,
        backgroundColor: isDarkThemed(theme) ? theme.palette.neutralLighter : ''
    };
    switch (status) {
        case 'failed':
            return React.createElement(react.TooltipHost, { content: strings.failedToSendTooltipText, "data-ui-id": "chat-composite-message-tooltip", calloutProps: Object.assign({}, calloutProps), styles: hostStyles$1 },
                React.createElement(MemoMessageStatusIcon, { shouldAnnounce: shouldAnnounce, iconName: "MessageFailed", iconClassName: react.mergeStyles(MessageStatusIndicatorErrorIconStyle, {
                        color: isDarkThemed(theme) ? theme.semanticColors.errorText : theme.palette.redDark
                    }, styles === null || styles === void 0 ? void 0 : styles.root), ariaLabel: strings.failedToSendAriaLabel }));
        case 'sending':
            return React.createElement(react.TooltipHost, { content: strings.sendingTooltipText, "data-ui-id": "chat-composite-message-tooltip", calloutProps: Object.assign({}, calloutProps), styles: hostStyles$1 },
                React.createElement(MemoMessageStatusIcon, { shouldAnnounce: shouldAnnounce, iconName: "MessageSending", iconClassName: react.mergeStyles(MessageStatusIndicatorIconStyle, {
                        color: theme.palette.themePrimary
                    }, styles === null || styles === void 0 ? void 0 : styles.root), ariaLabel: strings.sendingAriaLabel }));
        case 'seen':
            return React.createElement(react.TooltipHost, { calloutProps: Object.assign({}, calloutProps), "data-ui-id": "chat-composite-message-tooltip", styles: hostStyles$1, content: 
                // when it's just 1 to 1 texting, we don't need to know who has read the message, just show message as 'seen'
                // when readCount is 0, we have a bug, show 'seen' to cover up as a fall back
                // when participant count is 0, we have a bug, show 'seen' to cover up as a fall back
                readCount === 0 || remoteParticipantsCount && remoteParticipantsCount <= 1 || !readCount || !remoteParticipantsCount || strings.readByTooltipText === undefined ? strings.seenTooltipText : _formatString(strings.readByTooltipText, {
                    messageThreadReadCount: `${readCount}`,
                    remoteParticipantsCount: `${remoteParticipantsCount}`
                }), onTooltipToggle: () => {
                    if (onToggleToolTip) {
                        onToggleToolTip(!isTooltipToggled);
                        setIsTooltipToggled(!isTooltipToggled);
                    }
                } },
                React.createElement(MemoMessageStatusIcon, { shouldAnnounce: shouldAnnounce, iconName: "MessageSeen", iconClassName: react.mergeStyles({
                        color: theme.palette.themePrimary
                    }, styles === null || styles === void 0 ? void 0 : styles.root), ariaLabel: strings.seenAriaLabel }));
        case 'delivered':
            return React.createElement(react.TooltipHost, { calloutProps: Object.assign({}, calloutProps), content: strings.deliveredTooltipText, "data-ui-id": "chat-composite-message-tooltip", styles: hostStyles$1 },
                React.createElement(MemoMessageStatusIcon, { shouldAnnounce: shouldAnnounce, iconName: "MessageDelivered", iconClassName: react.mergeStyles(MessageStatusIndicatorIconStyle, {
                        color: theme.palette.themePrimary
                    }, styles === null || styles === void 0 ? void 0 : styles.root), ariaLabel: strings.deliveredAriaLabel }));
        default:
            return React.createElement(React.Fragment, null);
    }
};
// The TooltipHost root uses display: inline by default.
// To prevent sizing issues or tooltip positioning issues, we override to inline-block.
// For more details see "Icon Button with Tooltip" on https://developer.microsoft.com/en-us/fluentui#/controls/web/button
const hostStyles$1 = {
    root: {
        display: 'inline-block'
    }
};
const MemoMessageStatusIcon = React.memo((obj) => {
    return React.createElement(MessageStatusIcon, Object.assign({}, obj));
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component to display the status of a sent message.
 *
 * Adds an icon and tooltip corresponding to the message status.
 *
 * @public
 */
const MessageStatusIndicator = (props) => {
    const internalProps = Object.assign(Object.assign({}, props), { shouldAnnounce: true });
    return React.createElement(MessageStatusIndicatorInternal, Object.assign({}, internalProps));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const _ATTACHMENT_CARD_WIDTH_IN_REM = 12;
/**
 * @private
 */
const _ATTACHMENT_CARD_MARGIN_IN_PX = 2;
/**
 * @private
 */
const useAttachmentCardStyles = reactComponents.makeStyles({
    root: {
        '& div[role=toolbar]': Object.assign({}, reactComponents.shorthands.padding(0))
    },
    dynamicWidth: {
        minWidth: `${_ATTACHMENT_CARD_WIDTH_IN_REM}rem`
    },
    staticWidth: {
        width: `${_ATTACHMENT_CARD_WIDTH_IN_REM}rem`
    },
    fileIcon: {
        marginLeft: _pxToRem(4),
        // don't shrink the file icon container
        flexShrink: 0
    },
    content: {
        '> div': {
            overflow: 'hidden',
            textOverflow: 'ellipsis'
        }
    },
    title: {
        overflow: 'hidden',
        whiteSpace: 'nowrap',
        textOverflow: 'ellipsis'
    },
    actions: {
        // don't shrink the actions container
        flexShrink: 0
    }
});
/**
 * @private
 */
const titleTooltipContainerStyle = react.mergeStyles({
    width: '100%',
    textOverflow: 'ellipsis',
    whiteSpace: 'nowrap',
    overflowX: 'clip'
});
/**
 * @private
 */
const ATTACHMENT_CARD_MIN_PROGRESS = 0.05;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const ERROR_IMAGE_SVG_PATH = 'M2.85 2.15a.5.5 0 1 0-.7.7l1.4 1.41A2.99 2.99 0 0 0 3 6v8a3 3 0 0 0 3 3h8c.65 0 1.25-.2 1.74-.55l1.4 1.4a.5.5 0 0 0 .71-.7l-15-15Zm6.56 7.97a1.5 1.5 0 0 0-.46.31l-4.67 4.59A2 2 0 0 1 4 14V6a2 2 0 0 1 .28-1.02l5.13 5.14ZM6 16a2 2 0 0 1-1.01-.27l4.66-4.58c.2-.2.5-.2.7 0l4.66 4.58A2 2 0 0 1 14 16H6ZM16 6v7.88l.9.9A3 3 0 0 0 17 14V6a3 3 0 0 0-3-3H6a3 3 0 0 0-.78.1l.9.9H14a2 2 0 0 1 2 2Zm-2 1.5a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0Zm-1 0a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0Z';
/**
 * @private
 */
const BROKEN_IMAGE_SVG_DATA = `data:image/svg+xml,%3Csvg width='3rem' height='3rem' viewBox='0 0 24 24' xmlns='http://www.w3.org/2000/svg' fill='currentColor'%3E%3Cpath d='${ERROR_IMAGE_SVG_PATH}' fill='currentColor' /%3E%3C/svg%3E`;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// Minimum chat bubble width. This matches the minimum chat bubble width from FluentUI
// that can contain a message and a timestamp.
const CHAT_MESSAGE_CONTAINER_MIN_WIDTH_REM = 6.25;
// Chat messages should span just short of the width of the container.
// When calculating the width of a message we also must take into account
// the width of the avatar/gutter and the gap between the message and avatar/gutter.
const AVATAR_WIDTH_REM = 2;
const AVATAR_MARGIN_LEFT = 2.5;
const AVATAR_MESSAGE_GAP_REM = 0.125;
const MESSAGE_AMOUNT_OUT_FROM_EDGE_REM = 2;
// Avatars should display on top of chat messages when the chat thread is narrow
const MESSAGE_AVATAR_OVERLAP_REM = 0.925;
const CHAT_MESSAGE_ZINDEX = 1;
const AVATAR_ZINDEX = 2;
// new message button should be on top of chat message
const NEW_MESSAGE_BUTTON_ZINDEX = 2;
/**
 * @private
 */
const chatMyMessageActionMenuClassName = 'ChatMyMessage__actions';
/**
 * @private
 */
const messageThreadContainerStyle = react.mergeStyles({
    height: '100%',
    width: '100%',
    maxHeight: '100%',
    overflow: 'auto',
    position: 'relative',
    alignSelf: 'center'
});
/**
 * @private
 */
const messageThreadWrapperContainerStyle = react.mergeStyles({
    height: '100%',
    width: '100%',
    position: 'relative'
});
/**
 * @private
 */
const noMessageStatusStyle = react.mergeStyles({
    // This should match the size of the message status indicator icon to ensure
    // multiple messages sent by the user are aligned correctly.
    width: `${MESSAGE_STATUS_INDICATOR_SIZE_REM}rem`
});
/**
 * @private
 */
const useChatStyles = reactComponents.makeStyles({
    root: Object.assign(Object.assign(Object.assign({ 
        // chat components sets max width value to 1056px, override it to 100%
        maxWidth: '100%', paddingTop: '0.8rem', paddingBottom: '0.5rem', paddingRight: '0.6rem', paddingLeft: '0.6rem' }, reactComponents.shorthands.border('none')), reactComponents.shorthands.overflow('auto')), { 
        // `height: 100%` ensures that the Chat component covers 100% of it's parents height
        // to prevent intermittent scrollbars when GIFs are present in the chat.
        height: '100%', '& a:link': {
            color: reactComponents.tokens.colorBrandForegroundLink
        }, '& a:visited': {
            color: reactComponents.tokens.colorBrandForegroundLinkHover
        }, '& a:hover': {
            color: reactComponents.tokens.colorBrandForegroundLinkHover
        } })
});
/**
 * @private
 */
const useChatMessageRenderStyles = reactComponents.makeStyles({
    rootCommon: {},
    rootMessage: Object.assign(Object.assign(Object.assign({}, reactComponents.shorthands.padding('0')), reactComponents.shorthands.margin('0')), { maxWidth: '100%', minWidth: `${CHAT_MESSAGE_CONTAINER_MIN_WIDTH_REM}rem` }),
    rootMyMessage: Object.assign(Object.assign({ gridTemplateColumns: 'auto fit-content(0)', gridTemplateAreas: `
        "body status"
      `, columnGap: '0', gridGap: '0' }, reactComponents.shorthands.padding('0')), { marginTop: '0', marginRight: '0', marginBottom: '0', marginLeft: '50px', width: `calc(100% - 50px)` }),
    bodyCommon: Object.assign(Object.assign({}, reactComponents.shorthands.padding('0')), { marginRight: '0', marginBottom: '0', backgroundColor: 'transparent', maxWidth: '100%', minWidth: `${CHAT_MESSAGE_CONTAINER_MIN_WIDTH_REM}rem` }),
    bodyMyMessage: {
        width: '100%',
        marginTop: '0'
    },
    bodyWithoutAvatar: {
        marginLeft: `${AVATAR_MARGIN_LEFT}rem`,
        marginTop: '0'
    },
    bodyHiddenAvatar: {
        marginLeft: 0,
        marginTop: 0
    },
    bodyWithAvatar: {
        marginLeft: `0`,
        marginTop: '0.75rem'
    },
    avatarNoOverlap: {
        width: `calc(100% - ${AVATAR_WIDTH_REM + MESSAGE_AMOUNT_OUT_FROM_EDGE_REM + AVATAR_MESSAGE_GAP_REM}rem)`
    },
    avatarOverlap: {
        width: `calc(100% - ${AVATAR_WIDTH_REM + MESSAGE_AMOUNT_OUT_FROM_EDGE_REM - MESSAGE_AVATAR_OVERLAP_REM}rem)`
    }
});
/**
 * @private
 */
const useChatMyMessageStyles = reactComponents.makeStyles({
    root: {
        gridTemplateColumns: 'auto',
        gridTemplateAreas: `
        "body"
      `,
        gridGap: '0',
        paddingTop: '0',
        marginLeft: '0'
    },
    body: Object.assign(Object.assign({ paddingBottom: '10px', marginTop: '1rem', maxWidth: '100%', minWidth: `${CHAT_MESSAGE_CONTAINER_MIN_WIDTH_REM}rem`, marginLeft: '0rem' }, reactComponents.shorthands.border('1px', 'solid', 'transparent')), { '@media (hover: hover)': {
            '&:hover .ChatMyMessage__actions': {
                visibility: 'visible'
            }
        }, '&:focus .ChatMyMessage__actions': {
            visibility: 'visible'
        }, '& msft-mention': {
            color: reactComponents.tokens.colorStatusWarningBackground3,
            fontWeight: 600
        }, '& img': {
            maxWidth: '100%',
            height: 'auto'
        } }),
    bodyAttached: {
        marginTop: '0.125rem'
    },
    menu: Object.assign(Object.assign({ boxShadow: reactComponents.tokens.shadow4, backgroundColor: reactComponents.tokens.colorNeutralBackground1, position: 'absolute', top: '-19px', right: '0' }, reactComponents.shorthands.borderRadius(reactComponents.tokens.borderRadiusMedium)), { 
        // Ensure the focus border around the message bubble doesn't overlap on top of more options button
        zIndex: 2, lineHeight: reactComponents.tokens.lineHeightBase100, visibility: 'hidden', '&:hover, &:focus': {
            cursor: 'pointer',
            visibility: 'visible'
        } }),
    menuHidden: {
        visibility: 'hidden'
    },
    menuVisible: {
        visibility: 'visible'
    },
    multipleAttachmentsInViewing: {
        width: '100%',
        maxWidth: `${(_ATTACHMENT_CARD_WIDTH_IN_REM + _ATTACHMENT_CARD_MARGIN_IN_PX) * 2}rem`
    },
    multipleAttachmentsInEditing: {
        // when in editing state, the chat message width should not be
        // limited by content length but occupying the full width instead
        width: '100%',
        float: 'right'
    }
});
/**
 * @private
 */
const newMessageButtonContainerStyle = react.mergeStyles({
    position: 'absolute',
    zIndex: NEW_MESSAGE_BUTTON_ZINDEX,
    bottom: 0,
    right: '1.5rem'
});
/**
 * @private
 */
const useChatMessageStyles = reactComponents.makeStyles({
    root: {
        paddingTop: '0'
    },
    body: Object.assign(Object.assign({ maxWidth: '100%', minWidth: `${CHAT_MESSAGE_CONTAINER_MIN_WIDTH_REM}rem`, marginRight: '0rem', paddingBottom: '10px', zIndex: CHAT_MESSAGE_ZINDEX }, reactComponents.shorthands.border('1px', 'solid', 'transparent')), { '& > div:first-of-type': {
            flexWrap: 'wrap'
        }, '& msft-mention': {
            color: reactComponents.tokens.colorStatusWarningBackground3,
            fontWeight: reactComponents.tokens.fontWeightSemibold
        }, '& img': {
            maxWidth: '100% !important',
            // Add !important to make sure it won't be overridden by style defined in element
            height: 'auto !important'
        }, '& video': {
            maxWidth: '100% !important',
            // Add !important to make sure it won't be overridden by style defined in element
            height: 'auto !important'
        }, '& p': Object.assign({}, reactComponents.shorthands.marginBlock('0.125rem')), '& blockquote': Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({ backgroundColor: reactComponents.tokens.colorBrandBackgroundInverted, clear: 'left', minHeight: '2.25rem', width: 'fit-content', marginTop: '7px', marginRight: '0px', marginLeft: '0px', marginBottom: '7px', paddingTop: '7px', paddingRight: '15px', paddingLeft: '15px', paddingBottom: '7px' }, reactComponents.shorthands.border('solid')), reactComponents.shorthands.borderRadius('4px')), reactComponents.shorthands.borderWidth('1px')), reactComponents.shorthands.borderColor(reactComponents.tokens.colorNeutralStroke1Selected)), { borderLeftWidth: '4px' }), '& code': {
            whiteSpace: 'pre-wrap'
        } }),
    bodyWithoutAvatar: {
        marginTop: '0.125rem'
    },
    bodyWithAvatar: {
        marginTop: `0.375rem`
    },
    avatarNoOverlap: {
        marginLeft: `${ -2.5 + AVATAR_MESSAGE_GAP_REM}rem`
    },
    avatarOverlap: {
        marginLeft: `${ -2.5 - MESSAGE_AVATAR_OVERLAP_REM}rem`
    },
    multipleAttachments: {
        width: `${(_ATTACHMENT_CARD_WIDTH_IN_REM + _ATTACHMENT_CARD_MARGIN_IN_PX) * 2}rem`
    }
});
/**
 * @private
 */
const useChatMessageCommonStyles = reactComponents.makeStyles({
    body: {
        '& table': {
            backgroundColor: reactComponents.tokens.colorBrandBackgroundInverted,
            borderCollapse: 'collapse',
            tableLayout: 'auto',
            width: '100%',
            '& tr': Object.assign(Object.assign({}, reactComponents.shorthands.border('1px', 'solid', `${reactComponents.tokens.colorNeutralStrokeAccessible}`)), { '& td': Object.assign(Object.assign({}, reactComponents.shorthands.border('1px', 'solid', `${reactComponents.tokens.colorNeutralStrokeAccessible}`)), { wordBreak: 'normal', paddingTop: '0px', paddingRight: '5px' }) })
        }
    },
    bodyWithPlaceholderImage: {
        '& img[src=""]': {
            backgroundColor: reactComponents.tokens.colorNeutralBackground1Selected,
            // this ensures safari won't have default rendering when image source
            // becomes invalid such as empty string in this case
            fontSize: '0',
            position: 'relative',
            clipPath: 'inset(0.3px)',
            display: 'flex'
        },
        '& img[src=""]:after': {
            backgroundColor: reactComponents.tokens.colorNeutralBackground1Selected,
            content: `url("data:image/gif;base64,R0lGODlhAQABAAAAACw=")`,
            backgroundSize: 'center',
            position: 'absolute',
            width: '100%',
            height: '100%',
            top: '0',
            left: '0',
            display: 'block'
        },
        '& .broken-image-wrapper': Object.assign(Object.assign({ width: '12rem', height: '12rem', marginTop: '0.75rem', display: 'flex', justifyContent: 'center', alignItems: 'center' }, reactComponents.shorthands.outline('1px', 'solid', reactComponents.tokens.colorNeutralForegroundDisabled)), { backgroundColor: reactComponents.tokens.colorNeutralBackground2 }),
        '& .broken-image-wrapper:after': {
            content: `''`,
            maskImage: `url("${BROKEN_IMAGE_SVG_DATA}");`,
            width: '3rem',
            height: '3rem',
            backgroundColor: `${reactComponents.tokens.colorNeutralForeground2}`
        }
    },
    failed: {
        //TODO: can we reuse a theme color here?
        backgroundColor: 'rgba(168, 0, 0, 0.2)'
    },
    blocked: Object.assign(Object.assign({ maxWidth: '100%', minWidth: `${CHAT_MESSAGE_CONTAINER_MIN_WIDTH_REM}rem`, marginRight: '0rem', color: reactComponents.tokens.colorNeutralForeground2 }, reactComponents.shorthands.border('1px', 'solid', 'transparent')), { '& i': {
            paddingTop: '0.25rem'
        }, '& p': Object.assign(Object.assign({}, reactComponents.shorthands.marginBlock('0.125rem')), { paddingRight: '0.75rem', fontStyle: 'italic' }), '& a': Object.assign(Object.assign(Object.assign({}, reactComponents.shorthands.marginBlock('0.125rem')), { fontStyle: 'normal', color: reactComponents.tokens.colorBrandForegroundLink }), reactComponents.shorthands.textDecoration('none')) })
});
/**
 * @private
 */
const gutterWithAvatar = {
    paddingTop: '1.65rem',
    width: `${AVATAR_WIDTH_REM}rem`,
    position: 'relative',
    float: 'left',
    display: 'block',
    visibility: 'visible',
    zIndex: AVATAR_ZINDEX
};
/**
 * @private
 */
const gutterWithHiddenAvatar = Object.assign(Object.assign({}, gutterWithAvatar), { visibility: 'hidden', 
    // we use this hidden avatar just as a width placeholder
    // the placeholder is needed for responsive bubble width
    height: 0 });
/**
 * @private
 */
const newMessageButtonStyle = react.mergeStyles({
    float: 'right',
    width: 'fit-content'
});
/**
 * @private
 */
const buttonWithIconStyles$1 = {
    textContainer: {
        display: 'contents'
    }
};
/**
 * @private
 */
react.mergeStyles({
    border: 'none',
    minHeight: '1.5rem',
    '&:hover': {
        background: 'none'
    },
    '&:active': {
        background: 'none'
    }
});
/**
 * @private
 */
const DownIconStyle = react.mergeStyles({
    marginRight: '0.5em'
});
/** @private */
react.mergeStyles({
    width: '1.25rem',
    height: '1.25rem'
});
/** @private */
const messageTextContentStyles = react.mergeStyles({
    whiteSpace: 'pre-wrap'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const delay = (delay) => {
    return new Promise(function (resolve) {
        setTimeout(resolve, delay);
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A utility hook for providing the width of a parent element.
 * Returns updated width if parent/window resizes.
 * @param containerRef - Ref of a parent element whose width will be returned.
 * @internal
 */
const _useContainerWidth = (containerRef) => {
    const [width, setWidth] = React.useState(undefined);
    const observer = React.useRef(new ResizeObserver(entries => {
        if (!entries[0]) {
            return;
        }
        const { width } = entries[0].contentRect;
        if (Number.isNaN(width)) {
            setWidth(0);
        }
        else {
            setWidth(width);
        }
    }));
    React.useEffect(() => {
        if (containerRef.current) {
            observer.current.observe(containerRef.current);
        }
        const currentObserver = observer.current;
        return () => {
            currentObserver.disconnect();
        };
    }, [containerRef, observer]);
    return width;
};
/**
 * A utility hook for providing the height of a parent element.
 * Returns updated height if parent/window resizes.
 * @param containerRef - Ref of a parent element whose height will be returned.
 * @internal
 */
const _useContainerHeight = (containerRef) => {
    const [height, setHeight] = React.useState(undefined);
    const observer = React.useRef(new ResizeObserver(entries => {
        if (!entries[0]) {
            return;
        }
        const { height } = entries[0].contentRect;
        if (Number.isNaN(height)) {
            setHeight(0);
        }
        else {
            setHeight(height);
        }
    }));
    React.useEffect(() => {
        if (containerRef.current) {
            observer.current.observe(containerRef.current);
        }
        const currentObserver = observer.current;
        return () => {
            currentObserver.disconnect();
        };
    }, [containerRef, observer]);
    return height;
};
const NARROW_WIDTH_REM = 30;
const SHORT_HEIGHT_REM = 23.75;
/**
 * Utility function to determine if container width is narrow
 * @param containerWidthRem  container width in rem
 * @returns boolean
 */
const isNarrowWidth = (containerWidthRem) => containerWidthRem <= _convertRemToPx(NARROW_WIDTH_REM);
/**
 * Utility function to determine if container width is short
 * @param containerWidthRem  container height in rem
 * @returns boolean
 */
const isShortHeight = (containerHeightRem) => containerHeightRem <= _convertRemToPx(SHORT_HEIGHT_REM);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 *logic: Looking at message A, how do we know it's read number?
 * Assumption: if user read the latest message, user has read all messages before that
 * ReadReceipt behaviour: read receipt is only sent to the last message
 *
 * If participant read a message that is sent later than message A, then the participant has read message A
 * How do we check if the message is sent later than message A?
 * We compare if the messageID of the last read message is larger than or equal to the message A's id
 * Because messageID is the creation timestamp of each message
 * Timestamps are in epoch time so lecixographical ordering is the same as time ordering.
 *
 * if MessageId of B is larger than message Id of A, then B is created after A
 * if the last read message is created after the message A is sent, then user should have read message A as well */
var getParticipantsWhoHaveReadMessage = (message, readReceiptsBySenderId) => {
    return Object.entries(readReceiptsBySenderId)
        // Filter to only read receipts that match the message OR the participant has read a different message after this message has been created
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        .filter(([_, readReceipt]) => readReceipt.lastReadMessage >= message.messageId)
        // make sure the person is not removed from chat
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        .filter(([_, readReceipt]) => readReceipt.displayName && readReceipt.displayName !== '')
        // Map properties to useful array
        .map(([id, readReceipt]) => ({
        id,
        displayName: readReceipt.displayName
    }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// These color records are required for createV9Theme
// For more info, check https://react.fluentui.dev/iframe.html?viewMode=docs&id=concepts-migration-from-v8-components-theme-migration--page#compatible-themes
/**
 * @private
 */
const grey = {
    '8': '#141414',
    '18': '#2e2e2e',
    '20': '#333333',
    '22': '#383838',
    '24': '#3d3d3d',
    '92': '#ebebeb',
    '94': '#f0f0f0',
    '96': '#f5f5f5',
    '98': '#fafafa'};
/**
 * @private
 */
const whiteAlpha = {
    '5': 'rgba(255, 255, 255, 0.05)',
    '10': 'rgba(255, 255, 255, 0.1)',
    '20': 'rgba(255, 255, 255, 0.2)',
    '40': 'rgba(255, 255, 255, 0.4)',
    '50': 'rgba(255, 255, 255, 0.5)',
    '80': 'rgba(255, 255, 255, 0.8)'};
/**
 * @private
 */
const blackAlpha = {
    '5': 'rgba(0, 0, 0, 0.05)',
    '10': 'rgba(0, 0, 0, 0.1)',
    '20': 'rgba(0, 0, 0, 0.2)',
    '30': 'rgba(0, 0, 0, 0.3)',
    '40': 'rgba(0, 0, 0, 0.4)',
    '50': 'rgba(0, 0, 0, 0.5)'};
/**
 * @private
 */
const grey10Alpha = {
    '50': 'rgba(26, 26, 26, 0.5)'};
/**
 * @private
 */
const grey12Alpha = {
    '70': 'rgba(31, 31, 31, 0.7)'};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// These mappings are required for createV9Theme
// For more info, check https://react.fluentui.dev/iframe.html?viewMode=docs&id=concepts-migration-from-v8-components-theme-migration--page#compatible-themes
/**
 * Creates v9 color tokens from a v8 palette.
 */
const mapAliasColors = (palette, inverted) => {
    return {
        colorNeutralForeground1: palette.neutralPrimary,
        colorNeutralForeground1Hover: palette.neutralPrimary,
        colorNeutralForeground1Pressed: palette.neutralPrimary,
        colorNeutralForeground1Selected: palette.neutralPrimary,
        colorNeutralForeground2: palette.neutralSecondary,
        colorNeutralForeground2Hover: palette.neutralPrimary,
        colorNeutralForeground2Pressed: palette.neutralPrimary,
        colorNeutralForeground2Selected: palette.neutralPrimary,
        colorNeutralForeground2BrandHover: palette.themePrimary,
        colorNeutralForeground2BrandPressed: palette.themeDarkAlt,
        colorNeutralForeground2BrandSelected: palette.themePrimary,
        colorNeutralForeground3: palette.neutralTertiary,
        colorNeutralForeground3Hover: palette.neutralSecondary,
        colorNeutralForeground3Pressed: palette.neutralSecondary,
        colorNeutralForeground3Selected: palette.neutralSecondary,
        colorNeutralForeground3BrandHover: palette.themePrimary,
        colorNeutralForeground3BrandPressed: palette.themeDarkAlt,
        colorNeutralForeground3BrandSelected: palette.themePrimary,
        colorNeutralForeground4: palette.neutralQuaternary,
        colorNeutralForegroundDisabled: palette.neutralTertiaryAlt,
        colorNeutralForegroundInvertedDisabled: whiteAlpha[40],
        colorBrandForegroundLink: palette.themeDarkAlt,
        colorBrandForegroundLinkHover: palette.themeDark,
        colorBrandForegroundLinkPressed: palette.themeDarker,
        colorBrandForegroundLinkSelected: palette.themeDarkAlt,
        colorNeutralForeground2Link: palette.neutralSecondary,
        colorNeutralForeground2LinkHover: palette.neutralPrimary,
        colorNeutralForeground2LinkPressed: palette.neutralPrimary,
        colorNeutralForeground2LinkSelected: palette.neutralPrimary,
        colorCompoundBrandForeground1: palette.themePrimary,
        colorCompoundBrandForeground1Hover: palette.themeDarkAlt,
        colorCompoundBrandForeground1Pressed: palette.themeDark,
        colorBrandForeground1: palette.themePrimary,
        colorBrandForeground2: palette.themeDarkAlt,
        colorBrandForeground2Hover: palette.themeDarkAlt,
        colorBrandForeground2Pressed: palette.themeDarkAlt,
        colorNeutralForeground1Static: palette.neutralPrimary,
        colorNeutralForegroundInverted: palette.white,
        colorNeutralForegroundInvertedHover: palette.white,
        colorNeutralForegroundInvertedPressed: palette.white,
        colorNeutralForegroundInvertedSelected: palette.white,
        colorNeutralForegroundOnBrand: palette.white,
        colorNeutralForegroundStaticInverted: palette.white,
        colorNeutralForegroundInvertedLink: palette.white,
        colorNeutralForegroundInvertedLinkHover: palette.white,
        colorNeutralForegroundInvertedLinkPressed: palette.white,
        colorNeutralForegroundInvertedLinkSelected: palette.white,
        colorNeutralForegroundInverted2: palette.white,
        colorBrandForegroundInverted: palette.themeSecondary,
        colorBrandForegroundInvertedHover: palette.themeTertiary,
        colorBrandForegroundInvertedPressed: palette.themeSecondary,
        colorBrandForegroundOnLight: palette.themePrimary,
        colorBrandForegroundOnLightHover: palette.themeDarkAlt,
        colorBrandForegroundOnLightPressed: palette.themeDark,
        colorBrandForegroundOnLightSelected: palette.themeDark,
        colorNeutralBackground1: palette.white,
        colorNeutralBackground1Hover: palette.neutralLighter,
        colorNeutralBackground1Pressed: palette.neutralQuaternaryAlt,
        colorNeutralBackground1Selected: palette.neutralLight,
        colorNeutralBackground2: palette.neutralLighterAlt,
        colorNeutralBackground2Hover: palette.neutralLighter,
        colorNeutralBackground2Pressed: palette.neutralQuaternaryAlt,
        colorNeutralBackground2Selected: palette.neutralLight,
        colorNeutralBackground3: palette.neutralLighter,
        colorNeutralBackground3Hover: palette.neutralLight,
        colorNeutralBackground3Pressed: palette.neutralQuaternary,
        colorNeutralBackground3Selected: palette.neutralQuaternaryAlt,
        colorNeutralBackground4: palette.neutralLighter,
        colorNeutralBackground4Hover: palette.neutralLighterAlt,
        colorNeutralBackground4Pressed: palette.neutralLighter,
        colorNeutralBackground4Selected: palette.white,
        colorNeutralBackground5: palette.neutralLight,
        colorNeutralBackground5Hover: palette.neutralLighter,
        colorNeutralBackground5Pressed: palette.neutralLighter,
        colorNeutralBackground5Selected: palette.neutralLighterAlt,
        colorNeutralBackground6: palette.neutralLight,
        colorNeutralBackgroundStatic: grey[20],
        colorNeutralBackgroundInverted: palette.neutralSecondary,
        colorNeutralBackgroundAlpha: inverted ? grey10Alpha[50] : whiteAlpha[50],
        colorNeutralBackgroundAlpha2: inverted ? grey12Alpha[70] : whiteAlpha[80],
        colorSubtleBackground: 'transparent',
        colorSubtleBackgroundHover: palette.neutralLighter,
        colorSubtleBackgroundPressed: palette.neutralQuaternaryAlt,
        colorSubtleBackgroundSelected: palette.neutralLight,
        colorSubtleBackgroundLightAlphaHover: inverted ? whiteAlpha[10] : whiteAlpha[80],
        colorSubtleBackgroundLightAlphaPressed: inverted ? whiteAlpha[5] : whiteAlpha[50],
        colorSubtleBackgroundLightAlphaSelected: 'transparent',
        colorSubtleBackgroundInverted: 'transparent',
        colorSubtleBackgroundInvertedHover: blackAlpha[10],
        colorSubtleBackgroundInvertedPressed: blackAlpha[30],
        colorSubtleBackgroundInvertedSelected: blackAlpha[20],
        colorTransparentBackground: 'transparent',
        colorTransparentBackgroundHover: 'transparent',
        colorTransparentBackgroundPressed: 'transparent',
        colorTransparentBackgroundSelected: 'transparent',
        colorNeutralBackgroundDisabled: palette.neutralLighter,
        colorNeutralBackgroundInvertedDisabled: whiteAlpha[10],
        colorNeutralStencil1: palette.neutralLight,
        colorNeutralStencil2: palette.neutralLighterAlt,
        colorNeutralStencil1Alpha: inverted ? whiteAlpha[10] : blackAlpha[10],
        colorNeutralStencil2Alpha: inverted ? whiteAlpha[5] : blackAlpha[5],
        colorBackgroundOverlay: blackAlpha[40],
        colorScrollbarOverlay: blackAlpha[50],
        colorBrandBackground: palette.themePrimary,
        colorBrandBackgroundHover: palette.themeDarkAlt,
        colorBrandBackgroundPressed: palette.themeDarker,
        colorBrandBackgroundSelected: palette.themeDark,
        colorCompoundBrandBackground: palette.themePrimary,
        colorCompoundBrandBackgroundHover: palette.themeDarkAlt,
        colorCompoundBrandBackgroundPressed: palette.themeDark,
        colorBrandBackgroundStatic: palette.themePrimary,
        colorBrandBackground2: palette.themeLighterAlt,
        colorBrandBackground2Hover: palette.themeLighterAlt,
        colorBrandBackground2Pressed: palette.themeLighterAlt,
        colorBrandBackground3Static: palette.themeDark,
        colorBrandBackground4Static: palette.themeDarker,
        colorBrandBackgroundInverted: palette.white,
        colorBrandBackgroundInvertedHover: palette.themeLighterAlt,
        colorBrandBackgroundInvertedPressed: palette.themeLight,
        colorBrandBackgroundInvertedSelected: palette.themeLighter,
        colorNeutralCardBackground: inverted ? grey[20] : grey[98],
        colorNeutralCardBackgroundHover: inverted ? grey[24] : palette.white,
        colorNeutralCardBackgroundPressed: inverted ? grey[18] : grey[96],
        colorNeutralCardBackgroundSelected: inverted ? grey[22] : grey[92],
        colorNeutralCardBackgroundDisabled: inverted ? grey[8] : grey[94],
        colorNeutralStrokeAccessible: palette.neutralSecondary,
        colorNeutralStrokeAccessibleHover: palette.neutralSecondary,
        colorNeutralStrokeAccessiblePressed: palette.neutralSecondary,
        colorNeutralStrokeAccessibleSelected: palette.themePrimary,
        colorNeutralStroke1: palette.neutralQuaternary,
        colorNeutralStroke1Hover: palette.neutralTertiaryAlt,
        colorNeutralStroke1Pressed: palette.neutralTertiaryAlt,
        colorNeutralStroke1Selected: palette.neutralTertiaryAlt,
        colorNeutralStroke2: palette.neutralQuaternaryAlt,
        colorNeutralStroke3: palette.neutralLighter,
        colorNeutralStrokeSubtle: palette.neutralQuaternaryAlt,
        colorNeutralStrokeOnBrand: palette.white,
        colorNeutralStrokeOnBrand2: palette.white,
        colorNeutralStrokeOnBrand2Hover: palette.white,
        colorNeutralStrokeOnBrand2Pressed: palette.white,
        colorNeutralStrokeOnBrand2Selected: palette.white,
        colorBrandStroke1: palette.themePrimary,
        colorBrandStroke2: palette.themeLight,
        colorBrandStroke2Hover: palette.themeLight,
        colorBrandStroke2Pressed: palette.themeLight,
        colorBrandStroke2Contrast: palette.themeLight,
        colorCompoundBrandStroke: palette.themePrimary,
        colorCompoundBrandStrokeHover: palette.themeDarkAlt,
        colorCompoundBrandStrokePressed: palette.themeDark,
        colorNeutralStrokeDisabled: palette.neutralQuaternaryAlt,
        colorNeutralStrokeInvertedDisabled: whiteAlpha[40],
        colorTransparentStroke: 'transparent',
        colorTransparentStrokeInteractive: 'transparent',
        colorTransparentStrokeDisabled: 'transparent',
        colorNeutralStrokeAlpha: inverted ? whiteAlpha[10] : blackAlpha[5],
        colorNeutralStrokeAlpha2: whiteAlpha[20],
        colorStrokeFocus1: palette.white,
        colorStrokeFocus2: palette.black,
        colorNeutralShadowAmbient: 'rgba(0,0,0,0.12)',
        colorNeutralShadowKey: 'rgba(0,0,0,0.14)',
        colorNeutralShadowAmbientLighter: 'rgba(0,0,0,0.06)',
        colorNeutralShadowKeyLighter: 'rgba(0,0,0,0.07)',
        colorNeutralShadowAmbientDarker: 'rgba(0,0,0,0.20)',
        colorNeutralShadowKeyDarker: 'rgba(0,0,0,0.24)',
        colorBrandShadowAmbient: 'rgba(0,0,0,0.30)',
        colorBrandShadowKey: 'rgba(0,0,0,0.25)'
    };
};
/**
 * Creates v9 shadow tokens from v8 effects.
 */
const mapShadowTokens = (effects) => {
    return {
        shadow4: effects.elevation4,
        shadow8: effects.elevation8,
        shadow16: effects.elevation16,
        shadow64: effects.elevation64
    };
};
/**
 * Creates v9 border radius tokens from v8 effects
 */
const mapBorderRadiusTokens = (effects) => {
    return {
        borderRadiusSmall: effects.roundedCorner2,
        borderRadiusMedium: effects.roundedCorner4,
        borderRadiusLarge: effects.roundedCorner6
    };
};
/**
 * Creates a v9 theme from a v8 theme and base v9 theme.
 * FluentUI webLightTheme is used in case if no baseThemeV9 is provided.
 *
 * @private
 */
const createV9Theme = (themeV8, baseThemeV9) => {
    const baseTheme = reactComponents.webLightTheme;
    return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, baseTheme), mapAliasColors(themeV8.palette, themeV8.isInverted)), mapShadowTokens(themeV8.effects)), mapBorderRadiusTokens(themeV8.effects)), { colorBrandBackground2: themeV8.palette.themeLight, 
        // updated from palette.themeLighterAlt
        colorBrandBackground2Hover: themeV8.palette.themeLight, 
        // updated from palette.themeLighterAlt
        colorBrandBackground2Pressed: themeV8.palette.themeLight, 
        // updated from palette.themeLighterAlt
        colorStatusWarningBackground3: '#D83B01', errorText: themeV8.semanticColors.errorText, colorNeutralStroke1Selected: themeV8.palette.neutralQuaternary, colorNeutralForeground2: themeV8.palette.neutralSecondary, colorBrandForegroundLinkHover: themeV8.palette.themeDarker, colorNeutralBackground1Selected: themeV8.palette.neutralQuaternaryAlt });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useFluentV9Wrapper = reactComponents.makeStyles({
    body: Object.assign(Object.assign(Object.assign(Object.assign({ height: '100%' }, reactComponents.shorthands.margin(0)), reactComponents.shorthands.overflow('hidden')), reactComponents.shorthands.padding(0)), { width: '100%' })
});
/**
 * @private
 */
const FluentV9ThemeProvider = (props) => {
    const { v8Theme, children, className } = props;
    const v9Theme = createV9Theme(v8Theme);
    const dir = v8Theme.rtl ? 'rtl' : 'ltr';
    return (
    // TextDirectionProvider is needed to fix issue with direction value update in FluentProvider
    React.createElement(react$1.TextDirectionProvider, { dir: dir },
        React.createElement(FluentProviderWithStylesOverrides, { theme: v9Theme, dir: dir, className: className }, children)));
};
const FluentProviderWithStylesOverrides = props => {
    const classes = useFluentV9Wrapper();
    return React.createElement(reactComponents.FluentProvider, Object.assign({}, props, { className: reactComponents.mergeClasses(props.className, classes.body), applyStylesToPortals: false }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/// Adapted from: https://github.com/AlmeroSteyn/react-aria-live/blob/master/src/modules/MessageBlock.js
const offScreenStyle = {
    border: 0,
    clip: 'rect(0 0 0 0)',
    height: '1px',
    margin: '-1px',
    overflow: 'hidden',
    whiteSpace: 'nowrap',
    padding: 0,
    width: '1px',
    position: 'absolute'
};
/** @private */
const MessageBlock = (props) => React.createElement("div", { style: offScreenStyle, role: "log", "aria-live": props.ariaLive }, props.message ? props.message : '');

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/// Adapted from: https://github.com/AlmeroSteyn/react-aria-live/blob/master/src/modules/Announcer.js
/** @private */
const EMPTY_MESSAGE = {
    message: '',
    id: ''
};
/** @private */
const Announcer = (props) => {
    var _a, _b;
    const newAssertive = (_a = props.assertive) !== null && _a !== void 0 ? _a : EMPTY_MESSAGE;
    const oldAssertive = React.useRef(EMPTY_MESSAGE);
    const [activeAssertive1, setActiveAssertive1] = React.useState(EMPTY_MESSAGE);
    const [activeAssertive2, setActiveAssertive2] = React.useState(EMPTY_MESSAGE);
    const alternateAssertive = React.useRef(false);
    React.useEffect(() => {
        if (oldAssertive.current.message !== (newAssertive === null || newAssertive === void 0 ? void 0 : newAssertive.message) || oldAssertive.current.id !== (newAssertive === null || newAssertive === void 0 ? void 0 : newAssertive.id)) {
            setActiveAssertive1(alternateAssertive.current ? EMPTY_MESSAGE : newAssertive);
            setActiveAssertive2(alternateAssertive.current ? newAssertive : EMPTY_MESSAGE);
            oldAssertive.current = newAssertive;
            alternateAssertive.current = !alternateAssertive.current;
        }
    }, [newAssertive]);
    const newPolite = (_b = props.polite) !== null && _b !== void 0 ? _b : EMPTY_MESSAGE;
    const oldPolite = React.useRef(EMPTY_MESSAGE);
    const [activePolite1, setActivePolite1] = React.useState(EMPTY_MESSAGE);
    const [activePolite2, setActivePolite2] = React.useState(EMPTY_MESSAGE);
    const alternatePolite = React.useRef(false);
    React.useEffect(() => {
        if (oldPolite.current.message !== (newPolite === null || newPolite === void 0 ? void 0 : newPolite.message) || oldPolite.current.id !== (newPolite === null || newPolite === void 0 ? void 0 : newPolite.id)) {
            setActivePolite1(alternatePolite.current ? EMPTY_MESSAGE : newPolite);
            setActivePolite2(alternatePolite.current ? newPolite : EMPTY_MESSAGE);
            oldPolite.current = newPolite;
            alternatePolite.current = !alternatePolite.current;
        }
    }, [newPolite]);
    return React.createElement("div", null,
        React.createElement(MessageBlock, { ariaLive: "assertive", message: activeAssertive1.message }),
        React.createElement(MessageBlock, { ariaLive: "assertive", message: activeAssertive2.message }),
        React.createElement(MessageBlock, { ariaLive: "polite", message: activePolite1.message }),
        React.createElement(MessageBlock, { ariaLive: "polite", message: activePolite2.message }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/// Adapted from: https://github.com/AlmeroSteyn/react-aria-live/blob/master/src/modules/LiveAnnouncer.js
/** @private */
const LiveAnnouncer = (props) => {
    const [politeMessage, setPoliteMessage] = React.useState(EMPTY_MESSAGE);
    const [assertiveMessage, setAssertiveMessage] = React.useState(EMPTY_MESSAGE);
    const announcePolite = React.useCallback((message, id) => {
        setPoliteMessage({
            message,
            id
        });
    }, []);
    const announceAssertive = React.useCallback((message, id) => {
        setAssertiveMessage({
            message,
            id
        });
    }, []);
    const updateFunctions = React.useMemo(() => ({
        announcePolite,
        announceAssertive
    }), [announceAssertive, announcePolite]);
    return React.createElement(AnnouncerContext.Provider, { value: updateFunctions },
        props.children,
        React.createElement(Announcer, { assertive: assertiveMessage, polite: politeMessage }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Function to create a React.CSSProperties object from a v8 style object.
 * This function is still not ideal
 * as v8Style can use pseudo-class selectors that style objects can't process correctly.
 *
 * @private
 */
function createStyleFromV8Style(v8Style) {
    const result = {};
    if (v8Style === undefined || v8Style === null || typeof v8Style === 'boolean' || typeof v8Style === 'string') {
        return undefined;
    }
    else if (typeof v8Style === 'object') {
        // v8Style is a style object
        for (const record in v8Style) {
            if (typeof v8Style[record] === 'string') {
                // v8Style[record] is just a simple style
                const msSuffix = 'MS';
                if (record.startsWith(msSuffix)) {
                    // React.CSSProperties uses camelCase for MS properties but v8Style uses PascalCase
                    const newRecord = record.substring(0, msSuffix.length).toLowerCase + record.substring(msSuffix.length);
                    result[newRecord] = v8Style[record];
                }
                else {
                    result[record] = v8Style[record];
                }
            }
            else {
                result[record] = createStyleFromV8Style(v8Style[record]);
            }
        }
    }
    return result;
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const systemMessageIconStyle = react.mergeStyles({
    marginInlineEnd: '0.688rem'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const SystemMessage = (props) => {
    const { iconName, content } = props;
    const Icon = React.createElement(react.FontIcon, { iconName: iconName, className: react.mergeStyles(systemMessageIconStyle) });
    const [liveMessage, setLiveMessage] = React.useState('');
    React.useEffect(() => {
        // Timeout is needed to handle situation when the same user joins and leaves a chat a few times in a row, otherwise Narrator won't repeat the system message text.
        // Because delay value is not provided, setMessage function will be executed asynchronously in the next event cycle
        setTimeout(() => {
            setLiveMessage(content);
        });
    }, [content]);
    return React.createElement(React.Fragment, null,
        React.createElement(LiveMessage, { message: liveMessage, ariaLive: "polite", clearOnUnmount: true }),
        React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles(props === null || props === void 0 ? void 0 : props.containerStyle), role: "status", "aria-label": content, tabIndex: 0 },
            Icon,
            React.createElement(react.Text, { style: {
                    wordBreak: 'break-word'
                }, variant: 'small' }, content)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const DefaultSystemMessage = (props) => {
    var _a;
    const message = props.message;
    switch (message.messageType) {
        case 'system':
            switch (message.systemMessageType) {
                case 'content':
                    return React.createElement(SystemMessage, { iconName: (message.iconName ? message.iconName : ''), content: (_a = message.content) !== null && _a !== void 0 ? _a : '', containerStyle: props === null || props === void 0 ? void 0 : props.messageContainerStyle });
                case 'participantAdded':
                case 'participantRemoved':
                    return React.createElement(ParticipantSystemMessageComponent, { message: message, style: props.messageContainerStyle, defaultName: props.strings.noDisplayNameSub });
            }
    }
    return React.createElement(React.Fragment, null);
};
const ParticipantSystemMessageComponent = ({ message, style, defaultName }) => {
    const { strings } = useLocale$1();
    const participantsStr = generateParticipantsStr(message.participants, defaultName);
    const messageSuffix = message.systemMessageType === 'participantAdded' ? strings.messageThread.participantJoined : strings.messageThread.participantLeft;
    if (participantsStr !== '') {
        return React.createElement(SystemMessage, { iconName: (message.iconName ? message.iconName : ''), content: `${participantsStr} ${messageSuffix}`, containerStyle: style });
    }
    return React.createElement(React.Fragment, null);
};
const generateParticipantsStr = (participants, defaultName) => participants.map(participant => `${!participant.displayName || participant.displayName === '' ? defaultName : participant.displayName}`).join(', ');

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const MINIMUM_TOUCH_TARGET_HEIGHT_REM$1 = 3;
const errorTextColor = 'var(--errorText)';
/**
 * @private
 */
const iconWrapperStyle = (theme, isSubMenuOpen) => ({
    root: {
        margin: _pxToRem(3),
        // Show hover styles when the Edit/Delete menu is showing as this action button is still considered 'active'
        color: isSubMenuOpen ? theme.palette.black : theme.palette.neutralPrimary,
        strokeWidth: isSubMenuOpen ? _pxToRem(0.5) : _pxToRem(0),
        stroke: theme.palette.black,
        ':hover, :focus': {
            color: theme.palette.black,
            strokeWidth: _pxToRem(0.5)
        }
    }
});
/**
 * @private
 */
const chatMessageDateStyle = (theme) => react.mergeStyles({
    color: theme.palette.neutralSecondary,
    fontWeight: react.FontWeights.regular,
    fontSize: '0.75rem'
});
/**
 * @private
 */
const chatMessageAuthorStyle = react.mergeStyles({
    fontWeight: react.FontWeights.semibold,
    fontSize: '0.75rem'
});
/**
 * @private
 */
const chatMessageDateFailedStyle = (theme) => react.mergeStyles({
    color: theme.palette.neutralPrimaryAlt,
    fontWeight: react.FontWeights.regular,
    fontSize: '0.75rem'
});
/**
 * @private
 */
const chatMessageEditedTagStyle = (theme) => react.mergeStyles({
    fontWeight: react.FontWeights.semibold,
    color: theme.palette.neutralSecondary
});
/**
 * @private
 */
const chatMessageFailedTagStyle = (theme) => react.mergeStyles({
    fontWeight: react.FontWeights.regular,
    color: theme.semanticColors.errorText,
    fontSize: theme.fonts.smallPlus.fontSize,
    lineHeight: '1rem'
});
/**
 * @private
 */
const editChatMessageFailedTagStyle = react.mergeStyles({
    marginBottom: '0.5rem'
});
/**
 * @private
 */
const chatMessageFailedTagStackItemStyle = react.mergeStyles({
    alignSelf: 'end'
});
/**
 * @private
 */
const editChatMessageButtonsStackStyle = react.mergeStyles({
    padding: '0 0.5rem',
    marginTop: '-0.25rem'
});
/**
 * @private
 */
const chatMessageMenuStyle = react.mergeStyles({
    minWidth: '8.5rem',
    height: 'max-content',
    cursor: 'pointer',
    overflow: 'hidden',
    '-webkit-tap-highlight-color': 'transparent' // Disable tap highlight on Android
});
/**
 * @private
 */
const useChatMessageEditContainerStyles = reactComponents.makeStyles({
    body: Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, reactComponents.shorthands.padding(0)), { backgroundColor: 'transparent', boxSizing: 'border-box' }), reactComponents.shorthands.border(`${defaultSendBoxInactiveBorderThicknessREM}rem`, 'solid')), reactComponents.shorthands.borderRadius(reactComponents.tokens.borderRadiusMedium)), reactComponents.shorthands.margin('21px', '1px', '1px', '1px')), { 
        // Width should be updated on hover to include the border width change
        width: `calc(100% - ${defaultSendBoxActiveBorderThicknessREM}rem)`, '&:hover, &:active, &:focus, &:focus-within': Object.assign(Object.assign(Object.assign({}, reactComponents.shorthands.borderWidth(`${defaultSendBoxActiveBorderThicknessREM}rem`)), reactComponents.shorthands.margin('20px', 0, 0, 0)), { width: '100%' }) }),
    bodyAttached: {
        marginBottom: '1px',
        marginTop: '3px',
        // Width should be updated on hover to include the border width change
        width: `calc(100% - ${defaultSendBoxActiveBorderThicknessREM}rem)`,
        '&:hover, &:active, &:focus, &:focus-within': {
            marginTop: '2px',
            marginBottom: 0
        }
    },
    bodyError: Object.assign({}, reactComponents.shorthands.borderColor(errorTextColor)),
    bodyDefault: Object.assign(Object.assign({}, reactComponents.shorthands.borderColor(reactComponents.tokens.colorNeutralStrokeAccessible)), { '&:hover, &:active, &:focus, &:focus-within': Object.assign({}, reactComponents.shorthands.borderColor(reactComponents.tokens.colorCompoundBrandStroke)) })
});
/**
 * @private
 */
reactComponents.makeStyles({
    body: Object.assign(Object.assign(Object.assign(Object.assign({}, reactComponents.shorthands.padding(0)), { backgroundColor: 'transparent' }), reactComponents.shorthands.margin('1.25rem', 0, 0, 0)), { width: '100%' }),
    bodyAttached: {
        marginTop: '0.125rem',
        marginBottom: 0
    }
});
/**
 * Styles that can be applied to ensure flyout items have the minimum touch target size.
 *
 * @private
 */
const menuItemIncreasedSizeStyles = {
    root: {
        height: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM$1}rem`,
        lineHeight: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM$1}rem`,
        maxHeight: 'unset'
    },
    linkContent: {
        height: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM$1}rem`,
        lineHeight: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM$1}rem`,
        maxHeight: 'unset'
    },
    icon: {
        maxHeight: 'unset'
    }
};
/**
 * @private
 */
const menuIconStyleSet = {
    root: {
        height: 'calc(100% - 8px)',
        // Adjust for the pixel margin Fluent applies
        width: '1.25rem'
    }
};
/**
 * @private
 */
const menuSubIconStyleSet = {
    root: {
        height: 'unset',
        lineHeight: '100%',
        width: '1.25rem'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Chat message actions flyout that contains actions such as Edit Message, or Remove Message.
 *
 * @private
 */
const ChatMessageActionFlyout = (props) => {
    var _a, _b;
    const theme = react.useTheme();
    const messageReadByCount = (_a = props.messageReadBy) === null || _a === void 0 ? void 0 : _a.length;
    const sortedMessageReadyByList = [...((_b = props.messageReadBy) !== null && _b !== void 0 ? _b : [])].sort((a, b) => a.displayName.localeCompare(b.displayName));
    const messageReadByList = sortedMessageReadyByList === null || sortedMessageReadyByList === void 0 ? void 0 : sortedMessageReadyByList.map(person => {
        const personaOptions = {
            hidePersonaDetails: true,
            size: react.PersonaSize.size24,
            text: person.displayName,
            showOverflowTooltip: false,
            styles: {
                root: {
                    margin: '0.25rem'
                }
            }
        };
        const { onRenderAvatar } = props;
        return {
            'data-ui-id': 'chat-composite-message-contextual-menu-read-name-list-item',
            key: person.displayName,
            text: person.displayName,
            itemProps: {
                styles: props.increaseFlyoutItemSize ? menuItemIncreasedSizeStyles : undefined
            },
            onRenderIcon: () => {
                var _a;
                if (onRenderAvatar) {
                    const rendered = onRenderAvatar((_a = person.id) !== null && _a !== void 0 ? _a : '', personaOptions);
                    if (rendered) {
                        return rendered;
                    }
                }
                return React.createElement(react.Persona, Object.assign({}, personaOptions));
            },
            iconProps: {
                styles: menuIconStyleSet
            }
        };
    });
    const menuItems = React.useMemo(() => {
        const items = [{
                key: 'Edit',
                'data-testid': 'chat-composite-message-contextual-menu-edit-action',
                text: props.strings.editMessage,
                itemProps: {
                    styles: props.increaseFlyoutItemSize ? menuItemIncreasedSizeStyles : undefined
                },
                iconProps: {
                    iconName: 'MessageEdit',
                    styles: menuIconStyleSet
                },
                onClick: props.onEditClick
            }, {
                key: 'Remove',
                'data-testid': 'chat-composite-message-contextual-menu-remove-action',
                text: props.strings.removeMessage,
                itemProps: {
                    styles: props.increaseFlyoutItemSize ? menuItemIncreasedSizeStyles : undefined
                },
                iconProps: {
                    iconName: 'MessageRemove',
                    styles: menuIconStyleSet
                },
                onClick: props.onRemoveClick
            }];
        // only show read by x of y if more than 3 participants in total including myself
        // only show read by x of y if less than 20 participants in total including myself. This is because read by is not supported for 20 or more participants.
        // TODO: change strings.messageReadCount to be required if we can fallback to our own en-us strings for anything that Contoso doesn't provide
        if (props.remoteParticipantsCount && messageReadByCount !== undefined && props.remoteParticipantsCount >= 2 && props.remoteParticipantsCount < 19 && props.showMessageStatus && props.strings.messageReadCount && props.messageStatus !== 'failed') {
            items.push({
                key: 'Read Count',
                'data-ui-id': 'chat-composite-message-contextual-menu-read-info',
                text: _formatString(props.strings.messageReadCount, {
                    messageReadByCount: `${messageReadByCount}`,
                    remoteParticipantsCount: `${props.remoteParticipantsCount}`
                }),
                itemProps: {
                    styles: react.concatStyleSets({
                        linkContent: {
                            color: messageReadByCount > 0 ? theme.palette.neutralPrimary : theme.palette.neutralTertiary
                        },
                        root: {
                            borderTop: `1px solid ${theme.palette.neutralLighter}`
                        }
                    }, props.increaseFlyoutItemSize ? menuItemIncreasedSizeStyles : undefined)
                },
                calloutProps: preventUnwantedDismissProps,
                subMenuProps: {
                    items: messageReadByList !== null && messageReadByList !== void 0 ? messageReadByList : [],
                    calloutProps: preventUnwantedDismissProps,
                    styles: react.concatStyleSets({
                        root: {
                            maxWidth: _pxToRem(320),
                            span: {
                                overflow: 'hidden',
                                textOverflow: 'ellipsis'
                            }
                        }
                    })
                },
                iconProps: {
                    iconName: 'MessageSeen',
                    styles: {
                        root: {
                            color: messageReadByCount > 0 ? theme.palette.themeDarkAlt : theme.palette.neutralTertiary
                        }
                    }
                },
                submenuIconProps: {
                    iconName: 'HorizontalGalleryRightButton',
                    styles: menuSubIconStyleSet
                },
                disabled: messageReadByCount <= 0
            });
        }
        else if (props.messageStatus === 'failed' && props.strings.resendMessage) {
            items.push({
                key: 'Resend',
                text: props.strings.resendMessage,
                itemProps: {
                    styles: react.concatStyleSets({
                        linkContent: {
                            color: theme.palette.neutralPrimary
                        },
                        root: {
                            borderTop: `1px solid ${theme.palette.neutralLighter}`
                        }
                    }, props.increaseFlyoutItemSize ? menuItemIncreasedSizeStyles : undefined)
                },
                calloutProps: preventUnwantedDismissProps,
                iconProps: {
                    iconName: 'MessageResend',
                    styles: {
                        root: {
                            color: theme.palette.themeDarkAlt
                        }
                    }
                },
                onClick: props.onResendClick
            });
        }
        return items;
    }, [props.strings.editMessage, props.strings.removeMessage, props.strings.messageReadCount, props.strings.resendMessage, props.messageStatus, props.increaseFlyoutItemSize, props.onEditClick, props.onRemoveClick, props.onResendClick, props.remoteParticipantsCount, props.showMessageStatus, messageReadByCount, theme.palette.neutralPrimary, theme.palette.neutralTertiary, theme.palette.neutralLighter, theme.palette.themeDarkAlt, messageReadByList]);
    // gap space uses pixels
    return React.createElement(react.ContextualMenu, { alignTargetEdge: true, gapSpace: 2 /*px*/, isBeakVisible: false, items: menuItems, hidden: props.hidden, target: props.target, onDismiss: props.onDismiss, directionalHint: react.DirectionalHint.topRightEdge, className: chatMessageMenuStyle, calloutProps: calloutMenuProps });
};
/**
 * Similar to {@link preventDismissOnEvent}, but not prevent dismissing from scrolling, since it is causing bugs in chat thread.
 */
const preventUnwantedDismissProps = {
    preventDismissOnEvent: (ev) => {
        return ev.type === 'resize';
    }
};
const calloutMenuProps = Object.assign(Object.assign({}, preventUnwantedDismissProps), { styles: {
        root: {
            marginRight: '3px'
        }
    } });

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Props for the Chat.Message action menu.
 * This is the 3 dots that appear when hovering over one of your own chat messages.
 *
 * @private
 */
const chatMessageActionMenuProps = (menuProps) => {
    const { ariaLabel, enabled, theme, menuExpanded } = menuProps;
    // Show the action button while the flyout is open (otherwise this will dismiss when the pointer is hovered over the flyout)
    const showActionMenu = enabled || menuExpanded;
    const actionMenuProps = {
        children: React.createElement("div", { tabIndex: showActionMenu ? 0 : undefined, key: "menuButton", "data-testid": "chat-composite-message-action-icon", ref: menuProps.menuButtonRef, onClick: showActionMenu ? () => menuProps.onActionButtonClick() : undefined, style: {
                margin: showActionMenu ? '1px' : 0,
                minHeight: showActionMenu ? undefined : '30px'
            }, role: "button", "aria-label": showActionMenu ? ariaLabel : undefined, "aria-haspopup": showActionMenu, "aria-expanded": menuExpanded, onKeyDown: e => {
                // simulate <button> tag behavior
                if (showActionMenu && (e.key === 'Enter' || e.key === ' ')) {
                    menuProps.onActionButtonClick();
                }
            } }, showActionMenu ? React.createElement(react.Icon, { iconName: "ChatMessageOptions", "aria-label": ariaLabel, styles: iconWrapperStyle(theme, menuExpanded) }) : undefined)
    };
    return actionMenuProps;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const ChatMessageContent = (props) => {
    switch (props.message.contentType) {
        case 'text':
            return MessageContentAsText(props);
        case 'html':
            return MessageContentAsRichTextHTML(props);
        case 'richtext/html':
            return MessageContentAsRichTextHTML(props);
        default:
            console.warn('unknown message content type');
            return React.createElement(React.Fragment, null);
    }
};
const MessageContentWithLiveAria = (props) => {
    return React.createElement("div", { "data-ui-status": props.message.status, role: "text", "aria-label": props.ariaLabel, className: props.className },
        React.createElement(LiveMessage, { message: props.liveMessage, ariaLive: "polite" }),
        props.content);
};
const MessageContentAsRichTextHTML = (props) => {
    return React.createElement(MessageContentWithLiveAria, { message: props.message, liveMessage: generateLiveMessage(props), ariaLabel: messageContentAriaText(props), content: processHtmlToReact(props) });
};
const MessageContentAsText = (props) => {
    return React.createElement(MessageContentWithLiveAria, { message: props.message, liveMessage: generateLiveMessage(props), ariaLabel: messageContentAriaText(props), className: messageTextContentStyles, content: React.createElement(Linkify, { componentDecorator: (decoratedHref, decoratedText, key) => {
                return React.createElement(react.Link, { target: "_blank", href: decoratedHref, key: key }, decoratedText);
            } }, props.message.content) });
};
const extractContentForAllyMessage = (props) => {
    var _a;
    if (props.message.content || props.message.attachments) {
        // Replace all <img> tags with 'image' for aria.
        const parsedContent = DOMPurify.sanitize((_a = props.message.content) !== null && _a !== void 0 ? _a : '', {
            ALLOWED_TAGS: ['img'],
            RETURN_DOM_FRAGMENT: true
        });
        parsedContent.childNodes.forEach(child => {
            if (child.nodeName.toLowerCase() !== 'img') {
                return;
            }
            const imageTextNode = document.createElement('div');
            imageTextNode.innerHTML = 'image ';
            parsedContent.replaceChild(imageTextNode, child);
        });
        // Inject message attachment count for aria.
        // this is only applying to file attachments not for inline images.
        if (props.message.attachments && props.message.attachments.length > 0) {
            const attachmentCardDescription = attachmentCardGroupDescription(props);
            const attachmentTextNode = document.createElement('div');
            attachmentTextNode.innerHTML = `${attachmentCardDescription}`;
            parsedContent.appendChild(attachmentTextNode);
        }
        // Strip all html tags from the content for aria.
        let message = DOMPurify.sanitize(parsedContent, {
            ALLOWED_TAGS: []
        });
        // decode HTML entities so that screen reader can read the content properly.
        message = decodeEntities(message);
        return message;
    }
    return '';
};
const generateLiveMessage = (props) => {
    const messageContent = extractContentForAllyMessage(props);
    if (props.message.editedOn) {
        const liveAuthor = _formatString(props.strings.editedMessageLiveAuthorIntro, {
            author: `${props.message.senderDisplayName}`
        });
        return `${props.message.mine ? props.strings.editedMessageLocalUserLiveAuthorIntro : liveAuthor} ${messageContent}`;
    }
    else {
        const liveAuthor = _formatString(props.strings.liveAuthorIntro, {
            author: `${props.message.senderDisplayName}`
        });
        return `${props.message.mine ? '' : liveAuthor} ${messageContent} `;
    }
};
const messageContentAriaText = (props) => {
    var _a, _b;
    const message = extractContentForAllyMessage(props);
    return props.message.mine ? _formatString(props.strings.messageContentMineAriaText, {
        status: (_a = props.message.status) !== null && _a !== void 0 ? _a : '',
        message: message
    }) : _formatString(props.strings.messageContentAriaText, {
        status: (_b = props.message.status) !== null && _b !== void 0 ? _b : '',
        author: `${props.message.senderDisplayName}`,
        message: message
    });
};
const attachmentCardGroupDescription = (props) => {
    const attachments = props.message.attachments;
    return getAttachmentCountLiveMessage(attachments !== null && attachments !== void 0 ? attachments : [], props.strings.attachmentCardGroupMessage);
};
/**
 * @private
 */
const getAttachmentCountLiveMessage = (attachments, attachmentCardGroupMessage) => {
    if (attachments.length === 0) {
        return '';
    }
    return _formatString(attachmentCardGroupMessage, {
        attachmentCount: `${attachments.length}`
    });
};
const defaultOnRenderInlineImage = (inlineImage) => {
    return React.createElement("img", Object.assign({ key: inlineImage.imageAttributes.id, tabIndex: 0, "data-ui-id": inlineImage.imageAttributes.id }, inlineImage.imageAttributes));
};
const processHtmlToReact = (props) => {
    var _a;
    const options = {
        transform(reactNode, domNode) {
            var _a;
            if (domNode instanceof parse.Element && domNode.attribs) {
                // Transform custom rendering of mentions
                // Transform inline images
                if (domNode.name && domNode.name === 'img' && domNode.attribs && domNode.attribs.id) {
                    if (domNode.attribs.name) {
                        domNode.attribs['aria-label'] = domNode.attribs.name;
                    }
                    const imgProps = parse.attributesToProps(domNode.attribs);
                    const inlineImageProps = {
                        messageId: props.message.messageId,
                        imageAttributes: imgProps
                    };
                    return ((_a = props.inlineImageOptions) === null || _a === void 0 ? void 0 : _a.onRenderInlineImage) ? props.inlineImageOptions.onRenderInlineImage(inlineImageProps, defaultOnRenderInlineImage) : defaultOnRenderInlineImage(inlineImageProps);
                }
                // Transform links to open in new tab
                if (domNode.name === 'a' && React.isValidElement(reactNode)) {
                    return React.cloneElement(reactNode, {
                        target: '_blank',
                        rel: 'noreferrer noopener'
                    });
                }
            }
            // Pass through the original node
            return reactNode;
        }
    };
    return React.createElement(React.Fragment, null, parse((_a = props.message.content) !== null && _a !== void 0 ? _a : '', options));
};
const decodeEntities = (encodedString) => {
    // This regular expression matches HTML entities.
    const translate_re = /&(nbsp|amp|quot|lt|gt);/g;
    // This object maps HTML entities to their respective characters.
    const translate = {
        nbsp: ' ',
        amp: '&',
        quot: '"',
        lt: '<',
        gt: '>'
    };
    return encodedString
        // Find all matches of HTML entities defined in translate_re and
        // replace them with the corresponding character from the translate object.
        .replace(translate_re, function (match, entity) {
        var _a;
        return (_a = translate[entity]) !== null && _a !== void 0 ? _a : match;
    })
        // Find numeric entities (e.g., &#65;)
        // and replace them with the equivalent character using the String.fromCharCode method,
        // which converts Unicode values into characters.
        .replace(/&#(\d+);/gi, function (match, numStr) {
        const num = parseInt(numStr, 10);
        return String.fromCharCode(num);
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Conditionally modify locale strings passed to the attachment card
 * @returns attachment card strings
 */
const useLocaleAttachmentCardStringsTrampoline = () => {
    return {
        removeAttachment: '',
        uploadCompleted: '',
        uploading: '',
        attachmentMoreMenu: ''
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$L = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @internal
 * A component for displaying an attachment card with attachment icon and progress bar.
 *
 * `_AttachmentCard` internally uses the `Card` component from `@fluentui/react-components`. You can checkout the details about these components [here](https://react.fluentui.dev/?path=/docs/components-card).
 */
const _AttachmentCard = (props) => {
    var _a, _b, _c, _d, _e;
    const { attachment, menuActions, onActionHandlerFailed, selfResizing } = props;
    const attachmentCardStyles = useAttachmentCardStyles();
    const progress = React.useMemo(() => {
        return 'progress' in attachment ? attachment.progress : undefined;
    }, [attachment]);
    const isUploadInProgress = React.useMemo(() => {
        return progress !== undefined && progress >= 0 && progress < 1;
    }, [progress]);
    const [announcerString, setAnnouncerString] = React.useState(undefined);
    const localeStrings = useLocaleAttachmentCardStringsTrampoline();
    const uploadStartedString = (_b = (_a = props.strings) === null || _a === void 0 ? void 0 : _a.uploading) !== null && _b !== void 0 ? _b : localeStrings.uploading;
    const uploadCompletedString = (_d = (_c = props.strings) === null || _c === void 0 ? void 0 : _c.uploadCompleted) !== null && _d !== void 0 ? _d : localeStrings.uploadCompleted;
    React.useEffect(() => {
        if (isUploadInProgress) {
            setAnnouncerString(`${uploadStartedString} ${attachment.name}`);
        }
        else if (progress === 1) {
            setAnnouncerString(`${attachment.name} ${uploadCompletedString}`);
        }
        else {
            setAnnouncerString(undefined);
        }
    }, [progress, isUploadInProgress, attachment.name, uploadStartedString, uploadCompletedString]);
    const extension = React.useMemo(() => {
        const re = /(?:\.([^.]+))?$/;
        const match = re.exec(attachment.name);
        return match && match[1] ? match[1] : '';
    }, [attachment]);
    return React.createElement("div", { "data-is-focusable": true },
        React.createElement(reactComponents.Card, { className: reactComponents.mergeClasses(attachmentCardStyles.root, selfResizing ? attachmentCardStyles.dynamicWidth : attachmentCardStyles.staticWidth), size: "small", role: "listitem", appearance: "filled-alternative", "aria-label": attachment.name, "data-testid": 'attachment-card' },
            React.createElement(Announcer$1, { announcementString: announcerString, ariaLive: 'polite' }),
            React.createElement(reactComponents.CardHeader, { className: attachmentCardStyles.content, image: {
                    className: attachmentCardStyles.fileIcon,
                    children: React.createElement(react.Icon, { "data-ui-id": 'attachmenttype-icon', iconName: reactFileTypeIcons.getFileTypeIconProps({
                            extension: extension,
                            size: 24,
                            imageFileType: 'svg'
                        }).iconName })
                }, header: {
                    id: 'attachment-' + attachment.id,
                    children: React.createElement(react.TooltipHost, { content: attachment.name, calloutProps: {
                            gapSpace: 0,
                            target: '#attachment-' + attachment.id
                        }, hostClassName: titleTooltipContainerStyle },
                        React.createElement(reactComponents.Text, { className: attachmentCardStyles.title, "aria-label": attachment.name }, attachment.name))
                }, action: {
                    className: attachmentCardStyles.actions,
                    children: MappedMenuItems(menuActions, Object.assign(Object.assign({}, attachment), { url: (_e = attachment.url) !== null && _e !== void 0 ? _e : '' }), onActionHandlerFailed)
                } })),
        isUploadInProgress ? React.createElement(reactComponents.CardFooter, null,
            React.createElement(reactComponents.ProgressBar, { thickness: "medium", value: Math.max(progress !== null && progress !== void 0 ? progress : 0, ATTACHMENT_CARD_MIN_PROGRESS), shape: "rounded" })) : React.createElement(React.Fragment, null, " "));
};
const MappedMenuItems = (menuActions, attachment, handleOnClickError) => {
    const localeStrings = useLocaleAttachmentCardStringsTrampoline();
    const toolTipHostCloseDelayMs = 500;
    if (!menuActions[0]) {
        return React.createElement(React.Fragment, null);
    }
    return menuActions.length === 1 ? React.createElement(react.TooltipHost, { content: menuActions[0].name, closeDelay: toolTipHostCloseDelayMs },
        React.createElement(reactComponents.ToolbarButton, { "aria-label": menuActions[0].name, role: "button", icon: menuActions[0].icon, onClick: () => {
                var _a;
                try {
                    (_a = menuActions[0]) === null || _a === void 0 ? void 0 : _a.onClick(attachment);
                }
                catch (e) {
                    handleOnClickError === null || handleOnClickError === void 0 ? void 0 : handleOnClickError(e.message);
                }
            } })) : React.createElement(reactComponents.Toolbar, null,
        React.createElement(reactComponents.Menu, null,
            React.createElement(react.TooltipHost, { content: localeStrings.attachmentMoreMenu, closeDelay: toolTipHostCloseDelayMs },
                React.createElement(reactComponents.MenuTrigger, null,
                    React.createElement(reactComponents.ToolbarButton, { "aria-label": localeStrings.attachmentMoreMenu, role: "button", icon: React.createElement(react.Icon, { iconName: "AttachmentMoreMenu" }) }))),
            React.createElement(reactComponents.MenuPopover, null,
                React.createElement(reactComponents.MenuList, null, menuActions.map((menuItem, index) => React.createElement(reactComponents.MenuItem, { "aria-label": menuItem.name, key: index, icon: menuItem.icon, onClick: () => __awaiter$L(void 0, void 0, void 0, function* () {
                        try {
                            yield menuItem.onClick(attachment);
                        }
                        catch (e) {
                            handleOnClickError === null || handleOnClickError === void 0 ? void 0 : handleOnClickError(e.message);
                        }
                    }) }, menuItem.name))))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useAttachmentCardGroupStyles = reactComponents.makeStyles({
    root: {
        marginTop: '0.25rem'
    },
    singleAttachment: {
        width: `${_ATTACHMENT_CARD_WIDTH_IN_REM}rem`
    },
    multipleAttachments: {
        width: '100%'
    }
});
/**
 * @private
 */
const attachmentCardBaseStyles = react.mergeStyles({
    '& > *': {
        margin: `${_pxToRem(_ATTACHMENT_CARD_MARGIN_IN_PX)}`
    },
    /**
     * margin for children is overriden by parent stack, so adding left margin for each child
     */
    '& > *:not(:first-child)': {
        marginLeft: `${_pxToRem(_ATTACHMENT_CARD_MARGIN_IN_PX)}`
    }
});
/**
 * @private
 */
const attachmentCardGirdLayout = react.mergeStyles({
    display: 'grid',
    gridTemplateColumns: `repeat(auto-fit, minmax(${_ATTACHMENT_CARD_WIDTH_IN_REM}rem, 1fr))`
});
/**
 * @private
 */
const attachmentCardFlexLayout = react.mergeStyles({
    display: 'flex',
    flexWrap: 'wrap'
});
/**
 * @private
 */
const attachmentGroupDisabled = react.mergeStyles({
    opacity: '0.5',
    userSelect: 'none',
    pointerEvents: 'none'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * Props for `_AttachmentCardGroup` component.
 */
var _AttachmentCardGroupLayout;
(function (_AttachmentCardGroupLayout) {
    /**
     * Children are rendered in a grid layout with self resizing.
     */
    _AttachmentCardGroupLayout["Grid"] = "grid";
    /**
     * Children are rendered in a flex layout with no resizing.
     */
    _AttachmentCardGroupLayout["Flex"] = "flex";
})(_AttachmentCardGroupLayout || (_AttachmentCardGroupLayout = {}));
/**
 * @internal
 * Used with `_AttachmentCard` component where `_AttachmentCard` components are passed as children.
 * Renders the children equally spaced in multiple rows.
 */
const _AttachmentCardGroup = (props) => {
    const { children, ariaLabel, attachmentGroupLayout, disabled } = props;
    if (!children) {
        return React.createElement(React.Fragment, null);
    }
    return React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles(disabled && attachmentGroupDisabled, attachmentCardBaseStyles, attachmentGroupLayout === _AttachmentCardGroupLayout.Grid ? attachmentCardGirdLayout : attachmentCardFlexLayout), "aria-label": ariaLabel, role: "list" }, children);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 */
const _AttachmentDownloadCards = (props) => {
    const { attachments, message } = props;
    const localeStrings = useLocale$1().strings.messageThread;
    const attachmentCardGroupStyles = useAttachmentCardGroupStyles();
    const getMenuActions = React.useCallback((attachment, localeStrings, message, action) => {
        const defaultMenuActions = getDefaultMenuActions(localeStrings, message);
        try {
            const actions = action === null || action === void 0 ? void 0 : action(attachment, message);
            if (actions && actions.length > 0) {
                return actions;
            }
            else {
                return defaultMenuActions;
            }
        }
        catch (_a) {
            return defaultMenuActions;
        }
    }, []);
    const hasMultipleAttachments = React.useMemo(() => {
        var _a, _b;
        return ((_b = (_a = props.attachments) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 1;
    }, [props.attachments]);
    if (!attachments || attachments.length === 0 || !attachments) {
        return React.createElement(React.Fragment, null);
    }
    return React.createElement("div", { className: react$1.mergeClasses(attachmentCardGroupStyles.root, hasMultipleAttachments ? attachmentCardGroupStyles.multipleAttachments : attachmentCardGroupStyles.singleAttachment), "data-ui-id": "attachment-download-card-group" },
        React.createElement(_AttachmentCardGroup, { attachmentGroupLayout: _AttachmentCardGroupLayout.Grid }, attachments && attachments.map(attachment => React.createElement(_AttachmentCard, { attachment: attachment, key: attachment.id, menuActions: getMenuActions(attachment, localeStrings, message, props.actionsForAttachment), onActionHandlerFailed: props.onActionHandlerFailed, selfResizing: hasMultipleAttachments }))));
};
/**
 * @private
 */
const getDefaultMenuActions = (locale, chatMessage) => {
    let actionName = locale.openAttachment;
    // if message is sent by a Teams user, we need to use a different icon ("open")
    const isTeamsUser = _isIdentityMicrosoftTeamsUser(chatMessage === null || chatMessage === void 0 ? void 0 : chatMessage.senderId);
    if (isTeamsUser) {
        return [{
                name: actionName,
                icon: React.createElement(react.Icon, { iconName: "OpenAttachment" }),
                onClick: defaultOnClickHandler
            }];
    }
    // otherwise, use the default icon ("download")
    return [Object.assign(Object.assign({}, defaultAttachmentMenuAction), { name: actionName })];
};
/**
 *
 * The default action handler for downloading attachments. This handler will open the attachment's URL in a new tab.
 */
const defaultOnClickHandler = (attachment) => {
    return new Promise(resolve => {
        window.open(attachment.url, '_blank', 'noopener,noreferrer');
        resolve();
    });
};
/**
 * @beta
 *
 * The default menu action for downloading attachments. This action will open the attachment's URL in a new tab.
 */
const defaultAttachmentMenuAction = {
    /**
     *
     * name is used for aria-label only when there's one button. For multiple buttons, it's used as a label.
     * by default it's an unlocalized string when this is used as a imported constant,
     * but you can overwrite it with your own localized string.
     *
     * i.e. defaultAttachmentMenuAction.name = localize('Download');
     *
     * when no action is provided, the UI library will overwrite this name
     * with a localized string this string when it's used in the UI.
     */
    name: 'Download',
    // this is the icon shown on the right of the attachment card
    icon: React.createElement(react.Icon, { iconName: "DownloadAttachment", "data-ui-id": "attachment-download-card-download-icon" }),
    // this is the action that runs when the icon is clicked
    onClick: defaultOnClickHandler
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const formatTimeForChatMessage = (messageDate) => {
    return messageDate.toLocaleTimeString([], {
        hour: 'numeric',
        minute: '2-digit'
    });
};
/**
 * @private
 */
const formatDateForChatMessage = (messageDate) => {
    return messageDate.toLocaleDateString();
};
/**
 * Given a message date object in ISO8601 and a current date object, generates a user friendly timestamp text
 * using the system locale.
 * <time in locale format>.
 * Yesterday <time in locale format>.
 * <dateStrings day of week> <time in locale format>.
 * <date in locale format> <time in locale format>.
 *
 * If message is after yesterday, then only show the time.
 * If message is before yesterday and after day before yesterday, then show 'Yesterday' plus the time.
 * If message is before day before yesterday and within the current week, then show 'Monday/Tuesday/etc' plus the time.
 *   - We consider start of the week as Sunday. If current day is Sunday, then any time before that is in previous week.
 * If message is in previous or older weeks, then show date string plus the time.
 *
 * @param messageDate - date of message
 * @param currentDate - date used as offset to create the user friendly timestamp (e.g. to create 'Yesterday' instead of an absolute date)
 *
 * @private
 */
const formatTimestampForChatMessage = (messageDate, todayDate, dateStrings) => {
    // If message was in the same day timestamp string is just the time like '1:30 p.m.'.
    const startOfDay = new Date(todayDate.getFullYear(), todayDate.getMonth(), todayDate.getDate());
    if (messageDate > startOfDay) {
        return formatTimeForChatMessage(messageDate);
    }
    // If message was yesterday then timestamp string is like this 'Yesterday 1:30 p.m.'.
    const yesterdayDate = new Date(todayDate.getFullYear(), todayDate.getMonth(), todayDate.getDate() - 1);
    if (messageDate > yesterdayDate) {
        return dateStrings.yesterday + ' ' + formatTimeForChatMessage(messageDate);
    }
    // If message was before Sunday and today is Sunday (start of week) then timestamp string is like
    // '2021-01-10 1:30 p.m.'.
    const weekDay = todayDate.getDay();
    if (weekDay === 0) {
        return formatDateForChatMessage(messageDate) + ' ' + formatTimeForChatMessage(messageDate);
    }
    // If message was before first day of the week then timestamp string is like Monday 1:30 p.m.
    const firstDayOfTheWeekDate = new Date(todayDate.getFullYear(), todayDate.getMonth(), todayDate.getDate() - weekDay);
    if (messageDate > firstDayOfTheWeekDate) {
        return dayToDayName(messageDate.getDay(), dateStrings) + ' ' + formatTimeForChatMessage(messageDate);
    }
    // If message date is in previous or older weeks then timestamp string is like 2021-01-10 1:30 p.m.
    return formatDateForChatMessage(messageDate) + ' ' + formatTimeForChatMessage(messageDate);
};
const dayToDayName = (day, dateStrings) => {
    switch (day) {
        case 0:
            return dateStrings.sunday;
        case 1:
            return dateStrings.monday;
        case 2:
            return dateStrings.tuesday;
        case 3:
            return dateStrings.wednesday;
        case 4:
            return dateStrings.thursday;
        case 5:
            return dateStrings.friday;
        case 6:
            return dateStrings.saturday;
        default:
            throw new Error(`Invalid day [${day}] passed`);
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private
 * Return the string value for the FluentUI message attached prop based on the message's attached status.
 * @param attached - The message's attached status.
 */
const getFluentUIAttachedValue = (messageAttachedStatus) => {
    return messageAttachedStatus === 'top' || messageAttachedStatus === false ? 'top' : 'center';
};
/**
 * @private
 *  Get the message bubble content for the message.
 */
function getMessageBubbleContent(message, strings, userId, inlineImageOptions, 
/**
 * Optional callback to render message attachments in the message component.
 */
onRenderAttachmentDownloads, 
/**
 * Optional callback to define custom actions for attachments.
 */
actionsForAttachment) {
    return React.createElement("div", { className: "ui-chat__message__content" },
        React.createElement(ChatMessageContent, { message: message, strings: strings, inlineImageOptions: inlineImageOptions }),
        onRenderAttachmentDownloads ? onRenderAttachmentDownloads(message) : defaultOnRenderAttachmentDownloads(message, strings, actionsForAttachment));
}
/**
 * Default component for rendering attachment downloads.
 */
const defaultOnRenderAttachmentDownloads = (message, strings, actionsForAttachment) => {
    var _a;
    const attachments = 'attachments' in message ? message.attachments : undefined;
    return ((_a = attachments === null || attachments === void 0 ? void 0 : attachments.length) !== null && _a !== void 0 ? _a : 0) > 0 ? React.createElement(_AttachmentDownloadCards, { message: message, attachments: attachments, actionsForAttachment: actionsForAttachment, strings: {
            openAttachment: strings.openAttachment,
            attachmentCardGroupMessage: strings.attachmentCardGroupMessage
        } }) : React.createElement(React.Fragment, null);
};
/** @private */
const generateDefaultTimestamp = (createdOn, showDate, strings) => {
    const formattedTimestamp = showDate ? formatTimestampForChatMessage(createdOn, new Date(), strings) : formatTimeForChatMessage(createdOn);
    return formattedTimestamp;
};
// onDisplayDateTimeString from props overwrite onDisplayDateTimeString from locale
/** @private */
const generateCustomizedTimestamp = (createdOn, locale, onDisplayDateTimeString) => {
    return '';
};
/**
 * @private
 *  Get the edited tag for the message if it is edited.
 */
const getMessageEditedDetails = (message, theme, editedTag) => {
    const editedOn = 'editedOn' in message ? message.editedOn : undefined;
    if (message.messageType === 'chat' && editedOn) {
        return React.createElement("div", { className: chatMessageEditedTagStyle(theme) }, editedTag);
    }
    return undefined;
};
/**
 * Removes Fluent UI keyboard navigation styles from the given HTMLDivElement.
 *
 * This function removes the 'tabindex' and 'data-tabster' attributes from the provided
 * HTMLDivElement, making it non-focusable and removing any associated keyboard navigation styles.
 *
 * @param node - The HTMLDivElement from which to remove the keyboard navigation styles. If null, the function does nothing.
 */
const removeFluentUIKeyboardNavigationStyles = (node) => {
    if (node) {
        // Remove tabindex to make the div element non-focusable
        node.removeAttribute('tabindex');
        node.removeAttribute('data-tabster');
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const MessageBubble$1 = (props) => {
    var _a;
    const ids = useIdentifiers();
    const theme = useTheme();
    const locale = useLocale$1();
    const { userId, message, onRemoveClick, onResendClick, disableEditing, showDate, messageContainerStyle, strings, onEditClick, remoteParticipantsCount = 0, onRenderAvatar, showMessageStatus, messageStatus, inlineImageOptions, onDisplayDateTimeString, onRenderAttachmentDownloads, actionsForAttachment, shouldFocusFluentMessageBody } = props;
    const formattedTimestamp = React.useMemo(() => {
        const defaultTimeStamp = message.createdOn ? generateDefaultTimestamp(message.createdOn, showDate, strings) : undefined;
        const customTimestamp = message.createdOn ? generateCustomizedTimestamp(message.createdOn) : '';
        return customTimestamp || defaultTimeStamp;
    }, [locale, message.createdOn, onDisplayDateTimeString, showDate, strings]);
    // Track if the action menu was opened by touch - if so we increase the touch targets for the items
    const [wasInteractionByTouch, setWasInteractionByTouch] = React.useState(false);
    // `focused` state is used for show/hide actionMenu
    const [focused, setFocused] = React.useState(false);
    // The chat message action flyout should target the Chat.Message action menu if clicked,
    // or target the chat message if opened via touch press.
    // Undefined indicates the flyout menu should not be being shown.
    const messageRef = React.useRef(null);
    const messageActionButtonRef = React.useRef(null);
    const [chatMessageActionFlyoutTarget, setChatMessageActionFlyoutTarget] = React.useState(undefined);
    const chatActionsEnabled = !disableEditing && message.status !== 'sending' && !!message.mine;
    const [messageReadBy, setMessageReadBy] = React.useState([]);
    const actionMenuProps = chatMessageActionMenuProps({
        ariaLabel: (_a = strings.actionMenuMoreOptions) !== null && _a !== void 0 ? _a : '',
        enabled: chatActionsEnabled,
        menuButtonRef: messageActionButtonRef,
        menuExpanded: chatMessageActionFlyoutTarget === messageActionButtonRef,
        onActionButtonClick: () => {
            if (message.messageType === 'chat') {
                props.onActionButtonClick(message, setMessageReadBy);
                setChatMessageActionFlyoutTarget(messageActionButtonRef);
            }
        },
        theme
    });
    React.useEffect(() => {
        if (shouldFocusFluentMessageBody) {
            // set focus in the next render cycle to avoid focus being stolen by other components
            setTimeout(() => {
                var _a;
                (_a = messageRef.current) === null || _a === void 0 ? void 0 : _a.focus();
            });
        }
    }, [shouldFocusFluentMessageBody]);
    const onActionFlyoutDismiss = React.useCallback(() => {
        // When the flyout dismiss is called, since we control if the action flyout is visible
        // or not we need to set the target to undefined here to actually hide the action flyout
        setChatMessageActionFlyoutTarget(undefined);
    }, [setChatMessageActionFlyoutTarget]);
    const getMessageDetails = React.useCallback(() => {
        if (messageStatus === 'failed') {
            return React.createElement("div", { className: chatMessageFailedTagStyle(theme) }, strings.failToSendTag);
        }
        else {
            return getMessageEditedDetails(message, theme, strings.editedTag);
        }
    }, [message, messageStatus, strings.editedTag, strings.failToSendTag, theme]);
    const chatMyMessageStyles = useChatMyMessageStyles();
    const chatMessageCommonStyles = useChatMessageCommonStyles();
    const attached = message.attached === true ? 'center' : message.attached === 'bottom' ? 'bottom' : 'top';
    const getActionsMenu = React.useCallback(() => {
        return React.createElement("div", { className: reactComponents.mergeClasses(
            // add the static class name to use it in useChatMyMessageStyles
            chatMyMessageActionMenuClassName, chatMyMessageStyles.menu, 
            // Make actions menu visible when the message is focused or the flyout is shown
            focused || (chatMessageActionFlyoutTarget === null || chatMessageActionFlyoutTarget === void 0 ? void 0 : chatMessageActionFlyoutTarget.current) ? chatMyMessageStyles.menuVisible : chatMyMessageStyles.menuHidden) }, actionMenuProps === null || actionMenuProps === void 0 ? void 0 : actionMenuProps.children);
    }, [actionMenuProps === null || actionMenuProps === void 0 ? void 0 : actionMenuProps.children, chatMessageActionFlyoutTarget, chatMyMessageStyles.menu, chatMyMessageStyles.menuHidden, chatMyMessageStyles.menuVisible, focused]);
    const getContent = React.useCallback(() => {
        return React.createElement("div", null,
            getMessageBubbleContent(message, strings, userId, inlineImageOptions, onRenderAttachmentDownloads, actionsForAttachment),
            getActionsMenu());
    }, [actionsForAttachment, getActionsMenu, inlineImageOptions, message, onRenderAttachmentDownloads, strings, userId]);
    const chatMessage = React.createElement(React.Fragment, null,
        React.createElement("div", { key: props.message.messageId },
            React.createElement(reactChat.ChatMyMessage, { attached: attached, key: props.message.messageId, body: {
                    // messageContainerStyle used in className and style prop as style prop can't handle CSS selectors
                    className: reactComponents.mergeClasses(chatMessageCommonStyles.body, chatMyMessageStyles.body, props.message.status === 'failed' ? chatMessageCommonStyles.failed : undefined, attached !== 'top' ? chatMyMessageStyles.bodyAttached : undefined, react.mergeStyles(messageContainerStyle)),
                    style: Object.assign({}, createStyleFromV8Style(messageContainerStyle)),
                    ref: messageRef
                }, root: {
                    className: chatMyMessageStyles.root,
                    onBlur: e => {
                        // `focused` controls is focused the whole `ChatMessage` or any of its children. When we're navigating
                        // with keyboard the focused element will be changed and there is no way to use `:focus` selector
                        if (chatMessageActionFlyoutTarget === null || chatMessageActionFlyoutTarget === void 0 ? void 0 : chatMessageActionFlyoutTarget.current) {
                            // doesn't dismiss action button if flyout is open, otherwise, narrator's focus will stay on the closed action menu
                            return;
                        }
                        const shouldPreserveFocusState = e.currentTarget.contains(e.relatedTarget);
                        setFocused(shouldPreserveFocusState);
                    },
                    onFocus: () => {
                        // react onFocus is called even when nested component receives focus (i.e. it bubbles)
                        // so when focus moves within actionMenu, the `focus` state in chatMessage remains true, and keeps actionMenu visible
                        setFocused(true);
                    }
                }, "data-testid": "chat-composite-message", author: React.createElement(react.Text, { className: chatMessageDateStyle(theme) }, message.senderDisplayName), timestamp: React.createElement(react.Text, { className: props.message.status === 'failed' ? chatMessageDateFailedStyle(theme) : chatMessageDateStyle(theme), "data-testid": ids.messageTimestamp }, formattedTimestamp), details: getMessageDetails(), onTouchStart: () => setWasInteractionByTouch(true), onPointerDown: () => setWasInteractionByTouch(false), onKeyDown: () => setWasInteractionByTouch(false), onClick: () => {
                    if (!wasInteractionByTouch) {
                        return;
                    }
                    // If the message was touched via touch we immediately open the menu
                    // flyout (when using mouse the 3-dot menu that appears on hover
                    // must be clicked to open the flyout).
                    // In doing so here we set the target of the flyout to be the message and
                    // not the 3-dot menu button to position the flyout correctly.
                    setChatMessageActionFlyoutTarget(messageRef);
                    if (message.messageType === 'chat') {
                        props.onActionButtonClick(message, setMessageReadBy);
                    }
                } }, getContent())),
        chatActionsEnabled && React.createElement(ChatMessageActionFlyout, { hidden: !chatMessageActionFlyoutTarget, target: chatMessageActionFlyoutTarget, increaseFlyoutItemSize: wasInteractionByTouch, onDismiss: onActionFlyoutDismiss, onEditClick: onEditClick, onRemoveClick: onRemoveClick, onResendClick: onResendClick, strings: strings, messageReadBy: messageReadBy, messageStatus: messageStatus !== null && messageStatus !== void 0 ? messageStatus : 'failed', remoteParticipantsCount: remoteParticipantsCount, onRenderAvatar: onRenderAvatar, showMessageStatus: showMessageStatus }));
    return chatMessage;
};
/** @private */
const ChatMyMessageComponentAsMessageBubble = React.memo(MessageBubble$1);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const editBoxStyle = react.mergeStyles({
    marginTop: '0.0875rem',
    marginBottom: '0.0875rem'
});
/**
 * @private
 */
const editingButtonStyle = react.mergeStyles({
    margin: '0',
    width: '2.125rem',
    height: '2.125rem',
    padding: '0.375rem 0 0 0'
});
/**
 * @private
 */
const inputBoxIcon = react.mergeStyles({
    margin: 'auto',
    '&:hover svg': {
        stroke: 'currentColor'
    }
});
/**
 * @private
 */
react.mergeStyles(editorTextBoxButtonStyle, {
    '&:hover svg': {
        stroke: 'currentColor'
    }
});
/**
 * @private
 */
const editBoxWidthStyles = {
    minWidth: '6.25rem',
    maxWidth: '100%'
};
/**
 * @private
 */
const editBoxStyleSet = {
    root: editBoxWidthStyles
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const onRenderCancelIcon = (className) => {
    return React.createElement(react.Icon, { iconName: 'EditBoxCancel', className: className });
};
/**
 * @private
 */
const onRenderSubmitIcon = (className) => {
    return React.createElement(react.Icon, { iconName: 'EditBoxSubmit', className: className });
};
function isMessageEmpty(messageText) {
    return messageText.trim().length === 0;
}
/**
 * @private
 */
function getMessageState(messageText) {
    return isMessageEmpty(messageText) ? 'too short' : isMessageTooLong(messageText.length) ? 'too long' : 'OK';
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ChatMessageComponentAsEditBox = (props) => {
    const { onCancel, onSubmit, strings, message } = props;
    const [textValue, setTextValue] = React.useState(message.content || '');
    const editTextFieldRef = React.useRef(null);
    const theme = useTheme();
    const messageState = getMessageState(textValue);
    const submitEnabled = messageState === 'OK';
    const editContainerStyles = useChatMessageEditContainerStyles();
    const chatMyMessageStyles = useChatMyMessageStyles();
    React.useEffect(() => {
        var _a;
        (_a = editTextFieldRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, []);
    const setText = (event, newValue) => {
        setTextValue(newValue !== null && newValue !== void 0 ? newValue : '');
    };
    const textTooLongMessage = messageState === 'too long' ? _formatString(strings.editBoxTextLimit, {
        limitNumber: `${MAXIMUM_LENGTH_OF_MESSAGE}`
    }) : undefined;
    const iconClassName = React.useCallback((isHover) => {
        const color = isHover ? theme.palette.accent : theme.palette.neutralSecondary;
        return react.mergeStyles(inputBoxIcon, {
            color
        });
    }, [theme.palette.accent, theme.palette.neutralSecondary]);
    const onRenderThemedCancelIcon = React.useCallback((isHover) => {
        return onRenderCancelIcon(iconClassName(isHover));
    }, [iconClassName]);
    const onRenderThemedSubmitIcon = React.useCallback((isHover) => {
        return onRenderSubmitIcon(iconClassName(isHover));
    }, [iconClassName]);
    const editBoxStyles = React.useMemo(() => {
        return react.concatStyleSets(editBoxStyleSet, {
            textField: {
                borderColor: theme.palette.themePrimary
            }
        });
    }, [theme.palette.themePrimary]);
    const getContent = () => {
        return React.createElement(React.Fragment, null,
            React.createElement(InputBoxComponent, { "data-ui-id": "edit-box", textFieldRef: editTextFieldRef, inputClassName: editBoxStyle, placeholderText: strings.editBoxPlaceholderText, textValue: textValue, onChange: setText, onKeyDown: ev => {
                    if (ev.key === 'ArrowUp' || ev.key === 'ArrowDown') {
                        ev.stopPropagation();
                    }
                }, onEnterKeyDown: () => {
                    submitEnabled && onSubmit(textValue);
                }, supportNewline: false, maxLength: MAXIMUM_LENGTH_OF_MESSAGE, errorMessage: textTooLongMessage, styles: editBoxStyles }),
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "end", className: editChatMessageButtonsStackStyle, tokens: {
                    childrenGap: '0.25rem'
                } },
                message.failureReason && React.createElement(react.Stack.Item, { grow: true, align: "stretch", className: chatMessageFailedTagStackItemStyle },
                    React.createElement("div", { className: react.mergeStyles(chatMessageFailedTagStyle(theme), editChatMessageFailedTagStyle) }, message.failureReason)),
                React.createElement(react.Stack.Item, { align: "end" },
                    React.createElement(InputBoxButton, { className: editingButtonStyle, ariaLabel: strings.editBoxCancelButton, tooltipContent: strings.editBoxCancelButton, onRenderIcon: onRenderThemedCancelIcon, onClick: () => {
                            onCancel && onCancel(message.messageId);
                        }, id: 'dismissIconWrapper', "data-testId": "chat-message-edit-box-cancel-button" })),
                React.createElement(react.Stack.Item, { align: "end" },
                    React.createElement(InputBoxButton, { className: editingButtonStyle, ariaLabel: strings.editBoxSubmitButton, tooltipContent: strings.editBoxSubmitButton, onRenderIcon: onRenderThemedSubmitIcon, onClick: e => {
                            // it's very important to pass an empty attachment here
                            // so when user remvoes all attachments, UI can reflect it instantly
                            // if you set it to undefined, the attachments pre-edited would still be there
                            // until edit message event is received
                            submitEnabled && onSubmit(textValue);
                            e.stopPropagation();
                        }, id: 'submitIconWrapper', "data-testId": "chat-message-edit-box-submit-button" }))));
    };
    const attached = message.attached === true ? 'center' : message.attached === 'bottom' ? 'bottom' : 'top';
    return React.createElement(reactChat.ChatMyMessage, { attached: attached, root: {
            className: reactComponents.mergeClasses(chatMyMessageStyles.root)
        }, body: {
            className: reactComponents.mergeClasses(editContainerStyles.body, message.failureReason !== undefined ? editContainerStyles.bodyError : editContainerStyles.bodyDefault, attached !== 'top' ? editContainerStyles.bodyAttached : undefined)
        } }, getContent());
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ChatMessageComponentAsEditBoxPicker = (props) => {
    const simpleEditBox = React.useMemo(() => {
        return React.createElement(ChatMessageComponentAsEditBox, Object.assign({}, props));
    }, [props]);
    return simpleEditBox;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$K = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const ChatMyMessageComponent = (props) => {
    const { onDeleteMessage, onSendMessage, message, onCancelEditMessage, onUpdateMessage } = props;
    const [isEditing, setIsEditing] = React.useState(false);
    const [focusMessageAfterEditing, setFocusMessageAfterEditing] = React.useState(false);
    const onEditClick = React.useCallback(() => {
        setIsEditing(true);
        setFocusMessageAfterEditing(false);
    }, []);
    const clientMessageId = 'clientMessageId' in message ? message.clientMessageId : undefined;
    const content = 'content' in message ? message.content : undefined;
    const onRemoveClick = React.useCallback(() => {
        if (onDeleteMessage && message.messageId) {
            onDeleteMessage(message.messageId);
        }
        // when fail to send, message does not have message id, delete message using clientMessageId
        else if (onDeleteMessage && message.messageType === 'chat' && clientMessageId) {
            onDeleteMessage(clientMessageId);
        }
    }, [onDeleteMessage, message.messageId, message.messageType, clientMessageId]);
    const onResendClick = React.useCallback(() => {
        onDeleteMessage && clientMessageId && onDeleteMessage(clientMessageId);
        let newContent = content !== null && content !== void 0 ? content : '';
        onSendMessage && onSendMessage(newContent);
    }, [message, onDeleteMessage, clientMessageId, onSendMessage, content]);
    const onSubmitHandler = React.useCallback(
    // due to a bug in babel, we can't use arrow function here
    // affecting conditional-compile-remove(attachment-upload)
    function (text) {
        return __awaiter$K(this, void 0, void 0, function* () {
            yield (onUpdateMessage === null || onUpdateMessage === void 0 ? void 0 : onUpdateMessage(message.messageId, text));
            setIsEditing(false);
            setFocusMessageAfterEditing(true);
        });
    }, [message, onUpdateMessage]);
    const onCancelHandler = React.useCallback((messageId) => {
        onCancelEditMessage === null || onCancelEditMessage === void 0 ? void 0 : onCancelEditMessage(messageId);
        setIsEditing(false);
        setFocusMessageAfterEditing(true);
    }, [onCancelEditMessage]);
    if (isEditing && message.messageType === 'chat') {
        return React.createElement(ChatMessageComponentAsEditBoxPicker, { message: message, strings: props.strings, onSubmit: onSubmitHandler, onCancel: onCancelHandler });
    }
    else {
        return React.createElement(ChatMyMessageComponentAsMessageBubble, Object.assign({}, props, { onRemoveClick: onRemoveClick, onEditClick: onEditClick, onResendClick: onResendClick, onRenderAvatar: props.onRenderAvatar, strings: props.strings, inlineImageOptions: props.inlineImageOptions, shouldFocusFluentMessageBody: focusMessageAfterEditing }));
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * The component for rendering a chat message using Fluent UI components
 * and handling default and custom renderers.
 * This component handles rendering for chat message body, avatar and message status.
 * The chat message body, avatar and message status should be shown for both default and custom renderers.
 *
 * @private
 */
const FluentChatMyMessageComponent = (props) => {
    const { message, styles, shouldOverlapAvatarAndMessage, onRenderMessage, onRenderAvatar, showMessageStatus, onRenderMessageStatus, participantCount, readCount, onActionButtonClick, inlineImageOptions, userId, defaultStatusRenderer, statusToRender, actionsForAttachment, onRenderAttachmentDownloads } = props;
    const chatMessageRenderStyles = useChatMessageRenderStyles();
    // To rerender the defaultChatMessageRenderer if app running across days(every new day chat time stamp
    // needs to be regenerated), the dependency on "new Date().toDateString()"" is added.
    const defaultChatMessageRenderer = React.useCallback((messageProps) => {
        if (messageProps.message.messageType === 'chat') {
            return React.createElement(ChatMyMessageComponent, Object.assign({}, messageProps, { onRenderAttachmentDownloads: onRenderAttachmentDownloads, strings: messageProps.strings, message: messageProps.message, userId: userId, remoteParticipantsCount: participantCount ? participantCount - 1 : 0, shouldOverlapAvatarAndMessage: shouldOverlapAvatarAndMessage, onRenderAvatar: onRenderAvatar, showMessageStatus: showMessageStatus, messageStatus: messageProps.message.status, onActionButtonClick: onActionButtonClick, inlineImageOptions: inlineImageOptions, actionsForAttachment: actionsForAttachment }));
        }
        return React.createElement(React.Fragment, null);
    }, [onActionButtonClick, onRenderAvatar, participantCount, shouldOverlapAvatarAndMessage, showMessageStatus, userId, inlineImageOptions, onRenderAttachmentDownloads, actionsForAttachment,
        // eslint-disable-next-line react-hooks/exhaustive-deps
        new Date().toDateString()]);
    const messageRenderer = React.useCallback((messageProps) => {
        return onRenderMessage === undefined ? defaultChatMessageRenderer(Object.assign({}, messageProps)) : onRenderMessage(messageProps, defaultChatMessageRenderer);
    }, [defaultChatMessageRenderer, onRenderMessage]);
    const messageStatusRenderer = React.useCallback((onRenderMessageStatus, defaultStatusRenderer, showMessageStatus, participantCount, readCount) => {
        return showMessageStatus && statusToRender ? onRenderMessageStatus ? onRenderMessageStatus({
            status: message.status
        }) : defaultStatusRenderer(message, participantCount !== null && participantCount !== void 0 ? participantCount : 0, readCount !== null && readCount !== void 0 ? readCount : 0, message.status) : React.createElement("div", { className: react.mergeStyles(noMessageStatusStyle) });
    }, [message, statusToRender]);
    const attached = React.useMemo(() => {
        return getFluentUIAttachedValue(message.attached);
    }, [message.attached]);
    const myMessageRootProps = React.useMemo(() => {
        return {
            // myChatItemMessageContainer used in className and style prop as style prop can't handle CSS selectors
            className: reactComponents.mergeClasses(chatMessageRenderStyles.rootMyMessage, chatMessageRenderStyles.rootCommon, react.mergeStyles(styles === null || styles === void 0 ? void 0 : styles.myChatItemMessageContainer)),
            style: (styles === null || styles === void 0 ? void 0 : styles.myChatItemMessageContainer) !== undefined ? createStyleFromV8Style(styles === null || styles === void 0 ? void 0 : styles.myChatItemMessageContainer) : {},
            role: 'none'
        };
    }, [chatMessageRenderStyles.rootCommon, chatMessageRenderStyles.rootMyMessage, styles === null || styles === void 0 ? void 0 : styles.myChatItemMessageContainer]);
    const setMessageContainerRef = React.useCallback((node) => {
        removeFluentUIKeyboardNavigationStyles(node);
    }, []);
    const myMessageBodyProps = React.useMemo(() => {
        return {
            className: reactComponents.mergeClasses(chatMessageRenderStyles.bodyCommon, chatMessageRenderStyles.bodyMyMessage),
            ref: setMessageContainerRef
        };
    }, [chatMessageRenderStyles.bodyCommon, chatMessageRenderStyles.bodyMyMessage, setMessageContainerRef]);
    const myMessageStatusIcon = React.useMemo(() => {
        var _a;
        return React.createElement("div", { className: react.mergeStyles({
                paddingLeft: '0.25rem'
            }, (styles === null || styles === void 0 ? void 0 : styles.messageStatusContainer) ? styles.messageStatusContainer((_a = message.mine) !== null && _a !== void 0 ? _a : false) : '') }, message.status ? messageStatusRenderer(onRenderMessageStatus, defaultStatusRenderer, showMessageStatus, participantCount, readCount) : undefined);
    }, [defaultStatusRenderer, message.mine, message.status, messageStatusRenderer, onRenderMessageStatus, participantCount, readCount, showMessageStatus, styles]);
    // Fluent UI message components are used here as for default message renderer,
    // timestamp and author name should be shown but they aren't shown for custom renderer.
    // More investigations are needed to check if this can be simplified with states.
    // Status and avatar should be shown for both custom and default renderers.
    return React.createElement(reactChat.ChatMyMessage, { attached: attached, root: myMessageRootProps, body: myMessageBodyProps, statusIcon: myMessageStatusIcon }, messageRenderer(Object.assign({}, props)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const MessageBubble = (props) => {
    const ids = useIdentifiers();
    const theme = useTheme();
    const locale = useLocale$1();
    const { userId, message, showDate, messageContainerStyle, strings, onRenderAttachmentDownloads, inlineImageOptions, shouldOverlapAvatarAndMessage, actionsForAttachment, onDisplayDateTimeString } = props;
    const formattedTimestamp = React.useMemo(() => {
        const defaultTimeStamp = message.createdOn ? generateDefaultTimestamp(message.createdOn, showDate, strings) : undefined;
        const customTimestamp = message.createdOn ? generateCustomizedTimestamp(message.createdOn) : '';
        return customTimestamp || defaultTimeStamp;
    }, [locale, message.createdOn, onDisplayDateTimeString, showDate, strings]);
    const getMessageDetails = React.useCallback(() => {
        return getMessageEditedDetails(message, theme, strings.editedTag);
    }, [strings.editedTag, theme, message]);
    const getContent = React.useCallback(() => {
        return getMessageBubbleContent(message, strings, userId, inlineImageOptions, onRenderAttachmentDownloads, actionsForAttachment);
    }, [actionsForAttachment, inlineImageOptions, message, onRenderAttachmentDownloads, strings, userId]);
    const chatMessageCommonStyles = useChatMessageCommonStyles();
    const chatMessageStyles = useChatMessageStyles();
    const chatItemMessageContainerClassName = reactComponents.mergeClasses(chatMessageCommonStyles.body, chatMessageStyles.body, 
    // disable placeholder functionality for GA releases as it might confuse users
    chatMessageCommonStyles.bodyWithPlaceholderImage, props.message.status === 'failed' ? chatMessageCommonStyles.failed : undefined, shouldOverlapAvatarAndMessage ? chatMessageStyles.avatarOverlap : chatMessageStyles.avatarNoOverlap, message.attached === 'top' || message.attached === false ? chatMessageStyles.bodyWithAvatar : chatMessageStyles.bodyWithoutAvatar, 
    // messageContainerStyle used in className and style prop as style prop can't handle CSS selectors
    react.mergeStyles(messageContainerStyle));
    const attached = message.attached === true ? 'center' : message.attached === 'bottom' ? 'bottom' : 'top';
    const chatMessage = React.createElement(React.Fragment, null,
        React.createElement("div", { key: props.message.messageId },
            React.createElement(reactChat.ChatMessage, { attached: attached, key: props.message.messageId, root: {
                    className: chatMessageStyles.root
                }, author: React.createElement(react.Text, { className: chatMessageAuthorStyle }, message.senderDisplayName), body: {
                    className: chatItemMessageContainerClassName,
                    style: Object.assign({}, createStyleFromV8Style(messageContainerStyle))
                }, "data-testid": "chat-composite-message", timestamp: React.createElement(react.Text, { className: props.message.status === 'failed' ? chatMessageDateFailedStyle(theme) : chatMessageDateStyle(theme), "data-testid": ids.messageTimestamp }, formattedTimestamp), details: getMessageDetails() }, getContent())));
    return chatMessage;
};
/** @private */
const ChatMessageComponentAsMessageBubble = React.memo(MessageBubble);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * The component for rendering a chat message using Fluent UI components
 * and handling default and custom renderers.
 * This component handles rendering for chat message body, avatar and message status.
 * The chat message body, avatar and message status should be shown for both default and custom renderers.
 *
 * @private
 */
const FluentChatMessageComponent = (props) => {
    const { message, styles, shouldOverlapAvatarAndMessage, onRenderMessage, onRenderAvatar, inlineImageOptions, actionsForAttachment, userId, onRenderAttachmentDownloads } = props;
    const chatMessageRenderStyles = useChatMessageRenderStyles();
    // To rerender the defaultChatMessageRenderer if app running across days(every new day chat time stamp
    // needs to be regenerated), the dependency on "new Date().toDateString()"" is added.
    const defaultChatMessageRenderer = React.useCallback((messageProps) => {
        if (messageProps.message.messageType === 'chat') {
            return React.createElement(ChatMessageComponentAsMessageBubble, Object.assign({}, messageProps, { onRenderAttachmentDownloads: onRenderAttachmentDownloads, strings: messageProps.strings, message: messageProps.message, userId: userId, shouldOverlapAvatarAndMessage: shouldOverlapAvatarAndMessage, inlineImageOptions: inlineImageOptions, actionsForAttachment: actionsForAttachment }));
        }
        return React.createElement(React.Fragment, null);
    }, [onRenderAttachmentDownloads, userId, shouldOverlapAvatarAndMessage, inlineImageOptions, actionsForAttachment]);
    const messageRenderer = React.useCallback((messageProps) => {
        return onRenderMessage === undefined ? defaultChatMessageRenderer(Object.assign({}, messageProps)) : onRenderMessage(messageProps, defaultChatMessageRenderer);
    }, [defaultChatMessageRenderer, onRenderMessage]);
    const shouldShowAvatar = React.useMemo(() => {
        return message.attached === 'top' || message.attached === false;
    }, [message.attached]);
    const attached = React.useMemo(() => {
        return getFluentUIAttachedValue(message.attached);
    }, [message.attached]);
    const messageRootProps = React.useMemo(() => {
        return {
            className: reactComponents.mergeClasses(chatMessageRenderStyles.rootMessage, chatMessageRenderStyles.rootCommon)
        };
    }, [chatMessageRenderStyles.rootCommon, chatMessageRenderStyles.rootMessage]);
    const avatar = React.useMemo(() => {
        const chatAvatarStyle = shouldShowAvatar ? gutterWithAvatar : gutterWithHiddenAvatar;
        const personaOptions = {
            hidePersonaDetails: true,
            size: react.PersonaSize.size32,
            text: message.senderDisplayName,
            showOverflowTooltip: false
        };
        let renderedAvatar;
        if (onRenderAvatar) {
            const avatarComponent = onRenderAvatar === null || onRenderAvatar === void 0 ? void 0 : onRenderAvatar(message.senderId, personaOptions);
            if (!avatarComponent) {
                return undefined;
            }
            else {
                renderedAvatar = avatarComponent;
            }
        }
        return React.createElement("div", { className: react.mergeStyles(chatAvatarStyle) }, renderedAvatar ? renderedAvatar : React.createElement(react.Persona, Object.assign({}, personaOptions)));
    }, [message.senderDisplayName, message.senderId, onRenderAvatar, shouldShowAvatar]);
    const setMessageContainerRef = React.useCallback((node) => {
        removeFluentUIKeyboardNavigationStyles(node);
    }, []);
    const messageBodyProps = React.useMemo(() => {
        return {
            ref: setMessageContainerRef,
            // chatItemMessageContainer used in className and style prop as style prop can't handle CSS selectors
            className: reactComponents.mergeClasses(chatMessageRenderStyles.bodyCommon, !shouldShowAvatar ? avatar ? chatMessageRenderStyles.bodyWithoutAvatar : chatMessageRenderStyles.bodyHiddenAvatar : chatMessageRenderStyles.bodyWithAvatar, shouldOverlapAvatarAndMessage ? chatMessageRenderStyles.avatarOverlap : chatMessageRenderStyles.avatarNoOverlap, react.mergeStyles(styles === null || styles === void 0 ? void 0 : styles.chatItemMessageContainer)),
            style: (styles === null || styles === void 0 ? void 0 : styles.chatItemMessageContainer) !== undefined ? createStyleFromV8Style(styles === null || styles === void 0 ? void 0 : styles.chatItemMessageContainer) : {}
        };
    }, [setMessageContainerRef, chatMessageRenderStyles.bodyCommon, chatMessageRenderStyles.bodyWithoutAvatar, chatMessageRenderStyles.bodyHiddenAvatar, chatMessageRenderStyles.bodyWithAvatar, chatMessageRenderStyles.avatarOverlap, chatMessageRenderStyles.avatarNoOverlap, shouldShowAvatar, avatar, shouldOverlapAvatarAndMessage, styles === null || styles === void 0 ? void 0 : styles.chatItemMessageContainer]);
    // Fluent UI message components are used here as for default message renderer,
    // timestamp and author name should be shown but they aren't shown for custom renderer.
    // More investigations are needed to check if this can be simplified with states.
    // Avatar should be shown for both custom and default renderers.
    return React.createElement(reactChat.ChatMessage, { attached: attached, root: messageRootProps, body: messageBodyProps, avatar: avatar }, messageRenderer(Object.assign({}, props)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * The wrapper component to display different types of chat message.
 *
 * @private
 */
const ChatMessageComponentWrapper = (props) => {
    var _a;
    const { message, styles, onRenderMessage, key: messageKey } = props;
    const systemMessageStyle = React.useMemo(() => {
        return {
            paddingTop: '0.5rem'
        };
    }, []);
    const customMessageStyle = React.useMemo(() => {
        return {
            paddingTop: '1rem',
            paddingBottom: '0.25rem'
        };
    }, []);
    switch (message.messageType) {
        case 'chat':
            {
                const myChatMessageStyle = message.status === 'failed' ? (_a = styles === null || styles === void 0 ? void 0 : styles.failedMyChatMessageContainer) !== null && _a !== void 0 ? _a : styles === null || styles === void 0 ? void 0 : styles.myChatMessageContainer : styles === null || styles === void 0 ? void 0 : styles.myChatMessageContainer;
                const chatMessageStyle = styles === null || styles === void 0 ? void 0 : styles.chatMessageContainer;
                const messageContainerStyle = message.mine ? myChatMessageStyle : chatMessageStyle;
                return fluentChatComponent(Object.assign(Object.assign({}, props), { message: message, messageContainerStyle: messageContainerStyle }));
            }
        case 'system':
            {
                const messageContainerStyle = styles === null || styles === void 0 ? void 0 : styles.systemMessageContainer;
                const systemMessageComponent = onRenderMessage === undefined ? React.createElement(DefaultSystemMessage, Object.assign({}, props)) : onRenderMessage(Object.assign(Object.assign({}, props), { messageContainerStyle }), props => React.createElement(DefaultSystemMessage, Object.assign({}, props)));
                return React.createElement("div", { key: messageKey, style: systemMessageStyle }, systemMessageComponent);
            }
        default:
            {
                // We do not handle custom type message by default, users can handle custom type by using onRenderMessage function.
                const customMessageComponent = onRenderMessage === undefined ? React.createElement(React.Fragment, null) : onRenderMessage(Object.assign({}, props));
                return React.createElement("div", { key: messageKey, style: customMessageStyle }, customMessageComponent);
            }
    }
};
const fluentChatComponent = (props) => {
    if (props.message.mine === true) {
        return React.createElement(FluentChatMyMessageComponent, Object.assign({}, props));
    }
    else {
        return React.createElement(FluentChatMessageComponent, Object.assign({}, props));
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$J = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const isMessageSame = (first, second) => {
    return first.messageId === second.messageId && first.content === second.content && first.contentType === second.contentType && JSON.stringify(first.createdOn) === JSON.stringify(second.createdOn) && first.senderId === second.senderId && first.senderDisplayName === second.senderDisplayName && JSON.stringify(first.editedOn) === JSON.stringify(second.editedOn);
};
/**
 * Get the latest message from the message array.
 *
 * @param messages
 */
const getLatestChatMessage = (messages) => {
    for (let i = messages.length - 1; i >= 0; i--) {
        const message = messages[i];
        if ((message === null || message === void 0 ? void 0 : message.messageType) === 'chat' && !!message.createdOn) {
            return message;
        }
    }
    return undefined;
};
/**
 * Compare latestMessageFromPreviousMessages & latestMessageFromNewMessages to see if the new message is not from
 * current user.
 */
const isThereNewMessageNotFromCurrentUser = (userId, latestMessageFromPreviousMessages, latestMessageFromNewMessages) => {
    if (latestMessageFromNewMessages === undefined) {
        return false;
    }
    if (latestMessageFromPreviousMessages === undefined) {
        return latestMessageFromNewMessages.senderId !== userId;
    }
    return !isMessageSame(latestMessageFromNewMessages, latestMessageFromPreviousMessages) && latestMessageFromNewMessages.senderId !== userId;
};
/**
 * Returns true if the current user sent the latest message and false otherwise. It will ignore messages that have no
 * sender, messages that have failed to send, and messages from the current user that is marked as SEEN. This is meant
 * as an indirect way to detect if user is at bottom of the chat when the component updates with new messages. If we
 * updated this component due to current user sending a message we want to then call scrollToBottom.
 */
const didUserSendTheLatestMessage = (userId, latestMessageFromPreviousMessages, latestMessageFromNewMessages) => {
    if (latestMessageFromNewMessages === undefined) {
        return false;
    }
    if (latestMessageFromPreviousMessages === undefined) {
        return latestMessageFromNewMessages.senderId === userId;
    }
    return !isMessageSame(latestMessageFromNewMessages, latestMessageFromPreviousMessages) && latestMessageFromNewMessages.senderId === userId;
};
const DefaultJumpToNewMessageButton = (props) => {
    const { text, onClick } = props;
    return React.createElement(react.PrimaryButton, { className: newMessageButtonStyle, styles: buttonWithIconStyles$1, text: text, onClick: onClick, onRenderIcon: () => React.createElement(react.Icon, { iconName: "Down", className: DownIconStyle }) });
};
const memoizeAllMessages = memoizeFnAll((message, showMessageDate, showMessageStatus, strings, index, onUpdateMessage, onCancelEditMessage, onDeleteMessage, onSendMessage, disableEditing, lastSeenChatMessage, lastSendingChatMessage, lastDeliveredChatMessage) => {
    let key = message.messageId;
    let statusToRender = undefined;
    if (message.messageType === 'chat') {
        if ((!message.messageId || message.messageId === '') && 'clientMessageId' in message) {
            key = message.clientMessageId;
        }
        if (showMessageStatus && message.mine) {
            switch (message.messageId) {
                case lastSeenChatMessage:
                    {
                        statusToRender = 'seen';
                        break;
                    }
                case lastSendingChatMessage:
                    {
                        statusToRender = 'sending';
                        break;
                    }
                case lastDeliveredChatMessage:
                    {
                        statusToRender = 'delivered';
                        break;
                    }
            }
        }
        if (message.mine && message.status === 'failed') {
            statusToRender = 'failed';
        }
    }
    return {
        key: key !== null && key !== void 0 ? key : 'id_' + index,
        statusToRender,
        message,
        strings,
        showDate: showMessageDate,
        onUpdateMessage,
        onCancelEditMessage,
        onDeleteMessage,
        onSendMessage,
        disableEditing,
        showMessageStatus
    };
});
const getLastChatMessageIdWithStatus = (messages, status) => {
    for (let i = messages.length - 1; i >= 0; i--) {
        const message = messages[i];
        if ((message === null || message === void 0 ? void 0 : message.messageType) === 'chat' && message.status === status && message.mine) {
            return message.messageId;
        }
    }
    return undefined;
};
const getLastChatMessageForCurrentUser = (messages) => {
    for (let i = messages.length - 1; i >= 0; i--) {
        const message = messages[i];
        if ((message === null || message === void 0 ? void 0 : message.messageType) === 'chat' && message.mine) {
            return message;
        }
    }
    return undefined;
};
/**
 * `MessageThread` allows you to easily create a component for rendering chat messages, handling scrolling behavior of new/old messages and customizing icons & controls inside the chat thread.
 * @param props - of type MessageThreadProps
 *
 * Users will need to provide at least chat messages and userId to render the `MessageThread` component.
 * Users can also customize `MessageThread` by passing in their own Avatar, `MessageStatusIndicator` icon, `JumpToNewMessageButton`, `LoadPreviousMessagesButton` and the behavior of these controls.
 *
 * `MessageThread` internally uses the `Chat` component from `@fluentui-contrib/chat`. You can checkout the details about these components [here](https://microsoft.github.io/fluentui-contrib/react-chat/).
 *
 * @public
 */
const MessageThread = (props) => {
    var _a;
    const theme = useTheme();
    const chatBody = React.useMemo(() => {
        return React.createElement(FluentV9ThemeProvider, { v8Theme: theme },
            React.createElement(MessageThreadWrapper, Object.assign({}, props)));
    }, [theme, props]);
    return React.createElement("div", { className: react.mergeStyles(messageThreadContainerStyle, (_a = props.styles) === null || _a === void 0 ? void 0 : _a.root) }, chatBody);
};
/**
 * @private
 */
const MessageThreadWrapper = (props) => {
    var _a;
    const { messages: newMessages, userId, participantCount, readReceiptsBySenderId, styles, disableJumpToNewMessageButton = false, showMessageDate = false, showMessageStatus = false, numberOfChatMessagesToReload = 5, onMessageSeen, onRenderMessageStatus, onRenderAvatar, onLoadPreviousChatMessages, onRenderJumpToNewMessageButton, onRenderMessage, onUpdateMessage, onCancelEditMessage, onDeleteMessage, onSendMessage, inlineImageOptions } = props;
    // We need this state to wait for one tick and scroll to bottom after messages have been initialized.
    // Otherwise chatScrollDivRef.current.clientHeight is wrong if we scroll to bottom before messages are initialized.
    const [chatMessagesInitialized, setChatMessagesInitialized] = React.useState(false);
    const [isAtBottomOfScroll, setIsAtBottomOfScroll] = React.useState(true);
    const [forceUpdate, setForceUpdate] = React.useState(0);
    // Used to decide if should auto scroll to bottom or show "new message" button
    const [latestPreviousChatMessage, setLatestPreviousChatMessage] = React.useState(undefined);
    const [latestCurrentChatMessage, setLatestCurrentChatMessage] = React.useState(undefined);
    const [existsNewChatMessage, setExistsNewChatMessage] = React.useState(false);
    const [lastSeenChatMessage, setLastSeenChatMessage] = React.useState(undefined);
    const [lastDeliveredChatMessage, setLastDeliveredChatMessage] = React.useState(undefined);
    const [lastSendingChatMessage, setLastSendingChatMessage] = React.useState(undefined);
    // readCount and participantCount will only need to be updated on-fly when user hover on an indicator
    const [readCountForHoveredIndicator, setReadCountForHoveredIndicator] = React.useState(undefined);
    const localeStrings = useLocale$1().strings.messageThread;
    const strings = React.useMemo(() => (Object.assign(Object.assign({}, localeStrings), props.strings)), [localeStrings, props.strings]);
    // it is required to use useState for messages
    // as the scrolling logic requires re - render at a specific point in time
    const [messages, setMessages] = React.useState([]);
    // id for the latest deleted message
    const [latestDeletedMessageId, setLatestDeletedMessageId] = React.useState(undefined);
    // this value is used to check if a message is deleted for the previous value of messages array
    const previousMessagesRef = React.useRef([]);
    // an aria label for Narrator to notify when a message is deleted
    const [deletedMessageAriaLabel, setDeletedMessageAriaLabel] = React.useState(undefined);
    const onDeleteMessageCallback = React.useCallback((messageId) => __awaiter$J(void 0, void 0, void 0, function* () {
        if (!onDeleteMessage) {
            return;
        }
        try {
            // reset deleted message label in case if there was a value already (messages are deleted 1 after another)
            setDeletedMessageAriaLabel(undefined);
            setLatestDeletedMessageId(messageId);
            lastChatMessageStatus.current = 'deleted';
            // we should set up latestDeletedMessageId before the onDeleteMessage call
            // as otherwise in very rare cases the messages array can be updated before latestDeletedMessageId
            yield onDeleteMessage(messageId);
        }
        catch (e) {
            console.log('onDeleteMessage failed: messageId', messageId, 'error', e);
        }
    }), [onDeleteMessage]);
    const isAllChatMessagesLoadedRef = React.useRef(false);
    // isAllChatMessagesLoadedRef needs to be updated every time when a new adapter is set in order to display correct data
    // onLoadPreviousChatMessages is updated when a new adapter is set
    React.useEffect(() => {
        if (onLoadPreviousChatMessages) {
            isAllChatMessagesLoadedRef.current = false;
        }
    }, [onLoadPreviousChatMessages]);
    const previousTopRef = React.useRef(-1);
    const previousHeightRef = React.useRef(-1);
    const messageIdSeenByMeRef = React.useRef('');
    const chatScrollDivRef = React.useRef(null);
    const isLoadingChatMessagesRef = React.useRef(false);
    React.useEffect(() => {
        if (latestDeletedMessageId === undefined) {
            setDeletedMessageAriaLabel(undefined);
        }
        else {
            if (!previousMessagesRef.current.find(message => message.messageId === latestDeletedMessageId)) {
                // the message is deleted in previousMessagesRef
                // no need to update deletedMessageAriaLabel
                setDeletedMessageAriaLabel(undefined);
            }
            else if (!messages.find(message => message.messageId === latestDeletedMessageId)) {
                // the message is deleted in messages array but still exists in previousMessagesRef
                // update deletedMessageAriaLabel
                setDeletedMessageAriaLabel(strings.messageDeletedAnnouncementAriaLabel);
            }
            else {
                // the message exists in both arrays
                // no need to update deletedMessageAriaLabel
                setDeletedMessageAriaLabel(undefined);
            }
        }
        previousMessagesRef.current = messages;
    }, [latestDeletedMessageId, messages, strings.messageDeletedAnnouncementAriaLabel]);
    const messagesRef = React.useRef(messages);
    const setMessagesRef = (messagesWithAttachedValue) => {
        messagesRef.current = messagesWithAttachedValue;
        setMessages(messagesWithAttachedValue);
    };
    const isAtBottomOfScrollRef = React.useRef(isAtBottomOfScroll);
    const setIsAtBottomOfScrollRef = (isAtBottomOfScrollValue) => {
        isAtBottomOfScrollRef.current = isAtBottomOfScrollValue;
        setIsAtBottomOfScroll(isAtBottomOfScrollValue);
    };
    const chatMessagesInitializedRef = React.useRef(chatMessagesInitialized);
    const setChatMessagesInitializedRef = (chatMessagesInitialized) => {
        chatMessagesInitializedRef.current = chatMessagesInitialized;
        setChatMessagesInitialized(chatMessagesInitialized);
    };
    const chatThreadRef = React.useRef(null);
    // When the chat thread is narrow, we perform space optimizations such as overlapping
    // the avatar on top of the chat message and moving the chat accept/reject edit buttons
    // to a new line
    const chatThreadWidth = _useContainerWidth(chatThreadRef);
    const isNarrow = chatThreadWidth ? isNarrowWidth(chatThreadWidth) : false;
    /**
     * ClientHeight controls the number of messages to render. However ClientHeight will not be initialized after the
     * first render (not sure but I guess Fluent is updating it in hook which is after render maybe?) so we need to
     * trigger a re-render until ClientHeight is initialized. This force re-render should only happen once.
     */
    const clientHeight = (_a = chatThreadRef.current) === null || _a === void 0 ? void 0 : _a.clientHeight;
    // we try to only send those message status if user is scrolled to the bottom.
    const sendMessageStatusIfAtBottom = React.useCallback(() => __awaiter$J(void 0, void 0, void 0, function* () {
        if (!isAtBottomOfScrollRef.current || !document.hasFocus() || !messagesRef.current || messagesRef.current.length === 0 || !showMessageStatus) {
            return;
        }
        const messagesWithId = messagesRef.current.filter(message => {
            return message.messageType === 'chat' && !message.mine && !!message.messageId;
        });
        if (messagesWithId.length === 0) {
            return;
        }
        const lastMessage = messagesWithId[messagesWithId.length - 1];
        try {
            if (onMessageSeen && lastMessage && lastMessage.messageId && lastMessage.messageId !== messageIdSeenByMeRef.current) {
                yield onMessageSeen(lastMessage.messageId);
                messageIdSeenByMeRef.current = lastMessage.messageId;
            }
        }
        catch (e) {
            console.log('onMessageSeen Error', lastMessage, e);
        }
    }), [showMessageStatus, onMessageSeen]);
    const scrollToBottom = React.useCallback(() => {
        if (chatScrollDivRef.current) {
            chatScrollDivRef.current.scrollTop = chatScrollDivRef.current.scrollHeight;
        }
        setExistsNewChatMessage(false);
        setIsAtBottomOfScrollRef(true);
        sendMessageStatusIfAtBottom();
    }, [sendMessageStatusIfAtBottom]);
    const handleScrollToTheBottom = React.useCallback(() => {
        if (!chatScrollDivRef.current) {
            return;
        }
        const atBottom = Math.ceil(chatScrollDivRef.current.scrollTop) >= chatScrollDivRef.current.scrollHeight - chatScrollDivRef.current.clientHeight;
        if (atBottom) {
            sendMessageStatusIfAtBottom();
            if (!isAtBottomOfScrollRef.current) {
                scrollToBottom();
            }
        }
        setIsAtBottomOfScrollRef(atBottom);
    }, [scrollToBottom, sendMessageStatusIfAtBottom]);
    // Infinite scrolling + threadInitialize function
    const fetchNewMessageWhenAtTop = React.useCallback(() => __awaiter$J(void 0, void 0, void 0, function* () {
        if (!isLoadingChatMessagesRef.current) {
            if (onLoadPreviousChatMessages) {
                isLoadingChatMessagesRef.current = true;
                try {
                    // Fetch message until scrollTop reach the threshold for fetching new message
                    while (!isAllChatMessagesLoadedRef.current && chatScrollDivRef.current && chatScrollDivRef.current.scrollTop <= 500) {
                        isAllChatMessagesLoadedRef.current = yield onLoadPreviousChatMessages(numberOfChatMessagesToReload);
                        yield delay(200);
                    }
                }
                finally {
                    // Set isLoadingChatMessagesRef to false after messages are fetched
                    isLoadingChatMessagesRef.current = false;
                }
            }
        }
    }), [numberOfChatMessagesToReload, onLoadPreviousChatMessages]);
    // The below 2 of useEffects are design for fixing infinite scrolling problem
    // Scrolling element will behave differently when scrollTop = 0(it sticks at the top)
    // we need to get previousTop before it prepend contents
    // Execute order [newMessage useEffect] => get previousTop => dom update => [messages useEffect]
    React.useEffect(() => {
        if (!chatScrollDivRef.current) {
            return;
        }
        previousTopRef.current = chatScrollDivRef.current.scrollTop;
        previousHeightRef.current = chatScrollDivRef.current.scrollHeight;
    }, [newMessages]);
    React.useEffect(() => {
        if (!chatScrollDivRef.current) {
            return;
        }
        chatScrollDivRef.current.scrollTop = chatScrollDivRef.current.scrollHeight - (previousHeightRef.current - previousTopRef.current);
    }, [messages]);
    // Fetch more messages to make the scroll bar appear, infinity scroll is then handled in the handleScroll function.
    React.useEffect(() => {
        fetchNewMessageWhenAtTop();
    }, [fetchNewMessageWhenAtTop]);
    /**
     * One time run useEffects. Sets up listeners when component is mounted and tears down listeners when component
     * unmounts unless these function changed
     */
    React.useEffect(() => {
        window && window.addEventListener('click', sendMessageStatusIfAtBottom);
        window && window.addEventListener('focus', sendMessageStatusIfAtBottom);
        return () => {
            window && window.removeEventListener('click', sendMessageStatusIfAtBottom);
            window && window.removeEventListener('focus', sendMessageStatusIfAtBottom);
        };
    }, [sendMessageStatusIfAtBottom]);
    React.useEffect(() => {
        const chatScrollDiv = chatScrollDivRef.current;
        chatScrollDiv === null || chatScrollDiv === void 0 ? void 0 : chatScrollDiv.addEventListener('scroll', handleScrollToTheBottom);
        chatScrollDiv === null || chatScrollDiv === void 0 ? void 0 : chatScrollDiv.addEventListener('scroll', fetchNewMessageWhenAtTop);
        return () => {
            chatScrollDiv === null || chatScrollDiv === void 0 ? void 0 : chatScrollDiv.removeEventListener('scroll', handleScrollToTheBottom);
            chatScrollDiv === null || chatScrollDiv === void 0 ? void 0 : chatScrollDiv.removeEventListener('scroll', fetchNewMessageWhenAtTop);
        };
    }, [fetchNewMessageWhenAtTop, handleScrollToTheBottom]);
    React.useEffect(() => {
        if (clientHeight === undefined) {
            setForceUpdate(forceUpdate + 1);
            return;
        }
        // Only scroll to bottom if isAtBottomOfScrollRef is true
        isAtBottomOfScrollRef.current && scrollToBottom();
    }, [clientHeight, forceUpdate, scrollToBottom, chatMessagesInitialized]);
    React.useEffect(() => {
        var _a;
        const newStatus = (_a = getLastChatMessageForCurrentUser(newMessages)) === null || _a === void 0 ? void 0 : _a.status;
        if (newStatus !== undefined) {
            if (lastChatMessageStatus.current === 'deleted' && newStatus === 'sending') {
                // enforce message life cycle
                // message status should always be [ sending -> delivered -> seen (optional) -> deleted ] or [sending -> failed -> deleted]
                // not any other way around
                // therefore, if current message status is deleted, we should only update it if newStatus is sending
                lastChatMessageStatus.current = newStatus;
            }
            else if (lastChatMessageStatus.current !== 'deleted') {
                lastChatMessageStatus.current = newStatus;
            }
        }
        // The hook should depend on newMessages not on messages as otherwise it will skip the sending status for a first message
    }, [newMessages]);
    /**
     * This needs to run to update latestPreviousChatMessage & latestCurrentChatMessage.
     * These two states are used to manipulate scrollbar
     */
    React.useEffect(() => {
        setLatestPreviousChatMessage(getLatestChatMessage(messagesRef.current));
        setLatestCurrentChatMessage(getLatestChatMessage(newMessages));
        setMessagesRef(newMessages);
        !chatMessagesInitializedRef.current && setChatMessagesInitializedRef(true);
        setLastDeliveredChatMessage(getLastChatMessageIdWithStatus(newMessages, 'delivered'));
        setLastSeenChatMessage(getLastChatMessageIdWithStatus(newMessages, 'seen'));
        setLastSendingChatMessage(getLastChatMessageIdWithStatus(newMessages, 'sending'));
    }, [newMessages]);
    /**
     * This needs to run after messages are rendered so we can manipulate the scroll bar.
     */
    React.useEffect(() => {
        // If user just sent the latest message then we assume we can move user to bottom of scroll.
        if (isThereNewMessageNotFromCurrentUser(userId, latestPreviousChatMessage, latestCurrentChatMessage) && !isAtBottomOfScrollRef.current) {
            setExistsNewChatMessage(true);
        }
        else if (didUserSendTheLatestMessage(userId, latestPreviousChatMessage, latestCurrentChatMessage) || isAtBottomOfScrollRef.current) {
            scrollToBottom();
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [messages]);
    const lastChatMessageStatus = React.useRef(undefined);
    const participantCountRef = React.useRef(participantCount);
    const readReceiptsBySenderIdRef = React.useRef(readReceiptsBySenderId);
    participantCountRef.current = participantCount;
    readReceiptsBySenderIdRef.current = readReceiptsBySenderId;
    const onActionButtonClickMemo = React.useCallback((message, setMessageReadBy) => {
        if (participantCountRef.current && participantCountRef.current - 1 > 1 && readReceiptsBySenderIdRef.current) {
            setMessageReadBy(getParticipantsWhoHaveReadMessage(message, readReceiptsBySenderIdRef.current));
        }
    }, []);
    const defaultStatusRenderer = React.useCallback((message, participantCount, readCount, status) => {
        // we should only announce label if the message status isn't deleted
        // because after message is deleted, we now need to render statusIndicator for previous messages
        // and their status has been announced already and we should not announce them again
        const shouldAnnounce = lastChatMessageStatus.current !== 'deleted';
        const onToggleToolTip = (isToggled) => {
            if (isToggled && readReceiptsBySenderIdRef.current) {
                setReadCountForHoveredIndicator(getParticipantsWhoHaveReadMessage(message, readReceiptsBySenderIdRef.current).length);
            }
            else {
                setReadCountForHoveredIndicator(undefined);
            }
        };
        return React.createElement(MessageStatusIndicatorInternal, { status: status, readCount: readCount, onToggleToolTip: onToggleToolTip, 
            // -1 because participant count does not include myself
            remoteParticipantsCount: participantCount ? participantCount - 1 : 0, shouldAnnounce: shouldAnnounce });
    }, []);
    const theme = useTheme();
    const messagesToDisplay = React.useMemo(() => {
        return memoizeAllMessages(memoizedMessageFn => {
            return messages.map((message, index) => {
                return memoizedMessageFn(message, showMessageDate, showMessageStatus, strings, index, onUpdateMessage, onCancelEditMessage, onDeleteMessageCallback, onSendMessage, props.disableEditing, lastDeliveredChatMessage, lastSeenChatMessage, lastSendingChatMessage);
            });
        });
    }, [lastDeliveredChatMessage, lastSeenChatMessage, lastSendingChatMessage, messages, onCancelEditMessage, onDeleteMessageCallback, onSendMessage, onUpdateMessage, props.disableEditing, showMessageDate, showMessageStatus, strings]);
    const classes = useChatStyles();
    const chatArrowNavigationAttributes = reactComponents.useArrowNavigationGroup({
        axis: 'vertical',
        memorizeCurrent: false
    });
    return React.createElement("div", { className: react.mergeStyles(messageThreadWrapperContainerStyle), ref: chatThreadRef },
        existsNewChatMessage && !disableJumpToNewMessageButton && React.createElement("div", { className: react.mergeStyles(newMessageButtonContainerStyle, styles === null || styles === void 0 ? void 0 : styles.newMessageButtonContainer) }, onRenderJumpToNewMessageButton ? onRenderJumpToNewMessageButton({
            text: strings.newMessagesIndicator,
            onClick: scrollToBottom
        }) : React.createElement(DefaultJumpToNewMessageButton, { text: strings.newMessagesIndicator, onClick: scrollToBottom })),
        React.createElement(LiveAnnouncer, null,
            React.createElement(FluentV9ThemeProvider, { v8Theme: theme },
                React.createElement(reactChat.Chat
                // styles?.chatContainer used in className and style prop as style prop can't handle CSS selectors
                , Object.assign({ 
                    // styles?.chatContainer used in className and style prop as style prop can't handle CSS selectors
                    className: reactComponents.mergeClasses(classes.root, react.mergeStyles(styles === null || styles === void 0 ? void 0 : styles.chatContainer)), ref: chatScrollDivRef, style: Object.assign({}, createStyleFromV8Style(styles === null || styles === void 0 ? void 0 : styles.chatContainer)) }, chatArrowNavigationAttributes),
                    latestDeletedMessageId && React.createElement(Announcer$1, { key: latestDeletedMessageId, announcementString: deletedMessageAriaLabel, ariaLive: 'polite' }),
                    messagesToDisplay.map((message) => {
                        return React.createElement(MemoChatMessageComponentWrapper, Object.assign({}, message, { userId: userId, key: message.key, styles: styles, shouldOverlapAvatarAndMessage: isNarrow, strings: strings, onRenderAvatar: onRenderAvatar, onRenderMessage: onRenderMessage, onRenderMessageStatus: onRenderMessageStatus, defaultStatusRenderer: defaultStatusRenderer, onActionButtonClick: onActionButtonClickMemo, readCount: readCountForHoveredIndicator, participantCount: participantCount, inlineImageOptions: inlineImageOptions }));
                    })))));
};
const MemoChatMessageComponentWrapper = React.memo((obj) => {
    return React.createElement(ChatMessageComponentWrapper, Object.assign({}, obj));
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const container = () => react.mergeStyles({
    position: 'relative',
    // ensures child element's `position: absolute` is relative to this container
    display: 'contents'
});
/**
 * @private
 */
const loadingSpinnerContainer = () => react.mergeStyles({
    // Position centrally on top of content. Parent must have position: relative.
    position: 'absolute',
    top: '50%',
    bottom: '0',
    left: '50%',
    right: '0',
    transform: 'translate(-50%, -50%)'
});
/**
 * @private
 */
const loadSpinnerStyles$1 = {
    root: {
        height: '100%' // ensure height fills container
    },
    circle: {
        maxHeight: '5rem',
        height: '50%',
        width: 'unset',
        // remove default width applied by fluent for spinners
        aspectRatio: '1 / 1',
        // make height match width to ensure a circle shape
        borderWidth: '0.25em'
    }
};
/**
 * @private
 */
const mediaContainer = (theme) => react.mergeStyles({
    position: 'relative',
    height: '100%',
    width: '100%',
    background: 'transparent',
    display: 'flex',
    '& video': {
        borderRadius: theme.effects.roundedCorner4
    }
});
/**
 * @private
 */
const invertedVideoInPipStyle = (theme) => react.mergeStyles(mediaContainer(theme), {
    // The HTMLElement returned by the headless SDK is already inverted.
    // But in picture-in-picture mode, we do not want to invert the host HTMLElement.
    // Instead, we need to target the :picture-in-picture pseudoclass.
    //
    // First reset the host HTMLElement.
    transform: 'rotateY(180deg)',
    // This doesn't work yet on latest Edge.
    // Probably just not implemented yet.
    // picture-in-picture API is not yet stable: https://www.w3.org/TR/picture-in-picture/#css-pseudo-class
    '& video:picture-in-picture': {
        transform: 'rotateY(180deg)'
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Utility component to convert an HTMLElement with a video stream into a JSX element.
 *
 * Use to convert an HTMLElement returned by headless calling API into a component that can be rendered as a {@link VideoTile}.
 *
 * @public
 */
const StreamMedia = (props) => {
    const containerEl = React.useRef(null);
    const theme = useTheme();
    const { isMirrored, videoStreamElement, styles, loadingState = 'none' } = props;
    const [pipEnabled, setPipEnabled] = React.useState(false);
    React.useEffect(() => {
        const container = containerEl.current;
        if (!container) {
            return;
        }
        // If videoStreamElement changes, we clear the container to make sure we don't have duplicate, and replace it with
        // the new videoStreamElement. If videoStreamElement is undefined nothing is appended and container should be empty
        // and we don't render anyting.
        container.innerHTML = '';
        setPipEnabled(false);
        if (videoStreamElement) {
            videoStreamElement.addEventListener('enterpictureinpicture', () => {
                setPipEnabled(true);
            });
            videoStreamElement.addEventListener('leavepictureinpicture', () => {
                setPipEnabled(false);
            });
            container.appendChild(videoStreamElement);
        }
        return () => {
            container.innerHTML = '';
            setPipEnabled(false);
        };
    }, [videoStreamElement]);
    return React.createElement("div", { className: container() },
        React.createElement("div", { "data-ui-id": "stream-media-container", className: react.mergeStyles(isMirrored && pipEnabled ? invertedVideoInPipStyle(theme) : mediaContainer(theme), styles === null || styles === void 0 ? void 0 : styles.root), ref: containerEl }),
        loadingState === 'loading' && React.createElement("div", { className: loadingSpinnerContainer() },
            React.createElement(react.Spinner, { "data-ui-id": "stream-media-loading-spinner", styles: loadSpinnerStyles$1 })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const menuIconClass = 'ms-acs-participant-item-menu-icon';
/**
 * @private
 */
const participantItemContainerStyle = (options, theme) => {
    return {
        paddingTop: '0.25rem',
        paddingBottom: '0.25rem',
        display: 'flex',
        maxWidth: '100%',
        minWidth: '8rem',
        cursor: !options.clickable ? 'default' : 'pointer',
        alignItems: 'center',
        ':focus-visible': {
            [` .${menuIconClass}`]: {
                display: 'flex'
            },
            outline: `0.1rem solid ${theme.palette.themePrimary}`
        }
    };
};
/**
 * @private
 */
const displayNoneStyle = {
    display: 'none'
};
/**
 * @private
 */
const menuButtonContainerStyle = {
    width: 'auto'
};
/**
 * @private
 */
const participantStateMaxWidth = '5rem';
/**
 * @private
 */
const participantStateStringStyles$1 = {
    maxWidth: participantStateMaxWidth,
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    whiteSpace: 'nowrap',
    lineHeight: 'normal',
    marginLeft: '0.5rem',
    marginRight: 0
};
/**
 * @private
 */
const iconContainerStyle$1 = {
    display: 'flex',
    alignItems: 'center',
    height: '100%',
    paddingTop: '0.2rem'
};
/**
 * @private
 */
const iconStyles$3 = react.mergeStyles([menuIconClass, {
        display: 'flex',
        lineHeight: 0,
        // ensure the icon center is on the center line and not slightly above it
        alignItems: 'center'
    }]);
/**
 * @private
 */
const meContainerStyle = {
    paddingRight: '0.5rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component to render a calling or chat participant.
 *
 * Displays the participant's avatar, displayName and status as well as optional icons and context menu.
 *
 * @public
 */
const ParticipantItem = (props) => {
    var _a, _b, _c;
    const { userId, displayName, onRenderAvatar, menuItems, onRenderIcon, presence, styles, me, onClick, showParticipantOverflowTooltip } = props;
    const [itemHovered, setItemHovered] = React.useState(false);
    const [menuHidden, setMenuHidden] = React.useState(true);
    const containerRef = React.useRef(null);
    const theme = useTheme();
    const localeStrings = useLocale$1().strings.participantItem;
    const ids = useIdentifiers();
    const participantItemId = reactHooks.useId();
    const participantItemFlyoutId = reactHooks.useId();
    const hasFlyout = !!(menuItems && (menuItems === null || menuItems === void 0 ? void 0 : menuItems.length) > 0);
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const participantStateString = formatParticipantStateString(props, strings);
    const showMenuIcon = !participantStateString && (itemHovered || !menuHidden) && hasFlyout;
    // For 'me' show empty name so avatar will get 'Person' icon, when there is no name
    const meAvatarText = (displayName === null || displayName === void 0 ? void 0 : displayName.trim()) || '';
    const avatarOptions = {
        text: me ? meAvatarText : (displayName === null || displayName === void 0 ? void 0 : displayName.trim()) || strings.displayNamePlaceholder,
        size: react.PersonaSize.size32,
        presence: presence,
        initialsTextColor: 'white',
        showOverflowTooltip: showParticipantOverflowTooltip,
        showUnknownPersonaCoin: !me && (!(displayName === null || displayName === void 0 ? void 0 : displayName.trim()) || displayName === strings.displayNamePlaceholder)
    };
    const avatar = onRenderAvatar ? onRenderAvatar(userId !== null && userId !== void 0 ? userId : '', avatarOptions) : React.createElement(react.Persona, Object.assign({ className: react.mergeStyles({
            // Prevents persona text from being vertically truncated if a global line height is less than 1.15.
            lineHeight: '1.15rem'
        }, styles === null || styles === void 0 ? void 0 : styles.avatar) }, avatarOptions));
    const meTextStyle = React.useMemo(() => react.mergeStyles(meContainerStyle, {
        color: theme.palette.neutralSecondary
    }, styles === null || styles === void 0 ? void 0 : styles.me), [theme.palette.neutralSecondary, styles === null || styles === void 0 ? void 0 : styles.me]);
    const contextualMenuStyle = React.useMemo(() => react.mergeStyles({
        background: theme.palette.neutralLighterAlt
    }, styles === null || styles === void 0 ? void 0 : styles.menu), [theme.palette.neutralLighterAlt, styles === null || styles === void 0 ? void 0 : styles.menu]);
    const infoContainerStyle = React.useMemo(() => react.mergeStyles(iconContainerStyle$1, {
        color: theme.palette.neutralSecondary,
        marginLeft: 'auto'
    }, styles === null || styles === void 0 ? void 0 : styles.iconContainer), [theme.palette.neutralSecondary, styles === null || styles === void 0 ? void 0 : styles.iconContainer]);
    const onDismissMenu = () => {
        setItemHovered(false);
        setMenuHidden(true);
    };
    const menuButton = React.useMemo(() => React.createElement(react.Stack, { horizontal: true, horizontalAlign: "end", className: react.mergeStyles(menuButtonContainerStyle, {
            color: theme.palette.neutralPrimary
        }), title: strings.menuTitle, "aria-controls": participantItemFlyoutId, "data-ui-id": ids.participantItemMenuButton },
        React.createElement(react.Icon, { iconName: "ParticipantItemOptionsHovered", className: react.mergeStyles(iconStyles$3, !showMenuIcon ? displayNoneStyle : {}) })), [theme.palette.neutralPrimary, strings.menuTitle, participantItemFlyoutId, ids.participantItemMenuButton, showMenuIcon]);
    return React.createElement("div", { ref: containerRef, role: 'menuitem', id: participantItemId, "aria-label": (_b = (hasFlyout ? (_a = props.strings) === null || _a === void 0 ? void 0 : _a.participantItemWithMoreOptionsAriaLabel : undefined)) !== null && _b !== void 0 ? _b : (_c = props.strings) === null || _c === void 0 ? void 0 : _c.participantItemAriaLabel, "aria-labelledby": `${props.ariaLabelledBy} ${participantItemId}`, "aria-expanded": !menuHidden, "aria-disabled": hasFlyout || props.onClick ? false : true, "aria-haspopup": hasFlyout ? true : undefined, "aria-controls": participantItemFlyoutId, "data-is-focusable": hasFlyout, "data-ui-id": "participant-item", className: react.mergeStyles(participantItemContainerStyle({
            clickable: hasFlyout
        }, theme), styles === null || styles === void 0 ? void 0 : styles.root), onMouseEnter: () => setItemHovered(true), onMouseLeave: () => setItemHovered(false), onClick: () => {
            if (!participantStateString) {
                setItemHovered(true);
                setMenuHidden(false);
                onClick === null || onClick === void 0 ? void 0 : onClick(props);
            }
            if (!menuHidden) {
                onDismissMenu();
            }
        }, onKeyDown: (event) => {
            if (event.key === 'Enter' || event.key === ' ') {
                setMenuHidden(false);
            }
        }, tabIndex: hasFlyout ? 0 : undefined },
        React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles({
                flexGrow: 1,
                maxWidth: '100%',
                alignItems: 'center'
            }) },
            avatar,
            me && React.createElement(react.Text, { className: meTextStyle }, strings.isMeText),
            React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles(infoContainerStyle) },
                !showMenuIcon && onRenderIcon && onRenderIcon(props),
                !me && participantStateString ? React.createElement(react.Text, { "data-ui-id": "participant-item-state-string", className: react.mergeStyles(participantStateStringStyles$1) }, participantStateString) : React.createElement(React.Fragment, null, hasFlyout && React.createElement(React.Fragment, null,
                    menuButton,
                    React.createElement(react.ContextualMenu, { id: participantItemFlyoutId, items: menuItems, hidden: menuHidden, target: containerRef, onItemClick: onDismissMenu, onDismiss: onDismissMenu, directionalHint: react.DirectionalHint.bottomRightEdge, className: contextualMenuStyle, calloutProps: {
                            preventDismissOnEvent: _preventDismissOnEvent
                        } }))))));
};
/** @private */
const formatParticipantStateString = (props, strings) => {
    return props.participantState === 'EarlyMedia' || props.participantState === 'Ringing' ? strings === null || strings === void 0 ? void 0 : strings.participantStateRinging : props.participantState === 'Hold' ? strings === null || strings === void 0 ? void 0 : strings.participantStateHold : undefined;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Raised hand icon for ParticipantList and VideoTitle.
 *
 * @public
 */
const RaisedHandIcon = () => {
    return React.createElement("svg", { width: "20", height: "20", viewBox: "0 0 20 20", fill: "none", xmlns: "http://www.w3.org/2000/svg" },
        React.createElement("defs", null,
            React.createElement("radialGradient", { id: "paint0_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(16.6796 10.8658) rotate(161.935) scale(13.4702 20.3408)" },
                React.createElement("stop", { offset: "0.817521", stopColor: "#E99329", stopOpacity: "0" }),
                React.createElement("stop", { offset: "0.961397", stopColor: "#E99329" })),
            React.createElement("radialGradient", { id: "paint1_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(11.7464 4.86265) rotate(103.662) scale(15.5522 10.8511)" },
                React.createElement("stop", { offset: "0.750766", stopColor: "#F88456", stopOpacity: "0" }),
                React.createElement("stop", { offset: "1", stopColor: "#F88456" })),
            React.createElement("radialGradient", { id: "paint2_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(10.8862 4.86067) rotate(98.3094) scale(14.3189 9.99881)" },
                React.createElement("stop", { offset: "0.937186", stopColor: "#F27A74", stopOpacity: "0" }),
                React.createElement("stop", { offset: "1", stopColor: "#F27A74" })),
            React.createElement("radialGradient", { id: "paint3_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(17.3363 11.238) rotate(-144.906) scale(1.08553 4.6268)" },
                React.createElement("stop", { offset: "0.0636767", stopColor: "#EAB170" }),
                React.createElement("stop", { offset: "1", stopColor: "#F6B44B", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint4_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(17.1484 8.75) rotate(129.452) scale(4.85649 1.57945)" },
                React.createElement("stop", { stopColor: "#FFC748" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFC64A", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint5_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(17.6953 8.75) rotate(135) scale(1.1601 1.07723)" },
                React.createElement("stop", { stopColor: "#FFE964" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFE65F", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint6_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(15.7422 8.47656) rotate(128.853) scale(3.61154 0.774997)" },
                React.createElement("stop", { offset: "0.251132", stopColor: "#EDB019" }),
                React.createElement("stop", { offset: "1", stopColor: "#EEB018", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint7_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(15.6055 8.04687) rotate(125.92) scale(3.06292 0.505665)" },
                React.createElement("stop", { offset: "0.0171863", stopColor: "#D29E4C" }),
                React.createElement("stop", { offset: "1", stopColor: "#DAA23E", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint8_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(14.2578 9.47266) rotate(-56.3099) scale(1.05631 0.227513)" },
                React.createElement("stop", { offset: "0.297381", stopColor: "#FFD260" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFD361", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint9_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(12.4609 9.62891) rotate(107.216) scale(1.45176 3.15212)" },
                React.createElement("stop", { offset: "0.251132", stopColor: "#FFCF48" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFD04B", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint10_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(12.3242 9.02344) rotate(144.638) scale(0.742444 1.38192)" },
                React.createElement("stop", { offset: "0.420068", stopColor: "#F2AA23" }),
                React.createElement("stop", { offset: "1", stopColor: "#F2AA23", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint11_linear_10246_159312", x1: "5.72266", y1: "10.9399", x2: "5.07979", y2: "9.68081", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { offset: "0.376446", stopColor: "#FFC23E" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFC23E", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint12_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(3.32031 8.28125) rotate(85.0032) scale(7.17571 2.51091)" },
                React.createElement("stop", { stopColor: "#E6943A" }),
                React.createElement("stop", { offset: "1", stopColor: "#E7943A", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint13_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(5.46875 6.17188) rotate(87.0391) scale(6.80596 1.61557)" },
                React.createElement("stop", { stopColor: "#FFCF50" }),
                React.createElement("stop", { offset: "0.741378", stopColor: "#FFB847" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFB848", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint14_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(6.69922 6.95312) rotate(85.2393) scale(4.00062 0.583904)" },
                React.createElement("stop", { stopColor: "#E79F08" }),
                React.createElement("stop", { offset: "1", stopColor: "#E79F08", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint15_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(5.82031 7.76306) rotate(83.1071) scale(3.90579 0.814298)" },
                React.createElement("stop", { stopColor: "#FFD14D" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFD04E", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint16_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(6.69922 9.17969) rotate(136.548) scale(1.53355 0.783563)" },
                React.createElement("stop", { offset: "0.426346", stopColor: "#FFD260" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFD361", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint17_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(5.46875 5.48828) rotate(80.3401) scale(0.931172 0.709009)" },
                React.createElement("stop", { stopColor: "#FFDA62" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFDA60", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint18_linear_10246_159312", x1: "6.05957", y1: "7.05618", x2: "7.26562", y2: "7.03125", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { stopColor: "#DE890D" }),
                React.createElement("stop", { offset: "1", stopColor: "#F9B23D" })),
            React.createElement("linearGradient", { id: "paint19_linear_10246_159312", x1: "11.7919", y1: "10.3872", x2: "11.5711", y2: "7.83625", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { offset: "0.405208", stopColor: "#FFC23E" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFC23E", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint20_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(9.90363 2.31009) rotate(56.3221) scale(2.85335 11.4486)" },
                React.createElement("stop", { stopColor: "#E1AE3F" }),
                React.createElement("stop", { offset: "1", stopColor: "#E8AD3C", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint21_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(8.35938 5.19531) rotate(86.2538) scale(6.57655 1.44733)" },
                React.createElement("stop", { stopColor: "#FFCE4C" }),
                React.createElement("stop", { offset: "0.621882", stopColor: "#FFBA49" }),
                React.createElement("stop", { offset: "1", stopColor: "#FEB848", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint22_linear_10246_159312", x1: "8.0957", y1: "9.0918", x2: "7.91992", y2: "7.06055", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { offset: "0.168836", stopColor: "#FFC23E" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFC23E", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint23_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(5.82031 5.97656) rotate(78.518) scale(5.10211 0.93429)" },
                React.createElement("stop", { offset: "0.425088", stopColor: "#BD6606" }),
                React.createElement("stop", { offset: "1", stopColor: "#C77122", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint24_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(8.24219 5.89844) rotate(83.5653) scale(5.22825 0.629818)" },
                React.createElement("stop", { stopColor: "#FFCB4F" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFCD4C", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint25_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(9.29687 8.20313) rotate(154.799) scale(1.46784 0.851345)" },
                React.createElement("stop", { offset: "0.426346", stopColor: "#FFD260" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFD361", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint26_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(9.33594 4.98047) rotate(88.915) scale(5.15717 0.762321)" },
                React.createElement("stop", { offset: "0.342657", stopColor: "#E29E07" }),
                React.createElement("stop", { offset: "1", stopColor: "#DF9906", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint27_linear_10246_159312", x1: "9.14062", y1: "6.25", x2: "8.80978", y2: "6.26984", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { stopColor: "#B76001" }),
                React.createElement("stop", { offset: "0.647059", stopColor: "#E59714", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint28_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(8.125 3.51562) rotate(86.1859) scale(1.17448 1.07205)" },
                React.createElement("stop", { stopColor: "#FFDA62" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFDA60", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint29_linear_10246_159312", x1: "9.0625", y1: "5.68143", x2: "10.5816", y2: "5.72531", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { stopColor: "#D5850A" }),
                React.createElement("stop", { offset: "1", stopColor: "#F9B343", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint30_linear_10246_159312", x1: "9.0625", y1: "5.68143", x2: "9.55093", y2: "5.6712", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { stopColor: "#B76001" }),
                React.createElement("stop", { offset: "1", stopColor: "#E59714", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint31_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(11.3014 4.55861) rotate(90) scale(5.01272 0.983716)" },
                React.createElement("stop", { stopColor: "#FFCB45" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFC44A", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint32_linear_10246_159312", x1: "10.0391", y1: "8.40332", x2: "10.0195", y2: "7.2461", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { offset: "0.210964", stopColor: "#FFC23E" }),
                React.createElement("stop", { offset: "0.80669", stopColor: "#FFC23E", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint33_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(11.7188 8.04687) rotate(166.996) scale(1.16638 0.888873)" },
                React.createElement("stop", { offset: "0.426346", stopColor: "#FFCA5B" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFCF60", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint34_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(11.9922 4.76563) rotate(92.1409) scale(4.18261 0.38546)" },
                React.createElement("stop", { offset: "0.342657", stopColor: "#E29E07" }),
                React.createElement("stop", { offset: "1", stopColor: "#DF9906", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint35_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(10.9766 4.51172) rotate(90) scale(9.27734 1.28906)" },
                React.createElement("stop", { stopColor: "#FFCE4C" }),
                React.createElement("stop", { offset: "0.621882", stopColor: "#FFBA49" }),
                React.createElement("stop", { offset: "1", stopColor: "#FEB848", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint36_linear_10246_159312", x1: "12.3828", y1: "5.9375", x2: "11.6797", y2: "5.93971", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { stopColor: "#E59714" }),
                React.createElement("stop", { offset: "1", stopColor: "#E59714", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint37_linear_10246_159312", x1: "10.0195", y1: "8.82935", x2: "10.2318", y2: "8.00715", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { stopColor: "#FFC23E" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFC23E", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint38_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(12.2847 7.66848) rotate(-113.448) scale(6.85723 7.32906)" },
                React.createElement("stop", { offset: "0.869582", stopColor: "#D8A13D", stopOpacity: "0" }),
                React.createElement("stop", { offset: "1", stopColor: "#D8A13D" })),
            React.createElement("radialGradient", { id: "paint39_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(11.2891 1.91406) rotate(93.0128) scale(1.48643 1.29895)" },
                React.createElement("stop", { stopColor: "#FFE763" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFE55D", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint40_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(11.8164 8.53516) rotate(168.022) scale(1.41758 0.903842)" },
                React.createElement("stop", { offset: "0.426346", stopColor: "#FFD260" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFD361", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint41_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(13.4766 5.89844) rotate(92.6574) scale(9.26778 2.30713)" },
                React.createElement("stop", { stopColor: "#FFCE4C" }),
                React.createElement("stop", { offset: "0.431711", stopColor: "#FFBA49" }),
                React.createElement("stop", { offset: "1", stopColor: "#FEB848", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint42_linear_10246_159312", x1: "11.5863", y1: "6.2696", x2: "13.5726", y2: "6.42645", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { offset: "0.0662431", stopColor: "#E08F13" }),
                React.createElement("stop", { offset: "0.602743", stopColor: "#E7981B", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint43_linear_10246_159312", x1: "11.5863", y1: "6.2696", x2: "13.259", y2: "6.33379", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { offset: "0.0662431", stopColor: "#C2681D" }),
                React.createElement("stop", { offset: "0.602743", stopColor: "#E69717", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint44_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(14.9907 8.54439) rotate(-115.482) scale(6.44971 7.3339)" },
                React.createElement("stop", { offset: "0.869582", stopColor: "#D8A13D", stopOpacity: "0" }),
                React.createElement("stop", { offset: "1", stopColor: "#D8A13D" })),
            React.createElement("radialGradient", { id: "paint45_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(14.0625 5.66406) rotate(91.8841) scale(5.94071 1.40501)" },
                React.createElement("stop", { stopColor: "#FFDC3F" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFDA3C", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint46_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(14.1016 3.51563) rotate(99.1412) scale(1.27667 1.12346)" },
                React.createElement("stop", { stopColor: "#FFE966" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFE966", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint47_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(11.7139 8.84766) rotate(124.509) scale(0.853264 2.02461)" },
                React.createElement("stop", { offset: "0.251132", stopColor: "#F0A623" }),
                React.createElement("stop", { offset: "1", stopColor: "#F2AA23", stopOpacity: "0" })),
            React.createElement("linearGradient", { id: "paint48_linear_10246_159312", x1: "13.877", y1: "10.0293", x2: "13.5547", y2: "9.05273", gradientUnits: "userSpaceOnUse" },
                React.createElement("stop", { offset: "0.0912498", stopColor: "#FFCD47" }),
                React.createElement("stop", { offset: "1", stopColor: "#FFCD52", stopOpacity: "0" })),
            React.createElement("radialGradient", { id: "paint49_radial_10246_159312", cx: "0", cy: "0", r: "1", gradientUnits: "userSpaceOnUse", gradientTransform: "translate(14.8059 9.14063) rotate(99.1613) scale(2.21576 0.860587)" },
                React.createElement("stop", { offset: "0.251132", stopColor: "#D69211" }),
                React.createElement("stop", { offset: "1", stopColor: "#DAA23E", stopOpacity: "0" }))),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "#FFC23E" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint0_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint1_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint2_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint3_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint4_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint5_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint6_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint7_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint8_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint9_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4435 1.23817C11.6481 1.06813 12.2265 2.26542 12.08 3.30216C12.3425 2.83259 12.7902 2.56116 13.3593 2.57808C14.1793 2.60246 14.83 3.32606 14.8046 4.15738L14.5986 9.26659L15.4927 8.14205C16.0297 7.46657 17.0049 7.3596 17.6721 7.90298C18.3408 8.44763 18.4466 9.43906 17.9083 10.1161L17.5242 10.5991L17.5205 10.6047C17.1674 11.0632 16.4446 12.0018 15.5377 13.5195C15.2451 13.9644 14.991 14.4396 14.7373 14.9139C14.3755 15.5904 14.0146 16.2651 13.5446 16.8478C12.6103 18.0157 11.2012 18.7207 9.41406 18.7207C7.46969 18.7207 5.96858 17.7407 5.06836 16.2598C4.39645 15.1544 4.09135 13.756 4.01367 12.3535C3.90459 10.384 3.64472 8.30425 3.52479 6.33789C3.46522 5.36133 4.03401 4.62819 4.82415 4.58984C5.43983 4.55997 5.83432 4.75144 6.11318 5.06836C6.11318 5.06836 6.05957 4.77051 6.05957 4.15738C6.05957 3.54425 6.58827 2.73728 7.41201 2.67578C8.7206 2.57808 8.93545 3.65234 9.0624 4.15738C9.08953 3.10816 8.83779 1.46464 10.4435 1.23817Z", fill: "url(#paint10_radial_10246_159312)" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "#FFC23E" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "url(#paint11_linear_10246_159312)" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "url(#paint12_radial_10246_159312)" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "url(#paint13_radial_10246_159312)" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "url(#paint14_radial_10246_159312)" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "url(#paint15_radial_10246_159312)" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "url(#paint16_radial_10246_159312)" }),
        React.createElement("path", { d: "M3.52479 6.33728C3.46522 5.36072 4.03401 4.62758 4.82415 4.58924C5.43983 4.55936 5.83432 4.75084 6.11318 5.06775C6.20599 5.20142 6.40332 5.6084 6.4502 6.16699C6.49707 6.72559 6.63574 8.10547 6.69922 8.72559C6.66584 9.373 6.06222 10.7221 3.91477 10.9394C3.78945 9.4056 3.61634 7.8384 3.52479 6.33728Z", fill: "url(#paint17_radial_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "#FFC23E" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint18_linear_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint19_linear_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint20_radial_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint21_radial_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint22_linear_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint23_radial_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint24_radial_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint25_radial_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint26_radial_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint27_linear_10246_159312)" }),
        React.createElement("path", { d: "M6.11318 5.0685C6.11318 5.0685 6.05957 4.77065 6.05957 4.15752C6.05957 3.54439 6.58827 2.73742 7.41201 2.67592C8.7206 2.57821 8.93545 3.65248 9.0624 4.15752L9.19922 7.71484C9.08203 8.04036 8.67578 8.74219 7.98828 8.94531C7.30078 9.14844 6.82943 8.86068 6.67969 8.69141C6.60156 7.89062 6.4375 6.20312 6.40625 5.85938C6.375 5.51562 6.19785 5.18889 6.11318 5.0685Z", fill: "url(#paint28_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "#FFC23E" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint29_linear_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint30_linear_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint31_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint32_linear_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint33_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint34_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint35_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint36_linear_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint37_linear_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint38_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint39_radial_10246_159312)" }),
        React.createElement("path", { d: "M10.4437 1.23817C11.6483 1.06813 12.2267 2.26542 12.0802 3.30216C11.9943 3.45951 11.9337 3.79649 11.9141 3.94531L11.6992 8.10547C10.8008 9.25781 9.53125 8.96484 9.17969 7.71484L9.0626 4.15738C9.08973 3.10816 8.83799 1.46464 10.4437 1.23817Z", fill: "url(#paint40_radial_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "#FFC23E" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint41_radial_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint42_linear_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint43_linear_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint44_radial_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint45_radial_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint46_radial_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint47_radial_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint48_linear_10246_159312)" }),
        React.createElement("path", { d: "M11.8994 4.15737C11.9482 2.8956 12.7903 2.56181 13.3594 2.57873C14.1794 2.60311 14.8301 3.32672 14.8047 4.15803L14.5987 8.92578C14.5501 9.1505 14.207 9.94531 13.2227 9.96094C12.2383 9.97656 11.8066 8.711 11.7139 8.07626L11.8994 4.15737Z", fill: "url(#paint49_radial_10246_159312)" }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const participantListStyle$1 = react.mergeStyles({
    height: '100%',
    padding: '0.125rem'
});
/**
 * @private
 */
const participantListItemStyle = {
    root: {
        paddingLeft: '1rem',
        paddingRight: '1rem',
        ':focus-visible': {
            margin: '0.125rem'
        }
    }
};
/**
 * @private
 */
const iconStyles$2 = react.mergeStyles({
    display: 'flex',
    lineHeight: 0,
    alignItems: 'center'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const onRenderParticipantDefault = (participant, strings, myUserId, onRenderAvatar, createParticipantMenuItems, styles, onParticipantClick, showParticipantOverflowTooltip, participantAriaLabelledBy, theme, pinnedParticipants) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
    const callingParticipant = participant;
    let presence = undefined;
    if (callingParticipant) {
        presence = react.PersonaPresence.none;
    }
    const menuItems = createParticipantMenuItems && createParticipantMenuItems(participant);
    const formatDisplayName = (displayName) => {
        if (displayName && strings.attendeeRole) {
            return _formatString(displayName, {
                AttendeeRole: strings.attendeeRole
            });
        }
        return displayName;
    };
    const displayName = formatDisplayName(participant.displayName);
    const callingPalette = theme.callingPalette;
    const isPinned = pinnedParticipants && (pinnedParticipants === null || pinnedParticipants === void 0 ? void 0 : pinnedParticipants.includes(participant.userId));
    const isScreenSharing = callingParticipant === null || callingParticipant === void 0 ? void 0 : callingParticipant.isScreenSharing;
    const isMuted = callingParticipant === null || callingParticipant === void 0 ? void 0 : callingParticipant.isMuted;
    const hasRaisedHand = callingParticipant === null || callingParticipant === void 0 ? void 0 : callingParticipant.raisedHand;
    const isAudioPermitted = (callingParticipant === null || callingParticipant === void 0 ? void 0 : callingParticipant.mediaAccess) ? callingParticipant.mediaAccess.isAudioPermitted : true;
    const isVideoPermitted = (callingParticipant === null || callingParticipant === void 0 ? void 0 : callingParticipant.mediaAccess) ? callingParticipant.mediaAccess.isVideoPermitted : true;
    const shouldRenderParticipantIcon = isScreenSharing || isMuted || hasRaisedHand || isPinned || !isAudioPermitted || !isVideoPermitted;
    const onRenderIcon = shouldRenderParticipantIcon ? () => {
        var _a, _b;
        return React.createElement(react.Stack, { horizontal: true, tokens: {
                childrenGap: '0.5rem'
            } },
            callingParticipant.raisedHand && React.createElement(react.Stack, { horizontal: true, tokens: {
                    childrenGap: '0.2rem'
                }, style: {
                    alignItems: 'center',
                    padding: '0.1rem 0.2rem',
                    backgroundColor: theme === null || theme === void 0 ? void 0 : theme.palette.neutralLighter,
                    borderRadius: '0.3rem'
                } },
                callingParticipant.raisedHand.raisedHandOrderPosition && React.createElement(react.Stack.Item, null,
                    React.createElement(react.Text, null, (_a = callingParticipant.raisedHand) === null || _a === void 0 ? void 0 : _a.raisedHandOrderPosition)),
                React.createElement(react.Stack.Item, null,
                    React.createElement(RaisedHandIcon, null))),
            callingParticipant.isScreenSharing && React.createElement(react.Icon, { iconName: "ParticipantItemScreenShareStart", className: iconStyles$2, ariaLabel: strings.sharingIconLabel }),
            callingParticipant.spotlight && React.createElement(react.Icon, { iconName: "ParticipantItemSpotlighted", className: iconStyles$2 }),
            isPinned && React.createElement(react.Icon, { iconName: "ParticipantItemPinned", className: iconStyles$2 }),
            callingParticipant.mediaAccess && !callingParticipant.mediaAccess.isVideoPermitted ? React.createElement(react.Icon, { iconName: "ControlButtonCameraProhibited", className: iconStyles$2, ariaLabel: strings.mutedIconLabel }) : undefined,
            callingParticipant.mediaAccess && !((_b = callingParticipant.mediaAccess) === null || _b === void 0 ? void 0 : _b.isAudioPermitted) ? React.createElement(react.Icon, { iconName: "ControlButtonMicProhibited", className: iconStyles$2, ariaLabel: strings.mutedIconLabel }) : undefined,
            (callingParticipant.mediaAccess ? callingParticipant.mediaAccess.isAudioPermitted : true) && callingParticipant.isMuted ? React.createElement(react.Icon, { iconName: "ParticipantItemMicOff", className: iconStyles$2, ariaLabel: strings.mutedIconLabel }) : undefined);
    } : () => null;
    const onRenderAvatarWithRaiseHand = (callingParticipant === null || callingParticipant === void 0 ? void 0 : callingParticipant.raisedHand) && onRenderAvatar ? (userId, options, defaultOnRender) => onRenderAvatar(userId, Object.assign(Object.assign({}, options), { styles: {
            root: {
                border: callingPalette.raiseHandGold
            }
        } }), defaultOnRender) : onRenderAvatar;
    const ariaLabelTemplate = (_a = (menuItems && menuItems.length > 0 ? strings === null || strings === void 0 ? void 0 : strings.participantItemWithMoreOptionsAriaLabel : undefined)) !== null && _a !== void 0 ? _a : strings === null || strings === void 0 ? void 0 : strings.participantItemAriaLabel;
    const ariaLabel = _formatString(ariaLabelTemplate !== null && ariaLabelTemplate !== void 0 ? ariaLabelTemplate : '', {
        displayName: displayName !== null && displayName !== void 0 ? displayName : '',
        connectionState: (_b = formatParticipantStateString(callingParticipant, strings)) !== null && _b !== void 0 ? _b : '',
        mutedState: (_c = (callingParticipant.isMuted ? strings === null || strings === void 0 ? void 0 : strings.mutedIconLabel : undefined)) !== null && _c !== void 0 ? _c : '',
        micDisabledState: (_e = (((_d = callingParticipant.mediaAccess) === null || _d === void 0 ? void 0 : _d.isAudioPermitted) === false ? strings === null || strings === void 0 ? void 0 : strings.micDisabledIconLabel : undefined)) !== null && _e !== void 0 ? _e : '',
        cameraDisabledState: (_g = (((_f = callingParticipant.mediaAccess) === null || _f === void 0 ? void 0 : _f.isVideoPermitted) === false ? strings === null || strings === void 0 ? void 0 : strings.cameraDisabledIconLabel : undefined)) !== null && _g !== void 0 ? _g : '',
        sharingState: (_h = (callingParticipant.isScreenSharing ? strings === null || strings === void 0 ? void 0 : strings.sharingIconLabel : undefined)) !== null && _h !== void 0 ? _h : '',
        handRaisedState: (_p = (((_j = callingParticipant.raisedHand) === null || _j === void 0 ? void 0 : _j.raisedHandOrderPosition) ? _formatString((_k = strings === null || strings === void 0 ? void 0 : strings.handRaisedIconLabel) !== null && _k !== void 0 ? _k : '', {
            position: (_o = (_m = (_l = callingParticipant.raisedHand) === null || _l === void 0 ? void 0 : _l.raisedHandOrderPosition) === null || _m === void 0 ? void 0 : _m.toString()) !== null && _o !== void 0 ? _o : ''
        }) : undefined)) !== null && _p !== void 0 ? _p : ''
    });
    return React.createElement(ParticipantItem, { styles: styles, key: participant.userId, userId: participant.userId, displayName: displayName, me: myUserId ? participant.userId === myUserId : false, menuItems: menuItems, presence: presence, onRenderIcon: onRenderIcon, onRenderAvatar: onRenderAvatarWithRaiseHand, onClick: onParticipantClick ? () => onParticipantClick === null || onParticipantClick === void 0 ? void 0 : onParticipantClick(participant) : undefined, showParticipantOverflowTooltip: showParticipantOverflowTooltip, participantState: callingParticipant.state, ariaLabelledBy: participantAriaLabelledBy, strings: {
            participantItemAriaLabel: ariaLabel
        } });
};
/**
 * Sort participants by raised hand order position
 */
const sortParticipants = (participants) => {
    const isParticipantListCallParticipant = function (participant) {
        return 'raisedHand' in participant;
    };
    participants.sort((a, b) => {
        if (!isParticipantListCallParticipant(a) || !isParticipantListCallParticipant(b)) {
            return 0;
        }
        const callA = a;
        const callB = b;
        if (callA.raisedHand && callB.raisedHand) {
            return callA.raisedHand.raisedHandOrderPosition - callB.raisedHand.raisedHandOrderPosition;
        }
        else if (callA.raisedHand) {
            return -1;
        }
        else if (callB.raisedHand) {
            return 1;
        }
        return 0;
    });
    return participants;
};
const getParticipantsForDefaultRender = (participants, excludeMe, myUserId) => {
    if (!excludeMe || !myUserId) {
        return [...participants];
    }
    const userIndex = participants.map(p => p.userId).indexOf(myUserId);
    if (userIndex === -1) {
        return [...participants];
    }
    const remoteParticipants = [...participants];
    remoteParticipants.splice(userIndex, 1);
    return remoteParticipants;
};
/**
 * Component to render all calling or chat participants.
 *
 * By default, each participant is rendered with {@link ParticipantItem}. See {@link ParticipantListProps.onRenderParticipant} to override.
 *
 * @public
 */
const ParticipantList$n = (props) => {
    var _a, _b, _c, _d;
    const { excludeMe = false, myUserId, participants, onRemoveParticipant, onRenderAvatar, onRenderParticipant, onFetchParticipantMenuItems, showParticipantOverflowTooltip, participantAriaLabelledBy, pinnedParticipants } = props;
    const theme = useTheme();
    const ids = useIdentifiers();
    const participantItemStrings = useLocale$1().strings.participantItem;
    const displayedParticipants = React.useMemo(() => {
        return onRenderParticipant ? participants : getParticipantsForDefaultRender(participants, excludeMe, myUserId);
    }, [participants, excludeMe, myUserId, onRenderParticipant]);
    sortParticipants(displayedParticipants);
    const createParticipantMenuItems = React.useCallback((participant) => {
        var _a, _b;
        let menuItems = [];
        const participantIsRemovable = participant.isRemovable;
        if (participant.userId !== myUserId && onRemoveParticipant && participantIsRemovable) {
            menuItems.push({
                key: 'remove',
                text: participantItemStrings.removeButtonLabel,
                onClick: () => onRemoveParticipant(participant.userId),
                itemProps: {
                    styles: (_b = (_a = props.styles) === null || _a === void 0 ? void 0 : _a.participantItemStyles) === null || _b === void 0 ? void 0 : _b.participantSubMenuItemsStyles
                },
                iconProps: {
                    iconName: 'ContextMenuRemoveParticipant',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                'data-ui-id': ids.participantListRemoveParticipantButton
            });
        }
        if (onFetchParticipantMenuItems) {
            menuItems = onFetchParticipantMenuItems(participant.userId, myUserId, menuItems);
        }
        return menuItems;
    }, [ids.participantListRemoveParticipantButton, myUserId, onFetchParticipantMenuItems, onRemoveParticipant, (_b = (_a = props.styles) === null || _a === void 0 ? void 0 : _a.participantItemStyles) === null || _b === void 0 ? void 0 : _b.participantSubMenuItemsStyles, participantItemStrings.removeButtonLabel]);
    const participantItemStyles = React.useMemo(() => { var _a; return react.merge(participantListItemStyle, (_a = props.styles) === null || _a === void 0 ? void 0 : _a.participantItemStyles); }, [(_c = props.styles) === null || _c === void 0 ? void 0 : _c.participantItemStyles]);
    return React.createElement(react.Stack, { "data-ui-id": ids.participantList, className: react.mergeStyles(participantListStyle$1, (_d = props.styles) === null || _d === void 0 ? void 0 : _d.root), role: 'menu' }, displayedParticipants.map((participant) => onRenderParticipant ? onRenderParticipant(participant) : onRenderParticipantDefault(participant, participantItemStrings, myUserId, onRenderAvatar, createParticipantMenuItems, participantItemStyles, props.onParticipantClick, showParticipantOverflowTooltip, participantAriaLabelledBy, theme, pinnedParticipants)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * For keyboard navigation - when this component has active focus, enter key and space keys should have the same behavior as mouse click.
 *
 * @private
 */
const submitWithKeyboard = (e, onSubmit) => {
    if (e.key === 'Enter' || e.key === ' ') {
        onSubmit(e);
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Maps the individual item in menuProps.items passed in the {@link DrawerMenu} into a UI component.
 *
 * @private
 */
const DrawerMenuItem = (props) => {
    var _a;
    const theme = useTheme();
    const onClick = (ev) => props.onItemClick && props.onItemClick(ev, props.itemKey);
    const onKeyPress = (ev) => onClick && submitWithKeyboard(ev, onClick);
    const secondaryIcon = props.secondaryIconProps ? React.createElement(MenuItemIcon, Object.assign({}, props.secondaryIconProps)) : props.subMenuProps ? React.createElement(MenuItemIcon, { iconName: "ChevronRight" }) : undefined;
    return React.createElement(react.FocusZone, { shouldFocusOnMount: props.shouldFocusOnMount },
        React.createElement(react.Stack, { tabIndex: 0, role: "menuitem", horizontal: true, className: react.mergeStyles(drawerMenuItemRootStyles$1(theme.palette.neutralLight, theme.fonts.small), props.disabled ? disabledDrawerMenuItemRootStyles$1(theme.palette.neutralQuaternaryAlt) : undefined, (_a = props.styles) === null || _a === void 0 ? void 0 : _a.root), onKeyPress: props.disabled ? undefined : onKeyPress, onClick: props.disabled ? undefined : onClick, tokens: menuItemChildrenGap, id: props.id, "aria-label": props.ariaLabel },
            props.iconProps && React.createElement(react.Stack.Item, { role: "presentation", styles: props.disabled ? {
                    root: {
                        color: theme.palette.neutralTertiaryAlt
                    }
                } : undefined },
                React.createElement(MenuItemIcon, Object.assign({}, props.iconProps))),
            React.createElement(react.Stack.Item, { styles: drawerMenuItemTextStyles, grow: true },
                React.createElement(react.Text, { styles: props.disabled ? {
                        root: {
                            color: theme.palette.neutralTertiaryAlt
                        }
                    } : undefined }, props.text)),
            props.secondaryText && React.createElement(react.Stack.Item, { styles: drawerMenuItemTextStyles, className: react.mergeStyles(secondaryTextStyles) },
                React.createElement(react.Text, { styles: {
                        root: {
                            color: props.disabled ? theme.palette.neutralTertiaryAlt : theme.palette.neutralSecondary
                        }
                    } }, props.secondaryText)),
            props.secondaryComponent && React.createElement(react.Stack.Item, null, props.secondaryComponent),
            secondaryIcon && React.createElement(react.Stack.Item, null, secondaryIcon)));
};
const MenuItemIcon = (props) => React.createElement(react.FontIcon, Object.assign({ className: react.mergeStyles(iconStyles$1) }, props));
const menuItemChildrenGap = {
    childrenGap: '0.5rem'
};
const drawerMenuItemRootStyles$1 = (hoverBackground, fontSize) => (Object.assign(Object.assign({}, fontSize), { height: '3rem', lineHeight: '3rem', padding: '0rem 0.75rem', cursor: 'pointer', ':hover, :focus': {
        background: hoverBackground
    } }));
const disabledDrawerMenuItemRootStyles$1 = (background) => ({
    pointerEvents: 'none',
    background: background,
    ':hover, :focus': {
        background: background
    }
});
/** Ensure long text entries appropriately show ellipsis instead of wrapping to a new line or showing a scrollbar */
const drawerMenuItemTextStyles = {
    root: {
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap'
    }
};
const iconStyles$1 = {
    // Vertically center icons in the menu item. Using line-height does not work for centering fluent SVG icons.
    display: 'flex',
    alignItems: 'center',
    height: '100%',
    // This can be removed when we upgrade to fluent-react-icons v2 (that removes the inner span element)
    ' span': {
        display: 'flex',
        alignItems: 'center',
        height: '100%'
    }
};
const secondaryTextStyles = {
    // limit width for secondaryText in the menu item so it does not overlap with text on left.
    maxWidth: '50%'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Container that holds the content of the drawer
 *
 * @private
 */
const DrawerContentContainer = (props) => {
    var _a;
    const theme = react.useTheme();
    const backgroundColor = theme.palette.white;
    const borderRadius = theme.effects.roundedCorner4;
    const rootStyles = react.mergeStyles(containerStyles$4(backgroundColor, borderRadius), (_a = props.styles) === null || _a === void 0 ? void 0 : _a.root);
    return React.createElement(react.Stack, { className: rootStyles },
        props.heading && React.createElement(react.Stack, { className: react.mergeStyles(headingContainerStyles) },
            React.createElement(react.Text, { className: react.mergeStyles(headingStyles(theme)) }, props.heading)),
        props.children);
};
const containerStyles$4 = (backgroundColor, borderRadius) => (Object.assign({ background: backgroundColor, borderTopRightRadius: borderRadius, borderTopLeftRadius: borderRadius }, react.AnimationStyles.slideUpIn10));
const headingContainerStyles = {
    textAlign: 'center',
    width: '100%',
    padding: '0.5rem'
};
const headingStyles = (theme) => ({
    color: theme.palette.neutralSecondary,
    fontSize: theme.fonts.smallPlus.fontSize,
    lineHeight: '1rem',
    whiteSpace: 'nowrap',
    overflow: 'hidden',
    textOverflow: 'ellipsis'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * An element that fills the space the DrawerContentContainer does not take up.
 * This is the element that enables the light dismiss feature.
 *
 * @private
 */
const DrawerLightDismiss = (props) => {
    return React.createElement(react.Stack, { className: react.mergeStyles(lightDismissContainerStyles, props.styles), grow: true, onClick: () => props.onDismiss() });
};
const lightDismissContainerStyles = {
    root: {
        height: '100%'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A `Drawer` can be used to reveal lightweight views inside your application.
 * They appear from the bottom of the screen upwards and are light-dismissed.
 *
 * @internal
 */
const _DrawerSurface = (props) => {
    var _a, _b, _c, _d;
    const rootStyles = props.disableMaxHeight ? react.mergeStyles(drawerSurfaceStyles, (_a = props.styles) === null || _a === void 0 ? void 0 : _a.root) : react.mergeStyles(drawerSurfaceStyles, focusTrapZoneStyles, (_b = props.styles) === null || _b === void 0 ? void 0 : _b.root);
    const containerStyles = react.mergeStyleSets(drawerContentContainerStyles, (_c = props.styles) === null || _c === void 0 ? void 0 : _c.drawerContentContainer);
    return React.createElement(react.Stack, { className: rootStyles },
        React.createElement(DrawerLightDismiss, { styles: (_d = props.styles) === null || _d === void 0 ? void 0 : _d.lightDismissRoot, onDismiss: props.onLightDismiss }),
        React.createElement(react.FocusTrapZone, { onKeyDown: e => {
                if (e.key === 'Escape' || e.key === 'Esc') {
                    props.onLightDismiss && props.onLightDismiss();
                }
            }, 
            // Ensure when the focus trap has focus, the light dismiss area can still be clicked with mouse to dismiss.
            // Note: this still correctly captures keyboard focus, this just allows mouse click outside of the focus trap.
            isClickableOutsideFocusTrap: true },
            React.createElement(DrawerContentContainer, { styles: containerStyles, heading: props.heading }, props.children)));
};
const drawerSurfaceStyles = {
    width: '100%',
    height: '100%',
    background: 'rgba(0,0,0,0.4)'
};
const focusTrapZoneStyles = {
    // Targets FocusTrapZone in drawer.
    // Setting percentage to Height to transform a container does not work unless the
    // direct parent container also has a Height set other than 'auto'.
    '> div:nth-child(2)': {
        maxHeight: '75%',
        overflow: 'auto'
    }
};
const drawerContentContainerStyles = {
    root: {
        height: 'auto'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const isDrawerMenuItem = (item) => {
    return item.onRendererContent === undefined;
};
/**
 * Takes a set of menu items and returns a created menu inside a {@link _DrawerSurface}.
 *
 * @internal
 */
const _DrawerMenu = (props) => {
    var _a, _b;
    // This component breaks from a pure component pattern in order to internally support sub menus.
    // When a sub menu item is clicked the menu items displayed is updated to be that of the submenu.
    // To track this state we store a list of the keys clicked up until this point.
    const [selectedKeyPath, setSelectedKeyPath] = React.useState([]);
    // Get the menu items that should be rendered
    const menuItemsToRender = React.useMemo(() => {
        var _a;
        let items = props.items;
        for (const subMenuKey of selectedKeyPath) {
            items = (_a = items === null || items === void 0 ? void 0 : items.find(item => isDrawerMenuItem(item) && item.itemKey === subMenuKey)) === null || _a === void 0 ? void 0 : _a.subMenuProps;
        }
        return items;
    }, [props.items, selectedKeyPath]);
    // When an item is clicked and it contains a submenu, push the key for the submenu. This will ensure
    // a new render is triggered, menuItemsToRender will be re-calculated and the submenu will render.
    const onItemClick = React.useCallback((item, ev, itemKey) => {
        var _a;
        if (item.subMenuProps) {
            setSelectedKeyPath([...selectedKeyPath, item.itemKey]);
        }
        (_a = item.onItemClick) === null || _a === void 0 ? void 0 : _a.call(item, ev, itemKey);
    }, [selectedKeyPath]);
    // Ensure the first item has a border radius that matches the DrawerSurface
    const borderRadius = useTheme().effects.roundedCorner4;
    const firstItemStyle = (menuItemsToRender === null || menuItemsToRender === void 0 ? void 0 : menuItemsToRender[0]) && ((_a = menuItemsToRender[0]) === null || _a === void 0 ? void 0 : _a.styles);
    const modifiedFirstItemStyle = React.useMemo(() => react.merge(firstItemStyle !== null && firstItemStyle !== void 0 ? firstItemStyle : {}, {
        root: {
            borderTopRightRadius: borderRadius,
            borderTopLeftRadius: borderRadius
        }
    }), [firstItemStyle, borderRadius]);
    return React.createElement(_DrawerSurface, { disableMaxHeight: props.disableMaxHeight, styles: (_b = props.styles) === null || _b === void 0 ? void 0 : _b.drawerSurfaceStyles, onLightDismiss: props.onLightDismiss, heading: props.heading, "data-ui-id": "drawer-menu" },
        React.createElement(react.Stack, { styles: props.styles, role: "menu", "data-ui-id": "drawer-menu" }, menuItemsToRender === null || menuItemsToRender === void 0 ? void 0 :
            menuItemsToRender.slice(0, 1).map(item => {
                var _a;
                return isDrawerMenuItem(item) ? React.createElement(DrawerMenuItem, Object.assign({}, item, { key: `${item.itemKey}` + '0', shouldFocusOnMount: item.itemKey === 'reactions' ? false : true, styles: modifiedFirstItemStyle, onItemClick: item.itemKey === 'reactions' ? undefined : (ev, itemKey) => {
                        onItemClick(item, ev, itemKey);
                    } })) : (_a = item.onRendererContent) === null || _a === void 0 ? void 0 : _a.call(item);
            }), menuItemsToRender === null || menuItemsToRender === void 0 ? void 0 :
            menuItemsToRender.slice(1).map((item, i) => {
                var _a;
                return isDrawerMenuItem(item) ? React.createElement(DrawerMenuItem, Object.assign({}, item, { key: `${item.itemKey}` + `${i + 1}`, onItemClick: (ev, itemKey) => {
                        onItemClick(item, ev, itemKey);
                    } })) : (_a = item.onRendererContent) === null || _a === void 0 ? void 0 : _a.call(item);
            })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const playFrames$1 = react.memoizeFunction(() => react.keyframes({
    from: {
        backgroundPosition: '0px 8568px'
    },
    to: {
        backgroundPosition: '0px 0px'
    }
}));
/**
 * @param backgroundImage - the uri for the reaction emoji resource
 * @param animationPlayState - the value is either 'running' or 'paused' based on the mouse hover event
 *
 * @private
 */
const emojiStyles = (backgroundImage, frameCount) => {
    const imageResourceUrl = `url(${backgroundImage})`;
    const steps = frameCount !== null && frameCount !== void 0 ? frameCount : 51;
    return {
        display: 'flex',
        flexDirection: 'column',
        height: '100%',
        width: '100%',
        backgroundImage: imageResourceUrl,
        justifyContent: 'center',
        alignItems: 'center',
        backgroundSize: `2.75rem 133.875rem`,
        transition: 'opacity 2s',
        minWidth: '2.75rem',
        backgroundColor: 'transparent',
        transform: 'scale(0.6)',
        ':hover': {
            transform: 'scale(0.8)',
            animationName: playFrames$1(),
            animationDuration: '8.12s',
            animationTimingFunction: `steps(${steps})`,
            animationIterationCount: 'infinite',
            backgroundColor: 'transparent'
        },
        ':active': {
            backgroundColor: 'transparent'
        }
    };
};
/**
 *
 * @private
 */
const reactionToolTipHostStyle = () => {
    return {
        root: {
            display: 'flex',
            flexDirection: 'column',
            height: '100%',
            width: '2.75rem'
        }
    };
};
/**
 *
 * @private
 */
const mobileViewMenuItemStyle = () => {
    return {
        display: 'flex',
        justifyContent: 'space-between',
        alignItems: 'center',
        flexDirection: 'row',
        width: '100%',
        height: '2.625rem'
    };
};
/**
 * @param backgroundImage - the uri for the reaction emoji resource
 * @param animationPlayState - the value is either 'running' or 'paused' based on the mouse hover event
 *
 * @private
 */
const mobileViewEmojiStyles = (backgroundImage, animationPlayState) => {
    const imageResourceUrl = `url(${backgroundImage})`;
    return {
        display: 'flex',
        flexDirection: 'column',
        height: '100%',
        width: '2.75rem',
        backgroundImage: imageResourceUrl,
        animationName: playFrames$1(),
        animationDuration: '8.12s',
        animationTimingFunction: `steps(102)`,
        animationPlayState: animationPlayState,
        animationIterationCount: 'infinite',
        justifyContent: 'center',
        alignItems: 'center',
        backgroundPosition: 'center',
        backgroundSize: `2.75rem 133.875rem`,
        transition: 'opacity 2s',
        backgroundColor: 'transparent',
        transform: `${'scale(0.8)' }`
    };
};
/**
 * @private
 */
const reactionButtonStyles = (theme) => ({
    rootChecked: {
        background: theme.palette.themePrimary,
        color: theme.palette.white
    },
    rootCheckedHovered: {
        background: theme.palette.themePrimary,
        color: theme.palette.white
    },
    labelChecked: {
        color: theme.palette.white
    }
});
/**
 * @private
 */
const reactionItemButtonStyles = {
    root: {
        border: 'none',
        height: '2.75rem',
        width: '2.75rem'
    }
};
/**
 * @private
 */
const reactionButtonCalloutStyles = {
    container: {},
    root: {},
    beak: {},
    beakCurtain: {},
    calloutMain: {
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        flexDirection: 'row',
        width: '13.75rem',
        height: '2.625rem',
        // Ensure that when one emoji is hovered, the other emojis are partially faded out
        ':hover > :not(:hover)': {
            opacity: '0.5'
        }
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Maps the individual item in menuProps.items passed in the {@link DrawerMenu} into a UI component.
 *
 * @internal
 */
const _ReactionDrawerMenuItem = (props) => {
    var _a, _b, _c, _d, _e;
    const theme = useTheme();
    const resources = props.reactionResources;
    const emojiResource = new Map([['like', (_a = resources === null || resources === void 0 ? void 0 : resources.likeReaction) === null || _a === void 0 ? void 0 : _a.url], ['heart', (_b = resources === null || resources === void 0 ? void 0 : resources.heartReaction) === null || _b === void 0 ? void 0 : _b.url], ['laugh', (_c = resources === null || resources === void 0 ? void 0 : resources.laughReaction) === null || _c === void 0 ? void 0 : _c.url], ['applause', (_d = resources === null || resources === void 0 ? void 0 : resources.applauseReaction) === null || _d === void 0 ? void 0 : _d.url], ['surprised', (_e = resources === null || resources === void 0 ? void 0 : resources.surprisedReaction) === null || _e === void 0 ? void 0 : _e.url]]);
    const emojis = ['like', 'heart', 'laugh', 'applause', 'surprised'];
    const borderRadius = useTheme().effects.roundedCorner4;
    const modifiedFirstItemStyle = {
        root: {
            borderTopRightRadius: borderRadius,
            borderTopLeftRadius: borderRadius,
            marginTop: '12px'
        }
    };
    return React.createElement(react.Stack, { "data-ui-id": "reaction-mobile-drawer-menu-item", id: "reaction", role: "menuitem", horizontal: true, className: react.mergeStyles(drawerMenuItemRootStyles(theme.palette.neutralLight, theme.fonts.small), props.disabled ? disabledDrawerMenuItemRootStyles(theme.palette.neutralQuaternaryAlt) : undefined, modifiedFirstItemStyle.root) },
        React.createElement("div", { style: mobileViewMenuItemStyle() }, emojis.map((emoji, index) => {
            const resourceUrl = emojiResource.get(emoji.toString());
            return React.createElement(react.IconButton, { key: index, onClick: () => {
                    var _a;
                    (_a = props.onReactionClick) === null || _a === void 0 ? void 0 : _a.call(props, emoji);
                }, style: mobileViewEmojiStyles(resourceUrl ? resourceUrl : '', 'running') });
        })));
};
const drawerMenuItemRootStyles = (hoverBackground, fontSize) => (Object.assign(Object.assign({}, fontSize), { height: '3rem', lineHeight: '3rem', padding: '0rem 0.75rem', cursor: 'pointer', ':hover, :focus': {
        background: hoverBackground
    } }));
const disabledDrawerMenuItemRootStyles = (background) => ({
    pointerEvents: 'none',
    background: background,
    ':hover, :focus': {
        background: background
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const remoteVideoTileWrapperStyle = {
    height: '100%',
    width: '100%',
    position: 'relative'
};
/** @private */
const drawerMenuWrapperStyles = {
    root: {
        position: 'absolute',
        top: 0,
        left: 0,
        width: '100%',
        height: '100%',
        // Any zIndex > 0 will work because this is the only absolutely
        // positioned element in the container.
        zIndex: 2
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const callingComponentLogger = logger.createClientLogger('communication-react:calling-component');

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Helper hook to maintain the video stream lifecycle. This calls onCreateStreamView and onDisposeStreamView
 * appropriately based on react lifecycle events and prop changes.
 *
 * @remarks
 *
 * Notes on handling changes to scaling mode:
 *
 * Ideally we have access to the original StreamRenderView and can call view.updateScalingMode() and do not need to recreate the stream view.
 * However, to support backwards compat we cannot guarantee this. If we don't have access to the original StreamRenderView we need to dispose
 * the old view and create a new one.
 *
 * Supporting both of these scenarios became too complex and fragile. When we introduce a breaking change this should be update to ensure that
 * onCreateStreamView _must_ return a view object with updateScalingMode and update logic in this hook to call view.updateScalingMode instead
 * of recreating the stream.
 *
 * @private
 */
const useVideoStreamLifecycleMaintainer = (props) => {
    const { isMirrored, isScreenSharingOn, isStreamAvailable, onCreateStreamView, onDisposeStreamView, renderElementExists, scalingMode, streamId, isVideoPermitted } = props;
    const [videoStreamViewResult, setVideoStreamViewResult] = React.useState();
    React.useEffect(() => {
        var _a;
        if (isVideoPermitted !== false && isStreamAvailable && !renderElementExists) {
            (_a = onCreateStreamView === null || onCreateStreamView === void 0 ? void 0 : onCreateStreamView({
                isMirrored,
                scalingMode
            })) === null || _a === void 0 ? void 0 : _a.then(result => {
                result && setVideoStreamViewResult(result);
            });
        }
        // Always clean up element to make tile up to date and be able to dispose correctly
        return () => {
            if (renderElementExists) {
                // TODO: Remove `if isScreenSharingOn` when we isolate dispose behavior for screen share
                if (!isScreenSharingOn) {
                    onDisposeStreamView === null || onDisposeStreamView === void 0 ? void 0 : onDisposeStreamView();
                }
            }
            else {
                callingComponentLogger.warning('Stream view element does not exist when disposing stream view');
            }
        };
    }, [isMirrored, isScreenSharingOn, isStreamAvailable, onCreateStreamView, onDisposeStreamView, renderElementExists, scalingMode, streamId, isVideoPermitted]);
    // The execution order for above useEffect is onCreateRemoteStreamView =>(async time gap) RenderElement generated => element disposed => onDisposeRemoteStreamView
    // Element disposed could happen during async time gap, which still cause leaks for unused renderElement.
    // Need to do an entire cleanup when remoteTile gets disposed and make sure element gets correctly disposed
    React.useEffect(() => {
        return () => {
            // TODO: Remove `if isScreenSharingOn` when we isolate dispose behavior for screen share
            if (!isScreenSharingOn) {
                onDisposeStreamView === null || onDisposeStreamView === void 0 ? void 0 : onDisposeStreamView();
            }
        };
    }, [isScreenSharingOn, onDisposeStreamView]);
    return videoStreamViewResult;
};
/**
 * Extension of {@link useVideoStreamLifecycleMaintainer} specifically for local video streams
 *
 * @private
 */
const useLocalVideoStreamLifecycleMaintainer = (props) => {
    const { onCreateLocalStreamView, onDisposeLocalStreamView } = props;
    const onCreateStreamView = React.useMemo(() => (options) => {
        return onCreateLocalStreamView === null || onCreateLocalStreamView === void 0 ? void 0 : onCreateLocalStreamView(options);
    }, [onCreateLocalStreamView]);
    const onDisposeStreamView = React.useMemo(() => () => {
        onDisposeLocalStreamView === null || onDisposeLocalStreamView === void 0 ? void 0 : onDisposeLocalStreamView();
    }, [onDisposeLocalStreamView]);
    return useVideoStreamLifecycleMaintainer(Object.assign(Object.assign({}, props), { onCreateStreamView,
        onDisposeStreamView }));
};
/**
 * Extension of {@link useVideoStreamLifecycleMaintainer} specifically for remote video streams
 *
 * @private
 */
const useRemoteVideoStreamLifecycleMaintainer = (props) => {
    const { remoteParticipantId, onCreateRemoteStreamView, onDisposeRemoteStreamView } = props;
    const onCreateStreamView = React.useMemo(() => (options) => {
        return onCreateRemoteStreamView === null || onCreateRemoteStreamView === void 0 ? void 0 : onCreateRemoteStreamView(remoteParticipantId, options);
    }, [onCreateRemoteStreamView, remoteParticipantId]);
    const onDisposeStreamView = React.useMemo(() => () => {
        onDisposeRemoteStreamView === null || onDisposeRemoteStreamView === void 0 ? void 0 : onDisposeRemoteStreamView(remoteParticipantId);
    }, [onDisposeRemoteStreamView, remoteParticipantId]);
    return useVideoStreamLifecycleMaintainer(Object.assign(Object.assign({}, props), { onCreateStreamView,
        onDisposeStreamView }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useVideoTileContextualMenuProps = (props) => {
    var _a;
    const { participant, view, strings, isPinned, onPinParticipant, onUnpinParticipant, onUpdateScalingMode, disablePinMenuItem, toggleAnnouncerString, spotlightedParticipantUserIds = [], isSpotlighted, onStartSpotlight, onStopSpotlight, maxParticipantsToSpotlight, myUserId, onMuteParticipant, onForbidAudio, onPermitAudio, onForbidVideo, onPermitVideo } = props;
    const scalingMode = React.useMemo(() => {
        var _a;
        return (_a = props.participant.videoStream) === null || _a === void 0 ? void 0 : _a.scalingMode;
    }, [(_a = props.participant.videoStream) === null || _a === void 0 ? void 0 : _a.scalingMode]);
    const contextualMenuProps = React.useMemo(() => {
        var _a, _b;
        const items = [];
        if (onMuteParticipant && (strings === null || strings === void 0 ? void 0 : strings.muteParticipantMenuItemLabel)) {
            items.push({
                key: 'mute',
                text: strings === null || strings === void 0 ? void 0 : strings.muteParticipantMenuItemLabel,
                iconProps: {
                    iconName: 'ContextualMenuMicMutedIcon',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => onMuteParticipant(participant.userId),
                'data-ui-id': 'video-tile-mute-participant',
                ariaLabel: strings === null || strings === void 0 ? void 0 : strings.muteParticipantMenuItemLabel,
                disabled: participant.isMuted
            });
        }
        if (participant.canAudioBeForbidden && participant.mediaAccess && !participant.mediaAccess.isAudioPermitted && onPermitAudio) {
            items.push({
                key: 'permitAudio',
                text: strings === null || strings === void 0 ? void 0 : strings.permitAudioTileMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonMicOn',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => onPermitAudio([participant.userId]),
                'data-ui-id': 'video-tile-permit-audio',
                ariaLabel: strings === null || strings === void 0 ? void 0 : strings.permitAudioTileMenuLabel
            });
        }
        if (participant.canAudioBeForbidden && ((_a = participant.mediaAccess) === null || _a === void 0 ? void 0 : _a.isAudioPermitted) && onForbidAudio) {
            items.push({
                key: 'forbidAudio',
                text: strings === null || strings === void 0 ? void 0 : strings.forbidAudioTileMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonMicProhibited',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => onForbidAudio([participant.userId]),
                'data-ui-id': 'video-tile-forbid-audio',
                ariaLabel: strings === null || strings === void 0 ? void 0 : strings.forbidAudioTileMenuLabel
            });
        }
        if (participant.canVideoBeForbidden && participant.mediaAccess && !participant.mediaAccess.isVideoPermitted && onPermitVideo) {
            items.push({
                key: 'permitVideo',
                text: strings === null || strings === void 0 ? void 0 : strings.permitVideoTileMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonCameraOn',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => onPermitVideo([participant.userId]),
                'data-ui-id': 'video-tile-permit-video',
                ariaLabel: strings === null || strings === void 0 ? void 0 : strings.permitVideoTileMenuLabel
            });
        }
        if (participant.canVideoBeForbidden && ((_b = participant.mediaAccess) === null || _b === void 0 ? void 0 : _b.isVideoPermitted) && onForbidVideo) {
            items.push({
                key: 'forbidVideo',
                text: strings === null || strings === void 0 ? void 0 : strings.forbidVideoTileMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonCameraProhibited',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => onForbidVideo([participant.userId]),
                'data-ui-id': 'video-tile-forbid-video',
                ariaLabel: strings === null || strings === void 0 ? void 0 : strings.forbidVideoTileMenuLabel
            });
        }
        if (isPinned !== undefined) {
            if (isPinned && onUnpinParticipant && (strings === null || strings === void 0 ? void 0 : strings.unpinParticipantForMe)) {
                let unpinActionString = undefined;
                if (toggleAnnouncerString && strings.unpinParticipantMenuItemAriaLabel && participant.displayName) {
                    unpinActionString = _formatString(strings === null || strings === void 0 ? void 0 : strings.unpinParticipantMenuItemAriaLabel, {
                        participantName: participant.displayName
                    });
                }
                items.push({
                    key: 'unpin',
                    text: strings.unpinParticipantForMe,
                    iconProps: {
                        iconName: 'UnpinParticipant',
                        styles: {
                            root: {
                                lineHeight: '1rem',
                                textAlign: 'center'
                            }
                        }
                    },
                    onClick: () => {
                        onUnpinParticipant(participant.userId);
                        unpinActionString && (toggleAnnouncerString === null || toggleAnnouncerString === void 0 ? void 0 : toggleAnnouncerString(unpinActionString));
                    },
                    'data-ui-id': 'video-tile-unpin-participant-button',
                    ariaLabel: unpinActionString
                });
            }
            if (!isPinned && onPinParticipant && (strings === null || strings === void 0 ? void 0 : strings.pinParticipantForMe)) {
                let pinActionString = undefined;
                if (toggleAnnouncerString && strings.pinnedParticipantAnnouncementAriaLabel && participant.displayName) {
                    pinActionString = _formatString(strings === null || strings === void 0 ? void 0 : strings.pinnedParticipantAnnouncementAriaLabel, {
                        participantName: participant.displayName
                    });
                }
                items.push({
                    key: 'pin',
                    text: disablePinMenuItem ? strings.pinParticipantForMeLimitReached : strings.pinParticipantForMe,
                    iconProps: {
                        iconName: 'PinParticipant',
                        styles: {
                            root: {
                                lineHeight: '1rem',
                                textAlign: 'center'
                            }
                        }
                    },
                    onClick: () => {
                        onPinParticipant(participant.userId);
                        pinActionString && (toggleAnnouncerString === null || toggleAnnouncerString === void 0 ? void 0 : toggleAnnouncerString(pinActionString));
                    },
                    'data-ui-id': 'video-tile-pin-participant-button',
                    disabled: disablePinMenuItem || isSpotlighted,
                    ariaLabel: strings.pinParticipantForMe
                });
            }
        }
        if (isSpotlighted) {
            const stopSpotlightMenuLabel = myUserId === participant.userId ? strings === null || strings === void 0 ? void 0 : strings.stopSpotlightOnSelfVideoTileMenuLabel : strings === null || strings === void 0 ? void 0 : strings.stopSpotlightVideoTileMenuLabel;
            if (onStopSpotlight && participant.userId && (strings === null || strings === void 0 ? void 0 : strings.stopSpotlightVideoTileMenuLabel)) {
                items.push({
                    key: 'stopSpotlight',
                    text: stopSpotlightMenuLabel,
                    iconProps: {
                        iconName: 'StopSpotlightContextualMenuItem',
                        styles: {
                            root: {
                                lineHeight: 0
                            }
                        }
                    },
                    onClick: () => onStopSpotlight([participant.userId]),
                    ariaLabel: strings.stopSpotlightVideoTileMenuLabel
                });
            }
        }
        else {
            const startSpotlightMenuLabel = spotlightedParticipantUserIds && spotlightedParticipantUserIds.length > 0 ? strings === null || strings === void 0 ? void 0 : strings.addSpotlightVideoTileMenuLabel : strings === null || strings === void 0 ? void 0 : strings.startSpotlightVideoTileMenuLabel;
            const maxSpotlightedParticipantsReached = maxParticipantsToSpotlight ? spotlightedParticipantUserIds.length >= maxParticipantsToSpotlight : false;
            if (onStartSpotlight && participant.userId && startSpotlightMenuLabel) {
                items.push({
                    key: 'startSpotlight',
                    text: startSpotlightMenuLabel,
                    iconProps: {
                        iconName: 'StartSpotlightContextualMenuItem',
                        styles: {
                            root: {
                                lineHeight: 0
                            }
                        }
                    },
                    onClick: () => onStartSpotlight([participant.userId]),
                    ariaLabel: startSpotlightMenuLabel,
                    disabled: maxSpotlightedParticipantsReached,
                    title: maxSpotlightedParticipantsReached ? strings === null || strings === void 0 ? void 0 : strings.spotlightLimitReachedMenuTitle : undefined
                });
            }
        }
        if (scalingMode) {
            if (scalingMode === 'Crop' && (strings === null || strings === void 0 ? void 0 : strings.fitRemoteParticipantToFrame)) {
                items.push({
                    key: 'fitRemoteParticipantToFrame',
                    text: strings.fitRemoteParticipantToFrame,
                    iconProps: {
                        iconName: 'VideoTileScaleFit',
                        styles: {
                            root: {
                                lineHeight: '1rem',
                                textAlign: 'center'
                            }
                        }
                    },
                    onClick: () => {
                        onUpdateScalingMode === null || onUpdateScalingMode === void 0 ? void 0 : onUpdateScalingMode(participant.userId, 'Fit');
                        view === null || view === void 0 ? void 0 : view.updateScalingMode('Fit');
                    },
                    'data-ui-id': 'video-tile-fit-to-frame',
                    ariaLabel: strings.fitRemoteParticipantToFrame
                });
            }
            else if (scalingMode === 'Fit' && (strings === null || strings === void 0 ? void 0 : strings.fillRemoteParticipantFrame)) {
                items.push({
                    key: 'fillRemoteParticipantFrame',
                    text: strings.fillRemoteParticipantFrame,
                    iconProps: {
                        iconName: 'VideoTileScaleFill',
                        styles: {
                            root: {
                                lineHeight: '1rem',
                                textAlign: 'center'
                            }
                        }
                    },
                    onClick: () => {
                        onUpdateScalingMode === null || onUpdateScalingMode === void 0 ? void 0 : onUpdateScalingMode(participant.userId, 'Crop');
                        view === null || view === void 0 ? void 0 : view.updateScalingMode('Crop');
                    },
                    'data-ui-id': 'video-tile-fill-frame',
                    ariaLabel: strings.fillRemoteParticipantFrame
                });
            }
        }
        if (items.length === 0) {
            return undefined;
        }
        return {
            items,
            styles: {},
            calloutProps: {
                preventDismissOnEvent: _preventDismissOnEvent
            },
            shouldFocusOnContainer: false
        };
    }, [onMuteParticipant, strings, participant.isMuted, participant.userId, participant.displayName, isPinned, isSpotlighted, scalingMode, onUnpinParticipant, onPinParticipant, toggleAnnouncerString, disablePinMenuItem, myUserId, onStopSpotlight, spotlightedParticipantUserIds, maxParticipantsToSpotlight, onStartSpotlight, onUpdateScalingMode, view, participant.canAudioBeForbidden, participant.canVideoBeForbidden, participant.mediaAccess, onPermitAudio, onForbidAudio, onPermitVideo, onForbidVideo]);
    return contextualMenuProps;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const REACTION_START_DISPLAY_SIZE = 44;
/** @private */
const REACTION_NUMBER_OF_ANIMATION_FRAMES = 51;
/** @private */
const REACTION_SCREEN_SHARE_ANIMATION_TIME_MS = 4133;
/** @private */
const REACTION_DEFAULT_RESOURCE_FRAME_SIZE_PX = 128;
/** @private */
const getCombinedKey = (userId, reactionType, receivedAt) => {
    const receivedTime = receivedAt.toISOString();
    return userId + reactionType + receivedTime;
};
/** @private */
const getReceivedUnixTime = (receivedTime) => {
    return receivedTime.getTime();
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const rootStyles = {
    position: 'relative',
    height: '100%',
    width: '100%'
};
/** @private */
const videoTileHighContrastStyles = (theme) => ({
    '@media (forced-colors: active)': {
        border: `0.125rem solid ${theme.palette.black}`
    }
});
/**
 * @private
 */
const videoContainerStyles = {
    position: 'absolute',
    top: '0',
    left: '0',
    width: '100%',
    height: '100%',
    minWidth: '100%',
    minHeight: '100%',
    objectPosition: 'center',
    objectFit: 'cover',
    zIndex: 0
};
/**
 * @private
 */
const overlayContainerStyles = {
    width: '100%',
    height: '100%',
    zIndex: 5
};
/**
 * @private
 */
const tileInfoContainerStyle = react.mergeStyles({
    position: 'absolute',
    bottom: '0',
    left: '0',
    padding: '0.5rem',
    width: '100%'
});
/**
 * @private
 */
const disabledVideoHint = react.mergeStyles({
    backgroundColor: 'inherit',
    boxShadow: 'none',
    textAlign: 'left',
    overflow: 'hidden',
    whiteSpace: 'nowrap',
    alignItems: 'center',
    padding: '0.15rem',
    maxWidth: '100%'
});
/**
 * @private
 */
const displayNameStyle = {
    padding: '0.1rem',
    fontSize: '0.75rem',
    fontWeight: 600,
    // Text component will take body color by default (white in Dark Mode), so forcing it to be parent container color
    color: 'inherit',
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    maxWidth: '100%'
};
/**
 * @private
 */
const pinIconStyle = {
    padding: '0.125rem'
};
/**
 * @private
 */
const iconContainerStyle = {
    margin: 'auto 0.2rem',
    alignItems: 'center',
    '& svg': {
        display: 'block',
        // Similar to text color, icon color will be inherited from parent container
        color: 'inherit'
    }
};
/**
 * @private
 */
const participantStateStringStyles = (theme) => {
    return {
        minWidth: 'max-content',
        color: theme.palette.black,
        fontSize: '0.75rem',
        lineHeight: 'normal',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
        padding: '0.1rem'
    };
};
/**
 * @private
 */
const moreButtonStyles = (theme) => {
    return {
        root: {
            // To ensure that the button is clickable when there is a floating video tile
            zIndex: 1,
            top: '-0.125rem',
            height: '100%',
            padding: '0rem'
        },
        rootHovered: {
            background: 'none'
        },
        rootPressed: {
            background: 'none'
        },
        icon: {
            color: theme.palette.neutralPrimary
        },
        iconExpanded: {
            background: 'none',
            color: theme.palette.themePrimary
        }
    };
};
/**
 * @private
 */
const raiseHandContainerStyles = (theme, limitedSpace) => react.mergeStyles({
    alignItems: 'center',
    padding: '0.2rem 0.3rem',
    backgroundColor: theme.palette.white,
    opacity: 0.9,
    borderRadius: '1rem',
    margin: '0.5rem',
    width: 'fit-content',
    position: 'absolute',
    top: 0,
    height: 'fit-content'
}, limitedSpace && raiseHandLimitedSpaceStyles);
/**
 * @private
 */
const raiseHandLimitedSpaceStyles = {
    // position centrally
    marginLeft: 'auto',
    marginRight: 'auto',
    left: 0,
    right: 0,
    // position at the bottom
    bottom: 0
};
/**
 * @private
 */
const playFrames = react.memoizeFunction((frameHightPx, frameCount) => react.keyframes({
    from: {
        backgroundPosition: `0px 0px`
    },
    to: {
        backgroundPosition: `0px ${frameCount * -frameHightPx}px`
    }
}));
/**
 * @private
 */
const reactionRenderingStyle = (args) => {
    var _a;
    const imageUrl = `url(${args.spriteImageUrl})`;
    const steps = (_a = args.frameCount) !== null && _a !== void 0 ? _a : 0;
    const frameSizePx = args.rawFrameSize;
    return react.mergeStyles({
        height: `${frameSizePx}px`,
        width: `${frameSizePx}px`,
        overflow: 'hidden',
        animationName: playFrames(frameSizePx, steps),
        backgroundImage: imageUrl,
        animationDuration: `${REACTION_SCREEN_SHARE_ANIMATION_TIME_MS / 1000}s`,
        animationTimingFunction: `steps(${steps})`,
        animationPlayState: 'running',
        animationIterationCount: 'infinite',
        // Scale the emoji to fit the parent container
        transform: `scale(${args.emojiSize / frameSizePx})`,
        transformOrigin: 'top left'
    });
};
/**
 * @private
 */
const loadSpinnerStyles = (theme, isOverLay) => {
    return {
        circle: {
            width: '2rem',
            height: '2rem'
        },
        label: {
            fontSize: '1rem',
            color: isOverLay ? isDarkThemed(theme) ? theme.palette.black : theme.palette.white : theme.palette.themeDarkAlt
        }
    };
};
/**
 * @private
 */
const overlayStyles = () => {
    return {
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        backgroundColor: 'rgba(0, 0, 0, 0.3)'
    };
};
/**
 * @private
 */
const overlayStylesTransparent = () => {
    return {
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center'
    };
};
/**
 * @private
 */
const iconsGroupContainerStyle = {
    marginLeft: '0.2rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
function useLongPress(props) {
    const { onClick, onLongPress, touchEventsOnly = false } = props;
    const timerRef = React.useRef();
    const [isLongPress, setIsLongPress] = React.useState(false);
    const [action, setAction] = React.useState(false);
    React.useEffect(() => {
        if (timerRef.current) {
            clearTimeout(timerRef.current);
        }
        return () => {
            if (timerRef.current) {
                clearTimeout(timerRef.current);
            }
        };
    }, [onClick, onLongPress, touchEventsOnly]);
    const startPressTimer = React.useCallback(() => {
        setIsLongPress(false);
        timerRef.current = setTimeout(() => {
            setIsLongPress(true);
            onLongPress();
        }, 500);
    }, [onLongPress]);
    const handleOnClick = React.useCallback(() => {
        if (touchEventsOnly || !onClick) {
            return;
        }
        if (!isLongPress) {
            onClick();
        }
    }, [isLongPress, onClick, touchEventsOnly]);
    const handleOnKeyDown = React.useCallback(() => {
        if (touchEventsOnly) {
            return;
        }
        if (action) {
            setAction(false);
            startPressTimer();
        }
    }, [action, startPressTimer, touchEventsOnly]);
    const handleOnKeyUp = React.useCallback(() => {
        if (touchEventsOnly) {
            return;
        }
        setAction(true);
        clearTimeout(timerRef.current);
    }, [touchEventsOnly]);
    const handleOnMouseDown = React.useCallback(() => {
        if (touchEventsOnly) {
            return;
        }
        startPressTimer();
    }, [startPressTimer, touchEventsOnly]);
    const handleOnMouseUp = React.useCallback(() => {
        if (touchEventsOnly) {
            return;
        }
        clearTimeout(timerRef.current);
    }, [touchEventsOnly]);
    const handleOnTouchStart = React.useCallback(() => {
        startPressTimer();
    }, [startPressTimer]);
    const handleOnTouchEnd = React.useCallback(() => {
        if (!isLongPress && onClick) {
            onClick();
        }
        clearTimeout(timerRef.current);
    }, [onClick, isLongPress]);
    const handleOnTouchMove = React.useCallback(() => {
        clearTimeout(timerRef.current);
    }, []);
    return React.useMemo(() => ({
        onClick: handleOnClick,
        onMouseDown: handleOnMouseDown,
        onMouseUp: handleOnMouseUp,
        onTouchStart: handleOnTouchStart,
        onTouchEnd: handleOnTouchEnd,
        onKeyDown: handleOnKeyDown,
        onKeyUp: handleOnKeyUp,
        onTouchMove: handleOnTouchMove
    }), [handleOnClick, handleOnKeyDown, handleOnKeyUp, handleOnMouseDown, handleOnMouseUp, handleOnTouchEnd, handleOnTouchStart, handleOnTouchMove]);
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// Coin max size is set to PersonaSize.size100
const DEFAULT_PERSONA_MAX_SIZE_PX = 100;
// Coin min size is set PersonaSize.size32
const DEFAULT_PERSONA_MIN_SIZE_PX = 32;
const DefaultPlaceholder = (props) => {
    const { text, noVideoAvailableAriaLabel, coinSize, hidePersonaDetails } = props;
    return React.createElement(react.Stack, { className: react.mergeStyles({
            position: 'absolute',
            height: '100%',
            width: '100%'
        }) },
        React.createElement(react.Stack, { styles: defaultPersonaStyles$1 }, coinSize && React.createElement(react.Persona, { coinSize: coinSize, hidePersonaDetails: hidePersonaDetails, text: text !== null && text !== void 0 ? text : '', initialsTextColor: "white", "aria-label": noVideoAvailableAriaLabel !== null && noVideoAvailableAriaLabel !== void 0 ? noVideoAvailableAriaLabel : '', showOverflowTooltip: false })));
};
const defaultPersonaStyles$1 = {
    root: {
        margin: 'auto',
        maxHeight: '100%'
    }
};
const videoTileMoreMenuIconProps = {
    iconName: undefined,
    style: {
        display: 'none'
    }
};
const videoTileMoreMenuProps = {
    directionalHint: react.DirectionalHint.topLeftEdge,
    isBeakVisible: false,
    styles: {
        container: {
            maxWidth: '8rem'
        }
    }
};
const VideoTileMoreOptionsButton = (props) => {
    const locale = useLocale$1().strings.videoTile;
    const theme = useTheme();
    const { contextualMenu, canShowContextMenuButton, participantDisplayName, participantHandRaised, participantIsSpeaking, participantState, participantIsMuted, isMicDisabled, isCameraDisabled } = props;
    const [moreButtonAiraDescription, setMoreButtonAriaDescription] = React.useState('');
    React.useEffect(() => {
        setMoreButtonAriaDescription(formatMoreButtonAriaDescription(participantDisplayName, participantIsMuted, participantHandRaised, participantState, participantIsSpeaking, locale, isMicDisabled, isCameraDisabled));
    }, [participantDisplayName, participantHandRaised, participantIsMuted, participantIsSpeaking, participantState, locale, isMicDisabled, isCameraDisabled]);
    if (!contextualMenu) {
        return React.createElement(React.Fragment, null);
    }
    const optionsIcon = canShowContextMenuButton ? 'VideoTileMoreOptions' : undefined;
    return React.createElement(react.IconButton, { "data-ui-id": "video-tile-more-options-button", ariaLabel: moreButtonAiraDescription, styles: moreButtonStyles(theme), menuIconProps: videoTileMoreMenuIconProps, menuProps: Object.assign(Object.assign({}, videoTileMoreMenuProps), contextualMenu), iconProps: {
            iconName: optionsIcon
        } });
};
/**
 * A component to render the video stream for a single call participant.
 *
 * Use with {@link GridLayout} in a {@link VideoGallery}.
 *
 * @public
 */
const VideoTile = (props) => {
    const { children, displayName, initialsName, isMirrored, isMuted, isSpotlighted, isPinned, onRenderPlaceholder, renderElement, overlay: reactionOverlay, showLabel = true, showMuteIndicator = true, styles, userId, noVideoAvailableAriaLabel, isSpeaking, raisedHand, personaMinSize = DEFAULT_PERSONA_MIN_SIZE_PX, personaMaxSize = DEFAULT_PERSONA_MAX_SIZE_PX, contextualMenu, mediaAccess } = props;
    const [isHovered, setIsHovered] = React.useState(false);
    const [isFocused, setIsFocused] = React.useState(false);
    // need to set a default otherwise the resizeObserver will get stuck in an infinite loop.
    const [personaSize, setPersonaSize] = React.useState(1);
    const videoTileRef = React.useRef(null);
    const locale = useLocale$1();
    const theme = useTheme();
    const callingPalette = theme.callingPalette;
    const isVideoRendered = !!renderElement;
    const observer = React.useRef(new ResizeObserver((entries) => {
        if (!entries[0]) {
            return;
        }
        const { width, height } = entries[0].contentRect;
        const personaCalcSize = Math.min(width, height) / 3;
        // we only want to set the persona size if it has changed
        if (personaCalcSize !== personaSize) {
            setPersonaSize(Math.max(Math.min(personaCalcSize, personaMaxSize), personaMinSize));
        }
    }));
    React.useLayoutEffect(() => {
        if (videoTileRef.current) {
            observer.current.observe(videoTileRef.current);
        }
        const currentObserver = observer.current;
        return () => currentObserver.disconnect();
    }, [videoTileRef]);
    // TODO: Remove after calling sdk fix the keybaord focus
    React.useEffect(() => {
        var _a;
        // PPTLive stream id is null
        if ((_a = videoTileRef.current) === null || _a === void 0 ? void 0 : _a.id) {
            return;
        }
        let observer;
        if (videoTileRef.current) {
            observer = new MutationObserver(mutationsList => {
                for (const mutation of mutationsList) {
                    if (mutation.type === 'childList') {
                        const iframe = document.querySelector('iframe');
                        if (iframe) {
                            if (!iframe.getAttribute('tabIndex')) {
                                iframe.setAttribute('tabIndex', '-1');
                            }
                        }
                    }
                }
            });
            observer.observe(videoTileRef.current, {
                childList: true,
                subtree: true
            });
        }
        return () => {
            observer === null || observer === void 0 ? void 0 : observer.disconnect();
        };
    }, [displayName, renderElement]);
    const useLongPressProps = React.useMemo(() => {
        return {
            onLongPress: () => {
                var _a;
                (_a = props.onLongTouch) === null || _a === void 0 ? void 0 : _a.call(props);
            },
            touchEventsOnly: true
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [props.onLongTouch]);
    const longPressHandlers = useLongPress(useLongPressProps);
    const hoverHandlers = React.useMemo(() => {
        return {
            onMouseEnter: () => setIsHovered(true),
            onMouseLeave: () => setIsHovered(false),
            onFocus: () => setIsFocused(true),
            onBlur: () => setIsFocused(false)
        };
    }, []);
    const placeholderOptions = {
        userId,
        text: initialsName !== null && initialsName !== void 0 ? initialsName : displayName,
        noVideoAvailableAriaLabel,
        coinSize: personaSize,
        styles: defaultPersonaStyles$1,
        hidePersonaDetails: true
    };
    const videoHintWithBorderRadius = react.mergeStyles(disabledVideoHint, {
        borderRadius: theme.effects.roundedCorner4,
        backgroundColor: callingPalette.videoTileLabelBackgroundLight
    });
    const tileInfoStyle = React.useMemo(() => react.mergeStyles(isVideoRendered || props.alwaysShowLabelBackground ? videoHintWithBorderRadius : disabledVideoHint, styles === null || styles === void 0 ? void 0 : styles.displayNameContainer), [isVideoRendered, videoHintWithBorderRadius, styles === null || styles === void 0 ? void 0 : styles.displayNameContainer, props.alwaysShowLabelBackground]);
    const ids = useIdentifiers();
    const canShowLabel = showLabel && (displayName || showMuteIndicator && isMuted);
    const participantStateString = getParticipantStateString(props, locale);
    const canShowContextMenuButton = isHovered || isFocused;
    let raisedHandBackgroundColor = '';
    raisedHandBackgroundColor = callingPalette.raiseHandGold;
    const participantMediaAccessIcons = React.useMemo(() => canShowLabel || participantStateString ? getMediaAccessIcons(showMuteIndicator, isMuted, mediaAccess) : undefined, [canShowLabel, isMuted, mediaAccess, participantStateString, showMuteIndicator]);
    const canShowParticipantIcons = participantMediaAccessIcons || isSpotlighted || isPinned;
    return React.createElement(react.Stack, Object.assign({ "data-ui-id": ids.videoTile, className: react.mergeStyles(rootStyles, {
            background: theme.palette.neutralLighter,
            borderRadius: theme.effects.roundedCorner4
        }, isSpeaking || raisedHand ? {
            '&::after': {
                content: `''`,
                position: 'absolute',
                border: `0.25rem solid ${isSpeaking ? theme.palette.themePrimary : raisedHandBackgroundColor}`,
                borderRadius: theme.effects.roundedCorner4,
                width: '100%',
                height: '100%',
                pointerEvents: 'none'
            }
        } : {}, videoTileHighContrastStyles(theme), styles === null || styles === void 0 ? void 0 : styles.root) }, longPressHandlers),
        React.createElement("div", Object.assign({ ref: videoTileRef, style: {
                width: '100%',
                height: '100%'
            } }, hoverHandlers, { "data-is-focusable": true }),
            isVideoRendered ? React.createElement(react.Stack, { className: react.mergeStyles(videoContainerStyles, isMirrored && {
                    transform: 'scaleX(-1)'
                }, styles === null || styles === void 0 ? void 0 : styles.videoContainer) }, renderElement) : React.createElement(react.Stack, { className: react.mergeStyles(videoContainerStyles, {
                    opacity: participantStateString || props.participantState === 'Idle' ? 0.4 : 1
                }) }, onRenderPlaceholder ? onRenderPlaceholder(userId !== null && userId !== void 0 ? userId : '', placeholderOptions, DefaultPlaceholder) : React.createElement(DefaultPlaceholder, Object.assign({}, placeholderOptions))),
            reactionOverlay,
            (canShowLabel || participantStateString) && React.createElement(react.Stack, { horizontal: true, className: tileInfoContainerStyle, tokens: tileInfoContainerTokens },
                React.createElement(react.Stack, { horizontal: true, className: tileInfoStyle },
                    canShowLabel && React.createElement(react.Text, { className: react.mergeStyles(displayNameStyle), title: displayName, style: {
                            color: participantStateString ? theme.palette.neutralSecondary : 'inherit'
                        }, "data-ui-id": "video-tile-display-name" }, displayName),
                    participantStateString && React.createElement(react.Text, { className: react.mergeStyles(participantStateStringStyles(theme)) }, bracketedParticipantString(participantStateString, !!canShowLabel)),
                    canShowParticipantIcons && React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles(iconsGroupContainerStyle) },
                        participantMediaAccessIcons,
                        isSpotlighted && React.createElement(react.Stack, { className: react.mergeStyles(iconContainerStyle) },
                            React.createElement(react.Icon, { iconName: "VideoTileSpotlighted" })),
                        isPinned && React.createElement(react.Stack, { className: react.mergeStyles(iconContainerStyle) },
                            React.createElement(react.Icon, { iconName: "VideoTilePinned", className: react.mergeStyles(pinIconStyle) }))),
                    React.createElement(VideoTileMoreOptionsButton, { contextualMenu: contextualMenu, participantDisplayName: displayName, participantHandRaised: !!raisedHand, participantIsMuted: isMuted, participantState: participantStateString, participantIsSpeaking: isSpeaking, canShowContextMenuButton: canShowContextMenuButton, isMicDisabled: (mediaAccess === null || mediaAccess === void 0 ? void 0 : mediaAccess.isAudioPermitted) === false, isCameraDisabled: (mediaAccess === null || mediaAccess === void 0 ? void 0 : mediaAccess.isVideoPermitted) === false }))),
            children && React.createElement(react.Stack, { className: react.mergeStyles(overlayContainerStyles, styles === null || styles === void 0 ? void 0 : styles.overlayContainer) }, children),
            raisedHand && React.createElement(react.Stack, { horizontal: true, tokens: {
                    childrenGap: '0.2rem'
                }, className: raiseHandContainerStyles(theme, !canShowLabel) },
                React.createElement(react.Stack.Item, null,
                    React.createElement(react.Text, null, raisedHand.raisedHandOrderPosition)),
                React.createElement(react.Stack.Item, null,
                    React.createElement(RaisedHandIcon, null)))));
};
const getMediaAccessIcons = (showMuteIndicator, isMuted, mediaAccess) => {
    const cameraForbidIcon = mediaAccess && !(mediaAccess === null || mediaAccess === void 0 ? void 0 : mediaAccess.isVideoPermitted) ? React.createElement(react.Stack, { className: react.mergeStyles(iconContainerStyle) },
        React.createElement(react.Icon, { iconName: "ControlButtonCameraProhibitedSmall" })) : undefined;
    const micOffIcon = (mediaAccess ? mediaAccess.isAudioPermitted : true) && showMuteIndicator && isMuted ? React.createElement(react.Stack, { className: react.mergeStyles(iconContainerStyle) },
        React.createElement(react.Icon, { iconName: "VideoTileMicOff" })) : undefined;
    const micForbidIcon = mediaAccess && !(mediaAccess === null || mediaAccess === void 0 ? void 0 : mediaAccess.isAudioPermitted) && showMuteIndicator ? React.createElement(react.Stack, { className: react.mergeStyles(iconContainerStyle) },
        React.createElement(react.Icon, { iconName: "ControlButtonMicProhibitedSmall" })) : undefined;
    if (!(cameraForbidIcon || micOffIcon || micForbidIcon)) {
        return undefined;
    }
    return React.createElement(React.Fragment, null,
        cameraForbidIcon,
        micOffIcon,
        micForbidIcon);
};
const getParticipantStateString = (props, locale) => {
    const strings = Object.assign(Object.assign({}, locale.strings.videoTile), props.strings);
    return props.participantState === 'EarlyMedia' || props.participantState === 'Ringing' ? strings === null || strings === void 0 ? void 0 : strings.participantStateRinging : props.participantState === 'Hold' ? strings === null || strings === void 0 ? void 0 : strings.participantStateHold : undefined;
};
const tileInfoContainerTokens = {
    // A horizontal Stack sets the left margin to 0 for all it's children.
    // We need to allow the children to set their own margins
    childrenGap: 'none'
};
const bracketedParticipantString = (participantString, withBrackets) => {
    return withBrackets ? `(${participantString})` : participantString;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Calculates the participants that should be rendered based on the list of dominant
 * speakers and currently rendered participants in a call.
 * @param args - SmartDominantSpeakerParticipantsArgs
 * @returns VideoGalleryRemoteParticipant[] {@link @azure/communication-react#VideoGalleryRemoteParticipant}
 */
const smartDominantSpeakerParticipants = (args) => {
    const { participants, dominantSpeakers = [], currentParticipants = [], maxDominantSpeakers } = args;
    // Don't apply any logic if total number of video streams is less than max dominant speakers.
    if (participants.length <= maxDominantSpeakers) {
        return participants;
    }
    const participantsMap = participantsById(participants);
    // Only use the Max allowed dominant speakers that exist in participants
    const dominantSpeakerIds = dominantSpeakers.filter(id => !!participantsMap[id]).slice(0, maxDominantSpeakers);
    const newVisibleParticipantIds = currentParticipants.map(p => p.userId).slice(0, maxDominantSpeakers);
    const newDominantSpeakerIds = dominantSpeakerIds.filter(id => !newVisibleParticipantIds.includes(id));
    // Remove participants that are no longer dominant and replace them with new dominant speakers.
    for (let index = 0; index < maxDominantSpeakers; index++) {
        const newVisibleParticipantId = newVisibleParticipantIds[index];
        if (newVisibleParticipantId === undefined || !dominantSpeakerIds.includes(newVisibleParticipantId)) {
            const replacement = newDominantSpeakerIds.shift();
            if (!replacement) {
                break;
            }
            newVisibleParticipantIds[index] = replacement;
        }
    }
    let newVisibleParticipants = newVisibleParticipantIds.map(participantId => participantsMap[participantId]).filter(p => p !== undefined);
    const newVisibleParticipantIdsSet = new Set(newVisibleParticipantIds);
    const remainingParticipants = participants.filter(p => !newVisibleParticipantIdsSet.has(p.userId));
    newVisibleParticipants = newVisibleParticipants.concat(remainingParticipants);
    return newVisibleParticipants;
};
/**
 * maps the participants array to an object with userId as key
 * @private
 */
const participantsById = (participants) => {
    const response = {};
    participants.forEach(p => response[p.userId] = p);
    return response;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const DEFAULT_MAX_OVERFLOW_GALLERY_DOMINANT_SPEAKERS = 6;
const DEFAULT_MAX_VIDEO_SREAMS = 4;
/**
 * @private
 */
const MAX_GRID_PARTICIPANTS_NOT_LARGE_GALLERY = 9;
const getOrganizedParticipants = (props) => {
    const { remoteParticipants = [], dominantSpeakers = [], maxGridParticipants = DEFAULT_MAX_VIDEO_SREAMS, maxOverflowGalleryDominantSpeakers = DEFAULT_MAX_OVERFLOW_GALLERY_DOMINANT_SPEAKERS, layout, previousGridParticipants = [], previousOverflowParticipants = [] } = props;
    const callingParticipants = remoteParticipants.filter(p => p.state === 'Connecting' || p.state === 'Ringing');
    const callingParticipantsSet = new Set(callingParticipants.map(p => p.userId));
    const connectedParticipants = remoteParticipants.filter(p => !callingParticipantsSet.has(p.userId));
    const remoteParticipantsOrdered = putVideoParticipantsFirst(connectedParticipants);
    const videoParticipants = remoteParticipants.filter(p => { var _a; return (_a = p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable; });
    const participantsForGrid = layout === 'floatingLocalVideo' && videoParticipants.length > 0 ? videoParticipants : remoteParticipantsOrdered;
    let newGridParticipants = smartDominantSpeakerParticipants({
        participants: participantsForGrid,
        dominantSpeakers,
        currentParticipants: previousGridParticipants,
        maxDominantSpeakers: maxGridParticipants
    }).slice(0, maxGridParticipants);
    if (layout === 'speaker') {
        if (dominantSpeakers === null || dominantSpeakers === void 0 ? void 0 : dominantSpeakers[0]) {
            newGridParticipants = newGridParticipants.filter(p => p.userId === dominantSpeakers[0]);
        }
        else {
            newGridParticipants = newGridParticipants.slice(1);
        }
    }
    const gridParticipantSet = new Set(newGridParticipants.map(p => p.userId));
    const newOverflowGalleryParticipants = smartDominantSpeakerParticipants({
        participants: remoteParticipantsOrdered.filter(p => !gridParticipantSet.has(p.userId)),
        dominantSpeakers: dominantSpeakers,
        currentParticipants: previousOverflowParticipants,
        maxDominantSpeakers: maxOverflowGalleryDominantSpeakers
    });
    let gridParticipants = newGridParticipants;
    let overflowGalleryParticipants = newOverflowGalleryParticipants;
    // Add the participants being called into the call.
    // If there are already overflow participants, add these to the array of overflow participants
    // Otherwise, add the maximum number to the main grid, then put the rest in the overflow
    if (overflowGalleryParticipants.length === 0) {
        const numberOfCallingParticipantsInGrid = maxGridParticipants - gridParticipants.length;
        const gridCallingParticipants = callingParticipants.slice(0, numberOfCallingParticipantsInGrid);
        const overflowGalleryCallingParticipants = callingParticipants.slice(numberOfCallingParticipantsInGrid);
        gridParticipants = gridParticipants.concat(gridCallingParticipants);
        overflowGalleryParticipants = overflowGalleryCallingParticipants;
    }
    else {
        overflowGalleryParticipants = overflowGalleryParticipants.concat(callingParticipants);
    }
    return {
        gridParticipants,
        overflowGalleryParticipants
    };
};
/**
 * Hook to determine which participants should be in grid and overflow gallery and their order respectively
 * @private
 */
const useOrganizedParticipants = (props) => {
    var _a, _b;
    const spotlightedParticipantUserIds = (_a = props.spotlightedParticipantUserIds) !== null && _a !== void 0 ? _a : [];
    const pinnedParticipantUserIds = (_b = props.pinnedParticipantUserIds) !== null && _b !== void 0 ? _b : [];
    // Focussed participants are the participants that are either spotlighted or pinned. Ordered by spotlighted first and then pinned.
    // A set is used to dedupe participants.
    const focusedParticipantUserIdSet = new Set(spotlightedParticipantUserIds.concat(pinnedParticipantUserIds));
    const focusedParticipants = [...focusedParticipantUserIdSet].map(userId => props.remoteParticipants.find(p => p.userId === userId)).filter(p => p !== undefined);
    // Unfocused participants are the rest of the participants
    const unfocusedParticipants = props.remoteParticipants.filter(p => !focusedParticipantUserIdSet.has(p.userId));
    const currentGridParticipants = React.useRef([]);
    const currentOverflowGalleryParticipants = React.useRef([]);
    const organizedParticipantsArgs = Object.assign(Object.assign({}, props), { 
        // if there are focused participants then leave no room in the grid by setting maxGridParticipants to 0
        maxGridParticipants: focusedParticipants.length > 0 || props.isScreenShareActive ? 0 : props.maxGridParticipants, remoteParticipants: unfocusedParticipants, previousGridParticipants: currentGridParticipants.current, previousOverflowParticipants: currentOverflowGalleryParticipants.current });
    const organizedParticipants = getOrganizedParticipants(organizedParticipantsArgs);
    currentGridParticipants.current = organizedParticipants.gridParticipants;
    currentOverflowGalleryParticipants.current = organizedParticipants.overflowGalleryParticipants;
    return focusedParticipants.length > 0 ? {
        gridParticipants: props.isScreenShareActive ? [] : focusedParticipants,
        overflowGalleryParticipants: props.isScreenShareActive ? focusedParticipants.concat(organizedParticipants.overflowGalleryParticipants) : organizedParticipants.overflowGalleryParticipants
    } : organizedParticipants;
};
const putVideoParticipantsFirst = (remoteParticipants) => {
    const videoParticipants = [];
    const audioParticipants = [];
    remoteParticipants.forEach(p => {
        var _a;
        if ((_a = p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable) {
            videoParticipants.push(p);
        }
        else {
            audioParticipants.push(p);
        }
    });
    const remoteParticipantSortedByVideo = videoParticipants.concat(audioParticipants);
    return remoteParticipantSortedByVideo;
};
/**
 * @private
 */
const renderTiles = (gridParticipants, onRenderRemoteParticipant, maxRemoteVideoStreams, indexesToRender, overflowGalleryParticipants, dominantSpeakers) => {
    const _dominantSpeakers = dominantSpeakers !== null && dominantSpeakers !== void 0 ? dominantSpeakers : [];
    let streamsLeftToRender = maxRemoteVideoStreams;
    // Render the grid participants
    const participantWithStreamsToRenderInGrid = gridParticipants.filter(p => { var _a; return (_a = p === null || p === void 0 ? void 0 : p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable; });
    const dominantSpeakerWithStreamsToRenderInGrid = _dominantSpeakers.filter(userId => participantWithStreamsToRenderInGrid.find(p => (p === null || p === void 0 ? void 0 : p.userId) === userId)).slice(0, streamsLeftToRender);
    streamsLeftToRender = streamsLeftToRender - dominantSpeakerWithStreamsToRenderInGrid.length;
    const gridTiles = gridParticipants.map(p => {
        var _a;
        return onRenderRemoteParticipant(p, dominantSpeakerWithStreamsToRenderInGrid.includes(p.userId) || ((_a = p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable) && streamsLeftToRender-- > 0);
    });
    // Render the overflow participants
    const participantWithStreamsToRenderInOverflow = indexesToRender.map(i => {
        return overflowGalleryParticipants.at(i);
    }).filter(p => { var _a; return (_a = p === null || p === void 0 ? void 0 : p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable; });
    const dominantSpeakerWithStreamsToRenderInOverflow = _dominantSpeakers.filter(userId => participantWithStreamsToRenderInOverflow.find(p => (p === null || p === void 0 ? void 0 : p.userId) === userId)).slice(0, streamsLeftToRender);
    streamsLeftToRender = streamsLeftToRender - dominantSpeakerWithStreamsToRenderInOverflow.length;
    const overflowGalleryTiles = overflowGalleryParticipants.map(p => {
        var _a;
        return onRenderRemoteParticipant(p, dominantSpeakerWithStreamsToRenderInOverflow.includes(p.userId) || ((_a = p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable) && streamsLeftToRender-- > 0);
    });
    return {
        gridTiles,
        overflowGalleryTiles
    };
};
/**
 * @private
 */
const getEmojiResource = (reactionName, reactionResources) => {
    var _a, _b, _c, _d, _e;
    switch (reactionName) {
        case 'like':
            return (_a = reactionResources.likeReaction) === null || _a === void 0 ? void 0 : _a.url;
        case 'heart':
            return (_b = reactionResources.heartReaction) === null || _b === void 0 ? void 0 : _b.url;
        case 'laugh':
            return (_c = reactionResources.laughReaction) === null || _c === void 0 ? void 0 : _c.url;
        case 'applause':
            return (_d = reactionResources.applauseReaction) === null || _d === void 0 ? void 0 : _d.url;
        case 'surprised':
            return (_e = reactionResources.surprisedReaction) === null || _e === void 0 ? void 0 : _e.url;
    }
    return undefined;
};
/**
 * @private
 */
const getEmojiFrameCount = (reactionName, reactionResources) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    switch (reactionName) {
        case 'like':
            return (_b = (_a = reactionResources.likeReaction) === null || _a === void 0 ? void 0 : _a.frameCount) !== null && _b !== void 0 ? _b : 0;
        case 'heart':
            return (_d = (_c = reactionResources.heartReaction) === null || _c === void 0 ? void 0 : _c.frameCount) !== null && _d !== void 0 ? _d : 0;
        case 'laugh':
            return (_f = (_e = reactionResources.laughReaction) === null || _e === void 0 ? void 0 : _e.frameCount) !== null && _f !== void 0 ? _f : 0;
        case 'applause':
            return (_h = (_g = reactionResources.applauseReaction) === null || _g === void 0 ? void 0 : _g.frameCount) !== null && _h !== void 0 ? _h : 0;
        case 'surprised':
            return (_k = (_j = reactionResources.surprisedReaction) === null || _j === void 0 ? void 0 : _j.frameCount) !== null && _k !== void 0 ? _k : 0;
        default:
            return 0;
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Reaction overlay component for Grid
 *
 * Can be used with {@link MeetingReactionOverlay}.
 *
 * @internal
 */
const ParticipantVideoTileOverlay = React.memo((props) => {
    const { reaction, reactionResources, emojiSize = REACTION_START_DISPLAY_SIZE } = props;
    const [isValidImageSource, setIsValidImageSource] = React.useState(false);
    const backgroundImageUrl = reaction !== undefined && reactionResources !== undefined ? getEmojiResource(reaction === null || reaction === void 0 ? void 0 : reaction.reactionType, reactionResources) : undefined;
    const frameCount = reaction !== undefined && reactionResources !== undefined ? getEmojiFrameCount(reaction === null || reaction === void 0 ? void 0 : reaction.reactionType, reactionResources) : undefined;
    const currentUnixTimeStamp = Date.now();
    const receivedUnixTimestamp = reaction ? getReceivedUnixTime(reaction.receivedOn) : undefined;
    const canRenderReaction = (receivedUnixTimestamp ? currentUnixTimeStamp - receivedUnixTimestamp < REACTION_SCREEN_SHARE_ANIMATION_TIME_MS : false) && backgroundImageUrl !== undefined;
    React.useEffect(() => {
        if (!backgroundImageUrl || backgroundImageUrl.length === 0) {
            return;
        }
        fetch(`${backgroundImageUrl}`).then(res => {
            setIsValidImageSource(res.ok);
        }).catch(warning => console.warn(`Sprite image for animation rendering failed with warning: ${warning}`));
        return () => setIsValidImageSource(false);
    }, [backgroundImageUrl]);
    const spriteImageUrl = backgroundImageUrl !== null && backgroundImageUrl !== void 0 ? backgroundImageUrl : undefined;
    const reactionContainerStyles = React.useCallback(() => reactionRenderingStyle({
        spriteImageUrl: spriteImageUrl !== null && spriteImageUrl !== void 0 ? spriteImageUrl : '',
        emojiSize: emojiSize,
        frameCount: frameCount !== null && frameCount !== void 0 ? frameCount : REACTION_NUMBER_OF_ANIMATION_FRAMES,
        rawFrameSize: REACTION_DEFAULT_RESOURCE_FRAME_SIZE_PX
    }), [spriteImageUrl, emojiSize, frameCount]);
    return React.createElement(react.Stack, { className: react.mergeStyles(videoContainerStyles, {
            display: 'flex',
            justifyContent: 'center',
            alignItems: 'center',
            backgroundColor: canRenderReaction ? 'rgba(0, 0, 0, 0.5)' : 'transparent',
            borderRadius: '0.25rem'
        }) },
        React.createElement("div", { style: {
                height: '33.33%'
            } }),
        canRenderReaction && isValidImageSource && React.createElement("div", { style: {
                minHeight: `${emojiSize}px`,
                height: `${emojiSize}px`,
                width: `${emojiSize}px`
            } },
            React.createElement("div", { className: reactionContainerStyles() })));
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Calculate the start position for a new reaction in the prescriptive wave pattern
 * @private
 */
function generateStartPositionWave(index, halfCanvasWidth, isOriginAtCanvasCenter = true) {
    const midPointCoordinate = isOriginAtCanvasCenter ? halfCanvasWidth : 0;
    // If the # of reactions on the screen is 0 or a multiple of 25, then we set direction to 0.
    // Otherwise, every other reaction goes right, left in alternating directions.
    // To get alternating sequence, we take n % 2 (which gives 0 or 1), multiple result by 2
    // and subtract 1, which will result in -1 or 1
    const direction = index === 0 ? 0 : index % 2 * 2 - 1;
    if (direction === 0) {
        return midPointCoordinate;
    }
    // Now we get how far the reaction starts from center
    const adjustment = scaleStartPos(index);
    return midPointCoordinate + direction * adjustment * halfCanvasWidth;
}
/**
 * @private
 */
const reactionOverlayStyle = {
    bottom: '0',
    height: '50%',
    pointerEvents: 'none',
    position: 'absolute',
    width: '100%'
};
/**
 * @private
 */
function getReactionMovementStyle(reactionXPoint) {
    return {
        position: 'absolute',
        left: `${reactionXPoint}px`
    };
}
/**
 * Scale metric to determine the start position of a reaction in presentation mode to avoid overlap.
 * @private
 */
function scaleStartPos(index) {
    switch (index) {
        case 1:
        case 2:
            return 0.3;
        case 3:
        case 4:
            return 0.6;
        case 5:
        case 6:
            return 0.9;
        case 7:
        case 8:
            return 0.75;
        case 9:
        case 18:
            return 0.375;
        case 10:
        case 23:
            return 0.525;
        case 11:
        case 16:
            return 0.15;
        case 12:
        case 15:
            return 0.225;
        case 13:
        case 14:
            return 0.075;
        case 17:
        case 24:
            return 0.45;
        case 19:
        case 20:
            return 0.675;
        case 21:
        case 22:
            return 0.825;
        default:
            return 0;
    }
}
/**
 * We have only one bucket item for presentation style of the reaction animation.
 * We are choosing to keep the array so that, in future, with styles needed to get updated, one
 * can add new styles and apply from here, rather than updating over the same style. Later we can remove
 * the old ones.
 * It is for the ease of testing and implementation.
 * @private
 */
const ReactionStyleBucket = {
    sizeScale: 0.9,
    heightMaxScale: 0.7 * 0.95,
    opacityMax: 0.9
};
/**
 * Return a style bucket based on the number of active sprites.
 * For example, the first three reactions should appear at maximum
 * height, width, and opacity.
 * @private
 */
function getReactionStyleBucket() {
    // Having dynamic emoji size on rendering animation impacts performance of the animation itself.
    // So we are choosing to use a fixed size for all cases.
    return ReactionStyleBucket;
}
/**
 * @private
 */
const moveFrames = react.memoizeFunction((maxHeight, travelHeight) => react.keyframes({
    '0%': {
        transform: `translateY(${maxHeight}px)`
    },
    '100%': {
        transform: `translateY(${travelHeight}px)`
    }
}));
/**
 * @private
 */
const moveAnimationStyles = (maxHeight, travelHeight) => {
    return {
        animationName: moveFrames(maxHeight, travelHeight),
        animationFillMode: 'forwards',
        animationDuration: `4.133s`,
        animationTimingFunction: 'cubic-bezier(0, 0.83, 0.19, 1.09)'
    };
};
/**
 * @private
 */
const opacityTransition = react.memoizeFunction(maxOpacity => react.keyframes({
    '0%': {
        opacity: 0
    },
    '31.2%': {
        opacity: maxOpacity
    },
    '67.2%': {
        opacity: maxOpacity
    },
    '100%': {
        opacity: 0,
        display: 'none',
        visibility: 'hidden'
    }
}));
/**
 * @private
 */
const opacityAnimationStyles = (maxOpacity) => {
    return {
        animationName: opacityTransition(maxOpacity),
        animationFillMode: 'forwards',
        animationDuration: `4.133s`
    };
};
/**
 * @private
 */
const spriteFrames = react.memoizeFunction((numOfFrames, displaySizePx) => react.keyframes({
    from: {
        backgroundPosition: '0px 0px'
    },
    to: {
        backgroundPosition: `0px -${numOfFrames * displaySizePx}px`
    }
}));
/**
 * @private
 */
const spriteAnimationStyles = (numOfFrames, displaySizePx, imageUrl) => {
    return {
        height: `${displaySizePx}px`,
        width: `${displaySizePx}px`,
        backgroundImage: `url(${imageUrl})`,
        backgroundRepeat: 'no-repeat',
        animationName: spriteFrames(numOfFrames, displaySizePx),
        animationDuration: `${numOfFrames / 24}s`,
        animationFillMode: `forwards`,
        animationIterationCount: 'infinite',
        animationTimingFunction: `steps(${numOfFrames})`,
        backgroundSize: `${displaySizePx}px ${numOfFrames * displaySizePx}px`,
        transform: `scale(${displaySizePx}/128)`
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const MAX_NUMBER_OF_EMOJIS = 50;
const NUMBER_OF_EMOJI_TYPES = 5;
const REACTION_POSITION_ARRAY_SIZE = 55;
/**
 * The overlay responsible for rendering multiple reactions all at once in presentation mode
 * @internal
 */
const RemoteContentShareReactionOverlay = React.memo((props) => {
    const { reactionResources, localParticipant, remoteParticipants, hostDivHeight, hostDivWidth } = props;
    // Reactions that are currently being animated
    const [visibleReactions, setVisibleReactions] = React.useState([]);
    // Dictionary of userId to a reaction status. This is used to track the latest received reaction
    // per user to avoid animating the same reaction multiple times and to limit the number of
    // active reactions of a certain type.
    const latestReceivedReaction = React.useRef({});
    // Track the number of active reactions of each type to limit the number of active reactions
    // of a certain type.
    const activeTypeCount = React.useRef({
        like: 0,
        heart: 0,
        laugh: 0,
        applause: 0,
        surprised: 0
    });
    // Used to track the total number of reactions ever played. This is a helper variable
    // to calculate the reaction movement index (i.e. the .left position of the reaction)
    const visibleReactionPosition = React.useRef(new Array(REACTION_POSITION_ARRAY_SIZE).fill(false));
    const remoteParticipantReactions = React.useMemo(() => { var _a; return (_a = remoteParticipants === null || remoteParticipants === void 0 ? void 0 : remoteParticipants.map(remoteParticipant => remoteParticipant.reaction).filter((reaction) => !!reaction)) !== null && _a !== void 0 ? _a : []; }, [remoteParticipants]);
    const findFirstEmptyPosition = () => {
        return visibleReactionPosition.current.findIndex(item => item === false);
    };
    const updateVisibleReactions = React.useCallback((reaction, userId) => {
        var _a, _b;
        const combinedKey = getCombinedKey(userId, reaction.reactionType, reaction.receivedOn);
        const alreadyHandled = ((_a = latestReceivedReaction.current[userId]) === null || _a === void 0 ? void 0 : _a.id) === combinedKey;
        if (alreadyHandled) {
            return;
        }
        const activeCount = (_b = activeTypeCount.current[reaction.reactionType]) !== null && _b !== void 0 ? _b : 0;
        if (activeCount >= MAX_NUMBER_OF_EMOJIS / NUMBER_OF_EMOJI_TYPES) {
            latestReceivedReaction.current[userId] = {
                id: combinedKey,
                status: 'ignored'
            };
            return;
        }
        activeTypeCount.current[reaction.reactionType] += 1;
        latestReceivedReaction.current[userId] = {
            id: combinedKey,
            status: 'animating'
        };
        const reactionMovementIndex = findFirstEmptyPosition();
        visibleReactionPosition.current[reactionMovementIndex] = true;
        setVisibleReactions([...visibleReactions, {
                reaction: reaction,
                id: combinedKey,
                reactionMovementIndex: reactionMovementIndex,
                styleBucket: getReactionStyleBucket()
            }]);
        return;
    }, [activeTypeCount, visibleReactions]);
    const removeVisibleReaction = (reactionType, id, index) => {
        setVisibleReactions(visibleReactions.filter(reaction => reaction.id !== id));
        visibleReactionPosition.current[index] = false;
        activeTypeCount.current[reactionType] -= 1;
        Object.entries(latestReceivedReaction.current).forEach(([userId, reaction]) => {
            const userLastReaction = latestReceivedReaction.current[userId];
            if (reaction.id === id && userLastReaction) {
                userLastReaction.status = 'completedAnimating';
            }
        });
    };
    // Update visible reactions when local participant sends a reaction
    React.useEffect(() => {
        if (localParticipant === null || localParticipant === void 0 ? void 0 : localParticipant.reaction) {
            updateVisibleReactions(localParticipant.reaction, localParticipant.userId);
        }
    }, [localParticipant, updateVisibleReactions]);
    // Update visible reactions when remote participants send a reaction
    React.useEffect(() => {
        remoteParticipants === null || remoteParticipants === void 0 ? void 0 : remoteParticipants.map(participant => {
            if (participant === null || participant === void 0 ? void 0 : participant.reaction) {
                updateVisibleReactions(participant.reaction, participant.userId);
            }
        });
    }, [remoteParticipantReactions, remoteParticipants, updateVisibleReactions]);
    // Note: canRenderReaction shouldn't be needed as we remove the animation on the onAnimationEnd event
    const canRenderReaction = (reaction, id, movementIndex) => {
        // compare current time to reaction.received at and see if more than 4 seconds has elapsed
        const canRender = Date.now() - getReceivedUnixTime(reaction.receivedOn) < REACTION_SCREEN_SHARE_ANIMATION_TIME_MS;
        // Clean up the reaction if it's not in the visible reaction list
        if (!canRender) {
            removeVisibleReaction(reaction === null || reaction === void 0 ? void 0 : reaction.reactionType, id, movementIndex);
        }
        return canRender;
    };
    const styleBucket = () => getReactionStyleBucket();
    const displaySizePx = () => REACTION_START_DISPLAY_SIZE * styleBucket().sizeScale;
    const containerHeight = hostDivHeight !== null && hostDivHeight !== void 0 ? hostDivHeight : 0;
    const containerWidth = (hostDivWidth !== null && hostDivWidth !== void 0 ? hostDivWidth : 0) - displaySizePx();
    const leftPosition = (position) => generateStartPositionWave(position, containerWidth / 2, true);
    const reactionMovementStyle = (position) => getReactionMovementStyle(leftPosition(position));
    return React.createElement(react.Stack, { className: react.mergeStyles(videoContainerStyles, {
            display: 'flex',
            justifyContent: 'center',
            alignItems: 'center',
            backgroundColor: 'transparent'
        }) }, visibleReactions.map(reaction => {
        var _a;
        return React.createElement("div", { key: reaction.id, style: reactionOverlayStyle },
            React.createElement("div", { className: "reaction-item" }, canRenderReaction(reaction.reaction, reaction.id, reaction.reactionMovementIndex) &&
                // First div - Section that fixes the travel height and applies the movement animation
                // Second div - Keeps track of active sprites and responsible for marking, counting
                //              and removing reactions. Responsible for opacity controls as the sprite emoji animates
                // Third div - Responsible for calculating the point of X axis where the reaction will start animation
                // Fourth div - Play Animation as the other animation applies on the base play animation for the sprite
                React.createElement("div", { style: moveAnimationStyles(containerHeight / 2, 
                    // dividing by two because reactionOverlayStyle height is set to 50%
                    containerHeight / 2 * (1 - reaction.styleBucket.heightMaxScale)) },
                    React.createElement("div", { onAnimationEnd: () => {
                            removeVisibleReaction(reaction.reaction.reactionType, reaction.id, reaction.reactionMovementIndex);
                        }, style: opacityAnimationStyles(reaction.styleBucket.opacityMax) },
                        React.createElement("div", { style: reactionMovementStyle(reaction.reactionMovementIndex) },
                            React.createElement("div", { style: spriteAnimationStyles(REACTION_NUMBER_OF_ANIMATION_FRAMES, displaySizePx(), (_a = getEmojiResource(reaction === null || reaction === void 0 ? void 0 : reaction.reaction.reactionType, reactionResources)) !== null && _a !== void 0 ? _a : '') }))))));
    }));
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Multiplier to convert rem units to pixels.
 */
const REM_TO_PX_MULTIPLIER = 16;
/**
 * The travel height for reactions in Together Mode.
 * The reaction move overlay uses pixel units, so the seat position height, defined in rem, needs to be converted to pixels
 */
const REACTION_TRAVEL_HEIGHT = 0.35 * REM_TO_PX_MULTIPLIER;
/**
 * Defines the maximum travel height for reactions in Together Mode.
 * Ensures the reaction animation does not exceed the center point from the top.
 * Since the reaction move overlay uses pixel units, the seat position height (defined in rem) must be converted to pixels.
 */
const REACTION_MAX_TRAVEL_HEIGHT = 0.5 * REM_TO_PX_MULTIPLIER;
/**
 * The maximum width for displaying the participant's display name.
 */
const MAX_DISPLAY_NAME_WIDTH = 150;
/**
 * Sets the seating position for a participant in Together Mode.
 *
 * @param seatingPosition - The seating position information.
 * @returns The style object for the seating position.
 */
function setParticipantSeatingPosition(seatingPosition) {
    return {
        width: _pxToRem(seatingPosition.width),
        height: _pxToRem(seatingPosition.height),
        left: _pxToRem(seatingPosition.left),
        top: _pxToRem(seatingPosition.top)
    };
}
/**
 * Return a style bucket based on the number of active sprites.
 * For example, the first three reactions should appear at maximum
 * height, width, and opacity.
 * @private
 */
function setTogetherModeSeatPositionStyle(seatingPosition) {
    return {
        seatPosition: setParticipantSeatingPosition(seatingPosition)
    };
}
/**
 * The style for the Together Mode meeting overlay.
 */
const togetherModeMeetingOverlayStyle = {
    width: '100%',
    height: '100%',
    position: 'absolute',
    top: '0',
    left: '0'
};
/**
 * Generates the overlay style for a participant in Together Mode.
 *
 * @param seatingPosition - The seating position information.
 * @returns The style object for the participant overlay.
 */
function getTogetherModeParticipantOverlayStyle(seatingPositionStyle) {
    return Object.assign(Object.assign({}, seatingPositionStyle.seatPosition), { position: 'absolute' });
}
// Function to map a value from one range to another
const mapRange = (value, inMin, inMax, outMin, outMax) => {
    return outMin + (value - inMin) * (outMax - outMin) / (inMax - inMin);
};
/**
 * Calculate the reaction emoji scaled size based on width and height of the participant seat width and height.
 * This is needed when the browser is resized and the participant seat width and height changes.
 *
 * @param width - The width of the element.
 * @param height - The height of the element.
 * @returns The scaled size.
 */
const calculateScaledSize = (width, height) => {
    // Maximum participant seat width and height
    const maxSize = 600;
    // Minimum participant seat width and height
    const minSize = 200;
    // Minimum scaled width and height of the reaction emoji
    const minScaledSize = 35;
    // Maximum scaled width and height of the reaction emoji
    const maxScaledSize = 70;
    // Use width or height to determine scaling factor
    const size = Math.min(width, height);
    // Map the size to the desired range
    return mapRange(size, minSize, maxSize, minScaledSize, maxScaledSize);
};
/**
 * @private
 */
const togetherModeStreamRootStyle = {
    root: {
        width: '100%',
        height: '100%',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        position: 'absolute',
        top: 0,
        left: 0
    }
};
/**
 * @private
 */
const togetherModeIconStyle = () => {
    return {
        width: _pxToRem(20),
        flexShrink: 0
    };
};
/**
 * The style for the container holding the display name, raiseHand, spotlight and mute icons.
 * @private
 */
const togetherModeParticipantStatusContainer = (backgroundColor, borderRadius) => {
    return {
        backgroundColor,
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        gap: _pxToRem(2),
        margin: '0 auto',
        // Centers the container
        padding: `0 ${_pxToRem(5)}`,
        borderRadius,
        width: 'fit-content'
    };
};
/**
 * @private
 */
const togetherModeParticipantDisplayName = (isParticipantHovered, participantSeatingWidth, color) => {
    // expands the display name width when participant is hovered or clicked on else make it 70% of the participant seating width
    const width = isParticipantHovered || participantSeatingWidth * REM_TO_PX_MULTIPLIER > MAX_DISPLAY_NAME_WIDTH ? 'fit-content' : _pxToRem(0.7 * participantSeatingWidth * REM_TO_PX_MULTIPLIER);
    // For smaller displays, the display name is hidden only participant is hovered or clicked on for mobile view
    const showDisplayName = isParticipantHovered || participantSeatingWidth * REM_TO_PX_MULTIPLIER > MAX_DISPLAY_NAME_WIDTH ? 'inline-block' : 'none';
    return {
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
        textAlign: 'center',
        color,
        overflow: isParticipantHovered ? 'visible' : 'hidden',
        width,
        display: showDisplayName,
        fontSize: `${_pxToRem(13)}`,
        lineHeight: `${_pxToRem(20)}`,
        maxWidth: isParticipantHovered ? 'fit-content' : _pxToRem(0.7 * participantSeatingWidth * REM_TO_PX_MULTIPLIER)
    };
};
/**
 * @private
 */
const togetherModeParticipantEmojiSpriteStyle = (emojiSize, emojiScaledSize, participantSeatWidth) => {
    const participantSeatWidthInPixel = parseFloat(participantSeatWidth) * REM_TO_PX_MULTIPLIER;
    const emojiScaledSizeInPercent = 100 - emojiScaledSize / participantSeatWidthInPixel * 100;
    return {
        width: `${emojiSize}`,
        position: 'absolute',
        // Center the emoji sprite within the participant seat
        left: `${emojiScaledSizeInPercent / 2}%`
    };
};
/**
 * The style for the transition of the participant status container in Together Mode.
 * @private
 */
const participantStatusTransitionStyle = {
    position: 'absolute',
    bottom: `${_pxToRem(2)}`,
    width: 'fit-content',
    textAlign: 'center',
    transform: 'translate(-50%)',
    transition: 'width 0.3s ease, transform 0.3s ease',
    left: '50%'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * TogetherModeOverlay component renders an empty JSX element.
 *
 * @returns {JSX.Element} An empty JSX element.
 */
const TogetherModeOverlay = React.memo((props) => {
    const locale = useLocale$1();
    const theme = useTheme();
    const callingPalette = theme.callingPalette;
    const { emojiSize, reactionResources, remoteParticipants, localParticipant, togetherModeSeatPositions } = props;
    const [togetherModeParticipantStatus, setTogetherModeParticipantStatus] = React.useState({});
    const [hoveredParticipantID, setHoveredParticipantID] = React.useState('');
    /*
     * The useMemo hook is used to calculate the participant status for the Together Mode overlay.
     * It updates the togetherModeParticipantStatus state when there's a change in the remoteParticipants, localParticipant,
     * raisedHand, spotlight, isMuted, displayName, or hoveredParticipantID.
     */
    const updatedParticipantStatus = React.useMemo(() => {
        const allParticipants = [...remoteParticipants, localParticipant];
        const participantsWithVideoAvailable = allParticipants.filter(p => { var _a; return ((_a = p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable) && togetherModeSeatPositions[p.userId]; });
        const updatedSignals = {};
        for (const p of participantsWithVideoAvailable) {
            const { userId, reaction, raisedHand, spotlight, isMuted, displayName } = p;
            const seatingPosition = togetherModeSeatPositions[userId];
            if (seatingPosition) {
                updatedSignals[userId] = {
                    id: userId,
                    reaction: reactionResources && reaction,
                    isHandRaised: !!raisedHand,
                    isSpotlighted: !!spotlight,
                    isMuted,
                    displayName: displayName || locale.strings.videoGallery.displayNamePlaceholder,
                    showDisplayName: !!(spotlight || raisedHand || hoveredParticipantID === userId),
                    scaledSize: calculateScaledSize(seatingPosition.width, seatingPosition.height),
                    seatPositionStyle: setTogetherModeSeatPositionStyle(seatingPosition)
                };
            }
        }
        // This is used to remove the participants bounding box from the DOM when they are no longer in the stream
        const participantsNotInTogetherModeStream = Object.keys(togetherModeParticipantStatus).filter(id => !updatedSignals[id]);
        const newSignals = Object.assign(Object.assign({}, togetherModeParticipantStatus), updatedSignals);
        participantsNotInTogetherModeStream.forEach(id => {
            delete newSignals[id];
        });
        const hasSignalingChange = Object.keys(newSignals).some(key => JSON.stringify(newSignals[key]) !== JSON.stringify(togetherModeParticipantStatus[key]));
        const updateTogetherModeParticipantStatusState = hasSignalingChange || Object.keys(newSignals).length !== Object.keys(togetherModeParticipantStatus).length;
        return updateTogetherModeParticipantStatusState ? newSignals : togetherModeParticipantStatus;
    }, [remoteParticipants, localParticipant, togetherModeParticipantStatus, togetherModeSeatPositions, reactionResources, locale.strings.videoGallery.displayNamePlaceholder, hoveredParticipantID]);
    React.useEffect(() => {
        if (hoveredParticipantID && !updatedParticipantStatus[hoveredParticipantID]) {
            setHoveredParticipantID('');
        }
        setTogetherModeParticipantStatus(updatedParticipantStatus);
    }, [hoveredParticipantID, updatedParticipantStatus]);
    return React.createElement("div", { style: {
            position: 'absolute',
            width: '100%',
            height: '100%',
            overflow: 'hidden'
        } }, Object.values(togetherModeParticipantStatus).map(participantStatus => {
        var _a, _b;
        return participantStatus.id && React.createElement("div", { key: participantStatus.id, "data-ui-id": `together-mode-participant-${participantStatus.id}`, style: Object.assign({}, getTogetherModeParticipantOverlayStyle(participantStatus.seatPositionStyle)), onMouseEnter: () => setHoveredParticipantID(participantStatus.id), onMouseLeave: () => setHoveredParticipantID('') },
            React.createElement("div", null,
                participantStatus.showDisplayName && React.createElement("div", { style: Object.assign({}, participantStatusTransitionStyle) },
                    React.createElement("div", { style: Object.assign({}, togetherModeParticipantStatusContainer(callingPalette.videoTileLabelBackgroundLight, theme.effects.roundedCorner4)) },
                        participantStatus.isHandRaised && React.createElement(RaisedHandIcon, null),
                        participantStatus.showDisplayName && React.createElement(react.Text, { style: Object.assign({}, togetherModeParticipantDisplayName(hoveredParticipantID === participantStatus.id, parseFloat(participantStatus.seatPositionStyle.seatPosition.width), participantStatus.displayName ? theme.palette.neutralSecondary : 'inherit')) }, participantStatus.displayName),
                        participantStatus.isMuted && React.createElement(react.Stack, { className: react.mergeStyles(togetherModeIconStyle) },
                            React.createElement(react.Icon, { iconName: "VideoTileMicOff" })),
                        participantStatus.isSpotlighted && React.createElement(react.Stack, { className: react.mergeStyles(togetherModeIconStyle) },
                            React.createElement(react.Icon, { iconName: "VideoTileSpotlighted" })))),
                ((_a = participantStatus.reaction) === null || _a === void 0 ? void 0 : _a.reactionType) &&
                    // First div - Section that fixes the travel height and applies the movement animation
                    // Second div - Responsible for ensuring the sprite emoji is always centered in the participant seat position
                    // Third div - Play Animation as the other animation applies on the base play animation for the sprite
                    React.createElement("div", { style: moveAnimationStyles(parseFloat(participantStatus.seatPositionStyle.seatPosition.height) * REACTION_MAX_TRAVEL_HEIGHT, parseFloat(participantStatus.seatPositionStyle.seatPosition.height) * REACTION_TRAVEL_HEIGHT) },
                        React.createElement("div", { "data-ui-id": `together-mode-participant-reaction-${participantStatus.id}`, style: Object.assign({}, togetherModeParticipantEmojiSpriteStyle(emojiSize, participantStatus.scaledSize || 1, participantStatus.seatPositionStyle.seatPosition.width)) },
                            React.createElement("div", { style: spriteAnimationStyles(REACTION_NUMBER_OF_ANIMATION_FRAMES, participantStatus.scaledSize || 1, (_b = (participantStatus.reaction && getEmojiResource(participantStatus === null || participantStatus === void 0 ? void 0 : participantStatus.reaction.reactionType, reactionResources))) !== null && _b !== void 0 ? _b : '') })))));
    }));
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Emoji max size
 * @internal
 */
const DEFAULT_EMOJI_MAX_SIZE_PX = 70;
/**
 * Emoji min size
 * @internal
 */
const DEFAULT_EMOJI_MIN_SIZE_PX = 32;
/**
 * Emoji resize scale constant
 * @internal
 */
const REACTION_EMOJI_RESIZE_SCALE_CONSTANT = 3;
/**
 * Reaction overlay component
 *
 * Can be used with {@link VideoTile}.
 *
 * @internal
 */
const MeetingReactionOverlay = (props) => {
    const { overlayMode, reaction, reactionResources, localParticipant, remoteParticipants, togetherModeSeatPositions } = props;
    const [emojiSizePx, setEmojiSizePx] = React.useState(0);
    const [divHeight, setDivHeight] = React.useState(0);
    const [divWidth, setDivWidth] = React.useState(0);
    const videoTileRef = React.useRef(null);
    const observer = React.useRef(new ResizeObserver((entries) => {
        var _a;
        const domRect = (_a = entries.at(0)) === null || _a === void 0 ? void 0 : _a.contentRect;
        const width = domRect !== undefined ? domRect.width : 0;
        const height = domRect !== undefined ? domRect.height : 0;
        const reactionEmojiCalcSize = Math.min(width, height) / REACTION_EMOJI_RESIZE_SCALE_CONSTANT;
        // we only want to set the persona size if it has changed
        if (reactionEmojiCalcSize !== emojiSizePx) {
            setEmojiSizePx(Math.max(Math.min(reactionEmojiCalcSize, DEFAULT_EMOJI_MAX_SIZE_PX), DEFAULT_EMOJI_MIN_SIZE_PX));
        }
        if (height !== divHeight) {
            setDivHeight(height);
        }
        if (width !== divWidth) {
            setDivWidth(width);
        }
    }));
    React.useLayoutEffect(() => {
        if (videoTileRef.current) {
            observer.current.observe(videoTileRef.current);
        }
        const currentObserver = observer.current;
        return () => currentObserver.disconnect();
    }, [videoTileRef]);
    if (overlayMode === 'grid-tiles') {
        return React.createElement("div", { ref: videoTileRef, style: {
                width: '100%',
                height: '100%',
                pointerEvents: 'none'
            } },
            React.createElement(ParticipantVideoTileOverlay, { emojiSize: emojiSizePx, reaction: reaction, reactionResources: reactionResources }));
    }
    else if (props.overlayMode === 'screen-share' || props.overlayMode === 'content-share') {
        return React.createElement("div", { ref: videoTileRef, style: {
                width: '100%',
                height: '100%',
                pointerEvents: 'none'
            } },
            React.createElement(RemoteContentShareReactionOverlay, { hostDivHeight: divHeight, hostDivWidth: divWidth, reactionResources: reactionResources, localParticipant: localParticipant, remoteParticipants: remoteParticipants }));
    }
    else if (props.overlayMode === 'together-mode') {
        return React.createElement("div", { style: Object.assign({}, togetherModeMeetingOverlayStyle) },
            React.createElement(TogetherModeOverlay, { emojiSize: emojiSizePx, reactionResources: reactionResources, localParticipant: localParticipant !== null && localParticipant !== void 0 ? localParticipant : {}, remoteParticipants: remoteParticipants !== null && remoteParticipants !== void 0 ? remoteParticipants : [], togetherModeSeatPositions: togetherModeSeatPositions !== null && togetherModeSeatPositions !== void 0 ? togetherModeSeatPositions : {} }));
    }
    else {
        return React.createElement(React.Fragment, null);
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A memoized version of VideoTile for rendering remote participants. React.memo is used for a performance
 * boost by memoizing the same rendered component to avoid rerendering a VideoTile when its position in the
 * array changes causing a rerender in the parent component. https://reactjs.org/docs/react-api.html#reactmemo
 *
 * @internal
 */
const _RemoteVideoTile = React.memo((props) => {
    var _a, _b;
    const { isAvailable, isReceiving = true, 
    // default to true to prevent any breaking change
    isScreenSharingOn, onCreateRemoteStreamView, onDisposeRemoteStreamView, remoteVideoViewOptions, renderElement, userId, onRenderAvatar, showMuteIndicator, remoteParticipant, participantState, menuKind, isPinned, onPinParticipant, onUnpinParticipant, spotlightedParticipantUserIds, isSpotlighted, onStartSpotlight, onStopSpotlight, maxParticipantsToSpotlight, onMuteParticipant, onUpdateScalingMode, disablePinMenuItem, toggleAnnouncerString, strings, reactionResources, streamId, onForbidAudio, onPermitAudio, onForbidVideo, onPermitVideo } = props;
    const remoteVideoStreamProps = React.useMemo(() => ({
        isMirrored: remoteVideoViewOptions === null || remoteVideoViewOptions === void 0 ? void 0 : remoteVideoViewOptions.isMirrored,
        isScreenSharingOn,
        isStreamAvailable: isAvailable,
        isStreamReceiving: isReceiving,
        onCreateRemoteStreamView,
        onDisposeRemoteStreamView,
        remoteParticipantId: userId,
        renderElementExists: !!renderElement,
        scalingMode: remoteVideoViewOptions === null || remoteVideoViewOptions === void 0 ? void 0 : remoteVideoViewOptions.scalingMode,
        streamId,
        isVideoPermitted: remoteParticipant.mediaAccess ? remoteParticipant.mediaAccess.isVideoPermitted : true
    }), [isAvailable, isReceiving, isScreenSharingOn, onCreateRemoteStreamView, onDisposeRemoteStreamView, remoteVideoViewOptions === null || remoteVideoViewOptions === void 0 ? void 0 : remoteVideoViewOptions.isMirrored, remoteVideoViewOptions === null || remoteVideoViewOptions === void 0 ? void 0 : remoteVideoViewOptions.scalingMode, renderElement, userId, streamId, remoteParticipant.mediaAccess]);
    // Handle creating, destroying and updating the video stream as necessary
    const createVideoStreamResult = useRemoteVideoStreamLifecycleMaintainer(remoteVideoStreamProps);
    const contextualMenuProps = useVideoTileContextualMenuProps({
        participant: remoteParticipant,
        view: createVideoStreamResult === null || createVideoStreamResult === void 0 ? void 0 : createVideoStreamResult.view,
        strings: Object.assign({}, props.strings),
        isPinned,
        onPinParticipant,
        onUnpinParticipant,
        onUpdateScalingMode,
        disablePinMenuItem,
        toggleAnnouncerString,
        spotlightedParticipantUserIds,
        isSpotlighted,
        onStartSpotlight,
        onStopSpotlight,
        maxParticipantsToSpotlight,
        onMuteParticipant,
        onForbidAudio,
        onPermitAudio,
        onForbidVideo,
        onPermitVideo
    });
    const videoTileContextualMenuProps = React.useMemo(() => {
        if (menuKind !== 'contextual' || !contextualMenuProps) {
            return {};
        }
        return {
            contextualMenu: contextualMenuProps
        };
    }, [contextualMenuProps, menuKind]);
    const showLoadingIndicator = isAvailable && isReceiving === false && participantState !== 'Disconnected';
    const isReconnecting = participantState === 'Reconnecting';
    const [drawerMenuItemProps, setDrawerMenuItemProps] = React.useState([]);
    const renderVideoStreamElement = React.useMemo(() => {
        // Checking if renderElement is well defined or not as calling SDK has a number of video streams limitation which
        // implies that, after their threshold, all streams have no child (blank video)
        if ((!renderElement || !renderElement.childElementCount) && !isReconnecting) {
            // Returning `undefined` results in the placeholder with avatar being shown
            return undefined;
        }
        return React.createElement(StreamMedia, { videoStreamElement: renderElement !== null && renderElement !== void 0 ? renderElement : null, loadingState: showLoadingIndicator ? 'loading' : isReconnecting ? 'reconnecting' : 'none' });
    }, [isReconnecting, renderElement, showLoadingIndicator]);
    const onKeyDown = React.useCallback((e) => {
        if (e.key === 'Enter') {
            setDrawerMenuItemProps(convertContextualMenuItemsToDrawerMenuItemProps$1(contextualMenuProps, () => setDrawerMenuItemProps([])));
        }
    }, [setDrawerMenuItemProps, contextualMenuProps]);
    const attendeeRoleString = (_a = props.strings) === null || _a === void 0 ? void 0 : _a.attendeeRole;
    const formatDisplayName = (displayName, role) => {
        if (displayName && role) {
            return _formatString(displayName, {
                AttendeeRole: role
            });
        }
        return displayName;
    };
    const displayName = formatDisplayName(remoteParticipant.displayName ? remoteParticipant.displayName : strings.displayNamePlaceholder, attendeeRoleString);
    const formatInitialsName = () => {
        if (remoteParticipant.displayName && attendeeRoleString) {
            return _formatString(remoteParticipant.displayName, {
                AttendeeRole: attendeeRoleString
            });
        }
        return remoteParticipant.displayName;
    };
    const reactionOverlay = reactionResources && React.createElement(MeetingReactionOverlay, { overlayMode: "grid-tiles", reaction: remoteParticipant.reaction, reactionResources: reactionResources });
    return React.createElement(react.Stack, { tabIndex: menuKind === 'drawer' ? 0 : undefined, onKeyDown: menuKind === 'drawer' ? onKeyDown : undefined, style: remoteVideoTileWrapperStyle },
        React.createElement(VideoTile, Object.assign({ key: userId, userId: userId, initialsName: (_b = formatInitialsName()) !== null && _b !== void 0 ? _b : '', renderElement: renderVideoStreamElement, displayName: displayName, onRenderPlaceholder: onRenderAvatar, isMuted: remoteParticipant.isMuted, raisedHand: remoteParticipant.raisedHand, isSpeaking: remoteParticipant.isSpeaking, showMuteIndicator: showMuteIndicator, personaMinSize: props.personaMinSize, showLabel: props.showLabel, alwaysShowLabelBackground: props.alwaysShowLabelBackground, participantState: participantState }, videoTileContextualMenuProps, { isPinned: props.isPinned, onLongTouch: props.onLongTouch ? props.onLongTouch : () => setDrawerMenuItemProps(convertContextualMenuItemsToDrawerMenuItemProps$1(contextualMenuProps, () => setDrawerMenuItemProps([]))), isSpotlighted: isSpotlighted, overlay: reactionOverlay, mediaAccess: remoteParticipant.mediaAccess })),
        drawerMenuItemProps.length > 0 && React.createElement(react.Layer, { hostId: props.drawerMenuHostId },
            React.createElement(react.Stack, { styles: drawerMenuWrapperStyles },
                React.createElement(_DrawerMenu, { onLightDismiss: () => setDrawerMenuItemProps([]), items: drawerMenuItemProps, heading: displayName }))));
});
const convertContextualMenuItemsToDrawerMenuItemProps$1 = (contextualMenuProps, onLightDismiss) => {
    if (!contextualMenuProps) {
        return [];
    }
    return contextualMenuProps.items.map(item => {
        return {
            itemKey: item.key,
            text: item.text,
            iconProps: item.iconProps,
            disabled: item.disabled,
            onItemClick: () => {
                var _a;
                (_a = item.onClick) === null || _a === void 0 ? void 0 : _a.call(item);
                onLightDismiss === null || onLightDismiss === void 0 ? void 0 : onLightDismiss();
            }
        };
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const loadingStyle = react.mergeStyles({
    height: '100%',
    width: '100%'
});
/**
 * @private
 */
const loadingLabelStyles = {
    label: {
        maxWidth: '10rem',
        overflow: 'hidden',
        textOverflow: 'ellipsis'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A memoized version of VideoTile for rendering the remote screen share stream. React.memo is used for a performance
 * boost by memoizing the same rendered component to avoid rerendering this when the parent component rerenders.
 * https://reactjs.org/docs/react-api.html#reactmemo
 */
const RemoteScreenShare = React.memo((props) => {
    const { userId, displayName, isMuted, renderElement, onCreateRemoteStreamView, onDisposeRemoteStreamView, isReceiving, participantVideoScalingMode, reactionResources, localParticipant, remoteParticipants } = props;
    const locale = useLocale$1();
    if (!renderElement) {
        /**
         * TODO: We need to pass in the scaling mode of the screen share participant to this function because when we
         * call this it will recreate both streams (video and screen share) and we need to make sure that the scaling
         * mode is the same as before we started the screen share.
         *
         * We should deprecate the current function and replace it with a
         * createRemoteScreenShareStreamView and createRemoteVideoStreamView.
         */
        onCreateRemoteStreamView && onCreateRemoteStreamView(userId, participantVideoScalingMode);
    }
    React.useEffect(() => {
        return () => {
            // TODO: Isolate disposing behaviors for screenShare and videoStream
            onDisposeRemoteStreamView && onDisposeRemoteStreamView(userId);
        };
    }, [onDisposeRemoteStreamView, userId]);
    const loadingMessage = displayName ? _formatString(locale.strings.videoGallery.screenShareLoadingMessage, {
        participant: displayName
    }) : '';
    return React.createElement(VideoTile, { displayName: displayName, isMuted: isMuted, renderElement: renderElement ? React.createElement(StreamMedia, { videoStreamElement: renderElement, loadingState: isReceiving === false ? 'loading' : 'none' }) : undefined, onRenderPlaceholder: () => React.createElement(LoadingSpinner, { loadingMessage: loadingMessage }), overlay: reactionResources && React.createElement(MeetingReactionOverlay, { reactionResources: reactionResources, localParticipant: localParticipant, remoteParticipants: remoteParticipants, overlayMode: "screen-share" }) });
});
/**
 * LoadingSpinner component for displaying a loading spinner.
 *
 * @param {string} props.loadingMessage - The loading message to display.
 * @returns {JSX.Element} The JSX element representing the loading spinner.
 */
const LoadingSpinner = (props) => {
    return React.createElement(react.Stack, { verticalAlign: "center", className: loadingStyle },
        React.createElement(react.Spinner, { styles: loadingLabelStyles, label: props.loadingMessage, size: react.SpinnerSize.xSmall, "aria-live": 'assertive' }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const screenSharingContainerStyle = react.mergeStyles({
    width: '100%',
    height: '100%'
});
/**
 * @private
 */
const screenSharingNotificationContainerStyle = (theme) => react.mergeStyles({
    backgroundColor: 'inherit',
    padding: '1rem',
    maxWidth: '95%',
    borderRadius: theme.effects.roundedCorner4,
    color: theme.palette.neutralSecondary
});
/**
 * @private
 */
const screenSharingNotificationIconContainerStyle = react.mergeStyles({
    height: '2rem',
    lineHeight: 0
});
/**
 * @private
 */
const screenSharingNotificationIconStyle = (theme) => react.mergeStyles({
    // svg is (20px x 20px) but path is only (16px x 12px), so need to scale at 2.5 to get 40px
    transform: 'scale(2.5)',
    color: theme.palette.neutralTertiary
});
/**
 * @private
 */
const screenSharingNotificationTextStyle = react.mergeStyles({
    fontSize: '1rem',
    // Text component will take body color by default (white in Dark Mode), so forcing it to be parent container color
    color: 'inherit'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A memoized version of local screen share component. React.memo is used for a performance
 * boost by memoizing the same rendered component to avoid rerendering this when the parent component rerenders.
 * https://reactjs.org/docs/react-api.html#reactmemo
 */
const LocalScreenShare = React.memo((props) => {
    const { localParticipant, renderElement, isAvailable, onCreateLocalStreamView, onDisposeLocalScreenShareStreamView, localScreenShareView = 'stream' } = props;
    const locale = useLocale$1();
    const theme = useTheme();
    if (!renderElement && localScreenShareView === 'stream') {
        onCreateLocalStreamView && onCreateLocalStreamView();
    }
    React.useEffect(() => {
        return () => {
            // TODO: Isolate disposing behaviors for screenShare and videoStream
            onDisposeLocalScreenShareStreamView && onDisposeLocalScreenShareStreamView();
        };
    }, [onDisposeLocalScreenShareStreamView]);
    if (!localParticipant || !localParticipant.isScreenSharingOn) {
        return null;
    }
    const localScreenSharingPlaceholder = React.createElement(react.Stack, { horizontalAlign: "center", verticalAlign: "center", className: screenSharingContainerStyle },
        React.createElement(react.Stack, { horizontalAlign: "center", verticalAlign: "center", className: screenSharingNotificationContainerStyle(theme), tokens: {
                childrenGap: '1rem'
            } },
            React.createElement(react.Stack, { horizontal: true, verticalAlign: "center", className: screenSharingNotificationIconContainerStyle },
                React.createElement(react.Icon, { iconName: "ControlButtonScreenShareStart", className: screenSharingNotificationIconStyle(theme) })),
            React.createElement(react.Text, { className: screenSharingNotificationTextStyle, "aria-live": "polite" }, locale.strings.videoGallery.screenIsBeingSharedMessage)));
    const displayName = !(localParticipant === null || localParticipant === void 0 ? void 0 : localParticipant.displayName) ? locale.strings.videoGallery.displayNamePlaceholder : localParticipant === null || localParticipant === void 0 ? void 0 : localParticipant.displayName;
    const loadingMessage = locale.strings.videoGallery.localScreenShareLoadingMessage;
    if (localScreenShareView === 'placeholderMessage') {
        return React.createElement(VideoTile, { displayName: displayName, isMuted: localParticipant === null || localParticipant === void 0 ? void 0 : localParticipant.isMuted, onRenderPlaceholder: () => React.createElement(React.Fragment, null) }, localScreenSharingPlaceholder);
    }
    return React.createElement(VideoTile, { displayName: displayName, isMuted: localParticipant === null || localParticipant === void 0 ? void 0 : localParticipant.isMuted, renderElement: renderElement ? React.createElement(StreamMedia, { videoStreamElement: renderElement, loadingState: isAvailable === false ? 'loading' : 'none' }) : undefined, onRenderPlaceholder: () => React.createElement(LoadingSpinner, { loadingMessage: loadingMessage }), mediaAccess: localParticipant.mediaAccess });
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const videoGalleryOuterDivStyle = react.mergeStyles({
    position: 'relative',
    width: '100%',
    height: '100%'
});
/**
 * @private
 */
const localVideoCameraCycleButtonStyles = (theme, size) => {
    return {
        root: {
            position: 'absolute',
            width: size === 'large' ? _pxToRem(40) : _pxToRem(32),
            height: size === 'large' ? _pxToRem(40) : _pxToRem(32),
            right: '0rem',
            top: '0rem',
            color: '#FFFFFF',
            // only shows up on running video feed to we want to force specific colours.
            zIndex: 2,
            // shows the button directly over the local video feed.
            background: 'rgba(0,0,0,0.4)',
            borderRadius: theme.effects.roundedCorner2
        },
        rootFocused: {
            // styles to remove the unwanted white highlight and blue colour after tapping on button.
            color: '#FFFFFF',
            background: 'rgba(0,0,0,0.4)' // sets opacity of background to be visible on all backdrops in video stream.
        },
        rootDisabled: {
            // In the disabled state, FluentUI sets the position to relative. We need to override this to keep the button correctly positioned.
            position: 'absolute'
        },
        icon: {
            paddingLeft: _pxToRem(3),
            paddingRight: _pxToRem(3),
            margin: 0
        },
        flexContainer: {
            paddingBottom: _pxToRem(8),
            height: 'unset'
        }
    };
};
/**
 * @private
 */
const localVideoTileContainerStyles = {
    root: {
        width: '100%',
        height: '100%'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$I = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * local video tile camera cycle button - for use on mobile screens only.
 * @internal
 */
const LocalVideoCameraCycleButton = (props) => {
    const { cameras, selectedCamera, onSelectCamera, label, ariaDescription, size } = props;
    const theme = react.useTheme();
    const [waitForCamera, setWaitForCamera] = React.useState(false);
    const onChangeCameraClick = React.useCallback((device) => __awaiter$I(void 0, void 0, void 0, function* () {
        // Throttle changing camera to prevent too many callbacks
        if (onSelectCamera) {
            setWaitForCamera(true);
            try {
                yield onSelectCamera(device);
            }
            finally {
                setWaitForCamera(false);
            }
        }
    }), [onSelectCamera]);
    const disabled = !!waitForCamera;
    const cameraLoadingString = useLocale$1().strings.cameraButton.tooltipVideoLoadingContent;
    return React.createElement(react.IconButton, { "data-ui-id": 'local-camera-switcher-button', styles: localVideoCameraCycleButtonStyles(theme, size), disabled: disabled, iconProps: {
            iconName: 'LocalCameraSwitch'
        }, ariaLabel: label, ariaDescription: disabled ? cameraLoadingString : ariaDescription, "aria-live": 'polite', onClick: () => {
            if (cameras && cameras.length > 1 && selectedCamera !== undefined) {
                const index = cameras.findIndex(camera => selectedCamera.id === camera.id);
                const newCamera = cameras[(index + 1) % cameras.length];
                if (!newCamera) {
                    throw new Error('Camera not found');
                }
                onChangeCameraClick(newCamera);
            }
        } });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A memoized version of VideoTile for rendering local participant.
 *
 * @internal
 */
const _LocalVideoTile = React.memo((props) => {
    const { isAvailable, isMuted, onCreateLocalStreamView, onDisposeLocalStreamView, localVideoViewOptions, renderElement, userId, showLabel, alwaysShowLabelBackground, displayName, initialsName, onRenderAvatar, showMuteIndicator, styles, showCameraSwitcherInLocalPreview, localVideoCameraCycleButtonProps, localVideoCameraSwitcherLabel, localVideoSelectedDescription, raisedHand, reaction, isSpotlighted, spotlightedParticipantUserIds, onStartSpotlight, onStopSpotlight, maxParticipantsToSpotlight, menuKind, strings, reactionResources, isScreenSharingOn, mediaAccess } = props;
    const theme = useTheme();
    const localVideoStreamProps = React.useMemo(() => ({
        isMirrored: localVideoViewOptions === null || localVideoViewOptions === void 0 ? void 0 : localVideoViewOptions.isMirrored,
        isStreamAvailable: isAvailable,
        onCreateLocalStreamView,
        onDisposeLocalStreamView,
        renderElementExists: !!renderElement,
        scalingMode: localVideoViewOptions === null || localVideoViewOptions === void 0 ? void 0 : localVideoViewOptions.scalingMode,
        isVideoPermitted: mediaAccess ? mediaAccess.isVideoPermitted : true
    }), [isAvailable, localVideoViewOptions === null || localVideoViewOptions === void 0 ? void 0 : localVideoViewOptions.isMirrored, localVideoViewOptions === null || localVideoViewOptions === void 0 ? void 0 : localVideoViewOptions.scalingMode, onCreateLocalStreamView, onDisposeLocalStreamView, renderElement, mediaAccess]);
    // Handle creating, destroying and updating the video stream as necessary
    useLocalVideoStreamLifecycleMaintainer(localVideoStreamProps);
    const contextualMenuProps = useVideoTileContextualMenuProps({
        participant: {
            userId: userId !== null && userId !== void 0 ? userId : ''
        },
        strings: Object.assign({}, strings),
        spotlightedParticipantUserIds,
        isSpotlighted,
        onStartSpotlight,
        onStopSpotlight,
        maxParticipantsToSpotlight,
        myUserId: userId
    });
    const videoTileContextualMenuProps = React.useMemo(() => {
        if (menuKind !== 'contextual' || !contextualMenuProps) {
            return {};
        }
        return {
            contextualMenu: contextualMenuProps
        };
    }, [contextualMenuProps, menuKind]);
    const videoTileStyles = React.useMemo(() => {
        if (isSpotlighted) {
            return react.concatStyleSets({
                root: {
                    outline: `0.25rem solid ${theme.palette.neutralTertiaryAlt}`,
                    outlineOffset: '-0.25rem'
                }
            }, styles);
        }
        return styles;
    }, [isSpotlighted, theme.palette.neutralTertiaryAlt, styles]);
    const [drawerMenuItemProps, setDrawerMenuItemProps] = React.useState([]);
    const onKeyDown = React.useCallback((e) => {
        if (e.key === 'Enter') {
            setDrawerMenuItemProps(convertContextualMenuItemsToDrawerMenuItemProps(contextualMenuProps, () => setDrawerMenuItemProps([])));
        }
    }, [setDrawerMenuItemProps, contextualMenuProps]);
    const renderVideoStreamElement = React.useMemo(() => {
        // Checking if renderElement is well defined or not as calling SDK has a number of video streams limitation which
        // implies that, after their threshold, all streams have no child (blank video)
        if (!renderElement || !renderElement.childElementCount) {
            // Returning `undefined` results in the placeholder with avatar being shown
            return undefined;
        }
        return React.createElement(React.Fragment, null,
            React.createElement(FloatingLocalCameraCycleButton, { showCameraSwitcherInLocalPreview: showCameraSwitcherInLocalPreview !== null && showCameraSwitcherInLocalPreview !== void 0 ? showCameraSwitcherInLocalPreview : false, localVideoCameraCycleButtonProps: localVideoCameraCycleButtonProps, localVideoCameraSwitcherLabel: localVideoCameraSwitcherLabel, localVideoSelectedDescription: localVideoSelectedDescription }),
            React.createElement(StreamMedia, { videoStreamElement: renderElement, isMirrored: true }),
            props.participantsCount === 1 && !isScreenSharingOn && React.createElement(react.Stack, { className: react.mergeStyles(videoContainerStyles, overlayStyles()) },
                React.createElement(react.Spinner, { label: strings === null || strings === void 0 ? void 0 : strings.waitingScreenText, ariaLive: "assertive", role: "alert", labelPosition: "bottom", styles: loadSpinnerStyles(theme, true) })));
    }, [localVideoCameraCycleButtonProps, localVideoCameraSwitcherLabel, localVideoSelectedDescription, renderElement, showCameraSwitcherInLocalPreview, props.participantsCount, strings === null || strings === void 0 ? void 0 : strings.waitingScreenText, theme, isScreenSharingOn]);
    const videoTileOverlay = React.useMemo(() => {
        const reactionOverlay = reactionResources !== undefined ? React.createElement(MeetingReactionOverlay, { overlayMode: "grid-tiles", reaction: reaction, reactionResources: reactionResources }) : undefined;
        return reactionOverlay;
    }, [reaction, reactionResources]);
    const onRenderAvatarOneParticipant = React.useCallback(() => {
        return React.createElement(react.Stack, { className: react.mergeStyles(videoContainerStyles, overlayStylesTransparent()) },
            React.createElement(react.Spinner, { label: strings === null || strings === void 0 ? void 0 : strings.waitingScreenText, ariaLive: "assertive", labelPosition: "bottom", role: "alert", styles: loadSpinnerStyles(theme, false) }));
    }, [strings === null || strings === void 0 ? void 0 : strings.waitingScreenText, theme]);
    return React.createElement(react.Stack, { "data-ui-id": "local-video-tile", className: react.mergeStyles({
            width: '100%',
            height: '100%'
        }), onKeyDown: menuKind === 'drawer' ? onKeyDown : undefined },
        React.createElement(VideoTile, Object.assign({ key: userId !== null && userId !== void 0 ? userId : 'local-video-tile', userId: userId, renderElement: renderVideoStreamElement, showLabel: showLabel, alwaysShowLabelBackground: alwaysShowLabelBackground, displayName: displayName, initialsName: initialsName, styles: videoTileStyles, onRenderPlaceholder: props.participantsCount === 1 && !isScreenSharingOn ? onRenderAvatarOneParticipant : onRenderAvatar, isMuted: isMuted, showMuteIndicator: showMuteIndicator, personaMinSize: props.personaMinSize, raisedHand: raisedHand, isSpotlighted: isSpotlighted }, videoTileContextualMenuProps, { onLongTouch: () => setDrawerMenuItemProps(convertContextualMenuItemsToDrawerMenuItemProps(contextualMenuProps, () => setDrawerMenuItemProps([]))), overlay: videoTileOverlay, mediaAccess: mediaAccess }), drawerMenuItemProps.length > 0 && React.createElement(react.Layer, { hostId: props.drawerMenuHostId },
            React.createElement(react.Stack, { styles: drawerMenuWrapperStyles },
                React.createElement(_DrawerMenu, { onLightDismiss: () => setDrawerMenuItemProps([]), items: drawerMenuItemProps })))));
});
const FloatingLocalCameraCycleButton = (props) => {
    const { showCameraSwitcherInLocalPreview, localVideoCameraCycleButtonProps, localVideoCameraSwitcherLabel, localVideoSelectedDescription } = props;
    const ariaDescription = (localVideoCameraCycleButtonProps === null || localVideoCameraCycleButtonProps === void 0 ? void 0 : localVideoCameraCycleButtonProps.selectedCamera) && localVideoSelectedDescription && _formatString(localVideoSelectedDescription, {
        cameraName: localVideoCameraCycleButtonProps.selectedCamera.name
    });
    return React.createElement(react.Stack, { horizontalAlign: "end" }, showCameraSwitcherInLocalPreview && (localVideoCameraCycleButtonProps === null || localVideoCameraCycleButtonProps === void 0 ? void 0 : localVideoCameraCycleButtonProps.cameras) !== undefined && (localVideoCameraCycleButtonProps === null || localVideoCameraCycleButtonProps === void 0 ? void 0 : localVideoCameraCycleButtonProps.selectedCamera) !== undefined && (localVideoCameraCycleButtonProps === null || localVideoCameraCycleButtonProps === void 0 ? void 0 : localVideoCameraCycleButtonProps.onSelectCamera) !== undefined && React.createElement(LocalVideoCameraCycleButton, { cameras: localVideoCameraCycleButtonProps.cameras, selectedCamera: localVideoCameraCycleButtonProps.selectedCamera, onSelectCamera: localVideoCameraCycleButtonProps.onSelectCamera, label: localVideoCameraSwitcherLabel, ariaDescription: ariaDescription }));
};
const convertContextualMenuItemsToDrawerMenuItemProps = (contextualMenuProps, onLightDismiss) => {
    if (!contextualMenuProps) {
        return [];
    }
    return contextualMenuProps.items.map(item => {
        return {
            itemKey: item.key,
            text: item.text,
            iconProps: item.iconProps,
            disabled: item.disabled,
            onItemClick: () => {
                var _a;
                (_a = item.onClick) === null || _a === void 0 ? void 0 : _a.call(item);
                onLightDismiss === null || onLightDismiss === void 0 ? void 0 : onLightDismiss();
            }
        };
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const rootLayoutStyle$1 = {
    root: {
        position: 'relative',
        height: '100%',
        width: '100%',
        padding: '0.5rem'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const videoGalleryLayoutGap = {
    childrenGap: '0.5rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Horizontal Gallery button width in rem
 */
const HORIZONTAL_GALLERY_BUTTON_WIDTH = 1.75;
/**
 * @private
 */
const leftRightButtonStyles$1 = (theme) => {
    return {
        background: 'none',
        padding: 0,
        height: 'auto',
        minWidth: `${HORIZONTAL_GALLERY_BUTTON_WIDTH}rem`,
        maxWidth: `${HORIZONTAL_GALLERY_BUTTON_WIDTH}rem`,
        border: `1px solid ${theme.palette.neutralLight}`,
        borderRadius: theme.effects.roundedCorner4
    };
};
/**
 * Horizontal Gallery gap size in rem between tiles and buttons
 */
const HORIZONTAL_GALLERY_GAP = 0.5;
/**
 * @private
 */
const rootStyle$1 = {
    height: '100%',
    width: '100%',
    gap: `${HORIZONTAL_GALLERY_GAP}rem`
};
/**
 * @private
 */
const childrenContainerStyle$1 = {
    height: '100%',
    width: '100%',
    gap: `${HORIZONTAL_GALLERY_GAP}rem`
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Helper function to bucketize a given array of items into buckets of a specified size.
 *
 * @param arr array to bucketize
 * @param bucketSize number of children for each bucket
 * @returns nested array of given children
 *
 * @private
 */
function bucketize(arr, bucketSize) {
    const bucketArray = [];
    if (bucketSize <= 0) {
        return bucketArray;
    }
    for (let i = 0; i < arr.length; i += bucketSize) {
        bucketArray.push(arr.slice(i, i + bucketSize));
    }
    return bucketArray;
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * {@link HorizontalGallery} default children per page
 */
const DEFAULT_CHILDREN_PER_PAGE = 5;
/**
 * Renders a horizontal gallery that parents children horizontally. Handles pagination based on the childrenPerPage prop.
 * @param props - HorizontalGalleryProps {@link @azure/communication-react#HorizontalGalleryProps}
 * @returns
 */
const HorizontalGallery = (props) => {
    var _a;
    const { children, childrenPerPage = DEFAULT_CHILDREN_PER_PAGE, styles, onFetchTilesToRender } = props;
    const ids = useIdentifiers();
    const [page, setPage] = React.useState(0);
    const numberOfChildren = React.Children.count(children);
    const lastPage = Math.ceil(numberOfChildren / childrenPerPage) - 1;
    const indexesArray = React.useMemo(() => {
        return bucketize([...Array(numberOfChildren).keys()], childrenPerPage);
    }, [numberOfChildren, childrenPerPage]);
    React.useEffect(() => {
        var _a;
        if (onFetchTilesToRender && indexesArray) {
            onFetchTilesToRender((_a = indexesArray[page]) !== null && _a !== void 0 ? _a : []);
        }
    }, [indexesArray, onFetchTilesToRender, page]);
    const firstIndexOfCurrentPage = page * childrenPerPage;
    const clippedPage = firstIndexOfCurrentPage < numberOfChildren - 1 ? page : lastPage;
    const childrenOnCurrentPage = React.useMemo(() => {
        const indexes = indexesArray === null || indexesArray === void 0 ? void 0 : indexesArray[clippedPage];
        if (!indexes) {
            return [];
        }
        return indexes.map(index => React.Children.toArray(children)[index]);
    }, [indexesArray, clippedPage, children]);
    const showButtons = numberOfChildren > childrenPerPage;
    const disablePreviousButton = page === 0;
    const disableNextButton = page === lastPage;
    const childrenStyles = React.useMemo(() => {
        return {
            root: styles === null || styles === void 0 ? void 0 : styles.children
        };
    }, [styles === null || styles === void 0 ? void 0 : styles.children]);
    // If children per page is 0 or less return empty element
    if (childrenPerPage <= 0) {
        return React.createElement(React.Fragment, null);
    }
    return React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles(rootStyle$1, (_a = props.styles) === null || _a === void 0 ? void 0 : _a.root) },
        showButtons && React.createElement(HorizontalGalleryNavigationButton, { key: "previous-nav-button", icon: React.createElement(react.Icon, { iconName: "HorizontalGalleryLeftButton" }), styles: styles === null || styles === void 0 ? void 0 : styles.previousButton, onClick: () => setPage(Math.max(0, Math.min(lastPage, page - 1))), disabled: disablePreviousButton, identifier: ids.overflowGalleryLeftNavButton }),
        React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles(childrenContainerStyle$1) }, childrenOnCurrentPage.map((child, i) => {
            return React.createElement(react.Stack.Item, { styles: childrenStyles, key: i, "data-ui-id": ids.horizontalGalleryVideoTile }, child);
        })),
        showButtons && React.createElement(HorizontalGalleryNavigationButton, { key: "next-nav-button", icon: React.createElement(react.Icon, { iconName: "HorizontalGalleryRightButton" }), styles: styles === null || styles === void 0 ? void 0 : styles.nextButton, onClick: () => setPage(Math.min(lastPage, page + 1)), disabled: disableNextButton, identifier: ids.overflowGalleryRightNavButton }));
};
const HorizontalGalleryNavigationButton = (props) => {
    const theme = useTheme();
    return React.createElement(react.DefaultButton, { className: react.mergeStyles(leftRightButtonStyles$1(theme), props.styles), onClick: props.onClick, disabled: props.disabled, "data-ui-id": props.identifier }, props.icon);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
react.mergeStyles({
    position: 'relative',
    width: '100%',
    height: '100%'
});
/**
 * Small floating modal width and height in rem for small screen
 */
const SMALL_FLOATING_MODAL_SIZE_REM = {
    width: 3.625,
    height: 6.5
};
/**
 * Large floating modal width and height in rem for large screen
 * Aspect ratio: 16:9
 */
const LARGE_FLOATING_MODAL_SIZE_REM = {
    width: 13.75,
    height: 7.5
};
/**
 * Vertical gallery floating modal width and height in rem
 * Aspect ratio: 16:9
 */
const SHORT_VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM = {
    width: 9,
    height: 5.063
};
/**
 * Vertical gallery floating modal width and height in rem
 * Aspect ratio: 16:9
 */
const VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM = {
    width: 11,
    height: 6.25
};
/**
 * @private
 * z-index to ensure that the local video tile is above the video gallery.
 */
const LOCAL_VIDEO_TILE_ZINDEX = 2;
/**
 * @private
 */
const localVideoTileContainerStyle = (theme, localVideoTileSizeRem, screenSharePresent, overflowGalleryPosition) => {
    return {
        width: screenSharePresent ? `${localVideoTileSizeRem.width}rem` : '',
        height: screenSharePresent ? `${localVideoTileSizeRem.height}rem` : '',
        minWidth: screenSharePresent ? '' : `${localVideoTileSizeRem.width}rem`,
        minHeight: screenSharePresent ? '' : `${localVideoTileSizeRem.height}rem`,
        position: 'absolute',
        bottom: overflowGalleryPosition !== 'horizontalTop' ? `${dockedlocalVideoTileContainerPaddingRem}rem` : 'unset',
        top: overflowGalleryPosition === 'horizontalTop' ? `${dockedlocalVideoTileContainerPaddingRem}rem` : 'unset',
        borderRadius: theme.effects.roundedCorner4,
        overflow: 'hidden',
        right: `${dockedlocalVideoTileContainerPaddingRem}rem`
    };
};
/**
 * @private
 */
const localVideoTileWithControlsContainerStyle = (theme, localVideoTileSizeRem) => {
    return react.concatStyleSets(localVideoTileContainerStyle(theme, localVideoTileSizeRem), {
        root: {
            boxShadow: theme.effects.elevation8
        }
    });
};
/**
 * @private
 */
const floatingLocalVideoModalStyle = (theme, modalSizeRem) => {
    return react.concatStyleSets({
        main: react.mergeStyles(localVideoTileContainerStyle(theme, modalSizeRem), {
            bottom: `${floatinglocalVideoModalInitialPositionGapRem}rem`,
            right: `${floatinglocalVideoModalInitialPositionGapRem}rem`
        })
    }, {
        main: {
            boxShadow: theme.effects.elevation8,
            ':focus-within': {
                boxShadow: theme.effects.elevation16,
                border: `${_pxToRem(2)} solid ${theme.palette.neutralPrimary}`
            }
        }
    }, {
        // Needed to allow the videoGallery underneath to receive pointer events
        root: {
            pointerEvents: 'none'
        }
    }, localVideoModalStyles);
};
/**
 * Initial position gap of the floating local video modal.
 * ie. if this is 1rem, then floating local video modal would initially be positioned 1rem from
 * the bottom and 1rem from the right.
 * @private
 */
const floatinglocalVideoModalInitialPositionGapRem = 1;
/**
 * Padding of the docked local video tile container.
 * @private
 */
const dockedlocalVideoTileContainerPaddingRem = 0.5;
/**
 * @private
 */
const floatingLocalVideoTileStyle = {
    root: {
        position: 'absolute',
        zIndex: LOCAL_VIDEO_TILE_ZINDEX,
        height: '100%',
        width: '100%'
    }
};
/**
 * Styles for the local video tile modal when it is focused, will cause keyboard move icon to appear over video
 * @private
 */
const localVideoModalStyles = {
    keyboardMoveIconContainer: {
        zIndex: LOCAL_VIDEO_TILE_ZINDEX + 1 // zIndex to set the keyboard movement Icon above the other layers in the video tile.
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const horizontalGalleryContainerStyle = (shouldFloatLocalVideo, isNarrow) => {
    return {
        minHeight: isNarrow ? `${SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem` : `${LARGE_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem`,
        width: shouldFloatLocalVideo ? isNarrow ? `calc(100% - ${SMALL_FLOATING_MODAL_SIZE_REM.width}rem)` : `calc(100% - ${LARGE_FLOATING_MODAL_SIZE_REM.width}rem)` : '100%',
        paddingRight: '0.5rem'
    };
};
/**
 * @private
 */
const horizontalGalleryStyle = (isNarrow) => {
    return {
        children: isNarrow ? SMALL_HORIZONTAL_GALLERY_TILE_STYLE : LARGE_HORIZONTAL_GALLERY_TILE_STYLE
    };
};
/**
 * Small horizontal gallery tile size in rem
 * @private
 */
const SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM = {
    height: 6.5,
    width: 6.5
};
/**
 * Large horizontal gallery tile size in rem
 * @private
 */
const LARGE_HORIZONTAL_GALLERY_TILE_SIZE_REM = {
    height: 7.5,
    minWidth: 7.5,
    maxWidth: 13.43
};
/**
 * @private
 */
const SMALL_HORIZONTAL_GALLERY_TILE_STYLE = {
    minHeight: `${SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem`,
    minWidth: `${SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM.width}rem`,
    maxHeight: `${SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem`,
    maxWidth: `${SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM.width}rem`
};
/**
 * @private
 */
const LARGE_HORIZONTAL_GALLERY_TILE_STYLE = {
    minHeight: `${LARGE_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem`,
    minWidth: `${LARGE_HORIZONTAL_GALLERY_TILE_SIZE_REM.minWidth}rem`,
    maxHeight: `${LARGE_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem`,
    maxWidth: `${LARGE_HORIZONTAL_GALLERY_TILE_SIZE_REM.maxWidth}rem`,
    width: '100%',
    height: '100%'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * VerticalGallery tile size in rem:
 *
 * min - smallest possible size of the tile (90px)
 * max - Largest size we want the vertical tiles (144px)
 *
 * @private
 */
const SHORT_VERTICAL_GALLERY_TILE_SIZE_REM = {
    minHeight: 5.625,
    maxHeight: 9,
    width: 9
};
/**
 * VerticalGallery tile size in rem:
 *
 * min - smallest possible size of the tile (90px)
 * max - Largest size we want the vertical tiles (144px)
 *
 * @private
 */
const VERTICAL_GALLERY_TILE_SIZE_REM = {
    minHeight: 6.75,
    maxHeight: 11,
    width: 11
};
/**
 * Styles for the VerticalGallery's container set in parent.
 *
 * width is being increased by 1rem to account for the gap width desired for the VerticalGallery.
 *
 * @param shouldFloatLocalVideo whether rendered in floating layout or not
 * @returns Style set for VerticalGallery container.
 */
const verticalGalleryContainerStyle = (shouldFloatLocalVideo, isNarrow, isShort) => {
    return isNarrow && isShort ? {
        width: `${SHORT_VERTICAL_GALLERY_TILE_SIZE_REM.width}rem`,
        height: shouldFloatLocalVideo ? `calc(100% - ${SMALL_FLOATING_MODAL_SIZE_REM.height}rem)` : '100%',
        paddingBottom: '0.5rem'
    } : !isNarrow && isShort ? {
        width: `${SHORT_VERTICAL_GALLERY_TILE_SIZE_REM.width}rem`,
        height: shouldFloatLocalVideo ? `calc(100% - ${SHORT_VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM.height}rem)` : '100%',
        paddingBottom: '0.5rem'
    } : {
        width: `${VERTICAL_GALLERY_TILE_SIZE_REM.width}rem`,
        height: shouldFloatLocalVideo ? `calc(100% - ${VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM.height}rem)` : '100%',
        paddingBottom: '0.5rem'
    };
};
/**
 * @private
 */
const SHORT_VERTICAL_GALLERY_TILE_STYLE = {
    minHeight: `${SHORT_VERTICAL_GALLERY_TILE_SIZE_REM.minHeight}rem`,
    minWidth: `${SHORT_VERTICAL_GALLERY_TILE_SIZE_REM.width}rem`,
    maxHeight: `${SHORT_VERTICAL_GALLERY_TILE_SIZE_REM.maxHeight}rem`,
    maxWidth: `${SHORT_VERTICAL_GALLERY_TILE_SIZE_REM.width}rem`,
    width: '100%',
    height: '100%'
};
/**
 * @private
 */
const VERTICAL_GALLERY_TILE_STYLE = {
    minHeight: `${VERTICAL_GALLERY_TILE_SIZE_REM.minHeight}rem`,
    minWidth: `${VERTICAL_GALLERY_TILE_SIZE_REM.width}rem`,
    maxHeight: `${VERTICAL_GALLERY_TILE_SIZE_REM.maxHeight}rem`,
    maxWidth: `${VERTICAL_GALLERY_TILE_SIZE_REM.width}rem`,
    width: '100%',
    height: '100%'
};
/**
 * @private
 */
const verticalGalleryStyle = (isShort) => {
    return isShort ? {
        children: SHORT_VERTICAL_GALLERY_TILE_STYLE
    } : {
        children: VERTICAL_GALLERY_TILE_STYLE
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Helper function to calculate children per page for HorizontalGallery based on width of container, child, buttons, and
 * gaps in between
 *
 * @private
 */
const calculateHorizontalChildrenPerPage = (args) => {
    const { numberOfChildren, containerWidth, buttonWidthRem, gapWidthRem } = args;
    const childMinWidth = _convertRemToPx(LARGE_HORIZONTAL_GALLERY_TILE_SIZE_REM.minWidth);
    const gapWidth = _convertRemToPx(gapWidthRem);
    /** First check how many children can fit in containerWidth.
     *    __________________________________
     *   |                ||                |
     *   |                ||                |
     *   |________________||________________|
     *   <-----------containerWidth--------->
     *  containerWidth = n * childWidth + (n - 1) * gapWidth. Isolate n and take the floor.
     */
    const numberOfChildrenInContainer = Math.floor((containerWidth + gapWidth) / (childMinWidth + gapWidth));
    // If all children fit then return numberOfChildrenInContainer
    if (numberOfChildren <= numberOfChildrenInContainer) {
        return numberOfChildrenInContainer;
    }
    const buttonWidth = _convertRemToPx(buttonWidthRem);
    /** We know we need to paginate. So we need to subtract the buttonWidth twice and gapWidth twice from
     * containerWidth to compute childrenSpace
     *   <-----------containerWidth--------->
     *    __________________________________
     *   | ||             ||             || |
     *   |<||             ||             ||>|
     *   |_||_____________||_____________||_|
     *       <-------childrenSpace------>
     */
    const childrenSpace = containerWidth - 2 * buttonWidth - 2 * gapWidth;
    // Now that we have childrenSpace width we can figure out how many children can fit in childrenSpace.
    // childrenSpace = n * childWidth + (n - 1) * gapWidth. Isolate n and take the floor.
    return Math.max(Math.floor((childrenSpace + gapWidth) / (childMinWidth + gapWidth)), 1);
};
/**
 * Helper function to find the number of children for the VerticalGallery on each page.
 *
 * @private
 */
const calculateVerticalChildrenPerPage = (args) => {
    const { numberOfChildren, containerHeight, gapHeightRem, controlBarHeight, isShort } = args;
    const childMinHeightPx = _convertRemToPx(isShort ? SHORT_VERTICAL_GALLERY_TILE_SIZE_REM.minHeight : VERTICAL_GALLERY_TILE_SIZE_REM.minHeight);
    const gapHeightPx = _convertRemToPx(gapHeightRem);
    const controlBarHeightPx = _convertRemToPx(controlBarHeight);
    /** First check how many children can fit in containerHeight.
     *
     *   _________________
     *   |                |
     *   |                |
     *   |________________|
     *    _________________
     *   |                |
     *   |                |
     *   |________________|
     *
     *      <   n/m   >
     *
     * number of children = container height - (2* gap height + button height) / childMinHeight
     *
     * we want to find the maximum number of children at the smallest size we can fit in the gallery and then resize them
     * to fill in the space as much as possible
     *
     * First we will find the max number of children without any controls we can fit.
     */
    const maxNumberOfChildrenInContainer = Math.floor((containerHeight + gapHeightPx) / (childMinHeightPx + gapHeightPx));
    // if all of the children fit in the container just return the number of children
    if (numberOfChildren <= maxNumberOfChildrenInContainer) {
        return maxNumberOfChildrenInContainer;
    }
    /**
     * For the pagination we know the container height, the height of the button bar and the 2 times the gap
     * height, top tile and bottom tile above control bar. So the child space is calculated as:
     *
     *      space = height - controlbar - (2 * gap)
     */
    const childSpace = containerHeight - controlBarHeightPx - 2 * gapHeightPx;
    /**
     * Now that we have the childrenSpace height we can figure out how many Children can fir in the childrenSpace.
     * childrenSpace = n * childHeightMin + (n - 1) * gapHeight. isolate n and take the floor.
     *
     * We want to always return at least one video tile if there are children present.So we take the max.
     */
    return Math.max(Math.floor((childSpace + gapHeightPx) / (childMinHeightPx + gapHeightPx)), 1);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Wrapped HorizontalGallery that adjusts the number of items per page based on the
 * available width obtained from a ResizeObserver, width per child, gap width, and button width
 */
const ResponsiveHorizontalGallery = (props) => {
    const { gapWidthRem, buttonWidthRem = 0, onFetchTilesToRender, onChildrenPerPageChange } = props;
    const containerRef = React.useRef(null);
    const containerWidth = _useContainerWidth(containerRef);
    const leftPadding = containerRef.current ? parseFloat(getComputedStyle(containerRef.current).paddingLeft) : 0;
    const rightPadding = containerRef.current ? parseFloat(getComputedStyle(containerRef.current).paddingRight) : 0;
    let containerWidthWithoutPadding = Math.max((containerWidth !== null && containerWidth !== void 0 ? containerWidth : 0) - leftPadding - rightPadding, 0);
    if (Number.isNaN(containerWidthWithoutPadding)) {
        containerWidthWithoutPadding = 0;
    }
    const childrenPerPage = calculateHorizontalChildrenPerPage({
        numberOfChildren: React.Children.count(props.children),
        containerWidth: containerWidthWithoutPadding,
        gapWidthRem,
        buttonWidthRem
    });
    onChildrenPerPageChange === null || onChildrenPerPageChange === void 0 ? void 0 : onChildrenPerPageChange(childrenPerPage);
    return React.createElement("div", { "data-ui-id": "responsive-horizontal-gallery", ref: containerRef, className: react.mergeStyles(props.containerStyles) },
        React.createElement(HorizontalGallery, { childrenPerPage: childrenPerPage, styles: props.horizontalGalleryStyles, onFetchTilesToRender: onFetchTilesToRender }, props.children));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Vertical Gallery gap size in rem between tiles and buttons
 *
 * @private
 */
const VERTICAL_GALLERY_GAP = 0.5;
/**
 * @private
 */
const childrenContainerStyle = (pageControlBarHeight) => {
    return {
        width: '100%',
        height: `calc(100% - ${pageControlBarHeight + VERTICAL_GALLERY_GAP}rem)`,
        gap: `${VERTICAL_GALLERY_GAP}rem`
    };
};
/**
 * @private
 */
const rootStyle = {
    height: '100%',
    width: 'inherit',
    // this needs to inherit its parent's width otherwise will collapse if there are no children.
    gap: `${VERTICAL_GALLERY_GAP}rem`,
    position: 'relative'
};
/**
 * @private
 */
const pageNavigationControlBarContainerStyle = {
    height: '2rem',
    width: '100%',
    position: 'absolute',
    bottom: '0'
};
/**
 * @private
 */
const leftRightButtonStyles = (theme) => {
    return {
        background: 'none',
        padding: 0,
        height: 'auto',
        borderRadius: theme.effects.roundedCorner4,
        border: 'none',
        minWidth: '2rem'
    };
};
/**
 * @private
 */
const participantPageCounter = {
    lineHeight: '2rem',
    width: '100%',
    textAlign: 'center'
};
/**
 * @private
 */
const navIconStyles = {
    root: {
        lineHeight: '0'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * VerticalGallery is a overflow gallery for participants in the {@link VideoGallery} component. Stacks
 * participants on the Y-axis of the VideoGallery for better use of horizontal space.
 *
 * @public
 */
const VerticalGallery = (props) => {
    const { children, styles, childrenPerPage, onFetchTilesToRender } = props;
    const [page, setPage] = React.useState(1);
    const [buttonState, setButtonState] = React.useState({
        previous: true,
        next: true
    });
    const ids = useIdentifiers();
    const numberOfChildren = React.Children.count(children);
    const lastPage = Math.ceil(numberOfChildren / childrenPerPage);
    const indexesArray = React.useMemo(() => {
        return bucketize([...Array(numberOfChildren).keys()], childrenPerPage);
    }, [numberOfChildren, childrenPerPage]);
    React.useEffect(() => {
        var _a;
        if (onFetchTilesToRender && indexesArray) {
            onFetchTilesToRender((_a = indexesArray[page - 1]) !== null && _a !== void 0 ? _a : []);
        }
    }, [indexesArray, onFetchTilesToRender, page]);
    const firstIndexOfCurrentPage = (page - 1) * childrenPerPage;
    const clippedPage = firstIndexOfCurrentPage < numberOfChildren - 1 ? page : lastPage;
    const childrenOnCurrentPage = React.useMemo(() => {
        var _a;
        if (indexesArray[0] !== undefined) {
            return (_a = indexesArray[clippedPage - 1]) === null || _a === void 0 ? void 0 : _a.map(index => {
                return React.Children.toArray(children)[index];
            });
        }
        return;
    }, [indexesArray, clippedPage, children]);
    const showButtons = numberOfChildren > childrenPerPage;
    const onPreviousButtonClick = () => {
        setPage(page - 1);
    };
    const onNextButtonClick = () => {
        setPage(page + 1);
    };
    if (page > lastPage && lastPage > 0) {
        setPage(lastPage);
    }
    React.useEffect(() => {
        if (page > 1 && page < lastPage && showButtons) {
            // we are somewhere in between first and last pages.
            setButtonState({
                previous: false,
                next: false
            });
        }
        else if (page === 1 && showButtons) {
            // we are on the first page.
            setButtonState({
                previous: true,
                next: false
            });
        }
        else if (page === lastPage && showButtons) {
            // we are on the last page.
            setButtonState({
                previous: false,
                next: true
            });
        }
    }, [page, numberOfChildren, lastPage, showButtons]);
    const childContainerStyle = React.useMemo(() => {
        return {
            root: childrenContainerStyle(2)
        };
    }, []);
    const childrenStyles = React.useMemo(() => {
        return {
            root: styles === null || styles === void 0 ? void 0 : styles.children
        };
    }, [styles === null || styles === void 0 ? void 0 : styles.children]);
    if (childrenPerPage <= 0) {
        return React.createElement(React.Fragment, null);
    }
    return React.createElement(react.Stack, { className: react.mergeStyles(rootStyle, styles === null || styles === void 0 ? void 0 : styles.root) },
        React.createElement(react.Stack, { styles: childContainerStyle }, childrenOnCurrentPage === null || childrenOnCurrentPage === void 0 ? void 0 : childrenOnCurrentPage.map((child, i) => {
            return React.createElement(react.Stack.Item, { key: i, styles: childrenStyles, "data-ui-id": ids.verticalGalleryVideoTile }, child);
        })),
        showButtons && React.createElement(VerticalGalleryControlBar, { buttonsDisabled: buttonState, onPreviousButtonClick: onPreviousButtonClick, onNextButtonClick: onNextButtonClick, totalPages: lastPage, currentPage: page }));
};
const VerticalGalleryControlBar = (props) => {
    const { onNextButtonClick, onPreviousButtonClick, buttonsDisabled, currentPage, totalPages, styles } = props;
    const theme = useTheme();
    const ids = useIdentifiers();
    const strings = useLocale$1().strings.verticalGallery;
    const pageCounterContainerStyles = React.useMemo(() => {
        return react.mergeStyles(pageNavigationControlBarContainerStyle, styles === null || styles === void 0 ? void 0 : styles.root);
    }, [styles === null || styles === void 0 ? void 0 : styles.root]);
    const previousButtonSyles = React.useMemo(() => {
        return react.mergeStyles(leftRightButtonStyles(theme), styles === null || styles === void 0 ? void 0 : styles.previousButton);
    }, [styles === null || styles === void 0 ? void 0 : styles.previousButton, theme]);
    const pageCounterStyles = React.useMemo(() => {
        return react.mergeStyles(participantPageCounter, styles === null || styles === void 0 ? void 0 : styles.counter);
    }, [styles === null || styles === void 0 ? void 0 : styles.counter]);
    const nextButtonsStyles = React.useMemo(() => {
        return react.mergeStyles(leftRightButtonStyles(theme), styles === null || styles === void 0 ? void 0 : styles.nextButton);
    }, [styles === null || styles === void 0 ? void 0 : styles.nextButton, theme]);
    const controlBarSpacing = {
        childrenGap: '0.5rem'
    };
    const previousButtonAriaLabel = strings.leftNavButtonAriaLabel ? _formatString(strings.leftNavButtonAriaLabel, {
        current: `${currentPage}`,
        total: `${totalPages}`
    }) : undefined;
    const nextButtonAriaLabel = strings.rightNavButtonAriaLabel ? _formatString(strings.rightNavButtonAriaLabel, {
        current: `${currentPage}`,
        total: `${totalPages}`
    }) : undefined;
    return React.createElement(react.Stack, { horizontalAlign: "center", tokens: controlBarSpacing, horizontal: true, className: pageCounterContainerStyles },
        React.createElement(react.DefaultButton, { className: previousButtonSyles, onClick: onPreviousButtonClick, disabled: buttonsDisabled === null || buttonsDisabled === void 0 ? void 0 : buttonsDisabled.previous, ariaLabel: previousButtonAriaLabel, "data-ui-id": ids.overflowGalleryLeftNavButton },
            React.createElement(react.Icon, { iconName: "VerticalGalleryLeftButton", styles: navIconStyles })),
        React.createElement(react.Text, { "data-ui-id": ids.verticalGalleryPageCounter, className: pageCounterStyles }, `${currentPage} / ${totalPages}`),
        React.createElement(react.DefaultButton, { className: nextButtonsStyles, onClick: onNextButtonClick, disabled: buttonsDisabled === null || buttonsDisabled === void 0 ? void 0 : buttonsDisabled.next, ariaLabel: nextButtonAriaLabel, "data-ui-id": ids.overflowGalleryRightNavButton },
            React.createElement(react.Icon, { iconName: "VerticalGalleryRightButton", styles: navIconStyles })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Responsive container for the VerticalGallery Component. Performs calculations for number of children
 * for the VerticalGallery
 * @param props
 *
 * @beta
 */
const ResponsiveVerticalGallery = (props) => {
    var _a;
    const { children, containerStyles, verticalGalleryStyles, gapHeightRem, controlBarHeightRem, isShort, onFetchTilesToRender, onChildrenPerPageChange } = props;
    const containerRef = React.useRef(null);
    const containerHeight = _useContainerHeight(containerRef);
    const topPadding = containerRef.current ? parseFloat(getComputedStyle(containerRef.current).paddingTop) : 0;
    const bottomPadding = containerRef.current ? parseFloat(getComputedStyle(containerRef.current).paddingBottom) : 0;
    let containerHeightWithoutPadding = Math.max((containerHeight !== null && containerHeight !== void 0 ? containerHeight : 0) - topPadding - bottomPadding, 0);
    if (Number.isNaN(containerHeightWithoutPadding)) {
        containerHeightWithoutPadding = 0;
    }
    const childrenPerPage = calculateVerticalChildrenPerPage({
        numberOfChildren: (_a = React.Children.count(children)) !== null && _a !== void 0 ? _a : 0,
        containerHeight: containerHeightWithoutPadding,
        gapHeightRem,
        controlBarHeight: controlBarHeightRem !== null && controlBarHeightRem !== void 0 ? controlBarHeightRem : 2,
        isShort: isShort !== null && isShort !== void 0 ? isShort : false
    });
    onChildrenPerPageChange === null || onChildrenPerPageChange === void 0 ? void 0 : onChildrenPerPageChange(childrenPerPage);
    return React.createElement("div", { "data-ui-id": "responsive-vertical-gallery", ref: containerRef, className: react.mergeStyles(containerStyles) },
        React.createElement(VerticalGallery, { childrenPerPage: childrenPerPage, styles: verticalGalleryStyles, onFetchTilesToRender: onFetchTilesToRender }, children));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const scrollableHorizontalGalleryStyles = {
    root: {
        width: '100%',
        minHeight: `${SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem`,
        paddingRight: '0.5rem',
        '> *': SMALL_HORIZONTAL_GALLERY_TILE_STYLE
    }
};
/**
 * @private
 */
const scrollableHorizontalGalleryContainerStyles = {
    display: 'flex',
    minHeight: `${SMALL_HORIZONTAL_GALLERY_TILE_SIZE_REM.height}rem`,
    overflow: 'scroll',
    '-ms-overflow-style': 'none',
    'scrollbar-width': 'none',
    '::-webkit-scrollbar': {
        display: 'none'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component to display elements horizontally in a scrollable container
 * @private
 */
const ScrollableHorizontalGallery = (props) => {
    const { horizontalGalleryElements, onFetchTilesToRender, containerStyles } = props;
    React.useEffect(() => {
        const indexesArray = [...Array(horizontalGalleryElements === null || horizontalGalleryElements === void 0 ? void 0 : horizontalGalleryElements.length).keys()];
        if (onFetchTilesToRender && indexesArray) {
            onFetchTilesToRender(indexesArray !== null && indexesArray !== void 0 ? indexesArray : []);
        }
    }, [onFetchTilesToRender, horizontalGalleryElements === null || horizontalGalleryElements === void 0 ? void 0 : horizontalGalleryElements.length]);
    const ref = React.useRef();
    const { events: dragabbleEvents } = reactUseDraggableScroll.useDraggable(ref);
    return React.createElement("div", Object.assign({ ref: ref }, dragabbleEvents, { className: react.mergeStyles(scrollableHorizontalGalleryContainerStyles, containerStyles) }),
        React.createElement(react.Stack, { "data-ui-id": "scrollable-horizontal-gallery", horizontal: true, styles: scrollableHorizontalGalleryStyles, tokens: {
                childrenGap: '0.5rem'
            } }, horizontalGalleryElements));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A ResponsiveHorizontalGallery styled for the {@link VideoGallery}
 *
 * @private
 */
const OverflowGallery = (props) => {
    const { shouldFloatLocalVideo = false, onFetchTilesToRender, isNarrow = false, isShort = false, overflowGalleryElements, horizontalGalleryStyles, overflowGalleryPosition = 'horizontalBottom', verticalGalleryStyles, onChildrenPerPageChange, parentWidth } = props;
    const containerStyles = React.useMemo(() => {
        if (overflowGalleryPosition === 'verticalRight') {
            return verticalGalleryContainerStyle(shouldFloatLocalVideo, isNarrow, isShort);
        }
        return horizontalGalleryContainerStyle(shouldFloatLocalVideo, isNarrow);
    }, [shouldFloatLocalVideo, isShort, isNarrow, overflowGalleryPosition]);
    const galleryStyles = React.useMemo(() => {
        if (overflowGalleryPosition === 'verticalRight') {
            return react.concatStyleSets(verticalGalleryStyle(isShort), verticalGalleryStyles);
        }
        return react.concatStyleSets(horizontalGalleryStyle(isNarrow), horizontalGalleryStyles);
    }, [isNarrow, isShort, horizontalGalleryStyles, overflowGalleryPosition, verticalGalleryStyles]);
    const scrollableHorizontalGalleryContainerStyles = React.useMemo(() => {
        if (isNarrow && parentWidth) {
            return {
                width: shouldFloatLocalVideo ? `${_convertPxToRem(parentWidth) - SMALL_FLOATING_MODAL_SIZE_REM.width - 1}rem` : `${_convertPxToRem(parentWidth) - 1}rem`
            };
        }
        return undefined;
    }, [isNarrow, parentWidth, shouldFloatLocalVideo]);
    if (overflowGalleryPosition === 'verticalRight') {
        return React.createElement(ResponsiveVerticalGallery, { key: "responsive-vertical-gallery", containerStyles: containerStyles, verticalGalleryStyles: galleryStyles, controlBarHeightRem: HORIZONTAL_GALLERY_BUTTON_WIDTH, gapHeightRem: HORIZONTAL_GALLERY_GAP, isShort: isShort, onFetchTilesToRender: onFetchTilesToRender, onChildrenPerPageChange: onChildrenPerPageChange }, overflowGalleryElements ? overflowGalleryElements : [React.createElement(React.Fragment, null)]);
    }
    if (isNarrow) {
        // There are no pages for ScrollableHorizontalGallery so we will approximate the first 3 remote
        // participant tiles are visible
        onChildrenPerPageChange === null || onChildrenPerPageChange === void 0 ? void 0 : onChildrenPerPageChange(3);
        return React.createElement(ScrollableHorizontalGallery, { horizontalGalleryElements: overflowGalleryElements ? overflowGalleryElements : [React.createElement(React.Fragment, null)], onFetchTilesToRender: onFetchTilesToRender, key: "scrollable-horizontal-gallery", containerStyles: scrollableHorizontalGalleryContainerStyles });
    }
    return React.createElement(ResponsiveHorizontalGallery, { key: "responsive-horizontal-gallery", containerStyles: containerStyles, onFetchTilesToRender: onFetchTilesToRender, horizontalGalleryStyles: galleryStyles, buttonWidthRem: HORIZONTAL_GALLERY_BUTTON_WIDTH, gapWidthRem: HORIZONTAL_GALLERY_GAP, onChildrenPerPageChange: onChildrenPerPageChange }, overflowGalleryElements ? overflowGalleryElements : [React.createElement(React.Fragment, null)]);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * DefaultLayout displays remote participants, local video component, and screen sharing component in
 * a grid an overflow gallery.
 *
 * @private
 */
const DefaultLayout = (props) => {
    const { remoteParticipants = [], localParticipant, dominantSpeakers, localVideoComponent, screenShareComponent, onRenderRemoteParticipant, styles, maxRemoteVideoStreams, parentWidth, parentHeight, pinnedParticipantUserIds = [], overflowGalleryPosition = 'horizontalBottom', spotlightedParticipantUserIds = [] } = props;
    const isNarrow = parentWidth ? isNarrowWidth(parentWidth) : false;
    const isShort = parentHeight ? isShortHeight(parentHeight) : false;
    // This is for tracking the number of children in the first page of overflow gallery.
    // This number will be used for the maxOverflowGalleryDominantSpeakers when organizing the remote participants.
    // We need to add the local participant to the pinned participant count so we are placing the speakers correctly.
    const childrenPerPage = React.useRef(4);
    const remoteVideosOn = remoteParticipants.filter(p => { var _a; return (_a = p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable; }).length > 0;
    const { gridParticipants, overflowGalleryParticipants } = useOrganizedParticipants({
        remoteParticipants,
        localParticipant,
        dominantSpeakers,
        maxGridParticipants: remoteVideosOn ? maxRemoteVideoStreams : MAX_GRID_PARTICIPANTS_NOT_LARGE_GALLERY,
        isScreenShareActive: !!screenShareComponent,
        maxOverflowGalleryDominantSpeakers: screenShareComponent ? childrenPerPage.current - (pinnedParticipantUserIds.length + 1) % childrenPerPage.current : childrenPerPage.current,
        pinnedParticipantUserIds,
        layout: 'default',
        spotlightedParticipantUserIds
    });
    /**
     * instantiate indexes available to render with indexes available that would be on first page
     *
     * For some components which do not strictly follow the order of the array, we might
     * re-render the initial tiles -> dispose them -> create new tiles, we need to take care of
     * this case when those components are here
     */
    const [indexesToRender, setIndexesToRender] = React.useState([]);
    let { gridTiles, overflowGalleryTiles } = renderTiles(gridParticipants, onRenderRemoteParticipant, maxRemoteVideoStreams, indexesToRender, overflowGalleryParticipants, dominantSpeakers);
    if (localVideoComponent) {
        if (screenShareComponent || spotlightedParticipantUserIds.length > 0) {
            overflowGalleryTiles = [localVideoComponent].concat(overflowGalleryTiles);
        }
        else {
            gridTiles = [localVideoComponent].concat(gridTiles);
        }
    }
    const overflowGallery = React.useMemo(() => {
        if (overflowGalleryTiles.length === 0) {
            return null;
        }
        return React.createElement(OverflowGallery, { isNarrow: isNarrow, isShort: isShort, shouldFloatLocalVideo: false, overflowGalleryElements: overflowGalleryTiles, horizontalGalleryStyles: styles === null || styles === void 0 ? void 0 : styles.horizontalGallery, verticalGalleryStyles: styles === null || styles === void 0 ? void 0 : styles.verticalGallery, overflowGalleryPosition: overflowGalleryPosition, onFetchTilesToRender: setIndexesToRender, onChildrenPerPageChange: (n) => {
                childrenPerPage.current = n;
            }, parentWidth: parentWidth });
    }, [isNarrow, isShort, overflowGalleryTiles, styles === null || styles === void 0 ? void 0 : styles.horizontalGallery, overflowGalleryPosition, setIndexesToRender, styles === null || styles === void 0 ? void 0 : styles.verticalGallery, parentWidth]);
    return React.createElement(react.Stack, { horizontal: overflowGalleryPosition === 'verticalRight', styles: rootLayoutStyle$1, tokens: videoGalleryLayoutGap },
        props.overflowGalleryPosition === 'horizontalTop' ? overflowGallery : React.createElement(React.Fragment, null),
        screenShareComponent ? screenShareComponent : React.createElement(GridLayout, { key: "grid-layout", styles: styles === null || styles === void 0 ? void 0 : styles.gridLayout }, gridTiles),
        overflowGalleryTrampoline$2(overflowGallery, props.overflowGalleryPosition));
};
const overflowGalleryTrampoline$2 = (gallery, galleryPosition) => {
    return galleryPosition !== 'horizontalTop' ? gallery : React.createElement(React.Fragment, null);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/* eslint-disable @typescript-eslint/explicit-function-return-type */
/* eslint-disable @typescript-eslint/no-explicit-any */
const animationDuration = react.AnimationVariables.durationValue2;
const ZERO = {
    x: 0,
    y: 0
};
const DEFAULT_PROPS = {
    isOpen: false,
    isDarkOverlay: true,
    className: '',
    containerClassName: '',
    enableAriaHiddenSiblings: true
};
const getModalClassNames = react.classNamesFunction();
const getMoveDelta = (ev) => {
    let delta = 10;
    if (ev.shiftKey) {
        if (!ev.ctrlKey) {
            delta = 50;
        }
    }
    else if (ev.ctrlKey) {
        delta = 1;
    }
    return delta;
};
const useComponentRef = (props, focusTrapZone) => {
    React__namespace.useImperativeHandle(props.componentRef, () => ({
        focus() {
            if (focusTrapZone.current) {
                focusTrapZone.current.focus();
            }
        }
    }), [focusTrapZone]);
};
const ModalBase = React__namespace.forwardRef((propsWithoutDefaults, ref) => {
    var _a;
    const props = react.getPropsWithDefaults(DEFAULT_PROPS, propsWithoutDefaults);
    const { allowTouchBodyScroll, className, children, containerClassName, scrollableContentClassName, elementToFocusOnDismiss, firstFocusableSelector, forceFocusInsideTrap, ignoreExternalFocusing, isBlocking, isAlert, isClickableOutsideFocusTrap, isDarkOverlay, onDismiss, layerProps, overlay, isOpen, titleAriaId, styles, subtitleAriaId, theme, topOffsetFixed, responsiveMode, onLayerDidMount, isModeless, dragOptions, onDismissed, minDragPosition, maxDragPosition, dataUiId, keyEventElement = window } = props;
    const rootRef = React__namespace.useRef(null);
    const focusTrapZone = React__namespace.useRef(null);
    const focusTrapZoneElm = React__namespace.useRef(null);
    const mergedRef = reactHooks.useMergedRefs(rootRef, ref);
    const modalResponsiveMode = react.useResponsiveMode(mergedRef);
    const focusTrapZoneId = reactHooks.useId('ModalFocusTrapZone');
    const { setTimeout, clearTimeout } = reactHooks.useSetTimeout();
    const [isModalOpen, setIsModalOpen] = React__namespace.useState(isOpen);
    const [isVisible, setIsVisible] = React__namespace.useState(isOpen);
    const [coordinates, setCoordinates] = React__namespace.useState(ZERO);
    const [modalRectangleTop, setModalRectangleTop] = React__namespace.useState();
    const [isModalMenuOpen, { setTrue: setModalMenuOpen, setFalse: setModalMenuClose }] = reactHooks.useBoolean(false);
    const [_, forceUpdate] = React__namespace.useState(0);
    const forceUpdateCallback = React__namespace.useCallback(() => forceUpdate(prev => prev + 1), []);
    const internalState = reactHooks.useConst(() => ({
        onModalCloseTimer: 0,
        allowTouchBodyScroll,
        scrollableContent: null,
        lastSetCoordinates: ZERO,
        events: new react.EventGroup({})
    }));
    const { keepInBounds } = dragOptions || {};
    const isAlertRole = isAlert !== null && isAlert !== void 0 ? isAlert : (isBlocking && !isModeless);
    const layerClassName = layerProps === undefined ? '' : layerProps.className;
    const classNames = getModalClassNames(styles, {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        theme: theme,
        className,
        containerClassName,
        scrollableContentClassName,
        isOpen,
        isVisible,
        hasBeenOpened: internalState.hasBeenOpened,
        modalRectangleTop,
        topOffsetFixed,
        isModeless,
        layerClassName,
        windowInnerHeight: window === null || window === void 0 ? void 0 : window.innerHeight,
        isDefaultDragHandle: dragOptions && !dragOptions.dragHandleSelector
    });
    const mergedLayerProps = Object.assign(Object.assign({ eventBubblingEnabled: false }, layerProps), { onLayerDidMount: layerProps && layerProps.onLayerDidMount ? layerProps.onLayerDidMount : onLayerDidMount, insertFirst: isModeless, className: classNames.layer });
    // Allow the user to scroll within the modal but not on the body
    const allowScrollOnModal = React__namespace.useCallback((elt) => {
        if (elt) {
            if (internalState.allowTouchBodyScroll) {
                react.allowOverscrollOnElement(elt, internalState.events);
            }
            else {
                react.allowScrollOnElement(elt, internalState.events);
            }
        }
        else {
            internalState.events.off(internalState.scrollableContent);
        }
        internalState.scrollableContent = elt;
    }, [internalState]);
    const registerInitialModalPosition = () => {
        const dialogMain = focusTrapZoneElm.current;
        const modalRectangle = dialogMain === null || dialogMain === void 0 ? void 0 : dialogMain.getBoundingClientRect();
        if (modalRectangle) {
            if (topOffsetFixed) {
                setModalRectangleTop(modalRectangle.top);
            }
            if (keepInBounds) {
                // x/y are unavailable in IE, so use the equivalent left/top
                internalState.minPosition = minDragPosition !== null && minDragPosition !== void 0 ? minDragPosition : {
                    x: -modalRectangle.left,
                    y: -modalRectangle.top
                };
                internalState.maxPosition = maxDragPosition !== null && maxDragPosition !== void 0 ? maxDragPosition : {
                    x: modalRectangle.left,
                    y: modalRectangle.top
                };
                // Make sure the initial co-ordinates are within clamp bounds.
                setCoordinates({
                    x: getClampedAxis('x', coordinates.x),
                    y: getClampedAxis('y', coordinates.y)
                });
            }
        }
    };
    /**
     * Clamps an axis to a specified min and max position.
     *
     * @param axis A string that represents the axis (x/y).
     * @param position The position on the axis.
     */
    const getClampedAxis = React__namespace.useCallback((axis, position) => {
        const { minPosition, maxPosition } = internalState;
        if (keepInBounds && minPosition && maxPosition) {
            position = Math.max(minPosition[axis], position);
            position = Math.min(maxPosition[axis], position);
        }
        return position;
    }, [keepInBounds, internalState]);
    const handleModalClose = () => {
        var _a;
        internalState.lastSetCoordinates = ZERO;
        setModalMenuClose();
        internalState.isInKeyboardMoveMode = false;
        setIsModalOpen(false);
        setCoordinates(ZERO);
        (_a = internalState.disposeOnKeyUp) === null || _a === void 0 ? void 0 : _a.call(internalState);
        onDismissed === null || onDismissed === void 0 ? void 0 : onDismissed();
    };
    const handleDragStart = React__namespace.useCallback(() => {
        setModalMenuClose();
        internalState.isInKeyboardMoveMode = false;
    }, [internalState, setModalMenuClose]);
    const handleDrag = React__namespace.useCallback((ev, dragData) => {
        setCoordinates(prevValue => ({
            x: getClampedAxis('x', prevValue.x + dragData.delta.x),
            y: getClampedAxis('y', prevValue.y + dragData.delta.y)
        }));
    }, [getClampedAxis]);
    const handleDragStop = React__namespace.useCallback(() => {
        if (focusTrapZone.current) {
            focusTrapZone.current.focus();
        }
    }, []);
    const handleEnterKeyboardMoveMode = () => {
        // We need a global handleKeyDown event when we are in the move mode so that we can
        // handle the key presses and the components inside the modal do not get the events
        const handleKeyDown = (event) => {
            const ev = event;
            if (ev.altKey && ev.ctrlKey && ev.keyCode === react.KeyCodes.space) {
                // CTRL + ALT + SPACE is handled during keyUp
                ev.preventDefault();
                ev.stopPropagation();
                return;
            }
            const newLocal = ev.altKey || ev.keyCode === react.KeyCodes.escape;
            if (isModalMenuOpen && newLocal) {
                setModalMenuClose();
            }
            if (internalState.isInKeyboardMoveMode && (ev.keyCode === react.KeyCodes.escape || ev.keyCode === react.KeyCodes.enter)) {
                internalState.isInKeyboardMoveMode = false;
                forceUpdateCallback();
                ev.preventDefault();
                ev.stopPropagation();
            }
            if (internalState.isInKeyboardMoveMode) {
                let handledEvent = true;
                const delta = getMoveDelta(ev);
                switch (ev.keyCode) {
                    /* eslint-disable no-fallthrough */
                    case react.KeyCodes.escape:
                        setCoordinates(internalState.lastSetCoordinates);
                    case react.KeyCodes.enter:
                        {
                            // TODO: determine if fallthrough was intentional
                            /* eslint-enable no-fallthrough */
                            internalState.lastSetCoordinates = ZERO;
                            // setIsInKeyboardMoveMode(false);
                            break;
                        }
                    case react.KeyCodes.up:
                        {
                            setCoordinates(prevValue => ({
                                x: prevValue.x,
                                y: getClampedAxis('y', prevValue.y - delta)
                            }));
                            break;
                        }
                    case react.KeyCodes.down:
                        {
                            setCoordinates(prevValue => ({
                                x: prevValue.x,
                                y: getClampedAxis('y', prevValue.y + delta)
                            }));
                            break;
                        }
                    case react.KeyCodes.left:
                        {
                            setCoordinates(prevValue => ({
                                x: getClampedAxis('x', prevValue.x - delta),
                                y: prevValue.y
                            }));
                            break;
                        }
                    case react.KeyCodes.right:
                        {
                            setCoordinates(prevValue => ({
                                x: getClampedAxis('x', prevValue.x + delta),
                                y: prevValue.y
                            }));
                            break;
                        }
                    default:
                        {
                            handledEvent = false;
                        }
                }
                if (handledEvent) {
                    ev.preventDefault();
                    ev.stopPropagation();
                }
            }
        };
        internalState.lastSetCoordinates = coordinates;
        setModalMenuClose();
        internalState.isInKeyboardMoveMode = true;
        keyEventElement.addEventListener('keydown', handleKeyDown, true /* useCapture */);
        internalState.disposeOnKeyDown = () => {
            keyEventElement.removeEventListener('keydown', handleKeyDown, true /* useCapture */);
            internalState.disposeOnKeyDown = undefined;
        };
    };
    const handleExitKeyboardMoveMode = () => {
        var _a;
        internalState.lastSetCoordinates = ZERO;
        internalState.isInKeyboardMoveMode = false;
        (_a = internalState.disposeOnKeyDown) === null || _a === void 0 ? void 0 : _a.call(internalState);
    };
    const registerForKeyUp = () => {
        const handleKeyUp = (event) => {
            const ev = event;
            // Needs to handle the CTRL + ALT + SPACE key during keyup due to FireFox bug:
            // https://bugzilla.mozilla.org/show_bug.cgi?id=1220143
            if (ev.altKey && ev.ctrlKey && ev.keyCode === react.KeyCodes.space) {
                if (react.elementContains(internalState.scrollableContent, ev.target)) {
                    setModalMenuOpen();
                    ev.preventDefault();
                    ev.stopPropagation();
                }
            }
        };
        if (!internalState.disposeOnKeyUp) {
            keyEventElement.addEventListener('keyup', handleKeyUp, true /* useCapture */);
            internalState.disposeOnKeyUp = () => {
                keyEventElement.removeEventListener('keyup', handleKeyUp, true /* useCapture */);
                internalState.disposeOnKeyUp = undefined;
            };
        }
    };
    React__namespace.useEffect(() => {
        clearTimeout(internalState.onModalCloseTimer);
        // Opening the dialog
        if (isOpen) {
            // This must be done after the modal content has rendered
            requestAnimationFrame(() => setTimeout(registerInitialModalPosition, 0));
            setIsModalOpen(true);
            // Add a keyUp handler for all key up events once the dialog is open.
            if (dragOptions) {
                registerForKeyUp();
            }
            internalState.hasBeenOpened = true;
            setIsVisible(true);
        }
        // Closing the dialog
        if (!isOpen && isModalOpen) {
            internalState.onModalCloseTimer = setTimeout(handleModalClose, parseFloat(animationDuration) * 1000);
            setIsVisible(false);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps -- should only run if isModalOpen or isOpen mutates or if min/max drag bounds are updated.
    }, [isModalOpen, isOpen, minDragPosition, maxDragPosition]);
    reactHooks.useUnmount(() => {
        internalState.events.dispose();
    });
    useComponentRef(props, focusTrapZone);
    const modalContent = React__namespace.createElement(react.FocusTrapZone, Object.assign({ disabled: true, id: focusTrapZoneId, ref: focusTrapZoneElm, componentRef: focusTrapZone, className: classNames.main, elementToFocusOnDismiss: elementToFocusOnDismiss, isClickableOutsideFocusTrap: isModeless || isClickableOutsideFocusTrap || !isBlocking, ignoreExternalFocusing: ignoreExternalFocusing, forceFocusInsideTrap: forceFocusInsideTrap && !isModeless, firstFocusableSelector: firstFocusableSelector, focusPreviouslyFocusedInnerElement: true, onBlur: internalState.isInKeyboardMoveMode ? handleExitKeyboardMoveMode : undefined, "data-ui-id": dataUiId }, (_a = props.focusTrapZoneProps) !== null && _a !== void 0 ? _a : {}),
        dragOptions && internalState.isInKeyboardMoveMode && React__namespace.createElement("div", { className: classNames.keyboardMoveIconContainer }, dragOptions.keyboardMoveIconProps ? React__namespace.createElement(react.Icon, Object.assign({}, dragOptions.keyboardMoveIconProps)) : React__namespace.createElement(react.Icon, { iconName: "move", className: classNames.keyboardMoveIcon })),
        React__namespace.createElement("div", { ref: allowScrollOnModal, className: classNames.scrollableContent, "data-is-scrollable": true },
            dragOptions && isModalMenuOpen && React__namespace.createElement(dragOptions.menu, { items: [{
                        key: 'move',
                        text: dragOptions.moveMenuItemText,
                        onClick: handleEnterKeyboardMoveMode
                    }, {
                        key: 'close',
                        text: dragOptions.closeMenuItemText,
                        onClick: handleModalClose
                    }], onDismiss: setModalMenuClose, alignTargetEdge: true, coverTarget: true, directionalHint: react.DirectionalHint.topLeftEdge, directionalHintFixed: true, shouldFocusOnMount: true, target: internalState.scrollableContent }),
            children));
    return (
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    isModalOpen && modalResponsiveMode >= (responsiveMode || react.ResponsiveMode.small) && React__namespace.createElement(react.Layer, Object.assign({ ref: mergedRef }, mergedLayerProps),
        React__namespace.createElement(react.Popup, { role: isAlertRole ? 'alertdialog' : 'dialog', ariaLabelledBy: titleAriaId, ariaDescribedBy: subtitleAriaId, 
            // onDismiss={onDismiss}
            shouldRestoreFocus: !ignoreExternalFocusing, "aria-modal": !isModeless },
            React__namespace.createElement("div", { className: classNames.root, role: !isModeless ? 'document' : undefined },
                !isModeless && React__namespace.createElement(react.Overlay, Object.assign({ "aria-hidden": true, isDarkThemed: isDarkOverlay, onClick: isBlocking ? undefined : onDismiss, allowTouchBodyScroll: allowTouchBodyScroll }, overlay)),
                dragOptions ? React__namespace.createElement(DraggableZone, { handleSelector: dragOptions.dragHandleSelector || `#${focusTrapZoneId}`, preventDragSelector: "button", onStart: handleDragStart, onDragChange: handleDrag, onStop: handleDragStop, position: coordinates }, modalContent) : modalContent))) || null);
});
ModalBase.displayName = 'ModalBase';
const getDraggableZoneClassNames = react.memoizeFunction((className, isDragging) => {
    return {
        root: react.mergeStyles(className, isDragging && {
            touchAction: 'none',
            selectors: {
                '& *': {
                    userSelect: 'none'
                }
            }
        })
    };
});
const eventMapping = {
    touch: {
        start: 'touchstart',
        move: 'touchmove',
        stop: 'touchend'
    },
    mouse: {
        start: 'mousedown',
        move: 'mousemove',
        stop: 'mouseup'
    }
};
class DraggableZone extends React__namespace.Component {
    constructor(props) {
        super(props);
        this._currentEventType = eventMapping.mouse;
        this._events = [];
        this._onMouseDown = (event) => {
            const onMouseDown = React__namespace.Children.only(this.props.children).props.onMouseDown;
            if (onMouseDown) {
                onMouseDown(event);
            }
            this._currentEventType = eventMapping.mouse;
            return this._onDragStart(event);
        };
        this._onMouseUp = (event) => {
            const onMouseUp = React__namespace.Children.only(this.props.children).props.onMouseUp;
            if (onMouseUp) {
                onMouseUp(event);
            }
            this._currentEventType = eventMapping.mouse;
            return this._onDragStop(event);
        };
        this._onTouchStart = (event) => {
            const onTouchStart = React__namespace.Children.only(this.props.children).props.onTouchStart;
            if (onTouchStart) {
                onTouchStart(event);
            }
            this._currentEventType = eventMapping.touch;
            return this._onDragStart(event);
        };
        this._onTouchEnd = (event) => {
            const onTouchEnd = React__namespace.Children.only(this.props.children).props.onTouchEnd;
            if (onTouchEnd) {
                onTouchEnd(event);
            }
            this._currentEventType = eventMapping.touch;
            this._onDragStop(event);
        };
        this._onDragStart = (event) => {
            // Only handle left click for dragging
            if (typeof event.button === 'number' && event.button !== 0) {
                return false;
            }
            // If the target doesn't match the handleSelector OR
            // if the target does match the preventDragSelector, bail out
            if (this.props.handleSelector && !this._matchesSelector(event.target, this.props.handleSelector) || this.props.preventDragSelector && this._matchesSelector(event.target, this.props.preventDragSelector)) {
                return;
            }
            // Remember the touch identifier if this is a touch event so we can
            // distinguish between individual touches in multitouch scenarios
            // by remembering which touch point we were given
            this._touchId = this._getTouchId(event);
            const position = this._getControlPosition(event);
            if (position === undefined) {
                return;
            }
            const dragData = this._createDragDataFromPosition(position);
            this.props.onStart && this.props.onStart(event, dragData);
            this.setState({
                isDragging: true,
                lastPosition: position
            });
            // hook up the appropriate mouse/touch events to the body to ensure
            // smooth dragging
            this._events = [react.on(document.body, this._currentEventType.move, this._onDrag, true /* use capture phase */), react.on(document.body, this._currentEventType.stop, this._onDragStop, true /* use capture phase */)];
            return;
        };
        this._onDrag = (event) => {
            // Prevent scrolling on mobile devices
            if (event.type === 'touchmove') {
                event.preventDefault();
            }
            const position = this._getControlPosition(event);
            if (!position) {
                return;
            }
            // create the updated drag data from the position data
            const updatedData = this._createUpdatedDragData(this._createDragDataFromPosition(position));
            const updatedPosition = updatedData.position;
            this.props.onDragChange && this.props.onDragChange(event, updatedData);
            this.setState({
                position: updatedPosition,
                lastPosition: position
            });
        };
        this._onDragStop = (event) => {
            if (!this.state.isDragging) {
                return;
            }
            const position = this._getControlPosition(event);
            if (!position) {
                return;
            }
            const baseDragData = this._createDragDataFromPosition(position);
            // Set dragging to false and reset the lastPosition
            this.setState({
                isDragging: false,
                lastPosition: undefined
            });
            this.props.onStop && this.props.onStop(event, baseDragData);
            if (this.props.position) {
                this.setState({
                    position: this.props.position
                });
            }
            // Remove event handlers
            this._events.forEach(dispose => dispose());
        };
        this.state = {
            isDragging: false,
            position: this.props.position || {
                x: 0,
                y: 0
            },
            lastPosition: undefined
        };
    }
    componentDidUpdate(prevProps) {
        if (this.props.position && (!prevProps.position || this.props.position !== prevProps.position)) {
            this.setState({
                position: this.props.position
            });
        }
    }
    componentWillUnmount() {
        this._events.forEach(dispose => dispose());
    }
    render() {
        const child = React__namespace.Children.only(this.props.children);
        const { props } = child;
        const { position } = this.props;
        const { position: statePosition, isDragging } = this.state;
        let x = statePosition.x;
        let y = statePosition.y;
        if (position && !isDragging) {
            x = position.x;
            y = position.y;
        }
        return React__namespace.cloneElement(child, {
            style: Object.assign(Object.assign({}, props.style), { transform: `translate(${x}px, ${y}px)` }),
            className: getDraggableZoneClassNames(props.className, this.state.isDragging).root,
            onMouseDown: this._onMouseDown,
            onMouseUp: this._onMouseUp,
            onTouchStart: this._onTouchStart,
            onTouchEnd: this._onTouchEnd
        });
    }
    /**
     * Get the control position based off the event that fired
     * @param event - The event to get offsets from
     */
    _getControlPosition(event) {
        const touchObj = this._getActiveTouch(event);
        // did we get the right touch?
        if (this._touchId !== undefined && !touchObj) {
            return undefined;
        }
        const eventToGetOffset = touchObj || event;
        return {
            x: eventToGetOffset.clientX,
            y: eventToGetOffset.clientY
        };
    }
    /**
     * Get the active touch point that we have saved from the event's TouchList
     * @param event - The event used to get the TouchList for the active touch point
     */
    _getActiveTouch(event) {
        return event.targetTouches && this._findTouchInTouchList(event.targetTouches) || event.changedTouches && this._findTouchInTouchList(event.changedTouches);
    }
    /**
     * Get the initial touch identifier associated with the given event
     * @param event - The event that contains the TouchList
     */
    _getTouchId(event) {
        const touch = event.targetTouches && event.targetTouches[0] || event.changedTouches && event.changedTouches[0];
        if (touch) {
            return touch.identifier;
        }
        return;
    }
    /**
     * Returns if an element (or any of the element's parents) match the given selector
     */
    _matchesSelector(element, selector) {
        if (!element || element === document.body) {
            return false;
        }
        // eslint-disable-next-line @typescript-eslint/no-unsafe-function-type
        const matchesSelectorFn = element.matches || element.webkitMatchesSelector || element.msMatchesSelector; /* for IE */
        if (!matchesSelectorFn) {
            return false;
        }
        return matchesSelectorFn.call(element, selector) || this._matchesSelector(element.parentElement, selector);
    }
    /**
     * Attempts to find the Touch that matches the identifier  we stored in dragStart
     * @param touchList The TouchList to look for the stored identifier from dragStart
     */
    _findTouchInTouchList(touchList) {
        var _a;
        if (this._touchId === undefined) {
            return;
        }
        for (let i = 0; i < touchList.length; i++) {
            if (((_a = touchList[i]) === null || _a === void 0 ? void 0 : _a.identifier) === this._touchId) {
                return touchList[i];
            }
        }
        return undefined;
    }
    /**
     * Create DragData based off of the last known position and the new position passed in
     * @param position The new position as part of the drag
     */
    _createDragDataFromPosition(position) {
        const { lastPosition } = this.state;
        // If we have no lastPosition, use the given position
        // for last position
        if (lastPosition === undefined) {
            return {
                delta: {
                    x: 0,
                    y: 0
                },
                lastPosition: position,
                position
            };
        }
        return {
            delta: {
                x: position.x - lastPosition.x,
                y: position.y - lastPosition.y
            },
            lastPosition,
            position
        };
    }
    /**
     * Creates an updated DragData based off the current position and given baseDragData
     * @param baseDragData The base DragData (from _createDragDataFromPosition) used to calculate the updated positions
     */
    _createUpdatedDragData(baseDragData) {
        const { position } = this.state;
        return {
            position: {
                x: position.x + baseDragData.delta.x,
                y: position.y + baseDragData.delta.y
            },
            delta: baseDragData.delta,
            lastPosition: position
        };
    }
}
const globalClassNames = {
    root: 'ms-Modal',
    main: 'ms-Dialog-main',
    scrollableContent: 'ms-Modal-scrollableContent',
    isOpen: 'is-open',
    layer: 'ms-Modal-Layer'
};
const getStyles = (props) => {
    const { className, containerClassName, scrollableContentClassName, isOpen, isVisible, hasBeenOpened, modalRectangleTop, theme, topOffsetFixed, isModeless, layerClassName, isDefaultDragHandle, windowInnerHeight } = props;
    const { palette, effects, fonts } = theme;
    const classNames = react.getGlobalClassNames(globalClassNames, theme);
    return {
        root: [classNames.root, fonts.medium, {
                backgroundColor: 'transparent',
                position: isModeless ? 'absolute' : 'fixed',
                height: '100%',
                width: '100%',
                display: 'flex',
                alignItems: 'center',
                justifyContent: 'center',
                opacity: 0,
                pointerEvents: 'none',
                transition: `opacity ${animationDuration}`
            }, topOffsetFixed && typeof modalRectangleTop === 'number' && hasBeenOpened && {
                alignItems: 'flex-start'
            }, isOpen && classNames.isOpen, isVisible && {
                opacity: 1,
                pointerEvents: 'auto'
            }, className],
        main: [classNames.main, {
                boxShadow: effects.elevation64,
                borderRadius: effects.roundedCorner2,
                backgroundColor: palette.white,
                boxSizing: 'border-box',
                position: 'relative',
                textAlign: 'left',
                outline: '3px solid transparent',
                maxHeight: 'calc(100% - 32px)',
                maxWidth: 'calc(100% - 32px)',
                minHeight: '176px',
                minWidth: '288px',
                overflowY: 'auto',
                zIndex: isModeless ? react.ZIndexes.Layer : undefined
            }, topOffsetFixed && typeof modalRectangleTop === 'number' && hasBeenOpened && {
                top: modalRectangleTop
            }, isDefaultDragHandle && {
                cursor: 'move'
            }, containerClassName],
        scrollableContent: [classNames.scrollableContent, {
                overflowY: 'auto',
                flexGrow: 1,
                maxHeight: '100vh',
                selectors: {
                    ['@supports (-webkit-overflow-scrolling: touch)']: {
                        maxHeight: windowInnerHeight
                    }
                }
            }, scrollableContentClassName],
        layer: isModeless && [layerClassName, classNames.layer, {
                position: 'static',
                width: 'unset',
                height: 'unset'
            }],
        keyboardMoveIconContainer: {
            position: 'absolute',
            display: 'flex',
            justifyContent: 'center',
            width: '100%',
            padding: '3px 0px'
        },
        keyboardMoveIcon: {
            fontSize: fonts.xLargePlus.fontSize,
            width: '24px'
        }
    };
};
/** @internal */
const _ModalClone = react.styled(ModalBase, getStyles, undefined, {
    scope: 'Modal',
    fields: ['theme', 'styles', 'enableAriaHiddenSiblings']
});
_ModalClone.displayName = 'Modal';

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const DRAG_OPTIONS$1 = {
    moveMenuItemText: 'Move',
    closeMenuItemText: 'Close',
    menu: react.ContextualMenu,
    keepInBounds: true
};
// Manually override the max position used to keep the modal in the bounds of its container.
// This is a workaround for: https://github.com/microsoft/fluentui/issues/20122
// Because our modal starts in the bottom right corner, we can say that this is the max (i.e. rightmost and bottomost)
// position the modal can be dragged to.
const modalMaxDragPosition = {
    x: _convertRemToPx(floatinglocalVideoModalInitialPositionGapRem),
    y: _convertRemToPx(floatinglocalVideoModalInitialPositionGapRem)
};
/**
 * @private
 */
const FloatingLocalVideo = (props) => {
    var _a;
    const { localVideoComponent, layerHostId, localVideoSizeRem, parentWidth, parentHeight } = props;
    const theme = useTheme();
    const strings = useLocale$1().strings.videoGallery;
    // The minimum drag position is the top left of the video gallery. i.e. the modal (PiP) should not be able
    // to be dragged offscreen and these are the top and left bounds of that calculation.
    const modalMinDragPosition = React.useMemo(() => parentWidth && parentHeight ? {
        // We use -parentWidth/Height because our modal is positioned to start in the bottom right,
        // hence (0,0) is the bottom right of the video gallery.
        x: -parentWidth + _convertRemToPx(localVideoSizeRem.width) + _convertRemToPx(floatinglocalVideoModalInitialPositionGapRem),
        y: -parentHeight + _convertRemToPx(localVideoSizeRem.height) + _convertRemToPx(floatinglocalVideoModalInitialPositionGapRem)
    } : undefined, [parentHeight, parentWidth, localVideoSizeRem.width, localVideoSizeRem.height]);
    const modalStyles = React.useMemo(() => floatingLocalVideoModalStyle(theme, localVideoSizeRem), [theme, localVideoSizeRem]);
    const layerProps = React.useMemo(() => ({
        hostId: layerHostId
    }), [layerHostId]);
    return React.createElement(_ModalClone, { isOpen: true, isModeless: true, dragOptions: DRAG_OPTIONS$1, styles: modalStyles, layerProps: layerProps, maxDragPosition: modalMaxDragPosition, minDragPosition: modalMinDragPosition, dataUiId: "floating-local-video-host" },
        React.createElement(react.Stack, { "aria-label": (_a = strings.localVideoMovementAriaLabel) !== null && _a !== void 0 ? _a : strings.localVideoMovementLabel, tabIndex: 0, role: 'dialog', style: {
                pointerEvents: 'auto'
            } }, localVideoComponent));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const rootLayoutStyle = {
    root: {
        position: 'relative',
        height: '100%',
        width: '100%'
    }
};
/**
 * @private
 */
const innerLayoutStyle = {
    root: {
        position: 'relative',
        height: '100%',
        width: '100%',
        padding: '0.5rem'
    }
};
/**
 * @private
 */
const layerHostStyle = {
    position: 'absolute',
    left: 0,
    top: 0,
    width: '100%',
    height: '100%',
    overflow: 'hidden',
    // pointer events for layerHost set to none to make descendants interactive
    pointerEvents: 'none'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * FloatingLocalVideoLayout displays remote participants and a screen sharing component in
 * a grid and overflow gallery while floating the local video
 *
 * @private
 */
const FloatingLocalVideoLayout = (props) => {
    const { remoteParticipants = [], dominantSpeakers, localVideoComponent, screenShareComponent, onRenderRemoteParticipant, styles, maxRemoteVideoStreams, showCameraSwitcherInLocalPreview, parentWidth, parentHeight, overflowGalleryPosition = 'horizontalBottom', pinnedParticipantUserIds = [], localVideoTileSize, spotlightedParticipantUserIds } = props;
    const theme = useTheme();
    const isNarrow = parentWidth ? isNarrowWidth(parentWidth) : false;
    const isShort = parentHeight ? isShortHeight(parentHeight) : false;
    // This is for tracking the number of children in the first page of overflow gallery.
    // This number will be used for the maxOverflowGalleryDominantSpeakers when organizing the remote participants.
    const childrenPerPage = React.useRef(4);
    const remoteVideosOn = remoteParticipants.filter(p => { var _a; return (_a = p.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable; }).length > 0;
    const { gridParticipants, overflowGalleryParticipants } = useOrganizedParticipants({
        remoteParticipants,
        dominantSpeakers,
        maxGridParticipants: remoteVideosOn ? maxRemoteVideoStreams : MAX_GRID_PARTICIPANTS_NOT_LARGE_GALLERY,
        isScreenShareActive: !!screenShareComponent,
        maxOverflowGalleryDominantSpeakers: screenShareComponent ? childrenPerPage.current - pinnedParticipantUserIds.length % childrenPerPage.current : childrenPerPage.current,
        pinnedParticipantUserIds,
        layout: 'floatingLocalVideo',
        spotlightedParticipantUserIds
    });
    /**
     * instantiate indexes available to render with indexes available that would be on first page
     *
     * For some components which do not strictly follow the order of the array, we might
     * re-render the initial tiles -> dispose them -> create new tiles, we need to take care of
     * this case when those components are here
     */
    const [indexesToRender, setIndexesToRender] = React.useState([]);
    const { gridTiles, overflowGalleryTiles } = renderTiles(gridParticipants, onRenderRemoteParticipant, maxRemoteVideoStreams, indexesToRender, overflowGalleryParticipants, dominantSpeakers);
    const shouldFloatLocalVideo = remoteParticipants.length > 0;
    if (!shouldFloatLocalVideo && localVideoComponent) {
        gridTiles.push(localVideoComponent);
    }
    const layerHostId = reactHooks.useId('layerhost');
    const localVideoSizeRem = React.useMemo(() => {
        if (isNarrow && localVideoTileSize !== '16:9' || localVideoTileSize === '9:16') {
            return SMALL_FLOATING_MODAL_SIZE_REM;
        }
        if ((overflowGalleryTiles.length > 0 || screenShareComponent) && overflowGalleryPosition === 'verticalRight') {
            return isNarrow ? SMALL_FLOATING_MODAL_SIZE_REM : isShort ? SHORT_VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM : VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM;
        }
        if ((overflowGalleryTiles.length > 0 || screenShareComponent) && overflowGalleryPosition === 'horizontalBottom') {
            return localVideoTileSize === '16:9' || !isNarrow ? LARGE_FLOATING_MODAL_SIZE_REM : SMALL_FLOATING_MODAL_SIZE_REM;
        }
        return LARGE_FLOATING_MODAL_SIZE_REM;
    }, [overflowGalleryTiles.length, isNarrow, screenShareComponent, isShort, overflowGalleryPosition, localVideoTileSize]);
    const wrappedLocalVideoComponent = localVideoComponent && shouldFloatLocalVideo || screenShareComponent && localVideoComponent ?
        // When we use showCameraSwitcherInLocalPreview it disables dragging to allow keyboard navigation.
        showCameraSwitcherInLocalPreview ? React.createElement(react.Stack, { className: react.mergeStyles(localVideoTileWithControlsContainerStyle(theme, localVideoSizeRem), {
                boxShadow: theme.effects.elevation8,
                zIndex: LOCAL_VIDEO_TILE_ZINDEX
            }) }, localVideoComponent) : overflowGalleryTiles.length > 0 || screenShareComponent ? React.createElement(react.Stack, { className: react.mergeStyles(localVideoTileContainerStyle(theme, localVideoSizeRem, !!screenShareComponent, overflowGalleryPosition)) }, localVideoComponent) : React.createElement(FloatingLocalVideo, { localVideoComponent: localVideoComponent, layerHostId: layerHostId, localVideoSizeRem: localVideoSizeRem, parentWidth: parentWidth, parentHeight: parentHeight }) : undefined;
    const overflowGallery = React.useMemo(() => {
        if (overflowGalleryTiles.length === 0 && !screenShareComponent) {
            return null;
        }
        return React.createElement(OverflowGallery, { isShort: isShort, onFetchTilesToRender: setIndexesToRender, isNarrow: isNarrow, shouldFloatLocalVideo: !!localVideoComponent, overflowGalleryElements: overflowGalleryTiles, horizontalGalleryStyles: styles === null || styles === void 0 ? void 0 : styles.horizontalGallery, verticalGalleryStyles: styles === null || styles === void 0 ? void 0 : styles.verticalGallery, overflowGalleryPosition: overflowGalleryPosition, onChildrenPerPageChange: (n) => {
                childrenPerPage.current = n;
            }, parentWidth: parentWidth });
    }, [isNarrow, isShort, screenShareComponent, overflowGalleryTiles, styles === null || styles === void 0 ? void 0 : styles.horizontalGallery, overflowGalleryPosition, setIndexesToRender, styles === null || styles === void 0 ? void 0 : styles.verticalGallery, parentWidth, localVideoComponent]);
    return React.createElement(react.Stack, { styles: rootLayoutStyle },
        React.createElement(react.Stack, { horizontal: overflowGalleryPosition === 'verticalRight', styles: innerLayoutStyle, tokens: videoGalleryLayoutGap },
            props.overflowGalleryPosition === 'horizontalTop' ? overflowGallery : React.createElement(React.Fragment, null),
            screenShareComponent ? screenShareComponent : React.createElement(GridLayout, { key: "grid-layout", styles: styles === null || styles === void 0 ? void 0 : styles.gridLayout }, gridTiles),
            overflowGalleryTrampoline$1(overflowGallery, props.overflowGalleryPosition)),
        React.createElement(react.LayerHost, { id: layerHostId, className: react.mergeStyles(layerHostStyle) }),
        wrappedLocalVideoComponent);
};
const overflowGalleryTrampoline$1 = (gallery, galleryPosition) => {
    return galleryPosition !== 'horizontalTop' ? gallery : React.createElement(React.Fragment, null);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Layout for the gallery mode to highlight the current dominant speaker
 *
 * @private
 */
const SpeakerVideoLayout = (props) => {
    const { remoteParticipants = [], dominantSpeakers, localVideoComponent, screenShareComponent, onRenderRemoteParticipant, styles, maxRemoteVideoStreams, parentWidth, parentHeight, overflowGalleryPosition = 'horizontalBottom', pinnedParticipantUserIds = [], localVideoTileSize } = props;
    const theme = react.useTheme();
    const isNarrow = parentWidth ? isNarrowWidth(parentWidth) : false;
    const isShort = parentHeight ? isShortHeight(parentHeight) : false;
    // This is for tracking the number of children in the first page of overflow gallery.
    // This number will be used for the maxOverflowGalleryDominantSpeakers when organizing the remote participants.
    const childrenPerPage = React.useRef(4);
    const { gridParticipants, overflowGalleryParticipants } = useOrganizedParticipants({
        remoteParticipants,
        dominantSpeakers,
        maxGridParticipants: maxRemoteVideoStreams,
        isScreenShareActive: !!screenShareComponent,
        maxOverflowGalleryDominantSpeakers: screenShareComponent ? childrenPerPage.current - pinnedParticipantUserIds.length % childrenPerPage.current : childrenPerPage.current,
        pinnedParticipantUserIds,
        layout: 'speaker'
    });
    /**
     * instantiate indexes available to render with indexes available that would be on first page
     *
     * For some components which do not strictly follow the order of the array, we might
     * re-render the initial tiles -> dispose them -> create new tiles, we need to take care of
     * this case when those components are here
     */
    const [indexesToRender, setIndexesToRender] = React.useState([]);
    const { gridTiles, overflowGalleryTiles } = renderTiles(gridParticipants, onRenderRemoteParticipant, maxRemoteVideoStreams, indexesToRender, overflowGalleryParticipants, dominantSpeakers);
    const shouldFloatLocalVideo = remoteParticipants.length > 0;
    if (!shouldFloatLocalVideo && localVideoComponent) {
        gridTiles.push(localVideoComponent);
    }
    const layerHostId = reactHooks.useId('layerhost');
    const localVideoSizeRem = React.useMemo(() => {
        if (isNarrow || localVideoTileSize === '9:16') {
            return SMALL_FLOATING_MODAL_SIZE_REM;
        }
        if ((overflowGalleryTiles.length > 0 || screenShareComponent) && overflowGalleryPosition === 'verticalRight') {
            return isNarrow ? SMALL_FLOATING_MODAL_SIZE_REM : isShort ? SHORT_VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM : VERTICAL_GALLERY_FLOATING_MODAL_SIZE_REM;
        }
        if ((overflowGalleryTiles.length > 0 || screenShareComponent) && overflowGalleryPosition === 'horizontalBottom') {
            return localVideoTileSize === '16:9' || !isNarrow ? LARGE_FLOATING_MODAL_SIZE_REM : SMALL_FLOATING_MODAL_SIZE_REM;
        }
        return LARGE_FLOATING_MODAL_SIZE_REM;
    }, [overflowGalleryTiles.length, isNarrow, screenShareComponent, isShort, overflowGalleryPosition, localVideoTileSize]);
    const wrappedLocalVideoComponent = localVideoComponent || screenShareComponent && localVideoComponent ? React.createElement(react.Stack, { className: react.mergeStyles(localVideoTileContainerStyle(theme, localVideoSizeRem, !!screenShareComponent, overflowGalleryPosition)) }, localVideoComponent) : undefined;
    const overflowGallery = React.useMemo(() => {
        if (overflowGalleryTiles.length === 0 && !screenShareComponent) {
            return null;
        }
        return React.createElement(OverflowGallery, { isShort: isShort, onFetchTilesToRender: setIndexesToRender, isNarrow: isNarrow, shouldFloatLocalVideo: !!localVideoComponent, overflowGalleryElements: overflowGalleryTiles, horizontalGalleryStyles: styles === null || styles === void 0 ? void 0 : styles.horizontalGallery, verticalGalleryStyles: styles === null || styles === void 0 ? void 0 : styles.verticalGallery, overflowGalleryPosition: overflowGalleryPosition, onChildrenPerPageChange: (n) => {
                childrenPerPage.current = n;
            }, parentWidth: parentWidth });
    }, [isNarrow, isShort, screenShareComponent, overflowGalleryTiles, styles === null || styles === void 0 ? void 0 : styles.horizontalGallery, overflowGalleryPosition, setIndexesToRender, styles === null || styles === void 0 ? void 0 : styles.verticalGallery, parentWidth, localVideoComponent]);
    return React.createElement(react.Stack, { styles: rootLayoutStyle },
        React.createElement(react.Stack, { horizontal: overflowGalleryPosition === 'verticalRight', styles: innerLayoutStyle, tokens: videoGalleryLayoutGap },
            props.overflowGalleryPosition === 'horizontalTop' ? overflowGallery : React.createElement(React.Fragment, null),
            screenShareComponent ? screenShareComponent : React.createElement(GridLayout, { key: "grid-layout", styles: styles === null || styles === void 0 ? void 0 : styles.gridLayout }, gridTiles),
            overflowGalleryTrampoline(overflowGallery, props.overflowGalleryPosition)),
        React.createElement(react.LayerHost, { id: layerHostId, className: react.mergeStyles(layerHostStyle) }),
        wrappedLocalVideoComponent);
};
const overflowGalleryTrampoline = (gallery, galleryPosition) => {
    return galleryPosition !== 'horizontalTop' ? gallery : React.createElement(React.Fragment, null);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Video gallery layout to focus on the screenshare stream that is present in the call.
 */
const FocusedContentLayout = (props) => {
    const { screenShareComponent } = props;
    const layerHostId = reactHooks.useId('layerhost');
    return React.createElement(react.Stack, { styles: rootLayoutStyle },
        React.createElement(react.LayerHost, { id: layerHostId, className: react.mergeStyles(layerHostStyle) }),
        React.createElement(react.Stack, { styles: innerLayoutStyle, tokens: videoGalleryLayoutGap }, screenShareComponent ? screenShareComponent : React.createElement(React.Fragment, null)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A memoized version of local screen share component. React.memo is used for a performance
 * boost by memoizing the same rendered component to avoid rerendering this when the parent component rerenders.
 * https://reactjs.org/docs/react-api.html#reactmemo
 */
const TogetherModeLayout = (props) => {
    const { togetherModeStreamComponent } = props;
    return React.createElement(react.Stack, null, togetherModeStreamComponent);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A memoized version of local screen share component. React.memo is used for a performance
 * boost by memoizing the same rendered component to avoid rerendering this when the parent component rerenders.
 * https://reactjs.org/docs/react-api.html#reactmemo
 */
const TogetherModeStream = React.memo((props) => {
    var _a, _b, _c, _d;
    const { startTogetherModeEnabled, isTogetherModeActive, onCreateTogetherModeStreamView, onStartTogetherMode, onSetTogetherModeSceneSize, onDisposeTogetherModeStreamView, togetherModeStreams, containerWidth, containerHeight } = props;
    const [showLoadingIndicator, setShowLoadingIndicator] = React.useState(false);
    React.useEffect(() => {
        return () => {
            // TODO: Isolate disposing behaviors for screenShare and videoStream
            onDisposeTogetherModeStreamView && onDisposeTogetherModeStreamView();
        };
    }, [onDisposeTogetherModeStreamView]);
    // Trigger startTogetherMode only when needed
    React.useEffect(() => {
        if (startTogetherModeEnabled && !isTogetherModeActive) {
            onStartTogetherMode === null || onStartTogetherMode === void 0 ? void 0 : onStartTogetherMode();
        }
    }, [startTogetherModeEnabled, isTogetherModeActive, onStartTogetherMode]);
    // Create stream view if not already created
    React.useEffect(() => {
        var _a, _b;
        if (((_a = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _a === void 0 ? void 0 : _a.isAvailable) && !((_b = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _b === void 0 ? void 0 : _b.renderElement)) {
            onCreateTogetherModeStreamView === null || onCreateTogetherModeStreamView === void 0 ? void 0 : onCreateTogetherModeStreamView();
        }
    }, [(_a = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _a === void 0 ? void 0 : _a.renderElement, (_b = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _b === void 0 ? void 0 : _b.isAvailable, onCreateTogetherModeStreamView]);
    // Re-render the component if mainVideoStream isReceiving changes
    React.useEffect(() => {
        var _a;
        setShowLoadingIndicator(!((_a = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _a === void 0 ? void 0 : _a.isReceiving));
        // This effect will trigger a re-render when isReceiving changes
    }, [(_c = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _c === void 0 ? void 0 : _c.isReceiving]);
    // Update scene size only when container dimensions change
    React.useMemo(() => {
        if (onSetTogetherModeSceneSize && containerWidth && containerHeight) {
            onSetTogetherModeSceneSize(containerWidth, containerHeight);
        }
    }, [onSetTogetherModeSceneSize, containerWidth, containerHeight]);
    const stream = (_d = props.togetherModeStreams) === null || _d === void 0 ? void 0 : _d.mainVideoStream;
    return containerWidth && containerHeight ? React.createElement(react.Stack, { styles: togetherModeStreamRootStyle, horizontalAlign: "center", verticalAlign: "center", "data-ui-id": "together-mode-layout" },
        React.createElement(StreamMedia, { videoStreamElement: (stream === null || stream === void 0 ? void 0 : stream.renderElement) || null, isMirrored: true, loadingState: showLoadingIndicator ? 'loading' : 'none' }),
        React.createElement(MeetingReactionOverlay, { reactionResources: props.reactionResources || {}, localParticipant: props.localParticipant, remoteParticipants: props.remoteParticipants, togetherModeSeatPositions: props.seatingCoordinates, overlayMode: "together-mode" })) : React.createElement(React.Fragment, null);
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 * Currently the Calling JS SDK supports up to 4 remote video streams
 */
const DEFAULT_MAX_REMOTE_VIDEO_STREAMS = 4;
/**
 * @private
 * Styles to disable the selectivity of a text in video gallery
 */
const unselectable = {
    '-webkit-user-select': 'none',
    '-webkit-touch-callout': 'none',
    '-moz-user-select': 'none',
    '-ms-user-select': 'none',
    'user-select': 'none'
};
/**
 * @private
 * Default remote video tile menu options
 */
const DEFAULT_REMOTE_VIDEO_TILE_MENU_OPTIONS = {
    kind: 'contextual'
};
/**
 * @private
 * Maximum number of remote video tiles that can be pinned
 */
const MAX_PINNED_REMOTE_VIDEO_TILES$1 = 4;
/**
 * VideoGallery represents a layout of video tiles for a specific call.
 * It displays a {@link VideoTile} for the local user as well as for each remote participant who has joined the call.
 *
 * @public
 */
const VideoGallery = (props) => {
    var _a, _b, _c, _d;
    const { localParticipant, remoteParticipants = [], localVideoViewOptions, remoteVideoViewOptions, dominantSpeakers, onRenderLocalVideoTile, onRenderRemoteVideoTile, onCreateLocalStreamView, onDisposeLocalStreamView, onCreateRemoteStreamView, onDisposeRemoteScreenShareStreamView, onDisposeLocalScreenShareStreamView, onDisposeRemoteVideoStreamView, styles, layout, onRenderAvatar, showMuteIndicator, maxRemoteVideoStreams = DEFAULT_MAX_REMOTE_VIDEO_STREAMS, showCameraSwitcherInLocalPreview, localVideoCameraCycleButtonProps, onPinParticipant: onPinParticipantHandler, onUnpinParticipant: onUnpinParticipantHandler, remoteVideoTileMenu = DEFAULT_REMOTE_VIDEO_TILE_MENU_OPTIONS, overflowGalleryPosition = 'horizontalBottom', localVideoTileSize = 'followDeviceOrientation', spotlightedParticipants, onStartLocalSpotlight, onStartRemoteSpotlight, onStopLocalSpotlight, onStopRemoteSpotlight, maxParticipantsToSpotlight, reactionResources, videoTilesOptions, onMuteParticipant, startTogetherModeEnabled, isTogetherModeActive, onCreateTogetherModeStreamView, onStartTogetherMode, onSetTogetherModeSceneSize, togetherModeStreams, togetherModeSeatingCoordinates, onDisposeTogetherModeStreamView, onForbidAudio, onPermitAudio, onForbidVideo, onPermitVideo, localScreenShareView } = props;
    const ids = useIdentifiers();
    const theme = useTheme();
    const localeStrings = useLocale$1().strings.videoGallery;
    const strings = React.useMemo(() => (Object.assign(Object.assign({}, localeStrings), props.strings)), [localeStrings, props.strings]);
    const drawerMenuHostIdFromProp = remoteVideoTileMenu && remoteVideoTileMenu.kind === 'drawer' ? remoteVideoTileMenu.hostId : undefined;
    const drawerMenuHostId = reactHooks.useId('drawerMenuHost', drawerMenuHostIdFromProp);
    const localTileNotInGrid = (layout === 'floatingLocalVideo' || layout === 'speaker') && remoteParticipants.length > 0;
    const containerRef = React.useRef(null);
    const containerWidth = _useContainerWidth(containerRef);
    const containerHeight = _useContainerHeight(containerRef);
    const isNarrow = containerWidth ? isNarrowWidth(containerWidth) : false;
    const [pinnedParticipantsState, setPinnedParticipantsState] = React.useState([]);
    const [selectedScalingModeState, setselectedScalingModeState] = React.useState({});
    const onUpdateScalingMode = React.useCallback((remoteUserId, scalingMode) => {
        setselectedScalingModeState(current => (Object.assign(Object.assign({}, current), { [remoteUserId]: {
                scalingMode,
                isMirrored: remoteVideoViewOptions === null || remoteVideoViewOptions === void 0 ? void 0 : remoteVideoViewOptions.isMirrored
            } })));
    }, [remoteVideoViewOptions === null || remoteVideoViewOptions === void 0 ? void 0 : remoteVideoViewOptions.isMirrored]);
    React.useEffect(() => {
        var _a;
        (_a = props.pinnedParticipants) === null || _a === void 0 ? void 0 : _a.forEach(pinParticipant => {
            var _a;
            if (!((_a = props.remoteParticipants) === null || _a === void 0 ? void 0 : _a.find(t => t.userId === pinParticipant))) {
                // warning will be logged in the console when invalid participant id is passed in pinned participants
                console.warn('Invalid pinned participant UserId :' + pinParticipant);
            }
        });
    }, [props.pinnedParticipants, props.remoteParticipants]);
    // Use pinnedParticipants from props but if it is not defined use the maintained state of pinned participants
    const pinnedParticipants = React.useMemo(() => { var _a; return (_a = props.pinnedParticipants) !== null && _a !== void 0 ? _a : pinnedParticipantsState.filter(pinnedParticipantId => remoteParticipants.find(remoteParticipant => remoteParticipant.userId === pinnedParticipantId)); }, [props.pinnedParticipants, pinnedParticipantsState, remoteParticipants]);
    const showLocalVideoTileLabel = !(localTileNotInGrid && isNarrow && localVideoTileSize !== '16:9' || localVideoTileSize === '9:16') || layout === 'default';
    /**
     * Utility function for memoized rendering of LocalParticipant.
     */
    const localVideoTile = React.useMemo(() => {
        var _a, _b;
        if (localVideoTileSize === 'hidden') {
            return undefined;
        }
        if (onRenderLocalVideoTile) {
            return onRenderLocalVideoTile(localParticipant);
        }
        const isSpotlighted = !!localParticipant.spotlight;
        const localVideoTileStyles = react.concatStyleSets(localTileNotInGrid ? floatingLocalVideoTileStyle : {}, {
            root: {
                borderRadius: theme.effects.roundedCorner4
            }
        }, styles === null || styles === void 0 ? void 0 : styles.localVideo);
        const initialsName = !localParticipant.displayName ? '' : localParticipant.displayName;
        const showDisplayNameTrampoline = () => {
            return layout === 'default' ? strings.localVideoLabel : isNarrow && localVideoTileSize !== '16:9' ? '' : strings.localVideoLabel;
        };
        return React.createElement(react.Stack, { styles: localVideoTileContainerStyles, key: "local-video-tile-key" },
            React.createElement(_LocalVideoTile, { alwaysShowLabelBackground: videoTilesOptions === null || videoTilesOptions === void 0 ? void 0 : videoTilesOptions.alwaysShowLabelBackground, userId: localParticipant.userId, onCreateLocalStreamView: onCreateLocalStreamView, onDisposeLocalStreamView: onDisposeLocalStreamView, isAvailable: (_a = localParticipant === null || localParticipant === void 0 ? void 0 : localParticipant.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable, isMuted: localParticipant.isMuted, renderElement: (_b = localParticipant === null || localParticipant === void 0 ? void 0 : localParticipant.videoStream) === null || _b === void 0 ? void 0 : _b.renderElement, displayName: showDisplayNameTrampoline(), initialsName: initialsName, localVideoViewOptions: localVideoViewOptions, onRenderAvatar: onRenderAvatar, showLabel: showLocalVideoTileLabel, showMuteIndicator: showMuteIndicator, showCameraSwitcherInLocalPreview: showCameraSwitcherInLocalPreview, localVideoCameraCycleButtonProps: localVideoCameraCycleButtonProps, localVideoCameraSwitcherLabel: strings.localVideoCameraSwitcherLabel, localVideoSelectedDescription: strings.localVideoSelectedDescription, styles: localVideoTileStyles, raisedHand: localParticipant.raisedHand, reaction: localParticipant.reaction, spotlightedParticipantUserIds: spotlightedParticipants, isSpotlighted: isSpotlighted, onStartSpotlight: onStartLocalSpotlight, onStopSpotlight: onStopLocalSpotlight, maxParticipantsToSpotlight: maxParticipantsToSpotlight, menuKind: remoteVideoTileMenu ? remoteVideoTileMenu.kind === 'drawer' ? 'drawer' : 'contextual' : undefined, drawerMenuHostId: drawerMenuHostId, strings: strings, reactionResources: reactionResources, participantsCount: remoteParticipants.length + 1, isScreenSharingOn: localParticipant.isScreenSharingOn, mediaAccess: localParticipant.mediaAccess }));
    }, [isNarrow, localParticipant, localVideoCameraCycleButtonProps, localVideoViewOptions, onCreateLocalStreamView, onDisposeLocalStreamView, onRenderAvatar, onRenderLocalVideoTile, localTileNotInGrid, showCameraSwitcherInLocalPreview, showMuteIndicator, styles === null || styles === void 0 ? void 0 : styles.localVideo, theme.effects.roundedCorner4, localVideoTileSize, layout, showLocalVideoTileLabel, spotlightedParticipants, onStartLocalSpotlight, onStopLocalSpotlight, maxParticipantsToSpotlight, remoteVideoTileMenu, strings, drawerMenuHostId, reactionResources, videoTilesOptions, remoteParticipants.length]);
    const onPinParticipant = React.useCallback((userId) => {
        if (pinnedParticipants.length >= MAX_PINNED_REMOTE_VIDEO_TILES$1) {
            return;
        }
        if (!pinnedParticipants.includes(userId)) {
            setPinnedParticipantsState(pinnedParticipants.concat(userId));
        }
        onPinParticipantHandler === null || onPinParticipantHandler === void 0 ? void 0 : onPinParticipantHandler(userId);
    }, [pinnedParticipants, setPinnedParticipantsState, onPinParticipantHandler]);
    const onUnpinParticipant = React.useCallback((userId) => {
        setPinnedParticipantsState(pinnedParticipantsState.filter(p => p !== userId));
        onUnpinParticipantHandler === null || onUnpinParticipantHandler === void 0 ? void 0 : onUnpinParticipantHandler(userId);
    }, [pinnedParticipantsState, setPinnedParticipantsState, onUnpinParticipantHandler]);
    const [announcementString, setAnnouncementString] = React.useState('');
    /**
     * sets the announcement string for VideoGallery actions so that the screenreader will trigger
     */
    const toggleAnnouncerString = React.useCallback((announcement) => {
        setAnnouncementString(announcement);
        /**
         * Clears the announcer string after VideoGallery action allowing it to be re-announced.
         */
        setTimeout(() => {
            setAnnouncementString('');
        }, 3000);
    }, [setAnnouncementString]);
    const defaultOnRenderVideoTile = React.useCallback((participant, isVideoParticipant) => {
        const remoteVideoStream = participant.videoStream;
        const selectedScalingMode = remoteVideoStream ? selectedScalingModeState[participant.userId] : undefined;
        let isPinned = pinnedParticipants === null || pinnedParticipants === void 0 ? void 0 : pinnedParticipants.includes(participant.userId);
        const isSpotlighted = !!participant.spotlight;
        isPinned = isSpotlighted ? false : isPinned;
        const createViewOptions = () => {
            var _a, _b;
            if (selectedScalingMode) {
                return selectedScalingMode;
            }
            return (remoteVideoStream === null || remoteVideoStream === void 0 ? void 0 : remoteVideoStream.streamSize) && ((_a = remoteVideoStream.streamSize) === null || _a === void 0 ? void 0 : _a.height) > ((_b = remoteVideoStream.streamSize) === null || _b === void 0 ? void 0 : _b.width) ? {
                scalingMode: 'Fit',
                isMirrored: remoteVideoViewOptions === null || remoteVideoViewOptions === void 0 ? void 0 : remoteVideoViewOptions.isMirrored
            } : remoteVideoViewOptions;
        };
        return React.createElement(_RemoteVideoTile, { alwaysShowLabelBackground: videoTilesOptions === null || videoTilesOptions === void 0 ? void 0 : videoTilesOptions.alwaysShowLabelBackground, streamId: remoteVideoStream === null || remoteVideoStream === void 0 ? void 0 : remoteVideoStream.id, key: participant.userId, userId: participant.userId, remoteParticipant: participant, onCreateRemoteStreamView: isVideoParticipant ? onCreateRemoteStreamView : undefined, onDisposeRemoteStreamView: isVideoParticipant ? onDisposeRemoteVideoStreamView : undefined, isAvailable: isVideoParticipant ? remoteVideoStream === null || remoteVideoStream === void 0 ? void 0 : remoteVideoStream.isAvailable : false, isReceiving: isVideoParticipant ? remoteVideoStream === null || remoteVideoStream === void 0 ? void 0 : remoteVideoStream.isReceiving : false, renderElement: isVideoParticipant ? remoteVideoStream === null || remoteVideoStream === void 0 ? void 0 : remoteVideoStream.renderElement : undefined, remoteVideoViewOptions: createViewOptions(), onRenderAvatar: onRenderAvatar, showMuteIndicator: showMuteIndicator, strings: strings, participantState: participant.state, menuKind: participant.userId === localParticipant.userId ? undefined : remoteVideoTileMenu ? remoteVideoTileMenu.kind === 'drawer' ? 'drawer' : 'contextual' : undefined, drawerMenuHostId: drawerMenuHostId, onPinParticipant: onPinParticipant, onUnpinParticipant: onUnpinParticipant, onUpdateScalingMode: onUpdateScalingMode, isPinned: isPinned, disablePinMenuItem: pinnedParticipants.length >= MAX_PINNED_REMOTE_VIDEO_TILES$1, toggleAnnouncerString: toggleAnnouncerString, spotlightedParticipantUserIds: spotlightedParticipants, isSpotlighted: isSpotlighted, onStartSpotlight: onStartRemoteSpotlight, onStopSpotlight: onStopRemoteSpotlight, maxParticipantsToSpotlight: maxParticipantsToSpotlight, reactionResources: reactionResources, onMuteParticipant: onMuteParticipant, onForbidAudio: onForbidAudio, onPermitAudio: onPermitAudio, onForbidVideo: onForbidVideo, onPermitVideo: onPermitVideo });
    }, [selectedScalingModeState, pinnedParticipants, videoTilesOptions === null || videoTilesOptions === void 0 ? void 0 : videoTilesOptions.alwaysShowLabelBackground, onCreateRemoteStreamView, onDisposeRemoteVideoStreamView, onRenderAvatar, showMuteIndicator, strings, localParticipant.userId, remoteVideoTileMenu, drawerMenuHostId, onPinParticipant, onUnpinParticipant, onUpdateScalingMode, toggleAnnouncerString, spotlightedParticipants, onStartRemoteSpotlight, onStopRemoteSpotlight, maxParticipantsToSpotlight, reactionResources, onMuteParticipant, onForbidAudio, onPermitAudio, onForbidVideo, onPermitVideo, remoteVideoViewOptions]);
    const screenShareParticipant = remoteParticipants.find(participant => { var _a; return (_a = participant.screenShareStream) === null || _a === void 0 ? void 0 : _a.isAvailable; });
    const localScreenShareStreamComponent = React.createElement(LocalScreenShare, { localParticipant: localParticipant, renderElement: (_a = localParticipant.screenShareStream) === null || _a === void 0 ? void 0 : _a.renderElement, isAvailable: (_b = localParticipant.screenShareStream) === null || _b === void 0 ? void 0 : _b.isAvailable, onCreateLocalStreamView: onCreateLocalStreamView, onDisposeLocalScreenShareStreamView: onDisposeLocalScreenShareStreamView, localScreenShareView: localScreenShareView });
    const remoteScreenShareComponent = screenShareParticipant && React.createElement(RemoteScreenShare, Object.assign({}, screenShareParticipant, { renderElement: (_c = screenShareParticipant.screenShareStream) === null || _c === void 0 ? void 0 : _c.renderElement, onCreateRemoteStreamView: onCreateRemoteStreamView, onDisposeRemoteStreamView: onDisposeRemoteScreenShareStreamView, isReceiving: (_d = screenShareParticipant.screenShareStream) === null || _d === void 0 ? void 0 : _d.isReceiving, participantVideoScalingMode: selectedScalingModeState[screenShareParticipant.userId], localParticipant: localParticipant, remoteParticipants: remoteParticipants, reactionResources: reactionResources }));
    const screenShareComponent = remoteScreenShareComponent ? remoteScreenShareComponent : localParticipant.isScreenSharingOn ? localScreenShareStreamComponent : undefined;
    // Current implementation of capabilities is only based on user role.
    // This logic checks for the user role and if the user is a Teams user.
    const canSwitchToTogetherModeLayout = isTogetherModeActive || _isIdentityMicrosoftTeamsUser(localParticipant.userId) && startTogetherModeEnabled;
    const togetherModeStreamComponent = React.useMemo(() => 
    // Avoids unnecessary rendering of TogetherModeStream component when it is not needed
    !screenShareComponent && canSwitchToTogetherModeLayout && layout === 'togetherMode' ? React.createElement(TogetherModeStream, { startTogetherModeEnabled: startTogetherModeEnabled, isTogetherModeActive: isTogetherModeActive, onCreateTogetherModeStreamView: onCreateTogetherModeStreamView, onStartTogetherMode: onStartTogetherMode, onDisposeTogetherModeStreamView: onDisposeTogetherModeStreamView, onSetTogetherModeSceneSize: onSetTogetherModeSceneSize, togetherModeStreams: togetherModeStreams, seatingCoordinates: togetherModeSeatingCoordinates, localParticipant: localParticipant, remoteParticipants: remoteParticipants, reactionResources: reactionResources, containerWidth: containerWidth, containerHeight: containerHeight }) : undefined, [layout, screenShareComponent, canSwitchToTogetherModeLayout, startTogetherModeEnabled, isTogetherModeActive, onCreateTogetherModeStreamView, onStartTogetherMode, onDisposeTogetherModeStreamView, onSetTogetherModeSceneSize, togetherModeStreams, togetherModeSeatingCoordinates, localParticipant, remoteParticipants, reactionResources, containerWidth, containerHeight]);
    const layoutProps = React.useMemo(() => ({
        remoteParticipants,
        localParticipant,
        screenShareComponent,
        showCameraSwitcherInLocalPreview,
        maxRemoteVideoStreams,
        dominantSpeakers,
        styles,
        onRenderRemoteParticipant: onRenderRemoteVideoTile !== null && onRenderRemoteVideoTile !== void 0 ? onRenderRemoteVideoTile : defaultOnRenderVideoTile,
        localVideoComponent: localVideoTile,
        parentWidth: containerWidth,
        parentHeight: containerHeight,
        pinnedParticipantUserIds: pinnedParticipants,
        overflowGalleryPosition,
        localVideoTileSize,
        spotlightedParticipantUserIds: spotlightedParticipants
    }), [remoteParticipants, localParticipant, screenShareComponent, showCameraSwitcherInLocalPreview, maxRemoteVideoStreams, dominantSpeakers, styles, localVideoTile, containerWidth, containerHeight, onRenderRemoteVideoTile, defaultOnRenderVideoTile, pinnedParticipants, overflowGalleryPosition, localVideoTileSize, spotlightedParticipants]);
    const videoGalleryLayout = React.useMemo(() => {
        if (screenShareParticipant && layout === 'focusedContent') {
            return React.createElement(FocusedContentLayout, Object.assign({}, layoutProps));
        }
        if (layout === 'floatingLocalVideo') {
            return React.createElement(FloatingLocalVideoLayout, Object.assign({}, layoutProps));
        }
        if (layout === 'speaker') {
            return React.createElement(SpeakerVideoLayout, Object.assign({}, layoutProps));
        }
        // Teams users can switch to Together mode layout only if they have the capability,
        // while ACS users can do so only if Together mode is enabled.
        if (togetherModeStreamComponent && layout === 'togetherMode') {
            return React.createElement(TogetherModeLayout, { togetherModeStreamComponent: togetherModeStreamComponent });
        }
        return React.createElement(DefaultLayout, Object.assign({}, layoutProps));
    }, [layout, layoutProps, screenShareParticipant, togetherModeStreamComponent]);
    return React.createElement("div", { 
        // We don't assign an drawer menu host id to the VideoGallery when a drawerMenuHostId is assigned from props
        id: drawerMenuHostIdFromProp ? undefined : drawerMenuHostId, "data-ui-id": ids.videoGallery, ref: containerRef, className: react.mergeStyles(videoGalleryOuterDivStyle, styles === null || styles === void 0 ? void 0 : styles.root, unselectable) },
        videoGalleryLayout,
        React.createElement(Announcer$1, { announcementString: announcementString, ariaLive: "polite" }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const controlBarStyles$1 = {
    horizontal: {
        flexFlow: 'row nowrap'
    },
    vertical: {
        flexFlow: 'column nowrap',
        width: 'fit-content',
        maxWidth: '8rem'
    },
    dockedTop: {
        flexFlow: 'row nowrap',
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        top: 0,
        left: 0,
        width: '100%',
        minWidth: 'fit-content'
    },
    dockedBottom: {
        flexFlow: 'row nowrap',
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        bottom: 0,
        left: 0,
        width: '100%',
        minWidth: 'fit-content'
    },
    dockedLeft: {
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        top: 0,
        left: 0,
        height: '100%'
    },
    dockedRight: {
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        top: 0,
        right: 0,
        height: '100%'
    },
    floatingTop: {
        flexFlow: 'row nowrap',
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        top: '1rem',
        left: '50%',
        transform: 'translateX(-50%)',
        minWidth: 'fit-content',
        zIndex: 10
    },
    floatingBottom: {
        flexFlow: 'row nowrap',
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        bottom: '1rem',
        left: '50%',
        transform: 'translateX(-50%)',
        minWidth: 'fit-content',
        zIndex: 10
    },
    floatingLeft: {
        flexFlow: 'column nowrap',
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        top: '50%',
        left: '1rem',
        transform: 'translateY(-50%)',
        zIndex: 10
    },
    floatingRight: {
        flexFlow: 'column nowrap',
        justifyContent: 'center',
        overflow: 'hidden',
        position: 'absolute',
        top: '50%',
        right: '1rem',
        transform: 'translateY(-50%)',
        zIndex: 10
    }
};
/**
 * @private
 */
const controlButtonStyles = {
    root: {
        background: 'none',
        border: 'none',
        borderRadius: 0,
        minHeight: '3.5rem',
        minWidth: '3.5rem',
        width: '100%',
        maxWidth: '8rem',
        svg: {
            verticalAlign: 'text-top'
        }
    },
    splitButtonMenuButton: {
        border: 'none'
    },
    flexContainer: {
        display: 'unset'
    },
    label: {
        fontSize: '0.625rem',
        fontWeight: '400',
        lineHeight: '1rem',
        cursor: 'pointer',
        display: 'block',
        margin: '0rem 0.25rem',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap'
    }
};
/**
 * making it Partial as IContextualMenuStyles has all its props non-optional and we only need title to be defined here.
 *
 * @private
 */
const participantsButtonMenuPropsStyle = {
    title: {
        background: 'initial',
        paddingLeft: '.5rem',
        fontWeight: 600,
        fontSize: '.75rem'
    }
};
/**
 * Default styles for button flyout items
 *
 * @private
 */
const buttonFlyoutItemStyles = {
    icon: {
        lineHeight: 0
    },
    checkmarkIcon: {
        lineHeight: 0
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// The TooltipHost root uses display: inline by default.
// To prevent sizing issues or tooltip positioning issues, we override to inline-block.
// For more details see "Icon Button with Tooltip" on https://developer.microsoft.com/en-us/fluentui#/controls/web/button
const hostStyles = {
    root: {
        display: 'inline-block'
    }
};
/**
 * Tooltip that should wrap control bar buttons.
 *
 * @private
 */
const ControlButtonTooltip = (props) => {
    const theme = react.useTheme();
    const calloutStyle = {
        root: {
            padding: 0
        },
        calloutMain: {
            padding: '0.5rem'
        },
        beakCurtain: {
            marginBottom: '-1rem',
            backgroundColor: 'transparent'
        }
    };
    const calloutProps = {
        gapSpace: 4,
        styles: calloutStyle,
        backgroundColor: isDarkThemed(theme) ? theme.palette.neutralLighter : ''
    };
    return React.createElement(react.TooltipHost, Object.assign({}, props, { "data-ui-id": props.id, calloutProps: Object.assign(Object.assign({}, calloutProps), props.calloutProps), styles: react.mergeStyleSets(hostStyles, props.styles) }), props.children);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const DefaultRenderIcon = (props) => {
    return (props === null || props === void 0 ? void 0 : props.checked) ? (props === null || props === void 0 ? void 0 : props.onRenderOnIcon) ? props === null || props === void 0 ? void 0 : props.onRenderOnIcon() : null : (props === null || props === void 0 ? void 0 : props.onRenderOffIcon) ? props === null || props === void 0 ? void 0 : props.onRenderOffIcon() : null;
};
/**
 * Default button styled for the {@link ControlBar}.
 *
 * Use this component create custom buttons that are styled the same as other buttons provided by the UI Library.
 *
 * @public
 */
const ControlBarButton = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
    const componentStyles = react.concatStyleSets(controlButtonStyles, (_a = props.styles) !== null && _a !== void 0 ? _a : {});
    const labelText = (_d = (_b = props === null || props === void 0 ? void 0 : props.text) !== null && _b !== void 0 ? _b : (_c = props === null || props === void 0 ? void 0 : props.strings) === null || _c === void 0 ? void 0 : _c.label) !== null && _d !== void 0 ? _d : ((props === null || props === void 0 ? void 0 : props.checked) ? (_e = props === null || props === void 0 ? void 0 : props.strings) === null || _e === void 0 ? void 0 : _e.onLabel : (_f = props === null || props === void 0 ? void 0 : props.strings) === null || _f === void 0 ? void 0 : _f.offLabel);
    const tooltipContent = (_h = (_g = props === null || props === void 0 ? void 0 : props.strings) === null || _g === void 0 ? void 0 : _g.tooltipContent) !== null && _h !== void 0 ? _h : ((props === null || props === void 0 ? void 0 : props.disabled) ? (_j = props === null || props === void 0 ? void 0 : props.strings) === null || _j === void 0 ? void 0 : _j.tooltipDisabledContent : (props === null || props === void 0 ? void 0 : props.checked) ? (_k = props === null || props === void 0 ? void 0 : props.strings) === null || _k === void 0 ? void 0 : _k.tooltipOnContent : (_l = props === null || props === void 0 ? void 0 : props.strings) === null || _l === void 0 ? void 0 : _l.tooltipOffContent);
    const tooltipId = ((_m = props.tooltipId) !== null && _m !== void 0 ? _m : props.labelKey) ? props.labelKey + '-tooltip' : undefined;
    return React.createElement(ControlButtonTooltip, { hidden: props.disableTooltip, content: tooltipContent, id: tooltipId },
        React.createElement(react.DefaultButton, Object.assign({}, props, { styles: componentStyles, onRenderText: props.showLabel && props.onRenderText ? props.onRenderText : undefined, onRenderIcon: (_o = props.onRenderIcon) !== null && _o !== void 0 ? _o : DefaultRenderIcon, ariaLabel: (_r = (_q = (_p = props.splitButtonAriaLabel) !== null && _p !== void 0 ? _p : props.ariaLabel) !== null && _q !== void 0 ? _q : labelText) !== null && _r !== void 0 ? _r : tooltipContent, allowDisabledFocus: (_s = props.allowDisabledFocus) !== null && _s !== void 0 ? _s : true, "aria-describedby": tooltipId, menuTriggerKeyCode: react.KeyCodes.down, text: undefined }), props.showLabel ? labelText : React.createElement(React.Fragment, null)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * This is a helper component to define and unify icon colors
 *
 * @internal
 */
const _HighContrastAwareIcon = (props) => {
    const { iconName, disabled } = props;
    const theme = useTheme();
    // setting colors for the icons using color from theme, so in dark mode or other accessibility modes, they have pre-defined contrast colors
    // the media query is for when in specific window accessibility mode, change the svg colors
    return React.createElement(react.Icon, { iconName: iconName, className: react.mergeStyles({
            svg: {
                fill: disabled ? theme.palette.neutralTertiary : theme.palette.neutralPrimaryAlt,
                '@media (forced-colors: active) and (prefers-color-scheme: dark)': {
                    fill: disabled ? theme.palette.neutralPrimaryAlt : theme.palette.neutralTertiary
                }
            }
        }) });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const defaultLocalVideoViewOptions$1 = {
    scalingMode: 'Crop',
    isMirrored: true
};
/**
 * Generates default {@link IContextualMenuProps} for buttons that
 * show a drop-down to select devices to use.
 *
 * @internal
 */
const generateDefaultDeviceMenuProps = (props, strings, primaryActionItem, isSelectCamAllowed = true, isSelectMicAllowed = true) => {
    var _a, _b;
    const { microphones, speakers, cameras, selectedMicrophone, selectedSpeaker, selectedCamera, onSelectCamera, onSelectMicrophone, onSelectSpeaker } = props;
    const defaultMenuProps = {
        items: [],
        styles: props.styles,
        calloutProps: {
            styles: {
                root: {
                    // Confine the menu to the parents bounds.
                    // More info: https://github.com/microsoft/fluentui/issues/18835
                    // NB: 95% to keep some space for margin, drop shadow etc around the Callout.
                    maxWidth: '95%'
                }
            },
            preventDismissOnEvent: _preventDismissOnEvent
        }
    };
    const menuItemStyles = react.merge(buttonFlyoutItemStyles, (_b = (_a = props.styles) === null || _a === void 0 ? void 0 : _a.menuItemStyles) !== null && _b !== void 0 ? _b : {});
    if (cameras && selectedCamera && onSelectCamera && isSelectCamAllowed) {
        defaultMenuProps.items.push({
            key: 'cameras',
            itemType: react.ContextualMenuItemType.Section,
            sectionProps: {
                title: strings.cameraMenuTitle,
                items: [{
                        key: 'sectionCamera',
                        title: strings.cameraMenuTooltip,
                        subMenuProps: {
                            calloutProps: {
                                preventDismissOnEvent: _preventDismissOnEvent
                            },
                            items: cameras.map(camera => ({
                                key: camera.id,
                                text: camera.name,
                                title: camera.name,
                                iconProps: {
                                    iconName: 'ContextMenuCameraIcon',
                                    styles: {
                                        root: {
                                            lineHeight: 0
                                        }
                                    }
                                },
                                itemProps: {
                                    styles: menuItemStyles
                                },
                                canCheck: true,
                                isChecked: camera.id === (selectedCamera === null || selectedCamera === void 0 ? void 0 : selectedCamera.id),
                                onClick: () => {
                                    if (camera.id !== (selectedCamera === null || selectedCamera === void 0 ? void 0 : selectedCamera.id)) {
                                        onSelectCamera(camera, defaultLocalVideoViewOptions$1);
                                    }
                                }
                            }))
                        },
                        text: selectedCamera.name
                    }]
            }
        });
        if (primaryActionItem) {
            defaultMenuProps.items.push(primaryActionItem);
        }
    }
    if (microphones && selectedMicrophone && onSelectMicrophone && isSelectMicAllowed) {
        // Set props as Microphone if speakers can be enumerated else set as Audio Device
        const speakersAvailable = speakers && speakers.length > 0;
        const key = speakersAvailable ? 'sectionMicrophone' : 'sectionAudioDevice';
        const title = speakersAvailable ? strings.microphoneMenuTooltip : strings.audioDeviceMenuTooltip;
        defaultMenuProps.items.push({
            key: 'microphones',
            itemType: react.ContextualMenuItemType.Section,
            sectionProps: {
                title: strings.microphoneMenuTitle,
                items: [{
                        key: key,
                        title: title,
                        subMenuProps: {
                            calloutProps: {
                                preventDismissOnEvent: _preventDismissOnEvent
                            },
                            items: microphones.map(microphone => ({
                                key: microphone.id,
                                text: microphone.name,
                                title: microphone.name,
                                itemProps: {
                                    styles: menuItemStyles
                                },
                                iconProps: {
                                    iconName: 'ContextMenuMicIcon',
                                    styles: {
                                        root: {
                                            lineHeight: 0
                                        }
                                    }
                                },
                                canCheck: true,
                                isChecked: microphone.id === (selectedMicrophone === null || selectedMicrophone === void 0 ? void 0 : selectedMicrophone.id),
                                onClick: () => {
                                    if (microphone.id !== (selectedMicrophone === null || selectedMicrophone === void 0 ? void 0 : selectedMicrophone.id)) {
                                        onSelectMicrophone(microphone);
                                    }
                                }
                            }))
                        },
                        text: selectedMicrophone.name
                    }]
            }
        });
    }
    if (speakers && selectedSpeaker && onSelectSpeaker) {
        defaultMenuProps.items.push({
            key: 'speakers',
            itemType: react.ContextualMenuItemType.Section,
            sectionProps: {
                title: strings.speakerMenuTitle,
                items: [{
                        key: 'sectionSpeaker',
                        subMenuProps: {
                            calloutProps: {
                                preventDismissOnEvent: _preventDismissOnEvent
                            },
                            items: speakers.map(speaker => ({
                                key: speaker.id,
                                text: speaker.name,
                                title: speaker.name,
                                itemProps: {
                                    styles: menuItemStyles
                                },
                                iconProps: {
                                    iconName: 'ContextMenuSpeakerIcon',
                                    styles: {
                                        root: {
                                            lineHeight: 0
                                        }
                                    }
                                },
                                canCheck: true,
                                isChecked: speaker.id === (selectedSpeaker === null || selectedSpeaker === void 0 ? void 0 : selectedSpeaker.id),
                                onClick: () => {
                                    if (speaker.id !== (selectedSpeaker === null || selectedSpeaker === void 0 ? void 0 : selectedSpeaker.id)) {
                                        onSelectSpeaker(speaker);
                                    }
                                }
                            }))
                        },
                        text: selectedSpeaker.name
                    }]
            }
        });
    }
    if (microphones && selectedMicrophone && onSelectMicrophone && isSelectMicAllowed && primaryActionItem) {
        defaultMenuProps.items.push(primaryActionItem);
    }
    if (defaultMenuProps.items.length === 0) {
        // Avoids creating an empty context menu.
        return undefined;
    }
    return defaultMenuProps;
};
/**
 * A button to open a menu that controls device options.
 *
 * Can be used with {@link ControlBar}.
 *
 * @public
 */
const DevicesButton = (props) => {
    var _a, _b, _c;
    const { onRenderIcon } = props;
    const localeStrings = useLocale$1().strings.devicesButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const devicesButtonMenu = (_a = props.menuProps) !== null && _a !== void 0 ? _a : generateDefaultDeviceMenuProps(Object.assign(Object.assign({}, props), { styles: (_b = props.styles) === null || _b === void 0 ? void 0 : _b.menuStyles }), strings);
    const onRenderOptionsIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "ControlButtonOptions" });
    };
    return React.createElement(ControlBarButton, Object.assign({}, props, { menuProps: devicesButtonMenu, menuIconProps: {
            hidden: true
        }, onRenderIcon: onRenderIcon !== null && onRenderIcon !== void 0 ? onRenderIcon : onRenderOptionsIcon, strings: strings, labelKey: (_c = props.labelKey) !== null && _c !== void 0 ? _c : 'devicesButtonLabel' }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$H = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const defaultLocalVideoViewOptions = {
    scalingMode: 'Crop',
    isMirrored: true
};
/**
 * A button to turn camera on / off.
 *
 * Can be used with {@link ControlBar}.
 *
 * @public
 */
const CameraButton = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const { localVideoViewOptions, onToggleCamera, onSelectCamera } = props;
    const [waitForCamera, setWaitForCamera] = React.useState(false);
    const localeStrings = useLocale$1().strings.cameraButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const [announcerString, setAnnouncerString] = React.useState(undefined);
    const [announcerPresent, setAnnouncerPresent] = React.useState(false);
    const disabled = props.disabled || waitForCamera;
    const onRenderCameraOnIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: disabled, iconName: "ControlButtonCameraOn" });
    const onRenderCameraOffIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: disabled, iconName: "ControlButtonCameraOff" });
    if (waitForCamera && strings.tooltipVideoLoadingContent) {
        strings.tooltipDisabledContent = strings.tooltipVideoLoadingContent;
    }
    const cameraOn = props.checked;
    const cameraTurningOn = waitForCamera && !cameraOn;
    // Ensure Aria-label by default reflects the loading state of the camera
    const ariaLabel = ((_a = props.ariaLabel) !== null && _a !== void 0 ? _a : cameraTurningOn) ? strings.tooltipVideoLoadingContent : undefined;
    const splitButtonAriaString = cameraTurningOn ? strings.tooltipVideoLoadingContent : cameraOn ? strings.onSplitButtonAriaLabel : strings.offSplitButtonAriaLabel;
    const toggleAnnouncerString = React.useCallback((isCameraOn) => {
        setAnnouncerString(!isCameraOn ? strings.cameraActionTurnedOffAnnouncement : strings.cameraActionTurnedOnAnnouncement);
    }, [strings.cameraActionTurnedOffAnnouncement, strings.cameraActionTurnedOnAnnouncement]);
    const onVideoIsLoadingAnnouncementCallback = React.useCallback(() => {
        if (!cameraOn) {
            setAnnouncerString(strings.tooltipVideoLoadingContent);
        }
    }, [setAnnouncerString, cameraOn, strings.tooltipVideoLoadingContent]);
    const onToggleClick = React.useCallback(() => __awaiter$H(void 0, void 0, void 0, function* () {
        // Throttle click on camera, need to await onToggleCamera then allow another click
        if (onToggleCamera) {
            setWaitForCamera(true);
            try {
                yield onToggleCamera(localVideoViewOptions !== null && localVideoViewOptions !== void 0 ? localVideoViewOptions : defaultLocalVideoViewOptions);
                // allows for the setting of narrator strings triggering the announcer when camera is turned on or off.
                toggleAnnouncerString(!cameraOn);
            }
            finally {
                setWaitForCamera(false);
            }
        }
    }), [cameraOn, localVideoViewOptions, onToggleCamera, toggleAnnouncerString]);
    const onToggleClickCallback = React.useCallback(() => {
        onVideoIsLoadingAnnouncementCallback();
        onToggleClick();
    }, [onVideoIsLoadingAnnouncementCallback, onToggleClick]);
    const onChangeCameraClick = React.useCallback((device) => __awaiter$H(void 0, void 0, void 0, function* () {
        // Throttle changing camera to prevent too many callbacks
        if (onSelectCamera) {
            setWaitForCamera(true);
            try {
                yield onSelectCamera(device);
            }
            finally {
                setWaitForCamera(false);
            }
        }
    }), [onSelectCamera]);
    const splitButtonMenuItems = [];
    if (props.onClickVideoEffects) {
        splitButtonMenuItems.push({
            key: 'effects',
            'data-ui-id': 'camera-split-button-video-effects',
            text: strings.videoEffectsMenuItemTitle,
            iconProps: {
                iconName: 'ControlButtonVideoEffectsOption',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            onClick: () => {
                if (props.onClickVideoEffects) {
                    props.onClickVideoEffects(true);
                }
            }
        });
    }
    splitButtonMenuItems.push({
        key: 'cameraPrimaryAction',
        text: props.checked ? strings.onSplitButtonPrimaryActionCamera : strings.offSplitButtonPrimaryActionCamera,
        onClick: () => {
            onToggleClick();
        },
        iconProps: {
            iconName: props.checked ? 'SplitButtonPrimaryActionCameraOn' : 'SplitButtonPrimaryActionCameraOff',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        }
    });
    const splitButtonPrimaryAction = {
        key: 'primaryAction',
        title: 'toggle camera',
        itemType: react.ContextualMenuItemType.Section,
        sectionProps: {
            topDivider: true,
            items: splitButtonMenuItems
        }
    };
    const splitButtonMenuProps = React.useMemo(() => (Object.assign(Object.assign({}, props.splitButtonMenuProps), { className: 'camera-split-button' })), [props.splitButtonMenuProps]);
    return React.createElement(React.Fragment, null,
        announcerPresent && React.createElement(Announcer$1, { announcementString: announcerString, ariaLive: 'polite' }),
        React.createElement(ControlBarButton, Object.assign({}, props, { disabled: disabled, onClick: onToggleCamera ? onToggleClickCallback : props.onClick, onRenderOnIcon: (_b = props.onRenderOnIcon) !== null && _b !== void 0 ? _b : onRenderCameraOnIcon, onRenderOffIcon: (_c = props.onRenderOffIcon) !== null && _c !== void 0 ? _c : onRenderCameraOffIcon, strings: strings, labelKey: (_d = props.labelKey) !== null && _d !== void 0 ? _d : 'cameraButtonLabel', menuProps: (_e = props.menuProps) !== null && _e !== void 0 ? _e : (props.enableDeviceSelectionMenu ? generateDefaultDeviceMenuProps(Object.assign(Object.assign({}, props), { onSelectCamera: onChangeCameraClick, styles: (_f = props.styles) === null || _f === void 0 ? void 0 : _f.menuStyles }), strings, splitButtonPrimaryAction) : undefined), menuIconProps: ((_g = props.menuIconProps) !== null && _g !== void 0 ? _g : !props.enableDeviceSelectionMenu) ? {
                hidden: true
            } : undefined, split: (_h = props.split) !== null && _h !== void 0 ? _h : props.enableDeviceSelectionMenu, "aria-description": strings.cameraButtonAriaDescription, "aria-roledescription": props.enableDeviceSelectionMenu ? strings.cameraButtonSplitRoleDescription : undefined, ariaLabel: ariaLabel, splitButtonAriaLabel: props.enableDeviceSelectionMenu ? splitButtonAriaString : undefined, splitButtonMenuProps: splitButtonMenuProps, onFocus: () => setAnnouncerPresent(true), onBlur: () => setAnnouncerPresent(false) })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const mainDivStyle = {
    position: 'relative',
    height: '100%',
    width: '100%'
};
/**
 * A container for various buttons for call controls.
 *
 * Use with various call control buttons in this library, e.g., {@link CameraButton}, or your own instances of
 * {@link ControlBarButton} directly.
 *
 * @public
 */
const ControlBar = (props) => {
    const { styles, layout } = props;
    const theme = useTheme();
    const controlBarClassName = React.useMemo(() => {
        const controlBarStyle = controlBarStyles$1[layout !== null && layout !== void 0 ? layout : 'horizontal'];
        // if theme is dark and layout is floating then use palette.neutralQuaternaryAlt as background, otherwise use theme.palette.white
        const backgroundStyle = {
            background: isDarkThemed(theme) && (layout === null || layout === void 0 ? void 0 : layout.startsWith('floating')) ? theme.palette.neutralQuaternaryAlt : theme.palette.white
        };
        const borderAndBoxShadowStyle = (layout === null || layout === void 0 ? void 0 : layout.startsWith('floating')) ? {
            boxShadow: theme.effects.elevation16,
            borderRadius: theme.effects.roundedCorner6
        } : {};
        // if rtl is true and layout is either floatingTop or floatingBottom then we need to override the transform-style property
        // to translate 50% to right instead of the left
        const transformOverrideStyle = theme.rtl && (layout === 'floatingTop' || layout === 'floatingBottom') ? {
            transform: 'translateX(50%)'
        } : {};
        return react.mergeStyles(controlBarStyle, backgroundStyle, borderAndBoxShadowStyle, transformOverrideStyle, styles === null || styles === void 0 ? void 0 : styles.root);
    }, [layout, styles === null || styles === void 0 ? void 0 : styles.root, theme]);
    return React.createElement("div", { className: react.mergeStyles(mainDivStyle) },
        React.createElement(react.Stack, { className: controlBarClassName }, props.children));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const spacerClass = react.mergeStyles({
    width: 4
});
/**
 * A button to end an ongoing call.
 *
 * Can be used with {@link ControlBar}.
 *
 * @public
 */
const EndCallButton = (props) => {
    var _a, _b;
    const { styles, enableEndCallMenu, onHangUp } = props;
    const localeStrings = useLocale$1().strings.endCallButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const theme = useTheme();
    const isDarkTheme = isDarkThemed(theme);
    const componentStyles = react.concatStyleSets(isDarkTheme ? darkThemeCallButtonStyles : lightThemeCallButtonStyles, enableEndCallMenu ? menupButtonPadding : {}, styles !== null && styles !== void 0 ? styles : {});
    const onRenderEndCallIcon = React.useCallback(() => React.createElement(React.Fragment, null,
        React.createElement(react.Icon, { iconName: "ControlButtonEndCall" }),
        enableEndCallMenu && React.createElement("span", { className: spacerClass })), [enableEndCallMenu]);
    const defaultMenuProps = {
        items: [{
                key: 'endForSelf',
                text: localeStrings.leaveOption,
                title: localeStrings.leaveOption,
                onClick: () => {
                    onHangUp && onHangUp(false);
                }
            }, {
                key: 'endForEveryone',
                text: localeStrings.endCallOption,
                title: localeStrings.endCallOption,
                onClick: () => {
                    onHangUp && onHangUp(true);
                }
            }],
        styles: props.styles,
        calloutProps: {
            styles: {
                root: {
                    maxWidth: '95%'
                }
            },
            preventDismissOnEvent: _preventDismissOnEvent
        }
    };
    return React.createElement(React.Fragment, null,
        React.createElement(ControlBarButton, Object.assign({}, props, { menuProps: enableEndCallMenu ? defaultMenuProps : props.menuProps, onClick: onHangUp ? () => onHangUp() : props.onClick, styles: componentStyles, onRenderIcon: (_a = props.onRenderIcon) !== null && _a !== void 0 ? _a : onRenderEndCallIcon, strings: strings, labelKey: (_b = props.labelKey) !== null && _b !== void 0 ? _b : 'endCallButtonLabel' })));
};
const getButtonStyles = (theme) => {
    var _a;
    return ({
        regular: {
            color: theme.callingPalette.iconWhite,
            background: theme.callingPalette.callRed,
            '@media (forced-colors: active)': {
                forcedColorAdjust: 'auto',
                border: `1px ${(_a = theme.palette) === null || _a === void 0 ? void 0 : _a.neutralQuaternaryAlt} solid`
            },
            ' i': {
                color: theme.callingPalette.iconWhite
            },
            ':focus::after': {
                outlineColor: `${theme.callingPalette.iconWhite} !important`
            } // added !important to avoid override by FluentUI button styles
        },
        pressed: {
            color: theme.callingPalette.iconWhite,
            background: theme.callingPalette.callRedDarker,
            border: 'none',
            ' i': {
                color: theme.callingPalette.iconWhite
            },
            '@media (forced-colors: active)': {
                forcedColorAdjust: 'auto'
            }
        },
        hovered: {
            color: theme.callingPalette.iconWhite,
            background: theme.callingPalette.callRedDark,
            border: 'none',
            ' i': {
                color: theme.callingPalette.iconWhite
            },
            '@media (forced-colors: active)': {
                forcedColorAdjust: 'auto'
            }
        }
    });
};
const darkThemeButtonStyles = getButtonStyles(darkTheme);
const lightThemeButtonStyles = getButtonStyles(lightTheme);
// using media query to prevent windows from overwriting the button color
const darkThemeCallButtonStyles = {
    root: darkThemeButtonStyles.regular,
    rootHovered: darkThemeButtonStyles.hovered,
    rootPressed: darkThemeButtonStyles.pressed,
    label: {
        color: darkTheme.callingPalette.iconWhite
    },
    splitButtonMenuButton: Object.assign(Object.assign({}, darkThemeButtonStyles.regular), { borderTop: 'none', borderRight: 'none', borderBottom: 'none', '&:hover': darkThemeButtonStyles.hovered }),
    splitButtonMenuButtonChecked: darkThemeButtonStyles.hovered,
    splitButtonMenuButtonExpanded: darkThemeButtonStyles.pressed
};
const lightThemeCallButtonStyles = {
    root: lightThemeButtonStyles.regular,
    rootHovered: lightThemeButtonStyles.hovered,
    rootPressed: lightThemeButtonStyles.pressed,
    splitButtonMenuButton: Object.assign(Object.assign({}, lightThemeButtonStyles.regular), { borderTop: 'none', borderRight: 'none', borderBottom: 'none', '&:hover': lightThemeButtonStyles.hovered }),
    splitButtonMenuButtonChecked: lightThemeButtonStyles.hovered,
    splitButtonMenuButtonExpanded: lightThemeButtonStyles.pressed,
    splitButtonMenuFocused: lightThemeButtonStyles.pressed,
    label: {
        color: lightTheme.callingPalette.iconWhite
    }
};
const menupButtonPadding = {
    root: {
        padding: '0 0.75rem'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$G = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * A button to turn microphone on / off.
 *
 * Can be used with {@link ControlBar}.
 *
 * @public
 */
const MicrophoneButton = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    const { onToggleMicrophone } = props;
    const localeStrings = useLocale$1().strings.microphoneButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const [announcerString, setAnnouncerString] = React.useState(undefined);
    const [announcerPresent, setAnnouncerPresent] = React.useState(false);
    const isSplit = (_a = props.split) !== null && _a !== void 0 ? _a : props.enableDeviceSelectionMenu;
    // The button should be disabled when there are no mics. However if the button is a split button, if there are
    // no mics but there are speakers, then only the primary part of the button should be disabled to allow for
    // speaker change.
    const primaryDisabled = props.primaryDisabled || (isSplit && !((_b = props.microphones) === null || _b === void 0 ? void 0 : _b.length) ? true : undefined);
    const disabled = props.disabled || isSplit && !((_c = props.microphones) === null || _c === void 0 ? void 0 : _c.length) && !((_d = props.speakers) === null || _d === void 0 ? void 0 : _d.length) || !isSplit && props.microphones && ((_e = props.microphones) === null || _e === void 0 ? void 0 : _e.length) === 0;
    const onRenderMicOnIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: disabled, iconName: "ControlButtonMicOn" });
    };
    const onRenderMicOffIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: disabled, iconName: "ControlButtonMicOff" });
    };
    const isMicOn = props.checked;
    const splitButtonAriaString = isMicOn ? strings.onSplitButtonAriaLabel : strings.offSplitButtonAriaLabel;
    const toggleAnnouncerString = React.useCallback((isMicOn) => {
        setAnnouncerString(!isMicOn ? strings.microphoneActionTurnedOffAnnouncement : strings.microphoneActionTurnedOnAnnouncement);
    }, [strings.microphoneActionTurnedOffAnnouncement, strings.microphoneActionTurnedOnAnnouncement]);
    const onToggleClick = React.useCallback(() => __awaiter$G(void 0, void 0, void 0, function* () {
        if (onToggleMicrophone) {
            try {
                yield onToggleMicrophone();
                // allows for the setting of narrator strings triggering the announcer when microphone is turned on or off.
                toggleAnnouncerString(!isMicOn);
                // eslint-disable-next-line no-empty
            }
            finally { }
        }
    }), [isMicOn, onToggleMicrophone, toggleAnnouncerString]);
    const deepNoiseSuppressionToggleStyles = {
        root: {
            margin: '4px',
            padding: '0px 12px',
            flexFlow: 'row-reverse',
            justifyContent: 'space-between'
        },
        label: {
            fontWeight: 400
        }
    };
    const splitButtonMenuItems = [];
    if (props.showNoiseSuppressionButton) {
        splitButtonMenuItems.push({
            key: 'microphoneDNSToggle',
            onRender: () => {
                return React.createElement(react.Stack, { onClick: () => __awaiter$G(void 0, void 0, void 0, function* () {
                        var _a;
                        yield ((_a = props.onClickNoiseSuppression) === null || _a === void 0 ? void 0 : _a.call(props));
                        setAnnouncerString(props.isDeepNoiseSuppressionOn ? strings.deepNoiseSuppressionOnAnnouncement : strings.deepNoiseSuppressionOffAnnouncement);
                    }) },
                    React.createElement(react.Toggle, { label: strings.deepNoiseSuppressionTitle, checked: props.isDeepNoiseSuppressionOn, inlineLabel: true, styles: deepNoiseSuppressionToggleStyles }));
            }
        });
    }
    splitButtonMenuItems.push({
        key: 'microphonePrimaryAction',
        text: props.checked ? strings.onSplitButtonMicrophonePrimaryAction : strings.offSplitButtonMicrophonePrimaryAction,
        onClick: () => {
            onToggleClick();
        },
        iconProps: {
            iconName: props.checked ? 'SplitButtonPrimaryActionMicUnmuted' : 'SplitButtonPrimaryActionMicMuted',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        }
    });
    /**
     * We need to also include the primary action of the button to the
     * split button for mobile devices.
     */
    const splitButtonPrimaryAction = {
        key: 'primaryAction',
        title: 'toggle mic',
        itemType: react.ContextualMenuItemType.Section,
        sectionProps: {
            topDivider: true,
            items: splitButtonMenuItems
        }
    };
    return React.createElement(React.Fragment, null,
        announcerPresent && React.createElement(Announcer$1, { announcementString: announcerString, ariaLive: 'polite' }),
        React.createElement(ControlBarButton, Object.assign({}, props, { onClick: props.onToggleMicrophone ? onToggleClick : props.onClick, onRenderOnIcon: (_f = props.onRenderOnIcon) !== null && _f !== void 0 ? _f : onRenderMicOnIcon, onRenderOffIcon: (_g = props.onRenderOffIcon) !== null && _g !== void 0 ? _g : onRenderMicOffIcon, strings: strings, labelKey: (_h = props.labelKey) !== null && _h !== void 0 ? _h : 'microphoneButtonLabel', menuProps: (_j = props.menuProps) !== null && _j !== void 0 ? _j : (props.enableDeviceSelectionMenu ? generateDefaultDeviceMenuProps(Object.assign(Object.assign({}, props), { styles: (_k = props.styles) === null || _k === void 0 ? void 0 : _k.menuStyles }), strings, splitButtonPrimaryAction) : undefined), menuIconProps: ((_l = props.menuIconProps) !== null && _l !== void 0 ? _l : !props.enableDeviceSelectionMenu) ? {
                hidden: true
            } : undefined, split: (_m = props.split) !== null && _m !== void 0 ? _m : props.enableDeviceSelectionMenu, "aria-roledescription": props.enableDeviceSelectionMenu ? strings.microphoneButtonSplitRoleDescription : undefined, "aria-description": strings.microphoneAriaDescription, splitButtonAriaLabel: props.enableDeviceSelectionMenu ? splitButtonAriaString : undefined, disabled: disabled, primaryDisabled: primaryDisabled, onFocus: () => setAnnouncerPresent(true), onBlur: () => setAnnouncerPresent(false) })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A button to show a menu with calling or chat participants.
 *
 * Can be used with {@link ControlBar}.
 *
 * This button contains dropdown menu items defined through its property `menuProps`. By default, it can display the number of remote participants with the full list
 * as sub-menu and an option to mute all participants, as well as a copy-to-clipboard button to copy the call invitation URL.
 * This `menuProps` can be fully redefined and its property is of type [IContextualMenuProps](https://developer.microsoft.com/fluentui#/controls/web/contextualmenu#IContextualMenuProps).
 *
 * @public
 */
const ParticipantsButton = (props) => {
    var _a, _b, _c, _d;
    const { callInvitationURL, styles, onMuteAll, onRenderIcon, onRenderParticipantList, participants, myUserId, excludeMe, onRenderParticipant, onRenderAvatar, onRemoveParticipant, onFetchParticipantMenuItems, showParticipantOverflowTooltip } = props;
    const disabled = props.disabled;
    const [copyInviteLinkAnnouncerStrings, setCopyInviteLinkAnnouncerStrings] = React.useState('');
    const onRenderPeopleIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: disabled, iconName: "ControlButtonParticipants" });
    const ids = useIdentifiers();
    const onMuteAllCallback = React.useCallback(() => {
        if (onMuteAll) {
            onMuteAll();
        }
    }, [onMuteAll]);
    const defaultParticipantList = React.useCallback(() => {
        var _a;
        return React.createElement(ParticipantList$n, { participants: participants, myUserId: myUserId, excludeMe: excludeMe, onRenderParticipant: onRenderParticipant, onRenderAvatar: onRenderAvatar, onRemoveParticipant: onRemoveParticipant, onFetchParticipantMenuItems: onFetchParticipantMenuItems, styles: (_a = styles === null || styles === void 0 ? void 0 : styles.menuStyles) === null || _a === void 0 ? void 0 : _a.participantListStyles, showParticipantOverflowTooltip: showParticipantOverflowTooltip });
    }, [excludeMe, myUserId, onRemoveParticipant, onRenderAvatar, onRenderParticipant, participants, (_a = styles === null || styles === void 0 ? void 0 : styles.menuStyles) === null || _a === void 0 ? void 0 : _a.participantListStyles, onFetchParticipantMenuItems, showParticipantOverflowTooltip]);
    const onCopyCallback = React.useCallback(() => {
        if (callInvitationURL) {
            return copy(callInvitationURL);
        }
        return false;
    }, [callInvitationURL]);
    const localeStrings = useLocale$1().strings.participantsButton;
    const strings = React.useMemo(() => (Object.assign(Object.assign({}, localeStrings), props.strings)), [localeStrings, props.strings]);
    const participantCount = participants.length;
    /**
     * sets the announcement string for when the link is copied.
     */
    const toggleAnnouncerString = React.useCallback(() => {
        setCopyInviteLinkAnnouncerStrings(strings.copyInviteLinkActionedAriaLabel);
        /**
         * Clears the announcer string after the user clicks the
         * copyInviteLink button allowing it to be re-announced.
         */
        setTimeout(() => {
            setCopyInviteLinkAnnouncerStrings('');
        }, 3000);
    }, [strings.copyInviteLinkActionedAriaLabel]);
    const generateDefaultParticipantsSubMenuProps = React.useCallback(() => {
        var _a;
        const items = [];
        if (participantCount > 0) {
            items.push({
                key: 'participantListMenuItemKey',
                onRender: onRenderParticipantList !== null && onRenderParticipantList !== void 0 ? onRenderParticipantList : defaultParticipantList
            });
            items.push({
                key: 'participantsDivider1',
                itemType: react.ContextualMenuItemType.Divider
            });
            if (onMuteAll) {
                items.push({
                    key: 'muteAllKey',
                    text: strings.muteAllButtonLabel,
                    title: strings.muteAllButtonLabel,
                    styles: (_a = styles === null || styles === void 0 ? void 0 : styles.menuStyles) === null || _a === void 0 ? void 0 : _a.menuItemStyles,
                    iconProps: {
                        iconName: 'MicOff2'
                    },
                    onClick: onMuteAllCallback
                });
            }
        }
        return items;
    }, [participantCount, onRenderParticipantList, defaultParticipantList, onMuteAll, strings.muteAllButtonLabel, (_b = styles === null || styles === void 0 ? void 0 : styles.menuStyles) === null || _b === void 0 ? void 0 : _b.menuItemStyles, onMuteAllCallback]);
    const defaultMenuProps = React.useMemo(() => {
        var _a, _b;
        const menuProps = {
            title: strings.menuHeader,
            ariaLabel: strings.menuHeader,
            styles: react.merge(participantsButtonMenuPropsStyle, styles === null || styles === void 0 ? void 0 : styles.menuStyles),
            items: [],
            calloutProps: {
                preventDismissOnEvent: _preventDismissOnEvent
            }
        };
        if (participantCount > 0) {
            const participantIds = participants.map(p => p.userId);
            let participantCountWithoutMe = participantIds.length;
            if (excludeMe) {
                participantCountWithoutMe -= 1;
            }
            menuProps.items.push({
                key: 'participantCountKey',
                name: _formatString(strings.participantsListButtonLabel, {
                    numParticipants: `${participantCountWithoutMe}`
                }),
                itemProps: {
                    styles: (_a = styles === null || styles === void 0 ? void 0 : styles.menuStyles) === null || _a === void 0 ? void 0 : _a.menuItemStyles
                },
                iconProps: {
                    iconName: 'People'
                },
                subMenuProps: {
                    items: generateDefaultParticipantsSubMenuProps(),
                    calloutProps: {
                        styles: {
                            root: {
                                // Confine the menu to the parents bounds.
                                // More info: https://github.com/microsoft/fluentui/issues/18835
                                maxWidth: '100%'
                            }
                        },
                        style: {
                            maxHeight: '20rem'
                        },
                        // Disable dismiss on resize to work around a couple Fluent UI bugs
                        // See reasoning in the props for the parent menu.
                        preventDismissOnEvent: _preventDismissOnEvent
                    }
                },
                'data-ui-id': ids.participantButtonPeopleMenuItem
            });
        }
        if (callInvitationURL) {
            menuProps.items.push({
                key: 'InviteLinkKey',
                name: strings.copyInviteLinkButtonLabel,
                title: strings.copyInviteLinkButtonLabel,
                itemProps: {
                    styles: (_b = styles === null || styles === void 0 ? void 0 : styles.menuStyles) === null || _b === void 0 ? void 0 : _b.menuItemStyles
                },
                iconProps: {
                    iconName: 'Link'
                },
                onClick: () => {
                    onCopyCallback();
                    toggleAnnouncerString();
                }
            });
        }
        return menuProps;
    }, [strings.menuHeader, strings.participantsListButtonLabel, strings.copyInviteLinkButtonLabel, styles === null || styles === void 0 ? void 0 : styles.menuStyles, participantCount, callInvitationURL, participants, excludeMe, ids.participantButtonPeopleMenuItem, generateDefaultParticipantsSubMenuProps, onCopyCallback, toggleAnnouncerString]);
    return React.createElement(React.Fragment, null,
        React.createElement(Announcer$1, { announcementString: copyInviteLinkAnnouncerStrings, ariaLive: 'polite' }),
        React.createElement(ControlBarButton, Object.assign({}, props, { disabled: disabled, menuProps: (_c = props.menuProps) !== null && _c !== void 0 ? _c : defaultMenuProps, menuIconProps: {
                hidden: true
            }, onRenderIcon: onRenderIcon !== null && onRenderIcon !== void 0 ? onRenderIcon : onRenderPeopleIcon, strings: strings, "aria-label": strings.ariaLabel, labelKey: (_d = props.labelKey) !== null && _d !== void 0 ? _d : 'participantsButtonLabel' })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A button to start / stop screen sharing.
 *
 * Can be used with {@link ControlBar}.
 *
 * @public
 */
const ScreenShareButton = (props) => {
    var _a, _b, _c, _d;
    const localeStrings = useLocale$1().strings.screenShareButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const theme = react.useTheme();
    const styles = screenshareButtonStyles(theme);
    const onRenderScreenShareOnIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "ControlButtonScreenShareStop" });
    const onRenderScreenShareOffIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "ControlButtonScreenShareStart" });
    return React.createElement(ControlBarButton, Object.assign({}, props, { styles: react.mergeStyleSets(styles, props.styles), onClick: (_a = props.onToggleScreenShare) !== null && _a !== void 0 ? _a : props.onClick, onRenderOnIcon: (_b = props.onRenderOnIcon) !== null && _b !== void 0 ? _b : onRenderScreenShareOnIcon, onRenderOffIcon: (_c = props.onRenderOffIcon) !== null && _c !== void 0 ? _c : onRenderScreenShareOffIcon, strings: strings, labelKey: (_d = props.labelKey) !== null && _d !== void 0 ? _d : 'screenShareButtonLabel', disabled: props.disabled }));
};
const screenshareButtonStyles = (theme) => ({
    rootChecked: {
        background: theme.semanticColors.primaryButtonBackground,
        color: theme.semanticColors.primaryButtonText,
        ':focus::after': {
            outlineColor: `${react.DefaultPalette.white} !important`
        },
        // added !important to avoid override by FluentUI button styles
        '@media (forced-colors: active)': {
            border: '1px solid',
            borderColor: theme.palette.black
        }
    },
    rootCheckedHovered: {
        background: theme.semanticColors.primaryButtonBackgroundHovered,
        color: theme.semanticColors.primaryButtonTextHovered,
        ':focus::after': {
            outlineColor: `${react.DefaultPalette.white} !important`
        },
        // added !important to avoid override by FluentUI button styles
        '@media (forced-colors: active)': {
            border: '1px solid',
            borderColor: theme.palette.black
        }
    },
    rootCheckedPressed: {
        background: theme.semanticColors.primaryButtonBackgroundPressed,
        color: theme.semanticColors.primaryButtonTextPressed
    },
    labelChecked: {
        color: theme.semanticColors.primaryButtonText
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A button to start / stop screen sharing.
 *
 * Can be used with {@link ControlBar}.
 *
 * @public
 */
const RaiseHandButton = (props) => {
    var _a, _b, _c, _d;
    const localeStrings = useLocale$1().strings.raiseHandButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const theme = react.useTheme();
    const styles = raiseHandButtonStyles(theme);
    const onRenderRaiseHandIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "ControlButtonRaiseHand" });
    const onRenderLowerHandIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "ControlButtonLowerHand" });
    return React.createElement(ControlBarButton, Object.assign({}, props, { styles: react.concatStyleSets(styles, props.styles), onClick: (_a = props.onToggleRaiseHand) !== null && _a !== void 0 ? _a : props.onClick, onRenderOnIcon: (_b = props.onRenderOnIcon) !== null && _b !== void 0 ? _b : onRenderLowerHandIcon, onRenderOffIcon: (_c = props.onRenderOffIcon) !== null && _c !== void 0 ? _c : onRenderRaiseHandIcon, strings: strings, labelKey: (_d = props.labelKey) !== null && _d !== void 0 ? _d : 'raiseHandButtonLabel', "aria-label": props.checked ? strings.onLabel : strings.offLabel, "aria-description": props.checked ? strings.tooltipOnContent : strings.tooltipOffContent, disabled: props.disabled }));
};
const raiseHandButtonStyles = (theme) => ({
    rootChecked: {
        background: theme.semanticColors.primaryButtonBackground,
        color: theme.semanticColors.primaryButtonText,
        ':focus::after': {
            outlineColor: `${react.DefaultPalette.white} !important`
        },
        // added !important to avoid override by FluentUI button styles
        '@media (forced-colors: active)': {
            border: '1px solid',
            borderColor: theme.palette.black
        }
    },
    rootCheckedHovered: {
        background: theme.semanticColors.primaryButtonBackgroundHovered,
        color: theme.semanticColors.primaryButtonTextHovered,
        ':focus::after': {
            outlineColor: `${react.DefaultPalette.white} !important`
        },
        // added !important to avoid override by FluentUI button styles
        '@media (forced-colors: active)': {
            border: '1px solid',
            borderColor: theme.palette.black
        }
    },
    rootCheckedPressed: {
        background: theme.semanticColors.primaryButtonBackgroundPressed,
        color: theme.semanticColors.primaryButtonTextPressed
    },
    labelChecked: {
        color: theme.semanticColors.primaryButtonText
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A button to send reactions.
 *
 * Can be used with {@link ControlBar}.
 *
 * @public
 */
const ReactionButton = (props) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const localeStrings = useLocale$1().strings.reactionButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const theme = react.useTheme();
    const styles = reactionButtonStyles(theme);
    const onRenderIcon = () => React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "ReactionButtonIcon" });
    const emojis = ['like', 'heart', 'applause', 'laugh', 'surprised'];
    const emojiButtonTooltip = new Map([['like', strings.likeReactionTooltipContent], ['heart', strings.heartReactionTooltipContent], ['applause', strings.applauseReactionTooltipContent], ['laugh', strings.laughReactionTooltipContent], ['surprised', strings.surprisedReactionTooltipContent]]);
    const emojiResource = new Map([['like', (_a = props.reactionResources.likeReaction) === null || _a === void 0 ? void 0 : _a.url], ['heart', (_b = props.reactionResources.heartReaction) === null || _b === void 0 ? void 0 : _b.url], ['applause', (_c = props.reactionResources.applauseReaction) === null || _c === void 0 ? void 0 : _c.url], ['laugh', (_d = props.reactionResources.laughReaction) === null || _d === void 0 ? void 0 : _d.url], ['surprised', (_e = props.reactionResources.surprisedReaction) === null || _e === void 0 ? void 0 : _e.url]]);
    const calloutStyle = {
        root: {
            padding: 0
        },
        calloutMain: {
            padding: '0.5rem'
        }
    };
    const calloutProps = {
        styles: calloutStyle,
        backgroundColor: isDarkThemed(theme) ? theme.palette.neutralLighter : ''
    };
    const reactionButtonCalloutRef = React.useRef(null);
    const reactionButtonRef = React.useRef(null);
    const [calloutIsVisible, setCalloutIsVisible] = React.useState(false);
    return React.createElement(react.Stack, null,
        calloutIsVisible && React.createElement(react.Callout, { "data-ui-id": "reaction-sub-menu", isBeakVisible: false, styles: reactionButtonCalloutStyles, target: reactionButtonCalloutRef.current, onDismiss: () => {
                var _a;
                (_a = reactionButtonRef.current) === null || _a === void 0 ? void 0 : _a.focus();
                setCalloutIsVisible(false);
            } },
            React.createElement(react.FocusZone, { shouldFocusOnMount: true, style: {
                    height: '100%'
                } },
                React.createElement(react.Stack, { horizontal: true, style: {
                        height: 'inherit'
                    } }, emojis.map((emoji, index) => {
                    const resourceUrl = emojiResource.get(emoji);
                    const frameCount = props.reactionResources !== undefined ? getEmojiFrameCount(emoji, props.reactionResources) : 0;
                    const classname = react.mergeStyles(emojiStyles(resourceUrl ? resourceUrl : '', frameCount));
                    return React.createElement(react.TooltipHost, { key: index, "data-ui-id": index, hidden: props.disableTooltip, content: emojiButtonTooltip.get(emoji), styles: reactionToolTipHostStyle(), calloutProps: Object.assign({}, calloutProps) },
                        React.createElement(react.DefaultButton, { role: "menuitem", key: index, onClick: () => {
                                var _a;
                                props.onReactionClick(emoji);
                                (_a = reactionButtonRef.current) === null || _a === void 0 ? void 0 : _a.focus();
                                setCalloutIsVisible(false);
                            }, className: classname, styles: reactionItemButtonStyles, "aria-label": emojiButtonTooltip.get(emoji) }));
                })))),
        React.createElement("div", { ref: reactionButtonCalloutRef },
            React.createElement(ControlBarButton, Object.assign({}, props, { componentRef: reactionButtonRef, styles: react.concatStyleSets(styles, props.styles), onClick: () => setCalloutIsVisible(!calloutIsVisible), onRenderIcon: (_f = props.onRenderIcon) !== null && _f !== void 0 ? _f : onRenderIcon, strings: strings, split: true, labelKey: (_g = props.labelKey) !== null && _g !== void 0 ? _g : 'reactionButtonLabel', onRenderMenuIcon: () => React.createElement("div", null), disabled: props.disabled, ariaLabel: strings.ariaLabel }))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const PictureInPictureInPicturePrimaryTile = (props) => {
    const boxShadow = useTheme().effects.elevation8;
    return React.createElement(PictureInPictureInPictureTile, { styles: primaryTileStyles(props.orientation, boxShadow) }, props.children);
};
/** @private */
const PictureInPictureInPictureSecondaryTile = (props) => React.createElement(PictureInPictureInPictureTile, { styles: secondaryTileStyles(props.orientation) }, props.children);
const PictureInPictureInPictureTile = (props) => React.createElement(react.Stack, { styles: props.styles }, props.children);
const primaryTileStyles = react.memoizeFunction((orientation, themeElevation) => ({
    root: {
        borderRadius: '0.25rem',
        height: orientation === 'landscape' ? '5.5rem' : '8rem',
        width: orientation === 'landscape' ? '8rem' : '5.5rem',
        boxShadow: themeElevation,
        // PiPiP tiles were not designed to host scrollable content. If removed check no scrollbar is showing in e2e tests
        overflow: 'hidden'
    }
}));
const secondaryTileStyles = react.memoizeFunction((orientation) => ({
    root: {
        borderRadius: '0.25rem',
        height: orientation === 'landscape' ? '1.625rem' : '2rem',
        width: orientation === 'landscape' ? '2rem' : '1.625rem',
        // PiPiP tiles were not designed to host scrollable content. If removed check no scrollbar is showing in e2e tests
        overflow: 'hidden'
    }
}));

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Component that displays a video feed for use as a Picture-in-Picture style component.
 * It contains a secondary video feed resulting in an inner Picture-in-Picture style feed.
 *
 * @remarks
 * The double nature of the Picture-in-Picture styles is where this component gets its name; Picture-in-Picture-in-Picture.
 *
 * @internal
 */
const _PictureInPictureInPicture = (props) => {
    return React.createElement(PictureInPictureInPictureContainer, { onClick: props.onClick, primaryView: React.createElement(PictureInPictureInPicturePrimaryTile, Object.assign({}, props.primaryTile)), secondaryView: props.secondaryTile && React.createElement(PictureInPictureInPictureSecondaryTile, Object.assign({}, props.secondaryTile)), ariaLabel: props.strings.rootAriaLabel });
};
/**
 * Container for the picture in picture in picture component.
 * This governs positioning and floating of the secondary PiP.
 */
const PictureInPictureInPictureContainer = (props) => {
    const onKeyPress = (e) => props.onClick && submitWithKeyboard(e, props.onClick);
    return React.createElement("aside", { style: tileContainerStyles$1, onClick: props.onClick, onKeyPress: onKeyPress, "aria-label": props.ariaLabel, tabIndex: props.onClick ? 0 : -1, "data-ui-id": "picture-in-picture-in-picture-root" },
        props.primaryView,
        React.createElement("div", { style: secondaryTileFloatingStyles }, props.secondaryView));
};
const tileContainerStyles$1 = {
    display: 'flex',
    width: 'min-content',
    position: 'relative',
    cursor: 'pointer'
};
const secondaryTileFloatingStyles = {
    // The secondary tile should float above the primary tile, aligned to the bottom right.
    position: 'absolute',
    bottom: '0.125rem',
    right: '0.125rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const containerStyles$3 = (theme) => {
    return {
        maxWidth: '16rem',
        textAlign: 'center',
        background: `${theme.palette.white}`,
        borderRadius: '0.75rem',
        margin: 'auto'
    };
};
/**
 * @private
 */
const buttonStyles$1 = (theme) => ({
    root: {
        background: 'none',
        fontWeight: 600,
        fontSize: `${_pxToRem(20)}`,
        border: 'none',
        borderRadius: 0,
        width: '3rem',
        height: '3rem',
        padding: `${_pxToRem(16)}, ${_pxToRem(5)}, ${_pxToRem(16)}, ${_pxToRem(5)}`,
        minWidth: 0,
        minHeight: 0
    }
});
/**
 * @private
 */
const digitStyles = (theme) => {
    return {
        fontSize: '1.25rem',
        fontWeight: theme.fonts.medium.fontWeight,
        color: `${theme.palette.themePrimary}`
    };
};
/**
 * @private
 */
const textFieldStyles = (theme, buttonPresent) => ({
    field: {
        padding: 0,
        textAlign: 'center',
        fontSize: `${_pxToRem(18)}`,
        fontWeight: 400,
        width: `${buttonPresent ? '10rem' : '12rem'}`,
        height: '3rem',
        borderRadius: '0.5rem',
        position: 'relative',
        overflowX: 'hidden',
        textOverflow: 'clip'
    },
    root: {
        backgroundColor: `${theme.palette.neutralLighter}`,
        marginBottom: '0.625rem',
        height: '3rem',
        borderRadius: '0.5rem'
    },
    fieldGroup: {
        border: 'none',
        borderRadius: '0.5rem',
        width: '12rem',
        height: '3rem',
        backgroundColor: `${theme.palette.neutralLighter}`,
        ':after': {
            borderRadius: '0.5rem'
        }
    },
    errorMessage: {
        color: theme.semanticColors.errorText
    },
    suffix: {
        padding: 0,
        position: 'absolute',
        right: '0.25rem',
        top: '0.55rem',
        transform: 'scale(1.15)',
        background: 'unset'
    }
});
/**
 * @private
 */
const letterStyles = (theme) => {
    return {
        fontSize: '0.625rem',
        color: `${theme.palette.neutralSecondary}`,
        fontWeight: 400,
        margin: '0.125rem',
        minHeight: '0.75rem'
    };
};
/**
 * @private
 */
const iconButtonStyles$1 = (theme) => {
    return {
        root: {
            color: `${theme.palette.black}`,
            padding: 0,
            background: 'transparent'
        },
        icon: {
            height: 'auto',
            background: 'transparent',
            // Needed to keep the icon vertically centered.
            '> span': {
                display: 'flex'
            }
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 */
const _formatPhoneNumber = (phoneNumber, enchant) => {
    // if input value is falsy eg if the user deletes the input, then just return
    if (!phoneNumber) {
        return phoneNumber;
    }
    // Only enchant North American phone numbers that begin with country code (1)
    if (enchant && phoneNumber[0] === '1' && phoneNumber.length === 11) {
        phoneNumber = `+${phoneNumber}`;
    }
    // if phone number starts with 1, format like 1 (xxx)xxx-xxxx.
    // if phone number starts with +, we format like +x (xxx)xxx-xxxx.
    // For now we are only supporting NA phone number formatting with country code +x
    // first we chop off the countrycode then we add it on when returning
    let countryCodeNA = '';
    if (phoneNumber[0] === '1') {
        countryCodeNA = '1 ';
        phoneNumber = phoneNumber.slice(1, phoneNumber.length);
    }
    else if (phoneNumber[0] === '+') {
        countryCodeNA = phoneNumber.slice(0, 2) + ' ';
        phoneNumber = phoneNumber.slice(2, phoneNumber.length);
    }
    // phoneNumberLength is used to know when to apply our formatting for the phone number
    const phoneNumberLength = phoneNumber.length;
    // we need to return the value with no formatting if its less then four digits
    // this is to avoid weird behavior that occurs if you  format the area code too early
    // if phoneNumberLength is greater than 10 we don't do any formatting
    if (phoneNumberLength < 4 || phoneNumberLength > 10) {
        // no formatting in this case, remove ' ' behind countrycode
        return countryCodeNA.replace(' ', '') + phoneNumber;
    }
    // if phoneNumberLength is greater than 4 and less the 7 we start to return
    // the formatted number
    if (phoneNumberLength < 7) {
        return `${countryCodeNA}(${phoneNumber.slice(0, 3)}) ${phoneNumber.slice(3)}`;
    }
    // finally, if the phoneNumberLength is greater then seven, we add the last
    // bit of formatting and return it.
    return `${countryCodeNA}(${phoneNumber.slice(0, 3)}) ${phoneNumber.slice(3, 6)}-${phoneNumber.slice(6, phoneNumber.length)}`;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Class for playing individual DTMF Tones.
 *
 * @internal
 */
class Tone {
    constructor(context, frequency1, frequency2) {
        this.isPlaying = false;
        /**
         * Function to play the tone. will create new ocillators because they are one use objects so we need to make
         * new ones every time we play a tone.
         */
        this.play = () => {
            if (this.isPlaying) {
                return;
            }
            const gainNode = this.context.createGain();
            gainNode.gain.value = 0.1;
            gainNode.connect(this.context.destination);
            this.oscillatorNode1 = this.context.createOscillator();
            this.oscillatorNode1.frequency.value = this.frequency1;
            this.oscillatorNode1.connect(gainNode);
            this.oscillatorNode2 = this.context.createOscillator();
            this.oscillatorNode2.frequency.value = this.frequency2;
            this.oscillatorNode2.connect(gainNode);
            this.oscillatorNode1.start();
            this.oscillatorNode2.start();
            this.isPlaying = true;
        };
        /**
         * Function to stop the tone.
         */
        this.stop = () => {
            if (this.oscillatorNode1 && this.oscillatorNode2) {
                this.oscillatorNode1.stop();
                this.oscillatorNode2.stop();
                this.oscillatorNode1.disconnect();
                this.oscillatorNode2.disconnect();
                this.isPlaying = false;
            }
        };
        this.context = context;
        this.frequency1 = frequency1;
        this.frequency2 = frequency2;
    }
}
/**
 * Mapping of the different dtmf frequencies that are needed for the creation of sound that
 * matches the dtmf tones.
 *
 * @internal
 */
const dtmfFrequencies = {
    '1': {
        f1: 697,
        f2: 1209
    },
    '2': {
        f1: 697,
        f2: 1336
    },
    '3': {
        f1: 697,
        f2: 1477
    },
    '4': {
        f1: 770,
        f2: 1209
    },
    '5': {
        f1: 770,
        f2: 1336
    },
    '6': {
        f1: 770,
        f2: 1477
    },
    '7': {
        f1: 852,
        f2: 1209
    },
    '8': {
        f1: 852,
        f2: 1336
    },
    '9': {
        f1: 852,
        f2: 1477
    },
    '*': {
        f1: 941,
        f2: 1209
    },
    '0': {
        f1: 941,
        f2: 1336
    },
    '#': {
        f1: 941,
        f2: 1477
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$F = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const dialPadButtonsDefault = [[{
            digit: '1'
        }, {
            digit: '2',
            letter: 'ABC'
        }, {
            digit: '3',
            letter: 'DEF'
        }], [{
            digit: '4',
            letter: 'GHI'
        }, {
            digit: '5',
            letter: 'JKL'
        }, {
            digit: '6',
            letter: 'MNO'
        }], [{
            digit: '7',
            letter: 'PQRS'
        }, {
            digit: '8',
            letter: 'TUV'
        }, {
            digit: '9',
            letter: 'WXYZ'
        }], [{
            digit: '*'
        }, {
            digit: '0',
            letter: '+'
        }, {
            digit: '#'
        }]];
const DtmfTones = ['Num1', 'Num2', 'Num3', 'Num4', 'Num5', 'Num6', 'Num7', 'Num8', 'Num9', 'Star', 'Num0', 'Pound'];
const DialpadButton = (props) => {
    var _a, _b, _c, _d;
    const theme = react.useTheme();
    const { digit, index, onClick, onLongPress, longPressTrigger, dtmfToneAudioContext, disableDtmfPlayback } = props;
    const [buttonPressed, setButtonPressed] = React.useState(false);
    const dtmfToneSound = React.useRef(new Tone(dtmfToneAudioContext, dtmfFrequencies[digit].f1, dtmfFrequencies[digit].f2));
    const useLongPressProps = React.useMemo(() => ({
        onClick: () => __awaiter$F(void 0, void 0, void 0, function* () {
            onClick(digit, index);
        }),
        onLongPress: () => __awaiter$F(void 0, void 0, void 0, function* () {
            onLongPress(digit, index);
        }),
        touchEventsOnly: longPressTrigger === 'touch'
    }), [digit, index, longPressTrigger, onClick, onLongPress]);
    const longPressHandlers = useLongPress(useLongPressProps);
    return React.createElement(react.DefaultButton, Object.assign({ "data-test-id": `dialpad-button-${props.index}`, styles: react.concatStyleSets(buttonStyles$1(), (_a = props.styles) === null || _a === void 0 ? void 0 : _a.button) }, longPressHandlers, { onKeyDown: e => {
            if ((e.key === 'Enter' || e.key === ' ') && !buttonPressed) {
                if (!disableDtmfPlayback) {
                    dtmfToneSound.current.play();
                }
                longPressHandlers.onKeyDown();
                setButtonPressed(true);
                return;
            }
            if (e.key === 'Tab' || e.key === 'ArrowLeft' || e.key === 'ArrowRight' || e.key === 'ArrowUp' || e.key === 'ArrowDown') {
                dtmfToneSound.current.stop();
                return;
            }
            longPressHandlers.onKeyDown();
        }, onKeyUp: e => {
            if ((e.key === 'Enter' || e.key === ' ') && buttonPressed) {
                dtmfToneSound.current.stop();
                longPressHandlers.onKeyUp();
                setButtonPressed(false);
            }
            longPressHandlers.onKeyUp();
        }, onMouseDown: () => {
            if (!disableDtmfPlayback) {
                dtmfToneSound.current.play();
            }
            longPressHandlers.onMouseDown();
        }, onMouseUp: () => {
            dtmfToneSound.current.stop();
            longPressHandlers.onMouseUp();
        }, onMouseLeave: () => {
            dtmfToneSound.current.stop();
        }, onTouchStart: () => {
            if (!disableDtmfPlayback) {
                dtmfToneSound.current.play();
            }
            longPressHandlers.onTouchStart();
        }, onTouchEnd: () => {
            dtmfToneSound.current.stop();
            longPressHandlers.onTouchEnd();
        } }),
        React.createElement(react.Stack, null,
            React.createElement(react.Text, { className: react.mergeStyles(digitStyles(theme), (_b = props.styles) === null || _b === void 0 ? void 0 : _b.digit) }, props.digit),
            React.createElement(react.Text, { className: react.mergeStyles(letterStyles(theme), (_c = props.styles) === null || _c === void 0 ? void 0 : _c.letter) }, (_d = props.letter) !== null && _d !== void 0 ? _d : ' ')));
};
const DialpadContainer = (props) => {
    var _a, _b;
    const theme = react.useTheme();
    const { onSendDtmfTone, onClickDialpadButton, textFieldValue, onChange, showDeleteButton = true, longPressTrigger = 'mouseAndTouch', disableDtmfPlayback, dialpadMode = 'dialer', dtmfAudioContext } = props;
    const dtmfToneAudioContext = React.useRef(dtmfAudioContext ? dtmfAudioContext : new AudioContext());
    const [plainTextValue, setPlainTextValue] = React.useState(textFieldValue !== null && textFieldValue !== void 0 ? textFieldValue : '');
    const plainTextValuePreviousRenderValue = React.useRef(plainTextValue);
    React.useEffect(() => {
        if (plainTextValuePreviousRenderValue.current !== plainTextValue) {
            onChange === null || onChange === void 0 ? void 0 : onChange(plainTextValue);
        }
        plainTextValuePreviousRenderValue.current = plainTextValue;
    }, [plainTextValuePreviousRenderValue, plainTextValue, onChange]);
    React.useEffect(() => {
        setText(textFieldValue !== null && textFieldValue !== void 0 ? textFieldValue : '');
    }, [textFieldValue]);
    const onClickDialpad = (input, index) => {
        setText(plainTextValue + input);
        const tone = DtmfTones[index];
        if (onSendDtmfTone && tone) {
            onSendDtmfTone(tone);
        }
        if (onClickDialpadButton) {
            onClickDialpadButton(input, index);
        }
    };
    const onLongPressDialpad = (input, index) => {
        if (input === '0' && index === 10) {
            setText(plainTextValue + '+');
        }
        else {
            setText(plainTextValue + input);
        }
        const tone = DtmfTones[index];
        if (onSendDtmfTone && tone) {
            onSendDtmfTone(tone);
        }
        if (onClickDialpadButton) {
            onClickDialpadButton(input, index);
        }
    };
    const setText = (input) => {
        // remove non-valid characters from input: letters,special characters excluding +, *,#
        const plainInput = sanitizeInput(input);
        setPlainTextValue(plainInput);
    };
    const deleteNumbers = () => {
        const modifiedInput = plainTextValue.substring(0, plainTextValue.length - 1);
        setText(modifiedInput);
    };
    return React.createElement(react.Stack, { className: react.mergeStyles(containerStyles$3(theme), (_a = props.styles) === null || _a === void 0 ? void 0 : _a.root), "data-test-id": "dialpadContainer", "data-ui-id": "dialpadContainer", horizontalAlign: 'center' },
        dialpadMode === 'dialer' && React.createElement(react.TextField, { styles: react.concatStyleSets(textFieldStyles(theme, plainTextValue !== ''), (_b = props.styles) === null || _b === void 0 ? void 0 : _b.textField), value: textFieldValue ? textFieldValue : _formatPhoneNumber(plainTextValue), 
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            onChange: (e) => {
                setText(e.target.value);
            }, onClick: e => {
                e.preventDefault();
            }, placeholder: props.strings.placeholderText, "data-test-id": "dialpad-input", onRenderSuffix: () => {
                var _a;
                return React.createElement(React.Fragment, null, showDeleteButton && plainTextValue.length !== 0 && React.createElement(react.IconButton, { ariaLabel: props.strings.deleteButtonAriaLabel, onClick: deleteNumbers, styles: react.concatStyleSets(iconButtonStyles$1(theme), (_a = props.styles) === null || _a === void 0 ? void 0 : _a.deleteIcon), iconProps: {
                        iconName: 'DialpadBackspace'
                    } }));
            } }),
        React.createElement(react.FocusZone, null, dialPadButtonsDefault.map((rows, rowIndex) => {
            return React.createElement(react.Stack, { horizontal: true, key: `row_${rowIndex}`, horizontalAlign: "stretch", tokens: {
                    childrenGap: '1rem'
                } }, rows.map((button, columnIndex) => React.createElement(DialpadButton, { key: `button_${columnIndex}`, 
                /* row index = 0
                columnIndex: (0,1,2) => (0,1,2)
                row index = 1
                columnIndex: (0,1,2)=> (3,4,5)
                row index = 2
                columnIndex: (0,1,2)=> (6,7,8)
                row index = 3
                columnIndex: (0,1,2)=> (9,10,11)
                columnIndex + rowIndex*rows.length calculates the corresponding index for each button
                dialpad index:
                0 1 2
                3 4 5
                6 7 8
                9 10 11
                then use this index to locate the corresponding dtmf tones
                DtmfTones[index]
                */ index: columnIndex + rowIndex * rows.length, digit: button.digit, letter: button.letter, styles: props.styles, onClick: onClickDialpad, onLongPress: onLongPressDialpad, longPressTrigger: longPressTrigger, dtmfToneAudioContext: dtmfToneAudioContext.current, disableDtmfPlayback: disableDtmfPlayback })));
        })));
};
/**
 * A component to allow users to enter phone number through clicking on dialpad/using keyboard
 * It will return empty component for stable builds
 *
 * @public
 */
const Dialpad = (props) => {
    const localeStrings = useLocale$1().strings.dialpad;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    return React.createElement(DialpadContainer, Object.assign({ strings: strings }, props));
};
const sanitizeInput = (input) => {
    // remove non-valid characters from input: letters,special characters excluding +, *,#
    return input.replace(/[^\d*#+]/g, '');
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const helperTextStyle$1 = (theme) => react.mergeStyles({
    fontWeight: 600,
    fontSize: _pxToRem(16),
    lineHeight: _pxToRem(20),
    color: theme.palette.neutralPrimary,
    paddingTop: _pxToRem(24)
});
/**
 * @private
 */
const ratingHelperTextStyle = (theme) => react.mergeStyles({
    fontWeight: 400,
    fontSize: _pxToRem(12),
    lineHeight: _pxToRem(12),
    color: theme.palette.themePrimary,
    marginBottom: _pxToRem(8),
    marginTop: _pxToRem(8),
    textAlign: 'center'
});
/**
 * @private
 */
const ratingStyles = (theme) => {
    return {
        root: {
            textAlign: 'center',
            height: 'auto',
            marginTop: _pxToRem(8)
        },
        ratingStarIsLarge: {
            height: 'auto',
            padding: 0
        },
        ratingStar: {
            transform: 'scale(1.4)',
            margin: _pxToRem(6),
            height: 'auto'
        },
        ratingStarFront: {
            color: theme.palette.themePrimary
        }
    };
};
/**
 * @private
 */
const titleContainerClassName$3 = react.mergeStyles({
    paddingBottom: _pxToRem(6)
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A component to allow users to send numerical ratings regarding call quality
 *
 * @internal
 */
const _StarSurvey = (props) => {
    const { onStarRatingSelected, selectedIcon, unselectedIcon, strings } = props;
    const [rating, setRating] = React.useState(0);
    const [ratingHelperText, setRatingHelperText] = React.useState('');
    const theme = react.useTheme();
    const onRatingChange = React.useCallback((ev, rating) => {
        if (rating) {
            setRating(rating);
            switch (rating) {
                case 1:
                    setRatingHelperText(strings === null || strings === void 0 ? void 0 : strings.starSurveyOneStarText);
                    break;
                case 2:
                    setRatingHelperText(strings === null || strings === void 0 ? void 0 : strings.starSurveyTwoStarText);
                    break;
                case 3:
                    setRatingHelperText(strings === null || strings === void 0 ? void 0 : strings.starSurveyThreeStarText);
                    break;
                case 4:
                    setRatingHelperText(strings === null || strings === void 0 ? void 0 : strings.starSurveyFourStarText);
                    break;
                case 5:
                    setRatingHelperText(strings === null || strings === void 0 ? void 0 : strings.starSurveyFiveStarText);
                    break;
            }
            onStarRatingSelected === null || onStarRatingSelected === void 0 ? void 0 : onStarRatingSelected(rating);
        }
    }, [strings === null || strings === void 0 ? void 0 : strings.starSurveyOneStarText, strings === null || strings === void 0 ? void 0 : strings.starSurveyTwoStarText, strings === null || strings === void 0 ? void 0 : strings.starSurveyThreeStarText, strings === null || strings === void 0 ? void 0 : strings.starSurveyFourStarText, strings === null || strings === void 0 ? void 0 : strings.starSurveyFiveStarText, onStarRatingSelected]);
    return React.createElement(react.Stack, { verticalAlign: "center" },
        React.createElement(react.Stack, { className: titleContainerClassName$3 },
            React.createElement(react.Text, { className: helperTextStyle$1(theme) }, strings === null || strings === void 0 ? void 0 : strings.starSurveyHelperText)),
        React.createElement(react.Rating, { max: 5, size: react.RatingSize.Large, defaultRating: 0, allowZeroStars: true, rating: rating, onChange: onRatingChange, styles: ratingStyles(theme), icon: selectedIcon !== null && selectedIcon !== void 0 ? selectedIcon : 'SurveyStarIconFilled', unselectedIcon: unselectedIcon !== null && unselectedIcon !== void 0 ? unselectedIcon : 'SurveyStarIcon', ariaLabelFormat: strings === null || strings === void 0 ? void 0 : strings.starRatingAriaLabel }),
        React.createElement(react.Text, { className: ratingHelperTextStyle(theme) }, ratingHelperText));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const questionTextStyle$1 = (theme) => react.mergeStyles({
    fontWeight: 600,
    fontSize: _pxToRem(16),
    lineHeight: _pxToRem(20),
    color: theme.palette.neutralPrimary,
    marginTop: _pxToRem(8),
    marginBottom: _pxToRem(4)
});
/**
 * @private
 */
const helperTextStyle = (theme) => react.mergeStyles({
    fontWeight: 400,
    fontSize: _pxToRem(12),
    lineHeight: _pxToRem(16),
    marginTop: _pxToRem(4),
    color: theme.palette.neutralSecondary
});
/**
 * @private
 */
const checkboxClassName = react.mergeStyles({
    padding: _pxToRem(8),
    paddingLeft: 0
});
/**
 * @private
 */
const freeFormTextCheckboxStyles = {
    root: {
        paddingBottom: _pxToRem(8),
        width: '100%'
    },
    label: {
        width: '100%'
    }
};
/**
 * @private
 */
const freeFormTextFieldClassName = react.mergeStyles({
    width: '100%',
    marginLeft: _pxToRem(4),
    input: {
        padding: 0
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// Permit `any` in this file to handle the overloaded issue type. Todo: refactor CallSurvey to have PartialDeep<_CallSurvey> instead of any
// for tracking incomplete survey responses.
/* eslint-disable @typescript-eslint/no-explicit-any */
/**
 * A component to allow users to send numerical ratings regarding call quality
 *
 * @internal
 */
const _TagsSurvey = (props) => {
    const { callIssuesToTag, categoryHeadings, onConfirm, strings, showFreeFormTextField } = props;
    const [selectedTags, setSelectedTags] = React.useState({});
    const [textResponse, setTextResponse] = React.useState({});
    const [selectedTextResponse, setSelectedTextResponse] = React.useState({});
    const [checkedTextFields, setCheckedTextFields] = React.useState([]);
    const tags = React.useMemo(() => {
        const tags = {
            overallRating: [],
            audioRating: [],
            videoRating: [],
            screenshareRating: []
        };
        _getKeys(callIssuesToTag).forEach(issueCategory => {
            _getKeys(callIssuesToTag[issueCategory]).map(issue => {
                const issueCapitalized = (issue === null || issue === void 0 ? void 0 : issue.charAt(0).toUpperCase()) + (issue === null || issue === void 0 ? void 0 : issue.slice(1));
                const issueMessages = callIssuesToTag[issueCategory];
                if (tags[issueCategory]) {
                    tags[issueCategory].push({
                        message: issueMessages[issue],
                        issue: issueCapitalized
                    });
                }
                else {
                    tags[issueCategory] = [{
                            message: issueMessages[issue],
                            issue: issueCapitalized
                        }];
                }
            });
        });
        return tags;
    }, [callIssuesToTag]);
    const onChange = React.useCallback((issueCategory, checked, issue) => {
        if (checked) {
            if (issue) {
                setSelectedTags(prevState => {
                    const category = prevState === null || prevState === void 0 ? void 0 : prevState[issueCategory];
                    if (category && category.issues) {
                        category.issues = [category.issues, issue];
                    }
                    else {
                        prevState[issueCategory] = {
                            issues: [issue]
                        };
                    }
                    return prevState;
                });
            }
            else {
                setCheckedTextFields([...checkedTextFields, issueCategory]);
                setSelectedTextResponse(prevState => {
                    prevState[issueCategory] = textResponse[issueCategory];
                    return prevState;
                });
            }
        }
        else {
            if (issue) {
                setSelectedTags(prevState => {
                    var _a, _b;
                    // 'prevState[issueCategory]?.issues as ...' typing is required here to avoid a typescript limitation
                    // "This expression is not callable" caused by filter().
                    // More information can be found here: https://github.com/microsoft/TypeScript/issues/44373
                    const categoryIssues = (_b = (_a = prevState[issueCategory]) === null || _a === void 0 ? void 0 : _a.issues) === null || _b === void 0 ? void 0 : _b.filter((value) => value !== issue);
                    return Object.assign(Object.assign({}, prevState), { [issueCategory]: Object.assign(Object.assign({}, (prevState[issueCategory] || {})), { issues: categoryIssues }) });
                });
            }
            else {
                setCheckedTextFields(checkedTextFields.filter(id => id !== issueCategory));
                setSelectedTextResponse(prevState => {
                    delete prevState[issueCategory];
                    return prevState;
                });
            }
        }
    }, [textResponse, checkedTextFields]);
    const theme = react.useTheme();
    const onRenderLabel = React.useCallback((issueCategory) => {
        return React.createElement(react.TextField, { key: issueCategory, className: freeFormTextFieldClassName, underlined: true, placeholder: strings === null || strings === void 0 ? void 0 : strings.tagsSurveyTextFieldDefaultText, onChange: (e, v) => {
                if (v) {
                    setCheckedTextFields([...checkedTextFields, issueCategory]);
                    setTextResponse(prevState => {
                        prevState[issueCategory] = v;
                        return prevState;
                    });
                    setSelectedTextResponse(prevState => {
                        prevState[issueCategory] = v;
                        return prevState;
                    });
                }
            } });
    }, [strings === null || strings === void 0 ? void 0 : strings.tagsSurveyTextFieldDefaultText, checkedTextFields]);
    React.useEffect(() => {
        if (onConfirm) {
            onConfirm(selectedTags, selectedTextResponse);
        }
    }, [selectedTags, selectedTextResponse, onConfirm]);
    return React.createElement(React.Fragment, null,
        React.createElement(react.Stack, { verticalAlign: "center" },
            React.createElement(react.Text, { className: questionTextStyle$1(theme) }, strings === null || strings === void 0 ? void 0 : strings.tagsSurveyQuestion)),
        React.createElement(react.Pivot, null, _getKeys(tags).map((key, i) => {
            return React.createElement(react.PivotItem, { key: `key-${i}`, headerText: categoryHeadings[key], headerButtonProps: {
                    'data-order': i,
                    'data-title': key
                }, alwaysRender: true },
                tags[key].map((t, i) => {
                    return React.createElement(react.Checkbox, { className: checkboxClassName, key: `checkBox_${i}`, label: t.message, onChange: (ev, checked) => onChange(key, checked !== null && checked !== void 0 ? checked : false, t.issue) });
                }),
                showFreeFormTextField && React.createElement(react.Checkbox, { checked: checkedTextFields.includes(key), styles: freeFormTextCheckboxStyles, onChange: (ev, checked) => onChange(key, checked !== null && checked !== void 0 ? checked : false), onRenderLabel: () => {
                        return onRenderLabel(key);
                    } }));
        })),
        React.createElement(react.Text, { className: helperTextStyle(theme) }, strings === null || strings === void 0 ? void 0 : strings.tagsSurveyHelperText));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * a button to hold or resume a ongoing call.
 *
 * Can be used with {@link ControlBar}
 *
 * @param props - properties for the hold button.
 * @returns
 * @public
 */
const HoldButton = (props) => {
    const { onToggleHold, strings } = props;
    const onRenderHoldIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "HoldCallButton" });
    };
    const onRenderResumeIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "ResumeCall" });
    };
    return React.createElement(ControlBarButton, Object.assign({}, props, { strings: strings, onClick: onToggleHold !== null && onToggleHold !== void 0 ? onToggleHold : props.onClick, onRenderOnIcon: onRenderResumeIcon, onRenderOffIcon: onRenderHoldIcon }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const VideoEffectsItemContainerHeight = '3.375rem';
const VideoEffectsItemContainerWidth = '4.83rem';
const VideoEffectsItemContainerBorderHeight = '3.373rem';
const VideoEffectsItemContainerBorderWidth = '4.85rem';
const VideoEffectsItemContainerBorderSize = '2px';
/** @private */
const hiddenVideoEffectsItemContainerStyles = {
    root: {
        visibility: 'hidden',
        height: VideoEffectsItemContainerHeight,
        width: VideoEffectsItemContainerWidth,
        border: VideoEffectsItemContainerBorderSize
    }
};
/** @private */
const videoEffectsItemContainerStyles = (args) => {
    const borderDefaultThickness = '1px';
    const borderActiveThickness = '2px';
    return {
        root: {
            background: args.disabled ? args.theme.palette.neutralQuaternaryAlt : undefined,
            backgroundImage: args.backgroundImage ? `url(${args.backgroundImage})` : undefined,
            backgroundPosition: 'center',
            backgroundSize: 'cover',
            borderRadius: '0.25rem',
            color: args.theme.palette.neutralPrimary,
            cursor: args.disabled ? 'default' : 'pointer',
            height: VideoEffectsItemContainerHeight,
            width: VideoEffectsItemContainerWidth,
            outlineOffset: '-1px',
            outline: args.isSelected ? `${borderActiveThickness} solid ${args.theme.palette.themePrimary}` : `${borderDefaultThickness} solid ${args.theme.palette.neutralQuaternaryAlt}`,
            ':hover': {
                boxSizing: 'border-box',
                width: VideoEffectsItemContainerBorderWidth,
                height: VideoEffectsItemContainerBorderHeight,
                outlineOffset: '-1px',
                outline: args.disabled && !args.isSelected ? `${borderDefaultThickness} solid ${args.theme.palette.neutralQuaternaryAlt}` : `${borderActiveThickness} solid ${args.theme.palette.themePrimary}`
            },
            padding: '0.5rem'
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A component for displaying a Video Background Effect Option.
 *
 * @internal
 */
const _VideoEffectsItem = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const theme = react.useTheme();
    const isSelected = (_a = props.isSelected) !== null && _a !== void 0 ? _a : false;
    const disabled = (_b = props.disabled) !== null && _b !== void 0 ? _b : false;
    const backgroundImage = (_c = props.backgroundProps) === null || _c === void 0 ? void 0 : _c.url;
    const iconContainerStyles = react.mergeStyles({
        height: '1.25rem'
    }, (_d = props.styles) === null || _d === void 0 ? void 0 : _d.iconContainer);
    const textContainerStyles = react.mergeStyles({
        height: '1.25rem',
        maxWidth: '4.5rem',
        overflow: 'hidden'
    }, (_e = props.styles) === null || _e === void 0 ? void 0 : _e.textContainer);
    const containerStyles = React.useCallback(() => videoEffectsItemContainerStyles({
        theme,
        isSelected,
        disabled,
        backgroundImage
    }), [backgroundImage, disabled, isSelected, theme]);
    return React.createElement(react.TooltipHost, Object.assign({}, props.tooltipProps),
        React.createElement(react.Stack, { key: props.itemKey, className: react.mergeStyles((_f = props.styles) === null || _f === void 0 ? void 0 : _f.root), verticalAlign: "center", horizontalAlign: "center", "data-ui-id": `video-effects-item` },
            React.createElement(react.DefaultButton, { styles: containerStyles(), onClick: disabled ? undefined : () => { var _a; return (_a = props.onSelect) === null || _a === void 0 ? void 0 : _a.call(props, props.itemKey); }, componentRef: props.componentRef, ariaLabel: (_g = props.ariaLabel) !== null && _g !== void 0 ? _g : `${props.title}, ${(_h = props.tooltipProps) === null || _h === void 0 ? void 0 : _h.content}`, "aria-disabled": props.disabled, role: "radio", "aria-checked": isSelected },
                React.createElement(react.Stack, { horizontalAlign: 'center', className: react.mergeStyles({
                        width: '100%'
                    }) },
                    props.iconProps && React.createElement(react.Stack.Item, { className: iconContainerStyles },
                        React.createElement(react.Icon, Object.assign({}, props.iconProps))),
                    props.title && React.createElement(react.Stack.Item, { className: textContainerStyles },
                        React.createElement(react.Text, { variant: "small", styles: {
                                root: {
                                    whiteSpace: 'nowrap',
                                    overflow: 'hidden',
                                    textOverflow: 'ellipsis'
                                }
                            } }, props.title))))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Picker for choosing a Video Background Effect.
 *
 * @remarks
 * This functions similar to a radio group of buttons, where the user can select one of the options.
 *
 * @internal
 */
const _VideoBackgroundEffectsPicker = (props) => {
    var _a, _b, _c, _d, _e;
    const [componentControlledSelectedEffectKey, setComponentControlledSelectedEffectKey] = React.useState(props.defaultSelectedEffectKey);
    // Warn the developer if they use the component in an incorrect controlled way.
    reactHooks.useWarnings({
        name: 'VideoBackgroundEffectsPicker',
        props,
        controlledUsage: {
            onChangeProp: 'onChange',
            valueProp: 'selectedEffectKey',
            defaultValueProp: 'defaultSelectedEffectKey'
        }
    });
    const selectedEffect = (_a = props.selectedEffectKey) !== null && _a !== void 0 ? _a : componentControlledSelectedEffectKey;
    const setSelectedEffect = (selectedEffectKey) => {
        var _a;
        setComponentControlledSelectedEffectKey(selectedEffectKey);
        (_a = props.onChange) === null || _a === void 0 ? void 0 : _a.call(props, selectedEffectKey);
    };
    const convertedOptions = props.options.map(option => (Object.assign({ isSelected: option.itemKey === selectedEffect, onSelect: () => setSelectedEffect(option.itemKey) }, option)));
    const itemsPerRow = (_b = props.itemsPerRow) !== null && _b !== void 0 ? _b : 3;
    const optionsByRow = itemsPerRow === 'wrap' ? [convertedOptions] : chunk(convertedOptions, itemsPerRow);
    // If the final row is not full, fill it with hidden items to ensure layout.
    const fillCount = itemsPerRow === 'wrap' ? 0 : itemsPerRow - ((_d = (_c = optionsByRow[optionsByRow.length - 1]) === null || _c === void 0 ? void 0 : _c.length) !== null && _d !== void 0 ? _d : 0);
    const effectsLabelId = reactHooks.useId('effects-radio-label');
    return React.createElement(react.FocusZone, null,
        React.createElement(react.Stack, { tokens: {
                childrenGap: '0.5rem'
            }, role: "radiogroup", "aria-labelledby": effectsLabelId },
            React.createElement(react.Label, { id: effectsLabelId, className: react.mergeStyles((_e = props.styles) === null || _e === void 0 ? void 0 : _e.label) }, props.label),
            optionsByRow.map((options, rowIndex) => {
                var _a;
                return React.createElement(react.Stack, { className: react.mergeStyles((_a = props.styles) === null || _a === void 0 ? void 0 : _a.rowRoot), wrap: props.itemsPerRow === 'wrap', horizontal: true, key: rowIndex, tokens: {
                        childrenGap: '0.5rem'
                    }, "data-ui-id": "video-effects-picker-row" },
                    options.map((option, i) => {
                        if (i === 0 && rowIndex === 0) {
                            return React.createElement(_VideoEffectsItem, Object.assign({}, option, { itemKey: option.itemKey, key: option.itemKey, componentRef: props.componentRef }));
                        }
                        return React.createElement(_VideoEffectsItem, Object.assign({}, option, { itemKey: option.itemKey, key: option.itemKey }));
                    }),
                    fillCount > 0 && rowIndex === optionsByRow.length - 1 && Array.from({
                        length: fillCount
                    }).map((_, index) => React.createElement(react.Stack, { key: index, styles: hiddenVideoEffectsItemContainerStyles, "data-ui-id": "video-effects-hidden-item" })));
            })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const iconClassName = react.mergeStyles({
    marginRight: _pxToRem(8)
});
/**
 * @private
 */
const displayNameClassName = react.mergeStyles({
    fontWeight: 400,
    fontSize: _pxToRem(12),
    lineHeight: _pxToRem(16)
});
/**
 * @private
 */
const bannerTitleContainerClassName = react.mergeStyles({
    paddingBottom: _pxToRem(10)
});
/**
 * @private
 */
const isTypingClassName = (theme) => {
    return react.mergeStyles({
        backgroundColor: theme.palette.themeLighter,
        color: theme.palette.themeDarker,
        borderRadius: _pxToRem(4),
        marginLeft: _pxToRem(4),
        fontWeight: 400,
        paddingLeft: _pxToRem(4),
        paddingRight: _pxToRem(4),
        fontSize: _pxToRem(11),
        lineHeight: _pxToRem(16)
    });
};
/**
 * @private
 */
const rttContainerClassName = (theme, isTyping) => {
    return react.mergeStyles({
        borderLeft: isTyping ? `2px solid ${theme.palette.themeLighter}` : 'none'
    });
};
/**
 * @private
 */
const rttDisclosureBannerClassName = () => {
    return react.mergeStyles({
        padding: '0.25rem',
        paddingTop: '0.5rem'
    });
};
/**
 * @private
 */
const realTimeTextInputBoxStyles = (theme) => ({
    root: {
        marginBottom: _pxToRem(8)
    },
    fieldGroup: {
        borderRadius: _pxToRem(4),
        borderColor: theme.palette.neutralQuaternaryAlt
    }
});
/**
 * @private
 */
const expandIconClassName = (theme) => {
    return {
        root: {
            color: theme.palette.neutralPrimary,
            width: '1rem',
            height: '1rem'
        },
        rootHovered: {
            color: theme.palette.neutralDark
        }
    };
};
/**
 * @private
 */
const captionClassName = react.mergeStyles({
    fontWeight: 400,
    fontSize: _pxToRem(16),
    lineHeight: _pxToRem(22),
    width: '100%'
});
/**
 * @private
 */
const captionsContainerClassName = react.mergeStyles({
    height: '100%',
    margin: 0,
    overflow: 'auto',
    padding: 0,
    overflowX: 'hidden'
});
/**
 * @private
 */
const hiddenAnnouncementClassName = react.mergeStyles({
    position: 'absolute',
    width: '1px',
    height: '1px',
    padding: 0,
    overflow: 'hidden',
    clip: 'rect(0, 0, 0, 0)',
    whiteSpace: 'nowrap',
    border: 0
});
/**
 * @private
 */
const captionContainerClassName = react.mergeStyles({
    marginTop: _pxToRem(6),
    marginBottom: _pxToRem(6),
    textAlign: 'unset',
    // ensure RTL spoken language captions are appropriately aligned to the right
    overflowAnchor: 'auto',
    overflowWrap: 'break-word',
    wordBreak: 'break-word',
    // Additional breaking control for long words
    maxWidth: '100%',
    // Ensure it never expands beyond 100% of the container
    width: '100%',
    overflow: 'hidden',
    boxSizing: 'border-box',
    // Include padding in the width calculation
    whiteSpace: 'normal' // Ensure text wraps
});
/** Reset styling set by the `ul` element */
const resetUlStyling = {
    listStyleType: 'none',
    padding: 0,
    margin: 0
};
/**
 * @private
 */
const captionsBannerClassName = (formFactor, isExpanded) => {
    return react.mergeStyles(Object.assign(Object.assign({}, resetUlStyling), { overflowX: 'hidden', height: formFactor === 'compact' ? isExpanded ? '40vh' : '4.5rem' : '8.75rem', overflowY: 'auto' }));
};
/**
 * @private
 */
const captionsBannerFullHeightClassName = (theme) => {
    return react.mergeStyles(Object.assign(Object.assign({}, resetUlStyling), { overflowX: 'hidden', overflowY: 'auto', height: '100%', width: '100%', position: 'absolute', backgroundColor: theme.palette.white, left: 0 }));
};
/**
 * @private
 */
const loadingBannerStyles = (formFactor) => {
    return {
        root: {
            height: formFactor === 'compact' ? '4.5rem' : '8.75rem'
        }
    };
};
/**
 * @private
 */
const loadingBannerFullHeightStyles = (theme) => {
    return {
        root: {
            height: '100%',
            width: '100%',
            position: 'absolute',
            left: 0,
            backgroundColor: theme.palette.white
        }
    };
};
/**
 * @private
 */
const captionsContentContainerClassName = react.mergeStyles({
    width: '100%',
    paddingInlineEnd: _pxToRem(4)
});
/**
 * @private
 */
const displayNameContainerClassName = react.mergeStyles({
    overflow: 'hidden',
    textOverflow: 'ellipsis'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * A component for displaying a single line of caption
 */
const _Caption = (props) => {
    const { displayName, userId, captionText, onRenderAvatar } = props;
    const personaOptions = {
        hidePersonaDetails: true,
        size: react.PersonaSize.size32,
        text: displayName,
        showOverflowTooltip: false,
        imageShouldStartVisible: true,
        initialsTextColor: 'white',
        styles: {
            root: {
                margin: '0.25rem'
            }
        }
    };
    const userIcon = onRenderAvatar ? onRenderAvatar(userId !== null && userId !== void 0 ? userId : '', personaOptions) : React.createElement(react.Persona, Object.assign({}, personaOptions));
    return React.createElement(react.Stack, { horizontal: true, verticalAlign: "start", horizontalAlign: "start" },
        React.createElement(react.Stack.Item, { className: iconClassName }, userIcon),
        React.createElement(react.Stack, { verticalAlign: "start", className: captionsContentContainerClassName },
            React.createElement(react.Stack.Item, { className: displayNameContainerClassName },
                React.createElement(react.Text, { className: displayNameClassName }, displayName)),
            React.createElement(react.Stack.Item, { className: captionClassName, dir: "auto" }, captionText)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @public
 * A component for displaying a single line of RealTimeText
 */
const RealTimeText = (props) => {
    const { displayName, userId, message, onRenderAvatar, isTyping } = props;
    const theme = react.useTheme();
    const localeStrings = useLocale$1().strings.realTimeText;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const personaOptions = {
        hidePersonaDetails: true,
        size: react.PersonaSize.size32,
        text: displayName,
        showOverflowTooltip: false,
        imageShouldStartVisible: true,
        initialsTextColor: 'white',
        styles: {
            root: {
                margin: '0.25rem'
            }
        }
    };
    const userIcon = onRenderAvatar ? onRenderAvatar(userId !== null && userId !== void 0 ? userId : '', personaOptions) : React.createElement(react.Persona, Object.assign({}, personaOptions));
    return React.createElement(react.Stack, { horizontal: true, verticalAlign: "start", horizontalAlign: "start", className: rttContainerClassName(theme, isTyping !== null && isTyping !== void 0 ? isTyping : false) },
        React.createElement(react.Stack.Item, { className: iconClassName }, userIcon),
        React.createElement(react.Stack, { verticalAlign: "start", className: captionsContentContainerClassName },
            React.createElement(react.Stack, { className: displayNameContainerClassName, horizontal: true },
                React.createElement(react.Text, { className: displayNameClassName }, displayName),
                isTyping && React.createElement(react.Text, { className: isTypingClassName(theme) }, strings === null || strings === void 0 ? void 0 : strings.isTypingText)),
            React.createElement(react.Stack.Item, { className: captionClassName, dir: "auto" }, message)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const messageTextStyles = (theme) => ({
    root: {
        fontWeight: 400,
        fontSize: _pxToRem(14),
        lineHeight: _pxToRem(16),
        color: theme.palette.neutralSecondary,
        marginTop: '0.25rem'
    }
});
/**
 * @private
 */
const titleTextStyles = () => ({
    root: {
        fontWeight: 400,
        fontSize: _pxToRem(14),
        lineHeight: _pxToRem(16),
        alignSelf: 'center'
    }
});
/**
 * @private
 */
const containerStyles$2 = (theme) => ({
    root: {
        boxShadow: theme.effects.elevation8,
        width: '20rem',
        padding: '0.75rem',
        borderRadius: '0.25rem',
        position: 'relative',
        backgroundColor: theme.palette.white
    }
});
/**
 * @private
 */
const hiddenContainerStyles = (theme) => react.mergeStyles({
    boxShadow: theme.effects.elevation8,
    width: '19.5rem',
    borderRadius: '0.25rem',
    marginTop: '-0.65rem',
    height: '1rem'
});
/**
 * @private
 */
const notificationIconStyles = () => ({
    root: {
        fontSize: '1.25rem',
        alignSelf: 'center',
        marginRight: '0.5rem',
        svg: {
            width: '1.25rem',
            height: '1.25rem'
        }
    }
});
/**
 * @private
 */
const notificationLinkStyles = (theme) => ({
    root: {
        color: theme.palette.themePrimary,
        fontWeight: '600',
        paddingLeft: '0.25rem'
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A component to show notification messages on the UI.
 *
 * @public
 */
const Notification = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
    const strings = props.notificationStrings;
    const theme = react.useTheme();
    const [show, setShow] = React.useState(true);
    const { ariaLive = 'assertive', role = 'alert' } = props;
    if (props.autoDismiss) {
        setTimeout(() => {
            // After 5 seconds set the show value to false
            setShow(false);
        }, 5000);
        if (!show) {
            props.onDismiss && props.onDismiss();
            return React.createElement(React.Fragment, null);
        }
    }
    return React.createElement(react.Stack, { horizontalAlign: "center", "aria-live": ariaLive, role: role },
        React.createElement(react.Stack, { "data-ui-id": "notification-bar", styles: (_b = (_a = props.styles) === null || _a === void 0 ? void 0 : _a.root) !== null && _b !== void 0 ? _b : containerStyles$2(theme) },
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between" },
                React.createElement(react.Stack, { horizontal: true },
                    React.createElement(react.Icon, Object.assign({ styles: (_d = (_c = props.styles) === null || _c === void 0 ? void 0 : _c.icon) !== null && _d !== void 0 ? _d : notificationIconStyles(), iconName: (_f = (_e = props.notificationIconProps) === null || _e === void 0 ? void 0 : _e.iconName) !== null && _f !== void 0 ? _f : 'ErrorBadge' }, props.notificationIconProps)),
                    React.createElement(react.Text, { styles: (_h = (_g = props.styles) === null || _g === void 0 ? void 0 : _g.title) !== null && _h !== void 0 ? _h : titleTextStyles() }, strings === null || strings === void 0 ? void 0 : strings.title)),
                props.onDismiss && React.createElement(react.IconButton, { iconProps: cancelIcon, ariaLabel: strings === null || strings === void 0 ? void 0 : strings.dismissButtonAriaLabel, "aria-live": 'polite', onClick: props.onDismiss })),
            React.createElement("span", null,
                React.createElement(react.Text, { styles: (_k = (_j = props.styles) === null || _j === void 0 ? void 0 : _j.content) !== null && _k !== void 0 ? _k : messageTextStyles(theme) },
                    " ", strings === null || strings === void 0 ? void 0 :
                    strings.message),
                props.onClickLink && (strings === null || strings === void 0 ? void 0 : strings.linkLabel) && React.createElement(react.Link, { styles: (_m = (_l = props.styles) === null || _l === void 0 ? void 0 : _l.link) !== null && _m !== void 0 ? _m : notificationLinkStyles(theme), onClick: props.onClickLink }, strings === null || strings === void 0 ? void 0 : strings.linkLabel)),
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-evenly" },
                (strings === null || strings === void 0 ? void 0 : strings.secondaryButtonLabel) && React.createElement(react.DefaultButton, { onClick: props.onClickSecondaryButton, styles: (_p = (_o = props.styles) === null || _o === void 0 ? void 0 : _o.secondaryButton) !== null && _p !== void 0 ? _p : {
                        root: {
                            marginTop: '1rem'
                        }
                    } }, strings === null || strings === void 0 ? void 0 : strings.secondaryButtonLabel),
                (strings === null || strings === void 0 ? void 0 : strings.primaryButtonLabel) && React.createElement(react.PrimaryButton, { onClick: props.onClickPrimaryButton, styles: (_r = (_q = props.styles) === null || _q === void 0 ? void 0 : _q.primaryButton) !== null && _r !== void 0 ? _r : {
                        root: {
                            marginTop: '1rem'
                        }
                    } }, strings === null || strings === void 0 ? void 0 : strings.primaryButtonLabel))),
        props.showStackedEffect && React.createElement(react.Stack, { className: hiddenContainerStyles(theme) }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const rttContainerStyles = (theme) => ({
    root: {
        boxShadow: theme.effects.elevation8,
        width: 'fit-content',
        padding: '0.75rem',
        borderRadius: '0.25rem',
        position: 'relative',
        backgroundColor: theme.palette.white
    }
});
/**
 * @private
 */
const rttIconStyles = () => ({
    root: {
        fontSize: '1.5rem',
        paddingRight: '0.5rem'
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 * Banner to disclose that RTT is enabled for all participants for the entire duration of the meeting.
 */
const _RTTDisclosureBanner = (props) => {
    var _a;
    const strings = props.strings;
    const theme = react.useTheme();
    return React.createElement(Notification, { notificationStrings: {
            title: (_a = strings === null || strings === void 0 ? void 0 : strings.bannerTitle) !== null && _a !== void 0 ? _a : '',
            message: strings === null || strings === void 0 ? void 0 : strings.bannerContent,
            linkLabel: strings === null || strings === void 0 ? void 0 : strings.bannerLinkLabel
        }, notificationIconProps: {
            iconName: 'RealTimeTextIcon',
            styles: rttIconStyles()
        }, onClickLink: props.onClickLink, styles: {
            root: rttContainerStyles(theme)
        } });
};

/**
 * Sorts the captions and real time texts based on their timestamp.
 *
 * @private
 */
function sortCaptionsAndRealTimeTexts(captions, realTimeTexts) {
    const combinedList = [...(Array.isArray(captions) ? captions.map(caption => (Object.assign({}, caption))) : []), ...(realTimeTexts ? realTimeTexts.map(realTimeText => (Object.assign({}, realTimeText))) : [])];
    // Sort the combined list by comparing caption's timestamp with realTimeText's updatedTimestamp
    combinedList.sort((a, b) => {
        var _a, _b;
        const timestampA = 'captionText' in a ? new Date((_a = a.createdTimeStamp) !== null && _a !== void 0 ? _a : 0).getTime() : new Date(a.finalizedTimeStamp).getTime();
        const timestampB = 'captionText' in b ? new Date((_b = b.createdTimeStamp) !== null && _b !== void 0 ? _b : 0).getTime() : new Date(b.finalizedTimeStamp).getTime();
        return timestampA - timestampB;
    });
    return combinedList;
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const themedCaptionsSettingsModalStyle = (theme) => ({
    main: {
        borderRadius: theme.effects.roundedCorner6,
        padding: _pxToRem(24),
        width: _pxToRem(440),
        height: 'fit-content',
        overflowY: 'auto'
    }
});
/**
 * @private
 */
const titleClassName$2 = react.mergeStyles({
    fontWeight: 600,
    fontSize: _pxToRem(20),
    lineHeight: _pxToRem(28)
});
/**
 * @private
 */
const titleContainerClassName$2 = react.mergeStyles({
    paddingBottom: _pxToRem(20)
});
/**
 * @private
 */
const dropdownContainerClassName = react.mergeStyles({
    paddingTop: _pxToRem(16)
});
/**
 * @private
 */
const dropdownInfoTextStyle = (theme) => react.mergeStyles({
    fontWeight: 400,
    fontSize: _pxToRem(12),
    lineHeight: _pxToRem(16),
    color: theme.palette.neutralSecondary,
    paddingBottom: _pxToRem(24)
});
/**
 * @private
 */
const buttonsContainerClassName = react.mergeStyles({
    paddingTop: _pxToRem(16)
});
/**
 * @private
 */
const buttonStyles = (theme) => {
    return {
        root: {
            borderRadius: _pxToRem(2),
            margin: _pxToRem(8)
        },
        rootHovered: {
            backgroundColor: theme.palette.themePrimary,
            borderColor: theme.palette.themePrimary,
            color: theme.palette.white
        },
        rootFocused: {
            backgroundColor: theme.palette.themePrimary,
            borderColor: theme.palette.themePrimary,
            color: theme.palette.white
        },
        rootPressed: {
            backgroundColor: theme.palette.themePrimary,
            borderColor: theme.palette.themePrimary,
            color: theme.palette.white
        }
    };
};
/**
 * @private
 */
const defaultButtonStyles = () => {
    return {
        root: {
            borderRadius: _pxToRem(2),
            margin: _pxToRem(8)
        }
    };
};
/**
 * @private
 */
const dropdownStyles = {
    callout: {
        height: _pxToRem(300),
        overflow: 'auto'
    },
    dropdownOptionText: {
        textWrap: 'auto',
        overflow: 'unset'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const SCROLL_OFFSET_ALLOWANCE = 20;
/**
 * @public
 * A component for displaying a CaptionsBanner with user icon, displayName and captions text.
 */
const CaptionsBanner = (props) => {
    var _a, _b, _c;
    const { captions, realTimeTexts, isCaptionsOn, startCaptionsInProgress, onRenderAvatar, formFactor = 'default', captionsOptions, isRealTimeTextOn, onSendRealTimeText, latestLocalRealTimeText } = props;
    const localeStrings = useLocale$1().strings.captionsBanner;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const captionsScrollDivRef = React.useRef(null);
    const [isAtBottomOfScroll, setIsAtBottomOfScroll] = React.useState(true);
    const theme = react.useTheme();
    const [expandBannerHeight, setExpandBannerHeight] = React.useState(false);
    const [announcedRTT, setAnnouncedRTT] = React.useState([]);
    const [announcedCaption, setAnnouncedCaption] = React.useState([]);
    const [captionAnnouncementText, setCaptionAnnouncementText] = React.useState([]);
    const [rttAnnouncementText, setRTTAnnouncementText] = React.useState([]);
    const getTitle = () => {
        var _a, _b, _c;
        if (isCaptionsOn && isRealTimeTextOn) {
            return (_a = strings.captionsAndRealTimeTextContainerTitle) !== null && _a !== void 0 ? _a : '';
        }
        else if (isCaptionsOn) {
            return (_b = strings.captionsOnlyContainerTitle) !== null && _b !== void 0 ? _b : '';
        }
        else if (isRealTimeTextOn) {
            return (_c = strings.realTimeTextOnlyContainerTitle) !== null && _c !== void 0 ? _c : '';
        }
        return '';
    };
    // merge realtimetexts and captions into one array based on timestamp
    // Combine captions and realTimeTexts into one list
    const combinedList = React.useMemo(() => {
        var _a;
        return sortCaptionsAndRealTimeTexts(captions, (_a = realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.completedMessages) !== null && _a !== void 0 ? _a : []);
    }, [captions, realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.completedMessages]);
    const mergedCaptions = React.useMemo(() => {
        var _a;
        return [...combinedList, ...((_a = realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.currentInProgress) !== null && _a !== void 0 ? _a : []), realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.myInProgress].slice(-50);
    }, [combinedList, realTimeTexts]);
    const scrollToBottom = () => {
        if (captionsScrollDivRef.current) {
            captionsScrollDivRef.current.scrollTop = captionsScrollDivRef.current.scrollHeight;
        }
    };
    const handleScrollToTheBottom = React.useCallback(() => {
        if (!captionsScrollDivRef.current) {
            return;
        }
        const atBottom = Math.ceil(captionsScrollDivRef.current.scrollTop) >= captionsScrollDivRef.current.scrollHeight - captionsScrollDivRef.current.clientHeight - SCROLL_OFFSET_ALLOWANCE;
        setIsAtBottomOfScroll(atBottom);
    }, []);
    React.useEffect(() => {
        const captionsScrollDiv = captionsScrollDivRef.current;
        captionsScrollDiv === null || captionsScrollDiv === void 0 ? void 0 : captionsScrollDiv.addEventListener('scroll', handleScrollToTheBottom);
        return () => {
            captionsScrollDiv === null || captionsScrollDiv === void 0 ? void 0 : captionsScrollDiv.removeEventListener('scroll', handleScrollToTheBottom);
        };
    }, [handleScrollToTheBottom, isCaptionsOn, isRealTimeTextOn]);
    React.useEffect(() => {
        // only auto scroll to bottom is already is at bottom of scroll before new caption comes in
        if (isAtBottomOfScroll) {
            scrollToBottom();
        }
    }, [captions, realTimeTexts, isAtBottomOfScroll]);
    const [textFieldValue, setTextFieldValue] = React.useState('');
    React.useEffect(() => {
        // if the latest real time text sent by myself is final, clear the text field
        if (latestLocalRealTimeText && !latestLocalRealTimeText.isTyping) {
            setTextFieldValue('');
        }
    }, [latestLocalRealTimeText]);
    React.useEffect(() => {
        if (realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.completedMessages) {
            //filter out the messages that have already been announced
            const rTTMessagesToAnnounce = realTimeTexts.completedMessages.filter(message => !announcedRTT.includes(message.id));
            if (rTTMessagesToAnnounce.length > 0) {
                setRTTAnnouncementText(rTTMessagesToAnnounce);
                setAnnouncedRTT(prev => [...prev, ...rTTMessagesToAnnounce.map(message => message.id)]);
            }
        }
        if (captions.length > 0) {
            // filter out the captions that have already been announced
            const captionsToAnnounce = captions.filter(caption => !announcedCaption.includes(caption.id) && caption.isFinalized);
            if (captionsToAnnounce.length > 0) {
                setCaptionAnnouncementText(captionsToAnnounce);
                setAnnouncedCaption(prev => [...prev, ...captionsToAnnounce.map(caption => caption.id)]);
            }
        }
    }, [captions, realTimeTexts === null || realTimeTexts === void 0 ? void 0 : realTimeTexts.completedMessages, announcedRTT, announcedCaption]);
    const handleKeyDown = (event) => {
        if (event.key === 'Enter') {
            event.preventDefault();
            if (textFieldValue && onSendRealTimeText) {
                onSendRealTimeText(textFieldValue, true);
                setTextFieldValue('');
            }
        }
    };
    const realTimeTextDisclosureBannerStrings = {
        bannerTitle: (_a = strings.realTimeTextBannerTitle) !== null && _a !== void 0 ? _a : '',
        bannerContent: (_b = strings.realTimeTextBannerContent) !== null && _b !== void 0 ? _b : '',
        bannerLinkLabel: (_c = strings.realTimeTextBannerLinkLabel) !== null && _c !== void 0 ? _c : ''
    };
    const captionsAndRealTimeText = () => {
        return React.createElement(React.Fragment, null, mergedCaptions.filter(caption => caption).map(caption => {
            if ('message' in caption) {
                return React.createElement("div", { key: `RealTimeText - ${caption.id}`, className: captionContainerClassName, "data-is-focusable": true },
                    React.createElement(RealTimeText, Object.assign({}, caption)));
            }
            return React.createElement("div", { key: `Captions - ${caption.id}`, className: captionContainerClassName, "data-is-focusable": true },
                React.createElement(_Caption, Object.assign({}, caption, { onRenderAvatar: onRenderAvatar })));
        }));
    };
    return React.createElement(React.Fragment, null, (startCaptionsInProgress || isCaptionsOn || isRealTimeTextOn) && React.createElement(react.FocusZone, { shouldFocusOnMount: true, className: captionsContainerClassName, "data-ui-id": "captions-banner" },
        (rttAnnouncementText.length > 0 || (captionAnnouncementText === null || captionAnnouncementText === void 0 ? void 0 : captionAnnouncementText.length) > 0) && React.createElement("div", { "aria-live": "assertive", role: "alert", "aria-atomic": "true", className: hiddenAnnouncementClassName },
            React.createElement("span", null, rttAnnouncementText.map(text => React.createElement("span", null,
                strings.realTimeTextBannerTitle,
                " ",
                text.displayName,
                ": ",
                text.message))),
            React.createElement("span", null, captionAnnouncementText.map(text => React.createElement("span", null,
                strings.captionsOnlyContainerTitle,
                " ",
                text.displayName,
                ": ",
                text.captionText)))),
        (isCaptionsOn || isRealTimeTextOn) && formFactor === 'compact' && React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: bannerTitleContainerClassName },
            React.createElement(react.Text, { className: titleClassName$2 }, getTitle()),
            React.createElement(react.IconButton, { "data-ui-id": "captions-banner-expand-icon", iconProps: {
                    iconName: expandBannerHeight ? 'MinimizeIcon' : 'ExpandIcon'
                }, ariaLabel: expandBannerHeight ? strings.minimizeButtonAriaLabel : strings.expandButtonAriaLabel, onClick: () => setExpandBannerHeight(!expandBannerHeight), styles: expandIconClassName(theme) })),
        (isCaptionsOn || isRealTimeTextOn) && React.createElement("div", { ref: captionsScrollDivRef, className: (captionsOptions === null || captionsOptions === void 0 ? void 0 : captionsOptions.height) === 'full' ? captionsBannerFullHeightClassName(theme) : captionsBannerClassName(formFactor, expandBannerHeight), "data-ui-id": "captions-banner-inner", "data-is-focusable": true },
            isRealTimeTextOn && React.createElement(react.Stack, { className: rttDisclosureBannerClassName() },
                React.createElement(_RTTDisclosureBanner, { strings: realTimeTextDisclosureBannerStrings })),
            captionsAndRealTimeText()),
        isRealTimeTextOn && onSendRealTimeText && React.createElement(react.TextField, { styles: realTimeTextInputBoxStyles(theme), placeholder: strings.realTimeTextInputBoxDefaultText, value: textFieldValue, onKeyDown: handleKeyDown, onChange: (_, newValue) => {
                setTextFieldValue(newValue || '');
                onSendRealTimeText(newValue || '', false);
            }, maxLength: 2000, errorMessage: textFieldValue.length >= 2000 ? strings.realTimeTextInputErrorMessage : undefined }),
        !isCaptionsOn && !isRealTimeTextOn && React.createElement(react.Stack, { verticalAlign: "center", styles: (captionsOptions === null || captionsOptions === void 0 ? void 0 : captionsOptions.height) === 'full' ? loadingBannerFullHeightStyles(theme) : loadingBannerStyles(formFactor), "data-is-focusable": true },
            React.createElement(react.Spinner, { label: strings === null || strings === void 0 ? void 0 : strings.captionsBannerSpinnerText, ariaLive: "assertive", labelPosition: "right" }))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$E = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * a button to start or stop captions
 *
 * Can be used with {@link ControlBar}
 *
 * @param props - properties for the start captions button.
 * @public
 */
const StartCaptionsButton = (props) => {
    const { onStartCaptions, onStopCaptions, onSetSpokenLanguage, currentSpokenLanguage } = props;
    const localeStrings = useLocale$1().strings.startCaptionsButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const onRenderStartIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "CaptionsIcon" });
    };
    const onRenderOffIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "CaptionsOffIcon" });
    };
    const options = React.useMemo(() => {
        return {
            spokenLanguage: currentSpokenLanguage === '' ? defaultSpokenLanguage : currentSpokenLanguage
        };
    }, [currentSpokenLanguage]);
    const [hasSetSpokenLanguage, setHasSetSpokenLanguage] = React.useState(false);
    const onToggleStartCaptions = React.useCallback(() => __awaiter$E(void 0, void 0, void 0, function* () {
        if (props.checked) {
            onStopCaptions();
        }
        else {
            yield onStartCaptions(options);
        }
    }), [props.checked, onStartCaptions, onStopCaptions, options]);
    React.useEffect(() => {
        if (props.checked && !hasSetSpokenLanguage) {
            // set spoken language when start captions with a spoken language specified.
            // this is to fix the bug when a second user starts captions with a new spoken language, captions bot ignore that spoken language
            onSetSpokenLanguage(options.spokenLanguage);
            // we only need to call set spoken language once when first starting captions
            setHasSetSpokenLanguage(true);
        }
    }, [props.checked, onSetSpokenLanguage, options.spokenLanguage, hasSetSpokenLanguage]);
    return React.createElement(ControlBarButton, Object.assign({}, props, { strings: strings, onClick: onToggleStartCaptions !== null && onToggleStartCaptions !== void 0 ? onToggleStartCaptions : props.onClick, onRenderOnIcon: onRenderStartIcon, onRenderOffIcon: onRenderOffIcon }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * mapping between caption language and spoken language codes
 */
const _spokenLanguageToCaptionLanguage = {
    'ar-ae': 'ar',
    'ar-sa': 'ar',
    'da-dk': 'da',
    'de-de': 'de',
    'en-au': 'en',
    'en-ca': 'en',
    'en-gb': 'en',
    'en-in': 'en',
    'en-nz': 'en',
    'en-us': 'en',
    'es-es': 'es',
    'es-mx': 'es',
    'fi-fi': 'fi',
    'fr-ca': 'fr-ca',
    'fr-fr': 'fr',
    'hi-in': 'hi',
    'it-it': 'it',
    'ja-jp': 'ja',
    'ko-kr': 'ko',
    'nb-no': 'nb',
    'nl-be': 'nl',
    'nl-nl': 'nl',
    'pl-pl': 'pl',
    'pt-br': 'pt',
    'ru-ru': 'ru',
    'sv-se': 'sv',
    'zh-cn': 'zh-Hans',
    'zh-hk': 'zh-Hant',
    'cs-cz': 'cs',
    'pt-pt': 'pt-pt',
    'tr-tr': 'tr',
    'vi-vn': 'vi',
    'th-th': 'th',
    'he-il': 'he',
    'cy-gb': 'cy',
    'uk-ua': 'uk',
    'el-gr': 'el',
    'hu-hu': 'hu',
    'ro-ro': 'ro',
    'sk-sk': 'sk',
    'zh-tw': 'zh-Hant'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$D = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @public
 * a component for setting spoken languages
 */
const CaptionsSettingsModal = (props) => {
    const { supportedSpokenLanguages, supportedCaptionLanguages, currentSpokenLanguage, currentCaptionLanguage, isCaptionsFeatureActive, showModal, onSetSpokenLanguage, onSetCaptionLanguage, onDismissCaptionsSettings, onStartCaptions, changeCaptionLanguage = false } = props;
    const theme = react.useTheme();
    const localeStrings = useLocale$1().strings.captionsSettingsModal;
    const localSpokenLanguageStrings = useLocale$1().strings.spokenLanguages;
    const localCaptionLanguageStrings = useLocale$1().strings.captionLanguages;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const spokenLanguageStrings = React.useMemo(() => {
        return Object.assign(Object.assign({}, localSpokenLanguageStrings), props.spokenLanguageStrings);
    }, [localSpokenLanguageStrings, props.spokenLanguageStrings]);
    const captionLanguageStrings = React.useMemo(() => {
        return Object.assign(Object.assign({}, localCaptionLanguageStrings), props.captionLanguageStrings);
    }, [localCaptionLanguageStrings, props.captionLanguageStrings]);
    const [hasSetSpokenLanguage, setHasSetSpokenLanguage] = React.useState(false);
    const [selectedSpokenLanguage, setSelectedSpokenLanguage] = React.useState({
        key: currentSpokenLanguage || defaultSpokenLanguage,
        text: currentSpokenLanguage || defaultSpokenLanguage
    });
    const [selectedCaptionLanguage, setSelectedCaptionLanguage] = React.useState({
        key: currentCaptionLanguage || _spokenLanguageToCaptionLanguage[selectedSpokenLanguage.key],
        text: currentCaptionLanguage || _spokenLanguageToCaptionLanguage[selectedSpokenLanguage.key]
    });
    React.useEffect(() => {
        // set spoken language when start captions with a spoken language specified.
        // this is to fix the bug when a second user starts captions with a new spoken language, captions bot ignore that spoken language
        if (isCaptionsFeatureActive && !hasSetSpokenLanguage) {
            onSetSpokenLanguage(selectedSpokenLanguage.key);
            // we only need to call set spoken language once when first starting captions
            setHasSetSpokenLanguage(true);
        }
    }, [isCaptionsFeatureActive, onSetSpokenLanguage, selectedSpokenLanguage.key, hasSetSpokenLanguage]);
    const onDismiss = React.useCallback(() => {
        if (onDismissCaptionsSettings) {
            onDismissCaptionsSettings();
        }
    }, [onDismissCaptionsSettings]);
    const onConfirm = React.useCallback(() => __awaiter$D(void 0, void 0, void 0, function* () {
        const spokenLanguageCode = selectedSpokenLanguage.key;
        const captionLanguageCode = selectedCaptionLanguage.key;
        if (isCaptionsFeatureActive) {
            onSetSpokenLanguage(spokenLanguageCode);
            if (changeCaptionLanguage) {
                onSetCaptionLanguage && onSetCaptionLanguage(captionLanguageCode);
            }
        }
        else {
            yield onStartCaptions({
                spokenLanguage: spokenLanguageCode
            });
        }
        onDismiss();
    }), [onDismiss, isCaptionsFeatureActive, onSetSpokenLanguage, onSetCaptionLanguage, onStartCaptions, selectedSpokenLanguage.key, selectedCaptionLanguage.key, changeCaptionLanguage]);
    const spokenLanguageDropdownOptions = React.useMemo(() => {
        return supportedSpokenLanguages.map(languageCode => {
            return {
                key: languageCode,
                text: spokenLanguageStrings ? spokenLanguageStrings[languageCode] : languageCode
            };
        });
    }, [supportedSpokenLanguages, spokenLanguageStrings]);
    const captionLanguageDropdownOptions = React.useMemo(() => {
        return supportedCaptionLanguages === null || supportedCaptionLanguages === void 0 ? void 0 : supportedCaptionLanguages.map(languageCode => {
            return {
                key: languageCode,
                text: captionLanguageStrings ? captionLanguageStrings[languageCode] : languageCode
            };
        });
    }, [supportedCaptionLanguages, captionLanguageStrings]);
    const sortedSpokenLanguageDropdownOptions = React.useMemo(() => {
        const copy = [...spokenLanguageDropdownOptions];
        return copy.sort((a, b) => a.text > b.text ? 1 : -1);
    }, [spokenLanguageDropdownOptions]);
    const sortedCaptionLanguageDropdownOptions = React.useMemo(() => {
        const copy = [...(captionLanguageDropdownOptions !== null && captionLanguageDropdownOptions !== void 0 ? captionLanguageDropdownOptions : [])];
        return copy.sort((a, b) => a.text > b.text ? 1 : -1);
    }, [captionLanguageDropdownOptions]);
    const onSpokenLanguageChange = (event, option) => {
        if (option) {
            setSelectedSpokenLanguage(option);
        }
    };
    const onCaptionLanguageChange = (event, option) => {
        if (option) {
            setSelectedCaptionLanguage(option);
        }
    };
    const calloutProps = React.useMemo(() => ({
        preventDismissOnEvent: _preventDismissOnEvent
    }), []);
    const CaptionsSettingsComponent = React.useCallback(() => {
        const placeholderSpokenLanguage = currentSpokenLanguage !== null && currentSpokenLanguage !== void 0 ? currentSpokenLanguage : defaultSpokenLanguage;
        const placeholderCaptionLanguage = currentCaptionLanguage !== null && currentCaptionLanguage !== void 0 ? currentCaptionLanguage : _spokenLanguageToCaptionLanguage[placeholderSpokenLanguage];
        return React.createElement(react.Stack, null,
            React.createElement(react.Dropdown, { label: strings === null || strings === void 0 ? void 0 : strings.captionsSettingsSpokenLanguageDropdownLabel, selectedKey: selectedSpokenLanguage ? selectedSpokenLanguage.key : undefined, onChange: (ev, option) => onSpokenLanguageChange(ev, option), calloutProps: calloutProps, placeholder: placeholderSpokenLanguage, options: sortedSpokenLanguageDropdownOptions, styles: dropdownStyles }),
            React.createElement(react.Text, { className: dropdownInfoTextStyle(theme) }, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsSpokenLanguageDropdownInfoText),
            changeCaptionLanguage && React.createElement(React.Fragment, null,
                React.createElement(react.Dropdown, { label: strings === null || strings === void 0 ? void 0 : strings.captionsSettingsCaptionLanguageDropdownLabel, selectedKey: selectedCaptionLanguage ? selectedCaptionLanguage.key : undefined, onChange: (ev, option) => onCaptionLanguageChange(ev, option), calloutProps: calloutProps, placeholder: placeholderCaptionLanguage, options: sortedCaptionLanguageDropdownOptions, styles: dropdownStyles }),
                React.createElement(react.Text, { className: dropdownInfoTextStyle(theme) }, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsCaptionLanguageDropdownInfoText)));
    }, [calloutProps, currentSpokenLanguage, currentCaptionLanguage, sortedSpokenLanguageDropdownOptions, sortedCaptionLanguageDropdownOptions, selectedCaptionLanguage, selectedSpokenLanguage, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsSpokenLanguageDropdownInfoText, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsCaptionLanguageDropdownLabel, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsSpokenLanguageDropdownLabel, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsCaptionLanguageDropdownInfoText, theme, changeCaptionLanguage]);
    const CaptionsSettingsModalStyle = React.useMemo(() => themedCaptionsSettingsModalStyle(theme), [theme]);
    return React.createElement(React.Fragment, null, React.createElement(react.Modal, { titleAriaId: strings === null || strings === void 0 ? void 0 : strings.captionsSettingsModalAriaLabel, isOpen: showModal, onDismiss: onDismiss, isBlocking: true, styles: CaptionsSettingsModalStyle },
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: titleContainerClassName$2 },
            React.createElement(react.Text, { className: titleClassName$2, role: "heading", "aria-level": 1 }, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsModalTitle),
            React.createElement(react.IconButton, { iconProps: {
                    iconName: 'Cancel'
                }, ariaLabel: strings === null || strings === void 0 ? void 0 : strings.captionsSettingsCloseModalButtonAriaLabel, onClick: onDismiss, style: {
                    color: theme.palette.black
                } })),
        React.createElement(react.Stack, { className: dropdownContainerClassName }, CaptionsSettingsComponent()),
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: "end", className: buttonsContainerClassName },
            React.createElement(react.PrimaryButton, { styles: buttonStyles(theme), onClick: onConfirm },
                React.createElement("span", null, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsConfirmButtonLabel)),
            React.createElement(react.DefaultButton, { onClick: onDismiss, styles: defaultButtonStyles() },
                React.createElement("span", null, strings === null || strings === void 0 ? void 0 : strings.captionsSettingsCancelButtonLabel)))));
};

var __awaiter$C = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @public
 * a component for realTimeText modal
 */
const RealTimeTextModal = (props) => {
    const { showModal, onDismissModal, onStartRealTimeText } = props;
    const localeStrings = useLocale$1().strings.realTimeTextModal;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const theme = react.useTheme();
    const onDismiss = React.useCallback(() => {
        if (onDismissModal) {
            onDismissModal();
        }
    }, [onDismissModal]);
    const onConfirm = React.useCallback(() => __awaiter$C(void 0, void 0, void 0, function* () {
        if (onStartRealTimeText) {
            yield onStartRealTimeText();
        }
        onDismiss();
    }), [onDismiss, onStartRealTimeText]);
    const RealTimeTextModalStyle = React.useMemo(() => themedCaptionsSettingsModalStyle(theme), [theme]);
    return React.createElement(React.Fragment, null, React.createElement(react.Modal, { titleAriaId: strings === null || strings === void 0 ? void 0 : strings.realTimeTextModalAriaLabel, isOpen: showModal, onDismiss: onDismiss, isBlocking: true, styles: RealTimeTextModalStyle },
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: titleContainerClassName$2 },
            React.createElement(react.Text, { role: "heading", className: titleClassName$2, "aria-level": 1 }, strings === null || strings === void 0 ? void 0 : strings.realTimeTextModalTitle),
            React.createElement(react.IconButton, { iconProps: {
                    iconName: 'Cancel'
                }, ariaLabel: strings === null || strings === void 0 ? void 0 : strings.realTimeTextCloseModalButtonAriaLabel, onClick: onDismiss, style: {
                    color: theme.palette.black
                } })),
        React.createElement(react.Text, null, strings === null || strings === void 0 ? void 0 : strings.realTimeTextModalText),
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: "end", className: buttonsContainerClassName },
            React.createElement(react.PrimaryButton, { styles: buttonStyles(theme), onClick: onConfirm, "data-ui-id": "realTimeText-modal-confirm-button" },
                React.createElement("span", null, strings === null || strings === void 0 ? void 0 : strings.realTimeTextConfirmButtonLabel)),
            React.createElement(react.DefaultButton, { onClick: onDismiss, styles: defaultButtonStyles() },
                React.createElement("span", null, strings === null || strings === void 0 ? void 0 : strings.realTimeTextCancelButtonLabel)))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const themedPhoneInfoModalStyle = (theme) => ({
    main: {
        borderRadius: theme.effects.roundedCorner6,
        padding: _pxToRem(24),
        width: _pxToRem(600),
        height: 'fit-content',
        overflow: 'hidden'
    }
});
/**
 * @private
 */
const titleClassName$1 = react.mergeStyles({
    fontWeight: 600,
    fontSize: _pxToRem(20),
    lineHeight: _pxToRem(28)
});
/**
 * @private
 */
const titleContainerClassName$1 = react.mergeStyles({
    paddingBottom: _pxToRem(30)
});
/**
 * @private
 */
const stepTextStyle = react.mergeStyles({
    fontSize: _pxToRem(14),
    lineHeight: _pxToRem(40)
});
/**
 * @private
 */
const phoneInfoTextStyle$1 = react.mergeStyles({
    fontSize: _pxToRem(14),
    lineHeight: _pxToRem(40),
    fontWeight: react.FontWeights.semibold
});
/**
 * @private
 */
const phoneInfoIcon$1 = (theme) => {
    return react.mergeStyles({
        background: `${theme.palette.themeLighter}`,
        height: _pxToRem(36),
        width: _pxToRem(36),
        marginRight: _pxToRem(12),
        borderRadius: _pxToRem(18)
    });
};
/**
 * @private
 */
const phoneInfoInctructionLine = react.mergeStyles({
    marginBottom: _pxToRem(20)
});
/**
 * @private
 */
const phoneInfoStep$1 = react.mergeStyles({
    textAlign: 'right'
});
/**
 * @private
 */
const phoneInfoIconStyle$1 = (theme) => {
    return react.mergeStyles({
        padding: _pxToRem(8),
        color: `${theme.palette.themePrimary}`,
        zIndex: 2
    });
};
/**
 * @private
 */
const infoConnectionLinkStyle$1 = (theme) => {
    return react.mergeStyles({
        background: `${theme.palette.themeLighter}`,
        width: _pxToRem(2),
        position: 'relative',
        left: _pxToRem(19),
        top: _pxToRem(20),
        zIndex: 1
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @public
 * a component for setting spoken languages
 */
const MeetingConferencePhoneInfoModal = (props) => {
    var _a;
    const { conferencePhoneInfoList, showModal, onDismissMeetingPhoneInfoSettings } = props;
    const theme = react.useTheme();
    const strings = useLocale$1().strings.meetingConferencePhoneInfo;
    const onDismiss = React.useCallback(() => {
        if (onDismissMeetingPhoneInfoSettings) {
            onDismissMeetingPhoneInfoSettings();
        }
    }, [onDismissMeetingPhoneInfoSettings]);
    const PhoneInfoModalStyle = React.useMemo(() => themedPhoneInfoModalStyle(theme), [theme]);
    return React.createElement(React.Fragment, null,
        React.createElement(react.Modal, { titleAriaId: strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalTitle, isOpen: showModal, onDismiss: onDismiss, isBlocking: true, styles: PhoneInfoModalStyle },
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: titleContainerClassName$1 },
                React.createElement(react.Text, { role: "heading", "aria-level": 2, className: titleClassName$1 }, strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalTitle),
                React.createElement(react.IconButton, { iconProps: {
                        iconName: 'Cancel'
                    }, ariaLabel: strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalTitle, onClick: onDismiss, style: {
                        color: theme.palette.black
                    } })),
            conferencePhoneInfoList.length === 0 && React.createElement(react.Stack, { horizontal: true },
                React.createElement(react.Text, { className: stepTextStyle }, strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalNoPhoneAvailable)),
            conferencePhoneInfoList.length > 0 && React.createElement(react.Stack, null,
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", className: phoneInfoInctructionLine },
                    React.createElement(react.Stack.Item, { style: {
                            display: 'flex'
                        } },
                        React.createElement(react.Stack, { horizontal: true, className: phoneInfoStep$1 },
                            React.createElement(react.Stack, { className: infoConnectionLinkStyle$1(theme) }),
                            React.createElement(react.Stack.Item, { className: phoneInfoIcon$1(theme) },
                                React.createElement(react.Stack, { verticalAlign: "center", horizontalAlign: "center" },
                                    React.createElement(react.Icon, { iconName: "JoinByPhoneDialStepIcon", className: phoneInfoIconStyle$1(theme) }))),
                            React.createElement(react.Stack.Item, null,
                                React.createElement(react.Text, { className: stepTextStyle }, strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalDialIn)))),
                    React.createElement(react.Stack.Item, { className: phoneInfoStep$1 }, conferencePhoneInfoList.map((phoneNumber, index) => React.createElement(react.Stack.Item, { key: index },
                        React.createElement(react.Text, { className: phoneInfoTextStyle$1 },
                            _formatPhoneNumber(phoneNumber.phoneNumber, true),
                            ' ',
                            phoneNumber.isTollFree ? strings.meetingConferencePhoneInfoModalTollFree : strings.meetingConferencePhoneInfoModalToll),
                        React.createElement("br", null),
                        React.createElement(react.Text, { className: phoneInfoTextStyle$1 },
                            " ",
                            formatPhoneNumberInfo(phoneNumber, strings)))))),
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: phoneInfoInctructionLine },
                    React.createElement(react.Stack.Item, { style: {
                            display: 'flex'
                        } },
                        React.createElement(react.Stack, { horizontal: true },
                            React.createElement(react.Stack, { className: infoConnectionLinkStyle$1(theme) }),
                            React.createElement(react.Stack.Item, { className: phoneInfoIcon$1(theme) },
                                React.createElement(react.Stack, { verticalAlign: "center", horizontalAlign: "center" },
                                    React.createElement(react.Icon, { iconName: "JoinByPhoneConferenceIdIcon", className: phoneInfoIconStyle$1(theme) }))),
                            React.createElement(react.Stack.Item, null,
                                React.createElement(react.Text, { className: stepTextStyle }, strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalMeetingId)))),
                    React.createElement(react.Text, { className: phoneInfoTextStyle$1 }, formatMeetingId((_a = conferencePhoneInfoList[0]) === null || _a === void 0 ? void 0 : _a.conferenceId))),
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center" },
                    React.createElement(react.Stack, { horizontal: true },
                        React.createElement(react.Stack.Item, { className: phoneInfoIcon$1(theme), style: {
                                marginLeft: _pxToRem(2)
                            } },
                            React.createElement(react.Icon, { iconName: "JoinByPhoneWaitToBeAdmittedIcon", className: phoneInfoIconStyle$1(theme) })),
                        React.createElement(react.Stack.Item, null,
                            React.createElement(react.Text, { className: stepTextStyle }, strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalWait)))))));
};
/**
 * @internal
 * format phone number link
 */
const formatPhoneNumberLink = (phoneNumber) => {
    return `tel:+${phoneNumber.phoneNumber},,${phoneNumber.conferenceId}#`;
};
/**
 * @internal
 * format phone number
 */
const formatPhoneNumberInfo = (phoneNumber, strings) => {
    const templateText = phoneNumber.country && phoneNumber.city ? strings === null || strings === void 0 ? void 0 : strings.meetingConferencePhoneInfoModalTollGeoData : '';
    return (templateText === null || templateText === void 0 ? void 0 : templateText.replace('{country}', phoneNumber.country || '').replace('{city}', phoneNumber.city || '').trim()) || '';
};
/**
 * @internal
 * format meeting id
 */
const formatMeetingId = (meetingId) => {
    if (!meetingId) {
        return '';
    }
    if ((meetingId === null || meetingId === void 0 ? void 0 : meetingId.length) !== 9) {
        return meetingId;
    }
    return [meetingId.slice(0, 3), meetingId.slice(3, 6), meetingId.slice(6, 9)].join(' ') + '#';
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 * ErrorBoundary component to handle errors in React components.
 */
class _ErrorBoundary extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            hasError: false
        };
    }
    static getDerivedStateFromError() {
        // Update state so the next render will show the fallback UI.
        return {
            hasError: true
        };
    }
    componentDidCatch(error) {
        console.error(error);
    }
    render() {
        if (this.state.hasError) {
            // handle error
            return this.props.fallback;
        }
        // otherwise render children
        return this.props.children;
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A component to show notifications on the UI.
 * All strings that can be shown are accepted as the {@link NotificationStackProps.strings} so that they can be localized.
 * Active notifications are selected by {@link NotificationStackProps.activeNotifications}.
 *
 * This component internally tracks dismissed by the user.
 *   * Notifications that have an associated timestamp: The notification is shown on the UI again if it occurs after being dismissed.
 *   * Notifications that do not have a timestamp: The notification is dismissed until it disappears from the props.
 *         If the notification recurs, it is shown in the UI.
 *
 *
 * @public
 */
const NotificationStack = (props) => {
    var _a, _b;
    const localeStrings = useLocale$1().strings.notificationStack;
    const strings = (_a = props.strings) !== null && _a !== void 0 ? _a : localeStrings;
    const maxNotificationsToShow = (_b = props.maxNotificationsToShow) !== null && _b !== void 0 ? _b : 2;
    const trackDismissedNotificationsInternally = !props.onDismissNotification;
    // Timestamp for when this comopnent is first mounted.
    // Never updated through the lifecycle of this component.
    const mountTimestamp = React.useRef(new Date(Date.now()));
    const [dismissedNotifications, setDismissedNotifications] = React.useState([]);
    // dropDismissalsForInactiveNotifications only returns a new object if `dismissedErrors` actually changes.
    // Without this behaviour, this `useEffect` block would cause a render loop.
    React.useEffect(() => {
        trackDismissedNotificationsInternally && setDismissedNotifications(dropDismissalsForInactiveNotifications(props.activeNotifications, dismissedNotifications));
    }, [props.activeNotifications, dismissedNotifications, trackDismissedNotificationsInternally]);
    const activeNotifications = notificationsToShow(props.activeNotifications, dismissedNotifications, props.ignorePremountNotifications ? mountTimestamp.current : undefined);
    return React.createElement(react.Stack, { "data-ui-id": "notifications-stack", style: {
            width: 'fit-content'
        } }, activeNotifications.map((notification, index) => {
        if (index < maxNotificationsToShow) {
            const onDismiss = () => {
                var _a;
                trackDismissedNotificationsInternally ? setDismissedNotifications(dismissNotification(dismissedNotifications, notification)) : (_a = props.onDismissNotification) === null || _a === void 0 ? void 0 : _a.call(props, notification);
                notification.onDismiss && notification.onDismiss();
            };
            if (notification.type === 'assignedBreakoutRoomOpenedPromptJoin') {
                // If notification is of type assignedBreakoutRoomOpenedPromptJoin then set onClickSecondaryButton to
                // onDismiss if it is not defined
                notification.onClickSecondaryButton = notification.onClickSecondaryButton ? notification.onClickSecondaryButton : () => onDismiss();
            }
            return React.createElement("div", { key: index, style: {
                    marginBottom: `${index === maxNotificationsToShow - 1 ? 0 : '0.25rem'}`
                } },
                React.createElement(Notification, { notificationStrings: strings ? strings[notification.type] : undefined, notificationIconProps: NotificationIconProps(notification.type), onClickPrimaryButton: () => { var _a; return (_a = notification.onClickPrimaryButton) === null || _a === void 0 ? void 0 : _a.call(notification); }, onClickSecondaryButton: () => { var _a; return (_a = notification.onClickSecondaryButton) === null || _a === void 0 ? void 0 : _a.call(notification); }, onDismiss: onDismiss, showStackedEffect: index === maxNotificationsToShow - 1 && activeNotifications.length > maxNotificationsToShow, autoDismiss: notification.autoDismiss, ariaLive: notification.ariaLive }));
        }
        else {
            return React.createElement(React.Fragment, null);
        }
    }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A Notification component that is to be used to represent incoming calls to the end user.
 * Allows the user to accept or reject the incoming call.
 * @public
 */
const IncomingCallNotification$n = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const { callerName, alertText, avatarImage, onAcceptWithAudio, onAcceptWithVideo, onReject, onDismiss, personaSize, styles, strings, acceptOptions } = props;
    const theme = react.useTheme();
    const localeStrings = useLocale$1().strings.IncomingCallNotification;
    const formattedMessageString = localeStrings.incomingCallNotificationPlaceholderAlert && callerName ? _formatString(localeStrings.incomingCallNotificationPlaceholderAlert, {
        callerName: callerName
    }) : callerName;
    const acceptManuOptions = acceptOptions.showAcceptWithVideo ? {
        items: [{
                key: 'audio',
                text: (_a = strings === null || strings === void 0 ? void 0 : strings.incomingCallNotificationAcceptButtonLabel) !== null && _a !== void 0 ? _a : localeStrings.incomingCallNotificationAcceptButtonLabel,
                onClick: () => onAcceptWithAudio()
            }, {
                key: 'video',
                text: (_b = strings === null || strings === void 0 ? void 0 : strings.incomingCallNotificationAccceptWithVideoButtonLabel) !== null && _b !== void 0 ? _b : localeStrings.incomingCallNotificationAccceptWithVideoButtonLabel,
                onClick: () => onAcceptWithVideo()
            }]
    } : undefined;
    return React.createElement(react.Stack, { tokens: {
            childrenGap: '0.5rem'
        }, verticalAlign: "center", styles: (styles === null || styles === void 0 ? void 0 : styles.root) ? styles.root : incomingCallToastStyle(theme) },
        React.createElement(react.Stack, { horizontal: true, verticalAlign: "center", tokens: {
                childrenGap: personaSize ? '0.5rem' : '0rem'
            } },
            React.createElement(react.Stack, { horizontalAlign: "start", styles: (styles === null || styles === void 0 ? void 0 : styles.avatarContainer) ? styles.avatarContainer : incomingCallToastAvatarContainerStyle },
                React.createElement(react.Persona, { imageUrl: avatarImage, text: callerName, size: react.PersonaSize.size24, coinSize: personaSize, hidePersonaDetails: true, "aria-label": callerName })),
            React.createElement(react.Stack, { grow: true, horizontalAlign: "center", style: {
                    alignItems: 'flex-start',
                    fontFamily: 'Segoe UI'
                } },
                React.createElement(react.Text, { tabIndex: 0, "aria-live": 'assertive', "aria-label": alertText !== null && alertText !== void 0 ? alertText : formattedMessageString }, (_c = alertText !== null && alertText !== void 0 ? alertText : strings === null || strings === void 0 ? void 0 : strings.incomingCallNotificationPlaceholderAlert) !== null && _c !== void 0 ? _c : formattedMessageString)),
            React.createElement(react.IconButton, { ariaLabel: (_d = strings === null || strings === void 0 ? void 0 : strings.incomingCallNotificationDismissButtonAriaLabel) !== null && _d !== void 0 ? _d : localeStrings.incomingCallNotificationDismissButtonAriaLabel, iconProps: {
                    iconName: 'cancel'
                }, onClick: onDismiss, styles: dismissButtonStyle(theme) })),
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: 'center', tokens: {
                childrenGap: '0.5rem'
            } },
            React.createElement(react.PrimaryButton, { styles: (styles === null || styles === void 0 ? void 0 : styles.acceptButton) ? styles.acceptButton : incomingCallAcceptButtonStyle(theme), onClick: () => onAcceptWithAudio(), iconProps: {
                    iconName: 'IncomingCallNotificationAcceptIcon',
                    style: {
                        lineHeight: '1rem'
                    }
                }, ariaLabel: (_e = strings === null || strings === void 0 ? void 0 : strings.incomingCallNoticicationAcceptWithAudioAriaLabel) !== null && _e !== void 0 ? _e : localeStrings.incomingCallNoticicationAcceptWithAudioAriaLabel, menuProps: acceptManuOptions }, (_f = strings === null || strings === void 0 ? void 0 : strings.incomingCallNotificationAcceptButtonLabel) !== null && _f !== void 0 ? _f : localeStrings.incomingCallNotificationAcceptButtonLabel),
            React.createElement(react.DefaultButton, { styles: (styles === null || styles === void 0 ? void 0 : styles.rejectButton) ? styles.rejectButton : incomingCallRejectButtonStyle(theme), onClick: () => onReject(), iconProps: {
                    iconName: 'IncomingCallNotificationRejectIcon'
                }, ariaLabel: (_g = strings === null || strings === void 0 ? void 0 : strings.incomingCallNoticicationRejectAriaLabel) !== null && _g !== void 0 ? _g : localeStrings.incomingCallNoticicationRejectAriaLabel }, (_h = strings === null || strings === void 0 ? void 0 : strings.incomingCallNotificationRejectButtonLabel) !== null && _h !== void 0 ? _h : localeStrings.incomingCallNotificationRejectButtonLabel)));
};
const incomingCallToastStyle = (theme) => {
    return {
        root: {
            width: '20rem',
            background: theme.palette.white,
            opacity: 0.95,
            borderRadius: '0.5rem',
            boxShadow: theme.effects.elevation8,
            padding: '1rem'
        }
    };
};
const incomingCallToastAvatarContainerStyle = {
    root: {
        marginRight: '0.5rem'
    }
};
const dismissButtonStyle = (theme) => {
    return {
        root: {
            color: theme.palette.neutralPrimary,
            width: '1rem',
            height: '1rem'
        },
        rootHovered: {
            color: theme.palette.neutralDark
        }
    };
};
const incomingCallAcceptButtonStyle = (theme) => {
    return {
        root: {
            color: theme.palette.white,
            border: 'none',
            borderRadius: theme.effects.roundedCorner4,
            width: '100%',
            maxWidth: '8.875rem'
        },
        rootHovered: {
            color: theme.palette.white
        },
        icon: {
            height: '1.25rem'
        }
    };
};
const incomingCallRejectButtonStyle = (theme) => {
    return {
        root: {
            borderRadius: theme.effects.roundedCorner4,
            maxWidth: '8.875rem',
            width: '100%'
        },
        icon: {
            height: '1.25rem'
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Wrapper to manage multiple incoming calls
 * @param props - {@link IncomingCallManagerProps}
 * @returns
 * @public
 */
const IncomingCallStack = (props) => {
    const { activeIncomingCalls, removedIncomingCalls, onAcceptCall, onRejectCall, styles, strings, tabIndex } = props;
    return React.createElement(react.Stack, { tokens: {
            childrenGap: '0.25rem'
        }, role: 'group', tabIndex: tabIndex }, activeIncomingCalls.filter(incomingCall => !removedIncomingCalls.some(call => call.id === incomingCall.id)).map(incomingCall => {
        return React.createElement(IncomingCallNotification$n, { key: incomingCall.id, callerName: incomingCall.callerInfo.displayName, onAcceptWithAudio: () => onAcceptCall(incomingCall.id), onAcceptWithVideo: () => onAcceptCall(incomingCall.id, true), onReject: () => onRejectCall(incomingCall.id), onDismiss: () => onRejectCall(incomingCall.id), styles: styles, strings: strings, acceptOptions: {
                showAcceptWithVideo: incomingCall.videoAvailable
            } });
    }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * a button to start RealTimeText
 * based on accessibility requirement, real time text cannot be turned off once it is on
 *
 * Can be used with {@link ControlBar}
 *
 * @param props - properties for the start RealTimeText button.
 * @public
 */
const StartRealTimeTextButton = (props) => {
    const { onStartRealTimeText, isRealTimeTextOn } = props;
    const localeStrings = useLocale$1().strings.startRealTimeTextButton;
    const strings = Object.assign(Object.assign({}, localeStrings), props.strings);
    const onRenderStartIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "RealTimeTextIcon" });
    };
    return React.createElement(ControlBarButton, Object.assign({}, props, { strings: strings, onClick: onStartRealTimeText, onRenderOffIcon: onRenderStartIcon, disabled: props.disabled || isRealTimeTextOn }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 *
 * @private
 */
const createComponentStrings = (localizedStrings) => {
    const strings = Object.assign({}, en_US$1);
    _getKeys(localizedStrings).forEach(key => {
        // mark the value as unknown because the type changes based on the key.
        // this is unsafe at runtime as we could assign the wrong type based on the key here.
        // but typescript isn't smart enough to know that the key used across each access will result in the same type
        strings[key] = Object.assign(Object.assign({}, strings[key]), localizedStrings[key]);
    });
    return strings;
};

var participantItem$m={isMeText:"(you)",menuTitle:"More Options",removeButtonLabel:"Remove",sharingIconLabel:"Sharing",mutedIconLabel:"Muted",handRaisedIconLabel:"Hand raised - Position {position}",displayNamePlaceholder:"Unnamed participant",participantStateRinging:"Calling...",participantStateHold:"On hold",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, More Options",attendeeRole:"Attendee",micDisabledIconLabel:"Micophone disabled",cameraDisabledIconLabel:"Camera disabled"};var ParticipantList$m={overflowParticipantCount:"+{overflowCount} more"};var typingIndicator$m={singleUser:"{user} is typing ...",multipleUsers:"{users} are typing ...",multipleUsersAbbreviateOne:"{users} and 1 other are typing ...",multipleUsersAbbreviateMany:"{users} and {numOthers} others are typing ...",delimiter:", "};var sendBox$m={placeholderText:"Enter a message",textTooLong:"Your message length is over the maximum limit.",sendButtonAriaLabel:"Send message",attachmentUploadsPendingError:"File is uploading, please wait.",removeAttachment:"Remove file",uploading:"Uploading",uploadCompleted:"Upload completed",attachmentMoreMenu:"More Options"};var richTextSendBox$m={placeholderText:"Enter a message",textTooLong:"Your message length is over the maximum limit.",sendButtonAriaLabel:"Send message",attachmentUploadsPendingError:"File is uploading, please wait.",imageUploadsPendingError:"Image is uploading, please wait.",removeAttachment:"Remove file",uploading:"Uploading",uploadCompleted:"Upload completed",richTextBoldTooltip:"Bold",richTextItalicTooltip:"Italic",richTextUnderlineTooltip:"Underline",richTextBulletListTooltip:"Bulleted list",richTextNumberListTooltip:"Numbered list",richTextIncreaseIndentTooltip:"Increase indent",richTextDecreaseIndentTooltip:"Decrease indent",richTextInsertTableTooltip:"Insert table",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insert",richTextInsertTableMenuTitle:"Insert {column} x {row} table",richTextInsertRowAboveMenu:"Insert above",richTextInsertRowBelowMenu:"Insert below",richTextInsertColumnLeftMenu:"Insert left",richTextInsertColumnRightMenu:"Insert right",richTextDeleteRowOrColumnMenu:"Delete",richTextDeleteColumnMenu:"Delete column",richTextDeleteRowMenu:"Delete row",richTextDeleteTableMenu:"Delete table",richTextToolbarAriaLabel:"Message Formatting Toolbar",richTextToolbarMoreButtonAriaLabel:"More options",attachmentMoreMenu:"More Options",richTextNewBulletedListItemAnnouncement:"Bullet, new line",richTextNewNumberedListItemAnnouncement:"{0}, new line",richTextBulletedListAppliedAnnouncement:"Bulleted list applied",richTextNumberedListAppliedAnnouncement:"Numbered list applied"};var realTimeTextModal$m={realTimeTextModalAriaLabel:"Turn on RTT Modal",realTimeTextModalTitle:"Turn on RTT?",realTimeTextModalText:"Real-time text (RTT) will be turned on for everyone in the meeting.",realTimeTextConfirmButtonLabel:"Turn on",realTimeTextCancelButtonLabel:"Cancel",realTimeTextCloseModalButtonAriaLabel:"Close RTT Modal"};var captionsBanner$m={captionsBannerSpinnerText:"Starting captions...",realTimeTextInputBoxDefaultText:"Type message in real time",realTimeTextInputErrorMessage:"Message cannot exceed 2000 characters",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (real-time text) is enabled for all participants for the entire duration of the meeting.",realTimeTextBannerLinkLabel:"Learn more",captionsOnlyContainerTitle:"Captions",realTimeTextOnlyContainerTitle:"Real-Time Text",captionsAndRealTimeTextContainerTitle:"Captions and Real-Time Text",expandButtonAriaLabel:"Expand",minimizeButtonAriaLabel:"Collapse"};var mentionPopover$m={mentionPopoverHeader:"Suggestions"};var imageOverlay$m={downloadButtonLabel:"Download",dismissButtonAriaLabel:"Close",overlayVisibleMessage:"Image Details Overlay"};var messageStatusIndicator$m={deliveredAriaLabel:"Message sent",deliveredTooltipText:"Sent",seenAriaLabel:"Message seen by others",seenTooltipText:"Seen",readByTooltipText:"Read by {messageThreadReadCount} of {remoteParticipantsCount}",sendingAriaLabel:"Message sending",sendingTooltipText:"Sending",failedToSendAriaLabel:"Message failed to send",failedToSendTooltipText:"Failed to send"};var endCallButton$m={label:"Leave",tooltipContent:"Leave call",leaveOption:"Leave",endCallOption:"End call for everyone"};var cameraButton$m={onLabel:"Camera",offLabel:"Camera",tooltipDisabledContent:"Camera is disabled",tooltipOnContent:"Turn off camera",tooltipOffContent:"Turn on camera",tooltipVideoLoadingContent:"Video is loading",cameraMenuTitle:"Camera",cameraMenuTooltip:"Choose camera",cameraButtonSplitRoleDescription:"Split button",onSplitButtonAriaLabel:"Turn off camera",offSplitButtonAriaLabel:"Turn on camera",cameraActionTurnedOnAnnouncement:"Your camera has been turned on",cameraActionTurnedOffAnnouncement:"Your camera has been turned off",onSplitButtonPrimaryActionCamera:"Turn off camera",offSplitButtonPrimaryActionCamera:"Turn on camera",cameraPrimaryActionSplitButtonTitle:"Use camera",videoEffectsMenuItemTitle:"Effects",cameraButtonAriaDescription:"camera options"};var microphoneButton$m={onLabel:"Mic",offLabel:"Mic",tooltipDisabledContent:"Microphone is disabled",tooltipOnContent:"Mute microphone",tooltipOffContent:"Unmute microphone",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choose microphone",speakerMenuTitle:"Speaker",speakerMenuTooltip:"Choose speaker",microphoneButtonSplitRoleDescription:"Split button",onSplitButtonAriaLabel:"Mute mic",offSplitButtonAriaLabel:"Unmute mic",microphoneActionTurnedOnAnnouncement:"Your microphone has been turned on",microphoneActionTurnedOffAnnouncement:"Your microphone has been turned off",offSplitButtonMicrophonePrimaryAction:"Unmute mic",onSplitButtonMicrophonePrimaryAction:"Mute mic",microphonePrimaryActionSplitButtonTitle:"Use microphone",microphoneAriaDescription:"Audio options",deepNoiseSuppressionTitle:"Noise suppression",deepNoiseSuppressionOnAnnouncement:"Deep noise suppression has been turned on",deepNoiseSuppressionOffAnnouncement:"Deep noise suppression has been turned off"};var devicesButton$m={label:"Devices",tooltipContent:"Manage devices",cameraMenuTitle:"Camera",cameraMenuTooltip:"Choose camera",audioDeviceMenuTitle:"Audio Device",audioDeviceMenuTooltip:"Choose audio device",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choose microphone",speakerMenuTitle:"Speaker",speakerMenuTooltip:"Choose speaker"};var participantsButton$m={label:"People",ariaLabel:"People",tooltipContent:"Show participants",menuHeader:"In this call",participantsListButtonLabel:"{numParticipants} people",muteAllButtonLabel:"Mute all",copyInviteLinkButtonLabel:"Copy invite link",copyInviteLinkActionedAriaLabel:"Invite link copied"};var screenShareButton$m={onLabel:"Stop presenting",offLabel:"Present",tooltipDisabledContent:"Presenting is disabled",tooltipOnContent:"Presenting your screen",tooltipOffContent:"Present your screen"};var raiseHandButton$m={offLabel:"Raise",onLabel:"Lower",tooltipDisabledContent:"Raise Hand action is disabled",tooltipOnContent:"Lower Hand",tooltipOffContent:"Raise Hand"};var reactionButton$m={label:"React",ariaLabel:"React",tooltipDisabledContent:"Reaction action is disabled",tooltipContent:"Send a reaction",likeReactionTooltipContent:"Like",heartReactionTooltipContent:"Love",laughReactionTooltipContent:"Laugh",applauseReactionTooltipContent:"Applause",surprisedReactionTooltipContent:"Surprised"};var realTimeText$m={isTypingText:"RTT typing"};var captionsSettingsModal$m={captionsSettingsModalTitle:"Captions Settings",captionsSettingsSpokenLanguageDropdownLabel:"Spoken language",captionsSettingsCaptionLanguageDropdownLabel:"Captions language",captionsSettingsSpokenLanguageDropdownInfoText:"Language that everyone on this call is speaking.",captionsSettingsCaptionLanguageDropdownInfoText:"Captions will appear in this langugage.",captionsSettingsConfirmButtonLabel:"Confirm",captionsSettingsCancelButtonLabel:"Cancel",captionsSettingsModalAriaLabel:"Captions Setting Modal",captionsSettingsCloseModalButtonAriaLabel:"Close Captions Setting"};var spokenLanguages$m={"ar-ae":"Arabic - U.A.E.","ar-sa":"Arabic - Saudi Arabia","da-dk":"Danish","de-de":"German - Germany","en-au":"English - Australia","en-ca":"English - Canada","en-gb":"English - United Kingdom","en-in":"English - India","en-nz":"English - New Zealand","en-us":"English - United States","es-es":"Spanish - Spain (Modern Sort)","es-mx":"Spanish - Mexico","fi-fi":"Finnish","fr-ca":"French - Canada","fr-fr":"French - France","hi-in":"Hindi","it-it":"Italian - Italy","ja-jp":"Japanese","ko-kr":"Korean","nb-no":"Norwegian (Bokmål)","nl-be":"Dutch - Belgium","nl-nl":"Dutch - Netherlands","pl-pl":"Polish","pt-br":"Portuguese - Brazil","ru-ru":"Russian","sv-se":"Swedish","zh-cn":"Chinese - People's Republic of China","zh-hk":"Chinese - Hong Kong SAR","cs-cz":"Czech","pt-pt":"Portuguese - Portugal","tr-tr":"Turkish","vi-vn":"Vietnamese","th-th":"Thai","he-il":"Hebrew","cy-gb":"Welsh","uk-ua":"Ukrainian","el-gr":"Greek","hu-hu":"Hungarian","ro-ro":"Romanian","sk-sk":"Slovak","zh-tw":"Chinese - Taiwan"};var captionLanguages$m={ar:"Arabic",da:"Danish",de:"German",en:"English",es:"Spanish",fi:"Finnish","fr-ca":"French - Canada",fr:"French - France",hi:"Hindi",it:"Italian",ja:"Japanese",ko:"Korean",nb:"Norwegian (Bokmål)",nl:"Dutch",pl:"Polish",pt:"Portuguese - Brazil",ru:"Russian",sv:"Swedish","zh-Hans":"Chinese (Simplified)","zh-Hant":"Chinese (Traditional)",cs:"Czech","pt-pt":"Portuguese - Portugal",tr:"Turkish",vi:"Vietnamese",th:"Thai",he:"Hebrew",cy:"Welsh",uk:"Ukrainian",el:"Greek",hu:"Hungarian",ro:"Romanian",sk:"Slovak"};var startCaptionsButton$m={onLabel:"Turn on captions",offLabel:"Turn off captions",tooltipOnContent:"Turn off captions",tooltipOffContent:"Turn on captions"};var startRealTimeTextButton$m={label:"Turn on RTT for this call",tooltipOffContent:"Turn on RTT for this call"};var messageThread$m={yesterday:"Yesterday",sunday:"Sunday",monday:"Monday",tuesday:"Tuesday",wednesday:"Wednesday",thursday:"Thursday",friday:"Friday",saturday:"Saturday",participantJoined:"joined the chat.",participantLeft:"left the chat.",editMessage:"Edit",removeMessage:"Delete",resendMessage:"Try sending again",failToSendTag:"Failed to send",editedTag:"Edited",liveAuthorIntro:"Message from {author}",editedMessageLiveAuthorIntro:"Edited message from {author}",editedMessageLocalUserLiveAuthorIntro:"Edited message",messageContentAriaText:"{status} {author} said {message}",messageContentMineAriaText:"{status} You said {message}",editBoxTextLimit:"Your message is over the limit of {limitNumber} characters",editBoxPlaceholderText:"Edit your message",newMessagesIndicator:"New messages",noDisplayNameSub:"No name",editBoxCancelButton:"Cancel",editBoxSubmitButton:"Done",messageReadCount:"Read by {messageReadByCount} of {remoteParticipantsCount}",actionMenuMoreOptions:"More Options",downloadAttachment:"Download file",openAttachment:"Open file",blockedWarningText:"This message was deleted due to organisational policy.",blockedWarningLinkText:"Details",attachmentCardGroupMessage:"The message has {attachmentCount} attachment",messageDeletedAnnouncementAriaLabel:"The message is deleted",richTextBoldTooltip:"Bold",richTextItalicTooltip:"Italic",richTextUnderlineTooltip:"Underline",richTextBulletListTooltip:"Bulleted list",richTextNumberListTooltip:"Numbered list",richTextIncreaseIndentTooltip:"Increase indent",richTextDecreaseIndentTooltip:"Decrease indent",richTextInsertTableTooltip:"Insert table",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insert",richTextInsertTableMenuTitle:"Insert {column} x {row} table",richTextInsertRowAboveMenu:"Insert above",richTextInsertRowBelowMenu:"Insert below",richTextInsertColumnLeftMenu:"Insert left",richTextInsertColumnRightMenu:"Insert right",richTextDeleteRowOrColumnMenu:"Delete",richTextDeleteColumnMenu:"Delete column",richTextDeleteRowMenu:"Delete row",richTextDeleteTableMenu:"Delete table",richTextToolbarMoreButtonAriaLabel:"More options",attachmentUploadsPendingError:"File is uploading, please wait.",imageUploadsPendingError:"Image is uploading, please wait.",richTextNewBulletedListItemAnnouncement:"Bullet, new line",richTextNewNumberedListItemAnnouncement:"{0}, new line",richTextBulletedListAppliedAnnouncement:"Bulleted list applied",richTextNumberedListAppliedAnnouncement:"Numbered list applied"};var errorBar$m={unableToReachChatService:"You are offline",accessDenied:"Unable to access chat services - please check the user credentials provided",userNotInChatThread:"You are no longer in this chat thread",sendMessageNotInChatThread:"Failed to send message because you are no longer in this chat thread",sendMessageGeneric:"Failed to send message",callingNetworkFailure:"Troubling connecting call - you seem to be offline",startVideoGeneric:"Failed to start video",stopVideoGeneric:"Failed to stop video",muteGeneric:"Failed to mute microphone",unmuteGeneric:"Failed to unmute microphone",speakingWhileMuted:"Your microphone is muted",startScreenShareGeneric:"There was an issue starting screen share.",stopScreenShareGeneric:"Failed to stop screen sharing",callNetworkQualityLow:"Network quality is low.",teamsMeetingCallNetworkQualityLow:"Network quality is low.",callNoSpeakerFound:"No speakers or headphones found. Connect an audio device to hear the call.",callNoMicrophoneFound:"No microphones found. Connect an audio input device.",callMicrophoneAccessDenied:"Unable to access microphone. Click the lock in the address bar to grant permission to this webpage.",callMicrophoneAccessDeniedSafari:"Unable to access microphone. Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",callMicrophoneMutedBySystem:"You are muted by your system.",callMicrophoneUnmutedBySystem:"Your microphone recovered and you were unmuted by your system.",callMacOsMicrophoneAccessDenied:"Unable to access microphone. Grant microphone permission in your macOS privacy settings.",callLocalVideoFreeze:"Network bandwidth is poor. Your video may appear paused for others on the call.",callCameraAccessDenied:"Unable to access camera. Click the lock in the address bar to grant permission to this webpage.",callCameraAccessDeniedSafari:"Unable to access camera. Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",callCameraAlreadyInUse:"Unable to access camera. It may already be in use by another application.",callVideoStoppedBySystem:"Your video has been stopped by your system.",callVideoRecoveredBySystem:"Your video has resumed.",callMacOsCameraAccessDenied:"MacOS is blocking access to your camera. Update your privacy settings to allow this browser to access your camera.",callMacOsScreenShareAccessDenied:"MacOS is blocking screen sharing. Update your privacy settings to allow this browser to record your screen.",dismissButtonAriaLabel:"Close",failedToJoinCallGeneric:"Failed to join call.",failedToJoinCallInvalidMeetingLink:"Unable to join Meeting. Invalid Link.",cameraFrozenForRemoteParticipants:"Users in the call are having issues seeing your video. Please check your devices and network.",unableToStartVideoEffect:"Unable to apply video effect.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Cannot start spotlight on participant(s) because the maximum number of participants are already spotlighted.",mutedByRemoteParticipant:"You were muted by another participant."};var notificationStack$m={callingNetworkFailure:{title:"Troubling connecting call",message:"You seem to be offline. Please check your network connection and try again.",dismissButtonAriaLabel:"Close"},startVideoGeneric:{title:"Failed to start video.",dismissButtonAriaLabel:"Close"},stopVideoGeneric:{title:"Failed to stop video.",dismissButtonAriaLabel:"Close"},muteGeneric:{title:"Failed to mute microphone.",dismissButtonAriaLabel:"Close"},unmuteGeneric:{title:"Failed to unmute microphone.",dismissButtonAriaLabel:"Close"},speakingWhileMuted:{title:"You're muted.",dismissButtonAriaLabel:"Close"},startScreenShareGeneric:{title:"There was an issue starting screen share.",dismissButtonAriaLabel:"Close"},stopScreenShareGeneric:{title:"Failed to stop screen sharing.",dismissButtonAriaLabel:"Close"},callNetworkQualityLow:{title:"Network quality is low.",dismissButtonAriaLabel:"Close"},teamsMeetingCallNetworkQualityLow:{title:"Network quality is low.",dismissButtonAriaLabel:"Close",message:"Join this call from your phone for better sound. You can continue viewing the meeting on this device.",primaryButtonLabel:"Join by Phone"},callNoSpeakerFound:{title:"No speakers or headphones found",message:"Connect an audio device to hear the call.",dismissButtonAriaLabel:"Close"},callNoMicrophoneFound:{title:"No microphones found",message:"Connect an audio input device.",dismissButtonAriaLabel:"Close"},callMicrophoneAccessDenied:{title:"Unable to access microphone",message:"Click the lock in the address bar to grant permission to this webpage.",dismissButtonAriaLabel:"Close"},callMicrophoneAccessDeniedSafari:{title:"Unable to access microphone",message:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",dismissButtonAriaLabel:"Close"},callMicrophoneMutedBySystem:{title:"You are muted by your system.",dismissButtonAriaLabel:"Close"},callMicrophoneUnmutedBySystem:{title:"Your microphone recovered and you were unmuted by your system.",dismissButtonAriaLabel:"Close"},callMacOsMicrophoneAccessDenied:{title:"Unable to access microphone",message:"Grant microphone permission in your macOS privacy settings.",dismissButtonAriaLabel:"Close"},callLocalVideoFreeze:{title:"Network bandwidth is poor",message:"Your video may appear paused for others on the call.",dismissButtonAriaLabel:"Close"},callCameraAccessDenied:{title:"Unable to access camera",message:"Click the lock in the address bar to grant permission to this webpage.",dismissButtonAriaLabel:"Close"},callCameraAccessDeniedSafari:{title:"Unable to access camera",message:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",dismissButtonAriaLabel:"Close"},callCameraAlreadyInUse:{title:"Unable to access camera",message:"It may already be in use by another application.",dismissButtonAriaLabel:"Close"},callVideoStoppedBySystem:{title:"Your video has been stopped by your system.",dismissButtonAriaLabel:"Close"},callVideoRecoveredBySystem:{title:"Your video has resumed.",dismissButtonAriaLabel:"Close"},callMacOsCameraAccessDenied:{title:"MacOS is blocking access to your camera",message:"Update your privacy settings to allow this browser to access your camera.",dismissButtonAriaLabel:"Close"},callMacOsScreenShareAccessDenied:{title:"MacOS is blocking screen sharing",message:"Update your privacy settings to allow this browser to record your screen.",dismissButtonAriaLabel:"Close"},failedToJoinCallGeneric:{title:"Failed to join call.",dismissButtonAriaLabel:"Close"},failedToJoinCallInvalidMeetingLink:{title:"Unable to join Meeting. Invalid Link.",dismissButtonAriaLabel:"Close"},cameraFrozenForRemoteParticipants:{title:"Users in the call are having issues seeing your video",message:"Please check your devices and network.",dismissButtonAriaLabel:"Close"},unableToStartVideoEffect:{title:"Unable to apply video effect.",dismissButtonAriaLabel:"Close"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Cannot start spotlight on participant(s)",message:"The maximum number of participants are already spotlighted.",dismissButtonAriaLabel:"Close"},mutedByRemoteParticipant:{title:"You were muted by another participant.",dismissButtonAriaLabel:"Close"},recordingStarted:{title:"Recording has started.",message:"By joining, you are giving consent for this meeting to be transcribed.",dismissButtonAriaLabel:"Close"},transcriptionStarted:{title:"Transcription has started.",message:"By joining, you are giving consent for this meeting to be transcribed.",dismissButtonAriaLabel:"Close"},recordingStopped:{title:"Recording is being saved.",message:"Recording has stopped.",dismissButtonAriaLabel:"Close"},transcriptionStopped:{title:"Transcription is being saved.",message:"Transcription has stopped.",dismissButtonAriaLabel:"Close"},recordingAndTranscriptionStarted:{title:"Recording and transcription have started.",message:"By joining, you are giving consent for this meeting to be transcribed.",dismissButtonAriaLabel:"Close"},recordingAndTranscriptionStopped:{title:"Recording and transcription are being saved.",message:"Recording and transcription have stopped.",dismissButtonAriaLabel:"Close"},recordingStoppedStillTranscribing:{title:"Recording has stopped.",message:"You are now only transcribing this meeting.",dismissButtonAriaLabel:"Close"},transcriptionStoppedStillRecording:{title:"Transcription has stopped.",message:"You are now only recording this meeting.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomOpened:{title:"We'll move you to your assigned room in 5 seconds.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomChanged:{title:"Your breakout room has changed",message:"We'll move you to your assigned room in 5 seconds.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomClosed:{title:"Your breakout room has closed",message:"We'll move you back to your meeting in 5 seconds.",dismissButtonAriaLabel:"Close"},assignedBreakoutRoomOpenedPromptJoin:{title:"Join breakout room?",message:"You've been assigned to a breakout room.",dismissButtonAriaLabel:"Close",primaryButtonLabel:"Join room",secondaryButtonLabel:"Later"},breakoutRoomJoined:{title:"You have joined a breakout room.",dismissButtonAriaLabel:"Close"},breakoutRoomClosingSoon:{title:"Room time limit about to expire.",message:"This room will close in 30 seconds",dismissButtonAriaLabel:"Close"},capabilityTurnVideoOnPresent:{title:"Your camera has been enabled",message:"Turn it on if you'd like to share video.",dismissButtonAriaLabel:"Close"},capabilityTurnVideoOnAbsent:{title:"Your camera has been disabled",message:"You can no longer share video.",dismissButtonAriaLabel:"Close"},capabilityUnmuteMicPresent:{title:"Your mic has been enabled",message:"Unmute if you'd like to speak.",dismissButtonAriaLabel:"Close"},capabilityUnmuteMicAbsent:{title:"Your mic has been disabled",message:"You can no longer unmute.",dismissButtonAriaLabel:"Close"},togetherModeStarted:{title:"Togethermode has started",dismissButtonAriaLabel:"Close"},togetherModeEnded:{title:"Togethermode has ended",dismissButtonAriaLabel:"Close"},transcriptionError:{title:"Something went wrong with transcription.",message:"Transcription has stopped. please try again later",dismissButtonAriaLabel:"Close"},transcriptionStartedByYou:{title:"Transcription has started.",message:"Started by you. Let everyone know they are being transcribed.",dismissButtonAriaLabel:"Close"}};var videoGallery$m={screenIsBeingSharedMessage:"You are sharing your screen",screenShareLoadingMessage:"Loading {participant}'s screen",localScreenShareLoadingMessage:"Loading your screen",localVideoLabel:"You",localVideoCameraSwitcherLabel:"Switch camera",localVideoMovementLabel:"Movable Local Video Tile",localVideoMovementAriaLabel:"Movable Local Video Tile",localVideoSelectedDescription:"{cameraName} selected",displayNamePlaceholder:"Unnamed participant",fitRemoteParticipantToFrame:"Fit to frame",fillRemoteParticipantFrame:"Fill frame",pinParticipantForMe:"Pin for me",pinParticipantForMeLimitReached:"Pin (limit reached)",unpinParticipantForMe:"Unpin",pinParticipantMenuItemAriaLabel:"Pin {participantName}",unpinParticipantMenuItemAriaLabel:"Unpin {participantName}",pinnedParticipantAnnouncementAriaLabel:"Pinned {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Unpinned {participantName}",startSpotlightVideoTileMenuLabel:"Spotlight for everyone",addSpotlightVideoTileMenuLabel:"Add spotlight",spotlightLimitReachedMenuTitle:"Spotlight limit reached",stopSpotlightVideoTileMenuLabel:"Stop spotlighting",stopSpotlightOnSelfVideoTileMenuLabel:"Exit spotlight",attendeeRole:"Attendee",muteParticipantMenuItemLabel:"Mute",waitingScreenText:"Waiting for others to join",forbidAudio:"Disable mic",permitAudio:"Allow mic",forbidAudioTileMenuLabel:"Disable mic",permitAudioTileMenuLabel:"Allow mic",forbidVideo:"Disable camera",permitVideo:"Allow camera",forbidVideoTileMenuLabel:"Disable camera",permitVideoTileMenuLabel:"Allow camera"};var dialpad$m={placeholderText:"Enter phone number",deleteButtonAriaLabel:"Delete"};var holdButton$m={onLabel:"Resume",offLabel:"Hold",tooltipOnContent:"Resume call",tooltipOffContent:"Hold call"};var videoTile$m={participantStateRinging:"Calling...",participantReconnecting:"Reconnecting...",participantStateHold:"On hold",moreOptionsButtonAriaLabel:"More Options {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"muted",moreOptionsParticipantMutedStateUnmutedAriaLabel:"unmuted",moreOptionsParticipantHandRaisedAriaLabel:"Hand raised",moreOptionsParticipantIsSpeakingAriaLabel:"Speaking",moreOptionsParticipantMicDisabledAriaLabel:"Microphone disabled",moreOptionsParticipantCameraDisabledAriaLabel:"Camera disabled"};var CameraAndMicrophoneSitePermissionsRequest$m={primaryText:"Allow {appName} to use your camera and microphone",secondaryText:"This is so participants can see and hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera and microphone",ariaLabel:"Allow camera and microphone access"};var CameraSitePermissionsRequest$m={primaryText:"Allow {appName} to use your camera",secondaryText:"This is so participants can see you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera",ariaLabel:"Allow camera access"};var MicrophoneSitePermissionsRequest$m={primaryText:"Allow {appName} to use your microphone",secondaryText:"This is so participants can hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without microphone",ariaLabel:"Allow microphone access"};var CameraAndMicrophoneSitePermissionsCheck$m={primaryText:"Checking for camera and microphone access",secondaryText:"Allow access if prompted. This is so participants can see and hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera and microphone",ariaLabel:"Checking for camera and microphone access. Allow access if prompted."};var CameraSitePermissionsCheck$m={primaryText:"Checking for camera access",secondaryText:"Allow access if prompted. This is so participants can see you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without camera",ariaLabel:"Checking for camera access. Allow access if prompted."};var MicrophoneSitePermissionsCheck$m={primaryText:"Checking for microphone access",secondaryText:"Allow access if prompted. This is so participants can hear you.",linkText:"Need help? Get troubleshooting help",primaryButtonText:"Continue without microphone",ariaLabel:"Checking for microphone access. Allow access if prompted."};var CameraAndMicrophoneSitePermissionsDenied$m={primaryText:"Unable to access camera and microphone",secondaryText:"Click the lock icon in the address bar to grant microphone permissions to this webpage. A page refresh may be required.",primaryButtonText:"Continue without camera and microphone",linkText:"Need help? Get troubleshooting help"};var CameraAndMicrophoneSitePermissionsDeniedSafari$m={primaryText:"Unable to access camera and microphone",secondaryText:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",primaryButtonText:"Continue without camera and microphone",linkText:"Need help? Get troubleshooting help"};var CameraSitePermissionsDenied$m={primaryText:"Unable to access camera",secondaryText:"Click the lock icon in the address bar to grant camera permissions to this webpage. A page refresh may be required.",primaryButtonText:"Continue without camera",linkText:"Need help? Get troubleshooting help"};var MicrophoneSitePermissionsDenied$m={primaryText:"Unable to access microphone",secondaryText:"Click the lock icon in the address bar to grant microphone permissions to this webpage. A page refresh may be required.",primaryButtonText:"Continue without microphone",linkText:"Need help? Get troubleshooting help"};var CameraSitePermissionsDeniedSafari$m={primaryText:"Unable to access camera",secondaryText:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",primaryButtonText:"Continue without camera",linkText:"Need help? Get troubleshooting help"};var MicrophoneSitePermissionsDeniedSafari$m={primaryText:"Unable to access microphone",secondaryText:"Refresh the page to allow permissions, or check this browser’s settings and verify permissions are enabled for this website.",primaryButtonText:"Continue without microphone",linkText:"Need help? Get troubleshooting help"};var UnsupportedBrowser$m={primaryText:"Browser not supported",secondaryText:"Please join this call using a compatible browser.",moreHelpLinkText:"See compatibility requirements"};var UnsupportedBrowserVersion$m={primaryText:"Browser update needed",secondaryText:"To ensure the best call possible, please update your browser and then try joining the call again.",moreHelpLinkText:"See compatibility requirements",continueAnywayButtonText:"Start call without updating"};var UnsupportedOperatingSystem$m={primaryText:"Operating system not supported",secondaryText:"Please join this call using a device with a compatible operating system.",moreHelpLinkText:"See compatibility requirements"};var BrowserPermissionDenied$m={primaryText:"Can't use your camera or microphone",secondaryText:"Your browser might not have access to your camera or microphone. To fix this, open System Preferences.",primaryButtonText:"Try again",linkText:"Need help? Get troubleshooting help"};var BrowserPermissionDeniedIOS$m={primaryText:"Allow microphone access to continue",secondaryText:"So other participants can hear you.",primaryButtonText:"Try again",imageAltText:"Microphone and camera device permission location for iOS",linkText:"Need help? Get troubleshooting help",step1Text:"Go to the Settings app",step2Text:"Scroll down to settings for this browser",step3Text:"Turn on Microphone (Camera optional)",step4Text:"Try joining the call again",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$m={leftNavButtonAriaLabel:"previous page {current} of {total}",rightNavButtonAriaLabel:"next page {current} of {total}"};var meetingConferencePhoneInfo$m={meetingConferencePhoneInfoModalTitle:"Join call from your phone",meetingConferencePhoneInfoModalDialIn:"Dial number",meetingConferencePhoneInfoModalMeetingId:"Enter meeting ID",meetingConferencePhoneInfoModalWait:"Wait to be admitted by organiser",meetingConferencePhoneInfoModalNoPhoneAvailable:"No phone available",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Toll-free)",meetingConferencePhoneInfoModalToll:"(Toll)"};var IncomingCallNotification$m={incomingCallNotificationPlaceholderId:"Unknown User",incomingCallNotificationPlaceholderAlert:"{callerName} is calling",incomingCallNoticicationAcceptWithAudioAriaLabel:"Accept with audio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Accept with video",incomingCallNoticicationRejectAriaLabel:"Reject",incomingCallNotificationAcceptButtonLabel:"Accept",incomingCallNotificationRejectButtonLabel:"Decline",incomingCallNotificationAccceptWithVideoButtonLabel:"Accept with Video",incomingCallNotificationDismissButtonAriaLabel:"Dismiss"};var en_GB$1 = {participantItem:participantItem$m,ParticipantList:ParticipantList$m,typingIndicator:typingIndicator$m,sendBox:sendBox$m,richTextSendBox:richTextSendBox$m,realTimeTextModal:realTimeTextModal$m,captionsBanner:captionsBanner$m,mentionPopover:mentionPopover$m,imageOverlay:imageOverlay$m,messageStatusIndicator:messageStatusIndicator$m,endCallButton:endCallButton$m,cameraButton:cameraButton$m,microphoneButton:microphoneButton$m,devicesButton:devicesButton$m,participantsButton:participantsButton$m,screenShareButton:screenShareButton$m,raiseHandButton:raiseHandButton$m,reactionButton:reactionButton$m,realTimeText:realTimeText$m,captionsSettingsModal:captionsSettingsModal$m,spokenLanguages:spokenLanguages$m,captionLanguages:captionLanguages$m,startCaptionsButton:startCaptionsButton$m,startRealTimeTextButton:startRealTimeTextButton$m,messageThread:messageThread$m,errorBar:errorBar$m,notificationStack:notificationStack$m,videoGallery:videoGallery$m,dialpad:dialpad$m,holdButton:holdButton$m,videoTile:videoTile$m,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$m,CameraSitePermissionsRequest:CameraSitePermissionsRequest$m,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$m,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$m,CameraSitePermissionsCheck:CameraSitePermissionsCheck$m,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$m,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$m,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$m,CameraSitePermissionsDenied:CameraSitePermissionsDenied$m,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$m,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$m,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$m,UnsupportedBrowser:UnsupportedBrowser$m,UnsupportedBrowserVersion:UnsupportedBrowserVersion$m,UnsupportedOperatingSystem:UnsupportedOperatingSystem$m,BrowserPermissionDenied:BrowserPermissionDenied$m,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$m,verticalGallery:verticalGallery$m,meetingConferencePhoneInfo:meetingConferencePhoneInfo$m,IncomingCallNotification:IncomingCallNotification$m};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for English (GB).
 *
 * @public
 */
const COMPONENT_LOCALE_EN_GB = {
    strings: createComponentStrings(en_GB$1)
};

var participantItem$l={isMeText:"(أنت)",menuTitle:"المزيد من الخيارات",removeButtonLabel:"إزالة",sharingIconLabel:"جارٍ المشاركة",mutedIconLabel:"تم كتم الصوت",handRaisedIconLabel:"يد مرفوعة - الموضع {position}",displayNamePlaceholder:"مشارك غير مسمى",participantStateRinging:"يتم الآن الاتصال...",participantStateHold:"قيد الانتظار",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}، {handRaisedState}، {mutedState}، {micDisabledState}، {cameraDisabledState}، {sharingState}، {connectionState}، خيارات إضافية",attendeeRole:"الحضور",micDisabledIconLabel:"تم تعطيل الميكروفون",cameraDisabledIconLabel:"تم تعطيل الكاميرا"};var ParticipantList$l={overflowParticipantCount:"+{overflowCount} من الأشخاص الإضافيين"};var typingIndicator$l={singleUser:"{user} يكتب",multipleUsers:"يكتب {users} الآن...",multipleUsersAbbreviateOne:"جارٍ الكتابة بواسطة {users} وشخص آخر",multipleUsersAbbreviateMany:"{users} وآخرون {numOthers} يكتبون ...",delimiter:"و "};var sendBox$l={placeholderText:"أدخل رسالة",textTooLong:"تجاوز طول الرسالة الحد الأقصى.",sendButtonAriaLabel:"إرسال رسالة",attachmentUploadsPendingError:"يتم الآن تحميل الملف، الرجاء الانتظار.",removeAttachment:"إزالة الملف",uploading:"جارٍ التحميل",uploadCompleted:"اكتمل التحميل",attachmentMoreMenu:"خيارات إضافية"};var richTextSendBox$l={placeholderText:"أدخل رسالة",textTooLong:"تجاوز طول الرسالة الحد الأقصى.",sendButtonAriaLabel:"إرسال رسالة",attachmentUploadsPendingError:"يتم الآن تحميل الملف، الرجاء الانتظار.",imageUploadsPendingError:"يتم الآن تحميل الصورة، الرجاء الانتظار.",removeAttachment:"إزالة الملف",uploading:"يتم الآن التحميل",uploadCompleted:"اكتمل التحميل",richTextBoldTooltip:"غامق",richTextItalicTooltip:"مائل",richTextUnderlineTooltip:"مسطّر",richTextBulletListTooltip:"قائمة ذات تعداد نقطي",richTextNumberListTooltip:"قائمة مرقّمة",richTextIncreaseIndentTooltip:"زيادة المسافة البادئة",richTextDecreaseIndentTooltip:"إنقاص المسافة البادئة",richTextInsertTableTooltip:"إدراج جدول",richTextFormatButtonTooltip:"تنسيق",richTextInsertRowOrColumnMenu:"إدراج",richTextInsertTableMenuTitle:"إدراج جدول بـ {column} x {row}",richTextInsertRowAboveMenu:"إدراج أعلاه",richTextInsertRowBelowMenu:"إدراج أدناه",richTextInsertColumnLeftMenu:"إدراج يساراً",richTextInsertColumnRightMenu:"إدراج يميناً",richTextDeleteRowOrColumnMenu:"حذف",richTextDeleteColumnMenu:"حذف العمود",richTextDeleteRowMenu:"حذف الصف",richTextDeleteTableMenu:"حذف الجدول",richTextToolbarAriaLabel:"شريط أدوات تنسيق الرسائل",richTextToolbarMoreButtonAriaLabel:"خيارات إضافية",attachmentMoreMenu:"خيارات إضافية",richTextNewBulletedListItemAnnouncement:"رمز نقطي، سطر جديد",richTextNewNumberedListItemAnnouncement:"{0}، سطر جديد",richTextBulletedListAppliedAnnouncement:"تم تطبيق قائمة نقطية",richTextNumberedListAppliedAnnouncement:"تطبيق قائمة ذات تعداد رقمي"};var realTimeTextModal$l={realTimeTextModalAriaLabel:"تشغيل مربع حوار عرض الكتابة في الوقت الحقيقي (RTT) المشروط",realTimeTextModalTitle:"هل تريد تشغيل عرض الكتابة في الوقت الحقيقي؟",realTimeTextModalText:"سيتم تشغيل النص في الوقت الحقيقي (RTT) لكل شخص في الاجتماع.",realTimeTextConfirmButtonLabel:"تشغيل",realTimeTextCancelButtonLabel:"إلغاء الأمر",realTimeTextCloseModalButtonAriaLabel:"إغلاق مربع حوار عرض الكتابة في الوقت الحقيقي (RTT) المشروط"};var captionsBanner$l={captionsBannerSpinnerText:"يتم الآن بدء التسميات التوضيحية...",realTimeTextInputBoxDefaultText:"اكتب رسالة في الوقت الحقيقي",realTimeTextInputErrorMessage:"لا يمكن أن تتجاوز الرسالة 2000 حرف",realTimeTextBannerTitle:"عرض الكتابة في الوقت الحقيقي (RTT)",realTimeTextBannerContent:"تم تمكين RTT (نص الوقت الحقيقي) لجميع المشاركين طوال مدة الاجتماع.",realTimeTextBannerLinkLabel:"تعرّف على المزيد",captionsOnlyContainerTitle:"نص الكلام",realTimeTextOnlyContainerTitle:"نص حقيقي",captionsAndRealTimeTextContainerTitle:"التسميات التوضيحية والنص في الوقت الحقيقي",expandButtonAriaLabel:"توسيع",minimizeButtonAriaLabel:"طي"};var mentionPopover$l={mentionPopoverHeader:"الاقتراحات"};var imageOverlay$l={downloadButtonLabel:"تنزيل",dismissButtonAriaLabel:"إغلاق",overlayVisibleMessage:"تراكب تفاصيل الصور"};var messageStatusIndicator$l={deliveredAriaLabel:"تم إرسال الرسالة",deliveredTooltipText:"تم الإرسال",seenAriaLabel:"تمت مشاهدة الرسالة من قِبل الآخرين.",seenTooltipText:"تمت المشاهدة",readByTooltipText:"تمت القراءة بواسطة {messageThreadReadCount} من {remoteParticipantsCount}",sendingAriaLabel:"إرسال الرسالة",sendingTooltipText:"جاري الإرسال",failedToSendAriaLabel:"فشل إرسال الرسالة",failedToSendTooltipText:"فشل الإرسال"};var endCallButton$l={label:"مغادرة",tooltipContent:"مغادرة المكالمة",leaveOption:"مغادرة",endCallOption:"إنهاء المكالمة لدى الجميع"};var cameraButton$l={onLabel:"الكاميرا",offLabel:"الكاميرا",tooltipDisabledContent:"الكاميرا مُعطلة",tooltipOnContent:"إيقاف تشغيل الكاميرا",tooltipOffContent:"تشغيل الكاميرا",tooltipVideoLoadingContent:"جار تحميل الفيديو",cameraMenuTitle:"الكاميرا",cameraMenuTooltip:"اختيار الكاميرا",cameraButtonSplitRoleDescription:"زر الانقسام",onSplitButtonAriaLabel:"إيقاف تشغيل الكاميرا",offSplitButtonAriaLabel:"تشغيل الكاميرا",cameraActionTurnedOnAnnouncement:"تم تشغيل كاميراتك",cameraActionTurnedOffAnnouncement:"تم إيقاف تشغيل كاميراتك",onSplitButtonPrimaryActionCamera:"إيقاف تشغيل الكاميرا",offSplitButtonPrimaryActionCamera:"تشغيل الكاميرا",cameraPrimaryActionSplitButtonTitle:"استخدام الكاميرا",videoEffectsMenuItemTitle:"التأثيرات",cameraButtonAriaDescription:"خيارات الكاميرا"};var microphoneButton$l={onLabel:"الميكروفون",offLabel:"الميكروفون",tooltipDisabledContent:"تم تعطيل الميكروفون",tooltipOnContent:"كتم صوت الميكروفون",tooltipOffContent:"إلغاء كتم صوت الميكروفون",microphoneMenuTitle:"الميكروفون",microphoneMenuTooltip:"اختيار الميكروفون",speakerMenuTitle:"مكبر الصوت",speakerMenuTooltip:"اختيار مكبر صوت",microphoneButtonSplitRoleDescription:"زر الانقسام",onSplitButtonAriaLabel:"كتم الميكروفون",offSplitButtonAriaLabel:"إلغاء كتم الميكروفون",microphoneActionTurnedOnAnnouncement:"تم تشغيل الميكروفون",microphoneActionTurnedOffAnnouncement:"تم إيقاف الميكروفون",offSplitButtonMicrophonePrimaryAction:"إلغاء كتم الميكروفون",onSplitButtonMicrophonePrimaryAction:"كتم الميكروفون",microphonePrimaryActionSplitButtonTitle:"استخدام الميكروفون",microphoneAriaDescription:"خيارات الصوت",deepNoiseSuppressionTitle:"منع الضوضاء",deepNoiseSuppressionOnAnnouncement:"تم تشغيل منع الضوضاء العميقة",deepNoiseSuppressionOffAnnouncement:"تم إيقاف تشغيل منع الضوضاء العميقة"};var devicesButton$l={label:"الأجهزة",tooltipContent:"إدارة الأجهزة",cameraMenuTitle:"الكاميرا",cameraMenuTooltip:"اختيار الكاميرا",audioDeviceMenuTitle:"جهاز صوت",audioDeviceMenuTooltip:"إلغاء كتم صوت الجهاز",microphoneMenuTitle:"الميكروفون",microphoneMenuTooltip:"اختيار الميكروفون",speakerMenuTitle:"مكبر الصوت",speakerMenuTooltip:"اختيار مكبر صوت"};var participantsButton$l={label:"الأشخاص",ariaLabel:"الأشخاص",tooltipContent:"إظهار المشاركين",menuHeader:"في هذه المكالمة",participantsListButtonLabel:"{numParticipants} من الأشخاص",muteAllButtonLabel:"كتم صوت الجميع",copyInviteLinkButtonLabel:"نسخ ارتباط دعوة",copyInviteLinkActionedAriaLabel:"تم نسخ ارتباط الدعوة"};var screenShareButton$l={onLabel:"إيقاف التقديم",offLabel:"تقديم",tooltipDisabledContent:"تم تعطيل التقديم",tooltipOnContent:"تقديم العرض على شاشتك.",tooltipOffContent:"عرض الشاشة"};var raiseHandButton$l={offLabel:"رفع",onLabel:"خفض",tooltipDisabledContent:"تم تعطيل إجراء رفع اليد",tooltipOnContent:"خفض يد",tooltipOffContent:"رفع يد"};var reactionButton$l={label:"تفاعل",ariaLabel:"تفاعل",tooltipDisabledContent:"تم تعطيل إجراء التفاعل",tooltipContent:"إرسال تفاعل",likeReactionTooltipContent:"إعجاب",heartReactionTooltipContent:"حب",laughReactionTooltipContent:"ضحك",applauseReactionTooltipContent:"تصفيق",surprisedReactionTooltipContent:"مندهش"};var realTimeText$l={isTypingText:"عرض الكتابة في الوقت الحقيقي (RTT) قيد الكتابة"};var captionsSettingsModal$l={captionsSettingsModalTitle:"إعدادات التسميات التوضيحية",captionsSettingsSpokenLanguageDropdownLabel:"اللغة المنطوقة",captionsSettingsCaptionLanguageDropdownLabel:"لغة التسميات التوضيحية",captionsSettingsSpokenLanguageDropdownInfoText:"اللغة التي يتحدث بها كل شخص في هذه المكالمة.",captionsSettingsCaptionLanguageDropdownInfoText:"ستظهر التسميات التوضيحية بهذه اللغة.",captionsSettingsConfirmButtonLabel:"تأكيد",captionsSettingsCancelButtonLabel:"إلغاء الأمر",captionsSettingsModalAriaLabel:"مربع حوار مشروط لإعدادات نص الكلام",captionsSettingsCloseModalButtonAriaLabel:"إغلاق إعداد نص الكلام"};var spokenLanguages$l={"ar-ae":"العربية - الإمارات العربية المتحدة","ar-sa":"العربية - السعودية","da-dk":"الدانمركية","de-de":"الألمانية - ألمانيا","en-au":"الإنجليزية - أستراليا","en-ca":"الإنجليزية - كندا","en-gb":"الإنجليزية - المملكة المتحدة","en-in":"الإنجليزية - الهند","en-nz":"الإنجليزية - نيوزيلندا","en-us":"الإنجليزية - الولايات المتحدة","es-es":"الإسبانية - إسبانيا (فرز حديث)","es-mx":"الإسبانية - المكسيك","fi-fi":"الفنلندية","fr-ca":"الفرنسية - كندا","fr-fr":"الفرنسية - فرنسا","hi-in":"الهندية","it-it":"الإيطالية - إيطاليا","ja-jp":"اليابانية","ko-kr":"الكورية","nb-no":"النرويجية (بوكمال)","nl-be":"الهولندية - بلجيكا","nl-nl":"الهولندية - هولندا","pl-pl":"البولندية","pt-br":"البرتغالية - البرازيل","ru-ru":"الروسية","sv-se":"السويدية","zh-cn":"الصينية - جمهورية الصين الشعبية","zh-hk":"الصينية - منطقة هونغ كونغ الإدارية الخاصة","cs-cz":"التشيكية","pt-pt":"البرتغالية - البرتغال","tr-tr":"التركية","vi-vn":"الفيتنامية","th-th":"التايلاندية","he-il":"العبرية","cy-gb":"الويلزية","uk-ua":"الأوكرانية","el-gr":"اليونانية","hu-hu":"الهنغارية","ro-ro":"الرومانية","sk-sk":"السلوفاكية","zh-tw":"الصينية - تايوان"};var captionLanguages$l={ar:"العربية",da:"الدانمركية",de:"الألمانية",en:"الإنجليزية",es:"الإسبانية",fi:"الفنلندية","fr-ca":"الفرنسية - كندا",fr:"الفرنسية - فرنسا",hi:"الهندية",it:"الإيطالية",ja:"اليابانية",ko:"الكورية",nb:"النرويجية (بوكمال)",nl:"الهولندية",pl:"البولندية",pt:"البرتغالية - البرازيل",ru:"الروسية",sv:"السويدية","zh-Hans":"الصينية (المبسطة)","zh-Hant":"الصينية (التقليدية)",cs:"التشيكية","pt-pt":"البرتغالية - البرتغال",tr:"التركية",vi:"الفيتنامية",th:"التايلاندية",he:"العبرية",cy:"الويلزية",uk:"الأوكرانية",el:"اليونانية",hu:"الهنغارية",ro:"الرومانية",sk:"السلوفاكية"};var startCaptionsButton$l={onLabel:"تمكين نص الكلام",offLabel:"تعطيل نص الكلام",tooltipOnContent:"تعطيل نص الكلام",tooltipOffContent:"تمكين نص الكلام"};var startRealTimeTextButton$l={label:"تشغيل RTT لهذه المكالمة",tooltipOffContent:"تشغيل RTT لهذه المكالمة"};var messageThread$l={yesterday:"أمس",sunday:"الأحد",monday:"الاثنين",tuesday:"الثلاثاء",wednesday:"الأربعاء",thursday:"الخميس",friday:"الجمعة",saturday:"السبت",participantJoined:"انضم إلى المحادثة.",participantLeft:"قام بمغادرة الدردشة.",editMessage:"تحرير",removeMessage:"حذف",resendMessage:"محاولة الإرسال مرة أخرى",failToSendTag:"فشل الإرسال",editedTag:"تم التحرير",liveAuthorIntro:"رسالة من {author}",editedMessageLiveAuthorIntro:"تم تحرير الرسالة من {author}",editedMessageLocalUserLiveAuthorIntro:"رسالة محررة",messageContentAriaText:"{status} {author} قال {message}",messageContentMineAriaText:"{status} أنت قولت {message}",editBoxTextLimit:"تجاوزت رسالتك الحد الأقصى لعدد الأحرف {limitNumber}",editBoxPlaceholderText:"تحرير رسالتك",newMessagesIndicator:"رسائل جديدة",noDisplayNameSub:"لا يوجد اسم",editBoxCancelButton:"إلغاء الأمر",editBoxSubmitButton:"تم",messageReadCount:"تمت القراءة بواسطة {messageReadByCount} من {remoteParticipantsCount}",actionMenuMoreOptions:"المزيد من الخيارات",downloadAttachment:"تنزيل ملف",openAttachment:"فتح ملف",blockedWarningText:"تم حذف هذه الرسالة بسبب نهج تنظيمي.",blockedWarningLinkText:"تفاصيل",attachmentCardGroupMessage:"تحتوي الرسالة على مرفق {attachmentCount}",messageDeletedAnnouncementAriaLabel:"تم حذف الرسالة",richTextBoldTooltip:"غامق",richTextItalicTooltip:"مائل",richTextUnderlineTooltip:"تسطير",richTextBulletListTooltip:"قائمة ذات تعداد نقطي",richTextNumberListTooltip:"قائمة مرقّمة",richTextIncreaseIndentTooltip:"زيادة المسافة البادئة",richTextDecreaseIndentTooltip:"إنقاص المسافة البادئة",richTextInsertTableTooltip:"إدراج جدول",richTextFormatButtonTooltip:"تنسيق",richTextInsertRowOrColumnMenu:"إدراج",richTextInsertTableMenuTitle:"إدراج جدول بـ {column} x {row}",richTextInsertRowAboveMenu:"إدراج أعلاه",richTextInsertRowBelowMenu:"إدراج أدناه",richTextInsertColumnLeftMenu:"إدراج يساراً",richTextInsertColumnRightMenu:"إدراج يميناً",richTextDeleteRowOrColumnMenu:"حذف",richTextDeleteColumnMenu:"حذف العمود",richTextDeleteRowMenu:"حذف الصف",richTextDeleteTableMenu:"حذف الجدول",richTextToolbarMoreButtonAriaLabel:"خيارات إضافية",attachmentUploadsPendingError:"يتم الآن تحميل الملف، الرجاء الانتظار.",imageUploadsPendingError:"يتم الآن تحميل الصورة، الرجاء الانتظار.",richTextNewBulletedListItemAnnouncement:"رمز نقطي، سطر جديد",richTextNewNumberedListItemAnnouncement:"{0}، سطر جديد",richTextBulletedListAppliedAnnouncement:"تم تطبيق قائمة نقطية",richTextNumberedListAppliedAnnouncement:"تطبيق قائمة ذات تعداد رقمي"};var errorBar$l={unableToReachChatService:"أنت غير متصل",accessDenied:"يتعذر الوصول إلى خدمات الدردشة - الرجاء التحقق من بيانات اعتماد المستخدم المقدمة",userNotInChatThread:"لم تعد مضمناً في هذا ترابط الدردشة هذا",sendMessageNotInChatThread:"فشل إرسال الرسالة لأنك لم تعد في مؤشر ترابط الدردشة هذا",sendMessageGeneric:"فشل إرسال الرسالة",callingNetworkFailure:"مكالمة مزعجة - يبدو أنك غير متصل",startVideoGeneric:"فشل بدء الفيديو",stopVideoGeneric:"فشل إيقاف الفيديو",muteGeneric:"فشل كتم صوت الميكروفون",unmuteGeneric:"فشل إلغاء كتم صوت الميكروفون",speakingWhileMuted:"تم كتم صوت الميكروفون",startScreenShareGeneric:"حدثت مشكلة أثناء بدء مشاركة الشاشة.",stopScreenShareGeneric:"فشل إيقاف مشاركة الشاشة",callNetworkQualityLow:"جودة الشبكة مقبولة.",teamsMeetingCallNetworkQualityLow:"جودة الشبكة منخفضة.",callNoSpeakerFound:"لم يتم العثور على سماعات أو سماعات رأس. قم بتوصيل جهاز صوت لسماع المكالمة.",callNoMicrophoneFound:"لم يتم العثور على ميكروفونات. قم بتوصيل جهاز إدخال صوت.",callMicrophoneAccessDenied:"يتعذر الوصول إلى الميكروفون. انقر فوق التأمين في شريط العناوين لمنح الإذن لصفحة ويب هذه.",callMicrophoneAccessDeniedSafari:"يتعذر الوصول إلى الميكروفون. قم بتحديث الصفحة للسماح بالأذونات، أو تحقق من إعدادات هذا المستعرض وتحقق من تمكين الأذونات لموقع الويب هذا.",callMicrophoneMutedBySystem:"تم كتم صوتك بواسطة النظام.",callMicrophoneUnmutedBySystem:"تم استرداد الميكروفون وتم إلغاء كتم صوتك من قبل النظام.",callMacOsMicrophoneAccessDenied:"يتعذر الوصول إلى الميكروفون. امنح الميكروفون إذنا في إعدادات خصوصية macOS.",callLocalVideoFreeze:"النطاق الترددي للشبكة ضعيف. قد يظهر الفيديو متوقفا مؤقتا للآخرين في المكالمة.",callCameraAccessDenied:"يتعذر الوصول إلى الكاميرا. انقر فوق التأمين في شريط العناوين لمنح الإذن لصفحة ويب هذه.",callCameraAccessDeniedSafari:"يتعذر الوصول إلى الكاميرا. قم بتحديث الصفحة للسماح بالأذونات، أو تحقق من إعدادات هذا المستعرض وتحقق من تمكين الأذونات لموقع الويب هذا.",callCameraAlreadyInUse:"يتعذر الوصول إلى الكاميرا. قد يكون قيد الاستخدام بالفعل بواسطة تطبيق آخر.",callVideoStoppedBySystem:"تم إيقاف الفيديو الخاص بك من قبل النظام.",callVideoRecoveredBySystem:"تم استئناف الفيديو الخاص بك.",callMacOsCameraAccessDenied:"يقوم MacOS بحظر الوصول إلى الكاميرا الخاصة بك. قم بتحديث إعدادات الخصوصية للسماح لهذا المستعرض بالوصول إلى الكاميرا.",callMacOsScreenShareAccessDenied:"يقوم MacOS بحظر مشاركة الشاشة. قم بتحديث إعدادات الخصوصية للسماح لهذا المستعرض بتسجيل الشاشة.",dismissButtonAriaLabel:"إغلاق",failedToJoinCallGeneric:"فشل في الانضمام إلى المكالمة.",failedToJoinCallInvalidMeetingLink:"تعذر الانضمام إلى الاجتماع. ارتباط غير صالح.",cameraFrozenForRemoteParticipants:"يواجه المستخدمون في المكالمة مشكلات في رؤية الفيديو الخاص بك. الرجاء التحقق من أجهزتك وشبكتك.",unableToStartVideoEffect:"يتعذر تطبيق تأثير الفيديو.",startSpotlightWhileMaxParticipantsAreSpotlighted:"لا يمكن بدء تسليط الضوء على المشاركين (المشاركين) لأنه تم بالفعل تسليط الضوء على الحد الأقصى لعدد المشاركين.",mutedByRemoteParticipant:"تم كتم صوتك من قبل مشارك آخر."};var notificationStack$l={callingNetworkFailure:{title:"مكالمة مزعجة",message:"يبدو أنك غير متصل. الرجاء التحقق من اتصال الشبكة والمحاولة مرة أخرى.",dismissButtonAriaLabel:"إغلاق"},startVideoGeneric:{title:"فشل بدء الفيديو.",dismissButtonAriaLabel:"إغلاق"},stopVideoGeneric:{title:"فشل إيقاف الفيديو.",dismissButtonAriaLabel:"إغلاق"},muteGeneric:{title:"فشل كتم صوت الميكروفون.",dismissButtonAriaLabel:"إغلاق"},unmuteGeneric:{title:"فشل إلغاء كتم صوت الميكروفون.",dismissButtonAriaLabel:"إغلاق"},speakingWhileMuted:{title:"تم كتم صوتك.",dismissButtonAriaLabel:"إغلاق"},startScreenShareGeneric:{title:"حدثت مشكلة أثناء بدء مشاركة الشاشة.",dismissButtonAriaLabel:"إغلاق"},stopScreenShareGeneric:{title:"فشل إيقاف مشاركة الشاشة",dismissButtonAriaLabel:"إغلاق"},callNetworkQualityLow:{title:"جودة الشبكة مقبولة.",dismissButtonAriaLabel:"إغلاق"},teamsMeetingCallNetworkQualityLow:{title:"جودة الشبكة منخفضة.",dismissButtonAriaLabel:"إغلاق",message:"انضم إلى هذه المكالمة من هاتفك للحصول على صوت أفضل. يمكنك متابعة عرض الاجتماع على هذا الجهاز.",primaryButtonLabel:"انضمام عبر الهاتف"},callNoSpeakerFound:{title:"لم يتم العثور على سماعات أو سماعات رأس",message:"قم بتوصيل جهاز صوت لسماع المكالمة.",dismissButtonAriaLabel:"إغلاق"},callNoMicrophoneFound:{title:"لم يتم العثور على ميكروفونات",message:"قم بتوصيل جهاز إدخال صوت.",dismissButtonAriaLabel:"إغلاق"},callMicrophoneAccessDenied:{title:"يتعذر الوصول إلى الميكروفون",message:"انقر فوق التأمين في شريط العناوين لمنح الإذن لصفحة ويب هذه.",dismissButtonAriaLabel:"إغلاق"},callMicrophoneAccessDeniedSafari:{title:"يتعذر الوصول إلى الميكروفون",message:"قم بتحديث الصفحة للسماح بالأذونات، أو تحقق من إعدادات هذا المستعرض وتحقق من تمكين الأذونات لموقع الويب هذا.",dismissButtonAriaLabel:"إغلاق"},callMicrophoneMutedBySystem:{title:"تم كتم صوتك بواسطة النظام.",dismissButtonAriaLabel:"إغلاق"},callMicrophoneUnmutedBySystem:{title:"تم استرداد الميكروفون وتم إلغاء كتم صوتك من قبل النظام.",dismissButtonAriaLabel:"إغلاق"},callMacOsMicrophoneAccessDenied:{title:"يتعذر الوصول إلى الميكروفون",message:"امنح الميكروفون إذنا في إعدادات خصوصية macOS.",dismissButtonAriaLabel:"إغلاق"},callLocalVideoFreeze:{title:"النطاق الترددي للشبكة ضعيف.",message:"قد يظهر الفيديو متوقفا مؤقتا للآخرين في المكالمة.",dismissButtonAriaLabel:"إغلاق"},callCameraAccessDenied:{title:"تعذر الوصول إلى الكاميرا",message:"انقر فوق التأمين في شريط العناوين لمنح الإذن لصفحة ويب هذه.",dismissButtonAriaLabel:"إغلاق"},callCameraAccessDeniedSafari:{title:"تعذر الوصول إلى الكاميرا",message:"قم بتحديث الصفحة للسماح بالأذونات، أو تحقق من إعدادات هذا المستعرض وتحقق من تمكين الأذونات لموقع الويب هذا.",dismissButtonAriaLabel:"إغلاق"},callCameraAlreadyInUse:{title:"تعذر الوصول إلى الكاميرا",message:"قد يكون قيد الاستخدام بالفعل بواسطة تطبيق آخر.",dismissButtonAriaLabel:"إغلاق"},callVideoStoppedBySystem:{title:"تم إيقاف الفيديو الخاص بك من قبل النظام.",dismissButtonAriaLabel:"إغلاق"},callVideoRecoveredBySystem:{title:"تم استئناف الفيديو الخاص بك.",dismissButtonAriaLabel:"إغلاق"},callMacOsCameraAccessDenied:{title:"يقوم MacOS بحظر الوصول إلى الكاميرا",message:"قم بتحديث إعدادات الخصوصية للسماح لهذا المستعرض بالوصول إلى الكاميرا.",dismissButtonAriaLabel:"إغلاق"},callMacOsScreenShareAccessDenied:{title:"يقوم MacOS بحظر مشاركة الشاشة",message:"قم بتحديث إعدادات الخصوصية للسماح لهذا المستعرض بتسجيل الشاشة.",dismissButtonAriaLabel:"إغلاق"},failedToJoinCallGeneric:{title:"فشل الانضمام إلى المكالمة.",dismissButtonAriaLabel:"إغلاق"},failedToJoinCallInvalidMeetingLink:{title:"تعذر الانضمام إلى الاجتماع. ارتباط غير صالح.",dismissButtonAriaLabel:"إغلاق"},cameraFrozenForRemoteParticipants:{title:"يواجه المستخدمون في المكالمة مشكلات في رؤية الفيديو الخاص بك.",message:"الرجاء التحقق من أجهزتك وشبكتك.",dismissButtonAriaLabel:"إغلاق"},unableToStartVideoEffect:{title:"يتعذر تطبيق تأثير الفيديو.",dismissButtonAriaLabel:"إغلاق"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"يتعذر بدء تسليط ضوء على المشاركين.",message:"يتم تسليط الضوء على الحد الأقصى لعدد المشاركين بالفعل.",dismissButtonAriaLabel:"إغلاق"},mutedByRemoteParticipant:{title:"تم كتم صوتك من قبل مشارك آخر.",dismissButtonAriaLabel:"إغلاق"},recordingStarted:{title:"بدأ التسجيل.",message:"من خلال الانضمام، فأنت توافق على كتابة هذا الاجتماع.",dismissButtonAriaLabel:"إغلاق"},transcriptionStarted:{title:"بدأت كتابة الحديث.",message:"من خلال الانضمام، فأنت توافق على كتابة هذا الاجتماع.",dismissButtonAriaLabel:"إغلاق"},recordingStopped:{title:"يتم الآن حفظ التسجيل.",message:"لقد توقف التسجيل.",dismissButtonAriaLabel:"إغلاق"},transcriptionStopped:{title:"يتم الآن حفظ كتابة الحديث.",message:"توقفت كتابة الحديث.",dismissButtonAriaLabel:"إغلاق"},recordingAndTranscriptionStarted:{title:"بدء التسجيل والنسخ. ",message:"من خلال الانضمام، فأنت توافق على كتابة هذا الاجتماع.",dismissButtonAriaLabel:"إغلاق"},recordingAndTranscriptionStopped:{title:"يتم حفظ التسجيل وكتابة الحديث.",message:"التسجيل وكتابة الحديث متوقفان.",dismissButtonAriaLabel:"إغلاق"},recordingStoppedStillTranscribing:{title:"لقد توقف التسجيل.",message:"أنت الآن تقوم فقط بكتابة الحديث لهذا الاجتماع.",dismissButtonAriaLabel:"إغلاق"},transcriptionStoppedStillRecording:{title:"توقفت كتابة الحديث.",message:"أنت الآن تسجل هذا الاجتماع فقط.",dismissButtonAriaLabel:"إغلاق"},assignedBreakoutRoomOpened:{title:"سننقلك إلى الغرفة المعينة خلال 5 ثوان.",dismissButtonAriaLabel:"إغلاق"},assignedBreakoutRoomChanged:{title:"تم تغيير الغرفة الفرعية الخاصة بك",message:"سننقلك إلى الغرفة المعينة خلال 5 ثوان.",dismissButtonAriaLabel:"إغلاق"},assignedBreakoutRoomClosed:{title:"تم إغلاق الغرفة الفرعية الخاصة بك",message:"سنعيدك إلى اجتماعك خلال 5 ثوان.",dismissButtonAriaLabel:"إغلاق"},assignedBreakoutRoomOpenedPromptJoin:{title:"هل تريد الانضمام إلى الغرفة الفرعية؟",message:"تم تعيينك إلى غرفة اجتماع فرعية.",dismissButtonAriaLabel:"إغلاق",primaryButtonLabel:"الانضمام إلى الغرفة",secondaryButtonLabel:"لاحقاً"},breakoutRoomJoined:{title:"لقد قمت بالانضمام إلى غرفة الاجتماع فرعية.",dismissButtonAriaLabel:"‏إغلاق"},breakoutRoomClosingSoon:{title:"أوشكت صلاحية الحد الزمني للغرفة على الانتهاء.",message:"سيتم إغلاق هذه الغرفة في غضون 30 ثانية.",dismissButtonAriaLabel:"‏إغلاق"},capabilityTurnVideoOnPresent:{title:"تم تمكين الكاميرا لديك",message:"قم بتشغيلها إذا كنت ترغب في مشاركة الفيديو.",dismissButtonAriaLabel:"إغلاق"},capabilityTurnVideoOnAbsent:{title:"تم تعطيل الكاميرا",message:"لم يعد بإمكانك مشاركة الفيديو.",dismissButtonAriaLabel:"إغلاق"},capabilityUnmuteMicPresent:{title:"تم تمكين الميكروفون لديك",message:"قم بإلغاء كتم الصوت إذا أردت التحدث.",dismissButtonAriaLabel:"إغلاق"},capabilityUnmuteMicAbsent:{title:"تم تعطيل الميكروفون",message:"لم يعد بإمكانك إلغاء كتم الصوت.",dismissButtonAriaLabel:"إغلاق"},togetherModeStarted:{title:"تم بدء وضع التجميع",dismissButtonAriaLabel:"إغلاق"},togetherModeEnded:{title:"تم إنهاء وضع التجميع",dismissButtonAriaLabel:"إغلاق"},transcriptionError:{title:"حدث خطأ ما أثناء كتابة الحديث.",message:"تم إيقاف كتابة الحديث. الرجاء المحاولة مرة أخرى لاحقا",dismissButtonAriaLabel:"إغلاق"},transcriptionStartedByYou:{title:"بدأت كتابة الحديث.",message:"تم البدء بواسطتك. اسمح للجميع بمعرفة أنها تتم كتابة حدثيهم.",dismissButtonAriaLabel:"إغلاق"}};var videoGallery$l={screenIsBeingSharedMessage:"أنت تشارك شاشتك",screenShareLoadingMessage:"يتم الآن تحميل شاشة {participant}",localScreenShareLoadingMessage:"تحميل الشاشة",localVideoLabel:"أنت",localVideoCameraSwitcherLabel:"تبديل الكاميرا",localVideoMovementLabel:"لوحة فيديو محلية قابلة للنقل",localVideoMovementAriaLabel:"لوحة فيديو محلية قابلة للنقل",localVideoSelectedDescription:"تم تحديد {cameraName}",displayNamePlaceholder:"مشارك غير مسمى",fitRemoteParticipantToFrame:"الملاءمة داخل إطار",fillRemoteParticipantFrame:"تعبئة الإطار",pinParticipantForMe:"تثبيت نيابة عني",pinParticipantForMeLimitReached:"تثبيت (تم بلوغ الحد الأقصى)",unpinParticipantForMe:"إلغاء التثبيت",pinParticipantMenuItemAriaLabel:"تثبيت {participantName}",unpinParticipantMenuItemAriaLabel:"إزالة تثبيت {participantName}",pinnedParticipantAnnouncementAriaLabel:"تم تثبيت {participantName}",unpinnedParticipantAnnouncementAriaLabel:"تم إلغاء تثبيت {participantName}",startSpotlightVideoTileMenuLabel:"تسليط الضوء لدى الجميع",addSpotlightVideoTileMenuLabel:"إضافة تسليط ضوء",spotlightLimitReachedMenuTitle:"تم الوصول إلى الحد الأقصى لتسليط الضوء",stopSpotlightVideoTileMenuLabel:"التوقف عن تسليط الضوء",stopSpotlightOnSelfVideoTileMenuLabel:"الخروج من وضع تسليط الضوء",attendeeRole:"الحضور",muteParticipantMenuItemLabel:"كتم الصوت",waitingScreenText:"في انتظار انضمام الآخرين",forbidAudio:"تعطيل الميكروفون",permitAudio:"السماح بالميكروفون",forbidAudioTileMenuLabel:"تعطيل الميكروفون",permitAudioTileMenuLabel:"السماح بالميكروفون",forbidVideo:"تعطيل الكاميرا",permitVideo:"السماح بالكاميرا",forbidVideoTileMenuLabel:"تعطيل الكاميرا",permitVideoTileMenuLabel:"السماح بالكاميرا"};var dialpad$l={placeholderText:"أدخل رقم الهاتف",deleteButtonAriaLabel:"حذف"};var holdButton$l={onLabel:"استئناف",offLabel:"انتظار",tooltipOnContent:"استئناف المكالمة",tooltipOffContent:"تعليق المكالمة"};var videoTile$l={participantStateRinging:"يتم الآن الاتصال...",participantReconnecting:"يتم الآن إعادة الاتصال...",participantStateHold:"قيد الانتظار",moreOptionsButtonAriaLabel:"خيارات إضافية {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"مكتوم الصوت",moreOptionsParticipantMutedStateUnmutedAriaLabel:"غير مكتوم الصوت",moreOptionsParticipantHandRaisedAriaLabel:"يد مرفوعة",moreOptionsParticipantIsSpeakingAriaLabel:"يتحدث",moreOptionsParticipantMicDisabledAriaLabel:"تم تعطيل الميكروفون",moreOptionsParticipantCameraDisabledAriaLabel:"تم تعطيل الكاميرا"};var CameraAndMicrophoneSitePermissionsRequest$l={primaryText:"السماح {appName} باستخدام الكاميرا والميكروفون لديك",secondaryText:"هذا حتى يتمكن المشاركون من مشاهدتك والاستماع إليك.",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها",primaryButtonText:"المتابعة بدون الكاميرا والميكروفون",ariaLabel:"السماح بالوصول إلى الكاميرا والميكروفون"};var CameraSitePermissionsRequest$l={primaryText:"السماح {appName} باستخدام الكاميرا",secondaryText:"هذا حتى يتمكن المشاركون من مشاهدتك والاستماع إليك.",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها",primaryButtonText:"المتابعة دون كاميرا",ariaLabel:"السماح بالوصول إلى الكاميرا"};var MicrophoneSitePermissionsRequest$l={primaryText:"السماح {appName} باستخدام الميكروفون",secondaryText:"هذا حتى يتمكن المشاركون من الاستماع إليك.",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها",primaryButtonText:"المتابعة دون ميكروفون",ariaLabel:"السماح بالوصول إلى الميكروفون"};var CameraAndMicrophoneSitePermissionsCheck$l={primaryText:"التحقق من الوصول إلى الكاميرا والميكروفون",secondaryText:"السماح بالوصول إذا طُلب منك ذلك. هذا حتى يتمكن المشاركون من رؤيتك والاستماع إليك.",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها",primaryButtonText:"المتابعة بدون الكاميرا والميكروفون",ariaLabel:"التحقق من الوصول إلى الكاميرا والميكروفون. السماح بالوصول إذا طُلب منك ذلك."};var CameraSitePermissionsCheck$l={primaryText:"التحقق من الوصول إلى الكاميرا",secondaryText:"السماح بالوصول إذا طُلب منك ذلك. هذا حتى يتمكن المشاركون من رؤيتك.",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها",primaryButtonText:"المتابعة بدون كاميرا",ariaLabel:"التحقق من الوصول إلى الكاميرا. السماح بالوصول إذا طُلب منك ذلك."};var MicrophoneSitePermissionsCheck$l={primaryText:"التحقق من الوصول إلى الميكروفون",secondaryText:"السماح بالوصول إذا طُلب منك ذلك. هذا حتى يتمكن المشاركون من الاستماع إليك.",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها",primaryButtonText:"المتابعة بدون ميكروفون",ariaLabel:"التحقق من الوصول إلى الميكروفون. السماح بالوصول إذا طُلب منك ذلك."};var CameraAndMicrophoneSitePermissionsDenied$l={primaryText:"يتعذر الوصول إلى الكاميرا والميكروفون",secondaryText:"انقر فوق أيقونة التأمين في شريط العناوين لمنح أذونات الميكروفون لصفحة الويب هذه. قد يكون تحديث الصفحة مطلوبا.",primaryButtonText:"المتابعة دون الكاميرا والميكروفون",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها"};var CameraAndMicrophoneSitePermissionsDeniedSafari$l={primaryText:"يتعذر الوصول إلى الكاميرا والميكروفون",secondaryText:"قم بتحديث الصفحة للسماح بالأذونات، أو تحقق من إعدادات هذا المستعرض وتحقق من تمكين الأذونات لموقع الويب هذا.",primaryButtonText:"المتابعة دون الكاميرا والميكروفون",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها"};var CameraSitePermissionsDenied$l={primaryText:"تعذر الوصول إلى الكاميرا",secondaryText:"انقر فوق أيقونة التأمين في شريط العناوين لمنح أذونات الكاميرا لصفحة ويب هذه. قد يكون تحديث الصفحة مطلوبا.",primaryButtonText:"المتابعة دون كاميرا",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها"};var MicrophoneSitePermissionsDenied$l={primaryText:"يتعذر الوصول إلى الميكروفون",secondaryText:"انقر فوق أيقونة التأمين في شريط العناوين لمنح أذونات الميكروفون لصفحة الويب هذه. قد يكون تحديث الصفحة مطلوبا.",primaryButtonText:"المتابعة بدون ميكروفون",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها"};var CameraSitePermissionsDeniedSafari$l={primaryText:"تعذر الوصول إلى الكاميرا",secondaryText:"قم بتحديث الصفحة للسماح بالأذونات، أو تحقق من إعدادات هذا المستعرض وتحقق من تمكين الأذونات لموقع الويب هذا.",primaryButtonText:"المتابعة دون كاميرا",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها"};var MicrophoneSitePermissionsDeniedSafari$l={primaryText:"يتعذر الوصول إلى الميكروفون",secondaryText:"قم بتحديث الصفحة للسماح بالأذونات، أو تحقق من إعدادات هذا المستعرض وتحقق من تمكين الأذونات لموقع الويب هذا.",primaryButtonText:"المتابعة دون ميكروفون",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها"};var UnsupportedBrowser$l={primaryText:"المتصفح غير مدعوم",secondaryText:"الرجاء الانضمام إلى هذه المكالمة باستخدام مستعرض متوافق.",moreHelpLinkText:"الاطلاع على متطلبات التوافق"};var UnsupportedBrowserVersion$l={primaryText:"يلزم تحديث المستعرض",secondaryText:"لضمان أفضل مكالمة ممكنة، الرجاء تحديث المستعرض الخاص بك ثم محاولة الانضمام إلى المكالمة مرة أخرى.",moreHelpLinkText:"الاطلاع على متطلبات التوافق",continueAnywayButtonText:"بدء المكالمة دون تحديث"};var UnsupportedOperatingSystem$l={primaryText:"نظام التشغيل غير معتمد",secondaryText:"الرجاء الانضمام إلى هذه المكالمة باستخدام جهاز بنظام تشغيل متوافق.",moreHelpLinkText:"الاطلاع على متطلبات التوافق"};var BrowserPermissionDenied$l={primaryText:"يتعذر استخدام الكاميرا أو الميكروفون",secondaryText:"قد لا يكون لدى المستعرض إمكانية الوصول إلى الكاميرا أو الميكروفون لديك. لحل هذه المشكلة، افتح \"تفضيلات النظام\".",primaryButtonText:"حاول مرة أخرى",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها"};var BrowserPermissionDeniedIOS$l={primaryText:"السماح بالوصول إلى الميكروفون للمتابعة",secondaryText:"حتى يتمكن المشاركون الآخرون من سماعك.",primaryButtonText:"حاول مرة أخرى",imageAltText:"موقع إذن جهاز الكاميرا والميكروفون لـ iOS",linkText:"هل تحتاج إلى مساعدة؟ الحصول على تعليمات استكشاف الأخطاء وإصلاحها",step1Text:"انتقل إلى تطبيق الإعدادات",step2Text:"التمرير لأسفل للوصول إلى إعدادات هذا المستعرض",step3Text:"تشغيل الميكروفون (الكاميرا اختيارية)",step4Text:"حاول الانضمام إلى المكالمة مرة أخرى",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$l={leftNavButtonAriaLabel:"الصفحة السابقة {current} من {total}",rightNavButtonAriaLabel:"الصفحة التالية {current} من {total}"};var meetingConferencePhoneInfo$l={meetingConferencePhoneInfoModalTitle:"الانضمام إلى مكالمة من هاتفك",meetingConferencePhoneInfoModalDialIn:"رقم الاتصال",meetingConferencePhoneInfoModalMeetingId:"أدخل معرف الاجتماع",meetingConferencePhoneInfoModalWait:"انتظر حتى يتم القبول من قبل المنظم",meetingConferencePhoneInfoModalNoPhoneAvailable:"لا يتوفر هاتف",meetingConferencePhoneInfoModalTollGeoData:"{country}، {city}",meetingConferencePhoneInfoModalTollFree:"(رقم مجاني)",meetingConferencePhoneInfoModalToll:"(الرقم غير المجاني)"};var IncomingCallNotification$l={incomingCallNotificationPlaceholderId:"مستخدم غير معروف",incomingCallNotificationPlaceholderAlert:"{callerName} يتصل",incomingCallNoticicationAcceptWithAudioAriaLabel:"قبول مع الصوت",incomingCallNoticicationAcceptWithVideoAriaLabel:"قبول مع الفيديو",incomingCallNoticicationRejectAriaLabel:"رفض",incomingCallNotificationAcceptButtonLabel:"قبول",incomingCallNotificationRejectButtonLabel:"رفض",incomingCallNotificationAccceptWithVideoButtonLabel:"قبول مع الفيديو",incomingCallNotificationDismissButtonAriaLabel:"تجاهل"};var ar_SA$1 = {participantItem:participantItem$l,ParticipantList:ParticipantList$l,typingIndicator:typingIndicator$l,sendBox:sendBox$l,richTextSendBox:richTextSendBox$l,realTimeTextModal:realTimeTextModal$l,captionsBanner:captionsBanner$l,mentionPopover:mentionPopover$l,imageOverlay:imageOverlay$l,messageStatusIndicator:messageStatusIndicator$l,endCallButton:endCallButton$l,cameraButton:cameraButton$l,microphoneButton:microphoneButton$l,devicesButton:devicesButton$l,participantsButton:participantsButton$l,screenShareButton:screenShareButton$l,raiseHandButton:raiseHandButton$l,reactionButton:reactionButton$l,realTimeText:realTimeText$l,captionsSettingsModal:captionsSettingsModal$l,spokenLanguages:spokenLanguages$l,captionLanguages:captionLanguages$l,startCaptionsButton:startCaptionsButton$l,startRealTimeTextButton:startRealTimeTextButton$l,messageThread:messageThread$l,errorBar:errorBar$l,notificationStack:notificationStack$l,videoGallery:videoGallery$l,dialpad:dialpad$l,holdButton:holdButton$l,videoTile:videoTile$l,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$l,CameraSitePermissionsRequest:CameraSitePermissionsRequest$l,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$l,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$l,CameraSitePermissionsCheck:CameraSitePermissionsCheck$l,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$l,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$l,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$l,CameraSitePermissionsDenied:CameraSitePermissionsDenied$l,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$l,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$l,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$l,UnsupportedBrowser:UnsupportedBrowser$l,UnsupportedBrowserVersion:UnsupportedBrowserVersion$l,UnsupportedOperatingSystem:UnsupportedOperatingSystem$l,BrowserPermissionDenied:BrowserPermissionDenied$l,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$l,verticalGallery:verticalGallery$l,meetingConferencePhoneInfo:meetingConferencePhoneInfo$l,IncomingCallNotification:IncomingCallNotification$l};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Arabic (Saudi Arabia).
 *
 * @public
 */
const COMPONENT_LOCALE_AR_SA = {
    strings: createComponentStrings(ar_SA$1)
};

var participantItem$k={isMeText:"(vy)",menuTitle:"Další možnosti",removeButtonLabel:"Odebrat",sharingIconLabel:"Sdílení",mutedIconLabel:"Ztlumeno",handRaisedIconLabel:"Zvednutá ruka – pozice {position}",displayNamePlaceholder:"Nepojmenovaný účastník",participantStateRinging:"Volání…",participantStateHold:"Pozastavený hovor",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, Další možnosti",attendeeRole:"Účastník",micDisabledIconLabel:"Mikrofon zakázán",cameraDisabledIconLabel:"Kamera zakázána"};var ParticipantList$k={overflowParticipantCount:"+{overflowCount} další(ch)"};var typingIndicator$k={singleUser:"{user} píše...",multipleUsers:"{users} píší...",multipleUsersAbbreviateOne:"{users} a 1 další píší...",multipleUsersAbbreviateMany:"{users} a {numOthers} ostatní píší...",delimiter:", "};var sendBox$k={placeholderText:"Zadejte zprávu",textTooLong:"Délka vaší zprávy překračuje maximální limit.",sendButtonAriaLabel:"Poslat zprávu",attachmentUploadsPendingError:"Soubor se nahrává, počkejte prosím.",removeAttachment:"Odebrat soubor",uploading:"Nahrávání",uploadCompleted:"Nahrávání se dokončilo",attachmentMoreMenu:"Další možnosti"};var richTextSendBox$k={placeholderText:"Zadejte zprávu",textTooLong:"Délka vaší zprávy překračuje maximální limit.",sendButtonAriaLabel:"Poslat zprávu",attachmentUploadsPendingError:"Soubor se nahrává, počkejte prosím.",imageUploadsPendingError:"Obrázek se nahrává, počkejte prosím.",removeAttachment:"Odebrat soubor",uploading:"Nahrávání",uploadCompleted:"Nahrávání se dokončilo",richTextBoldTooltip:"Tučné",richTextItalicTooltip:"Kurzíva",richTextUnderlineTooltip:"Podtržení",richTextBulletListTooltip:"Seznam s odrážkami",richTextNumberListTooltip:"Číslovaný seznam",richTextIncreaseIndentTooltip:"Zvětšit odsazení",richTextDecreaseIndentTooltip:"Zmenšit odsazení",richTextInsertTableTooltip:"Vložit tabulku",richTextFormatButtonTooltip:"Formát",richTextInsertRowOrColumnMenu:"Vložit",richTextInsertTableMenuTitle:"Vložit {column} x {row} tabulka",richTextInsertRowAboveMenu:"Vložit nad",richTextInsertRowBelowMenu:"Vložit pod",richTextInsertColumnLeftMenu:"Vložit vlevo",richTextInsertColumnRightMenu:"Vložit vpravo",richTextDeleteRowOrColumnMenu:"Odstranit",richTextDeleteColumnMenu:"Odstranit sloupec",richTextDeleteRowMenu:"Odstranit řádek",richTextDeleteTableMenu:"Odstranit tabulku",richTextToolbarAriaLabel:"Panel nástrojů Formátování zpráv",richTextToolbarMoreButtonAriaLabel:"Další možnosti",attachmentMoreMenu:"Další možnosti",richTextNewBulletedListItemAnnouncement:"Odrážka, nový řádek",richTextNewNumberedListItemAnnouncement:"{0}, nový řádek",richTextBulletedListAppliedAnnouncement:"Byl použit seznam s odrážkami",richTextNumberedListAppliedAnnouncement:"Použil se číslovaný seznam."};var realTimeTextModal$k={realTimeTextModalAriaLabel:"Zapnout modální okno psaní v reálném čase",realTimeTextModalTitle:"Zapnout psaní v reálném čase?",realTimeTextModalText:"Text v reálném čase (RTT) se zapne pro všechny účastníky schůzky.",realTimeTextConfirmButtonLabel:"Zapnout",realTimeTextCancelButtonLabel:"Zrušit",realTimeTextCloseModalButtonAriaLabel:"Zavřít modální okno psaní v reálném čase"};var captionsBanner$k={captionsBannerSpinnerText:"Spouští se titulky...",realTimeTextInputBoxDefaultText:"Napište zprávu v reálném čase",realTimeTextInputErrorMessage:"Zpráva nesmí být delší než 2000 znaků",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (psaní v reálném čase) je povolené pro všechny účastníky po celou dobu trvání schůzky.",realTimeTextBannerLinkLabel:"Další informace",captionsOnlyContainerTitle:"Titulky",realTimeTextOnlyContainerTitle:"Text v reálném čase",captionsAndRealTimeTextContainerTitle:"Titulky a text v reálném čase",expandButtonAriaLabel:"Rozbalit",minimizeButtonAriaLabel:"Sbalit"};var mentionPopover$k={mentionPopoverHeader:"Návrhy"};var imageOverlay$k={downloadButtonLabel:"Stáhnout",dismissButtonAriaLabel:"Zavřít",overlayVisibleMessage:"Překrytí podrobností obrázku"};var messageStatusIndicator$k={deliveredAriaLabel:"Zpráva byla odeslána",deliveredTooltipText:"Odesláno",seenAriaLabel:"Zpráva zobrazena příjemci",seenTooltipText:"Zobrazeno",readByTooltipText:"Přečetl(a) {messageThreadReadCount} z {remoteParticipantsCount}",sendingAriaLabel:"Odesílání zprávy",sendingTooltipText:"Odesílání",failedToSendAriaLabel:"Zprávu se nepodařilo odeslat",failedToSendTooltipText:"Odeslání se nezdařilo"};var endCallButton$k={label:"Opustit",tooltipContent:"Opustit hovor",leaveOption:"Odejít",endCallOption:"Ukončit hovor pro všechny"};var cameraButton$k={onLabel:"Kamera",offLabel:"Kamera",tooltipDisabledContent:"Kamera je zakázaná",tooltipOnContent:"Vypnout kameru",tooltipOffContent:"Zapnout kameru",tooltipVideoLoadingContent:"Video se načítá",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Zvolit kameru",cameraButtonSplitRoleDescription:"Tlačítko rozdělení",onSplitButtonAriaLabel:"Vypnout kameru",offSplitButtonAriaLabel:"Zapnout kameru",cameraActionTurnedOnAnnouncement:"Vaše kamera je zapnutá",cameraActionTurnedOffAnnouncement:"Vaše kamera je vypnutá",onSplitButtonPrimaryActionCamera:"Vypnout kameru",offSplitButtonPrimaryActionCamera:"Zapnout kameru",cameraPrimaryActionSplitButtonTitle:"Použít kameru",videoEffectsMenuItemTitle:"Efekty",cameraButtonAriaDescription:"možnosti kamery"};var microphoneButton$k={onLabel:"Mikrofon",offLabel:"Mikrofon",tooltipDisabledContent:"Mikrofon je zakázaný",tooltipOnContent:"Ztlumit mikrofon",tooltipOffContent:"Zrušit ztlumení mikrofonu",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Zvolit mikrofon",speakerMenuTitle:"Reproduktor",speakerMenuTooltip:"Zvolit reproduktor",microphoneButtonSplitRoleDescription:"Tlačítko rozdělení",onSplitButtonAriaLabel:"Ztlumit mikrofon",offSplitButtonAriaLabel:"Zrušit ztlumení mikrofonu",microphoneActionTurnedOnAnnouncement:"Váš mikrofon je zapnutý",microphoneActionTurnedOffAnnouncement:"Váš mikrofon je vypnutý",offSplitButtonMicrophonePrimaryAction:"Zrušit ztlumení mikrofonu",onSplitButtonMicrophonePrimaryAction:"Ztlumit mikrofon",microphonePrimaryActionSplitButtonTitle:"Použít mikrofon",microphoneAriaDescription:"Možnosti zvuku",deepNoiseSuppressionTitle:"Potlačení šumu",deepNoiseSuppressionOnAnnouncement:"Potlačení hlubokého šumu je zapnuté",deepNoiseSuppressionOffAnnouncement:"Potlačení hlubokého šumu je vypnuté"};var devicesButton$k={label:"Zařízení",tooltipContent:"Spravovat zařízení",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Zvolit kameru",audioDeviceMenuTitle:"Zvukové zařízení",audioDeviceMenuTooltip:"Zvolit zvukové zařízení",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Zvolit mikrofon",speakerMenuTitle:"Reproduktor",speakerMenuTooltip:"Zvolit reproduktor"};var participantsButton$k={label:"Lidé",ariaLabel:"Lidé",tooltipContent:"Zobrazit účastníky",menuHeader:"V tomto hovoru",participantsListButtonLabel:"Počet lidí: {numParticipants}",muteAllButtonLabel:"Ztlumit všechny",copyInviteLinkButtonLabel:"Kopírovat odkaz pro pozvání",copyInviteLinkActionedAriaLabel:"Odkaz pozvánky se zkopíroval"};var screenShareButton$k={onLabel:"Zastavit prezentaci",offLabel:"Prezentovat",tooltipDisabledContent:"Sdílení je zakázáno",tooltipOnContent:"Sdílí se vaše obrazovka",tooltipOffContent:"Sdílet obrazovku"};var raiseHandButton$k={offLabel:"Zvednout",onLabel:"Dolů",tooltipDisabledContent:"Akce Zvednout ruku je zakázaná",tooltipOnContent:"Dát ruku dolů",tooltipOffContent:"Zvednout ruku"};var reactionButton$k={label:"Reagovat",ariaLabel:"Reagovat",tooltipDisabledContent:"Akce reakce je zakázaná",tooltipContent:"Odeslání reakce",likeReactionTooltipContent:"To se mi líbí",heartReactionTooltipContent:"Láska",laughReactionTooltipContent:"Smích",applauseReactionTooltipContent:"Aplaus",surprisedReactionTooltipContent:"Překvapení"};var realTimeText$k={isTypingText:"Probíhá psaní v reálném čase"};var captionsSettingsModal$k={captionsSettingsModalTitle:"Nastavení titulků",captionsSettingsSpokenLanguageDropdownLabel:"Mluvený jazyk",captionsSettingsCaptionLanguageDropdownLabel:"Jazyk titulků",captionsSettingsSpokenLanguageDropdownInfoText:"Jazyk, kterým mluví všichni účastníci tohoto hovoru.",captionsSettingsCaptionLanguageDropdownInfoText:"Titulky se zobrazí v tomto jazyce. ",captionsSettingsConfirmButtonLabel:"Potvrdit",captionsSettingsCancelButtonLabel:"Zrušit",captionsSettingsModalAriaLabel:"Modální nastavení titulků",captionsSettingsCloseModalButtonAriaLabel:"Nastavení titulků"};var spokenLanguages$k={"ar-ae":"Arabština - Spojené arabské emiráty","ar-sa":"Arabština - Saúdská Arábie","da-dk":"Dánština","de-de":"Němčina - Německo","en-au":"Angličtina - Austrálie","en-ca":"Angličtina - Kanada","en-gb":"Angličtina - Spojené království","en-in":"Angličtina - Indie","en-nz":"Angličtina - Nový Zéland","en-us":"Angličtina - Spojené státy americké","es-es":"Španělština - Španělsko (Modern Sort)","es-mx":"Španělština - Mexiko","fi-fi":"Finština","fr-ca":"Francouzština - Kanada","fr-fr":"Francouzština - Francie","hi-in":"Hindština","it-it":"Italština - Itálie","ja-jp":"Japonština","ko-kr":"Korejština","nb-no":"Norština (Bokmål)","nl-be":"Holandština – Belgie","nl-nl":"Holandština - Nizozemsko","pl-pl":"Polština","pt-br":"Portugalština – Brazílie","ru-ru":"Ruština","sv-se":"Švédština","zh-cn":"Čínština – Čínská lidová republika","zh-hk":"Čínština – Hong Kong SAR","cs-cz":"Čeština","pt-pt":"Portugalština – Portugalsko","tr-tr":"Turečtina","vi-vn":"Vietnamština","th-th":"Thajština","he-il":"Hebrejština","cy-gb":"Velština","uk-ua":"Ukrajinština","el-gr":"Řečtina","hu-hu":"Maďarština","ro-ro":"Rumunština","sk-sk":"Slovenština","zh-tw":"Čínština – Tchaj-wan"};var captionLanguages$k={ar:"Arabština",da:"Dánština",de:"Němčina",en:"Angličtina",es:"Španělština",fi:"Finština","fr-ca":"Francouzština - Kanada",fr:"Francouzština - Francie",hi:"Hindština",it:"Italština",ja:"Japonština",ko:"Korejština",nb:"Norština (Bokmål)",nl:"Nizozemština",pl:"Polština",pt:"Portugalština – Brazílie",ru:"Ruština",sv:"Švédština","zh-Hans":"Čínština (zjednodušená)","zh-Hant":"Čínština (tradiční)",cs:"Čeština","pt-pt":"Portugalština – Portugalsko",tr:"Turečtina",vi:"Vietnamština",th:"Thajština",he:"Hebrejština",cy:"Velština",uk:"Ukrajinština",el:"Řečtina",hu:"Maďarština",ro:"Rumunština",sk:"Slovenština"};var startCaptionsButton$k={onLabel:"Zapnout titulky",offLabel:"Vypnout titulky",tooltipOnContent:"Vypnout titulky",tooltipOffContent:"Zapnout titulky"};var startRealTimeTextButton$k={label:"Zapnout RTT pro tento hovor",tooltipOffContent:"Zapnout RTT pro tento hovor"};var messageThread$k={yesterday:"Včera",sunday:"Neděle",monday:"Pondělí",tuesday:"Úterý",wednesday:"Středa",thursday:"Čtvrtek",friday:"Pátek",saturday:"Sobota",participantJoined:"se připojil(a) k chatu.",participantLeft:"opustil(a) chat.",editMessage:"Upravit",removeMessage:"Odstranit",resendMessage:"Zkusit odeslat znovu",failToSendTag:"Odeslání se nezdařilo",editedTag:"Upraveno",liveAuthorIntro:"Zpráva od {author}",editedMessageLiveAuthorIntro:"Upravená zpráva od {author}",editedMessageLocalUserLiveAuthorIntro:"Upravená zpráva",messageContentAriaText:"{status} {author} řekl(a) {message}",messageContentMineAriaText:"{status} Řekli jste {message}.",editBoxTextLimit:"Vaše zpráva překročila limit {limitNumber} znaků.",editBoxPlaceholderText:"Upravit zprávu",newMessagesIndicator:"Nové zprávy",noDisplayNameSub:"Bez jména",editBoxCancelButton:"Zrušit",editBoxSubmitButton:"Hotovo",messageReadCount:"Přečetl(a) {messageReadByCount} z {remoteParticipantsCount}",actionMenuMoreOptions:"Další možnosti",downloadAttachment:"Stáhnout soubor",openAttachment:"Otevřít soubor",blockedWarningText:"Tato zpráva se odstranila kvůli zásadám organizace.",blockedWarningLinkText:"Podrobnosti",attachmentCardGroupMessage:"Zpráva obsahuje {attachmentCount} přílohu",messageDeletedAnnouncementAriaLabel:"Zpráva se odstranila.",richTextBoldTooltip:"Tučné",richTextItalicTooltip:"Kurzíva",richTextUnderlineTooltip:"Podtržení",richTextBulletListTooltip:"Seznam s odrážkami",richTextNumberListTooltip:"Číslovaný seznam",richTextIncreaseIndentTooltip:"Zvětšit odsazení",richTextDecreaseIndentTooltip:"Zmenšit odsazení",richTextInsertTableTooltip:"Vložit tabulku",richTextFormatButtonTooltip:"Formát",richTextInsertRowOrColumnMenu:"Vložit",richTextInsertTableMenuTitle:"Vložit {column} x {row} tabulka",richTextInsertRowAboveMenu:"Vložit nad",richTextInsertRowBelowMenu:"Vložit pod",richTextInsertColumnLeftMenu:"Vložit vlevo",richTextInsertColumnRightMenu:"Vložit vpravo",richTextDeleteRowOrColumnMenu:"Odstranit",richTextDeleteColumnMenu:"Odstranit sloupec",richTextDeleteRowMenu:"Odstranit řádek",richTextDeleteTableMenu:"Odstranit tabulku",richTextToolbarMoreButtonAriaLabel:"Další možnosti",attachmentUploadsPendingError:"Soubor se nahrává, počkejte prosím.",imageUploadsPendingError:"Obrázek se nahrává, počkejte prosím.",richTextNewBulletedListItemAnnouncement:"Odrážka, nový řádek",richTextNewNumberedListItemAnnouncement:"{0}, nový řádek",richTextBulletedListAppliedAnnouncement:"Použil se seznam s odrážkami",richTextNumberedListAppliedAnnouncement:"Použil se číslovaný seznam"};var errorBar$k={unableToReachChatService:"Jste offline",accessDenied:"Nepovedlo se získat přístup ke službám chatu. Zkontrolujte prosím zadané přihlašovací údaje uživatele.",userNotInChatThread:"Už nejste v tomto vlákně chatu",sendMessageNotInChatThread:"Zprávu se nepovedlo poslat, protože už nejste v tomto vláknu chatu.",sendMessageGeneric:"Zprávu se nepovedlo odeslat",callingNetworkFailure:"Máte problémy s připojením hovoru – vypadá to, že jste offline",startVideoGeneric:"Video se nepovedlo spustit",stopVideoGeneric:"Video se nepovedlo zastavit",muteGeneric:"Nepovedlo se ztlumit mikrofon",unmuteGeneric:"Nepovedlo se zrušit ztlumení mikrofonu",speakingWhileMuted:"Váš mikrofon je ztlumený",startScreenShareGeneric:"Při spouštění sdílení obrazovky došlo k problému.",stopScreenShareGeneric:"Nepovedlo se zastavit sdílení obrazovky",callNetworkQualityLow:"Kvalita sítě je nízká.",teamsMeetingCallNetworkQualityLow:"Kvalita sítě je nízká.",callNoSpeakerFound:"Nenašly se žádné reproduktory ani sluchátka. Připojte zvukové zařízení, abyste slyšeli hovor.",callNoMicrophoneFound:"Nenašly se žádné mikrofony. Připojte vstupní zvukové zařízení.",callMicrophoneAccessDenied:"Nepovedlo se získat přístup k mikrofonu. Kliknutím na zámek na panelu Adresa udělte oprávnění k této webové stránce.",callMicrophoneAccessDeniedSafari:"Nepovedlo se získat přístup k mikrofonu. Aktualizujte stránku, abyste povolili oprávnění, nebo zkontrolujte nastavení tohoto prohlížeče a ověřte, jestli jsou pro tento web povolená oprávnění.",callMicrophoneMutedBySystem:"Systém vás ztlumil.",callMicrophoneUnmutedBySystem:"Váš mikrofon se obnovil a systém vám zrušil ztlumení.",callMacOsMicrophoneAccessDenied:"Nepovedlo se získat přístup k mikrofonu. Udělte v nastavení ochrany osobních údajů pro macOS oprávnění k mikrofonu.",callLocalVideoFreeze:"Šířka pásma sítě je nekvalitní. Vaše video se může zobrazovat jako pozastavené pro ostatní účastníky hovoru.",callCameraAccessDenied:"Nepovedlo se získat přístup ke kameře. Kliknutím na zámek na panelu Adresa udělte oprávnění k této webové stránce.",callCameraAccessDeniedSafari:"Nejde získat přístup ke kameře. Aktualizujte stránku, abyste povolili oprávnění, nebo zkontrolujte nastavení tohoto prohlížeče a ověřte, jestli jsou pro tento web povolená oprávnění.",callCameraAlreadyInUse:"Nejde získat přístup ke kameře. Je možné, že ji už používá jiná aplikace.",callVideoStoppedBySystem:"Vaše video bylo zastaveno systémem.",callVideoRecoveredBySystem:"Vaše video se obnovilo.",callMacOsCameraAccessDenied:"MacOS blokuje přístup k vaší kameře. Aktualizujte si nastavení ochrany osobních údajů, aby tento prohlížeč měl přístup k vaší kameře.",callMacOsScreenShareAccessDenied:"MacOS blokuje sdílení obrazovky. Aktualizujte nastavení ochrany osobních údajů, aby tento prohlížeč mohl nahrávat vaši obrazovku.",dismissButtonAriaLabel:"Zavřít",failedToJoinCallGeneric:"K hovoru se nepovedlo připojit.",failedToJoinCallInvalidMeetingLink:"Ke schůzce se nelze připojit. Neplatný odkaz.",cameraFrozenForRemoteParticipants:"Uživatelé v hovoru mají problémy s zobrazením vašeho videa. Zkontrolujte prosím svá zařízení a síť.",unableToStartVideoEffect:"Nelze použít efekt videa.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Nelze zahájit vytváření hlavního výběru účastníků, protože je již v hlavním výběru maximální počet účastníků.",mutedByRemoteParticipant:"Byli jste ztlumení jiným účastníkem."};var notificationStack$k={callingNetworkFailure:{title:"Při připojování hovoru došlo k potížím",message:"Zdá se, že jste offline. Zkontrolujte si prosím síťové připojení a zkuste to znovu.",dismissButtonAriaLabel:"Zavřít"},startVideoGeneric:{title:"Video se nepovedlo spustit.",dismissButtonAriaLabel:"Zavřít"},stopVideoGeneric:{title:"Video se nepovedlo zastavit.",dismissButtonAriaLabel:"Zavřít"},muteGeneric:{title:"Nepovedlo se ztlumit mikrofon.",dismissButtonAriaLabel:"Zavřít"},unmuteGeneric:{title:"Nepovedlo se zrušit ztlumení mikrofonu.",dismissButtonAriaLabel:"Zavřít"},speakingWhileMuted:{title:"Jste ztlumení.",dismissButtonAriaLabel:"Zavřít"},startScreenShareGeneric:{title:"Při spouštění sdílení obrazovky došlo k problému.",dismissButtonAriaLabel:"Zavřít"},stopScreenShareGeneric:{title:"Nepovedlo se zastavit sdílení obrazovky.",dismissButtonAriaLabel:"Zavřít"},callNetworkQualityLow:{title:"Kvalita sítě je nízká.",dismissButtonAriaLabel:"Zavřít"},teamsMeetingCallNetworkQualityLow:{title:"Kvalita sítě je nízká.",dismissButtonAriaLabel:"Zavřít",message:"Připojte se k tomuto hovoru ze svého telefonu, abyste získali lepší zvuk. Můžete pokračovat v prohlížení schůzky na tomto zařízení.",primaryButtonLabel:"Připojit telefonem"},callNoSpeakerFound:{title:"Nenašly se žádné reproduktory ani sluchátka",message:"Připojte zvukové zařízení, abyste slyšeli hovor.",dismissButtonAriaLabel:"Zavřít"},callNoMicrophoneFound:{title:"Nenašly se žádné mikrofony",message:"Připojte vstupní zvukové zařízení.",dismissButtonAriaLabel:"Zavřít"},callMicrophoneAccessDenied:{title:"Nepovedlo se získat přístup k mikrofonu",message:"Kliknutím na zámek na panelu Adresa udělte oprávnění k této webové stránce.",dismissButtonAriaLabel:"Zavřít"},callMicrophoneAccessDeniedSafari:{title:"Nepovedlo se získat přístup k mikrofonu",message:"Aktualizujte stránku, abyste povolili oprávnění, nebo zkontrolujte nastavení tohoto prohlížeče a ověřte, jestli jsou pro tento web povolená oprávnění.",dismissButtonAriaLabel:"Zavřít"},callMicrophoneMutedBySystem:{title:"Systém vás ztlumil.",dismissButtonAriaLabel:"Zavřít"},callMicrophoneUnmutedBySystem:{title:"Váš mikrofon se obnovil a systém vám zrušil ztlumení.",dismissButtonAriaLabel:"Zavřít"},callMacOsMicrophoneAccessDenied:{title:"Nepovedlo se získat přístup k mikrofonu",message:"Udělte v nastavení ochrany osobních údajů pro macOS oprávnění k mikrofonu.",dismissButtonAriaLabel:"Zavřít"},callLocalVideoFreeze:{title:"Šířka pásma sítě je nekvalitní",message:"Vaše video se může zobrazovat jako pozastavené pro ostatní účastníky hovoru.",dismissButtonAriaLabel:"Zavřít"},callCameraAccessDenied:{title:"Nejde získat přístup ke kameře",message:"Kliknutím na zámek na panelu Adresa udělte oprávnění k této webové stránce.",dismissButtonAriaLabel:"Zavřít"},callCameraAccessDeniedSafari:{title:"Nejde získat přístup ke kameře",message:"Aktualizujte stránku, abyste povolili oprávnění, nebo zkontrolujte nastavení tohoto prohlížeče a ověřte, jestli jsou pro tento web povolená oprávnění.",dismissButtonAriaLabel:"Zavřít"},callCameraAlreadyInUse:{title:"Nejde získat přístup ke kameře",message:"Je možné, že ji už používá jiná aplikace.",dismissButtonAriaLabel:"Zavřít"},callVideoStoppedBySystem:{title:"Vaše video bylo zastaveno systémem.",dismissButtonAriaLabel:"Zavřít"},callVideoRecoveredBySystem:{title:"Vaše video se obnovilo.",dismissButtonAriaLabel:"Zavřít"},callMacOsCameraAccessDenied:{title:"MacOS blokuje přístup k vaší kameře",message:"Aktualizujte si nastavení ochrany osobních údajů, aby tento prohlížeč měl přístup k vaší kameře.",dismissButtonAriaLabel:"Zavřít"},callMacOsScreenShareAccessDenied:{title:"MacOS blokuje sdílení obrazovky",message:"Aktualizujte nastavení ochrany osobních údajů, aby tento prohlížeč mohl nahrávat vaši obrazovku.",dismissButtonAriaLabel:"Zavřít"},failedToJoinCallGeneric:{title:"K hovoru se nepovedlo připojit.",dismissButtonAriaLabel:"Zavřít"},failedToJoinCallInvalidMeetingLink:{title:"Ke schůzce se nelze připojit. Neplatný odkaz.",dismissButtonAriaLabel:"Zavřít"},cameraFrozenForRemoteParticipants:{title:"Uživatelé v hovoru mají problémy s zobrazením vašeho videa",message:"Zkontrolujte prosím svá zařízení a síť.",dismissButtonAriaLabel:"Zavřít"},unableToStartVideoEffect:{title:"Nelze použít efekt videa.",dismissButtonAriaLabel:"Zavřít"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Nelze spustit hlavní výběr u účastník(a/ů).",message:"Maximální počet účastníků je už vybraný.",dismissButtonAriaLabel:"Zavřít"},mutedByRemoteParticipant:{title:"Byli jste ztlumení jiným účastníkem.",dismissButtonAriaLabel:"Zavřít"},recordingStarted:{title:"Nahrávání se spustilo.",message:"Připojením vyjadřujete souhlas s přepisem této schůzky.",dismissButtonAriaLabel:"Zavřít"},transcriptionStarted:{title:"Přepis se spustil.",message:"Připojením vyjadřujete souhlas s přepisem této schůzky.",dismissButtonAriaLabel:"Zavřít"},recordingStopped:{title:"Probíhá ukládání záznamu.",message:"Nahrávání se zastavilo.",dismissButtonAriaLabel:"Zavřít"},transcriptionStopped:{title:"Přepis se ukládá.",message:"Přepis se zastavil.",dismissButtonAriaLabel:"Zavřít"},recordingAndTranscriptionStarted:{title:"Spustilo se nahrávání a přepis.",message:"Připojením vyjadřujete souhlas s přepisem této schůzky.",dismissButtonAriaLabel:"Zavřít"},recordingAndTranscriptionStopped:{title:"Záznam a přepis se ukládají.",message:"Nahrávání a přepis se zastavily.",dismissButtonAriaLabel:"Zavřít"},recordingStoppedStillTranscribing:{title:"Nahrávání se zastavilo.",message:"Nyní tuto schůzku pouze přepisujete.",dismissButtonAriaLabel:"Zavřít"},transcriptionStoppedStillRecording:{title:"Přepis se zastavil.",message:"Nyní tuto schůzku pouze nahráváte.",dismissButtonAriaLabel:"Zavřít"},assignedBreakoutRoomOpened:{title:"Za 5 sekund vás přesuneme do přiřazené místnosti.",dismissButtonAriaLabel:"Zavřít"},assignedBreakoutRoomChanged:{title:"Vaše skupinová místnost se změnila",message:"Za 5 sekund vás přesuneme do přiřazené místnosti.",dismissButtonAriaLabel:"Zavřít"},assignedBreakoutRoomClosed:{title:"Vaše skupinová místnost byla uzavřena",message:"Za 5 sekund vás přesuneme zpět na schůzku.",dismissButtonAriaLabel:"Zavřít"},assignedBreakoutRoomOpenedPromptJoin:{title:"Připojit se ke skupinové místnosti?",message:"Byli jste přiřazeni ke skupinové místnosti.",dismissButtonAriaLabel:"Zavřít",primaryButtonLabel:"Připojit se k místnosti",secondaryButtonLabel:"Později"},breakoutRoomJoined:{title:"Připojili jste se ke skupinové místnosti.",dismissButtonAriaLabel:"Zavřít"},breakoutRoomClosingSoon:{title:"Časový limit místnosti brzy vyprší.",message:"Tato místnost se zavře během 30 sekund.",dismissButtonAriaLabel:"Zavřít"},capabilityTurnVideoOnPresent:{title:"Vaše kamera je zapnutá",message:"Pokud byste chtěli sdílet video, zapněte tuto funkci.",dismissButtonAriaLabel:"Zavřít"},capabilityTurnVideoOnAbsent:{title:"Vaše kamera byla vypnuta",message:"Už nemůžete sdílet video.",dismissButtonAriaLabel:"Zavřít"},capabilityUnmuteMicPresent:{title:"Váš mikrofon je zapnutý.",message:"Zrušte ztlumení, pokud chcete mluvit.",dismissButtonAriaLabel:"Zavřít"},capabilityUnmuteMicAbsent:{title:"Váš mikrofon je vypnutý.",message:"Už nemůžete zrušit ztlumení.",dismissButtonAriaLabel:"Zavřít"},togetherModeStarted:{title:"Režim Togethermode byl spuštěn",dismissButtonAriaLabel:"Zavřít"},togetherModeEnded:{title:"Režim Togethermode skončil",dismissButtonAriaLabel:"Zavřít"},transcriptionError:{title:"Při přepisu došlo k nějaké chybě.",message:"Přepis se zastavil, zkuste to prosím znovu později",dismissButtonAriaLabel:"Zavřít"},transcriptionStartedByYou:{title:"Přepis se spustil.",message:"Spuštěno vámi. Dejte všem vědět, že se přepisují.",dismissButtonAriaLabel:"Zavřít"}};var videoGallery$k={screenIsBeingSharedMessage:"Sdílíte obrazovku",screenShareLoadingMessage:"Načítání obrazovky {participant}",localScreenShareLoadingMessage:"Načítá se obrazovka",localVideoLabel:"Vy",localVideoCameraSwitcherLabel:"Přepnout kameru",localVideoMovementLabel:"Dlaždice s místním videem s možností přesunutí",localVideoMovementAriaLabel:"Dlaždice s místním videem s možností přesunutí",localVideoSelectedDescription:"{cameraName} vybráno",displayNamePlaceholder:"Nepojmenovaný účastník",fitRemoteParticipantToFrame:"Přizpůsobit rámečku",fillRemoteParticipantFrame:"Vyplnit rámeček",pinParticipantForMe:"Připnout pro mě",pinParticipantForMeLimitReached:"Připnout (dosáhlo se limitu)",unpinParticipantForMe:"Odepnout",pinParticipantMenuItemAriaLabel:"Připnout {participantName}",unpinParticipantMenuItemAriaLabel:"Odepnout {participantName}",pinnedParticipantAnnouncementAriaLabel:"Připnutý účastník {participantName}",unpinnedParticipantAnnouncementAriaLabel:"{participantName} odepnut",startSpotlightVideoTileMenuLabel:"Přidat do hlavního výběru všem",addSpotlightVideoTileMenuLabel:"Přidat do hlavního výběru",spotlightLimitReachedMenuTitle:"Dosáhlo se limitu hlavního výběru",stopSpotlightVideoTileMenuLabel:"Zastavit přidávání do hlavního výběru",stopSpotlightOnSelfVideoTileMenuLabel:"Ukončit hlavní výběr",attendeeRole:"Účastník",muteParticipantMenuItemLabel:"Ztlumit",waitingScreenText:"Čeká se na připojení ostatních",forbidAudio:"Vypnout mikrofon",permitAudio:"Povolit mikrofon",forbidAudioTileMenuLabel:"Vypnout mikrofon",permitAudioTileMenuLabel:"Povolit mikrofon",forbidVideo:"Zakázat kameru",permitVideo:"Povolit kameru",forbidVideoTileMenuLabel:"Zakázat kameru",permitVideoTileMenuLabel:"Povolit kameru"};var dialpad$k={placeholderText:"Zadejte telefonní číslo",deleteButtonAriaLabel:"Odstranit"};var holdButton$k={onLabel:"Pokračovat",offLabel:"Pozastavit",tooltipOnContent:"Pokračovat v hovoru",tooltipOffContent:"Pozastavit hovor"};var videoTile$k={participantStateRinging:"Volání…",participantReconnecting:"Obnovuje se připojení...",participantStateHold:"Pozastavený hovor",moreOptionsButtonAriaLabel:"Další možnosti {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"ztlumeno",moreOptionsParticipantMutedStateUnmutedAriaLabel:"ztlumení zrušeno",moreOptionsParticipantHandRaisedAriaLabel:"Zvednutá ruka",moreOptionsParticipantIsSpeakingAriaLabel:"Mluví",moreOptionsParticipantMicDisabledAriaLabel:"Mikrofon zakázán",moreOptionsParticipantCameraDisabledAriaLabel:"Kamera zakázána"};var CameraAndMicrophoneSitePermissionsRequest$k={primaryText:"Povolit aplikaci {appName} používat kameru a mikrofon",secondaryText:"Toto je proto, aby vás účastníci viděli a slyšeli.",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží",primaryButtonText:"Pokračovat bez kamery a mikrofonu",ariaLabel:"Povolit přístup ke kameře a mikrofonu"};var CameraSitePermissionsRequest$k={primaryText:"Povolit aplikaci {appName} používat vaši kameru",secondaryText:"Toto je proto, aby vás účastníci viděli.",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží",primaryButtonText:"Pokračovat bez kamery",ariaLabel:"Povolit přístup ke kameře"};var MicrophoneSitePermissionsRequest$k={primaryText:"Povolit aplikaci {appName} používat váš mikrofon",secondaryText:"To je proto, aby vás účastníci mohli slyšet.",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží",primaryButtonText:"Pokračovat bez mikrofonu",ariaLabel:"Povolte přístup k mikrofonu"};var CameraAndMicrophoneSitePermissionsCheck$k={primaryText:"Kontrola přístupu ke kameře a mikrofonu",secondaryText:"Pokud se zobrazí výzva, povolte přístup. To je proto, aby vás účastníci slyšeli a viděli.",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží",primaryButtonText:"Pokračovat bez kamery a mikrofonu",ariaLabel:"Kontroluje se přístup k mikrofonu a kameře. Povolit přístup, pokud se zobrazí výzva."};var CameraSitePermissionsCheck$k={primaryText:"Kontroluje se přístup ke kameře",secondaryText:"Pokud se zobrazí výzva, povolte přístup. To je proto, aby vás účastníci viděli.",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží",primaryButtonText:"Pokračovat bez kamery",ariaLabel:"Kontroluje se přístup ke kameře. Povolit přístup, pokud se zobrazí výzva."};var MicrophoneSitePermissionsCheck$k={primaryText:"Kontroluje se přístup k mikrofonu",secondaryText:"Pokud se zobrazí výzva, povolte přístup. To je proto, aby vás účastníci slyšeli.",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží",primaryButtonText:"Pokračovat bez mikrofonu",ariaLabel:"Kontroluje se přístup k mikrofonu. Povolit přístup, pokud se zobrazí výzva."};var CameraAndMicrophoneSitePermissionsDenied$k={primaryText:"Nejde získat přístup ke kameře a mikrofonu",secondaryText:"Kliknutím na ikonu zámku na panelu Adresa udělte mikrofonu oprávnění k této webové stránce. Může být vyžadována aktualizace stránky.",primaryButtonText:"Pokračovat bez kamery a mikrofonu",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží"};var CameraAndMicrophoneSitePermissionsDeniedSafari$k={primaryText:"Nejde získat přístup ke kameře a mikrofonu.",secondaryText:"Aktualizujte stránku, abyste povolili oprávnění, nebo zkontrolujte nastavení tohoto prohlížeče a ověřte, jestli jsou pro tento web povolená oprávnění.",primaryButtonText:"Pokračovat bez kamery a mikrofonu",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží"};var CameraSitePermissionsDenied$k={primaryText:"Nejde získat přístup ke kameře",secondaryText:"Kliknutím na ikonu zámku na panelu Adresa udělte kameře oprávnění k této webové stránce. Může být vyžadována aktualizace stránky.",primaryButtonText:"Pokračovat bez kamery",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží"};var MicrophoneSitePermissionsDenied$k={primaryText:"Nepovedlo se získat přístup k mikrofonu",secondaryText:"Kliknutím na ikonu zámku na panelu Adresa udělte mikrofonu oprávnění k této webové stránce. Může být vyžadována aktualizace stránky.",primaryButtonText:"Pokračovat bez mikrofonu",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží"};var CameraSitePermissionsDeniedSafari$k={primaryText:"Nejde získat přístup ke kameře",secondaryText:"Aktualizujte stránku, abyste povolili oprávnění, nebo zkontrolujte nastavení tohoto prohlížeče a ověřte, jestli jsou pro tento web povolená oprávnění.",primaryButtonText:"Pokračovat bez kamery",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží"};var MicrophoneSitePermissionsDeniedSafari$k={primaryText:"Nepovedlo se získat přístup k mikrofonu",secondaryText:"Aktualizujte stránku, abyste povolili oprávnění, nebo zkontrolujte nastavení tohoto prohlížeče a ověřte, jestli jsou pro tento web povolená oprávnění.",primaryButtonText:"Pokračovat bez mikrofonu",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží"};var UnsupportedBrowser$k={primaryText:"Prohlížeč není podporovaný",secondaryText:"Připojte se k tomuto hovoru pomocí kompatibilního prohlížeče.",moreHelpLinkText:"Zobrazit požadavky na kompatibilitu"};var UnsupportedBrowserVersion$k={primaryText:"Je nutné aktualizovat prohlížeč",secondaryText:"Abyste zajistili nejlepší možný hovor, aktualizujte si prosím prohlížeč a zkuste se k hovoru připojit znovu.",moreHelpLinkText:"Zobrazit požadavky na kompatibilitu",continueAnywayButtonText:"Zahájit hovor bez aktualizace"};var UnsupportedOperatingSystem$k={primaryText:"Nepodporovaný operační systém",secondaryText:"Připojte se k tomuto hovoru pomocí zařízení s kompatibilním operačním systémem.",moreHelpLinkText:"Zobrazit požadavky na kompatibilitu"};var BrowserPermissionDenied$k={primaryText:"Kameru nebo mikrofon nejde použít",secondaryText:"Váš prohlížeč možná nemá přístup ke kameře nebo mikrofonu. Tento problém vyřešíte tak, že otevřete Předvolby systému.",primaryButtonText:"Zkusit znovu",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží"};var BrowserPermissionDeniedIOS$k={primaryText:"Povolit přístup k mikrofonu, aby bylo možné pokračovat",secondaryText:"Aby vás ostatní účastníci slyšeli.",primaryButtonText:"Zkusit znovu",imageAltText:"Umístění oprávnění mikrofonu a kamery pro iOS",linkText:"Potřebujete pomoc? Získat nápovědu k řešení potíží",step1Text:"Přejít do aplikace Nastavení",step2Text:"Přesuňte se dolů k nastavení tohoto prohlížeče",step3Text:"Zapnout mikrofon (kamera je volitelná)",step4Text:"Zkuste se připojit k hovoru znovu",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$k={leftNavButtonAriaLabel:"předchozí stránka {current} z(e) {total}",rightNavButtonAriaLabel:"další stránka {current} z(e) {total}"};var meetingConferencePhoneInfo$k={meetingConferencePhoneInfoModalTitle:"Připojit se k hovoru z vašeho telefonu",meetingConferencePhoneInfoModalDialIn:"Vytočit číslo",meetingConferencePhoneInfoModalMeetingId:"Zadejte ID schůzky",meetingConferencePhoneInfoModalWait:"Počkejte na přijetí organizátorem",meetingConferencePhoneInfoModalNoPhoneAvailable:"Není k dispozici žádný telefon.",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Bezplatné)",meetingConferencePhoneInfoModalToll:"(Placené)"};var IncomingCallNotification$k={incomingCallNotificationPlaceholderId:"Neznámý uživatel",incomingCallNotificationPlaceholderAlert:"Volá {callerName}",incomingCallNoticicationAcceptWithAudioAriaLabel:"Přijmout se zvukem",incomingCallNoticicationAcceptWithVideoAriaLabel:"Přijmout s videem",incomingCallNoticicationRejectAriaLabel:"Odmítnout",incomingCallNotificationAcceptButtonLabel:"Přijmout",incomingCallNotificationRejectButtonLabel:"Odmítnout",incomingCallNotificationAccceptWithVideoButtonLabel:"Přijmout s videem",incomingCallNotificationDismissButtonAriaLabel:"Zavřít"};var cs_CZ$1 = {participantItem:participantItem$k,ParticipantList:ParticipantList$k,typingIndicator:typingIndicator$k,sendBox:sendBox$k,richTextSendBox:richTextSendBox$k,realTimeTextModal:realTimeTextModal$k,captionsBanner:captionsBanner$k,mentionPopover:mentionPopover$k,imageOverlay:imageOverlay$k,messageStatusIndicator:messageStatusIndicator$k,endCallButton:endCallButton$k,cameraButton:cameraButton$k,microphoneButton:microphoneButton$k,devicesButton:devicesButton$k,participantsButton:participantsButton$k,screenShareButton:screenShareButton$k,raiseHandButton:raiseHandButton$k,reactionButton:reactionButton$k,realTimeText:realTimeText$k,captionsSettingsModal:captionsSettingsModal$k,spokenLanguages:spokenLanguages$k,captionLanguages:captionLanguages$k,startCaptionsButton:startCaptionsButton$k,startRealTimeTextButton:startRealTimeTextButton$k,messageThread:messageThread$k,errorBar:errorBar$k,notificationStack:notificationStack$k,videoGallery:videoGallery$k,dialpad:dialpad$k,holdButton:holdButton$k,videoTile:videoTile$k,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$k,CameraSitePermissionsRequest:CameraSitePermissionsRequest$k,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$k,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$k,CameraSitePermissionsCheck:CameraSitePermissionsCheck$k,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$k,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$k,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$k,CameraSitePermissionsDenied:CameraSitePermissionsDenied$k,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$k,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$k,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$k,UnsupportedBrowser:UnsupportedBrowser$k,UnsupportedBrowserVersion:UnsupportedBrowserVersion$k,UnsupportedOperatingSystem:UnsupportedOperatingSystem$k,BrowserPermissionDenied:BrowserPermissionDenied$k,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$k,verticalGallery:verticalGallery$k,meetingConferencePhoneInfo:meetingConferencePhoneInfo$k,IncomingCallNotification:IncomingCallNotification$k};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Czech (Czech Republic).
 *
 * @public
 */
const COMPONENT_LOCALE_CS_CZ = {
    strings: createComponentStrings(cs_CZ$1)
};

var participantItem$j={isMeText:"(chi)",menuTitle:"Mwy o Ddewisiadau",removeButtonLabel:"Tynnu",sharingIconLabel:"Rhannu",mutedIconLabel:"Wedi tewi",handRaisedIconLabel:"Llaw wedi'i chodi - Lleoliad {position}",displayNamePlaceholder:"Cyfranogwr dienw",participantStateRinging:"Wrthi'n galw...",participantStateHold:"Galwad yn aros",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, Mwy o Ddewisiadau",attendeeRole:"Mynychwr",micDisabledIconLabel:"Meicroffon wedi'i analluogi",cameraDisabledIconLabel:"Camera wedi'i analluogi"};var ParticipantList$j={overflowParticipantCount:"+{overflowCount} arall"};var typingIndicator$j={singleUser:"Mae {user} yn teipio ...",multipleUsers:"Mae {users} yn teipio ...",multipleUsersAbbreviateOne:"Mae {users} ac 1 arall yn teipio ...",multipleUsersAbbreviateMany:"Mae {users} a {numOthers} arall yn teipio ...",delimiter:", "};var sendBox$j={placeholderText:"Rhowch neges",textTooLong:"Mae hyd eich neges yn fwy na'r terfyn uchaf.",sendButtonAriaLabel:"Anfon neges",attachmentUploadsPendingError:"Wrthi'n llwytho'r ffeil i fyny, arhoswch funud.",removeAttachment:"Tynnu’r ffeil",uploading:"Wrthi'n llwytho i fyny",uploadCompleted:"Wedi gorffen llwytho i fyny",attachmentMoreMenu:"Mwy o Ddewisiadau"};var richTextSendBox$j={placeholderText:"Rhowch neges",textTooLong:"Mae hyd eich neges yn fwy na'r terfyn uchaf.",sendButtonAriaLabel:"Anfon neges",attachmentUploadsPendingError:"Wrthi'n lanlwytho'r ffeil, arhoswch funud.",imageUploadsPendingError:"Wrthi'n lanlwytho'r ddelwedd, arhoswch funud.",removeAttachment:"Tynnu'r ffeil",uploading:"Wrthi'n lanlwytho",uploadCompleted:"Wedi gorffen lanlwytho",richTextBoldTooltip:"Trwm",richTextItalicTooltip:"Italig",richTextUnderlineTooltip:"Tanlinellu",richTextBulletListTooltip:"Rhestr bwledi",richTextNumberListTooltip:"Rhestr wedi’i rhifo",richTextIncreaseIndentTooltip:"Cynyddu'r mewnoliad",richTextDecreaseIndentTooltip:"Lleihau'r mewnoliad",richTextInsertTableTooltip:"Mewnosod tabl",richTextFormatButtonTooltip:"Fformat",richTextInsertRowOrColumnMenu:"Mewnosod",richTextInsertTableMenuTitle:"Mewnosod tabl {row} x {column}",richTextInsertRowAboveMenu:"Mewnosod uwchben",richTextInsertRowBelowMenu:"Mewnosod isod",richTextInsertColumnLeftMenu:"Mewnosod i'r chwith",richTextInsertColumnRightMenu:"Mewnosod i'r dde",richTextDeleteRowOrColumnMenu:"Dileu",richTextDeleteColumnMenu:"Dileu'r golofn",richTextDeleteRowMenu:"Dileu'r rhes",richTextDeleteTableMenu:"Dileu'r tabl",richTextToolbarAriaLabel:"Bar Offer Fformatio Negeseuon",richTextToolbarMoreButtonAriaLabel:"Mwy o ddewisiadau",attachmentMoreMenu:"Mwy o Ddewisiadau",richTextNewBulletedListItemAnnouncement:"Bwled, llinell newydd",richTextNewNumberedListItemAnnouncement:"{0}, llinell newydd",richTextBulletedListAppliedAnnouncement:"Wedi defnyddio'r rhestr bwledi",richTextNumberedListAppliedAnnouncement:"Wedi defnyddio rhestr wedi'i rhifo"};var realTimeTextModal$j={realTimeTextModalAriaLabel:"Rhoi Moddol RTT ar waith",realTimeTextModalTitle:"Rhoi RTT ar waith?",realTimeTextModalText:"Bydd teipio mewn amser real (RTT) yn cael ei alluogi ar gyfer pawb yn y cyfarfod.",realTimeTextConfirmButtonLabel:"Rhoi ar waith",realTimeTextCancelButtonLabel:"Canslo",realTimeTextCloseModalButtonAriaLabel:"Cau'r Moddol RTT"};var captionsBanner$j={captionsBannerSpinnerText:"Wrthi'n dechrau'r capsiynau...",realTimeTextInputBoxDefaultText:"Teipiwch neges mewn amser go iawn",realTimeTextInputErrorMessage:"All y neges ddim bod yn fwy na 2000 o nodau",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"Mae RTT (testun amser real) wedi'i alluogi ar gyfer yr holl gyfranogwyr ar gyfer cyfnod cyfan y cyfarfod.",realTimeTextBannerLinkLabel:"Dysgu mwy",captionsOnlyContainerTitle:"Capsiynau",realTimeTextOnlyContainerTitle:"Testun Amser Real",captionsAndRealTimeTextContainerTitle:"Capsiynau a Thestun Amser Real",expandButtonAriaLabel:"Ehangu",minimizeButtonAriaLabel:"Crebachu"};var mentionPopover$j={mentionPopoverHeader:"Awgrymiadau"};var imageOverlay$j={downloadButtonLabel:"Lawrlwytho",dismissButtonAriaLabel:"Cau",overlayVisibleMessage:"Troshaen Manylion Delwedd"};var messageStatusIndicator$j={deliveredAriaLabel:"Wedi anfon y neges",deliveredTooltipText:"Anfonwyd",seenAriaLabel:"Neges wedi ei gweld gan bobl eraill",seenTooltipText:"Gwelwyd",readByTooltipText:"Wedi'i darllen gan {messageThreadReadCount} o {remoteParticipantsCount}",sendingAriaLabel:"Wrthi'n anfon y neges",sendingTooltipText:"Wrthi'n anfon",failedToSendAriaLabel:"Wedi methu anfon y neges",failedToSendTooltipText:"Wedi methu anfon"};var endCallButton$j={label:"Gadael",tooltipContent:"Gadael yr alwad",leaveOption:"Gadael",endCallOption:"Dod â'r alwad i ben ar gyfer pawb"};var cameraButton$j={onLabel:"Camera",offLabel:"Camera",tooltipDisabledContent:"Mae’r camera wedi'i analluogi",tooltipOnContent:"Diffodd y camera",tooltipOffContent:"Rhoi'r camera ar waith",tooltipVideoLoadingContent:"Wrthi'n llwytho'r fideo",cameraMenuTitle:"Camera",cameraMenuTooltip:"Dewis camera",cameraButtonSplitRoleDescription:"Botwm Hollti",onSplitButtonAriaLabel:"Diffodd y camera",offSplitButtonAriaLabel:"Rhoi'r camera ar waith",cameraActionTurnedOnAnnouncement:"Mae eich camera wedi'i roi ar waith",cameraActionTurnedOffAnnouncement:"Mae eich camera wedi'i ddiffodd",onSplitButtonPrimaryActionCamera:"Diffodd y camera",offSplitButtonPrimaryActionCamera:"Rhoi'r camera ar waith",cameraPrimaryActionSplitButtonTitle:"Defnyddio camera",videoEffectsMenuItemTitle:"Effeithiau",cameraButtonAriaDescription:"dewisiadau camera"};var microphoneButton$j={onLabel:"Meicroffon",offLabel:"Meicroffon",tooltipDisabledContent:"Mae'r meicroffon wedi'i analluogi",tooltipOnContent:"Tewi’r meicroffon",tooltipOffContent:"Dad-dewi’r meicroffon",microphoneMenuTitle:"Meicroffon",microphoneMenuTooltip:"Dewis meicroffon",speakerMenuTitle:"Seinydd",speakerMenuTooltip:"Dewis seinydd",microphoneButtonSplitRoleDescription:"Botwm Hollti",onSplitButtonAriaLabel:"Tewi'r meic",offSplitButtonAriaLabel:"Dad-dewi'r meic",microphoneActionTurnedOnAnnouncement:"Mae eich meicroffon wedi'i roi ar waith",microphoneActionTurnedOffAnnouncement:"Mae eich meicroffon wedi'i ddiffodd",offSplitButtonMicrophonePrimaryAction:"Dad-dewi’r meic",onSplitButtonMicrophonePrimaryAction:"Tewi'r meic",microphonePrimaryActionSplitButtonTitle:"Defnyddio'r meicroffon",microphoneAriaDescription:"Dewisiadau sain",deepNoiseSuppressionTitle:"Atal sŵn",deepNoiseSuppressionOnAnnouncement:"Mae atal sŵn dwfn wedi'i roi ar waith",deepNoiseSuppressionOffAnnouncement:"Mae atal sŵn dwfn wedi'i ddiffodd"};var devicesButton$j={label:"Dyfeisiau",tooltipContent:"Rheoli dyfeisiau",cameraMenuTitle:"Camera",cameraMenuTooltip:"Dewis camera",audioDeviceMenuTitle:"Dyfais Sain",audioDeviceMenuTooltip:"Dewis dyfais sain",microphoneMenuTitle:"Meicroffon",microphoneMenuTooltip:"Dewis meicroffon",speakerMenuTitle:"Siaradwr",speakerMenuTooltip:"Dewis seinydd"};var participantsButton$j={label:"Pobl",ariaLabel:"Pobl",tooltipContent:"Dangos cyfranogwyr",menuHeader:"Yn yr alwad hon",participantsListButtonLabel:"{numParticipants} o bobl",muteAllButtonLabel:"Tewi’r cyfan",copyInviteLinkButtonLabel:"Copïo dolen y gwahoddiad",copyInviteLinkActionedAriaLabel:"Wedi copïo dolen y gwahoddiad"};var screenShareButton$j={onLabel:"Stopio cyflwyno",offLabel:"Presennol",tooltipDisabledContent:"Mae cyflwyno wedi'i analluogi",tooltipOnContent:"Wrthi'n cyflwyno eich sgrin",tooltipOffContent:"Cyflwyno eich sgrin"};var raiseHandButton$j={offLabel:"Codi",onLabel:"Gostwng",tooltipDisabledContent:"Mae'r weithred Codi Llaw wedi'i hanalluogi",tooltipOnContent:"Gostwng Llaw",tooltipOffContent:"Codi Llaw"};var reactionButton$j={label:"Ymateb",ariaLabel:"Ymateb",tooltipDisabledContent:"Mae'r weithred ymateb wedi'i hanalluogi",tooltipContent:"Anfon ymateb",likeReactionTooltipContent:"Hoffi",heartReactionTooltipContent:"Caru",laughReactionTooltipContent:"Chwerthin",applauseReactionTooltipContent:"Curo Dwylo",surprisedReactionTooltipContent:"Wedi synnu"};var realTimeText$j={isTypingText:"Teipio RTT"};var captionsSettingsModal$j={captionsSettingsModalTitle:"Gosodiadau Capsiynau",captionsSettingsSpokenLanguageDropdownLabel:"Iaith lafar",captionsSettingsCaptionLanguageDropdownLabel:"Iaith capsiynau",captionsSettingsSpokenLanguageDropdownInfoText:"Yr iaith y mae pawb yn yr alwad hon yn ei siarad.",captionsSettingsCaptionLanguageDropdownInfoText:"Bydd capsiynau'n ymddangos yn yr iaith hon.",captionsSettingsConfirmButtonLabel:"Cadarnhau",captionsSettingsCancelButtonLabel:"Canslo",captionsSettingsModalAriaLabel:"Moddol Gosodiad Capsiynau",captionsSettingsCloseModalButtonAriaLabel:"Cau Gosodiad Capsiynau"};var spokenLanguages$j={"ar-ae":"Arabeg - E.A.U.","ar-sa":"Arabeg - Saudi Arabia","da-dk":"Daneg","de-de":"Almaeneg - Yr Almaen","en-au":"Saesneg - Awstralia","en-ca":"Saesneg - Canada","en-gb":"Saesneg - Y Deyrnas Unedig","en-in":"Saesneg - India","en-nz":"Saesneg - Seland Newydd","en-us":"Saesneg - Unol Daleithiau America","es-es":"Sbaeneg - Sbaen (Modern)","es-mx":"Sbaeneg - Mecsico","fi-fi":"Ffinneg","fr-ca":"Ffrangeg - Canada","fr-fr":"Ffrangeg - Ffrainc","hi-in":"Hindi","it-it":"Eidaleg - Yr Eidal","ja-jp":"Japaneeg","ko-kr":"Coreeg","nb-no":"Norwyeg (Bokmål)","nl-be":"Iseldireg - Gwlad Belg","nl-nl":"Iseldireg - Yr Iseldiroedd","pl-pl":"Pwyleg","pt-br":"Portiwgaleg - Brasil","ru-ru":"Rwseg","sv-se":"Swedeg","zh-cn":"Tsieinëeg - Gweriniaeth y Bobl Tsieina","zh-hk":"Tsieinëeg - Hong Kong SAR","cs-cz":"Tsieceg","pt-pt":"Portiwgaleg - Portiwgal","tr-tr":"Tyrceg","vi-vn":"Fietnameg","th-th":"Thai","he-il":"Hebraeg","cy-gb":"Cymraeg","uk-ua":"Wcreineg","el-gr":"Groeg","hu-hu":"Hwngareg","ro-ro":"Rwmaneg","sk-sk":"Slofaceg","zh-tw":"Tsieinëeg - Taiwan"};var captionLanguages$j={ar:"Arabeg",da:"Daneg",de:"Almaeneg",en:"Saesneg",es:"Sbaeneg",fi:"Ffinneg","fr-ca":"Ffrangeg - Canada",fr:"Ffrangeg - Ffrainc",hi:"Hindi",it:"Eidaleg",ja:"Japaneeg",ko:"Coreeg",nb:"Norwyeg (Bokmål)",nl:"Iseldireg",pl:"Pwyleg",pt:"Portiwgaleg - Brasil",ru:"Rwseg",sv:"Swedeg","zh-Hans":"Tsieinëeg (Syml)","zh-Hant":"Tsieinëeg (Traddodiadol)",cs:"Tsieceg","pt-pt":"Portiwgaleg - Portiwgal",tr:"Tyrceg",vi:"Fietnameg",th:"Thai",he:"Hebraeg",cy:"Cymraeg",uk:"Wcreineg",el:"Groeg",hu:"Hwngareg",ro:"Rwmaneg",sk:"Slofaceg"};var startCaptionsButton$j={onLabel:"Galluogi capsiynau",offLabel:"Analluogi capsiynau",tooltipOnContent:"Analluogi capsiynau",tooltipOffContent:"Galluogi capsiynau"};var startRealTimeTextButton$j={label:"Rhoi RTT ar waith ar gyfer yr alwad hon",tooltipOffContent:"Rhoi RTT ar waith ar gyfer yr alwad hon"};var messageThread$j={yesterday:"Ddoe",sunday:"Dydd Sul",monday:"Dydd Llun",tuesday:"Dydd Mawrth",wednesday:"Dydd Mercher",thursday:"Dydd Iau",friday:"Dydd Gwener",saturday:"Dydd Sadwrn",participantJoined:"wedi ymuno â'r sgwrs.",participantLeft:"wedi gadael y sgwrs.",editMessage:"Golygu",removeMessage:"Dileu",resendMessage:"Rhowch gynnig arall ar anfon",failToSendTag:"Wedi methu â'i hanfon",editedTag:"Golygwyd",liveAuthorIntro:"Neges gan {author}",editedMessageLiveAuthorIntro:"Neges wedi'i golygu gan {author}",editedMessageLocalUserLiveAuthorIntro:"Neges wedi’i golygu",messageContentAriaText:"{status} Dywedodd {author} {message}",messageContentMineAriaText:"{status} Dywedoch chi {message}",editBoxTextLimit:"Mae eich neges dros y terfyn o {limitNumber} nodau",editBoxPlaceholderText:"Golygu eich neges",newMessagesIndicator:"Negeseuon newydd",noDisplayNameSub:"Dim enw",editBoxCancelButton:"Canslo",editBoxSubmitButton:"Wedi gorffen",messageReadCount:"Wedi'i darllen gan {messageReadByCount} o {remoteParticipantsCount}",actionMenuMoreOptions:"Mwy o Ddewisiadau",downloadAttachment:"Llwytho'r ffeil i lawr",openAttachment:"Agor ffeil",blockedWarningText:"Mae'r neges hon wedi'i dileu oherwydd polisi sefydliadol.",blockedWarningLinkText:"Manylion",attachmentCardGroupMessage:"Mae gan y neges {attachmentCount} atodiad",messageDeletedAnnouncementAriaLabel:"Mae'r neges wedi cael ei dileu",richTextBoldTooltip:"Trwm",richTextItalicTooltip:"Italig",richTextUnderlineTooltip:"Tanlinellu",richTextBulletListTooltip:"Rhestr bwledi",richTextNumberListTooltip:"Rhestr wedi'i rhifo",richTextIncreaseIndentTooltip:"Cynyddu’r mewnoliad",richTextDecreaseIndentTooltip:"Lleihau’r mewnoliad",richTextInsertTableTooltip:"Mewnosod tabl",richTextFormatButtonTooltip:"Fformat",richTextInsertRowOrColumnMenu:"Mewnosod",richTextInsertTableMenuTitle:"Mewnosod tabl {row} x {column}",richTextInsertRowAboveMenu:"Mewnosod uwchben",richTextInsertRowBelowMenu:"Mewnosod isod",richTextInsertColumnLeftMenu:"Mewnosod i'r chwith",richTextInsertColumnRightMenu:"Mewnosod i'r dde",richTextDeleteRowOrColumnMenu:"Dileu",richTextDeleteColumnMenu:"Dileu'r golofn",richTextDeleteRowMenu:"Dileu'r rhes",richTextDeleteTableMenu:"Dileu'r tabl",richTextToolbarMoreButtonAriaLabel:"Mwy o ddewisiadau",attachmentUploadsPendingError:"Wrthi'n lanlwytho'r ffeil, arhoswch funud.",imageUploadsPendingError:"Wrthi'n lanlwytho'r ddelwedd, arhoswch funud.",richTextNewBulletedListItemAnnouncement:"Bwled, llinell newydd",richTextNewNumberedListItemAnnouncement:"{0}, llinell newydd",richTextBulletedListAppliedAnnouncement:"Wedi defnyddio'r rhestr bwledi",richTextNumberedListAppliedAnnouncement:"Wedi defnyddio rhestr wedi'i rhifo"};var errorBar$j={unableToReachChatService:"Rydych chi all-lein",accessDenied:"Does dim modd cael gafael ar wasanaethau sgwrsio - gwiriwch fanylion adnabod y defnyddiwr a roddwyd",userNotInChatThread:"Nid ydych yn yr edefyn sgwrsio hwn mwyach",sendMessageNotInChatThread:"Wedi methu anfon neges oherwydd dydych chi ddim yn y trywydd sgwrsio hwn mwyach",sendMessageGeneric:"Wedi methu anfon neges",callingNetworkFailure:"problem wrth gysylltu galwad - mae’n ymddangos eich bod all-lein",startVideoGeneric:"Wedi methu cychwyn y fideo",stopVideoGeneric:"Wedi methu stopio'r fideo",muteGeneric:"Wedi methu tewi'r meicroffon",unmuteGeneric:"Wedi methu dad-dewi'r meicroffon",speakingWhileMuted:"Mae eich meicroffon wedi tewi",startScreenShareGeneric:"Roedd problem wrth ddechrau rhannu sgrin.",stopScreenShareGeneric:"Wedi methu rhoi'r gorau i rannu sgrin",callNetworkQualityLow:"Mae ansawdd y rhwydwaith yn isel.",teamsMeetingCallNetworkQualityLow:"Mae ansawdd y rhwydwaith yn isel.",callNoSpeakerFound:"Heb ddod o hyd i seinyddion na clustffonau. Cysylltu dyfais sain i glywed yr alwad.",callNoMicrophoneFound:"Heb ganfod meicroffonau. Cysylltu dyfais mewnbwn sain.",callMicrophoneAccessDenied:"Does dim modd cael mynediad at y meicroffon. Cliciwch y clo yn y bar cyfeiriad i roi caniatâd i'r dudalen we hon.",callMicrophoneAccessDeniedSafari:"Nid oes modd cael mynediad at y meicroffon. Adnewyddwch y dudalen i ganiatáu hawliau, neu gwiriwch osodiadau'r porwr hwn a dilysu hawliau wedi'u galluogi ar gyfer y wefan hon.",callMicrophoneMutedBySystem:"Rydych chi wedi'ch tewi gan eich system.",callMicrophoneUnmutedBySystem:"Mae eich meicroffon wedi'i adfer a chawsoch chi eich dad-dewi gan eich system.",callMacOsMicrophoneAccessDenied:"Does dim modd cael mynediad at y meicroffon. Rhoi caniatâd meicroffon yn eich gosodiadau preifatrwydd macOS.",callLocalVideoFreeze:"Mae lled band y rhwydwaith yn wael. Efallai y bydd eich fideo wedi'i oedi ar gyfer eraill ar yr alwad.",callCameraAccessDenied:"Does dim modd cael mynediad at y camera. Cliciwch y clo yn y bar cyfeiriad i roi caniatâd i'r dudalen we hon.",callCameraAccessDeniedSafari:"Nid oes modd cael mynediad at y camera. Adnewyddwch y dudalen i ganiatáu hawliau, neu gwiriwch osodiadau'r porwr hwn a dilysu hawliau wedi'u galluogi ar gyfer y wefan hon.",callCameraAlreadyInUse:"Nid oes modd cael mynediad at y camera. Mae'n bosib bod rhaglen arall yn ei ddefnyddio'n barod.",callVideoStoppedBySystem:"Mae eich fideo wedi cael ei stopio gan eich system.",callVideoRecoveredBySystem:"Mae eich fideo wedi ailddechrau.",callMacOsCameraAccessDenied:"Mae MacOS yn rhwystro mynediad at eich camera. Diweddarwch eich gosodiadau preifatrwydd i ganiatáu i'r porwr hwn gael mynediad at eich camera.",callMacOsScreenShareAccessDenied:"Mae MacOS yn rhwystro rhannu sgrin. Diweddarwch eich gosodiadau preifatrwydd i ganiatáu i'r porwr hwn recordio eich sgrin.",dismissButtonAriaLabel:"Cau",failedToJoinCallGeneric:"Wedi methu ag ymuno â'r alwad.",failedToJoinCallInvalidMeetingLink:"Does dim modd ymuno â'r Cyfarfod. Dolen Annilys.",cameraFrozenForRemoteParticipants:"Mae defnyddwyr yn yr alwad yn cael trafferth gweld eich fideo. Tarwch olwg ar eich dyfeisiau a'ch rhwydwaith.",unableToStartVideoEffect:"Nid oes modd defnyddio'r effaith fideo.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Nid oes modd dechrau sbotolau ar gyfranogwr/cyfranogwyr oherwydd bod y nifer fwyaf o gyfranogwyr a ganiateir eisoes wedi'u hamlygu.",mutedByRemoteParticipant:"Rydych chi wedi cael eich tewi gan gyfranogwr arall."};var notificationStack$j={callingNetworkFailure:{title:"Mae problem wrth gysylltu'r alwad",message:"Mae’n ymddangos eich bod chi all-lein. Gwnewch yn siŵr eich bod chi wedi cysylltu â'r rhwydwaith a rhoi cynnig arall arni.",dismissButtonAriaLabel:"Cau"},startVideoGeneric:{title:"Wedi methu â dechrau'r fideo.",dismissButtonAriaLabel:"Cau"},stopVideoGeneric:{title:"Wedi methu â stopio'r fideo.",dismissButtonAriaLabel:"Cau"},muteGeneric:{title:"Wedi methu â thewi'r meicroffon.",dismissButtonAriaLabel:"Cau"},unmuteGeneric:{title:"Wedi methu â dad-dewi'r meicroffon.",dismissButtonAriaLabel:"Cau"},speakingWhileMuted:{title:"Rydych chi wedi'ch tewi.",dismissButtonAriaLabel:"Cau"},startScreenShareGeneric:{title:"Roedd problem wrth ddechrau rhannu sgrin.",dismissButtonAriaLabel:"Cau"},stopScreenShareGeneric:{title:"Wedi methu â stopio rhannu sgrin.",dismissButtonAriaLabel:"Cau"},callNetworkQualityLow:{title:"Mae ansawdd y rhwydwaith yn isel.",dismissButtonAriaLabel:"Cau"},teamsMeetingCallNetworkQualityLow:{title:"Mae ansawdd y rhwydwaith yn isel.",dismissButtonAriaLabel:"Cau",message:"Ymuno â'r alwad hon o'ch ffôn i gael sain gwell. Gallwch barhau i weld y cyfarfod ar y ddyfais hon.",primaryButtonLabel:"Ymuno dros y Ffôn"},callNoSpeakerFound:{title:"Heb ganfod seinyddion na chlustffonau",message:"Cysylltu dyfais sain i glywed yr alwad.",dismissButtonAriaLabel:"Cau"},callNoMicrophoneFound:{title:"Heb ganfod meicroffonau",message:"Cysylltwch ddyfais mewnbynnu sain.",dismissButtonAriaLabel:"Cau"},callMicrophoneAccessDenied:{title:"Does dim modd cael mynediad i'r meicroffon",message:"Cliciwch y clo yn y bar cyfeiriad i roi caniatâd i'r dudalen we hon.",dismissButtonAriaLabel:"Cau"},callMicrophoneAccessDeniedSafari:{title:"Does dim modd cael mynediad i'r meicroffon",message:"Adnewyddwch y dudalen i ganiatáu caniatâd, neu gwiriwch osodiadau'r porwr hwn a dilysu bod y caniatâd wedi'u galluogi ar gyfer y wefan hon.",dismissButtonAriaLabel:"Cau"},callMicrophoneMutedBySystem:{title:"Rydych chi wedi cael eich tewi gan eich system.",dismissButtonAriaLabel:"Cau"},callMicrophoneUnmutedBySystem:{title:"Mae eich meicroffon wedi'i adfer a chawsoch chi eich dad-dewi gan eich system.",dismissButtonAriaLabel:"Cau"},callMacOsMicrophoneAccessDenied:{title:"Does dim modd cael mynediad i'r meicroffon",message:"Rhoi caniatâd meicroffon yn eich gosodiadau preifatrwydd macOS.",dismissButtonAriaLabel:"Cau"},callLocalVideoFreeze:{title:"Mae lled band y rhwydwaith yn wael",message:"Efallai y bydd eich fideo wedi'i oedi ar gyfer eraill ar yr alwad.",dismissButtonAriaLabel:"Cau"},callCameraAccessDenied:{title:"Does dim modd cael mynediad i'r camera",message:"Cliciwch y clo yn y bar cyfeiriad i roi caniatâd i'r dudalen we hon.",dismissButtonAriaLabel:"Cau"},callCameraAccessDeniedSafari:{title:"Does dim modd cael mynediad i'r camera",message:"Adnewyddwch y dudalen i ganiatáu caniatâd, neu gwiriwch osodiadau'r porwr hwn a dilysu bod y caniatâd wedi'u galluogi ar gyfer y wefan hon.",dismissButtonAriaLabel:"Cau"},callCameraAlreadyInUse:{title:"Does dim modd cael mynediad i'r camera",message:"Mae'n bosib bod rhaglen arall yn ei ddefnyddio'n barod.",dismissButtonAriaLabel:"Cau"},callVideoStoppedBySystem:{title:"Mae eich fideo wedi cael ei stopio gan eich system.",dismissButtonAriaLabel:"Cau"},callVideoRecoveredBySystem:{title:"Mae eich fideo wedi ailddechrau.",dismissButtonAriaLabel:"Cau"},callMacOsCameraAccessDenied:{title:"Mae MacOS yn rhwystro mynediad i'ch camera",message:"Diweddarwch eich gosodiadau preifatrwydd i ganiatáu i'r porwr hwn gael mynediad at eich camera.",dismissButtonAriaLabel:"Cau"},callMacOsScreenShareAccessDenied:{title:"Mae MacOS yn rhwystro rhannu sgrin",message:"Diweddarwch eich gosodiadau preifatrwydd i ganiatáu i'r porwr hwn recordio eich sgrin.",dismissButtonAriaLabel:"Cau"},failedToJoinCallGeneric:{title:"Wedi methu ag ymuno â'r alwad.",dismissButtonAriaLabel:"Cau"},failedToJoinCallInvalidMeetingLink:{title:"Does dim modd ymuno â'r Cyfarfod. Dolen Annilys.",dismissButtonAriaLabel:"Cau"},cameraFrozenForRemoteParticipants:{title:"Mae defnyddwyr yn yr alwad yn cael trafferth gweld eich fideo",message:"Gwiriwch eich dyfeisiau a'ch rhwydwaith.",dismissButtonAriaLabel:"Cau"},unableToStartVideoEffect:{title:"Does dim modd defnyddio'r effaith fideo.",dismissButtonAriaLabel:"Cau"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Nid oes modd dechrau'r sbotolau ar y cyfranogwr/cyfranogwyr",message:"Mae'r sbotolau eisoes ar y nifer fwyaf o gyfranogwyr.",dismissButtonAriaLabel:"Cau"},mutedByRemoteParticipant:{title:"Rydych chi wedi cael eich tewi gan gyfranogwr arall.",dismissButtonAriaLabel:"Cau"},recordingStarted:{title:"Wedi dechrau recordio.",message:"Drwy ymuno, rydych yn cydsynio i drawsgrifio'r cyfarfod hwn.",dismissButtonAriaLabel:"Cau"},transcriptionStarted:{title:"Wedi dechrau trawsgrifio.",message:"Drwy ymuno, rydych yn cydsynio i drawsgrifio'r cyfarfod hwn.",dismissButtonAriaLabel:"Cau"},recordingStopped:{title:"Mae recordiad yn cael ei gadw.",message:"Wedi stopio recordio.",dismissButtonAriaLabel:"Cau"},transcriptionStopped:{title:"Mae trawsgrifiad yn cael ei gadw.",message:"Wedi stopio trawsgrifio.",dismissButtonAriaLabel:"Cau"},recordingAndTranscriptionStarted:{title:"Wedi dechrau recordio a thrawsgrifio.",message:"Drwy ymuno, rydych yn cydsynio i drawsgrifio'r cyfarfod hwn.",dismissButtonAriaLabel:"Cau"},recordingAndTranscriptionStopped:{title:"Mae recordiad a thrawsgrifiad yn cael eu cadw.",message:"Wedi stopio recordio a thrawsgrifio.",dismissButtonAriaLabel:"Cau"},recordingStoppedStillTranscribing:{title:"Wedi stopio recordio.",message:"Rydych yn awr yn trawsgrifio'r cyfarfod hwn yn unig.",dismissButtonAriaLabel:"Cau"},transcriptionStoppedStillRecording:{title:"Wedi stopio trawsgrifio.",message:"Rydych chi dim ond yn recordio'r cyfarfod hwn bellach.",dismissButtonAriaLabel:"Cau"},assignedBreakoutRoomOpened:{title:"Byddwn yn eich symud i'ch ystafell wedi'i neilltuo mewn 5 eiliad.",dismissButtonAriaLabel:"Cau"},assignedBreakoutRoomChanged:{title:"Mae ystafell eich cyfarfod ymylol wedi newid",message:"Byddwn yn eich symud i'ch ystafell wedi'i neilltuo mewn 5 eiliad.",dismissButtonAriaLabel:"Cau"},assignedBreakoutRoomClosed:{title:"Mae eich cyfarfod ymylol wedi cau",message:"Byddwn yn eich symud yn ôl i'ch cyfarfod mewn 5 eiliad.",dismissButtonAriaLabel:"Cau"},assignedBreakoutRoomOpenedPromptJoin:{title:"Ymuno â chyfarfod ymylol?",message:"Rydych chi wedi cael eich neilltuo i gyfarfod ymylol.",dismissButtonAriaLabel:"Cau",primaryButtonLabel:"Ymuno â’r ystafell",secondaryButtonLabel:"Yn nes ymlaen"},breakoutRoomJoined:{title:"Rydych chi wedi ymuno â chyfarfod ymylol.",dismissButtonAriaLabel:"Cau"},breakoutRoomClosingSoon:{title:"Mae terfyn amser yr ystafell ar fin dod i ben.",message:"Bydd yr ystafell hon yn cau mewn 30 eiliad",dismissButtonAriaLabel:"Cau"},capabilityTurnVideoOnPresent:{title:"Mae eich camera wedi cael ei alluogi",message:"Rhowch eich camera ar waith os hoffech chi rannu fideo",dismissButtonAriaLabel:"Cau"},capabilityTurnVideoOnAbsent:{title:"Mae eich camera wedi cael ei analluogi",message:"Does dim modd i chi rannu fideo bellach",dismissButtonAriaLabel:"Cau"},capabilityUnmuteMicPresent:{title:"Mae eich meic wedi cael ei alluogi",message:"Dad-dewch y meicroffon os hoffech chi siarad.",dismissButtonAriaLabel:"Cau"},capabilityUnmuteMicAbsent:{title:"Mae eich meic wedi cael ei analluogi",message:"Does dim modd i chi ddad-dewi bellach.",dismissButtonAriaLabel:"Cau"},togetherModeStarted:{title:"Mae Togethermode wedi dechrau",dismissButtonAriaLabel:"Cau"},togetherModeEnded:{title:"Mae Togethermode wedi dod i ben",dismissButtonAriaLabel:"Cau"},transcriptionError:{title:"Aeth rhywbeth o’i le gyda’r trawsgrifiad.",message:"Wedi stopio trawsgrifio. rhowch gynnig arall arni rywbryd eto",dismissButtonAriaLabel:"Cau"},transcriptionStartedByYou:{title:"Wedi dechrau trawsgrifio.",message:"Dechreuwyd gennych chi. Rhowch wybod i bawb eu bod yn cael eu trawsgrifio.",dismissButtonAriaLabel:"Cau"}};var videoGallery$j={screenIsBeingSharedMessage:"Rydych chi'n rhannu eich sgrin",screenShareLoadingMessage:"Wrthi'n llwytho sgrin {participant}",localScreenShareLoadingMessage:"Wrthi'n llwytho eich sgrin",localVideoLabel:"Chi",localVideoCameraSwitcherLabel:"Newid camera",localVideoMovementLabel:"Teilsen Fideo Lleol Symudol",localVideoMovementAriaLabel:"Teilsen Fideo Lleol Symudol",localVideoSelectedDescription:"Wedi dewis {cameraName}",displayNamePlaceholder:"Cyfranogwr dienw",fitRemoteParticipantToFrame:"Ffitio i'r ffrâm",fillRemoteParticipantFrame:"Llenwi'r ffrâm",pinParticipantForMe:"Pinio ar fy nghyfer i",pinParticipantForMeLimitReached:"Pinio (wedi cyrraedd y terfyn)",unpinParticipantForMe:"Dadbinio",pinParticipantMenuItemAriaLabel:"Pinio {participantName}",unpinParticipantMenuItemAriaLabel:"Dadbinio {participantName}",pinnedParticipantAnnouncementAriaLabel:"Wedi pinio {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Wedi dadbinio {participantName}",startSpotlightVideoTileMenuLabel:"Sbotolau i bawb",addSpotlightVideoTileMenuLabel:"Ychwanegu sbotolau",spotlightLimitReachedMenuTitle:"Wedi cyrraedd y terfyn sbotolau",stopSpotlightVideoTileMenuLabel:"Diffodd y sbotolau",stopSpotlightOnSelfVideoTileMenuLabel:"Gadael y sbotolau",attendeeRole:"Mynychwr",muteParticipantMenuItemLabel:"Tewi",waitingScreenText:"Yn aros i bobl eraill ymuno",forbidAudio:"Analluogi meic",permitAudio:"Caniatáu meic",forbidAudioTileMenuLabel:"Analluogi meic",permitAudioTileMenuLabel:"Caniatáu meic",forbidVideo:"Analluogi camera",permitVideo:"Caniatáu camera",forbidVideoTileMenuLabel:"Analluogi camera",permitVideoTileMenuLabel:"Caniatáu camera"};var dialpad$j={placeholderText:"Rhowch rif ffôn",deleteButtonAriaLabel:"Dileu"};var holdButton$j={onLabel:"Parhau",offLabel:"Rhoi i aros",tooltipOnContent:"Parhau â'r alwad",tooltipOffContent:"Rhoi'r alwad i Aros"};var videoTile$j={participantStateRinging:"Wrthi'n galw...",participantReconnecting:"Wrthi'n ailgysylltu...",participantStateHold:"Galwad yn aros",moreOptionsButtonAriaLabel:"Mwy o Ddewisiadau {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"wedi tewi",moreOptionsParticipantMutedStateUnmutedAriaLabel:"wedi dad-dewi",moreOptionsParticipantHandRaisedAriaLabel:"Llaw wedi’i chodi",moreOptionsParticipantIsSpeakingAriaLabel:"Siarad",moreOptionsParticipantMicDisabledAriaLabel:"Meicroffon wedi'i analluogi",moreOptionsParticipantCameraDisabledAriaLabel:"Camera wedi'i analluogi"};var CameraAndMicrophoneSitePermissionsRequest$j={primaryText:"Caniatáu i {appName} ddefnyddio eich camera a'ch meicroffon",secondaryText:"Mae hyn er mwyn i gyfranogwyr allu eich gweld a'ch clywed chi.",linkText:"Angen cymorth? Cael cymorth i ddatrys problemau",primaryButtonText:"Bwrw ymlaen heb y camera a'r meicroffon",ariaLabel:"Caniatáu mynediad i'r camera a'r meicroffon"};var CameraSitePermissionsRequest$j={primaryText:"Caniatáu i {appName} ddefnyddio eich camera",secondaryText:"Mae hyn er mwyn i gyfranogwyr allu eich gweld chi.",linkText:"Angen help? Cael cymorth i ddatrys problemau",primaryButtonText:"Bwrw ymlaen heb y camera",ariaLabel:"Caniatáu mynediad i'r camera"};var MicrophoneSitePermissionsRequest$j={primaryText:"Caniatáu i {appName} ddefnyddio eich meicroffon",secondaryText:"Mae hyn er mwyn i gyfranogwyr allu eich clywed chi.",linkText:"Angen cymorth? Cael cymorth i ddatrys problemau",primaryButtonText:"Bwrw ymlaen heb feicroffon",ariaLabel:"Caniatáu mynediad i'r meicroffon"};var CameraAndMicrophoneSitePermissionsCheck$j={primaryText:"Wrthi'n chwilio am fynediad i'r camera a'r meicroffon",secondaryText:"Caniatáu mynediad os gofynnir i chi wneud hynny. Mae hyn er mwyn i gyfranogwyr allu eich gweld a'ch clywed chi.",linkText:"Angen help? Cael cymorth i ddatrys problemau",primaryButtonText:"Bwrw ymlaen heb y camera a'r meicroffon",ariaLabel:"Wrthi'n chwilio am fynediad i'r camera a'r meicroffon. Caniatáu mynediad os gofynnir i chi wneud hynny."};var CameraSitePermissionsCheck$j={primaryText:"Wrthi'n chwilio am fynediad i'r camera",secondaryText:"Caniatáu mynediad os gofynnir i chi wneud hynny. Mae hyn er mwyn i gyfranogwyr eich gweld.",linkText:"Angen cymorth? Cael cymorth i ddatrys problemau",primaryButtonText:"Bwrw ymlaen heb y camera",ariaLabel:"Wrthi'n chwilio am fynediad i'r camera. Caniatáu mynediad os gofynnir i chi wneud hynny."};var MicrophoneSitePermissionsCheck$j={primaryText:"Wrthi'n chwilio am fynediad i’r meicroffon",secondaryText:"Caniatáu mynediad os gofynnir i chi wneud hynny. Mae hyn fel y gall cyfranogwyr eich clywed.",linkText:"Angen cymorth? Cael cymorth i ddatrys problemau",primaryButtonText:"Bwrw ymlaen heb feicroffon",ariaLabel:"Wrthi'n chwilio am fynediad i'r meicroffon. Caniatáu mynediad os gofynnir i chi wneud hynny."};var CameraAndMicrophoneSitePermissionsDenied$j={primaryText:"Nid oes modd cael mynediad at y camera a'r meicroffon",secondaryText:"Cliciwch yr eicon clo yn y bar cyfeiriad i roi hawliau i'r meicroffon i'r dudalen we hon. Efallai bydd angen adnewyddu tudalen.",primaryButtonText:"Bwrw ymlaen heb y camera a'r meicroffon",linkText:"Angen help? Cael cymorth i ddatrys problemau"};var CameraAndMicrophoneSitePermissionsDeniedSafari$j={primaryText:"Does dim modd cael mynediad i'r camera a'r meicroffon",secondaryText:"Adnewyddwch y dudalen i ganiatáu hawliau, neu gwiriwch osodiadau'r porwr hwn a dilysu hawliau wedi'u galluogi ar gyfer y wefan hon.",primaryButtonText:"Bwrw ymlaen heb y camera a'r meicroffon",linkText:"Angen cymorth? Cael cymorth i ddatrys problemau"};var CameraSitePermissionsDenied$j={primaryText:"Nid oes modd cael mynediad at y camera",secondaryText:"Cliciwch ar yr eicon clo yn y bar cyfeiriad i roi caniatâd camera i'r dudalen we hon. Efallai y bydd angen adnewyddu'r dudalen.",primaryButtonText:"Bwrw ymlaen heb y camera",linkText:"Angen help? Cael cymorth i ddatrys problemau"};var MicrophoneSitePermissionsDenied$j={primaryText:"Does dim modd cael mynediad i'r meicroffon",secondaryText:"Cliciwch yr eicon clo yn y bar cyfeiriad i roi hawliau i'r meicroffon i'r dudalen we hon. Efallai bydd angen adnewyddu tudalen.",primaryButtonText:"Bwrw ymlaen heb feicroffon",linkText:"Angen help? Cael cymorth i ddatrys problemau"};var CameraSitePermissionsDeniedSafari$j={primaryText:"Does dim modd cael mynediad i'r camera",secondaryText:"Adnewyddwch y dudalen i ganiatáu hawliau, neu gwiriwch osodiadau'r porwr hwn a dilysu hawliau wedi'u galluogi ar gyfer y wefan hon.",primaryButtonText:"Bwrw ymlaen heb y camera",linkText:"Angen help? Cael cymorth i ddatrys problemau"};var MicrophoneSitePermissionsDeniedSafari$j={primaryText:"Nid oes modd cael mynediad at y meicroffon",secondaryText:"Adnewyddwch y dudalen i ganiatáu hawliau, neu gwiriwch osodiadau'r porwr hwn a dilysu hawliau wedi'u galluogi ar gyfer y wefan hon.",primaryButtonText:"Bwrw ymlaen heb feicroffon",linkText:"Angen cymorth? Cael cymorth i ddatrys problemau"};var UnsupportedBrowser$j={primaryText:"Does dim modd delio â'r porwr",secondaryText:"Ymunwch â'r alwad hon gan ddefnyddio porwr cydnaws.",moreHelpLinkText:"Gweld gofynion cydnawsedd"};var UnsupportedBrowserVersion$j={primaryText:"Mae angen diweddaru'r porwr",secondaryText:"Er mwyn sicrhau’r alwad orau posibl, diweddarwch eich porwr a rhoi cynnig arall ar ymuno â'r alwad.",moreHelpLinkText:"Gweld gofynion cydnawsedd",continueAnywayButtonText:"Dechrau galwad heb ddiweddaru"};var UnsupportedOperatingSystem$j={primaryText:"Ni chefnogir system gweithredu",secondaryText:"Ymunwch â'r alwad hon gan ddefnyddio dyfais sydd â system weithredu gydnaws.",moreHelpLinkText:"Gweld gofynion cydnawsedd"};var BrowserPermissionDenied$j={primaryText:"Does dim modd defnyddio eich camera neu feicroffon",secondaryText:"Efallai nad oes gan eich porwr fynediad i'ch camera neu'ch meicroffon. I drwsio hyn, agorwch Dewisiadau'r System.",primaryButtonText:"Rhowch gynnig arall arni",linkText:"Angen help? Cael cymorth i ddatrys problemau"};var BrowserPermissionDeniedIOS$j={primaryText:"Caniatewch fynediad i'r meicroffon i barhau",secondaryText:"Er mwyn i gyfranogwyr allu eich clywed chi.",primaryButtonText:"Rhowch gynnig arall arni",imageAltText:"Lleoliad caniatâd meicroffon a dyfais camera ar gyfer iOS",linkText:"Angen help? Cael cymorth i ddatrys problemau",step1Text:"Ewch i'r ap Gosodiadau.",step2Text:"Sgroliwch i lawr i'r gosodiadau ar gyfer y porwr hwn",step3Text:"Rhowch y Meicroffon ar waith (Camera'n ddewisol)",step4Text:"Rhowch gynnig arall ar ymuno â'r alwad",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$j={leftNavButtonAriaLabel:"tudalen flaenorol {current} o {total}",rightNavButtonAriaLabel:"tudalen nesaf {current} o {total}"};var meetingConferencePhoneInfo$j={meetingConferencePhoneInfoModalTitle:"Ymuno â galwad o'ch ffôn",meetingConferencePhoneInfoModalDialIn:"Deialu rhif",meetingConferencePhoneInfoModalMeetingId:"Rhowch ID y cyfarfod",meetingConferencePhoneInfoModalWait:"Aros i'r trefnydd roi mynediad",meetingConferencePhoneInfoModalNoPhoneAvailable:"Dim ffôn ar gael",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Di-doll)",meetingConferencePhoneInfoModalToll:"(Toll)"};var IncomingCallNotification$j={incomingCallNotificationPlaceholderId:"Defnyddiwr Anhysbys",incomingCallNotificationPlaceholderAlert:"{callerName} yn galw",incomingCallNoticicationAcceptWithAudioAriaLabel:"Derbyn gyda sain",incomingCallNoticicationAcceptWithVideoAriaLabel:"Derbyn gyda fideo",incomingCallNoticicationRejectAriaLabel:"Gwrthod",incomingCallNotificationAcceptButtonLabel:"Derbyn",incomingCallNotificationRejectButtonLabel:"Gwrthod",incomingCallNotificationAccceptWithVideoButtonLabel:"Derbyn gyda Fideo",incomingCallNotificationDismissButtonAriaLabel:"Diystyru"};var cy_GB$1 = {participantItem:participantItem$j,ParticipantList:ParticipantList$j,typingIndicator:typingIndicator$j,sendBox:sendBox$j,richTextSendBox:richTextSendBox$j,realTimeTextModal:realTimeTextModal$j,captionsBanner:captionsBanner$j,mentionPopover:mentionPopover$j,imageOverlay:imageOverlay$j,messageStatusIndicator:messageStatusIndicator$j,endCallButton:endCallButton$j,cameraButton:cameraButton$j,microphoneButton:microphoneButton$j,devicesButton:devicesButton$j,participantsButton:participantsButton$j,screenShareButton:screenShareButton$j,raiseHandButton:raiseHandButton$j,reactionButton:reactionButton$j,realTimeText:realTimeText$j,captionsSettingsModal:captionsSettingsModal$j,spokenLanguages:spokenLanguages$j,captionLanguages:captionLanguages$j,startCaptionsButton:startCaptionsButton$j,startRealTimeTextButton:startRealTimeTextButton$j,messageThread:messageThread$j,errorBar:errorBar$j,notificationStack:notificationStack$j,videoGallery:videoGallery$j,dialpad:dialpad$j,holdButton:holdButton$j,videoTile:videoTile$j,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$j,CameraSitePermissionsRequest:CameraSitePermissionsRequest$j,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$j,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$j,CameraSitePermissionsCheck:CameraSitePermissionsCheck$j,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$j,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$j,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$j,CameraSitePermissionsDenied:CameraSitePermissionsDenied$j,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$j,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$j,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$j,UnsupportedBrowser:UnsupportedBrowser$j,UnsupportedBrowserVersion:UnsupportedBrowserVersion$j,UnsupportedOperatingSystem:UnsupportedOperatingSystem$j,BrowserPermissionDenied:BrowserPermissionDenied$j,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$j,verticalGallery:verticalGallery$j,meetingConferencePhoneInfo:meetingConferencePhoneInfo$j,IncomingCallNotification:IncomingCallNotification$j};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Welsh (GB).
 *
 * @public
 */
const COMPONENT_LOCALE_CY_GB = {
    strings: createComponentStrings(cy_GB$1)
};

var participantItem$i={isMeText:"(Sie)",menuTitle:"Weitere Optionen",removeButtonLabel:"Entfernen",sharingIconLabel:"Teilen",mutedIconLabel:"Stumm geschaltet",handRaisedIconLabel:"Hand erhoben – Position {position}",displayNamePlaceholder:"Teilnehmer ohne Namen",participantStateRinging:"Anrufen...",participantStateHold:"In der Warteschleife",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, weitere Optionen",attendeeRole:"Teilnehmer",micDisabledIconLabel:"Micophone deaktiviert",cameraDisabledIconLabel:"Kamera deaktiviert"};var ParticipantList$i={overflowParticipantCount:"+{overflowCount} weitere"};var typingIndicator$i={singleUser:"{user} schreibt ...",multipleUsers:"{users} schreiben ...",multipleUsersAbbreviateOne:"{users} und 1 andere Person schreiben ...",multipleUsersAbbreviateMany:"{users} und {numOthers} andere schreiben ...",delimiter:", "};var sendBox$i={placeholderText:"Nachricht eingeben",textTooLong:"Ihre Nachrichtenlänge überschreitet den maximalen Grenzwert.",sendButtonAriaLabel:"Nachricht senden",attachmentUploadsPendingError:"Datei wird hochgeladen, bitte warten.",removeAttachment:"Datei entfernen",uploading:"Hochladen",uploadCompleted:"Upload abgeschlossen",attachmentMoreMenu:"Mehr Optionen"};var richTextSendBox$i={placeholderText:"Nachricht eingeben",textTooLong:"Ihre Nachrichtenlänge überschreitet den maximalen Grenzwert.",sendButtonAriaLabel:"Nachricht senden",attachmentUploadsPendingError:"Datei wird hochgeladen, bitte warten.",imageUploadsPendingError:"Bild wird hochgeladen, bitte warten.",removeAttachment:"Datei entfernen",uploading:"Wird hochgeladen",uploadCompleted:"Upload abgeschlossen",richTextBoldTooltip:"Fett",richTextItalicTooltip:"Kursiv",richTextUnderlineTooltip:"Unterstreichen",richTextBulletListTooltip:"Aufzählungsliste",richTextNumberListTooltip:"Nummerierte Liste",richTextIncreaseIndentTooltip:"Einzug vergrößern",richTextDecreaseIndentTooltip:"Einzug verringern",richTextInsertTableTooltip:"Tabelle einfügen",richTextFormatButtonTooltip:"Formatieren",richTextInsertRowOrColumnMenu:"Einfügen",richTextInsertTableMenuTitle:"{column} x {row} Tabelle einfügen",richTextInsertRowAboveMenu:"Oben einfügen",richTextInsertRowBelowMenu:"Darunter einfügen",richTextInsertColumnLeftMenu:"Links einfügen",richTextInsertColumnRightMenu:"Rechts einfügen",richTextDeleteRowOrColumnMenu:"Löschen",richTextDeleteColumnMenu:"Spalte löschen",richTextDeleteRowMenu:"Zeile löschen",richTextDeleteTableMenu:"Tabelle löschen",richTextToolbarAriaLabel:"Symbolleiste für die Nachrichtenformatierung",richTextToolbarMoreButtonAriaLabel:"Weitere Optionen",attachmentMoreMenu:"Mehr Optionen",richTextNewBulletedListItemAnnouncement:"Aufzählungszeichen, neue Zeile",richTextNewNumberedListItemAnnouncement:"{0}, neue Zeile",richTextBulletedListAppliedAnnouncement:"Aufzählung angewendet",richTextNumberedListAppliedAnnouncement:"\"Nummerierte Liste\" angewendet."};var realTimeTextModal$i={realTimeTextModalAriaLabel:"RTT Modal aktivieren",realTimeTextModalTitle:"RTT aktivieren?",realTimeTextModalText:"Echtzeittext (RTT) wird für alle Teilnehmer der Besprechung aktiviert.",realTimeTextConfirmButtonLabel:"Aktivieren",realTimeTextCancelButtonLabel:"Abbrechen",realTimeTextCloseModalButtonAriaLabel:"RTT Modal schließen"};var captionsBanner$i={captionsBannerSpinnerText:"Untertitel werden gestartet...",realTimeTextInputBoxDefaultText:"Nachricht in Echtzeit eingeben",realTimeTextInputErrorMessage:"Die Nachricht darf 2000 Zeichen nicht überschreiten.",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (Echtzeittext) ist für alle Teilnehmer für die gesamte Dauer der Besprechung aktiviert.",realTimeTextBannerLinkLabel:"Weitere Informationen",captionsOnlyContainerTitle:"Untertitel",realTimeTextOnlyContainerTitle:"Echtzeittext",captionsAndRealTimeTextContainerTitle:"Untertitel und Echtzeittext",expandButtonAriaLabel:"Erweitern",minimizeButtonAriaLabel:"Reduzieren"};var mentionPopover$i={mentionPopoverHeader:"Vorschläge"};var imageOverlay$i={downloadButtonLabel:"Herunterladen",dismissButtonAriaLabel:"Schließen",overlayVisibleMessage:"Bilddetailsüberlagerung"};var messageStatusIndicator$i={deliveredAriaLabel:"Nachricht gesendet",deliveredTooltipText:"Gesendet",seenAriaLabel:"Nachricht von anderen gesehen",seenTooltipText:"Gesehen",readByTooltipText:"Gelesen von {messageThreadReadCount} von {remoteParticipantsCount}",sendingAriaLabel:"Nachricht wird gesendet",sendingTooltipText:"Wird gesendet",failedToSendAriaLabel:"Nachricht konnte nicht gesendet werden",failedToSendTooltipText:"Fehler beim Senden"};var endCallButton$i={label:"Verlassen",tooltipContent:"Anruf verlassen",leaveOption:"Beenden",endCallOption:"Anruf für alle beenden"};var cameraButton$i={onLabel:"Kamera",offLabel:"Kamera",tooltipDisabledContent:"Kamera ist deaktiviert",tooltipOnContent:"Kamera ausschalten",tooltipOffContent:"Kamera einschalten",tooltipVideoLoadingContent:"Video wird geladen",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Kamera auswählen",cameraButtonSplitRoleDescription:"Unterteilte Schaltfläche",onSplitButtonAriaLabel:"Kamera ausschalten",offSplitButtonAriaLabel:"Kamera einschalten",cameraActionTurnedOnAnnouncement:"Ihre Kamera wurde eingeschaltet.",cameraActionTurnedOffAnnouncement:"Ihre Kamera wurde ausgeschaltet.",onSplitButtonPrimaryActionCamera:"Kamera ausschalten",offSplitButtonPrimaryActionCamera:"Kamera einschalten",cameraPrimaryActionSplitButtonTitle:"Kamera verwenden",videoEffectsMenuItemTitle:"Effekte",cameraButtonAriaDescription:"Kameraoptionen"};var microphoneButton$i={onLabel:"Mikrofon",offLabel:"Mikrofon",tooltipDisabledContent:"Mikrofon ist deaktiviert",tooltipOnContent:"Mikrofon stummschalten",tooltipOffContent:"Mikrofonstummschaltung aufheben",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Mikrofon auswählen",speakerMenuTitle:"Lautsprecher",speakerMenuTooltip:"Lautsprecher auswählen",microphoneButtonSplitRoleDescription:"Unterteilte Schaltfläche",onSplitButtonAriaLabel:"Mikrofon stummschalten",offSplitButtonAriaLabel:"Stummschaltung des Mikrofons aufheben",microphoneActionTurnedOnAnnouncement:"Ihr Mikrofon wurde eingeschaltet.",microphoneActionTurnedOffAnnouncement:"Ihr Mikrofon wurde ausgeschaltet.",offSplitButtonMicrophonePrimaryAction:"Stummschaltung des Mikrofons aufheben",onSplitButtonMicrophonePrimaryAction:"Mikrofon stummschalten",microphonePrimaryActionSplitButtonTitle:"Mikrofon verwenden",microphoneAriaDescription:"Audiooptionen",deepNoiseSuppressionTitle:"Rauschunterdrückung",deepNoiseSuppressionOnAnnouncement:"Die Unterdrückung tiefer Geräusche wurde aktiviert.",deepNoiseSuppressionOffAnnouncement:"Die Unterdrückung tiefer Geräusche wurde deaktiviert."};var devicesButton$i={label:"Geräte",tooltipContent:"Geräte verwalten",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Kamera auswählen",audioDeviceMenuTitle:"Audiogerät",audioDeviceMenuTooltip:"Audiogerät auswählen",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Mikrofon auswählen",speakerMenuTitle:"Lautsprecher",speakerMenuTooltip:"Lautsprecher auswählen"};var participantsButton$i={label:"Personen",ariaLabel:"Personen",tooltipContent:"Teilnehmende anzeigen",menuHeader:"In diesem Anruf",participantsListButtonLabel:"{numParticipants} Personen",muteAllButtonLabel:"Alle stumm schalten",copyInviteLinkButtonLabel:"Einladungslink kopieren",copyInviteLinkActionedAriaLabel:"Einladungslink kopiert"};var screenShareButton$i={onLabel:"Präsentation beenden",offLabel:"Präsentieren",tooltipDisabledContent:"Präsentieren ist deaktiviert",tooltipOnContent:"Ihr Bildschirm wird präsentiert",tooltipOffContent:"Präsentieren Sie Ihren Bildschirm"};var raiseHandButton$i={offLabel:"Heben",onLabel:"Senken",tooltipDisabledContent:"Aktion „Hand heben“ ist deaktiviert",tooltipOnContent:"Hand senken",tooltipOffContent:"Hand heben"};var reactionButton$i={label:"Reagieren",ariaLabel:"Reagieren",tooltipDisabledContent:"Reaktionsaktion ist deaktiviert",tooltipContent:"Reaktion senden",likeReactionTooltipContent:"Gefällt mir",heartReactionTooltipContent:"Liebe",laughReactionTooltipContent:"Lachen",applauseReactionTooltipContent:"Applaus",surprisedReactionTooltipContent:"Überrascht"};var realTimeText$i={isTypingText:"RTT-Eingabe"};var captionsSettingsModal$i={captionsSettingsModalTitle:"Einstellungen für Untertitel",captionsSettingsSpokenLanguageDropdownLabel:"Gesprochene Sprache",captionsSettingsCaptionLanguageDropdownLabel:"Sprache der Untertitel",captionsSettingsSpokenLanguageDropdownInfoText:"Sprache, die jeder in diesem Anruf spricht.",captionsSettingsCaptionLanguageDropdownInfoText:"Untertitel werden in dieser Sprache angezeigt.",captionsSettingsConfirmButtonLabel:"Bestätigen",captionsSettingsCancelButtonLabel:"Abbrechen",captionsSettingsModalAriaLabel:"Untertiteleinstellung modal",captionsSettingsCloseModalButtonAriaLabel:"Einstellung „Untertitel“ schließen"};var spokenLanguages$i={"ar-ae":"Arabisch – Vereinigte Arabische Emirate","ar-sa":"Arabisch – Saudi-Arabien","da-dk":"Dänisch","de-de":"Deutsch – Deutschland","en-au":"Englisch – Australien","en-ca":"Englisch – Kanada","en-gb":"Englisch – Vereinigtes Königreich","en-in":"Englisch – Indien","en-nz":"Englisch – Neuseeland","en-us":"Englisch – USA","es-es":"Spanisch – Spanien (Moderne Sortierung)","es-mx":"Spanisch – Mexiko","fi-fi":"Finnisch","fr-ca":"Französisch (Kanada)","fr-fr":"Französisch (Frankreich)","hi-in":"Hindi","it-it":"Italienisch – Italien","ja-jp":"Japanisch","ko-kr":"Koreanisch","nb-no":"Norwegisch (Bokmål)","nl-be":"Niederländisch – Belgien","nl-nl":"Niederländisch – Niederlande","pl-pl":"Polnisch","pt-br":"Portugiesisch (Brasilien)","ru-ru":"Russisch","sv-se":"Schwedisch","zh-cn":"Chinesisch – Volksrepublik China","zh-hk":"Chinesisch – Hongkong (SAR)","cs-cz":"Tschechisch","pt-pt":"Portugiesisch (Portugal)","tr-tr":"Türkisch","vi-vn":"Vietnamesisch","th-th":"Thailändisch","he-il":"Hebräisch","cy-gb":"Walisisch","uk-ua":"Ukrainisch","el-gr":"Griechisch","hu-hu":"Ungarisch","ro-ro":"Rumänisch","sk-sk":"Slowakisch","zh-tw":"Chinesisch – Taiwan"};var captionLanguages$i={ar:"Arabisch",da:"Dänisch",de:"Deutsch",en:"Englisch",es:"Spanisch",fi:"Finnisch","fr-ca":"Französisch (Kanada)",fr:"Französisch (Frankreich)",hi:"Hindi",it:"Italienisch",ja:"Japanisch",ko:"Koreanisch",nb:"Norwegisch (Bokmål)",nl:"Niederländisch",pl:"Polnisch",pt:"Portugiesisch (Brasilien)",ru:"Russisch",sv:"Schwedisch","zh-Hans":"Chinesisch (vereinfacht)","zh-Hant":"Chinesisch (traditionell)",cs:"Tschechisch","pt-pt":"Portugiesisch (Portugal)",tr:"Türkisch",vi:"Vietnamesisch",th:"Thailändisch",he:"Hebräisch",cy:"Walisisch",uk:"Ukrainisch",el:"Griechisch",hu:"Ungarisch",ro:"Rumänisch",sk:"Slowakisch"};var startCaptionsButton$i={onLabel:"Beschriftungen aktivieren",offLabel:"Untertitel deaktivieren",tooltipOnContent:"Untertitel deaktivieren",tooltipOffContent:"Beschriftungen aktivieren"};var startRealTimeTextButton$i={label:"RTT für diesen Anruf aktivieren",tooltipOffContent:"RTT für diesen Anruf aktivieren"};var messageThread$i={yesterday:"Gestern",sunday:"Sonntag",monday:"Montag",tuesday:"Dienstag",wednesday:"Mittwoch",thursday:"Donnerstag",friday:"Freitag",saturday:"Samstag",participantJoined:"ist dem Chat beigetreten.",participantLeft:"hat den Chat verlassen.",editMessage:"Bearbeiten",removeMessage:"Löschen",resendMessage:"Erneut senden",failToSendTag:"Fehler beim Senden",editedTag:"Bearbeitet",liveAuthorIntro:"Nachricht von {author}",editedMessageLiveAuthorIntro:"Nachricht von {author} bearbeitet",editedMessageLocalUserLiveAuthorIntro:"Nachricht bearbeitet",messageContentAriaText:"{status} {author} hat {message} gesagt",messageContentMineAriaText:"{status} Sie haben {message} gesagt",editBoxTextLimit:"Ihre Nachricht überschreitet das Limit von {limitNumber} Zeichen.",editBoxPlaceholderText:"Bearbeiten Sie Ihre Nachricht",newMessagesIndicator:"Neue Nachrichten",noDisplayNameSub:"Kein Name",editBoxCancelButton:"Abbrechen",editBoxSubmitButton:"Fertig",messageReadCount:"Gelesen von {messageReadByCount} von {remoteParticipantsCount}",actionMenuMoreOptions:"Weitere Optionen",downloadAttachment:"Datei herunterladen",openAttachment:"Datei öffnen",blockedWarningText:"Diese Nachricht wurde aufgrund einer Organisationsrichtlinie gelöscht.",blockedWarningLinkText:"Details",attachmentCardGroupMessage:"Die Nachricht enthält {attachmentCount} Anlage.",messageDeletedAnnouncementAriaLabel:"Die Nachricht wurde gelöscht",richTextBoldTooltip:"Fett",richTextItalicTooltip:"Kursiv",richTextUnderlineTooltip:"Unterstreichen",richTextBulletListTooltip:"Aufzählungsliste",richTextNumberListTooltip:"Nummerierte Liste",richTextIncreaseIndentTooltip:"Einzug vergrößern",richTextDecreaseIndentTooltip:"Einzug verringern",richTextInsertTableTooltip:"Tabelle einfügen",richTextFormatButtonTooltip:"Formatieren",richTextInsertRowOrColumnMenu:"Einfügen",richTextInsertTableMenuTitle:"{column} x {row} Tabelle einfügen",richTextInsertRowAboveMenu:"Oben einfügen",richTextInsertRowBelowMenu:"Unten einfügen",richTextInsertColumnLeftMenu:"Links einfügen",richTextInsertColumnRightMenu:"Rechts einfügen",richTextDeleteRowOrColumnMenu:"Löschen",richTextDeleteColumnMenu:"Spalte löschen",richTextDeleteRowMenu:"Zeile löschen",richTextDeleteTableMenu:"Tabelle löschen",richTextToolbarMoreButtonAriaLabel:"Weitere Optionen",attachmentUploadsPendingError:"Datei wird hochgeladen, bitte warten.",imageUploadsPendingError:"Bild wird hochgeladen, bitte warten.",richTextNewBulletedListItemAnnouncement:"Aufzählungszeichen, neue Zeile",richTextNewNumberedListItemAnnouncement:"{0}, neue Zeile",richTextBulletedListAppliedAnnouncement:"Aufzählung angewendet",richTextNumberedListAppliedAnnouncement:"\"Nummerierte Liste\" angewendet."};var errorBar$i={unableToReachChatService:"Sie sind offline.",accessDenied:"Auf die Chatdienste kann nicht zugegriffen werden. Überprüfen Sie die angegebenen Benutzeranmeldeinformationen.",userNotInChatThread:"Sie sind nicht mehr in diesem Chatthread vertreten.",sendMessageNotInChatThread:"Fehler beim Senden der Nachricht, weil Sie sich nicht mehr in diesem Chatthread befinden.",sendMessageGeneric:"Fehler beim Senden der Nachricht",callingNetworkFailure:"Problematische Anrufverbindung – Sie scheinen offline zu sein",startVideoGeneric:"Video konnte nicht gestartet werden",stopVideoGeneric:"Video konnte nicht beendet werden",muteGeneric:"Mikrofon konnte nicht stummgeschaltet werden",unmuteGeneric:"Stummschaltung des Mikrofons konnte nicht beendet werden",speakingWhileMuted:"Ihr Mikrofon ist stummgeschaltet",startScreenShareGeneric:"Beim Starten der Bildschirmfreigabe ist ein Problem aufgetreten.",stopScreenShareGeneric:"Bildschirmfreigabe konnte nicht beendet werden",callNetworkQualityLow:"Die Netzwerkqualität ist niedrig.",teamsMeetingCallNetworkQualityLow:"Die Netzwerkqualität ist niedrig.",callNoSpeakerFound:"Keine Lautsprecher oder Kopfhörer gefunden. Schließen Sie ein Audiogerät an, um den Anruf zu hören.",callNoMicrophoneFound:"Es wurden keine Mikrofone gefunden. Schließen Sie ein Audioeingabegerät an.",callMicrophoneAccessDenied:"Auf das Mikrofon kann nicht zugegriffen werden. Klicken Sie auf die Sperre in der Adressleiste, um dieser Webseite Berechtigungen zu erteilen.",callMicrophoneAccessDeniedSafari:"Auf das Mikrofon kann nicht zugegriffen werden. Aktualisieren Sie die Seite, um Berechtigungen zuzulassen, oder überprüfen Sie die Einstellungen dieses Browsers, und stellen Sie sicher, dass die Berechtigungen für diese Website aktiviert sind.",callMicrophoneMutedBySystem:"Sie werden von Ihrem System stummgeschaltet.",callMicrophoneUnmutedBySystem:"Ihr Mikrofon wurde wiederhergestellt, und Ihr System hat die Stummschaltung aufgehoben.",callMacOsMicrophoneAccessDenied:"Auf das Mikrofon kann nicht zugegriffen werden. Erteilen Sie die Mikrofonberechtigung in Ihren macOS-Datenschutzeinstellungen.",callLocalVideoFreeze:"Die Netzwerkbandbreite ist schlecht. Ihr Video wird möglicherweise für andere Personen während des Anrufs angehalten angezeigt.",callCameraAccessDenied:"Auf die Kamera kann nicht zugegriffen werden. Klicken Sie auf die Sperre in der Adressleiste, um dieser Webseite Berechtigungen zu erteilen.",callCameraAccessDeniedSafari:"Auf die Kamera kann nicht zugegriffen werden. Aktualisieren Sie die Seite, um Berechtigungen zuzulassen, oder überprüfen Sie die Einstellungen dieses Browsers, und stellen Sie sicher, dass die Berechtigungen für diese Website aktiviert sind.",callCameraAlreadyInUse:"Auf die Kamera kann nicht zugegriffen werden. Sie wird möglicherweise bereits von einer anderen Anwendung verwendet.",callVideoStoppedBySystem:"Ihr Video wurde von Ihrem System beendet.",callVideoRecoveredBySystem:"Ihr Video wurde fortgesetzt.",callMacOsCameraAccessDenied:"MacOS blockiert den Zugriff auf Ihre Kamera. Aktualisieren Sie Ihre Datenschutzeinstellungen, damit dieser Browser auf Ihre Kamera zugreifen kann.",callMacOsScreenShareAccessDenied:"MacOS blockiert die Bildschirmfreigabe. Aktualisieren Sie Ihre Datenschutzeinstellungen, damit dieser Browser Ihren Bildschirm aufzeichnen kann.",dismissButtonAriaLabel:"Schließen",failedToJoinCallGeneric:"Fehler bei der Teilnahme am Anruf.",failedToJoinCallInvalidMeetingLink:"Teilnahme an Besprechung nicht möglich. Ungültiger Link.",cameraFrozenForRemoteParticipants:"Benutzer im Anruf haben Probleme, Ihr Video zu sehen. Überprüfen Sie Ihre Geräte und Ihr Netzwerk.",unableToStartVideoEffect:"Videoeffekt kann nicht angewendet werden.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Das Spotlight für Teilnehmende kann nicht gestartet werden, da die maximale Anzahl von Teilnehmenden bereits im Spotlight ist.",mutedByRemoteParticipant:"Sie wurden von einem anderen Teilnehmer stummgeschaltet."};var notificationStack$i={callingNetworkFailure:{title:"Problem beim Verbindungsaufbau",message:"Sie scheinen offline zu sein. Überprüfen Sie Ihre Netzwerkverbindung, und versuchen Sie es noch mal.",dismissButtonAriaLabel:"Schließen"},startVideoGeneric:{title:"Fehler beim Starten des Videos.",dismissButtonAriaLabel:"Schließen"},stopVideoGeneric:{title:"Fehler beim Beenden des Videos.",dismissButtonAriaLabel:"Schließen"},muteGeneric:{title:"Fehler beim Stummschalten des Mikrofons.",dismissButtonAriaLabel:"Schließen"},unmuteGeneric:{title:"Fehler beim Aufheben der Stummschaltung des Mikrofons.",dismissButtonAriaLabel:"Schließen"},speakingWhileMuted:{title:"Sie sind stummgeschaltet.",dismissButtonAriaLabel:"Schließen"},startScreenShareGeneric:{title:"Beim Starten der Bildschirmfreigabe ist ein Problem aufgetreten.",dismissButtonAriaLabel:"Schließen"},stopScreenShareGeneric:{title:"Fehler beim Beenden der Bildschirmfreigabe.",dismissButtonAriaLabel:"Schließen"},callNetworkQualityLow:{title:"Die Netzwerkqualität ist niedrig.",dismissButtonAriaLabel:"Schließen"},teamsMeetingCallNetworkQualityLow:{title:"Die Netzwerkqualität ist niedrig.",dismissButtonAriaLabel:"Schließen",message:"Nehmen Sie von Ihrem Telefon aus an diesem Anruf teil, um den Sound zu verbessern. Sie können die Besprechung auf diesem Gerät weiterhin anzeigen.",primaryButtonLabel:"Per Telefon teilnehmen"},callNoSpeakerFound:{title:"Keine Lautsprecher oder Kopfhörer gefunden",message:"Schließen Sie ein Audiogerät an, um den Anruf zu hören.",dismissButtonAriaLabel:"Schließen"},callNoMicrophoneFound:{title:"Keine Mikrofone gefunden",message:"Schließen Sie ein Audioeingabegerät an.",dismissButtonAriaLabel:"Schließen"},callMicrophoneAccessDenied:{title:"Zugriff auf Mikrofon nicht möglich",message:"Klicken Sie auf die Sperre in der Adressleiste, um dieser Webseite Berechtigungen zu erteilen.",dismissButtonAriaLabel:"Schließen"},callMicrophoneAccessDeniedSafari:{title:"Zugriff auf Mikrofon nicht möglich",message:"Aktualisieren Sie die Seite, um Berechtigungen zuzulassen, oder überprüfen Sie die Einstellungen dieses Browsers und stellen Sie sicher, dass die Berechtigungen für diese Website aktiviert sind.",dismissButtonAriaLabel:"Schließen"},callMicrophoneMutedBySystem:{title:"Sie werden von Ihrem System stummgeschaltet.",dismissButtonAriaLabel:"Schließen"},callMicrophoneUnmutedBySystem:{title:"Ihr Mikrofon wurde wiederhergestellt, und Ihr System hat die Stummschaltung aufgehoben.",dismissButtonAriaLabel:"Schließen"},callMacOsMicrophoneAccessDenied:{title:"Zugriff auf Mikrofon nicht möglich",message:"Erteilen Sie die Mikrofonberechtigung in Ihren macOS-Datenschutzeinstellungen.",dismissButtonAriaLabel:"Schließen"},callLocalVideoFreeze:{title:"Netzwerkbandbreite ist gering.",message:"Ihr Video wird möglicherweise für andere Teilnehmer des Anrufs angehalten angezeigt.",dismissButtonAriaLabel:"Schließen"},callCameraAccessDenied:{title:"Zugriff auf Kamera nicht möglich",message:"Klicken Sie auf die Sperre in der Adressleiste, um dieser Webseite Berechtigungen zu erteilen.",dismissButtonAriaLabel:"Schließen"},callCameraAccessDeniedSafari:{title:"Zugriff auf Kamera nicht möglich",message:"Aktualisieren Sie die Seite, um Berechtigungen zuzulassen, oder überprüfen Sie die Einstellungen dieses Browsers und stellen Sie sicher, dass die Berechtigungen für diese Website aktiviert sind.",dismissButtonAriaLabel:"Schließen"},callCameraAlreadyInUse:{title:"Zugriff auf Kamera nicht möglich",message:"Sie wird möglicherweise bereits von einer anderen Anwendung verwendet.",dismissButtonAriaLabel:"Schließen"},callVideoStoppedBySystem:{title:"Ihr Video wurde von Ihrem System beendet.",dismissButtonAriaLabel:"Schließen"},callVideoRecoveredBySystem:{title:"Ihr Video wurde fortgesetzt.",dismissButtonAriaLabel:"Schließen"},callMacOsCameraAccessDenied:{title:"MacOS blockiert den Zugriff auf Ihre Kamera.",message:"Aktualisieren Sie Ihre Datenschutzeinstellungen, damit dieser Browser auf Ihre Kamera zugreifen kann.",dismissButtonAriaLabel:"Schließen"},callMacOsScreenShareAccessDenied:{title:"MacOS blockiert die Bildschirmfreigabe.",message:"Aktualisieren Sie Ihre Datenschutzeinstellungen, damit dieser Browser Ihren Bildschirm aufzeichnen kann.",dismissButtonAriaLabel:"Schließen"},failedToJoinCallGeneric:{title:"Fehler bei der Teilnahme am Anruf.",dismissButtonAriaLabel:"Schließen"},failedToJoinCallInvalidMeetingLink:{title:"Teilnahme an Besprechung nicht möglich. Ungültiger Link.",dismissButtonAriaLabel:"Schließen"},cameraFrozenForRemoteParticipants:{title:"Benutzer im Anruf haben Probleme beim Anzeigen Ihres Videos.",message:"Überprüfen Sie Ihre Geräte und Ihr Netzwerk.",dismissButtonAriaLabel:"Schließen"},unableToStartVideoEffect:{title:"Videoeffekt kann nicht angewendet werden.",dismissButtonAriaLabel:"Schließen"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Spotlight auf Teilnehmer kann nicht gestartet werden.",message:"Die maximale Anzahl von Teilnehmern wird bereits hervorgehoben.",dismissButtonAriaLabel:"Schließen"},mutedByRemoteParticipant:{title:"Sie wurden von einem anderen Teilnehmer stummgeschaltet.",dismissButtonAriaLabel:"Schließen"},recordingStarted:{title:"Die Aufzeichnung wurde gestartet.",message:"Durch Ihre Teilnahme stimmen Sie zu, dass diese Besprechung transkribiert wird.",dismissButtonAriaLabel:"Schließen"},transcriptionStarted:{title:"Die Transkription wurde gestartet.",message:"Durch Ihre Teilnahme stimmen Sie zu, dass diese Besprechung transkribiert wird.",dismissButtonAriaLabel:"Schließen"},recordingStopped:{title:"Aufzeichnung wird gespeichert.",message:"Aufzeichnung wurde beendet.",dismissButtonAriaLabel:"Schließen"},transcriptionStopped:{title:"Die Transkription wird gespeichert.",message:"Die Transkription wurde beendet.",dismissButtonAriaLabel:"Schließen"},recordingAndTranscriptionStarted:{title:"Die Aufzeichnung und Transkription haben begonnen.",message:"Durch Ihre Teilnahme stimmen Sie zu, dass diese Besprechung transkribiert wird.",dismissButtonAriaLabel:"Schließen"},recordingAndTranscriptionStopped:{title:"Aufzeichnung und Transkription werden gespeichert.",message:"Die Aufzeichnung und Transkription wurden beendet.",dismissButtonAriaLabel:"Schließen"},recordingStoppedStillTranscribing:{title:"Aufzeichnung wurde beendet.",message:"Sie transkribieren diese Besprechung jetzt nur noch.",dismissButtonAriaLabel:"Schließen"},transcriptionStoppedStillRecording:{title:"Die Transkription wurde beendet.",message:"Sie zeichnen nun nur diese Besprechung auf.",dismissButtonAriaLabel:"Schließen"},assignedBreakoutRoomOpened:{title:"Wir verschieben Sie in 5 Sekunden zu Ihrem zugewiesenen Raum.",dismissButtonAriaLabel:"Schließen"},assignedBreakoutRoomChanged:{title:"Ihr Gruppenraum hat sich geändert.",message:"Wir verschieben Sie in 5 Sekunden zu Ihrem zugewiesenen Raum.",dismissButtonAriaLabel:"Schließen"},assignedBreakoutRoomClosed:{title:"Ihr Gruppenraum wurde geschlossen.",message:"Wir bringen Sie in 5 Sekunden zurück zu Ihrer Besprechung.",dismissButtonAriaLabel:"Schließen"},assignedBreakoutRoomOpenedPromptJoin:{title:"Gruppenraum betreten?",message:"Sie wurden einem Gruppenraum zugewiesen.",dismissButtonAriaLabel:"Schließen",primaryButtonLabel:"Raum betreten",secondaryButtonLabel:"Später"},breakoutRoomJoined:{title:"Sie sind einem Gruppenraum beigetreten.",dismissButtonAriaLabel:"Schließen"},breakoutRoomClosingSoon:{title:"Das Raumzeitlimit läuft bald ab.",message:"Dieser Raum wird in 30 Sekunden geschlossen.",dismissButtonAriaLabel:"Schließen"},capabilityTurnVideoOnPresent:{title:"Ihre Kamera wurde aktiviert",message:"Aktivieren Sie diese, wenn Sie Video freigeben möchten.",dismissButtonAriaLabel:"Schließen"},capabilityTurnVideoOnAbsent:{title:"Ihre Kamera wurde deaktiviert.",message:"Sie können kein Video mehr freigeben.",dismissButtonAriaLabel:"Schließen"},capabilityUnmuteMicPresent:{title:"Ihr Mikrofon wurde aktiviert",message:"Wenn Sie sprechen möchten, heben Sie die Stummschaltung auf.",dismissButtonAriaLabel:"Schließen"},capabilityUnmuteMicAbsent:{title:"Ihr Mikro wurde deaktiviert",message:"Sie können die Stummschaltung nicht mehr aufheben.",dismissButtonAriaLabel:"Schließen"},togetherModeStarted:{title:"Der Togethermode wurde gestartet",dismissButtonAriaLabel:"Schließen"},togetherModeEnded:{title:"Der Togethermode wurde beendet",dismissButtonAriaLabel:"Schließen"},transcriptionError:{title:"Bei der Transkription ist ein Fehler aufgetreten.",message:"Die Transkription wurde gestoppt. Bitte versuchen Sie es später erneut.",dismissButtonAriaLabel:"Schließen"},transcriptionStartedByYou:{title:"Die Transkription wurde gestartet.",message:"Von Ihnen gestartet. Teilen Sie allen mit, dass sie transkribiert werden.",dismissButtonAriaLabel:"Schließen"}};var videoGallery$i={screenIsBeingSharedMessage:"Sie teilen Ihren Bildschirm",screenShareLoadingMessage:"Der Bildschirm von {participant} wird geladen",localScreenShareLoadingMessage:"Ihr Bildschirm wird geladen.",localVideoLabel:"Sie",localVideoCameraSwitcherLabel:"Kamera wechseln",localVideoMovementLabel:"Verschiebbare lokale Videokachel",localVideoMovementAriaLabel:"Verschiebbare lokale Videokachel",localVideoSelectedDescription:"{cameraName} ausgewählt",displayNamePlaceholder:"Teilnehmer ohne Namen",fitRemoteParticipantToFrame:"An Rahmen anpassen",fillRemoteParticipantFrame:"Rahmen ausfüllen",pinParticipantForMe:"Für mich anheften",pinParticipantForMeLimitReached:"Anheften (Limit erreicht)",unpinParticipantForMe:"Lösen",pinParticipantMenuItemAriaLabel:"{participantName} anheften",unpinParticipantMenuItemAriaLabel:"{participantName} lösen",pinnedParticipantAnnouncementAriaLabel:"{participantName} angeheftet",unpinnedParticipantAnnouncementAriaLabel:"{participantName} gelöst",startSpotlightVideoTileMenuLabel:"Spotlight für alle",addSpotlightVideoTileMenuLabel:"Spotlight hinzufügen",spotlightLimitReachedMenuTitle:"Spotlight-Limit erreicht",stopSpotlightVideoTileMenuLabel:"Spotlight beenden",stopSpotlightOnSelfVideoTileMenuLabel:"Spotlight beenden",attendeeRole:"Teilnehmer",muteParticipantMenuItemLabel:"Stummschalten",waitingScreenText:"Warten auf andere Teilnehmer",forbidAudio:"Mikro deaktivieren",permitAudio:"Mikrofon zulassen",forbidAudioTileMenuLabel:"Mikro deaktivieren",permitAudioTileMenuLabel:"Mikrofon zulassen",forbidVideo:"Kamera deaktivieren",permitVideo:"Kamera zulassen",forbidVideoTileMenuLabel:"Kamera deaktivieren",permitVideoTileMenuLabel:"Kamera zulassen"};var dialpad$i={placeholderText:"Telefonnummer eingeben",deleteButtonAriaLabel:"Löschen"};var holdButton$i={onLabel:"Fortsetzen",offLabel:"Halten",tooltipOnContent:"Anruf fortsetzen",tooltipOffContent:"Anruf halten"};var videoTile$i={participantStateRinging:"Anrufen...",participantReconnecting:"Verbindung wird wiederhergestellt...",participantStateHold:"In der Warteschleife",moreOptionsButtonAriaLabel:"Weitere Optionen {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"stummgeschaltet",moreOptionsParticipantMutedStateUnmutedAriaLabel:"Stummschaltung aufgehoben",moreOptionsParticipantHandRaisedAriaLabel:"Hand erhoben",moreOptionsParticipantIsSpeakingAriaLabel:"Sprechen",moreOptionsParticipantMicDisabledAriaLabel:"Mikrofon deaktiviert",moreOptionsParticipantCameraDisabledAriaLabel:"Kamera deaktiviert"};var CameraAndMicrophoneSitePermissionsRequest$i={primaryText:"Zulassen, dass {appName} Ihre Kamera und Ihr Mikrofon verwendet",secondaryText:"So können die Teilnehmer Sie sehen und hören.",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung",primaryButtonText:"Ohne Kamera und Mikrofon fortfahren",ariaLabel:"Zugriff auf Kamera und Mikrofon zulassen"};var CameraSitePermissionsRequest$i={primaryText:"{appName} die Verwendung Ihrer Kamera erlauben",secondaryText:"So können die Teilnehmer Sie sehen.",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung",primaryButtonText:"Ohne Kamera fortfahren",ariaLabel:"Kamerazugriff zulassen"};var MicrophoneSitePermissionsRequest$i={primaryText:"{appName} die Verwendung Ihres Mikrofons erlauben",secondaryText:"So können die Teilnehmer Sie hören.",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung",primaryButtonText:"Ohne Mikrofon fortfahren",ariaLabel:"Mikrofonzugriff zulassen"};var CameraAndMicrophoneSitePermissionsCheck$i={primaryText:"Überprüfen des Kamera- und Mikrofonzugriffs",secondaryText:"Zugriff bei Aufforderung zulassen. Dies ist so, dass Teilnehmer Sie sehen und hören können.",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung",primaryButtonText:"Ohne Kamera und Mikrofon fortfahren",ariaLabel:"Überprüfen des Kamera- und Mikrofonzugriffs. Erlauben Sie den Zugriff, wenn Sie aufgefordert werden."};var CameraSitePermissionsCheck$i={primaryText:"Überprüfung des Kamerazugriffs",secondaryText:"Zugriff bei Aufforderung zulassen. Dies ist so, dass Teilnehmer Sie sehen können.",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung",primaryButtonText:"Ohne Kamera fortfahren",ariaLabel:"Überprüfen des Kamerazugriffs. Erlauben Sie den Zugriff, wenn Sie aufgefordert werden."};var MicrophoneSitePermissionsCheck$i={primaryText:"Überprüfen des Mikrofonzugriffs",secondaryText:"Zugriff bei Aufforderung zulassen. Dies ist so, dass Teilnehmer Sie hören können.",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung",primaryButtonText:"Ohne Mikrofon fortfahren",ariaLabel:"Der Mikrofonzugriff wird überprüft. Erlauben Sie den Zugriff, wenn Sie aufgefordert werden."};var CameraAndMicrophoneSitePermissionsDenied$i={primaryText:"Auf Kamera und Mikrofon kann nicht zugegriffen werden",secondaryText:"Klicken Sie auf das Schlosssymbol in der Adressleiste, um dieser Webseite Mikrofonberechtigungen zu erteilen. Möglicherweise ist eine Seitenaktualisierung erforderlich.",primaryButtonText:"Ohne Kamera und Mikrofon fortfahren",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung"};var CameraAndMicrophoneSitePermissionsDeniedSafari$i={primaryText:"Auf Kamera und Mikrofon kann nicht zugegriffen werden",secondaryText:"Aktualisieren Sie die Seite, um Berechtigungen zuzulassen, oder überprüfen Sie die Einstellungen dieses Browsers und stellen Sie sicher, dass die Berechtigungen für diese Website aktiviert sind.",primaryButtonText:"Ohne Kamera und Mikrofon fortfahren",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung"};var CameraSitePermissionsDenied$i={primaryText:"Auf die Kamera kann nicht zugegriffen werden",secondaryText:"Klicken Sie auf das Schlosssymbol in der Adressleiste, um dieser Webseite Kameraberechtigungen zu erteilen. Möglicherweise ist eine Seitenaktualisierung erforderlich.",primaryButtonText:"Ohne Kamera fortfahren",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung"};var MicrophoneSitePermissionsDenied$i={primaryText:"Auf das Mikrofon kann nicht zugegriffen werden",secondaryText:"Klicken Sie auf das Schlosssymbol in der Adressleiste, um dieser Webseite Mikrofonberechtigungen zu erteilen. Möglicherweise ist eine Seitenaktualisierung erforderlich.",primaryButtonText:"Ohne Mikrofon fortfahren",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung"};var CameraSitePermissionsDeniedSafari$i={primaryText:"Auf die Kamera kann nicht zugegriffen werden",secondaryText:"Aktualisieren Sie die Seite, um Berechtigungen zuzulassen, oder überprüfen Sie die Einstellungen dieses Browsers und stellen Sie sicher, dass die Berechtigungen für diese Website aktiviert sind.",primaryButtonText:"Ohne Kamera fortfahren",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung"};var MicrophoneSitePermissionsDeniedSafari$i={primaryText:"Auf das Mikrofon kann nicht zugegriffen werden",secondaryText:"Aktualisieren Sie die Seite, um Berechtigungen zuzulassen, oder überprüfen Sie die Einstellungen dieses Browsers und stellen Sie sicher, dass die Berechtigungen für diese Website aktiviert sind.",primaryButtonText:"Ohne Mikrofon fortfahren",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung"};var UnsupportedBrowser$i={primaryText:"Der Browser wird nicht unterstützt.",secondaryText:"Bitte nehmen Sie über einen kompatiblen Browser an diesem Anruf teil.",moreHelpLinkText:"Kompatibilitätsanforderungen anzeigen"};var UnsupportedBrowserVersion$i={primaryText:"Browserupdate erforderlich",secondaryText:"Um einen optimalen Anruf zu gewährleisten, aktualisieren Sie Ihren Browser, und versuchen Sie dann erneut, an dem Anruf teilzunehmen.",moreHelpLinkText:"Kompatibilitätsanforderungen anzeigen",continueAnywayButtonText:"Anruf ohne Aktualisierung starten"};var UnsupportedOperatingSystem$i={primaryText:"Das Betriebssystem wird nicht unterstützt.",secondaryText:"Nehmen Sie an diesem Anruf teil, indem Sie ein Gerät mit einem kompatiblen Betriebssystem verwenden.",moreHelpLinkText:"Kompatibilitätsanforderungen anzeigen"};var BrowserPermissionDenied$i={primaryText:"Kamera oder Mikrofon kann nicht verwendet werden",secondaryText:"Ihr Browser hat möglicherweise keinen Zugriff auf Ihre Kamera oder Ihr Mikrofon. Öffnen Sie die Systemeinstellungen, um dieses Problem zu beheben.",primaryButtonText:"Erneut versuchen",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung"};var BrowserPermissionDeniedIOS$i={primaryText:"Mikrofonzugriff zulassen, um fortzufahren",secondaryText:"Damit andere Teilnehmer Sie hören können.",primaryButtonText:"Erneut versuchen",imageAltText:"Berechtigungsspeicherort für Mikrofon- und Kamerageräte für iOS",linkText:"Benötigen Sie Hilfe? Hilfe zur Problembehandlung",step1Text:"Zur Einstellungs-App wechseln",step2Text:"Scrollen Sie nach unten zu den Einstellungen dieses Browsers",step3Text:"Mikrofon einschalten (Kamera optional)",step4Text:"Versuchen Sie erneut, dem Anruf beizutreten",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$i={leftNavButtonAriaLabel:"vorherige Seite {current} von {total}",rightNavButtonAriaLabel:"nächste Seite {current} von {total}"};var meetingConferencePhoneInfo$i={meetingConferencePhoneInfoModalTitle:"Von Ihrem Telefon aus an Anruf teilnehmen",meetingConferencePhoneInfoModalDialIn:"Nummer wählen",meetingConferencePhoneInfoModalMeetingId:"Besprechungs-ID eingeben",meetingConferencePhoneInfoModalWait:"Warten Sie, bis sie vom Organisator zugelassen werden.",meetingConferencePhoneInfoModalNoPhoneAvailable:"Kein Telefon verfügbar",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Gebührenfrei)",meetingConferencePhoneInfoModalToll:"(Gebührenpflichtig)"};var IncomingCallNotification$i={incomingCallNotificationPlaceholderId:"Unbekannter Benutzer",incomingCallNotificationPlaceholderAlert:"{callerName} ruft an",incomingCallNoticicationAcceptWithAudioAriaLabel:"Mit Audio annehmen",incomingCallNoticicationAcceptWithVideoAriaLabel:"Mit Video annehmen",incomingCallNoticicationRejectAriaLabel:"Ablehnen",incomingCallNotificationAcceptButtonLabel:"Akzeptieren",incomingCallNotificationRejectButtonLabel:"Ablehnen",incomingCallNotificationAccceptWithVideoButtonLabel:"Mit Video annehmen",incomingCallNotificationDismissButtonAriaLabel:"Verwerfen"};var de_DE$1 = {participantItem:participantItem$i,ParticipantList:ParticipantList$i,typingIndicator:typingIndicator$i,sendBox:sendBox$i,richTextSendBox:richTextSendBox$i,realTimeTextModal:realTimeTextModal$i,captionsBanner:captionsBanner$i,mentionPopover:mentionPopover$i,imageOverlay:imageOverlay$i,messageStatusIndicator:messageStatusIndicator$i,endCallButton:endCallButton$i,cameraButton:cameraButton$i,microphoneButton:microphoneButton$i,devicesButton:devicesButton$i,participantsButton:participantsButton$i,screenShareButton:screenShareButton$i,raiseHandButton:raiseHandButton$i,reactionButton:reactionButton$i,realTimeText:realTimeText$i,captionsSettingsModal:captionsSettingsModal$i,spokenLanguages:spokenLanguages$i,captionLanguages:captionLanguages$i,startCaptionsButton:startCaptionsButton$i,startRealTimeTextButton:startRealTimeTextButton$i,messageThread:messageThread$i,errorBar:errorBar$i,notificationStack:notificationStack$i,videoGallery:videoGallery$i,dialpad:dialpad$i,holdButton:holdButton$i,videoTile:videoTile$i,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$i,CameraSitePermissionsRequest:CameraSitePermissionsRequest$i,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$i,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$i,CameraSitePermissionsCheck:CameraSitePermissionsCheck$i,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$i,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$i,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$i,CameraSitePermissionsDenied:CameraSitePermissionsDenied$i,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$i,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$i,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$i,UnsupportedBrowser:UnsupportedBrowser$i,UnsupportedBrowserVersion:UnsupportedBrowserVersion$i,UnsupportedOperatingSystem:UnsupportedOperatingSystem$i,BrowserPermissionDenied:BrowserPermissionDenied$i,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$i,verticalGallery:verticalGallery$i,meetingConferencePhoneInfo:meetingConferencePhoneInfo$i,IncomingCallNotification:IncomingCallNotification$i};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for German (Germany).
 *
 * @public
 */
const COMPONENT_LOCALE_DE_DE = {
    strings: createComponentStrings(de_DE$1)
};

var participantItem$h={isMeText:"(tú)",menuTitle:"Más opciones",removeButtonLabel:"Quitar",sharingIconLabel:"Compartiendo",mutedIconLabel:"Silenciado",handRaisedIconLabel:"Mano levantada: posición {position}",displayNamePlaceholder:"Participante sin nombre",participantStateRinging:"Llamando...",participantStateHold:"En espera",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, Más opciones",attendeeRole:"Asistente",micDisabledIconLabel:"Micophone deshabilitado",cameraDisabledIconLabel:"Cámara deshabilitada"};var ParticipantList$h={overflowParticipantCount:"+{overflowCount} más"};var typingIndicator$h={singleUser:"{user} está escribiendo...",multipleUsers:"{users} están escribiendo...",multipleUsersAbbreviateOne:"{users} y 1 usuario más están escribiendo...",multipleUsersAbbreviateMany:"{users} y {numOthers} usuarios más están escribiendo...",delimiter:", "};var sendBox$h={placeholderText:"Escribir un mensaje",textTooLong:"La longitud del mensaje supera el límite máximo.",sendButtonAriaLabel:"Enviar mensaje",attachmentUploadsPendingError:"El archivo se está cargando. Espere.",removeAttachment:"Quitar archivo",uploading:"Cargando",uploadCompleted:"Carga completada",attachmentMoreMenu:"Más opciones"};var richTextSendBox$h={placeholderText:"Escribir un mensaje",textTooLong:"La longitud del mensaje supera el límite máximo.",sendButtonAriaLabel:"Enviar mensaje",attachmentUploadsPendingError:"El archivo se está cargando. Espere.",imageUploadsPendingError:"La imagen se está cargando. Espere.",removeAttachment:"Quitar archivo",uploading:"Cargando",uploadCompleted:"Carga completada",richTextBoldTooltip:"Negrita",richTextItalicTooltip:"Cursiva",richTextUnderlineTooltip:"Subrayado",richTextBulletListTooltip:"Lista con viñetas",richTextNumberListTooltip:"Lista numerada",richTextIncreaseIndentTooltip:"Aumentar sangría",richTextDecreaseIndentTooltip:"Reducir sangría",richTextInsertTableTooltip:"Insertar tabla",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Insertar",richTextInsertTableMenuTitle:"Insertar {column} x {row} tabla",richTextInsertRowAboveMenu:"Insertar arriba",richTextInsertRowBelowMenu:"Insertar debajo",richTextInsertColumnLeftMenu:"Insertar a la izquierda",richTextInsertColumnRightMenu:"Insertar a la derecha",richTextDeleteRowOrColumnMenu:"Eliminar",richTextDeleteColumnMenu:"Eliminar columna",richTextDeleteRowMenu:"Eliminar fila",richTextDeleteTableMenu:"Eliminar tabla",richTextToolbarAriaLabel:"Barra de herramientas de formato de mensaje",richTextToolbarMoreButtonAriaLabel:"Más opciones",attachmentMoreMenu:"Más opciones",richTextNewBulletedListItemAnnouncement:"Viñeta, nueva línea",richTextNewNumberedListItemAnnouncement:"{0}, nueva línea",richTextBulletedListAppliedAnnouncement:"Lista con viñetas aplicada",richTextNumberedListAppliedAnnouncement:"Lista numerada aplicada"};var realTimeTextModal$h={realTimeTextModalAriaLabel:"Activar modal RTT",realTimeTextModalTitle:"¿Activar RTT?",realTimeTextModalText:"Se activará el texto en tiempo real (RTT) para todos los participantes de la reunión.",realTimeTextConfirmButtonLabel:"Activar",realTimeTextCancelButtonLabel:"Cancelar",realTimeTextCloseModalButtonAriaLabel:"Cerrar modal RTT"};var captionsBanner$h={captionsBannerSpinnerText:"Iniciando subtítulos...",realTimeTextInputBoxDefaultText:"Escribir un mensaje en tiempo real",realTimeTextInputErrorMessage:"El mensaje no puede exceder los 2000 caracteres",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RtT (texto en tiempo real) está habilitado para todos los participantes durante toda la reunión.",realTimeTextBannerLinkLabel:"Más información",captionsOnlyContainerTitle:"Subtítulos",realTimeTextOnlyContainerTitle:"Texto en tiempo real",captionsAndRealTimeTextContainerTitle:"Subtítulos y texto en tiempo real",expandButtonAriaLabel:"Expandir",minimizeButtonAriaLabel:"Contraer"};var mentionPopover$h={mentionPopoverHeader:"Sugerencias"};var imageOverlay$h={downloadButtonLabel:"Descargar",dismissButtonAriaLabel:"Cerrar",overlayVisibleMessage:"Superposición de detalles de imagen"};var messageStatusIndicator$h={deliveredAriaLabel:"Mensaje enviado",deliveredTooltipText:"Enviado",seenAriaLabel:"Mensaje visto por otros usuarios",seenTooltipText:"Visto",readByTooltipText:"Leído por {messageThreadReadCount} de {remoteParticipantsCount}",sendingAriaLabel:"Envío de mensajes",sendingTooltipText:"Enviando",failedToSendAriaLabel:"Error al enviar el mensaje",failedToSendTooltipText:"Error al enviar"};var endCallButton$h={label:"Baja",tooltipContent:"Abandonar llamada",leaveOption:"Salir",endCallOption:"Finalizar la llamada para todos"};var cameraButton$h={onLabel:"Cámara",offLabel:"Cámara",tooltipDisabledContent:"La cámara está deshabilitada",tooltipOnContent:"Desconectar la cámara",tooltipOffContent:"Activar la cámara",tooltipVideoLoadingContent:"El vídeo se está cargando",cameraMenuTitle:"Cámara",cameraMenuTooltip:"Elegir cámara",cameraButtonSplitRoleDescription:"Botón de expansión",onSplitButtonAriaLabel:"Desconectar la cámara",offSplitButtonAriaLabel:"Activar la cámara",cameraActionTurnedOnAnnouncement:"La cámara se ha activado",cameraActionTurnedOffAnnouncement:"La cámara se ha desactivado",onSplitButtonPrimaryActionCamera:"Desconectar la cámara",offSplitButtonPrimaryActionCamera:"Activar la cámara",cameraPrimaryActionSplitButtonTitle:"Usar cámara",videoEffectsMenuItemTitle:"Efectos",cameraButtonAriaDescription:"opciones de cámara"};var microphoneButton$h={onLabel:"Micro",offLabel:"Micro",tooltipDisabledContent:"Micrófono deshabilitado",tooltipOnContent:"Silenciar micrófono",tooltipOffContent:"Reactivar micrófono",microphoneMenuTitle:"Micrófono",microphoneMenuTooltip:"Elegir micrófono",speakerMenuTitle:"Altavoz",speakerMenuTooltip:"Elegir altavoz",microphoneButtonSplitRoleDescription:"Botón de expansión",onSplitButtonAriaLabel:"Desactivar audio del micrófono",offSplitButtonAriaLabel:"Reactivar audio del micrófono",microphoneActionTurnedOnAnnouncement:"El micrófono se ha activado",microphoneActionTurnedOffAnnouncement:"El micrófono se ha desactivado",offSplitButtonMicrophonePrimaryAction:"Reactivar audio del micrófono",onSplitButtonMicrophonePrimaryAction:"Desactivar audio del micrófono",microphonePrimaryActionSplitButtonTitle:"Usar micrófono",microphoneAriaDescription:"Opciones de audio",deepNoiseSuppressionTitle:"Supresión de ruido",deepNoiseSuppressionOnAnnouncement:"Se ha activado la supresión profunda de ruido",deepNoiseSuppressionOffAnnouncement:"Se ha desactivado la supresión profunda de ruido"};var devicesButton$h={label:"Dispositivos",tooltipContent:"Administrar dispositivos",cameraMenuTitle:"Cámara",cameraMenuTooltip:"Elegir cámara",audioDeviceMenuTitle:"Dispositivo de audio",audioDeviceMenuTooltip:"Elegir dispositivo de audio",microphoneMenuTitle:"Micrófono",microphoneMenuTooltip:"Elegir micrófono",speakerMenuTitle:"Altavoz",speakerMenuTooltip:"Elegir altavoz"};var participantsButton$h={label:"Contactos",ariaLabel:"Personas",tooltipContent:"Mostrar participantes",menuHeader:"En esta llamada",participantsListButtonLabel:"{numParticipants} personas",muteAllButtonLabel:"Silenciar a todos",copyInviteLinkButtonLabel:"Copiar vínculo de invitación",copyInviteLinkActionedAriaLabel:"Vínculo de invitación copiado"};var screenShareButton$h={onLabel:"Dejar de presentar",offLabel:"Presentar",tooltipDisabledContent:"La presentación está deshabilitada",tooltipOnContent:"Presentando su pantalla",tooltipOffContent:"Presentar la pantalla"};var raiseHandButton$h={offLabel:"Participar",onLabel:"Bajar",tooltipDisabledContent:"La acción Levantar la mano está deshabilitada",tooltipOnContent:"Bajar la mano",tooltipOffContent:"Levantar la mano"};var reactionButton$h={label:"Reaccionar",ariaLabel:"Reaccionar",tooltipDisabledContent:"La acción de reacción está deshabilitada",tooltipContent:"Enviar una reacción",likeReactionTooltipContent:"Me gusta",heartReactionTooltipContent:"Amor",laughReactionTooltipContent:"Risa",applauseReactionTooltipContent:"Aplauso",surprisedReactionTooltipContent:"Sorprendido"};var realTimeText$h={isTypingText:"Escritura en tiempo real"};var captionsSettingsModal$h={captionsSettingsModalTitle:"Configuración de subtítulos",captionsSettingsSpokenLanguageDropdownLabel:"Idioma hablado",captionsSettingsCaptionLanguageDropdownLabel:"Idioma de subtítulos",captionsSettingsSpokenLanguageDropdownInfoText:"Idioma en el que están hablando todos los usuarios de esta llamada.",captionsSettingsCaptionLanguageDropdownInfoText:"Los subtítulos aparecerán en este idioma.",captionsSettingsConfirmButtonLabel:"Confirmar",captionsSettingsCancelButtonLabel:"Cancelar",captionsSettingsModalAriaLabel:"Configuración modal de subtítulos",captionsSettingsCloseModalButtonAriaLabel:"Cerrar configuración de subtítulos"};var spokenLanguages$h={"ar-ae":"Árabe - Emiratos Árabes Unidos","ar-sa":"Árabe (Arabia Saudí)","da-dk":"Danés","de-de":"Alemán (Alemania)","en-au":"Inglés (Australia)","en-ca":"Inglés (Canadá)","en-gb":"Inglés (Reino Unido)","en-in":"Inglés (India)","en-nz":"Inglés (Nueva Zelanda)","en-us":"Inglés (Estados Unidos)","es-es":"Español - España (alfabetización internacional)","es-mx":"Español (México)","fi-fi":"Finés","fr-ca":"Francés (Canadá)","fr-fr":"Francés (Francia)","hi-in":"Hindi","it-it":"Italiano (Italia)","ja-jp":"Japonés","ko-kr":"Coreano","nb-no":"Noruego (Bokmål)","nl-be":"Neerlandés (Bélgica)","nl-nl":"Neerlandés (Países Bajos)","pl-pl":"Polaco","pt-br":"Portugués (Brasil)","ru-ru":"Ruso","sv-se":"Sueco","zh-cn":"Chino - República Popular China","zh-hk":"Chino (RAE de Hong Kong)","cs-cz":"Checo","pt-pt":"Portugués (Portugal)","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Tailandés","he-il":"Hebreo","cy-gb":"Galés","uk-ua":"Ucraniano","el-gr":"Griego","hu-hu":"Húngaro","ro-ro":"Rumano","sk-sk":"Eslovaco","zh-tw":"Chino (Taiwán)"};var captionLanguages$h={ar:"Árabe",da:"Danés",de:"Alemán",en:"Inglés",es:"Español",fi:"Finés","fr-ca":"Francés (Canadá)",fr:"Francés (Francia)",hi:"Hindi",it:"Italiano",ja:"Japonés",ko:"Coreano",nb:"Noruego (Bokmål)",nl:"Neerlandés",pl:"Polaco",pt:"Portugués (Brasil)",ru:"Ruso",sv:"Sueco","zh-Hans":"Chino (simplificado)","zh-Hant":"Chino (tradicional)",cs:"Checo","pt-pt":"Portugués (Portugal)",tr:"Turco",vi:"Vietnamita",th:"Tailandés",he:"Hebreo",cy:"Galés",uk:"Ucraniano",el:"Griego",hu:"Húngaro",ro:"Rumano",sk:"Eslovaco"};var startCaptionsButton$h={onLabel:"Activar subtítulos",offLabel:"Desactivar subtítulos",tooltipOnContent:"Desactivar subtítulos",tooltipOffContent:"Activar subtítulos"};var startRealTimeTextButton$h={label:"Activar RTT para esta llamada",tooltipOffContent:"Activar RTT para esta llamada"};var messageThread$h={yesterday:"Ayer",sunday:"Domingo",monday:"Lunes",tuesday:"Martes",wednesday:"Miércoles",thursday:"Jueves",friday:"Viernes",saturday:"Sábado",participantJoined:"se unió al chat.",participantLeft:"abandonó el chat.",editMessage:"Editar",removeMessage:"Eliminar",resendMessage:"Intente enviar de nuevo",failToSendTag:"Error al enviar",editedTag:"Editado",liveAuthorIntro:"Mensaje de {author}",editedMessageLiveAuthorIntro:"Mensaje editado de {author}",editedMessageLocalUserLiveAuthorIntro:"Mensaje editado",messageContentAriaText:"{status} {author} dijo {message}",messageContentMineAriaText:"{status} Usted dijo {message}",editBoxTextLimit:"El mensaje supera el límite de {limitNumber} caracteres",editBoxPlaceholderText:"Editar el mensaje",newMessagesIndicator:"Mensajes nuevos",noDisplayNameSub:"Sin nombre",editBoxCancelButton:"Cancelar",editBoxSubmitButton:"Listo",messageReadCount:"Leído por {messageReadByCount} de {remoteParticipantsCount}",actionMenuMoreOptions:"Más opciones",downloadAttachment:"Descargar archivo",openAttachment:"Abrir archivo",blockedWarningText:"Este mensaje se eliminó debido a la directiva de la organización.",blockedWarningLinkText:"Detalles",attachmentCardGroupMessage:"El mensaje tiene {attachmentCount} archivo adjunto",messageDeletedAnnouncementAriaLabel:"El mensaje se ha eliminado",richTextBoldTooltip:"Negrita",richTextItalicTooltip:"Cursiva",richTextUnderlineTooltip:"Subrayado",richTextBulletListTooltip:"Lista con viñetas",richTextNumberListTooltip:"Lista numerada",richTextIncreaseIndentTooltip:"Aumentar sangría",richTextDecreaseIndentTooltip:"Reducir sangría",richTextInsertTableTooltip:"Insertar tabla",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Insertar",richTextInsertTableMenuTitle:"Insertar {column} x {row} tabla",richTextInsertRowAboveMenu:"Insertar arriba",richTextInsertRowBelowMenu:"Insertar debajo",richTextInsertColumnLeftMenu:"Insertar a la izquierda",richTextInsertColumnRightMenu:"Insertar a la derecha",richTextDeleteRowOrColumnMenu:"Eliminar",richTextDeleteColumnMenu:"Eliminar columna",richTextDeleteRowMenu:"Eliminar fila",richTextDeleteTableMenu:"Eliminar tabla",richTextToolbarMoreButtonAriaLabel:"Más opciones",attachmentUploadsPendingError:"El archivo se está cargando. Espere.",imageUploadsPendingError:"La imagen se está cargando. Espere.",richTextNewBulletedListItemAnnouncement:"Viñeta, nueva línea",richTextNewNumberedListItemAnnouncement:"{0}, nueva línea",richTextBulletedListAppliedAnnouncement:"Lista con viñetas aplicada",richTextNumberedListAppliedAnnouncement:"Lista numerada aplicada"};var errorBar$h={unableToReachChatService:"Sin conexión",accessDenied:"No se puede acceder a los servicios de chat. Compruebe las credenciales de usuario proporcionadas.",userNotInChatThread:"Ya no está en este hilo de chat",sendMessageNotInChatThread:"No se pudo enviar el mensaje porque ya no está en este hilo de chat",sendMessageGeneric:"Error al enviar el mensaje",callingNetworkFailure:"Problema al conectar la llamada; parece que no hay conexión",startVideoGeneric:"No se pudo iniciar el vídeo",stopVideoGeneric:"No se pudo detener el vídeo",muteGeneric:"No se pudo silenciar el micrófono",unmuteGeneric:"No se pudo reactivar el micrófono",speakingWhileMuted:"El micrófono está desactivado",startScreenShareGeneric:"Hubo un problema al iniciar la pantalla compartida.",stopScreenShareGeneric:"No se pudo detener la pantalla compartida",callNetworkQualityLow:"La calidad de la red es baja.",teamsMeetingCallNetworkQualityLow:"La calidad de la red es baja.",callNoSpeakerFound:"No se encontraron altavoces ni auriculares. Conecta un dispositivo de audio para escuchar la llamada.",callNoMicrophoneFound:"No se encontraron micrófonos. Conecta un dispositivo de entrada de audio.",callMicrophoneAccessDenied:"No se puede acceder al micrófono. Haga clic en el bloqueo de la barra de direcciones para conceder permiso a esta página web.",callMicrophoneAccessDeniedSafari:"No se puede acceder al micrófono. Actualice la página para permitir permisos o compruebe la configuración de este explorador y compruebe que los permisos estén habilitados para este sitio web.",callMicrophoneMutedBySystem:"Su sistema ha silenciado su audio.",callMicrophoneUnmutedBySystem:"Su micrófono se ha recuperado y el sistema ha activado el audio.",callMacOsMicrophoneAccessDenied:"No se puede acceder al micrófono. Conceda permiso al micrófono en la configuración de privacidad de macOS.",callLocalVideoFreeze:"El ancho de banda de red es deficiente. Es posible que el vídeo aparezca pausado para otros usuarios en la llamada.",callCameraAccessDenied:"No se puede acceder a la cámara. Haga clic en el bloqueo de la barra de direcciones para conceder permiso a esta página web.",callCameraAccessDeniedSafari:"No se puede acceder a la cámara. Actualice la página para permitir permisos o compruebe la configuración de este explorador y compruebe que los permisos estén habilitados para este sitio web.",callCameraAlreadyInUse:"No se puede acceder a la cámara. Puede que ya la esté usando otra aplicación.",callVideoStoppedBySystem:"El sistema ha detenido el vídeo.",callVideoRecoveredBySystem:"El vídeo se ha reanudado.",callMacOsCameraAccessDenied:"MacOS está bloqueando el acceso a la cámara. Actualiza la configuración de privacidad para permitir que este explorador acceda a la cámara.",callMacOsScreenShareAccessDenied:"MacOS está bloqueando el uso compartido de la pantalla. Actualice la configuración de privacidad para permitir que este explorador grabe la pantalla.",dismissButtonAriaLabel:"Cerrar",failedToJoinCallGeneric:"No se pudo unir a la llamada.",failedToJoinCallInvalidMeetingLink:"No se puede unir a la reunión. Vínculo no válido.",cameraFrozenForRemoteParticipants:"Los usuarios de la llamada tienen problemas para ver el vídeo. Compruebe los dispositivos y la red.",unableToStartVideoEffect:"No se puede aplicar el efecto de vídeo.",startSpotlightWhileMaxParticipantsAreSpotlighted:"No se puede iniciar el primer plano en los participantes porque el número máximo de participantes ya está puesto en primer plano.",mutedByRemoteParticipant:"Otro participante lo ha silenciado."};var notificationStack$h={callingNetworkFailure:{title:"Llamada de conexión perturbadora",message:"Parece que estás desconectado. Compruebe la conexión de red e inténtelo de nuevo.",dismissButtonAriaLabel:"Cerrar"},startVideoGeneric:{title:"No se pudo iniciar el vídeo.",dismissButtonAriaLabel:"Cerrar"},stopVideoGeneric:{title:"No se pudo detener el vídeo.",dismissButtonAriaLabel:"Cerrar"},muteGeneric:{title:"No se pudo silenciar el micrófono.",dismissButtonAriaLabel:"Cerrar"},unmuteGeneric:{title:"No se pudo reactivar el micrófono.",dismissButtonAriaLabel:"Cerrar"},speakingWhileMuted:{title:"Estás silenciado.",dismissButtonAriaLabel:"Cerrar"},startScreenShareGeneric:{title:"Hubo un problema al iniciar la pantalla compartida.",dismissButtonAriaLabel:"Cerrar"},stopScreenShareGeneric:{title:"No se pudo detener la pantalla compartida.",dismissButtonAriaLabel:"Cerrar"},callNetworkQualityLow:{title:"La calidad de la red es baja.",dismissButtonAriaLabel:"Cerrar"},teamsMeetingCallNetworkQualityLow:{title:"La calidad de la red es baja.",dismissButtonAriaLabel:"Cerrar",message:"Únete a esta llamada desde el teléfono para obtener un mejor sonido. Puede seguir viendo la reunión en este dispositivo.",primaryButtonLabel:"Unirse por Teléfono"},callNoSpeakerFound:{title:"No se encontraron altavoces ni auriculares",message:"Conecta un dispositivo de audio para escuchar la llamada.",dismissButtonAriaLabel:"Cerrar"},callNoMicrophoneFound:{title:"No se han encontrado micrófonos",message:"Conecta un dispositivo de entrada de audio.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneAccessDenied:{title:"No se puede acceder al micrófono",message:"Haga clic en el bloqueo de la barra de direcciones para conceder permiso a esta página web.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneAccessDeniedSafari:{title:"No se puede acceder al micrófono",message:"Actualice la página para permitir permisos o compruebe la configuración de este explorador y compruebe que los permisos estén habilitados para este sitio web.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneMutedBySystem:{title:"Su sistema ha silenciado su audio.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneUnmutedBySystem:{title:"Su micrófono se ha recuperado y el sistema ha activado el audio.",dismissButtonAriaLabel:"Cerrar"},callMacOsMicrophoneAccessDenied:{title:"No se puede acceder al micrófono",message:"Conceda permiso al micrófono en la configuración de privacidad de macOS.",dismissButtonAriaLabel:"Cerrar"},callLocalVideoFreeze:{title:"El ancho de banda de red es deficiente",message:"Es posible que el vídeo aparezca pausado para otros usuarios en la llamada.",dismissButtonAriaLabel:"Cerrar"},callCameraAccessDenied:{title:"No se puede acceder a la cámara",message:"Haga clic en el bloqueo de la barra de direcciones para conceder permiso a esta página web.",dismissButtonAriaLabel:"Cerrar"},callCameraAccessDeniedSafari:{title:"No se puede acceder a la cámara",message:"Actualice la página para permitir permisos o compruebe la configuración de este explorador y compruebe que los permisos estén habilitados para este sitio web.",dismissButtonAriaLabel:"Cerrar"},callCameraAlreadyInUse:{title:"No se puede acceder a la cámara",message:"Puede que ya la esté usando otra aplicación.",dismissButtonAriaLabel:"Cerrar"},callVideoStoppedBySystem:{title:"El sistema ha detenido el vídeo.",dismissButtonAriaLabel:"Cerrar"},callVideoRecoveredBySystem:{title:"El vídeo se ha reanudado.",dismissButtonAriaLabel:"Cerrar"},callMacOsCameraAccessDenied:{title:"MacOS está bloqueando el acceso a la cámara",message:"Actualiza la configuración de privacidad para permitir que este explorador acceda a la cámara.",dismissButtonAriaLabel:"Cerrar"},callMacOsScreenShareAccessDenied:{title:"MacOS está bloqueando la pantalla compartida",message:"Actualice la configuración de privacidad para permitir que este explorador grabe la pantalla.",dismissButtonAriaLabel:"Cerrar"},failedToJoinCallGeneric:{title:"No se pudo unir a la llamada.",dismissButtonAriaLabel:"Cerrar"},failedToJoinCallInvalidMeetingLink:{title:"No se puede unir a la reunión. Vínculo no válido.",dismissButtonAriaLabel:"Cerrar"},cameraFrozenForRemoteParticipants:{title:"Los usuarios de la llamada tienen problemas para ver el vídeo",message:"Compruebe los dispositivos y la red.",dismissButtonAriaLabel:"Cerrar"},unableToStartVideoEffect:{title:"No se puede aplicar el efecto de vídeo.",dismissButtonAriaLabel:"Cerrar"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"No se puede poner primer plano a los participantes",message:"Ya está puesto en primer plano el número máximo de participantes.",dismissButtonAriaLabel:"Cerrar"},mutedByRemoteParticipant:{title:"Otro participante lo ha silenciado.",dismissButtonAriaLabel:"Cerrar"},recordingStarted:{title:"La grabación se ha iniciado.",message:"Al unirse, da su consentimiento para que se transcriba esta reunión.",dismissButtonAriaLabel:"Cerrar"},transcriptionStarted:{title:"La transcripción se ha iniciado.",message:"Al unirse, da su consentimiento para que se transcriba esta reunión.",dismissButtonAriaLabel:"Cerrar"},recordingStopped:{title:"La grabación se está guardando.",message:"La grabación se ha detenido.",dismissButtonAriaLabel:"Cerrar"},transcriptionStopped:{title:"Se está guardando la transcripción.",message:"La transcripción se ha detenido.",dismissButtonAriaLabel:"Cerrar"},recordingAndTranscriptionStarted:{title:"Se han iniciado la grabación y la transcripción.",message:"Al unirse, da su consentimiento para que se transcriba esta reunión.",dismissButtonAriaLabel:"Cerrar"},recordingAndTranscriptionStopped:{title:"Se están guardando la grabación y la transcripción.",message:"Se han detenido la grabación y la transcripción.",dismissButtonAriaLabel:"Cerrar"},recordingStoppedStillTranscribing:{title:"La grabación se ha detenido.",message:"Solo está transcribiendo esta reunión.",dismissButtonAriaLabel:"Cerrar"},transcriptionStoppedStillRecording:{title:"La transcripción se ha detenido.",message:"Solo está grabando esta reunión.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomOpened:{title:"Le moveremos a la sala asignada en 5 segundos.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomChanged:{title:"La sala para sesión de subgrupo ha cambiado",message:"Le moveremos a la sala asignada en 5 segundos.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomClosed:{title:"La sala para sesión de subgrupo se ha cerrado",message:"Volveremos a la reunión en 5 segundos.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomOpenedPromptJoin:{title:"¿Quiere unirse a la sala para sesión de subgrupo?",message:"Se le ha asignado a una sala para sesión de subgrupo.",dismissButtonAriaLabel:"Cerrar",primaryButtonLabel:"Unirse a la sala",secondaryButtonLabel:"Más tarde"},breakoutRoomJoined:{title:"Se ha unido a una sala para sesión de subgrupo.",dismissButtonAriaLabel:"Cerrar"},breakoutRoomClosingSoon:{title:"Límite de tiempo de sala a punto de expirar.",message:"Esta sala se cerrará en 30 segundos",dismissButtonAriaLabel:"Cerrar"},capabilityTurnVideoOnPresent:{title:"La cámara se ha habilitado",message:"Active esta opción si quiere compartir vídeo.",dismissButtonAriaLabel:"Cerrar"},capabilityTurnVideoOnAbsent:{title:"Se deshabilitó la cámara",message:"Ya no puede compartir vídeo.",dismissButtonAriaLabel:"Cerrar"},capabilityUnmuteMicPresent:{title:"El micrófono se ha habilitado",message:"Reactive el audio si quiere hablar.",dismissButtonAriaLabel:"Cerrar"},capabilityUnmuteMicAbsent:{title:"El micro se ha deshabilitado.",message:"Ya no puede activar el audio.",dismissButtonAriaLabel:"Cerrar"},togetherModeStarted:{title:"El modo de colaboración se ha iniciado",dismissButtonAriaLabel:"Cerrar"},togetherModeEnded:{title:"El modo de colaboración ha finalizado",dismissButtonAriaLabel:"Cerrar"},transcriptionError:{title:"Se ha producido un error con la transcripción.",message:"La transcripción se ha detenido. Inténtelo de nuevo más tarde",dismissButtonAriaLabel:"Cerrar"},transcriptionStartedByYou:{title:"La transcripción se ha iniciado.",message:"Iniciado por usted. Informar a todos de que se les está transcribiendo.",dismissButtonAriaLabel:"Cerrar"}};var videoGallery$h={screenIsBeingSharedMessage:"Está compartiendo su pantalla",screenShareLoadingMessage:"Cargando la pantalla de {participant}",localScreenShareLoadingMessage:"Cargando la pantalla",localVideoLabel:"Tú",localVideoCameraSwitcherLabel:"Cambiar de cámara",localVideoMovementLabel:"Mosaico de vídeo local movible",localVideoMovementAriaLabel:"Mosaico de vídeo local movible",localVideoSelectedDescription:"{cameraName} seleccionada",displayNamePlaceholder:"Participante sin nombre",fitRemoteParticipantToFrame:"Ajustar al marco",fillRemoteParticipantFrame:"Marco de relleno",pinParticipantForMe:"Anclar para mí",pinParticipantForMeLimitReached:"Pin (límite alcanzado)",unpinParticipantForMe:"Desanclar",pinParticipantMenuItemAriaLabel:"Anclar {participantName}",unpinParticipantMenuItemAriaLabel:"Desanclar {participantName}",pinnedParticipantAnnouncementAriaLabel:"{participantName} anclado",unpinnedParticipantAnnouncementAriaLabel:"{participantName} desanclado",startSpotlightVideoTileMenuLabel:"Poner en primer plano para todos",addSpotlightVideoTileMenuLabel:"Agregar primer plano",spotlightLimitReachedMenuTitle:"Se alcanzó el límite de primer plano",stopSpotlightVideoTileMenuLabel:"Dejar de poner en primer plano",stopSpotlightOnSelfVideoTileMenuLabel:"Primer plano",attendeeRole:"Asistente",muteParticipantMenuItemLabel:"Silenciar",waitingScreenText:"Esperando a que se unan otros usuarios",forbidAudio:"Deshabilitar micrófono",permitAudio:"Permitir micrófono",forbidAudioTileMenuLabel:"Deshabilitar micrófono",permitAudioTileMenuLabel:"Permitir micrófono",forbidVideo:"Deshabilitar cámara",permitVideo:"Permitir cámara",forbidVideoTileMenuLabel:"Deshabilitar cámara",permitVideoTileMenuLabel:"Permitir cámara"};var dialpad$h={placeholderText:"Introduzca el número de teléfono",deleteButtonAriaLabel:"Eliminar"};var holdButton$h={onLabel:"Reanudar",offLabel:"En espera",tooltipOnContent:"Reanudar llamada",tooltipOffContent:"Poner llamada en espera"};var videoTile$h={participantStateRinging:"Llamando...",participantReconnecting:"Reconectando...",participantStateHold:"En espera",moreOptionsButtonAriaLabel:"Más opciones {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"audio desactivado",moreOptionsParticipantMutedStateUnmutedAriaLabel:"se reactivaron las notificaciones",moreOptionsParticipantHandRaisedAriaLabel:"Mano levantada",moreOptionsParticipantIsSpeakingAriaLabel:"Hablando",moreOptionsParticipantMicDisabledAriaLabel:"Micrófono deshabilitado",moreOptionsParticipantCameraDisabledAriaLabel:"Cámara deshabilitada"};var CameraAndMicrophoneSitePermissionsRequest$h={primaryText:"Permitir que {appName} usen la cámara y el micrófono",secondaryText:"Esto es para que los participantes puedan verle y oírle.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara ni micrófono",ariaLabel:"Permitir el acceso a la cámara y al micrófono"};var CameraSitePermissionsRequest$h={primaryText:"Permitir que {appName} use la cámara",secondaryText:"Esto sirve para que los participantes puedan verle.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara",ariaLabel:"Permitir el acceso a la cámara"};var MicrophoneSitePermissionsRequest$h={primaryText:"Permitir que {appName} use el micrófono",secondaryText:"Esto sirve para que los participantes puedan oírle.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin micrófono",ariaLabel:"Permitir el acceso al micrófono"};var CameraAndMicrophoneSitePermissionsCheck$h={primaryText:"Comprobando el acceso a la cámara y al micrófono",secondaryText:"Permitir el acceso si se le solicita. Esto es para que los participantes puedan verle y oírle.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara ni micrófono",ariaLabel:"Comprobando el acceso a la cámara y al micrófono. Permitir el acceso si se le solicita."};var CameraSitePermissionsCheck$h={primaryText:"Comprobando el acceso a la cámara",secondaryText:"Permitir el acceso si se le solicita. Esto es para que los participantes puedan verlo.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara",ariaLabel:"Comprobando el acceso a la cámara. Permitir el acceso si se le solicita."};var MicrophoneSitePermissionsCheck$h={primaryText:"Comprobando el acceso al micrófono",secondaryText:"Permitir el acceso si se le solicita. Esto es para que los participantes puedan oírle.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin micrófono",ariaLabel:"Comprobando el acceso al micrófono. Permitir el acceso si se le solicita."};var CameraAndMicrophoneSitePermissionsDenied$h={primaryText:"No se puede permitir el acceso a la cámara ni al micrófono.",secondaryText:"Haga clic en el icono de bloqueo de la barra de direcciones para conceder permisos de micrófono a esta página web. Es posible que sea necesario actualizar la página.",primaryButtonText:"Continuar sin cámara ni micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var CameraAndMicrophoneSitePermissionsDeniedSafari$h={primaryText:"No se puede permitir el acceso a la cámara ni al micrófono.",secondaryText:"Actualice la página para permitir permisos o compruebe la configuración de este explorador y compruebe que los permisos estén habilitados para este sitio web.",primaryButtonText:"Continuar sin cámara ni micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var CameraSitePermissionsDenied$h={primaryText:"No se puede acceder a la cámara",secondaryText:"Haga clic en el icono de bloqueo de la barra de direcciones para conceder permisos de cámara a esta página web. Es posible que sea necesario actualizar la página.",primaryButtonText:"Continuar sin cámara",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var MicrophoneSitePermissionsDenied$h={primaryText:"No se puede acceder al micrófono.",secondaryText:"Haga clic en el icono de bloqueo de la barra de direcciones para conceder permisos de micrófono a esta página web. Es posible que sea necesario actualizar la página.",primaryButtonText:"Continuar sin micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var CameraSitePermissionsDeniedSafari$h={primaryText:"No se puede acceder a la cámara.",secondaryText:"Actualice la página para permitir permisos o compruebe la configuración de este explorador y compruebe que los permisos estén habilitados para este sitio web.",primaryButtonText:"Continuar sin cámara",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var MicrophoneSitePermissionsDeniedSafari$h={primaryText:"No se puede acceder al micrófono",secondaryText:"Actualice la página para permitir permisos o compruebe la configuración de este explorador y compruebe que los permisos estén habilitados para este sitio web.",primaryButtonText:"Continuar sin micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var UnsupportedBrowser$h={primaryText:"Explorador no compatible",secondaryText:"Únase a esta llamada con un explorador compatible.",moreHelpLinkText:"Ver los requisitos de compatibilidad"};var UnsupportedBrowserVersion$h={primaryText:"Es necesario actualizar el explorador",secondaryText:"Para garantizar la mejor llamada posible, actualice el explorador e intente unirse de nuevo a la llamada.",moreHelpLinkText:"Ver los requisitos de compatibilidad",continueAnywayButtonText:"Iniciar llamada sin actualizar"};var UnsupportedOperatingSystem$h={primaryText:"Sistema operativo no compatible",secondaryText:"Únase a esta llamada con un dispositivo con un sistema operativo compatible.",moreHelpLinkText:"Ver los requisitos de compatibilidad"};var BrowserPermissionDenied$h={primaryText:"No se puede usar la cámara o el micrófono",secondaryText:"Es posible que el explorador no tenga acceso a la cámara o al micrófono. Para solucionar este problema, abra Preferencias del sistema.",primaryButtonText:"Intentar de nuevo",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var BrowserPermissionDeniedIOS$h={primaryText:"Permite el acceso al micrófono para continuar",secondaryText:"Para que otros participantes puedan oírte.",primaryButtonText:"Intentar de nuevo",imageAltText:"Ubicación de permisos del dispositivo de micrófono y cámara para iOS",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",step1Text:"Ve a la aplicación de Configuración",step2Text:"Desplázate hacia abajo hasta la configuración de este explorador",step3Text:"Activar micrófono (cámara opcional)",step4Text:"Intenta unirte de nuevo a la llamada",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$h={leftNavButtonAriaLabel:"página anterior {current} de {total}",rightNavButtonAriaLabel:"página siguiente {current} de {total}"};var meetingConferencePhoneInfo$h={meetingConferencePhoneInfoModalTitle:"Unirse a la llamada desde el teléfono",meetingConferencePhoneInfoModalDialIn:"Marcar número",meetingConferencePhoneInfoModalMeetingId:"Escribir el id. de reunión",meetingConferencePhoneInfoModalWait:"Esperar a ser admitido por el organizador",meetingConferencePhoneInfoModalNoPhoneAvailable:"No hay teléfono disponible",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Número gratuito)",meetingConferencePhoneInfoModalToll:"(Número de pago)"};var IncomingCallNotification$h={incomingCallNotificationPlaceholderId:"Usuario desconocido",incomingCallNotificationPlaceholderAlert:"{callerName} está llamando",incomingCallNoticicationAcceptWithAudioAriaLabel:"Aceptar con audio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Aceptar con vídeo",incomingCallNoticicationRejectAriaLabel:"Rechazar",incomingCallNotificationAcceptButtonLabel:"Aceptar",incomingCallNotificationRejectButtonLabel:"Rechazar",incomingCallNotificationAccceptWithVideoButtonLabel:"Aceptar con vídeo",incomingCallNotificationDismissButtonAriaLabel:"Descartar"};var es_ES$1 = {participantItem:participantItem$h,ParticipantList:ParticipantList$h,typingIndicator:typingIndicator$h,sendBox:sendBox$h,richTextSendBox:richTextSendBox$h,realTimeTextModal:realTimeTextModal$h,captionsBanner:captionsBanner$h,mentionPopover:mentionPopover$h,imageOverlay:imageOverlay$h,messageStatusIndicator:messageStatusIndicator$h,endCallButton:endCallButton$h,cameraButton:cameraButton$h,microphoneButton:microphoneButton$h,devicesButton:devicesButton$h,participantsButton:participantsButton$h,screenShareButton:screenShareButton$h,raiseHandButton:raiseHandButton$h,reactionButton:reactionButton$h,realTimeText:realTimeText$h,captionsSettingsModal:captionsSettingsModal$h,spokenLanguages:spokenLanguages$h,captionLanguages:captionLanguages$h,startCaptionsButton:startCaptionsButton$h,startRealTimeTextButton:startRealTimeTextButton$h,messageThread:messageThread$h,errorBar:errorBar$h,notificationStack:notificationStack$h,videoGallery:videoGallery$h,dialpad:dialpad$h,holdButton:holdButton$h,videoTile:videoTile$h,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$h,CameraSitePermissionsRequest:CameraSitePermissionsRequest$h,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$h,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$h,CameraSitePermissionsCheck:CameraSitePermissionsCheck$h,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$h,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$h,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$h,CameraSitePermissionsDenied:CameraSitePermissionsDenied$h,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$h,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$h,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$h,UnsupportedBrowser:UnsupportedBrowser$h,UnsupportedBrowserVersion:UnsupportedBrowserVersion$h,UnsupportedOperatingSystem:UnsupportedOperatingSystem$h,BrowserPermissionDenied:BrowserPermissionDenied$h,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$h,verticalGallery:verticalGallery$h,meetingConferencePhoneInfo:meetingConferencePhoneInfo$h,IncomingCallNotification:IncomingCallNotification$h};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Spanish (Spain).
 *
 * @public
 */
const COMPONENT_LOCALE_ES_ES = {
    strings: createComponentStrings(es_ES$1)
};

var participantItem$g={isMeText:"(tú)",menuTitle:"Más opciones",removeButtonLabel:"Quitar",sharingIconLabel:"Compartiendo",mutedIconLabel:"Silenciado",handRaisedIconLabel:"Mano levantada: posición {position}",displayNamePlaceholder:"Participante sin nombre",participantStateRinging:"Llamando...",participantStateHold:"En espera",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, Más opciones",attendeeRole:"Asistente",micDisabledIconLabel:"Micrófono deshabilitado",cameraDisabledIconLabel:"Cámara deshabilitada"};var ParticipantList$g={overflowParticipantCount:"+{overflowCount} más"};var typingIndicator$g={singleUser:"{user} está escribiendo...",multipleUsers:"{users} están escribiendo...",multipleUsersAbbreviateOne:"{users} y 1 usuario más están escribiendo...",multipleUsersAbbreviateMany:"{users} y {numOthers} usuarios más están escribiendo...",delimiter:", "};var sendBox$g={placeholderText:"Escribir un mensaje",textTooLong:"La longitud del mensaje supera el límite máximo.",sendButtonAriaLabel:"Enviar mensaje",attachmentUploadsPendingError:"El archivo se está cargando. Espera.",removeAttachment:"Quitar archivo",uploading:"Cargando",uploadCompleted:"Carga completada",attachmentMoreMenu:"Más opciones"};var richTextSendBox$g={placeholderText:"Escribir un mensaje",textTooLong:"La longitud del mensaje supera el límite máximo.",sendButtonAriaLabel:"Enviar mensaje",attachmentUploadsPendingError:"El archivo se está cargando. Espera.",imageUploadsPendingError:"La imagen se está cargando. Espera.",removeAttachment:"Quitar archivo",uploading:"Cargando",uploadCompleted:"Carga completada",richTextBoldTooltip:"Negrita",richTextItalicTooltip:"Cursiva",richTextUnderlineTooltip:"Subrayado",richTextBulletListTooltip:"Lista con viñetas",richTextNumberListTooltip:"Lista numerada",richTextIncreaseIndentTooltip:"Aumentar sangría",richTextDecreaseIndentTooltip:"Reducir sangría",richTextInsertTableTooltip:"Insertar tabla",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Insertar",richTextInsertTableMenuTitle:"Insertar tabla de {column} x {row}",richTextInsertRowAboveMenu:"Insertar arriba",richTextInsertRowBelowMenu:"Insertar debajo",richTextInsertColumnLeftMenu:"Insertar a la izquierda",richTextInsertColumnRightMenu:"Insertar a la derecha",richTextDeleteRowOrColumnMenu:"Eliminar",richTextDeleteColumnMenu:"Eliminar columna",richTextDeleteRowMenu:"Eliminar fila",richTextDeleteTableMenu:"Eliminar tabla",richTextToolbarAriaLabel:"Barra de herramientas de formato de mensaje",richTextToolbarMoreButtonAriaLabel:"Más opciones",attachmentMoreMenu:"Más opciones",richTextNewBulletedListItemAnnouncement:"Viñeta, nueva línea",richTextNewNumberedListItemAnnouncement:"{0}, nueva línea",richTextBulletedListAppliedAnnouncement:"Lista con viñetas aplicada",richTextNumberedListAppliedAnnouncement:"Lista numerada aplicada"};var realTimeTextModal$g={realTimeTextModalAriaLabel:"Activar modal RTT",realTimeTextModalTitle:"¿Activar RTT?",realTimeTextModalText:"Se activará el texto en tiempo real (RTT) para todos los participantes de la reunión.",realTimeTextConfirmButtonLabel:"Activar",realTimeTextCancelButtonLabel:"Cancelar",realTimeTextCloseModalButtonAriaLabel:"Cerrar modal RTT"};var captionsBanner$g={captionsBannerSpinnerText:"Iniciando subtítulos...",realTimeTextInputBoxDefaultText:"Escribir un mensaje en tiempo real",realTimeTextInputErrorMessage:"El mensaje no puede exceder los 2000 caracteres",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RtT (texto en tiempo real) está habilitado para todos los participantes durante toda la reunión.",realTimeTextBannerLinkLabel:"Más información",captionsOnlyContainerTitle:"Subtítulos",realTimeTextOnlyContainerTitle:"Texto en tiempo real",captionsAndRealTimeTextContainerTitle:"Subtítulos y texto en tiempo real",expandButtonAriaLabel:"Expandir",minimizeButtonAriaLabel:"Contraer"};var mentionPopover$g={mentionPopoverHeader:"Sugerencias"};var imageOverlay$g={downloadButtonLabel:"Descargar",dismissButtonAriaLabel:"Cerrar",overlayVisibleMessage:"Superposición de detalles de imagen"};var messageStatusIndicator$g={deliveredAriaLabel:"Mensaje enviado",deliveredTooltipText:"Enviado",seenAriaLabel:"Mensaje visto por otros usuarios",seenTooltipText:"Visto",readByTooltipText:"Leído por {messageThreadReadCount} de {remoteParticipantsCount}",sendingAriaLabel:"Envío de mensajes",sendingTooltipText:"Enviando",failedToSendAriaLabel:"Error al enviar el mensaje",failedToSendTooltipText:"Error al enviar"};var endCallButton$g={label:"Salir",tooltipContent:"Abandonar llamada",leaveOption:"Salir",endCallOption:"Finalizar la llamada para todos"};var cameraButton$g={onLabel:"Cámara",offLabel:"Cámara",tooltipDisabledContent:"La cámara está deshabilitada",tooltipOnContent:"Desactivar la cámara",tooltipOffContent:"Activar la cámara",tooltipVideoLoadingContent:"El vídeo se está cargando",cameraMenuTitle:"Cámara",cameraMenuTooltip:"Elegir cámara",cameraButtonSplitRoleDescription:"Botón de expansión",onSplitButtonAriaLabel:"Desactivar la cámara",offSplitButtonAriaLabel:"Activar la cámara",cameraActionTurnedOnAnnouncement:"La cámara se ha activado",cameraActionTurnedOffAnnouncement:"La cámara se ha desactivado",onSplitButtonPrimaryActionCamera:"Desactivar la cámara",offSplitButtonPrimaryActionCamera:"Activar la cámara",cameraPrimaryActionSplitButtonTitle:"Usar cámara",videoEffectsMenuItemTitle:"Efectos",cameraButtonAriaDescription:"opciones de cámara"};var microphoneButton$g={onLabel:"Micro",offLabel:"Micro",tooltipDisabledContent:"Micrófono deshabilitado",tooltipOnContent:"Silenciar micrófono",tooltipOffContent:"Reactivar micrófono",microphoneMenuTitle:"Micrófono",microphoneMenuTooltip:"Elegir micrófono",speakerMenuTitle:"Altavoz",speakerMenuTooltip:"Elegir altavoz",microphoneButtonSplitRoleDescription:"Botón de expansión",onSplitButtonAriaLabel:"Desactivar audio del micrófono",offSplitButtonAriaLabel:"Reactivar audio del micrófono",microphoneActionTurnedOnAnnouncement:"El micrófono se ha activado",microphoneActionTurnedOffAnnouncement:"El micrófono se ha desactivado",offSplitButtonMicrophonePrimaryAction:"Reactivar audio del micrófono",onSplitButtonMicrophonePrimaryAction:"Desactivar audio del micrófono",microphonePrimaryActionSplitButtonTitle:"Usar micrófono",microphoneAriaDescription:"Opciones de audio",deepNoiseSuppressionTitle:"Supresión de ruido",deepNoiseSuppressionOnAnnouncement:"Se ha activado la supresión profunda de ruido",deepNoiseSuppressionOffAnnouncement:"Se ha desactivado la supresión profunda de ruido"};var devicesButton$g={label:"Dispositivos",tooltipContent:"Administrar dispositivos",cameraMenuTitle:"Cámara",cameraMenuTooltip:"Elegir cámara",audioDeviceMenuTitle:"Dispositivo de audio",audioDeviceMenuTooltip:"Elige un dispositivo de audio",microphoneMenuTitle:"Micrófono",microphoneMenuTooltip:"Elegir micrófono",speakerMenuTitle:"Altavoz",speakerMenuTooltip:"Elegir altavoz"};var participantsButton$g={label:"Contactos",ariaLabel:"Personas",tooltipContent:"Mostrar participantes",menuHeader:"En esta llamada",participantsListButtonLabel:"{numParticipants} personas",muteAllButtonLabel:"Silenciar a todos",copyInviteLinkButtonLabel:"Copiar vínculo de invitación",copyInviteLinkActionedAriaLabel:"Vínculo de invitación copiado"};var screenShareButton$g={onLabel:"Dejar de presentar",offLabel:"Presentar",tooltipDisabledContent:"La presentación está deshabilitada",tooltipOnContent:"Presentando tu pantalla",tooltipOffContent:"Presentar tu pantalla"};var raiseHandButton$g={offLabel:"Levantar",onLabel:"Bajar",tooltipDisabledContent:"La acción Levantar la mano está deshabilitada",tooltipOnContent:"Bajar la mano",tooltipOffContent:"Levantar la mano"};var reactionButton$g={label:"Reaccionar",ariaLabel:"Reaccionar",tooltipDisabledContent:"La acción de reacción está deshabilitada",tooltipContent:"Enviar una reacción",likeReactionTooltipContent:"Me gusta",heartReactionTooltipContent:"Amor",laughReactionTooltipContent:"Risa",applauseReactionTooltipContent:"Aplauso",surprisedReactionTooltipContent:"Sorprendido"};var realTimeText$g={isTypingText:"Escritura en tiempo real"};var captionsSettingsModal$g={captionsSettingsModalTitle:"Configuración de subtítulos",captionsSettingsSpokenLanguageDropdownLabel:"Idioma hablado",captionsSettingsCaptionLanguageDropdownLabel:"Idioma de subtítulos",captionsSettingsSpokenLanguageDropdownInfoText:"Idioma en el que están hablando todos los usuarios de esta llamada.",captionsSettingsCaptionLanguageDropdownInfoText:"Los subtítulos aparecerán en este idioma.",captionsSettingsConfirmButtonLabel:"Confirmar",captionsSettingsCancelButtonLabel:"Cancelar",captionsSettingsModalAriaLabel:"Configuración modal de subtítulos",captionsSettingsCloseModalButtonAriaLabel:"Cerrar configuración de subtítulos"};var spokenLanguages$g={"ar-ae":"Árabe - Emiratos Árabes Unidos","ar-sa":"Árabe (Arabia Saudí)","da-dk":"Danés","de-de":"Alemán (Alemania)","en-au":"Inglés (Australia)","en-ca":"Inglés (Canadá)","en-gb":"Inglés (Reino Unido)","en-in":"Inglés (India)","en-nz":"Inglés (Nueva Zelanda)","en-us":"Inglés (Estados Unidos)","es-es":"Español - España (alfabetización internacional)","es-mx":"Español (México)","fi-fi":"Finés","fr-ca":"Francés (Canadá)","fr-fr":"Francés (Francia)","hi-in":"Hindi","it-it":"Italiano (Italia)","ja-jp":"Japonés","ko-kr":"Coreano","nb-no":"Noruego (Bokmål)","nl-be":"Neerlandés (Bélgica)","nl-nl":"Neerlandés (Países Bajos)","pl-pl":"Polaco","pt-br":"Portugués (Brasil)","ru-ru":"Ruso","sv-se":"Sueco","zh-cn":"Chino - República Popular China","zh-hk":"Chino (RAE de Hong Kong)","cs-cz":"Checo","pt-pt":"Portugués (Portugal)","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Tailandés","he-il":"Hebreo","cy-gb":"Galés","uk-ua":"Ucraniano","el-gr":"Griego","hu-hu":"Húngaro","ro-ro":"Rumano","sk-sk":"Eslovaco","zh-tw":"Chino (Taiwán)"};var captionLanguages$g={ar:"Árabe",da:"Danés",de:"Alemán",en:"Inglés",es:"Español",fi:"Finés","fr-ca":"Francés (Canadá)",fr:"Francés (Francia)",hi:"Hindi",it:"Italiano",ja:"Japonés",ko:"Coreano",nb:"Noruego (Bokmål)",nl:"Neerlandés",pl:"Polaco",pt:"Portugués (Brasil)",ru:"Ruso",sv:"Sueco","zh-Hans":"Chino (simplificado)","zh-Hant":"Chino (tradicional)",cs:"Checo","pt-pt":"Portugués (Portugal)",tr:"Turco",vi:"Vietnamita",th:"Tailandés",he:"Hebreo",cy:"Galés",uk:"Ucraniano",el:"Griego",hu:"Húngaro",ro:"Rumano",sk:"Eslovaco"};var startCaptionsButton$g={onLabel:"Activar subtítulos",offLabel:"Desactivar subtítulos",tooltipOnContent:"Desactivar subtítulos",tooltipOffContent:"Activar subtítulos"};var startRealTimeTextButton$g={label:"Activar RTT para esta llamada",tooltipOffContent:"Activar RTT para esta llamada"};var messageThread$g={yesterday:"Ayer",sunday:"Domingo",monday:"Lunes",tuesday:"Martes",wednesday:"Miércoles",thursday:"Jueves",friday:"Viernes",saturday:"Sábado",participantJoined:"se unió al chat.",participantLeft:"abandonó el chat.",editMessage:"Editar",removeMessage:"Eliminar",resendMessage:"Intenta enviar de nuevo",failToSendTag:"Error al enviar",editedTag:"Editado",liveAuthorIntro:"Mensaje de {author}",editedMessageLiveAuthorIntro:"Mensaje editado de {author}",editedMessageLocalUserLiveAuthorIntro:"Mensaje editado",messageContentAriaText:"{status} {author} dijo {message}",messageContentMineAriaText:"{status} Tú dijiste {message}",editBoxTextLimit:"Tu mensaje supera el límite de {limitNumber} caracteres",editBoxPlaceholderText:"Editar el mensaje",newMessagesIndicator:"Mensajes nuevos",noDisplayNameSub:"Sin nombre",editBoxCancelButton:"Cancelar",editBoxSubmitButton:"Listo",messageReadCount:"Leído por {messageReadByCount} de {remoteParticipantsCount}",actionMenuMoreOptions:"Más opciones",downloadAttachment:"Descargar archivo",openAttachment:"Abrir archivo",blockedWarningText:"Este mensaje se eliminó debido a la directiva de la organización.",blockedWarningLinkText:"Detalles",attachmentCardGroupMessage:"El mensaje tiene {attachmentCount} archivo adjunto",messageDeletedAnnouncementAriaLabel:"El mensaje se eliminó",richTextBoldTooltip:"Negrita",richTextItalicTooltip:"Cursiva",richTextUnderlineTooltip:"Subrayado",richTextBulletListTooltip:"Lista con viñetas",richTextNumberListTooltip:"Lista numerada",richTextIncreaseIndentTooltip:"Aumentar sangría",richTextDecreaseIndentTooltip:"Reducir sangría",richTextInsertTableTooltip:"Insertar tabla",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Insertar",richTextInsertTableMenuTitle:"Insertar tabla de {column} x {row}",richTextInsertRowAboveMenu:"Insertar arriba",richTextInsertRowBelowMenu:"Insertar debajo",richTextInsertColumnLeftMenu:"Insertar a la izquierda",richTextInsertColumnRightMenu:"Insertar a la derecha",richTextDeleteRowOrColumnMenu:"Eliminar",richTextDeleteColumnMenu:"Eliminar columna",richTextDeleteRowMenu:"Eliminar fila",richTextDeleteTableMenu:"Eliminar tabla",richTextToolbarMoreButtonAriaLabel:"Más opciones",attachmentUploadsPendingError:"El archivo se está cargando. Espera.",imageUploadsPendingError:"La imagen se está cargando. Espera.",richTextNewBulletedListItemAnnouncement:"Viñeta, nueva línea",richTextNewNumberedListItemAnnouncement:"{0}, nueva línea",richTextBulletedListAppliedAnnouncement:"Lista con viñetas aplicada",richTextNumberedListAppliedAnnouncement:"Lista numerada aplicada"};var errorBar$g={unableToReachChatService:"Estás sin conexión",accessDenied:"No se puede acceder a los servicios de chat. Comprueba las credenciales de usuario proporcionadas.",userNotInChatThread:"Ya no estás en este hilo de chat",sendMessageNotInChatThread:"No se pudo enviar el mensaje porque ya no estás en este hilo de chat",sendMessageGeneric:"Error al enviar el mensaje",callingNetworkFailure:"Problema al conectar la llamada: parece que no hay conexión",startVideoGeneric:"No se pudo iniciar el video",stopVideoGeneric:"No se pudo detener el video",muteGeneric:"No se pudo silenciar el micrófono",unmuteGeneric:"No se pudo reactivar el micrófono",speakingWhileMuted:"El micrófono está desactivado",startScreenShareGeneric:"Hubo un problema al iniciar la pantalla compartida.",stopScreenShareGeneric:"No se pudo detener la pantalla compartida",callNetworkQualityLow:"La calidad de la red es baja.",teamsMeetingCallNetworkQualityLow:"La calidad de la red es baja.",callNoSpeakerFound:"No se encontraron altavoces ni auriculares. Conecta un dispositivo de audio para escuchar la llamada.",callNoMicrophoneFound:"No se encontraron micrófonos. Conecta un dispositivo de entrada de audio.",callMicrophoneAccessDenied:"No se puede acceder al micrófono. Haz clic en el candado de la barra de direcciones para conceder permiso a esta página web.",callMicrophoneAccessDeniedSafari:"No se puede acceder al micrófono. Actualiza la página para habilitar permisos o comprueba la configuración de este navegador y que los permisos estén habilitados para este sitio web.",callMicrophoneMutedBySystem:"Tu sistema ha silenciado el audio.",callMicrophoneUnmutedBySystem:"Tu micrófono se ha recuperado y el sistema ha activado el audio.",callMacOsMicrophoneAccessDenied:"No se puede acceder al micrófono. Concede permiso al micrófono en la configuración de privacidad de macOS.",callLocalVideoFreeze:"El ancho de banda de red es deficiente. Es posible que el video aparezca pausado para otros usuarios en la llamada.",callCameraAccessDenied:"No se puede acceder a la cámara. Haz clic en el candado de la barra de direcciones para conceder permiso a esta página web.",callCameraAccessDeniedSafari:"No se puede acceder a la cámara. Actualiza la página para habilitar permisos o comprueba la configuración de este navegador y que los permisos estén habilitados para este sitio web.",callCameraAlreadyInUse:"No se puede acceder a la cámara. Puede que ya la esté usando otra aplicación.",callVideoStoppedBySystem:"El sistema ha detenido el video.",callVideoRecoveredBySystem:"El video se ha reanudado.",callMacOsCameraAccessDenied:"MacOS está bloqueando el acceso a la cámara. Actualiza la configuración de privacidad para permitir que este navegador acceda a la cámara.",callMacOsScreenShareAccessDenied:"MacOS está bloqueando el uso compartido de la pantalla. Actualiza la configuración de privacidad para permitir que este navegador grabe la pantalla.",dismissButtonAriaLabel:"Cerrar",failedToJoinCallGeneric:"No es posible unirse a la llamada.",failedToJoinCallInvalidMeetingLink:"No es posible unirse a la reunión. Vínculo no válido.",cameraFrozenForRemoteParticipants:"Los usuarios de la llamada tienen problemas para ver el video. Comprueba los dispositivos y la red.",unableToStartVideoEffect:"No se puede aplicar el efecto de video.",startSpotlightWhileMaxParticipantsAreSpotlighted:"No se puede iniciar el primer plano en los participantes porque el número máximo de participantes ya está puesto en primer plano.",mutedByRemoteParticipant:"Otro participante te ha silenciado."};var notificationStack$g={callingNetworkFailure:{title:"Problema al conectar la llamada",message:"Parece que estás desconectado. Comprueba la conexión de red e inténtalo de nuevo.",dismissButtonAriaLabel:"Cerrar"},startVideoGeneric:{title:"No se pudo iniciar el video.",dismissButtonAriaLabel:"Cerrar"},stopVideoGeneric:{title:"No se pudo detener el video.",dismissButtonAriaLabel:"Cerrar"},muteGeneric:{title:"No se pudo silenciar el micrófono.",dismissButtonAriaLabel:"Cerrar"},unmuteGeneric:{title:"No se pudo reactivar el micrófono.",dismissButtonAriaLabel:"Cerrar"},speakingWhileMuted:{title:"El audio está desactivado.",dismissButtonAriaLabel:"Cerrar"},startScreenShareGeneric:{title:"Hubo un problema al iniciar la pantalla compartida.",dismissButtonAriaLabel:"Cerrar"},stopScreenShareGeneric:{title:"No se pudo detener la pantalla compartida.",dismissButtonAriaLabel:"Cerrar"},callNetworkQualityLow:{title:"La calidad de la red es baja.",dismissButtonAriaLabel:"Cerrar"},teamsMeetingCallNetworkQualityLow:{title:"La calidad de la red es baja.",dismissButtonAriaLabel:"Cerrar",message:"Únete a esta llamada desde el teléfono para obtener un mejor sonido. Puedes seguir viendo la reunión en este dispositivo.",primaryButtonLabel:"Unirse por Teléfono"},callNoSpeakerFound:{title:"No se encontraron altavoces ni auriculares",message:"Conecta un dispositivo de audio para escuchar la llamada.",dismissButtonAriaLabel:"Cerrar"},callNoMicrophoneFound:{title:"No se han encontrado micrófonos",message:"Conecta un dispositivo de entrada de audio.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneAccessDenied:{title:"No se puede acceder al micrófono.",message:"Haz clic en el candado de la barra de direcciones para conceder permiso a esta página web.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneAccessDeniedSafari:{title:"No se puede acceder al micrófono.",message:"Actualiza la página para habilitar permisos o comprueba la configuración de este navegador y que los permisos estén habilitados para este sitio web.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneMutedBySystem:{title:"Tu sistema ha silenciado el audio.",dismissButtonAriaLabel:"Cerrar"},callMicrophoneUnmutedBySystem:{title:"Tu micrófono se ha recuperado y el sistema ha activado el audio.",dismissButtonAriaLabel:"Cerrar"},callMacOsMicrophoneAccessDenied:{title:"No se puede acceder al micrófono.",message:"Concede permiso al micrófono en la configuración de privacidad de macOS.",dismissButtonAriaLabel:"Cerrar"},callLocalVideoFreeze:{title:"El ancho de banda de red es deficiente",message:"Es posible que el video aparezca pausado para otros usuarios en la llamada.",dismissButtonAriaLabel:"Cerrar"},callCameraAccessDenied:{title:"No se puede acceder a la cámara",message:"Haz clic en el candado de la barra de direcciones para conceder permiso a esta página web.",dismissButtonAriaLabel:"Cerrar"},callCameraAccessDeniedSafari:{title:"No se puede acceder a la cámara",message:"Actualiza la página para habilitar permisos o comprueba la configuración de este navegador y que los permisos estén habilitados para este sitio web.",dismissButtonAriaLabel:"Cerrar"},callCameraAlreadyInUse:{title:"No se puede acceder a la cámara",message:"Puede que ya la esté usando otra aplicación.",dismissButtonAriaLabel:"Cerrar"},callVideoStoppedBySystem:{title:"El sistema ha detenido el video.",dismissButtonAriaLabel:"Cerrar"},callVideoRecoveredBySystem:{title:"El video se ha reanudado.",dismissButtonAriaLabel:"Cerrar"},callMacOsCameraAccessDenied:{title:"MacOS está bloqueando el acceso a la cámara",message:"Actualiza la configuración de privacidad para permitir que este navegador acceda a la cámara.",dismissButtonAriaLabel:"Cerrar"},callMacOsScreenShareAccessDenied:{title:"MacOS está bloqueando la pantalla compartida",message:"Actualiza la configuración de privacidad para permitir que este navegador grabe la pantalla.",dismissButtonAriaLabel:"Cerrar"},failedToJoinCallGeneric:{title:"No es posible unirse a la llamada.",dismissButtonAriaLabel:"Cerrar"},failedToJoinCallInvalidMeetingLink:{title:"No es posible unirse a la reunión. Vínculo no válido.",dismissButtonAriaLabel:"Cerrar"},cameraFrozenForRemoteParticipants:{title:"Los usuarios de la llamada tienen problemas para ver el video",message:"Comprueba los dispositivos y la red.",dismissButtonAriaLabel:"Cerrar"},unableToStartVideoEffect:{title:"No se puede aplicar el efecto de video.",dismissButtonAriaLabel:"Cerrar"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"No se puede poner primer plano a los participantes",message:"Ya está puesto en primer plano el número máximo de participantes.",dismissButtonAriaLabel:"Cerrar"},mutedByRemoteParticipant:{title:"Otro participante te ha silenciado.",dismissButtonAriaLabel:"Cerrar"},recordingStarted:{title:"La grabación se ha iniciado.",message:"Al unirte, das tu consentimiento para que se transcriba esta reunión.",dismissButtonAriaLabel:"Cerrar"},transcriptionStarted:{title:"La transcripción se ha iniciado.",message:"Al unirte, das tu consentimiento para que se transcriba esta reunión.",dismissButtonAriaLabel:"Cerrar"},recordingStopped:{title:"Se está guardando la grabación.",message:"La grabación se ha detenido.",dismissButtonAriaLabel:"Cerrar"},transcriptionStopped:{title:"Se está guardando la transcripción.",message:"La transcripción se ha detenido.",dismissButtonAriaLabel:"Cerrar"},recordingAndTranscriptionStarted:{title:"Se han iniciado la grabación y la transcripción.",message:"Al unirte, das tu consentimiento para que se transcriba esta reunión.",dismissButtonAriaLabel:"Cerrar"},recordingAndTranscriptionStopped:{title:"Se están guardando la grabación y la transcripción.",message:"Se han detenido la grabación y la transcripción.",dismissButtonAriaLabel:"Cerrar"},recordingStoppedStillTranscribing:{title:"La grabación se ha detenido.",message:"Solo estás transcribiendo esta reunión.",dismissButtonAriaLabel:"Cerrar"},transcriptionStoppedStillRecording:{title:"La transcripción se ha detenido.",message:"Solo estás grabando esta reunión.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomOpened:{title:"Te moveremos a la sala asignada en 5 segundos.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomChanged:{title:"La sala para sesión de subgrupo cambió",message:"Te moveremos a la sala asignada en 5 segundos.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomClosed:{title:"La sala para sesión de subgrupo se cerró",message:"Volveremos a la reunión en 5 segundos.",dismissButtonAriaLabel:"Cerrar"},assignedBreakoutRoomOpenedPromptJoin:{title:"¿Quieres unirte a la sala para sesión de subgrupo?",message:"Se te ha asignado a una sala para sesión de subgrupo.",dismissButtonAriaLabel:"Cerrar",primaryButtonLabel:"Unirse a la sala",secondaryButtonLabel:"Más tarde"},breakoutRoomJoined:{title:"Te has unido a una sala para sesión de subgrupo.",dismissButtonAriaLabel:"Cerrar"},breakoutRoomClosingSoon:{title:"El límite de tiempo de sala está por expirar.",message:"Esta sala se cerrará en 30 segundos",dismissButtonAriaLabel:"Cerrar"},capabilityTurnVideoOnPresent:{title:"La cámara se habilitó",message:"Activa esta opción si quieres compartir video.",dismissButtonAriaLabel:"Cerrar"},capabilityTurnVideoOnAbsent:{title:"Se deshabilitó la cámara",message:"Ya no puedes compartir video.",dismissButtonAriaLabel:"Cerrar"},capabilityUnmuteMicPresent:{title:"El micrófono se habilitó",message:"Reactiva el audio si quieres hablar.",dismissButtonAriaLabel:"Cerrar"},capabilityUnmuteMicAbsent:{title:"El micro se deshabilitó",message:"Ya no puedes activar el audio.",dismissButtonAriaLabel:"Cerrar"},togetherModeStarted:{title:"El modo de colaboración inició",dismissButtonAriaLabel:"Cerrar"},togetherModeEnded:{title:"El modo de colaboración finalizó",dismissButtonAriaLabel:"Cerrar"},transcriptionError:{title:"Se produjo un error con la transcripción.",message:"La transcripción se detuvo. Inténtalo de nuevo más tarde",dismissButtonAriaLabel:"Cerrar"},transcriptionStartedByYou:{title:"La transcripción se inició.",message:"Iniciado por ti. Informar a todos de que se les está transcribiendo.",dismissButtonAriaLabel:"Cerrar"}};var videoGallery$g={screenIsBeingSharedMessage:"Estás compartiendo tu pantalla",screenShareLoadingMessage:"Cargando la pantalla de {participant}",localScreenShareLoadingMessage:"Cargando la pantalla",localVideoLabel:"Tú",localVideoCameraSwitcherLabel:"Cambiar de cámara",localVideoMovementLabel:"Mosaico de video local móvil",localVideoMovementAriaLabel:"Mosaico de video local móvil",localVideoSelectedDescription:"{cameraName} seleccionada",displayNamePlaceholder:"Participante sin nombre",fitRemoteParticipantToFrame:"Ajustar al marco",fillRemoteParticipantFrame:"Marco de relleno",pinParticipantForMe:"Anclar para mí",pinParticipantForMeLimitReached:"Pin (límite alcanzado)",unpinParticipantForMe:"Desanclar",pinParticipantMenuItemAriaLabel:"Anclar {participantName}",unpinParticipantMenuItemAriaLabel:"Desanclar {participantName}",pinnedParticipantAnnouncementAriaLabel:"{participantName} anclado",unpinnedParticipantAnnouncementAriaLabel:"{participantName} desanclado",startSpotlightVideoTileMenuLabel:"Poner en primer plano para todos",addSpotlightVideoTileMenuLabel:"Agregar primer plano",spotlightLimitReachedMenuTitle:"Se alcanzó el límite de primer plano",stopSpotlightVideoTileMenuLabel:"Dejar de poner en primer plano",stopSpotlightOnSelfVideoTileMenuLabel:"Salir de primer plano",attendeeRole:"Asistente",muteParticipantMenuItemLabel:"Silenciar",waitingScreenText:"Esperando a que se unan otros usuarios",forbidAudio:"Deshabilitar micrófono",permitAudio:"Permitir micrófono",forbidAudioTileMenuLabel:"Deshabilitar micrófono",permitAudioTileMenuLabel:"Permitir micrófono",forbidVideo:"Deshabilitar cámara",permitVideo:"Permitir cámara",forbidVideoTileMenuLabel:"Deshabilitar cámara",permitVideoTileMenuLabel:"Permitir cámara"};var dialpad$g={placeholderText:"Escribir un número de teléfono",deleteButtonAriaLabel:"Eliminar"};var holdButton$g={onLabel:"Reanudar",offLabel:"En espera",tooltipOnContent:"Reanudar llamada",tooltipOffContent:"Poner llamada en espera"};var videoTile$g={participantStateRinging:"Llamando...",participantReconnecting:"Reconectando...",participantStateHold:"En espera",moreOptionsButtonAriaLabel:"Más opciones {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"audio desactivado",moreOptionsParticipantMutedStateUnmutedAriaLabel:"se reactivaron las notificaciones",moreOptionsParticipantHandRaisedAriaLabel:"Mano levantada",moreOptionsParticipantIsSpeakingAriaLabel:"Hablando",moreOptionsParticipantMicDisabledAriaLabel:"Micrófono deshabilitado",moreOptionsParticipantCameraDisabledAriaLabel:"Cámara deshabilitada"};var CameraAndMicrophoneSitePermissionsRequest$g={primaryText:"Permitir que {appName} usen la cámara y el micrófono",secondaryText:"Esto es para que los participantes puedan verte y oírte.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara ni micrófono",ariaLabel:"Permitir el acceso a la cámara y al micrófono"};var CameraSitePermissionsRequest$g={primaryText:"Permitir que {appName} use la cámara",secondaryText:"Esto es para que los participantes puedan verte.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara",ariaLabel:"Permitir el acceso a la cámara"};var MicrophoneSitePermissionsRequest$g={primaryText:"Permitir que {appName} use el micrófono",secondaryText:"Esto es para que los participantes puedan oírte.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin micrófono",ariaLabel:"Permitir el acceso al micrófono"};var CameraAndMicrophoneSitePermissionsCheck$g={primaryText:"Comprobando el acceso a la cámara y al micrófono",secondaryText:"Permite el acceso si se solicita. Esto es para que los participantes puedan verte y oírte.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara ni micrófono",ariaLabel:"Comprobando el acceso a la cámara y al micrófono. Permite el acceso si se solicita."};var CameraSitePermissionsCheck$g={primaryText:"Comprobando el acceso a la cámara",secondaryText:"Permite el acceso si se solicita. Esto es para que los participantes puedan verte.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin cámara",ariaLabel:"Comprobando el acceso a la cámara. Permite el acceso si se solicita."};var MicrophoneSitePermissionsCheck$g={primaryText:"Comprobando el acceso al micrófono",secondaryText:"Permite el acceso si se solicita. Esto es para que los participantes puedan oírte.",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",primaryButtonText:"Continuar sin micrófono",ariaLabel:"Comprobando el acceso al micrófono. Permite el acceso si se solicita."};var CameraAndMicrophoneSitePermissionsDenied$g={primaryText:"No se puede permitir el acceso a la cámara ni al micrófono.",secondaryText:"Haz clic en el ícono de bloqueo de la barra de direcciones para conceder permisos de micrófono a esta página web. Es posible que sea necesario actualizar la página.",primaryButtonText:"Continuar sin cámara ni micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var CameraAndMicrophoneSitePermissionsDeniedSafari$g={primaryText:"No se puede permitir el acceso a la cámara ni al micrófono.",secondaryText:"Actualiza la página para habilitar permisos o comprueba la configuración de este navegador y que los permisos estén habilitados para este sitio web.",primaryButtonText:"Continuar sin cámara ni micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var CameraSitePermissionsDenied$g={primaryText:"No se puede acceder a la cámara",secondaryText:"Haz clic en el ícono de bloqueo de la barra de direcciones para conceder permisos de cámara a esta página web. Es posible que sea necesario actualizar la página.",primaryButtonText:"Continuar sin cámara",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var MicrophoneSitePermissionsDenied$g={primaryText:"No se puede acceder al micrófono.",secondaryText:"Haz clic en el ícono de bloqueo de la barra de direcciones para conceder permisos de micrófono a esta página web. Es posible que sea necesario actualizar la página.",primaryButtonText:"Continuar sin micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var CameraSitePermissionsDeniedSafari$g={primaryText:"No se puede acceder a la cámara",secondaryText:"Actualiza la página para habilitar permisos o comprueba la configuración de este navegador y que los permisos estén habilitados para este sitio web.",primaryButtonText:"Continuar sin cámara",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var MicrophoneSitePermissionsDeniedSafari$g={primaryText:"No se puede acceder al micrófono.",secondaryText:"Actualiza la página para habilitar permisos o comprueba la configuración de este navegador y que los permisos estén habilitados para este sitio web.",primaryButtonText:"Continuar sin micrófono",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var UnsupportedBrowser$g={primaryText:"Navegador no compatible",secondaryText:"Únete a esta llamada con un navegador compatible.",moreHelpLinkText:"Ver los requisitos de compatibilidad"};var UnsupportedBrowserVersion$g={primaryText:"Es necesario actualizar el navegador",secondaryText:"Para garantizar la mejor llamada posible, actualiza el navegador e intenta unirte de nuevo a la llamada.",moreHelpLinkText:"Ver los requisitos de compatibilidad",continueAnywayButtonText:"Iniciar llamada sin actualizar"};var UnsupportedOperatingSystem$g={primaryText:"Sistema operativo no compatible",secondaryText:"Únete a esta llamada desde un dispositivo con un sistema operativo compatible.",moreHelpLinkText:"Ver los requisitos de compatibilidad"};var BrowserPermissionDenied$g={primaryText:"No se puede usar la cámara o el micrófono",secondaryText:"Es posible que el navegador no tenga acceso a la cámara o al micrófono. Para solucionar este problema, abre Preferencias del sistema.",primaryButtonText:"Intentar de nuevo",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas"};var BrowserPermissionDeniedIOS$g={primaryText:"Permite el acceso al micrófono para continuar",secondaryText:"Para que otros participantes puedan oírte.",primaryButtonText:"Intentar de nuevo",imageAltText:"Ubicación de permisos del dispositivo de micrófono y cámara para iOS",linkText:"¿Necesitas ayuda? Obtén ayuda para la solución de problemas",step1Text:"Ve a la aplicación de Configuración",step2Text:"Desplázate hacia abajo hasta la configuración de este navegador",step3Text:"Activar micrófono (cámara opcional)",step4Text:"Intenta unirte de nuevo a la llamada",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$g={leftNavButtonAriaLabel:"página anterior {current} de {total}",rightNavButtonAriaLabel:"página siguiente {current} de {total}"};var meetingConferencePhoneInfo$g={meetingConferencePhoneInfoModalTitle:"Unirse a la llamada desde el teléfono",meetingConferencePhoneInfoModalDialIn:"Marcar número",meetingConferencePhoneInfoModalMeetingId:"Escribir el id. de reunión",meetingConferencePhoneInfoModalWait:"Espera a ser admitido por el organizador",meetingConferencePhoneInfoModalNoPhoneAvailable:"No hay teléfono disponible",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Número gratuito)",meetingConferencePhoneInfoModalToll:"(Número de pago)"};var IncomingCallNotification$g={incomingCallNotificationPlaceholderId:"Usuario desconocido",incomingCallNotificationPlaceholderAlert:"{callerName} está llamando",incomingCallNoticicationAcceptWithAudioAriaLabel:"Aceptar con audio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Aceptar con video",incomingCallNoticicationRejectAriaLabel:"Rechazar",incomingCallNotificationAcceptButtonLabel:"Aceptar",incomingCallNotificationRejectButtonLabel:"Rechazar",incomingCallNotificationAccceptWithVideoButtonLabel:"Aceptar con vídeo",incomingCallNotificationDismissButtonAriaLabel:"Descartar"};var es_MX$1 = {participantItem:participantItem$g,ParticipantList:ParticipantList$g,typingIndicator:typingIndicator$g,sendBox:sendBox$g,richTextSendBox:richTextSendBox$g,realTimeTextModal:realTimeTextModal$g,captionsBanner:captionsBanner$g,mentionPopover:mentionPopover$g,imageOverlay:imageOverlay$g,messageStatusIndicator:messageStatusIndicator$g,endCallButton:endCallButton$g,cameraButton:cameraButton$g,microphoneButton:microphoneButton$g,devicesButton:devicesButton$g,participantsButton:participantsButton$g,screenShareButton:screenShareButton$g,raiseHandButton:raiseHandButton$g,reactionButton:reactionButton$g,realTimeText:realTimeText$g,captionsSettingsModal:captionsSettingsModal$g,spokenLanguages:spokenLanguages$g,captionLanguages:captionLanguages$g,startCaptionsButton:startCaptionsButton$g,startRealTimeTextButton:startRealTimeTextButton$g,messageThread:messageThread$g,errorBar:errorBar$g,notificationStack:notificationStack$g,videoGallery:videoGallery$g,dialpad:dialpad$g,holdButton:holdButton$g,videoTile:videoTile$g,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$g,CameraSitePermissionsRequest:CameraSitePermissionsRequest$g,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$g,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$g,CameraSitePermissionsCheck:CameraSitePermissionsCheck$g,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$g,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$g,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$g,CameraSitePermissionsDenied:CameraSitePermissionsDenied$g,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$g,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$g,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$g,UnsupportedBrowser:UnsupportedBrowser$g,UnsupportedBrowserVersion:UnsupportedBrowserVersion$g,UnsupportedOperatingSystem:UnsupportedOperatingSystem$g,BrowserPermissionDenied:BrowserPermissionDenied$g,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$g,verticalGallery:verticalGallery$g,meetingConferencePhoneInfo:meetingConferencePhoneInfo$g,IncomingCallNotification:IncomingCallNotification$g};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Spanish (Mexico).
 *
 * @public
 */
const COMPONENT_LOCALE_ES_MX = {
    strings: createComponentStrings(es_MX$1)
};

var participantItem$f={isMeText:"(sinä)",menuTitle:"Lisää vaihtoehtoja",removeButtonLabel:"Poista",sharingIconLabel:"Jakaminen",mutedIconLabel:"Mykistetty",handRaisedIconLabel:"Käsi ylhäällä – paikka {position}",displayNamePlaceholder:"Nimetön osallistuja",participantStateRinging:"Soitetaan...",participantStateHold:"Pidossa",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, lisää asetuksia",attendeeRole:"Osallistuja",micDisabledIconLabel:"Mikrofoni poistettu käytöstä",cameraDisabledIconLabel:"Kamera poistettu käytöstä"};var ParticipantList$f={overflowParticipantCount:"+{overflowCount} lisää"};var typingIndicator$f={singleUser:"{user} käyttäjä kirjoittaa parhaillaan...",multipleUsers:"{users} käyttäjää kirjoittaa parhaillaan...",multipleUsersAbbreviateOne:"{users} ja yksi muu käyttäjä kirjoittaa parhaillaan...",multipleUsersAbbreviateMany:"{users} ja {numOthers} muuta käyttäjää kirjoittaa parhaillaan...",delimiter:", "};var sendBox$f={placeholderText:"Kirjoita viesti",textTooLong:"Viestin pituus ylittää enimmäispituuden.",sendButtonAriaLabel:"Lähetä viesti",attachmentUploadsPendingError:"Ladataan palvelimeen... Odota hetki.",removeAttachment:"Poista tiedosto",uploading:"Ladataan palvelimeen",uploadCompleted:"Ladattu palvelimeen",attachmentMoreMenu:"Lisää asetuksia"};var richTextSendBox$f={placeholderText:"Kirjoita viesti",textTooLong:"Viestin pituus ylittää enimmäispituuden.",sendButtonAriaLabel:"Lähetä viesti",attachmentUploadsPendingError:"Ladataan palvelimeen... Odota hetki.",imageUploadsPendingError:"Kuvaa ladataan palvelimeen, odota.",removeAttachment:"Poista tiedosto",uploading:"Ladataan",uploadCompleted:"Lataus valmis",richTextBoldTooltip:"Lihavointi",richTextItalicTooltip:"Kursivointi",richTextUnderlineTooltip:"Alleviivaa",richTextBulletListTooltip:"Luettelomerkeillä varustettu luettelo",richTextNumberListTooltip:"Numeroitu luettelo",richTextIncreaseIndentTooltip:"Suurenna sisennystä",richTextDecreaseIndentTooltip:"Pienennä sisennystä",richTextInsertTableTooltip:"Lisää taulukko",richTextFormatButtonTooltip:"Muoto",richTextInsertRowOrColumnMenu:"Lisää",richTextInsertTableMenuTitle:"Lisää {row}x{column}-kokoinen taulukko",richTextInsertRowAboveMenu:"Lisää yläpuolelle",richTextInsertRowBelowMenu:"Lisää alapuolelle",richTextInsertColumnLeftMenu:"Lisää vasemmalle",richTextInsertColumnRightMenu:"Lisää oikealle",richTextDeleteRowOrColumnMenu:"Poista",richTextDeleteColumnMenu:"Poista sarake",richTextDeleteRowMenu:"Poista rivi",richTextDeleteTableMenu:"Poista taulukko",richTextToolbarAriaLabel:"Viestin muotoilun työkalurivi",richTextToolbarMoreButtonAriaLabel:"Lisää asetuksia",attachmentMoreMenu:"Lisää asetuksia",richTextNewBulletedListItemAnnouncement:"Luettelomerkki, uusi rivi",richTextNewNumberedListItemAnnouncement:"{0}, uusi rivi",richTextBulletedListAppliedAnnouncement:"Luettelomerkeillä varustettu luettelo käytössä",richTextNumberedListAppliedAnnouncement:"Numeroitu luettelo käytössä"};var realTimeTextModal$f={realTimeTextModalAriaLabel:"Ota modaalinen RTT-valintaikkuna käyttöön",realTimeTextModalTitle:"Otetaanko RTT käyttöön?",realTimeTextModalText:"Reaaliaikainen teksti (RTT) otetaan käyttöön kaikille kokouksen osallistuville.",realTimeTextConfirmButtonLabel:"Ota käyttöön",realTimeTextCancelButtonLabel:"Peruuta",realTimeTextCloseModalButtonAriaLabel:"Sulje modaalinen RTT-valintaikkuna"};var captionsBanner$f={captionsBannerSpinnerText:"Aloitetaan tekstitystä...",realTimeTextInputBoxDefaultText:"Kirjoita viesti reaaliaikaisesti",realTimeTextInputErrorMessage:"Viestissä voi olla enintään 2 000 merkkiä",realTimeTextBannerTitle:"Reaaliaikainen kirjoitus",realTimeTextBannerContent:"RTT (reaaliaikainen teksti) on käytössä kaikille osallistujille koko kokouksen keston ajan.",realTimeTextBannerLinkLabel:"Lue lisää",captionsOnlyContainerTitle:"Tekstitykset",realTimeTextOnlyContainerTitle:"Reaaliaikainen teksti",captionsAndRealTimeTextContainerTitle:"Tekstitykset ja reaaliaikainen teksti",expandButtonAriaLabel:"Laajenna",minimizeButtonAriaLabel:"Kutista"};var mentionPopover$f={mentionPopoverHeader:"Ehdotuksia"};var imageOverlay$f={downloadButtonLabel:"Lataa",dismissButtonAriaLabel:"Sulje",overlayVisibleMessage:"Kuvan tietojen peittokuva"};var messageStatusIndicator$f={deliveredAriaLabel:"Viesti on lähetetty",deliveredTooltipText:"Lähetetty",seenAriaLabel:"Muut ovat nähneet viestin",seenTooltipText:"Nähty",readByTooltipText:"{messageThreadReadCount}/{remoteParticipantsCount} osallistujaa on lukenut viestin",sendingAriaLabel:"Viestiä lähetetään",sendingTooltipText:"Lähetetään",failedToSendAriaLabel:"Viestin lähettäminen epäonnistui",failedToSendTooltipText:"Lähettäminen epäonnistui"};var endCallButton$f={label:"Poistu",tooltipContent:"Poistu puhelusta",leaveOption:"Poistu",endCallOption:"Lopeta kaikkien puhelu"};var cameraButton$f={onLabel:"Kamera",offLabel:"Kamera",tooltipDisabledContent:"Kamera on poissa käytöstä",tooltipOnContent:"Poista kamera käytöstä",tooltipOffContent:"Käynnistä kamera",tooltipVideoLoadingContent:"Videota ladataan",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Valitse kamera",cameraButtonSplitRoleDescription:"Jaettu painike",onSplitButtonAriaLabel:"Poista kamera käytöstä",offSplitButtonAriaLabel:"Ota kamera käyttöön",cameraActionTurnedOnAnnouncement:"Kamerasi on otettu käyttöön",cameraActionTurnedOffAnnouncement:"Kamerasi on poistettu käytöstä",onSplitButtonPrimaryActionCamera:"Poista kamera käytöstä",offSplitButtonPrimaryActionCamera:"Käynnistä kamera",cameraPrimaryActionSplitButtonTitle:"Käytä kameraa",videoEffectsMenuItemTitle:"Tehosteet",cameraButtonAriaDescription:"kamera-asetukset"};var microphoneButton$f={onLabel:"Mikrofoni",offLabel:"Mikrofoni",tooltipDisabledContent:"Mikrofoni on poissa käytöstä",tooltipOnContent:"Mykistä mikrofoni",tooltipOffContent:"Poista mikrofonin mykistys",microphoneMenuTitle:"Mikrofoni",microphoneMenuTooltip:"Valitse mikrofoni",speakerMenuTitle:"Kaiutin",speakerMenuTooltip:"Valitse kaiutin",microphoneButtonSplitRoleDescription:"Jaettu painike",onSplitButtonAriaLabel:"Mykistä mikrofoni",offSplitButtonAriaLabel:"Poista mikrofonin mykistys",microphoneActionTurnedOnAnnouncement:"Mikrofonisi on otettu käyttöön",microphoneActionTurnedOffAnnouncement:"Mikrofonisi on poistettu käytöstä",offSplitButtonMicrophonePrimaryAction:"Poista mikrofonin mykistys",onSplitButtonMicrophonePrimaryAction:"Mykistä mikrofoni",microphonePrimaryActionSplitButtonTitle:"Käytä mikrofonia",microphoneAriaDescription:"Ääniasetukset",deepNoiseSuppressionTitle:"Melunvaimennus",deepNoiseSuppressionOnAnnouncement:"Syvä melunvaimennus on otettu käyttöön",deepNoiseSuppressionOffAnnouncement:"Syvä melunvaimennus on poistettu käytöstä"};var devicesButton$f={label:"Laitteet",tooltipContent:"Hallitse laitteita",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Valitse kamera",audioDeviceMenuTitle:"Äänilaite",audioDeviceMenuTooltip:"Valitse äänilaite",microphoneMenuTitle:"Mikrofoni",microphoneMenuTooltip:"Valitse mikrofoni",speakerMenuTitle:"Kaiutin",speakerMenuTooltip:"Valitse kaiutin"};var participantsButton$f={label:"Ihmiset",ariaLabel:"Henkilöt",tooltipContent:"Näytä osallistujat",menuHeader:"Tässä puhelussa",participantsListButtonLabel:"{numParticipants} henkilöä",muteAllButtonLabel:"Mykistä kaikki",copyInviteLinkButtonLabel:"Kopioi kutsulinkki",copyInviteLinkActionedAriaLabel:"Kutsulinkki kopioitu"};var screenShareButton$f={onLabel:"Lopeta esitys",offLabel:"Esitä",tooltipDisabledContent:"Näytön jakaminen on poissa käytöstä",tooltipOnContent:"Näyttösi näytetään muille osallistujille",tooltipOffContent:"Näytä oma näyttösi"};var raiseHandButton$f={offLabel:"Nosta",onLabel:"Laske",tooltipDisabledContent:"Nosta käsi -toiminto on poistettu käytöstä",tooltipOnContent:"Laske käsi",tooltipOffContent:"Nosta käsi"};var reactionButton$f={label:"Reagoi",ariaLabel:"Reagoi",tooltipDisabledContent:"Reaktiotoiminto on poistettu käytöstä",tooltipContent:"Lähetä reaktio",likeReactionTooltipContent:"Tykkää",heartReactionTooltipContent:"Ihastu",laughReactionTooltipContent:"Naura",applauseReactionTooltipContent:"Suosionosoitus",surprisedReactionTooltipContent:"Yllättynyt"};var realTimeText$f={isTypingText:"RTT-kirjoitus"};var captionsSettingsModal$f={captionsSettingsModalTitle:"Tekstitysasetukset",captionsSettingsSpokenLanguageDropdownLabel:"Puhuttu kieli",captionsSettingsCaptionLanguageDropdownLabel:"Tekstitysten kieli",captionsSettingsSpokenLanguageDropdownInfoText:"Kieli, jota kaikki tähän puheluun osallistuvat puhuvat.",captionsSettingsCaptionLanguageDropdownInfoText:"Tekstitys näkyy tällä kielellä.",captionsSettingsConfirmButtonLabel:"Vahvista",captionsSettingsCancelButtonLabel:"Peruuta",captionsSettingsModalAriaLabel:"Tekstitysasetus, modaalinen valintaikkuna",captionsSettingsCloseModalButtonAriaLabel:"Sulje tekstitysasetus"};var spokenLanguages$f={"ar-ae":"arabia – Arabiemiirikunnat","ar-sa":"arabia – Saudi-Arabia","da-dk":"tanska","de-de":"saksa – Saksa","en-au":"englanti – Australia","en-ca":"englanti – Kanada","en-gb":"englanti – Yhdistynyt kuningaskunta","en-in":"englanti – Intia","en-nz":"englanti – Uusi-Seelanti","en-us":"englanti – Yhdysvallat","es-es":"espanja – Espanja (nykykieli)","es-mx":"espanja – Meksiko","fi-fi":"suomi","fr-ca":"ranska – Kanada","fr-fr":"ranska – Ranska","hi-in":"hindi","it-it":"italia – Italia","ja-jp":"japani","ko-kr":"korea","nb-no":"norja (bokmål)","nl-be":"hollanti – Belgia","nl-nl":"hollanti – Alankomaat","pl-pl":"puola","pt-br":"portugali (Brasilia)","ru-ru":"venäjä","sv-se":"ruotsi","zh-cn":"kiina – Kiinan kansantasavalta","zh-hk":"kiina – Hongkong, erityishallintoalue","cs-cz":"tšekki","pt-pt":"portugali – Portugali","tr-tr":"turkki","vi-vn":"vietnam","th-th":"thai","he-il":"heprea","cy-gb":"kymri","uk-ua":"ukraina","el-gr":"kreikka","hu-hu":"unkari","ro-ro":"romania","sk-sk":"slovakki","zh-tw":"kiina – Taiwan"};var captionLanguages$f={ar:"arabia",da:"tanska",de:"saksa",en:"englanti",es:"espanja",fi:"suomi","fr-ca":"ranska – Kanada",fr:"ranska – Ranska",hi:"hindi",it:"italia",ja:"japani",ko:"korea",nb:"norja (bokmål)",nl:"hollanti",pl:"puola",pt:"portugali (Brasilia)",ru:"venäjä",sv:"ruotsi","zh-Hans":"kiina (yksinkertaistettu)","zh-Hant":"kiina (perinteinen)",cs:"tšekki","pt-pt":"portugali – Portugali",tr:"turkki",vi:"vietnam",th:"thai",he:"heprea",cy:"kymri",uk:"ukraina",el:"kreikka",hu:"unkari",ro:"romania",sk:"slovakki"};var startCaptionsButton$f={onLabel:"Ota tekstitykset käyttöön",offLabel:"Poista tekstitykset käytöstä",tooltipOnContent:"Poista tekstitykset käytöstä",tooltipOffContent:"Ota tekstitykset käyttöön"};var startRealTimeTextButton$f={label:"Ota reaaliaikainen kirjoitus käyttöön tässä puhelussa",tooltipOffContent:"Ota reaaliaikainen kirjoitus käyttöön tässä puhelussa"};var messageThread$f={yesterday:"eilen",sunday:"sunnuntaina",monday:"maanantaina",tuesday:"tiistaina",wednesday:"keskiviikkona",thursday:"torstaina",friday:"perjantaina",saturday:"lauantaina",participantJoined:"liittyi keskusteluun.",participantLeft:"poistui keskustelusta.",editMessage:"Muokkaa",removeMessage:"Poista",resendMessage:"Yritä lähettää viesti uudelleen",failToSendTag:"Lähettäminen epäonnistui",editedTag:"Muokattu",liveAuthorIntro:"Viesti lähettäjältä {author}",editedMessageLiveAuthorIntro:"Muokattu viesti lähettäjältä {author}",editedMessageLocalUserLiveAuthorIntro:"Muokattu viesti",messageContentAriaText:"{status} {author} sanoi {message}",messageContentMineAriaText:"{status} Sanoit {message}",editBoxTextLimit:"Viesti ylittää {limitNumber} merkin rajan",editBoxPlaceholderText:"Muokkaa viestiäsi",newMessagesIndicator:"Uudet viestit",noDisplayNameSub:"Ei nimeä",editBoxCancelButton:"Peruuta",editBoxSubmitButton:"Valmis",messageReadCount:"{messageReadByCount}/{remoteParticipantsCount} osallistujaa on lukenut viestin",actionMenuMoreOptions:"Lisää vaihtoehtoja",downloadAttachment:"Lataa tiedosto",openAttachment:"Avaa tiedosto",blockedWarningText:"Tämä viesti poistettiin organisaation käytännön vuoksi.",blockedWarningLinkText:"Lisätiedot",attachmentCardGroupMessage:"Viestissä on {attachmentCount} liitettä",messageDeletedAnnouncementAriaLabel:"Viesti on poistettu",richTextBoldTooltip:"Lihavoi",richTextItalicTooltip:"Kursivointi",richTextUnderlineTooltip:"Alleviivaus",richTextBulletListTooltip:"Luettelomerkeillä varustettu luettelo",richTextNumberListTooltip:"Numeroitu luettelo",richTextIncreaseIndentTooltip:"Suurenna sisennystä",richTextDecreaseIndentTooltip:"Pienennä sisennystä",richTextInsertTableTooltip:"Lisää taulukko",richTextFormatButtonTooltip:"Muoto",richTextInsertRowOrColumnMenu:"Lisää",richTextInsertTableMenuTitle:"Lisää {row}x{column}-kokoinen taulukko",richTextInsertRowAboveMenu:"Lisää yläpuolelle",richTextInsertRowBelowMenu:"Lisää alapuolelle",richTextInsertColumnLeftMenu:"Lisää vasemmalle",richTextInsertColumnRightMenu:"Lisää oikealle",richTextDeleteRowOrColumnMenu:"Poista",richTextDeleteColumnMenu:"Poista sarake",richTextDeleteRowMenu:"Poista rivi",richTextDeleteTableMenu:"Poista taulukko",richTextToolbarMoreButtonAriaLabel:"Lisää asetuksia",attachmentUploadsPendingError:"Ladataan palvelimeen... Odota hetki.",imageUploadsPendingError:"Kuvaa ladataan palvelimeen, odota.",richTextNewBulletedListItemAnnouncement:"Luettelomerkki, uusi rivi",richTextNewNumberedListItemAnnouncement:"{0}, uusi rivi",richTextBulletedListAppliedAnnouncement:"Luettelomerkeillä varustettu luettelo käytössä",richTextNumberedListAppliedAnnouncement:"Numeroitu luettelo käytössä"};var errorBar$f={unableToReachChatService:"Olet offline-tilassa",accessDenied:"Keskustelupalveluihin ei saada yhteyttä – tarkista annetut käyttäjän tunnistetiedot",userNotInChatThread:"Et ole enää mukana tässä keskusteluketjussa",sendMessageNotInChatThread:"Viestin lähettäminen epäonnistui, koska et ole enää tässä keskusteluketjussa",sendMessageGeneric:"Viestin lähettäminen epäonnistui",callingNetworkFailure:"Puhelun yhdistämisessä on ongelma. Vaikutat olevan offline-tilassa",startVideoGeneric:"Videon käynnistäminen epäonnistui",stopVideoGeneric:"Videon pysäyttäminen epäonnistui",muteGeneric:"Mikrofonin mykistäminen epäonnistui",unmuteGeneric:"Mikrofonin mykistyksen poistaminen epäonnistui",speakingWhileMuted:"Mikrofonisi on mykistetty",startScreenShareGeneric:"Näytön jakamisen aloittamisessa ilmeni ongelma.",stopScreenShareGeneric:"Näytön jakamisen lopettaminen epäonnistui",callNetworkQualityLow:"Verkkoyhteyden laatu on huono.",teamsMeetingCallNetworkQualityLow:"Verkkoyhteyden laatu on huono.",callNoSpeakerFound:"Kaiuttimia tai kuulokkeita ei löytynyt. Yhdistä äänilaite voidaksesi kuulla puhelun.",callNoMicrophoneFound:"Mikrofoneja ei löytynyt. Yhdistä äänitulolaite.",callMicrophoneAccessDenied:"Mikrofonia ei voi käyttää. Myönnä käyttöoikeus tälle verkkosivulle napsauttamalla osoiterivin lukitusta.",callMicrophoneAccessDeniedSafari:"Mikrofonia ei voi käyttää. Salli käyttöoikeudet päivittämällä sivu tai tarkista tämän selaimen asetukset ja varmista, että käyttöoikeudet on otettu käyttöön tässä sivustossa.",callMicrophoneMutedBySystem:"Järjestelmäasetuksen perusteella sinut on mykistetty.",callMicrophoneUnmutedBySystem:"Mikrofonisi palautui käyttöön ja järjestelmä on poistanut sen mykistyksen.",callMacOsMicrophoneAccessDenied:"Mikrofonia ei voi käyttää. Salli mikrofonin käyttö macOS:in tietosuoja-asetuksissa.",callLocalVideoFreeze:"Verkon kaistanleveys on heikko. Videokuvasi saattaa näkyä keskeytettynä puhelun muille osallistujille.",callCameraAccessDenied:"Kameraa ei voi käyttää. Myönnä käyttöoikeus tälle verkkosivulle napsauttamalla osoiterivin lukitusta.",callCameraAccessDeniedSafari:"Kameraa ei voi käyttää. Salli käyttöoikeudet päivittämällä sivu tai tarkista tämän selaimen asetukset ja varmista, että käyttöoikeudet on otettu käyttöön tässä sivustossa.",callCameraAlreadyInUse:"Kameraa ei voi käyttää. Toinen sovellus saattaa jo käyttää sitä.",callVideoStoppedBySystem:"Järjestelmä on pysäyttänyt videosi.",callVideoRecoveredBySystem:"Videosi näytetään jälleen.",callMacOsCameraAccessDenied:"MacOS estää kameran käytön. Päivitä tietosuoja-asetukset, jotta selain voi käyttää kameraasi.",callMacOsScreenShareAccessDenied:"MacOS estää näytön jakamisen. Päivitä tietosuoja-asetukset, jotta selain voi tallentaa, mitä näytölläsi tapahtuu.",dismissButtonAriaLabel:"Sulje",failedToJoinCallGeneric:"Puheluun liittyminen epäonnistui.",failedToJoinCallInvalidMeetingLink:"Kokoukseen liittyminen ei onnistu. Linkki on virheellinen.",cameraFrozenForRemoteParticipants:"Puhelun käyttäjillä on videokuvan näkymiseen liittyviä ongelmia. Tarkista laitteesi ja verkkosi.",unableToStartVideoEffect:"Videotehosteen käyttäminen ei onnistu.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Osallistujien korostusta ei voi aloittaa, koska enimmäismäärä osallistujia on jo valokeilassa.",mutedByRemoteParticipant:"Toinen osallistuja mykisti sinut."};var notificationStack$f={callingNetworkFailure:{title:"Ongelma puhelun yhdistämisessä",message:"Olet ilmeisesti offline-tilassa. Tarkista verkkoyhteys ja yritä uudelleen.",dismissButtonAriaLabel:"Sulje"},startVideoGeneric:{title:"Videon käynnistäminen epäonnistui.",dismissButtonAriaLabel:"Sulje"},stopVideoGeneric:{title:"Videon pysäyttäminen epäonnistui.",dismissButtonAriaLabel:"Sulje"},muteGeneric:{title:"Mikrofonin mykistäminen epäonnistui.",dismissButtonAriaLabel:"Sulje"},unmuteGeneric:{title:"Mikrofonin mykistyksen poistaminen epäonnistui.",dismissButtonAriaLabel:"Sulje"},speakingWhileMuted:{title:"Ääni on mykistetty.",dismissButtonAriaLabel:"Sulje"},startScreenShareGeneric:{title:"Näytön jakamisen aloittamisessa ilmeni ongelma.",dismissButtonAriaLabel:"Sulje"},stopScreenShareGeneric:{title:"Näytön jakamisen lopettaminen epäonnistui.",dismissButtonAriaLabel:"Sulje"},callNetworkQualityLow:{title:"Verkkoyhteyden laatu on huono.",dismissButtonAriaLabel:"Sulje"},teamsMeetingCallNetworkQualityLow:{title:"Verkkoyhteyden laatu on huono.",dismissButtonAriaLabel:"Sulje",message:"Liity tähän puheluun puhelimestasi, jotta ääni on parempi. Voit jatkaa kokouksen tarkastelua tällä laitteella.",primaryButtonLabel:"Liity puhelimella"},callNoSpeakerFound:{title:"Kaiuttimia tai kuulokkeita ei löytynyt.",message:"Yhdistä äänilaite voidaksesi kuulla puhelun.",dismissButtonAriaLabel:"Sulje"},callNoMicrophoneFound:{title:"Mikrofoneja ei löytynyt",message:"Yhdistä äänensyöttölaite.",dismissButtonAriaLabel:"Sulje"},callMicrophoneAccessDenied:{title:"Mikrofonia ei voi käyttää",message:"Myönnä käyttöoikeus tälle verkkosivulle napsauttamalla osoiterivillä olevaa lukkoa.",dismissButtonAriaLabel:"Sulje"},callMicrophoneAccessDeniedSafari:{title:"Mikrofonia ei voi käyttää",message:"Salli käyttöoikeudet päivittämällä sivu tai tarkista tämän selaimen asetukset ja varmista, että käyttöoikeudet on otettu käyttöön tässä sivustossa.",dismissButtonAriaLabel:"Sulje"},callMicrophoneMutedBySystem:{title:"Järjestelmäasetuksen perusteella sinut on mykistetty.",dismissButtonAriaLabel:"Sulje"},callMicrophoneUnmutedBySystem:{title:"Mikrofonisi palautui käyttöön ja järjestelmä on poistanut sen mykistyksen.",dismissButtonAriaLabel:"Sulje"},callMacOsMicrophoneAccessDenied:{title:"Mikrofonia ei voi käyttää",message:"Myönnä mikrofonin käyttöoikeus macOS:n tietosuoja-asetuksissa.",dismissButtonAriaLabel:"Sulje"},callLocalVideoFreeze:{title:"Verkon kaistanleveys on heikko",message:"Videokuvasi saattaa näkyä keskeytettynä puhelun muille osallistujille.",dismissButtonAriaLabel:"Sulje"},callCameraAccessDenied:{title:"Kameraa ei voi käyttää",message:"Myönnä käyttöoikeus tälle verkkosivulle napsauttamalla osoiterivillä olevaa lukkoa.",dismissButtonAriaLabel:"Sulje"},callCameraAccessDeniedSafari:{title:"Kameraa ei voi käyttää",message:"Salli käyttöoikeudet päivittämällä sivu tai tarkista tämän selaimen asetukset ja varmista, että käyttöoikeudet on otettu käyttöön tässä sivustossa.",dismissButtonAriaLabel:"Sulje"},callCameraAlreadyInUse:{title:"Kameraa ei voi käyttää",message:"Toinen sovellus saattaa jo käyttää sitä.",dismissButtonAriaLabel:"Sulje"},callVideoStoppedBySystem:{title:"Järjestelmä on pysäyttänyt videosi.",dismissButtonAriaLabel:"Sulje"},callVideoRecoveredBySystem:{title:"Videosi näytetään jälleen.",dismissButtonAriaLabel:"Sulje"},callMacOsCameraAccessDenied:{title:"MacOS estää kameran käytön",message:"Päivitä tietosuoja-asetukset, jotta selain voi käyttää kameraasi.",dismissButtonAriaLabel:"Sulje"},callMacOsScreenShareAccessDenied:{title:"MacOS estää näytön jakamisen",message:"Päivitä tietosuoja-asetukset, jotta selain voi tallentaa, mitä näytölläsi tapahtuu.",dismissButtonAriaLabel:"Sulje"},failedToJoinCallGeneric:{title:"Puheluun liittyminen epäonnistui.",dismissButtonAriaLabel:"Sulje"},failedToJoinCallInvalidMeetingLink:{title:"Kokoukseen liittyminen ei onnistu. Linkki on virheellinen.",dismissButtonAriaLabel:"Sulje"},cameraFrozenForRemoteParticipants:{title:"Puhelun muilla osallistujilla on videokuvasi näkymiseen liittyviä ongelmia",message:"Tarkista laitteesi ja verkkosi.",dismissButtonAriaLabel:"Sulje"},unableToStartVideoEffect:{title:"Videotehosteen käyttäminen ei onnistu.",dismissButtonAriaLabel:"Sulje"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Osallistujien valokeilaa ei voi aloittaa",message:"Enimmäismäärä osallistujia on jo valokeilassa.",dismissButtonAriaLabel:"Sulje"},mutedByRemoteParticipant:{title:"Toinen osallistuja mykisti sinut.",dismissButtonAriaLabel:"Sulje"},recordingStarted:{title:"Tallennus on aloitettu.",message:"Liittymällä annat suostumuksesi kokouksen litteroimiseen.",dismissButtonAriaLabel:"Sulje"},transcriptionStarted:{title:"Transkriptio on aloitettu.",message:"Liittymällä annat suostumuksesi kokouksen litteroimiseen.",dismissButtonAriaLabel:"Sulje"},recordingStopped:{title:"Tallennetta tallennetaan.",message:"Tallennus on pysäytetty.",dismissButtonAriaLabel:"Sulje"},transcriptionStopped:{title:"Transkriptiota tallennetaan.",message:"Transkriptio on pysäytetty.",dismissButtonAriaLabel:"Sulje"},recordingAndTranscriptionStarted:{title:"Tallentaminen ja litterointi on aloitettu.",message:"Liittymällä annat suostumuksesi kokouksen litteroimiseen.",dismissButtonAriaLabel:"Sulje"},recordingAndTranscriptionStopped:{title:"Tallennusta ja transkriptiota tallennetaan.",message:"Tallentaminen ja transkriptio on pysäytetty.",dismissButtonAriaLabel:"Sulje"},recordingStoppedStillTranscribing:{title:"Tallennus on pysäytetty.",message:"Transkriptiota ollaan luomassa vain tästä kokouksesta.",dismissButtonAriaLabel:"Sulje"},transcriptionStoppedStillRecording:{title:"Transkriptio on pysäytetty.",message:"Tallennetta ollaan luomassa vain tästä kokouksesta.",dismissButtonAriaLabel:"Sulje"},assignedBreakoutRoomOpened:{title:"Siirrämme sinut määritettyihin huoneeseesi 5 sekunnin kuluttua.",dismissButtonAriaLabel:"Sulje"},assignedBreakoutRoomChanged:{title:"Pienryhmätilasi on muuttunut",message:"Siirrämme sinut määritettyihin huoneeseesi 5 sekunnin kuluttua.",dismissButtonAriaLabel:"Sulje"},assignedBreakoutRoomClosed:{title:"Pienryhmätilasi on suljettu",message:"Siirrämme sinut takaisin kokoukseen 5 sekunnin kuluttua.",dismissButtonAriaLabel:"Sulje"},assignedBreakoutRoomOpenedPromptJoin:{title:"Liitytäänkö pienryhmätilaan?",message:"Sinut on määritetty pienryhmätilaan.",dismissButtonAriaLabel:"Sulje",primaryButtonLabel:"Liity tilaan",secondaryButtonLabel:"Myöhemmin"},breakoutRoomJoined:{title:"Olet liittynyt pienryhmätilaan.",dismissButtonAriaLabel:"Sulje"},breakoutRoomClosingSoon:{title:"Pienryhmätilan aikaraja on täyttymässä.",message:"Tämä huone suljetaan 30 sekunnin kuluttua",dismissButtonAriaLabel:"Sulje"},capabilityTurnVideoOnPresent:{title:"Kamera on otettu käyttöön",message:"Ota se käyttöön, jos haluat jakaa videokuvaa.",dismissButtonAriaLabel:"Sulje"},capabilityTurnVideoOnAbsent:{title:"Kamerasi on pois käytöstä",message:"Et voi enää jakaa videokuvaa.",dismissButtonAriaLabel:"Sulje"},capabilityUnmuteMicPresent:{title:"Mikrofoni on otettu käyttöön",message:"Poista mikrofonin mykistys, jos haluat puhua.",dismissButtonAriaLabel:"Sulje"},capabilityUnmuteMicAbsent:{title:"Mikrofonisi on poissa käytöstä",message:"Et voi enää poistaa mykistystä.",dismissButtonAriaLabel:"Sulje"},togetherModeStarted:{title:"Yhdessä-tila on alkanut",dismissButtonAriaLabel:"Sulje"},togetherModeEnded:{title:"Yhdessä-tila on päättynyt",dismissButtonAriaLabel:"Sulje"},transcriptionError:{title:"Transkriptiossa tapahtui virhe.",message:"Transkriptio on pysäytetty. Yritä myöhemmin uudelleen",dismissButtonAriaLabel:"Sulje"},transcriptionStartedByYou:{title:"Transkriptio on aloitettu.",message:"Sinun aloittamasi. Kerro kaikille, että heidän puheensa muunnetaan tekstiksi.",dismissButtonAriaLabel:"Sulje"}};var videoGallery$f={screenIsBeingSharedMessage:"Näyttösi jaetaan parhaillaan",screenShareLoadingMessage:"Ladataan käyttäjän {participant} näyttöä",localScreenShareLoadingMessage:"Ladataan näyttöä",localVideoLabel:"Sinä",localVideoCameraSwitcherLabel:"Vaihda kameraa",localVideoMovementLabel:"Siirrettävä paikallisen videon ruutu",localVideoMovementAriaLabel:"Siirrettävä paikallisen videon ruutu",localVideoSelectedDescription:"{cameraName} on valittu",displayNamePlaceholder:"Nimetön osallistuja",fitRemoteParticipantToFrame:"Sovita kehykseen",fillRemoteParticipantFrame:"Täytä kehys",pinParticipantForMe:"Kiinnitä minulle",pinParticipantForMeLimitReached:"Kiinnitys (enimmäisraja saavutettu)",unpinParticipantForMe:"Poista kiinnitys",pinParticipantMenuItemAriaLabel:"Kiinnitä {participantName}",unpinParticipantMenuItemAriaLabel:"Poista henkilön {participantName} kiinnitys",pinnedParticipantAnnouncementAriaLabel:"Kiinnitetty {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Poistettu henkilön {participantName} kiinnitys",startSpotlightVideoTileMenuLabel:"Lisää valokeilaan kaikille",addSpotlightVideoTileMenuLabel:"Lisää valokeilaan",spotlightLimitReachedMenuTitle:"Spotlight-raja saavutettu",stopSpotlightVideoTileMenuLabel:"Poista valokeilasta",stopSpotlightOnSelfVideoTileMenuLabel:"Valokeila",attendeeRole:"Osallistuja",muteParticipantMenuItemLabel:"Mykistä",waitingScreenText:"Odotetaan että muut liittyvät",forbidAudio:"Poista mikrofoni käytöstä",permitAudio:"Salli mikrofoni",forbidAudioTileMenuLabel:"Poista mikrofoni käytöstä",permitAudioTileMenuLabel:"Salli mikrofoni",forbidVideo:"Poista kamera käytöstä",permitVideo:"Salli kamera",forbidVideoTileMenuLabel:"Poista kamera käytöstä",permitVideoTileMenuLabel:"Salli kamera"};var dialpad$f={placeholderText:"Anna puhelinnumero",deleteButtonAriaLabel:"Poista"};var holdButton$f={onLabel:"Jatka",offLabel:"Pito",tooltipOnContent:"Jatka puhelua",tooltipOffContent:"Aseta puhelu pitoon"};var videoTile$f={participantStateRinging:"Soitetaan...",participantReconnecting:"Yhdistetään uudelleen...",participantStateHold:"Pidossa",moreOptionsButtonAriaLabel:"Lisää {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"vaimennettu",moreOptionsParticipantMutedStateUnmutedAriaLabel:"vaimennus poistettu",moreOptionsParticipantHandRaisedAriaLabel:"Käsi nostettuna",moreOptionsParticipantIsSpeakingAriaLabel:"Puhuminen",moreOptionsParticipantMicDisabledAriaLabel:"Mikrofoni poistettu käytöstä",moreOptionsParticipantCameraDisabledAriaLabel:"Kamera poistettu käytöstä"};var CameraAndMicrophoneSitePermissionsRequest$f={primaryText:"Salli {appName} käyttää kameraa ja mikrofonia",secondaryText:"Näin osallistujat voivat nähdä ja kuulla sinua.",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita",primaryButtonText:"Jatka ilman kameraa ja mikrofonia",ariaLabel:"Salli kameran ja mikrofonin käyttö"};var CameraSitePermissionsRequest$f={primaryText:"Salli sovelluksen {appName} käyttää kameraasi",secondaryText:"Tätä tarvitaan, jotta muut osallistujat näkevät sinut.",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita",primaryButtonText:"Jatka ilman kameraa",ariaLabel:"Salli kameran käyttö"};var MicrophoneSitePermissionsRequest$f={primaryText:"Salli sovelluksen {appName} käyttää mikrofoniasi",secondaryText:"Tätä tarvitaan, jotta muut osallistujat kuulevat sinut.",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita",primaryButtonText:"Jatka ilman mikrofonia",ariaLabel:"Salli mikrofonin käyttö"};var CameraAndMicrophoneSitePermissionsCheck$f={primaryText:"Tarkistetaan kameran ja mikrofonin käyttöoikeutta",secondaryText:"Salli käyttö, jos niin kehotetaan tekemään. Näin osallistujat voivat nähdä ja kuulla sinua.",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita",primaryButtonText:"Jatka ilman kameraa ja mikrofonia",ariaLabel:"Tarkistetaan kameran ja mikrofonin käyttöoikeutta. Salli käyttö pyydettäessä."};var CameraSitePermissionsCheck$f={primaryText:"Tarkistetaan kameran käyttöoikeutta",secondaryText:"Salli käyttö, jos niin kehotetaan tekemään. Osallistujat näkevät sinut tässä.",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita",primaryButtonText:"Jatka ilman kameraa",ariaLabel:"Tarkistetaan kameran käyttöoikeutta. Salli käyttö pyydettäessä."};var MicrophoneSitePermissionsCheck$f={primaryText:"Tarkistetaan mikrofonin käyttöoikeutta",secondaryText:"Salli käyttö pyydettäessä. Tätä tarvitaan, jotta muut osallistujat kuulevat sinut.",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita",primaryButtonText:"Jatka ilman mikrofonia",ariaLabel:"Tarkistetaan mikrofonin käyttöoikeutta. Salli käyttö pyydettäessä."};var CameraAndMicrophoneSitePermissionsDenied$f={primaryText:"Kameraa ja mikrofonia ei voi käyttää",secondaryText:"Myönnä mikrofonin käyttöoikeudet tälle verkkosivulle napsauttamalla osoiterivin lukkokuvaketta. Sivun päivitys saattaa olla pakollinen.",primaryButtonText:"Jatka ilman kameraa ja mikrofonia",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita"};var CameraAndMicrophoneSitePermissionsDeniedSafari$f={primaryText:"Kameraa ja mikrofonia ei voi käyttää",secondaryText:"Salli käyttöoikeudet päivittämällä sivu tai tarkista tämän selaimen asetukset ja varmista, että käyttöoikeudet on otettu käyttöön tässä sivustossa.",primaryButtonText:"Jatka ilman kameraa ja mikrofonia",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita"};var CameraSitePermissionsDenied$f={primaryText:"Kameraa ei voi käyttää",secondaryText:"Myönnä kameran käyttöoikeudet tälle verkkosivulle napsauttamalla osoiterivin lukkokuvaketta. Sivun päivitys saattaa olla pakollinen.",primaryButtonText:"Jatka ilman kameraa",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita"};var MicrophoneSitePermissionsDenied$f={primaryText:"Mikrofonia ei voi käyttää",secondaryText:"Myönnä mikrofonin käyttöoikeudet tälle verkkosivulle napsauttamalla osoiterivin lukkokuvaketta. Sivun päivitys saattaa olla pakollinen.",primaryButtonText:"Jatka ilman mikrofonia",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita"};var CameraSitePermissionsDeniedSafari$f={primaryText:"Kameraa ei voi käyttää",secondaryText:"Salli käyttöoikeudet päivittämällä sivu tai tarkista tämän selaimen asetukset ja varmista, että käyttöoikeudet on otettu käyttöön tässä sivustossa.",primaryButtonText:"Jatka ilman kameraa",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita"};var MicrophoneSitePermissionsDeniedSafari$f={primaryText:"Mikrofonia ei voi käyttää",secondaryText:"Salli käyttöoikeudet päivittämällä sivu tai tarkista tämän selaimen asetukset ja varmista, että käyttöoikeudet on otettu käyttöön tässä sivustossa.",primaryButtonText:"Jatka ilman mikrofonia",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita"};var UnsupportedBrowser$f={primaryText:"Selainta ei tueta",secondaryText:"Liity tähän puheluun käyttämällä yhteensopivaa selainta.",moreHelpLinkText:"Näytä yhteensopivuusvaatimukset"};var UnsupportedBrowserVersion$f={primaryText:"Selain on päivitettävä",secondaryText:"Jotta puhelu olisi paras mahdollinen, päivitä selain ja yritä liittyä puheluun uudelleen.",moreHelpLinkText:"Näytä yhteensopivuusvaatimukset",continueAnywayButtonText:"Aloita puhelu päivittämättä"};var UnsupportedOperatingSystem$f={primaryText:"Käyttöjärjestelmää ei tueta",secondaryText:"Liity tähän puheluun käyttämällä laitetta, jossa on yhteensopiva käyttöjärjestelmä.",moreHelpLinkText:"Näytä yhteensopivuusvaatimukset"};var BrowserPermissionDenied$f={primaryText:"Kamerasi tai mikrofonisi käyttäminen ei onnistu",secondaryText:"Selaimesi ei ehkä voi käyttää kameraasi tai mikrofoniasi. Voit korjata tämän avaamalla Järjestelmäasetukset.",primaryButtonText:"Yritä uudelleen",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita"};var BrowserPermissionDeniedIOS$f={primaryText:"Jatka sallimalla mikrofonin käyttö",secondaryText:"Tarvitaan, jotta muut osallistujat kuulevat sinut.",primaryButtonText:"Yritä uudelleen",imageAltText:"Kohta, josta mikrofonin ja kameran käyttöoikeus löytyy iOS:issa",linkText:"Tarvitsetko apua? Hae vianmääritysohjeita",step1Text:"Siirry asetussovellukseen.",step2Text:"Vieritä alaspäin tämän selaimen asetuksiin",step3Text:"Ota mikrofoni käyttöön (kameran käyttö on valinnaista)",step4Text:"Yritä liittyä puheluun uudelleen",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$f={leftNavButtonAriaLabel:"edellinen sivu {current}/{total}",rightNavButtonAriaLabel:"seuraava sivu {current}/{total}"};var meetingConferencePhoneInfo$f={meetingConferencePhoneInfoModalTitle:"Liity puheluun puhelimestasi",meetingConferencePhoneInfoModalDialIn:"Valitse numero",meetingConferencePhoneInfoModalMeetingId:"Anna kokoustunnus",meetingConferencePhoneInfoModalWait:"Odota, että järjestäjä ottaa sinut vastaan",meetingConferencePhoneInfoModalNoPhoneAvailable:"Puhelinta ei ole käytettävissä",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Maksuton)",meetingConferencePhoneInfoModalToll:"(Maksullinen)"};var IncomingCallNotification$f={incomingCallNotificationPlaceholderId:"Tuntematon käyttäjä",incomingCallNotificationPlaceholderAlert:"{callerName} soittaa",incomingCallNoticicationAcceptWithAudioAriaLabel:"Vastaa äänipuhelulla",incomingCallNoticicationAcceptWithVideoAriaLabel:"Vastaa videopuhelulla",incomingCallNoticicationRejectAriaLabel:"Hylkää",incomingCallNotificationAcceptButtonLabel:"Hyväksy",incomingCallNotificationRejectButtonLabel:"Hylkää",incomingCallNotificationAccceptWithVideoButtonLabel:"Vastaa videopuhelulla",incomingCallNotificationDismissButtonAriaLabel:"Hylkää"};var fi_FI$1 = {participantItem:participantItem$f,ParticipantList:ParticipantList$f,typingIndicator:typingIndicator$f,sendBox:sendBox$f,richTextSendBox:richTextSendBox$f,realTimeTextModal:realTimeTextModal$f,captionsBanner:captionsBanner$f,mentionPopover:mentionPopover$f,imageOverlay:imageOverlay$f,messageStatusIndicator:messageStatusIndicator$f,endCallButton:endCallButton$f,cameraButton:cameraButton$f,microphoneButton:microphoneButton$f,devicesButton:devicesButton$f,participantsButton:participantsButton$f,screenShareButton:screenShareButton$f,raiseHandButton:raiseHandButton$f,reactionButton:reactionButton$f,realTimeText:realTimeText$f,captionsSettingsModal:captionsSettingsModal$f,spokenLanguages:spokenLanguages$f,captionLanguages:captionLanguages$f,startCaptionsButton:startCaptionsButton$f,startRealTimeTextButton:startRealTimeTextButton$f,messageThread:messageThread$f,errorBar:errorBar$f,notificationStack:notificationStack$f,videoGallery:videoGallery$f,dialpad:dialpad$f,holdButton:holdButton$f,videoTile:videoTile$f,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$f,CameraSitePermissionsRequest:CameraSitePermissionsRequest$f,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$f,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$f,CameraSitePermissionsCheck:CameraSitePermissionsCheck$f,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$f,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$f,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$f,CameraSitePermissionsDenied:CameraSitePermissionsDenied$f,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$f,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$f,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$f,UnsupportedBrowser:UnsupportedBrowser$f,UnsupportedBrowserVersion:UnsupportedBrowserVersion$f,UnsupportedOperatingSystem:UnsupportedOperatingSystem$f,BrowserPermissionDenied:BrowserPermissionDenied$f,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$f,verticalGallery:verticalGallery$f,meetingConferencePhoneInfo:meetingConferencePhoneInfo$f,IncomingCallNotification:IncomingCallNotification$f};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Finnish (Finland).
 *
 * @public
 */
const COMPONENT_LOCALE_FI_FI = {
    strings: createComponentStrings(fi_FI$1)
};

var participantItem$e={isMeText:"(vous)",menuTitle:"Plus d’options",removeButtonLabel:"Supprimer",sharingIconLabel:"Partage",mutedIconLabel:"Muet activé",handRaisedIconLabel:"Main levée – Position {position}",displayNamePlaceholder:"Participant sans nom",participantStateRinging:"Appel en cours...",participantStateHold:"En attente",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, autres options",attendeeRole:"Participant",micDisabledIconLabel:"Microphone désactivé",cameraDisabledIconLabel:"Caméra désactivée"};var ParticipantList$e={overflowParticipantCount:"+{overflowCount} plus"};var typingIndicator$e={singleUser:"{user} en train d’écrire...",multipleUsers:"{users} sont en train d'écrire...",multipleUsersAbbreviateOne:"{users} et 1 autre en train d'écrire...",multipleUsersAbbreviateMany:"{users} et {numOthers} autres en train d’écrire...",delimiter:", "};var sendBox$e={placeholderText:"Saisir un message",textTooLong:"La longueur de votre message dépasse la limite maximale.",sendButtonAriaLabel:"Envoyer un message",attachmentUploadsPendingError:"Chargement du fichier en cours, veuillez patienter.",removeAttachment:"Supprimer le fichier",uploading:"Chargement",uploadCompleted:"Téléchargement terminé",attachmentMoreMenu:"Plus d’options"};var richTextSendBox$e={placeholderText:"Saisir un message",textTooLong:"La longueur de votre message dépasse la limite maximale.",sendButtonAriaLabel:"Envoyer un message",attachmentUploadsPendingError:"Chargement du fichier en cours, veuillez patienter.",imageUploadsPendingError:"Chargement de l’image en cours, veuillez patienter.",removeAttachment:"Supprimer un fichier",uploading:"Téléchargement",uploadCompleted:"Chargement terminé",richTextBoldTooltip:"Gras",richTextItalicTooltip:"Italique",richTextUnderlineTooltip:"Souligner",richTextBulletListTooltip:"Liste à puces",richTextNumberListTooltip:"Liste numérotée",richTextIncreaseIndentTooltip:"Augmenter le retrait",richTextDecreaseIndentTooltip:"Réduire le retrait",richTextInsertTableTooltip:"Insérer un tableau",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insérer",richTextInsertTableMenuTitle:"Insérer une table {column} x {row}",richTextInsertRowAboveMenu:"Insérer au-dessus",richTextInsertRowBelowMenu:"Insérer l’élément ci-dessous",richTextInsertColumnLeftMenu:"Insérer à gauche",richTextInsertColumnRightMenu:"Insérer à droite",richTextDeleteRowOrColumnMenu:"Supprimer",richTextDeleteColumnMenu:"Supprimer la colonne",richTextDeleteRowMenu:"Supprimer une ligne",richTextDeleteTableMenu:"Supprimer le tableau",richTextToolbarAriaLabel:"Barre d'outils de formatage des messages",richTextToolbarMoreButtonAriaLabel:"Autres options",attachmentMoreMenu:"Plus d’options",richTextNewBulletedListItemAnnouncement:"Puce, nouvelle ligne",richTextNewNumberedListItemAnnouncement:"{0}, nouvelle ligne",richTextBulletedListAppliedAnnouncement:"Liste à puces appliquée",richTextNumberedListAppliedAnnouncement:"Liste numérotée appliquée"};var realTimeTextModal$e={realTimeTextModalAriaLabel:"Activer la boîte de dialogue modale RTT",realTimeTextModalTitle:"Activation de la RTT ?",realTimeTextModalText:"Le texte en temps réel (RTT) sera activé pour tous les participants à la réunion.",realTimeTextConfirmButtonLabel:"Activer",realTimeTextCancelButtonLabel:"Annuler",realTimeTextCloseModalButtonAriaLabel:"Fermer la boîte de dialogue modale RTT"};var captionsBanner$e={captionsBannerSpinnerText:"Démarrage des légendes...",realTimeTextInputBoxDefaultText:"Tapez votre message en temps réel",realTimeTextInputErrorMessage:"Le message ne peut pas dépasser 2 000 caractères",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (texte en temps réel) est activé pour tous les participants pendant toute la durée de la réunion.",realTimeTextBannerLinkLabel:"En savoir plus",captionsOnlyContainerTitle:"Sous-titres",realTimeTextOnlyContainerTitle:"Texte en temps réel",captionsAndRealTimeTextContainerTitle:"Légendes et texte en temps réel",expandButtonAriaLabel:"Développer",minimizeButtonAriaLabel:"Réduire"};var mentionPopover$e={mentionPopoverHeader:"Suggestions"};var imageOverlay$e={downloadButtonLabel:"Télécharger",dismissButtonAriaLabel:"Fermer",overlayVisibleMessage:"Superposition des détails de l’image"};var messageStatusIndicator$e={deliveredAriaLabel:"Message envoyé",deliveredTooltipText:"Envoyé",seenAriaLabel:"Message vu par d’autres personnes",seenTooltipText:"Vu",readByTooltipText:"Lu par {messageThreadReadCount} participants sur {remoteParticipantsCount}",sendingAriaLabel:"Envoi du message",sendingTooltipText:"Envoi",failedToSendAriaLabel:"Échec d'envoi du message",failedToSendTooltipText:"Échec de l'envoi"};var endCallButton$e={label:"Quitter",tooltipContent:"Quitter l'appel",leaveOption:"Partir",endCallOption:"Terminer l'appel pour tous"};var cameraButton$e={onLabel:"Caméra",offLabel:"Caméra",tooltipDisabledContent:"La caméra est désactivée.",tooltipOnContent:"Désactiver la caméra",tooltipOffContent:"Activer la caméra",tooltipVideoLoadingContent:"Chargement de la vidéo",cameraMenuTitle:"Caméra",cameraMenuTooltip:"Choisissez l'appareil photo",cameraButtonSplitRoleDescription:"Bouton partagé",onSplitButtonAriaLabel:"Désactiver la caméra",offSplitButtonAriaLabel:"Activer la caméra",cameraActionTurnedOnAnnouncement:"Votre caméra a été activée",cameraActionTurnedOffAnnouncement:"Votre caméra a été désactivée",onSplitButtonPrimaryActionCamera:"Désactiver l’appareil photo",offSplitButtonPrimaryActionCamera:"Activer la caméra",cameraPrimaryActionSplitButtonTitle:"Utiliser l’appareil photo",videoEffectsMenuItemTitle:"Effets",cameraButtonAriaDescription:"options de la caméra"};var microphoneButton$e={onLabel:"Microphone",offLabel:"Microphone",tooltipDisabledContent:"Le microphone est désactivé.",tooltipOnContent:"Désactiver le micro",tooltipOffContent:"Activer le micro",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choisissez le micro",speakerMenuTitle:"Haut-parleur",speakerMenuTooltip:"Choisissez le haut-parleur",microphoneButtonSplitRoleDescription:"Bouton partagé",onSplitButtonAriaLabel:"Désactiver le micro",offSplitButtonAriaLabel:"Réactiver le micro",microphoneActionTurnedOnAnnouncement:"Votre micro a été activé",microphoneActionTurnedOffAnnouncement:"Votre micro a été désactivé",offSplitButtonMicrophonePrimaryAction:"Réactiver le micro",onSplitButtonMicrophonePrimaryAction:"Désactiver le micro",microphonePrimaryActionSplitButtonTitle:"Utiliser le microphone",microphoneAriaDescription:"Options audio",deepNoiseSuppressionTitle:"Suppression du bruit",deepNoiseSuppressionOnAnnouncement:"La suppression du bruit profond a été activée",deepNoiseSuppressionOffAnnouncement:"La suppression du bruit profond a été désactivée"};var devicesButton$e={label:"Appareils",tooltipContent:"Gérer les appareils",cameraMenuTitle:"Appareil photo",cameraMenuTooltip:"Choisissez l'appareil photo",audioDeviceMenuTitle:"Périphérique audio",audioDeviceMenuTooltip:"Choisissez un périphérique audio",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choisissez le micro",speakerMenuTitle:"Haut-parleur",speakerMenuTooltip:"Choisissez le haut-parleur"};var participantsButton$e={label:"Personnes",ariaLabel:"Contacts",tooltipContent:"Afficher les participants",menuHeader:"Pendant cet appel",participantsListButtonLabel:"{numParticipants} contacts",muteAllButtonLabel:"Désactiver tous les micros",copyInviteLinkButtonLabel:"Copier le lien d'invitation",copyInviteLinkActionedAriaLabel:"Le lien d’invitation a été copié."};var screenShareButton$e={onLabel:"Arrêter la présentation",offLabel:"Présenter",tooltipDisabledContent:"La présentation est désactivée.",tooltipOnContent:"Présentation de votre écran",tooltipOffContent:"Présenter votre écran"};var raiseHandButton$e={offLabel:"Augmenter",onLabel:"Inférieur",tooltipDisabledContent:"L'action lever la main est désactivée",tooltipOnContent:"Baisser la main",tooltipOffContent:"Lever les mains"};var reactionButton$e={label:"Réagir",ariaLabel:"Réagir",tooltipDisabledContent:"L’action de réaction est désactivée",tooltipContent:"Envoyer une réaction",likeReactionTooltipContent:"J’aime",heartReactionTooltipContent:"Aimer",laughReactionTooltipContent:"Rire",applauseReactionTooltipContent:"Acclamation",surprisedReactionTooltipContent:"Surpris"};var realTimeText$e={isTypingText:"Saisie RTT"};var captionsSettingsModal$e={captionsSettingsModalTitle:"Paramètres des sous-titres",captionsSettingsSpokenLanguageDropdownLabel:"Langue parlée",captionsSettingsCaptionLanguageDropdownLabel:"Langue des sous-titres",captionsSettingsSpokenLanguageDropdownInfoText:"Langue parlée par tous les participants à cet appel.",captionsSettingsCaptionLanguageDropdownInfoText:"Les sous-titres apparaîtront dans cette langue.",captionsSettingsConfirmButtonLabel:"Confirmer",captionsSettingsCancelButtonLabel:"Annuler",captionsSettingsModalAriaLabel:"Paramétrage des légendes Modal",captionsSettingsCloseModalButtonAriaLabel:"Paramètres de fermeture des sous-titres"};var spokenLanguages$e={"ar-ae":"Arabe – E.A.U.","ar-sa":"Arabe – Arabie saoudite","da-dk":"Danois","de-de":"Allemand – Allemagne","en-au":"Anglais – Australie","en-ca":"Anglais – Canada","en-gb":"Anglais – Royaume-Uni","en-in":"Anglais – Inde","en-nz":"Anglais – Nouvelle-Zélande","en-us":"Anglais – États-Unis","es-es":"Espagnol – Espagne (tri moderne)","es-mx":"Espagnol – Mexique","fi-fi":"Finnois","fr-ca":"Français – Canada","fr-fr":"Français – France","hi-in":"Hindi","it-it":"Italien – Italie","ja-jp":"Japonais","ko-kr":"Coréen","nb-no":"Norvégien (Bokmål)","nl-be":"Néerlandais – Belgique","nl-nl":"Néerlandais – Pays-Bas","pl-pl":"Polonais","pt-br":"Portugais – Brésil","ru-ru":"Russe","sv-se":"Suédois","zh-cn":"Chinois – République populaire de Chine","zh-hk":"Chinois – Hong Kong SAR","cs-cz":"Tchèque","pt-pt":"Portugais – Portugal","tr-tr":"Turc","vi-vn":"Vietnamien","th-th":"Thaï","he-il":"Hébreu","cy-gb":"Gallois","uk-ua":"Ukrainien","el-gr":"Grec","hu-hu":"Hongrois","ro-ro":"Roumain","sk-sk":"Slovaque","zh-tw":"Chinois – Taiwan"};var captionLanguages$e={ar:"Arabe",da:"Danois",de:"Allemand",en:"Anglais",es:"Espagnol",fi:"Finnois","fr-ca":"Français – Canada",fr:"Français – France",hi:"Hindi",it:"Italien",ja:"Japonais",ko:"Coréen",nb:"Norvégien (Bokmål)",nl:"Néerlandais",pl:"Polonais",pt:"Portugais – Brésil",ru:"Russe",sv:"Suédois","zh-Hans":"Chinois (simplifié)","zh-Hant":"Chinois (traditionnel)",cs:"Tchèque","pt-pt":"Portugais – Portugal",tr:"Turc",vi:"Vietnamien",th:"Thaï",he:"Hébreu",cy:"Gallois",uk:"Ukrainien",el:"Grec",hu:"Hongrois",ro:"Roumain",sk:"Slovaque"};var startCaptionsButton$e={onLabel:"Activer les sous-titres",offLabel:"Désactiver les sous-titres",tooltipOnContent:"Désactiver les sous-titres",tooltipOffContent:"Activer les sous-titres"};var startRealTimeTextButton$e={label:"Activer RTT pour cet appel",tooltipOffContent:"Activer RTT pour cet appel"};var messageThread$e={yesterday:"Hier",sunday:"Dimanche",monday:"Lundi",tuesday:"Mardi",wednesday:"Mercredi",thursday:"Jeudi",friday:"Vendredi",saturday:"Samedi",participantJoined:"a rejoint la conversation.",participantLeft:"a quitté le conversation.",editMessage:"Modifier",removeMessage:"Supprimer",resendMessage:"Réessayer l'envoi",failToSendTag:"Échec de l'envoi",editedTag:"Modifié",liveAuthorIntro:"Message de {author}",editedMessageLiveAuthorIntro:"Message modifié de {author}",editedMessageLocalUserLiveAuthorIntro:"Message modifié",messageContentAriaText:"{status} {author} a dit {message}.",messageContentMineAriaText:"{status} Vous avez dit {message}.",editBoxTextLimit:"Votre message dépasse la limite de {limitNumber} caractères",editBoxPlaceholderText:"Modifier votre message",newMessagesIndicator:"Nouveaux messages",noDisplayNameSub:"Sans nom",editBoxCancelButton:"Annuler",editBoxSubmitButton:"Terminé",messageReadCount:"Lu par {messageReadByCount} participants sur {remoteParticipantsCount}",actionMenuMoreOptions:"Plus d’options",downloadAttachment:"Télécharger un fichier",openAttachment:"Ouvrir le fichier",blockedWarningText:"Ce message a été supprimé en raison d’une stratégie de l’organisation.",blockedWarningLinkText:"Détails",attachmentCardGroupMessage:"Le message a {attachmentCount} pièce jointe",messageDeletedAnnouncementAriaLabel:"Le message est supprimé",richTextBoldTooltip:"Gras",richTextItalicTooltip:"Italique",richTextUnderlineTooltip:"Souligner",richTextBulletListTooltip:"Liste à puces",richTextNumberListTooltip:"Liste numérotée",richTextIncreaseIndentTooltip:"Augmenter le retrait",richTextDecreaseIndentTooltip:"Réduire le retrait",richTextInsertTableTooltip:"Insérer un tableau",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insérer",richTextInsertTableMenuTitle:"Insérer une table {column} x {row}",richTextInsertRowAboveMenu:"Insérer au-dessus",richTextInsertRowBelowMenu:"Insérer l’élément ci-dessous",richTextInsertColumnLeftMenu:"Insérer à gauche",richTextInsertColumnRightMenu:"Insérer à droite",richTextDeleteRowOrColumnMenu:"Supprimer",richTextDeleteColumnMenu:"Supprimer la colonne",richTextDeleteRowMenu:"Supprimer une ligne",richTextDeleteTableMenu:"Supprimer le tableau",richTextToolbarMoreButtonAriaLabel:"Autres options",attachmentUploadsPendingError:"Chargement du fichier en cours, veuillez patienter.",imageUploadsPendingError:"Chargement de l’image en cours, veuillez patienter.",richTextNewBulletedListItemAnnouncement:"Puce, nouvelle ligne",richTextNewNumberedListItemAnnouncement:"{0}, nouvelle ligne",richTextBulletedListAppliedAnnouncement:"Liste à puces appliquée",richTextNumberedListAppliedAnnouncement:"Liste numérotée appliquée"};var errorBar$e={unableToReachChatService:"Vous êtes hors connexion",accessDenied:"Impossible d’accéder aux services de conversation. Vérifiez les informations d’identification de l’utilisateur fournies",userNotInChatThread:"Vous n’est plus dans ce thread de conversation",sendMessageNotInChatThread:"Échec de l’envoi du message car vous n’êtes plus dans ce thread de conversation",sendMessageGeneric:"Échec de l’envoi du message",callingNetworkFailure:"Perturbation de l’appel de connexion : vous semblez être hors connexion",startVideoGeneric:"Échec du démarrage de la vidéo",stopVideoGeneric:"Échec de l’arrêt de la vidéo",muteGeneric:"Échec de la désactivation du son du microphone",unmuteGeneric:"Échec de l’activation du son du microphone",speakingWhileMuted:"Votre micro est désactivé",startScreenShareGeneric:"Un problème est survenu durant le démarrage du partage d’écran.",stopScreenShareGeneric:"Échec de l’arrêt du partage d’écran",callNetworkQualityLow:"La qualité de réseau est faible.",teamsMeetingCallNetworkQualityLow:"La qualité de réseau est faible.",callNoSpeakerFound:"Aucun haut-parleur ou casque n’a été trouvé. Connectez un périphérique audio pour entendre l’appel.",callNoMicrophoneFound:"Aucun micro n’a été trouvé. Connectez un périphérique d’entrée audio.",callMicrophoneAccessDenied:"Impossible d’accéder au microphone. Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",callMicrophoneAccessDeniedSafari:"Impossible d’accéder au microphone. Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",callMicrophoneMutedBySystem:"Votre micro est désactivé par votre système.",callMicrophoneUnmutedBySystem:"Votre micro a été récupéré et la son a été activé par votre système.",callMacOsMicrophoneAccessDenied:"Impossible d’accéder au microphone. Accordez l’autorisation de microphone dans vos paramètres de confidentialité macOS.",callLocalVideoFreeze:"La bande passante réseau est médiocre. Votre vidéo peut apparaître suspendue pour les autres participants à l’appel.",callCameraAccessDenied:"Impossible d’accéder à la caméra. Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",callCameraAccessDeniedSafari:"Impossible d’accéder à la caméra. Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",callCameraAlreadyInUse:"Impossible d’accéder à la caméra. Il est peut-être déjà utilisé par une autre application.",callVideoStoppedBySystem:"Votre vidéo a été arrêtée par votre système.",callVideoRecoveredBySystem:"Votre vidéo a repris.",callMacOsCameraAccessDenied:"MacOS bloque l’accès à votre caméra. Mettez à jour vos paramètres de confidentialité pour autoriser ce navigateur à accéder à votre caméra.",callMacOsScreenShareAccessDenied:"MacOS bloque le partage d’écran. Mettez à jour vos paramètres de confidentialité pour permettre à ce navigateur d’enregistrer votre écran.",dismissButtonAriaLabel:"Fermer",failedToJoinCallGeneric:"Nous n’avons pas pu rejoindre l’appel.",failedToJoinCallInvalidMeetingLink:"Impossible de rejoindre la réunion. Lien non valide.",cameraFrozenForRemoteParticipants:"Les utilisateurs de l’appel rencontrent des problèmes pour voir votre vidéo. Vérifiez vos appareils et votre réseau.",unableToStartVideoEffect:"Impossible d’appliquer l’effet vidéo.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Impossible de lancer la mise en lumière sur le(s) participant(s), car le nombre maximum de participants est déjà mis en lumière.",mutedByRemoteParticipant:"Votre micro a été désactivé par un autre participant."};var notificationStack$e={callingNetworkFailure:{title:"Appel de connexion en cours",message:"Vous semblez être hors connexion. Vérifiez votre connexion réseau et réessayez.",dismissButtonAriaLabel:"Fermer"},startVideoGeneric:{title:"Échec du démarrage de la vidéo.",dismissButtonAriaLabel:"Fermer"},stopVideoGeneric:{title:"Échec de l’arrêt de la vidéo.",dismissButtonAriaLabel:"Fermer"},muteGeneric:{title:"Impossible de désactiver le micro.",dismissButtonAriaLabel:"Fermer"},unmuteGeneric:{title:"Échec de l’activation du micro.",dismissButtonAriaLabel:"Fermer"},speakingWhileMuted:{title:"Le son de votre micro est désactivé.",dismissButtonAriaLabel:"Fermer"},startScreenShareGeneric:{title:"Un problème est survenu durant le démarrage du partage d’écran.",dismissButtonAriaLabel:"Fermer"},stopScreenShareGeneric:{title:"Échec de l’arrêt du partage d’écran.",dismissButtonAriaLabel:"Fermer"},callNetworkQualityLow:{title:"La qualité de réseau est faible.",dismissButtonAriaLabel:"Fermer"},teamsMeetingCallNetworkQualityLow:{title:"La qualité de réseau est faible.",dismissButtonAriaLabel:"Fermer",message:"Rejoignez cet appel à partir de votre téléphone pour obtenir un meilleur son. Vous pouvez continuer à afficher la réunion sur cet appareil.",primaryButtonLabel:"Participer par téléphone"},callNoSpeakerFound:{title:"Haut-parleurs ou casque introuvables",message:"Connectez un périphérique audio pour entendre l’appel.",dismissButtonAriaLabel:"Fermer"},callNoMicrophoneFound:{title:"Aucun microphone trouvé",message:"Connectez un périphérique d’entrée audio.",dismissButtonAriaLabel:"Fermer"},callMicrophoneAccessDenied:{title:"Impossible d’accéder au microphone",message:"Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",dismissButtonAriaLabel:"Fermer"},callMicrophoneAccessDeniedSafari:{title:"Impossible d’accéder au microphone",message:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",dismissButtonAriaLabel:"Fermer"},callMicrophoneMutedBySystem:{title:"Votre micro est désactivé par votre système.",dismissButtonAriaLabel:"Fermer"},callMicrophoneUnmutedBySystem:{title:"Votre micro a été récupéré et la son a été activé par votre système.",dismissButtonAriaLabel:"Fermer"},callMacOsMicrophoneAccessDenied:{title:"Impossible d’accéder au microphone",message:"Accordez l’autorisation de microphone dans vos paramètres de confidentialité macOS.",dismissButtonAriaLabel:"Fermer"},callLocalVideoFreeze:{title:"La bande passante réseau est médiocre",message:"Votre vidéo peut apparaître suspendue pour d’autres participants à l’appel.",dismissButtonAriaLabel:"Fermer"},callCameraAccessDenied:{title:"Impossible d’accéder à la caméra",message:"Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",dismissButtonAriaLabel:"Fermer"},callCameraAccessDeniedSafari:{title:"Impossible d’accéder à la caméra",message:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",dismissButtonAriaLabel:"Fermer"},callCameraAlreadyInUse:{title:"Impossible d’accéder à la caméra",message:"Il est peut-être déjà utilisé par une autre application.",dismissButtonAriaLabel:"Fermer"},callVideoStoppedBySystem:{title:"Votre vidéo a été arrêtée par votre système.",dismissButtonAriaLabel:"Fermer"},callVideoRecoveredBySystem:{title:"Votre vidéo a repris.",dismissButtonAriaLabel:"Fermer"},callMacOsCameraAccessDenied:{title:"MacOS bloque l’accès à votre caméra",message:"Mettez à jour vos paramètres de confidentialité pour autoriser ce navigateur à accéder à votre caméra.",dismissButtonAriaLabel:"Fermer"},callMacOsScreenShareAccessDenied:{title:"MacOS bloque le partage d’écran",message:"Mettez à jour vos paramètres de confidentialité pour permettre à ce navigateur d’enregistrer votre écran.",dismissButtonAriaLabel:"Fermer"},failedToJoinCallGeneric:{title:"Nous n’avons pas pu rejoindre l’appel.",dismissButtonAriaLabel:"Fermer"},failedToJoinCallInvalidMeetingLink:{title:"Impossible de rejoindre la réunion. Lien non valide.",dismissButtonAriaLabel:"Fermer"},cameraFrozenForRemoteParticipants:{title:"Les utilisateurs de l’appel rencontrent des problèmes pour voir votre vidéo",message:"Vérifiez vos appareils et votre réseau.",dismissButtonAriaLabel:"Fermer"},unableToStartVideoEffect:{title:"Impossible d’appliquer l’effet vidéo.",dismissButtonAriaLabel:"Fermer"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Impossible de mettre en évidence le ou les participants",message:"Le nombre maximum de participants est déjà connu.",dismissButtonAriaLabel:"Fermer"},mutedByRemoteParticipant:{title:"Votre micro a été désactivé par un autre participant.",dismissButtonAriaLabel:"Fermer"},recordingStarted:{title:"L'enregistrement a démarré.",message:"En participant, vous autorisez la transcription de cette réunion.",dismissButtonAriaLabel:"Fermer"},transcriptionStarted:{title:"La transcription a démarré.",message:"En participant, vous autorisez la transcription de cette réunion.",dismissButtonAriaLabel:"Fermer"},recordingStopped:{title:"Enregistrement en cours de sauvegarde.",message:"L’enregistrement s’est arrêté.",dismissButtonAriaLabel:"Fermer"},transcriptionStopped:{title:"La transcription est en cours d’enregistrement.",message:"Arrêt de la transcription.",dismissButtonAriaLabel:"Fermer"},recordingAndTranscriptionStarted:{title:"Démarrage de l’enregistrement et de la transcription.",message:"En participant, vous autorisez la transcription de cette réunion.",dismissButtonAriaLabel:"Fermer"},recordingAndTranscriptionStopped:{title:"Sauvegarde en cours de l’enregistrement et de la transcription.",message:"L’enregistrement et la transcription ont pris fin.",dismissButtonAriaLabel:"Fermer"},recordingStoppedStillTranscribing:{title:"L’enregistrement s’est arrêté.",message:"À présent, vous ne transcrivez que cette réunion.",dismissButtonAriaLabel:"Fermer"},transcriptionStoppedStillRecording:{title:"Arrêt de la transcription.",message:"Vous n’enregistrez désormais que cette réunion.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomOpened:{title:"Nous allons vous déplacer vers la salle affectée dans 5 secondes.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomChanged:{title:"Votre salle pour petit groupe a changé",message:"Nous allons vous déplacer vers la salle affectée dans 5 secondes.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomClosed:{title:"Votre salle pour petit groupe s’est fermée",message:"Nous vous redéverrons à votre réunion dans 5 secondes.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomOpenedPromptJoin:{title:"Rejoindre la salle de réunion ?",message:"Vous avez reçu une invitation à rejoindre une salle pour petit groupe.",dismissButtonAriaLabel:"Fermer",primaryButtonLabel:"Rejoindre une salle",secondaryButtonLabel:"Plus tard"},breakoutRoomJoined:{title:"Vous avez rejoint une salle pour petit groupe.",dismissButtonAriaLabel:"Proche"},breakoutRoomClosingSoon:{title:"Limite de temps de la salle sur le point d’expirer.",message:"Cette salle se fermera dans 30 secondes.",dismissButtonAriaLabel:"Proche"},capabilityTurnVideoOnPresent:{title:"Votre caméra a été activée",message:"Activez-la si vous souhaitez partager une vidéo.",dismissButtonAriaLabel:"Fermer"},capabilityTurnVideoOnAbsent:{title:"Votre caméra est désactivée",message:"Vous ne pouvez plus partager de vidéo.",dismissButtonAriaLabel:"Fermer"},capabilityUnmuteMicPresent:{title:"Votre micro a été activé",message:"Activer le son si vous voulez parler.",dismissButtonAriaLabel:"Fermer"},capabilityUnmuteMicAbsent:{title:"Votre microphone est désactivé",message:"Vous ne pouvez plus activer le son.",dismissButtonAriaLabel:"Fermer"},togetherModeStarted:{title:"Togethermode a démarré",dismissButtonAriaLabel:"Fermez"},togetherModeEnded:{title:"Togethermode est terminé",dismissButtonAriaLabel:"Fermez"},transcriptionError:{title:"Désolé, une erreur s’est produite lors de la transcription.",message:"Désolé, la transcription a été arrêtée. Veuillez réessayer plus tard",dismissButtonAriaLabel:"Fermer"},transcriptionStartedByYou:{title:"La transcription a démarré.",message:"Démarrée par vous. Faites savoir à tout le monde qu'ils sont transcrits.",dismissButtonAriaLabel:"Fermer"}};var videoGallery$e={screenIsBeingSharedMessage:"Vous partagez votre écran.",screenShareLoadingMessage:"Chargement de l’écran de {participant}",localScreenShareLoadingMessage:"Chargement de votre écran",localVideoLabel:"Vous",localVideoCameraSwitcherLabel:"Changer de caméra",localVideoMovementLabel:"Vignette de vidéo locale déplaçable",localVideoMovementAriaLabel:"Mosaïque de vidéo locale déplaçable",localVideoSelectedDescription:"{cameraName} sélectionnée",displayNamePlaceholder:"Participant sans nom",fitRemoteParticipantToFrame:"Ajuster à l'image",fillRemoteParticipantFrame:"Remplir l'image",pinParticipantForMe:"Épingler pour moi",pinParticipantForMeLimitReached:"Épingler (limite atteinte)",unpinParticipantForMe:"Détacher",pinParticipantMenuItemAriaLabel:"Épingler {participantName}",unpinParticipantMenuItemAriaLabel:"Détacher {participantName}",pinnedParticipantAnnouncementAriaLabel:"{participantName} a été épinglé",unpinnedParticipantAnnouncementAriaLabel:"{participantName} a été détaché",startSpotlightVideoTileMenuLabel:"Mettre en évidence pour tout le monde",addSpotlightVideoTileMenuLabel:"Ajouter la mise en évidence",spotlightLimitReachedMenuTitle:"Limite Spotlight atteinte",stopSpotlightVideoTileMenuLabel:"Arrêter la mise en évidence",stopSpotlightOnSelfVideoTileMenuLabel:"Quitter la mise en évidence",attendeeRole:"Participant(e)",muteParticipantMenuItemLabel:"Désactiver le micro / muet",waitingScreenText:"En attente des autres participants",forbidAudio:"Désactiver le microphone",permitAudio:"Autoriser le micro",forbidAudioTileMenuLabel:"Désactiver le microphone",permitAudioTileMenuLabel:"Autoriser le micro",forbidVideo:"Désactiver la caméra",permitVideo:"Autoriser la caméra",forbidVideoTileMenuLabel:"Désactiver la caméra",permitVideoTileMenuLabel:"Autoriser la caméra"};var dialpad$e={placeholderText:"Entrer un numéro de téléphone",deleteButtonAriaLabel:"Supprimer"};var holdButton$e={onLabel:"Reprendre",offLabel:"Mettre en attente",tooltipOnContent:"Reprendre l’appel",tooltipOffContent:"Mettre l’appel en attente"};var videoTile$e={participantStateRinging:"Appel en cours...",participantReconnecting:"Reconnexion...",participantStateHold:"En attente",moreOptionsButtonAriaLabel:"Autres options {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"son désactivé",moreOptionsParticipantMutedStateUnmutedAriaLabel:"désactivé",moreOptionsParticipantHandRaisedAriaLabel:"Main levée",moreOptionsParticipantIsSpeakingAriaLabel:"Parler",moreOptionsParticipantMicDisabledAriaLabel:"Microphone désactivé",moreOptionsParticipantCameraDisabledAriaLabel:"Caméra désactivée"};var CameraAndMicrophoneSitePermissionsRequest$e={primaryText:"Autoriser {appName} à utiliser votre caméra et votre microphone",secondaryText:"Cela permet aux participants de vous voir et de vous entendre.",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans caméra ni micro",ariaLabel:"Autoriser l'accès à la caméra et au microphone"};var CameraSitePermissionsRequest$e={primaryText:"Autoriser {appName} à utiliser votre caméra",secondaryText:"Cela permet aux participants de vous voir.",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans caméra",ariaLabel:"Autoriser l’accès à la caméra"};var MicrophoneSitePermissionsRequest$e={primaryText:"Autoriser {appName} à utiliser votre micro",secondaryText:"Cela permet aux participants de vous écouter.",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans microphone",ariaLabel:"Autoriser l’accès au micro"};var CameraAndMicrophoneSitePermissionsCheck$e={primaryText:"Vérification de l’accès à la caméra et au microphone",secondaryText:"Autorisez l’accès si vous y êtes invité. Cela permet aux participants de vous voir et de vous entendre.",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans caméra ni micro",ariaLabel:"Vérification de l’accès à la caméra et au microphone. Autorisez l’accès si vous y êtes invité."};var CameraSitePermissionsCheck$e={primaryText:"Vérification de l’accès à la caméra",secondaryText:"Autorisez l’accès si vous y êtes invité. Cela permet aux participants de vous voir.",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans appareil photo",ariaLabel:"Vérification de l’accès à la caméra. Autorisez l’accès si vous y êtes invité."};var MicrophoneSitePermissionsCheck$e={primaryText:"Vérification de l’accès au microphone",secondaryText:"Autorisez l’accès si vous y êtes invité. Cela permet aux participants de vous entendre.",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans microphone",ariaLabel:"Vérification de l’accès au microphone. Autorisez l’accès si vous y êtes invité."};var CameraAndMicrophoneSitePermissionsDenied$e={primaryText:"Nous n’avons pas pu accéder au micro et à la caméra.",secondaryText:"Cliquez sur l’icône de verrouillage dans la barre d’adresses pour accorder des autorisations de microphone à cette page Web. Une actualisation de page peut être nécessaire.",primaryButtonText:"Continuer sans caméra ni microphone",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes"};var CameraAndMicrophoneSitePermissionsDeniedSafari$e={primaryText:"Nous n’avons pas pu accéder au micro et à la caméra.",secondaryText:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",primaryButtonText:"Continuer sans caméra ni microphone",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes"};var CameraSitePermissionsDenied$e={primaryText:"Impossible d’accéder à la caméra",secondaryText:"Cliquez sur l’icône de verrouillage dans la barre d’adresses pour accorder des autorisations d’appareil photo à cette page Web. Une actualisation de page peut être nécessaire.",primaryButtonText:"Continuer sans caméra",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes"};var MicrophoneSitePermissionsDenied$e={primaryText:"Nous n’avons pas pu accéder au micro.",secondaryText:"Cliquez sur l’icône de verrouillage dans la barre d’adresses pour accorder des autorisations de microphone à cette page Web. Une actualisation de page peut être nécessaire.",primaryButtonText:"Continuer sans microphone",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes"};var CameraSitePermissionsDeniedSafari$e={primaryText:"Nous n’avons pas pu accéder à la caméra.",secondaryText:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",primaryButtonText:"Continuer sans appareil photo",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes"};var MicrophoneSitePermissionsDeniedSafari$e={primaryText:"Impossible d’accéder au microphone",secondaryText:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",primaryButtonText:"Continuer sans microphone",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes"};var UnsupportedBrowser$e={primaryText:"Navigateur non pris en charge",secondaryText:"Rejoignez cet appel à l’aide d’un navigateur compatible.",moreHelpLinkText:"Voir les exigences de compatibilité"};var UnsupportedBrowserVersion$e={primaryText:"Mise à jour du navigateur nécessaire",secondaryText:"Pour garantir le meilleur appel possible, mettez à jour votre navigateur, puis réessayez de rejoindre l’appel.",moreHelpLinkText:"Voir les exigences de compatibilité",continueAnywayButtonText:"Démarrer l’appel sans mettre à jour"};var UnsupportedOperatingSystem$e={primaryText:"Système d'exploitation non pris en charge",secondaryText:"Rejoignez cet appel à l’aide d’un appareil avec un système d’exploitation compatible.",moreHelpLinkText:"Voir les exigences de compatibilité"};var BrowserPermissionDenied$e={primaryText:"Impossible d’utiliser votre caméra ou votre micro",secondaryText:"Votre navigateur n’a peut-être pas accès à votre caméra ou à votre micro. Pour résoudre ce problème, ouvrez Préférences système.",primaryButtonText:"Réessayer",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes"};var BrowserPermissionDeniedIOS$e={primaryText:"Autoriser l’accès au microphone pour continuer",secondaryText:"Ainsi, les autres participants peuvent vous entendre.",primaryButtonText:"Réessayer",imageAltText:"Emplacement des autorisations pour le microphone et la caméra pour iOS",linkText:"Besoin d’aide ? Obtenir de l’aide sur la résolution des problèmes",step1Text:"Accéder à l’application Paramètres",step2Text:"Faites défiler vers le bas pour accéder aux paramètres de ce navigateur",step3Text:"Activer le microphone (caméra facultative)",step4Text:"Réessayez de rejoindre l’appel",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$e={leftNavButtonAriaLabel:"page précédente {current} de {total}",rightNavButtonAriaLabel:"page suivante{current} de {total}"};var meetingConferencePhoneInfo$e={meetingConferencePhoneInfoModalTitle:"Rejoindre l’appel à partir de votre téléphone",meetingConferencePhoneInfoModalDialIn:"Composer le numéro",meetingConferencePhoneInfoModalMeetingId:"Entrer l’ID de réunion",meetingConferencePhoneInfoModalWait:"Attendez d’être admis par l’organisateur",meetingConferencePhoneInfoModalNoPhoneAvailable:"Aucun téléphone disponible",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(numéro gratuit)",meetingConferencePhoneInfoModalToll:"(numéro payant)"};var IncomingCallNotification$e={incomingCallNotificationPlaceholderId:"Utilisateur inconnu",incomingCallNotificationPlaceholderAlert:"{callerName} appelle",incomingCallNoticicationAcceptWithAudioAriaLabel:"Accepter avec l'audio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Accepter avec la vidéo",incomingCallNoticicationRejectAriaLabel:"Rejeter",incomingCallNotificationAcceptButtonLabel:"Accepter",incomingCallNotificationRejectButtonLabel:"Refuser",incomingCallNotificationAccceptWithVideoButtonLabel:"Accepter avec une vidéo",incomingCallNotificationDismissButtonAriaLabel:"Ignorer"};var fr_FR$1 = {participantItem:participantItem$e,ParticipantList:ParticipantList$e,typingIndicator:typingIndicator$e,sendBox:sendBox$e,richTextSendBox:richTextSendBox$e,realTimeTextModal:realTimeTextModal$e,captionsBanner:captionsBanner$e,mentionPopover:mentionPopover$e,imageOverlay:imageOverlay$e,messageStatusIndicator:messageStatusIndicator$e,endCallButton:endCallButton$e,cameraButton:cameraButton$e,microphoneButton:microphoneButton$e,devicesButton:devicesButton$e,participantsButton:participantsButton$e,screenShareButton:screenShareButton$e,raiseHandButton:raiseHandButton$e,reactionButton:reactionButton$e,realTimeText:realTimeText$e,captionsSettingsModal:captionsSettingsModal$e,spokenLanguages:spokenLanguages$e,captionLanguages:captionLanguages$e,startCaptionsButton:startCaptionsButton$e,startRealTimeTextButton:startRealTimeTextButton$e,messageThread:messageThread$e,errorBar:errorBar$e,notificationStack:notificationStack$e,videoGallery:videoGallery$e,dialpad:dialpad$e,holdButton:holdButton$e,videoTile:videoTile$e,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$e,CameraSitePermissionsRequest:CameraSitePermissionsRequest$e,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$e,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$e,CameraSitePermissionsCheck:CameraSitePermissionsCheck$e,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$e,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$e,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$e,CameraSitePermissionsDenied:CameraSitePermissionsDenied$e,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$e,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$e,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$e,UnsupportedBrowser:UnsupportedBrowser$e,UnsupportedBrowserVersion:UnsupportedBrowserVersion$e,UnsupportedOperatingSystem:UnsupportedOperatingSystem$e,BrowserPermissionDenied:BrowserPermissionDenied$e,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$e,verticalGallery:verticalGallery$e,meetingConferencePhoneInfo:meetingConferencePhoneInfo$e,IncomingCallNotification:IncomingCallNotification$e};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for French (France).
 *
 * @public
 */
const COMPONENT_LOCALE_FR_FR = {
    strings: createComponentStrings(fr_FR$1)
};

var participantItem$d={isMeText:"(vous)",menuTitle:"Plus d’options",removeButtonLabel:"Supprimer",sharingIconLabel:"Partage",mutedIconLabel:"Muet activé",handRaisedIconLabel:"Main levée – Position {position}",displayNamePlaceholder:"Participant sans nom",participantStateRinging:"Appel en cours...",participantStateHold:"En attente",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"Plus d’options {displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",attendeeRole:"Participant(e)",micDisabledIconLabel:"Microphone désactivé",cameraDisabledIconLabel:"Caméra désactivée"};var ParticipantList$d={overflowParticipantCount:"+{overflowCount} plus"};var typingIndicator$d={singleUser:"{user} en train d’écrire...",multipleUsers:"{users} sont en train d'écrire...",multipleUsersAbbreviateOne:"{users} et 1 autre en train d'écrire...",multipleUsersAbbreviateMany:"{users} et {numOthers} autres en train d’écrire...",delimiter:", "};var sendBox$d={placeholderText:"Saisir un message",textTooLong:"La longueur de votre message dépasse la limite maximale.",sendButtonAriaLabel:"Envoyer un message",attachmentUploadsPendingError:"Chargement du fichier en cours, veuillez patienter.",removeAttachment:"Supprimer un fichier",uploading:"Chargement",uploadCompleted:"Téléchargement terminé",attachmentMoreMenu:"Plus d’options"};var richTextSendBox$d={placeholderText:"Saisir un message",textTooLong:"La longueur de votre message dépasse la limite maximale.",sendButtonAriaLabel:"Envoyer un message",attachmentUploadsPendingError:"Chargement du fichier en cours, veuillez patienter.",imageUploadsPendingError:"Chargement de l’image en cours, veuillez patienter.",removeAttachment:"Supprimer un fichier",uploading:"Chargement",uploadCompleted:"Téléchargement terminé",richTextBoldTooltip:"Gras",richTextItalicTooltip:"Italique",richTextUnderlineTooltip:"Souligner",richTextBulletListTooltip:"Liste à puces",richTextNumberListTooltip:"Liste numérotée",richTextIncreaseIndentTooltip:"Augmenter le retrait",richTextDecreaseIndentTooltip:"Réduire le retrait",richTextInsertTableTooltip:"Insérer un tableau",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insérer",richTextInsertTableMenuTitle:"Insérer une table {column} x {row}",richTextInsertRowAboveMenu:"Insérer au-dessus",richTextInsertRowBelowMenu:"Insérer l’élément ci-dessous",richTextInsertColumnLeftMenu:"Insérer à gauche",richTextInsertColumnRightMenu:"Insérer à droite",richTextDeleteRowOrColumnMenu:"Supprimer",richTextDeleteColumnMenu:"Supprimer la colonne",richTextDeleteRowMenu:"Supprimer une ligne",richTextDeleteTableMenu:"Supprimer le tableau",richTextToolbarAriaLabel:"Barre d’outils de formatage des messages",richTextToolbarMoreButtonAriaLabel:"Autres options",attachmentMoreMenu:"Plus d’options",richTextNewBulletedListItemAnnouncement:"Puce, nouvelle ligne",richTextNewNumberedListItemAnnouncement:"{0}, nouvelle ligne",richTextBulletedListAppliedAnnouncement:"Liste à puces appliquée",richTextNumberedListAppliedAnnouncement:"Liste numérotée appliquée"};var realTimeTextModal$d={realTimeTextModalAriaLabel:"Activer la boîte de dialogue modale RTT",realTimeTextModalTitle:"Activation de la RTT?",realTimeTextModalText:"Le texte en temps réel (RTT) sera activé pour tous les participants à la réunion.",realTimeTextConfirmButtonLabel:"Activer",realTimeTextCancelButtonLabel:"Annuler",realTimeTextCloseModalButtonAriaLabel:"Fermer la boîte de dialogue modale RTT"};var captionsBanner$d={captionsBannerSpinnerText:"Démarrage des légendes...",realTimeTextInputBoxDefaultText:"Inscrivez votre message en temps réel",realTimeTextInputErrorMessage:"Le message ne peut pas dépasser 2000 caractères",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (texte en temps réel) est activé pour tous les participants pendant toute la durée de la réunion.",realTimeTextBannerLinkLabel:"En savoir plus",captionsOnlyContainerTitle:"Sous-titres en direct",realTimeTextOnlyContainerTitle:"Texte en temps réel",captionsAndRealTimeTextContainerTitle:"Sous-titres en direct et texte en temps réel",expandButtonAriaLabel:"Étendre",minimizeButtonAriaLabel:"Réduire"};var mentionPopover$d={mentionPopoverHeader:"Suggestions"};var imageOverlay$d={downloadButtonLabel:"Télécharger",dismissButtonAriaLabel:"Fermer",overlayVisibleMessage:"Superposition des détails de l’image"};var messageStatusIndicator$d={deliveredAriaLabel:"Message envoyé",deliveredTooltipText:"Envoyé",seenAriaLabel:"Message vu par d’autres personnes",seenTooltipText:"Vu",readByTooltipText:"Lu par {messageThreadReadCount} participants sur {remoteParticipantsCount}",sendingAriaLabel:"Envoi du message",sendingTooltipText:"Envoi",failedToSendAriaLabel:"Échec d'envoi du message",failedToSendTooltipText:"Échec de l'envoi"};var endCallButton$d={label:"Quitter",tooltipContent:"Quitter l'appel",leaveOption:"Quitter",endCallOption:"Terminer l'appel pour tous"};var cameraButton$d={onLabel:"Caméra",offLabel:"Caméra",tooltipDisabledContent:"La caméra est désactivée.",tooltipOnContent:"Désactiver la caméra",tooltipOffContent:"Activer la caméra",tooltipVideoLoadingContent:"Chargement de la vidéo",cameraMenuTitle:"Caméra",cameraMenuTooltip:"Choisissez la caméra.",cameraButtonSplitRoleDescription:"Bouton partagé",onSplitButtonAriaLabel:"Désactiver la caméra",offSplitButtonAriaLabel:"Activer la caméra",cameraActionTurnedOnAnnouncement:"Votre caméra a été activée.",cameraActionTurnedOffAnnouncement:"Votre caméra a été désactivée.",onSplitButtonPrimaryActionCamera:"Désactiver la caméra",offSplitButtonPrimaryActionCamera:"Activer la caméra",cameraPrimaryActionSplitButtonTitle:"Utiliser la caméra",videoEffectsMenuItemTitle:"Effets",cameraButtonAriaDescription:"options de la caméra"};var microphoneButton$d={onLabel:"Microphone",offLabel:"Microphone",tooltipDisabledContent:"Le microphone est désactivé.",tooltipOnContent:"Désactiver le micro",tooltipOffContent:"Activer le micro",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choisissez le micro.",speakerMenuTitle:"Haut-parleur",speakerMenuTooltip:"Choisissez le haut-parleur.",microphoneButtonSplitRoleDescription:"Bouton partagé",onSplitButtonAriaLabel:"Désactiver le micro",offSplitButtonAriaLabel:"Réactiver le micro",microphoneActionTurnedOnAnnouncement:"Votre micro a été activé.",microphoneActionTurnedOffAnnouncement:"Votre micro a été désactivé.",offSplitButtonMicrophonePrimaryAction:"Réactiver le micro",onSplitButtonMicrophonePrimaryAction:"Désactiver le micro",microphonePrimaryActionSplitButtonTitle:"Utiliser le microphone",microphoneAriaDescription:"Options audio",deepNoiseSuppressionTitle:"Suppression du bruit",deepNoiseSuppressionOnAnnouncement:"La suppression du bruit profond a été activée.",deepNoiseSuppressionOffAnnouncement:"La suppression du bruit profond a été désactivée."};var devicesButton$d={label:"Périphériques",tooltipContent:"Gérer les périphériques",cameraMenuTitle:"Caméra",cameraMenuTooltip:"Choisissez la caméra.",audioDeviceMenuTitle:"Appareil audio",audioDeviceMenuTooltip:"Choisissez un périphérique appareil audio.",microphoneMenuTitle:"Microphone",microphoneMenuTooltip:"Choisissez le micro.",speakerMenuTitle:"Haut-parleur",speakerMenuTooltip:"Choisissez le haut-parleur."};var participantsButton$d={label:"Personnes",ariaLabel:"Contacts",tooltipContent:"Afficher les participants",menuHeader:"Pendant cet appel",participantsListButtonLabel:"{numParticipants} contacts",muteAllButtonLabel:"Désactiver tous les micros",copyInviteLinkButtonLabel:"Copier le lien d'invitation",copyInviteLinkActionedAriaLabel:"Le lien d’invitation a été copié."};var screenShareButton$d={onLabel:"Arrêter la présentation",offLabel:"Présenter",tooltipDisabledContent:"La présentation est désactivée.",tooltipOnContent:"Présentation de votre écran",tooltipOffContent:"Présenter votre écran"};var raiseHandButton$d={offLabel:"Augmenter",onLabel:"Inférieur",tooltipDisabledContent:"L'action lever la main est désactivée.",tooltipOnContent:"Baisser la main",tooltipOffContent:"Lever les mains"};var reactionButton$d={label:"Réagir",ariaLabel:"Réagir",tooltipDisabledContent:"L’action de réaction est désactivée.",tooltipContent:"Envoyer une réaction",likeReactionTooltipContent:"J’aime",heartReactionTooltipContent:"Aimer",laughReactionTooltipContent:"Rire",applauseReactionTooltipContent:"Acclamation",surprisedReactionTooltipContent:"Surpris"};var realTimeText$d={isTypingText:"Saisie RTT"};var captionsSettingsModal$d={captionsSettingsModalTitle:"Paramètres des sous-titres",captionsSettingsSpokenLanguageDropdownLabel:"Langue parlée",captionsSettingsCaptionLanguageDropdownLabel:"Langue des sous-titres",captionsSettingsSpokenLanguageDropdownInfoText:"Langue parlée par tous les participants à cet appel.",captionsSettingsCaptionLanguageDropdownInfoText:"Les sous-titres apparaîtront dans cette langue.",captionsSettingsConfirmButtonLabel:"Confirmer",captionsSettingsCancelButtonLabel:"Annuler",captionsSettingsModalAriaLabel:"Paramétrage des légendes Modal",captionsSettingsCloseModalButtonAriaLabel:"Paramètres de fermeture des sous-titres"};var spokenLanguages$d={"ar-ae":"Arabe – E.A.U.","ar-sa":"Arabe – Arabie saoudite","da-dk":"Danois","de-de":"Allemand – Allemagne","en-au":"Anglais – Australie","en-ca":"Anglais – Canada","en-gb":"Anglais – Royaume-Uni","en-in":"Anglais – Inde","en-nz":"Anglais – Nouvelle-Zélande","en-us":"Anglais – États-Unis","es-es":"Espagnol – Espagne (tri moderne)","es-mx":"Espagnol – Mexique","fi-fi":"Finnois","fr-ca":"Français – Canada","fr-fr":"Français – France","hi-in":"Hindi","it-it":"Italien – Italie","ja-jp":"Japonais","ko-kr":"Coréen","nb-no":"Norvégien (Bokmål)","nl-be":"Néerlandais – Belgique","nl-nl":"Néerlandais – Pays-Bas","pl-pl":"Polonais","pt-br":"Portugais – Brésil","ru-ru":"Russe","sv-se":"Suédois","zh-cn":"Chinois – République populaire de Chine","zh-hk":"Chinois – Hong Kong SAR","cs-cz":"Tchèque","pt-pt":"Portugais – Portugal","tr-tr":"Turc","vi-vn":"Vietnamien","th-th":"Thaï","he-il":"Hébreu","cy-gb":"Gallois","uk-ua":"Ukrainien","el-gr":"Grec","hu-hu":"Hongrois","ro-ro":"Roumain","sk-sk":"Slovaque","zh-tw":"Chinois – Taiwan"};var captionLanguages$d={ar:"Arabe",da:"Danois",de:"Allemand",en:"Anglais",es:"Espagnol",fi:"Finnois","fr-ca":"Français – Canada",fr:"Français – France",hi:"Hindi",it:"Italien",ja:"Japonais",ko:"Coréen",nb:"Norvégien (Bokmål)",nl:"Néerlandais",pl:"Polonais",pt:"Portugais – Brésil",ru:"Russe",sv:"Suédois","zh-Hans":"Chinois (simplifié)","zh-Hant":"Chinois (traditionnel)",cs:"Tchèque","pt-pt":"Portugais – Portugal",tr:"Turc",vi:"Vietnamien",th:"Thaï",he:"Hébreu",cy:"Gallois",uk:"Ukrainien",el:"Grec",hu:"Hongrois",ro:"Roumain",sk:"Slovaque"};var startCaptionsButton$d={onLabel:"Activer les sous-titres",offLabel:"Désactiver les sous-titres",tooltipOnContent:"Désactiver les sous-titres",tooltipOffContent:"Activer les sous-titres"};var startRealTimeTextButton$d={label:"Activer RTT pour cet appel",tooltipOffContent:"Activer RTT pour cet appel"};var messageThread$d={yesterday:"Hier",sunday:"Dimanche",monday:"Lundi",tuesday:"Mardi",wednesday:"Mercredi",thursday:"Jeudi",friday:"Vendredi",saturday:"Samedi",participantJoined:"a rejoint le clavardage.",participantLeft:"a quitté le clavardage.",editMessage:"Modifier",removeMessage:"Supprimer",resendMessage:"Réessayer l'envoi",failToSendTag:"Échec de l'envoi",editedTag:"Modifié",liveAuthorIntro:"Message de {author}",editedMessageLiveAuthorIntro:"Message de {author} modifié",editedMessageLocalUserLiveAuthorIntro:"Message modifié",messageContentAriaText:"{status} {author} a dit {message}.",messageContentMineAriaText:"{status} Vous avez dit {message}.",editBoxTextLimit:"Votre message dépasse la limite de {limitNumber} caractères.",editBoxPlaceholderText:"Modifier votre message",newMessagesIndicator:"Nouveaux messages",noDisplayNameSub:"Sans nom",editBoxCancelButton:"Annuler",editBoxSubmitButton:"Terminé",messageReadCount:"Lu par {messageReadByCount} participants sur {remoteParticipantsCount}",actionMenuMoreOptions:"Plus d’options",downloadAttachment:"Télécharger un fichier",openAttachment:"Ouvrir le fichier",blockedWarningText:"Ce message a été supprimé en raison d’une stratégie de l’organisation.",blockedWarningLinkText:"Détails",attachmentCardGroupMessage:"Le message a {attachmentCount} pièce jointe.",messageDeletedAnnouncementAriaLabel:"Le message est supprimé.",richTextBoldTooltip:"Gras",richTextItalicTooltip:"Italique",richTextUnderlineTooltip:"Souligner",richTextBulletListTooltip:"Liste à puces",richTextNumberListTooltip:"Liste numérotée",richTextIncreaseIndentTooltip:"Augmenter le retrait",richTextDecreaseIndentTooltip:"Réduire le retrait",richTextInsertTableTooltip:"Insérer un tableau",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Insérer",richTextInsertTableMenuTitle:"Insérer une table {column} x {row}",richTextInsertRowAboveMenu:"Insérer au-dessus",richTextInsertRowBelowMenu:"Insérer l’élément ci-dessous",richTextInsertColumnLeftMenu:"Insérer à gauche",richTextInsertColumnRightMenu:"Insérer à droite",richTextDeleteRowOrColumnMenu:"Supprimer",richTextDeleteColumnMenu:"Supprimer la colonne",richTextDeleteRowMenu:"Supprimer une ligne",richTextDeleteTableMenu:"Supprimer le tableau",richTextToolbarMoreButtonAriaLabel:"Autres options",attachmentUploadsPendingError:"Chargement du fichier en cours, veuillez patienter.",imageUploadsPendingError:"Chargement de l’image en cours, veuillez patienter.",richTextNewBulletedListItemAnnouncement:"Puce, nouvelle ligne",richTextNewNumberedListItemAnnouncement:"{0}, nouvelle ligne",richTextBulletedListAppliedAnnouncement:"Liste à puces appliquée",richTextNumberedListAppliedAnnouncement:"Liste numérotée appliquée"};var errorBar$d={unableToReachChatService:"Vous êtes hors connexion.",accessDenied:"Impossible d’accéder aux services de clavardage. Vérifiez les informations d’identification de l’utilisateur(-trice) fournies.",userNotInChatThread:"Vous n’êtes plus dans ce fil de clavardage.",sendMessageNotInChatThread:"Échec de l’envoi du message, car vous n’êtes plus dans ce fil de clavardage.",sendMessageGeneric:"Échec de l’envoi du message",callingNetworkFailure:"Perturbation de l’appel de connexion : vous semblez être hors connexion.",startVideoGeneric:"Échec du démarrage de la vidéo",stopVideoGeneric:"Échec de l’arrêt de la vidéo",muteGeneric:"Échec de la désactivation du son du microphone",unmuteGeneric:"Échec de l’activation du son du microphone",speakingWhileMuted:"Votre micro est désactivé.",startScreenShareGeneric:"Désolé, un problème est survenu durant le démarrage du partage d’écran.",stopScreenShareGeneric:"Échec de l’arrêt du partage d’écran",callNetworkQualityLow:"La qualité de réseau est faible.",teamsMeetingCallNetworkQualityLow:"La qualité de réseau est faible.",callNoSpeakerFound:"Aucun haut-parleur ou casque n’a été trouvé. Connectez un périphérique audio pour entendre l’appel.",callNoMicrophoneFound:"Aucun micro n’a été trouvé. Connectez un périphérique d’entrée audio.",callMicrophoneAccessDenied:"Impossible d’accéder au microphone. Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",callMicrophoneAccessDeniedSafari:"Impossible d’accéder au microphone. Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site web.",callMicrophoneMutedBySystem:"Votre micro est désactivé par votre système.",callMicrophoneUnmutedBySystem:"Votre micro a été récupéré et la son a été activé par votre système.",callMacOsMicrophoneAccessDenied:"Impossible d’accéder au microphone. Accordez l’autorisation de microphone dans vos paramètres de confidentialité macOS.",callLocalVideoFreeze:"La bande passante réseau est médiocre. Votre vidéo peut apparaître suspendue pour les autres participants à l’appel.",callCameraAccessDenied:"Impossible d’accéder à la caméra. Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",callCameraAccessDeniedSafari:"Impossible d’accéder à la caméra. Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site Web.",callCameraAlreadyInUse:"Impossible d’accéder à la caméra. Il est peut-être déjà utilisé par une autre application.",callVideoStoppedBySystem:"Votre vidéo a été arrêtée par votre système.",callVideoRecoveredBySystem:"Votre vidéo a repris.",callMacOsCameraAccessDenied:"MacOS bloque l’accès à votre caméra. Mettez à jour vos paramètres de confidentialité pour autoriser ce navigateur à accéder à votre caméra.",callMacOsScreenShareAccessDenied:"MacOS bloque le partage d’écran. Mettez à jour vos paramètres de confidentialité pour permettre à ce navigateur d’enregistrer votre écran.",dismissButtonAriaLabel:"Fermer",failedToJoinCallGeneric:"Désolé, nous n’avons pas pu rejoindre l’appel.",failedToJoinCallInvalidMeetingLink:"Impossible de rejoindre la réunion. Lien non valide.",cameraFrozenForRemoteParticipants:"Les utilisateurs de l’appel rencontrent des problèmes pour voir votre vidéo. Vérifiez vos périphériques et votre réseau.",unableToStartVideoEffect:"Impossible d’appliquer l’effet vidéo.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Impossible de lancer la mise en évidence sur le(s) participant(s), car le nombre maximum de participants est déjà mis en évidence.",mutedByRemoteParticipant:"Votre micro a été désactivé par un(e) autre participant(e)."};var notificationStack$d={callingNetworkFailure:{title:"Appel de connexion en cours",message:"Vous semblez être hors connexion. Veuillez vérifier votre connexion réseau et réessayer.",dismissButtonAriaLabel:"Fermer"},startVideoGeneric:{title:"Échec du démarrage de la vidéo.",dismissButtonAriaLabel:"Fermer"},stopVideoGeneric:{title:"Échec de l’arrêt de la vidéo.",dismissButtonAriaLabel:"Fermer"},muteGeneric:{title:"Impossible de désactiver le son.",dismissButtonAriaLabel:"Fermer"},unmuteGeneric:{title:"Échec de l’activation du micro.",dismissButtonAriaLabel:"Fermer"},speakingWhileMuted:{title:"Le son de votre micro est désactivé.",dismissButtonAriaLabel:"Fermer"},startScreenShareGeneric:{title:"Désolé, un problème est survenu durant le démarrage du partage d’écran.",dismissButtonAriaLabel:"Fermer"},stopScreenShareGeneric:{title:"Échec de l’arrêt du partage d’écran.",dismissButtonAriaLabel:"Fermer"},callNetworkQualityLow:{title:"La qualité de réseau est faible.",dismissButtonAriaLabel:"Fermer"},teamsMeetingCallNetworkQualityLow:{title:"La qualité de réseau est faible.",dismissButtonAriaLabel:"Fermer",message:"Rejoignez cet appel à partir de votre téléphone pour obtenir un meilleur son. Vous pouvez continuer à afficher la réunion sur ce périphérique.",primaryButtonLabel:"Participer par téléphone"},callNoSpeakerFound:{title:"Haut-parleurs ou casque introuvables",message:"Connectez un appareil audio pour entendre l’appel.",dismissButtonAriaLabel:"Fermer"},callNoMicrophoneFound:{title:"Aucun microphone trouvé",message:"Connectez un périphérique d’entrée audio.",dismissButtonAriaLabel:"Fermer"},callMicrophoneAccessDenied:{title:"Désolé, nous n’avons pas pu accéder au micro.",message:"Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",dismissButtonAriaLabel:"Fermer"},callMicrophoneAccessDeniedSafari:{title:"Désolé, nous n’avons pas pu accéder au micro.",message:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site Web.",dismissButtonAriaLabel:"Fermer"},callMicrophoneMutedBySystem:{title:"Votre micro est désactivé par votre système.",dismissButtonAriaLabel:"Fermer"},callMicrophoneUnmutedBySystem:{title:"Votre micro a été récupéré et la son a été activé par votre système.",dismissButtonAriaLabel:"Fermer"},callMacOsMicrophoneAccessDenied:{title:"Désolé, nous n’avons pas pu accéder au micro.",message:"Accordez l’autorisation de microphone dans vos paramètres de confidentialité macOS.",dismissButtonAriaLabel:"Fermer"},callLocalVideoFreeze:{title:"La bande passante réseau est médiocre.",message:"Votre vidéo peut apparaître suspendue pour d’autres participants à l’appel.",dismissButtonAriaLabel:"Fermer"},callCameraAccessDenied:{title:"Désolé, nous n’avons pas pu accéder à la caméra.",message:"Cliquez sur le verrou dans la barre d’adresses pour accorder l’autorisation à cette page Web.",dismissButtonAriaLabel:"Fermer"},callCameraAccessDeniedSafari:{title:"Désolé, nous n’avons pas pu accéder à la caméra.",message:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site Web.",dismissButtonAriaLabel:"Fermer"},callCameraAlreadyInUse:{title:"Désolé, nous n’avons pas pu accéder à la caméra.",message:"Il est peut-être déjà utilisé par une autre application.",dismissButtonAriaLabel:"Fermer"},callVideoStoppedBySystem:{title:"Votre vidéo a été arrêtée par votre système.",dismissButtonAriaLabel:"Fermer"},callVideoRecoveredBySystem:{title:"Votre vidéo a repris.",dismissButtonAriaLabel:"Fermer"},callMacOsCameraAccessDenied:{title:"MacOS bloque l’accès à votre caméra.",message:"Mettez à jour vos paramètres de confidentialité pour autoriser ce navigateur à accéder à votre caméra.",dismissButtonAriaLabel:"Fermer"},callMacOsScreenShareAccessDenied:{title:"MacOS bloque le partage d’écran",message:"Mettez à jour vos paramètres de confidentialité pour permettre à ce navigateur d’enregistrer votre écran.",dismissButtonAriaLabel:"Fermer"},failedToJoinCallGeneric:{title:"Désolé, nous n’avons pas pu rejoindre l’appel.",dismissButtonAriaLabel:"Fermer"},failedToJoinCallInvalidMeetingLink:{title:"Impossible de rejoindre la réunion. Lien non valide.",dismissButtonAriaLabel:"Fermer"},cameraFrozenForRemoteParticipants:{title:"Les utilisateurs de l’appel rencontrent des problèmes pour voir votre vidéo.",message:"Vérifiez vos périphériques et votre réseau.",dismissButtonAriaLabel:"Fermer"},unableToStartVideoEffect:{title:"Impossible d’appliquer l’effet vidéo.",dismissButtonAriaLabel:"Fermer"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Impossible de mettre en évidence le ou les participants",message:"Le nombre maximum de participants est déjà mis en évidence.",dismissButtonAriaLabel:"Fermer"},mutedByRemoteParticipant:{title:"Votre micro a été désactivé par un(e) autre participant(e).",dismissButtonAriaLabel:"Fermer"},recordingStarted:{title:"L'enregistrement a démarré.",message:"En participant, vous autorisez la transcription de cette réunion.",dismissButtonAriaLabel:"Fermer"},transcriptionStarted:{title:"La transcription a démarré.",message:"En participant, vous autorisez la transcription de cette réunion.",dismissButtonAriaLabel:"Fermer"},recordingStopped:{title:"Enregistrement en cours de sauvegarde.",message:"L’enregistrement s’est arrêté.",dismissButtonAriaLabel:"Fermer"},transcriptionStopped:{title:"La transcription est en cours d’enregistrement.",message:"Arrêt de la transcription.",dismissButtonAriaLabel:"Fermer"},recordingAndTranscriptionStarted:{title:"Démarrage de l’enregistrement et de la transcription.",message:"En participant, vous autorisez la transcription de cette réunion.",dismissButtonAriaLabel:"Fermer"},recordingAndTranscriptionStopped:{title:"Sauvegarde en cours de l’enregistrement et de la transcription.",message:"L’enregistrement et la transcription ont pris fin.",dismissButtonAriaLabel:"Fermer"},recordingStoppedStillTranscribing:{title:"L’enregistrement s’est arrêté.",message:"À présent, vous ne transcrivez que cette réunion.",dismissButtonAriaLabel:"Fermer"},transcriptionStoppedStillRecording:{title:"Arrêt de la transcription.",message:"Vous n’enregistrez désormais que cette réunion.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomOpened:{title:"Nous allons vous déplacer vers la salle affectée dans 5 secondes.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomChanged:{title:"Votre salle pour petit groupe a changé.",message:"Nous allons vous déplacer vers la salle affectée dans 5 secondes.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomClosed:{title:"Votre salle pour petit groupe s’est fermée",message:"Nous vous redirigerons vers votre réunion dans 5 secondes.",dismissButtonAriaLabel:"Fermer"},assignedBreakoutRoomOpenedPromptJoin:{title:"Voulez-vous rejoindre cette salle pour petit groupe?",message:"Vous avez reçu une invitation à rejoindre une salle pour petit groupe.",dismissButtonAriaLabel:"Fermer",primaryButtonLabel:"Rejoindre une salle",secondaryButtonLabel:"Plus tard"},breakoutRoomJoined:{title:"Vous avez rejoint une salle pour petit groupe.",dismissButtonAriaLabel:"Fermer"},breakoutRoomClosingSoon:{title:"Limite de temps de la salle sur le point d’expirer.",message:"Cette salle se fermera dans 30 secondes.",dismissButtonAriaLabel:"Fermer"},capabilityTurnVideoOnPresent:{title:"Votre caméra a été activée",message:"Activez-la si vous souhaitez partager une vidéo.",dismissButtonAriaLabel:"Fermer"},capabilityTurnVideoOnAbsent:{title:"Votre caméra est désactivée",message:"Vous ne pouvez plus partager de vidéo.",dismissButtonAriaLabel:"Fermer"},capabilityUnmuteMicPresent:{title:"Votre microphone a été activé",message:"Activer le son si vous voulez parler.",dismissButtonAriaLabel:"Fermer"},capabilityUnmuteMicAbsent:{title:"Votre microphone est désactivé",message:"Vous ne pouvez plus activer le son.",dismissButtonAriaLabel:"Fermer"},togetherModeStarted:{title:"Togethermode a démarré",dismissButtonAriaLabel:"Fermez"},togetherModeEnded:{title:"Togethermode est terminé",dismissButtonAriaLabel:"Fermez"},transcriptionError:{title:"Désolé, une erreur s’est produite lors de la transcription.",message:"Désolé, la transcription a été arrêtée. Veuillez réessayer plus tard",dismissButtonAriaLabel:"Fermer"},transcriptionStartedByYou:{title:"La transcription a démarré.",message:"Démarrée par vous. Faites savoir à tout le monde qu'ils sont transcrits.",dismissButtonAriaLabel:"Fermer"}};var videoGallery$d={screenIsBeingSharedMessage:"Vous partagez votre écran.",screenShareLoadingMessage:"Chargement de l’écran de {participant}",localScreenShareLoadingMessage:"Chargement de votre écran",localVideoLabel:"Vous",localVideoCameraSwitcherLabel:"Changer de caméra",localVideoMovementLabel:"Vignette de vidéo locale déplaçable",localVideoMovementAriaLabel:"Vignette de vidéo locale déplaçable",localVideoSelectedDescription:"{cameraName} sélectionnée",displayNamePlaceholder:"Participant(e) sans nom",fitRemoteParticipantToFrame:"Ajuster à l'image",fillRemoteParticipantFrame:"Remplir l'image",pinParticipantForMe:"Épingler pour moi",pinParticipantForMeLimitReached:"Épingler (limite atteinte)",unpinParticipantForMe:"Détacher",pinParticipantMenuItemAriaLabel:"Épingler {participantName}",unpinParticipantMenuItemAriaLabel:"Détacher {participantName}",pinnedParticipantAnnouncementAriaLabel:"{participantName} a été épinglé.",unpinnedParticipantAnnouncementAriaLabel:"{participantName} a été détaché.",startSpotlightVideoTileMenuLabel:"Mettre en évidence pour tout le monde",addSpotlightVideoTileMenuLabel:"Ajouter la mise en évidence",spotlightLimitReachedMenuTitle:"Limite de mise en évidence atteinte",stopSpotlightVideoTileMenuLabel:"Arrêter la mise en évidence",stopSpotlightOnSelfVideoTileMenuLabel:"Quitter la mise en évidence",attendeeRole:"Participant(e)",muteParticipantMenuItemLabel:"Désactiver le micro/muet",waitingScreenText:"En attente des autres participants",forbidAudio:"Désactiver le micro",permitAudio:"Autoriser le micro",forbidAudioTileMenuLabel:"Désactiver le micro",permitAudioTileMenuLabel:"Autoriser le micro",forbidVideo:"Désactiver la caméra",permitVideo:"Autoriser la caméra",forbidVideoTileMenuLabel:"Désactiver la caméra",permitVideoTileMenuLabel:"Autoriser la caméra"};var dialpad$d={placeholderText:"Entrer un numéro de téléphone",deleteButtonAriaLabel:"Supprimer"};var holdButton$d={onLabel:"Reprendre",offLabel:"Mettre en attente",tooltipOnContent:"Reprendre l’appel",tooltipOffContent:"Mettre l’appel en attente"};var videoTile$d={participantStateRinging:"Appel en cours...",participantReconnecting:"Reconnexion...",participantStateHold:"En attente",moreOptionsButtonAriaLabel:"Plus d’options {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"son désactivé",moreOptionsParticipantMutedStateUnmutedAriaLabel:"son réactivé",moreOptionsParticipantHandRaisedAriaLabel:"Main levée",moreOptionsParticipantIsSpeakingAriaLabel:"Parler",moreOptionsParticipantMicDisabledAriaLabel:"Microphone désactivé",moreOptionsParticipantCameraDisabledAriaLabel:"Caméra désactivée"};var CameraAndMicrophoneSitePermissionsRequest$d={primaryText:"Autoriser {appName} à utiliser votre caméra et votre microphone",secondaryText:"Cela permet aux participants de vous voir et de vous entendre.",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans caméra ni micro",ariaLabel:"Autoriser l'accès à la caméra et au microphone"};var CameraSitePermissionsRequest$d={primaryText:"Autoriser {appName} à utiliser votre caméra",secondaryText:"Cela permet aux participants de vous voir.",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans caméra",ariaLabel:"Autoriser l’accès à la caméra"};var MicrophoneSitePermissionsRequest$d={primaryText:"Autoriser {appName} à utiliser votre micro",secondaryText:"Cela permet aux participants de vous écouter.",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans microphone",ariaLabel:"Autoriser l’accès au micro"};var CameraAndMicrophoneSitePermissionsCheck$d={primaryText:"Vérification de l’accès à la caméra et au microphone",secondaryText:"Autorisez l’accès si vous y êtes invité(e). Cela permet aux participants de vous voir et de vous entendre.",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans caméra ni micro",ariaLabel:"Vérification de l’accès à la caméra et au microphone. Autorisez l’accès si vous y êtes invité(e)."};var CameraSitePermissionsCheck$d={primaryText:"Vérification de l’accès à la caméra",secondaryText:"Autorisez l’accès si vous y êtes invité. Cela permet aux participants de vous voir.",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans caméra",ariaLabel:"Vérification de l’accès à la caméra. Autorisez l’accès si vous y êtes invité(e)."};var MicrophoneSitePermissionsCheck$d={primaryText:"Vérification de l’accès au microphone",secondaryText:"Autorisez l’accès si vous y êtes invité. Cela permet aux participants de vous entendre.",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes",primaryButtonText:"Continuer sans microphone",ariaLabel:"Vérification de l’accès au microphone. Autorisez l’accès si vous y êtes invité(e)."};var CameraAndMicrophoneSitePermissionsDenied$d={primaryText:"Désolé, nous n’avons pas pu accéder au micro et à la caméra.",secondaryText:"Cliquez sur l’icône de verrouillage dans la barre d’adresses pour accorder des autorisations de microphone à cette page Web. Une actualisation de page peut être nécessaire.",primaryButtonText:"Continuer sans caméra ni micro",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes"};var CameraAndMicrophoneSitePermissionsDeniedSafari$d={primaryText:"Désolé, nous n’avons pas pu accéder au micro et à la caméra.",secondaryText:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site Web.",primaryButtonText:"Continuer sans caméra ni micro",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes"};var CameraSitePermissionsDenied$d={primaryText:"Désolé, nous n’avons pas pu accéder à la caméra.",secondaryText:"Cliquez sur l’icône de verrouillage dans la barre d’adresses pour accorder des autorisations de caméra à cette page Web. Une actualisation de page peut être nécessaire.",primaryButtonText:"Continuer sans caméra",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes"};var MicrophoneSitePermissionsDenied$d={primaryText:"Désolé, nous n’avons pas pu accéder au micro.",secondaryText:"Cliquez sur l’icône de verrouillage dans la barre d’adresses pour accorder des autorisations de microphone à cette page Web. Une actualisation de page peut être nécessaire.",primaryButtonText:"Continuer sans microphone",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes"};var CameraSitePermissionsDeniedSafari$d={primaryText:"Désolé, nous n’avons pas pu accéder à la caméra.",secondaryText:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site Web.",primaryButtonText:"Continuer sans caméra",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes"};var MicrophoneSitePermissionsDeniedSafari$d={primaryText:"Désolé, nous n’avons pas pu accéder au micro.",secondaryText:"Actualisez la page pour autoriser les autorisations, ou vérifiez les paramètres de ce navigateur et vérifiez que les autorisations sont activées pour ce site Web.",primaryButtonText:"Continuer sans microphone",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes"};var UnsupportedBrowser$d={primaryText:"Navigateur non pris en charge",secondaryText:"Rejoignez cet appel à l’aide d’un navigateur compatible.",moreHelpLinkText:"Voir les exigences de compatibilité"};var UnsupportedBrowserVersion$d={primaryText:"Mise à jour du navigateur nécessaire",secondaryText:"Pour garantir le meilleur appel possible, veuillez mettre à jour votre navigateur, puis réessayer de rejoindre l’appel.",moreHelpLinkText:"Voir les exigences de compatibilité",continueAnywayButtonText:"Démarrer l’appel sans mettre à jour"};var UnsupportedOperatingSystem$d={primaryText:"Système d'exploitation non pris en charge",secondaryText:"Rejoignez cet appel à l’aide d’un périphérique avec un système d’exploitation compatible.",moreHelpLinkText:"Voir les exigences de compatibilité"};var BrowserPermissionDenied$d={primaryText:"Impossible d’utiliser votre caméra ou votre micro",secondaryText:"Votre navigateur n’a peut-être pas accès à votre caméra ou à votre micro. Pour résoudre ce problème, ouvrez Préférences système.",primaryButtonText:"Réessayer",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes"};var BrowserPermissionDeniedIOS$d={primaryText:"Autorisez l’accès au microphone pour continuer.",secondaryText:"Ainsi, les autres participants peuvent vous entendre.",primaryButtonText:"Réessayer",imageAltText:"Emplacement des autorisations pour le microphone et la caméra pour iOS",linkText:"Avez-vous besoin d’aide? Obtenir de l’aide sur la résolution des problèmes",step1Text:"Accéder à l’application Paramètres",step2Text:"Faites défiler vers le bas pour accéder aux paramètres de ce navigateur.",step3Text:"Activez le microphone (caméra facultative).",step4Text:"Réessayez de rejoindre l’appel.",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$d={leftNavButtonAriaLabel:"page précédente {current} de {total}",rightNavButtonAriaLabel:"page suivante{current} de {total}"};var meetingConferencePhoneInfo$d={meetingConferencePhoneInfoModalTitle:"Rejoindre l’appel à partir de votre téléphone",meetingConferencePhoneInfoModalDialIn:"Composer le numéro",meetingConferencePhoneInfoModalMeetingId:"Entrer l’ID de réunion",meetingConferencePhoneInfoModalWait:"Attendez d’être admis par l’organisateur(-trice).",meetingConferencePhoneInfoModalNoPhoneAvailable:"Aucun téléphone disponible",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(numéro gratuit)",meetingConferencePhoneInfoModalToll:"(numéro payant)"};var IncomingCallNotification$d={incomingCallNotificationPlaceholderId:"Utilisateur(-trice) inconnu(e)",incomingCallNotificationPlaceholderAlert:"{callerName} appelle",incomingCallNoticicationAcceptWithAudioAriaLabel:"Accepter avec l'audio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Accepter avec la vidéo",incomingCallNoticicationRejectAriaLabel:"Rejeter",incomingCallNotificationAcceptButtonLabel:"Accepter",incomingCallNotificationRejectButtonLabel:"Refuser",incomingCallNotificationAccceptWithVideoButtonLabel:"Accepter avec une vidéo",incomingCallNotificationDismissButtonAriaLabel:"Ignorer"};var fr_CA$1 = {participantItem:participantItem$d,ParticipantList:ParticipantList$d,typingIndicator:typingIndicator$d,sendBox:sendBox$d,richTextSendBox:richTextSendBox$d,realTimeTextModal:realTimeTextModal$d,captionsBanner:captionsBanner$d,mentionPopover:mentionPopover$d,imageOverlay:imageOverlay$d,messageStatusIndicator:messageStatusIndicator$d,endCallButton:endCallButton$d,cameraButton:cameraButton$d,microphoneButton:microphoneButton$d,devicesButton:devicesButton$d,participantsButton:participantsButton$d,screenShareButton:screenShareButton$d,raiseHandButton:raiseHandButton$d,reactionButton:reactionButton$d,realTimeText:realTimeText$d,captionsSettingsModal:captionsSettingsModal$d,spokenLanguages:spokenLanguages$d,captionLanguages:captionLanguages$d,startCaptionsButton:startCaptionsButton$d,startRealTimeTextButton:startRealTimeTextButton$d,messageThread:messageThread$d,errorBar:errorBar$d,notificationStack:notificationStack$d,videoGallery:videoGallery$d,dialpad:dialpad$d,holdButton:holdButton$d,videoTile:videoTile$d,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$d,CameraSitePermissionsRequest:CameraSitePermissionsRequest$d,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$d,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$d,CameraSitePermissionsCheck:CameraSitePermissionsCheck$d,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$d,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$d,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$d,CameraSitePermissionsDenied:CameraSitePermissionsDenied$d,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$d,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$d,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$d,UnsupportedBrowser:UnsupportedBrowser$d,UnsupportedBrowserVersion:UnsupportedBrowserVersion$d,UnsupportedOperatingSystem:UnsupportedOperatingSystem$d,BrowserPermissionDenied:BrowserPermissionDenied$d,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$d,verticalGallery:verticalGallery$d,meetingConferencePhoneInfo:meetingConferencePhoneInfo$d,IncomingCallNotification:IncomingCallNotification$d};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for French (Canada).
 *
 * @public
 */
const COMPONENT_LOCALE_FR_CA = {
    strings: createComponentStrings(fr_CA$1)
};

var participantItem$c={isMeText:"(אתה)",menuTitle:"אפשרויות נוספות",removeButtonLabel:"הסר",sharingIconLabel:"משתף",mutedIconLabel:"מושתק",handRaisedIconLabel:"יד מורמת - מיקום {position}",displayNamePlaceholder:"משתתף ללא שם",participantStateRinging:"מתקשר...",participantStateHold:"בהמתנה",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, אפשרויות נוספות",attendeeRole:"משתתף",micDisabledIconLabel:"המיקרופון לא זמין",cameraDisabledIconLabel:"המצלמה לא זמינה"};var ParticipantList$c={overflowParticipantCount:"+{overflowCount} נוספים"};var typingIndicator$c={singleUser:"{user} מקליד...",multipleUsers:"{users} מקלידים...",multipleUsersAbbreviateOne:"{users} ומשתמש אחד נוסף מקלידים...",multipleUsersAbbreviateMany:"{users} ו{numOthers} אחרים מקלידים...",delimiter:", "};var sendBox$c={placeholderText:"הזן הודעה",textTooLong:"אורך ההודעה שלך חורג מהמגבלה המרבית.",sendButtonAriaLabel:"שלח הודעה",attachmentUploadsPendingError:"הקובץ בהעלאה, נא המתן.",removeAttachment:"הסרת קובץ",uploading:"מעלה",uploadCompleted:"ההעלאה הושלמה",attachmentMoreMenu:"אפשרויות נוספות"};var richTextSendBox$c={placeholderText:"הזן הודעה",textTooLong:"אורך ההודעה שלך חורג מהמגבלה המרבית.",sendButtonAriaLabel:"שלח הודעה",attachmentUploadsPendingError:"הקובץ בהעלאה, נא המתן.",imageUploadsPendingError:"התמונה בהעלאה, נא המתן.",removeAttachment:"הסר קובץ",uploading:"מעלה",uploadCompleted:"ההעלאה הושלמה",richTextBoldTooltip:"מודגש",richTextItalicTooltip:"נטוי",richTextUnderlineTooltip:"קו תחתון",richTextBulletListTooltip:"רשימה עם תבליטים",richTextNumberListTooltip:"רשימה ממוספרת",richTextIncreaseIndentTooltip:"הגדל כניסה",richTextDecreaseIndentTooltip:"הקטן כניסה",richTextInsertTableTooltip:"הוסף טבלה",richTextFormatButtonTooltip:"תבנית",richTextInsertRowOrColumnMenu:"הוסף",richTextInsertTableMenuTitle:"הוסף טבלה {column} x‏ {row}",richTextInsertRowAboveMenu:"הוסף מעל",richTextInsertRowBelowMenu:"הוסף מתחת",richTextInsertColumnLeftMenu:"הוסף משמאל",richTextInsertColumnRightMenu:"הוסף מימין",richTextDeleteRowOrColumnMenu:"מחק",richTextDeleteColumnMenu:"מחק עמודה",richTextDeleteRowMenu:"מחק שורה",richTextDeleteTableMenu:"מחק טבלה",richTextToolbarAriaLabel:"סרגל כלים של עיצוב הודעה",richTextToolbarMoreButtonAriaLabel:"אפשרויות נוספות",attachmentMoreMenu:"אפשרויות נוספות",richTextNewBulletedListItemAnnouncement:"תבליט, שורה חדשה",richTextNewNumberedListItemAnnouncement:"{0}, שורה חדשה",richTextBulletedListAppliedAnnouncement:"הוחלה רשימה עם תבליטים",richTextNumberedListAppliedAnnouncement:"רשימה ממוספרת הוחלה"};var realTimeTextModal$c={realTimeTextModalAriaLabel:"הפעל תיבת דו-שיח מודאלית של RTT",realTimeTextModalTitle:"להפעיל RTT?",realTimeTextModalText:"טקסט בזמן אמת (RTT) יופעל עבור כל האנשים בפגישה.",realTimeTextConfirmButtonLabel:"הפעל",realTimeTextCancelButtonLabel:"ביטול",realTimeTextCloseModalButtonAriaLabel:"סגור תיבת דו-שיח מודאלית של RTT"};var captionsBanner$c={captionsBannerSpinnerText:"מאתחל כתוביות...",realTimeTextInputBoxDefaultText:"הקלד הודעה בזמן אמת",realTimeTextInputErrorMessage:"ההודעה אינה יכולה לחרוג מ- 2,000 תווים",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (טקסט בזמן אמת) זמין עבור כל המשתתפים למשך הזמן כולו של הפגישה.",realTimeTextBannerLinkLabel:"מידע נוסף",captionsOnlyContainerTitle:"כתוביות",realTimeTextOnlyContainerTitle:"טקסט בזמן אמת",captionsAndRealTimeTextContainerTitle:"כתוביות וטקסט בזמן אמת",expandButtonAriaLabel:"הרחב",minimizeButtonAriaLabel:"כווץ"};var mentionPopover$c={mentionPopoverHeader:"הצעות"};var imageOverlay$c={downloadButtonLabel:"הורד",dismissButtonAriaLabel:"סגור",overlayVisibleMessage:"שכבת-על של פרטי תמונה"};var messageStatusIndicator$c={deliveredAriaLabel:"ההודעה נשלחה",deliveredTooltipText:"‏‏נשלח",seenAriaLabel:"הודעה שנראתה על-ידי אחרים",seenTooltipText:"נראה",readByTooltipText:"נקרא על-ידי {messageThreadReadCount} מתוך {remoteParticipantsCount}",sendingAriaLabel:"שליחת הודעה",sendingTooltipText:"שולח",failedToSendAriaLabel:"שליחת ההודעה נכשלה",failedToSendTooltipText:"השליחה נכשלה"};var endCallButton$c={label:"עזוב",tooltipContent:"עזוב את השיחה",leaveOption:"צא",endCallOption:"סיים את השיחה לכולם"};var cameraButton$c={onLabel:"מצלמה",offLabel:"מצלמה",tooltipDisabledContent:"המצלמה אינה זמינה",tooltipOnContent:"כבה מצלמה",tooltipOffContent:"הפעל מצלמה",tooltipVideoLoadingContent:"הווידאו נטען",cameraMenuTitle:"מצלמה",cameraMenuTooltip:"בחר מצלמה",cameraButtonSplitRoleDescription:"לחצן פיצול",onSplitButtonAriaLabel:"כבה מצלמה",offSplitButtonAriaLabel:"הפעל את המצלמה",cameraActionTurnedOnAnnouncement:"המצלמה שלך הופעלה",cameraActionTurnedOffAnnouncement:"המצלמה שלך כבויה",onSplitButtonPrimaryActionCamera:"כבה מצלמה",offSplitButtonPrimaryActionCamera:"הפעל את המצלמה",cameraPrimaryActionSplitButtonTitle:"השתמש במצלמה",videoEffectsMenuItemTitle:"אפקטים",cameraButtonAriaDescription:"אפשרויות מצלמה"};var microphoneButton$c={onLabel:"מיקרופון",offLabel:"מיקרופון",tooltipDisabledContent:"המיקרופון אינו זמין",tooltipOnContent:"השתק מיקרופון",tooltipOffContent:"בטל השתקה של מיקרופון",microphoneMenuTitle:"מיקרופון",microphoneMenuTooltip:"בחר מיקרופון",speakerMenuTitle:"רמקול",speakerMenuTooltip:"בחר רמקול",microphoneButtonSplitRoleDescription:"לחצן פיצול",onSplitButtonAriaLabel:"השתק את המיקרופון",offSplitButtonAriaLabel:"בטל את השתקת המיקרופון",microphoneActionTurnedOnAnnouncement:"המיקרופון שלך הופעל",microphoneActionTurnedOffAnnouncement:"המיקרופון שלך כובה",offSplitButtonMicrophonePrimaryAction:"בטל את השתקת המיקרופון",onSplitButtonMicrophonePrimaryAction:"השתק את המיקרופון",microphonePrimaryActionSplitButtonTitle:"השתמש במיקרופון",microphoneAriaDescription:"אפשרויות שמע",deepNoiseSuppressionTitle:"העלמת רעש",deepNoiseSuppressionOnAnnouncement:"דיכוי רעשים עמוקים הופעל",deepNoiseSuppressionOffAnnouncement:"דיכוי רעשים עמוקים בוטל"};var devicesButton$c={label:"מכשירים",tooltipContent:"נהל מכשירים",cameraMenuTitle:"מצלמה",cameraMenuTooltip:"בחר מצלמה",audioDeviceMenuTitle:"התקן שמע",audioDeviceMenuTooltip:"בחר התקן שמע",microphoneMenuTitle:"מיקרופון",microphoneMenuTooltip:"בחר מיקרופון",speakerMenuTitle:"רמקול",speakerMenuTooltip:"בחר רמקול"};var participantsButton$c={label:"אנשים",ariaLabel:"אנשים",tooltipContent:"הצג משתתפים",menuHeader:"בשיחה זו",participantsListButtonLabel:"{numParticipants} אנשים",muteAllButtonLabel:"השתק הכל",copyInviteLinkButtonLabel:"העתק קישור הזמנה",copyInviteLinkActionedAriaLabel:"קישור ההזמנה הועתק"};var screenShareButton$c={onLabel:"הפסק להציג",offLabel:"נוכח",tooltipDisabledContent:"הצגה אינה זמינה",tooltipOnContent:"מציג את המסך שלך",tooltipOffContent:"הצג את המסך שלך"};var raiseHandButton$c={offLabel:"הרם",onLabel:"הורד",tooltipDisabledContent:"פעולת הרמת יד אינה זמינה",tooltipOnContent:"הורד יד",tooltipOffContent:"הרם יד"};var reactionButton$c={label:"תגובה",ariaLabel:"תגובה",tooltipDisabledContent:"פעולת התגובה אינה זמינה",tooltipContent:"שלח תגובה",likeReactionTooltipContent:"אהבתי",heartReactionTooltipContent:"אהבה",laughReactionTooltipContent:"צחוק",applauseReactionTooltipContent:"תשואות",surprisedReactionTooltipContent:"מופתע"};var realTimeText$c={isTypingText:"הקלדת RTT"};var captionsSettingsModal$c={captionsSettingsModalTitle:"הגדרות כתוביות",captionsSettingsSpokenLanguageDropdownLabel:"שפה מדוברת",captionsSettingsCaptionLanguageDropdownLabel:"שפת כתוביות",captionsSettingsSpokenLanguageDropdownInfoText:"השפה שבה כולם בשיחה הזו מדברים.",captionsSettingsCaptionLanguageDropdownInfoText:"הכתוביות יופיעו בשפה זו.",captionsSettingsConfirmButtonLabel:"אישור",captionsSettingsCancelButtonLabel:"ביטול",captionsSettingsModalAriaLabel:"מודאל הגדרת כתוביות",captionsSettingsCloseModalButtonAriaLabel:"סגור הגדרת כתוביות"};var spokenLanguages$c={"ar-ae":"ערבית - איחוד האמירויות","ar-sa":"ערבית - ערב הסעודית","da-dk":"דנית","de-de":"גרמנית - גרמניה","en-au":"אנגלית - אוסטרליה","en-ca":"אנגלית - קנדה","en-gb":"אנגלית - בריטניה","en-in":"אנגלית - הודו","en-nz":"אנגלית - ניו זילנד","en-us":"אנגלית - ארצות הברית","es-es":"ספרדית - ספרד (ספרדית מודרנית)","es-mx":"ספרדית - מקסיקו","fi-fi":"פינית","fr-ca":"צרפתית - קנדה","fr-fr":"צרפתית - צרפת","hi-in":"הינדי","it-it":"איטלקית - איטליה","ja-jp":"יפנית","ko-kr":"קוריאנית","nb-no":"נורווגית (Bokmål)","nl-be":"הולנדית - בלגיה","nl-nl":"הולנדית - הולנד","pl-pl":"פולנית","pt-br":"פורטוגזית - ברזיל","ru-ru":"רוסית","sv-se":"שוודית","zh-cn":"סינית - הרפובליקה העממית של סין","zh-hk":"סינית - הונג קונג","cs-cz":"צ'כית","pt-pt":"פורטוגזית - פורטוגל","tr-tr":"טורקית","vi-vn":"וייטנאמית","th-th":"תאית","he-il":"עברית","cy-gb":"וולשית","uk-ua":"אוקראינית","el-gr":"יוונית","hu-hu":"הונגרית","ro-ro":"רומנית","sk-sk":"סלובקית","zh-tw":"סינית - טייוואן"};var captionLanguages$c={ar:"ערבית",da:"דנית",de:"גרמנית",en:"אנגלית",es:"ספרדית",fi:"פינית","fr-ca":"צרפתית - קנדה",fr:"צרפתית - צרפת",hi:"הינדי",it:"איטלקית",ja:"יפנית",ko:"קוריאנית",nb:"נורווגית (Bokmål)",nl:"הולנדית",pl:"פולנית",pt:"פורטוגזית - ברזיל",ru:"רוסית",sv:"שוודית","zh-Hans":"סינית (פשוטה)","zh-Hant":"סינית (מסורתית)",cs:"צ'כית","pt-pt":"פורטוגזית - פורטוגל",tr:"טורקית",vi:"וייטנאמית",th:"תאית",he:"עברית",cy:"וולשית",uk:"אוקראינית",el:"יוונית",hu:"הונגרית",ro:"רומנית",sk:"סלובקית"};var startCaptionsButton$c={onLabel:"הפעל כתוביות",offLabel:"כבה כתוביות",tooltipOnContent:"כבה כתוביות",tooltipOffContent:"הפעל כתוביות"};var startRealTimeTextButton$c={label:"הפעל RTT עבור שיחה זו",tooltipOffContent:"הפעל RTT עבור שיחה זו"};var messageThread$c={yesterday:"אתמול",sunday:"יום ראשון",monday:"יום שני",tuesday:"שלישי",wednesday:"יום רביעי",thursday:"יום חמישי",friday:"יום שישי",saturday:"יום שבת",participantJoined:"הצטרף לצ'אט.",participantLeft:"עזב את הצ'אט.",editMessage:"ערוך",removeMessage:"מחק",resendMessage:"נסה לשלוח שוב",failToSendTag:"השליחה נכשלה",editedTag:"נערך",liveAuthorIntro:"הודעה מאת {author}",editedMessageLiveAuthorIntro:"הודעה ערוכה מ- {author}",editedMessageLocalUserLiveAuthorIntro:"הודעה ערוכה",messageContentAriaText:"{status} {author} אמר {message}",messageContentMineAriaText:"{status} אמרת {message}",editBoxTextLimit:"ההודעה שלך נמצאת מעבר למגבלה של {limitNumber} תווים",editBoxPlaceholderText:"ערוך את ההודעה שלך",newMessagesIndicator:"הודעות חדשות",noDisplayNameSub:"ללא שם",editBoxCancelButton:"בטל",editBoxSubmitButton:"בוצע",messageReadCount:"נקרא על-ידי {messageReadByCount} מתוך {remoteParticipantsCount}",actionMenuMoreOptions:"אפשרויות נוספות",downloadAttachment:"הורד קובץ",openAttachment:"פתח קובץ",blockedWarningText:"הודעה זו נמחקה עקב מדיניות ארגונית.",blockedWarningLinkText:"פרטים",attachmentCardGroupMessage:"ההודעה כוללת {attachmentCount} קבצים מצורפים.",messageDeletedAnnouncementAriaLabel:"ההודעה נמחקת",richTextBoldTooltip:"מודגש",richTextItalicTooltip:"הטיה",richTextUnderlineTooltip:"קו תחתון",richTextBulletListTooltip:"רשימה עם תבליטים",richTextNumberListTooltip:"רשימה ממוספרת",richTextIncreaseIndentTooltip:"הגדל כניסה",richTextDecreaseIndentTooltip:"הקטן כניסה",richTextInsertTableTooltip:"הוסף טבלה",richTextFormatButtonTooltip:"תבנית",richTextInsertRowOrColumnMenu:"הוסף",richTextInsertTableMenuTitle:"הוסף טבלה {column} x‏ {row}",richTextInsertRowAboveMenu:"הוסף מעל",richTextInsertRowBelowMenu:"הוסף מתחת",richTextInsertColumnLeftMenu:"הוסף משמאל",richTextInsertColumnRightMenu:"הוסף מימין",richTextDeleteRowOrColumnMenu:"מחק",richTextDeleteColumnMenu:"מחק עמודה",richTextDeleteRowMenu:"מחק שורה",richTextDeleteTableMenu:"מחק טבלה",richTextToolbarMoreButtonAriaLabel:"אפשרויות נוספות",attachmentUploadsPendingError:"הקובץ בהעלאה, נא המתן.",imageUploadsPendingError:"התמונה בהעלאה, נא המתן.",richTextNewBulletedListItemAnnouncement:"תבליט, שורה חדשה",richTextNewNumberedListItemAnnouncement:"{0}, שורה חדשה",richTextBulletedListAppliedAnnouncement:"הוחלה רשימה עם תבליטים",richTextNumberedListAppliedAnnouncement:"רשימה ממוספרת הוחלה"};var errorBar$c={unableToReachChatService:"אתה נמצא במצב לא מקוון",accessDenied:"לא ניתן לגשת אל שירותי צ'אט - בדוק את אישורי המשתמש שסופקו",userNotInChatThread:"אינך נמצא עוד בהליך משנה זה של צ'אט",sendMessageNotInChatThread:"שליחת ההודעה נכשלה מכיוון שאתה כבר לא נמצא בהליך משנה זה של צ'אט",sendMessageGeneric:"שליחת ההודעה נכשלה",callingNetworkFailure:"ממתקשה בחיבור השיחה - נראה שאתה לא מקוון",startVideoGeneric:"הפעלת סרטון הווידאו נכשלה",stopVideoGeneric:"הפסקת הווידאו נכשלה",muteGeneric:"השתקת המיקרופון נכשלה",unmuteGeneric:"ביטול השתקת המיקרופון נכשל",speakingWhileMuted:"המיקרופון שלך מושתק",startScreenShareGeneric:"הייתה בעיה בהפעלת שיתוף מסך.",stopScreenShareGeneric:"הפסקת שיתוף המסכים נכשלה",callNetworkQualityLow:"איכות הרשת נמוכה.",teamsMeetingCallNetworkQualityLow:"איכות הרשת נמוכה.",callNoSpeakerFound:"לא נמצאו רמקולים או אוזניות. חבר התקן שמע כדי לשמוע את השיחה.",callNoMicrophoneFound:"לא נמצאו מיקרופונים. חבר התקן קלט שמע.",callMicrophoneAccessDenied:"לא ניתן לגשת למיקרופון. לחץ על הנעילה בשורת הכתובת כדי להעניק הרשאה לדף אינטרנט זה.",callMicrophoneAccessDeniedSafari:"לא ניתן לגשת למיקרופון. רענן את הדף כדי לאפשר הרשאות, או בדוק את ההגדרות של דפדפן זה וודא שהרשאות זמינות עבור אתר אינטרנט זה.",callMicrophoneMutedBySystem:"אתה מושתק על-ידי המערכת שלך.",callMicrophoneUnmutedBySystem:"המיקרופון שלך התאושש והמערכת שלך ביטלה את ההשתקה.",callMacOsMicrophoneAccessDenied:"לא ניתן לגשת למיקרופון. הענק הרשאת מיקרופון בהגדרות הפרטיות של macOS.",callLocalVideoFreeze:"רוחב הפס של הרשת גרוע. ייתכן שהווידאו שלך יופיע מושהה עבור אחרים בשיחה.",callCameraAccessDenied:"לא ניתן לגשת למצלמה. לחץ על הנעילה בשורת הכתובת כדי להעניק הרשאה לדף אינטרנט זה.",callCameraAccessDeniedSafari:"לא ניתן לגשת למצלמה. רענן את הדף כדי לאפשר הרשאות, או בדוק את ההגדרות של דפדפן זה וודא שהרשאות זמינות עבור אתר אינטרנט זה.",callCameraAlreadyInUse:"לא ניתן לגשת למצלמה. ייתכן שהוא כבר נמצא בשימוש על-ידי יישום אחר.",callVideoStoppedBySystem:"הווידאו שלך הופסק על-ידי המערכת.",callVideoRecoveredBySystem:"הווידאו שלך התחדש.",callMacOsCameraAccessDenied:"MacOS חוסם את הגישה למצלמה שלך. עדכן את הגדרות הפרטיות שלך כדי לאפשר לדפדפן זה לגשת למצלמה שלך.",callMacOsScreenShareAccessDenied:"MacOS חוסם שיתוף מסכים. עדכן את הגדרות הפרטיות שלך כדי לאפשר לדפדפן זה להקליט את המסך שלך.",dismissButtonAriaLabel:"סגור",failedToJoinCallGeneric:"הצטרפות לשיחה נכשלה.",failedToJoinCallInvalidMeetingLink:"לא ניתן להצטרף לפגישה. קישור לא חוקי.",cameraFrozenForRemoteParticipants:"משתמשים בשיחה נתקלים בבעיות בהצגת הווידאו שלך. בדוק את המכשירים ואת הרשת שלך.",unableToStartVideoEffect:"לא ניתן להחיל אפקט וידאו.",startSpotlightWhileMaxParticipantsAreSpotlighted:"אין אפשרות להפעיל זרקור על משתתפים מכיוון שכבר יש זרקורים על המספר המרבי של משתתפים.",mutedByRemoteParticipant:"השתקת משתתף אחר."};var notificationStack$c={callingNetworkFailure:{title:"בעיה בחיבור השיחה",message:"נראה שאתה במצב לא מקוון. בדוק את חיבור הרשת שלך ונסה שוב.",dismissButtonAriaLabel:"סגור"},startVideoGeneric:{title:"הפעלת סרטון הווידאו נכשלה.",dismissButtonAriaLabel:"סגור"},stopVideoGeneric:{title:"הפסקת הווידאו נכשלה.",dismissButtonAriaLabel:"סגור"},muteGeneric:{title:"השתקת המיקרופון נכשלה.",dismissButtonAriaLabel:"סגור"},unmuteGeneric:{title:"ביטול השתקת המיקרופון נכשל.",dismissButtonAriaLabel:"סגור"},speakingWhileMuted:{title:"אתה מושתק.",dismissButtonAriaLabel:"סגור"},startScreenShareGeneric:{title:"הייתה בעיה בהפעלת שיתוף מסך.",dismissButtonAriaLabel:"סגור"},stopScreenShareGeneric:{title:"הפסקת שיתוף המסכים נכשלה.",dismissButtonAriaLabel:"סגור"},callNetworkQualityLow:{title:"איכות הרשת נמוכה.",dismissButtonAriaLabel:"סגור"},teamsMeetingCallNetworkQualityLow:{title:"איכות הרשת נמוכה.",dismissButtonAriaLabel:"סגור",message:"הצטרף לשיחה זו מהטלפון שלך לקבלת צליל טוב יותר. באפשרותך להמשיך להציג את הפגישה במכשיר זה.",primaryButtonLabel:"הצטרף בטלפון"},callNoSpeakerFound:{title:"לא נמצאו רמקולים או אוזניות",message:"חבר התקן שמע כדי לשמוע את השיחה.",dismissButtonAriaLabel:"סגור"},callNoMicrophoneFound:{title:"לא נמצאו מיקרופונים",message:"חבר התקן קלט שמע.",dismissButtonAriaLabel:"סגור"},callMicrophoneAccessDenied:{title:"לא ניתן לגשת למיקרופון",message:"לחץ על הנעילה בשורת הכתובת כדי להעניק הרשאה לדף אינטרנט זה.",dismissButtonAriaLabel:"סגור"},callMicrophoneAccessDeniedSafari:{title:"לא ניתן לגשת למיקרופון",message:"רענן את הדף כדי לאפשר הרשאות, או בדוק את ההגדרות של דפדפן זה וודא שהרשאות זמינות עבור אתר אינטרנט זה.",dismissButtonAriaLabel:"סגור"},callMicrophoneMutedBySystem:{title:"אתה מושתק על-ידי המערכת שלך.",dismissButtonAriaLabel:"סגור"},callMicrophoneUnmutedBySystem:{title:"המיקרופון שלך התאושש והמערכת שלך ביטלה את ההשתקה.",dismissButtonAriaLabel:"סגור"},callMacOsMicrophoneAccessDenied:{title:"לא ניתן לגשת למיקרופון",message:"הענק הרשאת מיקרופון בהגדרות הפרטיות של macOS.",dismissButtonAriaLabel:"סגור"},callLocalVideoFreeze:{title:"רוחב הפס של הרשת גרוע",message:"ייתכן שהווידאו שלך יופיע מושהה עבור אנשים אחרים בשיחה.",dismissButtonAriaLabel:"סגור"},callCameraAccessDenied:{title:"לא ניתן לגשת למצלמה",message:"לחץ על הנעילה בשורת הכתובת כדי להעניק הרשאה לדף אינטרנט זה.",dismissButtonAriaLabel:"סגור"},callCameraAccessDeniedSafari:{title:"לא ניתן לגשת למצלמה",message:"רענן את הדף כדי לאפשר הרשאות, או בדוק את ההגדרות של דפדפן זה וודא שהרשאות זמינות עבור אתר אינטרנט זה.",dismissButtonAriaLabel:"סגור"},callCameraAlreadyInUse:{title:"לא ניתן לגשת למצלמה",message:"ייתכן שהוא כבר נמצא בשימוש על-ידי יישום אחר.",dismissButtonAriaLabel:"סגור"},callVideoStoppedBySystem:{title:"הווידאו שלך הופסק על-ידי המערכת.",dismissButtonAriaLabel:"סגור"},callVideoRecoveredBySystem:{title:"הווידאו שלך התחדש.",dismissButtonAriaLabel:"סגור"},callMacOsCameraAccessDenied:{title:"MacOS חוסם את הגישה למצלמה שלך",message:"עדכן את הגדרות הפרטיות שלך כדי לאפשר לדפדפן זה לגשת למצלמה שלך.",dismissButtonAriaLabel:"סגור"},callMacOsScreenShareAccessDenied:{title:"MacOS חוסם שיתוף מסכים",message:"עדכן את הגדרות הפרטיות שלך כדי לאפשר לדפדפן זה להקליט את המסך שלך.",dismissButtonAriaLabel:"סגור"},failedToJoinCallGeneric:{title:"הצטרפות לשיחה נכשלה.",dismissButtonAriaLabel:"סגור"},failedToJoinCallInvalidMeetingLink:{title:"לא ניתן להצטרף לפגישה. קישור לא חוקי.",dismissButtonAriaLabel:"סגור"},cameraFrozenForRemoteParticipants:{title:"משתמשים בשיחה נתקלים בבעיות בהצגת הווידאו שלך",message:"בדוק את המכשירים ואת הרשת שלך.",dismissButtonAriaLabel:"סגור"},unableToStartVideoEffect:{title:"לא ניתן להחיל אפקט וידאו.",dismissButtonAriaLabel:"סגור"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"אין אפשרות להפעיל זרקור על משתתפים",message:"מספר המשתתפים המרבי כבר מודגש.",dismissButtonAriaLabel:"סגור"},mutedByRemoteParticipant:{title:"השתקת משתתף אחר.",dismissButtonAriaLabel:"סגור"},recordingStarted:{title:"ההקלטה התחילה.",message:"על-ידי הצטרפות, אתה נותן הסכמה לתעתק פגישה זו.",dismissButtonAriaLabel:"סגור"},transcriptionStarted:{title:"התמלול התחיל.",message:"על-ידי הצטרפות, אתה נותן הסכמה לתעתק פגישה זו.",dismissButtonAriaLabel:"סגור"},recordingStopped:{title:"ההקלטה נשמרת.",message:"ההקלטה הופסקה.",dismissButtonAriaLabel:"סגור"},transcriptionStopped:{title:"מתבצע תהליך שמירה של התמליל.",message:"התמלול הופסק.",dismissButtonAriaLabel:"סגור"},recordingAndTranscriptionStarted:{title:"הקלטה ותמלול התחילו.",message:"על-ידי הצטרפות, אתה נותן הסכמה לתעתק פגישה זו.",dismissButtonAriaLabel:"סגור"},recordingAndTranscriptionStopped:{title:"ההקלטה והתמלול בתהליך שמירה.",message:"ההקלטה והתמלול הופסקו.",dismissButtonAriaLabel:"סגור"},recordingStoppedStillTranscribing:{title:"ההקלטה הופסקה.",message:"אתה רק מתעתק את הפגישה כעת.",dismissButtonAriaLabel:"סגור"},transcriptionStoppedStillRecording:{title:"התמלול הופסק.",message:"אתה כעת רק מקליט את הפגישה.",dismissButtonAriaLabel:"סגור"},assignedBreakoutRoomOpened:{title:"נעביר אותך לחדר שהוקצה לך תוך 5 שניות.",dismissButtonAriaLabel:"סגור"},assignedBreakoutRoomChanged:{title:"פיצול החדרים שלך השתנה",message:"נעביר אותך לחדר שהוקצה לך תוך 5 שניות.",dismissButtonAriaLabel:"סגור"},assignedBreakoutRoomClosed:{title:"פיצול החדרים שלך נסגר",message:"נעביר אותך בחזרה לפגישה שלך בעוד 5 שניות.",dismissButtonAriaLabel:"סגור"},assignedBreakoutRoomOpenedPromptJoin:{title:"להצטרף לפיצול חדרים?",message:"הוקצית לפיצול חדר.",dismissButtonAriaLabel:"סגור",primaryButtonLabel:"הצטרף לחדר",secondaryButtonLabel:"מאוחר יותר"},breakoutRoomJoined:{title:"הצטרפת לפיצול חדרים.",dismissButtonAriaLabel:"סגור"},breakoutRoomClosingSoon:{title:"תוקף מגבלת זמן החדר עומד לפוג.",message:"חדר זה ייסגר בעוד 30 שניות",dismissButtonAriaLabel:"סגור"},capabilityTurnVideoOnPresent:{title:"המצלמה שלך הפכה לזמינה",message:"הפעל אותה אם ברצונך לשתף וידאו.",dismissButtonAriaLabel:"סגור"},capabilityTurnVideoOnAbsent:{title:"המצלמה שלך הושבתה",message:"לא תוכל עוד לשתף וידאו.",dismissButtonAriaLabel:"סגור"},capabilityUnmuteMicPresent:{title:"המיקרופון שלך הפך לזמין",message:"בטל את ההשתקה אם אתה מעוניין לדבר.",dismissButtonAriaLabel:"סגור"},capabilityUnmuteMicAbsent:{title:"המיקרופון שלך הפך ללא זמין",message:"לא תוכל עוד לבטל השתקה.",dismissButtonAriaLabel:"סגור"},togetherModeStarted:{title:"Togethermode התחיל",dismissButtonAriaLabel:"סגור"},togetherModeEnded:{title:"Togethermode הסתיים",dismissButtonAriaLabel:"סגור"},transcriptionError:{title:"משהו השתבש בתהליך התמלול.",message:"התמלול הופסק. נסה שוב מאוחר יותר",dismissButtonAriaLabel:"סגור"},transcriptionStartedByYou:{title:"התמלול התחיל.",message:"הופעל על-ידך. עדכן את כולם שמבוצע תמלול של השיחה.",dismissButtonAriaLabel:"סגור"}};var videoGallery$c={screenIsBeingSharedMessage:"אתה משתף את המסך שלך",screenShareLoadingMessage:"טוען את המסך של {participant}",localScreenShareLoadingMessage:"טוען את המסך שלך",localVideoLabel:"אתה",localVideoCameraSwitcherLabel:"החלף מצלמה",localVideoMovementLabel:"אריח וידאו מקומי נשלף",localVideoMovementAriaLabel:"אריח וידאו מקומי נשלף",localVideoSelectedDescription:"{cameraName} נבחר",displayNamePlaceholder:"משתתף ללא שם",fitRemoteParticipantToFrame:"התאם למסגרת",fillRemoteParticipantFrame:"מלא את המסגרת",pinParticipantForMe:"הצמד עבורי",pinParticipantForMeLimitReached:"הצמדה (הגעת למגבלה)",unpinParticipantForMe:"בטל הצמדה",pinParticipantMenuItemAriaLabel:"הצמד את {participantName}",unpinParticipantMenuItemAriaLabel:"בטל את הצמדת {participantName}",pinnedParticipantAnnouncementAriaLabel:"{participantName} הוצמד",unpinnedParticipantAnnouncementAriaLabel:"הצמדת {participantName} בוטלה",startSpotlightVideoTileMenuLabel:"הצבת זרקור עבור כולם",addSpotlightVideoTileMenuLabel:"הוסף זרקור",spotlightLimitReachedMenuTitle:"הגעת למגבלת הזרקורים",stopSpotlightVideoTileMenuLabel:"הפסק להציב זרקור",stopSpotlightOnSelfVideoTileMenuLabel:"צא מזרקור",attendeeRole:"משתתף",muteParticipantMenuItemLabel:"השתק",waitingScreenText:"מחכה שאחרים יצטרפו",forbidAudio:"הפוך את המיקרופון ללא זמין",permitAudio:"אפשר שימוש במיקרופון",forbidAudioTileMenuLabel:"הפוך את המיקרופון ללא זמין",permitAudioTileMenuLabel:"אפשר שימוש במיקרופון",forbidVideo:"הפוך את המצלמה ללא זמינה",permitVideo:"אפשר שימוש במצלמה",forbidVideoTileMenuLabel:"הפוך את המצלמה ללא זמינה",permitVideoTileMenuLabel:"אפשר שימוש במצלמה"};var dialpad$c={placeholderText:"הזן מספר טלפון",deleteButtonAriaLabel:"מחק"};var holdButton$c={onLabel:"המשך",offLabel:"החזק",tooltipOnContent:"חדש שיחה",tooltipOffContent:"השהה שיחה"};var videoTile$c={participantStateRinging:"מתקשר...",participantReconnecting:"מתחבר מחדש...",participantStateHold:"בהמתנה",moreOptionsButtonAriaLabel:"אפשרויות נוספות {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"מושתק",moreOptionsParticipantMutedStateUnmutedAriaLabel:"לא מושתק",moreOptionsParticipantHandRaisedAriaLabel:"‫כף יד מורמת",moreOptionsParticipantIsSpeakingAriaLabel:"מדבר",moreOptionsParticipantMicDisabledAriaLabel:"המיקרופון לא זמין",moreOptionsParticipantCameraDisabledAriaLabel:"המצלמה לא זמינה"};var CameraAndMicrophoneSitePermissionsRequest$c={primaryText:"אפשר {appName} להשתמש במצלמה ובמיקרופון שלך",secondaryText:"זה כדי שהמשתתפים יוכלו לראות ולשמוע אותך.",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות",primaryButtonText:"המשך ללא מצלמה ומיקרופון",ariaLabel:"אפשר גישה למצלמה ולמיקרופון"};var CameraSitePermissionsRequest$c={primaryText:"אפשר {appName} להשתמש במצלמה שלך",secondaryText:"זה כדי שהמשתתפים יוכלו לראות אותך.",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות",primaryButtonText:"המשך ללא מצלמה",ariaLabel:"אפשר גישה למצלמה"};var MicrophoneSitePermissionsRequest$c={primaryText:"אפשר {appName} להשתמש במיקרופון שלך",secondaryText:"זה כדי שהמשתתפים יוכלו לשמוע אותך.",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות",primaryButtonText:"המשך ללא מיקרופון",ariaLabel:"אפשר גישה למיקרופון"};var CameraAndMicrophoneSitePermissionsCheck$c={primaryText:"בודק אם קיימת גישה למצלמה ולמיקרופון",secondaryText:"אפשר גישה אם תתבקש לעשות זאת. זה כדי שהמשתתפים יוכלו לראות ולשמוע אותך.",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות",primaryButtonText:"המשך ללא מצלמה ומיקרופון",ariaLabel:"בודק אם קיימת גישה למצלמה ולמיקרופון. אפשר גישה אם תתבקש לעשות זאת."};var CameraSitePermissionsCheck$c={primaryText:"בודק אם קיימת גישה למצלמה",secondaryText:"אפשר גישה אם תתבקש לעשות זאת. זה כדי שהמשתתפים יוכלו לראות אותך.",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות",primaryButtonText:"המשך ללא מצלמה",ariaLabel:"בודק אם קיימת גישה למצלמה. אפשר גישה אם תתבקש לעשות זאת."};var MicrophoneSitePermissionsCheck$c={primaryText:"בודק אם קיימת גישה למיקרופון",secondaryText:"אפשר גישה אם תתבקש לעשות זאת. זה כדי שהמשתתפים יוכלו לשמוע אותך.",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות",primaryButtonText:"המשך ללא מיקרופון",ariaLabel:"בודק אם קיימת גישה למיקרופון. אפשר גישה אם תתבקש לעשות זאת."};var CameraAndMicrophoneSitePermissionsDenied$c={primaryText:"לא ניתן לגשת למצלמה ולמיקרופון",secondaryText:"לחץ על סמל המנעול בשורת הכתובת כדי להעניק הרשאות מיקרופון לדף אינטרנט זה. ייתכן שיידרש רענון דף.",primaryButtonText:"המשך ללא מצלמה ומיקרופון",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות"};var CameraAndMicrophoneSitePermissionsDeniedSafari$c={primaryText:"לא ניתן לגשת למצלמה ולמיקרופון",secondaryText:"רענן את הדף כדי לאפשר הרשאות, או בדוק את ההגדרות של דפדפן זה וודא שהרשאות זמינות עבור אתר אינטרנט זה.",primaryButtonText:"המשך ללא מצלמה ומיקרופון",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות"};var CameraSitePermissionsDenied$c={primaryText:"לא ניתן לגשת למצלמה",secondaryText:"לחץ על סמל המנעול בשורת הכתובת כדי להעניק הרשאות מצלמה לדף אינטרנט זה. ייתכן שיידרש רענון דף.",primaryButtonText:"המשך ללא מצלמה",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות"};var MicrophoneSitePermissionsDenied$c={primaryText:"לא ניתן לגשת למיקרופון",secondaryText:"לחץ על סמל המנעול בשורת הכתובת כדי להעניק הרשאות מיקרופון לדף אינטרנט זה. ייתכן שיידרש רענון דף.",primaryButtonText:"המשך ללא מיקרופון",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות"};var CameraSitePermissionsDeniedSafari$c={primaryText:"לא ניתן לגשת למצלמה",secondaryText:"רענן את הדף כדי לאפשר הרשאות, או בדוק את ההגדרות של דפדפן זה וודא שהרשאות זמינות עבור אתר אינטרנט זה.",primaryButtonText:"המשך ללא מצלמה",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות"};var MicrophoneSitePermissionsDeniedSafari$c={primaryText:"לא ניתן לגשת למיקרופון",secondaryText:"רענן את הדף כדי לאפשר הרשאות, או בדוק את ההגדרות של דפדפן זה וודא שהרשאות זמינות עבור אתר אינטרנט זה.",primaryButtonText:"המשך ללא מיקרופון",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות"};var UnsupportedBrowser$c={primaryText:"הדפדפן אינו נתמך",secondaryText:"הצטרף לשיחה זו באמצעות דפדפן תואם.",moreHelpLinkText:"ראה דרישות תאימות"};var UnsupportedBrowserVersion$c={primaryText:"‏‏נדרש עדכון דפדפן",secondaryText:"כדי להבטיח את השיחה הטובה ביותר האפשרית, עדכן את הדפדפן ולאחר מכן נסה להצטרף שוב לשיחה.",moreHelpLinkText:"ראה דרישות תאימות",continueAnywayButtonText:"התחל שיחה מבלי לעדכן"};var UnsupportedOperatingSystem$c={primaryText:"מערכת ההפעלה אינה נתמכת",secondaryText:"הצטרף לשיחה זו באמצעות התקן עם מערכת הפעלה תואמת.",moreHelpLinkText:"ראה דרישות תאימות"};var BrowserPermissionDenied$c={primaryText:"לא ניתן להשתמש במצלמה או במיקרופון שלך",secondaryText:"ייתכן שלדפדפן שלך אין גישה למצלמה או למיקרופון שלך. כדי לפתור בעיה זו, פתח את 'העדפות מערכת'.",primaryButtonText:"נסה שוב",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות"};var BrowserPermissionDeniedIOS$c={primaryText:"אפשר גישה למיקרופון כדי להמשיך",secondaryText:"כך שמשתתפים אחרים יוכלו לשמוע אותך.",primaryButtonText:"נסה שוב",imageAltText:"מיקום הרשאה של מיקרופון והתקן מצלמה עבור iOS",linkText:"זקוק לעזרה? קבל עזרה עבור פתרון בעיות",step1Text:"עבור אל האפליקציה 'הגדרות'",step2Text:"גלול למטה להגדרות עבור דפדפן זה",step3Text:"הפעל מיקרופון (מצלמה אופציונלית)",step4Text:"נסה להצטרף שוב לשיחה",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$c={leftNavButtonAriaLabel:"העמוד הקודם {current} מתוך {total}",rightNavButtonAriaLabel:"העמוד הבא {current} מתוך {total}"};var meetingConferencePhoneInfo$c={meetingConferencePhoneInfoModalTitle:"הצטרף לשיחה מהטלפון שלך",meetingConferencePhoneInfoModalDialIn:"חייג למספר",meetingConferencePhoneInfoModalMeetingId:"הזן מזהה פגישה",meetingConferencePhoneInfoModalWait:"המתן עד שהמארגן יכניס אותך",meetingConferencePhoneInfoModalNoPhoneAvailable:"אין טלפון זמין",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(שיחת חינם)",meetingConferencePhoneInfoModalToll:"(בתשלום)"};var IncomingCallNotification$c={incomingCallNotificationPlaceholderId:"משתמש לא ידוע",incomingCallNotificationPlaceholderAlert:"{callerName} מתקשר",incomingCallNoticicationAcceptWithAudioAriaLabel:"קבל עם שמע",incomingCallNoticicationAcceptWithVideoAriaLabel:"קבל עם וידאו",incomingCallNoticicationRejectAriaLabel:"דחה",incomingCallNotificationAcceptButtonLabel:"קבל",incomingCallNotificationRejectButtonLabel:"דחה",incomingCallNotificationAccceptWithVideoButtonLabel:"קבל עם וידאו",incomingCallNotificationDismissButtonAriaLabel:"בטל"};var he_IL$1 = {participantItem:participantItem$c,ParticipantList:ParticipantList$c,typingIndicator:typingIndicator$c,sendBox:sendBox$c,richTextSendBox:richTextSendBox$c,realTimeTextModal:realTimeTextModal$c,captionsBanner:captionsBanner$c,mentionPopover:mentionPopover$c,imageOverlay:imageOverlay$c,messageStatusIndicator:messageStatusIndicator$c,endCallButton:endCallButton$c,cameraButton:cameraButton$c,microphoneButton:microphoneButton$c,devicesButton:devicesButton$c,participantsButton:participantsButton$c,screenShareButton:screenShareButton$c,raiseHandButton:raiseHandButton$c,reactionButton:reactionButton$c,realTimeText:realTimeText$c,captionsSettingsModal:captionsSettingsModal$c,spokenLanguages:spokenLanguages$c,captionLanguages:captionLanguages$c,startCaptionsButton:startCaptionsButton$c,startRealTimeTextButton:startRealTimeTextButton$c,messageThread:messageThread$c,errorBar:errorBar$c,notificationStack:notificationStack$c,videoGallery:videoGallery$c,dialpad:dialpad$c,holdButton:holdButton$c,videoTile:videoTile$c,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$c,CameraSitePermissionsRequest:CameraSitePermissionsRequest$c,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$c,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$c,CameraSitePermissionsCheck:CameraSitePermissionsCheck$c,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$c,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$c,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$c,CameraSitePermissionsDenied:CameraSitePermissionsDenied$c,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$c,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$c,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$c,UnsupportedBrowser:UnsupportedBrowser$c,UnsupportedBrowserVersion:UnsupportedBrowserVersion$c,UnsupportedOperatingSystem:UnsupportedOperatingSystem$c,BrowserPermissionDenied:BrowserPermissionDenied$c,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$c,verticalGallery:verticalGallery$c,meetingConferencePhoneInfo:meetingConferencePhoneInfo$c,IncomingCallNotification:IncomingCallNotification$c};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Hebrew (Israel).
 *
 * @public
 */
const COMPONENT_LOCALE_HE_IL = {
    strings: createComponentStrings(he_IL$1)
};

var participantItem$b={isMeText:"(tu)",menuTitle:"Altre opzioni",removeButtonLabel:"Rimuovi",sharingIconLabel:"Condivisione",mutedIconLabel:"Disattivato",handRaisedIconLabel:"Mano alzata - Posizione {position}",displayNamePlaceholder:"Partecipante senza nome",participantStateRinging:"Chiamata in corso...",participantStateHold:"Per l'attesa",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, altre opzioni",attendeeRole:"Partecipante",micDisabledIconLabel:"Microfono disabilitato",cameraDisabledIconLabel:"Fotocamera disabilitata"};var ParticipantList$b={overflowParticipantCount:"+{overflowCount} altro"};var typingIndicator$b={singleUser:"{user} sta scrivendo ...",multipleUsers:"{users} stanno scrivendo ...",multipleUsersAbbreviateOne:"{users} e 1 altro stanno scrivendo ...",multipleUsersAbbreviateMany:"{users} e {numOthers} altri stanno scrivendo ...",delimiter:", "};var sendBox$b={placeholderText:"Immetti un messaggio",textTooLong:"La lunghezza del messaggio supera il limite massimo.",sendButtonAriaLabel:"Invio messaggio",attachmentUploadsPendingError:"È in corso il caricamento del file, attendere.",removeAttachment:"Rimuovi file",uploading:"Caricamento in corso",uploadCompleted:"Caricamento completato",attachmentMoreMenu:"Altre opzioni"};var richTextSendBox$b={placeholderText:"Immetti un messaggio",textTooLong:"La lunghezza del messaggio supera il limite massimo.",sendButtonAriaLabel:"Invia messaggio",attachmentUploadsPendingError:"È in corso il caricamento del file, attendere.",imageUploadsPendingError:"È in corso il caricamento dell'immagine, attendere.",removeAttachment:"Rimuovi file",uploading:"Caricamento",uploadCompleted:"Caricamento completato",richTextBoldTooltip:"Grassetto",richTextItalicTooltip:"Corsivo",richTextUnderlineTooltip:"Sottolineato",richTextBulletListTooltip:"Elenco puntato",richTextNumberListTooltip:"Elenco numerato",richTextIncreaseIndentTooltip:"Aumenta rientro",richTextDecreaseIndentTooltip:"Riduci rientro",richTextInsertTableTooltip:"Inserisci tabella",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Inserisci",richTextInsertTableMenuTitle:"Inserisci tabella {column} x {row}",richTextInsertRowAboveMenu:"Inserisci sopra",richTextInsertRowBelowMenu:"Inserisci sotto",richTextInsertColumnLeftMenu:"Inserisci a sinistra",richTextInsertColumnRightMenu:"Inserisci a destra",richTextDeleteRowOrColumnMenu:"Elimina",richTextDeleteColumnMenu:"Elimina colonna",richTextDeleteRowMenu:"Elimina riga",richTextDeleteTableMenu:"Elimina tabella",richTextToolbarAriaLabel:"Barra degli strumenti Formattazione messaggi",richTextToolbarMoreButtonAriaLabel:"Altre opzioni",attachmentMoreMenu:"Altre opzioni",richTextNewBulletedListItemAnnouncement:"Punto elenco, nuova riga",richTextNewNumberedListItemAnnouncement:"{0}, nuova riga",richTextBulletedListAppliedAnnouncement:"Elenco puntato applicato",richTextNumberedListAppliedAnnouncement:"Elenco numerato applicato"};var realTimeTextModal$b={realTimeTextModalAriaLabel:"Abilita finestra modale scrittura in tempo reale",realTimeTextModalTitle:"Abilitare la scrittura in tempo reale?",realTimeTextModalText:"Il testo in tempo reale verrà attivato per tutti gli utenti della riunione.",realTimeTextConfirmButtonLabel:"Abilita",realTimeTextCancelButtonLabel:"Annulla",realTimeTextCloseModalButtonAriaLabel:"Chiudi finestra modale scrittura in tempo reale"};var captionsBanner$b={captionsBannerSpinnerText:"Avvio dei sottotitoli in corso...",realTimeTextInputBoxDefaultText:"Digita messaggio in tempo reale",realTimeTextInputErrorMessage:"Il messaggio non può superare i 2000 caratteri",realTimeTextBannerTitle:"Scrittura in tempo reale",realTimeTextBannerContent:"RTT (testo in tempo reale) è abilitato per tutti i partecipanti per l'intera durata della riunione.",realTimeTextBannerLinkLabel:"Scopri di più",captionsOnlyContainerTitle:"Sottotitoli",realTimeTextOnlyContainerTitle:"Testo in tempo reale",captionsAndRealTimeTextContainerTitle:"Sottotitoli e testo in tempo reale",expandButtonAriaLabel:"Espandi",minimizeButtonAriaLabel:"Comprimi"};var mentionPopover$b={mentionPopoverHeader:"Suggerimenti"};var imageOverlay$b={downloadButtonLabel:"Scarica",dismissButtonAriaLabel:"Chiudi",overlayVisibleMessage:"Sovrapposizione dettagli immagine"};var messageStatusIndicator$b={deliveredAriaLabel:"Il messaggio è stato inviato",deliveredTooltipText:"Inviato",seenAriaLabel:"Messaggio visualizzato da altri utenti",seenTooltipText:"Visualizzato",readByTooltipText:"Letto da {messageThreadReadCount} di {remoteParticipantsCount}",sendingAriaLabel:"Invio del messaggio",sendingTooltipText:"Invio in corso",failedToSendAriaLabel:"Invio messaggio non riuscito",failedToSendTooltipText:"Invio non riuscito"};var endCallButton$b={label:"Permesso",tooltipContent:"Abbandona chiamata",leaveOption:"Abbandona",endCallOption:"Termina chiamata per tutti"};var cameraButton$b={onLabel:"Fotocamera",offLabel:"Fotocamera",tooltipDisabledContent:"La videocamera è disabilitata",tooltipOnContent:"Spegni la videocamera",tooltipOffContent:"Accendi la videocamera",tooltipVideoLoadingContent:"Il video è in corso di caricamento",cameraMenuTitle:"Fotocamera",cameraMenuTooltip:"Scegli fotocamera",cameraButtonSplitRoleDescription:"Pulsante di menu combinato",onSplitButtonAriaLabel:"Disabilita la videocamera",offSplitButtonAriaLabel:"Accendi la videocamera",cameraActionTurnedOnAnnouncement:"La videocamera è stata attivata",cameraActionTurnedOffAnnouncement:"La videocamera è stata disattivata",onSplitButtonPrimaryActionCamera:"Spegni la videocamera",offSplitButtonPrimaryActionCamera:"Accendi la videocamera",cameraPrimaryActionSplitButtonTitle:"Usa la fotocamera",videoEffectsMenuItemTitle:"Effetti",cameraButtonAriaDescription:"opzioni videocamera"};var microphoneButton$b={onLabel:"Microfono",offLabel:"Microfono",tooltipDisabledContent:"Il microfono è disabilitato",tooltipOnContent:"Disattiva microfono",tooltipOffContent:"Riattiva microfono",microphoneMenuTitle:"Microfono",microphoneMenuTooltip:"Scegli microfono",speakerMenuTitle:"Altoparlante",speakerMenuTooltip:"Scegli altoparlante",microphoneButtonSplitRoleDescription:"Pulsante di menu combinato",onSplitButtonAriaLabel:"Disattiva microfono",offSplitButtonAriaLabel:"Riattiva microfono",microphoneActionTurnedOnAnnouncement:"Il microfono è stato attivato",microphoneActionTurnedOffAnnouncement:"Il microfono è stato disattivato",offSplitButtonMicrophonePrimaryAction:"Riattiva microfono",onSplitButtonMicrophonePrimaryAction:"Disattiva microfono",microphonePrimaryActionSplitButtonTitle:"Usa il microfono",microphoneAriaDescription:"Opzioni audio",deepNoiseSuppressionTitle:"Eliminazione del rumore",deepNoiseSuppressionOnAnnouncement:"L'eliminazione del rumore profondo è stata attivata",deepNoiseSuppressionOffAnnouncement:"L'eliminazione del rumore profondo è stata disattivata"};var devicesButton$b={label:"Dispositivi",tooltipContent:"Gestisci dispositivi",cameraMenuTitle:"Fotocamera",cameraMenuTooltip:"Scegli fotocamera",audioDeviceMenuTitle:"Dispositivo audio",audioDeviceMenuTooltip:"Scegli dispositivo audio",microphoneMenuTitle:"Microfono",microphoneMenuTooltip:"Scegli microfono",speakerMenuTitle:"Altoparlante",speakerMenuTooltip:"Scegli altoparlante"};var participantsButton$b={label:"Persone",ariaLabel:"Persone",tooltipContent:"Mostra partecipanti",menuHeader:"In questa chiamata",participantsListButtonLabel:"{numParticipants} persone",muteAllButtonLabel:"Disattiva l'audio di tutti",copyInviteLinkButtonLabel:"Copia il collegamento dell’invito",copyInviteLinkActionedAriaLabel:"Collegamento di invito copiato"};var screenShareButton$b={onLabel:"Interrompi presentazione",offLabel:"Presentazione",tooltipDisabledContent:"La presentazione è disabilitata.",tooltipOnContent:"Presentazione dello schermo",tooltipOffContent:"Presenta lo schermo"};var raiseHandButton$b={offLabel:"Alza",onLabel:"Abbassa",tooltipDisabledContent:"L'azione Alza la mano è disabilitata",tooltipOnContent:"Abbassa la mano",tooltipOffContent:"Alza la mano"};var reactionButton$b={label:"Reagisci",ariaLabel:"Reagisci",tooltipDisabledContent:"L'azione di reazione è disabilitata",tooltipContent:"Invia una reazione",likeReactionTooltipContent:"Mi piace",heartReactionTooltipContent:"Amore",laughReactionTooltipContent:"Risata",applauseReactionTooltipContent:"Applauso",surprisedReactionTooltipContent:"Sorpresa"};var realTimeText$b={isTypingText:"Digitazione scrittura in tempo reale"};var captionsSettingsModal$b={captionsSettingsModalTitle:"Impostazioni sottotitoli",captionsSettingsSpokenLanguageDropdownLabel:"Lingua parlata",captionsSettingsCaptionLanguageDropdownLabel:"Lingua dei sottotitoli",captionsSettingsSpokenLanguageDropdownInfoText:"La lingua che parlano tutti i partecipanti a questa chiamata.",captionsSettingsCaptionLanguageDropdownInfoText:"I sottotitoli verranno visualizzati in questa lingua.",captionsSettingsConfirmButtonLabel:"Conferma",captionsSettingsCancelButtonLabel:"Annulla",captionsSettingsModalAriaLabel:"Finestra modale impostazioni sottotitoli",captionsSettingsCloseModalButtonAriaLabel:"Chiudi impostazione sottotitoli"};var spokenLanguages$b={"ar-ae":"Arabo - Emirati Arabi Uniti","ar-sa":"Arabo - Arabia Saudita","da-dk":"Danese","de-de":"Tedesco - Germania","en-au":"Inglese - Australia","en-ca":"Inglese - Canada","en-gb":"Inglese - Regno Unito","en-in":"Inglese - India","en-nz":"Inglese - Nuova Zelanda","en-us":"Inglese - Stati Uniti","es-es":"Spagnolo - Spagna (ordinamento moderno)","es-mx":"Spagnolo - Messico","fi-fi":"Finlandese","fr-ca":"Francese - Canada","fr-fr":"Francese - Francia","hi-in":"Hindi","it-it":"Italiano - Italia","ja-jp":"Giapponese","ko-kr":"Coreano","nb-no":"Norvegese (Bokmål)","nl-be":"Olandese - Belgio","nl-nl":"Olandese - Paesi Bassi","pl-pl":"Polacco","pt-br":"Portoghese - Brasile","ru-ru":"Russo","sv-se":"Svedese","zh-cn":"Cinese - Repubblica popolare cinese","zh-hk":"Cinese - RAS di Hong Kong","cs-cz":"Ceco","pt-pt":"Portoghese - Portogallo","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Thailandese","he-il":"Ebraico","cy-gb":"Gallese","uk-ua":"Ucraino","el-gr":"Greco","hu-hu":"Ungherese","ro-ro":"Rumeno","sk-sk":"Slovacco","zh-tw":"Cinese - Taiwan"};var captionLanguages$b={ar:"Arabo",da:"Danese",de:"Tedesco",en:"Inglese",es:"Spagnolo",fi:"Finlandese","fr-ca":"Francese - Canada",fr:"Francese - Francia",hi:"Hindi",it:"Italiano",ja:"Giapponese",ko:"Coreano",nb:"Norvegese (Bokmål)",nl:"Olandese",pl:"Polacco",pt:"Portoghese - Brasile",ru:"Russo",sv:"Svedese","zh-Hans":"Cinese (semplificato)","zh-Hant":"Cinese (tradizionale)",cs:"Ceco","pt-pt":"Portoghese - Portogallo",tr:"Turco",vi:"Vietnamita",th:"Thailandese",he:"Ebraico",cy:"Gallese",uk:"Ucraino",el:"Greco",hu:"Ungherese",ro:"Rumeno",sk:"Slovacco"};var startCaptionsButton$b={onLabel:"Abilita sottotitoli",offLabel:"Disabilita sottotitoli",tooltipOnContent:"Disabilita sottotitoli",tooltipOffContent:"Abilita sottotitoli"};var startRealTimeTextButton$b={label:"Attiva la scrittura in tempo reale per questa chiamata",tooltipOffContent:"Attiva la scrittura in tempo reale per questa chiamata"};var messageThread$b={yesterday:"Ieri",sunday:"Domenica",monday:"Lunedì",tuesday:"Martedì",wednesday:"Mercoledì",thursday:"Giovedì",friday:"Venerdì",saturday:"Sabato",participantJoined:"si è unito alla chat.",participantLeft:"ha abbandonato la chat.",editMessage:"Modifica",removeMessage:"Elimina",resendMessage:"Riprova a inviare",failToSendTag:"Invio non riuscito",editedTag:"Modificato",liveAuthorIntro:"Messaggio da {author}",editedMessageLiveAuthorIntro:"Messaggio modificato da {author}",editedMessageLocalUserLiveAuthorIntro:"Messaggio modificato",messageContentAriaText:"{status} {author} ha detto {message}",messageContentMineAriaText:"{status} Hai detto {message}",editBoxTextLimit:"Il messaggio ha superato il limite di caratteri {limitNumber}",editBoxPlaceholderText:"Modifica il messaggio",newMessagesIndicator:"Nuovi messaggi",noDisplayNameSub:"Nessun nome",editBoxCancelButton:"Annulla",editBoxSubmitButton:"Fatto",messageReadCount:"Letto da {messageReadByCount} di {remoteParticipantsCount}",actionMenuMoreOptions:"Altre opzioni",downloadAttachment:"Scarica file",openAttachment:"Apri file",blockedWarningText:"Questo messaggio è stato eliminato a causa dei criteri dell'organizzazione.",blockedWarningLinkText:"Dettagli",attachmentCardGroupMessage:"Il messaggio ha allegati: {attachmentCount}.",messageDeletedAnnouncementAriaLabel:"Il messaggio è stato eliminato",richTextBoldTooltip:"Grassetto",richTextItalicTooltip:"Corsivo",richTextUnderlineTooltip:"Sottolineato",richTextBulletListTooltip:"Elenco puntato",richTextNumberListTooltip:"Elenco numerato",richTextIncreaseIndentTooltip:"Aumenta rientro",richTextDecreaseIndentTooltip:"Riduci rientro",richTextInsertTableTooltip:"Inserisci tabella",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Inserisci",richTextInsertTableMenuTitle:"Inserisci tabella {column} x {row}",richTextInsertRowAboveMenu:"Inserisci sopra",richTextInsertRowBelowMenu:"Inserisci sotto",richTextInsertColumnLeftMenu:"Inserisci a sinistra",richTextInsertColumnRightMenu:"Inserisci a destra",richTextDeleteRowOrColumnMenu:"Elimina",richTextDeleteColumnMenu:"Elimina colonna",richTextDeleteRowMenu:"Elimina riga",richTextDeleteTableMenu:"Elimina tabella",richTextToolbarMoreButtonAriaLabel:"Altre opzioni",attachmentUploadsPendingError:"È in corso il caricamento del file, attendere.",imageUploadsPendingError:"È in corso il caricamento dell'immagine, attendere.",richTextNewBulletedListItemAnnouncement:"Punto elenco, nuova riga",richTextNewNumberedListItemAnnouncement:"{0}, nuova riga",richTextBulletedListAppliedAnnouncement:"Elenco puntato applicato",richTextNumberedListAppliedAnnouncement:"Elenco numerato applicato"};var errorBar$b={unableToReachChatService:"Sei offline",accessDenied:"Non è possibile accedere ai servizi di chat. Controllare le credenziali utente specificate",userNotInChatThread:"Non fai più parte di questo thread di chat",sendMessageNotInChatThread:"Non è stato possibile inviare il messaggio perché non sei più in questo thread di chat",sendMessageGeneric:"Impossibile inviare il messaggio",callingNetworkFailure:"Problema di connessione della chiamata: sembra che tu sia offline",startVideoGeneric:"Non è stato possibile avviare il video",stopVideoGeneric:"Non è stato possibile arrestare il video",muteGeneric:"Non è stato possibile disattivare l'audio del microfono",unmuteGeneric:"Non è stato possibile riattivare l'audio del microfono",speakingWhileMuted:"Il microfono è disattivato",startScreenShareGeneric:"Si è verificato un problema durante l'avvio della condivisione dello schermo.",stopScreenShareGeneric:"Non è stato possibile arrestare la condivisione dello schermo",callNetworkQualityLow:"La qualità della rete è bassa.",teamsMeetingCallNetworkQualityLow:"La qualità della rete è bassa.",callNoSpeakerFound:"Non sono stati trovati altoparlanti o cuffie. Connetti un dispositivo audio per ascoltare la chiamata.",callNoMicrophoneFound:"Nessun microfono trovato. Connetti un dispositivo di input audio.",callMicrophoneAccessDenied:"Impossibile accedere al microfono. Fare clic sul blocco nella barra degli indirizzi per concedere l'autorizzazione a questa pagina Web.",callMicrophoneAccessDeniedSafari:"Impossibile accedere al microfono. Aggiornare la pagina per consentire le autorizzazioni oppure controllare le impostazioni del browser e verificare che le autorizzazioni siano abilitate per questo sito Web.",callMicrophoneMutedBySystem:"Il sistema disattiva l'audio.",callMicrophoneUnmutedBySystem:"Il microfono è stato ripristinato e il sistema ha riattivato l'audio.",callMacOsMicrophoneAccessDenied:"Impossibile accedere al microfono. Concedi l'autorizzazione al microfono nelle impostazioni di privacy di macOS.",callLocalVideoFreeze:"Larghezza di banda di rete insufficiente. Il video potrebbe essere sospeso per gli altri partecipanti alla chiamata.",callCameraAccessDenied:"Non è possibile accedere alla fotocamera. Fare clic sul blocco nella barra degli indirizzi per concedere l'autorizzazione a questa pagina Web.",callCameraAccessDeniedSafari:"Non è possibile accedere alla fotocamera. Aggiornare la pagina per consentire le autorizzazioni oppure controllare le impostazioni del browser e verificare che le autorizzazioni siano abilitate per questo sito Web.",callCameraAlreadyInUse:"Non è possibile accedere alla fotocamera. Potrebbe essere già in uso da un'altra applicazione.",callVideoStoppedBySystem:"Il video è stato arrestato dal sistema.",callVideoRecoveredBySystem:"Il video è ripreso.",callMacOsCameraAccessDenied:"MacOS sta bloccando l'accesso alla fotocamera. Aggiorna le impostazioni di privacy per consentire a questo browser di accedere alla fotocamera.",callMacOsScreenShareAccessDenied:"MacOS sta bloccando la condivisione dello schermo. Aggiorna le impostazioni di privacy per consentire a questo browser di registrare lo schermo.",dismissButtonAriaLabel:"Chiudi",failedToJoinCallGeneric:"Non è stato possibile partecipare alla chiamata.",failedToJoinCallInvalidMeetingLink:"Impossibile partecipare alla riunione. Collegamento non valido.",cameraFrozenForRemoteParticipants:"Gli utenti della chiamata hanno problemi a visualizzare il video. Controllare i dispositivi e la rete.",unableToStartVideoEffect:"Non è possibile applicare l'effetto video.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Non è possibile avviare i contenuti in evidenza sui partecipanti perché il numero massimo di partecipanti è già in evidenza.",mutedByRemoteParticipant:"L'audio è stato disattivato da un altro partecipante."};var notificationStack$b={callingNetworkFailure:{title:"Problema di connessione della chiamata",message:"Sembra che tu sia offline. Controllare la connessione di rete e riprovare.",dismissButtonAriaLabel:"Chiudi"},startVideoGeneric:{title:"Non è stato possibile avviare il video.",dismissButtonAriaLabel:"Chiudi"},stopVideoGeneric:{title:"Non è stato possibile arrestare il video.",dismissButtonAriaLabel:"Chiudi"},muteGeneric:{title:"Non è stato possibile disattivare l'audio del microfono.",dismissButtonAriaLabel:"Chiudi"},unmuteGeneric:{title:"Non è stato possibile riattivare l'audio del microfono.",dismissButtonAriaLabel:"Chiudi"},speakingWhileMuted:{title:"L'audio è disattivato.",dismissButtonAriaLabel:"Chiudi"},startScreenShareGeneric:{title:"Si è verificato un problema durante l'avvio della condivisione dello schermo.",dismissButtonAriaLabel:"Chiudi"},stopScreenShareGeneric:{title:"Non è stato possibile arrestare la condivisione dello schermo.",dismissButtonAriaLabel:"Chiudi"},callNetworkQualityLow:{title:"La qualità della rete è bassa.",dismissButtonAriaLabel:"Chiudi"},teamsMeetingCallNetworkQualityLow:{title:"La qualità della rete è bassa.",dismissButtonAriaLabel:"Chiudi",message:"Partecipa a questa chiamata dal telefono per un suono migliore. Puoi continuare a visualizzare la riunione su questo dispositivo.",primaryButtonLabel:"Partecipa tramite telefono"},callNoSpeakerFound:{title:"Non sono stati trovati altoparlanti o cuffie",message:"Connetti un dispositivo audio per ascoltare la chiamata.",dismissButtonAriaLabel:"Chiudi"},callNoMicrophoneFound:{title:"Nessun microfono trovato",message:"Connetti un dispositivo di input audio.",dismissButtonAriaLabel:"Chiudi"},callMicrophoneAccessDenied:{title:"Impossibile accedere al microfono",message:"Fare clic sul blocco nella barra degli indirizzi per concedere l'autorizzazione a questa pagina Web.",dismissButtonAriaLabel:"Chiudi"},callMicrophoneAccessDeniedSafari:{title:"Impossibile accedere al microfono",message:"Aggiornare la pagina per consentire le autorizzazioni oppure controllare le impostazioni del browser e verificare che le autorizzazioni siano abilitate per questo sito Web.",dismissButtonAriaLabel:"Chiudi"},callMicrophoneMutedBySystem:{title:"Il sistema disattiva l'audio.",dismissButtonAriaLabel:"Chiudi"},callMicrophoneUnmutedBySystem:{title:"Il microfono è stato ripristinato e il sistema ha riattivato l'audio.",dismissButtonAriaLabel:"Chiudi"},callMacOsMicrophoneAccessDenied:{title:"Impossibile accedere al microfono",message:"Concedi l'autorizzazione al microfono nelle impostazioni di privacy di macOS.",dismissButtonAriaLabel:"Chiudi"},callLocalVideoFreeze:{title:"Larghezza di banda di rete insufficiente",message:"Il video potrebbe essere sospeso per gli altri partecipanti alla chiamata.",dismissButtonAriaLabel:"Chiudi"},callCameraAccessDenied:{title:"Impossibile accedere alla fotocamera",message:"Fare clic sul blocco nella barra degli indirizzi per concedere l'autorizzazione a questa pagina Web.",dismissButtonAriaLabel:"Chiudi"},callCameraAccessDeniedSafari:{title:"Impossibile accedere alla fotocamera",message:"Aggiornare la pagina per consentire le autorizzazioni oppure controllare le impostazioni del browser e verificare che le autorizzazioni siano abilitate per questo sito Web.",dismissButtonAriaLabel:"Chiudi"},callCameraAlreadyInUse:{title:"Impossibile accedere alla fotocamera",message:"Potrebbe essere già in uso da un'altra applicazione.",dismissButtonAriaLabel:"Chiudi"},callVideoStoppedBySystem:{title:"Il video è stato arrestato dal sistema.",dismissButtonAriaLabel:"Chiudi"},callVideoRecoveredBySystem:{title:"Il video è ripreso.",dismissButtonAriaLabel:"Chiudi"},callMacOsCameraAccessDenied:{title:"MacOS blocca l'accesso alla fotocamera",message:"Aggiorna le impostazioni di privacy per consentire a questo browser di accedere alla fotocamera.",dismissButtonAriaLabel:"Chiudi"},callMacOsScreenShareAccessDenied:{title:"MacOS sta bloccando la condivisione dello schermo",message:"Aggiorna le impostazioni di privacy per consentire a questo browser di registrare lo schermo.",dismissButtonAriaLabel:"Chiudi"},failedToJoinCallGeneric:{title:"Non è stato possibile partecipare alla chiamata.",dismissButtonAriaLabel:"Chiudi"},failedToJoinCallInvalidMeetingLink:{title:"Impossibile partecipare alla riunione. Collegamento non valido.",dismissButtonAriaLabel:"Chiudi"},cameraFrozenForRemoteParticipants:{title:"Gli utenti della chiamata hanno problemi a visualizzare il video",message:"Controllare i dispositivi e la rete.",dismissButtonAriaLabel:"Chiudi"},unableToStartVideoEffect:{title:"Non è possibile applicare l'effetto video.",dismissButtonAriaLabel:"Chiudi"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Impossibile avviare i contenuti in evidenza sui partecipanti",message:"Il numero massimo di partecipanti è già stato indicato.",dismissButtonAriaLabel:"Chiudi"},mutedByRemoteParticipant:{title:"L'audio è stato disattivato da un altro partecipante.",dismissButtonAriaLabel:"Chiudi"},recordingStarted:{title:"La registrazione è stata avviata.",message:"Partecipando, fornisci il consenso per la trascrizione di questa riunione.",dismissButtonAriaLabel:"Chiudi"},transcriptionStarted:{title:"La trascrizione è stata avviata.",message:"Partecipando, fornisci il consenso per la trascrizione di questa riunione.",dismissButtonAriaLabel:"Chiudi"},recordingStopped:{title:"È in corso il salvataggio della registrazione.",message:"La registrazione è stata arrestata.",dismissButtonAriaLabel:"Chiudi"},transcriptionStopped:{title:"È in corso il salvataggio della trascrizione.",message:"La trascrizione è stata arrestata.",dismissButtonAriaLabel:"Chiudi"},recordingAndTranscriptionStarted:{title:"Registrazione e trascrizione avviate.",message:"Partecipando, fornisci il consenso per la trascrizione di questa riunione.",dismissButtonAriaLabel:"Chiudi"},recordingAndTranscriptionStopped:{title:"È in corso il salvataggio della registrazione e della trascrizione.",message:"Registrazione e trascrizione interrotte.",dismissButtonAriaLabel:"Chiudi"},recordingStoppedStillTranscribing:{title:"La registrazione è stata arrestata.",message:"È in corso la trascrizione solo di questa riunione.",dismissButtonAriaLabel:"Chiudi"},transcriptionStoppedStillRecording:{title:"La trascrizione è stata arrestata.",message:"È in corso la registrazione solo di questa riunione.",dismissButtonAriaLabel:"Chiudi"},assignedBreakoutRoomOpened:{title:"Verrai spostato nella stanza assegnata tra 5 secondi.",dismissButtonAriaLabel:"Chiudi"},assignedBreakoutRoomChanged:{title:"La stanza di lavoro è cambiata",message:"Verrai spostato nella stanza assegnata tra 5 secondi.",dismissButtonAriaLabel:"Chiudi"},assignedBreakoutRoomClosed:{title:"Il gruppo di lavoro è chiuso",message:"Ti reinsegneremo alla riunione tra 5 secondi.",dismissButtonAriaLabel:"Chiudi"},assignedBreakoutRoomOpenedPromptJoin:{title:"Vuoi partecipare alla stanza di lavoro?",message:"Ti è stata assegnata una stanza di lavoro.",dismissButtonAriaLabel:"Chiudi",primaryButtonLabel:"Accedi alla stanza",secondaryButtonLabel:"Più tardi"},breakoutRoomJoined:{title:"Sei stato aggiunto alla stanza di lavoro.",dismissButtonAriaLabel:"Chiudi"},breakoutRoomClosingSoon:{title:"Il limite di tempo della stanza di lavoro sta per scadere.",message:"Questa stanza verrà chiusa tra 30 secondi",dismissButtonAriaLabel:"Chiudi"},capabilityTurnVideoOnPresent:{title:"La videocamera è stata abilitata",message:"Attiva se vuoi condividere il video.",dismissButtonAriaLabel:"Chiudi"},capabilityTurnVideoOnAbsent:{title:"La videocamera è stata disabilitata",message:"Non puoi più condividere il video.",dismissButtonAriaLabel:"Chiudi"},capabilityUnmuteMicPresent:{title:"Il microfono è stato abilitato",message:"Se vuoi parlare, puoi riattivare l'audio.",dismissButtonAriaLabel:"Chiudi"},capabilityUnmuteMicAbsent:{title:"Il tuo microfono è stato disattivato.",message:"Non puoi più riattivare l'audio.",dismissButtonAriaLabel:"Chiudi"},togetherModeStarted:{title:"Togethermode è stato avviato",dismissButtonAriaLabel:"Chiudi"},togetherModeEnded:{title:"Togethermode è terminato",dismissButtonAriaLabel:"Chiudi"},transcriptionError:{title:"Si è verificato un errore con la trascrizione.",message:"La trascrizione è stata interrotta. Riprova più tardi",dismissButtonAriaLabel:"Chiudi"},transcriptionStartedByYou:{title:"La trascrizione è stata avviata.",message:"Avviato da te. Informa tutti che la chiamata viene trascritta.",dismissButtonAriaLabel:"Chiudi"}};var videoGallery$b={screenIsBeingSharedMessage:"Stai condividendo lo schermo",screenShareLoadingMessage:"Caricamento dello schermo di {participant}",localScreenShareLoadingMessage:"Caricamento della schermata in corso...",localVideoLabel:"Tu",localVideoCameraSwitcherLabel:"Cambia videocamera",localVideoMovementLabel:"Riquadro video locale mobile",localVideoMovementAriaLabel:"Riquadro video locale mobile",localVideoSelectedDescription:"{cameraName} selezionata",displayNamePlaceholder:"Partecipante senza nome",fitRemoteParticipantToFrame:"Adatta all'inquadratura",fillRemoteParticipantFrame:"Riempi l'inquadratura",pinParticipantForMe:"Aggiungi per me",pinParticipantForMeLimitReached:"Segnaposto (limite raggiunto)",unpinParticipantForMe:"Rimuovi",pinParticipantMenuItemAriaLabel:"Aggiungi {participantName}",unpinParticipantMenuItemAriaLabel:"Rimuovi {participantName}",pinnedParticipantAnnouncementAriaLabel:"{participantName} aggiunto",unpinnedParticipantAnnouncementAriaLabel:"{participantName} sbloccato",startSpotlightVideoTileMenuLabel:"Metti in evidenza per tutti",addSpotlightVideoTileMenuLabel:"Metti in evidenza",spotlightLimitReachedMenuTitle:"È stato raggiunto il limite di contenuti in evidenza",stopSpotlightVideoTileMenuLabel:"Interrompi la posizione in evidenza",stopSpotlightOnSelfVideoTileMenuLabel:"Chiudi visualizzazione in evidenza",attendeeRole:"Partecipante",muteParticipantMenuItemLabel:"Disattiva audio",waitingScreenText:"In attesa di altri partecipanti",forbidAudio:"Disabilita microfono",permitAudio:"Consenti microfono",forbidAudioTileMenuLabel:"Disabilita microfono",permitAudioTileMenuLabel:"Consenti microfono",forbidVideo:"Disabilita videocamera",permitVideo:"Consenti videocamera",forbidVideoTileMenuLabel:"Disabilita videocamera",permitVideoTileMenuLabel:"Consenti videocamera"};var dialpad$b={placeholderText:"Inserisci numero di telefono",deleteButtonAriaLabel:"Elimina"};var holdButton$b={onLabel:"Riprendi",offLabel:"Mantenere",tooltipOnContent:"Riprendi chiamata",tooltipOffContent:"Metti in attesa chiamata"};var videoTile$b={participantStateRinging:"Chiamata in corso...",participantReconnecting:"Riconnessione in corso...",participantStateHold:"Per l'attesa",moreOptionsButtonAriaLabel:"Altre opzioni {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"audio disattivato",moreOptionsParticipantMutedStateUnmutedAriaLabel:"audio riattivato",moreOptionsParticipantHandRaisedAriaLabel:"Mano alzata",moreOptionsParticipantIsSpeakingAriaLabel:"Impegnato in una conversazione",moreOptionsParticipantMicDisabledAriaLabel:"Microfono disabilitato",moreOptionsParticipantCameraDisabledAriaLabel:"Fotocamera disabilitata"};var CameraAndMicrophoneSitePermissionsRequest$b={primaryText:"Consenti a {appName} di usare la fotocamera e il microfono",secondaryText:"In questo modo i partecipanti possono vederti e sentirti.",linkText:"Serve aiuto? Guida alla risoluzione dei problemi",primaryButtonText:"Continua senza fotocamera e microfono",ariaLabel:"Consenti l'accesso alla fotocamera e al microfono"};var CameraSitePermissionsRequest$b={primaryText:"Consenti a {appName} di usare la fotocamera",secondaryText:"In questo modo i partecipanti possono vederti.",linkText:"Serve aiuto? Guida alla risoluzione dei problemi",primaryButtonText:"Continua senza fotocamera",ariaLabel:"Consenti l'accesso alla fotocamera"};var MicrophoneSitePermissionsRequest$b={primaryText:"Consenti a {appName} di usare il microfono",secondaryText:"In questo modo i partecipanti possono sentirti.",linkText:"Serve aiuto? Guida alla risoluzione dei problemi",primaryButtonText:"Continua senza microfono",ariaLabel:"Consenti l'accesso al microfono"};var CameraAndMicrophoneSitePermissionsCheck$b={primaryText:"Verifica dell'accesso alla fotocamera e al microfono",secondaryText:"Consenti l'accesso se richiesto. In questo modo i partecipanti possono vederti e sentirti.",linkText:"Serve aiuto? Guida alla risoluzione dei problemi",primaryButtonText:"Continua senza fotocamera e microfono",ariaLabel:"Verifica dell'accesso alla fotocamera e al microfono. Consenti l'accesso se richiesto."};var CameraSitePermissionsCheck$b={primaryText:"Verifica dell'accesso alla fotocamera",secondaryText:"Consenti l'accesso se richiesto. In questo modo i partecipanti possono vederti.",linkText:"Serve aiuto? Guida alla risoluzione dei problemi",primaryButtonText:"Continua senza fotocamera",ariaLabel:"Verifica dell'accesso alla fotocamera. Consenti l'accesso se richiesto."};var MicrophoneSitePermissionsCheck$b={primaryText:"Verifica dell'accesso al microfono",secondaryText:"Consenti l'accesso se richiesto. In questo modo i partecipanti possono sentirti.",linkText:"Serve aiuto? Guida alla risoluzione dei problemi",primaryButtonText:"Continua senza microfono",ariaLabel:"Verifica dell'accesso al microfono. Consenti l'accesso se richiesto."};var CameraAndMicrophoneSitePermissionsDenied$b={primaryText:"Non è possibile accedere alla fotocamera e al microfono",secondaryText:"Fare clic sull'icona di blocco nella barra degli indirizzi per concedere le autorizzazioni del microfono a questa pagina Web. Potrebbe essere necessario un aggiornamento della pagina.",primaryButtonText:"Continua senza fotocamera e microfono",linkText:"Serve aiuto? Guida alla risoluzione dei problemi"};var CameraAndMicrophoneSitePermissionsDeniedSafari$b={primaryText:"Non è possibile accedere alla fotocamera e al microfono",secondaryText:"Aggiornare la pagina per consentire le autorizzazioni oppure controllare le impostazioni del browser e verificare che le autorizzazioni siano abilitate per questo sito Web.",primaryButtonText:"Continua senza fotocamera e microfono",linkText:"Serve aiuto? Guida alla risoluzione dei problemi"};var CameraSitePermissionsDenied$b={primaryText:"Impossibile accedere alla videocamera",secondaryText:"Fare clic sull'icona di blocco nella barra degli indirizzi per concedere le autorizzazioni della fotocamera a questa pagina Web. Potrebbe essere necessario un aggiornamento della pagina.",primaryButtonText:"Continua senza fotocamera",linkText:"Serve aiuto? Guida alla risoluzione dei problemi"};var MicrophoneSitePermissionsDenied$b={primaryText:"Impossibile accedere al microfono",secondaryText:"Fare clic sull'icona di blocco nella barra degli indirizzi per concedere le autorizzazioni del microfono a questa pagina Web. Potrebbe essere necessario un aggiornamento della pagina.",primaryButtonText:"Continua senza microfono",linkText:"Serve aiuto? Guida alla risoluzione dei problemi"};var CameraSitePermissionsDeniedSafari$b={primaryText:"Impossibile accedere alla videocamera",secondaryText:"Aggiornare la pagina per consentire le autorizzazioni oppure controllare le impostazioni del browser e verificare che le autorizzazioni siano abilitate per questo sito Web.",primaryButtonText:"Continua senza fotocamera",linkText:"Serve aiuto? Guida alla risoluzione dei problemi"};var MicrophoneSitePermissionsDeniedSafari$b={primaryText:"Impossibile accedere al microfono",secondaryText:"Aggiornare la pagina per consentire le autorizzazioni oppure controllare le impostazioni del browser e verificare che le autorizzazioni siano abilitate per questo sito Web.",primaryButtonText:"Continua senza microfono",linkText:"Serve aiuto? Guida alla risoluzione dei problemi"};var UnsupportedBrowser$b={primaryText:"Browser non supportato",secondaryText:"Partecipa alla chiamata utilizzando un browser compatibile.",moreHelpLinkText:"Visualizza i requisiti di compatibilità"};var UnsupportedBrowserVersion$b={primaryText:"Aggiornamento del browser necessario",secondaryText:"Per garantire la migliore chiamata possibile, aggiorna il browser e riprova a partecipare alla chiamata.",moreHelpLinkText:"Visualizza i requisiti di compatibilità",continueAnywayButtonText:"Avvia chiamata senza aggiornamento"};var UnsupportedOperatingSystem$b={primaryText:"Sistemi operativi non supportati",secondaryText:"Partecipa alla chiamata utilizzando un dispositivo con un sistema operativo compatibile.",moreHelpLinkText:"Visualizza i requisiti di compatibilità"};var BrowserPermissionDenied$b={primaryText:"Non è possibile usare la fotocamera o il microfono",secondaryText:"Il browser potrebbe non avere accesso alla fotocamera o al microfono. Per risolvere il problema, aprire Preferenze di sistema.",primaryButtonText:"Riprova",linkText:"Serve aiuto? Guida alla risoluzione dei problemi"};var BrowserPermissionDeniedIOS$b={primaryText:"Consenti l'accesso al microfono per continuare",secondaryText:"Così gli altri partecipanti possono sentirti.",primaryButtonText:"Riprova",imageAltText:"Posizione dell'autorizzazione del microfono e della fotocamera per iOS",linkText:"Serve aiuto? Guida alla risoluzione dei problemi",step1Text:"Vai alle impostazioni dell'app",step2Text:"Scorri verso il basso fino alle impostazioni per questo browser",step3Text:"Attiva microfono (fotocamera facoltativa)",step4Text:"Prova a partecipare di nuovo alla chiamata",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$b={leftNavButtonAriaLabel:"pagina precedente {current} di {total}",rightNavButtonAriaLabel:"pagina successiva {current} di {total}"};var meetingConferencePhoneInfo$b={meetingConferencePhoneInfoModalTitle:"Partecipa alla chiamata dal tuo telefono",meetingConferencePhoneInfoModalDialIn:"Componi il numero",meetingConferencePhoneInfoModalMeetingId:"Immetti l'ID riunione",meetingConferencePhoneInfoModalWait:"Attendi l'ammissione da parte dell'organizzatore",meetingConferencePhoneInfoModalNoPhoneAvailable:"Nessun telefono disponibile",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Numero verde)",meetingConferencePhoneInfoModalToll:"(Numero a tariffa)"};var IncomingCallNotification$b={incomingCallNotificationPlaceholderId:"Utente sconosciuto",incomingCallNotificationPlaceholderAlert:"{callerName} sta chiamando",incomingCallNoticicationAcceptWithAudioAriaLabel:"Accetta con audio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Accetta con video",incomingCallNoticicationRejectAriaLabel:"Rifiuta",incomingCallNotificationAcceptButtonLabel:"Accetta",incomingCallNotificationRejectButtonLabel:"Rifiuta",incomingCallNotificationAccceptWithVideoButtonLabel:"Accetta con video",incomingCallNotificationDismissButtonAriaLabel:"Ignora"};var it_IT$1 = {participantItem:participantItem$b,ParticipantList:ParticipantList$b,typingIndicator:typingIndicator$b,sendBox:sendBox$b,richTextSendBox:richTextSendBox$b,realTimeTextModal:realTimeTextModal$b,captionsBanner:captionsBanner$b,mentionPopover:mentionPopover$b,imageOverlay:imageOverlay$b,messageStatusIndicator:messageStatusIndicator$b,endCallButton:endCallButton$b,cameraButton:cameraButton$b,microphoneButton:microphoneButton$b,devicesButton:devicesButton$b,participantsButton:participantsButton$b,screenShareButton:screenShareButton$b,raiseHandButton:raiseHandButton$b,reactionButton:reactionButton$b,realTimeText:realTimeText$b,captionsSettingsModal:captionsSettingsModal$b,spokenLanguages:spokenLanguages$b,captionLanguages:captionLanguages$b,startCaptionsButton:startCaptionsButton$b,startRealTimeTextButton:startRealTimeTextButton$b,messageThread:messageThread$b,errorBar:errorBar$b,notificationStack:notificationStack$b,videoGallery:videoGallery$b,dialpad:dialpad$b,holdButton:holdButton$b,videoTile:videoTile$b,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$b,CameraSitePermissionsRequest:CameraSitePermissionsRequest$b,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$b,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$b,CameraSitePermissionsCheck:CameraSitePermissionsCheck$b,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$b,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$b,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$b,CameraSitePermissionsDenied:CameraSitePermissionsDenied$b,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$b,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$b,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$b,UnsupportedBrowser:UnsupportedBrowser$b,UnsupportedBrowserVersion:UnsupportedBrowserVersion$b,UnsupportedOperatingSystem:UnsupportedOperatingSystem$b,BrowserPermissionDenied:BrowserPermissionDenied$b,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$b,verticalGallery:verticalGallery$b,meetingConferencePhoneInfo:meetingConferencePhoneInfo$b,IncomingCallNotification:IncomingCallNotification$b};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Italian (Italy).
 *
 * @public
 */
const COMPONENT_LOCALE_IT_IT = {
    strings: createComponentStrings(it_IT$1)
};

var participantItem$a={isMeText:"(自分)",menuTitle:"その他のオプション",removeButtonLabel:"削除",sharingIconLabel:"共有",mutedIconLabel:"ミュート状態",handRaisedIconLabel:"挙手 - 位置 {position}",displayNamePlaceholder:"名前のない参加者",participantStateRinging:"呼び出し中...",participantStateHold:"保留中",participantItemAriaLabel:"{displayName}、{handRaisedState}、{mutedState}、{micDisabledState}、{cameraDisabledState}、{sharingState}、{connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}、{handRaisedState}、{mutedState}、{micDisabledState}、{cameraDisabledState}、{sharingState}、{connectionState}、その他のオプション",attendeeRole:"出席者",micDisabledIconLabel:"マイクが無効になっています",cameraDisabledIconLabel:"カメラが無効になっています"};var ParticipantList$a={overflowParticipantCount:"その他 {overflowCount} 個"};var typingIndicator$a={singleUser:"{user} さんが入力しています...",multipleUsers:"{users} さんが入力しています...",multipleUsersAbbreviateOne:"{users} さんとその他 1 人が入力しています...",multipleUsersAbbreviateMany:"{users} さんと他 {numOthers} 人が入力しています...",delimiter:","};var sendBox$a={placeholderText:"メッセージを入力",textTooLong:"メッセージの長さが上限を超えています。",sendButtonAriaLabel:"メッセージの送信",attachmentUploadsPendingError:"ファイルをアップロードしています。お待ちください。",removeAttachment:"ファイルの削除",uploading:"アップロード中",uploadCompleted:"アップロードが完了しました",attachmentMoreMenu:"その他のオプション"};var richTextSendBox$a={placeholderText:"メッセージを入力してください",textTooLong:"メッセージの長さが上限を超えています。",sendButtonAriaLabel:"メッセージの送信",attachmentUploadsPendingError:"ファイルをアップロードしています。お待ちください。",imageUploadsPendingError:"画像をアップロードしています。お待ちください。",removeAttachment:"ファイルを削除します",uploading:"アップロード中",uploadCompleted:"アップロード完了",richTextBoldTooltip:"太字",richTextItalicTooltip:"斜体",richTextUnderlineTooltip:"下線",richTextBulletListTooltip:"箇条書き",richTextNumberListTooltip:"番号付きリスト",richTextIncreaseIndentTooltip:"インデントを増やします",richTextDecreaseIndentTooltip:"インデントを減らします",richTextInsertTableTooltip:"表を挿入します",richTextFormatButtonTooltip:"形式",richTextInsertRowOrColumnMenu:"挿入",richTextInsertTableMenuTitle:"{column} x {row} の表を挿入する",richTextInsertRowAboveMenu:"上に挿入",richTextInsertRowBelowMenu:"下に挿入",richTextInsertColumnLeftMenu:"左に挿入",richTextInsertColumnRightMenu:"右に挿入",richTextDeleteRowOrColumnMenu:"削除",richTextDeleteColumnMenu:"列の削除",richTextDeleteRowMenu:"行の削除",richTextDeleteTableMenu:"表の削除",richTextToolbarAriaLabel:"メッセージの書式設定ツール バー",richTextToolbarMoreButtonAriaLabel:"その他のオプション",attachmentMoreMenu:"その他のオプション",richTextNewBulletedListItemAnnouncement:"行頭文字、改行",richTextNewNumberedListItemAnnouncement:"{0}、改行",richTextBulletedListAppliedAnnouncement:"箇条書きが適用されました",richTextNumberedListAppliedAnnouncement:"番号付きリストが適用されました"};var realTimeTextModal$a={realTimeTextModalAriaLabel:"RTT モーダルを有効にする",realTimeTextModalTitle:"RTT を有効にしますか?",realTimeTextModalText:"会議の全員に対してリアルタイム テキスト (RTT) が有効になります。",realTimeTextConfirmButtonLabel:"有効にする",realTimeTextCancelButtonLabel:"キャンセル",realTimeTextCloseModalButtonAriaLabel:"RTT モーダルを閉じる"};var captionsBanner$a={captionsBannerSpinnerText:"キャプションを開始しています...",realTimeTextInputBoxDefaultText:"リアルタイムにメッセージを入力",realTimeTextInputErrorMessage:"メッセージは 2,000 文字以下にする必要があります",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (リアルタイム テキスト) は、会議の全期間のすべての参加者に対して有効になっています。",realTimeTextBannerLinkLabel:"詳細情報",captionsOnlyContainerTitle:"キャプション",realTimeTextOnlyContainerTitle:"リアルタイム テキスト",captionsAndRealTimeTextContainerTitle:"キャプションとリアルタイム テキスト",expandButtonAriaLabel:"展開",minimizeButtonAriaLabel:"折りたたむ"};var mentionPopover$a={mentionPopoverHeader:"候補"};var imageOverlay$a={downloadButtonLabel:"ダウンロード",dismissButtonAriaLabel:"閉じる",overlayVisibleMessage:"画像の詳細のオーバーレイ"};var messageStatusIndicator$a={deliveredAriaLabel:"メッセージを送信しました",deliveredTooltipText:"送信しました",seenAriaLabel:"他のユーザーが表示したメッセージ",seenTooltipText:"既読",readByTooltipText:"{remoteParticipantsCount} の {messageThreadReadCount} による読み取り",sendingAriaLabel:"メッセージを送信しています",sendingTooltipText:"送信中",failedToSendAriaLabel:"メッセージの送信に失敗しました",failedToSendTooltipText:"送信できませんでした"};var endCallButton$a={label:"退出",tooltipContent:"通話を終了",leaveOption:"退出",endCallOption:"全員の通話を終了"};var cameraButton$a={onLabel:"カメラ",offLabel:"カメラ",tooltipDisabledContent:"カメラが無効になっています",tooltipOnContent:"カメラをオフ",tooltipOffContent:"カメラをオンにする",tooltipVideoLoadingContent:"ビデオの読み込み中です",cameraMenuTitle:"カメラ",cameraMenuTooltip:"カメラを選択する",cameraButtonSplitRoleDescription:"分割ボタン",onSplitButtonAriaLabel:"カメラをオフにする",offSplitButtonAriaLabel:"カメラをオンにする",cameraActionTurnedOnAnnouncement:"カメラがオンになっています",cameraActionTurnedOffAnnouncement:"カメラがオフになっています",onSplitButtonPrimaryActionCamera:"カメラをオフにする",offSplitButtonPrimaryActionCamera:"カメラをオンにする",cameraPrimaryActionSplitButtonTitle:"カメラを使用する",videoEffectsMenuItemTitle:"効果",cameraButtonAriaDescription:"カメラのオプション"};var microphoneButton$a={onLabel:"マイク",offLabel:"マイク",tooltipDisabledContent:"マイクが無効になっています",tooltipOnContent:"マイクをミュートにする",tooltipOffContent:"マイクのミュートを解除する",microphoneMenuTitle:"マイク",microphoneMenuTooltip:"マイクを選択する",speakerMenuTitle:"スピーカー",speakerMenuTooltip:"スピーカーを選択する",microphoneButtonSplitRoleDescription:"分割ボタン",onSplitButtonAriaLabel:"マイクをミュートする",offSplitButtonAriaLabel:"マイクのミュートを解除する",microphoneActionTurnedOnAnnouncement:"マイクがオンになっています",microphoneActionTurnedOffAnnouncement:"マイクがオフになっています",offSplitButtonMicrophonePrimaryAction:"マイクのミュートを解除する",onSplitButtonMicrophonePrimaryAction:"マイクをミュートする",microphonePrimaryActionSplitButtonTitle:"マイクを使用する",microphoneAriaDescription:"オーディオのオプション",deepNoiseSuppressionTitle:"ノイズ抑制",deepNoiseSuppressionOnAnnouncement:"ディープ ノイズ抑制がオンになっています",deepNoiseSuppressionOffAnnouncement:"ディープ ノイズ抑制がオフになっています"};var devicesButton$a={label:"デバイス",tooltipContent:"デバイスの管理",cameraMenuTitle:"カメラ",cameraMenuTooltip:"カメラを選択する",audioDeviceMenuTitle:"オーディオ デバイス",audioDeviceMenuTooltip:"オーディオ デバイスを選択する",microphoneMenuTitle:"マイク",microphoneMenuTooltip:"マイクを選択する",speakerMenuTitle:"スピーカー",speakerMenuTooltip:"スピーカーを選択する"};var participantsButton$a={label:"ユーザー",ariaLabel:"連絡先",tooltipContent:"参加者を表示",menuHeader:"この通話で",participantsListButtonLabel:"{numParticipants} 人",muteAllButtonLabel:"全員をミュート",copyInviteLinkButtonLabel:"招待用のリンクをコピー",copyInviteLinkActionedAriaLabel:"招待用のリンクがコピーされました"};var screenShareButton$a={onLabel:"プレゼンテーションの停止",offLabel:"提示",tooltipDisabledContent:"プレゼンテーションは無効です",tooltipOnContent:"画面の表示",tooltipOffContent:"画面のプレゼンテーション"};var raiseHandButton$a={offLabel:"手を挙げる",onLabel:"低い",tooltipDisabledContent:"手を挙げる操作が無効になっています",tooltipOnContent:"手を下げる",tooltipOffContent:"手を挙げる"};var reactionButton$a={label:"リアクションする",ariaLabel:"リアクションする",tooltipDisabledContent:"リアクション操作が無効になっています",tooltipContent:"リアクションを送信します",likeReactionTooltipContent:"いいね!",heartReactionTooltipContent:"大好き",laughReactionTooltipContent:"笑い",applauseReactionTooltipContent:"喝采",surprisedReactionTooltipContent:"驚いた"};var realTimeText$a={isTypingText:"RTT 入力"};var captionsSettingsModal$a={captionsSettingsModalTitle:"ライブ キャプションの設定",captionsSettingsSpokenLanguageDropdownLabel:"音声言語",captionsSettingsCaptionLanguageDropdownLabel:"ライブ キャプションの言語",captionsSettingsSpokenLanguageDropdownInfoText:"この通話の全員が話す言語。",captionsSettingsCaptionLanguageDropdownInfoText:"ライブ キャプションはこの言語で表示されます。",captionsSettingsConfirmButtonLabel:"確認する",captionsSettingsCancelButtonLabel:"キャンセル",captionsSettingsModalAriaLabel:"ライブ キャプション設定モーダル",captionsSettingsCloseModalButtonAriaLabel:"ライブ キャプション設定を閉じる"};var spokenLanguages$a={"ar-ae":"アラビア語 - U.A.E.","ar-sa":"アラビア語 - サウジアラビア","da-dk":"デンマーク語","de-de":"ドイツ語 - ドイツ","en-au":"英語 - オーストラリア","en-ca":"英語 - カナダ","en-gb":"英語 - イギリス","en-in":"英語 - インド","en-nz":"英語 - ニュージーランド","en-us":"英語 - 米国","es-es":"スペイン語 - スペイン (モダン)","es-mx":"スペイン語 - メキシコ","fi-fi":"フィンランド語","fr-ca":"フランス語 - カナダ","fr-fr":"フランス語 - フランス","hi-in":"ヒンディー語","it-it":"イタリア語 - イタリア","ja-jp":"日本語","ko-kr":"韓国語","nb-no":"ノルウェー語 (ブークモール)","nl-be":"オランダ語 - ベルギー","nl-nl":"オランダ語 - オランダ","pl-pl":"ポーランド語","pt-br":"ポルトガル語 - ブラジル","ru-ru":"ロシア語","sv-se":"スウェーデン語","zh-cn":"中国語 - 中国","zh-hk":"中国語 - 香港特別行政区","cs-cz":"チェコ語","pt-pt":"ポルトガル語 - ポルトガル","tr-tr":"トルコ語","vi-vn":"ベトナム語","th-th":"タイ語","he-il":"ヘブライ語","cy-gb":"ウェールズ語","uk-ua":"ウクライナ語","el-gr":"ギリシャ語","hu-hu":"ハンガリー語","ro-ro":"ルーマニア語","sk-sk":"スロバキア語","zh-tw":"中国語 - 台湾"};var captionLanguages$a={ar:"アラビア語",da:"デンマーク語",de:"ドイツ語",en:"英語",es:"スペイン語",fi:"フィンランド語","fr-ca":"フランス語 - カナダ",fr:"フランス語 - フランス",hi:"ヒンディー語",it:"イタリア語",ja:"日本語",ko:"韓国語",nb:"ノルウェー語 (ブークモール)",nl:"オランダ語",pl:"ポーランド語",pt:"ポルトガル語 - ブラジル",ru:"ロシア語",sv:"スウェーデン語","zh-Hans":"中国語 (簡体字)","zh-Hant":"中国語 (繁体字)",cs:"チェコ語","pt-pt":"ポルトガル語 - ポルトガル",tr:"トルコ語",vi:"ベトナム語",th:"タイ語",he:"ヘブライ語",cy:"ウェールズ語",uk:"ウクライナ語",el:"ギリシャ語",hu:"ハンガリー語",ro:"ルーマニア語",sk:"スロバキア語"};var startCaptionsButton$a={onLabel:"ライブ キャプションをオンにする",offLabel:"キャプションをオフにする",tooltipOnContent:"キャプションをオフにする",tooltipOffContent:"ライブ キャプションをオンにする"};var startRealTimeTextButton$a={label:"この呼び出しの RTT を有効にする",tooltipOffContent:"この呼び出しの RTT を有効にする"};var messageThread$a={yesterday:"昨日",sunday:"日曜日",monday:"月曜日",tuesday:"火曜日",wednesday:"水曜日",thursday:"木曜日",friday:"金曜日",saturday:"土曜日",participantJoined:"チャットに参加しました。",participantLeft:"チャットから退出しました。",editMessage:"編集",removeMessage:"削除",resendMessage:"もう一度送信",failToSendTag:"送信できませんでした",editedTag:"編集済み",liveAuthorIntro:"{author} からのメッセージ",editedMessageLiveAuthorIntro:"{author} さんからのメッセージを編集しました",editedMessageLocalUserLiveAuthorIntro:"メッセージを編集しました",messageContentAriaText:"{status} {author} さんが {message} と言いました",messageContentMineAriaText:"{status} あなたが {message} と言いました",editBoxTextLimit:"メッセージが{limitNumber}文字の制限を超えています",editBoxPlaceholderText:"メッセージを編集する",newMessagesIndicator:"新しいメッセージ",noDisplayNameSub:"名前がありません",editBoxCancelButton:"取り消し",editBoxSubmitButton:"完了",messageReadCount:"{remoteParticipantsCount} の {messageReadByCount} による読み取り",actionMenuMoreOptions:"その他のオプション",downloadAttachment:"ファイルをダウンロード",openAttachment:"ファイルを開く",blockedWarningText:"組織のポリシーにより、このメッセージは削除されました。",blockedWarningLinkText:"詳細",attachmentCardGroupMessage:"メッセージに添付ファイルが {attachmentCount} 個あります",messageDeletedAnnouncementAriaLabel:"メッセージが削除されます",richTextBoldTooltip:"太字",richTextItalicTooltip:"斜体",richTextUnderlineTooltip:"下線",richTextBulletListTooltip:"箇条書き",richTextNumberListTooltip:"番号付きリスト",richTextIncreaseIndentTooltip:"インデントを増やす",richTextDecreaseIndentTooltip:"インデントを減らす",richTextInsertTableTooltip:"表の挿入",richTextFormatButtonTooltip:"書式設定します",richTextInsertRowOrColumnMenu:"挿入",richTextInsertTableMenuTitle:"{column} x {row} の表を挿入する",richTextInsertRowAboveMenu:"上に挿入",richTextInsertRowBelowMenu:"下に挿入",richTextInsertColumnLeftMenu:"左に挿入",richTextInsertColumnRightMenu:"右に挿入",richTextDeleteRowOrColumnMenu:"削除",richTextDeleteColumnMenu:"列の削除",richTextDeleteRowMenu:"行の削除",richTextDeleteTableMenu:"テーブルの削除",richTextToolbarMoreButtonAriaLabel:"その他のオプション",attachmentUploadsPendingError:"ファイルをアップロードしています。お待ちください。",imageUploadsPendingError:"画像をアップロードしています。お待ちください。",richTextNewBulletedListItemAnnouncement:"行頭文字、改行",richTextNewNumberedListItemAnnouncement:"{0}、改行",richTextBulletedListAppliedAnnouncement:"箇条書きが適用されました",richTextNumberedListAppliedAnnouncement:"番号付きリストが適用されました"};var errorBar$a={unableToReachChatService:"オフラインです",accessDenied:"チャット サービスにアクセスできません- 指定されたユーザー資格情報をご確認ください",userNotInChatThread:"このチャット スレッドにはもう参加していません",sendMessageNotInChatThread:"このチャット スレッドに参加していないため、メッセージを送信できませんでした",sendMessageGeneric:"メッセージを送信できませんでした。",callingNetworkFailure:"通話の接続中に問題が発生しました。オフラインのようです。",startVideoGeneric:"ビデオを開始できませんでした",stopVideoGeneric:"ビデオを停止できませんでした",muteGeneric:"マイクをミュートできませんでした",unmuteGeneric:"マイクのミュートを解除できませんでした",speakingWhileMuted:"マイクがミュートになっています",startScreenShareGeneric:"画面共有の開始中に問題が発生しました。",stopScreenShareGeneric:"画面の共有を停止できませんでした",callNetworkQualityLow:"ネットワークの品質が低くなっています。",teamsMeetingCallNetworkQualityLow:"ネットワークの品質が低くなっています。",callNoSpeakerFound:"スピーカーまたはヘッドフォンが見つかりません。音声デバイスを接続して通話を聞きます。",callNoMicrophoneFound:"マイクが見つかりません。オーディオ入力デバイスを接続します。",callMicrophoneAccessDenied:"マイクにアクセスできません。アドレス バーのロックをクリックして、この Web ページにアクセス許可を付与します。",callMicrophoneAccessDeniedSafari:"マイクにアクセスできません。ページを更新してアクセス許可を可能にするか、このブラウザーの設定を確認し、この Web サイトのアクセス許可が有効になっていることを確認してください。",callMicrophoneMutedBySystem:"システムによってミュートになっています。",callMicrophoneUnmutedBySystem:"マイクが回復し、システムによってミュートが解除されました。",callMacOsMicrophoneAccessDenied:"マイクにアクセスできません。macOS のプライバシー設定でマイクへのアクセス許可を付与します。",callLocalVideoFreeze:"ネットワーク帯域幅が不十分です。通話中に他のユーザーのビデオが一時停止している可能性があります。",callCameraAccessDenied:"カメラにアクセスできません。アドレス バーのロックをクリックして、この Web ページにアクセス許可を付与します。",callCameraAccessDeniedSafari:"カメラにアクセスできません。ページを更新してアクセス許可を可能にするか、このブラウザーの設定を確認し、この Web サイトのアクセス許可が有効になっていることを確認してください。",callCameraAlreadyInUse:"カメラにアクセスできません。別のアプリケーションによって既に使用されている可能性があります。",callVideoStoppedBySystem:"ビデオはシステムによって停止されました。",callVideoRecoveredBySystem:"ビデオが再開されました。",callMacOsCameraAccessDenied:"MacOS がカメラへのアクセスをブロックしています。プライバシー設定を更新して、このブラウザーがカメラにアクセスできるようにします。",callMacOsScreenShareAccessDenied:"MacOS は画面の共有をブロックしています。プライバシー設定を更新して、このブラウザーで画面を録画できるようにします。",dismissButtonAriaLabel:"閉じる",failedToJoinCallGeneric:"通話に参加できませんでした",failedToJoinCallInvalidMeetingLink:"会議に参加できません。リンクが無効です。",cameraFrozenForRemoteParticipants:"通話のユーザーにビデオの表示で問題が発生しています。デバイスとネットワークを確認してください。",unableToStartVideoEffect:"ビデオ効果を適用できません。",startSpotlightWhileMaxParticipantsAreSpotlighted:"最大数の参加者に既にスポットライトが設定されているため、参加者にスポットライトを開始できません。",mutedByRemoteParticipant:"他の参加者によってミュートにされました。"};var notificationStack$a={callingNetworkFailure:{title:"通話の接続に問題があります",message:"オフラインのようです。ネットワーク接続を確認して、もう一度お試しください。",dismissButtonAriaLabel:"閉じる"},startVideoGeneric:{title:"ビデオを開始できませんでした。",dismissButtonAriaLabel:"閉じる"},stopVideoGeneric:{title:"ビデオを停止できませんでした。",dismissButtonAriaLabel:"閉じる"},muteGeneric:{title:"マイクをミュートできませんでした。",dismissButtonAriaLabel:"閉じる"},unmuteGeneric:{title:"マイクのミュートを解除できませんでした。",dismissButtonAriaLabel:"閉じる"},speakingWhileMuted:{title:"ミュートにしています。",dismissButtonAriaLabel:"閉じる"},startScreenShareGeneric:{title:"画面共有の開始中に問題が発生しました。",dismissButtonAriaLabel:"閉じる"},stopScreenShareGeneric:{title:"画面の共有を停止できませんでした。",dismissButtonAriaLabel:"閉じる"},callNetworkQualityLow:{title:"ネットワークの品質が低くなっています。",dismissButtonAriaLabel:"閉じる"},teamsMeetingCallNetworkQualityLow:{title:"ネットワークの品質が低くなっています。",dismissButtonAriaLabel:"閉じる",message:"サウンドを向上させるには、電話からこの通話に参加してください。このデバイスで引き続き会議を表示できます。",primaryButtonLabel:"電話で参加"},callNoSpeakerFound:{title:"スピーカーまたはヘッドフォンが見つかりません",message:"音声デバイスを接続して通話を聞いてください。",dismissButtonAriaLabel:"閉じる"},callNoMicrophoneFound:{title:"マイクが見つかりません",message:"オーディオ入力デバイスを接続します。",dismissButtonAriaLabel:"閉じる"},callMicrophoneAccessDenied:{title:"マイクにアクセスできません",message:"アドレス バーのロックをクリックして、この Web ページにアクセス許可を付与します。",dismissButtonAriaLabel:"閉じる"},callMicrophoneAccessDeniedSafari:{title:"マイクにアクセスできません",message:"ページを更新してアクセス許可を可能にするか、このブラウザーの設定を確認し、この Web サイトのアクセス許可が有効になっていることを確認してください。",dismissButtonAriaLabel:"閉じる"},callMicrophoneMutedBySystem:{title:"システムによってミュートになっています。",dismissButtonAriaLabel:"閉じる"},callMicrophoneUnmutedBySystem:{title:"マイクが回復し、システムによってミュートが解除されました。",dismissButtonAriaLabel:"閉じる"},callMacOsMicrophoneAccessDenied:{title:"マイクにアクセスできません",message:"macOS のプライバシー設定でマイクへのアクセス許可を付与します。",dismissButtonAriaLabel:"閉じる"},callLocalVideoFreeze:{title:"ネットワーク帯域幅が貧弱です",message:"通話中に他のユーザーのビデオが一時停止している可能性があります。",dismissButtonAriaLabel:"閉じる"},callCameraAccessDenied:{title:"カメラにアクセスできません",message:"アドレス バーのロックをクリックして、この Web ページにアクセス許可を付与します。",dismissButtonAriaLabel:"閉じる"},callCameraAccessDeniedSafari:{title:"カメラにアクセスできません",message:"ページを更新してアクセス許可を可能にするか、このブラウザーの設定を確認し、この Web サイトのアクセス許可が有効になっていることを確認してください。",dismissButtonAriaLabel:"閉じる"},callCameraAlreadyInUse:{title:"カメラにアクセスできません",message:"別のアプリケーションによって既に使用されている可能性があります。",dismissButtonAriaLabel:"閉じる"},callVideoStoppedBySystem:{title:"ビデオはシステムによって停止されました。",dismissButtonAriaLabel:"閉じる"},callVideoRecoveredBySystem:{title:"ビデオが再開されました。",dismissButtonAriaLabel:"閉じる"},callMacOsCameraAccessDenied:{title:"MacOS がカメラへのアクセスをブロックしています",message:"プライバシー設定を更新して、このブラウザーがカメラにアクセスできるようにします。",dismissButtonAriaLabel:"閉じる"},callMacOsScreenShareAccessDenied:{title:"MacOS で画面共有がブロックされています",message:"プライバシー設定を更新して、このブラウザーで画面を録画できるようにします。",dismissButtonAriaLabel:"閉じる"},failedToJoinCallGeneric:{title:"通話に参加できませんでした。",dismissButtonAriaLabel:"閉じる"},failedToJoinCallInvalidMeetingLink:{title:"会議に参加できません。リンクが無効です。",dismissButtonAriaLabel:"閉じる"},cameraFrozenForRemoteParticipants:{title:"通話中のユーザーにビデオの表示で問題が発生しています",message:"デバイスとネットワークをチェックしてください。",dismissButtonAriaLabel:"閉じる"},unableToStartVideoEffect:{title:"ビデオ効果を適用できません。",dismissButtonAriaLabel:"閉じる"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"参加者にスポットライトを開始できません",message:"参加者の最大数は既にスポットライトが設定されています。",dismissButtonAriaLabel:"閉じる"},mutedByRemoteParticipant:{title:"他の参加者によってミュートにされました。",dismissButtonAriaLabel:"閉じる"},recordingStarted:{title:"レコーディングが開始しました。",message:"参加すると、この会議のトランスクリプトの作成に同意したことになります。",dismissButtonAriaLabel:"閉じる"},transcriptionStarted:{title:"文字起こしが開始しました。",message:"参加すると、この会議のトランスクリプトの作成に同意したことになります。",dismissButtonAriaLabel:"閉じる"},recordingStopped:{title:"レコーディングを保存しています。",message:"レコーディングが停止しました。",dismissButtonAriaLabel:"閉じる"},transcriptionStopped:{title:"文字起こしを保存しています。",message:"文字起こしが停止しました。",dismissButtonAriaLabel:"閉じる"},recordingAndTranscriptionStarted:{title:"レコーディングとトランスクリプトが開始されました。",message:"参加すると、この会議のトランスクリプトの作成に同意したことになります。",dismissButtonAriaLabel:"閉じる"},recordingAndTranscriptionStopped:{title:"レコーディングとトランスクリプトを保存されました。",message:"レコーディングとトランスクリプトを停止しました。",dismissButtonAriaLabel:"閉じる"},recordingStoppedStillTranscribing:{title:"レコーディングが停止しました。",message:"この会議のトランスクリプトを作成しています。",dismissButtonAriaLabel:"閉じる"},transcriptionStoppedStillRecording:{title:"文字起こしが停止しました。",message:"今はこの会議の録画のみを行っています。",dismissButtonAriaLabel:"閉じる"},assignedBreakoutRoomOpened:{title:"5 秒後に、割り当てられた会議室に移動します。",dismissButtonAriaLabel:"閉じる"},assignedBreakoutRoomChanged:{title:"ブレークアウト ルームが変更されました",message:"5 秒後に、割り当てられた会議室に移動します。",dismissButtonAriaLabel:"閉じる"},assignedBreakoutRoomClosed:{title:"ブレークアウト ルームが終了しました",message:"5 秒後に会議に戻ります。",dismissButtonAriaLabel:"閉じる"},assignedBreakoutRoomOpenedPromptJoin:{title:"ブレークアウト ルームに参加しますか?",message:"ブレークアウト ルームに割り当てられました。",dismissButtonAriaLabel:"閉じる",primaryButtonLabel:"ルームに参加",secondaryButtonLabel:"後で"},breakoutRoomJoined:{title:"ブレークアウト ルームに参加しました。",dismissButtonAriaLabel:"閉じる"},breakoutRoomClosingSoon:{title:"ルームの時間制限がもうすぐ切れます。",message:"このルームは 30 秒後に終了します",dismissButtonAriaLabel:"閉じる"},capabilityTurnVideoOnPresent:{title:"カメラが有効になっています",message:"ビデオを共有したい場合はオンにしてください。",dismissButtonAriaLabel:"閉じる"},capabilityTurnVideoOnAbsent:{title:"カメラが無効になっています",message:"ビデオを共有できなくなりました。",dismissButtonAriaLabel:"閉じる"},capabilityUnmuteMicPresent:{title:"マイクが有効になりました",message:"音声機能を使用する場合は、ミュートを解除します。",dismissButtonAriaLabel:"閉じる"},capabilityUnmuteMicAbsent:{title:"マイクが無効になっています",message:"ミュートを解除できなくなりました。",dismissButtonAriaLabel:"閉じる"},togetherModeStarted:{title:"Togethermode が開始しました",dismissButtonAriaLabel:"閉じる"},togetherModeEnded:{title:"Togethermode が終了しました",dismissButtonAriaLabel:"閉じる"},transcriptionError:{title:"文字起こしに問題が発生しました。",message:"文字起こしが停止しました。後でもう一度お試しください",dismissButtonAriaLabel:"閉じる"},transcriptionStartedByYou:{title:"文字起こしが開始しました。",message:"あなたが開始しました。文字起こしが行われていることを全員に知らせてください。",dismissButtonAriaLabel:"閉じる"}};var videoGallery$a={screenIsBeingSharedMessage:"画面を共有しています",screenShareLoadingMessage:"{participant} の画面を読み込んでいます",localScreenShareLoadingMessage:"画面を読み込んでいます",localVideoLabel:"自分",localVideoCameraSwitcherLabel:"カメラを切り替え",localVideoMovementLabel:"移動可能なローカル ビデオ タイル",localVideoMovementAriaLabel:"移動可能なローカル ビデオ タイル",localVideoSelectedDescription:"{cameraName} が選択されました。",displayNamePlaceholder:"名前のない参加者",fitRemoteParticipantToFrame:"フレームに収める",fillRemoteParticipantFrame:"トリミングする",pinParticipantForMe:"自分用にピン留め",pinParticipantForMeLimitReached:"ピン留め (上限に達しました)",unpinParticipantForMe:"ピンを外す",pinParticipantMenuItemAriaLabel:"{participantName} さんをピン留めする",unpinParticipantMenuItemAriaLabel:"{participantName} さんのピンを外す",pinnedParticipantAnnouncementAriaLabel:"{participantName} さんをピン留めしました",unpinnedParticipantAnnouncementAriaLabel:"{participantName} さんのピンを外しました",startSpotlightVideoTileMenuLabel:"全員にスポットライトを設定",addSpotlightVideoTileMenuLabel:"スポットライトを追加",spotlightLimitReachedMenuTitle:"スポットライトの上限に達しました",stopSpotlightVideoTileMenuLabel:"スポットライトの設定を解除",stopSpotlightOnSelfVideoTileMenuLabel:"スポットライトを終了",attendeeRole:"出席者",muteParticipantMenuItemLabel:"ミュート",waitingScreenText:"他のユーザーの参加を待っています",forbidAudio:"マイクを無効にする",permitAudio:"マイクを許可する",forbidAudioTileMenuLabel:"マイクを無効にする",permitAudioTileMenuLabel:"マイクを許可する",forbidVideo:"カメラを無効にする",permitVideo:"カメラを許可する",forbidVideoTileMenuLabel:"カメラを無効にする",permitVideoTileMenuLabel:"カメラを許可する"};var dialpad$a={placeholderText:"電話番号を入力してください",deleteButtonAriaLabel:"削除"};var holdButton$a={onLabel:"再開",offLabel:"保留",tooltipOnContent:"通話の再開",tooltipOffContent:"通話の保留"};var videoTile$a={participantStateRinging:"呼び出し中...",participantReconnecting:"再接続中...",participantStateHold:"保留中",moreOptionsButtonAriaLabel:"その他のオプションの {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"ミュート中",moreOptionsParticipantMutedStateUnmutedAriaLabel:"ミュート解除",moreOptionsParticipantHandRaisedAriaLabel:"挙手しました",moreOptionsParticipantIsSpeakingAriaLabel:"話し中",moreOptionsParticipantMicDisabledAriaLabel:"マイクが無効になっています",moreOptionsParticipantCameraDisabledAriaLabel:"カメラが無効になっています"};var CameraAndMicrophoneSitePermissionsRequest$a={primaryText:"{appName}にカメラとマイクの使用を許可する",secondaryText:"これは、参加者があなたのことを見たり聞いたりできるようにするものです。",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示",primaryButtonText:"カメラとマイクなしで続行する",ariaLabel:"カメラとマイクへのアクセスが許可する"};var CameraSitePermissionsRequest$a={primaryText:"{appName} にカメラの使用を許可する",secondaryText:"これは、参加者に表示されるようにするためです。",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示",primaryButtonText:"カメラなしで続行",ariaLabel:"カメラへのアクセスを許可"};var MicrophoneSitePermissionsRequest$a={primaryText:"{appName} にマイクの使用を許可する",secondaryText:"これは、参加者に聞こえるようにするためです。",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示",primaryButtonText:"マイクなしで続行",ariaLabel:"マイクへのアクセスを許可する"};var CameraAndMicrophoneSitePermissionsCheck$a={primaryText:"カメラとマイクへのアクセスを確認しています",secondaryText:"プロンプトが表示されたら、アクセスを許可します。これは、参加者があなたを見たり聞いたりできるようにするためです。",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示",primaryButtonText:"カメラとマイクなしで続行する",ariaLabel:"カメラとマイクへのアクセスを確認しています。プロンプトが表示されたら、アクセスを許可します。"};var CameraSitePermissionsCheck$a={primaryText:"カメラへのアクセスを確認しています",secondaryText:"プロンプトが表示されたら、アクセスを許可します。これは、参加者があなたを見ることができるようにするためです。",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示",primaryButtonText:"カメラなしで続行する",ariaLabel:"カメラへのアクセスを確認しています。プロンプトが表示されたら、アクセスを許可します。"};var MicrophoneSitePermissionsCheck$a={primaryText:"マイクへのアクセスを確認しています",secondaryText:"プロンプトが表示されたら、アクセスを許可します。これは、参加者があなたの声を聞くことができるようにするためです。",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示",primaryButtonText:"マイクなしで続行する",ariaLabel:"マイクへのアクセスを確認しています。プロンプトが表示されたら、アクセスを許可します。"};var CameraAndMicrophoneSitePermissionsDenied$a={primaryText:"カメラとマイクにアクセスできません",secondaryText:"アドレス バーのロック アイコンをクリックして、この Web ページにマイクのアクセス許可を付与します。ページの更新が必要な可能性があります。",primaryButtonText:"カメラとマイクなしで続行",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示"};var CameraAndMicrophoneSitePermissionsDeniedSafari$a={primaryText:"カメラとマイクにアクセスできません",secondaryText:"ページを更新してアクセス許可を可能にするか、このブラウザーの設定を確認し、この Web サイトのアクセス許可が有効になっていることを確認してください。",primaryButtonText:"カメラとマイクなしで続行",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示"};var CameraSitePermissionsDenied$a={primaryText:"カメラにアクセスできません",secondaryText:"アドレス バーのロック アイコンをクリックして、この Web ページにカメラのアクセス許可を付与します。ページの更新が必要な可能性があります。",primaryButtonText:"カメラなしで続行",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示"};var MicrophoneSitePermissionsDenied$a={primaryText:"マイクにアクセスできません",secondaryText:"アドレス バーのロック アイコンをクリックして、この Web ページにマイクのアクセス許可を付与します。ページの更新が必要な可能性があります。",primaryButtonText:"マイクなしで続行する",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示"};var CameraSitePermissionsDeniedSafari$a={primaryText:"カメラにアクセスできません",secondaryText:"ページを更新してアクセス許可を可能にするか、このブラウザーの設定を確認し、この Web サイトのアクセス許可が有効になっていることを確認してください。",primaryButtonText:"カメラなしで続行する",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示"};var MicrophoneSitePermissionsDeniedSafari$a={primaryText:"マイクにアクセスできません",secondaryText:"ページを更新してアクセス許可を可能にするか、このブラウザーの設定を確認し、この Web サイトのアクセス許可が有効になっていることを確認してください。",primaryButtonText:"マイクなしで続行",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示"};var UnsupportedBrowser$a={primaryText:"サポートされていないブラウザーです",secondaryText:"互換性のあるブラウザーを使用して、この通話に参加してください。",moreHelpLinkText:"互換性要件を確認する"};var UnsupportedBrowserVersion$a={primaryText:"ブラウザーの更新が必要です",secondaryText:"可能な限り最善の通話を行うために、ブラウザーを更新してから、もう一度通話に参加してみてください。",moreHelpLinkText:"互換性要件を確認する",continueAnywayButtonText:"更新せずに通話を開始する"};var UnsupportedOperatingSystem$a={primaryText:"オペレーティング システムがサポートされていません",secondaryText:"互換性のあるオペレーティング システムのデバイスを使用して、この通話に参加してください。",moreHelpLinkText:"互換性要件を確認する"};var BrowserPermissionDenied$a={primaryText:"カメラまたはマイクを使用できません",secondaryText:"お使いのブラウザーはカメラまたはマイクにアクセスできない可能性があります。この問題を解決するには、[システム環境設定] を開きます。",primaryButtonText:"再試行",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示"};var BrowserPermissionDeniedIOS$a={primaryText:"マイクへのアクセスを許可して続行する",secondaryText:"そのため、他の参加者はあなたの声を聞くことができます。",primaryButtonText:"再試行",imageAltText:"iOS のマイクとカメラ デバイスのアクセス許可の場所",linkText:"ヘルプが必要ですか? トラブルシューティングのヘルプを表示",step1Text:"設定アプリに移動します。",step2Text:"このブラウザーの設定まで下にスクロール",step3Text:"マイクをオンにする (カメラはオプション)",step4Text:"もう一度通話に参加してみてください",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$a={leftNavButtonAriaLabel:"前のページ {current}/{total}",rightNavButtonAriaLabel:"次のページ {current}/{total}"};var meetingConferencePhoneInfo$a={meetingConferencePhoneInfoModalTitle:"電話から通話に参加",meetingConferencePhoneInfoModalDialIn:"ダイヤル番号",meetingConferencePhoneInfoModalMeetingId:"会議 ID の入力",meetingConferencePhoneInfoModalWait:"開催者に許可されるのを待つ",meetingConferencePhoneInfoModalNoPhoneAvailable:"利用可能な電話がありません",meetingConferencePhoneInfoModalTollGeoData:"{country}、{city}",meetingConferencePhoneInfoModalTollFree:"(無料電話番号)",meetingConferencePhoneInfoModalToll:"(有料電話番号)"};var IncomingCallNotification$a={incomingCallNotificationPlaceholderId:"不明なユーザー",incomingCallNotificationPlaceholderAlert:"{callerName} が呼び出しています",incomingCallNoticicationAcceptWithAudioAriaLabel:"オーディオで承諾する",incomingCallNoticicationAcceptWithVideoAriaLabel:"ビデオで承諾する",incomingCallNoticicationRejectAriaLabel:"拒否する",incomingCallNotificationAcceptButtonLabel:"承諾",incomingCallNotificationRejectButtonLabel:"拒否",incomingCallNotificationAccceptWithVideoButtonLabel:"ビデオで受ける",incomingCallNotificationDismissButtonAriaLabel:"閉じる"};var ja_JP$1 = {participantItem:participantItem$a,ParticipantList:ParticipantList$a,typingIndicator:typingIndicator$a,sendBox:sendBox$a,richTextSendBox:richTextSendBox$a,realTimeTextModal:realTimeTextModal$a,captionsBanner:captionsBanner$a,mentionPopover:mentionPopover$a,imageOverlay:imageOverlay$a,messageStatusIndicator:messageStatusIndicator$a,endCallButton:endCallButton$a,cameraButton:cameraButton$a,microphoneButton:microphoneButton$a,devicesButton:devicesButton$a,participantsButton:participantsButton$a,screenShareButton:screenShareButton$a,raiseHandButton:raiseHandButton$a,reactionButton:reactionButton$a,realTimeText:realTimeText$a,captionsSettingsModal:captionsSettingsModal$a,spokenLanguages:spokenLanguages$a,captionLanguages:captionLanguages$a,startCaptionsButton:startCaptionsButton$a,startRealTimeTextButton:startRealTimeTextButton$a,messageThread:messageThread$a,errorBar:errorBar$a,notificationStack:notificationStack$a,videoGallery:videoGallery$a,dialpad:dialpad$a,holdButton:holdButton$a,videoTile:videoTile$a,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$a,CameraSitePermissionsRequest:CameraSitePermissionsRequest$a,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$a,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$a,CameraSitePermissionsCheck:CameraSitePermissionsCheck$a,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$a,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$a,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$a,CameraSitePermissionsDenied:CameraSitePermissionsDenied$a,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$a,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$a,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$a,UnsupportedBrowser:UnsupportedBrowser$a,UnsupportedBrowserVersion:UnsupportedBrowserVersion$a,UnsupportedOperatingSystem:UnsupportedOperatingSystem$a,BrowserPermissionDenied:BrowserPermissionDenied$a,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$a,verticalGallery:verticalGallery$a,meetingConferencePhoneInfo:meetingConferencePhoneInfo$a,IncomingCallNotification:IncomingCallNotification$a};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Japanese (Japan).
 *
 * @public
 */
const COMPONENT_LOCALE_JA_JP = {
    strings: createComponentStrings(ja_JP$1)
};

var participantItem$9={isMeText:"(나)",menuTitle:"추가 옵션",removeButtonLabel:"제거",sharingIconLabel:"공유 중",mutedIconLabel:"음소거됨",handRaisedIconLabel:"손들기 - 위치 {position}",displayNamePlaceholder:"명명되지 않은 참가자",participantStateRinging:"전화 거는 중...",participantStateHold:"대기 중",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, 기타 옵션",attendeeRole:"참석자",micDisabledIconLabel:"마이크를 사용할 수 없음",cameraDisabledIconLabel:"카메라 사용 안 함"};var ParticipantList$9={overflowParticipantCount:"+{overflowCount}개 더 보기"};var typingIndicator$9={singleUser:"{user} 님이 입력 중...",multipleUsers:"{users} 님이 입력하는 중...",multipleUsersAbbreviateOne:"{users} 님 외 1명이 입력 중...",multipleUsersAbbreviateMany:"{users} 님 외 {numOthers} 님이 입력하는 중...",delimiter:"닫는 렌즈형 흰색 괄호"};var sendBox$9={placeholderText:"메시지를 입력하세요",textTooLong:"메시지 길이가 최대 제한을 초과했습니다.",sendButtonAriaLabel:"메시지 보내기",attachmentUploadsPendingError:"파일을 업로드하는 중입니다. 잠시만 기다려 주세요.",removeAttachment:"파일 제거",uploading:"업로드 중",uploadCompleted:"업로드 완료",attachmentMoreMenu:"옵션 더 보기"};var richTextSendBox$9={placeholderText:"메시지 입력",textTooLong:"메시지 길이가 최대 제한을 초과했습니다.",sendButtonAriaLabel:"메시지 보내기",attachmentUploadsPendingError:"파일을 업로드하는 중입니다. 잠시만 기다려 주세요.",imageUploadsPendingError:"이미지를 업로드하는 중입니다. 잠시만 기다려 주세요.",removeAttachment:"파일 제거",uploading:"업로드 중",uploadCompleted:"업로드 완료",richTextBoldTooltip:"굵게",richTextItalicTooltip:"기울임꼴",richTextUnderlineTooltip:"밑줄",richTextBulletListTooltip:"글머리 기호 목록",richTextNumberListTooltip:"번호 매기기 목록",richTextIncreaseIndentTooltip:"들여쓰기",richTextDecreaseIndentTooltip:"내어쓰기",richTextInsertTableTooltip:"표 삽입",richTextFormatButtonTooltip:"형식",richTextInsertRowOrColumnMenu:"삽입",richTextInsertTableMenuTitle:"{column} x {row} 표 삽입",richTextInsertRowAboveMenu:"위에 삽입",richTextInsertRowBelowMenu:"아래에 삽입",richTextInsertColumnLeftMenu:"왼쪽에 삽입",richTextInsertColumnRightMenu:"오른쪽에 삽입",richTextDeleteRowOrColumnMenu:"삭제",richTextDeleteColumnMenu:"열 삭제",richTextDeleteRowMenu:"행 삭제",richTextDeleteTableMenu:"표 삭제",richTextToolbarAriaLabel:"메시지 서식 도구 모음",richTextToolbarMoreButtonAriaLabel:"옵션 더 보기",attachmentMoreMenu:"옵션 더 보기",richTextNewBulletedListItemAnnouncement:"글머리 기호, 새 줄",richTextNewNumberedListItemAnnouncement:"{0}, 새 줄",richTextBulletedListAppliedAnnouncement:"글머리 기호 목록 적용됨",richTextNumberedListAppliedAnnouncement:"번호 매기기 목록 적용됨"};var realTimeTextModal$9={realTimeTextModalAriaLabel:"실시간 입력 모달 활성화",realTimeTextModalTitle:"실시간 입력을 활성화하시겠습니까?",realTimeTextModalText:"모임의 모든 사용자에 대해 RTT(실시간 텍스트)가 켜집니다.",realTimeTextConfirmButtonLabel:"활성화",realTimeTextCancelButtonLabel:"취소",realTimeTextCloseModalButtonAriaLabel:"실시간 입력 모달 닫기"};var captionsBanner$9={captionsBannerSpinnerText:"캡션 시작 중...",realTimeTextInputBoxDefaultText:"실시간으로 메시지 입력",realTimeTextInputErrorMessage:"메시지는 2000자를 초과할 수 없음",realTimeTextBannerTitle:"실시간 입력",realTimeTextBannerContent:"전체 모임 기간 동안 모든 참가자에 대해 RTT(실시간 텍스트)를 사용할 수 있습니다.",realTimeTextBannerLinkLabel:"자세히 알아보기",captionsOnlyContainerTitle:"캡션",realTimeTextOnlyContainerTitle:"실시간 텍스트",captionsAndRealTimeTextContainerTitle:"캡션 및 실시간 텍스트",expandButtonAriaLabel:"펼치기",minimizeButtonAriaLabel:"접기"};var mentionPopover$9={mentionPopoverHeader:"제안"};var imageOverlay$9={downloadButtonLabel:"다운로드",dismissButtonAriaLabel:"닫기",overlayVisibleMessage:"이미지 세부 정보 오버레이"};var messageStatusIndicator$9={deliveredAriaLabel:"메시지를 보냄",deliveredTooltipText:"보냄",seenAriaLabel:"다른 사용자가 본 메시지",seenTooltipText:"표시됨",readByTooltipText:"{messageThreadReadCount}/{remoteParticipantsCount}명이 읽음",sendingAriaLabel:"메시지를 보내는 중",sendingTooltipText:"보내는 중",failedToSendAriaLabel:"메시지를 보내지 못함",failedToSendTooltipText:"보내기 실패"};var endCallButton$9={label:"휴가",tooltipContent:"통화에서 나가기",leaveOption:"나가기",endCallOption:"모든 사람의 통화 종료"};var cameraButton$9={onLabel:"카메라",offLabel:"카메라",tooltipDisabledContent:"카메라가 비활성화되었습니다",tooltipOnContent:"카메라 끄기",tooltipOffContent:"카메라 켜기",tooltipVideoLoadingContent:"비디오를 로드하는 중입니다.",cameraMenuTitle:"카메라",cameraMenuTooltip:"카메라 선택",cameraButtonSplitRoleDescription:"분할 단추",onSplitButtonAriaLabel:"카메라 끄기",offSplitButtonAriaLabel:"카메라 켜기",cameraActionTurnedOnAnnouncement:"카메라가 켜져 있습니다.",cameraActionTurnedOffAnnouncement:"카메라가 꺼져 있습니다.",onSplitButtonPrimaryActionCamera:"카메라 끄기",offSplitButtonPrimaryActionCamera:"카메라 켜기",cameraPrimaryActionSplitButtonTitle:"카메라 사용",videoEffectsMenuItemTitle:"효과",cameraButtonAriaDescription:"카메라 옵션"};var microphoneButton$9={onLabel:"마이크",offLabel:"마이크",tooltipDisabledContent:"마이크가 비활성화되었습니다",tooltipOnContent:"마이크 음소거",tooltipOffContent:"마이크 음소거 해제",microphoneMenuTitle:"마이크",microphoneMenuTooltip:"마이크 선택",speakerMenuTitle:"발언자",speakerMenuTooltip:"스피커 선택",microphoneButtonSplitRoleDescription:"분할 단추",onSplitButtonAriaLabel:"마이크 음소거",offSplitButtonAriaLabel:"마이크 음소거 해제",microphoneActionTurnedOnAnnouncement:"마이크가 켜져 있습니다.",microphoneActionTurnedOffAnnouncement:"마이크가 꺼져 있습니다.",offSplitButtonMicrophonePrimaryAction:"마이크 음소거 해제",onSplitButtonMicrophonePrimaryAction:"마이크 음소거",microphonePrimaryActionSplitButtonTitle:"마이크 사용",microphoneAriaDescription:"오디오 옵션",deepNoiseSuppressionTitle:"노이즈 억제",deepNoiseSuppressionOnAnnouncement:"심층 노이즈 억제가 켜졌습니다.",deepNoiseSuppressionOffAnnouncement:"심층 노이즈 억제가 꺼졌습니다."};var devicesButton$9={label:"장치",tooltipContent:"장치 관리",cameraMenuTitle:"카메라",cameraMenuTooltip:"카메라 선택",audioDeviceMenuTitle:"오디오 장치",audioDeviceMenuTooltip:"오디오 장치 선택",microphoneMenuTitle:"마이크",microphoneMenuTooltip:"마이크 선택",speakerMenuTitle:"발언자",speakerMenuTooltip:"스피커 선택"};var participantsButton$9={label:"사용자",ariaLabel:"사용자",tooltipContent:"참가자 표시",menuHeader:"이 통화에서",participantsListButtonLabel:"{numParticipants}명",muteAllButtonLabel:"모두 음소거",copyInviteLinkButtonLabel:"초대 링크 복사",copyInviteLinkActionedAriaLabel:"초대 링크를 복사함"};var screenShareButton$9={onLabel:"프레젠테이션 중지",offLabel:"프레젠테이션",tooltipDisabledContent:"프레젠테이션 사용 안 함",tooltipOnContent:"화면을 프레젠테이션하는 중",tooltipOffContent:"화면 프레젠테이션"};var raiseHandButton$9={offLabel:"손 들기",onLabel:"낮추기",tooltipDisabledContent:"손 올리기 작업이 비활성화되었습니다.",tooltipOnContent:"손 내리기",tooltipOffContent:"손 들기"};var reactionButton$9={label:"반응",ariaLabel:"반응",tooltipDisabledContent:"반응 동작을 사용할 수 없습니다.",tooltipContent:"반응 보내기",likeReactionTooltipContent:"좋아요",heartReactionTooltipContent:"사랑",laughReactionTooltipContent:"크게 웃는 표정",applauseReactionTooltipContent:"박수",surprisedReactionTooltipContent:"놀란 표정"};var realTimeText$9={isTypingText:"실시간 입력 입력 중"};var captionsSettingsModal$9={captionsSettingsModalTitle:"캡션 설정",captionsSettingsSpokenLanguageDropdownLabel:"음성 언어",captionsSettingsCaptionLanguageDropdownLabel:"캡션 언어",captionsSettingsSpokenLanguageDropdownInfoText:"이 통화의 모든 사람이 말하는 언어입니다.",captionsSettingsCaptionLanguageDropdownInfoText:"캡션이 이 언어로 표시됩니다.",captionsSettingsConfirmButtonLabel:"확인",captionsSettingsCancelButtonLabel:"취소",captionsSettingsModalAriaLabel:"캡션 설정 모달",captionsSettingsCloseModalButtonAriaLabel:"캡션 설정 닫기"};var spokenLanguages$9={"ar-ae":"아랍어 - 아랍에미리트","ar-sa":"아라비아어 - 사우디 아라비아","da-dk":"덴마크어","de-de":"독일어 - 독일","en-au":"영어 - 오스트레일리아","en-ca":"영어 - 캐나다","en-gb":"영어 - 영국","en-in":"영어 - 인도","en-nz":"영어 - 뉴질랜드","en-us":"영어 - 미국","es-es":"스페인어 - 스페인(현대 정렬)","es-mx":"스페인어 - 멕시코","fi-fi":"핀란드어","fr-ca":"프랑스어 - 캐나다","fr-fr":"프랑스어 - 프랑스","hi-in":"힌디어","it-it":"이탈리아어 - 이탈리아","ja-jp":"일본어","ko-kr":"한국어","nb-no":"노르웨이어(보크몰)","nl-be":"네덜란드어 - 벨기에","nl-nl":"네덜란드어 - 네덜란드","pl-pl":"폴란드어","pt-br":"포르투갈어 - 브라질","ru-ru":"러시아어","sv-se":"스웨덴어","zh-cn":"중국어 - 중국","zh-hk":"중국어 - 홍콩 특별 행정구","cs-cz":"체코어","pt-pt":"포르투갈어 - 포르투갈","tr-tr":"터키어","vi-vn":"베트남어","th-th":"태국어","he-il":"히브리어","cy-gb":"웨일스어","uk-ua":"우크라이나어","el-gr":"그리스어","hu-hu":"헝가리어","ro-ro":"루마니아어","sk-sk":"슬로바키아어","zh-tw":"중국어 - 대만"};var captionLanguages$9={ar:"아랍어",da:"덴마크어",de:"독일어",en:"영어",es:"스페인어",fi:"핀란드어","fr-ca":"프랑스어 - 캐나다",fr:"프랑스어 - 프랑스",hi:"힌디어",it:"이탈리아어",ja:"일본어",ko:"한국어",nb:"노르웨이어(보크몰)",nl:"네덜란드어",pl:"폴란드어",pt:"포르투갈어 - 브라질",ru:"러시아어",sv:"스웨덴어","zh-Hans":"중국어(간체)","zh-Hant":"중국어(번체)",cs:"체코어","pt-pt":"포르투갈어 - 포르투갈",tr:"터키어",vi:"베트남어",th:"태국어",he:"히브리어",cy:"웨일스어",uk:"우크라이나어",el:"그리스어",hu:"헝가리어",ro:"루마니아어",sk:"슬로바키아어"};var startCaptionsButton$9={onLabel:"캡션 켜기",offLabel:"캡션 끄기",tooltipOnContent:"캡션 끄기",tooltipOffContent:"캡션 켜기"};var startRealTimeTextButton$9={label:"이 통화에 대한 RTT 켜기",tooltipOffContent:"이 통화에 대한 RTT 켜기"};var messageThread$9={yesterday:"어제",sunday:"일요일",monday:"월요일",tuesday:"화요일",wednesday:"수요일",thursday:"목요일",friday:"금요일",saturday:"토요일",participantJoined:"채팅에 참여했습니다.",participantLeft:"이 채팅을 나갔습니다.",editMessage:"편집",removeMessage:"삭제",resendMessage:"전송 다시 시도",failToSendTag:"보내기 실패",editedTag:"편집됨",liveAuthorIntro:"{author}의 메시지",editedMessageLiveAuthorIntro:"{author}의 편집한 메시지",editedMessageLocalUserLiveAuthorIntro:"편집한 메시지",messageContentAriaText:"{status} {author} 님의 메시지: {message}",messageContentMineAriaText:"{status} 내 메시지: {message}",editBoxTextLimit:"메시지가 {limitNumber} 문자 제한을 초과했습니다.",editBoxPlaceholderText:"메시지 편집",newMessagesIndicator:"새 메시지",noDisplayNameSub:"이름 없음",editBoxCancelButton:"취소",editBoxSubmitButton:"완료",messageReadCount:"{messageReadByCount}/{remoteParticipantsCount}명이 읽음",actionMenuMoreOptions:"추가 옵션",downloadAttachment:"파일 다운로드",openAttachment:"파일 열기",blockedWarningText:"조직 정책으로 인해 이 메시지가 삭제되었습니다.",blockedWarningLinkText:"세부 정보",attachmentCardGroupMessage:"메시지에 {attachmentCount}개의 첨부 파일이 있습니다.",messageDeletedAnnouncementAriaLabel:"메시지가 삭제되었습니다.",richTextBoldTooltip:"굵게",richTextItalicTooltip:"기울임꼴",richTextUnderlineTooltip:"밑줄",richTextBulletListTooltip:"글머리 기호 목록",richTextNumberListTooltip:"번호 매기기 목록",richTextIncreaseIndentTooltip:"들여쓰기",richTextDecreaseIndentTooltip:"내어쓰기",richTextInsertTableTooltip:"표 삽입",richTextFormatButtonTooltip:"형식",richTextInsertRowOrColumnMenu:"삽입",richTextInsertTableMenuTitle:"{column} x {row} 표 삽입",richTextInsertRowAboveMenu:"위에 삽입",richTextInsertRowBelowMenu:"아래에 삽입",richTextInsertColumnLeftMenu:"왼쪽에 삽입",richTextInsertColumnRightMenu:"오른쪽에 삽입",richTextDeleteRowOrColumnMenu:"삭제",richTextDeleteColumnMenu:"열 삭제",richTextDeleteRowMenu:"행 삭제",richTextDeleteTableMenu:"표 삭제",richTextToolbarMoreButtonAriaLabel:"옵션 더 보기",attachmentUploadsPendingError:"파일을 업로드하는 중입니다. 잠시만 기다려 주세요.",imageUploadsPendingError:"이미지를 업로드하는 중입니다. 잠시만 기다려 주세요.",richTextNewBulletedListItemAnnouncement:"글머리 기호, 새 줄",richTextNewNumberedListItemAnnouncement:"{0}, 새 줄",richTextBulletedListAppliedAnnouncement:"글머리 기호 목록 적용됨",richTextNumberedListAppliedAnnouncement:"번호 매기기 목록 적용됨"};var errorBar$9={unableToReachChatService:"오프라인 상태입니다.",accessDenied:"채팅 서비스에 액세스할 수 없습니다. 제공된 사용자 자격 증명을 확인하세요.",userNotInChatThread:"이 채팅 스레드에 더 이상 없습니다.",sendMessageNotInChatThread:"이 채팅 스레드에 더 이상 존재하지 않으므로 메시지를 보내지 못했습니다.",sendMessageGeneric:"메시지를 보내지 못했습니다",callingNetworkFailure:"통화 연결 중 - 오프라인 상태인 것 같습니다.",startVideoGeneric:"비디오를 시작하지 못함",stopVideoGeneric:"비디오를 중지하지 못함",muteGeneric:"마이크를 음소거하지 못함",unmuteGeneric:"마이크 음소거를 해제하지 못함",speakingWhileMuted:"마이크가 음소거됨",startScreenShareGeneric:"화면 공유를 시작하는 동안 문제가 발생했습니다.",stopScreenShareGeneric:"화면 공유를 중지하지 못함",callNetworkQualityLow:"네트워크 품질이 낮습니다.",teamsMeetingCallNetworkQualityLow:"네트워크 품질이 낮습니다.",callNoSpeakerFound:"스피커 또는 헤드폰을 찾을 수 없습니다. 오디오 장치를 연결하여 통화를 들을 수 있습니다.",callNoMicrophoneFound:"마이크를 찾을 수 없습니다. 오디오 입력 장치를 연결합니다.",callMicrophoneAccessDenied:"마이크에 액세스할 수 없습니다. 주소 표시줄에서 잠금을 클릭하여 이 웹 페이지에 권한을 부여합니다.",callMicrophoneAccessDeniedSafari:"마이크에 액세스할 수 없습니다. 사용 권한을 허용하려면 페이지를 새로 고치거나 이 브라우저의 설정을 확인하고 이 웹 사이트에 사용 권한이 설정되어 있는지 확인하세요.",callMicrophoneMutedBySystem:"시스템에 의해 음소거되었습니다.",callMicrophoneUnmutedBySystem:"마이크가 복구되었으며 시스템에서 음소거를 해제했습니다.",callMacOsMicrophoneAccessDenied:"마이크에 액세스할 수 없습니다. macOS 개인 정보 설정에서 마이크 권한을 부여합니다.",callLocalVideoFreeze:"네트워크 대역폭이 불량합니다. 통화 중에 다른 사람에 대해 비디오가 일시 중지된 것 같습니다.",callCameraAccessDenied:"카메라에 액세스할 수 없습니다. 주소 표시줄에서 잠금을 클릭하여 이 웹 페이지에 권한을 부여합니다.",callCameraAccessDeniedSafari:"카메라에 액세스할 수 없습니다. 사용 권한을 허용하려면 페이지를 새로 고치거나 이 브라우저의 설정을 확인하고 이 웹 사이트에 사용 권한이 설정되어 있는지 확인하세요.",callCameraAlreadyInUse:"카메라에 액세스할 수 없습니다. 다른 응용 프로그램에서 이미 사용 중일 수 있습니다.",callVideoStoppedBySystem:"시스템에 의해 비디오가 중지되었습니다.",callVideoRecoveredBySystem:"비디오가 다시 시작되었습니다.",callMacOsCameraAccessDenied:"MacOS에서 카메라에 대한 액세스를 차단하고 있습니다. 이 브라우저에서 카메라에 액세스할 수 있도록 개인 정보 설정을 업데이트합니다.",callMacOsScreenShareAccessDenied:"MacOS에서 화면 공유를 차단하고 있습니다. 이 브라우저에서 화면을 녹화할 수 있도록 개인 정보 설정을 업데이트합니다.",dismissButtonAriaLabel:"닫기",failedToJoinCallGeneric:"통화에 참가하지 못했습니다.",failedToJoinCallInvalidMeetingLink:"모임에 참가할 수 없습니다. 잘못된 링크입니다.",cameraFrozenForRemoteParticipants:"통화 중인 사용자에게 영상이 표시되는 데 문제가 있습니다. 장치와 네트워크를 확인하세요.",unableToStartVideoEffect:"비디오 효과를 적용할 수 없습니다.",startSpotlightWhileMaxParticipantsAreSpotlighted:"최대 참가자 수가 이미 스포트라이트되었으므로 참가자에서 스포트라이트를 시작할 수 없습니다.",mutedByRemoteParticipant:"다른 참가자가 나를 음소거했습니다."};var notificationStack$9={callingNetworkFailure:{title:"통화 연결 문제 발생",message:"오프라인 상태인 것 같습니다. 네트워크 연결을 확인하고 다시 시도하세요.",dismissButtonAriaLabel:"닫기"},startVideoGeneric:{title:"비디오를 시작하지 못했습니다.",dismissButtonAriaLabel:"닫기"},stopVideoGeneric:{title:"비디오를 중지하지 못했습니다.",dismissButtonAriaLabel:"닫기"},muteGeneric:{title:"마이크를 음소거하지 못했습니다.",dismissButtonAriaLabel:"닫기"},unmuteGeneric:{title:"마이크의 음소거를 해제하지 못했습니다.",dismissButtonAriaLabel:"닫기"},speakingWhileMuted:{title:"음소거했습니다.",dismissButtonAriaLabel:"닫기"},startScreenShareGeneric:{title:"화면 공유를 시작하는 동안 문제가 발생했습니다.",dismissButtonAriaLabel:"닫기"},stopScreenShareGeneric:{title:"화면 공유를 중지하지 못했습니다.",dismissButtonAriaLabel:"닫기"},callNetworkQualityLow:{title:"네트워크 품질이 낮습니다.",dismissButtonAriaLabel:"닫기"},teamsMeetingCallNetworkQualityLow:{title:"네트워크 품질이 낮습니다.",dismissButtonAriaLabel:"닫기",message:"더 나은 사운드를 위해 휴대폰에서 이 통화에 참여하세요. 이 장치에서 모임을 계속 볼 수 있습니다.",primaryButtonLabel:"전화로 참가"},callNoSpeakerFound:{title:"스피커 또는 헤드폰을 찾을 수 없음",message:"오디오 장치를 연결하여 통화를 들을 수 있습니다.",dismissButtonAriaLabel:"닫기"},callNoMicrophoneFound:{title:"마이크를 찾을 수 없습니다.",message:"오디오 입력 장치를 연결합니다.",dismissButtonAriaLabel:"닫기"},callMicrophoneAccessDenied:{title:"마이크에 액세스할 수 없음",message:"주소 표시줄에서 잠금을 클릭하여 이 웹 페이지에 권한을 부여합니다.",dismissButtonAriaLabel:"닫기"},callMicrophoneAccessDeniedSafari:{title:"마이크에 액세스할 수 없음",message:"사용 권한을 허용하려면 페이지를 새로 고치거나 이 브라우저의 설정을 확인하고 이 웹 사이트에 사용 권한이 설정되어 있는지 확인하세요.",dismissButtonAriaLabel:"닫기"},callMicrophoneMutedBySystem:{title:"시스템에 의해 음소거되었습니다.",dismissButtonAriaLabel:"닫기"},callMicrophoneUnmutedBySystem:{title:"마이크가 복구되었으며 시스템에서 음소거를 해제했습니다.",dismissButtonAriaLabel:"닫기"},callMacOsMicrophoneAccessDenied:{title:"마이크에 액세스할 수 없음",message:"macOS 개인 정보 설정에서 마이크 권한을 부여합니다.",dismissButtonAriaLabel:"닫기"},callLocalVideoFreeze:{title:"네트워크 대역폭이 나쁨",message:"통화 중에 다른 사람에 대해 비디오가 일시 중지된 것 같습니다.",dismissButtonAriaLabel:"닫기"},callCameraAccessDenied:{title:"카메라에 액세스할 수 없음",message:"주소 표시줄에서 잠금을 클릭하여 이 웹 페이지에 권한을 부여합니다.",dismissButtonAriaLabel:"닫기"},callCameraAccessDeniedSafari:{title:"카메라에 액세스할 수 없음",message:"사용 권한을 허용하려면 페이지를 새로 고치거나 이 브라우저의 설정을 확인하고 이 웹 사이트에 사용 권한이 설정되어 있는지 확인하세요.",dismissButtonAriaLabel:"닫기"},callCameraAlreadyInUse:{title:"카메라에 액세스할 수 없음",message:"다른 응용 프로그램에서 이미 사용 중일 수 있습니다.",dismissButtonAriaLabel:"닫기"},callVideoStoppedBySystem:{title:"시스템에 의해 비디오가 중지되었습니다.",dismissButtonAriaLabel:"닫기"},callVideoRecoveredBySystem:{title:"비디오를 다시 시작했습니다.",dismissButtonAriaLabel:"닫기"},callMacOsCameraAccessDenied:{title:"MacOS가 카메라에 대한 액세스를 차단하고 있습니다.",message:"이 브라우저에서 카메라에 액세스할 수 있도록 개인 정보 설정을 업데이트합니다.",dismissButtonAriaLabel:"닫기"},callMacOsScreenShareAccessDenied:{title:"MacOS에서 화면 공유를 차단하고 있습니다.",message:"이 브라우저에서 화면을 녹화할 수 있도록 개인 정보 설정을 업데이트합니다.",dismissButtonAriaLabel:"닫기"},failedToJoinCallGeneric:{title:"통화에 참가하지 못했습니다.",dismissButtonAriaLabel:"닫기"},failedToJoinCallInvalidMeetingLink:{title:"모임에 참가할 수 없습니다. 잘못된 링크입니다.",dismissButtonAriaLabel:"닫기"},cameraFrozenForRemoteParticipants:{title:"통화 중인 사용자에게 영상이 표시되는 데 문제가 있습니다.",message:"장치 및 네트워크를 확인하세요.",dismissButtonAriaLabel:"닫기"},unableToStartVideoEffect:{title:"비디오 효과를 적용할 수 없습니다.",dismissButtonAriaLabel:"닫기"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"참가자에 대한 스포트라이트를 시작할 수 없음",message:"최대 참가자 수가 이미 스포트라이트되었습니다.",dismissButtonAriaLabel:"닫기"},mutedByRemoteParticipant:{title:"다른 참가자가 나를 음소거했습니다.",dismissButtonAriaLabel:"닫기"},recordingStarted:{title:"녹음/녹화가 시작되었습니다.",message:"참가하면 이 모임이 전사되는 것에 동의하게 됩니다.",dismissButtonAriaLabel:"닫기"},transcriptionStarted:{title:"전사를 시작했습니다.",message:"참가하면 이 모임이 전사되는 것에 동의하게 됩니다.",dismissButtonAriaLabel:"닫기"},recordingStopped:{title:"녹음/녹화를 저장하는 중입니다.",message:"녹음/녹화를 중단했습니다.",dismissButtonAriaLabel:"닫기"},transcriptionStopped:{title:"대화 기록을 저장하는 중입니다.",message:"대화 내용 기록을 중단했습니다.",dismissButtonAriaLabel:"닫기"},recordingAndTranscriptionStarted:{title:"녹음/녹화와 전사를 시작했습니다.",message:"참가하면 이 모임이 전사되는 것에 동의하게 됩니다.",dismissButtonAriaLabel:"닫기"},recordingAndTranscriptionStopped:{title:"녹음/녹화와 전사를 저장하는 중입니다.",message:"녹음/녹화와 전사를 중단했습니다.",dismissButtonAriaLabel:"닫기"},recordingStoppedStillTranscribing:{title:"녹음/녹화를 중단했습니다.",message:"이제 이 모임의 전사만 진행합니다.",dismissButtonAriaLabel:"닫기"},transcriptionStoppedStillRecording:{title:"대화 내용 기록을 중단했습니다.",message:"이제 이 모임의 녹음/녹화만 진행합니다.",dismissButtonAriaLabel:"닫기"},assignedBreakoutRoomOpened:{title:"5초 후에 할당된 채팅방으로 이동합니다.",dismissButtonAriaLabel:"닫기"},assignedBreakoutRoomChanged:{title:"소규모 회의실이 변경되었습니다.",message:"5초 후에 할당된 채팅방으로 이동합니다.",dismissButtonAriaLabel:"닫기"},assignedBreakoutRoomClosed:{title:"소규모 회의실 종료됨",message:"5초 후에 모임으로 돌아갑니다.",dismissButtonAriaLabel:"닫기"},assignedBreakoutRoomOpenedPromptJoin:{title:"소규모 회의실에 참가하시겠어요?",message:"사용자가 소규모 회의실에 배정되었습니다.",dismissButtonAriaLabel:"닫기",primaryButtonLabel:"소회의실 참가",secondaryButtonLabel:"나중에"},breakoutRoomJoined:{title:"소회의실에 참가했습니다.",dismissButtonAriaLabel:"닫기"},breakoutRoomClosingSoon:{title:"소회의실 시간 제한이 곧 만료됩니다.",message:"이 소회의실은 30초 후에 닫힙니다.",dismissButtonAriaLabel:"닫기"},capabilityTurnVideoOnPresent:{title:"카메라가 사용하도록 설정되었습니다",message:"비디오를 공유하고자 하는 경우 이 기능을 켜세요.",dismissButtonAriaLabel:"닫기"},capabilityTurnVideoOnAbsent:{title:"카메라를 사용하지 않습니다",message:"더 이상 비디오를 공유할 수 없습니다.",dismissButtonAriaLabel:"닫기"},capabilityUnmuteMicPresent:{title:"마이크를 사용하도록 설정했습니다",message:"말하기를 원하는 경우 음소거를 해제하세요.",dismissButtonAriaLabel:"닫기"},capabilityUnmuteMicAbsent:{title:"마이크를 사용할 수 없습니다.",message:"더 이상 음소거 해제할 수 없음",dismissButtonAriaLabel:"닫기"},togetherModeStarted:{title:"Togethermode가 시작되었습니다.",dismissButtonAriaLabel:"닫기"},togetherModeEnded:{title:"Togethermode가 종료되었습니다.",dismissButtonAriaLabel:"닫기"},transcriptionError:{title:"대화 내용 기록에 문제가 발생했습니다.",message:"대화 내용 기록이 중단되었습니다. 나중에 다시 시도하세요.",dismissButtonAriaLabel:"닫기"},transcriptionStartedByYou:{title:"대화 내용 기록을 시작했습니다.",message:"사용자가 시작했습니다. 모든 사람들에게 대화가 기록되고 있다는 것을 알리세요.",dismissButtonAriaLabel:"닫기"}};var videoGallery$9={screenIsBeingSharedMessage:"화면을 공유 중입니다.",screenShareLoadingMessage:"{participant} 님의 화면 로드 중",localScreenShareLoadingMessage:"화면 로드 중",localVideoLabel:"나",localVideoCameraSwitcherLabel:"카메라 전환",localVideoMovementLabel:"이동 가능한 로컬 비디오 타일",localVideoMovementAriaLabel:"이동 가능한 로컬 비디오 타일",localVideoSelectedDescription:"{cameraName} 선택함",displayNamePlaceholder:"명명되지 않은 참가자",fitRemoteParticipantToFrame:"프레임에 맞춤",fillRemoteParticipantFrame:"프레임 채우기",pinParticipantForMe:"고정",pinParticipantForMeLimitReached:"핀 제한에 도달함",unpinParticipantForMe:"고정 해제",pinParticipantMenuItemAriaLabel:"{participantName} 고정",unpinParticipantMenuItemAriaLabel:"{participantName} 고정 해제",pinnedParticipantAnnouncementAriaLabel:"{participantName} 고정함",unpinnedParticipantAnnouncementAriaLabel:"{participantName} 고정 해제함",startSpotlightVideoTileMenuLabel:"모두를 위한 스포트라이트",addSpotlightVideoTileMenuLabel:"스포트라이트 추가",spotlightLimitReachedMenuTitle:"스포트라이트 한도 도달",stopSpotlightVideoTileMenuLabel:"스포트라이트 중지",stopSpotlightOnSelfVideoTileMenuLabel:"스포트라이트 종료",attendeeRole:"참석자",muteParticipantMenuItemLabel:"음소거",waitingScreenText:"다른 사용자의 참여를 기다리는 중",forbidAudio:"마이크 비활성화",permitAudio:"마이크 허용",forbidAudioTileMenuLabel:"마이크 비활성화",permitAudioTileMenuLabel:"마이크 허용",forbidVideo:"카메라 비활성화",permitVideo:"카메라 허용",forbidVideoTileMenuLabel:"카메라 비활성화",permitVideoTileMenuLabel:"카메라 허용"};var dialpad$9={placeholderText:"전화 번호를 입력하세요.",deleteButtonAriaLabel:"삭제"};var holdButton$9={onLabel:"계속",offLabel:"보류",tooltipOnContent:"통화 계속하기",tooltipOffContent:"통화 대기"};var videoTile$9={participantStateRinging:"전화 거는 중...",participantReconnecting:"다시 연결하는 중...",participantStateHold:"대기 중",moreOptionsButtonAriaLabel:"기타 옵션 {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"음소거됨",moreOptionsParticipantMutedStateUnmutedAriaLabel:"음소거 해제됨",moreOptionsParticipantHandRaisedAriaLabel:"손 들기",moreOptionsParticipantIsSpeakingAriaLabel:"말하기",moreOptionsParticipantMicDisabledAriaLabel:"마이크를 사용할 수 없음",moreOptionsParticipantCameraDisabledAriaLabel:"카메라 사용 안 함"};var CameraAndMicrophoneSitePermissionsRequest$9={primaryText:"{appName} 님이 카메라와 마이크를 사용하도록 허용",secondaryText:"참가자가 사용자를 보고 들을 수 있도록 합니다.",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기",primaryButtonText:"카메라 및 마이크 없이 계속",ariaLabel:"카메라 및 마이크 액세스 허용"};var CameraSitePermissionsRequest$9={primaryText:"{appName}에서 카메라를 사용하도록 허용",secondaryText:"참가자가 나를 볼 수 있도록 합니다.",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기",primaryButtonText:"카메라 없이 계속",ariaLabel:"카메라 액세스 허용"};var MicrophoneSitePermissionsRequest$9={primaryText:"{appName}에서 마이크를 사용하도록 허용",secondaryText:"참가자가 내 목소리를 들을 수 있도록 합니다.",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기",primaryButtonText:"마이크 없이 계속",ariaLabel:"마이크 액세스 허용"};var CameraAndMicrophoneSitePermissionsCheck$9={primaryText:"카메라 및 마이크 액세스 확인 중",secondaryText:"메시지가 표시되면 액세스를 허용합니다. 참가자가 사용자를 보고 들을 수 있도록 합니다.",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기",primaryButtonText:"카메라 및 마이크 없이 계속",ariaLabel:"카메라 및 마이크 액세스를 확인하는 중입니다. 메시지가 표시되면 액세스를 허용하세요."};var CameraSitePermissionsCheck$9={primaryText:"카메라 액세스 확인 중",secondaryText:"메시지가 표시되면 액세스를 허용합니다. 참가자가 사용자를 볼 수 있도록 합니다.",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기",primaryButtonText:"카메라 없이 계속",ariaLabel:"카메라 액세스를 확인하는 중입니다. 메시지가 표시되면 액세스를 허용하세요."};var MicrophoneSitePermissionsCheck$9={primaryText:"마이크 액세스 확인 중",secondaryText:"메시지가 표시되면 액세스를 허용합니다. 참가자가 들을 수 있도록 합니다.",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기",primaryButtonText:"마이크 없이 계속",ariaLabel:"마이크 액세스를 확인하는 중입니다. 메시지가 표시되면 액세스를 허용하세요."};var CameraAndMicrophoneSitePermissionsDenied$9={primaryText:"카메라 및 마이크에 액세스할 수 없음",secondaryText:"주소 표시줄에서 잠금 아이콘을 클릭하여 이 웹 페이지에 마이크 권한을 부여합니다. 페이지를 새로 고쳐야 할 수 있습니다.",primaryButtonText:"카메라 및 마이크 없이 계속",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기"};var CameraAndMicrophoneSitePermissionsDeniedSafari$9={primaryText:"카메라 및 마이크에 액세스할 수 없음",secondaryText:"사용 권한을 허용하려면 페이지를 새로 고치거나 이 브라우저의 설정을 확인하고 이 웹 사이트에 사용 권한이 설정되어 있는지 확인하세요.",primaryButtonText:"카메라 및 마이크 없이 계속",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기"};var CameraSitePermissionsDenied$9={primaryText:"카메라에 액세스할 수 없음",secondaryText:"주소 표시줄에서 잠금 아이콘을 클릭하여 이 웹 페이지에 카메라 권한을 부여합니다. 페이지를 새로 고쳐야 할 수 있습니다.",primaryButtonText:"카메라 없이 계속",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기"};var MicrophoneSitePermissionsDenied$9={primaryText:"마이크에 액세스할 수 없음",secondaryText:"주소 표시줄에서 잠금 아이콘을 클릭하여 이 웹 페이지에 마이크 권한을 부여합니다. 페이지를 새로 고쳐야 할 수 있습니다.",primaryButtonText:"마이크 없이 계속",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기"};var CameraSitePermissionsDeniedSafari$9={primaryText:"카메라에 액세스할 수 없음",secondaryText:"사용 권한을 허용하려면 페이지를 새로 고치거나 이 브라우저의 설정을 확인하고 이 웹 사이트에 사용 권한이 설정되어 있는지 확인하세요.",primaryButtonText:"카메라 없이 계속",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기"};var MicrophoneSitePermissionsDeniedSafari$9={primaryText:"마이크에 액세스할 수 없음",secondaryText:"사용 권한을 허용하려면 페이지를 새로 고치거나 이 브라우저의 설정을 확인하고 이 웹 사이트에 사용 권한이 설정되어 있는지 확인하세요.",primaryButtonText:"마이크 없이 계속",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기"};var UnsupportedBrowser$9={primaryText:"지원되지 않는 브라우저",secondaryText:"호환되는 브라우저를 사용하여 이 통화에 참여하세요.",moreHelpLinkText:"호환성 요구 사항 보기"};var UnsupportedBrowserVersion$9={primaryText:"브라우저 업데이트 필요",secondaryText:"최상의 통화를 하려면 브라우저를 업데이트한 다음 통화에 다시 참가해 보세요.",moreHelpLinkText:"호환성 요구 사항 보기",continueAnywayButtonText:"업데이트하지 않고 통화 시작"};var UnsupportedOperatingSystem$9={primaryText:"지원되지 않는 운영 체제",secondaryText:"호환되는 운영 체제가 있는 장치를 사용하여 이 통화에 참여하세요.",moreHelpLinkText:"호환성 요구 사항 보기"};var BrowserPermissionDenied$9={primaryText:"카메라 또는 마이크를 사용할 수 없음",secondaryText:"브라우저에서 카메라 또는 마이크에 액세스할 수 없을 수 있습니다. 이 문제를 해결하려면 시스템 기본 설정을 엽니다.",primaryButtonText:"다시 시도",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기"};var BrowserPermissionDeniedIOS$9={primaryText:"계속하려면 마이크 액세스 허용",secondaryText:"따라서 다른 참가자가 여러분의 의견을 들을 수 있습니다.",primaryButtonText:"다시 시도",imageAltText:"iOS용 마이크 및 카메라 장치 권한 위치",linkText:"도움이 필요하신가요? 문제 해결 도움말 보기",step1Text:"설정 앱으로 이동합니다.",step2Text:"이 브라우저의 설정에 Scroll down",step3Text:"마이크 켜기(카메라 선택 사항)",step4Text:"통화에 다시 참가해 보세요.",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$9={leftNavButtonAriaLabel:"이전 페이지 {current}/{total}",rightNavButtonAriaLabel:"다음 페이지 {current}/{total}"};var meetingConferencePhoneInfo$9={meetingConferencePhoneInfoModalTitle:"내 전화에서 통화 참가",meetingConferencePhoneInfoModalDialIn:"전화 걸기",meetingConferencePhoneInfoModalMeetingId:"모임 ID 입력",meetingConferencePhoneInfoModalWait:"이끌이가 허가할 때까지 기다리세요.",meetingConferencePhoneInfoModalNoPhoneAvailable:"사용할 수 있는 전화 없음",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(무료)",meetingConferencePhoneInfoModalToll:"(유료)"};var IncomingCallNotification$9={incomingCallNotificationPlaceholderId:"알 수 없는 사용자",incomingCallNotificationPlaceholderAlert:"{callerName}이(가) 호출 중입니다.",incomingCallNoticicationAcceptWithAudioAriaLabel:"오디오 수락",incomingCallNoticicationAcceptWithVideoAriaLabel:"비디오 수락",incomingCallNoticicationRejectAriaLabel:"거부",incomingCallNotificationAcceptButtonLabel:"수락",incomingCallNotificationRejectButtonLabel:"거절",incomingCallNotificationAccceptWithVideoButtonLabel:"영상 수락",incomingCallNotificationDismissButtonAriaLabel:"해제"};var ko_KR$1 = {participantItem:participantItem$9,ParticipantList:ParticipantList$9,typingIndicator:typingIndicator$9,sendBox:sendBox$9,richTextSendBox:richTextSendBox$9,realTimeTextModal:realTimeTextModal$9,captionsBanner:captionsBanner$9,mentionPopover:mentionPopover$9,imageOverlay:imageOverlay$9,messageStatusIndicator:messageStatusIndicator$9,endCallButton:endCallButton$9,cameraButton:cameraButton$9,microphoneButton:microphoneButton$9,devicesButton:devicesButton$9,participantsButton:participantsButton$9,screenShareButton:screenShareButton$9,raiseHandButton:raiseHandButton$9,reactionButton:reactionButton$9,realTimeText:realTimeText$9,captionsSettingsModal:captionsSettingsModal$9,spokenLanguages:spokenLanguages$9,captionLanguages:captionLanguages$9,startCaptionsButton:startCaptionsButton$9,startRealTimeTextButton:startRealTimeTextButton$9,messageThread:messageThread$9,errorBar:errorBar$9,notificationStack:notificationStack$9,videoGallery:videoGallery$9,dialpad:dialpad$9,holdButton:holdButton$9,videoTile:videoTile$9,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$9,CameraSitePermissionsRequest:CameraSitePermissionsRequest$9,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$9,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$9,CameraSitePermissionsCheck:CameraSitePermissionsCheck$9,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$9,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$9,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$9,CameraSitePermissionsDenied:CameraSitePermissionsDenied$9,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$9,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$9,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$9,UnsupportedBrowser:UnsupportedBrowser$9,UnsupportedBrowserVersion:UnsupportedBrowserVersion$9,UnsupportedOperatingSystem:UnsupportedOperatingSystem$9,BrowserPermissionDenied:BrowserPermissionDenied$9,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$9,verticalGallery:verticalGallery$9,meetingConferencePhoneInfo:meetingConferencePhoneInfo$9,IncomingCallNotification:IncomingCallNotification$9};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Korean (South Korea).
 *
 * @public
 */
const COMPONENT_LOCALE_KO_KR = {
    strings: createComponentStrings(ko_KR$1)
};

var participantItem$8={isMeText:"(deg)",menuTitle:"Flere alternativer",removeButtonLabel:"Fjern",sharingIconLabel:"Deling",mutedIconLabel:"Dempet",handRaisedIconLabel:"Hevet hånd – posisjon {position}",displayNamePlaceholder:"Deltaker uten navn",participantStateRinging:"Ringer ...",participantStateHold:"På vent",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, flere alternativer",attendeeRole:"Deltaker",micDisabledIconLabel:"Mikrofon deaktivert",cameraDisabledIconLabel:"Kamera deaktivert"};var ParticipantList$8={overflowParticipantCount:"+{overflowCount} flere"};var typingIndicator$8={singleUser:"{user} skriver ...",multipleUsers:"{users} skriver ...",multipleUsersAbbreviateOne:"{users} og 1 annen skriver ...",multipleUsersAbbreviateMany:"{users} og {numOthers} andre skriver ...",delimiter:", "};var sendBox$8={placeholderText:"Skriv en melding",textTooLong:"Meldingslengden er over maksimumsgrensen.",sendButtonAriaLabel:"Send melding",attachmentUploadsPendingError:"Filen lastes opp. Vent litt.",removeAttachment:"Fjern fil",uploading:"Laster opp",uploadCompleted:"Opplasting fullført",attachmentMoreMenu:"Flere alternativer"};var richTextSendBox$8={placeholderText:"Skriv en melding",textTooLong:"Meldingslengden er over maksimumsgrensen.",sendButtonAriaLabel:"Send melding",attachmentUploadsPendingError:"Filen lastes opp. Vent litt.",imageUploadsPendingError:"Bildet lastes opp. Vent litt.",removeAttachment:"Fjern fil",uploading:"Laster opp",uploadCompleted:"Opplasting fullført",richTextBoldTooltip:"Fet",richTextItalicTooltip:"Kursiv",richTextUnderlineTooltip:"Understreking",richTextBulletListTooltip:"Punktliste",richTextNumberListTooltip:"Nummerert liste",richTextIncreaseIndentTooltip:"Øk innrykk",richTextDecreaseIndentTooltip:"Reduser innrykk",richTextInsertTableTooltip:"Sett inn tabell",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Sett inn",richTextInsertTableMenuTitle:"Sett inn {column} x {row}-tabell",richTextInsertRowAboveMenu:"Sett inn ovenfor",richTextInsertRowBelowMenu:"Sett inn nedenfor",richTextInsertColumnLeftMenu:"Sett inn til venstre",richTextInsertColumnRightMenu:"Sett inn til høyre",richTextDeleteRowOrColumnMenu:"Slett",richTextDeleteColumnMenu:"Slett kolonne",richTextDeleteRowMenu:"Slett rad",richTextDeleteTableMenu:"Slett tabell",richTextToolbarAriaLabel:"Verktøylinje for meldingsformatering",richTextToolbarMoreButtonAriaLabel:"Flere alternativer",attachmentMoreMenu:"Flere alternativer",richTextNewBulletedListItemAnnouncement:"Punkt, ny linje",richTextNewNumberedListItemAnnouncement:"{0}, ny linje",richTextBulletedListAppliedAnnouncement:"Punktliste er brukt",richTextNumberedListAppliedAnnouncement:"Nummerert liste brukes"};var realTimeTextModal$8={realTimeTextModalAriaLabel:"Aktiver sperrende dialogboks for skriving i sanntid",realTimeTextModalTitle:"Aktiver skriving i sanntid?",realTimeTextModalText:"Sanntidstekst (RTT) aktiveres for alle i møtet.",realTimeTextConfirmButtonLabel:"Aktiver",realTimeTextCancelButtonLabel:"Avbryt",realTimeTextCloseModalButtonAriaLabel:"Lukk sperrende dialogboks for skriving i sanntid"};var captionsBanner$8={captionsBannerSpinnerText:"Starter teksting ...",realTimeTextInputBoxDefaultText:"Skriv inn meldingen i sanntid",realTimeTextInputErrorMessage:"Meldingen kan ikke overskride 2000 tegn",realTimeTextBannerTitle:"Skriving i sanntid",realTimeTextBannerContent:"RTT (sanntidstekst) er aktivert for alle deltakere hele varigheten for møtet.",realTimeTextBannerLinkLabel:"Mer informasjon",captionsOnlyContainerTitle:"Undertekster",realTimeTextOnlyContainerTitle:"Tekst i sanntid",captionsAndRealTimeTextContainerTitle:"Teksting og sanntidstekst",expandButtonAriaLabel:"Utvid",minimizeButtonAriaLabel:"Skjul"};var mentionPopover$8={mentionPopoverHeader:"Forslag"};var imageOverlay$8={downloadButtonLabel:"Last ned",dismissButtonAriaLabel:"Lukk",overlayVisibleMessage:"Detaljoverlegg for bilde"};var messageStatusIndicator$8={deliveredAriaLabel:"Melding sendt",deliveredTooltipText:"Sendt",seenAriaLabel:"Melding sett av andre",seenTooltipText:"Sett",readByTooltipText:"Lest av {messageThreadReadCount} av {remoteParticipantsCount}",sendingAriaLabel:"Sending av melding",sendingTooltipText:"Sender",failedToSendAriaLabel:"Melding ikke sendt",failedToSendTooltipText:"Kan ikke sende"};var endCallButton$8={label:"Forlat",tooltipContent:"Forlat samtale",leaveOption:"Forlat",endCallOption:"Avslutt samtale for alle"};var cameraButton$8={onLabel:"Kamera",offLabel:"Kamera",tooltipDisabledContent:"Kameraet er deaktivert",tooltipOnContent:"Slå av kamera",tooltipOffContent:"Slå på kamera",tooltipVideoLoadingContent:"Videoen lastes inn",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Velg kamera",cameraButtonSplitRoleDescription:"Rullegardinknapp",onSplitButtonAriaLabel:"Slå av kamera",offSplitButtonAriaLabel:"Slå på kamera",cameraActionTurnedOnAnnouncement:"Kameraet er slått på",cameraActionTurnedOffAnnouncement:"Kameraet er slått av.",onSplitButtonPrimaryActionCamera:"Slå av kamera",offSplitButtonPrimaryActionCamera:"Slå på kamera",cameraPrimaryActionSplitButtonTitle:"Bruk kamera",videoEffectsMenuItemTitle:"Effekter",cameraButtonAriaDescription:"kameraalternativer"};var microphoneButton$8={onLabel:"Mikrofon",offLabel:"Mikrofon",tooltipDisabledContent:"Mikrofonen er deaktivert",tooltipOnContent:"Demp mikrofon",tooltipOffContent:"Opphev demping av mikrofon",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Velg mikrofon",speakerMenuTitle:"Høyttaler",speakerMenuTooltip:"Velg høyttaler",microphoneButtonSplitRoleDescription:"Rullegardinknapp",onSplitButtonAriaLabel:"Demp mikrofon",offSplitButtonAriaLabel:"Opphev demping av mikrofon",microphoneActionTurnedOnAnnouncement:"Mikrofonen er slått på",microphoneActionTurnedOffAnnouncement:"Mikrofonen er slått av",offSplitButtonMicrophonePrimaryAction:"Opphev demping av mikrofon",onSplitButtonMicrophonePrimaryAction:"Demp mikrofon",microphonePrimaryActionSplitButtonTitle:"Bruk mikrofon",microphoneAriaDescription:"Lydalternativer",deepNoiseSuppressionTitle:"Støydemping",deepNoiseSuppressionOnAnnouncement:"Dyp støydemping er aktivert",deepNoiseSuppressionOffAnnouncement:"Dyp støydemping er deaktivert"};var devicesButton$8={label:"Enheter",tooltipContent:"Administrer enheter",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Velg kamera",audioDeviceMenuTitle:"Lydenhet",audioDeviceMenuTooltip:"Velg lydenhet",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Velg mikrofon",speakerMenuTitle:"Høyttaler",speakerMenuTooltip:"Velg høyttaler"};var participantsButton$8={label:"Personer",ariaLabel:"Personer",tooltipContent:"Vis deltakere",menuHeader:"I denne samtalen",participantsListButtonLabel:"{numParticipants} personer",muteAllButtonLabel:"Demp alle",copyInviteLinkButtonLabel:"Kopier invitasjonskobling",copyInviteLinkActionedAriaLabel:"Invitasjonskobling er kopiert"};var screenShareButton$8={onLabel:"Stopp presentasjon",offLabel:"Presenter",tooltipDisabledContent:"Presentasjon er deaktivert",tooltipOnContent:"Presenterer skjermen din",tooltipOffContent:"Presenter skjermen din"};var raiseHandButton$8={offLabel:"Rekk opp",onLabel:"Ta ned",tooltipDisabledContent:"Rekk opp hånden-handlingen er deaktivert",tooltipOnContent:"Ta ned hånden",tooltipOffContent:"Rekke opp hånden"};var reactionButton$8={label:"Reager",ariaLabel:"Reager",tooltipDisabledContent:"Reaksjonshandling er deaktivert",tooltipContent:"Send en reaksjon",likeReactionTooltipContent:"Liker",heartReactionTooltipContent:"Kjærlighet",laughReactionTooltipContent:"Latter",applauseReactionTooltipContent:"Applaus",surprisedReactionTooltipContent:"Overrasket"};var realTimeText$8={isTypingText:"Skriving i sanntid"};var captionsSettingsModal$8={captionsSettingsModalTitle:"Undertekstinnstillinger",captionsSettingsSpokenLanguageDropdownLabel:"Talespråk",captionsSettingsCaptionLanguageDropdownLabel:"Undertekstspråk",captionsSettingsSpokenLanguageDropdownInfoText:"Språket som alle i denne samtalen snakker.",captionsSettingsCaptionLanguageDropdownInfoText:"Undertekster vises på dette språket.",captionsSettingsConfirmButtonLabel:"Bekreft",captionsSettingsCancelButtonLabel:"Avbryt",captionsSettingsModalAriaLabel:"Sperre for innstillinger for undertekster",captionsSettingsCloseModalButtonAriaLabel:"Lukk innstilling for undertekster"};var spokenLanguages$8={"ar-ae":"Arabisk – De forente arabiske emirater","ar-sa":"Arabisk – Saudi-Arabia","da-dk":"Dansk","de-de":"Tysk – Tyskland","en-au":"Engelsk – Australia","en-ca":"Engelsk – Canada","en-gb":"Engelsk – Storbritannia","en-in":"Engelsk – India","en-nz":"Engelsk – New Zealand","en-us":"Engelsk – USA","es-es":"Spansk – Spania (moderne)","es-mx":"Spansk – Mexico","fi-fi":"Finsk","fr-ca":"Fransk – Canada","fr-fr":"Fransk – Frankrike","hi-in":"Hindi","it-it":"Italiensk – Italia","ja-jp":"Japansk","ko-kr":"Koreansk","nb-no":"Norsk (bokmål)","nl-be":"Nederlandsk – Belgia","nl-nl":"Nederlandsk – Nederland","pl-pl":"Polsk","pt-br":"Portugisisk – Brasil","ru-ru":"Russisk","sv-se":"Svensk","zh-cn":"Kinesisk – Folkerepublikken Kina","zh-hk":"Kinesisk – Hongkong SAR","cs-cz":"Tsjekkisk","pt-pt":"Portugisisk – Portugal","tr-tr":"Tyrkisk","vi-vn":"Vietnamesisk","th-th":"Thai","he-il":"Hebraisk","cy-gb":"Walisisk","uk-ua":"Ukrainsk","el-gr":"Gresk","hu-hu":"Ungarsk","ro-ro":"Rumensk","sk-sk":"Slovakisk","zh-tw":"Kinesisk – Taiwan"};var captionLanguages$8={ar:"Arabisk",da:"Dansk",de:"Tysk",en:"Engelsk",es:"Spansk",fi:"Finsk","fr-ca":"Fransk – Canada",fr:"Fransk – Frankrike",hi:"Hindi",it:"Italiensk",ja:"Japansk",ko:"Koreansk",nb:"Norsk (bokmål)",nl:"Nederlandsk",pl:"Polsk",pt:"Portugisisk – Brasil",ru:"Russisk",sv:"Svensk","zh-Hans":"Kinesisk (forenklet)","zh-Hant":"Kinesisk (tradisjonell)",cs:"Tsjekkisk","pt-pt":"Portugisisk – Portugal",tr:"Tyrkisk",vi:"Vietnamesisk",th:"Thai",he:"Hebraisk",cy:"Walisisk",uk:"Ukrainsk",el:"Gresk",hu:"Ungarsk",ro:"Rumensk",sk:"Slovakisk"};var startCaptionsButton$8={onLabel:"Slå på undertekster",offLabel:"Deaktiver undertekster",tooltipOnContent:"Deaktiver undertekster",tooltipOffContent:"Slå på undertekster"};var startRealTimeTextButton$8={label:"Aktiver RTT for denne samtalen",tooltipOffContent:"Aktiver RTT for denne samtalen"};var messageThread$8={yesterday:"I går",sunday:"Søndag",monday:"Mandag",tuesday:"Tirsdag",wednesday:"Onsdag",thursday:"Torsdag",friday:"Fredag",saturday:"Lørdag",participantJoined:"ble med i chatten.",participantLeft:"forlot chatten.",editMessage:"Rediger",removeMessage:"Slett",resendMessage:"Prøv å sende på nytt",failToSendTag:"Kan ikke sende",editedTag:"Redigert",liveAuthorIntro:"Melding fra {author}",editedMessageLiveAuthorIntro:"Redigert melding fra {author}",editedMessageLocalUserLiveAuthorIntro:"Redigert melding",messageContentAriaText:"{status} {author} sa {message}",messageContentMineAriaText:"{status} Du sa {message}",editBoxTextLimit:"Meldingen er over grensen på {limitNumber} tegn",editBoxPlaceholderText:"Rediger meldingen",newMessagesIndicator:"Nye meldinger",noDisplayNameSub:"Uten navn",editBoxCancelButton:"Avbryt",editBoxSubmitButton:"Fullført",messageReadCount:"Lest av {messageReadByCount} av {remoteParticipantsCount}",actionMenuMoreOptions:"Flere alternativer",downloadAttachment:"Last ned fil",openAttachment:"Åpne fil",blockedWarningText:"Denne meldingen ble slettet på grunn av en organisasjonspolicy.",blockedWarningLinkText:"Detaljer",attachmentCardGroupMessage:"Meldingen har {attachmentCount} vedlegg",messageDeletedAnnouncementAriaLabel:"Meldingen er slettet",richTextBoldTooltip:"Fet",richTextItalicTooltip:"Kursiv",richTextUnderlineTooltip:"Understreking",richTextBulletListTooltip:"Punktliste",richTextNumberListTooltip:"Nummerert liste",richTextIncreaseIndentTooltip:"Øk innrykk",richTextDecreaseIndentTooltip:"Reduser innrykk",richTextInsertTableTooltip:"Sett inn tabell",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Sett inn",richTextInsertTableMenuTitle:"Sett inn {column} x {row}-tabell",richTextInsertRowAboveMenu:"Sett inn ovenfor",richTextInsertRowBelowMenu:"Sett inn nedenfor",richTextInsertColumnLeftMenu:"Sett inn til venstre",richTextInsertColumnRightMenu:"Sett inn til høyre",richTextDeleteRowOrColumnMenu:"Slett",richTextDeleteColumnMenu:"Slett kolonne",richTextDeleteRowMenu:"Slett rad",richTextDeleteTableMenu:"Slett tabell",richTextToolbarMoreButtonAriaLabel:"Flere alternativer",attachmentUploadsPendingError:"Filen lastes opp. Vent litt.",imageUploadsPendingError:"Bildet lastes opp. Vent litt.",richTextNewBulletedListItemAnnouncement:"Punkt, ny linje",richTextNewNumberedListItemAnnouncement:"{0}, ny linje",richTextBulletedListAppliedAnnouncement:"Punktliste er brukt",richTextNumberedListAppliedAnnouncement:"Nummerert liste brukes"};var errorBar$8={unableToReachChatService:"Du er frakoblet",accessDenied:"Får ikke tilgang til chattjenester. Kontroller brukerlegitimasjonen som er angitt",userNotInChatThread:"Du er ikke lenger i denne chattetråden",sendMessageNotInChatThread:"Kan ikke sende meldingen fordi du ikke lenger er i denne chattetråden",sendMessageGeneric:"Kan ikke sende meldingen",callingNetworkFailure:"Problemer med å koble til samtalen – du ser ut til å være frakoblet",startVideoGeneric:"Kan ikke starte video",stopVideoGeneric:"Kan ikke stoppe video",muteGeneric:"Kan ikke dempe mikrofonen",unmuteGeneric:"Kan ikke oppheve demping av mikrofon",speakingWhileMuted:"Mikrofonen er dempet",startScreenShareGeneric:"Det oppstod et problem under start av skjermdeling.",stopScreenShareGeneric:"Kan ikke stoppe skjermdeling",callNetworkQualityLow:"Nettverkskvaliteten er lav.",teamsMeetingCallNetworkQualityLow:"Nettverkskvaliteten er lav.",callNoSpeakerFound:"Finner ingen høyttalere eller hodetelefoner. Koble til en lydenhet for å høre samtalen.",callNoMicrophoneFound:"Finner ingen mikrofoner. Koble til en lydinndataenhet.",callMicrophoneAccessDenied:"Får ikke tilgang til mikrofonen. Klikk låsen i adresselinjen for å gi tillatelse til denne nettsiden.",callMicrophoneAccessDeniedSafari:"Får ikke tilgang til mikrofonen. Oppdater siden for å tillate tillatelser, eller kontroller innstillingene for denne nettleseren og kontroller at tillatelser er aktivert for dette nettstedet.",callMicrophoneMutedBySystem:"Du er dempet av systemet.",callMicrophoneUnmutedBySystem:"Mikrofonen ble gjenopprettet, og dempingen ble opphevet av systemet.",callMacOsMicrophoneAccessDenied:"Får ikke tilgang til mikrofonen. Gi mikrofontillatelse i personverninnstillingene for macOS.",callLocalVideoFreeze:"Nettverksbåndbredden er dårlig. Videoen kan være midlertidig stanset for andre i samtalen.",callCameraAccessDenied:"Får ikke tilgang til kameraet. Klikk låsen i adresselinjen for å gi tillatelse til denne nettsiden.",callCameraAccessDeniedSafari:"Får ikke tilgang til kameraet. Oppdater siden for å tillate tillatelser, eller kontroller innstillingene for denne nettleseren og kontroller at tillatelser er aktivert for dette nettstedet.",callCameraAlreadyInUse:"Får ikke tilgang til kameraet. Det kan allerede være i bruk av et annet program.",callVideoStoppedBySystem:"Videoen er stoppet av systemet.",callVideoRecoveredBySystem:"Videoen er gjenopptatt.",callMacOsCameraAccessDenied:"MacOS blokkerer tilgang til kameraet. Oppdater personverninnstillingene for å gi denne nettleseren tilgang til kameraet.",callMacOsScreenShareAccessDenied:"MacOS blokkerer skjermdeling. Oppdater personverninnstillingene for å tillate at denne nettleseren registrerer skjermen.",dismissButtonAriaLabel:"Lukk",failedToJoinCallGeneric:"Kan ikke bli med i samtalen.",failedToJoinCallInvalidMeetingLink:"Kan ikke bli med i møtet. Ugyldig kobling.",cameraFrozenForRemoteParticipants:"Brukere i samtalen har problemer med å se videoen. Kontroller enhetene og nettverket.",unableToStartVideoEffect:"Kan ikke bruke videoeffekt.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Kan ikke starte rampelys på deltaker(e) fordi maksimalt antall deltakere allerede er satt i rampelyset.",mutedByRemoteParticipant:"Du har blitt dempet av en annen deltaker."};var notificationStack$8={callingNetworkFailure:{title:"Problemer med å koble til samtalen",message:"Du ser ut til å være frakoblet. Kontroller nettverkstilkoblingen, og prøv på nytt.",dismissButtonAriaLabel:"Lukk"},startVideoGeneric:{title:"Kan ikke starte video.",dismissButtonAriaLabel:"Lukk"},stopVideoGeneric:{title:"Kan ikke stoppe video.",dismissButtonAriaLabel:"Lukk"},muteGeneric:{title:"Kan ikke dempe mikrofonen.",dismissButtonAriaLabel:"Lukk"},unmuteGeneric:{title:"Kan ikke oppheve demping av mikrofon.",dismissButtonAriaLabel:"Lukk"},speakingWhileMuted:{title:"Du er dempet.",dismissButtonAriaLabel:"Lukk"},startScreenShareGeneric:{title:"Det oppstod et problem under start av skjermdeling.",dismissButtonAriaLabel:"Lukk"},stopScreenShareGeneric:{title:"Kan ikke stoppe skjermdeling.",dismissButtonAriaLabel:"Lukk"},callNetworkQualityLow:{title:"Nettverkskvaliteten er lav.",dismissButtonAriaLabel:"Lukk"},teamsMeetingCallNetworkQualityLow:{title:"Nettverkskvaliteten er lav.",dismissButtonAriaLabel:"Lukk",message:"Bli med i denne samtalen fra telefonen for bedre lyd. Du kan fortsette å vise møtet på denne enheten.",primaryButtonLabel:"Bli med via telefon"},callNoSpeakerFound:{title:"Finner ingen høyttalere eller hodetelefoner",message:"Koble til en lydenhet for å høre samtalen.",dismissButtonAriaLabel:"Lukk"},callNoMicrophoneFound:{title:"Finner ingen mikrofoner",message:"Koble til en lydinndataenhet.",dismissButtonAriaLabel:"Lukk"},callMicrophoneAccessDenied:{title:"Får ikke tilgang til mikrofonen",message:"Klikk låsen i adresselinjen for å gi tillatelse til denne nettsiden.",dismissButtonAriaLabel:"Lukk"},callMicrophoneAccessDeniedSafari:{title:"Får ikke tilgang til mikrofonen",message:"Oppdater siden for å tillate tillatelser, eller kontroller innstillingene for denne nettleseren, og kontroller at tillatelser er aktivert for dette nettstedet.",dismissButtonAriaLabel:"Lukk"},callMicrophoneMutedBySystem:{title:"Du er dempet av systemet.",dismissButtonAriaLabel:"Lukk"},callMicrophoneUnmutedBySystem:{title:"Mikrofonen ble gjenopprettet, og dempingen ble opphevet av systemet.",dismissButtonAriaLabel:"Lukk"},callMacOsMicrophoneAccessDenied:{title:"Får ikke tilgang til mikrofonen",message:"Gi mikrofontillatelse i personverninnstillingene for macOS.",dismissButtonAriaLabel:"Lukk"},callLocalVideoFreeze:{title:"Nettverksbåndbredden er dårlig",message:"Videoen kan være midlertidig stanset for andre i samtalen.",dismissButtonAriaLabel:"Lukk"},callCameraAccessDenied:{title:"Får ikke tilgang til kamera",message:"Klikk låsen i adresselinjen for å gi tillatelse til denne nettsiden.",dismissButtonAriaLabel:"Lukk"},callCameraAccessDeniedSafari:{title:"Får ikke tilgang til kamera",message:"Oppdater siden for å tillate tillatelser, eller kontroller innstillingene for denne nettleseren, og kontroller at tillatelser er aktivert for dette nettstedet.",dismissButtonAriaLabel:"Lukk"},callCameraAlreadyInUse:{title:"Får ikke tilgang til kamera",message:"Det kan allerede være i bruk av et annet program.",dismissButtonAriaLabel:"Lukk"},callVideoStoppedBySystem:{title:"Videoen er stoppet av systemet.",dismissButtonAriaLabel:"Lukk"},callVideoRecoveredBySystem:{title:"Videoen er gjenopptatt.",dismissButtonAriaLabel:"Lukk"},callMacOsCameraAccessDenied:{title:"MacOS blokkerer tilgang til kameraet",message:"Oppdater personverninnstillingene for å gi denne nettleseren tilgang til kameraet.",dismissButtonAriaLabel:"Lukk"},callMacOsScreenShareAccessDenied:{title:"MacOS blokkerer skjermdeling",message:"Oppdater personverninnstillingene for å tillate at denne nettleseren registrerer skjermen.",dismissButtonAriaLabel:"Lukk"},failedToJoinCallGeneric:{title:"Kan ikke bli med i samtalen.",dismissButtonAriaLabel:"Lukk"},failedToJoinCallInvalidMeetingLink:{title:"Kan ikke bli med i møtet. Ugyldig kobling.",dismissButtonAriaLabel:"Lukk"},cameraFrozenForRemoteParticipants:{title:"Brukere i samtalen har problemer med å se videoen",message:"Kontroller enhetene og nettverket.",dismissButtonAriaLabel:"Lukk"},unableToStartVideoEffect:{title:"Kan ikke bruke videoeffekt.",dismissButtonAriaLabel:"Lukk"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Kan ikke begynne å sette deltaker(e) i rampelyset",message:"Maksimalt antall deltakere er allerede satt i rampelyset.",dismissButtonAriaLabel:"Lukk"},mutedByRemoteParticipant:{title:"Du har blitt dempet av en annen deltaker.",dismissButtonAriaLabel:"Lukk"},recordingStarted:{title:"Innspilling er startet.",message:"Ved å bli med gir du samtykke til at dette møtet kan transkriberes.",dismissButtonAriaLabel:"Lukk"},transcriptionStarted:{title:"Transkripsjon er startet.",message:"Ved å bli med gir du samtykke til at dette møtet kan transkriberes.",dismissButtonAriaLabel:"Lukk"},recordingStopped:{title:"Opptaket blir lagret.",message:"Innspillingen har stoppet.",dismissButtonAriaLabel:"Lukk"},transcriptionStopped:{title:"Transkripsjonen blir lagret.",message:"Transkripsjon har stoppet.",dismissButtonAriaLabel:"Lukk"},recordingAndTranscriptionStarted:{title:"Opptak og transkripsjon har startet.",message:"Ved å bli med gir du samtykke til at dette møtet kan transkriberes.",dismissButtonAriaLabel:"Lukk"},recordingAndTranscriptionStopped:{title:"Opptak og transkripsjon lagres.",message:"Opptak og transkripsjon har stoppet.",dismissButtonAriaLabel:"Lukk"},recordingStoppedStillTranscribing:{title:"Innspillingen har stoppet.",message:"Du transkriberer nå bare dette møtet.",dismissButtonAriaLabel:"Lukk"},transcriptionStoppedStillRecording:{title:"Transkripsjon har stoppet.",message:"Du tar nå bare opp dette møtet.",dismissButtonAriaLabel:"Lukk"},assignedBreakoutRoomOpened:{title:"Vi flytter deg til det tilordnede rommet om fem sekunder.",dismissButtonAriaLabel:"Lukk"},assignedBreakoutRoomChanged:{title:"Grupperommet er endret",message:"Vi flytter deg til det tilordnede rommet om fem sekunder.",dismissButtonAriaLabel:"Lukk"},assignedBreakoutRoomClosed:{title:"Grupperommet ditt er lukket",message:"Vi flytter deg tilbake til møtet om fem sekunder.",dismissButtonAriaLabel:"Lukk"},assignedBreakoutRoomOpenedPromptJoin:{title:"Bli med i grupperommet?",message:"Du har blitt tilordnet et grupperom.",dismissButtonAriaLabel:"Lukk",primaryButtonLabel:"Bli med i rom",secondaryButtonLabel:"Senere"},breakoutRoomJoined:{title:"Du har blitt med i et grupperom.",dismissButtonAriaLabel:"Lukk"},breakoutRoomClosingSoon:{title:"Romtidsgrensen er i ferd med å utløpe.",message:"Dette rommet blir lukket om 30 sekunder",dismissButtonAriaLabel:"Lukk"},capabilityTurnVideoOnPresent:{title:"Kameraet er aktivert",message:"Slå den på hvis du vil dele video.",dismissButtonAriaLabel:"Lukk"},capabilityTurnVideoOnAbsent:{title:"Kameraet ditt er deaktivert",message:"Du kan ikke lenger dele video.",dismissButtonAriaLabel:"Lukk"},capabilityUnmuteMicPresent:{title:"Mikrofonen er aktivert",message:"Slå på lyden for å snakke.",dismissButtonAriaLabel:"Lukk"},capabilityUnmuteMicAbsent:{title:"Mikrofonen er deaktivert",message:"Du kan ikke lenger oppheve dempingen.",dismissButtonAriaLabel:"Lukk"},togetherModeStarted:{title:"Sammenmodus er startet",dismissButtonAriaLabel:"Lukk"},togetherModeEnded:{title:"Sammenmodus er avsluttet",dismissButtonAriaLabel:"Lukk"},transcriptionError:{title:"Noe gikk galt med transkripsjonen.",message:"Transkripsjonen er stoppet. Prøv på nytt senere",dismissButtonAriaLabel:"Lukk"},transcriptionStartedByYou:{title:"Transkripsjon er startet.",message:"Startet av deg. Sørg for at alle vet at de blir transkribert.",dismissButtonAriaLabel:"Lukk"}};var videoGallery$8={screenIsBeingSharedMessage:"Du deler skjermen",screenShareLoadingMessage:"Laster inn skjermen til {participant}",localScreenShareLoadingMessage:"Laster inn skjermen",localVideoLabel:"Du",localVideoCameraSwitcherLabel:"Bytt kamera",localVideoMovementLabel:"Flyttbar lokal videoflis",localVideoMovementAriaLabel:"Flyttbar lokal videoflis",localVideoSelectedDescription:"{cameraName} valgt",displayNamePlaceholder:"Deltaker uten navn",fitRemoteParticipantToFrame:"Tilpass til ramme",fillRemoteParticipantFrame:"Fyll ramme",pinParticipantForMe:"Fest for meg",pinParticipantForMeLimitReached:"Pin-kode (grense nådd)",unpinParticipantForMe:"Løsne",pinParticipantMenuItemAriaLabel:"Fest {participantName}",unpinParticipantMenuItemAriaLabel:"Løsne {participantName}",pinnedParticipantAnnouncementAriaLabel:"Festet {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Løsnet {participantName}",startSpotlightVideoTileMenuLabel:"Sett i rampelyset for alle",addSpotlightVideoTileMenuLabel:"Legg til rampelys",spotlightLimitReachedMenuTitle:"Rampelys-grensen er nådd",stopSpotlightVideoTileMenuLabel:"Fjern fra rampelyset",stopSpotlightOnSelfVideoTileMenuLabel:"Rampelys av",attendeeRole:"Deltaker",muteParticipantMenuItemLabel:"Demp",waitingScreenText:"Venter på at andre skal bli med",forbidAudio:"Deaktivere mikrofon",permitAudio:"Tillat mikrofon",forbidAudioTileMenuLabel:"Deaktivere mikrofon",permitAudioTileMenuLabel:"Tillat mikrofon",forbidVideo:"Deaktivere kamera",permitVideo:"Tillat kamera",forbidVideoTileMenuLabel:"Deaktivere kamera",permitVideoTileMenuLabel:"Tillat kamera"};var dialpad$8={placeholderText:"Skriv inn telefonnummeret",deleteButtonAriaLabel:"Slett"};var holdButton$8={onLabel:"Fortsett",offLabel:"Vent",tooltipOnContent:"Fortsett samtale",tooltipOffContent:"Sett samtalen på vent"};var videoTile$8={participantStateRinging:"Ringer ...",participantReconnecting:"Kobler til på nytt ...",participantStateHold:"På vent",moreOptionsButtonAriaLabel:"Flere alternativer {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"varsler er deaktivert",moreOptionsParticipantMutedStateUnmutedAriaLabel:"opphevet demping",moreOptionsParticipantHandRaisedAriaLabel:"Hånd rakt opp",moreOptionsParticipantIsSpeakingAriaLabel:"Snakke",moreOptionsParticipantMicDisabledAriaLabel:"Mikrofon deaktivert",moreOptionsParticipantCameraDisabledAriaLabel:"Kamera deaktivert"};var CameraAndMicrophoneSitePermissionsRequest$8={primaryText:"Tillat {appName} å bruke kameraet og mikrofonen",secondaryText:"Dette er for at deltakerne skal kunne se og høre deg.",linkText:"Trenger du hjelp? Få hjelp til feilsøking",primaryButtonText:"Fortsett uten kamera og mikrofon",ariaLabel:"Gi tilgang til kamera og mikrofon"};var CameraSitePermissionsRequest$8={primaryText:"Tillat at {appName} bruker kameraet",secondaryText:"Dette er for at deltakerne skal kunne se deg.",linkText:"Trenger du hjelp? Få hjelp til feilsøking",primaryButtonText:"Fortsett uten kamera",ariaLabel:"Gi kameratilgang"};var MicrophoneSitePermissionsRequest$8={primaryText:"Tillat at {appName} bruker mikrofonen",secondaryText:"Dette er for at deltakerne skal kunne høre deg.",linkText:"Trenger du hjelp? Få hjelp til feilsøking",primaryButtonText:"Fortsett uten mikrofon",ariaLabel:"Tillat mikrofontilgang"};var CameraAndMicrophoneSitePermissionsCheck$8={primaryText:"Ser etter tilgang til kamera og mikrofon",secondaryText:"Gi tilgang hvis du blir bedt om det. Dette er slik at deltakerne kan se og høre deg.",linkText:"Trenger du hjelp? Få hjelp til feilsøking",primaryButtonText:"Fortsett uten kamera og mikrofon",ariaLabel:"Ser etter tilgang til kamera og mikrofon. Gi tilgang hvis du blir bedt om det."};var CameraSitePermissionsCheck$8={primaryText:"Ser etter kameratilgang",secondaryText:"Gi tilgang hvis du blir bedt om det. Dette er slik at deltakerne kan se deg.",linkText:"Trenger du hjelp? Få hjelp til feilsøking",primaryButtonText:"Fortsett uten kamera",ariaLabel:"Ser etter kameratilgang. Gi tilgang hvis du blir bedt om det."};var MicrophoneSitePermissionsCheck$8={primaryText:"Ser etter mikrofontilgang",secondaryText:"Gi tilgang hvis du blir bedt om det. Dette er slik at deltakerne kan høre deg.",linkText:"Trenger du hjelp? Få hjelp til feilsøking",primaryButtonText:"Fortsett uten mikrofon",ariaLabel:"Ser etter mikrofontilgang. Gi tilgang hvis du blir bedt om det."};var CameraAndMicrophoneSitePermissionsDenied$8={primaryText:"Får ikke tilgang til kamera og mikrofon",secondaryText:"Klikk på låseikonet i adresselinjen for å gi mikrofontillatelser til denne nettsiden. En sideoppdatering kan være nødvendig.",primaryButtonText:"Fortsett uten kamera og mikrofon",linkText:"Trenger du hjelp? Få hjelp til feilsøking"};var CameraAndMicrophoneSitePermissionsDeniedSafari$8={primaryText:"Får ikke tilgang til kamera og mikrofon",secondaryText:"Oppdater siden for å tillate tillatelser, eller kontroller innstillingene for denne nettleseren og kontroller at tillatelser er aktivert for dette nettstedet.",primaryButtonText:"Fortsett uten kamera og mikrofon",linkText:"Trenger du hjelp? Få hjelp til feilsøking"};var CameraSitePermissionsDenied$8={primaryText:"Får ikke tilgang til kamera",secondaryText:"Klikk på låseikonet på adresselinjen for å gi kameratillatelser til denne nettsiden. En sideoppdatering kan være nødvendig.",primaryButtonText:"Fortsett uten kamera",linkText:"Trenger du hjelp? Få hjelp til feilsøking"};var MicrophoneSitePermissionsDenied$8={primaryText:"Får ikke tilgang til mikrofonen",secondaryText:"Klikk på låseikonet i adresselinjen for å gi mikrofontillatelser til denne nettsiden. En sideoppdatering kan være nødvendig.",primaryButtonText:"Fortsett uten mikrofon",linkText:"Trenger du hjelp? Få hjelp til feilsøking"};var CameraSitePermissionsDeniedSafari$8={primaryText:"Får ikke tilgang til kamera",secondaryText:"Oppdater siden for å tillate tillatelser, eller kontroller innstillingene for denne nettleseren og kontroller at tillatelser er aktivert for dette nettstedet.",primaryButtonText:"Fortsett uten kamera",linkText:"Trenger du hjelp? Få hjelp til feilsøking"};var MicrophoneSitePermissionsDeniedSafari$8={primaryText:"Får ikke tilgang til mikrofonen",secondaryText:"Oppdater siden for å tillate tillatelser, eller kontroller innstillingene for denne nettleseren og kontroller at tillatelser er aktivert for dette nettstedet.",primaryButtonText:"Fortsett uten mikrofon",linkText:"Trenger du hjelp? Få hjelp til feilsøking"};var UnsupportedBrowser$8={primaryText:"Nettleseren støttes ikke",secondaryText:"Bli med i denne samtalen ved hjelp av en kompatibel nettleser.",moreHelpLinkText:"Se kompatibilitetskrav"};var UnsupportedBrowserVersion$8={primaryText:"Nettleseroppdatering kreves",secondaryText:"Hvis du vil sikre best mulig samtale, må du oppdatere nettleseren og deretter prøve å bli med i samtalen på nytt.",moreHelpLinkText:"Se kompatibilitetskrav",continueAnywayButtonText:"Start anrop uten å oppdatere"};var UnsupportedOperatingSystem$8={primaryText:"Operativsystem støttes ikke",secondaryText:"Bli med i denne samtalen ved hjelp av en enhet med et kompatibelt operativsystem.",moreHelpLinkText:"Se kompatibilitetskrav"};var BrowserPermissionDenied$8={primaryText:"Kan ikke bruke kameraet eller mikrofonen",secondaryText:"Nettleseren har kanskje ikke tilgang til kameraet eller mikrofonen. Åpne Systemvalg for å løse dette problemet.",primaryButtonText:"Prøv på nytt",linkText:"Trenger du hjelp? Få hjelp til feilsøking"};var BrowserPermissionDeniedIOS$8={primaryText:"Gi mikrofontilgang for å fortsette",secondaryText:"Slik at andre deltakere kan høre deg.",primaryButtonText:"Prøv på nytt",imageAltText:"Plassering av mikrofon- og kameraenhetstillatelse for iOS",linkText:"Trenger du hjelp? Få hjelp til feilsøking",step1Text:"Gå til Innstillinger-appen",step2Text:"Rull ned til innstillinger for denne nettleseren",step3Text:"Slå på mikrofon (kamera valgfritt)",step4Text:"Prøv å bli med i samtalen på nytt",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$8={leftNavButtonAriaLabel:"forrige side {current} av {total}",rightNavButtonAriaLabel:"neste side {current} av {total}"};var meetingConferencePhoneInfo$8={meetingConferencePhoneInfoModalTitle:"Bli med i samtalen fra telefonen",meetingConferencePhoneInfoModalDialIn:"Slå nummer",meetingConferencePhoneInfoModalMeetingId:"Skriv inn møte-ID",meetingConferencePhoneInfoModalWait:"Vent på å bli godtatt inn av arrangøren",meetingConferencePhoneInfoModalNoPhoneAvailable:"Ingen telefon tilgjengelig",meetingConferencePhoneInfoModalTollGeoData:"{city}, {country}",meetingConferencePhoneInfoModalTollFree:"(gratis)",meetingConferencePhoneInfoModalToll:"(nummer med avgift)"};var IncomingCallNotification$8={incomingCallNotificationPlaceholderId:"Ukjent bruker",incomingCallNotificationPlaceholderAlert:"{callerName} ringer",incomingCallNoticicationAcceptWithAudioAriaLabel:"Godta med lyd",incomingCallNoticicationAcceptWithVideoAriaLabel:"Godta med video",incomingCallNoticicationRejectAriaLabel:"Avvis",incomingCallNotificationAcceptButtonLabel:"Godta",incomingCallNotificationRejectButtonLabel:"Avslå",incomingCallNotificationAccceptWithVideoButtonLabel:"Svar med video",incomingCallNotificationDismissButtonAriaLabel:"Lukk"};var nb_NO$1 = {participantItem:participantItem$8,ParticipantList:ParticipantList$8,typingIndicator:typingIndicator$8,sendBox:sendBox$8,richTextSendBox:richTextSendBox$8,realTimeTextModal:realTimeTextModal$8,captionsBanner:captionsBanner$8,mentionPopover:mentionPopover$8,imageOverlay:imageOverlay$8,messageStatusIndicator:messageStatusIndicator$8,endCallButton:endCallButton$8,cameraButton:cameraButton$8,microphoneButton:microphoneButton$8,devicesButton:devicesButton$8,participantsButton:participantsButton$8,screenShareButton:screenShareButton$8,raiseHandButton:raiseHandButton$8,reactionButton:reactionButton$8,realTimeText:realTimeText$8,captionsSettingsModal:captionsSettingsModal$8,spokenLanguages:spokenLanguages$8,captionLanguages:captionLanguages$8,startCaptionsButton:startCaptionsButton$8,startRealTimeTextButton:startRealTimeTextButton$8,messageThread:messageThread$8,errorBar:errorBar$8,notificationStack:notificationStack$8,videoGallery:videoGallery$8,dialpad:dialpad$8,holdButton:holdButton$8,videoTile:videoTile$8,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$8,CameraSitePermissionsRequest:CameraSitePermissionsRequest$8,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$8,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$8,CameraSitePermissionsCheck:CameraSitePermissionsCheck$8,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$8,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$8,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$8,CameraSitePermissionsDenied:CameraSitePermissionsDenied$8,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$8,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$8,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$8,UnsupportedBrowser:UnsupportedBrowser$8,UnsupportedBrowserVersion:UnsupportedBrowserVersion$8,UnsupportedOperatingSystem:UnsupportedOperatingSystem$8,BrowserPermissionDenied:BrowserPermissionDenied$8,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$8,verticalGallery:verticalGallery$8,meetingConferencePhoneInfo:meetingConferencePhoneInfo$8,IncomingCallNotification:IncomingCallNotification$8};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Norwegian Bokmål (Norway) .
 *
 * @public
 */
const COMPONENT_LOCALE_NB_NO = {
    strings: createComponentStrings(nb_NO$1)
};

var participantItem$7={isMeText:"(jij)",menuTitle:"Meer opties",removeButtonLabel:"Verwijderen",sharingIconLabel:"Delen",mutedIconLabel:"Gedempt",handRaisedIconLabel:"Opgestoken hand - Positie {position}",displayNamePlaceholder:"Naamloze deelnemer",participantStateRinging:"Bellen...",participantStateHold:"In de wacht",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, meer opties",attendeeRole:"Genodigde",micDisabledIconLabel:"Microfoon uitgeschakeld",cameraDisabledIconLabel:"Camera uitgeschakeld"};var ParticipantList$7={overflowParticipantCount:"+{overflowCount} meer"};var typingIndicator$7={singleUser:"{user} typt ...",multipleUsers:"{users} typen ...",multipleUsersAbbreviateOne:"{users} en 1 andere persoon typen ...",multipleUsersAbbreviateMany:"{users} en {numOthers} anderen typen ...",delimiter:", "};var sendBox$7={placeholderText:"Een bericht invoeren",textTooLong:"De lengte van uw bericht overschrijdt de maximumlimiet.",sendButtonAriaLabel:"Bericht verzenden",attachmentUploadsPendingError:"Bestand wordt geüpload, even geduld.",removeAttachment:"Bestand verwijderen",uploading:"Uploaden",uploadCompleted:"Upload voltooid",attachmentMoreMenu:"Meer opties"};var richTextSendBox$7={placeholderText:"Een bericht invoeren",textTooLong:"De lengte van uw bericht overschrijdt de maximumlimiet.",sendButtonAriaLabel:"Bericht verzenden",attachmentUploadsPendingError:"Bestand wordt geüpload, even geduld.",imageUploadsPendingError:"Afbeelding wordt geüpload, even geduld.",removeAttachment:"Bestand verwijderen",uploading:"Uploaden",uploadCompleted:"Upload voltooid",richTextBoldTooltip:"Vet",richTextItalicTooltip:"Cursief",richTextUnderlineTooltip:"Onderstrepen",richTextBulletListTooltip:"Lijst met opsommingstekens",richTextNumberListTooltip:"Genummerde lijst",richTextIncreaseIndentTooltip:"Inspringing vergroten",richTextDecreaseIndentTooltip:"Inspringing verkleinen",richTextInsertTableTooltip:"Tabel invoegen",richTextFormatButtonTooltip:"Indeling",richTextInsertRowOrColumnMenu:"Invoegen",richTextInsertTableMenuTitle:"{column} x {row} tabel invoegen",richTextInsertRowAboveMenu:"Hierboven invoegen",richTextInsertRowBelowMenu:"Hieronder invoegen",richTextInsertColumnLeftMenu:"Links invoegen",richTextInsertColumnRightMenu:"Rechts invoegen",richTextDeleteRowOrColumnMenu:"Verwijderen",richTextDeleteColumnMenu:"Kolom verwijderen",richTextDeleteRowMenu:"Rij verwijderen",richTextDeleteTableMenu:"Tabel verwijderen",richTextToolbarAriaLabel:"Werkbalk Berichtopmaak",richTextToolbarMoreButtonAriaLabel:"Meer opties",attachmentMoreMenu:"Meer opties",richTextNewBulletedListItemAnnouncement:"Opsommingsteken, nieuwe regel",richTextNewNumberedListItemAnnouncement:"{0}, nieuwe regel",richTextBulletedListAppliedAnnouncement:"Lijst met opsommingstekens toegepast",richTextNumberedListAppliedAnnouncement:"Genummerde lijst is toegepast"};var realTimeTextModal$7={realTimeTextModalAriaLabel:"RTT Modal inschakelen",realTimeTextModalTitle:"RTT inschakelen?",realTimeTextModalText:"Tekst in realtime (RTT) wordt ingeschakeld voor iedereen in de vergadering.",realTimeTextConfirmButtonLabel:"Inschakelen",realTimeTextCancelButtonLabel:"Annuleren",realTimeTextCloseModalButtonAriaLabel:"RTT Modaal sluiten"};var captionsBanner$7={captionsBannerSpinnerText:"Ondertitels starten...",realTimeTextInputBoxDefaultText:"Bericht in realtime typen",realTimeTextInputErrorMessage:"Het bericht mag niet langer zijn dan 2000 tekens",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (realtime-tekst) is ingeschakeld voor alle deelnemers gedurende de hele duur van de vergadering.",realTimeTextBannerLinkLabel:"Meer informatie",captionsOnlyContainerTitle:"Ondertitels",realTimeTextOnlyContainerTitle:"Real-time tekst",captionsAndRealTimeTextContainerTitle:"Ondertitels en realtime tekst",expandButtonAriaLabel:"Uitvouwen",minimizeButtonAriaLabel:"Samenvouwen"};var mentionPopover$7={mentionPopoverHeader:"Suggesties"};var imageOverlay$7={downloadButtonLabel:"Downloaden",dismissButtonAriaLabel:"Sluiten",overlayVisibleMessage:"Overlay voor details afbeelding"};var messageStatusIndicator$7={deliveredAriaLabel:"Bericht verzonden",deliveredTooltipText:"Verzonden",seenAriaLabel:"Bericht gezien door anderen",seenTooltipText:"Gezien",readByTooltipText:"Gelezen door {messageThreadReadCount} van {remoteParticipantsCount}",sendingAriaLabel:"Bericht verzenden",sendingTooltipText:"Verzenden",failedToSendAriaLabel:"Het bericht is niet verzonden",failedToSendTooltipText:"Kan niet verzenden"};var endCallButton$7={label:"Verlaten",tooltipContent:"Gesprek verlaten",leaveOption:"Verlaten",endCallOption:"Oproep beëindigen voor iedereen"};var cameraButton$7={onLabel:"Camera",offLabel:"Camera",tooltipDisabledContent:"Camera is uitgeschakeld",tooltipOnContent:"Camera uitschakelen",tooltipOffContent:"Camera inschakelen",tooltipVideoLoadingContent:"Video wordt geladen",cameraMenuTitle:"Camera",cameraMenuTooltip:"Camera kiezen",cameraButtonSplitRoleDescription:"Knop Splitsen",onSplitButtonAriaLabel:"Camera uitschakelen",offSplitButtonAriaLabel:"Camera inschakelen",cameraActionTurnedOnAnnouncement:"Uw camera is ingeschakeld",cameraActionTurnedOffAnnouncement:"Uw camera is uitgeschakeld",onSplitButtonPrimaryActionCamera:"Camera uitschakelen",offSplitButtonPrimaryActionCamera:"Camera inschakelen",cameraPrimaryActionSplitButtonTitle:"Camera gebruiken",videoEffectsMenuItemTitle:"Effecten",cameraButtonAriaDescription:"camera-opties"};var microphoneButton$7={onLabel:"Microfoon",offLabel:"Microfoon",tooltipDisabledContent:"Microfoon is uitgeschakeld",tooltipOnContent:"Microfoon uitschakelen",tooltipOffContent:"Microfoon inschakelen",microphoneMenuTitle:"Microfoon",microphoneMenuTooltip:"Microfoon kiezen",speakerMenuTitle:"Luidspreker",speakerMenuTooltip:"Luidspreker kiezen",microphoneButtonSplitRoleDescription:"Knop Splitsen",onSplitButtonAriaLabel:"Microfoon dempen",offSplitButtonAriaLabel:"Dempen opheffen voor microfoon",microphoneActionTurnedOnAnnouncement:"Uw microfoon is ingeschakeld",microphoneActionTurnedOffAnnouncement:"Uw microfoon is uitgeschakeld",offSplitButtonMicrophonePrimaryAction:"Dempen opheffen voor microfoon",onSplitButtonMicrophonePrimaryAction:"Microfoon dempen",microphonePrimaryActionSplitButtonTitle:"Microfoon gebruiken",microphoneAriaDescription:"Audio-opties",deepNoiseSuppressionTitle:"Ruisonderdrukking",deepNoiseSuppressionOnAnnouncement:"Diepe ruisonderdrukking is ingeschakeld",deepNoiseSuppressionOffAnnouncement:"Diepe ruisonderdrukking is uitgeschakeld"};var devicesButton$7={label:"Apparaten",tooltipContent:"Apparaten beheren",cameraMenuTitle:"Camera",cameraMenuTooltip:"Camera kiezen",audioDeviceMenuTitle:"Audioapparaat",audioDeviceMenuTooltip:"Audioapparaat kiezen",microphoneMenuTitle:"Microfoon",microphoneMenuTooltip:"Microfoon kiezen",speakerMenuTitle:"Spreker",speakerMenuTooltip:"Luidspreker kiezen"};var participantsButton$7={label:"Personen",ariaLabel:"Personen",tooltipContent:"Deelnemers weergeven",menuHeader:"In dit gesprek",participantsListButtonLabel:"{numParticipants} personen",muteAllButtonLabel:"Alles dempen",copyInviteLinkButtonLabel:"Uitnodigingskoppeling kopiëren",copyInviteLinkActionedAriaLabel:"Uitnodigingskoppeling gekopieerd"};var screenShareButton$7={onLabel:"Presenteren stoppen",offLabel:"Presenteren",tooltipDisabledContent:"Presenteren is uitgeschakeld",tooltipOnContent:"Uw scherm presenteren",tooltipOffContent:"Uw scherm presenteren"};var raiseHandButton$7={offLabel:"Opsteken",onLabel:"Laten zakken",tooltipDisabledContent:"Actie Hand opsteken is uitgeschakeld",tooltipOnContent:"Hand laten zakken",tooltipOffContent:"Hand opsteken"};var reactionButton$7={label:"Reageren",ariaLabel:"Reageren",tooltipDisabledContent:"Reactieactie is uitgeschakeld",tooltipContent:"Verzend een reactie",likeReactionTooltipContent:"Vind ik leuk",heartReactionTooltipContent:"Hartje",laughReactionTooltipContent:"Lachen",applauseReactionTooltipContent:"Applaus",surprisedReactionTooltipContent:"Verrast"};var realTimeText$7={isTypingText:"RTT typen"};var captionsSettingsModal$7={captionsSettingsModalTitle:"Instellingen voor ondertitels",captionsSettingsSpokenLanguageDropdownLabel:"Gesproken taal",captionsSettingsCaptionLanguageDropdownLabel:"Taal van ondertitels",captionsSettingsSpokenLanguageDropdownInfoText:"Taal die iedereen in dit gesprek spreekt.",captionsSettingsCaptionLanguageDropdownInfoText:"Bijschriften worden weergegeven in deze taal.",captionsSettingsConfirmButtonLabel:"Bevestigen",captionsSettingsCancelButtonLabel:"Annuleren",captionsSettingsModalAriaLabel:"Bijschriften instellen modaal",captionsSettingsCloseModalButtonAriaLabel:"Instelling voor ondertiteling sluiten"};var spokenLanguages$7={"ar-ae":"Arabisch - Verenigde Arabische Emiraten","ar-sa":"Arabisch - Saudi-Arabië","da-dk":"Deens","de-de":"Duits - Duitsland","en-au":"Engels - Australië","en-ca":"Engels - Canada","en-gb":"Engels - Verenigd Koninkrijk","en-in":"Engels - India","en-nz":"Engels - Nieuw-Zeeland","en-us":"Engels - Verenigde Staten","es-es":"Spaans - Spanje (modern)","es-mx":"Spaans - Mexico","fi-fi":"Fins","fr-ca":"Frans - Canada","fr-fr":"Frans - Frankrijk","hi-in":"Hindi","it-it":"Italiaans - Italië","ja-jp":"Japans","ko-kr":"Koreaans","nb-no":"Noors (Bokmål)","nl-be":"Nederlands - België","nl-nl":"Nederlands - Nederland","pl-pl":"Pools","pt-br":"Portugees - Brazilië","ru-ru":"Russisch","sv-se":"Zweeds","zh-cn":"Chinees (Volksrepubliek China)","zh-hk":"Chinees - Hongkong SAR","cs-cz":"Tsjechisch","pt-pt":"Portugees - Portugal","tr-tr":"Turks","vi-vn":"Vietnamees","th-th":"Thai","he-il":"Hebreeuws","cy-gb":"Welsh","uk-ua":"Oekraïens","el-gr":"Grieks","hu-hu":"Hongaars","ro-ro":"Roemeens","sk-sk":"Slowaaks","zh-tw":"Chinees - Taiwan"};var captionLanguages$7={ar:"Arabisch",da:"Deens",de:"Duits",en:"Engels",es:"Spaans",fi:"Fins","fr-ca":"Frans - Canada",fr:"Frans - Frankrijk",hi:"Hindi",it:"Italiaans",ja:"Japans",ko:"Koreaans",nb:"Noors (Bokmål)",nl:"Nederlands",pl:"Pools",pt:"Portugees - Brazilië",ru:"Russisch",sv:"Zweeds","zh-Hans":"Chinees (vereenvoudigd)","zh-Hant":"Chinees (traditioneel)",cs:"Tsjechisch","pt-pt":"Portugees - Portugal",tr:"Turks",vi:"Vietnamees",th:"Thai",he:"Hebreeuws",cy:"Welsh",uk:"Oekraïens",el:"Grieks",hu:"Hongaars",ro:"Roemeens",sk:"Slowaaks"};var startCaptionsButton$7={onLabel:"Bijschriften inschakelen",offLabel:"Ondertitels uitschakelen",tooltipOnContent:"Ondertitels uitschakelen",tooltipOffContent:"Bijschriften inschakelen"};var startRealTimeTextButton$7={label:"RTT inschakelen voor dit gesprek",tooltipOffContent:"RTT inschakelen voor dit gesprek"};var messageThread$7={yesterday:"Gisteren",sunday:"Zondag",monday:"Maandag",tuesday:"Dinsdag",wednesday:"Woensdag",thursday:"Donderdag",friday:"Vrijdag",saturday:"Zaterdag",participantJoined:"neemt nu deel aan de chat.",participantLeft:"heeft de chat verlaten.",editMessage:"Bewerken",removeMessage:"Verwijderen",resendMessage:"Probeer opnieuw te verzenden",failToSendTag:"Verzenden mislukt",editedTag:"Bewerkt",liveAuthorIntro:"Bericht van {author}",editedMessageLiveAuthorIntro:"Bewerkt bericht van {author}",editedMessageLocalUserLiveAuthorIntro:"Bewerkt bericht",messageContentAriaText:"{status} {author} zei {message}",messageContentMineAriaText:"{status} U zei {message}",editBoxTextLimit:"Uw bericht heeft de limiet van {limitNumber} tekens overschreden",editBoxPlaceholderText:"Uw bericht bewerken",newMessagesIndicator:"Nieuwe berichten",noDisplayNameSub:"Geen naam",editBoxCancelButton:"Annuleren",editBoxSubmitButton:"Gereed",messageReadCount:"Gelezen door {messageReadByCount} van {remoteParticipantsCount}",actionMenuMoreOptions:"Meer opties",downloadAttachment:"Bestand downloaden",openAttachment:"Bestand openen",blockedWarningText:"Dit bericht is verwijderd vanwege organisatiebeleid.",blockedWarningLinkText:"Details",attachmentCardGroupMessage:"Het bericht heeft {attachmentCount} bijlage",messageDeletedAnnouncementAriaLabel:"Het bericht is verwijderd",richTextBoldTooltip:"Vet",richTextItalicTooltip:"Cursief",richTextUnderlineTooltip:"Onderstrepen",richTextBulletListTooltip:"Lijst met opsommingstekens",richTextNumberListTooltip:"Genummerde lijst",richTextIncreaseIndentTooltip:"Inspringing vergroten",richTextDecreaseIndentTooltip:"Inspringing verkleinen",richTextInsertTableTooltip:"Tabel invoegen",richTextFormatButtonTooltip:"Indeling",richTextInsertRowOrColumnMenu:"Invoegen",richTextInsertTableMenuTitle:"{column} x {row} tabel invoegen",richTextInsertRowAboveMenu:"Hierboven invoegen",richTextInsertRowBelowMenu:"Hieronder invoegen",richTextInsertColumnLeftMenu:"Links invoegen",richTextInsertColumnRightMenu:"Rechts invoegen",richTextDeleteRowOrColumnMenu:"Verwijderen",richTextDeleteColumnMenu:"Kolom verwijderen",richTextDeleteRowMenu:"Rij verwijderen",richTextDeleteTableMenu:"Tabel verwijderen",richTextToolbarMoreButtonAriaLabel:"Meer opties",attachmentUploadsPendingError:"Bestand wordt geüpload, even geduld.",imageUploadsPendingError:"Afbeelding wordt geüpload, even geduld.",richTextNewBulletedListItemAnnouncement:"Opsommingsteken, nieuwe regel",richTextNewNumberedListItemAnnouncement:"{0}, nieuwe regel",richTextBulletedListAppliedAnnouncement:"Lijst met opsommingstekens toegepast",richTextNumberedListAppliedAnnouncement:"Genummerde lijst is toegepast"};var errorBar$7={unableToReachChatService:"U bent offline",accessDenied:"Kan geen toegang krijgen tot chatservices. Controleer de opgegeven gebruikersreferenties",userNotInChatThread:"U bent niet meer aanwezig in deze chatthread",sendMessageNotInChatThread:"Verzenden van bericht is mislukt omdat u zich niet meer in deze chatthread bevindt",sendMessageGeneric:"Kan bericht niet verzenden",callingNetworkFailure:"Er zijn problemen met het verbinden van een gesprek - Het lijkt erop dat u offline bent",startVideoGeneric:"Kan video niet starten",stopVideoGeneric:"Kan video niet stoppen",muteGeneric:"Kan microfoon niet dempen",unmuteGeneric:"Kan dempen van microfoon niet opheffen",speakingWhileMuted:"De microfoon is gedempt",startScreenShareGeneric:"Er is een probleem opgetreden bij het starten van scherm delen.",stopScreenShareGeneric:"Kan scherm delen niet stoppen",callNetworkQualityLow:"De netwerkkwaliteit is laag.",teamsMeetingCallNetworkQualityLow:"De netwerkkwaliteit is laag.",callNoSpeakerFound:"Geen luidsprekers of hoofdtelefoon gevonden. Sluit een audioapparaat aan om het gesprek te beluisteren.",callNoMicrophoneFound:"Geen microfoons gevonden. Sluit een audio-invoerapparaat aan.",callMicrophoneAccessDenied:"Kan geen toegang krijgen tot de microfoon. Klik op de vergrendeling in de adresbalk om toestemming te verlenen aan deze webpagina.",callMicrophoneAccessDeniedSafari:"Kan geen toegang krijgen tot de microfoon. Vernieuw de pagina om machtigingen toe te staan of controleer de instellingen van deze browser en controleer of machtigingen zijn ingeschakeld voor deze website.",callMicrophoneMutedBySystem:"U bent gedempt door jouw systeem.",callMicrophoneUnmutedBySystem:"De microfoon is hersteld en het dempen van de microfoon is door het systeem opgeheven.",callMacOsMicrophoneAccessDenied:"Kan geen toegang krijgen tot de microfoon. Verleen microfoonmachtiging in de privacyinstellingen voor macOS.",callLocalVideoFreeze:"De netwerkbandbreedte is slecht. Je video wordt mogelijk onderbroken weergegeven voor anderen in het gesprek.",callCameraAccessDenied:"Kan geen toegang krijgen tot de camera. Klik op de vergrendeling in de adresbalk om toestemming te verlenen aan deze webpagina.",callCameraAccessDeniedSafari:"Kan geen toegang krijgen tot de camera. Vernieuw de pagina om machtigingen toe te staan of controleer de instellingen van deze browser en controleer of machtigingen zijn ingeschakeld voor deze website.",callCameraAlreadyInUse:"Kan geen toegang krijgen tot de camera. Het wordt mogelijk al gebruikt door een andere toepassing.",callVideoStoppedBySystem:"Uw video is gestopt door uw systeem.",callVideoRecoveredBySystem:"Uw video is hervat.",callMacOsCameraAccessDenied:"MacOS blokkeert de toegang tot jouw camera. Werk je privacyinstellingen bij, zodat deze browser toegang heeft tot jouw camera.",callMacOsScreenShareAccessDenied:"MacOS blokkeert scherm delen. Werk je privacyinstellingen bij, zodat deze browser jouw scherm kan opnemen.",dismissButtonAriaLabel:"Sluiten",failedToJoinCallGeneric:"Deelnemen aan gesprek mislukt.",failedToJoinCallInvalidMeetingLink:"Kan niet deelnemen aan vergadering. Ongeldige koppeling.",cameraFrozenForRemoteParticipants:"Gebruikers in het gesprek ondervinden problemen bij het zien van uw video. Controleer uw apparaten en netwerk.",unableToStartVideoEffect:"Kan video-effect niet toepassen.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Kan de deelnemer(s) niet in de spotlight plaatsen omdat het maximum aantal deelnemers in de spotlight is bereikt.",mutedByRemoteParticipant:"U bent gedempt door een andere deelnemer."};var notificationStack$7={callingNetworkFailure:{title:"Er is geen verbinding gemaakt met het gesprek",message:"U lijkt offline te zijn. Controleer de netwerkverbinding en probeer het opnieuw.",dismissButtonAriaLabel:"Sluiten"},startVideoGeneric:{title:"De video is niet gestart.",dismissButtonAriaLabel:"Sluiten"},stopVideoGeneric:{title:"De video is niet gestopt.",dismissButtonAriaLabel:"Sluiten"},muteGeneric:{title:"De microfoon is niet gedempt.",dismissButtonAriaLabel:"Sluiten"},unmuteGeneric:{title:"Het dempen van de microfoon is niet opgeheven.",dismissButtonAriaLabel:"Sluiten"},speakingWhileMuted:{title:"Uw microfoon is gedempt.",dismissButtonAriaLabel:"Sluiten"},startScreenShareGeneric:{title:"Er is een probleem opgetreden bij het starten van scherm delen.",dismissButtonAriaLabel:"Sluiten"},stopScreenShareGeneric:{title:"Kan scherm delen niet stoppen.",dismissButtonAriaLabel:"Sluiten"},callNetworkQualityLow:{title:"De netwerkkwaliteit is laag.",dismissButtonAriaLabel:"Sluiten"},teamsMeetingCallNetworkQualityLow:{title:"De netwerkkwaliteit is laag.",dismissButtonAriaLabel:"Sluiten",message:"Neem deel aan dit gesprek vanaf uw telefoon voor een beter geluid. U kunt doorgaan met het weergeven van de vergadering op dit apparaat.",primaryButtonLabel:"Deelnemen via telefoon"},callNoSpeakerFound:{title:"Geen luidsprekers of hoofdtelefoon gevonden",message:"Sluit een audioapparaat aan om het gesprek te beluisteren.",dismissButtonAriaLabel:"Sluiten"},callNoMicrophoneFound:{title:"Geen microfoons gevonden",message:"Sluit een audio-invoerapparaat aan.",dismissButtonAriaLabel:"Sluiten"},callMicrophoneAccessDenied:{title:"Geen toegang tot microfoon",message:"Klik op de vergrendeling in de adresbalk om toestemming te verlenen aan deze webpagina.",dismissButtonAriaLabel:"Sluiten"},callMicrophoneAccessDeniedSafari:{title:"Geen toegang tot microfoon",message:"Vernieuw de pagina om machtigingen toe te staan of controleer de instellingen van deze browser en controleer of machtigingen zijn ingeschakeld voor deze website.",dismissButtonAriaLabel:"Sluiten"},callMicrophoneMutedBySystem:{title:"U bent gedempt door jouw systeem.",dismissButtonAriaLabel:"Sluiten"},callMicrophoneUnmutedBySystem:{title:"De microfoon is hersteld en het dempen van de microfoon is door het systeem opgeheven.",dismissButtonAriaLabel:"Sluiten"},callMacOsMicrophoneAccessDenied:{title:"Geen toegang tot microfoon",message:"Verleen microfoonmachtiging in de privacyinstellingen voor macOS.",dismissButtonAriaLabel:"Sluiten"},callLocalVideoFreeze:{title:"De netwerkbandbreedte is slecht",message:"Uw video wordt mogelijk onderbroken weergegeven voor anderen in het gesprek.",dismissButtonAriaLabel:"Sluiten"},callCameraAccessDenied:{title:"Geen toegang tot camera",message:"Klik op de vergrendeling in de adresbalk om toestemming te verlenen aan deze webpagina.",dismissButtonAriaLabel:"Sluiten"},callCameraAccessDeniedSafari:{title:"Geen toegang tot camera",message:"Vernieuw de pagina om machtigingen toe te staan of controleer de instellingen van deze browser en controleer of machtigingen zijn ingeschakeld voor deze website.",dismissButtonAriaLabel:"Sluiten"},callCameraAlreadyInUse:{title:"Geen toegang tot camera",message:"Het wordt mogelijk al gebruikt door een andere toepassing.",dismissButtonAriaLabel:"Sluiten"},callVideoStoppedBySystem:{title:"Uw video is gestopt door uw systeem.",dismissButtonAriaLabel:"Sluiten"},callVideoRecoveredBySystem:{title:"Uw video is hervat.",dismissButtonAriaLabel:"Sluiten"},callMacOsCameraAccessDenied:{title:"MacOS blokkeert de toegang tot uw camera",message:"Werk uw privacyinstellingen bij zodat deze browser toegang heeft tot uw camera.",dismissButtonAriaLabel:"Sluiten"},callMacOsScreenShareAccessDenied:{title:"Scherm delen wordt geblokkeerd door macOS",message:"Werk uw privacyinstellingen bij zodat deze browser uw scherm kan opnemen.",dismissButtonAriaLabel:"Sluiten"},failedToJoinCallGeneric:{title:"Deelnemen aan gesprek mislukt.",dismissButtonAriaLabel:"Sluiten"},failedToJoinCallInvalidMeetingLink:{title:"Kan niet deelnemen aan vergadering. Ongeldige koppeling.",dismissButtonAriaLabel:"Sluiten"},cameraFrozenForRemoteParticipants:{title:"Gebruikers in het gesprek ondervinden problemen bij het zien van uw video",message:"Controleer uw apparaten en netwerk.",dismissButtonAriaLabel:"Sluiten"},unableToStartVideoEffect:{title:"Kan video-effect niet toepassen.",dismissButtonAriaLabel:"Sluiten"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Kan spotlight niet starten op deelnemer(s)",message:"Het maximum aantal deelnemers is al gespotlight.",dismissButtonAriaLabel:"Sluiten"},mutedByRemoteParticipant:{title:"U bent gedempt door een andere deelnemer.",dismissButtonAriaLabel:"Sluiten"},recordingStarted:{title:"Opname is gestart.",message:"Door deel te nemen, geeft u toestemming voor het transcriberen van deze vergadering.",dismissButtonAriaLabel:"Sluiten"},transcriptionStarted:{title:"Transcriptie is gestart.",message:"Door deel te nemen, geeft u toestemming voor het transcriberen van deze vergadering.",dismissButtonAriaLabel:"Sluiten"},recordingStopped:{title:"De opname wordt opgeslagen.",message:"Opname is gestopt.",dismissButtonAriaLabel:"Sluiten"},transcriptionStopped:{title:"Transcriptie wordt opgeslagen.",message:"Transcriptie is gestopt.",dismissButtonAriaLabel:"Sluiten"},recordingAndTranscriptionStarted:{title:"Opname en transcriptie zijn gestart.",message:"Door deel te nemen, geeft u toestemming voor het transcriberen van deze vergadering.",dismissButtonAriaLabel:"Sluiten"},recordingAndTranscriptionStopped:{title:"Opname en transcriptie worden opgeslagen.",message:"Opname en transcriptie zijn gestopt.",dismissButtonAriaLabel:"Sluiten"},recordingStoppedStillTranscribing:{title:"Opname is gestopt.",message:"U bent nu alleen deze vergadering aan het transcriberen.",dismissButtonAriaLabel:"Sluiten"},transcriptionStoppedStillRecording:{title:"Transcriptie is gestopt.",message:"U bent nu alleen deze vergadering aan het opnemen.",dismissButtonAriaLabel:"Sluiten"},assignedBreakoutRoomOpened:{title:"U wordt over 5 seconden naar de toegewezen ruimte verplaatst.",dismissButtonAriaLabel:"Sluiten"},assignedBreakoutRoomChanged:{title:"Uw aparte vergaderruimte is gewijzigd",message:"U wordt over 5 seconden naar de toegewezen ruimte verplaatst.",dismissButtonAriaLabel:"Sluiten"},assignedBreakoutRoomClosed:{title:"Je aparte vergaderruimte is gesloten",message:"U wordt over 5 seconden terug naar de vergadering verplaatst.",dismissButtonAriaLabel:"Sluiten"},assignedBreakoutRoomOpenedPromptJoin:{title:"Deelnemen aan aparte vergaderruimte?",message:"U bent toegewezen aan een aparte vergaderruimte.",dismissButtonAriaLabel:"Sluiten",primaryButtonLabel:"Naar vergaderruimte",secondaryButtonLabel:"Later"},breakoutRoomJoined:{title:"U heeft deelgenomen aan een vergaderruimte.",dismissButtonAriaLabel:"Sluiten"},breakoutRoomClosingSoon:{title:"De tijdslimiet voor de ruimte verloopt binnenkort.",message:"Deze ruimte wordt in 30 seconden gesloten.",dismissButtonAriaLabel:"Sluiten"},capabilityTurnVideoOnPresent:{title:"Uw camera is ingeschakeld",message:"Schakel deze in als u video wilt delen.",dismissButtonAriaLabel:"Sluiten"},capabilityTurnVideoOnAbsent:{title:"Uw camera is uitgeschakeld",message:"U kunt geen video meer delen.",dismissButtonAriaLabel:"Sluiten"},capabilityUnmuteMicPresent:{title:"Uw microfoon is ingeschakeld",message:"Dempen opheffen als u wilt spreken.",dismissButtonAriaLabel:"Sluiten"},capabilityUnmuteMicAbsent:{title:"Uw microfoon is uitgeschakeld",message:"U kunt het dempen niet meer opheffen.",dismissButtonAriaLabel:"Sluiten"},togetherModeStarted:{title:"Togethermode is gestart",dismissButtonAriaLabel:"Sluiten"},togetherModeEnded:{title:"Togethermode is beëindigd",dismissButtonAriaLabel:"Sluiten"},transcriptionError:{title:"Er is iets misgegaan met de transcriptie.",message:"Transcriptie is gestopt. Probeer het later opnieuw",dismissButtonAriaLabel:"Sluiten"},transcriptionStartedByYou:{title:"Transcriptie is gestart.",message:"Gestart door jou. Laat iedereen weten dat ze worden getranscribeerd.",dismissButtonAriaLabel:"Sluiten"}};var videoGallery$7={screenIsBeingSharedMessage:"U deelt uw scherm",screenShareLoadingMessage:"Scherm van {participant} laden",localScreenShareLoadingMessage:"Uw scherm laden",localVideoLabel:"U",localVideoCameraSwitcherLabel:"Camera wisselen",localVideoMovementLabel:"Tegel voor verwisselbare lokale video",localVideoMovementAriaLabel:"Tegel voor verwisselbare lokale video",localVideoSelectedDescription:"{cameraName} geselecteerd",displayNamePlaceholder:"Naamloze deelnemer",fitRemoteParticipantToFrame:"Aanpassen aan frame",fillRemoteParticipantFrame:"Frame vullen",pinParticipantForMe:"Voor mij vastmaken",pinParticipantForMeLimitReached:"Pin (limiet bereikt)",unpinParticipantForMe:"Losmaken",pinParticipantMenuItemAriaLabel:"{participantName} vastmaken",unpinParticipantMenuItemAriaLabel:"{participantName} losmaken",pinnedParticipantAnnouncementAriaLabel:"{participantName} vastgemaakt",unpinnedParticipantAnnouncementAriaLabel:"{participantName} losgemaakt",startSpotlightVideoTileMenuLabel:"Spotlight voor iedereen",addSpotlightVideoTileMenuLabel:"Spotlight toevoegen",spotlightLimitReachedMenuTitle:"Spotlightlimiet bereikt",stopSpotlightVideoTileMenuLabel:"Spotlighten stoppen",stopSpotlightOnSelfVideoTileMenuLabel:"Spotlight afsluiten",attendeeRole:"Genodigde",muteParticipantMenuItemLabel:"Dempen",waitingScreenText:"Wachten tot anderen deelnemen",forbidAudio:"Microfoon uitschakelen",permitAudio:"Microfoon toestaan",forbidAudioTileMenuLabel:"Microfoon uitschakelen",permitAudioTileMenuLabel:"Microfoon toestaan",forbidVideo:"Camera uitschakelen",permitVideo:"Camera toestaan",forbidVideoTileMenuLabel:"Camera uitschakelen",permitVideoTileMenuLabel:"Camera toestaan"};var dialpad$7={placeholderText:"Telefoonnummer invoeren",deleteButtonAriaLabel:"Verwijderen"};var holdButton$7={onLabel:"Hervatten",offLabel:"Pauze",tooltipOnContent:"Gesprek hervatten",tooltipOffContent:"Gesprek in wachtrij plaatsen"};var videoTile$7={participantStateRinging:"Bellen...",participantReconnecting:"Opnieuw verbinding maken...",participantStateHold:"In de wacht",moreOptionsButtonAriaLabel:"Meer opties {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"gedempt",moreOptionsParticipantMutedStateUnmutedAriaLabel:"niet gedempt",moreOptionsParticipantHandRaisedAriaLabel:"Opgestoken hand",moreOptionsParticipantIsSpeakingAriaLabel:"Spreken",moreOptionsParticipantMicDisabledAriaLabel:"Microfoon uitgeschakeld",moreOptionsParticipantCameraDisabledAriaLabel:"Camera uitgeschakeld"};var CameraAndMicrophoneSitePermissionsRequest$7={primaryText:"Toestaan dat {appName} uw camera en microfoon gebruikt",secondaryText:"Dit is zodat deelnemers u kunnen zien en horen.",linkText:"Hulp nodig? Hulp bij probleemoplossing",primaryButtonText:"Doorgaan zonder camera en microfoon",ariaLabel:"Toegang tot camera en microfoon toestaan"};var CameraSitePermissionsRequest$7={primaryText:"{appName} toestaan om uw camera te gebruiken",secondaryText:"Dit is zodat deelnemers u kunnen zien.",linkText:"Hulp nodig? Hulp bij probleemoplossing",primaryButtonText:"Doorgaan zonder camera",ariaLabel:"Cameratoegang toestaan"};var MicrophoneSitePermissionsRequest$7={primaryText:"{appName} toestaan om uw microfoon te gebruiken",secondaryText:"Dit is zodat deelnemers u kunnen horen.",linkText:"Hulp nodig? Hulp bij probleemoplossing",primaryButtonText:"Doorgaan zonder microfoon",ariaLabel:"Microfoontoegang toestaan"};var CameraAndMicrophoneSitePermissionsCheck$7={primaryText:"Controleren op toegang tot camera en microfoon",secondaryText:"Toegang toestaan als hierom wordt gevraagd. Dit is zodat deelnemers u kunnen zien en horen.",linkText:"Hulp nodig? Hulp bij probleemoplossing",primaryButtonText:"Doorgaan zonder camera en microfoon",ariaLabel:"Controleren op toegang tot camera en microfoon. Toegang toestaan als hierom wordt gevraagd."};var CameraSitePermissionsCheck$7={primaryText:"Controleren op cameratoegang",secondaryText:"Toegang toestaan als hierom wordt gevraagd. Dit is zodat deelnemers u kunnen zien.",linkText:"Hulp nodig? Hulp bij probleemoplossing",primaryButtonText:"Doorgaan zonder camera",ariaLabel:"Controleren op cameratoegang. Toegang toestaan als hierom wordt gevraagd."};var MicrophoneSitePermissionsCheck$7={primaryText:"Controleren op toegang tot microfoon",secondaryText:"Toegang toestaan als hierom wordt gevraagd. Dit is zodat deelnemers u kunnen horen.",linkText:"Hulp nodig? Hulp bij probleemoplossing",primaryButtonText:"Doorgaan zonder microfoon",ariaLabel:"Controleren op toegang tot microfoon. Toegang toestaan als hierom wordt gevraagd."};var CameraAndMicrophoneSitePermissionsDenied$7={primaryText:"Geen toegang tot camera en microfoon",secondaryText:"Klik op het vergrendelingspictogram op de adresbalk om microfoonmachtigingen toe te kennen aan deze webpagina. Mogelijk is een paginavernieuwing vereist.",primaryButtonText:"Doorgaan zonder camera en microfoon",linkText:"Hulp nodig? Hulp bij probleemoplossing"};var CameraAndMicrophoneSitePermissionsDeniedSafari$7={primaryText:"Geen toegang tot camera en microfoon",secondaryText:"Vernieuw de pagina om machtigingen toe te staan of controleer de instellingen van deze browser en controleer of machtigingen zijn ingeschakeld voor deze website.",primaryButtonText:"Doorgaan zonder camera en microfoon",linkText:"Hulp nodig? Hulp bij probleemoplossing"};var CameraSitePermissionsDenied$7={primaryText:"Geen toegang tot camera",secondaryText:"Klik op het vergrendelingspictogram op de adresbalk om cameramachtigingen toe te kennen aan deze webpagina. Mogelijk is een paginavernieuwing vereist.",primaryButtonText:"Doorgaan zonder camera",linkText:"Hulp nodig? Hulp bij probleemoplossing"};var MicrophoneSitePermissionsDenied$7={primaryText:"Geen toegang tot microfoon",secondaryText:"Klik op het vergrendelingspictogram op de adresbalk om microfoonmachtigingen toe te kennen aan deze webpagina. Mogelijk is een paginavernieuwing vereist.",primaryButtonText:"Doorgaan zonder microfoon",linkText:"Hulp nodig? Hulp bij probleemoplossing"};var CameraSitePermissionsDeniedSafari$7={primaryText:"Geen toegang tot camera",secondaryText:"Vernieuw de pagina om machtigingen toe te staan of controleer de instellingen van deze browser en controleer of machtigingen zijn ingeschakeld voor deze website.",primaryButtonText:"Doorgaan zonder camera",linkText:"Hulp nodig? Hulp bij probleemoplossing"};var MicrophoneSitePermissionsDeniedSafari$7={primaryText:"Geen toegang tot microfoon",secondaryText:"Vernieuw de pagina om machtigingen toe te staan of controleer de instellingen van deze browser en controleer of machtigingen zijn ingeschakeld voor deze website.",primaryButtonText:"Doorgaan zonder microfoon",linkText:"Hulp nodig? Hulp bij probleemoplossing"};var UnsupportedBrowser$7={primaryText:"Browser wordt niet ondersteund",secondaryText:"Neem deel aan dit gesprek via een compatibele browser.",moreHelpLinkText:"Compatibiliteitsvereisten bekijken"};var UnsupportedBrowserVersion$7={primaryText:"Browserupdate vereist",secondaryText:"Werk uw browser bij en probeer opnieuw deel te nemen aan het gesprek om ervoor te zorgen dat u het beste kunt bellen.",moreHelpLinkText:"Compatibiliteitsvereisten bekijken",continueAnywayButtonText:"Oproep starten zonder bij te werken"};var UnsupportedOperatingSystem$7={primaryText:"Besturingssysteem niet ondersteund",secondaryText:"Neem deel aan dit gesprek via een apparaat met een compatibel besturingssysteem.",moreHelpLinkText:"Compatibiliteitsvereisten bekijken"};var BrowserPermissionDenied$7={primaryText:"Kan uw camera of microfoon niet gebruiken",secondaryText:"Uw browser heeft mogelijk geen toegang tot uw camera of microfoon. Open Systeemvoorkeuren om dit op te lossen.",primaryButtonText:"Opnieuw proberen",linkText:"Hulp nodig? Hulp bij probleemoplossing"};var BrowserPermissionDeniedIOS$7={primaryText:"Microfoontoegang toestaan om door te gaan",secondaryText:"Zodat andere deelnemers u kunnen horen.",primaryButtonText:"Opnieuw proberen",imageAltText:"Machtigingslocatie voor microfoon en cameraapparaat voor iOS",linkText:"Hulp nodig? Hulp bij probleemoplossing",step1Text:"Ga naar de app Instellingen",step2Text:"Schuif omlaag naar instellingen voor deze browser",step3Text:"Microfoon inschakelen (camera optioneel)",step4Text:"Probeer opnieuw deel te nemen aan het gesprek",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$7={leftNavButtonAriaLabel:"vorige pagina {current} van {total}",rightNavButtonAriaLabel:"volgende pagina {current} van {total}"};var meetingConferencePhoneInfo$7={meetingConferencePhoneInfoModalTitle:"Deelnemen aan gesprek vanaf uw telefoon",meetingConferencePhoneInfoModalDialIn:"Kiesnummer",meetingConferencePhoneInfoModalMeetingId:"Vergadering-id invoeren",meetingConferencePhoneInfoModalWait:"Wachten om toegelaten te worden door organisator",meetingConferencePhoneInfoModalNoPhoneAvailable:"Geen telefoon beschikbaar",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Gratis)",meetingConferencePhoneInfoModalToll:"(niet-gratis)"};var IncomingCallNotification$7={incomingCallNotificationPlaceholderId:"Onbekende gebruiker",incomingCallNotificationPlaceholderAlert:"{callerName} belt",incomingCallNoticicationAcceptWithAudioAriaLabel:"Accepteren met geluid",incomingCallNoticicationAcceptWithVideoAriaLabel:"Accepteren met video",incomingCallNoticicationRejectAriaLabel:"Weigeren",incomingCallNotificationAcceptButtonLabel:"Accepteren",incomingCallNotificationRejectButtonLabel:"Weigeren",incomingCallNotificationAccceptWithVideoButtonLabel:"Accepteren met video",incomingCallNotificationDismissButtonAriaLabel:"Sluiten"};var nl_NL$1 = {participantItem:participantItem$7,ParticipantList:ParticipantList$7,typingIndicator:typingIndicator$7,sendBox:sendBox$7,richTextSendBox:richTextSendBox$7,realTimeTextModal:realTimeTextModal$7,captionsBanner:captionsBanner$7,mentionPopover:mentionPopover$7,imageOverlay:imageOverlay$7,messageStatusIndicator:messageStatusIndicator$7,endCallButton:endCallButton$7,cameraButton:cameraButton$7,microphoneButton:microphoneButton$7,devicesButton:devicesButton$7,participantsButton:participantsButton$7,screenShareButton:screenShareButton$7,raiseHandButton:raiseHandButton$7,reactionButton:reactionButton$7,realTimeText:realTimeText$7,captionsSettingsModal:captionsSettingsModal$7,spokenLanguages:spokenLanguages$7,captionLanguages:captionLanguages$7,startCaptionsButton:startCaptionsButton$7,startRealTimeTextButton:startRealTimeTextButton$7,messageThread:messageThread$7,errorBar:errorBar$7,notificationStack:notificationStack$7,videoGallery:videoGallery$7,dialpad:dialpad$7,holdButton:holdButton$7,videoTile:videoTile$7,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$7,CameraSitePermissionsRequest:CameraSitePermissionsRequest$7,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$7,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$7,CameraSitePermissionsCheck:CameraSitePermissionsCheck$7,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$7,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$7,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$7,CameraSitePermissionsDenied:CameraSitePermissionsDenied$7,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$7,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$7,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$7,UnsupportedBrowser:UnsupportedBrowser$7,UnsupportedBrowserVersion:UnsupportedBrowserVersion$7,UnsupportedOperatingSystem:UnsupportedOperatingSystem$7,BrowserPermissionDenied:BrowserPermissionDenied$7,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$7,verticalGallery:verticalGallery$7,meetingConferencePhoneInfo:meetingConferencePhoneInfo$7,IncomingCallNotification:IncomingCallNotification$7};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Dutch (Netherlands).
 *
 * @public
 */
const COMPONENT_LOCALE_NL_NL = {
    strings: createComponentStrings(nl_NL$1)
};

var participantItem$6={isMeText:"(Ty)",menuTitle:"Więcej opcji",removeButtonLabel:"Usuń",sharingIconLabel:"Udostępnianie",mutedIconLabel:"Wyciszono",handRaisedIconLabel:"Podniesiona ręka — położenie {position}",displayNamePlaceholder:"Uczestnik bez nazwy",participantStateRinging:"Trwa łączenie…",participantStateHold:"Zawieszony",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, więcej opcji",attendeeRole:"Uczestnik",micDisabledIconLabel:"Mikrofon jest wyłączony",cameraDisabledIconLabel:"Kamera wyłączona"};var ParticipantList$6={overflowParticipantCount:"+{overflowCount} więcej"};var typingIndicator$6={singleUser:"Użytkownik {user} pisze...",multipleUsers:"{users} piszą...",multipleUsersAbbreviateOne:"{users} i 1 inny użytkownik piszą...",multipleUsersAbbreviateMany:"{users} i {numOthers} innych użytkowników piszą...",delimiter:", "};var sendBox$6={placeholderText:"Wprowadź wiadomość",textTooLong:"Długość wiadomości przekracza maksymalny limit.",sendButtonAriaLabel:"Wyślij wiadomość",attachmentUploadsPendingError:"Trwa przekazywanie pliku. Czekaj.",removeAttachment:"Usuń plik",uploading:"Przekazywanie",uploadCompleted:"Zakończono przekazywanie",attachmentMoreMenu:"Więcej opcji"};var richTextSendBox$6={placeholderText:"Wprowadź wiadomość",textTooLong:"Długość wiadomości przekracza maksymalny limit.",sendButtonAriaLabel:"Wyślij wiadomość",attachmentUploadsPendingError:"Trwa przekazywanie pliku. Czekaj.",imageUploadsPendingError:"Trwa przekazywanie obrazu. Czekaj.",removeAttachment:"Usuwanie pliku",uploading:"Przekazywanie",uploadCompleted:"Zakończono przekazywanie",richTextBoldTooltip:"Pogrubienie",richTextItalicTooltip:"Kursywa",richTextUnderlineTooltip:"Podkreślenie",richTextBulletListTooltip:"Lista punktowana",richTextNumberListTooltip:"Lista numerowana",richTextIncreaseIndentTooltip:"Zwiększ wcięcie",richTextDecreaseIndentTooltip:"Zmniejsz wcięcie",richTextInsertTableTooltip:"Wstaw tabelę",richTextFormatButtonTooltip:"Formatowanie",richTextInsertRowOrColumnMenu:"Wstaw",richTextInsertTableMenuTitle:"Wstaw tabelę {column} x {row}",richTextInsertRowAboveMenu:"Wstaw powyżej",richTextInsertRowBelowMenu:"Wstaw poniżej",richTextInsertColumnLeftMenu:"Wstaw z lewej",richTextInsertColumnRightMenu:"Wstaw z prawej",richTextDeleteRowOrColumnMenu:"Usuń",richTextDeleteColumnMenu:"Usuń kolumnę",richTextDeleteRowMenu:"Usuń wiersz",richTextDeleteTableMenu:"Usuń tabelę",richTextToolbarAriaLabel:"Pasek narzędzi formatowania wiadomości",richTextToolbarMoreButtonAriaLabel:"Więcej opcji",attachmentMoreMenu:"Więcej opcji",richTextNewBulletedListItemAnnouncement:"Punktor, nowy wiersz",richTextNewNumberedListItemAnnouncement:"{0}, nowy wiersz",richTextBulletedListAppliedAnnouncement:"Zastosowano listę punktowaną",richTextNumberedListAppliedAnnouncement:"Zastosowano listę numerowaną"};var realTimeTextModal$6={realTimeTextModalAriaLabel:"Włącz modalne okno dialogowe pisania w czasie rzeczywistym",realTimeTextModalTitle:"Włączyć pisanie w czasie rzeczywistym?",realTimeTextModalText:"Tekst w czasie rzeczywistym zostanie włączony dla wszystkich uczestników spotkania.",realTimeTextConfirmButtonLabel:"Włącz",realTimeTextCancelButtonLabel:"Anuluj",realTimeTextCloseModalButtonAriaLabel:"Zamknij modalne okno dialogowe pisania w czasie rzeczywistym"};var captionsBanner$6={captionsBannerSpinnerText:"Trwa uruchamianie napisów...",realTimeTextInputBoxDefaultText:"Pisz wiadomości w czasie rzeczywistym",realTimeTextInputErrorMessage:"Wiadomość nie może przekraczać 2000 znaków",realTimeTextBannerTitle:"Pisanie w czasie rzeczywistym",realTimeTextBannerContent:"Funkcja RTT (tekst w czasie rzeczywistym) jest włączona dla wszystkich uczestników przez cały czas trwania spotkania.",realTimeTextBannerLinkLabel:"Dowiedz się więcej",captionsOnlyContainerTitle:"Napisy",realTimeTextOnlyContainerTitle:"Pisanie w czasie rzeczywistym",captionsAndRealTimeTextContainerTitle:"Napisy i pisanie w czasie rzeczywistym",expandButtonAriaLabel:"Rozwiń",minimizeButtonAriaLabel:"Zwiń"};var mentionPopover$6={mentionPopoverHeader:"Sugestie"};var imageOverlay$6={downloadButtonLabel:"Pobierz",dismissButtonAriaLabel:"Zamknij",overlayVisibleMessage:"Nakładka szczegółów obrazu"};var messageStatusIndicator$6={deliveredAriaLabel:"Wiadomość została wysłana",deliveredTooltipText:"Wysłano",seenAriaLabel:"Wiadomość widziana przez inne osoby",seenTooltipText:"Wyświetlono",readByTooltipText:"Przeczytane przez {messageThreadReadCount} z {remoteParticipantsCount}",sendingAriaLabel:"Wysyłanie wiadomości",sendingTooltipText:"Wysyłanie",failedToSendAriaLabel:"Nie udało się wysłać wiadomości",failedToSendTooltipText:"Nie udało się wysłać"};var endCallButton$6={label:"Wyjdź",tooltipContent:"Opuść rozmowę",leaveOption:"Opuść",endCallOption:"Zakończ rozmowę dla wszystkich użytkowników"};var cameraButton$6={onLabel:"Kamera",offLabel:"Kamera",tooltipDisabledContent:"Kamera jest wyłączona",tooltipOnContent:"Wyłącz kamerę",tooltipOffContent:"Włącz kamerę",tooltipVideoLoadingContent:"Ładowanie wideo w toku",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Wybierz kamerę",cameraButtonSplitRoleDescription:"Przycisk podziału",onSplitButtonAriaLabel:"Wyłącz kamerę",offSplitButtonAriaLabel:"Włącz kamerę",cameraActionTurnedOnAnnouncement:"Kamera została włączona",cameraActionTurnedOffAnnouncement:"Kamera została wyłączona",onSplitButtonPrimaryActionCamera:"Wyłącz kamerę",offSplitButtonPrimaryActionCamera:"Włącz kamerę",cameraPrimaryActionSplitButtonTitle:"Użyj kamery",videoEffectsMenuItemTitle:"Efekty",cameraButtonAriaDescription:"opcje kamery"};var microphoneButton$6={onLabel:"Mikrofon",offLabel:"Mikrofon",tooltipDisabledContent:"Mikrofon jest wyłączony",tooltipOnContent:"Wycisz mikrofon",tooltipOffContent:"Wyłącz wyciszenie mikrofonu",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Wybierz mikrofon",speakerMenuTitle:"Głośnik",speakerMenuTooltip:"Wybierz głośnik",microphoneButtonSplitRoleDescription:"Przycisk podziału",onSplitButtonAriaLabel:"Wycisz mikrofon",offSplitButtonAriaLabel:"Wyłącz wyciszenie mikrofonu",microphoneActionTurnedOnAnnouncement:"Mikrofon został włączony",microphoneActionTurnedOffAnnouncement:"Mikrofon został wyłączony",offSplitButtonMicrophonePrimaryAction:"Wyłącz wyciszenie mikrofonu",onSplitButtonMicrophonePrimaryAction:"Wycisz mikrofon",microphonePrimaryActionSplitButtonTitle:"Użyj mikrofonu",microphoneAriaDescription:"Opcje dźwięku",deepNoiseSuppressionTitle:"Tłumienie szumu",deepNoiseSuppressionOnAnnouncement:"Głębokie tłumienie szumów zostało włączone",deepNoiseSuppressionOffAnnouncement:"Głębokie tłumienie szumów zostało wyłączone"};var devicesButton$6={label:"Urządzenia",tooltipContent:"Zarządzaj urządzeniami",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Wybierz kamerę",audioDeviceMenuTitle:"Urządzenie audio",audioDeviceMenuTooltip:"Wybierz urządzenie audio",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Wybierz mikrofon",speakerMenuTitle:"Głośnik",speakerMenuTooltip:"Wybierz głośnik"};var participantsButton$6={label:"Osoby",ariaLabel:"Osoby",tooltipContent:"Pokaż uczestników",menuHeader:"W tej rozmowie",participantsListButtonLabel:"Liczba osób: {numParticipants}",muteAllButtonLabel:"Wycisz wszystkich",copyInviteLinkButtonLabel:"Kopiuj link do zaproszenia",copyInviteLinkActionedAriaLabel:"Skopiowano link do zaproszenia"};var screenShareButton$6={onLabel:"Zakończ prezentację",offLabel:"Obecne",tooltipDisabledContent:"Prezentowanie jest wyłączone",tooltipOnContent:"Prezentowanie ekranu",tooltipOffContent:"Prezentuj swój ekran"};var raiseHandButton$6={offLabel:"Podnieś",onLabel:"Opuść",tooltipDisabledContent:"Akcja Podnieś rękę jest wyłączona",tooltipOnContent:"Opuść rękę",tooltipOffContent:"Podnieś rękę"};var reactionButton$6={label:"Zareaguj",ariaLabel:"Zareaguj",tooltipDisabledContent:"Akcja reakcji jest wyłączona",tooltipContent:"Wyślij reakcję",likeReactionTooltipContent:"Lubię",heartReactionTooltipContent:"Miłość",laughReactionTooltipContent:"Śmiech",applauseReactionTooltipContent:"Aplauz",surprisedReactionTooltipContent:"Niespodzianka"};var realTimeText$6={isTypingText:"Pisanie w czasie rzeczywistym"};var captionsSettingsModal$6={captionsSettingsModalTitle:"Ustawienia napisów",captionsSettingsSpokenLanguageDropdownLabel:"Język mówiony",captionsSettingsCaptionLanguageDropdownLabel:"Język napisów",captionsSettingsSpokenLanguageDropdownInfoText:"Język, w którym mówią wszyscy uczestnicy tej rozmowy.",captionsSettingsCaptionLanguageDropdownInfoText:"Napisy będą wyświetlane w tym języku.",captionsSettingsConfirmButtonLabel:"Potwierdź",captionsSettingsCancelButtonLabel:"Anuluj",captionsSettingsModalAriaLabel:"Modalne ustawianie podpisów",captionsSettingsCloseModalButtonAriaLabel:"Ustawienie podpisów"};var spokenLanguages$6={"ar-ae":"arabski — Zjednoczone Emiraty Arabskie","ar-sa":"arabski — Arabia Saudyjska","da-dk":"duński","de-de":"niemiecki — Niemcy","en-au":"angielski — Australia","en-ca":"angielski — Kanada","en-gb":"angielski — Zjednoczone Królestwo","en-in":"angielski — Indie","en-nz":"angielski — Nowa Zelandia","en-us":"angielski — Stany Zjednoczone","es-es":"hiszpański — Hiszpania (nowoczesny)","es-mx":"hiszpański — Meksyk","fi-fi":"fiński","fr-ca":"francuski — Kanada","fr-fr":"francuski — Francja","hi-in":"hindi","it-it":"włoski — Włochy","ja-jp":"japoński","ko-kr":"koreański","nb-no":"norweski (Bokmål)","nl-be":"niderlandzki — Belgia","nl-nl":"niderlandzki — Holandia","pl-pl":"polski","pt-br":"portugalski — Brazylia","ru-ru":"rosyjski","sv-se":"szwedzki","zh-cn":"chiński (Chińska Republika Ludowa)","zh-hk":"chiński — Hongkong SAR","cs-cz":"czeski","pt-pt":"portugalski — Portugalia","tr-tr":"turecki","vi-vn":"wietnamski","th-th":"tajski","he-il":"hebrajski","cy-gb":"walijski","uk-ua":"ukraiński","el-gr":"grecki","hu-hu":"węgierski","ro-ro":"rumuński","sk-sk":"słowacki","zh-tw":"chiński — Tajwan"};var captionLanguages$6={ar:"arabski",da:"duński",de:"niemiecki",en:"angielski",es:"hiszpański",fi:"fiński","fr-ca":"francuski — Kanada",fr:"francuski — Francja",hi:"hindi",it:"włoski",ja:"japoński",ko:"koreański",nb:"norweski (Bokmål)",nl:"holenderski",pl:"polski",pt:"portugalski — Brazylia",ru:"rosyjski",sv:"szwedzki","zh-Hans":"chiński (uproszczony)","zh-Hant":"chiński (tradycyjny)",cs:"czeski","pt-pt":"portugalski — Portugalia",tr:"turecki",vi:"wietnamski",th:"tajski",he:"hebrajski",cy:"walijski",uk:"ukraiński",el:"grecki",hu:"węgierski",ro:"rumuński",sk:"słowacki"};var startCaptionsButton$6={onLabel:"Włącz podpisy",offLabel:"Wyłącz podpisy",tooltipOnContent:"Wyłącz podpisy",tooltipOffContent:"Włącz podpisy"};var startRealTimeTextButton$6={label:"Włącz funkcję Pisanie w czasie rzeczywistym dla tego połączenia",tooltipOffContent:"Włącz funkcję Pisanie w czasie rzeczywistym dla tego połączenia"};var messageThread$6={yesterday:"Wczoraj",sunday:"Niedziela",monday:"Poniedziałek",tuesday:"Wtorek",wednesday:"Środa",thursday:"Czwartek",friday:"Piątek",saturday:"Sobota",participantJoined:"dołączył do czatu.",participantLeft:"opuścił czat.",editMessage:"Edytuj",removeMessage:"Usuń",resendMessage:"Spróbuj wysłać ponownie",failToSendTag:"Nie udało się wysłać",editedTag:"Edytowane",liveAuthorIntro:"Wiadomość od użytkownika {author}",editedMessageLiveAuthorIntro:"Edytowano wiadomość od użytkownika {author}",editedMessageLocalUserLiveAuthorIntro:"Edytowano wiadomość",messageContentAriaText:"{status} Użytkownik {author} powiedział: {message}",messageContentMineAriaText:"{status} Twoja wypowiedź: {message}",editBoxTextLimit:"Twoja wiadomość przekracza limit {limitNumber} znaków",editBoxPlaceholderText:"Edytuj wiadomość",newMessagesIndicator:"Nowe wiadomości",noDisplayNameSub:"Bez nazwy",editBoxCancelButton:"Anuluj",editBoxSubmitButton:"Gotowe",messageReadCount:"Przeczytane przez {messageReadByCount} z {remoteParticipantsCount}",actionMenuMoreOptions:"Więcej opcji",downloadAttachment:"Pobierz plik",openAttachment:"Otwórz plik",blockedWarningText:"Ta wiadomość została usunięta ze względu na zasady organizacyjne.",blockedWarningLinkText:"Szczegóły",attachmentCardGroupMessage:"Wiadomość ma {attachmentCount} załącznik",messageDeletedAnnouncementAriaLabel:"Wiadomość została usunięta",richTextBoldTooltip:"Pogrubienie",richTextItalicTooltip:"Kursywa",richTextUnderlineTooltip:"Podkreślenie",richTextBulletListTooltip:"Lista punktowana",richTextNumberListTooltip:"Lista numerowana",richTextIncreaseIndentTooltip:"Zwiększ wcięcie",richTextDecreaseIndentTooltip:"Zmniejsz wcięcie",richTextInsertTableTooltip:"Wstaw tabelę",richTextFormatButtonTooltip:"Formatowanie",richTextInsertRowOrColumnMenu:"Wstaw",richTextInsertTableMenuTitle:"Wstaw tabelę {column} x {row}",richTextInsertRowAboveMenu:"Wstaw powyżej",richTextInsertRowBelowMenu:"Wstaw poniżej",richTextInsertColumnLeftMenu:"Wstaw z lewej",richTextInsertColumnRightMenu:"Wstaw z prawej",richTextDeleteRowOrColumnMenu:"Usuń",richTextDeleteColumnMenu:"Usuń kolumnę",richTextDeleteRowMenu:"Usuń wiersz",richTextDeleteTableMenu:"Usuń tabelę",richTextToolbarMoreButtonAriaLabel:"Więcej opcji",attachmentUploadsPendingError:"Trwa przekazywanie pliku. Czekaj.",imageUploadsPendingError:"Trwa przekazywanie obrazu. Czekaj.",richTextNewBulletedListItemAnnouncement:"Punktor, nowy wiersz",richTextNewNumberedListItemAnnouncement:"{0}, nowy wiersz",richTextBulletedListAppliedAnnouncement:"Zastosowano listę punktowaną",richTextNumberedListAppliedAnnouncement:"Zastosowano listę numerowaną"};var errorBar$6={unableToReachChatService:"Jesteś w trybie offline",accessDenied:"Nie można uzyskać dostępu do usług czatu — sprawdź podane poświadczenia użytkownika",userNotInChatThread:"Nie jesteś już w tym wątku czatu",sendMessageNotInChatThread:"Nie można wysłać wiadomości, ponieważ nie jesteś już w tym wątku czatu",sendMessageGeneric:"Nie udało się wysłać wiadomości",callingNetworkFailure:"Problem z nawiązaniem połączenia — wygląda na to, że jesteś w trybie offline",startVideoGeneric:"Nie można uruchomić wideo",stopVideoGeneric:"Nie można zatrzymać wideo",muteGeneric:"Nie można wyciszyć mikrofonu",unmuteGeneric:"Nie można wyłączyć wyciszenia mikrofonu",speakingWhileMuted:"Mikrofon jest wyciszony",startScreenShareGeneric:"Wystąpił problem podczas uruchamiania udostępniania ekranu.",stopScreenShareGeneric:"Nie można zatrzymać udostępniania ekranu",callNetworkQualityLow:"Niska jakość sieci.",teamsMeetingCallNetworkQualityLow:"Niska jakość sieci.",callNoSpeakerFound:"Nie znaleziono głośników ani słuchawek. Podłącz urządzenie audio, aby słyszeć rozmowę.",callNoMicrophoneFound:"Nie znaleziono mikrofonów. Podłącz urządzenie wejściowe audio.",callMicrophoneAccessDenied:"Nie można uzyskać dostępu do mikrofonu. Kliknij blokadę na pasku adresu, aby udzielić uprawnienia do tej strony sieci Web.",callMicrophoneAccessDeniedSafari:"Nie można uzyskać dostępu do mikrofonu. Odśwież stronę, aby zezwolić na uprawnienia, lub sprawdź ustawienia tej przeglądarki i sprawdź, czy uprawnienia są włączone dla tej witryny internetowej.",callMicrophoneMutedBySystem:"System wyciszył Cię.",callMicrophoneUnmutedBySystem:"Mikrofon został odzyskany i system wyłączył wyciszenie.",callMacOsMicrophoneAccessDenied:"Nie można uzyskać dostępu do mikrofonu. Udziel uprawnienia do mikrofonu w ustawieniach prywatności systemu macOS.",callLocalVideoFreeze:"Przepustowość sieci jest słaba. Twoje wideo może być wstrzymane dla innych osób podczas rozmowy.",callCameraAccessDenied:"Nie można uzyskać dostępu do kamery. Kliknij blokadę na pasku adresu, aby udzielić uprawnienia do tej strony sieci Web.",callCameraAccessDeniedSafari:"Nie można uzyskać dostępu do kamery. Odśwież stronę, aby zezwolić na uprawnienia, lub sprawdź ustawienia tej przeglądarki i sprawdź, czy uprawnienia są włączone dla tej witryny internetowej.",callCameraAlreadyInUse:"Nie można uzyskać dostępu do kamery. Być może jest już używany przez inną aplikację.",callVideoStoppedBySystem:"Wideo zostało zatrzymane przez system.",callVideoRecoveredBySystem:"Wideo zostało wznowione.",callMacOsCameraAccessDenied:"System MacOS blokuje dostęp do kamery. Zaktualizuj ustawienia prywatności, aby zezwolić tej przeglądarce na dostęp do kamery.",callMacOsScreenShareAccessDenied:"System MacOS blokuje udostępnianie ekranu. Zaktualizuj ustawienia prywatności, aby zezwolić tej przeglądarce na rejestrowanie ekranu.",dismissButtonAriaLabel:"Zamknij",failedToJoinCallGeneric:"Nie można dołączyć do rozmowy.",failedToJoinCallInvalidMeetingLink:"Nie można dołączyć do spotkania. Nieprawidłowy link.",cameraFrozenForRemoteParticipants:"Użytkownicy w trakcie rozmowy mają problemy z wyświetleniem Twojego wideo. Sprawdź urządzenia i sieć.",unableToStartVideoEffect:"Nie można zastosować efektu wideo.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Nie można rozpocząć wyróżnienia uczestników, ponieważ maksymalna liczba uczestników jest już wyróżniona.",mutedByRemoteParticipant:"Inny uczestnik wyciszył Cię."};var notificationStack$6={callingNetworkFailure:{title:"Problem z nawiązaniem połączenia",message:"Wygląda na to, że jesteś w trybie offline. Sprawdź połączenie sieciowe i spróbuj ponownie.",dismissButtonAriaLabel:"Zamknij"},startVideoGeneric:{title:"Nie można uruchomić wideo.",dismissButtonAriaLabel:"Zamknij"},stopVideoGeneric:{title:"Nie można zatrzymać wideo.",dismissButtonAriaLabel:"Zamknij"},muteGeneric:{title:"Nie można wyciszyć mikrofonu.",dismissButtonAriaLabel:"Zamknij"},unmuteGeneric:{title:"Nie można wyłączyć wyciszenia mikrofonu.",dismissButtonAriaLabel:"Zamknij"},speakingWhileMuted:{title:"Wyciszenie jest włączone.",dismissButtonAriaLabel:"Zamknij"},startScreenShareGeneric:{title:"Wystąpił problem podczas uruchamiania udostępniania ekranu.",dismissButtonAriaLabel:"Zamknij"},stopScreenShareGeneric:{title:"Nie można zatrzymać udostępniania ekranu.",dismissButtonAriaLabel:"Zamknij"},callNetworkQualityLow:{title:"Niska jakość sieci.",dismissButtonAriaLabel:"Zamknij"},teamsMeetingCallNetworkQualityLow:{title:"Niska jakość sieci.",dismissButtonAriaLabel:"Zamknij",message:"Dołącz do tej rozmowy z telefonu, aby uzyskać lepszy dźwięk. Możesz kontynuować wyświetlanie spotkania na tym urządzeniu.",primaryButtonLabel:"Dołącz przez telefon"},callNoSpeakerFound:{title:"Nie odnaleziono głośników ani słuchawek",message:"Podłącz urządzenie audio, aby usłyszeć rozmowę.",dismissButtonAriaLabel:"Zamknij"},callNoMicrophoneFound:{title:"Nie znaleziono mikrofonów",message:"Podłącz urządzenie wejściowe dźwięku.",dismissButtonAriaLabel:"Zamknij"},callMicrophoneAccessDenied:{title:"Brak dostępu do mikrofonu",message:"Kliknij blokadę na pasku adresu, aby udzielić uprawnienia do tej strony sieci Web.",dismissButtonAriaLabel:"Zamknij"},callMicrophoneAccessDeniedSafari:{title:"Brak dostępu do mikrofonu",message:"Odśwież stronę, aby zezwolić na uprawnienia, lub sprawdź ustawienia tej przeglądarki i sprawdź, czy uprawnienia są włączone dla tej witryny internetowej.",dismissButtonAriaLabel:"Zamknij"},callMicrophoneMutedBySystem:{title:"System wyciszył Cię.",dismissButtonAriaLabel:"Zamknij"},callMicrophoneUnmutedBySystem:{title:"Mikrofon został odzyskany i system wyłączył wyciszenie.",dismissButtonAriaLabel:"Zamknij"},callMacOsMicrophoneAccessDenied:{title:"Brak dostępu do mikrofonu",message:"Udziel uprawnienia do mikrofonu w ustawieniach prywatności systemu macOS.",dismissButtonAriaLabel:"Zamknij"},callLocalVideoFreeze:{title:"Przepustowość sieci jest słaba",message:"Twoje wideo może być wstrzymane dla innych osób podczas rozmowy.",dismissButtonAriaLabel:"Zamknij"},callCameraAccessDenied:{title:"Brak dostępu do kamery",message:"Kliknij blokadę na pasku adresu, aby udzielić uprawnienia do tej strony sieci Web.",dismissButtonAriaLabel:"Zamknij"},callCameraAccessDeniedSafari:{title:"Brak dostępu do kamery",message:"Odśwież stronę, aby zezwolić na uprawnienia, lub sprawdź ustawienia tej przeglądarki i sprawdź, czy uprawnienia są włączone dla tej witryny internetowej.",dismissButtonAriaLabel:"Zamknij"},callCameraAlreadyInUse:{title:"Brak dostępu do kamery",message:"Być może jest już używany przez inną aplikację.",dismissButtonAriaLabel:"Zamknij"},callVideoStoppedBySystem:{title:"Wideo zostało zatrzymane przez system.",dismissButtonAriaLabel:"Zamknij"},callVideoRecoveredBySystem:{title:"Wideo zostało wznowione.",dismissButtonAriaLabel:"Zamknij"},callMacOsCameraAccessDenied:{title:"System MacOS blokuje dostęp do kamery",message:"Zaktualizuj ustawienia prywatności, aby zezwolić tej przeglądarce na dostęp do kamery.",dismissButtonAriaLabel:"Zamknij"},callMacOsScreenShareAccessDenied:{title:"System MacOS blokuje udostępnianie ekranu",message:"Zaktualizuj ustawienia prywatności, aby zezwolić tej przeglądarce na rejestrowanie ekranu.",dismissButtonAriaLabel:"Zamknij"},failedToJoinCallGeneric:{title:"Nie można dołączyć do rozmowy.",dismissButtonAriaLabel:"Zamknij"},failedToJoinCallInvalidMeetingLink:{title:"Nie można dołączyć do spotkania. Nieprawidłowy link.",dismissButtonAriaLabel:"Zamknij"},cameraFrozenForRemoteParticipants:{title:"Użytkownicy w trakcie rozmowy mają problemy z wyświetleniem Twojego wideo",message:"Sprawdź urządzenia i sieć.",dismissButtonAriaLabel:"Zamknij"},unableToStartVideoEffect:{title:"Nie można zastosować efektu wideo.",dismissButtonAriaLabel:"Zamknij"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Nie można rozpocząć wyróżniania uczestników",message:"Maksymalna liczba uczestników jest już wyróżniona.",dismissButtonAriaLabel:"Zamknij"},mutedByRemoteParticipant:{title:"Inny uczestnik wyciszył Cię.",dismissButtonAriaLabel:"Zamknij"},recordingStarted:{title:"Rozpoczęto nagrywanie.",message:"Dołączając, wyrażasz zgodę na transkrypcję tego spotkania.",dismissButtonAriaLabel:"Zamknij"},transcriptionStarted:{title:"Rozpoczęto transkrypcję.",message:"Dołączając, wyrażasz zgodę na transkrypcję tego spotkania.",dismissButtonAriaLabel:"Zamknij"},recordingStopped:{title:"Trwa zapisywanie nagrania.",message:"Zatrzymano nagrywanie.",dismissButtonAriaLabel:"Zamknij"},transcriptionStopped:{title:"Trwa zapisywanie transkrypcji.",message:"Zakończono transkrypcję.",dismissButtonAriaLabel:"Zamknij"},recordingAndTranscriptionStarted:{title:"Rozpoczęto nagrywanie i transkrypcję.",message:"Dołączając, wyrażasz zgodę na transkrypcję tego spotkania.",dismissButtonAriaLabel:"Zamknij"},recordingAndTranscriptionStopped:{title:"Trwa zapisywanie nagrania i transkrypcji.",message:"Zatrzymano nagrywanie i transkrypcję.",dismissButtonAriaLabel:"Zamknij"},recordingStoppedStillTranscribing:{title:"Zatrzymano nagrywanie.",message:"W tej chwili tylko Ty transkrybujesz to spotkanie.",dismissButtonAriaLabel:"Zamknij"},transcriptionStoppedStillRecording:{title:"Zakończono transkrypcję.",message:"W tej chwili tylko Ty nagrywasz to spotkanie.",dismissButtonAriaLabel:"Zamknij"},assignedBreakoutRoomOpened:{title:"Przeniesiemy Cię do przypisanego pokoju w ciągu 5 sekund.",dismissButtonAriaLabel:"Zamknij"},assignedBreakoutRoomChanged:{title:"Twój osobny pokój został zmieniony",message:"Przeniesiemy Cię do przypisanego pokoju w ciągu 5 sekund.",dismissButtonAriaLabel:"Zamknij"},assignedBreakoutRoomClosed:{title:"Osobny pokój został zamknięty",message:"Przeniesiemy Cię z powrotem do spotkania za 5 sekund.",dismissButtonAriaLabel:"Zamknij"},assignedBreakoutRoomOpenedPromptJoin:{title:"Dołączyć do osobnego pokoju?",message:"Przypisano Cię do osobnego pokoju.",dismissButtonAriaLabel:"Zamknij",primaryButtonLabel:"Dołącz do pokoju",secondaryButtonLabel:"Później"},breakoutRoomJoined:{title:"Dołączono do osobnego pokoju.",dismissButtonAriaLabel:"Zamknij"},breakoutRoomClosingSoon:{title:"Limit czasu pokoju wkrótce wygaśnie.",message:"Ten pokój zostanie zamknięty za 30 sekund",dismissButtonAriaLabel:"Zamknij"},capabilityTurnVideoOnPresent:{title:"Twoja kamera została włączona",message:"Włącz to, jeśli chcesz udostępnić wideo.",dismissButtonAriaLabel:"Zamknij"},capabilityTurnVideoOnAbsent:{title:"Twoja kamera została wyłączona",message:"Nie można już udostępniać wideo.",dismissButtonAriaLabel:"Zamknij"},capabilityUnmuteMicPresent:{title:"Twój mikrofon został włączony",message:"Wyłącz wyciszenie, jeśli chcesz przemawiać.",dismissButtonAriaLabel:"Zamknij"},capabilityUnmuteMicAbsent:{title:"Twój mikrofon został wyłączony",message:"Nie możesz już wyłączać wyciszenia.",dismissButtonAriaLabel:"Zamknij"},togetherModeStarted:{title:"Tryb Togethermode został uruchomiony",dismissButtonAriaLabel:"Zamknij"},togetherModeEnded:{title:"Tryb Togethermode został zakończony",dismissButtonAriaLabel:"Zamknij"},transcriptionError:{title:"Wystąpił problem z transkrypcją.",message:"Transkrypcja została zatrzymana. Spróbuj ponownie później",dismissButtonAriaLabel:"Zamknij"},transcriptionStartedByYou:{title:"Rozpoczęto transkrypcję.",message:"Rozpoczęte przez Ciebie. Poinformuj wszystkich, że są transkrybowani.",dismissButtonAriaLabel:"Zamknij"}};var videoGallery$6={screenIsBeingSharedMessage:"Udostępniasz ekran",screenShareLoadingMessage:"Ładowanie ekranu uczestnika {participant}",localScreenShareLoadingMessage:"Ładowanie ekranu",localVideoLabel:"Ty",localVideoCameraSwitcherLabel:"Przełącz kamerę",localVideoMovementLabel:"Ruchomy lokalny kafelek wideo",localVideoMovementAriaLabel:"Ruchomy lokalny kafelek wideo",localVideoSelectedDescription:"Wybrano kamerę {cameraName}",displayNamePlaceholder:"Uczestnik bez nazwy",fitRemoteParticipantToFrame:"Dopasuj do ramki",fillRemoteParticipantFrame:"Wypełnij ramkę",pinParticipantForMe:"Przypnij dla mnie",pinParticipantForMeLimitReached:"Przypięcie (osiągnięto limit)",unpinParticipantForMe:"Odepnij",pinParticipantMenuItemAriaLabel:"Przypnij uczestnika {participantName}",unpinParticipantMenuItemAriaLabel:"Odepnij uczestnika {participantName}",pinnedParticipantAnnouncementAriaLabel:"Przypięty uczestnik {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Odpięty uczestnik {participantName}",startSpotlightVideoTileMenuLabel:"Wyróżnij dla wszystkich",addSpotlightVideoTileMenuLabel:"Dodaj wyróżnienie",spotlightLimitReachedMenuTitle:"Osiągnięto limit wyróżnienia",stopSpotlightVideoTileMenuLabel:"Zatrzymaj wyróżnianie",stopSpotlightOnSelfVideoTileMenuLabel:"Zakończ wyróżnienie",attendeeRole:"Uczestnik",muteParticipantMenuItemLabel:"Wycisz",waitingScreenText:"Oczekiwanie na dołączenie innych osób",forbidAudio:"Wyłącz mikrofon",permitAudio:"Zezwalaj na mikrofon",forbidAudioTileMenuLabel:"Wyłącz mikrofon",permitAudioTileMenuLabel:"Zezwalaj na mikrofon",forbidVideo:"Wyłącz kamerę",permitVideo:"Zezwalaj na kamerę",forbidVideoTileMenuLabel:"Wyłącz kamerę",permitVideoTileMenuLabel:"Zezwalaj na kamerę"};var dialpad$6={placeholderText:"Wprowadź numer telefonu",deleteButtonAriaLabel:"Usuń"};var holdButton$6={onLabel:"Wznów",offLabel:"Zawieś",tooltipOnContent:"Wznów połączenie",tooltipOffContent:"Zawieś połączenie"};var videoTile$6={participantStateRinging:"Trwa łączenie…",participantReconnecting:"Trwa ponowne łączenie...",participantStateHold:"Zawieszony",moreOptionsButtonAriaLabel:"Więcej opcji {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"wyciszony",moreOptionsParticipantMutedStateUnmutedAriaLabel:"wyciszenie wyłączone",moreOptionsParticipantHandRaisedAriaLabel:"Podniesiona ręka",moreOptionsParticipantIsSpeakingAriaLabel:"Mówi",moreOptionsParticipantMicDisabledAriaLabel:"Mikrofon jest wyłączony",moreOptionsParticipantCameraDisabledAriaLabel:"Kamera wyłączona"};var CameraAndMicrophoneSitePermissionsRequest$6={primaryText:"Zezwalaj {appName} na używanie kamery i mikrofonu",secondaryText:"Dzięki temu uczestnicy będą mogli Cię zobaczyć i usłyszeć.",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów",primaryButtonText:"Kontynuuj bez kamery i mikrofonu",ariaLabel:"Zezwalaj na dostęp do kamery i mikrofonu"};var CameraSitePermissionsRequest$6={primaryText:"Zezwalaj aplikacji {appName} na korzystanie z kamery",secondaryText:"Dzięki temu uczestnicy będą mogli Cię zobaczyć.",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów",primaryButtonText:"Kontynuuj bez kamery",ariaLabel:"Zezwalaj na dostęp do kamery"};var MicrophoneSitePermissionsRequest$6={primaryText:"Zezwalaj aplikacji {appName} na używanie mikrofonu",secondaryText:"Dzięki temu uczestnicy będą mogli Cię usłyszeć.",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów",primaryButtonText:"Kontynuuj bez mikrofonu",ariaLabel:"Zezwalaj na dostęp do mikrofonu"};var CameraAndMicrophoneSitePermissionsCheck$6={primaryText:"Sprawdzanie dostępu do kamery i mikrofonu",secondaryText:"Zezwalaj na dostęp, jeśli zostanie wyświetlony monit. Dzięki temu uczestnicy będą mogli Cię zobaczyć i usłyszeć.",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów",primaryButtonText:"Kontynuuj bez kamery i mikrofonu",ariaLabel:"Sprawdzanie dostępu do kamery i mikrofonu. Zezwalaj na dostęp, jeśli zostanie wyświetlony monit."};var CameraSitePermissionsCheck$6={primaryText:"Sprawdzanie dostępu do kamery",secondaryText:"Zezwalaj na dostęp, jeśli zostanie wyświetlony monit. Dzięki temu uczestnicy będą mogli Cię zobaczyć.",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów",primaryButtonText:"Kontynuuj bez kamery",ariaLabel:"Sprawdzanie dostępu do kamery. Zezwalaj na dostęp, jeśli zostanie wyświetlony monit."};var MicrophoneSitePermissionsCheck$6={primaryText:"Sprawdzanie dostępu do mikrofonu",secondaryText:"Zezwalaj na dostęp, jeśli zostanie wyświetlony monit. Dzięki temu uczestnicy będą mogli Cię usłyszeć.",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów",primaryButtonText:"Kontynuuj bez mikrofonu",ariaLabel:"Sprawdzanie dostępu do mikrofonu. Zezwalaj na dostęp, jeśli zostanie wyświetlony monit."};var CameraAndMicrophoneSitePermissionsDenied$6={primaryText:"Brak dostępu do kamery i mikrofonu",secondaryText:"Kliknij ikonę blokady na pasku adresu, aby udzielić uprawnień mikrofonu do tej strony sieci Web. Odświeżenie strony może być wymagane.",primaryButtonText:"Kontynuuj bez kamery i mikrofonu",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów"};var CameraAndMicrophoneSitePermissionsDeniedSafari$6={primaryText:"Brak dostępu do kamery i mikrofonu",secondaryText:"Odśwież stronę, aby zezwolić na uprawnienia, lub sprawdź ustawienia tej przeglądarki i sprawdź, czy uprawnienia są włączone dla tej witryny internetowej.",primaryButtonText:"Kontynuuj bez kamery i mikrofonu",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów"};var CameraSitePermissionsDenied$6={primaryText:"Brak dostępu do kamery",secondaryText:"Kliknij ikonę blokady na pasku adresu, aby udzielić uprawnień kamery do tej strony sieci Web. Odświeżenie strony może być wymagane.",primaryButtonText:"Kontynuuj bez kamery",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów"};var MicrophoneSitePermissionsDenied$6={primaryText:"Brak dostępu do mikrofonu",secondaryText:"Kliknij ikonę blokady na pasku adresu, aby udzielić uprawnień mikrofonu do tej strony sieci Web. Odświeżenie strony może być wymagane.",primaryButtonText:"Kontynuuj bez mikrofonu",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów"};var CameraSitePermissionsDeniedSafari$6={primaryText:"Brak dostępu do kamery",secondaryText:"Odśwież stronę, aby zezwolić na uprawnienia, lub sprawdź ustawienia tej przeglądarki i sprawdź, czy uprawnienia są włączone dla tej witryny internetowej.",primaryButtonText:"Kontynuuj bez kamery",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów"};var MicrophoneSitePermissionsDeniedSafari$6={primaryText:"Brak dostępu do mikrofonu",secondaryText:"Odśwież stronę, aby zezwolić na uprawnienia, lub sprawdź ustawienia tej przeglądarki i sprawdź, czy uprawnienia są włączone dla tej witryny internetowej.",primaryButtonText:"Kontynuuj bez mikrofonu",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów"};var UnsupportedBrowser$6={primaryText:"Nieobsługiwana przeglądarka",secondaryText:"Dołącz do tej rozmowy przy użyciu zgodnej przeglądarki.",moreHelpLinkText:"Zobacz wymagania dotyczące zgodności"};var UnsupportedBrowserVersion$6={primaryText:"Wymagana aktualizacja przeglądarki",secondaryText:"Aby zapewnić najlepszą możliwą rozmowę, zaktualizuj przeglądarkę, a następnie spróbuj ponownie dołączyć do rozmowy.",moreHelpLinkText:"Zobacz wymagania dotyczące zgodności",continueAnywayButtonText:"Rozpocznij rozmowę bez aktualizowania"};var UnsupportedOperatingSystem$6={primaryText:"System operacyjny nie jest obsługiwany",secondaryText:"Dołącz do tej rozmowy przy użyciu urządzenia ze zgodnym systemem operacyjnym.",moreHelpLinkText:"Zobacz wymagania dotyczące zgodności"};var BrowserPermissionDenied$6={primaryText:"Nie można użyć kamery lub mikrofonu",secondaryText:"Twoja przeglądarka może nie mieć dostępu do kamery lub mikrofonu. Aby rozwiązać ten problem, otwórz okno Preferencje systemowe.",primaryButtonText:"Spróbuj ponownie",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów"};var BrowserPermissionDeniedIOS$6={primaryText:"Zezwalaj na dostęp do mikrofonu, aby kontynuować",secondaryText:"Inni uczestnicy będą mogli Cię usłyszeć.",primaryButtonText:"Spróbuj ponownie",imageAltText:"Lokalizacja uprawnień urządzenia z mikrofonem i kamerą dla systemu iOS",linkText:"Potrzebujesz pomocy? Uzyskaj pomoc dotyczącą rozwiązywania problemów",step1Text:"Przejdź do aplikacji Ustawienia",step2Text:"Przewiń w dół do ustawień dla tej przeglądarki",step3Text:"Włącz mikrofon (opcjonalnie kamerę)",step4Text:"Spróbuj ponownie dołączyć do rozmowy",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$6={leftNavButtonAriaLabel:"poprzednia strona {current} z {total}",rightNavButtonAriaLabel:"następna strona {current} z {total}"};var meetingConferencePhoneInfo$6={meetingConferencePhoneInfoModalTitle:"Dołącz do połączenia z Twojego telefonu",meetingConferencePhoneInfoModalDialIn:"Wybierz numer",meetingConferencePhoneInfoModalMeetingId:"Wprowadź identyfikator spotkania",meetingConferencePhoneInfoModalWait:"Zaczekaj na wpuszczenie przez organizatora",meetingConferencePhoneInfoModalNoPhoneAvailable:"Brak dostępnego telefonu",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Bezpłatny)",meetingConferencePhoneInfoModalToll:"(Płatny)"};var IncomingCallNotification$6={incomingCallNotificationPlaceholderId:"Nieznany użytkownik",incomingCallNotificationPlaceholderAlert:"{callerName} dzwoni",incomingCallNoticicationAcceptWithAudioAriaLabel:"Zaakceptuj z dźwiękiem",incomingCallNoticicationAcceptWithVideoAriaLabel:"Zaakceptuj z wideo",incomingCallNoticicationRejectAriaLabel:"Odrzuć",incomingCallNotificationAcceptButtonLabel:"Akceptuj",incomingCallNotificationRejectButtonLabel:"Odrzuć",incomingCallNotificationAccceptWithVideoButtonLabel:"Odbierz jako połączenie wideo",incomingCallNotificationDismissButtonAriaLabel:"Odrzuć"};var pl_PL$1 = {participantItem:participantItem$6,ParticipantList:ParticipantList$6,typingIndicator:typingIndicator$6,sendBox:sendBox$6,richTextSendBox:richTextSendBox$6,realTimeTextModal:realTimeTextModal$6,captionsBanner:captionsBanner$6,mentionPopover:mentionPopover$6,imageOverlay:imageOverlay$6,messageStatusIndicator:messageStatusIndicator$6,endCallButton:endCallButton$6,cameraButton:cameraButton$6,microphoneButton:microphoneButton$6,devicesButton:devicesButton$6,participantsButton:participantsButton$6,screenShareButton:screenShareButton$6,raiseHandButton:raiseHandButton$6,reactionButton:reactionButton$6,realTimeText:realTimeText$6,captionsSettingsModal:captionsSettingsModal$6,spokenLanguages:spokenLanguages$6,captionLanguages:captionLanguages$6,startCaptionsButton:startCaptionsButton$6,startRealTimeTextButton:startRealTimeTextButton$6,messageThread:messageThread$6,errorBar:errorBar$6,notificationStack:notificationStack$6,videoGallery:videoGallery$6,dialpad:dialpad$6,holdButton:holdButton$6,videoTile:videoTile$6,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$6,CameraSitePermissionsRequest:CameraSitePermissionsRequest$6,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$6,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$6,CameraSitePermissionsCheck:CameraSitePermissionsCheck$6,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$6,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$6,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$6,CameraSitePermissionsDenied:CameraSitePermissionsDenied$6,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$6,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$6,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$6,UnsupportedBrowser:UnsupportedBrowser$6,UnsupportedBrowserVersion:UnsupportedBrowserVersion$6,UnsupportedOperatingSystem:UnsupportedOperatingSystem$6,BrowserPermissionDenied:BrowserPermissionDenied$6,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$6,verticalGallery:verticalGallery$6,meetingConferencePhoneInfo:meetingConferencePhoneInfo$6,IncomingCallNotification:IncomingCallNotification$6};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Polish (Poland).
 *
 * @public
 */
const COMPONENT_LOCALE_PL_PL = {
    strings: createComponentStrings(pl_PL$1)
};

var participantItem$5={isMeText:"(você)",menuTitle:"Mais opções",removeButtonLabel:"Remover",sharingIconLabel:"Compartilhamento",mutedIconLabel:"Silenciado",handRaisedIconLabel:"Mão levantada - Posição {position}",displayNamePlaceholder:"Participante sem nome",participantStateRinging:"Chamando...",participantStateHold:"Em espera",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, Mais Opções",attendeeRole:"Participante",micDisabledIconLabel:"Microfone desabilitado",cameraDisabledIconLabel:"Câmera desabilitada"};var ParticipantList$5={overflowParticipantCount:"+{overflowCount} mais"};var typingIndicator$5={singleUser:"{user} está digitando ...",multipleUsers:"{users} estão digitando ...",multipleUsersAbbreviateOne:"{users} e 1 outro estão digitando...",multipleUsersAbbreviateMany:"{users} e {numOthers} outros estão digitando ...",delimiter:", "};var sendBox$5={placeholderText:"Inserir uma mensagem",textTooLong:"O comprimento da mensagem está acima do limite máximo.",sendButtonAriaLabel:"Enviar mensagem",attachmentUploadsPendingError:"O arquivo está sendo carregado, aguarde.",removeAttachment:"Remover arquivo",uploading:"Carregando",uploadCompleted:"Carregamento concluído",attachmentMoreMenu:"Mais Opções"};var richTextSendBox$5={placeholderText:"Inserir uma mensagem",textTooLong:"O comprimento da mensagem está acima do limite máximo.",sendButtonAriaLabel:"Enviar mensagem",attachmentUploadsPendingError:"O arquivo está sendo carregado, aguarde.",imageUploadsPendingError:"A imagem está sendo carregada, aguarde.",removeAttachment:"Remover arquivo",uploading:"Carregando",uploadCompleted:"Carregamento concluído",richTextBoldTooltip:"Negrito",richTextItalicTooltip:"Itálico",richTextUnderlineTooltip:"Sublinhar",richTextBulletListTooltip:"Lista com marcadores",richTextNumberListTooltip:"Lista numerada",richTextIncreaseIndentTooltip:"Aumentar recuo",richTextDecreaseIndentTooltip:"Diminuir recuo",richTextInsertTableTooltip:"Inserir tabela",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Inserir",richTextInsertTableMenuTitle:"Inserir tabela {column} x {row}",richTextInsertRowAboveMenu:"Inserir acima",richTextInsertRowBelowMenu:"Inserir abaixo",richTextInsertColumnLeftMenu:"Inserir à esquerda",richTextInsertColumnRightMenu:"Inserir à direita",richTextDeleteRowOrColumnMenu:"Excluir",richTextDeleteColumnMenu:"Excluir coluna",richTextDeleteRowMenu:"Excluir linha",richTextDeleteTableMenu:"Excluir tabela",richTextToolbarAriaLabel:"Barra de Ferramentas de Formatação de Mensagens",richTextToolbarMoreButtonAriaLabel:"Mais opções",attachmentMoreMenu:"Mais Opções",richTextNewBulletedListItemAnnouncement:"Com marcadores, nova linha",richTextNewNumberedListItemAnnouncement:"{0}, Nova linha",richTextBulletedListAppliedAnnouncement:"Lista com marcadores aplicada",richTextNumberedListAppliedAnnouncement:"Lista numerada aplicada"};var realTimeTextModal$5={realTimeTextModalAriaLabel:"Ativar Modal RTT",realTimeTextModalTitle:"Ativar a RTT?",realTimeTextModalText:"O texto em tempo real (RTT) será ativado para todos na reunião.",realTimeTextConfirmButtonLabel:"Ativar",realTimeTextCancelButtonLabel:"Cancelar",realTimeTextCloseModalButtonAriaLabel:"Fechar Modal RTT"};var captionsBanner$5={captionsBannerSpinnerText:"Iniciando legendas...",realTimeTextInputBoxDefaultText:"Digitar mensagem em tempo real",realTimeTextInputErrorMessage:"A mensagem não pode ter mais de 2000 caracteres",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"O RTT (texto em tempo real) está habilitado para todos os participantes durante toda a reunião.",realTimeTextBannerLinkLabel:"Saiba mais",captionsOnlyContainerTitle:"Legendas",realTimeTextOnlyContainerTitle:"Texto em Tempo Real",captionsAndRealTimeTextContainerTitle:"Legendas e Texto em Tempo Real",expandButtonAriaLabel:"Expandir",minimizeButtonAriaLabel:"Recolher"};var mentionPopover$5={mentionPopoverHeader:"Sugestões"};var imageOverlay$5={downloadButtonLabel:"Baixar",dismissButtonAriaLabel:"Fechar",overlayVisibleMessage:"Sobreposição de Detalhes da Imagem"};var messageStatusIndicator$5={deliveredAriaLabel:"Mensagem enviada",deliveredTooltipText:"Enviado",seenAriaLabel:"Mensagem vista por outras pessoas",seenTooltipText:"Visto",readByTooltipText:"Lido por {messageThreadReadCount} de {remoteParticipantsCount}",sendingAriaLabel:"Envio de mensagem",sendingTooltipText:"Enviando",failedToSendAriaLabel:"Falha ao enviar mensagem",failedToSendTooltipText:"Falha ao enviar"};var endCallButton$5={label:"Sair",tooltipContent:"Sair da chamada",leaveOption:"Sair",endCallOption:"Encerrar chamada para todos"};var cameraButton$5={onLabel:"Câmera",offLabel:"Câmera",tooltipDisabledContent:"A câmera está desabilitada",tooltipOnContent:"Desabilitar a câmera",tooltipOffContent:"Ligar a câmera",tooltipVideoLoadingContent:"O vídeo está carregando",cameraMenuTitle:"Câmera",cameraMenuTooltip:"Escolher câmera",cameraButtonSplitRoleDescription:"Botão de divisão",onSplitButtonAriaLabel:"Desligar a câmera",offSplitButtonAriaLabel:"Ligar a câmera",cameraActionTurnedOnAnnouncement:"A câmera foi ligada",cameraActionTurnedOffAnnouncement:"A câmera foi desligada",onSplitButtonPrimaryActionCamera:"Desligar a câmera",offSplitButtonPrimaryActionCamera:"Ligar a câmera",cameraPrimaryActionSplitButtonTitle:"Usar câmera",videoEffectsMenuItemTitle:"Efeitos",cameraButtonAriaDescription:"opções de câmera"};var microphoneButton$5={onLabel:"Microfone",offLabel:"Microfone",tooltipDisabledContent:"O microfone está desabilitado",tooltipOnContent:"Ativar mudo do microfone",tooltipOffContent:"Desativar mudo do microfone",microphoneMenuTitle:"Microfone",microphoneMenuTooltip:"Escolher microfone",speakerMenuTitle:"Alto-falante",speakerMenuTooltip:"Escolher alto-falante",microphoneButtonSplitRoleDescription:"Botão de divisão",onSplitButtonAriaLabel:"Ativar mudo do microfone",offSplitButtonAriaLabel:"Desativar mudo do microfone",microphoneActionTurnedOnAnnouncement:"O microfone foi ligado",microphoneActionTurnedOffAnnouncement:"O microfone foi desligado",offSplitButtonMicrophonePrimaryAction:"Desativar mudo do microfone",onSplitButtonMicrophonePrimaryAction:"Ativar mudo do microfone",microphonePrimaryActionSplitButtonTitle:"Usar o microfone",microphoneAriaDescription:"Opções de áudio",deepNoiseSuppressionTitle:"Supressão de ruído",deepNoiseSuppressionOnAnnouncement:"A supressão de ruído profundo foi ativada",deepNoiseSuppressionOffAnnouncement:"A supressão de ruído profundo foi desativada"};var devicesButton$5={label:"Dispositivos",tooltipContent:"Gerenciar dispositivos",cameraMenuTitle:"Câmera",cameraMenuTooltip:"Escolher câmera",audioDeviceMenuTitle:"Dispositivo de Áudio",audioDeviceMenuTooltip:"Escolher dispositivo de áudio",microphoneMenuTitle:"Microfone",microphoneMenuTooltip:"Escolher microfone",speakerMenuTitle:"Alto-falante",speakerMenuTooltip:"Escolher alto-falante"};var participantsButton$5={label:"Pessoas",ariaLabel:"Pessoas",tooltipContent:"Mostrar participantes",menuHeader:"Nesta chamada",participantsListButtonLabel:"{numParticipants} pessoas",muteAllButtonLabel:"Silenciar todos",copyInviteLinkButtonLabel:"Copiar o link de convite",copyInviteLinkActionedAriaLabel:"Link de convite copiado"};var screenShareButton$5={onLabel:"Interromper apresentação",offLabel:"Apresentação",tooltipDisabledContent:"A apresentação está desabilitada.",tooltipOnContent:"Apresentando sua tela",tooltipOffContent:"Apresentar sua tela"};var raiseHandButton$5={offLabel:"Levantar",onLabel:"Baixar",tooltipDisabledContent:"A ação Levantar a Mão está desabilitada",tooltipOnContent:"Baixar a Mão",tooltipOffContent:"Levantar a Mão"};var reactionButton$5={label:"Reagir",ariaLabel:"Reagir",tooltipDisabledContent:"A ação de reação está desabilitada",tooltipContent:"Enviar uma reação",likeReactionTooltipContent:"Curtir",heartReactionTooltipContent:"Amor",laughReactionTooltipContent:"Gargalhada",applauseReactionTooltipContent:"Aplausos",surprisedReactionTooltipContent:"Surpreso"};var realTimeText$5={isTypingText:"Digitação RTT"};var captionsSettingsModal$5={captionsSettingsModalTitle:"Configurações de Legendas",captionsSettingsSpokenLanguageDropdownLabel:"Idioma falado",captionsSettingsCaptionLanguageDropdownLabel:"Idioma das legendas",captionsSettingsSpokenLanguageDropdownInfoText:"Idioma que todos nesta chamada estão falando.",captionsSettingsCaptionLanguageDropdownInfoText:"As legendas serão exibidas neste idioma.",captionsSettingsConfirmButtonLabel:"Confirmar",captionsSettingsCancelButtonLabel:"Cancelar",captionsSettingsModalAriaLabel:"Modal de Configuração de Legendas",captionsSettingsCloseModalButtonAriaLabel:"Fechar Configuração de Legendas"};var spokenLanguages$5={"ar-ae":"Árabe - Emirados Árabes Unidos","ar-sa":"Árabe – Arábia Saudita","da-dk":"Dinamarquês","de-de":"Alemão – Alemanha","en-au":"Inglês – Austrália","en-ca":"Inglês – Canadá","en-gb":"Inglês – Reino Unido","en-in":"Inglês – Índia","en-nz":"Inglês – Nova Zelândia","en-us":"Inglês – Estados Unidos","es-es":"Espanhol - Espanha (Moderno)","es-mx":"Espanhol – México","fi-fi":"Finlandês","fr-ca":"Francês – Canadá","fr-fr":"Francês – França","hi-in":"Híndi","it-it":"Italiano – Itália","ja-jp":"Japonês","ko-kr":"Coreano","nb-no":"Norueguês (Bokmål)","nl-be":"Holandês – Bélgica","nl-nl":"Holandês – Países Baixos","pl-pl":"Polonês","pt-br":"Português – Brasil","ru-ru":"Russo","sv-se":"Sueco","zh-cn":"Chinês - República Popular da China","zh-hk":"Chinês - RAE de Hong Kong","cs-cz":"Tcheco","pt-pt":"Português – Portugal","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Tailandês","he-il":"Hebraico","cy-gb":"Galês","uk-ua":"Ucraniano","el-gr":"Grego","hu-hu":"Húngaro","ro-ro":"Romeno","sk-sk":"Eslovaco","zh-tw":"Chinês – Taiwan"};var captionLanguages$5={ar:"Árabe",da:"Dinamarquês",de:"Alemão",en:"Inglês",es:"Espanhol",fi:"Finlandês","fr-ca":"Francês – Canadá",fr:"Francês – França",hi:"Híndi",it:"Italiano",ja:"Japonês",ko:"Coreano",nb:"Norueguês (Bokmål)",nl:"Holandês",pl:"Polonês",pt:"Português – Brasil",ru:"Russo",sv:"Sueco","zh-Hans":"Chinês (Simplificado)","zh-Hant":"Chinês (Tradicional)",cs:"Tcheco","pt-pt":"Português – Portugal",tr:"Turco",vi:"Vietnamita",th:"Tailandês",he:"Hebraico",cy:"Galês",uk:"Ucraniano",el:"Grego",hu:"Húngaro",ro:"Romeno",sk:"Eslovaco"};var startCaptionsButton$5={onLabel:"Ativar legendas",offLabel:"Desativar legendas",tooltipOnContent:"Desativar legendas",tooltipOffContent:"Ativar legendas"};var startRealTimeTextButton$5={label:"Habilitar a RTT para esta chamada",tooltipOffContent:"Habilitar a RTT para esta chamada"};var messageThread$5={yesterday:"Ontem",sunday:"Domingo",monday:"Segunda",tuesday:"Terça-feira",wednesday:"Quarta-feira",thursday:"quinta-feira",friday:"Sexta-feira",saturday:"Sábado",participantJoined:"ingressou no chat.",participantLeft:"saiu do chat.",editMessage:"Editar",removeMessage:"Excluir",resendMessage:"Tente enviar novamente",failToSendTag:"Falha ao enviar",editedTag:"Editado",liveAuthorIntro:"Mensagem de {author}",editedMessageLiveAuthorIntro:"Mensagem editada de {author}",editedMessageLocalUserLiveAuthorIntro:"Mensagem editada",messageContentAriaText:"{status} {author} disse {message}",messageContentMineAriaText:"{status} Você disse {message}",editBoxTextLimit:"Sua mensagem está acima do limite de {limitNumber} caracteres",editBoxPlaceholderText:"Edite sua mensagem",newMessagesIndicator:"Novas mensagens",noDisplayNameSub:"Sem nome",editBoxCancelButton:"Cancelar",editBoxSubmitButton:"Concluído",messageReadCount:"Lido por {messageReadByCount} de {remoteParticipantsCount}",actionMenuMoreOptions:"Mais opções",downloadAttachment:"Baixar arquivo",openAttachment:"Abrir arquivo",blockedWarningText:"Esta mensagem foi excluída devido à política organizacional.",blockedWarningLinkText:"Detalhes",attachmentCardGroupMessage:"A mensagem tem {attachmentCount} anexo",messageDeletedAnnouncementAriaLabel:"A mensagem foi excluída",richTextBoldTooltip:"Negrito",richTextItalicTooltip:"Itálico",richTextUnderlineTooltip:"Sublinhado",richTextBulletListTooltip:"Lista com marcadores",richTextNumberListTooltip:"Lista numerada",richTextIncreaseIndentTooltip:"Aumentar recuo",richTextDecreaseIndentTooltip:"Diminuir recuo",richTextInsertTableTooltip:"Inserir tabela",richTextFormatButtonTooltip:"Formato",richTextInsertRowOrColumnMenu:"Inserir",richTextInsertTableMenuTitle:"Inserir tabela {column} x {row}",richTextInsertRowAboveMenu:"Inserir acima",richTextInsertRowBelowMenu:"Inserir abaixo",richTextInsertColumnLeftMenu:"Inserir à esquerda",richTextInsertColumnRightMenu:"Inserir à direita",richTextDeleteRowOrColumnMenu:"Excluir",richTextDeleteColumnMenu:"Excluir coluna",richTextDeleteRowMenu:"Excluir linha",richTextDeleteTableMenu:"Excluir tabela",richTextToolbarMoreButtonAriaLabel:"Mais opções",attachmentUploadsPendingError:"O arquivo está sendo carregado, aguarde.",imageUploadsPendingError:"A imagem está sendo carregada, aguarde.",richTextNewBulletedListItemAnnouncement:"Com marcadores, nova linha",richTextNewNumberedListItemAnnouncement:"{0}, Nova linha",richTextBulletedListAppliedAnnouncement:"Lista com marcadores aplicada",richTextNumberedListAppliedAnnouncement:"Lista numerada aplicada"};var errorBar$5={unableToReachChatService:"Você está offline",accessDenied:"Não é possível acessar os serviços de chat - verifique as credenciais do usuário fornecidas",userNotInChatThread:"Você não está mais neste tópico de chat",sendMessageNotInChatThread:"Falha ao enviar mensagem porque você não está mais neste thread de chat",sendMessageGeneric:"Falha ao enviar mensagem",callingNetworkFailure:"Chamada de conexão desabilitar - parece que você está offline",startVideoGeneric:"Falha ao iniciar o vídeo",stopVideoGeneric:"Falha ao interromper o vídeo",muteGeneric:"Falha ao ativar mudo do microfone",unmuteGeneric:"Falha ao desativar mudo do microfone",speakingWhileMuted:"O microfone está com o mudo ativado",startScreenShareGeneric:"Ocorreu um problema ao iniciar o compartilhamento de tela.",stopScreenShareGeneric:"Falha ao interromper o compartilhamento de tela",callNetworkQualityLow:"A qualidade da rede é baixa.",teamsMeetingCallNetworkQualityLow:"A qualidade da rede é baixa.",callNoSpeakerFound:"Nenhum alto-falante ou fone de ouvido encontrado. Conecte um dispositivo de áudio para ouvir a chamada.",callNoMicrophoneFound:"Nenhum microfone encontrado. Conecte um dispositivo de entrada de áudio.",callMicrophoneAccessDenied:"Não é possível acessar o microfone. Clique no bloqueio na barra de endereços para conceder permissão a esta página da Web.",callMicrophoneAccessDeniedSafari:"Não é possível acessar o microfone. Atualize a página para permitir permissões ou verifique as configurações deste navegador e verifique se as permissões estão habilitadas para este site.",callMicrophoneMutedBySystem:"Você está com as notificações desativada pelo sistema.",callMicrophoneUnmutedBySystem:"Seu microfone se recuperou e você foi desativado pelo sistema.",callMacOsMicrophoneAccessDenied:"Não é possível acessar o microfone. Conceda permissão ao microfone nas configurações de privacidade do macOS.",callLocalVideoFreeze:"A largura de banda da rede é ruim. Seu vídeo pode aparecer em pausa para outras pessoas na chamada.",callCameraAccessDenied:"Não é possível acessar a câmera. Clique no bloqueio na barra de endereços para conceder permissão a esta página da Web.",callCameraAccessDeniedSafari:"Não é possível acessar a câmera. Atualize a página para permitir permissões ou verifique as configurações deste navegador e verifique se as permissões estão habilitadas para este site.",callCameraAlreadyInUse:"Não é possível acessar a câmera. Talvez ele já esteja em uso por outro aplicativo.",callVideoStoppedBySystem:"Seu vídeo foi interrompido pelo sistema.",callVideoRecoveredBySystem:"Seu vídeo foi retomado.",callMacOsCameraAccessDenied:"O MacOS está bloqueando o acesso à sua câmera. Atualize suas configurações de privacidade para permitir que este navegador acesse sua câmera.",callMacOsScreenShareAccessDenied:"O MacOS está bloqueando o compartilhamento de tela. Atualize suas configurações de privacidade para permitir que este navegador grave sua tela.",dismissButtonAriaLabel:"Fechar",failedToJoinCallGeneric:"Ocorreu um erro ao ingressar na chamada.",failedToJoinCallInvalidMeetingLink:"Não é possível ingressar na Reunião. Link Inválido.",cameraFrozenForRemoteParticipants:"Os usuários na chamada estão enfrentando problemas para ver seu vídeo. Verifique seus dispositivos e a rede.",unableToStartVideoEffect:"Não é possível aplicar o efeito de vídeo.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Não é possível iniciar o destaque nos participantes porque o número máximo de participantes já está em destaque.",mutedByRemoteParticipant:"Você foi silenciado por outro participante."};var notificationStack$5={callingNetworkFailure:{title:"Chamada de conexão com problema",message:"Parece que você está offline. Verifique sua conexão de rede e tente novamente.",dismissButtonAriaLabel:"Fechar"},startVideoGeneric:{title:"Falha ao iniciar o vídeo.",dismissButtonAriaLabel:"Fechar"},stopVideoGeneric:{title:"Falha ao interromper o vídeo.",dismissButtonAriaLabel:"Fechar"},muteGeneric:{title:"Falha ao ativar mudo do microfone.",dismissButtonAriaLabel:"Fechar"},unmuteGeneric:{title:"Falha ao desativar mudo do microfone.",dismissButtonAriaLabel:"Fechar"},speakingWhileMuted:{title:"Você está com o mudo ativado.",dismissButtonAriaLabel:"Fechar"},startScreenShareGeneric:{title:"Ocorreu um problema ao iniciar o compartilhamento de tela.",dismissButtonAriaLabel:"Fechar"},stopScreenShareGeneric:{title:"Falha ao interromper o compartilhamento de tela.",dismissButtonAriaLabel:"Fechar"},callNetworkQualityLow:{title:"A qualidade da rede é baixa.",dismissButtonAriaLabel:"Fechar"},teamsMeetingCallNetworkQualityLow:{title:"A qualidade da rede é baixa.",dismissButtonAriaLabel:"Fechar",message:"Ingresse nesta chamada do seu telefone para melhorar o som. Você pode continuar exibindo a reunião neste dispositivo.",primaryButtonLabel:"Ingressar por Telefone"},callNoSpeakerFound:{title:"Nenhum alto-falante ou fone de ouvido encontrado",message:"Conecte um dispositivo de áudio para ouvir a chamada.",dismissButtonAriaLabel:"Fechar"},callNoMicrophoneFound:{title:"Nenhum microfone encontrado",message:"Conecte um dispositivo de entrada de áudio.",dismissButtonAriaLabel:"Fechar"},callMicrophoneAccessDenied:{title:"Não é possível acessar o microfone",message:"Clique no bloqueio na barra de endereços para conceder permissão a esta página da Web.",dismissButtonAriaLabel:"Fechar"},callMicrophoneAccessDeniedSafari:{title:"Não é possível acessar o microfone",message:"Atualize a página para permitir permissões ou verifique as configurações deste navegador e verifique se as permissões estão habilitadas para este site.",dismissButtonAriaLabel:"Fechar"},callMicrophoneMutedBySystem:{title:"Você está com as notificações desativada pelo sistema.",dismissButtonAriaLabel:"Fechar"},callMicrophoneUnmutedBySystem:{title:"Seu microfone se recuperou e você foi desativado pelo sistema.",dismissButtonAriaLabel:"Fechar"},callMacOsMicrophoneAccessDenied:{title:"Não é possível acessar o microfone",message:"Conceda permissão ao microfone nas configurações de privacidade do macOS.",dismissButtonAriaLabel:"Fechar"},callLocalVideoFreeze:{title:"A largura de banda da rede é insatisfatória",message:"Seu vídeo pode aparecer em pausa para outras pessoas na chamada.",dismissButtonAriaLabel:"Fechar"},callCameraAccessDenied:{title:"Não é possível acessar a câmera",message:"Clique no bloqueio na barra de endereços para conceder permissão a esta página da Web.",dismissButtonAriaLabel:"Fechar"},callCameraAccessDeniedSafari:{title:"Não é possível acessar a câmera",message:"Atualize a página para permitir permissões ou verifique as configurações deste navegador e verifique se as permissões estão habilitadas para este site.",dismissButtonAriaLabel:"Fechar"},callCameraAlreadyInUse:{title:"Não é possível acessar a câmera",message:"Talvez ele já esteja em uso por outro aplicativo.",dismissButtonAriaLabel:"Fechar"},callVideoStoppedBySystem:{title:"Seu vídeo foi interrompido pelo sistema.",dismissButtonAriaLabel:"Fechar"},callVideoRecoveredBySystem:{title:"Seu vídeo foi retomado.",dismissButtonAriaLabel:"Fechar"},callMacOsCameraAccessDenied:{title:"O MacOS está bloqueando o acesso à sua câmera",message:"Atualize suas configurações de privacidade para permitir que este navegador acesse sua câmera.",dismissButtonAriaLabel:"Fechar"},callMacOsScreenShareAccessDenied:{title:"O MacOS está bloqueando o compartilhamento de tela",message:"Atualize suas configurações de privacidade para permitir que este navegador grave sua tela.",dismissButtonAriaLabel:"Fechar"},failedToJoinCallGeneric:{title:"Falha ao ingressar na chamada.",dismissButtonAriaLabel:"Fechar"},failedToJoinCallInvalidMeetingLink:{title:"Não é possível ingressar na Reunião. Link Inválido.",dismissButtonAriaLabel:"Fechar"},cameraFrozenForRemoteParticipants:{title:"Os usuários na chamada estão enfrentando problemas para ver seu vídeo",message:"Verifique seus dispositivos e rede.",dismissButtonAriaLabel:"Fechar"},unableToStartVideoEffect:{title:"Não é possível aplicar o efeito de vídeo.",dismissButtonAriaLabel:"Fechar"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Não é possível iniciar o destaque nos participantes",message:"O número máximo de participantes já está em destaque.",dismissButtonAriaLabel:"Fechar"},mutedByRemoteParticipant:{title:"Você foi silenciado por outro participante.",dismissButtonAriaLabel:"Fechar"},recordingStarted:{title:"A gravação foi iniciada.",message:"Ao ingressar, você está dando consentimento para que esta reunião seja transcrita.",dismissButtonAriaLabel:"Fechar"},transcriptionStarted:{title:"A transcrição foi iniciada.",message:"Ao ingressar, você está dando consentimento para que esta reunião seja transcrita.",dismissButtonAriaLabel:"Fechar"},recordingStopped:{title:"A gravação está sendo salva.",message:"A gravação foi interrompida.",dismissButtonAriaLabel:"Fechar"},transcriptionStopped:{title:"A transcrição está sendo salva.",message:"A transcrição foi interrompida.",dismissButtonAriaLabel:"Fechar"},recordingAndTranscriptionStarted:{title:"A gravação e a transcrição foram iniciadas.",message:"Ao ingressar, você está dando consentimento para que esta reunião seja transcrita.",dismissButtonAriaLabel:"Fechar"},recordingAndTranscriptionStopped:{title:"A gravação e a transcrição estão sendo salvas.",message:"A gravação e a transcrição foram interrompidas.",dismissButtonAriaLabel:"Fechar"},recordingStoppedStillTranscribing:{title:"A gravação foi interrompida.",message:"Você agora está apenas transcrevendo esta reunião.",dismissButtonAriaLabel:"Fechar"},transcriptionStoppedStillRecording:{title:"A transcrição foi interrompida.",message:"Você agora está apenas gravando esta reunião.",dismissButtonAriaLabel:"Fechar"},assignedBreakoutRoomOpened:{title:"Vamos movê-lo para a sala atribuída em 5 segundos.",dismissButtonAriaLabel:"Fechar"},assignedBreakoutRoomChanged:{title:"Sua sala para sessão de grupo foi alterada",message:"Vamos movê-lo para a sala atribuída em 5 segundos.",dismissButtonAriaLabel:"Fechar"},assignedBreakoutRoomClosed:{title:"Sua sala para sessão de grupo foi fechada",message:"Vamos movê-lo de volta para a reunião em 5 segundos.",dismissButtonAriaLabel:"Fechar"},assignedBreakoutRoomOpenedPromptJoin:{title:"Ingressar na sala para sessão de grupo?",message:"Você foi atribuído a uma sala para sessão de grupo.",dismissButtonAriaLabel:"Fechar",primaryButtonLabel:"Ingressar na sala",secondaryButtonLabel:"Mais tarde"},breakoutRoomJoined:{title:"Você ingressou nem uma sala para sessão de grupo.",dismissButtonAriaLabel:"Fechar"},breakoutRoomClosingSoon:{title:"Limite de tempo de sala prestes a expirar.",message:"Esta sala será fechada em 30 segundos",dismissButtonAriaLabel:"Fechar"},capabilityTurnVideoOnPresent:{title:"Sua câmera foi habilitada",message:"Ativar se quiser compartilhar um vídeo.",dismissButtonAriaLabel:"Fechar"},capabilityTurnVideoOnAbsent:{title:"Sua câmera foi desabilitada",message:"Você não pode mais compartilhar vídeo.",dismissButtonAriaLabel:"Fechar"},capabilityUnmuteMicPresent:{title:"Seu microfone foi habilitado",message:"Desative mudo se quiser falar.",dismissButtonAriaLabel:"Fechar"},capabilityUnmuteMicAbsent:{title:"O microfone foi desabilitado",message:"Você não pode mais ativar o som.",dismissButtonAriaLabel:"Fechar"},togetherModeStarted:{title:"O modo Togethermode foi iniciado",dismissButtonAriaLabel:"Fechar"},togetherModeEnded:{title:"O modo Togethermode foi encerrado",dismissButtonAriaLabel:"Fechar"},transcriptionError:{title:"Algo deu errado com a transcrição.",message:"A transcrição foi interrompida. Tente novamente mais tarde",dismissButtonAriaLabel:"Fechar"},transcriptionStartedByYou:{title:"A transcrição foi iniciada.",message:"Iniciado por você. Informe a todos que eles estão sendo transcritos.",dismissButtonAriaLabel:"Fechar"}};var videoGallery$5={screenIsBeingSharedMessage:"Você está compartilhando sua tela",screenShareLoadingMessage:"Carregando a tela do {participant}",localScreenShareLoadingMessage:"Carregando sua tela",localVideoLabel:"Você",localVideoCameraSwitcherLabel:"Alternar câmera",localVideoMovementLabel:"Bloco de Vídeo Local Removível",localVideoMovementAriaLabel:"Bloco de Vídeo Local Removível",localVideoSelectedDescription:"{cameraName} selecionado",displayNamePlaceholder:"Participante sem nome",fitRemoteParticipantToFrame:"Ajustar ao quadro",fillRemoteParticipantFrame:"Preencher quadro",pinParticipantForMe:"Fixar para mim",pinParticipantForMeLimitReached:"PIN (limite alcançado)",unpinParticipantForMe:"Desafixar",pinParticipantMenuItemAriaLabel:"Fixar {participantName}",unpinParticipantMenuItemAriaLabel:"Desafixar {participantName}",pinnedParticipantAnnouncementAriaLabel:"Fixado {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Desafixado {participantName}",startSpotlightVideoTileMenuLabel:"Destacar para todos",addSpotlightVideoTileMenuLabel:"Adicionar destaque",spotlightLimitReachedMenuTitle:"Limite de destaque atingido",stopSpotlightVideoTileMenuLabel:"Parar de destacar",stopSpotlightOnSelfVideoTileMenuLabel:"Sair de destaque",attendeeRole:"Participante",muteParticipantMenuItemLabel:"Ativar mudo",waitingScreenText:"Aguardando o ingresso de outros participantes",forbidAudio:"Desabilitar microfone",permitAudio:"Permitir microfone",forbidAudioTileMenuLabel:"Desabilitar microfone",permitAudioTileMenuLabel:"Permitir microfone",forbidVideo:"Desabilitar câmera",permitVideo:"Permitir câmera",forbidVideoTileMenuLabel:"Desabilitar câmera",permitVideoTileMenuLabel:"Permitir câmera"};var dialpad$5={placeholderText:"Inserir o número do telefone",deleteButtonAriaLabel:"Excluir"};var holdButton$5={onLabel:"Retomar",offLabel:"Retenção",tooltipOnContent:"Retomar chamada",tooltipOffContent:"Manter chamada em espera"};var videoTile$5={participantStateRinging:"Chamando...",participantReconnecting:"Reconectando...",participantStateHold:"Em espera",moreOptionsButtonAriaLabel:"Mais Opções {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"silenciado",moreOptionsParticipantMutedStateUnmutedAriaLabel:"mudo desativado",moreOptionsParticipantHandRaisedAriaLabel:"Mão levantada",moreOptionsParticipantIsSpeakingAriaLabel:"Falando",moreOptionsParticipantMicDisabledAriaLabel:"Microfone desabilitado",moreOptionsParticipantCameraDisabledAriaLabel:"Câmera desabilitada"};var CameraAndMicrophoneSitePermissionsRequest$5={primaryText:"Permitir {appName} usar a câmera e o microfone",secondaryText:"Isso é para que os participantes possam ver e ouvir você.",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas",primaryButtonText:"Continuar sem câmera e microfone",ariaLabel:"Permitir acesso à câmera e ao microfone"};var CameraSitePermissionsRequest$5={primaryText:"Permitir que {appName} use a câmera",secondaryText:"Isso é para que os participantes possam ver você.",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas",primaryButtonText:"Continuar sem a câmera",ariaLabel:"Permitir acesso à câmera"};var MicrophoneSitePermissionsRequest$5={primaryText:"Permitir que {appName} use o microfone",secondaryText:"Isso é para que os participantes possam ouvir você.",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas",primaryButtonText:"Continuar sem o microfone",ariaLabel:"Permitir acesso ao microfone"};var CameraAndMicrophoneSitePermissionsCheck$5={primaryText:"Verificando o acesso à câmera e ao microfone",secondaryText:"Permitir acesso, se solicitado. Isso é para que os participantes possam ver e ouvir você.",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas",primaryButtonText:"Continuar sem câmera e microfone",ariaLabel:"Verificando o acesso à câmera e ao microfone. Permitir o acesso se solicitado."};var CameraSitePermissionsCheck$5={primaryText:"Verificando o acesso à câmera",secondaryText:"Permitir acesso, se solicitado. Isso é para que os participantes possam ver você.",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas",primaryButtonText:"Continuar sem câmera",ariaLabel:"Verificando o acesso à câmera. Permitir o acesso se solicitado."};var MicrophoneSitePermissionsCheck$5={primaryText:"Verificando o acesso ao microfone",secondaryText:"Permitir acesso, se solicitado. Isso é para que os participantes possam ouvir você.",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas",primaryButtonText:"Continuar sem microfone",ariaLabel:"Verificando acesso ao microfone. Permitir o acesso se solicitado."};var CameraAndMicrophoneSitePermissionsDenied$5={primaryText:"Não é possível acessar a câmera e o microfone",secondaryText:"Clique no ícone de bloqueio na barra de endereços para conceder permissões de microfone a esta página da Web. Uma atualização de página pode ser necessária.",primaryButtonText:"Continuar sem câmera e microfone",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas"};var CameraAndMicrophoneSitePermissionsDeniedSafari$5={primaryText:"Não é possível acessar a câmera e o microfone",secondaryText:"Atualize a página para permitir permissões ou verifique as configurações deste navegador e verifique se as permissões estão habilitadas para este site.",primaryButtonText:"Continuar sem câmera e microfone",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas"};var CameraSitePermissionsDenied$5={primaryText:"Não é possível acessar a câmera",secondaryText:"Clique no ícone de bloqueio na barra de endereços para conceder permissões de câmera a esta página da Web. Uma atualização de página pode ser necessária.",primaryButtonText:"Continuar sem a câmera",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas"};var MicrophoneSitePermissionsDenied$5={primaryText:"Não é possível acessar o microfone",secondaryText:"Clique no ícone de bloqueio na barra de endereços para conceder permissões de microfone a esta página da Web. Uma atualização de página pode ser necessária.",primaryButtonText:"Continuar sem microfone",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas"};var CameraSitePermissionsDeniedSafari$5={primaryText:"Não é possível acessar a câmera",secondaryText:"Atualize a página para permitir permissões ou verifique as configurações deste navegador e verifique se as permissões estão habilitadas para este site.",primaryButtonText:"Continuar sem câmera",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas"};var MicrophoneSitePermissionsDeniedSafari$5={primaryText:"Não é possível acessar o microfone",secondaryText:"Atualize a página para permitir permissões ou verifique as configurações deste navegador e verifique se as permissões estão habilitadas para este site.",primaryButtonText:"Continuar sem o microfone",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas"};var UnsupportedBrowser$5={primaryText:"Navegador sem suporte",secondaryText:"Ingresse nesta chamada usando um navegador compatível.",moreHelpLinkText:"Consulte os requisitos de compatibilidade"};var UnsupportedBrowserVersion$5={primaryText:"Atualização do navegador necessária",secondaryText:"Para garantir a melhor chamada possível, atualize o navegador e tente ingressar na chamada novamente.",moreHelpLinkText:"Consulte os requisitos de compatibilidade",continueAnywayButtonText:"Iniciar chamada sem atualizar"};var UnsupportedOperatingSystem$5={primaryText:"Sistema operacional sem suporte",secondaryText:"Ingresse nesta chamada usando um dispositivo com um sistema operacional compatível.",moreHelpLinkText:"Consulte os requisitos de compatibilidade"};var BrowserPermissionDenied$5={primaryText:"Não é possível usar sua câmera ou microfone",secondaryText:"Seu navegador pode não ter acesso à câmera ou ao microfone. Para corrigir isso, abra as Preferências do Sistema.",primaryButtonText:"Tentar novamente",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas"};var BrowserPermissionDeniedIOS$5={primaryText:"Permitir acesso ao microfone para continuar",secondaryText:"Para que outros participantes possam ouvi-lo.",primaryButtonText:"Tentar novamente",imageAltText:"Localização da permissão do microfone e do dispositivo da câmera para iOS",linkText:"Preciso de ajuda? Obtenha ajuda para solução de problemas",step1Text:"Acessar o aplicativo Configurações",step2Text:"Rolar para baixo até as configurações deste navegador",step3Text:"Ligar o microfone (Câmera opcional)",step4Text:"Tentar ingressar na chamada novamente",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$5={leftNavButtonAriaLabel:"página anterior {current} de {total}",rightNavButtonAriaLabel:"próxima página {current} de {total}"};var meetingConferencePhoneInfo$5={meetingConferencePhoneInfoModalTitle:"Participar da chamada pelo seu telefone",meetingConferencePhoneInfoModalDialIn:"Número de discagem",meetingConferencePhoneInfoModalMeetingId:"Inserir a ID da reunião",meetingConferencePhoneInfoModalWait:"Aguarde para ser aceito pelo organizador",meetingConferencePhoneInfoModalNoPhoneAvailable:"Nenhum telefone disponível",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Chamada gratuita)",meetingConferencePhoneInfoModalToll:"(Chamada tarifada)"};var IncomingCallNotification$5={incomingCallNotificationPlaceholderId:"Usuário Desconhecido",incomingCallNotificationPlaceholderAlert:"Chamada de {callerName}",incomingCallNoticicationAcceptWithAudioAriaLabel:"Aceitar com áudio",incomingCallNoticicationAcceptWithVideoAriaLabel:"Aceitar com vídeo",incomingCallNoticicationRejectAriaLabel:"Rejeitar",incomingCallNotificationAcceptButtonLabel:"Aceitar",incomingCallNotificationRejectButtonLabel:"Recusar",incomingCallNotificationAccceptWithVideoButtonLabel:"Aceitar com Vídeo",incomingCallNotificationDismissButtonAriaLabel:"Ignorar"};var pt_BR$1 = {participantItem:participantItem$5,ParticipantList:ParticipantList$5,typingIndicator:typingIndicator$5,sendBox:sendBox$5,richTextSendBox:richTextSendBox$5,realTimeTextModal:realTimeTextModal$5,captionsBanner:captionsBanner$5,mentionPopover:mentionPopover$5,imageOverlay:imageOverlay$5,messageStatusIndicator:messageStatusIndicator$5,endCallButton:endCallButton$5,cameraButton:cameraButton$5,microphoneButton:microphoneButton$5,devicesButton:devicesButton$5,participantsButton:participantsButton$5,screenShareButton:screenShareButton$5,raiseHandButton:raiseHandButton$5,reactionButton:reactionButton$5,realTimeText:realTimeText$5,captionsSettingsModal:captionsSettingsModal$5,spokenLanguages:spokenLanguages$5,captionLanguages:captionLanguages$5,startCaptionsButton:startCaptionsButton$5,startRealTimeTextButton:startRealTimeTextButton$5,messageThread:messageThread$5,errorBar:errorBar$5,notificationStack:notificationStack$5,videoGallery:videoGallery$5,dialpad:dialpad$5,holdButton:holdButton$5,videoTile:videoTile$5,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$5,CameraSitePermissionsRequest:CameraSitePermissionsRequest$5,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$5,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$5,CameraSitePermissionsCheck:CameraSitePermissionsCheck$5,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$5,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$5,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$5,CameraSitePermissionsDenied:CameraSitePermissionsDenied$5,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$5,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$5,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$5,UnsupportedBrowser:UnsupportedBrowser$5,UnsupportedBrowserVersion:UnsupportedBrowserVersion$5,UnsupportedOperatingSystem:UnsupportedOperatingSystem$5,BrowserPermissionDenied:BrowserPermissionDenied$5,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$5,verticalGallery:verticalGallery$5,meetingConferencePhoneInfo:meetingConferencePhoneInfo$5,IncomingCallNotification:IncomingCallNotification$5};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Portuguese (Brazil).
 *
 * @public
 */
const COMPONENT_LOCALE_PT_BR = {
    strings: createComponentStrings(pt_BR$1)
};

var participantItem$4={isMeText:"(Вы)",menuTitle:"Дополнительные параметры",removeButtonLabel:"Удалить",sharingIconLabel:"Общий доступ",mutedIconLabel:"Микрофон отключен",handRaisedIconLabel:"Поднята рука — позиция {position}",displayNamePlaceholder:"Участник без имени",participantStateRinging:"Выполняется вызов...",participantStateHold:"На удержании",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, дополнительные параметры",attendeeRole:"Участник",micDisabledIconLabel:"Микрофон отключен",cameraDisabledIconLabel:"Камера отключена"};var ParticipantList$4={overflowParticipantCount:"и еще {overflowCount}"};var typingIndicator$4={singleUser:"{user} вводит текст...",multipleUsers:"{users} вводят текст...",multipleUsersAbbreviateOne:"{users} и еще 1 вводят текст...",multipleUsersAbbreviateMany:"{users} и еще {numOthers} вводят текст...",delimiter:"; "};var sendBox$4={placeholderText:"Введите сообщение",textTooLong:"Длина сообщения превышает максимально допустимую.",sendButtonAriaLabel:"Отправить сообщение",attachmentUploadsPendingError:"Выполняется отправка файла. Подождите.",removeAttachment:"Удалить файл",uploading:"Отправка",uploadCompleted:"Отправка завершена",attachmentMoreMenu:"Дополнительные параметры"};var richTextSendBox$4={placeholderText:"Введите сообщение",textTooLong:"Длина сообщения превышает максимально допустимую.",sendButtonAriaLabel:"Отправить сообщение",attachmentUploadsPendingError:"Выполняется отправка файла. Подождите.",imageUploadsPendingError:"Выполняется отправка изображения. Подождите.",removeAttachment:"Удалить файл",uploading:"Отправка",uploadCompleted:"Отправка завершена",richTextBoldTooltip:"Полужирный",richTextItalicTooltip:"Курсив",richTextUnderlineTooltip:"Подчеркнуть",richTextBulletListTooltip:"Маркированный список",richTextNumberListTooltip:"Нумерованный список",richTextIncreaseIndentTooltip:"Увеличить отступ",richTextDecreaseIndentTooltip:"Уменьшить отступ",richTextInsertTableTooltip:"Вставить таблицу",richTextFormatButtonTooltip:"Формат",richTextInsertRowOrColumnMenu:"Вставить",richTextInsertTableMenuTitle:"Вставить таблицу {column} x {row}",richTextInsertRowAboveMenu:"Вставить сверху",richTextInsertRowBelowMenu:"Вставить снизу",richTextInsertColumnLeftMenu:"Вставить слева",richTextInsertColumnRightMenu:"Вставить справа",richTextDeleteRowOrColumnMenu:"Удалить",richTextDeleteColumnMenu:"Удалить столбец",richTextDeleteRowMenu:"Удалить строку",richTextDeleteTableMenu:"Удалить таблицу",richTextToolbarAriaLabel:"Панель инструментов форматирования сообщения",richTextToolbarMoreButtonAriaLabel:"Дополнительные параметры",attachmentMoreMenu:"Дополнительные параметры",richTextNewBulletedListItemAnnouncement:"Маркер, новая строка",richTextNewNumberedListItemAnnouncement:"{0}, новая строка",richTextBulletedListAppliedAnnouncement:"Применен маркированный список",richTextNumberedListAppliedAnnouncement:"Применен нумерованный список"};var realTimeTextModal$4={realTimeTextModalAriaLabel:"Модальное окно включения ввода в режиме реального времени",realTimeTextModalTitle:"Включить ввод в режиме реального времени?",realTimeTextModalText:"Текст в режиме реального времени (RTT) будет включен для всех участников собрания.",realTimeTextConfirmButtonLabel:"Включить",realTimeTextCancelButtonLabel:"Отмена",realTimeTextCloseModalButtonAriaLabel:"Закрыть модальное окно ввода в режиме реального времени"};var captionsBanner$4={captionsBannerSpinnerText:"Запуск субтитров...",realTimeTextInputBoxDefaultText:"Введите сообщение в режиме реального времени",realTimeTextInputErrorMessage:"Длина сообщения не может превышать 2000 символов",realTimeTextBannerTitle:"Ввод в режиме реального времени",realTimeTextBannerContent:"Ввод в режиме реального времени включен для всех участников в течение всего собрания.",realTimeTextBannerLinkLabel:"Подробнее",captionsOnlyContainerTitle:"Субтитры",realTimeTextOnlyContainerTitle:"Текст в режиме реального времени",captionsAndRealTimeTextContainerTitle:"Субтитры и текст в реальном времени",expandButtonAriaLabel:"Развернуть",minimizeButtonAriaLabel:"Свернуть"};var mentionPopover$4={mentionPopoverHeader:"Рекомендации"};var imageOverlay$4={downloadButtonLabel:"Скачать",dismissButtonAriaLabel:"Закрыть",overlayVisibleMessage:"Наложение сведений об изображении"};var messageStatusIndicator$4={deliveredAriaLabel:"Сообщение отправлено",deliveredTooltipText:"Отправлено",seenAriaLabel:"Сообщение просмотрено другими пользователями",seenTooltipText:"Просмотрено",readByTooltipText:"Прочитали: {messageThreadReadCount} из {remoteParticipantsCount}",sendingAriaLabel:"Отправка сообщения",sendingTooltipText:"Отправка",failedToSendAriaLabel:"Не удалось отправить сообщение",failedToSendTooltipText:"Не удалось отправить"};var endCallButton$4={label:"Выйти",tooltipContent:"Покинуть вызов",leaveOption:"Выйти",endCallOption:"Завершить звонок для всех"};var cameraButton$4={onLabel:"Камера",offLabel:"Камера",tooltipDisabledContent:"Камера отключена",tooltipOnContent:"Выключить камеру",tooltipOffContent:"Включить камеру",tooltipVideoLoadingContent:"Видео загружается",cameraMenuTitle:"Камера",cameraMenuTooltip:"Выбрать камеру",cameraButtonSplitRoleDescription:"Разворачивающаяся кнопка",onSplitButtonAriaLabel:"Отключить камеру",offSplitButtonAriaLabel:"Включить камеру",cameraActionTurnedOnAnnouncement:"Камера включена",cameraActionTurnedOffAnnouncement:"Камера отключена",onSplitButtonPrimaryActionCamera:"Выключить камеру",offSplitButtonPrimaryActionCamera:"Включить камеру",cameraPrimaryActionSplitButtonTitle:"Использовать камеру",videoEffectsMenuItemTitle:"Эффекты",cameraButtonAriaDescription:"параметры камеры"};var microphoneButton$4={onLabel:"Микрофон",offLabel:"Микрофон",tooltipDisabledContent:"Микрофон отключен",tooltipOnContent:"Отключить микрофон",tooltipOffContent:"Включить микрофон",microphoneMenuTitle:"Микрофон",microphoneMenuTooltip:"Выбрать микрофон",speakerMenuTitle:"Динамик",speakerMenuTooltip:"Выбрать динамик",microphoneButtonSplitRoleDescription:"Разворачивающаяся кнопка",onSplitButtonAriaLabel:"Отключить микрофон",offSplitButtonAriaLabel:"Включить микрофон",microphoneActionTurnedOnAnnouncement:"Ваш микрофон включен",microphoneActionTurnedOffAnnouncement:"Ваш микрофон отключен",offSplitButtonMicrophonePrimaryAction:"Включить микрофон",onSplitButtonMicrophonePrimaryAction:"Отключить микрофон",microphonePrimaryActionSplitButtonTitle:"Использовать микрофон",microphoneAriaDescription:"Параметры звука",deepNoiseSuppressionTitle:"Подавление шума",deepNoiseSuppressionOnAnnouncement:"Глубокое подавление шума включено",deepNoiseSuppressionOffAnnouncement:"Глубокое подавление шума отключено"};var devicesButton$4={label:"Устройства",tooltipContent:"Управление устройствами",cameraMenuTitle:"Камера",cameraMenuTooltip:"Выбрать камеру",audioDeviceMenuTitle:"Звуковое устройство",audioDeviceMenuTooltip:"Выбрать звуковое устройство",microphoneMenuTitle:"Микрофон",microphoneMenuTooltip:"Выбрать микрофон",speakerMenuTitle:"Динамик",speakerMenuTooltip:"Выбрать динамик"};var participantsButton$4={label:"Люди",ariaLabel:"Люди",tooltipContent:"Показать участников",menuHeader:"В этом звонке",participantsListButtonLabel:"Пользователей: {numParticipants}",muteAllButtonLabel:"Отключить все микрофоны",copyInviteLinkButtonLabel:"Копировать ссылку с приглашением",copyInviteLinkActionedAriaLabel:"Ссылка на приглашение скопирована"};var screenShareButton$4={onLabel:"Остановить демонстрацию",offLabel:"Демонстрировать",tooltipDisabledContent:"Демонстрация отключена",tooltipOnContent:"Демонстрация экрана",tooltipOffContent:"Демонстрировать экран"};var raiseHandButton$4={offLabel:"Поднять руку",onLabel:"Опустить",tooltipDisabledContent:"Действие \"Поднять руку\" отключено",tooltipOnContent:"Опустить руку",tooltipOffContent:"Поднять руку"};var reactionButton$4={label:"Отреагировать",ariaLabel:"Отреагировать",tooltipDisabledContent:"Действие реакции отключено",tooltipContent:"Отправка реакции",likeReactionTooltipContent:"Нравится",heartReactionTooltipContent:"Люблю",laughReactionTooltipContent:"Смеюсь",applauseReactionTooltipContent:"Аплодисменты",surprisedReactionTooltipContent:"Удивляюсь"};var realTimeText$4={isTypingText:"Ввод в режиме реального времени"};var captionsSettingsModal$4={captionsSettingsModalTitle:"Параметры субтитров",captionsSettingsSpokenLanguageDropdownLabel:"Язык речи",captionsSettingsCaptionLanguageDropdownLabel:"Язык субтитров",captionsSettingsSpokenLanguageDropdownInfoText:"Язык, на котором говорят все участники звонка.",captionsSettingsCaptionLanguageDropdownInfoText:"Субтитры будут отображаться на этом языке.",captionsSettingsConfirmButtonLabel:"Подтвердить",captionsSettingsCancelButtonLabel:"Отмена",captionsSettingsModalAriaLabel:"Модальное окно параметров субтитров",captionsSettingsCloseModalButtonAriaLabel:"Закрыть параметры субтитров"};var spokenLanguages$4={"ar-ae":"Арабский (ОАЭ)","ar-sa":"Арабский (Саудовская Аравия)","da-dk":"Датский","de-de":"Немецкий (Германия)","en-au":"Английский (Австралия)","en-ca":"Английский (Канада)","en-gb":"Английский (Соединенное Королевство)","en-in":"Английский (Индия)","en-nz":"Английский (Новая Зеландия)","en-us":"Английский (США)","es-es":"Испанский (Испания), современная сортировка","es-mx":"Испанский (Мексика)","fi-fi":"Финский","fr-ca":"Французский (Канада)","fr-fr":"Французский (Франция)","hi-in":"Хинди","it-it":"Итальянский (Италия)","ja-jp":"Японский","ko-kr":"Корейский","nb-no":"Норвежский (букмол)","nl-be":"Нидерландский (Бельгия)","nl-nl":"Нидерландский (Нидерланды)","pl-pl":"Польский","pt-br":"Португальский (Бразилия)","ru-ru":"Русский","sv-se":"Шведский","zh-cn":"Китайский (Китайская Народная Республика)","zh-hk":"Китайский (Гонконг, САР)","cs-cz":"Чешский","pt-pt":"Португальский (Португалия)","tr-tr":"Турецкий","vi-vn":"Вьетнамский","th-th":"Тайский","he-il":"Иврит","cy-gb":"Валлийский","uk-ua":"Украинский","el-gr":"Греческий","hu-hu":"Венгерский","ro-ro":"Румынский","sk-sk":"Словацкий","zh-tw":"Китайский (Тайвань)"};var captionLanguages$4={ar:"Арабский",da:"Датский",de:"Немецкий",en:"Английский",es:"Испанский",fi:"Финский","fr-ca":"Французский (Канада)",fr:"Французский (Франция)",hi:"Хинди",it:"Итальянский",ja:"Японский",ko:"Корейский",nb:"Норвежский (букмол)",nl:"Голландский",pl:"Польский",pt:"Португальский (Бразилия)",ru:"Русский",sv:"Шведский","zh-Hans":"Китайский (упрощенное письмо)","zh-Hant":"Китайский, традиционное письмо",cs:"Чешский","pt-pt":"Португальский (Португалия)",tr:"Турецкий",vi:"Вьетнамский",th:"Тайский",he:"Иврит",cy:"Валлийский",uk:"Украинский",el:"Греческий",hu:"Венгерский",ro:"Румынский",sk:"Словацкий"};var startCaptionsButton$4={onLabel:"Включить автоматические субтитры",offLabel:"Отключить автоматические субтитры",tooltipOnContent:"Отключить автоматические субтитры",tooltipOffContent:"Включить автоматические субтитры"};var startRealTimeTextButton$4={label:"Включить ввод в режиме реального времени для этого звонка",tooltipOffContent:"Включить ввод в режиме реального времени для этого звонка"};var messageThread$4={yesterday:"Вчера",sunday:"Воскресенье",monday:"Понедельник",tuesday:"Вторник",wednesday:"Среда",thursday:"Четверг",friday:"Пятница",saturday:"Суббота",participantJoined:"присоединился к чату.",participantLeft:"покинул чат.",editMessage:"Изменить",removeMessage:"Удалить",resendMessage:"Отправьте еще раз",failToSendTag:"Не удалось отправить",editedTag:"Изменено",liveAuthorIntro:"Сообщение от пользователя {author}",editedMessageLiveAuthorIntro:"Отредактированное сообщение от пользователя {author}",editedMessageLocalUserLiveAuthorIntro:"Отредактированное сообщение",messageContentAriaText:"{status} {author} говорит: {message}",messageContentMineAriaText:"{status} Вы сказали: {message}",editBoxTextLimit:"Превышено предельное число символов в {limitNumber} сообщении",editBoxPlaceholderText:"Отредактируйте сообщение",newMessagesIndicator:"Новые сообщения",noDisplayNameSub:"Без имени",editBoxCancelButton:"Отмена",editBoxSubmitButton:"Готово",messageReadCount:"Прочитали: {messageReadByCount} из {remoteParticipantsCount}",actionMenuMoreOptions:"Дополнительные параметры",downloadAttachment:"Скачать файл",openAttachment:"Открыть файл",blockedWarningText:"Это сообщение было удалено политикой организации.",blockedWarningLinkText:"Сведения",attachmentCardGroupMessage:"Сообщение содержит {attachmentCount} вложение",messageDeletedAnnouncementAriaLabel:"Сообщение удалено",richTextBoldTooltip:"Полужирный",richTextItalicTooltip:"Курсив",richTextUnderlineTooltip:"Подчеркнутый",richTextBulletListTooltip:"Маркированный список",richTextNumberListTooltip:"Нумерованный список",richTextIncreaseIndentTooltip:"Увеличить отступ",richTextDecreaseIndentTooltip:"Уменьшить отступ",richTextInsertTableTooltip:"Вставить таблицу",richTextFormatButtonTooltip:"Форматирование",richTextInsertRowOrColumnMenu:"Вставить",richTextInsertTableMenuTitle:"Вставить таблицу {column} x {row}",richTextInsertRowAboveMenu:"Вставить сверху",richTextInsertRowBelowMenu:"Вставить снизу",richTextInsertColumnLeftMenu:"Вставить слева",richTextInsertColumnRightMenu:"Вставить справа",richTextDeleteRowOrColumnMenu:"Удалить",richTextDeleteColumnMenu:"Удалить столбец",richTextDeleteRowMenu:"Удалить строку",richTextDeleteTableMenu:"Удалить таблицу",richTextToolbarMoreButtonAriaLabel:"Дополнительные параметры",attachmentUploadsPendingError:"Выполняется отправка файла. Подождите.",imageUploadsPendingError:"Выполняется отправка изображения. Подождите.",richTextNewBulletedListItemAnnouncement:"Маркер, новая строка",richTextNewNumberedListItemAnnouncement:"{0}, новая строка",richTextBulletedListAppliedAnnouncement:"Применен маркированный список",richTextNumberedListAppliedAnnouncement:"Применен нумерованный список"};var errorBar$4={unableToReachChatService:"Вы не в сети",accessDenied:"Не удалось получить доступ к службам чата. Проверьте предоставленные учетные данные пользователя",userNotInChatThread:"Вы больше не находитесь в цепочке этого чата",sendMessageNotInChatThread:"Не удалось отправить сообщение, так как вы больше не находитесь в цепочке этого чата",sendMessageGeneric:"Не удалось отправить сообщение",callingNetworkFailure:"Тревожное подключение вызова — кажется, вы не в сети",startVideoGeneric:"Не удалось запустить видео",stopVideoGeneric:"Не удалось остановить видео",muteGeneric:"Не удалось отключить микрофон",unmuteGeneric:"Не удалось включить микрофон",speakingWhileMuted:"Микрофон отключен",startScreenShareGeneric:"Возникла проблема с запуском демонстрации экрана.",stopScreenShareGeneric:"Не удалось остановить демонстрацию экрана",callNetworkQualityLow:"Качество работы сети: низкое.",teamsMeetingCallNetworkQualityLow:"Качество работы сети: низкое.",callNoSpeakerFound:"Динамики и наушники не найдены. Подключите звуковое устройство, чтобы услышать звонок.",callNoMicrophoneFound:"Микрофоны не найдены. Подключение звукового устройства ввода.",callMicrophoneAccessDenied:"Не удается получить доступ к микрофону. Щелкните блокировку в адресной строке, чтобы предоставить разрешение на эту веб-страницу.",callMicrophoneAccessDeniedSafari:"Не удается получить доступ к микрофону. Обновите страницу, чтобы разрешить разрешения, или проверьте параметры браузера и убедитесь, что для этого веб-сайта включены разрешения.",callMicrophoneMutedBySystem:"Система отключила ваш микрофон.",callMicrophoneUnmutedBySystem:"Микрофон восстановлен, и система включила ваш звук.",callMacOsMicrophoneAccessDenied:"Не удается получить доступ к микрофону. Предоставьте разрешение на доступ к микрофону в параметрах конфиденциальности macOS.",callLocalVideoFreeze:"Низкая пропускная способность сети. Ваше видео может отображаться как приостановленное для других участников вызова.",callCameraAccessDenied:"Не удается получить доступ к камере. Щелкните блокировку в адресной строке, чтобы предоставить разрешение на эту веб-страницу.",callCameraAccessDeniedSafari:"Не удается получить доступ к камере. Обновите страницу, чтобы разрешить разрешения, или проверьте параметры браузера и убедитесь, что для этого веб-сайта включены разрешения.",callCameraAlreadyInUse:"Не удается получить доступ к камере. Возможно, оно уже используется другим приложением.",callVideoStoppedBySystem:"Воспроизведение видео остановлено системой.",callVideoRecoveredBySystem:"Воспроизведение видео возобновлено.",callMacOsCameraAccessDenied:"MacOS блокирует доступ к камере. Обновите параметры конфиденциальности, чтобы разрешить этому браузеру доступ к камере.",callMacOsScreenShareAccessDenied:"MacOS блокирует демонстрацию экрана. Обновите параметры конфиденциальности, чтобы разрешить этому браузеру записывать ваш экран.",dismissButtonAriaLabel:"Закрыть",failedToJoinCallGeneric:"Не удалось присоединиться к звонку.",failedToJoinCallInvalidMeetingLink:"Не удалось присоединиться к собранию. Недопустимая ссылка.",cameraFrozenForRemoteParticipants:"У участников звонка возникают проблемы с просмотром вашего видео. Проверьте свои устройства и сеть.",unableToStartVideoEffect:"Не удалось применить эффект видео.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Не удается запустить закрепление участников, так как достигнуто предельное число закрепленных участников.",mutedByRemoteParticipant:"Ваш микрофон отключен другим участником."};var notificationStack$4={callingNetworkFailure:{title:"Проблемы с подключением к телефону",message:"Похоже, вы не в сети. Проверьте сетевое подключение и повторите попытку.",dismissButtonAriaLabel:"Закрыть"},startVideoGeneric:{title:"Не удалось запустить видео.",dismissButtonAriaLabel:"Закрыть"},stopVideoGeneric:{title:"Не удалось остановить видео.",dismissButtonAriaLabel:"Закрыть"},muteGeneric:{title:"Не удалось отключить микрофон.",dismissButtonAriaLabel:"Закрыть"},unmuteGeneric:{title:"Не удалось включить микрофон.",dismissButtonAriaLabel:"Закрыть"},speakingWhileMuted:{title:"Ваш микрофон отключен.",dismissButtonAriaLabel:"Закрыть"},startScreenShareGeneric:{title:"Возникла проблема с запуском демонстрации экрана.",dismissButtonAriaLabel:"Закрыть"},stopScreenShareGeneric:{title:"Не удалось остановить демонстрацию экрана.",dismissButtonAriaLabel:"Закрыть"},callNetworkQualityLow:{title:"Качество работы сети: низкое.",dismissButtonAriaLabel:"Закрыть"},teamsMeetingCallNetworkQualityLow:{title:"Качество работы сети: низкое.",dismissButtonAriaLabel:"Закрыть",message:"Присоединяйтесь к звонку с телефона, чтобы улучшить звук. Вы можете продолжить просмотр собрания на этом устройстве.",primaryButtonLabel:"Присоединиться по телефону"},callNoSpeakerFound:{title:"Динамики и наушники не найдены",message:"Подключите звуковое устройство, чтобы прослушать звонок.",dismissButtonAriaLabel:"Закрыть"},callNoMicrophoneFound:{title:"Микрофоны не найдены",message:"Подключите устройство ввода звука.",dismissButtonAriaLabel:"Закрыть"},callMicrophoneAccessDenied:{title:"Не удается получить доступ к микрофону",message:"Щелкните блокировку в адресной строке, чтобы предоставить разрешение на доступ к этой веб-странице.",dismissButtonAriaLabel:"Закрыть"},callMicrophoneAccessDeniedSafari:{title:"Не удается получить доступ к микрофону",message:"Обновите страницу, чтобы предоставить разрешения, или проверьте параметры браузера и включите разрешения для этого веб-сайта.",dismissButtonAriaLabel:"Закрыть"},callMicrophoneMutedBySystem:{title:"Система отключила ваш микрофон.",dismissButtonAriaLabel:"Закрыть"},callMicrophoneUnmutedBySystem:{title:"Микрофон восстановлен, и система включила ваш звук.",dismissButtonAriaLabel:"Закрыть"},callMacOsMicrophoneAccessDenied:{title:"Не удается получить доступ к микрофону",message:"Предоставьте разрешение на доступ к микрофону в параметрах конфиденциальности macOS.",dismissButtonAriaLabel:"Закрыть"},callLocalVideoFreeze:{title:"Низкая пропускная способность сети",message:"Ваше видео может отображаться как приостановленное для других участников звонка.",dismissButtonAriaLabel:"Закрыть"},callCameraAccessDenied:{title:"Не удается получить доступ к камере",message:"Щелкните блокировку в адресной строке, чтобы предоставить разрешение на доступ к этой веб-странице.",dismissButtonAriaLabel:"Закрыть"},callCameraAccessDeniedSafari:{title:"Не удается получить доступ к камере",message:"Обновите страницу, чтобы предоставить разрешения, или проверьте параметры браузера и включите разрешения для этого веб-сайта.",dismissButtonAriaLabel:"Закрыть"},callCameraAlreadyInUse:{title:"Не удается получить доступ к камере",message:"Возможно, оно уже используется другим приложением.",dismissButtonAriaLabel:"Закрыть"},callVideoStoppedBySystem:{title:"Воспроизведение видео остановлено системой.",dismissButtonAriaLabel:"Закрыть"},callVideoRecoveredBySystem:{title:"Воспроизведение видео возобновлено.",dismissButtonAriaLabel:"Закрыть"},callMacOsCameraAccessDenied:{title:"MacOS блокирует доступ к камере",message:"Обновите параметры конфиденциальности, чтобы разрешить этому браузеру доступ к камере.",dismissButtonAriaLabel:"Закрыть"},callMacOsScreenShareAccessDenied:{title:"MacOS блокирует совместное использование экрана",message:"Обновите параметры конфиденциальности, чтобы разрешить этому браузеру записывать ваш экран.",dismissButtonAriaLabel:"Закрыть"},failedToJoinCallGeneric:{title:"Не удалось присоединиться к вызову.",dismissButtonAriaLabel:"Закрыть"},failedToJoinCallInvalidMeetingLink:{title:"Не удалось присоединиться к собранию. Недопустимая ссылка.",dismissButtonAriaLabel:"Закрыть"},cameraFrozenForRemoteParticipants:{title:"У пользователей звонка возникли проблемы с просмотром видео",message:"Проверьте свои устройства и сеть.",dismissButtonAriaLabel:"Закрыть"},unableToStartVideoEffect:{title:"Не удалось применить эффект видео.",dismissButtonAriaLabel:"Закрыть"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Не удается запустить интересное для участников",message:"Максимальное число участников уже в фокусе.",dismissButtonAriaLabel:"Закрыть"},mutedByRemoteParticipant:{title:"Микрофон отключен другим участником.",dismissButtonAriaLabel:"Закрыть"},recordingStarted:{title:"Запись начата.",message:"Присоединяясь, вы даете согласие на транскрибирование этого собрания.",dismissButtonAriaLabel:"Закрыть"},transcriptionStarted:{title:"Транскрибирование запущено.",message:"Присоединяясь, вы даете согласие на транскрибирование этого собрания.",dismissButtonAriaLabel:"Закрыть"},recordingStopped:{title:"Сохранение записи.",message:"Запись прекращена.",dismissButtonAriaLabel:"Закрыть"},transcriptionStopped:{title:"Сохранение транскрибирования.",message:"Транскрибирование остановлено.",dismissButtonAriaLabel:"Закрыть"},recordingAndTranscriptionStarted:{title:"Начата запись и транскрибирование.",message:"Присоединяясь, вы даете согласие на транскрибирование этого собрания.",dismissButtonAriaLabel:"Закрыть"},recordingAndTranscriptionStopped:{title:"Сохранение записи и транскрибирования.",message:"Запись и транскрибирование остановлены.",dismissButtonAriaLabel:"Закрыть"},recordingStoppedStillTranscribing:{title:"Запись прекращена.",message:"Сейчас для этого собрания ведется только транскрибирование.",dismissButtonAriaLabel:"Закрыть"},transcriptionStoppedStillRecording:{title:"Транскрибирование остановлено.",message:"Сейчас для этого собрания ведется только запись.",dismissButtonAriaLabel:"Закрыть"},assignedBreakoutRoomOpened:{title:"Мы переместим вас в назначенную комнату через 5 секунд.",dismissButtonAriaLabel:"Закрыть"},assignedBreakoutRoomChanged:{title:"Переговорная комната изменилась",message:"Мы переместим вас в назначенную комнату через 5 секунд.",dismissButtonAriaLabel:"Закрыть"},assignedBreakoutRoomClosed:{title:"Ваша переговорная комната закрыта",message:"Мы перенастроим вас обратно на собрание через 5 секунд.",dismissButtonAriaLabel:"Закрыть"},assignedBreakoutRoomOpenedPromptJoin:{title:"Присоединиться к переговорной комнате?",message:"Вам назначена переговорная комната.",dismissButtonAriaLabel:"Закрыть",primaryButtonLabel:"Присоединиться к комнате",secondaryButtonLabel:"Позже"},breakoutRoomJoined:{title:"Вы присоединились к переговорной комнате.",dismissButtonAriaLabel:"Закрыть"},breakoutRoomClosingSoon:{title:"Скоро будет достигнуто ограничение времени для комнаты.",message:"Эта комната будет закрыта через 30 секунд",dismissButtonAriaLabel:"Закрыть"},capabilityTurnVideoOnPresent:{title:"Камера включена",message:"Включите этот параметр, если хотите демонстрировать видео.",dismissButtonAriaLabel:"Закрыть"},capabilityTurnVideoOnAbsent:{title:"Ваша камера отключена",message:"Вы больше не можете демонстрировать видео.",dismissButtonAriaLabel:"Закрыть"},capabilityUnmuteMicPresent:{title:"Ваш микрофон включен",message:"Включите микрофон, если хотите что-то сказать.",dismissButtonAriaLabel:"Закрыть"},capabilityUnmuteMicAbsent:{title:"Ваш микрофон отключен",message:"Вы больше не можете включать звук.",dismissButtonAriaLabel:"Закрыть"},togetherModeStarted:{title:"Режим \"Вместе\" запущен",dismissButtonAriaLabel:"Закрыть"},togetherModeEnded:{title:"Режим \"Вместе\" завершен",dismissButtonAriaLabel:"Закрыть"},transcriptionError:{title:"При транскрибировании произошла ошибка.",message:"Транскрибирование остановлено. Повторите попытку позже.",dismissButtonAriaLabel:"Закрыть"},transcriptionStartedByYou:{title:"Транскрибирование началось.",message:"Начато вами. Сообщите всем участникам, что их слова транскрибируются.",dismissButtonAriaLabel:"Закрыть"}};var videoGallery$4={screenIsBeingSharedMessage:"Вы демонстрируете свой экран",screenShareLoadingMessage:"Загрузка экрана пользователя {participant}",localScreenShareLoadingMessage:"Загрузка экрана",localVideoLabel:"Вы",localVideoCameraSwitcherLabel:"Переключить камеру",localVideoMovementLabel:"Перемещаемая локальная плитка видео",localVideoMovementAriaLabel:"Перемещаемая локальная плитка видео",localVideoSelectedDescription:"Выбрана камера {cameraName}",displayNamePlaceholder:"Участник без имени",fitRemoteParticipantToFrame:"Вписать в кадр",fillRemoteParticipantFrame:"Заполнить кадр",pinParticipantForMe:"Закрепить для меня",pinParticipantForMeLimitReached:"Закрепить (достигнуто ограничение)",unpinParticipantForMe:"Открепить",pinParticipantMenuItemAriaLabel:"Закрепить участника {participantName}",unpinParticipantMenuItemAriaLabel:"Открепить участника {participantName}",pinnedParticipantAnnouncementAriaLabel:"Участник {participantName} закреплен",unpinnedParticipantAnnouncementAriaLabel:"Участник {participantName} откреплен",startSpotlightVideoTileMenuLabel:"Закрепить видео для всех",addSpotlightVideoTileMenuLabel:"Добавить закрепление",spotlightLimitReachedMenuTitle:"Достигнуто предельное число закреплений",stopSpotlightVideoTileMenuLabel:"Не закреплять",stopSpotlightOnSelfVideoTileMenuLabel:"Открепить",attendeeRole:"Участник",muteParticipantMenuItemLabel:"Отключить микрофон",waitingScreenText:"Ожидание присоединения других участников",forbidAudio:"Отключить микрофон",permitAudio:"Разрешить использовать микрофон",forbidAudioTileMenuLabel:"Отключить микрофон",permitAudioTileMenuLabel:"Разрешить использовать микрофон",forbidVideo:"Выключить камеру",permitVideo:"Разрешить использовать камеру",forbidVideoTileMenuLabel:"Выключить камеру",permitVideoTileMenuLabel:"Разрешить использовать камеру"};var dialpad$4={placeholderText:"Введите номер телефона",deleteButtonAriaLabel:"Удалить"};var holdButton$4={onLabel:"Возобновить",offLabel:"Удержать",tooltipOnContent:"Возобновить звонок",tooltipOffContent:"Поставить звонок на удержание"};var videoTile$4={participantStateRinging:"Выполняется вызов...",participantReconnecting:"Повторное подключение...",participantStateHold:"На удержании",moreOptionsButtonAriaLabel:"Дополнительные параметры {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"звук отключен",moreOptionsParticipantMutedStateUnmutedAriaLabel:"звук включен",moreOptionsParticipantHandRaisedAriaLabel:"Поднята рука",moreOptionsParticipantIsSpeakingAriaLabel:"Говорит",moreOptionsParticipantMicDisabledAriaLabel:"Микрофон отключен",moreOptionsParticipantCameraDisabledAriaLabel:"Камера отключена"};var CameraAndMicrophoneSitePermissionsRequest$4={primaryText:"Разрешить {appName} использовать камеру и микрофон",secondaryText:"Это необходимо для того, чтобы участники могли видеть и слышать вас.",linkText:"Нужна помощь? Получите справку по устранению неполадок",primaryButtonText:"Продолжить без камеры и микрофона",ariaLabel:"Разрешить доступ к камере и микрофону"};var CameraSitePermissionsRequest$4={primaryText:"Разрешите приложению {appName} использовать камеру",secondaryText:"Это необходимо для того, чтобы участники могли вас видеть.",linkText:"Нужна помощь? Получите справку по устранению неполадок",primaryButtonText:"Продолжить без камеры",ariaLabel:"Разрешить доступ к камере"};var MicrophoneSitePermissionsRequest$4={primaryText:"Разрешите приложению {appName} использовать микрофон",secondaryText:"Это делается для того, чтобы участники могли вас слышать.",linkText:"Нужна помощь? Получите справку по устранению неполадок",primaryButtonText:"Продолжить без микрофона",ariaLabel:"Разрешить доступ к микрофону"};var CameraAndMicrophoneSitePermissionsCheck$4={primaryText:"Проверка доступа к камере и микрофону",secondaryText:"Разрешить доступ при запросе. Это делается для того, чтобы участники могли вас видеть и слышать.",linkText:"Нужна помощь? Получите справку по устранению неполадок",primaryButtonText:"Продолжить без камеры и микрофона",ariaLabel:"Проверка доступа к камере и микрофону. Разрешить доступ при запросе."};var CameraSitePermissionsCheck$4={primaryText:"Проверка доступа к камере",secondaryText:"Разрешить доступ при запросе. Это необходимо для того, чтобы участники могли вас видеть.",linkText:"Нужна помощь? Получите справку по устранению неполадок",primaryButtonText:"Продолжить без камеры",ariaLabel:"Проверка доступа к камере. Разрешить доступ при запросе."};var MicrophoneSitePermissionsCheck$4={primaryText:"Проверка доступа к микрофону",secondaryText:"Разрешить доступ при запросе. Это делается для того, чтобы участники могли вас слышать.",linkText:"Нужна помощь? Получите справку по устранению неполадок",primaryButtonText:"Продолжить без микрофона",ariaLabel:"Проверка доступа к микрофону. Разрешить доступ при запросе."};var CameraAndMicrophoneSitePermissionsDenied$4={primaryText:"Не удается получить доступ к камере и микрофону",secondaryText:"Щелкните значок блокировки в адресной строке, чтобы предоставить разрешение на доступ к микрофону для этой веб-страницы. Может потребоваться обновление страницы.",primaryButtonText:"Продолжить без камеры и микрофона",linkText:"Нужна помощь? Получите справку по устранению неполадок"};var CameraAndMicrophoneSitePermissionsDeniedSafari$4={primaryText:"Не удается получить доступ к камере и микрофону",secondaryText:"Обновите страницу, чтобы предоставить разрешения, или проверьте параметры браузера и включите разрешения для этого веб-сайта.",primaryButtonText:"Продолжить без камеры и микрофона",linkText:"Нужна помощь? Получите справку по устранению неполадок"};var CameraSitePermissionsDenied$4={primaryText:"Не удается получить доступ к камере",secondaryText:"Щелкните значок блокировки в адресной строке, чтобы предоставить разрешения камеры для этой веб-страницы. Может потребоваться обновить страницу.",primaryButtonText:"Продолжить без камеры",linkText:"Нужна помощь? Получите справку по устранению неполадок"};var MicrophoneSitePermissionsDenied$4={primaryText:"Не удается получить доступ к микрофону",secondaryText:"Щелкните значок блокировки в адресной строке, чтобы предоставить разрешение на доступ к микрофону для этой веб-страницы. Может потребоваться обновление страницы.",primaryButtonText:"Продолжить без микрофона",linkText:"Нужна помощь? Получите справку по устранению неполадок"};var CameraSitePermissionsDeniedSafari$4={primaryText:"Не удается получить доступ к камере",secondaryText:"Обновите страницу, чтобы предоставить разрешения, или проверьте параметры браузера и включите разрешения для этого веб-сайта.",primaryButtonText:"Продолжить без камеры",linkText:"Нужна помощь? Получите справку по устранению неполадок"};var MicrophoneSitePermissionsDeniedSafari$4={primaryText:"Не удается получить доступ к микрофону",secondaryText:"Обновите страницу, чтобы предоставить разрешения, или проверьте параметры браузера и включите разрешения для этого веб-сайта.",primaryButtonText:"Продолжить без микрофона",linkText:"Нужна помощь? Получите справку по устранению неполадок"};var UnsupportedBrowser$4={primaryText:"Браузер не поддерживается",secondaryText:"Присоединитесь к этому вызову с помощью совместимого браузера.",moreHelpLinkText:"См. требования к совместимости"};var UnsupportedBrowserVersion$4={primaryText:"Необходимо обновление браузера",secondaryText:"Чтобы обеспечить наилучший возможный звонок, обновите браузер и попробуйте присоединиться к звонку еще раз.",moreHelpLinkText:"См. требования к совместимости",continueAnywayButtonText:"Начать звонок без обновления"};var UnsupportedOperatingSystem$4={primaryText:"Операционная система не поддерживается",secondaryText:"Присоединитесь к этому вызову, используя устройство с совместимой операционной системой.",moreHelpLinkText:"См. требования к совместимости"};var BrowserPermissionDenied$4={primaryText:"Не удается использовать камеру или микрофон",secondaryText:"Возможно, у вашего браузера нет доступа к камере или микрофону. Чтобы устранить эту проблему, откройте параметры системы.",primaryButtonText:"Повторить попытку",linkText:"Нужна помощь? Получите справку по устранению неполадок"};var BrowserPermissionDeniedIOS$4={primaryText:"Разрешите доступ к микрофону, чтобы продолжить",secondaryText:"Таким образом, другие участники могут вас слышать.",primaryButtonText:"Повторить попытку",imageAltText:"Разрешение на доступ к микрофону и камере для iOS",linkText:"Нужна помощь? Получите справку по устранению неполадок",step1Text:"Перейдите в приложение \"Параметры\"",step2Text:"Прокрутите страницу вниз до параметров этого браузера",step3Text:"Включите микрофон (камера необязательна)",step4Text:"Попробуйте присоединиться к вызову еще раз",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$4={leftNavButtonAriaLabel:"предыдущая страница {current} из {total}",rightNavButtonAriaLabel:"следующая страница {current} из {total}"};var meetingConferencePhoneInfo$4={meetingConferencePhoneInfoModalTitle:"Присоединиться к вызову с телефона",meetingConferencePhoneInfoModalDialIn:"Набрать номер",meetingConferencePhoneInfoModalMeetingId:"Введите идентификатор собрания",meetingConferencePhoneInfoModalWait:"Подождите, пока организатор допустит вас",meetingConferencePhoneInfoModalNoPhoneAvailable:"Нет доступных телефонов",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(бесплатный)",meetingConferencePhoneInfoModalToll:"(платный)"};var IncomingCallNotification$4={incomingCallNotificationPlaceholderId:"Неизвестный пользователь",incomingCallNotificationPlaceholderAlert:"{callerName} звонит",incomingCallNoticicationAcceptWithAudioAriaLabel:"Ответить голосом",incomingCallNoticicationAcceptWithVideoAriaLabel:"Принять видеозвонок",incomingCallNoticicationRejectAriaLabel:"Отклонить",incomingCallNotificationAcceptButtonLabel:"Принять",incomingCallNotificationRejectButtonLabel:"Отклонить",incomingCallNotificationAccceptWithVideoButtonLabel:"Ответить с видео",incomingCallNotificationDismissButtonAriaLabel:"Закрыть"};var ru_RU$1 = {participantItem:participantItem$4,ParticipantList:ParticipantList$4,typingIndicator:typingIndicator$4,sendBox:sendBox$4,richTextSendBox:richTextSendBox$4,realTimeTextModal:realTimeTextModal$4,captionsBanner:captionsBanner$4,mentionPopover:mentionPopover$4,imageOverlay:imageOverlay$4,messageStatusIndicator:messageStatusIndicator$4,endCallButton:endCallButton$4,cameraButton:cameraButton$4,microphoneButton:microphoneButton$4,devicesButton:devicesButton$4,participantsButton:participantsButton$4,screenShareButton:screenShareButton$4,raiseHandButton:raiseHandButton$4,reactionButton:reactionButton$4,realTimeText:realTimeText$4,captionsSettingsModal:captionsSettingsModal$4,spokenLanguages:spokenLanguages$4,captionLanguages:captionLanguages$4,startCaptionsButton:startCaptionsButton$4,startRealTimeTextButton:startRealTimeTextButton$4,messageThread:messageThread$4,errorBar:errorBar$4,notificationStack:notificationStack$4,videoGallery:videoGallery$4,dialpad:dialpad$4,holdButton:holdButton$4,videoTile:videoTile$4,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$4,CameraSitePermissionsRequest:CameraSitePermissionsRequest$4,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$4,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$4,CameraSitePermissionsCheck:CameraSitePermissionsCheck$4,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$4,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$4,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$4,CameraSitePermissionsDenied:CameraSitePermissionsDenied$4,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$4,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$4,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$4,UnsupportedBrowser:UnsupportedBrowser$4,UnsupportedBrowserVersion:UnsupportedBrowserVersion$4,UnsupportedOperatingSystem:UnsupportedOperatingSystem$4,BrowserPermissionDenied:BrowserPermissionDenied$4,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$4,verticalGallery:verticalGallery$4,meetingConferencePhoneInfo:meetingConferencePhoneInfo$4,IncomingCallNotification:IncomingCallNotification$4};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Russian (Russia).
 *
 * @public
 */
const COMPONENT_LOCALE_RU_RU = {
    strings: createComponentStrings(ru_RU$1)
};

var participantItem$3={isMeText:"(du)",menuTitle:"Fler alternativ",removeButtonLabel:"Ta bort",sharingIconLabel:"Delar",mutedIconLabel:"Ljudet är avstängt",handRaisedIconLabel:"Upphöjt hand – position {position}",displayNamePlaceholder:"Namnlös deltagare",participantStateRinging:"Ringer...",participantStateHold:"Parkerat",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, fler alternativ",attendeeRole:"Deltagare",micDisabledIconLabel:"Mikrofonen är inaktiverad",cameraDisabledIconLabel:"Kameran är inaktiverad"};var ParticipantList$3={overflowParticipantCount:"+{overflowCount} till"};var typingIndicator$3={singleUser:"{user} skriver...",multipleUsers:"{users} skriver...",multipleUsersAbbreviateOne:"{users} och 1 annan skriver...",multipleUsersAbbreviateMany:"{users} och {numOthers} andra skriver...",delimiter:", "};var sendBox$3={placeholderText:"Skriv ett meddelande",textTooLong:"Meddelandelängden överskrider maxgränsen.",sendButtonAriaLabel:"Skicka meddelande",attachmentUploadsPendingError:"Filen laddas upp. Vänta.",removeAttachment:"Ta bort fil",uploading:"Laddar upp",uploadCompleted:"Uppladdningen har slutförts",attachmentMoreMenu:"Fler alternativ"};var richTextSendBox$3={placeholderText:"Skriv ett meddelande",textTooLong:"Meddelandelängden överskrider maxgränsen.",sendButtonAriaLabel:"Skicka meddelande",attachmentUploadsPendingError:"Filen laddas upp. Vänta.",imageUploadsPendingError:"Bilden laddas upp. Vänta.",removeAttachment:"Ta bort filen",uploading:"Laddar upp",uploadCompleted:"Uppladdningen har slutförts",richTextBoldTooltip:"Fet",richTextItalicTooltip:"Kursiv",richTextUnderlineTooltip:"Understruken",richTextBulletListTooltip:"Punktlista",richTextNumberListTooltip:"Numrerad lista",richTextIncreaseIndentTooltip:"Öka indrag",richTextDecreaseIndentTooltip:"Minska indrag",richTextInsertTableTooltip:"Infoga tabell",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Infoga",richTextInsertTableMenuTitle:"Infoga tabell på {column} x {row}",richTextInsertRowAboveMenu:"Infoga ovan",richTextInsertRowBelowMenu:"Infoga nedanför",richTextInsertColumnLeftMenu:"Infoga till vänster",richTextInsertColumnRightMenu:"Infoga till höger",richTextDeleteRowOrColumnMenu:"Ta bort",richTextDeleteColumnMenu:"Ta bort kolumn",richTextDeleteRowMenu:"Ta bort rad",richTextDeleteTableMenu:"Ta bort tabell",richTextToolbarAriaLabel:"Verktygsfältet Meddelandeformatering",richTextToolbarMoreButtonAriaLabel:"Fler alternativ",attachmentMoreMenu:"Fler alternativ",richTextNewBulletedListItemAnnouncement:"Punkt, ny rad",richTextNewNumberedListItemAnnouncement:"{0}, ny rad",richTextBulletedListAppliedAnnouncement:"Punktlista tillämpad",richTextNumberedListAppliedAnnouncement:"Numrerad tillämpad"};var realTimeTextModal$3={realTimeTextModalAriaLabel:"Aktivera RTT-modal",realTimeTextModalTitle:"Aktivera RTT?",realTimeTextModalText:"Realtidstext (RTT) aktiveras för alla i mötet.",realTimeTextConfirmButtonLabel:"Aktivera",realTimeTextCancelButtonLabel:"Avbryt",realTimeTextCloseModalButtonAriaLabel:"Stäng RTT-modal"};var captionsBanner$3={captionsBannerSpinnerText:"Startar undertexter ...",realTimeTextInputBoxDefaultText:"Skriv meddelande i realtid",realTimeTextInputErrorMessage:"Meddelandet får inte överstiga 2 000 tecken",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (realtidstext) är aktiverat för alla deltagare under hela mötets varaktighet.",realTimeTextBannerLinkLabel:"Mer information",captionsOnlyContainerTitle:"Undertext",realTimeTextOnlyContainerTitle:"Text i realtid",captionsAndRealTimeTextContainerTitle:"Undertexter och text i realtid",expandButtonAriaLabel:"Visa",minimizeButtonAriaLabel:"Dölj"};var mentionPopover$3={mentionPopoverHeader:"Förslag"};var imageOverlay$3={downloadButtonLabel:"Ladda ner",dismissButtonAriaLabel:"Stäng",overlayVisibleMessage:"Bilddetaljöverlägg"};var messageStatusIndicator$3={deliveredAriaLabel:"Meddelandet har skickats",deliveredTooltipText:"Skickat",seenAriaLabel:"Meddelande som andra ser",seenTooltipText:"Sett",readByTooltipText:"Läst av {messageThreadReadCount} av {remoteParticipantsCount}",sendingAriaLabel:"Meddelandet skickas",sendingTooltipText:"Skickar",failedToSendAriaLabel:"Meddelandet skickades inte",failedToSendTooltipText:"Inte skickat"};var endCallButton$3={label:"Lämna",tooltipContent:"Lämna samtal",leaveOption:"Lämna",endCallOption:"Avsluta samtal för alla"};var cameraButton$3={onLabel:"Kamera",offLabel:"Kamera",tooltipDisabledContent:"Kameran är inaktiverad",tooltipOnContent:"Stäng av kameran",tooltipOffContent:"Aktivera kameran",tooltipVideoLoadingContent:"Videon läses in",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Välj kamera",cameraButtonSplitRoleDescription:"Delningsknapp",onSplitButtonAriaLabel:"Stäng av kameran",offSplitButtonAriaLabel:"Aktivera kameran",cameraActionTurnedOnAnnouncement:"Kameran har aktiverats",cameraActionTurnedOffAnnouncement:"Kameran har stängts av",onSplitButtonPrimaryActionCamera:"Stäng av kameran",offSplitButtonPrimaryActionCamera:"Aktivera kameran",cameraPrimaryActionSplitButtonTitle:"Använd kamera",videoEffectsMenuItemTitle:"Effekter",cameraButtonAriaDescription:"kameraalternativ"};var microphoneButton$3={onLabel:"Mikrofon",offLabel:"Mikrofon",tooltipDisabledContent:"Mikrofonen är inaktiverad",tooltipOnContent:"Stäng av mikrofonen",tooltipOffContent:"Slå på mikrofonen",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Välj mikrofon",speakerMenuTitle:"Högtalare",speakerMenuTooltip:"Välj högtalare",microphoneButtonSplitRoleDescription:"Delningsknapp",onSplitButtonAriaLabel:"Inaktivera mikrofonen",offSplitButtonAriaLabel:"Slå på mikrofonen",microphoneActionTurnedOnAnnouncement:"Mikrofonen har aktiverats",microphoneActionTurnedOffAnnouncement:"Mikrofonen har inaktiverats",offSplitButtonMicrophonePrimaryAction:"Slå på mikrofonen",onSplitButtonMicrophonePrimaryAction:"Inaktivera mikrofonen",microphonePrimaryActionSplitButtonTitle:"Använd mikrofon",microphoneAriaDescription:"Ljudalternativ",deepNoiseSuppressionTitle:"Ljuddämpning",deepNoiseSuppressionOnAnnouncement:"Djup brusreducering har aktiverats",deepNoiseSuppressionOffAnnouncement:"Djup brusreducering har inaktiverats"};var devicesButton$3={label:"Enheter",tooltipContent:"Hantera enheter",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Välj kamera",audioDeviceMenuTitle:"Ljudenhet",audioDeviceMenuTooltip:"Välj ljudenhet",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Välj mikrofon",speakerMenuTitle:"Högtalare",speakerMenuTooltip:"Välj högtalare"};var participantsButton$3={label:"Personer",ariaLabel:"Kontakter",tooltipContent:"Visa deltagare",menuHeader:"I det här samtalet",participantsListButtonLabel:"{numParticipants} personer",muteAllButtonLabel:"Ljud av för alla",copyInviteLinkButtonLabel:"Kopiera inbjudningslänk",copyInviteLinkActionedAriaLabel:"Inbjudningslänken har kopierats"};var screenShareButton$3={onLabel:"Sluta presentera",offLabel:"Present",tooltipDisabledContent:"Presentationen är inaktiverad",tooltipOnContent:"Visar din skärm",tooltipOffContent:"Dela skärmen"};var raiseHandButton$3={offLabel:"Hand",onLabel:"Lägre",tooltipDisabledContent:"Åtgärden Räck upp hand är inaktiverad",tooltipOnContent:"Ta ned handen",tooltipOffContent:"Räck upp handen"};var reactionButton$3={label:"Reagera",ariaLabel:"Reagera",tooltipDisabledContent:"Reaktionsåtgärden är inaktiverad",tooltipContent:"Skicka en reaktion",likeReactionTooltipContent:"Gilla",heartReactionTooltipContent:"Kärlek",laughReactionTooltipContent:"Skrattar",applauseReactionTooltipContent:"Applåd",surprisedReactionTooltipContent:"Förvånad"};var realTimeText$3={isTypingText:"RTT-inmatning"};var captionsSettingsModal$3={captionsSettingsModalTitle:"Inställningar för undertexter",captionsSettingsSpokenLanguageDropdownLabel:"Talat språk",captionsSettingsCaptionLanguageDropdownLabel:"Språk för undertexter",captionsSettingsSpokenLanguageDropdownInfoText:"Språk som alla i det här samtalet talar.",captionsSettingsCaptionLanguageDropdownInfoText:"Undertexter visas på det här språket.",captionsSettingsConfirmButtonLabel:"Bekräfta",captionsSettingsCancelButtonLabel:"Avbryt",captionsSettingsModalAriaLabel:"Modalt fönster för inställning av undertexter",captionsSettingsCloseModalButtonAriaLabel:"Stäng inställningar för undertexter"};var spokenLanguages$3={"ar-ae":"Arabiska – Förenade Arabemiraten","ar-sa":"Arabiska – Saudiarabien","da-dk":"Danska","de-de":"Tyska – Tyskland","en-au":"Engelska – Australien","en-ca":"Engelska – Kanada","en-gb":"Engelska – Storbritannien","en-in":"Engelska – Indien","en-nz":"Engelska – Nya Zeeland","en-us":"Engelska – USA","es-es":"Spanska – Spanien (modern sortering)","es-mx":"Spanska – Mexiko","fi-fi":"Finska","fr-ca":"Franska – Kanada","fr-fr":"Franska – Frankrike","hi-in":"Hindi","it-it":"Italienska – Italien","ja-jp":"Japanska","ko-kr":"Koreanska","nb-no":"Norska (Bokmål)","nl-be":"Nederländska – Belgien","nl-nl":"Nederländska -–Nederländerna","pl-pl":"Polska","pt-br":"Portugisiska – Brasilien","ru-ru":"Ryska","sv-se":"Svenska","zh-cn":"Kinesiska (Kina),","zh-hk":"Kinesiska – Hongkong SAR","cs-cz":"Tjeckiska","pt-pt":"Portugisiska – Portugal","tr-tr":"Turkiska","vi-vn":"Vietnamesiska","th-th":"Thailändska","he-il":"Hebreiska","cy-gb":"Walesiska","uk-ua":"Ukrainska","el-gr":"Grekiska","hu-hu":"Ungerska","ro-ro":"Rumänska","sk-sk":"Slovakiska","zh-tw":"Kinesiska – Taiwan"};var captionLanguages$3={ar:"Arabiska",da:"Danska",de:"Tyska",en:"Engelska",es:"Spanska",fi:"Finska","fr-ca":"Franska – Kanada",fr:"Franska – Frankrike",hi:"Hindi",it:"Italienska",ja:"Japanska",ko:"Koreanska",nb:"Norska (Bokmål)",nl:"Nederländska",pl:"Polska",pt:"Portugisiska – Brasilien",ru:"Ryska",sv:"Svenska","zh-Hans":"Kinesiska (förenklad)","zh-Hant":"Kinesiska (traditionell)",cs:"Tjeckiska","pt-pt":"Portugisiska – Portugal",tr:"Turkiska",vi:"Vietnamesiska",th:"Thailändska",he:"Hebreiska",cy:"Walesiska",uk:"Ukrainska",el:"Grekiska",hu:"Ungerska",ro:"Rumänska",sk:"Slovakiska"};var startCaptionsButton$3={onLabel:"Aktivera undertexter",offLabel:"Stäng av undertexter",tooltipOnContent:"Stäng av undertexter",tooltipOffContent:"Aktivera undertexter"};var startRealTimeTextButton$3={label:"Aktivera RTT för det här samtalet",tooltipOffContent:"Aktivera RTT för det här samtalet"};var messageThread$3={yesterday:"I går",sunday:"Söndag",monday:"Måndag",tuesday:"Tisdag",wednesday:"Onsdag",thursday:"Torsdag",friday:"Fredag",saturday:"Lördag",participantJoined:"gick med i chatten.",participantLeft:"lämnade chatten.",editMessage:"Redigera",removeMessage:"Ta bort",resendMessage:"Försök skicka igen",failToSendTag:"Inte skickat",editedTag:"Redigerad",liveAuthorIntro:"Meddelande från {author}",editedMessageLiveAuthorIntro:"Redigerat meddelande från {author}",editedMessageLocalUserLiveAuthorIntro:"Redigerat meddelande",messageContentAriaText:"{status} {author} sa {message}",messageContentMineAriaText:"{status} Du sa {message}",editBoxTextLimit:"Meddelandet överskrider gränsen på {limitNumber} tecken",editBoxPlaceholderText:"Redigera meddelandet",newMessagesIndicator:"Nya meddelanden",noDisplayNameSub:"Inget namn",editBoxCancelButton:"Avbryt",editBoxSubmitButton:"Klar",messageReadCount:"Läst av {messageReadByCount} av {remoteParticipantsCount}",actionMenuMoreOptions:"Fler alternativ",downloadAttachment:"Ladda ner fil",openAttachment:"Öppna fil",blockedWarningText:"Det här meddelandet har tagits bort på grund av en organisationsprincip.",blockedWarningLinkText:"Information",attachmentCardGroupMessage:"Meddelandet har {attachmentCount} bifogade filer.",messageDeletedAnnouncementAriaLabel:"Meddelandet har tagits bort",richTextBoldTooltip:"Fet",richTextItalicTooltip:"Kursiv",richTextUnderlineTooltip:"Understryka",richTextBulletListTooltip:"Punktlista",richTextNumberListTooltip:"Numrerad lista",richTextIncreaseIndentTooltip:"Öka indrag",richTextDecreaseIndentTooltip:"Minska indrag",richTextInsertTableTooltip:"Infoga tabell",richTextFormatButtonTooltip:"Format",richTextInsertRowOrColumnMenu:"Infoga",richTextInsertTableMenuTitle:"Infoga tabell på {column} x {row}",richTextInsertRowAboveMenu:"Infoga ovan",richTextInsertRowBelowMenu:"Infoga nedanför",richTextInsertColumnLeftMenu:"Infoga till vänster",richTextInsertColumnRightMenu:"Infoga till höger",richTextDeleteRowOrColumnMenu:"Radera",richTextDeleteColumnMenu:"Ta bort kolumn",richTextDeleteRowMenu:"Ta bort rad",richTextDeleteTableMenu:"Ta bort tabell",richTextToolbarMoreButtonAriaLabel:"Fler alternativ",attachmentUploadsPendingError:"Filen laddas upp. Vänta.",imageUploadsPendingError:"Bilden laddas upp. Vänta.",richTextNewBulletedListItemAnnouncement:"Punkt, ny rad",richTextNewNumberedListItemAnnouncement:"{0}, ny rad",richTextBulletedListAppliedAnnouncement:"Punktlista tillämpad",richTextNumberedListAppliedAnnouncement:"Numrerad tillämpad"};var errorBar$3={unableToReachChatService:"Du är offline",accessDenied:"Det går inte att komma åt chatttjänster – kontrollera de angivna autentiseringsuppgifterna",userNotInChatThread:"Du är inte längre i den här chatttråden",sendMessageNotInChatThread:"Det gick inte att skicka meddelandet eftersom du inte längre är i den här chatttråden",sendMessageGeneric:"Meddelandet inte skickat",callingNetworkFailure:"Störande anslutningssamtal – du verkar vara offline",startVideoGeneric:"Det gick inte att starta videon",stopVideoGeneric:"Det gick inte att stoppa videon",muteGeneric:"Det gick inte att stänga av mikrofonen",unmuteGeneric:"Det gick inte att slå på mikrofonen",speakingWhileMuted:"Mikrofonens ljud har stängts av",startScreenShareGeneric:"Det uppstod ett problem när skärmdelningen skulle startas.",stopScreenShareGeneric:"Det gick inte att stoppa skärmdelning",callNetworkQualityLow:"Nätverkskvaliteten är låg.",teamsMeetingCallNetworkQualityLow:"Nätverkskvaliteten är låg.",callNoSpeakerFound:"Inga högtalare eller hörlurar hittades. Anslut en ljudenhet för att höra samtalet.",callNoMicrophoneFound:"Inga mikrofoner hittades. Anslut en ljudinmatningsenhet.",callMicrophoneAccessDenied:"Det går inte att komma åt mikrofonen. Klicka på låset i adressfältet om du vill bevilja behörighet till den här webbsidan.",callMicrophoneAccessDeniedSafari:"Det går inte att komma åt mikrofonen. Uppdatera sidan om du vill tillåta behörigheter, eller kontrollera webbläsarens inställningar och kontrollera att behörigheter är aktiverade för den här webbplatsen.",callMicrophoneMutedBySystem:"Du är avstängt av systemet.",callMicrophoneUnmutedBySystem:"Mikrofonen har återställts och datorn har slagit på ljudet.",callMacOsMicrophoneAccessDenied:"Det går inte att komma åt mikrofonen. Bevilja mikrofonbehörighet i dina sekretessinställningar för macOS.",callLocalVideoFreeze:"Nätverksbandbredden är dålig. Din video kan visas pausad för andra i samtalet.",callCameraAccessDenied:"Det går inte att komma åt kameran. Klicka på låset i adressfältet om du vill bevilja behörighet till den här webbsidan.",callCameraAccessDeniedSafari:"Det går inte att komma åt kameran. Uppdatera sidan om du vill tillåta behörigheter, eller kontrollera webbläsarens inställningar och kontrollera att behörigheter är aktiverade för den här webbplatsen.",callCameraAlreadyInUse:"Det går inte att komma åt kameran. Den kanske redan används av ett annat program.",callVideoStoppedBySystem:"Din video har stoppats av systemet.",callVideoRecoveredBySystem:"Videon har återupptagits.",callMacOsCameraAccessDenied:"MacOS blockerar åtkomsten till kameran. Uppdatera dina sekretessinställningar så att den här webbläsaren får åtkomst till kameran.",callMacOsScreenShareAccessDenied:"MacOS blockerar skärmdelning. Uppdatera dina sekretessinställningar så att den här webbläsaren kan spela in din skärm.",dismissButtonAriaLabel:"Stäng",failedToJoinCallGeneric:"Det gick inte att ansluta till samtalet.",failedToJoinCallInvalidMeetingLink:"Det går inte att ansluta till mötet. Ogiltig länk.",cameraFrozenForRemoteParticipants:"Användare i samtalet har problem med att se din video. Kontrollera dina enheter och nätverk.",unableToStartVideoEffect:"Det går inte att tillämpa videoeffekten.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Det går inte att starta spotlight på deltagare eftersom det maximala antalet deltagare redan är uppmärksammat.",mutedByRemoteParticipant:"Du har stängts av av en annan deltagare."};var notificationStack$3={callingNetworkFailure:{title:"Störande anslutningssamtal",message:"Du verkar vara offline. Kontrollera nätverksanslutningen och försök igen.",dismissButtonAriaLabel:"Stäng"},startVideoGeneric:{title:"Det gick inte att starta videon.",dismissButtonAriaLabel:"Stäng"},stopVideoGeneric:{title:"Det gick inte att stoppa videon.",dismissButtonAriaLabel:"Stäng"},muteGeneric:{title:"Det gick inte att stänga av mikrofonen.",dismissButtonAriaLabel:"Stäng"},unmuteGeneric:{title:"Det gick inte att slå på mikrofonen.",dismissButtonAriaLabel:"Stäng"},speakingWhileMuted:{title:"Din mikrofon har stängts av.",dismissButtonAriaLabel:"Stäng"},startScreenShareGeneric:{title:"Det uppstod ett problem när skärmdelningen skulle startas.",dismissButtonAriaLabel:"Stäng"},stopScreenShareGeneric:{title:"Det gick inte att stoppa skärmdelningen.",dismissButtonAriaLabel:"Stäng"},callNetworkQualityLow:{title:"Nätverkskvaliteten är låg.",dismissButtonAriaLabel:"Stäng"},teamsMeetingCallNetworkQualityLow:{title:"Nätverkskvaliteten är låg.",dismissButtonAriaLabel:"Stäng",message:"Anslut till det här samtalet från din telefon för bättre ljud. Du kan fortsätta att visa mötet på den här enheten.",primaryButtonLabel:"Anslut via telefon"},callNoSpeakerFound:{title:"Inga högtalare eller hörlurar hittades",message:"Anslut en ljudenhet för att höra samtalet.",dismissButtonAriaLabel:"Stäng"},callNoMicrophoneFound:{title:"Inga mikrofoner hittades",message:"Anslut en ljudinmatningsenhet.",dismissButtonAriaLabel:"Stäng"},callMicrophoneAccessDenied:{title:"Det går inte att komma åt mikrofonen",message:"Klicka på låset i adressfältet om du vill bevilja behörighet till den här webbsidan.",dismissButtonAriaLabel:"Stäng"},callMicrophoneAccessDeniedSafari:{title:"Det går inte att komma åt mikrofonen",message:"Uppdatera sidan om du vill tillåta behörigheter, eller kontrollera webbläsarens inställningar och kontrollera att behörigheter är aktiverade för den här webbplatsen.",dismissButtonAriaLabel:"Stäng"},callMicrophoneMutedBySystem:{title:"Du är avstängt av systemet.",dismissButtonAriaLabel:"Stäng"},callMicrophoneUnmutedBySystem:{title:"Mikrofonen har återställts och datorn har slagit på ljudet.",dismissButtonAriaLabel:"Stäng"},callMacOsMicrophoneAccessDenied:{title:"Det går inte att komma åt mikrofonen",message:"Bevilja mikrofonbehörighet i dina macOS-sekretessinställningar.",dismissButtonAriaLabel:"Stäng"},callLocalVideoFreeze:{title:"Nätverksbandbredden är dålig",message:"Din video kan visas pausad för andra i samtalet.",dismissButtonAriaLabel:"Stäng"},callCameraAccessDenied:{title:"Det går inte att komma åt kameran",message:"Klicka på låset i adressfältet om du vill bevilja behörighet till den här webbsidan.",dismissButtonAriaLabel:"Stäng"},callCameraAccessDeniedSafari:{title:"Det går inte att komma åt kameran",message:"Uppdatera sidan om du vill tillåta behörigheter, eller kontrollera webbläsarens inställningar och kontrollera att behörigheter är aktiverade för den här webbplatsen.",dismissButtonAriaLabel:"Stäng"},callCameraAlreadyInUse:{title:"Det går inte att komma åt kameran",message:"Den kanske redan används av ett annat program.",dismissButtonAriaLabel:"Stäng"},callVideoStoppedBySystem:{title:"Din video har stoppats av systemet.",dismissButtonAriaLabel:"Stäng"},callVideoRecoveredBySystem:{title:"Videon har återupptagits.",dismissButtonAriaLabel:"Stäng"},callMacOsCameraAccessDenied:{title:"MacOS blockerar åtkomsten till kameran",message:"Uppdatera dina sekretessinställningar så att den här webbläsaren får åtkomst till kameran.",dismissButtonAriaLabel:"Stäng"},callMacOsScreenShareAccessDenied:{title:"MacOS blockerar skärmdelning",message:"Uppdatera dina sekretessinställningar så att den här webbläsaren kan spela in din skärm.",dismissButtonAriaLabel:"Stäng"},failedToJoinCallGeneric:{title:"Det gick inte att ansluta till samtalet.",dismissButtonAriaLabel:"Stäng"},failedToJoinCallInvalidMeetingLink:{title:"Det går inte att ansluta till mötet. Ogiltig länk.",dismissButtonAriaLabel:"Stäng"},cameraFrozenForRemoteParticipants:{title:"Användare i samtalet har problem med att se din video",message:"Kontrollera dina enheter och nätverk.",dismissButtonAriaLabel:"Stäng"},unableToStartVideoEffect:{title:"Det går inte att tillämpa videoeffekten.",dismissButtonAriaLabel:"Stäng"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Det går inte att starta spotlight på deltagare",message:"Det maximala antalet deltagare är redan uppmärksammat.",dismissButtonAriaLabel:"Stäng"},mutedByRemoteParticipant:{title:"Du har stängts av av en annan deltagare.",dismissButtonAriaLabel:"Stäng"},recordingStarted:{title:"Inspelningen har startat.",message:"Genom att ansluta ger du ditt medgivande för att det här mötet ska transkriberas.",dismissButtonAriaLabel:"Stäng"},transcriptionStarted:{title:"Transkriptionen har startat.",message:"Genom att ansluta ger du ditt medgivande för att det här mötet ska transkriberas.",dismissButtonAriaLabel:"Stäng"},recordingStopped:{title:"Inspelningen sparas.",message:"Inspelningen har stoppats.",dismissButtonAriaLabel:"Stäng"},transcriptionStopped:{title:"Transkriptionen sparas.",message:"Transkriptionen har avslutats.",dismissButtonAriaLabel:"Stäng"},recordingAndTranscriptionStarted:{title:"Inspelning och transkription har startats.",message:"Genom att ansluta ger du ditt medgivande för att det här mötet ska transkriberas.",dismissButtonAriaLabel:"Stäng"},recordingAndTranscriptionStopped:{title:"Inspelning och transkription sparas.",message:"Inspelning och transkription har avslutats.",dismissButtonAriaLabel:"Stäng"},recordingStoppedStillTranscribing:{title:"Inspelningen har stoppats.",message:"Nu transkriberar du bara det här mötet.",dismissButtonAriaLabel:"Stäng"},transcriptionStoppedStillRecording:{title:"Transkriptionen har avslutats.",message:"Du spelar nu bara in det här mötet.",dismissButtonAriaLabel:"Stäng"},assignedBreakoutRoomOpened:{title:"Du flyttas till det tilldelade rummet om 5 sekunder.",dismissButtonAriaLabel:"Stäng"},assignedBreakoutRoomChanged:{title:"Ditt grupprum har ändrats",message:"Du flyttas till det tilldelade rummet om 5 sekunder.",dismissButtonAriaLabel:"Stäng"},assignedBreakoutRoomClosed:{title:"Grupprummet har stängts",message:"Du flyttas tillbaka till mötet om 5 sekunder.",dismissButtonAriaLabel:"Stäng"},assignedBreakoutRoomOpenedPromptJoin:{title:"Vill du ansluta till grupprummet?",message:"Du har tilldelats till ett separat rum.",dismissButtonAriaLabel:"Stäng",primaryButtonLabel:"Anslut till rummet",secondaryButtonLabel:"Senare"},breakoutRoomJoined:{title:"Du har gått med i ett grupprum.",dismissButtonAriaLabel:"Stäng"},breakoutRoomClosingSoon:{title:"Tidsgränsen för rummet upphör snart att gälla.",message:"Det här rummet stängs om 30 sekunder",dismissButtonAriaLabel:"Stäng"},capabilityTurnVideoOnPresent:{title:"Kameran har aktiverats",message:"Aktivera om du vill dela video.",dismissButtonAriaLabel:"Stäng"},capabilityTurnVideoOnAbsent:{title:"Kameran har inaktiverats",message:"Du kan inte längre dela video.",dismissButtonAriaLabel:"Stäng"},capabilityUnmuteMicPresent:{title:"Mikrofonen har aktiverats",message:"Slå på ljudet om du vill prata.",dismissButtonAriaLabel:"Stäng"},capabilityUnmuteMicAbsent:{title:"Mikrofonen har inaktiverats",message:"Du kan inte längre slå på ljudet.",dismissButtonAriaLabel:"Stäng"},togetherModeStarted:{title:"Tillsammans-läge har startats",dismissButtonAriaLabel:"Stäng"},togetherModeEnded:{title:"Tillsammans-läge har avslutats",dismissButtonAriaLabel:"Stäng"},transcriptionError:{title:"Ett fel uppstod med transkriptionen.",message:"Transkriptionen har stoppats. Försök igen senare",dismissButtonAriaLabel:"Stäng"},transcriptionStartedByYou:{title:"Transkriptionen har startat.",message:"Startades av dig. Berätta för alla att de transkriberas.",dismissButtonAriaLabel:"Stäng"}};var videoGallery$3={screenIsBeingSharedMessage:"Du delar din skärm",screenShareLoadingMessage:"Läser in {participant}s skärm",localScreenShareLoadingMessage:"Läser in skärmen",localVideoLabel:"Du",localVideoCameraSwitcherLabel:"Växla kamera",localVideoMovementLabel:"Flyttbar lokal videopanel",localVideoMovementAriaLabel:"Flyttbar lokal videopanel",localVideoSelectedDescription:"{cameraName} har valts",displayNamePlaceholder:"Namnlös deltagare",fitRemoteParticipantToFrame:"Anpassa till ruta",fillRemoteParticipantFrame:"Fyll ruta",pinParticipantForMe:"Fäst åt mig",pinParticipantForMeLimitReached:"Fäst (gränsen har nåtts)",unpinParticipantForMe:"Ta bort",pinParticipantMenuItemAriaLabel:"Fäst {participantName}",unpinParticipantMenuItemAriaLabel:"Ta bort {participantName}",pinnedParticipantAnnouncementAriaLabel:"Fäst {participantName}",unpinnedParticipantAnnouncementAriaLabel:"Borttagen {participantName}",startSpotlightVideoTileMenuLabel:"Uppmärksamma för alla",addSpotlightVideoTileMenuLabel:"Lägg till uppmärksammande",spotlightLimitReachedMenuTitle:"Spotlight-gränsen har nåtts",stopSpotlightVideoTileMenuLabel:"Sluta uppmärksamma",stopSpotlightOnSelfVideoTileMenuLabel:"Spotlight",attendeeRole:"Deltagare",muteParticipantMenuItemLabel:"Stäng av ljudet",waitingScreenText:"Väntar på att andra ska ansluta",forbidAudio:"Inaktivera mikrofon",permitAudio:"Tillåt mikrofon",forbidAudioTileMenuLabel:"Inaktivera mikrofon",permitAudioTileMenuLabel:"Tillåt mikrofon",forbidVideo:"Inaktivera kameran",permitVideo:"Tillåt kamera",forbidVideoTileMenuLabel:"Inaktivera kameran",permitVideoTileMenuLabel:"Tillåt kamera"};var dialpad$3={placeholderText:"Ange telefonnummer",deleteButtonAriaLabel:"Ta bort"};var holdButton$3={onLabel:"Återuppta",offLabel:"Bevarande",tooltipOnContent:"Återuppta samtal",tooltipOffContent:"Parkera samtal"};var videoTile$3={participantStateRinging:"Ringer...",participantReconnecting:"Ansluter igen...",participantStateHold:"Parkerat",moreOptionsButtonAriaLabel:"Fler alternativ {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"ljud av",moreOptionsParticipantMutedStateUnmutedAriaLabel:"ljud på",moreOptionsParticipantHandRaisedAriaLabel:"Uppräckt hand",moreOptionsParticipantIsSpeakingAriaLabel:"Talar",moreOptionsParticipantMicDisabledAriaLabel:"Mikrofonen är inaktiverad",moreOptionsParticipantCameraDisabledAriaLabel:"Kameran är inaktiverad"};var CameraAndMicrophoneSitePermissionsRequest$3={primaryText:"Tillåt {appName} använda kameran och mikrofonen",secondaryText:"Det här är så att deltagarna kan se och höra dig.",linkText:"Behöver du hjälp? Få hjälp med felsökning",primaryButtonText:"Fortsätt utan kamera och mikrofon",ariaLabel:"Tillåt kamera- och mikrofonåtkomst"};var CameraSitePermissionsRequest$3={primaryText:"Tillåt {appName} att använda kameran",secondaryText:"Det här är så att deltagarna kan se dig.",linkText:"Behöver du hjälp? Få hjälp med felsökning",primaryButtonText:"Fortsätt utan kamera",ariaLabel:"Tillåt kameraåtkomst"};var MicrophoneSitePermissionsRequest$3={primaryText:"Tillåt {appName} att använda mikrofonen",secondaryText:"Det här är så att deltagarna kan höra dig.",linkText:"Behöver du hjälp? Få hjälp med felsökning",primaryButtonText:"Fortsätt utan mikrofon",ariaLabel:"Tillåt mikrofonåtkomst"};var CameraAndMicrophoneSitePermissionsCheck$3={primaryText:"Söker efter åtkomst till kameran och mikrofonen",secondaryText:"Tillåt åtkomst om du uppmanas till det. Det här är så att deltagarna kan se och höra dig.",linkText:"Behöver du hjälp? Få hjälp med felsökning",primaryButtonText:"Fortsätt utan kamera och mikrofon",ariaLabel:"Söker efter åtkomst till kameran och mikrofonen. Tillåt åtkomst om du uppmanas att göra det."};var CameraSitePermissionsCheck$3={primaryText:"Söker efter kameraåtkomst",secondaryText:"Tillåt åtkomst om du uppmanas att göra det. Det här är så att deltagarna kan se dig.",linkText:"Behöver du hjälp? Få hjälp med felsökning",primaryButtonText:"Fortsätt utan kamera",ariaLabel:"Söker efter kameraåtkomst. Tillåt åtkomst om du uppmanas att göra det."};var MicrophoneSitePermissionsCheck$3={primaryText:"Söker efter mikrofonåtkomst",secondaryText:"Tillåt åtkomst om du uppmanas till det. Det här är så att deltagarna kan höra dig.",linkText:"Behöver du hjälp? Få hjälp med felsökning",primaryButtonText:"Fortsätt utan mikrofon",ariaLabel:"Söker efter mikrofonåtkomst. Tillåt åtkomst om du uppmanas att göra det."};var CameraAndMicrophoneSitePermissionsDenied$3={primaryText:"Det går inte att komma åt kameran och mikrofonen",secondaryText:"Klicka på låsikonen i adressfältet om du vill bevilja mikrofonbehörighet till den här webbsidan. Det kan krävas en siduppdatering.",primaryButtonText:"Fortsätt utan kamera och mikrofon",linkText:"Behöver du hjälp? Få hjälp med felsökning"};var CameraAndMicrophoneSitePermissionsDeniedSafari$3={primaryText:"Det går inte att komma åt kameran och mikrofonen",secondaryText:"Uppdatera sidan om du vill tillåta behörigheter, eller kontrollera webbläsarens inställningar och kontrollera att behörigheter är aktiverade för den här webbplatsen.",primaryButtonText:"Fortsätt utan kamera och mikrofon",linkText:"Behöver du hjälp? Få hjälp med felsökning"};var CameraSitePermissionsDenied$3={primaryText:"Det går inte att komma åt kameran",secondaryText:"Klicka på låsikonen i adressfältet om du vill bevilja kamerabehörigheter till den här webbsidan. Det kan krävas en siduppdatering.",primaryButtonText:"Fortsätt utan kamera",linkText:"Behöver du hjälp? Få hjälp med felsökning"};var MicrophoneSitePermissionsDenied$3={primaryText:"Det går inte att komma åt mikrofonen",secondaryText:"Klicka på låsikonen i adressfältet om du vill bevilja mikrofonbehörighet till den här webbsidan. Det kan krävas en siduppdatering.",primaryButtonText:"Fortsätt utan mikrofon",linkText:"Behöver du hjälp? Få hjälp med felsökning"};var CameraSitePermissionsDeniedSafari$3={primaryText:"Det går inte att komma åt kameran",secondaryText:"Uppdatera sidan om du vill tillåta behörigheter, eller kontrollera webbläsarens inställningar och kontrollera att behörigheter är aktiverade för den här webbplatsen.",primaryButtonText:"Fortsätt utan kamera",linkText:"Behöver du hjälp? Få hjälp med felsökning"};var MicrophoneSitePermissionsDeniedSafari$3={primaryText:"Det går inte att komma åt mikrofonen",secondaryText:"Uppdatera sidan om du vill tillåta behörigheter, eller kontrollera webbläsarens inställningar och kontrollera att behörigheter är aktiverade för den här webbplatsen.",primaryButtonText:"Fortsätt utan mikrofon",linkText:"Behöver du hjälp? Få hjälp med felsökning"};var UnsupportedBrowser$3={primaryText:"Webbläsaren stöds inte",secondaryText:"Anslut till samtalet med en kompatibel webbläsare.",moreHelpLinkText:"Se kompatibilitetskrav"};var UnsupportedBrowserVersion$3={primaryText:"Uppdatering av webbläsare krävs",secondaryText:"Uppdatera webbläsaren och försök sedan ansluta till samtalet igen för att säkerställa bästa möjliga samtal.",moreHelpLinkText:"Se kompatibilitetskrav",continueAnywayButtonText:"Starta samtal utan att uppdatera"};var UnsupportedOperatingSystem$3={primaryText:"Operativsystemet stöds inte",secondaryText:"Anslut till det här samtalet med en enhet med ett kompatibelt operativsystem.",moreHelpLinkText:"Se kompatibilitetskrav"};var BrowserPermissionDenied$3={primaryText:"Det går inte att använda kameran eller mikrofonen",secondaryText:"Din webbläsare kanske inte har åtkomst till din kamera eller mikrofon. Åtgärda detta genom att öppna Systeminställningar.",primaryButtonText:"Försök igen",linkText:"Behöver du hjälp? Få hjälp med felsökning"};var BrowserPermissionDeniedIOS$3={primaryText:"Tillåt mikrofonåtkomst för att fortsätta",secondaryText:"Så att andra deltagare kan höra dig.",primaryButtonText:"Försök igen",imageAltText:"Behörighetsplats för mikrofon och kameraenhet för iOS",linkText:"Behöver du hjälp? Få hjälp med felsökning",step1Text:"Gå till appen Inställningar",step2Text:"Rulla ned till inställningarna för den här webbläsaren",step3Text:"Aktivera mikrofon (kamera valfritt)",step4Text:"Försök ansluta till samtalet igen",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$3={leftNavButtonAriaLabel:"föregående sida {current} av {total}",rightNavButtonAriaLabel:"nästa sida {current} av {total}"};var meetingConferencePhoneInfo$3={meetingConferencePhoneInfoModalTitle:"Anslut till samtal från din telefon",meetingConferencePhoneInfoModalDialIn:"Slå nummer",meetingConferencePhoneInfoModalMeetingId:"Ange mötes-ID",meetingConferencePhoneInfoModalWait:"Vänta på att få tillträde av organisatören",meetingConferencePhoneInfoModalNoPhoneAvailable:"Ingen telefon är tillgänglig",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Avgiftsfritt)",meetingConferencePhoneInfoModalToll:"(Avgiftsbelagt)"};var IncomingCallNotification$3={incomingCallNotificationPlaceholderId:"Okänd användare",incomingCallNotificationPlaceholderAlert:"{callerName} ringer",incomingCallNoticicationAcceptWithAudioAriaLabel:"Acceptera med ljud",incomingCallNoticicationAcceptWithVideoAriaLabel:"Acceptera med video",incomingCallNoticicationRejectAriaLabel:"Avvisa",incomingCallNotificationAcceptButtonLabel:"Acceptera",incomingCallNotificationRejectButtonLabel:"Avböj",incomingCallNotificationAccceptWithVideoButtonLabel:"Acceptera med video",incomingCallNotificationDismissButtonAriaLabel:"Stäng"};var sv_SE$1 = {participantItem:participantItem$3,ParticipantList:ParticipantList$3,typingIndicator:typingIndicator$3,sendBox:sendBox$3,richTextSendBox:richTextSendBox$3,realTimeTextModal:realTimeTextModal$3,captionsBanner:captionsBanner$3,mentionPopover:mentionPopover$3,imageOverlay:imageOverlay$3,messageStatusIndicator:messageStatusIndicator$3,endCallButton:endCallButton$3,cameraButton:cameraButton$3,microphoneButton:microphoneButton$3,devicesButton:devicesButton$3,participantsButton:participantsButton$3,screenShareButton:screenShareButton$3,raiseHandButton:raiseHandButton$3,reactionButton:reactionButton$3,realTimeText:realTimeText$3,captionsSettingsModal:captionsSettingsModal$3,spokenLanguages:spokenLanguages$3,captionLanguages:captionLanguages$3,startCaptionsButton:startCaptionsButton$3,startRealTimeTextButton:startRealTimeTextButton$3,messageThread:messageThread$3,errorBar:errorBar$3,notificationStack:notificationStack$3,videoGallery:videoGallery$3,dialpad:dialpad$3,holdButton:holdButton$3,videoTile:videoTile$3,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$3,CameraSitePermissionsRequest:CameraSitePermissionsRequest$3,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$3,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$3,CameraSitePermissionsCheck:CameraSitePermissionsCheck$3,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$3,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$3,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$3,CameraSitePermissionsDenied:CameraSitePermissionsDenied$3,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$3,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$3,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$3,UnsupportedBrowser:UnsupportedBrowser$3,UnsupportedBrowserVersion:UnsupportedBrowserVersion$3,UnsupportedOperatingSystem:UnsupportedOperatingSystem$3,BrowserPermissionDenied:BrowserPermissionDenied$3,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$3,verticalGallery:verticalGallery$3,meetingConferencePhoneInfo:meetingConferencePhoneInfo$3,IncomingCallNotification:IncomingCallNotification$3};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Swedish (Sweden) .
 *
 * @public
 */
const COMPONENT_LOCALE_SV_SE = {
    strings: createComponentStrings(sv_SE$1)
};

var participantItem$2={isMeText:"(siz)",menuTitle:"Diğer Seçenekler",removeButtonLabel:"Kaldır",sharingIconLabel:"Paylaşım",mutedIconLabel:"Ses kapatıldı",handRaisedIconLabel:"El kaldırdı - Konum {position}",displayNamePlaceholder:"Adsız katılımcı",participantStateRinging:"Arıyor...",participantStateHold:"Beklemede",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}, Diğer Seçenekler",attendeeRole:"Katılımcı",micDisabledIconLabel:"Mikrofon devre dışı",cameraDisabledIconLabel:"Kamera devre dışı"};var ParticipantList$2={overflowParticipantCount:"{overflowCount} tane daha"};var typingIndicator$2={singleUser:"{user} yazıyor...",multipleUsers:"{users} yazıyor ...",multipleUsersAbbreviateOne:"{users} ve 1 kişi daha yazıyor...",multipleUsersAbbreviateMany:"{users} ve {numOthers} kişi daha yazıyor...",delimiter:", "};var sendBox$2={placeholderText:"Bir mesaj girin",textTooLong:"İletinizin uzunluğu üst sınırı aşıyor.",sendButtonAriaLabel:"İleti gönder",attachmentUploadsPendingError:"Karşıya yükleniyor... Lütfen bekleyin.",removeAttachment:"Dosyayı kaldır",uploading:"Karşıya yükleniyor",uploadCompleted:"Karşıya yükleme tamamlandı",attachmentMoreMenu:"Diğer Seçenekler"};var richTextSendBox$2={placeholderText:"Bir mesaj girin",textTooLong:"İletinizin uzunluğu üst sınırı aşıyor.",sendButtonAriaLabel:"Mesaj gönder",attachmentUploadsPendingError:"Karşıya yükleniyor... Lütfen bekleyin.",imageUploadsPendingError:"Karşıya yükleniyor... Lütfen bekleyin.",removeAttachment:"Dosyayı kaldır",uploading:"Yükleniyor",uploadCompleted:"Karşıya yükleme tamamlandı",richTextBoldTooltip:"Kalın",richTextItalicTooltip:"İtalik",richTextUnderlineTooltip:"Altı çizili",richTextBulletListTooltip:"Madde işaretli liste",richTextNumberListTooltip:"Numaralandırılmış liste",richTextIncreaseIndentTooltip:"Girintiyi artır",richTextDecreaseIndentTooltip:"Girintiyi azalt",richTextInsertTableTooltip:"Tablo ekle",richTextFormatButtonTooltip:"Biçim",richTextInsertRowOrColumnMenu:"Ekle",richTextInsertTableMenuTitle:"{column} x {row} tablosunu ekleme",richTextInsertRowAboveMenu:"Üste ekle",richTextInsertRowBelowMenu:"Aşağıya ekle",richTextInsertColumnLeftMenu:"Sola ekle",richTextInsertColumnRightMenu:"Sağa ekle",richTextDeleteRowOrColumnMenu:"Sil",richTextDeleteColumnMenu:"Sütunu sil",richTextDeleteRowMenu:"Satırı sil",richTextDeleteTableMenu:"Tabloyu sil",richTextToolbarAriaLabel:"İleti Biçimlendirme Araç Çubuğu",richTextToolbarMoreButtonAriaLabel:"Diğer seçenekler",attachmentMoreMenu:"Diğer Seçenekler",richTextNewBulletedListItemAnnouncement:"Madde işareti, yeni satır",richTextNewNumberedListItemAnnouncement:"{0}, yeni satır",richTextBulletedListAppliedAnnouncement:"Madde işaretli liste uygulandı",richTextNumberedListAppliedAnnouncement:"Numaralandırılmış liste uygulandı"};var realTimeTextModal$2={realTimeTextModalAriaLabel:"RTT Kalıcı İletişim Kutusunu Aç",realTimeTextModalTitle:"RTT açılsın mı?",realTimeTextModalText:"Gerçek zamanlı metin (RTT) toplantıdaki herkes için açılır.",realTimeTextConfirmButtonLabel:"Aç",realTimeTextCancelButtonLabel:"İptal",realTimeTextCloseModalButtonAriaLabel:"RTT Kalıcı İletişim Kutusunu Kapat"};var captionsBanner$2={captionsBannerSpinnerText:"Alt yazılar başlatılıyor...",realTimeTextInputBoxDefaultText:"İletiyi gerçek zamanlı yazın",realTimeTextInputErrorMessage:"İleti 2000 karakteri geçemez",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (gerçek zamanlı metin) toplantı süresi boyunca tüm katılımcılar için etkindir.",realTimeTextBannerLinkLabel:"Daha fazla bilgi edinin",captionsOnlyContainerTitle:"Alt yazılar",realTimeTextOnlyContainerTitle:"Gerçek Zamanlı Metin",captionsAndRealTimeTextContainerTitle:"Altyazılar ve Gerçek Zamanlı Metin",expandButtonAriaLabel:"Genişlet",minimizeButtonAriaLabel:"Daralt"};var mentionPopover$2={mentionPopoverHeader:"Öneriler"};var imageOverlay$2={downloadButtonLabel:"İndir",dismissButtonAriaLabel:"Kapat",overlayVisibleMessage:"Resim Ayrıntıları Katmanı"};var messageStatusIndicator$2={deliveredAriaLabel:"İleti gönderildi",deliveredTooltipText:"Gönderilen",seenAriaLabel:"İleti başkaları tarafından görüldü",seenTooltipText:"Görüldü",readByTooltipText:"{messageThreadReadCount} / {remoteParticipantsCount} katılımcı tarafından okundu",sendingAriaLabel:"İleti gönderiliyor",sendingTooltipText:"Gönderiliyor",failedToSendAriaLabel:"İleti gönderilemedi",failedToSendTooltipText:"Gönderilemedi"};var endCallButton$2={label:"Ayrıl",tooltipContent:"Aramadan ayrıl",leaveOption:"Ayrıl",endCallOption:"Aramayı herkes için sonlandır"};var cameraButton$2={onLabel:"Kamera",offLabel:"Kamera",tooltipDisabledContent:"Kamera devre dışı",tooltipOnContent:"Kamerayı kapat",tooltipOffContent:"Kamerayı aç",tooltipVideoLoadingContent:"Video yükleniyor",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Kamera seç",cameraButtonSplitRoleDescription:"Bölünmüş düğme",onSplitButtonAriaLabel:"Kamerayı kapat",offSplitButtonAriaLabel:"Kamerayı aç",cameraActionTurnedOnAnnouncement:"Kameranız açıldı",cameraActionTurnedOffAnnouncement:"Kameranız kapatıldı",onSplitButtonPrimaryActionCamera:"Kamerayı kapat",offSplitButtonPrimaryActionCamera:"Kamerayı aç",cameraPrimaryActionSplitButtonTitle:"Kamerayı kullan",videoEffectsMenuItemTitle:"Efektler",cameraButtonAriaDescription:"kamera seçenekleri"};var microphoneButton$2={onLabel:"Mikrofon",offLabel:"Mikrofon",tooltipDisabledContent:"Mikrofon devre dışı",tooltipOnContent:"Mikrofonun sesini kapat",tooltipOffContent:"Mikrofonun sesini aç",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Mikrofon seç",speakerMenuTitle:"Hoparlör",speakerMenuTooltip:"Hoparlör seç",microphoneButtonSplitRoleDescription:"Bölünmüş düğme",onSplitButtonAriaLabel:"Mikrofonu sessize al",offSplitButtonAriaLabel:"Mikrofonun sesini aç",microphoneActionTurnedOnAnnouncement:"Mikrofonunuz açıldı",microphoneActionTurnedOffAnnouncement:"Mikrofonunuz kapatıldı",offSplitButtonMicrophonePrimaryAction:"Mikrofonun sesini aç",onSplitButtonMicrophonePrimaryAction:"Mikrofonu sessize al",microphonePrimaryActionSplitButtonTitle:"Mikrofonu kullan",microphoneAriaDescription:"Ses seçenekleri",deepNoiseSuppressionTitle:"Gürültü engelleme",deepNoiseSuppressionOnAnnouncement:"Derin gürültü önleme açıldı",deepNoiseSuppressionOffAnnouncement:"Derin gürültü önleme kapatıldı"};var devicesButton$2={label:"Cihazlar",tooltipContent:"Cihazları yönet",cameraMenuTitle:"Kamera",cameraMenuTooltip:"Kamera seç",audioDeviceMenuTitle:"Ses Cihazı",audioDeviceMenuTooltip:"Ses cihazı seç",microphoneMenuTitle:"Mikrofon",microphoneMenuTooltip:"Mikrofon seç",speakerMenuTitle:"Hoparlör",speakerMenuTooltip:"Hoparlör seç"};var participantsButton$2={label:"Kişiler",ariaLabel:"Kişiler",tooltipContent:"Katılımcıları göster",menuHeader:"Bu aramada",participantsListButtonLabel:"{numParticipants} kişiler",muteAllButtonLabel:"Tümünün sesini kapat",copyInviteLinkButtonLabel:"Davet bağlantısını kopyala",copyInviteLinkActionedAriaLabel:"Davet bağlantısı kopyalandı"};var screenShareButton$2={onLabel:"Sunumu Durdur",offLabel:"Sun",tooltipDisabledContent:"Sunu devre dışı bırakıldı.",tooltipOnContent:"Ekranınızı sunuluyor",tooltipOffContent:"Ekranınızı sunun"};var raiseHandButton$2={offLabel:"Kaldır",onLabel:"İndir",tooltipDisabledContent:"El Kaldırma eylemi devre dışı bırakıldı",tooltipOnContent:"Eli İndirme",tooltipOffContent:"El Kaldırma"};var reactionButton$2={label:"Tepki ver",ariaLabel:"Tepki Ver",tooltipDisabledContent:"Tepki eylemi devre dışı bırakıldı",tooltipContent:"Tepki gönder",likeReactionTooltipContent:"Beğen",heartReactionTooltipContent:"Aşk",laughReactionTooltipContent:"Kahkaha",applauseReactionTooltipContent:"Alkış",surprisedReactionTooltipContent:"Şaşırmış"};var realTimeText$2={isTypingText:"RTT yazma"};var captionsSettingsModal$2={captionsSettingsModalTitle:"Açıklamalı Alt Yazı Ayarları",captionsSettingsSpokenLanguageDropdownLabel:"Konuşma dili",captionsSettingsCaptionLanguageDropdownLabel:"Açıklamalı alt yazı dili",captionsSettingsSpokenLanguageDropdownInfoText:"Bu çağrıdaki herkesin konuşma dili.",captionsSettingsCaptionLanguageDropdownInfoText:"Açıklamalı alt yazılar bu dilde görünür.",captionsSettingsConfirmButtonLabel:"Onayla",captionsSettingsCancelButtonLabel:"İptal",captionsSettingsModalAriaLabel:"Alt Yazı Ayarı Kalıcı İletişim Kutusu",captionsSettingsCloseModalButtonAriaLabel:"Alt Yazı Ayarını Kapat"};var spokenLanguages$2={"ar-ae":"Arapça - BAE","ar-sa":"Arapça - Suudi Arabistan","da-dk":"Danca","de-de":"Almanca - Almanya","en-au":"İngilizce - Avustralya","en-ca":"İngilizce - Kanada","en-gb":"İngilizce - Birleşik Krallık","en-in":"İngilizce - Hindistan","en-nz":"İngilizce - Yeni Zelanda","en-us":"İngilizce - Birleşik Devletler","es-es":"İspanyolca - İspanya (Modern)","es-mx":"İspanyolca - Meksika","fi-fi":"Fince","fr-ca":"Fransızca - Kanada","fr-fr":"Fransızca - Fransa","hi-in":"Hintçe","it-it":"İtalyanca - İtalya","ja-jp":"Japonca","ko-kr":"Korece","nb-no":"Norveççe (Bokmål)","nl-be":"Hollanda dili - Belçika","nl-nl":"Felemenkçe - Hollanda","pl-pl":"Lehçe","pt-br":"Portekizce - Brezilya","ru-ru":"Rusça","sv-se":"İsveççe","zh-cn":"Çince - Çin Halk Cumhuriyeti","zh-hk":"Çince - Hong Kong ÖİB","cs-cz":"Çekçe","pt-pt":"Portekizce - Portekiz","tr-tr":"Türkçe","vi-vn":"Vietnamca","th-th":"Tayca","he-il":"İbranice","cy-gb":"Galce","uk-ua":"Ukraynaca","el-gr":"Yunanca","hu-hu":"Macarca","ro-ro":"Rumence","sk-sk":"Slovakça","zh-tw":"Çince - Tayvan"};var captionLanguages$2={ar:"Arapça",da:"Danca",de:"Almanca",en:"İngilizce",es:"İspanyolca",fi:"Fince","fr-ca":"Fransızca - Kanada",fr:"Fransızca - Fransa",hi:"Hintçe",it:"İtalyanca",ja:"Japonca",ko:"Korece",nb:"Norveççe (Bokmål)",nl:"Felemenkçe",pl:"Lehçe",pt:"Portekizce - Brezilya",ru:"Rusça",sv:"İsveççe","zh-Hans":"Çince (Basitleştirilmiş)","zh-Hant":"Çince (Geleneksel)",cs:"Çekçe","pt-pt":"Portekizce - Portekiz",tr:"Türkçe",vi:"Vietnamca",th:"Tayca",he:"İbranice",cy:"Galce",uk:"Ukraynaca",el:"Yunanca",hu:"Macarca",ro:"Rumence",sk:"Slovakça"};var startCaptionsButton$2={onLabel:"Alt yazıları aç",offLabel:"Alt yazıları kapat",tooltipOnContent:"Alt yazıları kapat",tooltipOffContent:"Alt yazıları aç"};var startRealTimeTextButton$2={label:"Bu arama için RTT'yi aç",tooltipOffContent:"Bu arama için RTT'yi aç"};var messageThread$2={yesterday:"Dün",sunday:"Pazar",monday:"Pazartesi",tuesday:"Salı",wednesday:"Çarşamba",thursday:"Perşembe",friday:"Cuma",saturday:"Cumartesi",participantJoined:"sohbete katıldı.",participantLeft:"sohbetten ayrıldı.",editMessage:"Düzenle",removeMessage:"Sil",resendMessage:"Yeniden göndermeyi deneyin",failToSendTag:"Gönderilemedi",editedTag:"Düzenlendi",liveAuthorIntro:"{author} tarafından gönderilen ileti",editedMessageLiveAuthorIntro:"{author} tarafından düzenlenen ileti",editedMessageLocalUserLiveAuthorIntro:"Düzenlenmiş ileti",messageContentAriaText:"{status} {author} şunu dedi: {message}",messageContentMineAriaText:"{status} {message} dediniz",editBoxTextLimit:"İletiniz en fazla {limitNumber} aşıyor",editBoxPlaceholderText:"İletinizi düzenleyin",newMessagesIndicator:"Yeni iletiler",noDisplayNameSub:"Ad yok",editBoxCancelButton:"İptal",editBoxSubmitButton:"Bitti",messageReadCount:"{messageReadByCount} / {remoteParticipantsCount} katılımcı tarafından okundu",actionMenuMoreOptions:"Diğer Seçenekler",downloadAttachment:"Dosyayı indir",openAttachment:"Dosyayı aç",blockedWarningText:"Bu ileti kuruluş ilkesi nedeniyle silindi.",blockedWarningLinkText:"Ayrıntılar",attachmentCardGroupMessage:"İletinin {attachmentCount} eki var",messageDeletedAnnouncementAriaLabel:"İleti silindi",richTextBoldTooltip:"Kalın",richTextItalicTooltip:"İtalik",richTextUnderlineTooltip:"Altını Çiz",richTextBulletListTooltip:"Madde işaretli liste",richTextNumberListTooltip:"Numaralandırılmış liste",richTextIncreaseIndentTooltip:"Girintiyi artır",richTextDecreaseIndentTooltip:"Girintiyi azalt",richTextInsertTableTooltip:"Tablo ekle",richTextFormatButtonTooltip:"Biçim",richTextInsertRowOrColumnMenu:"Ekle",richTextInsertTableMenuTitle:"{column} x {row} tablosunu ekleme",richTextInsertRowAboveMenu:"Üste ekle",richTextInsertRowBelowMenu:"Aşağıya ekle",richTextInsertColumnLeftMenu:"Sola ekle",richTextInsertColumnRightMenu:"Sağa ekle",richTextDeleteRowOrColumnMenu:"Sil",richTextDeleteColumnMenu:"Sütunu sil",richTextDeleteRowMenu:"Satırı sil",richTextDeleteTableMenu:"Tabloyu sil",richTextToolbarMoreButtonAriaLabel:"Diğer seçenekler",attachmentUploadsPendingError:"Karşıya yükleniyor... Lütfen bekleyin.",imageUploadsPendingError:"Karşıya yükleniyor... Lütfen bekleyin.",richTextNewBulletedListItemAnnouncement:"Madde işareti, yeni satır",richTextNewNumberedListItemAnnouncement:"{0}, yeni satır",richTextBulletedListAppliedAnnouncement:"Madde işaretli liste uygulandı",richTextNumberedListAppliedAnnouncement:"Numaralandırılmış liste uygulandı"};var errorBar$2={unableToReachChatService:"Çevrimdışısınız",accessDenied:"Sohbet hizmetlerine erişilemedi. Lütfen sağlanan kullanıcı kimlik bilgilerini denetleyin",userNotInChatThread:"Artık bu sohbet yazışması içinde değilsiniz",sendMessageNotInChatThread:"Artık bu sohbet yazışması içinde olmadığınız için ileti gönderilemedi",sendMessageGeneric:"İleti gönderilemedi",callingNetworkFailure:"Arama bağlanırken sorun oluştu; çevrimdışı görünüyorsunuz",startVideoGeneric:"Video başlatılamadı",stopVideoGeneric:"Video durdurulamadı",muteGeneric:"Mikrofonun sesi kapatılamadı",unmuteGeneric:"Mikrofonun sesi açılamadı",speakingWhileMuted:"Mikrofonunuzun sesi kapalı",startScreenShareGeneric:"Ekran paylaşımı başlatılırken bir sorun oluştu.",stopScreenShareGeneric:"Ekran paylaşımı durdurulamadı",callNetworkQualityLow:"Ağ kalitesi düşük.",teamsMeetingCallNetworkQualityLow:"Ağ kalitesi düşük.",callNoSpeakerFound:"Hoparlör veya kulaklık bulunamadı. Aramayı dinlemek için bir ses aygıtı bağlayın.",callNoMicrophoneFound:"Mikrofon bulunamadı. Ses giriş cihazı bağlan.",callMicrophoneAccessDenied:"Mikrofona erişemiyor. Bu web sayfası için izin vermek için adres çubuğundaki kilidi tıklatın.",callMicrophoneAccessDeniedSafari:"Mikrofona erişemiyor. İzin vermek için sayfayı yenileyin veya bu tarayıcının ayarlarını denetleyin ve bu web sitesi için izinlerin etkinleştirildiğinden emin olun.",callMicrophoneMutedBySystem:"Sisteminiz tarafından sessize alındınız.",callMicrophoneUnmutedBySystem:"Mikrofonunuz kurtarıldı ve sisteminiz tarafından sesiniz açıldı.",callMacOsMicrophoneAccessDenied:"Mikrofona erişilemiyor. macOS gizlilik ayarlarınızda mikrofon izni verin.",callLocalVideoFreeze:"Ağ bant genişliği düşük. Görüntülü aramada başkaları için duraklatılmış görünüyor olabilir.",callCameraAccessDenied:"Kameraya erişemiyor. Bu web sayfası için izin vermek için adres çubuğundaki kilidi tıklatın.",callCameraAccessDeniedSafari:"Kameraya erişemiyor. İzin vermek için sayfayı yenileyin veya bu tarayıcının ayarlarını denetleyin ve bu web sitesi için izinlerin etkinleştirildiğinden emin olun.",callCameraAlreadyInUse:"Kameraya erişemiyor. Başka bir uygulama tarafından zaten kullanılıyor olabilir.",callVideoStoppedBySystem:"Videonuz sisteminiz tarafından durduruldu.",callVideoRecoveredBySystem:"Videonuz devam ediyor.",callMacOsCameraAccessDenied:"MacOS kameranıza erişimi engelliyor. Bu tarayıcının kameranıza erişmesine izin vermek için gizlilik ayarlarınızı güncelleştirin.",callMacOsScreenShareAccessDenied:"MacOS ekran paylaşımını engelliyor. Bu tarayıcının ekranınızı kaydetmesini izin vermek için gizlilik ayarlarınızı güncelleştirin.",dismissButtonAriaLabel:"Kapat",failedToJoinCallGeneric:"Aramaya katılınamadı.",failedToJoinCallInvalidMeetingLink:"Toplantıya katılınamıyor. Geçersiz Bağlantı.",cameraFrozenForRemoteParticipants:"Aramadaki kullanıcılar videonuzu görmekte sorun yaşıyor. Lütfen cihazlarınızı ve ağlarınızı kontrol edin.",unableToStartVideoEffect:"Video efekti uygulanamıyor.",startSpotlightWhileMaxParticipantsAreSpotlighted:"Öne çıkarılan katılımcı sayısı zaten üst sınırı geçtiğinden katılımcılarda öne çıkarma özelliği başlatılamıyor.",mutedByRemoteParticipant:"Başka bir katılımcı tarafından sessize alındınız."};var notificationStack$2={callingNetworkFailure:{title:"Arama bağlanırken sorun oluştu",message:"Çevrimdışı görünüyorsunuz. Lütfen ağ bağlantınızı denetleyin ve yeniden deneyin.",dismissButtonAriaLabel:"Kapat"},startVideoGeneric:{title:"Video başlatılamadı.",dismissButtonAriaLabel:"Kapat"},stopVideoGeneric:{title:"Video durdurulamadı.",dismissButtonAriaLabel:"Kapat"},muteGeneric:{title:"Mikrofonun sesi kapatılamadı.",dismissButtonAriaLabel:"Kapat"},unmuteGeneric:{title:"Mikrofonun sesi açılamadı.",dismissButtonAriaLabel:"Kapat"},speakingWhileMuted:{title:"Sesiniz kapatıldı.",dismissButtonAriaLabel:"Kapat"},startScreenShareGeneric:{title:"Ekran paylaşımı başlatılırken bir sorun oluştu.",dismissButtonAriaLabel:"Kapat"},stopScreenShareGeneric:{title:"Ekran paylaşımı durdurulamadı.",dismissButtonAriaLabel:"Kapat"},callNetworkQualityLow:{title:"Ağ kalitesi düşük.",dismissButtonAriaLabel:"Kapat"},teamsMeetingCallNetworkQualityLow:{title:"Ağ kalitesi düşük.",dismissButtonAriaLabel:"Kapat",message:"Daha iyi ses için telefonunuzdan bu aramaya katılın. Bu cihazda toplantıyı görüntülemeye devam edin.",primaryButtonLabel:"Telefonla Katıl"},callNoSpeakerFound:{title:"Hoparlör veya kulaklık yok",message:"Aramayı dinlemek için bir ses aygıtı bağlayın.",dismissButtonAriaLabel:"Kapat"},callNoMicrophoneFound:{title:"Mikrofon bulunamadı",message:"Ses giriş cihazı bağlayın.",dismissButtonAriaLabel:"Kapat"},callMicrophoneAccessDenied:{title:"Mikrofona erişilemiyor",message:"Bu web sayfası için izin vermek için adres çubuğundaki kilidi tıklatın.",dismissButtonAriaLabel:"Kapat"},callMicrophoneAccessDeniedSafari:{title:"Mikrofona erişilemiyor",message:"İzin vermek için sayfayı yenileyin veya bu tarayıcının ayarlarını denetleyin ve bu web sitesi için izinlerin etkinleştirildiğinden emin olun.",dismissButtonAriaLabel:"Kapat"},callMicrophoneMutedBySystem:{title:"Sisteminiz tarafından sessize alındınız.",dismissButtonAriaLabel:"Kapat"},callMicrophoneUnmutedBySystem:{title:"Mikrofonunuz kurtarıldı ve sisteminiz tarafından sesiniz açıldı.",dismissButtonAriaLabel:"Kapat"},callMacOsMicrophoneAccessDenied:{title:"Mikrofona erişilemiyor",message:"macOS gizlilik ayarlarınıza mikrofon izni verin.",dismissButtonAriaLabel:"Kapat"},callLocalVideoFreeze:{title:"Ağ bant genişliği düşük",message:"Görüntülü aramada başkaları için duraklatılmış görünüyor olabilir.",dismissButtonAriaLabel:"Kapat"},callCameraAccessDenied:{title:"Kameraya erişilemiyor",message:"Bu web sayfası için izin vermek için adres çubuğundaki kilidi tıklatın.",dismissButtonAriaLabel:"Kapat"},callCameraAccessDeniedSafari:{title:"Kameraya erişilemiyor",message:"İzin vermek için sayfayı yenileyin veya bu tarayıcının ayarlarını denetleyin ve bu web sitesi için izinlerin etkinleştirildiğinden emin olun.",dismissButtonAriaLabel:"Kapat"},callCameraAlreadyInUse:{title:"Kameraya erişilemiyor",message:"Başka bir uygulama tarafından zaten kullanılıyor olabilir.",dismissButtonAriaLabel:"Kapat"},callVideoStoppedBySystem:{title:"Videonuz sisteminiz tarafından durduruldu.",dismissButtonAriaLabel:"Kapat"},callVideoRecoveredBySystem:{title:"Videonuz devam ediyor.",dismissButtonAriaLabel:"Kapat"},callMacOsCameraAccessDenied:{title:"MacOS kameranıza erişimi engelliyor",message:"Bu tarayıcının kameranıza erişmesine izin vermek için gizlilik ayarlarınızı güncelleştirin.",dismissButtonAriaLabel:"Kapat"},callMacOsScreenShareAccessDenied:{title:"MacOS ekran paylaşımını engelliyor",message:"Bu tarayıcının ekranınızı kaydetmesini izin vermek için gizlilik ayarlarınızı güncelleştirin.",dismissButtonAriaLabel:"Kapat"},failedToJoinCallGeneric:{title:"Aramaya katılınamadı.",dismissButtonAriaLabel:"Kapat"},failedToJoinCallInvalidMeetingLink:{title:"Toplantıya katılınamıyor. Geçersiz Bağlantı.",dismissButtonAriaLabel:"Kapat"},cameraFrozenForRemoteParticipants:{title:"Aramadaki kullanıcılar videonuzu görmekte sorun yaşıyor",message:"Lütfen cihazlarınızı ve ağlarınızı kontrol edin.",dismissButtonAriaLabel:"Kapat"},unableToStartVideoEffect:{title:"Video efekti uygulanamıyor.",dismissButtonAriaLabel:"Kapat"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"Katılımcılarda öne çıkarma özelliği başlatılamıyor",message:"Maksimum katılımcı sayısı zaten öne çıkarıldı.",dismissButtonAriaLabel:"Kapat"},mutedByRemoteParticipant:{title:"Başka bir katılımcı tarafından sessize alındınız.",dismissButtonAriaLabel:"Kapat"},recordingStarted:{title:"Kayıt başladı.",message:"Katılarak bu toplantının dökümünün çıkarılmasına onay verirsiniz.",dismissButtonAriaLabel:"Kapat"},transcriptionStarted:{title:"Döküm başlatıldı.",message:"Katılarak bu toplantının dökümünün çıkarılmasına onay verirsiniz.",dismissButtonAriaLabel:"Kapat"},recordingStopped:{title:"Kayıt kaydediliyor.",message:"Kayıt durduruldu.",dismissButtonAriaLabel:"Kapat"},transcriptionStopped:{title:"Döküm kaydediliyor.",message:"Döküm durduruldu.",dismissButtonAriaLabel:"Kapat"},recordingAndTranscriptionStarted:{title:"Kayıt ve döküm başlatıldı.",message:"Katılarak bu toplantının dökümünün çıkarılmasına onay verirsiniz.",dismissButtonAriaLabel:"Kapat"},recordingAndTranscriptionStopped:{title:"Kayıt ve döküm kaydediliyor.",message:"Kayıt ve döküm durduruldu.",dismissButtonAriaLabel:"Kapat"},recordingStoppedStillTranscribing:{title:"Kayıt durduruldu.",message:"Şu anda yalnızca bu toplantının dökümünü oluşturuyorsunuz.",dismissButtonAriaLabel:"Kapat"},transcriptionStoppedStillRecording:{title:"Döküm durduruldu.",message:"Şu anda yalnızca bu toplantıyı kaydediyorsunuz.",dismissButtonAriaLabel:"Kapat"},assignedBreakoutRoomOpened:{title:"Sizi 5 saniye içinde atanan odanıza taşıyacağız.",dismissButtonAriaLabel:"Kapat"},assignedBreakoutRoomChanged:{title:"Tartışma odanız değişti",message:"Sizi 5 saniye içinde atanan odanıza taşıyacağız.",dismissButtonAriaLabel:"Kapat"},assignedBreakoutRoomClosed:{title:"Tartışma odanız kapatıldı",message:"5 saniye içinde toplantınıza geri taşınacak.",dismissButtonAriaLabel:"Kapat"},assignedBreakoutRoomOpenedPromptJoin:{title:"Tartışma odasına katılmak ister misiniz?",message:"Bir tartışma odasına atandınız.",dismissButtonAriaLabel:"Kapat",primaryButtonLabel:"Odaya katıl",secondaryButtonLabel:"Daha sonra"},breakoutRoomJoined:{title:"Tartışma odasına katıldınız.",dismissButtonAriaLabel:"Kapat"},breakoutRoomClosingSoon:{title:"Oda süre sınırı sona ermek üzere.",message:"Bu oda 30 sn. içinde kapanacak",dismissButtonAriaLabel:"Kapat"},capabilityTurnVideoOnPresent:{title:"Kameranız etkinleştirildi",message:"Video paylaşmak istiyorsanız bu öğeyi açın.",dismissButtonAriaLabel:"Kapat"},capabilityTurnVideoOnAbsent:{title:"Kameranız devre dışı bırakıldı",message:"Artık video paylaşamazsınız.",dismissButtonAriaLabel:"Kapat"},capabilityUnmuteMicPresent:{title:"Mikrofonunuz etkinleştirildi",message:"Konuşmak isterseniz sesi açın.",dismissButtonAriaLabel:"Kapat"},capabilityUnmuteMicAbsent:{title:"Mikrofonunuz devre dışı bırakıldı",message:"Artık sesi açamazsınız.",dismissButtonAriaLabel:"Kapat"},togetherModeStarted:{title:"Togethermode başlatıldı",dismissButtonAriaLabel:"Kapat"},togetherModeEnded:{title:"Togethermode sona erdi",dismissButtonAriaLabel:"Kapat"},transcriptionError:{title:"Transkripsiyonla ilgili bir sorun oluştu.",message:"Transkripsiyon durduruldu. Lütfen daha sonra yeniden deneyin",dismissButtonAriaLabel:"Kapat"},transcriptionStartedByYou:{title:"Transkripsiyon başlatıldı.",message:"Siz başlattınız. Transkripsiyon yapıldığını herkese bildirin.",dismissButtonAriaLabel:"Kapat"}};var videoGallery$2={screenIsBeingSharedMessage:"Ekranınızı paylaşıyorsunuz.",screenShareLoadingMessage:"{participant} adlı katılımcının ekranını yükleme",localScreenShareLoadingMessage:"Ekranınız yükleniyor",localVideoLabel:"Siz",localVideoCameraSwitcherLabel:"Kamerayı değiştir",localVideoMovementLabel:"Taşınabilir Yerel Video Kutucuğu",localVideoMovementAriaLabel:"Taşınabilir Yerel Video Kutucuğu",localVideoSelectedDescription:"{cameraName} seçildi",displayNamePlaceholder:"Adsız katılımcı",fitRemoteParticipantToFrame:"Çerçeveye sığdır",fillRemoteParticipantFrame:"Çerçeveyi doldur",pinParticipantForMe:"Benim için sabitle",pinParticipantForMeLimitReached:"Sabitleme (sınıra ulaşıldı)",unpinParticipantForMe:"Sabitlemeyi kaldır",pinParticipantMenuItemAriaLabel:"{participantName} adlı katılımcıyı sabitle",unpinParticipantMenuItemAriaLabel:"{participantName} adlı katılımcının sabitlemesini kaldır",pinnedParticipantAnnouncementAriaLabel:"{participantName} adlı katılımcı sabitlendi",unpinnedParticipantAnnouncementAriaLabel:"{participantName} adlı katılımcının sabitlemesi kaldırıldı",startSpotlightVideoTileMenuLabel:"Herkes için öne çıkar",addSpotlightVideoTileMenuLabel:"Öne çıkarma ekle",spotlightLimitReachedMenuTitle:"Öne çıkarma sınırına ulaşıldı",stopSpotlightVideoTileMenuLabel:"Öne çıkarmayı durdur",stopSpotlightOnSelfVideoTileMenuLabel:"Öne çıkarmadan çık",attendeeRole:"Katılımcı",muteParticipantMenuItemLabel:"Sesi kapat",waitingScreenText:"Diğerlerinin katılması bekleniyor",forbidAudio:"Mikrofonu devre dışı bırak",permitAudio:"Mikrofona izin ver",forbidAudioTileMenuLabel:"Mikrofonu devre dışı bırak",permitAudioTileMenuLabel:"Mikrofona izin ver",forbidVideo:"Kamerayı devre dışı bırak",permitVideo:"Kameraya izin ver",forbidVideoTileMenuLabel:"Kamerayı devre dışı bırak",permitVideoTileMenuLabel:"Kameraya izin ver"};var dialpad$2={placeholderText:"Telefon numarası gir",deleteButtonAriaLabel:"Sil"};var holdButton$2={onLabel:"Devam et",offLabel:"Beklet",tooltipOnContent:"Görüşmeye devam etme",tooltipOffContent:"Aramayı bekletme"};var videoTile$2={participantStateRinging:"Aranıyor...",participantReconnecting:"Yeniden bağlanıyor...",participantStateHold:"Beklemede",moreOptionsButtonAriaLabel:"Diğer Seçenekler {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"sesi kapalı",moreOptionsParticipantMutedStateUnmutedAriaLabel:"sesi açık",moreOptionsParticipantHandRaisedAriaLabel:"El kaldırdı",moreOptionsParticipantIsSpeakingAriaLabel:"Konuşuyor",moreOptionsParticipantMicDisabledAriaLabel:"Mikrofon devre dışı",moreOptionsParticipantCameraDisabledAriaLabel:"Kamera devre dışı"};var CameraAndMicrophoneSitePermissionsRequest$2={primaryText:"{appName} uygulamasının kameranızı ve mikrofonunuzu kullanmasına izin verin",secondaryText:"Bu, katılımcıların sizi görebilmesi ve duyabilmesi içindir.",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın",primaryButtonText:"Kamera ve mikrofon olmadan devam et",ariaLabel:"Kamera ve mikrofon erişimine izin ver"};var CameraSitePermissionsRequest$2={primaryText:"{appName} uygulamasının kameranızı kullanmasına izin verin",secondaryText:"Bu, katılımcıların sizi görebilmesi içindir.",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın",primaryButtonText:"Kamera olmadan devam et",ariaLabel:"Kamera erişimine izin ver"};var MicrophoneSitePermissionsRequest$2={primaryText:"{appName} uygulamasının mikrofonunuzu kullanmasına izin verin",secondaryText:"Bu, katılımcıların sizi duyabilmesi içindir.",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın",primaryButtonText:"Mikrofon olmadan devam et",ariaLabel:"Mikrofon erişimine izin ver"};var CameraAndMicrophoneSitePermissionsCheck$2={primaryText:"Kamera ve mikrofon erişimi denetleniyor",secondaryText:"İstenirse erişime izin verin. Bu, katılımcıların sizi görebilmesi ve duyabilmesi içindir.",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın",primaryButtonText:"Kamera ve mikrofon olmadan devam et",ariaLabel:"Kamera ve mikrofon erişimi denetleniyor. İstenirse erişime izin verin."};var CameraSitePermissionsCheck$2={primaryText:"Kamera erişimi denetleniyor",secondaryText:"İstenirse erişime izin verin. Bu, katılımcıların sizi görebilmesi içindir.",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın",primaryButtonText:"Kamera olmadan devam et",ariaLabel:"Kamera erişimi denetleniyor. İstenirse erişime izin verin."};var MicrophoneSitePermissionsCheck$2={primaryText:"Mikrofon erişimi denetleniyor",secondaryText:"İstenirse erişime izin verin. Bu, katılımcıların sizi duyabilmesi içindir.",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın",primaryButtonText:"Mikrofon olmadan devam et",ariaLabel:"Mikrofon erişimi denetleniyor. İstenirse erişime izin verin."};var CameraAndMicrophoneSitePermissionsDenied$2={primaryText:"Kamera ve mikrofona erişemiyor",secondaryText:"Bu web sayfası için mikrofon izinleri vermek için adres çubuğundaki kilit simgesine tıklayın. Sayfa yenilemesi gerekebilir.",primaryButtonText:"Kamera ve mikrofon olmadan devam et",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın"};var CameraAndMicrophoneSitePermissionsDeniedSafari$2={primaryText:"Kamera ve mikrofona erişemiyor",secondaryText:"İzin vermek için sayfayı yenileyin veya bu tarayıcının ayarlarını denetleyin ve bu web sitesi için izinlerin etkinleştirildiğinden emin olun.",primaryButtonText:"Kamera ve mikrofon olmadan devam et",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın"};var CameraSitePermissionsDenied$2={primaryText:"Kameraya erişilemiyor",secondaryText:"Bu web sayfasında kamera izinleri vermek için adres çubuğundaki kilit simgesine tıklayın. Sayfa yenilemesi gerekebilir.",primaryButtonText:"Kamera olmadan devam et",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın"};var MicrophoneSitePermissionsDenied$2={primaryText:"Mikrofona erişilemiyor",secondaryText:"Bu web sayfası için mikrofon izinleri vermek için adres çubuğundaki kilit simgesine tıklayın. Sayfa yenilemesi gerekebilir.",primaryButtonText:"Mikrofon olmadan devam et",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın"};var CameraSitePermissionsDeniedSafari$2={primaryText:"Kameraya erişilemiyor",secondaryText:"İzin vermek için sayfayı yenileyin veya bu tarayıcının ayarlarını denetleyin ve bu web sitesi için izinlerin etkinleştirildiğinden emin olun.",primaryButtonText:"Kamera olmadan devam et",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın"};var MicrophoneSitePermissionsDeniedSafari$2={primaryText:"Mikrofona erişilemiyor",secondaryText:"İzin vermek için sayfayı yenileyin veya bu tarayıcının ayarlarını denetleyin ve bu web sitesi için izinlerin etkinleştirildiğinden emin olun.",primaryButtonText:"Mikrofon olmadan devam et",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın"};var UnsupportedBrowser$2={primaryText:"Tarayıcı desteklenmiyor",secondaryText:"Lütfen uyumlu bir tarayıcı kullanarak bu aramaya katılın.",moreHelpLinkText:"Uyumluluk gereksinimlerini görüntüleyin"};var UnsupportedBrowserVersion$2={primaryText:"Tarayıcı güncelleştirmesi gerekiyor",secondaryText:"Mümkün olan en iyi arama deneyimi için lütfen tarayıcınızı güncelleştirin ve aramaya yeniden katılmayı deneyin.",moreHelpLinkText:"Uyumluluk gereksinimlerini görüntüleyin",continueAnywayButtonText:"Güncelleştirmeden arama başlat"};var UnsupportedOperatingSystem$2={primaryText:"İşletim sistemi desteklenmiyor",secondaryText:"Lütfen uyumlu bir işletim sistemine sahip bir cihaz kullanarak bu çağrıya katılın.",moreHelpLinkText:"Uyumluluk gereksinimlerini görüntüleyin"};var BrowserPermissionDenied$2={primaryText:"Kameranızı veya mikrofonunuzu kullanamazsınız",secondaryText:"Tarayıcınızın kameranıza veya mikrofonunuza erişimi olmayabilir. Bunu düzeltmek için Sistem Tercihleri'ni açın.",primaryButtonText:"Yeniden dene",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın"};var BrowserPermissionDeniedIOS$2={primaryText:"Devam etmek için mikrofon erişimine izin ver",secondaryText:"Böylece diğer katılımcılar sizi duyabilir.",primaryButtonText:"Yeniden dene",imageAltText:"iOS için mikrofon ve kamera cihazı izin konumu",linkText:"Yardıma mı ihtiyacınız var? Sorun giderme yardımı alın",step1Text:"Ayarlar uygulamasına gidin",step2Text:"Bu tarayıcının ayarlarına ilerleyin",step3Text:"Mikrofonu Aç (İsteğe bağlı kamera)",step4Text:"Aramaya tekrar katılmayı deneyin",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$2={leftNavButtonAriaLabel:"önceki sayfa {current}/{total}",rightNavButtonAriaLabel:"sonraki sayfa {current}/{total}"};var meetingConferencePhoneInfo$2={meetingConferencePhoneInfoModalTitle:"Telefonunuzdan aramaya katılın",meetingConferencePhoneInfoModalDialIn:"Numarayı çevirin",meetingConferencePhoneInfoModalMeetingId:"Toplantı kimliğini girin",meetingConferencePhoneInfoModalWait:"Düzenleyici tarafından kabul etmek için bekleyin",meetingConferencePhoneInfoModalNoPhoneAvailable:"Kullanılabilir telefon yok",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(Ücretsiz)",meetingConferencePhoneInfoModalToll:"(Ücretli)"};var IncomingCallNotification$2={incomingCallNotificationPlaceholderId:"Bilinmeyen Kullanıcı",incomingCallNotificationPlaceholderAlert:"{callerName} arıyor",incomingCallNoticicationAcceptWithAudioAriaLabel:"Sesli kabul et",incomingCallNoticicationAcceptWithVideoAriaLabel:"Görüntülü kabul et",incomingCallNoticicationRejectAriaLabel:"Reddet",incomingCallNotificationAcceptButtonLabel:"Kabul et",incomingCallNotificationRejectButtonLabel:"Reddet",incomingCallNotificationAccceptWithVideoButtonLabel:"Görüntülü Kabul Et",incomingCallNotificationDismissButtonAriaLabel:"Kapat"};var tr_TR$1 = {participantItem:participantItem$2,ParticipantList:ParticipantList$2,typingIndicator:typingIndicator$2,sendBox:sendBox$2,richTextSendBox:richTextSendBox$2,realTimeTextModal:realTimeTextModal$2,captionsBanner:captionsBanner$2,mentionPopover:mentionPopover$2,imageOverlay:imageOverlay$2,messageStatusIndicator:messageStatusIndicator$2,endCallButton:endCallButton$2,cameraButton:cameraButton$2,microphoneButton:microphoneButton$2,devicesButton:devicesButton$2,participantsButton:participantsButton$2,screenShareButton:screenShareButton$2,raiseHandButton:raiseHandButton$2,reactionButton:reactionButton$2,realTimeText:realTimeText$2,captionsSettingsModal:captionsSettingsModal$2,spokenLanguages:spokenLanguages$2,captionLanguages:captionLanguages$2,startCaptionsButton:startCaptionsButton$2,startRealTimeTextButton:startRealTimeTextButton$2,messageThread:messageThread$2,errorBar:errorBar$2,notificationStack:notificationStack$2,videoGallery:videoGallery$2,dialpad:dialpad$2,holdButton:holdButton$2,videoTile:videoTile$2,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$2,CameraSitePermissionsRequest:CameraSitePermissionsRequest$2,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$2,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$2,CameraSitePermissionsCheck:CameraSitePermissionsCheck$2,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$2,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$2,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$2,CameraSitePermissionsDenied:CameraSitePermissionsDenied$2,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$2,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$2,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$2,UnsupportedBrowser:UnsupportedBrowser$2,UnsupportedBrowserVersion:UnsupportedBrowserVersion$2,UnsupportedOperatingSystem:UnsupportedOperatingSystem$2,BrowserPermissionDenied:BrowserPermissionDenied$2,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$2,verticalGallery:verticalGallery$2,meetingConferencePhoneInfo:meetingConferencePhoneInfo$2,IncomingCallNotification:IncomingCallNotification$2};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Turkish (Turkey).
 *
 * @public
 */
const COMPONENT_LOCALE_TR_TR = {
    strings: createComponentStrings(tr_TR$1)
};

var participantItem$1={isMeText:"(我)",menuTitle:"更多选项",removeButtonLabel:"删除",sharingIconLabel:"共享",mutedIconLabel:"已静音",handRaisedIconLabel:"已举手 - 位置 {position}",displayNamePlaceholder:"未命名的参与者",participantStateRinging:"正在呼叫...",participantStateHold:"保留",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}、{handRaisedState}、{mutedState}、{micDisabledState}、{cameraDisabledState}、{sharingState}、{connectionState}、更多选项",attendeeRole:"与会者",micDisabledIconLabel:"麦克风已禁用",cameraDisabledIconLabel:"相机已禁用"};var ParticipantList$1={overflowParticipantCount:"和另外 {overflowCount} 人"};var typingIndicator$1={singleUser:"{user} 正在键入...",multipleUsers:"{users} 正在键入...",multipleUsersAbbreviateOne:"{users} 和另外 1 人正在键入...",multipleUsersAbbreviateMany:"{users} 和其他 {numOthers} 人正在键入...",delimiter:"、 "};var sendBox$1={placeholderText:"输入消息",textTooLong:"你的消息长度超过了最大限制。",sendButtonAriaLabel:"发送消息",attachmentUploadsPendingError:"正在上传文件，请稍候。",removeAttachment:"删除文件",uploading:"正在上传",uploadCompleted:"已完成上传",attachmentMoreMenu:"更多选项"};var richTextSendBox$1={placeholderText:"输入消息",textTooLong:"你的消息长度超过了最大限制。",sendButtonAriaLabel:"发送信息",attachmentUploadsPendingError:"正在上传文件，请稍候。",imageUploadsPendingError:"正在上传图像，请稍候。",removeAttachment:"删除文件",uploading:"正在上传",uploadCompleted:"已完成上传",richTextBoldTooltip:"粗体",richTextItalicTooltip:"斜体",richTextUnderlineTooltip:"下划线",richTextBulletListTooltip:"项目符号列表",richTextNumberListTooltip:"编号列表",richTextIncreaseIndentTooltip:"增加缩进量",richTextDecreaseIndentTooltip:"减少缩进量",richTextInsertTableTooltip:"插入表格",richTextFormatButtonTooltip:"格式",richTextInsertRowOrColumnMenu:"插入",richTextInsertTableMenuTitle:"插入 {column} x {row} 表",richTextInsertRowAboveMenu:"在上方插入",richTextInsertRowBelowMenu:"在下方插入",richTextInsertColumnLeftMenu:"在左侧插入",richTextInsertColumnRightMenu:"在右侧插入",richTextDeleteRowOrColumnMenu:"删除",richTextDeleteColumnMenu:"删除列",richTextDeleteRowMenu:"删除行",richTextDeleteTableMenu:"删除表格",richTextToolbarAriaLabel:"消息格式设置工具栏",richTextToolbarMoreButtonAriaLabel:"更多选项",attachmentMoreMenu:"更多选项",richTextNewBulletedListItemAnnouncement:"项目符号，新行",richTextNewNumberedListItemAnnouncement:"{0}，新行",richTextBulletedListAppliedAnnouncement:"已应用项目符号列表",richTextNumberedListAppliedAnnouncement:"已应用编号列表"};var realTimeTextModal$1={realTimeTextModalAriaLabel:"打开 RTT 模式",realTimeTextModalTitle:"是否启用 RTT?",realTimeTextModalText:"将为会议中的每个人启用实时文本 (RTT)。",realTimeTextConfirmButtonLabel:"启用",realTimeTextCancelButtonLabel:"取消",realTimeTextCloseModalButtonAriaLabel:"关闭 RTT 模式"};var captionsBanner$1={captionsBannerSpinnerText:"正在启动字幕...",realTimeTextInputBoxDefaultText:"实时键入消息",realTimeTextInputErrorMessage:"消息不能超过 2000 个字符",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (在会议的整个持续时间内为所有参与者启用实时文本)。",realTimeTextBannerLinkLabel:"了解详情",captionsOnlyContainerTitle:"字幕",realTimeTextOnlyContainerTitle:"实时文本",captionsAndRealTimeTextContainerTitle:"字幕和实时文本",expandButtonAriaLabel:"展开",minimizeButtonAriaLabel:"折叠"};var mentionPopover$1={mentionPopoverHeader:"建议"};var imageOverlay$1={downloadButtonLabel:"下载",dismissButtonAriaLabel:"关闭",overlayVisibleMessage:"图像详细信息覆盖"};var messageStatusIndicator$1={deliveredAriaLabel:"已发送消息",deliveredTooltipText:"已发送",seenAriaLabel:"其他人已查看消息",seenTooltipText:"已查看",readByTooltipText:"{messageThreadReadCount}/{remoteParticipantsCount} 人已读",sendingAriaLabel:"消息发送",sendingTooltipText:"正在发送",failedToSendAriaLabel:"消息发送失败",failedToSendTooltipText:"发送失败"};var endCallButton$1={label:"退出",tooltipContent:"离开通话",leaveOption:"退出",endCallOption:"结束所有人的通话"};var cameraButton$1={onLabel:"相机",offLabel:"相机",tooltipDisabledContent:"已禁用照相机",tooltipOnContent:"关闭摄像头",tooltipOffContent:"打开摄像头",tooltipVideoLoadingContent:"正在加载视频",cameraMenuTitle:"相机",cameraMenuTooltip:"选择“相机”",cameraButtonSplitRoleDescription:"拆分按钮",onSplitButtonAriaLabel:"关闭摄像头",offSplitButtonAriaLabel:"打开摄像头",cameraActionTurnedOnAnnouncement:"你的相机已打开",cameraActionTurnedOffAnnouncement:"你的相机已关闭",onSplitButtonPrimaryActionCamera:"关闭摄像头",offSplitButtonPrimaryActionCamera:"打开摄像头",cameraPrimaryActionSplitButtonTitle:"使用照相机",videoEffectsMenuItemTitle:"效果",cameraButtonAriaDescription:"摄像头选项"};var microphoneButton$1={onLabel:"麦克风",offLabel:"麦克风",tooltipDisabledContent:"已禁用麦克风",tooltipOnContent:"将麦克风静音",tooltipOffContent:"取消麦克风静音",microphoneMenuTitle:"麦克风",microphoneMenuTooltip:"选择“麦克风”",speakerMenuTitle:"扬声器",speakerMenuTooltip:"选择“扬声器”",microphoneButtonSplitRoleDescription:"拆分按钮",onSplitButtonAriaLabel:"将麦克风静音",offSplitButtonAriaLabel:"取消麦克风静音",microphoneActionTurnedOnAnnouncement:"你的麦克风已打开",microphoneActionTurnedOffAnnouncement:"你的麦克风已关闭",offSplitButtonMicrophonePrimaryAction:"取消麦克风静音",onSplitButtonMicrophonePrimaryAction:"将麦克风静音",microphonePrimaryActionSplitButtonTitle:"使用麦克风",microphoneAriaDescription:"音频选项",deepNoiseSuppressionTitle:"噪音抑制",deepNoiseSuppressionOnAnnouncement:"深层噪音抑制已打开",deepNoiseSuppressionOffAnnouncement:"深层噪音抑制已关闭"};var devicesButton$1={label:"设备",tooltipContent:"管理设备",cameraMenuTitle:"相机",cameraMenuTooltip:"选择“相机”",audioDeviceMenuTitle:"音频设备",audioDeviceMenuTooltip:"选择音频设备",microphoneMenuTitle:"麦克风",microphoneMenuTooltip:"选择“麦克风”",speakerMenuTitle:"免提",speakerMenuTooltip:"选择“扬声器”"};var participantsButton$1={label:"人员",ariaLabel:"人脉",tooltipContent:"显示参与者",menuHeader:"在此通话中",participantsListButtonLabel:"{numParticipants} 人",muteAllButtonLabel:"全部静音",copyInviteLinkButtonLabel:"复制邀请链接",copyInviteLinkActionedAriaLabel:"已复制邀请链接"};var screenShareButton$1={onLabel:"停止演示",offLabel:"演示",tooltipDisabledContent:"已禁用演示。",tooltipOnContent:"演示屏幕",tooltipOffContent:"演示您的屏幕"};var raiseHandButton$1={offLabel:"举手",onLabel:"放下",tooltipDisabledContent:"举手操作已禁用",tooltipOnContent:"手放下",tooltipOffContent:"举手"};var reactionButton$1={label:"回应",ariaLabel:"回应",tooltipDisabledContent:"已禁用表情符号操作",tooltipContent:"发送表情符号",likeReactionTooltipContent:"点赞",heartReactionTooltipContent:"喜欢",laughReactionTooltipContent:"大笑",applauseReactionTooltipContent:"鼓掌",surprisedReactionTooltipContent:"惊讶"};var realTimeText$1={isTypingText:"RTT 键入"};var captionsSettingsModal$1={captionsSettingsModalTitle:"字幕设置",captionsSettingsSpokenLanguageDropdownLabel:"口语",captionsSettingsCaptionLanguageDropdownLabel:"辅助字幕语言",captionsSettingsSpokenLanguageDropdownInfoText:"此通话中的每个人都在讲的语言。",captionsSettingsCaptionLanguageDropdownInfoText:"字幕将以此语言显示。",captionsSettingsConfirmButtonLabel:"确认",captionsSettingsCancelButtonLabel:"取消",captionsSettingsModalAriaLabel:"字幕设置模式",captionsSettingsCloseModalButtonAriaLabel:"关闭字幕设置"};var spokenLanguages$1={"ar-ae":"阿拉伯语 - 阿拉伯联合酋长国","ar-sa":"阿拉伯语 - 沙特阿拉伯","da-dk":"丹麦语","de-de":"德语 - 德国","en-au":"英语 - 澳大利亚","en-ca":"英语 - 加拿大","en-gb":"英语 - 英国","en-in":"英语 - 印度","en-nz":"英语 - 新西兰","en-us":"英语 - 美国","es-es":"西班牙语 - 西班牙(现代风格)","es-mx":"西班牙语 - 墨西哥","fi-fi":"芬兰语","fr-ca":"法语 - 加拿大","fr-fr":"法语 - 法国","hi-in":"印地语","it-it":"意大利语–意大利","ja-jp":"日语","ko-kr":"韩语","nb-no":"挪威语(博克马尔语)","nl-be":"荷兰语 - 比利时","nl-nl":"荷兰语 - 荷兰","pl-pl":"波兰语","pt-br":"葡萄牙语 - 巴西","ru-ru":"俄语","sv-se":"瑞典语","zh-cn":"中文 - 中华人民共和国","zh-hk":"中文 - 香港特别行政区","cs-cz":"捷克语","pt-pt":"葡萄牙语 - 葡萄牙","tr-tr":"土耳其语","vi-vn":"越南语","th-th":"泰语","he-il":"希伯来语","cy-gb":"威尔士语","uk-ua":"乌克兰语","el-gr":"希腊语","hu-hu":"匈牙利语","ro-ro":"罗马尼亚语","sk-sk":"斯洛伐克","zh-tw":"中文 – 台湾"};var captionLanguages$1={ar:"阿拉伯语",da:"丹麦语",de:"德语",en:"英语",es:"西班牙语",fi:"芬兰语","fr-ca":"法语 - 加拿大",fr:"法语 - 法国",hi:"印地语",it:"意大利语",ja:"日语",ko:"韩语",nb:"挪威语(博克马尔语)",nl:"荷兰语",pl:"波兰语",pt:"葡萄牙语 - 巴西",ru:"俄语",sv:"瑞典语","zh-Hans":"简体中文","zh-Hant":"中文(繁体)",cs:"捷克语","pt-pt":"葡萄牙语 - 葡萄牙",tr:"土耳其语",vi:"越南语",th:"泰语",he:"希伯来语",cy:"威尔士语",uk:"乌克兰语",el:"希腊语",hu:"匈牙利语",ro:"罗马尼亚语",sk:"斯洛伐克"};var startCaptionsButton$1={onLabel:"开启字幕",offLabel:"关闭辅助字幕",tooltipOnContent:"关闭辅助字幕",tooltipOffContent:"开启字幕"};var startRealTimeTextButton$1={label:"为此通话启用 RTT",tooltipOffContent:"为此通话启用 RTT"};var messageThread$1={yesterday:"昨天",sunday:"星期日",monday:"星期一",tuesday:"星期二",wednesday:"星期三",thursday:"星期四",friday:"星期五",saturday:"星期六",participantJoined:"已加入聊天。",participantLeft:"已退出聊天。",editMessage:"编辑",removeMessage:"删除",resendMessage:"尝试再次发送",failToSendTag:"发送失败",editedTag:"已编辑",liveAuthorIntro:"来自 {author} 的消息",editedMessageLiveAuthorIntro:"已编辑来自 {author} 的消息",editedMessageLocalUserLiveAuthorIntro:"编辑的消息",messageContentAriaText:"{status}{author} 说 {message}",messageContentMineAriaText:"{status}你说了 {message}",editBoxTextLimit:"你的消息超过了{limitNumber}个字符的限制",editBoxPlaceholderText:"编辑消息",newMessagesIndicator:"新消息",noDisplayNameSub:"无名称",editBoxCancelButton:"取消",editBoxSubmitButton:"完成",messageReadCount:"{messageReadByCount}/{remoteParticipantsCount} 人已读",actionMenuMoreOptions:"更多选项",downloadAttachment:"下载文件",openAttachment:"打开文件",blockedWarningText:"由于组织策略，此消息已被删除。",blockedWarningLinkText:"详细信息",attachmentCardGroupMessage:"该消息包含 {attachmentCount} 个附件",messageDeletedAnnouncementAriaLabel:"消息已删除",richTextBoldTooltip:"粗体",richTextItalicTooltip:"斜体",richTextUnderlineTooltip:"下划线",richTextBulletListTooltip:"项目符号列表",richTextNumberListTooltip:"编号列表",richTextIncreaseIndentTooltip:"增加缩进量",richTextDecreaseIndentTooltip:"减少缩进量",richTextInsertTableTooltip:"插入表格",richTextFormatButtonTooltip:"格式",richTextInsertRowOrColumnMenu:"插入",richTextInsertTableMenuTitle:"插入 {column} x {row} 表",richTextInsertRowAboveMenu:"在上方插入",richTextInsertRowBelowMenu:"在下方插入",richTextInsertColumnLeftMenu:"在左侧插入",richTextInsertColumnRightMenu:"在右侧插入",richTextDeleteRowOrColumnMenu:"删除",richTextDeleteColumnMenu:"删除列",richTextDeleteRowMenu:"删除行",richTextDeleteTableMenu:"删除表格",richTextToolbarMoreButtonAriaLabel:"更多选项",attachmentUploadsPendingError:"正在上传文件，请稍候。",imageUploadsPendingError:"正在上传图像，请稍候。",richTextNewBulletedListItemAnnouncement:"项目符号，新行",richTextNewNumberedListItemAnnouncement:"{0}，新行",richTextBulletedListAppliedAnnouncement:"已应用项目符号列表",richTextNumberedListAppliedAnnouncement:"已应用编号列表"};var errorBar$1={unableToReachChatService:"你已脱机",accessDenied:"无法访问聊天服务 - 请检查提供的用户凭据",userNotInChatThread:"你不再处于此聊天会话中",sendMessageNotInChatThread:"无法发送消息，因为你不再在此聊天会话中",sendMessageGeneric:"未能发送消息",callingNetworkFailure:"正在连接呼叫 - 你似乎处于脱机状态",startVideoGeneric:"无法启动视频",stopVideoGeneric:"无法停止视频",muteGeneric:"无法静音麦克风",unmuteGeneric:"无法取消麦克风静音",speakingWhileMuted:"你的麦克风已静音",startScreenShareGeneric:"启动屏幕共享时出现问题。",stopScreenShareGeneric:"无法停止屏幕共享",callNetworkQualityLow:"网络质量低。",teamsMeetingCallNetworkQualityLow:"网络质量低。",callNoSpeakerFound:"找不到扬声器或耳机。连接音频设备以收听呼叫。",callNoMicrophoneFound:"找不到麦克风。连接音频输入设备。",callMicrophoneAccessDenied:"无法访问麦克风。单击地址栏中的锁定以授予对此网页的权限。",callMicrophoneAccessDeniedSafari:"无法访问麦克风。刷新页面以允许权限，或检查此浏览器的设置并验证是否为此网站启用了权限。",callMicrophoneMutedBySystem:"系统已将你设为静音。",callMicrophoneUnmutedBySystem:"你的麦克风已恢复，系统已取消静音。",callMacOsMicrophoneAccessDenied:"无法访问麦克风。在 macOS 隐私设置中授予麦克风权限。",callLocalVideoFreeze:"网络带宽较差。你的视频在会议中的其他人看来可能会卡顿。",callCameraAccessDenied:"无法访问照相机。单击地址栏中的锁定以授予对此网页的权限。",callCameraAccessDeniedSafari:"无法访问照相机。刷新页面以允许权限，或检查此浏览器的设置并验证是否为此网站启用了权限。",callCameraAlreadyInUse:"无法访问照相机。它可能已被其他应用程序使用。",callVideoStoppedBySystem:"你的系统已停止你的视频。",callVideoRecoveredBySystem:"你的视频已恢复。",callMacOsCameraAccessDenied:"MacOS 正在阻止访问你的相机。更新你的隐私设置以允许此浏览器访问你的相机。",callMacOsScreenShareAccessDenied:"MacOS 正在阻止屏幕共享。更新你的隐私设置以允许此浏览器录制你的屏幕。",dismissButtonAriaLabel:"关闭",failedToJoinCallGeneric:"未能加入通话。",failedToJoinCallInvalidMeetingLink:"无法加入会议。无效的链接。",cameraFrozenForRemoteParticipants:"通话中的用户在查看视频时遇到问题。请检查你的设备和网络。",unableToStartVideoEffect:"无法应用视频效果。",startSpotlightWhileMaxParticipantsAreSpotlighted:"无法聚焦参与者，因为聚焦的参与者人数已达到上限。",mutedByRemoteParticipant:"另一个参与者已将你静音。"};var notificationStack$1={callingNetworkFailure:{title:"连接通话时遇到问题",message:"你似乎处于脱机状态。请检查网络连接，然后重试。",dismissButtonAriaLabel:"关闭"},startVideoGeneric:{title:"无法启动视频。",dismissButtonAriaLabel:"关闭"},stopVideoGeneric:{title:"无法停止视频。",dismissButtonAriaLabel:"关闭"},muteGeneric:{title:"无法将麦克风静音。",dismissButtonAriaLabel:"关闭"},unmuteGeneric:{title:"无法取消麦克风静音。",dismissButtonAriaLabel:"关闭"},speakingWhileMuted:{title:"你已静音。",dismissButtonAriaLabel:"关闭"},startScreenShareGeneric:{title:"启动屏幕共享时出现问题。",dismissButtonAriaLabel:"关闭"},stopScreenShareGeneric:{title:"无法停止屏幕共享。",dismissButtonAriaLabel:"关闭"},callNetworkQualityLow:{title:"网络质量低。",dismissButtonAriaLabel:"关闭"},teamsMeetingCallNetworkQualityLow:{title:"网络质量低。",dismissButtonAriaLabel:"关闭",message:"从你的手机加入此通话以获得更好的声音。你可以继续在此设备上查看会议。",primaryButtonLabel:"通过电话加入"},callNoSpeakerFound:{title:"找不到扬声器或耳机",message:"连接音频设备以收听呼叫。",dismissButtonAriaLabel:"关闭"},callNoMicrophoneFound:{title:"未找到麦克风",message:"连接音频输入设备。",dismissButtonAriaLabel:"关闭"},callMicrophoneAccessDenied:{title:"无法访问麦克风",message:"单击地址栏中的锁定以授予对此网页的权限。",dismissButtonAriaLabel:"关闭"},callMicrophoneAccessDeniedSafari:{title:"无法访问麦克风",message:"刷新页面以允许权限，或检查此浏览器的设置并验证是否为此网站启用了权限。",dismissButtonAriaLabel:"关闭"},callMicrophoneMutedBySystem:{title:"系统已将你设为静音。",dismissButtonAriaLabel:"关闭"},callMicrophoneUnmutedBySystem:{title:"你的麦克风已恢复，系统已取消静音。",dismissButtonAriaLabel:"关闭"},callMacOsMicrophoneAccessDenied:{title:"无法访问麦克风",message:"在 macOS 隐私设置中授予麦克风权限。",dismissButtonAriaLabel:"关闭"},callLocalVideoFreeze:{title:"网络带宽较差",message:"您的视频可能会对通话中的其他人暂停。",dismissButtonAriaLabel:"关闭"},callCameraAccessDenied:{title:"无法访问摄像头",message:"单击地址栏中的锁定以授予对此网页的权限。",dismissButtonAriaLabel:"关闭"},callCameraAccessDeniedSafari:{title:"无法访问摄像头",message:"刷新页面以允许权限，或检查此浏览器的设置并验证是否为此网站启用了权限。",dismissButtonAriaLabel:"关闭"},callCameraAlreadyInUse:{title:"无法访问摄像头",message:"它可能已被其他应用程序使用。",dismissButtonAriaLabel:"关闭"},callVideoStoppedBySystem:{title:"你的系统已停止你的视频。",dismissButtonAriaLabel:"关闭"},callVideoRecoveredBySystem:{title:"你的视频已恢复。",dismissButtonAriaLabel:"关闭"},callMacOsCameraAccessDenied:{title:"MacOS 正在阻止对摄像头的访问",message:"更新你的隐私设置以允许此浏览器访问你的相机。",dismissButtonAriaLabel:"关闭"},callMacOsScreenShareAccessDenied:{title:"MacOS 正在阻止屏幕共享",message:"更新你的隐私设置以允许此浏览器录制你的屏幕。",dismissButtonAriaLabel:"关闭"},failedToJoinCallGeneric:{title:"无法加入通话。",dismissButtonAriaLabel:"关闭"},failedToJoinCallInvalidMeetingLink:{title:"无法加入会议。无效的链接。",dismissButtonAriaLabel:"关闭"},cameraFrozenForRemoteParticipants:{title:"通话中的用户在查看视频时遇到问题",message:"请检查你的设备和网络。",dismissButtonAriaLabel:"关闭"},unableToStartVideoEffect:{title:"无法应用视频效果。",dismissButtonAriaLabel:"关闭"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"无法在参与者上启动聚焦",message:"已聚焦的最大参与者数。",dismissButtonAriaLabel:"关闭"},mutedByRemoteParticipant:{title:"另一个参与者已将你静音。",dismissButtonAriaLabel:"关闭"},recordingStarted:{title:"已开始录制。",message:"加入即表示你同意转录此会议。",dismissButtonAriaLabel:"关闭"},transcriptionStarted:{title:"听录已开始。",message:"加入即表示你同意转录此会议。",dismissButtonAriaLabel:"关闭"},recordingStopped:{title:"正在保存录制内容。",message:"录制已停止。",dismissButtonAriaLabel:"关闭"},transcriptionStopped:{title:"正在保存听录。",message:"听录已停止。",dismissButtonAriaLabel:"关闭"},recordingAndTranscriptionStarted:{title:"录制和听录已开始。",message:"加入即表示你同意转录此会议。",dismissButtonAriaLabel:"关闭"},recordingAndTranscriptionStopped:{title:"正在保存录制和听录。",message:"录制和听录已停止。",dismissButtonAriaLabel:"关闭"},recordingStoppedStillTranscribing:{title:"录制已停止。",message:"现在仅听录此会议。",dismissButtonAriaLabel:"关闭"},transcriptionStoppedStillRecording:{title:"听录已停止。",message:"现在仅录制此会议。",dismissButtonAriaLabel:"关闭"},assignedBreakoutRoomOpened:{title:"我们将在 5 秒后将你移动到你分配的聊天室。",dismissButtonAriaLabel:"关闭"},assignedBreakoutRoomChanged:{title:"你的分组讨论室已更改",message:"我们将在 5 秒后将你移动到你分配的聊天室。",dismissButtonAriaLabel:"关闭"},assignedBreakoutRoomClosed:{title:"你的分组讨论室已关闭",message:"我们将在 5 秒后将你移回会议。",dismissButtonAriaLabel:"关闭"},assignedBreakoutRoomOpenedPromptJoin:{title:"要加入分组讨论室吗?",message:"已将你分配到一间分组讨论室。",dismissButtonAriaLabel:"关闭",primaryButtonLabel:"加入分组讨论室",secondaryButtonLabel:"以后再说"},breakoutRoomJoined:{title:"你已加入分组讨论室。",dismissButtonAriaLabel:"关闭"},breakoutRoomClosingSoon:{title:"分组讨论室时间限制即将过期。",message:"此分组讨论室将在 30 秒内关闭",dismissButtonAriaLabel:"关闭"},capabilityTurnVideoOnPresent:{title:"已启用你的摄像头",message:"如果想要共享视频，请将其打开。",dismissButtonAriaLabel:"关闭"},capabilityTurnVideoOnAbsent:{title:"你的摄像头已被禁用",message:"你无法再共享视频。",dismissButtonAriaLabel:"关闭"},capabilityUnmuteMicPresent:{title:"已启用麦克风",message:"你想说话时，请取消静音。",dismissButtonAriaLabel:"关闭"},capabilityUnmuteMicAbsent:{title:"已禁用麦克风",message:"你无法再取消静音。",dismissButtonAriaLabel:"关闭"},togetherModeStarted:{title:"Togethermode 已启动",dismissButtonAriaLabel:"关闭"},togetherModeEnded:{title:"Togethermode 已结束",dismissButtonAriaLabel:"关闭"},transcriptionError:{title:"听录出现问题。",message:"听录已停止。请稍后重试",dismissButtonAriaLabel:"关闭"},transcriptionStartedByYou:{title:"听录已开始。",message:"已由你启动。告知所有人正在听录他们所说的内容。",dismissButtonAriaLabel:"关闭"}};var videoGallery$1={screenIsBeingSharedMessage:"你正在共享你的屏幕",screenShareLoadingMessage:"正在加载 {participant} 的屏幕",localScreenShareLoadingMessage:"正在加载屏幕",localVideoLabel:"你",localVideoCameraSwitcherLabel:"切换相机",localVideoMovementLabel:"可移动本地小窗视频",localVideoMovementAriaLabel:"可移动本地小窗视频",localVideoSelectedDescription:"已选择 {cameraName}",displayNamePlaceholder:"未命名的参与者",fitRemoteParticipantToFrame:"适应帧大小",fillRemoteParticipantFrame:"填充帧",pinParticipantForMe:"为我固定",pinParticipantForMeLimitReached:"固定(已达到上限)",unpinParticipantForMe:"取消固定",pinParticipantMenuItemAriaLabel:"固定 {participantName}",unpinParticipantMenuItemAriaLabel:"取消固定 {participantName}",pinnedParticipantAnnouncementAriaLabel:"已固定 {participantName}",unpinnedParticipantAnnouncementAriaLabel:"已取消固定 {participantName}",startSpotlightVideoTileMenuLabel:"对所有人聚焦",addSpotlightVideoTileMenuLabel:"添加聚焦",spotlightLimitReachedMenuTitle:"已达到聚焦限制",stopSpotlightVideoTileMenuLabel:"停止聚焦",stopSpotlightOnSelfVideoTileMenuLabel:"退出聚焦",attendeeRole:"与会者",muteParticipantMenuItemLabel:"静音",waitingScreenText:"正在等待其他人加入",forbidAudio:"禁用麦克风",permitAudio:"允许使用麦克风",forbidAudioTileMenuLabel:"禁用麦克风",permitAudioTileMenuLabel:"允许使用麦克风",forbidVideo:"禁用摄像头",permitVideo:"允许使用摄像头",forbidVideoTileMenuLabel:"禁用摄像头",permitVideoTileMenuLabel:"允许使用摄像头"};var dialpad$1={placeholderText:"输入电话号码",deleteButtonAriaLabel:"删除"};var holdButton$1={onLabel:"恢复",offLabel:"支持",tooltipOnContent:"继续呼叫",tooltipOffContent:"保持呼叫"};var videoTile$1={participantStateRinging:"正在呼叫...",participantReconnecting:"正在重新连接...",participantStateHold:"保留",moreOptionsButtonAriaLabel:"更多选项 {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"已静音",moreOptionsParticipantMutedStateUnmutedAriaLabel:"已取消静音",moreOptionsParticipantHandRaisedAriaLabel:"已举手",moreOptionsParticipantIsSpeakingAriaLabel:"讲话",moreOptionsParticipantMicDisabledAriaLabel:"麦克风已禁用",moreOptionsParticipantCameraDisabledAriaLabel:"相机已禁用"};var CameraAndMicrophoneSitePermissionsRequest$1={primaryText:"允许{appName}使用你的相机和麦克风",secondaryText:"这样参与者就可以看到并听到你的声音。",linkText:"需要帮助? 获取疑难解答帮助",primaryButtonText:"不使用相机和麦克风继续",ariaLabel:"允许相机和麦克风访问权限"};var CameraSitePermissionsRequest$1={primaryText:"允许 {appName} 使用你的相机",secondaryText:"这样参与者就可以看到你了。",linkText:"需要帮助? 获取疑难解答帮助",primaryButtonText:"不使用摄像头继续",ariaLabel:"允许相机访问权限"};var MicrophoneSitePermissionsRequest$1={primaryText:"允许 {appName} 使用你的麦克风",secondaryText:"这样参与者就可以听到你的声音。",linkText:"需要帮助? 获取疑难解答帮助",primaryButtonText:"不使用麦克风继续",ariaLabel:"允许麦克风访问权限"};var CameraAndMicrophoneSitePermissionsCheck$1={primaryText:"检查摄像头和麦克风访问权限",secondaryText:"如果出现提示，请允许访问。这样参与者就可以看到并听到你的声音。",linkText:"需要帮助? 获取疑难解答帮助",primaryButtonText:"不使用相机和麦克风继续",ariaLabel:"正在检查摄像头和麦克风访问权限。如果出现提示，请允许访问。"};var CameraSitePermissionsCheck$1={primaryText:"检查摄像头访问权限",secondaryText:"如果出现提示，请允许访问。这样参与者就可以看到你了。",linkText:"需要帮助? 获取疑难解答帮助",primaryButtonText:"不使用相机继续",ariaLabel:"正在检查摄像头访问权限。如果出现提示，请允许访问。"};var MicrophoneSitePermissionsCheck$1={primaryText:"正在检查麦克风访问权限",secondaryText:"如果出现提示，请允许访问。这样参与者就可以听到你的声音。",linkText:"需要帮助? 获取疑难解答帮助",primaryButtonText:"不使用麦克风继续",ariaLabel:"正在检查麦克风访问权限。如果出现提示，请允许访问。"};var CameraAndMicrophoneSitePermissionsDenied$1={primaryText:"无法访问相机和麦克风",secondaryText:"单击地址栏中的锁图标以向此网页授予麦克风权限。可能需要页面刷新。",primaryButtonText:"不使用摄像头和麦克风继续",linkText:"需要帮助? 获取疑难解答帮助"};var CameraAndMicrophoneSitePermissionsDeniedSafari$1={primaryText:"无法访问相机和麦克风",secondaryText:"刷新页面以允许权限，或检查此浏览器的设置并验证是否为此网站启用了权限。",primaryButtonText:"不使用摄像头和麦克风继续",linkText:"需要帮助? 获取疑难解答帮助"};var CameraSitePermissionsDenied$1={primaryText:"无法访问摄像头",secondaryText:"单击地址栏中的锁图标以向此网页授予相机权限。可能需要页面刷新。",primaryButtonText:"不使用摄像头继续",linkText:"需要帮助? 获取疑难解答帮助"};var MicrophoneSitePermissionsDenied$1={primaryText:"无法访问麦克风",secondaryText:"单击地址栏中的锁图标以向此网页授予麦克风权限。可能需要页面刷新。",primaryButtonText:"不使用麦克风继续",linkText:"需要帮助? 获取疑难解答帮助"};var CameraSitePermissionsDeniedSafari$1={primaryText:"无法访问相机",secondaryText:"刷新页面以允许权限，或检查此浏览器的设置并验证是否为此网站启用了权限。",primaryButtonText:"不使用相机继续",linkText:"需要帮助? 获取疑难解答帮助"};var MicrophoneSitePermissionsDeniedSafari$1={primaryText:"无法访问麦克风",secondaryText:"刷新页面以允许权限，或检查此浏览器的设置并验证是否为此网站启用了权限。",primaryButtonText:"不使用麦克风继续",linkText:"需要帮助? 获取疑难解答帮助"};var UnsupportedBrowser$1={primaryText:"浏览器不受支持",secondaryText:"请使用兼容的浏览器加入此通话。",moreHelpLinkText:"查看兼容性要求"};var UnsupportedBrowserVersion$1={primaryText:"需要浏览器更新",secondaryText:"为确保通话达到最佳效果，请更新浏览器，然后再次尝试加入呼叫。",moreHelpLinkText:"查看兼容性要求",continueAnywayButtonText:"在不更新的情况下启动呼叫"};var UnsupportedOperatingSystem$1={primaryText:"不支持操作系统",secondaryText:"请使用具有兼容的操作系统的设备加入此通话。",moreHelpLinkText:"查看兼容性要求"};var BrowserPermissionDenied$1={primaryText:"无法使用摄像头或麦克风",secondaryText:"你的浏览器可能无法访问你的相机或麦克风。若要解决此问题，请打开“系统首选项”。",primaryButtonText:"重试",linkText:"需要帮助? 获取疑难解答帮助"};var BrowserPermissionDeniedIOS$1={primaryText:"允许麦克风访问以继续",secondaryText:"这样其他参与者就可以听到你的声音。",primaryButtonText:"重试",imageAltText:"iOS 的麦克风和摄像头设备权限位置",linkText:"需要帮助? 获取疑难解答帮助",step1Text:"转到设置应用",step2Text:"向下滚动到此浏览器的设置",step3Text:"打开麦克风(摄像头可选)",step4Text:"再次尝试加入通话",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery$1={leftNavButtonAriaLabel:"上一页 {current} (共 {total} 页)",rightNavButtonAriaLabel:"下一页 {current} (共 {total} 页)"};var meetingConferencePhoneInfo$1={meetingConferencePhoneInfoModalTitle:"从手机加入呼叫",meetingConferencePhoneInfoModalDialIn:"拨号号码",meetingConferencePhoneInfoModalMeetingId:"输入会议 ID",meetingConferencePhoneInfoModalWait:"等待组织者接受",meetingConferencePhoneInfoModalNoPhoneAvailable:"没有可用的电话",meetingConferencePhoneInfoModalTollGeoData:"{country}，{city}",meetingConferencePhoneInfoModalTollFree:"(免费电话)",meetingConferencePhoneInfoModalToll:"(收费电话)"};var IncomingCallNotification$1={incomingCallNotificationPlaceholderId:"未知用户",incomingCallNotificationPlaceholderAlert:"{callerName} 正在通话",incomingCallNoticicationAcceptWithAudioAriaLabel:"语音接听",incomingCallNoticicationAcceptWithVideoAriaLabel:"视频接听",incomingCallNoticicationRejectAriaLabel:"拒绝",incomingCallNotificationAcceptButtonLabel:"接受",incomingCallNotificationRejectButtonLabel:"拒绝",incomingCallNotificationAccceptWithVideoButtonLabel:"使用视频接受",incomingCallNotificationDismissButtonAriaLabel:"关闭"};var zh_CN$1 = {participantItem:participantItem$1,ParticipantList:ParticipantList$1,typingIndicator:typingIndicator$1,sendBox:sendBox$1,richTextSendBox:richTextSendBox$1,realTimeTextModal:realTimeTextModal$1,captionsBanner:captionsBanner$1,mentionPopover:mentionPopover$1,imageOverlay:imageOverlay$1,messageStatusIndicator:messageStatusIndicator$1,endCallButton:endCallButton$1,cameraButton:cameraButton$1,microphoneButton:microphoneButton$1,devicesButton:devicesButton$1,participantsButton:participantsButton$1,screenShareButton:screenShareButton$1,raiseHandButton:raiseHandButton$1,reactionButton:reactionButton$1,realTimeText:realTimeText$1,captionsSettingsModal:captionsSettingsModal$1,spokenLanguages:spokenLanguages$1,captionLanguages:captionLanguages$1,startCaptionsButton:startCaptionsButton$1,startRealTimeTextButton:startRealTimeTextButton$1,messageThread:messageThread$1,errorBar:errorBar$1,notificationStack:notificationStack$1,videoGallery:videoGallery$1,dialpad:dialpad$1,holdButton:holdButton$1,videoTile:videoTile$1,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest$1,CameraSitePermissionsRequest:CameraSitePermissionsRequest$1,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest$1,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck$1,CameraSitePermissionsCheck:CameraSitePermissionsCheck$1,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck$1,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied$1,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari$1,CameraSitePermissionsDenied:CameraSitePermissionsDenied$1,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied$1,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari$1,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari$1,UnsupportedBrowser:UnsupportedBrowser$1,UnsupportedBrowserVersion:UnsupportedBrowserVersion$1,UnsupportedOperatingSystem:UnsupportedOperatingSystem$1,BrowserPermissionDenied:BrowserPermissionDenied$1,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS$1,verticalGallery:verticalGallery$1,meetingConferencePhoneInfo:meetingConferencePhoneInfo$1,IncomingCallNotification:IncomingCallNotification$1};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Chinese (Mainland China).
 *
 * @public
 */
const COMPONENT_LOCALE_ZH_CN = {
    strings: createComponentStrings(zh_CN$1)
};

var participantItem={isMeText:"(您)",menuTitle:"更多選項",removeButtonLabel:"移除",sharingIconLabel:"分享",mutedIconLabel:"已靜音",handRaisedIconLabel:"舉手- 位置 {position}",displayNamePlaceholder:"未命名的參與者",participantStateRinging:"撥號中...",participantStateHold:"電話保留",participantItemAriaLabel:"{displayName}, {handRaisedState}, {mutedState}, {micDisabledState}, {cameraDisabledState}, {sharingState}, {connectionState}",participantItemWithMoreOptionsAriaLabel:"{displayName}、{handRaisedState}、{mutedState}、{micDisabledState}、{cameraDisabledState}、{sharingState}、{connectionState}、其他選項",attendeeRole:"出席者",micDisabledIconLabel:"已停用麥克風",cameraDisabledIconLabel:"已停用相機"};var ParticipantList={overflowParticipantCount:"多出 {overflowCount} 個"};var typingIndicator={singleUser:"{user} 正在輸入...",multipleUsers:"{users} 正在輸入...",multipleUsersAbbreviateOne:"{users} 和其他 1 人正在輸入...",multipleUsersAbbreviateMany:"{users} 和其他 {numOthers} 人正在輸入...",delimiter:"， "};var sendBox={placeholderText:"輸入訊息",textTooLong:"您的訊息長度超過上限。",sendButtonAriaLabel:"傳送訊息",attachmentUploadsPendingError:"正在上傳檔案，請稍候。",removeAttachment:"移除檔案",uploading:"正在上傳",uploadCompleted:"上傳完成",attachmentMoreMenu:"其他選項"};var richTextSendBox={placeholderText:"輸入訊息",textTooLong:"您的訊息長度超過上限。",sendButtonAriaLabel:"傳送訊息",attachmentUploadsPendingError:"正在上傳檔案，請稍候。",imageUploadsPendingError:"正在上傳影像，請稍候。",removeAttachment:"移除檔案",uploading:"正在上傳",uploadCompleted:"上傳完成",richTextBoldTooltip:"粗體",richTextItalicTooltip:"斜體",richTextUnderlineTooltip:"底線",richTextBulletListTooltip:"項目符號清單",richTextNumberListTooltip:"編號清單",richTextIncreaseIndentTooltip:"增加縮排",richTextDecreaseIndentTooltip:"減少縮排",richTextInsertTableTooltip:"插入表格",richTextFormatButtonTooltip:"格式",richTextInsertRowOrColumnMenu:"插入",richTextInsertTableMenuTitle:"插入 {column} x {row} 表格",richTextInsertRowAboveMenu:"在上方插入",richTextInsertRowBelowMenu:"在下方插入",richTextInsertColumnLeftMenu:"於左方插入",richTextInsertColumnRightMenu:"於右方插入",richTextDeleteRowOrColumnMenu:"刪除",richTextDeleteColumnMenu:"刪除欄",richTextDeleteRowMenu:"刪除列",richTextDeleteTableMenu:"刪除表格",richTextToolbarAriaLabel:"訊息格式設定工具列",richTextToolbarMoreButtonAriaLabel:"其他選項",attachmentMoreMenu:"其他選項",richTextNewBulletedListItemAnnouncement:"項目符號，新行",richTextNewNumberedListItemAnnouncement:"{0}，新行",richTextBulletedListAppliedAnnouncement:"已套用項目符號清單",richTextNumberedListAppliedAnnouncement:"已套用編號清單"};var realTimeTextModal={realTimeTextModalAriaLabel:"開啟 RTT 強制回應",realTimeTextModalTitle:"開啟 RTT?",realTimeTextModalText:"將為會議中的每個人開啟即時文字 (RTT)。",realTimeTextConfirmButtonLabel:"開啟",realTimeTextCancelButtonLabel:"取消",realTimeTextCloseModalButtonAriaLabel:"關閉 RTT 強制回應"};var captionsBanner={captionsBannerSpinnerText:"正在啟動輔助字幕...",realTimeTextInputBoxDefaultText:"即時輸入訊息",realTimeTextInputErrorMessage:"訊息不得超過 2000 個字元",realTimeTextBannerTitle:"即時輸入",realTimeTextBannerContent:"已為整個會議期間的所有參與者啟用 RTT (即時文字)。",realTimeTextBannerLinkLabel:"深入了解",captionsOnlyContainerTitle:"輔助字幕",realTimeTextOnlyContainerTitle:"即時文字",captionsAndRealTimeTextContainerTitle:"輔助字幕與即時文字",expandButtonAriaLabel:"展開",minimizeButtonAriaLabel:"摺疊"};var mentionPopover={mentionPopoverHeader:"建議"};var imageOverlay={downloadButtonLabel:"下載",dismissButtonAriaLabel:"關閉",overlayVisibleMessage:"影像詳細資料覆疊"};var messageStatusIndicator={deliveredAriaLabel:"訊息已傳送",deliveredTooltipText:"已傳送",seenAriaLabel:"其他人已看到訊息",seenTooltipText:"已讀",readByTooltipText:"讀取者為 {messageThreadReadCount} / {remoteParticipantsCount}",sendingAriaLabel:"正在傳送訊息",sendingTooltipText:"正在傳送",failedToSendAriaLabel:"無法傳送訊息",failedToSendTooltipText:"無法傳送"};var endCallButton={label:"離開",tooltipContent:"離開通話",leaveOption:"離開",endCallOption:"結束所有通話"};var cameraButton={onLabel:"相機",offLabel:"相機",tooltipDisabledContent:"相機已停用",tooltipOnContent:"關閉攝影機",tooltipOffContent:"開啟相機",tooltipVideoLoadingContent:"正在載入影片",cameraMenuTitle:"相機",cameraMenuTooltip:"選擇攝影機",cameraButtonSplitRoleDescription:"[分割] 按鈕",onSplitButtonAriaLabel:"關閉攝影機",offSplitButtonAriaLabel:"開啟相機",cameraActionTurnedOnAnnouncement:"您的相機已開啟",cameraActionTurnedOffAnnouncement:"您的相機已關閉",onSplitButtonPrimaryActionCamera:"關閉攝影機",offSplitButtonPrimaryActionCamera:"開啟相機",cameraPrimaryActionSplitButtonTitle:"使用相機",videoEffectsMenuItemTitle:"效果",cameraButtonAriaDescription:"攝影機選項"};var microphoneButton={onLabel:"麥克風",offLabel:"麥克風",tooltipDisabledContent:"麥克風已停用",tooltipOnContent:"將麥克風設為靜音",tooltipOffContent:"取消麥克風靜音",microphoneMenuTitle:"麥克風",microphoneMenuTooltip:"選擇麥克風",speakerMenuTitle:"揚聲器",speakerMenuTooltip:"選擇喇叭",microphoneButtonSplitRoleDescription:"[分割] 按鈕",onSplitButtonAriaLabel:"將麥克風靜音",offSplitButtonAriaLabel:"取消麥克風靜音",microphoneActionTurnedOnAnnouncement:"您的麥克風已開啟",microphoneActionTurnedOffAnnouncement:"您的麥克風已關閉",offSplitButtonMicrophonePrimaryAction:"取消麥克風靜音",onSplitButtonMicrophonePrimaryAction:"將麥克風靜音",microphonePrimaryActionSplitButtonTitle:"使用麥克風",microphoneAriaDescription:"音訊選項",deepNoiseSuppressionTitle:"噪音抑制",deepNoiseSuppressionOnAnnouncement:"深噪音抑制已開啟",deepNoiseSuppressionOffAnnouncement:"深噪音抑制已關閉"};var devicesButton={label:"裝置",tooltipContent:"管理裝置",cameraMenuTitle:"攝影機",cameraMenuTooltip:"選擇攝影機",audioDeviceMenuTitle:"音訊裝置",audioDeviceMenuTooltip:"選擇音訊裝置",microphoneMenuTitle:"麥克風",microphoneMenuTooltip:"選擇麥克風",speakerMenuTitle:"演講者",speakerMenuTooltip:"選擇喇叭"};var participantsButton={label:"人員",ariaLabel:"人員",tooltipContent:"顯示參與者",menuHeader:"在此通話中",participantsListButtonLabel:"{numParticipants} 人員",muteAllButtonLabel:"將所有人設為靜音",copyInviteLinkButtonLabel:"複製邀請連結",copyInviteLinkActionedAriaLabel:"已複製邀請連結"};var screenShareButton={onLabel:"停止簡報",offLabel:"分享",tooltipDisabledContent:"簡報已停用",tooltipOnContent:"展示您的螢幕",tooltipOffContent:"展示您的螢幕"};var raiseHandButton={offLabel:"舉手",onLabel:"較低",tooltipDisabledContent:"舉手動作已停用",tooltipOnContent:"把手放下",tooltipOffContent:"舉手"};var reactionButton={label:"傳送表情符號",ariaLabel:"傳送表情符號",tooltipDisabledContent:"已停用表情符號動作",tooltipContent:"傳送表情符號",likeReactionTooltipContent:"按讚",heartReactionTooltipContent:"喜歡",laughReactionTooltipContent:"大笑",applauseReactionTooltipContent:"鼓掌",surprisedReactionTooltipContent:"驚訝"};var realTimeText={isTypingText:"RTT 輸入"};var captionsSettingsModal={captionsSettingsModalTitle:"字幕設定",captionsSettingsSpokenLanguageDropdownLabel:"口語語言",captionsSettingsCaptionLanguageDropdownLabel:"字幕語言",captionsSettingsSpokenLanguageDropdownInfoText:"此通話中的每個人在說的語言。",captionsSettingsCaptionLanguageDropdownInfoText:"字幕將以此語言顯示。",captionsSettingsConfirmButtonLabel:"確認",captionsSettingsCancelButtonLabel:"取消",captionsSettingsModalAriaLabel:"字幕設定模式",captionsSettingsCloseModalButtonAriaLabel:"關閉字幕設定"};var spokenLanguages={"ar-ae":"阿拉伯文 - 阿拉伯聯合大公國","ar-sa":"阿拉伯文 - 沙烏地阿拉伯","da-dk":"丹麥文","de-de":"德文 - 德國","en-au":"英文 - 澳大利亞","en-ca":"英文 - 加拿大","en-gb":"英文 - 英國","en-in":"英文 - 印度","en-nz":"英文 - 紐西蘭","en-us":"英文 - 美國","es-es":"西班牙文 - 西班牙 (現代排序)","es-mx":"西班牙文 - 墨西哥","fi-fi":"芬蘭文","fr-ca":"法文 - 加拿大","fr-fr":"法文 - 法國","hi-in":"印度文","it-it":"義大利文 - 義大利","ja-jp":"日文","ko-kr":"韓文","nb-no":"挪威文 (巴克摩)","nl-be":"荷蘭文 - 比利時","nl-nl":"荷蘭文 - 荷蘭","pl-pl":"波蘭文","pt-br":"葡萄牙文 - 巴西","ru-ru":"俄文","sv-se":"瑞典文","zh-cn":"中文 - 中華人民共和國","zh-hk":"中文 - 香港特別行政區","cs-cz":"捷克文","pt-pt":"葡萄牙文 - 葡萄牙","tr-tr":"土耳其文","vi-vn":"越南文","th-th":"泰文","he-il":"希伯來文","cy-gb":"威爾斯文","uk-ua":"烏克蘭文","el-gr":"希臘文","hu-hu":"匈牙利文","ro-ro":"羅馬尼亞文","sk-sk":"斯洛伐克文","zh-tw":"中文 - 台灣"};var captionLanguages={ar:"阿拉伯文",da:"丹麥文",de:"德文",en:"英文",es:"西班牙文",fi:"芬蘭文","fr-ca":"法文 - 加拿大",fr:"法文 - 法國",hi:"印度文",it:"義大利文",ja:"日文",ko:"韓文",nb:"挪威文 (巴克摩)",nl:"荷蘭文",pl:"波蘭文",pt:"葡萄牙文 - 巴西",ru:"俄文",sv:"瑞典文","zh-Hans":"中文 (簡體)","zh-Hant":"中文 (繁體)",cs:"捷克文","pt-pt":"葡萄牙文 - 葡萄牙",tr:"土耳其文",vi:"越南文",th:"泰文",he:"希伯來文",cy:"威爾斯文",uk:"烏克蘭文",el:"希臘文",hu:"匈牙利文",ro:"羅馬尼亞文",sk:"斯洛伐克文"};var startCaptionsButton={onLabel:"開啟字幕",offLabel:"關閉字幕",tooltipOnContent:"關閉字幕",tooltipOffContent:"開啟字幕"};var startRealTimeTextButton={label:"開啟此通話的即時輸入",tooltipOffContent:"開啟此通話的即時輸入"};var messageThread={yesterday:"昨天",sunday:"星期日",monday:"星期一",tuesday:"星期二",wednesday:"星期三",thursday:"星期四",friday:"星期五",saturday:"星期六",participantJoined:"已加入聊天。",participantLeft:"已離開聊天。",editMessage:"編輯",removeMessage:"刪除",resendMessage:"請嘗試再次傳送",failToSendTag:"無法傳送",editedTag:"已編輯",liveAuthorIntro:"來自 {author} 的訊息",editedMessageLiveAuthorIntro:"已編輯來自 {author} 的訊息",editedMessageLocalUserLiveAuthorIntro:"已編輯訊息",messageContentAriaText:"{status} {author} 說 {message}",messageContentMineAriaText:"{status} 您說 {message}",editBoxTextLimit:"您的訊息超過 {limitNumber} 個字元的限制",editBoxPlaceholderText:"編輯您的訊息",newMessagesIndicator:"新訊息",noDisplayNameSub:"沒有名稱",editBoxCancelButton:"取消",editBoxSubmitButton:"完成",messageReadCount:"讀取者為 {messageReadByCount} / {remoteParticipantsCount}",actionMenuMoreOptions:"更多選項",downloadAttachment:"下載檔案",openAttachment:"開啟檔案",blockedWarningText:"由於組織原則，已刪除此訊息。",blockedWarningLinkText:"詳細資料",attachmentCardGroupMessage:"這則訊息有 {attachmentCount} 個附件",messageDeletedAnnouncementAriaLabel:"訊息已刪除",richTextBoldTooltip:"粗體",richTextItalicTooltip:"斜體",richTextUnderlineTooltip:"底線",richTextBulletListTooltip:"項目符號清單",richTextNumberListTooltip:"編號清單",richTextIncreaseIndentTooltip:"增加縮排",richTextDecreaseIndentTooltip:"減少縮排",richTextInsertTableTooltip:"插入表格",richTextFormatButtonTooltip:"格式",richTextInsertRowOrColumnMenu:"插入",richTextInsertTableMenuTitle:"插入 {column} x {row} 表格",richTextInsertRowAboveMenu:"在上方插入",richTextInsertRowBelowMenu:"在下方插入",richTextInsertColumnLeftMenu:"於左方插入",richTextInsertColumnRightMenu:"於右方插入",richTextDeleteRowOrColumnMenu:"刪除",richTextDeleteColumnMenu:"刪除欄",richTextDeleteRowMenu:"刪除列",richTextDeleteTableMenu:"刪除表格",richTextToolbarMoreButtonAriaLabel:"其他選項",attachmentUploadsPendingError:"正在上傳檔案，請稍候。",imageUploadsPendingError:"正在上傳影像，請稍候。",richTextNewBulletedListItemAnnouncement:"項目符號，新行",richTextNewNumberedListItemAnnouncement:"{0}，新行",richTextBulletedListAppliedAnnouncement:"已套用項目符號清單",richTextNumberedListAppliedAnnouncement:"已套用編號清單"};var errorBar={unableToReachChatService:"您處於離線狀態",accessDenied:"無法存取聊天服務 - 請檢查提供的使用者認證",userNotInChatThread:"您已不在此聊天對話中",sendMessageNotInChatThread:"無法傳送訊息，因為您已不在此聊天對話中",sendMessageGeneric:"無法傳送訊息",callingNetworkFailure:"連線通話發生錯誤 - 您似乎已離線",startVideoGeneric:"無法啟動影片",stopVideoGeneric:"無法停止影片",muteGeneric:"無法將麥克風設為靜音",unmuteGeneric:"無法將麥克風取消靜音",speakingWhileMuted:"您的麥克風已設為靜音",startScreenShareGeneric:"啟動畫面分享時發生問題。",stopScreenShareGeneric:"無法停止螢幕畫面分享",callNetworkQualityLow:"您的網路品質不佳。",teamsMeetingCallNetworkQualityLow:"網路品質不佳。",callNoSpeakerFound:"找不到喇叭或耳機。請連接音訊裝置以聆聽通話。",callNoMicrophoneFound:"找不到麥克風。請連接音訊輸入裝置。",callMicrophoneAccessDenied:"無法存取麥克風。按一下網址列中的鎖定以授與此網頁的許可權。",callMicrophoneAccessDeniedSafari:"無法存取麥克風。重新整理頁面以允許許可權，或檢查此瀏覽器的設定，並確認已啟用此網站的許可權。",callMicrophoneMutedBySystem:"系統已將您設為靜音。",callMicrophoneUnmutedBySystem:"您的麥克風已復原，系統已取消您的靜音。",callMacOsMicrophoneAccessDenied:"無法存取麥克風。請在您的 MacOS 隱私權設定中授與麥克風權限。",callLocalVideoFreeze:"網路頻寬不佳。通話中的其他人可能會暫停您的視訊。",callCameraAccessDenied:"無法存取相機。按一下網址列中的鎖定以授與此網頁的許可權。",callCameraAccessDeniedSafari:"無法存取相機。重新整理頁面以允許許可權，或檢查此瀏覽器的設定，並確認已啟用此網站的許可權。",callCameraAlreadyInUse:"無法存取相機。其他應用程式可能已在使用它。",callVideoStoppedBySystem:"您的影片已遭系統停止。",callVideoRecoveredBySystem:"您的影片已繼續。",callMacOsCameraAccessDenied:"MacOS 已封鎖您的相機存取權限。請更新您的隱私權設定，允許此瀏覽器存取您的相機。",callMacOsScreenShareAccessDenied:"MacOS 正在封鎖螢幕畫面分享。更新您的隱私權設定，以允許此瀏覽器錄製您的螢幕。",dismissButtonAriaLabel:"關閉",failedToJoinCallGeneric:"無法加入通話。",failedToJoinCallInvalidMeetingLink:"無法加入會議。連結無效。",cameraFrozenForRemoteParticipants:"通話中的使用者在觀看您的視訊時發生問題。請檢查您的裝置和網路。",unableToStartVideoEffect:"無法套用視訊效果。",startSpotlightWhileMaxParticipantsAreSpotlighted:"無法啟動參與者的焦點，因為參與者人數已達焦點上限。",mutedByRemoteParticipant:"另一位參與者已將您設為靜音。"};var notificationStack={callingNetworkFailure:{title:"接通電話時發生問題",message:"您似乎已離線。請檢查您的網路連線，然後再試一次。",dismissButtonAriaLabel:"關閉"},startVideoGeneric:{title:"無法啟動影片。",dismissButtonAriaLabel:"關閉"},stopVideoGeneric:{title:"無法停止影片。",dismissButtonAriaLabel:"關閉"},muteGeneric:{title:"無法將麥克風設為靜音。",dismissButtonAriaLabel:"關閉"},unmuteGeneric:{title:"無法將麥克風取消靜音。",dismissButtonAriaLabel:"關閉"},speakingWhileMuted:{title:"已將您設為靜音。",dismissButtonAriaLabel:"關閉"},startScreenShareGeneric:{title:"啟動畫面分享時發生問題。",dismissButtonAriaLabel:"關閉"},stopScreenShareGeneric:{title:"無法停止螢幕畫面分享。",dismissButtonAriaLabel:"關閉"},callNetworkQualityLow:{title:"網路品質不佳。",dismissButtonAriaLabel:"關閉"},teamsMeetingCallNetworkQualityLow:{title:"網路品質不佳。",dismissButtonAriaLabel:"關閉",message:"從您的手機加入此通話以獲得更佳的音效。您可以繼續在此裝置上檢視會議。",primaryButtonLabel:"使用電話加入"},callNoSpeakerFound:{title:"找不到喇叭或耳機",message:"連接音訊裝置以聆聽通話。",dismissButtonAriaLabel:"關閉"},callNoMicrophoneFound:{title:"找不到麥克風",message:"請連結音訊輸入裝置。",dismissButtonAriaLabel:"關閉"},callMicrophoneAccessDenied:{title:"無法存取麥克風",message:"按兩下網址列中的鎖定以授與此網頁的許可權。",dismissButtonAriaLabel:"關閉"},callMicrophoneAccessDeniedSafari:{title:"無法存取麥克風",message:"重新整理頁面以允許許可權，或檢查此瀏覽器的設定，並確認已啟用此網站的權限。",dismissButtonAriaLabel:"關閉"},callMicrophoneMutedBySystem:{title:"系統已將您設為靜音。",dismissButtonAriaLabel:"關閉"},callMicrophoneUnmutedBySystem:{title:"您的麥克風已復原，系統已取消您的靜音。",dismissButtonAriaLabel:"關閉"},callMacOsMicrophoneAccessDenied:{title:"無法存取麥克風",message:"在您的macOS隱私權設定中授與麥克風許可權。",dismissButtonAriaLabel:"關閉"},callLocalVideoFreeze:{title:"網路頻寬不佳",message:"通話中的其他人可能會暫停您的視訊。",dismissButtonAriaLabel:"關閉"},callCameraAccessDenied:{title:"無法存取相機",message:"按兩下網址列中的鎖定以授與此網頁的許可權。",dismissButtonAriaLabel:"關閉"},callCameraAccessDeniedSafari:{title:"無法存取相機",message:"重新整理頁面以允許許可權，或檢查此瀏覽器的設定，並確認已啟用此網站的權限。",dismissButtonAriaLabel:"關閉"},callCameraAlreadyInUse:{title:"無法存取相機",message:"其他應用程式可能已在使用它。",dismissButtonAriaLabel:"關閉"},callVideoStoppedBySystem:{title:"您的影片已遭系統停止。",dismissButtonAriaLabel:"關閉"},callVideoRecoveredBySystem:{title:"您的影片已恢復播放。",dismissButtonAriaLabel:"關閉"},callMacOsCameraAccessDenied:{title:"MacOS 禁止存取您的相機",message:"更新您的隱私權設定，以允許此瀏覽器存取您的相機。",dismissButtonAriaLabel:"關閉"},callMacOsScreenShareAccessDenied:{title:"MacOS 正在封鎖螢幕畫面分享",message:"更新您的隱私權設定，以允許此瀏覽器錄製您的螢幕。",dismissButtonAriaLabel:"關閉"},failedToJoinCallGeneric:{title:"無法加入通話。",dismissButtonAriaLabel:"關閉"},failedToJoinCallInvalidMeetingLink:{title:"無法加入會議。連結無效。",dismissButtonAriaLabel:"關閉"},cameraFrozenForRemoteParticipants:{title:"通話中的用戶在觀看您的視訊時發生問題",message:"請檢查您的裝置和網路。",dismissButtonAriaLabel:"關閉"},unableToStartVideoEffect:{title:"無法套用視訊效果。",dismissButtonAriaLabel:"關閉"},startSpotlightWhileMaxParticipantsAreSpotlighted:{title:"無法在參與者上啟動焦點",message:"參與者人數上限已成為焦點。",dismissButtonAriaLabel:"關閉"},mutedByRemoteParticipant:{title:"另一位參與者已將您設為靜音。",dismissButtonAriaLabel:"關閉"},recordingStarted:{title:"錄製已啟動。",message:"一旦加入，即表示您同意轉譯此會議。",dismissButtonAriaLabel:"關閉"},transcriptionStarted:{title:"已開始轉錄。",message:"一旦加入，即表示您同意轉譯此會議。",dismissButtonAriaLabel:"關閉"},recordingStopped:{title:"正在儲存錄製內容。",message:"已停止錄製。",dismissButtonAriaLabel:"關閉"},transcriptionStopped:{title:"正在儲存轉錄。",message:"已停止轉錄。",dismissButtonAriaLabel:"關閉"},recordingAndTranscriptionStarted:{title:"已開始錄製並進行轉錄。",message:"一旦加入，即表示您同意轉譯此會議。",dismissButtonAriaLabel:"關閉"},recordingAndTranscriptionStopped:{title:"正在儲存錄製和轉錄。",message:"已停止錄製與轉錄。",dismissButtonAriaLabel:"關閉"},recordingStoppedStillTranscribing:{title:"已停止錄製。",message:"您現在只會轉錄此會議。",dismissButtonAriaLabel:"關閉"},transcriptionStoppedStillRecording:{title:"已停止轉錄。",message:"您現在只會錄製此會議。",dismissButtonAriaLabel:"關閉"},assignedBreakoutRoomOpened:{title:"我們將在 5 秒內將您移至指派的會議室。",dismissButtonAriaLabel:"關閉"},assignedBreakoutRoomChanged:{title:"您的分組討論區已變更",message:"我們將在 5 秒內將您移至指派的會議室。",dismissButtonAriaLabel:"關閉"},assignedBreakoutRoomClosed:{title:"您的分組討論區已關閉",message:"我們會在 5 秒後將您移回會議。",dismissButtonAriaLabel:"關閉"},assignedBreakoutRoomOpenedPromptJoin:{title:"加入分組討論區?",message:"系統已指派分組討論區給您。",dismissButtonAriaLabel:"關閉",primaryButtonLabel:"加入會議室",secondaryButtonLabel:"稍後"},breakoutRoomJoined:{title:"您已加入分組討論區。",dismissButtonAriaLabel:"關閉"},breakoutRoomClosingSoon:{title:"會議室時間限制即將到期。",message:"此會議室將在 30 秒鐘內關閉。",dismissButtonAriaLabel:"關閉"},capabilityTurnVideoOnPresent:{title:"已啟用您的鏡頭",message:"如果您想要分享視訊，請將它開啟。",dismissButtonAriaLabel:"關閉"},capabilityTurnVideoOnAbsent:{title:"已停用您的相機",message:"您無法再分享視訊。",dismissButtonAriaLabel:"關閉"},capabilityUnmuteMicPresent:{title:"已啟用您的麥克風",message:"如果您想要說話請取消靜音。",dismissButtonAriaLabel:"關閉"},capabilityUnmuteMicAbsent:{title:"您的麥克風已停用",message:"您無法再取消靜音。",dismissButtonAriaLabel:"關閉"},togetherModeStarted:{title:"[在一起模式] 已啟動",dismissButtonAriaLabel:"關閉"},togetherModeEnded:{title:"[在一起模式] 已結束",dismissButtonAriaLabel:"關閉"},transcriptionError:{title:"謄寫發生錯誤。",message:"已停止謄寫。請稍後再試",dismissButtonAriaLabel:"關閉"},transcriptionStartedByYou:{title:"已開始謄寫。",message:"由您啟動。讓所有人知道他們正在被謄寫。",dismissButtonAriaLabel:"關閉"}};var videoGallery={screenIsBeingSharedMessage:"您現在正在分享螢幕畫面。",screenShareLoadingMessage:"正在載入 {participant} 的螢幕畫面",localScreenShareLoadingMessage:"正在載入您的畫面",localVideoLabel:"您",localVideoCameraSwitcherLabel:"切換相機",localVideoMovementLabel:"可移動本機視訊磚",localVideoMovementAriaLabel:"可移動的區域影片標題",localVideoSelectedDescription:"已選取 {cameraName}",displayNamePlaceholder:"未命名的參與者",fitRemoteParticipantToFrame:"調整成畫面大小",fillRemoteParticipantFrame:"填滿畫面",pinParticipantForMe:"為我釘選",pinParticipantForMeLimitReached:"釘選 (已達到限制)",unpinParticipantForMe:"取消釘選",pinParticipantMenuItemAriaLabel:"釘選 {participantName}",unpinParticipantMenuItemAriaLabel:"取消釘選 {participantName}",pinnedParticipantAnnouncementAriaLabel:"已釘選 {participantName}",unpinnedParticipantAnnouncementAriaLabel:"已取消釘選 {participantName}",startSpotlightVideoTileMenuLabel:"設為每個人的焦點",addSpotlightVideoTileMenuLabel:"新增焦點",spotlightLimitReachedMenuTitle:"已達焦點限制",stopSpotlightVideoTileMenuLabel:"停止聚焦",stopSpotlightOnSelfVideoTileMenuLabel:"結束聚焦",attendeeRole:"出席者",muteParticipantMenuItemLabel:"靜音",waitingScreenText:"正在等待其他人加入",forbidAudio:"停用麥克風",permitAudio:"允許麥克風",forbidAudioTileMenuLabel:"停用麥克風",permitAudioTileMenuLabel:"允許麥克風",forbidVideo:"停用相機",permitVideo:"允許相機",forbidVideoTileMenuLabel:"停用相機",permitVideoTileMenuLabel:"允許相機"};var dialpad={placeholderText:"輸入電話號碼",deleteButtonAriaLabel:"刪除"};var holdButton={onLabel:"繼續",offLabel:"保留",tooltipOnContent:"繼續通話",tooltipOffContent:"保留通話"};var videoTile={participantStateRinging:"撥號中...",participantReconnecting:"正在重新連線...",participantStateHold:"電話保留",moreOptionsButtonAriaLabel:"其他選項 {displayName} {isMuted} {micDisabledState} {cameraDisabledState} {isHandRaised} {state} {isSpeaking}",moreOptionsParticipantMutedStateMutedAriaLabel:"已靜音",moreOptionsParticipantMutedStateUnmutedAriaLabel:"已取消靜音",moreOptionsParticipantHandRaisedAriaLabel:"已舉手",moreOptionsParticipantIsSpeakingAriaLabel:"說話",moreOptionsParticipantMicDisabledAriaLabel:"已停用麥克風",moreOptionsParticipantCameraDisabledAriaLabel:"已停用相機"};var CameraAndMicrophoneSitePermissionsRequest={primaryText:"允許 {appName} 使用您的相機和麥克風",secondaryText:"這樣參與者才能看到您並聽到您的聲音。",linkText:"需要協助嗎? 取得疑難排解說明",primaryButtonText:"在沒有相機和麥克風的情況下繼續",ariaLabel:"允許相機和麥克風存取"};var CameraSitePermissionsRequest={primaryText:"允許 {appName} 使用您的相機",secondaryText:"這樣參與者才能看到您。",linkText:"需要協助嗎? 取得疑難排解說明",primaryButtonText:"繼續但不使用相機",ariaLabel:"允許存取相機"};var MicrophoneSitePermissionsRequest={primaryText:"允許 {appName} 使用您的麥克風",secondaryText:"這樣參與者才能聽到您的聲音。",linkText:"需要協助嗎? 取得疑難排解說明",primaryButtonText:"在沒有麥克風的情況下繼續",ariaLabel:"允許麥克風存取"};var CameraAndMicrophoneSitePermissionsCheck={primaryText:"正在檢查相機和麥克風存取",secondaryText:"如果系統提示，則允許存取。這樣參與者才能看到您並聽到您的聲音。",linkText:"需要協助嗎? 取得疑難排解說明",primaryButtonText:"在沒有相機和麥克風的情況下繼續",ariaLabel:"正在檢查相機和麥克風存取。如果系統提示，則允許存取。"};var CameraSitePermissionsCheck={primaryText:"正在檢查相機存取",secondaryText:"如果系統提示，則允許存取。這樣參與者才能看到您。",linkText:"需要協助嗎? 取得疑難排解說明",primaryButtonText:"在沒有相機的情況下繼續",ariaLabel:"正在檢查相機存取。如果系統提示，則允許存取。"};var MicrophoneSitePermissionsCheck={primaryText:"正在檢查麥克風存取",secondaryText:"如果系統提示，則允許存取。這樣參與者才能聽到您的聲音。",linkText:"需要協助嗎? 取得疑難排解說明",primaryButtonText:"在沒有麥克風的情況下繼續",ariaLabel:"正在檢查麥克風存取。如果系統提示，則允許存取。"};var CameraAndMicrophoneSitePermissionsDenied={primaryText:"無法存取攝影機和麥克風",secondaryText:"按一下網址列中的鎖定圖示，以授與此網頁的麥克風許可權。可能需要重新整理頁面。",primaryButtonText:"在沒有相機和麥克風的情況下繼續",linkText:"需要協助嗎? 取得疑難排解說明"};var CameraAndMicrophoneSitePermissionsDeniedSafari={primaryText:"無法存取攝影機和麥克風",secondaryText:"重新整理頁面以允許許可權，或檢查此瀏覽器的設定，並確認已啟用此網站的權限。",primaryButtonText:"在沒有相機和麥克風的情況下繼續",linkText:"需要協助嗎? 取得疑難排解說明"};var CameraSitePermissionsDenied={primaryText:"無法存取相機",secondaryText:"按一下網址列中的鎖定圖示，以授與此網頁的相機許可權。可能需要重新整理頁面。",primaryButtonText:"在沒有相機的情況下繼續",linkText:"需要協助嗎? 取得疑難排解說明"};var MicrophoneSitePermissionsDenied={primaryText:"無法存取麥克風",secondaryText:"按一下網址列中的鎖定圖示，以授與此網頁的麥克風許可權。可能需要重新整理頁面。",primaryButtonText:"在沒有麥克風的情況下繼續",linkText:"需要協助嗎? 取得疑難排解說明"};var CameraSitePermissionsDeniedSafari={primaryText:"無法存取攝影機",secondaryText:"重新整理頁面以允許許可權，或檢查此瀏覽器的設定，並確認已啟用此網站的權限。",primaryButtonText:"在沒有相機的情況下繼續",linkText:"需要協助嗎? 取得疑難排解說明"};var MicrophoneSitePermissionsDeniedSafari={primaryText:"無法存取麥克風",secondaryText:"重新整理頁面以允許許可權，或檢查此瀏覽器的設定，並確認已啟用此網站的權限。",primaryButtonText:"繼續但不使用麥克風",linkText:"需要協助嗎? 取得疑難排解說明"};var UnsupportedBrowser={primaryText:"不支援瀏覽器",secondaryText:"請使用相容的瀏覽器加入此通話。",moreHelpLinkText:"查看相容性需求"};var UnsupportedBrowserVersion={primaryText:"需要更新瀏覽器",secondaryText:"為確保通話品質最佳，請更新瀏覽器，然後再次嘗試加入通話。",moreHelpLinkText:"查看相容性需求",continueAnywayButtonText:"不更新即開始通話"};var UnsupportedOperatingSystem={primaryText:"不支援作業系統",secondaryText:"請使用具有相容作業系統的裝置加入此通話。",moreHelpLinkText:"查看相容性需求"};var BrowserPermissionDenied={primaryText:"無法使用您的相機或麥克風",secondaryText:"您的瀏覽器可能無法存取您的相機或麥克風。若要修正此問題，請開啟 [系統喜好設定]。",primaryButtonText:"重試",linkText:"需要協助嗎? 取得疑難排解說明"};var BrowserPermissionDeniedIOS={primaryText:"允許存取麥克風以繼續",secondaryText:"讓其他參與者可聽到您的聲音。",primaryButtonText:"重試",imageAltText:"iOS 的麥克風和相機裝置權限位置",linkText:"需要協助嗎? 取得疑難排解說明",step1Text:"移至 [設定] 應用程式",step2Text:"向下捲動到此瀏覽器的設定",step3Text:"開啟麥克風 (相機選用)",step4Text:"再次嘗試加入通話",step1DigitText:"1",step2DigitText:"2",step3DigitText:"3",step4DigitText:"4"};var verticalGallery={leftNavButtonAriaLabel:"上一頁 (第 {current} 頁，共 {total} 頁)",rightNavButtonAriaLabel:"下一頁 (第 {current} 頁，共 {total} 頁)"};var meetingConferencePhoneInfo={meetingConferencePhoneInfoModalTitle:"從您的電話加入通話",meetingConferencePhoneInfoModalDialIn:"撥號號碼",meetingConferencePhoneInfoModalMeetingId:"輸入會議識別碼",meetingConferencePhoneInfoModalWait:"等候召集人獲准進入",meetingConferencePhoneInfoModalNoPhoneAvailable:"沒有可用的電話",meetingConferencePhoneInfoModalTollGeoData:"{country}, {city}",meetingConferencePhoneInfoModalTollFree:"(免付費電話)",meetingConferencePhoneInfoModalToll:"(付費)"};var IncomingCallNotification={incomingCallNotificationPlaceholderId:"未知的使用者",incomingCallNotificationPlaceholderAlert:"{callerName} 正在通話",incomingCallNoticicationAcceptWithAudioAriaLabel:"接受音訊",incomingCallNoticicationAcceptWithVideoAriaLabel:"接受視訊",incomingCallNoticicationRejectAriaLabel:"拒絕",incomingCallNotificationAcceptButtonLabel:"接受",incomingCallNotificationRejectButtonLabel:"拒絕",incomingCallNotificationAccceptWithVideoButtonLabel:"接受並開啟視訊",incomingCallNotificationDismissButtonAriaLabel:"關閉"};var zh_TW$1 = {participantItem:participantItem,ParticipantList:ParticipantList,typingIndicator:typingIndicator,sendBox:sendBox,richTextSendBox:richTextSendBox,realTimeTextModal:realTimeTextModal,captionsBanner:captionsBanner,mentionPopover:mentionPopover,imageOverlay:imageOverlay,messageStatusIndicator:messageStatusIndicator,endCallButton:endCallButton,cameraButton:cameraButton,microphoneButton:microphoneButton,devicesButton:devicesButton,participantsButton:participantsButton,screenShareButton:screenShareButton,raiseHandButton:raiseHandButton,reactionButton:reactionButton,realTimeText:realTimeText,captionsSettingsModal:captionsSettingsModal,spokenLanguages:spokenLanguages,captionLanguages:captionLanguages,startCaptionsButton:startCaptionsButton,startRealTimeTextButton:startRealTimeTextButton,messageThread:messageThread,errorBar:errorBar,notificationStack:notificationStack,videoGallery:videoGallery,dialpad:dialpad,holdButton:holdButton,videoTile:videoTile,CameraAndMicrophoneSitePermissionsRequest:CameraAndMicrophoneSitePermissionsRequest,CameraSitePermissionsRequest:CameraSitePermissionsRequest,MicrophoneSitePermissionsRequest:MicrophoneSitePermissionsRequest,CameraAndMicrophoneSitePermissionsCheck:CameraAndMicrophoneSitePermissionsCheck,CameraSitePermissionsCheck:CameraSitePermissionsCheck,MicrophoneSitePermissionsCheck:MicrophoneSitePermissionsCheck,CameraAndMicrophoneSitePermissionsDenied:CameraAndMicrophoneSitePermissionsDenied,CameraAndMicrophoneSitePermissionsDeniedSafari:CameraAndMicrophoneSitePermissionsDeniedSafari,CameraSitePermissionsDenied:CameraSitePermissionsDenied,MicrophoneSitePermissionsDenied:MicrophoneSitePermissionsDenied,CameraSitePermissionsDeniedSafari:CameraSitePermissionsDeniedSafari,MicrophoneSitePermissionsDeniedSafari:MicrophoneSitePermissionsDeniedSafari,UnsupportedBrowser:UnsupportedBrowser,UnsupportedBrowserVersion:UnsupportedBrowserVersion,UnsupportedOperatingSystem:UnsupportedOperatingSystem,BrowserPermissionDenied:BrowserPermissionDenied,BrowserPermissionDeniedIOS:BrowserPermissionDeniedIOS,verticalGallery:verticalGallery,meetingConferencePhoneInfo:meetingConferencePhoneInfo,IncomingCallNotification:IncomingCallNotification};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Chinese (Taiwan).
 *
 * @public
 */
const COMPONENT_LOCALE_ZH_TW = {
    strings: createComponentStrings(zh_TW$1)
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const checkIsSpeaking = (participant) => participant.isSpeaking && !participant.isMuted;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const convertRemoteParticipantToParticipantListParticipant = (userId, displayName, state, isMuted, isScreenSharing, isSpeaking, raisedHand, localUserCanRemoveOthers, reaction, spotlight, mediaAccess) => {
    const identifier = fromFlatCommunicationIdentifier(userId);
    return {
        userId,
        displayName,
        state,
        isMuted,
        isScreenSharing,
        isSpeaking,
        raisedHand,
        // ACS users can not remove Teams users.
        // Removing unknown types of users is undefined.
        isRemovable: (communicationCommon.getIdentifierKind(identifier).kind === 'communicationUser' || communicationCommon.getIdentifierKind(identifier).kind === 'phoneNumber') && localUserCanRemoveOthers,
        reaction,
        spotlight,
        mediaAccess
    };
};
/**
 * @private
 */
const memoizedConvertAllremoteParticipants = memoizeFnAll((userId, displayName, state, isMuted, isScreenSharing, isSpeaking, raisedHand, localUserCanRemoveOthers, reaction, spotlight, mediaAccess) => {
    return convertRemoteParticipantToParticipantListParticipant(userId, displayName, state, isMuted, isScreenSharing, isSpeaking, raisedHand, localUserCanRemoveOthers, reaction, spotlight, mediaAccess);
});
/**
 * @private
 */
const memoizedConvertToVideoTileReaction = memoizeOne((reactionState) => {
    return reactionState && reactionState.reactionMessage ? {
        reactionType: reactionState.reactionMessage.reactionType,
        receivedOn: reactionState.receivedOn
    } : undefined;
});
/**
 * @private
 */
const memoizedSpotlight = memoizeOne((spotlightedParticipants, userId) => {
    const spotlightOrder = spotlightedParticipants === null || spotlightedParticipants === void 0 ? void 0 : spotlightedParticipants.find(spotlightedParticipant => toFlatCommunicationIdentifier(spotlightedParticipant.identifier) === userId);
    return spotlightOrder ? {
        spotlightedOrderPosition: spotlightOrder.order
    } : undefined;
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @internal */
const _dominantSpeakersWithFlatId = (dominantSpeakers) => {
    var _a;
    return (_a = dominantSpeakers === null || dominantSpeakers === void 0 ? void 0 : dominantSpeakers.speakersList) === null || _a === void 0 ? void 0 : _a.map(toFlatCommunicationIdentifier);
};
/** @internal */
const _videoGalleryRemoteParticipantsMemo = (remoteParticipants, isHideAttendeeNamesEnabled, localUserRole) => {
    if (!remoteParticipants) {
        return [];
    }
    return memoizedAllConvertRemoteParticipant(memoizedFn => {
        return Object.values(remoteParticipants)
            /**
             * hiding participants who are inLobby, idle, or connecting in ACS clients till we can admit users through ACS clients.
             * phone users will be in the connecting state until they are connected to the call.
             */ .filter((participant) => {
            return !['InLobby', 'Idle', 'Connecting', 'Disconnected'].includes(participant.state) || communicationCommon.isPhoneNumberIdentifier(participant.identifier);
        }).map((participant) => {
            const state = _convertParticipantState(participant);
            const displayName = maskDisplayNameWithRole(participant.displayName, localUserRole, participant.role, isHideAttendeeNamesEnabled);
            const remoteParticipantReaction = memoizedConvertToVideoTileReaction(participant.reactionState);
            const spotlight = participant.spotlight;
            return memoizedFn(toFlatCommunicationIdentifier(participant.identifier), participant.isMuted, checkIsSpeaking(participant), participant.videoStreams, state, displayName, participant.raisedHand, participant.contentSharingStream, remoteParticipantReaction, spotlight, participant.mediaAccess, participant.role);
        });
    });
};
const memoizedAllConvertRemoteParticipant = memoizeFnAll((userId, isMuted, isSpeaking, videoStreams, state, displayName, raisedHand, contentSharingStream, reaction, spotlight, mediaAccess, role, signalStrength) => {
    return convertRemoteParticipantToVideoGalleryRemoteParticipant(userId, isMuted, isSpeaking, videoStreams, state, displayName, raisedHand, contentSharingStream, reaction, spotlight, signalStrength, mediaAccess, role);
});
/** @private */
const convertRemoteParticipantToVideoGalleryRemoteParticipant = (userId, isMuted, isSpeaking, videoStreams, state, displayName, raisedHand, contentSharingStream, reaction, spotlight, signalStrength, mediaAccess, role) => {
    const rawVideoStreamsArray = Object.values(videoStreams);
    let videoStream = undefined;
    let screenShareStream = undefined;
    /**
     * There is a bug from the calling sdk where if a user leaves and rejoins immediately
     * it adds 2 more potential streams this remote participant can use. The old 2 streams
     * still show as available and that is how we got a frozen stream in this case. The stopgap
     * until streams accurately reflect their availability is to always prioritize the latest streams of a certain type
     * e.g findLast instead of find
     */
    const sdkRemoteVideoStream = Object.values(rawVideoStreamsArray).findLast(i => i.mediaStreamType === 'Video' && i.isAvailable) || Object.values(rawVideoStreamsArray).findLast(i => i.mediaStreamType === 'Video');
    const sdkScreenShareStream = Object.values(rawVideoStreamsArray).findLast(i => i.mediaStreamType === 'ScreenSharing' && i.isAvailable) || Object.values(rawVideoStreamsArray).findLast(i => i.mediaStreamType === 'ScreenSharing');
    if (sdkRemoteVideoStream) {
        videoStream = convertRemoteVideoStreamToVideoGalleryStream(sdkRemoteVideoStream);
    }
    if (sdkScreenShareStream) {
        screenShareStream = convertRemoteVideoStreamToVideoGalleryStream(sdkScreenShareStream);
    }
    if (contentSharingStream) {
        screenShareStream = convertRemoteContentSharingStreamToVideoGalleryStream(contentSharingStream);
    }
    return {
        userId,
        displayName,
        isMuted,
        isSpeaking,
        videoStream,
        screenShareStream,
        isScreenSharingOn: screenShareStream !== undefined && screenShareStream.isAvailable,
        state,
        raisedHand,
        reaction,
        spotlight,
        mediaAccess,
        canAudioBeForbidden: role === 'Attendee',
        canVideoBeForbidden: role === 'Attendee'
    };
};
const convertRemoteVideoStreamToVideoGalleryStream = (stream) => {
    var _a, _b, _c;
    return {
        id: stream.id,
        isAvailable: stream.isAvailable,
        isReceiving: stream.isReceiving,
        isMirrored: (_a = stream.view) === null || _a === void 0 ? void 0 : _a.isMirrored,
        renderElement: (_b = stream.view) === null || _b === void 0 ? void 0 : _b.target,
        scalingMode: (_c = stream.view) === null || _c === void 0 ? void 0 : _c.scalingMode,
        streamSize: stream.streamSize
    };
};
const convertRemoteContentSharingStreamToVideoGalleryStream = (stream) => {
    return {
        isAvailable: !!stream,
        isReceiving: true,
        isMirrored: false,
        renderElement: stream
    };
};
/** @private */
const memoizeLocalParticipant = memoizeOne((identifier, displayName, isMuted, isScreenSharingOn, localVideoStream, localScreenSharingStream, role, raisedHand, reaction, localSpotlight, capabilities) => {
    var _a, _b, _c;
    return ({
        userId: identifier,
        displayName: displayName !== null && displayName !== void 0 ? displayName : '',
        isMuted: isMuted,
        isScreenSharingOn: isScreenSharingOn,
        videoStream: {
            isAvailable: !!localVideoStream,
            isMirrored: (_a = localVideoStream === null || localVideoStream === void 0 ? void 0 : localVideoStream.view) === null || _a === void 0 ? void 0 : _a.isMirrored,
            renderElement: (_b = localVideoStream === null || localVideoStream === void 0 ? void 0 : localVideoStream.view) === null || _b === void 0 ? void 0 : _b.target
        },
        screenShareStream: {
            isAvailable: !!localScreenSharingStream,
            renderElement: (_c = localScreenSharingStream === null || localScreenSharingStream === void 0 ? void 0 : localScreenSharingStream.view) === null || _c === void 0 ? void 0 : _c.target
        },
        role,
        raisedHand: raisedHand,
        reaction,
        spotlight: localSpotlight,
        capabilities,
        mediaAccess: {
            isAudioPermitted: (capabilities === null || capabilities === void 0 ? void 0 : capabilities.unmuteMic) ? capabilities.unmuteMic.isPresent : true,
            isVideoPermitted: (capabilities === null || capabilities === void 0 ? void 0 : capabilities.turnVideoOn) ? capabilities.turnVideoOn.isPresent : true
        }
    });
});
/** @private */
const memoizeSpotlightedParticipantIds = memoizeOne(spotlightedParticipants => spotlightedParticipants === null || spotlightedParticipants === void 0 ? void 0 : spotlightedParticipants.map((p) => toFlatCommunicationIdentifier(p.identifier)));
/** @private */
const memoizeTogetherModeStreams = memoizeOne(togetherModeStreams => {
    var _a, _b, _c, _d, _e, _f;
    return ({
        mainVideoStream: {
            id: (_a = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _a === void 0 ? void 0 : _a.id,
            isReceiving: (_b = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _b === void 0 ? void 0 : _b.isReceiving,
            isAvailable: (_c = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _c === void 0 ? void 0 : _c.isAvailable,
            renderElement: (_e = (_d = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _d === void 0 ? void 0 : _d.view) === null || _e === void 0 ? void 0 : _e.target,
            streamSize: (_f = togetherModeStreams === null || togetherModeStreams === void 0 ? void 0 : togetherModeStreams.mainVideoStream) === null || _f === void 0 ? void 0 : _f.streamSize
        }
    });
});
/** @private */
const memoizeTogetherModeSeatingPositions = memoizeOne(togetherModeSeatingCoordinates => togetherModeSeatingCoordinates);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const getRemoteParticipantsExcludingConsumers = reselect.createSelector([getRemoteParticipants$1], (remoteParticipants) => {
    {
        const newRemoteParticipants = Object.assign({}, remoteParticipants);
        Object.keys(newRemoteParticipants).forEach(k => {
            var _a;
            if (((_a = newRemoteParticipants[k]) === null || _a === void 0 ? void 0 : _a.role) === 'Consumer') {
                delete newRemoteParticipants[k];
            }
        });
        return newRemoteParticipants;
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Provides data attributes to {@link VideoGallery} component.
 * @public
 */
const videoGallerySelector = reselect.createSelector([getScreenShareRemoteParticipant, getRemoteParticipantsExcludingConsumers, getLocalVideoStreams$1, getIsMuted, getIsScreenSharingOn, getDisplayName$2, getIdentifier, getDominantSpeakers, getOptimalVideoCount, getRole$1, getLocalParticipantRaisedHand$1, isHideAttendeeNamesEnabled, getLocalParticipantReactionState, getSpotlightCallFeature, getCapabilities, getTogetherModeCallFeature], (screenShareRemoteParticipantId, remoteParticipants, localVideoStreams, isMuted, isScreenSharingOn, displayName, identifier, dominantSpeakers, optimalVideoCount, role, raisedHand, isHideAttendeeNamesEnabled, localParticipantReaction, spotlightCallFeature, capabilities, togetherModeCallFeature) => {
    const screenShareRemoteParticipant = screenShareRemoteParticipantId && remoteParticipants ? remoteParticipants[screenShareRemoteParticipantId] : undefined;
    const localVideoStream = localVideoStreams === null || localVideoStreams === void 0 ? void 0 : localVideoStreams.find(i => i.mediaStreamType === 'Video');
    const localScreenSharingStream = localVideoStreams === null || localVideoStreams === void 0 ? void 0 : localVideoStreams.find(i => i.mediaStreamType === 'ScreenSharing');
    const dominantSpeakerIds = _dominantSpeakersWithFlatId(dominantSpeakers);
    dominantSpeakerIds === null || dominantSpeakerIds === void 0 ? void 0 : dominantSpeakerIds.forEach((speaker, idx) => idx);
    const noRemoteParticipants = [];
    const localParticipantReactionState = memoizedConvertToVideoTileReaction(localParticipantReaction);
    const spotlightedParticipantIds = memoizeSpotlightedParticipantIds(spotlightCallFeature === null || spotlightCallFeature === void 0 ? void 0 : spotlightCallFeature.spotlightedParticipants);
    return {
        screenShareParticipant: screenShareRemoteParticipant ? convertRemoteParticipantToVideoGalleryRemoteParticipant(toFlatCommunicationIdentifier(screenShareRemoteParticipant.identifier), screenShareRemoteParticipant.isMuted, checkIsSpeaking(screenShareRemoteParticipant), screenShareRemoteParticipant.videoStreams, screenShareRemoteParticipant.state, screenShareRemoteParticipant.displayName, screenShareRemoteParticipant.raisedHand, screenShareRemoteParticipant.contentSharingStream, undefined, screenShareRemoteParticipant.spotlight, undefined, screenShareRemoteParticipant.mediaAccess, role) : undefined,
        localParticipant: memoizeLocalParticipant(identifier, displayName, isMuted, isScreenSharingOn, localVideoStream, localScreenSharingStream, role, raisedHand, localParticipantReactionState, spotlightCallFeature === null || spotlightCallFeature === void 0 ? void 0 : spotlightCallFeature.localParticipantSpotlight, capabilities),
        remoteParticipants: _videoGalleryRemoteParticipantsMemo(_updateUserDisplayNames(remoteParticipants ? Object.values(remoteParticipants) : noRemoteParticipants), isHideAttendeeNamesEnabled, role),
        dominantSpeakers: dominantSpeakerIds,
        maxRemoteVideoStreams: optimalVideoCount,
        spotlightedParticipants: spotlightedParticipantIds,
        maxParticipantsToSpotlight: spotlightCallFeature === null || spotlightCallFeature === void 0 ? void 0 : spotlightCallFeature.maxParticipantsToSpotlight,
        togetherModeStreams: memoizeTogetherModeStreams(togetherModeCallFeature === null || togetherModeCallFeature === void 0 ? void 0 : togetherModeCallFeature.streams),
        togetherModeSeatingCoordinates: memoizeTogetherModeSeatingPositions(togetherModeCallFeature === null || togetherModeCallFeature === void 0 ? void 0 : togetherModeCallFeature.seatingPositions),
        isTogetherModeActive: togetherModeCallFeature === null || togetherModeCallFeature === void 0 ? void 0 : togetherModeCallFeature.isActive,
        startTogetherModeEnabled: capabilities === null || capabilities === void 0 ? void 0 : capabilities.startTogetherMode.isPresent
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const convertRemoteParticipantsToParticipantListParticipants = (remoteParticipants, localUserCanRemoveOthers, isHideAttendeeNamesEnabled, localUserRole, spotlightedParticipants) => {
    const conversionCallback = (memoizeFn) => {
        return remoteParticipants
            // Filter out MicrosoftBot participants
            .filter((participant) => {
            return !communicationCommon.isMicrosoftTeamsAppIdentifier(participant.identifier);
        })
            /**
             * hiding participants who are inLobby, idle, or connecting in ACS clients till we can admit users through ACS clients.
             * phone users will be in the connecting state until they are connected to the call.
             */ .filter(participant => {
            return !['InLobby', 'Idle', 'Connecting', 'Disconnected'].includes(participant.state) || communicationCommon.isPhoneNumberIdentifier(participant.identifier);
        }).map((participant) => {
            const isScreenSharing = Object.values(participant.videoStreams).some(videoStream => videoStream.mediaStreamType === 'ScreenSharing' && videoStream.isAvailable);
            /**
             * We want to check the participant to see if they are a PSTN participant joining the call
             * and mapping their state to be 'Ringing'
             */
            const state = _convertParticipantState(participant);
            const displayName = maskDisplayNameWithRole(participant.displayName, localUserRole, participant.role, isHideAttendeeNamesEnabled);
            const remoteParticipantReaction = memoizedConvertToVideoTileReaction(participant.reactionState);
            const spotlight = memoizedSpotlight(spotlightedParticipants, toFlatCommunicationIdentifier(participant.identifier));
            return memoizeFn(toFlatCommunicationIdentifier(participant.identifier), displayName, state, participant.isMuted, isScreenSharing, participant.isSpeaking, participant.raisedHand, localUserCanRemoveOthers, remoteParticipantReaction, spotlight, participant.mediaAccess);
        }).sort((a, b) => {
            var _a, _b;
            const nameA = ((_a = a.displayName) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || '';
            const nameB = ((_b = b.displayName) === null || _b === void 0 ? void 0 : _b.toLowerCase()) || '';
            if (nameA < nameB) {
                return -1;
            }
            else if (nameA > nameB) {
                return 1;
            }
            else {
                return 0;
            }
        });
    };
    return memoizedConvertAllremoteParticipants(conversionCallback);
};
/**
 * Selects data that drives {@link ParticipantList} component.
 *
 * @public
 */
const participantListSelector = reselect.createSelector([getIdentifier, getDisplayName$2, getRemoteParticipantsExcludingConsumers, getIsScreenSharingOn, getIsMuted, getLocalParticipantRaisedHand$1, getRole$1, getParticipantCount, isHideAttendeeNamesEnabled, getLocalParticipantReactionState, getSpotlightCallFeature, getCapabilities], (userId, displayName, remoteParticipants, isScreenSharingOn, isMuted, raisedHand, role, partitipantCount, isHideAttendeeNamesEnabled, localParticipantReactionState, spotlightCallFeature, capabilities) => {
    const localUserCanRemoveOthers = localUserCanRemoveOthersTrampoline(role);
    const participants = remoteParticipants ? convertRemoteParticipantsToParticipantListParticipants(_updateUserDisplayNames(Object.values(remoteParticipants)), localUserCanRemoveOthers, isHideAttendeeNamesEnabled, role, spotlightCallFeature === null || spotlightCallFeature === void 0 ? void 0 : spotlightCallFeature.spotlightedParticipants) : [];
    participants.push({
        userId: userId,
        displayName: displayName,
        isScreenSharing: isScreenSharingOn,
        isMuted: isMuted,
        raisedHand: raisedHand,
        state: 'Connected',
        // Local participant can never remove themselves.
        isRemovable: false,
        reaction: memoizedConvertToVideoTileReaction(localParticipantReactionState),
        spotlight: memoizedSpotlight(spotlightCallFeature === null || spotlightCallFeature === void 0 ? void 0 : spotlightCallFeature.spotlightedParticipants, userId),
        mediaAccess: {
            isAudioPermitted: (capabilities === null || capabilities === void 0 ? void 0 : capabilities.unmuteMic) ? capabilities.unmuteMic.isPresent : true,
            isVideoPermitted: (capabilities === null || capabilities === void 0 ? void 0 : capabilities.turnVideoOn) ? capabilities.turnVideoOn.isPresent : true
        }
    });
    return {
        participants: participants,
        myUserId: userId
    };
});
const localUserCanRemoveOthersTrampoline = (role) => {
    return role === 'Presenter' || role === 'Unknown' || role === undefined;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Selects data that drives {@link ParticipantsButton} component.
 *
 * @public
 */
const participantsButtonSelector = reselect.createSelector([participantListSelector], (participantListProps) => {
    return participantListProps;
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Create a set of default handlers for given component. Memoization is applied to the result. Multiple invocations with
 * the same arguments will return the same handler instances. DeclarativeCallAgent, DeclarativeDeviceManager, and
 * DeclarativeCall may be undefined. If undefined, their associated handlers will not be created and returned.
 *
 * @param callClient - StatefulCallClient returned from
 *   {@link @azure/communication-react#createStatefulCallClient}.
 * @param callAgent - Instance of {@link @azure/communication-calling#CallClient}.
 * @param deviceManager - Instance of {@link @azure/communication-calling#DeviceManager}.
 * @param call - Instance of {@link @azure/communication-calling#Call}.
 * @param _ - React component that you want to generate handlers for.
 *
 * @public
 */
const createDefaultCallingHandlersForComponent = (callClient, callAgent, deviceManager, call, _Component) => {
    if (!callAgent && !call && !deviceManager) {
        return createDefaultCallingHandlers(callClient, callAgent, deviceManager, call);
    }
    if (callAgent && _isTeamsCallAgent(callAgent) && (!call || call && _isTeamsCall(call))) {
        return createDefaultTeamsCallingHandlers(callClient, callAgent, deviceManager, call);
    }
    if (callAgent && _isACSCallAgent(callAgent) && (!call || call && _isACSCall(call))) {
        return createDefaultCallingHandlers(callClient, callAgent, deviceManager, call);
    }
    throw new Error('CallAgent type and Call type are not compatible!');
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Hook to obtain a handler for a specified component.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
const useHandlers$3 = (component) => {
    var _a, _b, _c;
    const callClient = (_a = React.useContext(CallClientContext)) === null || _a === void 0 ? void 0 : _a.callClient;
    const deviceManager = useDeviceManager();
    const call = (_b = React.useContext(CallContext$1)) === null || _b === void 0 ? void 0 : _b.call;
    const callAgent = (_c = React.useContext(CallAgentContext)) === null || _c === void 0 ? void 0 : _c.callAgent;
    if (!callClient) {
        return undefined;
    }
    // Handle edge case, validate if call and callAgent are the same type (ACS/Teams)
    if (callAgent && _isTeamsCallAgent(callAgent)) {
        if (call && !_isTeamsCall(call)) {
            throw new Error('A TeamsCall must be provided when callAgent is TeamsCallAgent');
        }
    }
    if (callAgent && _isACSCallAgent(callAgent)) {
        if (call && !_isACSCall(call)) {
            throw new Error('A regular ACS Call must be provided when callAgent is regular CallAgent');
        }
    }
    return createDefaultCallingHandlersForComponent(callClient, callAgent, deviceManager, call);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Hook to obtain a selector for a specified component.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useSelector$4 = (selector, selectorProps) => {
    var _a, _b, _c;
    const callClient = (_a = React.useContext(CallClientContext)) === null || _a === void 0 ? void 0 : _a.callClient;
    const callId = (_c = (_b = React.useContext(CallContext$1)) === null || _b === void 0 ? void 0 : _b.call) === null || _c === void 0 ? void 0 : _c.id;
    // Keeps track of whether the current component is mounted or not. If it has unmounted, make sure we do not modify the
    // state or it will cause React warnings in the console. https://skype.visualstudio.com/SPOOL/_workitems/edit/2453212
    const mounted = React.useRef(false);
    React.useEffect(() => {
        mounted.current = true;
        return () => {
            mounted.current = false;
        };
    });
    const callIdConfigProps = React.useMemo(() => ({
        callId
    }), [callId]);
    const [props, setProps] = React.useState(callClient && selector ? selector(callClient.getState(), selectorProps !== null && selectorProps !== void 0 ? selectorProps : callIdConfigProps) : undefined);
    const propRef = React.useRef(props);
    propRef.current = props;
    React.useEffect(() => {
        if (!callClient || !selector) {
            return;
        }
        const onStateChange = (state) => {
            if (!mounted.current) {
                return;
            }
            const newProps = selector(state, selectorProps !== null && selectorProps !== void 0 ? selectorProps : callIdConfigProps);
            if (propRef.current !== newProps) {
                setProps(newProps);
            }
        };
        callClient.onStateChange(onStateChange);
        return () => {
            callClient.offStateChange(onStateChange);
        };
    }, [callClient, selector, selectorProps, callIdConfigProps, mounted]);
    return selector ? props : undefined;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Select the first 3 active errors from the state for the `ErrorBar` component.
 *
 * In case there are many errors, only the first three errors are returned to avoid
 * filling up the UI with too many errors.
 *
 * Invariants:
 *   - `ErrorType` is never repeated in the returned errors.
 *   - Errors are returned in a fixed order by `ErrorType`.
 *
 * @public
 */
const errorBarSelector$1 = reselect.createSelector([getLatestErrors$1, getDiagnostics, getDeviceManager$1, getEnvironmentInfo$1], (latestErrors, diagnostics, deviceManager, environmentInfo) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t;
    // The order in which the errors are returned is significant: The `ErrorBar` shows errors on the UI in that order.
    // There are several options for the ordering:
    //   - Sorted by when the errors happened (latest first / oldest first).
    //   - Stable sort by error type.
    //
    // We chose to stable sort by error type: We intend to show only a small number of errors on the UI and we do not
    // have timestamps for errors.
    const activeErrorMessages = [];
    const isSafari = () => {
        return (environmentInfo === null || environmentInfo === void 0 ? void 0 : environmentInfo.environment.browser) === 'safari';
    };
    const isMacOS = () => {
        return (environmentInfo === null || environmentInfo === void 0 ? void 0 : environmentInfo.environment.platform) === 'mac';
    };
    if (((_a = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.network.latest.networkReceiveQuality) === null || _a === void 0 ? void 0 : _a.value) === communicationCalling.DiagnosticQuality.Bad || ((_b = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.network.latest.networkReceiveQuality) === null || _b === void 0 ? void 0 : _b.value) === communicationCalling.DiagnosticQuality.Poor) {
        activeErrorMessages.push({
            type: 'callNetworkQualityLow'
        });
    }
    if (((_c = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.noSpeakerDevicesEnumerated) === null || _c === void 0 ? void 0 : _c.value) === true) {
        activeErrorMessages.push({
            type: 'callNoSpeakerFound'
        });
    }
    if (((_d = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.noMicrophoneDevicesEnumerated) === null || _d === void 0 ? void 0 : _d.value) === true) {
        activeErrorMessages.push({
            type: 'callNoMicrophoneFound'
        });
    }
    if (((_e = deviceManager.deviceAccess) === null || _e === void 0 ? void 0 : _e.audio) === false && isSafari()) {
        activeErrorMessages.push({
            type: 'callMicrophoneAccessDeniedSafari'
        });
    }
    if (((_f = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphonePermissionDenied) === null || _f === void 0 ? void 0 : _f.value) === true && isMacOS()) {
        activeErrorMessages.push({
            type: 'callMacOsMicrophoneAccessDenied'
        });
    }
    else if (((_g = deviceManager.deviceAccess) === null || _g === void 0 ? void 0 : _g.audio) === false && !isSafari() || ((_h = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphonePermissionDenied) === null || _h === void 0 ? void 0 : _h.value) === true && !isMacOS()) {
        activeErrorMessages.push({
            type: 'callMicrophoneAccessDenied'
        });
    }
    const microphoneMuteUnexpectedlyDiagnostic = (diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphoneMuteUnexpectedly) || (diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.microphoneNotFunctioning);
    if (microphoneMuteUnexpectedlyDiagnostic) {
        if (microphoneMuteUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Bad) {
            // Inform the user that microphone stopped working and inform them to start microphone again
            activeErrorMessages.push({
                type: 'callMicrophoneMutedBySystem'
            });
        }
        else if (microphoneMuteUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Good) {
            // Inform the user that microphone recovered
            activeErrorMessages.push({
                type: 'callMicrophoneUnmutedBySystem'
            });
        }
    }
    const cameraStoppedUnexpectedlyDiagnostic = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.cameraStoppedUnexpectedly;
    if (cameraStoppedUnexpectedlyDiagnostic) {
        if (cameraStoppedUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Bad) {
            // Inform the user that camera stopped working and inform them to start video again
            activeErrorMessages.push({
                type: 'callVideoStoppedBySystem'
            });
        }
        else if (cameraStoppedUnexpectedlyDiagnostic.value === communicationCalling.DiagnosticQuality.Good) {
            // Inform the user that camera recovered
            activeErrorMessages.push({
                type: 'callVideoRecoveredBySystem'
            });
        }
    }
    if (((_j = deviceManager.deviceAccess) === null || _j === void 0 ? void 0 : _j.video) === false && isSafari()) {
        activeErrorMessages.push({
            type: 'callCameraAccessDeniedSafari'
        });
    }
    else if (((_k = deviceManager.deviceAccess) === null || _k === void 0 ? void 0 : _k.video) === false) {
        activeErrorMessages.push({
            type: 'callCameraAccessDenied'
        });
    }
    else {
        if (((_l = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.cameraFreeze) === null || _l === void 0 ? void 0 : _l.value) === true) {
            activeErrorMessages.push({
                type: 'cameraFrozenForRemoteParticipants'
            });
        }
    }
    /**
     * show the Mac specific strings if the platform is detected as mac
     */
    if (((_m = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.cameraPermissionDenied) === null || _m === void 0 ? void 0 : _m.value) === true && isMacOS()) {
        activeErrorMessages.push({
            type: 'callMacOsCameraAccessDenied'
        });
    }
    /**
     * This UFD only works on mac still so we should only see it fire on mac.
     */
    if (((_o = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.screenshareRecordingDisabled) === null || _o === void 0 ? void 0 : _o.value) === true && isMacOS()) {
        activeErrorMessages.push({
            type: 'callMacOsScreenShareAccessDenied'
        });
    }
    else if (((_p = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.screenshareRecordingDisabled) === null || _p === void 0 ? void 0 : _p.value) === true) {
        activeErrorMessages.push({
            type: 'startScreenShareGeneric'
        });
    }
    // Prefer to show errors with privacy implications.
    appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.stopVideo', 'stopVideoGeneric');
    appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.mute', 'muteGeneric');
    appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.stopScreenSharing', 'stopScreenShareGeneric');
    if (((_q = latestErrors['Call.startVideo']) === null || _q === void 0 ? void 0 : _q.message) === 'Call.startVideo: Video operation failure SourceUnavailableError') {
        appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.startVideo', 'callCameraAlreadyInUse');
    }
    else if (((_r = latestErrors['Call.startVideo']) === null || _r === void 0 ? void 0 : _r.message) === 'Call.startVideo: Video operation failure permissionDeniedError') {
        appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.startVideo', 'callCameraAccessDenied');
    }
    else {
        appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.startVideo', 'startVideoGeneric');
    }
    appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.unmute', 'unmuteGeneric');
    appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.mutedByOthers', 'mutedByRemoteParticipant');
    appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'VideoEffectsFeature.startEffects', 'unableToStartVideoEffect');
    if (((_s = latestErrors['CallAgent.join']) === null || _s === void 0 ? void 0 : _s.message) === 'CallAgent.join: Invalid meeting link') {
        appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'CallAgent.join', 'failedToJoinCallInvalidMeetingLink');
    }
    else {
        appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'CallAgent.join', 'failedToJoinCallGeneric');
    }
    if ((_t = latestErrors['Call.feature']) === null || _t === void 0 ? void 0 : _t.message.match(/Call\.feature: startSpotlight failed\. \d+ is the max number of participants that can be Spotlighted/g)) {
        appendActiveErrorIfDefined(activeErrorMessages, latestErrors, 'Call.feature', 'startSpotlightWhileMaxParticipantsAreSpotlighted');
    }
    // We only return the first few errors to avoid filling up the UI with too many `MessageBar`s.
    activeErrorMessages.splice(maxErrorCount$1);
    return {
        activeErrorMessages: activeErrorMessages
    };
});
const appendActiveErrorIfDefined = (activeErrorMessages, latestErrors, target, activeErrorType) => {
    if (latestErrors[target] === undefined) {
        return;
    }
    activeErrorMessages.push({
        type: activeErrorType,
        timestamp: latestErrors[target].timestamp
    });
};
const maxErrorCount$1 = 3;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const emptySelector = () => ({});
/**
 * Get the selector for a specified component.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const getSelector$1 = (component) => {
    return findSelector$1(component);
};
const findSelector$1 = (component) => {
    // Dialpad only has handlers currently and doesn't require any props from the stateful layer so return the emptySelector
    if (component === Dialpad) {
        return emptySelector;
    }
    switch (component) {
        case VideoGallery:
            return videoGallerySelector;
        case MicrophoneButton:
            return microphoneButtonSelector;
        case CameraButton:
            return cameraButtonSelector;
        case ScreenShareButton:
            return screenShareButtonSelector;
        case DevicesButton:
            return devicesButtonSelector;
        case ParticipantList$n:
            return participantListSelector;
        case ParticipantsButton:
            return participantsButtonSelector;
        case EndCallButton:
            return emptySelector;
        case ErrorBar:
            return errorBarSelector$1;
        case RaiseHandButton:
            return raiseHandButtonSelector;
        case ReactionButton:
            return reactionButtonSelector;
        case NotificationStack:
            return notificationStackSelector;
        case HoldButton:
            return holdButtonSelector;
        case IncomingCallStack:
            return incomingCallStackSelector;
        case CaptionsBanner:
            return captionsBannerSelector;
        case CaptionsSettingsModal:
            return captionSettingsSelector;
        case StartCaptionsButton:
            return startCaptionsButtonSelector;
    }
    return undefined;
};
/**
 * Selector for new components that are conditionally compiled. Comment out when there is no CC'd components
 */
// const findConditionalCompiledSelector = (component: (props: any) => JSX.Element | undefined): any => {
//   switch (component) {
//   }
// };

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$B = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Create the default implementation of {@link ChatHandlers}.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * Returned object is memoized to avoid rerenders when used as props for React Components.
 *
 * @public
 */
const createDefaultChatHandlers = memoizeOne((chatClient, chatThreadClient) => {
    let messageIterator = undefined;
    let readReceiptIterator = undefined;
    return {
        // due to a bug in babel, we can't use arrow function here
        // affecting conditional-compile-remove(attachment-upload)
        onSendMessage: function (content, options) {
            return __awaiter$B(this, void 0, void 0, function* () {
                const sendMessageRequest = {
                    content
                };
                yield chatThreadClient.sendMessage(sendMessageRequest, Object.assign(Object.assign({}, options), { senderDisplayName: chatClient.getState().displayName }));
            });
        },
        // due to a bug in babel, we can't use arrow function here
        // affecting conditional-compile-remove(attachment-upload)
        onUpdateMessage: function (messageId, content) {
            return __awaiter$B(this, void 0, void 0, function* () {
                const updateMessageOptions = {
                    content
                };
                yield chatThreadClient.updateMessage(messageId, updateMessageOptions);
            });
        },
        onDeleteMessage: (messageId) => __awaiter$B(void 0, void 0, void 0, function* () {
            yield chatThreadClient.deleteMessage(messageId);
        }),
        // This handler is designed for chatThread to consume
        onMessageSeen: (chatMessageId) => __awaiter$B(void 0, void 0, void 0, function* () {
            yield chatThreadClient.sendReadReceipt({
                chatMessageId
            });
        }),
        onTyping: () => __awaiter$B(void 0, void 0, void 0, function* () {
            yield chatThreadClient.sendTypingNotification();
        }),
        onRemoveParticipant: (userId) => __awaiter$B(void 0, void 0, void 0, function* () {
            yield chatThreadClient.removeParticipant(fromFlatCommunicationIdentifier(userId));
        }),
        updateThreadTopicName: (topicName) => __awaiter$B(void 0, void 0, void 0, function* () {
            yield chatThreadClient.updateTopic(topicName);
        }),
        onLoadPreviousChatMessages: (messagesToLoad) => __awaiter$B(void 0, void 0, void 0, function* () {
            var _a, _b, _c;
            if (messageIterator === undefined) {
                // Lazy definition so that errors in the method call are reported correctly.
                // Also allows recovery via retries in case of transient errors.
                messageIterator = chatThreadClient.listMessages({
                    maxPageSize: 50
                });
            }
            if (readReceiptIterator === undefined) {
                readReceiptIterator = chatThreadClient.listReadReceipts();
            }
            // get the earliest message time
            let remainingMessagesToGet = messagesToLoad;
            let isAllChatMessagesLoaded = false;
            let earliestTime = Number.MAX_SAFE_INTEGER;
            while (remainingMessagesToGet >= 1) {
                const message = yield messageIterator.next();
                if ((_a = message === null || message === void 0 ? void 0 : message.value) === null || _a === void 0 ? void 0 : _a.id) {
                    if (parseInt(message.value.id) < earliestTime) {
                        earliestTime = parseInt(message.value.id);
                    }
                }
                if (((_b = message.value) === null || _b === void 0 ? void 0 : _b.type) && message.value.type === 'text') {
                    remainingMessagesToGet--;
                }
                // We have traversed all messages in this thread
                if (message.done) {
                    isAllChatMessagesLoaded = true;
                    break;
                }
            }
            // keep fetching read receipts until read receipt time < earlist message time
            let readReceipt = yield readReceiptIterator.next();
            while (!readReceipt.done && parseInt((_c = readReceipt === null || readReceipt === void 0 ? void 0 : readReceipt.value) === null || _c === void 0 ? void 0 : _c.chatMessageId) >= earliestTime) {
                readReceipt = yield readReceiptIterator.next();
            }
            return isAllChatMessagesLoaded;
        })
    };
});
/**
 * Create a set of default handlers for given component.
 *
 * Returned object is memoized (with reference to the arguments) to avoid
 * renders when used as props for React Components.
 *
 * @public
 */
const createDefaultChatHandlersForComponent = (chatClient, chatThreadClient, _) => {
    return createDefaultChatHandlers(chatClient, chatThreadClient);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ChatClientContext = React.createContext(undefined);
/**
 * A {@link React.Context} that stores a {@link StatefulChatClient}.
 *
 * Chat components from this package must be wrapped with a {@link ChatClientProvider}.
 *
 * @public
 */
const ChatClientProvider = (props) => {
    return React.createElement(ChatClientContext.Provider, { value: props.chatClient }, props.children);
};
/**
 * Hook to obtain {@link StatefulChatClient} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useChatClient = () => {
    const chatClient = React.useContext(ChatClientContext);
    if (!chatClient) {
        throw 'Please wrap components with ChatClientProvider and initialize a chat client before calling the hook!';
    }
    return chatClient;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ChatThreadClientContext = React.createContext(undefined);
/**
 * A {@link React.Context} that stores a {@link @azure/communication-chat#ChatThreadClient}.
 *
 * Chat components from this package must be wrapped with a {@link ChatThreadClientProvider}.
 *
 * @public
 */
const ChatThreadClientProvider = (props) => {
    return React.createElement(ChatThreadClientContext.Provider, { value: props.chatThreadClient }, props.children);
};
/**
 * Hook to obtain {@link @azure/communication-chat#ChatThreadClient} from the provider.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useChatThreadClient = () => {
    const chatThreadClient = React.useContext(ChatThreadClientContext);
    if (!chatThreadClient) {
        throw 'Please wrap components with ChatThreadClientProvider and initialize a chat thread client before calling the hook.';
    }
    return chatThreadClient;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Hook to obtain a handler for a specified component.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useHandlers$2 = (component) => {
    const chatClient = React.useContext(ChatClientContext);
    const chatThreadClient = React.useContext(ChatThreadClientContext);
    if (!chatThreadClient || !chatClient) {
        return undefined;
    }
    return createDefaultChatHandlersForComponent(chatClient, chatThreadClient);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Hook to obtain a selector for a specified component.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useSelector$3 = (selector, selectorProps) => {
    var _a;
    const chatClient = React.useContext(ChatClientContext);
    const threadId = (_a = React.useContext(ChatThreadClientContext)) === null || _a === void 0 ? void 0 : _a.threadId;
    // Keeps track of whether the current component is mounted or not. If it has unmounted, make sure we do not modify the
    // state or it will cause React warnings in the console. https://skype.visualstudio.com/SPOOL/_workitems/edit/2453212
    const mounted = React.useRef(false);
    React.useEffect(() => {
        mounted.current = true;
        return () => {
            mounted.current = false;
        };
    });
    const threadConfigProps = React.useMemo(() => ({
        threadId
    }), [threadId]);
    const [props, setProps] = React.useState(chatClient && selector ? selector(chatClient.getState(), selectorProps !== null && selectorProps !== void 0 ? selectorProps : threadConfigProps) : undefined);
    const propRef = React.useRef(props);
    propRef.current = props;
    React.useEffect(() => {
        if (!chatClient || !selector) {
            return;
        }
        const onStateChange = (state) => {
            const newProps = selector(state, selectorProps !== null && selectorProps !== void 0 ? selectorProps : threadConfigProps);
            if (propRef.current !== newProps) {
                setProps(newProps);
            }
        };
        chatClient.onStateChange(onStateChange);
        return () => {
            chatClient.offStateChange(onStateChange);
        };
    }, [chatClient, selector, selectorProps, threadConfigProps]);
    return (selector ? props : undefined);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const getUserId$1 = (state) => toFlatCommunicationIdentifier(state.userId);
/**
 * @private
 */
const getDisplayName$1 = (state) => state.displayName;
/**
 * @private
 */
const getChatMessages$1 = (state, props) => { var _a; return props.threadId && ((_a = state.threads[props.threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages) || {}; };
/**
 * @private
 */
const getParticipants = (state, props) => { var _a; return props.threadId && ((_a = state.threads[props.threadId]) === null || _a === void 0 ? void 0 : _a.participants) || {}; };
/**
 * @private
 */
const getReadReceipts = (state, props) => {
    var _a;
    return (_a = state.threads[props === null || props === void 0 ? void 0 : props.threadId]) === null || _a === void 0 ? void 0 : _a.readReceipts;
};
/**
 * @private
 */
const getIsLargeGroup = (state, props) => {
    var _a;
    const participants = (_a = state.threads[props.threadId]) === null || _a === void 0 ? void 0 : _a.participants;
    return !!participants && Object.values(participants).length > 20;
};
/**
 * @private
 */
const getLatestReadTime = (state, props) => { var _a; return props.threadId && ((_a = state.threads[props.threadId]) === null || _a === void 0 ? void 0 : _a.latestReadTime) || new Date(0); };
/**
 * @private
 */
const getTypingIndicators = (state, props) => {
    var _a;
    return props.threadId && ((_a = state.threads[props.threadId]) === null || _a === void 0 ? void 0 : _a.typingIndicators) || [];
};
/**
 * @private
 */
const getLatestErrors = (state) => state.latestErrors;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Selector for {@link SendBox} component.
 *
 * @public
 */
const sendBoxSelector = reselect.createSelector([getUserId$1, getDisplayName$1], (userId, displayName) => ({
    displayName: displayName,
    userId: userId
}));

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const MINIMUM_TYPING_INTERVAL_IN_MILLISECONDS = 8000;
/**
 * @private
 */
const PARTICIPANTS_THRESHOLD = 20;
/**
 * @private
 */
const MINUTE_IN_MS = 1000 * 60;
/**
 * @private
 *
 * TODO: Import enum from @azure/communication-chat when ChatMessageType shows up in new release version
 * (or define our type to decouple)
 */
const ACSKnownMessageType = {
    text: 'text',
    html: 'html',
    richtextHtml: 'richtext/html',
    participantAdded: 'participantAdded',
    participantRemoved: 'participantRemoved'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const compareMessages = (firstMessage, secondMessage) => {
    const firstDate = new Date(firstMessage.createdOn).getTime();
    const secondDate = new Date(secondMessage.createdOn).getTime();
    // Workaround to get a stable order of message before we can get sequenceId from chat service.
    // MessageIds are generated by time order the message persist, a good candidate other than time
    const idDelta = firstMessage.messageId && secondMessage.messageId ? Number.parseInt(firstMessage.messageId) - Number.parseInt(secondMessage.messageId) : 0;
    return firstDate - secondDate === 0 ? idDelta : firstDate - secondDate;
};

/**
 * @private
 */
const updateMessagesWithAttached = (chatMessagesWithStatus) => {
    chatMessagesWithStatus.sort(compareMessages);
    chatMessagesWithStatus.forEach((message, index, messages) => {
        var _a, _b;
        if (message.messageType === 'chat') {
            /**
             * Attached === true means it is within a group of messages in the current order
             * Attached === top/bottom means it is on the top/bottom boundary
             * Attached === false means it is just a single message
             * A group of messages: continuous messages that belong to the same sender and not intercepted by other senders.
             */
            let attached = false;
            const previousMessage = index > 0 ? messages[index - 1] : undefined;
            const nextMessage = index === messages.length - 1 ? undefined : messages[index + 1];
            const previousSenderId = (previousMessage === null || previousMessage === void 0 ? void 0 : previousMessage.messageType) === 'chat' ? previousMessage.senderId : undefined;
            const nextSenderId = (nextMessage === null || nextMessage === void 0 ? void 0 : nextMessage.messageType) === 'chat' ? nextMessage.senderId : undefined;
            const timediff = new Date((_a = message === null || message === void 0 ? void 0 : message.createdOn) !== null && _a !== void 0 ? _a : '').getTime() - new Date((_b = previousMessage === null || previousMessage === void 0 ? void 0 : previousMessage.createdOn) !== null && _b !== void 0 ? _b : '').getTime();
            const diffMins = Math.round(timediff / MINUTE_IN_MS); // minutes
            if (previousSenderId !== message.senderId) {
                attached = message.senderId === nextSenderId ? 'top' : false;
            }
            else if (diffMins && diffMins >= 5) {
                // if there are more than or equal to 5 mins time gap between messages do not attach and show time stamp
                attached = false;
            }
            else {
                attached = message.senderId === nextSenderId ? true : 'bottom';
            }
            message.attached = attached;
        }
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const memoizedAllConvertChatMessage = memoizeFnAll((_key, chatMessage, userId, isSeen, isLargeGroup) => {
    const messageType = chatMessage.type.toLowerCase();
    if (messageType === ACSKnownMessageType.text || messageType === ACSKnownMessageType.richtextHtml || messageType === ACSKnownMessageType.html) {
        return convertToUiChatMessage(chatMessage, userId, isSeen, isLargeGroup);
    }
    else {
        return convertToUiSystemMessage(chatMessage);
    }
});
const extractAttachmentMetadata = (metadata) => {
    const attachmentMetadata = metadata.fileSharingMetadata;
    if (!attachmentMetadata) {
        return [];
    }
    try {
        return JSON.parse(attachmentMetadata);
    }
    catch (e) {
        console.error(e);
        return [];
    }
};
const extractTeamsAttachmentsMetadata = (rawAttachments) => {
    const attachments = [];
    rawAttachments.forEach(rawAttachment => {
        var _a;
        const attachmentType = rawAttachment.attachmentType;
        if (attachmentType === 'file') {
            attachments.push({
                id: rawAttachment.id,
                name: (_a = rawAttachment.name) !== null && _a !== void 0 ? _a : '',
                url: extractAttachmentUrl(rawAttachment)
            });
        }
    });
    return {
        attachments
    };
};
const extractAttachmentUrl = (attachment) => {
    return attachment.previewUrl ? attachment.previewUrl : attachment.url || '';
};
const processChatMessageContent = (message) => {
    var _a, _b, _c, _d;
    let content = (_a = message.content) === null || _a === void 0 ? void 0 : _a.message;
    if (((_b = message.content) === null || _b === void 0 ? void 0 : _b.attachments) && ((_c = message.content) === null || _c === void 0 ? void 0 : _c.attachments.length) > 0 && sanitizedMessageContentType(message.type).includes('html')) {
        const attachments = (_d = message.content) === null || _d === void 0 ? void 0 : _d.attachments;
        // Fill in the src here
        if (content) {
            const document = new DOMParser().parseFromString(content !== null && content !== void 0 ? content : '', 'text/html');
            document.querySelectorAll('img').forEach(img => {
                var _a, _b, _c;
                const attachmentPreviewUrl = (_a = attachments.find(attachment => attachment.id === img.id)) === null || _a === void 0 ? void 0 : _a.previewUrl;
                if (attachmentPreviewUrl) {
                    const resourceCache = (_b = message.resourceCache) === null || _b === void 0 ? void 0 : _b[attachmentPreviewUrl];
                    const src = getResourceSourceUrl(resourceCache);
                    // if in error state
                    if (src === undefined) {
                        const brokenImageView = getBrokenImageViewNode(img);
                        (_c = img.parentElement) === null || _c === void 0 ? void 0 : _c.replaceChild(brokenImageView, img);
                    }
                    else {
                        // else in loading or success state
                        img.setAttribute('src', src);
                    }
                    setImageWidthAndHeight(img);
                }
            });
            content = document.body.innerHTML;
        }
        const teamsImageHtmlContent = attachments.filter(attachment => { var _a, _b; return attachment.attachmentType === 'image' && attachment.previewUrl !== undefined && !((_b = (_a = message.content) === null || _a === void 0 ? void 0 : _a.message) === null || _b === void 0 ? void 0 : _b.includes(attachment.id)); }).map(attachment => generateImageAttachmentImgHtml(message, attachment)).join('');
        if (teamsImageHtmlContent) {
            return (content !== null && content !== void 0 ? content : '') + teamsImageHtmlContent;
        }
    }
    return content;
};
const generateImageAttachmentImgHtml = (message, attachment) => {
    var _a;
    if (attachment.previewUrl !== undefined) {
        const contentType = extractAttachmentContentTypeFromName(attachment.name);
        const resourceCache = (_a = message.resourceCache) === null || _a === void 0 ? void 0 : _a[attachment.previewUrl];
        const src = getResourceSourceUrl(resourceCache);
        // if in error state
        if (src === undefined) {
            return `\r\n<p>${getBrokenImageViewNode().outerHTML}</p>`;
        }
        // else in loading or success state
        return `\r\n<p><img alt="image" src="${src}" itemscope="${contentType}" id="${attachment.id}"></p>`;
    }
    return '';
};
const getResourceSourceUrl = (result) => {
    if (result) {
        if (!result.error && result.sourceUrl) {
            // return sourceUrl for success state
            return result.sourceUrl;
        }
        else {
            // return undefined for error state
            return undefined;
        }
    }
    // return empty string for loading state
    return '';
};
const extractAttachmentContentTypeFromName = (name) => {
    if (name === undefined) {
        return '';
    }
    const indexOfLastDot = name.lastIndexOf('.');
    if (indexOfLastDot === undefined || indexOfLastDot < 0) {
        return '';
    }
    const contentType = name.substring(indexOfLastDot + 1);
    return contentType;
};
const setImageWidthAndHeight = (img) => {
    if (img) {
        // define aspect ratio explicitly to prevent image not being displayed correctly
        // in safari, this includes image placeholder for loading state
        const width = img.width;
        const height = img.height;
        img.style.aspectRatio = `${width}/${height}`;
    }
};
const extractAttachmentsMetadata = (message) => {
    var _a, _b;
    let attachments = [];
    if (message.metadata) {
        attachments = attachments.concat(extractAttachmentMetadata(message.metadata));
    }
    if ((_a = message.content) === null || _a === void 0 ? void 0 : _a.attachments) {
        const teamsAttachments = extractTeamsAttachmentsMetadata((_b = message.content) === null || _b === void 0 ? void 0 : _b.attachments);
        attachments = attachments.concat(teamsAttachments.attachments);
    }
    return {
        attachments: attachments.length > 0 ? attachments : undefined
    };
};
const convertToUiChatMessage = (message, userId, isSeen, isLargeGroup) => {
    const messageSenderId = message.sender !== undefined ? toFlatCommunicationIdentifier(message.sender) : userId;
    const { attachments } = extractAttachmentsMetadata(message);
    return {
        messageType: 'chat',
        createdOn: message.createdOn,
        content: processChatMessageContent(message),
        contentType: sanitizedMessageContentType(message.type),
        status: !isLargeGroup && message.status === 'delivered' && isSeen ? 'seen' : message.status,
        senderDisplayName: message.senderDisplayName,
        senderId: messageSenderId,
        messageId: message.id,
        clientMessageId: message.clientMessageId,
        editedOn: message.editedOn,
        deletedOn: message.deletedOn,
        mine: messageSenderId === userId,
        metadata: message.metadata,
        attachments
    };
};
const convertToUiSystemMessage = (message) => {
    var _a, _b, _c, _d, _e;
    const systemMessageType = message.type;
    if (systemMessageType === 'participantAdded' || systemMessageType === 'participantRemoved') {
        return {
            messageType: 'system',
            systemMessageType,
            createdOn: message.createdOn,
            participants: (_c = (_b = (_a = message.content) === null || _a === void 0 ? void 0 : _a.participants) === null || _b === void 0 ? void 0 : _b.filter((participant) => participant.displayName && participant.displayName !== '').map((participant) => ({
                userId: toFlatCommunicationIdentifier(participant.id),
                displayName: participant.displayName
            }))) !== null && _c !== void 0 ? _c : [],
            messageId: message.id,
            iconName: systemMessageType === 'participantAdded' ? 'PeopleAdd' : 'PeopleBlock'
        };
    }
    else {
        // Only topic updated type left, according to ACSKnown type
        return {
            messageType: 'system',
            systemMessageType: 'topicUpdated',
            createdOn: message.createdOn,
            topic: (_e = (_d = message.content) === null || _d === void 0 ? void 0 : _d.topic) !== null && _e !== void 0 ? _e : '',
            messageId: message.id,
            iconName: 'Edit'
        };
    }
};
/** Returns `true` if the message has participants and at least one participant has a display name. */
const hasValidParticipant = (chatMessage) => { var _a; return !!((_a = chatMessage.content) === null || _a === void 0 ? void 0 : _a.participants) && chatMessage.content.participants.some((p) => !!p.displayName); };
/**
 *
 * @private
 */
const messageThreadSelectorWithThread = () => reselect.createSelector([getUserId$1, getChatMessages$1, getLatestReadTime, getIsLargeGroup, getReadReceipts, getParticipants], (userId, chatMessages, latestReadTime, isLargeGroup, readReceipts = [], participants) => {
    // We can't get displayName in teams meeting interop for now, disable rr feature when it is teams interop
    const isTeamsInterop = Object.values(participants).find(p => 'microsoftTeamsUserId' in p.id) !== undefined;
    // get number of participants
    // filter out the non valid participants (no display name)
    // Read Receipt details will be disabled when participant count is 0
    const participantCount = isTeamsInterop ? undefined : Object.values(participants).filter(p => p.displayName && p.displayName !== '').length;
    // creating key value pairs of senderID: last read message information
    const readReceiptsBySenderId = {};
    // readReceiptsBySenderId[senderID] gets updated every time a new message is read by this sender
    // in this way we can make sure that we are only saving the latest read message id and read on time for each sender
    readReceipts.filter(r => r.sender && toFlatCommunicationIdentifier(r.sender) !== userId).forEach(r => {
        var _a, _b;
        readReceiptsBySenderId[toFlatCommunicationIdentifier(r.sender)] = {
            lastReadMessage: r.chatMessageId,
            displayName: (_b = (_a = participants[toFlatCommunicationIdentifier(r.sender)]) === null || _a === void 0 ? void 0 : _a.displayName) !== null && _b !== void 0 ? _b : ''
        };
    });
    // A function takes parameter above and generate return value
    const convertedMessages = memoizedAllConvertChatMessage(memoizedFn => Object.values(chatMessages).filter(message => message.type.toLowerCase() === ACSKnownMessageType.text || message.type.toLowerCase() === ACSKnownMessageType.richtextHtml || message.type.toLowerCase() === ACSKnownMessageType.html || message.type === ACSKnownMessageType.participantAdded && hasValidParticipant(message) || message.type === ACSKnownMessageType.participantRemoved && hasValidParticipant(message) ||
        // TODO: Add support for topicUpdated system messages in MessageThread component.
        // message.type === ACSKnownMessageType.topicUpdated ||
        message.clientMessageId !== undefined).filter(isMessageValidToRender).map(message => {
        var _a;
        return memoizedFn((_a = message.id) !== null && _a !== void 0 ? _a : message.clientMessageId, message, userId, message.createdOn <= latestReadTime, isLargeGroup);
    }));
    updateMessagesWithAttached(convertedMessages);
    return {
        userId,
        showMessageStatus: true,
        messages: convertedMessages,
        participantCount,
        readReceiptsBySenderId
    };
});
const sanitizedMessageContentType = (type) => {
    const lowerCaseType = type.toLowerCase();
    return lowerCaseType === 'text' || lowerCaseType === 'html' || lowerCaseType === 'richtext/html' ? lowerCaseType : 'unknown';
};
const getBrokenImageViewNode = (img) => {
    var _a;
    const wrapper = document.createElement('div');
    Array.from((_a = img === null || img === void 0 ? void 0 : img.attributes) !== null && _a !== void 0 ? _a : []).forEach(attr => {
        var _a;
        wrapper.setAttribute(attr.nodeName, (_a = attr.nodeValue) !== null && _a !== void 0 ? _a : '');
    });
    wrapper.setAttribute('class', 'broken-image-wrapper');
    wrapper.setAttribute('data-ui-id', 'broken-image-icon');
    return wrapper;
};
const isMessageValidToRender = (message) => {
    var _a, _b, _c, _d;
    if (message.deletedOn) {
        return false;
    }
    if (((_a = message.metadata) === null || _a === void 0 ? void 0 : _a.fileSharingMetadata) || ((_c = (_b = message.content) === null || _b === void 0 ? void 0 : _b.attachments) === null || _c === void 0 ? void 0 : _c.length)) {
        return true;
    }
    return !!(message.content && ((_d = message.content) === null || _d === void 0 ? void 0 : _d.message) !== '');
};
/**
 * Selector for {@link MessageThread} component.
 *
 * @public
 */
messageThreadSelectorWithThread();

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const filterTypingIndicators = (typingIndicators, userId) => {
    const filteredTypingIndicators = [];
    const seen = new Set();
    const date8SecondsAgo = new Date(Date.now() - MINIMUM_TYPING_INTERVAL_IN_MILLISECONDS);
    const reversedTypingIndicators = typingIndicators.toReversed();
    for (const typingIndicator of reversedTypingIndicators) {
        if (toFlatCommunicationIdentifier(typingIndicator.sender) === userId) {
            continue;
        }
        if (typingIndicator.receivedOn < date8SecondsAgo) {
            continue;
        }
        if (seen.has(toFlatCommunicationIdentifier(typingIndicator.sender))) {
            continue;
        }
        seen.add(toFlatCommunicationIdentifier(typingIndicator.sender));
        filteredTypingIndicators.push(typingIndicator);
    }
    return filteredTypingIndicators;
};
const convertSdkTypingIndicatorsToCommunicationParticipants = (typingIndicators, participants) => {
    return typingIndicators.map(typingIndicator => {
        var _a;
        return ({
            userId: toFlatCommunicationIdentifier(typingIndicator.sender),
            displayName: (_a = participants[toFlatCommunicationIdentifier(typingIndicator.sender)]) === null || _a === void 0 ? void 0 : _a.displayName
        });
    });
};
/**
 * Selector for {@link TypingIndicator} component.
 *
 * @public
 */
const typingIndicatorSelector = reselect.createSelector([getTypingIndicators, getParticipants, getUserId$1], (typingIndicators, participants, userId) => {
    // if the participant size reaches the threshold then return no typing users
    if (Object.values(participants).length >= PARTICIPANTS_THRESHOLD) {
        return {
            typingUsers: []
        };
    }
    // filter typing indicators to remove those that are from the duplicate users or current user as well as those older than a threshold
    const filteredTypingIndicators = filterTypingIndicators(typingIndicators, userId);
    const typingUsers = convertSdkTypingIndicatorsToCommunicationParticipants(filteredTypingIndicators, participants);
    return {
        typingUsers
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const convertChatParticipantsToCommunicationParticipants = (chatParticipants) => {
    return chatParticipants.map((participant) => {
        return {
            userId: toFlatCommunicationIdentifier(participant.id),
            displayName: participant.displayName,
            // ACS users can not remove Teams users.
            // Removing phone numbers or unknown types of users is undefined.
            isRemovable: communicationCommon.getIdentifierKind(participant.id).kind === 'communicationUser'
        };
    });
};
/**
 * get the moderator to help updating its display name if they are the local user or removing them from list of participants otherwise
 */
const getModerator = (participants) => {
    return participants.find(p => p.displayName === undefined);
};
/**
 * Selector for {@link ParticipantList} component.
 *
 * @public
 */
const chatParticipantListSelector = reselect__namespace.createSelector([getUserId$1, getParticipants, getDisplayName$1], (userId, chatParticipants, displayName) => {
    let participants = convertChatParticipantsToCommunicationParticipants(Object.values(chatParticipants));
    // Update the moderator display name if they are the local user, otherwise remove them from list of participants
    const moderator = getModerator(participants);
    if ((moderator === null || moderator === void 0 ? void 0 : moderator.userId) === userId) {
        moderator.displayName = displayName;
    }
    else {
        participants = participants.filter(p => p.displayName);
    }
    return {
        myUserId: userId,
        participants: participants
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Select the first fiew active errors from the state for the {@link ErrorBar} component.
 *
 * In case there are many errors, only a few top errors are returned to avoid
 * filling up the UI with too many errors.
 *
 * Invariants:
 *   - {@link ErrorType} is never repeated in the returned errors.
 *   - Errors are returned in a fixed order by {@link ErrorType}.
 *
 * @public
 */
const errorBarSelector = reselect.createSelector([getLatestErrors], (latestErrors) => {
    var _a;
    // The order in which the errors are returned is significant: The `ErrorBar` shows errors on the UI in that order.
    // There are several options for the ordering:
    //   - Sorted by when the errors happened (latest first / oldest first).
    //   - Stable sort by error type.
    //
    // We chose to stable sort by error type: We intend to show only a small number of errors on the UI and we do not
    // have timestamps for errors.
    const activeErrorMessages = [];
    let specificSendMessageErrorSeen = false;
    {
        const error = latestUnableToReachChatServiceError(latestErrors);
        if (error !== undefined) {
            activeErrorMessages.push(error);
        }
    }
    {
        const error = latestAccessDeniedError(latestErrors);
        if (error !== undefined) {
            activeErrorMessages.push(error);
        }
    }
    const sendMessageError = latestErrors['ChatThreadClient.sendMessage'];
    {
        const error = latestNotInThisThreadError(latestErrors);
        if (error !== undefined) {
            if (sendMessageError !== undefined) {
                activeErrorMessages.push({
                    type: 'sendMessageNotInChatThread',
                    // Set the latest timestamp of all the errors that translated to an active error.
                    timestamp: sendMessageError.timestamp > ((_a = error.timestamp) !== null && _a !== void 0 ? _a : 0) ? sendMessageError.timestamp : error.timestamp
                });
                specificSendMessageErrorSeen = true;
            }
            else {
                activeErrorMessages.push(error);
            }
        }
    }
    if (!specificSendMessageErrorSeen && sendMessageError !== undefined) {
        activeErrorMessages.push({
            type: 'sendMessageGeneric',
            timestamp: sendMessageError.timestamp
        });
    }
    // We only return the first few errors to avoid filling up the UI with too many `MessageBar`s.
    activeErrorMessages.splice(maxErrorCount);
    return {
        activeErrorMessages: activeErrorMessages
    };
});
const maxErrorCount = 3;
const accessErrorTargets = ['ChatThreadClient.getProperties', 'ChatThreadClient.listMessages', 'ChatThreadClient.listParticipants', 'ChatThreadClient.sendMessage', 'ChatThreadClient.sendTypingNotification'];
const latestUnableToReachChatServiceError = (latestErrors) => {
    return latestActiveErrorSatisfying(latestErrors, 'unableToReachChatService', (error) => {
        return !!error && !!error.innerError && 'code' in error.innerError && error.innerError.code === 'REQUEST_SEND_ERROR';
    });
};
const latestAccessDeniedError = (latestErrors) => {
    return latestActiveErrorSatisfying(latestErrors, 'accessDenied', (error) => {
        return !!error && !!error.innerError && 'statusCode' in error.innerError && error.innerError.statusCode === 401;
    });
};
const latestNotInThisThreadError = (latestErrors) => {
    return latestActiveErrorSatisfying(latestErrors, 'userNotInChatThread', (error) => {
        if (!error || !error.innerError) {
            return false;
        }
        // Explicitly ignore 400 REST error when listParticipants() is called and a BotContact MRI is found in the participants.
        // This check should be removed when the chat SDK has handled this issue. Note: The this does not stop the error being logged to the console.
        // To the best of our ability we have confirmed this to have no impact on the participantList returned (all valid participants are still returned), nor
        // does it have an impact on the participant list updating on other participants joining/leaving or on individual participant actions like removeParticipant.
        if (isErrorDueToBotContact(error)) {
            return false;
        }
        // Chat service returns 403 if a user has been removed from a thread.
        // Chat service returns either 400 or 404 if the thread ID is malformed, depending on how the thread ID is malformed.
        return [400, 403, 404].some(statusCode => 'statusCode' in error.innerError && error.innerError.statusCode === statusCode);
    });
};
const botContactMRIPrefix = '28:';
const isErrorDueToBotContact = (error) => 'statusCode' in error.innerError && error.innerError.statusCode === 400 && error.innerError.message.includes(`Identifier format is not supported (${botContactMRIPrefix}`);
const latestActiveErrorSatisfying = (errors, activeErrorType, predicate) => {
    const activeErrorMessages = [];
    for (const target of accessErrorTargets) {
        const error = errors[target];
        if (predicate(error)) {
            activeErrorMessages.push({
                type: activeErrorType,
                timestamp: error.timestamp
            });
        }
    }
    if (activeErrorMessages.length === 0) {
        return undefined;
    }
    // We're actually sure that both timestamps will always exist, because we set them above.
    activeErrorMessages.sort((a, b) => { var _a, _b, _c, _d; return ((_b = (_a = a.timestamp) === null || _a === void 0 ? void 0 : _a.getTime()) !== null && _b !== void 0 ? _b : 0) - ((_d = (_c = b.timestamp) === null || _c === void 0 ? void 0 : _c.getTime()) !== null && _d !== void 0 ? _d : 0); });
    return activeErrorMessages[activeErrorMessages.length - 1];
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Get the selector for a specified component.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const getSelector = (component) => {
    return findSelector(component);
};
const messageThreadSelectorsByThread = {};
const findSelector = (component) => {
    // For the message thread selector we need to create a new one for each thread
    // If we have just one for the entire app, then we will have updates when not expecting due to
    // the arguments changing
    const getMessageThreadSelector = () => {
        var _a, _b;
        const threadId = (_b = (_a = React.useContext(ChatThreadClientContext)) === null || _a === void 0 ? void 0 : _a.threadId) !== null && _b !== void 0 ? _b : 'default-id-when-not-in-provider';
        let messageThreadSelectorImpl = messageThreadSelectorsByThread[threadId];
        if (!messageThreadSelectorImpl) {
            messageThreadSelectorImpl = messageThreadSelectorWithThread();
            messageThreadSelectorsByThread[threadId] = messageThreadSelectorImpl;
        }
        return messageThreadSelectorImpl;
    };
    // Add component type check to assist in identification for usePropsFor
    // to avoid issue where production build does not have the component name
    switch (component) {
        case SendBox:
            return sendBoxSelector;
        case MessageThread:
            return getMessageThreadSelector();
        case TypingIndicator:
            return typingIndicatorSelector;
        case ParticipantList$n:
            return chatParticipantListSelector;
        case ErrorBar:
            return errorBarSelector;
    }
    return undefined;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Error thrown from failed {@link StatefulChatClient} methods.
 *
 * @public
 */
class ChatError extends Error {
    constructor(target, innerError, timestamp) {
        super();
        this.target = target;
        this.innerError = innerError;
        // Testing note: It is easier to mock Date::now() than the Date() constructor.
        this.timestamp = timestamp !== null && timestamp !== void 0 ? timestamp : new Date(Date.now());
        this.name = 'ChatError';
        this.message = `${this.target}: ${this.innerError.message}`;
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const Constants = {
    TYPING_INDICATOR_MAINTAIN_TIME: 8 * 1000};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const chatStatefulLogger = logger.createClientLogger('communication-react:chat-stateful');

var __awaiter$A = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
class ResourceDownloadQueue {
    constructor(context, authentication) {
        this._messagesNeedingResourceRetrieval = [];
        this.isActive = false;
        this._requestsToCancel = {};
        this._context = context;
        this._credential = authentication.credential;
        this._endpoint = authentication.endpoint;
    }
    containsMessageWithSameAttachments(message) {
        var _a, _b, _c;
        let contains = false;
        const incomingAttachment = (_a = message.content) === null || _a === void 0 ? void 0 : _a.attachments;
        if (incomingAttachment) {
            for (const m of this._messagesNeedingResourceRetrieval) {
                const existingAttachment = (_c = (_b = m.content) === null || _b === void 0 ? void 0 : _b.attachments) !== null && _c !== void 0 ? _c : [];
                contains = incomingAttachment.every((element, index) => element === existingAttachment[index]);
                if (contains) {
                    break;
                }
            }
        }
        return contains;
    }
    addMessage(message) {
        // make a copy of message and add to queue
        const copy = Object.assign({}, message);
        this._messagesNeedingResourceRetrieval.push(copy);
    }
    startQueue(threadId, operation, options) {
        return __awaiter$A(this, void 0, void 0, function* () {
            if (this.isActive) {
                return;
            }
            while (this._messagesNeedingResourceRetrieval.length > 0) {
                this.isActive = true;
                let message = this._messagesNeedingResourceRetrieval.shift();
                if (!message) {
                    this.isActive = false;
                    continue;
                }
                if (options) {
                    const singleUrl = options.singleUrl;
                    message = yield this.downloadSingleUrl(message, singleUrl, operation);
                }
                else {
                    message = yield this.downloadAllPreviewUrls(message, operation);
                }
                this._context.setChatMessage(threadId, message);
                this.isActive = false;
            }
        });
    }
    cancelAllRequests() {
        for (const cancelation of Object.values(this._requestsToCancel)) {
            cancelation.abortController.abort();
        }
        this._requestsToCancel = {};
    }
    cancelRequest(url) {
        var _a;
        if (this._requestsToCancel[url]) {
            (_a = this._requestsToCancel[url]) === null || _a === void 0 ? void 0 : _a.abortController.abort();
            delete this._requestsToCancel[url];
        }
    }
    downloadSingleUrl(message, resourceUrl, operation) {
        return __awaiter$A(this, void 0, void 0, function* () {
            const response = {
                sourceUrl: ''
            };
            try {
                const abortController = new AbortController();
                const blobUrl = yield this.downloadResource(operation, resourceUrl, abortController);
                response.sourceUrl = blobUrl;
            }
            catch (error) {
                response.error = error;
                delete this._requestsToCancel[resourceUrl];
            }
            message = Object.assign(Object.assign({}, message), { resourceCache: Object.assign(Object.assign({}, message.resourceCache), { [resourceUrl]: response }) });
            return message;
        });
    }
    downloadAllPreviewUrls(message, operation) {
        return __awaiter$A(this, void 0, void 0, function* () {
            var _a;
            const attachments = (_a = message.content) === null || _a === void 0 ? void 0 : _a.attachments;
            if (message.type === 'html' && attachments) {
                if (message.resourceCache === undefined) {
                    message.resourceCache = {};
                }
                for (const attachment of attachments) {
                    if (attachment.previewUrl && attachment.attachmentType === 'image') {
                        const response = {
                            sourceUrl: ''
                        };
                        try {
                            const abortController = new AbortController();
                            const blobUrl = yield this.downloadResource(operation, attachment.previewUrl, abortController);
                            response.sourceUrl = blobUrl;
                        }
                        catch (error) {
                            response.error = error;
                            delete this._requestsToCancel[attachment.previewUrl];
                        }
                        message.resourceCache[attachment.previewUrl] = response;
                    }
                }
            }
            return message;
        });
    }
    downloadResource(operation, url, abortController) {
        return __awaiter$A(this, void 0, void 0, function* () {
            this._requestsToCancel[url] = {
                src: url,
                abortController
            };
            const blobUrl = yield operation(url, {
                credential: this._credential,
                endpoint: this._endpoint
            }, {
                abortController
            });
            delete this._requestsToCancel[url];
            return blobUrl;
        });
    }
}
/**
 * @private
 */
const fetchImageSource = (src, authentication, options) => __awaiter$A(void 0, void 0, void 0, function* () {
    function fetchWithAuthentication(url, token, options) {
        return __awaiter$A(this, void 0, void 0, function* () {
            const headers = new Headers();
            headers.append('Authorization', `Bearer ${token}`);
            return yield fetchWithTimeout(url, {
                timeout: options.timeout,
                headers,
                abortController: options.abortController
            });
        });
    }
    function fetchWithTimeout(resource, options) {
        return __awaiter$A(this, void 0, void 0, function* () {
            // default timeout is 30 seconds
            const { timeout = 30000, abortController } = options;
            const id = setTimeout(() => {
                abortController.abort();
            }, timeout);
            const response = yield fetch(resource, Object.assign(Object.assign({}, options), { signal: abortController.signal }));
            clearTimeout(id);
            return response;
        });
    }
    const fetchUrl = new URL(src);
    const endpoint = new URL(authentication.endpoint);
    let token = '';
    if (fetchUrl.hostname === endpoint.hostname && fetchUrl.protocol === 'https:') {
        token = (yield authentication.credential.getToken()).token;
    }
    const response = yield fetchWithAuthentication(src, token, options);
    if (response.status >= 400) {
        throw new Error(`Failed to fetch image source. Status code: ${response.status}`);
    }
    const blob = yield response.blob();
    return URL.createObjectURL(blob);
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$z = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
immer.enableMapSet();
// Needed to generate state diff for verbose logging.
immer.enablePatches();
/**
 * @internal
 */
let ChatContext$1 = class ChatContext {
    constructor(maxListeners, credential, endpoint) {
        this._state = {
            userId: {
                id: ''
            },
            displayName: '',
            threads: {},
            latestErrors: {}
        };
        this._batchMode = false;
        this.typingIndicatorInterval = undefined;
        this._inlineImageQueue = undefined;
        this._fullsizeImageQueue = undefined;
        this._logger = logger.createClientLogger('communication-react:chat-context');
        this._emitter = new events.EventEmitter();
        if (credential) {
            this._inlineImageQueue = new ResourceDownloadQueue(this, {
                credential,
                endpoint: endpoint !== null && endpoint !== void 0 ? endpoint : ''
            });
            this._fullsizeImageQueue = new ResourceDownloadQueue(this, {
                credential,
                endpoint: endpoint !== null && endpoint !== void 0 ? endpoint : ''
            });
        }
        if (maxListeners) {
            this._emitter.setMaxListeners(maxListeners);
        }
    }
    getState() {
        return this._state;
    }
    modifyState(modifier) {
        const priorState = this._state;
        this._state = immer.produce(this._state, modifier, (patches) => {
            if (logger.getLogLevel() === 'verbose') {
                // Log to `info` because AzureLogger.verbose() doesn't show up in console.
                this._logger.info(`State change: ${_safeJSONStringify(patches)}`);
            }
        });
        if (!this._batchMode && this._state !== priorState) {
            this._emitter.emit('stateChanged', this._state);
        }
    }
    dispose() {
        this.modifyState((draft) => {
            var _a, _b;
            (_a = this._inlineImageQueue) === null || _a === void 0 ? void 0 : _a.cancelAllRequests();
            (_b = this._fullsizeImageQueue) === null || _b === void 0 ? void 0 : _b.cancelAllRequests();
            Object.values(draft.threads).forEach(thread => {
                Object.values(thread.chatMessages).forEach(message => {
                    const cache = message.resourceCache;
                    if (cache) {
                        Object.values(cache).forEach(resource => {
                            if (resource.sourceUrl) {
                                URL.revokeObjectURL(resource.sourceUrl);
                            }
                        });
                    }
                    message.resourceCache = undefined;
                });
            });
        });
        // Any item in queue should be removed.
    }
    downloadResourceToCache(threadId, messageId, resourceUrl) {
        return __awaiter$z(this, void 0, void 0, function* () {
            var _a;
            let message = (_a = this.getState().threads[threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages[messageId];
            if (message && this._fullsizeImageQueue) {
                if (!message.resourceCache) {
                    message = Object.assign(Object.assign({}, message), { resourceCache: {} });
                }
                // Need to discuss retry logic in case of failure
                this._fullsizeImageQueue.addMessage(message);
                yield this._fullsizeImageQueue.startQueue(threadId, fetchImageSource, {
                    singleUrl: resourceUrl
                });
            }
        });
    }
    removeResourceFromCache(threadId, messageId, resourceUrl) {
        this.modifyState((draft) => {
            var _a, _b, _c;
            const message = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages[messageId];
            if (message && this._fullsizeImageQueue && this._fullsizeImageQueue.containsMessageWithSameAttachments(message)) {
                (_b = this._fullsizeImageQueue) === null || _b === void 0 ? void 0 : _b.cancelRequest(resourceUrl);
            }
            else if (message && this._inlineImageQueue && this._inlineImageQueue.containsMessageWithSameAttachments(message)) {
                (_c = this._inlineImageQueue) === null || _c === void 0 ? void 0 : _c.cancelRequest(resourceUrl);
            }
            if (message && message.resourceCache && message.resourceCache[resourceUrl]) {
                const resource = message.resourceCache[resourceUrl];
                if (resource === null || resource === void 0 ? void 0 : resource.sourceUrl) {
                    URL.revokeObjectURL(resource.sourceUrl);
                }
                delete message.resourceCache[resourceUrl];
            }
        });
    }
    setThread(threadId, threadState) {
        this.modifyState((draft) => {
            draft.threads[threadId] = threadState;
        });
    }
    createThread(threadId, properties) {
        this.modifyState((draft) => {
            draft.threads[threadId] = {
                chatMessages: {},
                threadId: threadId,
                properties: properties,
                participants: {},
                readReceipts: [],
                typingIndicators: [],
                latestReadTime: new Date(0)
            };
        });
    }
    updateChatConfig(userId, displayName) {
        this.modifyState((draft) => {
            draft.displayName = displayName;
            draft.userId = userId;
        });
    }
    createThreadIfNotExist(threadId, properties) {
        const exists = Object.prototype.hasOwnProperty.call(this.getState().threads, threadId);
        if (!exists) {
            this.createThread(threadId, properties);
            return true;
        }
        return false;
    }
    updateThread(threadId, properties) {
        this.modifyState((draft) => {
            const thread = draft.threads[threadId];
            if (thread) {
                thread.properties = properties;
            }
        });
    }
    updateThreadTopic(threadId, topic) {
        this.modifyState((draft) => {
            if (topic === undefined) {
                return;
            }
            const thread = draft.threads[threadId];
            if (thread && !thread.properties) {
                thread.properties = {
                    topic: topic
                };
            }
            else if (thread && thread.properties) {
                thread.properties.topic = topic;
            }
        });
    }
    deleteThread(threadId) {
        this.modifyState((draft) => {
            const thread = draft.threads[threadId];
            if (thread) {
                delete draft.threads[threadId];
            }
        });
    }
    setChatMessages(threadId, messages) {
        this.modifyState((draft) => {
            const threadState = draft.threads[threadId];
            if (threadState) {
                threadState.chatMessages = messages;
            }
            // remove typing indicator when receive messages
            const thread = draft.threads[threadId];
            if (thread) {
                for (const message of Object.values(messages)) {
                    this.filterTypingIndicatorForUser(thread, message.sender);
                }
            }
        });
    }
    updateChatMessageContent(threadId, messagesId, content) {
        this.modifyState((draft) => {
            var _a;
            const chatMessage = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages[messagesId];
            if (chatMessage) {
                if (!chatMessage.content) {
                    chatMessage.content = {};
                }
                chatMessage.content.message = content;
            }
        });
    }
    deleteLocalMessage(threadId, localId) {
        let localMessageDeleted = false;
        this.modifyState((draft) => {
            var _a;
            const chatMessages = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages;
            const message = chatMessages ? chatMessages[localId] : undefined;
            if (chatMessages && message && message.clientMessageId) {
                delete chatMessages[message.clientMessageId];
                localMessageDeleted = true;
            }
        });
        return localMessageDeleted;
    }
    deleteMessage(threadId, id) {
        this.modifyState((draft) => {
            var _a;
            const chatMessages = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages;
            if (chatMessages) {
                delete chatMessages[id];
            }
        });
    }
    setParticipant(threadId, participant) {
        this.modifyState((draft) => {
            var _a;
            const participants = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.participants;
            if (participants) {
                participants[toFlatCommunicationIdentifier(participant.id)] = participant;
            }
        });
    }
    setParticipants(threadId, participants) {
        this.modifyState((draft) => {
            var _a;
            const participantsMap = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.participants;
            if (participantsMap) {
                for (const participant of participants) {
                    participantsMap[toFlatCommunicationIdentifier(participant.id)] = participant;
                }
            }
        });
    }
    deleteParticipants(threadId, participantIds) {
        this.modifyState((draft) => {
            var _a;
            const participants = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.participants;
            if (participants) {
                participantIds.forEach(id => {
                    delete participants[toFlatCommunicationIdentifier(id)];
                });
            }
        });
    }
    deleteParticipant(threadId, participantId) {
        this.modifyState((draft) => {
            var _a;
            const participants = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.participants;
            if (participants) {
                delete participants[toFlatCommunicationIdentifier(participantId)];
            }
        });
    }
    addReadReceipt(threadId, readReceipt) {
        this.modifyState((draft) => {
            const thread = draft.threads[threadId];
            const readReceipts = thread === null || thread === void 0 ? void 0 : thread.readReceipts;
            if (thread && readReceipts) {
                // TODO(prprabhu): Replace `this.getState()` with `draft`?
                if (readReceipt.sender !== this.getState().userId && thread.latestReadTime < readReceipt.readOn) {
                    thread.latestReadTime = readReceipt.readOn;
                }
                readReceipts.push(readReceipt);
            }
        });
    }
    startTypingIndicatorCleanUp() {
        if (this.typingIndicatorInterval) {
            return;
        }
        this.typingIndicatorInterval = window.setInterval(() => {
            let isTypingActive = false;
            this.modifyState((draft) => {
                for (const thread of Object.values(draft.threads)) {
                    const filteredTypingIndicators = thread.typingIndicators.filter(typingIndicator => {
                        const timeGap = Date.now() - typingIndicator.receivedOn.getTime();
                        return timeGap < Constants.TYPING_INDICATOR_MAINTAIN_TIME;
                    });
                    if (thread.typingIndicators.length !== filteredTypingIndicators.length) {
                        thread.typingIndicators = filteredTypingIndicators;
                    }
                    if (thread.typingIndicators.length > 0) {
                        isTypingActive = true;
                    }
                }
            });
            if (!isTypingActive && this.typingIndicatorInterval) {
                window.clearInterval(this.typingIndicatorInterval);
                this.typingIndicatorInterval = undefined;
            }
        }, 1000);
    }
    addTypingIndicator(threadId, typingIndicator) {
        this.modifyState((draft) => {
            const thread = draft.threads[threadId];
            if (thread) {
                const typingIndicators = thread.typingIndicators;
                typingIndicators.push(typingIndicator);
            }
        });
        // Make sure we only maintain a period of typing indicator for perf purposes
        this.startTypingIndicatorCleanUp();
    }
    setChatMessage(threadId, message) {
        this.parseAttachments(threadId, message);
        const { id: messageId, clientMessageId } = message;
        if (messageId || clientMessageId) {
            this.modifyState((draft) => {
                var _a;
                const threadMessages = (_a = draft.threads[threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages;
                const isLocalIdInMap = threadMessages && clientMessageId && threadMessages[clientMessageId];
                const messageKey = !messageId || isLocalIdInMap ? clientMessageId : messageId;
                if (threadMessages && messageKey) {
                    threadMessages[messageKey] = message;
                }
                // remove typing indicator when receive a message from a user
                const thread = draft.threads[threadId];
                if (thread) {
                    this.filterTypingIndicatorForUser(thread, message.sender);
                }
            });
        }
    }
    parseAttachments(threadId, message) {
        var _a;
        const attachments = (_a = message.content) === null || _a === void 0 ? void 0 : _a.attachments;
        if (message.type === 'html' && attachments && attachments.length > 0) {
            if (this._inlineImageQueue && !this._inlineImageQueue.containsMessageWithSameAttachments(message) && message.resourceCache === undefined) {
                // Need to discuss retry logic in case of failure
                this._inlineImageQueue.addMessage(message);
                this._inlineImageQueue.startQueue(threadId, fetchImageSource);
            }
        }
    }
    /**
     * Tees any errors encountered in an async function to the state.
     *
     * @param f Async function to execute.
     * @param target The error target to tee error to.
     * @returns Result of calling `f`. Also re-raises any exceptions thrown from `f`.
     * @throws ChatError. Exceptions thrown from `f` are tagged with the failed `target.
     */
    withAsyncErrorTeedToState(f, target) {
        return (...args) => __awaiter$z(this, void 0, void 0, function* () {
            try {
                return yield f(...args);
            }
            catch (error) {
                const chatError = toChatError(target, error);
                this.setLatestError(target, chatError);
                throw chatError;
            }
        });
    }
    /**
     * Tees any errors encountered in an function to the state.
     *
     * @param f Function to execute.
     * @param target The error target to tee error to.
     * @returns Result of calling `f`. Also re-raises any exceptions thrown from `f`.
     * @throws ChatError. Exceptions thrown from `f` are tagged with the failed `target.
     */
    withErrorTeedToState(f, target) {
        return (...args) => {
            try {
                chatStatefulLogger.info(`Chat stateful client target function called: ${target}`);
                return f(...args);
            }
            catch (error) {
                const chatError = toChatError(target, error);
                this.setLatestError(target, chatError);
                throw chatError;
            }
        };
    }
    setLatestError(target, error) {
        this.modifyState((draft) => {
            draft.latestErrors[target] = error;
        });
    }
    // This is a mutating function, only use it inside of a produce() function
    filterTypingIndicatorForUser(thread, userId) {
        if (!userId) {
            return;
        }
        const typingIndicators = thread.typingIndicators;
        const userIdAsKey = toFlatCommunicationIdentifier(userId);
        const filteredTypingIndicators = typingIndicators.filter(typingIndicator => toFlatCommunicationIdentifier(typingIndicator.sender) !== userIdAsKey);
        if (filteredTypingIndicators.length !== typingIndicators.length) {
            thread.typingIndicators = filteredTypingIndicators;
        }
    }
    /**
     * Batch updates to minimize `stateChanged` events across related operations.
     *
     * - A maximum of one `stateChanged` event is emitted, at the end of the operations.
     * - No `stateChanged` event is emitted if the state did not change through the operations.
     * - In case of an exception, state is reset to the prior value and no `stateChanged` event is emitted.
     *
     * All operations finished in this batch should be synchronous.
     * This function is not reentrant -- do not call batch() from within another batch().
     */
    batch(operations) {
        if (this._batchMode) {
            throw new Error('batch() called from within another batch()');
        }
        this._batchMode = true;
        const priorState = this._state;
        try {
            operations();
            if (this._state !== priorState) {
                this._emitter.emit('stateChanged', this._state);
            }
        }
        catch (e) {
            this._state = priorState;
            if (logger.getLogLevel() === 'verbose') {
                this._logger.warning(`State rollback to: ${_safeJSONStringify(priorState)}`);
            }
            throw e;
        }
        finally {
            this._batchMode = false;
        }
    }
    onStateChange(handler) {
        this._emitter.on('stateChanged', handler);
    }
    offStateChange(handler) {
        this._emitter.off('stateChanged', handler);
    }
};
const toChatError = (target, error) => {
    if (error instanceof Error) {
        return new ChatError(target, error);
    }
    return new ChatError(target, new Error(`${error}`));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const convertChatMessage = (message, status = 'delivered', clientMessageId) => {
    return Object.assign(Object.assign({}, message), { clientMessageId: clientMessageId, status });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$y = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues$1 = (window && window.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
// TODO: When we can get messageId of event from SDK, remove this
// Maximum time to look back message list when we receive a system event
const maxSyncTimeInMs = 10 * 1000;
/**
 * @private
 */
class EventSubscriber {
    constructor(chatClient, chatContext) {
        this.convertEventToChatMessage = (event) => {
            return convertChatMessage({
                id: event.id,
                version: event.version,
                content: {
                    message: event.message,
                    attachments: event.attachments
                },
                type: this.convertEventType(event.type),
                sender: event.sender,
                senderDisplayName: event.senderDisplayName,
                sequenceId: '',
                // Note: there is a bug in chatMessageReceived event that it is missing sequenceId
                createdOn: new Date(event.createdOn),
                editedOn: 'editedOn' in event ? event.editedOn : undefined,
                metadata: event.metadata
            });
        };
        // convert event type to chatMessage type, only possible type is 'html' and 'text' in chat event
        this.convertEventType = (type) => {
            const lowerCaseType = type.toLowerCase();
            if (lowerCaseType === 'richtext/html' || lowerCaseType === 'html') {
                return 'html';
            }
            else {
                return 'text';
            }
        };
        this.onChatMessageReceived = (event) => {
            var _a;
            // Today we are avoiding how to render these messages. In the future we can
            // remove this condition and handle this message appropriately.
            const messageEventType = event.type.toLowerCase();
            if (messageEventType !== 'text' && messageEventType !== 'richtext/html' && messageEventType !== 'html') {
                return;
            }
            const newMessage = this.convertEventToChatMessage(event);
            // Because of bug in chatMessageReceived event, if we already have that particular message in context, we want to
            // make sure to not overwrite the sequenceId when calling setChatMessage.
            const existingMessage = (_a = this.chatContext.getState().threads[event.threadId]) === null || _a === void 0 ? void 0 : _a.chatMessages[event.id];
            if (existingMessage) {
                newMessage.sequenceId = existingMessage.sequenceId;
            }
            this.chatContext.batch(() => {
                this.chatContext.createThreadIfNotExist(event.threadId);
                this.chatContext.setChatMessage(event.threadId, newMessage);
            });
        };
        this.onChatMessageDeleted = (event) => {
            this.chatContext.deleteMessage(event.threadId, event.id);
        };
        this.onChatMessageEdited = (event) => {
            const editedMessage = this.convertEventToChatMessage(event);
            this.chatContext.setChatMessage(event.threadId, convertChatMessage(editedMessage));
        };
        this.onParticipantsAdded = (event) => {
            const participantsToAdd = event.participantsAdded.map(participant => (Object.assign(Object.assign({}, participant), { shareHistoryTime: participant.shareHistoryTime ? new Date(participant.shareHistoryTime) : undefined })));
            this.chatContext.batch(() => {
                this.chatContext.createThreadIfNotExist(event.threadId);
                this.chatContext.setParticipants(event.threadId, participantsToAdd);
            });
            this.fetchLastParticipantMessage(event.threadId, 'participantAdded');
        };
        // This is a temporary fix that no participant message is received for onChatMessageReceived event, which should be handled by JS SDK.
        // Without the temporary fix, there are missing 'participant joined' and 'participant left' system messages in the chat thread.
        this.fetchLastParticipantMessage = (threadId, actionType) => __awaiter$y(this, void 0, void 0, function* () {
            var _a, e_1, _b, _c;
            try {
                for (var _d = true, _e = __asyncValues$1(this.chatClient.getChatThreadClient(threadId).listMessages({
                    startTime: new Date(Date.now() - maxSyncTimeInMs)
                })), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                    _c = _f.value;
                    _d = false;
                    const message = _c;
                    if (message.type === actionType) {
                        this.chatContext.setChatMessage(threadId, Object.assign(Object.assign({}, message), { status: 'delivered' }));
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
        this.onParticipantsRemoved = (event) => {
            const participantIds = event.participantsRemoved.map(participant => {
                return participant.id;
            });
            this.chatContext.deleteParticipants(event.threadId, participantIds);
            // If the current user is removed from the thread, do not fetch the last participant message
            // as they no longer have access to the thread.
            const currentUserId = toFlatCommunicationIdentifier(this.chatContext.getState().userId);
            const wasCurrentUserRemoved = participantIds.find(id => toFlatCommunicationIdentifier(id) === currentUserId);
            if (!wasCurrentUserRemoved) {
                this.fetchLastParticipantMessage(event.threadId, 'participantRemoved');
            }
        };
        this.onReadReceiptReceived = (event) => {
            const readReceipt = Object.assign(Object.assign({}, event), { sender: event.sender, readOn: new Date(event.readOn) });
            this.chatContext.batch(() => {
                this.chatContext.createThreadIfNotExist(event.threadId);
                this.chatContext.addReadReceipt(event.threadId, readReceipt);
            });
        };
        this.onTypingIndicatorReceived = (typingIndicator) => {
            this.chatContext.batch(() => {
                this.chatContext.createThreadIfNotExist(typingIndicator.threadId);
                this.chatContext.addTypingIndicator(typingIndicator.threadId, typingIndicator);
            });
        };
        this.onChatThreadCreated = (event) => {
            const properties = {
                topic: event.properties.topic
            };
            if (!this.chatContext.createThreadIfNotExist(event.threadId, properties)) {
                this.chatContext.updateThread(event.threadId, properties);
            }
        };
        this.onChatThreadDeleted = (event) => {
            this.chatContext.deleteThread(event.threadId);
        };
        this.onChatThreadPropertiesUpdated = (event) => {
            this.chatContext.updateThread(event.threadId, {
                topic: event.properties.topic
            });
        };
        this.subscribe = () => {
            this.chatClient.on('chatMessageReceived', this.onChatMessageReceived);
            this.chatClient.on('chatMessageDeleted', this.onChatMessageDeleted);
            this.chatClient.on('chatMessageEdited', this.onChatMessageEdited);
            this.chatClient.on('participantsAdded', this.onParticipantsAdded);
            this.chatClient.on('participantsRemoved', this.onParticipantsRemoved);
            this.chatClient.on('readReceiptReceived', this.onReadReceiptReceived);
            this.chatClient.on('typingIndicatorReceived', this.onTypingIndicatorReceived);
            this.chatClient.on('chatThreadCreated', this.onChatThreadCreated);
            this.chatClient.on('chatThreadDeleted', this.onChatThreadDeleted);
            this.chatClient.on('chatThreadPropertiesUpdated', this.onChatThreadPropertiesUpdated);
        };
        this.unsubscribe = () => {
            this.chatClient.off('chatMessageReceived', this.onChatMessageReceived);
            this.chatClient.off('chatMessageDeleted', this.onChatMessageDeleted);
            this.chatClient.off('chatMessageEdited', this.onChatMessageEdited);
            this.chatClient.off('participantsAdded', this.onParticipantsAdded);
            this.chatClient.off('participantsRemoved', this.onParticipantsRemoved);
            this.chatClient.off('readReceiptReceived', this.onReadReceiptReceived);
            this.chatClient.off('typingIndicatorReceived', this.onTypingIndicatorReceived);
            this.chatClient.off('chatThreadCreated', this.onChatThreadCreated);
            this.chatClient.off('chatThreadDeleted', this.onChatThreadDeleted);
            this.chatClient.off('chatThreadPropertiesUpdated', this.onChatThreadPropertiesUpdated);
        };
        this.chatClient = chatClient;
        this.chatContext = chatContext;
        this.subscribe();
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$x = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Create a decorated iterator
 * returned iterators.
 *
 * @param iteratorCreator the function to create the original iterator
 * @param context chatContext
 * @param decorateFn the function for the decorating behavior
 */
const createDecoratedIterator = (iteratorCreator, context, decorateFn) => {
    return (...args) => {
        const threadsIterator = iteratorCreator(...args);
        return {
            next() {
                return __awaiter$x(this, void 0, void 0, function* () {
                    const result = yield threadsIterator.next();
                    if (!result.done && result.value) {
                        decorateFn(result.value, context);
                    }
                    return result;
                });
            },
            [Symbol.asyncIterator]() {
                return this;
            },
            byPage: (settings = {}) => {
                const pages = threadsIterator.byPage(settings);
                return {
                    next() {
                        return __awaiter$x(this, void 0, void 0, function* () {
                            const result = yield pages.next();
                            const page = result.value;
                            if (!result.done && result.value) {
                                context.batch(() => {
                                    for (const item of page) {
                                        decorateFn(item, context);
                                    }
                                });
                            }
                            return result;
                        });
                    },
                    [Symbol.asyncIterator]() {
                        return this;
                    }
                };
            }
        };
    };
};
/**
 *
 * @param iteratorCreator Function that creates the base iteartor
 * @param context The ChatContext that stores all internal state.
 * @param target See {@link ChatContext.asyncTeeErrorToState}.
 * @returns A function to create an iterator that handles errors when iterting over the iterator from `iteratorCreator`.
 */
const createErrorHandlingIterator = (iteratorCreator, context, target) => {
    return (...args) => {
        const innerIter = iteratorCreator(...args);
        return {
            next: context.withAsyncErrorTeedToState(innerIter.next.bind(innerIter), target),
            [Symbol.asyncIterator]() {
                return this;
            },
            byPage: (settings = {}) => {
                const pages = innerIter.byPage(settings);
                return {
                    next: context.withAsyncErrorTeedToState(pages.next.bind(pages), target),
                    [Symbol.asyncIterator]() {
                        return this;
                    }
                };
            }
        };
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const createDecoratedListMessages = (chatThreadClient, context) => {
    const setMessage = (message, context) => {
        context.setChatMessage(chatThreadClient.threadId, convertChatMessage(message));
    };
    return createDecoratedIterator(createErrorHandlingIterator(context.withErrorTeedToState(chatThreadClient.listMessages.bind(chatThreadClient), 'ChatThreadClient.listMessages'), context, 'ChatThreadClient.listMessages'), context, setMessage);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const createDecoratedListReadReceipts = (chatThreadClient, context) => {
    const setReadReceipt = (readReceipt, context) => {
        context.addReadReceipt(chatThreadClient.threadId, Object.assign({}, readReceipt));
    };
    return createDecoratedIterator(createErrorHandlingIterator(context.withErrorTeedToState(chatThreadClient.listReadReceipts.bind(chatThreadClient), 'ChatThreadClient.listReadReceipts'), context, 'ChatThreadClient.listReadReceipts'), context, setReadReceipt);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const createDecoratedListParticipants = (chatThreadClient, context) => {
    const setParticipant = (participant, context) => {
        context.setParticipant(chatThreadClient.threadId, participant);
    };
    return createDecoratedIterator(createErrorHandlingIterator(context.withErrorTeedToState(chatThreadClient.listParticipants.bind(chatThreadClient), 'ChatThreadClient.listParticipants'), context, 'ChatThreadClient.listParticipants'), context, setParticipant);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$w = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
class ProxyChatThreadClient {
    constructor(context) {
        this._context = context;
    }
    get(chatThreadClient, prop) {
        switch (prop) {
            case 'listMessages':
                {
                    return createDecoratedListMessages(chatThreadClient, this._context);
                }
            case 'getMessage':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        const message = yield chatThreadClient.getMessage(...args);
                        this._context.setChatMessage(chatThreadClient.threadId, convertChatMessage(message));
                        return message;
                    }), 'ChatThreadClient.getMessage');
                }
            case 'sendMessage':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        var _a;
                        // Retry logic?
                        const [request, options] = args;
                        const { content } = request;
                        const clientMessageId = nanoid.nanoid(); // Generate a local short uuid for message
                        const newMessage = {
                            content: {
                                message: content
                            },
                            clientMessageId,
                            id: '',
                            type: (_a = options === null || options === void 0 ? void 0 : options.type) !== null && _a !== void 0 ? _a : 'text',
                            sequenceId: '',
                            version: '',
                            createdOn: new Date(),
                            status: 'sending',
                            senderDisplayName: this._context.getState().displayName,
                            sender: this._context.getState().userId,
                            metadata: options === null || options === void 0 ? void 0 : options.metadata
                        };
                        this._context.setChatMessage(chatThreadClient.threadId, newMessage);
                        let result = undefined;
                        try {
                            result = yield chatThreadClient.sendMessage(...args);
                        }
                        catch (e) {
                            this._context.setChatMessage(chatThreadClient.threadId, Object.assign(Object.assign({}, newMessage), { status: 'failed' }));
                            throw e;
                        }
                        if (result === null || result === void 0 ? void 0 : result.id) {
                            this._context.batch(() => {
                                if (!result) {
                                    return;
                                }
                                this._context.setChatMessage(chatThreadClient.threadId, Object.assign(Object.assign({}, newMessage), { clientMessageId: undefined, status: 'delivered', id: result.id }));
                                this._context.deleteLocalMessage(chatThreadClient.threadId, clientMessageId);
                            });
                        }
                        return result;
                    }), 'ChatThreadClient.sendMessage');
                }
            case 'addParticipants':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        const result = yield chatThreadClient.addParticipants(...args);
                        const [addRequest] = args;
                        const participantsToAdd = addRequest.participants;
                        this._context.setParticipants(chatThreadClient.threadId, participantsToAdd);
                        return result;
                    }), 'ChatThreadClient.addParticipants');
                }
            case 'deleteMessage':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        // DeleteMessage is able to either delete local one(for failed message) or synced message
                        const [messageId] = args;
                        if (this._context.deleteLocalMessage(chatThreadClient.threadId, messageId)) {
                            return {};
                        }
                        const result = yield chatThreadClient.deleteMessage(...args);
                        this._context.deleteMessage(chatThreadClient.threadId, messageId);
                        return result;
                    }), 'ChatThreadClient.deleteMessage');
                }
            case 'listParticipants':
                {
                    return createDecoratedListParticipants(chatThreadClient, this._context);
                }
            case 'listReadReceipts':
                {
                    return createDecoratedListReadReceipts(chatThreadClient, this._context);
                }
            case 'sendTypingNotification':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        return yield chatThreadClient.sendTypingNotification(...args);
                    }), 'ChatThreadClient.sendTypingNotification');
                }
            case 'removeParticipant':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        const result = yield chatThreadClient.removeParticipant(...args);
                        const [removeIdentifier] = args;
                        this._context.deleteParticipant(chatThreadClient.threadId, communicationCommon.getIdentifierKind(removeIdentifier));
                        return result;
                    }), 'ChatThreadClient.removeParticipant');
                }
            case 'updateMessage':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        const result = yield chatThreadClient.updateMessage(...args);
                        const [messageId, updateOption] = args;
                        this._context.updateChatMessageContent(chatThreadClient.threadId, messageId, updateOption === null || updateOption === void 0 ? void 0 : updateOption.content);
                        return result;
                    }), 'ChatThreadClient.updateMessage');
                }
            case 'updateTopic':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        const result = yield chatThreadClient.updateTopic(...args);
                        const [topic] = args;
                        this._context.updateThreadTopic(chatThreadClient.threadId, topic);
                        return result;
                    }), 'ChatThreadClient.updateTopic');
                }
            case 'getProperties':
                {
                    return this._context.withAsyncErrorTeedToState((...args) => __awaiter$w(this, void 0, void 0, function* () {
                        const result = yield chatThreadClient.getProperties(...args);
                        this._context.updateThread(chatThreadClient.threadId, result);
                        return result;
                    }), 'ChatThreadClient.getProperties');
                }
            default:
                return Reflect.get(chatThreadClient, prop);
        }
    }
}
/**
 * @private
 */
const chatThreadClientDeclaratify = (chatThreadClient, context) => {
    context.createThreadIfNotExist(chatThreadClient.threadId);
    return new Proxy(chatThreadClient, new ProxyChatThreadClient(context));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const createDecoratedListThreads = (chatClient, context) => {
    const setThreadProperties = (chatThreadItem, context) => {
        const properties = {
            topic: chatThreadItem.topic
        };
        if (!context.createThreadIfNotExist(chatThreadItem.id, properties)) {
            context.updateThread(chatThreadItem.id, properties);
        }
    };
    return createDecoratedIterator(createErrorHandlingIterator(context.withErrorTeedToState(chatClient.listChatThreads.bind(chatClient), 'ChatClient.listChatThreads'), context, 'ChatClient.listChatThreads'), context, setThreadProperties);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$v = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const proxyChatClient = {
    get: function (chatClient, prop, receiver) {
        // skip receiver.context call to avoid recursive bugs
        if (prop === 'context') {
            return Reflect.get(chatClient, prop);
        }
        const context = receiver.context;
        switch (prop) {
            case 'createChatThread':
                {
                    return context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$v(this, void 0, void 0, function* () {
                            const result = yield chatClient.createChatThread(...args);
                            const thread = result.chatThread;
                            if (thread) {
                                const [request] = args;
                                context.createThread(thread.id, {
                                    topic: request.topic
                                });
                            }
                            return result;
                        });
                    }, 'ChatClient.createChatThread');
                }
            case 'deleteChatThread':
                {
                    return context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$v(this, void 0, void 0, function* () {
                            const result = yield chatClient.deleteChatThread(...args);
                            context.deleteThread(args[0]);
                            return result;
                        });
                    }, 'ChatClient.deleteChatThread');
                }
            case 'listChatThreads':
                {
                    return createDecoratedListThreads(chatClient, context);
                }
            case 'getChatThreadClient':
                {
                    return function (...args) {
                        const chatThreadClient = chatClient.getChatThreadClient(...args);
                        // TODO(prprabhu): Ensure that thread properties are fetched into the ChatContext at this point.
                        // A new thread might be created here, but the properties will never be fetched.
                        return chatThreadClientDeclaratify(chatThreadClient, context);
                    };
                }
            case 'startRealtimeNotifications':
                {
                    return context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$v(this, void 0, void 0, function* () {
                            const ret = yield chatClient.startRealtimeNotifications(...args);
                            if (!receiver.eventSubscriber) {
                                receiver.eventSubscriber = new EventSubscriber(chatClient, context);
                            }
                            return ret;
                        });
                    }, 'ChatClient.startRealtimeNotifications');
                }
            case 'stopRealtimeNotifications':
                {
                    return context.withAsyncErrorTeedToState(function (...args) {
                        return __awaiter$v(this, void 0, void 0, function* () {
                            const ret = yield chatClient.stopRealtimeNotifications(...args);
                            if (receiver.eventSubscriber) {
                                receiver.eventSubscriber.unsubscribe();
                                receiver.eventSubscriber = undefined;
                            }
                            return ret;
                        });
                    }, 'ChatClient.stopRealtimeNotifications');
                }
            default:
                return Reflect.get(chatClient, prop);
        }
    }
};
/**
 * Creates a stateful ChatClient {@link StatefulChatClient} by proxying ChatClient
 * {@link @azure/communication-chat#ChatClient} with ProxyChatClient {@link ProxyChatClient} which then allows access
 * to state in a declarative way.
 *
 * @public
 */
const createStatefulChatClient = (args, options) => {
    return _createStatefulChatClientInner(args, options);
};
/**
 * This inner function is used to allow injection of TelemetryImplementationHint without changing the public API.
 *
 * @internal
 */
const _createStatefulChatClientInner = (args, options, telemetryImplementationHint = 'StatefulComponents') => {
    chatStatefulLogger.info(`Creating chat stateful client using library version: ${_getApplicationId(telemetryImplementationHint)}`);
    const tweakedOptions = Object.assign(Object.assign({}, options), { chatClientOptions: Object.assign(Object.assign({}, options === null || options === void 0 ? void 0 : options.chatClientOptions), { userAgentOptions: {
                userAgentPrefix: _getApplicationId(telemetryImplementationHint)
            } }) });
    return _createStatefulChatClientWithDeps(new communicationChat.ChatClient(args.endpoint, args.credential, tweakedOptions.chatClientOptions), args, tweakedOptions);
};
/**
 * Internal implementation of {@link createStatefulChatClient} for dependency injection.
 *
 * Used by tests. Should not be exported out of this package.
 * @internal
 */
const _createStatefulChatClientWithDeps = (chatClient, args, options) => {
    const context = new ChatContext$1(options === null || options === void 0 ? void 0 : options.maxStateChangeListeners, args.credential, args.endpoint);
    let eventSubscriber;
    context.updateChatConfig(communicationCommon.getIdentifierKind(args.userId), args.displayName);
    const proxy = new Proxy(chatClient, proxyChatClient);
    Object.defineProperty(proxy, 'context', {
        configurable: false,
        get: () => context
    });
    Object.defineProperty(proxy, 'eventSubscriber', {
        configurable: false,
        get: () => eventSubscriber,
        set: (val) => {
            eventSubscriber = val;
        }
    });
    Object.defineProperty(proxy, 'dispose', {
        configurable: false,
        value: () => context === null || context === void 0 ? void 0 : context.dispose()
    });
    Object.defineProperty(proxy, 'downloadResourceToCache', {
        configurable: false,
        value: (threadId, messageId, resourceUrl) => context === null || context === void 0 ? void 0 : context.downloadResourceToCache(threadId, messageId, resourceUrl)
    });
    Object.defineProperty(proxy, 'removeResourceFromCache', {
        configurable: false,
        value: (threadId, messageId, resourceUrl) => context === null || context === void 0 ? void 0 : context.removeResourceFromCache(threadId, messageId, resourceUrl)
    });
    Object.defineProperty(proxy, 'getState', {
        configurable: false,
        value: () => context === null || context === void 0 ? void 0 : context.getState()
    });
    Object.defineProperty(proxy, 'onStateChange', {
        configurable: false,
        value: (handler) => context === null || context === void 0 ? void 0 : context.onStateChange(handler)
    });
    Object.defineProperty(proxy, 'offStateChange', {
        configurable: false,
        value: (handler) => context === null || context === void 0 ? void 0 : context.offStateChange(handler)
    });
    return proxy;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const UNSUPPORTED_CHAT_THREAD_TYPE = ['@thread.tacv2', '@thread.skype'];
/**
 * @private
 */
const TEAMS_LIMITATION_LEARN_MORE = 'https://learn.microsoft.com/en-us/azure/communication-services/concepts/join-teams-meeting#limitations-and-known-issues';
/**
 * @remarks
 * This value is publicly documented in the ChatComposite API documentation.
 * Changing this value will require updating the API documentation.
 */
const CHAT_CONTAINER_MIN_WIDTH_REM = 17.5;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$u = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const createProfileStateModifier$1 = (onFetchProfile, notifyUpdate) => {
    const cachedDisplayName = {};
    return (state) => {
        var _a;
        const originalParticipants = (_a = state.thread) === null || _a === void 0 ? void 0 : _a.participants;
        (() => __awaiter$u(void 0, void 0, void 0, function* () {
            let shouldNotifyUpdates = false;
            if (!originalParticipants) {
                return;
            }
            for (const [key, participant] of Object.entries(originalParticipants)) {
                if (cachedDisplayName[key]) {
                    continue;
                }
                const profile = yield onFetchProfile(key, {
                    displayName: participant.displayName
                });
                if ((profile === null || profile === void 0 ? void 0 : profile.displayName) && participant.displayName !== (profile === null || profile === void 0 ? void 0 : profile.displayName)) {
                    cachedDisplayName[key] = profile === null || profile === void 0 ? void 0 : profile.displayName;
                    shouldNotifyUpdates = true;
                }
            }
            // notify update only when there is a change, which most likely will trigger modifier and setState again
            if (shouldNotifyUpdates) {
                notifyUpdate();
            }
        }))();
        const participantsModifier = createParticipantModifier$1((id, participant) => {
            if (cachedDisplayName[id]) {
                return Object.assign(Object.assign({}, participant), { displayName: cachedDisplayName[id] });
            }
            return undefined;
        });
        const modifiedParticipantState = participantsModifier(state);
        const chatMessagesModifier = createChatMessageModifier((id, chatMessage) => {
            var _a;
            const originalChatMessage = Object.assign({}, chatMessage);
            if ((_a = originalChatMessage.content) === null || _a === void 0 ? void 0 : _a.participants) {
                const newParticipants = originalChatMessage.content.participants.map((participant) => {
                    if (participant.id) {
                        if ('communicationUserId' in participant.id && cachedDisplayName[participant.id.communicationUserId]) {
                            return Object.assign(Object.assign({}, participant), { displayName: cachedDisplayName[participant.id.communicationUserId] });
                        }
                        else if ('microsoftTeamsUserId' in participant.id && 'rawId' in participant.id && participant.id.rawId && cachedDisplayName[participant.id.rawId]) {
                            return Object.assign(Object.assign({}, participant), { displayName: cachedDisplayName[participant.id.rawId] });
                        }
                        else if ('teamsAppId' in participant.id && 'rawId' in participant.id && participant.id.rawId && cachedDisplayName[participant.id.rawId]) {
                            return Object.assign(Object.assign({}, participant), { displayName: cachedDisplayName[participant.id.rawId] });
                        }
                        else if ('phoneNumber' in participant.id && 'rawId' in participant.id && participant.id.rawId && cachedDisplayName[participant.id.rawId]) {
                            return Object.assign(Object.assign({}, participant), { displayName: cachedDisplayName[participant.id.rawId] });
                        }
                        else if ('id' in participant.id && cachedDisplayName[participant.id.id]) {
                            return Object.assign(Object.assign({}, participant), { displayName: cachedDisplayName[participant.id.id] });
                        }
                        else {
                            return participant;
                        }
                    }
                    return participant;
                });
                originalChatMessage.content = Object.assign(Object.assign({}, originalChatMessage.content), { participants: newParticipants });
            }
            if (originalChatMessage.sender && originalChatMessage.senderDisplayName) {
                if (originalChatMessage.sender.kind === 'communicationUser' && originalChatMessage.sender.communicationUserId && cachedDisplayName[originalChatMessage.sender.communicationUserId]) {
                    originalChatMessage.senderDisplayName = cachedDisplayName[originalChatMessage.sender.communicationUserId];
                }
                else if (originalChatMessage.sender.kind === 'microsoftTeamsUser' && originalChatMessage.sender.rawId && cachedDisplayName[originalChatMessage.sender.rawId]) {
                    originalChatMessage.senderDisplayName = cachedDisplayName[originalChatMessage.sender.rawId];
                }
                else if (originalChatMessage.sender.kind === 'phoneNumber' && originalChatMessage.sender.phoneNumber && cachedDisplayName[originalChatMessage.sender.phoneNumber]) {
                    originalChatMessage.senderDisplayName = cachedDisplayName[originalChatMessage.sender.phoneNumber];
                }
                else if (originalChatMessage.sender.kind === 'unknown' && originalChatMessage.sender.id && cachedDisplayName[originalChatMessage.sender.id]) {
                    originalChatMessage.senderDisplayName = cachedDisplayName[originalChatMessage.sender.id];
                }
                else if (originalChatMessage.sender.kind === 'microsoftTeamsApp' && originalChatMessage.sender.rawId && cachedDisplayName[originalChatMessage.sender.rawId]) {
                    originalChatMessage.senderDisplayName = cachedDisplayName[originalChatMessage.sender.rawId];
                }
            }
            return Object.assign({}, originalChatMessage);
        });
        return chatMessagesModifier(modifiedParticipantState);
    };
};
/**
 * @private
 * This is the util function to create a participant modifier for remote participantList
 * It memoize previous original participant items and only update the changed participant
 * It takes in one modifier function to generate one single participant object, it returns undefined if the object keeps unmodified
 */
const createParticipantModifier$1 = (createModifiedParticipant) => {
    let previousParticipantState = undefined;
    let modifiedParticipants = {};
    const memoizedParticipants = {};
    return (state) => {
        var _a, _b, _c, _d;
        // if root state is the same, we don't need to update the participants
        if (((_a = state.thread) === null || _a === void 0 ? void 0 : _a.participants) !== previousParticipantState) {
            modifiedParticipants = {};
            const originalParticipants = Object.entries(((_b = state.thread) === null || _b === void 0 ? void 0 : _b.participants) || {});
            for (const [key, originalParticipant] of originalParticipants) {
                const modifiedParticipant = createModifiedParticipant(key, originalParticipant);
                if (modifiedParticipant === undefined) {
                    modifiedParticipants[key] = originalParticipant;
                    continue;
                }
                // Generate the new item if original cached item has been changed
                if (((_c = memoizedParticipants[key]) === null || _c === void 0 ? void 0 : _c.originalRef) !== originalParticipant) {
                    memoizedParticipants[key] = {
                        newParticipant: modifiedParticipant,
                        originalRef: originalParticipant
                    };
                }
                // the modified participant is always coming from the memoized cache, whether is was refreshed
                // from the previous closure or not
                const memoizedParticipant = memoizedParticipants[key];
                if (!memoizedParticipant) {
                    throw new Error('Participant modifier encountered an unhandled exception.');
                }
                modifiedParticipants[key] = memoizedParticipant.newParticipant;
            }
            previousParticipantState = (_d = state.thread) === null || _d === void 0 ? void 0 : _d.participants;
        }
        return Object.assign(Object.assign({}, state), { thread: Object.assign(Object.assign({}, state.thread), { participants: modifiedParticipants }) });
    };
};
/**
 * @private
 * This is the util function to create a chat message modifier for remote participantList
 * It memoize previous original messages and only update the changed sender display name
 * It takes in one modifier function to generate one single participant object, it returns undefined if the object keeps unmodified
 */
const createChatMessageModifier = (createModifiedChatMessage) => {
    let previousChatMessages;
    let modifiedChatMessages;
    const memoizedChatMessages = {};
    return (state) => {
        var _a, _b, _c, _d;
        if (((_a = state.thread) === null || _a === void 0 ? void 0 : _a.chatMessages) !== previousChatMessages) {
            modifiedChatMessages = {};
            const originalChatMessages = Object.entries(((_b = state.thread) === null || _b === void 0 ? void 0 : _b.chatMessages) || {});
            for (const [key, originalChatMessage] of originalChatMessages) {
                const modifiedChatMessage = createModifiedChatMessage(key, originalChatMessage);
                if (modifiedChatMessage === undefined) {
                    modifiedChatMessages[key] = originalChatMessage;
                    continue;
                }
                // Generate the new item if original cached item has been changed
                if (((_c = memoizedChatMessages[key]) === null || _c === void 0 ? void 0 : _c.originalRef) !== originalChatMessage) {
                    memoizedChatMessages[key] = {
                        newChatMessage: modifiedChatMessage,
                        originalRef: originalChatMessage
                    };
                }
                // the modified chat message is always coming from the memoized cache, whether is was refreshed
                // from the previous closure or not
                const memoizedChatMessage = memoizedChatMessages[key];
                if (!memoizedChatMessage) {
                    throw new Error('Participant modifier encountered an unhandled exception.');
                }
                modifiedChatMessages[key] = memoizedChatMessage.newChatMessage;
            }
            previousChatMessages = (_d = state.thread) === null || _d === void 0 ? void 0 : _d.chatMessages;
        }
        return Object.assign(Object.assign({}, state), { thread: Object.assign(Object.assign({}, state.thread), { chatMessages: modifiedChatMessages }) });
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$t = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (window && window.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
/**
 * Context of Chat, which is a centralized context for all state updates
 * @private
 */
class ChatContext {
    constructor(clientState, threadId, chatAdapterOptions) {
        this.emitter = new events.EventEmitter();
        const thread = clientState.threads[threadId];
        this.threadId = threadId;
        if (!thread) {
            throw 'Cannot find threadId, please initialize thread before use!';
        }
        this.state = {
            userId: clientState.userId,
            displayName: clientState.displayName,
            thread,
            latestErrors: clientState.latestErrors
        };
        this.displayNameModifier = (chatAdapterOptions === null || chatAdapterOptions === void 0 ? void 0 : chatAdapterOptions.onFetchProfile) ? createProfileStateModifier$1(chatAdapterOptions.onFetchProfile, () => {
            this.setState(this.getState());
        }) : undefined;
    }
    onStateChange(handler) {
        this.emitter.on('stateChanged', handler);
    }
    offStateChange(handler) {
        this.emitter.off('stateChanged', handler);
    }
    setState(state) {
        this.state = state;
        this.state = this.displayNameModifier ? this.displayNameModifier(state) : state;
        this.emitter.emit('stateChanged', this.state);
    }
    getState() {
        return this.state;
    }
    setError(error) {
        this.setState(Object.assign(Object.assign({}, this.state), { error }));
    }
    updateClientState(clientState) {
        const thread = clientState.threads[this.threadId];
        if (!thread) {
            throw 'Cannot find threadId, please make sure thread state is still in Stateful ChatClient.';
        }
        let updatedState = {
            userId: clientState.userId,
            displayName: clientState.displayName,
            thread,
            latestErrors: clientState.latestErrors
        };
        this.setState(updatedState);
    }
}
/**
 * @private
 */
class AzureCommunicationChatAdapter {
    constructor(chatClient, chatThreadClient, chatAdapterOptions) {
        this.emitter = new events.EventEmitter();
        this.bindAllPublicMethods();
        this.chatClient = chatClient;
        this.chatThreadClient = chatThreadClient;
        this.context = new ChatContext(chatClient.getState(), chatThreadClient.threadId, chatAdapterOptions);
        const onStateChange = (clientState) => {
            // unsubscribe when the instance gets disposed
            if (!this) {
                chatClient.offStateChange(onStateChange);
                return;
            }
            this.context.updateClientState(clientState);
        };
        this.handlers = createDefaultChatHandlers(chatClient, chatThreadClient);
        this.chatClient.onStateChange(onStateChange);
        this.subscribeAllEvents();
    }
    bindAllPublicMethods() {
        this.onStateChange = this.onStateChange.bind(this);
        this.offStateChange = this.offStateChange.bind(this);
        this.getState = this.getState.bind(this);
        this.dispose = this.dispose.bind(this);
        this.fetchInitialData = this.fetchInitialData.bind(this);
        this.sendMessage = this.sendMessage.bind(this);
        this.sendReadReceipt = this.sendReadReceipt.bind(this);
        this.sendTypingIndicator = this.sendTypingIndicator.bind(this);
        this.updateMessage = this.updateMessage.bind(this);
        this.deleteMessage = this.deleteMessage.bind(this);
        this.removeParticipant = this.removeParticipant.bind(this);
        this.setTopic = this.setTopic.bind(this);
        this.loadPreviousChatMessages = this.loadPreviousChatMessages.bind(this);
        this.on = this.on.bind(this);
        this.off = this.off.bind(this);
        this.downloadResourceToCache = this.downloadResourceToCache.bind(this);
        this.removeResourceFromCache = this.removeResourceFromCache.bind(this);
    }
    dispose() {
        this.unsubscribeAllEvents();
        this.chatClient.dispose();
    }
    fetchInitialData() {
        return __awaiter$t(this, void 0, void 0, function* () {
            // If get properties fails we dont want to try to get the participants after.
            yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                var _a, e_1, _b, _c;
                yield this.chatThreadClient.getProperties();
                try {
                    // Fetch all participants who joined before the local user.
                    for (var _d = true, _e = __asyncValues(this.chatThreadClient.listParticipants().byPage({
                        // Fetch 100 participants per page by default.
                        maxPageSize: 100
                    })), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                        _c = _f.value;
                        _d = false;
                        const _page = _c;
                        ;
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }));
        });
    }
    getState() {
        return this.context.getState();
    }
    onStateChange(handler) {
        this.context.onStateChange(handler);
    }
    offStateChange(handler) {
        this.context.offStateChange(handler);
    }
    sendMessage(content, options) {
        return __awaiter$t(this, void 0, void 0, function* () {
            yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                return yield this.handlers.onSendMessage(content, options);
            }));
        });
    }
    sendReadReceipt(chatMessageId) {
        return __awaiter$t(this, void 0, void 0, function* () {
            yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                yield this.handlers.onMessageSeen(chatMessageId);
            }));
        });
    }
    sendTypingIndicator() {
        return __awaiter$t(this, void 0, void 0, function* () {
            yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                yield this.handlers.onTyping();
            }));
        });
    }
    removeParticipant(userId) {
        return __awaiter$t(this, void 0, void 0, function* () {
            yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                yield this.handlers.onRemoveParticipant(userId);
            }));
        });
    }
    setTopic(topicName) {
        return __awaiter$t(this, void 0, void 0, function* () {
            yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                yield this.handlers.updateThreadTopicName(topicName);
            }));
        });
    }
    loadPreviousChatMessages(messagesToLoad) {
        return __awaiter$t(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                return yield this.handlers.onLoadPreviousChatMessages(messagesToLoad);
            }));
        });
    }
    updateMessage(messageId, content, options) {
        return __awaiter$t(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                // metadata is never used in the current stable
                return yield this.handlers.onUpdateMessage(messageId, content);
            }));
        });
    }
    deleteMessage(messageId) {
        return __awaiter$t(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$t(this, void 0, void 0, function* () {
                return yield this.handlers.onDeleteMessage(messageId);
            }));
        });
    }
    downloadResourceToCache(resourceDetails) {
        return __awaiter$t(this, void 0, void 0, function* () {
            this.chatClient.downloadResourceToCache(resourceDetails.threadId, resourceDetails.messageId, resourceDetails.resourceUrl);
        });
    }
    removeResourceFromCache(resourceDetails) {
        this.chatClient.removeResourceFromCache(resourceDetails.threadId, resourceDetails.messageId, resourceDetails.resourceUrl);
    }
    messageReceivedListener(event) {
        const isCurrentChatAdapterThread = event.threadId === this.chatThreadClient.threadId;
        if (!isCurrentChatAdapterThread) {
            return;
        }
        const message = convertEventToChatMessage(event);
        this.emitter.emit('messageReceived', {
            message
        });
        const currentUserId = toFlatCommunicationIdentifier(this.chatClient.getState().userId);
        if ((message === null || message === void 0 ? void 0 : message.sender) && toFlatCommunicationIdentifier(message.sender) === currentUserId) {
            this.emitter.emit('messageSent', {
                message
            });
        }
    }
    messageEditedListener(event) {
        const isCurrentChatAdapterThread = event.threadId === this.chatThreadClient.threadId;
        if (!isCurrentChatAdapterThread) {
            return;
        }
        const message = convertEventToChatMessage(event);
        this.emitter.emit('messageEdited', {
            message
        });
    }
    messageDeletedListener(event) {
        const isCurrentChatAdapterThread = event.threadId === this.chatThreadClient.threadId;
        if (!isCurrentChatAdapterThread) {
            return;
        }
        const message = convertEventToChatMessage(event);
        this.emitter.emit('messageDeleted', {
            message
        });
    }
    messageReadListener({ chatMessageId, recipient }) {
        const message = this.getState().thread.chatMessages[chatMessageId];
        if (message) {
            this.emitter.emit('messageRead', {
                message,
                readBy: recipient
            });
        }
    }
    participantsAddedListener({ addedBy, participantsAdded }) {
        this.emitter.emit('participantsAdded', {
            addedBy,
            participantsAdded
        });
    }
    participantsRemovedListener({ removedBy, participantsRemoved }) {
        this.emitter.emit('participantsRemoved', {
            removedBy,
            participantsRemoved
        });
    }
    chatThreadPropertiesUpdatedListener(event) {
        this.emitter.emit('topicChanged', {
            topic: event.properties.topic
        });
    }
    subscribeAllEvents() {
        this.chatClient.on('chatThreadPropertiesUpdated', this.chatThreadPropertiesUpdatedListener.bind(this));
        this.chatClient.on('participantsAdded', this.participantsAddedListener.bind(this));
        this.chatClient.on('participantsRemoved', this.participantsRemovedListener.bind(this));
        this.chatClient.on('chatMessageReceived', this.messageReceivedListener.bind(this));
        this.chatClient.on('chatMessageEdited', this.messageEditedListener.bind(this));
        this.chatClient.on('chatMessageDeleted', this.messageDeletedListener.bind(this));
        this.chatClient.on('readReceiptReceived', this.messageReadListener.bind(this));
        this.chatClient.on('participantsRemoved', this.participantsRemovedListener.bind(this));
    }
    unsubscribeAllEvents() {
        this.chatClient.off('chatThreadPropertiesUpdated', this.chatThreadPropertiesUpdatedListener.bind(this));
        this.chatClient.off('participantsAdded', this.participantsAddedListener.bind(this));
        this.chatClient.off('participantsRemoved', this.participantsRemovedListener.bind(this));
        this.chatClient.off('chatMessageReceived', this.messageReceivedListener.bind(this));
        this.chatClient.off('chatMessageEdited', this.messageEditedListener.bind(this));
        this.chatClient.off('chatMessageDeleted', this.messageDeletedListener.bind(this));
        this.chatClient.off('readReceiptReceived', this.messageReadListener.bind(this));
        this.chatClient.off('participantsRemoved', this.participantsRemovedListener.bind(this));
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    on(event, listener) {
        this.emitter.on(event, listener);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    off(event, listener) {
        this.emitter.off(event, listener);
    }
    asyncTeeErrorToEventEmitter(f) {
        return __awaiter$t(this, void 0, void 0, function* () {
            try {
                return yield f();
            }
            catch (error) {
                if (isChatError(error)) {
                    this.emitter.emit('error', error);
                }
                throw error;
            }
        });
    }
}
const convertEventToChatMessage = (event) => {
    return {
        id: event.id,
        version: event.version,
        content: isChatMessageDeletedEvent(event) ? undefined : {
            message: event.message
        },
        type: convertEventType(event.type),
        sender: event.sender,
        senderDisplayName: event.senderDisplayName,
        sequenceId: '',
        createdOn: new Date(event.createdOn),
        editedOn: isChatMessageEditedEvent(event) ? event.editedOn : undefined,
        deletedOn: isChatMessageDeletedEvent(event) ? event.deletedOn : undefined
    };
};
const isChatMessageEditedEvent = (event) => {
    return 'editedOn' in event;
};
const isChatMessageDeletedEvent = (event) => {
    return 'deletedOn' in event;
};
// only text/html message type will be received from event
const convertEventType = (type) => {
    const lowerCaseType = type.toLowerCase();
    if (lowerCaseType === 'richtext/html' || lowerCaseType === 'html') {
        return 'html';
    }
    else {
        return 'text';
    }
};
/**
 * Create a {@link ChatAdapter} backed by Azure Communication Services.
 *
 * This is the default implementation of {@link ChatAdapter} provided by this library.
 *
 * @public
 */
const createAzureCommunicationChatAdapter = (_a) => __awaiter$t(void 0, [_a], void 0, function* ({ endpoint: endpointUrl, userId, displayName, credential, threadId, chatAdapterOptions }) {
    return _createAzureCommunicationChatAdapterInner(endpointUrl, userId, displayName, credential, threadId, 'Chat', chatAdapterOptions);
});
/**
 * This inner function is used to allow injection of TelemetryImplementationHint without changing the public API.
 *
 * @internal
 */
const _createAzureCommunicationChatAdapterInner = function (endpoint_1, userId_1, displayName_1, credential_1, threadId_1) {
    return __awaiter$t(this, arguments, void 0, function* (endpoint, userId, displayName, credential, threadId, telemetryImplementationHint = 'Chat', chatAdapterOptions) {
        if (!_isValidIdentifier(userId)) {
            throw new Error('Provided userId is invalid. Please provide valid identifier object.');
        }
        const chatClient = _createStatefulChatClientInner({
            userId,
            displayName,
            endpoint,
            credential
        }, undefined, telemetryImplementationHint);
        const chatThreadClient = yield chatClient.getChatThreadClient(threadId);
        yield chatClient.startRealtimeNotifications();
        const adapter = yield createAzureCommunicationChatAdapterFromClient(chatClient, chatThreadClient, chatAdapterOptions);
        return adapter;
    });
};
/**
 * This inner function to create ChatAdapterPromise in case when threadID is not avaialble.
 * ThreadId is a promise to allow for lazy initialization of the adapter.
 * @internal
 */
const _createLazyAzureCommunicationChatAdapterInner = function (endpoint_1, userId_1, displayName_1, credential_1, threadId_1) {
    return __awaiter$t(this, arguments, void 0, function* (endpoint, userId, displayName, credential, threadId, telemetryImplementationHint = 'Chat', chatAdapterOptions) {
        if (!_isValidIdentifier(userId)) {
            throw new Error('Provided userId is invalid. Please provide valid identifier object.');
        }
        const chatClient = _createStatefulChatClientInner({
            userId,
            displayName,
            endpoint,
            credential
        }, undefined, telemetryImplementationHint);
        return threadId.then((threadId) => __awaiter$t(this, void 0, void 0, function* () {
            if (UNSUPPORTED_CHAT_THREAD_TYPE.some(t => threadId.includes(t))) {
                console.error(`Invalid Chat ThreadId: ${threadId}. Please note with Teams Channel Meetings, only Calling is supported and Chat is not currently supported. Read more: ${TEAMS_LIMITATION_LEARN_MORE}.`);
            }
            const chatThreadClient = yield chatClient.getChatThreadClient(threadId);
            yield chatClient.startRealtimeNotifications();
            const adapter = yield createAzureCommunicationChatAdapterFromClient(chatClient, chatThreadClient, chatAdapterOptions);
            return adapter;
        }));
    });
};
/**
 * A custom React hook to simplify the creation of {@link ChatAdapter}.
 *
 * Similar to {@link createAzureCommunicationChatAdapter}, but takes care of asynchronous
 * creation of the adapter internally.
 *
 * Allows arguments to be undefined so that you can respect the rule-of-hooks and pass in arguments
 * as they are created. The adapter is only created when all arguments are defined.
 *
 * Note that you must memoize the arguments to avoid recreating adapter on each render.
 * See storybook for typical usage examples.
 *
 * @public
 */
const useAzureCommunicationChatAdapter = (args, afterCreate, beforeDispose) => {
    const { credential, displayName, endpoint, threadId, userId } = args;
    // State update needed to rerender the parent component when a new adapter is created.
    const [adapter, setAdapter] = React.useState(undefined);
    // Ref needed for cleanup to access the old adapter created asynchronously.
    const adapterRef = React.useRef(undefined);
    const creatingAdapterRef = React.useRef(false);
    const afterCreateRef = React.useRef(undefined);
    const beforeDisposeRef = React.useRef(undefined);
    // These refs are updated on *each* render, so that the latest values
    // are used in the `useEffect` closures below.
    // Using a Ref ensures that new values for the callbacks do not trigger the
    // useEffect blocks, and a new adapter creation / distruction is not triggered.
    afterCreateRef.current = afterCreate;
    beforeDisposeRef.current = beforeDispose;
    React.useEffect(() => {
        if (!credential || !displayName || !endpoint || !threadId || !userId) {
            return;
        }
        (() => __awaiter$t(void 0, void 0, void 0, function* () {
            if (adapterRef.current) {
                // Dispose the old adapter when a new one is created.
                //
                // This clean up function uses `adapterRef` because `adapter` can not be added to the dependency array of
                // this `useEffect` -- we do not want to trigger a new adapter creation because of the first adapter
                // creation.
                if (beforeDisposeRef.current) {
                    yield beforeDisposeRef.current(adapterRef.current);
                }
                adapterRef.current.dispose();
                adapterRef.current = undefined;
            }
            if (creatingAdapterRef.current) {
                console.warn('Adapter is already being created, please see storybook for more information: https://azure.github.io/communication-ui-library/?path=/story/troubleshooting--page');
                return;
            }
            creatingAdapterRef.current = true;
            let newAdapter = yield createAzureCommunicationChatAdapter({
                credential,
                displayName,
                endpoint,
                threadId,
                userId
            });
            if (afterCreateRef.current) {
                newAdapter = yield afterCreateRef.current(newAdapter);
            }
            adapterRef.current = newAdapter;
            creatingAdapterRef.current = false;
            setAdapter(newAdapter);
        }))();
    }, 
    // Explicitly list all arguments so that caller doesn't have to memoize the `args` object.
    [adapterRef, afterCreateRef, beforeDisposeRef, credential, displayName, endpoint, threadId, userId]);
    // Dispose any existing adapter when the component unmounts.
    React.useEffect(() => {
        return () => {
            (() => __awaiter$t(void 0, void 0, void 0, function* () {
                if (adapterRef.current) {
                    if (beforeDisposeRef.current) {
                        yield beforeDisposeRef.current(adapterRef.current);
                    }
                    adapterRef.current.dispose();
                    adapterRef.current = undefined;
                }
            }))();
        };
    }, []);
    return adapter;
};
/**
 * Create a {@link ChatAdapter} using the provided {@link StatefulChatClient}.
 *
 * Useful if you want to keep a reference to {@link StatefulChatClient}.
 * Consider using {@link createAzureCommunicationChatAdapter} for a simpler API.
 *
 * @public
 */
function createAzureCommunicationChatAdapterFromClient(chatClient, chatThreadClient, chatAdapterOptions) {
    return __awaiter$t(this, void 0, void 0, function* () {
        return new AzureCommunicationChatAdapter(chatClient, chatThreadClient, chatAdapterOptions);
    });
}
const isChatError = (e) => {
    return 'target' in e && e['target'] !== undefined && 'innerError' in e && e['innerError'] !== undefined;
};

var call$n={cameraLabel:"Camera",noCamerasLabel:"No cameras found",cameraPermissionDenied:"Your browser is blocking access to your camera",cameraTurnedOff:"Your camera is turned off",chatButtonLabel:"Chat",close:"Close",complianceBannerNowOnlyRecording:"You are now only recording this meeting.",complianceBannerNowOnlyTranscription:"You are now only transcribing this meeting.",complianceBannerRecordingAndTranscriptionSaved:"Recording and transcription are being saved.",complianceBannerRecordingAndTranscriptionStarted:"Recording and transcription have started.",complianceBannerRecordingAndTranscriptionStopped:"Recording and transcription have stopped.",complianceBannerRecordingSaving:"Recording is being saved.",complianceBannerRecordingStarted:"Recording has started.",complianceBannerRecordingStopped:"Recording has stopped.",complianceBannerTranscriptionStarted:"Transcription has started.",complianceBannerTranscriptionConsent:"By joining, you are giving consent for this meeting to be transcribed.",complianceBannerTranscriptionSaving:"Transcription is being saved.",complianceBannerTranscriptionStopped:"Transcription has stopped.",configurationPageTitle:"Start a call",copyInviteLinkButtonLabel:"Copy invite link",copyInviteLinkButtonActionedLabel:"Link copied",copyInviteLinkActionedAriaLabel:"Invite link copied",defaultPlaceHolder:"Select an option",dismissSidePaneButtonLabel:"Close",videoEffectsPaneTitle:"Effects",videoEffectsPaneBackgroundSelectionTitle:"Background",videoEffectsPaneAriaLabel:"Video effects pane",configurationPageCameraIsLoadingLabel:"Video is loading...",configurationPageVideoEffectsButtonLabel:"Effects",unableToStartVideoEffect:"Unable to apply video effect.",blurBackgroundEffectButtonLabel:"Blur",blurBackgroundTooltip:"Blur Background",removeBackgroundEffectButtonLabel:"None",removeBackgroundTooltip:"Remove Background",cameraOffBackgroundEffectWarningText:"Your camera is off. Turn on camera to see video effect.",failedToJoinCallDueToNoNetworkMoreDetails:"Call was disconnected due to a network issue. Check your connection and join again.",failedToJoinCallDueToNoNetworkTitle:"Call disconnected",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"You were not granted entry in the call. If this was a mistake, re-join the call.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Dismissed from lobby",learnMore:"Learn more",leavingCallTitle:"Leaving...",leftCallMoreDetails:"If this was a mistake, re-join the call.",leftCallTitle:"You left the call",lobbyScreenConnectingToCallTitle:"Joining call",lobbyScreenWaitingToBeAdmittedTitle:"Waiting to be admitted",microphonePermissionDenied:"Your browser is blocking access to your microphone",microphoneToggleInLobbyNotAllowed:"Cannot mute or unmute while in lobby.",mutedMessage:"You're muted",networkReconnectMoreDetails:"Looks like something went wrong. We're trying to get back into the call.",networkReconnectTitle:"Hold on",notInvitedToRoomDetails:"You are not able to join this room because you do not have an invite.",notInvitedToRoomTitle:"Not invited to room",peopleButtonLabel:"People",selectedPeopleButtonLabel:"People Button Selected",peoplePaneTitle:"People",peoplePaneMoreButtonAriaLabel:"More",peopleButtonTooltipOpen:"Show participants",peopleButtonTooltipClose:"Hide participants",peoplePaneSubTitle:"In this call {numberOfPeople}",privacyPolicy:"Privacy policy",rejoinCallButtonLabel:"Re-join call",removedFromCallMoreDetails:"Another participant removed you from the call.",removedFromCallTitle:"You were removed",removeMenuLabel:"Remove",startSpotlightMenuLabel:"Spotlight for everyone",addSpotlightMenuLabel:"Add spotlight",stopSpotlightMenuLabel:"Stop spotlighting",stopSpotlightOnSelfMenuLabel:"Exit spotlight",spotlightLimitReachedMenuTitle:"Spotlight limit reached",stopAllSpotlightMenuLabel:"Stop all spotlights",returnToCallButtonAriaDescription:"Return to Call",returnToCallButtonAriaLabel:"Back",roomNotFoundDetails:"The room ID provided could not be found.",roomNotFoundTitle:"Room not found",roomNotValidDetails:"This room is not valid at this current time.",roomNotValidTitle:"Room not valid",inviteToRoomRemovedDetails:"Your invite to join this room was removed.",inviteToRoomRemovedTitle:"Invite to room removed",soundLabel:"Sound",noMicrophonesLabel:"No microphones found",noSpeakersLabel:"No speakers found",startCallButtonLabel:"Start call",openDialpadButtonLabel:"Dial phone number",peoplePaneAddPeopleButtonLabel:"Add People",dialpadStartCallButtonLabel:"Call",dialpadModalTitle:"Dial Phone Number",dialpadModalAriaLabel:"Dialpad",dialpadCloseModalButtonAriaLabel:"Close dialpad",moreButtonCallingLabel:"More",resumeCallButtonLabel:"Resume",resumingCallButtonLabel:"Resuming...",resumeCallButtonAriaLabel:"Resume call",resumingCallButtonAriaLabel:"Resume call",holdScreenLabel:"You're on hold",openDtmfDialpadLabel:"Show dialpad",dtmfDialpadPlaceholderText:"Enter number",outboundCallingNoticeString:"Calling...",participantJoinedNoticeString:"{displayName} joined",twoParticipantJoinedNoticeString:"{displayName1} and {displayName2} have joined",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} and {displayName3} have joined",participantLeftNoticeString:"{displayName} left",twoParticipantLeftNoticeString:"{displayName1} and {displayName2} have left",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} and {displayName3} have left",unnamedParticipantString:"unnamed participant",manyUnnamedParticipantsJoined:"unnamed participant and {numOfParticipants} other participants joined",manyUnnamedParticipantsLeft:"unnamed participant and {numOfParticipants} other participants left",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} and {numOfParticipants} other participants joined",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} and {numOfParticipants} other participants left",liveCaptionsLabel:"Live captions",captionsSettingsLabel:"Caption settings",startCaptionsButtonOnLabel:"Turn on captions",startCaptionsButtonOffLabel:"Turn off captions",startCaptionsButtonTooltipOnContent:"Turn off captions",startCaptionsButtonTooltipOffContent:"Turn on captions",captionsSettingsModalTitle:"Captions Settings",captionsSettingsSpokenLanguageDropdownLabel:"Spoken language",captionsSettingsCaptionLanguageDropdownLabel:"Captions language",captionsSettingsSpokenLanguageDropdownInfoText:"Language that everyone on this call is speaking.",captionsSettingsCaptionLanguageDropdownInfoText:"Captions will appear in this language.",captionsSettingsConfirmButtonLabel:"Confirm",captionsSettingsCancelButtonLabel:"Cancel",captionsSettingsModalAriaLabel:"Captions Setting Modal",captionsSettingsCloseModalButtonAriaLabel:"Close Captions Setting",captionsBannerMoreButtonCallingLabel:"Captions Settings More Options",captionsBannerMoreButtonTooltip:"More options",realTimeTextLabel:"Real-time text (RTT)",startRealTimeTextLabel:"Turn on RTT for this call",disabledStartRealTimeTextLabel:"RTT is currently turned on, it cannot be turned off",dismissModalAriaLabel:"Local and remote picture and picture, press enter to return to call",callRejectedTitle:"The call could not be completed",callRejectedMoreDetails:"The person you are trying to reach is unavailable. Please try again later.",callTimeoutTitle:"The call could not be completed",callTimeoutDetails:"The person you are trying to reach is unavailable. Please try again later.",callTimeoutBotTitle:"The call could not be completed",callTimeoutBotDetails:"The service that you are trying to reach is unavailable. Please try again later.",dtmfDialerButtonLabel:"Dialpad",dtmfDialerButtonTooltipOn:"Show dialpad",dtmfDialerButtonTooltipOff:"Hide dialpad",dtmfDialerMoreButtonLabelOn:"Show dialpad",dtmfDialerMoreButtonLabelOff:"Hide dialpad",phoneCallMoreButtonLabel:"Phone Call",spokenLanguageStrings:{"ar-ae":"Arabic - U.A.E.","ar-sa":"Arabic - Saudi Arabia","da-dk":"Danish","de-de":"German - Germany","en-au":"English - Australia","en-ca":"English - Canada","en-gb":"English - United Kingdom","en-in":"English - India","en-nz":"English - New Zealand","en-us":"English - United States","es-es":"Spanish - Spain (Modern Sort)","es-mx":"Spanish - Mexico","fi-fi":"Finnish","fr-ca":"French - Canada","fr-fr":"French - France","hi-in":"Hindi","it-it":"Italian - Italy","ja-jp":"Japanese","ko-kr":"Korean","nb-no":"Norwegian (Bokmål)","nl-be":"Dutch - Belgium","nl-nl":"Dutch - Netherlands","pl-pl":"Polish","pt-br":"Portuguese - Brazil","ru-ru":"Russian","sv-se":"Swedish","zh-cn":"Chinese - People's Republic of China","zh-hk":"Chinese - Hong Kong SAR","cs-cz":"Czech","pt-pt":"Portuguese - Portugal","tr-tr":"Turkish","vi-vn":"Vietnamese","th-th":"Thai","he-il":"Hebrew","cy-gb":"Welsh","uk-ua":"Ukrainian","el-gr":"Greek","hu-hu":"Hungarian","ro-ro":"Romanian","sk-sk":"Slovak","zh-tw":"Chinese - Taiwan"},captionLanguageStrings:{ar:"Arabic",da:"Danish",de:"German",en:"English",es:"Spanish",fi:"Finnish","fr-ca":"French - Canada",fr:"French - France",hi:"Hindi",it:"Italian",ja:"Japanese",ko:"Korean",nb:"Norwegian (Bokmål)",nl:"Dutch",pl:"Polish",pt:"Portuguese - Brazil",ru:"Russian",sv:"Swedish","zh-Hans":"Chinese (Simplified)","zh-Hant":"Chinese (Traditional)",cs:"Czech","pt-pt":"Portuguese - Portugal",tr:"Turkish",vi:"Vietnamese",th:"Thai",he:"Hebrew",cy:"Welsh",uk:"Ukrainian",el:"Greek",hu:"Hungarian",ro:"Romanian",sk:"Slovak"},captionsBannerSpinnerText:"Starting captions...",realTimeTextInputBoxDefaultText:"Type message in real-time",realTimeTextInputErrorMessage:"Message can not exceed 2000 characters",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (real-time text) is enabled for all participants for the entire duration of the meeting.",realTimeTextBannerLinkLabel:"Learn more",realTimeTextModalAriaLabel:"Turn on RTT Modal",realTimeTextModalTitle:"Turn on RTT?",realTimeTextModalText:"Real-time text (RTT) will be turned on for everyone in the meeting.",realTimeTextConfirmButtonLabel:"Turn on",realTimeTextCancelButtonLabel:"Cancel",realTimeTextCloseModalButtonAriaLabel:"Close RTT Modal",captionsOnlyContainerTitle:"Captions",realTimeTextOnlyContainerTitle:"Real-Time Text",captionsAndRealTimeTextContainerTitle:"Captions and Real-Time Text",expandButtonAriaLabel:"Expand",minimizeButtonAriaLabel:"Collapse",transferPageTransferorText:"Transferring...",transferPageTransferTargetText:"Connecting...",transferPageUnknownTransferorDisplayName:"Unknown",transferPageUnknownTransferTargetDisplayName:"Unknown",transferPageNoticeString:"You are being transferred",participantCouldNotBeReachedTitle:"Target participant is currently not available",participantCouldNotBeReachedMoreDetails:"Please contact participant when they are available",permissionToReachTargetParticipantNotAllowedTitle:"Permission to reach target participant is not allowed",permissionToReachTargetParticipantNotAllowedMoreDetails:"Please check that the target participant is in the same tenant",unableToResolveTenantTitle:"Unable to resolve tenant id for the target participant",unableToResolveTenantMoreDetails:"Please check the participant id is entered correctly",participantIdIsMalformedTitle:"Participant id is not in the correct format",participantIdIsMalformedMoreDetails:"Please check that the participant id is in the correct format",moreButtonGalleryControlLabel:"View",moreButtonGalleryPositionToggleLabel:"Move gallery to top",moreButtonGallerySpeakerLayoutLabel:"Speaker",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamic",moreButtonGalleryDefaultLayoutLabel:"Gallery view",moreButtonGalleryFocusedContentLayoutLabel:"Focus on content",moreButtonLargeGalleryDefaultLayoutLabel:"Large Gallery",moreButtonTogetherModeLayoutLabel:"Together mode",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Your camera has been disabled. You can no longer share video.",grantedDueToMeetingOption:"Your camera has been enabled. Turn it on if you'd like to share video."},unmuteMic:{lostDueToMeetingOption:"Your mic has been disabled. You can no longer unmute.",grantedDueToMeetingOption:"Your mic has been enabled. Unmute if you'd like to speak."},shareScreen:{lostDueToRoleChangeToAttendee:"Your role has been changed. Some actions, like sharing content, won't be available to you.",grantedDueToRoleChangeToPresenter:"You're a presenter. You can share content and facilitate the meeting."},hideAttendeeNames:{hideAttendeeNameAttendee:"Your name is hidden from other attendees. Organizers and presenters can see your real name.",hideAttendeeNamePresenter:"Attendee names are hidden. Only organizers and presenters can see names to protect attendee privacy."}},surveyTitle:"Help us improve",starSurveyHelperText:"How was the quality of the call?",starSurveyOneStarText:"The quality was bad.",starSurveyTwoStarText:"The quality was poor.",starSurveyThreeStarText:"The quality was good.",starSurveyFourStarText:"The quality was great.",starSurveyFiveStarText:"The quality was excellent.",surveyConfirmButtonLabel:"Send Feedback",starRatingAriaLabel:"Select {0} of {1} stars",surveySkipButtonLabel:"Skip",tagsSurveyQuestion:"What could have been better?",tagsSurveyTextFieldDefaultText:"Other, please specify",tagsSurveyHelperText:"Check any issues you experienced",endOfSurveyText:"Thank you for your feedback!",surveyIssues:{overallRating:{callCannotJoin:"I could not join call",callCannotInvite:"I could not invite others into the call",hadToRejoin:"I had to rejoin the call",callEndedUnexpectedly:"Call ended for me unexpectedly",otherIssues:"I was having other issues with the call"},audioRating:{noLocalAudio:"The other side could not hear any sound",noRemoteAudio:"I could not hear any sound",echo:"I heard echos on the call",audioNoise:"I heard noise on the call",lowVolume:"Volume was low",audioStoppedUnexpectedly:"Audio stopped unexpectedly",distortedSpeech:"Audio was distorted",audioInterruption:"Audio was interrupted",otherIssues:"I was having other audio issues in this call"},videoRating:{noVideoReceived:"I could not see any video",noVideoSent:"Others could not see me",lowQuality:"Video quality was low",freezes:"Video frozen",stoppedUnexpectedly:"Video stopped unexpectedly",darkVideoReceived:"I can only see dark screens when others turn on their camera",audioVideoOutOfSync:"Audio and Video was out of sync",otherIssues:"I was having other video issues in this call"},screenshareRating:{noContentLocal:"Other people could not see my screenshare",noContentRemote:"I could not see other people's screenshare",cannotPresent:"I could not present my screen",lowQuality:"Screen share quality was low",freezes:"Screen share frozen",stoppedUnexpectedly:"Screen share stopped unexpectedly",largeDelay:"Screen share has a large delay",otherIssues:"I was having other screen share issues in this call"}},surveyIssuesHeadingStrings:{overallRating:"Overall",audioRating:"Audio",videoRating:"Video",screenshareRating:"Presenting"},spotlightPrompt:{startSpotlightHeading:"Spotlight for everyone?",startSpotlightText:"You'll highlight this video for everyone in the meeting.",startSpotlightOnSelfText:"You'll highlight your video for everyone in the meeting.",startSpotlightConfirmButtonLabel:"Spotlight for everyone",startSpotlightCancelButtonLabel:"Cancel",stopSpotlightHeading:"Stop spotlighting this video?",stopSpotlightOnSelfHeading:"Exit spotlight?",stopAllSpotlightHeading:"Stop spotlighting all videos?",stopSpotlightText:"This video will no longer be highlighted for everyone in the meeting.",stopSpotlightOnSelfText:"Your video will no longer be highlighted for everyone in the meeting.",stopAllSpotlightText:"The videos will no longer be highlighted for everyone in the meeting.",stopSpotlightConfirmButtonLabel:"Stop spotlighting",stopSpotlightOnSelfConfirmButtonLabel:"Exit spotlight",stopSpotlightCancelButtonLabel:"Cancel",closeSpotlightPromptButtonLabel:"Close"},exitSpotlightButtonLabel:"Exit spotlight",exitSpotlightButtonTooltip:"Exit spotlight",leaveConfirmButtonLabel:"Leave",endCallConfirmButtonLabel:"End call",hangUpCancelButtonLabel:"Cancel",leaveConfirmDialogTitle:"Leave the call?",leaveConfirmDialogContent:"Are you sure you want to leave the call?",endCallConfirmDialogTitle:"End the call?",endCallConfirmDialogContent:"You'll end the call for everyone.",invalidMeetingIdentifier:"Invalid meeting ID or passcode",pinParticipantMenuLabel:"Pin for me",pinParticipantLimitReachedMenuLabel:"Pin (limit reached)",unpinParticipantMenuLabel:"Unpin",pinParticipantMenuItemAriaLabel:"Pin for me",unpinParticipantMenuItemAriaLabel:"Unpin",muteAllMenuLabel:"Mute all",muteAllDialogTitle:"Mute everyone?",muteAllDialogContent:"This will mute everyone in the meeting but you.",muteAllConfirmButtonLabel:"Mute all",muteAllCancelButtonLabel:"Cancel",joinBreakoutRoomButtonLabel:"Join breakout room",returnFromBreakoutRoomButtonLabel:"Leave room",leaveBreakoutRoomAndMeetingButtonLabel:"Leave meeting",breakoutRoomJoinedNotificationTitle:"You've joined Breakout room '{roomName}'",joinBreakoutRoomBannerTitle:"Breakout room '{roomName}' is open",joinBreakoutRoomBannerButtonLabel:"Join",returnFromBreakoutRoomBannerTitle:"Return to main meeting",returnFromBreakoutRoomBannerButtonLabel:"Rejoin",forbidAudioMenuLabel:"Disable mic",permitAudioMenuLabel:"Allow mic",forbidOthersAudioDialogTitle:"Disable mic for attendees?",forbidOthersAudioDialogContent:"Attendees won’t be able to unmute themselves. The organizer and presenters can let people unmute as needed.",forbidOthersAudioConfirmButtonLabel:"Disable mics",forbidOthersAudioCancelButtonLabel:"Cancel",permitOthersAudioDialogTitle:"Allow mic for attendees?",permitOthersAudioDialogContent:"Everyone in the meeting will be able to unmute themselves.",permitOthersAudioConfirmButtonLabel:"Allow mics",permitOthersAudioCancelButtonLabel:"Cancel",forbidOthersAudioMenuLabel:"Disable mic for attendees",permitOthersAudioMenuLabel:"Allow mic for attendees",forbidVideoMenuLabel:"Disable camera",permitVideoMenuLabel:"Allow camera",forbidOthersVideoDialogTitle:"Disable camera for attendees?",forbidOthersVideoDialogContent:"Attendees won’t be able to turn on video themselves. The organizer and presenters can let people turn on camera as needed.",forbidOthersVideoConfirmButtonLabel:"Disable cameras",forbidOthersVideoCancelButtonLabel:"Cancel",permitOthersVideoDialogTitle:"Allow camera for attendees?",permitOthersVideoDialogContent:"Everyone in the meeting will be able to turn on video themselves.",permitOthersVideoConfirmButtonLabel:"Allow cameras",permitOthersVideoCancelButtonLabel:"Cancel",forbidOthersVideoMenuLabel:"Disable camera for attendees",permitOthersVideoMenuLabel:"Allow camera for attendees"};var chat$n={chatListHeader:"In this chat",uploadAttachment:"Upload Attachment",uploadImageDataNotProvided:"Image data is not provided.",uploadImageIsTooLarge:"Image is too large. Choose one that's less than {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Uploading .{imageExtension} image is not allowed.",uploadImageFailed:"Unable to upload image. Please try again later."};var callWithChat$n={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"New Message",chatButtonTooltipClosedWithMessageCount:"Show chat ({unreadMessagesCount} unread)",chatButtonTooltipClose:"Hide chat",chatButtonTooltipOpen:"Show chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Copy invite link",copyInviteLinkButtonActionedLabel:"Link copied",copyInviteLinkActionedAriaLabel:"Invite link copied",dismissSidePaneButtonLabel:"Close",moreDrawerAudioDeviceMenuTitle:"Audio Device",moreDrawerButtonLabel:"More options",moreDrawerButtonTooltip:"More options",moreDrawerMicrophoneMenuTitle:"Microphone",moreDrawerSpeakerMenuTitle:"Speaker",moreDrawerCaptionsMenuTitle:"Live captions",moreDrawerSpokenLanguageMenuTitle:"Spoken language",moreDrawerCaptionLanguageMenuTitle:"Caption language",moreDrawerGalleryOptionsMenuTitle:"Gallery options",peopleButtonLabel:"People",selectedPeopleButtonLabel:"People Button Selected",peopleButtonTooltipOpen:"Show participants",peopleButtonTooltipClose:"Hide participants",peopleButtonTooltipOpenAriaLabel:"People button, Show participants",peopleButtonTooltipCloseAriaLabel:"People button, Hide participants",peoplePaneSubTitle:"In this call",peoplePaneTitle:"People",pictureInPictureTileAriaLabel:"Video Feeds. Click to return to call screen.",removeMenuLabel:"Remove",openDialpadButtonLabel:"Dial phone number",returnToCallButtonAriaDescription:"Return to Call",returnToCallButtonAriaLabel:"Back",peoplePaneAddPeopleButtonLabel:"Add People",dialpadStartCallButtonLabel:"Call",dialpadModalTitle:"Dial Phone Number",dialpadModalAriaLabel:"Dialpad",dialpadCloseModalButtonAriaLabel:"Close dialpad",openDtmfDialpadLabel:"Show dialpad",dtmfDialpadPlaceholderText:"Enter number",breakoutRoomChatPaneTitle:"Breakout Room Chat",chatContentSpinnerLabel:"Loading chat"};var en_US = {call:call$n,chat:chat$n,callWithChat:callWithChat$n};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for English (US)
 *
 * @public
 */
const COMPOSITE_LOCALE_EN_US = {
    component: COMPONENT_LOCALE_EN_US,
    strings: en_US
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Context for providing localized strings to components
 *
 * @private
 */
const LocaleContext = React.createContext(COMPOSITE_LOCALE_EN_US);
/**
 * Provider to provide localized strings for this library's composites.
 *
 * @private
 */
const LocalizationProvider = (props) => {
    const { locale, children } = props;
    return React.createElement(LocaleContext.Provider, { value: locale },
        React.createElement(LocalizationProvider$1, { locale: locale.component }, children));
};
/**
 * @private
 */
const useLocale = () => React.useContext(LocaleContext);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const SpinnerIcon = () => React.createElement(react.Spinner, { size: react.SpinnerSize.large });
/**
 * The default set of icons used by the composites directly (i.e. not via the components defined in this library).
 *
 * @public
 */
const COMPOSITE_ONLY_ICONS = {
    ChevronLeft: undefined,
    Link: undefined,
    LobbyScreenConnectingToCall: React.createElement(SpinnerIcon, null),
    LobbyScreenWaitingToBeAdmitted: React.createElement(SpinnerIcon, null),
    LocalDeviceSettingsCamera: React.createElement(reactIcons.Video20Filled, null),
    LocalDeviceSettingsMic: React.createElement(reactIcons.Mic20Filled, null),
    LocalDeviceSettingsSpeaker: React.createElement(reactIcons.Speaker220Filled, null),
    LocalPreviewPlaceholder: React.createElement(reactIcons.VideoOff20Filled, null),
    LocalCameraSwitch: React.createElement(reactIcons.CameraSwitch24Regular, null),
    ControlBarChatButtonActive: React.createElement(reactIcons.Chat20Filled, null),
    ControlBarChatButtonInactive: React.createElement(reactIcons.Chat20Regular, null),
    ControlButtonCameraProhibited: React.createElement(reactIcons.VideoProhibited20Filled, null),
    ControlButtonMicProhibited: React.createElement(reactIcons.MicProhibited20Filled, null),
    ControlButtonExitSpotlight: React.createElement(reactIcons.VideoPersonStarOff20Filled, null),
    ControlBarPeopleButton: React.createElement(reactIcons.People20Regular, null),
    MoreDrawerMicrophones: React.createElement(reactIcons.Mic20Regular, null),
    MoreDrawerPeople: React.createElement(reactIcons.People20Regular, null),
    MoreDrawerSpeakers: React.createElement(reactIcons.Speaker220Regular, null),
    MoreDrawerSelectedMicrophone: React.createElement(reactIcons.Mic20Filled, null),
    MoreDrawerSelectedSpeaker: React.createElement(reactIcons.Speaker220Filled, null),
    Muted: React.createElement(reactIcons.MicOff20Filled, null),
    NetworkReconnectIcon: React.createElement(reactIcons.CallMissed20Filled, null),
    NoticePageAccessDeniedTeamsMeeting: React.createElement(reactIcons.PersonDelete20Filled, null),
    NoticePageJoinCallFailedDueToNoNetwork: React.createElement(reactIcons.WifiWarning20Filled, null),
    NoticePageLeftCall: React.createElement(reactIcons.CallEnd20Filled, null),
    NoticePageRemovedFromCall: React.createElement(reactIcons.Info20Filled, null),
    NoticePageNotInvitedToRoom: React.createElement(reactIcons.Info20Filled, null),
    NoticePageRoomNotFound: React.createElement(reactIcons.Info20Filled, null),
    NoticePageRoomNotValid: React.createElement(reactIcons.Info20Filled, null),
    NoticePageCallRejected: React.createElement(reactIcons.Info20Filled, null),
    NoticePageCallTimeout: React.createElement(reactIcons.Info20Filled, null),
    PeoplePaneAddPerson: React.createElement(reactIcons.PersonAdd20Regular, null),
    PeoplePaneOpenDialpad: React.createElement(reactIcons.Dialpad20Regular, null),
    DialpadStartCall: React.createElement(reactIcons.Call20Regular, null),
    NoticePageInviteToRoomRemoved: React.createElement(reactIcons.Info20Filled, null),
    BlurVideoBackground: React.createElement(reactIcons.VideoBackgroundEffect20Regular, null),
    RemoveVideoBackgroundEffect: React.createElement(reactIcons.VideoPerson20Filled, null),
    GalleryOptions: React.createElement(reactIcons.Grid20Regular, null),
    OverflowGalleryTop: React.createElement(reactIcons.WindowHeaderHorizontal20Regular, null),
    SpeakerGalleryLayout: React.createElement(reactIcons.PersonSquare20Regular, null),
    DefaultGalleryLayout: React.createElement(reactIcons.TableSimple20Regular, null),
    FloatingLocalVideoGalleryLayout: React.createElement(reactIcons.BoardSplit20Regular, null),
    FocusedContentGalleryLayout: React.createElement(reactIcons.ContentView20Regular, null),
    LargeGalleryLayout: React.createElement(reactIcons.Table20Regular, null),
    DefaultCustomButton: React.createElement(reactIcons.Circle20Regular, null),
    DtmfDialpadButton: React.createElement(reactIcons.Dialpad20Regular, null),
    PhoneNumberButton: React.createElement(reactIcons.PersonCall20Regular, null),
    JoinByPhoneDialStepIcon: React.createElement(reactIcons.PersonCall20Regular, null),
    JoinByPhoneConferenceIdIcon: React.createElement(reactIcons.Dialpad20Regular, null),
    JoinByPhoneWaitToBeAdmittedIcon: React.createElement(reactIcons.Clock20Filled, null),
    PeoplePaneMoreButton: React.createElement(reactIcons.MoreHorizontal20Filled, null),
    StopAllSpotlightMenuButton: React.createElement(reactIcons.VideoPersonStarOff20Filled, null),
    TogetherModeLayout: React.createElement(reactIcons.PeopleAudience20Regular, null)
};
/**
 * The default set of icons that are available to used in the Composites.
 *
 * @public
 */
const DEFAULT_COMPOSITE_ICONS = Object.assign(Object.assign({}, DEFAULT_COMPONENT_ICONS), COMPOSITE_ONLY_ICONS);
/**
 * Icon wrapper to use when including customizable icons inside the CallComposite.
 * This wrapper ensures the icon name is being type-checked helping ensure no typos
 * and ensure that icon is customizable through the composite API.
 *
 * @private
 */
const CallCompositeIcon = (props) => React.createElement(react.FontIcon, Object.assign({}, props));
/**
 * Icon wrapper to use when including customizable icons inside the CallWithChatComposite.
 * This wrapper ensures the icon name is being type-checked helping ensure no typos
 * and ensure that icon is customizable through the composite API.
 *
 * @private
 */
const CallWithChatCompositeIcon = (props) => React.createElement(react.FontIcon, Object.assign({}, props));

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const globalLayerHostStyle = {
    position: 'fixed',
    zIndex: 1000000,
    inset: '0px',
    visibility: 'hidden'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 *
 * @param props
 * @returns
 */
const ACSAudioProvider = (props) => {
    const { audioContext, children } = props;
    const [stateAudioContext, setStateAudioContext] = React.useState(undefined);
    React.useEffect(() => {
        // Create the AudioContext only when the component is rendered
        setStateAudioContext(audioContext);
    }, [audioContext]);
    const alreadyWrapped = useAudio();
    if (alreadyWrapped) {
        return React.createElement(React.Fragment, null, children);
    }
    return React.createElement(ACSAudioContext.Provider, { value: stateAudioContext }, props.children);
};
/**
 * @private
 */
const ACSAudioContext = React.createContext(undefined);
/**
 * @private
 */
const useAudio = () => React.useContext(ACSAudioContext);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A base provider {@link React.Context} to wrap components with other required providers
 * (e.g. icons, FluentThemeProvider, LocalizationProvider).
 *
 * Required providers are only wrapped once, with all other instances only passing children.
 *
 * @private
 */
const BaseProvider = (props) => {
    const { fluentTheme, rtl, locale } = props;
    const globalLayerHostId = reactHooks.useId('composite-global-hostId');
    /**
     * Pass only the children if we previously registered icons, and have previously wrapped the children in
     * FluentThemeProvider and LocalizationProvider
     */
    const alreadyWrapped = useBase();
    if (alreadyWrapped) {
        return React.createElement(React.Fragment, null, props.children);
    }
    /**
     * Before registering fluent icons, we should check DEFAULT_COMPOSITE_ICONS and strip out the key value pairs where value is undefined
     */
    const iconsToRegister = {};
    Object.entries(DEFAULT_COMPOSITE_ICONS).forEach(([key, value]) => {
        if (value) {
            iconsToRegister[key] = value;
        }
    });
    /**
     * We register the default icon mappings merged with custom icons provided through props
     * to ensure all icons render correctly.
     */
    react.registerIcons({
        icons: Object.assign(Object.assign({}, iconsToRegister), props.icons)
    });
    /**
     * We need to create one context for the AudioProvider to ensure that we only have one instance of the AudioContext.
     */
    const compositeAudioContext = new AudioContext();
    // we use Customizer to override default LayerHost injected to <body />
    // which stop polluting global dom tree and increase compatibility with react-full-screen
    const CompositeElement = React.createElement(FluentThemeProvider, { fluentTheme: fluentTheme, rtl: rtl },
        // On mobile we expect the composite to fill the device screen, hence we set a meta property to have better OOBE.
        props.formFactor === 'mobile' && React.createElement("meta", { name: "viewport", content: "width=device-width" }),
        React.createElement(react.Customizer, { scopedSettings: {
                Layer: {
                    hostId: globalLayerHostId
                }
            } },
            React.createElement(ACSAudioProvider, { audioContext: compositeAudioContext },
                React.createElement(WithBackgroundColor, null, props.children))),
        React.createElement(react.LayerHost, { id: globalLayerHostId, className: react.mergeStyles(globalLayerHostStyle) }));
    const localizedElement = locale ? LocalizationProvider({
        locale,
        children: CompositeElement
    }) : CompositeElement;
    return React.createElement(BaseContext.Provider, { value: true }, localizedElement);
};
/**
 * @private
 */
const BaseContext = React.createContext(false);
/**
 * @private
 */
const useBase = () => React.useContext(BaseContext);
/**
 * @private
 * Provides a wrapper with a background color to ensure that composites always have a background color.
 * This is necessary to ensure that composites are not transparent,
 * and the background color of it's parent elements doesn't show through the composite.
 */
const WithBackgroundColor = (props) => {
    const { children } = props;
    const theme = react.useTheme();
    const className = react.mergeStyles({
        background: theme.semanticColors.bodyBackground,
        height: '100%',
        width: '100%',
        position: 'relative'
    });
    return React.createElement(react.Stack, { className: className }, children);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const ChatAdapterContext = React.createContext(undefined);
/**
 * @private
 */
const ChatAdapterProvider = (props) => {
    const { adapter } = props;
    return React.createElement(ChatAdapterContext.Provider, { value: adapter }, props.children);
};
/**
 * @private
 */
const useAdapter$1 = () => {
    const adapter = React.useContext(ChatAdapterContext);
    if (!adapter) {
        throw 'Cannot find adapter please initialize before usage.';
    }
    return adapter;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const CHAT_CONTAINER_MAX_WIDTH_REM = 41.25;
const CHAT_CONTAINER_MIN_HEIGHT_REM = 13;
/**
 * @private
 * z-index to ensure that chat container has lower z-index than participant pane
 */
const CHAT_CONTAINER_ZINDEX = 1;
/**
 * @private
 */
const chatScreenContainerStyle = react.mergeStyles({
    height: '100%',
    width: '100%',
    minHeight: `${CHAT_CONTAINER_MIN_HEIGHT_REM}rem`,
    minWidth: `${CHAT_CONTAINER_MIN_WIDTH_REM}rem`
});
/**
 * @private
 */
const chatContainer = react.mergeStyles({
    height: '100%',
    width: '100%',
    overflow: 'hidden'
});
/**
 * @private
 */
const chatArea = react.mergeStyles({
    height: '100%',
    width: '100%',
    overflow: 'auto',
    position: 'relative' // Ensure that the absolute children components are positioned relative to the chat area
});
/**
 * @private
 */
const chatWrapper = react.mergeStyles({
    height: '100%',
    width: '100%',
    overflow: 'visible',
    zIndex: CHAT_CONTAINER_ZINDEX
});
/**
 * @private
 */
const chatHeaderContainerStyle = react.mergeStyles({
    width: '100%',
    paddingLeft: '1.5rem',
    paddingRight: '1.5rem',
    '@media screen and (max-width: 25rem)': {
        paddingLeft: '5%',
        paddingRight: '5%'
    },
    borderBottom: '0.063rem solid #DDDDDD'
});
/**
 * @private
 */
const topicNameLabelStyle = react.mergeStyles({
    fontSize: '1.1rem',
    lineHeight: '2.5rem',
    fontWeight: 600,
    marginRight: '0.125rem',
    textOverflow: 'ellipsis',
    overflowY: 'hidden'
});
/**
 * @private
 */
const messageThreadChatCompositeStyles = react.memoizeFunction((background) => ({
    root: {
        maxWidth: `${CHAT_CONTAINER_MAX_WIDTH_REM}rem`
    },
    chatContainer: {
        background: background
    }
}));
/**
 * @private
 */
const typingIndicatorContainerStyles = {
    padding: '0rem 0.25rem'
};
/**
 * @private
 */
const sendboxContainerStyles = {
    maxWidth: `${CHAT_CONTAINER_MAX_WIDTH_REM}rem`,
    width: '100%',
    alignSelf: 'center'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$s = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * An Avatar component made using the `Persona` component.
 * It allows you to specify a `userId` and a `dataProvider` to retrieve the `AvatarPersonaData`.
 * Read more about `Persona` component at https://developer.microsoft.com/fluentui#/controls/web/persona
 *
 * @private
 */
const AvatarPersona = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const { userId, dataProvider, text, imageUrl, imageInitials, initialsColor, initialsTextColor, showOverflowTooltip } = props;
    const [data, setData] = React.useState();
    React.useEffect(() => {
        (() => __awaiter$s(void 0, void 0, void 0, function* () {
            if (dataProvider && userId) {
                const newData = yield dataProvider(userId);
                if (avatarDeepDifferenceCheck(data, newData)) {
                    setData(newData);
                }
            }
        }))();
    }, [data, dataProvider, userId]);
    let activePersona = '';
    if (props.allowActiveBorder) {
        // Display a border for raised handed participants in participant list
        activePersona = react.mergeStyles({
            border: 'solid 2px',
            borderColor: 'transparent',
            borderRadius: '50%',
            padding: '2px',
            boxSizing: 'content-box',
            margin: '-4px'
        });
        react.mergeStyles(activePersona, props.styles);
    }
    return React.createElement(react.Persona, Object.assign({}, props, { className: activePersona, text: (_a = data === null || data === void 0 ? void 0 : data.text) !== null && _a !== void 0 ? _a : text, imageUrl: (_b = data === null || data === void 0 ? void 0 : data.imageUrl) !== null && _b !== void 0 ? _b : imageUrl, imageInitials: (_c = data === null || data === void 0 ? void 0 : data.imageInitials) !== null && _c !== void 0 ? _c : imageInitials, initialsColor: (_d = data === null || data === void 0 ? void 0 : data.initialsColor) !== null && _d !== void 0 ? _d : initialsColor, initialsTextColor: (_f = (_e = data === null || data === void 0 ? void 0 : data.initialsTextColor) !== null && _e !== void 0 ? _e : initialsTextColor) !== null && _f !== void 0 ? _f : 'white', 
        // default disable tooltip unless specified
        showOverflowTooltip: showOverflowTooltip !== null && showOverflowTooltip !== void 0 ? showOverflowTooltip : false, showUnknownPersonaCoin: (_h = (_g = data === null || data === void 0 ? void 0 : data.showUnknownPersonaCoin) !== null && _g !== void 0 ? _g : props.showUnknownPersonaCoin) !== null && _h !== void 0 ? _h : false }));
};
const avatarDeepDifferenceCheck = (currentData, newData) => {
    return (currentData === null || currentData === void 0 ? void 0 : currentData.text) !== (newData === null || newData === void 0 ? void 0 : newData.text) || (currentData === null || currentData === void 0 ? void 0 : currentData.imageUrl) !== (newData === null || newData === void 0 ? void 0 : newData.imageUrl) || (currentData === null || currentData === void 0 ? void 0 : currentData.initialsColor) !== (newData === null || newData === void 0 ? void 0 : newData.initialsColor) || (currentData === null || currentData === void 0 ? void 0 : currentData.imageInitials) !== (newData === null || newData === void 0 ? void 0 : newData.imageInitials) || (currentData === null || currentData === void 0 ? void 0 : currentData.initialsTextColor) !== (newData === null || newData === void 0 ? void 0 : newData.initialsTextColor);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ChatHeader = (props) => {
    return React.createElement(react.Stack, { className: chatHeaderContainerStyle, horizontal: true },
        React.createElement(react.Stack.Item, { align: "center" },
            React.createElement("div", { className: topicNameLabelStyle }, props.topic)));
};
const getTopicName = (state, props) => {
    var _a, _b;
    return ((_b = (_a = state.threads[props.threadId]) === null || _a === void 0 ? void 0 : _a.properties) === null || _b === void 0 ? void 0 : _b.topic) || '';
};
/**
 * @private
 */
const getHeaderProps = reselect__namespace.createSelector([getTopicName], (topic) => {
    return {
        topic: topic
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useAdaptedSelector$1 = (selector, selectorProps) => {
    return useSelectorWithAdaptation$1(selector, adaptCompositeState$1, selectorProps);
};
/**
 * @private
 */
const useSelectorWithAdaptation$1 = (selector, adaptState, selectorProps) => {
    const adapter = useAdapter$1();
    // Keeps track of whether the current component is mounted or not. If it has unmounted, make sure we do not modify the
    // state or it will cause React warnings in the console. https://skype.visualstudio.com/SPOOL/_workitems/edit/2453212
    const mounted = React.useRef(false);
    React.useEffect(() => {
        mounted.current = true;
        return () => {
            mounted.current = false;
        };
    });
    const threadId = adapter.getState().thread.threadId;
    const threadConfigProps = React.useMemo(() => ({
        threadId
    }), [threadId]);
    const [props, setProps] = React.useState(selector(adaptState(adapter.getState()), threadConfigProps));
    const propRef = React.useRef(props);
    propRef.current = props;
    React.useEffect(() => {
        const onStateChange = (state) => {
            if (!mounted.current) {
                return;
            }
            const newProps = selector(adaptState(state), threadConfigProps);
            if (propRef.current !== newProps) {
                setProps(newProps);
            }
        };
        adapter.onStateChange(onStateChange);
        return () => {
            adapter.offStateChange(onStateChange);
        };
    }, [adaptState, adapter, selector, selectorProps, threadConfigProps]);
    return props;
};
const memoizeState$1 = memoizeOne((userId, displayName, threads, latestErrors) => ({
    userId,
    displayName,
    threads,
    latestErrors
}));
const memoizeThreads = memoizeOne((thread) => ({
    [thread.threadId]: thread
}));
const adaptCompositeState$1 = (compositeState) => {
    return memoizeState$1(compositeState.userId, compositeState.displayName, memoizeThreads(compositeState.thread), 
    // This is an unsafe type expansion.
    // compositeState.latestErrors can contain properties that are not valid in ChatErrors.
    //
    // But there is no way to check for valid property names at runtime:
    // - The set of valid property names is built from types in the @azure/communication-chat.
    //   Thus we don't have a literal array of allowed strings at runtime.
    // - Due to minification / uglification, the property names from the objects at runtime can't be used
    //   to compare against permissible values inferred from the types.
    //
    // This is not a huge problem -- it simply means that our adapted selector will include some extra operations
    // that are unknown to the UI component and data binding libraries. Generic handling of the errors (e.g.,
    // just displaying them in some UI surface) will continue to work for these operations. Handling of
    // specific operations (e.g., acting on errors related to permission issues) will ignore these operations.
    compositeState.latestErrors);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useHandlers$1 = (_component) => {
    return createCompositeHandlers$1(useAdapter$1());
};
const createCompositeHandlers$1 = memoizeOne((adapter) => ({
    // have to use `any` here so we don't import from Chat SDK
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    onSendMessage: (content, options) => {
        return adapter.sendMessage(content, options);
    },
    onLoadPreviousChatMessages: adapter.loadPreviousChatMessages,
    onMessageSeen: adapter.sendReadReceipt,
    onTyping: adapter.sendTypingIndicator,
    onRemoveParticipant: adapter.removeParticipant,
    updateThreadTopicName: adapter.setTopic,
    onUpdateMessage: function (messageId, content) {
        return adapter.updateMessage(messageId, content);
    },
    onDeleteMessage: adapter.deleteMessage
}));

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/* eslint-disable @typescript-eslint/no-explicit-any */
/**
 * @private
 */
const usePropsFor$2 = (component) => {
    const selector = getSelector(component);
    if (!selector) {
        throw new Error("Can't find the selector for component, please check supported component list");
    }
    return Object.assign(Object.assign({}, useAdaptedSelector$1(selector)), useHandlers$1());
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const sidePaneContainerStyles = {
    root: {
        height: '100%',
        padding: '0.5rem 0.25rem',
        width: '21.5rem'
    }
};
/**
 * @private
 */
({
    root: Object.assign(Object.assign({}, sidePaneContainerStyles), { display: 'none' })
});
/**
 * @private
 */
const sidePaneHeaderStyles = {
    root: {
        lineHeight: '1.5rem',
        fontSize: '1.125rem',
        padding: '0.75rem 0.75rem 0.75rem 0.25rem',
        fontWeight: '600'
    }
};
/**
 * @private
 */
const sidePaneHeaderContainerStyles = {
    root: {
        margin: '0 0.25rem'
    }
};
/**
 * @private
 */
const paneBodyContainer = {
    root: {
        flexDirection: 'column',
        display: 'flex'
    }
};
/**
 * @private
 */
const scrollableContainer = {
    root: {
        flexBasis: '0',
        flexGrow: '1',
        overflowY: 'auto'
    }
};
/**
 * @private
 */
const scrollableContainerContents = {
    root: {
        flexGrow: '1',
        flexBasis: '0',
        maxWidth: '100%',
        // Create a new stacking context so that `pipStyles` can set zIndex above the container.
        position: 'relative'
    }
};
/**
 * @private
 */
const containerContextStyles = {
    root: {
        position: 'absolute',
        height: '100%',
        width: '100%'
    }
};
/**
 * @private
 */
const peoplePaneContainerTokens = {
    childrenGap: '0.5rem'
};
/**
 * @private
 */
const participantListContainerPadding = {
    childrenGap: '0.5rem'
};
/**
 * @private
 */
const participantListStack = react.mergeStyles({
    height: '100%'
});
/**
 * @private
 */
const participantListContainerStyle = react.mergeStyles({
    height: '100%',
    overflowY: 'auto',
    overflowX: 'hidden'
});
/**
 * @private
 */
const participantListStyle = {
    root: {
        padding: '0rem'
    },
    participantItemStyles: {
        root: {
            padding: '0.5rem'
        }
    }
};
/**
 * @private
 */
const participantListMobileStyle = {
    root: {
        padding: '0rem'
    },
    participantItemStyles: {
        root: {
            padding: '0.5rem 1rem'
        }
    }
};
/**
 * @private
 */
const displayNameStyles$1 = {
    padding: '0.5rem',
    textOverflow: 'ellipsis',
    overflow: 'hidden',
    whiteSpace: 'nowrap'
};
/**
 * @private
 */
const headingMoreButtonStyles = (theme) => {
    return {
        root: {
            height: '100%',
            minWidth: '1.5rem',
            padding: '0.5rem 0.25rem',
            marginRight: '0.25rem',
            border: 'none',
            background: 'none'
        },
        rootHovered: {
            background: 'none'
        },
        rootPressed: {
            background: 'none'
        },
        rootFocused: {
            background: 'none'
        },
        icon: {
            color: theme.palette.neutralSecondary,
            display: 'flex',
            alignItems: 'center'
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const SendBoxPicker = (props) => {
    const sendBox = React.useMemo(() => React.createElement(SendBoxWrapper, Object.assign({}, props)), [props]);
    return sendBox;
};
// Move SendBox initialization to a separate component as otherwise it will cause re-render for Suspense and _ErrorBoundary when props and sendBoxProps are updated
const SendBoxWrapper = (props) => {
    const sendBoxProps = usePropsFor$2(SendBox);
    return React.createElement(SendBox, Object.assign({}, sendBoxProps, props));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useSelector$2 = (selector, selectorProps) => {
    // use selector with no adaptation
    return useSelectorWithAdaptation$1(selector, state => state, selectorProps);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const getUserId = (state) => state.userId;
/** @private */
const getThreadId = (state) => state.thread.threadId;
/** @private */
const getChatMessages = (state) => state.thread.chatMessages;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$r = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const ChatScreen = (props) => {
    const { onFetchAvatarPersonaData, onRenderMessage, onRenderTypingIndicator, options, styles, formFactor } = props;
    const defaultNumberOfChatMessagesToReload = 5;
    const [overlayImageItem, setOverlayImageItem] = React.useState();
    const [isImageOverlayOpen, setIsImageOverlayOpen] = React.useState(false);
    const adapter = useAdapter$1();
    const theme = useTheme();
    React.useEffect(() => {
        // Initial data should be always fetched by the composite(or external caller) instead of the adapter
        const fetchData = () => __awaiter$r(void 0, void 0, void 0, function* () {
            // Fetch initial data for adapter
            yield adapter.fetchInitialData();
            // Fetch initial set of messages. Without fetching messages here, if the Composite's adapter is changed the message thread does not load new messages.
            yield adapter.loadPreviousChatMessages(defaultNumberOfChatMessagesToReload);
        });
        fetchData();
    }, [adapter]);
    const messageThreadProps = usePropsFor$2(MessageThread);
    const typingIndicatorProps = usePropsFor$2(TypingIndicator);
    const headerProps = useAdaptedSelector$1(getHeaderProps);
    const errorBarProps = usePropsFor$2(ErrorBar);
    const overlayImageItemRef = React.useRef(overlayImageItem);
    overlayImageItemRef.current = overlayImageItem;
    const adapterChatMessages = useSelector$2(getChatMessages);
    React.useEffect(() => {
        var _a;
        if (overlayImageItemRef.current === undefined) {
            return;
        }
        const message = adapterChatMessages[overlayImageItemRef.current.messageId];
        if (message === undefined) {
            return;
        }
        const resourceFetchResult = (_a = message.resourceCache) === null || _a === void 0 ? void 0 : _a[overlayImageItemRef.current.imageUrl];
        if (overlayImageItemRef.current.imageSrc === '' && resourceFetchResult) {
            const fullSizeImageSrc = getResourceSourceUrl(resourceFetchResult);
            if (fullSizeImageSrc === undefined || fullSizeImageSrc === '' || overlayImageItemRef.current.imageSrc === fullSizeImageSrc) {
                return;
            }
            setOverlayImageItem(Object.assign(Object.assign({}, overlayImageItemRef.current), { imageSrc: fullSizeImageSrc }));
        }
    }, [adapterChatMessages]);
    const getResourceSourceUrl = (result) => {
        let src = '';
        if (result.error || !result.sourceUrl) {
            src = 'blob://';
        }
        else {
            src = result.sourceUrl;
        }
        return src;
    };
    const onRenderAvatarCallback = React.useCallback((userId, defaultOptions) => {
        return React.createElement(AvatarPersona, Object.assign({ userId: userId, hidePersonaDetails: true }, defaultOptions, { dataProvider: onFetchAvatarPersonaData }));
    }, [onFetchAvatarPersonaData]);
    const messageThreadStyles = React.useMemo(() => {
        return Object.assign({}, messageThreadChatCompositeStyles(theme.semanticColors.bodyBackground), styles === null || styles === void 0 ? void 0 : styles.messageThread);
    }, [styles === null || styles === void 0 ? void 0 : styles.messageThread, theme.semanticColors.bodyBackground]);
    const typingIndicatorStyles = React.useMemo(() => {
        return Object.assign({}, styles === null || styles === void 0 ? void 0 : styles.typingIndicator);
    }, [styles === null || styles === void 0 ? void 0 : styles.typingIndicator]);
    const sendBoxStyles = React.useMemo(() => {
        return Object.assign({}, styles === null || styles === void 0 ? void 0 : styles.sendBox);
    }, [styles === null || styles === void 0 ? void 0 : styles.sendBox]);
    const userIdObject = useSelector$2(getUserId);
    const userId = toFlatCommunicationIdentifier(userIdObject);
    const threadId = useSelector$2(getThreadId);
    const onInlineImageClicked = React.useCallback((attachmentId, messageId) => {
        var _a, _b, _c;
        const message = adapterChatMessages[messageId];
        const inlinedImages = (_b = (_a = message === null || message === void 0 ? void 0 : message.content) === null || _a === void 0 ? void 0 : _a.attachments) === null || _b === void 0 ? void 0 : _b.filter(attachment => {
            return attachment.attachmentType === 'image' && attachment.id === attachmentId;
        });
        const attachment = inlinedImages === null || inlinedImages === void 0 ? void 0 : inlinedImages[0];
        if (!attachment) {
            return;
        }
        let imageSrc = '';
        if (attachment.url) {
            const resourceFetchResult = (_c = message === null || message === void 0 ? void 0 : message.resourceCache) === null || _c === void 0 ? void 0 : _c[attachment.url];
            if (resourceFetchResult) {
                imageSrc = getResourceSourceUrl(resourceFetchResult);
            }
            else {
                adapter.downloadResourceToCache({
                    threadId,
                    messageId,
                    resourceUrl: attachment.url
                });
            }
        }
        const titleIconRenderOptions = {
            text: message === null || message === void 0 ? void 0 : message.senderDisplayName,
            size: react.PersonaSize.size32,
            showOverflowTooltip: false,
            imageAlt: message === null || message === void 0 ? void 0 : message.senderDisplayName
        };
        const messageSenderId = (message === null || message === void 0 ? void 0 : message.sender) !== undefined ? toFlatCommunicationIdentifier(message.sender) : userId;
        const titleIcon = onRenderAvatarCallback && onRenderAvatarCallback(messageSenderId, titleIconRenderOptions);
        const overlayImage = {
            title: (message === null || message === void 0 ? void 0 : message.senderDisplayName) || '',
            titleIcon: titleIcon,
            attachmentId: attachment.id,
            imageSrc: imageSrc,
            messageId: messageId,
            imageUrl: attachment.url || ''
        };
        setIsImageOverlayOpen(true);
        setOverlayImageItem(overlayImage);
    }, [adapter, adapterChatMessages, onRenderAvatarCallback, userId, threadId]);
    const onRenderInlineImage = React.useCallback((inlineImage, defaultOnRender) => {
        var _a, _b, _c;
        const message = adapterChatMessages[inlineImage.messageId];
        const attachment = (_b = (_a = message === null || message === void 0 ? void 0 : message.content) === null || _a === void 0 ? void 0 : _a.attachments) === null || _b === void 0 ? void 0 : _b.find(attachment => attachment.id === inlineImage.imageAttributes.id);
        if (attachment === undefined) {
            return defaultOnRender(inlineImage);
        }
        let pointerEvents = inlineImage.imageAttributes.src === '' ? 'none' : 'auto';
        const resourceCache = message === null || message === void 0 ? void 0 : message.resourceCache;
        if (resourceCache && attachment.previewUrl && resourceCache[attachment.previewUrl] && ((_c = resourceCache[attachment.previewUrl]) === null || _c === void 0 ? void 0 : _c.error)) {
            pointerEvents = 'none';
        }
        return React.createElement("span", { key: inlineImage.imageAttributes.id, onClick: () => onInlineImageClicked(inlineImage.imageAttributes.id || '', inlineImage.messageId), tabIndex: 0, role: "button", onKeyDown: e => {
                if (e.key === 'Enter') {
                    onInlineImageClicked(inlineImage.imageAttributes.id || '', inlineImage.messageId);
                }
            }, style: {
                cursor: 'pointer',
                pointerEvents
            } }, defaultOnRender(inlineImage));
    }, [adapterChatMessages, onInlineImageClicked]);
    const inlineImageOptions = React.useMemo(() => {
        return {
            onRenderInlineImage: onRenderInlineImage
        };
    }, [onRenderInlineImage]);
    const onDownloadButtonClicked = React.useCallback((imageSrc) => {
        if (imageSrc === '') {
            return;
        }
        if (react.isIOS()) {
            window.open(imageSrc, '_blank');
        }
        else {
            // Create a new anchor element
            const a = document.createElement('a');
            // Set the href and download attributes for the anchor element
            a.href = imageSrc;
            a.download = (overlayImageItem === null || overlayImageItem === void 0 ? void 0 : overlayImageItem.attachmentId) || '';
            a.rel = 'noopener noreferrer';
            a.target = '_blank';
            // Programmatically click the anchor element to trigger the download
            document.body.appendChild(a);
            a.click();
            document.body.removeChild(a);
        }
    }, [overlayImageItem === null || overlayImageItem === void 0 ? void 0 : overlayImageItem.attachmentId]);
    const onSendMessageHandler = React.useCallback(function (content) {
        return __awaiter$r(this, void 0, void 0, function* () {
            yield adapter.sendMessage(content, {});
        });
    }, [adapter]);
    const onUpdateMessageHandler = React.useCallback(function (messageId, content) {
        return __awaiter$r(this, void 0, void 0, function* () {
            yield messageThreadProps.onUpdateMessage(messageId, content);
        });
    }, [messageThreadProps]);
    return React.createElement(react.Stack, { className: chatContainer, grow: true },
        (options === null || options === void 0 ? void 0 : options.topic) !== false && React.createElement(ChatHeader, Object.assign({}, headerProps)),
        React.createElement(react.Stack, { className: chatArea, tokens: participantListContainerPadding, horizontal: true, grow: true },
            React.createElement(react.Stack, { className: chatWrapper, grow: true },
                (options === null || options === void 0 ? void 0 : options.errorBar) !== false && React.createElement(ErrorBar, Object.assign({}, errorBarProps)),
                React.createElement(MessageThread, Object.assign({}, messageThreadProps, { onUpdateMessage: onUpdateMessageHandler, onRenderAvatar: onRenderAvatarCallback, onRenderMessage: onRenderMessage, inlineImageOptions: inlineImageOptions, numberOfChatMessagesToReload: defaultNumberOfChatMessagesToReload, styles: messageThreadStyles })),
                React.createElement(react.Stack, { className: react.mergeStyles(sendboxContainerStyles) },
                    React.createElement("div", { className: react.mergeStyles(typingIndicatorContainerStyles) }, onRenderTypingIndicator ? onRenderTypingIndicator(typingIndicatorProps.typingUsers) : React.createElement(TypingIndicator, Object.assign({}, typingIndicatorProps, { styles: typingIndicatorStyles }))),
                    React.createElement(react.Stack, { horizontal: formFactor === 'mobile' },
                        formFactor === 'mobile',
                        React.createElement(react.Stack, { grow: true },
                            React.createElement(SendBoxPicker, { styles: sendBoxStyles, autoFocus: options === null || options === void 0 ? void 0 : options.autoFocus, 
                                // we need to overwrite onSendMessage for SendBox because we need to clear attachment state
                                // when submit button is clicked
                                onSendMessage: onSendMessageHandler })),
                        formFactor !== 'mobile')))),
        overlayImageItem && React.createElement(ImageOverlay, Object.assign({}, overlayImageItem, { isOpen: isImageOverlayOpen, onDismiss: () => {
                setOverlayImageItem(undefined);
                setIsImageOverlayOpen(false);
                adapter.removeResourceFromCache({
                    threadId,
                    messageId: overlayImageItem.messageId,
                    resourceUrl: overlayImageItem.imageUrl
                });
            }, onDownloadButtonClicked: onDownloadButtonClicked })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A customizable UI composite for the chat experience.
 *
 * @remarks Chat composite min width and height are respectively 17.5rem and 20rem (280px and 320px, with default rem at 16px)
 *
 * @public
 */
const ChatComposite = (props) => {
    const { adapter, options, onFetchAvatarPersonaData, onRenderTypingIndicator, onRenderMessage, onFetchParticipantMenuItems } = props;
    return React.createElement("div", { className: chatScreenContainerStyle },
        React.createElement(BaseProvider, Object.assign({}, props),
            React.createElement(ChatAdapterProvider, { adapter: adapter },
                React.createElement(ChatScreen, { options: options, onFetchAvatarPersonaData: onFetchAvatarPersonaData, onRenderTypingIndicator: onRenderTypingIndicator, onRenderMessage: onRenderMessage, onFetchParticipantMenuItems: onFetchParticipantMenuItems }))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const CallAdapterContext = React.createContext(undefined);
/**
 * @private
 */
const CallAdapterProvider = (props) => {
    const { adapter } = props;
    return React.createElement(CallAdapterContext.Provider, { value: adapter }, props.children);
};
/**
 * @private
 */
const useAdapter = () => {
    const adapter = React.useContext(CallAdapterContext);
    if (!adapter) {
        throw 'Cannot find adapter please initialize before usage.';
    }
    return adapter;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/* eslint-disable @typescript-eslint/no-explicit-any */
/**
 * @private
 */
const useAdaptedSelector = (selector, selectorProps) => {
    return useSelectorWithAdaptation(selector, adaptCompositeState, selectorProps);
};
/**
 * @private
 */
const useSelectorWithAdaptation = (selector, adaptState, selectorProps) => {
    var _a;
    const adapter = useAdapter();
    // Keeps track of whether the current component is mounted or not. If it has unmounted, make sure we do not modify the
    // state or it will cause React warnings in the console. https://skype.visualstudio.com/SPOOL/_workitems/edit/2453212
    const mounted = React.useRef(false);
    React.useEffect(() => {
        mounted.current = true;
        return () => {
            mounted.current = false;
        };
    });
    const callId = (_a = adapter.getState().call) === null || _a === void 0 ? void 0 : _a.id;
    const [props, setProps] = React.useState(selector(adaptState(adapter.getState()), {
        callId
    }));
    const propRef = React.useRef(props);
    propRef.current = props;
    React.useEffect(() => {
        const onStateChange = (state) => {
            var _a;
            if (!mounted.current) {
                return;
            }
            const newProps = selector(adaptState(state), {
                callId: (_a = state.call) === null || _a === void 0 ? void 0 : _a.id
            });
            if (propRef.current !== newProps) {
                setProps(newProps);
            }
        };
        adapter.onStateChange(onStateChange);
        return () => {
            adapter.offStateChange(onStateChange);
        };
    }, [adaptState, adapter, selector, selectorProps]);
    return props;
};
const memoizeState = memoizeOne((userId, deviceManager, calls, latestErrors, latestNotifications, displayName, alternateCallerId, environmentInfo) => ({
    userId,
    incomingCalls: {},
    incomingCallsEnded: {},
    callsEnded: {},
    deviceManager,
    callAgent: {
        displayName
    },
    calls,
    latestErrors,
    latestNotifications: latestNotifications !== null && latestNotifications !== void 0 ? latestNotifications : {},
    alternateCallerId,
    environmentInfo
}));
const memoizeCalls = memoizeOne((call) => call ? {
    [call.id]: call
} : {});
const adaptCompositeState = (compositeState) => {
    return memoizeState(compositeState.userId, compositeState.devices, memoizeCalls(compositeState.call), 
    // This is an unsafe type expansion.
    // compositeState.latestErrors can contain properties that are not valid in CallErrors.
    //
    // But there is no way to check for valid property names at runtime:
    // - The set of valid property names is built from types in the @azure/communication-calling.
    //   Thus we don't have a literal array of allowed strings at runtime.
    // - Due to minification / uglification, the property names from the objects at runtime can't be used
    //   to compare against permissible values inferred from the types.
    //
    // This is not a huge problem -- it simply means that our adapted selector will include some extra operations
    // that are unknown to the UI component and data binding libraries. Generic handling of the errors (e.g.,
    // just displaying them in some UI surface) will continue to work for these operations. Handling of
    // specific operations (e.g., acting on errors related to permission issues) will ignore these operations.
    compositeState.latestErrors, compositeState.latestNotifications, compositeState.displayName, compositeState.alternateCallerId, compositeState.environmentInfo);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Take the set of active notifications, and filter to only those that are newer than previously dismissed notifications or have never been dismissed.
 *
 * @private
 */
const filterLatestNotifications = (activeNotifications, trackedNotifications) => {
    const filteredNotifications = activeNotifications.filter(activeNotification => {
        const trackedNotification = trackedNotifications[activeNotification.type];
        return !trackedNotification || !trackedNotification.lastDismissedAt || trackedNotification.lastDismissedAt < trackedNotification.mostRecentlyActive;
    });
    return filteredNotifications;
};
/**
 * Maintain a record of the most recently active notification for each notification type.
 *
 * @private
 */
const updateTrackedNotificationsWithActiveNotifications = (existingTrackedNotifications, activeNotifications) => {
    var _a, _b;
    const trackedNotifications = {};
    // Only care about active notifications. If notifications are no longer active we do not track that they have been previously dismissed.
    for (const activeNotification of activeNotifications) {
        const existingTrackedNotification = existingTrackedNotifications[activeNotification.type];
        trackedNotifications[activeNotification.type] = {
            mostRecentlyActive: (_b = (_a = activeNotification.timestamp) !== null && _a !== void 0 ? _a : existingTrackedNotification === null || existingTrackedNotification === void 0 ? void 0 : existingTrackedNotification.mostRecentlyActive) !== null && _b !== void 0 ? _b : new Date(Date.now()),
            lastDismissedAt: existingTrackedNotification === null || existingTrackedNotification === void 0 ? void 0 : existingTrackedNotification.lastDismissedAt
        };
    }
    return trackedNotifications;
};
/**
 * Create a record for when the notification was most recently dismissed for tracking dismissed notifications.
 *
 * @private
 */
const trackNotificationAsDismissed = (notificationType, trackedNotifications) => {
    const now = new Date(Date.now());
    const existingNotification = trackedNotifications[notificationType];
    return Object.assign(Object.assign({}, trackedNotifications), { [notificationType]: Object.assign(Object.assign({}, (existingNotification || {})), { lastDismissedAt: now }) });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Subset of CallCompositePages that represent an end call state.
 * @private
 */
const END_CALL_PAGES = ['accessDeniedTeamsMeeting', 'joinCallFailedDueToNoNetwork', 'leftCall', 'removedFromCall'];

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const ACCESS_DENIED_TEAMS_MEETING_SUB_CODE = 5854;
const REMOTE_PSTN_USER_HUNG_UP = 560000;
const REMOVED_FROM_CALL_SUB_CODES = [5000, 5300, REMOTE_PSTN_USER_HUNG_UP];
const CALL_REJECTED_CODE$1 = 603;
const INVALID_MEETING_IDENTIFIER = 5751;
/** @private */
const ROOM_NOT_FOUND_SUB_CODE = 5732;
/** @private */
const ROOM_NOT_VALID_SUB_CODE = 5829;
/** @private */
const NOT_INVITED_TO_ROOM_SUB_CODE = 5828;
/** @private */
const INVITE_TO_ROOM_REMOVED_SUB_CODE = 5317;
/** @private */
const CALL_TIMEOUT_SUB_CODE = 10004;
/** @private */
const CALL_TIMEOUT_CODE = 487;
/** @private */
const BOT_TIMEOUT_CODE = 486;
/** @private */
const BOT_TIMEOUT_SUB_CODE = 10321;
/**
 * @private
 */
const isCameraOn = (state) => {
    if (state.call) {
        const stream = state.call.localVideoStreams.find(stream => stream.mediaStreamType === 'Video');
        return !!stream;
    }
    else {
        if (state.devices.selectedCamera) {
            const previewOn = _isPreviewOn(state.devices);
            return previewOn;
        }
    }
    return false;
};
/**
 * Reduce the set of call controls visible on mobile.
 * For example do not show screenshare button.
 *
 * @private
 */
const reduceCallControlsForMobile = (callControlOptions) => {
    if (callControlOptions === false) {
        return false;
    }
    // Ensure call controls a valid object.
    const reduceCallControlOptions = callControlOptions === true ? {} : callControlOptions || {};
    // Set to compressed mode when composite is optimized for mobile
    reduceCallControlOptions.displayType = 'compact';
    // Do not show screen share button when composite is optimized for mobile unless the developer
    // has explicitly opted in.
    if (reduceCallControlOptions.screenShareButton !== true) {
        reduceCallControlOptions.screenShareButton = false;
    }
    return reduceCallControlOptions;
};
var CallEndReasons;
(function (CallEndReasons) {
    CallEndReasons[CallEndReasons["LEFT_CALL"] = 0] = "LEFT_CALL";
    CallEndReasons[CallEndReasons["ACCESS_DENIED"] = 1] = "ACCESS_DENIED";
    CallEndReasons[CallEndReasons["REMOVED_FROM_CALL"] = 2] = "REMOVED_FROM_CALL";
    CallEndReasons[CallEndReasons["BAD_REQUEST"] = 3] = "BAD_REQUEST";
})(CallEndReasons || (CallEndReasons = {}));
const getCallEndReason = (call) => {
    var _a, _b, _c, _d;
    const remoteParticipantsEndedArray = Array.from(Object.values(call.remoteParticipantsEnded));
    /**
     * Handle the special case in a PSTN call where removing the last user kicks the caller out of the call.
     * The code and subcode is the same as when a user is removed from a teams interop call.
     * Hence, we look at the last remote participant removed to determine if the last participant removed was a phone number.
     * If yes, the caller was kicked out of the call, but we need to show them that they left the call.
     * Note: This check will only work for 1:1 PSTN Calls. The subcode is different for 1:N PSTN calls, and we do not need to handle that case.
     */
    if (remoteParticipantsEndedArray[0] && communicationCommon.isPhoneNumberIdentifier(remoteParticipantsEndedArray[0].identifier) && ((_a = call.callEndReason) === null || _a === void 0 ? void 0 : _a.subCode) !== REMOTE_PSTN_USER_HUNG_UP) {
        return CallEndReasons.LEFT_CALL;
    }
    if (((_b = call.callEndReason) === null || _b === void 0 ? void 0 : _b.subCode) && call.callEndReason.subCode === ACCESS_DENIED_TEAMS_MEETING_SUB_CODE) {
        return CallEndReasons.ACCESS_DENIED;
    }
    if (((_c = call.callEndReason) === null || _c === void 0 ? void 0 : _c.subCode) && REMOVED_FROM_CALL_SUB_CODES.includes(call.callEndReason.subCode)) {
        return CallEndReasons.REMOVED_FROM_CALL;
    }
    // If the call end reason code is 400, the call is ended due to a bad request. Keep this line at the bottom right before returning normal left call to catch the scenarios not including the ones above.
    if (((_d = call.callEndReason) === null || _d === void 0 ? void 0 : _d.code) === 400) {
        return CallEndReasons.BAD_REQUEST;
    }
    if (call.callEndReason) {
        // No error codes match, assume the user simply left the call regularly
        return CallEndReasons.LEFT_CALL;
    }
    throw new Error('No matching call end reason');
};
/**
 * Helper function for determine strings and icons for end call page
 * @private
 */
const getEndedCallPageProps = (locale, endedCall) => {
    var _a, _b, _c, _d, _e, _f, _g;
    let title = locale.strings.call.leftCallTitle;
    let moreDetails = locale.strings.call.leftCallMoreDetails;
    let disableStartCallButton = false;
    let iconName = 'NoticePageLeftCall';
    switch ((_a = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _a === void 0 ? void 0 : _a.subCode) {
        case ROOM_NOT_FOUND_SUB_CODE:
            if (locale.strings.call.roomNotFoundTitle) {
                title = locale.strings.call.roomNotFoundTitle;
                moreDetails = locale.strings.call.roomNotFoundDetails;
                disableStartCallButton = true;
                iconName = 'NoticePageRoomNotFound';
            }
            break;
        case ROOM_NOT_VALID_SUB_CODE:
            if (locale.strings.call.roomNotValidTitle) {
                title = locale.strings.call.roomNotValidTitle;
                moreDetails = locale.strings.call.roomNotValidDetails;
                disableStartCallButton = true;
                iconName = 'NoticePageRoomNotValid';
            }
            break;
        case NOT_INVITED_TO_ROOM_SUB_CODE:
            if (locale.strings.call.notInvitedToRoomTitle) {
                title = locale.strings.call.notInvitedToRoomTitle;
                moreDetails = locale.strings.call.notInvitedToRoomDetails;
                disableStartCallButton = true;
                iconName = 'NoticePageNotInvitedToRoom';
            }
            break;
        case INVITE_TO_ROOM_REMOVED_SUB_CODE:
            if (locale.strings.call.inviteToRoomRemovedTitle) {
                title = locale.strings.call.inviteToRoomRemovedTitle;
                moreDetails = locale.strings.call.inviteToRoomRemovedDetails;
                disableStartCallButton = true;
                iconName = 'NoticePageInviteToRoomRemoved';
            }
            break;
        case CALL_TIMEOUT_SUB_CODE:
            if (((_b = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _b === void 0 ? void 0 : _b.code) === CALL_TIMEOUT_CODE && locale.strings.call.callTimeoutTitle) {
                title = locale.strings.call.callTimeoutTitle;
                moreDetails = locale.strings.call.callTimeoutDetails;
                disableStartCallButton = true;
                iconName = 'NoticePageCallTimeout';
            }
            break;
        case BOT_TIMEOUT_SUB_CODE:
            if (((_c = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _c === void 0 ? void 0 : _c.code) === BOT_TIMEOUT_CODE && locale.strings.call.callTimeoutBotTitle) {
                title = locale.strings.call.callTimeoutBotTitle;
                moreDetails = locale.strings.call.callTimeoutBotDetails;
                disableStartCallButton = true;
                iconName = 'NoticePageCallTimeout';
            }
            break;
    }
    switch ((_d = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _d === void 0 ? void 0 : _d.code) {
        case CALL_REJECTED_CODE$1:
            if (locale.strings.call.callRejectedTitle) {
                title = locale.strings.call.callRejectedTitle;
                moreDetails = locale.strings.call.callRejectedMoreDetails;
                disableStartCallButton = true;
                iconName = 'NoticePageCallRejected';
            }
            break;
    }
    switch ((_e = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _e === void 0 ? void 0 : _e.subCode) {
        case 10037:
            if (locale.strings.call.participantCouldNotBeReachedTitle) {
                title = locale.strings.call.participantCouldNotBeReachedTitle;
                moreDetails = locale.strings.call.participantCouldNotBeReachedMoreDetails;
                disableStartCallButton = true;
            }
            break;
        case 10124:
            if (locale.strings.call.permissionToReachTargetParticipantNotAllowedTitle) {
                title = locale.strings.call.permissionToReachTargetParticipantNotAllowedTitle;
                moreDetails = locale.strings.call.permissionToReachTargetParticipantNotAllowedMoreDetails;
                disableStartCallButton = true;
            }
            break;
        case 10119:
            if (locale.strings.call.unableToResolveTenantTitle) {
                title = locale.strings.call.unableToResolveTenantTitle;
                moreDetails = locale.strings.call.unableToResolveTenantMoreDetails;
                disableStartCallButton = true;
            }
            break;
        case 10044:
            if (locale.strings.call.participantIdIsMalformedTitle) {
                title = locale.strings.call.participantIdIsMalformedTitle;
                moreDetails = locale.strings.call.participantIdIsMalformedMoreDetails;
                disableStartCallButton = true;
            }
            break;
    }
    switch ((_f = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _f === void 0 ? void 0 : _f.subCode) {
        case INVALID_MEETING_IDENTIFIER:
            if (locale.strings.call.callRejectedTitle) {
                title = locale.strings.call.callRejectedTitle;
                moreDetails = locale.strings.call.invalidMeetingIdentifier;
                disableStartCallButton = true;
            }
            break;
    }
    // keep this at the bottom to catch the scenarios not including the ones above.
    switch ((_g = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _g === void 0 ? void 0 : _g.code) {
        case 400:
            if (locale.strings.call.callRejectedTitle) {
                title = locale.strings.call.callRejectedTitle;
                disableStartCallButton = true;
            }
            break;
    }
    return {
        title,
        moreDetails,
        disableStartCallButton,
        iconName
    };
};
/**
 * Get the current call composite page based on the current call composite state
 *
 * @param Call - The current call state
 * @param previousCall - The state of the most recent previous call that has ended.
 *
 * @remarks - The previousCall state is needed to determine if the call has ended.
 * When the call ends a new call object is created, and so we must lookback at the
 * previous call state to understand how the call has ended. If there is no previous
 * call we know that this is a fresh call and can display the configuration page.
 *
 * @private
 */
const getCallCompositePage = (call, previousCall, transferCall, isReturningFromBreakoutRoom, unsupportedBrowserInfo) => {
    if (transferCall !== undefined) {
        return 'transferring';
    }
    if (isReturningFromBreakoutRoom) {
        return 'returningFromBreakoutRoom';
    }
    if (call) {
        // Must check for ongoing call *before* looking at any previous calls.
        // If the composite completes one call and joins another, the previous calls
        // will be populated, but not relevant for determining the page.
        // `_isInLobbyOrConnecting` needs to be checked first because `_isInCall` also returns true when call is in lobby.
        if (_isInLobbyOrConnecting(call === null || call === void 0 ? void 0 : call.state)) {
            return 'lobby';
            // `LocalHold` needs to be checked before `isInCall` since it is also a state that's considered in call.
        }
        else if ((call === null || call === void 0 ? void 0 : call.state) === 'LocalHold') {
            return 'hold';
        }
        else if ((call === null || call === void 0 ? void 0 : call.state) === 'Disconnecting') {
            return 'leaving';
        }
        else if (_isInCall(call === null || call === void 0 ? void 0 : call.state)) {
            return 'call';
        }
        else {
            // When the call object has been constructed after clicking , but before 'connecting' has been
            // set on the call object, we continue to show the configuration screen.
            // The call object does not correctly reflect local device state until `call.state` moves to `connecting`.
            // Moving to the 'lobby' page too soon leads to components that depend on the `call` object to show incorrect
            // transitional state.
            return 'configuration';
        }
    }
    if (previousCall) {
        const reason = getCallEndReason(previousCall);
        switch (reason) {
            case CallEndReasons.ACCESS_DENIED:
                return 'accessDeniedTeamsMeeting';
            case CallEndReasons.REMOVED_FROM_CALL:
                return 'removedFromCall';
            case CallEndReasons.BAD_REQUEST:
                return 'badRequest';
            case CallEndReasons.LEFT_CALL:
                if (previousCall.diagnostics.network.latest.noNetwork) {
                    return 'joinCallFailedDueToNoNetwork';
                }
                return 'leftCall';
        }
    }
    // No call state - show starting page (configuration)
    return 'configuration';
};
/** @private */
const IsCallEndedPage = (page) => END_CALL_PAGES.includes(page);
/**
 * Creates a new call control options object and sets the correct values for disabling
 * the buttons provided in the `disabledControls` array.
 * Returns a new object without changing the original object.
 * @param callControlOptions options for the call control component that need to be modified.
 * @param disabledControls An array of controls to disable.
 * @returns a copy of callControlOptions with disabledControls disabled
 * @private
 */
const disableCallControls = (callControlOptions, disabledControls) => {
    var _a;
    if (callControlOptions === false) {
        return false;
    }
    // Ensure we clone the prop if it is an object to ensure we do not mutate the original prop.
    let newOptions = (_a = (callControlOptions instanceof Object ? Object.assign({}, callControlOptions) : callControlOptions)) !== null && _a !== void 0 ? _a : {};
    if (newOptions === true || newOptions === undefined) {
        newOptions = disabledControls.reduce((acc, key) => {
            // @ts-expect-error TODO: fix noImplicitAny error here
            // Not solveable at this time due to typescript limitations. The typing is too complex for typescript to
            // understand. Will need to revisit when either typescript or the calling component bindings are updated.
            acc[key] = {
                disabled: true
            };
            return acc;
        }, {});
    }
    else {
        disabledControls.forEach(key => {
            // @ts-expect-error refer to above comment
            if (newOptions[key] !== false) {
                // @ts-expect-error refer to above comment
                newOptions[key] = {
                    disabled: true
                };
            }
        });
    }
    return newOptions;
};
/**
 * Check if a disabled object is provided for a button and returns if the button is disabled.
 * A button is only disabled if is explicitly set to disabled.
 *
 * @param option
 * @returns whether a button is disabled
 * @private
 */
const isDisabled$4 = (option) => {
    if (option === undefined || typeof option === 'boolean') {
        return false;
    }
    return option.disabled;
};
/**
 * Check if we are using safari browser
 * @private
 */
const _isSafari = (environmentInfo) => {
    return (environmentInfo === null || environmentInfo === void 0 ? void 0 : environmentInfo.environment.browser.toLowerCase()) === 'safari';
};
/**
 * @private
 * This is the util function to create a participant modifier for remote participantList
 * It memoize previous original participant items and only update the changed participant
 * It takes in one modifier function to generate one single participant object, it returns undefined if the object keeps unmodified
 */
const createParticipantModifier = (createModifiedParticipant) => {
    let previousParticipantState = undefined;
    let modifiedParticipants = {};
    const memoizedParticipants = {};
    return (state) => {
        var _a, _b, _c, _d;
        // if root state is the same, we don't need to update the participants
        if (((_a = state.call) === null || _a === void 0 ? void 0 : _a.remoteParticipants) !== previousParticipantState) {
            modifiedParticipants = {};
            const originalParticipants = Object.entries(((_b = state.call) === null || _b === void 0 ? void 0 : _b.remoteParticipants) || {});
            for (const [key, originalParticipant] of originalParticipants) {
                const modifiedParticipant = createModifiedParticipant(key, originalParticipant);
                if (modifiedParticipant === undefined) {
                    modifiedParticipants[key] = originalParticipant;
                    continue;
                }
                // Generate the new item if original cached item has been changed
                if (((_c = memoizedParticipants[key]) === null || _c === void 0 ? void 0 : _c.originalRef) !== originalParticipant) {
                    memoizedParticipants[key] = {
                        newParticipant: modifiedParticipant,
                        originalRef: originalParticipant
                    };
                }
                // the modified participant is always coming from the memoized cache, whether is was refreshed
                // from the previous closure or not
                const memoizedParticipant = memoizedParticipants[key];
                if (!memoizedParticipant) {
                    throw new Error('Participant modifier encountered an unhandled exception.');
                }
                modifiedParticipants[key] = memoizedParticipant.newParticipant;
            }
            previousParticipantState = (_d = state.call) === null || _d === void 0 ? void 0 : _d.remoteParticipants;
        }
        return Object.assign(Object.assign({}, state), { call: state.call ? Object.assign(Object.assign({}, state.call), { remoteParticipants: modifiedParticipants }) : undefined });
    };
};
/** @private */
const getBackgroundEffectFromSelectedEffect = (selectedEffect, VideoBackgroundEffectsDependency) => (selectedEffect === null || selectedEffect === void 0 ? void 0 : selectedEffect.effectName) === 'blur' ? VideoBackgroundEffectsDependency.createBackgroundBlurEffect() : (selectedEffect === null || selectedEffect === void 0 ? void 0 : selectedEffect.effectName) === 'replacement' ? VideoBackgroundEffectsDependency.createBackgroundReplacementEffect({
    backgroundImageUrl: selectedEffect.backgroundImageUrl
}) : undefined;
/**
 * @remarks this logic should mimic the onToggleCamera in the common call handlers.
 * @private
 */
const getSelectedCameraFromAdapterState = (state) => state.devices.selectedCamera || state.devices.cameras[0];
/**
 * Helper to determine if the adapter has a locator or targetCallees
 * @param locatorOrTargetCallees
 * @returns boolean to determine if the adapter has a locator or targetCallees, true is locator, false is targetCallees
 * @private
 */
const getLocatorOrTargetCallees = (locatorOrTargetCallees) => {
    return !!Array.isArray(locatorOrTargetCallees);
};
/**
 * Return different conditions based on the current and previous state of recording and transcribing
 *
 * @param callRecordState - The current call record state: on, off, stopped
 * @param callTranscribeState - The current call transcribe state: on, off, stopped
 *
 * @remarks - The stopped state means: previously on but currently off
 *
 * @private
 */
const computeVariant = (callRecordState, callTranscribeState) => {
    if (callRecordState === 'on' && callTranscribeState === 'on') {
        return 'recordingAndTranscriptionStarted';
    }
    else if (callRecordState === 'on' && callTranscribeState === 'off') {
        return 'recordingStarted';
    }
    else if (callRecordState === 'off' && callTranscribeState === 'on') {
        return 'transcriptionStarted';
    }
    else if (callRecordState === 'on' && callTranscribeState === 'stopped') {
        return 'transcriptionStoppedStillRecording';
    }
    else if (callRecordState === 'stopped' && callTranscribeState === 'on') {
        return 'recordingStoppedStillTranscribing';
    }
    else if (callRecordState === 'off' && callTranscribeState === 'stopped') {
        return 'transcriptionStopped';
    }
    else if (callRecordState === 'stopped' && callTranscribeState === 'off') {
        return 'recordingStopped';
    }
    else if (callRecordState === 'stopped' && callTranscribeState === 'stopped') {
        return 'recordingAndTranscriptionStopped';
    }
    else {
        return 'noState';
    }
};
/**
 * @private
 */
function determineStates(previous, current) {
    // if current state is on, then return on
    if (current) {
        return 'on';
    }
    // if current state is off
    else {
        // if previous state is on and current state is off, return stopped (on -> off)
        if (previous === 'on') {
            return 'stopped';
        }
        // otherwise remain previous state unchanged
        else {
            return previous;
        }
    }
}
// The debounce time for the stopped state to be shown after both states are stopped.
// This is to prevent stopped messages from being lost by transitioning to "Off" too
// quickly if the states are toggled in quick succession.
// This also prevents React strict mode from transitioning to "Off" too quickly.
const ComplianceNotificationOffDebounceTimeMs = 2000;
/**
 * Compute compliance notification based on latest compliance state and cached props.
 * @private
 */
function computeComplianceNotification(complianceProps, cachedProps) {
    // Only update cached props and variant if there is _some_ change in the latest props.
    // This ensures that state machine is only updated if there is an actual change in the props.
    const shouldUpdateCached = complianceProps.callRecordState !== cachedProps.current.latestBooleanState.callRecordState || complianceProps.callTranscribeState !== cachedProps.current.latestBooleanState.callTranscribeState;
    // The following three operations must be performed in this exact order:
    // [1]: Update cached state to transition the state machine.
    if (shouldUpdateCached) {
        cachedProps.current = {
            latestBooleanState: complianceProps,
            latestStringState: {
                callRecordState: determineStates(cachedProps.current.latestStringState.callRecordState, complianceProps.callRecordState),
                callTranscribeState: determineStates(cachedProps.current.latestStringState.callTranscribeState, complianceProps.callTranscribeState)
            },
            lastUpdated: Date.now()
        };
    }
    // [2]: If the callRecordState and callTranscribeState are both stopped for a predetermined amount of time, mark both states as off.
    // NOTE: this can be removed once lastStoppedRecording in the calling stateful client is GA.
    if (shouldUpdateCached && cachedProps.current.latestStringState.callRecordState === 'stopped' && cachedProps.current.latestStringState.callTranscribeState === 'stopped' && Date.now() - cachedProps.current.lastUpdated > ComplianceNotificationOffDebounceTimeMs) {
        // When both states are stopped, after displaying message "RECORDING_AND_TRANSCRIPTION_STOPPED", change both states to off (going back to the default state).
        cachedProps.current.latestStringState.callRecordState = 'off';
        cachedProps.current.latestStringState.callTranscribeState = 'off';
    }
    // [3]: Compute the variant, using the transitioned state machine.
    const variant = computeVariant(cachedProps.current.latestStringState.callRecordState, cachedProps.current.latestStringState.callTranscribeState);
    // If the variant is not 'noState', then show the notification.
    if (variant !== 'noState') {
        return {
            type: variant,
            timestamp: new Date(Date.now())
        };
    }
    else {
        return undefined;
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useSelector$1 = (selector, selectorProps) => {
    // use selector with no adaptation
    return useSelectorWithAdaptation(selector, state => state, selectorProps);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const getDisplayName = (state) => state.displayName;
/**
 * @private
 */
const getEndedCall = (state) => state.endedCall;
/**
 * @private
 */
const getCallStatus = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.state) !== null && _b !== void 0 ? _b : 'None'; };
/**
 * @private
 */
const getDeviceManager = (state) => state.devices;
/**
 * @private
 */
const getIsScreenShareOn = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.isScreenSharingOn) !== null && _b !== void 0 ? _b : false; };
/**
 * @private
 */
const getLocalParticipantRaisedHand = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.raiseHand.localParticipantRaisedHand; };
/**
 * @private
 */
const getCapabilites = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.capabilitiesFeature) === null || _b === void 0 ? void 0 : _b.capabilities; };
/**
 * @private
 */
const getMicrophones = (state) => state.devices.microphones;
/**
 * @private
 */
const getCameras = (state) => state.devices.cameras;
/**
 * @private
 */
const getRole = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.role; };
/**
 * @private
 */
const getPage = (state) => state.page;
/**
 * @private
 */
const getTransferCall = (state) => state.acceptedTransferCallState;
/**
 * @private
 */
const getLocalMicrophoneEnabled = (state) => state.isLocalPreviewMicrophoneEnabled;
/**
 * @private
 */
const getLocalVideoStreams = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.localVideoStreams; };
/**
 * @private
 */
const getIsTranscriptionActive = (state) => { var _a; return !!((_a = state.call) === null || _a === void 0 ? void 0 : _a.transcription.isTranscriptionActive); };
/**
 * @private
 */
const getIsRecordingActive = (state) => { var _a; return !!((_a = state.call) === null || _a === void 0 ? void 0 : _a.recording.isRecordingActive); };
/**
 * @private
 */
const getUserFacingDiagnostics = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.diagnostics; };
/**
 * @private
 */
const getDominantSpeakerInfo = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.dominantSpeakers; };
/**
 * @private
 */
const getRemoteParticipants = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.remoteParticipants; };
/**
 * @private
 */
const getEnvironmentInfo = (state) => state.environmentInfo;
/**
 * @private
 */
const getSelectedVideoEffect = (state) => state.selectedVideoBackgroundEffect;
/** @private */
const getCaptionsStatus = (state) => {
    var _a;
    return (_a = state.call) === null || _a === void 0 ? void 0 : _a.captionsFeature.isCaptionsFeatureActive;
};
/**
 * @private
 */
const getIsTeamsCall = (state) => state.isTeamsCall;
/**
 * @private
 */
const getIsTeamsMeeting = (state) => state.isTeamsMeeting;
/**
 * @private
 */
const getTeamsMeetingCoordinates = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.meetingConference) === null || _b === void 0 ? void 0 : _b.conferencePhones; };
/**
 * @private
 */
const getLatestCapabilitiesChangedInfo = (state) => {
    var _a, _b;
    return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.capabilitiesFeature) === null || _b === void 0 ? void 0 : _b.latestCapabilitiesChangeInfo;
};
/**
 * @private
 */
const getTargetCallees = (state) => state.targetCallees;
/**
 * @private
 */
const getStartTime = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.startTime; };
/**
 * @private
 */
const getSpotlightedParticipants = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.spotlight) === null || _b === void 0 ? void 0 : _b.spotlightedParticipants; };
/**
 * @private
 */
const getAssignedBreakoutRoom = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.breakoutRooms) === null || _b === void 0 ? void 0 : _b.assignedBreakoutRoom; };
/**
 * @private
 */
const getBreakoutRoomSettings = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.breakoutRooms) === null || _b === void 0 ? void 0 : _b.breakoutRoomSettings; };
/**
 * @private
 */
const getBreakoutRoomDisplayName = (state) => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.breakoutRooms) === null || _b === void 0 ? void 0 : _b.breakoutRoomDisplayName; };
/**
 * @private
 */
const getLatestNotifications = (state) => state.latestNotifications;
/** @private */
const getVideoEffectsDependency = (state) => state.onResolveVideoEffectDependency;
/** @private */
const getDeepNoiseSuppresionEffectsDependency = (state) => state.onResolveDeepNoiseSuppressionDependency;
/** @private */
const getDeepNoiseSuppresionIsOnByDefault = (state) => state.deepNoiseSuppressionOnByDefault;
/** @private */
const getHideDeepNoiseSupressionButton = (state) => state.hideDeepNoiseSuppressionButton;
/** @private */
const getReactionResources = (state) => state.reactions;
/** @private */
const getAlternateCallerId = (state) => state.alternateCallerId;
/** @private */
const getIsRoomsCall = (state) => state.isRoomsCall;
/** @private */
const getVideoBackgroundImages = (state) => state.videoBackgroundImages;
/**
 * @private
 * Gets the together mode streams state.
 * @param state - The current state of the call adapter.
 * @returns The together mode streams state or undefined.
 */
const getIsTogetherModeActive = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.togetherMode.isActive; };
/**
 * @private
 * Gets local participant's user id.
 * @param state - The current state of the call adapter.
 * @returns The local participant's user id or undefined.
 */
const getLocalUserId = (state) => state.userId;
/** @private */
const getMediaAccessSetting = (state) => { var _a; return (_a = state.call) === null || _a === void 0 ? void 0 : _a.meetingMediaAccess; };

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$q = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Private implementation of the hook
 * @private
 */
const useHandlers = (_component) => {
    const adapter = useAdapter();
    const capabilities = useSelector$1(getCapabilites);
    return createCompositeHandlers(adapter, capabilities);
};
const createCompositeHandlers = memoizeOne((adapter, capabilities) => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    return {
        onCreateLocalStreamView: (options) => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.createStreamView(undefined, options);
        }),
        onCreateRemoteStreamView: (userId, options) => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.createStreamView(userId, options);
        }),
        onHangUp: (forEveryone) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.leaveCall(forEveryone);
        }),
        onToggleHold: () => __awaiter$q(void 0, void 0, void 0, function* () {
            var _j;
            return ((_j = adapter.getState().call) === null || _j === void 0 ? void 0 : _j.state) === 'LocalHold' ? yield adapter.resumeCall() : yield adapter.holdCall();
        }),
        onAddParticipant: (participant, options) => __awaiter$q(void 0, void 0, void 0, function* () {
            if ('communicationUserId' in participant) {
                return yield adapter.addParticipant(participant);
            }
            else if ('phoneNumber' in participant) {
                return yield adapter.addParticipant(participant, options);
            }
        }),
        onSendDtmfTone: (dtmfTone) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.sendDtmfTone(dtmfTone);
        }),
        onRemoveParticipant: (userId) => __awaiter$q(void 0, void 0, void 0, function* () {
            if (typeof userId === 'string') {
                yield adapter.removeParticipant(userId);
            }
            else {
                yield adapter.removeParticipant(_toCommunicationIdentifier(userId));
            }
        }),
        onRaiseHand: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.raiseHand();
        }),
        onLowerHand: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.lowerHand();
        }),
        onToggleRaiseHand: () => __awaiter$q(void 0, void 0, void 0, function* () {
            var _k;
            if ((_k = adapter.getState().call) === null || _k === void 0 ? void 0 : _k.raiseHand.localParticipantRaisedHand) {
                yield adapter.lowerHand();
            }
            else {
                yield adapter.raiseHand();
            }
        }),
        onReactionClick: (reaction) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.onReactionClick(reaction);
        }),
        onSelectCamera: (deviceInfo, options) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.setCamera(deviceInfo, options);
        }),
        onSelectMicrophone: (deviceInfo) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.setMicrophone(deviceInfo);
        }),
        onSelectSpeaker: (deviceInfo) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.setSpeaker(deviceInfo);
        }),
        onStartCall: (participants, options) => {
            const rawIds = participants.map(participant => toFlatCommunicationIdentifier(participant));
            return adapter.startCall(rawIds, options);
        },
        onStartScreenShare: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.startScreenShare();
        }),
        onStopScreenShare: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.stopScreenShare();
        }),
        onToggleCamera: (options) => __awaiter$q(void 0, void 0, void 0, function* () {
            isCameraOn(adapter.getState()) ? yield adapter.stopCamera() : yield adapter.startCamera(options);
        }),
        onToggleMicrophone: () => __awaiter$q(void 0, void 0, void 0, function* () {
            var _l;
            return ((_l = adapter.getState().call) === null || _l === void 0 ? void 0 : _l.isMuted) ? yield adapter.unmute() : yield adapter.mute();
        }),
        onToggleScreenShare: () => __awaiter$q(void 0, void 0, void 0, function* () {
            var _m;
            return ((_m = adapter.getState().call) === null || _m === void 0 ? void 0 : _m.isScreenSharingOn) ? yield adapter.stopScreenShare() : yield adapter.startScreenShare();
        }),
        onStartLocalVideo: () => __awaiter$q(void 0, void 0, void 0, function* () {
            if (adapter.getState().call) {
                return adapter.startCamera();
            }
        }),
        onDisposeLocalStreamView: () => __awaiter$q(void 0, void 0, void 0, function* () {
            return adapter.disposeLocalVideoStreamView();
        }),
        onDisposeRemoteStreamView: (userId) => __awaiter$q(void 0, void 0, void 0, function* () {
            return adapter.disposeStreamView(userId);
        }),
        onDisposeRemoteScreenShareStreamView: (userId) => __awaiter$q(void 0, void 0, void 0, function* () {
            return adapter.disposeScreenShareStreamView(userId);
        }),
        onDisposeLocalScreenShareStreamView: () => __awaiter$q(void 0, void 0, void 0, function* () {
            return adapter.disposeScreenShareStreamView('');
        }),
        onDisposeRemoteVideoStreamView: (userId) => __awaiter$q(void 0, void 0, void 0, function* () {
            return adapter.disposeRemoteVideoStreamView(userId);
        }),
        onRemoveVideoBackgroundEffects: () => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.stopVideoBackgroundEffects();
        }),
        onBlurVideoBackground: (backgroundBlurConfig) => __awaiter$q(void 0, void 0, void 0, function* () {
            const blurConfig = Object.assign({ effectName: 'blur' }, backgroundBlurConfig);
            return yield adapter.startVideoBackgroundEffect(blurConfig);
        }),
        onReplaceVideoBackground: (backgroundReplacementConfig) => __awaiter$q(void 0, void 0, void 0, function* () {
            const replacementConfig = Object.assign({ effectName: 'replacement' }, backgroundReplacementConfig);
            return yield adapter.startVideoBackgroundEffect(replacementConfig);
        }),
        onStartNoiseSuppressionEffect: () => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.startNoiseSuppressionEffect();
        }),
        onStopNoiseSuppressionEffect: () => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.stopNoiseSuppressionEffect();
        }),
        onStartCaptions: (options) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.startCaptions(options);
        }),
        onStopCaptions: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.stopCaptions();
        }),
        onSetSpokenLanguage: (language) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.setSpokenLanguage(language);
        }),
        onSetCaptionLanguage: (language) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.setCaptionLanguage(language);
        }),
        onSubmitSurvey: (survey) => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.submitSurvey(survey);
        }),
        onStartSpotlight: (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.startSpotlight(userIds);
        }),
        onStopSpotlight: (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.stopSpotlight(userIds);
        }),
        onStopAllSpotlight: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.stopAllSpotlight();
        }),
        onStartLocalSpotlight: (capabilities === null || capabilities === void 0 ? void 0 : capabilities.spotlightParticipant.isPresent) ? () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.startSpotlight();
        }) : undefined,
        onStopLocalSpotlight: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.stopSpotlight();
        }),
        onStartRemoteSpotlight: (capabilities === null || capabilities === void 0 ? void 0 : capabilities.spotlightParticipant.isPresent) ? (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.startSpotlight(userIds);
        }) : undefined,
        onStopRemoteSpotlight: (capabilities === null || capabilities === void 0 ? void 0 : capabilities.removeParticipantsSpotlight.isPresent) ? (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.stopSpotlight(userIds);
        }) : undefined,
        onMuteParticipant: (userId) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.muteParticipant(userId);
        }),
        onMuteAllRemoteParticipants: () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.muteAllRemoteParticipants();
        }),
        onForbidAudio: ((_a = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersAudio) === null || _a === void 0 ? void 0 : _a.isPresent) ? (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.forbidAudio(userIds);
        }) : undefined,
        onPermitAudio: ((_b = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersAudio) === null || _b === void 0 ? void 0 : _b.isPresent) ? (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.permitAudio(userIds);
        }) : undefined,
        onForbidOthersAudio: ((_c = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersAudio) === null || _c === void 0 ? void 0 : _c.isPresent) ? () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.forbidOthersAudio();
        }) : undefined,
        onPermitOthersAudio: ((_d = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersAudio) === null || _d === void 0 ? void 0 : _d.isPresent) ? () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.permitOthersAudio();
        }) : undefined,
        onForbidVideo: ((_e = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersVideo) === null || _e === void 0 ? void 0 : _e.isPresent) ? (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.forbidVideo(userIds);
        }) : undefined,
        onPermitVideo: ((_f = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersVideo) === null || _f === void 0 ? void 0 : _f.isPresent) ? (userIds) => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.permitVideo(userIds);
        }) : undefined,
        onForbidOthersVideo: ((_g = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersVideo) === null || _g === void 0 ? void 0 : _g.isPresent) ? () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.forbidOthersVideo();
        }) : undefined,
        onPermitOthersVideo: ((_h = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersVideo) === null || _h === void 0 ? void 0 : _h.isPresent) ? () => __awaiter$q(void 0, void 0, void 0, function* () {
            yield adapter.permitOthersVideo();
        }) : undefined,
        onCreateTogetherModeStreamView: (options) => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.createTogetherModeStreamView(options);
        }),
        onStartTogetherMode: () => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.startTogetherMode();
        }),
        onSetTogetherModeSceneSize: (width, height) => {
            return adapter.setTogetherModeSceneSize(width, height);
        },
        onDisposeTogetherModeStreamView: () => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.disposeTogetherModeStreamView();
        }),
        onSendRealTimeText: (text, isFinalized) => __awaiter$q(void 0, void 0, void 0, function* () {
            return yield adapter.sendRealTimeText(text, isFinalized);
        })
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Private implementation of the hook
 * @private
 */
const usePropsFor$1 = (component) => {
    const selector = getSelector$1(component);
    if (!selector) {
        throw new Error("Can't find the selector for component, please check supported component list");
    }
    return Object.assign(Object.assign({}, useAdaptedSelector(selector)), useHandlers());
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const CallingCaptionsSettingsModal = (props) => {
    const CaptionsSettingsModalProps = usePropsFor$1(CaptionsSettingsModal);
    const strings = useLocale().strings.call;
    const modalStrings = {
        captionsSettingsModalTitle: strings.captionsSettingsModalTitle,
        captionsSettingsSpokenLanguageDropdownLabel: strings.captionsSettingsSpokenLanguageDropdownLabel,
        captionsSettingsCaptionLanguageDropdownLabel: strings.captionsSettingsCaptionLanguageDropdownLabel,
        captionsSettingsSpokenLanguageDropdownInfoText: strings.captionsSettingsSpokenLanguageDropdownInfoText,
        captionsSettingsCaptionLanguageDropdownInfoText: strings.captionsSettingsCaptionLanguageDropdownInfoText,
        captionsSettingsConfirmButtonLabel: strings.captionsSettingsConfirmButtonLabel,
        captionsSettingsCancelButtonLabel: strings.captionsSettingsCancelButtonLabel,
        captionsSettingsModalAriaLabel: strings.captionsSettingsModalAriaLabel,
        captionsSettingsCloseModalButtonAriaLabel: strings.captionsSettingsCloseModalButtonAriaLabel
    };
    const spokenLanguageStrings = strings.spokenLanguageStrings;
    const captionLanguageStrings = strings.captionLanguageStrings;
    return React.createElement(CaptionsSettingsModal, Object.assign({}, CaptionsSettingsModalProps, { strings: modalStrings, spokenLanguageStrings: spokenLanguageStrings, captionLanguageStrings: captionLanguageStrings, showModal: props.showCaptionsSettingsModal, onDismissCaptionsSettings: props.onDismissCaptionsSettings, changeCaptionLanguage: props.changeCaptionLanguage }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const MINIMUM_TOUCH_TARGET_HEIGHT_REM = 3;
/**
 * @private
 */
const groupCallLeaveButtonStyle = {
    root: {
        border: '0.125rem',
        borderRadius: 2,
        width: '6.5625rem'
    },
    flexContainer: {
        flexFlow: 'row'
    }
};
/**
 * @private
 */
const groupCallLeaveButtonCompressedStyle = {
    root: {
        border: '0',
        borderRadius: '0.5rem'
    },
    flexContainer: {
        flexFlow: 'row'
    }
};
/**
 * Styles that can be applied to ensure flyout items have the minimum touch target size.
 *
 * @private
 */
const buttonFlyoutIncreasedSizeStyles = {
    root: {
        height: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM}rem`,
        lineHeight: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM}rem`,
        maxHeight: 'unset',
        overflow: 'hidden'
    },
    linkContent: {
        height: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM}rem`,
        lineHeight: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM}rem`,
        maxHeight: 'unset'
    },
    icon: {
        maxHeight: 'unset',
        textAlign: 'center'
    },
    checkmarkIcon: {
        lineHeight: '2rem'
    }
};
/**
 * @private
 */
const participantButtonWithIncreasedTouchTargets = {
    menuStyles: {
        menuItemStyles: buttonFlyoutIncreasedSizeStyles,
        participantListStyles: {
            participantItemStyles: {
                root: {
                    height: `${MINIMUM_TOUCH_TARGET_HEIGHT_REM}rem`
                },
                participantSubMenuItemsStyles: buttonFlyoutIncreasedSizeStyles
            }
        }
    }
};
/**
 * @private
 */
const devicesButtonWithIncreasedTouchTargets = {
    menuStyles: {
        menuItemStyles: buttonFlyoutIncreasedSizeStyles
    }
};
/**
 * @private
 */
const controlButtonBaseStyle = {
    label: {
        minWidth: '1.25rem'
    }
};
/**
 * @private
 */
const concatButtonBaseStyles = (...styles) => {
    let result = controlButtonBaseStyle;
    styles.forEach(style => {
        result = react.concatStyleSets(result, style);
    });
    return result;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const icon$3 = () => React.createElement(reactIcons.MoreHorizontal20Filled, { key: 'chatOnIconKey', primaryFill: "currentColor" });
/**
 * @private
 */
const MoreButton = (props) => {
    return React.createElement(ControlBarButton, Object.assign({}, props, { labelKey: 'optionsButtonLabelKey', showLabel: props.showLabel, onRenderOnIcon: icon$3, onRenderOffIcon: icon$3, onClick: props.onClick, "data-ui-id": props['data-ui-id'] }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$p = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 *
 * @private
 */
const CaptionsBannerMoreButton = (props) => {
    const localeStrings = useLocale();
    const startCaptionsButtonProps = usePropsFor$1(StartCaptionsButton);
    const moreButtonStrings = React.useMemo(() => ({
        label: localeStrings.strings.call.captionsBannerMoreButtonCallingLabel,
        tooltipOffContent: localeStrings.strings.call.captionsBannerMoreButtonTooltip
    }), [localeStrings]);
    const moreButtonContextualMenuItems = [];
    const startCaptions = React.useCallback(() => __awaiter$p(void 0, void 0, void 0, function* () {
        yield startCaptionsButtonProps.onStartCaptions({
            spokenLanguage: startCaptionsButtonProps.currentSpokenLanguage
        });
    }), [startCaptionsButtonProps]);
    const stopCaptions = React.useCallback(() => __awaiter$p(void 0, void 0, void 0, function* () {
        yield startCaptionsButtonProps.onStopCaptions();
    }), [startCaptionsButtonProps]);
    moreButtonContextualMenuItems.push({
        key: 'ToggleCaptionsKey',
        text: startCaptionsButtonProps.checked ? localeStrings.strings.call.startCaptionsButtonTooltipOnContent : localeStrings.strings.call.startCaptionsButtonTooltipOffContent,
        onClick: () => {
            startCaptionsButtonProps.checked ? stopCaptions() : startCaptionsButtonProps.currentSpokenLanguage !== '' ? startCaptions() : props.onCaptionsSettingsClick && props.onCaptionsSettingsClick();
        },
        iconProps: {
            iconName: startCaptionsButtonProps.checked ? 'CaptionsOffIcon' : 'CaptionsIcon',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        },
        itemProps: {
            styles: buttonFlyoutIncreasedSizeStyles
        }
    });
    if (props.onCaptionsSettingsClick) {
        moreButtonContextualMenuItems.push({
            key: 'openCaptionsSettingsKey',
            id: 'common-call-composite-captions-settings-button',
            text: localeStrings.strings.call.captionsSettingsLabel,
            onClick: props.onCaptionsSettingsClick,
            iconProps: {
                iconName: 'CaptionsSettingsIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            disabled: !startCaptionsButtonProps.checked
        });
    }
    return React.createElement(MoreButton, Object.assign({}, props, { "data-ui-id": "captions-banner-more-button", strings: moreButtonStrings, menuIconProps: {
            hidden: true
        }, menuProps: {
            items: moreButtonContextualMenuItems,
            calloutProps: {
                preventDismissOnEvent: _preventDismissOnEvent
            }
        } }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const mobileViewBannerWidth = '95%';
/** @private */
const CallingCaptionsBanner = (props) => {
    var _a;
    const captionsBannerProps = usePropsFor$1(CaptionsBanner);
    const [isCaptionsSettingsOpen, setIsCaptionsSettingsOpen] = React.useState(false);
    const onClickCaptionsSettings = () => {
        setIsCaptionsSettingsOpen(true);
    };
    const onDismissCaptionsSettings = () => {
        setIsCaptionsSettingsOpen(false);
    };
    const containerClassName = react.mergeStyles(((_a = props.captionsOptions) === null || _a === void 0 ? void 0 : _a.height) === 'full' ? react.mergeStyles({
        position: 'absolute',
        height: '100%',
        width: '100%'
    }) : {
        position: 'relative'
    });
    const floatingChildClassName = react.mergeStyles({
        position: 'absolute',
        right: 0,
        top: 0
    });
    const strings = useLocale().strings.call;
    const captionsBannerStrings = {
        captionsBannerSpinnerText: strings.captionsBannerSpinnerText,
        realTimeTextInputBoxDefaultText: strings.realTimeTextInputBoxDefaultText,
        realTimeTextInputErrorMessage: strings.realTimeTextInputErrorMessage,
        realTimeTextBannerContent: strings.realTimeTextBannerContent,
        realTimeTextBannerTitle: strings.realTimeTextBannerTitle,
        realTimeTextBannerLinkLabel: strings.realTimeTextBannerLinkLabel,
        captionsOnlyContainerTitle: strings.captionsOnlyContainerTitle,
        realTimeTextOnlyContainerTitle: strings.realTimeTextOnlyContainerTitle,
        captionsAndRealTimeTextContainerTitle: strings.captionsAndRealTimeTextContainerTitle,
        expandButtonAriaLabel: strings.expandButtonAriaLabel,
        minimizeButtonAriaLabel: strings.minimizeButtonAriaLabel
    };
    const onRenderAvatar = React.useCallback((userId, options) => {
        return React.createElement(AvatarPersona, Object.assign({ userId: userId }, options, { dataProvider: props.onFetchAvatarPersonaData }));
    }, [props.onFetchAvatarPersonaData]);
    const { innerWidth: width } = window;
    const [windowWidth, setWindowWidth] = React.useState(width);
    React.useEffect(() => {
        function handleResize() {
            setWindowWidth(window.innerWidth);
        }
        window.addEventListener('resize', handleResize);
        return () => window.removeEventListener('resize', handleResize);
    }, []);
    const desktopViewBannerWidth = windowWidth > 620 ? '35rem' : '80%';
    return React.createElement(React.Fragment, null,
        isCaptionsSettingsOpen && React.createElement(CallingCaptionsSettingsModal, { showCaptionsSettingsModal: isCaptionsSettingsOpen, onDismissCaptionsSettings: onDismissCaptionsSettings, changeCaptionLanguage: props.useTeamsCaptions }),
        React.createElement("div", { className: containerClassName, role: "region", "aria-label": strings.liveCaptionsLabel },
            React.createElement(react.Stack, { horizontalAlign: "center" },
                React.createElement(react.Stack.Item, { style: {
                        width: props.isMobile ? mobileViewBannerWidth : desktopViewBannerWidth
                    } },
                    React.createElement(CaptionsBanner, Object.assign({ captionsOptions: props.captionsOptions, onRenderAvatar: onRenderAvatar, formFactor: props.isMobile ? 'compact' : 'default', strings: captionsBannerStrings }, captionsBannerProps, { isRealTimeTextOn: props.isRealTimeTextOn || captionsBannerProps.isRealTimeTextOn })))),
            !props.isMobile && captionsBannerProps.isCaptionsOn && React.createElement("div", { className: floatingChildClassName },
                React.createElement(CaptionsBannerMoreButton, { onCaptionsSettingsClick: onClickCaptionsSettings }))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const containerDivStyles = {
    position: 'relative',
    width: '100%',
    height: '100%'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * This minimum width is to make sure the side pane does not overlap with the composite when the screen is narrow.
 *
 * @private
 */
const compositeMinWidthRem = 30;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const Camera = (props) => {
    var _a;
    const cameraButtonProps = usePropsFor$1(CameraButton);
    const styles = React.useMemo(() => { var _a; return concatButtonBaseStyles((_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [props.styles]);
    const isRoomsCall = useSelector$1(getIsRoomsCall);
    const role = useSelector$1(getRole);
    const turnVideoOnCapability = (_a = useSelector$1(getCapabilites)) === null || _a === void 0 ? void 0 : _a.turnVideoOn;
    return React.createElement(CameraButton, Object.assign({ "data-ui-id": "call-composite-camera-button" }, cameraButtonProps, { showLabel: props.displayType !== 'compact', styles: styles, enableDeviceSelectionMenu: props.splitButtonsForDeviceSelection, disableTooltip: props.disableTooltip, disabled: cameraButtonProps.disabled || props.disabled || !!(isRoomsCall && role === 'Unknown'), onRenderOffIcon: turnVideoOnCapability && !turnVideoOnCapability.isPresent ? () => React.createElement(_HighContrastAwareIcon, { disabled: true, iconName: 'ControlButtonCameraProhibited' }) : undefined, onClickVideoEffects: props.onClickVideoEffects, componentRef: props.componentRef }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const Devices = (props) => {
    var _a;
    const devicesButtonProps = usePropsFor$1(DevicesButton);
    const role = (_a = useSelector$1(getRole)) !== null && _a !== void 0 ? _a : 'Unknown';
    const augmentedDeviceButtonProps = React.useMemo(() => (Object.assign(Object.assign({}, devicesButtonProps), { microphones: role === 'Consumer' ? [] : devicesButtonProps.microphones, cameras: role === 'Consumer' ? [] : devicesButtonProps.cameras })), [devicesButtonProps, role]);
    const styles = React.useMemo(() => { var _a; return concatButtonBaseStyles(props.increaseFlyoutItemSize ? devicesButtonWithIncreasedTouchTargets : {}, (_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [props.increaseFlyoutItemSize, props.styles]);
    const locale = useLocale();
    const onlyManageSpeakers = role === 'Consumer';
    const onRenderDevicesIcon = () => {
        return React.createElement(_HighContrastAwareIcon, { disabled: props.disabled, iconName: "OptionsSpeaker" });
    };
    return React.createElement(DevicesButton
    /* By setting `persistMenu?` to true, we prevent options menu from getting hidden every time a participant joins or leaves. */ , Object.assign({ 
        /* By setting `persistMenu?` to true, we prevent options menu from getting hidden every time a participant joins or leaves. */ persistMenu: true }, augmentedDeviceButtonProps, { showLabel: props.displayType !== 'compact', styles: styles, "data-ui-id": "calling-composite-devices-button", disabled: props.disabled, strings: getLabelFromRole(role, locale), onRenderIcon: onlyManageSpeakers ? onRenderDevicesIcon : undefined }));
};
const getLabelFromRole = (role, locale) => {
    if (role === 'Consumer') {
        return {
            label: locale.component.strings.microphoneButton.speakerMenuTitle
        };
    }
    return undefined;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const Prompt = (props) => {
    var _a;
    const theme = useTheme();
    const styles = react.concatStyleSets(modalStyles, (_a = props.styles) !== null && _a !== void 0 ? _a : {});
    return React.createElement(react.Modal, { styles: styles, isOpen: props.isOpen, onDismiss: props.onDismiss, isBlocking: false },
        React.createElement(react.Stack, { className: react.mergeStyles({
                position: 'relative'
            }) },
            React.createElement(react.Text, { className: react.mergeStyles({
                    fontWeight: 600,
                    fontSize: '1.25rem'
                }) }, props.heading),
            React.createElement(react.IconButton, { styles: iconButtonStyles(theme), iconProps: {
                    iconName: 'Cancel'
                }, onClick: props.onCancel, ariaLabel: props.closeButtonLabel })),
        React.createElement(react.Stack, { verticalAlign: "center", className: react.mergeStyles({
                minHeight: '6rem'
            }) },
            React.createElement(react.Text, { className: react.mergeStyles({
                    fontSize: '0.875rem'
                }) }, props.text)),
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: "end", tokens: {
                childrenGap: '0.5rem'
            } },
            React.createElement(react.PrimaryButton, { styles: buttonTextStyles, text: props.confirmButtonLabel, onClick: props.onConfirm }),
            React.createElement(react.DefaultButton, { styles: buttonTextStyles, text: props.cancelButtonLabel, onClick: props.onCancel })));
};
const iconButtonStyles = (theme) => ({
    root: {
        position: 'absolute',
        top: 0,
        right: 0,
        color: theme.palette.neutralPrimary
    },
    rootHovered: {
        color: theme.palette.neutralDark
    }
});
const modalStyles = {
    main: {
        padding: '1.5rem ',
        maxWidth: '30rem'
    }
};
const buttonTextStyles = {
    label: {
        fontSize: '0.875rem'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$o = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/** @private */
const EndCall = (props) => {
    var _a;
    const compactMode = props.displayType === 'compact';
    const hangUpButtonProps = usePropsFor$1(EndCallButton);
    const localeStrings = useLocale().strings;
    const endCallDiaglogLabels = React.useMemo(() => ({
        confirmButtonLabel: localeStrings.call.endCallConfirmButtonLabel,
        heading: localeStrings.call.endCallConfirmDialogTitle,
        text: localeStrings.call.leaveConfirmDialogContent,
        closeButtonLabel: localeStrings.call.close
    }), [localeStrings.call.endCallConfirmButtonLabel, localeStrings.call.endCallConfirmDialogTitle, localeStrings.call.leaveConfirmDialogContent, localeStrings.call.close]);
    const leaveDiaglogLabels = React.useMemo(() => ({
        confirmButtonLabel: localeStrings.call.leaveConfirmButtonLabel,
        heading: localeStrings.call.leaveConfirmDialogTitle,
        text: localeStrings.call.leaveConfirmDialogContent,
        closeButtonLabel: localeStrings.call.close
    }), [localeStrings.call.leaveConfirmButtonLabel, localeStrings.call.leaveConfirmDialogContent, localeStrings.call.leaveConfirmDialogTitle, localeStrings.call.close]);
    const [dialogLabels, setDialogLabels] = React.useState(leaveDiaglogLabels);
    const { onHangUp } = hangUpButtonProps;
    const [showHangUpConfirm, setShowHangUpConfirm] = React.useState(false);
    const toggleConfirm = React.useCallback(() => {
        setShowHangUpConfirm(!showHangUpConfirm);
    }, [showHangUpConfirm]);
    const onHangUpConfirm = React.useCallback((hangUpForEveryone) => {
        onHangUp && onHangUp(hangUpForEveryone);
        toggleConfirm();
    }, [onHangUp, toggleConfirm]);
    const hangUpOverride = React.useCallback((forEveryone) => __awaiter$o(void 0, void 0, void 0, function* () {
        if (props.mobileView || forEveryone === false /* value being false(not undefined) because it comes from endCall option */) {
            onHangUp();
            return;
        }
        setDialogLabels(forEveryone ? endCallDiaglogLabels : leaveDiaglogLabels);
        toggleConfirm();
    }), [endCallDiaglogLabels, leaveDiaglogLabels, onHangUp, props.mobileView, toggleConfirm]);
    const styles = React.useMemo(() => { var _a; return react.concatStyleSets(compactMode ? groupCallLeaveButtonCompressedStyle : groupCallLeaveButtonStyle, (_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [compactMode, props.styles]);
    const enableBreakoutRoomMenu = !!props.returnFromBreakoutRoom;
    const breakoutRoomMenuProps = {
        items: [{
                key: 'returnToMainMeeting',
                text: localeStrings.call.returnFromBreakoutRoomButtonLabel,
                title: localeStrings.call.returnFromBreakoutRoomButtonLabel,
                onClick: () => {
                    var _a;
                    (_a = props.returnFromBreakoutRoom) === null || _a === void 0 ? void 0 : _a.call(props);
                }
            }, {
                key: 'leaveRoomAndMainMeeting',
                text: localeStrings.call.leaveBreakoutRoomAndMeetingButtonLabel,
                title: localeStrings.call.leaveBreakoutRoomAndMeetingButtonLabel,
                onClick: () => onHangUp()
            }],
        styles: props.styles
    };
    return React.createElement(react.Stack, null,
        React.createElement(EndCallButton, Object.assign({ "data-ui-id": "call-composite-hangup-button" }, hangUpButtonProps, { onHangUp: props.disableEndCallModal ? onHangUp : hangUpOverride, styles: styles, showLabel: !compactMode, enableEndCallMenu: (_a = props.enableEndCallMenu) !== null && _a !== void 0 ? _a : false, menuProps: enableBreakoutRoomMenu ? breakoutRoomMenuProps : undefined })),
        React.createElement(Prompt, Object.assign({}, dialogLabels, { styles: {
                main: {
                    minWidth: '22.5rem',
                    padding: '1.5rem'
                }
            }, cancelButtonLabel: localeStrings.call.hangUpCancelButtonLabel, onConfirm: () => onHangUpConfirm(props.enableEndCallMenu), isOpen: showHangUpConfirm, onCancel: toggleConfirm })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const Microphone = (props) => {
    var _a;
    const microphoneButtonProps = usePropsFor$1(MicrophoneButton);
    const callStatus = useSelector$1(getCallStatus);
    const isLocalMicrophoneEnabled = useSelector$1(getLocalMicrophoneEnabled);
    const isRoomsCall = useSelector$1(getIsRoomsCall);
    const role = useSelector$1(getRole);
    const unmuteMicCapability = (_a = useSelector$1(getCapabilites)) === null || _a === void 0 ? void 0 : _a.unmuteMic;
    /**
     * When call is in connecting state, microphone button should be disabled.
     * This is due to to headless limitation where a call can not be muted/unmuted in lobby.
     */
    if (callStatus === 'Connecting') {
        // Lobby page should show the microphone status that was set on the local preview/configuration
        // page until the user successfully joins the call.
        microphoneButtonProps.checked = isLocalMicrophoneEnabled;
    }
    const styles = React.useMemo(() => { var _a; return concatButtonBaseStyles((_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [props.styles]);
    // tab focus on MicrophoneButton on page load
    return React.createElement(MicrophoneButton, Object.assign({ "data-ui-id": "call-composite-microphone-button" }, microphoneButtonProps, { isDeepNoiseSuppressionOn: props.isDeepNoiseSuppressionOn, onClickNoiseSuppression: props.onClickNoiseSuppression, showNoiseSuppressionButton: props.showNoiseSuppressionButton, showLabel: props.displayType !== 'compact', disableTooltip: props.disableTooltip, styles: styles, enableDeviceSelectionMenu: props.splitButtonsForDeviceSelection, disabled: microphoneButtonProps.disabled || props.disabled || !!(isRoomsCall && role === 'Unknown'), onRenderOffIcon: unmuteMicCapability && !unmuteMicCapability.isPresent ? () => React.createElement(_HighContrastAwareIcon, { disabled: true, iconName: 'ControlButtonMicProhibited' }) : undefined }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const Participants = (props) => {
    const participantsButtonProps = usePropsFor$1(ParticipantsButton);
    const participantsButtonStyles = React.useMemo(() => { var _a; return concatButtonBaseStyles(props.increaseFlyoutItemSize ? participantButtonWithIncreasedTouchTargets : {}, (_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [props.increaseFlyoutItemSize, props.styles]);
    return React.createElement(ParticipantsButton, Object.assign({ "data-ui-id": "call-composite-participants-button" }, participantsButtonProps, { showLabel: props.displayType !== 'compact', callInvitationURL: props.callInvitationURL, onFetchParticipantMenuItems: props.onFetchParticipantMenuItems, disabled: isDisabled$3(props.option) || props.disabled, styles: participantsButtonStyles, showParticipantOverflowTooltip: !props.isMobile }));
};
const isDisabled$3 = (option) => {
    if (option === undefined || option === true || option === false) {
        return false;
    }
    return option.disabled;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const ScreenShare = (props) => {
    const screenShareButtonProps = usePropsFor$1(ScreenShareButton);
    const styles = React.useMemo(() => { var _a; return concatButtonBaseStyles((_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [props.styles]);
    const isRoomsCall = useSelector$1(getIsRoomsCall);
    const role = useSelector$1(getRole);
    const screenShareButtonDisabled = () => {
        var _a;
        return (_a = screenShareButtonProps === null || screenShareButtonProps === void 0 ? void 0 : screenShareButtonProps.disabled) !== null && _a !== void 0 ? _a : isDisabled$2(props.option);
    };
    return React.createElement(ScreenShareButton, Object.assign({ "data-ui-id": "call-composite-screenshare-button" }, screenShareButtonProps, { showLabel: props.displayType !== 'compact', disabled: screenShareButtonDisabled() || props.disabled || !!(isRoomsCall && role === 'Unknown'), styles: styles }));
};
const isDisabled$2 = (option) => {
    if (option === undefined || option === true || option === false) {
        return false;
    }
    return option.disabled;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const icon$2 = () => React.createElement(CallCompositeIcon, { iconName: 'ControlButtonParticipants' });
/**
 * @private
 */
/** @beta */
const People = (props) => {
    const { strings, onRenderOnIcon, onRenderOffIcon, onClick } = props;
    const theme = useTheme();
    const styles = React.useMemo(() => {
        var _a;
        return react.concatStyleSets({
            rootChecked: {
                background: theme.palette.neutralLight
            }
        }, (_a = props.styles) !== null && _a !== void 0 ? _a : {}, controlButtonBaseStyle);
    }, [props.styles, theme.palette.neutralLight]);
    return React.createElement(ControlBarButton, Object.assign({}, props, { "data-ui-id": "call-composite-participants-button", strings: strings, labelKey: 'peopleButtonLabelKey', onRenderOnIcon: onRenderOnIcon !== null && onRenderOnIcon !== void 0 ? onRenderOnIcon : icon$2, onRenderOffIcon: onRenderOffIcon !== null && onRenderOffIcon !== void 0 ? onRenderOffIcon : icon$2, onClick: onClick, styles: styles }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const controlBarContainerStyles = {
    paddingTop: '0.25rem',
    paddingBottom: '0.25rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const VIDEO_GALLERY_Z_INDEX = 1;
// The control bar must be in a higher z-band than the video gallery so the drop shadow appears on top of the video gallery
/** @private */
const CONTROL_BAR_Z_INDEX = VIDEO_GALLERY_Z_INDEX + 1;
// The notification container should be in the highest z-band to ensure it shows on top of all other content.
const NOTIFICATION_CONTAINER_Z_INDEX = Math.max(CONTROL_BAR_Z_INDEX, VIDEO_GALLERY_Z_INDEX) + 1;
/** @private */
const DRAWER_Z_INDEX = NOTIFICATION_CONTAINER_Z_INDEX + 1;
/**
 * @private
 */
const callControlsContainerStyles = react.mergeStyles(controlBarContainerStyles, {
    zIndex: CONTROL_BAR_Z_INDEX
});
const containerStyle$2 = {
    width: '100%',
    position: 'relative',
    minHeight: '13rem' // linked to minimum space allocated to media gallery
};
/**
 * @private
 */
const containerStyleDesktop = react.mergeStyles(Object.assign(Object.assign({}, containerStyle$2), { minWidth: '30rem' }));
/**
 * @private
 */
const containerStyleMobile = react.mergeStyles(Object.assign(Object.assign({}, containerStyle$2), { minWidth: '17.5rem' }));
/**
 * @private
 */
const galleryParentContainerStyles = (backgroundColor) => ({
    root: {
        zIndex: VIDEO_GALLERY_Z_INDEX,
        width: '100%',
        background: backgroundColor
    }
});
/**
 * @private
 */
const mediaGalleryContainerStyles$1 = {
    root: {
        height: '100%',
        width: '100%'
    }
};
/**
 * @private
 */
const notificationsContainerStyles = {
    root: {
        width: '100%',
        position: 'absolute',
        top: 0,
        left: 0,
        padding: '1rem',
        zIndex: NOTIFICATION_CONTAINER_Z_INDEX,
        pointerEvents: 'none' // to allow the operation of controls underneath the notification container
    }
};
/**
 * @private
 */
const bannerNotificationStyles = {
    root: {
        zIndex: NOTIFICATION_CONTAINER_Z_INDEX,
        pointerEvents: 'auto' // to allow the dismissal or error and warning bars in the notification container
    }
};
/**
 * @private
 */
const notificationStackStyles = {
    root: {
        zIndex: NOTIFICATION_CONTAINER_Z_INDEX,
        pointerEvents: 'auto',
        // to allow the dismissal or error and warning bars in the notification container,
        marginBottom: '0.5rem'
    }
};
/**
 * @private
 */
const callArrangementContainerStyles = (verticalControlBar) => {
    return {
        root: {
            width: '100%',
            height: '100%',
            flexDirection: verticalControlBar ? 'unset' : 'column'
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const RaiseHand = (props) => {
    const raiseHandButtonProps = usePropsFor$1(RaiseHandButton);
    const callStatus = useSelector$1(getCallStatus);
    const styles = React.useMemo(() => { var _a; return concatButtonBaseStyles((_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [props.styles]);
    let raiseHandButtonDisabled = isDisabled$1(props.option);
    if (_isInLobbyOrConnecting(callStatus)) {
        raiseHandButtonDisabled = true;
    }
    return React.createElement(RaiseHandButton, Object.assign({ "data-ui-id": "call-composite-raisehand-button" }, raiseHandButtonProps, { showLabel: props.displayType !== 'compact', disabled: raiseHandButtonDisabled || props.disabled, styles: styles }));
};
const isDisabled$1 = (option) => {
    if (option === undefined || option === true || option === false) {
        return false;
    }
    return option.disabled;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// eslint-disable-next-line no-restricted-imports
/**
 * Max number of Custom Buttons in primary and secondary ControlBar
 * Does not include existing buttons in the controlBar.
 *
 * @private
 */
const CUSTOM_BUTTON_OPTIONS = {
    MAX_PRIMARY_DESKTOP_CUSTOM_BUTTONS: 3,
    MAX_PRIMARY_MOBILE_CUSTOM_BUTTONS: 1,
    MAX_SECONDARY_DESKTOP_CUSTOM_BUTTONS: 2
};
/** @private */
const generateCustomCallControlBarButton = (onFetchCustomButtonProps, displayType) => {
    const allButtonProps = onFetchCustomButtonProps === null || onFetchCustomButtonProps === void 0 ? void 0 : onFetchCustomButtonProps.map(callback => callback({
        displayType
    }));
    return {
        primary: generateCustomControlBarButtons('primary', allButtonProps),
        secondary: generateCustomControlBarButtons('secondary', allButtonProps),
        overflow: generateCustomControlBarButtons('overflow', allButtonProps)
    };
};
/** @private */
const generateCustomControlBarButtons = (placement, customButtons) => customButtons ? customButtons.filter(buttonProps => buttonProps.placement === placement).map((buttonProps, i) => internalProps => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    // onRenderButton is a custom prop that can be passed to override the default rendering of the button
    // This is useful for custom buttons that need to render more than just an icon and label and is used
    // for CallWithChat button notification badge. However, onRenderButton is not a public prop and is
    // not documented in the API. This is a temporary solution and will need to be revisited.
    if ('onRenderButton' in buttonProps) {
        return buttonProps['onRenderButton'](internalProps);
    }
    return React.createElement(ControlBarButton, { ariaDescription: (_b = (_a = buttonProps.strings) === null || _a === void 0 ? void 0 : _a.ariaDescription) !== null && _b !== void 0 ? _b : internalProps.ariaDescription, ariaLabel: (_f = (_d = (_c = buttonProps.strings) === null || _c === void 0 ? void 0 : _c.ariaLabel) !== null && _d !== void 0 ? _d : (_e = buttonProps.strings) === null || _e === void 0 ? void 0 : _e.tooltipContent) !== null && _f !== void 0 ? _f : internalProps.ariaLabel, disabled: (_g = buttonProps.disabled) !== null && _g !== void 0 ? _g : internalProps.disabled, id: (_h = buttonProps.id) !== null && _h !== void 0 ? _h : internalProps.id, key: `${buttonProps.placement}_${i}`, onClick: (_j = buttonProps.onItemClick) !== null && _j !== void 0 ? _j : internalProps.onClick, onRenderIcon: () => { var _a, _b, _c; return React.createElement(react.Icon, { iconName: (_c = (_a = buttonProps.iconName) !== null && _a !== void 0 ? _a : (_b = internalProps.iconProps) === null || _b === void 0 ? void 0 : _b.iconName) !== null && _c !== void 0 ? _c : 'DefaultCustomButton' }); }, showLabel: (_k = buttonProps.showLabel) !== null && _k !== void 0 ? _k : internalProps.showLabel, styles: react.mergeStyleSets(internalProps.styles), strings: buttonProps.strings });
}) : [];
/** @private */
const generateCustomDrawerButtons = (placement, customButtons) => customButtons ? customButtons.filter(buttonProps => buttonProps.placement === placement).map((buttonProps, i) => {
    var _a, _b;
    return (Object.assign(Object.assign({}, buttonProps), { disabled: buttonProps.disabled, iconProps: {
            iconName: (_a = buttonProps.iconName) !== null && _a !== void 0 ? _a : 'DefaultCustomButton',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        }, id: buttonProps.id, itemKey: `${buttonProps.placement}_${i}`, onItemClick: buttonProps.onItemClick, text: (_b = buttonProps.strings) === null || _b === void 0 ? void 0 : _b.label }));
}) : [];
/** @private */
const generateCustomCallDrawerButtons = (onFetchCustomButtonProps, displayType) => {
    const customButtons = onFetchCustomButtonProps === null || onFetchCustomButtonProps === void 0 ? void 0 : onFetchCustomButtonProps.map(callback => callback({
        displayType
    }));
    return {
        primary: generateCustomDrawerButtons('primary', customButtons),
        secondary: generateCustomDrawerButtons('secondary', customButtons),
        overflow: generateCustomDrawerButtons('overflow', customButtons)
    };
};
/** @private */
const generateCustomCallDesktopOverflowButtons = (onFetchCustomButtonProps, displayType) => {
    const customButtons = onFetchCustomButtonProps === null || onFetchCustomButtonProps === void 0 ? void 0 : onFetchCustomButtonProps.map(callback => callback({
        displayType
    }));
    return {
        primary: generateCustomDrawerButtons('primary', customButtons).map(drawerMenuItemToContextualMenuItem),
        secondary: generateCustomDrawerButtons('secondary', customButtons).map(drawerMenuItemToContextualMenuItem),
        overflow: generateCustomDrawerButtons('overflow', customButtons).map(drawerMenuItemToContextualMenuItem)
    };
};
/** @private */
const drawerMenuItemToContextualMenuItem = (item) => (Object.assign(Object.assign({}, item), { key: item.itemKey, onClick: item.onItemClick ? ev => {
        var _a;
        (_a = item.onItemClick) === null || _a === void 0 ? void 0 : _a.call(item, ev);
    } : undefined, subMenuProps: item.subMenuProps ? {
        items: item.subMenuProps.map(drawerMenuItemToContextualMenuItem)
    } : undefined }));
/** @private */
const onFetchCustomButtonPropsTrampoline = (options) => {
    let response = undefined;
    response = options === null || options === void 0 ? void 0 : options.onFetchCustomButtonProps;
    return response;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const Reaction = (props) => {
    const reactionButtonProps = usePropsFor$1(ReactionButton);
    const styles = React.useMemo(() => { var _a; return concatButtonBaseStyles((_a = props.styles) !== null && _a !== void 0 ? _a : {}); }, [props.styles]);
    const reactionButtonDisabled = isDisabled(props.option) || reactionButtonProps.disabled;
    return React.createElement(ReactionButton, Object.assign({ "data-ui-id": "call-composite-reaction-button" }, reactionButtonProps, { showLabel: props.displayType !== 'compact', disabled: reactionButtonDisabled || props.disabled, styles: styles, disableTooltip: props.disableTooltip, persistMenu: true, reactionResources: props.reactionResource }));
};
const isDisabled = (option) => {
    if (option === undefined || option === true || option === false) {
        return false;
    }
    return option.disabled;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const capabilitySelector = reselect__namespace.createSelector([getCapabilites], capabilities => {
    return {
        capabilities
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const callStatusSelector = reselect.createSelector([getCallStatus, getIsScreenShareOn], (callStatus, isScreenShareOn) => {
    return {
        callStatus,
        isScreenShareOn
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$n = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// Enforce a background color on control bar to ensure it matches the composite background color.
const controlBarStyles = react.memoizeFunction((background) => ({
    root: {
        background: background
    }
}));
const inferCallControlOptions = (mobileView, callControlOptions) => {
    if (callControlOptions === false) {
        return {};
    }
    const options = callControlOptions === true || callControlOptions === undefined ? {} : callControlOptions;
    if (mobileView) {
        // Set options to always not show screen share button for mobile
        options.screenShareButton = false;
    }
    return options;
};
/**
 * @private
 */
const CallControls = (props) => {
    var _a;
    const options = React.useMemo(() => inferCallControlOptions(!!props.isMobile, props.options), [props.isMobile, props.options]);
    const localeStrings = useLocale();
    const [isDeepNoiseSuppressionOn, setDeepNoiseSuppressionOn] = React.useState(false);
    const adapter = useAdapter();
    const startDeepNoiseSuppression = React.useCallback(() => __awaiter$n(void 0, void 0, void 0, function* () {
        yield adapter.startNoiseSuppressionEffect();
    }), [adapter]);
    const deepNoiseSuppresionEffectsDependency = useSelector$1(getDeepNoiseSuppresionEffectsDependency);
    const deepNoiseSuppressionOnByDefault = useSelector$1(getDeepNoiseSuppresionIsOnByDefault);
    React.useEffect(() => {
        if (deepNoiseSuppresionEffectsDependency && deepNoiseSuppressionOnByDefault) {
            startDeepNoiseSuppression();
            setDeepNoiseSuppressionOn(true);
        }
    }, [deepNoiseSuppresionEffectsDependency, deepNoiseSuppressionOnByDefault, startDeepNoiseSuppression]);
    const environmentInfo = useSelector$1(getEnvironmentInfo);
    const isSafari = _isSafari(environmentInfo);
    const hideDeepNoiseSuppressionButton = useSelector$1(getHideDeepNoiseSupressionButton);
    const showNoiseSuppressionButton = !!(deepNoiseSuppresionEffectsDependency && !hideDeepNoiseSuppressionButton && !isSafari);
    const onClickNoiseSuppression = React.useCallback(() => __awaiter$n(void 0, void 0, void 0, function* () {
        if (isDeepNoiseSuppressionOn) {
            yield adapter.stopNoiseSuppressionEffect();
            setDeepNoiseSuppressionOn(false);
        }
        else {
            yield adapter.startNoiseSuppressionEffect();
            setDeepNoiseSuppressionOn(true);
        }
    }), [adapter, isDeepNoiseSuppressionOn]);
    const peopleButtonStrings = React.useMemo(() => ({
        label: localeStrings.strings.call.peopleButtonLabel,
        tooltipOffContent: localeStrings.strings.call.peopleButtonTooltipOpen,
        tooltipOnContent: localeStrings.strings.call.peopleButtonTooltipClose
    }), [localeStrings]);
    const moreButtonStrings = React.useMemo(() => ({
        label: localeStrings.strings.call.moreButtonCallingLabel,
        tooltipOffContent: localeStrings.strings.callWithChat.moreDrawerButtonTooltip
    }), [localeStrings]);
    const holdButtonProps = usePropsFor$1(HoldButton);
    const raiseHandButtonProps = usePropsFor$1(RaiseHandButton);
    const capabilitiesSelector = useSelector$1(capabilitySelector);
    const callState = useSelector$1(callStatusSelector);
    const isReactionAllowed = callState.callStatus !== 'Connected' || !(capabilitiesSelector === null || capabilitiesSelector === void 0 ? void 0 : capabilitiesSelector.capabilities) || capabilitiesSelector.capabilities.useReactions.isPresent;
    const devicesButtonProps = usePropsFor$1(DevicesButton);
    let numberOfButtons = 0;
    const screenShareButtonIsEnabled = isEnabled$2(options === null || options === void 0 ? void 0 : options.screenShareButton);
    if (screenShareButtonIsEnabled) {
        numberOfButtons++;
    }
    const microphoneButtonIsEnabled = isEnabled$2(options === null || options === void 0 ? void 0 : options.microphoneButton);
    if (microphoneButtonIsEnabled) {
        numberOfButtons++;
    }
    const cameraButtonIsEnabled = isEnabled$2(options === null || options === void 0 ? void 0 : options.cameraButton);
    if (cameraButtonIsEnabled) {
        numberOfButtons++;
    }
    if (isEnabled$2(options === null || options === void 0 ? void 0 : options.endCallButton)) {
        numberOfButtons++;
    }
    const showParticipantsButtonInControlBar = isEnabled$2(options === null || options === void 0 ? void 0 : options.participantsButton) && !props.isMobile;
    if (showParticipantsButtonInControlBar) {
        numberOfButtons++;
    }
    const showReactionButtonInControlBar = isEnabled$2(options === null || options === void 0 ? void 0 : options.reactionButton) && isReactionAllowed && !props.isMobile;
    if (showReactionButtonInControlBar) {
        numberOfButtons++;
    }
    const isRoomsCall = useSelector$1(getIsRoomsCall);
    const moreButtonContextualMenuItems = () => {
        const items = [];
        if (props.isMobile && props.onPeopleButtonClicked && isEnabled$2(options === null || options === void 0 ? void 0 : options.participantsButton)) {
            items.push({
                key: 'peopleButtonKey',
                text: localeStrings.component.strings.participantsButton.label,
                onClick: () => {
                    if (props.onPeopleButtonClicked) {
                        props.onPeopleButtonClicked();
                    }
                },
                iconProps: {
                    iconName: 'ControlButtonParticipantsContextualMenuItem',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                itemProps: {
                    styles: buttonFlyoutIncreasedSizeStyles
                },
                disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.participantsButton),
                ['data-ui-id']: 'call-composite-more-menu-people-button'
            });
        }
        if (!isRoomsCall) {
            items.push({
                key: 'holdButtonKey',
                text: localeStrings.component.strings.holdButton.tooltipOffContent,
                onClick: () => {
                    holdButtonProps.onToggleHold();
                },
                iconProps: {
                    iconName: 'HoldCallContextualMenuItem',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                itemProps: {
                    styles: buttonFlyoutIncreasedSizeStyles
                },
                disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.holdButton),
                ['data-ui-id']: 'hold-button'
            });
        }
        return items;
    };
    const customDrawerButtons = React.useMemo(() => generateCustomCallDesktopOverflowButtons(onFetchCustomButtonPropsTrampoline(typeof options === 'object' ? options : undefined), typeof options === 'object' ? options.displayType : undefined), [options]);
    const moreButtonMenuItems = moreButtonContextualMenuItems();
    let showMoreButton = isEnabled$2(options === null || options === void 0 ? void 0 : options.moreButton) && moreButtonMenuItems.length > 0;
    if (showMoreButton) {
        numberOfButtons++;
    }
    const customButtons = React.useMemo(() => generateCustomCallControlBarButton(onFetchCustomButtonPropsTrampoline(options), options === null || options === void 0 ? void 0 : options.displayType), [options]);
    numberOfButtons += React.Children.count(customButtons['primary']) + React.Children.count(customButtons['secondary']);
    let showDevicesButtonInControlBar = isEnabled$2(options === null || options === void 0 ? void 0 : options.devicesButton);
    if (showDevicesButtonInControlBar && (props.isMobile ? numberOfButtons < 5 : true)) {
        numberOfButtons++;
    }
    else {
        showDevicesButtonInControlBar = false;
        showMoreButton = isEnabled$2(options === null || options === void 0 ? void 0 : options.moreButton);
    }
    const reactionResources = useSelector$1(getReactionResources);
    const raiseHandButtonIsEnabled = isEnabled$2(options === null || options === void 0 ? void 0 : options.raiseHandButton);
    let showRaiseHandButtonInControlBar = raiseHandButtonIsEnabled;
    const role = useSelector$1(getRole);
    const hideRaiseHandButtonInRoomsCall = isRoomsCall && role && ['Consumer', 'Unknown'].includes(role);
    if (showRaiseHandButtonInControlBar && (props.isMobile ? numberOfButtons < 5 : true)) {
        numberOfButtons++;
    }
    else {
        // If more button is not present but enabled then replace previous button (devices button) with more button
        if (!showMoreButton && isEnabled$2(options === null || options === void 0 ? void 0 : options.moreButton)) {
            showMoreButton = true;
            showDevicesButtonInControlBar = false;
        }
        showRaiseHandButtonInControlBar = false;
    }
    if (!showDevicesButtonInControlBar) {
        const devicesButtonMenu = generateDefaultDeviceMenuProps(devicesButtonProps, localeStrings.component.strings.devicesButton);
        moreButtonMenuItems.push({
            key: 'devicesButtonKey',
            text: localeStrings.component.strings.devicesButton.label,
            iconProps: {
                iconName: 'ControlButtonOptions',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            subMenuProps: devicesButtonMenu,
            ['data-ui-id']: 'call-composite-more-menu-devices-button'
        });
    }
    if (!showRaiseHandButtonInControlBar && !hideRaiseHandButtonInRoomsCall) {
        moreButtonMenuItems.push({
            key: 'raiseHandButtonKey',
            text: raiseHandButtonProps.checked ? localeStrings.component.strings.raiseHandButton.onLabel : localeStrings.component.strings.raiseHandButton.offLabel,
            onClick: () => {
                if (raiseHandButtonProps.onToggleRaiseHand) {
                    raiseHandButtonProps.onToggleRaiseHand();
                }
            },
            iconProps: {
                iconName: 'RaiseHandContextualMenuItem',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.raiseHandButton),
            ['data-ui-id']: 'call-composite-more-menu-raise-hand-button'
        });
    }
    // Custom Buttons in More Button Menu should always be the last items pushed into the moreButtonMenuItems array
    if (customDrawerButtons['primary']) {
        customDrawerButtons['primary'].slice(props.isMobile ? CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_MOBILE_CUSTOM_BUTTONS : CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_DESKTOP_CUSTOM_BUTTONS).forEach(element => {
            moreButtonMenuItems.push(Object.assign({ itemProps: {
                    styles: buttonFlyoutIncreasedSizeStyles
                } }, element));
        });
    }
    if (customDrawerButtons['secondary']) {
        customDrawerButtons['secondary'].forEach(element => {
            moreButtonMenuItems.push(Object.assign({ itemProps: {
                    styles: buttonFlyoutIncreasedSizeStyles
                } }, element));
        });
    }
    if (customDrawerButtons['overflow']) {
        customDrawerButtons['overflow'].forEach(element => {
            moreButtonMenuItems.push(Object.assign({ itemProps: {
                    styles: buttonFlyoutIncreasedSizeStyles
                } }, element));
        });
    }
    const theme = react.useTheme();
    // when props.options is false then we want to hide the whole control bar.
    if (props.options === false) {
        return React.createElement(React.Fragment, null);
    }
    return React.createElement(react.Stack, { horizontalAlign: "center", className: callControlsContainerStyles },
        React.createElement(react.Stack.Item, null,
            React.createElement(ControlBar, { layout: props.displayVertical ? 'vertical' : 'horizontal', styles: controlBarStyles(theme.semanticColors.bodyBackground) },
                microphoneButtonIsEnabled && React.createElement(Microphone, { displayType: options === null || options === void 0 ? void 0 : options.displayType, disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.microphoneButton), onClickNoiseSuppression: onClickNoiseSuppression, isDeepNoiseSuppressionOn: isDeepNoiseSuppressionOn, showNoiseSuppressionButton: showNoiseSuppressionButton }),
                cameraButtonIsEnabled && React.createElement(Camera, { displayType: options === null || options === void 0 ? void 0 : options.displayType, disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.cameraButton) }),
                showReactionButtonInControlBar && reactionResources && React.createElement(Reaction, { displayType: options === null || options === void 0 ? void 0 : options.displayType, reactionResource: reactionResources }),
                showRaiseHandButtonInControlBar && !hideRaiseHandButtonInRoomsCall && React.createElement(RaiseHand, { displayType: options === null || options === void 0 ? void 0 : options.displayType }),
                screenShareButtonIsEnabled && React.createElement(ScreenShare, { option: options === null || options === void 0 ? void 0 : options.screenShareButton, displayType: options === null || options === void 0 ? void 0 : options.displayType, disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.screenShareButton) }),
                showParticipantsButtonInControlBar && React.createElement(Participants, { option: options === null || options === void 0 ? void 0 : options.participantsButton, callInvitationURL: props.callInvitationURL, onFetchParticipantMenuItems: props.onFetchParticipantMenuItems, displayType: options === null || options === void 0 ? void 0 : options.displayType, increaseFlyoutItemSize: props.increaseFlyoutItemSize, isMobile: props.isMobile, disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.participantsButton) }) && React.createElement(People, { checked: props.peopleButtonChecked, ariaLabel: peopleButtonStrings === null || peopleButtonStrings === void 0 ? void 0 : peopleButtonStrings.label, showLabel: (options === null || options === void 0 ? void 0 : options.displayType) !== 'compact', onClick: props.onPeopleButtonClicked, "data-ui-id": "call-composite-people-button", strings: peopleButtonStrings, disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.participantsButton), disableTooltip: props.isMobile }),
                showDevicesButtonInControlBar && React.createElement(Devices, { displayType: options === null || options === void 0 ? void 0 : options.displayType, increaseFlyoutItemSize: props.increaseFlyoutItemSize, disabled: isDisabled$4(options === null || options === void 0 ? void 0 : options.devicesButton) }), (_a = customButtons['primary']) === null || _a === void 0 ? void 0 :
                _a.slice(0, props.isMobile ? CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_MOBILE_CUSTOM_BUTTONS : CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_DESKTOP_CUSTOM_BUTTONS).map((CustomButton, i) => {
                    return React.createElement(CustomButton, { key: `primary-custom-button-${i}`, 
                        // styles={commonButtonStyles}
                        showLabel: (options === null || options === void 0 ? void 0 : options.displayType) !== 'compact', disableTooltip: props.isMobile });
                }),
                showMoreButton && React.createElement(MoreButton, { disableTooltip: props.isMobile, "data-ui-id": "common-call-composite-more-button", strings: moreButtonStrings, menuIconProps: {
                        hidden: true
                    }, menuProps: {
                        items: moreButtonMenuItems
                    }, showLabel: (options === null || options === void 0 ? void 0 : options.displayType) !== 'compact' }),
                isEnabled$2(options === null || options === void 0 ? void 0 : options.endCallButton) && React.createElement(EndCall, { displayType: options === null || options === void 0 ? void 0 : options.displayType }))));
};
const isEnabled$2 = (option) => option !== false;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const icon$1 = () => React.createElement(CallCompositeIcon, { iconName: 'ControlBarPeopleButton' });
/**
 * @private
 */
const PeopleButton = (props) => {
    const { strings, onRenderOnIcon, onRenderOffIcon, onClick, peoplePaneDismissButtonRef, chatButtonPresent } = props;
    const theme = useTheme();
    const styles = React.useMemo(() => {
        var _a;
        return react.concatStyleSets({
            rootChecked: {
                background: theme.palette.neutralLight
            }
        }, (_a = props.styles) !== null && _a !== void 0 ? _a : {});
    }, [props.styles, theme.palette.neutralLight]);
    const handleTab = React.useCallback((event) => {
        if (event.key === 'Tab' && !event.shiftKey && (peoplePaneDismissButtonRef === null || peoplePaneDismissButtonRef === void 0 ? void 0 : peoplePaneDismissButtonRef.current) && !chatButtonPresent) {
            peoplePaneDismissButtonRef.current.focus();
            event.preventDefault();
        }
    }, [peoplePaneDismissButtonRef, chatButtonPresent]);
    const handleClick = React.useCallback((event) => {
        var _a;
        onClick === null || onClick === void 0 ? void 0 : onClick(event);
        if (chatButtonPresent) {
            (_a = peoplePaneDismissButtonRef === null || peoplePaneDismissButtonRef === void 0 ? void 0 : peoplePaneDismissButtonRef.current) === null || _a === void 0 ? void 0 : _a.focus();
            event.preventDefault();
        }
    }, [chatButtonPresent, onClick, peoplePaneDismissButtonRef]);
    return React.createElement(ControlBarButton, Object.assign({}, props, { strings: strings, labelKey: 'peopleButtonLabelKey', onRenderOnIcon: onRenderOnIcon !== null && onRenderOnIcon !== void 0 ? onRenderOnIcon : icon$1, onRenderOffIcon: onRenderOffIcon !== null && onRenderOffIcon !== void 0 ? onRenderOffIcon : icon$1, onClick: handleClick, onKeyDown: handleTab, styles: styles }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * The hook to bypass limitation of conditional build for CallWithChatComposite
 * Remove this and use useLocale() instead when merge CallWithChatComposite from beta
 *
 * @private
 */
const useCallWithChatCompositeStrings = () => {
    const locale = useLocale();
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    return locale.strings.callWithChat;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const mediaGallerySelector = reselect__namespace.createSelector([getLocalVideoStreams], localVideoStreams => {
    var _a, _b;
    return {
        isVideoStreamOn: !!((_b = (_a = localVideoStreams === null || localVideoStreams === void 0 ? void 0 : localVideoStreams.find(stream => stream.mediaStreamType === 'Video')) === null || _a === void 0 ? void 0 : _a.view) === null || _b === void 0 ? void 0 : _b.target)
    };
});
/**
 * Custom selector for this hook to retrieve all the participants that are currently
 * connected to the call.
 */
const getRemoteParticipantsConnectedSelector = reselect__namespace.createSelector([getRemoteParticipants], remoteParticipants => {
    const participants = Object.values(remoteParticipants !== null && remoteParticipants !== void 0 ? remoteParticipants : {});
    return participants.filter(p => p.state === 'Connected');
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * sets the announcement string whenever a Participant comes or goes from a call to be
 * used by the system narrator.
 *
 * @returns string to be used by the narrator and Announcer component
 *
 * @internal
 */
const useParticipantChangedAnnouncement = () => {
    const locale = useLocale().strings.call;
    const strings = React.useMemo(() => {
        return {
            participantJoinedNoticeString: locale.participantJoinedNoticeString,
            twoParticipantJoinedNoticeString: locale.twoParticipantJoinedNoticeString,
            threeParticipantJoinedNoticeString: locale.threeParticipantJoinedNoticeString,
            participantLeftNoticeString: locale.participantLeftNoticeString,
            twoParticipantLeftNoticeString: locale.twoParticipantLeftNoticeString,
            threeParticipantLeftNoticeString: locale.threeParticipantLeftNoticeString,
            unnamedParticipantString: locale.unnamedParticipantString,
            manyParticipantsJoined: locale.manyParticipantsJoined,
            manyParticipantsLeft: locale.manyParticipantsLeft,
            manyUnnamedParticipantsJoined: locale.manyUnnamedParticipantsJoined,
            manyUnnamedParticipantsLeft: locale.manyUnnamedParticipantsLeft
        };
    }, [locale]);
    const [announcerString, setAnnouncerString] = React.useState('');
    const currentParticipants = useSelector$1(getRemoteParticipantsConnectedSelector);
    /**
     * We want to use a useRef here since we want to not fire this hook based on the previous participants
     * this allows this value to be used in the hook without being in the dependency array.
     *
     * Note: By definition if this hook is used in another component it is not pure anymore.
     */
    const previousParticipants = React.useRef(currentParticipants);
    const resetAnnoucement = (string) => {
        setAnnouncerString(string);
    };
    React.useMemo(() => {
        const currentIds = currentParticipants.map(p => toFlatCommunicationIdentifier(p.identifier));
        const previousIds = previousParticipants.current.map(p => toFlatCommunicationIdentifier(p.identifier));
        const whoJoined = currentParticipants.filter(p => !previousIds.includes(toFlatCommunicationIdentifier(p.identifier)));
        const whoLeft = previousParticipants.current.filter(p => !currentIds.includes(toFlatCommunicationIdentifier(p.identifier)));
        if (whoJoined.length > 0) {
            resetAnnoucement(createAnnouncementString('joined', whoJoined, strings));
        }
        if (whoLeft.length > 0) {
            resetAnnoucement(createAnnouncementString('left', whoLeft, strings));
        }
        // Update cached value at the end.
        previousParticipants.current = currentParticipants;
    }, [currentParticipants, strings]);
    return announcerString;
};
/**
 * Generates the announcement string for when a participant joins or leaves a call.
 */
const createAnnouncementString = (direction, participants, strings) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t;
    /**
     * If there are no participants return empty string.
     */
    if (participants.length === 0) {
        return '';
    }
    /**
     * Filter participants into two arrays to put all the unnamed participants at the back of the
     * names being announced.
     */
    const unnamedParticipants = participants.filter(p => !p.displayName);
    const namedParicipants = participants.filter(p => p.displayName);
    const sortedParticipants = namedParicipants.concat(unnamedParticipants);
    /**
     * if there are only unnamed participants present in the array announce a special unnamed participants
     * only string.
     */
    if (sortedParticipants.filter(p => p.displayName).length === 0 && sortedParticipants.length > 1) {
        return _formatString(direction === 'joined' ? strings.manyUnnamedParticipantsJoined : strings.manyUnnamedParticipantsLeft, {
            numOfParticipants: (sortedParticipants.length - 1).toString()
        });
    }
    switch (sortedParticipants.length) {
        case 1:
            return _formatString(direction === 'joined' ? strings.participantJoinedNoticeString : strings.participantLeftNoticeString, {
                displayName: (_b = (_a = sortedParticipants[0]) === null || _a === void 0 ? void 0 : _a.displayName) !== null && _b !== void 0 ? _b : strings.unnamedParticipantString
            });
        case 2:
            return _formatString(direction === 'joined' ? strings.twoParticipantJoinedNoticeString : strings.twoParticipantLeftNoticeString, {
                displayName1: (_d = (_c = sortedParticipants[0]) === null || _c === void 0 ? void 0 : _c.displayName) !== null && _d !== void 0 ? _d : strings.unnamedParticipantString,
                displayName2: (_f = (_e = sortedParticipants[1]) === null || _e === void 0 ? void 0 : _e.displayName) !== null && _f !== void 0 ? _f : strings.unnamedParticipantString
            });
        case 3:
            return _formatString(direction === 'joined' ? strings.threeParticipantJoinedNoticeString : strings.threeParticipantLeftNoticeString, {
                displayName1: (_h = (_g = sortedParticipants[0]) === null || _g === void 0 ? void 0 : _g.displayName) !== null && _h !== void 0 ? _h : strings.unnamedParticipantString,
                displayName2: (_k = (_j = sortedParticipants[1]) === null || _j === void 0 ? void 0 : _j.displayName) !== null && _k !== void 0 ? _k : strings.unnamedParticipantString,
                displayName3: (_m = (_l = sortedParticipants[2]) === null || _l === void 0 ? void 0 : _l.displayName) !== null && _m !== void 0 ? _m : strings.unnamedParticipantString
            });
    }
    /**
     * If we have more than 3 participants joining we need to do something more to announce them
     * appropriately.
     *
     * We don't want to announce every name when more than 3 participants join at once so
     * we parse out the first 3 names we have and announce those with the number of others.
     */
    const numberOfExtraParticipants = sortedParticipants.length - 3;
    return _formatString(direction === 'joined' ? strings.manyParticipantsJoined : strings.manyParticipantsLeft, {
        displayName1: (_p = (_o = sortedParticipants[0]) === null || _o === void 0 ? void 0 : _o.displayName) !== null && _p !== void 0 ? _p : strings.unnamedParticipantString,
        displayName2: (_r = (_q = sortedParticipants[1]) === null || _q === void 0 ? void 0 : _q.displayName) !== null && _r !== void 0 ? _r : strings.unnamedParticipantString,
        displayName3: (_t = (_s = sortedParticipants[2]) === null || _s === void 0 ? void 0 : _s.displayName) !== null && _t !== void 0 ? _t : strings.unnamedParticipantString,
        numOfParticipants: numberOfExtraParticipants.toString()
    });
};
/**
 * determines if the media gallery should be replaced by the dtmf dialer
 * @param callees Target callees to determine if the dtmf dialer should be shown
 * @param remoteParticipants Remote participants to determine if the dtmf dialer should be shown if there are participants in the call
 * when joining
 * @returns whether the dialer should be the gallery content or not
 */
const showDtmfDialer = (callees, remoteParticipants, dialerOptions) => {
    let showDtmfDialerAuto = false;
    if (typeof dialerOptions === 'object' && 'dialerBehavior' in dialerOptions) {
        const hideDtmfDialerAlways = dialerOptions.dialerBehavior && dialerOptions.dialerBehavior === 'alwaysHide';
        const showDtmfDialerAlways = dialerOptions.dialerBehavior === 'alwaysShow';
        showDtmfDialerAuto = dialerOptions.dialerBehavior === 'autoShow' ? true : false;
        if (showDtmfDialerAlways) {
            return true;
        }
        if (hideDtmfDialerAlways) {
            return false;
        }
    }
    let showDtmfDialer = false;
    /**
     * We also want to check to see if the option is undefined. If this is the case we want this function
     * to fallback on the original logic so that it will also render the callControls to show and hide the dialpad
     * for the user.
     */
    if (showDtmfDialerAuto || dialerOptions === undefined || dialerOptions === false) {
        callees === null || callees === void 0 ? void 0 : callees.forEach(callee => {
            if (communicationCommon.isMicrosoftTeamsAppIdentifier(callee) || communicationCommon.isPhoneNumberIdentifier(callee)) {
                showDtmfDialer = true;
            }
        });
        if (remoteParticipants) {
            remoteParticipants.forEach(participant => {
                if (!('phoneNumber' in participant.identifier || 'teamsAppId' in participant.identifier)) {
                    showDtmfDialer = false;
                }
            });
        }
    }
    return showDtmfDialer;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$m = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 *
 * @private
 */
const DesktopMoreButton = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const localeStrings = useLocale();
    const holdButtonProps = usePropsFor$1(HoldButton);
    const startCaptionsButtonProps = usePropsFor$1(StartCaptionsButton);
    const realTimeTextProps = usePropsFor$1(CaptionsBanner);
    const startCaptions = React.useCallback(() => __awaiter$m(void 0, void 0, void 0, function* () {
        yield startCaptionsButtonProps.onStartCaptions({
            spokenLanguage: startCaptionsButtonProps.currentSpokenLanguage
        });
    }), [startCaptionsButtonProps]);
    const [focusedContentOn, setFocusedContentOn] = React.useState(false);
    const [previousLayout, setPreviousLayout] = React.useState((_a = props.userSetGalleryLayout) !== null && _a !== void 0 ? _a : 'floatingLocalVideo');
    const callees = useSelector$1(getTargetCallees);
    const allowDtmfDialer = showDtmfDialer(callees);
    const isTeamsMeeting = useSelector$1(getIsTeamsMeeting);
    const teamsMeetingCoordinates = useSelector$1(getTeamsMeetingCoordinates);
    const isTogetherModeActive = useSelector$1(getIsTogetherModeActive);
    const participantCapability = useSelector$1(getCapabilites);
    const participantId = useSelector$1(getLocalUserId);
    const isTeamsCall = useSelector$1(getIsTeamsCall);
    const [dtmfDialerChecked, setDtmfDialerChecked] = React.useState((_b = props.dtmfDialerPresent) !== null && _b !== void 0 ? _b : false);
    const moreButtonStrings = React.useMemo(() => ({
        label: localeStrings.strings.call.moreButtonCallingLabel,
        tooltipOffContent: localeStrings.strings.callWithChat.moreDrawerButtonTooltip
    }), [localeStrings]);
    const moreButtonContextualMenuItems = [];
    const menuSubIconStyleSet = {
        root: {
            height: 'unset',
            lineHeight: '100%',
            width: '1.25rem'
        }
    };
    if (props.callControls === true || ((_c = props.callControls) === null || _c === void 0 ? void 0 : _c.holdButton) !== false) {
        moreButtonContextualMenuItems.push({
            key: 'holdButtonKey',
            text: localeStrings.component.strings.holdButton.tooltipOffContent,
            onClick: () => {
                holdButtonProps.onToggleHold();
            },
            iconProps: {
                iconName: 'HoldCallContextualMenuItem',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            disabled: props.disableButtonsForHoldScreen
        });
    }
    // is captions feature is active
    if (props.isCaptionsSupported) {
        const captionsContextualMenuItems = [];
        moreButtonContextualMenuItems.push({
            key: 'liveCaptionsKey',
            id: 'common-call-composite-captions-button',
            text: localeStrings.strings.call.liveCaptionsLabel,
            iconProps: {
                iconName: 'CaptionsIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            disabled: props.disableButtonsForHoldScreen,
            subMenuProps: {
                id: 'captions-contextual-menu',
                items: captionsContextualMenuItems,
                calloutProps: {
                    preventDismissOnEvent: _preventDismissOnEvent
                }
            },
            submenuIconProps: {
                iconName: 'HorizontalGalleryRightButton',
                styles: menuSubIconStyleSet
            }
        });
        captionsContextualMenuItems.push({
            key: 'ToggleCaptionsKey',
            id: 'common-call-composite-captions-toggle-button',
            text: startCaptionsButtonProps.checked ? localeStrings.strings.call.startCaptionsButtonTooltipOnContent : localeStrings.strings.call.startCaptionsButtonTooltipOffContent,
            onClick: () => {
                startCaptionsButtonProps.checked ? startCaptionsButtonProps.onStopCaptions() : startCaptionsButtonProps.currentSpokenLanguage !== '' ? startCaptions() : props.onCaptionsSettingsClick && props.onCaptionsSettingsClick();
            },
            iconProps: {
                iconName: startCaptionsButtonProps.checked ? 'CaptionsOffIcon' : 'CaptionsIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            disabled: props.disableButtonsForHoldScreen
        });
        if (props.onCaptionsSettingsClick) {
            captionsContextualMenuItems.push({
                key: 'openCaptionsSettingsKey',
                id: 'common-call-composite-captions-settings-button',
                text: localeStrings.strings.call.captionsSettingsLabel,
                onClick: props.onCaptionsSettingsClick,
                iconProps: {
                    iconName: 'CaptionsSettingsIcon',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                itemProps: {
                    styles: buttonFlyoutIncreasedSizeStyles
                },
                disabled: props.disableButtonsForHoldScreen || !startCaptionsButtonProps.checked
            });
        }
    }
    //RTT
    if (props.isRealTimeTextSupported) {
        const realTimeTextContextualMenuItems = [];
        const rttDisabled = props.disableButtonsForHoldScreen || realTimeTextProps.isRealTimeTextOn || props.startRealTimeTextButtonChecked;
        moreButtonContextualMenuItems.push({
            key: 'realTimeTextKey',
            id: 'common-call-composite-rtt-button',
            text: localeStrings.strings.call.realTimeTextLabel,
            iconProps: {
                iconName: 'RealTimeTextIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            disabled: props.disableButtonsForHoldScreen,
            subMenuProps: {
                id: 'rtt-contextual-menu',
                items: realTimeTextContextualMenuItems,
                calloutProps: {
                    preventDismissOnEvent: _preventDismissOnEvent
                }
            },
            submenuIconProps: {
                iconName: 'HorizontalGalleryRightButton',
                styles: menuSubIconStyleSet
            }
        });
        realTimeTextContextualMenuItems.push({
            key: 'StartRealTimeTextKey',
            id: 'common-call-composite-rtt-start-button',
            text: localeStrings.strings.call.startRealTimeTextLabel,
            ariaLabel: rttDisabled ? localeStrings.strings.call.disabledStartRealTimeTextLabel : localeStrings.strings.call.startRealTimeTextLabel,
            onClick: props.onStartRealTimeTextClick,
            iconProps: {
                iconName: 'RealTimeTextIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            disabled: rttDisabled
        });
    }
    const dtmfDialerScreenOption = {
        key: 'dtmfDialerScreenKey',
        itemProps: {
            styles: buttonFlyoutIncreasedSizeStyles
        },
        text: !dtmfDialerChecked ? localeStrings.strings.call.dtmfDialerMoreButtonLabelOn : localeStrings.strings.call.dtmfDialerMoreButtonLabelOff,
        onClick: () => {
            props.onSetDialpadPage && props.onSetDialpadPage();
            setDtmfDialerChecked(!dtmfDialerChecked);
        },
        iconProps: {
            iconName: 'DtmfDialpadButton',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        }
    };
    /**
     * Only render the dtmf dialer if the dialpad for PSTN calls is not present
     */
    if (props.onSetDialpadPage && allowDtmfDialer) {
        if (props.callControls === true || ((_d = props.callControls) === null || _d === void 0 ? void 0 : _d.dtmfDialerButton) !== false) {
            moreButtonContextualMenuItems.push(dtmfDialerScreenOption);
        }
    }
    const joinByPhoneOption = {
        key: 'phoneCallKey',
        itemProps: {
            styles: buttonFlyoutIncreasedSizeStyles
        },
        text: localeStrings.strings.call.phoneCallMoreButtonLabel,
        onClick: () => {
            props.onMeetingPhoneInfoClick && props.onMeetingPhoneInfoClick();
        },
        iconProps: {
            iconName: 'PhoneNumberButton',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        }
    };
    /**
     * Only render the phone call button if meeting conordinates are present
     */
    if (props.teamsMeetingPhoneCallEnable && isTeamsMeeting && teamsMeetingCoordinates) {
        moreButtonContextualMenuItems.push(joinByPhoneOption);
    }
    if (props.onUserSetOverflowGalleryPositionChange) {
        const galleryOptions = {
            key: 'overflowGalleryPositionKey',
            iconProps: {
                iconName: 'GalleryOptions',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            submenuIconProps: {
                styles: menuSubIconStyleSet
            },
            text: localeStrings.strings.call.moreButtonGalleryControlLabel,
            disabled: props.disableButtonsForHoldScreen,
            subMenuProps: {
                items: [{
                        key: 'dynamicSelectionKey',
                        text: localeStrings.strings.call.moreButtonGalleryFloatingLocalLayoutLabel,
                        canCheck: true,
                        itemProps: {
                            styles: buttonFlyoutIncreasedSizeStyles
                        },
                        isChecked: props.userSetGalleryLayout === 'floatingLocalVideo',
                        onClick: () => {
                            props.onUserSetGalleryLayout && props.onUserSetGalleryLayout('floatingLocalVideo');
                            setFocusedContentOn(false);
                        },
                        iconProps: {
                            iconName: 'FloatingLocalVideoGalleryLayout',
                            styles: {
                                root: {
                                    lineHeight: 0
                                }
                            }
                        }
                    }, {
                        key: 'speakerSelectionKey',
                        text: localeStrings.strings.call.moreButtonGallerySpeakerLayoutLabel,
                        canCheck: true,
                        itemProps: {
                            styles: buttonFlyoutIncreasedSizeStyles
                        },
                        isChecked: props.userSetGalleryLayout === 'speaker',
                        onClick: () => {
                            props.onUserSetGalleryLayout && props.onUserSetGalleryLayout('speaker');
                            setFocusedContentOn(false);
                        },
                        iconProps: {
                            iconName: 'SpeakerGalleryLayout',
                            styles: {
                                root: {
                                    lineHeight: 0
                                }
                            }
                        }
                    }, {
                        key: 'focusedContentSelectionKey',
                        text: localeStrings.strings.call.moreButtonGalleryFocusedContentLayoutLabel,
                        canCheck: true,
                        itemProps: {
                            styles: buttonFlyoutIncreasedSizeStyles
                        },
                        isChecked: focusedContentOn,
                        onClick: () => {
                            var _a;
                            if (focusedContentOn === false) {
                                setPreviousLayout((_a = props.userSetGalleryLayout) !== null && _a !== void 0 ? _a : 'floatingLocalVideo');
                                props.onUserSetGalleryLayout && props.onUserSetGalleryLayout('focusedContent');
                                setFocusedContentOn(true);
                            }
                            else {
                                props.onUserSetGalleryLayout && props.onUserSetGalleryLayout(previousLayout);
                                setFocusedContentOn(false);
                            }
                        },
                        iconProps: {
                            iconName: 'FocusedContentGalleryLayout',
                            styles: {
                                root: {
                                    lineHeight: 0
                                }
                            }
                        }
                    }],
                calloutProps: {
                    preventDismissOnEvent: _preventDismissOnEvent
                }
            }
        };
        const togetherModeOption = {
            key: 'togetherModeSelectionKey',
            text: localeStrings.strings.call.moreButtonTogetherModeLayoutLabel,
            canCheck: true,
            itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            },
            isChecked: props.userSetGalleryLayout === 'togetherMode',
            onClick: () => {
                props.onUserSetGalleryLayout && props.onUserSetGalleryLayout('togetherMode');
                setFocusedContentOn(false);
            },
            disabled: !((participantId === null || participantId === void 0 ? void 0 : participantId.kind) === 'microsoftTeamsUser' && ((_e = participantCapability === null || participantCapability === void 0 ? void 0 : participantCapability.startTogetherMode) === null || _e === void 0 ? void 0 : _e.isPresent) || isTogetherModeActive),
            iconProps: {
                iconName: 'TogetherModeLayout',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            }
        };
        if (isTeamsCall || isTeamsMeeting) {
            (_g = (_f = galleryOptions.subMenuProps) === null || _f === void 0 ? void 0 : _f.items) === null || _g === void 0 ? void 0 : _g.push(togetherModeOption);
        }
        if (props.callControls === true || ((_h = props.callControls) === null || _h === void 0 ? void 0 : _h.galleryControlsButton) !== false) {
            moreButtonContextualMenuItems.push(galleryOptions);
        }
    }
    const customDrawerButtons = React.useMemo(() => generateCustomCallDesktopOverflowButtons(onFetchCustomButtonPropsTrampoline(typeof props.callControls === 'object' ? props.callControls : undefined), typeof props.callControls === 'object' ? props.callControls.displayType : undefined), [props.callControls]);
    customDrawerButtons['primary'].slice(CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_DESKTOP_CUSTOM_BUTTONS).forEach(element => {
        moreButtonContextualMenuItems.push(Object.assign({ itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            } }, element));
    });
    customDrawerButtons['secondary'].slice(CUSTOM_BUTTON_OPTIONS.MAX_SECONDARY_DESKTOP_CUSTOM_BUTTONS).forEach(element => {
        moreButtonContextualMenuItems.push(Object.assign({ itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            } }, element));
    });
    customDrawerButtons['overflow'].forEach(element => {
        moreButtonContextualMenuItems.push(Object.assign({ itemProps: {
                styles: buttonFlyoutIncreasedSizeStyles
            } }, element));
    });
    return React.createElement(MoreButton, Object.assign({}, props, { "data-ui-id": "common-call-composite-more-button", strings: moreButtonStrings, menuIconProps: {
            hidden: true
        }, menuProps: {
            shouldFocusOnContainer: false,
            items: moreButtonContextualMenuItems,
            calloutProps: {
                preventDismissOnEvent: _preventDismissOnEvent
            }
        } }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const icon = () => React.createElement(CallCompositeIcon, { iconName: 'DtmfDialpadButton' });
/**
 * Button for showing and hiding the dtmf dialer in the Call composite.
 * To be used only if the more button control is disabled. This button will replace
 * it in the control bar unless it is also disabled.
 * @private
 */
const DtmfDialpadButton = (props) => {
    const { strings, onRenderOnIcon, onRenderOffIcon, onClick, displayType } = props;
    const theme = react.useTheme();
    const locale = useLocale();
    const dialpadButtonStrings = {
        label: locale.strings.call.dtmfDialerButtonLabel,
        tooltipOnContent: locale.strings.call.dtmfDialerButtonTooltipOn,
        tooltipOffContent: locale.strings.call.dtmfDialerButtonTooltipOff
    };
    const styles = React.useMemo(() => {
        var _a;
        return react.concatStyleSets({
            rootChecked: {
                background: theme.palette.neutralLight
            }
        }, (_a = props.styles) !== null && _a !== void 0 ? _a : {});
    }, [props.styles, theme.palette.neutralLight]);
    return React.createElement(ControlBarButton, Object.assign({}, props, { strings: strings ? strings : dialpadButtonStrings, showLabel: displayType !== 'compact', labelKey: 'dtmfDialpadButtonLabelKey', onRenderOnIcon: onRenderOnIcon !== null && onRenderOnIcon !== void 0 ? onRenderOnIcon : icon, onRenderOffIcon: onRenderOffIcon !== null && onRenderOffIcon !== void 0 ? onRenderOffIcon : icon, onClick: onClick, styles: styles }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ExitSpotlightButton = (props) => {
    return React.createElement(ControlBarButton, Object.assign({ "data-ui-id": 'call-composite-exit-spotlight-button', labelKey: 'exitSpotlightButtonLabelKey', showLabel: props.displayType !== 'compact', onRenderIcon: () => React.createElement(_HighContrastAwareIcon, { iconName: 'ControlButtonExitSpotlight' }), onClick: props.onClick }, props));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Default Modal PIP related pixel measurements used for ModalLocalAndremotePIP and common/utils.
 *
 * @private
 */
const MODAL_PIP_DEFAULT_PX = {
    rightPositionPx: 16,
    bottomPositionPx: 64,
    widthPx: 88,
    heightPx: 128
};
/**
 * @private
 */
const getPipStyles = (theme) => ({
    modal: {
        main: Object.assign(Object.assign({ borderRadius: theme.effects.roundedCorner4, boxShadow: theme.effects.elevation8, 
            // Above the message thread / people pane.
            zIndex: 2 }, (theme.rtl ? {
            left: _pxToRem(MODAL_PIP_DEFAULT_PX.rightPositionPx)
        } : {
            right: _pxToRem(MODAL_PIP_DEFAULT_PX.rightPositionPx)
        })), { bottom: _pxToRem(MODAL_PIP_DEFAULT_PX.bottomPositionPx) })
    }
});
/**
 * @private
 */
const PIPContainerStyle = {
    root: {
        position: 'absolute',
        width: '100%',
        height: '100%',
        pointerEvents: 'none',
        ':focus-within': {
            outline: '1px solid #00000000'
        }
    }
};
/**
 * @private
 */
const hiddenStyle = react.concatStyleSets(PIPContainerStyle, {
    root: {
        display: 'none'
    }
});
/**
 * @private
 */
const modalStyle = {
    main: {
        minWidth: 'min-content',
        minHeight: 'min-content',
        position: 'absolute',
        overflow: 'hidden',
        // pointer events for root Modal div set to auto to make LocalAndRemotePIP interactive
        pointerEvents: 'auto',
        touchAction: 'none'
    },
    root: {
        width: '100%',
        height: '100%',
        // pointer events for root Modal div set to none to make descendants interactive
        pointerEvents: 'none',
        ':focus-within': {
            outline: '3px solid #00000000',
            outlineOffset: '-3px'
        }
    }
};
/**
 * Styles for layer host to bound the modal wrapping PiPiP in the mobile pane.
 * @private
 */
const modalLayerHostStyle = {
    display: 'flex',
    position: 'absolute',
    left: 0,
    top: 0,
    width: '100%',
    height: '100%',
    overflow: 'hidden',
    zIndex: '100000',
    // pointer events for layerHost set to none to make descendants interactive
    pointerEvents: 'none'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$l = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const isBoolean = (value) => {
    return value !== null && typeof value === 'boolean';
};
/**
 * @private
 */
// Use document.getElementById until Fluent's Stack supports componentRef property: https://github.com/microsoft/fluentui/issues/20410
const useMinMaxDragPosition = (modalLayerHostId, rtl) => {
    const modalHostRef = React.useRef(document.getElementById(modalLayerHostId));
    const modalHostWidth = _useContainerWidth(modalHostRef);
    const modalHostHeight = _useContainerHeight(modalHostRef);
    const minDragPosition = React.useMemo(() => modalHostWidth === undefined || modalHostHeight === undefined ? undefined : {
        x: MODAL_PIP_DEFAULT_PX.rightPositionPx - modalHostWidth + MODAL_PIP_DEFAULT_PX.widthPx,
        y: -1 * modalHostHeight + MODAL_PIP_DEFAULT_PX.heightPx + MODAL_PIP_DEFAULT_PX.bottomPositionPx
    }, [modalHostHeight, modalHostWidth, rtl]);
    const maxDragPosition = React.useMemo(() => modalHostWidth === undefined ? undefined : {
        x: MODAL_PIP_DEFAULT_PX.rightPositionPx,
        y: MODAL_PIP_DEFAULT_PX.bottomPositionPx
    }, [modalHostWidth, rtl]);
    return {
        minDragPosition: minDragPosition,
        maxDragPosition: maxDragPosition
    };
};
/**
 * @private
 */
const busyWait = (checkCondition, retryLimit) => __awaiter$l(void 0, void 0, void 0, function* () {
    const delayMs = 500;
    let retryCount = 0;
    while (!checkCondition()) {
        yield new Promise(resolve => setTimeout(resolve, delayMs));
        if (retryCount++ >= retryLimit) {
            break;
        }
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @internal
 */
const getMinutes$1 = (time) => {
    return Math.floor(getSeconds$1(time) / 60);
};
/**
 * @internal
 */
const getSeconds$1 = (time) => {
    return Math.floor(time / 1000);
};
/**
 * @internal
 */
const getHours$1 = (time) => {
    return Math.floor(getMinutes$1(time) / 60);
};
/**
 * @internal
 */
const getReadableTime$1 = (time) => {
    const hours = getHours$1(time);
    const readableMinutes = ('0' + getMinutes$1(time) % 60).slice(-2);
    const readableSeconds = ('0' + getSeconds$1(time) % 60).slice(-2);
    return `${hours > 0 ? hours + ':' : ''}${readableMinutes}:${readableSeconds}`;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// eslint-disable-next-line no-restricted-imports
/**
 * @private
 */
const Timer = (props) => {
    const { timeStampInfo } = props;
    const endTimeMs = new Date(timeStampInfo).getTime();
    const theme = react.useTheme();
    const [time, setTime] = React.useState(0);
    const timeRemaining = getReadableTime$1(time);
    React.useEffect(() => {
        const interval = setInterval(() => {
            setTime(Math.max(endTimeMs - Date.now(), 0));
        }, 10);
        return () => {
            clearInterval(interval);
        };
    }, [endTimeMs]);
    return React.createElement(react.Stack, { horizontal: true, className: timerContainerClass(theme), tokens: {
            childrenGap: '0.5rem'
        } },
        React.createElement(react.FontIcon, { iconName: 'Clock', className: iconClass$1 }),
        React.createElement(react.Text, { styles: timerTextStyle }, timeRemaining));
};
const timerContainerClass = (theme) => react.mergeStyles({
    backgroundColor: theme.palette.neutralLight,
    display: 'flex',
    alignItems: 'center',
    borderColor: theme.palette.neutralQuaternaryAlt,
    borderWidth: '0.0625rem',
    borderRadius: '0.25rem',
    padding: '0.5rem'
});
const iconClass$1 = react.mergeStyles({
    fontSize: 18
});
const timerTextStyle = {
    root: {
        color: 'inherit',
        fontWeight: 400,
        fontSize: _pxToRem(14),
        lineHeight: _pxToRem(20),
        margin: 'auto'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const CallingRealTimeTextModal = (props) => {
    const strings = useLocale().strings.call;
    const modalStrings = {
        realTimeTextModalTitle: strings.realTimeTextModalTitle,
        realTimeTextModalText: strings.realTimeTextModalText,
        realTimeTextConfirmButtonLabel: strings.realTimeTextConfirmButtonLabel,
        realTimeTextCancelButtonLabel: strings.realTimeTextCancelButtonLabel,
        realTimeTextModalAriaLabel: strings.realTimeTextModalAriaLabel,
        realTimeTextCloseModalButtonAriaLabel: strings.realTimeTextCloseModalButtonAriaLabel
    };
    return React.createElement(RealTimeTextModal, { strings: modalStrings, showModal: props.showModal, onDismissModal: props.onDismissModal, onStartRealTimeText: props.onStartRealTimeText });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$k = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const inferCommonCallControlOptions = (mobileView, commonCallControlOptions) => {
    if (commonCallControlOptions === false) {
        return false;
    }
    const options = commonCallControlOptions === true || commonCallControlOptions === undefined ? {} : commonCallControlOptions;
    if (mobileView) {
        // Set to compressed mode when composite is optimized for mobile
        options.displayType = 'compact';
        // Set options to always not show screen share button for mobile
        options.screenShareButton = false;
    }
    return options;
};
/**
 * @private
 */
const CommonCallControlBar = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
    const theme = react.useTheme();
    const rtl = theme.rtl;
    const controlBarContainerRef = React.useRef(null);
    const sidepaneControlsRef = React.useRef(null);
    const controlBarSizeRef = React.useRef(null);
    const [controlBarButtonsWidth, setControlBarButtonsWidth] = React.useState(0);
    const [panelsButtonsWidth, setPanelsButtonsWidth] = React.useState(0);
    const [controlBarContainerWidth, setControlBarContainerWidth] = React.useState(0);
    const [totalButtonsWidth, setTotalButtonsWidth] = React.useState(0);
    const [isOutOfSpace, setIsOutOfSpace] = React.useState(false);
    const callWithChatStrings = useCallWithChatCompositeStrings();
    const options = inferCommonCallControlOptions(props.mobileView, props.callControls);
    const [showCaptionsSettingsModal, setShowCaptionsSettingsModal] = React.useState(false);
    const [showRealTimeTextModal, setShowRealTimeTextModal] = React.useState(false);
    // If the hangup capability is not present, we default to true
    const isHangUpForEveryoneAllowed = (_a = useSelector$1(state => { var _a, _b; return (_b = (_a = state.call) === null || _a === void 0 ? void 0 : _a.capabilitiesFeature) === null || _b === void 0 ? void 0 : _b.capabilities.hangUpForEveryOne.isPresent; })) !== null && _a !== void 0 ? _a : true;
    const isTeams = useSelector$1(getIsTeamsCall);
    const assignedBreakoutRoom = useSelector$1(getAssignedBreakoutRoom);
    const breakoutRoomSettings = useSelector$1(getBreakoutRoomSettings);
    const handleResize = React.useCallback(() => {
        setControlBarButtonsWidth(controlBarContainerRef.current ? controlBarContainerRef.current.offsetWidth : 0);
        setPanelsButtonsWidth(sidepaneControlsRef.current ? sidepaneControlsRef.current.offsetWidth : 0);
        setControlBarContainerWidth(controlBarSizeRef.current ? controlBarSizeRef.current.offsetWidth : 0);
    }, []);
    // on load set inital width
    React.useEffect(() => {
        setControlBarButtonsWidth(controlBarContainerRef.current ? controlBarContainerRef.current.offsetWidth : 0);
        setPanelsButtonsWidth(sidepaneControlsRef.current ? sidepaneControlsRef.current.offsetWidth : 0);
        setControlBarContainerWidth(controlBarSizeRef.current ? controlBarSizeRef.current.offsetWidth : 0);
    }, []);
    // get the current width of control bar buttons and panel control buttons when browser size change
    React.useEffect(() => {
        window.addEventListener('resize', handleResize);
        return () => window.removeEventListener('resize', handleResize);
    }, [handleResize]);
    /* when size change, reset total buttons width and compare with the control bar container width
    if the total width of those buttons exceed container width, do not center the control bar buttons based on parent container width
    Instead let them take up the remaining white space on the left */
    React.useEffect(() => {
        // white space on the left when control bar buttons are centered based on container width + control bar buttons width + panel control buttons width
        setTotalButtonsWidth((controlBarContainerWidth - controlBarButtonsWidth) / 2 + controlBarButtonsWidth + panelsButtonsWidth);
    }, [controlBarButtonsWidth, panelsButtonsWidth, controlBarContainerWidth]);
    React.useEffect(() => {
        setIsOutOfSpace(totalButtonsWidth > controlBarContainerWidth);
    }, [totalButtonsWidth, controlBarContainerWidth]);
    const openCaptionsSettingsModal = React.useCallback(() => {
        setShowCaptionsSettingsModal(true);
    }, []);
    const openRealTimeTextModal = React.useCallback(() => {
        setShowRealTimeTextModal(true);
    }, []);
    const onDismissRealTimeTextModal = React.useCallback(() => {
        setShowRealTimeTextModal(false);
    }, []);
    const onDismissCaptionsSettings = React.useCallback(() => {
        setShowCaptionsSettingsModal(false);
    }, []);
    const peopleButtonStrings = React.useMemo(() => ({
        label: callWithChatStrings.peopleButtonLabel,
        selectedLabel: callWithChatStrings.selectedPeopleButtonLabel,
        tooltipOpenAriaLabel: callWithChatStrings.peopleButtonTooltipOpenAriaLabel,
        tooltipCloseAriaLabel: callWithChatStrings.peopleButtonTooltipCloseAriaLabel,
        tooltipOffContent: callWithChatStrings.peopleButtonTooltipOpen,
        tooltipOnContent: callWithChatStrings.peopleButtonTooltipClose
    }), [callWithChatStrings]);
    const moreButtonStrings = React.useMemo(() => ({
        label: callWithChatStrings.moreDrawerButtonLabel,
        tooltipContent: callWithChatStrings.moreDrawerButtonTooltip
    }), [callWithChatStrings]);
    const callStrings = useLocale().strings.call;
    const exitSpotlightButtonStrings = React.useMemo(() => ({
        label: callStrings.exitSpotlightButtonLabel,
        tooltipContent: callStrings.exitSpotlightButtonTooltip
    }), [callStrings]);
    const [isDeepNoiseSuppressionOn, setDeepNoiseSuppressionOn] = React.useState(false);
    const startDeepNoiseSuppression = React.useCallback(() => __awaiter$k(void 0, void 0, void 0, function* () {
        yield props.callAdapter.startNoiseSuppressionEffect();
    }), [props.callAdapter]);
    const environmentInfo = useSelector$1(getEnvironmentInfo);
    const isSafari = _isSafari(environmentInfo);
    React.useEffect(() => {
        if (props.callAdapter.getState().onResolveDeepNoiseSuppressionDependency && props.callAdapter.getState().deepNoiseSuppressionOnByDefault) {
            startDeepNoiseSuppression();
            setDeepNoiseSuppressionOn(true);
        }
    }, [props.callAdapter, startDeepNoiseSuppression]);
    const showNoiseSuppressionButton = props.callAdapter.getState().onResolveDeepNoiseSuppressionDependency && !props.callAdapter.getState().hideDeepNoiseSuppressionButton && !isSafari ? true : false;
    const onClickNoiseSuppression = React.useCallback(() => __awaiter$k(void 0, void 0, void 0, function* () {
        if (isDeepNoiseSuppressionOn) {
            yield props.callAdapter.stopNoiseSuppressionEffect();
            setDeepNoiseSuppressionOn(false);
        }
        else {
            yield props.callAdapter.startNoiseSuppressionEffect();
            setDeepNoiseSuppressionOn(true);
        }
    }), [props.callAdapter, isDeepNoiseSuppressionOn]);
    const centerContainerStyles = React.useMemo(() => {
        const styles = !props.mobileView ? desktopControlBarStyles : {};
        return react.mergeStyleSets(styles, {
            root: {
                // Enforce a background color on control bar to ensure it matches the composite background color.
                background: theme.semanticColors.bodyBackground
            }
        });
    }, [props.mobileView, theme.semanticColors.bodyBackground]);
    const screenShareButtonStyles = React.useMemo(() => !props.mobileView ? getDesktopScreenShareButtonStyles(theme) : undefined, [props.mobileView, theme]);
    const commonButtonStyles = React.useMemo(() => !props.mobileView ? getDesktopCommonButtonStyles(theme) : undefined, [props.mobileView, theme]);
    const endCallButtonStyles = React.useMemo(() => !props.mobileView ? getDesktopEndCallButtonStyles(theme) : undefined, [props.mobileView, theme]);
    const controlBarWrapperDesktopStyles = React.useMemo(
    // only center control bar buttons based on parent container if there are enough space on the screen and not mobile
    () => !props.mobileView && !isOutOfSpace ? rtl ? wrapperDesktopRtlStyles : wrapperDesktopStyles : {}, [props.mobileView, rtl, isOutOfSpace]);
    // only center control bar buttons based on parent container if there are enough space on the screen and not mobile
    const controlBarDesktopContainerStyles = React.useMemo(() => !props.mobileView && !isOutOfSpace ? {
        position: 'relative',
        minHeight: '4.5rem',
        width: '100%',
        display: 'flex',
        alignItems: 'center',
        paddingLeft: '1rem'
    } : {}, [props.mobileView, isOutOfSpace]);
    const customButtons = React.useMemo(() => generateCustomCallControlBarButton(onFetchCustomButtonPropsTrampoline(options !== false ? options : undefined), options !== false ? options === null || options === void 0 ? void 0 : options.displayType : undefined), [options]);
    const capabilitiesSelector = useSelector$1(capabilitySelector);
    const callState = useSelector$1(callStatusSelector);
    const isReactionAllowed = callState.callStatus !== 'Connected' || !(capabilitiesSelector === null || capabilitiesSelector === void 0 ? void 0 : capabilitiesSelector.capabilities) || capabilitiesSelector.capabilities.useReactions.isPresent;
    const canReturnToMainMeeting = breakoutRoomSettings && breakoutRoomSettings.disableReturnToMainMeeting === false;
    const returnFromBreakoutRoom = React.useCallback(() => props.callAdapter.returnFromBreakoutRoom(), [props.callAdapter]);
    // when options is false then we want to hide the whole control bar.
    if (options === false) {
        return React.createElement(React.Fragment, null);
    }
    const sideButtonsPresent = isEnabled$1(options.peopleButton) || isEnabled$1(options.chatButton) || customButtons['secondary'] !== undefined;
    const screenShareButtonIsEnabled = isEnabled$1(options === null || options === void 0 ? void 0 : options.screenShareButton);
    const microphoneButtonIsEnabled = isEnabled$1(options === null || options === void 0 ? void 0 : options.microphoneButton);
    const cameraButtonIsEnabled = isEnabled$1(options === null || options === void 0 ? void 0 : options.cameraButton);
    const showExitSpotlightButton = (options === null || options === void 0 ? void 0 : options.exitSpotlightButton) !== false;
    const showCaptionsButton = props.isCaptionsSupported && isEnabled$1(options.captionsButton);
    const showRealTimeTextButton = props.isRealTimeTextSupported && isEnabled$1(options.realTimeTextButton);
    const showTeamsMeetingPhoneCallButton = isEnabled$1(options === null || options === void 0 ? void 0 : options.teamsMeetingPhoneCallButton);
    const showDesktopMoreButton = isEnabled$1(options === null || options === void 0 ? void 0 : options.moreButton) && (isEnabled$1(options === null || options === void 0 ? void 0 : options.holdButton) || showCaptionsButton || props.onUserSetGalleryLayout || showRealTimeTextButton);
    const role = (_b = props.callAdapter.getState().call) === null || _b === void 0 ? void 0 : _b.role;
    const hideRaiseHandButtonInRoomsCall = props.callAdapter.getState().isRoomsCall && role && ['Consumer', 'Unknown'].includes(role);
    const reactionResources = props.callAdapter.getState().reactions;
    return React.createElement("div", { ref: controlBarSizeRef },
        React.createElement(CallAdapterProvider, { adapter: props.callAdapter },
            showCaptionsSettingsModal && React.createElement(CallingCaptionsSettingsModal, { showCaptionsSettingsModal: showCaptionsSettingsModal, onDismissCaptionsSettings: onDismissCaptionsSettings, changeCaptionLanguage: props.isCaptionsOn && props.useTeamsCaptions }),
            showRealTimeTextModal && React.createElement(CallingRealTimeTextModal, { showModal: showRealTimeTextModal, onDismissModal: onDismissRealTimeTextModal, onStartRealTimeText: props.onStartRealTimeText }),
            props.teamsMeetingConferenceModalPresent && React.createElement(MeetingConferencePhoneInfoModal, { conferencePhoneInfoList: (_e = (_d = (_c = props.callAdapter.getState().call) === null || _c === void 0 ? void 0 : _c.meetingConference) === null || _d === void 0 ? void 0 : _d.conferencePhones) !== null && _e !== void 0 ? _e : [], showModal: props.teamsMeetingConferenceModalPresent, onDismissMeetingPhoneInfoSettings: props.onToggleTeamsMeetingConferenceModal })),
        React.createElement(react.Stack, { horizontal: true, reversed: !props.mobileView && !isOutOfSpace, horizontalAlign: "space-between", className: react.mergeStyles(callControlsContainerStyles, controlBarContainerStyles, controlBarDesktopContainerStyles) },
            React.createElement(react.Stack.Item, { grow: true, className: react.mergeStyles(controlBarWrapperDesktopStyles) },
                React.createElement(CallAdapterProvider, { adapter: props.callAdapter },
                    React.createElement(react.Stack, { horizontalAlign: "center" },
                        React.createElement(react.Stack.Item, null,
                            React.createElement("div", { ref: controlBarContainerRef },
                                React.createElement(ControlBar, { layout: props.displayVertical ? 'vertical' : 'horizontal', styles: centerContainerStyles },
                                    !props.mobileView && assignedBreakoutRoom && assignedBreakoutRoom.state === 'open' &&
                                        // Breakout room settings are only defined in a breakout room so we use this to ensure
                                        // the button is not shown when already in a breakout room
                                        !breakoutRoomSettings && React.createElement(react.PrimaryButton, { text: callStrings.joinBreakoutRoomButtonLabel, onClick: () => __awaiter$k(void 0, void 0, void 0, function* () {
                                            assignedBreakoutRoom.join();
                                        }), styles: commonButtonStyles }),
                                    microphoneButtonIsEnabled && React.createElement(Microphone, { displayType: options.displayType, styles: commonButtonStyles, splitButtonsForDeviceSelection: !props.mobileView, disabled: props.disableButtonsForHoldScreen || isDisabled$4(options.microphoneButton), disableTooltip: props.mobileView, onClickNoiseSuppression: onClickNoiseSuppression, isDeepNoiseSuppressionOn: isDeepNoiseSuppressionOn, showNoiseSuppressionButton: showNoiseSuppressionButton }),
                                    cameraButtonIsEnabled && React.createElement(Camera, { displayType: options.displayType, styles: commonButtonStyles, splitButtonsForDeviceSelection: !props.mobileView, disabled: props.disableButtonsForHoldScreen || isDisabled$4(options.cameraButton), onClickVideoEffects: props.onClickVideoEffects, componentRef: props.cameraButtonRef, disableTooltip: props.mobileView }),
                                    !props.mobileView && isReactionAllowed && isEnabled$1(options.reactionButton) && reactionResources && React.createElement(Reaction, { displayType: options.displayType, styles: commonButtonStyles, disabled: props.disableButtonsForHoldScreen, reactionResource: reactionResources }),
                                    !props.mobileView && isEnabled$1(options.raiseHandButton) && !hideRaiseHandButtonInRoomsCall && React.createElement(RaiseHand, { displayType: options.displayType, styles: commonButtonStyles, disabled: props.disableButtonsForHoldScreen || isDisabled$4(options.microphoneButton) }),
                                    showDtmfDialerButton(options) && props.onSetDialpadPage !== undefined && React.createElement(DtmfDialpadButton, { styles: commonButtonStyles, displayType: options.displayType, onClick: () => {
                                            if (props.onSetDialpadPage !== undefined) {
                                                props.onSetDialpadPage();
                                            }
                                        } }),
                                    showExitSpotlightButton && props.onStopLocalSpotlight && React.createElement(ExitSpotlightButton, { displayType: options.displayType, onClick: props.onStopLocalSpotlight, styles: commonButtonStyles, strings: exitSpotlightButtonStrings }),
                                    screenShareButtonIsEnabled && React.createElement(ScreenShare, { option: options.screenShareButton, displayType: options.displayType, styles: screenShareButtonStyles, disabled: props.disableButtonsForHoldScreen || isDisabled$4(options.screenShareButton) }), (_f = customButtons['primary']) === null || _f === void 0 ? void 0 :
                                    _f.slice(0, props.mobileView ? CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_MOBILE_CUSTOM_BUTTONS : CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_DESKTOP_CUSTOM_BUTTONS).map((CustomButton, i) => {
                                        return React.createElement(CustomButton, { key: `primary-custom-button-${i}`, styles: commonButtonStyles, showLabel: options.displayType !== 'compact', disableTooltip: props.mobileView });
                                    }),
                                    props.mobileView && React.createElement(MoreButton, { "data-ui-id": "common-call-composite-more-button", strings: moreButtonStrings, onClick: props.onMoreButtonClicked, disabled: props.disableButtonsForLobbyPage, disableTooltip: props.mobileView }),
                                    !props.mobileView && showDesktopMoreButton && React.createElement(DesktopMoreButton, { disableButtonsForHoldScreen: props.disableButtonsForHoldScreen, styles: commonButtonStyles, onClickShowDialpad: props.onClickShowDialpad, callControls: props.callControls, isCaptionsSupported: showCaptionsButton, isRealTimeTextSupported: showRealTimeTextButton, onCaptionsSettingsClick: openCaptionsSettingsModal, onStartRealTimeTextClick: openRealTimeTextModal, startRealTimeTextButtonChecked: props.startRealTimeTextButtonChecked, onUserSetOverflowGalleryPositionChange: props.onUserSetOverflowGalleryPositionChange, onUserSetGalleryLayout: props.onUserSetGalleryLayout, userSetGalleryLayout: props.userSetGalleryLayout, dtmfDialerPresent: props.dtmfDialerPresent, onSetDialpadPage: props.onSetDialpadPage, teamsMeetingPhoneCallEnable: showTeamsMeetingPhoneCallButton, onMeetingPhoneInfoClick: props.onToggleTeamsMeetingConferenceModal }),
                                    React.createElement(EndCall, { displayType: "compact", mobileView: props.mobileView, styles: endCallButtonStyles, enableEndCallMenu: !isBoolean(props.callControls) && !isBoolean((_g = props.callControls) === null || _g === void 0 ? void 0 : _g.endCallButton) && !props.mobileView && isHangUpForEveryoneAllowed && !isTeams &&
                                            // Temporary disable it for Teams call, since capability does not give the right value
                                            ((_j = (_h = props.callControls) === null || _h === void 0 ? void 0 : _h.endCallButton) === null || _j === void 0 ? void 0 : _j.hangUpForEveryone) === 'endCallOptions' &&
                                            // Only show the end call menu when the call is connected, user should not be able to end the call for everyone
                                            // when they are not actively in the call to communicate they will.
                                            callState.callStatus === 'Connected' && !canReturnToMainMeeting, disableEndCallModal: !isBoolean(props.callControls) && !isBoolean((_k = props.callControls) === null || _k === void 0 ? void 0 : _k.endCallButton) && ((_m = (_l = props.callControls) === null || _l === void 0 ? void 0 : _l.endCallButton) === null || _m === void 0 ? void 0 : _m.disableEndCallModal), returnFromBreakoutRoom: canReturnToMainMeeting ? returnFromBreakoutRoom : undefined }))))))),
            !props.mobileView && sideButtonsPresent && React.createElement(react.Stack.Item, null,
                React.createElement("div", { ref: sidepaneControlsRef },
                    React.createElement(react.Stack, { horizontal: true, className: !props.mobileView ? react.mergeStyles(desktopButtonContainerStyle) : undefined },
                        isEnabled$1(options === null || options === void 0 ? void 0 : options.peopleButton) && React.createElement(PeopleButton, { checked: props.peopleButtonChecked, ariaLabel: peopleButtonStrings.label, showLabel: options.displayType !== 'compact', onClick: props.onPeopleButtonClicked, "data-ui-id": "common-call-composite-people-button", disabled: props.disableButtonsForLobbyPage || props.disableButtonsForHoldScreen || isDisabled$4(options.peopleButton), strings: peopleButtonStrings, styles: commonButtonStyles, componentRef: props.peopleButtonRef, chatButtonPresent: isEnabled$1(options.chatButton), peoplePaneDismissButtonRef: props.sidePaneDismissButtonRef }), (_o = customButtons['secondary']) === null || _o === void 0 ? void 0 :
                        _o.slice(0, CUSTOM_BUTTON_OPTIONS.MAX_SECONDARY_DESKTOP_CUSTOM_BUTTONS).map((CustomButton, i) => {
                            return React.createElement(CustomButton, { key: `secondary-custom-button-${i}`, styles: commonButtonStyles, showLabel: options.displayType !== 'compact' });
                        })))),
            (breakoutRoomSettings === null || breakoutRoomSettings === void 0 ? void 0 : breakoutRoomSettings.roomEndTime) && !props.mobileView && !isOutOfSpace && React.createElement(react.Stack.Item, null,
                React.createElement(Timer, { timeStampInfo: breakoutRoomSettings === null || breakoutRoomSettings === void 0 ? void 0 : breakoutRoomSettings.roomEndTime.toString() }))));
};
const desktopButtonContainerStyle = {
    padding: '0.75rem',
    columnGap: '0.5rem'
};
const desktopControlBarStyles = {
    root: desktopButtonContainerStyle
};
const wrapperDesktopStyles = {
    position: 'absolute',
    left: '50%',
    transform: 'translate(-50%, 0)'
};
const wrapperDesktopRtlStyles = {
    position: 'absolute',
    right: '50%',
    transform: 'translate(-50%, 0)'
};
/** @private */
const getDesktopCommonButtonStyles = (theme) => ({
    root: {
        border: `solid 1px ${theme.palette.neutralQuaternaryAlt}`,
        borderRadius: theme.effects.roundedCorner4,
        minHeight: '2.5rem',
        maxWidth: '12rem' // allot extra space than the regular ControlBarButton. This is to give extra room to have the icon beside the text.
    },
    flexContainer: {
        display: 'flex',
        flexFlow: 'row nowrap'
    },
    textContainer: {
        // Override the default so that label doesn't introduce a new block.
        display: 'inline',
        // Ensure width is set to permit child to show ellipsis when there is a label that is too long
        maxWidth: '100%'
    },
    label: {
        fontSize: theme.fonts.medium.fontSize,
        // Ensure there is enough space between the icon and text to allow for the unread messages badge in the chat button
        marginLeft: '0.625rem',
        // Ensure letters that go above and below the standard text line like 'g', 'y', 'j' are not clipped
        lineHeight: '1.5rem',
        // Do not allow very long button texts to ruin the control bar experience, instead ensure long text is truncated and shows ellipsis
        display: 'block',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap',
        overflow: 'hidden'
    },
    splitButtonMenuButton: {
        border: `solid 1px ${theme.palette.neutralQuaternaryAlt}`,
        borderTopRightRadius: theme.effects.roundedCorner4,
        borderBottomRightRadius: theme.effects.roundedCorner4,
        borderTopLeftRadius: '0',
        borderBottomLeftRadius: '0'
    },
    splitButtonMenuButtonChecked: {
        // Default colors the menu half similarly for :hover and when button is checked.
        // To align with how the left-half is styled, override the checked style.
        background: 'none'
    }
});
const getDesktopScreenShareButtonStyles = (theme) => {
    const overrideStyles = {
        border: 'none',
        background: theme.palette.themePrimary,
        color: theme.palette.white,
        '* > svg': {
            fill: theme.palette.white
        },
        '@media (forced-colors: active)': {
            border: '1px solid',
            borderColor: theme.palette.black
        }
    };
    const overrides = {
        rootChecked: overrideStyles,
        rootCheckedHovered: overrideStyles
    };
    return react.concatStyleSets(getDesktopCommonButtonStyles(theme), overrides);
};
const getDesktopEndCallButtonStyles = (theme) => {
    const overrides = {
        root: {
            // Suppress border around the dark-red button.
            border: 'none'
        },
        rootFocused: {
            '@media (forced-colors: active)': {
                background: 'highlight',
                color: 'highlightText',
                borderColor: theme.palette.black,
                borderRadius: 'unset',
                outline: `3px solid ${theme.palette.black}`
            }
        },
        icon: {
            '@media (forced-colors: active)': {
                ':focused': {
                    color: theme.palette.white
                }
            }
        }
    };
    return react.concatStyleSets(getDesktopCommonButtonStyles(theme), overrides);
};
const isEnabled$1 = (option) => option !== false;
const showDtmfDialerButton = (options) => {
    if (options.moreButton === false && options.dtmfDialerButton !== false) {
        return true;
    }
    else {
        return false;
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const captionSettingsDrawerStyles = (theme) => ({
    root: {
        overflow: 'auto'
    },
    drawerSurfaceStyles: {
        drawerContentContainer: {
            root: {
                span: {
                    fontWeight: 400,
                    fontSize: _pxToRem(14),
                    lineHeight: _pxToRem(22),
                    color: theme.palette.neutralDark
                }
            }
        }
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const CaptionLanguageSettingsDrawer = (props) => {
    var _a;
    const theme = useTheme();
    const onDrawerItemClick = React.useCallback((languageCode) => {
        props.selectLanguage(languageCode);
    }, [props]);
    const drawerItems = React.useMemo(() => {
        var _a;
        return _getKeys((_a = props.supportedLanguageStrings) !== null && _a !== void 0 ? _a : []).map(languageCode => ({
            itemKey: languageCode,
            text: props.supportedLanguageStrings ? props.supportedLanguageStrings[languageCode] : languageCode,
            onItemClick: () => onDrawerItemClick(languageCode),
            secondaryIconProps: props.currentLanguage === languageCode ? {
                iconName: 'Accept'
            } : undefined
        }));
    }, [props.currentLanguage, props.supportedLanguageStrings, onDrawerItemClick]);
    const sortedDrawerItems = React.useMemo(() => {
        const copy = [...drawerItems];
        return copy.sort((a, b) => a.text && b.text && a.text > b.text ? 1 : -1);
    }, [drawerItems]);
    return React.createElement(_DrawerMenu, { heading: (_a = props.strings) === null || _a === void 0 ? void 0 : _a.menuTitle, items: sortedDrawerItems !== null && sortedDrawerItems !== void 0 ? sortedDrawerItems : [], onLightDismiss: () => {
            props.setCurrentLanguage(props.currentLanguage);
            props.onLightDismiss();
        }, styles: captionSettingsDrawerStyles(theme) });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const themedToggleButtonStyle = (theme, checked) => {
    if (checked) {
        return {
            root: {
                margin: 0
            },
            pill: {
                backgroundColor: `${theme.palette.themePrimary} !important`
            },
            thumb: {
                backgroundColor: `${theme.palette.white} !important`
            }
        };
    }
    return {
        root: {
            margin: 0
        }
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const SpokenLanguageSettingsDrawer = (props) => {
    var _a;
    const theme = useTheme();
    const onDrawerItemClick = React.useCallback((languageCode) => {
        props.selectLanguage(languageCode);
    }, [props]);
    const drawerItems = React.useMemo(() => {
        var _a;
        return _getKeys((_a = props.supportedLanguageStrings) !== null && _a !== void 0 ? _a : []).map(languageCode => ({
            itemKey: languageCode,
            text: props.supportedLanguageStrings ? props.supportedLanguageStrings[languageCode] : languageCode,
            onItemClick: () => onDrawerItemClick(languageCode),
            secondaryIconProps: props.currentLanguage === languageCode ? {
                iconName: 'Accept'
            } : undefined
        }));
    }, [props.currentLanguage, props.supportedLanguageStrings, onDrawerItemClick]);
    const sortedDrawerItems = React.useMemo(() => {
        const copy = [...drawerItems];
        return copy.sort((a, b) => a.text && b.text && a.text > b.text ? 1 : -1);
    }, [drawerItems]);
    return React.createElement(_DrawerMenu, { heading: (_a = props.strings) === null || _a === void 0 ? void 0 : _a.menuTitle, items: sortedDrawerItems !== null && sortedDrawerItems !== void 0 ? sortedDrawerItems : [], onLightDismiss: () => {
            props.setCurrentLanguage(props.currentLanguage);
            props.onLightDismiss();
        }, styles: captionSettingsDrawerStyles(theme) });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$j = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const inferCallWithChatControlOptions = (callWithChatControls) => {
    if (callWithChatControls === false) {
        return false;
    }
    const options = callWithChatControls === true || callWithChatControls === undefined ? {} : callWithChatControls;
    return options;
};
/** @private */
const MoreDrawer = (props) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const theme = react.useTheme();
    const callAdapter = useAdapter();
    const drawerMenuItems = [];
    const { speakers, onSelectSpeaker, onLightDismiss } = props;
    const localeStrings = useLocale();
    const holdButtonProps = usePropsFor$1(HoldButton);
    const realTimeTextProps = usePropsFor$1(CaptionsBanner);
    const callees = useSelector$1(getTargetCallees);
    const participants = useSelector$1(getRemoteParticipantsConnectedSelector);
    const allowDtmfDialer = showDtmfDialer(callees, participants, props.dtmfDialerOptions);
    const [dtmfDialerChecked, setDtmfDialerChecked] = React.useState((_a = props.dtmfDialerPresent) !== null && _a !== void 0 ? _a : false);
    const [showRealTimeTextModal, setShowRealTimeTextModal] = React.useState(false);
    const openRealTimeTextModal = React.useCallback(() => {
        setShowRealTimeTextModal(true);
    }, []);
    const onDismissRealTimeTextModal = React.useCallback(() => {
        setShowRealTimeTextModal(false);
    }, []);
    const raiseHandButtonProps = usePropsFor$1(RaiseHandButton);
    const participantCapability = useSelector$1(getCapabilites);
    const participantId = useSelector$1(getLocalUserId);
    const isTogetherModeActive = useSelector$1(getIsTogetherModeActive);
    const isTeamsCall = useSelector$1(getIsTeamsCall);
    const isTeamsMeeting = getIsTeamsMeeting(callAdapter.getState());
    const onSpeakerItemClick = React.useCallback((_ev, itemKey) => {
        const selected = speakers === null || speakers === void 0 ? void 0 : speakers.find(speaker => speaker.id === itemKey);
        if (selected) {
            // This is unsafe - we're only passing in part of the argument to the handler.
            // But this is a known issue in our state.
            onSelectSpeaker(selected);
        }
        onLightDismiss();
    }, [speakers, onSelectSpeaker, onLightDismiss]);
    const drawerSelectionOptions = inferCallWithChatControlOptions(props.callControls);
    const showCaptionsButton = props.isCaptionsSupported && drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions.captionsButton);
    const showRealTimeTextButton = props.isRealTimeTextSupported && drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions.realTimeTextButton);
    if (props.reactionResources !== undefined) {
        drawerMenuItems.push({
            itemKey: 'reactions',
            onRendererContent: () => React.createElement(_ReactionDrawerMenuItem, { onReactionClick: (reaction) => __awaiter$j(void 0, void 0, void 0, function* () {
                    var _h;
                    (_h = props.onReactionClick) === null || _h === void 0 ? void 0 : _h.call(props, reaction);
                    onLightDismiss();
                }), reactionResources: props.reactionResources })
        });
    }
    if (props.speakers && props.speakers.length > 0) {
        drawerMenuItems.push({
            itemKey: 'speakers',
            disabled: props.disableButtonsForHoldScreen,
            text: props.strings.speakerMenuTitle,
            iconProps: {
                iconName: 'MoreDrawerSpeakers'
            },
            subMenuProps: props.speakers.map(speaker => ({
                itemKey: speaker.id,
                iconProps: {
                    iconName: isDeviceSelected(speaker, props.selectedSpeaker) ? 'MoreDrawerSelectedSpeaker' : 'MoreDrawerSpeakers'
                },
                text: speaker.name,
                onItemClick: onSpeakerItemClick,
                secondaryIconProps: isDeviceSelected(speaker, props.selectedSpeaker) ? {
                    iconName: 'Accept'
                } : undefined
            })),
            secondaryText: (_b = props.selectedSpeaker) === null || _b === void 0 ? void 0 : _b.name
        });
    }
    const { microphones, onSelectMicrophone } = props;
    const onMicrophoneItemClick = React.useCallback((_ev, itemKey) => {
        const selected = microphones === null || microphones === void 0 ? void 0 : microphones.find(mic => mic.id === itemKey);
        if (selected) {
            // This is unsafe - we're only passing in part of the argument to the handler.
            // But this is a known issue in our state.
            onSelectMicrophone(selected);
        }
        onLightDismiss();
    }, [microphones, onSelectMicrophone, onLightDismiss]);
    if (props.microphones && props.microphones.length > 0) {
        // Set props as Microphone if speakers can be enumerated else set as Audio Device
        const speakersAvailable = props.speakers && props.speakers.length > 0;
        const itemKey = speakersAvailable ? 'microphones' : 'audioDevices';
        const text = speakersAvailable ? props.strings.microphoneMenuTitle : props.strings.audioDeviceMenuTitle;
        const iconName = speakersAvailable ? 'MoreDrawerMicrophones' : 'MoreDrawerSpeakers';
        const selectedIconName = speakersAvailable ? 'MoreDrawerSelectedMicrophone' : 'MoreDrawerSelectedSpeaker';
        drawerMenuItems.push({
            itemKey: itemKey,
            disabled: props.disableButtonsForHoldScreen,
            text: text,
            iconProps: {
                iconName: iconName
            },
            subMenuProps: props.microphones.map(mic => ({
                itemKey: mic.id,
                iconProps: {
                    iconName: isDeviceSelected(mic, props.selectedMicrophone) ? selectedIconName : iconName
                },
                text: mic.name,
                onItemClick: onMicrophoneItemClick,
                secondaryIconProps: isDeviceSelected(mic, props.selectedMicrophone) ? {
                    iconName: 'Accept'
                } : undefined,
                disabled: drawerSelectionOptions !== false ? isDisabled$4(drawerSelectionOptions.microphoneButton) : undefined
            })),
            secondaryText: (_c = props.selectedMicrophone) === null || _c === void 0 ? void 0 : _c.name
        });
    }
    const dtmfDialerScreenOption = {
        itemKey: 'dtmfDialerScreenKey',
        text: !dtmfDialerChecked ? localeStrings.strings.call.dtmfDialerMoreButtonLabelOn : localeStrings.strings.call.dtmfDialerMoreButtonLabelOff,
        onItemClick: () => {
            if (props.onSetDialpadPage) {
                props.onSetDialpadPage();
            }
            setDtmfDialerChecked(!dtmfDialerChecked);
            onLightDismiss();
        },
        iconProps: {
            iconName: 'DtmfDialpadButton',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        }
    };
    /**
     * Only render the dtmf dialer if the dialpad for PSTN calls is not present
     */
    if (props.onSetDialpadPage && allowDtmfDialer && drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions.dtmfDialerButton)) {
        drawerMenuItems.push(dtmfDialerScreenOption);
    }
    const galleryLayoutOptions = {
        itemKey: 'galleryPositionKey',
        iconProps: {
            iconName: 'GalleryOptions',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        },
        disabled: props.disableButtonsForHoldScreen,
        text: localeStrings.strings.call.moreButtonGalleryControlLabel,
        subMenuProps: [{
                itemKey: 'dynamicSelectionKey',
                text: localeStrings.strings.call.moreButtonGalleryFloatingLocalLayoutLabel,
                onItemClick: () => {
                    props.onUserSetGalleryLayout && props.onUserSetGalleryLayout('floatingLocalVideo');
                    onLightDismiss();
                },
                iconProps: {
                    iconName: 'FloatingLocalVideoGalleryLayout',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                secondaryIconProps: props.userSetGalleryLayout === 'floatingLocalVideo' ? {
                    iconName: 'Accept'
                } : undefined
            }, {
                itemKey: 'focusedContentSelectionKey',
                text: localeStrings.strings.call.moreButtonGalleryFocusedContentLayoutLabel,
                onItemClick: () => {
                    props.onUserSetGalleryLayout && props.onUserSetGalleryLayout('focusedContent');
                    onLightDismiss();
                },
                iconProps: {
                    iconName: 'FocusedContentGalleryLayout',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                secondaryIconProps: props.userSetGalleryLayout === 'focusedContent' ? {
                    iconName: 'Accept'
                } : undefined
            }]
    };
    const togetherModeOption = {
        itemKey: 'togetherModeSelectionKey',
        text: localeStrings.strings.call.moreButtonTogetherModeLayoutLabel,
        onItemClick: () => {
            props.onUserSetGalleryLayout && props.onUserSetGalleryLayout('togetherMode');
            onLightDismiss();
        },
        iconProps: {
            iconName: 'TogetherModeLayout',
            styles: {
                root: {
                    lineHeight: 0
                }
            }
        },
        disabled: !((participantId === null || participantId === void 0 ? void 0 : participantId.kind) === 'microsoftTeamsUser' && ((_d = participantCapability === null || participantCapability === void 0 ? void 0 : participantCapability.startTogetherMode) === null || _d === void 0 ? void 0 : _d.isPresent) || isTogetherModeActive),
        secondaryIconProps: props.userSetGalleryLayout === 'default' ? {
            iconName: 'Accept'
        } : undefined
    };
    if (isTeamsCall || isTeamsMeeting) {
        (_e = galleryLayoutOptions.subMenuProps) === null || _e === void 0 ? void 0 : _e.push(togetherModeOption);
    }
    if (drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions === null || drawerSelectionOptions === void 0 ? void 0 : drawerSelectionOptions.galleryControlsButton)) {
        drawerMenuItems.push(galleryLayoutOptions);
    }
    if (drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions === null || drawerSelectionOptions === void 0 ? void 0 : drawerSelectionOptions.peopleButton)) {
        drawerMenuItems.push({
            itemKey: 'people',
            id: 'call-composite-drawer-people-button',
            text: props.strings.peopleButtonLabel,
            iconProps: {
                iconName: 'MoreDrawerPeople'
            },
            onItemClick: props.onPeopleButtonClicked,
            disabled: isDisabled$4(drawerSelectionOptions.peopleButton) || props.disableButtonsForHoldScreen
        });
    }
    if (drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions === null || drawerSelectionOptions === void 0 ? void 0 : drawerSelectionOptions.holdButton)) {
        drawerMenuItems.push({
            itemKey: 'holdButtonKey',
            disabled: props.disableButtonsForHoldScreen || isDisabled$4(drawerSelectionOptions.holdButton),
            text: localeStrings.component.strings.holdButton.tooltipOffContent,
            onItemClick: () => {
                holdButtonProps.onToggleHold();
                onLightDismiss();
            },
            iconProps: {
                iconName: 'HoldCallContextualMenuItem',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            }
        });
    }
    const role = (_f = callAdapter.getState().call) === null || _f === void 0 ? void 0 : _f.role;
    const hideRaiseHandButtonInRoomsCall = callAdapter.getState().isRoomsCall && role && ['Consumer', 'Unknown'].includes(role);
    if (drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions === null || drawerSelectionOptions === void 0 ? void 0 : drawerSelectionOptions.raiseHandButton) && !hideRaiseHandButtonInRoomsCall) {
        const raiseHandIcon = raiseHandButtonProps.checked ? 'LowerHandContextualMenuItem' : 'RaiseHandContextualMenuItem';
        drawerMenuItems.push({
            itemKey: 'raiseHandButtonKey',
            disabled: props.disableButtonsForHoldScreen || isDisabled$4(drawerSelectionOptions.raiseHandButton),
            text: raiseHandButtonProps.checked ? localeStrings.component.strings.raiseHandButton.onLabel : localeStrings.component.strings.raiseHandButton.offLabel,
            onItemClick: () => {
                if (raiseHandButtonProps.onToggleRaiseHand) {
                    raiseHandButtonProps.onToggleRaiseHand();
                }
                onLightDismiss();
            },
            iconProps: {
                iconName: raiseHandIcon,
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            }
        });
    }
    const teamsMeetingCoordinates = getTeamsMeetingCoordinates(callAdapter.getState());
    if (drawerSelectionOptions !== false && isEnabled(drawerSelectionOptions === null || drawerSelectionOptions === void 0 ? void 0 : drawerSelectionOptions.teamsMeetingPhoneCallButton) && isTeamsMeeting && teamsMeetingCoordinates) {
        drawerMenuItems.push({
            itemKey: 'phoneCallInfoKey',
            disabled: isDisabled$4(drawerSelectionOptions.teamsMeetingPhoneCallButton),
            text: localeStrings.strings.call.phoneCallMoreButtonLabel,
            onItemClick: () => {
                var _a;
                (_a = props.onClickMeetingPhoneInfo) === null || _a === void 0 ? void 0 : _a.call(props);
                onLightDismiss();
            },
            iconProps: {
                iconName: 'PhoneNumberButton',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            }
        });
    }
    //Captions drawer menu
    const supportedSpokenLanguageStrings = useLocale().strings.call.spokenLanguageStrings;
    //Captions drawer menu
    const supportedCaptionLanguageStrings = useLocale().strings.call.captionLanguageStrings;
    const captionSettingsProp = usePropsFor$1(CaptionsSettingsModal);
    const startCaptionsButtonProps = usePropsFor$1(StartCaptionsButton);
    const [isSpokenLanguageDrawerOpen, setIsSpokenLanguageDrawerOpen] = React.useState(false);
    const [isCaptionLanguageDrawerOpen, setIsCaptionLanguageDrawerOpen] = React.useState(false);
    // we don't display the setting modal to set the spoken language on mobile
    // so when spoken language is empty (not set), we default to 'en-us'
    const [currentSpokenLanguage, setCurrentSpokenLanguage] = React.useState(captionSettingsProp.currentSpokenLanguage && captionSettingsProp.currentSpokenLanguage !== '' ? captionSettingsProp.currentSpokenLanguage : 'en-us');
    const [currentCaptionLanguage, setCurrentCaptionLanguage] = React.useState((_g = captionSettingsProp.currentCaptionLanguage) !== null && _g !== void 0 ? _g : _spokenLanguageToCaptionLanguage[currentSpokenLanguage]);
    const onToggleChange = React.useCallback(() => __awaiter$j(void 0, void 0, void 0, function* () {
        if (!captionSettingsProp.isCaptionsFeatureActive) {
            yield startCaptionsButtonProps.onStartCaptions({
                spokenLanguage: currentSpokenLanguage
            });
        }
        else {
            startCaptionsButtonProps.onStopCaptions();
        }
    }), [captionSettingsProp.isCaptionsFeatureActive, startCaptionsButtonProps, currentSpokenLanguage]);
    if (showCaptionsButton) {
        const captionsDrawerItems = [];
        const spokenLanguageString = supportedSpokenLanguageStrings ? supportedSpokenLanguageStrings[currentSpokenLanguage] : currentSpokenLanguage;
        drawerMenuItems.push({
            itemKey: 'captions',
            id: 'common-call-composite-captions-button',
            disabled: props.disableButtonsForHoldScreen,
            text: props.strings.captionsMenuTitle,
            iconProps: {
                iconName: 'CaptionsIcon'
            },
            subMenuProps: captionsDrawerItems
        });
        captionsDrawerItems.push({
            itemKey: 'ToggleCaptionsKey',
            text: captionSettingsProp.isCaptionsFeatureActive ? localeStrings.strings.call.startCaptionsButtonTooltipOnContent : localeStrings.strings.call.startCaptionsButtonTooltipOffContent,
            iconProps: {
                iconName: captionSettingsProp.isCaptionsFeatureActive ? 'CaptionsOffIcon' : 'CaptionsIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            onItemClick: onToggleChange,
            disabled: props.disableButtonsForHoldScreen,
            secondaryComponent: React.createElement(react.Stack, { verticalFill: true, verticalAlign: "center" },
                React.createElement(react.Toggle, { id: "common-call-composite-captions-toggle-button", checked: captionSettingsProp.isCaptionsFeatureActive, styles: themedToggleButtonStyle(theme, captionSettingsProp.isCaptionsFeatureActive), onChange: onToggleChange }))
        });
        captionsDrawerItems.push({
            itemKey: 'ChangeSpokenLanguage',
            text: props.strings.spokenLanguageMenuTitle,
            id: 'common-call-composite-captions-spoken-settings-button',
            secondaryText: spokenLanguageString,
            iconProps: {
                iconName: 'ChangeSpokenLanguageIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            disabled: props.disableButtonsForHoldScreen || !captionSettingsProp.isCaptionsFeatureActive,
            onItemClick: () => {
                setIsSpokenLanguageDrawerOpen(true);
            },
            secondaryIconProps: {
                iconName: 'ChevronRight',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            }
        });
        if (props.useTeamsCaptions) {
            const captionLanguageString = supportedCaptionLanguageStrings ? supportedCaptionLanguageStrings[currentCaptionLanguage] : currentCaptionLanguage;
            captionsDrawerItems.push({
                itemKey: 'ChangeCaptionLanguage',
                text: props.strings.captionLanguageMenuTitle,
                id: 'common-call-composite-captions-subtitle-settings-button',
                secondaryText: captionLanguageString,
                iconProps: {
                    iconName: 'ChangeCaptionLanguageIcon',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                disabled: props.disableButtonsForHoldScreen || !captionSettingsProp.isCaptionsFeatureActive,
                onItemClick: () => {
                    setIsCaptionLanguageDrawerOpen(true);
                },
                secondaryIconProps: {
                    iconName: 'ChevronRight',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                }
            });
        }
    }
    const rttDisabled = props.disableButtonsForHoldScreen || realTimeTextProps.isRealTimeTextOn || props.startRealTimeTextButtonChecked;
    // rtt
    if (showRealTimeTextButton) {
        const realTimeTextDrawerItems = [];
        drawerMenuItems.push({
            itemKey: 'realTimeText',
            id: 'common-call-composite-rtt-button',
            disabled: props.disableButtonsForHoldScreen,
            text: localeStrings.strings.call.realTimeTextLabel,
            iconProps: {
                iconName: 'RealTimeTextIcon'
            },
            subMenuProps: realTimeTextDrawerItems
        });
        realTimeTextDrawerItems.push({
            itemKey: 'ToggleRTTKey',
            id: 'common-call-composite-rtt-start-button',
            text: localeStrings.strings.call.startRealTimeTextLabel,
            ariaLabel: rttDisabled ? localeStrings.strings.call.disabledStartRealTimeTextLabel : localeStrings.strings.call.startRealTimeTextLabel,
            iconProps: {
                iconName: 'RealTimeTextIcon',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            onItemClick: openRealTimeTextModal,
            disabled: rttDisabled,
            secondaryComponent: React.createElement(react.Stack, { verticalFill: true, verticalAlign: "center" },
                React.createElement(react.Toggle, { id: "common-call-composite-rtt-toggle-button", checked: realTimeTextProps.isRealTimeTextOn || props.startRealTimeTextButtonChecked, styles: themedToggleButtonStyle(theme, realTimeTextProps.isRealTimeTextOn || props.startRealTimeTextButtonChecked), onChange: openRealTimeTextModal }))
        });
    }
    const customDrawerButtons = React.useMemo(() => generateCustomCallDrawerButtons(onFetchCustomButtonPropsTrampoline(drawerSelectionOptions !== false ? drawerSelectionOptions : undefined), drawerSelectionOptions !== false ? drawerSelectionOptions === null || drawerSelectionOptions === void 0 ? void 0 : drawerSelectionOptions.displayType : undefined), [drawerSelectionOptions]);
    customDrawerButtons['primary'].slice(CUSTOM_BUTTON_OPTIONS.MAX_PRIMARY_MOBILE_CUSTOM_BUTTONS).forEach(element => {
        drawerMenuItems.push(element);
    });
    customDrawerButtons['secondary'].forEach(element => {
        drawerMenuItems.push(element);
    });
    customDrawerButtons['overflow'].forEach(element => {
        const customButtonProps = Object.assign({}, element);
        // Default Auto dismiss drawer on click unless specified
        if (customButtonProps.dismissDrawer !== false) {
            element = Object.assign(Object.assign({}, customButtonProps), { onItemClick: () => {
                    var _a;
                    (_a = customButtonProps.onItemClick) === null || _a === void 0 ? void 0 : _a.call(customButtonProps);
                    onLightDismiss();
                } });
        }
        drawerMenuItems.push(element);
    });
    return React.createElement(React.Fragment, null,
        showRealTimeTextModal && React.createElement(CallingRealTimeTextModal, { showModal: showRealTimeTextModal, onDismissModal: onDismissRealTimeTextModal, onStartRealTimeText: props.onStartRealTimeText }),
        isSpokenLanguageDrawerOpen && showCaptionsButton && React.createElement(SpokenLanguageSettingsDrawer, { onLightDismiss: props.onLightDismiss, selectLanguage: setCurrentSpokenLanguage, setCurrentLanguage: captionSettingsProp.onSetSpokenLanguage, currentLanguage: currentSpokenLanguage, strings: {
                menuTitle: props.strings.spokenLanguageMenuTitle
            }, supportedLanguageStrings: supportedSpokenLanguageStrings }),
        isCaptionLanguageDrawerOpen && showCaptionsButton && React.createElement(CaptionLanguageSettingsDrawer, { onLightDismiss: props.onLightDismiss, selectLanguage: setCurrentCaptionLanguage, setCurrentLanguage: captionSettingsProp.onSetCaptionLanguage, currentLanguage: currentCaptionLanguage, strings: {
                menuTitle: props.strings.captionLanguageMenuTitle
            }, supportedLanguageStrings: supportedCaptionLanguageStrings }),
        !isSpokenLanguageDrawerOpen && !isCaptionLanguageDrawerOpen && React.createElement(_DrawerMenu, { items: drawerMenuItems, onLightDismiss: props.onLightDismiss }));
};
const isDeviceSelected = (speaker, selectedSpeaker) => !!selectedSpeaker && speaker.id === selectedSpeaker.id;
const isEnabled = (option) => option !== false;

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const moreDrawerSelector = reselect__namespace.createSelector([getDeviceManager], deviceManager => {
    return {
        microphones: deviceManager.microphones,
        speakers: deviceManager.speakers,
        selectedMicrophone: deviceManager.selectedMicrophone,
        selectedSpeaker: deviceManager.selectedSpeaker
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const PreparedMoreDrawer = (props) => {
    const strings = useCallWithChatCompositeStrings();
    const moreDrawerStrings = React.useMemo(() => ({
        peopleButtonLabel: strings.peopleButtonLabel,
        audioDeviceMenuTitle: strings.moreDrawerAudioDeviceMenuTitle,
        microphoneMenuTitle: strings.moreDrawerMicrophoneMenuTitle,
        speakerMenuTitle: strings.moreDrawerSpeakerMenuTitle,
        captionsMenuTitle: strings.moreDrawerCaptionsMenuTitle,
        spokenLanguageMenuTitle: strings.moreDrawerSpokenLanguageMenuTitle,
        captionLanguageMenuTitle: strings.moreDrawerCaptionLanguageMenuTitle,
        galleryOptionsMenuTitle: strings.moreDrawerGalleryOptionsMenuTitle
    }), [strings]);
    const deviceProps = useSelector$1(moreDrawerSelector);
    const callHandlers = useHandlers();
    return React.createElement(MoreDrawer, Object.assign({}, props, deviceProps, callHandlers, { strings: moreDrawerStrings }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const DEFAULT_Z_INDEX = 1;
const mainScreenContainerStyle = {
    height: '100%',
    width: '100%'
};
/**
 * @private
 */
const mainScreenContainerStyleDesktop = react.mergeStyles(Object.assign(Object.assign({}, mainScreenContainerStyle), { minWidth: '20rem', 
    // max of min-width of composite pages (Call page) 400% zoom compliant size
    minHeight: '16rem' // max height of min-height of composite pages (Configuration page) 400% zoom compliant size
 }));
/**
 * @private
 */
const mainScreenContainerStyleMobile = react.mergeStyles(Object.assign(Object.assign({}, mainScreenContainerStyle), { minWidth: '17.5rem', 
    // max of min-width of composite pages (Call page)
    minHeight: '13rem' // max height of min-height of composite pages (Configuration page & Call page)
 }));
/**
 * @private
 * Drawer styles to be used to house the _DrawerComponent on top of other content on the screen.
 * @param zIndex: this defaults to DEFAULT_Z_INDEX if unset
 */
const drawerContainerStyles = (zIndex = DEFAULT_Z_INDEX) => ({
    root: {
        position: 'absolute',
        top: 0,
        left: 0,
        width: '100%',
        height: '100%',
        // Any zIndex > 0 will work because this is the only absolutely
        // positioned element in the container.
        zIndex: zIndex
    }
});
/**
 * @private
 * Page container style for leaving page
 */
const leavingPageStyle = (palette) => ({
    background: palette.neutralLighterAlt
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const SIDE_PANE_PADDING_LR_REM = 0.25;
/**
 * @private
 */
const mobilePaneStyle = {
    root: {
        width: '100%',
        height: '100%'
    }
};
/**
 * @private
 */
react.concatStyleSets(mobilePaneStyle, {
    root: {
        display: 'none'
    }
});
/**
 * @private
 */
const mobilePaneControlBarStyle = {
    root: {
        height: '3rem'
    }
};
/**
 * @private
 */
const mobilePaneBackButtonStyles = {
    root: {
        border: 'none',
        minWidth: '2.5rem',
        maxWidth: '2.875rem',
        height: '100%',
        background: 'none',
        padding: '0 1rem'
    },
    rootChecked: {
        background: 'none'
    },
    rootCheckedHovered: {
        background: 'none'
    }
};
/**
 * @private
 */
const mobilePaneHiddenIconStyles = react.concatStyleSets(mobilePaneBackButtonStyles, {
    root: {
        visibility: 'hidden'
    }
});
/**
 * @private
 */
const mobilePaneButtonStyles = {
    root: {
        border: 'none',
        borderBottom: '0.125rem solid transparent',
        width: '8rem',
        height: '100%',
        background: 'none',
        padding: '0'
    },
    rootChecked: {
        background: 'none'
    },
    rootCheckedHovered: {
        background: 'none'
    },
    flexContainer: {
        flexFlow: 'column',
        display: 'contents'
    },
    label: {
        fontSize: '1rem',
        fontWeight: 100,
        lineHeight: '2rem',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        whiteSpace: 'nowrap'
    },
    labelChecked: {
        fontWeight: 600
    }
};
/**
 * @private
 */
const hiddenStyles = {
    root: {
        display: 'none'
    }
};
/**
 * @private
 */
const sidePaneStyles = (maxWidth, minWidth) => ({
    root: {
        height: 'auto',
        width: '100%',
        padding: `0.5rem ${SIDE_PANE_PADDING_LR_REM}rem`,
        maxWidth: maxWidth !== null && maxWidth !== void 0 ? maxWidth : '21.5rem',
        minWidth: minWidth !== null && minWidth !== void 0 ? minWidth : `${CHAT_CONTAINER_MIN_WIDTH_REM + SIDE_PANE_PADDING_LR_REM * 2}rem`
    }
});
/**
 * @private
 */
const availableSpaceStyles = {
    root: {
        width: '100%',
        height: '100%'
    }
};
/**
 * @private
 */
const sidePaneTokens = {
    childrenGap: '0.5rem'
};
/** @private */
const paneHighContrastStyles = (theme) => react.mergeStyles({
    '@media (forced-colors: active)': {
        border: `0.125rem solid ${theme.palette.black}`,
        borderRadius: theme.effects.roundedCorner4,
        margin: '0.5rem 0.25rem 0.5rem 0rem'
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Context for side pane interaction
 *
 * @private
 */
const SidePaneContext = React.createContext({});
/**
 * Provider to provide localized strings for this library's composites.
 *
 * @private
 */
const SidePaneProvider = (props) => {
    return React.createElement(SidePaneContext.Provider, { value: props }, props.children);
};
/** @private */
const useSidePaneContext = () => React.useContext(SidePaneContext);
/** @private */
const useIsSidePaneOpen = () => {
    const { sidePaneRenderer, overrideSidePane } = useSidePaneContext();
    return !!(sidePaneRenderer || (overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.isActive));
};
/** @private */
const useIsParticularSidePaneOpen = (sidePaneId) => {
    const isSidePaneOpen = useIsSidePaneOpen();
    const { sidePaneRenderer } = useSidePaneContext();
    return isSidePaneOpen && (sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id) === sidePaneId;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Legacy header to be removed when we make a breaking change.
 * @private
 */
const PeopleAndChatHeader = (props) => {
    const { onClose, onChatButtonClicked, onPeopleButtonClicked, activeTab } = props;
    const theme = useTheme();
    const strings = useLocale().strings.call;
    const haveMultipleTabs = onChatButtonClicked && onPeopleButtonClicked;
    const mobilePaneButtonStylesThemed = React.useMemo(() => {
        return react.concatStyleSets(mobilePaneButtonStyles, {
            root: {
                width: '100%'
            },
            label: {
                fontSize: theme.fonts.medium.fontSize,
                fontWeight: theme.fonts.medium.fontWeight
            }
        }, haveMultipleTabs ? {
            rootChecked: {
                borderBottom: `0.125rem solid ${theme.palette.themePrimary}`
            }
        } : {});
    }, [theme, haveMultipleTabs]);
    return React.createElement(react.Stack, { horizontal: true, grow: true, styles: mobilePaneControlBarStyle },
        React.createElement(react.DefaultButton, { ariaLabel: strings.returnToCallButtonAriaLabel, ariaDescription: strings.returnToCallButtonAriaDescription, onClick: onClose, styles: mobilePaneBackButtonStyles, onRenderIcon: () => React.createElement(CallWithChatCompositeIcon, { iconName: "ChevronLeft" }), autoFocus: true }),
        React.createElement(react.Stack.Item, { grow: true, role: "tablist" },
            React.createElement(react.FocusZone, { direction: react.FocusZoneDirection.horizontal, className: react.mergeStyles(availableSpaceStyles.root) },
                React.createElement(react.Stack, { horizontal: true, styles: availableSpaceStyles },
                    React.createElement(react.Stack.Item, { grow: true }, onChatButtonClicked && React.createElement(react.DefaultButton, { onClick: onChatButtonClicked, styles: mobilePaneButtonStylesThemed, checked: activeTab === 'chat', "aria-selected": activeTab === 'chat', role: 'tab', disabled: props.disableChatButton }, strings.chatButtonLabel)),
                    React.createElement(react.Stack.Item, { grow: true }, onPeopleButtonClicked && React.createElement(react.DefaultButton, { onClick: onPeopleButtonClicked, styles: mobilePaneButtonStylesThemed, checked: activeTab === 'people', "aria-selected": activeTab === 'people', role: 'tab', disabled: props.disablePeopleButton }, strings.peopleButtonLabel))))), /* Hidden icon to take the same space as the actual back button on the left. */
        React.createElement(react.DefaultButton, { styles: mobilePaneHiddenIconStyles, onRenderIcon: () => React.createElement(CallWithChatCompositeIcon, { iconName: "ChevronLeft" }) }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const SidePane = (props) => {
    var _a;
    const { sidePaneRenderer, overrideSidePane } = useSidePaneContext();
    const renderingHiddenOverrideContent = (overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.renderer.contentRenderer) && overrideSidePane.persistRenderingWhenClosed && !overrideSidePane.isActive;
    const renderingOnlyHiddenContent = renderingHiddenOverrideContent && !sidePaneRenderer;
    const widthConstrainedStyles = React.useMemo(() => sidePaneStyles(props.maxWidth, props.minWidth), [props.maxWidth, props.minWidth]);
    const paneStyles = renderingOnlyHiddenContent ? hiddenStyles : props.mobileView ? availableSpaceStyles : widthConstrainedStyles;
    const Header = (_a = ((overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.isActive) ? overrideSidePane.renderer.headerRenderer : sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.headerRenderer)) !== null && _a !== void 0 ? _a : EmptyElement;
    /**
     * Legacy code to support old behavior of showing chat and people tab headers on mobile side pane.
     * To be removed in breaking change.
     */
    const overrideSidePaneId = (overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.isActive) ? overrideSidePane.renderer.id : undefined;
    const { updateSidePaneRenderer } = props;
    const closePane = React.useCallback(() => {
        updateSidePaneRenderer(undefined);
    }, [updateSidePaneRenderer]);
    const LegacyHeader = React.useMemo(() => {
        var _a;
        return React.createElement(PeopleAndChatHeader, { onClose: overrideSidePaneId === 'chat' ? (_a = props.onChatButtonClicked) !== null && _a !== void 0 ? _a : noop : closePane, activeTab: (sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id) === 'people' ? 'people' : 'chat', 
            // legacy arguments to be removed in breaking change:
            disablePeopleButton: props.disablePeopleButton, disableChatButton: props.disableChatButton, onPeopleButtonClicked: (sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id) === 'people' ? noop : props.onPeopleButtonClicked, onChatButtonClicked: overrideSidePaneId === 'chat' ? noop : props.onChatButtonClicked });
    }, [overrideSidePaneId, props.onChatButtonClicked, props.onPeopleButtonClicked, props.disablePeopleButton, props.disableChatButton, sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id, closePane]);
    const HeaderToRender = props.mobileView && (overrideSidePaneId === 'chat' || (sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id) === 'people') ? LegacyHeader : Header();
    const ContentRenderer = (overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.isActive) ? undefined : sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.contentRenderer;
    const OverrideContentRenderer = (overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.isActive) || (overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.persistRenderingWhenClosed) ? overrideSidePane.renderer.contentRenderer : undefined;
    const theme = useTheme();
    if (!ContentRenderer && !OverrideContentRenderer) {
        return React.createElement(EmptyElement, null);
    }
    return React.createElement(react.Stack, { "aria-label": props.ariaLabel, "data-is-focusable": !!props.ariaLabel, role: props.ariaLabel ? 'navigation' : undefined, tabIndex: props.ariaLabel ? 0 : undefined, verticalFill: true, grow: true, styles: paneStyles, "aria-modal": true, "data-ui-id": "SidePane", tokens: props.mobileView || !props.showAddPeopleButton && (sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id) === 'people' ? {} : sidePaneTokens, className: paneHighContrastStyles(theme) },
        HeaderToRender,
        React.createElement(react.Stack.Item, { verticalFill: true, grow: true, styles: paneBodyContainer },
            React.createElement(react.Stack, { verticalFill: true, styles: scrollableContainer },
                ContentRenderer && React.createElement(react.Stack.Item, { verticalFill: true, styles: scrollableContainerContents },
                    React.createElement(react.Stack, { styles: containerContextStyles }, ContentRenderer === null || ContentRenderer === void 0 ? void 0 : ContentRenderer())),
                OverrideContentRenderer && React.createElement(react.Stack.Item, { verticalFill: true, styles: !(overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.isActive) && (overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.persistRenderingWhenClosed) ? hiddenStyles : scrollableContainerContents },
                    React.createElement(react.Stack, { styles: containerContextStyles }, OverrideContentRenderer === null || OverrideContentRenderer === void 0 ? void 0 : OverrideContentRenderer())))));
};
// eslint-disable-next-line @typescript-eslint/no-empty-function
const noop = () => { };
const EmptyElement = () => React.createElement(React.Fragment, null);

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const SidePaneHeader = (props) => {
    const theme = useTheme();
    const sidePaneCloseButtonStyles = React.useMemo(() => ({
        root: {
            minWidth: '1.5rem',
            padding: '0.5rem 0.25rem',
            marginRight: '0.25rem',
            backgroundColor: theme.semanticColors.bodyBackground,
            '@media (forced-colors: active)': {
                border: `0.1rem solid ${theme.palette.neutralSecondary}`,
                borderRadius: theme.effects.roundedCorner4
            }
        },
        icon: {
            color: theme.palette.neutralSecondary
        },
        iconHovered: {
            color: theme.palette.neutralSecondary
        },
        iconPressed: {
            color: theme.palette.neutralSecondary
        }
    }), [theme.palette.neutralSecondary, theme.semanticColors.bodyBackground, theme.effects.roundedCorner4]);
    const handleShiftTab = React.useCallback((event) => {
        var _a, _b;
        if (event.key === 'Tab' && event.shiftKey && !props.chatButtonPresent) {
            (_b = (_a = props.paneOpenerButton) === null || _a === void 0 ? void 0 : _a.current) === null || _b === void 0 ? void 0 : _b.focus();
            event.preventDefault();
        }
    }, [props.chatButtonPresent, props.paneOpenerButton]);
    if (props.mobileView) {
        return React.createElement(SidePaneMobileHeader, Object.assign({}, props));
    }
    return React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", styles: sidePaneHeaderContainerStyles, verticalAlign: "center" },
        React.createElement(react.Stack.Item, { role: "heading", styles: sidePaneHeaderStyles, "aria-label": props.headingText, "aria-level": 2 }, props.headingText),
        React.createElement(react.Stack.Item, null,
            React.createElement(react.CommandBarButton, { ariaLabel: props.dismissSidePaneButtonAriaLabel, styles: sidePaneCloseButtonStyles, iconProps: {
                    iconName: 'cancel'
                }, onClick: () => {
                    var _a, _b;
                    props.onClose();
                    (_b = (_a = props.paneOpenerButton) === null || _a === void 0 ? void 0 : _a.current) === null || _b === void 0 ? void 0 : _b.focus();
                }, onKeyDown: handleShiftTab, componentRef: props.dismissButtonComponentRef })));
};
const SidePaneMobileHeader = (props) => {
    const { headingText, dismissSidePaneButtonAriaLabel, dismissSidePaneButtonAriaDescription, onClose } = props;
    const theme = useTheme();
    const mobilePaneButtonStylesThemed = React.useMemo(() => {
        return react.concatStyleSets(mobilePaneButtonStyles, {
            root: {
                width: '100%'
            },
            label: {
                fontSize: theme.fonts.medium.fontSize,
                fontWeight: theme.fonts.medium.fontWeight
            }
        });
    }, [theme]);
    return React.createElement(react.Stack, { horizontal: true, grow: true, styles: mobilePaneControlBarStyle },
        React.createElement(react.DefaultButton, { ariaLabel: dismissSidePaneButtonAriaLabel, ariaDescription: dismissSidePaneButtonAriaDescription, onClick: onClose, styles: mobilePaneBackButtonStyles, onRenderIcon: () => React.createElement(CallWithChatCompositeIcon, { iconName: "ChevronLeft" }), autoFocus: true }),
        React.createElement(react.Stack.Item, { grow: true },
            React.createElement(react.DefaultButton, { checked: true, styles: mobilePaneButtonStylesThemed }, headingText)),
        React.createElement(react.DefaultButton, { styles: mobilePaneHiddenIconStyles, ariaHidden: true, onRenderIcon: () => React.createElement(CallWithChatCompositeIcon, { iconName: "ChevronLeft" }) }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ParticipantListWithHeading = (props) => {
    const { onFetchAvatarPersonaData, onFetchParticipantMenuItems, title, participantListProps, headingMoreButtonAriaLabel, onClickHeadingMoreButton, headingMoreButtonMenuProps, pinnedParticipants } = props;
    const subheadingUniqueId = reactHooks.useId();
    const theme = react.useTheme();
    const tooltipId = react.getId('text-tooltip');
    const subheadingStyleThemed = React.useMemo(() => ({
        root: {
            h2: {
                color: theme.palette.neutralSecondary,
                margin: props.isMobile ? '0.5rem 1rem' : '0.5rem',
                fontSize: theme.fonts.smallPlus.fontSize,
                fontWeight: 'normal'
            }
        }
    }), [theme.palette.neutralSecondary, theme.fonts.smallPlus.fontSize, props.isMobile]);
    return React.createElement(react.Stack, { className: participantListStack },
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center" },
            React.createElement(react.Stack.Item, { grow: true, styles: subheadingStyleThemed, "aria-label": title, id: subheadingUniqueId },
                React.createElement("h2", null, paneTitleTrampoline(title !== null && title !== void 0 ? title : ''))),
            (onClickHeadingMoreButton || (headingMoreButtonMenuProps === null || headingMoreButtonMenuProps === void 0 ? void 0 : headingMoreButtonMenuProps.items) && headingMoreButtonMenuProps.items.length > 0) && React.createElement(react.Stack.Item, null,
                React.createElement(react.DefaultButton, { "data-ui-id": "people-pane-header-more-button", ariaLabel: headingMoreButtonAriaLabel, styles: headingMoreButtonStyles(theme), iconProps: {
                        iconName: 'PeoplePaneMoreButton'
                    }, onClick: onClickHeadingMoreButton ? () => onClickHeadingMoreButton() : undefined, menuProps: props.onClickHeadingMoreButton ? undefined : props.headingMoreButtonMenuProps, onRenderMenuIcon: () => null }))),
        React.createElement(react.Stack, { className: participantListContainerStyle },
            React.createElement(ParticipantList$n, Object.assign({}, participantListProps, { pinnedParticipants: pinnedParticipants, styles: props.isMobile ? participantListMobileStyle : participantListStyle, onRenderAvatar: (userId, options) => React.createElement(React.Fragment, null,
                    React.createElement(AvatarPersona, Object.assign({ "data-ui-id": "chat-composite-participant-custom-avatar", userId: userId }, options, { hidePersonaDetails: !!(options === null || options === void 0 ? void 0 : options.text), dataProvider: onFetchAvatarPersonaData, allowActiveBorder: true })),
                    (options === null || options === void 0 ? void 0 : options.text) && React.createElement("div", { style: displayNameStyles$1 },
                        React.createElement(react.TooltipHost, { content: options === null || options === void 0 ? void 0 : options.text, id: tooltipId, overflowMode: react.TooltipOverflowMode.Parent },
                            React.createElement(react.Text, { nowrap: false, "aria-labelledby": tooltipId }, options === null || options === void 0 ? void 0 : options.text)))), onFetchParticipantMenuItems: onFetchParticipantMenuItems, showParticipantOverflowTooltip: !props.isMobile, participantAriaLabelledBy: subheadingUniqueId }))));
};
const paneTitleTrampoline = (paneTitle, totalParticipantCount) => {
    const participantCountString = {
        numberOfPeople: ' '
    };
    return _formatString(paneTitle, participantCountString);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
/**
 * @private
 */
const peoplePaneContainerStyle = {
    root: {
        position: 'relative',
        maxHeight: '100%',
        overflow: 'hidden'
    }
};
/**
 * @private
 */
const participantListContainerStyles = {
    root: {
        overflowY: 'scroll'
    }
};
/**
 * @private
 */
const copyLinkButtonContainerStyles = {
    root: {
        width: '100%',
        padding: '0.5rem 1rem'
    }
};
/**
 * @private
 */
const copyLinkButtonStackStyles = {
    root: {
        marginLeft: '0.5rem',
        marginRight: '0.5rem'
    }
};
/**
 * @private
 */
const copyLinkButtonStyles = {
    root: {
        height: '2.5rem',
        width: '100%'
    },
    textContainer: {
        display: 'contents'
    }
};
/**
 * @private
 */
const themedCopyLinkButtonStyles$1 = (mobileView, theme) => ({
    root: {
        minHeight: mobileView ? '3rem' : '2.5rem',
        borderRadius: mobileView ? theme.effects.roundedCorner6 : theme.effects.roundedCorner4
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Convert IContextualMenuItem to _DrawerMenuItemProps
 * @param contextualMenu - IContextualMenuItem to convert
 * @param onDrawerMenuItemClick - callback to call when converted DrawerMenuItem is clicked
 * @returns DrawerMenuItem
 * @private
 */
const convertContextualMenuItemToDrawerMenuItem = (contextualMenu, onDrawerMenuItemClick) => {
    return {
        itemKey: contextualMenu.key,
        onItemClick: () => {
            var _a;
            (_a = contextualMenu.onClick) === null || _a === void 0 ? void 0 : _a.call(contextualMenu);
            onDrawerMenuItemClick();
        },
        iconProps: contextualMenu.iconProps,
        text: contextualMenu.text,
        disabled: contextualMenu.disabled
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const themedCopyLinkButtonStyles = (theme, mobileView) => ({
    root: {
        minHeight: mobileView ? '3rem' : '2.5rem',
        borderRadius: mobileView ? theme.effects.roundedCorner6 : theme.effects.roundedCorner4,
        width: '100%'
    },
    textContainer: {
        display: 'contents'
    }
});
/**
 * @private
 */
const themedMenuStyle = (theme) => ({
    root: {
        borderRadius: theme.effects.roundedCorner6
    }
});
/**
 * @private
 */
const iconStyles = {
    // overwrite default line height which results in icons having different size
    lineHeight: '0',
    width: '1.125rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const themeddialpadModalStyle = (theme) => ({
    main: {
        borderRadius: theme.effects.roundedCorner6,
        padding: '1rem'
    }
});
/**
 * @private
 */
const themedDialpadStyle = (isMobile, theme) => ({
    root: {
        padding: 0,
        marginLeft: 0,
        marginRight: 0,
        maxWidth: '100%'
    },
    textField: {
        root: {
            borderBottom: `1px solid ${theme.palette.neutralTertiaryAlt}`
        },
        field: {
            backgroundColor: theme.palette.white,
            fontSize: theme.fonts.large.fontSize,
            padding: '0 0.5rem',
            textAlign: isMobile ? 'center' : 'left',
            ':active': {
                padding: '0 0.5rem'
            }
        }
    },
    deleteIcon: {
        root: {
            backgroundColor: theme.palette.white
        }
    },
    digit: {
        color: theme.palette.themeDarkAlt
    }
});
/**
 * @private
 */
const themedCallButtonStyle = (theme) => ({
    root: {
        fontWeight: theme.fonts.medium.fontWeight,
        fontSize: theme.fonts.medium.fontSize,
        width: '100%',
        height: '2.5rem',
        borderRadius: 3,
        padding: '0.625rem'
    },
    textContainer: {
        display: 'contents'
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const CallingDialpad = (props) => {
    const { strings, isMobile, showDialpad, onDismissDialpad, onAddParticipant, alternateCallerId } = props;
    const [textFieldInput, setTextFieldInput] = React.useState('');
    const theme = react.useTheme();
    const onDismissTriggered = () => {
        setTextFieldInput('');
        onDismissDialpad();
    };
    const onClickCall = () => {
        if (onAddParticipant) {
            /**
             * Format the phone number in dialpad textfield to make sure the phone number is in E.164 format.
             * We assume the input number always include countrycode
             */
            const phoneNumber = {
                phoneNumber: '+' + textFieldInput.replace(/\D/g, '').replaceAll(' ', '')
            };
            onAddParticipant(phoneNumber, {
                alternateCallerId: {
                    phoneNumber: alternateCallerId
                }
            });
            onDismissTriggered();
        }
    };
    const dialpadModalStyle = React.useMemo(() => themeddialpadModalStyle(theme), [theme]);
    const dialpadStyle = React.useMemo(() => themedDialpadStyle(isMobile, theme), [theme, isMobile]);
    const callButtonStyle = React.useMemo(() => themedCallButtonStyle(theme), [theme]);
    const dialpadComponent = () => {
        return React.createElement(React.Fragment, null,
            React.createElement(Dialpad, { styles: dialpadStyle, onChange: setTextFieldInput, longPressTrigger: isMobile ? 'touch' : 'mouseAndTouch' }),
            React.createElement(react.PrimaryButton, { text: strings.dialpadStartCallButtonLabel, onRenderIcon: () => React.createElement(CallWithChatCompositeIcon, { iconName: "DialpadStartCall" }), onClick: onClickCall, styles: callButtonStyle, disabled: textFieldInput === '' }));
    };
    if (isMobile) {
        return React.createElement(react.Stack, { "data-ui-id": "call-with-chat-composite-dialpad" }, showDialpad && React.createElement(react.Stack, { styles: drawerContainerStyles() },
            React.createElement(_DrawerSurface, { onLightDismiss: onDismissTriggered, disableMaxHeight: true },
                React.createElement(react.Stack, { style: {
                        padding: '1rem'
                    } }, dialpadComponent()))));
    }
    return React.createElement(React.Fragment, null, React.createElement(react.Modal, { titleAriaId: strings.dialpadModalAriaLabel, isOpen: showDialpad, onDismiss: onDismissTriggered, isBlocking: true, styles: dialpadModalStyle, "data-ui-id": "call-with-chat-composite-dialpad" },
        React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center" },
            React.createElement(react.Text, null, strings.dialpadModalTitle),
            React.createElement(react.IconButton, { iconProps: {
                    iconName: 'Cancel'
                }, ariaLabel: strings.dialpadCloseModalButtonAriaLabel, onClick: onDismissTriggered, style: {
                    color: theme.palette.black
                } })),
        React.createElement(react.Stack, { style: {
                overflow: 'hidden'
            } }, dialpadComponent())));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const CalloutWithIcon = (props) => {
    return React.createElement(react.Callout, { styles: calloutStyle, target: `#${props.targetId}`, directionalHint: react.DirectionalHint.topCenter, beakWidth: BEAK_WIDTH_PX, gapSpace: GAP_SPACE_PX, doNotLayer: props.doNotLayer },
        React.createElement(react.Stack, { horizontal: true, styles: calloutRootStackStyle, tokens: {
                childrenGap: '0.375rem'
            } },
            React.createElement(react.FontIcon, { iconName: "Checkmark", className: iconClass }),
            React.createElement(react.Text, null, props.text)));
};
const calloutStyle = {
    root: {
        padding: '0.375rem 0.75rem 0.375rem 0.5rem',
        borderRadius: '0.25rem'
    },
    beakCurtain: {
        borderRadius: '0.25rem'
    },
    container: {
        zIndex: 0
    }
};
const calloutRootStackStyle = {
    root: {
        display: 'flex',
        alignItems: 'center'
    }
};
const BEAK_WIDTH_PX = 8;
const GAP_SPACE_PX = 2;
const iconClass = react.mergeStyles({
    fontSize: 18
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const AddPeopleDropdown = (props) => {
    var _a, _b;
    const theme = react.useTheme();
    const { inviteLink, strings, mobileView, onAddParticipant, alternateCallerId, onCopyInviteLink, inviteLinkCopiedRecently } = props;
    const [showDialpad, setShowDialpad] = React.useState(false);
    const [announcerStrings, setAnnouncerStrings] = React.useState();
    const menuStyleThemed = React.useMemo(() => themedMenuStyle(theme), [theme]);
    const copyLinkButtonStylesThemed = React.useMemo(() => themedCopyLinkButtonStyles(theme, mobileView), [mobileView, theme]);
    const defaultMenuProps = React.useMemo(() => {
        const menuProps = {
            styles: menuStyleThemed,
            items: [],
            useTargetWidth: true,
            calloutProps: {
                preventDismissOnEvent: _preventDismissOnEvent
            },
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            onMenuOpened(contextualMenu) {
                setAnnouncerStrings(undefined);
            }
        };
        if (inviteLink) {
            menuProps.items.push({
                key: 'InviteLinkKey',
                text: strings.copyInviteLinkButtonLabel,
                itemProps: {
                    styles: copyLinkButtonStylesThemed
                },
                iconProps: {
                    iconName: 'Link',
                    style: iconStyles
                },
                onClick: () => {
                    setAnnouncerStrings(strings.copyInviteLinkActionedAriaLabel);
                    copy(inviteLink);
                    onCopyInviteLink === null || onCopyInviteLink === void 0 ? void 0 : onCopyInviteLink();
                }
            });
        }
        // only show the dialpad option when alternateCallerId is set
        if (alternateCallerId) {
            menuProps.items.push({
                key: 'DialpadKey',
                text: strings.openDialpadButtonLabel,
                itemProps: {
                    styles: copyLinkButtonStylesThemed
                },
                iconProps: {
                    iconName: 'PeoplePaneOpenDialpad',
                    style: iconStyles
                },
                onClick: () => setShowDialpad(true),
                'data-ui-id': 'call-dial-phone-number-button'
            });
        }
        return menuProps;
    }, [menuStyleThemed, inviteLink, alternateCallerId, strings.copyInviteLinkButtonLabel, strings.copyInviteLinkActionedAriaLabel, strings.openDialpadButtonLabel, copyLinkButtonStylesThemed, onCopyInviteLink]);
    const onDismissDialpad = () => {
        setShowDialpad(false);
    };
    const [addPeopleDrawerMenuItems, setAddPeopleDrawerMenuItems] = React.useState([]);
    const setDrawerMenuItemsForAddPeople = React.useMemo(() => {
        return () => {
            const drawerMenuItems = defaultMenuProps.items.map((contextualMenu) => convertContextualMenuItemToDrawerMenuItem(contextualMenu, () => setAddPeopleDrawerMenuItems([])));
            setAddPeopleDrawerMenuItems(drawerMenuItems);
        };
    }, [defaultMenuProps, setAddPeopleDrawerMenuItems]);
    const calloutButtonId = reactHooks.useId('callout-button');
    if (mobileView) {
        return React.createElement(react.Stack, null,
            React.createElement(Announcer$1, { ariaLive: 'assertive', announcementString: announcerStrings }),
            defaultMenuProps.items.length > 0 && React.createElement(react.Stack.Item, { styles: copyLinkButtonContainerStyles },
                React.createElement(react.PrimaryButton, { autoFocus: true, id: calloutButtonId, onClick: setDrawerMenuItemsForAddPeople, styles: copyLinkButtonStylesThemed, onRenderIcon: () => React.createElement(CallWithChatCompositeIcon, { iconName: "PeoplePaneAddPerson" }), text: strings.peoplePaneAddPeopleButtonLabel, "data-ui-id": "call-add-people-button" }),
                inviteLinkCopiedRecently && React.createElement(CalloutWithIcon, { targetId: calloutButtonId, text: (_a = strings.copyInviteLinkButtonActionedLabel) !== null && _a !== void 0 ? _a : '', doNotLayer: true })),
            addPeopleDrawerMenuItems.length > 0 && React.createElement(react.Stack, { styles: drawerContainerStyles(), "data-ui-id": "call-add-people-dropdown" },
                React.createElement(_DrawerMenu, { disableMaxHeight: true, onLightDismiss: () => setAddPeopleDrawerMenuItems([]), items: addPeopleDrawerMenuItems })),
            alternateCallerId && React.createElement(CallingDialpad, { isMobile: true, strings: strings, showDialpad: showDialpad, onDismissDialpad: onDismissDialpad, onAddParticipant: onAddParticipant, alternateCallerId: alternateCallerId }));
    }
    return React.createElement(React.Fragment, null, React.createElement(react.Stack, null,
        React.createElement(Announcer$1, { ariaLive: 'assertive', announcementString: announcerStrings }),
        alternateCallerId && React.createElement(CallingDialpad, { isMobile: false, strings: strings, showDialpad: showDialpad, onDismissDialpad: onDismissDialpad, onAddParticipant: onAddParticipant, alternateCallerId: alternateCallerId }),
        defaultMenuProps.items.length > 0 && React.createElement(react.Stack, { styles: copyLinkButtonStackStyles },
            React.createElement(react.DefaultButton, { id: calloutButtonId, onRenderIcon: () => React.createElement(CallWithChatCompositeIcon, { iconName: "PeoplePaneAddPerson" }), text: strings.peoplePaneAddPeopleButtonLabel, menuProps: defaultMenuProps, styles: copyLinkButtonStylesThemed, "data-ui-id": "call-add-people-button" }),
            inviteLinkCopiedRecently && React.createElement(CalloutWithIcon, { targetId: calloutButtonId, text: (_b = strings.copyInviteLinkButtonActionedLabel) !== null && _b !== void 0 ? _b : '' }))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const AddPeopleButton = (props) => {
    const { inviteLink, mobileView, strings, participantList } = props;
    const theme = react.useTheme();
    const [copyInviteLinkAnnouncerStrings, setCopyInviteLinkAnnouncerStrings] = React.useState('');
    React.useMemo(() => react.concatStyleSets(copyLinkButtonStyles, themedCopyLinkButtonStyles$1(mobileView, theme)), [mobileView, theme]);
    /**
     * sets the announcement string for when the link is copied.
     */
    React.useCallback(() => {
        setCopyInviteLinkAnnouncerStrings(strings.copyInviteLinkActionedAriaLabel);
        /**
         * Clears the announcer string after the user clicks the
         * copyInviteLink button allowing it to be re-announced.
         */
        setTimeout(() => {
            setCopyInviteLinkAnnouncerStrings('');
        }, 3000);
    }, [strings.copyInviteLinkActionedAriaLabel]);
    const dateInviteLinkCopied = React.useRef(undefined);
    const [inviteLinkCopiedRecently, setInviteLinkCopiedRecently] = React.useState(false);
    const onCopyInviteLink = React.useCallback(() => {
        setInviteLinkCopiedRecently(true);
        dateInviteLinkCopied.current = Date.now();
        setTimeout(() => {
            if (dateInviteLinkCopied.current && Date.now() - dateInviteLinkCopied.current >= 2000) {
                setInviteLinkCopiedRecently(false);
            }
        }, 2000);
    }, [setInviteLinkCopiedRecently, dateInviteLinkCopied]);
    reactHooks.useId('callout-button');
    if (mobileView) {
        return React.createElement(AddPeopleDropdown, { strings: Object.assign({}, strings), mobileView: mobileView, inviteLink: inviteLink, onAddParticipant: props.onAddParticipant, alternateCallerId: props.alternateCallerId, onCopyInviteLink: onCopyInviteLink, inviteLinkCopiedRecently: inviteLinkCopiedRecently });
    }
    else {
        return React.createElement(react.Stack, { tokens: peoplePaneContainerTokens, "data-ui-id": "people-pane-content", verticalFill: true },
            React.createElement(AddPeopleDropdown, { strings: strings, mobileView: mobileView, inviteLink: inviteLink, onAddParticipant: props.onAddParticipant, alternateCallerId: props.alternateCallerId, onCopyInviteLink: onCopyInviteLink, inviteLinkCopiedRecently: inviteLinkCopiedRecently }),
            React.createElement(react.Stack.Item, { grow: true, styles: {
                    root: {
                        overflowY: 'hidden'
                    }
                } }, participantList));
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$i = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const PeoplePaneContent = (props) => {
    const { inviteLink, onFetchParticipantMenuItems, setDrawerMenuItems, setParticipantActioned, participantListHeadingMoreButtonProps } = props;
    const adapter = useAdapter();
    const localeStrings = useLocale();
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    const getStrings = () => {
        return localeStrings.strings.call;
    };
    const strings = getStrings();
    const removeParticipantFromCall = React.useCallback((participantId) => __awaiter$i(void 0, void 0, void 0, function* () {
        yield adapter.removeParticipant(participantId);
    }), [adapter]);
    const addParticipantToCall = React.useCallback((participant, options) => __awaiter$i(void 0, void 0, void 0, function* () {
        yield adapter.addParticipant(participant, options);
    }), [adapter]);
    const participantListDefaultProps = usePropsFor$1(ParticipantList$n);
    const removeButtonAllowed = canRemoveParticipants(props.role);
    const setDrawerMenuItemsForParticipant = React.useMemo(() => {
        return (participant) => {
            if (participant) {
                setParticipantActioned === null || setParticipantActioned === void 0 ? void 0 : setParticipantActioned(participant.userId);
                let contextualMenuItems = createDefaultContextualMenuItems(participant, strings, removeButtonAllowed && participant.isRemovable ? participantListDefaultProps.onRemoveParticipant : undefined, participantListDefaultProps.myUserId);
                if (onFetchParticipantMenuItems) {
                    contextualMenuItems = onFetchParticipantMenuItems(participant.userId, participantListDefaultProps.myUserId, contextualMenuItems);
                }
                const drawerMenuItems = contextualMenuItems.map((contextualMenu) => convertContextualMenuItemToDrawerMenuItem(contextualMenu, () => setDrawerMenuItems([])));
                setDrawerMenuItems(drawerMenuItems);
            }
        };
    }, [strings, participantListDefaultProps.onRemoveParticipant, participantListDefaultProps.myUserId, removeButtonAllowed, onFetchParticipantMenuItems, setDrawerMenuItems, setParticipantActioned]);
    const setDrawerMenuItemsForParticipantListHeadingMoreButton = React.useMemo(() => {
        const drawerMenuItems = participantListHeadingMoreButtonProps === null || participantListHeadingMoreButtonProps === void 0 ? void 0 : participantListHeadingMoreButtonProps.items.map((contextualMenu) => convertContextualMenuItemToDrawerMenuItem(contextualMenu, () => setDrawerMenuItems([])));
        return drawerMenuItems && drawerMenuItems.length > 0 ? () => setDrawerMenuItems(drawerMenuItems) : undefined;
    }, [participantListHeadingMoreButtonProps === null || participantListHeadingMoreButtonProps === void 0 ? void 0 : participantListHeadingMoreButtonProps.items, setDrawerMenuItems]);
    const participantListProps = React.useMemo(() => {
        const onRemoveAParticipant = (participantId) => __awaiter$i(void 0, void 0, void 0, function* () { return removeParticipantFromCall(participantId); });
        return Object.assign(Object.assign({}, participantListDefaultProps), { 
            // Passing undefined callback for mobile to avoid context menus for participants in ParticipantList are clicked
            onRemoveParticipant: props.mobileView ? undefined : onRemoveAParticipant, 
            // We want the drawer menu items to appear when participants in ParticipantList are clicked
            onParticipantClick: props.mobileView ? setDrawerMenuItemsForParticipant : undefined });
    }, [participantListDefaultProps, props.mobileView, setDrawerMenuItemsForParticipant, removeParticipantFromCall]);
    const participantList = React.createElement(ParticipantListWithHeading, { isMobile: props.mobileView, participantListProps: participantListProps, onFetchAvatarPersonaData: props.onFetchAvatarPersonaData, onFetchParticipantMenuItems: props.mobileView ? undefined : onFetchParticipantMenuItems, title: strings.peoplePaneSubTitle, headingMoreButtonAriaLabel: localeStrings.strings.call.peoplePaneMoreButtonAriaLabel, onClickHeadingMoreButton: props.mobileView ? setDrawerMenuItemsForParticipantListHeadingMoreButton : undefined, headingMoreButtonMenuProps: props.participantListHeadingMoreButtonProps, pinnedParticipants: props.pinnedParticipants });
    if (props.mobileView) {
        return React.createElement(react.Stack, { verticalFill: true, styles: peoplePaneContainerStyle, tokens: peoplePaneContainerTokens, "data-ui-id": "people-pane-content" },
            React.createElement(react.Stack.Item, { grow: true, styles: participantListContainerStyles }, participantList),
            React.createElement(AddPeopleButton, { inviteLink: inviteLink, mobileView: props.mobileView, participantList: participantList, strings: strings, onAddParticipant: addParticipantToCall, alternateCallerId: props.alternateCallerId }));
    }
    return React.createElement(AddPeopleButton, { inviteLink: inviteLink, mobileView: props.mobileView, participantList: participantList, strings: strings, onAddParticipant: addParticipantToCall, alternateCallerId: props.alternateCallerId });
};
/**
 * Create default contextual menu items for particant
 * @param participant - participant to create contextual menu items for
 * @param strings - localized strings for menu item text
 * @param onRemoveParticipant - callback to remove participant
 * @param localParticipantUserId - Local participant user id
 * @returns - IContextualMenuItem[]
 */
const createDefaultContextualMenuItems = (participant, strings, onRemoveParticipant, localParticipantUserId) => {
    const menuItems = [];
    if (onRemoveParticipant && (participant === null || participant === void 0 ? void 0 : participant.userId) !== localParticipantUserId) {
        menuItems.push({
            key: 'remove',
            text: strings.removeMenuLabel,
            role: 'menuitem',
            onClick: () => {
                if (participant === null || participant === void 0 ? void 0 : participant.userId) {
                    onRemoveParticipant === null || onRemoveParticipant === void 0 ? void 0 : onRemoveParticipant(participant === null || participant === void 0 ? void 0 : participant.userId);
                }
            },
            iconProps: {
                iconName: 'ContextMenuRemoveParticipant',
                styles: {
                    root: {
                        lineHeight: 0
                    }
                }
            },
            'data-ui-id': 'participant-list-remove-participant-button'
        });
    }
    return menuItems;
};
const canRemoveParticipants = (role) => {
    // TODO: We should be using the removeParticipant capability here but there is an SDK bug for Rooms where a
    // Presenter's removeParticipant capability is {isPresent: false, reason: 'CapabilityNotApplicableForTheCallType'}.
    // But a Presenter in Rooms should be able to remove participants according to the following documentation
    // https://learn.microsoft.com/en-us/azure/communication-services/concepts/rooms/room-concept#predefined-participant-roles-and-permissions
    const canRemove = role === 'Presenter' || role === 'Unknown' || role === undefined;
    return canRemove;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const PEOPLE_SIDE_PANE_ID = 'people';
/** @private */
const usePeoplePane = (props) => {
    const { updateSidePaneRenderer, inviteLink, onFetchAvatarPersonaData, onFetchParticipantMenuItems, setDrawerMenuItems, mobileView, peopleButtonRef, setParticipantActioned, spotlightedParticipantUserIds, onStartLocalSpotlight, onStopLocalSpotlight, onStartRemoteSpotlight, onStopRemoteSpotlight, onStopAllSpotlight, maxParticipantsToSpotlight, onMuteParticipant, pinnedParticipants, onPinParticipant, onUnpinParticipant, disablePinMenuItem, onMuteAllRemoteParticipants, onForbidAudio, onPermitAudio, onForbidOthersAudio, onPermitOthersAudio, onForbidVideo, onPermitVideo, onForbidOthersVideo, onPermitOthersVideo, meetingMediaAccess, sidePaneDismissButtonRef, chatButtonPresent } = props;
    const closePane = React.useCallback(() => {
        var _a;
        updateSidePaneRenderer(undefined);
        (_a = peopleButtonRef === null || peopleButtonRef === void 0 ? void 0 : peopleButtonRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, [peopleButtonRef, updateSidePaneRenderer]);
    const localeStrings = useLocale().strings.call;
    const remoteParticipants = useSelector$1(getRemoteParticipants);
    const [showMuteAllPrompt, setShowMuteAllPrompt] = React.useState(false);
    const muteAllPromptLabels = React.useMemo(() => ({
        confirmButtonLabel: localeStrings.muteAllConfirmButtonLabel,
        heading: localeStrings.muteAllDialogTitle,
        text: localeStrings.muteAllDialogContent,
        cancelButtonLabel: localeStrings.muteAllCancelButtonLabel
    }), [localeStrings.muteAllConfirmButtonLabel, localeStrings.muteAllDialogTitle, localeStrings.muteAllDialogContent, localeStrings.muteAllCancelButtonLabel]);
    const onMuteAllPromptConfirm = React.useCallback(() => {
        onMuteAllRemoteParticipants && onMuteAllRemoteParticipants();
        setShowMuteAllPrompt(false);
    }, [onMuteAllRemoteParticipants, setShowMuteAllPrompt]);
    const [showForbidOthersAudioPrompt, setShowForbidOthersAudioPrompt] = React.useState(false);
    const [showPermitOthersAudioPrompt, setShowPermitOthersAudioPrompt] = React.useState(false);
    const [showForbidOthersVideoPrompt, setShowForbidOthersVideoPrompt] = React.useState(false);
    const [showPermitOthersVideoPrompt, setShowPermitOthersVideoPrompt] = React.useState(false);
    const onForbidAllAttendeesPromptConfirm = React.useCallback(() => {
        onForbidOthersAudio && onForbidOthersAudio();
        setShowForbidOthersAudioPrompt(false);
    }, [onForbidOthersAudio, setShowForbidOthersAudioPrompt]);
    const onPermitAllAttendeesPromptConfirm = React.useCallback(() => {
        onPermitOthersAudio && onPermitOthersAudio();
        setShowPermitOthersAudioPrompt(false);
    }, [onPermitOthersAudio, setShowPermitOthersAudioPrompt]);
    const onForbidOthersVideoPromptConfirm = React.useCallback(() => {
        onForbidOthersVideo && onForbidOthersVideo();
        setShowForbidOthersVideoPrompt(false);
    }, [onForbidOthersVideo, setShowForbidOthersVideoPrompt]);
    const onPermitOthersVideoPromptConfirm = React.useCallback(() => {
        onPermitOthersVideo && onPermitOthersVideo();
        setShowPermitOthersVideoPrompt(false);
    }, [onPermitOthersVideo, setShowPermitOthersVideoPrompt]);
    const sidePaneHeaderMenuProps = React.useMemo(() => {
        const menuItems = [];
        if (onMuteAllRemoteParticipants && remoteParticipants) {
            let isAllMuted = true;
            if (remoteParticipants) {
                for (const participant of Object.values(remoteParticipants)) {
                    if (!participant.isMuted) {
                        isAllMuted = false;
                        break;
                    }
                }
            }
            menuItems.push({
                ['data-ui-id']: 'people-pane-mute-all-remote-participants',
                key: 'muteAllRemoteParticipants',
                text: localeStrings.muteAllMenuLabel,
                iconProps: {
                    iconName: 'ContextualMenuMicMutedIcon',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => {
                    setShowMuteAllPrompt(true);
                },
                ariaLabel: localeStrings.muteAllMenuLabel,
                disabled: isAllMuted
            });
        }
        let hasAttendee = false;
        if (remoteParticipants) {
            for (const participant of Object.values(remoteParticipants)) {
                if (participant.role && participant.role === 'Attendee') {
                    hasAttendee = true;
                    break;
                }
            }
        }
        const isMeetingAudioPermitted = meetingMediaAccess ? meetingMediaAccess.isAudioPermitted : true;
        const isMeetingVideoPermitted = meetingMediaAccess ? meetingMediaAccess.isVideoPermitted : true;
        if (onForbidOthersAudio && remoteParticipants) {
            hasAttendee && isMeetingAudioPermitted && menuItems.push({
                ['data-ui-id']: 'people-pane-forbid-all-attendees-audio',
                key: 'forbidOthersAudio',
                text: localeStrings.forbidOthersAudioMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonMicProhibited',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => {
                    setShowForbidOthersAudioPrompt(true);
                },
                ariaLabel: localeStrings.forbidOthersAudioMenuLabel,
                disabled: !hasAttendee
            });
        }
        if (onPermitOthersAudio && remoteParticipants) {
            hasAttendee && !isMeetingAudioPermitted && menuItems.push({
                ['data-ui-id']: 'people-pane-permit-all-attendees-audio',
                key: 'permitOthersAudio',
                text: localeStrings.permitOthersAudioMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonMicOn',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => {
                    setShowPermitOthersAudioPrompt(true);
                },
                ariaLabel: localeStrings.permitOthersAudioMenuLabel,
                disabled: !hasAttendee
            });
        }
        if (onForbidOthersVideo && remoteParticipants) {
            hasAttendee && isMeetingVideoPermitted && menuItems.push({
                ['data-ui-id']: 'people-pane-forbid-all-attendees-video',
                key: 'forbidOthersVideo',
                text: localeStrings.forbidOthersVideoMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonCameraProhibitedSmall',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => {
                    setShowForbidOthersVideoPrompt(true);
                },
                ariaLabel: localeStrings.forbidOthersVideoMenuLabel,
                disabled: !hasAttendee
            });
        }
        if (onPermitOthersVideo && remoteParticipants) {
            hasAttendee && !isMeetingVideoPermitted && menuItems.push({
                ['data-ui-id']: 'people-pane-permit-all-attendees-video',
                key: 'permitOthersVideo',
                text: localeStrings.permitOthersVideoMenuLabel,
                iconProps: {
                    iconName: 'ControlButtonCameraOn',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => {
                    setShowPermitOthersVideoPrompt(true);
                },
                ariaLabel: localeStrings.permitOthersVideoMenuLabel,
                disabled: !hasAttendee
            });
        }
        if (onStopAllSpotlight && spotlightedParticipantUserIds && spotlightedParticipantUserIds.length > 0) {
            menuItems.push({
                key: 'stopAllSpotlightKey',
                text: localeStrings.stopAllSpotlightMenuLabel,
                iconProps: {
                    iconName: 'StopAllSpotlightMenuButton',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => {
                    onStopAllSpotlight();
                },
                ariaLabel: localeStrings.stopAllSpotlightMenuLabel
            });
        }
        return {
            items: menuItems
        };
    }, [onMuteAllRemoteParticipants, remoteParticipants, meetingMediaAccess, onForbidOthersAudio, onPermitOthersAudio, onForbidOthersVideo, onPermitOthersVideo, onStopAllSpotlight, spotlightedParticipantUserIds, localeStrings.muteAllMenuLabel, localeStrings.forbidOthersAudioMenuLabel, localeStrings.permitOthersAudioMenuLabel, localeStrings.forbidOthersVideoMenuLabel, localeStrings.permitOthersVideoMenuLabel, localeStrings.stopAllSpotlightMenuLabel]);
    const onRenderHeader = React.useCallback(() => React.createElement(SidePaneHeader, { onClose: closePane, paneOpenerButton: peopleButtonRef, headingText: localeStrings.peoplePaneTitle, dismissSidePaneButtonAriaLabel: localeStrings.dismissSidePaneButtonLabel, mobileView: mobileView !== null && mobileView !== void 0 ? mobileView : false, dismissButtonComponentRef: sidePaneDismissButtonRef, chatButtonPresent: chatButtonPresent }), [closePane, peopleButtonRef, localeStrings.peoplePaneTitle, localeStrings.dismissSidePaneButtonLabel, mobileView, sidePaneDismissButtonRef, chatButtonPresent]);
    const onFetchParticipantMenuItemsForCallComposite = React.useCallback((participantId, myUserId, defaultMenuItems) => {
        var _a, _b;
        let isPinned = pinnedParticipants === null || pinnedParticipants === void 0 ? void 0 : pinnedParticipants.includes(participantId);
        const _defaultMenuItems = [];
        const isSpotlighted = spotlightedParticipantUserIds === null || spotlightedParticipantUserIds === void 0 ? void 0 : spotlightedParticipantUserIds.includes(participantId);
        const isMe = myUserId === participantId;
        isPinned = isSpotlighted ? false : isPinned;
        if (onMuteParticipant && !isMe && remoteParticipants && remoteParticipants[participantId]) {
            const participant = remoteParticipants[participantId];
            const isMuted = !!(participant === null || participant === void 0 ? void 0 : participant.isMuted);
            _defaultMenuItems.push({
                key: 'mute',
                text: 'Mute',
                role: 'menuitem',
                iconProps: {
                    iconName: 'ContextualMenuMicMutedIcon',
                    styles: {
                        root: {
                            lineHeight: 0
                        }
                    }
                },
                onClick: () => {
                    onMuteParticipant(participantId);
                },
                'data-ui-id': 'participant-item-mute-participant',
                ariaLabel: 'Mute',
                disabled: isMuted
            });
        }
        const remoteParticipant = remoteParticipants === null || remoteParticipants === void 0 ? void 0 : remoteParticipants[participantId];
        if ((remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.mediaAccess) && !remoteParticipant.mediaAccess.isAudioPermitted && (remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.role) === 'Attendee' && onPermitAudio) {
            _defaultMenuItems.push({
                key: 'permit-audio',
                text: localeStrings.permitAudioMenuLabel,
                role: 'menuitem',
                iconProps: {
                    iconName: 'ControlButtonMicOn',
                    styles: {
                        root: {
                            lineHeight: '1rem',
                            textAlign: 'center'
                        }
                    }
                },
                onClick: () => {
                    onPermitAudio([participantId]);
                },
                'data-ui-id': 'participant-item-permit-microphone-button',
                ariaLabel: localeStrings.permitAudioMenuLabel
            });
        }
        if (((_a = remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.mediaAccess) === null || _a === void 0 ? void 0 : _a.isAudioPermitted) && (remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.role) === 'Attendee' && onForbidAudio) {
            _defaultMenuItems.push({
                key: 'forbid-audio',
                text: localeStrings.forbidAudioMenuLabel,
                role: 'menuitem',
                iconProps: {
                    iconName: 'ControlButtonMicProhibited',
                    styles: {
                        root: {
                            lineHeight: '1rem',
                            textAlign: 'center'
                        }
                    }
                },
                onClick: () => {
                    onForbidAudio([participantId]);
                },
                'data-ui-id': 'participant-item-forbid-microphone-button',
                ariaLabel: localeStrings.forbidAudioMenuLabel
            });
        }
        if ((remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.mediaAccess) && !remoteParticipant.mediaAccess.isVideoPermitted && (remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.role) === 'Attendee' && onPermitVideo) {
            _defaultMenuItems.push({
                key: 'permit-video',
                text: localeStrings.permitVideoMenuLabel,
                role: 'menuitem',
                iconProps: {
                    iconName: 'ControlButtonCameraOn',
                    styles: {
                        root: {
                            lineHeight: '1rem',
                            textAlign: 'center'
                        }
                    }
                },
                onClick: () => {
                    onPermitVideo([participantId]);
                },
                'data-ui-id': 'participant-item-permit-camera-button',
                ariaLabel: localeStrings.permitVideoMenuLabel
            });
        }
        if (((_b = remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.mediaAccess) === null || _b === void 0 ? void 0 : _b.isVideoPermitted) && (remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.role) === 'Attendee' && onForbidVideo) {
            _defaultMenuItems.push({
                key: 'forbid-video',
                text: localeStrings.forbidVideoMenuLabel,
                role: 'menuitem',
                iconProps: {
                    iconName: 'ControlButtonCameraProhibitedSmall',
                    styles: {
                        root: {
                            lineHeight: '1rem',
                            textAlign: 'center'
                        }
                    }
                },
                onClick: () => {
                    onForbidVideo([participantId]);
                },
                'data-ui-id': 'participant-item-forbid-camera-button',
                ariaLabel: localeStrings.forbidVideoMenuLabel
            });
        }
        if (isSpotlighted) {
            const stopSpotlightMenuText = isMe ? localeStrings.stopSpotlightOnSelfMenuLabel : localeStrings.stopSpotlightMenuLabel;
            const onStopSpotlight = isMe ? onStopLocalSpotlight : onStopRemoteSpotlight ? () => {
                onStopRemoteSpotlight([participantId]);
            } : undefined;
            if (onStopSpotlight && stopSpotlightMenuText) {
                _defaultMenuItems.push({
                    key: 'stop-spotlight',
                    text: stopSpotlightMenuText,
                    role: 'menuitem',
                    onClick: onStopSpotlight,
                    iconProps: {
                        iconName: 'StopSpotlightContextualMenuItem',
                        styles: {
                            root: {
                                lineHeight: 0
                            }
                        }
                    },
                    ariaLabel: stopSpotlightMenuText
                });
            }
        }
        else {
            const startSpotlightMenuText = spotlightedParticipantUserIds && spotlightedParticipantUserIds.length > 0 ? localeStrings.addSpotlightMenuLabel : localeStrings.startSpotlightMenuLabel;
            const maxSpotlightedParticipantsReached = maxParticipantsToSpotlight ? spotlightedParticipantUserIds ? spotlightedParticipantUserIds.length >= maxParticipantsToSpotlight : false : false;
            const onStartSpotlight = isMe ? onStartLocalSpotlight : onStartRemoteSpotlight ? () => {
                onStartRemoteSpotlight([participantId]);
            } : undefined;
            if (onStartSpotlight && startSpotlightMenuText) {
                _defaultMenuItems.push({
                    key: 'start-spotlight',
                    text: startSpotlightMenuText,
                    role: 'menuitem',
                    onClick: onStartSpotlight,
                    iconProps: {
                        iconName: 'StartSpotlightContextualMenuItem',
                        styles: {
                            root: {
                                lineHeight: 0
                            }
                        }
                    },
                    ariaLabel: startSpotlightMenuText,
                    disabled: maxSpotlightedParticipantsReached,
                    title: maxSpotlightedParticipantsReached ? localeStrings.spotlightLimitReachedMenuTitle : undefined
                });
            }
        }
        if (!isMe && isPinned !== undefined) {
            if (isPinned && onUnpinParticipant && (localeStrings === null || localeStrings === void 0 ? void 0 : localeStrings.unpinParticipantMenuLabel)) {
                _defaultMenuItems.push({
                    key: 'unpin',
                    text: localeStrings === null || localeStrings === void 0 ? void 0 : localeStrings.unpinParticipantMenuLabel,
                    role: 'menuitem',
                    iconProps: {
                        iconName: 'UnpinParticipant',
                        styles: {
                            root: {
                                lineHeight: '1rem',
                                textAlign: 'center'
                            }
                        }
                    },
                    onClick: () => {
                        onUnpinParticipant === null || onUnpinParticipant === void 0 ? void 0 : onUnpinParticipant(participantId);
                    },
                    'data-ui-id': 'participant-item-unpin-participant-button',
                    ariaLabel: localeStrings.unpinParticipantMenuItemAriaLabel
                });
            }
            if (!isPinned && onPinParticipant && (localeStrings === null || localeStrings === void 0 ? void 0 : localeStrings.pinParticipantMenuLabel)) {
                _defaultMenuItems.push({
                    key: 'pin',
                    text: disablePinMenuItem ? localeStrings.pinParticipantLimitReachedMenuLabel : localeStrings.pinParticipantMenuLabel,
                    role: 'menuitem',
                    iconProps: {
                        iconName: 'PinParticipant',
                        styles: {
                            root: {
                                lineHeight: '1rem',
                                textAlign: 'center'
                            }
                        }
                    },
                    onClick: () => {
                        onPinParticipant(participantId);
                    },
                    'data-ui-id': 'participant-item-pin-participant-button',
                    disabled: disablePinMenuItem || isSpotlighted,
                    ariaLabel: localeStrings.pinParticipantMenuItemAriaLabel
                });
            }
        }
        if (defaultMenuItems) {
            _defaultMenuItems.push(...defaultMenuItems);
        }
        return onFetchParticipantMenuItems ? onFetchParticipantMenuItems(participantId, myUserId, _defaultMenuItems) : _defaultMenuItems;
    }, [pinnedParticipants, spotlightedParticipantUserIds, onMuteParticipant, remoteParticipants, onPermitAudio, onForbidAudio, onPermitVideo, onForbidVideo, onFetchParticipantMenuItems, localeStrings.permitAudioMenuLabel, localeStrings.forbidAudioMenuLabel, localeStrings.permitVideoMenuLabel, localeStrings.forbidVideoMenuLabel, localeStrings.stopSpotlightOnSelfMenuLabel, localeStrings.stopSpotlightMenuLabel, localeStrings.addSpotlightMenuLabel, localeStrings.startSpotlightMenuLabel, localeStrings.spotlightLimitReachedMenuTitle, localeStrings === null || localeStrings === void 0 ? void 0 : localeStrings.unpinParticipantMenuLabel, localeStrings.pinParticipantMenuLabel, localeStrings.unpinParticipantMenuItemAriaLabel, localeStrings.pinParticipantLimitReachedMenuLabel, localeStrings.pinParticipantMenuItemAriaLabel, onStopLocalSpotlight, onStopRemoteSpotlight, maxParticipantsToSpotlight, onStartLocalSpotlight, onStartRemoteSpotlight, onUnpinParticipant, onPinParticipant, disablePinMenuItem]);
    const role = useSelector$1(getRole);
    const alternateCallerId = useSelector$1(getAlternateCallerId);
    const onRenderContent = React.useCallback(() => {
        return React.createElement(React.Fragment, null,
            React.createElement(Prompt, Object.assign({}, muteAllPromptLabels, { styles: {
                    main: {
                        minWidth: '22.5rem',
                        padding: '1.5rem'
                    }
                }, onConfirm: () => onMuteAllPromptConfirm(), isOpen: showMuteAllPrompt, onCancel: () => setShowMuteAllPrompt(false) })),
            React.createElement(Prompt, { heading: localeStrings.forbidOthersAudioDialogTitle, text: localeStrings.forbidOthersAudioDialogContent, confirmButtonLabel: localeStrings.forbidOthersAudioConfirmButtonLabel, cancelButtonLabel: localeStrings.forbidOthersAudioCancelButtonLabel, styles: {
                    main: {
                        minWidth: '22.5rem',
                        padding: '1.5rem'
                    }
                }, onConfirm: () => onForbidAllAttendeesPromptConfirm(), isOpen: showForbidOthersAudioPrompt, onCancel: () => setShowForbidOthersAudioPrompt(false) }),
            React.createElement(Prompt, { heading: localeStrings.permitOthersAudioDialogTitle, text: localeStrings.permitOthersAudioDialogContent, confirmButtonLabel: localeStrings.permitOthersAudioConfirmButtonLabel, cancelButtonLabel: localeStrings.permitOthersAudioCancelButtonLabel, styles: {
                    main: {
                        minWidth: '22.5rem',
                        padding: '1.5rem'
                    }
                }, onConfirm: () => onPermitAllAttendeesPromptConfirm(), isOpen: showPermitOthersAudioPrompt, onCancel: () => setShowForbidOthersAudioPrompt(false) }),
            React.createElement(Prompt, { heading: localeStrings.forbidOthersVideoDialogTitle, text: localeStrings.forbidOthersVideoDialogContent, confirmButtonLabel: localeStrings.forbidOthersVideoConfirmButtonLabel, cancelButtonLabel: localeStrings.forbidOthersVideoCancelButtonLabel, styles: {
                    main: {
                        minWidth: '22.5rem',
                        padding: '1.5rem'
                    }
                }, onConfirm: () => onForbidOthersVideoPromptConfirm(), isOpen: showForbidOthersVideoPrompt, onCancel: () => setShowForbidOthersVideoPrompt(false) }),
            React.createElement(Prompt, { heading: localeStrings.permitOthersVideoDialogTitle, text: localeStrings.permitOthersVideoDialogContent, confirmButtonLabel: localeStrings.permitOthersVideoConfirmButtonLabel, cancelButtonLabel: localeStrings.permitOthersVideoCancelButtonLabel, styles: {
                    main: {
                        minWidth: '22.5rem',
                        padding: '1.5rem'
                    }
                }, onConfirm: () => onPermitOthersVideoPromptConfirm(), isOpen: showPermitOthersVideoPrompt, onCancel: () => setShowForbidOthersVideoPrompt(false) }),
            React.createElement(PeoplePaneContent, { inviteLink: inviteLink, onFetchAvatarPersonaData: onFetchAvatarPersonaData, onFetchParticipantMenuItems: onFetchParticipantMenuItemsForCallComposite, setDrawerMenuItems: setDrawerMenuItems, mobileView: mobileView, setParticipantActioned: setParticipantActioned, participantListHeadingMoreButtonProps: sidePaneHeaderMenuProps, pinnedParticipants: pinnedParticipants, role: role, alternateCallerId: alternateCallerId }));
    }, [muteAllPromptLabels, showMuteAllPrompt, localeStrings.forbidOthersAudioDialogTitle, localeStrings.forbidOthersAudioDialogContent, localeStrings.forbidOthersAudioConfirmButtonLabel, localeStrings.forbidOthersAudioCancelButtonLabel, localeStrings.permitOthersAudioDialogTitle, localeStrings.permitOthersAudioDialogContent, localeStrings.permitOthersAudioConfirmButtonLabel, localeStrings.permitOthersAudioCancelButtonLabel, localeStrings.forbidOthersVideoDialogTitle, localeStrings.forbidOthersVideoDialogContent, localeStrings.forbidOthersVideoConfirmButtonLabel, localeStrings.forbidOthersVideoCancelButtonLabel, localeStrings.permitOthersVideoDialogTitle, localeStrings.permitOthersVideoDialogContent, localeStrings.permitOthersVideoConfirmButtonLabel, localeStrings.permitOthersVideoCancelButtonLabel, showForbidOthersAudioPrompt, showPermitOthersAudioPrompt, showForbidOthersVideoPrompt, showPermitOthersVideoPrompt, inviteLink, onFetchAvatarPersonaData, onFetchParticipantMenuItemsForCallComposite, setDrawerMenuItems, mobileView, setParticipantActioned, sidePaneHeaderMenuProps, pinnedParticipants, role, alternateCallerId, onMuteAllPromptConfirm, onForbidAllAttendeesPromptConfirm, onPermitAllAttendeesPromptConfirm, onForbidOthersVideoPromptConfirm, onPermitOthersVideoPromptConfirm]);
    const sidePaneRenderer = React.useMemo(() => ({
        headerRenderer: onRenderHeader,
        contentRenderer: onRenderContent,
        id: PEOPLE_SIDE_PANE_ID
    }), [onRenderContent, onRenderHeader]);
    const openPane = React.useCallback(() => {
        updateSidePaneRenderer(sidePaneRenderer);
    }, [sidePaneRenderer, updateSidePaneRenderer]);
    const isOpen = useIsParticularSidePaneOpen(PEOPLE_SIDE_PANE_ID);
    // Update pane renderer if it is open and the openPane dep changes
    React.useEffect(() => {
        if (isOpen) {
            openPane();
        }
    }, [isOpen, openPane]);
    return {
        openPeoplePane: openPane,
        closePeoplePane: closePane,
        isPeoplePaneOpen: isOpen
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const phoneInfoContainerTokens = {
    childrenGap: '0.5rem'
};
/**
 * @private
 */
const phoneInfoContainerStyle = {
    root: {
        position: 'relative',
        maxHeight: '100%',
        overflow: 'hidden'
    }
};
/**
 * @private
 */
const titleClassName = react.mergeStyles({
    fontWeight: 600,
    fontSize: _pxToRem(20),
    lineHeight: _pxToRem(28)
});
/**
 * @private
 */
const titleContainerClassName = react.mergeStyles({
    paddingBottom: _pxToRem(20)
});
/**
 * @private
 */
const phoneInfoLabelStyle = react.mergeStyles({
    fontSize: _pxToRem(14),
    lineHeight: _pxToRem(40)
});
/**
 * @private
 */
const phoneInfoTextStyle = react.mergeStyles({
    fontSize: _pxToRem(14),
    lineHeight: _pxToRem(40),
    fontWeight: react.FontWeights.semibold
});
/**
 * @private
 */
const phoneInfoIcon = (theme) => {
    return react.mergeStyles({
        background: `${theme.palette.themeLighter}`,
        height: _pxToRem(36),
        width: _pxToRem(36),
        marginRight: _pxToRem(12),
        borderRadius: _pxToRem(18)
    });
};
/**
 * @private
 */
const phoneInfoInstructionLine = react.mergeStyles({
    padding: '0.5rem 1rem'
});
/**
 * @private
 */
const phoneInfoStep = react.mergeStyles({
    minWidth: _pxToRem(150),
    textAlign: 'right'
});
/**
 * @private
 */
const phoneInfoIconStyle = (theme) => {
    return react.mergeStyles({
        padding: _pxToRem(8),
        color: `${theme.palette.themePrimary}`,
        zIndex: 2
    });
};
/**
 * @private
 */
const infoConnectionLinkStyle = (theme) => {
    return react.mergeStyles({
        background: `${theme.palette.themeLighter}`,
        width: _pxToRem(2),
        position: 'relative',
        left: _pxToRem(19),
        top: _pxToRem(20),
        zIndex: 1
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// eslint-disable-next-line no-restricted-imports
/**
 * @private
 */
const MeetingPhoneInfoPaneContent = (props) => {
    const { conferencePhoneInfoList } = props;
    const theme = react.useTheme();
    const localeStrings = useLocale().component.strings.meetingConferencePhoneInfo;
    if (props.mobileView) {
        return React.createElement(react.Stack, { verticalFill: true, styles: phoneInfoContainerStyle, tokens: phoneInfoContainerTokens, "data-ui-id": "phone-info-pane-content" },
            (conferencePhoneInfoList === undefined || conferencePhoneInfoList.length === 0) && React.createElement(react.Stack, { horizontal: true },
                React.createElement(react.Text, { className: phoneInfoTextStyle }, localeStrings.meetingConferencePhoneInfoModalNoPhoneAvailable)),
            conferencePhoneInfoList && conferencePhoneInfoList[0] && React.createElement(react.Stack, null,
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", className: phoneInfoInstructionLine },
                    React.createElement(react.Stack.Item, null,
                        React.createElement(react.Stack, { horizontal: true, className: phoneInfoStep },
                            React.createElement(react.Stack.Item, { className: phoneInfoIcon(theme) },
                                React.createElement(react.Stack, { verticalAlign: "center", horizontalAlign: "center" },
                                    React.createElement(react.Icon, { iconName: "JoinByPhoneDialStepIcon", className: phoneInfoIconStyle(theme) }))),
                            React.createElement(react.Stack.Item, null,
                                React.createElement(react.Text, { className: phoneInfoLabelStyle }, localeStrings.meetingConferencePhoneInfoModalDialIn)))),
                    React.createElement(react.Stack.Item, { className: phoneInfoStep }, conferencePhoneInfoList.map((phoneNumber, index) => React.createElement(react.Stack.Item, { key: index },
                        React.createElement(react.Link, { className: phoneInfoTextStyle, href: formatPhoneNumberLink(phoneNumber) }, _formatPhoneNumber(phoneNumber.phoneNumber, true)),
                        React.createElement(react.Text, { className: phoneInfoTextStyle },
                            ' ',
                            phoneNumber.isTollFree ? localeStrings.meetingConferencePhoneInfoModalTollFree : localeStrings.meetingConferencePhoneInfoModalToll),
                        React.createElement("br", null),
                        React.createElement(react.Text, { className: phoneInfoTextStyle },
                            " ",
                            formatPhoneNumberInfo(phoneNumber, localeStrings)))))),
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: phoneInfoInstructionLine },
                    React.createElement(react.Stack.Item, null,
                        React.createElement(react.Stack, { horizontal: true },
                            React.createElement(react.Stack.Item, { className: phoneInfoIcon(theme) },
                                React.createElement(react.Icon, { iconName: "JoinByPhoneConferenceIdIcon", className: phoneInfoIconStyle(theme) })),
                            React.createElement(react.Stack.Item, null,
                                React.createElement(react.Text, { className: phoneInfoLabelStyle }, localeStrings.meetingConferencePhoneInfoModalMeetingId)))),
                    React.createElement(react.Text, { className: phoneInfoTextStyle },
                        conferencePhoneInfoList[0].conferenceId,
                        "#")),
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: phoneInfoInstructionLine },
                    React.createElement(react.Stack, { horizontal: true },
                        React.createElement(react.Stack.Item, { className: phoneInfoIcon(theme) },
                            React.createElement(react.Icon, { iconName: "JoinByPhoneWaitToBeAdmittedIcon", className: phoneInfoIconStyle(theme) })),
                        React.createElement(react.Stack.Item, null,
                            React.createElement(react.Text, { className: phoneInfoLabelStyle }, localeStrings.meetingConferencePhoneInfoModalWait))))));
    }
    return React.createElement(react.Stack, { verticalFill: true, styles: phoneInfoContainerStyle, tokens: phoneInfoContainerTokens, "data-ui-id": "phone-info-pane-content" });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const SIDE_PANE_ID = 'Meeting Phone Info';
/** @private */
const useMeetingPhoneInfoPane = (props) => {
    const { updateSidePaneRenderer, mobileView, conferencePhoneInfo } = props;
    const closePane = React.useCallback(() => {
        updateSidePaneRenderer(undefined);
    }, [updateSidePaneRenderer]);
    const localeStrings = useLocale().component.strings.meetingConferencePhoneInfo;
    const callStrings = useLocale().strings.call;
    const onRenderHeader = React.useCallback(() => React.createElement(SidePaneHeader, { onClose: closePane, headingText: localeStrings.meetingConferencePhoneInfoModalTitle, dismissSidePaneButtonAriaLabel: callStrings.dismissSidePaneButtonLabel, mobileView: mobileView !== null && mobileView !== void 0 ? mobileView : false }), [mobileView, closePane, localeStrings, callStrings]);
    const onRenderContent = React.useCallback(() => {
        return React.createElement(MeetingPhoneInfoPaneContent, { mobileView: mobileView, conferencePhoneInfoList: conferencePhoneInfo });
    }, [mobileView, conferencePhoneInfo]);
    const sidePaneRenderer = React.useMemo(() => ({
        headerRenderer: onRenderHeader,
        contentRenderer: onRenderContent,
        id: SIDE_PANE_ID
    }), [onRenderContent, onRenderHeader]);
    const openPane = React.useCallback(() => {
        updateSidePaneRenderer(sidePaneRenderer);
    }, [sidePaneRenderer, updateSidePaneRenderer]);
    const isOpen = useIsParticularSidePaneOpen(SIDE_PANE_ID);
    // Update pane renderer if it is open and the openPane dep changes
    React.useEffect(() => {
        if (isOpen) {
            openPane();
        }
    }, [isOpen, openPane]);
    return {
        openMeetingPhoneInfoPane: openPane,
        closeMeetingPhoneInfoPane: closePane,
        isMeetingPhoneInfoPaneOpen: isOpen
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const activeVideoBackgroundEffectSelector = reselect__namespace.createSelector([getSelectedVideoEffect], (selectedVideoBackgroundEffect) => {
    if (selectedVideoBackgroundEffect && (selectedVideoBackgroundEffect.effectName === 'blur' || selectedVideoBackgroundEffect.effectName === 'none')) {
        return selectedVideoBackgroundEffect.effectName;
    }
    else if (selectedVideoBackgroundEffect && selectedVideoBackgroundEffect.effectName === 'replacement') {
        return selectedVideoBackgroundEffect === null || selectedVideoBackgroundEffect === void 0 ? void 0 : selectedVideoBackgroundEffect.key;
    }
    return 'none';
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const localVideoSelector = reselect__namespace.createSelector([callStatusSelector, getDeviceManager, getLocalVideoStreams], (callStatus, deviceManager, localVideoStreams) => {
    var _a, _b;
    let localVideoStream;
    if (_isInCall(callStatus.callStatus)) {
        localVideoStream = localVideoStreams === null || localVideoStreams === void 0 ? void 0 : localVideoStreams.find(i => i.mediaStreamType === 'Video');
    }
    else if (_isPreviewOn(deviceManager)) {
        // TODO: we should take in a LocalVideoStream that developer wants to use as their 'Preview' view. We should also
        // handle cases where 'Preview' view is in progress and not necessary completed.
        localVideoStream = deviceManager.unparentedViews[0];
    }
    return {
        isAvailable: !!localVideoStream,
        isMirrored: (_a = localVideoStream === null || localVideoStream === void 0 ? void 0 : localVideoStream.view) === null || _a === void 0 ? void 0 : _a.isMirrored,
        renderElement: (_b = localVideoStream === null || localVideoStream === void 0 ? void 0 : localVideoStream.view) === null || _b === void 0 ? void 0 : _b.target,
        activeVideoEffects: localVideoStream === null || localVideoStream === void 0 ? void 0 : localVideoStream.videoEffects
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$h = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Pane that is used to show video effects button
 * @private
 */
/** @beta */
const VideoEffectsPaneContent = (props) => {
    var _a;
    const { onDismissError, activeVideoEffectError, activeVideoEffectChange } = props;
    const locale = useLocale();
    const adapter = useAdapter();
    const strings = locale.strings.call;
    const activeVideoEffects = (_a = useSelector$1(localVideoSelector).activeVideoEffects) === null || _a === void 0 ? void 0 : _a.activeEffects;
    const selectableVideoEffects = React.useMemo(() => {
        const videoEffects = [{
                itemKey: 'none',
                iconProps: {
                    iconName: 'RemoveVideoBackgroundEffect'
                },
                title: strings.removeBackgroundEffectButtonLabel,
                tooltipProps: {
                    content: strings.removeBackgroundTooltip
                }
            }, {
                itemKey: 'blur',
                iconProps: {
                    iconName: 'BlurVideoBackground'
                },
                title: strings.blurBackgroundEffectButtonLabel,
                tooltipProps: {
                    content: strings.blurBackgroundTooltip
                }
            }];
        if (props.backgroundImages) {
            props.backgroundImages.forEach((img) => {
                var _a;
                videoEffects.push({
                    itemKey: img.key,
                    backgroundProps: {
                        url: img.url
                    },
                    tooltipProps: {
                        content: (_a = img.tooltipText) !== null && _a !== void 0 ? _a : ''
                    }
                });
            });
        }
        return videoEffects;
    }, [strings.removeBackgroundEffectButtonLabel, strings.removeBackgroundTooltip, strings.blurBackgroundEffectButtonLabel, strings.blurBackgroundTooltip, props.backgroundImages]);
    const onEffectChange = React.useCallback((effectKey) => __awaiter$h(void 0, void 0, void 0, function* () {
        if (effectKey === 'blur') {
            const blurEffect = {
                effectName: effectKey
            };
            activeVideoEffectChange({
                type: 'blur',
                timestamp: new Date(Date.now())
            });
            yield adapter.startVideoBackgroundEffect(blurEffect);
            adapter.updateSelectedVideoBackgroundEffect(blurEffect);
        }
        else if (effectKey === 'none') {
            const noneEffect = {
                effectName: effectKey
            };
            yield adapter.stopVideoBackgroundEffects();
            adapter.updateSelectedVideoBackgroundEffect(noneEffect);
        }
        else {
            const backgroundImg = selectableVideoEffects.find(effect => {
                return effect.itemKey === effectKey;
            });
            if (backgroundImg && backgroundImg.backgroundProps) {
                const replaceEffect = {
                    effectName: 'replacement',
                    key: effectKey,
                    backgroundImageUrl: backgroundImg.backgroundProps.url
                };
                activeVideoEffectChange({
                    type: 'replacement',
                    timestamp: new Date(Date.now())
                });
                yield adapter.startVideoBackgroundEffect(replaceEffect);
                adapter.updateSelectedVideoBackgroundEffect(replaceEffect);
            }
        }
    }), [adapter, activeVideoEffectChange, selectableVideoEffects]);
    if (activeVideoEffectError && activeVideoEffects && activeVideoEffects.length === 0) {
        const noneEffect = {
            effectName: 'none'
        };
        adapter.updateSelectedVideoBackgroundEffect(noneEffect);
    }
    return VideoEffectsPaneTrampoline(onDismissError, props.updateFocusHandle, activeVideoEffectError, selectableVideoEffects, onEffectChange);
};
const VideoEffectsPaneTrampoline = (onDismissError, updateFocusHandle, activeVideoEffectError, selectableVideoEffects, onEffectChange) => {
    const selectedEffect = useSelector$1(activeVideoBackgroundEffectSelector);
    const isCameraOn = useSelector$1(localVideoSelector).isAvailable;
    const showWarning = !isCameraOn && selectedEffect !== 'none';
    const locale = useLocale();
    return React.createElement(react.Stack, { tokens: {
            childrenGap: '0.75rem'
        }, className: react.mergeStyles({
            paddingLeft: '0.5rem'
        }) },
        activeVideoEffectError && isCameraOn && React.createElement(react.MessageBar, { messageBarType: react.MessageBarType.error, onDismiss: () => onDismissError(activeVideoEffectError), role: "alert", "aria-live": "assertive" }, locale.strings.call.unableToStartVideoEffect),
        showWarning && React.createElement(react.MessageBar, { messageBarType: react.MessageBarType.warning, role: "alert", "aria-live": "assertive" }, locale.strings.call.cameraOffBackgroundEffectWarningText),
        React.createElement(_VideoBackgroundEffectsPicker, { label: locale.strings.call.videoEffectsPaneBackgroundSelectionTitle, styles: backgroundPickerStyles, options: selectableVideoEffects !== null && selectableVideoEffects !== void 0 ? selectableVideoEffects : [], onChange: onEffectChange, selectedEffectKey: selectedEffect, componentRef: updateFocusHandle }));
};
const backgroundPickerStyles = {
    label: {
        fontSize: '0.75rem',
        lineHeight: '0.5rem',
        fontWeight: '400'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const VIDEO_EFFECTS_SIDE_PANE_ID = 'videoeffects';
/** @private */
const VIDEO_EFFECTS_SIDE_PANE_WIDTH_REM = 17.5;
/** @private */
const useVideoEffectsPane = (updateSidePaneRenderer, mobileView, latestErrors, onDismissError, cameraButtonRef) => {
    const closePane = React.useCallback(() => {
        var _a;
        updateSidePaneRenderer(undefined);
        (_a = cameraButtonRef === null || cameraButtonRef === void 0 ? void 0 : cameraButtonRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, [cameraButtonRef, updateSidePaneRenderer]);
    const locale = useLocale();
    const onRenderHeader = React.useCallback(() => {
        var _a, _b;
        return React.createElement(SidePaneHeader, { onClose: closePane, headingText: locale.strings.call.videoEffectsPaneTitle, dismissSidePaneButtonAriaLabel: (_b = (_a = locale.strings.call.dismissSidePaneButtonLabel) !== null && _a !== void 0 ? _a : locale.strings.callWithChat.dismissSidePaneButtonLabel) !== null && _b !== void 0 ? _b : 'Close', mobileView: mobileView });
    }, [closePane, locale.strings, mobileView]);
    const latestVideoEffectError = latestErrors.find(error => error.type === 'unableToStartVideoEffect');
    const updateFocusHandle = React.useMemo(() => React.createRef(), []);
    const backgroundImages = useSelector$1(getVideoBackgroundImages);
    const onRenderContent = React.useCallback(() => {
        return React.createElement(VideoEffectsPaneContent, { onDismissError: onDismissError, activeVideoEffectError: latestVideoEffectError, activeVideoEffectChange: () => {
                // Clear any existing video effects error when the user clicks on a new video effect
                latestVideoEffectError && (onDismissError === null || onDismissError === void 0 ? void 0 : onDismissError(latestVideoEffectError));
            }, updateFocusHandle: updateFocusHandle, backgroundImages: backgroundImages });
    }, [latestVideoEffectError, onDismissError, updateFocusHandle, backgroundImages]);
    const sidePaneRenderer = React.useMemo(() => ({
        headerRenderer: onRenderHeader,
        contentRenderer: onRenderContent,
        id: VIDEO_EFFECTS_SIDE_PANE_ID
    }), [onRenderContent, onRenderHeader]);
    const openPane = React.useCallback(() => {
        updateSidePaneRenderer(sidePaneRenderer);
        // Run in a setTimeout as it must be called only once the imperative handle is available
        setTimeout(() => { var _a; return (_a = updateFocusHandle.current) === null || _a === void 0 ? void 0 : _a.focus(); }, 0);
    }, [sidePaneRenderer, updateSidePaneRenderer, updateFocusHandle]);
    const isOpen = useIsParticularSidePaneOpen(VIDEO_EFFECTS_SIDE_PANE_ID);
    // Update pane renderer if it is open and the openPane dep changes
    React.useEffect(() => {
        if (isOpen) {
            openPane();
        }
    }, [isOpen, openPane]);
    const togglePane = React.useCallback(() => {
        if (isOpen) {
            closePane();
        }
        else {
            openPane();
        }
    }, [closePane, isOpen, openPane]);
    return {
        openVideoEffectsPane: openPane,
        closeVideoEffectsPane: closePane,
        toggleVideoEffectsPane: togglePane,
        isVideoEffectsPaneOpen: isOpen
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const LocalAndRemotePIP = (props) => {
    var _a, _b;
    const { localParticipant, remoteParticipant, onCreateLocalStreamView, onDisposeLocalStreamView, onCreateRemoteStreamView, onDisposeRemoteStreamView } = props;
    const localVideoTileProps = React.useMemo(() => {
        var _a, _b;
        return ({
            onCreateLocalStreamView,
            onDisposeLocalStreamView,
            localVideoViewOptions: localVideoViewOptions$3,
            displayName: localParticipant.displayName,
            showLabel: false,
            showMuteIndicator: false,
            showCameraSwitcherInLocalPreview: false,
            isAvailable: (_a = localParticipant.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable,
            renderElement: (_b = localParticipant.videoStream) === null || _b === void 0 ? void 0 : _b.renderElement,
            raisedHand: localParticipant.raisedHand
        });
    }, [localParticipant.displayName, (_a = localParticipant.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable, (_b = localParticipant.videoStream) === null || _b === void 0 ? void 0 : _b.renderElement, onCreateLocalStreamView, onDisposeLocalStreamView, localParticipant.raisedHand]);
    const remoteVideoTileProps = React.useMemo(() => {
        var _a, _b, _c;
        return !remoteParticipant ? undefined : {
            remoteParticipant: remoteParticipant,
            onCreateRemoteStreamView,
            onDisposeRemoteStreamView,
            remoteVideoViewOptions: remoteVideoViewOptions$1,
            displayName: remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.displayName,
            showLabel: false,
            showMuteIndicator: false,
            isAvailable: (_a = remoteParticipant.videoStream) === null || _a === void 0 ? void 0 : _a.isAvailable,
            isReceiving: (_b = remoteParticipant.videoStream) === null || _b === void 0 ? void 0 : _b.isReceiving,
            renderElement: (_c = remoteParticipant.videoStream) === null || _c === void 0 ? void 0 : _c.renderElement,
            userId: remoteParticipant.userId,
            key: remoteParticipant.userId
        };
    }, [remoteParticipant, onCreateRemoteStreamView, onDisposeRemoteStreamView]);
    const locale = useLocale();
    const ariaLabel = locale.strings.callWithChat.pictureInPictureTileAriaLabel;
    const strings = React.useMemo(() => ({
        rootAriaLabel: ariaLabel
    }), [ariaLabel]);
    // If there are no remote participants, show the local participant as the primary tile
    const primaryTileProps = React.useMemo(() => ({
        children: remoteVideoTileProps ? React.createElement(_RemoteVideoTile, Object.assign({}, remoteVideoTileProps, { strings: locale.component.strings.videoGallery, onLongTouch: () => {
                /* Add to disable scaling */
            } })) : React.createElement(_LocalVideoTile, Object.assign({}, localVideoTileProps)),
        // TODO: when the calling SDK provides height/width stream information - update this to reflect the stream orientation.
        orientation: 'portrait'
    }), [localVideoTileProps, remoteVideoTileProps, locale.component.strings.videoGallery]);
    // If we are showing the local participant as the primary tile, show nothing for the secondary tile
    const secondaryTileProps = React.useMemo(() => remoteVideoTileProps ? {
        children: React.createElement(_LocalVideoTile, Object.assign({}, localVideoTileProps, { personaMinSize: 20 })),
        // TODO: when the calling SDK provides height/width stream information - update this to reflect the stream orientation.
        orientation: 'portrait'
    } : undefined, [localVideoTileProps, remoteVideoTileProps]);
    return React.createElement(_PictureInPictureInPicture, { onClick: props.onClick, strings: strings, primaryTile: primaryTileProps, secondaryTile: secondaryTileProps });
};
const localVideoViewOptions$3 = {
    scalingMode: 'Crop',
    isMirrored: true
};
const remoteVideoViewOptions$1 = {
    scalingMode: 'Crop',
    isMirrored: false
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Get the most dominant remote participant, if no dominant speaker IDs exist, this uses
 * one of the remote participants with no bias towards which one.
 *
 * @private
 */
const dominantRemoteParticipantSelector = reselect__namespace.createSelector([getRemoteParticipants, getDominantSpeakerInfo], (remoteParticipants, dominantSpeakerInfo) => {
    const dominantSpeakers = _dominantSpeakersWithFlatId(dominantSpeakerInfo);
    const dominantRemoteParticipant = remoteParticipants && Object.keys(remoteParticipants).length > 0 ? findDominantRemoteParticipant(remoteParticipants, dominantSpeakers !== null && dominantSpeakers !== void 0 ? dominantSpeakers : []) : undefined;
    return dominantRemoteParticipant;
});
const findDominantRemoteParticipant = (remoteParticipants, dominantSpeakerIds) => {
    let dominantRemoteParticipantId = dominantSpeakerIds[0];
    // Fallback to using the first remote participant if there are no dominant speaker IDs
    // or if the dominant speaker is no longer available in the list of remoteParticipantIds
    const remoteParticipantIds = Object.keys(remoteParticipants);
    if (!dominantRemoteParticipantId || !remoteParticipantIds.includes(dominantRemoteParticipantId)) {
        dominantRemoteParticipantId = remoteParticipantIds[0];
    }
    return dominantRemoteParticipantId ? remoteParticipants[dominantRemoteParticipantId] : undefined;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Get the first remote participant that is spotlighted if any
 *
 * @private
 */
const getFirstSpotlightedRemoteParticipant = reselect__namespace.createSelector([getRemoteParticipants, getSpotlightedParticipants], (remoteParticipants, spotlightedParticipants) => {
    const spotlightedParticipantUserIds = spotlightedParticipants ? spotlightedParticipants.map(p => toFlatCommunicationIdentifier(p.identifier)) : [];
    const firstSpotlightedRemoteParticipant = remoteParticipants ? findFirstSpotlightedRemoteParticipant(remoteParticipants, spotlightedParticipantUserIds) : undefined;
    return firstSpotlightedRemoteParticipant;
});
const findFirstSpotlightedRemoteParticipant = (remoteParticipants, spotlightedParticipantUserIds) => {
    const remoteParticipantIds = Object.keys(remoteParticipants);
    const spotlightedRemoteParticipantUserIds = spotlightedParticipantUserIds.filter(p => remoteParticipantIds.includes(p));
    return spotlightedRemoteParticipantUserIds[0] ? remoteParticipants[spotlightedRemoteParticipantUserIds[0]] : undefined;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Picture in picture in picture needs to display the most-dominant remote speaker, as well as the local participant video.
 * @private
 */
const localAndRemotePIPSelector = reselect__namespace.createSelector([getDisplayName, dominantRemoteParticipantSelector, localVideoSelector, getLocalParticipantRaisedHand, getFirstSpotlightedRemoteParticipant], (displayName, dominantRemoteParticipantState, localVideoStreamInfo, raisedHand, firstSpotlightedRemoteParticipantState) => {
    let remoteParticipantState = dominantRemoteParticipantState;
    if (firstSpotlightedRemoteParticipantState) {
        remoteParticipantState = firstSpotlightedRemoteParticipantState;
    }
    const remoteParticipant = remoteParticipantState ? _videoGalleryRemoteParticipantsMemo(_updateUserDisplayNames([remoteParticipantState]))[0] : undefined;
    return {
        localParticipant: {
            displayName,
            videoStream: localVideoStreamInfo,
            raisedHand
        },
        remoteParticipant
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Drag options for Modal in {@link ModalLocalAndRemotePIP} component
 */
const DRAG_OPTIONS = {
    moveMenuItemText: 'Move',
    closeMenuItemText: 'Close',
    menu: react.ContextualMenu,
    keepInBounds: true
};
/**
 * A wrapping component with a draggable {@link LocalAndRemotePIP} component that is bound to a LayerHost component with id
 * specified by `modalLayerHostId` prop
 * @private
 */
const ModalLocalAndRemotePIP = (props) => {
    var _a;
    const rootStyles = props.hidden ? hiddenStyle : PIPContainerStyle;
    const role = useSelector$1(getRole);
    const locale = useLocale();
    const pictureInPictureProps = useSelector$1(localAndRemotePIPSelector);
    const [touchStartTouches, setTouchStartTouches] = React.useState(null);
    const onTouchEnd = React.useCallback((event) => {
        var _a;
        if (touchStartTouches && touchStartTouches[0] && touchStartTouches.length === 1 && event.changedTouches[0] && event.changedTouches.length === 1) {
            const touchStartTouch = touchStartTouches[0];
            const touchEndTouch = event.changedTouches[0];
            if (Math.abs(touchStartTouch.clientX - touchEndTouch.clientX) < 10 && Math.abs(touchStartTouch.clientY - touchEndTouch.clientY) < 10) {
                (_a = props.onDismissSidePane) === null || _a === void 0 ? void 0 : _a.call(props);
            }
        }
    }, [props, touchStartTouches]);
    const onTouchStart = React.useCallback((event) => {
        setTouchStartTouches(event.touches);
    }, []);
    const onKeyDown = React.useCallback((event) => {
        var _a;
        if (event.key === 'Enter' || event.key === ' ') {
            (_a = props.onDismissSidePane) === null || _a === void 0 ? void 0 : _a.call(props);
        }
    }, [props]);
    const pictureInPictureHandlers = useHandlers();
    const remoteParticipant = pictureInPictureProps.remoteParticipant;
    const localAndRemotePIP = React.useMemo(() => {
        var _a, _b, _c, _d;
        if (role === 'Consumer' && (remoteParticipant === null || remoteParticipant === void 0 ? void 0 : remoteParticipant.userId)) {
            return React.createElement(react.Stack, { tabIndex: 0, "aria-label": (_b = (_a = props.strings) === null || _a === void 0 ? void 0 : _a.dismissModalAriaLabel) !== null && _b !== void 0 ? _b : '', onKeyDown: onKeyDown },
                React.createElement(_RemoteVideoTile, Object.assign({ strings: locale.component.strings.videoGallery }, remoteParticipant, { remoteParticipant: remoteParticipant })));
        }
        return React.createElement(react.Stack, { tabIndex: 0, "aria-label": (_d = (_c = props.strings) === null || _c === void 0 ? void 0 : _c.dismissModalAriaLabel) !== null && _d !== void 0 ? _d : '', onKeyDown: onKeyDown },
            React.createElement(LocalAndRemotePIP, Object.assign({}, pictureInPictureProps, pictureInPictureHandlers, { remoteParticipant: remoteParticipant })));
    }, [role, onKeyDown, pictureInPictureProps, props, pictureInPictureHandlers, locale.component.strings.videoGallery, remoteParticipant]);
    if (role === 'Consumer' && !remoteParticipant) {
        return null;
    }
    const modalStylesThemed = react.concatStyleSets(modalStyle, (_a = props.styles) === null || _a === void 0 ? void 0 : _a.modal);
    return React.createElement(react.Stack, { styles: rootStyles },
        React.createElement(react.Stack, { onTouchStart: onTouchStart, onTouchEnd: onTouchEnd },
            React.createElement(_ModalClone, { isOpen: true, isModeless: true, dragOptions: DRAG_OPTIONS, styles: modalStylesThemed, layerProps: {
                    hostId: props.modalLayerHostId
                }, minDragPosition: props.minDragPosition, maxDragPosition: props.maxDragPosition }, 
            // Only render LocalAndRemotePIP when this component is NOT hidden because VideoGallery needs to have
            // possession of the dominant remote participant video stream
            !props.hidden && localAndRemotePIP)));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Notification bar for capabilities changed
 * @private
 */
const CapabilitiesChangedNotificationBar = (props) => {
    const locale = useLocale();
    return React.createElement(react.Stack, { "data-ui-id": "capabilities-changed-notification-bar-stack" }, props.capabilitiesChangedNotifications.map(notification => {
        const message = getCapabilityChangedNotificationString(notification, locale.strings.call.capabilityChangedNotification);
        if (!message) {
            return null;
        }
        const iconProps = getCustomMessageBarIconProps(notification);
        return React.createElement(react.MessageBar, { key: notification.capabilityName, styles: messageBarStyles, messageBarType: react.MessageBarType.warning, dismissIconProps: {
                iconName: 'ErrorBarClear'
            }, onDismiss: () => props.onDismissNotification(notification), messageBarIconProps: iconProps }, message);
    }));
};
const getCapabilityChangedNotificationString = (notification, strings) => {
    var _a, _b, _c, _d;
    switch (notification.capabilityName) {
        case 'shareScreen':
            if (notification.isPresent && notification.changedReason === 'RoleChanged' && notification.role === 'Presenter') {
                return (_a = strings === null || strings === void 0 ? void 0 : strings.shareScreen) === null || _a === void 0 ? void 0 : _a.grantedDueToRoleChangeToPresenter;
            }
            if (!notification.isPresent && notification.changedReason === 'RoleChanged' && notification.role === 'Attendee') {
                return (_b = strings === null || strings === void 0 ? void 0 : strings.shareScreen) === null || _b === void 0 ? void 0 : _b.lostDueToRoleChangeToAttendee;
            }
            break;
        case 'viewAttendeeNames':
            if (!notification.isPresent && notification.changedReason === 'MeetingOptionOrOrganizerPolicyChanged' && notification.role === 'Attendee') {
                return (_c = strings === null || strings === void 0 ? void 0 : strings.hideAttendeeNames) === null || _c === void 0 ? void 0 : _c.hideAttendeeNameAttendee;
            }
            if (!notification.isPresent && notification.changedReason === 'MeetingOptionOrOrganizerPolicyChanged' && notification.role === 'Presenter') {
                return (_d = strings === null || strings === void 0 ? void 0 : strings.hideAttendeeNames) === null || _d === void 0 ? void 0 : _d.hideAttendeeNamePresenter;
            }
    }
    return undefined;
};
const getCustomMessageBarIconProps = (notification) => {
    let iconName = undefined;
    switch (notification.capabilityName) {
        case 'turnVideoOn':
            if (notification.isPresent) {
                iconName = 'ControlButtonCameraOn';
            }
            else {
                iconName = 'ControlButtonCameraProhibited';
            }
            break;
        case 'unmuteMic':
            if (notification.isPresent) {
                iconName = 'ControlButtonMicOn';
            }
            else {
                iconName = 'ControlButtonMicProhibited';
            }
            break;
        default:
            return undefined;
    }
    return {
        iconName,
        styles: {
            root: {
                '> *': {
                    height: '1rem',
                    width: '1rem'
                }
            }
        }
    };
};
const messageBarStyles = {
    innerText: {
        alignSelf: 'center'
    },
    icon: {
        height: 0
    },
    content: {
        lineHeight: 'inherit'
    },
    dismissal: {
        height: 0,
        paddingTop: '0.8rem'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$g = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @internal
 */
const useLocalSpotlightCallbacksWithPrompt = (onStartLocalSpotlight, onStopLocalSpotlight, setIsPromptOpen, setPromptProps) => {
    const strings = useLocale().strings.call;
    return React.useMemo(() => {
        if (!setIsPromptOpen || !setPromptProps) {
            return {
                onStartLocalSpotlightWithPrompt: onStartLocalSpotlight,
                onStopLocalSpotlightWithPrompt: onStopLocalSpotlight
            };
        }
        return {
            onStartLocalSpotlightWithPrompt: onStartLocalSpotlight ? getStartLocalSpotlightWithPromptCallback(onStartLocalSpotlight, setIsPromptOpen, setPromptProps, strings) : undefined,
            onStopLocalSpotlightWithPrompt: onStopLocalSpotlight ? getStopLocalSpotlightWithPromptCallback(onStopLocalSpotlight, setIsPromptOpen, setPromptProps, strings) : undefined
        };
    }, [onStartLocalSpotlight, onStopLocalSpotlight, setIsPromptOpen, setPromptProps, strings]);
};
const getStartLocalSpotlightWithPromptCallback = (onStartSpotlight, setIsPromptOpen, setPromptProps, strings) => {
    return () => __awaiter$g(void 0, void 0, void 0, function* () {
        setPromptProps({
            heading: strings.spotlightPrompt.startSpotlightHeading,
            text: strings.spotlightPrompt.startSpotlightOnSelfText,
            confirmButtonLabel: strings.spotlightPrompt.startSpotlightConfirmButtonLabel,
            cancelButtonLabel: strings.spotlightPrompt.startSpotlightCancelButtonLabel,
            closeButtonLabel: strings.spotlightPrompt.closeSpotlightPromptButtonLabel,
            onConfirm: () => {
                onStartSpotlight();
                setIsPromptOpen(false);
            },
            onCancel: () => setIsPromptOpen(false)
        });
        setIsPromptOpen(true);
    });
};
const getStopLocalSpotlightWithPromptCallback = (onStopSpotlight, setIsPromptOpen, setPromptProps, strings) => {
    return () => __awaiter$g(void 0, void 0, void 0, function* () {
        setPromptProps({
            heading: strings.spotlightPrompt.stopSpotlightOnSelfHeading,
            text: strings.spotlightPrompt.stopSpotlightOnSelfText,
            confirmButtonLabel: strings.spotlightPrompt.stopSpotlightOnSelfConfirmButtonLabel,
            cancelButtonLabel: strings.spotlightPrompt.stopSpotlightCancelButtonLabel,
            onConfirm: () => {
                onStopSpotlight();
                setIsPromptOpen(false);
            },
            onCancel: () => setIsPromptOpen(false)
        });
        setIsPromptOpen(true);
    });
};
/**
 * @internal
 */
const useRemoteSpotlightCallbacksWithPrompt = (onStartRemoteSpotlight, onStopRemoteSpotlight, setIsPromptOpen, setPromptProps) => {
    const strings = useLocale().strings.call;
    return React.useMemo(() => {
        if (!setIsPromptOpen || !setPromptProps) {
            return {
                onStartRemoteSpotlightWithPrompt: onStartRemoteSpotlight,
                onStopRemoteSpotlightWithPrompt: onStopRemoteSpotlight
            };
        }
        return {
            onStartRemoteSpotlightWithPrompt: onStartRemoteSpotlight ? getStartRemoteSpotlightWithPromptCallback(onStartRemoteSpotlight, setIsPromptOpen, setPromptProps, strings) : undefined,
            onStopRemoteSpotlightWithPrompt: onStopRemoteSpotlight ? getStopRemoteSpotlightWithPromptCallback(onStopRemoteSpotlight, setIsPromptOpen, setPromptProps, strings) : undefined
        };
    }, [onStartRemoteSpotlight, onStopRemoteSpotlight, setIsPromptOpen, setPromptProps, strings]);
};
const getStartRemoteSpotlightWithPromptCallback = (onStartSpotlight, setIsPromptOpen, setPromptProps, strings) => {
    return (userIds) => __awaiter$g(void 0, void 0, void 0, function* () {
        if (userIds.length > 1) {
            onStartSpotlight(userIds);
        }
        setPromptProps({
            heading: strings.spotlightPrompt.startSpotlightHeading,
            text: strings.spotlightPrompt.startSpotlightText,
            confirmButtonLabel: strings.spotlightPrompt.startSpotlightConfirmButtonLabel,
            cancelButtonLabel: strings.spotlightPrompt.startSpotlightCancelButtonLabel,
            closeButtonLabel: strings.spotlightPrompt.closeSpotlightPromptButtonLabel,
            onConfirm: () => {
                onStartSpotlight(userIds);
                setIsPromptOpen(false);
            },
            onCancel: () => setIsPromptOpen(false)
        });
        setIsPromptOpen(true);
    });
};
const getStopRemoteSpotlightWithPromptCallback = (onStopSpotlight, setIsPromptOpen, setPromptProps, strings) => {
    return (userIds) => __awaiter$g(void 0, void 0, void 0, function* () {
        if (userIds.length > 1) {
            onStopSpotlight(userIds);
        }
        setPromptProps({
            heading: strings.spotlightPrompt.stopSpotlightHeading,
            text: strings.spotlightPrompt.stopSpotlightText,
            confirmButtonLabel: strings.spotlightPrompt.stopSpotlightConfirmButtonLabel,
            cancelButtonLabel: strings.spotlightPrompt.stopSpotlightCancelButtonLabel,
            closeButtonLabel: strings.spotlightPrompt.closeSpotlightPromptButtonLabel,
            onConfirm: () => {
                onStopSpotlight(userIds);
                setIsPromptOpen(false);
            },
            onCancel: () => setIsPromptOpen(false)
        });
        setIsPromptOpen(true);
    });
};
/**
 * @internal
 */
const useStopAllSpotlightCallbackWithPrompt = (stopAllSpotlight, setIsPromptOpen, setPromptProps) => {
    const strings = useLocale().strings.call;
    return React.useMemo(() => {
        if (!setIsPromptOpen || !setPromptProps) {
            return {
                stopAllSpotlightWithPrompt: stopAllSpotlight
            };
        }
        return {
            stopAllSpotlightWithPrompt: stopAllSpotlight ? getStopAllSpotlightCallbackWithPromptCallback(stopAllSpotlight, setIsPromptOpen, setPromptProps, strings) : undefined
        };
    }, [stopAllSpotlight, setIsPromptOpen, setPromptProps, strings]);
};
const getStopAllSpotlightCallbackWithPromptCallback = (stopAllSpotlight, setIsPromptOpen, setPromptProps, strings) => {
    return () => __awaiter$g(void 0, void 0, void 0, function* () {
        setPromptProps({
            heading: strings.spotlightPrompt.stopAllSpotlightHeading,
            text: strings.spotlightPrompt.stopAllSpotlightText,
            confirmButtonLabel: strings.spotlightPrompt.stopSpotlightConfirmButtonLabel,
            cancelButtonLabel: strings.spotlightPrompt.stopSpotlightCancelButtonLabel,
            closeButtonLabel: strings.spotlightPrompt.closeSpotlightPromptButtonLabel,
            onConfirm: () => {
                stopAllSpotlight();
                setIsPromptOpen(false);
            },
            onCancel: () => setIsPromptOpen(false)
        });
        setIsPromptOpen(true);
    });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useCompositeStringsForNotificationStackStrings = (locale) => {
    const breakoutRoomDisplayName = useSelector$1(getBreakoutRoomDisplayName);
    const notificationStackStrings = locale.component.strings.notificationStack;
    if (notificationStackStrings.breakoutRoomJoined && locale.strings.call.breakoutRoomJoinedNotificationTitle && breakoutRoomDisplayName) {
        notificationStackStrings.breakoutRoomJoined.title = locale.strings.call.breakoutRoomJoinedNotificationTitle.replace('{roomName}', breakoutRoomDisplayName);
    }
    return notificationStackStrings;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * A component to show a banner in the UI.
 *
 * @private
 */
const Banner = (props) => {
    var _a, _b;
    const strings = props.strings;
    const theme = react.useTheme();
    return React.createElement(react.Stack, { horizontalAlign: "center" },
        React.createElement(react.Stack, { "data-ui-id": "banner", className: containerStyles$1(theme) },
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between" },
                React.createElement(react.Stack, { horizontal: true },
                    ((_a = props.iconProps) === null || _a === void 0 ? void 0 : _a.iconName) && React.createElement(react.Icon, Object.assign({ className: bannerIconClassName, iconName: (_b = props.iconProps) === null || _b === void 0 ? void 0 : _b.iconName }, props.iconProps)),
                    React.createElement(react.Text, { className: titleTextClassName }, strings === null || strings === void 0 ? void 0 : strings.title)),
                props.primaryButton ? React.createElement(react.PrimaryButton, { text: strings === null || strings === void 0 ? void 0 : strings.primaryButtonLabel, ariaLabel: strings === null || strings === void 0 ? void 0 : strings.primaryButtonLabel, onClick: props.onClickButton }) : React.createElement(react.DefaultButton, { text: strings === null || strings === void 0 ? void 0 : strings.primaryButtonLabel, ariaLabel: strings === null || strings === void 0 ? void 0 : strings.primaryButtonLabel, onClick: props.onClickButton }))));
};
const titleTextClassName = react.mergeStyles({
    fontWeight: 400,
    fontSize: _pxToRem(14),
    lineHeight: _pxToRem(16),
    alignSelf: 'center'
});
const containerStyles$1 = (theme) => react.mergeStyles({
    boxShadow: theme.effects.elevation8,
    width: '20rem',
    padding: '0.75rem',
    borderRadius: '0.25rem',
    position: 'relative',
    backgroundColor: theme.palette.white
});
const bannerIconClassName = react.mergeStyles({
    fontSize: '1.25rem',
    alignSelf: 'center',
    marginRight: '0.5rem',
    svg: {
        width: '1.25rem',
        height: '1.25rem'
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const BreakoutRoomsBanner = (props) => {
    const { locale, adapter } = props;
    const assignedBreakoutRoom = useSelector$1(getAssignedBreakoutRoom);
    const breakoutRoomSettings = useSelector$1(getBreakoutRoomSettings);
    const latestNotifications = useSelector$1(getLatestNotifications);
    if (assignedBreakoutRoom && assignedBreakoutRoom.state === 'open' &&
        // Breakout room settings are only defined in a breakout room so we use this to ensure
        // the button is not shown when already in a breakout room
        !breakoutRoomSettings && !latestNotifications['assignedBreakoutRoomOpened']) {
        return React.createElement(react.Stack, { styles: bannerNotificationStyles },
            React.createElement(Banner, { strings: {
                    title: assignedBreakoutRoom.displayName ? locale.strings.call.joinBreakoutRoomBannerTitle.replace('{roomName}', assignedBreakoutRoom.displayName) : locale.strings.call.joinBreakoutRoomBannerTitle,
                    primaryButtonLabel: locale.strings.call.joinBreakoutRoomBannerButtonLabel
                }, onClickButton: () => assignedBreakoutRoom.join(), iconProps: {
                    iconName: 'NotificationBarBreakoutRoomPromptJoin'
                }, primaryButton: true }));
    }
    else if (breakoutRoomSettings && breakoutRoomSettings.disableReturnToMainMeeting !== true) {
        return React.createElement(react.Stack, { styles: bannerNotificationStyles },
            React.createElement(Banner, { strings: {
                    title: locale.strings.call.returnFromBreakoutRoomBannerTitle,
                    primaryButtonLabel: locale.strings.call.returnFromBreakoutRoomBannerButtonLabel
                }, onClickButton: () => adapter.returnFromBreakoutRoom(), iconProps: {
                    iconName: 'NotificationBarBreakoutRoomClosingSoon'
                } }));
    }
    return null;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 * Maximum number of remote video tiles that can be pinned
 */
const MAX_PINNED_REMOTE_VIDEO_TILES = 4;
/**
 * @private
 */
const CallArrangement = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
    const containerClassName = React.useMemo(() => {
        return props.mobileView ? containerStyleMobile : containerStyleDesktop;
    }, [props.mobileView]);
    const theme = useTheme();
    const callGalleryStyles = React.useMemo(() => galleryParentContainerStyles(theme.palette.neutralLighterAlt), [theme.palette.neutralLighterAlt]);
    const peopleButtonRef = React.useRef(null);
    const cameraButtonRef = React.useRef(null);
    const sidePaneDismissButtonRef = React.useRef(null);
    const containerRef = React.useRef(null);
    const containerWidth = _useContainerWidth(containerRef);
    const containerHeight = _useContainerHeight(containerRef);
    const isInLobby = _isInLobbyOrConnecting(useSelector$1(callStatusSelector).callStatus);
    const { updateSidePaneRenderer } = props;
    const isInLocalHold = useSelector$1(getPage) === 'hold';
    const capabilities = useSelector$1(getCapabilites);
    const adapter = useAdapter();
    const [participantActioned, setParticipantActioned] = React.useState();
    const remoteParticipants = useSelector$1(getRemoteParticipants);
    const [drawerMenuItems, setDrawerMenuItems] = React.useState([]);
    React.useEffect(() => {
        const participantIsActionedButIsNotPresent = participantActioned && (remoteParticipants === null || remoteParticipants === void 0 ? void 0 : remoteParticipants[participantActioned]) === undefined;
        if (participantIsActionedButIsNotPresent) {
            setDrawerMenuItems([]);
        }
    }, [participantActioned, remoteParticipants]);
    const conferencePhoneInfo = useSelector$1(getTeamsMeetingCoordinates);
    const meetingPhoneInfoPaneProps = {
        updateSidePaneRenderer,
        mobileView: props.mobileView,
        conferencePhoneInfo: conferencePhoneInfo
    };
    const peoplePaneProps = React.useMemo(() => {
        var _a;
        return ({
            updateSidePaneRenderer,
            setDrawerMenuItems,
            inviteLink: props.callControlProps.callInvitationURL,
            onFetchAvatarPersonaData: props.onFetchAvatarPersonaData,
            onFetchParticipantMenuItems: (_a = props.callControlProps) === null || _a === void 0 ? void 0 : _a.onFetchParticipantMenuItems,
            mobileView: props.mobileView,
            peopleButtonRef,
            setParticipantActioned,
            sidePaneDismissButtonRef,
            chatButtonPresent: !!props.onCloseChatPane
        });
    }, [updateSidePaneRenderer, props.callControlProps.callInvitationURL, (_a = props.callControlProps) === null || _a === void 0 ? void 0 : _a.onFetchParticipantMenuItems, props.onFetchAvatarPersonaData, props.mobileView, peopleButtonRef, setParticipantActioned, sidePaneDismissButtonRef, props.onCloseChatPane]);
    const locale = useLocale();
    const role = useSelector$1(getRole);
    const videoGalleryProps = usePropsFor$1(VideoGallery);
    const muteAllHandlers = useHandlers();
    const { setPromptProps, setIsPromptOpen, hideSpotlightButtons } = props;
    const { onStartLocalSpotlight, onStopLocalSpotlight, onStartRemoteSpotlight, onStopRemoteSpotlight, onMuteParticipant, spotlightedParticipants, maxParticipantsToSpotlight, localParticipant, onForbidAudio, onPermitAudio, onForbidVideo, onPermitVideo } = videoGalleryProps;
    const [showTeamsMeetingConferenceModal, setShowTeamsMeetingConferenceModal] = React.useState(false);
    const toggleTeamsMeetingConferenceModal = React.useCallback(() => {
        setShowTeamsMeetingConferenceModal(!showTeamsMeetingConferenceModal);
    }, [showTeamsMeetingConferenceModal]);
    const { isMeetingPhoneInfoPaneOpen, openMeetingPhoneInfoPane, closeMeetingPhoneInfoPane } = useMeetingPhoneInfoPane(Object.assign({}, meetingPhoneInfoPaneProps));
    const toggleMeetingPhoneInfoPane = React.useCallback(() => {
        if (isMeetingPhoneInfoPaneOpen) {
            closeMeetingPhoneInfoPane();
        }
        else {
            openMeetingPhoneInfoPane();
        }
    }, [closeMeetingPhoneInfoPane, isMeetingPhoneInfoPaneOpen, openMeetingPhoneInfoPane]);
    const onMeetingPhoneInfoClicked = React.useCallback(() => {
        setShowDrawer(false);
        toggleMeetingPhoneInfoPane();
    }, [toggleMeetingPhoneInfoPane]);
    const { pinnedParticipants, setPinnedParticipants } = props;
    const onPinParticipant = React.useCallback((userId) => {
        if (pinnedParticipants && pinnedParticipants.length >= MAX_PINNED_REMOTE_VIDEO_TILES) {
            return;
        }
        if (pinnedParticipants && setPinnedParticipants && !pinnedParticipants.includes(userId)) {
            setPinnedParticipants(pinnedParticipants.concat(userId));
        }
    }, [pinnedParticipants, setPinnedParticipants]);
    const onUnpinParticipant = React.useCallback((userId) => {
        if (pinnedParticipants && setPinnedParticipants) {
            setPinnedParticipants(pinnedParticipants.filter(participantId => participantId !== userId));
        }
    }, [setPinnedParticipants, pinnedParticipants]);
    const pinPeoplePaneProps = React.useMemo(() => {
        return {
            pinnedParticipants: pinnedParticipants,
            onPinParticipant: onPinParticipant,
            onUnpinParticipant: onUnpinParticipant,
            disablePinMenuItem: pinnedParticipants && pinnedParticipants.length >= MAX_PINNED_REMOTE_VIDEO_TILES
        };
    }, [onPinParticipant, onUnpinParticipant, pinnedParticipants]);
    const { onStartLocalSpotlightWithPrompt, onStopLocalSpotlightWithPrompt } = useLocalSpotlightCallbacksWithPrompt(onStartLocalSpotlight, onStopLocalSpotlight, setIsPromptOpen, setPromptProps);
    const { onStartRemoteSpotlightWithPrompt, onStopRemoteSpotlightWithPrompt } = useRemoteSpotlightCallbacksWithPrompt(onStartRemoteSpotlight, onStopRemoteSpotlight, setIsPromptOpen, setPromptProps);
    const canRemoveSpotlight = capabilities === null || capabilities === void 0 ? void 0 : capabilities.removeParticipantsSpotlight.isPresent;
    const stopAllSpotlight = React.useMemo(() => canRemoveSpotlight ? () => adapter.stopAllSpotlight() : undefined, [canRemoveSpotlight, adapter]);
    const { stopAllSpotlightWithPrompt } = useStopAllSpotlightCallbackWithPrompt(stopAllSpotlight, setIsPromptOpen, setPromptProps);
    const onMuteParticipantPeoplePaneProps = React.useMemo(() => {
        var _a, _b;
        return {
            onMuteParticipant: ((_a = capabilities === null || capabilities === void 0 ? void 0 : capabilities.muteOthers) === null || _a === void 0 ? void 0 : _a.isPresent) || role === 'Unknown' ? onMuteParticipant : undefined,
            onMuteAllRemoteParticipants: ((_b = capabilities === null || capabilities === void 0 ? void 0 : capabilities.muteOthers) === null || _b === void 0 ? void 0 : _b.isPresent) || role === 'Unknown' ? muteAllHandlers.onMuteAllRemoteParticipants : undefined
        };
    }, [onMuteParticipant, role, capabilities === null || capabilities === void 0 ? void 0 : capabilities.muteOthers, muteAllHandlers.onMuteAllRemoteParticipants]);
    const spotlightPeoplePaneProps = React.useMemo(() => {
        return {
            spotlightedParticipantUserIds: spotlightedParticipants,
            onStartLocalSpotlight: hideSpotlightButtons ? undefined : onStartLocalSpotlightWithPrompt,
            onStopLocalSpotlight: hideSpotlightButtons ? undefined : onStopLocalSpotlightWithPrompt,
            onStartRemoteSpotlight: hideSpotlightButtons ? undefined : onStartRemoteSpotlightWithPrompt,
            onStopRemoteSpotlight: hideSpotlightButtons ? undefined : onStopRemoteSpotlightWithPrompt,
            onStopAllSpotlight: hideSpotlightButtons ? undefined : stopAllSpotlightWithPrompt,
            maxParticipantsToSpotlight
        };
    }, [hideSpotlightButtons, maxParticipantsToSpotlight, onStartLocalSpotlightWithPrompt, onStartRemoteSpotlightWithPrompt, onStopLocalSpotlightWithPrompt, onStopRemoteSpotlightWithPrompt, stopAllSpotlightWithPrompt, spotlightedParticipants]);
    const meetingMediaAccess = useSelector$1(getMediaAccessSetting);
    const canForbidOthersAudio = !!((_b = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersAudio) === null || _b === void 0 ? void 0 : _b.isPresent);
    const canForbidOthersVideo = !!((_c = capabilities === null || capabilities === void 0 ? void 0 : capabilities.forbidOthersVideo) === null || _c === void 0 ? void 0 : _c.isPresent);
    const onToggleParticipantMicPeoplePaneProps = React.useMemo(() => {
        return {
            onForbidAudio: canForbidOthersAudio ? onForbidAudio : undefined,
            onPermitAudio: canForbidOthersAudio ? onPermitAudio : undefined,
            onForbidOthersAudio: canForbidOthersAudio ? muteAllHandlers.onForbidOthersAudio : undefined,
            onPermitOthersAudio: canForbidOthersAudio ? muteAllHandlers.onPermitOthersAudio : undefined,
            onForbidVideo: canForbidOthersVideo ? onForbidVideo : undefined,
            onPermitVideo: canForbidOthersVideo ? onPermitVideo : undefined,
            onForbidOthersVideo: canForbidOthersVideo ? muteAllHandlers.onForbidOthersVideo : undefined,
            onPermitOthersVideo: canForbidOthersVideo ? muteAllHandlers.onPermitOthersVideo : undefined,
            meetingMediaAccess
        };
    }, [canForbidOthersAudio, onForbidAudio, onPermitAudio, muteAllHandlers.onForbidOthersAudio, muteAllHandlers.onPermitOthersAudio, muteAllHandlers.onForbidOthersVideo, muteAllHandlers.onPermitOthersVideo, canForbidOthersVideo, onForbidVideo, onPermitVideo, meetingMediaAccess]);
    const { isPeoplePaneOpen, openPeoplePane, closePeoplePane } = usePeoplePane(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, peoplePaneProps), spotlightPeoplePaneProps), onMuteParticipantPeoplePaneProps), pinPeoplePaneProps), onToggleParticipantMicPeoplePaneProps));
    const togglePeoplePane = React.useCallback(() => {
        if (isPeoplePaneOpen) {
            closePeoplePane();
        }
        else {
            openPeoplePane();
        }
    }, [closePeoplePane, isPeoplePaneOpen, openPeoplePane]);
    React.useEffect(() => {
        if (isInLocalHold) {
            // close side pane on local hold
            updateSidePaneRenderer(undefined);
        }
    }, [updateSidePaneRenderer, isInLocalHold, isPeoplePaneOpen, closePeoplePane]);
    const isSidePaneOpen = useIsSidePaneOpen();
    const [renderGallery, setRenderGallery] = React.useState(!isSidePaneOpen && props.mobileView);
    React.useEffect(() => {
        if (isSidePaneOpen && props.mobileView) {
            setRenderGallery(false);
        }
        else {
            setRenderGallery(true);
        }
    }, [props.mobileView, isSidePaneOpen]);
    const modalStrings = {
        dismissModalAriaLabel: locale.strings.call.dismissModalAriaLabel
    };
    const isMobileWithActivePane = props.mobileView && isSidePaneOpen;
    const callCompositeContainerCSS = React.useMemo(() => {
        return {
            display: isMobileWithActivePane ? 'none' : 'flex',
            minWidth: props.mobileView ? 'unset' : `${compositeMinWidthRem}rem`,
            width: '100%',
            height: '100%',
            position: 'relative'
        };
    }, [isMobileWithActivePane, props.mobileView]);
    const onResolveVideoEffectDependency = useSelector$1(getVideoEffectsDependency);
    const { openVideoEffectsPane, isVideoEffectsPaneOpen } = useVideoEffectsPane(props.updateSidePaneRenderer, props.mobileView, props.latestErrors, props.onDismissError, cameraButtonRef);
    const [showDrawer, setShowDrawer] = React.useState(false);
    const onMoreButtonClicked = React.useCallback(() => {
        setShowDrawer(true);
    }, []);
    const closeDrawer = React.useCallback(() => {
        setShowDrawer(false);
    }, []);
    const onMoreDrawerPeopleClicked = React.useCallback(() => {
        setShowDrawer(false);
        togglePeoplePane();
    }, [togglePeoplePane]);
    const drawerContainerStylesValue = React.useMemo(() => drawerContainerStyles(DRAWER_Z_INDEX), []);
    let filteredLatestErrorNotifications = props.showErrorNotifications ? props.latestErrors : [];
    const isCameraOn = useSelector$1(localVideoSelector).isAvailable;
    // TODO: move this logic to the error bar selector once role is plumbed from the headless SDK
    if (role === 'Consumer' && (props.errorBarProps || props.showErrorNotifications)) {
        filteredLatestErrorNotifications = filteredLatestErrorNotifications.filter(e => e.type !== 'callCameraAccessDenied' && e.type !== 'callCameraAccessDeniedSafari');
    }
    if (props.doNotShowCameraAccessNotifications) {
        filteredLatestErrorNotifications = filteredLatestErrorNotifications.filter(e => e.type !== 'callCameraAccessDenied' && e.type !== 'callCameraAccessDeniedSafari');
    }
    const isVideoPaneOpen = useIsParticularSidePaneOpen(VIDEO_EFFECTS_SIDE_PANE_ID);
    if ((isVideoPaneOpen || !isCameraOn) && (props.errorBarProps || props.showErrorNotifications)) {
        filteredLatestErrorNotifications = filteredLatestErrorNotifications.filter(e => e.type !== 'unableToStartVideoEffect');
    }
    const isTeamsMeeting = useSelector$1(getIsTeamsMeeting);
    const isTeamsCall = useSelector$1(getIsTeamsCall);
    // Teams captions are only available in Teams meetings or Teams calls.
    // For Teams Meetings, we need to check if the capability is present.
    const useTeamsCaptions = isTeamsMeeting && ((_d = capabilities === null || capabilities === void 0 ? void 0 : capabilities.setCaptionLanguage) === null || _d === void 0 ? void 0 : _d.isPresent) || isTeamsCall;
    const hasJoinedCall = useSelector$1(getCallStatus) === 'Connected';
    const isCaptionsOn = useSelector$1(getCaptionsStatus);
    const minMaxDragPosition = useMinMaxDragPosition(props.modalLayerHostId);
    const pipStyles = React.useMemo(() => getPipStyles(theme), [theme]);
    const galleryContainerStyles = React.useMemo(() => {
        var _a;
        return Object.assign(Object.assign({}, mediaGalleryContainerStyles$1), (((_a = props === null || props === void 0 ? void 0 : props.captionsOptions) === null || _a === void 0 ? void 0 : _a.height) === 'full' ? {
            root: {
                postion: 'absolute'
            }
        } : {}));
    }, [(_e = props === null || props === void 0 ? void 0 : props.captionsOptions) === null || _e === void 0 ? void 0 : _e.height]);
    if (isTeamsMeeting) {
        filteredLatestErrorNotifications.filter(notification => notification.type === 'teamsMeetingCallNetworkQualityLow').forEach(notification => {
            notification.onClickPrimaryButton = props.mobileView ? toggleMeetingPhoneInfoPane : toggleTeamsMeetingConferenceModal;
        });
    }
    const verticalControlBar = props.mobileView && containerWidth && containerHeight && containerWidth / containerHeight > 1 ? true : false;
    // Filter out shareScreen capability notifications if on mobile
    const filteredCapabilitesChangedNotifications = props.mobileView ? (_f = props.capabilitiesChangedNotificationBarProps) === null || _f === void 0 ? void 0 : _f.capabilitiesChangedNotifications.filter(notification => notification.capabilityName !== 'shareScreen') : (_g = props.capabilitiesChangedNotificationBarProps) === null || _g === void 0 ? void 0 : _g.capabilitiesChangedNotifications;
    const notificationStackStrings = useCompositeStringsForNotificationStackStrings(locale);
    let latestNotifications = props.latestNotifications;
    // Filter out breakout room notification that prompts user to join breakout room when in mobile view. We will
    // replace it with a non-dismissible banner
    latestNotifications = props.mobileView ? (latestNotifications !== null && latestNotifications !== void 0 ? latestNotifications : []).filter(notification => notification.type !== 'assignedBreakoutRoomOpenedPromptJoin') : latestNotifications;
    const reactionResources = useSelector$1(getReactionResources);
    const [openRealTimeText, setOpenRealTimeText] = React.useState(false);
    const onStartRealTimeText = React.useCallback(() => {
        setOpenRealTimeText(true);
    }, []);
    return React.createElement("div", { ref: containerRef, className: react.mergeStyles(containerDivStyles), id: props.id },
        React.createElement(react.Stack, { verticalFill: true, horizontalAlign: "stretch", className: containerClassName, "data-ui-id": props.dataUiId },
            React.createElement(react.Stack, { reversed: true, horizontal: verticalControlBar, grow: true, styles: callArrangementContainerStyles(verticalControlBar) },
                React.createElement(react.Stack, { horizontal: true, grow: true },
                    React.createElement(react.Stack.Item, { style: callCompositeContainerCSS },
                        React.createElement(react.Stack.Item, { styles: callGalleryStyles, grow: true },
                            React.createElement(react.Stack, { verticalFill: true, styles: galleryContainerStyles },
                                !((_h = props.notificationOptions) === null || _h === void 0 ? void 0 : _h.hideAllNotifications) && React.createElement(react.Stack.Item, { styles: notificationsContainerStyles },
                                    props.mobileView && React.createElement(BreakoutRoomsBanner, { locale: locale, adapter: adapter }),
                                    props.showErrorNotifications && React.createElement(react.Stack, { styles: notificationStackStyles, horizontalAlign: "center", verticalAlign: "center" },
                                        React.createElement(NotificationStack, { onDismissNotification: props.onDismissError, activeNotifications: filteredLatestErrorNotifications })),
                                    latestNotifications && React.createElement(react.Stack, { styles: notificationStackStyles, horizontalAlign: "center", verticalAlign: "center" },
                                        React.createElement(NotificationStack, { activeNotifications: latestNotifications, onDismissNotification: props.onDismissNotification, strings: notificationStackStrings })),
                                    props.capabilitiesChangedNotificationBarProps && props.capabilitiesChangedNotificationBarProps.capabilitiesChangedNotifications.length > 0 && React.createElement(react.Stack, { styles: bannerNotificationStyles },
                                        React.createElement(CapabilitiesChangedNotificationBar, Object.assign({}, props.capabilitiesChangedNotificationBarProps, { capabilitiesChangedNotifications: filteredCapabilitesChangedNotifications !== null && filteredCapabilitesChangedNotifications !== void 0 ? filteredCapabilitesChangedNotifications : [] })))),
                                renderGallery && props.onRenderGalleryContent && props.onRenderGalleryContent(),
                                !isInLocalHold && React.createElement(CallingCaptionsBanner, { captionsOptions: props.captionsOptions, isMobile: props.mobileView, onFetchAvatarPersonaData: props.onFetchAvatarPersonaData, useTeamsCaptions: useTeamsCaptions, isRealTimeTextOn: openRealTimeText })))),
                    React.createElement(SidePane, { mobileView: props.mobileView, maxWidth: isVideoPaneOpen ? `${VIDEO_EFFECTS_SIDE_PANE_WIDTH_REM}rem` : undefined, minWidth: isVideoPaneOpen ? `${VIDEO_EFFECTS_SIDE_PANE_WIDTH_REM}rem` : undefined, updateSidePaneRenderer: props.updateSidePaneRenderer, onPeopleButtonClicked: props.mobileView && !shouldShowPeopleTabHeaderButton(props.callControlProps.options) ? undefined : togglePeoplePane, disablePeopleButton: typeof props.callControlProps.options !== 'boolean' && isDisabled$4((_j = props.callControlProps.options) === null || _j === void 0 ? void 0 : _j.participantsButton), onChatButtonClicked: (_k = props.mobileChatTabHeader) === null || _k === void 0 ? void 0 : _k.onClick, disableChatButton: (_l = props.mobileChatTabHeader) === null || _l === void 0 ? void 0 : _l.disabled, showAddPeopleButton: !!props.callControlProps.callInvitationURL, ariaLabel: isVideoEffectsPaneOpen ? locale.strings.call.videoEffectsPaneAriaLabel : undefined }),
                    props.mobileView && React.createElement(ModalLocalAndRemotePIP, { modalLayerHostId: props.modalLayerHostId, hidden: !isSidePaneOpen, styles: pipStyles, strings: modalStrings, minDragPosition: minMaxDragPosition.minDragPosition, maxDragPosition: minMaxDragPosition.maxDragPosition, onDismissSidePane: () => {
                            closePeoplePane();
                            if (props.onCloseChatPane) {
                                props.onCloseChatPane();
                            }
                        } }),
                    drawerMenuItems.length > 0 && React.createElement(react.Stack, { styles: drawerContainerStyles() },
                        React.createElement(_DrawerMenu, { onLightDismiss: () => setDrawerMenuItems([]), items: drawerMenuItems }))),
                ((_m = props.callControlProps) === null || _m === void 0 ? void 0 : _m.options) !== false && !isMobileWithActivePane && React.createElement(react.Stack, { verticalAlign: 'center', className: react.mergeStyles({
                        zIndex: CONTROL_BAR_Z_INDEX,
                        padding: verticalControlBar ? '0.25rem' : 'unset'
                    }) }, isLegacyCallControlEnabled((_o = props.callControlProps) === null || _o === void 0 ? void 0 : _o.options) ? React.createElement(CallControls, Object.assign({}, props.callControlProps, { containerWidth: containerWidth, containerHeight: containerHeight, isMobile: props.mobileView, peopleButtonChecked: isPeoplePaneOpen, onPeopleButtonClicked: togglePeoplePane, displayVertical: verticalControlBar })) : React.createElement(CommonCallControlBar, Object.assign({}, props.callControlProps, { callControls: props.callControlProps.options, callAdapter: adapter, mobileView: props.mobileView, disableButtonsForLobbyPage: isInLobby, disableButtonsForHoldScreen: isInLocalHold, peopleButtonChecked: isPeoplePaneOpen, onPeopleButtonClicked: togglePeoplePane, onMoreButtonClicked: onMoreButtonClicked, isCaptionsSupported: useTeamsCaptions && hasJoinedCall || hasJoinedCall, isRealTimeTextSupported: hasJoinedCall && props.isRTTSupportedCall, onStartRealTimeText: onStartRealTimeText, startRealTimeTextButtonChecked: openRealTimeText, useTeamsCaptions: useTeamsCaptions, isCaptionsOn: isCaptionsOn, onClickVideoEffects: onResolveVideoEffectDependency ? openVideoEffectsPane : undefined, displayVertical: verticalControlBar, onUserSetOverflowGalleryPositionChange: props.onUserSetOverflowGalleryPositionChange, onUserSetGalleryLayout: props.onUserSetGalleryLayoutChange, userSetGalleryLayout: props.userSetGalleryLayout, onSetDialpadPage: props.onSetDialpadPage, dtmfDialerPresent: props.dtmfDialerPresent, peopleButtonRef: peopleButtonRef, cameraButtonRef: cameraButtonRef, onStopLocalSpotlight: !hideSpotlightButtons && localParticipant.spotlight ? onStopLocalSpotlightWithPrompt : undefined, onToggleTeamsMeetingConferenceModal: toggleTeamsMeetingConferenceModal, teamsMeetingConferenceModalPresent: showTeamsMeetingConferenceModal, sidePaneDismissButtonRef: sidePaneDismissButtonRef }))),
                ((_p = props.callControlProps) === null || _p === void 0 ? void 0 : _p.options) !== false && showDrawer && React.createElement(react.Stack, { styles: drawerContainerStylesValue },
                    React.createElement(PreparedMoreDrawer, { callControls: props.callControlProps.options, onLightDismiss: closeDrawer, onPeopleButtonClicked: onMoreDrawerPeopleClicked, disableButtonsForHoldScreen: isInLocalHold, isCaptionsSupported: useTeamsCaptions && hasJoinedCall || hasJoinedCall, isRealTimeTextSupported: hasJoinedCall && props.isRTTSupportedCall, onStartRealTimeText: onStartRealTimeText, startRealTimeTextButtonChecked: openRealTimeText, useTeamsCaptions: useTeamsCaptions, onUserSetGalleryLayout: props.onUserSetGalleryLayoutChange, userSetGalleryLayout: props.userSetGalleryLayout, onSetDialpadPage: props.onSetDialpadPage, dtmfDialerPresent: props.dtmfDialerPresent, dtmfDialerOptions: props.dtmfDialerOptions, reactionResources: reactionResources, onClickMeetingPhoneInfo: onMeetingPhoneInfoClicked })))));
};
const isLegacyCallControlEnabled = (options) => {
    return !!options && options !== true && (options === null || options === void 0 ? void 0 : options.legacyControlBarExperience) === true;
};
const shouldShowPeopleTabHeaderButton = (callControls) => {
    if (callControls === undefined || callControls === true) {
        return true;
    }
    if (callControls === false) {
        return false;
    }
    return callControls.participantsButton !== false && callControls.peopleButton !== false;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Provides data attributes to {@link LocalVideoCameraCycleButton} component.
 * @public
 */
const localVideoCameraCycleButtonSelector = reselect.createSelector([getDeviceManager], deviceManager => {
    return {
        cameras: deviceManager.cameras,
        selectedCamera: deviceManager.selectedCamera
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const VideoGalleryStyles = {
    root: {
        height: '100%',
        minHeight: '10rem',
        // space affordance to ensure media gallery is never collapsed
        minWidth: '6rem'
    }
};
const localVideoViewOptions$2 = {
    scalingMode: 'Crop',
    isMirrored: true
};
const remoteVideoViewOptions = {
    scalingMode: 'Crop'
};
/**
 * @private
 */
const MediaGallery = (props) => {
    var _a, _b;
    const { pinnedParticipants = [], setPinnedParticipants, setIsPromptOpen, setPromptProps, hideSpotlightButtons, videoTilesOptions, captionsOptions, compositeContainerAspectRatio } = props;
    const videoGalleryProps = usePropsFor$1(VideoGallery);
    const cameraSwitcherCameras = useSelector$1(localVideoCameraCycleButtonSelector);
    const cameraSwitcherCallback = useHandlers();
    const announcerString = useParticipantChangedAnnouncement();
    const userRole = useSelector$1(getRole);
    const capabilities = useSelector$1(getCapabilites);
    const isRoomsCall = useSelector$1(getIsRoomsCall);
    const reactionResources = useSelector$1(getReactionResources);
    const containerRef = React.useRef(null);
    const containerWidth = _useContainerWidth(containerRef);
    const containerHeight = _useContainerHeight(containerRef);
    const layoutBasedOnTilePosition = getVideoGalleryLayoutBasedOnLocalOptions((_a = props.localVideoTileOptions) === null || _a === void 0 ? void 0 : _a.position);
    const cameraSwitcherProps = React.useMemo(() => {
        return Object.assign(Object.assign({}, cameraSwitcherCallback), cameraSwitcherCameras);
    }, [cameraSwitcherCallback, cameraSwitcherCameras]);
    const onRenderAvatar = React.useCallback((userId, options) => {
        return React.createElement(react.Stack, { className: react.mergeStyles({
                position: 'absolute',
                height: '100%',
                width: '100%'
            }) },
            React.createElement(react.Stack, { styles: {
                    root: {
                        margin: 'auto',
                        maxHeight: '100%'
                    }
                } }, (options === null || options === void 0 ? void 0 : options.coinSize) && React.createElement(AvatarPersona, Object.assign({ userId: userId }, options, { dataProvider: props.onFetchAvatarPersonaData }))));
    }, [props.onFetchAvatarPersonaData]);
    const remoteVideoTileMenuOptions = React.useMemo(() => {
        var _a;
        return ((_a = props.remoteVideoTileMenuOptions) === null || _a === void 0 ? void 0 : _a.isHidden) ? false : props.isMobile ? {
            kind: 'drawer',
            hostId: props.drawerMenuHostId
        } : {
            kind: 'contextual'
        };
    }, [(_b = props.remoteVideoTileMenuOptions) === null || _b === void 0 ? void 0 : _b.isHidden, props.isMobile, props.drawerMenuHostId]);
    const overflowGalleryPosition = React.useMemo(() => {
        return containerWidth && containerHeight && containerWidth / containerHeight >= 16 / 9 ? 'verticalRight' : 'horizontalBottom';
    }, [containerWidth, containerHeight]);
    const { onStartLocalSpotlight, onStopLocalSpotlight, onStartRemoteSpotlight, onStopRemoteSpotlight } = videoGalleryProps;
    const { onStartLocalSpotlightWithPrompt, onStopLocalSpotlightWithPrompt } = useLocalSpotlightCallbacksWithPrompt(onStartLocalSpotlight, onStopLocalSpotlight, setIsPromptOpen, setPromptProps);
    const { onStartRemoteSpotlightWithPrompt, onStopRemoteSpotlightWithPrompt } = useRemoteSpotlightCallbacksWithPrompt(onStartRemoteSpotlight, onStopRemoteSpotlight, setIsPromptOpen, setPromptProps);
    const galleryStyles = React.useMemo(() => {
        return Object.assign(Object.assign({}, VideoGalleryStyles), ((captionsOptions === null || captionsOptions === void 0 ? void 0 : captionsOptions.height) === 'full' ? {
            root: {
                postion: 'absolute'
            }
        } : {}));
    }, [captionsOptions === null || captionsOptions === void 0 ? void 0 : captionsOptions.height]);
    const onPinParticipant = React.useMemo(() => {
        return setPinnedParticipants ? (userId) => {
            if (!pinnedParticipants.includes(userId)) {
                setPinnedParticipants(pinnedParticipants.concat(userId));
            }
        } : undefined;
    }, [setPinnedParticipants, pinnedParticipants]);
    const onUnpinParticipant = React.useMemo(() => {
        return setPinnedParticipants ? (userId) => {
            setPinnedParticipants(pinnedParticipants.filter(participantId => participantId !== userId));
        } : undefined;
    }, [setPinnedParticipants, pinnedParticipants]);
    const VideoGalleryMemoized = React.useMemo(() => {
        var _a;
        const layoutBasedOnUserSelection = () => {
            return props.localVideoTileOptions ? layoutBasedOnTilePosition : props.userSetGalleryLayout;
        };
        return React.createElement(VideoGallery, Object.assign({}, videoGalleryProps, { videoTilesOptions: videoTilesOptions, localVideoViewOptions: localVideoViewOptions$2, remoteVideoViewOptions: remoteVideoViewOptions, styles: galleryStyles, layout: layoutBasedOnUserSelection(), showCameraSwitcherInLocalPreview: props.isMobile, localVideoCameraCycleButtonProps: cameraSwitcherProps, onRenderAvatar: onRenderAvatar, remoteVideoTileMenu: remoteVideoTileMenuOptions, overflowGalleryPosition: overflowGalleryPosition, localVideoTileSize: props.localVideoTileOptions === false || userRole === 'Consumer' || isRoomsCall && userRole === 'Unknown' ? 'hidden' : props.isMobile && compositeContainerAspectRatio && compositeContainerAspectRatio < 1 ? '9:16' : '16:9', pinnedParticipants: pinnedParticipants, onPinParticipant: onPinParticipant, onUnpinParticipant: onUnpinParticipant, reactionResources: reactionResources, onStartLocalSpotlight: hideSpotlightButtons ? undefined : onStartLocalSpotlightWithPrompt, onStopLocalSpotlight: hideSpotlightButtons ? undefined : onStopLocalSpotlightWithPrompt, onStartRemoteSpotlight: hideSpotlightButtons ? undefined : onStartRemoteSpotlightWithPrompt, onStopRemoteSpotlight: hideSpotlightButtons ? undefined : onStopRemoteSpotlightWithPrompt, onMuteParticipant: ((_a = capabilities === null || capabilities === void 0 ? void 0 : capabilities.muteOthers) === null || _a === void 0 ? void 0 : _a.isPresent) || userRole === 'Unknown' ? videoGalleryProps.onMuteParticipant : undefined, localScreenShareView: props.localScreenShareView }));
    }, [videoGalleryProps, videoTilesOptions, galleryStyles, props.isMobile, props.localVideoTileOptions, props.userSetGalleryLayout, cameraSwitcherProps, onRenderAvatar, remoteVideoTileMenuOptions, overflowGalleryPosition, userRole, isRoomsCall, pinnedParticipants, onPinParticipant, onUnpinParticipant, reactionResources, hideSpotlightButtons, onStartLocalSpotlightWithPrompt, onStopLocalSpotlightWithPrompt, onStartRemoteSpotlightWithPrompt, onStopRemoteSpotlightWithPrompt, layoutBasedOnTilePosition, capabilities === null || capabilities === void 0 ? void 0 : capabilities.muteOthers, props.localScreenShareView, compositeContainerAspectRatio]);
    return React.createElement("div", { ref: containerRef, style: mediaGalleryContainerStyles },
        React.createElement(Announcer$1, { announcementString: announcerString, ariaLive: 'polite' }),
        VideoGalleryMemoized);
};
const mediaGalleryContainerStyles = {
    width: '100%',
    height: '100%'
};
const getVideoGalleryLayoutBasedOnLocalOptions = (localTileOptions) => {
    return localTileOptions === 'grid' ? 'default' : 'floatingLocalVideo';
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// By default, absolute positioned elements are overlayed over static ones.
// So we need to explicitly layer the background and contents.
const OVERLAY_BACKGROUND_ZINDEX = 0;
const OVERLAY_CONTENT_ZINDEX = OVERLAY_BACKGROUND_ZINDEX + 1;
/**
 * @private
 */
const containerStyles = {
    root: {
        height: '100%',
        width: '100%',
        position: 'relative',
        padding: '0.5rem'
    }
};
/**
 * @private
 */
const overlayBackgroundStyles = isVideoReady => {
    return {
        position: 'absolute',
        height: '100%',
        width: '100%',
        background: isVideoReady ? 'black' : undefined,
        opacity: isVideoReady ? 0.6 : undefined,
        zIndex: OVERLAY_BACKGROUND_ZINDEX
    };
};
/**
 * @private
 */
const overlayContentStyles = {
    height: '100%',
    width: '100%',
    zIndex: OVERLAY_CONTENT_ZINDEX
};
/**
 * @private
 */
const videoTileStyles = {
    root: {
        height: '100%',
        width: '100%'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const onRenderEmptyPlaceholder = () => React.createElement(React.Fragment, null);
const localVideoViewOptions$1 = {
    scalingMode: 'Crop',
    isMirrored: true
};
/**
 * @private
 */
const ExpandedLocalVideoTile = (props) => {
    var _a;
    const videoStream = props.localParticipantVideoStream;
    const isVideoReady = (_a = videoStream === null || videoStream === void 0 ? void 0 : videoStream.isAvailable) !== null && _a !== void 0 ? _a : false;
    const renderElement = videoStream === null || videoStream === void 0 ? void 0 : videoStream.renderElement;
    React.useEffect(() => {
        if (videoStream && isVideoReady) {
            props.onCreateLocalStreamView && props.onCreateLocalStreamView(localVideoViewOptions$1).catch(err => console.log('Can not render video', err));
        }
    }, [isVideoReady, videoStream, props, renderElement]);
    return React.createElement(react.Stack, { horizontalAlign: "center", verticalAlign: "center", styles: containerStyles, grow: true },
        React.createElement(VideoTile, { styles: videoTileStyles, renderElement: renderElement ? React.createElement(StreamMedia, { videoStreamElement: renderElement }) : undefined, onRenderPlaceholder: onRenderEmptyPlaceholder }, props.overlayContent && React.createElement(React.Fragment, null,
            React.createElement(react.Stack, { className: react.mergeStyles(overlayBackgroundStyles(isVideoReady)) }),
            React.createElement(react.Stack, { horizontalAlign: "center", verticalAlign: "center", className: react.mergeStyles(overlayContentStyles) }, props.overlayContent))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const containerStyle$1 = {
    // Ensure some space around the text on a narrow viewport.
    margin: '1rem',
    gap: `1.5rem`
};
/**
 * @private
 */
const headerContainerStyle = {
    gap: `1.5rem`,
    height: `auto`
};
/**
 * @private
 */
const titleContainerStyle = {
    gap: `1rem`
};
/**
 * @private
 */
const titleStyle$1 = (palette, isVideoReady) => ({
    fontSize: '1.25rem',
    fontWeight: react.FontWeights.semibold,
    color: isVideoReady ? 'white' : palette.neutralPrimary,
    textAlign: 'center'
});
/**
 * @private
 */
const moreDetailsStyle$1 = (palette, isVideoReady) => ({
    fontSize: '1rem',
    color: isVideoReady ? 'white' : palette.neutralPrimary,
    textAlign: 'center'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const NetworkReconnectTile = (props) => {
    var _a;
    const videoStream = props.localParticipantVideoStream;
    const isVideoReady = (_a = videoStream === null || videoStream === void 0 ? void 0 : videoStream.isAvailable) !== null && _a !== void 0 ? _a : false;
    const palette = useTheme().palette;
    const strings = useLocale().strings.call;
    const localeStrings = useLocale().component.strings.meetingConferencePhoneInfo;
    const theme = useTheme();
    const handlers = useHandlers();
    const isTeamsMeeting = useSelector$1(getIsTeamsMeeting);
    const meetingCoordinates = useSelector$1(getTeamsMeetingCoordinates);
    return React.createElement(ExpandedLocalVideoTile, Object.assign({ localParticipantVideoStream: props.localParticipantVideoStream, overlayContent: React.createElement(react.Stack, { verticalFill: true, horizontalAlign: "center", verticalAlign: "center", className: react.mergeStyles(containerStyle$1), "aria-atomic": true },
            React.createElement(react.Stack, { role: "alert", verticalFill: true, horizontalAlign: "center", verticalAlign: "center", className: react.mergeStyles(headerContainerStyle) },
                React.createElement(react.Stack, { horizontal: true, className: react.mergeStyles(titleContainerStyle) },
                    React.createElement(CallCompositeIcon, { iconName: "NetworkReconnectIcon", className: react.mergeStyles(titleStyle$1(palette, isVideoReady)) }),
                    React.createElement(react.Text, { className: react.mergeStyles(titleStyle$1(palette, isVideoReady)) }, strings.networkReconnectTitle)),
                React.createElement(react.Text, { className: react.mergeStyles(moreDetailsStyle$1(palette, isVideoReady)) }, strings.networkReconnectMoreDetails)),
            isTeamsMeeting && meetingCoordinates && meetingCoordinates[0] && React.createElement(react.Stack, null,
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "center", verticalAlign: "center", className: titleContainerClassName },
                    React.createElement(react.Text, { className: titleClassName }, localeStrings.meetingConferencePhoneInfoModalTitle)),
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", className: phoneInfoInstructionLine },
                    React.createElement(react.Stack, { className: infoConnectionLinkStyle(theme) }),
                    React.createElement(react.Stack.Item, null,
                        React.createElement(react.Stack, { horizontal: true, className: phoneInfoStep },
                            React.createElement(react.Stack.Item, { className: phoneInfoIcon(theme) },
                                React.createElement(react.Stack, { verticalAlign: "center", horizontalAlign: "center" },
                                    React.createElement(react.Icon, { iconName: "JoinByPhoneDialStepIcon", className: phoneInfoIconStyle(theme) }))),
                            React.createElement(react.Stack.Item, null,
                                React.createElement(react.Text, { className: phoneInfoLabelStyle }, localeStrings.meetingConferencePhoneInfoModalDialIn)))),
                    React.createElement(react.Stack.Item, { className: phoneInfoStep }, meetingCoordinates.map((phoneNumber, index) => React.createElement(react.Stack.Item, { key: index },
                        React.createElement(react.Text, { className: phoneInfoTextStyle },
                            props.isMobile && React.createElement(react.Link, { className: phoneInfoTextStyle, href: formatPhoneNumberLink(phoneNumber) }, _formatPhoneNumber(phoneNumber.phoneNumber, true)),
                            !props.isMobile && React.createElement(react.Text, { className: phoneInfoTextStyle }, _formatPhoneNumber(phoneNumber.phoneNumber, true)),
                            ' ',
                            phoneNumber.isTollFree ? localeStrings.meetingConferencePhoneInfoModalTollFree : localeStrings.meetingConferencePhoneInfoModalToll),
                        React.createElement("br", null),
                        React.createElement(react.Text, { className: phoneInfoTextStyle },
                            " ",
                            formatPhoneNumberInfo(phoneNumber, localeStrings)))))),
                React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: phoneInfoInstructionLine },
                    React.createElement(react.Stack.Item, null,
                        React.createElement(react.Stack, { horizontal: true },
                            !props.isMobile && React.createElement(react.Stack, { className: infoConnectionLinkStyle(theme) }),
                            React.createElement(react.Stack.Item, { className: phoneInfoIcon(theme) },
                                React.createElement(react.Icon, { iconName: "JoinByPhoneConferenceIdIcon", className: phoneInfoIconStyle(theme) })),
                            React.createElement(react.Stack.Item, null,
                                React.createElement(react.Text, { className: phoneInfoLabelStyle }, localeStrings.meetingConferencePhoneInfoModalMeetingId)))),
                    React.createElement(react.Text, { className: phoneInfoTextStyle },
                        meetingCoordinates[0].conferenceId,
                        "#")),
                !props.isMobile && React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", verticalAlign: "center", className: phoneInfoInstructionLine },
                    React.createElement(react.Stack, { horizontal: true },
                        React.createElement(react.Stack.Item, { className: phoneInfoIcon(theme), style: {
                                marginLeft: _pxToRem(2)
                            } },
                            React.createElement(react.Icon, { iconName: "JoinByPhoneWaitToBeAdmittedIcon", className: phoneInfoIconStyle(theme) })),
                        React.createElement(react.Stack.Item, null,
                            React.createElement(react.Text, { className: phoneInfoLabelStyle }, localeStrings.meetingConferencePhoneInfoModalWait)))))) }, handlers));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const complianceBannerSelector = reselect__namespace.createSelector([getIsTranscriptionActive, getIsRecordingActive], (isTranscriptionActive, isRecordingActive) => {
    return {
        callTranscribeState: isTranscriptionActive,
        callRecordState: isRecordingActive
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const mutedNotificationSelector = reselect__namespace.createSelector([getUserFacingDiagnostics], diagnostics => {
    var _a;
    return {
        speakingWhileMuted: !!((_a = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.media.latest.speakingWhileMicrophoneIsMuted) === null || _a === void 0 ? void 0 : _a.value)
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const lobbySelector = reselect__namespace.createSelector([localVideoSelector], localVideoStreamInfo => {
    return {
        localParticipantVideoStream: localVideoStreamInfo
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const networkReconnectTileSelector = reselect__namespace.createSelector([getUserFacingDiagnostics, lobbySelector], (diagnostics, lobbyProps) => {
    var _a;
    return {
        networkReconnectValue: (_a = diagnostics === null || diagnostics === void 0 ? void 0 : diagnostics.network.latest.networkReconnect) === null || _a === void 0 ? void 0 : _a.value,
        localParticipantVideoStream: lobbyProps.localParticipantVideoStream
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * styles for hold pane timer
 *
 * @private
 */
const DtmfDialpadContentTimerStyles = {
    root: {
        color: 'inherit',
        fontWeight: 600,
        fontSize: _pxToRem(20),
        lineHeight: _pxToRem(28),
        margin: 'auto'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$f = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const DtmfDialpadPageContent = (props) => {
    const theme = react.useTheme();
    const targetCallees = useSelector$1(getTargetCallees);
    const remoteParticipants = useSelector$1(getRemoteParticipants);
    const calleeId = targetCallees === null || targetCallees === void 0 ? void 0 : targetCallees[0];
    let calleeName;
    if (remoteParticipants) {
        const remoteParticipantValues = Object.values(remoteParticipants);
        if (calleeId && communicationCommon.isPhoneNumberIdentifier(calleeId)) {
            calleeName = calleeId.phoneNumber;
        }
        else {
            calleeName = remoteParticipantValues.find(p => p.identifier === calleeId);
        }
    }
    const adapter = useAdapter();
    return React.createElement(react.Stack, { style: {
            height: '100%',
            width: '100%',
            background: theme.palette.white
        } },
        React.createElement(react.Stack, { verticalAlign: 'center', style: {
                margin: 'auto'
            } },
            React.createElement(DtmfDialerContentTimer, null),
            React.createElement(react.Text, { style: {
                    margin: 'auto'
                } }, calleeName && calleeName !== 'Unnamed participant' ? calleeName === null || calleeName === void 0 ? void 0 : calleeName.toString() : ''),
            React.createElement(Dialpad, { onSendDtmfTone: (tone) => __awaiter$f(void 0, void 0, void 0, function* () {
                    yield adapter.sendDtmfTone(tone);
                }), longPressTrigger: props.mobileView ? 'touch' : 'mouseAndTouch', dialpadMode: 'dtmf', dtmfAudioContext: props.compositeAudioContext })));
};
const DtmfDialerContentTimer = () => {
    const [time, setTime] = React.useState(0);
    const elapsedTime = getReadableTime$1(time);
    const statefulStartTime = useSelector$1(getStartTime);
    const startTime = React.useMemo(() => {
        return statefulStartTime !== null && statefulStartTime !== void 0 ? statefulStartTime : new Date(Date.now());
    }, [statefulStartTime]);
    React.useEffect(() => {
        const interval = setInterval(() => {
            var _a;
            setTime(new Date(Date.now()).getTime() - ((_a = startTime === null || startTime === void 0 ? void 0 : startTime.getTime()) !== null && _a !== void 0 ? _a : 0));
        }, 10);
        return () => {
            clearInterval(interval);
        };
    }, [startTime]);
    return React.createElement(react.Text, { styles: DtmfDialpadContentTimerStyles }, elapsedTime);
};
/**
 * @internal
 */
const DtmfDialpadPage = (props) => {
    return React.createElement(DtmfDialpadPageContent, Object.assign({}, props, { mobileView: props.mobileView }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const CallPage = (props) => {
    var _a, _b;
    const { callInvitationURL, onFetchAvatarPersonaData, onFetchParticipantMenuItems, options, mobileView, galleryLayout = 'floatingLocalVideo', onUserSetGalleryLayoutChange, userSetOverflowGalleryPosition = 'Responsive', onSetUserSetOverflowGalleryPosition, onCloseChatPane, pinnedParticipants = [], setPinnedParticipants, compositeAudioContext, disableAutoShowDtmfDialer = {
        dialerBehavior: 'autoShow'
    }, latestNotifications, onDismissNotification } = props;
    // To use useProps to get these states, we need to create another file wrapping Call,
    // It seems unnecessary in this case, so we get the updated states using this approach.
    const { callStatus } = useSelector$1(callStatusSelector);
    const mediaGalleryProps = useSelector$1(mediaGallerySelector);
    const mediaGalleryHandlers = useHandlers();
    const complianceBannerProps = useSelector$1(complianceBannerSelector);
    const errorBarProps = usePropsFor$1(ErrorBar);
    const mutedNotificationProps = useSelector$1(mutedNotificationSelector);
    const networkReconnectTileProps = useSelector$1(networkReconnectTileSelector);
    const remoteParticipantsConnected = useSelector$1(getRemoteParticipantsConnectedSelector);
    const callees = useSelector$1(getTargetCallees);
    const renderDtmfDialerFromStart = showDtmfDialer(callees, remoteParticipantsConnected, disableAutoShowDtmfDialer);
    const [dtmfDialerPresent, setDtmfDialerPresent] = React.useState(renderDtmfDialerFromStart);
    const isPstnCall = callees === null || callees === void 0 ? void 0 : callees.some(callee => communicationCommon.isPhoneNumberIdentifier(callee));
    const isCTECall = useSelector$1(state => state.userId.kind === 'microsoftTeamsUser');
    const containerRef = React.useRef(null);
    const containerWidth = _useContainerWidth(containerRef);
    const containerHeight = _useContainerHeight(containerRef);
    const containerAspectRatio = containerWidth && containerHeight ? containerWidth / containerHeight : 0;
    const strings = useLocale().strings.call;
    const pinnedParticipantsChecked = React.useMemo(() => pinnedParticipants.filter(pinnedParticipant => remoteParticipantsConnected.find(remoteParticipant => toFlatCommunicationIdentifier(remoteParticipant.identifier) === pinnedParticipant)), [pinnedParticipants, remoteParticipantsConnected]);
    // Reduce the controls shown when mobile view is enabled.
    const callControlOptions = mobileView ? reduceCallControlsForMobile(options === null || options === void 0 ? void 0 : options.callControls) : options === null || options === void 0 ? void 0 : options.callControls;
    const drawerMenuHostId = reactHooks.useId('drawerMenuHost');
    const [isPromptOpen, setIsPromptOpen] = React.useState(false);
    const [promptProps, setPromptProps] = React.useState();
    const page = useSelector$1(state => state.page);
    const userId = useSelector$1(state => state.userId);
    const displayName = useSelector$1(state => state.displayName);
    const onRenderAvatar = React.useCallback((userId, options) => {
        return React.createElement(react.Stack, { className: react.mergeStyles({
                position: 'absolute',
                height: '100%',
                width: '100%'
            }) },
            React.createElement(react.Stack, { styles: {
                    root: {
                        margin: 'auto',
                        maxHeight: '100%'
                    }
                } }, (options === null || options === void 0 ? void 0 : options.coinSize) && React.createElement(AvatarPersona, Object.assign({ userId: userId }, options, { dataProvider: props.onFetchAvatarPersonaData }))));
    }, [props.onFetchAvatarPersonaData]);
    let galleryContentWhenNotInCall = React.createElement(React.Fragment, null);
    if (!_isInCall(callStatus) && page === 'returningFromBreakoutRoom') {
        galleryContentWhenNotInCall = React.createElement(VideoTile, { userId: toFlatCommunicationIdentifier(userId), displayName: displayName, initialsName: displayName, onRenderPlaceholder: onRenderAvatar });
    }
    const onRenderGalleryContentTrampoline = () => {
        var _a, _b;
        if (dtmfDialerPresent) {
            return React.createElement(DtmfDialpadPage, { mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, options: props.options, updateSidePaneRenderer: props.updateSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, latestErrors: props.latestErrors, onDismissError: props.onDismissError, capabilitiesChangedNotificationBarProps: props.capabilitiesChangedNotificationBarProps, onSetDialpadPage: () => setDtmfDialerPresent(!dtmfDialerPresent), dtmfDialerPresent: dtmfDialerPresent, compositeAudioContext: compositeAudioContext });
        }
        else {
            return React.createElement(MediaGallery, Object.assign({ isMobile: mobileView }, mediaGalleryProps, mediaGalleryHandlers, { onFetchAvatarPersonaData: onFetchAvatarPersonaData, remoteVideoTileMenuOptions: options === null || options === void 0 ? void 0 : options.remoteVideoTileMenuOptions, drawerMenuHostId: drawerMenuHostId, localVideoTileOptions: options === null || options === void 0 ? void 0 : options.localVideoTile, userSetOverflowGalleryPosition: userSetOverflowGalleryPosition, userSetGalleryLayout: galleryLayout, pinnedParticipants: pinnedParticipantsChecked, setPinnedParticipants: setPinnedParticipants, setIsPromptOpen: setIsPromptOpen, setPromptProps: setPromptProps, hideSpotlightButtons: (_a = options === null || options === void 0 ? void 0 : options.spotlight) === null || _a === void 0 ? void 0 : _a.hideSpotlightButtons, videoTilesOptions: options === null || options === void 0 ? void 0 : options.videoTilesOptions, captionsOptions: options === null || options === void 0 ? void 0 : options.captionsBanner, localScreenShareView: (_b = options === null || options === void 0 ? void 0 : options.galleryOptions) === null || _b === void 0 ? void 0 : _b.localScreenShareView, compositeContainerAspectRatio: containerAspectRatio }));
        }
    };
    return React.createElement("div", { ref: containerRef, style: {
            height: '100%',
            width: '100%'
        } },
        React.createElement(CallArrangement, { id: drawerMenuHostId, complianceBannerProps: Object.assign(Object.assign({}, complianceBannerProps), { strings }), errorBarProps: (options === null || options === void 0 ? void 0 : options.errorBar) !== false && errorBarProps, showErrorNotifications: (_a = options === null || options === void 0 ? void 0 : options.errorBar) !== null && _a !== void 0 ? _a : true, mutedNotificationProps: mutedNotificationProps, callControlProps: {
                callInvitationURL: callInvitationURL,
                onFetchParticipantMenuItems: onFetchParticipantMenuItems,
                options: callControlOptions,
                increaseFlyoutItemSize: mobileView
            }, onFetchAvatarPersonaData: onFetchAvatarPersonaData, mobileView: mobileView, modalLayerHostId: props.modalLayerHostId, onRenderGalleryContent: () => _isInCall(callStatus) ? isNetworkHealthy(networkReconnectTileProps.networkReconnectValue) ? onRenderGalleryContentTrampoline() : React.createElement(NetworkReconnectTile, Object.assign({}, networkReconnectTileProps, { isMobile: mobileView })) : galleryContentWhenNotInCall, updateSidePaneRenderer: props.updateSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, onCloseChatPane: onCloseChatPane, dataUiId: 'call-page', latestErrors: props.latestErrors, latestNotifications: latestNotifications, onDismissError: props.onDismissError, onDismissNotification: onDismissNotification, onUserSetOverflowGalleryPositionChange: onSetUserSetOverflowGalleryPosition, onUserSetGalleryLayoutChange: onUserSetGalleryLayoutChange, userSetGalleryLayout: galleryLayout, capabilitiesChangedNotificationBarProps: props.capabilitiesChangedNotificationBarProps, onSetDialpadPage: () => setDtmfDialerPresent(!dtmfDialerPresent), dtmfDialerPresent: dtmfDialerPresent, setIsPromptOpen: setIsPromptOpen, setPromptProps: setPromptProps, hideSpotlightButtons: (_b = options === null || options === void 0 ? void 0 : options.spotlight) === null || _b === void 0 ? void 0 : _b.hideSpotlightButtons, pinnedParticipants: pinnedParticipantsChecked, setPinnedParticipants: setPinnedParticipants, captionsOptions: options === null || options === void 0 ? void 0 : options.captionsBanner, dtmfDialerOptions: disableAutoShowDtmfDialer, notificationOptions: props.notificationOptions, isRTTSupportedCall: !isPstnCall && !isCTECall }),
        React.createElement(Prompt, Object.assign({ isOpen: isPromptOpen, onDismiss: () => setIsPromptOpen(false) }, promptProps)));
};
/**
 * @private
 */
const isNetworkHealthy = (value) => {
    // We know that the value is actually of type DiagnosticQuality for this diagnostic.
    // We ignore any boolen values, considering the network to still be healthy.
    // Thus, only DiagnosticQuality.Poor or .Bad indicate network problems.
    return value === true || value === false || value === undefined || value === communicationCalling.DiagnosticQuality.Good;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const DROPDOWN_HEIGHT_REM = 2.25;
/**
 * @private
 */
const mainStackTokens = {
    childrenGap: '0.5rem'
};
/**
 * @private
 */
const soundStackTokens = {
    childrenGap: '0.5rem'
};
/**
 * @private
 */
const dropDownStyles = (theme) => ({
    caretDownWrapper: {
        height: `${DROPDOWN_HEIGHT_REM}rem`,
        lineHeight: `${DROPDOWN_HEIGHT_REM}rem`
    },
    dropdownItem: {
        fontSize: '0.875rem',
        height: `${DROPDOWN_HEIGHT_REM}rem`,
        background: theme.palette.white
    },
    dropdown: {
        height: `${DROPDOWN_HEIGHT_REM}rem`,
        width: '100%',
        svg: {
            verticalAlign: 'top'
        }
    },
    title: {
        fontSize: '0.875rem',
        height: `${DROPDOWN_HEIGHT_REM}rem`,
        lineHeight: '2rem',
        borderRadius: '0.25rem',
        border: `1px solid ${theme.palette.neutralQuaternaryAlt}`
    },
    label: {
        fontWeight: 600,
        fontSize: '0.875rem',
        // Add z-index to ensure labels are rendered above the configuration section background
        zIndex: 1
    },
    errorMessage: {
        fontSize: '0.875rem'
    }
});
/**
 * @private
 */
const dropDownTitleIconStyles = react.mergeStyles({
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    minWidth: '0',
    maxWidth: '100%',
    overflowWrap: 'break-word',
    margin: '.063rem'
});
/**
 * @private
 */
const optionIconStyles = react.mergeStyles({
    marginRight: '8px',
    verticalAlign: 'text-top'
});
/**
 * @private
 */
const deviceSelectionContainerStyles = {
    root: {
        /**
         * this is to move the device selection container above the larger border
         * container that is occluding messages
         */
        zIndex: 1
    }
};

/**
 * @private
 */
const ConfigurationPageCameraDropdown = (props) => {
    return props.cameraGrantedDropdown;
};

/**
 * @private
 */
const ConfigurationPageMicDropdown = (props) => {
    return props.micGrantedDropdown;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const CONFIGURATION_PAGE_SECTION_MIN_WIDTH_REM = 11;
/** @private */
const CONFIGURATION_PAGE_SECTION_MAX_WIDTH_REM = 20.625;
/** @private */
const CONFIGURATION_PAGE_SECTION_HEIGHT_REM = 13.625;
const LOGO_HEIGHT_REM = 3;
/**
 * @private
 */
const configurationStackTokensDesktop = {
    childrenGap: '1.5rem'
};
/**
 * @private
 */
const configurationStackTokensMobile = {
    childrenGap: '0.5rem'
};
/**
 * @private
 */
const deviceConfigurationStackTokens = {
    childrenGap: '1.5rem'
};
/** @private */
const configurationContainerStyle = (desktop, backgroundImageUrl) => ({
    display: 'flex',
    flexFlow: 'column',
    height: '100%',
    position: 'relative',
    width: '100%',
    padding: desktop ? '2rem 1rem 2rem 1rem' : '1rem 1rem 2rem 1rem',
    minWidth: desktop ? '20rem' // sum of min-width from children + ChildrenGap * (nb of children - 1) + padding * 2 = (11 + 11) + (2 * 1) + 0.5 * 2
        : '16rem',
    // from LocalPreview: ControlBar width + 0.5 * 2 for spacing + padding * 2 = 14 + 0.5 * 2 + 0.5 * 2
    minHeight: desktop ? '22rem' // max height of SelectionContainer + padding * 2 = 21 + 0.5 * 2
        : '13rem',
    '::before': !backgroundImageUrl ? undefined : Object.assign({ content: '""', position: 'absolute', top: 0, left: 0, zIndex: 0, width: '100%', height: '100%', backgroundImage: `url(${backgroundImageUrl})`, backgroundSize: 'cover', backgroundPosition: 'center' }, react.AnimationStyles.fadeIn500)
});
/**
 * @private
 */
const configurationSectionStyle = {
    root: {
        width: '100%',
        minWidth: `${CONFIGURATION_PAGE_SECTION_MIN_WIDTH_REM}rem`,
        maxWidth: `${CONFIGURATION_PAGE_SECTION_MAX_WIDTH_REM}rem`
    }
};
/**
 * @private
 */
const selectionContainerStyle = (theme, noSpeakerDropdownShown) => react.mergeStyles({
    width: '100%',
    minHeight: noSpeakerDropdownShown ? 'auto' : `${CONFIGURATION_PAGE_SECTION_HEIGHT_REM}rem`,
    padding: '1rem',
    borderRadius: theme.effects.roundedCorner6,
    border: `0.0625rem solid ${theme.palette.neutralLight}`,
    overflow: 'hidden',
    // do not let child background overflow the curved corners
    boxShadow: theme.effects.elevation4,
    // Style the background of the container to have partial transparency.
    // Using `before:` pseudo-element to avoid having to wrap the content in an extra div.
    // Ideally rgba would be used but we cannot garauntee the format of theme.palette.white
    // to parse it correctly.
    position: 'relative',
    ':before': {
        content: '""',
        position: 'absolute',
        top: 0,
        bottom: 0,
        left: 0,
        right: 0,
        zIndex: 0,
        background: theme.palette.white,
        opacity: 0.9
    }
});
/**
 * @private
 */
const titleContainerStyleDesktop = (theme) => react.mergeStyles({
    fontSize: '1.2rem',
    lineHeight: '1rem',
    fontWeight: 600
}, configurationPageTextDecoration(theme));
/**
 * @private
 */
const titleContainerStyleMobile = (theme) => react.mergeStyles({
    fontSize: '1.0625rem',
    lineHeight: '1.375rem',
    fontWeight: 600,
    textAlign: 'center'
}, configurationPageTextDecoration(theme));
/**
 * Ensure configuration page text is legible on top of a background image.
 * @private
 */
const configurationPageTextDecoration = (theme) => {
    return {
        textShadow: `0px 0px 8px ${theme.palette.whiteTranslucent40}`,
        fill: theme.semanticColors.bodyText,
        svg: {
            // Fix SVGs not displaying the correct color in Desert high contrast mode
            fill: 'currentColor'
        },
        stroke: theme.palette.whiteTranslucent40,
        paintOrder: 'stroke fill',
        strokeWidth: _pxToRem(1.5),
        text: {
            letterSpacing: '-0.02rem' // cope with extra width due to stroke width
        },
        '@media (forced-colors: active)': {
            textShadow: 'none',
            stroke: 'none'
        }
    };
};
/**
 * @private
 */
const callDetailsContainerStyles = {
    root: {
        textAlign: 'center',
        maxWidth: _pxToRem(700),
        alignSelf: 'center'
    }
};
const callDetailsStyle = (theme) => ({
    fontSize: '0.9375',
    lineHeight: '1.25rem',
    textShadow: `0px 0px 8px ${theme.palette.whiteTranslucent40}`,
    marginTop: '-0.33rem' // compensate for extra padding around the CallTitle that avoids the SVG shadowing being cut off
});
/**
 * @private
 */
const callDetailsStyleDesktop = (theme) => react.mergeStyles(callDetailsStyle(theme));
/**
 * @private
 */
const callDetailsStyleMobile = (theme) => react.mergeStyles(callDetailsStyle(theme), {
    marginBottom: '0.5rem',
    textAlign: 'center'
});
/**
 * @private
 */
const startCallButtonContainerStyleDesktop = {
    root: {
        paddingTop: '1.125rem'
    }
};
/**
 * @private
 */
const startCallButtonContainerStyleMobile = {
    root: {
        textAlign: 'center'
    }
};
/**
 * @private
 */
const startCallButtonStyleMobile = react.mergeStyles({
    width: '100%',
    maxWidth: 'unset',
    borderRadius: '0.25rem',
    height: '3.25rem'
});
/**
 * @private
 */
const startCallButtonStyleDesktop = react.mergeStyles({
    borderRadius: '0.25rem',
    width: 'auto',
    height: '2.25rem'
});
/** @private */
const cameraAndVideoEffectsContainerStyleDesktop = {
    root: {
        alignItems: 'center'
    }
};
/**
 * @private
 */
const effectsButtonStyles = (theme, disabled) => {
    return {
        root: {
            background: 'transparent',
            border: 'none',
            color: theme.palette.themePrimary,
            // Top and bottom padding needs to be 5px to match the label padding
            padding: '5px 0.25rem',
            ':hover, :focus': disabled ? {} : {
                color: theme.palette.themePrimary
            },
            svg: {
                height: '1rem',
                width: '1rem'
            }
        },
        labelHovered: disabled ? {} : {
            color: theme.palette.themePrimary
        },
        rootChecked: disabled ? {} : {
            color: theme.palette.themePrimary
        },
        rootHovered: disabled ? {} : {
            color: theme.palette.themePrimary
        },
        rootPressed: disabled ? {} : {
            color: theme.palette.themePrimary
        },
        rootFocused: disabled ? {} : {
            color: theme.palette.themePrimary,
            outline: '1px solid',
            outlineOffset: '-2px'
        }
    };
};
/** @private */
const fillWidth = react.mergeStyles({
    width: '100%',
    position: 'relative'
});
/** @private */
const configurationCenteredContent = (fillsHeight, hasLogo) => react.mergeStyles({
    width: '100%',
    position: 'relative',
    // If the content does not fill the height, center it vertically.
    // We do not include the logo in the centering per design. This allows it to fade
    // in and not shift the content. To exclude the logo, we subtract the logo height
    // and margin from the actual height. This allows the flex box's natural centering
    // to appropriately center the content.
    height: `calc(100% - ${!fillsHeight && hasLogo ? `${LOGO_HEIGHT_REM}rem` : '0rem'})`
});
/** @private */
const panelStyles = {
    content: {
        display: 'flex',
        flexBasis: '100%',
        paddingLeft: '0rem',
        // remove default padding
        paddingRight: '0rem' // remove default padding
    },
    commands: {
        paddingTop: '0rem' // remove default padding
    },
    scrollableContent: {
        display: 'flex',
        flexDirection: 'column',
        height: '100%'
    }
};
/** @private */
const panelFocusProps = {
    forceFocusInsideTrap: false
};
/**
 * @private
 */
const logoStyles = (shape) => ({
    root: {
        overflow: 'initial',
        // prevent the image being clipped
        display: 'flex',
        justifyContent: 'center'
    },
    image: {
        height: `${LOGO_HEIGHT_REM}rem`,
        borderRadius: shape === 'circle' ? '100%' : undefined,
        aspectRatio: shape === 'circle' ? '1 / 1' : undefined,
        objectFit: shape === 'circle' ? 'cover' : undefined
    }
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$e = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const getDropDownList = (list) => {
    // Remove duplicates
    const noDuplicates = new Map();
    for (const item of list) {
        noDuplicates.set(item.id, item);
    }
    const dropdownList = [];
    for (const item of noDuplicates.values()) {
        dropdownList.push({
            key: item.id,
            text: item.name === '' ? item.deviceType : item.name
        });
    }
    return dropdownList;
};
const getOptionIcon = (type) => {
    if (type === 'Camera') {
        return React.createElement(CallCompositeIcon, { iconName: "LocalDeviceSettingsCamera", className: optionIconStyles });
    }
    else if (type === 'Microphone') {
        return React.createElement(CallCompositeIcon, { iconName: "LocalDeviceSettingsMic", className: optionIconStyles });
    }
    else if (type === 'Speaker') {
        return React.createElement(CallCompositeIcon, { iconName: "LocalDeviceSettingsSpeaker", className: optionIconStyles });
    }
    else {
        return undefined;
    }
};
const onRenderTitle = (iconType, props) => {
    var _a;
    return props ? React.createElement("div", { className: dropDownTitleIconStyles },
        getOptionIcon(iconType),
        React.createElement("span", null, (_a = props[0]) === null || _a === void 0 ? void 0 : _a.text)) : React.createElement(React.Fragment, null);
};
const localVideoViewOptions = {
    scalingMode: 'Crop',
    isMirrored: true
};
/**
 * @private
 */
const LocalDeviceSettings = (props) => {
    var _a;
    const theme = useTheme();
    const locale = useLocale();
    const adapter = useAdapter();
    const onResolveVideoEffectDependency = useSelector$1(getVideoEffectsDependency);
    const defaultPlaceHolder = locale.strings.call.defaultPlaceHolder;
    const cameraLabel = locale.strings.call.cameraLabel;
    const soundLabel = locale.strings.call.soundLabel;
    const noSpeakersLabel = locale.strings.call.noSpeakersLabel;
    const noCameraLabel = locale.strings.call.noCamerasLabel;
    const noMicLabel = locale.strings.call.noMicrophonesLabel;
    const role = useSelector$1(getRole);
    const cameraPermissionGranted = props.cameraPermissionGranted;
    const micPermissionGranted = props.microphonePermissionGranted;
    const roleCanUseCamera = role !== 'Consumer';
    const roleCanUseMic = role !== 'Consumer';
    // TODO: speaker permission is tied to microphone permission (when you request 'audio' permission using the SDK) its
    // actually granting access to query both microphone and speaker. However the browser popup asks you explicity for
    // 'microphone'. This needs investigation on how we want to handle this and maybe needs follow up with SDK team.
    React.useEffect(() => {
        if (cameraPermissionGranted) {
            adapter.queryCameras();
        }
        if (micPermissionGranted) {
            adapter.queryMicrophones();
        }
        adapter.querySpeakers();
    }, [adapter, cameraPermissionGranted, micPermissionGranted]);
    const hasCameras = props.cameras.length > 0;
    const hasMicrophones = props.microphones.length > 0;
    const hasSpeakers = props.speakers.length > 0;
    const environmentInfo = useSelector$1(getEnvironmentInfo);
    const isSafariWithNoSpeakers = _isSafari(environmentInfo) && !hasSpeakers;
    const cameraLabelId = reactHooks.useId('camera-label');
    const soundLabelId = reactHooks.useId('sound-label');
    const cameraGrantedDropdown = React.createElement(react.Dropdown, { "data-ui-id": "call-composite-local-camera-settings", "aria-labelledby": cameraLabelId, placeholder: hasCameras ? defaultPlaceHolder : noCameraLabel, options: cameraPermissionGranted ? getDropDownList(props.cameras) : [{
                key: 'deniedOrUnknown',
                text: ''
            }], styles: dropDownStyles(theme), disabled: !cameraPermissionGranted || !hasCameras, errorMessage: props.cameraPermissionGranted === undefined || props.cameraPermissionGranted ? undefined : locale.strings.call.cameraPermissionDenied, defaultSelectedKey: micPermissionGranted ? props.selectedCamera ? props.selectedCamera.id : props.cameras ? (_a = props.cameras[0]) === null || _a === void 0 ? void 0 : _a.id : '' : 'deniedOrUnknown', onChange: (event, option, index) => __awaiter$e(void 0, void 0, void 0, function* () {
            const camera = props.cameras[index !== null && index !== void 0 ? index : 0];
            if (camera) {
                yield props.onSelectCamera(camera, localVideoViewOptions);
            }
            else {
                console.error('No cameras available');
            }
        }), onRenderTitle: (props) => onRenderTitle('Camera', props) });
    const micGrantedDropdown = React.createElement(React.Fragment, null, roleCanUseMic && React.createElement(react.Dropdown, { "aria-labelledby": soundLabelId, placeholder: hasMicrophones ? defaultPlaceHolder : noMicLabel, styles: dropDownStyles(theme), disabled: !micPermissionGranted || !hasMicrophones, errorMessage: props.microphonePermissionGranted === undefined || props.microphonePermissionGranted ? undefined : locale.strings.call.microphonePermissionDenied, options: micPermissionGranted ? getDropDownList(props.microphones) : [{
                key: 'deniedOrUnknown',
                text: ''
            }], defaultSelectedKey: micPermissionGranted ? props.selectedMicrophone ? props.selectedMicrophone.id : defaultDeviceId(props.microphones) : 'deniedOrUnknown', onChange: (event, option, index) => {
            const microphone = props.microphones[index !== null && index !== void 0 ? index : 0];
            if (microphone) {
                props.onSelectMicrophone(microphone);
            }
            else {
                console.error('No microphones available');
            }
        }, onRenderTitle: (props) => onRenderTitle('Microphone', props) }));
    const speakerDropdown = React.createElement(react.Dropdown, { "aria-labelledby": soundLabelId, placeholder: hasSpeakers ? defaultPlaceHolder : noSpeakersLabel, styles: dropDownStyles(theme), disabled: props.speakers.length === 0, options: getDropDownList(props.speakers), defaultSelectedKey: props.selectedSpeaker ? props.selectedSpeaker.id : defaultDeviceId(props.speakers), onChange: (event, option, index) => {
            const speaker = props.speakers[index !== null && index !== void 0 ? index : 0];
            if (speaker) {
                props.onSelectSpeaker(speaker);
            }
            else {
                console.error('No speakers available');
            }
        }, onRenderTitle: (props) => onRenderTitle('Speaker', props) });
    return React.createElement(react.Stack, { "data-ui-id": "call-composite-device-settings", tokens: mainStackTokens, styles: deviceSelectionContainerStyles },
        roleCanUseCamera && React.createElement(react.Stack, null,
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "space-between", styles: cameraAndVideoEffectsContainerStyleDesktop },
                React.createElement(react.Label, { id: cameraLabelId, className: react.mergeStyles(dropDownStyles(theme).label) }, cameraLabel),
                onResolveVideoEffectDependency && React.createElement(react.DefaultButton, { iconProps: {
                        iconName: 'ConfigurationScreenVideoEffectsButton'
                    }, styles: effectsButtonStyles(theme, !cameraPermissionGranted), onClick: props.onClickVideoEffects, disabled: !cameraPermissionGranted, "data-ui-id": 'call-config-video-effects-button' }, locale.strings.call.configurationPageVideoEffectsButtonLabel)),
            React.createElement(ConfigurationPageCameraDropdown, { cameraGrantedDropdown: cameraGrantedDropdown, cameraPermissionGranted: cameraPermissionGranted !== null && cameraPermissionGranted !== void 0 ? cameraPermissionGranted : false, ariaLabelledby: cameraLabelId })),
        React.createElement(react.Stack, null,
            React.createElement(react.Label, { id: soundLabelId, className: react.mergeStyles(dropDownStyles(theme).label) }, soundLabel),
            React.createElement(react.Stack, { "data-ui-id": "call-composite-sound-settings", tokens: soundStackTokens },
                React.createElement(ConfigurationPageMicDropdown, { micGrantedDropdown: micGrantedDropdown, micPermissionGranted: micPermissionGranted !== null && micPermissionGranted !== void 0 ? micPermissionGranted : false, ariaLabelledby: soundLabelId }),
                isSafariWithNoSpeakers ? React.createElement(React.Fragment, null) : speakerDropdown)));
};
const defaultDeviceId = (devices) => {
    var _a;
    if (devices.length === 0) {
        return undefined;
    }
    const defaultDevice = devices.find(device => device.isSystemDefault);
    if (defaultDevice) {
        return defaultDevice.id;
    }
    return (_a = devices[0]) === null || _a === void 0 ? void 0 : _a.id;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const videoCameraIconStyle = react.mergeStyles({
    marginRight: '0.375rem',
    fontSize: '1.375rem'
});
/**
 * @private
 */
const buttonStyle = react.mergeStyles({
    fontWeight: 600,
    fontSize: '0.875rem',
    // 14px
    height: '2.75rem',
    width: '100%'
});
/**
 * @private
 */
const buttonWithIconStyles = {
    textContainer: {
        display: 'contents'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const StartCallButton = (props) => {
    const { rejoinCall } = props;
    const locale = useLocale();
    return React.createElement(react.PrimaryButton, Object.assign({}, props, { "data-ui-id": "call-composite-start-call-button", className: react.mergeStyles(buttonStyle, props.className), styles: buttonWithIconStyles, text: rejoinCall ? locale.strings.call.rejoinCallButtonLabel : locale.strings.call.startCallButtonLabel, onRenderIcon: props.hideIcon ? undefined : () => React.createElement(reactIcons.Video20Filled, { className: videoCameraIconStyle }) }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const devicePermissionSelector = reselect__namespace.createSelector([getDeviceManager], deviceManager => {
    return {
        video: deviceManager.deviceAccess ? deviceManager.deviceAccess.video : undefined,
        audio: deviceManager.deviceAccess ? deviceManager.deviceAccess.audio : undefined
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const localPreviewSelector = reselect__namespace.createSelector([getDeviceManager], deviceManager => {
    var _a;
    // TODO: we should take in a LocalVideoStream that developer wants to use as their 'Preview' view. We should also
    // handle cases where 'Preview' view is in progress and not necessary completed.
    const view = deviceManager.cameras.length > 0 &&
        // This ensures that the video stream is reset if camera devices are removed.
        deviceManager.unparentedViews.length > 0 && ((_a = deviceManager.unparentedViews[0]) === null || _a === void 0 ? void 0 : _a.view);
    return {
        videoStreamElement: view ? view.target : null
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const localPreviewContainerStyle = (theme) => ({
    borderRadius: theme.effects.roundedCorner6,
    overflow: 'hidden' // do not let child background overflow the curved corners
});
/**
 * @private
 */
const localPreviewContainerStyleDesktop = (theme) => react.mergeStyles(localPreviewContainerStyle(theme), {
    width: '100%',
    height: `${CONFIGURATION_PAGE_SECTION_HEIGHT_REM}rem`,
    minWidth: `${CONFIGURATION_PAGE_SECTION_MIN_WIDTH_REM}rem`,
    maxWidth: `${CONFIGURATION_PAGE_SECTION_MAX_WIDTH_REM}rem`,
    border: `0.0625rem solid ${theme.palette.neutralLight}`,
    borderRadius: theme.effects.roundedCorner6,
    boxShadow: theme.effects.elevation4
});
/**
 * @private
 */
const localPreviewContainerStyleMobile = (theme) => react.mergeStyles(localPreviewContainerStyle(theme), {
    width: '100%',
    height: '100%',
    minHeight: `${CONFIGURATION_PAGE_SECTION_HEIGHT_REM}rem`
});
/**
 * @private
 */
const cameraOffLabelStyle = react.mergeStyles({
    fontSize: '0.75rem' // 12px
});
/**
 * @private
 */
const localPreviewTileStyle = {
    root: {
        '@media (forced-colors: active)': {
            borderColor: '#FFFFFF',
            border: '1px solid'
        }
    }
};
/**
 * @private
 */
const localPreviewButtonStyle = (isMobile) => {
    return {
        root: {
            minWidth: !isMobile ? _pxToRem(75) : 'unset',
            // we only want to enforce this for desktop views
            '@media (forced-colors: active)': {
                ':focus': {
                    border: '1px solid !important' // we need bang important to override fluents colours
                },
                ':focus::after': {
                    border: 'unset !important',
                    // we need bang important to override fluents colours
                    outline: 'unset !important'
                }
            }
        },
        rootFocused: !isMobile ? {
            outline: '1px solid',
            outlineOffset: '-2px'
        } : {},
        rootChecked: !isMobile ? {
            ':focus': {
                outline: '1px solid',
                outlineOffset: '-2px'
            }
        } : {}
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$d = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const LocalPreview = (props) => {
    var _a;
    const locale = useLocale();
    const cameraButtonProps = usePropsFor$1(CameraButton);
    const localPreviewProps = useSelector$1(localPreviewSelector);
    const devicesButtonProps = usePropsFor$1(DevicesButton);
    const { audio: microphonePermissionGranted, video: cameraPermissionGranted } = useSelector$1(devicePermissionSelector);
    const isLocalMicrophoneEnabled = useSelector$1(getLocalMicrophoneEnabled);
    const adapter = useAdapter();
    const onToggleMic = React.useCallback(() => __awaiter$d(void 0, void 0, void 0, function* () {
        isLocalMicrophoneEnabled ? adapter.mute() : adapter.unmute();
    }), [adapter, isLocalMicrophoneEnabled]);
    const hasNoSpeakers = !devicesButtonProps.speakers.length;
    const hasNoDevices = devicesButtonProps.cameras.length === 0 && devicesButtonProps.microphones.length === 0 && devicesButtonProps.speakers.length === 0;
    const hasCameras = devicesButtonProps.cameras.length > 0;
    const hasMicrophones = devicesButtonProps.microphones.length > 0;
    const cameraLoadingString = (_a = locale.strings.call.configurationPageCameraIsLoadingLabel) !== null && _a !== void 0 ? _a : locale.strings.call.cameraTurnedOff;
    const previewCameraString = props.cameraLoading ? cameraLoadingString : locale.strings.call.cameraTurnedOff;
    const theme = useTheme();
    const onRenderPlaceholder = React.useCallback(() => {
        return React.createElement(react.Stack, { className: react.mergeStyles({
                width: '100%',
                height: '100%',
                // The text should be centered in the local preview with the camera icon
                // appearing above the text. To compensate for the camera icon's height,
                // we add a negative margin to the top of the container.
                marginTop: '-0.8rem'
            }), verticalAlign: "center" },
            React.createElement(react.Stack.Item, { align: "center" },
                React.createElement(CallCompositeIcon, { iconName: "LocalPreviewPlaceholder", className: react.mergeStyles(cameraOffLabelStyle, {
                        color: theme.palette.neutralTertiary
                    }) })),
            React.createElement(react.Stack.Item, { align: "center" },
                React.createElement(react.Text, { className: react.mergeStyles(cameraOffLabelStyle, {
                        color: theme.palette.neutralSecondary
                    }) }, previewCameraString)));
    }, [theme, previewCameraString]);
    const devicesButtonStyles = props.mobileView ? {
        menuStyles: {
            menuItemStyles: buttonFlyoutIncreasedSizeStyles
        }
    } : undefined;
    return React.createElement(react.Stack, { "data-ui-id": "call-composite-local-preview", className: props.mobileView ? localPreviewContainerStyleMobile(theme) : localPreviewContainerStyleDesktop(theme) },
        React.createElement(VideoTile, { styles: localPreviewTileStyle, renderElement: (localPreviewProps === null || localPreviewProps === void 0 ? void 0 : localPreviewProps.videoStreamElement) ? React.createElement(StreamMedia, { videoStreamElement: localPreviewProps.videoStreamElement }) : undefined, onRenderPlaceholder: onRenderPlaceholder },
            React.createElement(ControlBar, { layout: "floatingBottom" },
                React.createElement(MicrophoneButton, { "data-ui-id": "call-composite-local-device-settings-microphone-button", checked: isLocalMicrophoneEnabled, onToggleMicrophone: onToggleMic, disabled: !microphonePermissionGranted || !hasMicrophones, showLabel: true, 
                    // disable tooltip as it obscures list of devices on mobile
                    strings: props.mobileView ? {
                        tooltipOnContent: '',
                        tooltipOffContent: '',
                        tooltipDisabledContent: ''
                    } : {}, styles: localPreviewButtonStyle(props.mobileView) }),
                React.createElement(CameraButton, Object.assign({ "data-ui-id": "call-composite-local-device-settings-camera-button" }, cameraButtonProps, { onToggleCamera: props.onToggleCamera, showLabel: true, disabled: !cameraPermissionGranted || !hasCameras || props.cameraLoading, 
                    // disable tooltip as it obscures list of devices on mobile
                    strings: props.mobileView ? {
                        tooltipOnContent: '',
                        tooltipOffContent: '',
                        tooltipDisabledContent: '',
                        tooltipVideoLoadingContent: ''
                    } : {}, styles: localPreviewButtonStyle(props.mobileView) })),
                props.showDevicesButton && React.createElement(DevicesButton, Object.assign({ "data-ui-id": "call-composite-local-device-settings-options-button" }, devicesButtonProps, { 
                    // disable button whilst all other buttons are disabled
                    disabled: !microphonePermissionGranted && !cameraPermissionGranted && hasNoSpeakers || hasNoDevices, showLabel: true, 
                    // disable tooltip as it obscures list of devices on mobile
                    strings: props.mobileView ? {
                        tooltipContent: ''
                    } : {}, styles: devicesButtonStyles })))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ConfigurationPageErrorBar = (props) => {
    const { errorBarProps } = props;
    if (errorBarProps.activeErrorMessages.length === 0) {
        return React.createElement(React.Fragment, null);
    }
    return React.createElement(react.FocusZone, { shouldFocusOnMount: true },
        React.createElement(ErrorBar, Object.assign({}, errorBarProps, { onDismissError: props.onDismissError })));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * An SVG element component that wraps inner text to fit the width of the SVG.
 * @private
 */
const SvgWithWordWrapping = (props) => {
    const { width, text, lineHeightPx, bufferHeightPx, role } = props;
    const svgRef = React.useRef(null);
    const calculationTextElement = React.useRef(null);
    const visibleTextElement = React.useRef(null);
    const [height, setHeight] = React.useState(0);
    // useLayoutEffect ensures that the calculationTextElement is rendered before being used for calculations.
    // Using useLayoutEffect over useEffect ensures we do not get a layout shift when the visibleTextElement is rendered
    // and the height is updated. This is because useLayoutEffect runs synchronously after DOM mutations but
    // before the browser has a chance to paint. See https://reactjs.org/docs/hooks-reference.html#uselayouteffect
    // for more details.
    React.useLayoutEffect(() => {
        if (text && calculationTextElement.current && visibleTextElement.current) {
            const numLines = convertTextToWrappedText(calculationTextElement.current, visibleTextElement.current, width, lineHeightPx);
            setHeight(numLines * lineHeightPx);
        }
    }, [width, lineHeightPx, text]);
    return React.createElement("svg", { "aria-level": 1, role: role, width: width, height: height + bufferHeightPx, ref: svgRef, xmlns: "http://www.w3.org/2000/svg" },
        React.createElement("text", { height: 0, ref: calculationTextElement, style: {
                visibility: 'hidden'
            } }, text),
        React.createElement("text", { ref: visibleTextElement, x: "0", y: bufferHeightPx / 4, role: "heading", "aria-level": 1 }));
};
/**
 * Wrap text in tspan elements to fit the width of the SVG
 * @param baseTextElement The text element to create the wrapped text from.
 * @param outputTextElement The text element to insert the wrapped text into.
 * @param maxWidth The maximum width of the text element.
 * @param lineHeightPx The height of each line in pixels.
 * @returns The number of lines of text.
 */
const convertTextToWrappedText = (inputTextElement, outputTextElement, maxWidth, lineHeightPx) => {
    var _a, _b;
    const words = (_b = (_a = inputTextElement.textContent) === null || _a === void 0 ? void 0 : _a.split(' ')) !== null && _b !== void 0 ? _b : [];
    if (words.length === 0 || words[0] === '') {
        throw new Error('Text element must contain text');
    }
    // The current line being built.
    let line = '';
    // Running total of the number of lines.
    let numLines = 0;
    // First, clear the output text element.
    outputTextElement.textContent = '';
    // Iterate through each word and create a tspan element for each line.
    for (let i = 0; i < words.length; i++) {
        const testLine = line + words[i] + ' ';
        const testWidth = inputTextElement.getSubStringLength(0, testLine.length);
        if (testWidth > maxWidth && i > 0) {
            const newLine = constructTSpanLine(line, lineHeightPx);
            outputTextElement.appendChild(newLine);
            line = words[i] + ' ';
            numLines++;
        }
        else {
            line = testLine;
        }
    }
    // Add the last line.
    const newLine = constructTSpanLine(line, lineHeightPx);
    outputTextElement.appendChild(newLine);
    numLines++;
    // Return the number of lines to calculate the height of the SVG.
    return numLines;
};
/**
 * Create a tspan element for a line of text, with text set to be centered.
 * @param line The line of text.
 * @param lineHeightPx The height of each line in pixels.
 * @returns The tspan element.
 */
const constructTSpanLine = (line, lineHeightPx) => {
    const tspan = document.createElementNS('http://www.w3.org/2000/svg', 'tspan');
    tspan.textContent = line;
    tspan.setAttribute('x', '50%');
    tspan.setAttribute('dy', `${lineHeightPx}px`);
    tspan.setAttribute('text-anchor', 'middle');
    return tspan;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$c = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const ConfigurationPage = (props) => {
    var _a;
    const { startCallHandler, mobileView, modalLayerHostId, joinCallOptions = {
        microphoneCheck: 'requireMicrophoneAvailable'
    } } = props;
    const theme = useTheme();
    const options = useAdaptedSelector(getSelector$1(DevicesButton));
    const localDeviceSettingsHandlers = useHandlers();
    const { video: cameraPermissionGranted, audio: microphonePermissionGranted } = useSelector$1(devicePermissionSelector);
    const configContainerRef = React.useRef(null);
    const configWidth = _useContainerWidth(configContainerRef);
    /**
     * We want to stack the two sections (preview and devices) when the container is less than 450 wide.
     * We lose size calculation when the container is less than 450 wide, so we stack the two sections.
     */
    const stackConfig = !!configWidth && configWidth < 450;
    const errorBarProps = usePropsFor$1(ErrorBar);
    const microphones = useSelector$1(getMicrophones);
    const environmentInfo = useSelector$1(getEnvironmentInfo);
    let disableStartCallButton = (!microphonePermissionGranted || (microphones === null || microphones === void 0 ? void 0 : microphones.length) === 0) && joinCallOptions.microphoneCheck === 'requireMicrophoneAvailable';
    const role = useSelector$1(getRole);
    const isCameraOn = useSelector$1(localVideoSelector).isAvailable;
    const [cameraLoading, setCameraLoading] = React.useState(false);
    const switchCamera = React.useCallback((device, options) => __awaiter$c(void 0, void 0, void 0, function* () {
        // Only set camera to be loading if we are switching source while the camera is on
        setCameraLoading(isCameraOn);
        try {
            yield localDeviceSettingsHandlers.onSelectCamera(device, options);
        }
        finally {
            setCameraLoading(false);
        }
    }), [localDeviceSettingsHandlers, isCameraOn]);
    const { onToggleCamera } = usePropsFor$1(CameraButton);
    const toggleCamera = React.useCallback((options) => __awaiter$c(void 0, void 0, void 0, function* () {
        // Only set camera to loading if we are turning on the camera (i.e. the camera was off)
        setCameraLoading(!isCameraOn);
        try {
            yield onToggleCamera(options);
        }
        finally {
            setCameraLoading(false);
        }
    }), [isCameraOn, onToggleCamera]);
    let filteredLatestErrors = props.latestErrors;
    // TODO: move this logic to the error bar selector once role is plumbed from the headless SDK
    if (role !== 'Consumer') {
        filteredLatestErrors = filteredLatestErrors.filter(e => e.type !== 'callCameraAccessDenied' && e.type !== 'callCameraAccessDeniedSafari');
    }
    if ((useIsParticularSidePaneOpen('videoeffects') || !isCameraOn) && errorBarProps) {
        filteredLatestErrors = filteredLatestErrors.filter(e => e.type !== 'unableToStartVideoEffect');
    }
    if (role === 'Consumer') {
        // If user's role permissions do not allow access to the microphone button then DO NOT disable the start call button
        // because microphone device permission is not needed for the user's role
        disableStartCallButton = false;
    }
    const locale = useLocale();
    const title = locale.strings.call.configurationPageTitle.length > 0 ? React.createElement(react.Stack.Item, { className: mobileView ? titleContainerStyleMobile(theme) : titleContainerStyleDesktop(theme) },
        React.createElement(SvgWithWordWrapping, { width: mobileView ? 325 : 445, lineHeightPx: 16 * 1.5, bufferHeightPx: 16, text: locale.strings.call.configurationPageTitle, role: "heading" })) : React.createElement(React.Fragment, null);
    const callDescription = locale.strings.call.configurationPageCallDetails && React.createElement(react.Stack.Item, { className: mobileView ? callDetailsStyleMobile(theme) : callDetailsStyleDesktop(theme) }, locale.strings.call.configurationPageCallDetails);
    const mobileWithPreview = mobileView && role !== 'Consumer'; // When permission API is not available, we want to show screen saying checking for access (disappears on its own)
    // then based on permission setting, we show permission denied or nothing
    const { toggleVideoEffectsPane, closeVideoEffectsPane, isVideoEffectsPaneOpen } = useVideoEffectsPane(props.updateSidePaneRenderer, mobileView, props.latestErrors, props.onDismissError);
    const startCall = React.useCallback(() => __awaiter$c(void 0, void 0, void 0, function* () {
        closeVideoEffectsPane();
        startCallHandler();
    }), [startCallHandler, closeVideoEffectsPane]);
    const panelLayerProps = React.useMemo(() => ({
        hostId: modalLayerHostId
    }), [modalLayerHostId]);
    const filteredErrorBarProps = React.useMemo(() => (Object.assign(Object.assign({}, errorBarProps), { activeErrorMessages: filteredLatestErrors })), [errorBarProps, filteredLatestErrors]);
    const containerStyles = React.useMemo(() => { var _a; return configurationContainerStyle(!mobileView, (_a = props.backgroundImage) === null || _a === void 0 ? void 0 : _a.url); }, [mobileView, (_a = props.backgroundImage) === null || _a === void 0 ? void 0 : _a.url]);
    return React.createElement("div", { ref: configContainerRef, className: react.mergeStyles(containerStyles) },
        React.createElement(react.Stack, { styles: bannerNotificationStyles },
            React.createElement(ConfigurationPageErrorBar, { errorBarProps: filteredErrorBarProps, onDismissError: props.onDismissError })),
        React.createElement(react.Stack, { verticalFill: true, grow: true, horizontal: true, className: fillWidth },
            React.createElement(react.Stack, { className: configurationCenteredContent(mobileWithPreview, !!props.logo), verticalAlign: stackConfig && !mobileView ? undefined : 'center', verticalFill: mobileWithPreview, tokens: mobileWithPreview ? configurationStackTokensMobile : configurationStackTokensDesktop },
                React.createElement(react.Stack.Item, { styles: callDetailsContainerStyles },
                    React.createElement(Logo, { logo: props.logo }),
                    title,
                    callDescription),
                React.createElement(react.Stack, { horizontal: configHorizontal(mobileWithPreview, mobileView, stackConfig), horizontalAlign: mobileWithPreview ? 'stretch' : 'center', verticalFill: mobileWithPreview, tokens: deviceConfigurationStackTokens },
                    role !== 'Consumer' && React.createElement(LocalPreview, { mobileView: mobileWithPreview, showDevicesButton: mobileView, onToggleCamera: toggleCamera, cameraLoading: cameraLoading && !isCameraOn }),
                    React.createElement(react.Stack, { styles: mobileView ? undefined : configurationSectionStyle },
                        !mobileWithPreview && React.createElement(react.Stack, { className: mobileView ? undefined : selectionContainerStyle(theme, _isSafari(environmentInfo)) },
                            React.createElement(LocalDeviceSettings, Object.assign({}, options, localDeviceSettingsHandlers, { onSelectCamera: switchCamera, cameraPermissionGranted: cameraPermissionGrantedTrampoline(cameraPermissionGranted), microphonePermissionGranted: micPermissionGrantedTrampoline(microphonePermissionGranted), onClickVideoEffects: toggleVideoEffectsPane }))),
                        React.createElement(react.Stack, { styles: mobileWithPreview ? startCallButtonContainerStyleMobile : startCallButtonContainerStyleDesktop, horizontalAlign: mobileWithPreview ? 'stretch' : 'end' },
                            React.createElement(StartCallButton, { className: mobileWithPreview ? startCallButtonStyleMobile : startCallButtonStyleDesktop, onClick: startCall, disabled: disableStartCallButton, hideIcon: true }))))),
            React.createElement(react.Panel, { isOpen: isVideoEffectsPaneOpen, hasCloseButton: false, isBlocking: false, isHiddenOnDismiss: false, styles: panelStyles, focusTrapZoneProps: panelFocusProps, layerProps: panelLayerProps, type: react.PanelType.custom, customWidth: `${VIDEO_EFFECTS_SIDE_PANE_WIDTH_REM}rem` },
                React.createElement(SidePane, { ariaLabel: isVideoEffectsPaneOpen ? locale.strings.call.videoEffectsPaneAriaLabel : undefined, mobileView: props.mobileView, updateSidePaneRenderer: props.updateSidePaneRenderer, maxWidth: `${VIDEO_EFFECTS_SIDE_PANE_WIDTH_REM}rem`, minWidth: `${VIDEO_EFFECTS_SIDE_PANE_WIDTH_REM}rem` }))));
};
const cameraPermissionGrantedTrampoline = (cameraPermissionGranted, videoState) => {
    return cameraPermissionGranted;
};
const micPermissionGrantedTrampoline = (microphonePermissionGranted, audioState) => {
    return microphonePermissionGranted;
};
const Logo = (props) => {
    if (!props.logo) {
        return React.createElement(React.Fragment, null);
    }
    return React.createElement(react.Image, { styles: logoStyles(props.logo.shape), src: props.logo.url, alt: props.logo.alt });
};
const configHorizontal = (mobileWithPreview, isMobile, configTooNarrow) => {
    if (configTooNarrow && !isMobile) {
        return false;
    }
    return !mobileWithPreview;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const containerStyle = {
    maxWidth: '22.5rem',
    // Ensure some space around the text on a narrow viewport.
    margin: '1rem'
};
/**
 * @private
 */
const containerItemGap = {
    childrenGap: '0.5rem'
};
/**
 * @private
 */
const rejoinCallButtonContainerStyles = {
    root: {
        paddingTop: '1.125rem',
        borderRadius: '0.25rem'
    }
};
/**
 * @private
 */
const titleStyles = {
    fontSize: '1.25rem',
    fontWeight: 600
};
/**
 * @private
 */
const moreDetailsStyles = {
    fontSize: '1rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Generic page with a title and more details text for serving up a notice to the user.
 *
 * @private
 */
function NoticePage(props) {
    const adapter = useAdapter();
    const callees = useSelector$1(getTargetCallees);
    return React.createElement(react.Stack, { className: react.mergeStyles(props.pageStyle), verticalFill: true, verticalAlign: "center", horizontalAlign: "center", "data-ui-id": props.dataUiId, "aria-atomic": true },
        React.createElement(react.Stack, { className: react.mergeStyles(containerStyle), tokens: containerItemGap },
            props.iconName && React.createElement(CallCompositeIcon, { iconName: props.iconName }),
            React.createElement(react.Text, { className: react.mergeStyles(titleStyles), "aria-live": "assertive", role: "alert" }, props.title),
            React.createElement(react.Text, { className: react.mergeStyles(moreDetailsStyles), "aria-live": "assertive" }, props.moreDetails),
            !props.disableStartCallButton && React.createElement(react.Stack, { styles: rejoinCallButtonContainerStyles },
                React.createElement(StartCallButton, { onClick: () => {
                        if (callees && callees.length > 0) {
                            adapter.startCall(callees);
                        }
                        else {
                            adapter.joinCall();
                        }
                    }, disabled: false, rejoinCall: true, autoFocus: true }))));
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const overlayContainerStyle = {
    // Ensure some space around the text on a narrow viewport.
    margin: '1rem'
};
/**
 * @private
 */
const titleStyle = (palette, isVideoReady) => {
    return {
        fontSize: '1rem',
        color: isVideoReady ? 'white' : palette.themePrimary,
        textAlign: 'center'
    };
};
/**
 * @private
 */
const moreDetailsStyle = (palette, isVideoReady) => ({
    fontSize: '0.75rem',
    color: isVideoReady ? 'white' : palette.themePrimary,
    textAlign: 'center'
});
/**
 * @private
 */
const localCameraSwitcherContainerStyles = {
    root: {
        position: 'absolute',
        top: '0rem',
        right: '0rem'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const LobbyTile = (props) => {
    var _a;
    const videoStream = props.localParticipantVideoStream;
    const isVideoReady = (_a = videoStream === null || videoStream === void 0 ? void 0 : videoStream.isAvailable) !== null && _a !== void 0 ? _a : false;
    const cameraSwitcherCameras = useSelector$1(localVideoCameraCycleButtonSelector);
    const cameraSwitcherCallback = useHandlers();
    const palette = useTheme().palette;
    const handlers = useHandlers();
    return React.createElement(ExpandedLocalVideoTile, Object.assign({ localParticipantVideoStream: props.localParticipantVideoStream, overlayContent: props.overlayProps ? React.createElement(react.Stack, { verticalFill: true, horizontalAlign: "center", verticalAlign: "center", className: react.mergeStyles(overlayContainerStyle), "aria-atomic": true },
            props.showLocalVideoCameraCycleButton && isVideoReady && React.createElement(react.Stack.Item, { styles: localCameraSwitcherContainerStyles },
                React.createElement(LocalVideoCameraCycleButton, Object.assign({}, cameraSwitcherCallback, cameraSwitcherCameras, { size: 'large' }))),
            React.createElement(react.Stack.Item, { className: react.mergeStyles(titleStyle(palette, isVideoReady)) }, props.overlayProps.overlayIcon),
            React.createElement(react.Text, { className: react.mergeStyles(titleStyle(palette, isVideoReady)), "aria-live": "assertive", "data-ui-id": "lobbyScreenTitle", role: "alert" }, props.overlayProps.title),
            props.overlayProps.moreDetails && React.createElement(react.Text, { className: react.mergeStyles(moreDetailsStyle(palette, isVideoReady)), "aria-live": "assertive" }, props.overlayProps.moreDetails)) : undefined }, handlers));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const LobbyPage = (props) => {
    var _a, _b, _c, _d, _e, _f;
    const errorBarProps = usePropsFor$1(ErrorBar);
    const lobbyProps = useSelector$1(lobbySelector);
    const strings = useLocale().strings.call;
    const callState = useSelector$1(getCallStatus);
    const inLobby = callState === 'InLobby';
    const participants = (_a = useSelector$1(getRemoteParticipants)) !== null && _a !== void 0 ? _a : {};
    // Reduce the controls shown when mobile view is enabled.
    let callControlOptions = props.mobileView ? reduceCallControlsForMobile((_b = props.options) === null || _b === void 0 ? void 0 : _b.callControls) : (_c = props.options) === null || _c === void 0 ? void 0 : _c.callControls;
    callControlOptions = disableCallControls(callControlOptions, ['screenShareButton', 'participantsButton']);
    return React.createElement(CallArrangement, { complianceBannerProps: {
            strings
        }, errorBarProps: ((_d = props.options) === null || _d === void 0 ? void 0 : _d.errorBar) !== false && errorBarProps, showErrorNotifications: (_f = (_e = props.options) === null || _e === void 0 ? void 0 : _e.errorBar) !== null && _f !== void 0 ? _f : true, callControlProps: {
            options: callControlOptions,
            increaseFlyoutItemSize: props.mobileView
        }, mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, onRenderGalleryContent: () => React.createElement(LobbyTile, Object.assign({}, lobbyProps, { showLocalVideoCameraCycleButton: props.mobileView, overlayProps: overlayProps(strings, inLobby, Object.values(participants)) })), dataUiId: 'lobby-page', updateSidePaneRenderer: props.updateSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, latestErrors: props.latestErrors, onDismissError: props.onDismissError, latestNotifications: props.latestNotifications, onDismissNotification: props.onDismissNotification });
};
const overlayProps = (strings, inLobby, remoteParticipants) => {
    /**
     * Only grab the first participant because there will only be one in this situation.
     * when starting a call with multiple people the call goes to the connected state and composite goes directly to
     * videoGallery.
     *
     * We also need to check the participant state since in a group call the remote participants array will populate just before
     * the user joins. In this situation we also check the participant states. in a groupCall the state of the participants
     * will be 'Idle'.
     */
    const outboundCallParticipant = remoteParticipants[0] && ['Ringing', 'Connecting'].includes(remoteParticipants[0].state) && remoteParticipants.length === 1 ? remoteParticipants[0] : undefined;
    return inLobby ? overlayPropsWaitingToBeAdmitted(strings) : outboundCallParticipant ? overlayPropsOutboundCall(strings, outboundCallParticipant) : overlayPropsConnectingToCall(strings);
};
const overlayPropsConnectingToCall = (strings) => ({
    title: strings.lobbyScreenConnectingToCallTitle,
    moreDetails: strings.lobbyScreenConnectingToCallMoreDetails,
    overlayIcon: React.createElement(CallCompositeIcon, { iconName: "LobbyScreenConnectingToCall" })
});
const overlayPropsWaitingToBeAdmitted = (strings) => ({
    title: strings.lobbyScreenWaitingToBeAdmittedTitle,
    moreDetails: strings.lobbyScreenWaitingToBeAdmittedMoreDetails,
    overlayIcon: React.createElement(CallCompositeIcon, { iconName: "LobbyScreenWaitingToBeAdmitted" })
});
const overlayPropsOutboundCall = (strings, participant) => {
    var _a;
    if (communicationCommon.isPhoneNumberIdentifier(participant.identifier)) {
        return {
            title: participant.identifier.phoneNumber,
            moreDetails: strings.outboundCallingNoticeString
        };
    }
    else {
        return {
            title: (_a = strings.outboundCallingNoticeString) !== null && _a !== void 0 ? _a : ''
        };
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const pageContainer = {
    width: '100%',
    height: '100%'
};
/**
 * @private
 */
const avatarStyles = {
    opacity: 0.4
};
/**
 * @private
 */
const tileContainerStyles = {
    position: 'absolute',
    top: '0',
    left: '0',
    width: '100%',
    height: '100%',
    minWidth: '100%',
    minHeight: '100%',
    objectPosition: 'center',
    objectFit: 'cover',
    zIndex: 0
};
/**
 * @private
 */
const tileContentStyles = {
    width: '100%',
    position: 'absolute',
    top: '50%',
    transform: 'translate(0, -50%)',
    display: 'flex',
    justifyContent: 'center'
};
/**
 * @private
 */
const defaultPersonaStyles = {
    root: {
        margin: 'auto'
    }
};
/**
 * @private
 */
const displayNameStyles = {
    textAlign: 'center',
    fontSize: '1.5rem',
    fontWeight: 400
};
/**
 * @private
 */
const spinnerStyles = {
    circle: {
        borderWidth: '0.125rem'
    }
};
/**
 * @private
 */
const statusTextStyles = {
    textAlign: 'center',
    fontSize: '1rem'
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const TransferPage = (props) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const errorBarProps = usePropsFor$1(ErrorBar);
    const strings = useLocale().strings.call;
    const remoteParticipants = useSelector$1(getRemoteParticipants);
    const transferCall = useSelector$1(getTransferCall);
    const [announcerString, setAnnouncerString] = React.useState(undefined);
    // Reduce the controls shown when mobile view is enabled.
    const callControlOptions = props.mobileView ? reduceCallControlsForMobile((_a = props.options) === null || _a === void 0 ? void 0 : _a.callControls) : (_b = props.options) === null || _b === void 0 ? void 0 : _b.callControls;
    const transferor = React.useMemo(() => { var _a; return remoteParticipants ? (_a = Object.values(remoteParticipants)) === null || _a === void 0 ? void 0 : _a[0] : undefined; }, [remoteParticipants]);
    const transferTarget = React.useMemo(() => { var _a; return (transferCall === null || transferCall === void 0 ? void 0 : transferCall.remoteParticipants) ? (_a = Object.values(transferCall.remoteParticipants)) === null || _a === void 0 ? void 0 : _a[0] : undefined; }, [transferCall]);
    const pageSubject = React.useMemo(() => {
        if (transferCall && (transferTarget === null || transferTarget === void 0 ? void 0 : transferTarget.displayName)) {
            return 'transferTarget';
        }
        return 'transferor';
    }, [transferCall, transferTarget === null || transferTarget === void 0 ? void 0 : transferTarget.displayName]);
    React.useEffect(() => {
        setAnnouncerString(strings.transferPageNoticeString);
    }, [strings.transferPageNoticeString]);
    let transferTileParticipant = transferor;
    if (pageSubject === 'transferTarget') {
        transferTileParticipant = transferTarget;
    }
    let transferParticipantDisplayName = (_c = transferor === null || transferor === void 0 ? void 0 : transferor.displayName) !== null && _c !== void 0 ? _c : strings.transferPageUnknownTransferorDisplayName;
    if (pageSubject === 'transferTarget') {
        transferParticipantDisplayName = (_d = transferTarget === null || transferTarget === void 0 ? void 0 : transferTarget.displayName) !== null && _d !== void 0 ? _d : strings.transferPageUnknownTransferTargetDisplayName;
    }
    return React.createElement(react.Stack, { className: react.mergeStyles(pageContainer) },
        React.createElement(Announcer$1, { announcementString: announcerString, ariaLive: "polite" }),
        React.createElement(CallArrangement, { complianceBannerProps: {
                strings
            }, 
            // Ignore errors from before current call. This avoids old errors from showing up when a user re-joins a call.
            errorBarProps: ((_e = props.options) === null || _e === void 0 ? void 0 : _e.errorBar) !== false && errorBarProps, showErrorNotifications: (_g = (_f = props.options) === null || _f === void 0 ? void 0 : _f.errorBar) !== null && _g !== void 0 ? _g : true, callControlProps: {
                options: callControlOptions,
                increaseFlyoutItemSize: props.mobileView
            }, mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, onRenderGalleryContent: () => React.createElement(TransferTile, { userId: transferTileParticipant ? toFlatCommunicationIdentifier(transferTileParticipant === null || transferTileParticipant === void 0 ? void 0 : transferTileParticipant.identifier) : undefined, displayName: transferParticipantDisplayName, initialsName: transferParticipantDisplayName, statusText: pageSubject === 'transferTarget' ? strings.transferPageTransferTargetText : strings.transferPageTransferorText, onFetchAvatarPersonaData: props.onFetchAvatarPersonaData }), dataUiId: 'transfer-page', updateSidePaneRenderer: props.updateSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, latestErrors: props.latestErrors, onDismissError: props.onDismissError, onDismissNotification: props.onDismissNotification, latestNotifications: props.latestNotifications }));
};
const TransferTile = (props) => {
    const { displayName, initialsName, userId, onFetchAvatarPersonaData, statusText } = props;
    const [personaSize, setPersonaSize] = React.useState();
    const tileRef = React.useRef(null);
    const observer = React.useRef(new ResizeObserver((entries) => {
        if (!entries[0]) {
            return;
        }
        const { width, height } = entries[0].contentRect;
        const personaSize = Math.min(width, height) / 2;
        setPersonaSize(Math.max(Math.min(personaSize, 150), 32));
    }));
    React.useLayoutEffect(() => {
        if (tileRef.current) {
            observer.current.observe(tileRef.current);
        }
        const currentObserver = observer.current;
        return () => currentObserver.disconnect();
    }, [observer, tileRef]);
    const placeholderOptions = React.useMemo(() => ({
        userId,
        text: initialsName !== null && initialsName !== void 0 ? initialsName : displayName,
        coinSize: personaSize,
        styles: defaultPersonaStyles,
        hidePersonaDetails: true
    }), [userId, initialsName, displayName, personaSize]);
    const defaultOnRenderAvatar = React.useCallback(() => {
        return personaSize ? React.createElement(AvatarPersona, Object.assign({}, placeholderOptions, { dataProvider: onFetchAvatarPersonaData, className: react.mergeStyles(avatarStyles) })) : React.createElement(React.Fragment, null);
    }, [placeholderOptions, onFetchAvatarPersonaData, personaSize]);
    const defaultAvatar = React.useMemo(() => defaultOnRenderAvatar(), [defaultOnRenderAvatar]);
    return React.createElement("div", { ref: tileRef, className: react.mergeStyles(tileContainerStyles), "data-is-focusable": true },
        React.createElement(react.Stack, { className: react.mergeStyles(tileContentStyles), tokens: {
                childrenGap: '1rem'
            } },
            React.createElement(react.Stack, { horizontalAlign: "center", tokens: {
                    childrenGap: '0.5rem'
                } },
                defaultAvatar,
                React.createElement(react.Text, { className: react.mergeStyles(displayNameStyles) }, displayName)),
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "center", verticalAlign: "center", tokens: {
                    childrenGap: '0.5rem'
                } },
                React.createElement(react.Spinner, { size: react.SpinnerSize.large, className: react.mergeStyles(spinnerStyles) }),
                React.createElement(react.Text, { className: react.mergeStyles(statusTextStyles) }, statusText))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * styles for hold pane resume button
 *
 * @private
 */
const resumeButtonStyles = {
    root: {
        borderRadius: _pxToRem(4),
        padding: `${_pxToRem(6)} ${_pxToRem(20)} `
    },
    label: {
        fontWeight: 400,
        display: 'flex',
        fontSize: _pxToRem(14)
    },
    labelDisabled: {
        fontWeight: 400,
        display: 'flex',
        fontSize: _pxToRem(14)
    }
};
/**
 * styles for hold pane main text
 *
 * @private
 */
const holdPaneLabelStyles = {
    root: {
        color: 'inherit',
        fontWeight: 600,
        fontHeight: _pxToRem(22),
        fontSize: _pxToRem(16),
        marginTop: '0.5rem',
        marginBottom: '1.5rem'
    }
};
/**
 * styles for hold pane timer
 *
 * @private
 */
const holdPaneTimerStyles = {
    root: {
        color: 'inherit',
        fontWeight: 600,
        fontSize: _pxToRem(20),
        lineHeight: _pxToRem(28),
        margin: 'auto'
    }
};
/**
 * styles for hold pane container
 *
 * @private
 */
const paneStyles = {
    root: {
        width: '100%',
        height: '100%',
        background: 'inherit'
    }
};
/**
 * styles for the hold pane content container
 *
 * @private
 */
const holdPaneContentStyles = {
    root: {
        display: 'flex',
        margin: 'auto',
        flexDirection: 'column',
        justifyContent: 'center',
        alignContent: 'center',
        alignItems: 'center'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$b = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Hold pane to display when the user places themselves on hold
 *
 * @beta
 */
const HoldPane = () => {
    var _a, _b, _c, _d, _e;
    const holdButtonProps = usePropsFor$1(HoldButton);
    const locale = useLocale();
    const strings = {
        holdScreenLabel: (_a = locale.strings.call.holdScreenLabel) !== null && _a !== void 0 ? _a : '',
        resumeCallButtonLabel: (_b = locale.strings.call.resumeCallButtonLabel) !== null && _b !== void 0 ? _b : '',
        resumeCallButtonAriaLabel: (_c = locale.strings.call.resumeCallButtonAriaLabel) !== null && _c !== void 0 ? _c : '',
        resumingCallButtonLabel: (_d = locale.strings.call.resumingCallButtonLabel) !== null && _d !== void 0 ? _d : '',
        resumingCallButtonAriaLabel: (_e = locale.strings.call.resumingCallButtonAriaLabel) !== null && _e !== void 0 ? _e : ''
    };
    const [time, setTime] = React.useState(0);
    const elapsedTime = getReadableTime(time);
    const startTime = React.useRef(performance.now());
    const [resumingCall, setResumingCall] = React.useState(false);
    React.useEffect(() => {
        const interval = setInterval(() => {
            setTime(performance.now() - startTime.current);
        }, 10);
        return () => {
            clearInterval(interval);
        };
    }, [startTime]);
    return React.createElement(react.Stack, { styles: paneStyles },
        React.createElement(react.Stack, { horizontal: true, styles: holdPaneContentStyles },
            React.createElement(react.Text, { styles: holdPaneTimerStyles }, elapsedTime),
            React.createElement(react.Text, { styles: holdPaneLabelStyles, role: "alert", "aria-live": "assertive" }, strings.holdScreenLabel),
            React.createElement(react.PrimaryButton, { text: !resumingCall ? strings.resumeCallButtonLabel : strings.resumingCallButtonLabel, ariaLabel: !resumingCall ? strings.resumeCallButtonAriaLabel : strings.resumingCallButtonAriaLabel, styles: resumeButtonStyles, disabled: resumingCall, onClick: () => __awaiter$b(void 0, void 0, void 0, function* () {
                    setResumingCall(true);
                    try {
                        yield holdButtonProps.onToggleHold();
                    }
                    catch (e) {
                        setResumingCall(false);
                        throw e;
                    }
                }), "data-ui-id": "hold-page-resume-call-button", autoFocus: true })));
};
const getMinutes = (time) => {
    return Math.floor(getSeconds(time) / 60);
};
const getSeconds = (time) => {
    return Math.floor(time / 1000);
};
const getHours = (time) => {
    return Math.floor(getMinutes(time) / 60);
};
/**
 * @internal
 */
const getReadableTime = (time) => {
    const hours = getHours(time);
    const readableMinutes = ('0' + getMinutes(time) % 60).slice(-2);
    const readableSeconds = ('0' + getSeconds(time) % 60).slice(-2);
    return `${hours > 0 ? hours + ':' : ''}${readableMinutes}:${readableSeconds}`;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @beta
 */
const HoldPage = (props) => {
    var _a, _b, _c, _d, _e;
    const errorBarProps = usePropsFor$1(ErrorBar);
    const strings = useLocale().strings.call;
    let callControlOptions = props.mobileView ? reduceCallControlsForMobile((_a = props.options) === null || _a === void 0 ? void 0 : _a.callControls) : (_b = props.options) === null || _b === void 0 ? void 0 : _b.callControls;
    callControlOptions = disableCallControls(callControlOptions, ['cameraButton', 'microphoneButton', 'devicesButton', 'screenShareButton', 'holdButton']);
    return React.createElement(CallArrangement, { complianceBannerProps: {
            strings
        }, errorBarProps: ((_c = props.options) === null || _c === void 0 ? void 0 : _c.errorBar) !== false && errorBarProps, showErrorNotifications: (_e = (_d = props.options) === null || _d === void 0 ? void 0 : _d.errorBar) !== null && _e !== void 0 ? _e : true, callControlProps: {
            options: callControlOptions,
            increaseFlyoutItemSize: props.mobileView
        }, mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, onRenderGalleryContent: () => React.createElement(HoldPane, null), dataUiId: 'hold-page', updateSidePaneRenderer: props.updateSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, latestErrors: props.latestErrors, latestNotifications: props.latestNotifications, onDismissError: props.onDismissError, onDismissNotification: props.onDismissNotification });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const deviceCountSelector = reselect.createSelector([getCameras, getMicrophones], (cameras, microphones) => {
    return {
        camerasCount: cameras.length,
        microphonesCount: microphones.length
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const capabilitiesChangedInfoAndRoleSelector = reselect__namespace.createSelector([getLatestCapabilitiesChangedInfo, getRole], (capabilitiesChangeInfo, participantRole) => {
    return {
        capabilitiesChangeInfo,
        participantRole
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Create a record for when the notification was most recently dismissed for tracking dismissed notifications.
 *
 * @private
 */
const useTrackedCapabilityChangedNotifications = (capabilitiesChangedAndRoleInfo) => {
    const [trackedCapabilityChangedNotifications, setTrackedCapabilityChangedNotifications] = React.useState({});
    const activeNotifications = React.useRef({});
    React.useEffect(() => {
        activeNotifications.current = updateLatestCapabilityChangedNotificationMap(capabilitiesChangedAndRoleInfo, activeNotifications.current);
        setTrackedCapabilityChangedNotifications(prev => updateTrackedCapabilityChangedNotificationsWithActiveNotifications(prev, Object.values(activeNotifications.current)));
    }, [capabilitiesChangedAndRoleInfo]);
    const onDismissCapabilityChangedNotification = React.useCallback((notification) => {
        setTrackedCapabilityChangedNotifications(prev => trackCapabilityChangedNotificationAsDismissed(notification.capabilityName, prev));
    }, []);
    const latestCapabilityChangedNotifications = React.useMemo(() => filterLatestCapabilityChangedNotifications(Object.values(activeNotifications.current), trackedCapabilityChangedNotifications), [trackedCapabilityChangedNotifications]);
    return {
        capabilitiesChangedNotifications: latestCapabilityChangedNotifications,
        onDismissNotification: onDismissCapabilityChangedNotification
    };
};
/**
 * Take the set of active notifications, and filter to only those that are newer than previously dismissed notifications or have never been dismissed.
 *
 * @private
 */
const filterLatestCapabilityChangedNotifications = (activeNotifications, trackedNotifications) => {
    const filteredNotifications = activeNotifications.filter(activeNotification => {
        const trackedNotification = trackedNotifications[activeNotification.capabilityName];
        return !trackedNotification || !trackedNotification.lastDismissedAt || trackedNotification.lastDismissedAt < trackedNotification.mostRecentlyActive;
    });
    return filteredNotifications;
};
/**
 * Maintain a record of the most recently active notification for each capability name.
 *
 * @private
 */
const updateTrackedCapabilityChangedNotificationsWithActiveNotifications = (existingTrackedNotifications, activeNotifications) => {
    var _a, _b;
    const trackedNotifications = {};
    // Only care about active notifications. If notifications are no longer active we do not track that they have been previously dismissed.
    for (const activeNotification of activeNotifications) {
        const existingTrackedNotification = existingTrackedNotifications[activeNotification.capabilityName];
        trackedNotifications[activeNotification.capabilityName] = {
            mostRecentlyActive: (_b = (_a = activeNotification.timestamp) !== null && _a !== void 0 ? _a : existingTrackedNotification === null || existingTrackedNotification === void 0 ? void 0 : existingTrackedNotification.mostRecentlyActive) !== null && _b !== void 0 ? _b : new Date(Date.now()),
            lastDismissedAt: existingTrackedNotification === null || existingTrackedNotification === void 0 ? void 0 : existingTrackedNotification.lastDismissedAt
        };
    }
    return trackedNotifications;
};
/**
 * Create a record for when the notification was most recently dismissed for tracking dismissed notifications.
 *
 * @private
 */
const trackCapabilityChangedNotificationAsDismissed = (capabilityName, trackedNotifications) => {
    const now = new Date(Date.now());
    const existingNotification = trackedNotifications[capabilityName];
    return Object.assign(Object.assign({}, trackedNotifications), { [capabilityName]: Object.assign(Object.assign({}, (existingNotification || {})), { lastDismissedAt: now }) });
};
const updateLatestCapabilityChangedNotificationMap = (capabilitiesChangedInfoAndRole, activeNotifications) => {
    var _a, _b;
    if (!capabilitiesChangedInfoAndRole.capabilitiesChangeInfo) {
        return activeNotifications;
    }
    for (const [capabilityKey, newCapabilityValue] of Object.entries(capabilitiesChangedInfoAndRole.capabilitiesChangeInfo.newValue)) {
        // Cast is safe because we are iterating over the enum keys on the object.entries where
        // newCapabilityValue typing is correctly returned. Object.entries limitations
        // always returns string for the key
        const capabilityName = capabilityKey;
        // If the active notification for a capability has the same `isPresent` value and the same reason as the new
        // capability value from the SDK then we will not create a new notification to avoid redundancy
        if (activeNotifications[capabilityName] && newCapabilityValue.isPresent === ((_a = activeNotifications[capabilityName]) === null || _a === void 0 ? void 0 : _a.isPresent) && capabilitiesChangedInfoAndRole.capabilitiesChangeInfo.reason === ((_b = activeNotifications[capabilityName]) === null || _b === void 0 ? void 0 : _b.changedReason)) {
            continue;
        }
        // All initial values of capabilities are not present with reason 'FeatureNotSupported'. So we should not show a
        // notification for them when they initially become present at the start of the call
        const oldCapabilityValue = capabilitiesChangedInfoAndRole.capabilitiesChangeInfo.oldValue[capabilityName];
        if (newCapabilityValue.isPresent === true && (oldCapabilityValue === null || oldCapabilityValue === void 0 ? void 0 : oldCapabilityValue.isPresent) === false && (oldCapabilityValue === null || oldCapabilityValue === void 0 ? void 0 : oldCapabilityValue.reason) === 'FeatureNotSupported' && capabilitiesChangedInfoAndRole.capabilitiesChangeInfo.reason === 'MeetingOptionOrOrganizerPolicyChanged') {
            continue;
        }
        // Do not show the first time the screenshare capability is present when the user's role is resolved to mirror
        // Teams behaviour
        if (capabilityName === 'shareScreen' && activeNotifications['shareScreen'] === undefined && newCapabilityValue.isPresent === true && (oldCapabilityValue === null || oldCapabilityValue === void 0 ? void 0 : oldCapabilityValue.isPresent) === false && capabilitiesChangedInfoAndRole.capabilitiesChangeInfo.reason === 'RoleChanged') {
            continue;
        }
        const newCapabilityChangeNotification = {
            capabilityName: capabilityName,
            isPresent: newCapabilityValue.isPresent,
            changedReason: capabilitiesChangedInfoAndRole.capabilitiesChangeInfo.reason,
            role: capabilitiesChangedInfoAndRole.participantRole,
            timestamp: new Date(Date.now())
        };
        activeNotifications[capabilityName] = newCapabilityChangeNotification;
    }
    return activeNotifications;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const useEndedCallConsoleErrors = (endedCall) => {
    var _a;
    React.useEffect(() => {
        var _a;
        switch ((_a = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _a === void 0 ? void 0 : _a.subCode) {
            case ROOM_NOT_FOUND_SUB_CODE:
                console.error('Call ended because the room id provided through the adapter locator could not be found');
                break;
            case ROOM_NOT_VALID_SUB_CODE:
                console.error('Call ended because this room is currently not valid.');
                break;
            case NOT_INVITED_TO_ROOM_SUB_CODE:
                console.error('Call ended because you are not invited to this room.');
                break;
            case INVITE_TO_ROOM_REMOVED_SUB_CODE:
                console.error('Call ended because your invite to this room has been removed.');
                break;
        }
    }, [(_a = endedCall === null || endedCall === void 0 ? void 0 : endedCall.callEndReason) === null || _a === void 0 ? void 0 : _a.subCode]);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const StarSurvey = (props) => {
    const { onStarRatingSelected } = props;
    const strings = useLocale().strings.call;
    const StarSurveyStrings = {
        starSurveyHelperText: strings.starSurveyHelperText,
        starSurveyOneStarText: strings.starSurveyOneStarText,
        starSurveyTwoStarText: strings.starSurveyTwoStarText,
        starSurveyThreeStarText: strings.starSurveyThreeStarText,
        starSurveyFourStarText: strings.starSurveyFourStarText,
        starSurveyFiveStarText: strings.starSurveyFiveStarText,
        starRatingAriaLabel: strings.starRatingAriaLabel
    };
    return React.createElement(_StarSurvey, { strings: StarSurveyStrings, onStarRatingSelected: onStarRatingSelected });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/** @private */
const TagsSurvey = (props) => {
    const strings = useLocale().strings.call;
    const tagsSurveyStrings = {
        tagsSurveyQuestion: strings.tagsSurveyQuestion,
        tagsSurveyHelperText: strings.tagsSurveyHelperText,
        tagsSurveyTextFieldDefaultText: strings.tagsSurveyTextFieldDefaultText
    };
    return React.createElement(_TagsSurvey, { callIssuesToTag: strings.surveyIssues, categoryHeadings: strings.surveyIssuesHeadingStrings, onConfirm: props.onConfirm, strings: tagsSurveyStrings, showFreeFormTextField: props.showFreeFormTextField });
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const SurveyContent = (props) => {
    const [showTagsSurvey, setShowTagsSurvey] = React.useState(false);
    const onStarRatingSelected = (ratings) => {
        if (ratings < 5) {
            setShowTagsSurvey(true);
            props.setShowSubmitFeedbackButton(true);
        }
        else {
            props.setShowSubmitFeedbackButton(true);
        }
        props.setRatings(ratings);
    };
    const onIssuesSelected = (selectedTags, improvementSuggestions) => {
        props.setIssuesSelected(selectedTags);
        props.setShowSubmitFeedbackButton(true);
        if (improvementSuggestions && props.setImprovementSuggestions) {
            props.setImprovementSuggestions(improvementSuggestions);
        }
    };
    return React.createElement(react.Stack, { "data-ui-id": "call-composite-survey" },
        React.createElement(StarSurvey, { onStarRatingSelected: onStarRatingSelected }),
        showTagsSurvey && React.createElement(TagsSurvey, { onConfirm: onIssuesSelected, showFreeFormTextField: !(props.setImprovementSuggestions === undefined) }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Page shown after the survey is submitted.
 * @private
 */
function ThankYouForFeedbackPage(props) {
    const strings = useLocale().strings.call;
    return React.createElement(react.Stack, { verticalFill: true, verticalAlign: "center", horizontalAlign: "center", "data-ui-id": 'thank-you-page', "aria-atomic": true },
        React.createElement(react.Stack, { className: react.mergeStyles(containerStyle), tokens: containerItemGap },
            props.iconName && React.createElement(CallCompositeIcon, { iconName: props.iconName }),
            React.createElement(react.Text, { className: react.mergeStyles(titleStyles), "aria-live": "assertive" }, strings.endOfSurveyText)));
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const questionTextStyle = (theme) => react.mergeStyles({
    fontWeight: 600,
    fontSize: _pxToRem(20),
    lineHeight: _pxToRem(20),
    color: theme.palette.neutralPrimary
});
/**
 * @private
 */
const surveyContainerStyle = (isMobile) => react.mergeStyles(isMobile ? {
    width: '20rem',
    position: 'absolute',
    top: '10%',
    left: '50%',
    transform: 'translateX(-50%)'
} : {
    width: '24rem',
    position: 'absolute',
    top: '20%',
    left: '50%',
    transform: 'translateX(-50%)'
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$a = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/** @private */
const Survey = (props) => {
    const { onSubmitSurvey, onSurveySubmittedCustom, onSurveyClosed, iconName, title, moreDetails, disableStartCallButton, isMobile } = props;
    const strings = useLocale().strings.call;
    const [ratings, setRatings] = React.useState(0);
    const [issuesSelected, setIssuesSelected] = React.useState();
    const [showSubmitFeedbackButton, setShowSubmitFeedbackButton] = React.useState(false);
    const [improvementSuggestions, setImprovementSuggestions] = React.useState({});
    const [showDefaultAfterSubmitScreen, setShowDefaultAfterSubmitScreen] = React.useState(false);
    const [showDefaultAfterDismissedScreen, setShowDefaultAfterDismissedScreen] = React.useState(false);
    const [submitButtonDisabled, setSubmitButtonDisabled] = React.useState(false);
    const theme = react.useTheme();
    return React.createElement(React.Fragment, null,
        showDefaultAfterSubmitScreen && React.createElement(ThankYouForFeedbackPage, { iconName: iconName }),
        showDefaultAfterDismissedScreen && React.createElement(NoticePage, { iconName: iconName, title: title, moreDetails: moreDetails, dataUiId: 'left-call-page', disableStartCallButton: disableStartCallButton }),
        !showDefaultAfterSubmitScreen && !showDefaultAfterDismissedScreen && React.createElement(react.Stack, { verticalAlign: "center", className: surveyContainerStyle(!!isMobile) },
            React.createElement(react.Text, { className: questionTextStyle(theme) }, strings.surveyTitle),
            React.createElement(SurveyContent, { setShowSubmitFeedbackButton: (showButton) => {
                    setShowSubmitFeedbackButton(showButton);
                }, setRatings: (rating) => {
                    setRatings(rating);
                }, setIssuesSelected: (issuesSelected) => {
                    setIssuesSelected(issuesSelected);
                }, setImprovementSuggestions: onSurveySubmittedCustom ? (improvementSuggestions) => {
                    setImprovementSuggestions(improvementSuggestions);
                } : undefined }),
            React.createElement(react.Stack, { horizontal: true, horizontalAlign: "end" },
                React.createElement(react.DefaultButton, { style: {
                        marginTop: '1rem',
                        marginRight: '0.5rem'
                    }, onClick: () => {
                        if (onSurveyClosed) {
                            onSurveyClosed('skipped');
                        }
                        else {
                            setShowDefaultAfterDismissedScreen(true);
                        }
                    } }, strings.surveySkipButtonLabel),
                showSubmitFeedbackButton && React.createElement(react.PrimaryButton, { disabled: submitButtonDisabled, style: {
                        marginTop: '1rem',
                        marginLeft: '0.5rem'
                    }, onClick: () => __awaiter$a(void 0, void 0, void 0, function* () {
                        const surveyResults = {
                            overallRating: {
                                score: ratings
                            }
                        };
                        if (issuesSelected === null || issuesSelected === void 0 ? void 0 : issuesSelected.overallRating) {
                            surveyResults.overallRating = {
                                score: ratings,
                                issues: issuesSelected.overallRating.issues
                            };
                        }
                        if (issuesSelected === null || issuesSelected === void 0 ? void 0 : issuesSelected.audioRating) {
                            surveyResults.audioRating = {
                                score: ratings,
                                issues: issuesSelected.audioRating.issues
                            };
                        }
                        if (issuesSelected === null || issuesSelected === void 0 ? void 0 : issuesSelected.screenshareRating) {
                            surveyResults.screenshareRating = {
                                score: ratings,
                                issues: issuesSelected.screenshareRating.issues
                            };
                        }
                        if (issuesSelected === null || issuesSelected === void 0 ? void 0 : issuesSelected.videoRating) {
                            surveyResults.videoRating = {
                                score: ratings,
                                issues: issuesSelected.videoRating.issues
                            };
                        }
                        if (onSubmitSurvey) {
                            // disable submit button while waiting for the survey to submit
                            setSubmitButtonDisabled(true);
                            // submitting survey results to calling
                            onSubmitSurvey(surveyResults).then(res => {
                                var _a, _b;
                                // if contoso provided callback to handle their own survey data, send over the submitted survey results
                                if (onSurveySubmittedCustom) {
                                    onSurveySubmittedCustom((_a = res === null || res === void 0 ? void 0 : res.callId) !== null && _a !== void 0 ? _a : '', (_b = res === null || res === void 0 ? void 0 : res.id) !== null && _b !== void 0 ? _b : '', surveyResults, improvementSuggestions).then(() => setSubmitButtonDisabled(false));
                                }
                                else {
                                    // if callback is not provided, enable the submit button after survey is submitted
                                    setSubmitButtonDisabled(false);
                                }
                                // redirect to new screen
                                if (onSurveyClosed) {
                                    // redirect to new screen according to contoso's callback set up
                                    onSurveyClosed('sent');
                                }
                                else {
                                    // if call back not provided, redirect to default screen
                                    setShowDefaultAfterSubmitScreen(true);
                                }
                            }).catch(e => {
                                // if there is an error submitting the survey, log the error in the console
                                console.log('error when submitting survey: ' + e);
                                // if contoso provided redirect callback, pass contoso the error so they can redirect to a corresponding error screen
                                if (onSurveyClosed) {
                                    onSurveyClosed('error', e);
                                }
                            });
                        }
                    }) }, strings.surveyConfirmButtonLabel))));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Generic page with a title and more details text for serving up a notice to the user.
 *
 * @private
 */
function SurveyPage(props) {
    var _a, _b;
    const handlers = useHandlers();
    return React.createElement(react.Stack, { verticalFill: true, verticalAlign: "center", horizontalAlign: "center", "data-ui-id": props.dataUiId, "aria-atomic": true },
        React.createElement(react.Stack, { className: react.mergeStyles(containerStyle$1), tokens: containerItemGap },
            React.createElement(Survey, Object.assign({}, handlers, props, { onSurveySubmittedCustom: (_a = props.surveyOptions) === null || _a === void 0 ? void 0 : _a.onSurveySubmitted, onSurveyClosed: (_b = props.surveyOptions) === null || _b === void 0 ? void 0 : _b.onSurveyClosed, isMobile: props.mobileView }))));
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$9 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const isShowing = (overrideSidePane) => {
    return !!(overrideSidePane === null || overrideSidePane === void 0 ? void 0 : overrideSidePane.isActive);
};
const MainScreen = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
    const adapter = useAdapter();
    const { camerasCount, microphonesCount } = useSelector$1(deviceCountSelector);
    const hasCameras = camerasCount > 0;
    const hasMicrophones = microphonesCount > 0;
    const role = useSelector$1(getRole);
    const { video: cameraHasPermission, audio: micHasPermission } = useSelector$1(devicePermissionSelector);
    React.useEffect(() => {
        (() => __awaiter$9(void 0, void 0, void 0, function* () {
            const constrain = getQueryOptions({
                role
            });
            const permissionsResult = yield adapter.askDevicePermission(constrain);
            if (permissionsResult === null || permissionsResult === void 0 ? void 0 : permissionsResult.audio) {
                adapter.queryMicrophones();
            }
            if (permissionsResult === null || permissionsResult === void 0 ? void 0 : permissionsResult.video) {
                adapter.queryCameras();
            }
            adapter.querySpeakers();
        }))();
    }, [adapter, role,
        // Ensure we re-ask for permissions if the number of devices goes from 0 -> n during a call
        // as we cannot request permissions when there are no devices.
        hasCameras, hasMicrophones,
        // Ensure we re-query for devices when permission for the device is granted.
        cameraHasPermission, micHasPermission]);
    const { callInvitationUrl, onFetchAvatarPersonaData, onFetchParticipantMenuItems } = props;
    const page = useSelector$1(getPage);
    const endedCall = useSelector$1(getEndedCall);
    const [sidePaneRenderer, setSidePaneRenderer] = React.useState();
    const [injectedSidePaneProps, setInjectedSidePaneProps] = React.useState();
    const [userSetGalleryLayout, setUserSetGalleryLayout] = React.useState((_c = (_b = (_a = props.options) === null || _a === void 0 ? void 0 : _a.galleryOptions) === null || _b === void 0 ? void 0 : _b.layout) !== null && _c !== void 0 ? _c : 'floatingLocalVideo');
    const [userSetOverflowGalleryPosition, setUserSetOverflowGalleryPosition] = React.useState('Responsive');
    const overridePropsRef = React.useRef(props.overrideSidePane);
    React.useEffect(() => {
        setInjectedSidePaneProps(props.overrideSidePane);
        // When the injected side pane is opened, clear the previous side pane active state.
        // this ensures when the injected side pane is "closed", the previous side pane is not "re-opened".
        if (!isShowing(overridePropsRef.current) && isShowing(props.overrideSidePane)) {
            setSidePaneRenderer(undefined);
        }
        overridePropsRef.current = props.overrideSidePane;
    }, [props.overrideSidePane]);
    const onSidePaneIdChange = props.onSidePaneIdChange;
    React.useEffect(() => {
        onSidePaneIdChange === null || onSidePaneIdChange === void 0 ? void 0 : onSidePaneIdChange(sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id);
    }, [sidePaneRenderer === null || sidePaneRenderer === void 0 ? void 0 : sidePaneRenderer.id, onSidePaneIdChange]);
    // When the call ends ensure the side pane is set to closed to prevent the side pane being open if the call is re-joined.
    React.useEffect(() => {
        const closeSidePane = () => {
            setSidePaneRenderer(undefined);
        };
        const resetUserGalleryLayout = () => {
            if (userSetGalleryLayout === 'togetherMode') {
                setUserSetGalleryLayout('floatingLocalVideo');
            }
        };
        const handleCallEnded = () => {
            closeSidePane();
            resetUserGalleryLayout();
        };
        adapter.on('callEnded', handleCallEnded);
        return () => {
            adapter.off('callEnded', handleCallEnded);
        };
    }, [adapter, userSetGalleryLayout]);
    const compositeAudioContext = useAudio();
    const capabilitiesChangedInfoAndRole = useSelector$1(capabilitiesChangedInfoAndRoleSelector);
    const capabilitiesChangedNotificationBarProps = useTrackedCapabilityChangedNotifications(capabilitiesChangedInfoAndRole);
    // Track the last dismissed errors of any error kind to prevent errors from re-appearing on subsequent page navigation
    // This works by tracking the most recent timestamp of any active error type.
    // And then tracking when that error type was last dismissed.
    const activeErrors = usePropsFor$1(ErrorBar).activeErrorMessages;
    const activeInCallErrors = usePropsFor$1(NotificationStack).activeErrorMessages;
    const activeNotifications = usePropsFor$1(NotificationStack).activeNotifications;
    const complianceProps = useSelector$1(complianceBannerSelector);
    const cachedProps = React.useRef({
        latestBooleanState: {
            callTranscribeState: false,
            callRecordState: false
        },
        latestStringState: {
            callTranscribeState: 'off',
            callRecordState: 'off'
        },
        lastUpdated: Date.now()
    });
    const complianceNotification = React.useMemo(() => {
        return computeComplianceNotification(complianceProps, cachedProps);
    }, [complianceProps, cachedProps]);
    React.useEffect(() => {
        if (complianceNotification) {
            for (let i = activeNotifications.length - 1; i >= 0; i--) {
                const notification = activeNotifications[i];
                if (notification && ['recordingStarted', 'transcriptionStarted', 'recordingStopped', 'transcriptionStopped', 'recordingAndTranscriptionStarted', 'recordingAndTranscriptionStopped', 'recordingStoppedStillTranscribing', 'transcriptionStoppedStillRecording'].includes(notification.type)) {
                    activeNotifications.splice(i, 1);
                }
            }
            activeNotifications.push(complianceNotification);
        }
        setTrackedNotifications(prev => updateTrackedNotificationsWithActiveNotifications(prev, activeNotifications));
    }, [complianceNotification, activeNotifications]);
    const [trackedErrors, setTrackedErrors] = React.useState({});
    const [trackedInCallErrors, setTrackedInCallErrors] = React.useState({});
    const [trackedNotifications, setTrackedNotifications] = React.useState({});
    React.useEffect(() => {
        setTrackedErrors(prev => updateTrackedNotificationsWithActiveNotifications(prev, activeErrors));
        setTrackedInCallErrors(prev => updateTrackedNotificationsWithActiveNotifications(prev, activeInCallErrors));
    }, [activeErrors, activeInCallErrors]);
    const onDismissError = React.useCallback((error) => {
        setTrackedErrors(prev => trackNotificationAsDismissed(error.type, prev));
        setTrackedInCallErrors(prev => trackNotificationAsDismissed(error.type, prev));
    }, []);
    const onDismissNotification = React.useCallback((notification) => {
        setTrackedNotifications(prev => trackNotificationAsDismissed(notification.type, prev));
    }, []);
    const latestErrors = React.useMemo(() => filterLatestNotifications(activeErrors, trackedErrors), [activeErrors, trackedErrors]);
    const latestInCallErrors = React.useMemo(() => filterLatestNotifications(activeInCallErrors, trackedInCallErrors), [activeInCallErrors, trackedInCallErrors]);
    const latestNotifications = React.useMemo(() => {
        const result = filterLatestNotifications(activeNotifications, trackedNotifications);
        // sort notifications by timestamp from earliest to latest
        result.sort((a, b) => { var _a, _b; return ((_a = a.timestamp) !== null && _a !== void 0 ? _a : new Date(Date.now())).getTime() - ((_b = b.timestamp) !== null && _b !== void 0 ? _b : new Date(Date.now())).getTime(); });
        return result;
    }, [activeNotifications, trackedNotifications]);
    const callees = useSelector$1(getTargetCallees);
    const locale = useLocale();
    const palette = useTheme().palette;
    const alternateCallerId = useSelector$1(getAlternateCallerId);
    const leavePageStyle = React.useMemo(() => leavingPageStyle(palette), [palette]);
    let pageElement;
    const [pinnedParticipants, setPinnedParticipants] = React.useState([]);
    switch (page) {
        case 'configuration':
            pageElement = React.createElement(ConfigurationPage, { joinCallOptions: (_d = props.options) === null || _d === void 0 ? void 0 : _d.joinCallOptions, mobileView: props.mobileView, startCallHandler: () => {
                    if (callees) {
                        adapter.startCall(callees, alternateCallerId ? {
                            alternateCallerId: {
                                phoneNumber: alternateCallerId
                            }
                        } : {});
                    }
                    else {
                        adapter.joinCall({
                            microphoneOn: 'keep',
                            cameraOn: 'keep'
                        });
                    }
                }, updateSidePaneRenderer: setSidePaneRenderer, latestErrors: latestErrors, onDismissError: onDismissError, modalLayerHostId: props.modalLayerHostId, capabilitiesChangedNotificationBarProps: capabilitiesChangedNotificationBarProps, logo: (_f = (_e = props.options) === null || _e === void 0 ? void 0 : _e.branding) === null || _f === void 0 ? void 0 : _f.logo, backgroundImage: (_h = (_g = props.options) === null || _g === void 0 ? void 0 : _g.branding) === null || _h === void 0 ? void 0 : _h.backgroundImage });
            break;
        case 'accessDeniedTeamsMeeting':
            pageElement = React.createElement(NoticePage, { iconName: "NoticePageAccessDeniedTeamsMeeting", title: locale.strings.call.failedToJoinTeamsMeetingReasonAccessDeniedTitle, moreDetails: locale.strings.call.failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails, dataUiId: 'access-denied-teams-meeting-page' });
            break;
        case 'removedFromCall':
            pageElement = React.createElement(NoticePage, { iconName: "NoticePageRemovedFromCall", title: locale.strings.call.removedFromCallTitle, moreDetails: locale.strings.call.removedFromCallMoreDetails, dataUiId: 'removed-from-call-page' });
            break;
        case 'joinCallFailedDueToNoNetwork':
            pageElement = React.createElement(NoticePage, { iconName: "NoticePageJoinCallFailedDueToNoNetwork", title: locale.strings.call.failedToJoinCallDueToNoNetworkTitle, moreDetails: locale.strings.call.failedToJoinCallDueToNoNetworkMoreDetails, dataUiId: 'join-call-failed-due-to-no-network-page' });
            break;
        case 'leaving':
            pageElement = React.createElement(NoticePage, { title: (_j = locale.strings.call.leavingCallTitle) !== null && _j !== void 0 ? _j : 'Leaving...', dataUiId: 'leaving-page', pageStyle: leavePageStyle, disableStartCallButton: true });
            break;
        case 'badRequest':
            {
                const { title, moreDetails, disableStartCallButton, iconName } = getEndedCallPageProps(locale, endedCall);
                pageElement = React.createElement(NoticePage, { iconName: iconName, title: title, moreDetails: callees ? '' : moreDetails, dataUiId: 'left-call-page', disableStartCallButton: disableStartCallButton });
                break;
            }
        case 'leftCall':
            {
                const { title, moreDetails, disableStartCallButton, iconName } = getEndedCallPageProps(locale, endedCall);
                if (!((_l = (_k = props.options) === null || _k === void 0 ? void 0 : _k.surveyOptions) === null || _l === void 0 ? void 0 : _l.disableSurvey)) {
                    pageElement = React.createElement(SurveyPage, { dataUiId: 'left-call-page', surveyOptions: (_m = props.options) === null || _m === void 0 ? void 0 : _m.surveyOptions, iconName: iconName, title: title, moreDetails: moreDetails, disableStartCallButton: disableStartCallButton, mobileView: props.mobileView });
                    break;
                }
                pageElement = React.createElement(NoticePage, { iconName: iconName, title: title, moreDetails: callees ? '' : moreDetails, dataUiId: 'left-call-page', disableStartCallButton: disableStartCallButton });
                break;
            }
        case 'lobby':
            pageElement = React.createElement(LobbyPage, { mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, options: props.options, updateSidePaneRenderer: setSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, latestErrors: latestInCallErrors, latestNotifications: latestNotifications, onDismissError: onDismissError, onDismissNotification: onDismissNotification, capabilitiesChangedNotificationBarProps: capabilitiesChangedNotificationBarProps });
            break;
        case 'transferring':
            pageElement = React.createElement(TransferPage, { mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, options: props.options, updateSidePaneRenderer: setSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, onFetchAvatarPersonaData: onFetchAvatarPersonaData, latestErrors: latestInCallErrors, latestNotifications: latestNotifications, onDismissError: onDismissError, onDismissNotification: onDismissNotification, capabilitiesChangedNotificationBarProps: capabilitiesChangedNotificationBarProps });
            break;
        case 'call':
            pageElement = React.createElement(CallPage, { callInvitationURL: callInvitationUrl, onFetchAvatarPersonaData: onFetchAvatarPersonaData, onFetchParticipantMenuItems: onFetchParticipantMenuItems, mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, options: props.options, updateSidePaneRenderer: setSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, onCloseChatPane: props.onCloseChatPane, latestErrors: latestInCallErrors, latestNotifications: latestNotifications, onDismissError: onDismissError, onDismissNotification: onDismissNotification, galleryLayout: userSetGalleryLayout, onUserSetGalleryLayoutChange: setUserSetGalleryLayout, onSetUserSetOverflowGalleryPosition: setUserSetOverflowGalleryPosition, userSetOverflowGalleryPosition: userSetOverflowGalleryPosition, capabilitiesChangedNotificationBarProps: capabilitiesChangedNotificationBarProps, pinnedParticipants: pinnedParticipants, setPinnedParticipants: setPinnedParticipants, compositeAudioContext: compositeAudioContext, disableAutoShowDtmfDialer: (_o = props.options) === null || _o === void 0 ? void 0 : _o.disableAutoShowDtmfDialer, notificationOptions: (_p = props.options) === null || _p === void 0 ? void 0 : _p.notificationOptions });
            break;
        case 'hold':
            pageElement = React.createElement(React.Fragment, null, React.createElement(HoldPage, { mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, options: props.options, updateSidePaneRenderer: setSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, latestErrors: latestInCallErrors, latestNotifications: latestNotifications, onDismissError: onDismissError, onDismissNotification: onDismissNotification, capabilitiesChangedNotificationBarProps: capabilitiesChangedNotificationBarProps }));
            break;
    }
    if (page === 'returningFromBreakoutRoom') {
        pageElement = React.createElement(CallPage, { callInvitationURL: callInvitationUrl, onFetchAvatarPersonaData: onFetchAvatarPersonaData, onFetchParticipantMenuItems: onFetchParticipantMenuItems, mobileView: props.mobileView, modalLayerHostId: props.modalLayerHostId, options: props.options, updateSidePaneRenderer: setSidePaneRenderer, mobileChatTabHeader: props.mobileChatTabHeader, onCloseChatPane: props.onCloseChatPane, latestErrors: latestInCallErrors, latestNotifications: latestNotifications, onDismissError: onDismissError, onDismissNotification: onDismissNotification, galleryLayout: userSetGalleryLayout, onUserSetGalleryLayoutChange: setUserSetGalleryLayout, onSetUserSetOverflowGalleryPosition: setUserSetOverflowGalleryPosition, userSetOverflowGalleryPosition: userSetOverflowGalleryPosition, capabilitiesChangedNotificationBarProps: capabilitiesChangedNotificationBarProps, pinnedParticipants: pinnedParticipants, setPinnedParticipants: setPinnedParticipants, compositeAudioContext: compositeAudioContext, disableAutoShowDtmfDialer: (_q = props.options) === null || _q === void 0 ? void 0 : _q.disableAutoShowDtmfDialer, notificationOptions: (_r = props.options) === null || _r === void 0 ? void 0 : _r.notificationOptions });
    }
    useEndedCallConsoleErrors(endedCall);
    if (!pageElement) {
        throw new Error('Invalid call composite page');
    }
    return React.createElement(SidePaneProvider, { sidePaneRenderer: sidePaneRenderer, overrideSidePane: injectedSidePaneProps }, pageElement);
};
/**
 * A customizable UI composite for calling experience.
 *
 * @remarks Call composite min width/height are as follow:
 * - mobile: 17.5rem x 21rem (280px x 336px, with default rem at 16px)
 * - desktop: 30rem x 22rem (480px x 352px, with default rem at 16px)
 *
 * @public
 */
const CallComposite = (props) => React.createElement(CallCompositeInner, Object.assign({}, props));
/** @private */
const CallCompositeInner = (props) => {
    const { adapter, callInvitationUrl, onFetchAvatarPersonaData, onFetchParticipantMenuItems, options, formFactor = 'desktop' } = props;
    const mobileView = formFactor === 'mobile';
    const modalLayerHostId = reactHooks.useId('modalLayerhost');
    const mainScreenContainerClassName = React.useMemo(() => {
        return mobileView ? mainScreenContainerStyleMobile : mainScreenContainerStyleDesktop;
    }, [mobileView]);
    return React.createElement("div", { className: mainScreenContainerClassName },
        React.createElement(BaseProvider, Object.assign({}, props),
            React.createElement(CallAdapterProvider, { adapter: adapter },
                React.createElement(MainScreen, { callInvitationUrl: callInvitationUrl, onFetchAvatarPersonaData: onFetchAvatarPersonaData, onFetchParticipantMenuItems: onFetchParticipantMenuItems, mobileView: mobileView, modalLayerHostId: modalLayerHostId, options: options, onSidePaneIdChange: props.onSidePaneIdChange, overrideSidePane: props.overrideSidePane, mobileChatTabHeader: props.mobileChatTabHeader, onCloseChatPane: props.onCloseChatPane }),
                // This layer host is for ModalLocalAndRemotePIP in SidePane. This LayerHost cannot be inside the SidePane
                // because when the SidePane is hidden, ie. style property display is 'none', it takes up no space. This causes problems when dragging
                // the Modal because the draggable bounds thinks it has no space and will always return to its initial position after dragging.
                // Additionally, this layer host cannot be in the Call Arrangement as it needs to be rendered before useMinMaxDragPosition() in
                // common/utils useRef is called.
                // Warning: this is fragile and works because the call arrangement page is only rendered after the call has connected and thus this
                // LayerHost will be guaranteed to have rendered (and subsequently mounted in the DOM). This ensures the DOM element will be available
                // before the call to `document.getElementById(modalLayerHostId)` is made.
                React.createElement(react.LayerHost, { id: modalLayerHostId, className: react.mergeStyles(modalLayerHostStyle) }))));
};
const getQueryOptions = (options) => {
    if (options.role === 'Consumer') {
        return {
            video: false,
            audio: true
        };
    }
    return {
        video: true,
        audio: true
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const compositeLogger = logger.createClientLogger('communication-react:composite');

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class ParticipantSubscriber {
    constructor(participant, emitter) {
        this.isMutedChangedHandler = () => {
            this.emitter.emit('isMutedChanged', {
                participantId: this.participant.identifier,
                isMuted: this.participant.isMuted
            });
        };
        this.displayNameChangedHandler = () => {
            this.emitter.emit('displayNameChanged', {
                participantId: this.participant.identifier,
                displayName: this.participant.displayName
            });
        };
        this.isSpeakingChangedHandler = () => {
            this.emitter.emit('isSpeakingChanged', {
                participantId: this.participant.identifier,
                isSpeaking: this.participant.isSpeaking
            });
        };
        this.videoStreamsUpdatedHandler = (event) => {
            this.emitter.emit('videoStreamsUpdated', event);
        };
        this.participant = participant;
        this.emitter = emitter;
        this.subscribeParticipantEvents();
    }
    subscribeParticipantEvents() {
        this.participant.on('isMutedChanged', this.isMutedChangedHandler);
        this.participant.on('displayNameChanged', this.displayNameChangedHandler);
        this.participant.on('isSpeakingChanged', this.isSpeakingChangedHandler);
        this.participant.on('videoStreamsUpdated', this.videoStreamsUpdatedHandler);
    }
    unsubscribeAll() {
        this.participant.off('isMutedChanged', this.isMutedChangedHandler);
        this.participant.off('displayNameChanged', this.displayNameChangedHandler);
        this.participant.off('isSpeakingChanged', this.isSpeakingChangedHandler);
        this.participant.off('videoStreamsUpdated', this.videoStreamsUpdatedHandler);
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
class DiagnosticsForwarder {
    constructor(emitter, call) {
        this.unsubscribe = () => {
            this._diagnostics.network.off('diagnosticChanged', this.networkDiagnosticsChanged.bind(this));
            this._diagnostics.media.off('diagnosticChanged', this.mediaDiagnosticsChanged.bind(this));
        };
        this._diagnostics = call.feature(communicationCalling.Features.UserFacingDiagnostics);
        this._emitter = emitter;
        this.subscribe();
    }
    subscribe() {
        this._diagnostics.network.on('diagnosticChanged', this.networkDiagnosticsChanged.bind(this));
        this._diagnostics.media.on('diagnosticChanged', this.mediaDiagnosticsChanged.bind(this));
    }
    networkDiagnosticsChanged(args) {
        const event = Object.assign({ type: 'network' }, args);
        this._emitter.emit('diagnosticChanged', event);
    }
    mediaDiagnosticsChanged(args) {
        const event = Object.assign({ type: 'media' }, args);
        this._emitter.emit('diagnosticChanged', event);
    }
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 *
 * This is used to create correct handler for generic agent type
 */
function createHandlers(callClient, callAgent, deviceManager, call, options) {
    // Call can be either undefined or ACS Call
    if (_isACSCallAgent(callAgent) && (!call || call && _isACSCall(call))) {
        return createDefaultCallingHandlers(callClient, callAgent, deviceManager, call, options);
    }
    if (_isTeamsCallAgent(callAgent) && (!call || call && _isTeamsCall(call))) {
        return createDefaultTeamsCallingHandlers(callClient, callAgent, deviceManager, call, options);
    }
    throw new Error('Unhandled agent type');
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$8 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * @private
 */
const createProfileStateModifier = (onFetchProfile, notifyUpdate) => {
    const cachedDisplayName = {};
    return (state) => {
        var _a;
        const originalParticipants = (_a = state.call) === null || _a === void 0 ? void 0 : _a.remoteParticipants;
        (() => __awaiter$8(void 0, void 0, void 0, function* () {
            let shouldNotifyUpdates = false;
            if (!originalParticipants) {
                return;
            }
            for (const [key, participant] of Object.entries(originalParticipants)) {
                if (cachedDisplayName[key]) {
                    continue;
                }
                const profile = yield onFetchProfile(key, {
                    displayName: participant.displayName
                });
                if ((profile === null || profile === void 0 ? void 0 : profile.displayName) && participant.displayName !== (profile === null || profile === void 0 ? void 0 : profile.displayName)) {
                    cachedDisplayName[key] = profile === null || profile === void 0 ? void 0 : profile.displayName;
                    shouldNotifyUpdates = true;
                }
            }
            // notify update only when there is a change, which most likely will trigger modifier and setState again
            if (shouldNotifyUpdates) {
                notifyUpdate();
            }
        }))();
        const participantsModifier = createParticipantModifier((id, participant) => {
            if (cachedDisplayName[id]) {
                return Object.assign(Object.assign({}, participant), { displayName: cachedDisplayName[id] });
            }
            return undefined;
        });
        return participantsModifier(state);
    };
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const CALL_REJECTED_CODE = 603;
const CALL_ENDED_CODE = 0;
const CALL_TRANSFER_SUBCODE = 7015;
/**
 * @private
 */
class CallingSoundSubscriber {
    constructor(call, callees, sounds) {
        this.playingSounds = false;
        this.onCallStateChanged = () => {
            this.call.on('stateChanged', () => {
                var _a, _b, _c, _d, _e, _f, _g;
                if (shouldPlayRinging(this.call, this.callees) && ((_a = this.soundsLoaded) === null || _a === void 0 ? void 0 : _a.callRingingSound)) {
                    this.soundsLoaded.callRingingSound.loop = true;
                    this.playSound(this.soundsLoaded.callRingingSound);
                    this.playingSounds = true;
                }
                if (!shouldPlayRinging(this.call, this.callees) && ((_b = this.soundsLoaded) === null || _b === void 0 ? void 0 : _b.callRingingSound)) {
                    this.soundsLoaded.callRingingSound.loop = false;
                    this.soundsLoaded.callRingingSound.pause();
                    this.playingSounds = false;
                }
                if (this.call.state === 'Disconnected') {
                    if (((_c = this.soundsLoaded) === null || _c === void 0 ? void 0 : _c.callBusySound) && ((_d = this.call.callEndReason) === null || _d === void 0 ? void 0 : _d.code) === CALL_REJECTED_CODE) {
                        this.playSound(this.soundsLoaded.callBusySound);
                        this.playingSounds = true;
                    }
                    else if (((_e = this.soundsLoaded) === null || _e === void 0 ? void 0 : _e.callEndedSound) && ((_f = this.call.callEndReason) === null || _f === void 0 ? void 0 : _f.code) === CALL_ENDED_CODE && ((_g = this.call.callEndReason) === null || _g === void 0 ? void 0 : _g.subCode) !== CALL_TRANSFER_SUBCODE) {
                        this.playSound(this.soundsLoaded.callEndedSound);
                        this.playingSounds = true;
                    }
                }
            });
        };
        this.call = call;
        this.callees = callees;
        if (sounds) {
            this.soundsLoaded = this.loadSounds(sounds);
            this.subscribeCallSoundEvents();
        }
    }
    subscribeCallSoundEvents() {
        this.onCallStateChanged();
    }
    unsubscribeAll() {
        var _a, _b;
        (_a = this.call) === null || _a === void 0 ? void 0 : _a.off('stateChanged', this.onCallStateChanged);
        if ((_b = this.soundsLoaded) === null || _b === void 0 ? void 0 : _b.callRingingSound) {
            this.soundsLoaded.callRingingSound.pause();
        }
    }
    pauseSounds() {
        var _a, _b, _c;
        if ((_a = this.soundsLoaded) === null || _a === void 0 ? void 0 : _a.callRingingSound) {
            this.soundsLoaded.callRingingSound.pause();
            this.playingSounds = false;
        }
        if ((_b = this.soundsLoaded) === null || _b === void 0 ? void 0 : _b.callEndedSound) {
            this.soundsLoaded.callEndedSound.pause();
            this.playingSounds = false;
        }
        if ((_c = this.soundsLoaded) === null || _c === void 0 ? void 0 : _c.callBusySound) {
            this.soundsLoaded.callBusySound.pause();
            this.playingSounds = false;
        }
    }
    loadSounds(sounds) {
        var _a, _b, _c;
        let callEndedSound;
        if (sounds === null || sounds === void 0 ? void 0 : sounds.callEnded) {
            callEndedSound = new Audio((_a = sounds === null || sounds === void 0 ? void 0 : sounds.callEnded) === null || _a === void 0 ? void 0 : _a.url);
            callEndedSound.preload = 'auto';
        }
        let callRingingSound;
        if (sounds === null || sounds === void 0 ? void 0 : sounds.callRinging) {
            callRingingSound = new Audio((_b = sounds === null || sounds === void 0 ? void 0 : sounds.callRinging) === null || _b === void 0 ? void 0 : _b.url);
            callRingingSound.preload = 'auto';
        }
        let callBusySound;
        if (sounds === null || sounds === void 0 ? void 0 : sounds.callBusy) {
            callBusySound = new Audio((_c = sounds === null || sounds === void 0 ? void 0 : sounds.callBusy) === null || _c === void 0 ? void 0 : _c.url);
            callBusySound.preload = 'auto';
        }
        return {
            callEndedSound,
            callRingingSound,
            callBusySound
        };
    }
    playSound(sound) {
        sound.play().catch(e => {
            console.error(e, 'Failed to play sound, check loader config to make sure it is correct');
        });
    }
}
/**
 * Helper function to allow the calling sound subscriber to determine when to play the ringing
 * sound when making an outbound call.
 */
const shouldPlayRinging = (call, callees) => {
    if (callees && callees[0] && (call.state === 'Ringing' || call.state === 'Connecting')) {
        return true;
    }
    else {
        return false;
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$7 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * For each time that we use the hook {@link useSelector} in the {@link CallComposite} we add another listener
 * to the `stateChanged` event on the this adapter. This number is set in relation to the number of
 * times that we are using the hook useSelector in the CallComposite.
 *
 * We will need to update this as the threshold is reached with more usages of useSelector.
 */
const EVENT_LISTENER_WARNING_WARNING_LIMIT = 125;
/** Context of call, which is a centralized context for all state updates */
class CallContext {
    constructor(clientState, isTeamsCall, isTeamsMeeting, isRoomsCall, options, targetCallees) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        this.emitter = new events.EventEmitter();
        this.isReturningFromBreakoutRoom = false;
        this.state = {
            isLocalPreviewMicrophoneEnabled: false,
            userId: clientState.userId,
            displayName: (_a = clientState.callAgent) === null || _a === void 0 ? void 0 : _a.displayName,
            devices: clientState.deviceManager,
            call: undefined,
            targetCallees: targetCallees,
            page: 'configuration',
            latestErrors: clientState.latestErrors,
            latestNotifications: clientState.latestNotifications,
            isTeamsCall,
            isTeamsMeeting,
            isRoomsCall,
            alternateCallerId: options === null || options === void 0 ? void 0 : options.alternateCallerId,
            environmentInfo: clientState.environmentInfo,
            videoBackgroundImages: (_b = options === null || options === void 0 ? void 0 : options.videoBackgroundOptions) === null || _b === void 0 ? void 0 : _b.videoBackgroundImages,
            onResolveVideoEffectDependency: (_c = options === null || options === void 0 ? void 0 : options.videoBackgroundOptions) === null || _c === void 0 ? void 0 : _c.onResolveDependency,
            onResolveDeepNoiseSuppressionDependency: (_d = options === null || options === void 0 ? void 0 : options.deepNoiseSuppressionOptions) === null || _d === void 0 ? void 0 : _d.onResolveDependency,
            deepNoiseSuppressionOnByDefault: (_f = (_e = options === null || options === void 0 ? void 0 : options.deepNoiseSuppressionOptions) === null || _e === void 0 ? void 0 : _e.deepNoiseSuppressionOnByDefault) !== null && _f !== void 0 ? _f : true,
            hideDeepNoiseSuppressionButton: (_h = (_g = options === null || options === void 0 ? void 0 : options.deepNoiseSuppressionOptions) === null || _g === void 0 ? void 0 : _g.hideDeepNoiseSuppressionButton) !== null && _h !== void 0 ? _h : false,
            selectedVideoBackgroundEffect: undefined,
            cameraStatus: undefined,
            sounds: options === null || options === void 0 ? void 0 : options.callingSounds,
            reactions: options === null || options === void 0 ? void 0 : options.reactionResources
        };
        this.emitter.setMaxListeners((_j = options === null || options === void 0 ? void 0 : options.maxListeners) !== null && _j !== void 0 ? _j : EVENT_LISTENER_WARNING_WARNING_LIMIT);
        this.bindPublicMethods();
        this.displayNameModifier = (options === null || options === void 0 ? void 0 : options.onFetchProfile) ? createProfileStateModifier(options.onFetchProfile, () => {
            this.setState(this.getState());
        }) : undefined;
    }
    bindPublicMethods() { }
    onStateChange(handler) {
        this.emitter.on('stateChanged', handler);
    }
    offStateChange(handler) {
        this.emitter.off('stateChanged', handler);
    }
    setState(state) {
        this.state = this.displayNameModifier ? this.displayNameModifier(state) : state;
        // This context privately tracks how captions was started to determine if captions is running only in the background.
        // If so we should not show the UI.
        this.state = captionsUIVisibilityModifier(this.state);
        this.emitter.emit('stateChanged', this.state);
    }
    updateCaptionsStartedInBackground(captionsStartedInBackground) {
        var _a;
        const captionsTriggers = (_a = this.state.captionsTriggers) !== null && _a !== void 0 ? _a : {};
        captionsTriggers.captionsStartedInBackground = captionsStartedInBackground;
        this.setState(Object.assign(Object.assign({}, this.state), { captionsTriggers }));
    }
    updateCaptionsStartedWithUI(captionsStartedWithUI) {
        var _a;
        const captionsTriggers = (_a = this.state.captionsTriggers) !== null && _a !== void 0 ? _a : {};
        captionsTriggers.captionsStartedWithUI = captionsStartedWithUI;
        this.setState(Object.assign(Object.assign({}, this.state), { captionsTriggers }));
    }
    getState() {
        return this.state;
    }
    setIsLocalMicrophoneEnabled(isLocalPreviewMicrophoneEnabled) {
        this.setState(Object.assign(Object.assign({}, this.state), { isLocalPreviewMicrophoneEnabled }));
    }
    // This is the key to find current call object in client state
    setCurrentCallId(callId) {
        this.callId = callId;
    }
    setTargetCallee(targetCallees) {
        this.setState(Object.assign(Object.assign({}, this.state), { targetCallees }));
    }
    setIsReturningFromBreakoutRoom(isReturningFromBreakoutRoom) {
        this.isReturningFromBreakoutRoom = isReturningFromBreakoutRoom;
    }
    onCallEnded(handler) {
        this.emitter.on('callEnded', handler);
    }
    offCallEnded(handler) {
        this.emitter.off('callEnded', handler);
    }
    updateClientState(clientState) {
        var _a, _b, _c;
        let call = this.callId ? clientState.calls[this.callId] : undefined;
        const latestEndedCall = clientState.callsEnded ? findLatestEndedCall(clientState.callsEnded) : undefined;
        // As the state is transitioning to a new state, trigger appropriate callback events.
        const oldPage = this.state.page;
        const latestAcceptedTransfer = (call === null || call === void 0 ? void 0 : call.transfer.acceptedTransfers) ? findLatestAcceptedTransfer(call.transfer.acceptedTransfers) : undefined;
        const transferCall = latestAcceptedTransfer ? clientState.calls[latestAcceptedTransfer.callId] : undefined;
        if ((call === null || call === void 0 ? void 0 : call.state) === 'Connected' || (call === null || call === void 0 ? void 0 : call.state) === 'Connecting') {
            this.setIsReturningFromBreakoutRoom(false);
        }
        let isReturningFromBreakoutRoom = false;
        isReturningFromBreakoutRoom = this.isReturningFromBreakoutRoom;
        const newPage = getCallCompositePage(call, latestEndedCall, transferCall, isReturningFromBreakoutRoom);
        if (!IsCallEndedPage(oldPage) && IsCallEndedPage(newPage)) {
            /**
             * We want to make sure that the id of the call that is ending
             * is the same as the call in the adapter as this is a scenario where
             * the call has ended and not been transferred and report the codes for this call.
             */
            if (this.callId === (latestEndedCall === null || latestEndedCall === void 0 ? void 0 : latestEndedCall.id)) {
                this.emitter.emit('callEnded', {
                    callId: latestEndedCall === null || latestEndedCall === void 0 ? void 0 : latestEndedCall.id,
                    code: (_a = latestEndedCall === null || latestEndedCall === void 0 ? void 0 : latestEndedCall.callEndReason) === null || _a === void 0 ? void 0 : _a.code,
                    subCode: (_b = latestEndedCall === null || latestEndedCall === void 0 ? void 0 : latestEndedCall.callEndReason) === null || _b === void 0 ? void 0 : _b.subCode
                });
            }
            // Reset the callId to undefined as the call has ended.
            this.setCurrentCallId(undefined);
            // Make sure that the call is set to undefined in the state.
            call = undefined;
        }
        if (this.state.page) {
            this.setState(Object.assign(Object.assign({}, this.state), { userId: clientState.userId, displayName: (_c = clientState.callAgent) === null || _c === void 0 ? void 0 : _c.displayName, call, page: newPage, endedCall: latestEndedCall, devices: clientState.deviceManager, latestErrors: clientState.latestErrors, latestNotifications: clientState.latestNotifications, cameraStatus: (call === null || call === void 0 ? void 0 : call.localVideoStreams.find(s => s.mediaStreamType === 'Video')) || clientState.deviceManager.unparentedViews.find(s => s.mediaStreamType === 'Video') ? 'On' : 'Off', acceptedTransferCallState: transferCall }));
        }
    }
    setBackroundPickerImages(videoBackgroundImages) {
        this.setState(Object.assign(Object.assign({}, this.state), { videoBackgroundImages }));
    }
    setSelectedVideoBackgroundEffect(selectedVideoBackgroundEffect) {
        this.setState(Object.assign(Object.assign({}, this.state), { selectedVideoBackgroundEffect }));
    }
    setAcceptedTransferCall(call) {
        this.setState(Object.assign(Object.assign({}, this.state), { acceptedTransferCallState: call }));
    }
}
const findLatestEndedCall = (calls) => {
    var _a, _b, _c, _d;
    const callStates = Object.values(calls);
    if (callStates.length === 0) {
        return undefined;
    }
    let latestCall = callStates[0];
    for (const call of callStates.slice(1)) {
        if (((_b = (_a = call.endTime) === null || _a === void 0 ? void 0 : _a.getTime()) !== null && _b !== void 0 ? _b : 0) > ((_d = (_c = latestCall === null || latestCall === void 0 ? void 0 : latestCall.endTime) === null || _c === void 0 ? void 0 : _c.getTime()) !== null && _d !== void 0 ? _d : 0)) {
            latestCall = call;
        }
    }
    return latestCall;
};
const findLatestAcceptedTransfer = (acceptedTransfers) => {
    var _a, _b, _c, _d;
    const acceptedTransferValues = Object.values(acceptedTransfers);
    if (acceptedTransferValues.length === 0) {
        return undefined;
    }
    let latestAcceptedTransfer = acceptedTransferValues[0];
    for (const acceptedTransfer of acceptedTransferValues.slice(1)) {
        if (((_b = (_a = acceptedTransfer.timestamp) === null || _a === void 0 ? void 0 : _a.getTime()) !== null && _b !== void 0 ? _b : 0) > ((_d = (_c = latestAcceptedTransfer === null || latestAcceptedTransfer === void 0 ? void 0 : latestAcceptedTransfer.timestamp) === null || _c === void 0 ? void 0 : _c.getTime()) !== null && _d !== void 0 ? _d : 0)) {
            latestAcceptedTransfer = acceptedTransfer;
        }
    }
    return latestAcceptedTransfer;
};
/**
 * @private
 */
class AzureCommunicationCallAdapter {
    get call() {
        return this._call;
    }
    set call(newCall) {
        this.resetDiagnosticsForwarder(newCall);
        this._call = newCall;
    }
    constructor(callClient, locatorOrTargetCalless, callAgent, deviceManager, options) {
        var _a, _b, _c;
        this.participantSubscribers = new Map();
        this.emitter = new events.EventEmitter();
        this.isMyMutedChanged = () => {
            var _a;
            this.emitter.emit('isMutedChanged', {
                participantId: this.getState().userId,
                isMuted: (_a = this.call) === null || _a === void 0 ? void 0 : _a.isMuted
            });
        };
        this.bindPublicMethods();
        this.callClient = callClient;
        this.callAgent = callAgent;
        this.targetCallees = getLocatorOrTargetCallees(locatorOrTargetCalless) === true ? locatorOrTargetCalless : undefined;
        this.locator = getLocatorOrTargetCallees(locatorOrTargetCalless) === false ? locatorOrTargetCalless : undefined;
        this.deviceManager = deviceManager;
        const isTeamsMeeting = this.locator ? 'meetingLink' in this.locator || 'meetingId' in this.locator : false;
        let isTeamsCall;
        (_a = this.targetCallees) === null || _a === void 0 ? void 0 : _a.forEach(callee => {
            if (communicationCommon.isMicrosoftTeamsUserIdentifier(callee) || communicationCommon.isMicrosoftTeamsAppIdentifier(callee)) {
                isTeamsCall = true;
            }
        });
        const isRoomsCall = this.locator ? 'roomId' in this.locator : false;
        this.onResolveVideoBackgroundEffectsDependency = (_b = options === null || options === void 0 ? void 0 : options.videoBackgroundOptions) === null || _b === void 0 ? void 0 : _b.onResolveDependency;
        this.onResolveDeepNoiseSuppressionDependency = (_c = options === null || options === void 0 ? void 0 : options.deepNoiseSuppressionOptions) === null || _c === void 0 ? void 0 : _c.onResolveDependency;
        this.context = new CallContext(callClient.getState(), !!isTeamsCall, isTeamsMeeting, isRoomsCall, options, this.targetCallees);
        this.context.onCallEnded(endCallData => this.emitter.emit('callEnded', endCallData));
        const onStateChange = (clientState) => {
            var _a, _b, _c, _d, _e;
            // unsubscribe when the instance gets disposed
            if (!this) {
                callClient.offStateChange(onStateChange);
                return;
            }
            // `updateClientState` searches for the current call from all the calls in the state using a cached `call.id`
            // from the call object. `call.id` can change during a call. We must update the cached `call.id` before
            // calling `updateClientState` so that we find the correct state object for the call even when `call.id`
            // has changed.
            // https://github.com/Azure/communication-ui-library/pull/1820
            if ((_a = this.call) === null || _a === void 0 ? void 0 : _a.id) {
                this.context.setCurrentCallId(this.call.id);
            }
            // if the call hits the connected state we want to pause all calling sounds if playing.
            if (((_b = this.call) === null || _b === void 0 ? void 0 : _b.state) === 'Connected' && ((_c = this.callingSoundSubscriber) === null || _c === void 0 ? void 0 : _c.playingSounds)) {
                this.callingSoundSubscriber.pauseSounds();
            }
            // If the call connects we need to clean up any previous unparentedViews
            if ((((_d = this.call) === null || _d === void 0 ? void 0 : _d.state) === 'InLobby' || ((_e = this.call) === null || _e === void 0 ? void 0 : _e.state) === 'Connected') && this.callClient.getState().deviceManager.unparentedViews.length > 0) {
                this.callClient.getState().deviceManager.unparentedViews.forEach(view => {
                    this.callClient.disposeView(undefined, undefined, view);
                });
            }
            this.context.updateClientState(clientState);
        };
        this.handlers = createHandlers(callClient, callAgent, deviceManager, undefined, {
            onResolveVideoBackgroundEffectsDependency: this.onResolveVideoBackgroundEffectsDependency,
            onResolveDeepNoiseSuppressionDependency: this.onResolveDeepNoiseSuppressionDependency
        });
        this.onClientStateChange = onStateChange;
        this.subscribeDeviceManagerEvents();
        this.callClient.onStateChange(onStateChange);
        if (this.callAgent.kind === 'CallAgent') {
            const onCallsUpdated = (args) => {
                var _a;
                if (!((_a = this.call) === null || _a === void 0 ? void 0 : _a.id)) {
                    return;
                }
                const removedCall = args.removed.find(call => { var _a; return call.id === ((_a = this.call) === null || _a === void 0 ? void 0 : _a.id); });
                if (!removedCall) {
                    return;
                }
                const removedCallState = this.callClient.getState().callsEnded[removedCall.id];
                if (!removedCallState) {
                    return;
                }
                const latestAcceptedTransfer = findLatestAcceptedTransfer(removedCallState.transfer.acceptedTransfers);
                const _callAgent = callAgent;
                const transferCall = _callAgent.calls.find((call) => call.id === (latestAcceptedTransfer === null || latestAcceptedTransfer === void 0 ? void 0 : latestAcceptedTransfer.callId));
                if (!transferCall) {
                    return;
                }
                this.processNewCall(transferCall);
            };
            this.callAgent.on('callsUpdated', onCallsUpdated);
        }
        if (this.callAgent.kind === 'TeamsCallAgent') {
            const onTeamsCallsUpdated = (args) => {
                var _a;
                if (!((_a = this.call) === null || _a === void 0 ? void 0 : _a.id)) {
                    return;
                }
                const removedCall = args.removed.find(call => { var _a; return call.id === ((_a = this.call) === null || _a === void 0 ? void 0 : _a.id); });
                if (!removedCall) {
                    return;
                }
                const removedCallState = this.callClient.getState().callsEnded[removedCall.id];
                if (!removedCallState) {
                    return;
                }
                const latestAcceptedTransfer = findLatestAcceptedTransfer(removedCallState.transfer.acceptedTransfers);
                const _callAgent = callAgent;
                const transferCall = _callAgent.calls.find((call) => call.id === (latestAcceptedTransfer === null || latestAcceptedTransfer === void 0 ? void 0 : latestAcceptedTransfer.callId));
                if (!transferCall) {
                    return;
                }
                this.processNewCall(transferCall);
            };
            this.callAgent.on('callsUpdated', onTeamsCallsUpdated);
        }
    }
    // TODO: update this to include the 'selectedCameraChanged' when calling adds it to the device manager
    subscribeDeviceManagerEvents() {
        this.deviceManager.on('selectedMicrophoneChanged', () => {
            this.emitter.emit('selectedMicrophoneChanged');
        });
        this.deviceManager.on('selectedSpeakerChanged', () => {
            this.emitter.emit('selectedSpeakerChanged');
        });
    }
    bindPublicMethods() {
        this.onStateChange.bind(this);
        this.offStateChange.bind(this);
        this.getState.bind(this);
        this.dispose.bind(this);
        this.joinCall.bind(this);
        this.leaveCall.bind(this);
        this.setCamera.bind(this);
        this.setMicrophone.bind(this);
        this.setSpeaker.bind(this);
        this.askDevicePermission.bind(this);
        this.queryCameras.bind(this);
        this.queryMicrophones.bind(this);
        this.querySpeakers.bind(this);
        this.startCamera.bind(this);
        this.stopCamera.bind(this);
        this.mute.bind(this);
        this.unmute.bind(this);
        this.startCall.bind(this);
        this.startScreenShare.bind(this);
        this.stopScreenShare.bind(this);
        this.raiseHand.bind(this);
        this.onReactionClick.bind(this);
        this.lowerHand.bind(this);
        this.removeParticipant.bind(this);
        this.createStreamView.bind(this);
        this.disposeStreamView.bind(this);
        this.createTogetherModeStreamView.bind(this);
        this.startTogetherMode.bind(this);
        this.setTogetherModeSceneSize.bind(this);
        this.disposeTogetherModeStreamView.bind(this);
        this.disposeScreenShareStreamView.bind(this);
        this.disposeRemoteVideoStreamView.bind(this);
        this.disposeLocalVideoStreamView.bind(this);
        this.on.bind(this);
        this.off.bind(this);
        this.processNewCall.bind(this);
        this.addParticipant.bind(this);
        this.holdCall.bind(this);
        this.resumeCall.bind(this);
        this.sendDtmfTone.bind(this);
        this.startCaptions.bind(this);
        this.stopCaptions.bind(this);
        this.setSpokenLanguage.bind(this);
        this.setCaptionLanguage.bind(this);
        this.sendRealTimeText.bind(this);
        this.startVideoBackgroundEffect.bind(this);
        this.stopVideoBackgroundEffects.bind(this);
        this.updateBackgroundPickerImages.bind(this);
        this.startNoiseSuppressionEffect.bind(this);
        this.stopNoiseSuppressionEffect.bind(this);
        this.submitSurvey.bind(this);
        this.startSpotlight.bind(this);
        this.stopSpotlight.bind(this);
        this.stopAllSpotlight.bind(this);
        this.muteParticipant.bind(this);
        this.muteAllRemoteParticipants.bind(this);
        this.forbidOthersAudio.bind(this);
        this.permitOthersAudio.bind(this);
        this.forbidOthersAudio.bind(this);
        this.permitOthersAudio.bind(this);
    }
    dispose() {
        this.resetDiagnosticsForwarder();
        this.callClient.offStateChange(this.onClientStateChange);
        this.callAgent.dispose();
    }
    queryCameras() {
        return __awaiter$7(this, void 0, void 0, function* () {
            const startTime = new Date().getTime();
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                const cameras = yield this.deviceManager.getCameras();
                const endTime = new Date().getTime();
                compositeLogger.info('time to query cameras', endTime - startTime, 'ms');
                return cameras;
            }));
        });
    }
    queryMicrophones() {
        return __awaiter$7(this, void 0, void 0, function* () {
            const startTime = new Date().getTime();
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                const microphones = yield this.deviceManager.getMicrophones();
                const endTime = new Date().getTime();
                compositeLogger.info('time to query microphones', endTime - startTime, 'ms');
                return microphones;
            }));
        });
    }
    querySpeakers() {
        return __awaiter$7(this, void 0, void 0, function* () {
            const startTime = new Date().getTime();
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                const speakers = (yield this.deviceManager.isSpeakerSelectionAvailable) ? this.deviceManager.getSpeakers() : [];
                const endTime = new Date().getTime();
                compositeLogger.info('time to query speakers', endTime - startTime, 'ms');
                return speakers;
            }));
        });
    }
    askDevicePermission(constrain) {
        return __awaiter$7(this, void 0, void 0, function* () {
            const startTime = new Date().getTime();
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                const result = yield this.deviceManager.askDevicePermission(constrain);
                const endTime = new Date().getTime();
                compositeLogger.info('time to query askDevicePermissions', endTime - startTime, 'ms');
                return result;
            }));
        });
    }
    joinCall(options) {
        var _a, _b;
        if (_isInCall((_b = (_a = this.getState().call) === null || _a === void 0 ? void 0 : _a.state) !== null && _b !== void 0 ? _b : 'None')) {
            throw new Error('You are already in the call!');
        }
        else if (this.locator === undefined) {
            throw new Error('Locator is not defined!');
        }
        return this.teeErrorToEventEmitter(() => {
            // Default to keeping camera/mic on if no override argument specified
            let shouldCameraBeOnInCall = this.getState().cameraStatus === 'On';
            let shouldMicrophoneBeOnInCall = this.getState().isLocalPreviewMicrophoneEnabled;
            // Apply override arguments
            if (typeof options === 'boolean') {
                // Deprecated joinCall API (boolen)
                shouldMicrophoneBeOnInCall = options;
            }
            else if (typeof options === 'object') {
                // Options bag API
                if (options.microphoneOn && options.microphoneOn !== 'keep') {
                    shouldMicrophoneBeOnInCall = options.microphoneOn;
                }
                if (options.cameraOn && options.cameraOn !== 'keep') {
                    shouldCameraBeOnInCall = options.cameraOn;
                }
            }
            const audioOptions = {
                muted: !shouldMicrophoneBeOnInCall
            };
            const selectedCamera = getSelectedCameraFromAdapterState(this.getState());
            const videoOptions = selectedCamera && shouldCameraBeOnInCall ? {
                localVideoStreams: [new communicationCalling.LocalVideoStream(selectedCamera)]
            } : {};
            const call = this._joinCall(audioOptions, videoOptions);
            this.originCall = call;
            this.processNewCall(call);
            return call;
        });
    }
    _joinCall(audioOptions, videoOptions) {
        const isTeamsMeeting = this.locator ? 'meetingLink' in this.locator : false;
        const isTeamsMeetingId = this.locator ? 'meetingId' in this.locator : false;
        const isRoomsCall = this.locator ? 'roomId' in this.locator : false;
        if (_isTeamsCallAgent(this.callAgent)) {
            if (isTeamsMeeting) {
                return this.callAgent.join(this.locator, {
                    audioOptions,
                    videoOptions
                });
            }
            if (isTeamsMeetingId) {
                return this.callAgent.join(this.locator, {
                    audioOptions,
                    videoOptions
                });
            }
            throw new Error('Locator not supported by TeamsCallAgent');
        }
        if (isTeamsMeeting) {
            return this.callAgent.join(this.locator, {
                audioOptions,
                videoOptions
            });
        }
        if (isTeamsMeetingId) {
            return this.callAgent.join(this.locator, {
                audioOptions,
                videoOptions
            });
        }
        if (isRoomsCall) {
            return this.callAgent.join(this.locator, {
                audioOptions,
                videoOptions
            });
        }
        return this.callAgent.join(this.locator, {
            audioOptions,
            videoOptions
        });
    }
    createStreamView(remoteUserId, options) {
        return __awaiter$7(this, void 0, void 0, function* () {
            if (remoteUserId === undefined) {
                return yield this.handlers.onCreateLocalStreamView(options);
            }
            else {
                return yield this.handlers.onCreateRemoteStreamView(remoteUserId, options);
            }
        });
    }
    disposeStreamView(remoteUserId) {
        return __awaiter$7(this, void 0, void 0, function* () {
            if (remoteUserId === undefined) {
                yield this.handlers.onDisposeLocalStreamView();
            }
            else {
                yield this.handlers.onDisposeRemoteStreamView(remoteUserId);
            }
        });
    }
    disposeScreenShareStreamView(remoteUserId) {
        return __awaiter$7(this, void 0, void 0, function* () {
            if (remoteUserId !== '') {
                yield this.handlers.onDisposeRemoteScreenShareStreamView(remoteUserId);
            }
            else {
                yield this.handlers.onDisposeLocalScreenShareStreamView();
            }
        });
    }
    disposeRemoteVideoStreamView(remoteUserId) {
        return __awaiter$7(this, void 0, void 0, function* () {
            yield this.handlers.onDisposeRemoteVideoStreamView(remoteUserId);
        });
    }
    disposeLocalVideoStreamView() {
        return __awaiter$7(this, void 0, void 0, function* () {
            yield this.handlers.onDisposeLocalStreamView();
        });
    }
    createTogetherModeStreamView(options) {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.handlers.onCreateTogetherModeStreamView(options);
        });
    }
    startTogetherMode() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.handlers.onStartTogetherMode();
        });
    }
    setTogetherModeSceneSize(width, height) {
        return this.handlers.onSetTogetherModeSceneSize(width, height);
    }
    disposeTogetherModeStreamView() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.handlers.onDisposeTogetherModeStreamView();
        });
    }
    leaveCall(forEveryone) {
        return __awaiter$7(this, void 0, void 0, function* () {
            if (this.getState().page === 'transferring') {
                const transferCall = this.callAgent.calls.filter(call => { var _a; return call.id === ((_a = this.getState().acceptedTransferCallState) === null || _a === void 0 ? void 0 : _a.id); })[0];
                transferCall === null || transferCall === void 0 ? void 0 : transferCall.hangUp();
            }
            yield this.handlers.onHangUp(forEveryone);
            this.unsubscribeCallEvents();
            this.handlers = createHandlers(this.callClient, this.callAgent, this.deviceManager, this.call, {
                onResolveVideoBackgroundEffectsDependency: this.onResolveVideoBackgroundEffectsDependency,
                onResolveDeepNoiseSuppressionDependency: this.onResolveDeepNoiseSuppressionDependency
            });
            // We set the adapter.call object to undefined immediately when a call is ended.
            // We do not set the context.callId to undefined because it is a part of the immutable data flow loop.
            this.call = undefined;
            this.stopCamera();
            this.mute();
        });
    }
    setCamera(device, options) {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                yield this.handlers.onSelectCamera(device, options);
            }));
        });
    }
    setMicrophone(device) {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                yield this.handlers.onSelectMicrophone(device);
            }));
        });
    }
    setSpeaker(device) {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                yield this.handlers.onSelectSpeaker(device);
            }));
        });
    }
    startCamera(options) {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                if (!isCameraOn(this.getState())) {
                    // First kick off the effect on the local device before starting the camera in the call.
                    // This prevents the effect not being applied for a brief moment when the camera is started.
                    {
                        const selectedEffect = this.getState().selectedVideoBackgroundEffect;
                        const selectedCamera = getSelectedCameraFromAdapterState(this.getState());
                        if (selectedEffect && selectedCamera && this.onResolveVideoBackgroundEffectsDependency) {
                            const stream = new communicationCalling.LocalVideoStream(selectedCamera);
                            const effect = getBackgroundEffectFromSelectedEffect(selectedEffect, yield this.onResolveVideoBackgroundEffectsDependency());
                            if (effect) {
                                yield stream.feature(communicationCalling.Features.VideoEffects).startEffects(effect);
                            }
                            else {
                                yield stream.feature(communicationCalling.Features.VideoEffects).stopEffects();
                            }
                        }
                    }
                    yield this.handlers.onToggleCamera(options);
                }
            }));
        });
    }
    stopCamera() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                if (isCameraOn(this.getState())) {
                    yield this.handlers.onToggleCamera();
                }
            }));
        });
    }
    mute() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                var _a, _b;
                this.context.setIsLocalMicrophoneEnabled(false);
                if (_isInCall((_a = this.call) === null || _a === void 0 ? void 0 : _a.state) && !((_b = this.call) === null || _b === void 0 ? void 0 : _b.isMuted)) {
                    yield this.handlers.onToggleMicrophone();
                }
            }));
        });
    }
    unmute() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                var _a, _b, _c;
                this.context.setIsLocalMicrophoneEnabled(true);
                if ((_isInCall((_a = this.call) === null || _a === void 0 ? void 0 : _a.state) || _isInLobbyOrConnecting((_b = this.call) === null || _b === void 0 ? void 0 : _b.state)) && ((_c = this.call) === null || _c === void 0 ? void 0 : _c.isMuted)) {
                    yield this.handlers.onToggleMicrophone();
                }
            }));
        });
    }
    startScreenShare() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                var _a;
                if (!((_a = this.call) === null || _a === void 0 ? void 0 : _a.isScreenSharingOn)) {
                    yield this.handlers.onToggleScreenShare();
                }
            }));
        });
    }
    stopScreenShare() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                var _a;
                if ((_a = this.call) === null || _a === void 0 ? void 0 : _a.isScreenSharingOn) {
                    yield this.handlers.onToggleScreenShare();
                }
            }));
        });
    }
    raiseHand() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                yield this.handlers.onToggleRaiseHand();
            }));
        });
    }
    lowerHand() {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                yield this.handlers.onToggleRaiseHand();
            }));
        });
    }
    onReactionClick(reaction) {
        return __awaiter$7(this, void 0, void 0, function* () {
            return yield this.asyncTeeErrorToEventEmitter(() => __awaiter$7(this, void 0, void 0, function* () {
                yield this.handlers.onReactionClick(reaction);
            }));
        });
    }
    startVideoBackgroundEffect(videoBackgroundEffect) {
        return __awaiter$7(this, void 0, void 0, function* () {
            if (this.isBlurEffect(videoBackgroundEffect)) {
                const blurConfig = videoBackgroundEffect;
                yield this.handlers.onBlurVideoBackground(blurConfig);
            }
            else if (this.isReplacementEffect(videoBackgroundEffect)) {
                const replaceConfig = videoBackgroundEffect;
                yield this.handlers.onReplaceVideoBackground(replaceConfig);
            }
        });
    }
    stopVideoBackgroundEffects() {
        return __awaiter$7(this, void 0, void 0, function* () {
            yield this.handlers.onRemoveVideoBackgroundEffects();
        });
    }
    updateBackgroundPickerImages(backgroundImages) {
        this.context.setBackroundPickerImages(backgroundImages);
    }
    updateSelectedVideoBackgroundEffect(selectedVideoBackground) {
        this.context.setSelectedVideoBackgroundEffect(selectedVideoBackground);
    }
    startNoiseSuppressionEffect() {
        return __awaiter$7(this, void 0, void 0, function* () {
            yield this.handlers.onStartNoiseSuppressionEffect();
        });
    }
    stopNoiseSuppressionEffect() {
        return __awaiter$7(this, void 0, void 0, function* () {
            yield this.handlers.onStopNoiseSuppressionEffect();
        });
    }
    startCall(participants, options) {
        var _a, _b;
        if (_isInCall((_b = (_a = this.getState().call) === null || _a === void 0 ? void 0 : _a.state) !== null && _b !== void 0 ? _b : 'None')) {
            throw new Error('You are already in the call.');
        }
        const isCameraOn = this.getState().cameraStatus === 'On';
        const selectedCamera = getSelectedCameraFromAdapterState(this.getState());
        /* we only configure the video options here since the Calling SDK always unmutes the participant when starting a call */
        const startCallVideoOptions = selectedCamera ? {
            videoOptions: isCameraOn ? {
                localVideoStreams: [new communicationCalling.LocalVideoStream(selectedCamera)]
            } : undefined
        } : {};
        const combinedCallOptions = Object.assign(Object.assign({}, startCallVideoOptions), options);
        const idsToAdd = participants.map(participant => {
            const backendId = _toCommunicationIdentifier(participant);
            if ('phoneNumber' in backendId) {
                if ((options === null || options === void 0 ? void 0 : options.alternateCallerId) === undefined) {
                    throw new Error('Unable to start call, PSTN user present with no alternateCallerId.');
                }
            }
            return backendId;
        });
        this.context.setTargetCallee(idsToAdd);
        const call = this.handlers.onStartCall(idsToAdd, combinedCallOptions);
        if (!call) {
            throw new Error('Unable to start call.');
        }
        this.processNewCall(call);
        return call;
    }
    processNewCall(call) {
        this.call = call;
        this.context.setCurrentCallId(call.id);
        // Resync state after callId is set
        this.context.updateClientState(this.callClient.getState());
        this.handlers = createHandlers(this.callClient, this.callAgent, this.deviceManager, this.call, {
            onResolveVideoBackgroundEffectsDependency: this.onResolveVideoBackgroundEffectsDependency,
            onResolveDeepNoiseSuppressionDependency: this.onResolveDeepNoiseSuppressionDependency
        });
        this.subscribeCallEvents();
    }
    isBlurEffect(effect) {
        return effect.effectName === 'blur';
    }
    isReplacementEffect(effect) {
        return effect.effectName === 'replacement';
    }
    removeParticipant(userId) {
        return __awaiter$7(this, void 0, void 0, function* () {
            let participant = userId;
            participant = _toCommunicationIdentifier(userId);
            this.handlers.onRemoveParticipant(participant);
        });
    }
    addParticipant(participant, options) {
        return __awaiter$7(this, void 0, void 0, function* () {
            if (communicationCommon.isPhoneNumberIdentifier(participant) && options) {
                this.handlers.onAddParticipant(participant, options);
            }
            else if (communicationCommon.isCommunicationUserIdentifier(participant)) {
                this.handlers.onAddParticipant(participant);
            }
        });
    }
    holdCall() {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            if (((_a = this.call) === null || _a === void 0 ? void 0 : _a.state) !== 'LocalHold') {
                if ((_b = this.call) === null || _b === void 0 ? void 0 : _b.isLocalVideoStarted) {
                    this.stopCamera().then(() => {
                        this.handlers.onToggleHold();
                    });
                }
                else {
                    this.handlers.onToggleHold();
                }
            }
        });
    }
    resumeCall() {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a;
            if (((_a = this.call) === null || _a === void 0 ? void 0 : _a.state) === 'LocalHold') {
                this.handlers.onToggleHold().then(() => {
                    var _a;
                    if (((_a = this.call) === null || _a === void 0 ? void 0 : _a.feature(communicationCalling.Features.Capabilities).capabilities.turnVideoOn.isPresent) === false) {
                        this.stopCamera();
                    }
                });
            }
        });
    }
    sendDtmfTone(dtmfTone) {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onSendDtmfTone(dtmfTone);
        });
    }
    startCaptions(options) {
        return __awaiter$7(this, void 0, void 0, function* () {
            if (options === null || options === void 0 ? void 0 : options.startInBackground) {
                this.context.updateCaptionsStartedInBackground(true);
            }
            else {
                this.context.updateCaptionsStartedWithUI(true);
            }
            this.handlers.onStartCaptions(options);
        });
    }
    stopCaptions(options) {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a;
            if (options === null || options === void 0 ? void 0 : options.stopInBackground) {
                this.context.updateCaptionsStartedInBackground(false);
            }
            this.context.updateCaptionsStartedWithUI(false);
            // Only stop captions if they are not still running in the background
            if (!((_a = this.context.getState().captionsTriggers) === null || _a === void 0 ? void 0 : _a.captionsStartedInBackground)) {
                this.handlers.onStopCaptions();
            }
        });
    }
    setCaptionLanguage(language) {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onSetCaptionLanguage(language);
        });
    }
    setSpokenLanguage(language) {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onSetSpokenLanguage(language);
        });
    }
    sendRealTimeText(text, isFinalized) {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onSendRealTimeText(text, isFinalized);
        });
    }
    submitSurvey(survey) {
        return __awaiter$7(this, void 0, void 0, function* () {
            return this.handlers.onSubmitSurvey(survey);
        });
    }
    muteParticipant(userId) {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onMuteParticipant(userId);
        });
    }
    muteAllRemoteParticipants() {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onMuteAllRemoteParticipants();
        });
    }
    startSpotlight(userIds) {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onStartSpotlight(userIds);
        });
    }
    stopSpotlight(userIds) {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onStopSpotlight(userIds);
        });
    }
    stopAllSpotlight() {
        return __awaiter$7(this, void 0, void 0, function* () {
            this.handlers.onStopAllSpotlight();
        });
    }
    forbidAudio(userIds) {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onForbidAudio) === null || _b === void 0 ? void 0 : _b.call(_a, userIds);
        });
    }
    permitAudio(userIds) {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onPermitAudio) === null || _b === void 0 ? void 0 : _b.call(_a, userIds);
        });
    }
    forbidOthersAudio() {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onForbidOthersAudio) === null || _b === void 0 ? void 0 : _b.call(_a);
        });
    }
    permitOthersAudio() {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onPermitOthersAudio) === null || _b === void 0 ? void 0 : _b.call(_a);
        });
    }
    forbidVideo(userIds) {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onForbidVideo) === null || _b === void 0 ? void 0 : _b.call(_a, userIds);
        });
    }
    permitVideo(userIds) {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onPermitVideo) === null || _b === void 0 ? void 0 : _b.call(_a, userIds);
        });
    }
    forbidOthersVideo() {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onForbidOthersVideo) === null || _b === void 0 ? void 0 : _b.call(_a);
        });
    }
    permitOthersVideo() {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b;
            (_b = (_a = this.handlers).onPermitOthersVideo) === null || _b === void 0 ? void 0 : _b.call(_a);
        });
    }
    returnFromBreakoutRoom() {
        return __awaiter$7(this, void 0, void 0, function* () {
            var _a, _b, _c;
            const callState = ((_a = this.call) === null || _a === void 0 ? void 0 : _a.id) ? this.callClient.getState().calls[this.call.id] : undefined;
            const assignedBreakoutRoom = (_b = callState === null || callState === void 0 ? void 0 : callState.breakoutRooms) === null || _b === void 0 ? void 0 : _b.assignedBreakoutRoom;
            if (!assignedBreakoutRoom) {
                throw new Error('Could not return from breakout room because assigned breakout room state could not be retrieved.');
            }
            this.context.setIsReturningFromBreakoutRoom(true);
            const mainMeeting = yield assignedBreakoutRoom.returnToMainMeeting();
            // Mute main meeting if the breakout room was muted
            if ((_c = this.call) === null || _c === void 0 ? void 0 : _c.isMuted) {
                mainMeeting.mute();
            }
            this.originCall = mainMeeting;
            this.processNewCall(mainMeeting);
        });
    }
    getState() {
        return this.context.getState();
    }
    onStateChange(handler) {
        this.context.onStateChange(handler);
    }
    offStateChange(handler) {
        this.context.offStateChange(handler);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    on(event, listener) {
        this.emitter.on(event, listener);
    }
    subscribeToCaptionEvents() {
        var _a;
        const captionsFeature = (_a = this.call) === null || _a === void 0 ? void 0 : _a.feature(communicationCalling.Features.Captions);
        if ((captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.captions.kind) === 'TeamsCaptions' && (this.context.getState().isTeamsCall || this.context.getState().isTeamsMeeting)) {
            const teamsCaptionsFeature = captionsFeature.captions;
            teamsCaptionsFeature.on('CaptionsReceived', this.teamsCaptionsReceived.bind(this));
            teamsCaptionsFeature.on('CaptionsActiveChanged', this.isCaptionsActiveChanged.bind(this));
            teamsCaptionsFeature.on('CaptionLanguageChanged', this.isCaptionLanguageChanged.bind(this));
            teamsCaptionsFeature.on('SpokenLanguageChanged', this.isSpokenLanguageChanged.bind(this));
        }
        else {
            const acsCaptionsFeature = captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.captions;
            acsCaptionsFeature.on('CaptionsReceived', this.captionsReceived.bind(this));
            acsCaptionsFeature.on('CaptionsActiveChanged', this.isCaptionsActiveChanged.bind(this));
            acsCaptionsFeature.on('SpokenLanguageChanged', this.isSpokenLanguageChanged.bind(this));
            captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.on('CaptionsKindChanged', this.captionsKindChanged.bind(this));
        }
    }
    unsubscribeFromCaptionEvents() {
        var _a;
        const captionsFeature = (_a = this.call) === null || _a === void 0 ? void 0 : _a.feature(communicationCalling.Features.Captions);
        if ((captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.captions.kind) === 'TeamsCaptions' && (this.context.getState().isTeamsCall || this.context.getState().isTeamsMeeting)) {
            const teamsCaptionsFeature = captionsFeature.captions;
            teamsCaptionsFeature.off('CaptionsReceived', this.teamsCaptionsReceived.bind(this));
            teamsCaptionsFeature.off('CaptionsActiveChanged', this.isCaptionsActiveChanged.bind(this));
            teamsCaptionsFeature.off('CaptionLanguageChanged', this.isCaptionLanguageChanged.bind(this));
            teamsCaptionsFeature.off('SpokenLanguageChanged', this.isSpokenLanguageChanged.bind(this));
        }
        else {
            const acsCaptionsFeature = captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.captions;
            acsCaptionsFeature.off('CaptionsReceived', this.captionsReceived.bind(this));
            acsCaptionsFeature.off('CaptionsActiveChanged', this.isCaptionsActiveChanged.bind(this));
            acsCaptionsFeature.off('SpokenLanguageChanged', this.isSpokenLanguageChanged.bind(this));
            captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.off('CaptionsKindChanged', this.captionsKindChanged.bind(this));
        }
    }
    subscribeCallEvents() {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        if (this.call) {
            this.callingSoundSubscriber = new CallingSoundSubscriber(this.call, this.getState().targetCallees, this.getState().sounds);
        }
        (_a = this.call) === null || _a === void 0 ? void 0 : _a.on('remoteParticipantsUpdated', this.onRemoteParticipantsUpdated.bind(this));
        (_b = this.call) === null || _b === void 0 ? void 0 : _b.on('isMutedChanged', this.isMyMutedChanged.bind(this));
        (_c = this.call) === null || _c === void 0 ? void 0 : _c.on('isScreenSharingOnChanged', this.isScreenSharingOnChanged.bind(this));
        (_d = this.call) === null || _d === void 0 ? void 0 : _d.on('idChanged', this.callIdChanged.bind(this));
        (_e = this.call) === null || _e === void 0 ? void 0 : _e.on('roleChanged', this.roleChanged.bind(this));
        this.subscribeToCaptionEvents();
        (_f = this.call) === null || _f === void 0 ? void 0 : _f.feature(communicationCalling.Features.Transfer).on('transferAccepted', this.transferAccepted.bind(this));
        (_g = this.call) === null || _g === void 0 ? void 0 : _g.feature(communicationCalling.Features.Capabilities).on('capabilitiesChanged', this.capabilitiesChanged.bind(this));
        (_h = this.call) === null || _h === void 0 ? void 0 : _h.feature(communicationCalling.Features.Spotlight).on('spotlightChanged', this.spotlightChanged.bind(this));
        try {
            (_j = this.call) === null || _j === void 0 ? void 0 : _j.feature(communicationCalling.Features.RealTimeText).on('realTimeTextReceived', this.realTimeTextReceived.bind(this));
        }
        catch (_l) {
            console.log('RealTimeText feature is not supported');
        }
        const breakoutRoomsFeature = (_k = this.call) === null || _k === void 0 ? void 0 : _k.feature(communicationCalling.Features.BreakoutRooms);
        if (breakoutRoomsFeature) {
            breakoutRoomsFeature.on('breakoutRoomsUpdated', this.breakoutRoomsUpdated.bind(this));
        }
    }
    unsubscribeCallEvents() {
        var _a, _b, _c, _d, _e, _f, _g;
        for (const subscriber of this.participantSubscribers.values()) {
            subscriber.unsubscribeAll();
        }
        this.participantSubscribers.clear();
        (_a = this.call) === null || _a === void 0 ? void 0 : _a.off('remoteParticipantsUpdated', this.onRemoteParticipantsUpdated.bind(this));
        (_b = this.call) === null || _b === void 0 ? void 0 : _b.off('isMutedChanged', this.isMyMutedChanged.bind(this));
        (_c = this.call) === null || _c === void 0 ? void 0 : _c.off('isScreenSharingOnChanged', this.isScreenSharingOnChanged.bind(this));
        (_d = this.call) === null || _d === void 0 ? void 0 : _d.off('idChanged', this.callIdChanged.bind(this));
        (_e = this.call) === null || _e === void 0 ? void 0 : _e.off('roleChanged', this.roleChanged.bind(this));
        this.unsubscribeFromCaptionEvents();
        try {
            (_f = this.call) === null || _f === void 0 ? void 0 : _f.feature(communicationCalling.Features.RealTimeText).off('realTimeTextReceived', this.realTimeTextReceived.bind(this));
        }
        catch (_h) {
            console.log('RealTimeText feature is not supported');
        }
        if (this.callingSoundSubscriber) {
            this.callingSoundSubscriber.unsubscribeAll();
        }
        const breakoutRoomsFeature = (_g = this.call) === null || _g === void 0 ? void 0 : _g.feature(communicationCalling.Features.BreakoutRooms);
        if (breakoutRoomsFeature) {
            breakoutRoomsFeature.off('breakoutRoomsUpdated', this.breakoutRoomsUpdated.bind(this));
        }
    }
    captionsKindChanged() {
        var _a;
        const captionsFeature = (_a = this.call) === null || _a === void 0 ? void 0 : _a.feature(communicationCalling.Features.Captions);
        const teamsCaptionsFeature = captionsFeature === null || captionsFeature === void 0 ? void 0 : captionsFeature.captions;
        teamsCaptionsFeature.on('CaptionsReceived', this.teamsCaptionsReceived.bind(this));
        teamsCaptionsFeature.on('CaptionsActiveChanged', this.isCaptionsActiveChanged.bind(this));
        teamsCaptionsFeature.on('CaptionLanguageChanged', this.isCaptionLanguageChanged.bind(this));
        teamsCaptionsFeature.on('SpokenLanguageChanged', this.isSpokenLanguageChanged.bind(this));
    }
    onRemoteParticipantsUpdated({ added, removed }) {
        if (added && added.length > 0) {
            this.emitter.emit('participantsJoined', {
                joined: added
            });
        }
        if (removed && removed.length > 0) {
            this.emitter.emit('participantsLeft', {
                removed: removed
            });
        }
        added.forEach(participant => {
            this.participantSubscribers.set(toFlatCommunicationIdentifier(participant.identifier), new ParticipantSubscriber(participant, this.emitter));
        });
        removed.forEach(participant => {
            const subscriber = this.participantSubscribers.get(toFlatCommunicationIdentifier(participant.identifier));
            subscriber && subscriber.unsubscribeAll();
            this.participantSubscribers.delete(toFlatCommunicationIdentifier(participant.identifier));
        });
    }
    isScreenSharingOnChanged() {
        var _a;
        this.emitter.emit('isLocalScreenSharingActiveChanged', {
            isScreenSharingOn: (_a = this.call) === null || _a === void 0 ? void 0 : _a.isScreenSharingOn
        });
    }
    teamsCaptionsReceived(captionsInfo) {
        this.emitter.emit('captionsReceived', {
            captionsInfo
        });
    }
    captionsReceived(captionsInfo) {
        this.emitter.emit('captionsReceived', {
            captionsInfo
        });
    }
    realTimeTextReceived(realTimeText) {
        this.emitter.emit('realTimeTextReceived', {
            realTimeText
        });
    }
    isCaptionsActiveChanged() {
        var _a;
        const captionsFeature = (_a = this.call) === null || _a === void 0 ? void 0 : _a.feature(communicationCalling.Features.Captions).captions;
        this.emitter.emit('isCaptionsActiveChanged', {
            isActive: captionsFeature.isCaptionsFeatureActive
        });
    }
    isSpokenLanguageChanged() {
        var _a;
        const captionsFeature = (_a = this.call) === null || _a === void 0 ? void 0 : _a.feature(communicationCalling.Features.Captions).captions;
        this.emitter.emit('isSpokenLanguageChanged', {
            activeSpokenLanguage: captionsFeature.activeSpokenLanguage
        });
    }
    isCaptionLanguageChanged() {
        var _a;
        const captionsFeature = (_a = this.call) === null || _a === void 0 ? void 0 : _a.feature(communicationCalling.Features.Captions).captions;
        this.emitter.emit('isCaptionLanguageChanged', {
            activeCaptionLanguage: captionsFeature.activeCaptionLanguage
        });
    }
    transferAccepted(args) {
        this.emitter.emit('transferAccepted', args);
    }
    capabilitiesChanged(data) {
        var _a, _b;
        if (((_a = data.newValue.turnVideoOn) === null || _a === void 0 ? void 0 : _a.isPresent) === false) {
            // stopCamera is handled by web sdk when video hard muted.
            this.disposeLocalVideoStreamView();
        }
        if (((_b = data.newValue.shareScreen) === null || _b === void 0 ? void 0 : _b.isPresent) === false) {
            this.stopScreenShare();
        }
        this.emitter.emit('capabilitiesChanged', data);
    }
    roleChanged() {
        var _a, _b;
        if (((_a = this.call) === null || _a === void 0 ? void 0 : _a.role) === 'Consumer') {
            (_b = this.call) === null || _b === void 0 ? void 0 : _b.feature(communicationCalling.Features.RaiseHand).lowerHand();
        }
        this.emitter.emit('roleChanged');
    }
    spotlightChanged(args) {
        this.emitter.emit('spotlightChanged', args);
    }
    breakoutRoomsUpdated(eventData) {
        if (eventData.type === 'assignedBreakoutRooms') {
            this.assignedBreakoutRoomUpdated(eventData.data);
        }
        else if (eventData.type === 'join') {
            this.breakoutRoomJoined(eventData.data);
        }
        this.emitter.emit('breakoutRoomsUpdated', eventData);
    }
    assignedBreakoutRoomUpdated(breakoutRoom) {
        var _a, _b, _c;
        if (!((_a = this.call) === null || _a === void 0 ? void 0 : _a.id)) {
            return;
        }
        if (((_b = this.originCall) === null || _b === void 0 ? void 0 : _b.id) !== ((_c = this.call) === null || _c === void 0 ? void 0 : _c.id) && (!breakoutRoom || breakoutRoom.state === 'closed')) {
            this.returnFromBreakoutRoom();
        }
    }
    breakoutRoomJoined(call) {
        var _a, _b;
        if (((_a = this.call) === null || _a === void 0 ? void 0 : _a.id) !== call.id) {
            // Mute the breakout room call if the main call was muted
            if ((_b = this.call) === null || _b === void 0 ? void 0 : _b.isMuted) {
                call.mute();
            }
            this.processNewCall(call);
        }
        // Hang up other breakout room calls in case we are joining a new breakout room while already in one
        this.hangupOtherBreakoutRoomCalls(call.id);
    }
    hangupOtherBreakoutRoomCalls(currentBreakoutRoomCallId) {
        // Get other breakout room calls by checking which other calls have breakout room settings defined
        const otherBreakoutRoomCallStates = Object.values(this.callClient.getState().calls).filter(callState => {
            var _a;
            return ((_a = callState.breakoutRooms) === null || _a === void 0 ? void 0 : _a.breakoutRoomSettings) && callState.id !== currentBreakoutRoomCallId;
        });
        // Hang up other breakout room calls
        for (const breakoutRoom of otherBreakoutRoomCallStates) {
            const breakoutRoomCall = this.callAgent.calls.find(callAgentCall => {
                return callAgentCall.id === breakoutRoom.id;
            });
            breakoutRoomCall === null || breakoutRoomCall === void 0 ? void 0 : breakoutRoomCall.hangUp();
        }
    }
    callIdChanged() {
        var _a;
        ((_a = this.call) === null || _a === void 0 ? void 0 : _a.id) && this.emitter.emit('callIdChanged', {
            callId: this.call.id
        });
    }
    resetDiagnosticsForwarder(newCall) {
        if (this.diagnosticsForwarder) {
            this.diagnosticsForwarder.unsubscribe();
        }
        if (newCall) {
            this.diagnosticsForwarder = new DiagnosticsForwarder(this.emitter, newCall);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    off(event, listener) {
        this.emitter.off(event, listener);
    }
    asyncTeeErrorToEventEmitter(f) {
        return __awaiter$7(this, void 0, void 0, function* () {
            try {
                return yield f();
            }
            catch (error) {
                if (isCallError(error)) {
                    this.emitter.emit('error', error);
                }
                throw error;
            }
        });
    }
    teeErrorToEventEmitter(f) {
        try {
            return f();
        }
        catch (error) {
            if (isCallError(error)) {
                this.emitter.emit('error', error);
            }
            throw error;
        }
    }
}
/**
 * @public
 */
/**
 * Create a {@link CallAdapter} backed by Azure Communication Services.
 *
 * This is the default implementation of {@link CallAdapter} provided by this library.
 *
 * Note: `displayName` can be a maximum of 256 characters.
 *
 * @public
 */
function createAzureCommunicationCallAdapter(args) {
    return __awaiter$7(this, void 0, void 0, function* () {
        if (communicationCommon.isMicrosoftTeamsUserIdentifier(args.userId)) {
            throw new Error('Microsoft Teams user identifier is not supported by AzureCommunicationCallAdapter. Instead use TeamsCallAdapter.');
        }
        return _createAzureCommunicationCallAdapterInner({
            userId: args.userId,
            displayName: args.displayName,
            credential: args.credential,
            locator: args.locator,
            targetCallees: args.targetCallees,
            options: args.options
        });
    });
}
/**
 * This inner function is used to allow injection of TelemetryImplementationHint without changing the public API.
 *
 * @internal
 */
const _createAzureCommunicationCallAdapterInner = (_a) => __awaiter$7(void 0, [_a], void 0, function* ({ userId, displayName, credential, locator, targetCallees, options, telemetryImplementationHint = 'Call' }) {
    if (!_isValidIdentifier(userId)) {
        throw new Error('Invalid identifier. Please provide valid identifier object.');
    }
    const callClient = _createStatefulCallClientInner({
        userId
    }, undefined, telemetryImplementationHint);
    const callAgent = yield callClient.createCallAgent(credential, {
        displayName
    });
    let adapter;
    if (locator) {
        adapter = createAzureCommunicationCallAdapterFromClient(callClient, callAgent, locator, options);
    }
    else {
        adapter = createAzureCommunicationCallAdapterFromClient(callClient, callAgent, targetCallees, options);
    }
    return adapter;
});
/**
 * @public
 */
const createTeamsCallAdapter = (args) => __awaiter$7(void 0, void 0, void 0, function* () {
    const { userId, credential, options } = args;
    if (communicationCommon.isCommunicationUserIdentifier(userId)) {
        throw new Error('Communication User identifier is not supported by TeamsCallAdapter, please use our AzureCommunicationCallAdapter.');
    }
    const callClient = _createStatefulCallClientInner({
        userId
    }, undefined, 'Call');
    const callAgent = yield callClient.createTeamsCallAgent(credential, {
        undefined
    });
    const adapter = createTeamsCallAdapterFromClient(callClient, callAgent, args.locator, options);
    return adapter;
});
/**
 * @private
 */
function useAzureCommunicationCallAdapterGeneric(args, adapterKind = 'AzureCommunication', afterCreate, beforeDispose) {
    const { credential, userId } = args;
    const locator = 'locator' in args ? args.locator : undefined;
    const targetCallees = 'targetCallees' in args ? args.targetCallees : undefined;
    const displayName = 'displayName' in args ? args.displayName : undefined;
    const options = 'options' in args ? args.options : undefined;
    // State update needed to rerender the parent component when a new adapter is created.
    const [adapter, setAdapter] = React.useState(undefined);
    // Ref needed for cleanup to access the old adapter created asynchronously.
    const adapterRef = React.useRef(undefined);
    const creatingAdapterRef = React.useRef(false);
    const afterCreateRef = React.useRef(undefined);
    const beforeDisposeRef = React.useRef(undefined);
    // These refs are updated on *each* render, so that the latest values
    // are used in the `useEffect` closures below.
    // Using a Ref ensures that new values for the callbacks do not trigger the
    // useEffect blocks, and a new adapter creation / distruction is not triggered.
    afterCreateRef.current = afterCreate;
    beforeDisposeRef.current = beforeDispose;
    React.useEffect(() => {
        if (!credential || !locator && !targetCallees || !userId) {
            return;
        }
        if (adapterKind === 'AzureCommunication' && !displayName) {
            return;
        }
        (() => __awaiter$7(this, void 0, void 0, function* () {
            if (adapterRef.current) {
                // Dispose the old adapter when a new one is created.
                //
                // This clean up function uses `adapterRef` because `adapter` can not be added to the dependency array of
                // this `useEffect` -- we do not want to trigger a new adapter creation because of the first adapter
                // creation.
                if (beforeDisposeRef.current) {
                    yield beforeDisposeRef.current(adapterRef.current);
                }
                adapterRef.current.dispose();
                adapterRef.current = undefined;
            }
            let newAdapter = undefined;
            if (adapterKind === 'AzureCommunication') {
                // This is just the type check to ensure that displayName is defined.
                if (!displayName) {
                    throw new Error('Unreachable code, displayName already checked above.');
                }
                if (creatingAdapterRef.current) {
                    console.warn('Adapter is already being created, please see storybook for more information: https://azure.github.io/communication-ui-library/?path=/story/troubleshooting--page');
                    return;
                }
                creatingAdapterRef.current = true;
                if (locator) {
                    newAdapter = (yield createAzureCommunicationCallAdapter({
                        credential,
                        displayName: displayName,
                        locator,
                        userId: userId,
                        options
                    }));
                }
                else if (targetCallees) {
                    newAdapter = (yield createAzureCommunicationCallAdapter({
                        credential,
                        displayName: displayName,
                        targetCallees,
                        userId: userId,
                        options
                    }));
                }
            }
            else if (adapterKind === 'Teams') {
                if (creatingAdapterRef.current) {
                    console.warn('Adapter is already being created, skipping creation.');
                    return;
                }
                creatingAdapterRef.current = true;
                if (targetCallees) ;
                else {
                    newAdapter = (yield createTeamsCallAdapter({
                        credential,
                        locator: locator,
                        userId: userId,
                        options
                    }));
                }
            }
            else {
                throw new Error('Unreachable code, unknown adapterKind');
            }
            if (!newAdapter) {
                throw Error('Unreachable code! Get undefined adapter');
            }
            if (afterCreateRef.current) {
                newAdapter = yield afterCreateRef.current(newAdapter);
            }
            adapterRef.current = newAdapter;
            creatingAdapterRef.current = false;
            setAdapter(newAdapter);
        }))();
    }, 
    // Explicitly list all arguments so that caller doesn't have to memoize the `args` object.
    [adapterRef, afterCreateRef, beforeDisposeRef, adapterKind, credential, locator, userId, displayName, options, targetCallees]);
    // Dispose any existing adapter when the component unmounts.
    React.useEffect(() => {
        return () => {
            (() => __awaiter$7(this, void 0, void 0, function* () {
                if (adapterRef.current) {
                    if (beforeDisposeRef.current) {
                        yield beforeDisposeRef.current(adapterRef.current);
                    }
                    adapterRef.current.dispose();
                    adapterRef.current = undefined;
                }
            }))();
        };
    }, []);
    return adapter;
}
/**
 * A custom React hook to simplify the creation of {@link CallAdapter}.
 *
 * Similar to {@link createAzureCommunicationCallAdapter}, but takes care of asynchronous
 * creation of the adapter internally.
 *
 * Allows arguments to be undefined so that you can respect the rule-of-hooks and pass in arguments
 * as they are created. The adapter is only created when all arguments are defined.
 *
 * Note that you must memoize the arguments to avoid recreating adapter on each render.
 * See storybook for typical usage examples.
 *
 * @public
 */
const useAzureCommunicationCallAdapter = (args, afterCreate, beforeDispose) => {
    return useAzureCommunicationCallAdapterGeneric(args, 'AzureCommunication', afterCreate, beforeDispose);
};
/**
 * A custom React hook to simplify the creation of {@link TeamsCallAdapter}.
 *
 * Similar to {@link createTeamsAzureCommunicationCallAdapter}, but takes care of asynchronous
 * creation of the adapter internally.
 *
 * Allows arguments to be undefined so that you can respect the rule-of-hooks and pass in arguments
 * as they are created. The adapter is only created when all arguments are defined.
 *
 * Note that you must memoize the arguments to avoid recreating adapter on each render.
 * See storybook for typical usage examples.
 *
 * @public
 */
const useTeamsCallAdapter = (args, afterCreate, beforeDispose) => {
    return useAzureCommunicationCallAdapterGeneric(args, 'Teams', afterCreate, beforeDispose);
};
/**
 * Create a {@link CallAdapter} using the provided {@link StatefulCallClient}.
 *
 * Useful if you want to keep a reference to {@link StatefulCallClient}.
 * Consider using {@link createAzureCommunicationCallAdapter} for a simpler API.
 *
 * @public
 */
function createAzureCommunicationCallAdapterFromClient(callClient, callAgent, locatorOrtargetCallees, options) {
    return __awaiter$7(this, void 0, void 0, function* () {
        const deviceManager = (yield callClient.getDeviceManager());
        yield Promise.all([deviceManager.getCameras(), deviceManager.getMicrophones()]);
        if (deviceManager.isSpeakerSelectionAvailable) {
            yield deviceManager.getSpeakers();
        }
        if (getLocatorOrTargetCallees(locatorOrtargetCallees)) {
            return new AzureCommunicationCallAdapter(callClient, locatorOrtargetCallees, callAgent, deviceManager, options);
        }
        else {
            return new AzureCommunicationCallAdapter(callClient, locatorOrtargetCallees, callAgent, deviceManager, options);
        }
    });
}
/**
 * Create a {@link TeamsCallAdapter} using the provided {@link StatefulCallClient}.
 *
 * Useful if you want to keep a reference to {@link StatefulCallClient}.
 * Consider using {@link createAzureCommunicationCallAdapter} for a simpler API.
 *
 * @public
 */
const createTeamsCallAdapterFromClient = (callClient, callAgent, locator, options) => __awaiter$7(void 0, void 0, void 0, function* () {
    const deviceManager = (yield callClient.getDeviceManager());
    yield Promise.all([deviceManager.getCameras(), deviceManager.getMicrophones()]);
    if (deviceManager.isSpeakerSelectionAvailable) {
        yield deviceManager.getSpeakers();
    }
    if (Array.isArray(locator)) {
        return new AzureCommunicationCallAdapter(callClient, locator, callAgent, deviceManager, options);
    }
    else {
        return new AzureCommunicationCallAdapter(callClient, locator, callAgent, deviceManager, options);
    }
});
const captionsUIVisibilityModifier = (state) => {
    var _a, _b, _c;
    // To hide captions to the UI, set isCaptionsFeatureActive to false and startCaptionsInProgress to false
    if (((_a = state.call) === null || _a === void 0 ? void 0 : _a.captionsFeature) && ((_b = state.captionsTriggers) === null || _b === void 0 ? void 0 : _b.captionsStartedInBackground) && !((_c = state.captionsTriggers) === null || _c === void 0 ? void 0 : _c.captionsStartedWithUI)) {
        return Object.assign(Object.assign({}, state), { call: Object.assign(Object.assign({}, state.call), { captionsFeature: Object.assign(Object.assign({}, state.call.captionsFeature), { isCaptionsFeatureActive: false, startCaptionsInProgress: false }) }) });
    }
    return state;
};
const isCallError = (e) => {
    return 'target' in e && 'innerError' in e;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const compositeOuterContainerStyles = {
    root: {
        width: '100%',
        // Create a new stacking context so that DrawerMenu can be positioned absolutely.
        position: 'relative'
    }
};
/** @private */
const callCompositeContainerStyles = (isMobile) => ({
    root: {
        // Start a new stacking context so that any `position:absolute` elements
        // inside the call composite do not compete with its siblings.
        position: 'relative',
        width: '100%',
        minWidth: isMobile ? 'unset' : `${compositeMinWidthRem}rem`
    }
});
/**
 * Style for stack containing the chat spinner used to delay showing chat composite until it is ready.
 * @private
 */
const chatSpinnerContainerStyles = {
    root: {
        width: '100%',
        height: '100%',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$6 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
/**
 * Facade around the CallWithChatAdapter to satisfy the call adapter interface.
 *
 * @private
 */
class CallWithChatBackedCallAdapter {
    constructor(callWithChatAdapter) {
        // For onStateChange we must convert CallWithChat state to chat state. This involves creating a new handler to be passed into the onStateChange.
        // In order to unsubscribe the handler when offStateChange is called we must have a mapping of the original handler to the newly created handler.
        this.eventStore = new Map();
        this.on = (event, listener) => {
            switch (event) {
                case 'error':
                    return this.callWithChatAdapter.on('callError', listener);
                case 'participantsJoined':
                    return this.callWithChatAdapter.on('callParticipantsJoined', listener);
                case 'participantsLeft':
                    return this.callWithChatAdapter.on('callParticipantsLeft', listener);
                default:
                    return this.callWithChatAdapter.on(event, listener);
            }
        };
        this.off = (event, listener) => {
            switch (event) {
                case 'error':
                    return this.callWithChatAdapter.off('callError', listener);
                case 'participantsJoined':
                    return this.callWithChatAdapter.off('callParticipantsJoined', listener);
                case 'participantsLeft':
                    return this.callWithChatAdapter.off('callParticipantsLeft', listener);
                default:
                    return this.callWithChatAdapter.off(event, listener);
            }
        };
        this.onStateChange = (handler) => {
            const convertedHandler = (state) => {
                handler(callAdapterStateFromCallWithChatAdapterState(state));
            };
            this.callWithChatAdapter.onStateChange(convertedHandler);
            this.eventStore.set(handler, convertedHandler);
        };
        this.offStateChange = (handler) => {
            const convertedHandler = this.eventStore.get(handler);
            convertedHandler && this.callWithChatAdapter.offStateChange(convertedHandler);
        };
        this.getState = () => callAdapterStateFromCallWithChatAdapterState(this.callWithChatAdapter.getState());
        this.dispose = () => this.callWithChatAdapter.dispose();
        this.joinCall = (options) => {
            if (typeof options === 'boolean') {
                return this.callWithChatAdapter.joinCall(options);
            }
            else {
                return this.callWithChatAdapter.joinCall(options);
            }
        };
        this.leaveCall = (forEveryone) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.leaveCall(forEveryone); });
        this.startCall = (participants, options) => {
            if (participants.every((participant) => typeof participant === 'string')) {
                return this.callWithChatAdapter.startCall(participants, options);
            }
            else {
                return this.callWithChatAdapter.startCall(participants, options);
            }
        };
        this.setCamera = (sourceId, options) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.setCamera(sourceId, options); });
        this.setMicrophone = (sourceId) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.setMicrophone(sourceId); });
        this.setSpeaker = (sourceId) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.setSpeaker(sourceId); });
        this.askDevicePermission = (constraints) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.askDevicePermission(constraints); });
        this.queryCameras = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.queryCameras(); });
        this.queryMicrophones = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.queryMicrophones(); });
        this.querySpeakers = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.querySpeakers(); });
        this.startCamera = (options) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.startCamera(options); });
        this.stopCamera = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.stopCamera(); });
        this.mute = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.mute(); });
        this.unmute = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.unmute(); });
        this.startScreenShare = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.startScreenShare(); });
        this.stopScreenShare = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.stopScreenShare(); });
        this.raiseHand = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.raiseHand(); });
        this.lowerHand = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.lowerHand(); });
        this.onReactionClick = (reaction) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.onReactionClick(reaction); });
        this.removeParticipant = (userId) => __awaiter$6(this, void 0, void 0, function* () {
            let participant = userId;
            participant = _toCommunicationIdentifier(userId);
            yield this.callWithChatAdapter.removeParticipant(participant);
        });
        this.createStreamView = (remoteUserId, options) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.createStreamView(remoteUserId, options); });
        this.createTogetherModeStreamView = (options) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.createTogetherModeStreamView(options); });
        this.startTogetherMode = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.startTogetherMode(); });
        this.setTogetherModeSceneSize = (width, height) => this.callWithChatAdapter.setTogetherModeSceneSize(width, height);
        this.disposeStreamView = (remoteUserId, options) => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.disposeStreamView(remoteUserId, options); });
        this.disposeTogetherModeStreamView = () => __awaiter$6(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.disposeTogetherModeStreamView(); });
        this.holdCall = () => __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.holdCall();
        });
        this.resumeCall = () => __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.resumeCall();
        });
        this.sendDtmfTone = (dtmfTone) => __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.sendDtmfTone(dtmfTone);
        });
        this.callWithChatAdapter = callWithChatAdapter;
    }
    disposeScreenShareStreamView(remoteUserId) {
        return this.callWithChatAdapter.disposeScreenShareStreamView(remoteUserId);
    }
    disposeRemoteVideoStreamView(remoteUserId) {
        return this.callWithChatAdapter.disposeRemoteVideoStreamView(remoteUserId);
    }
    disposeLocalVideoStreamView() {
        return this.callWithChatAdapter.disposeLocalVideoStreamView();
    }
    addParticipant(participant, options) {
        return __awaiter$6(this, void 0, void 0, function* () {
            if (communicationCommon.isPhoneNumberIdentifier(participant) && options) {
                return this.callWithChatAdapter.addParticipant(participant, options);
            }
            else {
                return this.callWithChatAdapter.addParticipant(participant);
            }
        });
    }
    startCaptions(options) {
        return __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.startCaptions(options);
        });
    }
    stopCaptions(options) {
        return __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.stopCaptions(options);
        });
    }
    setCaptionLanguage(language) {
        return __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.setCaptionLanguage(language);
        });
    }
    setSpokenLanguage(language) {
        return __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.setSpokenLanguage(language);
        });
    }
    sendRealTimeText(text, isFinalized) {
        return __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.sendRealTimeText(text, isFinalized);
        });
    }
    startVideoBackgroundEffect(videoBackgroundEffect) {
        return __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.startVideoBackgroundEffect(videoBackgroundEffect);
        });
    }
    stopVideoBackgroundEffects() {
        return __awaiter$6(this, void 0, void 0, function* () {
            yield this.callWithChatAdapter.stopVideoBackgroundEffects();
        });
    }
    updateBackgroundPickerImages(backgroundImages) {
        return this.callWithChatAdapter.updateBackgroundPickerImages(backgroundImages);
    }
    updateSelectedVideoBackgroundEffect(selectedVideoBackground) {
        return this.callWithChatAdapter.updateSelectedVideoBackgroundEffect(selectedVideoBackground);
    }
    startNoiseSuppressionEffect() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.startNoiseSuppressionEffect();
        });
    }
    stopNoiseSuppressionEffect() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.stopNoiseSuppressionEffect();
        });
    }
    submitSurvey(survey) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.submitSurvey(survey);
        });
    }
    startSpotlight(userIds) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.startSpotlight(userIds);
        });
    }
    stopSpotlight(userIds) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.stopSpotlight(userIds);
        });
    }
    stopAllSpotlight() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.stopAllSpotlight();
        });
    }
    muteParticipant(userId) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.muteParticipant(userId);
        });
    }
    muteAllRemoteParticipants() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.muteAllRemoteParticipants();
        });
    }
    returnFromBreakoutRoom() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.returnFromBreakoutRoom();
        });
    }
    forbidAudio(userIds) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.forbidAudio(userIds);
        });
    }
    permitAudio(userIds) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.permitAudio(userIds);
        });
    }
    forbidOthersAudio() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.forbidOthersAudio();
        });
    }
    permitOthersAudio() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.permitOthersAudio();
        });
    }
    forbidVideo(userIds) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.forbidAudio(userIds);
        });
    }
    permitVideo(userIds) {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.permitAudio(userIds);
        });
    }
    forbidOthersVideo() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.forbidOthersAudio();
        });
    }
    permitOthersVideo() {
        return __awaiter$6(this, void 0, void 0, function* () {
            return this.callWithChatAdapter.permitOthersAudio();
        });
    }
}
function callAdapterStateFromCallWithChatAdapterState(callWithChatAdapterState) {
    return {
        isLocalPreviewMicrophoneEnabled: callWithChatAdapterState.isLocalPreviewMicrophoneEnabled,
        page: callWithChatAdapterState.page,
        userId: callWithChatAdapterState.userId,
        displayName: callWithChatAdapterState.displayName,
        call: callWithChatAdapterState.call,
        devices: callWithChatAdapterState.devices,
        isTeamsCall: callWithChatAdapterState.isTeamsCall,
        isTeamsMeeting: callWithChatAdapterState.isTeamsMeeting,
        isRoomsCall: false,
        latestErrors: callWithChatAdapterState.latestCallErrors,
        latestNotifications: callWithChatAdapterState.latestCallNotifications,
        alternateCallerId: callWithChatAdapterState.alternateCallerId,
        environmentInfo: callWithChatAdapterState.environmentInfo,
        videoBackgroundImages: callWithChatAdapterState.videoBackgroundImages,
        onResolveVideoEffectDependency: callWithChatAdapterState.onResolveVideoEffectDependency,
        onResolveDeepNoiseSuppressionDependency: callWithChatAdapterState.onResolveDeepNoiseSuppressionDependency,
        deepNoiseSuppressionOnByDefault: callWithChatAdapterState.deepNoiseSuppressionOnByDefault,
        hideDeepNoiseSuppressionButton: callWithChatAdapterState.hideDeepNoiseSuppressionButton,
        selectedVideoBackgroundEffect: callWithChatAdapterState.selectedVideoBackgroundEffect,
        reactions: callWithChatAdapterState.reactions
    };
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$5 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/no-unused-vars */
/**
 * Facade around the CallWithChatAdapter to satisfy the chat adapter interface.
 *
 * @private
 */
class CallWithChatBackedChatAdapter {
    constructor(callWithChatAdapter) {
        // For onStateChange we must convert CallWithChat state to chat state. This involves creating a new handler to be passed into the onStateChange.
        // In order to unsubscribe the handler when offStateChange is called we must have a mapping of the original handler to the newly created handler.
        this.eventStore = new Map();
        this.fetchInitialData = () => __awaiter$5(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.fetchInitialData(); });
        // due to a bug in babel, we can't use arrow function here
        // affecting conditional-compile-remove(attachment-upload)
        // have to bind this since the scope of 'this' is lost when the function is passed as a callback
        this.sendMessageHandler = function (content) {
            return __awaiter$5(this, void 0, void 0, function* () {
                yield this.callWithChatAdapter.sendMessage(content);
            });
        };
        this.sendMessage = this.sendMessageHandler.bind(this);
        this.sendReadReceipt = (chatMessageId) => __awaiter$5(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.sendReadReceipt(chatMessageId); });
        this.sendTypingIndicator = () => __awaiter$5(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.sendTypingIndicator(); });
        this.removeParticipant = (userId) => __awaiter$5(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.removeParticipant(userId); });
        this.loadPreviousChatMessages = (messagesToLoad) => __awaiter$5(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.loadPreviousChatMessages(messagesToLoad); });
        this.dispose = () => this.callWithChatAdapter.dispose();
        this.onStateChange = (handler) => {
            const convertedHandler = (state) => {
                !!state.chat && handler(chatAdapterStateFromCallWithChatAdapterState(state));
            };
            this.callWithChatAdapter.onStateChange(convertedHandler);
            this.eventStore.set(handler, convertedHandler);
        };
        this.offStateChange = (handler) => {
            const convertedHandler = this.eventStore.get(handler);
            convertedHandler && this.callWithChatAdapter.offStateChange(convertedHandler);
        };
        this.getState = () => chatAdapterStateFromCallWithChatAdapterState(this.callWithChatAdapter.getState());
        /* eslint-disable @typescript-eslint/explicit-module-boundary-types */
        this.on = (event, listener) => {
            switch (event) {
                case 'error':
                    return this.callWithChatAdapter.on('chatError', listener);
                case 'participantsAdded':
                    return this.callWithChatAdapter.on('chatParticipantsAdded', listener);
                case 'participantsRemoved':
                    return this.callWithChatAdapter.on('chatParticipantsRemoved', listener);
                default:
                    return this.callWithChatAdapter.on(event, listener);
            }
        };
        this.off = (event, listener) => {
            switch (event) {
                case 'error':
                    return this.callWithChatAdapter.off('chatError', listener);
                case 'participantsAdded':
                    return this.callWithChatAdapter.off('chatParticipantsAdded', listener);
                case 'participantsRemoved':
                    return this.callWithChatAdapter.off('chatParticipantsRemoved', listener);
                default:
                    return this.callWithChatAdapter.off(event, listener);
            }
        };
        // due to a bug in babel, we can't use arrow function here
        // affecting conditional-compile-remove(attachment-upload)
        // have to bind this since the scope of 'this' is lost when the function is passed as a callback
        this.updateMessageHandler = function (messageId, content, options) {
            return __awaiter$5(this, void 0, void 0, function* () {
                yield this.callWithChatAdapter.updateMessage(messageId, content, options);
            });
        };
        this.updateMessage = this.updateMessageHandler.bind(this);
        this.deleteMessage = (messageId) => __awaiter$5(this, void 0, void 0, function* () { return yield this.callWithChatAdapter.deleteMessage(messageId); });
        this.clearErrors = (errorTypes) => {
            throw new Error(`Method not supported in CallWithChatComposite.`);
        };
        this.setTopic = (topicName) => __awaiter$5(this, void 0, void 0, function* () {
            throw new Error(`Chat Topics are not supported in CallWithChatComposite.`);
        });
        this.callWithChatAdapter = callWithChatAdapter;
    }
    downloadResourceToCache(resourceDetails) {
        return __awaiter$5(this, void 0, void 0, function* () {
            this.callWithChatAdapter.downloadResourceToCache(resourceDetails);
        });
    }
    removeResourceFromCache(resourceDetails) {
        this.callWithChatAdapter.removeResourceFromCache(resourceDetails);
    }
}
function chatAdapterStateFromCallWithChatAdapterState(callWithChatAdapterState) {
    if (!callWithChatAdapterState.chat) {
        // Return some empty state if chat is not initialized yet
        return {
            userId: callWithChatAdapterState.userId,
            displayName: callWithChatAdapterState.displayName || '',
            thread: {
                chatMessages: {},
                participants: {
                    [toFlatCommunicationIdentifier(callWithChatAdapterState.userId)]: {
                        id: callWithChatAdapterState.userId
                    }
                },
                threadId: '',
                readReceipts: [],
                typingIndicators: [],
                latestReadTime: new Date()
            },
            latestErrors: callWithChatAdapterState.latestChatErrors
        };
    }
    return {
        userId: callWithChatAdapterState.userId,
        displayName: callWithChatAdapterState.displayName || '',
        thread: callWithChatAdapterState.chat,
        latestErrors: callWithChatAdapterState.latestChatErrors
    };
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const ChatButton = (props) => {
    const strings = Object.assign({ label: props.label }, props.strings);
    const theme = useTheme();
    const styles = React.useMemo(() => {
        var _a;
        return react.concatStyleSets({
            rootChecked: {
                background: theme.palette.neutralLight
            }
        }, (_a = props.styles) !== null && _a !== void 0 ? _a : {});
    }, [props.styles, theme.palette.neutralLight]);
    return React.createElement(ControlBarButton, Object.assign({}, props, { labelKey: 'chatButtonLabelKey', strings: strings, onClick: props.onClick, styles: styles }));
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
const NotificationIcon = (props) => {
    const { chatMessagesCount, label } = props;
    const theme = react.useTheme();
    const renderNumber = (numberOfMessages) => {
        if (numberOfMessages < 1) {
            return React.createElement(React.Fragment, null);
        }
        else {
            const textNumberOfMessages = numberOfMessages < 9 ? numberOfMessages : '9+';
            return React.createElement(react.Text, { role: 'status', "aria-label": textNumberOfMessages + (label !== null && label !== void 0 ? label : ''), styles: notificationTextStyles(theme) }, textNumberOfMessages);
        }
    };
    return React.createElement(react.Stack, { "data-ui-id": "call-with-chat-composite-chat-button-unread-icon", horizontalAlign: "center", verticalAlign: "center", styles: notificationIconContainerStyles(theme) },
        React.createElement(react.Stack, null, renderNumber(chatMessagesCount)));
};
const notificationIconPaddingREM = 0.225;
const notificationSizeREM = 1;
const notificationIconContainerStyles = react.memoizeFunction((theme) => ({
    root: {
        borderRadius: `${notificationSizeREM}rem`,
        // Create a css circle. This should match the height.
        height: `${notificationSizeREM}rem`,
        minWidth: `${notificationSizeREM}rem`,
        // use min-width over width as we want to extend the width of the notification icon when contents is more than one character (e.g. 9+)
        background: theme.palette.themePrimary,
        border: `0.0625rem solid white`,
        // border should always be white
        padding: `${notificationIconPaddingREM}rem`,
        // positioning to place the badge within the button appropriately.
        position: 'absolute',
        top: `-${0.5 - notificationIconPaddingREM / 2}rem`,
        left: `${0.5 + notificationIconPaddingREM / 2}rem`
    }
}));
const notificationTextStyles = react.memoizeFunction((theme) => ({
    root: {
        color: 'white',
        fontSize: theme.fonts.xSmall.fontSize
    }
}));

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const filledIcon = React.createElement(CallWithChatCompositeIcon, { iconName: 'ControlBarChatButtonActive' });
const regularIcon = React.createElement(CallWithChatCompositeIcon, { iconName: 'ControlBarChatButtonInactive' });
/**
 * @private
 */
const ChatButtonWithUnreadMessagesBadge = (props) => {
    var _a, _b, _c, _d;
    const { newMessageLabel, unreadChatMessagesCount, hideUnreadChatMessagesBadge } = props;
    const baseIcon = props.showLabel ? regularIcon : filledIcon;
    const callWithChatStrings = useCallWithChatCompositeStrings();
    const numberOfMsgToolTip = ((_a = props.strings) === null || _a === void 0 ? void 0 : _a.tooltipOffContent) && unreadChatMessagesCount > 0 ? _formatString(callWithChatStrings.chatButtonTooltipClosedWithMessageCount, {
        unreadMessagesCount: `${unreadChatMessagesCount}`
    }) : undefined;
    const chatStrings = React.useMemo(() => {
        var _a, _b, _c;
        return ({
            label: (_a = props.strings) === null || _a === void 0 ? void 0 : _a.label,
            tooltipOffContent: numberOfMsgToolTip ? numberOfMsgToolTip : (_b = props.strings) === null || _b === void 0 ? void 0 : _b.tooltipOffContent,
            tooltipOnContent: (_c = props.strings) === null || _c === void 0 ? void 0 : _c.tooltipOnContent
        });
    }, [numberOfMsgToolTip, (_b = props.strings) === null || _b === void 0 ? void 0 : _b.label, (_c = props.strings) === null || _c === void 0 ? void 0 : _c.tooltipOffContent, (_d = props.strings) === null || _d === void 0 ? void 0 : _d.tooltipOnContent]);
    const onRenderOnIcon = React.useCallback(() => baseIcon, [baseIcon]);
    const notificationOnIcon = React.useCallback(() => {
        return React.createElement(react.Stack, { styles: chatNotificationContainerStyles },
            unreadChatMessagesCount > 0 && !hideUnreadChatMessagesBadge && React.createElement(NotificationIcon, { chatMessagesCount: unreadChatMessagesCount, label: newMessageLabel }),
            baseIcon);
    }, [unreadChatMessagesCount, newMessageLabel, baseIcon, hideUnreadChatMessagesBadge]);
    return React.createElement(ChatButton, Object.assign({}, props, { "data-ui-id": "call-with-chat-composite-chat-button", onRenderOffIcon: notificationOnIcon, onRenderOnIcon: onRenderOnIcon, strings: chatStrings }));
};
const chatNotificationContainerStyles = {
    root: {
        display: 'inline',
        position: 'relative'
    }
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Used by the CallWithChatComposite to track unread messages for showing as a badge on the Chat Button.
 * @private
 */
const useUnreadMessagesTracker = (chatAdapter, isChatPaneVisible, isChatInitiazed) => {
    // Store messageIds of unread messages
    const [unreadChatMessages, setUnreadChatMessages] = React.useState(new Set());
    React.useEffect(() => {
        // Clear unread messages when chat pane is opened
        if (isChatPaneVisible || !isChatInitiazed) {
            setUnreadChatMessages(new Set());
            return;
        }
        // Increment unread messages when a new message is received and the chat pane is closed
        const incrementUnreadChatMessagesCount = (event) => {
            if (!isChatPaneVisible && validNewChatMessage(event.message)) {
                setUnreadChatMessages(prevUnreadChatMessages => {
                    const newUnreadChatMessages = new Set(prevUnreadChatMessages);
                    newUnreadChatMessages.add(event.message.id);
                    return newUnreadChatMessages;
                });
            }
        };
        // Decrement unread messages when a message is deleted and the chat pane is closed
        const decrementUnreadChatMessagesCount = (event) => {
            if (!isChatPaneVisible) {
                setUnreadChatMessages(prevUnreadChatMessages => {
                    const newUnreadChatMessages = new Set(prevUnreadChatMessages);
                    newUnreadChatMessages.delete(event.message.id);
                    return newUnreadChatMessages;
                });
            }
        };
        chatAdapter.on('messageReceived', incrementUnreadChatMessagesCount);
        chatAdapter.on('messageDeleted', decrementUnreadChatMessagesCount);
        return () => {
            chatAdapter.off('messageReceived', incrementUnreadChatMessagesCount);
            chatAdapter.off('messageDeleted', decrementUnreadChatMessagesCount);
        };
    }, [chatAdapter, setUnreadChatMessages, isChatPaneVisible, isChatInitiazed]);
    return unreadChatMessages.size;
};
/**
 * Helper function to determine if the message in the event is a valid one from a user.
 * Display name is used since system messages will not have one.
 */
const validNewChatMessage = (message) => !!message.senderDisplayName && (message.type === 'text' || message.type === 'html');

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const CallWithChatScreen = (props) => {
    var _a, _b;
    const { callWithChatAdapter, fluentTheme, formFactor = 'desktop' } = props;
    const { surveyOptions } = props;
    const mobileView = formFactor === 'mobile';
    if (!callWithChatAdapter) {
        throw new Error('CallWithChatAdapter is undefined');
    }
    const callAdapter = React.useMemo(() => new CallWithChatBackedCallAdapter(callWithChatAdapter), [callWithChatAdapter]);
    const [currentCallState, setCurrentCallState] = React.useState();
    const [isChatInitialized, setIsChatInitialized] = React.useState(false);
    const [currentPage, setCurrentPage] = React.useState();
    const [isChatOpen, setIsChatOpen] = React.useState(false);
    const containerRef = React.useRef(null);
    React.useEffect(() => {
        const updateCallWithChatPage = (newState) => {
            var _a;
            setCurrentPage(newState.page);
            setCurrentCallState((_a = newState.call) === null || _a === void 0 ? void 0 : _a.state);
            setIsChatInitialized(newState.chat ? true : false);
        };
        updateCallWithChatPage(callWithChatAdapter.getState());
        callWithChatAdapter.onStateChange(updateCallWithChatPage);
        return () => {
            callWithChatAdapter.offStateChange(updateCallWithChatPage);
        };
    }, [callWithChatAdapter]);
    const chatAdapter = React.useMemo(() => {
        return new CallWithChatBackedChatAdapter(callWithChatAdapter);
    }, [callWithChatAdapter]);
    /** Constant setting of id for the parent stack of the composite */
    const compositeParentDivId = reactHooks.useId('callWithChatCompositeParentDiv-internal');
    const closeChat = React.useCallback(() => {
        setIsChatOpen(false);
    }, []);
    const openChat = React.useCallback(() => {
        setIsChatOpen(true);
        // timeout is required to give the window time to render the sendbox so we have something to send focus to.
        // TODO: Selecting elements in the DOM via attributes is not stable. We should expose an API from ChatComposite to be able to focus on the sendbox.
        const chatFocusTimeout = setInterval(() => {
            const callWithChatCompositeRootDiv = document.querySelector(`[id="${compositeParentDivId}"]`);
            const sendbox = callWithChatCompositeRootDiv === null || callWithChatCompositeRootDiv === void 0 ? void 0 : callWithChatCompositeRootDiv.querySelector(`[id="sendbox"]`);
            if (sendbox !== null) {
                sendbox.focus();
                clearInterval(chatFocusTimeout);
            }
        }, 3);
        setTimeout(() => {
            clearInterval(chatFocusTimeout);
        }, 300);
    }, [compositeParentDivId]);
    const isOnHold = currentPage === 'hold';
    React.useEffect(() => {
        if (isOnHold) {
            closeChat();
        }
    }, [closeChat, isOnHold]);
    const hasJoinedCall = !!(currentPage && hasJoinedCallFn(currentPage, currentCallState !== null && currentCallState !== void 0 ? currentCallState : 'None'));
    const toggleChat = React.useCallback(() => {
        isChatOpen || !hasJoinedCall ? closeChat() : openChat();
    }, [closeChat, hasJoinedCall, isChatOpen, openChat]);
    const callWithChatStrings = useCallWithChatCompositeStrings();
    const chatButtonStrings = React.useMemo(() => ({
        label: callWithChatStrings.chatButtonLabel,
        tooltipOffContent: callWithChatStrings.chatButtonTooltipOpen,
        tooltipOnContent: callWithChatStrings.chatButtonTooltipClose
    }), [callWithChatStrings]);
    const theme = useTheme();
    const commonButtonStyles = React.useMemo(() => !mobileView ? getDesktopCommonButtonStyles(theme) : undefined, [mobileView, theme]);
    const showChatButton = checkShowChatButton(props.callControls);
    const chatButtonDisabled = showChatButton && (checkChatButtonIsDisabled(props.callControls) || !hasJoinedCall || isOnHold);
    const chatTabHeaderProps = React.useMemo(() => mobileView && showChatButton ? {
        onClick: toggleChat,
        disabled: chatButtonDisabled
    } : undefined, [chatButtonDisabled, mobileView, toggleChat, showChatButton]);
    const unreadChatMessagesCount = useUnreadMessagesTracker(chatAdapter, isChatOpen, isChatInitialized);
    const customChatButton = React.useCallback((args) => ({
        placement: mobileView ? 'primary' : 'secondary',
        onRenderButton: () => React.createElement(ChatButtonWithUnreadMessagesBadge, { checked: isChatOpen, showLabel: args.displayType !== 'compact', onClick: toggleChat, disabled: chatButtonDisabled, strings: chatButtonStrings, styles: commonButtonStyles, newMessageLabel: callWithChatStrings.chatButtonNewMessageNotificationLabel, unreadChatMessagesCount: unreadChatMessagesCount, 
            // As chat is disabled when on hold, we don't want to show the unread badge when on hold
            hideUnreadChatMessagesBadge: isOnHold, disableTooltip: mobileView })
    }), [callWithChatStrings.chatButtonNewMessageNotificationLabel, chatButtonStrings, commonButtonStyles, isChatOpen, chatButtonDisabled, mobileView, toggleChat, unreadChatMessagesCount, isOnHold]);
    const callControlOptionsFromProps = React.useMemo(() => (Object.assign({}, (typeof props.callControls === 'object' ? props.callControls : {}))), [props.callControls]);
    const injectedCustomButtonsFromProps = React.useMemo(() => {
        var _a;
        return [...((_a = callControlOptionsFromProps.onFetchCustomButtonProps) !== null && _a !== void 0 ? _a : [])];
    }, [callControlOptionsFromProps]);
    const callCompositeOptions = React.useMemo(() => ({
        callControls: props.callControls === false ? false : Object.assign(Object.assign({}, callControlOptionsFromProps), { onFetchCustomButtonProps: [...(showChatButton ? [customChatButton] : []), ...injectedCustomButtonsFromProps], legacyControlBarExperience: false }),
        remoteVideoTileMenuOptions: props.remoteVideoTileMenuOptions,
        galleryOptions: props.galleryOptions,
        localVideoTile: props.localVideoTile,
        surveyOptions: surveyOptions,
        branding: {
            logo: props.logo,
            backgroundImage: props.backgroundImage
        },
        spotlight: props.spotlight,
        joinCallOptions: props.joinCallOptions
    }), [props.callControls, callControlOptionsFromProps, showChatButton, customChatButton, injectedCustomButtonsFromProps, props.galleryOptions, props.localVideoTile, props.remoteVideoTileMenuOptions, surveyOptions, props.logo, props.backgroundImage, props.spotlight, props.joinCallOptions]);
    const chatCompositeOptions = React.useMemo(() => ({
        topic: false
    }), []);
    const chatSpinnerLabel = useLocale().strings.callWithChat.chatContentSpinnerLabel;
    const onRenderChatContent = React.useCallback(() => {
        if (!isChatInitialized) {
            return React.createElement(react.Stack, { styles: chatSpinnerContainerStyles },
                React.createElement(react.Spinner, { label: chatSpinnerLabel, size: react.SpinnerSize.large }));
        }
        return React.createElement(ChatComposite, { adapter: chatAdapter, fluentTheme: theme, options: chatCompositeOptions, onFetchAvatarPersonaData: props.onFetchAvatarPersonaData });
    }, [chatAdapter, props.onFetchAvatarPersonaData, chatCompositeOptions, theme, isChatInitialized, chatSpinnerLabel]);
    let chatPaneTitle = callWithChatStrings.chatPaneTitle;
    // If breakout room settings are defined then we know we are in a breakout room so we should
    // use the breakout room chat pane title.
    if ((_b = (_a = callAdapter.getState().call) === null || _a === void 0 ? void 0 : _a.breakoutRooms) === null || _b === void 0 ? void 0 : _b.breakoutRoomSettings) {
        chatPaneTitle = callWithChatStrings.breakoutRoomChatPaneTitle;
    }
    const sidePaneHeaderRenderer = React.useCallback(() => { var _a; return React.createElement(SidePaneHeader, { headingText: chatPaneTitle, onClose: closeChat, dismissSidePaneButtonAriaLabel: (_a = callWithChatStrings.dismissSidePaneButtonLabel) !== null && _a !== void 0 ? _a : '', mobileView: mobileView, chatButtonPresent: showChatButton }); }, [chatPaneTitle, closeChat, callWithChatStrings.dismissSidePaneButtonLabel, mobileView, showChatButton]);
    const sidePaneContentRenderer = React.useMemo(() => hasJoinedCall ? onRenderChatContent : undefined, [hasJoinedCall, onRenderChatContent]);
    const sidePaneRenderer = React.useMemo(() => ({
        contentRenderer: sidePaneContentRenderer,
        headerRenderer: sidePaneHeaderRenderer,
        id: 'chat'
    }), [sidePaneContentRenderer, sidePaneHeaderRenderer]);
    const overrideSidePaneProps = React.useMemo(() => ({
        renderer: sidePaneRenderer,
        isActive: isChatOpen,
        persistRenderingWhenClosed: true
    }), [isChatOpen, sidePaneRenderer]);
    const onSidePaneIdChange = React.useCallback((sidePaneId) => {
        // If the pane is switched to something other than chat, removing rendering chat.
        if (sidePaneId && sidePaneId !== 'chat') {
            closeChat();
        }
    }, [closeChat]);
    // When the call ends ensure the side pane is set to closed to prevent the side pane being open if the call is re-joined.
    React.useEffect(() => {
        callAdapter.on('callEnded', closeChat);
        return () => {
            callAdapter.off('callEnded', closeChat);
        };
    }, [callAdapter, closeChat]);
    return React.createElement("div", { ref: containerRef, className: react.mergeStyles(containerDivStyles) },
        React.createElement(react.Stack, { verticalFill: true, grow: true, styles: compositeOuterContainerStyles, id: compositeParentDivId },
            React.createElement(react.Stack, { horizontal: true, grow: true },
                React.createElement(react.Stack.Item, { grow: true, styles: callCompositeContainerStyles(mobileView) },
                    React.createElement(CallCompositeInner, Object.assign({}, props, { formFactor: formFactor, options: callCompositeOptions, adapter: callAdapter, fluentTheme: fluentTheme, callInvitationUrl: props.joinInvitationURL, overrideSidePane: overrideSidePaneProps, onSidePaneIdChange: onSidePaneIdChange, mobileChatTabHeader: chatTabHeaderProps, onCloseChatPane: closeChat }))))));
};
/**
 * CallWithChatComposite brings together key components to provide a full call with chat experience out of the box.
 *
 * @public
 */
const CallWithChatComposite = (props) => {
    var _a, _b;
    const { adapter, fluentTheme, rtl, formFactor, joinInvitationURL, options } = props;
    return React.createElement(BaseProvider, { fluentTheme: fluentTheme, rtl: rtl, locale: props.locale, icons: props.icons, formFactor: props.formFactor },
        React.createElement(CallWithChatScreen, Object.assign({}, props, { callWithChatAdapter: adapter, formFactor: formFactor, callControls: options === null || options === void 0 ? void 0 : options.callControls, joinInvitationURL: joinInvitationURL, fluentTheme: fluentTheme, remoteVideoTileMenuOptions: options === null || options === void 0 ? void 0 : options.remoteVideoTileMenuOptions, localVideoTile: options === null || options === void 0 ? void 0 : options.localVideoTile, galleryOptions: options === null || options === void 0 ? void 0 : options.galleryOptions, logo: (_a = options === null || options === void 0 ? void 0 : options.branding) === null || _a === void 0 ? void 0 : _a.logo, backgroundImage: (_b = options === null || options === void 0 ? void 0 : options.branding) === null || _b === void 0 ? void 0 : _b.backgroundImage, surveyOptions: options === null || options === void 0 ? void 0 : options.surveyOptions, spotlight: options === null || options === void 0 ? void 0 : options.spotlight, joinCallOptions: options === null || options === void 0 ? void 0 : options.joinCallOptions })));
};
const hasJoinedCallFn = (page, callStatus) => {
    return page === 'call' && (callStatus === 'Connected' || callStatus === 'RemoteHold' || callStatus === 'Disconnecting') || page === 'hold' && (callStatus === 'LocalHold' || callStatus === 'Disconnecting');
};
const checkShowChatButton = (callControls) => {
    if (callControls === undefined || callControls === true) {
        return true;
    }
    if (callControls === false) {
        return false;
    }
    return callControls.chatButton !== false;
};
const checkChatButtonIsDisabled = (callControls) => {
    return typeof callControls === 'object' && isDisabled$4(callControls === null || callControls === void 0 ? void 0 : callControls.chatButton);
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * @private
 */
function callWithChatAdapterStateFromBackingStates(callAdapter) {
    const callAdapterState = callAdapter.getState();
    return {
        call: callAdapterState.call,
        chat: undefined,
        userId: callAdapterState.userId,
        page: callAdapterState.page,
        displayName: callAdapterState.displayName,
        devices: callAdapterState.devices,
        isLocalPreviewMicrophoneEnabled: callAdapterState.isLocalPreviewMicrophoneEnabled,
        isTeamsCall: callAdapterState.isTeamsCall,
        isTeamsMeeting: callAdapterState.isTeamsMeeting,
        latestCallErrors: callAdapterState.latestErrors,
        latestCallNotifications: callAdapterState.latestNotifications,
        latestChatErrors: {},
        alternateCallerId: callAdapterState.alternateCallerId,
        environmentInfo: callAdapterState.environmentInfo,
        videoBackgroundImages: callAdapterState.videoBackgroundImages,
        onResolveVideoEffectDependency: callAdapterState.onResolveVideoEffectDependency,
        onResolveDeepNoiseSuppressionDependency: callAdapterState.onResolveDeepNoiseSuppressionDependency,
        selectedVideoBackgroundEffect: callAdapterState.selectedVideoBackgroundEffect,
        /** Hide attendee names in teams meeting */
        hideAttendeeNames: callAdapterState.hideAttendeeNames,
        reactions: callAdapterState.reactions
    };
}
/**
 * @private
 */
function mergeChatAdapterStateIntoCallWithChatAdapterState(existingCallWithChatAdapterState, chatAdapterState) {
    return Object.assign(Object.assign({}, existingCallWithChatAdapterState), { chat: chatAdapterState.thread, latestChatErrors: chatAdapterState.latestErrors });
}
/**
 * @private
 */
function mergeCallAdapterStateIntoCallWithChatAdapterState(existingCallWithChatAdapterState, callAdapterState) {
    return Object.assign(Object.assign({}, existingCallWithChatAdapterState), { userId: callAdapterState.userId, page: callAdapterState.page, displayName: callAdapterState.displayName, devices: callAdapterState.devices, call: callAdapterState.call, isLocalPreviewMicrophoneEnabled: callAdapterState.isLocalPreviewMicrophoneEnabled, isTeamsCall: callAdapterState.isTeamsCall, isTeamsMeeting: callAdapterState.isTeamsMeeting, latestCallErrors: callAdapterState.latestErrors, latestCallNotifications: callAdapterState.latestNotifications, videoBackgroundImages: callAdapterState.videoBackgroundImages, onResolveVideoEffectDependency: callAdapterState.onResolveVideoEffectDependency, onResolveDeepNoiseSuppressionDependency: callAdapterState.onResolveDeepNoiseSuppressionDependency, deepNoiseSuppressionOnByDefault: callAdapterState.deepNoiseSuppressionOnByDefault, hideDeepNoiseSuppressionButton: callAdapterState.hideDeepNoiseSuppressionButton, selectedVideoBackgroundEffect: callAdapterState.selectedVideoBackgroundEffect });
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$4 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * For each time that we use the hook {@link useSelector} in the {@link CallWithChatComposite} we add another listener
 * to the `stateChanged` event on the this adapter. This number is set in relation to the number of
 * times that we are using the hook useSelector in the CallComposite.
 *
 * We will need to update this as the threshold is reached with more usages of useSelector.
 */
const MAX_EVENT_LISTENERS = 125;
/** Context of Call with Chat, which is a centralized context for all state updates */
class CallWithChatContext {
    constructor(clientState, maxListeners = MAX_EVENT_LISTENERS) {
        this.emitter = new events.EventEmitter();
        this.state = clientState;
        this.emitter.setMaxListeners(maxListeners);
    }
    onStateChange(handler) {
        this.emitter.on('stateChanged', handler);
    }
    offStateChange(handler) {
        this.emitter.off('stateChanged', handler);
    }
    setState(state) {
        this.state = state;
        this.emitter.emit('stateChanged', this.state);
    }
    getState() {
        return this.state;
    }
    updateClientState(clientState) {
        this.setState(clientState);
    }
    updateClientStateWithChatState(chatAdapterState) {
        this.updateClientState(mergeChatAdapterStateIntoCallWithChatAdapterState(this.state, chatAdapterState));
    }
    unsetChatState() {
        this.updateClientState(Object.assign(Object.assign({}, this.state), { chat: undefined }));
    }
    updateClientStateWithCallState(callAdapterState) {
        this.updateClientState(mergeCallAdapterStateIntoCallWithChatAdapterState(this.state, callAdapterState));
    }
}
/**
 * CallWithChat adapter backed by Azure Communication Services.
 * Created for easy use with the {@link CallWithChatComposite}.
 */
class AzureCommunicationCallWithChatAdapter {
    // This function is used to store a listener to the internal map of chat event listeners and should be
    // used when a listener subscribes to a chat event
    storeChatEventListener(event, listener) {
        var _a;
        if (!this.chatEventListeners.has(event)) {
            this.chatEventListeners.set(event, new Set());
        }
        (_a = this.chatEventListeners.get(event)) === null || _a === void 0 ? void 0 : _a.add(listener);
    }
    // This function is used to remove a listener from the internal map of chat event listeners and should
    // be used when a listener unsubscribes from a chat event
    removeChatEventListener(event, listener) {
        var _a;
        if (this.chatEventListeners.has(event)) {
            (_a = this.chatEventListeners.get(event)) === null || _a === void 0 ? void 0 : _a.delete(listener);
        }
    }
    constructor(callAdapter, chatAdapter) {
        this.emitter = new events.EventEmitter();
        this.isAdapterDisposed = false;
        // This map is used to store the listeners subscribed to chat events so that we can re-subscribe them
        // to the new chat adapter when the thread id changes
        this.chatEventListeners = new Map();
        this.bindPublicMethods();
        this.callAdapter = callAdapter;
        this.context = new CallWithChatContext(callWithChatAdapterStateFromBackingStates(callAdapter));
        const onChatStateChange = (newChatAdapterState) => {
            this.context.updateClientStateWithChatState(newChatAdapterState);
        };
        this.onChatStateChange = onChatStateChange;
        if (chatAdapter) {
            this.updateChatAdapter(chatAdapter);
            this.originCallChatAdapter = chatAdapter;
        }
        const onCallStateChange = (newCallAdapterState) => {
            this.context.updateClientStateWithCallState(newCallAdapterState);
        };
        this.callAdapter.onStateChange(onCallStateChange);
        this.callAdapter.on('breakoutRoomsUpdated', (eventData) => __awaiter$4(this, void 0, void 0, function* () {
            var _a, _b;
            if (eventData.type === 'join') {
                yield this.breakoutRoomJoined(eventData.data);
            }
            else if (eventData.type === 'assignedBreakoutRooms') {
                if (!eventData.data || eventData.data.state === 'closed') {
                    if (this.originCallChatAdapter && ((_a = this.originCallChatAdapter) === null || _a === void 0 ? void 0 : _a.getState().thread.threadId) !== ((_b = this.context.getState().chat) === null || _b === void 0 ? void 0 : _b.threadId)) {
                        this.updateChatAdapter(this.originCallChatAdapter);
                    }
                }
            }
        }));
        this.callAdapter.on('callEnded', () => {
            var _a, _b, _c;
            // If the call ended is a breakout room call with breakout room settings then update the chat adapter to the
            // origin call
            if ((_b = (_a = this.context.getState().call) === null || _a === void 0 ? void 0 : _a.breakoutRooms) === null || _b === void 0 ? void 0 : _b.breakoutRoomSettings) {
                // Unsubscribe from chat adapter state changes
                (_c = this.chatAdapter) === null || _c === void 0 ? void 0 : _c.offStateChange(this.onChatStateChange);
                // Unassign chat adapter
                this.chatAdapter = undefined;
                // Set chat state to undefined to ensure that the chat thread of the breakout room is not shown
                this.context.unsetChatState();
                // Update chat state to the origin call chat adapter
                if (this.originCallChatAdapter) {
                    this.updateChatAdapter(this.originCallChatAdapter);
                }
            }
        });
        this.onCallStateChange = onCallStateChange;
    }
    breakoutRoomJoined(call) {
        return __awaiter$4(this, void 0, void 0, function* () {
            var _a;
            const targetThreadId = call.info.threadId;
            // If the chat adapter is not on the target thread then we need to switch to the breakout room chat adapter
            if (targetThreadId && this.chatAdapter && this.chatAdapter.getState().thread.threadId !== targetThreadId) {
                // Unsubscribe from chat adapter state changes
                this.chatAdapter.offStateChange(this.onChatStateChange);
                // Set chat state to undefined to prevent showing chat thread of origin call
                this.context.unsetChatState();
                // Check if the breakout room chat adapter has been initialized
                this.breakoutRoomChatAdapter = yield this.setBreakoutRoomChatAdapterToThread(targetThreadId);
                // Wait for the user to be added to the thread of chat adapter before updating the current chat adapter
                // to avoid chat errors of not having access to the chat thread. This delayed access to the chat thread
                // is also seen in Teams
                // Check up to 20 times every 500ms and then continue.
                yield busyWait(() => {
                    var _a, _b;
                    // If the call adapter's call id has been changed, then stop waiting and don't update the chat adapter
                    if (((_a = this.context.getState().call) === null || _a === void 0 ? void 0 : _a.id) !== call.id) {
                        return true;
                    }
                    const userAddedToThread = !!((_b = this.breakoutRoomChatAdapter) === null || _b === void 0 ? void 0 : _b.getState().thread.participants[toFlatCommunicationIdentifier(this.context.getState().userId)]);
                    return userAddedToThread;
                }, 20);
                // If the call adapter's call has been changed while for waiting for breakout room chat adapter to be ready
                // then don't update the chat adapter
                if (((_a = this.context.getState().call) === null || _a === void 0 ? void 0 : _a.id) !== call.id) {
                    return;
                }
                this.updateChatAdapter(this.breakoutRoomChatAdapter);
            }
        });
    }
    setBreakoutRoomChatAdapterToThread(targetThreadId) {
        return __awaiter$4(this, void 0, void 0, function* () {
            if (this.breakoutRoomChatAdapter) {
                // If the breakout room chat adapter is not set on the target thread then unsubscribe, dispose, and
                // reinitialize the chat adapter for the target thread
                if (this.breakoutRoomChatAdapter.getState().thread.threadId !== targetThreadId) {
                    this.breakoutRoomChatAdapter.offStateChange(this.onChatStateChange);
                    this.breakoutRoomChatAdapter.dispose();
                    const newBreakoutRoomChatAdapter = yield this.createNewChatAdapterForThread(targetThreadId);
                    return newBreakoutRoomChatAdapter;
                }
                else {
                    return this.breakoutRoomChatAdapter;
                }
            }
            else {
                // Initiliaze the breakout room chat adapter for the target thread
                const newBreakoutRoomChatAdapter = yield this.createNewChatAdapterForThread(targetThreadId);
                return newBreakoutRoomChatAdapter;
            }
        });
    }
    setChatAdapterPromise(chatAdapter) {
        chatAdapter.then(adapter => {
            if (!this.isAdapterDisposed) {
                this.updateChatAdapter(adapter);
                this.originCallChatAdapter = adapter;
            }
        });
    }
    setCreateChatAdapterCallback(chatThreadCallBack) {
        this.createChatAdapterCallback = chatThreadCallBack;
    }
    createNewChatAdapterForThread(threadId) {
        if (this.createChatAdapterCallback) {
            return this.createChatAdapterCallback(threadId);
        }
        throw new Error('Unable to create chat adapter for thread because createChatAdapterCallback is not set');
    }
    updateChatAdapter(chatAdapter) {
        var _a;
        (_a = this.chatAdapter) === null || _a === void 0 ? void 0 : _a.offStateChange(this.onChatStateChange);
        this.updateChatEventListeners(chatAdapter);
        this.chatAdapter = chatAdapter;
        this.chatAdapter.onStateChange(this.onChatStateChange);
        this.context.updateClientStateWithChatState(chatAdapter.getState());
        this.emitter.emit('chatInitialized', this.chatAdapter);
    }
    updateChatEventListeners(chatAdapter) {
        var _a;
        for (const [eventName, listeners] of this.chatEventListeners) {
            for (const listener of listeners) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                (_a = this.chatAdapter) === null || _a === void 0 ? void 0 : _a.off(eventName, listener);
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                chatAdapter.on(eventName, listener);
            }
        }
    }
    bindPublicMethods() {
        this.joinCall.bind(this);
        this.leaveCall.bind(this);
        this.startCall.bind(this);
        this.onStateChange.bind(this);
        this.offStateChange.bind(this);
        this.getState.bind(this);
        this.dispose.bind(this);
        this.setCamera.bind(this);
        this.setMicrophone.bind(this);
        this.setSpeaker.bind(this);
        this.askDevicePermission.bind(this);
        this.queryCameras.bind(this);
        this.queryMicrophones.bind(this);
        this.querySpeakers.bind(this);
        this.startCamera.bind(this);
        this.stopCamera.bind(this);
        this.mute.bind(this);
        this.unmute.bind(this);
        this.startScreenShare.bind(this);
        this.stopScreenShare.bind(this);
        this.raiseHand.bind(this);
        this.lowerHand.bind(this);
        this.onReactionClick.bind(this);
        this.removeParticipant.bind(this);
        this.createStreamView.bind(this);
        this.disposeStreamView.bind(this);
        this.disposeScreenShareStreamView.bind(this);
        this.fetchInitialData.bind(this);
        this.sendMessage.bind(this);
        this.sendReadReceipt.bind(this);
        this.sendTypingIndicator.bind(this);
        this.loadPreviousChatMessages.bind(this);
        this.updateMessage.bind(this);
        this.deleteMessage.bind(this);
        this.on.bind(this);
        this.off.bind(this);
        this.downloadResourceToCache = this.downloadResourceToCache.bind(this);
        this.removeResourceFromCache = this.removeResourceFromCache.bind(this);
        this.holdCall.bind(this);
        this.resumeCall.bind(this);
        this.addParticipant.bind(this);
        this.sendDtmfTone.bind(this);
        this.startCaptions.bind(this);
        this.stopCaptions.bind(this);
        this.setSpokenLanguage.bind(this);
        this.setCaptionLanguage.bind(this);
        this.sendRealTimeText.bind(this);
        this.startVideoBackgroundEffect.bind(this);
        this.stopVideoBackgroundEffects.bind(this);
        this.updateBackgroundPickerImages.bind(this);
        this.startNoiseSuppressionEffect.bind(this);
        this.stopNoiseSuppressionEffect.bind(this);
    }
    /** Join existing Call. */
    joinCall(options) {
        if (typeof options === 'boolean') {
            return this.callAdapter.joinCall(options);
        }
        else {
            return this.callAdapter.joinCall(options);
        }
    }
    /** Leave current Call. */
    leaveCall(forEveryone) {
        return __awaiter$4(this, void 0, void 0, function* () {
            // Only remove self from the GroupCall. Contoso must manage access to Chat.
            yield this.callAdapter.leaveCall(forEveryone);
        });
    }
    /** Start a new Call. */
    startCall(participants, options) {
        if (participants.length === 0) {
            throw new Error('At least one participant is required to start a call');
        }
        if (typeof participants[0] === 'string') {
            return this.callAdapter.startCall(participants, options);
        }
        else {
            return this.callAdapter.startCall(participants, options);
        }
    }
    /**
     * Subscribe to state change events.
     * @param handler - handler to be called when the state changes. This is passed the new state.
     */
    onStateChange(handler) {
        this.context.onStateChange(handler);
    }
    /**
     * Unsubscribe to state change events.
     * @param handler - handler to be no longer called when state changes.
     */
    offStateChange(handler) {
        this.context.offStateChange(handler);
    }
    /** Get current Call and Chat state. */
    getState() {
        return this.context.getState();
    }
    /** Dispose of the current CallWithChatAdapter. */
    dispose() {
        this.isAdapterDisposed = true;
        if (this.chatAdapter) {
            this.chatAdapter.offStateChange(this.onChatStateChange);
            this.chatAdapter.dispose();
        }
        this.callAdapter.offStateChange(this.onCallStateChange);
        this.callAdapter.dispose();
        // Clear chat event listeners
        this.chatEventListeners.clear();
    }
    /** Remove a participant from the Call only. */
    removeParticipant(userId) {
        return __awaiter$4(this, void 0, void 0, function* () {
            let participant = userId;
            participant = _toCommunicationIdentifier(userId);
            yield this.callAdapter.removeParticipant(participant);
        });
    }
    setCamera(device, options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.setCamera(device, options);
        });
    }
    /** Set the microphone to be used in the Call. */
    setMicrophone(device) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.setMicrophone(device);
        });
    }
    /** Set the speaker to be used in the Call. */
    setSpeaker(device) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.setSpeaker(device);
        });
    }
    askDevicePermission(constraints) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.askDevicePermission(constraints);
        });
    }
    /** Query for available cameras. */
    queryCameras() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.queryCameras();
        });
    }
    /** Query for available microphones. */
    queryMicrophones() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.queryMicrophones();
        });
    }
    /** Query for available speakers. */
    querySpeakers() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.querySpeakers();
        });
    }
    /** Start the camera for the user in the Call. */
    startCamera(options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.startCamera(options);
        });
    }
    /** Stop the camera for the user in the Call. */
    stopCamera() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.stopCamera();
        });
    }
    /** Mute the user in the Call. */
    mute() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.mute();
        });
    }
    /** Unmute the user in the Call. */
    unmute() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.unmute();
        });
    }
    /** Trigger the user to start screen share. */
    startScreenShare() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.startScreenShare();
        });
    }
    /** Stop the current active screen share. */
    stopScreenShare() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.stopScreenShare();
        });
    }
    /** Raise hand for local user. */
    raiseHand() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.raiseHand();
        });
    }
    /** Lower hand for local user. */
    lowerHand() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.lowerHand();
        });
    }
    /** Reaction clicked by the local user. */
    onReactionClick(reaction) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.onReactionClick(reaction);
        });
    }
    /** Create a stream view for a remote participants video feed. */
    createStreamView(remoteUserId, options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.createStreamView(remoteUserId, options);
        });
    }
    /** Dispose of a created stream view of a remote participants video feed. */
    disposeStreamView(remoteUserId, options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.disposeStreamView(remoteUserId, options);
        });
    }
    /** Dispose of a remote screen share */
    disposeScreenShareStreamView(remoteUserId) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.disposeScreenShareStreamView(remoteUserId);
        });
    }
    /** Dispose of a remote video stream */
    disposeRemoteVideoStreamView(remoteUserId) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.disposeRemoteVideoStreamView(remoteUserId);
        });
    }
    /** Dispose of the local video stream */
    disposeLocalVideoStreamView() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.disposeLocalVideoStreamView();
        });
    }
    /** Create a together mode stream view */
    createTogetherModeStreamView(options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.createTogetherModeStreamView(options);
        });
    }
    /** Start together mode for all participants */
    startTogetherMode() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.startTogetherMode();
        });
    }
    /** Set together mode scene size */
    setTogetherModeSceneSize(width, height) {
        return this.callAdapter.setTogetherModeSceneSize(width, height);
    }
    /** Dispose together mode video stream */
    disposeTogetherModeStreamView() {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.disposeTogetherModeStreamView();
        });
    }
    /** Fetch initial Call and Chat data such as chat messages. */
    fetchInitialData() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.executeWithResolvedChatAdapter(adapter => {
                return adapter.fetchInitialData();
            });
        });
    }
    /** Send a chat message. */
    sendMessage(content) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.executeWithResolvedChatAdapter(adapter => {
                return adapter.sendMessage(content);
            });
        });
    }
    /** Send a chat read receipt. */
    sendReadReceipt(chatMessageId) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.executeWithResolvedChatAdapter(adapter => {
                return adapter.sendReadReceipt(chatMessageId);
            });
        });
    }
    /** Send an isTyping indicator. */
    sendTypingIndicator() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.executeWithResolvedChatAdapter(adapter => {
                return adapter.sendTypingIndicator();
            });
        });
    }
    /** Load previous Chat messages. */
    loadPreviousChatMessages(messagesToLoad) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.executeWithResolvedChatAdapter(adapter => {
                return adapter.loadPreviousChatMessages(messagesToLoad);
            });
        });
    }
    /** Update an existing message. */
    updateMessage(messageId, content, options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.executeWithResolvedChatAdapter(adapter => {
                return adapter.updateMessage(messageId, content, options);
            });
        });
    }
    /** Delete an existing message. */
    deleteMessage(messageId) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.executeWithResolvedChatAdapter(adapter => {
                return adapter.deleteMessage(messageId);
            });
        });
    }
    downloadResourceToCache(resourceDetails) {
        return __awaiter$4(this, void 0, void 0, function* () {
            this.executeWithResolvedChatAdapter(adapter => {
                adapter.downloadResourceToCache(resourceDetails);
            });
        });
    }
    removeResourceFromCache(resourceDetails) {
        this.executeWithResolvedChatAdapter(adapter => {
            adapter.removeResourceFromCache(resourceDetails);
        });
    }
    holdCall() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.holdCall();
        });
    }
    resumeCall() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.resumeCall();
        });
    }
    addParticipant(participant, options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            if (communicationCommon.isCommunicationUserIdentifier(participant)) {
                return yield this.callAdapter.addParticipant(participant);
            }
            else {
                return yield this.callAdapter.addParticipant(participant, options);
            }
        });
    }
    sendDtmfTone(dtmfTone) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.sendDtmfTone(dtmfTone);
        });
    }
    startCaptions(options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.startCaptions(options);
        });
    }
    stopCaptions(options) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.stopCaptions(options);
        });
    }
    setCaptionLanguage(language) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.setCaptionLanguage(language);
        });
    }
    setSpokenLanguage(language) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.setSpokenLanguage(language);
        });
    }
    sendRealTimeText(text, isFinalized) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.sendRealTimeText(text, isFinalized);
        });
    }
    startVideoBackgroundEffect(videoBackgroundEffect) {
        return __awaiter$4(this, void 0, void 0, function* () {
            yield this.callAdapter.startVideoBackgroundEffect(videoBackgroundEffect);
        });
    }
    stopVideoBackgroundEffects() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.stopVideoBackgroundEffects();
        });
    }
    updateBackgroundPickerImages(backgroundImages) {
        return this.callAdapter.updateBackgroundPickerImages(backgroundImages);
    }
    updateSelectedVideoBackgroundEffect(selectedVideoBackground) {
        return this.callAdapter.updateSelectedVideoBackgroundEffect(selectedVideoBackground);
    }
    startNoiseSuppressionEffect() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.startNoiseSuppressionEffect();
        });
    }
    stopNoiseSuppressionEffect() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return yield this.callAdapter.stopNoiseSuppressionEffect();
        });
    }
    submitSurvey(survey) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.submitSurvey(survey);
        });
    }
    startSpotlight(userIds) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.startSpotlight(userIds);
        });
    }
    stopSpotlight(userIds) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.stopSpotlight(userIds);
        });
    }
    stopAllSpotlight() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.stopAllSpotlight();
        });
    }
    muteParticipant(userId) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.muteParticipant(userId);
        });
    }
    muteAllRemoteParticipants() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.muteAllRemoteParticipants();
        });
    }
    returnFromBreakoutRoom() {
        return __awaiter$4(this, void 0, void 0, function* () {
            var _a, _b, _c, _d;
            if (this.originCallChatAdapter && ((_a = this.context.getState().chat) === null || _a === void 0 ? void 0 : _a.threadId) !== this.originCallChatAdapter.getState().thread.threadId) {
                (_b = this.breakoutRoomChatAdapter) === null || _b === void 0 ? void 0 : _b.dispose();
                this.updateChatAdapter(this.originCallChatAdapter);
            }
            // Check if breakout room settings are defined to verify the user is in a breakout room before
            // returning to the origin call.
            const breakoutRoomSettings = (_d = (_c = this.context.getState().call) === null || _c === void 0 ? void 0 : _c.breakoutRooms) === null || _d === void 0 ? void 0 : _d.breakoutRoomSettings;
            if (breakoutRoomSettings) {
                yield this.callAdapter.returnFromBreakoutRoom();
            }
        });
    }
    forbidAudio(userIds) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.forbidAudio(userIds);
        });
    }
    permitAudio(userIds) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.permitAudio(userIds);
        });
    }
    forbidOthersAudio() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.forbidOthersAudio();
        });
    }
    permitOthersAudio() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.permitOthersAudio();
        });
    }
    forbidVideo(userIds) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.forbidVideo(userIds);
        });
    }
    permitVideo(userIds) {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.permitVideo(userIds);
        });
    }
    forbidOthersVideo() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.forbidOthersVideo();
        });
    }
    permitOthersVideo() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.callAdapter.permitOthersVideo();
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    on(event, listener) {
        switch (event) {
            case 'callParticipantsJoined':
                this.callAdapter.on('participantsJoined', listener);
                break;
            case 'callParticipantsLeft':
                this.callAdapter.on('participantsLeft', listener);
                break;
            case 'callEnded':
                this.callAdapter.on('callEnded', listener);
                break;
            case 'isMutedChanged':
                this.callAdapter.on('isMutedChanged', listener);
                break;
            case 'callIdChanged':
                this.callAdapter.on('callIdChanged', listener);
                break;
            case 'isLocalScreenSharingActiveChanged':
                this.callAdapter.on('isLocalScreenSharingActiveChanged', listener);
                break;
            case 'displayNameChanged':
                this.callAdapter.on('displayNameChanged', listener);
                break;
            case 'isSpeakingChanged':
                this.callAdapter.on('isSpeakingChanged', listener);
                break;
            case 'selectedMicrophoneChanged':
                this.callAdapter.on('selectedMicrophoneChanged', listener);
                break;
            case 'selectedSpeakerChanged':
                this.callAdapter.on('selectedSpeakerChanged', listener);
                break;
            case 'captionsReceived':
                this.callAdapter.on('captionsReceived', listener);
                break;
            case 'realTimeTextReceived':
                this.callAdapter.on('realTimeTextReceived', listener);
                break;
            case 'isCaptionsActiveChanged':
                this.callAdapter.on('isCaptionsActiveChanged', listener);
                break;
            case 'isCaptionLanguageChanged':
                this.callAdapter.on('isCaptionLanguageChanged', listener);
                break;
            case 'isSpokenLanguageChanged':
                this.callAdapter.on('isSpokenLanguageChanged', listener);
                break;
            case 'messageReceived':
                this.storeChatEventListener('messageReceived', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('messageReceived', listener);
                });
                break;
            case 'messageEdited':
                this.storeChatEventListener('messageEdited', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('messageEdited', listener);
                });
                break;
            case 'messageDeleted':
                this.storeChatEventListener('messageDeleted', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('messageDeleted', listener);
                });
                break;
            case 'messageSent':
                this.storeChatEventListener('messageSent', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('messageSent', listener);
                });
                break;
            case 'messageRead':
                this.storeChatEventListener('messageRead', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('messageRead', listener);
                });
                break;
            case 'chatParticipantsAdded':
                this.storeChatEventListener('chatParticipantsAdded', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('participantsAdded', listener);
                });
                break;
            case 'chatParticipantsRemoved':
                this.storeChatEventListener('chatParticipantsRemoved', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('participantsRemoved', listener);
                });
                break;
            case 'callError':
                this.callAdapter.on('error', listener);
                break;
            case 'chatError':
                this.storeChatEventListener('chatError', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.on('error', listener);
                });
                break;
            case 'chatInitialized':
                this.emitter.on(event, listener);
                break;
            case 'capabilitiesChanged':
                this.callAdapter.on('capabilitiesChanged', listener);
                break;
            case 'spotlightChanged':
                this.callAdapter.on('spotlightChanged', listener);
                break;
            case 'breakoutRoomsUpdated':
                this.callAdapter.on('breakoutRoomsUpdated', listener);
                break;
            default:
                throw `Unknown AzureCommunicationCallWithChatAdapter Event: ${event}`;
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    off(event, listener) {
        switch (event) {
            case 'callParticipantsJoined':
                this.callAdapter.off('participantsJoined', listener);
                break;
            case 'callParticipantsLeft':
                this.callAdapter.off('participantsLeft', listener);
                break;
            case 'callEnded':
                this.callAdapter.off('callEnded', listener);
                break;
            case 'isMutedChanged':
                this.callAdapter.off('isMutedChanged', listener);
                break;
            case 'callIdChanged':
                this.callAdapter.off('callIdChanged', listener);
                break;
            case 'isLocalScreenSharingActiveChanged':
                this.callAdapter.off('isLocalScreenSharingActiveChanged', listener);
                break;
            case 'displayNameChanged':
                this.callAdapter.off('displayNameChanged', listener);
                break;
            case 'isSpeakingChanged':
                this.callAdapter.off('isSpeakingChanged', listener);
                break;
            case 'selectedMicrophoneChanged':
                this.callAdapter.off('selectedMicrophoneChanged', listener);
                break;
            case 'selectedSpeakerChanged':
                this.callAdapter.off('selectedSpeakerChanged', listener);
                break;
            case 'captionsReceived':
                this.callAdapter.off('captionsReceived', listener);
                break;
            case 'realTimeTextReceived':
                this.callAdapter.off('realTimeTextReceived', listener);
                break;
            case 'isCaptionsActiveChanged':
                this.callAdapter.off('isCaptionsActiveChanged', listener);
                break;
            case 'isCaptionLanguageChanged':
                this.callAdapter.off('isCaptionLanguageChanged', listener);
                break;
            case 'isSpokenLanguageChanged':
                this.callAdapter.off('isSpokenLanguageChanged', listener);
                break;
            case 'messageReceived':
                this.removeChatEventListener('messageReceived', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('messageReceived', listener);
                });
                break;
            case 'messageEdited':
                this.removeChatEventListener('messageEdited', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('messageEdited', listener);
                });
                break;
            case 'messageDeleted':
                this.removeChatEventListener('messageDeleted', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('messageDeleted', listener);
                });
                break;
            case 'messageSent':
                this.removeChatEventListener('messageSent', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('messageSent', listener);
                });
                break;
            case 'messageRead':
                this.removeChatEventListener('messageRead', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('messageRead', listener);
                });
                break;
            case 'chatParticipantsAdded':
                this.removeChatEventListener('chatParticipantsAdded', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('participantsAdded', listener);
                });
                break;
            case 'chatParticipantsRemoved':
                this.removeChatEventListener('chatParticipantsRemoved', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('participantsRemoved', listener);
                });
                break;
            case 'callError':
                this.callAdapter.off('error', listener);
                break;
            case 'chatError':
                this.removeChatEventListener('chatError', listener);
                this.executeWithResolvedChatAdapter(adapter => {
                    adapter.off('error', listener);
                });
                break;
            case 'chatInitialized':
                this.emitter.off(event, listener);
                break;
            case 'capabilitiesChanged':
                this.callAdapter.off('capabilitiesChanged', listener);
                break;
            case 'spotlightChanged':
                this.callAdapter.off('spotlightChanged', listener);
                break;
            case 'breakoutRoomsUpdated':
                this.callAdapter.off('breakoutRoomsUpdated', listener);
                break;
            default:
                throw `Unknown AzureCommunicationCallWithChatAdapter Event: ${event}`;
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    executeWithResolvedChatAdapter(callback) {
        if (!this.chatAdapter) {
            console.error('Chat is not initialized');
        }
        else {
            return callback(this.chatAdapter);
        }
    }
}
/**
 * Arguments for use in {@link createAzureCommunicationCallWithChatAdapter} to join a Group Call with an associated Chat thread.
 * @private
 */
class CallAndChatProvider {
    constructor(locator) {
        this.locator = locator;
    }
    isCallInfoRequired() {
        return false;
    }
    getChatThreadPromise() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return this.getChatThread();
        });
    }
    getChatThread() {
        return this.locator.chatThreadId;
    }
}
/**
 * Arguments for use in {@link createAzureCommunicationCallWithChatAdapter} to join a Teams meeting with an associated Chat thread.
 *
 * @private
 */
class TeamsMeetingLinkProvider {
    constructor(locator, callAdapterPromise) {
        this.locator = locator;
        this.callAdapterPromise = callAdapterPromise;
    }
    isCallInfoRequired() {
        return true;
    }
    getChatThread() {
        throw new Error('Chat thread ID should be retrieved from call.callInfo using method getChatThreadPromise');
    }
    getChatThreadPromise() {
        return __awaiter$4(this, void 0, void 0, function* () {
            {
                // Wait for the call to be connected and get the chat thread ID from `call.callInfo`.
                const chatThreadPromise = new Promise(resolve => {
                    this.callAdapterPromise.then(callAdapter => {
                        // Ensure function is idempotent by removing any existing subscription.
                        this.callAdapterSubscription && callAdapter.offStateChange(this.callAdapterSubscription);
                        this.callAdapterSubscription = (state) => {
                            var _a, _b, _c;
                            if (((_a = state.call) === null || _a === void 0 ? void 0 : _a.state) === 'Connected' && ((_b = state.call.info) === null || _b === void 0 ? void 0 : _b.threadId)) {
                                this.callAdapterSubscription && callAdapter.offStateChange(this.callAdapterSubscription);
                                this.callAdapterSubscription = undefined;
                                resolve((_c = state.call.info) === null || _c === void 0 ? void 0 : _c.threadId);
                            }
                        };
                        callAdapter.onStateChange(this.callAdapterSubscription);
                    });
                });
                return chatThreadPromise;
            }
        });
    }
}
/**
 * Arguments for use in {@link createAzureCommunicationCallWithChatAdapter} to join a Teams meeting using meeting id.
 *
 * @private
 */
class TeamsMeetingIdProvider {
    constructor(locator, callAdapter) {
        this.locator = locator;
        this.callAdapter = callAdapter;
    }
    isCallInfoRequired() {
        return true;
    }
    getChatThread() {
        throw new Error('Chat thread ID is not available for Teams meeting ID');
    }
    /**
     * Wait call to be connected to get thread ID.
     * @returns the chat thread ID for the given meeting ID.
     */
    getChatThreadPromise() {
        return __awaiter$4(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                const stateChangeListener = (state) => {
                    var _a, _b, _c;
                    if (((_a = state.call) === null || _a === void 0 ? void 0 : _a.state) === 'Connected' && ((_b = state.call.info) === null || _b === void 0 ? void 0 : _b.threadId)) {
                        this.callAdapter.then(adapter => {
                            adapter.offStateChange(stateChangeListener);
                        });
                        resolve((_c = state.call.info) === null || _c === void 0 ? void 0 : _c.threadId);
                    }
                };
                this.callAdapter.then(adapter => {
                    var _a, _b, _c;
                    const callState = (_a = adapter.getState().call) === null || _a === void 0 ? void 0 : _a.state;
                    const threadId = (_c = (_b = adapter.getState().call) === null || _b === void 0 ? void 0 : _b.info) === null || _c === void 0 ? void 0 : _c.threadId;
                    if (callState === 'Connected' && threadId) {
                        resolve(threadId);
                    }
                    else {
                        adapter.onStateChange(stateChangeListener);
                    }
                });
            });
        });
    }
}
/**
 * Create a CallWithChatAdapter backed by Azure Communication services
 * to plug into the {@link CallWithChatComposite}.
 *
 * @public
 */
const createAzureCommunicationCallWithChatAdapter = (_a) => __awaiter$4(void 0, [_a], void 0, function* ({ userId, displayName, credential, endpoint, locator, alternateCallerId, callAdapterOptions }) {
    const callAdapterLocator = isTeamsMeetingLocator(locator) ? locator : locator.callLocator;
    const callAdapter = _createAzureCommunicationCallAdapterInner({
        userId,
        displayName,
        credential,
        locator: callAdapterLocator,
        alternateCallerId,
        options: callAdapterOptions,
        telemetryImplementationHint: 'CallWithChat'
    });
    const chatThreadAdapter = _createChatThreadAdapterInner(locator, callAdapter);
    const chatAdapterOptions = {
        onFetchProfile: callAdapterOptions === null || callAdapterOptions === void 0 ? void 0 : callAdapterOptions.onFetchProfile
    };
    if (chatThreadAdapter.isCallInfoRequired()) {
        const callWithChatAdapter = new AzureCommunicationCallWithChatAdapter(yield callAdapter);
        const chatAdapterPromise = _createLazyAzureCommunicationChatAdapterInner(endpoint, userId, displayName, credential, chatThreadAdapter.getChatThreadPromise(), 'CallWithChat', chatAdapterOptions);
        callWithChatAdapter.setChatAdapterPromise(chatAdapterPromise);
        callWithChatAdapter.setCreateChatAdapterCallback((threadId) => _createAzureCommunicationChatAdapterInner(endpoint, userId, displayName, credential, threadId, 'CallWithChat', chatAdapterOptions));
        return callWithChatAdapter;
    }
    else {
        const chatAdapter = _createAzureCommunicationChatAdapterInner(endpoint, userId, displayName, credential, chatThreadAdapter.getChatThread(), 'CallWithChat', chatAdapterOptions);
        const callWithChatAdapter = new AzureCommunicationCallWithChatAdapter(yield callAdapter, yield chatAdapter);
        callWithChatAdapter.setCreateChatAdapterCallback((threadId) => _createAzureCommunicationChatAdapterInner(endpoint, userId, displayName, credential, threadId, 'CallWithChat', chatAdapterOptions));
        return callWithChatAdapter;
    }
});
/**
 * A custom React hook to simplify the creation of {@link CallWithChatAdapter}.
 *
 * Similar to {@link createAzureCommunicationCallWithChatAdapter}, but takes care of asynchronous
 * creation of the adapter internally.
 *
 * Allows arguments to be undefined so that you can respect the rule-of-hooks and pass in arguments
 * as they are created. The adapter is only created when all arguments are defined.
 *
 * Note that you must memoize the arguments to avoid recreating adapter on each render.
 * See storybook for typical usage examples.
 *
 * @public
 */
const useAzureCommunicationCallWithChatAdapter = (args, afterCreate, beforeDispose) => {
    const { credential, displayName, endpoint, locator, userId, alternateCallerId, callAdapterOptions } = args;
    // State update needed to rerender the parent component when a new adapter is created.
    const [adapter, setAdapter] = React.useState(undefined);
    // Ref needed for cleanup to access the old adapter created asynchronously.
    const adapterRef = React.useRef(undefined);
    const creatingAdapterRef = React.useRef(false);
    const afterCreateRef = React.useRef(undefined);
    const beforeDisposeRef = React.useRef(undefined);
    // These refs are updated on *each* render, so that the latest values
    // are used in the `useEffect` closures below.
    // Using a Ref ensures that new values for the callbacks do not trigger the
    // useEffect blocks, and a new adapter creation / distruction is not triggered.
    afterCreateRef.current = afterCreate;
    beforeDisposeRef.current = beforeDispose;
    React.useEffect(() => {
        if (!credential || !displayName || !endpoint || !locator || !userId) {
            return;
        }
        (() => __awaiter$4(void 0, void 0, void 0, function* () {
            if (adapterRef.current) {
                // Dispose the old adapter when a new one is created.
                //
                // This clean up function uses `adapterRef` because `adapter` can not be added to the dependency array of
                // this `useEffect` -- we do not want to trigger a new adapter creation because of the first adapter
                // creation.
                if (beforeDisposeRef.current) {
                    yield beforeDisposeRef.current(adapterRef.current);
                }
                adapterRef.current.dispose();
                adapterRef.current = undefined;
            }
            if (creatingAdapterRef.current) {
                console.warn('Adapter is already being created, please see storybook for more information: https://azure.github.io/communication-ui-library/?path=/story/troubleshooting--page');
                return;
            }
            creatingAdapterRef.current = true;
            let newAdapter = yield createAzureCommunicationCallWithChatAdapter({
                credential,
                displayName,
                endpoint,
                locator,
                userId,
                alternateCallerId,
                callAdapterOptions
            });
            if (afterCreateRef.current) {
                newAdapter = yield afterCreateRef.current(newAdapter);
            }
            adapterRef.current = newAdapter;
            creatingAdapterRef.current = false;
            setAdapter(newAdapter);
        }))();
    }, 
    // Explicitly list all arguments so that caller doesn't have to memoize the `args` object.
    [adapterRef, afterCreateRef, alternateCallerId, beforeDisposeRef, credential, displayName, endpoint, locator, userId, callAdapterOptions]);
    // Dispose any existing adapter when the component unmounts.
    React.useEffect(() => {
        return () => {
            (() => __awaiter$4(void 0, void 0, void 0, function* () {
                if (adapterRef.current) {
                    if (beforeDisposeRef.current) {
                        yield beforeDisposeRef.current(adapterRef.current);
                    }
                    adapterRef.current.dispose();
                    adapterRef.current = undefined;
                }
            }))();
        };
    }, []);
    return adapter;
};
/**
 * Create a {@link CallWithChatAdapter} using the provided {@link StatefulChatClient} and {@link StatefulCallClient}.
 *
 * Useful if you want to keep a reference to {@link StatefulChatClient} and {@link StatefulCallClient}.
 * Please note that chatThreadClient has to be created by StatefulChatClient via chatClient.getChatThreadClient(chatThreadId) API.
 * Consider using {@link createAzureCommunicationCallWithChatAdapter} for a simpler API.
 *
 * @public
 */
const createAzureCommunicationCallWithChatAdapterFromClients = (_b) => __awaiter$4(void 0, [_b], void 0, function* ({ callClient, callAgent, callLocator, chatClient, chatThreadClient, callAdapterOptions }) {
    const callAdapter = yield createAzureCommunicationCallAdapterFromClient(callClient, callAgent, callLocator, callAdapterOptions);
    const chatAdapter = yield createAzureCommunicationChatAdapterFromClient(chatClient, chatThreadClient);
    const callWithChatAdapter = new AzureCommunicationCallWithChatAdapter(callAdapter, chatAdapter);
    callWithChatAdapter.setCreateChatAdapterCallback((threadId) => createAzureCommunicationChatAdapterFromClient(chatClient, chatClient.getChatThreadClient(threadId)));
    return callWithChatAdapter;
});
const isTeamsMeetingLocator = (locator) => {
    return 'meetingLink' in locator || 'meetingId' in locator;
};
const _createChatThreadAdapterInner = (locator, adapter) => {
    if ('meetingLink' in locator) {
        return new TeamsMeetingLinkProvider(locator, adapter);
    }
    if ('meetingId' in locator) {
        return new TeamsMeetingIdProvider(locator, adapter);
    }
    return new CallAndChatProvider(locator);
};

var call$m={cameraLabel:"Camera",noCamerasLabel:"No cameras found",cameraPermissionDenied:"Your browser is blocking access to your camera",cameraTurnedOff:"Your camera is turned off",chatButtonLabel:"Chat",close:"Close",complianceBannerNowOnlyRecording:"You are now only recording this meeting.",complianceBannerNowOnlyTranscription:"You are now only transcribing this meeting.",complianceBannerRecordingAndTranscriptionSaved:"Recording and transcription are being saved.",complianceBannerRecordingAndTranscriptionStarted:"Recording and transcription have started.",complianceBannerRecordingAndTranscriptionStopped:"Recording and transcription have stopped.",complianceBannerRecordingSaving:"Recording is being saved.",complianceBannerRecordingStarted:"Recording has started.",complianceBannerRecordingStopped:"Recording has stopped.",complianceBannerTranscriptionStarted:"Transcription has started.",complianceBannerTranscriptionConsent:"By joining, you are giving consent for this meeting to be transcribed.",complianceBannerTranscriptionSaving:"Transcription is being saved.",complianceBannerTranscriptionStopped:"Transcription has stopped.",configurationPageTitle:"Start a call",copyInviteLinkButtonLabel:"Copy invite link",copyInviteLinkButtonActionedLabel:"Link copied",copyInviteLinkActionedAriaLabel:"Invite link copied",defaultPlaceHolder:"Select an option",dismissSidePaneButtonLabel:"Close",videoEffectsPaneTitle:"Effects",videoEffectsPaneBackgroundSelectionTitle:"Background",videoEffectsPaneAriaLabel:"Video effects pane",configurationPageCameraIsLoadingLabel:"Video is loading...",configurationPageVideoEffectsButtonLabel:"Effects",unableToStartVideoEffect:"Unable to apply video effect.",blurBackgroundEffectButtonLabel:"Blur",blurBackgroundTooltip:"Blur Background",removeBackgroundEffectButtonLabel:"None",removeBackgroundTooltip:"Remove Background",cameraOffBackgroundEffectWarningText:"Your camera is off. Turn on camera to see video effect.",failedToJoinCallDueToNoNetworkMoreDetails:"Call was disconnected due to a network issue. Check your connection and join again.",failedToJoinCallDueToNoNetworkTitle:"Call disconnected",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"You were not granted entry in the call. If this was a mistake, re-join the call.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Dismissed from lobby",learnMore:"Learn more",leavingCallTitle:"Leaving...",leftCallMoreDetails:"If this was a mistake, re-join the call.",leftCallTitle:"You left the call",lobbyScreenConnectingToCallTitle:"Joining call",lobbyScreenWaitingToBeAdmittedTitle:"Waiting to be admitted",microphonePermissionDenied:"Your browser is blocking access to your microphone",microphoneToggleInLobbyNotAllowed:"Cannot mute or unmute while in lobby.",mutedMessage:"You're muted",networkReconnectMoreDetails:"Looks like something went wrong. We're trying to get back into the call.",networkReconnectTitle:"Hold on",notInvitedToRoomDetails:"You are not able to join this room because you do not have an invite.",notInvitedToRoomTitle:"Not invited to room",peopleButtonLabel:"People",selectedPeopleButtonLabel:"People Button Selected",peoplePaneTitle:"People",peoplePaneMoreButtonAriaLabel:"More",peopleButtonTooltipOpen:"Show participants",peopleButtonTooltipClose:"Hide participants",peoplePaneSubTitle:"In this call {numberOfPeople}",privacyPolicy:"Privacy policy",rejoinCallButtonLabel:"Re-join call",removedFromCallMoreDetails:"Another participant removed you from the call.",removedFromCallTitle:"You were removed",removeMenuLabel:"Remove",startSpotlightMenuLabel:"Spotlight for everyone",addSpotlightMenuLabel:"Add spotlight",stopSpotlightMenuLabel:"Stop spotlighting",stopSpotlightOnSelfMenuLabel:"Exit spotlight",spotlightLimitReachedMenuTitle:"Spotlight limit reached",stopAllSpotlightMenuLabel:"Stop all spotlights",returnToCallButtonAriaDescription:"Return to Call",returnToCallButtonAriaLabel:"Back",roomNotFoundDetails:"The room ID provided could not be found.",roomNotFoundTitle:"Room not found",roomNotValidDetails:"This room is not valid at this current time.",roomNotValidTitle:"Room not valid",inviteToRoomRemovedDetails:"Your invite to join this room was removed.",inviteToRoomRemovedTitle:"Invite to room removed",soundLabel:"Sound",noMicrophonesLabel:"No microphones found",noSpeakersLabel:"No speakers found",startCallButtonLabel:"Start call",openDialpadButtonLabel:"Dial phone number",peoplePaneAddPeopleButtonLabel:"Add People",dialpadStartCallButtonLabel:"Call",dialpadModalTitle:"Dial Phone Number",dialpadModalAriaLabel:"Dialpad",dialpadCloseModalButtonAriaLabel:"Close dialpad",moreButtonCallingLabel:"More",resumeCallButtonLabel:"Resume",resumingCallButtonLabel:"Resuming...",resumeCallButtonAriaLabel:"Resume call",resumingCallButtonAriaLabel:"Resume call",holdScreenLabel:"You're on hold",openDtmfDialpadLabel:"Show dialpad",dtmfDialpadPlaceholderText:"Enter number",outboundCallingNoticeString:"Calling...",participantJoinedNoticeString:"{displayName} joined",twoParticipantJoinedNoticeString:"{displayName1} and {displayName2} have joined",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} and {displayName3} have joined",participantLeftNoticeString:"{displayName} left",twoParticipantLeftNoticeString:"{displayName1} and {displayName2} have left",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} and {displayName3} have left",unnamedParticipantString:"unnamed participant",manyUnnamedParticipantsJoined:"unnamed participant and {numOfParticipants} other participants joined",manyUnnamedParticipantsLeft:"unnamed participant and {numOfParticipants} other participants left",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} and {numOfParticipants} other participants joined",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} and {numOfParticipants} other participants left",liveCaptionsLabel:"Live captions",captionsSettingsLabel:"Caption settings",startCaptionsButtonOnLabel:"Turn on captions",startCaptionsButtonOffLabel:"Turn off captions",startCaptionsButtonTooltipOnContent:"Turn off captions",startCaptionsButtonTooltipOffContent:"Turn on captions",captionsSettingsModalTitle:"Captions Settings",captionsSettingsSpokenLanguageDropdownLabel:"Spoken language",captionsSettingsCaptionLanguageDropdownLabel:"Captions language",captionsSettingsSpokenLanguageDropdownInfoText:"Language that everyone on this call is speaking.",captionsSettingsCaptionLanguageDropdownInfoText:"Captions will appear in this language.",captionsSettingsConfirmButtonLabel:"Confirm",captionsSettingsCancelButtonLabel:"Cancel",captionsSettingsModalAriaLabel:"Captions Setting Modal",captionsSettingsCloseModalButtonAriaLabel:"Close Captions Setting",captionsBannerMoreButtonCallingLabel:"Captions Settings More Options",captionsBannerMoreButtonTooltip:"More options",realTimeTextLabel:"Real-time text (RTT)",startRealTimeTextLabel:"Turn on RTT for this call",disabledStartRealTimeTextLabel:"RTT is currently turned on, it cannot be turned off",dismissModalAriaLabel:"Local and remote picture and picture, press enter to return to call",callRejectedTitle:"The call could not be completed",callRejectedMoreDetails:"The person you are trying to reach is unavailable. Please try again later.",callTimeoutTitle:"The call could not be completed",callTimeoutDetails:"The person you are trying to reach is unavailable. Please try again later.",callTimeoutBotTitle:"The call could not be completed",callTimeoutBotDetails:"The service that you are trying to reach is unavailable. Please try again later.",dtmfDialerButtonLabel:"Dialpad",dtmfDialerButtonTooltipOn:"Show dialpad",dtmfDialerButtonTooltipOff:"Hide dialpad",dtmfDialerMoreButtonLabelOn:"Show dialpad",dtmfDialerMoreButtonLabelOff:"Hide dialpad",phoneCallMoreButtonLabel:"Phone Call",spokenLanguageStrings:{"ar-ae":"Arabic - U.A.E.","ar-sa":"Arabic - Saudi Arabia","da-dk":"Danish","de-de":"German - Germany","en-au":"English - Australia","en-ca":"English - Canada","en-gb":"English - United Kingdom","en-in":"English - India","en-nz":"English - New Zealand","en-us":"English - United States","es-es":"Spanish - Spain (Modern Sort)","es-mx":"Spanish - Mexico","fi-fi":"Finnish","fr-ca":"French - Canada","fr-fr":"French - France","hi-in":"Hindi","it-it":"Italian - Italy","ja-jp":"Japanese","ko-kr":"Korean","nb-no":"Norwegian (Bokmål)","nl-be":"Dutch - Belgium","nl-nl":"Dutch - Netherlands","pl-pl":"Polish","pt-br":"Portuguese - Brazil","ru-ru":"Russian","sv-se":"Swedish","zh-cn":"Chinese - People's Republic of China","zh-hk":"Chinese - Hong Kong SAR","cs-cz":"Czech","pt-pt":"Portuguese - Portugal","tr-tr":"Turkish","vi-vn":"Vietnamese","th-th":"Thai","he-il":"Hebrew","cy-gb":"Welsh","uk-ua":"Ukrainian","el-gr":"Greek","hu-hu":"Hungarian","ro-ro":"Romanian","sk-sk":"Slovak","zh-tw":"Chinese - Taiwan"},captionLanguageStrings:{ar:"Arabic",da:"Danish",de:"German",en:"English",es:"Spanish",fi:"Finnish","fr-ca":"French - Canada",fr:"French - France",hi:"Hindi",it:"Italian",ja:"Japanese",ko:"Korean",nb:"Norwegian (Bokmål)",nl:"Dutch",pl:"Polish",pt:"Portuguese - Brazil",ru:"Russian",sv:"Swedish","zh-Hans":"Chinese (Simplified)","zh-Hant":"Chinese (Traditional)",cs:"Czech","pt-pt":"Portuguese - Portugal",tr:"Turkish",vi:"Vietnamese",th:"Thai",he:"Hebrew",cy:"Welsh",uk:"Ukrainian",el:"Greek",hu:"Hungarian",ro:"Romanian",sk:"Slovak"},captionsBannerSpinnerText:"Starting captions...",realTimeTextInputBoxDefaultText:"Type message in real-time",realTimeTextInputErrorMessage:"Message cannot exceed 2000 characters",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (real-time text) is enabled for all participants for the entire duration of the meeting.",realTimeTextBannerLinkLabel:"Learn more",realTimeTextModalAriaLabel:"Turn on RTT Modal",realTimeTextModalTitle:"Turn on RTT?",realTimeTextModalText:"Real-time text (RTT) will be turned on for everyone in the meeting.",realTimeTextConfirmButtonLabel:"Turn on",realTimeTextCancelButtonLabel:"Cancel",realTimeTextCloseModalButtonAriaLabel:"Close RTT Modal",captionsOnlyContainerTitle:"Captions",realTimeTextOnlyContainerTitle:"Real-Time Text",captionsAndRealTimeTextContainerTitle:"Captions and Real-Time Text",expandButtonAriaLabel:"Expand",minimizeButtonAriaLabel:"Collapse",transferPageTransferorText:"Transferring...",transferPageTransferTargetText:"Connecting...",transferPageUnknownTransferorDisplayName:"Unknown",transferPageUnknownTransferTargetDisplayName:"Unknown",transferPageNoticeString:"You are being transferred",participantCouldNotBeReachedTitle:"Target participant is currently not available",participantCouldNotBeReachedMoreDetails:"Please contact participant when they are available",permissionToReachTargetParticipantNotAllowedTitle:"Permission to reach target participant is not allowed",permissionToReachTargetParticipantNotAllowedMoreDetails:"Please check that the target participant is in the same tenant",unableToResolveTenantTitle:"Unable to resolve tenant id for the target participant",unableToResolveTenantMoreDetails:"Please check the participant id is entered correctly",participantIdIsMalformedTitle:"Participant id is not in the correct format",participantIdIsMalformedMoreDetails:"Please check that the participant id is in the correct format",moreButtonGalleryControlLabel:"View",moreButtonGalleryPositionToggleLabel:"Move gallery to top",moreButtonGallerySpeakerLayoutLabel:"Speaker",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamic",moreButtonGalleryDefaultLayoutLabel:"Gallery view",moreButtonGalleryFocusedContentLayoutLabel:"Focus on content",moreButtonLargeGalleryDefaultLayoutLabel:"Large Gallery",moreButtonTogetherModeLayoutLabel:"Together mode",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Your camera has been disabled. You can no longer share video.",grantedDueToMeetingOption:"Your camera has been enabled. Turn it on if you'd like to share video."},unmuteMic:{lostDueToMeetingOption:"Your mic has been disabled. You can no longer unmute.",grantedDueToMeetingOption:"Your mic has been enabled. Unmute if you'd like to speak."},shareScreen:{lostDueToRoleChangeToAttendee:"Your role has been changed. Some actions, like sharing content, won't be available to you.",grantedDueToRoleChangeToPresenter:"You're a presenter. You can share content and facilitate the meeting."},hideAttendeeNames:{hideAttendeeNameAttendee:"Your name is hidden from other attendees. Organisers and presenters can see your real name.",hideAttendeeNamePresenter:"Attendee names are hidden. Only organisers and presenters can see names to protect attendee privacy."}},surveyTitle:"Help us improve",starSurveyHelperText:"How was the quality of the call?",starSurveyOneStarText:"The quality was bad.",starSurveyTwoStarText:"The quality was poor.",starSurveyThreeStarText:"The quality was good.",starSurveyFourStarText:"The quality was great.",starSurveyFiveStarText:"The quality was excellent.",surveyConfirmButtonLabel:"Send Feedback",starRatingAriaLabel:"Select {0} of {1} stars",surveySkipButtonLabel:"Skip",tagsSurveyQuestion:"What could have been better?",tagsSurveyTextFieldDefaultText:"Other, please specify",tagsSurveyHelperText:"Check any issues you experienced",endOfSurveyText:"Thank you for your feedback!",surveyIssues:{overallRating:{callCannotJoin:"I could not join call",callCannotInvite:"I could not invite others into the call",hadToRejoin:"I had to rejoin the call",callEndedUnexpectedly:"Call ended for me unexpectedly",otherIssues:"I was having other issues with the call"},audioRating:{noLocalAudio:"The other side could not hear any sound",noRemoteAudio:"I could not hear any sound",echo:"I heard echos on the call",audioNoise:"I heard noise on the call",lowVolume:"Volume was low",audioStoppedUnexpectedly:"Audio stopped unexpectedly",distortedSpeech:"Audio was distorted",audioInterruption:"Audio was interrupted",otherIssues:"I was having other audio issues in this call"},videoRating:{noVideoReceived:"I could not see any video",noVideoSent:"Others could not see me",lowQuality:"Video quality was low",freezes:"Video frozen",stoppedUnexpectedly:"Video stopped unexpectedly",darkVideoReceived:"I can only see dark screens when others turn on their camera",audioVideoOutOfSync:"Audio and Video was out of sync",otherIssues:"I was having other video issues in this call"},screenshareRating:{noContentLocal:"Other people could not see my screenshare",noContentRemote:"I could not see other people's screenshare",cannotPresent:"I could not present my screen",lowQuality:"Screen share quality was low",freezes:"Screen share frozen",stoppedUnexpectedly:"Screen share stopped unexpectedly",largeDelay:"Screen share has a large delay",otherIssues:"I was having other screen share issues in this call"}},surveyIssuesHeadingStrings:{overallRating:"Overall",audioRating:"Audio",videoRating:"Video",screenshareRating:"Presenting"},spotlightPrompt:{startSpotlightHeading:"Spotlight for everyone?",startSpotlightText:"You'll highlight this video for everyone in the meeting.",startSpotlightOnSelfText:"You'll highlight your video for everyone in the meeting.",startSpotlightConfirmButtonLabel:"Spotlight for everyone",startSpotlightCancelButtonLabel:"Cancel",stopSpotlightHeading:"Stop spotlighting this video?",stopSpotlightOnSelfHeading:"Exit spotlight?",stopAllSpotlightHeading:"Stop spotlighting all videos?",stopSpotlightText:"This video will no longer be highlighted for everyone in the meeting.",stopSpotlightOnSelfText:"Your video will no longer be highlighted for everyone in the meeting.",stopAllSpotlightText:"The videos will no longer be highlighted for everyone in the meeting.",stopSpotlightConfirmButtonLabel:"Stop spotlighting",stopSpotlightOnSelfConfirmButtonLabel:"Exit spotlight",stopSpotlightCancelButtonLabel:"Cancel",closeSpotlightPromptButtonLabel:"Close"},exitSpotlightButtonLabel:"Exit spotlight",exitSpotlightButtonTooltip:"Exit spotlight",leaveConfirmButtonLabel:"Leave",endCallConfirmButtonLabel:"End call",hangUpCancelButtonLabel:"Cancel",leaveConfirmDialogTitle:"Leave the call?",leaveConfirmDialogContent:"Are you sure you want to leave the call?",endCallConfirmDialogTitle:"End the call?",endCallConfirmDialogContent:"You'll end the call for everyone.",invalidMeetingIdentifier:"Invalid meeting ID or passcode",pinParticipantMenuLabel:"Pin for me",pinParticipantLimitReachedMenuLabel:"Pin (limit reached)",unpinParticipantMenuLabel:"Unpin",pinParticipantMenuItemAriaLabel:"Pin for me",unpinParticipantMenuItemAriaLabel:"Unpin",muteAllMenuLabel:"Mute all",muteAllDialogTitle:"Mute everyone?",muteAllDialogContent:"This will mute everyone in the meeting but you.",muteAllConfirmButtonLabel:"Mute all",muteAllCancelButtonLabel:"Cancel",joinBreakoutRoomButtonLabel:"Join breakout room",returnFromBreakoutRoomButtonLabel:"Leave room",leaveBreakoutRoomAndMeetingButtonLabel:"Leave meeting",breakoutRoomJoinedNotificationTitle:"You've joined Breakout room '{roomName}'",joinBreakoutRoomBannerTitle:"Breakout room '{roomName}' is open",joinBreakoutRoomBannerButtonLabel:"Join",returnFromBreakoutRoomBannerTitle:"Return to main meeting",returnFromBreakoutRoomBannerButtonLabel:"Rejoin",forbidAudioMenuLabel:"Disable mic",permitAudioMenuLabel:"Allow mic",forbidOthersAudioDialogTitle:"Disable mic for attendees?",forbidOthersAudioDialogContent:"Attendees won’t be able to unmute themselves. The organiser and presenters can let people unmute as needed.",forbidOthersAudioConfirmButtonLabel:"Disable mics",forbidOthersAudioCancelButtonLabel:"Cancel",permitOthersAudioDialogTitle:"Allow mic for attendees?",permitOthersAudioDialogContent:"Everyone in the meeting will be able to unmute themselves.",permitOthersAudioConfirmButtonLabel:"Allow mics",permitOthersAudioCancelButtonLabel:"Cancel",forbidOthersAudioMenuLabel:"Disable mic for attendees",permitOthersAudioMenuLabel:"Allow mic for attendees",forbidVideoMenuLabel:"Disable camera",permitVideoMenuLabel:"Allow camera",forbidOthersVideoDialogTitle:"Disable camera for attendees?",forbidOthersVideoDialogContent:"Attendees won’t be able to turn on video themselves. The organiser and presenters can let people turn on camera as needed.",forbidOthersVideoConfirmButtonLabel:"Disable cameras",forbidOthersVideoCancelButtonLabel:"Cancel",permitOthersVideoDialogTitle:"Allow camera for attendees?",permitOthersVideoDialogContent:"Everyone in the meeting will be able to turn on video themselves.",permitOthersVideoConfirmButtonLabel:"Allow cameras",permitOthersVideoCancelButtonLabel:"Cancel",forbidOthersVideoMenuLabel:"Disable camera for attendees",permitOthersVideoMenuLabel:"Allow camera for attendees"};var chat$m={chatListHeader:"In this chat",uploadAttachment:"Upload Attachment",uploadImageDataNotProvided:"Image data is not provided.",uploadImageIsTooLarge:"Image is too large. Choose one that's less than {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Uploading .{imageExtension} image is not allowed.",uploadImageFailed:"Unable to upload image. Please try again later."};var callWithChat$m={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"New Message",chatButtonTooltipClosedWithMessageCount:"Show chat ({unreadMessagesCount} unread)",chatButtonTooltipClose:"Hide chat",chatButtonTooltipOpen:"Show chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Copy invite link",copyInviteLinkButtonActionedLabel:"Link copied",copyInviteLinkActionedAriaLabel:"Invite link copied",dismissSidePaneButtonLabel:"Close",moreDrawerAudioDeviceMenuTitle:"Audio Device",moreDrawerButtonLabel:"More options",moreDrawerButtonTooltip:"More options",moreDrawerMicrophoneMenuTitle:"Microphone",moreDrawerSpeakerMenuTitle:"Speaker",moreDrawerCaptionsMenuTitle:"Live captions",moreDrawerSpokenLanguageMenuTitle:"Spoken language",moreDrawerCaptionLanguageMenuTitle:"Caption language",moreDrawerGalleryOptionsMenuTitle:"Gallery options",peopleButtonLabel:"People",selectedPeopleButtonLabel:"People Button Selected",peopleButtonTooltipOpen:"Show participants",peopleButtonTooltipClose:"Hide participants",peopleButtonTooltipOpenAriaLabel:"People button, Show participants",peopleButtonTooltipCloseAriaLabel:"People button, Hide participants",peoplePaneSubTitle:"In this call",peoplePaneTitle:"People",pictureInPictureTileAriaLabel:"Video Feeds. Click to return to call screen.",removeMenuLabel:"Remove",openDialpadButtonLabel:"Dial phone number",returnToCallButtonAriaDescription:"Return to Call",returnToCallButtonAriaLabel:"Back",peoplePaneAddPeopleButtonLabel:"Add People",dialpadStartCallButtonLabel:"Call",dialpadModalTitle:"Dial Phone Number",dialpadModalAriaLabel:"Dialpad",dialpadCloseModalButtonAriaLabel:"Close dialpad",openDtmfDialpadLabel:"Show dialpad",dtmfDialpadPlaceholderText:"Enter number",breakoutRoomChatPaneTitle:"Breakout Room Chat",chatContentSpinnerLabel:"Loading chat"};var en_GB = {call:call$m,chat:chat$m,callWithChat:callWithChat$m};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 *
 * @private
 */
const createCompositeStrings = (localizedStrings) => {
    const strings = Object.assign({}, en_US);
    _getKeys(localizedStrings).forEach(key => {
        // mark the value as unknown because the type changes based on the key.
        // this is unsafe at runtime as we could assign the wrong type based on the key here.
        // but typescript isn't smart enough to know that the key used across each access will result in the same type
        strings[key] = Object.assign(Object.assign({}, strings[key]), localizedStrings[key]);
    });
    return strings;
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for English (British)
 *
 * @public
 */
const COMPOSITE_LOCALE_EN_GB = {
    component: COMPONENT_LOCALE_EN_GB,
    strings: createCompositeStrings(en_GB)
};

var call$l={cameraLabel:"الكاميرا",noCamerasLabel:"لا توجد أي كاميرات",cameraPermissionDenied:"يقوم المستعرض بحظر الوصول إلى الكاميرا",cameraTurnedOff:"تم إيقاف تشغيل الكاميرا",chatButtonLabel:"دردشة",close:"إغلاق",complianceBannerNowOnlyRecording:"أنت الآن تسجل هذا الاجتماع فقط.",complianceBannerNowOnlyTranscription:"أنت الآن تقوم فقط بكتابة الحديث لهذا الاجتماع.",complianceBannerRecordingAndTranscriptionSaved:"يتم حفظ التسجيل وكتابة الحديث.",complianceBannerRecordingAndTranscriptionStarted:"بدء التسجيل والنسخ. ",complianceBannerRecordingAndTranscriptionStopped:"التسجيل وكتابة الحديث متوقفان.",complianceBannerRecordingSaving:"يتم حفظ التسجيل.",complianceBannerRecordingStarted:"تم بدء التسجيل.",complianceBannerRecordingStopped:"تم إيقاف التسجيل.",complianceBannerTranscriptionStarted:"كتابة الحديث مبدوءة.",complianceBannerTranscriptionConsent:"من خلال الانضمام، فأنت توافق على كتابة هذا الاجتماع.",complianceBannerTranscriptionSaving:"يتم حفظ كتابة الحديث.",complianceBannerTranscriptionStopped:"توقفت كتابة الحديث.",configurationPageTitle:"بدء مكالمة",copyInviteLinkButtonLabel:"نسخ ارتباط دعوة",copyInviteLinkButtonActionedLabel:"تم نسخ الارتباط",copyInviteLinkActionedAriaLabel:"تم نسخ ارتباط الدعوة",defaultPlaceHolder:"تحديد خيار",dismissSidePaneButtonLabel:"إغلاق",videoEffectsPaneTitle:"التأثيرات",videoEffectsPaneBackgroundSelectionTitle:"الخلفية",videoEffectsPaneAriaLabel:"جزء تأثيرات الفيديو",configurationPageCameraIsLoadingLabel:"يتم الآن تحميل الفيديو...",configurationPageVideoEffectsButtonLabel:"التأثيرات",unableToStartVideoEffect:"يتعذر تطبيق تأثير الفيديو.",blurBackgroundEffectButtonLabel:"تمويه",blurBackgroundTooltip:"تمويه الخلفية",removeBackgroundEffectButtonLabel:"لا شيء",removeBackgroundTooltip:"إزالة الخلفية",cameraOffBackgroundEffectWarningText:"الكاميرا قيد الإيقاف. قم بتشغيل الكاميرا لمشاهدة تأثير الفيديو.",failedToJoinCallDueToNoNetworkMoreDetails:"تم قطع الاتصال بسبب مشكلة في الشبكة. تحقق من اتصالك وانضم مرة أخرى.",failedToJoinCallDueToNoNetworkTitle:"‏‏تم قطع اتصال المكالمة",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"لم يتم منحك إدخالا في المكالمة. إذا كان هذا خطأ، فعاود الانضمام إلى المكالمة.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"تم التجاهل من ساحة الانتظار",learnMore:"تعرّف على المزيد",leavingCallTitle:"تتم الآن المغادرة...",leftCallMoreDetails:"إذا كان هذا خطأ، فعاود الانضمام إلى المكالمة.",leftCallTitle:"قمت بمغادرة المكالمة",lobbyScreenConnectingToCallTitle:"جارٍ الانضمام إلى مكالمة",lobbyScreenWaitingToBeAdmittedTitle:"في انتظار السماح",microphonePermissionDenied:"يقوم المستعرض بحظر الوصول إلى الميكروفون",microphoneToggleInLobbyNotAllowed:"لا يمكن كتم الصوت أو إلغاء كتمه أثناء وجودك في ساحة الانتظار.",mutedMessage:"تم كتم صوتك",networkReconnectMoreDetails:"يبدو أنه قد حدث خطأ ما. نحاول معاودة الاتصال.",networkReconnectTitle:"انتظار",notInvitedToRoomDetails:"لا يمكنك الانضمام إلى هذه الغرفة لأنه ليس لديك دعوة.",notInvitedToRoomTitle:"غير مدعو إلى الغرفة",peopleButtonLabel:"الأشخاص",selectedPeopleButtonLabel:"تم تحديد زر الأشخاص",peoplePaneTitle:"الأشخاص",peoplePaneMoreButtonAriaLabel:"المزيد",peopleButtonTooltipOpen:"إظهار المشاركين",peopleButtonTooltipClose:"إخفاء المشاركين",peoplePaneSubTitle:"في هذه المكالمة {numberOfPeople}",privacyPolicy:"نهج الخصوصية",rejoinCallButtonLabel:"الانضمام إلى المكالمة",removedFromCallMoreDetails:"قام مشارك آخر بإزالتك من المكالمة.",removedFromCallTitle:"تمت إزالتك",removeMenuLabel:"إزالة",startSpotlightMenuLabel:"تسليط الضوء لدى الجميع",addSpotlightMenuLabel:"إضافة تسليط ضوء",stopSpotlightMenuLabel:"التوقف عن تسليط الضوء",stopSpotlightOnSelfMenuLabel:"الخروج من وضع تسليط الضوء",spotlightLimitReachedMenuTitle:"تم الوصول إلى الحد الأقصى لتسليط الضوء",stopAllSpotlightMenuLabel:"إيقاف جميع عمليات تسليط الضوء",returnToCallButtonAriaDescription:"رجوع إلى المكالمة",returnToCallButtonAriaLabel:"رجوع",roomNotFoundDetails:"تعذر العثور على معرف الغرفة الذي تم توفيره.",roomNotFoundTitle:"لم يتم العثور على الغرفة",roomNotValidDetails:"هذه الغرفة غير صالحة في الوقت الحالي.",roomNotValidTitle:"الغرفة غير صالحة",inviteToRoomRemovedDetails:"تمت إزالة دعوتك للانضمام إلى هذه الغرفة.",inviteToRoomRemovedTitle:"تمت إزالة الدعوة إلى الغرفة",soundLabel:"صوت",noMicrophonesLabel:"لم يتم العثور على ميكروفونات",noSpeakersLabel:"لم يتم العثور على سماعات",startCallButtonLabel:"بدء مكالمة",openDialpadButtonLabel:"اطلب رقم الهاتف",peoplePaneAddPeopleButtonLabel:"إضافة أشخاص",dialpadStartCallButtonLabel:"مكالمة",dialpadModalTitle:"اطلب رقم الهاتف",dialpadModalAriaLabel:"لوحة الطلب",dialpadCloseModalButtonAriaLabel:"إغلاق لوحة الطلب",moreButtonCallingLabel:"المزيد",resumeCallButtonLabel:"استئناف",resumingCallButtonLabel:"يتم الآن الاستئناف...",resumeCallButtonAriaLabel:"استئناف المكالمة",resumingCallButtonAriaLabel:"استئناف المكالمة",holdScreenLabel:"أنت قيد الانتظار",openDtmfDialpadLabel:"عرض لوحة الطلب",dtmfDialpadPlaceholderText:"أدخل عدداً",outboundCallingNoticeString:"يتم الآن الاتصال...",participantJoinedNoticeString:"انضم {displayName}",twoParticipantJoinedNoticeString:"انضم {displayName1} و{displayName2}",threeParticipantJoinedNoticeString:"انضم {displayName1} و{displayName2} و{displayName3}",participantLeftNoticeString:"غادر {displayName}",twoParticipantLeftNoticeString:"غادر {displayName1} و{displayName2}",threeParticipantLeftNoticeString:"غادر {displayName1} و{displayName2} و{displayName3}",unnamedParticipantString:"مشارك غير مسمى",manyUnnamedParticipantsJoined:"انضم مشارك غير مسمى و{numOfParticipants} من المشاركين الآخرين",manyUnnamedParticipantsLeft:"غادر مشارك غير مسمى و{numOfParticipants} من المشاركين الآخرين",manyParticipantsJoined:"انضم {displayName1} {displayName2} {displayName3} {numOfParticipants} مشاركين آخرين",manyParticipantsLeft:"غادر {displayName1} {displayName2} {displayName3} والمشاركون الآخرون {numOfParticipants}",liveCaptionsLabel:"نص الكلام مباشرةً",captionsSettingsLabel:"إعدادات نص الكلام",startCaptionsButtonOnLabel:"تمكين نص الكلام",startCaptionsButtonOffLabel:"تعطيل نص الكلام",startCaptionsButtonTooltipOnContent:"تعطيل نص الكلام",startCaptionsButtonTooltipOffContent:"تمكين نص الكلام",captionsSettingsModalTitle:"إعدادات التسميات التوضيحية",captionsSettingsSpokenLanguageDropdownLabel:"اللغة المنطوقة",captionsSettingsCaptionLanguageDropdownLabel:"لغة التسميات التوضيحية",captionsSettingsSpokenLanguageDropdownInfoText:"اللغة التي يتحدث بها كل شخص في هذه المكالمة.",captionsSettingsCaptionLanguageDropdownInfoText:"سيظهر نص الكلام بهذه اللغة.",captionsSettingsConfirmButtonLabel:"تأكيد",captionsSettingsCancelButtonLabel:"إلغاء",captionsSettingsModalAriaLabel:"مربع حوار إعدادات نص الكلام المشروط",captionsSettingsCloseModalButtonAriaLabel:"إغلاق إعداد نص الكلام",captionsBannerMoreButtonCallingLabel:"خيارات إضافية لإعدادات التسميات التوضيحية",captionsBannerMoreButtonTooltip:"خيارات إضافية",realTimeTextLabel:"عرض الكتابة في الوقت الحقيقي (RTT)",startRealTimeTextLabel:"تشغيل RTT لهذه المكالمة",disabledStartRealTimeTextLabel:"RTT قيد التشغيل حاليا، ولا يمكن إيقاف تشغيله",dismissModalAriaLabel:"صورة وصورة محلية وبعيدة، اضغط على مفتاح الإدخال للعودة إلى المكالمة",callRejectedTitle:"إكمال المكالمة غير ممكن",callRejectedMoreDetails:"الشخص الذي تحاول الوصول إليه غير متوفر. الرجاء المحاولة مرة أخرى لاحقا.",callTimeoutTitle:"إكمال المكالمة غير ممكن",callTimeoutDetails:"الشخص الذي تحاول الوصول إليه غير متوفر. الرجاء المحاولة مرة أخرى لاحقا.",callTimeoutBotTitle:"إكمال المكالمة غير ممكن",callTimeoutBotDetails:"الخدمة التي تحاول الوصول إليها غير متوفرة. الرجاء المحاولة مرة أخرى لاحقا.",dtmfDialerButtonLabel:"لوحة الطلب",dtmfDialerButtonTooltipOn:"عرض لوحة الطلب",dtmfDialerButtonTooltipOff:"إخفاء لوحة الطلب",dtmfDialerMoreButtonLabelOn:"عرض لوحة الطلب",dtmfDialerMoreButtonLabelOff:"إخفاء لوحة الطلب",phoneCallMoreButtonLabel:"مكالمة هاتفية",spokenLanguageStrings:{"ar-ae":"العربية - الإمارات العربية المتحدة","ar-sa":"العربية - السعودية","da-dk":"الدانمركية","de-de":"الألمانية - ألمانيا","en-au":"الإنجليزية - أستراليا","en-ca":"الإنجليزية - كندا","en-gb":"الإنجليزية - المملكة المتحدة","en-in":"الإنجليزية - الهند","en-nz":"الإنجليزية - نيوزيلندا","en-us":"الإنجليزية - الولايات المتحدة","es-es":"الإسبانية - إسبانيا (فرز حديث)","es-mx":"الإسبانية - المكسيك","fi-fi":"الفنلندية","fr-ca":"الفرنسية - كندا","fr-fr":"الفرنسية - فرنسا","hi-in":"الهندية","it-it":"الإيطالية - إيطاليا","ja-jp":"اليابانية","ko-kr":"الكورية","nb-no":"النرويجية (بوكمال)","nl-be":"الهولندية - بلجيكا","nl-nl":"الهولندية - هولندا","pl-pl":"البولندية","pt-br":"البرتغالية - البرازيل","ru-ru":"الروسية","sv-se":"السويدية","zh-cn":"الصينية - جمهورية الصين الشعبية","zh-hk":"الصينية - منطقة هونغ كونغ الإدارية الخاصة","cs-cz":"التشيكية","pt-pt":"البرتغالية - البرتغال","tr-tr":"التركية","vi-vn":"الفيتنامية","th-th":"التايلاندية","he-il":"العبرية","cy-gb":"الويلزية","uk-ua":"الأوكرانية","el-gr":"اليونانية","hu-hu":"الهنغارية","ro-ro":"الرومانية","sk-sk":"السلوفاكية","zh-tw":"الصينية - تايوان"},captionLanguageStrings:{ar:"العربية",da:"الدانمركية",de:"الألمانية",en:"الإنجليزية",es:"الإسبانية",fi:"الفنلندية","fr-ca":"الفرنسية - كندا",fr:"الفرنسية - فرنسا",hi:"الهندية",it:"الإيطالية",ja:"اليابانية",ko:"الكورية",nb:"النرويجية (بوكمال)",nl:"الهولندية",pl:"البولندية",pt:"البرتغالية - البرازيل",ru:"الروسية",sv:"السويدية","zh-Hans":"الصينية (المبسطة)","zh-Hant":"الصينية (التقليدية)",cs:"التشيكية","pt-pt":"البرتغالية - البرتغال",tr:"التركية",vi:"الفيتنامية",th:"التايلاندية",he:"العبرية",cy:"الويلزية",uk:"الأوكرانية",el:"اليونانية",hu:"الهنغارية",ro:"الرومانية",sk:"السلوفاكية"},captionsBannerSpinnerText:"يتم الآن بدء التسميات التوضيحية...",realTimeTextInputBoxDefaultText:"اكتب رسالة في الوقت الحقيقي",realTimeTextInputErrorMessage:"لا يمكن أن تتجاوز الرسالة 2000 حرف",realTimeTextBannerTitle:"عرض الكتابة في الوقت الحقيقي (RTT)",realTimeTextBannerContent:"تم تمكين RTT (نص الوقت الحقيقي) لجميع المشاركين طوال مدة الاجتماع.",realTimeTextBannerLinkLabel:"تعرّف على المزيد",realTimeTextModalAriaLabel:"تشغيل مربع حوار عرض الكتابة في الوقت الحقيقي (RTT) المشروط",realTimeTextModalTitle:"هل تريد تشغيل عرض الكتابة في الوقت الحقيقي؟",realTimeTextModalText:"سيتم تشغيل النص في الوقت الحقيقي (RTT) لكل شخص في الاجتماع.",realTimeTextConfirmButtonLabel:"تشغيل",realTimeTextCancelButtonLabel:"إلغاء الأمر",realTimeTextCloseModalButtonAriaLabel:"إغلاق مربع حوار عرض الكتابة في الوقت الحقيقي (RTT) المشروط",captionsOnlyContainerTitle:"نص الكلام",realTimeTextOnlyContainerTitle:"نص حقيقي",captionsAndRealTimeTextContainerTitle:"التسميات التوضيحية والنص في الوقت الحقيقي",expandButtonAriaLabel:"توسيع",minimizeButtonAriaLabel:"طي",transferPageTransferorText:"يتم الآن التحويل...",transferPageTransferTargetText:"جارٍ الاتصال...",transferPageUnknownTransferorDisplayName:"غير معروف",transferPageUnknownTransferTargetDisplayName:"غير معروف",transferPageNoticeString:"يجري نقلك",participantCouldNotBeReachedTitle:"المشارك المستهدف غير متوفر حاليا",participantCouldNotBeReachedMoreDetails:"الرجاء الاتصال بالمشارك عندما يكون متوفرا",permissionToReachTargetParticipantNotAllowedTitle:"غير مسموح بإذن الوصول إلى المشارك الهدف",permissionToReachTargetParticipantNotAllowedMoreDetails:"الرجاء التحقق من أن المشارك الهدف موجود في نفس المستأجر",unableToResolveTenantTitle:"تعذر حل معرف المستأجر للمشارك الهدف",unableToResolveTenantMoreDetails:"الرجاء التحقق من إدخال معرف المشارك بشكل صحيح",participantIdIsMalformedTitle:"تنسيق معرف المشارك غير صحيح",participantIdIsMalformedMoreDetails:"الرجاء التحقق من أن معرف المشارك بالتنسيق الصحيح",moreButtonGalleryControlLabel:"عرض",moreButtonGalleryPositionToggleLabel:"نقل المعرض إلى الأعلى",moreButtonGallerySpeakerLayoutLabel:"مكبر الصوت",moreButtonGalleryFloatingLocalLayoutLabel:"ديناميكي",moreButtonGalleryDefaultLayoutLabel:"طريقة عرض المعرض",moreButtonGalleryFocusedContentLayoutLabel:"التركيز على المحتوى",moreButtonLargeGalleryDefaultLayoutLabel:"معرض كبير",moreButtonTogetherModeLayoutLabel:"وضع التجميع",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"تم تعطيل الكاميرا الخاصة بك. لم يعد بإمكانك مشاركة الفيديو.",grantedDueToMeetingOption:"تم تمكين الكاميرا الخاصة بك. قم بتشغيله إذا كنت ترغب في مشاركة الفيديو."},unmuteMic:{lostDueToMeetingOption:"لقد تم تعطيل الميكروفون الخاص بك. لم يعد بإمكانك إلغاء كتم الصوت.",grantedDueToMeetingOption:"تم تمكين الميكروفون لديك. قم بإلغاء كتم الصوت إذا كنت ترغب في التحدث."},shareScreen:{lostDueToRoleChangeToAttendee:"تم تغيير دورك. لن تتوفر بعض الإجراءات، مثل مشاركة المحتوى، لك.",grantedDueToRoleChangeToPresenter:"أنت مقدم العرض. يمكنك مشاركة المحتوى وتسهيل الاجتماع."},hideAttendeeNames:{hideAttendeeNameAttendee:"اسمك مخفي عن الحضور الآخرين. يمكن للمنظمين والمقدمين رؤية اسمك الحقيقي.",hideAttendeeNamePresenter:"أسماء الحضور مخفية. يمكن للمنظمين والمقدمين فقط رؤية الأسماء لحماية خصوصية الحضور."}},surveyTitle:"ساعدنا في تحسين خدماتنا",starSurveyHelperText:"كيف كانت جودة المكالمة؟",starSurveyOneStarText:"كانت الجودة سيئة.",starSurveyTwoStarText:"كانت الجودة رديئة.",starSurveyThreeStarText:"كانت الجودة جيدة.",starSurveyFourStarText:"كانت الجودة رائعة.",starSurveyFiveStarText:"كانت الجودة ممتازة.",surveyConfirmButtonLabel:"إرسال ملاحظات",starRatingAriaLabel:"حدد {0} من {1} نجوم",surveySkipButtonLabel:"تخطٍ",tagsSurveyQuestion:"ما الذي كان يمكن أن يكون أفضل؟",tagsSurveyTextFieldDefaultText:"أخرى، يُرجى التحديد",tagsSurveyHelperText:"تحقق من أي مشاكل واجهتك",endOfSurveyText:"شكراً لك على تقديم ملاحظاتك!",surveyIssues:{overallRating:{callCannotJoin:"تعذر علي الانضمام إلى المكالمة",callCannotInvite:"لم أتمكن من دعوة الآخرين إلى المكالمة",hadToRejoin:"اضطررت إلى إعادة الانضمام إلى المكالمة",callEndedUnexpectedly:"انتهت المكالمة عندي بشكل غير متوقع",otherIssues:"كنت أواجه مشكلات أخرى في المكالمة"},audioRating:{noLocalAudio:"لم تسمع الجهة الأخرى أي صوت",noRemoteAudio:"لم أسمع أي صوت",echo:"سمعت صدى في المكالمة",audioNoise:"سمعت ضوضاء في المكالمة",lowVolume:"كان مستوى الصوت منخفضاً",audioStoppedUnexpectedly:"تم إيقاف الصوت بشكل غير متوقع",distortedSpeech:"تم تشويه الصوت",audioInterruption:"تمت مقاطعة الصوت",otherIssues:"كنت أواجه مشكلات صوتية أخرى في هذه المكالمة"},videoRating:{noVideoReceived:"لم أتمكن من مشاهدة أي فيديو",noVideoSent:"لم يتمكن الآخرون من رؤيتي",lowQuality:"جودة الفيديو منخفضة",freezes:"الفيديو مجمد",stoppedUnexpectedly:"توقف الفيديو بشكل غير متوقع",darkVideoReceived:"أرى الشاشات الداكنة فقط عندما يقوم الآخرون بتشغيل الكاميرا الخاصة بهم",audioVideoOutOfSync:"لم تتم مزامنة الصوت والفيديو",otherIssues:"كنت أواجه مشكلات فيديو أخرى في هذه المكالمة"},screenshareRating:{noContentLocal:"تعذر على الأشخاص الآخرين رؤية مشاركة الشاشة",noContentRemote:"لم أتمكن من رؤية مشاركة الشاشة الخاصة بأشخاص آخرين",cannotPresent:"تعذر علي تقديم الشاشة",lowQuality:"كانت جودة مشاركة الشاشة منخفضة",freezes:"تم تجميد مشاركة الشاشة",stoppedUnexpectedly:"توقفت مشاركة الشاشة بشكل غير متوقع",largeDelay:"مشاركة الشاشة لها تأخير كبير",otherIssues:"كنت أواجه مشكلات أخرى في مشاركة الشاشة في هذه المكالمة"}},surveyIssuesHeadingStrings:{overallRating:"بشكل عام",audioRating:"صوت",videoRating:"فيديو",screenshareRating:"تقديم"},spotlightPrompt:{startSpotlightHeading:"هل تريد تسليط الضوء للجميع؟",startSpotlightText:"ستسلط الضوء على هذا الفيديو لجميع المشاركين في الاجتماع.",startSpotlightOnSelfText:"ستبرز مقطع الفيديو الخاص بك لجميع المشاركين في الاجتماع.",startSpotlightConfirmButtonLabel:"تسليط الضوء للجميع",startSpotlightCancelButtonLabel:"إلغاء",stopSpotlightHeading:"هل تريد إيقاف تسليط الضوء على مقطع الفيديو هذا؟",stopSpotlightOnSelfHeading:"هل تريد إنهاء تسليط الضوء؟",stopAllSpotlightHeading:"التوقف عن تسليط الضوء على جميع ملفات الفيديو؟",stopSpotlightText:"لن يتم تمييز الفيديو الخاص بك بعد الآن لجميع المشاركين في الاجتماع.",stopSpotlightOnSelfText:"لن يتم تمييز الفيديو الخاص بك بعد الآن لجميع المشاركين في الاجتماع.",stopAllSpotlightText:"لن يتم تمييز مقاطع الفيديو بعد الآن للجميع في الاجتماع.",stopSpotlightConfirmButtonLabel:"التوقف عن تسليط الضوء",stopSpotlightOnSelfConfirmButtonLabel:"الخروج من وضع تسليط الضوء",stopSpotlightCancelButtonLabel:"إلغاء",closeSpotlightPromptButtonLabel:"إغلاق"},exitSpotlightButtonLabel:"الخروج من وضع تسليط الضوء",exitSpotlightButtonTooltip:"الخروج من وضع تسليط الضوء",leaveConfirmButtonLabel:"مغادرة",endCallConfirmButtonLabel:"إنهاء المكالمة",hangUpCancelButtonLabel:"إلغاء",leaveConfirmDialogTitle:"هل تريد مغادرة المكالمة؟",leaveConfirmDialogContent:"هل تريد بالتأكيد مغادرة المكالمة؟",endCallConfirmDialogTitle:"هل تريد إنهاء المكالمة؟",endCallConfirmDialogContent:"ستنهي المكالمة لدى الجميع.",invalidMeetingIdentifier:"معرف اجتماع أو رمز مرور غير صالح",pinParticipantMenuLabel:"تثبيت نيابة عني",pinParticipantLimitReachedMenuLabel:"تثبيت (تم بلوغ الحد الأقصى)",unpinParticipantMenuLabel:"إزالة التثبيت",pinParticipantMenuItemAriaLabel:"تثبيت نيابة عني",unpinParticipantMenuItemAriaLabel:"إزالة التثبيت",muteAllMenuLabel:"كتم صوت الجميع",muteAllDialogTitle:"هل تريد كتم صوت الجميع؟",muteAllDialogContent:"سيؤدي ذلك إلى كتم صوت الجميع في الاجتماع باستثننائك.",muteAllConfirmButtonLabel:"كتم صوت الجميع",muteAllCancelButtonLabel:"إلغاء",joinBreakoutRoomButtonLabel:"انضمام إلى غرفة فرعية",returnFromBreakoutRoomButtonLabel:"مغادرة الغرفة",leaveBreakoutRoomAndMeetingButtonLabel:"مغادرة الاجتماع",breakoutRoomJoinedNotificationTitle:"لقد انضممت إلى الغرفة الفرعية '{roomName}'",joinBreakoutRoomBannerTitle:"الغرفة الفرعية '{roomName}' مفتوحة",joinBreakoutRoomBannerButtonLabel:"انضمام",returnFromBreakoutRoomBannerTitle:"رجوع إلى الاجتماع الرئيسي",returnFromBreakoutRoomBannerButtonLabel:"إعادة الانضمام",forbidAudioMenuLabel:"تعطيل الميكروفون",permitAudioMenuLabel:"السماح بالميكروفون",forbidOthersAudioDialogTitle:"هل تريد تعطيل الميكروفون للحضور؟",forbidOthersAudioDialogContent:"لن يتمكن الحضور من إلغاء كتم الصوت لأنفسهم. يمكن للمنظم والمقدمين السماح للأشخاص بإلغاء كتم الصوت حسب الحاجة.",forbidOthersAudioConfirmButtonLabel:"تعطيل الميكروفونات",forbidOthersAudioCancelButtonLabel:"إلغاء الأمر",permitOthersAudioDialogTitle:"هل تريد السماح بالميكروفون لدى الحضور؟",permitOthersAudioDialogContent:"سيتمكن أي شخص في الاجتماع من إلغاء كتم الصوت لأنفسهم.",permitOthersAudioConfirmButtonLabel:"السماح بالميكروفونات",permitOthersAudioCancelButtonLabel:"إلغاء الأمر",forbidOthersAudioMenuLabel:"تعطيل الميكروفون للحضور",permitOthersAudioMenuLabel:"السماح بالميكروفون للحضور",forbidVideoMenuLabel:"تعطيل الكاميرا",permitVideoMenuLabel:"السماح بالكاميرا",forbidOthersVideoDialogTitle:"هل تريد تعطيل الكاميرا لدى الحضور؟",forbidOthersVideoDialogContent:"لن يتمكن الحضور من تشغيل الفيديو بأنفسهم. يمكن للمنظم والمقدمين السماح للأشخاص بتشغيل الكاميرا حسب الحاجة.",forbidOthersVideoConfirmButtonLabel:"تعطيل الكاميرات",forbidOthersVideoCancelButtonLabel:"إلغاء الأمر",permitOthersVideoDialogTitle:"هل تريد السماح بالكاميرا لدى الحضور؟",permitOthersVideoDialogContent:"سيتمكن كل شخص في الاجتماع من تشغيل الفيديو بأنفسهم.",permitOthersVideoConfirmButtonLabel:"السماح بالكاميرات",permitOthersVideoCancelButtonLabel:"إلغاء الأمر",forbidOthersVideoMenuLabel:"تعطيل الكاميرا للحضور",permitOthersVideoMenuLabel:"السماح بالكاميرا للحضور"};var chat$l={chatListHeader:"في هذه الدردشة",uploadAttachment:"تحميل المرفق",uploadImageDataNotProvided:"لم يتم توفير بيانات الصورة.",uploadImageIsTooLarge:"الصورة كبيرة جدا. اختر جهازا أقل من {maxImageSize} ميغابايت.",uploadImageExtensionIsNotAllowed:"تحميل الصورة .{imageExtension} غير مسموح به.",uploadImageFailed:"تعذر تحميل الصورة. الرجاء المحاولة مرة أخرى لاحقا."};var callWithChat$l={chatButtonLabel:"دردشة",chatButtonNewMessageNotificationLabel:"رسالة جديدة",chatButtonTooltipClosedWithMessageCount:"إظهار الدردشة ({unreadMessagesCount} غير مقروءة)",chatButtonTooltipClose:"إخفاء الدردشة",chatButtonTooltipOpen:"إظهار الدردشة",chatPaneTitle:"دردشة",copyInviteLinkButtonLabel:"نسخ ارتباط دعوة",copyInviteLinkButtonActionedLabel:"تم نسخ الارتباط",copyInviteLinkActionedAriaLabel:"تم نسخ ارتباط الدعوة",dismissSidePaneButtonLabel:"إغلاق",moreDrawerAudioDeviceMenuTitle:"جهاز صوت",moreDrawerButtonLabel:"المزيد من الخيارات",moreDrawerButtonTooltip:"المزيد من الخيارات",moreDrawerMicrophoneMenuTitle:"الميكروفون",moreDrawerSpeakerMenuTitle:"مكبر الصوت",moreDrawerCaptionsMenuTitle:"نص الكلام مباشرةً",moreDrawerSpokenLanguageMenuTitle:"اللغة المنطوقة",moreDrawerCaptionLanguageMenuTitle:"لغة التسمية التوضيحية",moreDrawerGalleryOptionsMenuTitle:"خيارات المعرض",peopleButtonLabel:"الأشخاص",selectedPeopleButtonLabel:"تم تحديد زر الأشخاص",peopleButtonTooltipOpen:"إظهار المشاركين",peopleButtonTooltipClose:"إخفاء المشاركين",peopleButtonTooltipOpenAriaLabel:"زر الأشخاص، إظهار المشاركين",peopleButtonTooltipCloseAriaLabel:"زر الأشخاص، إخفاء المشاركين",peoplePaneSubTitle:"في هذه المكالمة",peoplePaneTitle:"الأشخاص",pictureInPictureTileAriaLabel:"موجزات الفيديو. انقر للعودة إلى شاشة المكالمة.",removeMenuLabel:"إزالة",openDialpadButtonLabel:"اطلب رقم الهاتف",returnToCallButtonAriaDescription:"رجوع إلى المكالمة",returnToCallButtonAriaLabel:"رجوع",peoplePaneAddPeopleButtonLabel:"إضافة أشخاص",dialpadStartCallButtonLabel:"مكالمة",dialpadModalTitle:"اطلب رقم الهاتف",dialpadModalAriaLabel:"لوحة الطلب",dialpadCloseModalButtonAriaLabel:"إغلاق لوحة الطلب",openDtmfDialpadLabel:"عرض لوحة الطلب",dtmfDialpadPlaceholderText:"أدخل عدداً",breakoutRoomChatPaneTitle:"دردشة الغرفة الفرعية",chatContentSpinnerLabel:"يتم الآن تحميل الدردشة"};var ar_SA = {call:call$l,chat:chat$l,callWithChat:callWithChat$l};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Arabic (Saudi Arabia)
 *
 * @public
 */
const COMPOSITE_LOCALE_AR_SA = {
    component: COMPONENT_LOCALE_AR_SA,
    strings: createCompositeStrings(ar_SA)
};

var call$k={cameraLabel:"Kamera",noCamerasLabel:"Nebyly nalezeny žádné kamery",cameraPermissionDenied:"Váš prohlížeč blokuje přístup ke kameře.",cameraTurnedOff:"Máte vypnutou kameru.",chatButtonLabel:"Chat",close:"Zavřít",complianceBannerNowOnlyRecording:"Nyní tuto schůzku pouze nahráváte.",complianceBannerNowOnlyTranscription:"Nyní tuto schůzku pouze přepisujete.",complianceBannerRecordingAndTranscriptionSaved:"Záznam a přepis se ukládají.",complianceBannerRecordingAndTranscriptionStarted:"Spustilo se nahrávání a přepis.",complianceBannerRecordingAndTranscriptionStopped:"Nahrávání a přepis se zastavily.",complianceBannerRecordingSaving:"Probíhá ukládání záznamu.",complianceBannerRecordingStarted:"Nahrávání se spustilo.",complianceBannerRecordingStopped:"Nahrávání se zastavilo.",complianceBannerTranscriptionStarted:"Přepis se spustil.",complianceBannerTranscriptionConsent:"Připojením vyjadřujete souhlas s přepisem této schůzky.",complianceBannerTranscriptionSaving:"Přepis se ukládá.",complianceBannerTranscriptionStopped:"Přepis se zastavil.",configurationPageTitle:"Začít hovor",copyInviteLinkButtonLabel:"Kopírovat odkaz pro pozvání",copyInviteLinkButtonActionedLabel:"Odkaz byl zkopírován.",copyInviteLinkActionedAriaLabel:"Odkaz pozvánky se zkopíroval",defaultPlaceHolder:"Vyberte jednu možnost",dismissSidePaneButtonLabel:"Zavřít",videoEffectsPaneTitle:"Efekty",videoEffectsPaneBackgroundSelectionTitle:"Pozadí",videoEffectsPaneAriaLabel:"Podokno video efektů",configurationPageCameraIsLoadingLabel:"Video se načítá…",configurationPageVideoEffectsButtonLabel:"Efekty",unableToStartVideoEffect:"Nelze použít efekt videa.",blurBackgroundEffectButtonLabel:"Rozostření",blurBackgroundTooltip:"Rozostřit pozadí",removeBackgroundEffectButtonLabel:"Žádná",removeBackgroundTooltip:"Odebrat pozadí",cameraOffBackgroundEffectWarningText:"Vaše kamera je vypnutá. Pokud chcete vidět efekt videa, zapněte kameru.",failedToJoinCallDueToNoNetworkMoreDetails:"Hovor byl odpojen z důvodu problému se sítí. Zkontrolujte připojení a připojte se znovu.",failedToJoinCallDueToNoNetworkTitle:"Hovor odpojen",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Vstup do hovoru vám nebyl povolen. Pokud se jednalo o chybu, znovu se do hovoru přihlaste.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Odchod z předsálí",learnMore:"Další informace",leavingCallTitle:"Opouštíte hovor...",leftCallMoreDetails:"Pokud se jedná o chybu, připojte se k hovoru znovu.",leftCallTitle:"Opustili jste hovor",lobbyScreenConnectingToCallTitle:"Připojování k hovoru",lobbyScreenWaitingToBeAdmittedTitle:"Čeká se na přijetí",microphonePermissionDenied:"Váš prohlížeč blokuje přístup k mikrofonu.",microphoneToggleInLobbyNotAllowed:"V předsálí nejde ztlumit ani zrušit ztlumení.",mutedMessage:"Jste ztlumení",networkReconnectMoreDetails:"Vypadá to, že se něco nepovedlo. Pokoušíme se znovu připojit k hovoru.",networkReconnectTitle:"Vydržte",notInvitedToRoomDetails:"K této místnosti se nemůžete připojit, protože nemáte pozvánku.",notInvitedToRoomTitle:"Nepozván(a) do místnosti",peopleButtonLabel:"Lidé",selectedPeopleButtonLabel:"Vybráno tlačítko Lidé",peoplePaneTitle:"Lidé",peoplePaneMoreButtonAriaLabel:"Více",peopleButtonTooltipOpen:"Zobrazit účastníky",peopleButtonTooltipClose:"Skrýt účastníky",peoplePaneSubTitle:"V tomto hovoru {numberOfPeople}",privacyPolicy:"Zásady ochrany osobních údajů",rejoinCallButtonLabel:"Znovu se připojit k hovoru",removedFromCallMoreDetails:"Jiný účastník vás odebral z hovoru.",removedFromCallTitle:"Byli jste odebráni",removeMenuLabel:"Odebrat",startSpotlightMenuLabel:"Přidat do hlavního výběru všem",addSpotlightMenuLabel:"Přidat do hlavního výběru",stopSpotlightMenuLabel:"Zastavit přidávání do hlavního výběru",stopSpotlightOnSelfMenuLabel:"Ukončit hlavní výběr",spotlightLimitReachedMenuTitle:"Dosáhlo se limitu hlavního výběru",stopAllSpotlightMenuLabel:"Zastavit všechny výběry",returnToCallButtonAriaDescription:"Pokračovat v hovoru",returnToCallButtonAriaLabel:"Zpět",roomNotFoundDetails:"Zadané ID místnosti se nepovedlo najít.",roomNotFoundTitle:"Místnost nebyla nalezena",roomNotValidDetails:"Tato místnost není v tuto chvíli platná.",roomNotValidTitle:"Neplatná místnost",inviteToRoomRemovedDetails:"Vaše pozvánka k připojení k této místnosti se odebrala.",inviteToRoomRemovedTitle:"Pozvánka do místnosti se odebrala",soundLabel:"Zvuk",noMicrophonesLabel:"Nenašly se žádné mikrofony",noSpeakersLabel:"Nenašly se žádné reproduktory",startCallButtonLabel:"Zahájit hovor",openDialpadButtonLabel:"Vytočit telefonní číslo",peoplePaneAddPeopleButtonLabel:"Přidat lidi",dialpadStartCallButtonLabel:"Volat",dialpadModalTitle:"Vytočit telefonní číslo",dialpadModalAriaLabel:"Číselník",dialpadCloseModalButtonAriaLabel:"Zavřít číselník",moreButtonCallingLabel:"Další",resumeCallButtonLabel:"Pokračovat",resumingCallButtonLabel:"Obnovování…",resumeCallButtonAriaLabel:"Pokračovat v hovoru",resumingCallButtonAriaLabel:"Pokračovat v hovoru",holdScreenLabel:"Váš hovor je přidržený",openDtmfDialpadLabel:"Zobrazit číselník",dtmfDialpadPlaceholderText:"Zadejte číslo",outboundCallingNoticeString:"Volání…",participantJoinedNoticeString:"{displayName} se připojil(a)",twoParticipantJoinedNoticeString:"{displayName1} a {displayName2} se připojili",threeParticipantJoinedNoticeString:"Uživatelé {displayName1}, {displayName2} a {displayName3} se připojili",participantLeftNoticeString:"{displayName} se odpojil(a)",twoParticipantLeftNoticeString:"{displayName1} a {displayName2} se odpojili",threeParticipantLeftNoticeString:"Uživatelé {displayName1}, {displayName2} a {displayName3} se odpojili",unnamedParticipantString:"nepojmenovaný účastník",manyUnnamedParticipantsJoined:"nepojmenovaný účastník a {numOfParticipants} další účastníci se připojili",manyUnnamedParticipantsLeft:"nepojmenovaný účastník a {numOfParticipants} další účastníci se odpojili",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} a {numOfParticipants} se připojili další účastníci",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} a {numOfParticipants} další účastníci",liveCaptionsLabel:"Živé titulky",captionsSettingsLabel:"Nastavení titulků",startCaptionsButtonOnLabel:"Zapnout titulky",startCaptionsButtonOffLabel:"Vypnout titulky",startCaptionsButtonTooltipOnContent:"Vypnout titulky",startCaptionsButtonTooltipOffContent:"Zapnout titulky",captionsSettingsModalTitle:"Nastavení titulků",captionsSettingsSpokenLanguageDropdownLabel:"Mluvený jazyk",captionsSettingsCaptionLanguageDropdownLabel:"Jazyk titulků",captionsSettingsSpokenLanguageDropdownInfoText:"Jazyk, kterým mluví všichni účastníci tohoto hovoru.",captionsSettingsCaptionLanguageDropdownInfoText:"Titulky se zobrazí v tomto jazyce.",captionsSettingsConfirmButtonLabel:"Potvrdit",captionsSettingsCancelButtonLabel:"Zrušit",captionsSettingsModalAriaLabel:"Modální nastavení titulků",captionsSettingsCloseModalButtonAriaLabel:"Nastavení titulků",captionsBannerMoreButtonCallingLabel:"Další možnosti nastavení titulků",captionsBannerMoreButtonTooltip:"Další možnosti",realTimeTextLabel:"Psaní v reálném čase (RTT)",startRealTimeTextLabel:"Zapnout RTT pro tento hovor",disabledStartRealTimeTextLabel:"Psaní v reálném čase je aktuálně zapnuté, nedá se vypnout",dismissModalAriaLabel:"Místní a vzdálený obrázek a obrázek, stisknutím klávesy Enter se vrátíte k hovoru",callRejectedTitle:"Hovor nešlo spojit.",callRejectedMoreDetails:"Osoba, se kterou se pokoušíte spojit, není k dispozici. Zkuste to prosím znovu později.",callTimeoutTitle:"Hovor nešlo spojit.",callTimeoutDetails:"Osoba, se kterou se pokoušíte spojit, není k dispozici. Zkuste to prosím znovu později.",callTimeoutBotTitle:"Hovor nešlo spojit.",callTimeoutBotDetails:"Služba, ke které se pokoušíte získat přístup, není k dispozici. Zkuste to prosím znovu později.",dtmfDialerButtonLabel:"Číselník",dtmfDialerButtonTooltipOn:"Zobrazit číselník",dtmfDialerButtonTooltipOff:"Skrýt číselník",dtmfDialerMoreButtonLabelOn:"Zobrazit číselník",dtmfDialerMoreButtonLabelOff:"Skrýt číselník",phoneCallMoreButtonLabel:"Telefonní hovor",spokenLanguageStrings:{"ar-ae":"Arabština - Spojené arabské emiráty","ar-sa":"Arabština - Saúdská Arábie","da-dk":"Dánština","de-de":"Němčina - Německo","en-au":"Angličtina - Austrálie","en-ca":"Angličtina - Kanada","en-gb":"Angličtina - Spojené království","en-in":"Angličtina - Indie","en-nz":"Angličtina - Nový Zéland","en-us":"Angličtina - Spojené státy americké","es-es":"Španělština - Španělsko (Modern Sort)","es-mx":"Španělština - Mexiko","fi-fi":"Finština","fr-ca":"Francouzština - Kanada","fr-fr":"Francouzština - Francie","hi-in":"Hindština","it-it":"Italština - Itálie","ja-jp":"Japonština","ko-kr":"Korejština","nb-no":"Norština (Bokmål)","nl-be":"Holandština – Belgie","nl-nl":"Holandština - Nizozemsko","pl-pl":"Polština","pt-br":"Portugalština – Brazílie","ru-ru":"Ruština","sv-se":"Švédština","zh-cn":"Čínština – Čínská lidová republika","zh-hk":"Čínština – Hong Kong SAR","cs-cz":"Čeština","pt-pt":"Portugalština – Portugalsko","tr-tr":"Turečtina","vi-vn":"Vietnamština","th-th":"Thajština","he-il":"Hebrejština","cy-gb":"Velština","uk-ua":"Ukrajinština","el-gr":"Řečtina","hu-hu":"Maďarština","ro-ro":"Rumunština","sk-sk":"Slovenština","zh-tw":"Čínština – Tchaj-wan"},captionLanguageStrings:{ar:"Arabština",da:"Dánština",de:"Němčina",en:"Angličtina",es:"Španělština",fi:"Finština","fr-ca":"Francouzština - Kanada",fr:"Francouzština - Francie",hi:"Hindština",it:"Italština",ja:"Japonština",ko:"Korejština",nb:"Norština (Bokmål)",nl:"Nizozemština",pl:"Polština",pt:"Portugalština – Brazílie",ru:"Ruština",sv:"Švédština","zh-Hans":"Čínština (zjednodušená)","zh-Hant":"Čínština (tradiční)",cs:"Čeština","pt-pt":"Portugalština – Portugalsko",tr:"Turečtina",vi:"Vietnamština",th:"Thajština",he:"Hebrejština",cy:"Velština",uk:"Ukrajinština",el:"Řečtina",hu:"Maďarština",ro:"Rumunština",sk:"Slovenština"},captionsBannerSpinnerText:"Spouští se titulky...",realTimeTextInputBoxDefaultText:"Napište zprávu v reálném čase",realTimeTextInputErrorMessage:"Zpráva nesmí být delší než 2000 znaků",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (psaní v reálném čase) je povolené pro všechny účastníky po celou dobu trvání schůzky.",realTimeTextBannerLinkLabel:"Další informace",realTimeTextModalAriaLabel:"Zapnout modální okno psaní v reálném čase",realTimeTextModalTitle:"Zapnout psaní v reálném čase?",realTimeTextModalText:"Text v reálném čase (RTT) se zapne pro všechny účastníky schůzky.",realTimeTextConfirmButtonLabel:"Zapnout",realTimeTextCancelButtonLabel:"Zrušit",realTimeTextCloseModalButtonAriaLabel:"Zavřít modální okno psaní v reálném čase",captionsOnlyContainerTitle:"Titulky",realTimeTextOnlyContainerTitle:"Text v reálném čase",captionsAndRealTimeTextContainerTitle:"Titulky a text v reálném čase",expandButtonAriaLabel:"Rozbalit",minimizeButtonAriaLabel:"Sbalit",transferPageTransferorText:"Probíhá přenos...",transferPageTransferTargetText:"Připojování...",transferPageUnknownTransferorDisplayName:"Neznámý",transferPageUnknownTransferTargetDisplayName:"Neznámý",transferPageNoticeString:"Přepojujeme vás",participantCouldNotBeReachedTitle:"Cílový účastník není v tuto chvíli k dispozici",participantCouldNotBeReachedMoreDetails:"Obraťte se prosím na účastníka, až bude k dispozici",permissionToReachTargetParticipantNotAllowedTitle:"Oprávnění k dosažení cílového účastníka není povolené",permissionToReachTargetParticipantNotAllowedMoreDetails:"Zkontrolujte prosím, jestli je cílový účastník ve stejném tenantovi.",unableToResolveTenantTitle:"Nepovedlo se přeložit ID tenanta pro cílového účastníka",unableToResolveTenantMoreDetails:"Zkontrolujte prosím, jestli je správně zadané ID účastníka.",participantIdIsMalformedTitle:"ID účastníka nemá správný formát.",participantIdIsMalformedMoreDetails:"Zkontrolujte prosím, jestli je ID účastníka ve správném formátu.",moreButtonGalleryControlLabel:"Zobrazit",moreButtonGalleryPositionToggleLabel:"Přesunout galerii na začátek",moreButtonGallerySpeakerLayoutLabel:"Reproduktor",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamické",moreButtonGalleryDefaultLayoutLabel:"Zobrazení galerie",moreButtonGalleryFocusedContentLayoutLabel:"Přenést hlavní pozornost na obsah",moreButtonLargeGalleryDefaultLayoutLabel:"Velká galerie",moreButtonTogetherModeLayoutLabel:"Režim Společně",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Vaše kamera byla zakázána. Už nemůžete sdílet video.",grantedDueToMeetingOption:"Vaše kamera je povolená. Pokud chcete sdílet video, zapněte ji."},unmuteMic:{lostDueToMeetingOption:"Váš mikrofon byl zakázán. Už nemůžete zrušit ztlumení.",grantedDueToMeetingOption:"Váš mikrofon je povolený. Zrušte ztlumení, pokud chcete mluvit."},shareScreen:{lostDueToRoleChangeToAttendee:"Vaše role se změnila. Některé akce, jako je sdílení obsahu, pro vás nebudou k dispozici.",grantedDueToRoleChangeToPresenter:"Jste prezentující. Můžete sdílet obsah a moderovat schůzku."},hideAttendeeNames:{hideAttendeeNameAttendee:"Vaše jméno je před ostatními účastníky skryté. Organizátoři a přednášející uvidí vaše skutečné jméno.",hideAttendeeNamePresenter:"Jména účastníků jsou skrytá. Pro ochranu soukromí účastníků uvidí jména jenom organizátoři a přednášející."}},surveyTitle:"Pomozte nám se zlepšovat",starSurveyHelperText:"Jaká byla kvalita hovoru?",starSurveyOneStarText:"Kvalita byla špatná.",starSurveyTwoStarText:"Kvalita byla slabá.",starSurveyThreeStarText:"Kvalita byla dobrá.",starSurveyFourStarText:"Kvalita byla skvělá.",starSurveyFiveStarText:"Kvalita byla vynikající.",surveyConfirmButtonLabel:"Poslat zpětnou vazbu",starRatingAriaLabel:"Vybrat {0} z(e) {1} hvězdiček",surveySkipButtonLabel:"Přeskočit",tagsSurveyQuestion:"Co by mohlo být lepší?",tagsSurveyTextFieldDefaultText:"Jiné, upřesněte prosím",tagsSurveyHelperText:"Zkontrolujte všechny problémy, se které jste zaznamenali.",endOfSurveyText:"Děkujeme za váš názor!",surveyIssues:{overallRating:{callCannotJoin:"Nepovedlo se mi připojit se k hovoru",callCannotInvite:"Nepovedlo se mi pozvat ostatní do hovoru.",hadToRejoin:"Musel(a) jsem se znovu připojit k hovoru",callEndedUnexpectedly:"Hovor pro mě neočekávaně skončil",otherIssues:"Měl(a) jsem s hovorem jiné problémy"},audioRating:{noLocalAudio:"Druhá strana neslyšela žádný zvuk",noRemoteAudio:"Vůbec jsem neslyšel(a) zvuk",echo:"Při hovoru jsem slyšel(a) ozvěny",audioNoise:"Při hovoru jsem slyšel(a) šum",lowVolume:"Hlasitost byla nízká",audioStoppedUnexpectedly:"Zvuk se neočekávaně zastavil",distortedSpeech:"Zvuk byl zkreslený",audioInterruption:"Došlo k přerušení zvuku",otherIssues:"Během tohoto hovoru došlo k jiným problémům se zvukem"},videoRating:{noVideoReceived:"Vůbec se mi nezobrazovalo video",noVideoSent:"Ostatní mě neviděli",lowQuality:"Kvalita videa byla nízká.",freezes:"Video se zablokuje",stoppedUnexpectedly:"Video se neočekávaně zastavilo",darkVideoReceived:"Když ostatní zapnou kameru, vidím jenom tmavé obrazovky",audioVideoOutOfSync:"Zvuk a video se nesynchronizovaly",otherIssues:"V tomto hovoru došlo k jiným problémům s videem"},screenshareRating:{noContentLocal:"Ostatní uživatelé neviděli moje sdílení obrazovky",noContentRemote:"Nepovedlo se mi zobrazit sdílení obrazovky jiných lidí",cannotPresent:"Nepovedlo se mi ukázat obrazovku",lowQuality:"Kvalita sdílení obrazovky byla nízká",freezes:"Zablokované sdílení obrazovky",stoppedUnexpectedly:"Sdílení obrazovky se neočekávaně zastavilo",largeDelay:"Sdílení obrazovky má velké zpoždění",otherIssues:"Měl(a) jsem jiné problémy se sdílením obrazovky v tomto hovoru"}},surveyIssuesHeadingStrings:{overallRating:"Celkem",audioRating:"Zvuk",videoRating:"Video",screenshareRating:"Prezentování"},spotlightPrompt:{startSpotlightHeading:"Přidat do hlavního výběru všem?",startSpotlightText:"Toto video se zvýrazní všem účastníkům schůzky.",startSpotlightOnSelfText:"Vaše video se zvýrazní všem účastníkům schůzky.",startSpotlightConfirmButtonLabel:"Přidat do hlavního výběru všem",startSpotlightCancelButtonLabel:"Zrušit",stopSpotlightHeading:"Chcete přestat zobrazovat toto video v hlavním výběru?",stopSpotlightOnSelfHeading:"Ukončit hlavní výběr?",stopAllSpotlightHeading:"Zastavit zobrazování všech videí v hlavním výběru?",stopSpotlightText:"Účastníkům se už toto video nebude zvýrazňovat.",stopSpotlightOnSelfText:"Účastníkům se už vaše video nebude zvýrazňovat.",stopAllSpotlightText:"Účastníkům schůzky se už videa nebudou zvýrazňovat.",stopSpotlightConfirmButtonLabel:"Zastavit přidávání do hlavního výběru",stopSpotlightOnSelfConfirmButtonLabel:"Ukončit hlavní výběr",stopSpotlightCancelButtonLabel:"Zrušit",closeSpotlightPromptButtonLabel:"Zavřít"},exitSpotlightButtonLabel:"Ukončit hlavní výběr",exitSpotlightButtonTooltip:"Ukončit hlavní výběr",leaveConfirmButtonLabel:"Odejít",endCallConfirmButtonLabel:"Ukončit hovor",hangUpCancelButtonLabel:"Zrušit",leaveConfirmDialogTitle:"Chcete z hovoru odejít?",leaveConfirmDialogContent:"Opravdu chcete hovor opustit?",endCallConfirmDialogTitle:"Chcete hovor ukončit?",endCallConfirmDialogContent:"Hovor ukončíte pro všechny.",invalidMeetingIdentifier:"Neplatné ID schůzky nebo heslo",pinParticipantMenuLabel:"Připnout pro mě",pinParticipantLimitReachedMenuLabel:"Připnout (dosáhlo se limitu)",unpinParticipantMenuLabel:"Odepnout",pinParticipantMenuItemAriaLabel:"Připnout pro mě",unpinParticipantMenuItemAriaLabel:"Odepnout",muteAllMenuLabel:"Ztlumit všechny",muteAllDialogTitle:"Chcete všechny ztlumit?",muteAllDialogContent:"Ztlumí všechny účastníky schůzky kromě vás.",muteAllConfirmButtonLabel:"Ztlumit všechny",muteAllCancelButtonLabel:"Zrušit",joinBreakoutRoomButtonLabel:"Připojí se ke skupinové místnosti.",returnFromBreakoutRoomButtonLabel:"Opustit místnost",leaveBreakoutRoomAndMeetingButtonLabel:"Opustit schůzku",breakoutRoomJoinedNotificationTitle:"Připojili jste se ke skupinové místnosti {roomName}",joinBreakoutRoomBannerTitle:"Skupinová místnost {roomName} je otevřená",joinBreakoutRoomBannerButtonLabel:"Připojit se",returnFromBreakoutRoomBannerTitle:"Vrátit se k hlavní schůzce",returnFromBreakoutRoomBannerButtonLabel:"Znovu připojit",forbidAudioMenuLabel:"Vypnout mikrofon",permitAudioMenuLabel:"Povolit mikrofon",forbidOthersAudioDialogTitle:"Zakázat účastníkům mikrofon?",forbidOthersAudioDialogContent:"Účastníci nebudou moct zrušit ztlumení. Organizátor a prezentující můžou lidem podle potřeby povolit zrušit ztlumení.",forbidOthersAudioConfirmButtonLabel:"Zakázat mikrofony",forbidOthersAudioCancelButtonLabel:"Zrušit",permitOthersAudioDialogTitle:"Povolit účastníkům mikrofon?",permitOthersAudioDialogContent:"Všichni účastníci schůzky si budou moct zrušit ztlumení.",permitOthersAudioConfirmButtonLabel:"Povolit mikrofony",permitOthersAudioCancelButtonLabel:"Zrušit",forbidOthersAudioMenuLabel:"Zakázat účastníkům mikrofon",permitOthersAudioMenuLabel:"Povolit mikrofon pro účastníky",forbidVideoMenuLabel:"Zakázat kameru",permitVideoMenuLabel:"Povolit kameru",forbidOthersVideoDialogTitle:"Zakázat účastníkům kameru?",forbidOthersVideoDialogContent:"Účastníci nebudou moct zapnout video sami. Organizátor a přednášející mohou lidem umožnit zapnout kameru podle potřeby.",forbidOthersVideoConfirmButtonLabel:"Zakázat kamery",forbidOthersVideoCancelButtonLabel:"Zrušit",permitOthersVideoDialogTitle:"Povolit účastníkům kameru?",permitOthersVideoDialogContent:"Všichni účastníci schůzky budou moct video zapnout sami.",permitOthersVideoConfirmButtonLabel:"Povolit kamery",permitOthersVideoCancelButtonLabel:"Zrušit",forbidOthersVideoMenuLabel:"Zakázat účastníkům kameru",permitOthersVideoMenuLabel:"Povolit kameru pro účastníky"};var chat$k={chatListHeader:"V tomto chatu",uploadAttachment:"Odeslat přílohu",uploadImageDataNotProvided:"Data obrázku nebyla zadána.",uploadImageIsTooLarge:"Obrázek je příliš velký. Zvolte míň než {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Nahrávání. Obrázek {imageExtension} není povolený.",uploadImageFailed:"Obrázek nejde nahrát. Zkuste to prosím znovu později."};var callWithChat$k={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"Nová zpráva",chatButtonTooltipClosedWithMessageCount:"Zobrazit chat (počet nepřečtených zpráv: {unreadMessagesCount})",chatButtonTooltipClose:"Skrýt chat",chatButtonTooltipOpen:"Zobrazit chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Kopírovat odkaz pro pozvání",copyInviteLinkButtonActionedLabel:"Odkaz byl zkopírován.",copyInviteLinkActionedAriaLabel:"Odkaz pozvánky se zkopíroval",dismissSidePaneButtonLabel:"Zavřít",moreDrawerAudioDeviceMenuTitle:"Zvukové zařízení",moreDrawerButtonLabel:"Další možnosti",moreDrawerButtonTooltip:"Další možnosti",moreDrawerMicrophoneMenuTitle:"Mikrofon",moreDrawerSpeakerMenuTitle:"Mluvčí",moreDrawerCaptionsMenuTitle:"Živé titulky",moreDrawerSpokenLanguageMenuTitle:"Mluvený jazyk",moreDrawerCaptionLanguageMenuTitle:"Jazyk titulků",moreDrawerGalleryOptionsMenuTitle:"Možnosti galerie",peopleButtonLabel:"Lidé",selectedPeopleButtonLabel:"Vybráno tlačítko Lidé",peopleButtonTooltipOpen:"Zobrazit účastníky",peopleButtonTooltipClose:"Skrýt účastníky",peopleButtonTooltipOpenAriaLabel:"Tlačítko Lidé, Zobrazit účastníky",peopleButtonTooltipCloseAriaLabel:"Tlačítko Lidé, Skrýt účastníky",peoplePaneSubTitle:"V tomto hovoru",peoplePaneTitle:"Lidé",pictureInPictureTileAriaLabel:"Informační kanály videa. Kliknutím se vrátíte na obrazovku hovoru.",removeMenuLabel:"Odebrat",openDialpadButtonLabel:"Vytočit telefonní číslo",returnToCallButtonAriaDescription:"Pokračovat v hovoru",returnToCallButtonAriaLabel:"Zpět",peoplePaneAddPeopleButtonLabel:"Přidat lidi",dialpadStartCallButtonLabel:"Volat",dialpadModalTitle:"Vytočit telefonní číslo",dialpadModalAriaLabel:"Číselník",dialpadCloseModalButtonAriaLabel:"Zavřít číselník",openDtmfDialpadLabel:"Zobrazit číselník",dtmfDialpadPlaceholderText:"Zadejte číslo",breakoutRoomChatPaneTitle:"Chat ve skupinové místnosti",chatContentSpinnerLabel:"Načítá se chat"};var cs_CZ = {call:call$k,chat:chat$k,callWithChat:callWithChat$k};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Czech (Czech Republic)
 *
 * @public
 */
const COMPOSITE_LOCALE_CS_CZ = {
    component: COMPONENT_LOCALE_CS_CZ,
    strings: createCompositeStrings(cs_CZ)
};

var call$j={cameraLabel:"Camera",noCamerasLabel:"Heb ganfod camerâu",cameraPermissionDenied:"Mae eich porwr yn rhwystro mynediad at eich camera",cameraTurnedOff:"Mae eich camera wedi'i ddiffodd",chatButtonLabel:"Sgwrs",close:"Cau",complianceBannerNowOnlyRecording:"Rydych chi ond yn recordio'r cyfarfod hwn bellach.",complianceBannerNowOnlyTranscription:"Rydych yn awr yn trawsgrifio'r cyfarfod hwn yn unig.",complianceBannerRecordingAndTranscriptionSaved:"Mae recordiad a thrawsgrifiad yn cael eu cadw.",complianceBannerRecordingAndTranscriptionStarted:"Wedi dechrau recordio a thrawsgrifio.",complianceBannerRecordingAndTranscriptionStopped:"Wedi stopio recordio a thrawsgrifio.",complianceBannerRecordingSaving:"Mae’r recordiad yn cael ei gadw.",complianceBannerRecordingStarted:"Recordio wedi dechrau.",complianceBannerRecordingStopped:"Recordio wedi stopio.",complianceBannerTranscriptionStarted:"Wedi dechrau’r trawsgrifiad.",complianceBannerTranscriptionConsent:"Drwy ymuno, rydych yn cydsynio i drawsgrifio'r cyfarfod hwn.",complianceBannerTranscriptionSaving:"Mae trawsgrifiad yn cael ei gadw.",complianceBannerTranscriptionStopped:"Wedi stopio’r trawsgrifiad.",configurationPageTitle:"Dechrau galwad",copyInviteLinkButtonLabel:"Copïo dolen y gwahoddiad",copyInviteLinkButtonActionedLabel:"Wedi copïo'r ddolen",copyInviteLinkActionedAriaLabel:"Wedi copïo dolen y gwahoddiad",defaultPlaceHolder:"Dewiswch opsiwn",dismissSidePaneButtonLabel:"Cau",videoEffectsPaneTitle:"Effeithiau",videoEffectsPaneBackgroundSelectionTitle:"Cefndir",videoEffectsPaneAriaLabel:"Cwarel effeithiau fideo",configurationPageCameraIsLoadingLabel:"Mae’r fideo’n llwytho...",configurationPageVideoEffectsButtonLabel:"Effeithiau",unableToStartVideoEffect:"Does dim modd defnyddio'r effaith fideo.",blurBackgroundEffectButtonLabel:"Pylu",blurBackgroundTooltip:"Niwlio'r Cefndir",removeBackgroundEffectButtonLabel:"Dim",removeBackgroundTooltip:"Tynnu'r Cefndir",cameraOffBackgroundEffectWarningText:"Mae eich camera wedi'i ddiffodd. Rhowch y camera ar waith i weld yr effaith fideo.",failedToJoinCallDueToNoNetworkMoreDetails:"Cafodd yr alwad ei datgysylltu oherwydd problem rhwydwaith. Tarwch olwg ar eich cysylltiad ac ymuno eto.",failedToJoinCallDueToNoNetworkTitle:"Galwad wedi'i datgysylltu",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Doeddech chi ddim wedi cael mynediad yn yr alwad. Os mai camgymeriad oedd hyn, ailymunwch â'r alwad.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Wedi'i anfon ymaith o'r ystafell aros",learnMore:"Dysgu mwy",leavingCallTitle:"Wrthi'n gadael...",leftCallMoreDetails:"Os mai camgymeriad oedd hyn, ailymunwch â'r alwad.",leftCallTitle:"Fe adawoch chi'r alwad",lobbyScreenConnectingToCallTitle:"Ymuno â galwad",lobbyScreenWaitingToBeAdmittedTitle:"Wrthi’n aros am dderbyniad",microphonePermissionDenied:"Mae eich porwr yn rhwystro mynediad at eich meicroffon",microphoneToggleInLobbyNotAllowed:"Does dim modd tewi na dad-dewi tra yn yr ystafell aros.",mutedMessage:"Rydych chi wedi'ch tewi",networkReconnectMoreDetails:"Mae'n debyg bod rhywbeth wedi mynd o'i le. Rydyn ni'n ceisio mynd yn ôl i mewn i'r alwad.",networkReconnectTitle:"Daliwch arni",notInvitedToRoomDetails:"Does dim modd i chi ymuno â'r ystafell hon oherwydd does gennych chi ddim gwahoddiad.",notInvitedToRoomTitle:"Heb wahoddiad i'r ystafell",peopleButtonLabel:"Pobl",selectedPeopleButtonLabel:"Wedi Dewis y Botwm Pobl",peoplePaneTitle:"Pobl",peoplePaneMoreButtonAriaLabel:"Mwy",peopleButtonTooltipOpen:"Dangos cyfranogwyr",peopleButtonTooltipClose:"Cuddio’r cyfranogwyr",peoplePaneSubTitle:"Yn yr alwad hon {numberOfPeople}",privacyPolicy:"Polisi preifatrwydd",rejoinCallButtonLabel:"Ail-ymuno â'r alwad",removedFromCallMoreDetails:"Mae cyfranogwr arall wedi eich tynnu o'r alwad.",removedFromCallTitle:"Cawsoch eich tynnu",removeMenuLabel:"Tynnu",startSpotlightMenuLabel:"Sbotolau i bawb",addSpotlightMenuLabel:"Ychwanegu sbotolau",stopSpotlightMenuLabel:"Diffodd y sbotolau",stopSpotlightOnSelfMenuLabel:"Gadael y sbotolau",spotlightLimitReachedMenuTitle:"Wedi cyrraedd y terfyn sbotolau",stopAllSpotlightMenuLabel:"Stopio pob sbotolau",returnToCallButtonAriaDescription:"Mynd yn ôl i'r Alwad",returnToCallButtonAriaLabel:"Yn ôl",roomNotFoundDetails:"Doedd dim modd canfod ID yr ystafell a roddwyd.",roomNotFoundTitle:"Heb ganfod yr ystafell",roomNotValidDetails:"Dydy'r ystafell hon ddim yn ddilys ar hyn o bryd.",roomNotValidTitle:"Dydy'r ystafell ddim yn ddilys",inviteToRoomRemovedDetails:"Wedi tynnu eich gwahoddiad i ymuno â'r ystafell hon.",inviteToRoomRemovedTitle:"Wedi tynnu'r gwahoddiad i'r ystafell",soundLabel:"Sain",noMicrophonesLabel:"Heb ganfod meicroffonau",noSpeakersLabel:"Heb ganfod seinyddion",startCallButtonLabel:"Dechrau galwad",openDialpadButtonLabel:"Deialu rhif ffôn",peoplePaneAddPeopleButtonLabel:"Ychwanegu Pobl",dialpadStartCallButtonLabel:"Galwad",dialpadModalTitle:"Deialu Rhif Ffôn",dialpadModalAriaLabel:"Pad deialu",dialpadCloseModalButtonAriaLabel:"Cau'r pad deialu",moreButtonCallingLabel:"Mwy",resumeCallButtonLabel:"Ailgydio",resumingCallButtonLabel:"Wrthi'n ailgydio...",resumeCallButtonAriaLabel:"Ailgydio yn yr alwad",resumingCallButtonAriaLabel:"Ailgydio yn yr alwad",holdScreenLabel:"Mae eich galwad yn aros",openDtmfDialpadLabel:"Dangos y pad deialu",dtmfDialpadPlaceholderText:"Cofnodi rhif",outboundCallingNoticeString:"Wrthi'n galw...",participantJoinedNoticeString:"Mae {displayName} wedi ymuno",twoParticipantJoinedNoticeString:"Mae {displayName1} a {displayName2} wedi ymuno",threeParticipantJoinedNoticeString:"Mae {displayName1}, {displayName2} a {displayName3} wedi ymuno",participantLeftNoticeString:"Mae {displayName} wedi gadael",twoParticipantLeftNoticeString:"Mae {displayName1} a {displayName2} wedi gadael",threeParticipantLeftNoticeString:"Mae {displayName1}, {displayName2} a {displayName3} wedi gadael",unnamedParticipantString:"cyfranogwr dienw",manyUnnamedParticipantsJoined:"Mae cyfranogwr dienw a {numOfParticipants} o gyfranogwyr eraill wedi ymuno",manyUnnamedParticipantsLeft:"Mae cyfranogwr dienw a {numOfParticipants} o gyfranogwyr eraill wedi gadael",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} a {numOfParticipants} cyfranogwyr eraill wedi ymuno",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} a {numOfParticipants} cyfranogwyr eraill ar ôl",liveCaptionsLabel:"Capsiynau byw",captionsSettingsLabel:"Gosodiadau capsiynau",startCaptionsButtonOnLabel:"Rhoi capsiynau ar waith",startCaptionsButtonOffLabel:"Diffodd capsiynau",startCaptionsButtonTooltipOnContent:"Diffodd capsiynau",startCaptionsButtonTooltipOffContent:"Rhoi capsiynau ar waith",captionsSettingsModalTitle:"Gosodiadau Capsiynau",captionsSettingsSpokenLanguageDropdownLabel:"Iaith lafar",captionsSettingsCaptionLanguageDropdownLabel:"Iaith capsiynau",captionsSettingsSpokenLanguageDropdownInfoText:"Yr iaith y mae pawb yn yr alwad hon yn ei siarad.",captionsSettingsCaptionLanguageDropdownInfoText:"Bydd capsiynau’n ymddangos yn yr iaith hon.",captionsSettingsConfirmButtonLabel:"Cadarnhau",captionsSettingsCancelButtonLabel:"Canslo",captionsSettingsModalAriaLabel:"Moddol Gosodiad Capsiynau",captionsSettingsCloseModalButtonAriaLabel:"Cau Gosodiad Capsiynau",captionsBannerMoreButtonCallingLabel:"Mwy o Ddewisiadau Gosodiadau Capsiynau",captionsBannerMoreButtonTooltip:"Mwy o ddewisiadau",realTimeTextLabel:"Testun amser go iawn (RTT)",startRealTimeTextLabel:"Rhoi RTT ar waith ar gyfer yr alwad hon",disabledStartRealTimeTextLabel:"Mae RTT ar waith ar hyn o bryd, nid oes modd ei ddiffodd",dismissModalAriaLabel:"Llun a llun lleol a phell, pwyswch Enter i fynd yn ôl i'r alwad",callRejectedTitle:"Doedd dim modd cwblhau'r alwad",callRejectedMoreDetails:"Dydy'r unigolyn rydych chi'n ceisio cysylltu â nhw ddim ar gael. Rhowch gynnig arall arni rywbryd eto.",callTimeoutTitle:"Doedd dim modd cwblhau'r alwad",callTimeoutDetails:"Dydy'r unigolyn rydych chi'n ceisio cysylltu â nhw ddim ar gael. Rhowch gynnig arall arni rywbryd eto.",callTimeoutBotTitle:"Doedd dim modd cwblhau'r alwad",callTimeoutBotDetails:"Dydy'r gwasanaeth rydych chi'n ceisio cysylltu ag ef ddim ar gael. Rhowch gynnig arall arni rywbryd eto.",dtmfDialerButtonLabel:"Pad deialu",dtmfDialerButtonTooltipOn:"Dangos y pad deialu",dtmfDialerButtonTooltipOff:"Cuddio'r pad deialu",dtmfDialerMoreButtonLabelOn:"Dangos y pad deialu",dtmfDialerMoreButtonLabelOff:"Cuddio'r pad deialu",phoneCallMoreButtonLabel:"Galwad Ffôn",spokenLanguageStrings:{"ar-ae":"Arabeg - E.A.U.","ar-sa":"Arabeg - Saudi Arabia","da-dk":"Daneg","de-de":"Almaeneg - Yr Almaen","en-au":"Saesneg - Awstralia","en-ca":"Saesneg - Canada","en-gb":"Saesneg - Y Deyrnas Unedig","en-in":"Saesneg - India","en-nz":"Saesneg - Seland Newydd","en-us":"Saesneg - Unol Daleithiau America","es-es":"Sbaeneg - Sbaen (Modern)","es-mx":"Sbaeneg - Mecsico","fi-fi":"Ffinneg","fr-ca":"Ffrangeg - Canada","fr-fr":"Ffrangeg - Ffrainc","hi-in":"Hindi","it-it":"Eidaleg - Yr Eidal","ja-jp":"Japaneeg","ko-kr":"Coreeg","nb-no":"Norwyeg (Bokmål)","nl-be":"Iseldireg - Gwlad Belg","nl-nl":"Iseldireg - Yr Iseldiroedd","pl-pl":"Pwyleg","pt-br":"Portiwgaleg - Brasil","ru-ru":"Rwseg","sv-se":"Swedeg","zh-cn":"Tsieinëeg - Gweriniaeth y Bobl Tsieina","zh-hk":"Tsieinëeg - Hong Kong SAR","cs-cz":"Tsieceg","pt-pt":"Portiwgaleg - Portiwgal","tr-tr":"Tyrceg","vi-vn":"Fietnameg","th-th":"Thai","he-il":"Hebraeg","cy-gb":"Cymraeg","uk-ua":"Wcreineg","el-gr":"Groeg","hu-hu":"Hwngareg","ro-ro":"Rwmaneg","sk-sk":"Slofaceg","zh-tw":"Tsieinëeg - Taiwan"},captionLanguageStrings:{ar:"Arabeg",da:"Daneg",de:"Almaeneg",en:"Saesneg",es:"Sbaeneg",fi:"Ffinneg","fr-ca":"Ffrangeg - Canada",fr:"Ffrangeg - Ffrainc",hi:"Hindi",it:"Eidaleg",ja:"Japaneeg",ko:"Coreeg",nb:"Norwyeg (Bokmål)",nl:"Iseldireg",pl:"Pwyleg",pt:"Portiwgaleg - Brasil",ru:"Rwseg",sv:"Swedeg","zh-Hans":"Tsieinëeg (Syml)","zh-Hant":"Tsieinëeg (Traddodiadol)",cs:"Tsieceg","pt-pt":"Portiwgaleg - Portiwgal",tr:"Tyrceg",vi:"Fietnameg",th:"Thai",he:"Hebraeg",cy:"Cymraeg",uk:"Wcreineg",el:"Groeg",hu:"Hwngareg",ro:"Rwmaneg",sk:"Slofaceg"},captionsBannerSpinnerText:"Wrthi'n dechrau'r capsiynau...",realTimeTextInputBoxDefaultText:"Teipiwch neges mewn amser go iawn",realTimeTextInputErrorMessage:"All y neges ddim bod yn fwy na 2000 o nodau",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"Mae RTT (testun amser real) wedi'i alluogi ar gyfer yr holl gyfranogwyr ar gyfer cyfnod cyfan y cyfarfod.",realTimeTextBannerLinkLabel:"Dysgu mwy",realTimeTextModalAriaLabel:"Galluogi’r moddol RTT",realTimeTextModalTitle:"Galluogi RTT?",realTimeTextModalText:"Bydd teipio mewn amser real (RTT) yn cael ei alluogi ar gyfer pawb yn y cyfarfod.",realTimeTextConfirmButtonLabel:"Galluogi",realTimeTextCancelButtonLabel:"Canslo",realTimeTextCloseModalButtonAriaLabel:"Cau'r Moddol RTT",captionsOnlyContainerTitle:"Capsiynau",realTimeTextOnlyContainerTitle:"Testun Amser Real",captionsAndRealTimeTextContainerTitle:"Capsiynau a Thestun Amser Real",expandButtonAriaLabel:"Ehangu",minimizeButtonAriaLabel:"Crebachu",transferPageTransferorText:"Wrthi'n trosglwyddo...",transferPageTransferTargetText:"Wrthi'n cysylltu...",transferPageUnknownTransferorDisplayName:"Dim manylion",transferPageUnknownTransferTargetDisplayName:"Dim manylion",transferPageNoticeString:"Rydych chi'n cael eich trosglwyddo",participantCouldNotBeReachedTitle:"Dydy'r cyfranogwr targed ddim ar gael ar hyn o bryd",participantCouldNotBeReachedMoreDetails:"Cysylltwch â'r cyfranogwr pan fydd ar gael",permissionToReachTargetParticipantNotAllowedTitle:"Ni chaniateir caniatâd i gyrraedd y cyfranogwr targed",permissionToReachTargetParticipantNotAllowedMoreDetails:"Gwnewch yn siŵr bod y cyfranogwr targed yn yr un tenant",unableToResolveTenantTitle:"Does dim modd datrys id tenant ar gyfer y cyfranogwr targed",unableToResolveTenantMoreDetails:"Gwnewch yn siŵr bod yr id cyfranogwr wedi'i roi'n gywir",participantIdIsMalformedTitle:"Dydy'r id cyfranogwr ddim yn y fformat cywir",participantIdIsMalformedMoreDetails:"Gwnewch yn siŵr bod id y cyfranogwr yn y fformat cywir",moreButtonGalleryControlLabel:"Gweld",moreButtonGalleryPositionToggleLabel:"Symud yr oriel i'r brig",moreButtonGallerySpeakerLayoutLabel:"Seinydd",moreButtonGalleryFloatingLocalLayoutLabel:"Deinamig",moreButtonGalleryDefaultLayoutLabel:"Gwedd oriel",moreButtonGalleryFocusedContentLayoutLabel:"Canolbwyntio ar y cynnwys",moreButtonLargeGalleryDefaultLayoutLabel:"Oriel Fawr",moreButtonTogetherModeLayoutLabel:"Modd Ynghyd",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Mae eich camera wedi'i analluogi. Does dim modd i chi rannu fideo mwyach.",grantedDueToMeetingOption:"Mae eich camera wedi'i alluogi. Rhowch eich camera ar waith os hoffech chi rannu fideo."},unmuteMic:{lostDueToMeetingOption:"Mae eich meic wedi'i analluogi. Does dim modd i chi ddad-dewi mwyach.",grantedDueToMeetingOption:"Mae eich meic wedi cael ei alluogi. Dad-dewi’r meic os hoffech chi siarad."},shareScreen:{lostDueToRoleChangeToAttendee:"Mae eich rôl wedi cael ei newid. Fydd rhai gweithredoedd, fel rhannu cynnwys, ddim ar gael i chi.",grantedDueToRoleChangeToPresenter:"Rydych chi'n gyflwynydd. Gallwch rannu cynnwys a hwyluso'r cyfarfod."},hideAttendeeNames:{hideAttendeeNameAttendee:"Mae eich enw wedi'i guddio rhag mynychwyr eraill. Gall trefnwyr a chyflwynwyr weld eich enw go iawn.",hideAttendeeNamePresenter:"Mae enwau mynychwyr wedi'u cuddio. Dim ond trefnwyr a chyflwynwyr sy'n gallu gweld enwau i warchod preifatrwydd y mynychwr."}},surveyTitle:"Helpwch ni i wella",starSurveyHelperText:"Sut oedd ansawdd yr alwad?",starSurveyOneStarText:"Roedd yr ansawdd yn wael.",starSurveyTwoStarText:"Roedd yr ansawdd yn wael.",starSurveyThreeStarText:"Roedd yr ansawdd yn dda.",starSurveyFourStarText:"Roedd yr ansawdd yn wych.",starSurveyFiveStarText:"Roedd yr ansawdd yn ardderchog.",surveyConfirmButtonLabel:"Anfon Adborth",starRatingAriaLabel:"Dewis {0} o {1} seren",surveySkipButtonLabel:"Hepgor",tagsSurveyQuestion:"Beth allai fod yn well?",tagsSurveyTextFieldDefaultText:"Arall, nodwch os gwelwch yn dda",tagsSurveyHelperText:"Gwiriwch unrhyw broblemau a gawsoch chi",endOfSurveyText:"Diolch am eich adborth!",surveyIssues:{overallRating:{callCannotJoin:"Doeddwn i ddim yn gallu ymuno â'r alwad",callCannotInvite:"Doeddwn i ddim yn gallu gwahodd eraill i'r alwad",hadToRejoin:"Roedd yn rhaid i fi ailymuno â'r alwad",callEndedUnexpectedly:"Daeth yr alwad i ben ar fy nghyfer i yn annisgwyl",otherIssues:"Roeddwn i'n cael problemau eraill gyda'r alwad"},audioRating:{noLocalAudio:"Doedd yr ochr arall ddim yn gallu clywed dim byd",noRemoteAudio:"Doeddwn i ddim yn gallu clywed dim byd",echo:"Roeddwn i'n clywed adlais ar yr alwad",audioNoise:"Roeddwn i'n clywed synau ar yr alwad",lowVolume:"Roedd lefel y sain yn isel",audioStoppedUnexpectedly:"Daeth y sain i ben yn annisgwyl",distortedSpeech:"Roedd y sain wedi'i haflunio",audioInterruption:"Torrwyd ar draws y sain",otherIssues:"Roeddwn i'n cael problemau sain eraill yn yr alwad hon"},videoRating:{noVideoReceived:"Doeddwn i ddim yn gallu gweld unrhyw fideo",noVideoSent:"Doedd pobl eraill ddim yn gallu fy ngweld i",lowQuality:"Roedd ansawdd y fideo yn isel",freezes:"Mae'r fideo wedi rhewi",stoppedUnexpectedly:"Daeth y fideo i ben yn annisgwyl",darkVideoReceived:"Rydw i'n gweld dim ond sgriniau tywyll pan fo pobl eraill yn rhoi eu camerâu ar waith",audioVideoOutOfSync:"Doedd y Sain a'r Fideo ddim wedi'u cysoni",otherIssues:"Roeddwn i'n cael problemau fideo eraill yn yr alwad hon"},screenshareRating:{noContentLocal:"Doedd pobl eraill ddim yn gallu gweld fy rhannu sgrin",noContentRemote:"Doeddwn i ddim yn gallu gweld rhannu sgrin pobl eraill",cannotPresent:"Doedd dim modd i fi gyflwyno fy sgrin",lowQuality:"Roedd ansawdd rhannu sgrin yn isel",freezes:"Mae rhannu sgrin wedi rhewi",stoppedUnexpectedly:"Stopiodd rhannu sgrin yn annisgwyl",largeDelay:"Mae cryn dipyn o ran rhannu sgrin",otherIssues:"Roeddwn i'n cael problemau rhannu sgrin eraill yn yr alwad hon"}},surveyIssuesHeadingStrings:{overallRating:"Cyffredinol",audioRating:"Sain",videoRating:"Fideo",screenshareRating:"Cyflwyno"},spotlightPrompt:{startSpotlightHeading:"Sbotolau i bawb?",startSpotlightText:"Byddwch chi’n amlygu'r fideo hwn ar gyfer pawb yn y cyfarfod.",startSpotlightOnSelfText:"Byddwch chi’n amlygu eich fideo ar gyfer pawb yn y cyfarfod.",startSpotlightConfirmButtonLabel:"Sbotolau i bawb",startSpotlightCancelButtonLabel:"Canslo",stopSpotlightHeading:"Stopio rhoi sbotolau ar y fideo hwn?",stopSpotlightOnSelfHeading:"Gadael y sbotolau?",stopAllSpotlightHeading:"Stopio rhoi sbotolau ar yr holl fideos?",stopSpotlightText:"Ni fydd y fideo hwn yn cael ei amlygu ar gyfer pawb yn y cyfarfod mwyach.",stopSpotlightOnSelfText:"Fydd eich fideo ddim yn cael ei amlygu ar gyfer pawb yn y cyfarfod mwyach.",stopAllSpotlightText:"Ni fydd y fideos bellach yn cael eu hamlygu i bawb yn y cyfarfod.",stopSpotlightConfirmButtonLabel:"Diffodd y sbotolau",stopSpotlightOnSelfConfirmButtonLabel:"Gadael y sbotolau",stopSpotlightCancelButtonLabel:"Canslo",closeSpotlightPromptButtonLabel:"Cau"},exitSpotlightButtonLabel:"Gadael y sbotolau",exitSpotlightButtonTooltip:"Gadael y sbotolau",leaveConfirmButtonLabel:"Gadael",endCallConfirmButtonLabel:"Gorffen yr alwad",hangUpCancelButtonLabel:"Canslo",leaveConfirmDialogTitle:"Gadael yr alwad?",leaveConfirmDialogContent:"Ydych chi'n siŵr eich bod am adael yr alwad?",endCallConfirmDialogTitle:"Dod â'r alwad i ben?",endCallConfirmDialogContent:"Byddwch chi'n dod â'r alwad i ben ar gyfer pawb.",invalidMeetingIdentifier:"ID neu god cyfrin cyfarfod annilys",pinParticipantMenuLabel:"Pinio ar fy nghyfer i",pinParticipantLimitReachedMenuLabel:"Pinio (wedi cyrraedd y terfyn)",unpinParticipantMenuLabel:"Dadbinio",pinParticipantMenuItemAriaLabel:"Pinio ar fy nghyfer i",unpinParticipantMenuItemAriaLabel:"Dadbinio",muteAllMenuLabel:"Tewi pawb",muteAllDialogTitle:"Tewi pawb?",muteAllDialogContent:"Bydd hyn yn tewi pawb yn y cyfarfod ond chi.",muteAllConfirmButtonLabel:"Tewi pawb",muteAllCancelButtonLabel:"Canslo",joinBreakoutRoomButtonLabel:"Ymuno â'r cyfarfod ymylol",returnFromBreakoutRoomButtonLabel:"Gadael yr ystafell",leaveBreakoutRoomAndMeetingButtonLabel:"Gadael y cyfarfod",breakoutRoomJoinedNotificationTitle:"Rydych chi wedi ymuno â'r Cyfarfod Ymylol '{roomName}'",joinBreakoutRoomBannerTitle:"Mae'r cyfarfod ymylol '{roomName}' ar agor",joinBreakoutRoomBannerButtonLabel:"Ymuno",returnFromBreakoutRoomBannerTitle:"Mynd yn ôl i'r prif gyfarfod",returnFromBreakoutRoomBannerButtonLabel:"Ailymuno",forbidAudioMenuLabel:"Analluogi meic",permitAudioMenuLabel:"Caniatáu meic",forbidOthersAudioDialogTitle:"Analluogi meic ar gyfer mynychwyr?",forbidOthersAudioDialogContent:"Fydd dim modd i fynychwyr ddad-dewi eu hunain. Gall y trefnydd a'r cyflwynwyr ganiatáu i bobl ddad-dewi yn ôl yr angen.",forbidOthersAudioConfirmButtonLabel:"Analluogi meicroffonau",forbidOthersAudioCancelButtonLabel:"Canslo",permitOthersAudioDialogTitle:"Caniatáu meic ar gyfer mynychwyr?",permitOthersAudioDialogContent:"Bydd pawb yn y cyfarfod yn gallu dad-dewi eu hunain.",permitOthersAudioConfirmButtonLabel:"Caniatáu meicroffonau",permitOthersAudioCancelButtonLabel:"Canslo",forbidOthersAudioMenuLabel:"Analluogi meic ar gyfer mynychwyr",permitOthersAudioMenuLabel:"Caniatáu meic ar gyfer mynychwyr",forbidVideoMenuLabel:"Analluogi camera",permitVideoMenuLabel:"Caniatáu camera",forbidOthersVideoDialogTitle:"Analluogi camera ar gyfer mynychwyr?",forbidOthersVideoDialogContent:"Fydd dim modd i fynychwyr alluogi fideo eu hunain. Gall y trefnydd a'r cyflwynwyr ganiatáu i bobl alluogi camera yn ôl yr angen.",forbidOthersVideoConfirmButtonLabel:"Analluogi camerâu",forbidOthersVideoCancelButtonLabel:"Canslo",permitOthersVideoDialogTitle:"Caniatáu camera ar gyfer mynychwyr?",permitOthersVideoDialogContent:"Bydd pawb yn y cyfarfod yn gallu rhoi'r fideo eu hunain ar waith.",permitOthersVideoConfirmButtonLabel:"Caniatáu camerâu",permitOthersVideoCancelButtonLabel:"Canslo",forbidOthersVideoMenuLabel:"Analluogi camera ar gyfer mynychwyr",permitOthersVideoMenuLabel:"Caniatáu camera ar gyfer mynychwyr"};var chat$j={chatListHeader:"Yn y sgwrs hon",uploadAttachment:"Llwytho Atodiad i Fyny",uploadImageDataNotProvided:"Dydy data'r ddelwedd ddim wedi'i ddarparu.",uploadImageIsTooLarge:"Mae'r ddelwedd yn rhy fawr. Dewiswch un sy'n llai na {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Ni chaniateir llwytho'r ddelwedd .{imageExtension} i fyny.",uploadImageFailed:"Does dim modd llwytho'r ddelwedd i fyny. Rhowch gynnig arall arni rywbryd eto."};var callWithChat$j={chatButtonLabel:"Sgwrs",chatButtonNewMessageNotificationLabel:"Neges Newydd",chatButtonTooltipClosedWithMessageCount:"Dangos sgwrs ({unreadMessagesCount} heb ei darllen)",chatButtonTooltipClose:"Cuddio'r sgwrs",chatButtonTooltipOpen:"Dangos y sgwrs",chatPaneTitle:"Sgwrs",copyInviteLinkButtonLabel:"Copïo dolen y gwahoddiad",copyInviteLinkButtonActionedLabel:"Wedi copïo'r ddolen",copyInviteLinkActionedAriaLabel:"Wedi copïo dolen y gwahoddiad",dismissSidePaneButtonLabel:"Cau",moreDrawerAudioDeviceMenuTitle:"Dyfais Sain",moreDrawerButtonLabel:"Mwy o ddewisiadau",moreDrawerButtonTooltip:"Mwy o ddewisiadau",moreDrawerMicrophoneMenuTitle:"Meicroffon",moreDrawerSpeakerMenuTitle:"Siaradwr",moreDrawerCaptionsMenuTitle:"Capsiynau byw",moreDrawerSpokenLanguageMenuTitle:"Iaith lafar",moreDrawerCaptionLanguageMenuTitle:"Iaith capsiwn",moreDrawerGalleryOptionsMenuTitle:"Dewisiadau oriel",peopleButtonLabel:"Pobl",selectedPeopleButtonLabel:"Wedi Dewis y Botwm Pobl",peopleButtonTooltipOpen:"Dangos y cyfranogwyr",peopleButtonTooltipClose:"Cuddio’r cyfranogwyr",peopleButtonTooltipOpenAriaLabel:"Botwm Pobl, Dangos cyfranogwyr",peopleButtonTooltipCloseAriaLabel:"Botwm Pobl, Cuddio cyfranogwyr",peoplePaneSubTitle:"Yn yr alwad hon",peoplePaneTitle:"Pobl",pictureInPictureTileAriaLabel:"Crynodebau Fideo. Cliciwch i fynd yn ôl i sgrin yr alwad.",removeMenuLabel:"Tynnu",openDialpadButtonLabel:"Deialu rhif ffôn",returnToCallButtonAriaDescription:"Mynd yn ôl i'r Alwad",returnToCallButtonAriaLabel:"Yn ôl",peoplePaneAddPeopleButtonLabel:"Ychwanegu Pobl",dialpadStartCallButtonLabel:"Galwad",dialpadModalTitle:"Deialu Rhif Ffôn",dialpadModalAriaLabel:"Pad deialu",dialpadCloseModalButtonAriaLabel:"Cau'r pad deialu",openDtmfDialpadLabel:"Dangos y pad deialu",dtmfDialpadPlaceholderText:"Cofnodi rhif",breakoutRoomChatPaneTitle:"Sgwrs y Cyfarfod Ymylol",chatContentSpinnerLabel:"Wrthi'n llwytho'r sgwrs"};var cy_GB = {call:call$j,chat:chat$j,callWithChat:callWithChat$j};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Welsh (GB)
 *
 * @public
 */
const COMPOSITE_LOCALE_CY_GB = {
    component: COMPONENT_LOCALE_CY_GB,
    strings: createCompositeStrings(cy_GB)
};

var call$i={cameraLabel:"Kamera",noCamerasLabel:"Keine Kameras gefunden",cameraPermissionDenied:"Ihr Browser blockiert den Zugriff auf Ihre Kamera.",cameraTurnedOff:"Ihre Kamera ist deaktiviert.",chatButtonLabel:"Chat",close:"Schließen",complianceBannerNowOnlyRecording:"Sie zeichnen nun nur diese Besprechung auf.",complianceBannerNowOnlyTranscription:"Sie transkribieren diese Besprechung jetzt nur noch.",complianceBannerRecordingAndTranscriptionSaved:"Aufzeichnung und Transkription werden gespeichert.",complianceBannerRecordingAndTranscriptionStarted:"Die Aufzeichnung und Transkription haben begonnen.",complianceBannerRecordingAndTranscriptionStopped:"Die Aufzeichnung und Transkription wurden beendet.",complianceBannerRecordingSaving:"Aufzeichnung wird gespeichert.",complianceBannerRecordingStarted:"Die Aufzeichnung wurde gestartet.",complianceBannerRecordingStopped:"Aufzeichnung wurde beendet.",complianceBannerTranscriptionStarted:"Die Transkription hat begonnen.",complianceBannerTranscriptionConsent:"Durch Ihre Teilnahme stimmen Sie zu, dass diese Besprechung transkribiert wird.",complianceBannerTranscriptionSaving:"Die Transkription wird gespeichert.",complianceBannerTranscriptionStopped:"Die Transkription wurde beendet.",configurationPageTitle:"Einen Anruf beginnen",copyInviteLinkButtonLabel:"Einladungslink kopieren",copyInviteLinkButtonActionedLabel:"Link kopiert",copyInviteLinkActionedAriaLabel:"Gruppeneinladungslink kopiert",defaultPlaceHolder:"Option auswählen",dismissSidePaneButtonLabel:"Schließen",videoEffectsPaneTitle:"Effekte",videoEffectsPaneBackgroundSelectionTitle:"Hintergrund",videoEffectsPaneAriaLabel:"Bereich \"Videoeffekte\"",configurationPageCameraIsLoadingLabel:"Video wird geladen...",configurationPageVideoEffectsButtonLabel:"Effekte",unableToStartVideoEffect:"Videoeffekt kann nicht angewendet werden.",blurBackgroundEffectButtonLabel:"Unscharf machen",blurBackgroundTooltip:"Hintergrund unscharf machen",removeBackgroundEffectButtonLabel:"Keine",removeBackgroundTooltip:"Hintergrund entfernen",cameraOffBackgroundEffectWarningText:"Ihre Kamera ist ausgeschaltet. Aktivieren Sie die Kamera, um den Videoeffekt anzuzeigen.",failedToJoinCallDueToNoNetworkMoreDetails:"Der Anruf wurde aufgrund eines Netzwerkproblems getrennt. Überprüfen Sie Ihre Verbindung, und treten Sie erneut bei.",failedToJoinCallDueToNoNetworkTitle:"Anruf unterbrochen",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Ihnen wurde keine Teilnahme am Anruf gewährt. Wenn dies ein Fehler war, treten Sie dem Anruf erneut bei.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Aus Wartebereich entlassen",learnMore:"Weitere Informationen",leavingCallTitle:"Verlässt Besprechung...",leftCallMoreDetails:"Wenn dies ein Fehler war, nehmen Sie erneut am Anruf teil.",leftCallTitle:"Sie haben den Anruf verlassen",lobbyScreenConnectingToCallTitle:"Teilnehmen am Anruf",lobbyScreenWaitingToBeAdmittedTitle:"Warten auf die Zulassung",microphonePermissionDenied:"Ihr Browser blockiert den Zugriff auf Ihr Mikrofon.",microphoneToggleInLobbyNotAllowed:"Stummschalten oder Aufheben der Stummschaltung im Wartebereich nicht zulässig.",mutedMessage:"Sie sind stummgeschaltet.",networkReconnectMoreDetails:"Offensichtlich ist etwas schief gegangen. Wir versuchen, Sie wieder mit dem Anruf zu verbinden.",networkReconnectTitle:"Einen Augenblick bitte",notInvitedToRoomDetails:"Sie können diesem Raum ohne Einladung nicht beitreten.",notInvitedToRoomTitle:"Nicht zum Raum eingeladen",peopleButtonLabel:"Kontakte",selectedPeopleButtonLabel:"Schaltfläche \"Personen\" ausgewählt",peoplePaneTitle:"Kontakte",peoplePaneMoreButtonAriaLabel:"Mehr",peopleButtonTooltipOpen:"Teilnehmende anzeigen",peopleButtonTooltipClose:"Teilnehmende ausblenden",peoplePaneSubTitle:"In diesem Anruf {numberOfPeople}",privacyPolicy:"Datenschutzrichtlinie",rejoinCallButtonLabel:"Am Anruf erneut teilnehmen",removedFromCallMoreDetails:"Ein anderer Teilnehmer hat Sie aus dem Anruf entfernt.",removedFromCallTitle:"Sie wurden entfernt",removeMenuLabel:"Entfernen",startSpotlightMenuLabel:"Spotlight für alle",addSpotlightMenuLabel:"Spotlight hinzufügen",stopSpotlightMenuLabel:"Spotlight beenden",stopSpotlightOnSelfMenuLabel:"Spotlight beenden",spotlightLimitReachedMenuTitle:"Spotlight-Limit erreicht",stopAllSpotlightMenuLabel:"Alle Spotlights beenden",returnToCallButtonAriaDescription:"Zurück zum Anruf",returnToCallButtonAriaLabel:"Zurück",roomNotFoundDetails:"Die angegebene Raum-ID wurde nicht gefunden.",roomNotFoundTitle:"Raum nicht gefunden",roomNotValidDetails:"Dieser Raum ist zurzeit nicht gültig.",roomNotValidTitle:"Raum ungültig",inviteToRoomRemovedDetails:"Ihre Einladung, diesem Raum beizutreten, wurde entfernt.",inviteToRoomRemovedTitle:"Einladung zum Raum entfernt",soundLabel:"Sound",noMicrophonesLabel:"Keine Mikrofone gefunden",noSpeakersLabel:"Keine Lautsprecher gefunden",startCallButtonLabel:"Anruf beginnen",openDialpadButtonLabel:"Telefonnummer wählen",peoplePaneAddPeopleButtonLabel:"Personen hinzufügen",dialpadStartCallButtonLabel:"Anrufen",dialpadModalTitle:"Telefonnummer wählen",dialpadModalAriaLabel:"Wähltastatur",dialpadCloseModalButtonAriaLabel:"Wähltastatur schließen",moreButtonCallingLabel:"Mehr",resumeCallButtonLabel:"Fortsetzen",resumingCallButtonLabel:"Wird fortgesetzt…",resumeCallButtonAriaLabel:"Anruf fortsetzen",resumingCallButtonAriaLabel:"Anruf fortsetzen",holdScreenLabel:"Sie sind in der Warteschleife.",openDtmfDialpadLabel:"Wähltastatur anzeigen",dtmfDialpadPlaceholderText:"Nummer eingeben",outboundCallingNoticeString:"Anrufen...",participantJoinedNoticeString:"{displayName} ist beigetreten.",twoParticipantJoinedNoticeString:"{displayName1} und {displayName2} sind beigetreten.",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} und {displayName3} sind beigetreten.",participantLeftNoticeString:"{displayName} ist gegangen.",twoParticipantLeftNoticeString:"{displayName1} und {displayName2} sind gegangen.",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} und {displayName3} sind gegangen.",unnamedParticipantString:"unbenannter Teilnehmer",manyUnnamedParticipantsJoined:"unbenannter Teilnehmer und {numOfParticipants} weitere Teilnehmer sind beigetreten.",manyUnnamedParticipantsLeft:"unbenannter Teilnehmer und {numOfParticipants} weitere Teilnehmer sind gegangen.",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} und {numOfParticipants} anderen Teilnehmern sind beigetreten.",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} und {numOfParticipants} anderen Teilnehmer verlassen",liveCaptionsLabel:"Liveuntertitel",captionsSettingsLabel:"Untertiteleinstellungen",startCaptionsButtonOnLabel:"Beschriftungen aktivieren",startCaptionsButtonOffLabel:"Untertitel deaktivieren",startCaptionsButtonTooltipOnContent:"Untertitel deaktivieren",startCaptionsButtonTooltipOffContent:"Beschriftungen aktivieren",captionsSettingsModalTitle:"Einstellungen für Untertitel",captionsSettingsSpokenLanguageDropdownLabel:"Gesprochene Sprache",captionsSettingsCaptionLanguageDropdownLabel:"Sprache der Untertitel",captionsSettingsSpokenLanguageDropdownInfoText:"Sprache, die jeder in diesem Anruf spricht.",captionsSettingsCaptionLanguageDropdownInfoText:"Untertitel werden in dieser Sprache angezeigt.",captionsSettingsConfirmButtonLabel:"Bestätigen",captionsSettingsCancelButtonLabel:"Abbrechen",captionsSettingsModalAriaLabel:"Untertiteleinstellung modal",captionsSettingsCloseModalButtonAriaLabel:"Einstellung „Untertitel“ schließen",captionsBannerMoreButtonCallingLabel:"Untertiteleinstellungen – Weitere Optionen",captionsBannerMoreButtonTooltip:"Weitere Optionen",realTimeTextLabel:"Echtzeittext (RTT)",startRealTimeTextLabel:"RTT für diesen Anruf aktivieren",disabledStartRealTimeTextLabel:"RTT ist zurzeit aktiviert und kann nicht ausgeschaltet werden.",dismissModalAriaLabel:"Lokales Bild und Remotebild. Drücken Sie die EINGABETASTE, um zum Anruf zurückzukehren.",callRejectedTitle:"Der Anruf konnte nicht ausgeführt werden.",callRejectedMoreDetails:"Die Person, die Sie erreichen möchten, ist nicht verfügbar. Versuchen Sie es später noch mal.",callTimeoutTitle:"Der Anruf konnte nicht ausgeführt werden.",callTimeoutDetails:"Die Person, die Sie erreichen möchten, ist nicht verfügbar. Versuchen Sie es später noch mal.",callTimeoutBotTitle:"Der Anruf konnte nicht ausgeführt werden.",callTimeoutBotDetails:"Der Dienst, den Sie erreichen möchten, ist nicht verfügbar. Versuchen Sie es später noch mal.",dtmfDialerButtonLabel:"Wähltastatur",dtmfDialerButtonTooltipOn:"Wähltastatur anzeigen",dtmfDialerButtonTooltipOff:"Wähltastatur ausblenden",dtmfDialerMoreButtonLabelOn:"Wähltastatur anzeigen",dtmfDialerMoreButtonLabelOff:"Wähltastatur ausblenden",phoneCallMoreButtonLabel:"Telefonanruf",spokenLanguageStrings:{"ar-ae":"Arabisch – Vereinigte Arabische Emirate","ar-sa":"Arabisch – Saudi-Arabien","da-dk":"Dänisch","de-de":"Deutsch – Deutschland","en-au":"Englisch – Australien","en-ca":"Englisch – Kanada","en-gb":"Englisch - Vereinigtes Königreich","en-in":"Englisch – Indien","en-nz":"Englisch – Neuseeland","en-us":"Englisch – USA","es-es":"Spanisch – Spanien (Moderne Sortierung)","es-mx":"Spanisch – Mexiko","fi-fi":"Finnisch","fr-ca":"Französisch (Kanada)","fr-fr":"Französisch (Frankreich)","hi-in":"Hindi","it-it":"Italienisch – Italien","ja-jp":"Japanisch","ko-kr":"Koreanisch","nb-no":"Norwegisch (Bokmål)","nl-be":"Niederländisch – Belgien","nl-nl":"Niederländisch – Niederlande","pl-pl":"Polnisch","pt-br":"Portugiesisch (Brasilien)","ru-ru":"Russisch","sv-se":"Schwedisch","zh-cn":"Chinesisch – Volksrepublik China","zh-hk":"Chinesisch – Hongkong (SAR)","cs-cz":"Tschechisch","pt-pt":"Portugiesisch (Portugal)","tr-tr":"Türkisch","vi-vn":"Vietnamesisch","th-th":"Thailändisch","he-il":"Hebräisch","cy-gb":"Walisisch","uk-ua":"Ukrainisch","el-gr":"Griechisch","hu-hu":"Ungarisch","ro-ro":"Rumänisch","sk-sk":"Slowakisch","zh-tw":"Chinesisch – Taiwan"},captionLanguageStrings:{ar:"Arabisch",da:"Dänisch",de:"Deutsch",en:"Englisch",es:"Spanisch",fi:"Finnisch","fr-ca":"Französisch – Kanada",fr:"Französisch – Frankreich",hi:"Hindi",it:"Italienisch",ja:"Japanisch",ko:"Koreanisch",nb:"Norwegisch (Bokmål)",nl:"Niederländisch",pl:"Polnisch",pt:"Portugiesisch – Brasilien",ru:"Russisch",sv:"Schwedisch","zh-Hans":"Chinesisch (vereinfacht)","zh-Hant":"Chinesisch (traditionell)",cs:"Tschechisch","pt-pt":"Portugiesisch – Portugal",tr:"Türkisch",vi:"Vietnamesisch",th:"Thailändisch",he:"Hebräisch",cy:"Walisisch",uk:"Ukrainisch",el:"Griechisch",hu:"Ungarisch",ro:"Rumänisch",sk:"Slowakisch"},captionsBannerSpinnerText:"Untertitel werden gestartet...",realTimeTextInputBoxDefaultText:"Nachricht in Echtzeit eingeben",realTimeTextInputErrorMessage:"Die Nachricht darf 2000 Zeichen nicht überschreiten.",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (Echtzeittext) ist für alle Teilnehmer für die gesamte Dauer der Besprechung aktiviert.",realTimeTextBannerLinkLabel:"Weitere Informationen",realTimeTextModalAriaLabel:"RTT Modal aktivieren",realTimeTextModalTitle:"RTT aktivieren?",realTimeTextModalText:"Echtzeittext (RTT) wird für alle Teilnehmer der Besprechung aktiviert.",realTimeTextConfirmButtonLabel:"Aktivieren",realTimeTextCancelButtonLabel:"Abbrechen",realTimeTextCloseModalButtonAriaLabel:"RTT Modal schließen",captionsOnlyContainerTitle:"Untertitel",realTimeTextOnlyContainerTitle:"Echtzeittext",captionsAndRealTimeTextContainerTitle:"Untertitel und Echtzeittext",expandButtonAriaLabel:"Erweitern",minimizeButtonAriaLabel:"Reduzieren",transferPageTransferorText:"Übertragung...",transferPageTransferTargetText:"Verbindung wird hergestellt...",transferPageUnknownTransferorDisplayName:"Unbekannt",transferPageUnknownTransferTargetDisplayName:"Unbekannt",transferPageNoticeString:"Sie werden versetzt",participantCouldNotBeReachedTitle:"Der Zielteilnehmer ist zurzeit nicht verfügbar",participantCouldNotBeReachedMoreDetails:"Wenden Sie sich an den Teilnehmer, wenn er verfügbar ist",permissionToReachTargetParticipantNotAllowedTitle:"Die Berechtigung zum Erreichen des Zielteilnehmers ist nicht zulässig",permissionToReachTargetParticipantNotAllowedMoreDetails:"Überprüfen Sie, ob sich der Zielteilnehmer im selben Mandanten befindet",unableToResolveTenantTitle:"Die Mandanten-ID für den Zielteilnehmer kann nicht aufgelöst werden",unableToResolveTenantMoreDetails:"Überprüfen Sie, ob die Teilnehmer-ID richtig eingegeben wurde.",participantIdIsMalformedTitle:"Die Teilnehmer-ID weist nicht das richtige Format auf",participantIdIsMalformedMoreDetails:"Überprüfen Sie, ob die Teilnehmer-ID das richtige Format aufweist.",moreButtonGalleryControlLabel:"Anzeigen",moreButtonGalleryPositionToggleLabel:"Katalog nach oben verschieben",moreButtonGallerySpeakerLayoutLabel:"Lautsprecher",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamisch",moreButtonGalleryDefaultLayoutLabel:"Profilfotoansicht",moreButtonGalleryFocusedContentLayoutLabel:"Fokus auf Inhalte",moreButtonLargeGalleryDefaultLayoutLabel:"Große Galerie",moreButtonTogetherModeLayoutLabel:"Zusammen-Modus",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Ihre Kamera wurde deaktiviert. Sie können kein Video mehr freigeben.",grantedDueToMeetingOption:"Ihre Kamera wurde aktiviert. Aktivieren Sie diese Option, wenn Sie Video freigeben möchten."},unmuteMic:{lostDueToMeetingOption:"Ihr Mikro wurde deaktiviert. Sie können die Stummschaltung nicht mehr aufheben.",grantedDueToMeetingOption:"Ihr Mikro wurde aktiviert. Heben Sie die Stummschaltung auf, wenn Sie sprechen möchten."},shareScreen:{lostDueToRoleChangeToAttendee:"Ihre Rolle wurde geändert. Einige Aktionen, z. B. das Teilen von Inhalten, stehen Ihnen nicht zur Verfügung.",grantedDueToRoleChangeToPresenter:"Sie sind Referent. Sie können Inhalte teilen und die Besprechung unterstützen."},hideAttendeeNames:{hideAttendeeNameAttendee:"Ihr Name ist für andere Teilnehmer ausgeblendet. Organisatoren und Referenten können Ihren richtigen Namen sehen.",hideAttendeeNamePresenter:"Teilnehmernamen sind ausgeblendet. Nur Organisatoren und Referenten können Namen sehen, um die Privatsphäre der Teilnehmer zu schützen."}},surveyTitle:"Helfen Sie uns, besser zu werden.",starSurveyHelperText:"Wie war die Qualität des Anrufs?",starSurveyOneStarText:"Die Qualität war schlecht.",starSurveyTwoStarText:"Die Qualität war schlecht.",starSurveyThreeStarText:"Die Qualität war gut.",starSurveyFourStarText:"Die Qualität war toll.",starSurveyFiveStarText:"Die Qualität war ausgezeichnet.",surveyConfirmButtonLabel:"Feedback senden",starRatingAriaLabel:"{0} von {1} Sternen auswählen",surveySkipButtonLabel:"Überspringen",tagsSurveyQuestion:"Was hätte besser sein können?",tagsSurveyTextFieldDefaultText:"Sonstiges, bitte angeben",tagsSurveyHelperText:"Überprüfen Sie alle aufgetretenen Probleme",endOfSurveyText:"Vielen Dank für Ihr Feedback!",surveyIssues:{overallRating:{callCannotJoin:"Ich konnte dem Anruf nicht beitreten",callCannotInvite:"Ich konnte andere nicht zum Anruf einladen",hadToRejoin:"Ich musste dem Anruf erneut beitreten",callEndedUnexpectedly:"Anruf für mich unerwartet beendet",otherIssues:"Ich hatte andere Probleme mit dem Anruf"},audioRating:{noLocalAudio:"Die andere Seite konnte keinen Ton hören",noRemoteAudio:"Ich konnte keinen Ton hören",echo:"Ich habe Echos beim Anruf gehört",audioNoise:"Ich habe Rauschen beim Anruf gehört",lowVolume:"Die Lautstärke war niedrig",audioStoppedUnexpectedly:"Audio wurde unerwartet beendet",distortedSpeech:"Audio war verzerrt",audioInterruption:"Audio wurde unterbrochen",otherIssues:"In diesem Anruf gab es andere Audioprobleme"},videoRating:{noVideoReceived:"Ich konnte kein Video sehen.",noVideoSent:"Andere konnten mich nicht sehen",lowQuality:"Die Videoqualität war niedrig",freezes:"Video gefroren",stoppedUnexpectedly:"Video unerwartet beendet",darkVideoReceived:"Ich kann nur dunkle Bildschirme sehen, wenn andere ihre Kamera einschalten",audioVideoOutOfSync:"Audio und Video waren nicht synchronisiert",otherIssues:"In diesem Anruf gab es andere Videoprobleme"},screenshareRating:{noContentLocal:"Andere Personen konnten meine Bildschirmfreigabe nicht sehen",noContentRemote:"Ich konnte die Bildschirmfreigabe anderer Personen nicht sehen",cannotPresent:"Ich konnte meinen Bildschirm nicht präsentieren",lowQuality:"Qualität der Bildschirmfreigabe war niedrig",freezes:"Bildschirmfreigabe gefroren",stoppedUnexpectedly:"Bildschirmfreigabe unerwartet beendet",largeDelay:"Die Bildschirmfreigabe hat eine große Verzögerung",otherIssues:"Ich hatte andere Probleme mit der Bildschirmfreigabe in diesem Anruf."}},surveyIssuesHeadingStrings:{overallRating:"Insgesamt",audioRating:"Audio",videoRating:"Video",screenshareRating:"Hält Präsentation"},spotlightPrompt:{startSpotlightHeading:"Spotlight für alle?",startSpotlightText:"Sie heben dieses Video für alle Teilnehmer der Besprechung hervor.",startSpotlightOnSelfText:"Sie setzen Ihr Video für alle Teilnehmer der Besprechung ins Spotlight.",startSpotlightConfirmButtonLabel:"Spotlight für alle",startSpotlightCancelButtonLabel:"Abbrechen",stopSpotlightHeading:"Dieses Video aus dem Spotlight nehmen?",stopSpotlightOnSelfHeading:"Spotlight verlassen?",stopAllSpotlightHeading:"Spotlighting aller Videos beenden?",stopSpotlightText:"Dieses Video wird nicht mehr für alle Teilnehmer der Besprechung hervorgehoben.",stopSpotlightOnSelfText:"Ihr Video wird nicht mehr für alle Teilnehmer der Besprechung hervor gehoben.",stopAllSpotlightText:"Die Videos werden nicht mehr für alle Teilnehmer der Besprechung hervorgehoben.",stopSpotlightConfirmButtonLabel:"Spotlight beenden",stopSpotlightOnSelfConfirmButtonLabel:"Spotlight beenden",stopSpotlightCancelButtonLabel:"Abbrechen",closeSpotlightPromptButtonLabel:"Schließen"},exitSpotlightButtonLabel:"Spotlight beenden",exitSpotlightButtonTooltip:"Spotlight beenden",leaveConfirmButtonLabel:"Beenden",endCallConfirmButtonLabel:"Gespräch beenden",hangUpCancelButtonLabel:"Abbrechen",leaveConfirmDialogTitle:"Gespräch beenden?",leaveConfirmDialogContent:"Möchten Sie den Anruf wirklich verlassen?",endCallConfirmDialogTitle:"Gespräch beenden?",endCallConfirmDialogContent:"Sie beenden das Gespräch für alle.",invalidMeetingIdentifier:"Ungültige Besprechungs-ID oder -Kennung",pinParticipantMenuLabel:"Für mich anheften",pinParticipantLimitReachedMenuLabel:"Anheften (Limit erreicht)",unpinParticipantMenuLabel:"Lösen",pinParticipantMenuItemAriaLabel:"Für mich anheften",unpinParticipantMenuItemAriaLabel:"Lösen",muteAllMenuLabel:"Alle stummschalten",muteAllDialogTitle:"Alle stummschalten?",muteAllDialogContent:"Hierdurch werden alle in der Besprechung stummgeschaltet, außer Ihnen.",muteAllConfirmButtonLabel:"Alle stummschalten",muteAllCancelButtonLabel:"Abbrechen",joinBreakoutRoomButtonLabel:"Breakout-Room beitreten",returnFromBreakoutRoomButtonLabel:"Raum verlassen",leaveBreakoutRoomAndMeetingButtonLabel:"Besprechung verlassen",breakoutRoomJoinedNotificationTitle:"Sie sind dem Gruppenraum „{roomName}“ beigetreten.",joinBreakoutRoomBannerTitle:"Der Gruppenraum „{roomName}“ ist geöffnet.",joinBreakoutRoomBannerButtonLabel:"Beitreten",returnFromBreakoutRoomBannerTitle:"Zurück zur Hauptbesprechung",returnFromBreakoutRoomBannerButtonLabel:"Erneut teilnehmen",forbidAudioMenuLabel:"Mikro deaktivieren",permitAudioMenuLabel:"Mikrofon zulassen",forbidOthersAudioDialogTitle:"Mikrofon für Teilnehmer deaktivieren?",forbidOthersAudioDialogContent:"Die Teilnehmer können ihre Stummschaltung nicht selber aufheben. Der Organisator und die Referenten können Personen bei Bedarf erlauben, die Stummschaltung aufzuheben.",forbidOthersAudioConfirmButtonLabel:"Mikrofone deaktivieren",forbidOthersAudioCancelButtonLabel:"Abbrechen",permitOthersAudioDialogTitle:"Mikrofon für Teilnehmer zulassen?",permitOthersAudioDialogContent:"Jeder Besprechungsteilnehmer kann seine Stummschaltung aufheben.",permitOthersAudioConfirmButtonLabel:"Mikrofone zulassen",permitOthersAudioCancelButtonLabel:"Abbrechen",forbidOthersAudioMenuLabel:"Mikrofone für Teilnehmer deaktivieren",permitOthersAudioMenuLabel:"Mikrofon für Teilnehmer zulassen",forbidVideoMenuLabel:"Kamera deaktivieren",permitVideoMenuLabel:"Kamera zulassen",forbidOthersVideoDialogTitle:"Kamera für Teilnehmer deaktivieren?",forbidOthersVideoDialogContent:"Teilnehmer können Video nicht selbst aktivieren. Der Organisator und die Referenten können Personen die Kamera nach Bedarf einschalten lassen.",forbidOthersVideoConfirmButtonLabel:"Kameras deaktivieren",forbidOthersVideoCancelButtonLabel:"Abbrechen",permitOthersVideoDialogTitle:"Kamera für Teilnehmer zulassen?",permitOthersVideoDialogContent:"Jeder in der Besprechung kann Video selbst aktivieren.",permitOthersVideoConfirmButtonLabel:"Kameras zulassen",permitOthersVideoCancelButtonLabel:"Abbrechen",forbidOthersVideoMenuLabel:"Kamera für Teilnehmer deaktivieren",permitOthersVideoMenuLabel:"Kamera für Teilnehmer zulassen"};var chat$i={chatListHeader:"In diesem Chat",uploadAttachment:"Anlage hochladen",uploadImageDataNotProvided:"Bilddaten werden nicht bereitgestellt.",uploadImageIsTooLarge:"Das Bild ist zu groß. Wählen Sie einen Wert aus, der kleiner als {maxImageSize} MB ist.",uploadImageExtensionIsNotAllowed:"Das Hochladen von .{imageExtension} Image ist nicht zulässig.",uploadImageFailed:"Das Bild kann nicht hochgeladen werden. Versuchen Sie es später noch mal."};var callWithChat$i={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"Neue Nachricht",chatButtonTooltipClosedWithMessageCount:"Chat anzeigen ({unreadMessagesCount} ungelesen)",chatButtonTooltipClose:"Chat ausblenden",chatButtonTooltipOpen:"Chat anzeigen",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Einladungslink kopieren",copyInviteLinkButtonActionedLabel:"Link kopiert",copyInviteLinkActionedAriaLabel:"Gruppeneinladungslink kopiert",dismissSidePaneButtonLabel:"Schließen",moreDrawerAudioDeviceMenuTitle:"Audiogerät",moreDrawerButtonLabel:"Weitere Optionen",moreDrawerButtonTooltip:"Weitere Optionen",moreDrawerMicrophoneMenuTitle:"Mikrofon",moreDrawerSpeakerMenuTitle:"Lautsprecher",moreDrawerCaptionsMenuTitle:"Liveuntertitel",moreDrawerSpokenLanguageMenuTitle:"Gesprochene Sprache",moreDrawerCaptionLanguageMenuTitle:"Sprache für Untertitel",moreDrawerGalleryOptionsMenuTitle:"Katalogoptionen",peopleButtonLabel:"Personen",selectedPeopleButtonLabel:"Schaltfläche \"Personen\" ausgewählt",peopleButtonTooltipOpen:"Teilnehmende anzeigen",peopleButtonTooltipClose:"Teilnehmende ausblenden",peopleButtonTooltipOpenAriaLabel:"Schaltfläche „Personen“, Teilnehmer anzeigen",peopleButtonTooltipCloseAriaLabel:"Schaltfläche „Personen“, Teilnehmer ausblenden",peoplePaneSubTitle:"In diesem Anruf",peoplePaneTitle:"Personen",pictureInPictureTileAriaLabel:"Videofeeds. Klicken Sie hier, um zum Anrufbildschirm zurückzukehren.",removeMenuLabel:"Entfernen",openDialpadButtonLabel:"Telefonnummer wählen",returnToCallButtonAriaDescription:"Zurück zum Anruf",returnToCallButtonAriaLabel:"Zurück",peoplePaneAddPeopleButtonLabel:"Personen hinzufügen",dialpadStartCallButtonLabel:"Anrufen",dialpadModalTitle:"Telefonnummer wählen",dialpadModalAriaLabel:"Wähltastatur",dialpadCloseModalButtonAriaLabel:"Wähltastatur schließen",openDtmfDialpadLabel:"Wähltastatur anzeigen",dtmfDialpadPlaceholderText:"Nummer eingeben",breakoutRoomChatPaneTitle:"Chat im Gruppenraum",chatContentSpinnerLabel:"Chat wird geladen"};var de_DE = {call:call$i,chat:chat$i,callWithChat:callWithChat$i};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for German (Germany)
 *
 * @public
 */
const COMPOSITE_LOCALE_DE_DE = {
    component: COMPONENT_LOCALE_DE_DE,
    strings: createCompositeStrings(de_DE)
};

var call$h={cameraLabel:"Cámara",noCamerasLabel:"No se han encontrado cámaras",cameraPermissionDenied:"El explorador está bloqueando el acceso a la cámara",cameraTurnedOff:"La cámara está apagada",chatButtonLabel:"Chat",close:"Cerrar",complianceBannerNowOnlyRecording:"Solo está grabando esta reunión.",complianceBannerNowOnlyTranscription:"Solo está transcribiendo esta reunión.",complianceBannerRecordingAndTranscriptionSaved:"Se están guardando la grabación y la transcripción.",complianceBannerRecordingAndTranscriptionStarted:"Se han iniciado la grabación y la transcripción.",complianceBannerRecordingAndTranscriptionStopped:"Se han detenido la grabación y la transcripción.",complianceBannerRecordingSaving:"La grabación se está guardando.",complianceBannerRecordingStarted:"Se ha iniciado la grabación.",complianceBannerRecordingStopped:"La grabación se ha detenido.",complianceBannerTranscriptionStarted:"La transcripción se ha iniciado.",complianceBannerTranscriptionConsent:"Al unirse, da su consentimiento para que se transcriba esta reunión.",complianceBannerTranscriptionSaving:"Se está guardando la transcripción.",complianceBannerTranscriptionStopped:"La transcripción se ha detenido.",configurationPageTitle:"Iniciar una llamada",copyInviteLinkButtonLabel:"Copiar vínculo de invitación",copyInviteLinkButtonActionedLabel:"Vínculo copiado",copyInviteLinkActionedAriaLabel:"Vínculo de invitación copiado",defaultPlaceHolder:"Seleccionar una opción",dismissSidePaneButtonLabel:"Cerrar",videoEffectsPaneTitle:"Efectos",videoEffectsPaneBackgroundSelectionTitle:"Fondo",videoEffectsPaneAriaLabel:"Panel de efectos de vídeo",configurationPageCameraIsLoadingLabel:"El vídeo se está cargando...",configurationPageVideoEffectsButtonLabel:"Efectos",unableToStartVideoEffect:"No se puede aplicar el efecto de vídeo.",blurBackgroundEffectButtonLabel:"Desenfoque",blurBackgroundTooltip:"Desenfocar fondo",removeBackgroundEffectButtonLabel:"Ninguno",removeBackgroundTooltip:"Quitar fondo",cameraOffBackgroundEffectWarningText:"La cámara está apagada. Enciende la cámara para ver el efecto de vídeo.",failedToJoinCallDueToNoNetworkMoreDetails:"Se desconectó la llamada debido a un problema de red. Compruebe la conexión y únase de nuevo.",failedToJoinCallDueToNoNetworkTitle:"Llamada desconectada",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"No se te concedió la entrada a la llamada. Si se trata de un error, vuelve a unirte.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Rechazado de la sala de espera",learnMore:"Más información",leavingCallTitle:"Saliendo...",leftCallMoreDetails:"Si se trata de un error, vuelve a unirte a la llamada.",leftCallTitle:"Has abandonado la llamada",lobbyScreenConnectingToCallTitle:"Uniéndose a la llamada",lobbyScreenWaitingToBeAdmittedTitle:"Esperando a ser admitido",microphonePermissionDenied:"El explorador está bloqueando el acceso al micrófono",microphoneToggleInLobbyNotAllowed:"No se puede silenciar ni reactivar el audio mientras se está en la sala de espera.",mutedMessage:"Estás silenciado",networkReconnectMoreDetails:"Parece que hubo un problema. Estamos intentando reconectar la llamada.",networkReconnectTitle:"Espere",notInvitedToRoomDetails:"No puede unirse a esta sala porque no tiene una invitación.",notInvitedToRoomTitle:"No invitado a la sala",peopleButtonLabel:"Contactos",selectedPeopleButtonLabel:"Se seleccionó el botón Contactos",peoplePaneTitle:"Contactos",peoplePaneMoreButtonAriaLabel:"Más",peopleButtonTooltipOpen:"Mostrar participantes",peopleButtonTooltipClose:"Ocultar participantes",peoplePaneSubTitle:"En esta llamada {numberOfPeople}",privacyPolicy:"Directiva de privacidad",rejoinCallButtonLabel:"Volver a unirse a la llamada",removedFromCallMoreDetails:"Otro participante te ha eliminado de la llamada.",removedFromCallTitle:"Fuiste eliminado",removeMenuLabel:"Quitar",startSpotlightMenuLabel:"Poner en primer plano para todos",addSpotlightMenuLabel:"Agregar primer plano",stopSpotlightMenuLabel:"Dejar de poner en primer plano",stopSpotlightOnSelfMenuLabel:"Primer plano",spotlightLimitReachedMenuTitle:"Se alcanzó el límite de primer plano",stopAllSpotlightMenuLabel:"Detener todos los primeros planos",returnToCallButtonAriaDescription:"Volver a llamada",returnToCallButtonAriaLabel:"Volver",roomNotFoundDetails:"No se encontró el Id. de sala proporcionado.",roomNotFoundTitle:"No se ha encontrado la sala",roomNotValidDetails:"Esta sala no es válida en este momento.",roomNotValidTitle:"Sala no válida",inviteToRoomRemovedDetails:"Se ha quitado la invitación para unirse a esta sala.",inviteToRoomRemovedTitle:"Invitación a sala quitada",soundLabel:"Sonido",noMicrophonesLabel:"No se han encontrado micrófonos",noSpeakersLabel:"No se han encontrado altavoces",startCallButtonLabel:"Iniciar llamada",openDialpadButtonLabel:"Marcar número de teléfono",peoplePaneAddPeopleButtonLabel:"Agregar contactos",dialpadStartCallButtonLabel:"Llamar",dialpadModalTitle:"Marcar número de teléfono",dialpadModalAriaLabel:"Teclado de marcado",dialpadCloseModalButtonAriaLabel:"Cerrar teclado de marcado",moreButtonCallingLabel:"Más",resumeCallButtonLabel:"Reanudar",resumingCallButtonLabel:"Reanudando…",resumeCallButtonAriaLabel:"Reanudar llamada",resumingCallButtonAriaLabel:"Reanudar llamada",holdScreenLabel:"Está en espera",openDtmfDialpadLabel:"Mostrar teclado de marcado",dtmfDialpadPlaceholderText:"Introducir número",outboundCallingNoticeString:"Llamando...",participantJoinedNoticeString:"{displayName} unido",twoParticipantJoinedNoticeString:"{displayName1} y {displayName2} se han unido",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} y {displayName3} se han unido",participantLeftNoticeString:"{displayName} se ha retirado",twoParticipantLeftNoticeString:"{displayName1} y {displayName2} se han retirado",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} y {displayName3} se han retirado",unnamedParticipantString:"participante sin nombre",manyUnnamedParticipantsJoined:"participante sin nombre y {numOfParticipants} participantes más se han unido",manyUnnamedParticipantsLeft:"participante sin nombre y {numOfParticipants} participantes más se han retirado",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} y {numOfParticipants} se han unido otros participantes",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} y {numOfParticipants} restantes",liveCaptionsLabel:"Subtítulos en directo",captionsSettingsLabel:"Configuración de subtítulos",startCaptionsButtonOnLabel:"Activar subtítulos",startCaptionsButtonOffLabel:"Desactivar subtítulos",startCaptionsButtonTooltipOnContent:"Desactivar subtítulos",startCaptionsButtonTooltipOffContent:"Activar subtítulos",captionsSettingsModalTitle:"Configuración de subtítulos",captionsSettingsSpokenLanguageDropdownLabel:"Idioma hablado",captionsSettingsCaptionLanguageDropdownLabel:"Idioma de subtítulos",captionsSettingsSpokenLanguageDropdownInfoText:"Idioma en el que están hablando todos los usuarios de esta llamada.",captionsSettingsCaptionLanguageDropdownInfoText:"Los subtítulos aparecerán en este idioma.",captionsSettingsConfirmButtonLabel:"Confirmar",captionsSettingsCancelButtonLabel:"Cancelar",captionsSettingsModalAriaLabel:"Configuración modal de subtítulos",captionsSettingsCloseModalButtonAriaLabel:"Cerrar configuración de subtítulos",captionsBannerMoreButtonCallingLabel:"Configuración de subtítulos Más opciones",captionsBannerMoreButtonTooltip:"Más opciones",realTimeTextLabel:"Texto en tiempo real (RTT)",startRealTimeTextLabel:"Activar RTT para esta llamada",disabledStartRealTimeTextLabel:"RTT está activado actualmente, no se puede desactivar",dismissModalAriaLabel:"Imagen e imagen local y remota, presiona Entrar para volver a la llamada",callRejectedTitle:"No se pudo completar la llamada",callRejectedMoreDetails:"La persona con la que quieres contactarte no está disponible. Inténtalo de nuevo más tarde.",callTimeoutTitle:"No se pudo completar la llamada",callTimeoutDetails:"La persona con la que quieres contactarte no está disponible. Inténtalo de nuevo más tarde.",callTimeoutBotTitle:"No se pudo completar la llamada",callTimeoutBotDetails:"El servicio al que intenta conectarse no está disponible. Vuelva a intentarlo más tarde.",dtmfDialerButtonLabel:"Teclado de marcado",dtmfDialerButtonTooltipOn:"Mostrar teclado de marcado",dtmfDialerButtonTooltipOff:"Ocultar teclado de marcado",dtmfDialerMoreButtonLabelOn:"Mostrar teclado de marcado",dtmfDialerMoreButtonLabelOff:"Ocultar teclado de marcado",phoneCallMoreButtonLabel:"Llamada telefónica",spokenLanguageStrings:{"ar-ae":"Árabe - Emiratos Árabes Unidos","ar-sa":"Árabe - Arabia Saudí","da-dk":"Danés","de-de":"Alemán - Alemania","en-au":"Inglés - Australia","en-ca":"Inglés (Canadá)","en-gb":"Inglés - Reino Unido","en-in":"Inglés (India)","en-nz":"Inglés (Nueva Zelanda)","en-us":"Inglés - Estados Unidos","es-es":"Español - España (alfabetización internacional)","es-mx":"Español (México)","fi-fi":"Finés","fr-ca":"Francés (Canadá)","fr-fr":"Francés - Francia","hi-in":"Hindi","it-it":"Italiano (Italia)","ja-jp":"Japonés","ko-kr":"Coreano","nb-no":"Noruego (Bokmlå)","nl-be":"Neerlandés - Bélgica","nl-nl":"Neerlandés (Países Bajos)","pl-pl":"Polaco","pt-br":"Portugués (Brasil)","ru-ru":"Ruso","sv-se":"Sueco","zh-cn":"Chino - República Popular China","zh-hk":"Chino - Hong Kong (RAE)","cs-cz":"Checo","pt-pt":"Portugués de Portugal","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Tailandés","he-il":"Hebreo","cy-gb":"Galés","uk-ua":"Ucraniano","el-gr":"Griego","hu-hu":"Húngaro","ro-ro":"Rumano","sk-sk":"Eslovaco","zh-tw":"Chino - Taiwán"},captionLanguageStrings:{ar:"Árabe",da:"Danés",de:"Alemán",en:"Inglés",es:"Español",fi:"Finés","fr-ca":"Francés (Canadá)",fr:"Francés - Francia",hi:"Hindi",it:"Italiano",ja:"Japonés",ko:"Coreano",nb:"Noruego (Bokmål)",nl:"Neerlandés",pl:"Polaco",pt:"Portugués (Brasil)",ru:"Ruso",sv:"Sueco","zh-Hans":"Chino (simplificado)","zh-Hant":"Chino (tradicional)",cs:"Checo","pt-pt":"Portugués de Portugal",tr:"Turco",vi:"Vietnamita",th:"Tailandés",he:"Hebreo",cy:"Galés",uk:"Ucraniano",el:"Griego",hu:"Húngaro",ro:"Rumano",sk:"Eslovaco"},captionsBannerSpinnerText:"Iniciando subtítulos...",realTimeTextInputBoxDefaultText:"Escribir un mensaje en tiempo real",realTimeTextInputErrorMessage:"El mensaje no puede exceder los 2000 caracteres",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RtT (texto en tiempo real) está habilitado para todos los participantes durante toda la reunión.",realTimeTextBannerLinkLabel:"Más información",realTimeTextModalAriaLabel:"Activar modal RTT",realTimeTextModalTitle:"¿Activar RTT?",realTimeTextModalText:"Se activará el texto en tiempo real (RTT) para todos los participantes de la reunión.",realTimeTextConfirmButtonLabel:"Activar",realTimeTextCancelButtonLabel:"Cancelar",realTimeTextCloseModalButtonAriaLabel:"Cerrar modal RTT",captionsOnlyContainerTitle:"Subtítulos",realTimeTextOnlyContainerTitle:"Texto en tiempo real",captionsAndRealTimeTextContainerTitle:"Subtítulos y texto en tiempo real",expandButtonAriaLabel:"Expandir",minimizeButtonAriaLabel:"Contraer",transferPageTransferorText:"Transfiriendo...",transferPageTransferTargetText:"Conectando...",transferPageUnknownTransferorDisplayName:"Desconocido",transferPageUnknownTransferTargetDisplayName:"Desconocido",transferPageNoticeString:"Está siendo transferido",participantCouldNotBeReachedTitle:"El participante de destino no está disponible actualmente",participantCouldNotBeReachedMoreDetails:"Póngase en contacto con el participante cuando esté disponible",permissionToReachTargetParticipantNotAllowedTitle:"No se permite el permiso para llegar al participante objetivo",permissionToReachTargetParticipantNotAllowedMoreDetails:"Compruebe que el participante de destino está en el mismo inquilino.",unableToResolveTenantTitle:"No se puede resolver el id. de inquilino para el participante de destino",unableToResolveTenantMoreDetails:"Compruebe que el id. de participante se ha escrito correctamente.",participantIdIsMalformedTitle:"El id. de participante no tiene el formato correcto",participantIdIsMalformedMoreDetails:"Compruebe que el id. de participante tiene el formato correcto",moreButtonGalleryControlLabel:"Ver",moreButtonGalleryPositionToggleLabel:"Mover galería a la parte superior",moreButtonGallerySpeakerLayoutLabel:"Orador",moreButtonGalleryFloatingLocalLayoutLabel:"Dinámico",moreButtonGalleryDefaultLayoutLabel:"Vista galería",moreButtonGalleryFocusedContentLayoutLabel:"Centrarse en el contenido",moreButtonLargeGalleryDefaultLayoutLabel:"Galería de gran tamaño",moreButtonTogetherModeLayoutLabel:"Modo conferencia",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"La cámara se ha deshabilitado. Ya no puedes compartir vídeo.",grantedDueToMeetingOption:"Tu cámara ha sido habilitada. Actívala si quiere compartir un vídeo."},unmuteMic:{lostDueToMeetingOption:"El micrófono se ha deshabilitado. Ya no se puede reactivar el audio.",grantedDueToMeetingOption:"Se ha habilitado el micro. Reactive el audio si quiere hablar."},shareScreen:{lostDueToRoleChangeToAttendee:"Tu rol ha sido cambiado. Algunas acciones, como compartir contenido, no estarán disponibles para usted.",grantedDueToRoleChangeToPresenter:"Es moderador. Puede compartir contenido y facilitar la reunión."},hideAttendeeNames:{hideAttendeeNameAttendee:"Su nombre está oculto para otros asistentes. Los organizadores y moderadores pueden ver su nombre real.",hideAttendeeNamePresenter:"Los nombres de los asistentes están ocultos. Solo los organizadores y moderadores pueden ver los nombres para proteger la privacidad de los asistentes."}},surveyTitle:"Ayúdenos a mejorar",starSurveyHelperText:"¿Qué tal era la calidad de la llamada?",starSurveyOneStarText:"La calidad era mala.",starSurveyTwoStarText:"La calidad era deficiente.",starSurveyThreeStarText:"La calidad era buena.",starSurveyFourStarText:"La calidad era estupenda.",starSurveyFiveStarText:"La calidad era excelente.",surveyConfirmButtonLabel:"Enviar comentarios",starRatingAriaLabel:"Seleccione {0} de {1} estrellas",surveySkipButtonLabel:"Omitir",tagsSurveyQuestion:"¿Qué podría haber sido mejor?",tagsSurveyTextFieldDefaultText:"Otro, especifique",tagsSurveyHelperText:"Marque los problemas que haya experimentado",endOfSurveyText:"Gracias por sus comentarios.",surveyIssues:{overallRating:{callCannotJoin:"No pude unirme a la llamada",callCannotInvite:"No podía invitar a nadie a la llamada",hadToRejoin:"He tenido que volver a unirme a la llamada",callEndedUnexpectedly:"La llamada finalizó inesperadamente",otherIssues:"Estaba teniendo otros problemas con la llamada"},audioRating:{noLocalAudio:"Los otros participantes no podían oír nada",noRemoteAudio:"No pude oír nada",echo:"He oído ecos en la llamada",audioNoise:"Se oía ruido en la llamada",lowVolume:"El volumen era bajo",audioStoppedUnexpectedly:"El audio se detuvo inesperadamente",distortedSpeech:"El audio se distorsionaba",audioInterruption:"Se interrumpió el audio",otherIssues:"Estaba teniendo otros problemas de audio en esta llamada"},videoRating:{noVideoReceived:"No pude ver ningún vídeo",noVideoSent:"Los demás usuarios no me veían",lowQuality:"La calidad de vídeo era baja",freezes:"Vídeo congelado",stoppedUnexpectedly:"El vídeo se detuvo inesperadamente",darkVideoReceived:"Solo puedo ver pantallas oscuras cuando otros usuarios activan su cámara",audioVideoOutOfSync:"Audio y vídeo no sincronizados",otherIssues:"Estaba teniendo otros problemas de vídeo en esta llamada"},screenshareRating:{noContentLocal:"Otras personas no pudieron ver mi pantalla compartida",noContentRemote:"No pude ver el la pantalla compartida de otras personas",cannotPresent:"No pude presentar mi pantalla",lowQuality:"La calidad de la pantalla compartida era baja",freezes:"Pantalla compartida congelada",stoppedUnexpectedly:"El la pantalla compartida se detuvo inesperadamente",largeDelay:"La pantalla compartida tiene mucho retraso",otherIssues:"Estaba teniendo otros problemas de pantalla compartida en esta llamada"}},surveyIssuesHeadingStrings:{overallRating:"General",audioRating:"Audio",videoRating:"Vídeo",screenshareRating:"Presentación"},spotlightPrompt:{startSpotlightHeading:"¿Poner en primer plano para todos?",startSpotlightText:"Este vídeo lo resaltará para todos en la reunión.",startSpotlightOnSelfText:"Resaltará el vídeo para todos los participantes de la reunión.",startSpotlightConfirmButtonLabel:"Poner en primer plano para todos",startSpotlightCancelButtonLabel:"Cancelar",stopSpotlightHeading:"¿Dejar de poner este vídeo en primer plano?",stopSpotlightOnSelfHeading:"¿Salir del primer plano?",stopAllSpotlightHeading:"¿Quieres dejar de poner en primer plano todos los vídeos?",stopSpotlightText:"Este vídeo dejará de estar resaltado para todos los asistentes a la reunión.",stopSpotlightOnSelfText:"Su vídeo ya no resaltará para todos en la reunión.",stopAllSpotlightText:"El vídeo ya no se pondrá en primer plano para todos los participantes de la reunión.",stopSpotlightConfirmButtonLabel:"Dejar de poner en primer plano",stopSpotlightOnSelfConfirmButtonLabel:"Salir del primer plano",stopSpotlightCancelButtonLabel:"Cancelar",closeSpotlightPromptButtonLabel:"Cerrar"},exitSpotlightButtonLabel:"Salir del primer plano",exitSpotlightButtonTooltip:"Salir del primer plano",leaveConfirmButtonLabel:"Salir",endCallConfirmButtonLabel:"Finalizar llamada",hangUpCancelButtonLabel:"Cancelar",leaveConfirmDialogTitle:"¿Abandonar la llamada?",leaveConfirmDialogContent:"¿Está seguro de que desea abandonar la llamada?",endCallConfirmDialogTitle:"¿Finalizar la llamada?",endCallConfirmDialogContent:"Finalizará la llamada para todos los usuarios.",invalidMeetingIdentifier:"Id. de reunión o código de acceso no válidos",pinParticipantMenuLabel:"Anclar para mí",pinParticipantLimitReachedMenuLabel:"Pin (límite alcanzado)",unpinParticipantMenuLabel:"Desanclar",pinParticipantMenuItemAriaLabel:"Anclar para mí",unpinParticipantMenuItemAriaLabel:"Desanclar",muteAllMenuLabel:"Silenciar todo",muteAllDialogTitle:"¿Silenciar a todos?",muteAllDialogContent:"Esto silenciará a todos los participantes de la reunión, excepto usted.",muteAllConfirmButtonLabel:"Silenciar todo",muteAllCancelButtonLabel:"Cancelar",joinBreakoutRoomButtonLabel:"Unirse a la sala para sesión de subgrupo",returnFromBreakoutRoomButtonLabel:"Salir de la sala",leaveBreakoutRoomAndMeetingButtonLabel:"Salir de la reunión",breakoutRoomJoinedNotificationTitle:"Se has unido a la sala para sesión de subgrupo '{roomName}'",joinBreakoutRoomBannerTitle:"La sala para sesión de subgrupo '{roomName}' está abierta",joinBreakoutRoomBannerButtonLabel:"Unirse",returnFromBreakoutRoomBannerTitle:"Regresar a la reunión principal",returnFromBreakoutRoomBannerButtonLabel:"Volver a unirse",forbidAudioMenuLabel:"Deshabilitar micrófono",permitAudioMenuLabel:"Permitir micrófono",forbidOthersAudioDialogTitle:"¿Quiere deshabilitar el micrófono para los asistentes?",forbidOthersAudioDialogContent:"Los asistentes no podrán activar su audio. El organizador y los moderadores podrán permitir que la gente active su audio según sea necesario.",forbidOthersAudioConfirmButtonLabel:"Deshabilitar micrófonos",forbidOthersAudioCancelButtonLabel:"Cancelar",permitOthersAudioDialogTitle:"¿Permitir a los asistentes usar el micrófono?",permitOthersAudioDialogContent:"Todos los usuarios de la reunión podrán reactivar su audio.",permitOthersAudioConfirmButtonLabel:"Permitir micrófonos",permitOthersAudioCancelButtonLabel:"Cancelar",forbidOthersAudioMenuLabel:"Deshabilitar micrófono para los asistentes",permitOthersAudioMenuLabel:"Permitir el micrófono para los asistentes",forbidVideoMenuLabel:"Deshabilitar cámara",permitVideoMenuLabel:"Permitir cámara",forbidOthersVideoDialogTitle:"¿Quiere deshabilitar la cámara de los asistentes?",forbidOthersVideoDialogContent:"Los asistentes no podrán activar el vídeo ellos mismos. El organizador y los moderadores pueden permitir que los usuarios enciendan la cámara según sea necesario.",forbidOthersVideoConfirmButtonLabel:"Deshabilitar cámaras",forbidOthersVideoCancelButtonLabel:"Cancelar",permitOthersVideoDialogTitle:"¿Permitir a los asistentes usar la cámara?",permitOthersVideoDialogContent:"Todos los participantes de la reunión podrán activar el vídeo ellos mismos.",permitOthersVideoConfirmButtonLabel:"Permitir cámaras",permitOthersVideoCancelButtonLabel:"Cancelar",forbidOthersVideoMenuLabel:"Deshabilitar la cámara de los asistentes",permitOthersVideoMenuLabel:"Permitir la cámara para los asistentes"};var chat$h={chatListHeader:"En este chat",uploadAttachment:"Cargar datos adjuntos",uploadImageDataNotProvided:"No se proporcionaron datos de imagen.",uploadImageIsTooLarge:"La imagen es demasiado grande. Elige uno que sea inferior a {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"No se permite cargar una imagen .{imageExtension}.",uploadImageFailed:"No se puede cargar la imagen. Vuelva a intentarlo más tarde."};var callWithChat$h={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"Nuevo mensaje",chatButtonTooltipClosedWithMessageCount:"Mostrar chat ({unreadMessagesCount} no leídos)",chatButtonTooltipClose:"Ocultar chat",chatButtonTooltipOpen:"Mostrar chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Copiar vínculo de invitación",copyInviteLinkButtonActionedLabel:"Vínculo copiado",copyInviteLinkActionedAriaLabel:"Vínculo de invitación copiado",dismissSidePaneButtonLabel:"Cerrar",moreDrawerAudioDeviceMenuTitle:"Dispositivo de audio",moreDrawerButtonLabel:"Más opciones",moreDrawerButtonTooltip:"Más opciones",moreDrawerMicrophoneMenuTitle:"Micrófono",moreDrawerSpeakerMenuTitle:"Altavoz",moreDrawerCaptionsMenuTitle:"Subtítulos en directo",moreDrawerSpokenLanguageMenuTitle:"Idioma hablado",moreDrawerCaptionLanguageMenuTitle:"Idioma de subtítulos",moreDrawerGalleryOptionsMenuTitle:"Opciones de la Galería",peopleButtonLabel:"Contactos",selectedPeopleButtonLabel:"Se seleccionó el botón Contactos",peopleButtonTooltipOpen:"Mostrar participantes",peopleButtonTooltipClose:"Ocultar participantes",peopleButtonTooltipOpenAriaLabel:"Botón Contactos, Mostrar participantes",peopleButtonTooltipCloseAriaLabel:"Botón Contactos, Ocultar participantes",peoplePaneSubTitle:"En esta llamada",peoplePaneTitle:"Contactos",pictureInPictureTileAriaLabel:"Fuentes de vídeo. Hacer clic para volver a la pantalla de la llamada.",removeMenuLabel:"Quitar",openDialpadButtonLabel:"Marcar número de teléfono",returnToCallButtonAriaDescription:"Volver a llamada",returnToCallButtonAriaLabel:"Volver",peoplePaneAddPeopleButtonLabel:"Agregar contactos",dialpadStartCallButtonLabel:"Llamar",dialpadModalTitle:"Marcar número de teléfono",dialpadModalAriaLabel:"Teclado de marcado",dialpadCloseModalButtonAriaLabel:"Cerrar teclado de marcado",openDtmfDialpadLabel:"Mostrar teclado de marcado",dtmfDialpadPlaceholderText:"Introducir número",breakoutRoomChatPaneTitle:"Chat de sala para sesión de subgrupo",chatContentSpinnerLabel:"Cargando chat"};var es_ES = {call:call$h,chat:chat$h,callWithChat:callWithChat$h};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Spanish (Spain)
 *
 * @public
 */
const COMPOSITE_LOCALE_ES_ES = {
    component: COMPONENT_LOCALE_ES_ES,
    strings: createCompositeStrings(es_ES)
};

var call$g={cameraLabel:"Cámara",noCamerasLabel:"No se han encontrado cámaras",cameraPermissionDenied:"El explorador está bloqueando el acceso a la cámara",cameraTurnedOff:"La cámara está apagada",chatButtonLabel:"Chat",close:"Cerrar",complianceBannerNowOnlyRecording:"Ahora solo estás grabando esta reunión.",complianceBannerNowOnlyTranscription:"Ahora solo estás transcribiendo esta reunión.",complianceBannerRecordingAndTranscriptionSaved:"Se están guardando la grabación y la transcripción.",complianceBannerRecordingAndTranscriptionStarted:"Se han iniciado la grabación y la transcripción.",complianceBannerRecordingAndTranscriptionStopped:"Se han detenido la grabación y la transcripción.",complianceBannerRecordingSaving:"La grabación se está guardando.",complianceBannerRecordingStarted:"La grabación se inició.",complianceBannerRecordingStopped:"La grabación se ha detenido.",complianceBannerTranscriptionStarted:"La transcripción se ha iniciado.",complianceBannerTranscriptionConsent:"Al unirte, das tu consentimiento para que se transcriba esta reunión.",complianceBannerTranscriptionSaving:"Se está guardando la transcripción.",complianceBannerTranscriptionStopped:"La transcripción se ha detenido.",configurationPageTitle:"Iniciar una llamada",copyInviteLinkButtonLabel:"Copiar vínculo de invitación",copyInviteLinkButtonActionedLabel:"Vínculo copiado",copyInviteLinkActionedAriaLabel:"Vínculo de invitación copiado",defaultPlaceHolder:"Seleccionar una opción",dismissSidePaneButtonLabel:"Cerrar",videoEffectsPaneTitle:"Efectos",videoEffectsPaneBackgroundSelectionTitle:"Fondo",videoEffectsPaneAriaLabel:"Panel de efectos de video",configurationPageCameraIsLoadingLabel:"El video se está cargando...",configurationPageVideoEffectsButtonLabel:"Efectos",unableToStartVideoEffect:"No se puede aplicar el efecto de video.",blurBackgroundEffectButtonLabel:"Desenfoque",blurBackgroundTooltip:"Desenfocar fondo",removeBackgroundEffectButtonLabel:"Ninguno",removeBackgroundTooltip:"Quitar fondo",cameraOffBackgroundEffectWarningText:"La cámara está apagada. Enciende la cámara para ver el efecto de video.",failedToJoinCallDueToNoNetworkMoreDetails:"Se desconectó la llamada debido a un problema de red. Comprueba la conexión y únete de nuevo.",failedToJoinCallDueToNoNetworkTitle:"Llamada desconectada",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"No se te concedió la entrada a la llamada. Si se trata de un error, vuelve a unirte.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Rechazado de la sala de espera",learnMore:"Más información",leavingCallTitle:"Saliendo...",leftCallMoreDetails:"Si se trata de un error, vuelve a unirte a la llamada.",leftCallTitle:"Has abandonado la llamada",lobbyScreenConnectingToCallTitle:"Uniéndose a la llamada",lobbyScreenWaitingToBeAdmittedTitle:"Esperando a ser admitido",microphonePermissionDenied:"El explorador está bloqueando el acceso al micrófono",microphoneToggleInLobbyNotAllowed:"No se puede silenciar ni reactivar el audio mientras se está en la sala de espera.",mutedMessage:"Estás silenciado",networkReconnectMoreDetails:"Parece que hubo un problema. Estamos intentando reconectar la llamada.",networkReconnectTitle:"Espera",notInvitedToRoomDetails:"No puedes unirte a esta sala porque no tienes una invitación.",notInvitedToRoomTitle:"No invitado a la sala",peopleButtonLabel:"Contactos",selectedPeopleButtonLabel:"Se seleccionó el botón Contactos",peoplePaneTitle:"Contactos",peoplePaneMoreButtonAriaLabel:"Más",peopleButtonTooltipOpen:"Mostrar participantes",peopleButtonTooltipClose:"Ocultar participantes",peoplePaneSubTitle:"En esta llamada {numberOfPeople}",privacyPolicy:"Declaración de privacidad",rejoinCallButtonLabel:"Volver a unirse a la llamada",removedFromCallMoreDetails:"Otro participante te ha eliminado de la llamada.",removedFromCallTitle:"Fuiste eliminado",removeMenuLabel:"Quitar",startSpotlightMenuLabel:"Poner en primer plano para todos",addSpotlightMenuLabel:"Agregar primer plano",stopSpotlightMenuLabel:"Dejar de poner en primer plano",stopSpotlightOnSelfMenuLabel:"Salir de primer plano",spotlightLimitReachedMenuTitle:"Se alcanzó el límite de primer plano",stopAllSpotlightMenuLabel:"Detener todos los primeros planos",returnToCallButtonAriaDescription:"Volver a llamada",returnToCallButtonAriaLabel:"Volver",roomNotFoundDetails:"No se encontró el Id. de sala proporcionado.",roomNotFoundTitle:"No se ha encontrado la sala",roomNotValidDetails:"Esta sala no es válida en este momento.",roomNotValidTitle:"Sala no válida",inviteToRoomRemovedDetails:"Se ha quitado la invitación para unirse a esta sala.",inviteToRoomRemovedTitle:"Invitación a sala quitada",soundLabel:"Sonido",noMicrophonesLabel:"No se han encontrado micrófonos",noSpeakersLabel:"No se han encontrado altavoces",startCallButtonLabel:"Iniciar llamada",openDialpadButtonLabel:"Marcar número de teléfono",peoplePaneAddPeopleButtonLabel:"Agregar contactos",dialpadStartCallButtonLabel:"Llamar",dialpadModalTitle:"Marcar número de teléfono",dialpadModalAriaLabel:"Teclado de marcado",dialpadCloseModalButtonAriaLabel:"Cerrar teclado de marcado",moreButtonCallingLabel:"Más",resumeCallButtonLabel:"Reanudar",resumingCallButtonLabel:"Reanudando…",resumeCallButtonAriaLabel:"Reanudar llamada",resumingCallButtonAriaLabel:"Reanudar llamada",holdScreenLabel:"Está en espera",openDtmfDialpadLabel:"Mostrar teclado de marcado",dtmfDialpadPlaceholderText:"Introducir número",outboundCallingNoticeString:"Llamando...",participantJoinedNoticeString:"{displayName} unido",twoParticipantJoinedNoticeString:"{displayName1} y {displayName2} se han unido",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} y {displayName3} se han unido",participantLeftNoticeString:"{displayName} se ha retirado",twoParticipantLeftNoticeString:"{displayName1} y {displayName2} se han retirado",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} y {displayName3} se han retirado",unnamedParticipantString:"participante sin nombre",manyUnnamedParticipantsJoined:"participante sin nombre y {numOfParticipants} participantes más se han unido",manyUnnamedParticipantsLeft:"participante sin nombre y {numOfParticipants} participantes más se han retirado",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} y {numOfParticipants} se han unido otros participantes",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} y {numOfParticipants} restantes",liveCaptionsLabel:"Subtítulos en directo",captionsSettingsLabel:"Configuración de subtítulos",startCaptionsButtonOnLabel:"Activar subtítulos",startCaptionsButtonOffLabel:"Desactivar subtítulos",startCaptionsButtonTooltipOnContent:"Desactivar subtítulos",startCaptionsButtonTooltipOffContent:"Activar subtítulos",captionsSettingsModalTitle:"Configuración de subtítulos",captionsSettingsSpokenLanguageDropdownLabel:"Idioma hablado",captionsSettingsCaptionLanguageDropdownLabel:"Idioma de subtítulos",captionsSettingsSpokenLanguageDropdownInfoText:"Idioma en el que están hablando todos los usuarios de esta llamada.",captionsSettingsCaptionLanguageDropdownInfoText:"Los subtítulos aparecerán en este idioma.",captionsSettingsConfirmButtonLabel:"Confirmar",captionsSettingsCancelButtonLabel:"Cancelar",captionsSettingsModalAriaLabel:"Configuración modal de subtítulos",captionsSettingsCloseModalButtonAriaLabel:"Cerrar configuración de subtítulos",captionsBannerMoreButtonCallingLabel:"Más opciones de configuración de subtítulos",captionsBannerMoreButtonTooltip:"Más opciones",realTimeTextLabel:"Escritura en tiempo real (RTT)",startRealTimeTextLabel:"Activar RTT para esta llamada",disabledStartRealTimeTextLabel:"RTT está activado actualmente, no se puede desactivar",dismissModalAriaLabel:"Imagen e imagen local y remota, presiona Entrar para volver a la llamada",callRejectedTitle:"No se pudo completar la llamada",callRejectedMoreDetails:"La persona con la que quieres contactarte no está disponible. Inténtalo de nuevo más tarde.",callTimeoutTitle:"No se pudo completar la llamada",callTimeoutDetails:"La persona con la que quieres contactarte no está disponible. Inténtalo de nuevo más tarde.",callTimeoutBotTitle:"No se pudo completar la llamada",callTimeoutBotDetails:"El servicio al que intentas conectarte no está disponible. Vuelve a intentarlo más tarde.",dtmfDialerButtonLabel:"Teclado de marcado",dtmfDialerButtonTooltipOn:"Mostrar teclado de marcado",dtmfDialerButtonTooltipOff:"Ocultar teclado de marcado",dtmfDialerMoreButtonLabelOn:"Mostrar teclado de marcado",dtmfDialerMoreButtonLabelOff:"Ocultar teclado de marcado",phoneCallMoreButtonLabel:"Llamada telefónica",spokenLanguageStrings:{"ar-ae":"Árabe - Emiratos Árabes Unidos","ar-sa":"Árabe - Arabia Saudí","da-dk":"Danés","de-de":"Alemán - Alemania","en-au":"Inglés - Australia","en-ca":"Inglés (Canadá)","en-gb":"Inglés - Reino Unido","en-in":"Inglés (India)","en-nz":"Inglés (Nueva Zelanda)","en-us":"Inglés - Estados Unidos","es-es":"Español - España (alfabetización internacional)","es-mx":"Español (México)","fi-fi":"Finés","fr-ca":"Francés (Canadá)","fr-fr":"Francés - Francia","hi-in":"Hindi","it-it":"Italiano (Italia)","ja-jp":"Japonés","ko-kr":"Coreano","nb-no":"Noruego (Bokmål)","nl-be":"Neerlandés - Bélgica","nl-nl":"Neerlandés (Países Bajos)","pl-pl":"Polaco","pt-br":"Portugués (Brasil)","ru-ru":"Ruso","sv-se":"Sueco","zh-cn":"Chino - República Popular China","zh-hk":"Chino - Hong Kong (RAE)","cs-cz":"Checo","pt-pt":"Portugués de Portugal","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Tailandés","he-il":"Hebreo","cy-gb":"Galés","uk-ua":"Ucraniano","el-gr":"Griego","hu-hu":"Húngaro","ro-ro":"Rumano","sk-sk":"Eslovaco","zh-tw":"Chino - Taiwán"},captionLanguageStrings:{ar:"Árabe",da:"Danés",de:"Alemán",en:"Inglés",es:"Español",fi:"Finés","fr-ca":"Francés (Canadá)",fr:"Francés - Francia",hi:"Hindi",it:"Italiano",ja:"Japonés",ko:"Coreano",nb:"Noruego (Bokmål)",nl:"Neerlandés",pl:"Polaco",pt:"Portugués (Brasil)",ru:"Ruso",sv:"Sueco","zh-Hans":"Chino (simplificado)","zh-Hant":"Chino (tradicional)",cs:"Checo","pt-pt":"Portugués de Portugal",tr:"Turco",vi:"Vietnamita",th:"Tailandés",he:"Hebreo",cy:"Galés",uk:"Ucraniano",el:"Griego",hu:"Húngaro",ro:"Rumano",sk:"Eslovaco"},captionsBannerSpinnerText:"Iniciando subtítulos...",realTimeTextInputBoxDefaultText:"Escribir un mensaje en tiempo real",realTimeTextInputErrorMessage:"El mensaje no puede exceder los 2000 caracteres",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RtT (texto en tiempo real) está habilitado para todos los participantes durante toda la reunión.",realTimeTextBannerLinkLabel:"Más información",realTimeTextModalAriaLabel:"Activar modal RTT",realTimeTextModalTitle:"¿Activar RTT?",realTimeTextModalText:"Se activará el texto en tiempo real (RTT) para todos los participantes de la reunión.",realTimeTextConfirmButtonLabel:"Activar",realTimeTextCancelButtonLabel:"Cancelar",realTimeTextCloseModalButtonAriaLabel:"Cerrar modal RTT",captionsOnlyContainerTitle:"Subtítulos",realTimeTextOnlyContainerTitle:"Texto en tiempo real",captionsAndRealTimeTextContainerTitle:"Subtítulos y texto en tiempo real",expandButtonAriaLabel:"Expandir",minimizeButtonAriaLabel:"Contraer",transferPageTransferorText:"Transfiriendo...",transferPageTransferTargetText:"Conectando...",transferPageUnknownTransferorDisplayName:"Desconocido",transferPageUnknownTransferTargetDisplayName:"Desconocido",transferPageNoticeString:"Está siendo transferido",participantCouldNotBeReachedTitle:"El participante de destino no está disponible actualmente",participantCouldNotBeReachedMoreDetails:"Ponte en contacto con el participante cuando esté disponible",permissionToReachTargetParticipantNotAllowedTitle:"No se permite el permiso para llegar al participante objetivo",permissionToReachTargetParticipantNotAllowedMoreDetails:"Comprueba que el participante de destino está en el mismo inquilino.",unableToResolveTenantTitle:"No se puede resolver el id. de inquilino para el participante de destino",unableToResolveTenantMoreDetails:"Comprueba que el id. de participante se ha escrito correctamente.",participantIdIsMalformedTitle:"El id. de participante no tiene el formato correcto",participantIdIsMalformedMoreDetails:"Comprueba que el id. de participante tiene el formato correcto",moreButtonGalleryControlLabel:"Ver",moreButtonGalleryPositionToggleLabel:"Mover galería a la parte superior",moreButtonGallerySpeakerLayoutLabel:"Altavoz",moreButtonGalleryFloatingLocalLayoutLabel:"Dinámico",moreButtonGalleryDefaultLayoutLabel:"Vista galería",moreButtonGalleryFocusedContentLayoutLabel:"Centrarse en el contenido",moreButtonLargeGalleryDefaultLayoutLabel:"Galería de gran tamaño",moreButtonTogetherModeLayoutLabel:"Modo conferencia",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"La cámara se ha deshabilitado. Ya no puedes compartir video.",grantedDueToMeetingOption:"Tu cámara ha sido habilitada. Actívala si quieres compartir un video."},unmuteMic:{lostDueToMeetingOption:"El micrófono se ha deshabilitado. Ya no se puede reactivar el audio.",grantedDueToMeetingOption:"Se ha habilitado el micro. Reactiva el audio si quieres hablar."},shareScreen:{lostDueToRoleChangeToAttendee:"Tu rol ha sido cambiado. Algunas acciones, como compartir contenido, no estarán disponibles para ti.",grantedDueToRoleChangeToPresenter:"Eres moderador. Puedes compartir contenido y facilitar la reunión."},hideAttendeeNames:{hideAttendeeNameAttendee:"Tu nombre está oculto para otros asistentes. Los organizadores y moderadores pueden ver tu nombre real.",hideAttendeeNamePresenter:"Los nombres de los asistentes están ocultos. Solo los organizadores y moderadores pueden ver los nombres para proteger la privacidad de los asistentes."}},surveyTitle:"Ayúdanos a mejorar",starSurveyHelperText:"¿Qué tal era la calidad de la llamada?",starSurveyOneStarText:"La calidad era mala.",starSurveyTwoStarText:"La calidad era deficiente.",starSurveyThreeStarText:"La calidad era buena.",starSurveyFourStarText:"La calidad era estupenda.",starSurveyFiveStarText:"La calidad ha sido excelente.",surveyConfirmButtonLabel:"Enviar comentarios",starRatingAriaLabel:"Selecciona {0} de {1} estrellas",surveySkipButtonLabel:"Omitir",tagsSurveyQuestion:"¿Qué podría haber sido mejor?",tagsSurveyTextFieldDefaultText:"Otro, especifica",tagsSurveyHelperText:"Marca los problemas que hayas experimentado",endOfSurveyText:"Gracias por tus comentarios.",surveyIssues:{overallRating:{callCannotJoin:"No pude unirme a la llamada",callCannotInvite:"No podía invitar a nadie a la llamada",hadToRejoin:"He tenido que volver a unirme a la llamada",callEndedUnexpectedly:"La llamada finalizó inesperadamente",otherIssues:"Estaba teniendo otros problemas con la llamada"},audioRating:{noLocalAudio:"Los otros participantes no podían oír nada",noRemoteAudio:"No pude oír nada",echo:"Oía eco en la llamada",audioNoise:"Se oía ruido en la llamada",lowVolume:"El volumen era bajo",audioStoppedUnexpectedly:"El audio se detuvo inesperadamente",distortedSpeech:"El audio se distorsionaba",audioInterruption:"Se interrumpió el audio",otherIssues:"Estaba teniendo otros problemas de audio en esta llamada"},videoRating:{noVideoReceived:"No pude ver ningún video",noVideoSent:"Los demás usuarios no me veían",lowQuality:"La calidad de vídeo era baja",freezes:"Video congelado",stoppedUnexpectedly:"El video se detuvo inesperadamente",darkVideoReceived:"Solo puedo ver pantallas oscuras cuando otros usuarios activan su cámara",audioVideoOutOfSync:"El audio y el video no estaban sincronizados",otherIssues:"Estaba teniendo otros problemas de video en esta llamada"},screenshareRating:{noContentLocal:"Otras personas no pudieron ver mi pantalla compartida",noContentRemote:"No pude ver la pantalla compartida de otras personas",cannotPresent:"No pude presentar mi pantalla",lowQuality:"La calidad de la pantalla compartida era baja",freezes:"Pantalla compartida congelada",stoppedUnexpectedly:"La pantalla compartida se detuvo inesperadamente",largeDelay:"La pantalla compartida tiene mucho retraso",otherIssues:"Estaba teniendo otros problemas de pantalla compartida en esta llamada"}},surveyIssuesHeadingStrings:{overallRating:"General",audioRating:"Audio",videoRating:"Video",screenshareRating:"Presentando"},spotlightPrompt:{startSpotlightHeading:"¿Poner en primer plano para todos?",startSpotlightText:"Resaltarás este video para todos en la reunión.",startSpotlightOnSelfText:"Resaltarás el video para todos los participantes de la reunión.",startSpotlightConfirmButtonLabel:"Poner en primer plano para todos",startSpotlightCancelButtonLabel:"Cancelar",stopSpotlightHeading:"¿Dejar de poner este video en primer plano?",stopSpotlightOnSelfHeading:"¿Salir del primer plano?",stopAllSpotlightHeading:"¿Quieres dejar de poner en primer plano todos los videos?",stopSpotlightText:"Este video dejará de estar resaltado para todos en la reunión.",stopSpotlightOnSelfText:"Tu video ya no resaltará para todos en la reunión.",stopAllSpotlightText:"El video ya no se resaltará para todos los participantes de la reunión.",stopSpotlightConfirmButtonLabel:"Dejar de poner en primer plano",stopSpotlightOnSelfConfirmButtonLabel:"Salir de primer plano",stopSpotlightCancelButtonLabel:"Cancelar",closeSpotlightPromptButtonLabel:"Cerrar"},exitSpotlightButtonLabel:"Salir de primer plano",exitSpotlightButtonTooltip:"Salir de primer plano",leaveConfirmButtonLabel:"Salir",endCallConfirmButtonLabel:"Finalizar llamada",hangUpCancelButtonLabel:"Cancelar",leaveConfirmDialogTitle:"¿Abandonar la llamada?",leaveConfirmDialogContent:"¿Estás seguro de que deseas abandonar la llamada?",endCallConfirmDialogTitle:"¿Finalizar la llamada?",endCallConfirmDialogContent:"Finalizarás la llamada para todos los usuarios.",invalidMeetingIdentifier:"Id. de reunión o código de acceso no válidos",pinParticipantMenuLabel:"Anclar para mí",pinParticipantLimitReachedMenuLabel:"Pin (límite alcanzado)",unpinParticipantMenuLabel:"Desanclar",pinParticipantMenuItemAriaLabel:"Anclar para mí",unpinParticipantMenuItemAriaLabel:"Desanclar",muteAllMenuLabel:"Silenciar todo",muteAllDialogTitle:"¿Silenciar a todos?",muteAllDialogContent:"Esto silenciará a todos los participantes de la reunión, excepto a ti.",muteAllConfirmButtonLabel:"Silenciar todo",muteAllCancelButtonLabel:"Cancelar",joinBreakoutRoomButtonLabel:"Únete a la sala para sesión de subgrupo",returnFromBreakoutRoomButtonLabel:"Salir de la sala",leaveBreakoutRoomAndMeetingButtonLabel:"Salir de la reunión",breakoutRoomJoinedNotificationTitle:"Te has unido a la sala para sesión de subgrupo '{roomName}'",joinBreakoutRoomBannerTitle:"La sala para sesión de subgrupo '{roomName}' está abierta",joinBreakoutRoomBannerButtonLabel:"Unirse",returnFromBreakoutRoomBannerTitle:"Regresar a la reunión principal",returnFromBreakoutRoomBannerButtonLabel:"Volver a unirse",forbidAudioMenuLabel:"Deshabilitar micrófono",permitAudioMenuLabel:"Permitir micrófono",forbidOthersAudioDialogTitle:"¿Quieres deshabilitar el micrófono para los asistentes?",forbidOthersAudioDialogContent:"Los asistentes no podrán activar su audio. El organizador y los moderadores podrán permitir que la gente active su audio según sea necesario.",forbidOthersAudioConfirmButtonLabel:"Deshabilitar micrófonos",forbidOthersAudioCancelButtonLabel:"Cancelar",permitOthersAudioDialogTitle:"¿Permitir a los asistentes usar el micrófono?",permitOthersAudioDialogContent:"Todos los usuarios de la reunión podrán reactivar su audio.",permitOthersAudioConfirmButtonLabel:"Permitir micrófonos",permitOthersAudioCancelButtonLabel:"Cancelar",forbidOthersAudioMenuLabel:"Deshabilitar micrófono para los asistentes",permitOthersAudioMenuLabel:"Permitir el micrófono para los asistentes",forbidVideoMenuLabel:"Deshabilitar cámara",permitVideoMenuLabel:"Permitir cámara",forbidOthersVideoDialogTitle:"¿Quieres deshabilitar la cámara de los asistentes?",forbidOthersVideoDialogContent:"Los asistentes no podrán activar el video ellos mismos. El organizador y los moderadores pueden permitir que los usuarios enciendan la cámara según sea necesario.",forbidOthersVideoConfirmButtonLabel:"Deshabilitar cámaras",forbidOthersVideoCancelButtonLabel:"Cancelar",permitOthersVideoDialogTitle:"¿Permitir a los asistentes usar la cámara?",permitOthersVideoDialogContent:"Todos los participantes de la reunión podrán activar el video ellos mismos.",permitOthersVideoConfirmButtonLabel:"Permitir cámaras",permitOthersVideoCancelButtonLabel:"Cancelar",forbidOthersVideoMenuLabel:"Deshabilitar la cámara de los asistentes",permitOthersVideoMenuLabel:"Permitir la cámara para los asistentes"};var chat$g={chatListHeader:"En este chat",uploadAttachment:"Cargar datos adjuntos",uploadImageDataNotProvided:"No se proporcionaron datos de imagen.",uploadImageIsTooLarge:"La imagen es demasiado grande. Elige uno que sea inferior a {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"No se permite cargar una imagen .{imageExtension}.",uploadImageFailed:"No se puede cargar la imagen. Vuelve a intentarlo más tarde."};var callWithChat$g={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"Nuevo mensaje",chatButtonTooltipClosedWithMessageCount:"Mostrar chat ({unreadMessagesCount} no leídos)",chatButtonTooltipClose:"Ocultar chat",chatButtonTooltipOpen:"Mostrar chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Copiar vínculo de invitación",copyInviteLinkButtonActionedLabel:"Vínculo copiado",copyInviteLinkActionedAriaLabel:"Vínculo de invitación copiado",dismissSidePaneButtonLabel:"Cerrar",moreDrawerAudioDeviceMenuTitle:"Dispositivo de audio",moreDrawerButtonLabel:"Más opciones",moreDrawerButtonTooltip:"Más opciones",moreDrawerMicrophoneMenuTitle:"Micrófono",moreDrawerSpeakerMenuTitle:"Altavoz",moreDrawerCaptionsMenuTitle:"Subtítulos en directo",moreDrawerSpokenLanguageMenuTitle:"Idioma hablado",moreDrawerCaptionLanguageMenuTitle:"Idioma de subtítulos",moreDrawerGalleryOptionsMenuTitle:"Opciones de la Galería",peopleButtonLabel:"Contactos",selectedPeopleButtonLabel:"Se seleccionó el botón Contactos",peopleButtonTooltipOpen:"Mostrar participantes",peopleButtonTooltipClose:"Ocultar participantes",peopleButtonTooltipOpenAriaLabel:"Botón Contactos, Mostrar participantes",peopleButtonTooltipCloseAriaLabel:"Botón Contactos, Ocultar participantes",peoplePaneSubTitle:"En esta llamada",peoplePaneTitle:"Contactos",pictureInPictureTileAriaLabel:"Fuentes de video. Hacer clic para volver a la pantalla de la llamada.",removeMenuLabel:"Quitar",openDialpadButtonLabel:"Marcar número de teléfono",returnToCallButtonAriaDescription:"Volver a llamada",returnToCallButtonAriaLabel:"Volver",peoplePaneAddPeopleButtonLabel:"Agregar contactos",dialpadStartCallButtonLabel:"Llamar",dialpadModalTitle:"Marcar número de teléfono",dialpadModalAriaLabel:"Teclado de marcado",dialpadCloseModalButtonAriaLabel:"Cerrar teclado de marcado",openDtmfDialpadLabel:"Mostrar teclado de marcado",dtmfDialpadPlaceholderText:"Introducir número",breakoutRoomChatPaneTitle:"Chat de sala para sesión de subgrupo",chatContentSpinnerLabel:"Cargando chat"};var es_MX = {call:call$g,chat:chat$g,callWithChat:callWithChat$g};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Spanish (Mexico)
 *
 * @public
 */
const COMPOSITE_LOCALE_ES_MX = {
    component: COMPONENT_LOCALE_ES_MX,
    strings: createCompositeStrings(es_MX)
};

var call$f={cameraLabel:"Kamera",noCamerasLabel:"Kameroita ei löytynyt",cameraPermissionDenied:"Selaimesi estää kameran käytön",cameraTurnedOff:"Kamerasi on pois käytöstä",chatButtonLabel:"Keskustelu",close:"Sulje",complianceBannerNowOnlyRecording:"Tallennetta ollaan luomassa vain tästä kokouksesta.",complianceBannerNowOnlyTranscription:"Transkriptiota ollaan luomassa vain tästä kokouksesta.",complianceBannerRecordingAndTranscriptionSaved:"Tallennusta ja transkriptiota tallennetaan.",complianceBannerRecordingAndTranscriptionStarted:"Tallentaminen ja litterointi on aloitettu.",complianceBannerRecordingAndTranscriptionStopped:"Tallentaminen ja litterointi on pysäytetty.",complianceBannerRecordingSaving:"Tallennetta tallennetaan.",complianceBannerRecordingStarted:"Tallentaminen on aloitettu.",complianceBannerRecordingStopped:"Tallentaminen on pysäytetty.",complianceBannerTranscriptionStarted:"Transkriptio on aloitettu.",complianceBannerTranscriptionConsent:"Liittymällä annat suostumuksesi kokouksen litteroimiseen.",complianceBannerTranscriptionSaving:"Transkriptiota tallennetaan.",complianceBannerTranscriptionStopped:"Transkriptio on pysäytetty.",configurationPageTitle:"Aloita puhelu",copyInviteLinkButtonLabel:"Kopioi kutsulinkki",copyInviteLinkButtonActionedLabel:"Linkki kopioitu",copyInviteLinkActionedAriaLabel:"Kutsulinkki kopioitu",defaultPlaceHolder:"Valitse vaihtoehto",dismissSidePaneButtonLabel:"Sulje",videoEffectsPaneTitle:"Tehosteet",videoEffectsPaneBackgroundSelectionTitle:"Tausta",videoEffectsPaneAriaLabel:"Videotehosteet-ruutu",configurationPageCameraIsLoadingLabel:"Videota ladataan...",configurationPageVideoEffectsButtonLabel:"Tehosteet",unableToStartVideoEffect:"Videotehosteen käyttäminen ei onnistu.",blurBackgroundEffectButtonLabel:"Sumenna",blurBackgroundTooltip:"Sumenna tausta",removeBackgroundEffectButtonLabel:"Ei mitään",removeBackgroundTooltip:"Poista tausta",cameraOffBackgroundEffectWarningText:"Kamera ei ole käytössä. Ota kamera käyttöön nähdäksesi videotehosteen.",failedToJoinCallDueToNoNetworkMoreDetails:"Puhelu katkaistiin verkko-ongelman vuoksi. Tarkista yhteys ja liity uudelleen.",failedToJoinCallDueToNoNetworkTitle:"Puhelu on katkennut",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Sinulle ei myönnetty puheluun merkintää. Jos tämä oli virhe, liity puheluun uudelleen.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Poistettu odotustilasta",learnMore:"Lisätietoja",leavingCallTitle:"Poistumassa...",leftCallMoreDetails:"Jos tämä oli virhe, liity puheluun uudelleen.",leftCallTitle:"Poistuit puhelusta",lobbyScreenConnectingToCallTitle:"Liitytään puheluun",lobbyScreenWaitingToBeAdmittedTitle:"Odotetaan hyväksymistä mukaan kokoukseen",microphonePermissionDenied:"Selaimesi estää mikrofonin käytön",microphoneToggleInLobbyNotAllowed:"Mykistämistoimintoa ei voi käyttää odotustilassa.",mutedMessage:"Mikrofonisi on mykistetty",networkReconnectMoreDetails:"Vaikuttaa siltä, että tapahtui virhe. Sinua yritetään saada takaisin mukaan puheluun.",networkReconnectTitle:"Odota",notInvitedToRoomDetails:"Et voi liittyä tähän huoneeseen, koska sinulla ei ole kutsua.",notInvitedToRoomTitle:"Ei kutsuttu huoneeseen",peopleButtonLabel:"Ihmiset",selectedPeopleButtonLabel:"Henkilöt-painike valittu",peoplePaneTitle:"Ihmiset",peoplePaneMoreButtonAriaLabel:"Lisää",peopleButtonTooltipOpen:"Näytä osallistujat",peopleButtonTooltipClose:"Piilota osallistujat",peoplePaneSubTitle:"Tässä puhelussa {numberOfPeople}",privacyPolicy:"Tietosuojakäytännöt",rejoinCallButtonLabel:"Liity uudelleen puheluun",removedFromCallMoreDetails:"Toinen osallistuja poisti sinut puhelusta.",removedFromCallTitle:"Sinut on poistettu",removeMenuLabel:"Poista",startSpotlightMenuLabel:"Lisää valokeilaan kaikille",addSpotlightMenuLabel:"Lisää valokeilaan",stopSpotlightMenuLabel:"Poista valokeilasta",stopSpotlightOnSelfMenuLabel:"Valokeila",spotlightLimitReachedMenuTitle:"Spotlight-raja saavutettu",stopAllSpotlightMenuLabel:"Poista kaikki valokeilat",returnToCallButtonAriaDescription:"Palaa puheluun",returnToCallButtonAriaLabel:"Takaisin",roomNotFoundDetails:"Annettua huonetunnusta ei löydy.",roomNotFoundTitle:"Tilaa ei löytynyt",roomNotValidDetails:"Tämä huone ei ole kelvollinen tällä hetkellä.",roomNotValidTitle:"Huone ei ole kelvollinen",inviteToRoomRemovedDetails:"Kutsusi liittyä tähän huoneeseen poistettiin.",inviteToRoomRemovedTitle:"Kutsu huoneeseen poistettu",soundLabel:"Ääni",noMicrophonesLabel:"Mikrofoneja ei löytynyt",noSpeakersLabel:"Kaiuttimia ei löytynyt",startCallButtonLabel:"Aloita puhelu",openDialpadButtonLabel:"Valitse puhelinnumero",peoplePaneAddPeopleButtonLabel:"Lisää ihmisiä",dialpadStartCallButtonLabel:"Soita",dialpadModalTitle:"Valitse puhelinnumero",dialpadModalAriaLabel:"Valintapaneeli",dialpadCloseModalButtonAriaLabel:"Sulje valintapaneeli",moreButtonCallingLabel:"Lisää",resumeCallButtonLabel:"Jatka",resumingCallButtonLabel:"Jatketaan...",resumeCallButtonAriaLabel:"Jatka puhelua",resumingCallButtonAriaLabel:"Jatka puhelua",holdScreenLabel:"Olet pidossa",openDtmfDialpadLabel:"Näytä valintapaneeli",dtmfDialpadPlaceholderText:"Syötä numero",outboundCallingNoticeString:"Soitetaan...",participantJoinedNoticeString:"{displayName} liittyi",twoParticipantJoinedNoticeString:"{displayName1} ja {displayName2} ovat liittyneet",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} ja {displayName3} ovat liittyneet",participantLeftNoticeString:"{displayName} poistui",twoParticipantLeftNoticeString:"{displayName1} ja {displayName2} ovat poistuneet",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} ja {displayName3} ovat poistuneet",unnamedParticipantString:"nimetön osallistuja",manyUnnamedParticipantsJoined:"nimetön osallistuja ja {numOfParticipants} muuta osallistujaa liittyi",manyUnnamedParticipantsLeft:"nimetön osallistuja ja {numOfParticipants} muuta osallistujaa poistuivat",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} ja {numOfParticipants} muuta osallistujaa liittyivät",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} ja {numOfParticipants} muuta osallistujaa poistuivat",liveCaptionsLabel:"Live-tekstitykset",captionsSettingsLabel:"Tekstitysasetukset",startCaptionsButtonOnLabel:"Ota tekstitykset käyttöön",startCaptionsButtonOffLabel:"Poista tekstitykset käytöstä",startCaptionsButtonTooltipOnContent:"Poista tekstitykset käytöstä",startCaptionsButtonTooltipOffContent:"Ota tekstitykset käyttöön",captionsSettingsModalTitle:"Tekstitysasetukset",captionsSettingsSpokenLanguageDropdownLabel:"Puhuttu kieli",captionsSettingsCaptionLanguageDropdownLabel:"Tekstitysten kieli",captionsSettingsSpokenLanguageDropdownInfoText:"Kieli, jota kaikki tähän puheluun osallistuvat puhuvat.",captionsSettingsCaptionLanguageDropdownInfoText:"Tekstitys näkyy tällä kielellä.",captionsSettingsConfirmButtonLabel:"Vahvista",captionsSettingsCancelButtonLabel:"Peruuta",captionsSettingsModalAriaLabel:"Tekstitysasetus, modaalinen valintaikkuna",captionsSettingsCloseModalButtonAriaLabel:"Sulje tekstitysasetus",captionsBannerMoreButtonCallingLabel:"Tekstitysasetusten lisäasetukset",captionsBannerMoreButtonTooltip:"Lisää vaihtoehtoja",realTimeTextLabel:"Reaaliaikainen teksti (RTT)",startRealTimeTextLabel:"Ota reaaliaikainen kirjoitus käyttöön tässä puhelussa",disabledStartRealTimeTextLabel:"RTT on tällä hetkellä käytössä, sitä ei voi poistaa käytöstä",dismissModalAriaLabel:"Paikallinen kuva ja etäkuva, palaa puheluun painamalla Enter-näppäintä",callRejectedTitle:"Puhelua ei voitu soittaa",callRejectedMoreDetails:"Henkilö, jota yrität tavoittaa, ei ole tavoitettavissa. Yritä myöhemmin uudelleen.",callTimeoutTitle:"Puhelua ei voitu soittaa",callTimeoutDetails:"Henkilö, jota yrität tavoittaa, ei ole tavoitettavissa. Yritä myöhemmin uudelleen.",callTimeoutBotTitle:"Puhelua ei voitu soittaa",callTimeoutBotDetails:"Palvelu, jota yrität tavoittaa, ei ole käytettävissä. Yritä myöhemmin uudelleen.",dtmfDialerButtonLabel:"Valintapaneeli",dtmfDialerButtonTooltipOn:"Näytä valintapaneeli",dtmfDialerButtonTooltipOff:"Piilota valintapaneeli",dtmfDialerMoreButtonLabelOn:"Näytä valintapaneeli",dtmfDialerMoreButtonLabelOff:"Piilota valintapaneeli",phoneCallMoreButtonLabel:"Puhelu",spokenLanguageStrings:{"ar-ae":"arabia – Arabiemiirikunnat","ar-sa":"arabia – Saudi-Arabia","da-dk":"tanska","de-de":"saksa – Saksa","en-au":"englanti – Australia","en-ca":"englanti – Kanada","en-gb":"englanti – Yhdistynyt kuningaskunta","en-in":"englanti – Intia","en-nz":"englanti – Uusi-Seelanti","en-us":"englanti – Yhdysvallat","es-es":"espanja – Espanja (nykykieli)","es-mx":"espanja – Meksiko","fi-fi":"suomi","fr-ca":"ranska – Kanada","fr-fr":"ranska – Ranska","hi-in":"hindi","it-it":"italia – Italia","ja-jp":"japani","ko-kr":"korea","nb-no":"norja (bokmål)","nl-be":"hollanti – Belgia","nl-nl":"hollanti – Alankomaat","pl-pl":"puola","pt-br":"portugali – Brasilia","ru-ru":"venäjä","sv-se":"ruotsi","zh-cn":"kiina – Kiinan kansantasavalta","zh-hk":"kiina – Hongkong, erityishallintoalue","cs-cz":"tšekki","pt-pt":"portugali – Portugali","tr-tr":"turkki","vi-vn":"vietnam","th-th":"thai","he-il":"heprea","cy-gb":"kymri","uk-ua":"ukraina","el-gr":"kreikka","hu-hu":"unkari","ro-ro":"romania","sk-sk":"slovakki","zh-tw":"kiina – Taiwan"},captionLanguageStrings:{ar:"arabia",da:"tanska",de:"saksa",en:"englanti",es:"espanja",fi:"suomi","fr-ca":"ranska – Kanada",fr:"ranska – Ranska",hi:"hindi",it:"italia",ja:"japani",ko:"korea",nb:"norja (bokmål)",nl:"hollanti",pl:"puola",pt:"portugali – Brasilia",ru:"venäjä",sv:"ruotsi","zh-Hans":"kiina (yksinkertaistettu)","zh-Hant":"kiinalainen (perinteinen)",cs:"tšekki","pt-pt":"portugali – Portugali",tr:"turkki",vi:"vietnam",th:"thai",he:"heprea",cy:"kymri",uk:"ukraina",el:"kreikka",hu:"unkari",ro:"romania",sk:"slovakki"},captionsBannerSpinnerText:"Aloitetaan tekstitystä...",realTimeTextInputBoxDefaultText:"Kirjoita viesti reaaliaikaisesti",realTimeTextInputErrorMessage:"Viestissä voi olla enintään 2 000 merkkiä",realTimeTextBannerTitle:"Reaaliaikainen kirjoitus",realTimeTextBannerContent:"RTT (reaaliaikainen teksti) on käytössä kaikille osallistujille koko kokouksen keston ajan.",realTimeTextBannerLinkLabel:"Lue lisää",realTimeTextModalAriaLabel:"Ota modaalinen RTT-valintaikkuna käyttöön",realTimeTextModalTitle:"Otetaanko RTT käyttöön?",realTimeTextModalText:"Reaaliaikainen teksti (RTT) otetaan käyttöön kaikille kokouksen osallistuville.",realTimeTextConfirmButtonLabel:"Ota käyttöön",realTimeTextCancelButtonLabel:"Peruuta",realTimeTextCloseModalButtonAriaLabel:"Sulje modaalinen RTT-valintaikkuna",captionsOnlyContainerTitle:"Tekstitykset",realTimeTextOnlyContainerTitle:"Reaaliaikainen teksti",captionsAndRealTimeTextContainerTitle:"Tekstitykset ja reaaliaikainen teksti",expandButtonAriaLabel:"Laajenna",minimizeButtonAriaLabel:"Kutista",transferPageTransferorText:"Siirretään...",transferPageTransferTargetText:"Yhdistetään...",transferPageUnknownTransferorDisplayName:"Tuntematon",transferPageUnknownTransferTargetDisplayName:"Tuntematon",transferPageNoticeString:"Sinut siirretään",participantCouldNotBeReachedTitle:"Kohdeosallistuja ei ole tällä hetkellä tavoitettavissa",participantCouldNotBeReachedMoreDetails:"Ota yhteyttä osallistujaan, kun hän on tavoitettavissa",permissionToReachTargetParticipantNotAllowedTitle:"Kohdeosallistujan tavoittamisoikeutta ei sallita",permissionToReachTargetParticipantNotAllowedMoreDetails:"Tarkista, että kohdeosallistuja on samassa vuokraajassa",unableToResolveTenantTitle:"Kohdeosallistujan vuokraajan tunnusta ei voi selvittää",unableToResolveTenantMoreDetails:"Tarkista, että osallistujan tunnus on kirjoitettu oikein",participantIdIsMalformedTitle:"Osallistujan tunnus ei ole oikeassa muodossa",participantIdIsMalformedMoreDetails:"Tarkista, että osallistujan tunnus on oikeassa muodossa",moreButtonGalleryControlLabel:"Näkymä",moreButtonGalleryPositionToggleLabel:"Siirrä valikoima ylimmäiseksi",moreButtonGallerySpeakerLayoutLabel:"Kaiutin",moreButtonGalleryFloatingLocalLayoutLabel:"Dynaaminen",moreButtonGalleryDefaultLayoutLabel:"Valikoimanäkymä",moreButtonGalleryFocusedContentLayoutLabel:"Huomio sisältöön",moreButtonLargeGalleryDefaultLayoutLabel:"Suuri galleria",moreButtonTogetherModeLayoutLabel:"Yhdessä-tila",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Kamerasi on poistettu käytöstä. Et voi enää jakaa videota.",grantedDueToMeetingOption:"Kamera on otettu käyttöön. Ota se käyttöön, jos haluat jakaa videota."},unmuteMic:{lostDueToMeetingOption:"Mikrofonisi on poistettu käytöstä. Et voi enää poistaa mykistystä.",grantedDueToMeetingOption:"Mikrofoni on otettu käyttöön. Poista mykistys, jos haluat puhua."},shareScreen:{lostDueToRoleChangeToAttendee:"Roolisi on muuttunut. Jotkin toiminnot, kuten sisällön jakaminen, eivät ole käytettävissäsi.",grantedDueToRoleChangeToPresenter:"Olet esittäjä. Voit jakaa sisältöä ja johtaa kokousta."},hideAttendeeNames:{hideAttendeeNameAttendee:"Nimesi on piilotettu muilta osallistujilta. Järjestäjät ja esittäjät voivat nähdä oikean nimesi.",hideAttendeeNamePresenter:"Osallistujien nimet on piilotettu. Vain järjestäjät ja esittäjät voivat nähdä nimet osallistujien yksityisyyden suojaamiseksi."}},surveyTitle:"Auta kehittämään tuotetta",starSurveyHelperText:"Millainen oli puhelun äänenlaatu?",starSurveyOneStarText:"Laatu oli huono.",starSurveyTwoStarText:"Laatu oli heikko.",starSurveyThreeStarText:"Laatu oli hyvä.",starSurveyFourStarText:"Laatu oli erinomainen.",starSurveyFiveStarText:"Laatu oli erinomainen.",surveyConfirmButtonLabel:"Lähetä palautetta",starRatingAriaLabel:"Valitse {0}/{1} tähteä",surveySkipButtonLabel:"Ohita",tagsSurveyQuestion:"Mikä olisi voinut olla paremmin?",tagsSurveyTextFieldDefaultText:"Muu, anna kuvaus",tagsSurveyHelperText:"Tarkista havaitsematsi ongelmat",endOfSurveyText:"Kiitos palautteestasi!",surveyIssues:{overallRating:{callCannotJoin:"En pystynyt liittymään puheluun",callCannotInvite:"En pystynyt kutsumaan muita mukaan puheluun",hadToRejoin:"Minun oli pakko liittyä puheluun uudelleen",callEndedUnexpectedly:"Puhelu päättyi odottamatta",otherIssues:"Minulla oli muita ongelmia puhelun kanssa"},audioRating:{noLocalAudio:"Toinen osapuoli ei kuullut mitään",noRemoteAudio:"En kuullut mitään",echo:"Kuulin kaikua puhelun aikana",audioNoise:"Kuulin taustamelua puhelun aikana",lowVolume:"Äänenvoimakkuus oli matala",audioStoppedUnexpectedly:"Ääni lakkasi odottamatta kuulumasta",distortedSpeech:"Äänessä esiintyi vääristymiä",audioInterruption:"Ääni keskeytyi",otherIssues:"Minulla oli puhelun aikana muita ääneen liittyviä ongelmia"},videoRating:{noVideoReceived:"En nähnyt videokuvaa",noVideoSent:"Muut eivät pystyneet näkemään minua",lowQuality:"Videokuva oli laadultaan heikkoa",freezes:"Video jumitti",stoppedUnexpectedly:"Videokuva pysähtyi odottamatta",darkVideoReceived:"Näen ainoastaan tummia näyttöjä, kun toiset ottavat kameransa käyttöön",audioVideoOutOfSync:"Ääni ja kuva olivat epätahdissa",otherIssues:"Minulla oli puhelun aikana muita videokuvaan liittyviä ongelmia"},screenshareRating:{noContentLocal:"Muut henkilöt eivät nähneet jakamaani näyttöä",noContentRemote:"En nähnyt muiden jakamaa näyttöä",cannotPresent:"En voinut esittää näyttöni sisältöä",lowQuality:"Näytön jakaminen oli laadultaan heikkoa",freezes:"Näytön jakaminen jumitti",stoppedUnexpectedly:"Näytön jakaminen keskeytyi odottamatta",largeDelay:"Näytön jakamisessa ilmenee suurta viivettä",otherIssues:"Minulla oli puhelun aikana muita näytön jakamiseen liittyviä ongelmia"}},surveyIssuesHeadingStrings:{overallRating:"Yleinen",audioRating:"Äänet",videoRating:"Video",screenshareRating:"Esitys käynnissä"},spotlightPrompt:{startSpotlightHeading:"Lisätäänkö valokeilaan kaikille?",startSpotlightText:"Korostat videon kaikille kokouksen osallistujille.",startSpotlightOnSelfText:"Korostat videosi kaikille kokouksen osallistujille.",startSpotlightConfirmButtonLabel:"Lisää valokeilaan kaikille",startSpotlightCancelButtonLabel:"Peruuta",stopSpotlightHeading:"Haluatko poistaa videon valokeilasta?",stopSpotlightOnSelfHeading:"Haluatko poistua valokeilasta?",stopAllSpotlightHeading:"Poistetaanko kaikki videot valokeilasta?",stopSpotlightText:"Videota ei enää korosteta kaikille kokouksen osallistujille.",stopSpotlightOnSelfText:"Videota ei enää korosteta kaikille kokouksen osallistujille.",stopAllSpotlightText:"Videoita ei enää korosteta kaikille kokouksen osallistujille.",stopSpotlightConfirmButtonLabel:"Poista valokeilasta",stopSpotlightOnSelfConfirmButtonLabel:"Valokeila",stopSpotlightCancelButtonLabel:"Peruuta",closeSpotlightPromptButtonLabel:"Sulje"},exitSpotlightButtonLabel:"Poistu valokeilasta",exitSpotlightButtonTooltip:"Poistu valokeilasta",leaveConfirmButtonLabel:"Poistu",endCallConfirmButtonLabel:"Lopeta puhelu",hangUpCancelButtonLabel:"Peruuta",leaveConfirmDialogTitle:"Poistutaanko puhelusta?",leaveConfirmDialogContent:"Haluatko varmasti poistua puhelusta?",endCallConfirmDialogTitle:"Lopetetaanko puhelu?",endCallConfirmDialogContent:"Puhelu lopetetaan kaikkien osallistujien osalta.",invalidMeetingIdentifier:"Virheellinen kokoustunnus tai tunnuskoodi",pinParticipantMenuLabel:"Kiinnitä minulle",pinParticipantLimitReachedMenuLabel:"Kiinnitä (enimmäisraja saavutettu)",unpinParticipantMenuLabel:"Poista kiinnitys",pinParticipantMenuItemAriaLabel:"Kiinnitä minulle",unpinParticipantMenuItemAriaLabel:"Poista kiinnitys",muteAllMenuLabel:"Mykistä kaikki",muteAllDialogTitle:"Mykistetäänkö kaikki?",muteAllDialogContent:"Tämä toiminto mykistää kaikki muut kokouksen osallistujat paitsi sinut.",muteAllConfirmButtonLabel:"Mykistä kaikki",muteAllCancelButtonLabel:"Peruuta",joinBreakoutRoomButtonLabel:"Liity pienryhmätilaan",returnFromBreakoutRoomButtonLabel:"Poistu kokoustilasta",leaveBreakoutRoomAndMeetingButtonLabel:"Poistu kokouksesta",breakoutRoomJoinedNotificationTitle:"Olet liittynyt pienryhmätilaan {roomName}",joinBreakoutRoomBannerTitle:"Pienryhmätila {roomName} on avoinna",joinBreakoutRoomBannerButtonLabel:"Liity",returnFromBreakoutRoomBannerTitle:"Palaa pääkokoukseen",returnFromBreakoutRoomBannerButtonLabel:"Liity uudelleen",forbidAudioMenuLabel:"Poista mikrofoni käytöstä",permitAudioMenuLabel:"Salli mikrofoni",forbidOthersAudioDialogTitle:"Poistetaanko mikrofoni osallistujien käytöstä?",forbidOthersAudioDialogContent:"Osallistujat eivät voi poistaa omaa mykistystään. Järjestäjä ja esittäjät voivat sallia muiden mykistyksen poistamisen.",forbidOthersAudioConfirmButtonLabel:"Poista mikrofonit käytöstä",forbidOthersAudioCancelButtonLabel:"Peruuta",permitOthersAudioDialogTitle:"Sallitaanko mikrofoni osallistujille?",permitOthersAudioDialogContent:"Kaikki kokoukseen osallistuvat voivat poistaa mykistyksensä.",permitOthersAudioConfirmButtonLabel:"Salli mikrofonit",permitOthersAudioCancelButtonLabel:"Peruuta",forbidOthersAudioMenuLabel:"Poista mikrofoni osallistujien käytöstä",permitOthersAudioMenuLabel:"Salli mikrofoni osallistujille",forbidVideoMenuLabel:"Poista kamera käytöstä",permitVideoMenuLabel:"Salli kamera",forbidOthersVideoDialogTitle:"Poistetaanko kamera osallistujien käytöstä?",forbidOthersVideoDialogContent:"Osallistujat eivät voi ottaa itse videota käyttöön. Järjestäjä ja esittäjät voivat antaa käyttäjien ottaa kameran käyttöön tarvittaessa.",forbidOthersVideoConfirmButtonLabel:"Poista kamerat käytöstä",forbidOthersVideoCancelButtonLabel:"Peruuta",permitOthersVideoDialogTitle:"Sallitaanko kamera osallistujille?",permitOthersVideoDialogContent:"Kaikki kokouksen osallistujat voivat itse ottaa videon käyttöön.",permitOthersVideoConfirmButtonLabel:"Salli kamerat",permitOthersVideoCancelButtonLabel:"Peruuta",forbidOthersVideoMenuLabel:"Poista kamera osallistujien käytöstä",permitOthersVideoMenuLabel:"Salli kamera osallistujille"};var chat$f={chatListHeader:"Tässä keskustelussa",uploadAttachment:"Lataa liite",uploadImageDataNotProvided:"Kuvatietoja ei ole annettu.",uploadImageIsTooLarge:"Kuva on liian suuri. Valitse sellainen, jonka koko on alle {maxImageSize} Mt.",uploadImageExtensionIsNotAllowed:".{imageExtension}-kuvan lataamista ei sallita.",uploadImageFailed:"Kuvaa ei voi ladata palvelimeen. Yritä myöhemmin uudelleen."};var callWithChat$f={chatButtonLabel:"Keskustelu",chatButtonNewMessageNotificationLabel:"Uusi viesti",chatButtonTooltipClosedWithMessageCount:"Näytä keskustelu ({unreadMessagesCount} lukematonta viestiä)",chatButtonTooltipClose:"Piilota keskustelu",chatButtonTooltipOpen:"Näytä keskustelu",chatPaneTitle:"Keskustelu",copyInviteLinkButtonLabel:"Kopioi kutsulinkki",copyInviteLinkButtonActionedLabel:"Linkki kopioitu",copyInviteLinkActionedAriaLabel:"Kutsulinkki kopioitu",dismissSidePaneButtonLabel:"Sulje",moreDrawerAudioDeviceMenuTitle:"Äänilaite",moreDrawerButtonLabel:"Lisää vaihtoehtoja",moreDrawerButtonTooltip:"Lisää vaihtoehtoja",moreDrawerMicrophoneMenuTitle:"Mikrofoni",moreDrawerSpeakerMenuTitle:"Kaiutin",moreDrawerCaptionsMenuTitle:"Live-tekstitykset",moreDrawerSpokenLanguageMenuTitle:"Puhuttu kieli",moreDrawerCaptionLanguageMenuTitle:"Tekstityksen kieli",moreDrawerGalleryOptionsMenuTitle:"Valikoiman asetukset",peopleButtonLabel:"Ihmiset",selectedPeopleButtonLabel:"Henkilöt-painike valittu",peopleButtonTooltipOpen:"Näytä osallistujat",peopleButtonTooltipClose:"Piilota osallistujat",peopleButtonTooltipOpenAriaLabel:"Henkilöt-painike, Näytä osallistujat",peopleButtonTooltipCloseAriaLabel:"Henkilöt-painike, Piilota osallistujat",peoplePaneSubTitle:"Tässä puhelussa",peoplePaneTitle:"Ihmiset",pictureInPictureTileAriaLabel:"Videosyötteet. Palaa puhelunäyttöön napsauttamalla.",removeMenuLabel:"Poista",openDialpadButtonLabel:"Valitse puhelinnumero",returnToCallButtonAriaDescription:"Palaa puheluun",returnToCallButtonAriaLabel:"Takaisin",peoplePaneAddPeopleButtonLabel:"Lisää ihmisiä",dialpadStartCallButtonLabel:"Soita",dialpadModalTitle:"Valitse puhelinnumero",dialpadModalAriaLabel:"Valintapaneeli",dialpadCloseModalButtonAriaLabel:"Sulje valintapaneeli",openDtmfDialpadLabel:"Näytä valintapaneeli",dtmfDialpadPlaceholderText:"Syötä numero",breakoutRoomChatPaneTitle:"Pienryhmätilan keskustelu",chatContentSpinnerLabel:"Ladataan keskustelua"};var fi_FI = {call:call$f,chat:chat$f,callWithChat:callWithChat$f};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Finnish (Finland)
 *
 * @public
 */
const COMPOSITE_LOCALE_FI_FI = {
    component: COMPONENT_LOCALE_FI_FI,
    strings: createCompositeStrings(fi_FI)
};

var call$e={cameraLabel:"Appareil photo",noCamerasLabel:"Aucune caméra trouvée",cameraPermissionDenied:"Votre navigateur bloque l’accès à votre caméra",cameraTurnedOff:"Votre caméra est désactivée",chatButtonLabel:"Conversation",close:"Fermer",complianceBannerNowOnlyRecording:"Vous n’enregistrez désormais que cette réunion.",complianceBannerNowOnlyTranscription:"À présent, vous ne transcrivez que cette réunion.",complianceBannerRecordingAndTranscriptionSaved:"Sauvegarde en cours de l’enregistrement et de la transcription.",complianceBannerRecordingAndTranscriptionStarted:"Démarrage de l’enregistrement et de la transcription.",complianceBannerRecordingAndTranscriptionStopped:"L’enregistrement et la transcription ont pris fin.",complianceBannerRecordingSaving:"Sauvegarde en cours de l'enregistrement.",complianceBannerRecordingStarted:"L’enregistrement a démarré.",complianceBannerRecordingStopped:"L’enregistrement a pris fin.",complianceBannerTranscriptionStarted:"Démarrage de la transcription.",complianceBannerTranscriptionConsent:"En participant, vous autorisez la transcription de cette réunion.",complianceBannerTranscriptionSaving:"Sauvegarde en cours de la transcription.",complianceBannerTranscriptionStopped:"La transcription a pris fin.",configurationPageTitle:"Lancer un appel",copyInviteLinkButtonLabel:"Copier le lien d'invitation",copyInviteLinkButtonActionedLabel:"Lien copié",copyInviteLinkActionedAriaLabel:"Le lien d’invitation a été copié.",defaultPlaceHolder:"Sélectionnez une option",dismissSidePaneButtonLabel:"Fermer",videoEffectsPaneTitle:"Effets",videoEffectsPaneBackgroundSelectionTitle:"Arrière-plan",videoEffectsPaneAriaLabel:"Volet Effets vidéo",configurationPageCameraIsLoadingLabel:"Chargement de la vidéo...",configurationPageVideoEffectsButtonLabel:"Effets",unableToStartVideoEffect:"Impossible d’appliquer l’effet vidéo.",blurBackgroundEffectButtonLabel:"Flouter",blurBackgroundTooltip:"Flouter l'arrière-plan",removeBackgroundEffectButtonLabel:"Aucun",removeBackgroundTooltip:"Supprimer l’arrière-plan",cameraOffBackgroundEffectWarningText:"Votre caméra est désactivée. Activez la caméra pour voir l’effet vidéo.",failedToJoinCallDueToNoNetworkMoreDetails:"L’appel a été déconnecté en raison d’un problème réseau. Vérifiez votre connexion et rejoignez-la à nouveau.",failedToJoinCallDueToNoNetworkTitle:"Appel déconnecté",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Vous n’avez pas reçu d’entrée dans l’appel. S’il s’agissait d’une erreur, rejoignez à nouveau l’appel.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Rejeté de la salle d’attente",learnMore:"En savoir plus",leavingCallTitle:"Quitter...",leftCallMoreDetails:"S’il s’agissait d’une erreur, rejoignez à nouveau l’appel.",leftCallTitle:"Vous avez quitté l’appel.",lobbyScreenConnectingToCallTitle:"Connexion à l'appel",lobbyScreenWaitingToBeAdmittedTitle:"En attente d’admission",microphonePermissionDenied:"Votre navigateur bloque l’accès à votre micro",microphoneToggleInLobbyNotAllowed:"Impossible de désactiver ou d’activer le son dans la salle d’attente.",mutedMessage:"Le son de votre micro est désactivé.",networkReconnectMoreDetails:"On dirait qu'une erreur est survenue. Nous tentons de vous reconnecter à l’appel.",networkReconnectTitle:"Patience",notInvitedToRoomDetails:"Vous ne pouvez pas rejoindre cette salle, car vous n’avez pas d’invitation.",notInvitedToRoomTitle:"N’a pas été invité à la salle",peopleButtonLabel:"Personnes",selectedPeopleButtonLabel:"Bouton Contacts sélectionné",peoplePaneTitle:"Personnes",peoplePaneMoreButtonAriaLabel:"Autres",peopleButtonTooltipOpen:"Afficher les participants",peopleButtonTooltipClose:"Masquer les participants",peoplePaneSubTitle:"Dans cet appel {numberOfPeople}",privacyPolicy:"Politique de confidentialité",rejoinCallButtonLabel:"Rejoindre l’appel à nouveau.",removedFromCallMoreDetails:"Un autre participant vous a supprimé de l’appel.",removedFromCallTitle:"Vous avez été supprimé.",removeMenuLabel:"Supprimer",startSpotlightMenuLabel:"Mettre en évidence pour tout le monde",addSpotlightMenuLabel:"Ajouter la mise en évidence",stopSpotlightMenuLabel:"Arrêter la mise en évidence",stopSpotlightOnSelfMenuLabel:"Quitter la mise en évidence",spotlightLimitReachedMenuTitle:"Limite Spotlight atteinte",stopAllSpotlightMenuLabel:"Arrêtez tous les projecteurs",returnToCallButtonAriaDescription:"Revenir à l’appel",returnToCallButtonAriaLabel:"Précédent",roomNotFoundDetails:"L’ID de la salle fourni est introuvable.",roomNotFoundTitle:"Salle introuvable",roomNotValidDetails:"Cette salle n’est pas valide pour l’instant.",roomNotValidTitle:"Salle non valide",inviteToRoomRemovedDetails:"Votre invitation à rejoindre cette salle a été supprimée.",inviteToRoomRemovedTitle:"Invitation à la salle supprimée",soundLabel:"Son",noMicrophonesLabel:"Aucun microphone trouvé",noSpeakersLabel:"Aucun haut-parleur trouvé",startCallButtonLabel:"Lancer l'appel",openDialpadButtonLabel:"Composer le numéro de téléphone",peoplePaneAddPeopleButtonLabel:"Ajouter des personnes",dialpadStartCallButtonLabel:"Appeler",dialpadModalTitle:"Composer le numéro de téléphone",dialpadModalAriaLabel:"Clavier",dialpadCloseModalButtonAriaLabel:"Fermer le clavier",moreButtonCallingLabel:"Plus",resumeCallButtonLabel:"Reprendre",resumingCallButtonLabel:"Reprise...",resumeCallButtonAriaLabel:"Reprendre l’appel",resumingCallButtonAriaLabel:"Reprendre l’appel",holdScreenLabel:"Vous avez été mis(e) en attente.",openDtmfDialpadLabel:"Afficher le clavier",dtmfDialpadPlaceholderText:"Saisir un numéro",outboundCallingNoticeString:"Appel en cours...",participantJoinedNoticeString:"{displayName} a rejoint",twoParticipantJoinedNoticeString:"{displayName1} et {displayName2} ont rejoint",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} et {displayName3} ont rejoint",participantLeftNoticeString:"{displayName} a quitté",twoParticipantLeftNoticeString:"{displayName1} et {displayName2} sont partis",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} et {displayName3} ont quitté",unnamedParticipantString:"participant sans nom",manyUnnamedParticipantsJoined:"participant sans nom et {numOfParticipants} autres participants ont rejoint",manyUnnamedParticipantsLeft:"participant sans nom et {numOfParticipants} autres participants ont quitté",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} et {numOfParticipants} autres participants ont rejoint",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} et {numOfParticipants} autres participants restants",liveCaptionsLabel:"Sous-titres en direct",captionsSettingsLabel:"Paramètres de sous-titres",startCaptionsButtonOnLabel:"Activer les sous-titres",startCaptionsButtonOffLabel:"Désactiver les sous-titres",startCaptionsButtonTooltipOnContent:"Désactiver les sous-titres",startCaptionsButtonTooltipOffContent:"Activer les sous-titres",captionsSettingsModalTitle:"Paramètres des sous-titres",captionsSettingsSpokenLanguageDropdownLabel:"Langue parlée",captionsSettingsCaptionLanguageDropdownLabel:"Langue des sous-titres",captionsSettingsSpokenLanguageDropdownInfoText:"Langue parlée par tous les participants à cet appel.",captionsSettingsCaptionLanguageDropdownInfoText:"Les sous-titres apparaîtront dans cette langue.",captionsSettingsConfirmButtonLabel:"Confirmer",captionsSettingsCancelButtonLabel:"Annuler",captionsSettingsModalAriaLabel:"Paramétrage des légendes Modal",captionsSettingsCloseModalButtonAriaLabel:"Paramètres de fermeture des sous-titres",captionsBannerMoreButtonCallingLabel:"Autres options de paramètres de sous-titres",captionsBannerMoreButtonTooltip:"Autres options",realTimeTextLabel:"Texte en temps réel (RTT)",startRealTimeTextLabel:"Activer RTT pour cet appel",disabledStartRealTimeTextLabel:"RTT est actuellement activé, il ne peut pas être désactivé",dismissModalAriaLabel:"Image et image locales et distantes, appuyez sur entrée pour revenir à l'appel",callRejectedTitle:"Nous n’avons pas pu effectuer l’appel",callRejectedMoreDetails:"La personne que vous essayez de rejoindre n’est pas disponible. Veuillez réessayer plus tard.",callTimeoutTitle:"Nous n’avons pas pu effectuer l’appel",callTimeoutDetails:"La personne que vous essayez de rejoindre n’est pas disponible. Veuillez réessayer plus tard.",callTimeoutBotTitle:"Nous n’avons pas pu effectuer l’appel",callTimeoutBotDetails:"Le service que vous essayez d’atteindre n’est pas disponible. Veuillez réessayer plus tard.",dtmfDialerButtonLabel:"Clavier",dtmfDialerButtonTooltipOn:"Afficher le clavier",dtmfDialerButtonTooltipOff:"Masquez le clavier",dtmfDialerMoreButtonLabelOn:"Afficher le clavier",dtmfDialerMoreButtonLabelOff:"Masquez le clavier",phoneCallMoreButtonLabel:"Appel téléphonique",spokenLanguageStrings:{"ar-ae":"Arabe – E.A.U.","ar-sa":"Arabe – Arabie saoudite","da-dk":"Danois","de-de":"Allemand – Allemagne","en-au":"Anglais – Australie","en-ca":"Anglais – Canada","en-gb":"Anglais – Royaume-Uni","en-in":"Anglais – Inde","en-nz":"Anglais – Nouvelle-Zélande","en-us":"Anglais – États-Unis","es-es":"Espagnol – Espagne (tri moderne)","es-mx":"Espagnol – Mexique","fi-fi":"Finnois","fr-ca":"Français – Canada","fr-fr":"Français – France","hi-in":"Hindi","it-it":"Italien – Italie","ja-jp":"Japonais","ko-kr":"Coréen","nb-no":"Norvégien (Bokmål)","nl-be":"Néerlandais – Belgique","nl-nl":"Néerlandais – Pays-Bas","pl-pl":"Polonais","pt-br":"Portugais – Brésil","ru-ru":"Russe","sv-se":"Suédois","zh-cn":"Chinois – République populaire de Chine","zh-hk":"Chinois – Hong Kong SAR","cs-cz":"Tchèque","pt-pt":"Portugais – Portugal","tr-tr":"Turc","vi-vn":"Vietnamien","th-th":"Thaï","he-il":"Hébreu","cy-gb":"Gallois","uk-ua":"Ukrainien","el-gr":"Grec","hu-hu":"Hongrois","ro-ro":"Roumain","sk-sk":"Slovaque","zh-tw":"Chinois – Taiwan"},captionLanguageStrings:{ar:"Arabe",da:"Danois",de:"Allemand",en:"Anglais",es:"Espagnol",fi:"Finnois","fr-ca":"Français – Canada",fr:"Français – France",hi:"Hindi",it:"Italien",ja:"Japonais",ko:"Coréen",nb:"Norvégien (Bokmål)",nl:"Néerlandais",pl:"Polonais",pt:"Portugais – Brésil",ru:"Russe",sv:"Suédois","zh-Hans":"Chinois (simplifié)","zh-Hant":"Chinois (traditionnel)",cs:"Tchèque","pt-pt":"Portugais – Portugal",tr:"Turc",vi:"Vietnamien",th:"Thaï",he:"Hébreu",cy:"Gallois",uk:"Ukrainien",el:"Grec",hu:"Hongrois",ro:"Roumain",sk:"Slovaque"},captionsBannerSpinnerText:"Démarrage des sous-titres...",realTimeTextInputBoxDefaultText:"Tapez votre message en temps réel",realTimeTextInputErrorMessage:"Le message ne peut pas dépasser 2 000 caractères",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (texte en temps réel) est activé pour tous les participants pendant toute la durée de la réunion.",realTimeTextBannerLinkLabel:"En savoir plus",realTimeTextModalAriaLabel:"Activer la boîte de dialogue modale RTT",realTimeTextModalTitle:"Activer la RTT ?",realTimeTextModalText:"Le texte en temps réel (RTT) sera activé pour tous les participants à la réunion.",realTimeTextConfirmButtonLabel:"Activer",realTimeTextCancelButtonLabel:"Annuler",realTimeTextCloseModalButtonAriaLabel:"Fermer la boîte de dialogue modale RTT",captionsOnlyContainerTitle:"Sous-titres",realTimeTextOnlyContainerTitle:"Texte en temps réel",captionsAndRealTimeTextContainerTitle:"Légendes et texte en temps réel",expandButtonAriaLabel:"Développer",minimizeButtonAriaLabel:"Réduire",transferPageTransferorText:"Transfert en cours...",transferPageTransferTargetText:"Connexion en cours...",transferPageUnknownTransferorDisplayName:"Inconnu",transferPageUnknownTransferTargetDisplayName:"Inconnu",transferPageNoticeString:"Vous êtes en cours de transfert",participantCouldNotBeReachedTitle:"Le participant cible n’est pas disponible actuellement",participantCouldNotBeReachedMoreDetails:"Contactez le participant lorsqu’il est disponible",permissionToReachTargetParticipantNotAllowedTitle:"L’autorisation d’atteindre le participant cible n’est pas autorisée",permissionToReachTargetParticipantNotAllowedMoreDetails:"Veuillez case activée que le participant cible se trouve dans le même client",unableToResolveTenantTitle:"Impossible de résoudre l’ID de locataire pour le participant cible",unableToResolveTenantMoreDetails:"Veuillez case activée l’ID de participant est entré correctement",participantIdIsMalformedTitle:"Le format de l’ID de participant n’est pas correct",participantIdIsMalformedMoreDetails:"Veuillez case activée que le format de l’ID de participant est correct",moreButtonGalleryControlLabel:"Afficher",moreButtonGalleryPositionToggleLabel:"Déplacer la galerie vers le haut",moreButtonGallerySpeakerLayoutLabel:"Haut-parleur",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamique",moreButtonGalleryDefaultLayoutLabel:"Vue Galerie",moreButtonGalleryFocusedContentLayoutLabel:"Focalisation sur le contenu",moreButtonLargeGalleryDefaultLayoutLabel:"Grande galerie",moreButtonTogetherModeLayoutLabel:"Mode ensemble",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Votre caméra a été désactivée. Vous ne pouvez plus partager de vidéo.",grantedDueToMeetingOption:"Votre caméra a été activée. Activez-la si vous souhaitez partager la vidéo."},unmuteMic:{lostDueToMeetingOption:"Votre micro a été désactivé. Vous ne pouvez plus activer le son.",grantedDueToMeetingOption:"Votre microphone a été activé. Activez le microphone pour parler."},shareScreen:{lostDueToRoleChangeToAttendee:"Votre rôle a été modifié. Certaines actions, telles que le partage de contenu, ne seront pas disponibles pour vous.",grantedDueToRoleChangeToPresenter:"Vous êtes présentateur. Vous pouvez partager du contenu et organiser la réunion."},hideAttendeeNames:{hideAttendeeNameAttendee:"Votre nom est masqué pour les autres participants. Les organisateurs et les présentateurs peuvent voir votre vrai nom.",hideAttendeeNamePresenter:"Les noms des participants sont masqués. Afin de protéger la vie privée des participants, seuls les organisateurs et les présentateurs peuvent visualiser les noms."}},surveyTitle:"Aidez-nous à nous améliorer",starSurveyHelperText:"Quelle a été la qualité globale de cet appel ?",starSurveyOneStarText:"La qualité était mauvaise.",starSurveyTwoStarText:"La qualité était mauvaise.",starSurveyThreeStarText:"La qualité était bonne.",starSurveyFourStarText:"La qualité était excellente.",starSurveyFiveStarText:"La qualité était excellente.",surveyConfirmButtonLabel:"Envoyer des commentaires",starRatingAriaLabel:"Sélectionner {0} étoiles sur {1}",surveySkipButtonLabel:"Ignorer",tagsSurveyQuestion:"Que pouvons-nous améliorer ?",tagsSurveyTextFieldDefaultText:"Autre, veuillez préciser",tagsSurveyHelperText:"Cochez les problèmes que vous avez rencontrés",endOfSurveyText:"Merci pour vos commentaires !",surveyIssues:{overallRating:{callCannotJoin:"Je n’ai pas pu rejoindre l’appel",callCannotInvite:"Je n’ai pas pu inviter d’autres personnes à participer à l’appel",hadToRejoin:"J’ai dû rejoindre l’appel",callEndedUnexpectedly:"L’appel s’est terminé pour moi de manière inattendue",otherIssues:"J’avais d’autres problèmes avec l’appel"},audioRating:{noLocalAudio:"Mon interlocuteur n'entendait aucun son",noRemoteAudio:"Je n'entendais aucun son",echo:"J’ai entendu des échos lors de l’appel",audioNoise:"Il y avait des bruits parasites",lowVolume:"Le volume était faible",audioStoppedUnexpectedly:"L’audio s’est arrêté de manière inattendue",distortedSpeech:"L’audio a été altéré",audioInterruption:"L’audio a été interrompu",otherIssues:"J’ai rencontré d’autres problèmes audio pendant cet appel"},videoRating:{noVideoReceived:"La vidéo ne s'affichait pas sur mon écran",noVideoSent:"Les autres personnes n’ont pas pu me voir",lowQuality:"La qualité de la vidéo était faible",freezes:"Vidéo figée",stoppedUnexpectedly:"La vidéo s'est arrêtée de manière inattendue",darkVideoReceived:"Je ne vois que des écrans sombres lorsque d'autres personnes allument leur caméra",audioVideoOutOfSync:"L’audio et la vidéo n’étaient pas synchronisés",otherIssues:"J’ai rencontré d’autres problèmes vidéo pendant cet appel"},screenshareRating:{noContentLocal:"Les autres personnes n’ont pas pu voir mon partage d’écran",noContentRemote:"Je n’ai pas pu voir le partage d’écran d’autres personnes",cannotPresent:"Je n’ai pas pu présenter mon écran",lowQuality:"La qualité du partage d’écran était faible",freezes:"Partage d’écran figé",stoppedUnexpectedly:"Le partage d’écran s’est arrêté de manière inattendue",largeDelay:"Le partage d’écran a un retard important",otherIssues:"Je rencontre d’autres problèmes de partage d’écran dans cet appel"}},surveyIssuesHeadingStrings:{overallRating:"Globale",audioRating:"Audio",videoRating:"Vidéo",screenshareRating:"En présentation"},spotlightPrompt:{startSpotlightHeading:"Mettre en évidence pour tout le monde ?",startSpotlightText:"Votre vidéo ne sera plus mise en surbrillance pour tous les participants à la réunion.",startSpotlightOnSelfText:"Vous mettez en surbrillance votre vidéo pour tous les participants à la réunion.",startSpotlightConfirmButtonLabel:"Mettre en évidence pour tout le monde",startSpotlightCancelButtonLabel:"Annuler",stopSpotlightHeading:"Arrêter la mise en évidence de cette vidéo ?",stopSpotlightOnSelfHeading:"Quitter la mise en évidence ?",stopAllSpotlightHeading:"Voulez-vous arrêter de mettre en évidence toutes les vidéos ?",stopSpotlightText:"Cette vidéo ne sera plus mise en évidence pour tous les participants à la réunion.",stopSpotlightOnSelfText:"Votre vidéo ne sera plus mise en surbrillance pour tous les participants à la réunion.",stopAllSpotlightText:"Les vidéo n’apparaissent plus en surbrillance pour tous les participants à la réunion.",stopSpotlightConfirmButtonLabel:"Arrêter la mise en évidence",stopSpotlightOnSelfConfirmButtonLabel:"Quitter la mise en évidence",stopSpotlightCancelButtonLabel:"Annuler",closeSpotlightPromptButtonLabel:"Fermez"},exitSpotlightButtonLabel:"Quitter la mise en évidence",exitSpotlightButtonTooltip:"Quitter la mise en évidence",leaveConfirmButtonLabel:"Partir",endCallConfirmButtonLabel:"Fin d’appel",hangUpCancelButtonLabel:"Annuler",leaveConfirmDialogTitle:"Quitter l'appel ?",leaveConfirmDialogContent:"Voulez-vous vraiment quitter l’appel ?",endCallConfirmDialogTitle:"Mettre fin à l'appel ?",endCallConfirmDialogContent:"Vous mettrez fin à l'appel pour tout le monde.",invalidMeetingIdentifier:"ID ou code secret de réunion non valide",pinParticipantMenuLabel:"Épingler pour moi",pinParticipantLimitReachedMenuLabel:"Épingler (limite atteinte)",unpinParticipantMenuLabel:"Désépingler",pinParticipantMenuItemAriaLabel:"Épingler pour moi",unpinParticipantMenuItemAriaLabel:"Désépingler",muteAllMenuLabel:"Désactiver le son de tous",muteAllDialogTitle:"Désactiver le son de tout le monde ?",muteAllDialogContent:"Cette opération va désactiver le son de tous les participants à la réunion, sauf vous.",muteAllConfirmButtonLabel:"Désactiver le son de tous",muteAllCancelButtonLabel:"Annuler",joinBreakoutRoomButtonLabel:"Rejoindre la salle de réunion",returnFromBreakoutRoomButtonLabel:"Quitter la salle",leaveBreakoutRoomAndMeetingButtonLabel:"Quitter la réunion",breakoutRoomJoinedNotificationTitle:"Vous avez rejoint la salle pour petit groupe « {roomName} »",joinBreakoutRoomBannerTitle:"La salle pour petit groupe « {roomName} » est ouverte",joinBreakoutRoomBannerButtonLabel:"Joindre",returnFromBreakoutRoomBannerTitle:"Revenir à la réunion principale",returnFromBreakoutRoomBannerButtonLabel:"Rejoindre",forbidAudioMenuLabel:"Désactiver le micro",permitAudioMenuLabel:"Autoriser le micro",forbidOthersAudioDialogTitle:"Désactiver le micro des participants ?",forbidOthersAudioDialogContent:"Les participants ne pourront pas activer leur micro. L’organisateur et les présentateurs peuvent permettre aux utilisateurs de désactiver le son selon les besoins.",forbidOthersAudioConfirmButtonLabel:"Désactiver les micros",forbidOthersAudioCancelButtonLabel:"Annuler",permitOthersAudioDialogTitle:"Autoriser le micro pour des participants ?",permitOthersAudioDialogContent:"Chaque participant à la réunion sera en mesure d’activer le son.",permitOthersAudioConfirmButtonLabel:"Autoriser les micros",permitOthersAudioCancelButtonLabel:"Annuler",forbidOthersAudioMenuLabel:"Désactiver le micro des participants",permitOthersAudioMenuLabel:"Autoriser le micro des participants",forbidVideoMenuLabel:"Désactiver la caméra",permitVideoMenuLabel:"Autoriser la caméra",forbidOthersVideoDialogTitle:"Désactiver la caméra des participants ?",forbidOthersVideoDialogContent:"Les participants ne pourront pas activer la vidéo eux-mêmes. L’organisateur et les présentateurs peuvent autoriser les utilisateurs à activer la caméra selon les besoins.",forbidOthersVideoConfirmButtonLabel:"Désactiver les caméras",forbidOthersVideoCancelButtonLabel:"Annuler",permitOthersVideoDialogTitle:"Autoriser la caméra des participants ?",permitOthersVideoDialogContent:"Tous les participants à la réunion pourront activer la vidéo eux-mêmes.",permitOthersVideoConfirmButtonLabel:"Autoriser les caméras",permitOthersVideoCancelButtonLabel:"Annuler",forbidOthersVideoMenuLabel:"Désactiver la caméra des participants",permitOthersVideoMenuLabel:"Autoriser la caméra des participants"};var chat$e={chatListHeader:"Dans cette conversation",uploadAttachment:"Charger une pièce jointe",uploadImageDataNotProvided:"Aucune donnée d'image n'est fournie.",uploadImageIsTooLarge:"L’image est trop grande. Choisissez une taille inférieure à {maxImageSize} Mo.",uploadImageExtensionIsNotAllowed:"Il est interdit de télécharger une image .{imageExtension}.",uploadImageFailed:"Impossible de charger l’image. Veuillez réessayer plus tard."};var callWithChat$e={chatButtonLabel:"Clavardage",chatButtonNewMessageNotificationLabel:"Nouveau message",chatButtonTooltipClosedWithMessageCount:"Afficher la conversation ({unreadMessagesCount} non lu)",chatButtonTooltipClose:"Masquer la conversation",chatButtonTooltipOpen:"Afficher la conversation",chatPaneTitle:"Clavardage",copyInviteLinkButtonLabel:"Copier le lien d'invitation",copyInviteLinkButtonActionedLabel:"Lien copié",copyInviteLinkActionedAriaLabel:"Le lien d’invitation a été copié.",dismissSidePaneButtonLabel:"Fermer",moreDrawerAudioDeviceMenuTitle:"Périphérique audio",moreDrawerButtonLabel:"Autres options",moreDrawerButtonTooltip:"Autres options",moreDrawerMicrophoneMenuTitle:"Microphone",moreDrawerSpeakerMenuTitle:"Haut-parleur",moreDrawerCaptionsMenuTitle:"Sous-titres en direct",moreDrawerSpokenLanguageMenuTitle:"Langue parlée",moreDrawerCaptionLanguageMenuTitle:"Langue des sous-titres",moreDrawerGalleryOptionsMenuTitle:"Options de galerie",peopleButtonLabel:"Personnes",selectedPeopleButtonLabel:"Bouton Contacts sélectionné",peopleButtonTooltipOpen:"Afficher les participants",peopleButtonTooltipClose:"Masquer les participants",peopleButtonTooltipOpenAriaLabel:"Bouton Contacts, Afficher les participants",peopleButtonTooltipCloseAriaLabel:"Bouton Contacts, Masquer les participants",peoplePaneSubTitle:"Pendant cet appel",peoplePaneTitle:"Personnes",pictureInPictureTileAriaLabel:"Flux vidéo. Cliquez pour revenir à l’écran d’appel.",removeMenuLabel:"Supprimer",openDialpadButtonLabel:"Composer le numéro de téléphone",returnToCallButtonAriaDescription:"Revenir à l’appel",returnToCallButtonAriaLabel:"Précédent",peoplePaneAddPeopleButtonLabel:"Ajouter des personnes",dialpadStartCallButtonLabel:"Appeler",dialpadModalTitle:"Composer le numéro de téléphone",dialpadModalAriaLabel:"Clavier",dialpadCloseModalButtonAriaLabel:"Fermer le clavier",openDtmfDialpadLabel:"Afficher le clavier",dtmfDialpadPlaceholderText:"Saisir un numéro",breakoutRoomChatPaneTitle:"Conversation dans une salle pour petit groupe",chatContentSpinnerLabel:"Chargement de la conversation"};var fr_FR = {call:call$e,chat:chat$e,callWithChat:callWithChat$e};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for French (France)
 *
 * @public
 */
const COMPOSITE_LOCALE_FR_FR = {
    component: COMPONENT_LOCALE_FR_FR,
    strings: createCompositeStrings(fr_FR)
};

var call$d={cameraLabel:"Caméra",noCamerasLabel:"Aucune caméra trouvée",cameraPermissionDenied:"Votre navigateur bloque l’accès à votre caméra",cameraTurnedOff:"Votre caméra est désactivée",chatButtonLabel:"Clavardage",close:"Fermer",complianceBannerNowOnlyRecording:"Vous n’enregistrez désormais que cette réunion.",complianceBannerNowOnlyTranscription:"À présent, vous ne transcrivez que cette réunion.",complianceBannerRecordingAndTranscriptionSaved:"Sauvegarde en cours de l’enregistrement et de la transcription.",complianceBannerRecordingAndTranscriptionStarted:"Démarrage de l’enregistrement et de la transcription.",complianceBannerRecordingAndTranscriptionStopped:"L’enregistrement et la transcription ont pris fin.",complianceBannerRecordingSaving:"Enregistrement en cours de sauvegarde.",complianceBannerRecordingStarted:"L’enregistrement a démarré.",complianceBannerRecordingStopped:"L’enregistrement s’est arrêté.",complianceBannerTranscriptionStarted:"La transcription a démarré.",complianceBannerTranscriptionConsent:"En participant, vous autorisez la transcription de cette réunion.",complianceBannerTranscriptionSaving:"La transcription est en cours d’enregistrement.",complianceBannerTranscriptionStopped:"Arrêt de la transcription.",configurationPageTitle:"Lancer un appel",copyInviteLinkButtonLabel:"Copier le lien d’invitation",copyInviteLinkButtonActionedLabel:"Lien copié",copyInviteLinkActionedAriaLabel:"Le lien d’invitation a été copié.",defaultPlaceHolder:"Sélectionner une option",dismissSidePaneButtonLabel:"Fermer",videoEffectsPaneTitle:"Effets",videoEffectsPaneBackgroundSelectionTitle:"Arrière-plan",videoEffectsPaneAriaLabel:"Volet Effets vidéo",configurationPageCameraIsLoadingLabel:"Chargement de la vidéo...",configurationPageVideoEffectsButtonLabel:"Effets",unableToStartVideoEffect:"Impossible d’appliquer l’effet vidéo.",blurBackgroundEffectButtonLabel:"Flouter",blurBackgroundTooltip:"Flou d’arrière-plan",removeBackgroundEffectButtonLabel:"Aucun",removeBackgroundTooltip:"Supprimer l’arrière-plan",cameraOffBackgroundEffectWarningText:"Votre caméra est désactivée. Activez la caméra pour voir l’effet vidéo.",failedToJoinCallDueToNoNetworkMoreDetails:"L’appel a été déconnecté en raison d’un problème réseau. Vérifiez votre connexion et rejoignez-le à nouveau.",failedToJoinCallDueToNoNetworkTitle:"Appel déconnecté",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Vous n’avez pas reçu d’entrée dans l’appel. S’il s’agissait d’une erreur, rejoignez à nouveau l’appel.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Rejeté de la salle d’attente",learnMore:"En savoir plus",leavingCallTitle:"Départ en cours...",leftCallMoreDetails:"S’il s’agissait d’une erreur, rejoignez à nouveau l’appel.",leftCallTitle:"Vous avez quitté l’appel.",lobbyScreenConnectingToCallTitle:"Rejoindre l’appel",lobbyScreenWaitingToBeAdmittedTitle:"En attente d’admission",microphonePermissionDenied:"Votre navigateur bloque l’accès à votre micro",microphoneToggleInLobbyNotAllowed:"Impossible de désactiver ou d’activer le son dans la salle d’attente.",mutedMessage:"Le son de votre micro est désactivé",networkReconnectMoreDetails:"On dirait qu'une erreur est survenue. Nous tentons de vous reconnecter à l’appel.",networkReconnectTitle:"Tenez bon",notInvitedToRoomDetails:"Vous ne pouvez pas rejoindre cette salle, car vous n’avez pas d’invitation.",notInvitedToRoomTitle:"N’a pas été invité à la salle",peopleButtonLabel:"Personnes",selectedPeopleButtonLabel:"Bouton Contacts sélectionné",peoplePaneTitle:"Personnes",peoplePaneMoreButtonAriaLabel:"Plus",peopleButtonTooltipOpen:"Afficher les participants",peopleButtonTooltipClose:"Masquer les participants",peoplePaneSubTitle:"Dans cet appel {numberOfPeople}",privacyPolicy:"Politique de confidentialité",rejoinCallButtonLabel:"Rejoindre à nouveau l’appel.",removedFromCallMoreDetails:"Un autre participant vous a supprimé de l’appel.",removedFromCallTitle:"Vous avez été supprimé.",removeMenuLabel:"Supprimer",startSpotlightMenuLabel:"Mettre en évidence pour tout le monde",addSpotlightMenuLabel:"Ajouter la mise en évidence",stopSpotlightMenuLabel:"Arrêter la mise en évidence",stopSpotlightOnSelfMenuLabel:"Quitter la mise en évidence",spotlightLimitReachedMenuTitle:"Limite Spotlight atteinte",stopAllSpotlightMenuLabel:"Arrêter tous les projecteurs",returnToCallButtonAriaDescription:"Revenir à l’appel",returnToCallButtonAriaLabel:"Précédent",roomNotFoundDetails:"L’ID de la salle fourni est introuvable.",roomNotFoundTitle:"Salle introuvable",roomNotValidDetails:"Cette salle n’est pas valide pour l’instant.",roomNotValidTitle:"Salle non valide",inviteToRoomRemovedDetails:"Votre invitation à rejoindre cette salle a été supprimée.",inviteToRoomRemovedTitle:"Invitation à la salle supprimée",soundLabel:"Son",noMicrophonesLabel:"Aucun microphone trouvé",noSpeakersLabel:"Aucun haut-parleur trouvé",startCallButtonLabel:"Lancer l’appel",openDialpadButtonLabel:"Composer le numéro de téléphone",peoplePaneAddPeopleButtonLabel:"Ajouter des personnes",dialpadStartCallButtonLabel:"Appeler",dialpadModalTitle:"Composer le numéro de téléphone",dialpadModalAriaLabel:"Clavier",dialpadCloseModalButtonAriaLabel:"Fermer le clavier",moreButtonCallingLabel:"Plus",resumeCallButtonLabel:"Reprendre",resumingCallButtonLabel:"Reprise...",resumeCallButtonAriaLabel:"Reprendre l’appel",resumingCallButtonAriaLabel:"Reprendre l’appel",holdScreenLabel:"Vous avez été mis(e) en attente.",openDtmfDialpadLabel:"Afficher le pavé ",dtmfDialpadPlaceholderText:"Entrer un numéro",outboundCallingNoticeString:"Appel en cours...",participantJoinedNoticeString:"{displayName} a rejoint",twoParticipantJoinedNoticeString:"{displayName1} et {displayName2} ont rejoint",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} et {displayName3} ont rejoint",participantLeftNoticeString:"{displayName} a quitté",twoParticipantLeftNoticeString:"{displayName1} et {displayName2} sont partis",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} et {displayName3} ont quitté",unnamedParticipantString:"participant(e) sans nom",manyUnnamedParticipantsJoined:"participant sans nom et {numOfParticipants} autres participants ont rejoint",manyUnnamedParticipantsLeft:"participant sans nom et {numOfParticipants} autres participants ont quitté",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} et {numOfParticipants} autres participants ont rejoint",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} et {numOfParticipants} autres participants restants",liveCaptionsLabel:"Sous-titres en direct",captionsSettingsLabel:"Paramètres de sous-titre",startCaptionsButtonOnLabel:"Activer les sous-titres",startCaptionsButtonOffLabel:"Désactiver les sous-titres",startCaptionsButtonTooltipOnContent:"Désactiver les sous-titres",startCaptionsButtonTooltipOffContent:"Activer les sous-titres",captionsSettingsModalTitle:"Paramètres des sous-titres",captionsSettingsSpokenLanguageDropdownLabel:"Langue parlée",captionsSettingsCaptionLanguageDropdownLabel:"Langue des sous-titres",captionsSettingsSpokenLanguageDropdownInfoText:"Langue parlée par tous les participants à cet appel.",captionsSettingsCaptionLanguageDropdownInfoText:"Les sous-titres apparaîtront dans cette langue.",captionsSettingsConfirmButtonLabel:"Confirmer",captionsSettingsCancelButtonLabel:"Annuler",captionsSettingsModalAriaLabel:"Modalités de réglage des sous-titres",captionsSettingsCloseModalButtonAriaLabel:"Paramètres de fermeture des sous-titres",captionsBannerMoreButtonCallingLabel:"Autres options de paramètres de sous-titres",captionsBannerMoreButtonTooltip:"Autres options",realTimeTextLabel:"Texte en temps réel (RTT)",startRealTimeTextLabel:"Activer RTT pour cet appel",disabledStartRealTimeTextLabel:"La RTT est actuellement activée, elle ne peut pas être désactivée",dismissModalAriaLabel:"Images locales et distantes, appuyez sur entrée pour revenir à l'appel",callRejectedTitle:"Nous n’avons pas pu effectuer l’appel",callRejectedMoreDetails:"La personne que vous essayez de rejoindre n’est pas disponible. Veuillez réessayer plus tard.",callTimeoutTitle:"Nous n’avons pas pu effectuer l’appel",callTimeoutDetails:"La personne que vous essayez de rejoindre n’est pas disponible. Veuillez réessayer plus tard.",callTimeoutBotTitle:"Nous n’avons pas pu effectuer l’appel",callTimeoutBotDetails:"Le service auquel ou essayez d’accéder n’est pas disponible. Veuillez réessayer plus tard.",dtmfDialerButtonLabel:"Clavier",dtmfDialerButtonTooltipOn:"Afficher le pavé ",dtmfDialerButtonTooltipOff:"Masquez le clavier",dtmfDialerMoreButtonLabelOn:"Afficher le pavé ",dtmfDialerMoreButtonLabelOff:"Masquez le clavier",phoneCallMoreButtonLabel:"Appel téléphonique",spokenLanguageStrings:{"ar-ae":"Arabe (É.A.U.)","ar-sa":"Arabe (Arabie saoudite)","da-dk":"Danois","de-de":"Allemand (Allemagne)","en-au":"Anglais - Australie","en-ca":"Anglais (Canada)","en-gb":"Anglais (Royaume-Uni)","en-in":"Anglais (Inde)","en-nz":"Anglais (Nouvelle-Zélande)","en-us":"Anglais (États-Unis)","es-es":"Espagnol (Espagne, moderne)","es-mx":"Espagnol (Mexique)","fi-fi":"Finnois","fr-ca":"Français (Canada)","fr-fr":"Français (France)","hi-in":"Hindi","it-it":"Italien (Italie)","ja-jp":"Japonais","ko-kr":"Coréen","nb-no":"Norvégien (Bokmål)","nl-be":"Flamand (Belgique)","nl-nl":"Néerlandais (Pays-Bas)","pl-pl":"Polonais","pt-br":"Portugais (Brésil)","ru-ru":"Russe","sv-se":"Suédois","zh-cn":"Chinois (République populaire de Chine),","zh-hk":"Chinois (Hong Kong (R.A.S.))","cs-cz":"Tchèque","pt-pt":"Portugais (Portugal)","tr-tr":"Turc","vi-vn":"Vietnamien","th-th":"Thaï","he-il":"Hébreu","cy-gb":"Gallois","uk-ua":"Ukrainien","el-gr":"Grec","hu-hu":"Hongrois","ro-ro":"Roumain","sk-sk":"Slovaque","zh-tw":"Chinois – Taiwan"},captionLanguageStrings:{ar:"Arabe",da:"Danois",de:"Allemand",en:"Anglais",es:"Espagnol",fi:"Finnois","fr-ca":"Français (Canada)",fr:"Français (France)",hi:"Hindi",it:"Italien",ja:"Japonais",ko:"Coréen",nb:"Norvégien (Bokmål)",nl:"Néerlandais",pl:"Polonais",pt:"Portugais (Brésil)",ru:"Russe",sv:"Suédois","zh-Hans":"Chinois (simplifié)","zh-Hant":"Chinois (traditionnel)",cs:"Tchèque","pt-pt":"Portugais (Portugal)",tr:"Turc",vi:"Vietnamien",th:"Thaï",he:"Hébreu",cy:"Gallois",uk:"Ukrainien",el:"Grec",hu:"Hongrois",ro:"Roumain",sk:"Slovaque"},captionsBannerSpinnerText:"Démarrage des sous-titres...",realTimeTextInputBoxDefaultText:"Tapez votre message en temps réel.",realTimeTextInputErrorMessage:"Le message ne peut pas dépasser 2000 caractères",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (texte en temps réel) est activé pour tous les participants pendant toute la durée de la réunion.",realTimeTextBannerLinkLabel:"En savoir plus",realTimeTextModalAriaLabel:"Activer la boîte de dialogue modale RTT",realTimeTextModalTitle:"Activation de la RTT?",realTimeTextModalText:"Le texte en temps réel (RTT) sera activé pour tous les participants à la réunion.",realTimeTextConfirmButtonLabel:"Activer",realTimeTextCancelButtonLabel:"Annuler",realTimeTextCloseModalButtonAriaLabel:"Fermer la boîte de dialogue modale RTT",captionsOnlyContainerTitle:"Sous-titres en direct",realTimeTextOnlyContainerTitle:"Texte en temps réel",captionsAndRealTimeTextContainerTitle:"Sous-titres en direct et texte en temps réel",expandButtonAriaLabel:"Étendre",minimizeButtonAriaLabel:"Réduire",transferPageTransferorText:"Transfert en cours...",transferPageTransferTargetText:"Connexion en cours...",transferPageUnknownTransferorDisplayName:"Inconnu",transferPageUnknownTransferTargetDisplayName:"Inconnu",transferPageNoticeString:"Vous êtes en train d’être transféré",participantCouldNotBeReachedTitle:"Le participant cible n’est actuellement pas disponible",participantCouldNotBeReachedMoreDetails:"Veuillez communiquer avec le participant lorsqu’il sera disponible",permissionToReachTargetParticipantNotAllowedTitle:"L’autorisation d’atteindre le participant cible n’est pas valable",permissionToReachTargetParticipantNotAllowedMoreDetails:"Veuillez vérifier que le participant cible est dans le même locataire",unableToResolveTenantTitle:"Impossible de résoudre l’ID de locataire pour le participant cible",unableToResolveTenantMoreDetails:"Veuillez vérifier que l’ID du participant est entré correctement",participantIdIsMalformedTitle:"Le format de l’ID de participant n’est pas correct",participantIdIsMalformedMoreDetails:"Veuillez vérifier que l’identifiant du participant est dans le bon format",moreButtonGalleryControlLabel:"Afficher",moreButtonGalleryPositionToggleLabel:"Déplacer la galerie vers le haut",moreButtonGallerySpeakerLayoutLabel:"Haut-parleur",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamique",moreButtonGalleryDefaultLayoutLabel:"Vue Galerie",moreButtonGalleryFocusedContentLayoutLabel:"Focalisation sur le contenu",moreButtonLargeGalleryDefaultLayoutLabel:"Grande galerie",moreButtonTogetherModeLayoutLabel:"Mode Ensemble",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Votre caméra a été désactivée. Vous ne pouvez plus partager de vidéo.",grantedDueToMeetingOption:"Votre appareil photo a été activé. Activez-le si vous souhaitez partager une vidéo."},unmuteMic:{lostDueToMeetingOption:"Votre micro a été désactivé. Vous ne pouvez plus activer le son.",grantedDueToMeetingOption:"Votre microphone a été activé. Activez le microphone pour parler."},shareScreen:{lostDueToRoleChangeToAttendee:"Votre rôle a été modifié. Certaines actions, telles que le partage de contenu, ne seront pas disponibles pour vous.",grantedDueToRoleChangeToPresenter:"Vous êtes présentateur. Vous pouvez partager du contenu et organiser la réunion."},hideAttendeeNames:{hideAttendeeNameAttendee:"Votre nom est masqué pour les autres participants. Les organisateurs et les présentateurs peuvent voir votre vrai nom.",hideAttendeeNamePresenter:"Les noms des participants sont masqués. Afin de protéger la confidentialité des participants, seuls les organisateurs et les présentateurs peuvent visualiser les noms."}},surveyTitle:"Aidez-nous à nous améliorer",starSurveyHelperText:"Quelle a été la qualité globale de cet appel?",starSurveyOneStarText:"La qualité était mauvaise.",starSurveyTwoStarText:"La qualité était mauvaise.",starSurveyThreeStarText:"La qualité était bonne.",starSurveyFourStarText:"La qualité était excellente.",starSurveyFiveStarText:"La qualité était excellente.",surveyConfirmButtonLabel:"Envoyer des commentaires",starRatingAriaLabel:"Sélectionner {0} étoiles sur {1}",surveySkipButtonLabel:"Ignorer",tagsSurveyQuestion:"Que pouvons-nous améliorer?",tagsSurveyTextFieldDefaultText:"Autre, veuillez préciser.",tagsSurveyHelperText:"Cochez les problèmes que vous avez rencontrés.",endOfSurveyText:"Merci pour vos commentaires!",surveyIssues:{overallRating:{callCannotJoin:"Je n’ai pas pu rejoindre l’appel",callCannotInvite:"Je n’ai pas pu inviter d’autres personnes à participer à l’appel",hadToRejoin:"J’ai dû rejoindre l’appel",callEndedUnexpectedly:"L’appel s’est terminé pour moi de manière brusque",otherIssues:"J’avais d’autres problèmes avec l’appel"},audioRating:{noLocalAudio:"Mon interlocuteur n’entendait aucun son",noRemoteAudio:"Je n’entendais aucun son",echo:"J’ai entendu des échos lors de l’appel",audioNoise:"Il y avait des bruits parasites",lowVolume:"Faible volume sonore",audioStoppedUnexpectedly:"L’audio s’est arrêté de manière inattendue",distortedSpeech:"Le fichier audio a été altéré",audioInterruption:"L’audio a été interrompu",otherIssues:"J’avais d’autres problèmes audio dans cet appel"},videoRating:{noVideoReceived:"La vidéo ne s'affichait pas sur mon écran",noVideoSent:"D’autres ne pouvaient pas me voir",lowQuality:"La qualité de la vidéo était faible",freezes:"Vidéo figée",stoppedUnexpectedly:"La vidéo s'est arrêtée brusquement.",darkVideoReceived:"Je ne peux voir les écrans sombres que lorsque d’autres allument leur appareil photo",audioVideoOutOfSync:"L’audio et la vidéo n’étaient pas synchronisés",otherIssues:"J’ai rencontré d’autres problèmes vidéo pendant cet appel"},screenshareRating:{noContentLocal:"Les autres n’ont pas pu voir mon partage d’écran",noContentRemote:"Je ne pouvais pas voir le partage d’écran des autres",cannotPresent:"Je n’ai pas pu présenter mon écran",lowQuality:"La qualité du partage d’écran était faible",freezes:"Partage d’écran figé",stoppedUnexpectedly:"Le partage d’écran s’est arrêté de manière inattendue",largeDelay:"Le partage d’écran accuse un retard important",otherIssues:"Je rencontre d’autres problèmes de partage d’écran dans cet appel"}},surveyIssuesHeadingStrings:{overallRating:"Globale",audioRating:"Audio",videoRating:"Vidéo",screenshareRating:"En présentation"},spotlightPrompt:{startSpotlightHeading:"Voulez-vous mettre en évidence pour tout le monde?",startSpotlightText:"Votre vidéo ne sera plus mise en surbrillance pour tous les participants à la réunion.",startSpotlightOnSelfText:"Vous mettez en surbrillance votre vidéo pour tous les participants à la réunion.",startSpotlightConfirmButtonLabel:"Mettre en évidence pour tout le monde",startSpotlightCancelButtonLabel:"Annuler",stopSpotlightHeading:"Voulez-vous arrêter la mise en évidence de cette vidéo?",stopSpotlightOnSelfHeading:"Voulez-vous quitter la mise en évidence?",stopAllSpotlightHeading:"Voulez-vous arrêter de mettre en évidence toutes les vidéos?",stopSpotlightText:"Cette vidéo ne sera plus mise en évidence pour tous les participants à la réunion.",stopSpotlightOnSelfText:"Votre vidéo ne sera plus mise en surbrillance pour tous les participants à la réunion.",stopAllSpotlightText:"Les vidéo n’apparaissent plus en surbrillance pour tous les participants à la réunion.",stopSpotlightConfirmButtonLabel:"Arrêter la mise en évidence",stopSpotlightOnSelfConfirmButtonLabel:"Quitter la mise en évidence",stopSpotlightCancelButtonLabel:"Annuler",closeSpotlightPromptButtonLabel:"Fermer"},exitSpotlightButtonLabel:"Quitter la mise en évidence",exitSpotlightButtonTooltip:"Quitter la mise en évidence",leaveConfirmButtonLabel:"Quitter",endCallConfirmButtonLabel:"Raccrocher",hangUpCancelButtonLabel:"Annuler",leaveConfirmDialogTitle:"Quitter l’appel?",leaveConfirmDialogContent:"Voulez-vous vraiment quitter l’appel?",endCallConfirmDialogTitle:"Mettre fin à l'appel?",endCallConfirmDialogContent:"Vous mettrez fin à l’appel pour tout le monde.",invalidMeetingIdentifier:"ID de réunion ou code d’accès non valide",pinParticipantMenuLabel:"Épingler pour moi",pinParticipantLimitReachedMenuLabel:"Épingler (limite atteinte)",unpinParticipantMenuLabel:"Désépingler",pinParticipantMenuItemAriaLabel:"Épingler pour moi",unpinParticipantMenuItemAriaLabel:"Désépingler",muteAllMenuLabel:"Désactiver tous les micros",muteAllDialogTitle:"Couper le son pour tout le monde?",muteAllDialogContent:"Cette opération va désactiver le son de tous les autres participants à la réunion et pas le vôtre.",muteAllConfirmButtonLabel:"Désactiver tous les micros",muteAllCancelButtonLabel:"Annuler",joinBreakoutRoomButtonLabel:"Rejoindre la salle pour petit groupe",returnFromBreakoutRoomButtonLabel:"Quitter la salle",leaveBreakoutRoomAndMeetingButtonLabel:"Quitter la réunion",breakoutRoomJoinedNotificationTitle:"Vous avez rejoint la salle pour petit groupe « {roomName} »",joinBreakoutRoomBannerTitle:"La salle pour petit groupe « {roomName} » est ouverte",joinBreakoutRoomBannerButtonLabel:"Rejoindre",returnFromBreakoutRoomBannerTitle:"Revenir à la réunion principale",returnFromBreakoutRoomBannerButtonLabel:"Rejoindre",forbidAudioMenuLabel:"Désactiver le micro",permitAudioMenuLabel:"Autoriser le micro",forbidOthersAudioDialogTitle:"Voulez-vous désactiver le micro des participants?",forbidOthersAudioDialogContent:"Les participants ne pourront pas activer leur micro. L’organisateur(-trice) et les présentateurs peuvent permettre aux utilisateurs de désactiver le son selon les besoins.",forbidOthersAudioConfirmButtonLabel:"Désactiver les micros",forbidOthersAudioCancelButtonLabel:"Annuler",permitOthersAudioDialogTitle:"Voulez-vous autoriser le micro pour des participants?",permitOthersAudioDialogContent:"Chaque participant(e) à la réunion sera en mesure d’activer le son.",permitOthersAudioConfirmButtonLabel:"Autoriser les micros",permitOthersAudioCancelButtonLabel:"Annuler",forbidOthersAudioMenuLabel:"Désactiver le micro des participants",permitOthersAudioMenuLabel:"Autoriser le micro des participants",forbidVideoMenuLabel:"Désactiver la caméra",permitVideoMenuLabel:"Autoriser la caméra",forbidOthersVideoDialogTitle:"Voulez-vous désactiver la caméra des participants?",forbidOthersVideoDialogContent:"Les participants ne pourront pas activer la vidéo eux-mêmes. L’organisateur(-trice) et les présentateurs peuvent autoriser les utilisateurs à activer la caméra selon les besoins.",forbidOthersVideoConfirmButtonLabel:"Désactiver les caméras",forbidOthersVideoCancelButtonLabel:"Annuler",permitOthersVideoDialogTitle:"Voulez-vous autoriser la caméra des participants?",permitOthersVideoDialogContent:"Tous les participants à la réunion pourront activer la vidéo eux-mêmes.",permitOthersVideoConfirmButtonLabel:"Autoriser les caméras",permitOthersVideoCancelButtonLabel:"Annuler",forbidOthersVideoMenuLabel:"Désactiver la caméra des participants",permitOthersVideoMenuLabel:"Autoriser la caméra des participants"};var chat$d={chatListHeader:"Dans ce clavardage",uploadAttachment:"Charger une pièce jointe",uploadImageDataNotProvided:"Aucune donnée d’image n’est fournie.",uploadImageIsTooLarge:"L’image est trop grande. Choisissez une taille inférieure à {maxImageSize} Mo.",uploadImageExtensionIsNotAllowed:"Il est interdit de téléverser l’image {imageExtension}.",uploadImageFailed:"Impossible de téléverser l’image. Veuillez réessayer plus tard."};var callWithChat$d={chatButtonLabel:"Clavardage",chatButtonNewMessageNotificationLabel:"Nouveau message",chatButtonTooltipClosedWithMessageCount:"Afficher le clavardage ({unreadMessagesCount} non lu)",chatButtonTooltipClose:"Masquer le clavardage",chatButtonTooltipOpen:"Afficher le clavardage",chatPaneTitle:"Clavardage",copyInviteLinkButtonLabel:"Copier le lien d’invitation",copyInviteLinkButtonActionedLabel:"Lien copié",copyInviteLinkActionedAriaLabel:"Le lien d’invitation a été copié.",dismissSidePaneButtonLabel:"Fermer",moreDrawerAudioDeviceMenuTitle:"Périphérique audio",moreDrawerButtonLabel:"Autres options",moreDrawerButtonTooltip:"Autres options",moreDrawerMicrophoneMenuTitle:"Microphone",moreDrawerSpeakerMenuTitle:"Haut-parleur",moreDrawerCaptionsMenuTitle:"Sous-titres en direct",moreDrawerSpokenLanguageMenuTitle:"Langue parlée",moreDrawerCaptionLanguageMenuTitle:"Langue des sous-titres",moreDrawerGalleryOptionsMenuTitle:"Options de galerie",peopleButtonLabel:"Personnes",selectedPeopleButtonLabel:"Bouton Contacts sélectionné",peopleButtonTooltipOpen:"Afficher les participants",peopleButtonTooltipClose:"Masquer les participants",peopleButtonTooltipOpenAriaLabel:"Bouton Contacts, Afficher les participants",peopleButtonTooltipCloseAriaLabel:"Bouton Contacts, Masquer les participants",peoplePaneSubTitle:"Pendant cet appel",peoplePaneTitle:"Personnes",pictureInPictureTileAriaLabel:"Flux vidéo. Cliquez pour revenir à l’écran d’appel.",removeMenuLabel:"Supprimer",openDialpadButtonLabel:"Composer le numéro de téléphone",returnToCallButtonAriaDescription:"Revenir à l’appel",returnToCallButtonAriaLabel:"Précédent",peoplePaneAddPeopleButtonLabel:"Ajouter des personnes",dialpadStartCallButtonLabel:"Appeler",dialpadModalTitle:"Composer le numéro de téléphone",dialpadModalAriaLabel:"Clavier",dialpadCloseModalButtonAriaLabel:"Fermer le clavier",openDtmfDialpadLabel:"Afficher le pavé ",dtmfDialpadPlaceholderText:"Entrer un numéro",breakoutRoomChatPaneTitle:"Conversation dans une salle pour petit groupe",chatContentSpinnerLabel:"Chargement de la session de messagerie instantanée"};var fr_CA = {call:call$d,chat:chat$d,callWithChat:callWithChat$d};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for French (Canada)
 *
 * @public
 */
const COMPOSITE_LOCALE_FR_CA = {
    component: COMPONENT_LOCALE_FR_CA,
    strings: createCompositeStrings(fr_CA)
};

var call$c={cameraLabel:"מצלמה",noCamerasLabel:"לא נמצאו מצלמות",cameraPermissionDenied:"הדפדפן שלך חוסם את הגישה למצלמה שלך",cameraTurnedOff:"המצלמה שלך כבויה",chatButtonLabel:"צ'אט",close:"סגור",complianceBannerNowOnlyRecording:"אתה כעת רק מקליט את הפגישה.",complianceBannerNowOnlyTranscription:"אתה רק מתעתק את הפגישה כעת.",complianceBannerRecordingAndTranscriptionSaved:"ההקלטה והתמלול בתהליך שמירה.",complianceBannerRecordingAndTranscriptionStarted:"הקלטה ותמלול התחילו.",complianceBannerRecordingAndTranscriptionStopped:"ההקלטה והתמלול הופסקו.",complianceBannerRecordingSaving:"ההקלטה נשמרת.",complianceBannerRecordingStarted:"ההקלטה התחילה.",complianceBannerRecordingStopped:"ההקלטה הופסקה.",complianceBannerTranscriptionStarted:"התמלול התחיל.",complianceBannerTranscriptionConsent:"על-ידי הצטרפות, אתה נותן הסכמה לתעתק פגישה זו.",complianceBannerTranscriptionSaving:"מתבצע תהליך שמירה של התמלול.",complianceBannerTranscriptionStopped:"התמלול הופסק.",configurationPageTitle:"התחל שיחה",copyInviteLinkButtonLabel:"העתק קישור להזמנה",copyInviteLinkButtonActionedLabel:"הקישור הועתק",copyInviteLinkActionedAriaLabel:"קישור ההזמנה הועתק",defaultPlaceHolder:"בחר אפשרות",dismissSidePaneButtonLabel:"סגור",videoEffectsPaneTitle:"אפקטים",videoEffectsPaneBackgroundSelectionTitle:"רקע",videoEffectsPaneAriaLabel:"חלונית אפקטי וידאו",configurationPageCameraIsLoadingLabel:"הווידאו נטען...",configurationPageVideoEffectsButtonLabel:"אפקטים",unableToStartVideoEffect:"לא ניתן להחיל אפקט וידאו.",blurBackgroundEffectButtonLabel:"טשטוש",blurBackgroundTooltip:"טשטש רקע",removeBackgroundEffectButtonLabel:"ללא",removeBackgroundTooltip:"הסר רקע",cameraOffBackgroundEffectWarningText:"המצלמה שלך כבויה. הפעל מצלמה כדי לראות את אפקט הווידאו.",failedToJoinCallDueToNoNetworkMoreDetails:"השיחה נותקה עקב בעיית רשת. בדוק את החיבור שלך והצטרף שוב.",failedToJoinCallDueToNoNetworkTitle:"‏‏השיחה נותקה",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"לא הוענקה לך כניסה בשיחה. אם זו היתה טעות, הצטרף מחדש לשיחה.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"הוצא מחדר הכניסה",learnMore:"למידע נוסף",leavingCallTitle:"עוזב...",leftCallMoreDetails:"אם זו היתה טעות, הצטרף מחדש לשיחה.",leftCallTitle:"עזבת את הצ'אט",lobbyScreenConnectingToCallTitle:"מצטרף לשיחה",lobbyScreenWaitingToBeAdmittedTitle:"ממתין לאישור כניסה",microphonePermissionDenied:"הדפדפן שלך חוסם את הגישה למיקרופון שלך",microphoneToggleInLobbyNotAllowed:"אין אפשרות להשתיק או לבטל השתקה בחדר הכניסה.",mutedMessage:"אתה מושתק",networkReconnectMoreDetails:"נראה שמשהו השתבש. אנו מנסים להיכנס שוב לשיחה.",networkReconnectTitle:"המתן",notInvitedToRoomDetails:"אין לך אפשרות להצטרף לחדר זה משום שאין לך הזמנה.",notInvitedToRoomTitle:"לא הוזמן לחדר",peopleButtonLabel:"אנשים",selectedPeopleButtonLabel:"לחצן 'אנשים' נבחר",peoplePaneTitle:"אנשים",peoplePaneMoreButtonAriaLabel:"עוד",peopleButtonTooltipOpen:"הצג משתתפים",peopleButtonTooltipClose:"הסתר משתתפים",peoplePaneSubTitle:"בשיחה זו {numberOfPeople}",privacyPolicy:"מדיניות פרטיות",rejoinCallButtonLabel:"הצטרף לשיחה מחדש",removedFromCallMoreDetails:"משתתף אחר הסיר אותך מהשיחה.",removedFromCallTitle:"הוסרת",removeMenuLabel:"הסר",startSpotlightMenuLabel:"הצבת זרקור עבור כולם",addSpotlightMenuLabel:"הוסף זרקור",stopSpotlightMenuLabel:"הפסק להציב זרקור",stopSpotlightOnSelfMenuLabel:"צא מזרקור",spotlightLimitReachedMenuTitle:"הגעת למגבלת הזרקורים",stopAllSpotlightMenuLabel:"הפסק את כל הצבות הזרקור",returnToCallButtonAriaDescription:"חזור לשיחה",returnToCallButtonAriaLabel:"הקודם",roomNotFoundDetails:"מזהה החדר שסופק לא נמצא.",roomNotFoundTitle:"החדר לא נמצא",roomNotValidDetails:"חדר זה אינו חוקי כעת.",roomNotValidTitle:"החדר אינו חוקי",inviteToRoomRemovedDetails:"ההזמנה שלך להצטרפות לחדר זה הוסרה.",inviteToRoomRemovedTitle:"ההזמנה לחדר הוסרה",soundLabel:"צליל",noMicrophonesLabel:"לא נמצאו מיקרופונים",noSpeakersLabel:"לא נמצאו רמקולים",startCallButtonLabel:"התחל שיחה",openDialpadButtonLabel:"חייג למספר הטלפון",peoplePaneAddPeopleButtonLabel:"הוסף אנשים",dialpadStartCallButtonLabel:"שיחה",dialpadModalTitle:"חייג למספר הטלפון",dialpadModalAriaLabel:"לוח חיוג",dialpadCloseModalButtonAriaLabel:"סגור לוח חיוג",moreButtonCallingLabel:"עוד",resumeCallButtonLabel:"המשך",resumingCallButtonLabel:"מחדש...",resumeCallButtonAriaLabel:"חדש שיחה",resumingCallButtonAriaLabel:"חדש שיחה",holdScreenLabel:"אתה בהמתנה",openDtmfDialpadLabel:"הצג לוח חיוג",dtmfDialpadPlaceholderText:"הזן מספר",outboundCallingNoticeString:"מתקשר...",participantJoinedNoticeString:"{displayName} הצטרף/ה",twoParticipantJoinedNoticeString:"{displayName1} ו{displayName2} הצטרפו",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} ו{displayName3} עזבו",participantLeftNoticeString:"{displayName} עזב/ה",twoParticipantLeftNoticeString:"{displayName1} ו{displayName2} עזבו",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} ו{displayName3} עזבו",unnamedParticipantString:"משתתף/ת ללא שם",manyUnnamedParticipantsJoined:"משתתף/ת ללא שם ו{numOfParticipants} משתתפים אחרים הצטרפו",manyUnnamedParticipantsLeft:"משתתף/ת ללא שם ו{numOfParticipants} משתתפים אחרים",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} ו{numOfParticipants} משתתפים אחרים הצטרפו",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} ו{numOfParticipants} משתתפים אחרים נותרו",liveCaptionsLabel:"כתוביות חיות",captionsSettingsLabel:"הגדרות כתוביות",startCaptionsButtonOnLabel:"הפעל כתוביות",startCaptionsButtonOffLabel:"כבה כתוביות",startCaptionsButtonTooltipOnContent:"כבה כתוביות",startCaptionsButtonTooltipOffContent:"הפעל כתוביות",captionsSettingsModalTitle:"הגדרות כתוביות",captionsSettingsSpokenLanguageDropdownLabel:"שפה מדוברת",captionsSettingsCaptionLanguageDropdownLabel:"שפת כתוביות",captionsSettingsSpokenLanguageDropdownInfoText:"השפה שכולם בשיחה זו מדברים.",captionsSettingsCaptionLanguageDropdownInfoText:"הכתוביות יופיעו בשפה זו.",captionsSettingsConfirmButtonLabel:"אשר",captionsSettingsCancelButtonLabel:"ביטול",captionsSettingsModalAriaLabel:"מודאל הגדרת כתוביות",captionsSettingsCloseModalButtonAriaLabel:"סגור הגדרת כתוביות",captionsBannerMoreButtonCallingLabel:"הגדרות כתוביות אפשרויות נוספות",captionsBannerMoreButtonTooltip:"אפשרויות נוספות",realTimeTextLabel:"טקסט בזמן אמת (RTT)",startRealTimeTextLabel:"הפעל RTT עבור שיחה זו",disabledStartRealTimeTextLabel:"RTT מופעל כעת, לא ניתן לבטל אותו",dismissModalAriaLabel:"תמונה מקומית ותמונה מרוחקת, הקש על Enter כדי לחזור לשיחה.",callRejectedTitle:"לא הייתה אפשרות להשלים את השיחה.",callRejectedMoreDetails:"האדם שאליו אתה מנסה להגיע אינו זמין. נסה שוב מאוחר יותר.",callTimeoutTitle:"לא הייתה אפשרות להשלים את השיחה.",callTimeoutDetails:"האדם שאליו אתה מנסה להגיע אינו זמין. נסה שוב מאוחר יותר.",callTimeoutBotTitle:"לא הייתה אפשרות להשלים את השיחה.",callTimeoutBotDetails:"השירות אליו אתה מנסה להגיע אינו זמין. נסה שוב מאוחר יותר.",dtmfDialerButtonLabel:"לוח חיוג",dtmfDialerButtonTooltipOn:"הצג לוח חיוג",dtmfDialerButtonTooltipOff:"הסתר לוח חיוג",dtmfDialerMoreButtonLabelOn:"הצג לוח חיוג",dtmfDialerMoreButtonLabelOff:"הסתר לוח חיוג",phoneCallMoreButtonLabel:"שיחת טלפון",spokenLanguageStrings:{"ar-ae":"ערבית - איחוד האמירויות","ar-sa":"ערבית - ערב הסעודית","da-dk":"דנית","de-de":"גרמנית - גרמניה","en-au":"אנגלית - אוסטרליה","en-ca":"אנגלית - קנדה","en-gb":"אנגלית - בריטניה","en-in":"אנגלית - הודו","en-nz":"אנגלית - ניו זילנד","en-us":"אנגלית - ארצות הברית","es-es":"ספרדית - ספרד (ספרדית מודרנית)","es-mx":"ספרדית - מקסיקו","fi-fi":"פינית","fr-ca":"צרפתית - קנדה","fr-fr":"צרפתית - צרפת","hi-in":"הינדי","it-it":"איטלקית - איטליה","ja-jp":"יפנית","ko-kr":"קוריאנית","nb-no":"נורווגית (בוּקמוֹל)","nl-be":"הולנדית - בלגיה","nl-nl":"הולנדית - הולנד","pl-pl":"פולנית","pt-br":"פורטוגזית - ברזיל","ru-ru":"רוסית","sv-se":"שוודית","zh-cn":"סינית - הרפובליקה העממית של סין","zh-hk":"סינית - הונג קונג","cs-cz":"צ׳כית","pt-pt":"פורטוגזית - פורטוגל","tr-tr":"טורקית","vi-vn":"וייטנאמית","th-th":"תאית","he-il":"עברית","cy-gb":"וולשית","uk-ua":"אוקראינית","el-gr":"יוונית","hu-hu":"הונגרית","ro-ro":"רומנית","sk-sk":"סלובקית","zh-tw":"סינית - טייוואן"},captionLanguageStrings:{ar:"ערבית",da:"דנית",de:"גרמנית",en:"אנגלית",es:"ספרדית",fi:"פינית","fr-ca":"צרפתית - קנדה",fr:"צרפתית - צרפת",hi:"הינדי",it:"איטלקית",ja:"יפנית",ko:"קוריאנית",nb:"נורווגית (בוּקמוֹל)",nl:"הולנדית",pl:"פולנית",pt:"פורטוגזית - ברזיל",ru:"רוסית",sv:"שוודית","zh-Hans":"סינית (פשוטה)","zh-Hant":"סינית (מסורתית)",cs:"צ׳כית","pt-pt":"פורטוגזית - פורטוגל",tr:"טורקית",vi:"וייטנאמית",th:"תאית",he:"עברית",cy:"וולשית",uk:"אוקראינית",el:"יוונית",hu:"הונגרית",ro:"רומנית",sk:"סלובקית"},captionsBannerSpinnerText:"מאתחל כתוביות...",realTimeTextInputBoxDefaultText:"הקלד הודעה בזמן אמת",realTimeTextInputErrorMessage:"ההודעה אינה יכולה לחרוג מ- 2,000 תווים",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (טקסט בזמן אמת) זמין עבור כל המשתתפים למשך הזמן כולו של הפגישה.",realTimeTextBannerLinkLabel:"מידע נוסף",realTimeTextModalAriaLabel:"הפעל תיבת דו-שיח מודאלית של RTT",realTimeTextModalTitle:"להפעיל RTT?",realTimeTextModalText:"טקסט בזמן אמת (RTT) יופעל עבור כל האנשים בפגישה.",realTimeTextConfirmButtonLabel:"הפעל",realTimeTextCancelButtonLabel:"ביטול",realTimeTextCloseModalButtonAriaLabel:"סגור תיבת דו-שיח מודאלית של RTT",captionsOnlyContainerTitle:"כתוביות",realTimeTextOnlyContainerTitle:"טקסט בזמן אמת",captionsAndRealTimeTextContainerTitle:"כתוביות וטקסט בזמן אמת",expandButtonAriaLabel:"הרחב",minimizeButtonAriaLabel:"כווץ",transferPageTransferorText:"מעביר...",transferPageTransferTargetText:"מתחבר...",transferPageUnknownTransferorDisplayName:"לא ידוע",transferPageUnknownTransferTargetDisplayName:"לא ידוע",transferPageNoticeString:"אתה מועבר כעת",participantCouldNotBeReachedTitle:"ממשתתף היעד אינו זמין כעת",participantCouldNotBeReachedMoreDetails:"פנה למשתתף כאשר הוא זמין",permissionToReachTargetParticipantNotAllowedTitle:"אין הרשאה להגיע למשתתף היעד",permissionToReachTargetParticipantNotAllowedMoreDetails:"בדוק שמשתתף היעד נמצא באותו דייר",unableToResolveTenantTitle:"לא ניתן לזהות מזהה דייר עבור משתתף היעד",unableToResolveTenantMoreDetails:"בדוק שמזהה המשתתף הוזן כראוי",participantIdIsMalformedTitle:"מזהה המשתתף אינו בתבנית הנכונה",participantIdIsMalformedMoreDetails:"בדוק שמזהה המשתתף מופיע בתבנית הנכונה",moreButtonGalleryControlLabel:"הצג",moreButtonGalleryPositionToggleLabel:"העבר את הגלריה לחלק העליון",moreButtonGallerySpeakerLayoutLabel:"רמקול",moreButtonGalleryFloatingLocalLayoutLabel:"דינמי",moreButtonGalleryDefaultLayoutLabel:"תצוגת גלריה",moreButtonGalleryFocusedContentLayoutLabel:"התמקדות בתוכן",moreButtonLargeGalleryDefaultLayoutLabel:"גלריה גדולה",moreButtonTogetherModeLayoutLabel:"מצב ביחד",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"המצלמה שלך הפכה ללא זמינה. כבר אינך יכול לשתף וידאו.",grantedDueToMeetingOption:"המצלמה שלך הפכה לזמינה. הפעל אותה אם ברצונך לשתף וידאו."},unmuteMic:{lostDueToMeetingOption:"המיקרופון שלך הפך ללא זמין. לא ניתן עוד לבטל השתקה.",grantedDueToMeetingOption:"המיקרופון שלך הפך לזמין. בטל את ההשתקה אם ברצונך לדבר."},shareScreen:{lostDueToRoleChangeToAttendee:"התפקיד שלך השתנה. פעולות מסוימות, כגון שיתוף תוכן, לא יהיו זמינות עבורך.",grantedDueToRoleChangeToPresenter:"אתה מציג. באפשרותך לשתף תוכן ולסייע בפגישה."},hideAttendeeNames:{hideAttendeeNameAttendee:"שמך מוסתר ממשתתפים אחרים. מארגנים ומציגים יכולים לראות את שמך האמיתי.",hideAttendeeNamePresenter:"שמות המשתתפים מוסתרים. רק מארגנים ומציגים יכולים לראות את השמות, כדי להגן על פרטיות המשתתפים."}},surveyTitle:"עזור לנו להשתפר",starSurveyHelperText:"מה היתה האיכות של שיחה זו?",starSurveyOneStarText:"האיכות הייתה גרועה.",starSurveyTwoStarText:"האיכות הייתה ירודה.",starSurveyThreeStarText:"האיכות הייתה טובה.",starSurveyFourStarText:"האיכות הייתה מעולה.",starSurveyFiveStarText:"האיכות הייתה מצוינת.",surveyConfirmButtonLabel:"שלח משוב",starRatingAriaLabel:"בחר {0} מתוך {1} כוכבים",surveySkipButtonLabel:"דלג",tagsSurveyQuestion:"מה יכול היה להיות טוב יותר?",tagsSurveyTextFieldDefaultText:"אחר, נא לציין",tagsSurveyHelperText:"בדוק את כל הבעיות שבהן נתקלת",endOfSurveyText:"תודה על המשוב!",surveyIssues:{overallRating:{callCannotJoin:"לא הצלחתי להצטרף לשיחה",callCannotInvite:"לא הצלחתי להזמין אחרים לשיחה",hadToRejoin:"הייתי צריך להצטרף מחדש לשיחה",callEndedUnexpectedly:"השיחה הסתיימה עבורי באופן בלתי צפוי",otherIssues:"נתקלתי בבעיות אחרות בשיחה"},audioRating:{noLocalAudio:"הצד השני בשיחה לא הצליח לשמוע שום צליל",noRemoteAudio:"לא יכולתי לשמוע אף צליל",echo:"שמעתי קולות הד בשיחה",audioNoise:"שמעתי רעש בשיחה",lowVolume:"עוצמת הקול הייתה נמוכה",audioStoppedUnexpectedly:"השמע הופסק באופן בלתי צפוי",distortedSpeech:"השמע היה מעוות",audioInterruption:"השמע הופסק",otherIssues:"נתקלתי בבעיות שמע אחרות בשיחה זו"},videoRating:{noVideoReceived:"לא ראיתי את הצד השני",noVideoSent:"אנשים אחרים לא הצליחו לראות אותי",lowQuality:"איכות הווידאו הייתה נמוכה",freezes:"הווידאו קפא",stoppedUnexpectedly:"הווידאו נעצר באופן בלתי צפוי",darkVideoReceived:"אני יכול לראות רק מסכים כהים כאשר אחרים מפעילים את המצלמה שלהם",audioVideoOutOfSync:"השמע והווידאו לא היו מסונכרנים",otherIssues:"נתקלתי בבעיות וידאו אחרות בשיחה זו"},screenshareRating:{noContentLocal:"אנשים אחרים לא הצליחו לראות את שיתוף המסך שלי",noContentRemote:"לא הצלחתי לראות את שיתוף המסך של אנשים אחרים",cannotPresent:"לא הצלחתי להציג את המסך שלי",lowQuality:"איכות שיתוף המסך הייתה נמוכה",freezes:"שיתוף מסך הוקפא",stoppedUnexpectedly:"שיתוף המסך הופסק באופן בלתי צפוי",largeDelay:"לשיתוף המסך יש עיכוב גדול",otherIssues:"נתקלתי בבעיות אחרות בשיתוף מסכים בשיחה זו"}},surveyIssuesHeadingStrings:{overallRating:"סך הכל",audioRating:"שם ציוד השמע",videoRating:"סרטון",screenshareRating:"מציג"},spotlightPrompt:{startSpotlightHeading:"לסמן בזרקור עבור כולם?",startSpotlightText:"תסמן סרטון וידאו זה עבור כל האנשים בפגישה.",startSpotlightOnSelfText:"תסמן את סרטון הווידאו שלך עבור כל האנשים בפגישה.",startSpotlightConfirmButtonLabel:"הצבת זרקור עבור כולם",startSpotlightCancelButtonLabel:"ביטול",stopSpotlightHeading:"להפסיק להאיר בזרקור וידאו זה?",stopSpotlightOnSelfHeading:"לצאת מזרקור?",stopAllSpotlightHeading:"להפסיק להציב זרקור על כל סרטוני הווידאו?",stopSpotlightText:"וידאו זה לא יסומן עוד עבור כל האנשים בפגישה.",stopSpotlightOnSelfText:"סרטון הווידאו שלך לא יסומן עוד עבור כל האנשים בפגישה.",stopAllSpotlightText:"סרטוני הווידאו לא יסומנו עוד עבור כל האנשים בפגישה.",stopSpotlightConfirmButtonLabel:"הפסק להציב זרקור",stopSpotlightOnSelfConfirmButtonLabel:"צא מזרקור",stopSpotlightCancelButtonLabel:"ביטול",closeSpotlightPromptButtonLabel:"סגור"},exitSpotlightButtonLabel:"צא מהזרקור",exitSpotlightButtonTooltip:"צא מהזרקור",leaveConfirmButtonLabel:"צא",endCallConfirmButtonLabel:"סיים שיחה",hangUpCancelButtonLabel:"ביטול",leaveConfirmDialogTitle:"לצאת מהשיחה?",leaveConfirmDialogContent:"האם אתה בטוח שברצונך לעזוב את השיחה?",endCallConfirmDialogTitle:"לסיים את השיחה?",endCallConfirmDialogContent:"פעולה זאת תסיים את השיחה עבור כל המשתתפים.",invalidMeetingIdentifier:"מזהה פגישה או קוד סיסמה לא חוקיים",pinParticipantMenuLabel:"הצמד עבורי",pinParticipantLimitReachedMenuLabel:"הצמדה (הגעת למגבלה)",unpinParticipantMenuLabel:"בטל הצמדה",pinParticipantMenuItemAriaLabel:"הצמד עבורי",unpinParticipantMenuItemAriaLabel:"בטל הצמדה",muteAllMenuLabel:"השתק הכל",muteAllDialogTitle:"האם להשתיק את כולם?",muteAllDialogContent:"פעולה זו תשתיק את כל האנשים בפגישה חוץ ממך.",muteAllConfirmButtonLabel:"השתק הכל",muteAllCancelButtonLabel:"ביטול",joinBreakoutRoomButtonLabel:"הצטרף לחדר בריחה",returnFromBreakoutRoomButtonLabel:"עזוב את החדר",leaveBreakoutRoomAndMeetingButtonLabel:"עזוב פגישה",breakoutRoomJoinedNotificationTitle:"הצטרפת לפיצול חדרים '{roomName}'",joinBreakoutRoomBannerTitle:"פיצול החדרים '{roomName}' פתוח",joinBreakoutRoomBannerButtonLabel:"הצטרף",returnFromBreakoutRoomBannerTitle:"חזרה לפגישה הראשית",returnFromBreakoutRoomBannerButtonLabel:"הצטרף מחדש",forbidAudioMenuLabel:"הפוך את המיקרופון ללא זמין",permitAudioMenuLabel:"אפשר שימוש במיקרופון",forbidOthersAudioDialogTitle:"האם להפוך מיקרופון ללא זמין למשתתפים?",forbidOthersAudioDialogContent:"משתתפים לא יוכלו לבטל את ההשתקה שלהם. המארגן והמציגים יכולים לאפשר לאנשים לבטל השתקה לפי הצורך.",forbidOthersAudioConfirmButtonLabel:"הפוך את המיקרופונים ללא זמינים",forbidOthersAudioCancelButtonLabel:"ביטול",permitOthersAudioDialogTitle:"האם לאפשר שימוש במיקרופון עבור המשתתפים?",permitOthersAudioDialogContent:"כל האנשים בפגישה יוכלו לבטל את ההשתקה של עצמם.",permitOthersAudioConfirmButtonLabel:"אפשר שימוש במיקרופונים",permitOthersAudioCancelButtonLabel:"ביטול",forbidOthersAudioMenuLabel:"הפוך את המיקרופון ללא זמין עבור המשתתפים",permitOthersAudioMenuLabel:"אפשר למשתתפים להשתמש במיקרופון",forbidVideoMenuLabel:"הפוך את המצלמה ללא זמינה",permitVideoMenuLabel:"אפשר שימוש במצלמה",forbidOthersVideoDialogTitle:"האם להפוך את המצלמה ללא זמינה עבור המשתתפים?",forbidOthersVideoDialogContent:"משתתפים לא יוכלו להפעיל וידאו בעצמם. המארגן והמציגים יכולים לאפשר לאנשים להפעיל את המצלמה לפי הצורך.",forbidOthersVideoConfirmButtonLabel:"הפוך את המצלמות ללא זמינות",forbidOthersVideoCancelButtonLabel:"ביטול",permitOthersVideoDialogTitle:"האם לאפשר שימוש במצלמה עבור המשתתפים?",permitOthersVideoDialogContent:"כל המשתתפים בפגישה יוכלו להפעיל וידאו בעצמם.",permitOthersVideoConfirmButtonLabel:"אפשר שימוש במצלמות",permitOthersVideoCancelButtonLabel:"ביטול",forbidOthersVideoMenuLabel:"הפוך את המצלמה ללא זמינה עבור משתתפים",permitOthersVideoMenuLabel:"אפשר למשתתפים גישה למצלמה"};var chat$c={chatListHeader:"בצ'אט זה",uploadAttachment:"העלה קובץ מצורף",uploadImageDataNotProvided:"נתוני התמונה לא סופקו.",uploadImageIsTooLarge:"התמונה גדולה מדי. בחר אחד קטן יותר מ{maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"העלאת תמונה {imageExtension} אסורה.",uploadImageFailed:"לא ניתן להעלות את התמונה. נסה שוב מאוחר יותר."};var callWithChat$c={chatButtonLabel:"צ'אט",chatButtonNewMessageNotificationLabel:"הודעה חדשה",chatButtonTooltipClosedWithMessageCount:"הצג צ'אט ({unreadMessagesCount} שלא נקראו)",chatButtonTooltipClose:"הסתר צ׳אט",chatButtonTooltipOpen:"הצג צ'אט",chatPaneTitle:"צ'אט",copyInviteLinkButtonLabel:"העתק קישור להזמנה",copyInviteLinkButtonActionedLabel:"הקישור הועתק",copyInviteLinkActionedAriaLabel:"קישור ההזמנה הועתק",dismissSidePaneButtonLabel:"סגור",moreDrawerAudioDeviceMenuTitle:"התקן שמע",moreDrawerButtonLabel:"אפשרויות נוספות",moreDrawerButtonTooltip:"אפשרויות נוספות",moreDrawerMicrophoneMenuTitle:"מיקרופון",moreDrawerSpeakerMenuTitle:"רמקול",moreDrawerCaptionsMenuTitle:"כתוביות חיות",moreDrawerSpokenLanguageMenuTitle:"שפה מדוברת",moreDrawerCaptionLanguageMenuTitle:"שפת כתוביות",moreDrawerGalleryOptionsMenuTitle:"אפשרויות גלריה",peopleButtonLabel:"אנשים",selectedPeopleButtonLabel:"לחצן 'אנשים' נבחר",peopleButtonTooltipOpen:"הצג משתתפים",peopleButtonTooltipClose:"הסתר משתתפים",peopleButtonTooltipOpenAriaLabel:"לחצן אנשים, הצג משתתפים",peopleButtonTooltipCloseAriaLabel:"לחצן אנשים, הסתר משתתפים",peoplePaneSubTitle:"בשיחה זו",peoplePaneTitle:"אנשים",pictureInPictureTileAriaLabel:"הזנות וידאו. לחץ כדי לחזור למסך השיחה.",removeMenuLabel:"הסר",openDialpadButtonLabel:"חייג למספר הטלפון",returnToCallButtonAriaDescription:"חזור לשיחה",returnToCallButtonAriaLabel:"הקודם",peoplePaneAddPeopleButtonLabel:"הוסף אנשים",dialpadStartCallButtonLabel:"שיחה",dialpadModalTitle:"חייג למספר הטלפון",dialpadModalAriaLabel:"לוח חיוג",dialpadCloseModalButtonAriaLabel:"סגור לוח חיוג",openDtmfDialpadLabel:"הצג לוח חיוג",dtmfDialpadPlaceholderText:"הזן מספר",breakoutRoomChatPaneTitle:"צ'אט פיצול חדרים",chatContentSpinnerLabel:"טוען את הצ'אט"};var he_IL = {call:call$c,chat:chat$c,callWithChat:callWithChat$c};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Hebrew (Israel)
 *
 * @public
 */
const COMPOSITE_LOCALE_HE_IL = {
    component: COMPONENT_LOCALE_HE_IL,
    strings: createCompositeStrings(he_IL)
};

var call$b={cameraLabel:"Fotocamera",noCamerasLabel:"Nessuna fotocamera trovata",cameraPermissionDenied:"Il browser blocca l'accesso alla fotocamera",cameraTurnedOff:"La videocamera è disabilitata",chatButtonLabel:"Chat",close:"Chiudi",complianceBannerNowOnlyRecording:"È in corso la registrazione solo di questa riunione.",complianceBannerNowOnlyTranscription:"È in corso la trascrizione solo di questa riunione.",complianceBannerRecordingAndTranscriptionSaved:"È in corso il salvataggio della registrazione e della trascrizione.",complianceBannerRecordingAndTranscriptionStarted:"Registrazione e trascrizione avviate.",complianceBannerRecordingAndTranscriptionStopped:"Registrazione e trascrizione interrotte.",complianceBannerRecordingSaving:"Salvataggio della registrazione in corso.",complianceBannerRecordingStarted:"Registrazione avviata.",complianceBannerRecordingStopped:"Registrazione arrestata.",complianceBannerTranscriptionStarted:"Trascrizione avviata.",complianceBannerTranscriptionConsent:"Partecipando, fornisci il consenso per la trascrizione di questa riunione.",complianceBannerTranscriptionSaving:"È in corso il salvataggio della trascrizione.",complianceBannerTranscriptionStopped:"Trascrizione interrotta.",configurationPageTitle:"Avvia una chiamata",copyInviteLinkButtonLabel:"Copia il collegamento dell’invito",copyInviteLinkButtonActionedLabel:"Collegamento copiato",copyInviteLinkActionedAriaLabel:"Collegamento di invito copiato",defaultPlaceHolder:"Seleziona un'opzione",dismissSidePaneButtonLabel:"Chiudi",videoEffectsPaneTitle:"Effetti",videoEffectsPaneBackgroundSelectionTitle:"In background",videoEffectsPaneAriaLabel:"Riquadro Effetti video",configurationPageCameraIsLoadingLabel:"Il video è in corso di caricamento ...",configurationPageVideoEffectsButtonLabel:"Effetti",unableToStartVideoEffect:"Non è possibile applicare l'effetto video.",blurBackgroundEffectButtonLabel:"Sfocatura",blurBackgroundTooltip:"Sfoca sfondo",removeBackgroundEffectButtonLabel:"Nessuno",removeBackgroundTooltip:"Rimuovi sfondo",cameraOffBackgroundEffectWarningText:"La fotocamera è disattivata. Attiva la fotocamera per vedere l'effetto video.",failedToJoinCallDueToNoNetworkMoreDetails:"La chiamata è stata disconnessa a causa di un problema di rete. Controlla la connessione e partecipa di nuovo.",failedToJoinCallDueToNoNetworkTitle:"Chiamata interrotta",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Non ti è stato concesso l’accesso alla chiamata. Se si è trattato di un errore, partecipa di nuovo alla chiamata.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Ignorato dalla sala di attesa",learnMore:"Ulteriori informazioni",leavingCallTitle:"In uscita...",leftCallMoreDetails:"Se si è trattato di un errore, partecipa di nuovo alla chiamata.",leftCallTitle:"Hai abbandonato la chiamata",lobbyScreenConnectingToCallTitle:"Partecipazione alla chiamata",lobbyScreenWaitingToBeAdmittedTitle:"In attesa di essere ammessi",microphonePermissionDenied:"Il browser blocca l'accesso al microfono",microphoneToggleInLobbyNotAllowed:"Impossibile attivare o disattivare l'audio nella sala di attesa.",mutedMessage:"L'audio è disattivato.",networkReconnectMoreDetails:"Sembra si sia verificato un errore. Stiamo cercando di ristabilire la connessione alla chiamata.",networkReconnectTitle:"Attendi",notInvitedToRoomDetails:"Non puoi partecipare a questa chat room perché non hai un invito.",notInvitedToRoomTitle:"Non invitato alla stanza",peopleButtonLabel:"Persone",selectedPeopleButtonLabel:"Pulsante Persone selezionato",peoplePaneTitle:"Persone",peoplePaneMoreButtonAriaLabel:"Altro",peopleButtonTooltipOpen:"Mostra partecipanti",peopleButtonTooltipClose:"Nascondi partecipanti",peoplePaneSubTitle:"In questa chiamata {numberOfPeople}",privacyPolicy:"Informativa sulla privacy",rejoinCallButtonLabel:"Partecipa alla chiamata",removedFromCallMoreDetails:"Un altro partecipante ti ha rimosso dalla chiamata.",removedFromCallTitle:"Sei stato rimosso",removeMenuLabel:"Rimuovi",startSpotlightMenuLabel:"Metti in evidenza per tutti",addSpotlightMenuLabel:"Metti in evidenza",stopSpotlightMenuLabel:"Interrompi la posizione in evidenza",stopSpotlightOnSelfMenuLabel:"Chiudi visualizzazione in evidenza",spotlightLimitReachedMenuTitle:"È stato raggiunto il limite di contenuti in evidenza",stopAllSpotlightMenuLabel:"Smetti di mostrare in evidenza",returnToCallButtonAriaDescription:"Torna alla chiamata",returnToCallButtonAriaLabel:"Indietro",roomNotFoundDetails:"L'ID stanza specificato non è stato trovato.",roomNotFoundTitle:"La stanza non è stata trovata",roomNotValidDetails:"Questa stanza non è valida in questo momento.",roomNotValidTitle:"Stanza non valida",inviteToRoomRemovedDetails:"L'invito a partecipare a questa stanza è stato rimosso.",inviteToRoomRemovedTitle:"L’invito alla stanza è stato rimosso",soundLabel:"Suono",noMicrophonesLabel:"Nessun microfono trovato",noSpeakersLabel:"Nessun altoparlante trovato",startCallButtonLabel:"Avvia chiamata",openDialpadButtonLabel:"Componi il numero di telefono",peoplePaneAddPeopleButtonLabel:"Aggiungi persone",dialpadStartCallButtonLabel:"Chiamata",dialpadModalTitle:"Componi il numero di telefono",dialpadModalAriaLabel:"Tastiera",dialpadCloseModalButtonAriaLabel:"Chiudi tastiera",moreButtonCallingLabel:"Altro",resumeCallButtonLabel:"Riprendi",resumingCallButtonLabel:"Ripresa in corso...",resumeCallButtonAriaLabel:"Riprendi chiamata",resumingCallButtonAriaLabel:"Riprendi chiamata",holdScreenLabel:"In attesa.",openDtmfDialpadLabel:"Mostra tastiera",dtmfDialpadPlaceholderText:"Immetti il numero",outboundCallingNoticeString:"Chiamata in corso...",participantJoinedNoticeString:"{displayName} si è unito alla riunione",twoParticipantJoinedNoticeString:"{displayName1} e {displayName2} si sono uniti alla riunione",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} e {displayName3} si sono uniti alla riunione",participantLeftNoticeString:"{displayName} è uscito dalla riunione",twoParticipantLeftNoticeString:"{displayName1} e {displayName2} sono usciti dalla riunione",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} e {displayName3} sono usciti dalla riunione",unnamedParticipantString:"partecipante senza nome",manyUnnamedParticipantsJoined:"partecipante senza nome e {numOfParticipants} altri partecipanti si sono uniti alla riunione",manyUnnamedParticipantsLeft:"partecipante senza nome e {numOfParticipants} altri partecipanti sono usciti dalla riunione",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} e {numOfParticipants} altri partecipanti si sono uniti",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} e {numOfParticipants} altri partecipanti rimasti",liveCaptionsLabel:"Sottotitoli in tempo reale",captionsSettingsLabel:"Impostazioni sottotitoli",startCaptionsButtonOnLabel:"Attiva i sottotitoli",startCaptionsButtonOffLabel:"Disabilita sottotitoli",startCaptionsButtonTooltipOnContent:"Disabilita sottotitoli",startCaptionsButtonTooltipOffContent:"Attiva i sottotitoli",captionsSettingsModalTitle:"Impostazioni sottotitoli",captionsSettingsSpokenLanguageDropdownLabel:"Lingua parlata",captionsSettingsCaptionLanguageDropdownLabel:"Lingua dei sottotitoli",captionsSettingsSpokenLanguageDropdownInfoText:"La lingua che parlano tutti i partecipanti a questa chiamata.",captionsSettingsCaptionLanguageDropdownInfoText:"I sottotitoli verranno visualizzati in questa lingua.",captionsSettingsConfirmButtonLabel:"Conferma",captionsSettingsCancelButtonLabel:"Annulla",captionsSettingsModalAriaLabel:"Finestra modale impostazioni sottotitoli",captionsSettingsCloseModalButtonAriaLabel:"Chiudi impostazione sottotitoli",captionsBannerMoreButtonCallingLabel:"Altre opzioni Impostazioni sottotitoli",captionsBannerMoreButtonTooltip:"Altre opzioni",realTimeTextLabel:"Scrittura in tempo reale (RTT)",startRealTimeTextLabel:"Attiva la scrittura in tempo reale per questa chiamata",disabledStartRealTimeTextLabel:"RTT è attualmente attivato, non può essere disattivato",dismissModalAriaLabel:"Immagine e immagine locale e remota, premi INVIO per tornare alla chiamata",callRejectedTitle:"Non è stato possibile completare la chiamata",callRejectedMoreDetails:"La persona che stai tentando di raggiungere non è disponibile. Riprovare più tardi.",callTimeoutTitle:"Non è stato possibile completare la chiamata",callTimeoutDetails:"La persona che stai tentando di raggiungere non è disponibile. Riprovare più tardi.",callTimeoutBotTitle:"Non è stato possibile completare la chiamata",callTimeoutBotDetails:"Il servizio che si sta tentando di raggiungere non è disponibile. Riprovare più tardi.",dtmfDialerButtonLabel:"Tastiera",dtmfDialerButtonTooltipOn:"Mostra tastiera",dtmfDialerButtonTooltipOff:"Nascondi tastierino",dtmfDialerMoreButtonLabelOn:"Mostra tastiera",dtmfDialerMoreButtonLabelOff:"Nascondi tastierino",phoneCallMoreButtonLabel:"Chiamata telefonica",spokenLanguageStrings:{"ar-ae":"Arabo - Emirati Arabi Uniti","ar-sa":"Arabo - Arabia Saudita","da-dk":"Danese","de-de":"Tedesco - Germania","en-au":"Inglese - Australia","en-ca":"Inglese - Canada","en-gb":"Inglese - Regno Unito","en-in":"Inglese - India","en-nz":"Inglese - Nuova Zelanda","en-us":"Inglese - Stati Uniti","es-es":"Spagnolo - Spagna (ordinamento moderno)","es-mx":"Spagnolo - Messico","fi-fi":"Finlandese","fr-ca":"Francese - Canada","fr-fr":"Francese - Francia","hi-in":"Hindi","it-it":"Italiano - Italia","ja-jp":"Giapponese","ko-kr":"Coreano","nb-no":"Norvegese (Bokmål)","nl-be":"Olandese - Belgio","nl-nl":"Olandese - Paesi Bassi","pl-pl":"Polacco","pt-br":"Portoghese - Brasile","ru-ru":"Russo","sv-se":"Svedese","zh-cn":"Cinese - Repubblica popolare cinese","zh-hk":"Cinese - Hong Kong R.A.S.","cs-cz":"Ceco","pt-pt":"Portoghese - Portogallo","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Thailandese","he-il":"Ebraico","cy-gb":"Gallese","uk-ua":"Ucraino","el-gr":"Greco","hu-hu":"Ungherese","ro-ro":"Rumeno","sk-sk":"Slovacco","zh-tw":"Cinese - Taiwan"},captionLanguageStrings:{ar:"Arabo",da:"Danese",de:"Tedesco",en:"Inglese",es:"Spagnolo",fi:"Finlandese","fr-ca":"Francese - Canada",fr:"Francese - Francia",hi:"Hindi",it:"Italiano",ja:"Giapponese",ko:"Coreano",nb:"Norvegese (Bokmål)",nl:"Olandese",pl:"Polacco",pt:"Portoghese - Brasile",ru:"Russo",sv:"Svedese","zh-Hans":"Cinese (semplificato)","zh-Hant":"Cinese (tradizionale)",cs:"Ceco","pt-pt":"Portoghese - Portogallo",tr:"Turco",vi:"Vietnamita",th:"Thailandese",he:"Ebraico",cy:"Gallese",uk:"Ucraino",el:"Greco",hu:"Ungherese",ro:"Rumeno",sk:"Slovacco"},captionsBannerSpinnerText:"Avvio dei sottotitoli in corso...",realTimeTextInputBoxDefaultText:"Digita messaggio in tempo reale",realTimeTextInputErrorMessage:"Il messaggio non può superare i 2000 caratteri",realTimeTextBannerTitle:"Scrittura in tempo reale",realTimeTextBannerContent:"RTT (testo in tempo reale) è abilitato per tutti i partecipanti per l'intera durata della riunione.",realTimeTextBannerLinkLabel:"Scopri di più",realTimeTextModalAriaLabel:"Abilita finestra modale scrittura in tempo reale",realTimeTextModalTitle:"Abilitare la scrittura in tempo reale?",realTimeTextModalText:"Il testo in tempo reale verrà attivato per tutti gli utenti della riunione.",realTimeTextConfirmButtonLabel:"Abilita",realTimeTextCancelButtonLabel:"Annulla",realTimeTextCloseModalButtonAriaLabel:"Chiudi finestra modale scrittura in tempo reale",captionsOnlyContainerTitle:"Sottotitoli",realTimeTextOnlyContainerTitle:"Testo in tempo reale",captionsAndRealTimeTextContainerTitle:"Sottotitoli e testo in tempo reale",expandButtonAriaLabel:"Espandi",minimizeButtonAriaLabel:"Comprimi",transferPageTransferorText:"Trasferimento in corso...",transferPageTransferTargetText:"Connessione in corso...",transferPageUnknownTransferorDisplayName:"Sconosciuto",transferPageUnknownTransferTargetDisplayName:"Sconosciuto",transferPageNoticeString:"Stai per essere trasferito",participantCouldNotBeReachedTitle:"Il partecipante di riferimento non è attualmente disponibile",participantCouldNotBeReachedMoreDetails:"Contatta il partecipante quando è disponibile",permissionToReachTargetParticipantNotAllowedTitle:"L'autorizzazione per raggiungere il partecipante di riferimento non è consentita",permissionToReachTargetParticipantNotAllowedMoreDetails:"Verificare che il partecipante di riferimento si trovi nello stesso tenant",unableToResolveTenantTitle:"Non è possibile risolvere l'ID tenant per il partecipante di destinazione",unableToResolveTenantMoreDetails:"Verifica che l'ID partecipante sia stato immesso correttamente",participantIdIsMalformedTitle:"Il formato dell'ID partecipante non è corretto",participantIdIsMalformedMoreDetails:"Verifica che il formato dell'ID partecipante sia corretto",moreButtonGalleryControlLabel:"Visualizza",moreButtonGalleryPositionToggleLabel:"Sposta raccolta in alto",moreButtonGallerySpeakerLayoutLabel:"Altoparlante",moreButtonGalleryFloatingLocalLayoutLabel:"Dinamico",moreButtonGalleryDefaultLayoutLabel:"Visualizzazione raccolta",moreButtonGalleryFocusedContentLayoutLabel:"Focus sul contenuto",moreButtonLargeGalleryDefaultLayoutLabel:"Galleria estesa",moreButtonTogetherModeLayoutLabel:"Modalità Insieme",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"La fotocamera è stata disattivata. Non è più possibile condividere video.",grantedDueToMeetingOption:"La fotocamera è stata abilitata. Attivala se vuoi condividere il video."},unmuteMic:{lostDueToMeetingOption:"Il microfono è stato disabilitato. Non puoi più riattivare l'audio.",grantedDueToMeetingOption:"Il microfono è stato abilitato. Riattiva l'audio se vuoi parlare."},shareScreen:{lostDueToRoleChangeToAttendee:"Il tuo ruolo è stato modificato. Alcune azioni, come la condivisione di contenuti, non saranno disponibili per te.",grantedDueToRoleChangeToPresenter:"Sei un relatore. Puoi condividere i contenuti e facilitare la riunione."},hideAttendeeNames:{hideAttendeeNameAttendee:"Il tuo nome è nascosto agli altri partecipanti. Gli organizzatori e i relatori possono vedere il tuo vero nome.",hideAttendeeNamePresenter:"I nomi dei partecipanti sono nascosti. Solo gli organizzatori e i relatori possono visualizzare i nomi per proteggere la privacy dei partecipanti."}},surveyTitle:"Aiutaci a migliorare",starSurveyHelperText:"Come valuti la qualità della chiamata?",starSurveyOneStarText:"La qualità era pessima.",starSurveyTwoStarText:"La qualità era scadente.",starSurveyThreeStarText:"La qualità era buona.",starSurveyFourStarText:"La qualità era ottima.",starSurveyFiveStarText:"La qualità è stata eccellente.",surveyConfirmButtonLabel:"Invia feedback",starRatingAriaLabel:"Seleziona {0} di {1} stelle",surveySkipButtonLabel:"Salta",tagsSurveyQuestion:"Cosa avrebbe potuto essere migliore?",tagsSurveyTextFieldDefaultText:"Altro, specifica",tagsSurveyHelperText:"Controlla eventuali problemi riscontrati",endOfSurveyText:"Grazie per il feedback!",surveyIssues:{overallRating:{callCannotJoin:"Non sono riuscito a partecipare alla chiamata",callCannotInvite:"Non ho potuto invitare altri partecipanti alla chiamata",hadToRejoin:"Ho dovuto partecipare di nuovo alla chiamata",callEndedUnexpectedly:"La chiamata è terminata per me in modo imprevisto",otherIssues:"Ho avuto altri problemi con la chiamata"},audioRating:{noLocalAudio:"L'altro utente non mi sentiva",noRemoteAudio:"Non riuscivo a sentire niente",echo:"Ho sentito eco durante la chiamata",audioNoise:"Ho sentito rumore durante la chiamata",lowVolume:"Il volume era basso",audioStoppedUnexpectedly:"L’audio si è interrotto improvvisamente",distortedSpeech:"L'audio era distorto",audioInterruption:"L'audio è stato interrotto",otherIssues:"Ho avuto altri problemi audio in questa chiamata"},videoRating:{noVideoReceived:"Non vedevo nulla",noVideoSent:"Gli altri utenti non potevano vedermi",lowQuality:"La qualità del video era bassa",freezes:"Video bloccato",stoppedUnexpectedly:"Il video si è interrotto improvvisamente",darkVideoReceived:"Posso vedere gli schermi scuri solo quando altri utenti accendono la fotocamera",audioVideoOutOfSync:"Audio e immagini non erano sincronizzati",otherIssues:"Ho avuto altri problemi video in questa chiamata"},screenshareRating:{noContentLocal:"Altre persone non possono vedere la condivisione dello schermo",noContentRemote:"Non ho potuto visualizzare la condivisione dello schermo di altre persone",cannotPresent:"Non ho potuto presentare lo schermo",lowQuality:"La qualità della condivisione dello schermo è bassa",freezes:"Condivisione schermo bloccata",stoppedUnexpectedly:"Condivisione dello schermo interrotta in modo imprevisto",largeDelay:"La condivisione dello schermo presenta un ritardo elevato",otherIssues:"Ho avuto altri problemi di condivisione dello schermo in questa chiamata"}},surveyIssuesHeadingStrings:{overallRating:"Generale",audioRating:"Audio",videoRating:"Video",screenshareRating:"Presentazione in corso"},spotlightPrompt:{startSpotlightHeading:"Mettere in evidenza per tutti?",startSpotlightText:"Questo video sarà in evidenza per tutti i partecipanti alla riunione.",startSpotlightOnSelfText:"Il tuo video sarà in evidenza per tutti i partecipanti alla riunione.",startSpotlightConfirmButtonLabel:"Metti in evidenza per tutti",startSpotlightCancelButtonLabel:"Annulla",stopSpotlightHeading:"Smettere di mostrare in evidenza questo video?",stopSpotlightOnSelfHeading:"Uscire dalla messa in evidenza?",stopAllSpotlightHeading:"Vuoi smettere di mostrare in evidenza tutti i video?",stopSpotlightText:"Questo video non sarà più in evidenza per tutti i partecipanti alla riunione.",stopSpotlightOnSelfText:"Il tuo video non sarà più in evidenza per tutti i partecipanti alla riunione.",stopAllSpotlightText:"I video non saranno più in evidenza per tutti i partecipanti della riunione.",stopSpotlightConfirmButtonLabel:"Smetti di mettere in evidenza",stopSpotlightOnSelfConfirmButtonLabel:"Esci da visualizzazione in evidenza",stopSpotlightCancelButtonLabel:"Annulla",closeSpotlightPromptButtonLabel:"Chiudi"},exitSpotlightButtonLabel:"Esci da visualizzazione in evidenza",exitSpotlightButtonTooltip:"Esci da visualizzazione in evidenza",leaveConfirmButtonLabel:"Abbandona",endCallConfirmButtonLabel:"Termina chiamata",hangUpCancelButtonLabel:"Annulla",leaveConfirmDialogTitle:"Vuoi abbandonare la chiamata?",leaveConfirmDialogContent:"Vuoi abbandonare la chiamata?",endCallConfirmDialogTitle:"Vuoi terminare la chiamata?",endCallConfirmDialogContent:"La chiamata terminerà per tutti gli utenti.",invalidMeetingIdentifier:"ID riunione o passcode non valido",pinParticipantMenuLabel:"Aggiungi per me",pinParticipantLimitReachedMenuLabel:"Segnaposto (limite raggiunto)",unpinParticipantMenuLabel:"Rimuovere dall’alto",pinParticipantMenuItemAriaLabel:"Aggiungi per me",unpinParticipantMenuItemAriaLabel:"Rimuovere dall’alto",muteAllMenuLabel:"Disattivare l'audio di tutti",muteAllDialogTitle:"Disattivare l'audio di tutti?",muteAllDialogContent:"Questa operazione disattiverà l'audio di tutte le persone nella riunione, tranne te.",muteAllConfirmButtonLabel:"Disattivare l'audio di tutti",muteAllCancelButtonLabel:"Annullare",joinBreakoutRoomButtonLabel:"Partecipa al gruppo di lavoro",returnFromBreakoutRoomButtonLabel:"Esci dalla stanza",leaveBreakoutRoomAndMeetingButtonLabel:"Abbandona riunione",breakoutRoomJoinedNotificationTitle:"Sei stato aggiunto alla stanza di lavoro '{roomName}'",joinBreakoutRoomBannerTitle:"La stanza di lavoro '{roomName}' è aperta",joinBreakoutRoomBannerButtonLabel:"Partecipa",returnFromBreakoutRoomBannerTitle:"Torna alla riunione principale",returnFromBreakoutRoomBannerButtonLabel:"Partecipa di nuovo",forbidAudioMenuLabel:"Disabilita microfono",permitAudioMenuLabel:"Consenti microfono",forbidOthersAudioDialogTitle:"Disabilita il microfono dei partecipanti?",forbidOthersAudioDialogContent:"I partecipanti non potranno riattivare l'audio. L'organizzatore e i relatori possono consentire ai partecipanti di riattivare l'audio se necessario.",forbidOthersAudioConfirmButtonLabel:"Disabilita microfoni",forbidOthersAudioCancelButtonLabel:"Annulla",permitOthersAudioDialogTitle:"Consentire il microfono dei partecipanti?",permitOthersAudioDialogContent:"Tutti i partecipanti alla riunione saranno in grado di riattivare l’audio.",permitOthersAudioConfirmButtonLabel:"Consenti microfoni",permitOthersAudioCancelButtonLabel:"Annulla",forbidOthersAudioMenuLabel:"Disabilita il microfono dei partecipanti",permitOthersAudioMenuLabel:"Abilita il microfono dei partecipanti",forbidVideoMenuLabel:"Disabilita videocamera",permitVideoMenuLabel:"Consenti videocamera",forbidOthersVideoDialogTitle:"Disabilitare la videocamera dei partecipanti?",forbidOthersVideoDialogContent:"I partecipanti non potranno attivare autonomamente il video. L'organizzatore e i relatori possono consentire alle persone di attivare la fotocamera se necessario.",forbidOthersVideoConfirmButtonLabel:"Disabilita videocamere",forbidOthersVideoCancelButtonLabel:"Annulla",permitOthersVideoDialogTitle:"Consentire la videocamera dei partecipanti?",permitOthersVideoDialogContent:"Tutti gli utenti della riunione potranno attivare autonomamente il video.",permitOthersVideoConfirmButtonLabel:"Consenti videocamere",permitOthersVideoCancelButtonLabel:"Annulla",forbidOthersVideoMenuLabel:"Disabilita videocamera dei partecipanti",permitOthersVideoMenuLabel:"Consenti videocamera dei partecipanti"};var chat$b={chatListHeader:"In questa chat",uploadAttachment:"Carica allegato",uploadImageDataNotProvided:"I dati dell'immagine non sono stati forniti.",uploadImageIsTooLarge:"L'immagine è troppo grande. Scegliere un valore inferiore a {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Il caricamento dell’immagine .{imageExtension} non è consentito.",uploadImageFailed:"Non è possibile caricare l'immagine. Riprovare più tardi."};var callWithChat$b={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"Nuovo messaggio",chatButtonTooltipClosedWithMessageCount:"Mostra chat ({unreadMessagesCount} non letto)",chatButtonTooltipClose:"Nascondi chat",chatButtonTooltipOpen:"Mostra chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Copia il collegamento dell’invito",copyInviteLinkButtonActionedLabel:"Collegamento copiato",copyInviteLinkActionedAriaLabel:"Collegamento di invito copiato",dismissSidePaneButtonLabel:"Chiudi",moreDrawerAudioDeviceMenuTitle:"Dispositivo audio",moreDrawerButtonLabel:"Altre opzioni",moreDrawerButtonTooltip:"Altre opzioni",moreDrawerMicrophoneMenuTitle:"Microfono",moreDrawerSpeakerMenuTitle:"Altoparlante",moreDrawerCaptionsMenuTitle:"Sottotitoli in tempo reale",moreDrawerSpokenLanguageMenuTitle:"Lingua parlata",moreDrawerCaptionLanguageMenuTitle:"Lingua sottotitoli",moreDrawerGalleryOptionsMenuTitle:"Opzioni della raccolta",peopleButtonLabel:"Persone",selectedPeopleButtonLabel:"Pulsante Persone selezionato",peopleButtonTooltipOpen:"Mostra partecipanti",peopleButtonTooltipClose:"Nascondi partecipanti",peopleButtonTooltipOpenAriaLabel:"pulsante Persone, Mostra partecipanti",peopleButtonTooltipCloseAriaLabel:"Pulsante Persone, Nascondi partecipanti",peoplePaneSubTitle:"In questa chiamata",peoplePaneTitle:"Persone",pictureInPictureTileAriaLabel:"Feed video. Fare clic per tornare alla schermata della chiamata.",removeMenuLabel:"Rimuovi",openDialpadButtonLabel:"Componi il numero di telefono",returnToCallButtonAriaDescription:"Torna alla chiamata",returnToCallButtonAriaLabel:"Indietro",peoplePaneAddPeopleButtonLabel:"Aggiungi persone",dialpadStartCallButtonLabel:"Chiamata",dialpadModalTitle:"Componi il numero di telefono",dialpadModalAriaLabel:"Tastiera",dialpadCloseModalButtonAriaLabel:"Chiudi tastiera",openDtmfDialpadLabel:"Mostra tastiera",dtmfDialpadPlaceholderText:"Immetti il numero",breakoutRoomChatPaneTitle:"Chat della stanza di lavoro",chatContentSpinnerLabel:"Caricamento della chat"};var it_IT = {call:call$b,chat:chat$b,callWithChat:callWithChat$b};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Italian (Italy)
 *
 * @public
 */
const COMPOSITE_LOCALE_IT_IT = {
    component: COMPONENT_LOCALE_IT_IT,
    strings: createCompositeStrings(it_IT)
};

var call$a={cameraLabel:"カメラ",noCamerasLabel:"カメラが見つかりません",cameraPermissionDenied:"お使いのブラウザーがカメラへのアクセスをブロックしています",cameraTurnedOff:"カメラはオフになっています",chatButtonLabel:"チャット",close:"閉じる",complianceBannerNowOnlyRecording:"今はこの会議の録画のみを行っています。",complianceBannerNowOnlyTranscription:"この会議のトランスクリプトを作成しています。",complianceBannerRecordingAndTranscriptionSaved:"レコーディングとトランスクリプトを保存されました。",complianceBannerRecordingAndTranscriptionStarted:"レコーディングとトランスクリプトが開始されました。",complianceBannerRecordingAndTranscriptionStopped:"レコーディングとトランスクリプトを停止しました。",complianceBannerRecordingSaving:"レコーディングを保存しています。",complianceBannerRecordingStarted:"レコーディングを開始しました。",complianceBannerRecordingStopped:"レコーディングを停止しました。",complianceBannerTranscriptionStarted:"トランスクリプトが開始しました。",complianceBannerTranscriptionConsent:"参加すると、この会議のトランスクリプトの作成に同意したことになります。",complianceBannerTranscriptionSaving:"トランスクリプトを保存されました。",complianceBannerTranscriptionStopped:"トランスクリプトが停止しました。",configurationPageTitle:"通話を開始",copyInviteLinkButtonLabel:"招待用のリンクをコピー",copyInviteLinkButtonActionedLabel:"リンクをコピーしました",copyInviteLinkActionedAriaLabel:"招待用のリンクがコピーされました",defaultPlaceHolder:"オプションを選択する",dismissSidePaneButtonLabel:"閉じる",videoEffectsPaneTitle:"効果",videoEffectsPaneBackgroundSelectionTitle:"バックグラウンド",videoEffectsPaneAriaLabel:"ビデオ効果ウィンドウ",configurationPageCameraIsLoadingLabel:"ビデオの読み込み中です...",configurationPageVideoEffectsButtonLabel:"効果",unableToStartVideoEffect:"ビデオ効果を適用できません。",blurBackgroundEffectButtonLabel:"ぼかし",blurBackgroundTooltip:"背景をぼかす",removeBackgroundEffectButtonLabel:"なし",removeBackgroundTooltip:"背景の削除",cameraOffBackgroundEffectWarningText:"カメラがオフになっています。カメラをオンにすると、ビデオ効果が表示されます。",failedToJoinCallDueToNoNetworkMoreDetails:"ネットワークの問題により、通話が切断されました。接続を確認して、もう一度参加してください。",failedToJoinCallDueToNoNetworkTitle:"通話が切断されました",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"通話のエントリが許可されませんでした。これが間違いだった場合は、通話に再参加してください。",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"ロビーから消去しました",learnMore:"詳細情報",leavingCallTitle:"退出中...",leftCallMoreDetails:"これが間違いだった場合は、通話に再度参加してください。",leftCallTitle:"通話を終了しました",lobbyScreenConnectingToCallTitle:"通話に参加中",lobbyScreenWaitingToBeAdmittedTitle:"承認待機中",microphonePermissionDenied:"お使いのブラウザーがマイクへのアクセスをブロックしています",microphoneToggleInLobbyNotAllowed:"ロビーにいる間はミュートまたはミュート解除できません。",mutedMessage:"ミュートになっています",networkReconnectMoreDetails:"問題が発生したようです。通話に戻ろうとしています。",networkReconnectTitle:"お待ちください",notInvitedToRoomDetails:"招待がないため、このルームに参加できません。",notInvitedToRoomTitle:"ルームに招待されていません",peopleButtonLabel:"連絡先",selectedPeopleButtonLabel:"[ユーザー] ボタンが選択されました",peoplePaneTitle:"連絡先",peoplePaneMoreButtonAriaLabel:"その他",peopleButtonTooltipOpen:"参加者を表示",peopleButtonTooltipClose:"参加者を非表示",peoplePaneSubTitle:"この通話では {numberOfPeople} 人",privacyPolicy:"プライバシー ポリシー",rejoinCallButtonLabel:"通話に再度参加",removedFromCallMoreDetails:"別の参加者があなたを通話から削除しました。",removedFromCallTitle:"削除されました",removeMenuLabel:"削除",startSpotlightMenuLabel:"全員にスポットライトを設定",addSpotlightMenuLabel:"スポットライトを追加",stopSpotlightMenuLabel:"スポットライトの設定を解除",stopSpotlightOnSelfMenuLabel:"スポットライトを終了",spotlightLimitReachedMenuTitle:"スポットライトの上限に達しました",stopAllSpotlightMenuLabel:"すべてのスポットライトを停止",returnToCallButtonAriaDescription:"通話に戻る",returnToCallButtonAriaLabel:"戻る",roomNotFoundDetails:"指定されたルーム ID が見つかりませんでした。",roomNotFoundTitle:"ルームが見つかりません",roomNotValidDetails:"このルームは現在無効です。",roomNotValidTitle:"ルームが無効です",inviteToRoomRemovedDetails:"このルームに参加するための招待が削除されました。",inviteToRoomRemovedTitle:"ルームへの招待が削除されました",soundLabel:"音声",noMicrophonesLabel:"マイクが見つかりません",noSpeakersLabel:"話者が見つかりません",startCallButtonLabel:"通話を開始",openDialpadButtonLabel:"電話番号をダイヤルする",peoplePaneAddPeopleButtonLabel:"ユーザーの追加",dialpadStartCallButtonLabel:"通話",dialpadModalTitle:"電話番号をダイヤルする",dialpadModalAriaLabel:"ダイヤルパッド",dialpadCloseModalButtonAriaLabel:"ダイヤルパッドを閉じる",moreButtonCallingLabel:"その他",resumeCallButtonLabel:"再開",resumingCallButtonLabel:"再開しています...",resumeCallButtonAriaLabel:"通話の再開",resumingCallButtonAriaLabel:"通話の再開",holdScreenLabel:"保留中です",openDtmfDialpadLabel:"ダイヤルパッドを表示",dtmfDialpadPlaceholderText:"番号を入力してください",outboundCallingNoticeString:"呼び出し中。。。",participantJoinedNoticeString:"{displayName} が参加しました",twoParticipantJoinedNoticeString:"{displayName1} さんと {displayName2} さんが参加しました",threeParticipantJoinedNoticeString:"{displayName1} さん, {displayName2} さんと {displayName3} さんが参加しました",participantLeftNoticeString:"{displayName} さんが退出しました",twoParticipantLeftNoticeString:"{displayName1} さんと {displayName2} さんが退出しました",threeParticipantLeftNoticeString:"{displayName1} さん, {displayName2} さんと {displayName3} さんが退出しました",unnamedParticipantString:"名前のない参加者",manyUnnamedParticipantsJoined:"名前のない参加者と他の参加者 {numOfParticipants} 人が参加しました",manyUnnamedParticipantsLeft:"名前のない参加者と他の参加者 {numOfParticipants} 人が退出しました",manyParticipantsJoined:"{displayName1}、{displayName2}、{displayName3}、その他の参加者{numOfParticipants}参加しました",manyParticipantsLeft:"{displayName1}、{displayName2}、{displayName3}、その他の参加者{numOfParticipants}退出しました",liveCaptionsLabel:"ライブ キャプション",captionsSettingsLabel:"字幕の設定",startCaptionsButtonOnLabel:"キャプションをオンにする",startCaptionsButtonOffLabel:"キャプションをオフにする",startCaptionsButtonTooltipOnContent:"キャプションをオフにする",startCaptionsButtonTooltipOffContent:"キャプションをオンにする",captionsSettingsModalTitle:"ライブ キャプションの設定",captionsSettingsSpokenLanguageDropdownLabel:"音声言語",captionsSettingsCaptionLanguageDropdownLabel:"ライブ キャプションの言語",captionsSettingsSpokenLanguageDropdownInfoText:"この通話の全員が話している言語です。",captionsSettingsCaptionLanguageDropdownInfoText:"キャプションはこの言語で表示されます。",captionsSettingsConfirmButtonLabel:"確認",captionsSettingsCancelButtonLabel:"キャンセル",captionsSettingsModalAriaLabel:"字幕設定モーダル",captionsSettingsCloseModalButtonAriaLabel:"字幕設定を閉じる",captionsBannerMoreButtonCallingLabel:"キャプション設定のその他のオプション",captionsBannerMoreButtonTooltip:"その他のオプション",realTimeTextLabel:"リアルタイム テキスト (RTT)",startRealTimeTextLabel:"この呼び出しの RTT を有効にする",disabledStartRealTimeTextLabel:"RTT は現在オンになっています。オフにすることはできません",dismissModalAriaLabel:"ローカルとリモートの画像と画像。通話に戻るには Enter キーを押します",callRejectedTitle:"通話を完了できませんでした",callRejectedMoreDetails:"連絡を取ろうとしているユーザーは利用できません。後でもう一度お試しください。",callTimeoutTitle:"通話を完了できませんでした",callTimeoutDetails:"連絡を取ろうとしているユーザーは利用できません。後でもう一度お試しください。",callTimeoutBotTitle:"通話を完了できませんでした",callTimeoutBotDetails:"接続しようとしているサービスは利用できません。後でもう一度お試しください。",dtmfDialerButtonLabel:"ダイヤルパッド",dtmfDialerButtonTooltipOn:"ダイヤルパッドを表示",dtmfDialerButtonTooltipOff:"ダイヤル パッドを非表示",dtmfDialerMoreButtonLabelOn:"ダイヤルパッドを表示",dtmfDialerMoreButtonLabelOff:"ダイヤル パッドを非表示",phoneCallMoreButtonLabel:"電話での通話",spokenLanguageStrings:{"ar-ae":"アラビア語 - U.A.E.","ar-sa":"アラビア語 - サウジアラビア","da-dk":"デンマーク語","de-de":"ドイツ語 - ドイツ","en-au":"英語 - オーストラリア","en-ca":"英語 - カナダ","en-gb":"英語 - イギリス","en-in":"英語 - インド","en-nz":"英語 - ニュージーランド","en-us":"英語 - 米国","es-es":"スペイン語 - スペイン (モダン)","es-mx":"スペイン語 - メキシコ","fi-fi":"フィンランド語","fr-ca":"フランス語 - カナダ","fr-fr":"フランス語 - フランス","hi-in":"ヒンディー語","it-it":"イタリア語 - イタリア","ja-jp":"日本語","ko-kr":"韓国語","nb-no":"ノルウェー語 (ブークモール)","nl-be":"オランダ語 - ベルギー","nl-nl":"オランダ語 - オランダ","pl-pl":"ポーランド語","pt-br":"ポルトガル語 - ブラジル","ru-ru":"ロシア語","sv-se":"スウェーデン語","zh-cn":"中国語 - 中国","zh-hk":"中国語 - 香港特別行政区","cs-cz":"チェコ語","pt-pt":"ポルトガル語 - ポルトガル","tr-tr":"トルコ語","vi-vn":"ベトナム語","th-th":"タイ語","he-il":"ヘブライ語","cy-gb":"ウェールズ語","uk-ua":"ウクライナ語","el-gr":"ギリシャ語","hu-hu":"ハンガリー語","ro-ro":"ルーマニア語","sk-sk":"スロバキア語","zh-tw":"中国語 - 台湾"},captionLanguageStrings:{ar:"アラビア語",da:"デンマーク語",de:"ドイツ語",en:"英語",es:"スペイン語",fi:"フィンランド語","fr-ca":"フランス語 - カナダ",fr:"フランス語 - フランス",hi:"ヒンディー語",it:"イタリア語",ja:"日本語",ko:"韓国語",nb:"ノルウェー語 (ブークモール)",nl:"オランダ語",pl:"ポーランド語",pt:"ポルトガル語 - ブラジル",ru:"ロシア語",sv:"スウェーデン語","zh-Hans":"中国語 (簡体字)","zh-Hant":"中国語 (繁体字)",cs:"チェコ語","pt-pt":"ポルトガル語 - ポルトガル",tr:"トルコ語",vi:"ベトナム語",th:"タイ語",he:"ヘブライ語",cy:"ウェールズ語",uk:"ウクライナ語",el:"ギリシャ語",hu:"ハンガリー語",ro:"ルーマニア語",sk:"スロバキア語"},captionsBannerSpinnerText:"キャプションを開始しています...",realTimeTextInputBoxDefaultText:"リアルタイムにメッセージを入力",realTimeTextInputErrorMessage:"メッセージは 2,000 文字以下にする必要があります",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (リアルタイム テキスト) は、会議の全期間のすべての参加者に対して有効になっています。",realTimeTextBannerLinkLabel:"詳細情報",realTimeTextModalAriaLabel:"RTT モーダルを有効にする",realTimeTextModalTitle:"RTT を有効にしますか?",realTimeTextModalText:"会議の全員に対してリアルタイム テキスト (RTT) が有効になります。",realTimeTextConfirmButtonLabel:"有効にする",realTimeTextCancelButtonLabel:"キャンセル",realTimeTextCloseModalButtonAriaLabel:"RTT モーダルを閉じる",captionsOnlyContainerTitle:"キャプション",realTimeTextOnlyContainerTitle:"リアルタイム テキスト",captionsAndRealTimeTextContainerTitle:"キャプションとリアルタイム テキスト",expandButtonAriaLabel:"展開",minimizeButtonAriaLabel:"折りたたむ",transferPageTransferorText:"転送しています...",transferPageTransferTargetText:"接続中...",transferPageUnknownTransferorDisplayName:"不明",transferPageUnknownTransferTargetDisplayName:"不明",transferPageNoticeString:"転送中です",participantCouldNotBeReachedTitle:"ターゲットの参加者は現在連絡できません",participantCouldNotBeReachedMoreDetails:"参加者が連絡可能になったら連絡してください",permissionToReachTargetParticipantNotAllowedTitle:"ターゲットの参加者への連絡が許可されていません",permissionToReachTargetParticipantNotAllowedMoreDetails:"ターゲットの参加者が同じテナントにいることを確認してください",unableToResolveTenantTitle:"ターゲット参加者のテナント ID を解決できません",unableToResolveTenantMoreDetails:"参加者 ID が正しく入力チェックしてください",participantIdIsMalformedTitle:"参加者 ID の形式が正しくありません",participantIdIsMalformedMoreDetails:"参加者 ID の形式が正しいことをチェックしてください",moreButtonGalleryControlLabel:"表示",moreButtonGalleryPositionToggleLabel:"ギャラリーを上に移動",moreButtonGallerySpeakerLayoutLabel:"スピーカー",moreButtonGalleryFloatingLocalLayoutLabel:"ダイナミック",moreButtonGalleryDefaultLayoutLabel:"ギャラリー ビュー",moreButtonGalleryFocusedContentLayoutLabel:"コンテンツにフォーカスを移動",moreButtonLargeGalleryDefaultLayoutLabel:"ラージ ギャラリー",moreButtonTogetherModeLayoutLabel:"Together モード",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"カメラが無効になっています。ビデオを共有できなくなりました。",grantedDueToMeetingOption:"カメラが有効になりました。ビデオを共有する場合はオンにします。"},unmuteMic:{lostDueToMeetingOption:"マイクが無効になっています。ミュートを解除できません。",grantedDueToMeetingOption:"マイクが有効になりました。音声機能を使用する場合は、ミュートを解除します。"},shareScreen:{lostDueToRoleChangeToAttendee:"ロールが変更されました。コンテンツの共有などの一部の操作は利用できません。",grantedDueToRoleChangeToPresenter:"あなたは発表者です。コンテンツの共有と会議の進行を行うことができます。"},hideAttendeeNames:{hideAttendeeNameAttendee:"自分の名前は他の出席者には表示されません。開催者と発表者にあなたの名前が表示されます。",hideAttendeeNamePresenter:"出席者の名前は非表示です。出席者のプライバシーを守るため、開催者と発表者のみ名前を見ることができます。"}},surveyTitle:"品質向上にご協力ください",starSurveyHelperText:"通話の音質はどうでしたか?",starSurveyOneStarText:"品質が悪かった。",starSurveyTwoStarText:"品質は低かった。",starSurveyThreeStarText:"品質は良かった。",starSurveyFourStarText:"品質は優れていた。",starSurveyFiveStarText:"品質は非常に優れていた。",surveyConfirmButtonLabel:"フィードバックの送信",starRatingAriaLabel:"{1} 個中 {0} 個の星を選択する",surveySkipButtonLabel:"スキップ",tagsSurveyQuestion:"何が良くなりましたか?",tagsSurveyTextFieldDefaultText:"その他、特定してください",tagsSurveyHelperText:"発生した問題を確認する",endOfSurveyText:"フィードバックをお送りいただき、ありがとうございます。",surveyIssues:{overallRating:{callCannotJoin:"通話に参加できなかった",callCannotInvite:"他のユーザーを通話に招待できなかった",hadToRejoin:"通話にもう参加し直す必要があった",callEndedUnexpectedly:"通話が突然終了した",otherIssues:"通話で他の問題が発生していた"},audioRating:{noLocalAudio:"相手側で音が聞こえませんでした",noRemoteAudio:"音がまったく聞こえなかった",echo:"通話でエコーが聞こえた",audioNoise:"通話中にノイズが聞こえた",lowVolume:"音量が小さかった",audioStoppedUnexpectedly:"オーディオが突然停止した",distortedSpeech:"オーディオが歪んでいた",audioInterruption:"オーディオが中断された",otherIssues:"この通話で他のオーディオの問題があった"},videoRating:{noVideoReceived:"ビデオを表示できなかった",noVideoSent:"他のユーザーに自分が表示されなかった",lowQuality:"ビデオの品質が低かった",freezes:"ビデオがフリーズした",stoppedUnexpectedly:"ビデオが突然停止した",darkVideoReceived:"他のユーザーがカメラをオンにした場合にのみ暗い画面を表示できる",audioVideoOutOfSync:"オーディオとビデオがずれている",otherIssues:"この通話で他のビデオの問題があった"},screenshareRating:{noContentLocal:"他のユーザーが自分の画面共有を表示できなかった",noContentRemote:"他のユーザーのスクリーン共有が表示されなかった",cannotPresent:"画面をプレゼンテーションできなかった",lowQuality:"画面共有の品質が低かった",freezes:"画面共有がフリーズした",stoppedUnexpectedly:"画面共有が突然停止した",largeDelay:"画面共有に大きな遅延がある",otherIssues:"この通話で他の画面共有の問題があった"}},surveyIssuesHeadingStrings:{overallRating:"全体",audioRating:"オーディオ",videoRating:"動画",screenshareRating:"発表中"},spotlightPrompt:{startSpotlightHeading:"全員にスポットライトを設定しますか?",startSpotlightText:"会議の全員にこのビデオをハイライトします。",startSpotlightOnSelfText:"会議の全員にビデオをハイライトします。",startSpotlightConfirmButtonLabel:"全員にスポットライトを設定",startSpotlightCancelButtonLabel:"キャンセル",stopSpotlightHeading:"このビデオへのスポットライトの設定を解除しますか?",stopSpotlightOnSelfHeading:"スポットライトを終了しますか?",stopAllSpotlightHeading:"すべてのビデオへのスポットライトの設定を解除しますか?",stopSpotlightText:"このビデオは、会議の全員に対して強調表示されなくなります。",stopSpotlightOnSelfText:"ビデオは、会議の全員に対して強調表示されなくなります。",stopAllSpotlightText:"ビデオは、会議の全員に対して強調表示されなくなります。",stopSpotlightConfirmButtonLabel:"スポットライトの設定を解除",stopSpotlightOnSelfConfirmButtonLabel:"スポットライトを終了",stopSpotlightCancelButtonLabel:"キャンセル",closeSpotlightPromptButtonLabel:"閉じる"},exitSpotlightButtonLabel:"スポットライトを終了",exitSpotlightButtonTooltip:"スポットライトを終了",leaveConfirmButtonLabel:"退出",endCallConfirmButtonLabel:"通話の終了",hangUpCancelButtonLabel:"キャンセル",leaveConfirmDialogTitle:"通話から退出しますか?",leaveConfirmDialogContent:"通話から退出してよろしいですか?",endCallConfirmDialogTitle:"通話を終了しますか?",endCallConfirmDialogContent:"全員に対する通話が終了されます。",invalidMeetingIdentifier:"無効なミーティング ID またはパスコード",pinParticipantMenuLabel:"自分用にピン留め",pinParticipantLimitReachedMenuLabel:"ピン留め (上限に達しました)",unpinParticipantMenuLabel:"ピン留めを外す",pinParticipantMenuItemAriaLabel:"自分用にピン留め",unpinParticipantMenuItemAriaLabel:"ピン留めを外す",muteAllMenuLabel:"全員をミュート",muteAllDialogTitle:"全員をミュートにしますか?",muteAllDialogContent:"会議に参加している自分以外の全員がミュートになります。",muteAllConfirmButtonLabel:"全員をミュート",muteAllCancelButtonLabel:"取り消す",joinBreakoutRoomButtonLabel:"ブレークアウト ルームに参加",returnFromBreakoutRoomButtonLabel:"ルームから退出",leaveBreakoutRoomAndMeetingButtonLabel:"会議から退出",breakoutRoomJoinedNotificationTitle:"ブレークアウト ルーム '{roomName}' に参加しました",joinBreakoutRoomBannerTitle:"ブレークアウト ルーム '{roomName}' が開いています",joinBreakoutRoomBannerButtonLabel:"参加",returnFromBreakoutRoomBannerTitle:"メインの会議に戻る",returnFromBreakoutRoomBannerButtonLabel:"もう一度参加",forbidAudioMenuLabel:"マイクを無効にする",permitAudioMenuLabel:"マイクを許可する",forbidOthersAudioDialogTitle:"出席者のマイクを無効にしますか?",forbidOthersAudioDialogContent:"出席者は自身のミュートを解除できません。開催者と発表者は、必要に応じて出席者のミュートを解除できます。",forbidOthersAudioConfirmButtonLabel:"マイクを無効にする",forbidOthersAudioCancelButtonLabel:"キャンセル",permitOthersAudioDialogTitle:"出席者のマイクを許可しますか?",permitOthersAudioDialogContent:"会議に参加している全員が、ミュートを解除できます。",permitOthersAudioConfirmButtonLabel:"マイクを許可する",permitOthersAudioCancelButtonLabel:"キャンセル",forbidOthersAudioMenuLabel:"出席者のマイクを無効にする",permitOthersAudioMenuLabel:"出席者のマイクを許可する",forbidVideoMenuLabel:"カメラを無効にする",permitVideoMenuLabel:"カメラを許可する",forbidOthersVideoDialogTitle:"出席者のカメラを無効にしますか?",forbidOthersVideoDialogContent:"出席者は自分でビデオをオンにすることはできません。開催者と発表者は、必要に応じてカメラをオンにすることができます。",forbidOthersVideoConfirmButtonLabel:"カメラを無効にする",forbidOthersVideoCancelButtonLabel:"キャンセル",permitOthersVideoDialogTitle:"出席者のカメラを許可しますか?",permitOthersVideoDialogContent:"会議の参加者全員がビデオを自分でオンにできます。",permitOthersVideoConfirmButtonLabel:"カメラを許可する",permitOthersVideoCancelButtonLabel:"キャンセル",forbidOthersVideoMenuLabel:"出席者のカメラを無効にする",permitOthersVideoMenuLabel:"出席者のカメラを許可する"};var chat$a={chatListHeader:"このチャット内",uploadAttachment:"添付ファイルのアップロード",uploadImageDataNotProvided:"画像データが提供されていません。",uploadImageIsTooLarge:"画像が大きすぎます。{maxImageSize} MB 未満のものを選択してください。",uploadImageExtensionIsNotAllowed:".{imageExtension} 形式の画像はアップロードできません。",uploadImageFailed:"画像をアップロードできません。後でもう一度お試しください。"};var callWithChat$a={chatButtonLabel:"チャット",chatButtonNewMessageNotificationLabel:"新規メッセージ",chatButtonTooltipClosedWithMessageCount:"チャットを表示する ({unreadMessagesCount} 件の未読)",chatButtonTooltipClose:"チャットを非表示",chatButtonTooltipOpen:"チャットを表示",chatPaneTitle:"チャット",copyInviteLinkButtonLabel:"招待用のリンクをコピー",copyInviteLinkButtonActionedLabel:"リンクをコピーしました",copyInviteLinkActionedAriaLabel:"招待用のリンクがコピーされました",dismissSidePaneButtonLabel:"閉じる",moreDrawerAudioDeviceMenuTitle:"オーディオ デバイス",moreDrawerButtonLabel:"その他のオプション",moreDrawerButtonTooltip:"その他のオプション",moreDrawerMicrophoneMenuTitle:"マイク",moreDrawerSpeakerMenuTitle:"スピーカー",moreDrawerCaptionsMenuTitle:"ライブ キャプション",moreDrawerSpokenLanguageMenuTitle:"音声言語",moreDrawerCaptionLanguageMenuTitle:"字幕言語",moreDrawerGalleryOptionsMenuTitle:"ギャラリーのオプション",peopleButtonLabel:"ユーザー",selectedPeopleButtonLabel:"[ユーザー] ボタンが選択されました",peopleButtonTooltipOpen:"参加者を表示",peopleButtonTooltipClose:"参加者を非表示",peopleButtonTooltipOpenAriaLabel:"[ユーザー] ボタン、参加者を表示",peopleButtonTooltipCloseAriaLabel:"[ユーザー] ボタン、参加者を非表示",peoplePaneSubTitle:"この通話で",peoplePaneTitle:"ユーザー",pictureInPictureTileAriaLabel:"ビデオ フィード。 クリックすると通話画面に戻ります。",removeMenuLabel:"削除",openDialpadButtonLabel:"電話番号をダイヤルする",returnToCallButtonAriaDescription:"通話に戻る",returnToCallButtonAriaLabel:"戻る",peoplePaneAddPeopleButtonLabel:"ユーザーの追加",dialpadStartCallButtonLabel:"通話",dialpadModalTitle:"電話番号をダイヤルする",dialpadModalAriaLabel:"ダイヤルパッド",dialpadCloseModalButtonAriaLabel:"ダイヤルパッドを閉じる",openDtmfDialpadLabel:"ダイヤルパッドを表示",dtmfDialpadPlaceholderText:"番号を入力してください",breakoutRoomChatPaneTitle:"ブレークアウト ルーム チャット",chatContentSpinnerLabel:"チャットを読み込んでいます"};var ja_JP = {call:call$a,chat:chat$a,callWithChat:callWithChat$a};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Japanese (Japan)
 *
 * @public
 */
const COMPOSITE_LOCALE_JA_JP = {
    component: COMPONENT_LOCALE_JA_JP,
    strings: createCompositeStrings(ja_JP)
};

var call$9={cameraLabel:"카메라",noCamerasLabel:"카메라를 찾을 수 없음",cameraPermissionDenied:"브라우저에서 카메라에 대한 액세스를 차단하고 있습니다.",cameraTurnedOff:"카메라가 꺼져 있음",chatButtonLabel:"채팅",close:"닫기",complianceBannerNowOnlyRecording:"이제 이 모임의 녹음/녹화만 진행합니다.",complianceBannerNowOnlyTranscription:"이제 이 모임의 전사만 진행합니다.",complianceBannerRecordingAndTranscriptionSaved:"녹음/녹화와 전사를 저장하는 중입니다.",complianceBannerRecordingAndTranscriptionStarted:"녹음/녹화와 전사를 시작했습니다.",complianceBannerRecordingAndTranscriptionStopped:"녹음/녹화와 전사를 중단했습니다.",complianceBannerRecordingSaving:"녹음/녹화를 저장하는 중입니다.",complianceBannerRecordingStarted:"녹음/녹화를 시작했습니다.",complianceBannerRecordingStopped:"녹음/녹화를 중단했습니다.",complianceBannerTranscriptionStarted:"전사를 시작했습니다.",complianceBannerTranscriptionConsent:"참가하면 이 모임이 전사되는 것에 동의하게 됩니다.",complianceBannerTranscriptionSaving:"전사를 저장하는 중입니다.",complianceBannerTranscriptionStopped:"전사를 중단했습니다.",configurationPageTitle:"통화 시작",copyInviteLinkButtonLabel:"초대 링크 복사",copyInviteLinkButtonActionedLabel:"링크 복사됨",copyInviteLinkActionedAriaLabel:"초대 링크를 복사함",defaultPlaceHolder:"옵션 선택",dismissSidePaneButtonLabel:"닫기",videoEffectsPaneTitle:"효과",videoEffectsPaneBackgroundSelectionTitle:"백그라운드",videoEffectsPaneAriaLabel:"동영상 효과 창",configurationPageCameraIsLoadingLabel:"비디오를 로드하는 중...",configurationPageVideoEffectsButtonLabel:"효과",unableToStartVideoEffect:"비디오 효과를 적용할 수 없습니다.",blurBackgroundEffectButtonLabel:"흐리기",blurBackgroundTooltip:"배경 흐리게",removeBackgroundEffectButtonLabel:"없음",removeBackgroundTooltip:"배경 제거",cameraOffBackgroundEffectWarningText:"카메라가 꺼져 있습니다. 비디오 효과를 보려면 카메라를 켜세요.",failedToJoinCallDueToNoNetworkMoreDetails:"네트워크 문제로 인해 통화 연결이 끊어졌습니다. 연결을 확인하고 다시 참가하세요.",failedToJoinCallDueToNoNetworkTitle:"통화 연결 끊김",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"통화에 참가할 수 있는 권한이 없습니다. 실수라면 통화에 다시 참가하세요.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"대기실에서 해제됐습니다.",learnMore:"자세한 정보",leavingCallTitle:"종료 중...",leftCallMoreDetails:"실수라면 통화에 다시 참가하세요.",leftCallTitle:"통화를 종료했습니다.",lobbyScreenConnectingToCallTitle:"통화에 참가",lobbyScreenWaitingToBeAdmittedTitle:"승인을 기다리는 중",microphonePermissionDenied:"브라우저에서 마이크에 대한 액세스를 차단하고 있습니다.",microphoneToggleInLobbyNotAllowed:"대기실에 있는 동안에는 음소거하거나 음소거를 해제할 수 없습니다.",mutedMessage:"음소거 되었습니다.",networkReconnectMoreDetails:"문제가 발생한 것 같습니다. 통화에 다시 연결하려고 합니다.",networkReconnectTitle:"잠시만",notInvitedToRoomDetails:"초대가 없으므로 이 채팅방에 참가할 수 없습니다.",notInvitedToRoomTitle:"소회의실에 초대되지 않음",peopleButtonLabel:"사용자",selectedPeopleButtonLabel:"사용자 단추 선택됨",peoplePaneTitle:"사용자",peoplePaneMoreButtonAriaLabel:"더 보기",peopleButtonTooltipOpen:"참가자 표시",peopleButtonTooltipClose:"참가자 숨기기",peoplePaneSubTitle:"이 통화에 {numberOfPeople}명",privacyPolicy:"개인 정보 취급 방침",rejoinCallButtonLabel:"통화 다시 참가",removedFromCallMoreDetails:"다른 참가자가 귀하를 통화에서 제거했습니다.",removedFromCallTitle:"통화에서 제거됐습니다.",removeMenuLabel:"제거",startSpotlightMenuLabel:"모두를 위한 스포트라이트",addSpotlightMenuLabel:"스포트라이트 추가",stopSpotlightMenuLabel:"스포트라이트 중지",stopSpotlightOnSelfMenuLabel:"스포트라이트 종료",spotlightLimitReachedMenuTitle:"스포트라이트 한도 도달",stopAllSpotlightMenuLabel:"모든 스포트라이트 중지",returnToCallButtonAriaDescription:"통화로 돌아가기",returnToCallButtonAriaLabel:"뒤로",roomNotFoundDetails:"제공된 소회의실 ID를 찾을 수 없습니다.",roomNotFoundTitle:"회의실을 찾을 수 없음",roomNotValidDetails:"현재 이 소회의실은 유효하지 않습니다.",roomNotValidTitle:"소회의실이 유효하지 않음",inviteToRoomRemovedDetails:"이 소회의실에 참가하기 위한 초대가 제거되었습니다.",inviteToRoomRemovedTitle:"소회의실 초대 제거됨",soundLabel:"소리",noMicrophonesLabel:"마이크를 찾을 수 없음",noSpeakersLabel:"스피커를 찾을 수 없음",startCallButtonLabel:"통화 시작",openDialpadButtonLabel:"전화 번호 전화 걸기",peoplePaneAddPeopleButtonLabel:"사용자 추가",dialpadStartCallButtonLabel:"통화",dialpadModalTitle:"전화 번호 전화 걸기",dialpadModalAriaLabel:"다이얼패드",dialpadCloseModalButtonAriaLabel:"다이얼패드 닫기",moreButtonCallingLabel:"기타",resumeCallButtonLabel:"계속",resumingCallButtonLabel:"다시 시작 중...",resumeCallButtonAriaLabel:"통화 다시 시작",resumingCallButtonAriaLabel:"통화 계속하기",holdScreenLabel:"대기 중입니다.",openDtmfDialpadLabel:"다이얼패드 표시",dtmfDialpadPlaceholderText:"번호 입력",outboundCallingNoticeString:"전화 거는 중...",participantJoinedNoticeString:"{displayName} 님이 참가",twoParticipantJoinedNoticeString:"{displayName1} 및 {displayName2} 님이 참가",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} 및 {displayName3} 님이 참가",participantLeftNoticeString:"{displayName} 님이 떠남",twoParticipantLeftNoticeString:"{displayName1} 및 {displayName2} 님이 떠남",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} 및 {displayName3} 님이 떠남",unnamedParticipantString:"명명되지 않은 참가자",manyUnnamedParticipantsJoined:"명명되지 않은 참가자 및 {numOfParticipants}명의 다른 참가자가 참가",manyUnnamedParticipantsLeft:"명명되지 않은 참가자 및 {numOfParticipants}명의 다른 참가자가 떠남",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} 및 {numOfParticipants} 다른 참가자가 참가했습니다.",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} 및 다른 참가자 {numOfParticipants} 남음",liveCaptionsLabel:"라이브 캡션",captionsSettingsLabel:"캡션 설정",startCaptionsButtonOnLabel:"캡션 켜기",startCaptionsButtonOffLabel:"캡션 끄기",startCaptionsButtonTooltipOnContent:"캡션 끄기",startCaptionsButtonTooltipOffContent:"캡션 켜기",captionsSettingsModalTitle:"캡션 설정",captionsSettingsSpokenLanguageDropdownLabel:"음성 언어",captionsSettingsCaptionLanguageDropdownLabel:"캡션 언어",captionsSettingsSpokenLanguageDropdownInfoText:"이 통화의 모든 사람이 말하는 언어입니다.",captionsSettingsCaptionLanguageDropdownInfoText:"캡션은 이 언어로 표시됩니다.",captionsSettingsConfirmButtonLabel:"확인하기",captionsSettingsCancelButtonLabel:"취소하기",captionsSettingsModalAriaLabel:"캡션 설정 모달",captionsSettingsCloseModalButtonAriaLabel:"캡션 설정 닫기",captionsBannerMoreButtonCallingLabel:"캡션 설정 추가 옵션",captionsBannerMoreButtonTooltip:"기타 옵션",realTimeTextLabel:"실시간 텍스트(RTT)",startRealTimeTextLabel:"이 통화에 대한 RTT 켜기",disabledStartRealTimeTextLabel:"RTT가 현재 켜져 있으므로 끌 수 없습니다.",dismissModalAriaLabel:"로컬 및 원격 사진 및 그림, Enter 키를 눌러 통화로 돌아가기",callRejectedTitle:"통화를 완료할 수 없습니다.",callRejectedMoreDetails:"연결하려는 사용자와 지금 대화할 수 없습니다. 나중에 다시 시도하세요.",callTimeoutTitle:"통화를 완료할 수 없습니다.",callTimeoutDetails:"연결하려는 사용자와 지금 대화할 수 없습니다. 나중에 다시 시도하세요.",callTimeoutBotTitle:"통화를 완료할 수 없습니다.",callTimeoutBotDetails:"연결하려는 서비스를 사용할 수 없습니다. 나중에 다시 시도하세요.",dtmfDialerButtonLabel:"다이얼패드",dtmfDialerButtonTooltipOn:"다이얼패드 표시",dtmfDialerButtonTooltipOff:"다이얼패드 숨기기",dtmfDialerMoreButtonLabelOn:"다이얼패드 표시",dtmfDialerMoreButtonLabelOff:"다이얼패드 숨기기",phoneCallMoreButtonLabel:"전화 통화",spokenLanguageStrings:{"ar-ae":"아랍어 - 아랍에미리트","ar-sa":"아라비아어 - 사우디 아라비아","da-dk":"덴마크어","de-de":"독일어 - 독일","en-au":"영어 - 오스트레일리아","en-ca":"영어 - 캐나다","en-gb":"영어 - 영국","en-in":"영어 - 인도","en-nz":"영어 - 뉴질랜드","en-us":"영어 - 미국","es-es":"스페인어 - 스페인(현대 정렬)","es-mx":"스페인어 - 멕시코","fi-fi":"핀란드어","fr-ca":"프랑스어 - 캐나다","fr-fr":"프랑스어 - 프랑스","hi-in":"힌디어","it-it":"이탈리아어 - 이탈리아","ja-jp":"일본어","ko-kr":"한국어","nb-no":"노르웨이어(복말)","nl-be":"네덜란드어 - 벨기에","nl-nl":"네덜란드어 - 네덜란드","pl-pl":"폴란드어","pt-br":"포르투갈어 - 브라질","ru-ru":"러시아어","sv-se":"스웨덴어","zh-cn":"중국어 - 중국","zh-hk":"중국어 - 홍콩 특별 행정구","cs-cz":"체코어","pt-pt":"포르투갈어 - 포르투갈","tr-tr":"터키어","vi-vn":"베트남어","th-th":"태국어","he-il":"히브리어","cy-gb":"웨일스어","uk-ua":"우크라이나어","el-gr":"그리스어","hu-hu":"헝가리어","ro-ro":"루마니아어","sk-sk":"슬로바키아어","zh-tw":"중국어 - 대만"},captionLanguageStrings:{ar:"아랍어",da:"덴마크어",de:"독일어",en:"영어",es:"스페인어",fi:"핀란드어","fr-ca":"프랑스어 - 캐나다",fr:"프랑스어 - 프랑스",hi:"힌디어",it:"이탈리아어",ja:"일본어",ko:"한국어",nb:"노르웨이어(복말)",nl:"네덜란드어",pl:"폴란드어",pt:"포르투갈어 - 브라질",ru:"러시아어",sv:"스웨덴어","zh-Hans":"중국어(간체)","zh-Hant":"중국어(번체)",cs:"체코어","pt-pt":"포르투갈어 - 포르투갈",tr:"터키어",vi:"베트남어",th:"태국어",he:"히브리어",cy:"웨일스어",uk:"우크라이나어",el:"그리스어",hu:"헝가리어",ro:"루마니아어",sk:"슬로바키아어"},captionsBannerSpinnerText:"캡션을 시작하는 중...",realTimeTextInputBoxDefaultText:"실시간으로 메시지 입력",realTimeTextInputErrorMessage:"메시지는 2000자를 초과할 수 없음",realTimeTextBannerTitle:"실시간 입력",realTimeTextBannerContent:"전체 모임 기간 동안 모든 참가자에 대해 RTT(실시간 텍스트)를 사용할 수 있습니다.",realTimeTextBannerLinkLabel:"자세한 정보",realTimeTextModalAriaLabel:"실시간 입력 모달 사용",realTimeTextModalTitle:"실시간 입력을 사용하시겠습니까?",realTimeTextModalText:"모임의 모든 사용자에 대해 RTT(실시간 텍스트)가 켜집니다.",realTimeTextConfirmButtonLabel:"사용",realTimeTextCancelButtonLabel:"취소",realTimeTextCloseModalButtonAriaLabel:"실시간 입력 모달 닫기",captionsOnlyContainerTitle:"캡션",realTimeTextOnlyContainerTitle:"실시간 텍스트",captionsAndRealTimeTextContainerTitle:"캡션 및 실시간 텍스트",expandButtonAriaLabel:"펼치기",minimizeButtonAriaLabel:"접기",transferPageTransferorText:"전송 중...",transferPageTransferTargetText:"연결 중...",transferPageUnknownTransferorDisplayName:"알 수 없음",transferPageUnknownTransferTargetDisplayName:"알 수 없음",transferPageNoticeString:"통화가 전달되고 있습니다.",participantCouldNotBeReachedTitle:"대상 참가자를 현재 사용할 수 없습니다.",participantCouldNotBeReachedMoreDetails:"참가자를 사용할 수 있는 경우 참가자에게 문의하세요.",permissionToReachTargetParticipantNotAllowedTitle:"대상 참가자에 연결할 수 있는 권한이 허용되지 않습니다.",permissionToReachTargetParticipantNotAllowedMoreDetails:"대상 참가자가 동일한 테넌트에 있는지 검사.",unableToResolveTenantTitle:"대상 참가자의 테넌트 ID를 resolve 수 없습니다.",unableToResolveTenantMoreDetails:"참가자 ID가 올바르게 입력되었는지 검사.",participantIdIsMalformedTitle:"참가자 ID의 형식이 잘못되었습니다.",participantIdIsMalformedMoreDetails:"참가자 ID가 올바른 형식인지 검사.",moreButtonGalleryControlLabel:"보기",moreButtonGalleryPositionToggleLabel:"갤러리를 맨 위로 이동",moreButtonGallerySpeakerLayoutLabel:"발표자",moreButtonGalleryFloatingLocalLayoutLabel:"동적",moreButtonGalleryDefaultLayoutLabel:"갤러리 보기",moreButtonGalleryFocusedContentLayoutLabel:"콘텐츠에 포커스 맞추기",moreButtonLargeGalleryDefaultLayoutLabel:"대형 갤러리",moreButtonTogetherModeLayoutLabel:"함께 모드",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"카메라를 사용할 수 없습니다. 더 이상 비디오를 공유할 수 없습니다.",grantedDueToMeetingOption:"카메라를 사용하도록 설정했습니다. 비디오를 공유하려면 켜세요."},unmuteMic:{lostDueToMeetingOption:"마이크를 사용할 수 없습니다. 더 이상 음소거를 해제할 수 없습니다.",grantedDueToMeetingOption:"마이크 사용 가능합니다. 말하려면 음소거를 해제하세요."},shareScreen:{lostDueToRoleChangeToAttendee:"역할이 변경되었습니다. 콘텐츠 공유와 같은 일부 작업은 사용할 수 없습니다.",grantedDueToRoleChangeToPresenter:"귀하는 발표자입니다. 콘텐츠를 공유하고 모임을 활성화할 수 있습니다."},hideAttendeeNames:{hideAttendeeNameAttendee:"다른 참석자가 내 이름을 볼 수 없도록 숨겨집니다. 주최자와 발표자는 내 실명을 볼 수 있습니다.",hideAttendeeNamePresenter:"참석자 이름이 숨겨집니다. 참석자 개인 정보를 보호하기 위해 주최자와 발표자만 이름을 볼 수 있습니다."}},surveyTitle:"사용자 환경 개선",starSurveyHelperText:"통화 품질은 어떠했습니까?",starSurveyOneStarText:"품질이 매우 나쁩니다.",starSurveyTwoStarText:"품질이 나쁩니다.",starSurveyThreeStarText:"품질이 좋습니다.",starSurveyFourStarText:"품질이 매우 좋습니다.",starSurveyFiveStarText:"품질이 매우 좋았습니다.",surveyConfirmButtonLabel:"피드백 보내기",starRatingAriaLabel:"별 {0}/{1}개 선택",surveySkipButtonLabel:"건너뛰기",tagsSurveyQuestion:"개선할 사항을 알려주세요.",tagsSurveyTextFieldDefaultText:"기타(구체적으로)",tagsSurveyHelperText:"발생한 문제를 확인하세요.",endOfSurveyText:"피드백을 보내주셔서 감사합니다!",surveyIssues:{overallRating:{callCannotJoin:"통화에 참가할 수 없습니다.",callCannotInvite:"다른 사람을 통화에 초대할 수 없습니다.",hadToRejoin:"통화에 다시 참가해야 했습니다.",callEndedUnexpectedly:"예기치 않게 통화가 종료되었습니다.",otherIssues:"통화에 다른 문제가 발생했습니다."},audioRating:{noLocalAudio:"상대방이 아무 소리도 듣지 못합니다",noRemoteAudio:"아무 소리도 들리지 않습니다.",echo:"통화에서 에코가 들립니다.",audioNoise:"통화 중에 잡음이 들렸습니다.",lowVolume:"볼륨이 작습니다",audioStoppedUnexpectedly:"오디오가 예기치 않게 중지됨",distortedSpeech:"오디오가 왜곡되었습니다.",audioInterruption:"오디오가 중단되었습니다.",otherIssues:"이 통화에서 다른 오디오 문제가 발생했습니다."},videoRating:{noVideoReceived:"영상이 표시되지 않았습니다.",noVideoSent:"다른 사용자가 나를 볼 수 없습니다.",lowQuality:"비디오 품질이 낮음",freezes:"비디오 동결",stoppedUnexpectedly:"영상이 중간에 멈췄습니다.",darkVideoReceived:"다른 사람이 카메라를 켤 때만 어두운 화면을 볼 수 있습니다.",audioVideoOutOfSync:"오디오 및 비디오가 동기화되지 않았습니다.",otherIssues:"이 통화에서 다른 비디오 문제가 발생했습니다."},screenshareRating:{noContentLocal:"다른 사용자가 내 화면 공유를 볼 수 없습니다.",noContentRemote:"다른 사람의 화면 공유를 볼 수 없습니다.",cannotPresent:"내 화면을 표시할 수 없습니다.",lowQuality:"화면 공유 품질이 낮음",freezes:"화면 공유가 고정되었습니다.",stoppedUnexpectedly:"화면 공유가 예기치 않게 중지됨",largeDelay:"화면 공유에 큰 지연이 있음",otherIssues:"이 통화에서 다른 화면 공유 문제가 있었습니다."}},surveyIssuesHeadingStrings:{overallRating:"전체",audioRating:"오디오",videoRating:"비디오",screenshareRating:"프레젠테이션 중"},spotlightPrompt:{startSpotlightHeading:"모든 사용자에게 스포트라이트하시겠습니까?",startSpotlightText:"모임에 참석한 모든 사용자에 대해 이 비디오를 강조합니다.",startSpotlightOnSelfText:"모임에 참석한 모든 사용자에 대해 비디오를 강조합니다.",startSpotlightConfirmButtonLabel:"모든 사용자에게 스포트라이트",startSpotlightCancelButtonLabel:"취소",stopSpotlightHeading:"이 비디오 스포트라이트를 중지할까요?",stopSpotlightOnSelfHeading:"스포트라이트를 종료하시겠습니까?",stopAllSpotlightHeading:"모든 비디오 스포트라이트를 중지할까요?",stopSpotlightText:"모임에 참석한 모든 사용자에게 이 비디오가 더 이상 강조되지 않습니다.",stopSpotlightOnSelfText:"모임에 참석한 모든 사용자에 대해 비디오가 더 이상 강조되지 않습니다.",stopAllSpotlightText:"해당 비디오는 모임에 참석한 모든 사용자에 대해 더 이상 강조되지 않습니다.",stopSpotlightConfirmButtonLabel:"스포트라이트 중지",stopSpotlightOnSelfConfirmButtonLabel:"스포트라이트 종료",stopSpotlightCancelButtonLabel:"취소",closeSpotlightPromptButtonLabel:"닫기"},exitSpotlightButtonLabel:"스포트라이트 종료",exitSpotlightButtonTooltip:"스포트라이트 종료",leaveConfirmButtonLabel:"나가기",endCallConfirmButtonLabel:"통화 종료",hangUpCancelButtonLabel:"취소",leaveConfirmDialogTitle:"통화에서 나가시겠습니까?",leaveConfirmDialogContent:"통화에서 나가시겠습니까?",endCallConfirmDialogTitle:"통화를 종료하시겠습니까?",endCallConfirmDialogContent:"모든 사용자의 통화가 종료됩니다.",invalidMeetingIdentifier:"유효하지 않은 모임 ID 또는 암호",pinParticipantMenuLabel:"고정",pinParticipantLimitReachedMenuLabel:"고정(제한에 도달함)",unpinParticipantMenuLabel:"고정 해제",pinParticipantMenuItemAriaLabel:"고정",unpinParticipantMenuItemAriaLabel:"고정 해제",muteAllMenuLabel:"모두 음소거",muteAllDialogTitle:"모든 사용자를 음소거하시겠습니까?",muteAllDialogContent:"나를 제외한 모임 참여자를 모두 음소거합니다.",muteAllConfirmButtonLabel:"모두 음소거",muteAllCancelButtonLabel:"취소",joinBreakoutRoomButtonLabel:"소규모 회의실 참가",returnFromBreakoutRoomButtonLabel:"회의실 나가기",leaveBreakoutRoomAndMeetingButtonLabel:"모임 나가기",breakoutRoomJoinedNotificationTitle:"소규모 회의실 '{roomName}'에 참가했습니다.",joinBreakoutRoomBannerTitle:"소규모 회의실 '{roomName}'이(가) 열려 있습니다.",joinBreakoutRoomBannerButtonLabel:"참가",returnFromBreakoutRoomBannerTitle:"주 모임으로 돌아가기",returnFromBreakoutRoomBannerButtonLabel:"다시 참가",forbidAudioMenuLabel:"마이크 비활성화",permitAudioMenuLabel:"마이크 허용",forbidOthersAudioDialogTitle:"참석자의 마이크를 비활성화하시겠어요?",forbidOthersAudioDialogContent:"참석자 본인이 음소거를 해제할 수 없습니다. 모임의 이끌이와 발표자는 필요에 따라 사람들이 음소거를 해제하도록 설정할 수 있습니다.",forbidOthersAudioConfirmButtonLabel:"마이크를 비활성화",forbidOthersAudioCancelButtonLabel:"취소",permitOthersAudioDialogTitle:"참석자의 마이크 사용을 허용하시겠어요?",permitOthersAudioDialogContent:"모임의 모든 사용자가 직접 음소거를 해제할 수 있습니다.",permitOthersAudioConfirmButtonLabel:"마이크 허용",permitOthersAudioCancelButtonLabel:"취소",forbidOthersAudioMenuLabel:"참석자의 마이크 비활성화",permitOthersAudioMenuLabel:"참석자의 마이크 허용",forbidVideoMenuLabel:"카메라 비활성화",permitVideoMenuLabel:"카메라 허용",forbidOthersVideoDialogTitle:"참석자의 카메라 사용을 비활성화하시겠어요?",forbidOthersVideoDialogContent:"참석자는 비디오를 직접 켤 수 없습니다. 이끌이와 발표자가 필요에 따라 카메라를 켜도록 할 수 있습니다.",forbidOthersVideoConfirmButtonLabel:"카메라 비활성화",forbidOthersVideoCancelButtonLabel:"취소",permitOthersVideoDialogTitle:"참석자의 카메라 사용을 허용하시겠어요?",permitOthersVideoDialogContent:"모임의 모든 사용자가 비디오를 직접 켤 수 있습니다.",permitOthersVideoConfirmButtonLabel:"카메라 허용",permitOthersVideoCancelButtonLabel:"취소",forbidOthersVideoMenuLabel:"참석자의 카메라 비활성화",permitOthersVideoMenuLabel:"참석자의 카메라 허용"};var chat$9={chatListHeader:"채팅 참가자",uploadAttachment:"첨부 파일 업로드",uploadImageDataNotProvided:"이미지 데이터가 제공되지 않았습니다.",uploadImageIsTooLarge:"이미지가 너무 큽 수 있습니다. {maxImageSize} 미만인 항목을 선택하세요.",uploadImageExtensionIsNotAllowed:".{imageExtension} 이미지 업로드는 허용되지 않습니다.",uploadImageFailed:"이미지를 업로드할 수 없습니다. 나중에 다시 시도하세요."};var callWithChat$9={chatButtonLabel:"채팅",chatButtonNewMessageNotificationLabel:"새 메시지",chatButtonTooltipClosedWithMessageCount:"채팅 표시({unreadMessagesCount} 읽지 않음)",chatButtonTooltipClose:"채팅 숨기기",chatButtonTooltipOpen:"채팅 표시",chatPaneTitle:"채팅",copyInviteLinkButtonLabel:"초대 링크 복사",copyInviteLinkButtonActionedLabel:"링크 복사됨",copyInviteLinkActionedAriaLabel:"초대 링크를 복사함",dismissSidePaneButtonLabel:"닫기",moreDrawerAudioDeviceMenuTitle:"오디오 장치",moreDrawerButtonLabel:"기타 옵션",moreDrawerButtonTooltip:"기타 옵션",moreDrawerMicrophoneMenuTitle:"마이크",moreDrawerSpeakerMenuTitle:"발화자",moreDrawerCaptionsMenuTitle:"라이브 캡션",moreDrawerSpokenLanguageMenuTitle:"음성 언어",moreDrawerCaptionLanguageMenuTitle:"캡션 언어",moreDrawerGalleryOptionsMenuTitle:"갤러리 옵션",peopleButtonLabel:"사용자",selectedPeopleButtonLabel:"사용자 단추 선택됨",peopleButtonTooltipOpen:"참가자 표시",peopleButtonTooltipClose:"참가자 숨기기",peopleButtonTooltipOpenAriaLabel:"사용자 단추, 참가자 표시",peopleButtonTooltipCloseAriaLabel:"사용자 단추, 참가자 숨기기",peoplePaneSubTitle:"이 통화에서",peoplePaneTitle:"사용자",pictureInPictureTileAriaLabel:"비디오 피드입니다. 클릭하면 통화 화면으로 돌아갑니다.",removeMenuLabel:"제거",openDialpadButtonLabel:"전화 번호 전화 걸기",returnToCallButtonAriaDescription:"통화로 돌아가기",returnToCallButtonAriaLabel:"뒤로",peoplePaneAddPeopleButtonLabel:"사용자 추가",dialpadStartCallButtonLabel:"통화",dialpadModalTitle:"전화 번호 전화 걸기",dialpadModalAriaLabel:"다이얼패드",dialpadCloseModalButtonAriaLabel:"다이얼패드 닫기",openDtmfDialpadLabel:"다이얼패드 표시",dtmfDialpadPlaceholderText:"번호 입력",breakoutRoomChatPaneTitle:"소규모 회의실 채팅",chatContentSpinnerLabel:"채팅 로드 중"};var ko_KR = {call:call$9,chat:chat$9,callWithChat:callWithChat$9};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Korean (South Korea)
 *
 * @public
 */
const COMPOSITE_LOCALE_KO_KR = {
    component: COMPONENT_LOCALE_KO_KR,
    strings: createCompositeStrings(ko_KR)
};

var call$8={cameraLabel:"Kamera",noCamerasLabel:"Fant ingen kameraer",cameraPermissionDenied:"Nettleseren blokkerer tilgang til kameraet",cameraTurnedOff:"Kameraet er slått av",chatButtonLabel:"Chat",close:"Lukk",complianceBannerNowOnlyRecording:"Du tar nå bare opp dette møtet.",complianceBannerNowOnlyTranscription:"Du transkriberer nå bare dette møtet.",complianceBannerRecordingAndTranscriptionSaved:"Opptak og transkripsjon lagres.",complianceBannerRecordingAndTranscriptionStarted:"Opptak og transkripsjon har startet.",complianceBannerRecordingAndTranscriptionStopped:"Innspilling og transkripsjon har stoppet.",complianceBannerRecordingSaving:"Opptaket blir lagret.",complianceBannerRecordingStarted:"Innspillingen er startet.",complianceBannerRecordingStopped:"Opptaket har stoppet.",complianceBannerTranscriptionStarted:"Transkripsjon har startet.",complianceBannerTranscriptionConsent:"Ved å bli med gir du samtykke til at dette møtet kan transkriberes.",complianceBannerTranscriptionSaving:"Transkripsjonen blir lagret.",complianceBannerTranscriptionStopped:"Transkripsjon er stoppet.",configurationPageTitle:"Start en samtale",copyInviteLinkButtonLabel:"Kopier invitasjonskobling",copyInviteLinkButtonActionedLabel:"Koblingen er kopiert",copyInviteLinkActionedAriaLabel:"Invitasjonskobling er kopiert",defaultPlaceHolder:"Velg et alternativ",dismissSidePaneButtonLabel:"Lukk",videoEffectsPaneTitle:"Effekter",videoEffectsPaneBackgroundSelectionTitle:"Bakgrunn",videoEffectsPaneAriaLabel:"Videoeffekter-ruten",configurationPageCameraIsLoadingLabel:"Videoen lastes inn ...",configurationPageVideoEffectsButtonLabel:"Effekter",unableToStartVideoEffect:"Kan ikke bruke videoeffekt.",blurBackgroundEffectButtonLabel:"Uskarp bakgrunn",blurBackgroundTooltip:"Gjør bakgrunnen uskarp",removeBackgroundEffectButtonLabel:"Ingen",removeBackgroundTooltip:"Fjern bakgrunnen",cameraOffBackgroundEffectWarningText:"Kameraet er av. Slå på kameraet for å se videoeffekten.",failedToJoinCallDueToNoNetworkMoreDetails:"Samtalen ble frakoblet på grunn av et nettverksproblem. Kontroller tilkoblingen og bli med på nytt.",failedToJoinCallDueToNoNetworkTitle:"Samtale frakoblet",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Du ble ikke gitt adgang i samtalen. Hvis dette var en feil, må du bli med i samtalen på nytt.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Avvist fra lobbyen",learnMore:"Mer informasjon",leavingCallTitle:"Forlater...",leftCallMoreDetails:"Hvis dette var en feil, må du bli med i samtalen på nytt.",leftCallTitle:"Du forlot samtalen",lobbyScreenConnectingToCallTitle:"Blir med i samtalen",lobbyScreenWaitingToBeAdmittedTitle:"Venter på å bli sluppet inn",microphonePermissionDenied:"Nettleseren blokkerer tilgang til mikrofonen",microphoneToggleInLobbyNotAllowed:"Kan ikke dempe eller oppheve demping i lobbyen.",mutedMessage:"Du er dempet.",networkReconnectMoreDetails:"Det ser ut til at noe gikk galt. Vi prøver å gå tilbake til samtalen.",networkReconnectTitle:"Vent litt",notInvitedToRoomDetails:"Du kan ikke bli med i dette rommet fordi du ikke har en invitasjon.",notInvitedToRoomTitle:"Ikke invitert til rommet",peopleButtonLabel:"Personer",selectedPeopleButtonLabel:"Personer-knappen er valgt",peoplePaneTitle:"Personer",peoplePaneMoreButtonAriaLabel:"Mer",peopleButtonTooltipOpen:"Vis deltakere",peopleButtonTooltipClose:"Skjul deltakere",peoplePaneSubTitle:"I denne samtalen {numberOfPeople}",privacyPolicy:"Personvernerklæring",rejoinCallButtonLabel:"Bli med i samtale på nytt",removedFromCallMoreDetails:"En annen deltaker fjernet deg fra samtalen.",removedFromCallTitle:"Du ble fjernet",removeMenuLabel:"Fjern",startSpotlightMenuLabel:"Sett i rampelyset for alle",addSpotlightMenuLabel:"Legg til rampelys",stopSpotlightMenuLabel:"Fjern fra rampelyset",stopSpotlightOnSelfMenuLabel:"Rampelys av",spotlightLimitReachedMenuTitle:"Rampelys-grensen er nådd",stopAllSpotlightMenuLabel:"Stopp rampelyset for alle",returnToCallButtonAriaDescription:"Gå tilbake til samtale",returnToCallButtonAriaLabel:"Tilbake",roomNotFoundDetails:"Finner ikke den angitte rom-IDen.",roomNotFoundTitle:"Finner ikke rommet",roomNotValidDetails:"Dette rommet er ikke gyldig for øyeblikket.",roomNotValidTitle:"Ugyldig rom",inviteToRoomRemovedDetails:"Invitasjonen til å bli med i dette rommet ble fjernet.",inviteToRoomRemovedTitle:"Invitasjon til rommet er fjernet",soundLabel:"Lyd",noMicrophonesLabel:"Fant ingen mikrofoner",noSpeakersLabel:"Fant ingen høyttalere",startCallButtonLabel:"Start samtale",openDialpadButtonLabel:"Slå telefonnummeret",peoplePaneAddPeopleButtonLabel:"Legg til personer",dialpadStartCallButtonLabel:"Ringe",dialpadModalTitle:"Slå telefonnummeret",dialpadModalAriaLabel:"Numerisk tastatur",dialpadCloseModalButtonAriaLabel:"Lukk numerisk tastatur",moreButtonCallingLabel:"Mer",resumeCallButtonLabel:"Fortsett",resumingCallButtonLabel:"Gjenopptar …",resumeCallButtonAriaLabel:"Fortsett samtale",resumingCallButtonAriaLabel:"Fortsett samtale",holdScreenLabel:"Du er satt på vent",openDtmfDialpadLabel:"Vis numerisk tastatur",dtmfDialpadPlaceholderText:"Skriv inn nummer",outboundCallingNoticeString:"Ringer …",participantJoinedNoticeString:"{displayName} ble med",twoParticipantJoinedNoticeString:"{displayName1} og {displayName2} har blitt med",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} og {displayName3} har blitt med",participantLeftNoticeString:"{displayName} igjen",twoParticipantLeftNoticeString:"{displayName1} og {displayName2} har forlatt",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} og {displayName3} har forlatt",unnamedParticipantString:"deltaker uten navn",manyUnnamedParticipantsJoined:"deltaker uten navn og {numOfParticipants} andre deltakere ble med",manyUnnamedParticipantsLeft:"deltaker uten navn og {numOfParticipants} andre deltakere igjen",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} og {numOfParticipants} andre deltakere ble med",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} og {numOfParticipants} andre deltakere igjen",liveCaptionsLabel:"Undertekster",captionsSettingsLabel:"Innstillinger for undertekster",startCaptionsButtonOnLabel:"Slå på undertekster",startCaptionsButtonOffLabel:"Deaktiver undertekster",startCaptionsButtonTooltipOnContent:"Deaktiver undertekster",startCaptionsButtonTooltipOffContent:"Slå på undertekster",captionsSettingsModalTitle:"Tekstinnstillinger",captionsSettingsSpokenLanguageDropdownLabel:"Talespråk",captionsSettingsCaptionLanguageDropdownLabel:"Undertekstspråk",captionsSettingsSpokenLanguageDropdownInfoText:"Språket som alle i denne samtalen snakker.",captionsSettingsCaptionLanguageDropdownInfoText:"Undertekster vises på dette språket.",captionsSettingsConfirmButtonLabel:"Bekreft",captionsSettingsCancelButtonLabel:"Avbryt",captionsSettingsModalAriaLabel:"Sperre for innstillinger for undertekster",captionsSettingsCloseModalButtonAriaLabel:"Lukk innstilling for undertekster",captionsBannerMoreButtonCallingLabel:"Flere alternativer for innstillinger for teksting",captionsBannerMoreButtonTooltip:"Flere alternativer",realTimeTextLabel:"Skriving i sanntid (RTT)",startRealTimeTextLabel:"Aktiver RTT for denne samtalen",disabledStartRealTimeTextLabel:"RTT er slått på for øyeblikket, den kan ikke deaktiveres",dismissModalAriaLabel:"Lokalt og eksternt bilde og bilde, trykk på Enter for å gå tilbake til samtalen",callRejectedTitle:"Kan ikke fullføre samtalen",callRejectedMoreDetails:"Personen du prøver å nå, er ikke tilgjengelig. Prøv på nytt senere.",callTimeoutTitle:"Kan ikke fullføre samtalen",callTimeoutDetails:"Personen du prøver å nå, er ikke tilgjengelig. Prøv på nytt senere.",callTimeoutBotTitle:"Kan ikke fullføre samtalen",callTimeoutBotDetails:"Tjenesten du prøver å nå, er ikke tilgjengelig. Prøv på nytt senere.",dtmfDialerButtonLabel:"Numerisk tastatur",dtmfDialerButtonTooltipOn:"Vis numerisk tastatur",dtmfDialerButtonTooltipOff:"Skjul numerisk tastatur",dtmfDialerMoreButtonLabelOn:"Vis numerisk tastatur",dtmfDialerMoreButtonLabelOff:"Skjul numerisk tastatur",phoneCallMoreButtonLabel:"Telefonsamtale",spokenLanguageStrings:{"ar-ae":"Arabisk – De forente arabiske emirater","ar-sa":"Arabisk – Saudi-Arabia","da-dk":"Dansk","de-de":"Tysk – Tyskland","en-au":"Engelsk – Australia","en-ca":"Engelsk – Canada","en-gb":"Engelsk – Storbritannia","en-in":"Engelsk – India","en-nz":"Engelsk – New Zealand","en-us":"Engelsk – USA","es-es":"Spansk – Spania (moderne)","es-mx":"Spansk – Mexico","fi-fi":"Finsk","fr-ca":"Fransk – Canada","fr-fr":"Fransk – Frankrike","hi-in":"Hindi","it-it":"Italiensk – Italia","ja-jp":"Japansk","ko-kr":"Koreansk","nb-no":"Norsk (bokmål)","nl-be":"Nederlandsk – Belgia","nl-nl":"Nederlandsk – Nederland","pl-pl":"Polsk","pt-br":"Portugisisk – Brasil","ru-ru":"Russisk","sv-se":"Svensk","zh-cn":"Kinesisk – Folkerepublikken Kina","zh-hk":"Kinesisk – Hongkong SAR","cs-cz":"Tsjekkisk","pt-pt":"Portugisisk – Portugal","tr-tr":"Tyrkisk","vi-vn":"Vietnamesisk","th-th":"Thai","he-il":"Hebraisk","cy-gb":"Walisisk","uk-ua":"Ukrainsk","el-gr":"Gresk","hu-hu":"Ungarsk","ro-ro":"Rumensk","sk-sk":"Slovakisk","zh-tw":"Kinesisk – Taiwan"},captionLanguageStrings:{ar:"Arabisk",da:"Dansk",de:"Tysk",en:"Engelsk",es:"Spansk",fi:"Finsk","fr-ca":"Fransk – Canada",fr:"Fransk – Frankrike",hi:"Hindi",it:"Italiensk",ja:"Japansk",ko:"Koreansk",nb:"Norsk (bokmål)",nl:"Nederlandsk",pl:"Polsk",pt:"Portugisisk – Brasil",ru:"Russisk",sv:"Svensk","zh-Hans":"Kinesisk (forenklet)","zh-Hant":"Kinesisk (tradisjonell)",cs:"Tsjekkisk","pt-pt":"Portugisisk – Portugal",tr:"Tyrkisk",vi:"Vietnamesisk",th:"Thai",he:"Hebraisk",cy:"Walisisk",uk:"Ukrainsk",el:"Gresk",hu:"Ungarsk",ro:"Rumensk",sk:"Slovakisk"},captionsBannerSpinnerText:"Starter teksting...",realTimeTextInputBoxDefaultText:"Skriv inn melding i sanntid",realTimeTextInputErrorMessage:"Meldingen kan ikke overskride 2000 tegn",realTimeTextBannerTitle:"Skriving i sanntid",realTimeTextBannerContent:"RTT (sanntidstekst) er aktivert for alle deltakere hele varigheten for møtet.",realTimeTextBannerLinkLabel:"Mer informasjon",realTimeTextModalAriaLabel:"Aktiver sperrende dialogboks for skriving i sanntid",realTimeTextModalTitle:"Aktiver skriving i sanntid?",realTimeTextModalText:"Sanntidstekst (RTT) aktiveres for alle i møtet.",realTimeTextConfirmButtonLabel:"Aktiver",realTimeTextCancelButtonLabel:"Avbryt",realTimeTextCloseModalButtonAriaLabel:"Lukk sperrende dialogboks for skriving i sanntid",captionsOnlyContainerTitle:"Undertekster",realTimeTextOnlyContainerTitle:"Tekst i sanntid",captionsAndRealTimeTextContainerTitle:"Teksting og sanntidstekst",expandButtonAriaLabel:"Utvid",minimizeButtonAriaLabel:"Skjul",transferPageTransferorText:"Overfører...",transferPageTransferTargetText:"Kobler til...",transferPageUnknownTransferorDisplayName:"Ukjent",transferPageUnknownTransferTargetDisplayName:"Ukjent",transferPageNoticeString:"Anropet blir overført",participantCouldNotBeReachedTitle:"Måldeltaker er for øyeblikket ikke tilgjengelig",participantCouldNotBeReachedMoreDetails:"Kontakt deltakeren når vedkommende er tilgjengelig",permissionToReachTargetParticipantNotAllowedTitle:"Tillatelse til å nå måldeltaker er ikke tillatt",permissionToReachTargetParticipantNotAllowedMoreDetails:"Kontroller at måldeltakeren er i samme leier",unableToResolveTenantTitle:"Kan ikke løse leier-ID for måldeltakeren",unableToResolveTenantMoreDetails:"Kontroller at deltaker-ID-en er riktig angitt",participantIdIsMalformedTitle:"Deltaker-ID-en har ikke riktig format",participantIdIsMalformedMoreDetails:"Kontroller at deltaker-ID-en har riktig format",moreButtonGalleryControlLabel:"Vis",moreButtonGalleryPositionToggleLabel:"Flytt galleri øverst",moreButtonGallerySpeakerLayoutLabel:"Høyttaler",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamisk",moreButtonGalleryDefaultLayoutLabel:"Gallerivisning",moreButtonGalleryFocusedContentLayoutLabel:"Fokuser på innhold",moreButtonLargeGalleryDefaultLayoutLabel:"Stort galleri",moreButtonTogetherModeLayoutLabel:"Sammen-modus",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Kameraet er deaktivert. Du kan ikke lenger dele video.",grantedDueToMeetingOption:"Kameraet er aktivert. Slå den på hvis du vil dele video."},unmuteMic:{lostDueToMeetingOption:"Mikrofonen er deaktivert. Du kan ikke lenger oppheve dempingen.",grantedDueToMeetingOption:"Mikrofonen er aktivert. Opphev demping hvis du vil snakke."},shareScreen:{lostDueToRoleChangeToAttendee:"Rollen din er endret. Noen handlinger, for eksempel deling av innhold, vil ikke være tilgjengelige for deg.",grantedDueToRoleChangeToPresenter:"Du er presentatør. Du kan dele innhold og tilrettelegge møtet."},hideAttendeeNames:{hideAttendeeNameAttendee:"Navnet ditt er skjult for andre deltakere. Arrangører og presentatører kan se ditt virkelige navn.",hideAttendeeNamePresenter:"Deltakernavn er skjult. Bare arrangører og presentatører kan se navn for å beskytte deltakernes personvern."}},surveyTitle:"Hjelp oss med å bli bedre",starSurveyHelperText:"Hvordan var kvaliteten på samtalen?",starSurveyOneStarText:"Kvaliteten var dårlig.",starSurveyTwoStarText:"Kvaliteten var dårlig.",starSurveyThreeStarText:"Kvaliteten var bra.",starSurveyFourStarText:"Kvaliteten var meget bra.",starSurveyFiveStarText:"Kvaliteten var meget bra.",surveyConfirmButtonLabel:"Send tilbakemelding",starRatingAriaLabel:"Velg {0} av {1} stjerner",surveySkipButtonLabel:"Hopp over",tagsSurveyQuestion:"Hva kunne ha vært bedre?",tagsSurveyTextFieldDefaultText:"Annet, spesifiser",tagsSurveyHelperText:"Merk eventuelle problemer du har opplevd.",endOfSurveyText:"Takk for tilbakemeldingen!",surveyIssues:{overallRating:{callCannotJoin:"Jeg kunne ikke bli i samtalen",callCannotInvite:"Jeg kunne ikke invitere noen til samtalen",hadToRejoin:"Jeg måtte bli med i samtalen på nytt",callEndedUnexpectedly:"Samtalen ble avbrutt",otherIssues:"Jeg hadde andre problemer med samtalen"},audioRating:{noLocalAudio:"Den andre parten hørte ikke noe lyd",noRemoteAudio:"Jeg hørte ikke lyd",echo:"Jeg hørte ekko i samtalen",audioNoise:"Jeg hørte støy i samtalen",lowVolume:"Volumet var lavt",audioStoppedUnexpectedly:"Lyd stoppet uventet",distortedSpeech:"Lyden var forvrengt",audioInterruption:"Lyden ble avbrutt",otherIssues:"Jeg hadde andre problemer med lyden i denne samtalen"},videoRating:{noVideoReceived:"Jeg kunne ikke se videoen",noVideoSent:"Andre kunne ikke se meg",lowQuality:"Videokvaliteten var dårlig",freezes:"Videoen frøs",stoppedUnexpectedly:"Video stoppet uventet",darkVideoReceived:"Jeg kan bare se mørke skjermer når andre slår på kameraet",audioVideoOutOfSync:"Lyd og video er ikke synkronisert",otherIssues:"Jeg hadde andre problemer med videoen i denne samtalen"},screenshareRating:{noContentLocal:"Andre personer kunne ikke se skjermdelingen min",noContentRemote:"Jeg kunne ikke se skjermdelingen til andre personer",cannotPresent:"Jeg kunne ikke vise skjermen min",lowQuality:"Kvaliteten på skjermdelingen var dårlig",freezes:"Skjermdeling frøs",stoppedUnexpectedly:"Skjermdelingen ble avbrutt",largeDelay:"Skjermdelingen har en stor forsinkelse",otherIssues:"Jeg hadde andre problemer med skjermdeling i denne samtalen"}},surveyIssuesHeadingStrings:{overallRating:"Samlet vurdering",audioRating:"Lyd",videoRating:"Video",screenshareRating:"Presentering"},spotlightPrompt:{startSpotlightHeading:"Sett i rampelyset for alle?",startSpotlightText:"Du merker denne videoen for alle i møtet.",startSpotlightOnSelfText:"Du merker videoen for alle i møtet.",startSpotlightConfirmButtonLabel:"Sett i rampelyset for alle",startSpotlightCancelButtonLabel:"Avbryt",stopSpotlightHeading:"Slutte å sette denne videoen i rampelyset?",stopSpotlightOnSelfHeading:"Fjern rampelyset?",stopAllSpotlightHeading:"Vil du stoppe rampelyset for alle videoer?",stopSpotlightText:"Denne videoen vil ikke lenger være uthevet for alle i møtet.",stopSpotlightOnSelfText:"Videoen vil ikke lenger være uthevet for alle i møtet.",stopAllSpotlightText:"Videoene vil ikke lenger være uthevet for alle i møtet.",stopSpotlightConfirmButtonLabel:"Fjern fra rampelyset",stopSpotlightOnSelfConfirmButtonLabel:"Rampelys av",stopSpotlightCancelButtonLabel:"Avbryt",closeSpotlightPromptButtonLabel:"Lukk"},exitSpotlightButtonLabel:"Rampelys av",exitSpotlightButtonTooltip:"Rampelys av",leaveConfirmButtonLabel:"Forlat",endCallConfirmButtonLabel:"Avslutt samtalen",hangUpCancelButtonLabel:"Avbryt",leaveConfirmDialogTitle:"Forlat chatten",leaveConfirmDialogContent:"Er du sikker på at du vil forlate samtalen?",endCallConfirmDialogTitle:"Avslutt samtalen?",endCallConfirmDialogContent:"Du avslutter møtet for alle.",invalidMeetingIdentifier:"Ugyldig møte-ID eller passord",pinParticipantMenuLabel:"Fest for meg",pinParticipantLimitReachedMenuLabel:"Fest (grensen er nådd)",unpinParticipantMenuLabel:"Løsne",pinParticipantMenuItemAriaLabel:"Fest for meg",unpinParticipantMenuItemAriaLabel:"Løsne",muteAllMenuLabel:"Demp alle",muteAllDialogTitle:"Vil du dempe alle?",muteAllDialogContent:"Dette demper alle i møtet, bortsett fra deg selv.",muteAllConfirmButtonLabel:"Demp alle",muteAllCancelButtonLabel:"Avbryt",joinBreakoutRoomButtonLabel:"Bli med i grupperommet",returnFromBreakoutRoomButtonLabel:"Forlat rom",leaveBreakoutRoomAndMeetingButtonLabel:"Forlat møte",breakoutRoomJoinedNotificationTitle:"Du har blitt med i grupperommet {roomName}",joinBreakoutRoomBannerTitle:"Grupperommet {roomName} er åpent",joinBreakoutRoomBannerButtonLabel:"Bli med",returnFromBreakoutRoomBannerTitle:"Gå tilbake til hovedmøte",returnFromBreakoutRoomBannerButtonLabel:"Bli med på nytt",forbidAudioMenuLabel:"Deaktiver mikrofon",permitAudioMenuLabel:"Tillat mikrofon",forbidOthersAudioDialogTitle:"Deaktivere mikrofon for deltakere?",forbidOthersAudioDialogContent:"Deltakere kan ikke oppheve dempingen selv. Arrangøren og presentatørene kan la personer oppheve dempingen ved behov.",forbidOthersAudioConfirmButtonLabel:"Deaktiver mikrofoner",forbidOthersAudioCancelButtonLabel:"Avbryt",permitOthersAudioDialogTitle:"Tillat mikrofon for deltakere?",permitOthersAudioDialogContent:"Alle i møtet vil kunne oppheve dempingen.",permitOthersAudioConfirmButtonLabel:"Tillat mikrofoner",permitOthersAudioCancelButtonLabel:"Avbryt",forbidOthersAudioMenuLabel:"Deaktiver mikrofon for deltakere",permitOthersAudioMenuLabel:"Tillat mikrofon for deltakere",forbidVideoMenuLabel:"Deaktiver kamera",permitVideoMenuLabel:"Tillat kamera",forbidOthersVideoDialogTitle:"Deaktiver kamera for deltakere?",forbidOthersVideoDialogContent:"Deltakere kan ikke slå på videoen selv. Arrangøren og presentatørene kan la personer slå på kameraet etter behov.",forbidOthersVideoConfirmButtonLabel:"Deaktiver kameraer",forbidOthersVideoCancelButtonLabel:"Avbryt",permitOthersVideoDialogTitle:"Tillat kamera for deltakere?",permitOthersVideoDialogContent:"Alle i møtet kan slå på videoen selv.",permitOthersVideoConfirmButtonLabel:"Tillat kameraer",permitOthersVideoCancelButtonLabel:"Avbryt",forbidOthersVideoMenuLabel:"Deaktiver kamera for deltakere",permitOthersVideoMenuLabel:"Tillat kamera for deltakere"};var chat$8={chatListHeader:"I denne chatten",uploadAttachment:"Last opp vedlegg",uploadImageDataNotProvided:"Bildedata er ikke angitt.",uploadImageIsTooLarge:"Bildet er for stort. Velg en som er mindre enn {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Opplasting av .{imageExtension}-bilde er ikke tillatt.",uploadImageFailed:"Kan ikke laste opp bilde. Prøv på nytt senere."};var callWithChat$8={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"Ny melding",chatButtonTooltipClosedWithMessageCount:"Vis chat ({unreadMessagesCount} ulest)",chatButtonTooltipClose:"Skjul chat",chatButtonTooltipOpen:"Vis chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Kopier invitasjonskobling",copyInviteLinkButtonActionedLabel:"Koblingen er kopiert",copyInviteLinkActionedAriaLabel:"Invitasjonskobling er kopiert",dismissSidePaneButtonLabel:"Lukk",moreDrawerAudioDeviceMenuTitle:"Lydenhet",moreDrawerButtonLabel:"Flere alternativer",moreDrawerButtonTooltip:"Flere alternativer",moreDrawerMicrophoneMenuTitle:"Mikrofon",moreDrawerSpeakerMenuTitle:"Høyttaler",moreDrawerCaptionsMenuTitle:"Undertekster",moreDrawerSpokenLanguageMenuTitle:"Talespråk",moreDrawerCaptionLanguageMenuTitle:"Språk for undertekst",moreDrawerGalleryOptionsMenuTitle:"Gallerialternativer",peopleButtonLabel:"Personer",selectedPeopleButtonLabel:"Personer-knappen er valgt",peopleButtonTooltipOpen:"Vis deltakere",peopleButtonTooltipClose:"Skjul deltakere",peopleButtonTooltipOpenAriaLabel:"Personer-knapp, Vis deltakere",peopleButtonTooltipCloseAriaLabel:"Personer-knapp, Skjul deltakere",peoplePaneSubTitle:"I denne samtalen",peoplePaneTitle:"Personer",pictureInPictureTileAriaLabel:"Videofeeder. Klikk for å gå tilbake til samtaleskjermen.",removeMenuLabel:"Fjern",openDialpadButtonLabel:"Slå telefonnummeret",returnToCallButtonAriaDescription:"Gå tilbake til samtale",returnToCallButtonAriaLabel:"Tilbake",peoplePaneAddPeopleButtonLabel:"Legg til personer",dialpadStartCallButtonLabel:"Ringe",dialpadModalTitle:"Slå telefonnummeret",dialpadModalAriaLabel:"Numerisk tastatur",dialpadCloseModalButtonAriaLabel:"Lukk numerisk tastatur",openDtmfDialpadLabel:"Vis numerisk tastatur",dtmfDialpadPlaceholderText:"Skriv inn nummer",breakoutRoomChatPaneTitle:"Grupperomchat",chatContentSpinnerLabel:"Laster inn chat"};var nb_NO = {call:call$8,chat:chat$8,callWithChat:callWithChat$8};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Norwegian Bokmål (Norway)
 *
 * @public
 */
const COMPOSITE_LOCALE_NB_NO = {
    component: COMPONENT_LOCALE_NB_NO,
    strings: createCompositeStrings(nb_NO)
};

var call$7={cameraLabel:"Camera",noCamerasLabel:"Geen camera's gevonden",cameraPermissionDenied:"Uw browser blokkeert de toegang tot uw camera",cameraTurnedOff:"Camera is uitgeschakeld",chatButtonLabel:"Chatten",close:"Sluiten",complianceBannerNowOnlyRecording:"U bent nu alleen deze vergadering aan het opnemen.",complianceBannerNowOnlyTranscription:"U bent nu alleen deze vergadering aan het transcriberen.",complianceBannerRecordingAndTranscriptionSaved:"Opname en transcriptie worden opgeslagen.",complianceBannerRecordingAndTranscriptionStarted:"Opname en transcriptie zijn gestart.",complianceBannerRecordingAndTranscriptionStopped:"Opname en transcriptie zijn gestopt.",complianceBannerRecordingSaving:"De opname wordt opgeslagen.",complianceBannerRecordingStarted:"De opname is gestart.",complianceBannerRecordingStopped:"De opname is gestopt.",complianceBannerTranscriptionStarted:"Transcriptie is gestart.",complianceBannerTranscriptionConsent:"Door deel te nemen, geeft u toestemming voor het transcriberen van deze vergadering.",complianceBannerTranscriptionSaving:"Transcriptie wordt opgeslagen.",complianceBannerTranscriptionStopped:"Transcriptie is gestopt.",configurationPageTitle:"Een oproep starten",copyInviteLinkButtonLabel:"Uitnodigingskoppeling kopiëren",copyInviteLinkButtonActionedLabel:"De koppeling is gekopieerd",copyInviteLinkActionedAriaLabel:"Uitnodigingskoppeling gekopieerd",defaultPlaceHolder:"Een optie selecteren",dismissSidePaneButtonLabel:"Sluiten",videoEffectsPaneTitle:"Effecten",videoEffectsPaneBackgroundSelectionTitle:"Achtergrond",videoEffectsPaneAriaLabel:"Deelvenster video-effectenVideo",configurationPageCameraIsLoadingLabel:"Video wordt geladen...",configurationPageVideoEffectsButtonLabel:"Effecten",unableToStartVideoEffect:"Kan video-effect niet toepassen.",blurBackgroundEffectButtonLabel:"Vervagen",blurBackgroundTooltip:"Achtergrond vervagen",removeBackgroundEffectButtonLabel:"Geen",removeBackgroundTooltip:"Achtergrond verwijderen",cameraOffBackgroundEffectWarningText:"Uw camera is uitgeschakeld. Schakel de camera in om het video-effect te zien.",failedToJoinCallDueToNoNetworkMoreDetails:"De verbinding is verbroken vanwege een netwerkprobleem. Controleer de verbinding en neem opnieuw deel.",failedToJoinCallDueToNoNetworkTitle:"Verbinding verbroken",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"U hebt geen toegang gekregen in het gesprek. Als dit een fout is, neemt u opnieuw deel aan het gesprek.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Verwijderd uit lobby",learnMore:"Meer informatie",leavingCallTitle:"Verlaten...",leftCallMoreDetails:"Als dit een vergissing was, neemt u opnieuw deel aan het gesprek.",leftCallTitle:"U hebt het gesprek verlaten",lobbyScreenConnectingToCallTitle:"Deelnemen aan oproep",lobbyScreenWaitingToBeAdmittedTitle:"Wachten om te worden toegelaten",microphonePermissionDenied:"Uw browser blokkeert de toegang tot uw microfoon",microphoneToggleInLobbyNotAllowed:"Kan niet dempen of dempen opheffen in de lobby.",mutedMessage:"Uw geluid is gedempt",networkReconnectMoreDetails:"Er is iets misgegaan. We proberen terug te gaan naar het gesprek.",networkReconnectTitle:"Wacht even",notInvitedToRoomDetails:"U kunt niet deelnemen aan deze ruimte omdat u geen uitnodiging hebt.",notInvitedToRoomTitle:"Niet uitgenodigd voor ruimte",peopleButtonLabel:"Personen",selectedPeopleButtonLabel:"Knop Personen geselecteerd",peoplePaneTitle:"Personen",peoplePaneMoreButtonAriaLabel:"Meer",peopleButtonTooltipOpen:"Deelnemers weergeven",peopleButtonTooltipClose:"Deelnemers verbergen",peoplePaneSubTitle:"In dit gesprek {numberOfPeople}",privacyPolicy:"Privacybeleid",rejoinCallButtonLabel:"Opnieuw deelnemen aan gesprek",removedFromCallMoreDetails:"Een andere deelnemer heeft u uit het gesprek verwijderd.",removedFromCallTitle:"U bent verwijderd",removeMenuLabel:"Verwijderen",startSpotlightMenuLabel:"Spotlight voor iedereen",addSpotlightMenuLabel:"Spotlight toevoegen",stopSpotlightMenuLabel:"Spotlighten stoppen",stopSpotlightOnSelfMenuLabel:"Spotlight afsluiten",spotlightLimitReachedMenuTitle:"Spotlightlimiet bereikt",stopAllSpotlightMenuLabel:"Alle spotlights stoppen",returnToCallButtonAriaDescription:"Terug naar gesprek",returnToCallButtonAriaLabel:"Terug",roomNotFoundDetails:"Kan de opgegeven ruimte-id niet vinden.",roomNotFoundTitle:"Ruimte niet gevonden",roomNotValidDetails:"Deze ruimte is momenteel niet geldig.",roomNotValidTitle:"De ruimte is niet geldig",inviteToRoomRemovedDetails:"Uw uitnodiging om deel te nemen aan deze ruimte is verwijderd.",inviteToRoomRemovedTitle:"Uitnodiging voor ruimte verwijderd",soundLabel:"Geluid",noMicrophonesLabel:"Geen microfoons gevonden",noSpeakersLabel:"Geen luidsprekers gevonden",startCallButtonLabel:"Gesprek starten",openDialpadButtonLabel:"Telefoonnummer kiezen",peoplePaneAddPeopleButtonLabel:"Personen toevoegen",dialpadStartCallButtonLabel:"Bellen",dialpadModalTitle:"Telefoonnummer kiezen",dialpadModalAriaLabel:"Telefoonkiezer",dialpadCloseModalButtonAriaLabel:"Toetsenblok sluiten",moreButtonCallingLabel:"Meer",resumeCallButtonLabel:"Hervatten",resumingCallButtonLabel:"Hervatten...",resumeCallButtonAriaLabel:"Gesprek hervatten",resumingCallButtonAriaLabel:"Gesprek hervatten",holdScreenLabel:"U staat in de wacht",openDtmfDialpadLabel:"Toetsenblok weergeven",dtmfDialpadPlaceholderText:"Nummer invoeren",outboundCallingNoticeString:"Bellen...",participantJoinedNoticeString:"{displayName} neemt deel",twoParticipantJoinedNoticeString:"{displayName1} en {displayName2} nemen deel",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} en {displayName3} nemen deel",participantLeftNoticeString:"{displayName} is vertrokken",twoParticipantLeftNoticeString:"{displayName1} en {displayName2} zijn vertrokken",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} en {displayName3} zijn vertrokken",unnamedParticipantString:"naamloze deelnemer",manyUnnamedParticipantsJoined:"naamloze deelnemer en {numOfParticipants} andere deelnemers nemen deel",manyUnnamedParticipantsLeft:"naamloze deelnemer en {numOfParticipants} andere deelnemers zijn vertrokken",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} en {numOfParticipants} andere deelnemers",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} en {numOfParticipants} andere deelnemers resterend",liveCaptionsLabel:"Liveondertitels",captionsSettingsLabel:"Instellingen voor ondertiteling",startCaptionsButtonOnLabel:"Bijschriften inschakelen",startCaptionsButtonOffLabel:"Ondertitels uitschakelen",startCaptionsButtonTooltipOnContent:"Ondertitels uitschakelen",startCaptionsButtonTooltipOffContent:"Bijschriften inschakelen",captionsSettingsModalTitle:"Instellingen voor ondertitels",captionsSettingsSpokenLanguageDropdownLabel:"Gesproken taal",captionsSettingsCaptionLanguageDropdownLabel:"Taal van ondertitels",captionsSettingsSpokenLanguageDropdownInfoText:"Taal die iedereen in dit gesprek spreekt.",captionsSettingsCaptionLanguageDropdownInfoText:"Ondertitels worden weergegeven in deze taal.",captionsSettingsConfirmButtonLabel:"Bevestigen",captionsSettingsCancelButtonLabel:"Annuleren",captionsSettingsModalAriaLabel:"Bijschriften instellen modaal",captionsSettingsCloseModalButtonAriaLabel:"Instelling voor ondertiteling sluiten",captionsBannerMoreButtonCallingLabel:"Instellingen voor ondertitels - Meer opties",captionsBannerMoreButtonTooltip:"Meer opties",realTimeTextLabel:"Typen in real time (RTT)",startRealTimeTextLabel:"RTT inschakelen voor dit gesprek",disabledStartRealTimeTextLabel:"RTT is momenteel ingeschakeld en kan niet worden uitgeschakeld",dismissModalAriaLabel:"Lokale en externe afbeelding en afbeelding, druk op Enter om terug te keren naar het gesprek",callRejectedTitle:"De aanroep kan niet worden voltooid",callRejectedMoreDetails:"De persoon die u probeert te bereiken, is niet beschikbaar. Probeer het later opnieuw.",callTimeoutTitle:"De aanroep kan niet worden voltooid",callTimeoutDetails:"De persoon die u probeert te bereiken, is niet beschikbaar. Probeer het later opnieuw.",callTimeoutBotTitle:"De aanroep kan niet worden voltooid",callTimeoutBotDetails:"De service die u probeert te bereiken, is niet beschikbaar. Probeer het later opnieuw.",dtmfDialerButtonLabel:"Toetsenblok",dtmfDialerButtonTooltipOn:"Toetsenblok weergeven",dtmfDialerButtonTooltipOff:"Toetsenblok verbergen",dtmfDialerMoreButtonLabelOn:"Toetsenblok weergeven",dtmfDialerMoreButtonLabelOff:"Toetsenblok verbergen",phoneCallMoreButtonLabel:"Oproep",spokenLanguageStrings:{"ar-ae":"Arabisch - Verenigde Arabische Emiraten","ar-sa":"Arabisch - Saudi-Arabië","da-dk":"Deens","de-de":"Duits - Duitsland","en-au":"Engels - Australië","en-ca":"Engels - Canada","en-gb":"Engels - Verenigd Koninkrijk","en-in":"Engels - India","en-nz":"Engels - Nieuw-Zeeland","en-us":"Engels - Verenigde Staten","es-es":"Spaans - Spanje (modern)","es-mx":"Spaans - Mexico","fi-fi":"Fins","fr-ca":"Frans - Canada","fr-fr":"Frans - Frankrijk","hi-in":"Hindi","it-it":"Italiaans - Italië","ja-jp":"Japans","ko-kr":"Koreaans","nb-no":"Noors (Bokmål)","nl-be":"Nederlands - België","nl-nl":"Nederlands - Nederland","pl-pl":"Pools","pt-br":"Portugees - Brazilië","ru-ru":"Russisch","sv-se":"Zweeds","zh-cn":"Chinees (Volksrepubliek China)","zh-hk":"Chinees - Hongkong SAR","cs-cz":"Tsjechisch","pt-pt":"Portugees - Portugal","tr-tr":"Turks","vi-vn":"Vietnamees","th-th":"Thai","he-il":"Hebreeuws","cy-gb":"Welsh","uk-ua":"Oekraïens","el-gr":"Grieks","hu-hu":"Hongaars","ro-ro":"Roemeens","sk-sk":"Slowaaks","zh-tw":"Chinees - Taiwan"},captionLanguageStrings:{ar:"Arabisch",da:"Deens",de:"Duits",en:"Engels",es:"Spaans",fi:"Fins","fr-ca":"Frans - Canada",fr:"Frans - Frankrijk",hi:"Hindi",it:"Italiaans",ja:"Japans",ko:"Koreaans",nb:"Noors (Bokmål)",nl:"Nederlands",pl:"Pools",pt:"Portugees - Brazilië",ru:"Russisch",sv:"Zweeds","zh-Hans":"Vereenvoudigd Chinees","zh-Hant":"Traditioneel Chinees",cs:"Tsjechisch","pt-pt":"Portugees - Portugal",tr:"Turks",vi:"Vietnamees",th:"Thai",he:"Hebreeuws",cy:"Welsh",uk:"Oekraïens",el:"Grieks",hu:"Hongaars",ro:"Roemeens",sk:"Slowaaks"},captionsBannerSpinnerText:"Ondertitels starten...",realTimeTextInputBoxDefaultText:"Bericht in realtime typen",realTimeTextInputErrorMessage:"Het bericht mag niet langer zijn dan 2000 tekens",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (realtime-tekst) is ingeschakeld voor alle deelnemers gedurende de hele duur van de vergadering.",realTimeTextBannerLinkLabel:"Meer informatie",realTimeTextModalAriaLabel:"RTT Modal inschakelen",realTimeTextModalTitle:"RTT inschakelen?",realTimeTextModalText:"Tekst in realtime (RTT) wordt ingeschakeld voor iedereen in de vergadering.",realTimeTextConfirmButtonLabel:"Inschakelen",realTimeTextCancelButtonLabel:"Annuleren",realTimeTextCloseModalButtonAriaLabel:"RTT Modaal sluiten",captionsOnlyContainerTitle:"Ondertitels",realTimeTextOnlyContainerTitle:"Real-time tekst",captionsAndRealTimeTextContainerTitle:"Ondertitels en realtime tekst",expandButtonAriaLabel:"Uitvouwen",minimizeButtonAriaLabel:"Samenvouwen",transferPageTransferorText:"Doorverbinden...",transferPageTransferTargetText:"Verbinding maken...",transferPageUnknownTransferorDisplayName:"Onbekend",transferPageUnknownTransferTargetDisplayName:"Onbekend",transferPageNoticeString:"U wordt doorgeschakeld",participantCouldNotBeReachedTitle:"Doeldeelnemer is momenteel niet beschikbaar",participantCouldNotBeReachedMoreDetails:"Neem contact op met de deelnemer wanneer deze beschikbaar is",permissionToReachTargetParticipantNotAllowedTitle:"Machtiging om de doeldeelnemer te bereiken is niet toegestaan",permissionToReachTargetParticipantNotAllowedMoreDetails:"Controleer of de doeldeelnemer zich in dezelfde tenant bevindt",unableToResolveTenantTitle:"Kan de tenant-id voor de doeldeelnemer niet omzetten",unableToResolveTenantMoreDetails:"Controleer of de deelnemer-id juist is ingevoerd",participantIdIsMalformedTitle:"Deelnemer-id heeft niet de juiste indeling",participantIdIsMalformedMoreDetails:"Controleer of de deelnemer-id de juiste indeling heeft",moreButtonGalleryControlLabel:"Bekijken",moreButtonGalleryPositionToggleLabel:"Galerie naar boven verplaatsen",moreButtonGallerySpeakerLayoutLabel:"Luidspreker",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamisch",moreButtonGalleryDefaultLayoutLabel:"Galerieweergave",moreButtonGalleryFocusedContentLayoutLabel:"Focus op inhoud",moreButtonLargeGalleryDefaultLayoutLabel:"Grote galerie",moreButtonTogetherModeLayoutLabel:"Modus Samen",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Uw camera is uitgeschakeld. U kunt geen video meer delen.",grantedDueToMeetingOption:"Uw camera is ingeschakeld. Schakel deze optie in als u video wilt delen."},unmuteMic:{lostDueToMeetingOption:"Uw microfoon is uitgeschakeld. U kunt het dempen niet meer opheffen.",grantedDueToMeetingOption:"Uw microfoon is ingeschakeld. Hef het dempen op als u wilt spreken."},shareScreen:{lostDueToRoleChangeToAttendee:"Uw rol is gewijzigd. Sommige acties, zoals het delen van inhoud, zijn niet beschikbaar voor u.",grantedDueToRoleChangeToPresenter:"U bent een presentator. U kunt inhoud delen en de vergadering vergemakkelijken."},hideAttendeeNames:{hideAttendeeNameAttendee:"Uw naam is verborgen voor andere genodigden. Organisatoren en presentatoren kunnen uw echte naam zien.",hideAttendeeNamePresenter:"Namen van genodigden zijn verborgen. Alleen organisatoren en presentatoren kunnen namen zien om de privacy van deelnemers te beschermen."}},surveyTitle:"Help mee met verbeteren",starSurveyHelperText:"Wat was de kwaliteit van het gesprek?",starSurveyOneStarText:"De kwaliteit was erg slecht.",starSurveyTwoStarText:"De kwaliteit was slecht.",starSurveyThreeStarText:"De kwaliteit was goed.",starSurveyFourStarText:"De kwaliteit was geweldig.",starSurveyFiveStarText:"De kwaliteit was uitstekend.",surveyConfirmButtonLabel:"Feedback verzenden",starRatingAriaLabel:"Selecteer {0} van {1} sterren",surveySkipButtonLabel:"Overslaan",tagsSurveyQuestion:"Wat had beter kunnen zijn?",tagsSurveyTextFieldDefaultText:"Overig, geef een toelichting",tagsSurveyHelperText:"Controleer eventuele problemen die u hebt ondervonden",endOfSurveyText:"Hartelijk dank voor uw feedback.",surveyIssues:{overallRating:{callCannotJoin:"Ik kan niet deelnemen aan de oproep",callCannotInvite:"Ik kan niet iemand uitnodigen voor de oproep",hadToRejoin:"Ik moest opnieuw deelnemen aan de oproep",callEndedUnexpectedly:"Oproep is onverwacht beëindigd voor mij",otherIssues:"Ik had andere problemen met de oproep"},audioRating:{noLocalAudio:"De ander kon geen geluid horen",noRemoteAudio:"Ik kon geen geluid horen",echo:"Ik hoorde echo's tijdens de oproep",audioNoise:"Ik hoorde een ruis tijdens de oproep",lowVolume:"Het volume was laag",audioStoppedUnexpectedly:"Audio is onverwacht gestopt",distortedSpeech:"Audio is vervormd",audioInterruption:"Audio is onderbroken",otherIssues:"Ik had andere audioproblemen in deze oproep"},videoRating:{noVideoReceived:"Ik kon geen video zien",noVideoSent:"Anderen konden mij niet zien",lowQuality:"Videokwaliteit was laag",freezes:"Video bevroren",stoppedUnexpectedly:"De video stopte onverwacht",darkVideoReceived:"Ik zie alleen donkere schermen wanneer anderen hun camera inschakelen",audioVideoOutOfSync:"Audio en video niet gesynchroniseerd",otherIssues:"Ik had andere audioproblemen in deze oproep"},screenshareRating:{noContentLocal:"Anderen kunnen mijn gedeelde scherm niet zien",noContentRemote:"Ik kon het gedeelde scherm van anderen niet zien",cannotPresent:"Ik kon mijn scherm niet presenteren",lowQuality:"De kwaliteit van scherm delen was laag",freezes:"Scherm delen bevroren",stoppedUnexpectedly:"Scherm delen is onverwacht gestopt",largeDelay:"Scherm dele heeft een grote vertraging",otherIssues:"Ik had andere problemen met het delen van het scherm in dit gesprek"}},surveyIssuesHeadingStrings:{overallRating:"Algemeen",audioRating:"Audio",videoRating:"Video",screenshareRating:"Presenteren"},spotlightPrompt:{startSpotlightHeading:"Spotlight voor iedereen?",startSpotlightText:"U markeert deze video voor iedereen in de vergadering.",startSpotlightOnSelfText:"U markeert deze video voor iedereen in de vergadering.",startSpotlightConfirmButtonLabel:"Spotlight voor iedereen",startSpotlightCancelButtonLabel:"Annuleren",stopSpotlightHeading:"Wilt u deze video niet meer spotlighten?",stopSpotlightOnSelfHeading:"Spotlight afsluiten?",stopAllSpotlightHeading:"Stoppen met het uitlichten van alle video's?",stopSpotlightText:"Deze video wordt niet meer gemarkeerd voor iedereen in de vergadering.",stopSpotlightOnSelfText:"Uw video wordt niet meer gemarkeerd voor iedereen in de vergadering.",stopAllSpotlightText:"De video's worden niet meer gemarkeerd voor iedereen in de vergadering.",stopSpotlightConfirmButtonLabel:"Spotlighten stoppen",stopSpotlightOnSelfConfirmButtonLabel:"Spotlight afsluiten",stopSpotlightCancelButtonLabel:"Annuleren",closeSpotlightPromptButtonLabel:"Sluiten"},exitSpotlightButtonLabel:"Spotlight afsluiten",exitSpotlightButtonTooltip:"Spotlight afsluiten",leaveConfirmButtonLabel:"Verlaten",endCallConfirmButtonLabel:"Gesprek beëindigen",hangUpCancelButtonLabel:"Annuleren",leaveConfirmDialogTitle:"Het gesprek verlaten?",leaveConfirmDialogContent:"Weet u zeker dat u het gesprek wilt verlaten?",endCallConfirmDialogTitle:"Het gesprek beëindigen?",endCallConfirmDialogContent:"U beëindigt het gesprek voor iedereen.",invalidMeetingIdentifier:"Ongeldige vergaderings-id of wachtwoordcode",pinParticipantMenuLabel:"Voor mij vastmaken",pinParticipantLimitReachedMenuLabel:"Pin (limiet bereikt)",unpinParticipantMenuLabel:"Losmaken",pinParticipantMenuItemAriaLabel:"Voor mij vastmaken",unpinParticipantMenuItemAriaLabel:"Losmaken",muteAllMenuLabel:"Allen dempen",muteAllDialogTitle:"Iedereen dempen?",muteAllDialogContent:"Hiermee wordt iedereen in de vergadering gedempt, behalve u.",muteAllConfirmButtonLabel:"Allen dempen",muteAllCancelButtonLabel:"Annuleren",joinBreakoutRoomButtonLabel:"Deelnemen aan aparte vergaderruimte",returnFromBreakoutRoomButtonLabel:"Vergaderruimte verlaten",leaveBreakoutRoomAndMeetingButtonLabel:"Vergadering verlaten",breakoutRoomJoinedNotificationTitle:"U neemt deel aan de aparte vergaderruimte '{roomName}'",joinBreakoutRoomBannerTitle:"Aparte vergaderruimte '{roomName}' is geopend",joinBreakoutRoomBannerButtonLabel:"Bijwonen",returnFromBreakoutRoomBannerTitle:"Terugkeren naar hoofdvergadering",returnFromBreakoutRoomBannerButtonLabel:"Opnieuw deelnemen",forbidAudioMenuLabel:"Microfoon uitschakelen",permitAudioMenuLabel:"Microfoon toestaan",forbidOthersAudioDialogTitle:"Microfoon uitschakelen voor deelnemers?",forbidOthersAudioDialogContent:"Deelnemers kunnen het dempen niet zelf opheffen. De organisator en de presentatoren kunnen indien nodig mensen het dempen laten opheffen.",forbidOthersAudioConfirmButtonLabel:"Microfoons uitschakelen",forbidOthersAudioCancelButtonLabel:"Annuleren",permitOthersAudioDialogTitle:"Microfoon toestaan voor deelnemers?",permitOthersAudioDialogContent:"Iedereen in de vergadering kan het gedempte eigen geluid opheffen.",permitOthersAudioConfirmButtonLabel:"Microfoons toestaan",permitOthersAudioCancelButtonLabel:"Annuleren",forbidOthersAudioMenuLabel:"Microfoon uitschakelen voor deelnemers",permitOthersAudioMenuLabel:"Microfoon toestaan voor deelnemers",forbidVideoMenuLabel:"Camera uitschakelen",permitVideoMenuLabel:"Camera toestaan",forbidOthersVideoDialogTitle:"Camera uitschakelen voor deelnemers?",forbidOthersVideoDialogContent:"Deelnemers kunnen video niet zelf inschakelen. De organisator en presentatoren kunnen mensen de camera laten inschakelen als dat nodig is.",forbidOthersVideoConfirmButtonLabel:"Camera's uitschakelen",forbidOthersVideoCancelButtonLabel:"Annuleren",permitOthersVideoDialogTitle:"Camera toestaan voor deelnemers?",permitOthersVideoDialogContent:"Iedereen in de vergadering kan zelf video inschakelen.",permitOthersVideoConfirmButtonLabel:"Camera's toestaan",permitOthersVideoCancelButtonLabel:"Annuleren",forbidOthersVideoMenuLabel:"Camera uitschakelen voor deelnemers",permitOthersVideoMenuLabel:"Camera toestaan voor deelnemers"};var chat$7={chatListHeader:"In deze chat",uploadAttachment:"Bijlage uploaden",uploadImageDataNotProvided:"Er zijn geen afbeeldingsgegevens opgegeven.",uploadImageIsTooLarge:"Afbeelding is te groot. Kies een optie die kleiner is dan {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Het uploaden van {imageExtension} afbeelding is niet toegestaan.",uploadImageFailed:"Kan afbeelding niet uploaden. Probeer het later opnieuw."};var callWithChat$7={chatButtonLabel:"Chatten",chatButtonNewMessageNotificationLabel:"Nieuw bericht",chatButtonTooltipClosedWithMessageCount:"Chat weergeven ({unreadMessagesCount} ongelezen)",chatButtonTooltipClose:"Chat verbergen",chatButtonTooltipOpen:"Chat weergeven",chatPaneTitle:"Chatten",copyInviteLinkButtonLabel:"Uitnodigingskoppeling kopiëren",copyInviteLinkButtonActionedLabel:"De koppeling is gekopieerd",copyInviteLinkActionedAriaLabel:"Uitnodigingskoppeling gekopieerd",dismissSidePaneButtonLabel:"Sluiten",moreDrawerAudioDeviceMenuTitle:"Audioapparaat",moreDrawerButtonLabel:"Meer opties",moreDrawerButtonTooltip:"Meer opties",moreDrawerMicrophoneMenuTitle:"Microfoon",moreDrawerSpeakerMenuTitle:"Luidspreker",moreDrawerCaptionsMenuTitle:"Liveondertitels",moreDrawerSpokenLanguageMenuTitle:"Gesproken taal",moreDrawerCaptionLanguageMenuTitle:"Taal van ondertiteling",moreDrawerGalleryOptionsMenuTitle:"Galerieopties",peopleButtonLabel:"Personen",selectedPeopleButtonLabel:"Knop Personen geselecteerd",peopleButtonTooltipOpen:"Deelnemers weergeven",peopleButtonTooltipClose:"Deelnemers verbergen",peopleButtonTooltipOpenAriaLabel:"Knop Personen, Deelnemers weergeven",peopleButtonTooltipCloseAriaLabel:"Knop Personen, Deelnemers verbergen",peoplePaneSubTitle:"In dit gesprek",peoplePaneTitle:"Personen",pictureInPictureTileAriaLabel:"Videofeeds. Klik om terug te keren naar het oproepscherm.",removeMenuLabel:"Verwijderen",openDialpadButtonLabel:"Telefoonnummer kiezen",returnToCallButtonAriaDescription:"Terug naar gesprek",returnToCallButtonAriaLabel:"Terug",peoplePaneAddPeopleButtonLabel:"Personen toevoegen",dialpadStartCallButtonLabel:"Bellen",dialpadModalTitle:"Telefoonnummer kiezen",dialpadModalAriaLabel:"Telefoonkiezer",dialpadCloseModalButtonAriaLabel:"Toetsenblok sluiten",openDtmfDialpadLabel:"Toetsenblok weergeven",dtmfDialpadPlaceholderText:"Nummer invoeren",breakoutRoomChatPaneTitle:"Aparte vergaderruimte chat",chatContentSpinnerLabel:"Chat aan het laden"};var nl_NL = {call:call$7,chat:chat$7,callWithChat:callWithChat$7};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Dutch (Netherlands)
 *
 * @public
 */
const COMPOSITE_LOCALE_NL_NL = {
    component: COMPONENT_LOCALE_NL_NL,
    strings: createCompositeStrings(nl_NL)
};

var call$6={cameraLabel:"Kamera",noCamerasLabel:"Nie znaleziono żadnej kamery",cameraPermissionDenied:"Przeglądarka blokuje dostęp do kamery",cameraTurnedOff:"Twoja kamera jest wyłączona",chatButtonLabel:"Czat",close:"Zamknij",complianceBannerNowOnlyRecording:"W tej chwili tylko Ty nagrywasz to spotkanie.",complianceBannerNowOnlyTranscription:"W tej chwili tylko Ty transkrybujesz to spotkanie.",complianceBannerRecordingAndTranscriptionSaved:"Trwa zapisywanie nagrania i transkrypcji.",complianceBannerRecordingAndTranscriptionStarted:"Rozpoczęto nagrywanie i transkrypcję.",complianceBannerRecordingAndTranscriptionStopped:"Zatrzymano nagrywanie i transkrypcję.",complianceBannerRecordingSaving:"Trwa zapisywanie nagrania.",complianceBannerRecordingStarted:"Rozpoczęto nagrywanie.",complianceBannerRecordingStopped:"Nagranie zostało zatrzymane.",complianceBannerTranscriptionStarted:"Rozpoczęto transkrypcję.",complianceBannerTranscriptionConsent:"Dołączając, wyrażasz zgodę na transkrypcję tego spotkania.",complianceBannerTranscriptionSaving:"Trwa zapisywanie transkrypcji.",complianceBannerTranscriptionStopped:"Zakończono transkrypcję.",configurationPageTitle:"Rozpocznij rozmowę",copyInviteLinkButtonLabel:"Kopiuj link do zaproszenia",copyInviteLinkButtonActionedLabel:"Skopiowano link",copyInviteLinkActionedAriaLabel:"Skopiowano link do zaproszenia",defaultPlaceHolder:"Wybierz opcję",dismissSidePaneButtonLabel:"Zamknij",videoEffectsPaneTitle:"Efekty",videoEffectsPaneBackgroundSelectionTitle:"Tło",videoEffectsPaneAriaLabel:"Okienko efektów wideo",configurationPageCameraIsLoadingLabel:"Ładowanie wideo w toku...",configurationPageVideoEffectsButtonLabel:"Efekty",unableToStartVideoEffect:"Nie można zastosować efektu wideo.",blurBackgroundEffectButtonLabel:"Rozmycie",blurBackgroundTooltip:"Rozmyj tło",removeBackgroundEffectButtonLabel:"Brak",removeBackgroundTooltip:"Usuń tło",cameraOffBackgroundEffectWarningText:"Kamera jest wyłączona. Włącz kamerę, aby zobaczyć efekt wideo.",failedToJoinCallDueToNoNetworkMoreDetails:"Połączenie zostało rozłączone z powodu problemu z siecią. Sprawdź połączenie i dołącz ponownie.",failedToJoinCallDueToNoNetworkTitle:"Rozmowa została rozłączona",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Nie udzielono Ci dostępu do rozmowy. Jeśli to pomyłka, dołącz ponownie do rozmowy.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Odrzucono z poczekalni",learnMore:"Dowiedz się więcej",leavingCallTitle:"Opuszczanie…",leftCallMoreDetails:"Jeśli to pomyłka, dołącz ponownie do rozmowy.",leftCallTitle:"Opuszczono rozmowę",lobbyScreenConnectingToCallTitle:"Dołączanie do rozmowy",lobbyScreenWaitingToBeAdmittedTitle:"Oczekiwanie na przyjęcie",microphonePermissionDenied:"Twoja przeglądarka blokuje dostęp do mikrofonu",microphoneToggleInLobbyNotAllowed:"Nie można wyciszyć ani wyłączyć wyciszenia w poczekalni.",mutedMessage:"Wyciszenie jest włączone",networkReconnectMoreDetails:"Wygląda na to, że wystąpił problem. Próbujemy wrócić do rozmowy.",networkReconnectTitle:"Czekaj",notInvitedToRoomDetails:"Nie możesz dołączyć do tego pokoju, ponieważ nie masz zaproszenia.",notInvitedToRoomTitle:"Nie zaproszono do pokoju",peopleButtonLabel:"Osoby",selectedPeopleButtonLabel:"Wybrano przycisk Kontakty",peoplePaneTitle:"Osoby",peoplePaneMoreButtonAriaLabel:"Więcej",peopleButtonTooltipOpen:"Pokaż uczestników",peopleButtonTooltipClose:"Ukryj uczestników",peoplePaneSubTitle:"W tej rozmowie {numberOfPeople}",privacyPolicy:"Zasady ochrony prywatności",rejoinCallButtonLabel:"Dołącz ponownie do rozmowy",removedFromCallMoreDetails:"Inny uczestnik usunął Cię z rozmowy.",removedFromCallTitle:"Usunięto Cię",removeMenuLabel:"Usuń",startSpotlightMenuLabel:"Wyróżnij dla wszystkich",addSpotlightMenuLabel:"Dodaj wyróżnienie",stopSpotlightMenuLabel:"Zatrzymaj wyróżnianie",stopSpotlightOnSelfMenuLabel:"Zakończ wyróżnienie",spotlightLimitReachedMenuTitle:"Osiągnięto limit wyróżnienia",stopAllSpotlightMenuLabel:"Zatrzymaj wszystkie wyróżnienia",returnToCallButtonAriaDescription:"Wróć do rozmowy",returnToCallButtonAriaLabel:"Wstecz",roomNotFoundDetails:"Nie można odnaleźć podanego identyfikatora pokoju.",roomNotFoundTitle:"Nie znaleziono pokoju",roomNotValidDetails:"Ten pokój jest obecnie nieprawidłowy.",roomNotValidTitle:"Pokój jest nieprawidłowy",inviteToRoomRemovedDetails:"Twoje zaproszenie do dołączenia do tego pokoju zostało usunięte.",inviteToRoomRemovedTitle:"Usunięto zaproszenie do pokoju",soundLabel:"Dźwięk",noMicrophonesLabel:"Nie znaleziono mikrofonów",noSpeakersLabel:"Nie znaleziono głośników",startCallButtonLabel:"Zadzwoń",openDialpadButtonLabel:"Wybierz numer telefonu",peoplePaneAddPeopleButtonLabel:"Dodaj osoby",dialpadStartCallButtonLabel:"Zadzwoń",dialpadModalTitle:"Wybieranie numeru telefonu",dialpadModalAriaLabel:"Klawiatura numeryczna",dialpadCloseModalButtonAriaLabel:"Zamknij klawiaturę numeryczną",moreButtonCallingLabel:"Więcej",resumeCallButtonLabel:"Wznów",resumingCallButtonLabel:"Trwa wznawianie...",resumeCallButtonAriaLabel:"Wznów połączenie",resumingCallButtonAriaLabel:"Wznów połączenie",holdScreenLabel:"Połączenie zostało zawieszone.",openDtmfDialpadLabel:"Pokaż klawiaturę numeryczną",dtmfDialpadPlaceholderText:"Wprowadź numer",outboundCallingNoticeString:"Trwa łączenie…",participantJoinedNoticeString:"użytkownik {displayName} dołączył",twoParticipantJoinedNoticeString:"użytkownicy {displayName1} i {displayName2} dołączyli",threeParticipantJoinedNoticeString:"użytkownicy {displayName1}, {displayName2} i {displayName3} dołączyli",participantLeftNoticeString:"uczestnik {displayName} opuścił",twoParticipantLeftNoticeString:"uczestnicy {displayName1} i {displayName2} opuścili",threeParticipantLeftNoticeString:"uczestnicy {displayName1}, {displayName2} i {displayName3} opuścili",unnamedParticipantString:"uczestnik bez nazwy",manyUnnamedParticipantsJoined:"uczestnik bez nazwy i {numOfParticipants} innych uczestników dołączyli",manyUnnamedParticipantsLeft:"uczestnik bez nazwy i {numOfParticipants} innych uczestników opuścili",manyParticipantsJoined:"dołączono {displayName1}, {displayName2}, {displayName3} i {numOfParticipants} innych uczestników",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} i {numOfParticipants} innych uczestników pozostało",liveCaptionsLabel:"Napisy na żywo",captionsSettingsLabel:"Ustawienia napisów",startCaptionsButtonOnLabel:"Włącz podpisy",startCaptionsButtonOffLabel:"Wyłącz podpisy",startCaptionsButtonTooltipOnContent:"Wyłącz podpisy",startCaptionsButtonTooltipOffContent:"Włącz podpisy",captionsSettingsModalTitle:"Ustawienia napisów",captionsSettingsSpokenLanguageDropdownLabel:"Język mówiony",captionsSettingsCaptionLanguageDropdownLabel:"Język napisów",captionsSettingsSpokenLanguageDropdownInfoText:"Język, w którym mówią wszyscy uczestnicy tej rozmowy.",captionsSettingsCaptionLanguageDropdownInfoText:"Podpisy będą wyświetlane w tym języku.",captionsSettingsConfirmButtonLabel:"Potwierdź",captionsSettingsCancelButtonLabel:"Anuluj",captionsSettingsModalAriaLabel:"Modalne ustawianie podpisów",captionsSettingsCloseModalButtonAriaLabel:"Ustawienie podpisów",captionsBannerMoreButtonCallingLabel:"Więcej opcji ustawień napisów",captionsBannerMoreButtonTooltip:"Więcej opcji",realTimeTextLabel:"Pisanie w czasie rzeczywistym (RTT)",startRealTimeTextLabel:"Włącz funkcję Pisanie w czasie rzeczywistym dla tego połączenia",disabledStartRealTimeTextLabel:"Funkcja RTT jest obecnie włączona, nie można jej wyłączyć",dismissModalAriaLabel:"Lokalny i zdalny obraz i obraz, naciśnij klawisz Enter, aby powrócić do rozmowy",callRejectedTitle:"Nie można zakończyć rozmowy",callRejectedMoreDetails:"Osoba, z którą próbujesz się skontaktować, jest niedostępna. Spróbuj ponownie później.",callTimeoutTitle:"Nie można zakończyć rozmowy",callTimeoutDetails:"Osoba, z którą próbujesz się skontaktować, jest niedostępna. Spróbuj ponownie później.",callTimeoutBotTitle:"Nie można zakończyć rozmowy",callTimeoutBotDetails:"Usługa, z którą próbujesz się skontaktować, jest niedostępna. Spróbuj ponownie później.",dtmfDialerButtonLabel:"Klawiatura numeryczna",dtmfDialerButtonTooltipOn:"Pokaż klawiaturę numeryczną",dtmfDialerButtonTooltipOff:"Ukryj klawiaturę numeryczną",dtmfDialerMoreButtonLabelOn:"Pokaż klawiaturę numeryczną",dtmfDialerMoreButtonLabelOff:"Ukryj klawiaturę numeryczną",phoneCallMoreButtonLabel:"Rozmowa telefoniczna",spokenLanguageStrings:{"ar-ae":"arabski — Zjednoczone Emiraty Arabskie","ar-sa":"arabski — Arabia Saudyjska","da-dk":"duński","de-de":"niemiecki — Niemcy","en-au":"angielski — Australia","en-ca":"angielski — Kanada","en-gb":"angielski — Zjednoczone Królestwo","en-in":"angielski — Indie","en-nz":"angielski — Nowa Zelandia","en-us":"angielski — Stany Zjednoczone","es-es":"hiszpański — Hiszpania (nowoczesny)","es-mx":"hiszpański — Meksyk","fi-fi":"fiński","fr-ca":"francuski — Kanada","fr-fr":"francuski — Francja","hi-in":"hindi","it-it":"włoski — Włochy","ja-jp":"japoński","ko-kr":"koreański","nb-no":"norweski (Bokmål)","nl-be":"niderlandzki — Belgia","nl-nl":"niderlandzki — Holandia","pl-pl":"polski","pt-br":"portugalski — Brazylia","ru-ru":"rosyjski","sv-se":"szwedzki","zh-cn":"chiński (Chińska Republika Ludowa)","zh-hk":"chiński — Hongkong SAR","cs-cz":"czeski","pt-pt":"portugalski — Portugalia","tr-tr":"turecki","vi-vn":"wietnamski","th-th":"tajski","he-il":"hebrajski","cy-gb":"walijski","uk-ua":"ukraiński","el-gr":"grecki","hu-hu":"węgierski","ro-ro":"rumuński","sk-sk":"słowacki","zh-tw":"chiński — Tajwan"},captionLanguageStrings:{ar:"arabski",da:"duński",de:"niemiecki",en:"Angielski",es:"hiszpański",fi:"fiński","fr-ca":"francuski — Kanada",fr:"francuski — Francja",hi:"hindi",it:"włoski",ja:"japoński",ko:"koreański",nb:"norweski (Bokmål)",nl:"niderlandzki",pl:"polski",pt:"portugalski (Brazylia)",ru:"rosyjski",sv:"szwedzki","zh-Hans":"Chiński (uproszczony)","zh-Hant":"Chiński (tradycyjny)",cs:"czeski","pt-pt":"portugalski (Portugalia)",tr:"turecki",vi:"wietnamski",th:"tajski",he:"hebrajski",cy:"walijski",uk:"ukraiński",el:"grecki",hu:"węgierski",ro:"rumuński",sk:"słowacki"},captionsBannerSpinnerText:"Trwa uruchamianie napisów...",realTimeTextInputBoxDefaultText:"Pisanie wiadomości w czasie rzeczywistym",realTimeTextInputErrorMessage:"Wiadomość nie może przekraczać 2000 znaków",realTimeTextBannerTitle:"Pisanie w czasie rzeczywistym",realTimeTextBannerContent:"Funkcja RTT (tekst w czasie rzeczywistym) jest włączona dla wszystkich uczestników przez cały czas trwania spotkania.",realTimeTextBannerLinkLabel:"Dowiedz się więcej",realTimeTextModalAriaLabel:"Włącz modalne okno dialogowe pisania w czasie rzeczywistym",realTimeTextModalTitle:"Włączyć pisanie w czasie rzeczywistym?",realTimeTextModalText:"Tekst w czasie rzeczywistym zostanie włączony dla wszystkich uczestników spotkania.",realTimeTextConfirmButtonLabel:"Włącz",realTimeTextCancelButtonLabel:"Anuluj",realTimeTextCloseModalButtonAriaLabel:"Zamknij modalne okno dialogowe pisania w czasie rzeczywistym",captionsOnlyContainerTitle:"Napisy",realTimeTextOnlyContainerTitle:"Pisanie w czasie rzeczywistym",captionsAndRealTimeTextContainerTitle:"Napisy i pisanie w czasie rzeczywistym",expandButtonAriaLabel:"Rozwiń",minimizeButtonAriaLabel:"Zwiń",transferPageTransferorText:"Trwa przełączanie...",transferPageTransferTargetText:"Trwa łączenie...",transferPageUnknownTransferorDisplayName:"Nieznany",transferPageUnknownTransferTargetDisplayName:"Nieznany",transferPageNoticeString:"Jesteś w trakcie przekierowania",participantCouldNotBeReachedTitle:"Uczestnik docelowy jest obecnie niedostępny",participantCouldNotBeReachedMoreDetails:"Skontaktuj się z uczestnikiem, gdy będzie dostępny",permissionToReachTargetParticipantNotAllowedTitle:"Uprawnienie do dotarcia do uczestnika docelowego jest niedozwolone",permissionToReachTargetParticipantNotAllowedMoreDetails:"Sprawdź, czy uczestnik docelowy znajduje się w tej samej dzierżawie",unableToResolveTenantTitle:"Nie można rozpoznać identyfikatora dzierżawy dla uczestnika docelowego",unableToResolveTenantMoreDetails:"Sprawdź, czy identyfikator uczestnika został wprowadzony poprawnie",participantIdIsMalformedTitle:"Identyfikator uczestnika ma niepoprawny format",participantIdIsMalformedMoreDetails:"Sprawdź, czy identyfikator uczestnika ma poprawny format",moreButtonGalleryControlLabel:"Zobacz",moreButtonGalleryPositionToggleLabel:"Przenieś galerię na górę",moreButtonGallerySpeakerLayoutLabel:"Głośnik",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamiczny",moreButtonGalleryDefaultLayoutLabel:"Widok galerii",moreButtonGalleryFocusedContentLayoutLabel:"Skoncentruj się na zawartości",moreButtonLargeGalleryDefaultLayoutLabel:"Duża galeria",moreButtonTogetherModeLayoutLabel:"Tryb Razem",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Kamera została wyłączona. Nie możesz już udostępniać wideo.",grantedDueToMeetingOption:"Kamera została włączona. Włącz ją, jeśli chcesz udostępnić wideo."},unmuteMic:{lostDueToMeetingOption:"Mikrofon został wyłączony. Nie można już wyłączyć wyciszenia.",grantedDueToMeetingOption:"Mikrofon został włączony. Wyłącz wyciszenie, jeśli chcesz mówić."},shareScreen:{lostDueToRoleChangeToAttendee:"Rola została zmieniona. Niektóre akcje, takie jak udostępnianie zawartości, nie będą dla Ciebie dostępne.",grantedDueToRoleChangeToPresenter:"Jesteś osobą prowadzącą. Możesz udostępniać zawartość i prowadzić spotkanie."},hideAttendeeNames:{hideAttendeeNameAttendee:"Twoje imię i nazwisko jest ukryte przed innymi uczestnikami. Organizatorzy i osoby prowadzące mogą zobaczyć Twoje prawdziwe imię.",hideAttendeeNamePresenter:"Imiona i nazwiska uczestników są ukryte. Tylko organizatorzy i osoby prowadzące mogą wyświetlać imiona i nazwiska, aby chronić prywatność uczestników."}},surveyTitle:"Pomóż nam w ulepszaniu",starSurveyHelperText:"Jaka była jakość połączenia?",starSurveyOneStarText:"Jakość była zła.",starSurveyTwoStarText:"Jakość była niska.",starSurveyThreeStarText:"Jakość była dobra.",starSurveyFourStarText:"Jakość była doskonała.",starSurveyFiveStarText:"Jakość była świetna.",surveyConfirmButtonLabel:"Prześlij opinię",starRatingAriaLabel:"Wybierz {0} z {1} gwiazdek",surveySkipButtonLabel:"Pomiń",tagsSurveyQuestion:"Co można było zrobić lepiej?",tagsSurveyTextFieldDefaultText:"Inny — prosimy określić",tagsSurveyHelperText:"Sprawdź wszelkie napotkane problemy",endOfSurveyText:"Dziękujemy za opinię!",surveyIssues:{overallRating:{callCannotJoin:"Nie mogę dołączyć do rozmowy",callCannotInvite:"Nie mogę zaprosić innych osób do rozmowy",hadToRejoin:"Konieczne było ponowne dołączenie do rozmowy",callEndedUnexpectedly:"Rozmowa została nieoczekiwanie zakończona",otherIssues:"Wystąpiły inne problemy z połączeniem"},audioRating:{noLocalAudio:"Mój rozmówca nic nie słyszał",noRemoteAudio:"Nie było słychać żadnego dźwięku",echo:"Podczas rozmowy słychać było echo",audioNoise:"Słyszę szum podczas rozmowy",lowVolume:"Poziom głośności był zbyt niski",audioStoppedUnexpectedly:"Dźwięk został nieoczekiwanie zatrzymany",distortedSpeech:"Dźwięk był zniekształcony",audioInterruption:"Dźwięk został przerwany",otherIssues:"Występowały inne problemy z dźwiękiem podczas tego połączenia"},videoRating:{noVideoReceived:"Nie było nic widać",noVideoSent:"Inne osoby mnie nie widziały",lowQuality:"Jakość wideo była niska",freezes:"Wideo zablokowane",stoppedUnexpectedly:"Wideo zostało nieoczekiwanie zatrzymane",darkVideoReceived:"Widzę ciemne ekrany tylko wtedy, gdy inni włączają kamerę",audioVideoOutOfSync:"Dźwięk i obraz nie były zsynchronizowane",otherIssues:"Występowały inne problemy z wideo podczas tego połączenia"},screenshareRating:{noContentLocal:"Inne osoby nie widziały mojego udostępnionego ekranu",noContentRemote:"Nie widzę zrzutów ekranu innych osób",cannotPresent:"Nie mogę zaprezentować ekranu",lowQuality:"Jakość udostępniania ekranu była niska",freezes:"Udostępnianie ekranu zablokowane",stoppedUnexpectedly:"Udostępnianie ekranu zostało nieoczekiwanie zatrzymane",largeDelay:"Udostępnianie ekranu ma duże opóźnienie",otherIssues:"Wystąpiły inne problemy z udostępnianiem ekranu podczas tej rozmowy"}},surveyIssuesHeadingStrings:{overallRating:"Ogólne",audioRating:"Dźwięk",videoRating:"Wideo",screenshareRating:"W trakcie prezentacji"},spotlightPrompt:{startSpotlightHeading:"Wyróżnić dla wszystkich?",startSpotlightText:"Wyróżnisz ten klip wideo dla wszystkich uczestników spotkania.",startSpotlightOnSelfText:"Wyróżnisz swój klip wideo dla wszystkich uczestników spotkania.",startSpotlightConfirmButtonLabel:"Wyróżnij dla wszystkich",startSpotlightCancelButtonLabel:"Anuluj",stopSpotlightHeading:"Zatrzymać wyróżnianie tego klipu wideo?",stopSpotlightOnSelfHeading:"Zakończyć wyróżnianie?",stopAllSpotlightHeading:"Zatrzymać wyróżnianie wszystkich klipów wideo?",stopSpotlightText:"Klip wideo nie będzie już wyróżniany dla wszystkich osób na spotkaniu.",stopSpotlightOnSelfText:"Twój klip wideo nie będzie już wyróżniany dla wszystkich osób na spotkaniu.",stopAllSpotlightText:"Klipy wideo nie będą już wyróżniane dla wszystkich osób na spotkaniu.",stopSpotlightConfirmButtonLabel:"Zatrzymaj wyróżnianie",stopSpotlightOnSelfConfirmButtonLabel:"Zakończ wyróżnienie",stopSpotlightCancelButtonLabel:"Anuluj",closeSpotlightPromptButtonLabel:"Zamknij"},exitSpotlightButtonLabel:"Wyjdź z ważnych informacji",exitSpotlightButtonTooltip:"Wyjdź z ważnych informacji",leaveConfirmButtonLabel:"Opuść",endCallConfirmButtonLabel:"Zakończ połączenie",hangUpCancelButtonLabel:"Anuluj",leaveConfirmDialogTitle:"Opuścić połączenie?",leaveConfirmDialogContent:"Czy na pewno chcesz opuścić rozmowę?",endCallConfirmDialogTitle:"Zakończyć połączenie?",endCallConfirmDialogContent:"Zakończysz połączenie dla wszystkich.",invalidMeetingIdentifier:"Nieprawidłowy identyfikator spotkania lub kod dostępu",pinParticipantMenuLabel:"Przypnij dla mnie",pinParticipantLimitReachedMenuLabel:"Przypięcie (osiągnięto limit)",unpinParticipantMenuLabel:"Odepnij",pinParticipantMenuItemAriaLabel:"Przypnij dla mnie",unpinParticipantMenuItemAriaLabel:"Odepnij",muteAllMenuLabel:"Wycisz wszystkich",muteAllDialogTitle:"Wyciszyć wszystkich?",muteAllDialogContent:"Spowoduje to wyciszenie wszystkich osób na spotkaniu oprócz Ciebie.",muteAllConfirmButtonLabel:"Wycisz wszystkich",muteAllCancelButtonLabel:"Anuluj",joinBreakoutRoomButtonLabel:"Dołącz do osobnego pokoju",returnFromBreakoutRoomButtonLabel:"Opuść pokój",leaveBreakoutRoomAndMeetingButtonLabel:"Opuść spotkanie",breakoutRoomJoinedNotificationTitle:"Dołączono do osobnego pokoju „{roomName}”",joinBreakoutRoomBannerTitle:"Osobny pokój „{roomName}” jest otwarty",joinBreakoutRoomBannerButtonLabel:"Dołącz",returnFromBreakoutRoomBannerTitle:"Powrót do spotkania głównego",returnFromBreakoutRoomBannerButtonLabel:"Dołącz ponownie",forbidAudioMenuLabel:"Wyłącz mikrofon",permitAudioMenuLabel:"Zezwalaj na mikrofon",forbidOthersAudioDialogTitle:"Wyłączyć mikrofon uczestnikom?",forbidOthersAudioDialogContent:"Uczestnicy nie będą mogli samodzielnie wyłączyć wyciszenia. Organizator i osoby prowadzące mogą zezwolić innym osobom na wyłączanie wyciszenia w razie potrzeby.",forbidOthersAudioConfirmButtonLabel:"Wyłącz mikrofony",forbidOthersAudioCancelButtonLabel:"Anuluj",permitOthersAudioDialogTitle:"Czy zezwolić uczestnikom na używanie mikrofonu?",permitOthersAudioDialogContent:"Każdy uczestnik spotkania będzie mógł wyłączyć swoje wyciszenie.",permitOthersAudioConfirmButtonLabel:"Zezwalaj na mikrofony",permitOthersAudioCancelButtonLabel:"Anuluj",forbidOthersAudioMenuLabel:"Wyłącz mikrofon uczestników",permitOthersAudioMenuLabel:"Zezwalaj uczestnikom na używanie mikrofonu",forbidVideoMenuLabel:"Wyłącz kamerę",permitVideoMenuLabel:"Zezwalaj na kamerę",forbidOthersVideoDialogTitle:"Wyłączyć kamerę uczestników?",forbidOthersVideoDialogContent:"Uczestnicy nie będą mogli samodzielnie włączyć wideo. Organizator i osoby prowadzące mogą zezwolić innym osobom na włączenie kamery w razie potrzeby.",forbidOthersVideoConfirmButtonLabel:"Wyłącz kamery",forbidOthersVideoCancelButtonLabel:"Anuluj",permitOthersVideoDialogTitle:"Czy zezwolić uczestnikom na używanie kamer?",permitOthersVideoDialogContent:"Każdy uczestnik spotkania będzie mógł samodzielnie włączyć wideo.",permitOthersVideoConfirmButtonLabel:"Zezwalaj na kamery",permitOthersVideoCancelButtonLabel:"Anuluj",forbidOthersVideoMenuLabel:"Wyłącz kamerę uczestnikom",permitOthersVideoMenuLabel:"Zezwalaj uczestnikom na używanie kamery"};var chat$6={chatListHeader:"W tym czacie",uploadAttachment:"Przekaż załącznik",uploadImageDataNotProvided:"Nie udostępniono danych obrazu.",uploadImageIsTooLarge:"Obraz jest za duży. Wybierz rozmiar mniejszy niż {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Przekazywanie obrazu .{imageExtension} nie jest dozwolone.",uploadImageFailed:"Nie można przekazać obrazu. Spróbuj ponownie później."};var callWithChat$6={chatButtonLabel:"Czat",chatButtonNewMessageNotificationLabel:"Nowa wiadomość",chatButtonTooltipClosedWithMessageCount:"Pokaż czat (nieprzeczytane: {unreadMessagesCount})",chatButtonTooltipClose:"Ukryj czat",chatButtonTooltipOpen:"Pokaż czat",chatPaneTitle:"Czat",copyInviteLinkButtonLabel:"Kopiuj link do zaproszenia",copyInviteLinkButtonActionedLabel:"Skopiowano link",copyInviteLinkActionedAriaLabel:"Skopiowano link do zaproszenia",dismissSidePaneButtonLabel:"Zamknij",moreDrawerAudioDeviceMenuTitle:"Urządzenie audio",moreDrawerButtonLabel:"Więcej opcji",moreDrawerButtonTooltip:"Więcej opcji",moreDrawerMicrophoneMenuTitle:"Mikrofon",moreDrawerSpeakerMenuTitle:"Głośnik",moreDrawerCaptionsMenuTitle:"Napisy na żywo",moreDrawerSpokenLanguageMenuTitle:"Język mówiony",moreDrawerCaptionLanguageMenuTitle:"Język napisów",moreDrawerGalleryOptionsMenuTitle:"Opcje galerii",peopleButtonLabel:"Osoby",selectedPeopleButtonLabel:"Wybrano przycisk Kontakty",peopleButtonTooltipOpen:"Pokaż uczestników",peopleButtonTooltipClose:"Ukryj uczestników",peopleButtonTooltipOpenAriaLabel:"Przycisk Kontakty, Pokaż uczestników",peopleButtonTooltipCloseAriaLabel:"Przycisk Kontakty, Ukryj uczestników",peoplePaneSubTitle:"W tej rozmowie",peoplePaneTitle:"Osoby",pictureInPictureTileAriaLabel:"Kanały wideo. Kliknij, aby powrócić do ekranu rozmowy.",removeMenuLabel:"Usuń",openDialpadButtonLabel:"Wybierz numer telefonu",returnToCallButtonAriaDescription:"Wróć do rozmowy",returnToCallButtonAriaLabel:"Wstecz",peoplePaneAddPeopleButtonLabel:"Dodaj osoby",dialpadStartCallButtonLabel:"Zadzwoń",dialpadModalTitle:"Wybieranie numeru telefonu",dialpadModalAriaLabel:"Klawiatura numeryczna",dialpadCloseModalButtonAriaLabel:"Zamknij klawiaturę numeryczną",openDtmfDialpadLabel:"Pokaż klawiaturę numeryczną",dtmfDialpadPlaceholderText:"Wprowadź numer",breakoutRoomChatPaneTitle:"Czat osobnego pokoju",chatContentSpinnerLabel:"Ładowanie czatu"};var pl_PL = {call:call$6,chat:chat$6,callWithChat:callWithChat$6};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Polish (Poland)
 *
 * @public
 */
const COMPOSITE_LOCALE_PL_PL = {
    component: COMPONENT_LOCALE_PL_PL,
    strings: createCompositeStrings(pl_PL)
};

var call$5={cameraLabel:"Câmera",noCamerasLabel:"Nenhuma câmera encontrada",cameraPermissionDenied:"Seu navegador está bloqueando o acesso à câmera",cameraTurnedOff:"Sua câmera está desativada",chatButtonLabel:"Chat",close:"Fechar",complianceBannerNowOnlyRecording:"Você agora está apenas gravando esta reunião.",complianceBannerNowOnlyTranscription:"Você agora está apenas transcrevendo esta reunião.",complianceBannerRecordingAndTranscriptionSaved:"A gravação e a transcrição estão sendo salvas.",complianceBannerRecordingAndTranscriptionStarted:"A gravação e a transcrição foram iniciadas.",complianceBannerRecordingAndTranscriptionStopped:"A gravação e a transcrição foram interrompidas.",complianceBannerRecordingSaving:"A gravação está sendo salva.",complianceBannerRecordingStarted:"A gravação foi iniciada.",complianceBannerRecordingStopped:"A gravação foi interrompida.",complianceBannerTranscriptionStarted:"A transcrição foi iniciada.",complianceBannerTranscriptionConsent:"Ao ingressar, você está dando consentimento para que esta reunião seja transcrita.",complianceBannerTranscriptionSaving:"A transcrição está sendo salva.",complianceBannerTranscriptionStopped:"A transcrição foi interrompida.",configurationPageTitle:"Iniciar uma chamada",copyInviteLinkButtonLabel:"Copiar o link de convite",copyInviteLinkButtonActionedLabel:"Link copiado",copyInviteLinkActionedAriaLabel:"Link de convite copiado",defaultPlaceHolder:"Selecionar uma opção",dismissSidePaneButtonLabel:"Fechar",videoEffectsPaneTitle:"Efeitos",videoEffectsPaneBackgroundSelectionTitle:"Tela de fundo",videoEffectsPaneAriaLabel:"Painel de efeitos de vídeo",configurationPageCameraIsLoadingLabel:"O vídeo está carregando...",configurationPageVideoEffectsButtonLabel:"Efeitos",unableToStartVideoEffect:"Não é possível aplicar o efeito de vídeo.",blurBackgroundEffectButtonLabel:"Desfoque",blurBackgroundTooltip:"Desfocar a Tela de fundo",removeBackgroundEffectButtonLabel:"Nenhum",removeBackgroundTooltip:"Remover Tela de fundo",cameraOffBackgroundEffectWarningText:"A câmera está desligada. Ligue a câmera para ver o efeito de vídeo.",failedToJoinCallDueToNoNetworkMoreDetails:"A chamada foi desconectada devido a um problema de rede. Verifique sua conexão e ingresse novamente.",failedToJoinCallDueToNoNetworkTitle:"Chamada desconectada",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Você não recebeu entrada na chamada. Se isso foi um erro, ingresse novamente na chamada.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Ignorado do lobby",learnMore:"Saber mais",leavingCallTitle:"Saindo...",leftCallMoreDetails:"Se isso foi um erro, ingresse novamente na chamada.",leftCallTitle:"Você saiu da chamada",lobbyScreenConnectingToCallTitle:"Ingressando na chamada",lobbyScreenWaitingToBeAdmittedTitle:"Esperando para ser admitido",microphonePermissionDenied:"Seu navegador está bloqueando o acesso ao microfone",microphoneToggleInLobbyNotAllowed:"Não é possível ativar ou desativar mudo enquanto estiver no lobby.",mutedMessage:"Você está com o mudo ativado",networkReconnectMoreDetails:"Parece que algo deu errado. Estamos tentando retornar à chamada.",networkReconnectTitle:"Em espera",notInvitedToRoomDetails:"Você não pode ingressar nesta sala porque não tem um convite.",notInvitedToRoomTitle:"Não convidado para a sala",peopleButtonLabel:"Pessoas",selectedPeopleButtonLabel:"Botão Pessoas Selecionado",peoplePaneTitle:"Pessoas",peoplePaneMoreButtonAriaLabel:"Mais",peopleButtonTooltipOpen:"Mostrar participantes",peopleButtonTooltipClose:"Ocultar participantes",peoplePaneSubTitle:"Nessa chamada {numberOfPeople}",privacyPolicy:"Política de privacidade",rejoinCallButtonLabel:"Reingressar na chamada",removedFromCallMoreDetails:"Outro participante removeu você da chamada.",removedFromCallTitle:"Você foi removido",removeMenuLabel:"Remover",startSpotlightMenuLabel:"Destacar para todos",addSpotlightMenuLabel:"Adicionar destaque",stopSpotlightMenuLabel:"Parar de destacar",stopSpotlightOnSelfMenuLabel:"Sair de destaque",spotlightLimitReachedMenuTitle:"Limite de destaque atingido",stopAllSpotlightMenuLabel:"Parar todos os refletores",returnToCallButtonAriaDescription:"Retornar à Chamada",returnToCallButtonAriaLabel:"Voltar",roomNotFoundDetails:"Não foi possível encontrar a ID da sala fornecida.",roomNotFoundTitle:"Sala não encontrada",roomNotValidDetails:"No momento, esta sala não é válida.",roomNotValidTitle:"Sala inválida",inviteToRoomRemovedDetails:"Seu convite para ingressar nesta sala foi removido.",inviteToRoomRemovedTitle:"O convite para a sala foi removido",soundLabel:"Som",noMicrophonesLabel:"Nenhum microfone encontrado",noSpeakersLabel:"Nenhum alto-falante encontrado",startCallButtonLabel:"Iniciar chamada",openDialpadButtonLabel:"Número de telefone de discagem",peoplePaneAddPeopleButtonLabel:"Adicionar Pessoas",dialpadStartCallButtonLabel:"Chamar",dialpadModalTitle:"Número de Telefone de Discagem",dialpadModalAriaLabel:"Teclado de discagem",dialpadCloseModalButtonAriaLabel:"Fechar teclado",moreButtonCallingLabel:"Mais",resumeCallButtonLabel:"Retomar",resumingCallButtonLabel:"Retomando…",resumeCallButtonAriaLabel:"Retomar chamada",resumingCallButtonAriaLabel:"Retomar chamada",holdScreenLabel:"Você está em espera",openDtmfDialpadLabel:"Mostrar teclado de discagem",dtmfDialpadPlaceholderText:"Digite o número",outboundCallingNoticeString:"Chamando...",participantJoinedNoticeString:"{displayName} ingressou",twoParticipantJoinedNoticeString:"{displayName1} e {displayName2} ingressaram",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} e {displayName3} ingressaram",participantLeftNoticeString:"{displayName} saiu",twoParticipantLeftNoticeString:"{displayName1} e {displayName2} saíram",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} e {displayName3} saíram",unnamedParticipantString:"participante sem nome",manyUnnamedParticipantsJoined:"participante sem nome e outros {numOfParticipants} participantes ingressaram",manyUnnamedParticipantsLeft:"participante sem nome e outros {numOfParticipants} participantes saíram",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} e {numOfParticipants} outros participantes ingressaram",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} e {numOfParticipants} outros participantes saíram",liveCaptionsLabel:"Legendas ao Vivo",captionsSettingsLabel:"Configurações de legenda",startCaptionsButtonOnLabel:"Ativar legendas",startCaptionsButtonOffLabel:"Desativar legendas",startCaptionsButtonTooltipOnContent:"Desativar legendas",startCaptionsButtonTooltipOffContent:"Ativar legendas",captionsSettingsModalTitle:"Configurações de Legendas",captionsSettingsSpokenLanguageDropdownLabel:"Idioma falado",captionsSettingsCaptionLanguageDropdownLabel:"Idioma das legendas",captionsSettingsSpokenLanguageDropdownInfoText:"Idioma que todos nesta chamada estão falando.",captionsSettingsCaptionLanguageDropdownInfoText:"As legendas aparecerão neste idioma.",captionsSettingsConfirmButtonLabel:"Confirmar",captionsSettingsCancelButtonLabel:"Cancelar",captionsSettingsModalAriaLabel:"Configuração de Legendas Modal",captionsSettingsCloseModalButtonAriaLabel:"Fechar Configuração de Legendas",captionsBannerMoreButtonCallingLabel:"Mais Opções de Configurações de Legendas",captionsBannerMoreButtonTooltip:"Mais opções",realTimeTextLabel:"Digitação em tempo real (RTT)",startRealTimeTextLabel:"Habilitar a RTT para esta chamada",disabledStartRealTimeTextLabel:"O RTT está ativado no momento, não pode ser desativado",dismissModalAriaLabel:"Imagem e imagem local e remota, pressione Enter para retornar à chamada",callRejectedTitle:"Não foi possível concluir a chamada",callRejectedMoreDetails:"A pessoa que você está tentando acessar não está disponível. Tente novamente mais tarde.",callTimeoutTitle:"Não foi possível concluir a chamada",callTimeoutDetails:"A pessoa que você está tentando acessar não está disponível. Tente novamente mais tarde.",callTimeoutBotTitle:"Não foi possível concluir a chamada",callTimeoutBotDetails:"O serviço que você está tentando acessar não está disponível. Tente novamente mais tarde.",dtmfDialerButtonLabel:"Teclado de discagem",dtmfDialerButtonTooltipOn:"Mostrar teclado de discagem",dtmfDialerButtonTooltipOff:"Ocultar teclado de discagem",dtmfDialerMoreButtonLabelOn:"Mostrar teclado de discagem",dtmfDialerMoreButtonLabelOff:"Ocultar teclado de discagem",phoneCallMoreButtonLabel:"Chamada Telefônica",spokenLanguageStrings:{"ar-ae":"Árabe - Emirados Árabes Unidos","ar-sa":"Árabe - Arábia Saudita","da-dk":"Dinamarquês","de-de":"Alemão - Alemanha","en-au":"Inglês - Austrália","en-ca":"Inglês - Canadá","en-gb":"Inglês - Reino Unido","en-in":"Inglês - Índia","en-nz":"Inglês - Nova Zelândia","en-us":"Inglês - Estados Unidos","es-es":"Espanhol - Espanha (Moderno)","es-mx":"Espanhol - México","fi-fi":"Finlandês","fr-ca":"Francês - Canadá","fr-fr":"Francês - França","hi-in":"Híndi","it-it":"Italiano - Itália","ja-jp":"Japonês","ko-kr":"Coreano","nb-no":"Norueguês (Bokmål)","nl-be":"Holandês - Bélgica","nl-nl":"Holandês - Países Baixos","pl-pl":"Polonês","pt-br":"Português - Brasil","ru-ru":"Russo","sv-se":"Sueco","zh-cn":"Chinês - República Popular da China","zh-hk":"Chinês - RAE de Hong Kong","cs-cz":"Tcheco","pt-pt":"Português - Portugal","tr-tr":"Turco","vi-vn":"Vietnamita","th-th":"Tailandês","he-il":"Hebraico","cy-gb":"Galês","uk-ua":"Ucraniano","el-gr":"Grego","hu-hu":"Húngaro","ro-ro":"Romeno","sk-sk":"Eslovaco","zh-tw":"Chinês - Taiwan"},captionLanguageStrings:{ar:"Árabe",da:"Dinamarquês",de:"Alemão",en:"Inglês",es:"Espanhol",fi:"Finlandês","fr-ca":"Francês - Canadá",fr:"Francês - França",hi:"Híndi",it:"Italiano",ja:"Japonês",ko:"Coreano",nb:"Norueguês (Bokmål)",nl:"Holandês",pl:"Polonês",pt:"Português - Brasil",ru:"Russo",sv:"Sueco","zh-Hans":"Chinês (Simplificado)","zh-Hant":"Chinês (Tradicional)",cs:"Tcheco","pt-pt":"Português - Portugal",tr:"Turco",vi:"Vietnamita",th:"Tailandês",he:"Hebraico",cy:"Galês",uk:"Ucraniano",el:"Grego",hu:"Húngaro",ro:"Romeno",sk:"Eslovaco"},captionsBannerSpinnerText:"Iniciando legendas...",realTimeTextInputBoxDefaultText:"Digitar mensagem em tempo real",realTimeTextInputErrorMessage:"A mensagem não pode ter mais de 2000 caracteres",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"O RTT (texto em tempo real) está habilitado para todos os participantes durante toda a reunião.",realTimeTextBannerLinkLabel:"Saiba mais",realTimeTextModalAriaLabel:"Ativar Modal RTT",realTimeTextModalTitle:"Ativar a RTT?",realTimeTextModalText:"O texto em tempo real (RTT) será ativado para todos na reunião.",realTimeTextConfirmButtonLabel:"Ativar",realTimeTextCancelButtonLabel:"Cancelar",realTimeTextCloseModalButtonAriaLabel:"Fechar Modal RTT",captionsOnlyContainerTitle:"Legendas",realTimeTextOnlyContainerTitle:"Texto em Tempo Real",captionsAndRealTimeTextContainerTitle:"Legendas e Texto em Tempo Real",expandButtonAriaLabel:"Expandir",minimizeButtonAriaLabel:"Recolher",transferPageTransferorText:"Transferindo...",transferPageTransferTargetText:"Conectando...",transferPageUnknownTransferorDisplayName:"Desconhecido",transferPageUnknownTransferTargetDisplayName:"Desconhecido",transferPageNoticeString:"Você está sendo transferido",participantCouldNotBeReachedTitle:"O participante de destino não está disponível no momento",participantCouldNotBeReachedMoreDetails:"Entre em contato com o participante quando ele estiver disponível",permissionToReachTargetParticipantNotAllowedTitle:"A permissão para alcançar o participante de destino não é permitida",permissionToReachTargetParticipantNotAllowedMoreDetails:"Verifique se o participante de destino está no mesmo locatário",unableToResolveTenantTitle:"Não é possível resolve ID do locatário para o participante de destino",unableToResolveTenantMoreDetails:"Verifique se a ID do participante foi inserida corretamente",participantIdIsMalformedTitle:"A ID do participante não está no formato correto",participantIdIsMalformedMoreDetails:"Verifique marcar a ID do participante está no formato correto",moreButtonGalleryControlLabel:"Exibir",moreButtonGalleryPositionToggleLabel:"Mover a galeria para o topo",moreButtonGallerySpeakerLayoutLabel:"Viva-voz",moreButtonGalleryFloatingLocalLayoutLabel:"Dinâmica",moreButtonGalleryDefaultLayoutLabel:"Modo de exibição de galeria",moreButtonGalleryFocusedContentLayoutLabel:"Foco no conteúdo",moreButtonLargeGalleryDefaultLayoutLabel:"Galeria Grande",moreButtonTogetherModeLayoutLabel:"Modo Conferência",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Sua câmera foi desabilitada. Você não pode mais compartilhar vídeo.",grantedDueToMeetingOption:"Sua câmera foi habilitada. Ative-o se quiser compartilhar vídeo."},unmuteMic:{lostDueToMeetingOption:"Seu microfone foi desabilitado. Você não pode mais ativar o som.",grantedDueToMeetingOption:"O seu microfone foi habilitado. Desative o mudo se quiser falar."},shareScreen:{lostDueToRoleChangeToAttendee:"Sua função foi alterada. Algumas ações, como o compartilhamento de conteúdo, não estarão disponíveis para você.",grantedDueToRoleChangeToPresenter:"Você é um apresentador. Você pode compartilhar conteúdo e facilitar a reunião."},hideAttendeeNames:{hideAttendeeNameAttendee:"Seu nome está oculto para outros participantes. Organizadores e apresentadores podem ver seu nome verdadeiro.",hideAttendeeNamePresenter:"Os nomes dos participantes estão ocultos. Somente organizadores e apresentadores podem ver os nomes para proteger a privacidade dos participantes."}},surveyTitle:"Ajude-nos a melhorar",starSurveyHelperText:"Como estava a qualidade da chamada?",starSurveyOneStarText:"A qualidade era ruim.",starSurveyTwoStarText:"A qualidade era ruim.",starSurveyThreeStarText:"A qualidade era boa.",starSurveyFourStarText:"A qualidade era ótima.",starSurveyFiveStarText:"A qualidade estava excelente.",surveyConfirmButtonLabel:"Enviar Comentários",starRatingAriaLabel:"Selecione {0} de {1} estrelas",surveySkipButtonLabel:"Ignorar",tagsSurveyQuestion:"O que poderia ter sido melhor?",tagsSurveyTextFieldDefaultText:"Outro; especifique",tagsSurveyHelperText:"Marque quaisquer problemas que você tenha enfrentado.",endOfSurveyText:"Agradecemos seus comentários!",surveyIssues:{overallRating:{callCannotJoin:"Não foi possível ingressar na chamada",callCannotInvite:"Não foi possível convidar outras pessoas para a chamada",hadToRejoin:"Eu tive que reingressar na chamada",callEndedUnexpectedly:"A chamada foi encerrada inesperadamente para mim",otherIssues:"Eu tive outros problemas com a chamada"},audioRating:{noLocalAudio:"O outro lado não conseguiu ouvir nenhum som",noRemoteAudio:"Não consegui ouvir o som",echo:"Ouvi ecos na chamada",audioNoise:"Ouvi ruído na chamada",lowVolume:"O volume estava baixo",audioStoppedUnexpectedly:"O áudio foi interrompido inesperadamente",distortedSpeech:"O áudio estava distorcido",audioInterruption:"O áudio foi interrompido",otherIssues:"Eu tive outros problemas de áudio nesta chamada"},videoRating:{noVideoReceived:"Não consegui ver o vídeo",noVideoSent:"Outras pessoas não puderam me ver",lowQuality:"A qualidade do vídeo estava baixa",freezes:"O vídeo ficou congelado",stoppedUnexpectedly:"O vídeo foi interrompido inesperadamente",darkVideoReceived:"Só consigo ver telas escuras quando outras pessoas ativam a câmera",audioVideoOutOfSync:"O Áudio e o Vídeo estavam fora de sincronia",otherIssues:"Eu tive outros problemas de vídeo nesta chamada"},screenshareRating:{noContentLocal:"Outras pessoas não puderam ver meu compartilhamento de tela",noContentRemote:"Não foi possível ver o compartilhamento de tela de outras pessoas",cannotPresent:"Não foi possível apresentar minha tela",lowQuality:"A qualidade do compartilhamento de tela estava baixa",freezes:"O compartilhamento de tela ficou congelado",stoppedUnexpectedly:"O compartilhamento de tela foi interrompido inesperadamente",largeDelay:"O compartilhamento de tela tem um grande atraso",otherIssues:"Eu estava tendo outros problemas de compartilhamento de tela nesta chamada"}},surveyIssuesHeadingStrings:{overallRating:"De modo geral",audioRating:"Áudio",videoRating:"Vídeo",screenshareRating:"Apresentando"},spotlightPrompt:{startSpotlightHeading:"Destacar para todo mundo?",startSpotlightText:"Você vai realçar este vídeo para todos os participantes da reunião.",startSpotlightOnSelfText:"Você realçará seu vídeo para todos os participantes da reunião.",startSpotlightConfirmButtonLabel:"Destacar para todo mundo",startSpotlightCancelButtonLabel:"Cancelar",stopSpotlightHeading:"Parar de dar destaque a este vídeo?",stopSpotlightOnSelfHeading:"Sair do destaque?",stopAllSpotlightHeading:"Parar de destacar todos os vídeos?",stopSpotlightText:"Esse vídeo não será mais realçado para todos os participantes da reunião.",stopSpotlightOnSelfText:"Seu vídeo não será mais realçado para todos os participantes da reunião.",stopAllSpotlightText:"Os vídeos não serão mais destacados para todos na reunião.",stopSpotlightConfirmButtonLabel:"Parar de destacar",stopSpotlightOnSelfConfirmButtonLabel:"Sair do destaque",stopSpotlightCancelButtonLabel:"Cancelar",closeSpotlightPromptButtonLabel:"Fechar"},exitSpotlightButtonLabel:"Sair do destaque",exitSpotlightButtonTooltip:"Sair do destaque",leaveConfirmButtonLabel:"Sair",endCallConfirmButtonLabel:"Encerrar chamada",hangUpCancelButtonLabel:"Cancelar",leaveConfirmDialogTitle:"Sair da chamada?",leaveConfirmDialogContent:"Tem certeza de que deseja sair da chamada?",endCallConfirmDialogTitle:"Encerrar a chamada?",endCallConfirmDialogContent:"Você encerrará a chamada para todos.",invalidMeetingIdentifier:"ID ou senha de reunião inválida",pinParticipantMenuLabel:"Fixar para mim",pinParticipantLimitReachedMenuLabel:"Fixar (limite alcançado)",unpinParticipantMenuLabel:"Desafixar",pinParticipantMenuItemAriaLabel:"Fixar para mim",unpinParticipantMenuItemAriaLabel:"Desafixar",muteAllMenuLabel:"Ativar mudo de todos",muteAllDialogTitle:"Ativar mudo para todos?",muteAllDialogContent:"Isso ativará o mudo para todas as pessoas na reunião, exceto para você.",muteAllConfirmButtonLabel:"Ativar mudo de todos",muteAllCancelButtonLabel:"Cancelar",joinBreakoutRoomButtonLabel:"Ingressar na sala da sessão de grupo",returnFromBreakoutRoomButtonLabel:"Sair da sala",leaveBreakoutRoomAndMeetingButtonLabel:"Sair da reunião",breakoutRoomJoinedNotificationTitle:"Você ingressou na sala para sessão de grupo '{roomName}'",joinBreakoutRoomBannerTitle:"A sala para sessão de grupo '{roomName}' está aberta",joinBreakoutRoomBannerButtonLabel:"Ingressar",returnFromBreakoutRoomBannerTitle:"Voltar à reunião principal",returnFromBreakoutRoomBannerButtonLabel:"Reingressar",forbidAudioMenuLabel:"Desligar microfone",permitAudioMenuLabel:"Permitir microfone",forbidOthersAudioDialogTitle:"Desligar microfone para os participantes?",forbidOthersAudioDialogContent:"Participantes não poderão ativar o som. O organizador e os apresentadores podem ativar o som das pessoas conforme necessário.",forbidOthersAudioConfirmButtonLabel:"Desligar microfones",forbidOthersAudioCancelButtonLabel:"Cancelar",permitOthersAudioDialogTitle:"Permitir microfone para os participantes?",permitOthersAudioDialogContent:"Todos na reunião poderão desativar o mudo.",permitOthersAudioConfirmButtonLabel:"Permitir microfones",permitOthersAudioCancelButtonLabel:"Cancelar",forbidOthersAudioMenuLabel:"Desligar microfone para os participantes",permitOthersAudioMenuLabel:"Permitir microfone para os participantes",forbidVideoMenuLabel:"Desabilitar câmera",permitVideoMenuLabel:"Permitir câmera",forbidOthersVideoDialogTitle:"Desabilitar câmera para os participantes?",forbidOthersVideoDialogContent:"Os participantes não poderão ativar o vídeo por conta própria. O organizador e os apresentadores podem permitir que as pessoas ativem a câmera conforme necessário.",forbidOthersVideoConfirmButtonLabel:"Desabilitar câmeras",forbidOthersVideoCancelButtonLabel:"Cancelar",permitOthersVideoDialogTitle:"Permitir câmera para os participantes?",permitOthersVideoDialogContent:"Todos na reunião poderão ativar o vídeo por conta própria.",permitOthersVideoConfirmButtonLabel:"Permitir câmeras",permitOthersVideoCancelButtonLabel:"Cancelar",forbidOthersVideoMenuLabel:"Desabilitar a câmera para os participantes",permitOthersVideoMenuLabel:"Permitir câmera para os participantes"};var chat$5={chatListHeader:"Neste chat",uploadAttachment:"Carregar anexo",uploadImageDataNotProvided:"Os dados da imagem não foram fornecidos.",uploadImageIsTooLarge:"A imagem é muito grande. Escolha um que seja menor que {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"O upload da imagem .{imageExtension} não é permitido.",uploadImageFailed:"Não é possível carregar a imagem. Tente novamente mais tarde."};var callWithChat$5={chatButtonLabel:"Chat",chatButtonNewMessageNotificationLabel:"Nova mensagem",chatButtonTooltipClosedWithMessageCount:"Mostrar chat ({unreadMessagesCount} não lido)",chatButtonTooltipClose:"Ocultar chat",chatButtonTooltipOpen:"Mostrar chat",chatPaneTitle:"Chat",copyInviteLinkButtonLabel:"Copiar o link de convite",copyInviteLinkButtonActionedLabel:"Link copiado",copyInviteLinkActionedAriaLabel:"Link de convite copiado",dismissSidePaneButtonLabel:"Fechar",moreDrawerAudioDeviceMenuTitle:"Dispositivo de Áudio",moreDrawerButtonLabel:"Mais opções",moreDrawerButtonTooltip:"Mais opções",moreDrawerMicrophoneMenuTitle:"Microfone",moreDrawerSpeakerMenuTitle:"Alto-falante",moreDrawerCaptionsMenuTitle:"Legendas ao Vivo",moreDrawerSpokenLanguageMenuTitle:"Idioma falado",moreDrawerCaptionLanguageMenuTitle:"Idioma da legenda",moreDrawerGalleryOptionsMenuTitle:"Opções da galeria",peopleButtonLabel:"Pessoas",selectedPeopleButtonLabel:"Botão Pessoas Selecionado",peopleButtonTooltipOpen:"Mostrar Participantes",peopleButtonTooltipClose:"Ocultar participantes",peopleButtonTooltipOpenAriaLabel:"Botão Pessoas, Mostrar participantes",peopleButtonTooltipCloseAriaLabel:"Botão Pessoas, Ocultar participantes",peoplePaneSubTitle:"Nesta chamada",peoplePaneTitle:"Pessoas",pictureInPictureTileAriaLabel:"Feeds de vídeo. Clique para retornar à tela de chamada.",removeMenuLabel:"Remover",openDialpadButtonLabel:"Número de telefone de discagem",returnToCallButtonAriaDescription:"Retornar à Chamada",returnToCallButtonAriaLabel:"Voltar",peoplePaneAddPeopleButtonLabel:"Adicionar Pessoas",dialpadStartCallButtonLabel:"Chamar",dialpadModalTitle:"Número de Telefone de Discagem",dialpadModalAriaLabel:"Teclado de discagem",dialpadCloseModalButtonAriaLabel:"Fechar teclado",openDtmfDialpadLabel:"Mostrar teclado de discagem",dtmfDialpadPlaceholderText:"Digite o número",breakoutRoomChatPaneTitle:"Chat da Sala para Sessão de Grupo",chatContentSpinnerLabel:"Carregando chat"};var pt_BR = {call:call$5,chat:chat$5,callWithChat:callWithChat$5};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Portuguese (Brazil)
 *
 * @public
 */
const COMPOSITE_LOCALE_PT_BR = {
    component: COMPONENT_LOCALE_PT_BR,
    strings: createCompositeStrings(pt_BR)
};

var call$4={cameraLabel:"Камера",noCamerasLabel:"Камеры не найдены",cameraPermissionDenied:"Браузер блокирует доступ к вашей камере",cameraTurnedOff:"Камера выключена",chatButtonLabel:"Чат",close:"Закрыть",complianceBannerNowOnlyRecording:"Сейчас для этого собрания ведется только запись.",complianceBannerNowOnlyTranscription:"Сейчас для этого собрания ведется только транскрибирование.",complianceBannerRecordingAndTranscriptionSaved:"Сохранение записи и транскрибирования.",complianceBannerRecordingAndTranscriptionStarted:"Начата запись и транскрибирование.",complianceBannerRecordingAndTranscriptionStopped:"Запись и транскрибирование остановлены.",complianceBannerRecordingSaving:"Сохранение записи.",complianceBannerRecordingStarted:"Запись начата.",complianceBannerRecordingStopped:"Запись остановлена.",complianceBannerTranscriptionStarted:"Транскрибирование запущено.",complianceBannerTranscriptionConsent:"Присоединяясь, вы даете согласие на транскрибирование этого собрания.",complianceBannerTranscriptionSaving:"Сохранение транскрибирования.",complianceBannerTranscriptionStopped:"Транскрибирование остановлено.",configurationPageTitle:"Позвонить",copyInviteLinkButtonLabel:"Копировать ссылку с приглашением",copyInviteLinkButtonActionedLabel:"Ссылка скопирована",copyInviteLinkActionedAriaLabel:"Ссылка на приглашение в группу скопирована",defaultPlaceHolder:"Выберите один из вариантов",dismissSidePaneButtonLabel:"Закрыть",videoEffectsPaneTitle:"Эффекты",videoEffectsPaneBackgroundSelectionTitle:"Фон",videoEffectsPaneAriaLabel:"Панель видеоэффектов",configurationPageCameraIsLoadingLabel:"Загрузка видео...",configurationPageVideoEffectsButtonLabel:"Эффекты",unableToStartVideoEffect:"Не удалось применить эффект видео.",blurBackgroundEffectButtonLabel:"Размытие",blurBackgroundTooltip:"Размытие фона",removeBackgroundEffectButtonLabel:"Нет",removeBackgroundTooltip:"Удалить фон",cameraOffBackgroundEffectWarningText:"Камера выключена. Включите камеру, чтобы увидеть видеоэффект.",failedToJoinCallDueToNoNetworkMoreDetails:"Звонок был отключен из-за проблемы с сетью. Проверьте подключение и присоединитесь еще раз.",failedToJoinCallDueToNoNetworkTitle:"Вызов прерван",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Вам не предоставили доступ к беседе. Если это произошло по ошибке, заново присоединитесь к беседе.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Вы были удалены из \"зала ожидания\"",learnMore:"Подробнее",leavingCallTitle:"Выход...",leftCallMoreDetails:"Если это произошло по ошибке, заново присоединитесь к беседе.",leftCallTitle:"Вы покинули беседу",lobbyScreenConnectingToCallTitle:"Подключение к звонку",lobbyScreenWaitingToBeAdmittedTitle:"Ожидание допуска",microphonePermissionDenied:"Браузер блокирует доступ к вашему микрофону",microphoneToggleInLobbyNotAllowed:"Невозможно отключить или включить звук, находясь в \"зале ожидания\".",mutedMessage:"Ваш звук отключен",networkReconnectMoreDetails:"Похоже, что-то пошло не так. Мы пытаемся вернуться к звонку.",networkReconnectTitle:"Подождите",notInvitedToRoomDetails:"Вы не можете присоединиться к этой комнате, так как у вас нет приглашения.",notInvitedToRoomTitle:"Нет приглашения в комнату",peopleButtonLabel:"Люди",selectedPeopleButtonLabel:"Нажата кнопка \"Люди\"",peoplePaneTitle:"Люди",peoplePaneMoreButtonAriaLabel:"Еще",peopleButtonTooltipOpen:"Показать участников",peopleButtonTooltipClose:"Скрыть участников",peoplePaneSubTitle:"В этом вызове {numberOfPeople}",privacyPolicy:"Политика конфиденциальности",rejoinCallButtonLabel:"Повторно присоединиться к звонку",removedFromCallMoreDetails:"Другой участник удалил вас из беседы.",removedFromCallTitle:"Вы были удалены",removeMenuLabel:"Удалить",startSpotlightMenuLabel:"Закрепить видео для всех",addSpotlightMenuLabel:"Добавить закрепление",stopSpotlightMenuLabel:"Не закреплять",stopSpotlightOnSelfMenuLabel:"Открепить",spotlightLimitReachedMenuTitle:"Достигнуто предельное число закреплений",stopAllSpotlightMenuLabel:"Отменить все закрепления",returnToCallButtonAriaDescription:"Вернуться к звонку",returnToCallButtonAriaLabel:"Назад",roomNotFoundDetails:"Не удалось найти указанный идентификатор комнаты.",roomNotFoundTitle:"Помещение не найдено",roomNotValidDetails:"В данный момент эта комната недопустима.",roomNotValidTitle:"Недопустимая комната",inviteToRoomRemovedDetails:"Ваше приглашение присоединиться к этой комнате было удалено.",inviteToRoomRemovedTitle:"Приглашение в комнату удалено",soundLabel:"Звук",noMicrophonesLabel:"Микрофоны не найдены",noSpeakersLabel:"Динамики не найдены",startCallButtonLabel:"Начать звонок",openDialpadButtonLabel:"Набрать номер телефона",peoplePaneAddPeopleButtonLabel:"Добавить людей",dialpadStartCallButtonLabel:"Позвонить",dialpadModalTitle:"Набор номера телефона",dialpadModalAriaLabel:"Панель набора номера",dialpadCloseModalButtonAriaLabel:"Закрыть панель набора номера",moreButtonCallingLabel:"Дополнительно",resumeCallButtonLabel:"Возобновить",resumingCallButtonLabel:"Возобновление…",resumeCallButtonAriaLabel:"Возобновить звонок",resumingCallButtonAriaLabel:"Возобновить вызов",holdScreenLabel:"Ваш звонок на удержании",openDtmfDialpadLabel:"Показать панель набора номера",dtmfDialpadPlaceholderText:"Введите номер",outboundCallingNoticeString:"Выполняется вызов...",participantJoinedNoticeString:"Пользователь {displayName} присоединился",twoParticipantJoinedNoticeString:"{displayName1} и {displayName2} присоединились",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} и {displayName3} присоединились",participantLeftNoticeString:"Пользователь {displayName} ушел",twoParticipantLeftNoticeString:"{displayName1} и {displayName2} ушли",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} и {displayName3} ушли",unnamedParticipantString:"участник без имени",manyUnnamedParticipantsJoined:"участник без имени и еще {numOfParticipants} участников присоединились",manyUnnamedParticipantsLeft:"участник без имени и еще {numOfParticipants} участников ушли",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} и {numOfParticipants} других участников",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} и {numOfParticipants} других участников",liveCaptionsLabel:"Автоматические субтитры",captionsSettingsLabel:"Параметры субтитров",startCaptionsButtonOnLabel:"Включить автоматические субтитры",startCaptionsButtonOffLabel:"Отключить автоматические субтитры",startCaptionsButtonTooltipOnContent:"Отключить автоматические субтитры",startCaptionsButtonTooltipOffContent:"Включить автоматические субтитры",captionsSettingsModalTitle:"Параметры субтитров",captionsSettingsSpokenLanguageDropdownLabel:"Язык речи",captionsSettingsCaptionLanguageDropdownLabel:"Язык субтитров",captionsSettingsSpokenLanguageDropdownInfoText:"Язык, на котором говорят все участники звонка.",captionsSettingsCaptionLanguageDropdownInfoText:"Субтитры будут отображаться на этом языке.",captionsSettingsConfirmButtonLabel:"Подтвердить",captionsSettingsCancelButtonLabel:"Отменить",captionsSettingsModalAriaLabel:"Модальное окно параметров субтитров",captionsSettingsCloseModalButtonAriaLabel:"Закрыть параметры субтитров",captionsBannerMoreButtonCallingLabel:"Дополнительные возможности субтитров",captionsBannerMoreButtonTooltip:"Дополнительные параметры",realTimeTextLabel:"Текст в реальном времени (RTT)",startRealTimeTextLabel:"Включить ввод в режиме реального времени для этого звонка",disabledStartRealTimeTextLabel:"Ввод в режиме реального времени в настоящее время включен, его нельзя отключить",dismissModalAriaLabel:"Локальное и удаленное изображение. Нажмите клавишу ВВОД, чтобы вернуться к звонку.",callRejectedTitle:"Не удалось совершить звонок",callRejectedMoreDetails:"Пользователь, с которым вы пытаетесь связаться, недоступен. Повторите попытку позже.",callTimeoutTitle:"Не удалось совершить звонок",callTimeoutDetails:"Пользователь, с которым вы пытаетесь связаться, недоступен. Повторите попытку позже.",callTimeoutBotTitle:"Не удалось совершить звонок",callTimeoutBotDetails:"Служба, к которому вы пытаетесь подключиться, недоступна. Повторите попытку позже.",dtmfDialerButtonLabel:"Панель набора номера",dtmfDialerButtonTooltipOn:"Показать панель набора номера",dtmfDialerButtonTooltipOff:"Скрыть панель набора номера",dtmfDialerMoreButtonLabelOn:"Показать панель набора номера",dtmfDialerMoreButtonLabelOff:"Скрыть панель набора номера",phoneCallMoreButtonLabel:"Звонок",spokenLanguageStrings:{"ar-ae":"Арабский (ОАЭ)","ar-sa":"Арабский (Саудовская Аравия)","da-dk":"Датский","de-de":"Немецкий (Германия)","en-au":"Английский (Австралия)","en-ca":"Английский (Канада)","en-gb":"Английский (Соединенное Королевство)","en-in":"Английский (Индия)","en-nz":"Английский (Новая Зеландия)","en-us":"Английский (США)","es-es":"Испанский (Испания), современная сортировка","es-mx":"Испанский (Мексика)","fi-fi":"Финский","fr-ca":"Французский (Канада)","fr-fr":"Французский (Франция)","hi-in":"Хинди","it-it":"Итальянский (Италия)","ja-jp":"Японский","ko-kr":"Корейский","nb-no":"Норвежский (букмол)","nl-be":"Нидерландский (Бельгия)","nl-nl":"Нидерландский — Нидерланды","pl-pl":"Польский","pt-br":"Португальский (Бразилия)","ru-ru":"Русский","sv-se":"Шведский","zh-cn":"Китайский (Китайская Народная Республика)","zh-hk":"Китайский (Гонконг, САР)","cs-cz":"Чешский","pt-pt":"Португальский (Португалия)","tr-tr":"Турецкий","vi-vn":"Вьетнамский","th-th":"Тайский","he-il":"Иврит","cy-gb":"Валлийский","uk-ua":"Украинский","el-gr":"Греческий","hu-hu":"Венгерский","ro-ro":"Румынский","sk-sk":"Словацкий","zh-tw":"Китайский (Тайвань)"},captionLanguageStrings:{ar:"Арабский",da:"Датский",de:"Немецкий",en:"Английский",es:"Испанский",fi:"Финский","fr-ca":"Французский (Канада)",fr:"Французский (Франция)",hi:"Хинди",it:"Итальянский",ja:"Японский",ko:"Корейский",nb:"Норвежский (букмол)",nl:"Нидерландский",pl:"Польский",pt:"Португальский (Бразилия)",ru:"Русский",sv:"Шведский","zh-Hans":"китайский, упрощенное письмо","zh-Hant":"Китайский (традиционное письмо)",cs:"Чешский","pt-pt":"Португальский (Португалия)",tr:"Турецкий",vi:"Вьетнамский",th:"Тайский",he:"Иврит",cy:"Валлийский",uk:"Украинский",el:"Греческий",hu:"Венгерский",ro:"Румынский",sk:"Словацкий"},captionsBannerSpinnerText:"Запуск субтитров...",realTimeTextInputBoxDefaultText:"Введите сообщение в режиме реального времени",realTimeTextInputErrorMessage:"Длина сообщения не может превышать 2000 символов",realTimeTextBannerTitle:"Ввод в режиме реального времени",realTimeTextBannerContent:"Ввод в режиме реального времени включен для всех участников в течение всего собрания.",realTimeTextBannerLinkLabel:"Подробнее",realTimeTextModalAriaLabel:"Модальное окно включения ввода в режиме реального времени",realTimeTextModalTitle:"Включить ввод в режиме реального времени?",realTimeTextModalText:"Текст в режиме реального времени (RTT) будет включен для всех участников собрания.",realTimeTextConfirmButtonLabel:"Включить",realTimeTextCancelButtonLabel:"Отмена",realTimeTextCloseModalButtonAriaLabel:"Закрыть модальное окно ввода в режиме реального времени",captionsOnlyContainerTitle:"Субтитры",realTimeTextOnlyContainerTitle:"Текст в режиме реального времени",captionsAndRealTimeTextContainerTitle:"Субтитры и текст в режиме реального времени",expandButtonAriaLabel:"Развернуть",minimizeButtonAriaLabel:"Свернуть",transferPageTransferorText:"Идет переключение...",transferPageTransferTargetText:"Подключение...",transferPageUnknownTransferorDisplayName:"Неизвестно",transferPageUnknownTransferTargetDisplayName:"Неизвестно",transferPageNoticeString:"Для вас выполняется передача",participantCouldNotBeReachedTitle:"Целевой участник сейчас недоступен",participantCouldNotBeReachedMoreDetails:"Обратитесь к участнику, когда он будет доступен",permissionToReachTargetParticipantNotAllowedTitle:"Разрешение на связь с целевым участником отсутствует",permissionToReachTargetParticipantNotAllowedMoreDetails:"Проверьте, что целевой участник находится в том же клиенте",unableToResolveTenantTitle:"Не удалось разрешить идентификатор клиента для целевого участника",unableToResolveTenantMoreDetails:"Проверьте, что идентификатор участника введен правильно",participantIdIsMalformedTitle:"Неправильный формат идентификатора участника",participantIdIsMalformedMoreDetails:"Проверьте, что идентификатор участника указан в правильном формате",moreButtonGalleryControlLabel:"Просмотр",moreButtonGalleryPositionToggleLabel:"Переместить коллекцию вверх",moreButtonGallerySpeakerLayoutLabel:"Докладчик",moreButtonGalleryFloatingLocalLayoutLabel:"Динамический",moreButtonGalleryDefaultLayoutLabel:"Представление галереи",moreButtonGalleryFocusedContentLayoutLabel:"Фокусировка на содержимом",moreButtonLargeGalleryDefaultLayoutLabel:"Большая галерея",moreButtonTogetherModeLayoutLabel:"Режим \"Вместе\"",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Камера отключена. Вы больше не можете делиться видео.",grantedDueToMeetingOption:"Камера включена. Включите этот параметр, если хотите поделиться видео."},unmuteMic:{lostDueToMeetingOption:"Ваш микрофон отключен. Вы больше не можете включить звук.",grantedDueToMeetingOption:"Ваш микрофон подключен. Включите звук, если хотите выступить."},shareScreen:{lostDueToRoleChangeToAttendee:"Ваша роль изменилась. Некоторые действия, например общий доступ к содержимому, будут вам недоступны.",grantedDueToRoleChangeToPresenter:"Вы ведущий. Вы можете предоставлять доступ к содержимому и помогать вести собрание."},hideAttendeeNames:{hideAttendeeNameAttendee:"Ваше имя скрыто от других участников. Организаторы и выступающие могут видеть ваше настоящее имя.",hideAttendeeNamePresenter:"Имена участников скрыты. Для защиты конфиденциальности участников имена видят только организаторы и выступающие."}},surveyTitle:"Улучшение продукта",starSurveyHelperText:"Как вы оцениваете качество звонка?",starSurveyOneStarText:"Качество было очень плохое.",starSurveyTwoStarText:"Качество было плохое.",starSurveyThreeStarText:"Качество было хорошее.",starSurveyFourStarText:"Качество было отличное.",starSurveyFiveStarText:"Качество было превосходное.",surveyConfirmButtonLabel:"Отправить отзыв",starRatingAriaLabel:"Выберите {0} из {1} звезд",surveySkipButtonLabel:"Пропустить",tagsSurveyQuestion:"Что можно было улучшить?",tagsSurveyTextFieldDefaultText:"Другое, укажите",tagsSurveyHelperText:"Отметьте все проблемы, с которыми вы столкнулись",endOfSurveyText:"Спасибо за отзыв!",surveyIssues:{overallRating:{callCannotJoin:"Не удалось присоединиться к звонку",callCannotInvite:"Не удалось пригласить других участников в звонок",hadToRejoin:"Мне пришлось повторно присоединиться к звонку",callEndedUnexpectedly:"Звонок неожиданно завершился",otherIssues:"У меня возникли другие проблемы с вызовом"},audioRating:{noLocalAudio:"У моего собеседника не было звука",noRemoteAudio:"У меня не было звука.",echo:"Во время звонка было слышно эхо",audioNoise:"Во время звонка был слышен шум",lowVolume:"Слишком низкая громкость",audioStoppedUnexpectedly:"Звук неожиданно остановился",distortedSpeech:"Звук искажен",audioInterruption:"Звук был прерван",otherIssues:"У меня возникли другие проблемы со звуком во время этого звонка"},videoRating:{noVideoReceived:"У меня не отображалось видео",noVideoSent:"Другие пользователи не слышали меня",lowQuality:"Низкое качество видео",freezes:"Видео зависло",stoppedUnexpectedly:"Видео неожиданно остановилось",darkVideoReceived:"Я вижу темные экраны, когда другие включают камеру",audioVideoOutOfSync:"Звук и видео не синхронизированы",otherIssues:"У меня возникли другие проблемы с видео во время этого звонка"},screenshareRating:{noContentLocal:"Другие пользователи не могут видеть мою демонстрацию экрана",noContentRemote:"Не удалось увидеть демонстрацию экрана других людей",cannotPresent:"Не удалось показать экран",lowQuality:"Низкое качество демонстрации экрана",freezes:"Демонстрация экрана зависла",stoppedUnexpectedly:"Демонстрация экрана неожиданно остановилась",largeDelay:"Демонстрация экрана имеет большую задержку",otherIssues:"У меня возникли другие проблемы с демонстрацией экрана во время этого звонка"}},surveyIssuesHeadingStrings:{overallRating:"Общий",audioRating:"Звук",videoRating:"Видео",screenshareRating:"Выступление"},spotlightPrompt:{startSpotlightHeading:"Пометить как важное для всех?",startSpotlightText:"Вы выделите это видео для всех участников собрания.",startSpotlightOnSelfText:"Вы выделите свое видео для всех участников собрания.",startSpotlightConfirmButtonLabel:"Отметить видео как важное для всех",startSpotlightCancelButtonLabel:"Отмена",stopSpotlightHeading:"Больше не рекомендовать это видео?",stopSpotlightOnSelfHeading:"Выйти из работы с важным?",stopAllSpotlightHeading:"Отменить закрепление всех видео?",stopSpotlightText:"Ваше видео больше не будет помечено как важное для всех участников собрания.",stopSpotlightOnSelfText:"Ваше видео больше не будет выделено для всех участников собрания.",stopAllSpotlightText:"Видео больше не будут выделены для всех участников собрания.",stopSpotlightConfirmButtonLabel:"Больше не помечать важное",stopSpotlightOnSelfConfirmButtonLabel:"Выйти из важного",stopSpotlightCancelButtonLabel:"Отмена",closeSpotlightPromptButtonLabel:"Закрыть"},exitSpotlightButtonLabel:"Выйти из важного",exitSpotlightButtonTooltip:"Выйти из важного",leaveConfirmButtonLabel:"Выйти",endCallConfirmButtonLabel:"Завершить звонок",hangUpCancelButtonLabel:"Отмена",leaveConfirmDialogTitle:"Покинуть звонок?",leaveConfirmDialogContent:"Вы действительно хотите покинуть звонок?",endCallConfirmDialogTitle:"Завершить звонок?",endCallConfirmDialogContent:"Вы завершите звонок для всех пользователей.",invalidMeetingIdentifier:"Недопустимый идентификатор собрания или секретный код",pinParticipantMenuLabel:"Закрепить для меня",pinParticipantLimitReachedMenuLabel:"Закрепить (достигнуто ограничение)",unpinParticipantMenuLabel:"Открепить",pinParticipantMenuItemAriaLabel:"Закрепить для меня",unpinParticipantMenuItemAriaLabel:"Открепить",muteAllMenuLabel:"Отключить все микрофоны",muteAllDialogTitle:"Отключить микрофоны всех участников?",muteAllDialogContent:"Будут отключены микрофоны всех участников собрания, кроме вас.",muteAllConfirmButtonLabel:"Отключить все микрофоны",muteAllCancelButtonLabel:"Отмена",joinBreakoutRoomButtonLabel:"Присоединиться к переговорной комнате",returnFromBreakoutRoomButtonLabel:"Покинуть комнату",leaveBreakoutRoomAndMeetingButtonLabel:"Покинуть собрание",breakoutRoomJoinedNotificationTitle:"Вы присоединились к переговорной комнате \"{roomName}\"",joinBreakoutRoomBannerTitle:"Переговорная комната \"{roomName}\" открыта",joinBreakoutRoomBannerButtonLabel:"Присоединиться",returnFromBreakoutRoomBannerTitle:"Вернуться в главное собрание",returnFromBreakoutRoomBannerButtonLabel:"Присоединиться снова",forbidAudioMenuLabel:"Отключить микрофон",permitAudioMenuLabel:"Разрешить использовать микрофон",forbidOthersAudioDialogTitle:"Отключить микрофон для участников?",forbidOthersAudioDialogContent:"Участники не смогут самостоятельно включать звук. Организатор и выступающие могут разрешить участникам включать звук при необходимости.",forbidOthersAudioConfirmButtonLabel:"Отключить микрофоны",forbidOthersAudioCancelButtonLabel:"Отмена",permitOthersAudioDialogTitle:"Разрешить участникам использовать микрофон?",permitOthersAudioDialogContent:"Все участники собрания смогут включить свой микрофон.",permitOthersAudioConfirmButtonLabel:"Разрешить использовать микрофоны",permitOthersAudioCancelButtonLabel:"Отмена",forbidOthersAudioMenuLabel:"Отключить микрофон для участников",permitOthersAudioMenuLabel:"Разрешить участникам использовать микрофон",forbidVideoMenuLabel:"Выключить камеру",permitVideoMenuLabel:"Разрешить использовать камеру",forbidOthersVideoDialogTitle:"Отключить камеру для участников?",forbidOthersVideoDialogContent:"Участники не смогут сами включать видео. Организатор и выступающие могут разрешить участникам включать камеру при необходимости.",forbidOthersVideoConfirmButtonLabel:"Отключить камеры",forbidOthersVideoCancelButtonLabel:"Отмена",permitOthersVideoDialogTitle:"Разрешить участникам использовать камеру?",permitOthersVideoDialogContent:"Все участники собрания смогут включить видео самостоятельно.",permitOthersVideoConfirmButtonLabel:"Разрешить использовать камеры",permitOthersVideoCancelButtonLabel:"Отмена",forbidOthersVideoMenuLabel:"Отключить камеру для участников",permitOthersVideoMenuLabel:"Разрешить участникам использовать камеру"};var chat$4={chatListHeader:"В этом чате",uploadAttachment:"Отправить вложение",uploadImageDataNotProvided:"Данные образа не предоставлены.",uploadImageIsTooLarge:"Изображение слишком велико. Выберите размер меньше {maxImageSize} МБ.",uploadImageExtensionIsNotAllowed:"Отправка образа .{imageExtension} не разрешена.",uploadImageFailed:"Не удалось отправить изображение. Повторите попытку позже."};var callWithChat$4={chatButtonLabel:"Чат",chatButtonNewMessageNotificationLabel:"Новое сообщение",chatButtonTooltipClosedWithMessageCount:"Показать чат (непрочитано: {unreadMessagesCount})",chatButtonTooltipClose:"Скрыть чат",chatButtonTooltipOpen:"Показать чат",chatPaneTitle:"Чат",copyInviteLinkButtonLabel:"Копировать ссылку с приглашением",copyInviteLinkButtonActionedLabel:"Ссылка скопирована",copyInviteLinkActionedAriaLabel:"Ссылка на приглашение в группу скопирована",dismissSidePaneButtonLabel:"Закрыть",moreDrawerAudioDeviceMenuTitle:"Звуковое устройство",moreDrawerButtonLabel:"Дополнительные параметры",moreDrawerButtonTooltip:"Дополнительные параметры",moreDrawerMicrophoneMenuTitle:"Микрофон",moreDrawerSpeakerMenuTitle:"Динамик",moreDrawerCaptionsMenuTitle:"Автоматические субтитры",moreDrawerSpokenLanguageMenuTitle:"Язык речи",moreDrawerCaptionLanguageMenuTitle:"Язык субтитров",moreDrawerGalleryOptionsMenuTitle:"Параметры коллекции",peopleButtonLabel:"Люди",selectedPeopleButtonLabel:"Нажата кнопка \"Люди\"",peopleButtonTooltipOpen:"Показать участников",peopleButtonTooltipClose:"Скрыть участников",peopleButtonTooltipOpenAriaLabel:"Кнопка \"Люди\", показать участников",peopleButtonTooltipCloseAriaLabel:"Кнопка \"Люди\", скрыть участников",peoplePaneSubTitle:"В этом звонке",peoplePaneTitle:"Люди",pictureInPictureTileAriaLabel:"Ленты видео. Щелкните, чтобы вернуться на экран вызова.",removeMenuLabel:"Удалить",openDialpadButtonLabel:"Набрать номер телефона",returnToCallButtonAriaDescription:"Вернуться к звонку",returnToCallButtonAriaLabel:"Назад",peoplePaneAddPeopleButtonLabel:"Добавить людей",dialpadStartCallButtonLabel:"Позвонить",dialpadModalTitle:"Набор номера телефона",dialpadModalAriaLabel:"Панель набора номера",dialpadCloseModalButtonAriaLabel:"Закрыть панель набора номера",openDtmfDialpadLabel:"Показать панель набора номера",dtmfDialpadPlaceholderText:"Введите номер",breakoutRoomChatPaneTitle:"Чат переговорной комнаты",chatContentSpinnerLabel:"Загрузка чата"};var ru_RU = {call:call$4,chat:chat$4,callWithChat:callWithChat$4};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Russian (Russia)
 *
 * @public
 */
const COMPOSITE_LOCALE_RU_RU = {
    component: COMPONENT_LOCALE_RU_RU,
    strings: createCompositeStrings(ru_RU)
};

var call$3={cameraLabel:"Kamera",noCamerasLabel:"Ingen kamera hittades",cameraPermissionDenied:"Din webbläsare blockerar åtkomsten till kameran",cameraTurnedOff:"Din kamera är avstängd",chatButtonLabel:"Chatt",close:"Stäng",complianceBannerNowOnlyRecording:"Du spelar nu bara in det här mötet.",complianceBannerNowOnlyTranscription:"Nu transkriberar du bara det här mötet.",complianceBannerRecordingAndTranscriptionSaved:"Inspelning och transkription sparas.",complianceBannerRecordingAndTranscriptionStarted:"Inspelning och transkription har startats.",complianceBannerRecordingAndTranscriptionStopped:"Inspelning och transkription har avslutats.",complianceBannerRecordingSaving:"Inspelningen sparas.",complianceBannerRecordingStarted:"Inspelningen har startat.",complianceBannerRecordingStopped:"Inspelningen har stoppats.",complianceBannerTranscriptionStarted:"Transkriptionen har startat.",complianceBannerTranscriptionConsent:"Genom att ansluta ger du ditt medgivande för att det här mötet ska transkriberas.",complianceBannerTranscriptionSaving:"Transkriptionen sparas.",complianceBannerTranscriptionStopped:"Transkriptionen har avslutats.",configurationPageTitle:"Starta ett samtal",copyInviteLinkButtonLabel:"Kopiera inbjudningslänk",copyInviteLinkButtonActionedLabel:"Länken har kopierats",copyInviteLinkActionedAriaLabel:"Inbjudningslänken har kopierats",defaultPlaceHolder:"Välj ett alternativ",dismissSidePaneButtonLabel:"Stäng",videoEffectsPaneTitle:"Effekter",videoEffectsPaneBackgroundSelectionTitle:"Bakgrund",videoEffectsPaneAriaLabel:"Fönstret Videoeffekter",configurationPageCameraIsLoadingLabel:"Videon läses in...",configurationPageVideoEffectsButtonLabel:"Effekter",unableToStartVideoEffect:"Det går inte att tillämpa videoeffekten.",blurBackgroundEffectButtonLabel:"Oskärpa",blurBackgroundTooltip:"Gör bakgrunden oskarp",removeBackgroundEffectButtonLabel:"Ingen",removeBackgroundTooltip:"Ta bort bakgrund",cameraOffBackgroundEffectWarningText:"Kameran är avstängd. Aktivera kameran för att se videoeffekten.",failedToJoinCallDueToNoNetworkMoreDetails:"Samtalet kopplades från på grund av ett nätverksproblem. Kontrollera anslutningen och anslut igen.",failedToJoinCallDueToNoNetworkTitle:"Samtalet har kopplats från",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Du har inte beviljats något bidrag i samtalet. Om det var ett misstag ansluter du till samtalet igen.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Avvisad från lobbyn",learnMore:"Mer information",leavingCallTitle:"Lämnar…",leftCallMoreDetails:"Anslut till samtalet igen om det var ett misstag.",leftCallTitle:"Du lämnade samtalet",lobbyScreenConnectingToCallTitle:"Ansluter till samtalet",lobbyScreenWaitingToBeAdmittedTitle:"Väntar på att bli insläst",microphonePermissionDenied:"Din webbläsare blockerar åtkomsten till mikrofonen",microphoneToggleInLobbyNotAllowed:"Det går inte att stänga av ljudet eller slå på ljudet i lobbyn.",mutedMessage:"Din mikrofon har stängts av",networkReconnectMoreDetails:"Något verkar ha gått fel. Vi försöker komma in i samtalet igen.",networkReconnectTitle:"Vänta",notInvitedToRoomDetails:"Du kan inte ansluta till det här rummet eftersom du inte har någon inbjudan.",notInvitedToRoomTitle:"Inte inbjuden till rummet",peopleButtonLabel:"Personer",selectedPeopleButtonLabel:"Knappen Kontakter har valts",peoplePaneTitle:"Personer",peoplePaneMoreButtonAriaLabel:"Mer",peopleButtonTooltipOpen:"Visa deltagare",peopleButtonTooltipClose:"Dölj deltagare",peoplePaneSubTitle:"I det här samtalet {numberOfPeople}",privacyPolicy:"Sekretesspolicy",rejoinCallButtonLabel:"Anslut till samtal igen",removedFromCallMoreDetails:"En annan deltagare tog bort dig från samtalet.",removedFromCallTitle:"Du har tagits bort",removeMenuLabel:"Ta bort",startSpotlightMenuLabel:"Uppmärksamma för alla",addSpotlightMenuLabel:"Lägg till uppmärksammande",stopSpotlightMenuLabel:"Sluta uppmärksamma",stopSpotlightOnSelfMenuLabel:"Spotlight",spotlightLimitReachedMenuTitle:"Spotlight-gränsen har nåtts",stopAllSpotlightMenuLabel:"Avsluta alla uppmärksammanden",returnToCallButtonAriaDescription:"Återgå till samtal",returnToCallButtonAriaLabel:"Tillbaka",roomNotFoundDetails:"Det gick inte att hitta det angivna rums-ID:t.",roomNotFoundTitle:"Rummet hittades inte",roomNotValidDetails:"Det här rummet är inte giltigt just nu.",roomNotValidTitle:"Rummet är inte giltigt",inviteToRoomRemovedDetails:"Din inbjudan att ansluta till det här rummet har tagits bort.",inviteToRoomRemovedTitle:"Inbjudan till rummet har tagits bort",soundLabel:"Ljud",noMicrophonesLabel:"Inga mikrofoner hittades",noSpeakersLabel:"Inga högtalare hittades",startCallButtonLabel:"Starta samtal",openDialpadButtonLabel:"Ring telefonnummer",peoplePaneAddPeopleButtonLabel:"Lägg till personer",dialpadStartCallButtonLabel:"Ringa upp",dialpadModalTitle:"Ring telefonnummer",dialpadModalAriaLabel:"Knappsats",dialpadCloseModalButtonAriaLabel:"Stäng knappsats",moreButtonCallingLabel:"Mer",resumeCallButtonLabel:"Återuppta",resumingCallButtonLabel:"Återupptar",resumeCallButtonAriaLabel:"Återuppta samtal",resumingCallButtonAriaLabel:"Återuppta samtal",holdScreenLabel:"Du är parkerad",openDtmfDialpadLabel:"Visa knappsats",dtmfDialpadPlaceholderText:"Ange nummer",outboundCallingNoticeString:"Ringer...",participantJoinedNoticeString:"{displayName} har anslutits",twoParticipantJoinedNoticeString:"{displayName1} och {displayName2} har anslutit",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} och {displayName3} gick med",participantLeftNoticeString:"{displayName} kvar",twoParticipantLeftNoticeString:"{displayName1} och {displayName2} har lämnat",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} och {displayName3} har lämnat",unnamedParticipantString:"namnlös deltagare",manyUnnamedParticipantsJoined:"namnlös deltagare och {numOfParticipants} andra deltagare gick med",manyUnnamedParticipantsLeft:"namnlös deltagare och {numOfParticipants} andra deltagare kvar",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} och {numOfParticipants} andra deltagare har anslutit",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} och {numOfParticipants} andra deltagare har lämnat",liveCaptionsLabel:"Liveundertexter",captionsSettingsLabel:"Inställningar för undertexter",startCaptionsButtonOnLabel:"Aktivera undertexter",startCaptionsButtonOffLabel:"Stäng av undertexter",startCaptionsButtonTooltipOnContent:"Stäng av undertexter",startCaptionsButtonTooltipOffContent:"Aktivera undertexter",captionsSettingsModalTitle:"Inställningar för undertexter",captionsSettingsSpokenLanguageDropdownLabel:"Talat språk",captionsSettingsCaptionLanguageDropdownLabel:"Språk för undertexter",captionsSettingsSpokenLanguageDropdownInfoText:"Språk som alla i det här samtalet talar.",captionsSettingsCaptionLanguageDropdownInfoText:"Undertexter visas på det här språket.",captionsSettingsConfirmButtonLabel:"Bekräfta",captionsSettingsCancelButtonLabel:"Avbryt",captionsSettingsModalAriaLabel:"Modalt fönster för inställning av undertexter",captionsSettingsCloseModalButtonAriaLabel:"Stäng inställningar för undertexter",captionsBannerMoreButtonCallingLabel:"Inställningar för undertexter – Fler alternativ",captionsBannerMoreButtonTooltip:"Fler alternativ",realTimeTextLabel:"Skrift i realtid",startRealTimeTextLabel:"Aktivera RTT för det här samtalet",disabledStartRealTimeTextLabel:"RTT är för närvarande aktiverat och kan inte stängas av",dismissModalAriaLabel:"Lokal bild och fjärrbild och bild, tryck på Retur för att återgå till samtalet",callRejectedTitle:"Det gick inte att slutföra samtalet",callRejectedMoreDetails:"Personen du försöker nå är inte tillgänglig. Försök igen senare.",callTimeoutTitle:"Det gick inte att slutföra samtalet",callTimeoutDetails:"Personen du försöker nå är inte tillgänglig. Försök igen senare.",callTimeoutBotTitle:"Det gick inte att slutföra samtalet",callTimeoutBotDetails:"Tjänsten som du försöker nå är inte tillgänglig. Försök igen senare.",dtmfDialerButtonLabel:"Knappsats",dtmfDialerButtonTooltipOn:"Visa knappsats",dtmfDialerButtonTooltipOff:"Dölj knappsatsen",dtmfDialerMoreButtonLabelOn:"Visa knappsats",dtmfDialerMoreButtonLabelOff:"Dölj knappsatsen",phoneCallMoreButtonLabel:"Telefonsamtal",spokenLanguageStrings:{"ar-ae":"Arabiska – Förenade Arabemiraten","ar-sa":"Arabiska – Saudiarabien","da-dk":"Danska","de-de":"Tyska – Tyskland","en-au":"Engelska – Australien","en-ca":"Engelska – Kanada","en-gb":"Engelska – Storbritannien","en-in":"Engelska – Indien","en-nz":"Engelska – Nya Zeeland","en-us":"Engelska – USA","es-es":"Spanska – Spanien (modern sortering)","es-mx":"Spanska – Mexiko","fi-fi":"Finska","fr-ca":"Franska – Kanada","fr-fr":"Franska – Frankrike","hi-in":"Hindi","it-it":"Italienska – Italien","ja-jp":"Japanska","ko-kr":"Koreanska","nb-no":"Norska (bokmål)","nl-be":"Nederländska – Belgien","nl-nl":"Nederländska -–Nederländerna","pl-pl":"Polska","pt-br":"Portugisiska – Brasilien","ru-ru":"Ryska","sv-se":"Svenska","zh-cn":"Kinesiska (Kina),","zh-hk":"Kinesiska – Hongkong SAR","cs-cz":"Tjeckiska","pt-pt":"Portugisiska – Portugal","tr-tr":"Turkiska","vi-vn":"Vietnamesiska","th-th":"Thailändska","he-il":"Hebreiska","cy-gb":"Walesiska","uk-ua":"Ukrainska","el-gr":"Grekiska","hu-hu":"Ungerska","ro-ro":"Rumänska","sk-sk":"Slovakiska","zh-tw":"Kinesiska – Taiwan"},captionLanguageStrings:{ar:"Arabiska",da:"Danska",de:"Tyska",en:"Engelska",es:"Spanska",fi:"Finska","fr-ca":"Franska – Kanada",fr:"Franska – Frankrike",hi:"Hindi",it:"Italienska",ja:"Japanska",ko:"Koreanska",nb:"Norska (bokmål)",nl:"Nederländska",pl:"Polska",pt:"Portugisiska – Brasilien",ru:"Ryska",sv:"Svenska","zh-Hans":"Kinesiska (förenklad)","zh-Hant":"Kinesiska (traditionell)",cs:"Tjeckiska","pt-pt":"Portugisiska – Portugal",tr:"Turkiska",vi:"Vietnamesiska",th:"Thailändska",he:"Hebreiska",cy:"Walesiska",uk:"Ukrainska",el:"Grekiska",hu:"Ungerska",ro:"Rumänska",sk:"Slovakiska"},captionsBannerSpinnerText:"Startar undertexter ...",realTimeTextInputBoxDefaultText:"Skriv meddelande i realtid",realTimeTextInputErrorMessage:"Meddelandet får inte överstiga 2 000 tecken",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (realtidstext) är aktiverat för alla deltagare under hela mötets varaktighet.",realTimeTextBannerLinkLabel:"Mer information",realTimeTextModalAriaLabel:"Aktivera RTT-modal",realTimeTextModalTitle:"Aktivera RTT?",realTimeTextModalText:"Realtidstext (RTT) aktiveras för alla i mötet.",realTimeTextConfirmButtonLabel:"Aktivera",realTimeTextCancelButtonLabel:"Avbryt",realTimeTextCloseModalButtonAriaLabel:"Stäng RTT-modal",captionsOnlyContainerTitle:"Bildtexter",realTimeTextOnlyContainerTitle:"Skrift i realtid",captionsAndRealTimeTextContainerTitle:"Undertexter och text i realtid",expandButtonAriaLabel:"Visa",minimizeButtonAriaLabel:"Dölj",transferPageTransferorText:"Överför...",transferPageTransferTargetText:"Ansluter...",transferPageUnknownTransferorDisplayName:"Okänd",transferPageUnknownTransferTargetDisplayName:"Okänd",transferPageNoticeString:"Ditt samtal överförs",participantCouldNotBeReachedTitle:"Måldeltagaren är för närvarande inte tillgänglig",participantCouldNotBeReachedMoreDetails:"Kontakta deltagaren när de är tillgängliga",permissionToReachTargetParticipantNotAllowedTitle:"Behörighet att nå måldeltagaren är inte tillåten",permissionToReachTargetParticipantNotAllowedMoreDetails:"Kontrollera att måldeltagaren finns i samma klientorganisation",unableToResolveTenantTitle:"Det gick inte att matcha klientorganisations-ID för måldeltagaren",unableToResolveTenantMoreDetails:"Kontrollera att deltagar-ID:t har angetts korrekt",participantIdIsMalformedTitle:"Deltagar-ID:t har inte rätt format",participantIdIsMalformedMoreDetails:"Kontrollera att deltagar-ID:t har rätt format",moreButtonGalleryControlLabel:"Visa",moreButtonGalleryPositionToggleLabel:"Flytta galleriet högst upp",moreButtonGallerySpeakerLayoutLabel:"Högtalare",moreButtonGalleryFloatingLocalLayoutLabel:"Dynamisk",moreButtonGalleryDefaultLayoutLabel:"Gallerivy",moreButtonGalleryFocusedContentLayoutLabel:"Fokusera på innehållet",moreButtonLargeGalleryDefaultLayoutLabel:"Stort galleri",moreButtonTogetherModeLayoutLabel:"Tillsammans-läge",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Kameran har inaktiverats. Du kan inte längre dela video.",grantedDueToMeetingOption:"Kameran har aktiverats. Aktivera det om du vill dela video."},unmuteMic:{lostDueToMeetingOption:"Mikrofonen har inaktiverats. Du kan inte längre slå på ljudet.",grantedDueToMeetingOption:"Din mikrofon har aktiverats. Slå på ljudet om du vill prata."},shareScreen:{lostDueToRoleChangeToAttendee:"Din roll har ändrats. Vissa åtgärder, som att dela innehåll, är inte tillgängliga för dig.",grantedDueToRoleChangeToPresenter:"Du är presentatör. Du kan dela innehåll och underlätta mötet."},hideAttendeeNames:{hideAttendeeNameAttendee:"Ditt namn är dolt för andra deltagare. Organisatörer och presentatörer kan se ditt riktiga namn.",hideAttendeeNamePresenter:"Deltagarnas namn är dolda. Endast organisatörer och presentatörer kan se namn för att skydda deltagarnas integritet."}},surveyTitle:"Hjälp oss att förbättra",starSurveyHelperText:"Hur var kvaliteten på samtalet?",starSurveyOneStarText:"Kvaliteten var dålig.",starSurveyTwoStarText:"Kvaliteten var inte bra.",starSurveyThreeStarText:"Kvaliteten var bra.",starSurveyFourStarText:"Kvaliteten var utmärkt.",starSurveyFiveStarText:"Kvaliteten var mycket bra.",surveyConfirmButtonLabel:"Skicka feedback",starRatingAriaLabel:"Markera {0} av {1} stjärnor",surveySkipButtonLabel:"Hoppa över",tagsSurveyQuestion:"Vad kan förbättras?",tagsSurveyTextFieldDefaultText:"Annat, specificera",tagsSurveyHelperText:"Kontrollera eventuella problem som du har upplevt",endOfSurveyText:"Tack för din feedback!",surveyIssues:{overallRating:{callCannotJoin:"Jag kunde inte ansluta till samtalet",callCannotInvite:"Jag kunde inte bjuda in andra till samtalet",hadToRejoin:"Jag var tvungen att återansluta till samtalet",callEndedUnexpectedly:"Samtalet avslutades oväntat åt mig",otherIssues:"Jag hade andra problem med samtalet"},audioRating:{noLocalAudio:"Den jag talade med kunde inte höra något alls",noRemoteAudio:"Jag kunde inte höra något alls",echo:"Jag hörde eko i samtalet",audioNoise:"Jag hörde störande ljud i samtalet",lowVolume:"Volymen var för låg",audioStoppedUnexpectedly:"Ljudet avbröts plötsligt",distortedSpeech:"Ljudet förvrängdes",audioInterruption:"Ljudet avbröts",otherIssues:"Jag hade andra ljudproblem i det här samtalet"},videoRating:{noVideoReceived:"Jag kunde inte se någon video",noVideoSent:"De andra deltagarna kunde inte se mig",lowQuality:"Videokvaliteten var låg",freezes:"Videon är frusen",stoppedUnexpectedly:"Videon avbröts plötsligt",darkVideoReceived:"Jag kan bara se mörka skärmar när andra aktiverar kameran",audioVideoOutOfSync:"Ljud och video var inte synkroniserade",otherIssues:"Jag hade andra videoproblem i det här samtalet"},screenshareRating:{noContentLocal:"Andra personer kunde inte se min skärmdelning",noContentRemote:"Jag kunde inte se andra personers skärmdelning",cannotPresent:"Jag kunde inte presentera min skärm",lowQuality:"Skärmdelningskvaliteten var låg",freezes:"Skärmdelningen är frusen",stoppedUnexpectedly:"Skärmdelningen stoppades oväntat",largeDelay:"Skärmdelningen har en stor fördröjning",otherIssues:"Jag hade andra skärmdelningsproblem i det här samtalet"}},surveyIssuesHeadingStrings:{overallRating:"Övergripande",audioRating:"Ljud",videoRating:"Video",screenshareRating:"Presenterar"},spotlightPrompt:{startSpotlightHeading:"Uppmärksamma för alla?",startSpotlightText:"Du kommer markera den här videon för alla i mötet.",startSpotlightOnSelfText:"Du kommer att markera din video för alla i mötet.",startSpotlightConfirmButtonLabel:"Uppmärksamma för alla",startSpotlightCancelButtonLabel:"Avbryt",stopSpotlightHeading:"Sluta att uppmärksamma den här videon?",stopSpotlightOnSelfHeading:"Avsluta uppmärksammande?",stopAllSpotlightHeading:"Vill du sluta uppmärksamma alla videor?",stopSpotlightText:"Den här videon markeras inte längre för alla i mötet.",stopSpotlightOnSelfText:"Din video kommer inte längre vara markerad för alla i mötet.",stopAllSpotlightText:"Videorna kommer inte längre att markeras för alla i mötet.",stopSpotlightConfirmButtonLabel:"Sluta uppmärksamma",stopSpotlightOnSelfConfirmButtonLabel:"Spotlight",stopSpotlightCancelButtonLabel:"Avbryt",closeSpotlightPromptButtonLabel:"Stäng"},exitSpotlightButtonLabel:"Spotlight",exitSpotlightButtonTooltip:"Spotlight",leaveConfirmButtonLabel:"Lämna",endCallConfirmButtonLabel:"Avsluta samtal",hangUpCancelButtonLabel:"Avbryt",leaveConfirmDialogTitle:"Vill du lämna samtalet?",leaveConfirmDialogContent:"Vill du lämna samtalet?",endCallConfirmDialogTitle:"Vill du avsluta samtalet?",endCallConfirmDialogContent:"Du avslutar samtalet för alla.",invalidMeetingIdentifier:"Ogiltigt mötes-ID eller lösenord",pinParticipantMenuLabel:"Fäst åt mig",pinParticipantLimitReachedMenuLabel:"Fäst (gränsen har nåtts)",unpinParticipantMenuLabel:"Ta bort",pinParticipantMenuItemAriaLabel:"Fäst åt mig",unpinParticipantMenuItemAriaLabel:"Ta bort",muteAllMenuLabel:"Stäng av ljudet för alla",muteAllDialogTitle:"Stäng av ljudet för alla?",muteAllDialogContent:"Detta stänger av ljudet för alla i mötet utom för dig.",muteAllConfirmButtonLabel:"Stäng av ljudet för alla",muteAllCancelButtonLabel:"Avbryt",joinBreakoutRoomButtonLabel:"Anslut till separat rum",returnFromBreakoutRoomButtonLabel:"Lämna rum",leaveBreakoutRoomAndMeetingButtonLabel:"Lämna möte",breakoutRoomJoinedNotificationTitle:"Du har gått med i grupprummet {roomName}",joinBreakoutRoomBannerTitle:"Grupprummet {roomName} är öppet",joinBreakoutRoomBannerButtonLabel:"Gå med",returnFromBreakoutRoomBannerTitle:"Återgå till huvudmötet",returnFromBreakoutRoomBannerButtonLabel:"Anslut igen",forbidAudioMenuLabel:"Inaktivera mikrofon",permitAudioMenuLabel:"Tillåt mikrofon",forbidOthersAudioDialogTitle:"Vill du inaktivera mikrofonen för deltagare?",forbidOthersAudioDialogContent:"Deltagarna kan inte slå på ljudet själva. Organisatören och presentatören kan tillåta att deltagare slår på ljudet vid behov.",forbidOthersAudioConfirmButtonLabel:"Inaktivera mikrofoner",forbidOthersAudioCancelButtonLabel:"Avbryt",permitOthersAudioDialogTitle:"Vill du tillåta mikrofon för deltagare?",permitOthersAudioDialogContent:"Alla på mötet kommer att kunna sätta på ljudet.",permitOthersAudioConfirmButtonLabel:"Tillåt mikrofoner",permitOthersAudioCancelButtonLabel:"Avbryt",forbidOthersAudioMenuLabel:"Inaktivera mikrofon för deltagare",permitOthersAudioMenuLabel:"Tillåt mikrofon för deltagare",forbidVideoMenuLabel:"Inaktivera kameran",permitVideoMenuLabel:"Tillåt kamera",forbidOthersVideoDialogTitle:"Vill du inaktivera kameran för deltagare?",forbidOthersVideoDialogContent:"Deltagarna kommer inte att kunna aktivera video själva. Organisatören och presentatörerna kan låta användarna aktivera kameran efter behov.",forbidOthersVideoConfirmButtonLabel:"Inaktivera kameror",forbidOthersVideoCancelButtonLabel:"Avbryt",permitOthersVideoDialogTitle:"Vill du tillåta kamera för deltagare?",permitOthersVideoDialogContent:"Alla i mötet kommer att kunna aktivera video själva.",permitOthersVideoConfirmButtonLabel:"Tillåt kameror",permitOthersVideoCancelButtonLabel:"Avbryt",forbidOthersVideoMenuLabel:"Inaktivera kameran för deltagare",permitOthersVideoMenuLabel:"Tillåt kamera för deltagare"};var chat$3={chatListHeader:"I den här chatten",uploadAttachment:"Ladda upp bifogad fil",uploadImageDataNotProvided:"Bilddata har inte angetts.",uploadImageIsTooLarge:"Bilden är för stor. Välj en som är mindre än {maxImageSize} MB.",uploadImageExtensionIsNotAllowed:"Uppladdning av .{imageExtension}-bilder tillåts inte.",uploadImageFailed:"Det gick inte att ladda upp bilden. Försök igen senare."};var callWithChat$3={chatButtonLabel:"Chatt",chatButtonNewMessageNotificationLabel:"Nytt meddelande",chatButtonTooltipClosedWithMessageCount:"Visa chatt ({unreadMessagesCount} oläst)",chatButtonTooltipClose:"Dölj chatt",chatButtonTooltipOpen:"Visa chatt",chatPaneTitle:"Chatt",copyInviteLinkButtonLabel:"Kopiera inbjudningslänk",copyInviteLinkButtonActionedLabel:"Länken har kopierats",copyInviteLinkActionedAriaLabel:"Inbjudningslänken har kopierats",dismissSidePaneButtonLabel:"Stäng",moreDrawerAudioDeviceMenuTitle:"Ljudenhet",moreDrawerButtonLabel:"Fler alternativ",moreDrawerButtonTooltip:"Fler alternativ",moreDrawerMicrophoneMenuTitle:"Mikrofon",moreDrawerSpeakerMenuTitle:"Högtalare",moreDrawerCaptionsMenuTitle:"Liveundertexter",moreDrawerSpokenLanguageMenuTitle:"Talat språk",moreDrawerCaptionLanguageMenuTitle:"Språk för undertext",moreDrawerGalleryOptionsMenuTitle:"Gallerialternativ",peopleButtonLabel:"Personer",selectedPeopleButtonLabel:"Knappen Kontakter har valts",peopleButtonTooltipOpen:"Visa deltagare",peopleButtonTooltipClose:"Dölj deltagare",peopleButtonTooltipOpenAriaLabel:"Knappen Personer, visa deltagare",peopleButtonTooltipCloseAriaLabel:"Knappen Personer, dölj deltagare",peoplePaneSubTitle:"I det här samtalet",peoplePaneTitle:"Personer",pictureInPictureTileAriaLabel:"Videofeeds. Klicka för att återgå till samtalsskärmen.",removeMenuLabel:"Ta bort",openDialpadButtonLabel:"Ring telefonnummer",returnToCallButtonAriaDescription:"Återgå till samtal",returnToCallButtonAriaLabel:"Tillbaka",peoplePaneAddPeopleButtonLabel:"Lägg till personer",dialpadStartCallButtonLabel:"Ringa upp",dialpadModalTitle:"Ring telefonnummer",dialpadModalAriaLabel:"Knappsats",dialpadCloseModalButtonAriaLabel:"Stäng knappsats",openDtmfDialpadLabel:"Visa knappsats",dtmfDialpadPlaceholderText:"Ange nummer",breakoutRoomChatPaneTitle:"Grupprumschatt",chatContentSpinnerLabel:"Läser in chatt"};var sv_SE = {call:call$3,chat:chat$3,callWithChat:callWithChat$3};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Swedish (Sweden)
 *
 * @public
 */
const COMPOSITE_LOCALE_SV_SE = {
    component: COMPONENT_LOCALE_SV_SE,
    strings: createCompositeStrings(sv_SE)
};

var call$2={cameraLabel:"Kamera",noCamerasLabel:"Kamera bulunamadı",cameraPermissionDenied:"Tarayıcınız kameranıza erişimi engelliyor",cameraTurnedOff:"Kameranız kapatıldı",chatButtonLabel:"Sohbet",close:"Kapat",complianceBannerNowOnlyRecording:"Şu anda yalnızca bu toplantıyı kaydediyorsunuz.",complianceBannerNowOnlyTranscription:"Şu anda yalnızca bu toplantının dökümünü oluşturuyorsunuz.",complianceBannerRecordingAndTranscriptionSaved:"Kayıt ve döküm kaydediliyor.",complianceBannerRecordingAndTranscriptionStarted:"Kayıt ve döküm başlatıldı.",complianceBannerRecordingAndTranscriptionStopped:"Kayıt ve döküm durduruldu.",complianceBannerRecordingSaving:"Kayıt kaydediliyor.",complianceBannerRecordingStarted:"Kayıt başladı.",complianceBannerRecordingStopped:"Kayıt durduruldu.",complianceBannerTranscriptionStarted:"Döküm başlatıldı.",complianceBannerTranscriptionConsent:"Katılarak bu toplantının dökümünün çıkarılmasına onay verirsiniz.",complianceBannerTranscriptionSaving:"Döküm kaydediliyor.",complianceBannerTranscriptionStopped:"Döküm durduruldu.",configurationPageTitle:"Arama başlatın",copyInviteLinkButtonLabel:"Davet bağlantısını kopyala",copyInviteLinkButtonActionedLabel:"Bağlantı kopyalandı",copyInviteLinkActionedAriaLabel:"Davet bağlantısı kopyalandı",defaultPlaceHolder:"Bir seçenek belirleyin",dismissSidePaneButtonLabel:"Kapat",videoEffectsPaneTitle:"Efektler",videoEffectsPaneBackgroundSelectionTitle:"Arka plan",videoEffectsPaneAriaLabel:"Video efektleri paneli",configurationPageCameraIsLoadingLabel:"Video yükleniyor...",configurationPageVideoEffectsButtonLabel:"Efektler",unableToStartVideoEffect:"Video efekti uygulanamıyor.",blurBackgroundEffectButtonLabel:"Bulanıklaştır",blurBackgroundTooltip:"Arka Planı Bulanıklaştır",removeBackgroundEffectButtonLabel:"Seçilmedi",removeBackgroundTooltip:"Arka Planı Kaldır",cameraOffBackgroundEffectWarningText:"Kameranız kapalı. Video efektini görmek için kamerayı açın.",failedToJoinCallDueToNoNetworkMoreDetails:"Bir ağ sorunu nedeniyle arama bağlantısı kesildi. Bağlantınızı kontrol edin ve yeniden katılın.",failedToJoinCallDueToNoNetworkTitle:"Arama kesildi",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"Aramaya katılmanıza izin verilmedi. Bu işlem yanlışlıkla yapıldıysa aramaya yeniden katılın.",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"Lobiden çıkarıldı",learnMore:"Daha fazla bilgi edinin",leavingCallTitle:"Ayrılıyor...",leftCallMoreDetails:"Bu bir hataysa aramaya yeniden katılın.",leftCallTitle:"Aramadan ayrıldınız",lobbyScreenConnectingToCallTitle:"Aramaya katılıyorsunuz",lobbyScreenWaitingToBeAdmittedTitle:"Kabul edilmeyi bekliyor",microphonePermissionDenied:"Tarayıcınız mikrofonunuza erişimi engelliyor",microphoneToggleInLobbyNotAllowed:"Lobideyken sesi kapatılamaz veya açılamaz.",mutedMessage:"Sesiniz kapatıldı",networkReconnectMoreDetails:"Bir sorun oluştu gibi görünüyor. Çağrıya geri alınmaya çalışılıyor.",networkReconnectTitle:"Bekleyin",notInvitedToRoomDetails:"Davetiniz olmadığı için bu odaya katılamazsınız.",notInvitedToRoomTitle:"Odaya davet edilmedi",peopleButtonLabel:"Kişiler",selectedPeopleButtonLabel:"Kişiler Düğmesi Seçildi",peoplePaneTitle:"Kişiler",peoplePaneMoreButtonAriaLabel:"Daha fazla",peopleButtonTooltipOpen:"Katılımcıları göster",peopleButtonTooltipClose:"Katılımcıları gizle",peoplePaneSubTitle:"Bu aramada {numberOfPeople} var",privacyPolicy:"Gizlilik ilkesi",rejoinCallButtonLabel:"Çağrıya yeniden katıl",removedFromCallMoreDetails:"Başka bir katılımcı sizi aramadan çıkardı.",removedFromCallTitle:"Çıkarıldınız",removeMenuLabel:"Kaldır",startSpotlightMenuLabel:"Herkes için öne çıkar",addSpotlightMenuLabel:"Öne çıkarma ekle",stopSpotlightMenuLabel:"Öne çıkarmayı durdur",stopSpotlightOnSelfMenuLabel:"Öne çıkarmadan çık",spotlightLimitReachedMenuTitle:"Öne çıkarma sınırına ulaşıldı",stopAllSpotlightMenuLabel:"Tüm öne çıkarmaları durdur",returnToCallButtonAriaDescription:"Aramaya Dön",returnToCallButtonAriaLabel:"Geri",roomNotFoundDetails:"Sağlanan oda kimliği bulunamadı.",roomNotFoundTitle:"Oda bulunamadı",roomNotValidDetails:"Bu oda şu anda geçerli değil.",roomNotValidTitle:"Oda geçerli değil",inviteToRoomRemovedDetails:"Bu odaya katılma davetiniz kaldırıldı.",inviteToRoomRemovedTitle:"Odaya davet kaldırıldı",soundLabel:"Ses",noMicrophonesLabel:"Mikrofon bulunamadı",noSpeakersLabel:"Hoparlör bulunamadı",startCallButtonLabel:"Arama başlat",openDialpadButtonLabel:"Telefon numarasını çevir",peoplePaneAddPeopleButtonLabel:"Kişi Ekle",dialpadStartCallButtonLabel:"Ara",dialpadModalTitle:"Telefon Numarasını Çevir",dialpadModalAriaLabel:"Tuş takımı",dialpadCloseModalButtonAriaLabel:"Tuş takımını kapat",moreButtonCallingLabel:"Diğer",resumeCallButtonLabel:"Devam edin",resumingCallButtonLabel:"Devam ediyor...",resumeCallButtonAriaLabel:"Görüşmeye devam edin",resumingCallButtonAriaLabel:"Görüşmeye devam edin",holdScreenLabel:"Beklemeye alındınız",openDtmfDialpadLabel:"Tuş takımını göster",dtmfDialpadPlaceholderText:"Numara girin",outboundCallingNoticeString:"Aranıyor...",participantJoinedNoticeString:"{displayName} katıldı",twoParticipantJoinedNoticeString:"{displayName1} ve {displayName2} katıldı",threeParticipantJoinedNoticeString:"{displayName1}, {displayName2} ve {displayName3} katıldı",participantLeftNoticeString:"{displayName} ayrıldı",twoParticipantLeftNoticeString:"{displayName1} ve {displayName2} ayrıldı",threeParticipantLeftNoticeString:"{displayName1}, {displayName2} ve {displayName3} ayrıldı",unnamedParticipantString:"adsız katılımcı",manyUnnamedParticipantsJoined:"adsız katılımcı ve {numOfParticipants} katılımcı katıldı",manyUnnamedParticipantsLeft:"adsız katılımcı ve {numOfParticipants} katılımcı ayrıldı",manyParticipantsJoined:"{displayName1}, {displayName2}, {displayName3} ve {numOfParticipants} katılımcı katıldı",manyParticipantsLeft:"{displayName1}, {displayName2}, {displayName3} ve {numOfParticipants} katılımcı kaldı",liveCaptionsLabel:"Açıklamalı canlı alt yazılar",captionsSettingsLabel:"Açıklamalı alt yazı ayarları",startCaptionsButtonOnLabel:"Açıklamalı alt yazıları açma",startCaptionsButtonOffLabel:"Açıklamalı alt yazıları kapat",startCaptionsButtonTooltipOnContent:"Açıklamalı alt yazıları kapat",startCaptionsButtonTooltipOffContent:"Açıklamalı alt yazıları açma",captionsSettingsModalTitle:"Açıklamalı Alt Yazı Ayarları",captionsSettingsSpokenLanguageDropdownLabel:"Konuşulan dil",captionsSettingsCaptionLanguageDropdownLabel:"Açıklamalı alt yazı dili",captionsSettingsSpokenLanguageDropdownInfoText:"Bu çağrıdaki herkesin konuşma dili.",captionsSettingsCaptionLanguageDropdownInfoText:"Açıklamalı alt yazılar bu dilde görünür.",captionsSettingsConfirmButtonLabel:"Onayla",captionsSettingsCancelButtonLabel:"İptal",captionsSettingsModalAriaLabel:"Açıklamalı Alt Yazı Ayarı Kalıcı",captionsSettingsCloseModalButtonAriaLabel:"Açıklamalı Alt Yazı Ayarını Kapat",captionsBannerMoreButtonCallingLabel:"Alt Yazı Ayarları Diğer Seçenekler",captionsBannerMoreButtonTooltip:"Diğer seçenekler",realTimeTextLabel:"Gerçek zamanlı metin (RTT)",startRealTimeTextLabel:"Bu arama için RTT'yi aç",disabledStartRealTimeTextLabel:"RTT şu anda açık, kapatılamaz",dismissModalAriaLabel:"Yerel ve uzak resim ve resim, aramaya geri dönmek için Enter tuşuna basın",callRejectedTitle:"Arama tamamlanamadı",callRejectedMoreDetails:"Ulaşmaya çalıştığınız kişi kullanılamıyor. Lütfen daha sonra yeniden deneyin.",callTimeoutTitle:"Arama tamamlanamadı",callTimeoutDetails:"Ulaşmaya çalıştığınız kişi kullanılamıyor. Lütfen daha sonra yeniden deneyin.",callTimeoutBotTitle:"Arama tamamlanamadı",callTimeoutBotDetails:"Erişmeye çalıştığınız hizmet kullanılamıyor. Lütfen daha sonra yeniden deneyin.",dtmfDialerButtonLabel:"Tuş takımı",dtmfDialerButtonTooltipOn:"Tuş takımını göster",dtmfDialerButtonTooltipOff:"Sayısal tuş takımını gizle",dtmfDialerMoreButtonLabelOn:"Tuş takımını göster",dtmfDialerMoreButtonLabelOff:"Sayısal tuş takımını gizle",phoneCallMoreButtonLabel:"Telefon Görüşmesi",spokenLanguageStrings:{"ar-ae":"Arapça - BAE","ar-sa":"Arapça - Suudi Arabistan","da-dk":"Danca","de-de":"Almanca - Almanya","en-au":"İngilizce - Avustralya","en-ca":"İngilizce - Kanada","en-gb":"İngilizce - Birleşik Krallık","en-in":"İngilizce - Hindistan","en-nz":"İngilizce - Yeni Zelanda","en-us":"İngilizce - ABD","es-es":"İspanyolca - İspanya (Modern)","es-mx":"İspanyolca - Meksika","fi-fi":"Fince","fr-ca":"Fransızca - Kanada","fr-fr":"Fransızca - Fransa","hi-in":"Hintçe","it-it":"İtalyanca - İtalya","ja-jp":"Japonca","ko-kr":"Korece","nb-no":"Norveççe (Bokmål)","nl-be":"Hollanda dili - Belçika","nl-nl":"Felemenkçe - Hollanda","pl-pl":"Lehçe","pt-br":"Portekizce - Brezilya","ru-ru":"Rusça","sv-se":"İsveççe","zh-cn":"Çince - Çin Halk Cumhuriyeti","zh-hk":"Çince - Hong Kong ÖİB","cs-cz":"Çekçe","pt-pt":"Portekizce - Portekiz","tr-tr":"Türkçe","vi-vn":"Vietnamca","th-th":"Tayca","he-il":"İbranice","cy-gb":"Galce","uk-ua":"Ukraynaca","el-gr":"Yunanca","hu-hu":"Macarca","ro-ro":"Rumence","sk-sk":"Slovakça","zh-tw":"Çince - Tayvan"},captionLanguageStrings:{ar:"Arapça",da:"Danca",de:"Almanca",en:"İngilizce",es:"İspanyolca",fi:"Fince","fr-ca":"Fransızca - Kanada",fr:"Fransızca - Fransa",hi:"Hintçe",it:"İtalyanca",ja:"Japonca",ko:"Korece",nb:"Norveççe (Bokmål)",nl:"Felemenkçe",pl:"Lehçe",pt:"Portekizce - Brezilya",ru:"Rusça",sv:"İsveççe","zh-Hans":"Çince (Basitleştirilmiş)","zh-Hant":"Çince (Geleneksel)",cs:"Çekçe","pt-pt":"Portekizce - Portekiz",tr:"Türkçe",vi:"Vietnamca",th:"Tayca",he:"İbranice",cy:"Galce",uk:"Ukraynaca",el:"Yunanca",hu:"Macarca",ro:"Rumence",sk:"Slovakça"},captionsBannerSpinnerText:"Alt yazılar başlatılıyor...",realTimeTextInputBoxDefaultText:"İletiyi gerçek zamanlı yazın",realTimeTextInputErrorMessage:"İleti 2000 karakteri geçemez",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (gerçek zamanlı metin) toplantı süresi boyunca tüm katılımcılar için etkindir.",realTimeTextBannerLinkLabel:"Daha fazla bilgi edinin",realTimeTextModalAriaLabel:"RTT Kalıcı İletişim Kutusunu Aç",realTimeTextModalTitle:"RTT açılsın mı?",realTimeTextModalText:"Gerçek zamanlı metin (RTT) toplantıdaki herkes için açılır.",realTimeTextConfirmButtonLabel:"Aç",realTimeTextCancelButtonLabel:"İptal",realTimeTextCloseModalButtonAriaLabel:"RTT Kalıcı İletişim Kutusunu Kapat",captionsOnlyContainerTitle:"Alt yazılar",realTimeTextOnlyContainerTitle:"Gerçek Zamanlı Metin",captionsAndRealTimeTextContainerTitle:"Altyazılar ve Gerçek Zamanlı Metin",expandButtonAriaLabel:"Genişlet",minimizeButtonAriaLabel:"Daralt",transferPageTransferorText:"Aktarılıyor...",transferPageTransferTargetText:"Bağlanıyor...",transferPageUnknownTransferorDisplayName:"Bilinmiyor",transferPageUnknownTransferTargetDisplayName:"Bilinmiyor",transferPageNoticeString:"Çağrı aktarılıyor",participantCouldNotBeReachedTitle:"Hedef katılımcı şu anda uygun değil",participantCouldNotBeReachedMoreDetails:"Uygun olduğunda katılımcıyla iletişim kurun",permissionToReachTargetParticipantNotAllowedTitle:"Hedef katılımcıya erişim iznine izin verilmiyor",permissionToReachTargetParticipantNotAllowedMoreDetails:"Lütfen hedef katılımcının aynı kiracıda olup olmadığını denetleyin",unableToResolveTenantTitle:"Hedef katılımcının kiracı kimliği çözülemiyor",unableToResolveTenantMoreDetails:"Lütfen katılımcı kimliğinin doğru giril olup olmadığını denetleyin",participantIdIsMalformedTitle:"Katılımcı kimliği doğru biçimde değil",participantIdIsMalformedMoreDetails:"Lütfen katılımcı kimliğinin doğru biçimde olduğundan emin olun",moreButtonGalleryControlLabel:"Görünüm",moreButtonGalleryPositionToggleLabel:"Galeriyi en üste taşı",moreButtonGallerySpeakerLayoutLabel:"Konuşmacı",moreButtonGalleryFloatingLocalLayoutLabel:"Dinamik",moreButtonGalleryDefaultLayoutLabel:"Galeri görünümü",moreButtonGalleryFocusedContentLayoutLabel:"İçeriğe odaklı",moreButtonLargeGalleryDefaultLayoutLabel:"Büyük Galeri",moreButtonTogetherModeLayoutLabel:"Birlikte modu",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"Kameranız devre dışı bırakıldı. Artık video paylaşasınız.",grantedDueToMeetingOption:"Kameranız etkinleştirildi. Video paylaşmak istiyorsanız açın."},unmuteMic:{lostDueToMeetingOption:"Mikrofonunuz devre dışı bırakıldı. Artık sesi açamaznız.",grantedDueToMeetingOption:"Mikrofonunuz etkinleştirildi. Konuşmak istiyorsanız sesi açın."},shareScreen:{lostDueToRoleChangeToAttendee:"Rolünüz değiştirildi. İçerik paylaşma gibi bazı eylemler kullanılamaz.",grantedDueToRoleChangeToPresenter:"Sunucuyuz. İçerik paylaşabilir ve toplantıya yardımcı olabilirsiniz."},hideAttendeeNames:{hideAttendeeNameAttendee:"Adınız diğer katılımcılardan gizlendi. Düzenleyiciler ve sunucular gerçek adınızı görebilir.",hideAttendeeNamePresenter:"Katılımcı adları gizlidir. Katılımcı gizliliğini korumak için yalnızca düzenleyiciler ve sunucular adları görebilir."}},surveyTitle:"Geliştirmemize yardımcı olun",starSurveyHelperText:"Arama kalitesi nasıldı?",starSurveyOneStarText:"Kalite kötüydü.",starSurveyTwoStarText:"Kalite düşüktü.",starSurveyThreeStarText:"Kalite iyiydi.",starSurveyFourStarText:"Kalite harikaydı.",starSurveyFiveStarText:"Kalite mükemmeldi.",surveyConfirmButtonLabel:"Geri Bildirim Gönder",starRatingAriaLabel:"{1} üzerinden {0} yıldız seçin",surveySkipButtonLabel:"Atla",tagsSurveyQuestion:"Neleri iyileştirebiliriz?",tagsSurveyTextFieldDefaultText:"Diğer, lütfen belirtin",tagsSurveyHelperText:"Karşılaştığınız tüm sorunları işaretleyin",endOfSurveyText:"Geri bildiriminiz için teşekkür ederiz!",surveyIssues:{overallRating:{callCannotJoin:"Çağrıya katılamadım",callCannotInvite:"Başkalarını çağrıya davet edemedim",hadToRejoin:"Çağrıya yeniden katılmam gerekirdi",callEndedUnexpectedly:"Arama benim için beklenmedik bir şekilde sona erdi",otherIssues:"Aramayla ilgili başka sorunlar yaşıyorum"},audioRating:{noLocalAudio:"Karşı taraf ses duyamadı",noRemoteAudio:"Ses duyamadım",echo:"Aramada yankılar duydum",audioNoise:"Aramada parazit vardı",lowVolume:"Ses düzeyi düşüktü",audioStoppedUnexpectedly:"Ses beklenmedik şekilde durduruldu",distortedSpeech:"Ses bozuktu",audioInterruption:"Ses kesildi",otherIssues:"Bu aramada başka ses sorunlarıyla karşılaştım"},videoRating:{noVideoReceived:"Görüntü yoktu",noVideoSent:"Diğerleri beni göremedi",lowQuality:"Video kalitesi düşüktü",freezes:"Video dondu",stoppedUnexpectedly:"Görüntü beklenmedik şekilde durdu",darkVideoReceived:"Başkaları kameralarını etkinleştirildiğinde sadece siyah ekranlar görüyorum",audioVideoOutOfSync:"Ses ve Videoda senkron sorunu vardı",otherIssues:"Bu aramada başka video sorunlarıyla karşılaştım"},screenshareRating:{noContentLocal:"Diğer kişiler ekran paylaşımımı göremedi",noContentRemote:"Diğer kişilerin ekran paylaşımını göremedim",cannotPresent:"Ekranımı gösteremedim",lowQuality:"Ekran paylaşımı kalitesi düşüktü",freezes:"Ekran paylaşımı donduruldu",stoppedUnexpectedly:"Ekran paylaşımı beklenmedik şekilde durdu",largeDelay:"Ekran paylaşımında büyük bir gecikme var",otherIssues:"Bu aramada başka ekran paylaşımı sorunlarıyla karşılaştım"}},surveyIssuesHeadingStrings:{overallRating:"Genel",audioRating:"Ses",videoRating:"Video",screenshareRating:"Sunu"},spotlightPrompt:{startSpotlightHeading:"Herkes için öne çıkarılsın mı?",startSpotlightText:"Bu videoyu toplantıdaki herkes için öne çıkaracaksınız.",startSpotlightOnSelfText:"Videonuzu toplantıdaki herkes için öne çıkaracaksınız.",startSpotlightConfirmButtonLabel:"Herkes için öne çıkar",startSpotlightCancelButtonLabel:"İptal",stopSpotlightHeading:"Bu video için öne çıkarma durdurulsun mu?",stopSpotlightOnSelfHeading:"Öne çıkarmadan çıkılsın mı?",stopAllSpotlightHeading:"Tüm videolarda öne çıkarma durdurulsun mu?",stopSpotlightText:"Bu video artık toplantıdaki herkes için öne çıkarılmayacak.",stopSpotlightOnSelfText:"Videonuz toplantıdaki herkes için artık öne çıkarılmayacak.",stopAllSpotlightText:"Artık videolar toplantıdaki herkes için öne çıkarılmayacak.",stopSpotlightConfirmButtonLabel:"Öne çıkarmayı durdur",stopSpotlightOnSelfConfirmButtonLabel:"Öne çıkarmadan çık",stopSpotlightCancelButtonLabel:"İptal",closeSpotlightPromptButtonLabel:"Kapat"},exitSpotlightButtonLabel:"Öne çıkarmadan çık",exitSpotlightButtonTooltip:"Öne çıkarmadan çık",leaveConfirmButtonLabel:"Ayrıl",endCallConfirmButtonLabel:"Aramayı sonlandır",hangUpCancelButtonLabel:"İptal",leaveConfirmDialogTitle:"Aramadan çıkılsın mı?",leaveConfirmDialogContent:"Aramadan ayrılmak istediğinizden emin misiniz?",endCallConfirmDialogTitle:"Arama sonlandırılsın mı?",endCallConfirmDialogContent:"Aramayı herkes için sonlandırıyorsunuz.",invalidMeetingIdentifier:"Geçersiz toplantı kimliği veya geçiş kodu",pinParticipantMenuLabel:"Benim için sabitle",pinParticipantLimitReachedMenuLabel:"Sabitleme (sınıra ulaşıldı)",unpinParticipantMenuLabel:"Sabitlemeyi kaldır",pinParticipantMenuItemAriaLabel:"Benim için sabitle",unpinParticipantMenuItemAriaLabel:"Sabitlemeyi kaldır",muteAllMenuLabel:"Tümünü sessize al",muteAllDialogTitle:"Herkesi sessize almak mı istiyorsunuz?",muteAllDialogContent:"Bu, toplantıda sizin dışınızdaki herkesi sessize alır.",muteAllConfirmButtonLabel:"Tümünü sessize al",muteAllCancelButtonLabel:"İptal",joinBreakoutRoomButtonLabel:"Tartışma odasına katıl",returnFromBreakoutRoomButtonLabel:"Odadan ayrıl",leaveBreakoutRoomAndMeetingButtonLabel:"Toplantıdan ayrıl",breakoutRoomJoinedNotificationTitle:"'{roomName}' Tartışma odasına katıldınız",joinBreakoutRoomBannerTitle:"'{roomName}' Tartışma odası açık",joinBreakoutRoomBannerButtonLabel:"Katıl",returnFromBreakoutRoomBannerTitle:"Ana toplantıya geri dön",returnFromBreakoutRoomBannerButtonLabel:"Yeniden Katıl",forbidAudioMenuLabel:"Mikrofonu devre dışı bırak",permitAudioMenuLabel:"Mikrofona izin ver",forbidOthersAudioDialogTitle:"Mikrofon katılımcılar için devre dışı bırakılsın mı?",forbidOthersAudioDialogContent:"Katılımcılar kendi seslerini açamaz. Düzenleyici ve sunucular, gerektiğinde kişilerin sesi açmasını sağlayabilir.",forbidOthersAudioConfirmButtonLabel:"Mikrofonları devre dışı bırak",forbidOthersAudioCancelButtonLabel:"İptal",permitOthersAudioDialogTitle:"Katılımcıların mikrofona erişimine izin verilsin mi?",permitOthersAudioDialogContent:"Toplantıdaki herkes kendi sesini kapatabilir.",permitOthersAudioConfirmButtonLabel:"Mikrofonlara izin ver",permitOthersAudioCancelButtonLabel:"İptal",forbidOthersAudioMenuLabel:"Mikrofonu katılımcılar için devre dışı bırak",permitOthersAudioMenuLabel:"Katılımcıların mikrofona erişimine izin ver",forbidVideoMenuLabel:"Kamerayı devre dışı bırak",permitVideoMenuLabel:"Kameraya izin ver",forbidOthersVideoDialogTitle:"Kamera katılımcılar için devre dışı bırakılsın mı?",forbidOthersVideoDialogContent:"Katılımcılar videoyu kendileri etkileştiremezler. Düzenleyici ve sunucular, gerektiğinde kişilerin kameralarını etkinleştirmelerine izin verebilir.",forbidOthersVideoConfirmButtonLabel:"Kameraları devre dışı bırak",forbidOthersVideoCancelButtonLabel:"İptal",permitOthersVideoDialogTitle:"Katılımcıların kameraya erişimine izin verilsin mi?",permitOthersVideoDialogContent:"Toplantıdaki herkes videoyu kendileri açabilir.",permitOthersVideoConfirmButtonLabel:"Kameralara izin ver",permitOthersVideoCancelButtonLabel:"İptal",forbidOthersVideoMenuLabel:"Kamerayı katılımcılar için devre dışı bırak",permitOthersVideoMenuLabel:"Katılımcıların kameraya erişimine izin ver"};var chat$2={chatListHeader:"Bu sohbette",uploadAttachment:"Eki Karşıya Yükle",uploadImageDataNotProvided:"Görüntü verileri sağlanmadı.",uploadImageIsTooLarge:"Resim çok büyük. Mb'den küçük bir {maxImageSize} seçin.",uploadImageExtensionIsNotAllowed:".{imageExtension} görüntüsünün karşıya yüklemesine izin verilmiyor.",uploadImageFailed:"Resim karşıya yüklenemiyor. Lütfen daha sonra yeniden deneyin."};var callWithChat$2={chatButtonLabel:"Sohbet",chatButtonNewMessageNotificationLabel:"Yeni İleti",chatButtonTooltipClosedWithMessageCount:"Sohbeti göster ({unreadMessagesCount} okunmamış)",chatButtonTooltipClose:"Sohbeti gizle",chatButtonTooltipOpen:"Sohbeti göster",chatPaneTitle:"Sohbet",copyInviteLinkButtonLabel:"Davet bağlantısını kopyala",copyInviteLinkButtonActionedLabel:"Bağlantı kopyalandı",copyInviteLinkActionedAriaLabel:"Davet bağlantısı kopyalandı",dismissSidePaneButtonLabel:"Kapat",moreDrawerAudioDeviceMenuTitle:"Ses Cihazı",moreDrawerButtonLabel:"Diğer seçenekler",moreDrawerButtonTooltip:"Diğer seçenekler",moreDrawerMicrophoneMenuTitle:"Mikrofon",moreDrawerSpeakerMenuTitle:"Hoparlör",moreDrawerCaptionsMenuTitle:"Açıklamalı canlı alt yazılar",moreDrawerSpokenLanguageMenuTitle:"Konuşulan dil",moreDrawerCaptionLanguageMenuTitle:"Açıklamalı alt yazı dili",moreDrawerGalleryOptionsMenuTitle:"Galeri seçenekleri",peopleButtonLabel:"Kişiler",selectedPeopleButtonLabel:"Kişiler Düğmesi Seçildi",peopleButtonTooltipOpen:"Katılımcıları göster",peopleButtonTooltipClose:"Katılımcıları gizle",peopleButtonTooltipOpenAriaLabel:"Kişiler düğmesi, Katılımcıları göster",peopleButtonTooltipCloseAriaLabel:"Kişiler düğmesi, Katılımcıları gizle",peoplePaneSubTitle:"Bu aramada",peoplePaneTitle:"Kişiler",pictureInPictureTileAriaLabel:"Video Akışları. Çağrı ekranına dönmek için tıklayın.",removeMenuLabel:"Kaldır",openDialpadButtonLabel:"Telefon numarasını çevir",returnToCallButtonAriaDescription:"Aramaya Dön",returnToCallButtonAriaLabel:"Geri",peoplePaneAddPeopleButtonLabel:"Kişi Ekle",dialpadStartCallButtonLabel:"Ara",dialpadModalTitle:"Telefon Numarasını Çevir",dialpadModalAriaLabel:"Tuş takımı",dialpadCloseModalButtonAriaLabel:"Tuş takımını kapat",openDtmfDialpadLabel:"Tuş takımını göster",dtmfDialpadPlaceholderText:"Numara girin",breakoutRoomChatPaneTitle:"Tartışma Odası Sohbeti",chatContentSpinnerLabel:"Sohbet yükleniyor"};var tr_TR = {call:call$2,chat:chat$2,callWithChat:callWithChat$2};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Turkish (Turkey)
 *
 * @public
 */
const COMPOSITE_LOCALE_TR_TR = {
    component: COMPONENT_LOCALE_TR_TR,
    strings: createCompositeStrings(tr_TR)
};

var call$1={cameraLabel:"照相机",noCamerasLabel:"未找到摄像头",cameraPermissionDenied:"你的浏览器正在阻止对相机的访问",cameraTurnedOff:"你的相机已关闭",chatButtonLabel:"聊天",close:"关闭",complianceBannerNowOnlyRecording:"现在仅录制此会议。",complianceBannerNowOnlyTranscription:"现在仅听录此会议。",complianceBannerRecordingAndTranscriptionSaved:"正在保存录制和听录。",complianceBannerRecordingAndTranscriptionStarted:"录制和听录已开始。",complianceBannerRecordingAndTranscriptionStopped:"录制和听录已停止。",complianceBannerRecordingSaving:"正在保存录制。",complianceBannerRecordingStarted:"录制已开始。",complianceBannerRecordingStopped:"录制已停止。",complianceBannerTranscriptionStarted:"听录已开始。",complianceBannerTranscriptionConsent:"加入即表示你同意转录此会议。",complianceBannerTranscriptionSaving:"正在保存听录。",complianceBannerTranscriptionStopped:"听录已停止。",configurationPageTitle:"开始通话",copyInviteLinkButtonLabel:"复制邀请链接",copyInviteLinkButtonActionedLabel:"已复制链接",copyInviteLinkActionedAriaLabel:"已复制邀请链接",defaultPlaceHolder:"选择一个选项",dismissSidePaneButtonLabel:"关闭",videoEffectsPaneTitle:"效果",videoEffectsPaneBackgroundSelectionTitle:"背景",videoEffectsPaneAriaLabel:"视频效果窗格",configurationPageCameraIsLoadingLabel:"正在加载视频...",configurationPageVideoEffectsButtonLabel:"效果",unableToStartVideoEffect:"无法应用视频效果。",blurBackgroundEffectButtonLabel:"模糊",blurBackgroundTooltip:"模糊背景",removeBackgroundEffectButtonLabel:"无",removeBackgroundTooltip:"删除背景",cameraOffBackgroundEffectWarningText:"你的相机已关闭。打开相机以查看视频效果。",failedToJoinCallDueToNoNetworkMoreDetails:"由于网络问题，呼叫已断开连接。请检查你的连接，然后重新加入。",failedToJoinCallDueToNoNetworkTitle:"通话已断开",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"未在通话中向你授予条目。如果这是一个错误，请重新加入呼叫。",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"已从大厅中关闭",learnMore:"了解详细信息",leavingCallTitle:"正在退出...",leftCallMoreDetails:"如果这是一个错误，请重新加入呼叫。",leftCallTitle:"你已离开通话",lobbyScreenConnectingToCallTitle:"加入通话",lobbyScreenWaitingToBeAdmittedTitle:"正在等待被拒绝",microphonePermissionDenied:"你的浏览器正在阻止对麦克风的访问",microphoneToggleInLobbyNotAllowed:"在大厅中时无法静音或取消静音。",mutedMessage:"你已静音",networkReconnectMoreDetails:"似乎出现了问题。我们正在努力恢复通话。",networkReconnectTitle:"稍等",notInvitedToRoomDetails:"你无法加入此聊天室，因为你没有邀请。",notInvitedToRoomTitle:"未受邀加入会议室",peopleButtonLabel:"人员",selectedPeopleButtonLabel:"已选择“人员”按钮",peoplePaneTitle:"人员",peoplePaneMoreButtonAriaLabel:"更多",peopleButtonTooltipOpen:"显示参与者",peopleButtonTooltipClose:"隐藏参与者",peoplePaneSubTitle:"在此通话中 {numberOfPeople}",privacyPolicy:"隐私策略",rejoinCallButtonLabel:"重新加入呼叫",removedFromCallMoreDetails:"另一名参与者已将你从通话中删除。",removedFromCallTitle:"你已被删除",removeMenuLabel:"删除",startSpotlightMenuLabel:"对所有人聚焦",addSpotlightMenuLabel:"添加聚焦",stopSpotlightMenuLabel:"停止聚焦",stopSpotlightOnSelfMenuLabel:"退出聚焦",spotlightLimitReachedMenuTitle:"已达到聚焦限制",stopAllSpotlightMenuLabel:"停止所有聚焦",returnToCallButtonAriaDescription:"返回通话",returnToCallButtonAriaLabel:"返回",roomNotFoundDetails:"找不到提供的会议室 ID。",roomNotFoundTitle:"未找到会议室。",roomNotValidDetails:"此会议室当前无效。",roomNotValidTitle:"会议室无效",inviteToRoomRemovedDetails:"已删除你加入此会议室的邀请。",inviteToRoomRemovedTitle:"已删除加入会议室的邀请",soundLabel:"声音",noMicrophonesLabel:"未找到麦克风",noSpeakersLabel:"未找到扬声器",startCallButtonLabel:"开始通话",openDialpadButtonLabel:"拨打电话号码",peoplePaneAddPeopleButtonLabel:"添加人员",dialpadStartCallButtonLabel:"呼叫",dialpadModalTitle:"拨打电话号码",dialpadModalAriaLabel:"拨号盘",dialpadCloseModalButtonAriaLabel:"关闭拨号盘",moreButtonCallingLabel:"更多",resumeCallButtonLabel:"恢复",resumingCallButtonLabel:"正在恢复...",resumeCallButtonAriaLabel:"继续呼叫",resumingCallButtonAriaLabel:"继续通话",holdScreenLabel:"你的通话处于保留状态",openDtmfDialpadLabel:"显示拨号盘",dtmfDialpadPlaceholderText:"输入号码",outboundCallingNoticeString:"正在呼叫...",participantJoinedNoticeString:"{displayName} 已加入",twoParticipantJoinedNoticeString:"{displayName1} 和 {displayName2} 已加入",threeParticipantJoinedNoticeString:"{displayName1}、{displayName2} 和 {displayName3} 已加入",participantLeftNoticeString:"{displayName} 已离开",twoParticipantLeftNoticeString:"{displayName1} 和 {displayName2} 已离开",threeParticipantLeftNoticeString:"{displayName1}、{displayName2} 和 {displayName3} 已离开",unnamedParticipantString:"未命名的参与者",manyUnnamedParticipantsJoined:"未命名的参与者和其他 {numOfParticipants} 名参与者已加入",manyUnnamedParticipantsLeft:"未命名的参与者和其他 {numOfParticipants} 名参与者已离开",manyParticipantsJoined:"{displayName1}、{displayName2}、{displayName3}和其他{numOfParticipants}位参与者已加入",manyParticipantsLeft:"{displayName1}、{displayName2}、{displayName3}和其他{numOfParticipants}位参与者已离开",liveCaptionsLabel:"实时辅助字幕",captionsSettingsLabel:"字幕设置",startCaptionsButtonOnLabel:"开启字幕",startCaptionsButtonOffLabel:"关闭辅助字幕",startCaptionsButtonTooltipOnContent:"关闭辅助字幕",startCaptionsButtonTooltipOffContent:"开启字幕",captionsSettingsModalTitle:"字幕设置",captionsSettingsSpokenLanguageDropdownLabel:"口述语言",captionsSettingsCaptionLanguageDropdownLabel:"辅助字幕语言",captionsSettingsSpokenLanguageDropdownInfoText:"此通话中的每个人都在讲的语言。",captionsSettingsCaptionLanguageDropdownInfoText:"字幕将以此语言显示。",captionsSettingsConfirmButtonLabel:"确认",captionsSettingsCancelButtonLabel:"取消",captionsSettingsModalAriaLabel:"字幕设置模式",captionsSettingsCloseModalButtonAriaLabel:"关闭字幕设置",captionsBannerMoreButtonCallingLabel:"字幕设置更多选项",captionsBannerMoreButtonTooltip:"更多选项",realTimeTextLabel:"实时文本(RTT)",startRealTimeTextLabel:"为此通话启用 RTT",disabledStartRealTimeTextLabel:"RTT 当前已打开，无法关闭",dismissModalAriaLabel:"本地和远程图片和图像，按 Enter 返回呼叫",callRejectedTitle:"无法完成呼叫",callRejectedMoreDetails:"您尝试联系的人员不可用。请稍后重试。",callTimeoutTitle:"无法完成呼叫",callTimeoutDetails:"您尝试联系的人员不可用。请稍后重试。",callTimeoutBotTitle:"无法完成呼叫",callTimeoutBotDetails:"你尝试访问的服务不可用。请稍后重试。",dtmfDialerButtonLabel:"拨号盘",dtmfDialerButtonTooltipOn:"显示拨号盘",dtmfDialerButtonTooltipOff:"隐藏拨号盘",dtmfDialerMoreButtonLabelOn:"显示拨号盘",dtmfDialerMoreButtonLabelOff:"隐藏拨号盘",phoneCallMoreButtonLabel:"电话呼叫",spokenLanguageStrings:{"ar-ae":"阿拉伯语 - 阿拉伯联合酋长国","ar-sa":"阿拉伯语 - 沙特阿拉伯","da-dk":"丹麦语","de-de":"德语 - 德国","en-au":"英语 - 澳大利亚","en-ca":"英语 - 加拿大","en-gb":"英语 - 英国","en-in":"英语 - 印度","en-nz":"英语 - 新西兰","en-us":"英语 - 美国","es-es":"西班牙语 - 西班牙(现代风格)","es-mx":"西班牙语 - 墨西哥","fi-fi":"芬兰语","fr-ca":"法语 - 加拿大","fr-fr":"法语 - 法国","hi-in":"印地语","it-it":"意大利语–意大利","ja-jp":"日语","ko-kr":"朝鲜语","nb-no":"挪威语(博克马尔语)","nl-be":"荷兰语 - 比利时","nl-nl":"荷兰语 – 荷兰","pl-pl":"波兰语","pt-br":"葡萄牙语 - 巴西","ru-ru":"俄语","sv-se":"瑞典语","zh-cn":"中文 - 中华人民共和国","zh-hk":"中文 - 香港特别行政区","cs-cz":"捷克语","pt-pt":"葡萄牙语 - 葡萄牙","tr-tr":"土耳其语","vi-vn":"越南语","th-th":"泰语","he-il":"希伯来语","cy-gb":"威尔士语","uk-ua":"乌克兰语","el-gr":"希腊语","hu-hu":"匈牙利语","ro-ro":"罗马尼亚语","sk-sk":"斯洛伐克语","zh-tw":"中文 – 台湾"},captionLanguageStrings:{ar:"阿拉伯语",da:"丹麦语",de:"德语",en:"英语",es:"西班牙语",fi:"芬兰语","fr-ca":"法语 - 加拿大",fr:"法语 - 法国",hi:"印地语",it:"意大利语",ja:"日语",ko:"朝鲜语",nb:"挪威语(博克马尔语)",nl:"荷兰语",pl:"波兰语",pt:"葡萄牙语 - 巴西",ru:"俄语",sv:"瑞典语","zh-Hans":"中文(简体)","zh-Hant":"中文(繁体)",cs:"捷克语","pt-pt":"葡萄牙语 - 葡萄牙",tr:"土耳其语",vi:"越南语",th:"泰语",he:"希伯来语",cy:"威尔士语",uk:"乌克兰语",el:"希腊语",hu:"匈牙利语",ro:"罗马尼亚语",sk:"斯洛伐克语"},captionsBannerSpinnerText:"正在启动字幕...",realTimeTextInputBoxDefaultText:"实时键入消息",realTimeTextInputErrorMessage:"消息不能超过 2000 个字符",realTimeTextBannerTitle:"RTT",realTimeTextBannerContent:"RTT (在会议的整个持续时间内为所有参与者启用实时文本)。",realTimeTextBannerLinkLabel:"了解详细信息",realTimeTextModalAriaLabel:"打开 RTT 模式",realTimeTextModalTitle:"是否启用 RTT?",realTimeTextModalText:"将为会议中的每个人启用实时文本 (RTT)。",realTimeTextConfirmButtonLabel:"启用",realTimeTextCancelButtonLabel:"取消",realTimeTextCloseModalButtonAriaLabel:"关闭 RTT 模式",captionsOnlyContainerTitle:"字幕",realTimeTextOnlyContainerTitle:"实时文本",captionsAndRealTimeTextContainerTitle:"字幕和实时文本",expandButtonAriaLabel:"展开",minimizeButtonAriaLabel:"折叠",transferPageTransferorText:"正在转接...",transferPageTransferTargetText:"正在连接...",transferPageUnknownTransferorDisplayName:"未知",transferPageUnknownTransferTargetDisplayName:"未知",transferPageNoticeString:"正在转接中",participantCouldNotBeReachedTitle:"目标参与者当前不可用",participantCouldNotBeReachedMoreDetails:"当参与者可用时，请联系他们",permissionToReachTargetParticipantNotAllowedTitle:"不允许访问目标参与者的权限",permissionToReachTargetParticipantNotAllowedMoreDetails:"请检查目标参与者位于同一租户中",unableToResolveTenantTitle:"无法解析目标参与者的租户 ID",unableToResolveTenantMoreDetails:"请检查正确输入参与者 ID",participantIdIsMalformedTitle:"参与者 ID 的格式不正确",participantIdIsMalformedMoreDetails:"请检查参与者 ID 的格式正确",moreButtonGalleryControlLabel:"查看",moreButtonGalleryPositionToggleLabel:"将库移到顶部",moreButtonGallerySpeakerLayoutLabel:"扬声器",moreButtonGalleryFloatingLocalLayoutLabel:"动态",moreButtonGalleryDefaultLayoutLabel:"库视图",moreButtonGalleryFocusedContentLayoutLabel:"关注内容",moreButtonLargeGalleryDefaultLayoutLabel:"大型画廊模式",moreButtonTogetherModeLayoutLabel:"同框场景模式",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"你的照相机已禁用。你无法再共享视频。",grantedDueToMeetingOption:"你的相机已启用。如果要共享视频，请将其打开。"},unmuteMic:{lostDueToMeetingOption:"你的麦克风已被禁用。你无法再取消静音。",grantedDueToMeetingOption:"你的麦克风已启用。如果想要讲话，请取消静音。"},shareScreen:{lostDueToRoleChangeToAttendee:"你的角色已更改。某些操作，如共享内容，对你不可用。",grantedDueToRoleChangeToPresenter:"你是演示者。你可以共享内容并促进会议。"},hideAttendeeNames:{hideAttendeeNameAttendee:"你的姓名对其他与会者隐藏。组织者和演示者可以看到你的真实姓名。",hideAttendeeNamePresenter:"与会者姓名已隐藏。为保护与会者隐私，只有组织者和演示者才能查看姓名。"}},surveyTitle:"帮助我们改进",starSurveyHelperText:"通话质量如何?",starSurveyOneStarText:"质量很差。",starSurveyTwoStarText:"质量较差。",starSurveyThreeStarText:"质量良好。",starSurveyFourStarText:"质量很棒。",starSurveyFiveStarText:"质量很好。",surveyConfirmButtonLabel:"发送反馈",starRatingAriaLabel:"选择 {0} 星(共 {1} 星)",surveySkipButtonLabel:"跳过",tagsSurveyQuestion:"如何改进?",tagsSurveyTextFieldDefaultText:"其他，请说明",tagsSurveyHelperText:"检查遇到的任何问题",endOfSurveyText:"感谢你的反馈!",surveyIssues:{overallRating:{callCannotJoin:"我无法加入通话",callCannotInvite:"我无法邀请其他人加入通话",hadToRejoin:"我不得不重新加入通话",callEndedUnexpectedly:"通话意外结束",otherIssues:"我在通话时遇到其他问题"},audioRating:{noLocalAudio:"对方听不到任何声音",noRemoteAudio:"我听不到任何声音",echo:"我在通话中听到回音",audioNoise:"我在通话时听到噪音",lowVolume:"音量太低",audioStoppedUnexpectedly:"视频意外停止",distortedSpeech:"音频已失真",audioInterruption:"音频已中断",otherIssues:"我在此通话中遇到其他音频问题"},videoRating:{noVideoReceived:"我看不到视频",noVideoSent:"其他人看不到我",lowQuality:"视频质量低",freezes:"视频已冻结",stoppedUnexpectedly:"视频意外停止",darkVideoReceived:"我只能在其他人打开相机时看到深色屏幕",audioVideoOutOfSync:"音频和视频不同步",otherIssues:"我在此通话中遇到其他视频问题"},screenshareRating:{noContentLocal:"其他人看不到我的屏幕共享",noContentRemote:"我看不到其他人的屏幕共享",cannotPresent:"我无法显示我的屏幕",lowQuality:"屏幕共享质量低",freezes:"屏幕共享已冻结",stoppedUnexpectedly:"屏幕共享意外停止",largeDelay:"屏幕共享具有较大的延迟",otherIssues:"我在此通话中遇到其他屏幕共享问题"}},surveyIssuesHeadingStrings:{overallRating:"整体",audioRating:"音频",videoRating:"视频",screenshareRating:"正在演示"},spotlightPrompt:{startSpotlightHeading:"对所有人聚焦?",startSpotlightText:"你将对会议中的每个人突出显示此视频。",startSpotlightOnSelfText:"你将对会议中的每个人突出显示你的视频。",startSpotlightConfirmButtonLabel:"对所有人聚焦",startSpotlightCancelButtonLabel:"取消",stopSpotlightHeading:"停止聚焦此视频?",stopSpotlightOnSelfHeading:"退出聚焦?",stopAllSpotlightHeading:"是否停止聚焦所有视频?",stopSpotlightText:"将不再为会议中的每个人突出显示此视频。",stopSpotlightOnSelfText:"将不再对会议中的每个人突出显示你的视频。",stopAllSpotlightText:"将不再对会议中的每个人突出显示该视频。",stopSpotlightConfirmButtonLabel:"停止聚焦",stopSpotlightOnSelfConfirmButtonLabel:"退出聚焦",stopSpotlightCancelButtonLabel:"取消",closeSpotlightPromptButtonLabel:"关闭"},exitSpotlightButtonLabel:"退出聚焦",exitSpotlightButtonTooltip:"退出聚焦",leaveConfirmButtonLabel:"退出",endCallConfirmButtonLabel:"结束通话",hangUpCancelButtonLabel:"取消",leaveConfirmDialogTitle:"离开聊天?",leaveConfirmDialogContent:"是否确实要离开此聊天?",endCallConfirmDialogTitle:"结束通话?",endCallConfirmDialogContent:"你将结束所有人的通话。",invalidMeetingIdentifier:"会议 ID 或密码无效",pinParticipantMenuLabel:"为我固定",pinParticipantLimitReachedMenuLabel:"固定(已达到上限)",unpinParticipantMenuLabel:"取消固定",pinParticipantMenuItemAriaLabel:"为我固定",unpinParticipantMenuItemAriaLabel:"取消固定",muteAllMenuLabel:"全部静音",muteAllDialogTitle:"对所有人静音?",muteAllDialogContent:"这会将会议中除你之外的所有人静音。",muteAllConfirmButtonLabel:"全部静音",muteAllCancelButtonLabel:"取消",joinBreakoutRoomButtonLabel:"加入分组讨论室",returnFromBreakoutRoomButtonLabel:"离开房间",leaveBreakoutRoomAndMeetingButtonLabel:"离开会议",breakoutRoomJoinedNotificationTitle:"你已加入分组讨论室“{roomName}”",joinBreakoutRoomBannerTitle:"分组讨论室“{roomName}”已打开",joinBreakoutRoomBannerButtonLabel:"加入",returnFromBreakoutRoomBannerTitle:"返回到主会议",returnFromBreakoutRoomBannerButtonLabel:"重新加入",forbidAudioMenuLabel:"禁用麦克风",permitAudioMenuLabel:"允许使用麦克风",forbidOthersAudioDialogTitle:"为与会者禁用麦克风?",forbidOthersAudioDialogContent:"与会者不能将自己取消静音。组织者和演示者可以根据需要允许与会人员取消静音。",forbidOthersAudioConfirmButtonLabel:"禁用麦克风",forbidOthersAudioCancelButtonLabel:"取消",permitOthersAudioDialogTitle:"是否允许与会者使用麦克风?",permitOthersAudioDialogContent:"会议中的每个人都能够自行取消静音。",permitOthersAudioConfirmButtonLabel:"允许使用麦克风",permitOthersAudioCancelButtonLabel:"取消",forbidOthersAudioMenuLabel:"为与会者禁用麦克风",permitOthersAudioMenuLabel:"允许与会者使用麦克风",forbidVideoMenuLabel:"禁用摄像头",permitVideoMenuLabel:"允许使用摄像头",forbidOthersVideoDialogTitle:"为与会者禁用摄像头?",forbidOthersVideoDialogContent:"与会者将无法自己打开视频。组织者和演示者可以允许用户根据需要打开摄像头。",forbidOthersVideoConfirmButtonLabel:"禁用摄像头",forbidOthersVideoCancelButtonLabel:"取消",permitOthersVideoDialogTitle:"是否允许与会者使用摄像头?",permitOthersVideoDialogContent:"会议中的每个人都能够自己打开视频。",permitOthersVideoConfirmButtonLabel:"允许使用摄像头",permitOthersVideoCancelButtonLabel:"取消",forbidOthersVideoMenuLabel:"为与会者禁用摄像头",permitOthersVideoMenuLabel:"允许与会者使用摄像头"};var chat$1={chatListHeader:"在此聊天中",uploadAttachment:"上传附件",uploadImageDataNotProvided:"未提供图像数据。",uploadImageIsTooLarge:"图像太大。选择小于 {maxImageSize} MB 的选项。",uploadImageExtensionIsNotAllowed:"上传。不允许使用 {imageExtension} 图像。",uploadImageFailed:"无法上传图像。请稍后重试。"};var callWithChat$1={chatButtonLabel:"聊天",chatButtonNewMessageNotificationLabel:"新建消息",chatButtonTooltipClosedWithMessageCount:"显示聊天({unreadMessagesCount} 条未读)",chatButtonTooltipClose:"隐藏聊天",chatButtonTooltipOpen:"显示聊天",chatPaneTitle:"聊天",copyInviteLinkButtonLabel:"复制邀请链接",copyInviteLinkButtonActionedLabel:"已复制链接",copyInviteLinkActionedAriaLabel:"已复制邀请链接",dismissSidePaneButtonLabel:"关闭",moreDrawerAudioDeviceMenuTitle:"音频设备",moreDrawerButtonLabel:"更多选项",moreDrawerButtonTooltip:"更多选项",moreDrawerMicrophoneMenuTitle:"麦克风",moreDrawerSpeakerMenuTitle:"扬声器",moreDrawerCaptionsMenuTitle:"实时辅助字幕",moreDrawerSpokenLanguageMenuTitle:"口述语言",moreDrawerCaptionLanguageMenuTitle:"字幕语言",moreDrawerGalleryOptionsMenuTitle:"库选项",peopleButtonLabel:"人员",selectedPeopleButtonLabel:"已选择“人员”按钮",peopleButtonTooltipOpen:"显示参与者",peopleButtonTooltipClose:"隐藏参与者",peopleButtonTooltipOpenAriaLabel:"“人员”按钮，显示参与者",peopleButtonTooltipCloseAriaLabel:"“人员”按钮，隐藏参与者",peoplePaneSubTitle:"在此通话中",peoplePaneTitle:"人员",pictureInPictureTileAriaLabel:"视频源。单击以返回到通话屏幕。",removeMenuLabel:"删除",openDialpadButtonLabel:"拨打电话号码",returnToCallButtonAriaDescription:"返回通话",returnToCallButtonAriaLabel:"返回",peoplePaneAddPeopleButtonLabel:"添加人员",dialpadStartCallButtonLabel:"呼叫",dialpadModalTitle:"拨打电话号码",dialpadModalAriaLabel:"拨号盘",dialpadCloseModalButtonAriaLabel:"关闭拨号盘",openDtmfDialpadLabel:"显示拨号盘",dtmfDialpadPlaceholderText:"输入号码",breakoutRoomChatPaneTitle:"分组讨论室聊天",chatContentSpinnerLabel:"正在加载聊天"};var zh_CN = {call:call$1,chat:chat$1,callWithChat:callWithChat$1};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Chinese (Mainland China)
 *
 * @public
 */
const COMPOSITE_LOCALE_ZH_CN = {
    component: COMPONENT_LOCALE_ZH_CN,
    strings: createCompositeStrings(zh_CN)
};

var call={cameraLabel:"相機",noCamerasLabel:"找不到相機",cameraPermissionDenied:"您的瀏覽器禁止存取您的相機",cameraTurnedOff:"您的相機已關閉",chatButtonLabel:"聊天",close:"關閉",complianceBannerNowOnlyRecording:"您現在只會錄製此會議。",complianceBannerNowOnlyTranscription:"您現在只會轉錄此會議。",complianceBannerRecordingAndTranscriptionSaved:"正在儲存錄製和轉錄。",complianceBannerRecordingAndTranscriptionStarted:"已開始錄製並進行轉錄。",complianceBannerRecordingAndTranscriptionStopped:"已停止錄製與轉錄。",complianceBannerRecordingSaving:"正在儲存錄製內容。",complianceBannerRecordingStarted:"已開始錄製。",complianceBannerRecordingStopped:"已停止錄製。",complianceBannerTranscriptionStarted:"已開始轉錄。",complianceBannerTranscriptionConsent:"一旦加入，即表示您同意轉譯此會議。",complianceBannerTranscriptionSaving:"正在儲存轉錄。",complianceBannerTranscriptionStopped:"已停止轉錄。",configurationPageTitle:"開始通話",copyInviteLinkButtonLabel:"複製邀請連結",copyInviteLinkButtonActionedLabel:"已複製連結",copyInviteLinkActionedAriaLabel:"已複製邀請連結",defaultPlaceHolder:"選取選項",dismissSidePaneButtonLabel:"關閉",videoEffectsPaneTitle:"效果",videoEffectsPaneBackgroundSelectionTitle:"背景",videoEffectsPaneAriaLabel:"視訊效果​​窗格",configurationPageCameraIsLoadingLabel:"正在載入影片...",configurationPageVideoEffectsButtonLabel:"效果",unableToStartVideoEffect:"無法套用視訊效果。",blurBackgroundEffectButtonLabel:"模糊",blurBackgroundTooltip:"模糊背景",removeBackgroundEffectButtonLabel:"無",removeBackgroundTooltip:"移除背景",cameraOffBackgroundEffectWarningText:"您的相機已關閉。開啟相機以查看影片效果。",failedToJoinCallDueToNoNetworkMoreDetails:"由於網路問題，通話已中斷連線。檢查您的連線並再次加入。",failedToJoinCallDueToNoNetworkTitle:"通話已斷線",failedToJoinTeamsMeetingReasonAccessDeniedMoreDetails:"您未被授予加入通話的權限。如果有誤，請重新加入通話。",failedToJoinTeamsMeetingReasonAccessDeniedTitle:"已從大廳關閉",learnMore:"深入了解",leavingCallTitle:"正在離開...",leftCallMoreDetails:"如果有誤，請重新加入通話。",leftCallTitle:"您已離開通話",lobbyScreenConnectingToCallTitle:"正在加入通話",lobbyScreenWaitingToBeAdmittedTitle:"正在等候認可",microphonePermissionDenied:"您的瀏覽器禁止存取您的麥克風",microphoneToggleInLobbyNotAllowed:"無法在大廳中靜音或取消靜音。",mutedMessage:"您已設為靜音",networkReconnectMoreDetails:"似乎發生錯誤。我們正在嘗試讓您返回通話。",networkReconnectTitle:"請稍候",notInvitedToRoomDetails:"您無法加入此聊天室，因為您沒有邀請。",notInvitedToRoomTitle:"未受邀至會議室",peopleButtonLabel:"人員",selectedPeopleButtonLabel:"已選取人員按鈕",peoplePaneTitle:"人員",peoplePaneMoreButtonAriaLabel:"其他",peopleButtonTooltipOpen:"顯示參與者",peopleButtonTooltipClose:"隱藏參與者",peoplePaneSubTitle:"在此通話 {numberOfPeople}",privacyPolicy:"隱私權原則",rejoinCallButtonLabel:"重新加入通話",removedFromCallMoreDetails:"另一名參與者已將您自通話中移除。",removedFromCallTitle:"您已遭移除",removeMenuLabel:"移除",startSpotlightMenuLabel:"設為每個人的焦點",addSpotlightMenuLabel:"新增焦點",stopSpotlightMenuLabel:"停止聚焦",stopSpotlightOnSelfMenuLabel:"結束聚焦",spotlightLimitReachedMenuTitle:"已達焦點限制",stopAllSpotlightMenuLabel:"停止所有聚焦",returnToCallButtonAriaDescription:"返回通話",returnToCallButtonAriaLabel:"返回",roomNotFoundDetails:"找不到提供的會議室識別碼。",roomNotFoundTitle:"找不到會議室",roomNotValidDetails:"此會議室目前無效。",roomNotValidTitle:"會議室無效",inviteToRoomRemovedDetails:"您加入此聊天室的邀請已移除。",inviteToRoomRemovedTitle:"已移除加入聊天室的邀請",soundLabel:"音效",noMicrophonesLabel:"找不到麥克風",noSpeakersLabel:"找不到喇叭",startCallButtonLabel:"開始通話",openDialpadButtonLabel:"撥打電話號碼",peoplePaneAddPeopleButtonLabel:"新增人員",dialpadStartCallButtonLabel:"通話",dialpadModalTitle:"撥打電話號碼",dialpadModalAriaLabel:"撥號鍵台",dialpadCloseModalButtonAriaLabel:"關閉撥號鍵台",moreButtonCallingLabel:"其他",resumeCallButtonLabel:"繼續",resumingCallButtonLabel:"正在繼續...",resumeCallButtonAriaLabel:"繼續通話",resumingCallButtonAriaLabel:"繼續通話",holdScreenLabel:"您的通話已保留",openDtmfDialpadLabel:"顯示撥號盤",dtmfDialpadPlaceholderText:"輸入號碼",outboundCallingNoticeString:"撥號中...",participantJoinedNoticeString:"{displayName} 已加入",twoParticipantJoinedNoticeString:"{displayName1} 和 {displayName2} 已加入",threeParticipantJoinedNoticeString:"{displayName1}、{displayName2} 和 {displayName3} 已加入",participantLeftNoticeString:"{displayName} 已離開",twoParticipantLeftNoticeString:"{displayName1} 和 {displayName2} 已離開",threeParticipantLeftNoticeString:"{displayName1}、{displayName2} 和 {displayName3} 已離開",unnamedParticipantString:"未命名的參與者",manyUnnamedParticipantsJoined:"未命名的參與者和 {numOfParticipants} 個其他參與者已加入",manyUnnamedParticipantsLeft:"未命名的參與者和 {numOfParticipants} 個其他參與者已離開",manyParticipantsJoined:"{displayName1}、{displayName2}、{displayName3}和其他{numOfParticipants}位參與者已加入",manyParticipantsLeft:"{displayName1}、{displayName2}、{displayName3}和其他{numOfParticipants}位參與者",liveCaptionsLabel:"即時字幕",captionsSettingsLabel:"字幕設定",startCaptionsButtonOnLabel:"開啟字幕",startCaptionsButtonOffLabel:"關閉字幕",startCaptionsButtonTooltipOnContent:"關閉字幕",startCaptionsButtonTooltipOffContent:"開啟字幕",captionsSettingsModalTitle:"字幕設定",captionsSettingsSpokenLanguageDropdownLabel:"口語語言",captionsSettingsCaptionLanguageDropdownLabel:"字幕語言",captionsSettingsSpokenLanguageDropdownInfoText:"此通話中的每個人在說的語言。",captionsSettingsCaptionLanguageDropdownInfoText:"字幕將會以此語言顯示。",captionsSettingsConfirmButtonLabel:"確認",captionsSettingsCancelButtonLabel:"取消",captionsSettingsModalAriaLabel:"字幕設定模式",captionsSettingsCloseModalButtonAriaLabel:"關閉字幕設定",captionsBannerMoreButtonCallingLabel:"字幕設定更多選項",captionsBannerMoreButtonTooltip:"更多選項",realTimeTextLabel:"即時輸入 (RTT)",startRealTimeTextLabel:"開啟此通話的即時輸入",disabledStartRealTimeTextLabel:"RTT 目前已開啟，無法關閉",dismissModalAriaLabel:"本機和遠端圖片，按 Enter 以返回通話",callRejectedTitle:"無法完成通話",callRejectedMoreDetails:"您嘗試連絡的人員無法使用。請稍後再試。",callTimeoutTitle:"無法完成通話",callTimeoutDetails:"您嘗試連絡的人員無法使用。請稍後再試。",callTimeoutBotTitle:"無法完成通話",callTimeoutBotDetails:"您嘗試連線的服務無法使用。請稍後再試。",dtmfDialerButtonLabel:"撥號鍵台",dtmfDialerButtonTooltipOn:"顯示撥號盤",dtmfDialerButtonTooltipOff:"隱藏撥號盤",dtmfDialerMoreButtonLabelOn:"顯示撥號盤",dtmfDialerMoreButtonLabelOff:"隱藏撥號盤",phoneCallMoreButtonLabel:"電話",spokenLanguageStrings:{"ar-ae":"阿拉伯文 - 阿拉伯聯合大公國","ar-sa":"阿拉伯文 - 沙烏地阿拉伯","da-dk":"丹麥文","de-de":"德文 - 德國","en-au":"英文 - 澳大利亞","en-ca":"英文 - 加拿大","en-gb":"英文 - 英國","en-in":"英文 - 印度","en-nz":"英文 - 紐西蘭","en-us":"英文 - 美國","es-es":"西班牙文 - 西班牙 (現代排序)","es-mx":"西班牙文 - 墨西哥","fi-fi":"芬蘭文","fr-ca":"法文 - 加拿大","fr-fr":"法文 - 法國","hi-in":"印地文","it-it":"義大利文 - 義大利","ja-jp":"日文","ko-kr":"韓文","nb-no":"挪威文 (巴克摩)","nl-be":"荷蘭文 - 比利時","nl-nl":"荷蘭文 - 荷蘭","pl-pl":"波蘭文","pt-br":"葡萄牙文 - 巴西","ru-ru":"俄文","sv-se":"瑞典文","zh-cn":"中文 - 中華人民共和國","zh-hk":"中文 - 香港特別行政區","cs-cz":"捷克文","pt-pt":"葡萄牙文 - 葡萄牙","tr-tr":"土耳其文","vi-vn":"越南文","th-th":"泰文","he-il":"希伯來文","cy-gb":"威爾斯文","uk-ua":"烏克蘭文","el-gr":"希臘文","hu-hu":"匈牙利文","ro-ro":"羅馬尼亞文","sk-sk":"斯洛伐克文","zh-tw":"中文 - 台灣"},captionLanguageStrings:{ar:"阿拉伯文",da:"丹麥文",de:"德文",en:"英文",es:"西班牙文",fi:"芬蘭文","fr-ca":"法文 - 加拿大",fr:"法文 - 法國",hi:"印地文",it:"義大利文",ja:"日文",ko:"韓文",nb:"挪威文 (巴克摩)",nl:"荷蘭文",pl:"波蘭文",pt:"葡萄牙文 - 巴西",ru:"俄文",sv:"瑞典文","zh-Hans":"中文 (簡體)","zh-Hant":"中文 (繁體)",cs:"捷克文","pt-pt":"葡萄牙文 - 葡萄牙",tr:"土耳其文",vi:"越南文",th:"泰文",he:"希伯來文",cy:"威爾斯文",uk:"烏克蘭文",el:"希臘文",hu:"匈牙利文",ro:"羅馬尼亞文",sk:"斯洛伐克文"},captionsBannerSpinnerText:"正在啟動輔助字幕...",realTimeTextInputBoxDefaultText:"即時輸入訊息",realTimeTextInputErrorMessage:"訊息不得超過 2000 個字元",realTimeTextBannerTitle:"即時輸入",realTimeTextBannerContent:"已為整個會議期間的所有參與者啟用 RTT (即時文字)。",realTimeTextBannerLinkLabel:"深入了解",realTimeTextModalAriaLabel:"開啟 RTT 強制回應",realTimeTextModalTitle:"開啟 RTT?",realTimeTextModalText:"將為會議中的每個人開啟即時文字 (RTT)。",realTimeTextConfirmButtonLabel:"開啟",realTimeTextCancelButtonLabel:"取消",realTimeTextCloseModalButtonAriaLabel:"關閉 RTT 強制回應",captionsOnlyContainerTitle:"輔助字幕",realTimeTextOnlyContainerTitle:"即時文字",captionsAndRealTimeTextContainerTitle:"輔助字幕與即時文字",expandButtonAriaLabel:"展開",minimizeButtonAriaLabel:"摺疊",transferPageTransferorText:"正在轉接...",transferPageTransferTargetText:"正在連線...",transferPageUnknownTransferorDisplayName:"未知",transferPageUnknownTransferTargetDisplayName:"未知",transferPageNoticeString:"正在將您轉接",participantCouldNotBeReachedTitle:"目標參與者目前沒空",participantCouldNotBeReachedMoreDetails:"請在參與者有空時聯絡他們",permissionToReachTargetParticipantNotAllowedTitle:"不允許連絡目標參與者的權限",permissionToReachTargetParticipantNotAllowedMoreDetails:"請檢查目標參與者是否位於相同的租用戶中",unableToResolveTenantTitle:"無法解析目標參與者的租用戶識別碼",unableToResolveTenantMoreDetails:"請檢查已正確輸入參與者識別碼",participantIdIsMalformedTitle:"參與者識別碼的格式不正確",participantIdIsMalformedMoreDetails:"請檢查參與者識別碼的格式是否正確",moreButtonGalleryControlLabel:"檢視",moreButtonGalleryPositionToggleLabel:"將圖庫移至頂端",moreButtonGallerySpeakerLayoutLabel:"揚聲器",moreButtonGalleryFloatingLocalLayoutLabel:"動態",moreButtonGalleryDefaultLayoutLabel:"圖庫檢視",moreButtonGalleryFocusedContentLayoutLabel:"聚焦內容",moreButtonLargeGalleryDefaultLayoutLabel:"大型圖庫",moreButtonTogetherModeLayoutLabel:"在一起模式",capabilityChangedNotification:{turnVideoOn:{lostDueToMeetingOption:"您的相機已停用。您無法再分享視訊。",grantedDueToMeetingOption:"您的相機已啟用。如果您想要分享視訊，請將其開啟。"},unmuteMic:{lostDueToMeetingOption:"您的麥克風已停用。您無法再取消靜音。",grantedDueToMeetingOption:"您的麥克風已啟用。若您想進行語音，請取消靜音。"},shareScreen:{lostDueToRoleChangeToAttendee:"您的角色已變更。某些動作，例如共用內容，將無法供您使用。",grantedDueToRoleChangeToPresenter:"您就是簡報者。您可以分享內容並協助會議。"},hideAttendeeNames:{hideAttendeeNameAttendee:"其他出席者看不到您的名字。召集人和簡報者可以看到您的真實姓名。",hideAttendeeNamePresenter:"已隱藏出席者名稱。只有召集人和簡報者可以看到名稱以保護出席者隱私權。"}},surveyTitle:"協助我們改善",starSurveyHelperText:"通話的品質如何?",starSurveyOneStarText:"品質劣等。",starSurveyTwoStarText:"品質差。",starSurveyThreeStarText:"品質佳。",starSurveyFourStarText:"品質很好。",starSurveyFiveStarText:"品質極佳。",surveyConfirmButtonLabel:"傳送意見反應",starRatingAriaLabel:"選取 {0} / {1} 顆星",surveySkipButtonLabel:"跳過",tagsSurveyQuestion:"什麼部分本來能做得更好?",tagsSurveyTextFieldDefaultText:"其他，請註明",tagsSurveyHelperText:"檢查您遇到的任何問題",endOfSurveyText:"感謝您提供意見反應!",surveyIssues:{overallRating:{callCannotJoin:"我無法加入通話",callCannotInvite:"無法邀請其他人加入通話",hadToRejoin:"我必須重新加入通話",callEndedUnexpectedly:"我的通話意外結束",otherIssues:"我在通話中遇到其他問題"},audioRating:{noLocalAudio:"另一方聽不到任何聲音",noRemoteAudio:"我聽不到任何聲音",echo:"我在通話中聽到回音",audioNoise:"我在通話時聽到雜音",lowVolume:"音量很小",audioStoppedUnexpectedly:"音訊意外停止",distortedSpeech:"音訊失真",audioInterruption:"音訊已中斷",otherIssues:"我在這次通話中遇到其他音訊問題"},videoRating:{noVideoReceived:"我看不到任何視訊",noVideoSent:"其他人看不到我",lowQuality:"視訊品質低落",freezes:"視訊凍結",stoppedUnexpectedly:"視訊意外停止",darkVideoReceived:"只有當其他人開啟他們的相機時，我才能看到深色螢幕",audioVideoOutOfSync:"音訊和視訊不同步",otherIssues:"我在這次通話中遇到其他視訊問題"},screenshareRating:{noContentLocal:"其他人看不到我的螢幕畫面分享",noContentRemote:"我看不到其他人的螢幕畫面分享",cannotPresent:"我無法展示我的螢幕",lowQuality:"螢幕畫面分享品質太低",freezes:"螢幕畫面分享已凍結",stoppedUnexpectedly:"螢幕畫面分享意外停止",largeDelay:"螢幕畫面分享有大量的延遲",otherIssues:"我在這次通話中遇到其他螢幕畫面分享問題"}},surveyIssuesHeadingStrings:{overallRating:"整體",audioRating:"音訊",videoRating:"影片",screenshareRating:"正在進行簡報"},spotlightPrompt:{startSpotlightHeading:"要設為每個人的精選嗎?",startSpotlightText:"您將向會議中的每個人醒目提示此影片。",startSpotlightOnSelfText:"您將向會議中的每個人醒目提示您的影片。",startSpotlightConfirmButtonLabel:"設為每個人的精選",startSpotlightCancelButtonLabel:"取消",stopSpotlightHeading:"要停止聚焦這部影片嗎?",stopSpotlightOnSelfHeading:"結束精選?",stopAllSpotlightHeading:"是否要停止聚焦所有影片?",stopSpotlightText:"不會再向會議中的每個人醒目提示該影片。",stopSpotlightOnSelfText:"不會再向會議中的每個人醒目提示您的影片。",stopAllSpotlightText:"不會再向會議中的每個人醒目提示此影片。",stopSpotlightConfirmButtonLabel:"停止選為精選",stopSpotlightOnSelfConfirmButtonLabel:"結束精選",stopSpotlightCancelButtonLabel:"取消",closeSpotlightPromptButtonLabel:"關閉"},exitSpotlightButtonLabel:"結束精選",exitSpotlightButtonTooltip:"結束精選",leaveConfirmButtonLabel:"離開",endCallConfirmButtonLabel:"結束通話",hangUpCancelButtonLabel:"取消",leaveConfirmDialogTitle:"離開通話?",leaveConfirmDialogContent:"確定要離開此通話?",endCallConfirmDialogTitle:"結束通話?",endCallConfirmDialogContent:"您將結束所有人的通話。",invalidMeetingIdentifier:"會議標識碼或密碼無效",pinParticipantMenuLabel:"為我釘選",pinParticipantLimitReachedMenuLabel:"釘選 (已達到限制)",unpinParticipantMenuLabel:"取消釘選",pinParticipantMenuItemAriaLabel:"為我釘選",unpinParticipantMenuItemAriaLabel:"取消釘選",muteAllMenuLabel:"將所有人設為靜音",muteAllDialogTitle:"將所有人設為靜音?",muteAllDialogContent:"這會將會議中您以外的所有人設為靜音。",muteAllConfirmButtonLabel:"將所有人設為靜音",muteAllCancelButtonLabel:"取消",joinBreakoutRoomButtonLabel:"加入分組討論區",returnFromBreakoutRoomButtonLabel:"離開分組討論區",leaveBreakoutRoomAndMeetingButtonLabel:"離開會議",breakoutRoomJoinedNotificationTitle:"您已加入分組討論區 '{roomName}'",joinBreakoutRoomBannerTitle:"分組討論區 '{roomName}' 已開放",joinBreakoutRoomBannerButtonLabel:"加入",returnFromBreakoutRoomBannerTitle:"返回主要會議",returnFromBreakoutRoomBannerButtonLabel:"重新加入",forbidAudioMenuLabel:"停用麥克風",permitAudioMenuLabel:"允許麥克風",forbidOthersAudioDialogTitle:"要停用出席者的麥克風嗎?",forbidOthersAudioDialogContent:"出席者將無法自行取消靜音。召集人與簡報者可以視需要讓人員取消靜音。",forbidOthersAudioConfirmButtonLabel:"停用麥克風",forbidOthersAudioCancelButtonLabel:"取消",permitOthersAudioDialogTitle:"要允許出席者的麥克風嗎?",permitOthersAudioDialogContent:"會議中的每個人都可以自行解除靜音。",permitOthersAudioConfirmButtonLabel:"允許麥克風",permitOthersAudioCancelButtonLabel:"取消",forbidOthersAudioMenuLabel:"停用出席者的麥克風",permitOthersAudioMenuLabel:"允許出席者使用麥克風",forbidVideoMenuLabel:"停用相機",permitVideoMenuLabel:"允許相機",forbidOthersVideoDialogTitle:"要停用出席者的相機嗎?",forbidOthersVideoDialogContent:"出席者將無法自行開啟視訊。召集人和演示者可以讓人員視需要開啟攝影機。",forbidOthersVideoConfirmButtonLabel:"停用相機",forbidOthersVideoCancelButtonLabel:"取消",permitOthersVideoDialogTitle:"要允許出席者的相機嗎?",permitOthersVideoDialogContent:"會議中的每個人都能自行開啟視訊。",permitOthersVideoConfirmButtonLabel:"允許相機",permitOthersVideoCancelButtonLabel:"取消",forbidOthersVideoMenuLabel:"停用出席者的鏡頭",permitOthersVideoMenuLabel:"允許出席者使用相機"};var chat={chatListHeader:"在此聊天中",uploadAttachment:"上傳附件",uploadImageDataNotProvided:"未提供影像資料。",uploadImageIsTooLarge:"影像太大。選擇小於 {maxImageSize} MB 的選項。",uploadImageExtensionIsNotAllowed:"不允許上傳 .{imageExtension} 影像。",uploadImageFailed:"無法上傳影像。請稍後再試。"};var callWithChat={chatButtonLabel:"聊天",chatButtonNewMessageNotificationLabel:"新訊息",chatButtonTooltipClosedWithMessageCount:"顯示聊天 ({unreadMessagesCount} 個未讀取)",chatButtonTooltipClose:"隱藏聊天",chatButtonTooltipOpen:"顯示聊天",chatPaneTitle:"聊天",copyInviteLinkButtonLabel:"複製邀請連結",copyInviteLinkButtonActionedLabel:"已複製連結",copyInviteLinkActionedAriaLabel:"已複製邀請連結",dismissSidePaneButtonLabel:"關閉",moreDrawerAudioDeviceMenuTitle:"音訊裝置",moreDrawerButtonLabel:"更多選項",moreDrawerButtonTooltip:"更多選項",moreDrawerMicrophoneMenuTitle:"麥克風",moreDrawerSpeakerMenuTitle:"揚聲器",moreDrawerCaptionsMenuTitle:"即時字幕",moreDrawerSpokenLanguageMenuTitle:"口語語言",moreDrawerCaptionLanguageMenuTitle:"字幕語言",moreDrawerGalleryOptionsMenuTitle:"圖庫選項",peopleButtonLabel:"人員",selectedPeopleButtonLabel:"已選取人員按鈕",peopleButtonTooltipOpen:"顯示參與者",peopleButtonTooltipClose:"隱藏參與者",peopleButtonTooltipOpenAriaLabel:"[人員] 按鈕，顯示參與者",peopleButtonTooltipCloseAriaLabel:"[人員] 按鈕，隱藏參與者",peoplePaneSubTitle:"在此通話中",peoplePaneTitle:"人員",pictureInPictureTileAriaLabel:"影片摘要。按一下以返回通話畫面。",removeMenuLabel:"移除",openDialpadButtonLabel:"撥打電話號碼",returnToCallButtonAriaDescription:"返回通話",returnToCallButtonAriaLabel:"返回",peoplePaneAddPeopleButtonLabel:"新增人員",dialpadStartCallButtonLabel:"通話",dialpadModalTitle:"撥打電話號碼",dialpadModalAriaLabel:"撥號鍵台",dialpadCloseModalButtonAriaLabel:"關閉撥號鍵台",openDtmfDialpadLabel:"顯示撥號盤",dtmfDialpadPlaceholderText:"輸入號碼",breakoutRoomChatPaneTitle:"分組討論區聊天",chatContentSpinnerLabel:"正在載入聊天"};var zh_TW = {call:call,chat:chat,callWithChat:callWithChat};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Locale for Chinese (Taiwan)
 *
 * @public
 */
const COMPOSITE_LOCALE_ZH_TW = {
    component: COMPONENT_LOCALE_ZH_TW,
    strings: createCompositeStrings(zh_TW)
};

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$3 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 *
 * Dependency resolution for video background effects using lazy loading.
 * @public
 */
const onResolveVideoEffectDependencyLazy = () => __awaiter$3(void 0, void 0, void 0, function* () {
    const module = yield import('@azure/communication-calling-effects');
    const createBackgroundBlurEffect = (config) => {
        return new module.BackgroundBlurEffect(config);
    };
    const createBackgroundReplacementEffect = (config) => {
        return new module.BackgroundReplacementEffect(config);
    };
    const VideoBackgroundEffectsDependency = {
        createBackgroundBlurEffect,
        createBackgroundReplacementEffect
    };
    return VideoBackgroundEffectsDependency;
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$2 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Dependency resolution for video background effects.
 *
 * @public
 */
const onResolveVideoEffectDependency = () => __awaiter$2(void 0, void 0, void 0, function* () {
    const createBackgroundBlurEffect = (config) => {
        return new communicationCallingEffects.BackgroundBlurEffect(config);
    };
    const createBackgroundReplacementEffect = (config) => {
        return new communicationCallingEffects.BackgroundReplacementEffect(config);
    };
    const VideoBackgroundEffectsDependency = {
        createBackgroundBlurEffect,
        createBackgroundReplacementEffect
    };
    return VideoBackgroundEffectsDependency;
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter$1 = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 *
 * Dependency resolution for video background effects using lazy loading.
 * @public
 */
const onResolveDeepNoiseSuppressionDependencyLazy = () => __awaiter$1(void 0, void 0, void 0, function* () {
    const module = yield import('@azure/communication-calling-effects');
    const audioEffect = {
        noiseSuppression: new module.DeepNoiseSuppressionEffect()
    };
    return {
        deepNoiseSuppressionEffect: audioEffect
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter = (window && window.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 *
 * Dependency resolution for video background effects using lazy loading.
 * @public
 */
const onResolveDeepNoiseSuppressionDependency = () => __awaiter(void 0, void 0, void 0, function* () {
    const audioEffect = {
        noiseSuppression: new communicationCallingEffects.DeepNoiseSuppressionEffect()
    };
    return {
        deepNoiseSuppressionEffect: audioEffect
    };
});

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
/**
 * Hook to obtain a selector for a specified component.
 *
 * Useful when implementing a custom component that utilizes the providers
 * exported from this library.
 *
 * @public
 */
const useSelector = (selector, selectorProps, type) => {
    // Because of react hooks rules, hooks can't be conditionally called
    // We call both call and chat hooks and detect current context
    // Return undefined and skip execution when not in that context
    const callingMode = !type || type === 'calling';
    const chatMode = !type || type === 'chat';
    const callProps = useSelector$4(callingMode ? selector : undefined, selectorProps);
    const chatProps = useSelector$3(chatMode ? selector : undefined, selectorProps);
    return callProps !== null && callProps !== void 0 ? callProps : chatProps;
};
/**
 * Primary hook to get all hooks necessary for a React Component from this library.
 *
 * To call this hook, the component requires to be wrapped under these providers:
 *
 * 1. For chat components: {@link ChatClientProvider} and {@link ChatThreadClientProvider}.
 *
 * 2. For calling components: {@link CallClientProvider}, {@link CallAgentProvider} and {@link CallAgentProvider}.
 *
 * Most straightforward usage of a components looks like:
 *
 * @example
 * ```
 *     import { ParticipantList, usePropsFor } from '@azure/communication-react';
 *
 *     const App = (): JSX.Element => {
 *         // ... code to setup Providers ...
 *
 *         return <ParticipantList {...usePropsFor(ParticipantList)}/>
 *     }
 * ```
 *
 * @public
 */
const usePropsFor = (component, type) => {
    const callingSelector = type === 'calling' || !type ? getSelector$1(component) : undefined;
    const chatSelector = type === 'chat' || !type ? getSelector(component) : undefined;
    const callProps = useSelector$4(callingSelector);
    const chatProps = useSelector$3(chatSelector);
    const callingHandlers = useHandlers$3();
    const chatHandlers = useHandlers$2();
    if (chatProps) {
        if (!chatHandlers) {
            throw 'Please initialize chatClient and chatThreadClient first!';
        }
        return Object.assign(Object.assign({}, chatProps), chatHandlers);
    }
    if (callProps) {
        if (!callingHandlers) {
            throw 'Please initialize callClient first!';
        }
        return Object.assign(Object.assign({}, callProps), callingHandlers);
    }
    if (!chatSelector && !callingSelector) {
        throw "Can't find corresponding selector for this component. Please check the supported components from Azure Communication UI Feature Component List.";
    }
    else {
        throw 'Could not find props for this component, ensure the component is wrapped by appropriate providers.';
    }
};

exports.COMPONENT_LOCALE_AR_SA = COMPONENT_LOCALE_AR_SA;
exports.COMPONENT_LOCALE_CS_CZ = COMPONENT_LOCALE_CS_CZ;
exports.COMPONENT_LOCALE_CY_GB = COMPONENT_LOCALE_CY_GB;
exports.COMPONENT_LOCALE_DE_DE = COMPONENT_LOCALE_DE_DE;
exports.COMPONENT_LOCALE_EN_GB = COMPONENT_LOCALE_EN_GB;
exports.COMPONENT_LOCALE_EN_US = COMPONENT_LOCALE_EN_US;
exports.COMPONENT_LOCALE_ES_ES = COMPONENT_LOCALE_ES_ES;
exports.COMPONENT_LOCALE_ES_MX = COMPONENT_LOCALE_ES_MX;
exports.COMPONENT_LOCALE_FI_FI = COMPONENT_LOCALE_FI_FI;
exports.COMPONENT_LOCALE_FR_CA = COMPONENT_LOCALE_FR_CA;
exports.COMPONENT_LOCALE_FR_FR = COMPONENT_LOCALE_FR_FR;
exports.COMPONENT_LOCALE_HE_IL = COMPONENT_LOCALE_HE_IL;
exports.COMPONENT_LOCALE_IT_IT = COMPONENT_LOCALE_IT_IT;
exports.COMPONENT_LOCALE_JA_JP = COMPONENT_LOCALE_JA_JP;
exports.COMPONENT_LOCALE_KO_KR = COMPONENT_LOCALE_KO_KR;
exports.COMPONENT_LOCALE_NB_NO = COMPONENT_LOCALE_NB_NO;
exports.COMPONENT_LOCALE_NL_NL = COMPONENT_LOCALE_NL_NL;
exports.COMPONENT_LOCALE_PL_PL = COMPONENT_LOCALE_PL_PL;
exports.COMPONENT_LOCALE_PT_BR = COMPONENT_LOCALE_PT_BR;
exports.COMPONENT_LOCALE_RU_RU = COMPONENT_LOCALE_RU_RU;
exports.COMPONENT_LOCALE_SV_SE = COMPONENT_LOCALE_SV_SE;
exports.COMPONENT_LOCALE_TR_TR = COMPONENT_LOCALE_TR_TR;
exports.COMPONENT_LOCALE_ZH_CN = COMPONENT_LOCALE_ZH_CN;
exports.COMPONENT_LOCALE_ZH_TW = COMPONENT_LOCALE_ZH_TW;
exports.COMPOSITE_LOCALE_AR_SA = COMPOSITE_LOCALE_AR_SA;
exports.COMPOSITE_LOCALE_CS_CZ = COMPOSITE_LOCALE_CS_CZ;
exports.COMPOSITE_LOCALE_CY_GB = COMPOSITE_LOCALE_CY_GB;
exports.COMPOSITE_LOCALE_DE_DE = COMPOSITE_LOCALE_DE_DE;
exports.COMPOSITE_LOCALE_EN_GB = COMPOSITE_LOCALE_EN_GB;
exports.COMPOSITE_LOCALE_EN_US = COMPOSITE_LOCALE_EN_US;
exports.COMPOSITE_LOCALE_ES_ES = COMPOSITE_LOCALE_ES_ES;
exports.COMPOSITE_LOCALE_ES_MX = COMPOSITE_LOCALE_ES_MX;
exports.COMPOSITE_LOCALE_FI_FI = COMPOSITE_LOCALE_FI_FI;
exports.COMPOSITE_LOCALE_FR_CA = COMPOSITE_LOCALE_FR_CA;
exports.COMPOSITE_LOCALE_FR_FR = COMPOSITE_LOCALE_FR_FR;
exports.COMPOSITE_LOCALE_HE_IL = COMPOSITE_LOCALE_HE_IL;
exports.COMPOSITE_LOCALE_IT_IT = COMPOSITE_LOCALE_IT_IT;
exports.COMPOSITE_LOCALE_JA_JP = COMPOSITE_LOCALE_JA_JP;
exports.COMPOSITE_LOCALE_KO_KR = COMPOSITE_LOCALE_KO_KR;
exports.COMPOSITE_LOCALE_NB_NO = COMPOSITE_LOCALE_NB_NO;
exports.COMPOSITE_LOCALE_NL_NL = COMPOSITE_LOCALE_NL_NL;
exports.COMPOSITE_LOCALE_PL_PL = COMPOSITE_LOCALE_PL_PL;
exports.COMPOSITE_LOCALE_PT_BR = COMPOSITE_LOCALE_PT_BR;
exports.COMPOSITE_LOCALE_RU_RU = COMPOSITE_LOCALE_RU_RU;
exports.COMPOSITE_LOCALE_SV_SE = COMPOSITE_LOCALE_SV_SE;
exports.COMPOSITE_LOCALE_TR_TR = COMPOSITE_LOCALE_TR_TR;
exports.COMPOSITE_LOCALE_ZH_CN = COMPOSITE_LOCALE_ZH_CN;
exports.COMPOSITE_LOCALE_ZH_TW = COMPOSITE_LOCALE_ZH_TW;
exports.COMPOSITE_ONLY_ICONS = COMPOSITE_ONLY_ICONS;
exports.CallAgentProvider = CallAgentProvider;
exports.CallClientProvider = CallClientProvider;
exports.CallComposite = CallComposite;
exports.CallProvider = CallProvider;
exports.CallWithChatComposite = CallWithChatComposite;
exports.CameraButton = CameraButton;
exports.CaptionsBanner = CaptionsBanner;
exports.CaptionsSettingsModal = CaptionsSettingsModal;
exports.ChatClientProvider = ChatClientProvider;
exports.ChatComposite = ChatComposite;
exports.ChatThreadClientProvider = ChatThreadClientProvider;
exports.ControlBar = ControlBar;
exports.ControlBarButton = ControlBarButton;
exports.DEFAULT_COMPONENT_ICONS = DEFAULT_COMPONENT_ICONS;
exports.DEFAULT_COMPOSITE_ICONS = DEFAULT_COMPOSITE_ICONS;
exports.DevicesButton = DevicesButton;
exports.Dialpad = Dialpad;
exports.EndCallButton = EndCallButton;
exports.ErrorBar = ErrorBar;
exports.FluentThemeProvider = FluentThemeProvider;
exports.GridLayout = GridLayout;
exports.HoldButton = HoldButton;
exports.ImageOverlay = ImageOverlay;
exports.IncomingCallNotification = IncomingCallNotification$n;
exports.IncomingCallStack = IncomingCallStack;
exports.LocalizationProvider = LocalizationProvider$1;
exports.MessageStatusIndicator = MessageStatusIndicator;
exports.MessageThread = MessageThread;
exports.MicrophoneButton = MicrophoneButton;
exports.Notification = Notification;
exports.NotificationStack = NotificationStack;
exports.ParticipantItem = ParticipantItem;
exports.ParticipantList = ParticipantList$n;
exports.ParticipantsButton = ParticipantsButton;
exports.RaiseHandButton = RaiseHandButton;
exports.ReactionButton = ReactionButton;
exports.RealTimeText = RealTimeText;
exports.RealTimeTextModal = RealTimeTextModal;
exports.ScreenShareButton = ScreenShareButton;
exports.SendBox = SendBox;
exports.StartCaptionsButton = StartCaptionsButton;
exports.StartRealTimeTextButton = StartRealTimeTextButton;
exports.StreamMedia = StreamMedia;
exports.TypingIndicator = TypingIndicator;
exports.VideoGallery = VideoGallery;
exports.VideoTile = VideoTile;
exports._IdentifierProvider = _IdentifierProvider;
exports.createAzureCommunicationCallAdapter = createAzureCommunicationCallAdapter;
exports.createAzureCommunicationCallAdapterFromClient = createAzureCommunicationCallAdapterFromClient;
exports.createAzureCommunicationCallWithChatAdapter = createAzureCommunicationCallWithChatAdapter;
exports.createAzureCommunicationCallWithChatAdapterFromClients = createAzureCommunicationCallWithChatAdapterFromClients;
exports.createAzureCommunicationChatAdapter = createAzureCommunicationChatAdapter;
exports.createAzureCommunicationChatAdapterFromClient = createAzureCommunicationChatAdapterFromClient;
exports.createDefaultCallingHandlers = createDefaultCallingHandlers;
exports.createDefaultChatHandlers = createDefaultChatHandlers;
exports.createDefaultTeamsCallingHandlers = createDefaultTeamsCallingHandlers;
exports.createStatefulCallClient = createStatefulCallClient;
exports.createStatefulChatClient = createStatefulChatClient;
exports.createTeamsCallAdapter = createTeamsCallAdapter;
exports.createTeamsCallAdapterFromClient = createTeamsCallAdapterFromClient;
exports.darkTheme = darkTheme;
exports.fromFlatCommunicationIdentifier = fromFlatCommunicationIdentifier;
exports.getCallingSelector = getSelector$1;
exports.getChatSelector = getSelector;
exports.lightTheme = lightTheme;
exports.onResolveDeepNoiseSuppressionDependency = onResolveDeepNoiseSuppressionDependency;
exports.onResolveDeepNoiseSuppressionDependencyLazy = onResolveDeepNoiseSuppressionDependencyLazy;
exports.onResolveVideoEffectDependency = onResolveVideoEffectDependency;
exports.onResolveVideoEffectDependencyLazy = onResolveVideoEffectDependencyLazy;
exports.toFlatCommunicationIdentifier = toFlatCommunicationIdentifier;
exports.useAzureCommunicationCallAdapter = useAzureCommunicationCallAdapter;
exports.useAzureCommunicationCallWithChatAdapter = useAzureCommunicationCallWithChatAdapter;
exports.useAzureCommunicationChatAdapter = useAzureCommunicationChatAdapter;
exports.useCall = useCall;
exports.useCallAgent = useCallAgent;
exports.useCallClient = useCallClient;
exports.useChatClient = useChatClient;
exports.useChatThreadClient = useChatThreadClient;
exports.useDeviceManager = useDeviceManager;
exports.usePropsFor = usePropsFor;
exports.useSelector = useSelector;
exports.useTeamsCall = useTeamsCall;
exports.useTeamsCallAdapter = useTeamsCallAdapter;
exports.useTeamsCallAgent = useTeamsCallAgent;
exports.useTheme = useTheme;
//# sourceMappingURL=index.js.map
